//This software module was originally developed by CSELT in the course of   //
//development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.             //
//This software module is an implementation of a part of one or more MPEG-4 //
//Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems        //
//(ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 Systems     //
//(ISO/IEC 14496-1) free license to this software module or modifications   //
//thereof for use in hardware or software products claiming conformance to  //
//the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use this software//
//module in hardware or software products are advised that its use may      //
//infringe existing patents.                                                //
//The original developer of this software module and his/her company, the   //
//subsequent editors and their companies, and ISO/IEC have no liability     //
//for use of this software module or modifications thereof in an            //
//implementation. Copyright is not released for non MPEG-4 Systems          //
//(ISO/IEC 14496-1) conforming products. CSELT S.pA. retains full right     //
//to use the code for his/her own purpose, assign or donate the code to a   //
//third party and to inhibit third parties from using the code for non      //
//MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright      //
//notice must be included in all copies or derivative works.                //
//Copyright  1999.                                                         //
//This work has been partly developed within the framework of the ESPRIT    //
//European Research Project  "MPEG-4 PC".                                   //
//////////////////////////////////////////////////////////////////////////////
//
// File: AR-DS-StreamInfo.h
//
// Author: Matthew Leditschke 
//
// Modified: Gianluca Di Cagno CSELT  gianluca.dicagno@cselt.it,
//           Ananda Allys France Telecom
//
// Description:
//   The definition of a class which encapsulates all of the details
//   of an audio stream that is being played by the DirectSound based
//   audio renderer.
//
// Created: August, 1997.
//
//
///////////////////////////////////////////////////////////////////////////


#include <mmsystem.h>
#include <dsound.h>


// The length of the buffer for a stream (in milliseconds)
const int BUFFER_LENGTH = 640;
// How often these buffers will be updated (in milliseconds)
const int BUFFER_UPDATE_TIME = 156; 

class AudioSourceProxy;


class StreamInfo
{
public:
	LPDIRECTSOUND m_pDS;			// DirectSound object which controls the sound card
	WAVEFORMATEX m_wf;				// Format of the audio being played
	DSBUFFERDESC m_bufDesc;			// DirectSound buffer descriptor
	LPDIRECTSOUNDBUFFER m_pDSBuf;	// DirectSound streaming buffer object
	DWORD m_startTime;				// When to start playing this buffer
	AudioSourceProxy* m_pSrc;		// Pointer to the node to get data from
	DWORD m_nextWrite;				// Buffer offset of where to write next data
	BOOL m_bPlaying;				// TRUE if the stream is playing.
	BOOL m_bReallyPlaying;			// TRUE if the stream is REALLY playing.
	FILE* m_rawOutput;				// Only for debugging

	// Constructor, set default values
	StreamInfo(LPDIRECTSOUND pDS, AudioSourceProxy* pSrc, DWORD startTime);

	// Destructor, shut down the stream.
	~StreamInfo();

	// Create a streaming buffer.  Return DS_OK if created OK.
	int Create(DWORD nSamplesPerSec, DWORD nBitsPerSample, WORD nNumChannels);

	// Calculate the maximum size of sound buffer data which can be written
	// to the given stream.  i.e. how much free space there is in buffer.
	DWORD GetMaxWriteSize();

	// Set the write pointer to the first unread byte.
	void ResetWritePointer();

	// Write the given data to the stream buffer.
	void WriteData(LPBYTE pData, DWORD nNumBytes, float fIntensity=1.0);

	// Write the given number of bytes worth of silence into the buffer.
	void WriteSilence(DWORD nNumBytes, BOOL updateOffset);

	// Start the stream playing
	void Play();
	void Stop();
	// Return TRUE if the format of this stream is known, otherwise
	// return FALSE.
	BOOL IsFormatKnown() const { return (m_wf.nChannels != 0); }

	// Return TRUE if the stream is already playing, FALSE otherwise.
	BOOL IsPlaying() const { return m_bPlaying; }

	// Return TRUE if the stream is REALLY playing, FALSE otherwise.
	// Initially just plays 156 bytes and stops for a while !!!
	BOOL IsReallyPlaying(); 

	// Return the number of bytes per millisecond
	double GetBytesPerMs() { return (((double)(m_wf.nSamplesPerSec * m_wf.wBitsPerSample) / 8000)*m_wf.nChannels); }
	void SetInitialCTS (DWORD dwCTS);	// Zvi added
    
	void SetLastCorrection(DWORD correction) {m_dwLastCorrection=correction;}

	DWORD GetLastCorrection() {return m_dwLastCorrection;}

	double GetDivergenceTillGood(int dwClockDivergence,DWORD dwObjectClock);

	void UpDateFrequency(double divergenceTillGood);

	DWORD GetThreshold(){return m_dwThreshold;}
private:
	DWORD m_dwInitialCTS;	// (Zvi) added for clock adjustment	
	DWORD m_dwLastCorrection; // last time audio frequency adjusted
	DWORD m_dwThreshold;
}; /* class StreamInfo */


// ===============================================================================
