//This software module was originally developed by CSELT in the course of   //
//development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.             //
//This software module is an implementation of a part of one or more MPEG-4 //
//Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems        //
//(ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 Systems     //
//(ISO/IEC 14496-1) free license to this software module or modifications   //
//thereof for use in hardware or software products claiming conformance to  //
//the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use this software//
//module in hardware or software products are advised that its use may      //
//infringe existing patents.                                                //
//The original developer of this software module and his/her company, the   //
//subsequent editors and their companies, and ISO/IEC have no liability     //
//for use of this software module or modifications thereof in an            //
//implementation. Copyright is not released for non MPEG-4 Systems          //
//(ISO/IEC 14496-1) conforming products. CSELT S.pA. retains full right     //
//to use the code for his/her own purpose, assign or donate the code to a   //
//third party and to inhibit third parties from using the code for non      //
//MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright      //
//notice must be included in all copies or derivative works.                //
//Copyright  1999.                                                         //
//This work has been partly developed within the framework of the ESPRIT    //
//European Research Project  "MPEG-4 PC".                                   //
//////////////////////////////////////////////////////////////////////////////
//
// File: AudRend-CSELT-DS.h
//
// Author: Matthew Leditschke 
//
// Modified: Gianluca Di Cagno  gianluca.dicagno@cselt.it
//
// Description:
//   The definition of a class which provides a DirectSound based audio
//   renderer.
//
// Created: August, 1997.
//
//
///////////////////////////////////////////////////////////////////////////


#ifndef AUD_REND_CSELT_DS_H
#define AUD_REND_CSELT_DS_H


#include "AudRend-CSELT.h"
#include "ztl.h"
#include <mmsystem.h>
#include <dsound.h>


class StreamInfo;


class AudioRendererDS: public AudioRenderer
{
	//
	// Public functions
	//
public:

	// Construct the audio renderer, given a pointer to the current
	// Presenter object, and the handle of the window the application
	// is running in.
	AudioRendererDS(Presenter* pPresenter, HWND hWnd);

	// Class destructor.
	virtual ~AudioRendererDS();

	// This is the main loop of the audio renderer thread.
	void Main();

	// Methods inherited from the AudioRenderer base class.
	virtual void GetCapabilities (Capabilities* pPresentationCaps);
	virtual void Init();
	virtual void Terminate();
	virtual void Start();
	virtual void AddSourceNode(AudioSourceProxy* pSourceNode,
							   DWORD startTime, DWORD nSamplesPerSec,
							   DWORD nBitsPerSample, WORD nNumChannels);
	virtual void RemoveSourceNode(AudioSourceProxy* pSourceNode);
	virtual DWORD GetBufferLengthMs();

    
   
	//
	// Protected implementation details
	//
protected:


	//
	// Private implementation details
	//
private:

	// The DirectSound object which controls the sound card
	LPDIRECTSOUND m_pDS;

	// This is the audio format output by the sound card
	WAVEFORMATEX m_wFmt;

	// Used to flags the presenter thread when it is time to finish.
	BOOL m_bActive;
	BOOL m_bPaused;
	// This mutex is used by Terminate() to wait until the thread
	// has finished.
	ZMutex m_mtxThread;

	// This mutex controls access to the array of stream information
	// data, as multiple threads access this.
	ZMutex m_mtxStreamArray;

	// The array of sound streams which have been created
	ZArray<StreamInfo*> m_streams;

	// Given a stream for which the audio data format is not known, query
	// the node for this information, and then update the output format
	// that the sound card is currently using.
	void UpdateOutputFormat(StreamInfo* pStream);

};


#endif // AUD_REND_CSELT_DS_H
