//This software module was originally developed by CSELT in the course of   //
//development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.             //
//This software module is an implementation of a part of one or more MPEG-4 //
//Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems        //
//(ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 Systems     //
//(ISO/IEC 14496-1) free license to this software module or modifications   //
//thereof for use in hardware or software products claiming conformance to  //
//the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use this software//
//module in hardware or software products are advised that its use may      //
//infringe existing patents.                                                //
//The original developer of this software module and his/her company, the   //
//subsequent editors and their companies, and ISO/IEC have no liability     //
//for use of this software module or modifications thereof in an            //
//implementation. Copyright is not released for non MPEG-4 Systems          //
//(ISO/IEC 14496-1) conforming products. CSELT S.pA. retains full right     //
//to use the code for his/her own purpose, assign or donate the code to a   //
//third party and to inhibit third parties from using the code for non      //
//MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright      //
//notice must be included in all copies or derivative works.                //
//Copyright  1999.                                                         //
//This work has been partly developed within the framework of the ESPRIT    //
//European Research Project  "MPEG-4 PC".                                   //
//////////////////////////////////////////////////////////////////////////////
//
// File: AudRend-CSELT-Sgi.h
//
// Author: Matthew Leditschke 
//
// Modified:
//
// Description:
//   The definition of a class which provides an audio renderer for the
//   SGI platform.
//
// Created: August, 1997.
//
//
///////////////////////////////////////////////////////////////////////////


#ifndef AUD_REND_CSELT_SGI_H
#define AUD_REND_CSELT_SGI_H


#include "AudRend-CSELT.h"
#include "ztl.h"
extern "C" {
#include <dmedia/audio.h>
}

class StreamInfo;


class AudioRendererSgi: public AudioRenderer
{
	//
	// Public functions
	//
public:

	// Construct the audio renderer, given a pointer to the current
	// Presenter object.
	AudioRendererSgi(Presenter* pPresenter);

	// Class destructor.
	virtual ~AudioRendererSgi();

	// This is the main loop of the audio renderer thread.
	void Main();
	static void StartThread(void* params);

	// Methods inherited from the AudioRenderer base class.
	virtual void GetCapabilities (Capabilities* pPresentationCaps);
	virtual void Init();
	virtual void Terminate();
	virtual void Start();
	virtual void AddSourceNode(AudioSource* pAudioSourceNode,
							   DWORD startTime, DWORD nSamplesPerSec,
							   DWORD nBitsPerSample, WORD nNumChannels);
	virtual void RemoveSourceNode(AudioSource* pAudioSourceNode);


	//
	// Protected implementation details
	//
protected:


	//
	// Private implementation details
	//
private:

	// The audio configuration parameters for the output device
	ALconfig m_audioConfig;

	// The output audio port
	ALport m_audioPort;

	// The settings of the output port
	DWORD m_samplesPerSec;
	DWORD m_bitsPerSample;
	WORD  m_numChannels;
	DWORD m_bytesPerSample;

	// The size of the output queue (in samples, not bytes)
	long m_outputQueueSize;

	// This is the buffer used to mix samples
	char* m_mixBuf;

	// Used to flags the presenter thread when it is time to finish.
	BOOL m_bActive;

	// This mutex is used by Terminate() to wait until the thread
	// has finished.
	ZMutex m_mtxThread;

	// This mutex controls access to the array of stream information
	// data, as multiple threads access this.
	ZMutex m_mtxStreamArray;

	// The array of sound streams which have been created
	ZArray<StreamInfo*> m_streams;

	// This function is called when the first audio output stream is known. 
	// The audio output format settings of this first stream are given to
	// this function, and the output format of the audio device is set to
	// this.  If other streams are added which have a different output
	// format, then their format will need to be changed in software.
	void InitAudioOutput(DWORD nSamplesPerSec, DWORD nBitsPerSample,
						 WORD nNumChannels);

	// Combine the given audio data with the audio data already in the
	// mux buffer.  The offset specifies the offset (in bytes) of where
	// to start combining the data.
	void CombineData(DWORD offset, LPBYTE pSrc, DWORD lenSrc);

	// Write silence into the mux buffer, starting at the given offset
	// (in bytes), and the fullness specifies the number of samples in
	// the buffer.
	void WriteSilence(DWORD offset, DWORD fullness);
};


#endif // AUD_REND_CSELT_DS_H
