//////////////////////////////////////////////////////////////////////////////
//This software module was originally developed by CSELT in the course of   //
//development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.             //
//This software module is an implementation of a part of one or more MPEG-4 //
//Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems        //
//(ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 Systems     //
//(ISO/IEC 14496-1) free license to this software module or modifications   //
//thereof for use in hardware or software products claiming conformance to  //
//the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use this software//
//module in hardware or software products are advised that its use may      //
//infringe existing patents.                                                //
//The original developer of this software module and his/her company, the   //
//subsequent editors and their companies, and ISO/IEC have no liability     //
//for use of this software module or modifications thereof in an            //
//implementation. Copyright is not released for non MPEG-4 Systems          //
//(ISO/IEC 14496-1) conforming products. CSELT S.pA. retains full right     //
//to use the code for his/her own purpose, assign or donate the code to a   //
//third party and to inhibit third parties from using the code for non      //
//MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright      //
//notice must be included in all copies or derivative works.                //
//Copyright  1999.                                                         //
//This work has been partly developed within the framework of the ESPRIT    //
//European Research Project  "MPEG-4 PC".                                   //
//////////////////////////////////////////////////////////////////////////////
//
// File: AudRend-CSELT.cpp
//
// Author: Matthew Leditschke
//
// Modified: Gianluca Di Cagno  gianluca.dicagno@cselt.it
//           Ananda Allys France Telecom
//
// Description:
//   The implementation of an abstract base class which performs audio
//   rendering.  Actual implementations of the audio renderer will
//   derive from this class.
//
// Created: August, 1997.
//
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include <assert.h>
#include "AudRend-CSELT.h"

// Include the details of the Windows 95/NT specific audio renderers
#ifdef WIN32
#include "AudRend-CSELT-DS.h"


#include "AudRend-MM.h"
#endif

// Include the details of the SGI specific audio renderers
#if sgi
#include "AudRend-CSELT-Sgi.h"
#endif

// Include the details of the sun specific audio renderers
#if sun
#endif


// ============================================================================

//
// AudioRenderer::Create
//
// Return a pointer to an audio renderer.  Use this to construct
// an audio render appropriate for the current platform.  A pointer
// to the presenter, and a platform specific parameter are given.
//

AudioRenderer* AudioRenderer::Create(Presenter* pPresenter,
									 void* pInfo)
{
	AudioRenderer* pAudRend;

// Construct a Windows 95/NT specific audio renderer
#ifdef WIN32
   pAudRend = new AudioRendererDS(pPresenter, (HWND)pInfo);

// create an AudioRendererMM
//   pAudRend = new AudioRendererMM(pPresenter, (HWND)pInfo);
#endif

// Construct an SGI specific audio renderer
#if sgi
	pAudRend = new AudioRendererSgi(pPresenter);
#endif

// Construct a sun specific audio renderer
#if sun
#endif

	pAudRend->m_pPresenter = pPresenter;

	return pAudRend;
} /* AudioRenderer::Create */


// ============================================================================
