//This software module was originally developed by CSELT in the course of   //
//development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.             //
//This software module is an implementation of a part of one or more MPEG-4 //
//Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems        //
//(ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 Systems     //
//(ISO/IEC 14496-1) free license to this software module or modifications   //
//thereof for use in hardware or software products claiming conformance to  //
//the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use this software//
//module in hardware or software products are advised that its use may      //
//infringe existing patents.                                                //
//The original developer of this software module and his/her company, the   //
//subsequent editors and their companies, and ISO/IEC have no liability     //
//for use of this software module or modifications thereof in an            //
//implementation. Copyright is not released for non MPEG-4 Systems          //
//(ISO/IEC 14496-1) conforming products. CSELT S.pA. retains full right     //
//to use the code for his/her own purpose, assign or donate the code to a   //
//third party and to inhibit third parties from using the code for non      //
//MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright      //
//notice must be included in all copies or derivative works.                //
//Copyright  1999.                                                         //
//This work has been partly developed within the framework of the ESPRIT    //
//European Research Project  "MPEG-4 PC".                                   //
//////////////////////////////////////////////////////////////////////////////
//
// File: AudRend-CSELT.h
//
// Author: Matthew Leditschke, 
//
// Modified: Gianluca Di Cagno  gianluca.dicagno@cselt.it
//
// Description:
//   The definition of an abstract base class which performs audio
//   rendering.  Actual implementations of the audio renderer will
//   derive from this class.
//
// Created: August, 1997.
//
//
///////////////////////////////////////////////////////////////////////////


#ifndef AUD_REND_CSELT_H
#define AUD_REND_CSELT_H


class Presenter;
class Capabilities;
class AudioSourceProxy;


class AudioRenderer
{
	//
	// Public functions
	//
public:

	static AudioRenderer* Create(Presenter* pPresenter,
								 void* pInfo = NULL);
	// Return a pointer to an audio renderer.  Use this to construct
	// an audio render appropriate for the current platform.  A pointer
	// to the presenter, and a platform specific parameter are given.

	virtual ~AudioRenderer() {}
	// We need a virtual desctructor

	virtual void GetCapabilities (Capabilities* pPresentationCaps) = 0;
	// Get the capabilities of this audio renderer.

	virtual void Init() = 0;
	// Initialise the renderer.  This will be called once for every file
	// that is opened.

	virtual void Terminate() = 0;
	// Stops rendering and performs cleanups, including terminating the
	// audio renderer thread.  This will be called once for every file
	// that is closed.

	virtual void Start() = 0;
	// This function is called once all of the audio nodes in the scene
	// have been known to have registered themselves with the audio
	// renderer.  It is anticipated that this will be called after the
	// scene tree is first rendered, during which the audio source nodes
	// will have registered themselves.  This method will also start the
	// main system clock running.

	virtual void AddSourceNode(AudioSourceProxy* pSourceNode,
							   DWORD startTime, DWORD nSamplesPerSec,
							   DWORD nBitsPerSample, WORD nNumChannels) = 0;
	// Add the given AudioSource node to the list of audio source nodes
	// which need to be played.  If a node is already in the list, then
	// just leave it there.  The time at which the audio is to start
	// playing is also given, along with the audio data format.
	//
	// It is intended that this method will be called when PreRender() is
	// first called on this proxy node.

	virtual void RemoveSourceNode(AudioSourceProxy* pSourceNode) = 0;
	// Remove the given AudioSource node from the list of audio source
	// nodes which need to be played.  If the given node is not in the
	// list, nothing happens.

	virtual DWORD GetBufferLengthMs() = 0;
	// Returns the BufferLength in milliseconds, that the audio renderer has.
	// This delay is the average time that it takes samples taken from
	// the AudioSourceNode to be played.

	// Return TRUE if the audio renderer was initialised successfully.
	BOOL InitialisedOK() { return m_initOK; }


	//
	// Protected implementation details
	//
protected:

	// Remember a pointer to the presenter that this renderer is part of
	Presenter* m_pPresenter;

	// Stores if the audio renderer was initialised successfully.
	BOOL m_initOK;
};


#endif // AUD_REND_CSELT_H


// Emacs uses these...
// Local Variables:
// mode: c++
// tab-width: 4
// End:
