///////////////////////////////////////////////////////////////////////////
//                                                                       //
// Copyright (c) 1997 ISO/IEC All Rights Reserved                        //
//                                                                       //
// This software module was developed by                                 //
//                                                                       //
// CSELT (Centro Studi e Laboratori Telecomunicazioni S.p.A)             //
//                                                                       //
// in the course of development of the MPEG-4 standard. This software    //
// module is an implementation of a part of one or more MPEG-4 tools as  //
// specified by the MPEG-4 standard.                                     //
//                                                                       //
// The copyright of this software belongs to ISO/IEC. ISO/IEC gives use  //
// of the MPEG-4 standard free license to use this  software module or   //
// modifications thereof for hardware or software products claiming      //
// conformance to the MPEG-4 standard.                                   //
//                                                                       //
// Those intending to use this software module in hardware or software   //
// products are advised that use may infringe existing  patents. The     //
// original developers of this software module and their companies, the  //
// subsequent editors and their companies, and ISO/IEC have no liability //
// and ISO/IEC have no liability for use of this software module or      //
// modification thereof in an implementation.                            //
//                                                                       //
///////////////////////////////////////////////////////////////////////////
//
// File: AudRend-CSELT-DS.h
//
// Author: Matthew Leditschke, Ananda Allys
//
// Description:
//   The definition of a class which provides a DirectSound based audio
//   renderer.
//
// Created: August, 1997.
//
// Modified: 1998
//
///////////////////////////////////////////////////////////////////////////


#ifndef AUD_REND_MM_H
#define AUD_REND_MM_H


#include "AudRend-CSELT.h"
#include "ztl.h"
#include <mmsystem.h>
#include <dsound.h>

typedef struct {
   AudioSourceProxy *m_pSrc;
   DWORD m_startTime;
   int   m_number;
} t_mmstream;


class AudioRendererMM : public AudioRenderer
{
	//
	// Public functions
	//
public:

	// Construct the audio renderer, given a pointer to the current
	// Presenter object, and the handle of the window the application
	// is running in.
	AudioRendererMM(Presenter* pPresenter, HWND hWnd);

	// Class destructor.
	virtual ~AudioRendererMM();

	// This is the main loop of the audio renderer thread.
	void Main();

	// Methods inherited from the AudioRenderer base class.
	virtual void GetCapabilities (Capabilities* pPresentationCaps);
	virtual void Init();
	virtual void Terminate();
	virtual void Start();
	virtual void AddSourceNode(AudioSourceProxy* pSourceNode,
							   DWORD startTime, DWORD nSamplesPerSec,
							   DWORD nBitsPerSample, WORD nNumChannels);
	virtual void RemoveSourceNode(AudioSourceProxy* pSourceNode);
	virtual DWORD GetBufferLengthMs();


	//
	// Protected implementation details
	//
protected:


	//
	// Private implementation details
	//
private:

	// This is the audio format output by the sound card
	WAVEFORMATEX m_wFmt;

	// Used to flags the presenter thread when it is time to finish.
	BOOL m_bActive;

	// This mutex is used by Terminate() to wait until the thread
	// has finished.
	ZMutex m_mtxThread;

	// This mutex controls access to the array of stream information
	// data, as multiple threads access this.
	ZMutex m_mtxStreamArray;

	// The array of sound streams which have been created
   ZArray<t_mmstream *> m_streams;

	// Given a stream for which the audio data format is not known, query
	// the node for this information, and then update the output format
	// that the sound card is currently using.
	//void UpdateOutputFormat(StreamInfo* pStream);

};


#endif // AUD_REND_MM_H
