//This software module was originally developed by CSELT in the course of   //
//development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.             //
//This software module is an implementation of a part of one or more MPEG-4 //
//Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems        //
//(ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 Systems     //
//(ISO/IEC 14496-1) free license to this software module or modifications   //
//thereof for use in hardware or software products claiming conformance to  //
//the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use this software//
//module in hardware or software products are advised that its use may      //
//infringe existing patents.                                                //
//The original developer of this software module and his/her company, the   //
//subsequent editors and their companies, and ISO/IEC have no liability     //
//for use of this software module or modifications thereof in an            //
//implementation. Copyright is not released for non MPEG-4 Systems          //
//(ISO/IEC 14496-1) conforming products. CSELT S.pA. retains full right     //
//to use the code for his/her own purpose, assign or donate the code to a   //
//third party and to inhibit third parties from using the code for non      //
//MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright      //
//notice must be included in all copies or derivative works.                //
//Copyright  1999.                                                         //
//This work has been partly developed within the framework of the ESPRIT    //
//European Research Project  "MPEG-4 PC".                                   //
//////////////////////////////////////////////////////////////////////////////
//
// File:  AudioSourceProxy.h
//
// Author: Matthew Leditschke
//
// Modified: Gianluca Di Cagno  gianluca.dicagno@cselt.it
//
// Description:
//   The definition of the proxy object used by the presenter
//   to render the AudioSource node.
//
// Created: August, 1997.
//
//
///////////////////////////////////////////////////////////////////////////
/*-------------------------------------------------------------------------
MPEGJ
AUTHORS:    Viswanathan Swaminathan - vishy@eng.sun.com
			Robert Schmidt(RLS)
			Praveen Alavilli(PA)
			YUNTAI KYONG(YT) - yuntaikyong@ieee.org
-------------------------------------------------------------------------*/


#ifndef AUDIO_SOURCE_PROXY_H
#define AUDIO_SOURCE_PROXY_H


#include "Nodes-CSELT.h"
#include "Traversable.h"

#include "sound2Dproxy.h"


//for function pointer table - PA
#ifdef MPGJ
#include "..\mpegj\MPEGJTerminalInterface.h"
#endif


class AudioSourceProxy : public Traversable
{
public:

	#ifdef MPGJ
	/***** MPEGJ _ MERGE - handle to the function table - PA *****/
	MPJTerminalDispatchTablePtr di;
	/***** -end - ****/
	#endif // MPEGJ


	AudioSourceProxy(MediaObject* pMO);
	~AudioSourceProxy();
	void DeleteObject();

	AudioSource* GetAudioSource() const { return (AudioSource*)GetObject(); }
	// Return a pointer to the AudioSource node that this node is the proxy for

	MediaStreamImp* GetStream() const { return GetAudioSource()->GetStream(); }

	// Return a pointer to the stream that this node reads from.

	void GetData(DWORD nBytesRequired, LPBYTE& rpData, DWORD& rnBytesGiven,DWORD &);
	// This function is used by the audio renderer to get more data
	// to play.  The audio render will ask for a given number of bytes
	// (which will be a multiple of the number of bytes required for a
	// single sample), and the audio source node will return a pointer
	// to some data and the number of bytes of data pointed to.
	//
	// The number of bytes returned can be less than that asked for.
	// If fewer bytes were returned to the audio renderer than expected,
	// then the audio renderer will ask for more.  If zero bytes are
	// returned, then the audio renderer will assume that no data can be
	// given, and will insert silence.

	void DataUsed(LPBYTE pData, DWORD nBytesGiven);
	// This function is used by the audio renderer to indicate that it
	// has finished with some data given to it by GetData().  The
	// parameters given to this function will be the same as those
	// returned by GetData().

   void        SetSound2DProxy(Sound2DProxy *pSound2DProxy) { m_pSound2DProxy = pSound2DProxy; }
   Sound2DProxy *GetSound2DProxy()                        { return m_pSound2DProxy;  }


   // Set and return Sound2DProxy associated with SourceNode
	float GetIntensity();
   // Functions inherited from the base class
	void PreRender(Effects & effects);
	float GetDrawingOrder() const { return (float)0; }
	BOOL IsFinished() const { return m_isFinished; }

	BOOL m_registeredWithRenderer;

private:
//	BOOL m_registeredWithRenderer;
	BOOL m_skippedHeader;
	DWORD m_partialTime;
	DWORD m_partialLen;
	DWORD m_partialStoreAllocatedSize;
	unsigned char* m_partialStore;
	BOOL m_isFinished;
	long m_startUpDelay;
	BOOL m_firstTimeAtStartTime;
	BOOL m_firstTimeAtStopTime;
	int m_timesStarted;
	bool m_bIsFirstFrame;

   Sound2DProxy *m_pSound2DProxy;
};


#endif // AUDIO_SOURCE_PROXY_H


// Emacs uses these...
// Local Variables:
// mode: c++
// tab-width: 4
// End:
