//////////////////////////////////////////////////////////////////////////////
//This software module was originally developed by CSELT in the course of   //
//development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.             //
//This software module is an implementation of a part of one or more MPEG-4 //
//Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems        //
//(ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 Systems     //
//(ISO/IEC 14496-1) free license to this software module or modifications   //
//thereof for use in hardware or software products claiming conformance to  //
//the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use this software//
//module in hardware or software products are advised that its use may      //
//infringe existing patents.                                                //
//The original developer of this software module and his/her company, the   //
//subsequent editors and their companies, and ISO/IEC have no liability     //
//for use of this software module or modifications thereof in an            //
//implementation. Copyright is not released for non MPEG-4 Systems          //
//(ISO/IEC 14496-1) conforming products. CSELT S.pA. retains full right     //
//to use the code for his/her own purpose, assign or donate the code to a   //
//third party and to inhibit third parties from using the code for non      //
//MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright      //
//notice must be included in all copies or derivative works.                //
//Copyright  1999.                                                         //
//This work has been partly developed within the framework of the ESPRIT    //
//European Research Project  "MPEG-4 PC".                                   //
//////////////////////////////////////////////////////////////////////////////
//
// File:  Background2DProxy.cpp
//
// Author: Matthew Leditschke 
//
// Modified:Gianluca Di Cagno  gianluca.dicagno@cselt.it
//
// Description:
//   The implementation of the proxy object used by the presenter
//   to render the Background2D node.
//
// Created: September, 1997.
//
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include "Nodes-CSELT.h"
#include "Background2DProxy.h"
#include "Bindable.h"
#include "trace.h"
// =====================================================================

//
// Presenter::InitBackground2D
//
// Initialise the proxy data structure and attach it to the render data
// pointer of the given media object node.
//

 Background2DProxy* Presenter::InitBackground2D(Background2D* pObj)
{
	return (new Background2DProxy(pObj)); 
}


// =====================================================================

//
// Background2DProxy::Background2DProxy
//
// Class constructor.
//

Background2DProxy::Background2DProxy(MediaObject* pMO) : DrawableImage(pMO)
{
	m_firstTime = TRUE;
	
	//if it is the first Background encountered, bound it
	if(GetVisualRenderer()->GetBackgroundStack()->IsEmpty())
		GetBackground2D()->isBound = TRUE;
	
	GetStack()->AddBindable(this);

	m_context.m_pDrawable = this;

	m_context.m_updateRequired=FALSE;
	m_context.m_needsDrawing=FALSE;
	m_context.m_transform.SetIdentity();
	m_context.GetClippingRect()->MakeEmpty();
	m_context.GetClippedBoundingRect()->MakeEmpty();

	m_previousBackColor = Color3f(0,0,0);

} /* Background2DProxy::Background2DProxy */

Background2DProxy::~Background2DProxy() 
{
	
} /* Background2DProxy::~Background2DProxy */
void Background2DProxy::DeleteObject() 
{
	GetStack()->RemoveBindable(this);
} /* Background2DProxy::DeleteObject */


// =====================================================================

//
// Background2DProxy::PreRender
//
// Call the PreRender() method from the base class, and then adjust the
// bounding rectangle which has just been calculated, so that the
// top-left corner of the bounding rectangle is at the top-left corner
// of the screen, if the screen height is equal to the object height,
// to the difference in the other case.

void Background2DProxy::PreRender(Effects & effects)
{
	// The first time that this is called, the stream needs to be
	// started.
	if (m_firstTime)
	{
		m_firstTime = FALSE;
		GetBackground2D()->OpenStream();
	}

   	
//	DrawableContext *pContext= InitBackgroundContext(effects);
	if(effects.m_ParentGroup)
		effects.m_ParentGroup->AddToDrawableContextList(&m_context);
		
	
	//do nothing if the Background is not on top of the Backgrounds stack
	//should I check the stack too?
	if ((!effects.isSwitchedOn)||(!(Bool)GetBackground2D()->isBound)) 
		return;

	unsigned int iScreenWidth = GetVisualRenderer()->GetScreenWidth();
	unsigned int iScreenHeight = GetVisualRenderer()->GetScreenHeight();
	int width = iScreenWidth; 
	int height = iScreenHeight; 
	
	m_screenBRect = BRect(-width/2,height/2,iScreenWidth,iScreenHeight);
	
	if	((GetBackground2D()->GetUrl(0))!=NULL)
	{
		DrawableImage::PreRender(effects,&m_context);
		m_context.GetBoundingRect()->left = -width/2;
		m_context.GetBoundingRect()->top =  height/2;
	}
	else
	{
		
				
		m_context.GetBoundingRect()->MakeEmpty();
		
		m_context.GetBoundingRect()->SetDimensions(m_screenBRect);
		if (!((Color3f)GetBackground2D()->backColor==m_previousBackColor))
		{
			m_context.m_needsDrawing=TRUE;
			m_previousBackColor=(Color3f)GetBackground2D()->backColor;
		}
	}
/*	m_boundingRect.left = 0;
	m_boundingRect.top  = m_boundingRect.height; */
	
	//UpdateBackgroundBRect(pContext);
} /* Background2DProxy::PreRender */

// =====================================================================
//Background2DProxy::Draw
//
//
void Background2DProxy::Draw(BRect const* pClipBRect,DrawableContext *pDrawableContext)
{
	if	((GetBackground2D()->GetUrl(0))!=NULL)
	{
		//if the clipping rect is empty the clipped and the boundin are equal
		//if not this means that a layer or layout exist, and they have set the 
		//clipping rect of the background and then called the UpdateBoundingRect
		//function of this node to set the clippedrect**removed
		if(m_context.GetClippingRect()->IsEmpty()) //always TRUE clippin of background is removed 
			m_context.GetClippedBoundingRect()->SetDimensions(*pClipBRect);
		DrawableImage::Draw(pClipBRect,&m_context);
		
	}
	else
	{
		BRect visibleRect = *pClipBRect;
		if(m_context.GetClippingRect()->IsEmpty()) //always TRUE clippin of background is removed
			m_context.GetClippedBoundingRect()->SetDimensions(*pClipBRect);
		visibleRect.Intersection(m_context.GetClippedBoundingRect());
		if (visibleRect.IsEmpty())
			return;
		GetVisualRenderer()->DisplayRectangle(visibleRect,(* m_context.GetClippedBoundingRect())/*m_drawRect*/, GetBackground2D()->backColor,
							  1, TRUE, 0);	
	
	}
	//restore context default 
	m_context.m_updateRequired=FALSE;
	m_context.m_needsDrawing=FALSE;

} /* Background2DProxy::Draw */


// =====================================================================
//Background2DProxy::InitBackgroundContext
//
//

/*DrawableContext * Background2DProxy::InitBackgroundContext(Effects &effects)
{
	DrawableContext *pContext= GetVisualRenderer()->GetBackgroundContext();
	
	pContext->m_isSwitchedOn = effects.isSwitchedOn;
    
	pContext->m_pDrawable=this;
	
	
	return pContext;
} /*InitBackgroundContext*/
// ========================================================================

//
// Background2D ::Onset_bind
//
// This function is called when the set_fraction data member has changed,
// and it has already been updated when this function is called.  When
// this happens, calculate the value that corresponds with this fraction,
// and update the value_changed data member.
//
//add to the denstrunctor of proxy the  call onsetbind =FALSE
void  Background2D::Onset_bind() 
{
	Background2DProxy* pBackground2DProxy = ((Background2DProxy*)GetRenderingObject());
	
	BindableStack *pBackStack = pBackground2DProxy->GetStack();
	
	if( ! (Bool) set_bind)
	{
		BOOL isOnTop= pBackStack->IsOnTop(pBackground2DProxy);
		TRACE_PRINT("SET_BIND=FALSE\n");
		//remove node from the stack, if it is not there do nothing 
		
		if(!pBackStack->RemoveBindable(pBackground2DProxy))
			return;
		
		//if it was on the top of the stack
		if(isOnTop)
		{
			isBound=FALSE;
		//	pBackStack->RemoveBindable(pBackground2DProxy);
				
			// try to pop the stack
			Background2DProxy* pBackOnTop=pBackStack ->GetTop();
			if(pBackOnTop!=NULL)
				pBackOnTop->GetBackground2D()->isBound=TRUE;
		}
	}
	else //Set_Bind TRUE
	{
		if(pBackStack->IsOnTop(pBackground2DProxy))
			return;
		else
		{
			TRACE_PRINT("SET_BIND=TRUE\n");
			Background2DProxy* pBackOnTop=pBackStack ->GetTop();
			if(pBackOnTop!=NULL)
			{
				TRACE_PRINT("pBackOnTop!=NULL\n");
				pBackOnTop->GetBackground2D()->isBound=FALSE;
				BOOL result=pBackStack->MoveToTop(pBackground2DProxy);
				TRACE_PRINT1("result, %d\n",result);
				isBound=TRUE;
			}
		}
		
		
	}

}
BindableStack * Background2DProxy::GetStack()
{
	return GetVisualRenderer()->GetBackgroundStack();
}


void Background2DProxy::UpdateBoundingRect(DrawableContext * pContext)
{
//**	removed** background in layer not implemented.
	assert(0);
	/*	BRect *pClippingBRect=pContext->GetClippingRect();
		if(!pClippingBRect->IsEmpty())
		{
			BRect clippedBRect=*(pContext->GetBoundingRect());
		    clippedBRect.Intersection(pClippingBRect);
			pContext->GetClippedBoundingRect()->SetDimensions(clippedBRect);
		}*/

}