//////////////////////////////////////////////////////////////////////////////
//This software module was originally developed by CSELT in the course of   //
//development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.             //
//This software module is an implementation of a part of one or more MPEG-4 //
//Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems        //
//(ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 Systems     //
//(ISO/IEC 14496-1) free license to this software module or modifications   //
//thereof for use in hardware or software products claiming conformance to  //
//the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use this software//
//module in hardware or software products are advised that its use may      //
//infringe existing patents.                                                //
//The original developer of this software module and his/her company, the   //
//subsequent editors and their companies, and ISO/IEC have no liability     //
//for use of this software module or modifications thereof in an            //
//implementation. Copyright is not released for non MPEG-4 Systems          //
//(ISO/IEC 14496-1) conforming products. CSELT S.pA. retains full right     //
//to use the code for his/her own purpose, assign or donate the code to a   //
//third party and to inhibit third parties from using the code for non      //
//MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright      //
//notice must be included in all copies or derivative works.                //
//Copyright  1999.                                                         //
//This work has been partly developed within the framework of the ESPRIT    //
//European Research Project  "MPEG-4 PC".                                   //
//////////////////////////////////////////////////////////////////////////////
//
// File:  BitmapProxy.cpp
//
// Author: Gianluca Di Cagno  gianluca.dicagno@cselt.it
//
// Modified:
//
// Description:
//   The implementation of the proxy object used by the presenter
//   to render the Bitmap node.
//
// Created: 1998.
//
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include "Nodes-CSELT.h"
#include "BitmapProxy.h"
#include "DrawableImage.h"
#include "Stream.h"
#include "trace.h"


// =====================================================================

//
// Presenter::InitBitmap
//
// Initialise the proxy data structure and attach it to the render data
// pointer of the given media object node.
//

 BitmapProxy * Presenter::InitBitmap(Bitmap* pObj)
 {
	return new BitmapProxy(pObj);
 }


// =====================================================================

//
// BitmapProxy::BitmapProxy
//
// Class constructor.
//

BitmapProxy::BitmapProxy(MediaObject* pMO) : GeometryNode(pMO)
{
	
} /* BitmapProxy::BitmapProxy */






// =====================================================================

//
// BitmapProxy::PreRender
//
//

void BitmapProxy::PreRender(Effects & effects)
{
	DrawableContext *pContext=InitPreRender(effects);
	
	// indeed here we should return if pContext is NULL
	// but we need to stop the associated decoders so we must call
	// the texture node prerender function either way
	

	


	MediaObject* pAppearance = ((MediaObject*)pContext->GetAppearance());

	if(pAppearance)
	{
		DrawableImage * pTextureProxy;
		void *pTexture=((Appearance *)pAppearance)->texture;
		if(pTexture)	
		{
			pTextureProxy =(DrawableImage*)((MediaObject*)pTexture)->GetRenderingObject();
	
			if(pTextureProxy!=NULL)
			{
				//we can render the node
				//if this drawable is within a Layout or Form node
					//add his drawableContext to the list of drawableContexts
					//of the parent node. 
					//this is useful for postprocessing.

					if(effects.m_ParentGroup)
						effects.m_ParentGroup->AddToDrawableContextList(pContext);
	
				//if pContext is NULL next function MUST return
					pTextureProxy->PreRender(effects,pContext);
	
					if (!pContext->IsSwitchedOn()) 
						return;
			

					if(effects.m_ParentGroup==NULL)//necessary for postprocessing style nodes 
						UpdateBoundingRect(pContext);
			}	
		}
	}

} /* BitmapProxy::PreRender */


// =====================================================================
//
// BitmapProxy::Draw
//
//
void BitmapProxy::Draw(BRect const* pClipBRect,DrawableContext * pDrawableContext)
{
	MediaObject* pAppearance = ((MediaObject*)pDrawableContext->GetAppearance());

	if(pAppearance)
	{
		DrawableImage * pTextureProxy;
		void *pTexture=((Appearance *)pAppearance)->texture;
		if(pTexture)	
		{
			pTextureProxy =(DrawableImage*)((MediaObject*)pTexture)->GetRenderingObject();
	
			if(pTextureProxy!=NULL)
			{

//	DrawableImage * pTexture = (DrawableImage *) ((MediaObject*)pDrawableContext->GetAppearance()->texture)->GetRenderingObject();
  // in the Drawable Context there is the bounding box info
  // but also pointers to the associated Appearance node 
  // which contains the material and so on.
  // The texture draw function shall then render the bitmap.
			  pTextureProxy->Draw(pClipBRect,pDrawableContext);
			}
		}
	}
} /* BitmapProxy::Draw */

// =====================================================================

