//////////////////////////////////////////////////////////////////////////////
//This software module was originally developed by CSELT in the course of   //
//development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.             //
//This software module is an implementation of a part of one or more MPEG-4 //
//Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems        //
//(ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 Systems     //
//(ISO/IEC 14496-1) free license to this software module or modifications   //
//thereof for use in hardware or software products claiming conformance to  //
//the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use this software//
//module in hardware or software products are advised that its use may      //
//infringe existing patents.                                                //
//The original developer of this software module and his/her company, the   //
//subsequent editors and their companies, and ISO/IEC have no liability     //
//for use of this software module or modifications thereof in an            //
//implementation. Copyright is not released for non MPEG-4 Systems          //
//(ISO/IEC 14496-1) conforming products. CSELT S.pA. retains full right     //
//to use the code for his/her own purpose, assign or donate the code to a   //
//third party and to inhibit third parties from using the code for non      //
//MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright      //
//notice must be included in all copies or derivative works.                //
//Copyright  1999.                                                         //
//This work has been partly developed within the framework of the ESPRIT    //
//European Research Project  "MPEG-4 PC".                                   //
//////////////////////////////////////////////////////////////////////////////
//
// File:  CircleProxy.cpp
//
// Author: Antonio Arena
//
// Modified:
//
// Description:
//   The implementation of the proxy object used by the presenter
//   to render the Circle node.
//
// Created: September, 1997.
//
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include "Nodes-CSELT.h"
#include "CircleProxy.h"
#include "trace.h"
#include <assert.h>

// =====================================================================

//
// Presenter::InitCircle
//
// Initialise the proxy data structure and attach it to the render data
// pointer of the given media object node.
//

CircleProxy* Presenter::InitCircle(Circle* pObj)
{
	return (new CircleProxy(pObj)); 
}



// ======================================================================

//
// CircleProxy::PreRender
//
//

void CircleProxy::PreRender(Effects & effects)
{
	DrawableContext *pContext=InitPreRender(effects);
	
	if (!pContext->IsSwitchedOn()) 
		return;

	//if this drawable is within a Layout or Form node
	//add his drawableContext to the list of drawableContexts
	//of the parent node. 
	//this is useful for postprocessing.

	if(effects.m_ParentGroup)
		effects.m_ParentGroup->AddToDrawableContextList(pContext);

	// If the transparency for this node is 1, or it is not filled and
	// the line width is 0, then it doesn't need to be drawn, and its
	// bounding rectangle is empty.
	if ((GetTransparency(pContext) > 1 - ALMOST_ZERO) ||
		(! GetFilled(pContext) && (GetWidth(pContext) == 0)))
	{
		pContext->GetBoundingRect()->MakeEmpty();
		UpdateBoundingRect(pContext);
		return;
	}

	assert(!pContext->m_transform.ContainsRotation());

	// Put the center always in the origin
	// and then apply the parent's transformations
	// which also determine the circle's bounding box.
	int radius = (float)GetCircle()->radius;

	// Calculate bounding rectangle taking into account the width!
	m_lpPoints[0].x = m_lpPoints[1].y = -radius-GetWidth(pContext)/2;
	m_lpPoints[0].y = m_lpPoints[1].x = radius+GetWidth(pContext)/2;



	// Find out where the circle should be drawn
	for (int i = 0; i< 2; i++)
	{
		pContext->m_transform.Apply(m_lpPoints[i].x, m_lpPoints[i].y);
		
	}

	pContext->GetBoundingRect()->SetDimensions(m_lpPoints, 2);

    // restore the bounding rectangle coordinates
	
	m_lpPoints[0].x = m_lpPoints[1].y = -radius;
	m_lpPoints[0].y = m_lpPoints[1].x = radius;
		 

	// again sorry..
	for (i = 0; i< 2; i++)
	{
			pContext->m_transform.Apply(m_lpPoints[i].x, m_lpPoints[i].y);
	
	}
	
	if(effects.orderHasChanged)
			pContext->m_updateRequired = TRUE;

	if(effects.m_ParentGroup==NULL)//necessary for postprocessing style nodes 
		UpdateBoundingRect(pContext);
} /* CircleProxy::PreRender */


// ============================================================================

//
// CircleProxy::Draw
//
//

void CircleProxy::Draw(BRect const* pClipBRect,DrawableContext * pContext)
{
	assert(pClipBRect != NULL);

	BRect visibleRect = *pClipBRect;
	visibleRect.Intersection(pContext->GetClippedBoundingRect());
	if (visibleRect.IsEmpty())
		return;

	// Now draw the circle
	GetVisualRenderer()->DisplayCircle(visibleRect, m_lpPoints, GetEmissiveColor(pContext), 
		GetWidth(pContext), GetFilled(pContext));
} /* CircleProxy::Draw */


// ===========================================================================
