/***********************************************************************
This software module was originally developed by Mikael Bourges-Sevenier
(Mindego Inc.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Mindego Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Mikael Bourges-Sevenier at mikael@mindego.com.
----------------------------------------------------------------------*/
#include "OpenGl/OpenGl.h"
#include "CoordinateInterpolator4DProxy.h"

CoordinateInterpolator4DProxy::CoordinateInterpolator4DProxy(MediaObject * poNode) :
	InterpolatorNodeProxy(poNode)
{}

inline BOOL CoordinateInterpolator4DProxy::SetValue(CoordinateInterpolator4D * poCI, int iIndex)
{
  int arraySize = poCI->keyValue.GetLength() / poCI->key.GetLength();
  int offset = arraySize * iIndex;

  // sanity test
  assert(poCI->keyValue.GetLength() % poCI->key.GetLength() == 0);

  // Get rid of all old values
  poCI->value_changed.Remove();

  // Set coordinates directly
  for (int j = 0; j < arraySize; ++j)
    poCI->value_changed.Append(poCI->keyValue[offset + j]);

  return TRUE;
}

BOOL CoordinateInterpolator4DProxy::Evaluate()
{
  CoordinateInterpolator4D * poCI = (CoordinateInterpolator4D *) GetMediaObject();

  // Check if we have any keys
  if (poCI->key.GetLength() > 0)
  {
    // If we only have one key
    if (poCI->key.GetLength() == 1)
    {
      return SetValue(poCI, 0);
    }
    else // more keys
    {
      // If set_fraction is smaller than smallest key
      if (poCI->set_fraction <= poCI->key[0])
      {
        return SetValue(poCI, 0);
      }
      else
      {
        // If set_fraction is larger than largest key
        if (poCI->set_fraction >= poCI->key[poCI->key.GetLength() - 1])
        {
          return SetValue(poCI, poCI->key.GetLength() - 1);
        }
        else
        {
          // Search the key space for fraction
          for (int i = poCI->key.GetLength() - 2; i >= 0 ; --i)
          {
            if (poCI->key[i] == poCI->set_fraction)
            {
              // Found exact match
              return SetValue(poCI, i);
            }
            else
            {
              if (poCI->key[i] < poCI->set_fraction)
              {
                // Didn't find exact match; must interpolate
                return InterpolateValue( 
                  i, i + 1, 
                  ((poCI->key[i + 1] - poCI->set_fraction) / 
                  (poCI->key[i + 1] - poCI->key[i])) );
              }
            }
          } // for
        }
      }
    } // one or more keys
  } // some key

  return FALSE;
}

BOOL CoordinateInterpolator4DProxy::InterpolateValue(int nMinIndex, int nMaxIndex, float fFraction)
{
  CoordinateInterpolator4D * poCI = (CoordinateInterpolator4D *) GetMediaObject();

  int arraySize = poCI->keyValue.GetLength() / poCI->key.GetLength();
  int min = arraySize * nMinIndex;
  int max = arraySize * nMaxIndex;

  // Sanity test

  assert(poCI->keyValue.GetLength() % poCI->key.GetLength() == 0);

  // Get rid of all old values

  poCI->value_changed.Remove();

  // Interpolate coordinates

  for (int i = 0; i < arraySize; ++i) {
    Vec4f tempCoordinate;

    tempCoordinate.x = (poCI->keyValue[max + i].x + (poCI->keyValue[min + i].x - poCI->keyValue[max + i].x) * fFraction);
    tempCoordinate.y = (poCI->keyValue[max + i].y + (poCI->keyValue[min + i].y - poCI->keyValue[max + i].y) * fFraction);
    tempCoordinate.z = (poCI->keyValue[max + i].z + (poCI->keyValue[min + i].z - poCI->keyValue[max + i].z) * fFraction);
    tempCoordinate.q = (poCI->keyValue[max + i].q + (poCI->keyValue[min + i].q - poCI->keyValue[max + i].q) * fFraction);

    poCI->value_changed.Append(tempCoordinate);
  }

  return TRUE;
}
