//////////////////////////////////////////////////////////////////////////////
//This software module was originally developed by CSELT in the course of   //
//development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.             //
//This software module is an implementation of a part of one or more MPEG-4 //
//Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems        //
//(ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 Systems     //
//(ISO/IEC 14496-1) free license to this software module or modifications   //
//thereof for use in hardware or software products claiming conformance to  //
//the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use this software//
//module in hardware or software products are advised that its use may      //
//infringe existing patents.                                                //
//The original developer of this software module and his/her company, the   //
//subsequent editors and their companies, and ISO/IEC have no liability     //
//for use of this software module or modifications thereof in an            //
//implementation. Copyright is not released for non MPEG-4 Systems          //
//(ISO/IEC 14496-1) conforming products. CSELT S.pA. retains full right     //
//to use the code for his/her own purpose, assign or donate the code to a   //
//third party and to inhibit third parties from using the code for non      //
//MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright      //
//notice must be included in all copies or derivative works.                //
//Copyright  1999.                                                         //
//This work has been partly developed within the framework of the ESPRIT    //
//European Research Project  "MPEG-4 PC".                                   //
//////////////////////////////////////////////////////////////////////////////
// File:  Curve2DProxy.cpp
//
// Author: Antonio Arena, CSELT, arena@kim.cselt.it
//
// Modified: March, 1998.
//           Frederic Bouilhaguet, ENST, bouilhaguet@enst.fr
// Description:
//   The implementation of the proxy object used by the presenter
//   to render the Curve node.
//
// Created: September, 1997.
//
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include "Nodes-CSELT.h"
#include "Curve2DProxy.h"
#include "Coordinate2DProxy.h"
#include "trace.h"
#include <math.h>
#include <assert.h>

// =====================================================================

//
// Presenter::InitCurve2D
//
// Initialise the proxy data structure and attach it to the render data
// pointer of the given media object node.
//

Curve2DProxy* Presenter::InitCurve2D(Curve2D* pObj)
{
	return (new Curve2DProxy(pObj));
}


// ======================================================================

//
// Curve2DProxy::Curve2DProxy
//
// Class constructor.
//

Curve2DProxy::Curve2DProxy(MediaObject* pMO) : GeometryNode(pMO)
{
	m_lpPoints = NULL;
	m_shadows  = NULL;
	m_nPointsAllocatedLength = 0;
	m_lpTypes=NULL;
	m_lpPts=NULL;
} /* Curve2DProxy::Curve2DProxy */


// ======================================================================

//
// Curve2DProxy::~Curve2DProxy
//
// Class destructor .
//

Curve2DProxy::~Curve2DProxy()
{
	if (m_lpPoints != NULL)	delete [] m_lpPoints;
	if (m_shadows != NULL)	delete [] m_shadows;

	if (m_lpTypes != NULL)	delete [] m_lpTypes;
	if (m_lpPts != NULL)	delete [] m_lpPts;



} /* Curve2DProxy::~Curve2DProxy */





// ======================================================================

//
// Curve2DProxy::PreRender
//
//

void Curve2DProxy::PreRender(Effects & effects)
{
	DrawableContext *pContext=InitPreRender(effects);
	
	if (!pContext->IsSwitchedOn()) 
		return;
	
	if(effects.m_ParentGroup)
		effects.m_ParentGroup->AddToDrawableContextList(pContext);

	// If the transparency for this node is 1, or it is not filled and
	// the line width is 0, then it doesn't need to be drawn, and its
	// bounding rectangle is empty.
	if ((GetTransparency(pContext) > 1 - ALMOST_ZERO) ||
		(! GetFilled(pContext) && (GetWidth(pContext) == 0)))
	{
		pContext->GetBoundingRect()->MakeEmpty();
		UpdateBoundingRect(pContext);
		return;
	}

	// Allocate memory for the local copy of the points (only if the
	// current array is not large enough).
	m_nPointsLength = GetCoordinate2D()->point.GetLength();
	if (m_nPointsLength > m_nPointsAllocatedLength)
	{
		// Delete the previous arrays - if required
		if (m_lpPoints != NULL)	delete [] m_lpPoints;
		

		// Create the new arrays
		m_lpPoints = new PointF[m_nPointsLength];
		
		m_nPointsAllocatedLength = m_nPointsLength;


		m_needsTypesAlloc = TRUE;
		m_needsNewPtsAlloc = TRUE;

	}


	// Find out where the points and the shadow lie
	for ( int i =0; i < m_nPointsLength; i++)
	{
		m_lpPoints[i].x = (int) GetCoordinate2D()->point[i].x; 
		m_lpPoints[i].y = (int) GetCoordinate2D()->point[i].y;		
	}


		// Allocate memory for the local types 
		if ( m_needsTypesAlloc )
		{
			// Delete the previous arrays - if required
			if (m_lpTypes != NULL)	delete [] m_lpTypes;

			// Create the new TYPES arrays.
			m_lpTypes = new int[2*m_nPointsLength];

		}

		// Set the local types array with DirectDraw identifiers.
		int origNumTypes = GetCurve()->type.GetLength();
		int newNumTypes = 0;
		for ( i = 0; i < origNumTypes; i++) {
			int typeNum = GetType(i);
			switch (typeNum) {
				case 0 : 
					m_lpTypes[newNumTypes] = PT_MOVETO;
					break;
				case 1 :
					m_lpTypes[newNumTypes] = PT_LINETO;
					break;
				case 2 :
					m_lpTypes[newNumTypes]   = PT_BEZIERTO;
					m_lpTypes[newNumTypes++] = PT_BEZIERTO;
					m_lpTypes[newNumTypes++] = PT_BEZIERTO;
					break;
				case 3 :
					m_lpTypes[newNumTypes]   = PT_BEZIERTO;
					m_lpTypes[newNumTypes++] = PT_BEZIERTO;
					m_lpTypes[newNumTypes++] = PT_BEZIERTO;
					break;
			}
			newNumTypes++;
		}

		// Allocate memory for the local points.
		if ( m_needsNewPtsAlloc )
		{
			// Delete the previous arrays - if required
			if (m_lpPts != NULL)	delete [] m_lpPts;
		
			// Create the new arrays 
			//(with the same size as the new types one).
			m_lpPts = new PointF[newNumTypes];
		}


		// Set the new POINTS array to use in the DirectDraw's PolyDraw() function.
		int origCount = 0, newCount = 0;
		for ( i = 0; i < origNumTypes; i++) {
			int typeNum = GetType(i);
			switch (typeNum) {
				case 0 : // MoveTo : One coordinate pair is consumed. 
					m_lpPts[newCount].x = m_lpPoints[origCount].x; // Nx
					m_lpPts[newCount].y = m_lpPoints[origCount].y; // Ny	
					break;
				case 1 : // LineTo : One coordinate pair is consumed.
					m_lpPts[newCount].x = m_lpPoints[origCount].x; // Nx
					m_lpPts[newCount].y = m_lpPoints[origCount].y; // Ny
					break;
				case 2 : // CurveTO : Three coordinate pairs are consumed.
					m_lpPts[newCount].x = m_lpPoints[origCount].x;	  // C1x	
					m_lpPts[newCount].y = m_lpPoints[origCount].y;	  // C1y
					m_lpPts[++newCount].x = m_lpPoints[++origCount].x; // C2x
					m_lpPts[newCount].y = m_lpPoints[origCount].y; // C2y
					m_lpPts[++newCount].x = m_lpPoints[++origCount].x; // Nx
					m_lpPts[newCount].y = m_lpPoints[origCount].y; // Ny
					break;
				case 3 : // NextCurveto : Two  coordinate pairs are consumed, one is calculated
					 // C1              = 2*P              -                C2
					m_lpPts[newCount].x = 2*m_lpPts[newCount-1].x - m_lpPts[newCount-2].x;
					m_lpPts[newCount].y = 2*m_lpPts[newCount-1].y - m_lpPts[newCount-2].y;	

					m_lpPts[newCount++].x = m_lpPoints[origCount].x; // C2x
					m_lpPts[newCount++].y = m_lpPoints[origCount].y; // C2y
					m_lpPts[newCount++].x = m_lpPoints[origCount++].x; // Nx
					m_lpPts[newCount++].y = m_lpPoints[origCount++].y; // Ny
					break;
			}				 
			newCount++;
			origCount++;
		}
		m_nPtsLength = newCount;





	// Apply the parent's transformations
	// to both the points and the shadows
	for (i =0; i < m_nPtsLength; i++)
	{
		pContext->m_transform.Apply(m_lpPts[i].x, m_lpPts[i].y);
		
	}

	//void BRect::SetDimensions(PointF const* lpPoints, int numPoints)
	pContext->GetBoundingRect()->SetDimensions((PointF*)m_lpPts, m_nPtsLength);

/*
	// Apply the parent's transformations
	// to both the points and the shadows
	for (i =0; i < m_nPointsLength; i++)
	{
		pContext->m_transform.Apply(m_lpPoints[i].x, m_lpPoints[i].y);
		
	}

	pContext->GetBoundingRect()->SetDimensions(m_lpPoints, m_nLength);

*/

	if(effects.orderHasChanged)
			pContext->m_updateRequired = TRUE;

	if(effects.m_ParentGroup==NULL)//necessary for postprocessing style nodes 
		UpdateBoundingRect(pContext);

} /* Curve2DProxy::PreRender */


// ============================================================================

//
// Curve2DProxy::Draw
//
//

void Curve2DProxy::Draw(BRect const* pClipBRect,DrawableContext * pContext)
{
	assert(pClipBRect != NULL);

	BRect visibleRect = *pClipBRect;
	visibleRect.Intersection(pContext->GetClippedBoundingRect());
	if (visibleRect.IsEmpty())
		return;

	// Draw the curve

	// Use the DirectDraw's PolyDraw() function.
	GetVisualRenderer()->DisplayCurve(visibleRect, 
									  m_lpPoints,
									  m_lpTypes,	 // line and curve identifiers.			
									  GetEmissiveColor(pContext), 
									  GetWidth(pContext), 
									  m_nPtsLength); // new count of points.
/*
	// Use the DirectDraw's Polybezier() function.
	GetVisualRenderer()->DisplayCurve(visibleRect, 
									  m_lpPoints,
									  NULL,
									  GetEmissiveColor(pContext), 
									  GetWidth(pContext), 
									  m_nPointsLength);
*/
} /* Curve2DProxy::Draw */


// ============================================================================
