//////////////////////////////////////////////////////////////////////////////
//This software module was originally developed by CSELT in the course of   //
//development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.             //
//This software module is an implementation of a part of one or more MPEG-4 //
//Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems        //
//(ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 Systems     //
//(ISO/IEC 14496-1) free license to this software module or modifications   //
//thereof for use in hardware or software products claiming conformance to  //
//the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use this software//
//module in hardware or software products are advised that its use may      //
//infringe existing patents.                                                //
//The original developer of this software module and his/her company, the   //
//subsequent editors and their companies, and ISO/IEC have no liability     //
//for use of this software module or modifications thereof in an            //
//implementation. Copyright is not released for non MPEG-4 Systems          //
//(ISO/IEC 14496-1) conforming products. CSELT S.pA. retains full right     //
//to use the code for his/her own purpose, assign or donate the code to a   //
//third party and to inhibit third parties from using the code for non      //
//MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright      //
//notice must be included in all copies or derivative works.                //
//Copyright  1999.                                                         //
//This work has been partly developed within the framework of the ESPRIT    //
//European Research Project  "MPEG-4 PC".                                   //
//////////////////////////////////////////////////////////////////////////////
//
// File:  Curve2DProxy.h
//
// Author: Antonio Arena, CSELT, arena@kim.cselt.it
//
// Modified: March, 1998.
//           Frederic Bouilhaguet, ENST, bouilhaguet@enst.fr
// Description:
//   The definition of the proxy object used by the presenter
//   to render the Curve node.
//
// Created: September, 1997.
//
// Modified:
//
///////////////////////////////////////////////////////////////////////////


#ifndef CURVE2D_PROXY_H
#define CURVE2D_PROXY_H


#include "GeometryNode.h"


class Curve2DProxy : public GeometryNode
{
public:
	Curve2DProxy(MediaObject* pMO);
	~Curve2DProxy();

	// Return a pointer to the Curve node that this node is the proxy for
	Curve2D* GetCurve() const { return (Curve2D*)GetObject(); }

	// Return a pointer to the Coordinate2D object belonging to the
	// Curve2D node that this node is the proxy for.
	Coordinate2D* GetCoordinate2D() const
	{ return (Coordinate2D*)(GetCurve()->point); }


	// Return a type belonging to the
	// Curve2D node that this node is the proxy for.
	/*int* GetType() const 
	{ return (int*) (GetCurve()->type); }*/
	int GetType(int i)  const { return (int )GetCurve()->type[i]; }

	// Return the fineness value belonging to the
	// Curve2D node that this node is the proxy for.
	/*float* GetFineness() const 
	{ return (float*) (GetCurve()->fineness); }*/



	// Functions inherited from the base class
	virtual void PreRender(Effects & effects);

	virtual void Draw(BRect const* pClipBRect,DrawableContext * pContext);

public:
	PointF* m_lpPoints;
	int m_nPointsLength;
	int m_nPointsAllocatedLength;
	PointF* m_shadows;

	int* m_lpTypes;	
	//int m_nTypesLength;
	PointF* m_lpPts;
	int m_nPtsLength;


private :

	BOOL m_needsTypesAlloc;
	BOOL m_needsNewPtsAlloc;

};


#endif // CURVE2D_PROXY_H
