//This software module was originally developed by CSELT in the course of   //
//development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.             //
//This software module is an implementation of a part of one or more MPEG-4 //
//Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems        //
//(ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 Systems     //
//(ISO/IEC 14496-1) free license to this software module or modifications   //
//thereof for use in hardware or software products claiming conformance to  //
//the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use this software//
//module in hardware or software products are advised that its use may      //
//infringe existing patents.                                                //
//The original developer of this software module and his/her company, the   //
//subsequent editors and their companies, and ISO/IEC have no liability     //
//for use of this software module or modifications thereof in an            //
//implementation. Copyright is not released for non MPEG-4 Systems          //
//(ISO/IEC 14496-1) conforming products. CSELT S.pA. retains full right     //
//to use the code for his/her own purpose, assign or donate the code to a   //
//third party and to inhibit third parties from using the code for non      //
//MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright      //
//notice must be included in all copies or derivative works.                //
//Copyright  1999.                                                         //
//This work has been partly developed within the framework of the ESPRIT    //
//European Research Project  "MPEG-4 PC".                                   //
//////////////////////////////////////////////////////////////////////////////
//
// File: DirectDraw.cpp
//
// Author: Matthew Leditschke 
//
// Modified: Gianluca Di Cagno  gianluca.dicagno@cselt.it
//
// Description:
//   The implementation of a class which encapsulates all of the
//   DirectDraw related functions.
//
// Created: April, 1997
//
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"

#ifdef	  ACTIVEX_COMPONENT
#include "Mpeg4Cselt2DActivex.h"
#include "Mpeg4Cselt2DActivexCtl.h"
#elif defined (NETSCAPE_PLUGIN)
#include "applicationwrapper.h"
#else
#include "MPEG4Player.h"
#include "MPEG4PlayerView.h"
#include "WarningMessage.h"

#endif


#include <assert.h>
#include <ddraw.h>
#include <mmsystem.h>
#include "DirectDraw.h"
#include "dx-errors.h"
#include "trace.h"


// Define this if the DirectDraw clipper is to be used
//  - this also means that the display window is not top-most
#define USE_CLIPPER


// =============================================================================
		
//
// DirectDraw::DirectDraw
//
// Constructor - set all pointers to NULL.
//

DirectDraw::DirectDraw()
{
	lpDD = NULL;
	surfacePrimary = NULL;
	surfaceRGB = NULL;
	surfacePool = NULL;
	lpClipper = NULL;
} /* DirectDraw::DirectDraw */


// =============================================================================
		
//
// DirectDraw::Init
//
// This function initialises the direct draw routines, and creates
// all of the surfaces that are required.  Return NULL if everything
// went OK, otherwise return a string describing the error.
//

char* DirectDraw::Init (HWND hwnd, unsigned int compositeWidth,
						unsigned int compositeHeight)
{
    HRESULT ddrval;
	static char errorMessage[200];

   /*
    * Create the main DirectDraw object.
    *
    * This function takes care of initializing COM and Constructing
    * the DirectDraw object.
    */
    ddrval = DirectDrawCreate (NULL, &lpDD, NULL);
    if (ddrval != DD_OK)
    {
		strcpy (errorMessage, "Error in DirectDrawCreate():\n");
		strcat (errorMessage, HRESULTtoText(ddrval));
        return (errorMessage);
    }

	/*
	 * Set the cooperation level
	 *  - I would like to operate in a window, and so don't need to
	 *    set exclusive or fullscreen.
	 */
	ddrval = lpDD->SetCooperativeLevel (hwnd, DDSCL_NORMAL);
    if (ddrval != DD_OK)
    {
		strcpy (errorMessage, "Error in SetCooperativeLevel():\n");
		strcat (errorMessage, HRESULTtoText(ddrval));
        return (errorMessage);
    }

	/*
	 * Create a primary (on screen) surface to write to
	 */
	DDSURFACEDESC surfaceDesc;
	memset (&surfaceDesc, 0, sizeof(surfaceDesc));
	surfaceDesc.dwSize = sizeof(surfaceDesc);
	surfaceDesc.dwFlags = DDSD_CAPS;
	surfaceDesc.ddsCaps.dwCaps = DDSCAPS_PRIMARYSURFACE;

	ddrval = lpDD->CreateSurface (&surfaceDesc, &surfacePrimary, NULL);
	if (ddrval != DD_OK)
	{
		strcpy (errorMessage, "Error in CreateSurface():\n");
		strcat (errorMessage, HRESULTtoText(ddrval));
        return (errorMessage);
    }

	/*
	 * Determine what pixel formats this display can handle.
	 *  - in particular, what is the RGB format.
	 */
	DDPIXELFORMAT pixelFmt;
	memset (&pixelFmt, 0, sizeof(pixelFmt));
	pixelFmt.dwSize = sizeof(pixelFmt);
	ddrval = surfacePrimary->GetPixelFormat (&pixelFmt);
	if (ddrval != DD_OK)
	{
		strcpy (errorMessage, "Error in GetPixelFormat():\n");
		strcat (errorMessage, HRESULTtoText(ddrval));
        return (errorMessage);
    }

	// Handle RGB displays
	errorMessage[0] = 0;
	if (pixelFmt.dwFlags & DDPF_RGB)
	{
		// Handle 16 bits per pixel displays
		if (pixelFmt.dwRGBBitCount == 16)
		{
			if ((pixelFmt.dwRBitMask == 0xf800) &&
				(pixelFmt.dwGBitMask == 0x07e0) &&
				(pixelFmt.dwBBitMask == 0x001f))
				rgbPixelFormat = RGB565;

			else if ((pixelFmt.dwRBitMask == 0x7c00) &&
				(pixelFmt.dwGBitMask == 0x03e0) &&
				(pixelFmt.dwBBitMask == 0x001f))
				rgbPixelFormat = RGB555;

			else
				sprintf (errorMessage,
						 "Unsupported 16 bpp RGB masks: %04x %04x %04x",
						 pixelFmt.dwRBitMask,
						 pixelFmt.dwGBitMask,
						 pixelFmt.dwBBitMask);
		}

		// Handle 24 bits per pixel displays
		else if (pixelFmt.dwRGBBitCount == 24)
		{
			if ((pixelFmt.dwRBitMask == 0xff0000) &&
				(pixelFmt.dwGBitMask == 0x00ff00) &&
				(pixelFmt.dwBBitMask == 0x0000ff))
				rgbPixelFormat = RGB24bpp;
			else
				sprintf (errorMessage,
						 "Unsupported 24 bpp RGB masks: %06x %06x %06x",
						 pixelFmt.dwRBitMask,
						 pixelFmt.dwGBitMask,
						 pixelFmt.dwBBitMask);
		}
		// Handle 32 bits per pixel displays
		else if (pixelFmt.dwRGBBitCount == 32)
		{
			if ((pixelFmt.dwRBitMask == 0xff0000) &&
				(pixelFmt.dwGBitMask == 0x00ff00) &&
				(pixelFmt.dwBBitMask == 0x0000ff))
				rgbPixelFormat = RGB32bpp;
			else
				sprintf (errorMessage,
						 "Unsupported 32 bpp RGB masks: %08x %08x %08x",
						 pixelFmt.dwRBitMask,
						 pixelFmt.dwGBitMask,
						 pixelFmt.dwBBitMask);
		}
		else
			strcpy (errorMessage, "Can only handle 16, 24 and 32 bit per pixel displays");
	}
	else
		strcpy (errorMessage, "This doesn't appear to be an RGB display!!\n");

	// Abort if an error has occurred
	if (errorMessage[0] != 0)
		return (errorMessage);

#ifdef USE_CLIPPER
    /*
	 * Create a clipper to ensure that our drawing stays inside our window.
	 */
    ddrval = lpDD->CreateClipper (0, &lpClipper, NULL);
    if (ddrval != DD_OK)
    {
		strcpy (errorMessage, "Error in CreateClipper():\n");
		strcat (errorMessage, HRESULTtoText(ddrval));
        return (errorMessage);
    }

	/*
	 * Setting it to our hwnd gives the clipper the coordinates from our window.
	 */
    ddrval = lpClipper->SetHWnd (0, hwnd);
    if (ddrval != DD_OK)
    {
		strcpy (errorMessage, "Error in SetHWnd():\n");
		strcat (errorMessage, HRESULTtoText(ddrval));
        return (errorMessage);
    }

    /*
	 * Attach the clipper to the primary surface.
	 */
    ddrval = surfacePrimary->SetClipper (lpClipper);
    if (ddrval != DD_OK)
    {
		strcpy (errorMessage, "Error in SetClipper():\n");
		strcat (errorMessage, HRESULTtoText(ddrval));
        return (errorMessage);
    }
#endif // USE_CLIPPER

	// Create the offscreen RGB surface
	HRESULT returnValue;
	BOOL bIsAllFine;
	bIsAllFine=Resize(compositeWidth, compositeHeight,returnValue);
	if(bIsAllFine)  
		return NULL;
	else
	{
		strcpy (errorMessage, "Error in SetClipper():\n");
		strcat (errorMessage, HRESULTtoText(returnValue));
        return (errorMessage);
	
	}

	
} /* DirectDraw::Init */


// =============================================================================

//
// DirectDraw::Resize
//
// Call this function to resize the rendering surfaces.  Return NULL
// if everything went OK, otherwise return a string describing the error.
//

BOOL DirectDraw::Resize (int newWidth, int newHeight,HRESULT &returnValue)
{
	
	
		// Delete the current and currently existing surfaces
	if (surfaceRGB != NULL)
	    surfaceRGB->Release();
	
	if (surfacePool != NULL)
	    surfacePool->Release();
	DDSURFACEDESC surfaceDesc;
	
		//
	// Create an offscreen RGB surface.
	///
    memset (&surfaceDesc, 0, sizeof(surfaceDesc));
    surfaceDesc.dwSize = sizeof(surfaceDesc);
    surfaceDesc.dwFlags = DDSD_CAPS | DDSD_HEIGHT | DDSD_WIDTH;
    surfaceDesc.ddsCaps.dwCaps = DDSCAPS_OFFSCREENPLAIN;
    surfaceDesc.dwWidth = newWidth;
    surfaceDesc.dwHeight = newHeight;
	
    returnValue = lpDD->CreateSurface (&surfaceDesc, &surfaceRGB, NULL);

	

    if (returnValue != DD_OK)
    	return FALSE;
    
	
	//
	// Create an offscreen Pool surface.
	///
    memset (&surfaceDesc, 0, sizeof(surfaceDesc));
    surfaceDesc.dwSize = sizeof(surfaceDesc);
    surfaceDesc.dwFlags = DDSD_CAPS | DDSD_HEIGHT | DDSD_WIDTH;
    surfaceDesc.ddsCaps.dwCaps = DDSCAPS_OFFSCREENPLAIN;
    surfaceDesc.dwWidth = newWidth;
    surfaceDesc.dwHeight =newHeight;
	
    returnValue = lpDD->CreateSurface (&surfaceDesc, &surfacePool, NULL);

	if (returnValue != DD_OK)
    	return FALSE;

	
	if(surfaceRGB)
	{
		BOOL bIsOk;

		bIsOk	= EraseSurface(NULL,returnValue);
		if(!bIsOk)
			return FALSE;
	
	}

	return TRUE;
} /* DirectDraw::Resize */


// =============================================================================

//
// DirectDraw::Shutdown
//
// Call this function to shut down the direct draw routines.  Currently this
// function deallocates the alternate YUV frame store and releases the main
// DirectDraw object.
//

void DirectDraw::Shutdown()
{
	if (surfaceRGB != NULL)
	    surfaceRGB->Release();
	if (surfacePool != NULL)
	    surfacePool->Release();
	if (lpClipper != NULL)
		lpClipper->Release();
	if (lpDD != NULL)
		lpDD->Release();

	surfaceRGB = NULL;
	surfacePool = NULL;
	surfacePrimary = NULL;
	lpClipper = NULL;
	lpDD = NULL;
} /* DirectDraw::Shutdown */


// =============================================================================
		
//
// DirectDraw::~DirectDraw
//
// Destructor - Calls shutdown.
//

DirectDraw::~DirectDraw()
{
	Shutdown();
} /* DirectDraw::~DirectDraw */


// =============================================================================

//
// DirectDraw::RenderSurface:
//
// This function renders the specifed region of the offscreen RGB surface
// into the destination rectangle provided.  The destination rectangle is
// given in screen coordinates.
//

BOOL DirectDraw::RenderSurface(RECT* pSrcRect, RECT* pDestRect,HRESULT & returnValue)
{
	assert(pSrcRect != NULL);
	assert(pDestRect != NULL);

	// Ensure that the source and destination have the same dimensions
	assert(pSrcRect->right - pSrcRect->left == pDestRect->right - pDestRect->left);
	assert(pSrcRect->bottom - pSrcRect->top == pDestRect->bottom - pDestRect->top);
   
/*
	TRACE_PRINT4("SrcRect: left = %d, top = %d, right = %d, bottom = %d\n",
				 pSrcRect->left, pSrcRect->top, pSrcRect->right, pSrcRect->bottom);
	TRACE_PRINT4("DestRect: left = %d, top = %d, right = %d, bottom = %d\n",
				 pDestRect->left, pDestRect->top, pDestRect->right, pDestRect->bottom);
*/

	// Copy the specifed rectangle
	returnValue = surfacePrimary->Blt(pDestRect, surfaceRGB, pSrcRect,
										 DDBLT_ASYNC, NULL);

	if ((returnValue!= DD_OK) && (returnValue !=DDERR_WASSTILLDRAWING ))
	{
		return FALSE;
		/*char errorMessage [200];
		strcpy (errorMessage, "Error in Blt():\n");
		strcat (errorMessage, HRESULTtoText(ddrval));
		#ifndef NETSCAPE_PLUGIN
		GetView() -> PostMessage(WM_DDRAW_PROBLEM, NULL,(LPARAM)(LPCSTR )errorMessage);
		#else 
		::PostMessage((GetExecutive()->GetPresenter())->GetWindow(),
						WM_DDRAW_PROBLEM,
						NULL,
						(LPARAM)(LPCSTR )errorMessage);
		#endif*/
	
	}
	return TRUE;
} /* DirectDraw::RenderSurface */


// =============================================================================

//
// DirectDraw::LockSurface
//
// Lock the offscreen RGB surface for writing.  The details of the
// surface are written into the given surface descriptor.  If an
// error occurs, a dialog box is displayed which shows the error.
// Once writing to this surface has finished, call UnlockSurface().
//
//newRenderer
BOOL DirectDraw::LockSurface(DDSURFACEDESC& surfaceDesc,WriteToSurface surf,HRESULT & returnValue)
{
	returnValue =DD_OK;
	surfaceDesc.dwSize = sizeof(surfaceDesc);
	
	switch (surf)
	{
		case RGB :

			while ((returnValue = surfaceRGB->Lock(NULL, &surfaceDesc, DDLOCK_WAIT, NULL)) != DD_OK)
					TRACE_PRINT("DirectDraw->LockSurface !=DD_OK\n");
			
			break;

		case POOL :
			while ((returnValue = surfacePool->Lock(NULL, &surfaceDesc, DDLOCK_WAIT, NULL)) == DDERR_WASSTILLDRAWING)
				TRACE_PRINT("DirectDraw->LockSurface !=DD_OK\n");
			break;
	}
    
	if(returnValue != DD_OK)
		return FALSE; 
	else
		return TRUE;
} /* DirectDraw::LockSurface */



// =============================================================================

//
// DirectDraw::UnlockSurface
//
// Unlock the offscreen RGB surface, now that it has been finished
// writing to.  If an error occurs, a dialog box is displayed which
// shows the error.
//

BOOL DirectDraw::UnlockSurface(WriteToSurface surf,HRESULT & returnValue)
{
	
	switch (surf)
	{
		case RGB :

			returnValue = surfaceRGB->Unlock (NULL);
			break;

		case POOL :

			returnValue = surfacePool->Unlock(NULL);	
			break;
		default:
			returnValue = 1; //error
	}
	
	if (returnValue  != DD_OK)
    	return FALSE;
		
    return TRUE;
} /* DirectDraw::UnlockSurface */


// =============================================================================

//
// DirectDraw::EraseSurface
//
// Erase the specifed region of the offscreen RGB surface, filling
// it with black.  If the rectangle is NULL, erase the entire
// surface.  The coordinates used in the rectangle are in window
// coordinates, with the origin at the top-left of the window.
//

BOOL DirectDraw::EraseSurface(RECT const* pRect, HRESULT & returnValue)
{
	DDSURFACEDESC surfaceDesc;

	BOOL bIsAllFine;
	
	bIsAllFine = LockSurface(surfaceDesc,RGB,returnValue);

	if(!bIsAllFine)
		return FALSE;
	
	if (pRect == NULL)
		memset(surfaceDesc.lpSurface, 0,
			   surfaceDesc.lPitch * surfaceDesc.dwHeight);
	else
	{
		int bytesPerPixel = (int)((surfaceDesc.ddpfPixelFormat.dwRGBBitCount - 1) / 8) + 1;

		for (int y = pRect->top; y < pRect->bottom; y++)
			memset((char*)surfaceDesc.lpSurface
					+ y * surfaceDesc.lPitch
					+ pRect->left * bytesPerPixel,
				   0, (pRect->right - pRect->left) * bytesPerPixel);
	}

	bIsAllFine = UnlockSurface(RGB,returnValue);

	if(!bIsAllFine)
		return FALSE;

	return TRUE;
} /* DirectDraw::EraseSurface */


// =============================================================================

//
// DirectDraw::GetHDC
//
// Get a handle to the device context to use for the offscreen RGB surface.
//

BOOL DirectDraw::GetHDC(HDC& hDC,HRESULT & returnValue)
{
	
	if(surfaceRGB==NULL)
				return FALSE;

	if((returnValue=surfaceRGB->IsLost()) !=DD_OK)
		        return FALSE;
	
	
	
	if((returnValue = surfaceRGB->GetDC(&hDC))!=DD_OK)
				return FALSE;

    return TRUE;

	
} /* DirectDraw::GetHDC */


// =============================================================================

//
// DirectDraw::ReleaseHDC
//
// Release the handle to the device context for the offscreen RGB surface.
//

BOOL DirectDraw::ReleaseHDC(HDC& hDC,HRESULT & returnValue)
{
	returnValue= surfaceRGB->ReleaseDC(hDC);
	if (returnValue != DD_OK)
		return FALSE;
	return TRUE;
} /* DirectDraw::ReleaseHDC */


// =============================================================================

BOOL DirectDraw::ScaleOffScreenRGB(RECT src,RECT dstScaled,HRESULT & returnValue)
{

 
	returnValue=surfaceRGB->Blt(&dstScaled,surfacePool ,&src,0,NULL);
	if (returnValue != DD_OK)
		return FALSE;
	return TRUE;

}