//This software module was originally developed by CSELT in the course of   //
//development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.             //
//This software module is an implementation of a part of one or more MPEG-4 //
//Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems        //
//(ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 Systems     //
//(ISO/IEC 14496-1) free license to this software module or modifications   //
//thereof for use in hardware or software products claiming conformance to  //
//the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use this software//
//module in hardware or software products are advised that its use may      //
//infringe existing patents.                                                //
//The original developer of this software module and his/her company, the   //
//subsequent editors and their companies, and ISO/IEC have no liability     //
//for use of this software module or modifications thereof in an            //
//implementation. Copyright is not released for non MPEG-4 Systems          //
//(ISO/IEC 14496-1) conforming products. CSELT S.pA. retains full right     //
//to use the code for his/her own purpose, assign or donate the code to a   //
//third party and to inhibit third parties from using the code for non      //
//MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright      //
//notice must be included in all copies or derivative works.                //
//Copyright  1999.                                                         //
//This work has been partly developed within the framework of the ESPRIT    //
//European Research Project  "MPEG-4 PC".                                   //
//////////////////////////////////////////////////////////////////////////////
//
// File: DirectDraw.h
//
// Author: Matthew Leditschke 
//
// Modified: Gianluca Di Cagno  gianluca.dicagno@cselt.it
//
// Description:
//   The definition of a class which encapsulates all of the
//   DirectDraw related functions.
//
// Created: April, 1997
//
//
///////////////////////////////////////////////////////////////////////////


#ifndef DIRECT_DRAW_H
#define DIRECT_DRAW_H


#include <ddraw.h>
#include "BRect.h"


// The different types of RGB displays that are supported
enum RGBpixelType { RGB565, RGB555, RGB24bpp, RGB32bpp, BGR32bpp };
enum WriteToSurface {RGB,POOL};

class DirectDraw
{
	//
	// Public functions
	//
public:

	// Constructor - set all pointers to NULL.
	DirectDraw::DirectDraw();

	// This function initialises the direct draw routines, and creates
	// all of the surfaces that are required.  Return NULL if everything
	// went OK, otherwise return a string describing the error.
	char* Init (HWND hwnd, unsigned int compositeWidth,
				unsigned int compositeHeight);

	// Call this function to resize the rendering surfaces.  Return TRUE
	// if everything went OK, otherwise return FALSE and a string describing the error.
	BOOL Resize (int newWidth, int newHeight,HRESULT & returnValue);

	// Call this function to shut down the direct draw routines.  Currently
	// this function deallocates the alternate YUV frame store and releases
	// the main DirectDraw object.
	void Shutdown();

	// Destructor - Calls shutdown.
	DirectDraw::~DirectDraw();

	// This function renders the specifed region of the offscreen RGB surface
	// into the destination rectangle provided.  The destination rectangle is
	// given in screen coordinates.
	BOOL RenderSurface(RECT* pSrcRect, RECT* pDestRect,HRESULT & returnValue);

	// Lock the offscreen RGB surface for writing.  The details of the
	// surface are written into the given surface descriptor.  If an
	// error occurs, a dialog box is displayed which shows the error.
	// Once writing to this surface has finished, call UnlockSurface().
	BOOL LockSurface(DDSURFACEDESC& surfaceDesc,WriteToSurface surf,HRESULT & retunValue);
	
	

	// Unlock the offscreen RGB surface, now that it has been finished
	// writing to.  If an error occurs, a dialog box is displayed which
	// shows the error.
	BOOL UnlockSurface(WriteToSurface surf,HRESULT & retunValue);
	
	// Erase the specifed region of the offscreen RGB surface, filling
	// it with black.  If the rectangle is NULL, erase the entire
	// surface.  The coordinates used in the rectangle are in window
	// coordinates, with the origin at the top-left of the window.
	BOOL EraseSurface(RECT const* pUpdateRect ,HRESULT & retunValue);

	// Get a handle to the device context to use for the offscreen RGB
	// surface.
	BOOL GetHDC(HDC& hDC,HRESULT & retunValue);

	// Release the handle to the device context for the offscreen RGB
	// surface.
	BOOL ReleaseHDC(HDC& hDC,HRESULT & retunValue);

	// Return the RGB pixel format
	RGBpixelType GetRGBpixelFormat() const { return rgbPixelFormat; }

	BOOL ScaleOffScreenRGB(RECT src,RECT dstScaled,HRESULT & retunValue);

	//
	// Private implementation details
	//
private:

	// The DirectDraw object
	LPDIRECTDRAW lpDD;

	// The surfaces which are created
	LPDIRECTDRAWSURFACE	surfacePrimary;	// DirectDraw primary surface
	LPDIRECTDRAWSURFACE	surfaceRGB;		// RGB back buffer surface
	LPDIRECTDRAWSURFACE	surfacePool;    // surface for effects (stretching,..) 
	// DirectDraw clipper object
	LPDIRECTDRAWCLIPPER	lpClipper;

	// The RGB pixel type of the current display
	RGBpixelType rgbPixelFormat;

	// This memory is used instead of surfaceYUVall is hardware YUV to RGB is not used
//	unsigned char* compositeYUVframe;
};


#endif // DIRECT_DRAW_H
