//This software module was originally developed by CSELT in the course of   //
//development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.             //
//This software module is an implementation of a part of one or more MPEG-4 //
//Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems        //
//(ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 Systems     //
//(ISO/IEC 14496-1) free license to this software module or modifications   //
//thereof for use in hardware or software products claiming conformance to  //
//the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use this software//
//module in hardware or software products are advised that its use may      //
//infringe existing patents.                                                //
//The original developer of this software module and his/her company, the   //
//subsequent editors and their companies, and ISO/IEC have no liability     //
//for use of this software module or modifications thereof in an            //
//implementation. Copyright is not released for non MPEG-4 Systems          //
//(ISO/IEC 14496-1) conforming products. CSELT S.pA. retains full right     //
//to use the code for his/her own purpose, assign or donate the code to a   //
//third party and to inhibit third parties from using the code for non      //
//MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright      //
//notice must be included in all copies or derivative works.                //
//Copyright  1999.                                                         //
//This work has been partly developed within the framework of the ESPRIT    //
//European Research Project  "MPEG-4 PC".                                   //
//////////////////////////////////////////////////////////////////////////////
//
// File:  DiscSensorProxy.cpp
//
// Author: Gianluca Di Cagno, CSELT, Gianluca.Dicagno@cselt.it
//
// Modified:
//
// Description:
//   The implementation of the proxy object used by the presenter
//   to handle the DiscSensor2D node.
//
// Created: February, 1999.
//
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"

#include "Nodes-CSELT.h"
#include "DiscSensorProxy.h"
#include <math.h>
#include "trace.h"

#define PI		3.14159265358979323846		/* pi */


// Storage for this static data member
BOOL DiscSensorProxy::m_movementGrabbed = FALSE;


// =====================================================================

//
// Presenter::InitDiscSensor
//
// Initialise the proxy data structure and attach it to the render data
// pointer of the given media object node.
//

DiscSensorProxy* Presenter::InitDiscSensor(DiscSensor* pObj)
{
	return new DiscSensorProxy(pObj);  
}




// =====================================================================

//
// DiscSensorProxy::OnUserEvent
//
// This function is called when a user event has occurred above a node
// which is in the same grouping node hierarchy as this sensor node.
//
// If this node is registered with the presenter to received events,
// it will be called a second time for this scene draw.
//
// If this plane sensor is enabled, then generate the required events
// and return TRUE, otherwise return FALSE.
//

BOOL DiscSensorProxy::OnUserEvent(EventInfo const* pEvent)
{
	TRACE_PRINT("In DiscSensorProxy::OnUserEvent():\n");

	// Only generate events if this node is enabled, and no other
	// plane sensor node has grabbed all motion events
	if (((BOOL)(Bool)GetDiscSensor()->enabled) &&
		((!IsMovementGrabbed()) ||
		((IsMovementGrabbed()) && (m_mouseDown))))
	{
		
	   
	  // this function is called when the mouse is not on a object
	  
	  if ((pEvent->pass == 2) && (m_prevPass == 2))
		{
		 if (pEvent->m_event == EVENT_MOUSE_UP)
	      {
			((Presenter*)GetPresenter())->UnregisterEventHandler(this);
		    m_registered = FALSE;
			if(((Presenter*)GetPresenter())->GetDrawableTouched()!=NULL)
			 {
				((Presenter*)GetPresenter())->SetIsTracking(FALSE);
	            ((Presenter*)GetPresenter())->GetDrawableTouched()->m_isBeingTracked=FALSE;
			 }
			SetGrabbedOff();
			
		    m_mouseDown = FALSE;
		  }

		 if (!m_mouseDown)
		 {
				((Presenter*)GetPresenter())->UnregisterEventHandler(this);
				m_registered = FALSE;
		 }
		 return TRUE;
	  }
   	// when the mouse button is first pressed change
	// the state of the sensor

	  if ((pEvent->m_event == EVENT_MOUSE_DOWN) && (!m_mouseDown))
	   {
		
	    	SetGrabbedOn();
			m_mouseDown = TRUE;
			Vec2f point(pEvent->m_x, pEvent->m_y);
			m_angleStartDrag=PointToAngle(point);
		}

	// set m_mouse down FALSE  if the button returns up

	  if (pEvent->m_event == EVENT_MOUSE_UP)
	    {
			((Presenter*)GetPresenter())->UnregisterEventHandler(this);
		    m_registered = FALSE;
			SetGrabbedOff();
		    
    		// this is needed because this function is called even if
			// you click on a background where there isn't an object.

			if(((Presenter*)GetPresenter())->GetDrawableTouched()!=NULL)
			 {
			
			//the tracking has finished signal this to the presenter

		    	((Presenter*)GetPresenter())->SetIsTracking(FALSE);
			
            // the drawing order of the object touched must return to normal

			    ((Presenter*)GetPresenter())->GetDrawableTouched()->m_isBeingTracked=FALSE;
			 }
			
			m_mouseDown = FALSE;
			
			//update offset field if autoOffset field is enabled

			if((Bool)(GetDiscSensor()->autoOffset))
			{
				Vec2f point(pEvent->m_x, pEvent->m_y);
				double angle = PointToAngle(point);
				GetDiscSensor()->offset=angle-m_angleStartDrag+GetDiscSensor()->offset;
			}
            return TRUE;
		}

	// Generate a trackPoint_changed event and translation changed
	// if mouse moves after the mouse has been clicked. 
        
 	  if ((pEvent->m_event == EVENT_MOUSE_MOVE) && (m_mouseDown))
		{
				
			SetGrabbedOn();
		
			// then we signal to the presenter not look 
			// which object the user has touched (because we know)

			((Presenter*)GetPresenter())->SetIsTracking(TRUE);

			// Now we can produce the event trackpoint changed (raw output)
			Vec2f point(pEvent->m_x, pEvent->m_y);
	   		GetDiscSensor()->trackPoint_changed = point;
			
			/*double pointAngleCoeff=point.y/point.x;
			
			double centerAngleCoeff=((Vec2f)(GetDiscSensor()->center)).x/((Vec2f)(GetDiscSensor()->center)).y;

			double AngleCoeffMultiple =pointAngleCoeff*centerAngleCoeff+1;
			double AngleCoeffDifference = pointAngleCoeff-centerAngleCoeff;

			double Result=AngleCoeffDifference/AngleCoeffMultiple;
			
			double angle = atan( Result );
			*/
			
			double angle = PointToAngle(point);

			// calculate translation_changed event (relative output) from:
			// m_pointStartDrag (start dragging point in screen coordinates)
			// point (current tracking  point in screen coordinates)
			// DiscSensor offset 

	
			// check for bounds for the output 	
			if(GetDiscSensor()->minAngle<GetDiscSensor()->maxAngle)
			{
				if(angle<GetDiscSensor()->minAngle)
					angle=GetDiscSensor()->minAngle;
			}
			
			
			
			// finally produce the event rotation_changed 

			GetDiscSensor()->rotation_changed = angle-m_angleStartDrag+GetDiscSensor()->offset;
			
		}

	// If not already done, register this node with the presenter
	  ((Presenter*)GetPresenter())->RegisterEventHandler(this);
	  m_registered = TRUE;
	  m_prevPass = pEvent->pass;
	  return TRUE;
     }

	// Otherwise, if not enabled, return FALSE
	else
		return FALSE;
} /* DiscSensorProxy::OnUserEvent */


// =====================================================================

//
// DiscSensorProxy::IsFinished
//
// A DiscSensor2D node is considered as finished when it is not enabled.
//

BOOL DiscSensorProxy::IsFinished() const
{
	return !((BOOL)(Bool)GetDiscSensor()->enabled);
} /* DiscSensorProxy::IsFinished */


// ========================================================================
//
//	DiscSensorProxy::PointToAngle
//
//  calculate rotation angle from the point resulting from the dragging 
//  it uses this formula:
//  old one angle =arctg((y2/x2-y1/x1)/(1+y2/x2*y1/x1))
//  new one  4*pi*(x/screenWidth)-0.5 From Jean Claude Dufourd

double DiscSensorProxy::PointToAngle(Vec2f &point)
{

	int screenHeight=GetPresenter()->GetVisualRenderer()->GetScreenHeight();
	int screenWidth=GetPresenter()->GetVisualRenderer()->GetScreenWidth();
	double angleX,angleY,angle;
	angleX=angleY=angle=0;
	if(((point.y>=0)&&(point.x>=0))||((point.y>=0)&&(point.x<=0)))
	{
		angleX= -4*PI*((point.x/screenWidth)-0.5);
		angleY=	+4*3.14*((point.y/screenHeight)-0.5);
	}
	else if((point.y<=0)&&(point.x>=0))

	{
		angleX= +4*PI*((point.x/screenWidth)-0.5);
		angleY=	+4*PI*((point.y/screenHeight)-0.5);
	}
	
	else if((point.y<=0)&&(point.x<=0))
	{
	
		angleX= +4*PI*((point.x/screenWidth)-0.5);
		angleY=	-4*PI*((point.y/screenHeight)-0.5);
	}
	
		angle =angleX+angleY;
	return angle;
	
}