//This software module was originally developed by CSELT in the course of   //
//development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.             //
//This software module is an implementation of a part of one or more MPEG-4 //
//Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems        //
//(ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 Systems     //
//(ISO/IEC 14496-1) free license to this software module or modifications   //
//thereof for use in hardware or software products claiming conformance to  //
//the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use this software//
//module in hardware or software products are advised that its use may      //
//infringe existing patents.                                                //
//The original developer of this software module and his/her company, the   //
//subsequent editors and their companies, and ISO/IEC have no liability     //
//for use of this software module or modifications thereof in an            //
//implementation. Copyright is not released for non MPEG-4 Systems          //
//(ISO/IEC 14496-1) conforming products. CSELT S.pA. retains full right     //
//to use the code for his/her own purpose, assign or donate the code to a   //
//third party and to inhibit third parties from using the code for non      //
//MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright      //
//notice must be included in all copies or derivative works.                //
//Copyright  1999.                                                         //
//This work has been partly developed within the framework of the ESPRIT    //
//European Research Project  "MPEG-4 PC".                                   //
//////////////////////////////////////////////////////////////////////////////
//
// File:  DrawableContext.h
//
// Author: Gianluca Di Cagno  gianluca.dicagno@cselt.it
//
// Modified:
//
// Description:
//   The definition of a class which stores Drawable node information
//   gathered during the scene traversal
//
//   
// Created: November 1998
//
//
///////////////////////////////////////////////////////////////////////////


#ifndef DRAWCTX_H
#define DRAWCTX_H

#include "BRect.h"
#include "TrMatrix2D.h"

class ShapeProxy;
class Appearance;

class DrawableContext{


	public:
	  	
		
		BOOL m_isTransparent;
		BOOL m_updateRequired;
		BOOL m_hasStreamFinished;
		BOOL m_isSwitchedOn;
	
		BOOL m_needsDrawing;
	    
		BRect *GetBoundingRect(	){return  &m_boundingRect;}
		// Returns the bounding rectangle which completely encloses this
		// object.  The bounding rectange uses the BIFS coordinate system,
		// with the origin of the output display is in the middle of the
		// window, with increasing X and Y to the right and up.
		BRect *GetClippingRect(	){return  &m_clippingRect;}
		// Returns the clipping rect of the drawable, i.e. the layer or form
		// bounding box.
		BRect *GetClippedBoundingRect(){ return &m_clippedBoundingRect; }
		// Intersection of clipping and bounding rect, real object rect to draw 
		float GetCreateOrder()	{return 1; }
		
		class UserEventHandlerProxy *GetUserEventHandler(){return m_pHandler;}
	
		const BOOL  NeedsDrawing(); 
		
		void UpdateBoundingRect();
		
		void BuildRefreshRect(BRect * pToBeRefreshed);
		
		void ResetVisitsList(); 
		
		BOOL IsPointOver(int x, int y) { return GetBoundingRect()->IsPointIn(x, y); }
		
		// Return TRUE if the given point, in BIFS coordinates, is above the
		// current shape.  Return FALSE otherwise.  The default implementation
		// uses the bounding rectangle of the drawable.
		BOOL IsSwitchedOn(){return m_isSwitchedOn;}
		BOOL IsTransparent(){return m_isTransparent;}
		void * GetDrawableProxy(){return m_pDrawable;}

		Appearance* GetAppearance() { return m_pAppearance;}
		
		void SetAppearance(Appearance *pAppearance) {m_pAppearance=pAppearance;}
		
		ShapeProxy* GetShapeProxy() const { return m_pShapeProxy; }
		
		void SetShapeProxy(ShapeProxy * pShapeProxy) {m_pShapeProxy=pShapeProxy;}
		
		void SetEventHandler(UserEventHandlerProxy * pHandler) {m_pHandler=pHandler;}
		
		void SetClippingRect(BRect *clipRect) {m_clippingRect=*clipRect;}

		TrMatrix2D m_transform;

		BOOL m_isBeingTracked;
		void * m_pDrawable;
		
		//Useful for Layout Postprocessing
		void SetIsTextNode(BOOL value)	{m_isTextNode=value;}
		
		BOOL IsTextNode() {return  m_isTextNode;}

	private :
		BRect		  m_boundingRect;
		BRect         m_clippingRect;
		BRect         m_clippedBoundingRect;

		//the pointer the the drawable event handler

		UserEventHandlerProxy* m_pHandler;
		
		//pointers to the drawable Appearance and Shape nodes
		Appearance*  m_pAppearance;
		ShapeProxy* m_pShapeProxy;

		
		BOOL m_isTextNode;
};	

#endif