//This software module was originally developed by CSELT in the course of   //
//development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.             //
//This software module is an implementation of a part of one or more MPEG-4 //
//Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems        //
//(ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 Systems     //
//(ISO/IEC 14496-1) free license to this software module or modifications   //
//thereof for use in hardware or software products claiming conformance to  //
//the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use this software//
//module in hardware or software products are advised that its use may      //
//infringe existing patents.                                                //
//The original developer of this software module and his/her company, the   //
//subsequent editors and their companies, and ISO/IEC have no liability     //
//for use of this software module or modifications thereof in an            //
//implementation. Copyright is not released for non MPEG-4 Systems          //
//(ISO/IEC 14496-1) conforming products. CSELT S.pA. retains full right     //
//to use the code for his/her own purpose, assign or donate the code to a   //
//third party and to inhibit third parties from using the code for non      //
//MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright      //
//notice must be included in all copies or derivative works.                //
//Copyright  1999.                                                         //
//This work has been partly developed within the framework of the ESPRIT    //
//European Research Project  "MPEG-4 PC".                                   //
//////////////////////////////////////////////////////////////////////////////
//
// File:  DrawableImage.cpp
//
// Author: Matthew Leditschke
//
// Modified: Gianluca Di Cagno  gianluca.dicagno@cselt.it
//
// Description:
//   The implementation of a class which implements functionality common
//   to nodes which need to display images, such as the Image2DRenderer,
//   Movie2DRenderer and Background2DRenderer nodes.
//
// Created: September, 1997.
//
//
///////////////////////////////////////////////////////////////////////////
/*-------------------------------------------------------------------------
MPEGJ
AUTHORS:    Viswanathan Swaminathan - vishy@eng.sun.com
			Robert Schmidt(RLS)
			Praveen Alavilli(PA)
			YUNTAI KYONG(YT) - yuntaikyong@ieee.org
-------------------------------------------------------------------------*/


#include "defs.h"
#include <assert.h>
#include "Stream.h"
#include "Clock.h"
#include "DrawableImage.h"
#include "VisRend.h"
#include "trace.h"
#include "resource.h"
#include "ThreadInfoDlg.h"

enum {VIDEO_DELAY = 0};

// =====================================================================

//
// DrawableImage::DrawableImage
//
// Class constructor.
//

DrawableImage::DrawableImage(MediaObject* pMO) : DrawableNode(pMO)
{


	m_isFinished = FALSE;
	m_startUpDelay = 0;
	m_statusInDraw = 0;
	m_bIsALeftTopImage =TRUE;



	m_dataPtr=NULL;
	m_dataSize=0;
	m_status=0;

	m_scenesDrawn=1; //init to 1 to handle first fetch
	m_frameWasFetched=FALSE;

	#ifdef MPGJ
	// MPEGJ - MERGE
	/**** load the function table to throw events - PA*/
	di = MPJGetTerminalDispatchTable();
	/**** -end MPEGJ- */
	#endif




} /* DrawableImage::DrawableImage */


// =====================================================================

//
// DrawableImage::~DrawableImage
//
// Class destructor.
//

DrawableImage::~DrawableImage()
{
//	GetStream()->Terminate();
//	GetStreamConsumer()->CloseStream();
} /* DrawableImage::~DrawableImage */


// =====================================================================

//
// DrawableImage::OnEndOfStream
//
// This method is called when the end of a stream is detected in either
// the PreRender() method or the Draw() method.
//

void DrawableImage::OnEndOfStream()
{
	GetStreamConsumer()->CloseStream();
	m_isFinished = TRUE;
	m_frameWasFetched=FALSE;
} /* DrawableImage::OnEndOfStream */




// =====================================================================



//
// DrawableImage::Draw
//
// Draw the part of the contents of the Texure node which lies within
// the given bounding rectangle.  This function will need to perform
// the drawing, even if m_updateRequire is FALSE.
//
// The dimensions of the image will already have been read, and are
// given by the dimensions of the bounding rectangle.
//

void DrawableImage::Draw(BRect const* pClipBRect,DrawableContext * pContext) {
    DrawInImage(pClipBRect,pContext, NULL);
}

void DrawableImage::DrawInImage(BRect const* pClipBRect,DrawableContext * pContext, SrcRGBAImage * pImage)
{


	assert(pClipBRect != NULL);

	// Only proceed once a stream has been attached
	if (GetStream() == NULL)
		return;

	#ifdef MPGJ
	// ************************* MPEGJ *******************************
	// Do no displaying if the stream being handled is an MPEGJ stream.
	if(GetStreamConsumer()->GetStreamFormat() == MediaFormat :: MPEGJ )
		return;
	// ************************* MPEGJ *******************************
	#endif


	//Get the portion of the Image that needs drawing
	BRect *pImageBoundingRect=pContext->GetBoundingRect();


	// Modify the image size to only show the part which lies within
	// the given bounding rectangle.
	BRect visibleRect = *pContext->GetClippedBoundingRect();
	visibleRect.Intersection(pClipBRect);

	// Only proceed if there is actually something to display
	if (visibleRect.IsEmpty())
		return;

	// If the PreRender method succeed then the frame to draw
	// has already been fetched. If not we have to fetch a frame
	int dataSize;
	DWORD rdwTime;
	BOOL bEOU;
	LPVOID dataPtr;

	if(!m_frameWasFetched)
	{
		dataPtr = GetStreamConsumer()->Fetch(dataSize, rdwTime, bEOU, m_statusInDraw,
										TRUE, FALSE, VIDEO_DELAY, TRUE);


		TRACE_PRINT1("Drawable fetch in draw: %d\n",rdwTime);
		// If there is no data, return
		if (dataPtr == NULL||dataPtr==END_OF_STREAM)
		{
			TRACE_PRINT("DrawableImage::Draw(): NULL data pointer returned.\n");
			return;
		}

		// If data block returned has 0 size, return
		else if (dataSize == 0)
		{
			TRACE_PRINT("DrawableImage::Draw(): 0 data size returned.\n");
			GetStream()->Release();
			return;
		}
	}

	else
	{
		dataPtr= m_dataPtr;
		dataSize=m_dataSize;
	}
	Decoder * pIstDec = GetStreamConsumer()->GetDecoder();

	int left,top,height,width,pitch;

	DecoderImage const* pImgDetails;

	if(pIstDec->GetDataOffset(NULL)==0)
	{
		 left = (int)pIstDec->GetAttributes(OFFSETX,(LPBYTE)dataPtr);//to delete if the *1 control is to be trashed
		 top = -(int) pIstDec->GetAttributes(OFFSETY,(LPBYTE)dataPtr);
		 width  = pIstDec->GetAttributes(FRAME_WIDTH,(LPBYTE)dataPtr); //new
		 height = pIstDec->GetAttributes(FRAME_HEIGHT,(LPBYTE)dataPtr); //new
		 pitch =pIstDec->GetAttributes(PITCH,(LPBYTE)dataPtr);

	}

	else
	{
		 pImgDetails = (DecoderImage*)dataPtr;
		 left = pImgDetails->offsetX;
		 top  = -pImgDetails->offsetY;
		 width  = pImgDetails->width; //new
		 height = pImgDetails->height; //new
		 pitch = pImgDetails->pitch;
	}

	//calculate the scale factors

	float scaleX= (float)(pImageBoundingRect->width) / (float)width;
	float scaleY= (float)(pImageBoundingRect->height) / (float)height;

	// Display the image, base on its format.  The image parameters
	// are set to point to only the visible portion of the image
	if ((GetStreamConsumer()->GetStreamFormat().m_code == MediaFormat :: YUV) ||
		(GetStreamConsumer()->GetStreamFormat().m_code == MediaFormat :: YUVA))
	{
		SrcYUVAImage image;
		image.width	 = min(visibleRect.width/scaleX,   width);
		image.height = min(visibleRect.height/scaleY, height);
		TRACE_PRINT2("imax :width%d,height %d\n",image.width,image.height);
		image.pitch	 = pitch;
		image.left	 = visibleRect.left;
		image.top	 = visibleRect.top;


/* can't understand this, the code looks ok  without this...

		// Ensure that the dimensions are even,
		image.left   = 2 * (int)(image.left   / 2);
		image.top    = 2 * (int)(image.top    / 2);
		image.width  = 2 * (int)(image.width  / 2);
		image.height = 2 * (int)(image.height / 2);
*/

		image.scaleX= scaleX;
		image.scaleY= scaleY;

/**
		assert(tmpfile != NULL);
		fwrite((unsigned char*)dataPtr + sizeof(DecoderImage),
			   pImgDetails->width * pImgDetails->height * 3 / 2, 1, tmpfile);
		fclose(tmpfile);
**/


		// Get a pointer to the top-left pixel in the Y band
		int bandYsize =  width*height;
/**
		int offset = 0;
		if (m_boundingRect.top > visibleRect.top)
			offset = (m_boundingRect.top - visibleRect.top) * image.pitch;
		if (visibleRect.left > m_boundingRect.left)
			offset += visibleRect.left - m_boundingRect.left;
**/
		int offset	 = (pImageBoundingRect->top - visibleRect.top) * image.pitch*scaleX
						+ (visibleRect.left - pImageBoundingRect->left);

		//if the image was scaled we must calculate the offset of the non scaled image from
		// the offset of the scaled one.
		if(offset!=0&&(scaleX!=1||scaleY!=1))
		{
			int xOri,yOri;

			yOri=((offset)/pImageBoundingRect->width)/scaleY;
			xOri=((offset)%pImageBoundingRect->width)/scaleX;
			offset=(width*yOri+xOri);

		}



		dataPtr		 = (unsigned char*)dataPtr + pIstDec->GetDataOffset(NULL);
		image.pY	 = (unsigned char*)dataPtr + offset;

		// Get a pointer to the top-left pixel in the A band
		if (GetStreamConsumer()->GetStreamFormat().m_code == MediaFormat :: YUV)
			image.pA = NULL;
		else
			image.pA = (unsigned char*)dataPtr + bandYsize * 3 / 2 +  offset;

		// Get a pointer to the top-left pixel in the U and V band
/**
		offset = 0;
		if (m_boundingRect.top > visibleRect.top)
			offset = (int)((m_boundingRect.top - visibleRect.top) / 2) * image.pitch;
		if (visibleRect.left > m_boundingRect.left)
			offset += (visibleRect.left - m_boundingRect.left) / 2;
**/
		offset		 = (int)((pImageBoundingRect->top - visibleRect.top) / 4) * image.pitch*scaleX
						+ (visibleRect.left - pImageBoundingRect->left) / 2;

		//if the image was scaled we must calculate the offset of the non scaled image from
		// the offset of the scaled one.
		if(offset!=0&&(scaleX!=1||scaleY!=1))
		{
			int xOri,yOri;

			yOri=((offset)/pImageBoundingRect->width)/scaleY;
			xOri=((offset)%pImageBoundingRect->width)/scaleX;
			offset=(width*yOri+xOri);

		}

		image.pU	 = (unsigned char*)dataPtr + bandYsize  +  offset;
		image.pV	 = (unsigned char*)dataPtr + bandYsize * 5 / 4 +  offset;

		GetVisualRenderer()->DisplayYUV420AImage(image);


	}

	else if ((GetStreamConsumer()->GetStreamFormat().m_code == MediaFormat :: RGB) ||
			 (GetStreamConsumer()->GetStreamFormat().m_code == MediaFormat :: BGR))
	{
		SrcRGBAImage image;
		image.width	 = min(visibleRect.width/scaleX,   width);
		image.height = min(visibleRect.height/scaleY, height);
		image.pitch	 = pIstDec->GetAttributes(PITCH,(LPBYTE)dataPtr);
		image.left	 = visibleRect.left;
		image.top	 = visibleRect.top;

		image.scaleX= scaleX;
		image.scaleY= scaleY;

		// Get a pointer to the top-left pixel
		int offset	 = ((pImageBoundingRect->top - visibleRect.top)) * image.pitch*scaleX
						+ ((visibleRect.left - pImageBoundingRect->left)) * 3;

		//if the image was scaled we must calculate the offset of the non scaled image from
		// the offset of the scaled one.
		if(offset!=0&&(scaleX!=1||scaleY!=1))
		{
			int xOri,yOri;

			yOri=((offset/3)/pImageBoundingRect->width)/scaleY;
			xOri=((offset/3)%pImageBoundingRect->width)/scaleX;
			offset=(width*yOri+xOri)*3;

		}
		if (GetStreamConsumer()->GetStreamFormat().m_code == MediaFormat :: RGB)
		{
			image.pR	 = (unsigned char*)dataPtr+pIstDec->GetDataOffset(NULL)+offset; //+ sizeof(DecoderImage) + offset;
			image.pG	 = image.pR + 1;
			image.pB	 = image.pR + 2;
		}
		else
		{
			image.pB	 = (unsigned char*)dataPtr+pIstDec->GetDataOffset(NULL)+offset; //+ sizeof(DecoderImage) + offset;
			image.pG	 = image.pB + 1;
			image.pR	 = image.pB + 2;
		}

		image.pA	 = NULL;
		image.ptrInc = 3;

		if (!pImage) {
			GetVisualRenderer()->DisplayRGBAImage(image);
		} else {
			pImage->top = image.top;
            pImage->left = image.left;
			pImage->width = image.width;
			pImage->height = image.height;
			pImage->pitch = image.pitch;
			pImage->scaleX = image.scaleX;
			pImage->scaleY = image.scaleY;
			pImage->pR = image.pR;
			pImage->pG = image.pG;
			pImage->pB = image.pB;
			pImage->pA = image.pA;
			pImage->ptrInc = image.ptrInc;
		}
	}

	else
	{
		assert(0);		// Shouldn't get here!
	}



/*#ifdef WRITE_TRACE
	if (dataSize == 176 * 144 * 3 + 1 + pIstDec->GetDataOffset(NULL) )
		fprintf(traceFile, "H263: ");
	else if (dataSize == 176 * 144 * 3 / 2+pIstDec->GetDataOffset(NULL))
		fprintf(traceFile, "H263: ");
	else
		fprintf(traceFile, "JPEG: ");

	DWORD currentTime =  GetStreamConsumer()->GetExecutive()->GetPresenter()->GetTime();

	TRACE_PRINT4("Displaying for time %ld at %ld  CL %ld (%ld)\n", rdwTime,
				 currentTime, GetStream()->GetClock(),GetStream()->GetClock()-rdwTime);
#endif*/

	// Release this unit from the stream, now that it is finished with,
	// but leave the data there in case nothing new has arrived when this
	// function is next called.
	if(!m_frameWasFetched)
		GetStream()->Release(FALSE);

	// Also handle the end of the stream
	if (m_statusInDraw == -1)
	{
//		OnEndOfStream();
		TRACE_PRINT("DrawableImage::Draw(): end of stream detected.\n");
	}


} /* DrawableImage::Draw */



#ifdef MPGJ
/*MPEGJ - Merge - PA */
static int isEvent = 0;
// END - MPEGJ
#endif // MPEGJ

// =====================================================================
// DrawableImage::NeedsDrawing

BOOL DrawableImage::NeedsDrawing(DrawableContext * pContext)
{
		return pContext->m_updateRequired;
} /*DrawableImage::NeedsDrawing*/

// =====================================================================
//
// DrawableImage::PreRender
//
// Call the DrawableNode class's method to remember the effects and to
// register with the visual renderer.  Open the stream to see if the
// contents has changed, and hence needs to be redrawn.
//
// This function does not call UpdateBoundingRect(), as the derived
// class may modify the bounding rectangle.  The derived class MUST
// call UpdateBoundingRect().
//


void DrawableImage::PreRender(Effects & effects,DrawableContext *pContext)
{
	// Only proceed once a stream has been attached
	if (GetStream() == NULL)
		return;

	#ifdef MPGJ
	// ************************* MPEGJ *******************************
	// Do no displaying if the stream being handled is an MPEGJ stream.
	if(GetStreamConsumer()->GetStreamFormat() == MediaFormat :: MPEGJ )
		return;
	// ************************* MPEGJ *******************************
	#endif


	// Decide if the stream contains a new frame to display

	DWORD rdwTime;
	BOOL bEOU;
	MediaFormat frameFormat=GetStreamConsumer()->GetStreamFormat().m_code;

	// try to fetch a frame only if this is first time
	// this node is traversed in a single PreRendering
	// step

	if(m_scenesDrawn!=GetPresenter()->GetScenesDrawn())
	{


	//release previously fetched frame if this is first visit

		if(m_frameWasFetched)
		{
			GetStream()->Release(FALSE);
			m_frameWasFetched=FALSE;
		}


	//fetch new frame

		m_dataPtr = GetStreamConsumer()->Fetch(m_dataSize, rdwTime, bEOU, m_status, TRUE,
										FALSE, VIDEO_DELAY,	TRUE);

		TRACE_PRINT1("Drawable fetched a frame in PreRender: %d\n",rdwTime);

		m_scenesDrawn=GetPresenter()->GetScenesDrawn();
	}


	//now m_dataPtr,m_dataSize,m_status contain new frame information

	if(m_status == -1)
		  pContext->m_hasStreamFinished=TRUE;

	// If there is no data, no updating is required
	if ((m_dataPtr == NULL)||(m_dataPtr == END_OF_STREAM) )
	{
		TRACE_PRINT("DrawableImage::PreRender(): NULL data pointer returned.\n");

		pContext->m_updateRequired = FALSE;
		if( m_dataPtr != END_OF_STREAM ) {

			#ifdef MPGJ

			isEvent++;
			// MPEGJ - MERGE
			if ( isEvent == 2 ) {
				/* if dataptr is null continuously for some time, then
				 we don't need to throw the same event multiple times
				 throw the event only for the first time and ignore
				 remaining - PA */

				/* throw decoder underflow event here
				/* audio renderer is ready but there is no data associated with
					this node - means the decoder is not supplying data to it */
				/***** handle to the function table - PA *****/
				MPJTerminalDispatchTablePtr di = MPJGetTerminalDispatchTable();
				if( di != NULL ) {
					// DECODER_UNDERFLOW
					// go till end of list and add the new event there
					rendererQueue* temp;
					temp = &(di->rEQ);
					if( *temp == NULL )
						*temp = new rendererEventQueue( 0 );
					else {
						while ( (*temp)->next != NULL )
							*temp = (*temp)->next;
						(*temp)->next = new rendererEventQueue( 0 );
					}
					if( di->isStartedDispatching == 1) {
					// signal the thread to throw the event
						int count = 0;
						count = ResumeThread(di->hThrow);
					}
				}
			}
			// END - MPEGJ - PA
			#endif // MPEGJ
		}

	}


	// If data block returned has 0 size, no updating is required
	else if (m_dataSize == 0)
	{
		TRACE_PRINT("DrawableImage::PreRender(): 0 data size returned.\n");
		GetStream()->Release();
		pContext->m_updateRequired = FALSE;
	}


	// Otherwise we got a frame. It can be a new one or and old one (recurrent)
	// anyway we build the bounding box and the Drawable Context and then set
	// updateRequired to TRUE if this is a new frame.
	else
	{
		//remember we need to call Release()
		m_frameWasFetched=TRUE;

		// If the output of the decoder is YUVA, then we assume that
		// part of the image may be transparent.
		if (frameFormat == MediaFormat :: YUVA)
			pContext->m_isTransparent = TRUE;
		else
			pContext->m_isTransparent = FALSE;

		// get a pointer to the current istantiated decoder
		// so that we could know how to retrieve image size,
		// pitch,offset.

		Decoder * pIstDec =GetStreamConsumer()->GetDecoder();

		int left,top,height,width;
		DecoderImage const* pImgDetails;

		// check if this decoder has constant frame size

		if(pIstDec->GetDataOffset(NULL)==0)
		{

			m_ImageDimensions.width  = width = pIstDec->GetAttributes(FRAME_WIDTH,(LPBYTE)m_dataPtr); //new
			m_ImageDimensions.height = height = pIstDec->GetAttributes(FRAME_HEIGHT,(LPBYTE)m_dataPtr); //new
			m_ImageDimensions.left = left = (int)pIstDec->GetAttributes(OFFSETX,(LPBYTE)m_dataPtr);
			m_ImageDimensions.top = top = -(int) pIstDec->GetAttributes(OFFSETY,(LPBYTE)m_dataPtr);
		}

		else
		{
			 pImgDetails = (DecoderImage*)m_dataPtr;
			 m_ImageDimensions.left = left = pImgDetails->offsetX;
			 m_ImageDimensions.top  =  top = -pImgDetails->offsetY;
			 m_ImageDimensions.width  = width = pImgDetails->width;
			 m_ImageDimensions.height = height = pImgDetails->height;
		}
#ifdef ORIGIN_CENTER
	// let's make the bounding box of this object
	// be centered in the center of the image..
	// In Mpeg-4 the Frame Height can be different from the Vop Height.
		m_ImageDimensions.left=left-=pIstDec->GetAttributes(FRAME_WIDTH,(LPBYTE)m_dataPtr)/2;
		m_ImageDimensions.top =top+= pIstDec->GetAttributes(FRAME_HEIGHT,(LPBYTE)m_dataPtr)/2;

 #endif
		pContext->m_transform.Apply(left,top);
		//for  scaling
		pContext->m_transform.ApplyScale(width,height);



		// If the output format is YUV, we need to ensure that the left and
		// top are even
		if ((frameFormat == MediaFormat :: YUV) ||
			(frameFormat == MediaFormat :: YUVA))
		{
			left = 2 * (int)(left / 2);
			top  = 2 * (int)(top / 2);
		}

		// Set the dimensions of the bounding rectangle
		pContext->GetBoundingRect()->SetDimensions((int)left, (int)top,
									 (int) width,
									 (int) height);


		//if the frame we got is recurrent, we just build the object bounding box
		// and set updateRequired to FALSE

		(m_status != 0) ? pContext->m_updateRequired = FALSE : pContext->m_updateRequired = TRUE;

		if(effects.orderHasChanged)
			pContext->m_updateRequired = TRUE;
			// Also handle the end of the stream


	/*	if (m_status == -1 )
		{
			TRACE_PRINT("DrawableImage::PreRender(): end of stream detected.\n");
			OnEndOfStream();
		}*/
	}
} /* DrawableImage::PreRender */












