//This software module was originally developed by CSELT in the course of   //
//development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.             //
//This software module is an implementation of a part of one or more MPEG-4 //
//Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems        //
//(ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 Systems     //
//(ISO/IEC 14496-1) free license to this software module or modifications   //
//thereof for use in hardware or software products claiming conformance to  //
//the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use this software//
//module in hardware or software products are advised that its use may      //
//infringe existing patents.                                                //
//The original developer of this software module and his/her company, the   //
//subsequent editors and their companies, and ISO/IEC have no liability     //
//for use of this software module or modifications thereof in an            //
//implementation. Copyright is not released for non MPEG-4 Systems          //
//(ISO/IEC 14496-1) conforming products. CSELT S.pA. retains full right     //
//to use the code for his/her own purpose, assign or donate the code to a   //
//third party and to inhibit third parties from using the code for non      //
//MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright      //
//notice must be included in all copies or derivative works.                //
//Copyright  1999.                                                         //
//This work has been partly developed within the framework of the ESPRIT    //
//European Research Project  "MPEG-4 PC".                                   //
//////////////////////////////////////////////////////////////////////////////
//
// File:  DrawableImage.h
//
// Author: Matthew Leditschke
//
// Modified: Gianluca Di Cagno  gianluca.dicagno@cselt.it
//
// Description:
//   The definition of a class which implements functionality common to
//   nodes which need to display images, such as the Image2DRenderer,
//   Movie2DRenderer and Background2DRenderer nodes.
//
// Created: September, 1997.
//
//
///////////////////////////////////////////////////////////////////////////

/*-------------------------------------------------------------------------
MPEGJ
AUTHORS:    Viswanathan Swaminathan - vishy@eng.sun.com
			Robert Schmidt(RLS)
			Praveen Alavilli(PA)
			YUNTAI KYONG(YT) - yuntaikyong@ieee.org
-------------------------------------------------------------------------*/

#ifndef DRAWABLE_IMAGE_H
#define DRAWABLE_IMAGE_H

#include "Presenter.h"
#include "Nodes.h"
#include "DrawableNode.h"
#include "StreamCo.h"

// MPEGJ - MERGE
#ifdef MPGJ
#include "..\mpegj\MPEGJTerminalInterface.h" //for function table - PA
#endif
// END- MPEGJ


class DrawableImage : public DrawableNode
{
public:
	#ifdef MPGJ
	// MPEGJ - MERGE
	/***** handle to the function table -PA  */
	MPJTerminalDispatchTablePtr di ;
	/*** -end MPEGJ - */
	#endif

	// Class constructor & destructor
	DrawableImage(MediaObject* pMO);
	~DrawableImage();

	// Return a pointer to the StreamConsumer node that this node is
	// the proxy for.
	StreamConsumer* GetStreamConsumer() const { return (StreamConsumer*)GetObject(); }

	// Return a pointer to the stream that this node reads from.
	MediaStreamImp* GetStream() const { return GetStreamConsumer()->GetStream(); }




	virtual void PreRender(Effects & effects,DrawableContext *pContext);


	virtual BOOL NeedsDrawing(DrawableContext * pContext) ;

	virtual void Draw(BRect const* pClipBRect,DrawableContext * pDrawableContext);
    void DrawInImage(BRect const* pClipBRect,DrawableContext * pDrawableContext, SrcRGBAImage * pImage);
	BOOL IsFinished() const { return m_isFinished; }
    MediaObject * GetDrawableParent() const {return (MediaObject*)GetObject()->GetParent(0);}

    // those are used by mattetextureProxy
    LPVOID getDataPtr() const { return m_dataPtr;}
    int getStatus() const { return m_status; }
    int getDataSize() const { return m_dataSize;}
    BOOL getFrameWasFetched() const { return m_frameWasFetched; }

	// Protected implementation details
protected:


	long m_startUpDelay;
	int m_statusInDraw;

	// This method is called when the end of a stream is detected in either
	// the PreRender() method or the Draw() method.
	virtual void OnEndOfStream();


	// Private implementation details
private:
	BOOL m_isFinished;
	int m_preRenderOffsetX;
	int m_preRenderOffsetY;


	BRect m_ImageDimensions;
	// composition frame attributes returned by the fetch function
	LPVOID m_dataPtr;
	int m_status;
	int m_dataSize;
	// size of the previous frame fetched

	// TRUE if a frame has been fetched.
	BOOL   m_frameWasFetched;


	// useful to know if a node is traversed more than one time
	// in a single PreRendering step

	DWORD m_scenesDrawn;

};

#endif // DRAWABLE_IMAGE_H
