//This software module was originally developed by CSELT in the course of   //
//development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.             //
//This software module is an implementation of a part of one or more MPEG-4 //
//Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems        //
//(ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 Systems     //
//(ISO/IEC 14496-1) free license to this software module or modifications   //
//thereof for use in hardware or software products claiming conformance to  //
//the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use this software//
//module in hardware or software products are advised that its use may      //
//infringe existing patents.                                                //
//The original developer of this software module and his/her company, the   //
//subsequent editors and their companies, and ISO/IEC have no liability     //
//for use of this software module or modifications thereof in an            //
//implementation. Copyright is not released for non MPEG-4 Systems          //
//(ISO/IEC 14496-1) conforming products. CSELT S.pA. retains full right     //
//to use the code for his/her own purpose, assign or donate the code to a   //
//third party and to inhibit third parties from using the code for non      //
//MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright      //
//notice must be included in all copies or derivative works.                //
//Copyright  1999.                                                         //
//This work has been partly developed within the framework of the ESPRIT    //
//European Research Project  "MPEG-4 PC".                                   //
//////////////////////////////////////////////////////////////////////////////
//
// File:  DrawableNode.cpp
//
// Author: Matthew Leditschke 
//
// Modified: Gianluca Di Cagno  gianluca.dicagno@cselt.it
//
// Description:
//   The implementation of methods of the class which encapsulates all
//   of the features of a node which can be drawn by the visual renderer.
//   This file contains the implementation of methods common to all nodes
//   which are drawable.
//
// Created: August, 1997.
//
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include <assert.h>
#include "Nodes-CSELT.h"
#include "DrawableNode.h"
#include "VisRend.h"
#include "trace.h"



// The storage for the static data member of the DrawableNode class
int DrawableNode::m_numDrawablesCreated = 0;


// =====================================================================

//
// DrawableNode::DrawableNode
//
// Class constructor - initialise the associated variables, and record
// the order in which this node was created (used to determine the
// order to drawn nodes).
//

DrawableNode::DrawableNode(MediaObject* pMO) : Traversable(pMO)
{
	

	m_isBeingTracked=FALSE;
	m_bIsALeftTopImage=FALSE;
	

	m_orderInDrawableList=-1;

	m_currentBRList=NULL;
	m_previousBRList=NULL;
	m_nVisits=0;
	m_nCurrentBBox=0;
	m_currentBRListSize=0;
	m_nPreviousBBox=0;
} /* DrawableNode::DrawableNode */

// =====================================================================

//
// DrawableNode::~DrawableNode
//
// Class denstructor 

DrawableNode::~DrawableNode() 
{
// add the current Bounding Box list to the part of the screen
// which needs cleaning.
	for(int i=0;i<m_nCurrentBBox;i++)
		GetVisualRenderer()->m_toBeClean.Union(m_currentBRList[i]);
	
	
/*	if (m_orderInDrawableList>=0)
		GetVisualRenderer()->SetToNull(m_orderInDrawableList);*/
	for( i=0;i<m_currentBRListSize;i++)
		delete m_currentBRList[i];
	delete [] m_currentBRList;
	for(i=0;i<m_currentBRListSize;i++)
		delete m_previousBRList[i];
	delete [] m_previousBRList;
} /* DrawableNode::DrawableNode */

// =====================================================================

//
// DrawableNode::InitPreRender
//
// The method will store a copy of all of the effects parameters
// in a DrawableContext taken from the Visual Renderer,registering
// this node with the visual renderer so that it is drawn.
// This function returns the pointer of the DrawableContext, NULL
// if this node has been switched off,in which case the calling 
// function must return immediately.
//

DrawableContext * DrawableNode::InitPreRender(Effects const& effects)
{
	
	//If this is the first time this node is being visited during
	//a single prerendering step, save the previous BRect list 
	if(m_nVisits++==0)
	{
		for(int z=0;z<m_nCurrentBBox;z++)
			*m_previousBRList[z]=*m_currentBRList[z];
		m_nPreviousBBox=m_nCurrentBBox;
		m_nCurrentBBox=0;
	}
	
	//Get a empty context from the Visual Renderer.
	DrawableContext *pContext= GetVisualRenderer()->GetDrawableContext();
	
	//Save effects

	pContext->m_transform = effects.transform;


	

    
	pContext->m_isSwitchedOn = effects.isSwitchedOn;
    
	pContext->m_pDrawable=this;
    
		
	// save the pointer of the appearance and shape objects
	pContext->SetAppearance(effects.m_pAppearance);
	pContext->SetShapeProxy(effects.m_pShapeProxy);
	
	// save the pointer of the sensor which affects 
	// the drawable
	pContext->SetEventHandler(effects.m_pHandler);
	
	
	pContext->GetBoundingRect()->MakeEmpty();
	
	// If this node is switched off, then set its bounding rectangle
	// to empty, and update the overall changed bounding rectangle.
	// NULL  is returned.
	if (!effects.isSwitchedOn)
		UpdateBoundingRect(pContext);
	   
	BRect nullBRect(0,0,0,0);
	pContext ->SetClippingRect(&nullBRect);
	
	return pContext;
} /* DrawableNode::InitPreRender */


// =====================================================================

//
// DrawableNode::UpdateBoundingRect
//
// Call this function from the PreRender() method of a derived
// class, once the m_boundingRect data member has been set.
//

void DrawableNode::UpdateBoundingRect(DrawableContext * pContext)
{
	
	if(pContext!=NULL)
	{
		if (m_nCurrentBBox >= m_currentBRListSize)
		{	
			m_currentBRListSize += 10;
			
			m_currentBRList = (BRect * *) realloc(m_currentBRList,
			m_currentBRListSize * sizeof(BRect *));
			m_previousBRList = (BRect * *) realloc(m_previousBRList,
			m_currentBRListSize * sizeof(BRect *));

			for(int i=m_nCurrentBBox;i<m_currentBRListSize;i++)
				m_currentBRList[i]=new BRect;
			for( i=m_nCurrentBBox;i<m_currentBRListSize;i++)
				m_previousBRList[i]=new BRect;
		}

//		BRect *pClippingBRect=pContext->GetClippingRect();
//		if(pClippingBRect->IsEmpty())
			*m_currentBRList[m_nCurrentBBox++]=*(pContext->GetBoundingRect());
/*		else
		{
			BRect clippedBRect=*(pContext->GetBoundingRect());
		    clippedBRect.Intersection(pClippingBRect);
			*m_currentBRList[m_nCurrentBBox++]=clippedBRect;

		}*/

	}

} /* DrawableNode::UpdateBoundingRect */

// =====================================================================

// DrawableNode::LookInPreviousBBoxList
//
// Call this function to look in the previous Bounding Box list of a drawable
// node if the argument BBox is found. If so, remove that BB from the previous
// list.
// All the BBox that will not be removed, will be cleaned during the rendering 
// process
//

BOOL DrawableNode:: LookInPreviousBBoxList(BRect *pBRect)
{
 for(int i=0;i<m_nPreviousBBox;i++)
	if(!m_previousBRList[i]->IsEmpty())
	{
	/*	TRACE_PRINT1("Previous Box %d: ", i);
		m_previousBRList[i]->Print(traceFile);
		TRACE_PRINT("\n");
		TRACE_PRINT1("Current Box %d: ", i);
		pBRect->Print(traceFile);
		TRACE_PRINT("\n");*/
		if(*m_previousBRList[i]==*pBRect)
		{
			m_previousBRList[i]->MakeEmpty(); // for now just empty the rect			
			return TRUE;
		}
	}
 return FALSE;
} /*DrawableNode::LookInPreviousBBoxList*/

// =====================================================================

// DrawableNode::GetPreviousBRectToRefresh
// 
// Call this function to get the rect that needs to be refreshed, if any.
// This is called to delete the previous rects of a node when a node changes
// position.
//
void  DrawableNode::GetPreviousBRectToRefresh(BRect *pRectToBeRefreshed)
{
	
	for(int i=0;i<m_nPreviousBBox;i++)
		if(!m_previousBRList[i]->IsEmpty())
		{
/*          TRACE_PRINT1("To Refresh %d: ", i);
			m_previousBRList[i]->Print(traceFile);
			TRACE_PRINT("\n");*/
			pRectToBeRefreshed->Union(m_previousBRList[i]);
		}
	
} /*DrawableNode::GetPreviousBRectToRefresh*/


void DrawableNode::ResetCurrentBBoxList()
{
	for(int i=0;i<m_nCurrentBBox;i++)
		m_currentBRList[i]->MakeEmpty();
		

}
