//This software module was originally developed by CSELT in the course of   //
//development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.             //
//This software module is an implementation of a part of one or more MPEG-4 //
//Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems        //
//(ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 Systems     //
//(ISO/IEC 14496-1) free license to this software module or modifications   //
//thereof for use in hardware or software products claiming conformance to  //
//the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use this software//
//module in hardware or software products are advised that its use may      //
//infringe existing patents.                                                //
//The original developer of this software module and his/her company, the   //
//subsequent editors and their companies, and ISO/IEC have no liability     //
//for use of this software module or modifications thereof in an            //
//implementation. Copyright is not released for non MPEG-4 Systems          //
//(ISO/IEC 14496-1) conforming products. CSELT S.pA. retains full right     //
//to use the code for his/her own purpose, assign or donate the code to a   //
//third party and to inhibit third parties from using the code for non      //
//MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright      //
//notice must be included in all copies or derivative works.                //
//Copyright  1999.                                                         //
//This work has been partly developed within the framework of the ESPRIT    //
//European Research Project  "MPEG-4 PC".                                   //
//////////////////////////////////////////////////////////////////////////////
//
// File:  DrawableText.cpp
//
// Authors: Matthew Leditschke 
//
// Modified: Gianluca Di Cagno  gianluca.dicagno@cselt.it
//
// Description:
//   The implementation of a class which implements functionality common
//   to nodes which need to display text, namely the Text2D and
//   StreamingText nodes.
//
// Created: October, 1997.
//
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include "DrawableText.h"
#include "trace.h"


// =====================================================================

//
// DrawableText::DrawableText
//
// Class constructor - initial array of lines of text.
//

DrawableText::DrawableText(MediaObject* pMO) : GeometryNode(pMO)
{
	m_numLines = 0;
	m_lines = NULL;
	m_numLinesAllocated = -1;
	m_bIsALeftTopImage=TRUE;
	m_lengthSet=-1;

} /* DrawableText::DrawableText */


// =====================================================================

//
// DrawableText::~DrawableText
//
// Class destructor - delete array of lines of text.
//

DrawableText::~DrawableText()
{
	if (m_lines != NULL)
		free(m_lines);
} /* DrawableText::~DrawableText */


// =====================================================================

//
// DrawableText::GetFStype
//
// Return the 'type' field from the FontStyle node.  If this no is
// not present, use the default value.
//
const char* DrawableText::GetFSfamily() const
{
	if (GetFontStyle() == NULL)
		return "";
	else
		return (LPCSTR)(const ZString&)GetFontStyle()->family[0];
} /* DrawableText::GetFSfamily */


// =====================================================================

//
// DrawableText::GetFSstyle
//
// Return the 'style' field from the FontStyle node.  If this no is
// not present, use the default value.
//

const char *  DrawableText::GetFSstyle() const      //GDchange
{
	if (GetFontStyle() == NULL)
		return "";
	else
	return (LPCSTR)(const ZString&)GetFontStyle()->style;
}
 /* DrawableText::GetFSstyle */


// =====================================================================

//
// DrawableText::GetFSsize
//
// Return the 'size' field from the FontStyle node.  If this no is
// not present, use the default value.
//

float DrawableText::GetFSsize() const
{
	if (GetFontStyle() == NULL)
		return 1;
	else
		return GetFontStyle()->size;
} /* DrawableText::GetFSsize */

// =====================================================================

//
// DrawableText::GetFSjustify
//
// Return the 'justify' field from the FontStyle node.  If this no is
// not present, use the default value.
//

const char * DrawableText::GetFSjustify(int i) const
{
	if (GetFontStyle() == NULL||(i>=GetFontStyle()->justify.GetLength()))
		return "FIRST";
	else
	return (LPCSTR)(const ZString&)GetFontStyle()->justify[i];
} /* DrawableText:: GetFSjustify */


// =====================================================================

//
// DrawableText::GetFShorizontal
//
// Return the 'horizontal' field from the FontStyle node.  If this no is
// not present, use the default value.
//

BOOL DrawableText::GetFShorizontal() const
{
	if (GetFontStyle() == NULL)
		return TRUE;
	else
	return (Bool)GetFontStyle()->horizontal;
} /* DrawableText:: GetFShorizontal */



// =====================================================================

//
// DrawableText::GetFSspacing
//
// Return the 'spacing' field from the FontStyle node.  If this no is
// not present, use the default value.
//

int DrawableText::GetFSspacing() const
{
	if (GetFontStyle() == NULL)
		return 1;
	else
		return GetFontStyle()->spacing;
} /* DrawableText::GetFSspacing */




// =====================================================================

//
// DrawableText::GetFSlanguage
//
// Return the 'language' field from the FontStyle node.  If this no is
// not present, use the default value.
//

const char* DrawableText::GetFSlanguage() const
{
	if (GetFontStyle() == NULL)
		return "";
	else
		return (LPCSTR)(const ZString&)GetFontStyle()->language;
} /* DrawableText::GetFSlanguage */

// =====================================================================

//
// DrawableText::GetFSleftToRight
//
// Return the 'leftToRight' field from the FontStyle node.  If this no is
// not present, use the default value.
//


int DrawableText::GetFSleftToRight() const
{
  	if (GetFontStyle() == NULL)
		return TRUE;
	else
		return (Bool)(GetFontStyle()->leftToRight);
}  /* DrawableText::GetFSleftToRight */

// =====================================================================

//
// DrawableText::GetAscentAndDescent
//
// the font ascent and descent are returned, assume lines have same ascent
// and descent
void DrawableText::GetAscentAndDescent(int &ascent,int &descent)
{
	if (m_lines)
	{
	
		ascent=m_lines[0].ascent;
		descent=m_lines[0].descent;
	}
		


} /*DrawableText::GetAscentAndDescent*/




// =====================================================================

//
// DrawableText::GetFStopToBottom
//
// Return the 'topToBottom' field from the FontStyle2D node.  If this no is
// not present, use the default value.
//


int DrawableText::GetFStopToBottom() const
{
  	if (GetFontStyle() == NULL)
		return TRUE;
	else
		return (Bool)GetFontStyle()->topToBottom;
}  /* DrawableText::GetFStopToBottom */


// =====================================================================

//
// DrawableText::PreRender
//
// This function will be called when the dimensions of the text string,
// stored in m_data, need to be determined.  The m_boundingRect data
// member will be set to store the new bounding rectangle.  The base
// class PreRender() and UpdateBoundingRect() methods WILL NOT be
// called.
//


void DrawableText::PreRender(Effects &,DrawableContext *pContext )
{
// Ananda - maybe could it be a way but I cannot add any member
// Ananda - in this class ????? Here is problem. DAMN IT
/*
   DWORD  presTime = GetPresenter()->GetTime();

   // Check if the Appearance node has been updated
   Appearance *pAppearance = GetAppearance();
   if (pAppearance != NULL && pAppearance->GetUpdateTime() < presTime)
   {
      // Check if the Material node has been updated
      Material2D *pMaterial = (Material2D *) (MediaObject *) pAppearance->material;
      if ((pMaterial != NULL) && (pMaterial->GetUpdateTime() < presTime))
      {      
         //Check if the LineProperties node has been updated
         LineProperties* pLineProperties = pMaterial->lineProps;
         if((pLineProperties != NULL) && (pLineProperties->GetUpdateTime() < presTime))
         {
            //Check if the FontStyle node has been updated
            FontStyle* pFontStyle = GetFontStyle();
            if((pFontStyle != NULL) && (pFontStyle->GetUpdateTime() < presTime))
            {
               // Because nothing has change the bounding rectangle is the same as
               // before. Just restore it from the saved copy and return...
               m_boundingRect = m_lastBoundingRect;
               return;
            }
         }
      }
   }
*/
   pContext->SetIsTextNode(TRUE);

   // Determine the properties of the font to use
   m_fontName = GetFSfamily();
   
   if ((m_fontName == NULL)  || (m_fontName[0] == 0))
      m_fontName = "SERIF";

   m_fontSize = GetFSsize();
   if (m_fontSize == -1)
      m_fontSize = 12;
    
   const char * pStyle = GetFSstyle();
   
   m_bold = !strcmp("BOLD",pStyle);
   m_italic = !strcmp("ITALIC",pStyle);
   m_underline = !strcmp("UNDERLINED",pStyle);


   
   BRect *pTextBRect= pContext->GetBoundingRect();
 
 
  // Set the font which is to be used to calculate the size of the
  // text string when rendered
  VisualRenderer* pVR = GetVisualRenderer();
  void* pFont = pVR->SetFont(m_fontName, m_fontSize, m_bold,
                       m_italic, m_underline);

  
	

  pTextBRect->MakeEmpty();

  if(pFont==NULL)    
	  //Probably the call to the Visual Renderer failed because the surfaces are empty
	  return;
  // Determine the dimension of every line of text in the string
  
  
     
  for(int z=0;z<m_numLines;z++)
  {	
        
	  pVR->GetTextSize(pFont, (m_lines[z].pString),
           m_lines[z].numChars,
           m_lines[z].width,
           m_lines[z].height,
		   m_lines[z].ascent,
		   m_lines[z].descent);

		int length = GetLength(z);
		
		if (length !=-1)
		{
			assert(length != 0);
			m_lengthSet=length;
		}
		if (m_lengthSet != -1)
			m_lines[z].length=m_lengthSet;
		
		// Update the size of the bounding rectangle
        
		
		if (((int)m_lines[z].length ? (int)m_lines[z].length :(int)m_lines[z].width) > pTextBRect->width)
        {
           pTextBRect->width = ((int)m_lines[z].length ? (int)m_lines[z].length :(int)m_lines[z].width);
        }
        pTextBRect->height += (int)m_lines[z].height;
  }
     
  

  // Unset the font now that we have finished with it.
  pVR->UnsetFont(pFont);



   // At the moment we can only handle text which has undergone
   // only translation.
   assert(pContext->m_transform.IsOnlyTranslation());

   // Translate the text string, according to the transform parameters
   float left = pTextBRect->left;
   float top  = pTextBRect->top;


   pContext->m_transform.Apply(left, top);
   pTextBRect->left = left + 0.5;
   pTextBRect->top  = top + 0.5;

   JustifyBoundingRect(pTextBRect);

   
      
} /* DrawableText::PreRender */


// =====================================================================

//
// DrawableText::Draw
//
// Render the text string stored in the m_data field, using the following
// appearance fields:
//   diffuseColor, shadow & shadPos.
//

void DrawableText::Draw(BRect const* pClipBRect,DrawableContext * pContext)
{
	// Set the font which is to be used
	VisualRenderer* pVR = GetVisualRenderer();
	void* pFont = pVR->SetFont(m_fontName, m_fontSize, m_bold,
							   m_italic, m_underline);

	// Determine where to start drawing the text
	int x =pContext->GetBoundingRect()->left;
	int y =pContext->GetBoundingRect()->top; 

	BRect visibleRect = *pClipBRect;
	visibleRect.Intersection(pContext->GetClippedBoundingRect());

	// Display each of the lines in the string
	for (int line = 0; line < m_numLines; line++)
	{

// justify test, the windows aligment used is always TA_TOP | TA_LEFT
// then the x coordinate is adjusted in the following 				
		
		int iOldX=x;

		if(GetFShorizontal())
		{
			CString csJustify = GetFSjustify(0);
			if(csJustify=="FIRST"||csJustify=="BEGIN")
			{	
				x=x;
			}	
			else if(csJustify=="MIDDLE")
			{
				x+=(pContext->GetBoundingRect()->width-
					((int)m_lines[line].length ? (int)m_lines[line].length :(int)m_lines[line].width))/2;
			}

			else if(csJustify=="END")
			{
				x+=(pContext->GetBoundingRect()->width-
					((int)m_lines[line].length ? (int)m_lines[line].length :(int)m_lines[line].width));
				
			}
			
		}
		
		// check wether the Text lenght field is used, if so we should add
		// some spaces between the characters to fill the length (only positive
		// stretching). Anyway the pixel to logical function still doesn't work
		// well

		if(m_lines[line].length)
		{
			int size=m_lines[line].length;
			pVR->PixelToLogical(pFont,size );
			float  fspace=(float)(size)/(float)m_lines[line].numChars;
			fspace+=0.5;
			int ispace=fspace;
			pVR->SetCharacterInterspace(pFont,ispace);
		}



		pVR->DisplayText(pFont, (m_lines[line].pString),
						 m_lines[line].numChars, x, y,
						 GetEmissiveColor(pContext), &visibleRect);
		if(m_lines[line].length)
			pVR->SetCharacterInterspace(pFont,0);
		
		x=iOldX;

		// Move onto the next line
		if(!strcmp(GetFSjustify(1),"FIRST"))
			y -= m_lines[line].ascent;
		else		
		y -= m_lines[line].height;
	}

	// Unset the font now that we have finished with it.
	pVR->UnsetFont(pFont);
} /* DrawableText::Draw */


// =====================================================================
void DrawableText::JustifyBoundingRect(BRect *pTextBRect)
{
// Justify the BoundingRect 
	if(GetFShorizontal())
		{
			CString csJustify = GetFSjustify(0);
			if(csJustify=="FIRST"||csJustify=="BEGIN")
			{	
				pTextBRect->left=pTextBRect->left;
				
			}	
			else if(csJustify=="MIDDLE")
			{
				pTextBRect->left-=pTextBRect->width/2;
				
			}

			else if(csJustify=="END")
			{
				pTextBRect->left-=pTextBRect->width;
			
			}
		
			csJustify = GetFSjustify(1);
			
			if(csJustify=="FIRST")
			{	
				pTextBRect->top+=m_lines[0].ascent;
			}	
			else if(csJustify=="BEGIN")
			{
				pTextBRect->top=pTextBRect->top;
			}
			else if(csJustify=="MIDDLE")
			{
				pTextBRect->top+=m_lines[0].height/2; 

			}

			else if(csJustify=="END")
			{
				pTextBRect->top+=pTextBRect->height;
			
			}
		}
	else 
	{
		TRACE_PRINT("ONLY JUSTIFY HORIZONTAL SUPPORTED\n");
		assert(1);
	}


}