//This software module was originally developed by CSELT in the course of   //
//development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.             //
//This software module is an implementation of a part of one or more MPEG-4 //
//Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems        //
//(ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 Systems     //
//(ISO/IEC 14496-1) free license to this software module or modifications   //
//thereof for use in hardware or software products claiming conformance to  //
//the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use this software//
//module in hardware or software products are advised that its use may      //
//infringe existing patents.                                                //
//The original developer of this software module and his/her company, the   //
//subsequent editors and their companies, and ISO/IEC have no liability     //
//for use of this software module or modifications thereof in an            //
//implementation. Copyright is not released for non MPEG-4 Systems          //
//(ISO/IEC 14496-1) conforming products. CSELT S.pA. retains full right     //
//to use the code for his/her own purpose, assign or donate the code to a   //
//third party and to inhibit third parties from using the code for non      //
//MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright      //
//notice must be included in all copies or derivative works.                //
//Copyright  1999.                                                         //
//This work has been partly developed within the framework of the ESPRIT    //
//European Research Project  "MPEG-4 PC".                                   //
//////////////////////////////////////////////////////////////////////////////
//
// File:  Text2DProxy.h
//
// Authors: Matthew Leditschke 
//
// Modified: Gianluca Di Cagno  gianluca.dicagno@cselt.it
//
// Description:
//   The definition of a class which implements functionality common to
//   nodes which need to display text, namely the Text and
//   StreamingText nodes.
//
// Created: October, 1997.
//
//
///////////////////////////////////////////////////////////////////////////


#ifndef DRAWABLE_TEXT_H
#define DRAWABLE_TEXT_H


#include "GeometryNode.h"
#include "BRect.h"


class LineOfText
{
public:
	const char* pString;
	int numChars;		// Number of characters in the line
	unsigned int width;	// Dimension, in pixels, of the line
	unsigned int height;
	unsigned int length; //render width of the string 
	unsigned int ascent;
	unsigned int descent;
};


class DrawableText : public GeometryNode
{
public:
	DrawableText(MediaObject* pMO);
	~DrawableText();

	// Return a pointer to the FontStyle object belonging to the
	// Text node that this node is the proxy for.  To be overridden
	// by deriving classes.
	virtual FontStyle* GetFontStyle() const = 0;

	// Return the value of the length field of a Text node.  Nodes
	// which do not have this field must return -1.
	virtual int GetLength(int i) const { return -1; }

	// Functions inherited from the base class
	virtual void PreRender(Effects & effects,DrawableContext *pContext);
	virtual void Draw(BRect const* pClipBRect,DrawableContext * pContext);
	void GetAscentAndDescent(int &ascent,int &descent);
	void JustifyBoundingRect(BRect *pTextBRect);

	//
	// Protected implementation details
	//
protected:

	// Functions to get the value from the FontStyle node, which
	// use the default values if this node is missing
	const char * GetFSfamily() const;
    BOOL GetFShorizontal() const;
	const char * GetFSjustify(int i) const;
	const char* GetFSlanguage() const;
	BOOL GetFSleftToRight() const;
	float GetFSsize() const;
	int GetFSspacing() const;
	const char * GetFSstyle() const;
	int GetFStopToBottom() const;
		

	LPCSTR m_fontName;		// Font and formatting of the string
	int m_fontSize;
	BOOL m_bold;
	BOOL m_italic;
	BOOL m_underline;

	const char* m_data;		// The string of text to display
	int m_numLines;			// Number of lines in the string
	LineOfText* m_lines;	// An array of the lines of text
	int m_numLinesAllocated;// Allocated size of array
	int m_lengthSet;
	UINT m_justifyFlag;
};


#endif // DRAWABLE_TEXT_H
