//This software module was originally developed by CSELT in the course of   //
//development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.             //
//This software module is an implementation of a part of one or more MPEG-4 //
//Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems        //
//(ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 Systems     //
//(ISO/IEC 14496-1) free license to this software module or modifications   //
//thereof for use in hardware or software products claiming conformance to  //
//the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use this software//
//module in hardware or software products are advised that its use may      //
//infringe existing patents.                                                //
//The original developer of this software module and his/her company, the   //
//subsequent editors and their companies, and ISO/IEC have no liability     //
//for use of this software module or modifications thereof in an            //
//implementation. Copyright is not released for non MPEG-4 Systems          //
//(ISO/IEC 14496-1) conforming products. CSELT S.pA. retains full right     //
//to use the code for his/her own purpose, assign or donate the code to a   //
//third party and to inhibit third parties from using the code for non      //
//MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright      //
//notice must be included in all copies or derivative works.                //
//Copyright  1999.                                                         //
//This work has been partly developed within the framework of the ESPRIT    //
//European Research Project  "MPEG-4 PC".                                   //
//////////////////////////////////////////////////////////////////////////////
//
//
// File:  FormProxy.cpp
//
// Author:  Gianluca Di Cagno  gianluca.dicagno@cselt.it
//
// Modified:
//
// Description:
//   The implementation of the proxy object used by the presenter
//   to handle the Form node.
//
// Created: February 1999.
//
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include "MediaObj.h"
#include "Presenter.h"
#include "nodes.h"
#include "FormProxy.h"
#include "DrawableNode.h"

LPCTSTR sAllowedConstraint[]=
{
 _T("AL"),
 _T("AH"),
 _T("AR"),
 _T("AT"),
 _T("AV"),
 _T("AB"),
 _T("SH"),
 _T("SHin"),
 _T("SV"),
 _T("SVin")
} ;
#define NUMBER_OF_CONSTRAINTS 20

ConstraintsFactoryBase *FormProxy :: TheConstraintsFactory [NUMBER_OF_CONSTRAINTS];


// =====================================================================

//
// Presenter::InitForm
//
// Initialise the proxy data structure and attach it to the render data
// pointer of the given media object node.
//

FormProxy* Presenter::InitForm(Form* pObj)
{
	return new  FormProxy(pObj);
}

// =====================================================================

//
// FormProxy::FormProxy
//
// 
//


FormProxy::FormProxy(MediaObject* pMO): GroupNodeProxy(pMO)
{

	m_drawableContextList=NULL;
	m_numberOfContexts = m_ContextListSize=0;
	m_nAllocatedLength = 0;
	m_nLength=0;
	m_IndexOfChildrenToApply=NULL;

} /* FormProxy::FormProxy */


// =====================================================================

//
// FormProxy::~FormProxy
//
//  

FormProxy::~FormProxy()
{
	delete m_drawableContextList;
}

// =====================================================================

//
// FormProxy::PreRender
//
// PreRender the children in the order specified by the Form
// order field.
// 
//

void FormProxy::PreRender(Effects &effects)
{
	
	 
	 m_nLength= GetObject()->GetChildCount();
	 if (m_nLength > m_nAllocatedLength)
	 {
		// Delete the previous arrays - if required
		if (m_IndexOfChildrenToApply)
			delete [] m_IndexOfChildrenToApply;
		m_IndexOfChildrenToApply=new int [m_nLength*2];
		m_nAllocatedLength = m_nLength;
	 }
	 
	 
	 effects.m_ParentGroup=this;
	 
	 //create the form bounding rect which is referred in the GroupsIndexList as 0 
	 CreateFormContext(effects);
	 
	 
	 //create children  bounding box
	 int i;
	 for ( i = 0; i <m_nLength; i++)
	 {
		Traversable * pTrav=GetChild(i);
		if(pTrav)
		 pTrav->PreRender(effects);
	 }
	 
	 
	 int *pSetOfNodes;
	 int iNumberOfObjectsInAGroup;
	 int x=0;
	 
	 	
	 // go through the group field, create then a joint list with the groups field
	 // and apply the constrain 
	 while(GetNextSetFromGroupsIndex(pSetOfNodes,iNumberOfObjectsInAGroup))
	 {
		CreateJointGroupsAndGroupsIndexList(pSetOfNodes,iNumberOfObjectsInAGroup);
		 ApplyConstraint(GetForm()->constraints[x++],m_IndexOfChildrenToApply,iNumberOfObjectsInAGroup);
	 
	 }
	 //finally set the clipping rectangle and Update the current BBox list of each drawable
	 //node, this part should be encapsulated better since can be reused for layout
	 for (i=1;i<m_nLength+1;i++)
		{
	 		m_drawableContextList[i]->SetClippingRect(m_drawableContextList[0]->GetBoundingRect());
			((DrawableNode *)m_drawableContextList[i]->GetDrawableProxy())->UpdateBoundingRect(m_drawableContextList[i]);
		}
	 
	 m_numberOfContexts=0;
	 m_ContextListSize=0;
	 effects.m_ParentGroup=NULL;
} /* FormProxy::PreRender */



// =====================================================================
//
// FormProxy::GetNextSetFromGroupsIndex
// returns a pointer to the indexes of DrawableContext taken from 
// the Contextset of nodes from the groupsIndex field of a Form
// each group is separated by a -1
//	

BOOL FormProxy::GetNextSetFromGroupsIndex(const int *&pSetOfNodes,int &iNumberOfObjects)	
{
	static int iCounter=0; //seems ok: warning.
	static int iLastCounter=0;
	
	if(iCounter>=GetForm()->groupsIndex.GetLength())
	{
		iLastCounter=iCounter=0;
		return FALSE;
	}
	while(GetForm()->groupsIndex[iCounter++]!=-1)
		;
	
	
	pSetOfNodes =& GetForm()->groupsIndex[iLastCounter];
	iNumberOfObjects=iCounter-iLastCounter-1;
	iLastCounter=iCounter;
	
	return TRUE;
}  /*  FormProxy::FormProxy */ 
 

// =====================================================================
//
// FormProxy::ApplyConstrain
// returns a pointer to the indexes of DrawableContext taken from 
// the Contextset of nodes from the groupsIndex field of a Form
// each group is separated by a -1
//	

void  FormProxy::ApplyConstraint(ZString zsConstraint,const int * iSetOfNodes,int iSetLength)
{
	int i=0;

	CString csCopyofConstraint((LPCSTR)zsConstraint);
	
	CString csToken;
	
	//GetFirstToken (example "SH xxx", SH is first token)
	int iFirstTokenEnd=csCopyofConstraint.Find(' ');
	
	if(iFirstTokenEnd!=-1)
		csToken=csCopyofConstraint.Left(iFirstTokenEnd);
	else
		csToken=csCopyofConstraint;
	//

	//find right factory for the token
	while(_tcscmp(csToken,sAllowedConstraint[i++]))
					;	
		
	//allocate the constraint using the right factory
	Constraint * pConstraint= (Constraint *)(TheConstraintsFactory[--i])->CreateConstraint();		 
	
	//save form pointer in Costraint class
	pConstraint->m_pFormProxy=this;
	
	//Get second token
	int iSpace;
	if(iFirstTokenEnd!=-1)
	{
		csToken=csCopyofConstraint.Mid(iFirstTokenEnd);
		csToken.Remove(' ');
		if (!csToken.IsEmpty())
		{
			iSpace=_ttoi(csToken);
			pConstraint->m_iSpace=iSpace;
		}
	}
	//
	pConstraint->m_drawableContextList=m_drawableContextList;
	pConstraint->Apply(iSetOfNodes,iSetLength);
	delete pConstraint;

}     /*FormProxy::ApplyConstrain*/






// =====================================================================

//
// FormProxy::CreateGroupsFieldByIndex
//
// 
//

BOOL FormProxy::GetGroupsFieldByIndex(const int *& pGroups,int &iNumberOfChildren,int index)
{
 
	 int iCounter=0;
	 int iLastCounter=0;
	
	
	
	while(index-->0)
	{
		pGroups=&GetForm()->groups[iCounter];
		iLastCounter=iCounter;
		while(GetForm()->groups[iCounter++]!=-1)//add exception code
				;


		
	}
	
	iNumberOfChildren=iCounter-iLastCounter-1;
	return TRUE;

}  /*FormProxy::GetGroupsFieldByIndex */

// =====================================================================

//
// FormProxy::CreateFormContext
//
// 
//


void FormProxy::CreateFormContext(Effects &effects)
{
	//create the form drawable context, improve encapsulation
	
	 m_drawableContext.GetBoundingRect()->height = ((Vec2f)GetForm()->size).y;
	 m_drawableContext.GetBoundingRect()->width = ((Vec2f)GetForm()->size).x;
	  m_drawableContext.GetBoundingRect()->left=-m_drawableContext.GetBoundingRect()->width/2;
	 m_drawableContext.GetBoundingRect()->top=+m_drawableContext.GetBoundingRect()->height/2;
	 effects.transform.Apply(m_drawableContext.GetBoundingRect()->left,m_drawableContext.GetBoundingRect()->top);		
	 AddToDrawableContextList(&m_drawableContext);
	//
} /* FormProxy::CreateFormContext */   



// =====================================================================
//
// FormProxy::CreateJointGroupsAndGroupsIndexList
// returns a pointer to the indexes of DrawableContext taken from 
// the Contextset of nodes from the groupsIndex field of a Form
// each group is separated by a -1
//	
void FormProxy::CreateJointGroupsAndGroupsIndexList(const int * pSetOfNodes,int &iNumberOfObjectsInAGroup)
{
	
		 int h=0;
		 
	 	 
		 for (int i=0;i<iNumberOfObjectsInAGroup;i++)
			{
				if (pSetOfNodes[i]>GetNumberOfFormChildren())
				{
		
			
					int *pGroups;
					int iNumberOfChildren;
					int index=(pSetOfNodes[i]-GetNumberOfFormChildren());			
					
								
					GetGroupsFieldByIndex(pGroups,iNumberOfChildren,index);
			
						
					int z;
					for(z=0;z<iNumberOfChildren;z++)
						m_IndexOfChildrenToApply[h++]=pGroups[z];
					
					
					continue;
				}
				m_IndexOfChildrenToApply[h++]=pSetOfNodes[i];
								
			}
		 iNumberOfObjectsInAGroup=h;
}  /*FormProxy::CreateJointGroupsAndGroupsIndexList*/


// =====================================================================
//
// AL::Apply
// 
// constrain group to the xmin of the lefttop component
//	

void AL::Apply(const int*pGroup,int iNumberOfElementsInGroup)
{
	int iXmin;
	int i;
		
	if(m_iSpace==-1)
	{
		iXmin=m_drawableContextList[pGroup[0]]->GetBoundingRect()->left;
	
		for (i=1;i<iNumberOfElementsInGroup;i++)
		{
			if ((m_drawableContextList[pGroup[i]]->GetBoundingRect()->left)<iXmin)
				iXmin= m_drawableContextList[pGroup[i]]->GetBoundingRect()->left;
		}
	
		for (i=0;i<iNumberOfElementsInGroup;i++) 
		{
			//should skip the form node itself
			if(pGroup[i]!=0)
				m_drawableContextList[pGroup[i]]->GetBoundingRect()->left=iXmin;
					
		}
	}
	else
		for (i=1;i<iNumberOfElementsInGroup;i++) 
		{
			//should skip the form node itself
			if(pGroup[i]!=0)
				m_drawableContextList[pGroup[i]]->GetBoundingRect()->left=m_drawableContextList[pGroup[0]]->GetBoundingRect()->left+m_iSpace;
					
		}
		
}  /*AL::Apply*/

// =====================================================================
//
// AR::Apply
//  
// constrain group to the xmax of the right most component
//	
void AR::Apply(const int*pGroup,int iNumberOfElementsInGroup)
{
	int iXmax;
	int i;
		
	if(m_iSpace==-1)
	{
		iXmax=m_drawableContextList[pGroup[0]]->GetBoundingRect()->left+m_drawableContextList[pGroup[0]]->GetBoundingRect()->width;
		
		for (i=1;i<iNumberOfElementsInGroup;i++)
		{
			if ((m_drawableContextList[pGroup[i]]->GetBoundingRect()->left+m_drawableContextList[pGroup[i]]->GetBoundingRect()->width)>iXmax)
				iXmax= m_drawableContextList[pGroup[i]]->GetBoundingRect()->left+m_drawableContextList[pGroup[i]]->GetBoundingRect()->width;
		}
		
		for (i=0;i<iNumberOfElementsInGroup;i++) //should skip the form node itself
		{
			if(pGroup[i]!=0)
				m_drawableContextList[pGroup[i]]->GetBoundingRect()->left=iXmax-m_drawableContextList[pGroup[i]]->GetBoundingRect()->width;
					
		}
	}
	else
		for (i=1;i<iNumberOfElementsInGroup;i++) //should skip the form node itself
		{
			if(pGroup[i]!=0)
				m_drawableContextList[pGroup[i]]->GetBoundingRect()->left=
					m_drawableContextList[pGroup[0]]->GetBoundingRect()->left+m_drawableContextList[pGroup[0]]->GetBoundingRect()->width	
					-m_iSpace-m_drawableContextList[pGroup[i]]->GetBoundingRect()->width;
				
					
		}
		
} /*AR::Apply*/

// =====================================================================
//
// AT::Apply
// 
// constrain group to the ymax of the top most component
// 
//	
void AT::Apply(const int*pGroup,int iNumberOfElementsInGroup)
{
	int iYmax;
	int i;
		
	if(m_iSpace==-1)
	{
		iYmax=m_drawableContextList[pGroup[0]]->GetBoundingRect()->top;
		
		for (i=1;i<iNumberOfElementsInGroup;i++)
		{
			if (m_drawableContextList[pGroup[i]]->GetBoundingRect()->top>iYmax)
				iYmax = m_drawableContextList[pGroup[i]]->GetBoundingRect()->top;
		}
		
		for (i=0;i<iNumberOfElementsInGroup;i++) //should skip the form node itself
		{
			if(pGroup[i]!=0)
				m_drawableContextList[pGroup[i]]->GetBoundingRect()->top=iYmax;
					
		}
	}
	else
	{
		for (i=1;i<iNumberOfElementsInGroup;i++) //should skip the form node itself
			if(pGroup[i]!=0)
				m_drawableContextList[pGroup[i]]->GetBoundingRect()->top=
					m_drawableContextList[pGroup[0]]->GetBoundingRect()->top-m_iSpace;
					
		
	}
		
} /*AT::Apply*/

// =====================================================================
//
// AB::Apply
// 
// constrain group to the ymin of the bottom-most component
// 
//	
void AB::Apply(const int*pGroup,int iNumberOfElementsInGroup)
{
	int iYmin;
	int i;
		
	
	if(m_iSpace==-1)
	{
		iYmin=m_drawableContextList[pGroup[0]]->GetBoundingRect()->top-m_drawableContextList[pGroup[0]]->GetBoundingRect()->height;
		
		for (i=1;i<iNumberOfElementsInGroup;i++)
		{
			if ((m_drawableContextList[pGroup[i]]->GetBoundingRect()->top-m_drawableContextList[pGroup[i]]->GetBoundingRect()->height)<iYmin)
				iYmin= m_drawableContextList[pGroup[i]]->GetBoundingRect()->top-m_drawableContextList[pGroup[i]]->GetBoundingRect()->height;
		}
		
		for (i=0;i<iNumberOfElementsInGroup;i++) //should skip the form node itself
		{
			if(pGroup[i]!=0)
				m_drawableContextList[pGroup[i]]->GetBoundingRect()->top=iYmin+m_drawableContextList[pGroup[i]]->GetBoundingRect()->height;
					
		}
	}
	else
	{
		for (i=1;i<iNumberOfElementsInGroup;i++) //should skip the form node itself
		{
			if(pGroup[i]!=0)
				m_drawableContextList[pGroup[i]]->GetBoundingRect()->top=
						m_drawableContextList[pGroup[0]]->GetBoundingRect()->top-
						m_drawableContextList[pGroup[0]]->GetBoundingRect()->height+
						m_iSpace+
						m_drawableContextList[pGroup[i]]->GetBoundingRect()->height;
				
			
					
		}
	}
		
} /*AB::Apply*/






// =====================================================================
//
// SH::Apply
// 
// The differences between the xmin of each component and the xmax of the
// previous one all become equal, first and last already constrained 
// horizontally
	

void SH::Apply(const int*pGroup,int iNumberOfElementsInGroup)
{
	
	int i;
	int iSumOfChildrenLength=0;
	int  iSpaceBetweenObjects;

	if(m_iSpace==-1)
	{
		float fSpaceBetweenObjects=m_drawableContextList[pGroup[iNumberOfElementsInGroup-1]]->GetBoundingRect()->left-
							 m_drawableContextList[pGroup[0]]->GetBoundingRect()->left-
							 m_drawableContextList[pGroup[0]]->GetBoundingRect()->width;

		for (i=1;i<iNumberOfElementsInGroup-1;i++)
			iSumOfChildrenLength+= m_drawableContextList[pGroup[i]]->GetBoundingRect()->width;

		fSpaceBetweenObjects-=iSumOfChildrenLength;
		fSpaceBetweenObjects/=(iNumberOfElementsInGroup-1);
		iSpaceBetweenObjects=fSpaceBetweenObjects+0.5;
	}
	else
		iSpaceBetweenObjects=m_iSpace;
	
	
	for (i=1;i<iNumberOfElementsInGroup-1;i++) //should skip the form node itself
	{
		if(pGroup[i]!=0)
			m_drawableContextList[pGroup[i]]->GetBoundingRect()->left=
			m_drawableContextList[pGroup[i-1]]->GetBoundingRect()->left+
			m_drawableContextList[pGroup[i-1]]->GetBoundingRect()->width+
			iSpaceBetweenObjects;
				
	}
	if(m_iSpace!=-1)
		if(pGroup[iNumberOfElementsInGroup-1]!=0)
			m_drawableContextList[pGroup[iNumberOfElementsInGroup-1]]->GetBoundingRect()->left=
			m_drawableContextList[pGroup[iNumberOfElementsInGroup-2]]->GetBoundingRect()->left+
			m_drawableContextList[pGroup[iNumberOfElementsInGroup-2]]->GetBoundingRect()->width+
			iSpaceBetweenObjects;
		
} /*SH::Apply*/
// =====================================================================
//
// SHin::Apply
//  
// differences between the xmin of each component and the xmax of the 
// previous one all become equal, references the edges of the  form
//	
void SHin::Apply(const int*pGroup,int iNumberOfElementsInGroup)
{
	
	int i;
	int iSumOfChildrenLength=0;
	int  iSpaceBetweenObjects;

	float fSpaceBetweenObjects=m_drawableContextList[0]->GetBoundingRect()->width;

	for (i=0;i<iNumberOfElementsInGroup;i++)
		iSumOfChildrenLength+= m_drawableContextList[pGroup[i]]->GetBoundingRect()->width;

	fSpaceBetweenObjects-=iSumOfChildrenLength;
	fSpaceBetweenObjects/=(iNumberOfElementsInGroup+1);
	iSpaceBetweenObjects=fSpaceBetweenObjects+0.5;
		
	m_drawableContextList[pGroup[0]]->GetBoundingRect()->left=
		m_drawableContextList[0]->GetBoundingRect()->left+iSpaceBetweenObjects;

		
	for (i=1;i<iNumberOfElementsInGroup;i++) //should skip the form node itself
	{
		if(pGroup[i]!=0)
			m_drawableContextList[pGroup[i]]->GetBoundingRect()->left=
			m_drawableContextList[pGroup[i-1]]->GetBoundingRect()->left+
			m_drawableContextList[pGroup[i-1]]->GetBoundingRect()->width+
			iSpaceBetweenObjects;
				
	}
	
		
} /*SHin::Apply*/

// =====================================================================
//
// SV::Apply
// 
// The differences between the ymin of each component and the ymax
// of the previous one all become equal, the first and the last 
// component shall be constrained vertically already
void SV::Apply(const int*pGroup,int iNumberOfElementsInGroup)
{
	
	int i;
	int iSumOfChildrenLength=0;
	int  iSpaceBetweenObjects;

	if(m_iSpace==-1)
	{
		float fSpaceBetweenObjects=m_drawableContextList[pGroup[iNumberOfElementsInGroup-1]]->GetBoundingRect()->top-
							 (m_drawableContextList[pGroup[0]]->GetBoundingRect()->top+
							 m_drawableContextList[pGroup[0]]->GetBoundingRect()->height);

		for (i=1;i<iNumberOfElementsInGroup-1;i++)
			iSumOfChildrenLength+= m_drawableContextList[pGroup[i]]->GetBoundingRect()->height;

		fSpaceBetweenObjects-=iSumOfChildrenLength;
		fSpaceBetweenObjects/=(iNumberOfElementsInGroup-1);
		iSpaceBetweenObjects=fSpaceBetweenObjects+0.5;
	}
	else
		iSpaceBetweenObjects=m_iSpace;
	
	
	for (i=1;i<iNumberOfElementsInGroup-1;i++) //should skip the form node itself
	{
		if(pGroup[i]!=0)
			m_drawableContextList[pGroup[i]]->GetBoundingRect()->top=
			m_drawableContextList[pGroup[i-1]]->GetBoundingRect()->top+
			m_drawableContextList[pGroup[i-1]]->GetBoundingRect()->height+
			iSpaceBetweenObjects;
				
	}
	if(m_iSpace!=-1)
		if(pGroup[iNumberOfElementsInGroup-1]!=0)
			m_drawableContextList[pGroup[iNumberOfElementsInGroup-1]]->GetBoundingRect()->top=
			m_drawableContextList[pGroup[iNumberOfElementsInGroup-2]]->GetBoundingRect()->top+
			m_drawableContextList[pGroup[iNumberOfElementsInGroup-2]]->GetBoundingRect()->height+
			iSpaceBetweenObjects;
		
} /*SV::Apply*/
// =====================================================================
//
// SVin::Apply
// The differences between the ymin of each component and the ymax
// of the previous one all become equal, references are the edges of
// the layout 
	
void SVin::Apply(const int*pGroup,int iNumberOfElementsInGroup)
{
	
	int i;
	int iSumOfChildrenLength=0;
	int  iSpaceBetweenObjects;

	float fSpaceBetweenObjects=m_drawableContextList[0]->GetBoundingRect()->height;

	for (i=0;i<iNumberOfElementsInGroup;i++)
		iSumOfChildrenLength+= m_drawableContextList[pGroup[i]]->GetBoundingRect()->height;

	fSpaceBetweenObjects-=iSumOfChildrenLength;
	fSpaceBetweenObjects/=(iNumberOfElementsInGroup+1);
	iSpaceBetweenObjects=fSpaceBetweenObjects+0.5;
		
	m_drawableContextList[pGroup[0]]->GetBoundingRect()->top=
		m_drawableContextList[0]->GetBoundingRect()->top+iSpaceBetweenObjects;

		
	for (i=1;i<iNumberOfElementsInGroup;i++) //should skip the form node itself
	{
		if(pGroup[i]!=0)
			m_drawableContextList[pGroup[i]]->GetBoundingRect()->top=
			m_drawableContextList[pGroup[i-1]]->GetBoundingRect()->top+
			m_drawableContextList[pGroup[i-1]]->GetBoundingRect()->height+
			iSpaceBetweenObjects;
				
	}
	
		
} /*SVin::Apply*/