//This software module was originally developed by CSELT in the course of   //
//development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.             //
//This software module is an implementation of a part of one or more MPEG-4 //
//Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems        //
//(ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 Systems     //
//(ISO/IEC 14496-1) free license to this software module or modifications   //
//thereof for use in hardware or software products claiming conformance to  //
//the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use this software//
//module in hardware or software products are advised that its use may      //
//infringe existing patents.                                                //
//The original developer of this software module and his/her company, the   //
//subsequent editors and their companies, and ISO/IEC have no liability     //
//for use of this software module or modifications thereof in an            //
//implementation. Copyright is not released for non MPEG-4 Systems          //
//(ISO/IEC 14496-1) conforming products. CSELT S.pA. retains full right     //
//to use the code for his/her own purpose, assign or donate the code to a   //
//third party and to inhibit third parties from using the code for non      //
//MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright      //
//notice must be included in all copies or derivative works.                //
//Copyright  1999.                                                         //
//This work has been partly developed within the framework of the ESPRIT    //
//European Research Project  "MPEG-4 PC".                                   //
//////////////////////////////////////////////////////////////////////////////
//
// File:  GeometryNode.h
//
// Author: Matthew Leditschke 
//
// Modified: Gianluca Di Cagno  gianluca.dicagno@cselt.it
//
// Description:
//
//   The implementation of a class which contains details specific to
//   geometry nodes.
//
// Created: September, 1997.
//
//
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include "GeometryNode.h"

#include "trace.h"
// ========================================================================

//
// GeometryNode::NeedsDrawing
//
// Returns TRUE if this geometry node needs to be drawn (because it
// has changed since it was last drawn).  This will be true if the
// node has been updated (via BIFS or ROUTES), or the appearance node
// associated with this node has changed, or the transformation has
// changed.
//

BOOL GeometryNode::NeedsDrawing(DrawableContext * pContext) 
{
  //  #GB
  //  DWORD  presTime = GetPresenter()->GetTime(); 
	DWORD  presTime = GetFlexTime();
    
	// Check if an object is being tracked, in this case, you need 
	// to redraw all the object (for now)
    
	if (GetPresenter()->GetIsTracking())
	{
		TRACE_PRINT("IsTracking\n");
		return TRUE;
	}
	
	if(pContext->m_updateRequired)
			return TRUE;

   
	// Check if the Appearance node has been updated
	Appearance* pAppearance = pContext->GetAppearance();
	if (pAppearance != NULL)
	{
		if (pAppearance->GetUpdateTime() >= presTime)
		{
			TRACE_PRINT("IspAppearance\n");
	//		TRACE_PRINT2("UpdateTime:%d,presTime %d/n",pAppearance->GetUpdateTime(),presTime);
			return TRUE;
		}

		// Check if the material node has been updated
		Material2D* pMaterial = (Material2D*) (MediaObject*) pAppearance->material;
	     
		if ((pMaterial != NULL) && (pMaterial->GetUpdateTime() >= presTime))
		{
			TRACE_PRINT("IspMaterial\n");
			return TRUE;
		}

		// Check if the texture has changed
		MediaObject* pTexture = (MediaObject*)pContext->GetAppearance()->texture;
		
		if (pTexture !=NULL)
		{
			DrawableNode * pDrawableNode = (DrawableNode *) pTexture->GetRenderingObject();
	    
			if((pDrawableNode != NULL) && (pDrawableNode->NeedsDrawing(pContext)))
				return TRUE;
		}
				
		//Check if the LineProperties node has been updated
		if ((pMaterial && pMaterial->lineProps != NULL_OBJECT)
			&& (pMaterial->lineProps->GetNodeCode() == CODE_LineProperties) ) {

			LineProperties *pLineProperties = (LineProperties *) ((MediaObject *) pMaterial->lineProps);
		
			if((pLineProperties != NULL) && (pLineProperties->GetUpdateTime() >= presTime))
			{
				TRACE_PRINT("islineprops\n");
				return TRUE;
			}
		}
}

	

	// Check if the geometry node itself has been updated
	// GDC modified the control changing ">=" in "<=": checking phase
	
//	TRACE_PRINT2("UpdateTime %d , presTime = %d",GetObject()->GetUpdateTime(), presTime);
//	TRACE_PRINT("\n");
    //if((GetObject()->GetUpdateTime()<=(presTime+100)) || (GetObject()->GetUpdateTime() >=(presTime-100)))
 
	//Perche non usare il metodo OnUpdate ?

	if(GetObject()->GetUpdateTime()+100 >= presTime)
	{
		TRACE_PRINT("IsUpdateTime\n");
		return TRUE;
	}


		return FALSE;
} /* GeometryNode::NeedsDrawing */


// ========================================================================

//
// GeometryNode::FindTouchSensor
//
// Call FindTouchSensor() of this node's enclosing grouping node, which
// is the parent of the Shape node that this node is the geometry for.
// The pointer to the node context is needed to find the Shape node

void GeometryNode::FindTouchSensor(EventInfo const* pEvent,DrawableContext * pContext)
{
	// Go through all of the parents
	int whichParent = 0;
	MediaObject* pParent =(MediaObject *) pContext->GetShapeProxy()->GetObject()->GetParent(whichParent);
	while (pParent != NULL)
	{
		((Traversable*)pParent->GetRenderingObject())->FindTouchSensor(pEvent,pContext);
		whichParent++;
		pParent = (MediaObject*) pContext->GetShapeProxy()->GetObject()->GetParent(whichParent);
	}
} /* GeometryNode::FindTouchSensor */


// =====================================================================

//
// GeometryNode::GetEmissiveColor
//
// Return the 'EmissiveColor' field from Material2D node in the
// Appearance node associated with this geometry object.  If either
// the Appearance node or the Material2D node is not present, then
// the default value is used.
//

Color3f GeometryNode::GetEmissiveColor(DrawableContext *pDrawableContext) const
{
	Appearance * pAppearance = pDrawableContext->GetAppearance();

	if ((pAppearance == NULL) || (pAppearance->material == NULL))
		return Color3f(0.8, 0.8, 0.8);
	else
		return (Color3f) ((Material2D*)(MediaObject*)pAppearance->material)->emissiveColor;
} /* GeometryNode::GetEmissiveColor */


// =====================================================================

//
// GeometryNode::GetTransparency
//
// Return the 'transparency' field from Material2D node in the
// Appearance node associated with this geometry object.  If either
// the Appearance node or the Material2D node is not present, then
// the default value is used.
//

float GeometryNode::GetTransparency(DrawableContext *pDrawableContext) const
{
	Appearance * pAppearance = pDrawableContext->GetAppearance();

	if ((pAppearance == NULL) || (pAppearance->material == NULL))
		return 0.0;
	else
		return ((Material2D*)(MediaObject*)pAppearance->material)->transparency;
} /* GeometryNode::GetTransparency */


// =====================================================================

//
// GeometryNode::GetFilled
//
// Return the 'filled' field from Properties2D node in the
// Appearance node associated with this geometry object.  If either
// the Appearance node or the Properties2D node is not present, then
// the default value is used.
//

Bool GeometryNode::GetFilled(DrawableContext *pDrawableContext) const
{
	Appearance * pAppearance = pDrawableContext->GetAppearance();

	if ((pAppearance == NULL) || (pAppearance->material == NULL))
		return FALSE;
	else
		return (Bool)((Material2D*)(MediaObject*)pAppearance->material)->filled;
} /* GeometryNode::GetFilled */



// =====================================================================

//
// GeometryNode::GetWidth
//
// Return the 'width' field from Properties2D node in the
// Appearance node associated with this geometry object.  If either
// the Appearance node or the Properties2D node is not present, then
// the default value is used.
//

float GeometryNode::GetWidth(DrawableContext *pDrawableContext) const
{
	Appearance * pAppearance = pDrawableContext->GetAppearance();

	if ((pAppearance== NULL) || (pAppearance->material == NULL) || (((Material2D*)(MediaObject*)pAppearance->material)->lineProps==NULL_OBJECT))
		return 1;

	MediaObject *pLP = (MediaObject*) ((Material2D*)(MediaObject*)pAppearance->material)->lineProps;
	if (pLP->GetNodeCode() != CODE_LineProperties) return 1;
	return (float) ((LineProperties *) pLP)->width;
} /* GeometryNode::GetWidth */


// =====================================================================

