//This software module was originally developed by CSELT in the course of   //
//development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.             //
//This software module is an implementation of a part of one or more MPEG-4 //
//Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems        //
//(ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 Systems     //
//(ISO/IEC 14496-1) free license to this software module or modifications   //
//thereof for use in hardware or software products claiming conformance to  //
//the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use this software//
//module in hardware or software products are advised that its use may      //
//infringe existing patents.                                                //
//The original developer of this software module and his/her company, the   //
//subsequent editors and their companies, and ISO/IEC have no liability     //
//for use of this software module or modifications thereof in an            //
//implementation. Copyright is not released for non MPEG-4 Systems          //
//(ISO/IEC 14496-1) conforming products. CSELT S.pA. retains full right     //
//to use the code for his/her own purpose, assign or donate the code to a   //
//third party and to inhibit third parties from using the code for non      //
//MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright      //
//notice must be included in all copies or derivative works.                //
//Copyright  1999.                                                         //
//This work has been partly developed within the framework of the ESPRIT    //
//European Research Project  "MPEG-4 PC".                                   //
//////////////////////////////////////////////////////////////////////////////
//
// File:  GeometryNode.h
//
// Author: Matthew Leditschke 
//
// Modified: Gianluca Di Cagno  gianluca.dicagno@cselt.it
//
// Description:
//   The definition of a class which contains details specific to
//   geometry nodes.
//
// Created: September, 1997.
//
//
///////////////////////////////////////////////////////////////////////////


#ifndef GEOMETRY_NODE_H
#define GEOMETRY_NODE_H



#include "Nodes-CSELT.h"
#include "DrawableNode.h"
#include  "ShapeProxy.h"


class GeometryNode : public DrawableNode
{
	//
	// Public functions
	//
public:

	// Class constructor - set the pointer to the media object that
	// this object is attached to, and also the pointer to the visual
	// renderer.
	GeometryNode(MediaObject* pMO) : DrawableNode(pMO) {}

	// Set and get the Apperance for this geometry node.
	void SetAppearance(Appearance* pNode) { m_pAppearance = pNode; }
	Appearance* GetAppearance() const { return m_pAppearance; }

	// Set and get the pointer to the parent ShapeProxy node
	void SetShapeProxy(class ShapeProxy* pShapeProxy)
	{ m_pShapeProxy = pShapeProxy; }
	ShapeProxy* GetShapeProxy() const { return m_pShapeProxy; }

	// Returns TRUE if this geometry node needs to be drawn (because it
	// has changed since it was last drawn).  This will be true if the
	// node has been updated (via BIFS or ROUTES).
	virtual BOOL NeedsDrawing(DrawableContext * pContext);

	// Assume that the Geometry node will be transparent (which is
	// always the case for Circle and Text).  Other classes can override
	// this if required.
	virtual BOOL IsTransparent() const { return TRUE; }

    MediaObject * GetDrawableParent() const 
	{return (MediaObject*)GetShapeProxy()->GetObject()->GetParent(0);} 

	// Functions to get the values from the Appearance, Material2D 
	//  nodes, which use the default values if these node are
	// missing.
	Color3f GetEmissiveColor(DrawableContext *pDrawableContext) const;
	float GetTransparency(DrawableContext *pDrawableContext) const;
	Bool GetFilled(DrawableContext *pDrawableContext) const;
	

	float GetWidth(DrawableContext *pDrawableContext) const;

	// Functions inherited from base classes
	virtual void FindTouchSensor(EventInfo const* pEvent,DrawableContext * pContext);
	BOOL IsFinished() const { return TRUE; }


	//
	// Private implementation details
	//
private:
    Appearance*  m_pAppearance;
	class ShapeProxy* m_pShapeProxy;
};


#endif // GEOMETRY_NODE_H
