//This software module was originally developed by CSELT in the course of   //
//development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.             //
//This software module is an implementation of a part of one or more MPEG-4 //
//Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems        //
//(ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 Systems     //
//(ISO/IEC 14496-1) free license to this software module or modifications   //
//thereof for use in hardware or software products claiming conformance to  //
//the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use this software//
//module in hardware or software products are advised that its use may      //
//infringe existing patents.                                                //
//The original developer of this software module and his/her company, the   //
//subsequent editors and their companies, and ISO/IEC have no liability     //
//for use of this software module or modifications thereof in an            //
//implementation. Copyright is not released for non MPEG-4 Systems          //
//(ISO/IEC 14496-1) conforming products. CSELT S.pA. retains full right     //
//to use the code for his/her own purpose, assign or donate the code to a   //
//third party and to inhibit third parties from using the code for non      //
//MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright      //
//notice must be included in all copies or derivative works.                //
//Copyright  1999.                                                         //
//This work has been partly developed within the framework of the ESPRIT    //
//European Research Project  "MPEG-4 PC".                                   //
//////////////////////////////////////////////////////////////////////////////
//
// File:  GroupNodeProxy.cpp
//
// Author: Matthew Leditschke
//
// Modified:  Gianluca Di Cagno CSELT  gianluca.dicagno@cselt.it,
//            Ananda Allys France Telecom
//
// Description:
//   The implementation of a class which encapsulates all of the features
//   of a node which are common to nodes which have children.
//
// Created: September, 1997.
//
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include "Nodes-CSELT.h"
#include "GroupNodeProxy.h"


// =====================================================================

//
// GroupNodeProxy::UpdateTimeSensor
//
// UpdateTimeSensor nodes which are the children of this node.
//


void GroupNodeProxy::UpdateTimeSensor()
{
	int nNbChildren= GetObject()->GetChildCount();

	for (int i = 0; i < nNbChildren; i++)
	{
		Traversable * pTrav=GetChild(i);

		
		if (pTrav == NULL)
		{
			//notify node proxy absence
			return;
		}

		pTrav->UpdateTimeSensor();
	}
} /* GroupNodeProxy::UpdateTimeSensor */




// =====================================================================

//
// GroupNodeProxy::PreRenderChildren
//
// Apply the PreRender method to all of the children of this node.
//

void GroupNodeProxy::PreRenderChildren(Effects & effects)
{
	 int length= GetObject()->GetChildCount();
	 
	 for (int i = 0; i <length; i++)
	 {
		Traversable * pTrav=GetChild(i);
		if(pTrav)
		 if(pTrav->IsSensor())
			effects.m_pHandler=(class UserEventHandlerProxy *)pTrav;
	 }

	 for ( i = 0; i <length; i++)
	 {
		Traversable * pTrav=GetChild(i);
		if(pTrav)
		 pTrav->PreRender(effects);
	 }
	 //restore sensor handler
	 effects.m_pHandler=NULL;
} /* GroupNodeProxy::PreRender */







// =====================================================================

//
// GroupNodeProxy::IsFinished
//
// A grouping node is only considered as finished when all of its
// children have finished.
//

BOOL GroupNodeProxy::IsFinished() const
{
	for (int i = 0; i < GetObject()->GetChildCount(); i++)
		if (!GetChild(i)->IsFinished())
			return FALSE;

	return TRUE;
} /* GroupNodeProxy::IsFinished */




//extensions for Form and Layout nodes. These nodes require post processing to the
//children drawableContext, hence the group node should have a list of 
//children drawablecontexts.

//should be a template...


void GroupNodeProxy::AddToDrawableContextList(DrawableContext* pDrawableContext)
{
	// If the array of drawable nodes is not large enough, make it larger
	if (m_numberOfContexts >= m_ContextListSize)
	{
		m_ContextListSize += 10;
		m_drawableContextList = (DrawableContext**)realloc(m_drawableContextList,
		m_ContextListSize * sizeof(DrawableContext*));
	}

	// Add the details of this presentable media object to the list
	m_drawableContextList[m_numberOfContexts] = pDrawableContext;
	m_numberOfContexts++;
} /* GroupNodeProxy:: AddToDrawList */

//-------------------------------------------------------------------------------
// #GB: set FlexTime for this grouping node and porpagate to the children
//-------------------------------------------------------------------------------
void GroupNodeProxy::SetFlexTime(const PresenterContext &pcCurrentTime)
{
  m_pcTimeContext = pcCurrentTime;

  int iChildrenCount = GetObject()->GetChildCount();
  for(int i=0; i<iChildrenCount; i++) {
    Traversable* t = GetChild(i);
	if (t) t->SetFlexTime(pcCurrentTime);
  }
}

