//This software module was originally developed by CSELT in the course of   //
//development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.             //
//This software module is an implementation of a part of one or more MPEG-4 //
//Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems        //
//(ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 Systems     //
//(ISO/IEC 14496-1) free license to this software module or modifications   //
//thereof for use in hardware or software products claiming conformance to  //
//the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use this software//
//module in hardware or software products are advised that its use may      //
//infringe existing patents.                                                //
//The original developer of this software module and his/her company, the   //
//subsequent editors and their companies, and ISO/IEC have no liability     //
//for use of this software module or modifications thereof in an            //
//implementation. Copyright is not released for non MPEG-4 Systems          //
//(ISO/IEC 14496-1) conforming products. CSELT S.pA. retains full right     //
//to use the code for his/her own purpose, assign or donate the code to a   //
//third party and to inhibit third parties from using the code for non      //
//MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright      //
//notice must be included in all copies or derivative works.                //
//Copyright  1999.                                                         //
//This work has been partly developed within the framework of the ESPRIT    //
//European Research Project  "MPEG-4 PC".                                   //
//////////////////////////////////////////////////////////////////////////////
//
// File:  ImageTextureProxy.cpp
//
// Author: Gianluca Di Cagno  gianluca.dicagno@cselt.it
//
// Modified:
//
// Description:
//   The implementation of the proxy object used by the presenter
//   to render the ImageTexture node.
//
// Created: 16 December, 1998.
//
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include "Nodes-CSELT.h"
#include "ImageTextureProxy.h"
#include "Stream.h"
#include "trace.h"


// =====================================================================

//
// Presenter::InitImageTexture
//
// Initialise the proxy data structure and attach it to the render data
// pointer of the given media object node.
//

 ImageTextureProxy * Presenter::InitImageTexture(ImageTexture* pObj)
 {
	return new ImageTextureProxy(pObj);
 }


// =====================================================================

//
// ImageTextureProxy::ImageTextureProxy
//
// Class constructor.
//

ImageTextureProxy::ImageTextureProxy(MediaObject* pMO) : DrawableImage(pMO)
{
	m_firstTime=TRUE;
} /* ImageTextureProxy::ImageTextureProxy */




// =====================================================================

//
// ImageTextureProxy::PreRender
//
//

void ImageTextureProxy::PreRender(Effects & effects,DrawableContext * pContext)
{
	
	// The first time that this is called, the stream needs to be
	// started.
	if (m_firstTime/*&&effects.isSwitchedOn*/) //open point 
	{
		m_firstTime = FALSE;
		GetImageTexture()->OpenStream();
	}
	
	
	if (!pContext->IsSwitchedOn())
	   return;
	
	
	DrawableImage::PreRender(effects,pContext);

	pContext->m_isTransparent=IsTransparent(pContext);
	
} /* ImageTextureProxy::PreRender */


// =====================================================================

//
// ImageTextureProxy::IsTransparent
//
//

 BOOL ImageTextureProxy::IsTransparent(DrawableContext * pDrawableContext)
 {
 	if(pDrawableContext->m_isTransparent)
		return TRUE; //base class already set it
	 Appearance * pAppearance = pDrawableContext->GetAppearance();
	if (pAppearance)
	{
	if ((pAppearance->material == NULL))
		return FALSE;
	
	if(((Material2D*)(MediaObject*)pAppearance->material)->transparency==0)
			return FALSE;
		else 
			return TRUE;
		
	}
	return FALSE;
	
 
 } /* ImageTextureProxy::IsTransparent */
