//This software module was originally developed by CSELT in the course of   //
//development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.             //
//This software module is an implementation of a part of one or more MPEG-4 //
//Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems        //
//(ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 Systems     //
//(ISO/IEC 14496-1) free license to this software module or modifications   //
//thereof for use in hardware or software products claiming conformance to  //
//the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use this software//
//module in hardware or software products are advised that its use may      //
//infringe existing patents.                                                //
//The original developer of this software module and his/her company, the   //
//subsequent editors and their companies, and ISO/IEC have no liability     //
//for use of this software module or modifications thereof in an            //
//implementation. Copyright is not released for non MPEG-4 Systems          //
//(ISO/IEC 14496-1) conforming products. CSELT S.pA. retains full right     //
//to use the code for his/her own purpose, assign or donate the code to a   //
//third party and to inhibit third parties from using the code for non      //
//MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright      //
//notice must be included in all copies or derivative works.                //
//Copyright  1999.                                                         //
//This work has been partly developed within the framework of the ESPRIT    //
//European Research Project  "MPEG-4 PC".                                   //
//////////////////////////////////////////////////////////////////////////////
//
// File:  IndexedLineSet2DProxy.cpp
//
// Author: Matthew Leditschke 
//
// Modified: Gianluca Di Cagno  gianluca.dicagno@cselt.it
//
// Description:
//   The implementation of the proxy object used by the presenter
//   to render the IndexedLineSet2D node.
//
// Created: August, 1997.
//
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include "Nodes-CSELT.h"
#include "IndexedLineSetProxy.h"
#include "ColorProxy.h"
#include "Coordinate2DProxy.h"
#include "trace.h"


// =====================================================================

//
// Presenter::InitIndexedLineSet2D
//
// Initialise the proxy data structure and attach it to the render data
// pointer of the given media object node.
//

IndexedLineSet2DProxy* Presenter::InitIndexedLineSet2D(IndexedLineSet2D* pObj)
{
	return new IndexedLineSet2DProxy(pObj);
}




// ======================================================================

//
// IndexedLineSet2DProxy::PreRender
//
//

void IndexedLineSet2DProxy::PreRender(Effects & effects)
{
	DrawableContext *pContext=InitPreRender(effects);
	
	if (!pContext->IsSwitchedOn()) 
		return;

	if(effects.m_ParentGroup)
		effects.m_ParentGroup->AddToDrawableContextList(pContext);

	// If the transparency for this node is 1, then it doesn't need
	// to be drawn, and its bounding rectangle is empty
	if (GetTransparency(pContext) > 1 - ALMOST_ZERO)
	{
		pContext->GetBoundingRect()->MakeEmpty();
		UpdateBoundingRect(pContext);
		return;
	}

	// Allocate memory for the local copy of the points (only if the
	// current array is not large enough).
	m_nLength = GetCoordinate2D()->point.GetLength();
	if (m_nLength > m_nAllocatedLength)
	{
		// Delete the previous arrays - if required
		if (m_lpPoints)
			delete [] m_lpPoints;
		

		// Create the new arrays
		m_lpPoints = new PointF[m_nLength];
		
		m_nAllocatedLength = m_nLength;
	}

	m_nColorLength = GetIndexLineSet()->colorIndex.GetLength();
	if (m_nColorLength > m_nPrevColorLength)
	{
		// update the previous length
		m_nPrevColorLength = m_nColorLength;

		// Delete the previous arrays - if required
		// and create the new one
		if (m_pColorIndex) 
			delete[] m_pColorIndex;
		m_pColorIndex = new int[m_nColorLength];
	}
	// Put in the color values
	for (int i = 0; i < m_nColorLength; i++)
		m_pColorIndex[i] = (int)(GetIndexLineSet()->colorIndex[i]);

	m_nCoordLength = GetIndexLineSet()->coordIndex.GetLength();
	if (m_nCoordLength > m_nPrevCoordLength)
	{
		// update the previous length
		m_nPrevCoordLength = m_nCoordLength;

		// Delete the previous arrays - if required
		// and create the new one
		if (m_pCoordIndex) 
			delete[] m_pCoordIndex;
		m_pCoordIndex = new int[m_nCoordLength];
	}
	// Put in the coordinate values
	for (i = 0; i < m_nCoordLength; i++)
		m_pCoordIndex[i] = (int)(GetIndexLineSet()->coordIndex[i]);

	// Determine where the points are
	for (i =0; i < m_nLength; i++)
	{
		m_lpPoints[i].x = GetCoordinate2D()->point[i].x; 
		m_lpPoints[i].y = GetCoordinate2D()->point[i].y;
	}


	// Determine where these points lie after the transformation 
	// has been applied.
	for (i = 0; i < m_nLength; i++)
	{
		pContext->m_transform.Apply(m_lpPoints[i].x, m_lpPoints[i].y);
		
	}

	// Set the bounding box
	pContext->GetBoundingRect()->SetDimensions(m_lpPoints, m_nLength);
	

	if(effects.m_ParentGroup==NULL)//necessary for postprocessing style nodes 
		UpdateBoundingRect(pContext);

	if(effects.orderHasChanged)
			pContext->m_updateRequired = TRUE;
} /* IndexedLineSet2DProxy::PreRender */


// ============================================================================

//
// IndexedLineSet2DProxy::Draw
//
// Draw the lines.
//

void IndexedLineSet2DProxy::Draw(BRect const* pClipBRect,DrawableContext * pContext)
{
	assert(pClipBRect != NULL);
	VisualRenderer* pVR = GetVisualRenderer();

	// The amount of the lines to draw lies within both the clipping
	// rectangle and the lines own bounding rectangle.
	BRect visibleRect = *pClipBRect;
	visibleRect.Intersection(pContext->GetClippedBoundingRect());
	if (visibleRect.IsEmpty())
		return;

	int i = 0;


	

	// Draw the lines. 
	// If the color node isn't NULL, use it
	if (GetColor())
	{
		if (m_nCoordLength > 0)
		{
			if (GetVertex())
			{
				if (m_nColorLength > 0)
				{
				}
				else
				{
				}
			}
			else
			{
				if (m_nColorLength > 0)
				{
					// Join the lines according to the coordIndex array
					// And use the colors indicised by the colorIndex array
					int count;
					for (i = 0, count = 0; i < m_nCoordLength; i += 3, count++)
						pVR->DisplayLines(visibleRect, 
										  m_lpPoints[m_pCoordIndex[i]].x,
										  m_lpPoints[m_pCoordIndex[i]].y,
										  m_lpPoints[m_pCoordIndex[i + 1]].x,
										  m_lpPoints[m_pCoordIndex[i + 1]].y,
										  GetColor()->color[m_pColorIndex[count]],
										  GetWidth(pContext));
				}
				else
				{
					// Join the lines according to the coordIndex array
					// And use the colors indicised sequentially
					int count;
					for (i = 0, count = 0; i < m_nCoordLength; i += 3, count++)
						pVR->DisplayLines(visibleRect, 
										  m_lpPoints[m_pCoordIndex[i]].x,
										  m_lpPoints[m_pCoordIndex[i]].y,
										  m_lpPoints[m_pCoordIndex[i + 1]].x,
										  m_lpPoints[m_pCoordIndex[i + 1]].y,
										  GetColor()->color[count],
										  GetWidth(pContext));
				}
			}
		}
		else
		{
			if (GetVertex())
			{
				if (m_nColorLength > 0)
				{
				}
				else
				{
				}
			}
			else
			{
				if (m_nColorLength > 0)
				{
					// Join the lines sequentially
					// And use the colors indicised by the colorIndex array
					for (i = 0; i < m_nLength - 1; i++)
						pVR->DisplayLines(visibleRect, 
										  m_lpPoints[i].x,
										  m_lpPoints[i].y,
										  m_lpPoints[i + 1].x,
										  m_lpPoints[i + 1].y,
										  GetColor()->color[m_pColorIndex[i]],
										  GetWidth(pContext));

					// Now join the last vertex with the first
					pVR->DisplayLines(visibleRect, 
									  m_lpPoints[m_nLength - 1].x,
									  m_lpPoints[m_nLength - 1].y,
									  m_lpPoints[0].x,
									  m_lpPoints[0].y,
									  GetColor()->color[m_pColorIndex[i]],
									  GetWidth(pContext));
				}
				else
				{
					// Join the lines sequentially
					// And use the colors indicised sequentially
					for (i = 0; i < m_nLength - 1; i++)
						pVR->DisplayLines(visibleRect, 
										  m_lpPoints[i].x,
										  m_lpPoints[i].y,
										  m_lpPoints[i + 1].x,
										  m_lpPoints[i + 1].y,
										  GetColor()->color[i],
										  GetWidth(pContext));

					// Now join the last vertex with the first
					pVR->DisplayLines(visibleRect, 
									  m_lpPoints[m_nLength - 1].x,
									  m_lpPoints[m_nLength - 1].y,
									  m_lpPoints[0].x,
									  m_lpPoints[0].y,
									  GetColor()->color[i],
									  GetWidth(pContext));
				}
			}
		}
	}
	// Draw the lines 
	// and use the Material diffuseColor
	else
	{
		Color3f sameColor = GetEmissiveColor(pContext);

		if (m_nCoordLength > 0)
		{
			if (GetVertex())
			{
			}
			else
			{
				// Join the lines according to the coordIndex array
				int count;
				for (i = 0, count = 0; i < m_nCoordLength; i += 3, count++)
					pVR->DisplayLines(visibleRect, 
									  m_lpPoints[m_pCoordIndex[i]].x,
									  m_lpPoints[m_pCoordIndex[i]].y,
									  m_lpPoints[m_pCoordIndex[i + 1]].x,
									  m_lpPoints[m_pCoordIndex[i + 1]].y,
									  sameColor,
									  GetWidth(pContext));
			}
		}
		else
		{
			if (GetVertex())
			{
			}
			else
			{
				// Join the lines sequentially
				// And use the Material diffusColor
				for (i = 0; i < m_nLength - 1; i++)
					pVR->DisplayLines(visibleRect, 
									  m_lpPoints[i].x,
									  m_lpPoints[i].y,
									  m_lpPoints[i + 1].x,
									  m_lpPoints[i + 1].y,
									  sameColor,
									  GetWidth(pContext));

				// Now join the last vertex with the first
				pVR->DisplayLines(visibleRect, 
								  m_lpPoints[m_nLength - 1].x,
								  m_lpPoints[m_nLength - 1].y,
								  m_lpPoints[0].x,
								  m_lpPoints[0].y,
								  sameColor,
								  GetWidth(pContext));
			}
		}
	}
} /* IndexedLineSet2DProxy::Draw */


// ===========================================================================
