//This software module was originally developed by CSELT in the course of   //
//development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.             //
//This software module is an implementation of a part of one or more MPEG-4 //
//Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems        //
//(ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 Systems     //
//(ISO/IEC 14496-1) free license to this software module or modifications   //
//thereof for use in hardware or software products claiming conformance to  //
//the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use this software//
//module in hardware or software products are advised that its use may      //
//infringe existing patents.                                                //
//The original developer of this software module and his/her company, the   //
//subsequent editors and their companies, and ISO/IEC have no liability     //
//for use of this software module or modifications thereof in an            //
//implementation. Copyright is not released for non MPEG-4 Systems          //
//(ISO/IEC 14496-1) conforming products. CSELT S.pA. retains full right     //
//to use the code for his/her own purpose, assign or donate the code to a   //
//third party and to inhibit third parties from using the code for non      //
//MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright      //
//notice must be included in all copies or derivative works.                //
//Copyright  1999.                                                         //
//This work has been partly developed within the framework of the ESPRIT    //
//European Research Project  "MPEG-4 PC".                                   //
//////////////////////////////////////////////////////////////////////////////
//
// File:  IndexedLineSet2DProxy.h
//
// Author: Matthew Leditschke 
//
// Modified: Gianluca Di Cagno  gianluca.dicagno@cselt.it
//
// Description:
//   The definition of the proxy object used by the presenter
//   to render the IndexedLineSet2D node.
//
// Created: September, 1997.
//
//
///////////////////////////////////////////////////////////////////////////


#ifndef INDEXEDLINESET_PROXY_H
#define INDEXEDLINESET_PROXY_H


#include "GeometryNode.h"
#include "VisRend.h"


class IndexedLineSet2DProxy : public GeometryNode
{
public:
	PointF* m_lpPoints;
	PointF* m_shadows;
	int m_nLength;
	int m_nAllocatedLength;

public:
	IndexedLineSet2DProxy(MediaObject* pMO) : GeometryNode(pMO) 
	{
		m_nAllocatedLength = 0;
		m_nPrevColorLength = 0;
		m_nPrevCoordLength = 0;
		m_lpPoints = NULL;
		m_shadows = NULL;
		m_pColorIndex = NULL;
		m_pCoordIndex = NULL;
	}

	~IndexedLineSet2DProxy() 
	{ 
		if (m_lpPoints)
			delete[] m_lpPoints;
		if (m_shadows)
			delete[] m_shadows;
		if (m_pColorIndex)
			delete[] m_pColorIndex; 
		if (m_pCoordIndex)
			delete[] m_pCoordIndex;
	}

	// Return a pointer to the IndexedLineSet2D node that this node is the proxy for
	IndexedLineSet2D* GetIndexLineSet() const { return (IndexedLineSet2D*)GetObject(); }
	BOOL Rotated(DrawableContext *pContext) { return pContext->m_transform.ContainsRotation(); }

	// Return a pointer to the Color object belonging to the
	// node for which this node is the proxy
	Color* GetColor() const
	{ return (Color*)(MediaObject*)(GetIndexLineSet()->color); }

	// Return a pointer to the Properties2D object belonging to the
	// Appearance node that this node is the proxy for.
	Coordinate2D* GetCoordinate2D() const
	{ return (Coordinate2D*)(GetIndexLineSet()->coord); }

	// Return the colorPerVertex value
	BOOL GetVertex() const
	{
		return (BOOL)(Bool)(GetIndexLineSet()->colorPerVertex);
	}

	// Functions inherited from the base class
	virtual void PreRender(Effects & effects);
	virtual void Draw(BRect const* pClipBRect,DrawableContext * pDrawableContext);

	// Private implementation details
private:
	int* m_pColorIndex;
	int  m_nColorLength;
	int  m_nPrevColorLength;

	int* m_pCoordIndex;
	int  m_nCoordLength;
	int  m_nPrevCoordLength;
};

#endif // INDEXEDLINESET_PROXY_H
