//This software module was originally developed by CSELT in the course of   //
//development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.             //
//This software module is an implementation of a part of one or more MPEG-4 //
//Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems        //
//(ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 Systems     //
//(ISO/IEC 14496-1) free license to this software module or modifications   //
//thereof for use in hardware or software products claiming conformance to  //
//the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use this software//
//module in hardware or software products are advised that its use may      //
//infringe existing patents.                                                //
//The original developer of this software module and his/her company, the   //
//subsequent editors and their companies, and ISO/IEC have no liability     //
//for use of this software module or modifications thereof in an            //
//implementation. Copyright is not released for non MPEG-4 Systems          //
//(ISO/IEC 14496-1) conforming products. CSELT S.pA. retains full right     //
//to use the code for his/her own purpose, assign or donate the code to a   //
//third party and to inhibit third parties from using the code for non      //
//MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright      //
//notice must be included in all copies or derivative works.                //
//Copyright  1999.                                                         //
//This work has been partly developed within the framework of the ESPRIT    //
//European Research Project  "MPEG-4 PC".                                   //
//////////////////////////////////////////////////////////////////////////////
//
// File:  Layer2DProxy.cpp
//
// Author:  Gianluca Di Cagno  gianluca.dicagno@cselt.it
//
// 
//
// Description:
//   The implementation of the proxy object used by the presenter
//   to handle the Layer2D node.
//
// Created: March 1999.
//
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include "MediaObj.h"
#include "Presenter.h"
#include "nodes.h"
#include "Layer2DProxy.h"
#include "DrawableNode.h"
#include "trace.h"

// =====================================================================

//
// Presenter::InitLayer2D
//
// Initialise the proxy data structure and attach it to the render data
// pointer of the given media object node.
//

Layer2DProxy* Presenter::InitLayer2D(Layer2D* pObj)
{
	return new  Layer2DProxy(pObj);
}


// =====================================================================

//
// Layer2DProxy::PreRender
//
// PreRender the children in the order specified by the Form
// order field.
// 
//

void Layer2DProxy::PreRender(Effects &effects)
{
	
	 
	 m_nLength= GetObject()->GetChildCount();
	  
	 
	 effects.m_ParentGroup=this;
	 
	 //create the form bounding rect which is referred in the GroupsIndexList as 0 

	 m_drawableContext.GetBoundingRect()->height = ((Vec2f)GetLayer2D()->size).y;
	 m_drawableContext.GetBoundingRect()->width = ((Vec2f)GetLayer2D()->size).x;
	 m_drawableContext.GetBoundingRect()->left=-m_drawableContext.GetBoundingRect()->width/2;
	 m_drawableContext.GetBoundingRect()->top=+m_drawableContext.GetBoundingRect()->height/2;
	 /*if(m_drawableContext.GetBoundingRect()->width!=704)
	 {
		TRACE_PRINT1("LayoutSize.x=%f\n",((Vec2f)GetLayer2D()->size).x);
		TRACE_PRINT1("LayoutSize.y=%f\n",((Vec2f)GetLayer2D()->size).y);
	 }*/
	 effects.transform.Apply(m_drawableContext.GetBoundingRect()->left,m_drawableContext.GetBoundingRect()->top);		
	 AddToDrawableContextList(&m_drawableContext);
	 
	
	 
	 //create children  bounding box
	 int i;
 	 for ( i = 0; i <m_nLength; i++)
	 {
		Traversable * pTrav=GetChild(i);
		if(pTrav)
		 pTrav->PreRender(effects);
	 }

	 	 
	 
	 //finally set the clipping rectangle and Update the current BBox list of each drawable
	 //node, this part should be encapsulated better since can be reused for layout
	 //cicle on m_numberOfContexts since in layers transforms are allowed.
	 for (i=1;i<m_numberOfContexts;i++)
	 	{
	 		if(m_drawableContextList[i])	
			{
				m_drawableContextList[i]->SetClippingRect(m_drawableContextList[0]->GetBoundingRect());
				//if the layer bounding rect is empty don't display the children;
				if(m_drawableContext.GetBoundingRect()->IsEmpty())
					m_drawableContextList[i]->GetBoundingRect()->MakeEmpty();
				((DrawableNode *)m_drawableContextList[i]->GetDrawableProxy())->UpdateBoundingRect(m_drawableContextList[i]);
			}
			

		}
	 
	 m_numberOfContexts=0;
	 m_ContextListSize=0;
	 effects.m_ParentGroup=NULL;
} /* Layer2DProxy::PreRender */
