//This software module was originally developed by CSELT in the course of   //
//development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.             //
//This software module is an implementation of a part of one or more MPEG-4 //
//Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems        //
//(ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 Systems     //
//(ISO/IEC 14496-1) free license to this software module or modifications   //
//thereof for use in hardware or software products claiming conformance to  //
//the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use this software//
//module in hardware or software products are advised that its use may      //
//infringe existing patents.                                                //
//The original developer of this software module and his/her company, the   //
//subsequent editors and their companies, and ISO/IEC have no liability     //
//for use of this software module or modifications thereof in an            //
//implementation. Copyright is not released for non MPEG-4 Systems          //
//(ISO/IEC 14496-1) conforming products. CSELT S.pA. retains full right     //
//to use the code for his/her own purpose, assign or donate the code to a   //
//third party and to inhibit third parties from using the code for non      //
//MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright      //
//notice must be included in all copies or derivative works.                //
//Copyright  1999.                                                         //
//This work has been partly developed within the framework of the ESPRIT    //
//European Research Project  "MPEG-4 PC".                                   //
//////////////////////////////////////////////////////////////////////////////
//
// File:  LayoutProxy.cpp
//
// Author:  Gianluca Di Cagno  gianluca.dicagno@cselt.it
//
// Modified: Frederic Bouilhaguet ENST
//
// Description:
//   The implementation of the proxy object used by the presenter
//   to handle the Layout node.
//
// Created: March 1999.
//
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include "MediaObj.h"
#include "Presenter.h"
#include "nodes.h"
#include "LayoutProxy.h"
#include "DrawableNode.h"
#include "DrawableText.h"

// =====================================================================

//
// Presenter::InitLayout
//
// Initialise the proxy data structure and attach it to the render data
// pointer of the given media object node.
//

LayoutProxy* Presenter::InitLayout(Layout* pObj)
{
	return new  LayoutProxy(pObj);
}

// =====================================================================

//
// LayoutProxy::LayoutProxy
//
// 
//


LayoutProxy::LayoutProxy(MediaObject* pMO): GroupNodeProxy(pMO)
{
	m_nAllocatedLength = 0;
	m_nLength=0;
	m_LastUpdateTime=0xffffffff;
	m_drawableContextList=NULL;
	m_numberOfContexts = m_ContextListSize=0;
	m_previousPos=NULL;
	m_nScrolledOut=0;
	m_AllScrolledOut=FALSE;
	m_maxAscent=0;
	m_maxHeight=0;
	m_startTime=0;
} /* LayoutProxy::LayoutProxy */


// =====================================================================

//
// LayoutProxy::~LayoutProxy
//
//  

LayoutProxy::~LayoutProxy()
{
	delete m_drawableContextList;
	if (m_previousPos)
	  delete [] m_previousPos;
}

const char * LayoutProxy::GetJustify(int i) const
{
	if (i>GetLayout()->justify.GetLength())
		return "";
	else
	return (LPCSTR)(const ZString&)GetLayout()->justify[i];
} /* LayoutProxy:: GetJustify */

// =====================================================================

//
// LayoutProxy::PreRender
//
// PreRender the children in the order specified by the Layout
// order field.
// 
//

void LayoutProxy::PreRender(Effects &effects)
{
	
	 
	 
	  m_nLength= GetObject()->GetChildCount();

	 if (m_nLength > m_nAllocatedLength-1)
	 {
		// Delete the previous arrays - if required
		if (m_previousPos)
			delete [] m_previousPos;
		m_previousPos=new int [m_nLength+1];
		m_nAllocatedLength = m_nLength+1;
	 }
	 effects.m_ParentGroup=this;
	 
	 //create the Layout bounding rect which is referred in the GroupIndexList as 0 
	 CreateLayoutContext(effects);
	 
	 
	 //create children  bounding box
	 int i;
	 for ( i = 0; i <m_nLength; i++)
	 {
		Traversable * pTrav=GetChild(i);
		if(pTrav)
		 pTrav->PreRender(effects);
	 }

	 //do nothing if children do not need rendering
	 if(m_numberOfContexts==1)
	 {
		m_numberOfContexts=0;
		m_ContextListSize=0;
		effects.m_ParentGroup=NULL;
		 return;
	 
	 }
		
	 if(m_startTime==0)
		// #GB: m_startTime=GetPresenter()->GetTime();
    m_startTime = GetFlexTime(); // #GB:

	 
	 

		ApplyFormats();
		ScrollObjects();
	
	 
	
	 
	 
	 //finally set the clipping rectangle and Update the current BBox list of each drawable
	 //node, this part should be encapsulated better since can be reused for Layout
	 for (i=1;i<m_nLength+1;i++)
		{
	 		m_drawableContextList[i]->SetClippingRect(m_drawableContextList[0]->GetBoundingRect());
			((DrawableNode *)m_drawableContextList[i]->GetDrawableProxy())->UpdateBoundingRect(m_drawableContextList[i]);
		}
	 
	 m_numberOfContexts=0;
	 m_ContextListSize=0;
	 effects.m_ParentGroup=NULL;
} /* LayoutProxy::PreRender */


// =====================================================================
//
// LayoutProxy::ApplyFormats
// returns a pointer to the indexes of DrawableContext taken from 
// the Contextset of nodes from the groupIndex field of a Form
// each group is separated by a -1
//	

void LayoutProxy::ApplyFormats()
{
	// now in m_drawableContextList there is the list of objects to 
	
	GetMaxChildrenHeight(m_maxHeight,m_maxAscent,m_maxDescent,m_maxWidth);
	
	BRect *pLayoutBRect = m_drawableContextList[0]->GetBoundingRect();

	m_step = m_maxHeight * GetLayout()->spacing;

	if((Bool)(GetLayout()->horizontal))
		{
			// horizontal aligment
			
			//set left coordinate of the first object looking to the major axis attribute
			
			CString csJustify = GetJustify(0);
			if(csJustify=="FIRST"||csJustify=="BEGIN")
			{
				m_drawableContextList[1]->GetBoundingRect()->left=pLayoutBRect->left;
			
			}
			else if(csJustify=="MIDDLE")
			{
			
				m_drawableContextList[1]->GetBoundingRect()->left=pLayoutBRect->left+
																  (pLayoutBRect->width-
																  m_ChildrenWidth)/2;			
			
			
			}
			else if(csJustify=="END")
			{
			
				m_drawableContextList[1]->GetBoundingRect()->left=pLayoutBRect->left+
																  pLayoutBRect->width-
																  m_ChildrenWidth;		
			
			}             
			
			//set top coordinate of the first object looking to the minor axis attribute
			//set also left and top of the other objects taking into account the space
			//between them
 
			csJustify = GetJustify(1);

			if(csJustify=="FIRST") //baseline alignment
			{	
				
				for (int i=1;i<m_numberOfContexts;i++)
				{
					//if this is a text node get ascent and set the top
					//NB text node font style is not USED, needs spec checking
					if(m_drawableContextList[i]->IsTextNode())
					{
						int ascent,descent;
						((DrawableText *)m_drawableContextList[i]->m_pDrawable)->GetAscentAndDescent(ascent,descent);
						m_drawableContextList[i]->GetBoundingRect()->top= pLayoutBRect->top-m_maxHeight+ascent;

					}
					else
					m_drawableContextList[i]->GetBoundingRect()->top= pLayoutBRect->top-m_maxHeight+m_drawableContextList[i]->GetBoundingRect()->height;
					if(i!=1)
						m_drawableContextList[i]->GetBoundingRect()->left=m_drawableContextList[i-1]->GetBoundingRect()->left+
																		  m_drawableContextList[i-1]->GetBoundingRect()->width+
																		  GetLayout()->spacing;
				}
				// to add for wrap: set size here to max(maxAscent,maxHeight)+maxDescent
				
			}	
			else if(csJustify=="BEGIN")  //top aligment
			{
				
				
				for (int i=1;i<m_numberOfContexts;i++)
				{
					m_drawableContextList[i]->GetBoundingRect()->top= pLayoutBRect->top;
					if(i!=1)
						m_drawableContextList[i]->GetBoundingRect()->left=m_drawableContextList[i-1]->GetBoundingRect()->left+
																		  m_drawableContextList[i-1]->GetBoundingRect()->width+
																		  GetLayout()->spacing;
									
				}
				// to add for wrap: set size here to max(maxAscent+maxDescent,maxHeight)
			
			} 
			else if(csJustify=="MIDDLE") //(top-bottom)/2 aligment
			{
				for (int i=1;i<m_numberOfContexts;i++)
				{
					m_drawableContextList[i]->GetBoundingRect()->top= pLayoutBRect->top-(m_maxHeight-m_drawableContextList[i]->GetBoundingRect()->height)/2;
					if(i!=1)
							m_drawableContextList[i]->GetBoundingRect()->left=m_drawableContextList[i-1]->GetBoundingRect()->left+
																			  m_drawableContextList[i-1]->GetBoundingRect()->width+
																			  GetLayout()->spacing;
						
				}
				// to add for wrap: set size here to max(maxAscent+maxDescent,maxHeight)

			}															 
																		
			
			else if(csJustify=="END") //bottom aligment
			{
				for (int i=1;i<m_numberOfContexts;i++)
				{
					m_drawableContextList[i]->GetBoundingRect()->top= pLayoutBRect->top-m_maxHeight+m_drawableContextList[i]->GetBoundingRect()->height;
					if(i!=1)
						m_drawableContextList[i]->GetBoundingRect()->left=m_drawableContextList[i-1]->GetBoundingRect()->left+
																		  m_drawableContextList[i-1]->GetBoundingRect()->width+
																		  GetLayout()->spacing;
				}
			}

		}
	else
		//Horizontal false, vertical aligment
		//Add topToBottom
		{
			
			CString csJustify = GetJustify(0);
		//NEEDS SPEC CHECKING !!!
			if(csJustify=="FIRST")
				
				m_drawableContextList[1]->GetBoundingRect()->top= pLayoutBRect->top;
																				
				
			else if(csJustify=="BEGIN")
			
				m_drawableContextList[1]->GetBoundingRect()->top= pLayoutBRect->top;
			

			else if(csJustify=="MIDDLE")
			
				m_drawableContextList[1]->GetBoundingRect()->top= pLayoutBRect->top-(pLayoutBRect->height-m_ChildrenHeight)/2;
																		 
																		
			

			else if(csJustify=="END")
			
				m_drawableContextList[1]->GetBoundingRect()->top=pLayoutBRect->top-pLayoutBRect->height+m_ChildrenHeight;
		//NEEDS SPEC CHECKING !!!!
			
			
			csJustify = GetJustify(1);
			
			if(csJustify=="FIRST"||csJustify=="BEGIN")
			{	
				for (int i=1;i<m_numberOfContexts;i++) 
				{
					m_drawableContextList[i]->GetBoundingRect()->left=m_drawableContextList[0]->GetBoundingRect()->left;
					if(i!=1)
						m_drawableContextList[i]->GetBoundingRect()->top=m_drawableContextList[i-1]->GetBoundingRect()->top-
																	 m_drawableContextList[i-1]->GetBoundingRect()->height
																	 -			
																	 m_step;
				}
			}	
			else if(csJustify=="MIDDLE")
			{
				for (int i=1;i<m_numberOfContexts;i++)
				{
					m_drawableContextList[i]->GetBoundingRect()->left=-(m_drawableContextList[i]->GetBoundingRect()->width)/
						2;
					if(i!=1)
						m_drawableContextList[i]->GetBoundingRect()->top=m_drawableContextList[i-1]->GetBoundingRect()->top-
																	 m_drawableContextList[i-1]->GetBoundingRect()->height-			
																	 m_step;
				}
			}
			else if(csJustify=="END")
			{
				for (int i=1;i<m_numberOfContexts;i++)
				{
					m_drawableContextList[i]->GetBoundingRect()->left=
						m_drawableContextList[0]->GetBoundingRect()->left+
						m_drawableContextList[0]->GetBoundingRect()->width-
						m_drawableContextList[i]->GetBoundingRect()->width;
					if(i!=1)
						m_drawableContextList[i]->GetBoundingRect()->top=m_drawableContextList[i-1]->GetBoundingRect()->top-
																	 m_drawableContextList[i-1]->GetBoundingRect()->height-			
																	 m_step;
				}
			}
			
		}			
}     /*LayoutProxy::ApplyFormats*/

void LayoutProxy::ScrollObjects()
{
	
	
	
	BOOL loop =(Bool)GetLayout()->loop;
	BOOL scrollVertical =(Bool)GetLayout()->scrollVertical;
	float scrollRate = GetLayout()->scrollRate;
	BRect *pLayoutBRect=m_drawableContextList[0]->GetBoundingRect();

	DWORD dwUpdateTime;	
	if((dwUpdateTime=GetLayout()->GetUpdateTime())!=m_LastUpdateTime)
	{

		for (int i=1;i<m_numberOfContexts;i++)
		{
			if(scrollVertical)
				 m_previousPos[i]=m_drawableContextList[i]->GetBoundingRect()->top;
			else
				 m_previousPos[i]=m_drawableContextList[i]->GetBoundingRect()->left;

			 m_LastUpdateTime=dwUpdateTime;
		}
	}
    BOOL bScroll=FALSE;
	// #GB: DWORD currentTime=GetPresenter()->GetTime();
  DWORD currentTime = GetFlexTime(); // #GB:
	if((currentTime-m_startTime)>=1000)
	{
		m_startTime=currentTime;
		bScroll=TRUE;
	
	}
	for (int i=1;i<m_numberOfContexts;i++)
			{

				
				BRect * pBRect = m_drawableContextList[i]->GetBoundingRect();
				
				
				
				if(bScroll)
					m_previousPos[i]+=scrollRate;
				
				if(scrollVertical)
					pBRect->top=m_previousPos[i];
				else
					pBRect->left=m_previousPos[i];
				
				if(loop)
					if(!pBRect->Overlaps(pLayoutBRect)&&m_AllScrolledOut)
					{
						if(scrollVertical)
						{
							if(i==1)
								m_previousPos[i]=(pLayoutBRect->top-pLayoutBRect->height);
							else
								m_previousPos[i] = m_previousPos[i-1]-m_drawableContextList[i-1]->GetBoundingRect()->height-m_step;
						
						}
						else
						//	m_previousPos[i]=pLayoutBRect->left- m_drawableContextList[i]->GetBoundingRect()->width;
							m_previousPos[i]=m_previousPos[i]-pLayoutBRect->width-m_maxWidth;//add space here

					}
					else if(!pBRect->Overlaps(pLayoutBRect)&&!m_AllScrolledOut)
					{
						pBRect->MakeEmpty();
						++m_nScrolledOut;
					}
			}		
	if(	m_AllScrolledOut)
	{
			m_AllScrolledOut=FALSE;
			m_nScrolledOut=0;
	}
	if(m_nScrolledOut==m_numberOfContexts-1)
		m_AllScrolledOut=TRUE;
	else m_nScrolledOut=0;

}


// =====================================================================

//
// LayoutProxy::CreateLayoutContext
//
// 
//


void LayoutProxy::CreateLayoutContext(Effects &effects)
{
	//create the Layout drawable context, improve encapsulation
	
	 m_drawableContext.GetBoundingRect()->height = ((Vec2f)GetLayout()->size).y;
	 m_drawableContext.GetBoundingRect()->width = ((Vec2f)GetLayout()->size).x;
	 m_drawableContext.GetBoundingRect()->left=-m_drawableContext.GetBoundingRect()->width/2;
	 m_drawableContext.GetBoundingRect()->top=+m_drawableContext.GetBoundingRect()->height/2;
	 effects.transform.Apply(m_drawableContext.GetBoundingRect()->left,m_drawableContext.GetBoundingRect()->top);		
	 AddToDrawableContextList(&m_drawableContext);
	//
} /* LayoutProxy::CreateLayoutContext */   



void LayoutProxy::GetMaxChildrenHeight(int &maxHeight,int &maxAscent,int &maxDescent,int &maxWidth)
{
	m_ChildrenHeight=maxHeight=m_drawableContextList[1]->GetBoundingRect()->height;
	m_ChildrenWidth=maxWidth=m_drawableContextList[1]->GetBoundingRect()->width;

	if(m_drawableContextList[1]->IsTextNode())
		((DrawableText *)m_drawableContextList[1]->m_pDrawable)->GetAscentAndDescent(maxAscent,maxDescent);
	else
	{
		maxAscent=0;
		maxDescent=0;
	}
				

	for (int i=2;i<m_numberOfContexts;i++)
	{
		if(m_drawableContextList[i]->GetBoundingRect()->height>maxHeight)
			maxHeight=m_drawableContextList[i]->GetBoundingRect()->height;
		
		if(m_drawableContextList[i]->GetBoundingRect()->width>maxWidth)
			maxWidth=m_drawableContextList[i]->GetBoundingRect()->width;
		
		if(m_drawableContextList[i]->IsTextNode())
		{
			int ascent,descent;
			((DrawableText *)m_drawableContextList[i]->m_pDrawable)->GetAscentAndDescent(ascent,descent);
			(ascent>maxAscent)?maxAscent=ascent:1;
			(descent>maxDescent)?maxDescent=descent:1;
		}
		m_ChildrenWidth+=m_drawableContextList[i]->GetBoundingRect()->width;
		m_ChildrenHeight+=m_drawableContextList[i]->GetBoundingRect()->height;
	}
	//add interobjects spacing
	m_ChildrenWidth+=(m_numberOfContexts-2)*GetLayout()->spacing;
	m_ChildrenHeight+=(m_numberOfContexts-2)*GetLayout()->spacing;
}
