/****************************************************************************/
/*                                                            				*/
/* Copyright (c) 1997 ISO/IEC All Rights Reserved                			*/
/*                                                            				*/
/* This software module was developed by									*/
/*																			*/
/* CSELT  Centro Studi e Laboratori Telecomunicazioni                       */
/*                                                            				*/
/* in the course of development of the MPEG-4 standard. This software       */
/* module is an implementation of a part of one or more MPEG-4 tools as     */
/* specified by the MPEG-4 standard.             							*/
/*                                                            				*/
/* The copyright of this software belongs to ISO/IEC. ISO/IEC gives use     */
/* of the MPEG-4 standard free license to use this  software module or      */
/* modifications thereof for hardware or software products claiming         */
/* conformance to the MPEG-4 standard        								*/
/*                                                            				*/
/* Those intending to use this software module in hardware or software      */
/* products are advised that use may infringe existing  patents. The        */
/* original developers of this software module and their companies, the     */
/* subsequent editors and their companies, and ISO/IEC have no liability    */
/* and ISO/IEC have no liability for use of this software module or         */
/* modification thereof in an implementation.                               */
/*                                                            				*/
/****************************************************************************/

/****************************************************************************
 *
 * File:        MPEG4Dec.cpp
 *
 * Author:      Franco Casalino, Antonio Arena
 * Created:     04.Sept.97
 *
 * Description: MPEG-4 Decoder class definition
 * 
 * Comments: Work derived from the MoMuSys Group's software
 *
 ****************************************************************************/

#include "defs.h"
#include "MPEG4Dec.h"
#include <assert.h>


// Define this to write out the input bitstream to a file
//#define WRITE_INPUT

#ifdef WRITE_INPUT
static FILE* outfile = NULL;
#endif


MPEG4Decoder :: MPEG4Decoder (class Executive *pExecutive) : Decoder (pExecutive)
{
	m_decoderHandle = NULL;
	m_firstCall = TRUE;
	m_lastReturnValue = TRUE;

#ifdef WRITE_INPUT
	outfile = fopen("mpeg4decIn.mp4v","wb");
	assert(outfile != NULL);
#endif
}


MPEG4Decoder :: ~MPEG4Decoder()
{
	if (m_decoderHandle != NULL)
	{
		Mpeg4DecoderShutdown(m_decoderHandle);
		Mpeg4DecoderDeallocFrame(&m_decoderFrame);
	}
#ifdef WRITE_INPUT
	fclose(outfile);
#endif
}

//static int frameNum = 0;


BOOL MPEG4Decoder :: Decode(LPBYTE pInput,  int nInputLength,
							LPBYTE pOutput, int &rnOutputLength)
{
	// Initialise the decoder only the first time this is called
	if (m_firstCall)
	{
		m_decoderHandle = Mpeg4DecoderInit(pInput, nInputLength);
		assert(m_decoderHandle != NULL);
		Mpeg4DecoderInitFrame (&m_decoderFrame);
	}

	// Otherwise only reset the bitstream if the last call was
	// successful
	else if (m_lastReturnValue == TRUE)
		resetBitstream(m_decoderHandle, pInput, nInputLength);

	// Read in the header of the next picture VOP, returning its
	// dimensions and offset.  Only do this if the last call
	// was successful.
	int prematureEOF;
	if (m_lastReturnValue == TRUE)
	{
		prematureEOF = Mpeg4DecoderDecodeHeader(m_decoderHandle, &m_decoderFrame);
		assert(prematureEOF == 0);
	}
	int width  = m_decoderFrame.activeWidth;
	int height = m_decoderFrame.activeHeight;

	// If the given output buffer will not be large enough to hold the
	// frame being decoded, then return FALSE and set the required size.
	if (width * height * 5 / 2 + (int)sizeof(DecoderImage) > rnOutputLength)
	{
		rnOutputLength = width * height * 5 / 2 + sizeof(DecoderImage);
		m_lastReturnValue = FALSE;
		return m_lastReturnValue;
	}

#ifdef WRITE_INPUT
	fwrite(pInput, nInputLength, 1, outfile);
#endif

	// Set up the pointers for the output frame
	unsigned char* decodedY = pOutput + sizeof(DecoderImage);
	unsigned char* decodedU = decodedY + width * height;
	unsigned char* decodedV = decodedU + width * height / 4;
	unsigned char* decodedA = decodedV + width * height / 4;

	// Decode the body of the frame
	prematureEOF = Mpeg4DecoderDecodeBody(m_decoderHandle, decodedY,
										  decodedU, decodedV, decodedA,
										  width, height);
	assert(prematureEOF == 0);

/*
	frameNum++;
	char name[50];
	sprintf(name, "frame%03d-%dx%d.yuv", frameNum,
			m_decoderFrame.activeWidth, m_decoderFrame.activeHeight);
	FILE* tmpfile = fopen(name,"wb");
	assert(tmpfile != NULL);
	fwrite(decodedY,
		   m_decoderFrame.activeWidth * m_decoderFrame.activeHeight * 3 / 2,
		   1, tmpfile);
	fclose(tmpfile);
*/

	// Store the dimensions of the image
	DecoderImage* pImgDetails = (DecoderImage*)pOutput;
	pImgDetails->width   = m_decoderFrame.activeWidth;
	pImgDetails->height  = m_decoderFrame.activeHeight;
	pImgDetails->pitch   = m_decoderFrame.activeWidth;
	pImgDetails->offsetX = m_decoderFrame.offsetX;
	pImgDetails->offsetY = m_decoderFrame.offsetY;

	// Determine how much data was written to the buffer
	rnOutputLength = width * height * 5 / 2 + sizeof(DecoderImage);

	m_firstCall = FALSE;
	m_lastReturnValue = TRUE;
	return m_lastReturnValue;
}

FORMAT MPEG4Decoder :: SetFormat(Capabilities * pPresentationCapabilities)
{
	FORMAT format;
	for (int i = 0; i < pPresentationCapabilities->GetFormatCapabilityLength(); i++) {
		format = pPresentationCapabilities->GetFormatCapability(i);
		if (format == FORMAT_YUVA)
			return FORMAT_YUVA;
	}
	return FORMAT_UNKNOWN;
}


int MPEG4Decoder :: GetOptimalInputSize() { 
	return (5000); 
}


int MPEG4Decoder :: GetOptimalOutputSize() { 
	return (176 * 144 * 5 / 2 + sizeof(DecoderImage)); 
}


int MPEG4Decoder :: GetRecommendedInputBufferSize() { 
	return 3 * GetOptimalInputSize(); 
}


int MPEG4Decoder :: GetRecommendedOutputBufferSize() { 
	return 3 * GetOptimalOutputSize(); 
}

static DecoderFactory<MPEG4Decoder> Mpg4Factory (5);

