// MPEG4Player.cpp : Defines the class behaviors for the application.
//

#include "defs.h"
#include <assert.h>
#include "Clock.h"
#include "Presenter.h"
#include "trace.h"
#include "version.h"
#include "MPEG4Player.h"
#include "MainFrm.h"
#include "MPEG4PlayerDoc.h"
#include "MPEG4PlayerView.h"
#include "WarningMessage.h"
#include "ThreadInfoDlg.h"
#include "ThreadList.h"
#include "ViewWorldInfo.h"
#include "root.h"
#include <crtdbg.h>
#include "PlayListImpl.h"

extern FILE* traceFile;

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CMPEG4PlayerApp

BEGIN_MESSAGE_MAP(CMPEG4PlayerApp, CWinApp)
	//{{AFX_MSG_MAP(CMPEG4PlayerApp)
	ON_COMMAND(ID_APP_ABOUT, OnAppAbout)
	ON_COMMAND(ID_OPTIONS_25FPS, OnOptions25fps)
	ON_UPDATE_COMMAND_UI(ID_OPTIONS_25FPS, OnUpdateOptions25fps)
	ON_COMMAND(ID_OPTIONS_15FPS, OnOptions15fps)
	ON_UPDATE_COMMAND_UI(ID_OPTIONS_15FPS, OnUpdateOptions15fps)
	ON_COMMAND(ID_OPTIONS_10FPS, OnOptions10fps)
	ON_UPDATE_COMMAND_UI(ID_OPTIONS_10FPS, OnUpdateOptions10fps)
	ON_COMMAND(ID_OPTIONS_5FPS, OnOptions5fps)
	ON_UPDATE_COMMAND_UI(ID_OPTIONS_5FPS, OnUpdateOptions5fps)
	ON_COMMAND(ID_OPTIONS_LOOP_CONTINUOUSLY, OnOptionsLoopContinuously)
	ON_UPDATE_COMMAND_UI(ID_OPTIONS_LOOP_CONTINUOUSLY, OnUpdateOptionsLoopContinuously)
	ON_COMMAND(ID_VIEW_THREAD_INFO, OnViewThreadInfo)
	ON_COMMAND(ID_VIEW_WORLD_INFO, OnViewWorldInfo)
	ON_COMMAND(ID_OPTIONS_CLEAR_ON_END, OnOptionsClearOnEnd)
	ON_UPDATE_COMMAND_UI(ID_OPTIONS_CLEAR_ON_END, OnUpdateOptionsClearOnEnd)
	ON_COMMAND(ID_FILE_OPEN, OnFileOpen)
	ON_COMMAND(ID_GO_BACK, OnGoBack)
	ON_COMMAND(ID_GO_FORWARD, OnGoForward)
	ON_UPDATE_COMMAND_UI(ID_GO_BACK, OnUpdateGoBack)
	ON_UPDATE_COMMAND_UI(ID_GO_FORWARD, OnUpdateGoForward)
	ON_COMMAND(ID_GO_REPLAY, OnGoReplay)
	ON_UPDATE_COMMAND_UI(ID_GO_REPLAY, OnUpdateGoReplay)
	ON_COMMAND(ID_FILE_OPENFILE, OnFileOpenfile)
	//}}AFX_MSG_MAP
	// Standard file based document commands
	ON_COMMAND(ID_FILE_NEW, CWinApp::OnFileNew)
	ON_COMMAND(ID_FILE_OPEN, CWinApp::OnFileOpen)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMPEG4PlayerApp construction

CMPEG4PlayerApp::CMPEG4PlayerApp()
{
	m_pExecutive = NULL;
}

/////////////////////////////////////////////////////////////////////////////
// The one and only CMPEG4PlayerApp object

CMPEG4PlayerApp theApp;

/////////////////////////////////////////////////////////////////////////////
// CMPEG4PlayerApp initialization

BOOL CMPEG4PlayerApp::InitInstance()
{
	// Standard initialization
	// If you are not using these features and wish to reduce the size
	//  of your final executable, you should remove from the following
	//  the specific initialization routines you do not need.
	AfxEnableControlContainer();
	
	//**Set a Breakpoint on a specific memory allocation**//
	
//	_CrtSetBreakAlloc(14216);

#ifdef _AFXDLL
	Enable3dControls();			// Call this when using MFC in a shared DLL
#else
	Enable3dControlsStatic();	// Call this when linking to MFC statically
#endif

	LoadStdProfileSettings(9);  // Load standard INI file options (including MRU)

	// Open the logfile (also send stdout to this logfile)
#ifdef WRITE_TRACE
	traceFile = fopen ("trace.log", "w");
//	traceFile = freopen ("trace.log", "w", stdout);
	assert(traceFile != NULL);
	TRACE_PRINT("Starting IM1Player FDIS:\n\n");
#endif

	AddToThreadList("FrameApp", (int)GetCurrentThread());

	// Read the flags which are stored in the application's .INI file
	m_loopContinuously = AfxGetApp()->GetProfileInt("Options", "LoopContinuously", FALSE);
	m_clearOnEnd = AfxGetApp()->GetProfileInt("Options", "ClearOnEnd", FALSE);

	// Register the application's document templates.  Document templates
	//  serve as the connection between documents, frame windows and views.

	CSingleDocTemplate* pDocTemplate;
	pDocTemplate = new CSingleDocTemplate(
		IDR_MAINFRAME,
		RUNTIME_CLASS(CMPEG4PlayerDoc),
		RUNTIME_CLASS(CMainFrame),       // main SDI frame window
		RUNTIME_CLASS(CMPEG4PlayerView));
	AddDocTemplate(pDocTemplate);

	// Enable DDE Execute open
	EnableShellOpen();
	RegisterShellFileTypes(TRUE);

	// Parse command line for standard shell commands, DDE, file open
	CCommandLineInfo cmdInfo;
	ParseCommandLine(cmdInfo);

	// Dispatch commands specified on the command line
	if (!ProcessShellCommand(cmdInfo))
		return FALSE;

	// Enable drag/drop open
	m_pMainWnd->DragAcceptFiles();

	// Load the list of network addresses from the INI file
	GetProfileNetworkAddresses();

	// Set the default size of the window
	((CMainFrame*)m_pMainWnd)->SetSize(640,480);

	// create and initialize the playlist
	m_playList = new PlayListImpl();

	GetView()->SetFocus(); // the focus must be on the main window
// GUIDO : check all allocations/deallocations
//	afxMemDF |= checkAlwaysMemDF ;

	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// CAboutDlg dialog used for App About

class CAboutDlg : public CDialog
{
public:
	CAboutDlg();

// Dialog Data
	//{{AFX_DATA(CAboutDlg)
	enum { IDD = IDD_ABOUTBOX };
	CString m_version;
	CString m_date;
	//}}AFX_DATA

	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CAboutDlg)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	//}}AFX_VIRTUAL

// Implementation
protected:
	//{{AFX_MSG(CAboutDlg)
		// No message handlers
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
{
	//{{AFX_DATA_INIT(CAboutDlg)
	//}}AFX_DATA_INIT
}

void CAboutDlg::DoDataExchange(CDataExchange*)
{
/*	m_version = VERSION_NUMBER
#ifdef WRITE_TRACE
		" (log)"
#else
		""
#endif
#ifdef _DEBUG
		" (debug)";
#else
		"";
#endif

	m_date = VERSION_DATE;
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAboutDlg)
	DDX_Text(pDX, IDC_VERSION, m_version);
	DDX_Text(pDX, IDC_DATE, m_date);
	//}}AFX_DATA_MAP*/
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
	//{{AFX_MSG_MAP(CAboutDlg)
		// No message handlers
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

// App command to run the dialog
void CMPEG4PlayerApp::OnAppAbout()
{
	CAboutDlg aboutDlg;
	aboutDlg.DoModal();
}

/////////////////////////////////////////////////////////////////////////////

//
// CMPEG4PlayerApp::CreateExecutive
//
// Called when the view is first initialised.
//

void CMPEG4PlayerApp::CreateExecutive(HWND hWnd)
{
	// Create the executive object
	m_pExecutive = new Executive(this, hWnd,traceFile,TRACE_PARSE, &m_deviceFilter);
	
	
	// Set the initial frame rate
	Presenter* pPres = (Presenter *) GetExecutive()->GetPresenter();

	if(pPres==NULL)
	{
		AfxMessageBox("Create Presenter Failed, check registry");
		delete m_pExecutive;
		m_pExecutive=NULL;
		return;
	}

	float frameRate = GetApp()->GetProfileFloat("Options", "FrameRate", 15.0);
	pPres->SetTargetFrameRate(frameRate);
	

	// Tell the visual renderer if it is to clear the screen when
	// the end of a file is reached
	pPres->GetVisualRenderer()->SetClearOnEndFlag(m_clearOnEnd);

	// If the presenter couldn't be initialise correctly, exit
	if (!((Presenter*)m_pExecutive->GetPresenter())->InitialisedOK())
		::PostQuitMessage(-1);

	// CC 010124
	m_deviceFilter.SetVisualRenderer(pPres->GetVisualRenderer());
	m_keyboardInstance.SetDeviceFilter(&m_deviceFilter);
	m_Web3DKeySensorInstance.SetDeviceFilter(&m_deviceFilter);
	m_mouseInstance.SetDeviceFilter(&m_deviceFilter);
	m_deviceFilter.DevicePlugin_Attach_Req(&m_keyboardInstance);
	m_deviceFilter.DevicePlugin_Attach_Req(&m_mouseInstance);
	m_deviceFilter.DevicePlugin_Attach_Req(&m_Web3DKeySensorInstance);

	
	
	// If the file has not yet been opened, as is the case if the file
	// was opened by double clicking on it, the file is now opened.
	if (GetDocument()->DocumentNeedsOpening())
		GetDocument()->OpenDocument();
	

	

} /* CMPEG4PlayerApp::CreateExecutive */


/////////////////////////////////////////////////////////////////////////////

void CALLBACK EXPORT TimerProc(HWND /*hWnd*/, UINT /*nMsg*/,
							   UINT /*nIDEvent*/, DWORD /*dwTime*/)
{
	Presenter* pPres = (Presenter *) GetApp()->GetExecutive()->GetPresenter();

	if(!pPres->IsActive())
	{
		GetStatusBar()->SetWindowText("Loading..");
		return;
	}
	int ms = timeGetTime() - pPres->GetStartTime();
	float fps = pPres->GetActualFrameRate();

	// Format the time as MM.SS.HH
	char text[30];
	sprintf(text, "%02d:%02d.%02d  %.1ffps",
			((int)(ms / 1000)) / 60,
			((int)(ms / 1000)) % 60,
			(ms % 1000) / 10,
			fps);

	// Update the pane in the status bar
	GetStatusBar()->SetWindowText(text);
} /* TimerProc */

/////////////////////////////////////////////////////////////////////////////

//
// CMPEG4PlayerApp::OnConnect
//
// This is called by the Executive when a session has been established,
// indicating if it was established successfully or not.
//
// If successful, start the timer running to update the clock.
// If not, display a warning message.
//

void CMPEG4PlayerApp::OnConnect(RESPONSE result)
{
	if (result == RESPONSE_OK)
	{
		TRACE_PRINT("Session successfully started.\n");
		
	
		// Start a timer to update the running time display
		m_timerID = GetMainFrame()->SetTimer(1, 250, TimerProc);

		// Record that this file has been successfully opened.
		GetDocument()->m_fileOpened = TRUE;
		m_smpFileOpen.P();
	}

	else
	{
		
	//	OnError(result);
		
		/*// Prepare a message to give to the user as to why the connection
		// was not successful.
		char message[200];
		strcpy(message, "The session could not be established because ");
		if (result == FILE_NOT_FOUND)
			strcat(message, "the specifed file could not be found.");
		else if (result == CANNOT_OPEN_FILE)
			strcat(message, "the specifed file could not be opened.");
		else if (result == IP_ADDRESS_NOT_FOUND)
			strcat(message, "the specifed IP address could not be found.");
		else if (result == CANNOT_OPEN_SOCKET)
			strcat(message, "a socket could not be opened.");
		else if (result == MEMORY_PROBLEM)
			strcat(message, "of memory problems.");
		else if (result == UNEXPECTED_EOF)
			strcat(message, "an unexpected end of file was encountered.");
		else
			strcat(message, "an unknown error occurred.");

		CWarningMessage warningMessageDialog;
		warningMessageDialog.m_message = message;
		warningMessageDialog.DoModal();*/
	}
} /* CMPEG4PlayerApp::OnConnect */


/////////////////////////////////////////////////////////////////////////////

//
// CMPEG4PlayerApp::OnFinish
//
// This is called by the Executive when the multiplexer has finished
// reading streams.  This doesn't do anything, as the scene is assumed
// to be finished when the presenter has finished.
//

void CMPEG4PlayerApp::OnFinish(RESPONSE reason)
{
	TRACE_PRINT("End of session notification received.\n");

	// Remember the reason for finishing
	m_reason = reason;
}

void CMPEG4PlayerApp::OnError(NON_FATAL_ERROR nERROR, va_list /*args*/)
{
   
#ifndef NETSCAPE_PLUGIN
//		GetView() -> PostMessage(WM_NON_FATAL_ERROR, (WPARAM)nERROR,(LPARAM)0);
		CMPEG4PlayerView *pView = GetView();
		assert(pView);
		if (pView == NULL)
			return;
		pView -> PostMessage(WM_NON_FATAL_ERROR, (WPARAM)nERROR,(LPARAM)0);
#else 
		::PostMessage(((Presenter *)GetExecutive()->GetPresenter())->GetWindow(),
						WM_NON_FATAL_ERROR,
						(WPARAM)nERROR,
						(LPARAM)0);
#endif
}


//
// CMPEG4PlayerApp::OnSceneFinished
//
// This method is called when the presenter has indicated (by posting
// a message) that it has finished rendering the scene.  When this
// occurs, the executive is stopped, which completes the termination
// of the session.
//
// If the session terminated because of the end of the service, and
// looping enabled, then the file will be restarted.
//

void CMPEG4PlayerApp::OnSceneFinished()
{
	TRACE_PRINT("End of scene notification received.\n");

	// Stop the Executive running.
//	m_pExecutive->Stop();

	// Flag that the file is now closed
	m_smpFileOpen.V();

	// Stop the timer which is updating the display of the running time.
//	GetMainFrame()->KillTimer(m_timerID);
	if(GetMainFrame()!=NULL)
		GetMainFrame()->KillTimer(m_timerID);

	// Close the document.
//	GetDocument()->CloseDocument();

	// If the Loop Continuously option is selected,
	// send a message to the main thread to restart the file.
	// (this is done instead of calling OnFileRestart() directly,
	// because they are in different threads.
/*	if ((m_reason == END_OF_SERVICE) && (m_loopContinuously))
		GetMainFrame()->PostMessage(WM_COMMAND, ID_FILE_RESTART);
	else
		// Indicate on the status bar that the sequence has finished
	
		GetStatusBar()->SetWindowText("Finished.");*/
	//TO be viewed

} /* CMPEG4PlayerApp::OnSceneFinished */


/////////////////////////////////////////////////////////////////////////////
// CMPEG4PlayerApp commands

int CMPEG4PlayerApp::ExitInstance() 
{
	TRACE_PRINT("Entering CMPEG4PlayerApp::ExitInstance()\n");

	// Write the current frame rate to the .INI file
	if (GetExecutive() != NULL)
		GetApp()->WriteProfileFloat("Options", "FrameRate",
			((Presenter*)GetExecutive()->GetPresenter())->GetTargetFrameRate());

	// and also write the list of network addresses
	WriteProfileNetworkAddresses();

	// Delete the executive object (it has aready been stopped
	// by CloseDocument()).
	if (m_pExecutive)
		delete m_pExecutive;

	// Delete the thread info dialog box - if required
	if (pThreadInfoDlg != NULL)
		delete pThreadInfoDlg;
	EmptyThreadList();

	// Empty the list of entered network addresses
	for (int i = 0; i < m_networkAddresses.GetLength(); i++)
		free(m_networkAddresses[i]);

#ifdef SIMULATE_BIFS_DECODER
	extern char* globalFileName;

	if (globalFileName != NULL)
		free(globalFileName);
#endif

	delete m_playList;

	TRACE_PRINT("Leaving CMPEG4PlayerApp::ExitInstance()\n");
	_CrtCheckMemory();
	return CWinApp::ExitInstance();
} /* CMPEG4PlayerApp::ExitInstance */


/////////////////////////////////////////////////////////////////////////////

//
// GetDocument
//
// Return a pointer to the current document.
//

CMPEG4PlayerDoc* GetDocument()
{
	// Get the first document template
	POSITION pos = AfxGetApp()->GetFirstDocTemplatePosition();
	CDocTemplate* pDocTemplate = AfxGetApp()->GetNextDocTemplate(pos);

	// Get the first document in the template
	pos = pDocTemplate->GetFirstDocPosition();
	CDocument* pDoc = pDocTemplate->GetNextDoc(pos);

	// Return this document
	return (CMPEG4PlayerDoc*)pDoc;
} /* GetDocument */

//
// GetView
//
// Return a pointer to the current view.
//

CMPEG4PlayerView* GetView()
{
 CMPEG4PlayerDoc * pDoc = GetDocument(); 
 assert(pDoc);
 if (pDoc == NULL)
	 return NULL;
 POSITION pos  = pDoc-> GetFirstViewPosition();

 return (CMPEG4PlayerView *)pDoc-> GetNextView(pos);
} /* GetView */


/////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////

//
// These functions handle the changes to the frame rate, and the
// display of the tick marks which show which frame rate is current.
//

void CMPEG4PlayerApp::OnOptions25fps() 
{
	((Presenter*)GetExecutive()->GetPresenter())->SetTargetFrameRate(25);
}

void CMPEG4PlayerApp::OnUpdateOptions25fps(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck((int)((Presenter*)GetExecutive()->GetPresenter())->GetTargetFrameRate() == 25);
}

void CMPEG4PlayerApp::OnOptions15fps() 
{
	((Presenter*)GetExecutive()->GetPresenter())->SetTargetFrameRate(15);
}

void CMPEG4PlayerApp::OnUpdateOptions15fps(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck((int)((Presenter*)GetExecutive()->GetPresenter())->GetTargetFrameRate() == 15);
}

void CMPEG4PlayerApp::OnOptions10fps() 
{
	((Presenter*)GetExecutive()->GetPresenter())->SetTargetFrameRate(10);
}

void CMPEG4PlayerApp::OnUpdateOptions10fps(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck((int)((Presenter*)GetExecutive()->GetPresenter())->GetTargetFrameRate() == 10);
}

void CMPEG4PlayerApp::OnOptions5fps() 
{
((Presenter*)GetExecutive()->GetPresenter())->SetTargetFrameRate(5);
}

void CMPEG4PlayerApp::OnUpdateOptions5fps(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck((int)((Presenter*)GetExecutive()->GetPresenter())->GetTargetFrameRate() == 5);
}


/////////////////////////////////////////////////////////////////////////////

//
// These functions set and show the state of the m_loopContinuously flag.
//

void CMPEG4PlayerApp::OnOptionsLoopContinuously() 
{
	// Toggle the value
	m_loopContinuously = (m_loopContinuously) ? FALSE : TRUE;

	// Update this change in the .INI file
	AfxGetApp()->WriteProfileInt("Options", "LoopContinuously", m_loopContinuously);
}

void CMPEG4PlayerApp::OnUpdateOptionsLoopContinuously(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(m_loopContinuously);
}


/////////////////////////////////////////////////////////////////////////////

//
// These functions set and show the state of the m_clearOnEnd flag.
//

void CMPEG4PlayerApp::OnOptionsClearOnEnd() 
{
	// Toggle the value
	m_clearOnEnd = (m_clearOnEnd) ? FALSE : TRUE;

	// Update this change in the .INI file
	AfxGetApp()->WriteProfileInt("Options", "ClearOnEnd", m_clearOnEnd);
    
	// Tell the visual renderer about this change
	Presenter* pPres = (Presenter *) GetExecutive()->GetPresenter();
	pPres->GetVisualRenderer()->SetClearOnEndFlag(m_clearOnEnd);
}

void CMPEG4PlayerApp::OnUpdateOptionsClearOnEnd(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(m_clearOnEnd);
}


/////////////////////////////////////////////////////////////////////////////

float CMPEG4PlayerApp::GetProfileFloat(LPCTSTR lpszSection,
									   LPCTSTR lpszEntry,
									   float fDefault)
{
	CString str = GetProfileString(lpszSection, lpszEntry);
	if (str.IsEmpty())
		return fDefault;
	else
		return atof(str);
} /* CMPEG4PlayerApp::GetProfileFloat */


/////////////////////////////////////////////////////////////////////////////

BOOL CMPEG4PlayerApp::WriteProfileFloat(LPCTSTR lpszSection,
										LPCTSTR lpszEntry,
										float value)
{
	char str[20];
	sprintf(str, "%.4f", value);
	return WriteProfileString(lpszSection, lpszEntry, str);
} /* CMPEG4PlayerApp::WriteProfileFloat */


/////////////////////////////////////////////////////////////////////////////

void CMPEG4PlayerApp::GetProfileNetworkAddresses()
{
	CString str = GetProfileString("State", "NetworkAddresses");
	if (!str.IsEmpty())
	{
		char* localStr = strdup((LPCTSTR)str);
		char* pStr = localStr;

		// Break the string into separate string, using the commas
		// as separators
		while (pStr != NULL)
		{
			// NULL terminate the substring (if required)
			char* pEnd = strchr(pStr, ',');
			if (pEnd != NULL)
				*pEnd = 0;

			// Make a copy of the substring
			m_networkAddresses += strdup(pStr);

			// Move onto the next string
			if (pEnd != NULL)
				pStr = pEnd + 1;
			else
				pStr = pEnd;
		}
       
		free(localStr);
	}
} /* CMPEG4PlayerApp::GetProfileNetworkAddresses */


/////////////////////////////////////////////////////////////////////////////

void CMPEG4PlayerApp::WriteProfileNetworkAddresses()
{
	// If there are no network addresses, write an empty string
	if (m_networkAddresses.GetLength() == 0)
		WriteProfileString("State", "NetworkAddresses", "");

	// Otherwise, combine all of the network addresses into a single,
	// comma separated list in order to store it in the application's
	// profile.
	else
	{
		int requiredLength = 0;
		for (int i = 0; i < m_networkAddresses.GetLength(); i++)
			requiredLength += strlen(m_networkAddresses[i]) + 1;

		char* combinedString = new char[requiredLength];
		strcpy(combinedString, m_networkAddresses[0]);
		for (i = 1; i < m_networkAddresses.GetLength(); i++)
		{
			strcat(combinedString, ",");
			strcat(combinedString, m_networkAddresses[i]);
		}

		WriteProfileString("State", "NetworkAddresses", combinedString);
		delete combinedString;
	}
} /* CMPEG4PlayerApp::GetProfileNetworkAddresses */


/////////////////////////////////////////////////////////////////////////////

//
// CMPEG4PlayerApp::OnViewThreadInfo
//
// Show information about the threads that are running.
//

void CMPEG4PlayerApp::OnViewThreadInfo() 
{
	if (pThreadInfoDlg == NULL)
		pThreadInfoDlg = new CThreadInfoDlg;
	else
		pThreadInfoDlg->ShowWindow(SW_SHOW);
} /* CMPEG4PlayerApp::OnViewThreadInfo */


/////////////////////////////////////////////////////////////////////////////

void CMPEG4PlayerApp::OnViewWorldInfo() 
{
	CViewWorldInfo dialog;
	dialog.DoModal();
} /* CMPEG4PlayerApp::OnViewWorldInfo */


/////////////////////////////////////////////////////////////////////////////


void CMPEG4PlayerApp::OnFileOpen() 
{
	m_playList->showDialog();
}

void CMPEG4PlayerApp::OnGoBack() 
{
	// TODO: Add your command handler code here
	// Open the previous file of the playlist if possible
	if (m_playList->hasPreviousTrack())
	{
		GetView()->GetDocument()->CloseDocument();
		OpenDocumentFile(m_playList->getPreviousTrack() ); 	
	}
}

void CMPEG4PlayerApp::OnGoForward() 
{
	// TODO: Add your command handler code here
	// Open the next file of the playlist if possible
	if (m_playList->hasNextTrack())
	{
		GetView()->GetDocument()->CloseDocument();
		OpenDocumentFile(m_playList->getNextTrack() ); 
	}
	
}

void CMPEG4PlayerApp::OnUpdateGoBack(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	if (m_playList && m_playList->hasPreviousTrack())
		pCmdUI->Enable(true);
	else
		pCmdUI->Enable(false);
}

void CMPEG4PlayerApp::OnUpdateGoForward(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	if (m_playList && m_playList->hasNextTrack())
		pCmdUI->Enable(true);
	else
		pCmdUI->Enable(false);
	
}

void CMPEG4PlayerApp::OnGoReplay() 
{
	// TODO: Add your command handler code here
	if ( *(m_playList->getCurrentTrack()) != '\0') // playlist not empty
	{
		//CloseAllDocuments()?
		GetView()->GetDocument()->CloseDocument();
		OpenDocumentFile(m_playList->getCurrentTrack());
		GetView()->GetDocument()->CloseDocument();
		GetView()->GetDocument()->OpenDocument();
	}
}

void CMPEG4PlayerApp::OnUpdateGoReplay(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	if (m_playList && *(m_playList->getCurrentTrack()) != '\0')
		pCmdUI->Enable(true);
	else
		pCmdUI->Enable(false);
	
}

void CMPEG4PlayerApp::OnFileOpenfile() 
{
	// TODO: Add your command handler code here
	char BASED_CODE szFilter[] = "Mpeg4 file formats (*.mp4;*.trif)|*.mp4;*.trif|All files (*.*)|*.*||";
	CFileDialog fd(TRUE,NULL,NULL,OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT,szFilter);

	fd.DoModal();
	OpenDocumentFile(fd.GetPathName( ) ); 
}
