// MPEG4Player.h : main header file for the MPEG4PLAYER application
//

#ifndef __AFXWIN_H__
	#error include 'defs.h' before including this file for PCH
#endif

#include "resource.h"       // main symbols
#include "PlaylistImpl.h"
#include "App.h"
#include "Executiv.h"
#include "../UI/UIDevice.h"
#include <webbrowser2.h> 

class CThreadInfoDlg;


// User defined messages
#define WM_SCENE_FINISHED (WM_USER + 1)
#define WM_DDRAW_PROBLEM  (WM_USER + 2)
#define WM_WEB_BASE WM_USER + 100

#define WM_WEB_CREATE   (WM_WEB_BASE + 1)
#define WM_WEB_SHOW     (WM_WEB_BASE + 2)
#define WM_WEB_HIDE     (WM_WEB_BASE + 3)
#define WM_WEB_NAVIGATE (WM_WEB_BASE + 4)
#define WM_WEB_REDRAW   (WM_WEB_BASE + 5)
#define WM_NAVIGATE (WM_WEB_BASE + 6)
#define WM_SETSIZE (WM_WEB_BASE + 6)
#define WM_NON_FATAL_ERROR (WM_USER + 10)

/////////////////////////////////////////////////////////////////////////////
// CMPEG4PlayerApp:
// See MPEG4Player.cpp for the implementation of this class
//

class CMPEG4PlayerApp : public CWinApp, public Application
{
public:
	CMPEG4PlayerApp();

	// Implementation of a playlist
	PlayListImpl *m_playList;
	
	// Functions defined in the Application class
	void OnConnect(RESPONSE result);
	void OnFinish(RESPONSE reason);

	// Handles the message generated when the presenter has
	// finished rendering the scene.
	void OnSceneFinished();

	void OnError(NON_FATAL_ERROR nError, va_list);
	
	//not used in IM1-2D
	void OnFatalError (FATAL_ERROR, LPCSTR ) {}
	int Navigate(LPCSTR ) {return 1;}
	void OverSensor(unsigned int ) {}
	void SetSize(unsigned int , unsigned int ) {}
	void ViewportStackChanged(class BindableStack *) {}
	virtual const char *GetApplicationName() { return "IM1-2D";}
	virtual const char *GetApplicationVersion() { return "3.8";}
	virtual const char *GetWorldURL() { return "Unknown";}

	// Return a pointer to the executive
	Executive* GetExecutive() const { return m_pExecutive; }

	// CC 010123
	StringSensorInstance *GetKeyboardDevice() { return &m_keyboardInstance; }
	KeySensorInstance *GetWeb3DKeySensorDevice() { return &m_Web3DKeySensorInstance; }
	MouseSensorInstance *GetMouseDevice() { return &m_mouseInstance; }
	DeviceFilter *GetFilter() { return &m_deviceFilter; }
	
	// Called when the view is first initialised
	void CreateExecutive(HWND hWnd);

	// Read and write floating point values to the INI file
	float GetProfileFloat(LPCTSTR lpszSection, LPCTSTR lpszEntry,
						  float fDefault);
	BOOL WriteProfileFloat(LPCTSTR lpszSection, LPCTSTR lpszEntry,
						   float value);

	// Read and write the list of network address to the INI file
	void GetProfileNetworkAddresses();
	void WriteProfileNetworkAddresses();

	// Return a reference to the array of network addresses
	ZArray<LPSTR>& GetAddressList() { return m_networkAddresses; }


public:
	// The mutex is set while the file is open
	ZSemaphore m_smpFileOpen;
	//html browser 
	CWebBrowser2  *   m_pBrowser;
private:

	// CC 010123
	DeviceFilter		m_deviceFilter;
	StringSensorInstance	m_keyboardInstance;
	KeySensorInstance	m_Web3DKeySensorInstance;
	MouseSensorInstance		m_mouseInstance;

	// The executive object
	Executive* m_pExecutive;

	// This is TRUE if a sequence should loop continuously
	BOOL m_loopContinuously;

	// This is TRUE if the screen should be cleared when the end
	// of a file is reached.
	BOOL m_clearOnEnd;

	// The timer used to update the playing time
	UINT m_timerID;

	// An array of network address that have been entered
	ZArray<LPSTR> m_networkAddresses;

	// The reason the session finished
	RESPONSE m_reason;


// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CMPEG4PlayerApp)
	public:
	virtual BOOL InitInstance();
	virtual int ExitInstance();
	//}}AFX_VIRTUAL

// Implementation

	//{{AFX_MSG(CMPEG4PlayerApp)
	afx_msg void OnAppAbout();
	afx_msg void OnOptions25fps();
	afx_msg void OnUpdateOptions25fps(CCmdUI* pCmdUI);
	afx_msg void OnOptions15fps();
	afx_msg void OnUpdateOptions15fps(CCmdUI* pCmdUI);
	afx_msg void OnOptions10fps();
	afx_msg void OnUpdateOptions10fps(CCmdUI* pCmdUI);
	afx_msg void OnOptions5fps();
	afx_msg void OnUpdateOptions5fps(CCmdUI* pCmdUI);
	afx_msg void OnOptionsLoopContinuously();
	afx_msg void OnUpdateOptionsLoopContinuously(CCmdUI* pCmdUI);
	afx_msg void OnViewThreadInfo();
	afx_msg void OnViewWorldInfo();
	afx_msg void OnOptionsClearOnEnd();
	afx_msg void OnUpdateOptionsClearOnEnd(CCmdUI* pCmdUI);
	afx_msg void OnFileOpen();
	afx_msg void OnGoBack();
	afx_msg void OnGoForward();
	afx_msg void OnUpdateGoBack(CCmdUI* pCmdUI);
	afx_msg void OnUpdateGoForward(CCmdUI* pCmdUI);
	afx_msg void OnGoReplay();
	afx_msg void OnUpdateGoReplay(CCmdUI* pCmdUI);
	afx_msg void OnFileOpenfile();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};


// Return a pointer to the current application object
inline CMPEG4PlayerApp* GetApp() { return (CMPEG4PlayerApp*)AfxGetApp(); }

// Return a pointer to the main frame window
class CMainFrame;
inline CMainFrame* GetMainFrame() { return (CMainFrame*)GetApp()->m_pMainWnd; }

// Return a pointer to the current document.
class CMPEG4PlayerDoc;
class CMPEG4PlayerView;
CMPEG4PlayerDoc* GetDocument();
CMPEG4PlayerView* GetView();

/////////////////////////////////////////////////////////////////////////////
