// MPEG4PlayerDoc.cpp : implementation of the CMPEG4PlayerDoc class
//

#include "defs.h"
#include <assert.h>
#include "trace.h"
#include "Clock.h"
#include "MPEG4Player.h"
#include "MainFrm.h"
#include "MPEG4PlayerDoc.h"
#include "EnterNetworkAddress.h"
#include "Presenter.h"

#include "MPEG4PlayerView.h"
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CMPEG4PlayerDoc

IMPLEMENT_DYNCREATE(CMPEG4PlayerDoc, CDocument)

BEGIN_MESSAGE_MAP(CMPEG4PlayerDoc, CDocument)
	//{{AFX_MSG_MAP(CMPEG4PlayerDoc)
	ON_COMMAND(ID_FILE_RESTART, OnFileRestart)
	ON_UPDATE_COMMAND_UI(ID_FILE_RESTART, OnUpdateFileRestart)
	ON_COMMAND(ID_FILE_OPEN_NETWORK, OnFileOpenNetwork)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMPEG4PlayerDoc construction/destruction

CMPEG4PlayerDoc::CMPEG4PlayerDoc()
{
	m_fileOpened = FALSE;
	m_webFileOpened= FALSE;
	m_fileName.Empty();
}

CMPEG4PlayerDoc::~CMPEG4PlayerDoc()
{
}

BOOL CMPEG4PlayerDoc::OnNewDocument()
{
//	if (!CDocument::OnNewDocument())
//		return FALSE;

	// TODO: add reinitialization code here
	// (SDI documents will reuse this document)

	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// CMPEG4PlayerDoc serialization

void CMPEG4PlayerDoc::Serialize(CArchive& /*ar*/)
{
}

/////////////////////////////////////////////////////////////////////////////
// CMPEG4PlayerDoc diagnostics

#ifdef _DEBUG
void CMPEG4PlayerDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CMPEG4PlayerDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG


/////////////////////////////////////////////////////////////////////////////

//
// CMPEG4PlayerDoc::DocumentNeedsOpening
//
// This returns TRUE if a file needs to be opened when the executive is
// started.  This only happens when a file is opened in response to being
// double clicked.
//

BOOL CMPEG4PlayerDoc::DocumentNeedsOpening()
{
	return ((!m_fileName.IsEmpty()) && (!m_fileOpened));
} /* CMPEG4PlayerDoc::DocumentNeedsOpening */


/////////////////////////////////////////////////////////////////////////////

//
// CMPEG4PlayerDoc::OpenDocument
//
// Open the file whose name is given by the m_fileName data member of
// the class.  Because Executive::Start() is an asynchronous function,
// we can only assume that the file has been successfully opened when
// Application::OnConnect() is called.
//


// disable "assignment within conditional expression"
#pragma warning (disable : 4706)

BOOL CMPEG4PlayerDoc::OpenDocument() 
{
	// Close the current file - if one is open
	if (m_fileOpened)
		CloseDocument();

	//Check whether in the command line there is "-U" option
	LPTSTR p_strCLine;
	char *p;
	p_strCLine=GetCommandLine();
	if ((p=strstr(p_strCLine," -U")) || (p=strstr(p_strCLine," -u")))
	{
		p+=4;
		m_fileName=p;//now m_fileName should contain 
	}                //the passed command line unmodified
	                 //excepting for -U/-u
	BROWSER eKind=AnalyzeUrl(m_fileName);
	switch (eKind)
	{
		case HTML:
				m_fileOpened=TRUE;
				m_webFileOpened=TRUE;
				GetView() -> PostMessage(WM_WEB_NAVIGATE, NULL,(LPARAM)(LPCSTR )m_fileName);

			
			break;
		case MPEG:
			
			GetApp()->GetExecutive()->Start(m_fileName);
			TRACE_PRINT("Starting the executive, given a file or URL.\n");
			break;
			
		default:
			GetApp()->GetExecutive()->Start(m_fileName);
			TRACE_PRINT("Starting the executive, given a file or URL.\n");
	}
	
	return TRUE;
} /* CMPEG4PlayerDoc::OpenDocument */

#pragma warning (default : 4706)
	


/////////////////////////////////////////////////////////////////////////////

BOOL CMPEG4PlayerDoc::OnOpenDocument(LPCTSTR lpszPathName) 
{
	// Remember the name of the file
	m_fileName = lpszPathName;

#ifdef SIMULATE_BIFS_DECODER
	extern char* globalFileName;

	if (globalFileName != NULL)
		free(globalFileName);

	// Store the name of the file in a global variable, used when
	// simulating the BIFS decoder, so that it knows which scene
	// to create.
	// Get the base file name to decide which type of scene to create
	const char* tmpPtr = strrchr(lpszPathName, '\\');
	ASSERT(tmpPtr != NULL);
	tmpPtr++;
	globalFileName = _strdup(tmpPtr);
#endif

	// If the executive has not yet been created (as is the case if
	// this application started in response to a file being double
	// clicked, then delay opening the document).
	if (GetApp()->GetExecutive() == NULL)
		return TRUE;
	else
		// Open the document with this name
		return OpenDocument();
} /* CMPEG4PlayerDoc::OnOpenDocument */


/////////////////////////////////////////////////////////////////////////////

void CMPEG4PlayerDoc::OnFileOpenNetwork() 
{
	// Close the current file - if one is open
	if (m_fileOpened)
		CloseDocument();

	CEnterNetworkAddress dialog;
	int result = dialog.DoModal();
	
	if (result == IDOK)
	{
		const char* pAddress = dialog.GetNetworkAddress();
		GetApp()->GetExecutive()->Start( pAddress);
		m_fileOpened=TRUE;
		m_fileName=pAddress;
	}

} /* CMPEG4PlayerDoc::OnFileOpenNetwork */


/////////////////////////////////////////////////////////////////////////////

void CMPEG4PlayerDoc::OnFileRestart() 
{
	// Only proceed if there is a previous file to restart
	if (!m_fileName.IsEmpty())
	{
		// Close the current file
		CloseDocument();

		// Open the previous file, and start it playing
		OpenDocument();
	}
} /* CMPEG4PlayerDoc::OnFileRestart */


/////////////////////////////////////////////////////////////////////////////

void CMPEG4PlayerDoc::CloseDocument() 
{
	// Only proceed if  a file is actually open
	if (m_fileOpened)
	{
		// Stop the executive running this session
		GetApp()->GetExecutive()->Stop();
		m_fileOpened = FALSE;

		// Wait for the application to be notified that the file
		// has been closed
		GetApp()->m_smpFileOpen.P();
		GetApp()->m_smpFileOpen.V();
		if(m_webFileOpened)
				::ShowWindow( GetApp()->m_pBrowser->GetSafeHwnd(), SW_HIDE );

	}
} /* CMPEG4PlayerDoc::CloseDocument */


/////////////////////////////////////////////////////////////////////////////

void CMPEG4PlayerDoc::OnCloseDocument() 
{
	TRACE_PRINT("Entering CMPEG4PlayerDoc::OnCloseDocument()\n");
	CloseDocument();	
	CDocument::OnCloseDocument();
	TRACE_PRINT("Leaving CMPEG4PlayerDoc::OnCloseDocument()\n");
} /* CMPEG4PlayerDoc::OnCloseDocument */


/////////////////////////////////////////////////////////////////////////////

void CMPEG4PlayerDoc::OnUpdateFileRestart(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(!m_fileName.IsEmpty());
}


/////////////////////////////////////////////////////////////////////////////
BROWSER CMPEG4PlayerDoc::AnalyzeUrl(const CString url)
{
	LPCTSTR html[] ={"www.","http://",".htm"} ; 

	int tag=0;
	int index=-1;
	CString urlString (url);
	while ((index==-1)&&(tag<3))
		 index= urlString.Find(html[tag++]);
	if(index!=-1)
	    return HTML;	
	char *mpeg[] ={".trif"};
	index =-1;
	tag=0;
	while ((index==-1)&&(tag<1))
		 index= urlString.Find(mpeg[tag++]);
	if(index!=-1)
	    return MPEG;
	char *mp4[] ={".mpg4"};
	index =-1;
	tag=0;
	while ((index==-1)&&(tag<1))
		 index= urlString.Find(mp4[tag++]);
	if(index!=-1)
	    return MPEG;
	return  UNKNOWN;

}	
