//This software module was originally developed by CSELT in the course of   //
//development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.             //
//This software module is an implementation of a part of one or more MPEG-4 //
//Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems        //
//(ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 Systems     //
//(ISO/IEC 14496-1) free license to this software module or modifications   //
//thereof for use in hardware or software products claiming conformance to  //
//the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use this software//
//module in hardware or software products are advised that its use may      //
//infringe existing patents.                                                //
//The original developer of this software module and his/her company, the   //
//subsequent editors and their companies, and ISO/IEC have no liability     //
//for use of this software module or modifications thereof in an            //
//implementation. Copyright is not released for non MPEG-4 Systems          //
//(ISO/IEC 14496-1) conforming products. CSELT S.pA. retains full right     //
//to use the code for his/her own purpose, assign or donate the code to a   //
//third party and to inhibit third parties from using the code for non      //
//MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright      //
//notice must be included in all copies or derivative works.                //
//Copyright  1999.                                                         //
//This work has been partly developed within the framework of the ESPRIT    //
//European Research Project  "MPEG-4 PC".                                   //
//////////////////////////////////////////////////////////////////////////////
//
// File:  MatteTextureProxy.cpp
//
// Author: marc Patoureaux marc@envivio.com
//
// Modified:

// Description:
//   The implementation of the proxy object used by the presenter
//   to render the MatteTexture node.
//   all functions are not implemented, but the most difficult ones are
//   adding new functions wouldn't had any complexity.
//   note that alpha surface are replaces by the R value of regular 3 colour surfaces
//   the reason being at the time of this development, IM1 cannot decode regular alpha surfaces
//
// Created: 18 november 2001.
//
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include "Nodes-CSELT.h"
#include "MatteTextureProxy.h"
#include "Stream.h"
#include "trace.h"
#ifdef DEBUG_CONSOLE
#include "Mpeg4Cselt2DActivexctl.h"
#endif

// =====================================================================

//
// Presenter::InitMatteTexture
//
// Initialise the proxy data structure and attach it to the render data
// pointer of the given media object node.
//
enum {VIDEO_DELAY = 0};

static LPCSTR s_token[] = {
    "INVALID",
    "OFFSET",
    "SCALE",
    "BLUR",
    "BIAS",
    "COLOR_MATRIX",
    "REPLACE_ALPHA",
    "MULTIPLY_ALPHA",
    "CROSS_FADE",
    "BLEND",
    "ADD",
    "A",
    "B",
    "REVEAL"
};


MatteTextureProxy * Presenter::InitMatteTexture(MatteTexture* pObj)
 {
	return new MatteTextureProxy(pObj);
 }

// =====================================================================

//
// MatteTextureProxy::MatteTextureProxy
//
// Class constructor.
//

MatteTextureProxy::MatteTextureProxy(MediaObject* pMO) : DrawableImage(pMO)
{
    m_data = NULL;
    m_dataSize = 0;

} /* MatteTextureProxy::MatteTextureProxy */

MatteTextureProxy::~MatteTextureProxy() {
    delete[] m_data;
}	
	

void MatteTextureProxy::DeleteObject()
{
	MediaObject * a_mediaObject;
    RenderingObject * a_renderingObject;
    
    a_mediaObject = (MediaObject*) GetMatteTexture()->surfaceA;
    if (a_mediaObject) {
        a_renderingObject = a_mediaObject->GetRenderingObject();
        if (a_renderingObject) {
            ((DrawableImage* ) a_renderingObject)->DeleteObject();
        }
    }
    a_mediaObject = (MediaObject*) GetMatteTexture()->surfaceB;
    if (a_mediaObject) {
        a_renderingObject = a_mediaObject->GetRenderingObject();
        if (a_renderingObject) {
            ((DrawableImage* ) a_renderingObject)->DeleteObject();
        }
    }
    a_mediaObject = (MediaObject*) GetMatteTexture()->alphaSurface;
    if (a_mediaObject) {
        a_renderingObject = a_mediaObject->GetRenderingObject();
        if (a_renderingObject) {
            ((DrawableImage* ) a_renderingObject)->DeleteObject();
        }
    }
}

// =====================================================================

//
// MatteTextureProxy::PreRender
//
//

void MatteTextureProxy::PreRender(Effects & effects,DrawableContext * pContext)
{
    MediaObject * a_mediaObject;
    RenderingObject * a_renderingObject;

    a_mediaObject = (MediaObject*) GetMatteTexture()->surfaceA;
    if (a_mediaObject) {
        a_renderingObject = a_mediaObject->GetRenderingObject();
        if (a_renderingObject) {
            ((DrawableImage* ) a_renderingObject)->PreRender(effects,pContext);
        }
    }
    a_mediaObject = (MediaObject*) GetMatteTexture()->surfaceB;
    if (a_mediaObject) {
        a_renderingObject = a_mediaObject->GetRenderingObject();
        if (a_renderingObject) {
            ((DrawableImage* ) a_renderingObject)->PreRender(effects,pContext);
        }
    }
    a_mediaObject = (MediaObject*) GetMatteTexture()->alphaSurface;
    if (a_mediaObject) {
        a_renderingObject = a_mediaObject->GetRenderingObject();
        if (a_renderingObject) {
            ((DrawableImage* ) a_renderingObject)->PreRender(effects,pContext);
        }
    }
} /* MatteTextureProxy::PreRender */

void MatteTextureProxy::Draw(BRect const* pClipBRect,DrawableContext * pContext) {

	MediaObject * a_mediaObject;
    RenderingObject * a_renderingObject;

	SrcRGBAImage dstImage; // dst image should be at resolution of surfaceB

	SrcRGBAImage image1; // this will contain datas from surfaceB
	
	//draw surfaceB into image1
	a_mediaObject = (MediaObject*) GetMatteTexture()->surfaceB;
    if (a_mediaObject) {
        a_renderingObject = a_mediaObject->GetRenderingObject();
        if (a_renderingObject) { // surfaceB
            ((DrawableImage* ) a_renderingObject)->DrawInImage(pClipBRect,pContext,&image1);
        }
    }
	
	// test if draw was successfull
	if (!image1.pR || !image1.pG || !image1.pB) {
		return;
	}

    bool a_overwrite = GetMatteTexture()->overwrite.GetValue();

    if (m_dataSize !=  image1.width*image1.height*image1.ptrInc) {
		m_dataSize = image1.width*image1.height*image1.ptrInc;
		delete[] m_data;
		m_data = new unsigned char[m_dataSize];
	}
	if ((3*image1.width) == image1.pitch && image1.pR < image1.pG) {
        if (!a_overwrite) {
		    memcpy(m_data,image1.pR,m_dataSize*sizeof(unsigned char));
        } else {
            m_data = image1.pR;
        }
	} else {
		int idx;
        unsigned char * ptr = m_data;
		for (unsigned int l = 0; l < image1.height; l++) {
			for (unsigned int k = 0; k < image1.width; k++) {
			    idx = l*image1.pitch + 3*k;
		        *ptr++ = image1.pR[idx]; // m_data R
		        *ptr++ = image1.pG[idx]; // m_data G
		        *ptr++ = image1.pB[idx];
		    }
		}
    }
	dstImage.top = image1.top;
    dstImage.left = image1.left;
	dstImage.width = image1.width;
	dstImage.height = image1.height;
	dstImage.pitch = image1.width * 3;
	dstImage.scaleX = 1;
	dstImage.scaleY = 1;
	dstImage.pR = m_data;
	dstImage.pG = m_data+1;
	dstImage.pB = m_data+2;
	dstImage.pA = NULL;
	dstImage.ptrInc = 3;


    // now parse the operation to figure out if we will need the other texture fields...

    parseOperation((LPCSTR) GetMatteTexture()->operation.GetValue());

    switch(m_token) {
        
	    case BIAS :
	        applyBIAS(dstImage);
	        break;
	    case SCALE :
	        applySCALE(dstImage);
	        break;
	    case BLUR :
	        applyBLUR(dstImage);
	        break;
	    case REPLACE_ALPHA : // here we have 2 surfaces as input
        case CROSS_FADE : {   // the output will have the size of surfaceB (spec)
        	SrcRGBAImage image2;
        	// choose right field
        	if (m_token == CROSS_FADE) {
        		a_mediaObject = (MediaObject*) GetMatteTexture()->surfaceA;
        	} else { // m_token = REPLACE_ALPHA
				a_mediaObject = (MediaObject*) GetMatteTexture()->alphaSurface;
			}
			// draw field into image2
		    if (a_mediaObject) {
		        a_renderingObject = a_mediaObject->GetRenderingObject();
		        if (a_renderingObject) { // depend
		            ((DrawableImage* ) a_renderingObject)->DrawInImage(pClipBRect,pContext,&image2);
                } else {
                    return;
                }
            } else {
                return;
            }
		    
            if (!image2.pR || !image2.pG || !image2.pB) {
		        return;
	        }
		    
            if (m_token == CROSS_FADE) {
        		applyCROSS_FADE(dstImage,image2);
        	} else { // m_token = REPLACE_ALPHA
				applyREPLACE_ALPHA(dstImage,image2);
			}
            
            break;
        }
        case REVEAL : {// here we have 3 surfaces as input
            SrcRGBAImage imageA;
            SrcRGBAImage imageAlpha;

            a_mediaObject = (MediaObject*) GetMatteTexture()->surfaceA;
            if (a_mediaObject) {
		        a_renderingObject = a_mediaObject->GetRenderingObject();
		        if (a_renderingObject) { // depend
		            ((DrawableImage* ) a_renderingObject)->DrawInImage(pClipBRect,pContext,&imageA);
                } else {
                    return;
                }
            } else {
                return;
            }
            // check not empty
            if (!imageA.pR || !imageA.pG || !imageA.pB) {
		        return;
	        }

            a_mediaObject = (MediaObject*) GetMatteTexture()->alphaSurface;
            if (a_mediaObject) {
		        a_renderingObject = a_mediaObject->GetRenderingObject();
		        if (a_renderingObject) { // depend
		            ((DrawableImage* ) a_renderingObject)->DrawInImage(pClipBRect,pContext,&imageAlpha);
                } else {
                    return;
                }
            } else {
                return;
            }
            // check not empty
            if (!imageAlpha.pR || !imageAlpha.pG || !imageAlpha.pB) {
		        return;
	        }

            applyREVEAL(dstImage,imageA,imageAlpha);
            break;
            }
        default :
            break;
    }

    GetVisualRenderer()->DisplayRGBAImage(dstImage);

} /* DrawableImage::Draw */

void MatteTextureProxy::parseOperation(LPCSTR operation) {
    for (int i = INVALID ; i < NBOTOKEN; i++) {
        if (!stricmp(operation,s_token[i])) {
            m_token = (oToken) i;
            return;
        }
    }
    m_token = INVALID;
}

void MatteTextureProxy::applyBIAS(SrcRGBAImage & image) {
    long a_parameter[4];
    int i;
    unsigned int j;
    for ( i = 0; i < 4; i ++) {
        if (i < GetMatteTexture()->parameter.GetLength()) {
            a_parameter[i] = 255 * GetMatteTexture()->parameter[i];
        } else {
            a_parameter[i] = 0;
        }
    }
    unsigned char * ptr = image.pR;
    unsigned int totalsize = image.width*image.height;
    long value;
    for ( j = 0; j < totalsize; j++) {
        for ( i = 0; i < 3; i ++) { // loop for R,G,B
            value = *ptr + a_parameter[i];
            if (value > 255) { 
                value = 255;
            } else if (value < 0) {
                value = 0;
            }
            *ptr++ = (unsigned char) value;
        }
    }
}

void MatteTextureProxy::applySCALE(SrcRGBAImage & image) {
    float a_parameter[4];
    int i;
    for ( i = 0; i < 4; i ++) {
        if (i < GetMatteTexture()->parameter.GetLength()) {
            a_parameter[i] = GetMatteTexture()->parameter[i];
        } else {
            a_parameter[i] = 0;
        }
    }
    float value;
    for ( i = 0; i < 3; i ++) {
        for (unsigned int j = 0; j < image.width*image.height; j++) {
            value = ((float) image.pR[3*j+i]) / 255 * a_parameter[i];
            if (value > 1) { 
                value = 1;
            } else if (value < 0) {
                value = 0;
            }
            image.pR[3*j+i] = (unsigned char) (value *255);
        }
    }
}

float b_kernel(float x, float s) {
    if (s != 0) {
        return exp(-x*x/(2*s*s));
    } else if (x==0) {
        return 10;
    } else {
        return 0;
    }
}

void MatteTextureProxy::applyBLUR(SrcRGBAImage & image) {
	unsigned char * originalData = new unsigned char[3*image.width*image.height];
	memcpy(originalData, image.pR, 3*image.width*image.height * sizeof(unsigned char));
    // This is a basic integration of the blur kernel function !!!!!
    // this can be better done...
    float a_parameter[2];
    int i,l,m;
    unsigned int j,k;
    switch (GetMatteTexture()->parameter.GetLength()) {
    case 0:
        return;
    case 1:
        a_parameter[0] = GetMatteTexture()->parameter[0];
        a_parameter[1] = a_parameter[0];
        break;
    default :
        a_parameter[0] = GetMatteTexture()->parameter[0];
        a_parameter[1] = GetMatteTexture()->parameter[1];
    }

    if (a_parameter[0] == 0 && a_parameter[1] == 0) {
        return;
    }

    int Xcount = 0;
    while (b_kernel(Xcount+0.5, a_parameter[0]) > 0.05) {
        Xcount++;
    }
    int Ycount = 0;
    while (b_kernel(Ycount+0.5, a_parameter[1]) > 0.05) {
        Ycount++;
    }
    if (Xcount == 0 && Ycount == 0) {
        return;
    }

    float * a_XRatio = new float[2*Xcount +1];
    float * a_YRatio = new float[2*Ycount +1];
    float Xsum = 0;
    float Ysum = 0;

    for ( i = 0; i <= Xcount; i++) {
        a_XRatio[Xcount+i] = 0;
        for ( int f = -5; f < 5; f++) {
            if ((10 * i + f) < 0) {
                continue;
            }
            a_XRatio[Xcount+i] += b_kernel((float)(10*i + f) / 10.0, a_parameter[0]) * 0.1;
        }
        Xsum += 2*a_XRatio[Xcount+i];
        if (i == 0) {
            a_XRatio[Xcount-i] *= 2;
        } else {
            a_XRatio[Xcount-i] = a_XRatio[Xcount+i];
        }
    }
    for ( i = 0; i <= Ycount; i++) {
        a_YRatio[Ycount+i] = 0;
        for ( int f = -5; f < 5; f++) {
            if ((10 * i + f) < 0) {
                continue;
            }
            a_YRatio[Ycount+i] += b_kernel((float)(10*i + f) / 10.0, a_parameter[1]) * 0.1;
        }
        Ysum += 2*a_YRatio[Ycount+i];
        if (i == 0) {
            a_YRatio[Ycount-i] *= 2;
        } else {
            a_YRatio[Ycount-i] = a_YRatio[Ycount+i];
        }
    }

    for (i = -Xcount; i <= Xcount; i++) {
        a_XRatio[Xcount+i] /= Xsum;
    }
    for (i = -Ycount; i <= Ycount; i++) {
        a_YRatio[Ycount+i] /= Ysum;
    }


    for ( i = 0; i < 3; i ++) {
        for ( j = 0; j < image.width; j++) {
            for ( k = 0; k < image.height; k++) {
                unsigned char a_value = 0;
                for (l = -Xcount; l <= Xcount; l++) {
                    for (m = -Ycount; m <= Ycount; m++) {
                        if (k+m < 0 
                            || k+m >= image.height
                            || j+l < 0
                            || j+l >= image.width) {
                            continue;
                        }
                        a_value += a_XRatio[Xcount+l]*a_YRatio[Ycount+m]*originalData[3*(j+l+image.width*(k+m))+i];
                    }
                }
                image.pR[3*(j+image.width*k)+i] = a_value;
            }
        }
    }
    delete[] a_XRatio;
    delete[] a_YRatio;
}

void MatteTextureProxy::applyCROSS_FADE(SrcRGBAImage & B, SrcRGBAImage & A) {
    float a_fraction = (float) GetMatteTexture()->fraction;
    if (a_fraction > 1) { 
        a_fraction = 1;
    } else if (a_fraction < 0) {
        a_fraction = 0;
    }

    float scaleX = (float) A.width / (float) B.width;
    float scaleY = (float) A.height / (float) B.height;

    // B is supposed to have pitch = 3*width
    int Bpitch = B.pitch;
    int Bidx = 0;
    int Aidx0 = 0;
    int Aidx;
    for (unsigned int Y = 0; Y < B.height; Y++) {
        Bidx = Bpitch * Y;
        Aidx0 = A.pitch*((int) (scaleY*Y));
        for (unsigned int X = 0; X < B.width; X++) {        
            Aidx = Aidx0 + 3*((int) (scaleX*X));
            B.pR[Bidx] = (unsigned char) ( a_fraction * B.pR[Bidx] + (1-a_fraction) * A.pR[Aidx]);
            B.pG[Bidx] = (unsigned char) ( a_fraction * B.pG[Bidx] + (1-a_fraction) * A.pG[Aidx]);
            B.pB[Bidx] = (unsigned char) ( a_fraction * B.pB[Bidx] + (1-a_fraction) * A.pB[Aidx]);
            Bidx += 3;
        }
    }
}

void MatteTextureProxy::applyREPLACE_ALPHA(SrcRGBAImage & B, SrcRGBAImage & A) {
    float scaleX = (float) A.width / (float) B.width;
    float scaleY = (float) A.height / (float) B.height;

    int Bpitch = B.pitch;
    unsigned int height = B.height;
    unsigned int width = B.width;
    int Bidx = 0;
    int Aidx0 = 0;
    int Aidx;
    for (unsigned int Y = 0; Y < height; Y++) {
        Bidx = Bpitch * Y;
        Aidx0 = A.pitch*((int) (scaleY*Y));
        for (unsigned int X = 0; X < width; X++) {        
            Aidx = Aidx0 + 3*((int) (scaleX*X));
            // instead of computing with Alpha Value, the function is computed with the 
            // Red value of image alpha...
            // this is to be fixxed... somehow !
            // pR being replaced by pA for image A... but
            // then a content with Alpha value activated has to be created !
            B.pR[Bidx] = (unsigned char) (((int) B.pR[Bidx]) * ((int) A.pR[Aidx]) / 255);
            B.pG[Bidx] = (unsigned char) (((int) B.pG[Bidx]) * ((int) A.pR[Aidx]) / 255);
            B.pB[Bidx] = (unsigned char) (((int) B.pB[Bidx]) * ((int) A.pR[Aidx]) / 255);
            Bidx += 3;
        }
    }
}

void MatteTextureProxy::applyREVEAL(SrcRGBAImage & B, SrcRGBAImage & A, SrcRGBAImage & Alpha) {
    float a_fraction  = (float) GetMatteTexture()->fraction;
    float a_parameter = (float) GetMatteTexture()->parameter[0];

    int height = B.height;
    int width = B.width;
    int pitch = B.pitch;

    // B size is the output size ! ( moreover B is already the output image )
    float scaleXA =     (float) A.width /  (float) B.width;
    float scaleYA =     (float) A.height / (float) B.height;
    float scaleXAlpha = (float) Alpha.width /  (float) B.width;
    float scaleYAlpha = (float) Alpha.height / (float) B.height;

    int AlphaIdx0;
    int AlphaIdx;
    int Bidx = 0;
    int Aidx;
    int Aidx0;

    for ( int Y = 0; Y < height; Y++) {
        AlphaIdx0 = Alpha.pitch*((int) (scaleYAlpha*Y));
        Aidx0 = A.pitch*((int) (scaleYA*Y));
        Bidx = pitch * Y;
        for (int X = 0; X < width; X++) {
            AlphaIdx = AlphaIdx0 + 3*((int) (scaleXAlpha * X));
            Aidx = Aidx0 + 3*((int) (scaleXA * X));
            if (Alpha.pR[AlphaIdx] < (255*(a_fraction - a_parameter))) {
                B.pR[Bidx] = A.pR[Aidx];
                B.pG[Bidx] = A.pG[Aidx];
                B.pB[Bidx] = A.pB[Aidx];
            } else if (Alpha.pR[AlphaIdx] < (255*(a_fraction + a_parameter))) {
                float a_ratio = 0.5 + ((((float)Alpha.pR[AlphaIdx])/255.0) - a_fraction) / a_parameter / 2;
                if (a_ratio > 1) { 
                    a_ratio = 1;
                } else if (a_ratio < 0) {
                    a_ratio = 0;
                }
                B.pR[Bidx] = a_ratio*B.pR[Bidx] + (1-a_ratio)*A.pR[Aidx];
                B.pG[Bidx] = a_ratio*B.pG[Bidx] + (1-a_ratio)*A.pG[Aidx];
                B.pB[Bidx] = a_ratio*B.pB[Bidx] + (1-a_ratio)*A.pB[Aidx];
            }
            Bidx += 3;
        }
    }
}