//This software module was originally developed by CSELT in the course of   //
//development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.             //
//This software module is an implementation of a part of one or more MPEG-4 //
//Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems        //
//(ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 Systems     //
//(ISO/IEC 14496-1) free license to this software module or modifications   //
//thereof for use in hardware or software products claiming conformance to  //
//the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use this software//
//module in hardware or software products are advised that its use may      //
//infringe existing patents.                                                //
//The original developer of this software module and his/her company, the   //
//subsequent editors and their companies, and ISO/IEC have no liability     //
//for use of this software module or modifications thereof in an            //
//implementation. Copyright is not released for non MPEG-4 Systems          //
//(ISO/IEC 14496-1) conforming products. CSELT S.pA. retains full right     //
//to use the code for his/her own purpose, assign or donate the code to a   //
//third party and to inhibit third parties from using the code for non      //
//MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright      //
//notice must be included in all copies or derivative works.                //
//Copyright  1999.                                                         //
//This work has been partly developed within the framework of the ESPRIT    //
//European Research Project  "MPEG-4 PC".                                   //
//////////////////////////////////////////////////////////////////////////////
//
// File:  MovieTextureProxy.cpp
//
// Author: Gianluca Di Cagno  gianluca.dicagno@cselt.it
//
// Modified:

// Description:
//   The implementation of the proxy object used by the presenter
//   to render the MovieTexture node.
//
// Created: 16 December, 1998.
//
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include "Nodes-CSELT.h"
#include "MovieTextureProxy.h"
#include "Stream.h"
#include "trace.h"
#include "ShapeProxy.h"			// ZL
#ifdef DEBUG_CONSOLE
#include "Mpeg4Cselt2DActivexctl.h"
#endif

// =====================================================================

//
// Presenter::InitMovieTexture
//
// Initialise the proxy data structure and attach it to the render data
// pointer of the given media object node.
//

 MovieTextureProxy * Presenter::InitMovieTexture(MovieTexture* pObj)
 {
	return new MovieTextureProxy(pObj);
 }


// =====================================================================

//
// MovieTextureProxy::MovieTextureProxy
//
// Class constructor.
//

MovieTextureProxy::MovieTextureProxy(MediaObject* pMO) : DrawableImage(pMO)
{
	m_firstTimeAtStartTime = TRUE;
	m_firstTimeAtStopTime  = TRUE;
	m_endOfTimeReached = FALSE;
	m_startTime = 0;
	m_timesStarted = 0;
	m_OpenStreamIsBeingUsed=FALSE;
	#ifdef DEBUG_CONSOLE
//	const char *pDecInfo=GetMovieTexture()->GetDecoder()->GetInfo();
//	::PostMessage(GetPresenter()->GetHWND() ,WM_DEC_INFO, NULL,(LPARAM)(LPCSTR )pDecInfo);
	


#endif

} /* MovieTextureProxy::MovieTextureProxy */
MovieTextureProxy::~MovieTextureProxy()
{
}	
	

void MovieTextureProxy::DeleteObject()
{
	GetPresenter()->RemoveOpenStream(this);
}


// =====================================================================

//
// MovieTextureProxy::PreRender
//
//

void MovieTextureProxy::PreRender(Effects & effects,DrawableContext * pContext)
{

	/*-------------------------------------------------------------
	startTime / stopTime fields need to be offset by current time on first access
	-------------------------------------------------------------*/
	GetMovieTexture() -> startTime.ApplyOffset (pContext ->GetShapeProxy () ->GetTimeContext ());
	GetMovieTexture() -> stopTime.ApplyOffset (pContext ->GetShapeProxy () ->GetTimeContext ());

	//if this node has been switched off,stop the associated decoder
	
	if (!pContext->IsSwitchedOn())
		   {
		//let's try to stop the decoder cause the node has been switched off
		
		//OnEndOfStream();
		// Set flags and events which are required at the end of a stream.
		//m_firstTimeAtStopTime = FALSE;
		//m_firstTimeAtStartTime = TRUE;
	
        return;
		   }
	
	// This gets the time since the start of the scene
	// (convert the time to seconds, as used by BIFS)
// #GB:	double currentTime = (double)((Presenter*)GetPresenter())->GetTime() / 1000;
// ZL: changed to use the Shape parent because the MovieTexture doesn't have up to date
// time information
  double currentTime = pContext ->GetShapeProxy () ->GetFlexTime()/1000.;

	// If the start time has changed since this was last called, reset things
	 if ((double)GetMovieTexture()->startTime != m_startTime)
		m_endOfTimeReached = FALSE;
	m_startTime = (double)GetMovieTexture()->startTime;

	// Only proceed if the end time has not been reached
	
	if (m_endOfTimeReached)
	      return;

	// Only proceed if the start time has been reached
	if (currentTime < m_startTime)
		return;

	// Determine if the stop time has been reached
	if ((currentTime >= (double)GetMovieTexture()->stopTime) &&
		((double)GetMovieTexture()->stopTime >= m_startTime))
	{
		// If this is the first time, generate an isActive=FALSE event,
		// but only if the stream was previously active
		if ((m_firstTimeAtStopTime) && ((Bool)GetMovieTexture()->isActive))
			OnEndOfStream();
	}

	// otherwise the movie needs to be played
	else
	{
		// If this is the first time, generate an isActive=TRUE event
		if (m_firstTimeAtStartTime)
		{
			
			MovieTexture* pMovieTexture = GetMovieTexture();

			m_firstTimeAtStartTime = FALSE;
			m_firstTimeAtStopTime = TRUE;
			pMovieTexture->isActive = TRUE;
			m_timesStarted++;

			
			//here I should calculate currentTime-m_startTime
			//and seek the video to the correspondent position 
			//This would enable start and pause and continue.



			if (m_timesStarted > 1)
			{
	
				m_startTime = (double)pMovieTexture->startTime;
				pMovieTexture->OpenStream();
				TRACE_PRINT1("Video: Starting at %g\n", m_startTime);
				return;                       
			}

			// we also need to start the stream
			pMovieTexture->OpenStream();
			GetPresenter()->AddToOpenStreams(this);

            TRACE_PRINT1("Video: Starting at %g\n", m_startTime);

		}

		DrawableImage::PreRender(effects,pContext);

		if(pContext->m_hasStreamFinished==TRUE)
		{
			if((Bool)GetMovieTexture()->loop)
			{
				GetMovieTexture()->CloseStream();
				GetMovieTexture()->OpenStream();
			}
		}
		m_OpenStreamIsBeingUsed=TRUE;
		
	}
} /* MovieTextureProxy::PreRender */

void MovieTextureProxy::OnEndOfStream()
{
	// Call the base class's method
	DrawableImage::OnEndOfStream();

	// Set flags and events which are required at the end of a stream.
	m_firstTimeAtStopTime = FALSE;
	m_firstTimeAtStartTime = TRUE;
	m_endOfTimeReached = TRUE;
	GetMovieTexture()->isActive = FALSE;
} /* MovieTextureProxy::OnEndOfStream */



// =====================================================================
