//This software module was originally developed by CSELT in the course of   //
//development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.             //
//This software module was originally developed by CSELT in the course of   //
//development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.             //
//This software module is an implementation of a part of one or more MPEG-4 //
//Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems        //
//(ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 Systems     //
//(ISO/IEC 14496-1) free license to this software module or modifications   //
//thereof for use in hardware or software products claiming conformance to  //
//the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use this software//
//module in hardware or software products are advised that its use may      //
//infringe existing patents.                                                //
//The original developer of this software module and his/her company, the   //
//subsequent editors and their companies, and ISO/IEC have no liability     //
//for use of this software module or modifications thereof in an            //
//implementation. Copyright is not released for non MPEG-4 Systems          //
//(ISO/IEC 14496-1) conforming products. CSELT S.pA. retains full right     //
//to use the code for his/her own purpose, assign or donate the code to a   //
//third party and to inhibit third parties from using the code for non      //
//MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright      //
//notice must be included in all copies or derivative works.                //
//Copyright  1999.                                                         //
//This work has been partly developed within the framework of the ESPRIT    //
//European Research Project  "MPEG-4 PC".                                   //
//////////////////////////////////////////////////////////////////////////////
// File:  Nodes-CSELT.cpp
//
// Author: Matthew Leditschke 
//
// Modified: Gianluca Di Cagno  gianluca.dicagno@cselt.it
//
// Description:
//   This file contains the implementation of the renderer independent
//   portions of the nodes.
//
// Created: August, 1997.
//
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include <assert.h>
#include "Nodes-CSELT.h"
//#include "WrldInfo.h"
#include "NodeCode.h"
#include "ValuatorProxy.h"
#include "TermCapProxy.h"

// ========================================================================

//
// Interpolate
//
// Given two keys, and their corresponding values, and a fraction value
// which lies between the two key values, return the interpolated value.
//

static float Interpolate(float key1, float keyValue1,
						 float key2, float keyValue2,
						 float fraction)
{
	// Check that the fraction lies between the two key values
	assert((fraction >= key1) && (fraction <= key2));

	// Determine how far between the keys this fraction is,
	// allowing for (nearly) identical key values.
	float intervalFraction;
	float keyDiff = key2 - key1;
	if ((keyDiff > -1e-10) && (keyDiff < 1e-10))
		intervalFraction = 0;
	else
		intervalFraction = (fraction - key1) / keyDiff;

	// Interpolate between the key values using this fraction
	float newValue = (keyValue2 - keyValue1) * intervalFraction + keyValue1;

	return newValue;
} /* Interpolate */
// ========================================================================

//
// Background2D ::Onset_bind
//
// This function is called when the set_fraction data member has changed,
// and it has already been updated when this function is called.  When
// this happens, calculate the value that corresponds with this fraction,
// and update the value_changed data member.
//

/*void class Background2D::Onset_bind() 
{
	Background2DProxy* pBackground2DProxy=((Background2DProxy*)GetRenderingObject());

	if(set_bind==FALSE)
	{
		pBackground2DProxy->RemoveBackgroundFromTheStack();
		isBound=FALSE;
	}
	else
	{
		pBackground2DProxy->MoveBackgroundToTheTop();
		isBound=TRUE;
	}

}*/
//
// CoordinateInterpolator2D::Onset_fraction
//
// This function is called when the set_fraction data member has changed,
// and it has already been updated when this function is called.  When
// this happens, calculate the value that corresponds with this fraction,
// and update the value_changed data member.
//

void CoordinateInterpolator2D::Onset_fraction()
{
	// Ensure that the key and keyValue arrays are the same length,
	// and that they are not empty.
	
	int keyValueLength=keyValue.GetLength();
	int keyLength=key.GetLength();
	
	assert((keyValueLength %keyLength)==0  );
	assert(keyLength != 0);
	
	int numberOfElementsPerKey =keyValueLength/keyLength;
	
	int z;
	
	// The given fraction is less than the specified range

	if ((float)set_fraction < key[0])
	{
		for(z=0;z<numberOfElementsPerKey-1;z++)
			value_changed.SetAt(z, keyValue[z],FALSE);  //don't call UpdateField immedietly
		value_changed.SetAt(numberOfElementsPerKey-1, keyValue[numberOfElementsPerKey-1],TRUE);
	}

	// The given fraction is greater than the specified range
	else if ((float)set_fraction >= key[key.GetLength() - 1])
	{
		for(z=0;z<numberOfElementsPerKey-1;z++)
			value_changed.SetAt(z,keyValue[(keyValue.GetLength())-numberOfElementsPerKey+z],FALSE);
		value_changed.SetAt(numberOfElementsPerKey-1, keyValue[(keyValue.GetLength())-numberOfElementsPerKey+numberOfElementsPerKey-1],TRUE);

	}

	// The given fraction lies in the specified range of key values
	else
	{
		for (int k = 1; k < key.GetLength(); k++)
		{
			// Find the key values the fraction lies between
			if (((float)set_fraction >= key[k - 1]) &&
				((float)set_fraction < key[k]))
			{
				// and then interpolate the new value
				Vec2f newValue;

				for(z=0;z<numberOfElementsPerKey-1;z++)
				{
					
					newValue.x = Interpolate(key[k - 1], keyValue[(k - 1)*numberOfElementsPerKey+z].x,
											 key[k], keyValue[(k)*numberOfElementsPerKey+z].x,
											 (float)set_fraction);
					newValue.y = Interpolate(key[k - 1], keyValue[(k - 1)*numberOfElementsPerKey+z].y,
											 key[k], keyValue[(k)*numberOfElementsPerKey+z].y,
											 (float)set_fraction);
					value_changed.SetAt(z,newValue,FALSE);
				}

				//store last value changed and call  UpdateField
				
				newValue.x = Interpolate(key[k - 1], keyValue[(k - 1)*numberOfElementsPerKey+numberOfElementsPerKey-1].x,
										 key[k], keyValue[(k)*numberOfElementsPerKey+numberOfElementsPerKey-1].x,
										 (float)set_fraction);
				newValue.y = Interpolate(key[k - 1], keyValue[(k - 1)*numberOfElementsPerKey+numberOfElementsPerKey-1].y,
										 key[k], keyValue[(k)*numberOfElementsPerKey+numberOfElementsPerKey-1].y,
										 (float)set_fraction);
				value_changed.SetAt(numberOfElementsPerKey-1,newValue,TRUE);

				break;
			}
		}
	}
	
} /* CoordinateInterpolator2D::Onset_fraction */
// ========================================================================

//
// ColorInterpolator::Onset_fraction
//
// This function is called when the set_fraction data member has changed,
// and it has already been updated when this function is called.  When
// this happens, calculate the value that corresponds with this fraction,
// and update the value_changed data member.
//

void ColorInterpolator::Onset_fraction()
{
	// Ensure that the key and keyValue arrays are the same length,
	// and that they are not empty.
	assert(key.GetLength() == keyValue.GetLength());
	assert(key.GetLength() != 0);

	// The given fraction is less than the specified range
	if ((float)set_fraction < key[0])
		value_changed = keyValue[0];

	// The given fraction is greater than the specified range
	else if ((float)set_fraction >= key[key.GetLength() - 1])
		value_changed = keyValue[key.GetLength() - 1];

	// The given fraction lies in the specified range of key values
	else
	{
		for (int k = 1; k < key.GetLength(); k++)
		{
			// Find the key values the fraction lies between
			if (((float)set_fraction >= key[k - 1]) &&
				((float)set_fraction < key[k]))
			{
				// and then interpolate the new value
				Color3f newValue;
				newValue.r = Interpolate(key[k - 1], keyValue[k - 1].r,
										 key[k], keyValue[k].r,
										 (float)set_fraction);
				newValue.g = Interpolate(key[k - 1], keyValue[k - 1].g,
										 key[k], keyValue[k].g,
										 (float)set_fraction);
				newValue.b = Interpolate(key[k - 1], keyValue[k - 1].b,
										 key[k], keyValue[k].b,
										 (float)set_fraction);
				value_changed = newValue;
				break;
			}
		}
	}
} /* ColorInterpolator::Onset_fraction */




//
// PositionInterpolator2D::Onset_fraction
//
// This function is called when the set_fraction data member has changed,
// and it has already been updated when this function is called.  When
// this happens, calculate the value that corresponds with this fraction,
// and update the value_changed data member.
//

void PositionInterpolator2D::Onset_fraction()
{
	// Ensure that the key and keyValue arrays are the same length,
	// and that they are not empty.
	assert(key.GetLength() == keyValue.GetLength());
	assert(key.GetLength() != 0);

	// The given fraction is less than the specified range
	if ((float)set_fraction < key[0])
		value_changed = keyValue[0];

	// The given fraction is greater than the specified range
	else if ((float)set_fraction >= key[key.GetLength() - 1])
		value_changed = keyValue[key.GetLength() - 1];

	// The given fraction lies in the specified range of key values
	else
	{
		for (int k = 1; k < key.GetLength(); k++)
		{
			// Find the key values the fraction lies between
			if (((float)set_fraction >= key[k - 1]) &&
				((float)set_fraction < key[k]))
			{
				// and then interpolate the new value
				Vec2f newValue;
				newValue.x = Interpolate(key[k - 1], keyValue[k - 1].x,
										 key[k], keyValue[k].x,
										 (float)set_fraction);
				newValue.y = Interpolate(key[k - 1], keyValue[k - 1].y,
										 key[k], keyValue[k].y,
										 (float)set_fraction);
				value_changed = newValue;
				break;
			}
		}
	}
} /* PositionInterpolator2D::Onset_fraction */



// ========================================================================

//
// ScalarInterpolator::Onset_fraction
//
// This function is called when the set_fraction data member has changed,
// and it has already been updated when this function is called.  When
// this happens, calculate the value that corresponds with this fraction,
// and update the value_changed data member.
//

void ScalarInterpolator::Onset_fraction()
{
	// Ensure that the key and keyValue arrays are the same length,
	// and that they are not empty.
	assert(key.GetLength() == keyValue.GetLength());
	assert(key.GetLength() != 0);

	// The given fraction is less than the specified range
	if ((float)set_fraction < key[0])
		value_changed = keyValue[0];

	// The given fraction is greater than the specified range
	else if ((float)set_fraction >= key[key.GetLength() - 1])
		value_changed = keyValue[key.GetLength() - 1];

	// The given fraction lies in the specified range of key values
	else
	{
		for (int k = 1; k < key.GetLength(); k++)
		{
			// Find the key values the fraction lies between
			if (((float)set_fraction >= key[k - 1]) &&
				((float)set_fraction < key[k]))
			{
				// and then interpolate the new value
				float newValue = Interpolate(key[k - 1], keyValue[k - 1],
											 key[k], keyValue[k],
											 (float)set_fraction);
				value_changed = newValue;
				break;
			}
		}
	}
} /* ScalarInterpolator::Onset_fraction */


// ========================================================================
//
// Valuator ::  All Methods share the same implementation.
//
// Implementation of all the Valuator methods, indeed only the first one 
// is defined, the other are all defined in terms of the first one.
// It just signals the all the fields, checks if they are connected to 
// routes, and finally route their initial value. old
//


void Valuator :: OninSFBool()
 {
      
	ValuatorProxy* pValuatorProxy=((ValuatorProxy*)GetRenderingObject());
	Bool b = (Bool)inSFBool;
	pValuatorProxy->StoreInValues((float)b);
	pValuatorProxy->Assign();  
}

 void Valuator :: OninSFColor()
 {
	ValuatorProxy* pValuatorProxy=((ValuatorProxy*)GetRenderingObject());
	pValuatorProxy->StoreInValues(((Color3f)inSFColor).r,((Color3f)inSFColor).g,((Color3f)inSFColor).b);
	pValuatorProxy->Assign();
 }

 void Valuator :: OninMFColor()
 {
	ValuatorProxy* pValuatorProxy=((ValuatorProxy*)GetRenderingObject());
	pValuatorProxy->StoreInValues(((Color3f)(inMFColor[0])).r,((Color3f)(inMFColor[0])).g,((Color3f)(inMFColor[0])).b);
	pValuatorProxy->Assign();
 }

 void Valuator :: OninSFFloat()
 {
	ValuatorProxy* pValuatorProxy=((ValuatorProxy*)GetRenderingObject());
	pValuatorProxy->StoreInValues(inSFFloat);
	pValuatorProxy->Assign();
 }

 void Valuator :: OninMFFloat()
 {
	ValuatorProxy* pValuatorProxy=((ValuatorProxy*)GetRenderingObject());
	pValuatorProxy->StoreInValues(inMFFloat[0]);
	pValuatorProxy->Assign();
 }

 void Valuator :: OninSFInt32()
 {
	ValuatorProxy* pValuatorProxy=((ValuatorProxy*)GetRenderingObject());
   	pValuatorProxy->StoreInValues((float)((int)inSFInt32));
	pValuatorProxy->Assign();
 }
 
 void Valuator :: OninMFInt32()
 {
	ValuatorProxy* pValuatorProxy=((ValuatorProxy*)GetRenderingObject());
   	pValuatorProxy->StoreInValues((float)((int)(inMFInt32[0])));
	pValuatorProxy->Assign();
 }
 
 void Valuator :: OninSFRotation()
 {
	ValuatorProxy* pValuatorProxy=((ValuatorProxy*)GetRenderingObject());
	pValuatorProxy->StoreInValues(((Rotation)inSFRotation).x,((Rotation)inSFRotation).y,((Rotation)inSFRotation).z,((Rotation)inSFRotation).theta);
	pValuatorProxy->Assign();
 }

 void Valuator :: OninMFRotation()
 {
	ValuatorProxy* pValuatorProxy=((ValuatorProxy*)GetRenderingObject());
	pValuatorProxy->StoreInValues(((Rotation)(inMFRotation[0])).x,((Rotation)(inMFRotation[0])).y,((Rotation)(inMFRotation[0])).z,((Rotation)(inMFRotation[0])).theta);
	pValuatorProxy->Assign();
 }

 void Valuator :: OninSFString()
 {
	 outSFString.OnFieldUpdate();
 }

 void Valuator :: OninMFString()
 {
	 outMFString.OnFieldUpdate();
 }

 void Valuator :: OninSFTime()
 {
	ValuatorProxy* pValuatorProxy=((ValuatorProxy*)GetRenderingObject());
	pValuatorProxy->StoreInValues((float)inSFTime);
	pValuatorProxy->Assign();
 }

 void Valuator :: OninSFVec2f()
 {
	ValuatorProxy* pValuatorProxy=((ValuatorProxy*)GetRenderingObject());
	pValuatorProxy->StoreInValues(((Vec2f)inSFVec2f).x,((Vec2f)inSFVec2f).y);
	pValuatorProxy->Assign();
 }

 void Valuator :: OninMFVec2f()
 {
	ValuatorProxy* pValuatorProxy=((ValuatorProxy*)GetRenderingObject());
	pValuatorProxy->StoreInValues(((Vec2f)(inMFVec2f[0])).x,((Vec2f)(inMFVec2f[0])).y);
	pValuatorProxy->Assign();
 }

 void Valuator :: OninSFVec3f()
 {
	ValuatorProxy* pValuatorProxy=((ValuatorProxy*)GetRenderingObject());
	pValuatorProxy->StoreInValues(((Vec3f)inSFVec3f).x,((Vec3f)inSFVec3f).y,((Vec3f)inSFVec3f).z);
	pValuatorProxy->Assign();
 }

 void Valuator :: OninMFVec3f()
 {
	ValuatorProxy* pValuatorProxy=((ValuatorProxy*)GetRenderingObject());
	pValuatorProxy->StoreInValues(((Vec3f)(inMFVec3f[0])).x,((Vec3f)(inMFVec3f[0])).y,((Vec3f)(inMFVec3f[0])).z);
	pValuatorProxy->Assign();
 }

 //TermCap

 void TermCap :: Onevaluate() 
 {
	TermCapProxy* pTermCapProxy=((TermCapProxy*)GetRenderingObject());
	pTermCapProxy->Query(capability);

 }