//This software module was originally developed by CSELT in the course of   //
//development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.             //
//This software module is an implementation of a part of one or more MPEG-4 //
//Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems        //
//(ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 Systems     //
//(ISO/IEC 14496-1) free license to this software module or modifications   //
//thereof for use in hardware or software products claiming conformance to  //
//the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use this software//
//module in hardware or software products are advised that its use may      //
//infringe existing patents.                                                //
//The original developer of this software module and his/her company, the   //
//subsequent editors and their companies, and ISO/IEC have no liability     //
//for use of this software module or modifications thereof in an            //
//implementation. Copyright is not released for non MPEG-4 Systems          //
//(ISO/IEC 14496-1) conforming products. CSELT S.pA. retains full right     //
//to use the code for his/her own purpose, assign or donate the code to a   //
//third party and to inhibit third parties from using the code for non      //
//MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright      //
//notice must be included in all copies or derivative works.                //
//Copyright  1999.                                                         //
//This work has been partly developed within the framework of the ESPRIT    //
//European Research Project  "MPEG-4 PC".                                   //
//////////////////////////////////////////////////////////////////////////////
//
// File:  OrderedGroupProxy.cpp
//
// Author:  Gianluca Di Cagno  gianluca.dicagno@cselt.it
//
// Modified:
//
// Description:
//   The implementation of the proxy object used by the presenter
//   to handle the OrderedGroup node.
//
// Created: December 1998.
//
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include "MediaObj.h"
#include "Presenter.h"
#include "nodes.h"
#include "OrderedGroupProxy.h"
#include "DrawableNode.h"
#include "trace.h"


static int ComparePriority(const void* elem1, const void* elem2)
{
	priorityPosition * child1 = (priorityPosition *)elem1;
	priorityPosition * child2 = (priorityPosition *)elem2;

	// Compare drawing orders
	if (child1->priority < child2->priority)
		return -1;
	else if (child1->priority > child2->priority)
		return +1;
	else return 0;

}
// =====================================================================

//
// Presenter::InitOrderedGroup
//
// Initialise the proxy data structure and attach it to the render data
// pointer of the given media object node.
//

OrderedGroupProxy* Presenter::InitOrderedGroup(OrderedGroup* pObj)
{
	return new  OrderedGroupProxy(pObj);
}

// =====================================================================

//
// OrderedGroupProxy::~OrderedGroupProxy
//
//  

OrderedGroupProxy::~OrderedGroupProxy()
{
	if(m_priorityPositionList!=NULL)
		{
			delete[] m_priorityPositionList;	
			m_priorityPositionList=NULL;
			delete[] m_previousPriorityPositionList;
		}
}

// =====================================================================

//
// OrderedGroupProxy::PreRender
//
// PreRender the children in the order specified by the OrderedGroup
// order field.
// 
//

void OrderedGroupProxy::PreRender(Effects &effects)
{
	DWORD dwUpdateTime;
	int i;

	
	
	int length= GetObject()->GetChildCount();

	if(GetOrderedGroup()->order.GetLength()==0)
	{
		PreRenderChildren(effects);
		return;
	}

	//check whether the OrderedGroup node has changed 
	
	if((dwUpdateTime=GetObject()->GetUpdateTime())!=m_LastUpdateTime)
	{
		m_LastUpdateTime=dwUpdateTime;
		TRACE_PRINT1("ORDER TIME UPDATE %d",dwUpdateTime);
		if(m_priorityPositionList!=NULL)
		{
			delete[] m_previousPriorityPositionList;
			m_previousPriorityPositionList=m_priorityPositionList;
		/*	delete[] m_priorityPositionList;	
			m_priorityPositionList=NULL;*/
		}
		
		m_priorityPositionList = new priorityPosition[length];
		for(i=0;i<length;i++)
		{
			(*(m_priorityPositionList+i)).position=i;
			(*(m_priorityPositionList+i)).priority=GetOrderedGroup()->order[i];
		}
		effects.orderHasChanged=TRUE;
		
		qsort(m_priorityPositionList,length,sizeof(priorityPosition),ComparePriority);
	}
	
	// PreRender the children, in the new order.
	for ( i = 0; i <length; i++)
	{
		Traversable * pTrav= GetChild((*(m_priorityPositionList+i)).position);
		if (pTrav)
			if(pTrav->IsSensor())
				effects.m_pHandler=(class UserEventHandlerProxy *)pTrav;
	}

	for( i=0;i<length;i++)
	{
//		if((*(m_previousPriorityPositionList+i)).priority!=(*(m_priorityPositionList+i)).priority)
			Traversable * pTrav= GetChild((*(m_priorityPositionList+i)).position);
			if(pTrav)
				pTrav->PreRender(effects);
			
	}
	
	effects.orderHasChanged=FALSE;
} 





// =====================================================================
