//This software module was originally developed by CSELT in the course of   //
//development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.             //
//This software module is an implementation of a part of one or more MPEG-4 //
//Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems        //
//(ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 Systems     //
//(ISO/IEC 14496-1) free license to this software module or modifications   //
//thereof for use in hardware or software products claiming conformance to  //
//the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use this software//
//module in hardware or software products are advised that its use may      //
//infringe existing patents.                                                //
//The original developer of this software module and his/her company, the   //
//subsequent editors and their companies, and ISO/IEC have no liability     //
//for use of this software module or modifications thereof in an            //
//implementation. Copyright is not released for non MPEG-4 Systems          //
//(ISO/IEC 14496-1) conforming products. CSELT S.pA. retains full right     //
//to use the code for his/her own purpose, assign or donate the code to a   //
//third party and to inhibit third parties from using the code for non      //
//MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright      //
//notice must be included in all copies or derivative works.                //
//Copyright  1999.                                                         //
//This work has been partly developed within the framework of the ESPRIT    //
//European Research Project  "MPEG-4 PC".                                   //
//////////////////////////////////////////////////////////////////////////////
//
// File:  PlaneSensor2DProxy.cpp
//
// Author: Gianluca Di Cagno, CSELT, Gianluca.Dicagno@cselt.it
//
// Modified:
//
// Description:
//   The implementation of the proxy object used by the presenter
//   to handle the PlaneSensor2D node.
//
// Created: December, 1997.
//
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include "Nodes-CSELT.h"
#include "PlaneSensor2DProxy.h"
#include "trace.h"


// Storage for this static data member
BOOL PlaneSensor2DProxy::m_movementGrabbed = FALSE;


// =====================================================================

//
// Presenter::InitPlaneSensor2D
//
// Initialise the proxy data structure and attach it to the render data
// pointer of the given media object node.
//

PlaneSensor2DProxy* Presenter::InitPlaneSensor2D(PlaneSensor2D* pObj)
{
	return new PlaneSensor2DProxy(pObj);  
}




// =====================================================================

//
// PlaneSensor2DProxy::OnUserEvent
//
// This function is called when a user event has occurred above a node
// which is in the same grouping node hierarchy as this sensor node.
//
// If this node is registered with the presenter to received events,
// it will be called a second time for this scene draw.
//
// If this plane sensor is enabled, then generate the required events
// and return TRUE, otherwise return FALSE.
//
#ifdef ppp
BOOL PlaneSensor2DProxy::OnUserEvent(EventInfo const* pEvent)
{
	TRACE_PRINT("In PlaneSensor2DProxy::OnUserEvent():\n");

	// Only generate events if this node is enabled, and no other
	// plane sensor node has grabbed all motion events
	if (((BOOL)(Bool)GetPlaneSensor2D()->enabled) &&
		((!IsMovementGrabbed()) ||
		((IsMovementGrabbed()) && (m_mouseDown))))
	{
		
	  // this is something like a fallback,
	  // a piece of code that is executed if 
	  // this function is called when the mouse is not on a object
	  
	  if ((pEvent->pass == 2) && (m_prevPass == 2))
		{
		 if (pEvent->m_event == EVENT_MOUSE_UP)
	      {
			((Presenter*)GetPresenter())->UnregisterEventHandler(this);
		    m_registered = FALSE;
			if(((Presenter*)GetPresenter())->GetDrawableTouched()!=NULL)
			 {
				((Presenter*)GetPresenter())->SetIsTracking(FALSE);
	            ((Presenter*)GetPresenter())->GetDrawableTouched()->m_isBeingTracked=FALSE;
			 }
			SetGrabbedOff();
			
		    m_mouseDown = FALSE;
		    return TRUE;
		  }
		 if ((pEvent->m_event == EVENT_MOUSE_MOVE) && (m_mouseDown))
		 {
				
			SetGrabbedOn();
			m_mouseDown = TRUE;
			
			

    	    if(((Presenter*)GetPresenter())->GetDrawableTouched()!=NULL)
			 {
			  ((Presenter*)GetPresenter())->SetIsTracking(TRUE);
			  ((Presenter*)GetPresenter())->GetDrawableTouched()->m_isBeingTracked=TRUE;
			 }
			Vec2f point(pEvent->m_x, pEvent->m_y);
	   		
			GetPlaneSensor2D()->trackPoint_changed = point;
			
			m_pointStartDrag=point;
			Vec2f tmpPoint;
			tmpPoint.x=point.x-m_pointStartDrag.x+((Vec2f)(GetPlaneSensor2D()->offset)).x;
			tmpPoint.y=point.y-m_pointStartDrag.y+((Vec2f)(GetPlaneSensor2D()->offset)).y;

			GetPlaneSensor2D()->translation_changed = tmpPoint;

		}

		 if (!m_mouseDown)
			{
				((Presenter*)GetPresenter())->UnregisterEventHandler(this);
				m_registered = FALSE;
			}
		 return TRUE;
	 }
   	// when the mouse button is first pressed change
	// the state of the sensor

	  if ((pEvent->m_event == EVENT_MOUSE_DOWN) && (!m_mouseDown))
	   {
		
	    	SetGrabbedOn();
			m_mouseDown = TRUE;
    	//	Vec2f point(pEvent->m_x, pEvent->m_y);
	    //	GetPlaneSensor2D()->trackPoint_changed = point;


	   }

	// set m_mouse down FALSE  if the button returns up

	  if (pEvent->m_event == EVENT_MOUSE_UP)
	    {
		((Presenter*)GetPresenter())->UnregisterEventHandler(this);
		    m_registered = FALSE;
			SetGrabbedOff();
		    
    		// this is needed because this function is called even if
			// you click on a background where there isn't an object.

			if(((Presenter*)GetPresenter())->GetDrawableTouched()!=NULL)
			 {
			
			//the tracking has finished signal this to the presenter

		    	((Presenter*)GetPresenter())->SetIsTracking(FALSE);
			
            // the drawing order of the object touched must return to normal

			    ((Presenter*)GetPresenter())->GetDrawableTouched()->m_isBeingTracked=FALSE;
			 }
			
			m_mouseDown = FALSE;
			

            return TRUE;
		}

	// Generate a trackPoint_changed event if mouse moves after the
	// mouse has been clicked. 
        
 	  if ((pEvent->m_event == EVENT_MOUSE_MOVE) && (m_mouseDown))
		{
				
			SetGrabbedOn();
			// First we set the mousedown flag
			m_mouseDown = TRUE;
			
			// this is needed because this function is called even if
			// you click on a background where there isn't an object.
			
			if(((Presenter*)GetPresenter())->GetDrawableTouched()!=NULL)
			  {
			// then we signal to the presenter not look 
			// which object the user has touched (because we know)

			   ((Presenter*)GetPresenter())->SetIsTracking(TRUE);
		   
		   // lastly if one object is being tracked we set the drawable node attribute
		   // m_isBeingtracked to TRUE, in  this way the drawing order of the 
		   // object increases and you can understand where are you tracking the object.
		
			   ((Presenter*)GetPresenter())->GetDrawableTouched()->m_isBeingTracked=TRUE;
			  }
			
			// Now we can produce the event point changed
			Vec2f point(pEvent->m_x, pEvent->m_y);
	   		GetPlaneSensor2D()->trackPoint_changed = point;
			
			
			Vec2f tmpPoint;
			tmpPoint.x=point.x-m_pointStartDrag.x+((Vec2f)(GetPlaneSensor2D()->offset)).x;
			tmpPoint.y=point.y-m_pointStartDrag.y+((Vec2f)(GetPlaneSensor2D()->offset)).y;

			GetPlaneSensor2D()->translation_changed = tmpPoint;
			

		}

	// If not already done, register this node with the presenter
	  ((Presenter*)GetPresenter())->RegisterEventHandler(this);
	  m_registered = TRUE;
	  m_prevPass = pEvent->pass;
	  return TRUE;
     }

	// Otherwise, if not enabled, return FALSE
	else
		return FALSE;
} /* PlaneSensor2DProxy::OnUserEvent */
#endif


BOOL PlaneSensor2DProxy::OnUserEvent(EventInfo const* pEvent)
{
	TRACE_PRINT("In PlaneSensor2DProxy::OnUserEvent():\n");

	// Only generate events if this node is enabled, and no other
	// plane sensor node has grabbed all motion events
	if (((BOOL)(Bool)GetPlaneSensor2D()->enabled) &&
		((!IsMovementGrabbed()) ||
		((IsMovementGrabbed()) && (m_mouseDown))))
	{
		
	   
	  // this function is called when the mouse is not on a object
	  
	  if ((pEvent->pass == 2) && (m_prevPass == 2))
		{
		 if (pEvent->m_event == EVENT_MOUSE_UP)
	      {
			((Presenter*)GetPresenter())->UnregisterEventHandler(this);
		    m_registered = FALSE;
			if(((Presenter*)GetPresenter())->GetDrawableTouched()!=NULL)
			 {
				((Presenter*)GetPresenter())->SetIsTracking(FALSE);
	            ((Presenter*)GetPresenter())->GetDrawableTouched()->m_isBeingTracked=FALSE;
			 }
			SetGrabbedOff();
			
		    m_mouseDown = FALSE;
		  }

		 if (!m_mouseDown)
		 {
				((Presenter*)GetPresenter())->UnregisterEventHandler(this);
				m_registered = FALSE;
		 }
		 return TRUE;
	  }
   	// when the mouse button is first pressed change
	// the state of the sensor

	  if ((pEvent->m_event == EVENT_MOUSE_DOWN) && (!m_mouseDown))
	   {
		
	    	SetGrabbedOn();
			m_mouseDown = TRUE;
			Vec2f point(pEvent->m_x, pEvent->m_y);
			m_pointStartDrag=point;
		}

	// set m_mouse down FALSE  if the button returns up

	  if (pEvent->m_event == EVENT_MOUSE_UP)
	    {
			((Presenter*)GetPresenter())->UnregisterEventHandler(this);
		    m_registered = FALSE;
			SetGrabbedOff();
		    
    		// this is needed because this function is called even if
			// you click on a background where there isn't an object.

			if(((Presenter*)GetPresenter())->GetDrawableTouched()!=NULL)
			 {
			
			//the tracking has finished signal this to the presenter

		    	((Presenter*)GetPresenter())->SetIsTracking(FALSE);
			
            // the drawing order of the object touched must return to normal

			    ((Presenter*)GetPresenter())->GetDrawableTouched()->m_isBeingTracked=FALSE;
			 }
			
			m_mouseDown = FALSE;
			
			//update offset field if autoOffset field is enabled

			if((Bool)(GetPlaneSensor2D()->autoOffset))
			{
				Vec2f point(pEvent->m_x, pEvent->m_y);
				Vec2f tmpPoint;
				tmpPoint.x=point.x-m_pointStartDrag.x+((Vec2f)(GetPlaneSensor2D()->offset)).x;
				tmpPoint.y=point.y-m_pointStartDrag.y+((Vec2f)(GetPlaneSensor2D()->offset)).y;
				GetPlaneSensor2D()->offset=tmpPoint;
			}
            return TRUE;
		}

	// Generate a trackPoint_changed event and translation changed
	// if mouse moves after the mouse has been clicked. 
        
 	  if ((pEvent->m_event == EVENT_MOUSE_MOVE) && (m_mouseDown))
		{
				
			SetGrabbedOn();
		
			// then we signal to the presenter not look 
			// which object the user has touched (because we know)

			((Presenter*)GetPresenter())->SetIsTracking(TRUE);
		   /*
		   //to remove start
		   // lastly if one object is being tracked we set the drawable node attribute
		   // m_isBeingtracked to TRUE, in  this way the drawing order of the 
		   // object increases and you can understand where are you tracking the object.
		
			   ((Presenter*)GetPresenter())->GetDrawableTouched()->m_isBeingTracked=TRUE;
			
			// to remove end
			*/
			// Now we can produce the event trackpoint changed (raw output)
			Vec2f point(pEvent->m_x, pEvent->m_y);
	   		GetPlaneSensor2D()->trackPoint_changed = point;
			
			
			// calculate translation_changed event (relative output) from:
			// m_pointStartDrag (start dragging point in screen coordinates)
			// point (current tracking  point in screen coordinates)
			// PlaneSensor2D offset 

			Vec2f tmpPoint;
			tmpPoint.x=point.x-m_pointStartDrag.x+((Vec2f)(GetPlaneSensor2D()->offset)).x;
			tmpPoint.y=point.y-m_pointStartDrag.y+((Vec2f)(GetPlaneSensor2D()->offset)).y;

			// check for bounds for the output 	
			if(((Vec2f)(GetPlaneSensor2D()->minPosition)).x<((Vec2f)(GetPlaneSensor2D()->maxPosition)).x)
			{
				if(tmpPoint.x<((Vec2f)(GetPlaneSensor2D()->minPosition)).x)
					tmpPoint.x=((Vec2f)(GetPlaneSensor2D()->minPosition)).x;
				if(tmpPoint.x>((Vec2f)(GetPlaneSensor2D()->maxPosition)).x)
					tmpPoint.x=((Vec2f)(GetPlaneSensor2D()->maxPosition)).x;
			}
			else if((((Vec2f)(GetPlaneSensor2D()->minPosition)).x==((Vec2f)(GetPlaneSensor2D()->maxPosition)).x))
			{
				tmpPoint.x= ((Vec2f)(GetPlaneSensor2D()->minPosition)).x;
			}
			
			if(((Vec2f)(GetPlaneSensor2D()->minPosition)).y<((Vec2f)(GetPlaneSensor2D()->maxPosition)).y)
			
			{
				if(tmpPoint.y<((Vec2f)(GetPlaneSensor2D()->minPosition)).y)
					tmpPoint.y=((Vec2f)(GetPlaneSensor2D()->minPosition)).y;
				if(tmpPoint.y>((Vec2f)(GetPlaneSensor2D()->maxPosition)).y)
					tmpPoint.y=((Vec2f)(GetPlaneSensor2D()->maxPosition)).y;
			}
			
			else if((((Vec2f)(GetPlaneSensor2D()->minPosition)).y==((Vec2f)(GetPlaneSensor2D()->maxPosition)).y))
			{
				tmpPoint.y= ((Vec2f)(GetPlaneSensor2D()->minPosition)).y;
			}
			// finally produce the event translation changed 

			GetPlaneSensor2D()->translation_changed = tmpPoint;
			
		}

	// If not already done, register this node with the presenter
	  ((Presenter*)GetPresenter())->RegisterEventHandler(this);
	  m_registered = TRUE;
	  m_prevPass = pEvent->pass;
	  return TRUE;
     }

	// Otherwise, if not enabled, return FALSE
	else
		return FALSE;
} /* PlaneSensor2DProxy::OnUserEvent */


// =====================================================================

//
// PlaneSensor2DProxy::IsFinished
//
// A PlaneSensor2D node is considered as finished when it is not enabled.
//

BOOL PlaneSensor2DProxy::IsFinished() const
{
	return !((BOOL)(Bool)GetPlaneSensor2D()->enabled);
} /* PlaneSensor2DProxy::IsFinished */


// ========================================================================
