//This software module was originally developed by CSELT in the course of   //
//development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.             //
//This software module is an implementation of a part of one or more MPEG-4 //
//Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems        //
//(ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 Systems     //
//(ISO/IEC 14496-1) free license to this software module or modifications   //
//thereof for use in hardware or software products claiming conformance to  //
//the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use this software//
//module in hardware or software products are advised that its use may      //
//infringe existing patents.                                                //
//The original developer of this software module and his/her company, the   //
//subsequent editors and their companies, and ISO/IEC have no liability     //
//for use of this software module or modifications thereof in an            //
//implementation. Copyright is not released for non MPEG-4 Systems          //
//(ISO/IEC 14496-1) conforming products. CSELT S.pA. retains full right     //
//to use the code for his/her own purpose, assign or donate the code to a   //
//third party and to inhibit third parties from using the code for non      //
//MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright      //
//notice must be included in all copies or derivative works.                //
//Copyright  1999.                                                         //
//This work has been partly developed within the framework of the ESPRIT    //
//European Research Project  "MPEG-4 PC".                                   //
//////////////////////////////////////////////////////////////////////////////
//
// File: Presenter.cpp
//
// Author: Matthew Leditschke
//
// Modified: Gianluca Di Cagno  gianluca.dicagno@cselt.it
//
// Description:
//   The implementation of a class which represents a Presenter,
//   composed of a visual and audio rendering engine.
//
// Created: April, 1997
//
//
///////////////////////////////////////////////////////////////////////////

/*-------------------------------------------------------------------------
MPEGJ
AUTHORS:    Viswanathan Swaminathan - vishy@eng.sun.com
			Robert Schmidt(RLS)
			Praveen Alavilli(PA)
			YUNTAI KYONG(YT) - yuntaikyong@ieee.org
-------------------------------------------------------------------------*/

#include "defs.h"
#include <assert.h>
#include "Presenter.h"
#include "Executiv.h"
#include "MediaObj.h"
#include "Caps.h"
#include "Clock.h"
#include "trace.h"
#include "Nodes-CSELT.h"
#include "DrawableNode.h"
#include "ThreadList.h"
#include "UserEventHandlerProxy.h"
//handle unused open streams
#include "MovieTextureProxy.h"
#include "SynthesizedTextureProxy.h"

#ifdef	  STAND_ALONE
#include "mpeg4player.h"
#include "MainFrm.h"
#elif defined(ACTIVEX_COMPONENT)
#include "Mpeg4Cselt2DActivex.h"
#elif defined(NETSCAPE_PLUGIN)
#endif




FILE* traceFile;



// ========================================================================

//
// Presenter::Presenter
//
// Class constructor
//  - instantiates the visual and audio renderers
//

Presenter::Presenter(Executive* pExecutive, HWND hWnd)
  : PresenterFlex (pExecutive, hWnd)
{
	TRACE_PRINT("Entering Presenter::Presenter()\n");
    m_isTracking=FALSE;
	// We need to do the following to make our nodes derived from those in
	// Nodes.h work
	OVERLOAD_NODE(AnchorCSELT);
	OVERLOAD_NODE(AppearanceCSELT);
	OVERLOAD_NODE(AudioSourceCSELT);
	OVERLOAD_NODE(Background2DCSELT);
	OVERLOAD_NODE(BitmapCSELT);
	OVERLOAD_NODE(CircleCSELT);
	OVERLOAD_NODE(ConditionalCSELT);
	OVERLOAD_NODE(ColorCSELT);
	OVERLOAD_NODE(ColorInterpolatorCSELT);
	OVERLOAD_NODE(Coordinate2DCSELT);
	OVERLOAD_NODE(Curve2DCSELT);
	OVERLOAD_NODE(DiscSensorCSELT);
	OVERLOAD_NODE(FontStyleCSELT);
	OVERLOAD_NODE(FormCSELT);
	OVERLOAD_NODE(GroupCSELT);
	OVERLOAD_NODE(IndexedLineSet2DCSELT);
	OVERLOAD_NODE(InlineCSELT);
	OVERLOAD_NODE(ImageTextureCSELT);
	OVERLOAD_NODE(Layer2DCSELT);
	OVERLOAD_NODE(LayoutCSELT);
	OVERLOAD_NODE(LinePropertiesCSELT);
	OVERLOAD_NODE(Material2DCSELT);
	OVERLOAD_NODE(MatteTextureCSELT);
	OVERLOAD_NODE(MovieTextureCSELT);
	OVERLOAD_NODE(OrderedGroupCSELT);
	OVERLOAD_NODE(PointSet2DCSELT);
	OVERLOAD_NODE(PositionInterpolator2DCSELT);
	OVERLOAD_NODE(PlaneSensor2DCSELT);
    OVERLOAD_NODE(ProximitySensor2DCSELT);
	OVERLOAD_NODE(RectangleCSELT);
	OVERLOAD_NODE(ScalarInterpolatorCSELT);
	OVERLOAD_NODE(ScriptCSELT);
	OVERLOAD_NODE(ShapeCSELT);
	OVERLOAD_NODE(Sound2DCSELT);
	OVERLOAD_NODE(SwitchCSELT);
  OVERLOAD_NODE(TemporalTransformCSELT); //PAB+:
  OVERLOAD_NODE(TemporalGroupCSELT); //PAB+:
	OVERLOAD_NODE(TextCSELT);
	OVERLOAD_NODE(TermCapCSELT);
	OVERLOAD_NODE(TimeSensorCSELT);
	OVERLOAD_NODE(TouchSensorCSELT);
	OVERLOAD_NODE(Transform2DCSELT);
	OVERLOAD_NODE(ValuatorCSELT);
	OVERLOAD_NODE(WorldInfoCSELT);
	OVERLOAD_NODE(SynthesizedTextureCSELT);


	m_bActive = FALSE;
	m_pScene = NULL;
	m_actualFrameRate = 0;
	m_currentTime = 0;
	m_scenesDrawnSinceStart = 0;
	// The default frame rate is 15 frames per second
	m_frameRate = 15;
	m_newFrameRate = 15;
	m_bPaused=FALSE;
	// Remember the pointer to the Executive
	m_pExecutive = pExecutive;

	// Instantiate the renderers
	m_pVisualRenderer = VisualRenderer::Create(this, (void*)hWnd);
	m_pAudioRenderer  = AudioRenderer::Create(this, (void*)hWnd);
    //added for Mike animation.
//	m_fLogFile = fopen ("log.txt", "w");
	TRACE_PRINT("Leaving Presenter::Presenter()\n");

	#ifdef MPGJ
	// MPEGJ - MERGE
	/***** load function table */
	di = MPJGetTerminalDispatchTable();
	// END - MPEGJ - PA
	#endif // MPEGJ

} /* Presenter::Presenter */


// ========================================================================

//
// Presenter::~Presenter
//
// Class destructor - calls terminate if it hasn't already been called.
//

Presenter::~Presenter()
{
	TRACE_PRINT("Entering Presenter::~Presenter()\n");

	// Terminate the presenter (just to make sure that it is done)
	if (m_bActive)
		Terminate();

	// Delete the visual and audio renderers
	delete m_pVisualRenderer;
	delete m_pAudioRenderer;

//	fclose (m_fLogFile);
	TRACE_PRINT("Leaving Presenter::~Presenter()\n");
} /* Presenter::~Presenter */


// ========================================================================

//
// Presenter::GetCapabilities
//
// Combine the capabilities of the video and audio renderers.
//

void Presenter::GetCapabilities (Capabilities* pCaps)
{
	int i, j;
	CapabilitiesImp caps;

	// Get the capabilities of the two renderers
	m_pVisualRenderer->GetCapabilities(pCaps);
	m_pAudioRenderer->GetCapabilities(&caps);

	// Combine the capabilities into a single object
	j = pCaps->GetFormatCapabilityLength();
	for (i = 0; i < caps.GetFormatCapabilityLength(); i++)
		pCaps->AddFormatCapability(caps.GetFormatCapability(i), j++);
} /* Presenter::GetCapabilities */


// ========================================================================

//
// Presenter::Init
//
//
// This method will be called only once when the browser comes up.
//

void Presenter::Init(/*MediaObject* pScene*/)
{
	TRACE_PRINT("Entering Presenter::Init()\n");

	// The default effects given to PreRender()
	m_defEffects.transform.SetIdentity();
	m_defEffects.isSwitchedOn = TRUE;
	m_defEffects.m_pHandler=NULL;
	m_defEffects.m_ParentGroup=NULL;
	m_defEffects.orderHasChanged=FALSE;
	m_defEffects.m_pAppearance=NULL;
	m_defEffects.m_pShapeProxy=NULL;
	m_defEffects.scaleX=1;
	m_defEffects.scaleY=1;

	TRACE_PRINT("Leaving Presenter::Init()\n");
	// Call the method in the base class
	PresenterBase::Init();

} /* Presenter::Init */

// ========================================================================

//
// Presenter::Start
//
//
// This method will be called once everytime a session starts.
//

void Presenter :: Start ()
{
	// Remember the pointer to the root scene object
	m_pScene = GetScene();
	int height,width;

	//get BIFS Author scene size
	height=((Root *)m_pScene)->GetPixelHeight();
	width=((Root *)m_pScene)->GetPixelWidth();
	if(height!=0&&width!=0)
	{
		#ifdef STAND_ALONE
		LPARAM lparam =MAKELPARAM(width,height);
		GetMainFrame()->PostMessage(WM_SETSIZE,NULL,lparam);
		#endif
	}
	//we can only handle pixel metric, sorry
//	ASSERT(((Root *)m_pScene)->IsPixelMetric());

	// Flag that the presenter is now active
	m_bActive = TRUE;
	// Initialise the renderers
	m_pVisualRenderer->Init();
	m_pAudioRenderer->Init();
	m_scenesDrawnSinceRateChage = 0;

	m_startingTime = timeGetTime();
	m_sceneUpdatePeriod = 1000.0 / m_frameRate;
	ResetFrameRateCalculation();
	m_scenesDrawnSinceStart = 0;
	AskScreenAccess();
	m_pVisualRenderer->Clear();
	m_pVisualRenderer->Paint();
	ReleaseScreenAccess();
	PresenterBase::Start();
}	/* Presenter::Start */
// ========================================================================

//
// Presenter::Terminate
//
// Terminates the visual and audio rendering, performs cleanups.
// This method will be called once for every file that is closed.
//

void Presenter::Terminate()
{
	TRACE_PRINT("Entering Presenter::Terminate():\n");

	// Flag that the presenter is no longer active
	m_bActive = FALSE;

	//restore currentTime to initial value for the next scene
	m_currentTime = 0;

	// Wait for this thread to terminate
	m_mtxThread.P ();
	m_mtxThread.V ();

	// Stop the visual and audio renders
	m_pVisualRenderer->Terminate();
	m_pAudioRenderer->Terminate();

	//Added for the correct restart
	m_eventHandlersArray.Clear();
	for (int i = m_eventArray.GetLength (); --i >= 0;)
		delete m_eventArray[i];

//	m_eventArray.Clear();
	// Call the method in the base class
	PresenterBase::Terminate();

	TRACE_PRINT("Leaving Presenter::Terminate():\n");
} /* Presenter::Terminate */


// ========================================================================

//
// Presenter::OnUserEvent
//
// Called by the executive when a user event occurs.  Add the details of
// this event to the list of events that have arrived to be process,
// since the scene was last drawn.
//
// This function will be called by the thread which is running the frame
// application.
//

void Presenter::OnUserEvent(USER_EVENT event, int x, int y,
								 UINT controlKeysStatus)
{
	// Only deal with events when the presenter is active
	if (m_bActive)
	{
		TRACE_PRINT2("Pres: Received event at (%d,%d)\n", x, y);

		m_mtxEventArray.P();
		m_eventArray += new EventInfo(event, x, y, controlKeysStatus);
		m_mtxEventArray.V();
	}
} /* Presenter::OnUserEvent */


// ========================================================================

//
// Presenter::RegisterEventHandler
//
// Add the given UserEventHandlerProxy node to the list
// of nodes which need to be informed of user events.
//

void Presenter::RegisterEventHandler(UserEventHandlerProxy* pNode)
{
	m_mtxEventHandlersArray.P();

	// Only add this node if it is not already in the list
	BOOL inList = FALSE;
	for (int i = 0; i < m_eventHandlersArray.GetLength(); i++)
		if (m_eventHandlersArray[i] == pNode)
			inList = TRUE;

	if (!inList)
		m_eventHandlersArray += pNode;

	m_mtxEventHandlersArray.V();
} /* Presenter::RegisterEventHandler */


// ========================================================================

//
// Presenter::UnregisterEventHandler
//
// Remove the given UserEventHandlerProxy node from the
// list of nodes which need to be informed of user events.
//

void Presenter::UnregisterEventHandler(UserEventHandlerProxy* pNode)
{
	m_mtxEventHandlersArray.P();
	m_eventHandlersArray -= pNode;
	m_mtxEventHandlersArray.V();
} /* Presenter::UnregisterEventHandler */


// ========================================================================

//
// Presenter::ProcessEvents
//
// Go through all of the events that are in the event array, and for each
// event do the following:
//   - Translate (x,y) to BIFS coordinates
//   - Go through the list of drawable nodes, looking for the first which
//     contains the given point.
//   - For the found node, search for a TouchSensor node to handle this
//     event.
//
// This function will be called by the thread which is running the
// presenter.
//

void Presenter::ProcessEvents()
{
	m_mtxEventArray.P();

	while (m_eventArray.GetLength() > 0)
	{
		// Get the first event
		EventInfo* pEvent = m_eventArray[0];
		pEvent->pass = 1;
//check for pause or resume special messages

		TRACE_PRINT2("Pres: Processing event at (%d,%d)\n",
					 pEvent->m_x, pEvent->m_y);

		// Translate the coordinates from screen to BIFS
#ifdef ORIGIN_CENTER
		pEvent->m_x -= m_pVisualRenderer->GetScreenWidth() / 2;
		pEvent->m_y  = m_pVisualRenderer->GetScreenHeight() / 2 - pEvent->m_y;
#else

 	    pEvent->m_y  = m_pVisualRenderer->GetScreenHeight()  - pEvent->m_y;
#endif
		// Look through the drawable nodes for a node under this point

		//This is a patch nothing more by now, this part of code  is very
		//critical because when you track an object, then the mouse happens that
		//points on another node, this cause the FindNode function to change drawable
		//then the result is that if the node has IsLeftObject property different this
		//causes a problem for the pEvent m_x m_y members.
		//DrawableNode *  pDrawable;

		//only the PlaneSensor2D will set this.
		//if no object is being tracked, find which object has been touched
		DrawableContext *pContext=NULL;

		if(!GetIsTracking())
		{
			pContext= m_pVisualRenderer->FindNode(pEvent->m_x, pEvent->m_y);
        	if(pContext)
				m_pDrawableTouched=	(DrawableNode *)pContext->GetDrawableProxy();
			m_pTrackingContext=pContext;
		}
		else
			pContext=m_pTrackingContext;

		if (pContext != NULL)
		 {

			 // We need to know if the node found is a geometric or a
		     // video node because they are drawn in different ways:
		     // geometric needs a center, video needs a top corner.
#ifndef ORIGIN_CENTER
			 if(m_pDrawableTouched->IsLeftTopObject())
				{

			        pEvent->m_x -= m_pDrawableTouched->GetBoundingRect()->width/2;
					pEvent->m_y += m_pDrawableTouched->GetBoundingRect()->height/2;


				}
#endif
			  // If a node has been found, look for a sensor node to handle
		      // this event
			 UserEventHandlerProxy *m_pHandler=pContext->GetUserEventHandler();

			 if(m_pHandler)
				 m_pHandler->OnUserEvent(pEvent);

			// m_pDrawableTouched->FindTouchSensor(pEvent,pContext);
	      }

		// Now start a delicate pass, pass 2 that can be called without pass 1!!

		// Call the OnUserEvent method of all nodes which have registered
		pEvent->pass = 2;
		for (int i = 0; i < m_eventHandlersArray.GetLength(); i++)
			m_eventHandlersArray[i]->OnUserEvent(pEvent);

        // We have finished with this event, remove it from the array
		m_eventArray.DeleteFrom(0);
		delete pEvent;
	}

	m_mtxEventArray.V();
} /* Presenter::ProcessEvents */

#ifdef MPGJ
/* MPEGJ-Merge - PA*/
/********** code added to save prevous data  *****/
unsigned int frames_till_lastcall = 0;
unsigned int time_at_lastcall = 0;
// END - MPEGJ
#endif



// =====================================================================

//
// Presenter::Render
//
// This is the main loop of the presenter thread, which is called by
// main loop  in the base class.
//



void Presenter::Render()
{
	// Mark the thread as running
	m_mtxThread.P();

//	TRACE_PRINT("Entering Presenter::Main():\n");

	// This loop runs while the presenter is active, which is up to
	// either the next Terminate() call, or all nodes have finished.

	while (m_bActive) // m_bActive
	{

		PresenterDrivenEvents(); //PAB+:

		// Get a pointer to the current root proxy object (will it change?)
		Traversable* pRootProxy =(Traversable*) GetScene()->GetRenderingObject();

		// If the frame rate has changed, reset the timing related variables.
 		if (m_frameRate != m_newFrameRate)
		{
//			TRACE_PRINT2("Changing frame rate from %.2f to %.2f\n",
//						 m_frameRate, m_newFrameRate);
			m_frameRate = m_newFrameRate;
			m_sceneUpdatePeriod = 1000.0 / m_frameRate;
			m_scenesDrawnSinceRateChage = 0;
		  	m_startingTime = timeGetTime();
		}

        m_currentTime = timeGetTime() - GetStartTime();

#ifdef STAND_ALONE
		if (m_scenesDrawnSinceStart == 0)
			GetApp()->OnConnect(RESPONSE_OK);
#endif
		// Find the nodes which have had user events occur above them
		ProcessEvents();

		//if we are in pause mode don't traverse the tree
		if(m_bPaused)
			continue;

		GetExecutive () -> LockScene ();

    // #GB: presenter used to have its own time, we now use the core one
    PresenterContext pcCurrentTimeContext;
    pcCurrentTimeContext.SetTime(GetScene()->GetSceneTime());
    // #GB: Propagate the new FlexTime in the scene graph
    pRootProxy->SetFlexTime(pcCurrentTimeContext);

		// Update the TimeSensor nodes in the scene
    	pRootProxy->UpdateTimeSensor();

		// Render the scene
		TRACE_PRINT("Before scene PreRender.\n");

		m_mtxScreenAccess.P();

		m_pVisualRenderer->ResetDrawList();

		pRootProxy->PreRender(m_defEffects);
		GetExecutive()->ActivateRoutes ();
		GetExecutive()->RequestChannels();

		TRACE_PRINT("After scene PreRender, before scene draw.\n");

		CheckUnusedOpenStreams();
		CheckUnusedOpenSynthesizedTextureStreams();

		// If this is the first time that the scene has been rendered,
		// then we need to start the audio renderer

		m_pVisualRenderer->DrawScene();

		m_mtxScreenAccess.V();

		m_scenesDrawnSinceStart++;
		TRACE_PRINT("After scene draw.\n");

		// If this is the first time that the scene has been rendered,
		// then we need to start the audio renderer
		if (m_scenesDrawnSinceStart == 1)
		{
			m_pAudioRenderer->Start();

	 		}
		// If all of the nodes in the scene have finished, then the
		// Presenter is also finished.
	/*	if (pRootProxy->IsFinished())
		{
			TRACE_PRINT("Pres: The scene is finished.\n");
			m_bActive = FALSE;
			break;
		} */

		// Determine how long to wait before starting the next composition
		//  - do this by determining the number of clock ticks which have
		//    elapsed since starting, and the number of the clock tick counter
		//    when the next composition should start.
		//
		// The time from the start of the thread is used instead of the time
		// from the start of this composition, to get more accurate timing,
		// avoiding jitter in small scale time readings.
		m_scenesDrawnSinceRateChage++;
		//Paris ?
		unsigned int elapsedTC = timeGetTime() - m_startingTime;
		unsigned int startNextTC = m_scenesDrawnSinceRateChage * (unsigned int)m_sceneUpdatePeriod;

		// Determine the current actual frame rate.
		UpdateFrameRateCalculation(elapsedTC);

		// Only sleep if ahead of schedule
		TRACE_PRINT1("Pres: Sleeping for %d ms.\n", startNextTC - elapsedTC);
		GetExecutive () -> UnlockScene ();

		if (startNextTC > elapsedTC)
			Sleep(startNextTC - elapsedTC);

		#ifdef MPGJ
		/********MPEGJ_ Merge - code added to throw events - PA ********/
		else { // calculations for finding out missed frames

			unsigned int frames_tmp = m_scenesDrawnSinceRateChage -frames_till_lastcall;
			unsigned int time_tmp = elapsedTC - time_at_lastcall;
			unsigned int actual_num_of_frames = (time_tmp/1000 ) * m_frameRate;
			if ( actual_num_of_frames > frames_tmp ) {
				// missed frames : throw event now
				// set the MISSED_FRAMES variable in the table
				di = MPJGetTerminalDispatchTable();;
				if( di != NULL ) {
					di = MPJGetTerminalDispatchTable();
					// go till end of list and add the new event there
					rendererQueue* temp;
					temp = &(di->rEQ);
					if( *temp == NULL ) {
						*temp = new rendererEventQueue( 1 );
					}
					else {
						while ( (*temp)->next != NULL )
							*temp = (*temp)->next;
						(*temp)->next = new rendererEventQueue( 1 );
					}

					if( di->isStartedDispatching == 1) {
						// signal the thread to throw the event
						if ( di->isJVMStarted == 1 ) {
							int count = 0;
							count = ResumeThread(di->hThrow);
						}
					}
				}
			}
		}
		// END - MPEGJ
		#endif

	}

//	TRACE_PRINT("Leaving Presenter::Main():\n");

//Put here code to handle player exit

	// Flag that the thread is terminating
	m_mtxThread.V();
//To be reworked
#ifdef STAND_ALONE
	GetApp()->OnSceneFinished();
#endif

//

} /* Presenter::Render */

// ======================================================================

//
// Presenter::Paint
// Pass the paint message from the Executive onto the visual renderer
//

void Presenter::Paint(void *info)
	{
	//	m_mtxScreenAccess.P();

	    if((info==NULL)&&!GetExecutive()->m_ExecutiveStartCalled)
		{
//			TRACE_PRINT("displayed logo \n");
			GetVisualRenderer()->DisplayLogo();

		}
		(info==NULL) ? m_pVisualRenderer->RefreshScreen():1;
		m_pVisualRenderer->Paint(info);
	//	m_mtxScreenAccess.V();
	} /* Presenter::Paint*/

// ========================================================================

//
// Presenter::WaitTillFinished
//
// This function will only return when all of the nodes in the
// scene have finished, which means that the Presenter thread
// has finished.
//

void Presenter::WaitTillFinished()
{
	m_mtxThread.P ();
	m_mtxThread.V ();
} /* Presenter::WaitTillFinished */


// ========================================================================

//
// Presenter::ResetFrameRateCalculation
//
// Reset the frame rate calculation variables.
//

void Presenter::ResetFrameRateCalculation()
{
	m_nextElapsedTime = NUM_ELAPSED_TIMES - 1;
	for (int i = 0; i < (int)NUM_ELAPSED_TIMES; i++)
		m_elapsedTimes[i] = 0;
	m_actualFrameRate = 0;
} /* Presenter::ResetFrameRateCalculation */


// =====================================================================

//
// Presenter::UpdateFrameRateCalculation
//
// Update the frame rate calculation, by giving the next time at
// which a frame has been presented.
//

void Presenter::UpdateFrameRateCalculation(unsigned int displayTime)
{
	// Put the given time into the ring buffer of the most recent
	// NUM_ELAPSED_TIMES times.
	m_nextElapsedTime = (m_nextElapsedTime + 1) % NUM_ELAPSED_TIMES;
	m_elapsedTimes[m_nextElapsedTime] = displayTime;

	// Look back through the recent display times, for a frame which
	// was displayed at least a second (1000 ms) ago.  This is used to
	// determine the average frame rate for the past second.
	int i = m_nextElapsedTime;
	int frames = 0;
	while ((displayTime - m_elapsedTimes[i] < 1000) &&
		   (frames < NUM_ELAPSED_TIMES))
	{
		i = (i == 0) ? (NUM_ELAPSED_TIMES - 1) : (i - 1);
		frames++;
	}

	// And now calculate the frame rate
	if (displayTime == m_elapsedTimes[i])
		m_actualFrameRate = m_frameRate;
	else
		m_actualFrameRate = 1000.0 * (float)frames / (float)(displayTime - m_elapsedTimes[i]);
} /* Presenter::UpdateFrameRateCalculation */

void Presenter::CheckUnusedOpenStreams()
{
	int length= m_openStreams.GetLength();
	if(length==0)
		return;

	for(int i=0;i<length;i++)
		if(m_openStreams[i]->GetOpenStreamFlag())
			m_openStreams[i]->ResetOpenStreamFlag();
		else
		{
			m_openStreams[i]->OnEndOfStream();
			m_openStreams.DeleteFrom(i--);
			length--;
		}

} /* Presenter::CheckUnusedOpenStreams */
void Presenter::RemoveOpenStream(class MovieTextureProxy* pMovie)
{
	int length= m_openStreams.GetLength();

	if(length==0)
		return;

	for(int i=0;i<length;i++)
		if(m_openStreams[i]==pMovie)
		{
			m_openStreams.DeleteFrom(i--);
			--length;
		}
}

void Presenter::CheckUnusedOpenSynthesizedTextureStreams()
{
	int length= m_openSynthesizedTextureStreams.GetLength();
	if(length==0)
		return;

	for(int i=0;i<length;i++)
		if(m_openSynthesizedTextureStreams[i]->GetOpenStreamFlag())
			m_openSynthesizedTextureStreams[i]->ResetOpenStreamFlag();
		else
		{
			m_openSynthesizedTextureStreams[i]->OnEndOfStream();
			m_openSynthesizedTextureStreams.DeleteFrom(i--);
			length--;
		}

} /* Presenter::CheckUnusedOpenSynthesizedTextureStreams */

void Presenter::RemoveOpenSynthesizedTextureStream(class SynthesizedTextureProxy* pMovie)
{
	int length= m_openSynthesizedTextureStreams.GetLength();

	if(length==0)
		return;

	for(int i=0;i<length;i++)
		if(m_openSynthesizedTextureStreams[i]==pMovie)
		{
			m_openSynthesizedTextureStreams.DeleteFrom(i--);
			--length;
		}
}

// =====================================================================

//
// Presenter::DeleteObject
//
// The generic rendering proxy object deleter.
//

void Presenter::DeleteObject(MediaObject* pObject)
{
	Traversable* pProxy = (Traversable*)(pObject->GetRenderingObject());
	if (pProxy != NULL)
		delete pProxy;

} /* Presenter::DeleteObject */

//ALL FOR MIKE!!!
/*-----------------------------------------------------------------
Return the sysmem time minus the clocko offset of the Presenter's clock
-----------------------------------------------------------------*/
DWORD Presenter :: GetClockTime ()
{
	return timeGetTime () - m_dwOffset;
}

  /*-----------------------------------------------------------------
Format a string into the log file
-----------------------------------------------------------------*/
void Presenter :: Logf (LPCSTR szFormat, ...)
{
/*	fprintf (
		m_fLogFile,
		"Time: %6lu(%6lu): ",
		timeGetTime () - m_startingTime,
		GetClockTime ()
	);*/
	va_list argptr;
	va_start (argptr, szFormat);
//	vfprintf (m_fLogFile, szFormat, argptr);
	va_end (argptr);
} /* Logf */

// =====================================================================


// Emacs uses these...
// Local Variables:
// mode: c++
// tab-width: 4
// End:
