//This software module was originally developed by CSELT in the course of   //
//development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.             //
//This software module is an implementation of a part of one or more MPEG-4 //
//Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems        //
//(ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 Systems     //
//(ISO/IEC 14496-1) free license to this software module or modifications   //
//thereof for use in hardware or software products claiming conformance to  //
//the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use this software//
//module in hardware or software products are advised that its use may      //
//infringe existing patents.                                                //
//The original developer of this software module and his/her company, the   //
//subsequent editors and their companies, and ISO/IEC have no liability     //
//for use of this software module or modifications thereof in an            //
//implementation. Copyright is not released for non MPEG-4 Systems          //
//(ISO/IEC 14496-1) conforming products. CSELT S.pA. retains full right     //
//to use the code for his/her own purpose, assign or donate the code to a   //
//third party and to inhibit third parties from using the code for non      //
//MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright      //
//notice must be included in all copies or derivative works.                //
//Copyright  1999.                                                         //
//This work has been partly developed within the framework of the ESPRIT    //
//European Research Project  "MPEG-4 PC".                                   //
//////////////////////////////////////////////////////////////////////////////
//
// File: Presenter.h
//
// Authors: Matthew Leditschke
//
// Modified:  Gianluca Di Cagno  gianluca.dicagno@cselt.it
//
// Description:
//   This class defines a Presenter, composed of a visual and audio
//   rendering engine.
//
// Created: August, 1997
//
//
///////////////////////////////////////////////////////////////////////////
/*-------------------------------------------------------------------------
MPEGJ
AUTHORS:    Viswanathan Swaminathan - vishy@eng.sun.com
			Robert Schmidt(RLS)
			Praveen Alavilli(PA)
			YUNTAI KYONG(YT) - yuntaikyong@ieee.org
-------------------------------------------------------------------------*/


#ifndef PRESENTER_H
#define PRESENTER_H


//PAB-:#include "PresBase.h"
#include "../FlexTime/PresenterFlex.h"//PAB+:
#include "AudRend-CSELT.h"
#include "VisRend.h"
#include "ztl.h"
#include "Im1defs.h"
#include "Traversable.h"

#ifdef MPGJ
//MPEGJ - MERGE
#include "..\mpegj\MPEGJTerminalInterface.h"
	// for pointer table
// END - MPEGJ - PA
#endif // MPEGJ



#ifdef STAND_ALONE
#include <webbrowser2.h>
#endif
class Executive;
class NodeDepthInfo;
class Capabilities;
class MediaObject;
class AudioRenderer;
class VisualRenderer;
class UserEventHandlerProxy;





//
// A class which stores all of the details of an event.
//
class EventInfo
{
public:
	EventInfo(USER_EVENT event, int x, int y, UINT controlKeysStatus)
	{
		m_event = event;
		m_x = x;
		m_y = y;
		m_controlKeysStatus = controlKeysStatus;
	}

	USER_EVENT m_event;
	int m_x;
	int m_y;
	UINT m_controlKeysStatus;
	int pass;
};


//
// The Presenter class
//
//PAB-:class Presenter : public PresenterBase
class Presenter : public PresenterFlex //PAB+:

{
	//
	// Public functions
	//
public:
	#ifdef MPGJ
	// MPEGJ - MERGE
	/*** handle to the function pointer  ***/
	MPJTerminalDispatchTablePtr di;
	/**** - end MPEGJ - PA */
	#endif


	ZArray<EventInfo*> m_eventArray;
	// Class constructor
	//  - instantiates the visual and audio renderers
	Presenter(Executive* pExecutive, HWND hWnd);

	// Class destructor - calls terminate if it hasn't already been called
	~Presenter();

	// Get the capabilities of the video and audio renderers
	void GetCapabilities (Capabilities* pPresentationCaps);

	// Initialise the renderers, and start presenting the scene
	// (whose root object is pointed to by pScene).
	void Init(/*MediaObject* pScene*/);

	//This method will be called once for every file that is opened
	void Start ();

	// Stops rendering, performs cleanups and terminates the object
	// thread.  This method will be called once for every file that
	// is closed.
	void Terminate();

	// Return the current presenter time, which is the time since the
	// presenter was started, and this is only updated before a scene
	// is displayed.  The time returned is in milliseconds.
	DWORD GetTime() { return m_currentTime; }

	DWORD GetStartTime() { return m_startingTime; }

	// Called by the executive when a user event occurs.
	void OnUserEvent(USER_EVENT event, int x, int y,
					 UINT uControlKeysStatus);

	// Return TRUE if the renderers were initialised successfully.
	BOOL InitialisedOK()
	{ return (m_pAudioRenderer->InitialisedOK() && m_pVisualRenderer->InitialisedOK()); }



	// This is the main loop of the presenter thread, which is called by
	// main loop  in the base class.
	void Render();

	// Pass the paint message from the Executive onto the visual renderer
	void Paint(void *info);
	//{ (info==NULL) ? m_pVisualRenderer->RefreshScreen():1;m_pVisualRenderer->Paint(info); }

	// This function will only return when all of the nodes in the
	// scene have finished, which means that the Presenter thread
	// has finished.
	void WaitTillFinished();

	// Add the given UserEventHandlerProxy node to the list
	// of nodes which need to be informed of user events.
	void RegisterEventHandler(UserEventHandlerProxy* pNode);

	// Remove the given UserEventHandlerProxy node from the
	// list of nodes which need to be informed of user events.
	void UnregisterEventHandler(UserEventHandlerProxy* pNode);

	// Tell the visual renderer to display frames at this rate.  The
	// default rate for the visual renderer is 15 frames per second.
	void SetTargetFrameRate(float newFrameRate)
		{ if (newFrameRate > 0) m_newFrameRate = newFrameRate; }

	// Get the visual renderer's current target frame rate.
	float GetTargetFrameRate() const
		{ return m_newFrameRate; }

    // Get granularity
    int GetIterationInterval () { return 20;}  //changed!!

    // functions to set and retrieve the isTracking member

	BOOL GetIsTracking() { return m_isTracking;}

	void SetIsTracking(BOOL value) {m_isTracking=value;}

	DrawableNode * GetDrawableTouched() { return m_pDrawableTouched;}


	// Return the actual frame rate that the visual renderer is producing.
	// This will differ from the target frame rate if there is not enough
	// computing power.  The rate returned will have been obtained by
	// averaging over the last second.
	float GetActualFrameRate() const
	{ return m_actualFrameRate; }

	VisualRenderer *GetVisualRenderer () {return m_pVisualRenderer;}
	AudioRenderer *GetAudioRenderer () {return m_pAudioRenderer;}
	Executive* GetExecutive() {return m_pExecutive;}

	BOOL IsActive() {return m_bActive;}
	void AskScreenAccess(){m_mtxScreenAccess.P();}
	void ReleaseScreenAccess(){m_mtxScreenAccess.V();}
	DWORD GetScenesDrawn(){return  m_scenesDrawnSinceStart;}
	HWND GetHWND(){return GetWindow();}

	void AddToOpenSynthesizedTextureStreams(class SynthesizedTextureProxy* pMovie){ m_openSynthesizedTextureStreams+=pMovie;}
	void CheckUnusedOpenSynthesizedTextureStreams();
	void RemoveOpenSynthesizedTextureStream(class SynthesizedTextureProxy* pMovie);

	//handle unused video open streams
	void AddToOpenStreams(class MovieTextureProxy* pMovie){ m_openStreams+=pMovie;}
	void CheckUnusedOpenStreams();
	void RemoveOpenStream(class MovieTextureProxy* pMovie);

	// The following functions are called by the constructors of the
	// nodes, and are used to create the proxy object associated with
	// every node which is used to render it.
	// All of these functions take a single argument, being a pointer
	// to the node.

	class AnchorProxy * InitAnchor(class Anchor*);
	class AppearanceProxy * InitAppearance(class Appearance*);
	class AudioSourceProxy * InitAudioSource(class AudioSource*);
	class Background2DProxy * InitBackground2D(class Background2D*);
	class BitmapProxy *  	InitBitmap(class Bitmap *);
	class CircleProxy		 * InitCircle(class Circle*);
	class ConditionalProxy * InitConditional(class Conditional *);
	class ColorProxy * InitColor(class Color*);
	class ColorInterpolatorProxy * InitColorInterpolator(class ColorInterpolator*);
	class Coordinate2DProxy * InitCoordinate2D(class Coordinate2D*);
	class CoordinateInterpolator2DProxy * InitCoordinateInterpolator2D(class CoordinateInterpolator2D*);
	class Curve2DProxy * InitCurve2D(class Curve2D*);
	class DiscSensorProxy * InitDiscSensor(class DiscSensor*);
	class FontStyleProxy * InitFontStyle(class FontStyle*);
	class FormProxy * InitForm(class Form*);
	class GroupProxy * InitGroup(class Group*);
	class IndexedLineSet2DProxy * InitIndexedLineSet2D(class IndexedLineSet2D*);
	class ImageTextureProxy *  	InitImageTexture(class ImageTexture *);
	class LayoutProxy * InitLayout(class Layout*);
	class Layer2DProxy * InitLayer2D(class Layer2D*);
	class InlineProxy * InitInline(class InlineImp*);
	class LinePropertiesProxy * InitLineProperties(class LineProperties*);
	class Material2DProxy * InitMaterial2D(class Material2D*);
	class MatteTextureProxy *  	InitMatteTexture(class MatteTexture *);
	class MovieTextureProxy *  	InitMovieTexture(class MovieTexture *);
  class MultiTextureProxy * InitMultiTexture(class MultiTexture* );
	class OrderedGroupProxy * InitOrderedGroup(class OrderedGroup*);
	class PointSet2DProxy * InitPointSet2D(class PointSet2D*);
	class PlaneSensor2DProxy * InitPlaneSensor2D(class PlaneSensor2D*);
	class ProximitySensor2DProxy * InitProximitySensor2D(class ProximitySensor2D*);
	class PositionInterpolator2DProxy * InitPositionInterpolator2D(class PositionInterpolator2D*);
	class RectangleProxy * InitRectangle(class Rectangle*);
	class ScriptProxy * InitScript(class ScriptImp*);
	class ScalarInterpolatorProxy * InitScalarInterpolator(class ScalarInterpolator*);
	class ShapeProxy * InitShape(class Shape*);
	class Sound2DProxy * InitSound2D(class Sound2D*);
	class SwitchProxy * InitSwitch(class Switch*);
	class TextProxy * InitText(class Text*);
	class TermCapProxy * InitTermCap(class TermCap*);
	class TimeSensorProxy * InitTimeSensor(class TimeSensor*);
	class TouchSensorProxy * InitTouchSensor(class TouchSensor*);
	class Transform2DProxy * InitTransform2D(class Transform2D*);
	class ValuatorProxy * InitValuator(class Valuator*);
	class WorldInfoProxy * InitWorldInfo(class WorldInfo*);
	class TemporalGroupProxy * InitTemporalGroup(class TemporalGroup*); //PAB+:
  class TemporalTransformProxy * InitTemporalTransform(class TemporalTransform*); //PAB+:
  	class SynthesizedTextureProxy *  	InitSynthesizedTexture(class SynthesizedTexture *);


	// This is the generic object destroyer, to be called by the
	// destructors of the nodes.
	void DeleteObject(class MediaObject* pObject);

    // MBS: log function to be added to all presenter to trace presenter's activity
  void Logf (LPCSTR szFormat, ...);
  FILE *m_fLogFile;
  DWORD GetClockTime ();
  DWORD  m_dwOffset;
  //
	// Private implementation details
	//
private:
	void ProcessEvents();

	// Flags the presenter thread when it is time to finish
	BOOL m_bActive;

	// This mutex is used by Terminate() to wait until the thread
	// has finished.
	ZMutex m_mtxThread;

	// The output frame rate (frames per second)
	float m_frameRate;

	// This holds the new value of the frame rate until it is made
	// the current frame rate.
	float m_newFrameRate;

	// This stores the actual frame rate as calculated by the main loop
	// of the presenter.
	float m_actualFrameRate;

	// The time that has elapsed since the presenter was started.
	DWORD m_currentTime;

	// Reset the frame rate calculation variables
	void ResetFrameRateCalculation();

	// Update the frame rate calculation, by giving the next time at
	// which a frame has been presented.
	void UpdateFrameRateCalculation(unsigned int nextTime);

    // Patch member to check if someone is doing the track of an object.
	BOOL m_isTracking;

	// The pointer of touched drawable needs to be used by the Sensors.

	DrawableNode * m_pDrawableTouched;

	// Data members pertaining to the calculation of the actual frame
	// rate.
#define NUM_ELAPSED_TIMES 25
	unsigned int m_nextElapsedTime;
	unsigned int m_elapsedTimes[NUM_ELAPSED_TIMES];


	// The array of events that need to be processed
	//ZArray<EventInfo*> m_eventArray;
	ZMutex m_mtxEventArray;

	// The array of nodes to be notified of user events
	ZArray<UserEventHandlerProxy*> m_eventHandlersArray;
	ZMutex m_mtxEventHandlersArray;

	// Other variables relating to the timing of the presenter
	unsigned long m_scenesDrawnSinceRateChage;
	unsigned long m_startingTime;
	float m_sceneUpdatePeriod;
	DWORD m_scenesDrawnSinceStart;

	// The default effects given to PreRender()
	Effects m_defEffects;

	VisualRenderer *m_pVisualRenderer;
	AudioRenderer *m_pAudioRenderer;
	Executive *m_pExecutive;
	MediaObject *m_pScene;
	ZMutex m_mtxScreenAccess;
	DrawableContext *m_pTrackingContext;
	BOOL m_bPaused;
	ZArray<MovieTextureProxy *> m_openStreams;
	ZArray<SynthesizedTextureProxy *> m_openSynthesizedTextureStreams;

};


#endif // PRESENTER_H


// Emacs uses these...
// Local Variables:
// mode: c++
// tab-width: 4
// End:
