//This software module was originally developed by CSELT in the course of   //
//development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.             //
//This software module is an implementation of a part of one or more MPEG-4 //
//Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems        //
//(ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 Systems     //
//(ISO/IEC 14496-1) free license to this software module or modifications   //
//thereof for use in hardware or software products claiming conformance to  //
//the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use this software//
//module in hardware or software products are advised that its use may      //
//infringe existing patents.                                                //
//The original developer of this software module and his/her company, the   //
//subsequent editors and their companies, and ISO/IEC have no liability     //
//for use of this software module or modifications thereof in an            //
//implementation. Copyright is not released for non MPEG-4 Systems          //
//(ISO/IEC 14496-1) conforming products. CSELT S.pA. retains full right     //
//to use the code for his/her own purpose, assign or donate the code to a   //
//third party and to inhibit third parties from using the code for non      //
//MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright      //
//notice must be included in all copies or derivative works.                //
//Copyright  1999.                                                         //
//This work has been partly developed within the framework of the ESPRIT    //
//European Research Project  "MPEG-4 PC".                                   //
//////////////////////////////////////////////////////////////////////////////
//
// File:  ProximitySensor2DProxy.cpp
//
// Author: Gianluca Di Cagno, CSELT, Gianluca.Dicagno@cselt.it
//
// Modified:
//
// Description:
//   The implementation of the proxy object used by the presenter
//   to handle the ProximitySensor2D node.
//
// Created: February 1999
//
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include "Nodes-CSELT.h"
#include "ProximitySensor2DProxy.h"
#include "trace.h"



// =====================================================================

//
// Presenter::InitProximitySensor2DProxy
//
// Initialise the proxy data structure and attach it to the render data
// pointer of the given media object node.
//

ProximitySensor2DProxy* Presenter::InitProximitySensor2D(ProximitySensor2D* pObj)
{
	return new  ProximitySensor2DProxy(pObj);   
}

// =====================================================================

//
// ProximitySensor2DProxy::PreRender()
//
// If this node is enabled, store the region to be sensed
// 
//
void ProximitySensor2DProxy::PreRender(Effects &)
{
	
	DWORD dwUpdateTime;
	 if((dwUpdateTime=GetProximitySensor2D()->GetUpdateTime())!=m_LastUpdateTime)	
	 {
		 (Bool)(GetProximitySensor2D()->enabled) ?
			((Presenter*)GetPresenter())->RegisterEventHandler(this) :
			((Presenter*)GetPresenter())->UnregisterEventHandler(this);
		 ProximitySensor2D* pProximity = GetProximitySensor2D();	
		 float left=((Vec2f)pProximity->center).x-(((Vec2f)pProximity->size).x)/2;
		 float top =((Vec2f)pProximity->center).y+(((Vec2f)pProximity->size).y)/2;
		 float width=((Vec2f)pProximity->size).x;
		 float heigth =((Vec2f)pProximity->size).y;
		 BRect tempRegion(left,top,width,heigth);
		 m_sensedRegion=tempRegion;
		 m_LastUpdateTime=dwUpdateTime;
	 }

} /*ProximitySensor2DProxy::Render()*/



// =====================================================================

//
// ProximitySensor2DProxy::OnUserEvent
//
// If this node is enabled, this function is called whenever a user 
// event is generated. If the mouse pointer is within the region
// to be sensed the ProximitySensor events are generated.
//

BOOL ProximitySensor2DProxy::OnUserEvent(EventInfo const* pEvent)
{
	TRACE_PRINT("In ProximitySensor2DProxy::OnUserEvent():\n");

	// Only generate events if this node is enabled, and no other
	// sensor node has grabbed all motion events
	if (((Bool)GetProximitySensor2D()->enabled))
	{
		
	 // This gets the time since the start of the scene
	 // (convert the time to seconds, as used by BIFS)

	 // #GB: double currentTime = (double)((Presenter*)GetPresenter())->GetTime() / 1000;
   double currentTime = GetFlexTime()/1000;

	 if ((pEvent->m_event == EVENT_MOUSE_DOWN)|| (pEvent->m_event == EVENT_MOUSE_MOVE) )
	   {
		    
		    if(m_sensedRegion.IsPointIn(pEvent->m_x, pEvent->m_y))
            {
	    		
				Vec2f point(pEvent->m_x, pEvent->m_y);
				GetProximitySensor2D()->position_changed=point;

			
				if(!m_JustEntered)
				{
					
					m_InRegion=TRUE;
					GetProximitySensor2D()->isActive=TRUE;
					GetProximitySensor2D()->enterTime=currentTime;
				}
				

				
			}
			else
				if(m_InRegion)
				{
					m_InRegion=FALSE;
					m_JustEntered=FALSE;
					GetProximitySensor2D()->exitTime=currentTime;
					GetProximitySensor2D()->isActive=FALSE;
				}
            
	   }

	// If not already done, register this node with the presenter
	//  GetPresenter()->RegisterEventHandler(this);
	
	  return TRUE;
	}



	// Otherwise, if not enabled, return FALSE
	else
		return FALSE;
} /* ProximitySensor2DProxy::OnUserEvent */


// =====================================================================

//
// ProximitySensor2DProxy::IsFinished
//
// A ProximitySensor2DProxy node is considered as finished when it is not enabled.
//

BOOL ProximitySensor2DProxy::IsFinished() const
{
	return !((BOOL)(Bool)GetProximitySensor2D()->enabled);
} /* ProximitySensor2DProxy::IsFinished */


// ========================================================================
