//This software module was originally developed by CSELT in the course of   //
//development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.             //
//This software module is an implementation of a part of one or more MPEG-4 //
//Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems        //
//(ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 Systems     //
//(ISO/IEC 14496-1) free license to this software module or modifications   //
//thereof for use in hardware or software products claiming conformance to  //
//the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use this software//
//module in hardware or software products are advised that its use may      //
//infringe existing patents.                                                //
//The original developer of this software module and his/her company, the   //
//subsequent editors and their companies, and ISO/IEC have no liability     //
//for use of this software module or modifications thereof in an            //
//implementation. Copyright is not released for non MPEG-4 Systems          //
//(ISO/IEC 14496-1) conforming products. CSELT S.pA. retains full right     //
//to use the code for his/her own purpose, assign or donate the code to a   //
//third party and to inhibit third parties from using the code for non      //
//MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright      //
//notice must be included in all copies or derivative works.                //
//Copyright  1999.                                                         //
//This work has been partly developed within the framework of the ESPRIT    //
//European Research Project  "MPEG-4 PC".                                   //
//////////////////////////////////////////////////////////////////////////////
//
// File:  Rectangle.cpp
//
// Author: Matthew Leditschke 
//
// Modified: Gianluca Di Cagno  gianluca.dicagno@cselt.it
//
// Description:
//   The implementation of the AudioSource node.
//
// Created: August, 1997.
//
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include "Nodes-CSELT.h"
#include "RectangleProxy.h"
//#include "Material2D.h"
#include "trace.h"


// =====================================================================

//
// Presenter::InitRectangle
//
// Initialise the proxy data structure and attach it to the render data
// pointer of the given media object node.
//


RectangleProxy* Presenter::InitRectangle(class Rectangle * pObj)
{
	return new  RectangleProxy(pObj);    
}


// ======================================================================

//
// RectangleProxy::PreRender
//
//

void RectangleProxy::PreRender(Effects & effects)
{

	
	DrawableContext *pContext=InitPreRender(effects);
	if (!pContext->IsSwitchedOn()) 
		return;
	//if this drawable is within a Layout or Form node
	//add his drawableContext to the list of drawableContexts
	//of the parent node. 
	//this is useful for postprocessing.

	if(effects.m_ParentGroup)
		effects.m_ParentGroup->AddToDrawableContextList(pContext);
	
	// If the transparency for this node is 1, or it is not filled and
	// the line width is 0, then it doesn't need to be drawn, and its
	// bounding rectangle is empty.
	if ((GetTransparency(pContext) > 1 - ALMOST_ZERO) ||
		(! GetFilled(pContext) && (GetWidth(pContext) == 0)))
	{
		pContext->GetBoundingRect()->MakeEmpty();
		if(effects.m_ParentGroup==NULL)//necessary for postprocessing style nodes 
			UpdateBoundingRect(pContext);

		return;
	}

	// Determine where the four corners of the rectangle are
	m_corners[0].x = -((Vec2f)GetRectangle()->size).x / 2;
	m_corners[0].y = ((Vec2f)GetRectangle()->size).y / 2;
	m_corners[1].x = ((Vec2f)GetRectangle()->size).x / 2;
	m_corners[1].y = ((Vec2f)GetRectangle()->size).y / 2;
	m_corners[2].x = ((Vec2f)GetRectangle()->size).x / 2;
	m_corners[2].y = -((Vec2f)GetRectangle()->size).y / 2;
	m_corners[3].x = -((Vec2f)GetRectangle()->size).x / 2;
	m_corners[3].y = -((Vec2f)GetRectangle()->size).y / 2;


	// Determine where these points lie after the transformation has been
	// applied.
	for (int i = 0; i < 4; i++)
	{
		(pContext->m_transform).Apply(m_corners[i].x, m_corners[i].y);


	}

	// A rotated rectangle will be treated much like a polygon
	if (pContext->m_transform.ContainsRotation())
	{
		// Find the bounding box for this rotated rectangle
		pContext->GetBoundingRect()->SetDimensions(m_corners, 4);
		
	}

	// A non-rotated rectangle is handled separately
	else
	{
		// The bounding rectangle of the rectangle itself
		m_drawRect.SetDimensions(m_corners[0].x, m_corners[0].y, 
								 m_corners[2].x - m_corners[0].x + 1,
								 m_corners[0].y - m_corners[2].y + 1);
		*(pContext->GetBoundingRect()) = m_drawRect;

		
	}
	//nodes like the form and layout needs some form of postprocessing on the children
	//hence we register the bounding rect now, the form node prerender function will do
	//this.
	
	if(effects.m_ParentGroup==NULL)//necessary for postprocessing style nodes 
		UpdateBoundingRect(pContext);

	if(effects.orderHasChanged)
			pContext->m_updateRequired = TRUE;

	pContext->m_isTransparent=IsTransparent(pContext);
} /* RectangleProxy::PreRender */


// ============================================================================

//
// RectangleProxy::Draw
//
// Draw the rectangle.  If the rectangle is rotated, it is drawn as a
// polygon, otherwise it is drawn as a rectangle.  Transparency is
// only supported for non-rotated filled rectangles, which do not
// have a shadow.
//

void RectangleProxy::Draw(BRect const* pClipBRect,DrawableContext * pDrawableContext)
{
	
	assert(pClipBRect != NULL);
	VisualRenderer* pVR = GetVisualRenderer();

	// The amount of the rectangle to draw lies within both the clipping
	// rectangle and the rectangle's own bounding rectangle (eventualy clipped by
	// its parent clipping area).
	BRect visibleRect = *pClipBRect;
	visibleRect.Intersection(pDrawableContext->GetClippedBoundingRect());


	if (visibleRect.IsEmpty())
		return;

	// Get the transparency value to use
	float transparency = GetTransparency(pDrawableContext);


	// And now draw the rectangle itself
	if (pDrawableContext->m_transform.ContainsRotation())
		pVR->DisplayPolygon(visibleRect, m_corners, 4, GetEmissiveColor(pDrawableContext),
							GetWidth(pDrawableContext), GetFilled(pDrawableContext));
	else
		pVR->DisplayRectangle(visibleRect,(* pDrawableContext->GetBoundingRect())/*m_drawRect*/, GetEmissiveColor(pDrawableContext),
							  GetWidth(pDrawableContext), GetFilled(pDrawableContext), transparency);
} /* RectangleProxy::Draw */


// ===========================================================================

//
// RectangleProxy::IsTransparent
//
// The Rectangle is not transparent if the following conditions hold:
//  - DrawableNode::IsTransparent() is FALSE,
//  - it is filled,
//  - there is no transparency,
//  - the rectangle is not rotated
//


// ===========================================================================
BOOL RectangleProxy::IsTransparent(DrawableContext *pContext) const
{
	if  ((GetFilled(pContext)) && (GetTransparency(pContext) == 0) &&
		(!pContext->m_transform.ContainsRotation()))
		return FALSE;
	else
		return TRUE;
} /* RectangleProxy::IsTransparent */

