//This software module was originally developed by CSELT in the course of   //
//development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.             //
//This software module is an implementation of a part of one or more MPEG-4 //
//Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems        //
//(ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 Systems     //
//(ISO/IEC 14496-1) free license to this software module or modifications   //
//thereof for use in hardware or software products claiming conformance to  //
//the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use this software//
//module in hardware or software products are advised that its use may      //
//infringe existing patents.                                                //
//The original developer of this software module and his/her company, the   //
//subsequent editors and their companies, and ISO/IEC have no liability     //
//for use of this software module or modifications thereof in an            //
//implementation. Copyright is not released for non MPEG-4 Systems          //
//(ISO/IEC 14496-1) conforming products. CSELT S.pA. retains full right     //
//to use the code for his/her own purpose, assign or donate the code to a   //
//third party and to inhibit third parties from using the code for non      //
//MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright      //
//notice must be included in all copies or derivative works.                //
//Copyright  1999.                                                         //
//This work has been partly developed within the framework of the ESPRIT    //
//European Research Project  "MPEG-4 PC".                                   //
//////////////////////////////////////////////////////////////////////////////
//
// File:  Sound2DProxy.cpp
//
// Authors: Matthew Leditschke 
//
// Modified: Gianluca Di Cagno  gianluca.dicagno@cselt.it
//
// Description:
//   The implementation of the proxy object used by the presenter
//   to handle the Sound2D node.
//
// Created: October, 1997.
//
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include "Nodes-CSELT.h"
#include "Sound2DProxy.h"




// ===================================================================

//
// Presenter::InitSound
//
// Initialise the proxy data structure and attach it to the render data
// pointer of the given media object node.
//

Sound2DProxy* Presenter::InitSound2D(Sound2D* pObj)
{
	return new  Sound2DProxy(pObj);
}




// ===================================================================

//
// Sound2DProxy::PreRender
//
// Call the PreRender() method of the sound node associated with this
// node.
//

void Sound2DProxy::PreRender(Effects & effects)
{


	if (GetAudioNode() != NULL)
	{
		m_pAudioSourceProxy=((AudioSourceProxy *)(GetAudioNode()->GetRenderingObject()));
		if(!m_pAudioSourceProxy)
		{
			assert(0);
		//	TRACE_PRINT("Unsupported Audio source node");
			return;

		}
	    m_pAudioSourceProxy->SetSound2DProxy(this);
		m_pAudioSourceProxy->PreRender(effects);
	}
			
} /* Sound2DProxy::PreRender */


// ===================================================================

//
// Sound2DProxy::IsFinished
//
// Return TRUE if the sound node associated with this node is finished,
// otherwise return FALSE.
//

BOOL Sound2DProxy::IsFinished() const
{
	if (GetAudioNode() != NULL)
		return ((Traversable*)(GetAudioNode()->GetRenderingObject()))->IsFinished();
	else
		return TRUE;
} /* Sound2DProxy::IsFinished */


// ===================================================================
Sound2DProxy::~Sound2DProxy()
{

}

void Sound2DProxy::DeleteObject()
{
	if(m_pAudioSourceProxy)
	{
		m_pAudioSourceProxy->SetSound2DProxy(NULL);

		if (m_pAudioSourceProxy->m_registeredWithRenderer)
		{
			GetAudioRenderer()->RemoveSourceNode(m_pAudioSourceProxy);
			m_pAudioSourceProxy->m_registeredWithRenderer=FALSE;
		}
	}
}