//This software module was originally developed by CSELT in the course of   //
//development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.             //
//This software module is an implementation of a part of one or more MPEG-4 //
//Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems        //
//(ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 Systems     //
//(ISO/IEC 14496-1) free license to this software module or modifications   //
//thereof for use in hardware or software products claiming conformance to  //
//the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use this software//
//module in hardware or software products are advised that its use may      //
//infringe existing patents.                                                //
//The original developer of this software module and his/her company, the   //
//subsequent editors and their companies, and ISO/IEC have no liability     //
//for use of this software module or modifications thereof in an            //
//implementation. Copyright is not released for non MPEG-4 Systems          //
//(ISO/IEC 14496-1) conforming products. CSELT S.pA. retains full right     //
//to use the code for his/her own purpose, assign or donate the code to a   //
//third party and to inhibit third parties from using the code for non      //
//MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright      //
//notice must be included in all copies or derivative works.                //
//Copyright  1999.                                                         //
//This work has been partly developed within the framework of the ESPRIT    //
//European Research Project  "MPEG-4 PC".                                   //
//////////////////////////////////////////////////////////////////////////////
//
// File:  Transform2DProxy.cpp
//
// Author: Matthew Leditschke 
//
// Modified: Gianluca Di Cagno  gianluca.dicagno@cselt.it
//
// Description:
//   The implementation of the proxy object used by the presenter
//   to handle the Switch node.
//
// Created: October, 1997.
//
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include "Nodes-CSELT.h"
#include "SwitchProxy.h"
#include "trace.h"


// =====================================================================

//
// Presenter::InitSwitch
//
// Initialise the proxy data structure and attach it to the render data
// pointer of the given media object node.
//

SwitchProxy* Presenter::InitSwitch(Switch* pObj)
{
	return new  SwitchProxy(pObj); 
}


// =====================================================================

//
// SwitchProxy::PreRender
//
// Add the transform parameters of this node to the given effects, and
// then apply the PreRender method to all of the children of this node.
//

void SwitchProxy::PreRender(Effects & effects)
{
	Effects newEffects        = effects;
   int     nNumberOfChildren = GetSwitch()->choice.GetLength();
   int     nWhichChoice      = GetSwitch()->whichChoice;


   // - if this the first PreRender calls children PreRender
   // - this is needed because channels are read sequentially
   // - and if one is not read the next are blocked. 
   
   if (m_bFirstPreRender)
   {
	   for (int i = 0; i < nNumberOfChildren; i++)
	   {
		   if ((i == nWhichChoice) && (effects.isSwitchedOn))
		 	   newEffects.isSwitchedOn = TRUE;
		   else
			   newEffects.isSwitchedOn = FALSE;
         
		   ((Traversable *) GetSwitch()->choice[i]->GetRenderingObject())->PreRender(newEffects);
	   }
      m_bFirstPreRender = FALSE;
   }
   else
   {
      if (nWhichChoice != m_nLastChoice)
      {
         if (m_nLastChoice >= 0 && m_nLastChoice < nNumberOfChildren)
         {
            newEffects.isSwitchedOn = FALSE;
            
		      ((Traversable *) GetSwitch()->choice[m_nLastChoice]->GetRenderingObject())->PreRender(newEffects);
         }
         m_nLastChoice = nWhichChoice;
      }
      
     
      if (nWhichChoice >= 0 && nWhichChoice < nNumberOfChildren)
      {
         newEffects.isSwitchedOn = effects.isSwitchedOn;
         ((Traversable *) GetSwitch()->choice[nWhichChoice]->GetRenderingObject())->PreRender(newEffects);
      }
   }


} /* SwitchProxy::PreRender */

void SwitchProxy::UpdateTimeSensor()
{
	 
	int     nWhichChoice      = GetSwitch()->whichChoice;
	int     nNumberOfChildren = GetSwitch()->choice.GetLength();
	
	if((nWhichChoice!=-1)&&(nWhichChoice<nNumberOfChildren))
		((Traversable *) GetSwitch()->choice[nWhichChoice]->GetRenderingObject())->UpdateTimeSensor();
	
} /* SwitchProxy::UpdateTimeSensor */




// =====================================================================
