/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/

#include "defs.h"
#include "Nodes-CSELT.h"
#include "SynthesizedTextureProxy.h"
#include "Stream.h"
#include "trace.h"
#include "ShapeProxy.h"			// ZL
#ifdef DEBUG_CONSOLE
#include "Mpeg4Cselt2DActivexctl.h"
#endif

#include	"St.h"

// =====================================================================

//
// Presenter::InitSynthesizedTexture
//
// Initialise the proxy data structure and attach it to the render data
// pointer of the given media object node.
//

 SynthesizedTextureProxy * Presenter::InitSynthesizedTexture(SynthesizedTexture* pObj)
 {
	return new SynthesizedTextureProxy(pObj);
 }


// =====================================================================

//
// SynthesizedTextureProxy::SynthesizedTextureProxy
//
// Class constructor.
//

SynthesizedTextureProxy::SynthesizedTextureProxy(MediaObject* pMO) : DrawableImage(pMO)
{
	m_firstTimeAtStartTime = TRUE;
	m_firstTimeAtStopTime  = TRUE;
	m_endOfTimeReached = FALSE;
	m_startTime = 0;
	m_timesStarted = 0;
	m_OpenStreamIsBeingUsed=FALSE;


	m_st = NULL;
	m_im = NULL;
	#ifdef DEBUG_CONSOLE
//	const char *pDecInfo=GetSynthesizedTexture()->GetDecoder()->GetInfo();
//	::PostMessage(GetPresenter()->GetHWND() ,WM_DEC_INFO, NULL,(LPARAM)(LPCSTR )pDecInfo);



#endif

} /* SynthesizedTextureProxy::SynthesizedTextureProxy */
SynthesizedTextureProxy::~SynthesizedTextureProxy()
{
}


void SynthesizedTextureProxy::DeleteObject()
{
	GetPresenter()->RemoveOpenSynthesizedTextureStream(this);
}


void SynthesizedTextureProxy::Draw(BRect const* pClipBRect,DrawableContext * pContext)
{
	//Get the portion of the Image that needs drawing
	BRect *pImageBoundingRect=pContext->GetBoundingRect();


	// Modify the image size to only show the part which lies within
	// the given bounding rectangle.
	BRect visibleRect = *pContext->GetClippedBoundingRect();
	visibleRect.Intersection(pClipBRect);

	if (visibleRect.IsEmpty())
		return;



	if( m_im == NULL )	return;
	int	width = ((image_type*)m_im)->column;
	int	height = ((image_type*)m_im)->row;
	unsigned char *dataPtr = ((image_type*)m_im)->data;


	float scaleX= (float)(pImageBoundingRect->width) / (float)width;
	float scaleY= (float)(pImageBoundingRect->height) / (float)height;

	scaleX = scaleY = 1.0;




SrcRGBAImage image;

	image.width	 = min(visibleRect.width/scaleX,   width);
	image.height = min(visibleRect.height/scaleY, height);
	// number of byte between line
	image.pitch = 0; // scaleX = sacleY = 0;
//	image.pitch	 = pIstDec->GetAttributes(PITCH,(LPBYTE)dataPtr);
	image.pitch = 4*image.width;
	image.left	 = visibleRect.left;
	image.top	 = visibleRect.top;


	image.left	 += (visibleRect.width - width)/2;
	image.top	 -= (visibleRect.height - height)/2;

	image.scaleX= scaleX;
	image.scaleY= scaleY;

		// Get a pointer to the top-left pixel
	int offset	 = ((pImageBoundingRect->top - visibleRect.top)) * image.pitch*scaleX
						+ ((visibleRect.left - pImageBoundingRect->left)) * 3;

		//if the image was scaled we must calculate the offset of the non scaled image from
		// the offset of the scaled one.
	if(offset!=0&&(scaleX!=1||scaleY!=1))
	{
		int xOri,yOri;

		yOri=((offset/3)/pImageBoundingRect->width)/scaleY;
		xOri=((offset/3)%pImageBoundingRect->width)/scaleX;
		offset=(width*yOri+xOri)*3;

	}


	offset = 0;
	if (GetStreamConsumer()->GetStreamFormat().m_code == MediaFormat :: RGB)
	{
		image.pB	 = (unsigned char*)dataPtr+offset;
		image.pG	 = image.pB + 1;
		image.pR	 = image.pB + 2;
	}
	else
	{
		image.pR	 = (unsigned char*)dataPtr+offset;
		image.pG	 = image.pR + 1;
		image.pB	 = image.pR + 2;
	}

	image.pA	 = NULL;
	image.ptrInc = 4;


	GetVisualRenderer()->DisplayRGBAImage(image);
}


BOOL SynthesizedTextureProxy::NeedsDrawing(DrawableContext * pContext)
{
		int a;
	a =5;
		return pContext->m_updateRequired;
} /*DrawableImage::NeedsDrawing*/

// =====================================================================

//
// SynthesizedTextureProxy::PreRender
//
//

void SynthesizedTextureProxy::PreRender(Effects & effects,DrawableContext * pContext)
{

	/*-------------------------------------------------------------
	startTime / stopTime fields need to be offset by current time on first access
	-------------------------------------------------------------*/
	GetSynthesizedTexture() -> startTime.ApplyOffset (pContext ->GetShapeProxy () ->GetTimeContext ());
	GetSynthesizedTexture() -> stopTime.ApplyOffset (pContext ->GetShapeProxy () ->GetTimeContext ());

	//if this node has been switched off,stop the associated decoder

	if (!pContext->IsSwitchedOn())
		   {
		//let's try to stop the decoder cause the node has been switched off

		//OnEndOfStream();
		// Set flags and events which are required at the end of a stream.
		//m_firstTimeAtStopTime = FALSE;
		//m_firstTimeAtStartTime = TRUE;

        return;
		   }

	// This gets the time since the start of the scene
	// (convert the time to seconds, as used by BIFS)
// #GB:	double currentTime = (double)((Presenter*)GetPresenter())->GetTime() / 1000;
// ZL: changed to use the Shape parent because the SynthesizedTexture doesn't have up to date
// time information
  double currentTime = pContext ->GetShapeProxy () ->GetFlexTime()/1000.;

	// If the start time has changed since this was last called, reset things
	 if ((double)GetSynthesizedTexture()->startTime != m_startTime)
		m_endOfTimeReached = FALSE;
	m_startTime = (double)GetSynthesizedTexture()->startTime;

	// Only proceed if the end time has not been reached

	if (m_endOfTimeReached)
	      return;

	// Only proceed if the start time has been reached
	if (currentTime < m_startTime)
		return;

	// Determine if the stop time has been reached
	if ((currentTime >= (double)GetSynthesizedTexture()->stopTime) &&
		((double)GetSynthesizedTexture()->stopTime >= m_startTime))
	{
		// If this is the first time, generate an isActive=FALSE event,
		// but only if the stream was previously active
		if ((m_firstTimeAtStopTime) && ((Bool)GetSynthesizedTexture()->isActive))
			OnEndOfStream();
	}

	// otherwise the movie needs to be played
	else
	{
		// If this is the first time, generate an isActive=TRUE event
		if (m_firstTimeAtStartTime)
		{

			SynthesizedTexture* pSynthesizedTexture = GetSynthesizedTexture();

			m_firstTimeAtStartTime = FALSE;
			m_firstTimeAtStopTime = TRUE;
			pSynthesizedTexture->isActive = TRUE;
			m_timesStarted++;


			//here I should calculate currentTime-m_startTime
			//and seek the video to the correspondent position
			//This would enable start and pause and continue.



			if (m_timesStarted > 1)
			{
/*
				m_startTime = (double)pSynthesizedTexture->startTime;
				pSynthesizedTexture->OpenStream();
*/
				if( m_st == NULL ) {
					LoadStream(pContext);
					if( m_st == NULL )	return;
				}
				TRACE_PRINT1("Video: Starting at %g\n", m_startTime);
				return;
			}

			// we also need to start the stream

			pSynthesizedTexture->OpenStream();

			if( m_st == NULL ) {
				LoadStream(pContext);
				if( m_st == NULL )	return;
			}

			GetPresenter()->AddToOpenSynthesizedTextureStreams(this);

            TRACE_PRINT1("Video: Starting at %g\n", m_startTime);

		}


//ST
	if( m_st == NULL ) {
		LoadStream(pContext);
		if( m_st == NULL )	return;

//		GetPresenter()->InvalidateScene(NULL);
	}


	int now = pContext ->GetShapeProxy () ->GetFlexTime();
	int frame = m_st->GetNextFrameToPlay( now );
//	if( frame <0 )	return;

	if( frame >= 0 )
		m_im = m_st->Render( frame, (image_type *)m_im );
//	if( m_im1 == NULL )
//		m_im1 = (void *)image_create( ((image_type *)m_im)->row, ((image_type *)m_im)->column, 4, 1, NULL );

//	draw_image1( m_pTexture, (image_type *)m_im, (image_type *)m_im1 );


// ST


		DrawableImage::PreRender(effects,pContext);

		if(pContext->m_hasStreamFinished==TRUE)
		{
			if((Bool)GetSynthesizedTexture()->loop)
			{
				GetSynthesizedTexture()->CloseStream();
				GetSynthesizedTexture()->OpenStream();
			}
		}
		m_OpenStreamIsBeingUsed=TRUE;

	}
} /* SynthesizedTextureProxy::PreRender */

void SynthesizedTextureProxy::OnEndOfStream()
{
	// Call the base class's method
	DrawableImage::OnEndOfStream();

	// Set flags and events which are required at the end of a stream.
	m_firstTimeAtStopTime = FALSE;
	m_firstTimeAtStartTime = TRUE;
	m_endOfTimeReached = TRUE;
	GetSynthesizedTexture()->isActive = FALSE;
} /* SynthesizedTextureProxy::OnEndOfStream */




int
SynthesizedTextureProxy::LoadStream( DrawableContext * pContext )
{

//	if( m_bFirstTime == TRUE )	return(0);

	MediaStreamImp *pStream = GetSynthesizedTexture()->GetStream();
	if (!pStream) return(0);



	DWORD rdwTime;
	BOOL bEOU;
	LPVOID dataPtr;
	int dataSize;
	int status;

	rdwTime = 0;

	dataPtr = GetStreamConsumer()->Fetch(dataSize, rdwTime, bEOU, status, TRUE,
													FALSE, 0,	TRUE);

	// If there is no data, no updating is required
	if ((dataPtr == NULL) || (dataPtr == END_OF_STREAM) ) {
		return( -1 );
	}


	if( m_st == NULL )
		m_st = new CSt;

	if( m_st->Decode( (char*)dataPtr, dataSize ) < 0 ){
		delete m_st;
		m_st = NULL;
		return( -1 );
	}


//	int	height = m_st->GetFrameHeight();
//	int	width = m_st->GetFrameWidth();


//	int	left,	top;
//	left = top = 0;
//	pContext->m_transform.Apply(left, top);
		//for  scaling
//	pContext->m_transform.ApplyScale(width,height);

	UpdateTranslation();
	UpdateRotation();
	UpdateLoop();

	return( 1 );
}

// =====================================================================


void SynthesizedTextureProxy::UpdateTranslation()
{
vec3d	*av;
int	i;

SynthesizedTexture *st = GetSynthesizedTexture();


	int	len = st->translation.GetLength();

	if( len == 0 )	return;

	av = (vec3d *) malloc( sizeof(vec3d) * len );


	for( i = 0 ; i < len ; i++ ){
		av[i].x = st->translation[i].x;
		av[i].y = st->translation[i].y;
		av[i].z = st->translation[i].z;
	}


	m_st->SetTranslation( av, len );




	free( av );
}


void SynthesizedTextureProxy::UpdateRotation()
{
vec3d	*av;
int	i;

SynthesizedTexture *st = GetSynthesizedTexture();


	int	len = st->rotation.GetLength();

	if( len == 0 )	return;

	av = (vec3d *) malloc( sizeof(vec3d) * len );


	for( i = 0 ; i < len ; i++ ){
		av[i].x = av[i].y = av[i].z = 0;
		if( st->rotation[i].x == 0 && st->rotation[i].y == 1 && st->rotation[i].z == 0 ){
			av[i].y = st->rotation[i].theta * 180 /3.14;
		}
	}


	m_st->SetRotation( av, len );




	free( av );
}

void SynthesizedTextureProxy::UpdateLoop()
{
Bool	loop;
SynthesizedTexture *st = GetSynthesizedTexture();
	loop = (Bool)st->loop;
	m_st->SetLoop( loop );
}