/***********************************************************************
    This software module  was  originally  developed by Patrice Bazerque
	(IBM Research)
    in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)
    standard.  This software module is an implementation of a part of
    one or more MPEG-4 Systems (ISO/IEC 14496-1) tools  as  specified
    by  the  MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC gives
    users of the MPEG-4 Systems (ISO/IEC 14496-1)  free  license  to
    this software module or modifications thereof for use in hardware
    or software products claiming conformance to the  MPEG-4  Systems
    (ISO/IEC 14496-1). Those intending to use this software module in
    hardware or software  products  are  advised  that  its  use  may
    infringe   existing  patents.  The  original  developer  of  this
    software module and his/her company, the subsequent  editors  and
    their  companies,  and  ISO/IEC have no liability for use of this
    software module or modifications thereof  in  an  implementation.
    Copyright  is not released for non MPEG-4 Systems (ISO/IEC 14496-
    1) conforming  products.  IBM
    retain  full  right to use the code for their own purpose, assign
    or donate the code to a third party and to inhibit third  parties
    from  using  the  code  for  non MPEG-4 Systems (ISO/IEC 14496-1)
    conforming products. This copyright notice must  be  included  in
    all copies or derivative works. Copyright  1999.
************************************************************************/

/**************************************************************************
 * Project Im1Player
 * INTERNATIONAL STANDARD ORGANIZATION
 * ISO/IEC JTC1/SC29/WG11
 * 
 * SUBSYSTEM: BIFS Encoder/Decoder
 * FILE:      TemporalGroupProxy.cpp
 * AUTHOR:    Patrice Bazerque (IBM Research)
 * OVERVIEW:  The implementation of the proxy object used by the presenter
 *   to handle the TemporalGroup node.
 * 
 * Comment:   Contact: mykim@us.ibm.com
 *
 **************************************************************************/

#include "defs.h"
#include "Nodes-CSELT.h"
#include "TemporalGroupProxy.h"
#include "trace.h"


//-----------------------------------------------------------------------------
//  TemporalGroupProxy::TemporalGroupProxy
//
//  Class constructor.
//-----------------------------------------------------------------------------

TemporalGroupProxy::TemporalGroupProxy(MediaObject* pMO) : Traversable(pMO)
{
	m_isActive = FALSE;
} /* TemporalGroupProxy::TemporalGroupProxy */


//-----------------------------------------------------------------------------
//  Presenter::InitTemporalGroup
//
//  Initialise the proxy data structure and attach it to the render data
//  pointer of the given media object node.
//-----------------------------------------------------------------------------
TemporalGroupProxy* Presenter::InitTemporalGroup(TemporalGroup* pObj)
{
	return new  TemporalGroupProxy(pObj); 
}


//-----------------------------------------------------------------------------
//  TemporalGroupProxy::IsFinished
//
//  A TemporalGroup node is considered as finished when it is not active.
//-----------------------------------------------------------------------------
BOOL TemporalGroupProxy::IsFinished() const
{
	return !m_isActive;
} /* TemporalGroupProxy::IsFinished */


//-----------------------------------------------------------------------------
// update 
//-----------------------------------------------------------------------------
void TemporalGroupProxy::UpdateTimeSensor(/*Effects &eEffect*/)
{
	MFGenericNode *pChildren = GetObject()->GetChildrenField();
  
  int iChildrenCount = GetObject()->GetChildCount();
	for(int i=0; i<iChildrenCount; i++)
	{
		Traversable* trav = (Traversable*) ((*pChildren)[i])->GetRenderingObject();
		trav->UpdateTimeSensor(/*eEffect*/);
	}
}


//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
void TemporalGroupProxy::SetFlexTime(const PresenterContext &pcCurrentTime)
{
  m_pcTimeContext = pcCurrentTime;

	MFGenericNode *pChildren = GetObject()->GetChildrenField();
  int nNbChildren= GetObject()->GetChildCount();
	for(int i=0; i<nNbChildren; i++)
	{
		Traversable* trav = (Traversable*) ((*pChildren)[i])->GetRenderingObject();
		trav->SetFlexTime(m_pcTimeContext);
	}
}
