//This software module was originally developed by CSELT in the course of   //
//development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.             //
//This software module is an implementation of a part of one or more MPEG-4 //
//Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems        //
//(ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 Systems     //
//(ISO/IEC 14496-1) free license to this software module or modifications   //
//thereof for use in hardware or software products claiming conformance to  //
//the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use this software//
//module in hardware or software products are advised that its use may      //
//infringe existing patents.                                                //
//The original developer of this software module and his/her company, the   //
//subsequent editors and their companies, and ISO/IEC have no liability     //
//for use of this software module or modifications thereof in an            //
//implementation. Copyright is not released for non MPEG-4 Systems          //
//(ISO/IEC 14496-1) conforming products. CSELT S.pA. retains full right     //
//to use the code for his/her own purpose, assign or donate the code to a   //
//third party and to inhibit third parties from using the code for non      //
//MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright      //
//notice must be included in all copies or derivative works.                //
//Copyright  1999.                                                         //
//This work has been partly developed within the framework of the ESPRIT    //
//European Research Project  "MPEG-4 PC".                                   //
//////////////////////////////////////////////////////////////////////////////
//
// File:  TextProxy.cpp
//
// Authors: Matthew Leditschke 
//
// Modified: Gianluca Di Cagno  gianluca.dicagno@cselt.it
//
// Description:
//   The implementation of the proxy object used by the presenter
//   to render the Text node.
//
// Created: September, 1997.
//
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include "Nodes-CSELT.h"
#include "TextProxy.h"
#include "FontStyleProxy.h"


// =====================================================================

//
// Presenter::InitText
//
// Initialise the proxy data structure and attach it to the render data
// pointer of the given media object node.
//

TextProxy* Presenter::InitText(Text* pObj)
{
return new  TextProxy(pObj); 
}




// =====================================================================

//
// TextProxy::PreRender
//
// Call the DrawableNode class's method to remember the effects and to
// register with the visual renderer.  Then, if either the Text2D node
// or its associated FontStyle node has changed since this was last
// drawn, determine the new layout of the text.
//

void TextProxy::PreRender(Effects & effects)
{
	DrawableContext *pContext=InitPreRender(effects);
	
	if (!pContext->IsSwitchedOn()) 
		return;

	if(effects.m_ParentGroup)
		effects.m_ParentGroup->AddToDrawableContextList(pContext);

	// If the transparency for this node is 1, then it doesn't need
	// to be drawn, and its bounding rectangle is empty
	if (GetTransparency(pContext) > 1 - ALMOST_ZERO)
	{
		pContext->GetBoundingRect()->MakeEmpty();
		return;
	}

	// Only proceed if the text string is not empty
	//char const* pData = (LPCSTR)((const ZString&)GetText()->string);
	char const* pData = GetText()->string[0];
	if ((pData == NULL) || (pData[0] == 0))
	{
		pContext->GetBoundingRect()->MakeEmpty();
		return;
	}

	// Only layout the text if either the Text or FontStyle node
	// has changed.
/* (always do this until ROUTEs have been fixed to update the time)
	DWORD presTime = GetPresenter()->GetTime();
	if ((GetText2D()->GetUpdateTime() >= presTime) ||
		((GetFontStyle2D() != NULL) &&
		 (GetFontStyle2D()->GetUpdateTime() >= presTime)))
 */
	{
		
		m_numLines=GetText()->string.GetLength();

		if (m_numLines != m_numLinesAllocated)
            {
               m_numLinesAllocated = m_numLines;
               int newSize = m_numLinesAllocated * sizeof(LineOfText);
               m_lines = (LineOfText *)realloc(m_lines, newSize);
            }
		
		
		
		for(int z=0;z<m_numLines;z++)
		{
			m_lines[z].pString=GetText()->string[z];
			m_lines[z].numChars =GetText()->string[z].GetLength();
			m_lines[z].length=0;
		}
		// Make a local copy of the text string for the base class to use
		m_data = pData;

		// and then determine where the text string is to be displayed
		DrawableText::PreRender(effects,pContext);
	}
	if(effects.m_ParentGroup==NULL)//necessary for postprocessing style nodes 
		UpdateBoundingRect(pContext);
	
	if(effects.orderHasChanged)
			pContext->m_updateRequired = TRUE;
} /* TextProxy::PreRender */


// =====================================================================

//
// TextProxy::Draw
//
// Draw the text string associated with this node
//

void TextProxy::Draw(BRect const* pClipBRect,DrawableContext *pContext)
{
	if(pContext->GetBoundingRect()->IsEmpty())
		return;
	DrawableText::Draw(pClipBRect,pContext);
} /* TextProxy::Draw */


// =====================================================================
