//This software module was originally developed by CSELT in the course of   //
//development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.             //
//This software module is an implementation of a part of one or more MPEG-4 //
//Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems        //
//(ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 Systems     //
//(ISO/IEC 14496-1) free license to this software module or modifications   //
//thereof for use in hardware or software products claiming conformance to  //
//the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use this software//
//module in hardware or software products are advised that its use may      //
//infringe existing patents.                                                //
//The original developer of this software module and his/her company, the   //
//subsequent editors and their companies, and ISO/IEC have no liability     //
//for use of this software module or modifications thereof in an            //
//implementation. Copyright is not released for non MPEG-4 Systems          //
//(ISO/IEC 14496-1) conforming products. CSELT S.pA. retains full right     //
//to use the code for his/her own purpose, assign or donate the code to a   //
//third party and to inhibit third parties from using the code for non      //
//MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright      //
//notice must be included in all copies or derivative works.                //
//Copyright  1999.                                                         //
//This work has been partly developed within the framework of the ESPRIT    //
//European Research Project  "MPEG-4 PC".                                   //
//////////////////////////////////////////////////////////////////////////////
// File:  TimeSensorProxy.cpp
//
// Author: Matthew Leditschke 
//
// Modified: Gianluca Di Cagno  gianluca.dicagno@cselt.it
//
// Description:
//   The implementation of the proxy object used by the presenter
//   to handle the TimeSensor node.
//
// Created: September, 1997.
//
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include "Nodes-CSELT.h"
#include "TimeSensorProxy.h"
#include <math.h>
#include "trace.h"
#include "stream.h"

// =====================================================================

//
// Presenter::InitTimeSensor
//
// Initialise the proxy data structure and attach it to the render data
// pointer of the given media object node.
//

TimeSensorProxy* Presenter::InitTimeSensor(TimeSensor* pObj)
{
	return new  TimeSensorProxy(pObj); 
}


// =====================================================================

//
// TimeSensorProxy::TimeSensorProxy
//
// Class constructor.
//

TimeSensorProxy::TimeSensorProxy(MediaObject* pMO) : Traversable(pMO)
{
	m_isActive = FALSE;
	m_prevEnabled = TRUE;
	m_prevActive = FALSE;
	m_endOfTimeReached = FALSE;
	m_startTime = 0;
} /* TimeSensorProxy::TimeSensorProxy */


// =====================================================================

//
// TimeSensorProxy::UpdateTimeSensor
//
// Update the TimeSensor node which this node is the proxy for.  This
// involves the following:
//   - determine if the timer has been made enabled or disabled,
//   - starting the timer is the start time is reached,
//   - update the time eventOut field,
//   - update the fraction_changed eventOut field,
//   - update the cycleTime eventOut field if a new cycle has been started.
//   - stoping the timer is the stop time is reached,
//

void TimeSensorProxy::UpdateTimeSensor()
{
	// This gets the time since the start of the scene
	// (convert the time to seconds, as used by BIFS)
	//double currentTime = (double)((Presenter*)GetPresenter())->GetTime() / 1000;

  
	/*-------------------------------------------------------------
	ZL: startTime / stopTime fields need to be offset by current time on first access
	-------------------------------------------------------------*/
	GetTimeSensor() -> startTime.ApplyOffset (GetTimeContext ());
	GetTimeSensor() -> stopTime.ApplyOffset (GetTimeContext ());

  double currentTime;
  // #GB: currentTime = (double)GetObject()->GetClock()/1000;
  currentTime = GetFlexTime()/1000.0;

  	// If this time sensor is not enabled, set isActive to FALSE
	// (only do this if the last time it was enabled)
	if (! (Bool) GetTimeSensor()->enabled )
	{
		if (m_prevEnabled == TRUE)
		{
			GetTimeSensor()->isActive = FALSE;
			
			GetTimeSensor()->time = currentTime;
			
			m_isActive = FALSE;
		}
		m_prevEnabled = FALSE;
		return;
	}
	m_prevEnabled = TRUE;

	// If the start time has changed since this was last called, reset things
	if ((double)GetTimeSensor()->startTime != m_startTime)
		m_endOfTimeReached = FALSE;
	m_startTime = (double)GetTimeSensor()->startTime;

	TRACE_PRINT5("isActive = %d, endReached = %d, currentTime = %.2f, startTime = %.2f, stopTime = %.2f\n",
				 m_isActive, m_endOfTimeReached, currentTime,
				 (double)GetTimeSensor()->startTime,
				 (double)GetTimeSensor()->stopTime);

	// Only proceed if the end time has not been reached
	if (m_endOfTimeReached)
		return;

	// Only proceed if the start time has been reached
	if ((currentTime < m_startTime)||(m_startTime==-1))
		return;

	// If this is the first time that the start time has been reached
	if (m_prevActive == FALSE)
	{
		// Activate this node
		GetTimeSensor()->isActive = TRUE;
		m_isActive = TRUE;
		m_prevActive = TRUE;
		m_cycleStartTime = currentTime;

		// Also send a cycleTime eventOut (as fraction is 0)
		GetTimeSensor()->cycleTime = m_cycleStartTime;

		// Store copies of the fields which can't change while this is active
		m_cycleInterval = (double)GetTimeSensor()->cycleInterval;
		m_enabled = (Bool)GetTimeSensor()->enabled;
		m_loop = (Bool)GetTimeSensor()->loop;
	}

	// Update the time eventOut field
	GetTimeSensor()->time = currentTime;

	// If a new cycle has started, and looping is FALSE, then this time
	// sensor is no longer active.
	if (currentTime - m_cycleStartTime > m_cycleInterval)
	{
		if (m_loop == FALSE)
		{
         
	      GetTimeSensor()->fraction_changed = 1.0;

			GetTimeSensor()->isActive = FALSE;
			m_isActive = FALSE;
			m_endOfTimeReached = TRUE;
			m_prevActive = FALSE;
			return;
		}

		// Otherwise update the cycleTime eventOut field
		else
		{
			// Allow for the occurrence of a number of cycle times
			m_cycleStartTime = currentTime - fmod(currentTime, m_cycleInterval);
			GetTimeSensor()->cycleTime = m_cycleStartTime;
		}
	}

	// Update the fraction_changed eventOut field,
	double newFraction = (currentTime - m_cycleStartTime) / m_cycleInterval;
	GetTimeSensor()->fraction_changed = newFraction;

	
	//handle special  case starttime=0,stoptime=0,the time sensor
	//is always running
	if((currentTime>=0)&&((double)GetTimeSensor()->stopTime)==0&&(m_startTime==0))
		return;
	
	// Determine if it is time to stop the time sensor
	if ((currentTime >= (double)GetTimeSensor()->stopTime) &&
		((double)GetTimeSensor()->stopTime > m_startTime))
	{
		GetTimeSensor()->isActive = FALSE;
		m_isActive = FALSE;
		m_endOfTimeReached = TRUE;
		m_prevActive = FALSE;
		return;
	}

} /* TimeSensorProxy::UpdateTimeSensor */





// =====================================================================

//
// TimeSensorProxy::IsFinished
//
// A TimeSensor node is considered as finished when it is not active.
//

BOOL TimeSensorProxy::IsFinished() const
{
	return !m_isActive;
} /* TimeSensorProxy::IsFinished */


// ========================================================================
