//This software module was originally developed by CSELT in the course of   //
//development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.             //
//This software module is an implementation of a part of one or more MPEG-4 //
//Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems        //
//(ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 Systems     //
//(ISO/IEC 14496-1) free license to this software module or modifications   //
//thereof for use in hardware or software products claiming conformance to  //
//the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use this software//
//module in hardware or software products are advised that its use may      //
//infringe existing patents.                                                //
//The original developer of this software module and his/her company, the   //
//subsequent editors and their companies, and ISO/IEC have no liability     //
//for use of this software module or modifications thereof in an            //
//implementation. Copyright is not released for non MPEG-4 Systems          //
//(ISO/IEC 14496-1) conforming products. CSELT S.pA. retains full right     //
//to use the code for his/her own purpose, assign or donate the code to a   //
//third party and to inhibit third parties from using the code for non      //
//MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright      //
//notice must be included in all copies or derivative works.                //
//Copyright  1999.                                                         //
//This work has been partly developed within the framework of the ESPRIT    //
//European Research Project  "MPEG-4 PC".                                   //
//////////////////////////////////////////////////////////////////////////////
//
// File:  TouchSensorProxy.cpp
//
// Author: Matthew Leditschke 
//
// Modified: Gianluca Di Cagno  gianluca.dicagno@cselt.it
//
// Description:
//   The implementation of the proxy object used by the presenter
//   to handle the TouchSensor node.
//
// Created: October, 1997.
//
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include "Nodes-CSELT.h"
#include "TouchSensorProxy.h"
#include "trace.h"


// Storage for this static data member
BOOL TouchSensorProxy::m_movementGrabbed = FALSE;


// =====================================================================

//
// Presenter::InitTouchSensor
//
// Initialise the proxy data structure and attach it to the render data
// pointer of the given media object node.
//

TouchSensorProxy* Presenter::InitTouchSensor(TouchSensor* pObj)
{
	return new  TouchSensorProxy(pObj);
}





// =====================================================================

//
// TouchSensorProxy::OnUserEvent
//
// This function is called when a user event has occurred above a node
// which is in the same grouping node hierarchy as this sensor node.
//
// If this node is registered with the presenter to received events,
// it will be called a second time for this scene draw.
//
// If this touch sensor is enabled, then generate the required events
// and return TRUE, otherwise return FALSE.
//

BOOL TouchSensorProxy::OnUserEvent(EventInfo const* pEvent)
{
	TRACE_PRINT("In TouchSensorProxy::OnUserEvent():\n");

	// Only generate events if this node is enabled, and no other
	// touch sensor node has grabbed all motion events
	if (((BOOL)(Bool)GetTouchSensor()->enabled) &&
		((!IsMovementGrabbed()) ||
		((IsMovementGrabbed()) && (m_mouseDown))))
	{
		// If this is called during pass 2, and the previous pass was
		// also pass 2, then the mouse has moved off the geometry
		// object.
		if ((pEvent->pass == 2) && (m_prevPass == 2))
		{
			// Generate an isOver=FALSE event if this is a mouse move
			if (pEvent->m_event == EVENT_MOUSE_MOVE)
			{
				if ((BOOL)(Bool)GetTouchSensor()->isOver)
					GetTouchSensor()->isOver = FALSE;
			}

			// Generate an isActive=FALSE event if the button is up
			if (pEvent->m_event == EVENT_MOUSE_UP)
			{
				GetTouchSensor()->isActive = FALSE;
				SetGrabbedOff();
			  	m_mouseDown = FALSE;
            			// Also generate a touchTime event (in seconds)
        //  #GB: GetTouchSensor()->touchTime = (Time) ((Presenter*)GetPresenter())->GetTime() / 1000.0;
        GetTouchSensor()->touchTime = (Time) GetFlexTime()/1000.0;

        //	TRACE_PRINT1("TouchTime %d",GetTouchSensor()->touchTime); 
			}

			// Unregister with the presenter if the mouse is now up
			if (!m_mouseDown)
			{
				((Presenter*)GetPresenter())->UnregisterEventHandler(this);
				m_registered = FALSE;
			}
			return TRUE;
		}

		// Generate an IsOver event if this is the first time that the
		// mouse has moved here
		if ((pEvent->m_event == EVENT_MOUSE_MOVE) &&
			((BOOL)(Bool)GetTouchSensor()->isOver == FALSE))
			GetTouchSensor()->isOver = TRUE;

		// If the button is first pressed, then generate an
		// isActive = TRUE event, and grab all movement events
		if ((pEvent->m_event == EVENT_MOUSE_DOWN) && (!m_mouseDown))
		{
			GetTouchSensor()->isActive = TRUE;
			SetGrabbedOn();
			m_mouseDown = TRUE;
		}

		// If the button is first released, then generate an
		// isActive = FALSE event, and ungrab all movement events.
		if ((pEvent->m_event == EVENT_MOUSE_UP) && (m_mouseDown))
		{
			GetTouchSensor()->isActive = FALSE;
			SetGrabbedOff();
			m_mouseDown = FALSE;

			// Also generate a touchTime event (in seconds)
      // #GB
      //GetTouchSensor()->touchTime = (Time)((Presenter*)GetPresenter())->GetTime() / 1000.0;
			GetTouchSensor()->touchTime = (Time)GetFlexTime()/1000.0;

    } //  TRACE_PRINT1("TouchTime %d",GetTouchSensor()->touchTime); 

		// Generate a hitPoint_changed event
		Vec2f point(pEvent->m_x, pEvent->m_y);
//		GetTouchSensor()->hitPoint_changed = point;

		// If not already done, register this node with the presenter
		((Presenter*)GetPresenter())->RegisterEventHandler(this);
		m_registered = TRUE;

		m_prevPass = pEvent->pass;
		return TRUE;
	}

	// Otherwise, if not enabled, return FALSE
	else
		return FALSE;
} /* TouchSensorProxy::OnUserEvent */


// =====================================================================

//
// TouchSensorProxy::IsFinished
//
// A TouchSensor node is considered as finished when it is not enabled.
//

BOOL TouchSensorProxy::IsFinished() const
{
	return !((BOOL)(Bool)GetTouchSensor()->enabled);
} /* TouchSensorProxy::IsFinished */


// ========================================================================
