//This software module was originally developed by CSELT in the course of   //
//development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.             //
//This software module is an implementation of a part of one or more MPEG-4 //
//Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems        //
//(ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 Systems     //
//(ISO/IEC 14496-1) free license to this software module or modifications   //
//thereof for use in hardware or software products claiming conformance to  //
//the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use this software//
//module in hardware or software products are advised that its use may      //
//infringe existing patents.                                                //
//The original developer of this software module and his/her company, the   //
//subsequent editors and their companies, and ISO/IEC have no liability     //
//for use of this software module or modifications thereof in an            //
//implementation. Copyright is not released for non MPEG-4 Systems          //
//(ISO/IEC 14496-1) conforming products. CSELT S.pA. retains full right     //
//to use the code for his/her own purpose, assign or donate the code to a   //
//third party and to inhibit third parties from using the code for non      //
//MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright      //
//notice must be included in all copies or derivative works.                //
//Copyright  1999.                                                         //
//This work has been partly developed within the framework of the ESPRIT    //
//European Research Project  "MPEG-4 PC".                                   //
//////////////////////////////////////////////////////////////////////////////
//
// File:  TrMatrix2D.cpp
//
// Author: Matthew Leditschke 
//
// Modified: Gianluca Di Cagno CSELT  gianluca.dicagno@cselt.it,
//           Ananda Allys, France Telecom
//
// Description:
//   The implementation of a class which implements a 2-D transform
//   matrix.
//
// Created: August, 1997.
//
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include <stdio.h>
#include <math.h>
#include "TrMatrix2D.h"


// ======================================================================

//
// TrMatrix2D::SetIdentity
//
// Set the matrix to the identity matrix.
//


void TrMatrix2D::SetIdentity()
{
   memset(m_data,0,sizeof(m_data));

	for (int cell = 0; cell < 3; cell++)
		m_data[cell][cell] = (float)1;
} /* TrMatrix2D::SetIdentity */






// ======================================================================

//
// TrMatrix2D::SetTranslation
//
// Set the matrix to represent the given translation.
//

void TrMatrix2D::SetTranslation(float dx, float dy)
{
	SetIdentity();
	m_data[0][2] = dx;
	m_data[1][2] = dy;
} /* TrMatrix2D::SetTranslation */


// ======================================================================

//
// TrMatrix2D::SetScale
//
// Set the matrix to represent the scale (in X and Y).
//

void TrMatrix2D::SetScale(float sx, float sy)
{
	SetIdentity();
	m_data[0][0] = sx;
	m_data[1][1] = sy;
} /* TrMatrix2D::SetScale */


// ======================================================================

//
// TrMatrix2D::SetRotation
//
// Set the matrix to represent the given anticlockwise rotation
// (in radians).
//

void TrMatrix2D::SetRotation(float angle)
{
	m_data[0][0] = (float)cos((double)angle);
	m_data[0][1] = -(float)sin((double)angle);
	m_data[0][2] = (float)0;
	m_data[1][0] = (float)sin((double)angle);
	m_data[1][1] = (float)cos((double)angle);
	m_data[1][2] = (float)0;
	m_data[2][0] = (float)0;
	m_data[2][1] = (float)0;
	m_data[2][2] = (float)1;
} /* TrMatrix2D::SetRotation */


// ======================================================================

//
// TrMatrix2D::IsOnlyTranslation
//
// Return TRUE if the transform matrix specifies only translation.
//

BOOL TrMatrix2D::IsOnlyTranslation() const
{
	if ((m_data[0][0] == (float)1) && (m_data[0][1] == (float)0) &&
		(m_data[1][0] == (float)0) && (m_data[1][1] == (float)1))
		return TRUE;
	else
		return FALSE;
} /* TrMatrix2D::IsOnlyTranslation */


// ======================================================================

//
// TrMatrix2D::ContainsRotation
//
// Return TRUE if the transform matrix contains some rotation.
//

BOOL TrMatrix2D::ContainsRotation() const
{
	if ((m_data[0][1] == (float)0) && (m_data[1][0] == (float)0))
		return FALSE;
	else
		return TRUE;
} /* TrMatrix2D::ContainsRotation */


// ======================================================================

//
// TrMatrix2D::Apply
//
// Apply the transform to the given point (given two floats).  This
// assumes that the given point is placed in a column vector, and then
// the matrix multiplication is performed.
//

void TrMatrix2D::Apply(float& x, float& y) const
{
	float newX = m_data[0][0] * x + m_data[0][1] * y + m_data[0][2];
	float newY = m_data[1][0] * x + m_data[1][1] * y + m_data[1][2];
	x = newX;
	y = newY;
} /* TrMatrix2D::Apply */


// ======================================================================

//
// TrMatrix2D::Apply
//
// Apply the transform to the given point (given two long integers).  This
// assumes that the given point is placed in a column vector, and then
// the matrix multiplication is performed.
//

void TrMatrix2D::Apply(long& x, long& y) const
{
	float newX = m_data[0][0] * (float)x + m_data[0][1] * (float)y + m_data[0][2];
	float newY = m_data[1][0] * (float)x + m_data[1][1] * (float)y + m_data[1][2];
	x = (long)(newX + 0.5);
	y = (long)(newY + 0.5);
} /* TrMatrix2D::Apply */


// ======================================================================

//
// TrMatrix2D::Apply
//
// Apply the transform to the given point (given two long integers).  This
// assumes that the given point is placed in a column vector, and then
// the matrix multiplication is performed.
//

void TrMatrix2D::Apply(int& x, int& y) const
{
	float newX = m_data[0][0] * (float)x + m_data[0][1] * (float)y + m_data[0][2];
	float newY = m_data[1][0] * (float)x + m_data[1][1] * (float)y + m_data[1][2];
	(newX>=0) ?	(x = (int)(newX + 0.5)) : (x=(int)(newX-0.5));
	(newY>=0) ?	(y = (int)(newY + 0.5)) : (y=(int)(newY-0.5));
	//y = (int)(newY + 0.5);
} /* TrMatrix2D::Apply */

// ======================================================================

//
// TrMatrix2D::ApplyScale
//
// Apply the transform to the given point (given two long integers).  This
// assumes that the given point is placed in a column vector, and then
// the matrix multiplication is performed.
//

void TrMatrix2D::ApplyScale(int& x, int& y) const
{
	double scaleX = sqrt((double (m_data[0][0] * m_data[0][0] + m_data[0][1] * m_data[0][1])));
	double scaleY = sqrt((double (m_data[1][0] * m_data[1][0] + m_data[1][1] * m_data[1][1])));
	x*=scaleX;
	y*=scaleY;
	//y = (int)(newY + 0.5);
} /* TrMatrix2D::Apply */



// ======================================================================

//
// operator*(TrMatrix2D, TrMatrix2D)
//
// Multiple two matricies together, returning a third.
//


TrMatrix2D operator* (const TrMatrix2D& mat1,
                             const TrMatrix2D& mat2)
{
   TrMatrix2D result;
   float fTmp;

   for (int r = 0; r < 3; r++)
   {
      for (int c = 0; c < 3; c++)
      {
         fTmp = (float)0;
         for (int i = 0; i < 3; i++)
         {
            fTmp += mat1.m_data[r][i] * mat2.m_data[i][c];
         }
         result.m_data[r][c] = fTmp;
      }
   }

   return result;
} /* operator*(TrMatrix2D, TrMatrix2D) */




// ======================================================================

//
// operator==(TrMatrix2D, TrMatrix2D)
//
// Test for equality, returning 1 if the two matricies are the same,
// or 0 if they are different.
//

int operator== (const TrMatrix2D& mat1,
				const TrMatrix2D& mat2)
{
	for (int r = 0; r < 3; r++)
		for (int c = 0; c < 3; c++)
			if (mat1.m_data[r][c] != mat2.m_data[r][c])
				return 0;

	return 1;
} /* operator==(TrMatrix2D, TrMatrix2D) */


// ======================================================================

//
// operator==(TrMatrix2D, TrMatrix2D)
//
// Test for inequality, returning 1 if the two matricies are the same,
// or 0 if they are different.
//

int operator!= (const TrMatrix2D& mat1,
				const TrMatrix2D& mat2)
{
	return !operator==(mat1, mat2);
} /* operator!=(TrMatrix2D, TrMatrix2D) */


// ======================================================================

//
// TrMatrix2D::Print
//
// Print out the contents of this matrix (only intended for debugging).
//

void TrMatrix2D::Print(FILE* outfile) const
{
	for (int row = 0; row < 3; row++)
	{
		for (int col = 0; col < 3; col++)
			fprintf(outfile, "%8.4f ", m_data[row][col]);
		fprintf(outfile,"\n");
	}
} /* TrMatrix2D::Print */


// ======================================================================

#ifdef MAKE_TEST_APP

int main()
{
	TrMatrix2D m1, m2, m3;

	m1.SetIdentity();
	m1.Print(stdout);
	printf("\n");

	m2.SetTranslation((float)10, (float)20);
	m2.Print(stdout);
	printf("\n");

	m3 = m2 * m1;
	m3.Print(stdout);
	printf("\n");

	return 0;
} /* main */

#endif // MAKE_TEST_APP

// ======================================================================
