//This software module was originally developed by CSELT in the course of   //
//development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.             //
//This software module is an implementation of a part of one or more MPEG-4 //
//Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems        //
//(ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 Systems     //
//(ISO/IEC 14496-1) free license to this software module or modifications   //
//thereof for use in hardware or software products claiming conformance to  //
//the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use this software//
//module in hardware or software products are advised that its use may      //
//infringe existing patents.                                                //
//The original developer of this software module and his/her company, the   //
//subsequent editors and their companies, and ISO/IEC have no liability     //
//for use of this software module or modifications thereof in an            //
//implementation. Copyright is not released for non MPEG-4 Systems          //
//(ISO/IEC 14496-1) conforming products. CSELT S.pA. retains full right     //
//to use the code for his/her own purpose, assign or donate the code to a   //
//third party and to inhibit third parties from using the code for non      //
//MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright      //
//notice must be included in all copies or derivative works.                //
//Copyright  1999.                                                         //
//This work has been partly developed within the framework of the ESPRIT    //
//European Research Project  "MPEG-4 PC".                                   //
//////////////////////////////////////////////////////////////////////////////
//
// File:  TrMatrix2D.h
//
// Author: Matthew Leditschke
//
// Modified:
//
// Description:
//   The definition of a class which implements a 2-D transform
//   matrix.
//
// Created: August, 1997.
//
//
///////////////////////////////////////////////////////////////////////////


#ifndef TRANSFORM_MATRIX_2D_H
#define TRANSFORM_MATRIX_2D_H


class TrMatrix2D
{
	//
	// Public functions
	//
public:
	
	
	// Set the matrix to the identity matrix
	void SetIdentity();

	// Set the matrix to represent the given translation
	void SetTranslation(float dx, float dy);

	// Set the matrix to represent the scale (in X and Y)
	void SetScale(float sx, float sy);

	// Set the matrix to represent the given anticlockwise rotation
	// (in radians)
	void SetRotation(float angle);

	// Return TRUE if the transform matrix specifies only translation.
	BOOL IsOnlyTranslation() const;

	// Return TRUE if the transform matrix contains some rotation.
	BOOL ContainsRotation() const;

	// Apply the transform to the given point.  This assumes that the
	// given point is placed in a column vector, and then the matrix
	// multiplication is performed.
	void Apply(float& x, float& y) const;
	void Apply(long& x, long& y) const;
	void Apply(int& x, int& y) const;
     
	void ApplyScale(int& width,int& height) const;

	// Multiple two matricies together, returning a third
	friend TrMatrix2D operator* (const TrMatrix2D& mat1,
										const TrMatrix2D& mat2);

	// Test for equality
	friend int operator== (const TrMatrix2D& mat1,
						   const TrMatrix2D& mat2);

	// Test for inequality
	friend int operator!= (const TrMatrix2D& mat1,
						   const TrMatrix2D& mat2);

	// Print out the contents of this matrix (only intended for debugging)
	void Print(FILE* outfile) const;


	//
	// Private implementation details
	//
public:
	float m_data[3][3];
};


#endif // TRANSFORM_MATRIX_2D_H

