//This software module was originally developed by CSELT in the course of   //
//development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.             //
//This software module is an implementation of a part of one or more MPEG-4 //
//Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems        //
//(ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 Systems     //
//(ISO/IEC 14496-1) free license to this software module or modifications   //
//thereof for use in hardware or software products claiming conformance to  //
//the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use this software//
//module in hardware or software products are advised that its use may      //
//infringe existing patents.                                                //
//The original developer of this software module and his/her company, the   //
//subsequent editors and their companies, and ISO/IEC have no liability     //
//for use of this software module or modifications thereof in an            //
//implementation. Copyright is not released for non MPEG-4 Systems          //
//(ISO/IEC 14496-1) conforming products. CSELT S.pA. retains full right     //
//to use the code for his/her own purpose, assign or donate the code to a   //
//third party and to inhibit third parties from using the code for non      //
//MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright      //
//notice must be included in all copies or derivative works.                //
//Copyright  1999.                                                         //
//This work has been partly developed within the framework of the ESPRIT    //
//European Research Project  "MPEG-4 PC".                                   //
//////////////////////////////////////////////////////////////////////////////
//
// File:  Transform2DProxy.cpp
//
// Author: Matthew Leditschke 
//
// Modified: Gianluca Di Cagno  gianluca.dicagno@cselt.it
//
// Description:
//   The implementation of the proxy object used by the presenter
//   to render the Transform2D node.
//
// Created: September, 1997.
//
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include "Nodes-CSELT.h"
#include "Transform2DProxy.h"
#include "trace.h"


// =====================================================================

//
// Presenter::InitTransform2D
//
// Initialise the proxy data structure and attach it to the render data
// pointer of the given media object node.
//

Transform2DProxy* Presenter::InitTransform2D(Transform2D* pObj)
{
	return new  Transform2DProxy(pObj);
}


// =====================================================================

//
// Transform2DProxy::PreRender
//
// Add the transform parameters of this node to the given effects, and
// then apply the PreRender method to all of the children of this node.
//

void Transform2DProxy::PreRender(Effects & effects)
{
	Effects newEffects;

	//
	// Determine the matrix for this transform
	//
	TrMatrix2D C, Cinv;		// Center of rotation
	C.SetTranslation(((Vec2f)GetTransform2D()->center).x,
					 ((Vec2f)GetTransform2D()->center).y);
	Cinv.SetTranslation(-((Vec2f)GetTransform2D()->center).x,
						-((Vec2f)GetTransform2D()->center).y);
//	TRACE_PRINT("\nC:\n");
//	C.Print(traceFile);
//	TRACE_PRINT("\nCinv:\n");
//	Cinv.Print(traceFile);

	TrMatrix2D R;			// Rotation
	R.SetRotation((float)GetTransform2D()->rotationAngle);
//	TRACE_PRINT("\nR:\n");
//	R.Print(traceFile);

	TrMatrix2D S;			// Scale
	S.SetScale(((Vec2f)GetTransform2D()->scale).x,
			   ((Vec2f)GetTransform2D()->scale).y);
//	TRACE_PRINT("\nS:\n");
//	S.Print(traceFile);

	TrMatrix2D SR, SRinv;	// Scale orientation
	SR.SetRotation((float)GetTransform2D()->scaleOrientation);
	SRinv.SetRotation(-(float)GetTransform2D()->scaleOrientation);
//	TRACE_PRINT("\nSR:\n");
//	SR.Print(traceFile);
//	TRACE_PRINT("\nSRinv:\n");
//	SRinv.Print(traceFile);

	TrMatrix2D T;			// Translation
	T.SetTranslation(((Vec2f)GetTransform2D()->translation).x,
					 ((Vec2f)GetTransform2D()->translation).y);
//	TRACE_PRINT("\nT:\n");
//	T.Print(traceFile);

	// Determine the new transform matrix and drawing order
	newEffects.transform = effects.transform * T * C * R * SR * S * SRinv * Cinv;
	
	
	
	
	newEffects.scaleX=((Vec2f)GetTransform2D()->scale).x;
	newEffects.scaleY=((Vec2f)GetTransform2D()->scale).y;

	newEffects.center.x=((Vec2f)GetTransform2D()->center).x;
	newEffects.center.y=((Vec2f)GetTransform2D()->center).y;
	// Copy over the isSwitchOn field unchanged
	newEffects.orderHasChanged=effects.orderHasChanged;
	newEffects.isSwitchedOn = effects.isSwitchedOn;
	newEffects.m_pHandler=effects.m_pHandler;
	

	newEffects.m_ParentGroup=effects.m_ParentGroup;
	// Apply these new effects to the children nodes
	PreRenderChildren(newEffects);
} /* Transform2DProxy::PreRender */



// =====================================================================
