//This software module was originally developed by CSELT in the course of   //
//development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.             //
//This software module is an implementation of a part of one or more MPEG-4 //
//Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems        //
//(ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 Systems     //
//(ISO/IEC 14496-1) free license to this software module or modifications   //
//thereof for use in hardware or software products claiming conformance to  //
//the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use this software//
//module in hardware or software products are advised that its use may      //
//infringe existing patents.                                                //
//The original developer of this software module and his/her company, the   //
//subsequent editors and their companies, and ISO/IEC have no liability     //
//for use of this software module or modifications thereof in an            //
//implementation. Copyright is not released for non MPEG-4 Systems          //
//(ISO/IEC 14496-1) conforming products. CSELT S.pA. retains full right     //
//to use the code for his/her own purpose, assign or donate the code to a   //
//third party and to inhibit third parties from using the code for non      //
//MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright      //
//notice must be included in all copies or derivative works.                //
//Copyright  1999.                                                         //
//This work has been partly developed within the framework of the ESPRIT    //
//European Research Project  "MPEG-4 PC".                                   //
//////////////////////////////////////////////////////////////////////////////
//
// File:  Traversable.cpp
//
// Author: Matthew Leditschke 
//
// Description: Gianluca Di Cagno  gianluca.dicagno@cselt.it
//
//   The implementation of methods of the class which encapsulates all
//   of the features of a nodes used by the visual renderer.
//   This file contains the implementation of methods common to all nodes
//   which are traversable.
//
// Created: August, 1997.
//
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include <assert.h>
#include "Traversable.h"
#include "Presenter.h"

// ========================================================================

//
// Traversable::Traversable
//
// Class constructor.  Set the pointer to the media object that
// this object is attached to, and get pointers to the presenter
// and the visual and audio renderers, which can be used by
// derived classes.
//

Traversable::Traversable(MediaObject* pMO) : m_pMediaObject(pMO)
{
	m_pPresenter      = (Presenter*)pMO->GetExecutive()->GetPresenter();
	m_pVisualRenderer = m_pPresenter->GetVisualRenderer();
	m_pAudioRenderer  = m_pPresenter->GetAudioRenderer();
} /* Traversable::Traversable */





// ========================================================================

//
// Traversable::BoolToStr
//
// Return a string which displays the value of the given Bool.
//

const char* Traversable::BoolToStr(Bool b) const
{
	if (b)
		return "TRUE";
	else
		return "FALSE";
} /* Traversable::BoolToStr */


// ========================================================================

//
// Traversable::FindTouchSensor
//
// This is the default behaviour for a leaf node, which involves calling
// FindTouchSensor() of its enclosing grouping node.
//

void Traversable::FindTouchSensor(EventInfo const* pEvent,DrawableContext *pContext)
{
	// Go through all of the parents
	int whichParent = 0;
	MediaObject* pParent = (MediaObject *) GetObject()->GetParent(whichParent);
	while (pParent != NULL)
	{
		((Traversable*)pParent->GetRenderingObject())->FindTouchSensor(pEvent,pContext);
		whichParent++;
		pParent = (MediaObject *) GetObject()->GetParent(whichParent);
	}
} /* Traversable::FindTouchSensor */


// ========================================================================
