//This software module was originally developed by CSELT in the course of   //
//development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.             //
//This software module is an implementation of a part of one or more MPEG-4 //
//Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems        //
//(ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 Systems     //
//(ISO/IEC 14496-1) free license to this software module or modifications   //
//thereof for use in hardware or software products claiming conformance to  //
//the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use this software//
//module in hardware or software products are advised that its use may      //
//infringe existing patents.                                                //
//The original developer of this software module and his/her company, the   //
//subsequent editors and their companies, and ISO/IEC have no liability     //
//for use of this software module or modifications thereof in an            //
//implementation. Copyright is not released for non MPEG-4 Systems          //
//(ISO/IEC 14496-1) conforming products. CSELT S.pA. retains full right     //
//to use the code for his/her own purpose, assign or donate the code to a   //
//third party and to inhibit third parties from using the code for non      //
//MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright      //
//notice must be included in all copies or derivative works.                //
//Copyright  1999.                                                         //
//This work has been partly developed within the framework of the ESPRIT    //
//European Research Project  "MPEG-4 PC".                                   //
//////////////////////////////////////////////////////////////////////////////
//
// File:  Traversable.h
//
// Author: Matthew Leditschke 
//
// Modified: Gianluca Di Cagno  gianluca.dicagno@cselt.it
//
// Description:
//   The definition of the base object used by the visual renderer,
//   which encapsulates the features of a node which is traversable.
//
// Created: August, 1997.
//
//
///////////////////////////////////////////////////////////////////////////


#ifndef TRAVERSABLE_H
#define TRAVERSABLE_H


#include "MediaObj.h"
#include "Executiv.h"
#include "TrMatrix2D.h"
#include "ztypes.h"
#include  "RendObj.h"
#include  "drawablecontext.h"
#include "../core/PresBase.h" //#PAB+:

class Presenter;
class VisualRenderer;
class AudioRenderer;
class EventInfo;


//
// This class is used to store all of the attributes which effect how
// a node in the scene graph can be displayed.
//
class Effects
{
public:
	TrMatrix2D transform;
	
	
	int scaleX;
	int scaleY;
	Vec2f center;
	BOOL isSwitchedOn; // (0 off 1 on -1 deleting)

	BOOL orderHasChanged;

	class Appearance*  m_pAppearance;
	class ShapeProxy* m_pShapeProxy;
	//pointer to sensor 
	class UserEventHandlerProxy* m_pHandler;
	//the eventual special node (layout,form) who affects the 
	//rendering of his children
	class GroupNodeProxy *  m_ParentGroup;
};


class Traversable : public RenderingObject
{
	//
	// Public functions
	//
public:

	Traversable(MediaObject* pMO);
	// Class constructor.  Set the pointer to the media object that
	// this object is attached to, and get pointers to the presenter
	// and the visual and audio renderers, which can be used by
	// derived classes.

	virtual ~Traversable() {}
	// We need a virtual destructor

	virtual void UpdateTimeSensor() {}
	// This function will recursively traverse the scene tree,
	// updating all TimeSensor nodes.  This traversal of the scene
	// tree will be done before the PreRender traversal, as the time
	// sensor nodes will generate changes to fields which need to be
	// reflected in the drawing of the scene.
	//
	// The default implementation is to do nothing.  Grouping nodes
	// and the TimeSensor node will override this.

	virtual void PreRender(Effects & effects) = 0;

	// This function will recursively traverse the scene tree,
	// determining where every object will appear (cumulation of
	// all transformation matricies).
	//
	// During this traversal, a node which is to be drawn has to call
	// VisualRenderer::AddToDrawList().  Even if the node has not
	// changed since PreRender() was last called, the node must
	// still be registered with the VisualRenderer.

	
	
	virtual BOOL IsFinished() const = 0;
	// Return TRUE if this node has finished, or FALSE otherwise.  Once
	// all nodes in the scene are finished, the Presenter stops rendering
	// the scene.

	virtual void FindTouchSensor(EventInfo const* pEvent,DrawableContext *pContext);
	// Look for a touch sensor node.  Calling this function will do
	// one of two things.  If this is a leave node, it will call
	// FindTouchSensor() of its enclosing grouping node.  If this is
	// a grouping node, then OnUserEvent() will be called for all
	// nodes, stopping when the first TRUE return value is found.
	// If not, then FindTouchSensor() of the enclosing grouping node
	// is called, until the top of the scene tree is reached.

	virtual BOOL OnUserEvent(EventInfo const*) { return FALSE; }
	// Return TRUE if this is a TouchSensor node (which will handle
	// the given event), otherwise return FALSE.  The default
	// implementation returns FALSE.


	MediaObject* GetObject() const { return m_pMediaObject; }
	// Return a pointer to the MediaObject derived object that this
	// object is attached to.
	virtual BOOL IsSensor() { return FALSE; }
	// Return TRUE if this is a TouchSensor node (which will handle
	// the given event), otherwise return FALSE.  The default
	// implementation returns FALSE.This is called in the prerendering
	// step, and the pointer of the sensor is then stored in the drawable
	// context if TRUE is returned.

  // #GB use to get/set the flextime time
  virtual DWORD GetFlexTime() const
  {
    return m_pcTimeContext.GetTime();
  }
  virtual void SetFlexTime(const PresenterContext &pcCurrentTime)
  {
    m_pcTimeContext = pcCurrentTime;
  }
	// ZL: needed to offset startTime/stopTime
	const PresenterContext *GetTimeContext () const {return &m_pcTimeContext;}


	//
	// Protected implementation details
	//
protected:


	const char* BoolToStr(Bool b) const;
	// Return a string which displays the value of the given Bool.

	Presenter* GetPresenter() const { return m_pPresenter; }
	// Return a pointer to the presenter

	VisualRenderer* GetVisualRenderer() const { return m_pVisualRenderer; }
	// Return a pointer to the visual renderer

	AudioRenderer* GetAudioRenderer() const { return m_pAudioRenderer; }
	// Return a pointer to the audio renderer

  // #GB: This PresenterContext will store the FlexTime
  PresenterContext m_pcTimeContext;


	//
	// Private implementation details
	//
private:

	// A pointer to the MediaObject derived object that this
	// object is attached to.
	MediaObject* m_pMediaObject;

	// Remember the presenter that this object can use.
	Presenter* m_pPresenter;

	// Remember the visual renderer that this object can use.
	VisualRenderer* m_pVisualRenderer;

	// Remember the audio renderer that this object can use.
	AudioRenderer* m_pAudioRenderer;
};


#endif // TRAVERSABLE_H
