//This software module was originally developed by Panasonic Singapore      //
//Laboratories and CSELT in the course of development of the MPEG-4 Systems //
//(ISO/IEC 14496-1) standard.                                               //
//This software module is an implementation of a part of one or more MPEG-4 //
//Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems        //
//(ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 Systems     //
//(ISO/IEC 14496-1) free license to this software module or modifications   //
//thereof for use in hardware or software products claiming conformance to  //
//the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use this software//
//module in hardware or software products are advised that its use may      //
//infringe existing patents.                                                //
//The original developer of this software module and his/her company, the   //
//subsequent editors and their companies, and ISO/IEC have no liability     //
//for use of this software module or modifications thereof in an            //
//implementation. Copyright is not released for non MPEG-4 Systems          //
//(ISO/IEC 14496-1) conforming products. Panasonic Singapore Laboratories   //
//and CSELT S.pA. retains full right to use the code for his/her own        //
//purpose, assign or donate the code to a third party and to inhibit third  //
//parties from using the code for non MPEG-4 Systems (ISO/IEC 14496-1)      //
//conforming products. This copyright notice must be included in all copies //
//or derivative works.                                                      //
//Copyright  1999.                                                         //
//This work has been partly developed within the framework of the ESPRIT    //
//European Research Project  "MPEG-4 PC".                                   //
//////////////////////////////////////////////////////////////////////////////
//
// File:  ValuatorProxy.cpp
//
// Author: Law Chee Yong Panasonic Singapore Laboratories 
//
// Modified: Gianluca Di Cagno gianluca.dicagno@cselt.it
//
// Description:
//   The implementation of the proxy object used by the presenter
//   to render the ValuatorProxy node.
//   
//
// Created: January, 1999.
//
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include "Nodes-CSELT.h"
#include "ValuatorProxy.h"



// =====================================================================

//
// PresenterCSELT::InitValuator
//
// Initialise the proxy data structure and attach it to the render data
// pointer of the given media object node.
//

ValuatorProxy* Presenter::InitValuator(Valuator* pObj)
{
	return new  ValuatorProxy(pObj);
}
void ValuatorProxy::StoreInValues(float x,float y,float z,float w)
{
	m_pfIn[0] = x;
	m_pfIn[1] = y;
	m_pfIn[2] = z;
	m_pfIn[3] = y;
}

void ValuatorProxy::Assign()
{
	
	Valuator * pValuator=GetValuator();
	m_pfOut[0] = pValuator->Factor1 * m_pfIn[0] + pValuator->Offset1;
	m_pfOut[1] = pValuator->Factor2 * m_pfIn[1] + pValuator->Offset2;
	m_pfOut[2] = pValuator->Factor3 * m_pfIn[2] + pValuator->Offset3;
	m_pfOut[3] = pValuator->Factor4 * m_pfIn[3] + pValuator->Offset4;

	int nNoOfElements = pValuator->outMFColor.GetLength();
	int nCount;

	Color3f c3f;
	for (nCount = 0; nCount < nNoOfElements; nCount++) {
		c3f.r = m_pfOut[0];
		c3f.g = m_pfOut[1];
		c3f.b = m_pfOut[2];
		(pValuator->outMFColor.SetAt(nCount, c3f, FALSE));
	}

	nNoOfElements = pValuator->outMFFloat.GetLength();
	for (nCount = 0; nCount < nNoOfElements; nCount++) {
		(pValuator->outMFFloat.SetAt(nCount, m_pfOut[0], FALSE));
	}

	nNoOfElements = pValuator->outMFInt32.GetLength();
	for (nCount = 0; nCount < nNoOfElements; nCount++) {
		(pValuator->outMFInt32.SetAt(nCount, (int)(m_pfOut[0]), FALSE));
	}

	Rotation ro;
	nNoOfElements = pValuator->outMFRotation.GetLength();
	for (nCount = 0; nCount < nNoOfElements; nCount++) {
		ro.x = m_pfOut[0];
		ro.y = m_pfOut[1];
		ro.z = m_pfOut[2];
		ro.theta = m_pfOut[3];
		(pValuator->outMFRotation.SetAt(nCount, ro, FALSE));
	}

	Vec2f v2f;
	nNoOfElements = pValuator->outMFVec2f.GetLength();
	for (nCount = 0; nCount < nNoOfElements; nCount++) {
		v2f.x = m_pfOut[0];
		v2f.y = m_pfOut[1];
		(pValuator->outMFVec2f.SetAt(nCount, v2f, FALSE));
	}

	Vec3f v3f;
	nNoOfElements = pValuator->outMFVec3f.GetLength();
	for (nCount = 0; nCount < nNoOfElements; nCount++) {
		v3f.x = m_pfOut[0];
		v3f.y = m_pfOut[1];
		v3f.z = m_pfOut[2];
		(pValuator->outMFVec3f.SetAt(nCount, v3f, FALSE));
	}


	//Single values

	pValuator->outSFBool = (BOOL)(m_pfOut[0]);

	c3f = (Color3f)pValuator->outSFColor;
	c3f.r = m_pfOut[0];
	c3f.g = m_pfOut[1];
	c3f.b = m_pfOut[2];
	pValuator->outSFColor = c3f;

	pValuator->outSFFloat = m_pfOut[0];

	pValuator->outSFInt32 = (int)m_pfOut[0];

	ro = (Rotation)pValuator->outSFRotation;
	ro.x = m_pfOut[0];
	ro.y = m_pfOut[1];
	ro.z = m_pfOut[2];
	ro.theta = m_pfOut[3];
	pValuator->outSFRotation = ro;

	//pValuator->inSFTime = (double)m_pfOut[0];
	pValuator->outSFTime = (double)m_pfOut[0];
	v2f = (Vec2f)pValuator->outSFVec2f;
	v2f.x = m_pfOut[0];
	v2f.y = m_pfOut[1];
	pValuator->outSFVec2f = v2f;

	v3f = (Vec3f)pValuator->outSFVec3f;
	v3f.x = m_pfOut[0];
	v3f.y = m_pfOut[1];
	v3f.z = m_pfOut[2];
	pValuator->outSFVec3f = v3f;

	// CC 010129
	char *c = new char[100];
	sprintf(c,"%f",m_pfOut[0]);
	SFString string(c);
	pValuator->outSFString = string;
	pValuator->outSFString.OnFieldUpdate();

	if(pValuator->outMFString.GetLength())
		pValuator->outMFString.Remove ();
	pValuator->outMFString.Append((ZString)string);
	// end CC 010129
	
	pValuator->outSFBool.OnFieldUpdate();
	pValuator->outSFColor.OnFieldUpdate();
	pValuator->outMFColor.OnFieldUpdate();
	pValuator->outSFFloat.OnFieldUpdate();
	pValuator->outMFFloat.OnFieldUpdate();
	pValuator->outSFInt32.OnFieldUpdate();
	pValuator->outMFInt32.OnFieldUpdate();
	pValuator->outSFRotation.OnFieldUpdate();
	pValuator->outMFRotation.OnFieldUpdate();
	pValuator->outSFTime.OnFieldUpdate();
	pValuator->outSFVec2f.OnFieldUpdate();
	pValuator->outMFVec2f.OnFieldUpdate();
	pValuator->outSFVec3f.OnFieldUpdate();
	pValuator->outMFVec3f.OnFieldUpdate();
}