//This software module was originally developed by CSELT in the course of   //
//development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.             //
//This software module is an implementation of a part of one or more MPEG-4 //
//Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems        //
//(ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 Systems     //
//(ISO/IEC 14496-1) free license to this software module or modifications   //
//thereof for use in hardware or software products claiming conformance to  //
//the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use this software//
//module in hardware or software products are advised that its use may      //
//infringe existing patents.                                                //
//The original developer of this software module and his/her company, the   //
//subsequent editors and their companies, and ISO/IEC have no liability     //
//for use of this software module or modifications thereof in an            //
//implementation. Copyright is not released for non MPEG-4 Systems          //
//(ISO/IEC 14496-1) conforming products. CSELT S.pA. retains full right     //
//to use the code for his/her own purpose, assign or donate the code to a   //
//third party and to inhibit third parties from using the code for non      //
//MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright      //
//notice must be included in all copies or derivative works.                //
//Copyright  1999.                                                         //
//This work has been partly developed within the framework of the ESPRIT    //
//European Research Project  "MPEG-4 PC".                                   //
//////////////////////////////////////////////////////////////////////////////
//
// File: VisRend-CSELT-DD.cpp
//
// Author: Matthew Leditschke 
//
// Modified: Gianluca Di Cagno CSELT gianluca.dicagno@cselt.it
//
// Description:
//   The implementation of a class which performs visual rendering using
//   DirectDraw under Windows 95/NT.
//
// Created: August, 1997
//
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include <assert.h>
#include "VisRend-CSELT-DD.h"
#include "Copy.h"
#include "Caps.h"
#include "Executiv.h"
#include "Presenter.h"
#include "Clock.h"
#include "dx-errors.h"

#ifdef STAND_ALONE

#include "MPEG4Player.h"
#include "MPEG4PlayerView.h"
#include "FatalError.h"
#elif defined(ACTIVEX_COMPONENT)

#include "Mpeg4Cselt2DActivex.h"
#include "Mpeg4Cselt2DActivexCtl.h"
#elif defined(NETSCAPE_PLUGIN)
#include "ApplicationWrapper.h"
#endif


#include "trace.h"

// =====================================================================

//
// VisualRendererDD::VisualRendererDD
//
// Class constructor - given a pointer to the presenter and a handle
// to the window to display in.
//

VisualRendererDD::VisualRendererDD(Presenter* pPresenter, HWND hWnd)
{
	TRACE_PRINT("Entering VisualRendererDD::VisualRendererDD()\n");

	m_initOK = TRUE;

	// Remember the handle to the window in which the drawing takes place
	m_hWnd = hWnd;

	// Remember a pointer to the presenter that this renderer is part of
	m_pPresenter = pPresenter;

    //default size  just to init DirectDraw for Windowed Application the a
	//resize will be called.
	RECT WindowSize;
	GetClientRect(hWnd,&WindowSize);
//#ifdef STAND_ALONE
	m_outputWidth  = WindowSize.right;
	m_outputHeight = WindowSize.bottom;
//#elif defined(ACTIVEX_COMPONENT) //size taken from the HTML page for a activex control

//	m_outputWidth  = GetApp()->m_sizeX;
//	m_outputHeight = GetApp()->m_sizeY;
//#elif defined(NETSCAPE_PLUGIN)


//	m_outputWidth = ((NetscapeApp *) GetPresenter()->GetExecutive()->GetApp())->GetSizeX();
//	m_outputHeight = ((NetscapeApp *) GetPresenter()->GetExecutive()->GetApp())->GetSizeY();
//#endif



	// Initialise the DirectDraw routines
	m_pDirectDraw = new DirectDraw;
	char* errorString = m_pDirectDraw->Init(hWnd, m_outputWidth, m_outputHeight);
	
	// If an error has occurred, display a message and then exit the
	// application
	if (errorString != NULL)
	{
		char * displayError = new char[strlen(errorString)+1];
		strcpy(displayError,errorString);
		
		#ifndef NETSCAPE_PLUGIN
		::PostMessage(m_hWnd,WM_DDRAW_PROBLEM, NULL,(LPARAM)(LPCSTR )displayError);
		#else 
		::PostMessage(((NetscapeApp *) GetPresenter()->GetExecutive()->GetApp())->GetWindow(),
					WM_DDRAW_PROBLEM,
					NULL,
					(LPARAM)(LPCSTR )displayError);
		#endif
		

		/*CFatalError fatalErrorDialog;
		fatalErrorDialog.m_message = errorString;
		fatalErrorDialog.DoModal();*/

		m_initOK = FALSE;
	}

	// Erase the output window
	if (m_initOK)
		Clear();
		
	

	TRACE_PRINT("Leaving VisualRendererDD::VisualRendererDD()\n");

	m_nDraw=0;

	m_bErrorDetected=FALSE;
} /* VisualRendererDD::VisualRendererDD */


// =====================================================================

//
// VisualRendererDD::~VisualRendererDD
//
// Class destructor - shuts down the DirectDraw routines.
//

VisualRendererDD::~VisualRendererDD()
{
	TRACE_PRINT("Entering VisualRendererDD::~VisualRendererDD()\n");

	m_pDirectDraw->Shutdown();
	delete m_pDirectDraw;

	TRACE_PRINT("Leaving VisualRendererDD::~VisualRendererDD()\n");
} /* VisualRendererDD::~VisualRendererDD */


// =====================================================================

//
// VisualRendererDD::Init
//
// Initialise the renderer.  Call the base class implementation.
//

void VisualRendererDD::Init()
{
	VisualRenderer::Init();
} /* VisualRendererDD::Init */


// ========================================================================

//
// VisualRendererDD::Terminate
//
// Stops rendering and performs cleanups.
// Erase the contents of the window if required.
//

void VisualRendererDD::Terminate()
{
	TRACE_PRINT("Entering VisualRendererDD::Terminate():\n");

	VisualRenderer::Terminate();

	if (m_clearOnEnd)
	{
		TRACE_PRINT("Erasing the contents of the window.\n");
		Clear();
		Paint();
	}

	TRACE_PRINT("Leaving VisualRendererDD::Terminate():\n");
} /* VisualRendererDD::Terminate */


// ========================================================================

//
// VisualRendererDD::Paint
//
// Copy the given region of the offscreen bitmap to the corresponding
// region of the onscreen window.  The region is given in BIFS coordinates.
// If the given rectangle pointer is NULL, the entire window is redrawn.
//

void VisualRendererDD::Paint(void* pInfo)
{
	BRect* pBRect = (BRect*)pInfo;

	// Determine the source rectangle
	RECT srcRect;
	if (pBRect == NULL)
		SetRect(&srcRect, 0, 0, m_outputWidth, m_outputHeight);
	else
	{
		// Convert from BIFS coords to window coords
     
#ifdef ORIGIN_CENTER		
		srcRect.left   = pBRect->left + m_outputWidth / 2;
		srcRect.top    = m_outputHeight / 2 - pBRect->top;
		srcRect.right  = srcRect.left + pBRect->width;
		srcRect.bottom = srcRect.top  + pBRect->height;
#else		
		srcRect.left   = pBRect->left; // + m_outputWidth / 2;
		srcRect.top    = m_outputHeight - pBRect->top;
		srcRect.right  = srcRect.left + pBRect->width;
		srcRect.bottom = srcRect.top  + pBRect->height;
#endif
	}

	// Get the screen coordinates of the window to draw in
	RECT destRect;
	
	if(!IsWindow(m_hWnd))
		return;

	GetWindowRect(m_hWnd, &destRect);
	destRect.right=destRect.left+srcRect.right;
	destRect.bottom=destRect.top+srcRect.bottom;
// Reduce by 1 in every direction to allow for the border
//  destRect.left += 1;
//	destRect.top  += 1;
//	destRect.right  -= 1;
//	destRect.bottom -= 1;

	// If the rectangle is given, modify the destination rectangle
	// to include only this area.
	if (pBRect != NULL)
	{
		// Convert from BIFS coordinates to screen coordinates
#ifdef ORIGIN_CENTER		
		destRect.left  += pBRect->left + m_outputWidth / 2;
		destRect.top   += m_outputHeight / 2 - pBRect->top;
		destRect.right  = destRect.left + pBRect->width;
		destRect.bottom = destRect.top  + pBRect->height;
#else
        
		destRect.left  += pBRect->left; // + m_outputWidth / 2;
		destRect.top   += m_outputHeight  - pBRect->top;
		destRect.right  = destRect.left + pBRect->width;
		destRect.bottom = destRect.top  + pBRect->height;
#endif
	}

	// Copy the given rectangle from the offscreen surface to the onscreen
	HRESULT returnValue;
//DisplayLogo();
	if(!(m_pDirectDraw->RenderSurface(&srcRect, &destRect,returnValue)))
	{
		PostError(returnValue);
		/*
		char errorMessage [200];
		strcpy (errorMessage, "Error in Blt():\n");
		strcat (errorMessage, HRESULTtoText(returnValue));
		#ifndef NETSCAPE_PLUGIN
		GetView() -> PostMessage(WM_DDRAW_PROBLEM, NULL,(LPARAM)(LPCSTR )errorMessage);
		#else 
		::PostMessage(((NetscapeApp *) GetPresenter()->GetExecutive()->GetApp())->GetWindow(),
					WM_DDRAW_PROBLEM,
					NULL,
					(LPARAM)(LPCSTR )errorMessage);
		#endif*/
	
	
	}
	
//	TRACE_PRINT("In VisualRenderer::Paint():\n");
} /* VisualRendererDD::Paint */


// ========================================================================

//
// VisualRendererDD::GetCapabilities
//
// Get the capabilities of this visual renderer.
//

void VisualRendererDD::GetCapabilities (Capabilities* pCaps)
{

	
	pCaps->AddFormatCapability(MediaFormat (MediaFormat :: RGB), 0);
	pCaps->AddFormatCapability(MediaFormat (MediaFormat :: YUV), 1);
	pCaps->AddFormatCapability(MediaFormat (MediaFormat :: YUVA), 2);
} /* VisualRendererDD::GetCapabilities */


// ========================================================================

//
// VisualRendererDD::Clear
//
// Completely erase the specifed region of the offscreen drawing
// area, but do not display the surface.  If the given rectangle
// is NULL, erase the entire offscreen area.
//

void VisualRendererDD::Clear(BRect const* pBRect)
{
	BOOL bIsAllFine;
	HRESULT returnValue;
	RECT updateRect;
	if (pBRect == NULL)
	{
		bIsAllFine=m_pDirectDraw->EraseSurface(NULL,returnValue);
		if (!bIsAllFine)
			PostError(returnValue);
	}
	else
	{
		// Convert from BIFS coords to window coords
#ifdef ORIGIN_CENTER
		updateRect.left   = pBRect->left + m_outputWidth / 2;
		updateRect.top    = m_outputHeight / 2 - pBRect->top;
		updateRect.right  = updateRect.left + pBRect->width;
		updateRect.bottom = updateRect.top  + pBRect->height;
#else
		updateRect.left   = pBRect->left;// + m_outputWidth / 2;
		updateRect.top    = m_outputHeight  - pBRect->top;
		updateRect.right  = updateRect.left + pBRect->width;
		updateRect.bottom = updateRect.top  + pBRect->height;
#endif

		bIsAllFine=m_pDirectDraw->EraseSurface(&updateRect,returnValue);
		if (!bIsAllFine)
			PostError(returnValue);
	}
} /* VisualRendererDD::Clear */


// ========================================================================

//
// VisualRendererDD::DisplayRGBAImage
//
// Display the given RGBA image, which includes where to display
// the top-left pixel in the image.  This uses the BIFS coordinate
// system, with the origin of the output display is in the middle
// of the window, with increasing X and Y to the right and up.
//

void VisualRendererDD::DisplayRGBAImage(SrcRGBAImage const& srcImage)
{
	// Change the (left,top) point from having the origin in the
	// middle of the screen to having the origin at the top-left
	// of the output window
	SrcRGBAImage flippedSrcImage = srcImage;

#ifdef ORIGIN_CENTER

	/*flippedSrcImage.top = m_outputHeight / 2 - flippedSrcImage.top;
	flippedSrcImage.left += m_outputWidth / 2;*/

	flippedSrcImage.top = m_outputHeight / 2 - flippedSrcImage.top;
	flippedSrcImage.left += m_outputWidth / 2;

#else

	flippedSrcImage.top = m_outputHeight  - flippedSrcImage.top;
//	flippedSrcImage.left += m_outputWidth / 2;
#endif

	// Lock the surface for writing
	DDSURFACEDESC ddSurfaceDesc;
	//Test if scaling is required from source to dest, 
	//if so copy pixel to third surface and then 
	//blit to secondary..
	HRESULT returnValue;
	BOOL bIsAllFine;
	((srcImage.scaleX==1)&&(srcImage.scaleY==1)) ? bIsAllFine=m_pDirectDraw->LockSurface(ddSurfaceDesc,RGB,returnValue) :bIsAllFine=m_pDirectDraw->LockSurface(ddSurfaceDesc,POOL,returnValue);

	if(!bIsAllFine)
		PostError(returnValue);

	//if the image is scaled copy it starting from the top of the POOL
	//surface
	
	if((srcImage.scaleX!=1)||(srcImage.scaleY!=1))
	{
		flippedSrcImage.top = 0;
	    flippedSrcImage.left = 0;
	}
	SurfaceDesc surfaceDesc;
	surfaceDesc.dwWidth   = ddSurfaceDesc.dwWidth;
	surfaceDesc.dwHeight  = ddSurfaceDesc.dwHeight;
	surfaceDesc.lPitch    = ddSurfaceDesc.lPitch;
	surfaceDesc.lpSurface = ddSurfaceDesc.lpSurface;

	// Copy the image (or a portion of it)
	switch (m_pDirectDraw->GetRGBpixelFormat())
	{
	case RGB555:
		CopyRGBAToRGB555(flippedSrcImage, surfaceDesc);
		break;
	case RGB565:
		CopyRGBAToRGB565(flippedSrcImage, surfaceDesc);
		break;
	case RGB24bpp:
		CopyRGBAToRGB24bpp(flippedSrcImage, surfaceDesc);
		break;
	case RGB32bpp:
		CopyRGBAToRGB32bpp(flippedSrcImage, surfaceDesc);
		break;
	case BGR32bpp:
		CopyRGBAToBGR32bpp(flippedSrcImage, surfaceDesc);
		break;
	default:
		assert(0);		// Shouldn't get here.
		break;
	}

	// Finished writing to the surface, so unlock it
	((srcImage.scaleX==1)&&(srcImage.scaleY==1)) ? bIsAllFine=m_pDirectDraw->UnlockSurface(RGB,returnValue) :bIsAllFine=m_pDirectDraw->UnlockSurface(POOL,returnValue);
	
	if(!bIsAllFine)
		PostError(returnValue);

	//scale it if required (hardware scale if supported)
	if (!((srcImage.scaleX==1)&&(srcImage.scaleY==1)))
	{
		
		if(((srcImage.height)<=1)||((srcImage.width)<=1))
			return;
		RECT rectSrc,rectDst;
		rectSrc.left=flippedSrcImage.left;
		rectSrc.top=flippedSrcImage.top;
		rectSrc.right=	rectSrc.left+flippedSrcImage.width;
		rectSrc.bottom=	rectSrc.top+flippedSrcImage.height;
		rectDst.left=srcImage.left+m_outputWidth / 2;
		rectDst.top=m_outputHeight / 2 - srcImage.top;
		rectDst.right=rectDst.left+flippedSrcImage.width*srcImage.scaleX;
		rectDst.bottom=rectDst.top+flippedSrcImage.height*srcImage.scaleY;
		HRESULT returnValue;
		BOOL bIsAllFine;
		bIsAllFine=m_pDirectDraw->ScaleOffScreenRGB(rectSrc,rectDst,returnValue);
		if(!bIsAllFine)
			PostError(returnValue);

	}
} /* VisualRendererDD::DisplayRGBAImage */


// ========================================================================

//
// VisualRendererDD::DisplayYUV420AImage
//
// Display the given YUV4:2:0 image, which includes where to display
// the top-left pixel in the image.  This uses the BIFS coordinate
// system, with the origin of the output display is in the middle
// of the window, with increasing X and Y to the right and up.
//

void VisualRendererDD::DisplayYUV420AImage(SrcYUVAImage const& srcImage)
{
/*	can't understand this, the code looks ok  without this...
	// The offsets must be even
	assert ((srcImage.left % 2) == 0);
	assert ((srcImage.top % 2) == 0);

//	 The source width and height must be even - for now
	assert ((srcImage.width % 2) == 0);
	assert ((srcImage.height % 2) == 0);

*/

	// Change the (left,top) point from having the origin in the
	// middle of the screen to having the origin at the top-left
	// of the output window
	SrcYUVAImage flippedSrcImage = srcImage;
#ifdef ORIGIN_CENTER	
	flippedSrcImage.top = m_outputHeight / 2 - flippedSrcImage.top;
	flippedSrcImage.left += m_outputWidth / 2;
#else
    
	flippedSrcImage.top = m_outputHeight  - flippedSrcImage.top;
	//flippedSrcImage.left += m_outputWidth / 2;
#endif	
	// Lock the surface for writing
	DDSURFACEDESC ddSurfaceDesc;
	
	//Test if scaling is required from source to dest, 
	//if so copy pixel to third surface and then 
	//blit to secondary..
	
	HRESULT returnValue;
	BOOL bIsAllFine;
	((srcImage.scaleX==1)&&(srcImage.scaleY==1)) ? bIsAllFine=m_pDirectDraw->LockSurface(ddSurfaceDesc,RGB,returnValue) :bIsAllFine=m_pDirectDraw->LockSurface(ddSurfaceDesc,POOL,returnValue);

	if(!bIsAllFine)
		PostError(returnValue);
	
	//if the image is scaled copy it starting from the top of the POOL
	//surface
	
	if((srcImage.scaleX!=1)||(srcImage.scaleY!=1))
	{
		flippedSrcImage.top = 0;
	    flippedSrcImage.left = 0;
	}
	
	
	SurfaceDesc surfaceDesc;
	surfaceDesc.dwWidth   = ddSurfaceDesc.dwWidth;
	surfaceDesc.dwHeight  = ddSurfaceDesc.dwHeight;
	surfaceDesc.lPitch    = ddSurfaceDesc.lPitch;
	surfaceDesc.lpSurface = ddSurfaceDesc.lpSurface;
	
	// Copy the image (or a portion of it)
	switch (m_pDirectDraw->GetRGBpixelFormat())
	{
	case RGB555:
		CopyYUVToRGB555(flippedSrcImage, surfaceDesc);
		break;
	case RGB565:
		CopyYUVToRGB565(flippedSrcImage, surfaceDesc);
		break;
	case RGB24bpp:
		CopyYUVToRGB24bpp(flippedSrcImage, surfaceDesc);
		break;
	case RGB32bpp:
		CopyYUVToRGB32bpp(flippedSrcImage, surfaceDesc);
		break;
	case BGR32bpp:
		CopyYUVToBGR32bpp(flippedSrcImage, surfaceDesc);
		break;
	default:
		assert(0);		// Shouldn't get here.
		break;
	}

	// Finished writing to the surface, so unlock it

	((srcImage.scaleX==1)&&(srcImage.scaleY==1)) ? bIsAllFine=m_pDirectDraw->UnlockSurface(RGB,returnValue) :bIsAllFine=m_pDirectDraw->UnlockSurface(POOL,returnValue);
	
	if(!bIsAllFine)
		PostError(returnValue);

	//scale it if required (hardware scale if supported)
	if (!((srcImage.scaleX==1)&&(srcImage.scaleY==1)))
	{
	
		if(((srcImage.height)<=1)||((srcImage.width)<=1))
			return;
		RECT rectSrc,rectDst;
		rectSrc.left=flippedSrcImage.left;
		rectSrc.top=flippedSrcImage.top;
		rectSrc.right=	rectSrc.left+flippedSrcImage.width;
		rectSrc.bottom=	rectSrc.top+flippedSrcImage.height;
		rectDst.left=srcImage.left+m_outputWidth / 2;
		rectDst.top=m_outputHeight / 2 - srcImage.top;
		rectDst.right=rectDst.left+flippedSrcImage.width*srcImage.scaleX;
		rectDst.bottom=rectDst.top+flippedSrcImage.height*srcImage.scaleY;
		HRESULT returnValue;
		BOOL bIsAllFine;
		bIsAllFine=m_pDirectDraw->ScaleOffScreenRGB(rectSrc,rectDst,returnValue);
		if(!bIsAllFine)
			PostError(returnValue);

	}
} /* VisualRendererDD::DisplayYUV420AImage */


// ========================================================================

//
// VisualRendererDD::DisplayCircle
//
// Display the given shape, which includes where to display
// the top-left pixel in the image.  This uses the BIFS coordinate
// system, with the origin of the output display is in the middle
// of the window, with increasing X and Y to the right and up.
//
void VisualRendererDD::DisplayCircle(BRect const& pClipBRect, 
									 PointF const* lpPoints,
									 Color3f pixelColor, int width, 
									 BOOL filled)
{
	HDC hdc;
	HRGN hrgn;
	HRESULT returnValue;
	if(!m_pDirectDraw->GetHDC(hdc,returnValue))  //do nothing if a dc can't be taken
		return;                
#ifndef ORIGIN_CENTER
	// left bottom origin
	int left = pClipBRect.left; //+ m_outputWidth / 2;
	int top = m_outputHeight - pClipBRect.top;
	int right = left + pClipBRect.width;
	int bottom = top + pClipBRect.height;
#else    
	int left = pClipBRect.left + m_outputWidth / 2;
	int top = m_outputHeight/2 - pClipBRect.top;
	int right = left + pClipBRect.width;
	int bottom = top + pClipBRect.height;
#endif

	// this must consider the width!!
	hrgn = CreateRectRgn(left-width/2, top-width/2, right+width/2, bottom+width/2); 
    SelectClipRgn(hdc, hrgn); 

	HGDIOBJ hOldBrush = 0;
	HGDIOBJ hOldPen = 0;
	CBrush brush;

	int color = RGB(255 * pixelColor.r, 255 * pixelColor.g, 255 * pixelColor.b);
	CPen pen(PS_SOLID, width, color);
	hOldPen = SelectObject(hdc, pen);

	// A sort of translation bifs center origin 
#ifndef ORIGIN_CENTER	
	// left bottom origin
	int xPos1 = lpPoints[0].x;//  + m_outputWidth / 2; 
	int yPos1 = m_outputHeight  - lpPoints[0].y; 
	int xPos2 = lpPoints[1].x;//  + m_outputWidth / 2; 
	int yPos2 = m_outputHeight  - lpPoints[1].y; 
#else
	int xPos1 = lpPoints[0].x  + m_outputWidth / 2; 
	int yPos1 = m_outputHeight/2  - lpPoints[0].y; 
	int xPos2 = lpPoints[1].x  + m_outputWidth / 2; 
	int yPos2 = m_outputHeight/2  - lpPoints[1].y; 
#endif

	if (filled)
	{
		brush.CreateSolidBrush(color);
		hOldBrush = SelectObject(hdc, brush);
	}
	else
		hOldBrush = SelectObject(hdc, GetStockObject(HOLLOW_BRUSH));

	Ellipse(hdc, xPos1, yPos1, xPos2, yPos2);

	if (hOldBrush != 0)
		SelectObject(hdc, hOldBrush);
		//DeleteObject(SelectObject(hdc, hOldBrush));

	if (hOldPen != 0)
		SelectObject(hdc, hOldPen);
		//DeleteObject(SelectObject(hdc, hOldPen));

    pen.DeleteObject();   // palm 03-sep-98
	brush.DeleteObject(); // palm 03-sep-98
	DeleteObject( hrgn ); // palm 03-sep-98

	SelectClipRgn(hdc, NULL); 
	m_pDirectDraw->ReleaseHDC(hdc,returnValue);

	TRACE_PRINT("After DisplayCircle\n");
} /* VisualRendererDD::DisplayCircle */


// ========================================================================

//
// VisualRendererDD::DisplayCurve
//
// Display the given shape, which includes where to display
// the top-left pixel in the image.  This uses the BIFS coordinate
// system, with the origin of the output display is in the middle
// of the window, with increasing X and Y to the right and up.
//
void VisualRendererDD::DisplayCurve(BRect const& pClipBRect, 
									PointF const* lpPoints,
									int const* /*lpTypes*/,					
								    Color3f pixelColor, 
									int width, int length)
{
	HDC hdc;
	HRESULT returnValue;
	if(!m_pDirectDraw->GetHDC(hdc,returnValue))  //do nothing if a dc can't be taken
		return;                
#ifdef ORIGIN_CENTER
	
	int left = pClipBRect.left + m_outputWidth / 2;
	int top = m_outputHeight / 2 - pClipBRect.top;
#else	
	int left = pClipBRect.left;
	int top = m_outputHeight  - pClipBRect.top;
#endif

	int right = left + pClipBRect.width;
	int bottom = top + pClipBRect.height;
	HRGN hrgn = CreateRectRgn(left, top, right, bottom); 
    SelectClipRgn(hdc, hrgn); 

	POINT* points = new POINT[length];

	HGDIOBJ hOldPen = 0;
	int color = RGB(255 * pixelColor.r, 255 * pixelColor.g, 255 * pixelColor.b);
	CPen pen(PS_SOLID, width, color);
	hOldPen = SelectObject(hdc, pen);

	for (int i = 0; i < length; i++)
	{
		#ifdef ORIGIN_CENTER
		points[i].x = lpPoints[i].x + m_outputWidth / 2; 
		points[i].y = m_outputHeight / 2 - lpPoints[i].y; 
		#else
		points[i].x = lpPoints[i].x; 
		points[i].y = m_outputHeight  - lpPoints[i].y; 
		#endif

	}
	

	PolyBezier(hdc, points, length);
/*
	PolyDraw(	hdc,		// handle to the device context
				points,		// pointer to array of points
				lpTypes,	// pointer to line and curve identifiers
				length );	// count of points

*/
	delete [] points;

	if (hOldPen != 0)
		DeleteObject(SelectObject(hdc, hOldPen));

	pen.DeleteObject();   

    SelectClipRgn(hdc, NULL); 
	m_pDirectDraw->ReleaseHDC(hdc,returnValue);

	TRACE_PRINT("After DisplayCurve\n");
}


// ========================================================================

//
// VisualRendererDD::DisplayPolygon
//
// Draw the portion of the given polygon which lies within the given
// clipping rectangle.  The polygon is given by an array of points
// (locations given in BFS coordinates).  The colour, line width
// and filled flag are also given.
//

void VisualRendererDD::DisplayPolygon(BRect const& clipBRect,
									  PointF const* lpPoints, int numPoints,
									  Color3f const& pixelColor,
									  int width, BOOL filled)
{
	HDC hdc;
	HRESULT returnValue;
	if(!m_pDirectDraw->GetHDC(hdc,returnValue))  //do nothing if a dc can't be taken
		return;                

	// Set up a clipping region, in screen coordinates
	#ifdef ORIGIN_CENTER
	int left = clipBRect.left + m_outputWidth / 2;
	int top = m_outputHeight / 2 - clipBRect.top;
	#else 
    int left = clipBRect.left;
	int top = m_outputHeight  - clipBRect.top;
	#endif
	int right = left + clipBRect.width;
	int bottom = top + clipBRect.height;
	HRGN hrgn = CreateRectRgn(left, top, right, bottom); 
    SelectClipRgn(hdc, hrgn); 

	// Set up the pen to draw this polygon with
	int color = RGB(255 * pixelColor.r, 255 * pixelColor.g, 255 * pixelColor.b);
	CPen pen(PS_SOLID, width, color);
	HGDIOBJ hOldPen = SelectObject(hdc, pen);

	// Make a local copy of the array of points, converting from BIFS to
	// screen coordinates at the same time.
	POINT* points = new POINT [numPoints];
	for (int i = 0; i < numPoints; i++)
	{
		#ifdef ORIGIN_CENTER
		points[i].x = lpPoints[i].x + m_outputWidth /2;
		points[i].y = m_outputHeight / 2 - lpPoints[i].y;
		#else
		points[i].x = lpPoints[i].x;
		points[i].y = m_outputHeight - lpPoints[i].y;
		#endif
	}

	// Set the brush to either filled or hollow, if the polygon is to
	// be filled or not.
	HGDIOBJ hOldBrush;
	CBrush brush;
	if (filled)
	{
		brush.CreateSolidBrush(color);
		hOldBrush = SelectObject(hdc, brush);
	}
	else
		hOldBrush = SelectObject(hdc, GetStockObject(HOLLOW_BRUSH));

	// Draw the polygon
	Polygon(hdc, points, numPoints);

	// Release stuff
	delete [] points;
	if (hOldBrush != 0)
		DeleteObject(SelectObject(hdc, hOldBrush));

	if (hOldPen != 0)
		DeleteObject(SelectObject(hdc, hOldPen));

	pen.DeleteObject();   // palm 03-sep-98
	brush.DeleteObject(); // palm 03-sep-98
	DeleteObject( hrgn ); // palm 03-sep-98

    SelectClipRgn(hdc, NULL); 
	m_pDirectDraw->ReleaseHDC(hdc,returnValue);
} /* VisualRendererDD::DisplayPolygon */


// ========================================================================

//
// VisualRendererDD::DisplayRectangle
//
// Draw a rectangle, whose size and location is given by drawBRect,
// only showing that portion which lies within the given clipping
// rectangle.  The colour, line width, filled flag and transparency
// are also given.  All coordinates are given in BIFS coordinates.
//
// If the rectangle is filled, then the rectangle is drawn by writing
// directly to the frame store.  If not, the windows routines are used.
//

void VisualRendererDD::DisplayRectangle(BRect const& clipBRect,
										BRect const& drawBRect,
										Color3f const& pixelColor,
										int width, BOOL filled,
										float transparency)
{
	// Determine how much of the rectangle is to be drawn
	BRect visibleRect = drawBRect;
	visibleRect.Intersection(clipBRect);
	
	#ifdef ORIGIN_CENTER
	visibleRect.top = m_outputHeight / 2 - visibleRect.top;
	visibleRect.left += m_outputWidth / 2;
	#else
	visibleRect.top = m_outputHeight  - visibleRect.top;
	#endif
    
    
	// Change the coordinates of the rectangle to draw from BIFS
	// to screen coordinates.
	BRect screenRect = drawBRect;

	#ifdef ORIGIN_CENTER
	screenRect.top = m_outputHeight / 2 - screenRect.top;
	screenRect.left += m_outputWidth / 2;
	#else
    screenRect.top = m_outputHeight  - screenRect.top;
	#endif	

	// Change the clip rectangle from BIFS to screen coordinates.
	BRect screenClip = clipBRect;
	#ifdef ORIGIN_CENTER
	screenClip.top = m_outputHeight / 2 - screenClip.top;
	screenClip.left += m_outputWidth / 2;
	#else
    screenClip.top = m_outputHeight  - screenClip.top;
	#endif
 
	// If the rectangle is filled, write directly to the frame store
	if (filled)
	{
		// Lock the surface for writing
		DDSURFACEDESC ddSurfaceDesc;
		HRESULT returnValue;
		BOOL bIsAllFine;
	
		bIsAllFine= m_pDirectDraw->LockSurface(ddSurfaceDesc,RGB,returnValue);
		
		if(!bIsAllFine)
			PostError(returnValue);

		SurfaceDesc surfaceDesc;
		surfaceDesc.dwWidth   = ddSurfaceDesc.dwWidth;
		surfaceDesc.dwHeight  = ddSurfaceDesc.dwHeight;
		surfaceDesc.lPitch    = ddSurfaceDesc.lPitch;
		surfaceDesc.lpSurface = ddSurfaceDesc.lpSurface;

		// Copy the image (or a portion of it)
		switch (m_pDirectDraw->GetRGBpixelFormat())
		{
		case RGB555:
			FillRGB555(visibleRect, surfaceDesc, pixelColor, transparency);
			break;
		case RGB565:
			FillRGB565(visibleRect, surfaceDesc, pixelColor, transparency);
			break;
		case RGB24bpp:
			FillRGB24bpp(visibleRect, surfaceDesc, pixelColor, transparency);
			break;
		case RGB32bpp:
			FillRGB32bpp(visibleRect, surfaceDesc, pixelColor, transparency);
			break;
		case BGR32bpp:
			FillBGR32bpp(visibleRect, surfaceDesc, pixelColor, transparency);
			break;
		default:
			assert(0);		// Shouldn't get here.
			break;
		}

		// Finished writing to the surface, so unlock it
		m_pDirectDraw->UnlockSurface(RGB,returnValue);
	}

	// Only the outline of the rectangle is drawn, use windows routines
	else
	{
		HDC hdc;
		HRESULT returnValue;
		if(!m_pDirectDraw->GetHDC(hdc,returnValue))  //do nothing if a dc can't be taken
			return;                
		CDC dc;
		dc.Attach(hdc);

		// Set up a clipping region, in screen coordinates
		HRGN hrgn = CreateRectRgn(screenClip.left, screenClip.top,
								  screenClip.left + screenClip.width,
								  screenClip.top + screenClip.height); 
		SelectClipRgn(hdc, hrgn); 

		// Set up the pen to draw this polygon with
		int color = RGB(255 * pixelColor.r, 255 * pixelColor.g, 255 * pixelColor.b);
		CPen pen(PS_SOLID, width, color);
		HGDIOBJ hOldPen = SelectObject(hdc, pen);

		// Set up a hollow brush
		HGDIOBJ hOldBrush = SelectObject(hdc, GetStockObject(HOLLOW_BRUSH));

		// Draw the frame of the rectangle
		dc.Rectangle(screenRect.left, screenRect.top,
					 screenRect.left + screenRect.width,
					 screenRect.top + screenRect.height);

		// Release stuff
		if (hOldBrush != 0)
			DeleteObject(SelectObject(hdc, hOldBrush));

		if (hOldPen != 0)
			DeleteObject(SelectObject(hdc, hOldPen));
		
		pen.DeleteObject();   // palm 03-sep-98
		DeleteObject( hrgn ); // palm 03-sep-98
		
		dc.Detach();
		SelectClipRgn(hdc, NULL); 
		m_pDirectDraw->ReleaseHDC(hdc,returnValue);
	}
} /* VisualRendererDD::DisplayRectangle */


// ========================================================================

void VisualRendererDD::DisplayLines(BRect const& clipBRect, 
									int const& startPos1, int const& startPos2,
									int const& endPos1, int const& endPos2,
								    Color3f const& pixelColor, 
								    int width)
{
	HDC hdc;
	HRESULT returnValue;
	if(!m_pDirectDraw->GetHDC(hdc,returnValue))  //do nothing if a dc can't be taken
		return;                

	// Set up a clipping region, in screen coordinates
	#ifdef ORIGIN_CENTER
	int left = clipBRect.left + m_outputWidth / 2;
	int top = m_outputHeight / 2 - clipBRect.top;
	#else
	int left = clipBRect.left;
	int top = m_outputHeight  - clipBRect.top;
	#endif

	int right = left + clipBRect.width;
	int bottom = top + clipBRect.height;
	HRGN hrgn = CreateRectRgn(left, top, right, bottom); 
    SelectClipRgn(hdc, hrgn); 

	// Set up the pen to draw this polygon with
	int color = RGB(255 * pixelColor.r, 255 * pixelColor.g, 255 * pixelColor.b);
	CPen pen(PS_SOLID, width, color);
	HGDIOBJ hOldPen = SelectObject(hdc, pen);

	// Make a local copy of the array of points, converting from BIFS to
	// screen coordinates at the same time.
	int xPos1, xPos2, yPos1, yPos2;
	
	#ifdef ORIGIN_CENTER
	xPos1 = startPos1 + m_outputWidth /2;
	yPos1 = m_outputHeight / 2 - startPos2;
	xPos2 = endPos1 + m_outputWidth /2;
	yPos2 = m_outputHeight / 2 - endPos2;
	#else
    xPos1 = startPos1;
	yPos1 = m_outputHeight - startPos2;
	xPos2 = endPos1;
	yPos2 = m_outputHeight - endPos2;
	#endif

	// Move the current position to the start position
	MoveToEx(hdc, xPos1, yPos1, NULL);

	// Draw the line
	LineTo(hdc, xPos2, yPos2);

	// Release stuff
	if (hOldPen != 0)
		DeleteObject(SelectObject(hdc, hOldPen));
	
	pen.DeleteObject();   // palm 03-sep-98
	DeleteObject( hrgn ); // palm 03-sep-98
    
	SelectClipRgn(hdc, NULL); 
	m_pDirectDraw->ReleaseHDC(hdc,returnValue);
} /* VisualRendererDD::DisplayLines */


// ========================================================================

// A structure storing details about a font and its associated device
// context.
typedef struct
{
	HDC hDC;
	HFONT hFont;
	HGDIOBJ hPrevFont;
} FontInfo;


// =====================================================================

//
// VisualRendererDD::SetFont
//
// Return a renderer specific handle to the requested font.
//

void* VisualRendererDD::SetFont(LPCTSTR fontName, int fontSize,
								BOOL bold, BOOL italic, BOOL underline)
{
	FontInfo* pFontInfo = new FontInfo;
	
	// This is just because sometimes the surfaces are empty
	// so the hdc is null, in the DrawScene function this 
	// it's not a problem since the draw function of the text node is
	// not called but the only exception is the PreRender for the text
	// that calls the Visual Renderer
	BRect screenBRect((int)GetScreenWidth() / -2, (int)GetScreenHeight() / 2,
					  (int)GetScreenWidth(), (int)GetScreenHeight());
	
	if((screenBRect.IsEmpty()))
		return NULL; 
	
	HRESULT returnValue;

	if(!m_pDirectDraw->GetHDC(pFontInfo->hDC,returnValue))  //do nothing if a dc can't be taken
		return NULL;       
	
	// Create a description of the required font,
	POINT pt;
	pt.x=0;
	pt.y=fontSize; //fontSize;
	DPtoLP(pFontInfo->hDC,&pt,1);

	LOGFONT font;
	font.lfHeight = pt.y;//-(int)(fabs(fontSize)/10.0+0.5);//-MulDiv(fontSize, GetDeviceCaps(pFontInfo->hDC, LOGPIXELSY), 72);
	font.lfWidth = 0;
	font.lfEscapement = 0;
	font.lfOrientation = 0;
	font.lfWeight = (bold) ? FW_BOLD : FW_NORMAL;
	font.lfItalic = italic;
	font.lfUnderline = underline;
	font.lfStrikeOut = FALSE;
	font.lfCharSet = ANSI_CHARSET;
	font.lfOutPrecision = OUT_DEFAULT_PRECIS;
	font.lfClipPrecision = CLIP_DEFAULT_PRECIS;
	font.lfQuality = DEFAULT_QUALITY;
	font.lfPitchAndFamily = DEFAULT_PITCH | FF_DONTCARE;
	if(_tcscmp(fontName,_T("SANS"))==0) 
		_tcscpy(font.lfFaceName, _T("Arial"));
	else if (_tcscmp(fontName,_T("SERIF"))==0)   
		_tcscpy(font.lfFaceName, _T("Times New Roman"));
	else if (_tcscmp(fontName,_T("TYPEWRITER"))==0)   
		_tcscpy(font.lfFaceName, _T("Courier New"));

	    		
	

	// and now create the font,
	pFontInfo->hFont = CreateFontIndirect(&font);
	assert(pFontInfo->hFont != NULL);
	
	TEXTMETRIC  tm;                 // Actual Font metrics

       


	

	// and make it the current font.
	pFontInfo->hPrevFont = SelectObject(pFontInfo->hDC, pFontInfo->hFont);

		GetTextMetrics( pFontInfo->hDC, &tm );
	//font.lfHeight = font.lfHeight-tm.tmExternalLeading;//-(int)(fabs(fontSize)/10.0+0.5);//-MulDiv(fontSize, GetDeviceCaps(pFontInfo->hDC, LOGPIXELSY), 72);

	//pFontInfo->hPrevFont = SelectObject(pFontInfo->hDC, pFontInfo->hFont);

	return pFontInfo;
} /* VisualRendererDD::SetFont */


// =====================================================================

//
// VisualRendererDD::UnsetFont
//
// Stop using the font which was set using SetFont().  Every call
// to SetFont() must be accompanied by a call to UnsetFont().
//


void VisualRendererDD::UnsetFont(void* pFont)
{
	assert(pFont != NULL);
	FontInfo* pFontInfo = (FontInfo*)pFont;

	// Unselect the font (by putting back the previous one), delete it,
	// and then release the device context.
	SelectObject(pFontInfo->hDC, pFontInfo->hPrevFont);
	DeleteObject(pFontInfo->hFont);
	HRESULT returnValue;
	m_pDirectDraw->ReleaseHDC(pFontInfo->hDC,returnValue);

	delete pFont;
} /* VisualRendererDD::UnsetFont */


// =====================================================================

//
// VisualRendererDD::GetTextSize
//
// Return the width and the height of the given text string, using
// a font return by SetFont().
//

void VisualRendererDD::GetTextSize(void* pFont, LPCTSTR string,
								   int length, unsigned int& width,
								   unsigned int& height,unsigned int &ascent,
								   unsigned int &descent)
{
	assert(pFont != NULL);
	FontInfo* pFontInfo = (FontInfo*)pFont;

	
	SIZE stringSize;
	GetTextExtentPoint32(pFontInfo->hDC, string, length, &stringSize);
	width  = stringSize.cx;
	//height = stringSize.cy;
	
	TEXTMETRIC ptm;
	GetTextMetrics( pFontInfo->hDC,  &ptm);   
	ascent=ptm.tmAscent;
	descent=ptm.tmDescent;
	height=ptm.tmHeight;//+ptm.tmExternalLeading;

} /* VisualRendererDD::GetTextSize */

void VisualRendererDD::SetCharacterInterspace(void *  pFont,int space)
{
	assert(pFont != NULL);
	FontInfo* pFontInfo = (FontInfo*)pFont;
	
	SetTextCharacterExtra(pFontInfo->hDC,space);
//	::SetTextJustification(pFontInfo->hDC,width,nSpaces);
}

void VisualRendererDD::PixelToLogical(void *  pFont,int& size )
{
	assert(pFont != NULL);


	FontInfo* pFontInfo = (FontInfo*)pFont;
	
	
	POINT pt;
	pt.x=size;
	pt.y=0;
	DPtoLP(pFontInfo->hDC,&pt,1);
	size=pt.x;
	/*
	SIZE sizeWinExt;
	int value1=GetWindowExtEx(pFontInfo->hDC,&sizeWinExt);
	SIZE sizeVpExt;
	int value=GetViewportExtEx(pFontInfo->hDC,&sizeVpExt);
	size = MulDiv(size, abs(sizeWinExt.cx), abs(sizeVpExt.cx));*/
	
	//::SetTextAlign(pFontInfo->hDC,formatflag);
}



// =====================================================================

//
// VisualRendererDD::DisplayText
//
// Display a string with the given length, at the specifed location
// and using the given colour, clipping the output to the given
// rectangle.  The x,y point and the clip rectangle are given in
// BIFS coordinates.
//
// Only one line of text will be given at a time.


void VisualRendererDD::DisplayText(void* pFont, LPCTSTR string,
								   int length, int x, int y,
								   Color3f const& color,
								   BRect const* pClipBRect)
{
	assert(pFont != NULL);
	FontInfo* pFontInfo = (FontInfo*)pFont;

	// Set the colour of the text string
	SetTextColor(pFontInfo->hDC,
				 RGB(color.r * 255, color.g * 255, color.b * 255));

	// There will be no background colour
	SetBkMode(pFontInfo->hDC, TRANSPARENT);

	// And set the text alignment within the clipping rectangle
	::SetTextAlign(pFontInfo->hDC, TA_TOP | TA_LEFT);

	// Set up the clipping rectangle (& convert BIFS to screen coords)
	RECT clipRect;

	
		#ifdef ORIGIN_CENTER
			clipRect.left   = pClipBRect->left + m_outputWidth / 2;
			clipRect.top    = m_outputHeight / 2 - pClipBRect->top;
			clipRect.right  = clipRect.left + pClipBRect->width;
			clipRect.bottom = clipRect.top + pClipBRect->height;
		#else
			clipRect.left   = x;
			clipRect.top    = m_outputHeight  - y;
		#endif


	// set up clipping region 
	CRgn ClippingRegion;
	ClippingRegion.CreateRectRgnIndirect(&clipRect);
	SelectClipRgn (pFontInfo->hDC,ClippingRegion);

	// render text string
	TextOut(pFontInfo->hDC,x + m_outputWidth / 2,m_outputHeight / 2 - y,string,length);
	
	// restore clipping region 

	SelectClipRgn (pFontInfo->hDC,NULL);
	
} /* VisualRendererDD::DisplayText */






// ========================================================================

//
// VisualRendererDD::DisplayPoint
//
// Set the specified pixel to the given colour, using the provided level
// of transparency.  The location of the pixel to set is given in BIFS
// coordinates, with the origin of the output display is in the middle
// of the window, with increasing X and Y to the right and up.
//
void VisualRendererDD::DisplayPoint(int xPos, int yPos, Color3f const* pPixelColor,
									float transparency)
{
	// If the transparency is high enough so that the point is see through, return
	if (transparency > (float)254 / (float)255)
		return;

	// Convert the pixel location from BIFS to screen coordinates
	
	#ifdef ORIGIN_CENTER
	yPos = m_outputHeight / 2 - yPos;
	xPos += m_outputWidth / 2;
	#else
    yPos = m_outputHeight  - yPos;
	#endif	

	// Lock the surface for writing
	DDSURFACEDESC ddSurfaceDesc;
	HRESULT returnValue;
	BOOL bIsAllFine;
	bIsAllFine = m_pDirectDraw->LockSurface(ddSurfaceDesc,RGB,returnValue);
	if(!bIsAllFine)
		PostError(returnValue);

	SurfaceDesc surfaceDesc;
	surfaceDesc.dwWidth   = ddSurfaceDesc.dwWidth;
	surfaceDesc.dwHeight  = ddSurfaceDesc.dwHeight;
	surfaceDesc.lPitch    = ddSurfaceDesc.lPitch;
	surfaceDesc.lpSurface = ddSurfaceDesc.lpSurface;

	// Determine the new pixel colour
	int red   = pPixelColor->r * 255 + 0.5;
	int green = pPixelColor->g * 255 + 0.5;
	int blue  = pPixelColor->b * 255 + 0.5;

	// Copy the image (or a portion of it)
	switch (m_pDirectDraw->GetRGBpixelFormat())
	{
	case RGB555:
		{
			// Get a pointer to the pixel to change
			unsigned short* pDest = (unsigned short*)((unsigned char*)((unsigned char*)surfaceDesc.lpSurface + yPos * surfaceDesc.lPitch + 2 * xPos));

			if (transparency > ALMOST_ZERO)
			{
				// Get the RGB values underneath the pixel
				int oldRed   = (*pDest & 0x7c00) >> 7;
				int oldGreen = (*pDest & 0x03e0) >> 2;
				int oldBlue  = (*pDest & 0x001f) << 3;

				// Apply the transparency
				ApplyTransparencyToRGB(red, green, blue, transparency, oldRed, oldGreen, oldBlue);
			}

			// Combine the values, 5 bits red, 5 bits green and 5 bits blue
			*pDest++ = MAKE_RGB555_PIXEL(red, green, blue);
		}
		break;

	case RGB565:
		{
			// Get a pointer to the pixel to change
			unsigned short* pDest = (unsigned short*)((unsigned char*)((unsigned char*)surfaceDesc.lpSurface + yPos * surfaceDesc.lPitch + 2 * xPos));

			if (transparency > ALMOST_ZERO)
			{
				// Get the RGB values underneath the pixel
				int oldRed   = (*pDest & 0xf800) >> 8;
				int oldGreen = (*pDest & 0x07e0) >> 3;
				int oldBlue  = (*pDest & 0x001f) << 3;

				// Apply the transparency
				ApplyTransparencyToRGB(red, green, blue, transparency, oldRed, oldGreen, oldBlue);
			}

			// Combine the values, 5 bits red, 6 bits green and 5 bits blue
			*pDest++ = MAKE_RGB565_PIXEL(red, green, blue);
		}
		break;

	case RGB24bpp:
		{
			// Get a pointer to the pixel to change
			unsigned char* pDest = (unsigned char*)surfaceDesc.lpSurface
									+ yPos * surfaceDesc.lPitch	+ 3 * xPos;

			if (transparency > ALMOST_ZERO)
			{
				// Get the RGB values underneath the pixel
				int oldRed   = pDest[2];
				int oldGreen = pDest[1];
				int oldBlue  = pDest[0];

				// Apply the transparency
				ApplyTransparencyToRGB(red, green, blue, transparency, oldRed, oldGreen, oldBlue);
			}

			// Store the 8 bit values for blue, green and red
			*pDest++ = (unsigned char) blue;
			*pDest++ = (unsigned char) green;
			*pDest++ = (unsigned char) red;
		}
		break;

	case RGB32bpp:
		{
			// Get a pointer to the pixel to change
			unsigned long* pDest = (unsigned long*)((unsigned char*)((unsigned char*)surfaceDesc.lpSurface + yPos * surfaceDesc.lPitch
									+ 4 * xPos));

			if (transparency > ALMOST_ZERO)
			{
				// Get the RGB values underneath the pixel
				int oldRed   = (*pDest & 0xff0000) >> 16;
				int oldGreen = (*pDest & 0xff00) >> 8;
				int oldBlue  = (*pDest & 0xff);

				// Apply the transparency
				ApplyTransparencyToRGB(red, green, blue, transparency, oldRed, oldGreen, oldBlue);
			}

			// Combine the values, 8 bits red, 8 bits green and 8 bits blue
			*pDest++ = MAKE_RGB32bpp_PIXEL(red, green, blue);
		}
		break;

	case BGR32bpp:
		{
			// Get a pointer to the pixel to change
			unsigned long* pDest = (unsigned long*)((unsigned char*)((unsigned char*)surfaceDesc.lpSurface + yPos * surfaceDesc.lPitch
									+ 4 * xPos));

			if (transparency > ALMOST_ZERO)
			{
				// Get the RGB values underneath the pixel
				int oldRed   = (*pDest & 0xff);
				int oldGreen = (*pDest & 0xff00) >> 8;
				int oldBlue  = (*pDest & 0xff0000) >> 16;

				// Apply the transparency
				ApplyTransparencyToRGB(red, green, blue, transparency, oldRed, oldGreen, oldBlue);
			}

			// Combine the values, 8 bits red, 8 bits green and 8 bits blue
			*pDest++ = MAKE_BGR32bpp_PIXEL(red, green, blue);
		}
		break;

	default:
		assert(0);		// Shouldn't get here.
		break;
	}

	// Finished writing to the surface, so unlock it
	m_pDirectDraw->UnlockSurface(RGB,returnValue);
} /* VisualRendererDD::DisplayPoint */


// =====================================================================

 void VisualRendererDD::SetScreenSize(unsigned int newX,
							   unsigned int newY)

 {
	m_outputWidth=newX;
 
	m_outputHeight=newY;
	
	HRESULT returnValue;
	BOOL bIsAllFine;
	bIsAllFine=m_pDirectDraw->Resize(newX,newY,returnValue);
	if(!bIsAllFine)
		PostError(returnValue);
 
 }
 void VisualRendererDD::PostError(HRESULT returnValue) 
{
		
	if (!m_bErrorDetected)
	{
		char *errorMessage=new char [200];
		strcpy (errorMessage, _T("Rendering Library DirectX:"));
		strcat (errorMessage, HRESULTtoText(returnValue));
		#ifndef NETSCAPE_PLUGIN
		::PostMessage(m_hWnd,WM_DDRAW_PROBLEM, NULL,(LPARAM)(LPCSTR )errorMessage);
		#else 
		::PostMessage(((NetscapeApp *) GetPresenter()->GetExecutive()->GetApp())->GetWindow(),
						WM_DDRAW_PROBLEM,
						NULL,
						(LPARAM)(LPCSTR )errorMessage);
		#endif
		m_bErrorDetected=TRUE;
	}
}



 void VisualRendererDD::DisplayLogo()
{
	HDC hdc;
	CBitmap OurLogo;
	BITMAP              bm;
	HRESULT returnValue;
	
	HDC                 hdcImage;
	OurLogo.LoadBitmap(IDB_BITMAP1);
    hdcImage = CreateCompatibleDC(NULL);
	if(!m_pDirectDraw->GetHDC(hdc,returnValue))  //do nothing if a dc can't be taken
		return;                
	//HRGN hrgn = CreateRectRgn(0,0,640,480); 
    //SelectClipRgn(hdc, hrgn); 
	
	SelectObject(hdcImage,(OurLogo));
	
	////
	//
    // get size of the bitmap
    //
    GetObject(OurLogo, sizeof(bm), &bm);    // get size of bitmap
    /*dx = dx == 0 ? bm.bmWidth  : dx;    // use the passed size, unless zero
    dy = dy == 0 ? bm.bmHeight : dy;*/

    //
    // get size of surface.
    //
    /*ddsd.dwSize = sizeof(ddsd);
    ddsd.dwFlags = DDSD_HEIGHT | DDSD_WIDTH;
    pdds->GetSurfaceDesc(&ddsd);*/

    
	StretchBlt(hdc, 0, 0, m_outputWidth, m_outputHeight, hdcImage, 0, 0, bm.bmWidth, bm.bmHeight, SRCCOPY);
	

    DeleteDC(hdcImage);
	///
	m_pDirectDraw->ReleaseHDC(hdc,returnValue);
		
	OurLogo.DeleteObject();
 }