//This software module was originally developed by CSELT in the course of   //
//development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.             //
//This software module is an implementation of a part of one or more MPEG-4 //
//Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems        //
//(ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 Systems     //
//(ISO/IEC 14496-1) free license to this software module or modifications   //
//thereof for use in hardware or software products claiming conformance to  //
//the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use this software//
//module in hardware or software products are advised that its use may      //
//infringe existing patents.                                                //
//The original developer of this software module and his/her company, the   //
//subsequent editors and their companies, and ISO/IEC have no liability     //
//for use of this software module or modifications thereof in an            //
//implementation. Copyright is not released for non MPEG-4 Systems          //
//(ISO/IEC 14496-1) conforming products. CSELT S.pA. retains full right     //
//to use the code for his/her own purpose, assign or donate the code to a   //
//third party and to inhibit third parties from using the code for non      //
//MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright      //
//notice must be included in all copies or derivative works.                //
//Copyright  1999.                                                         //
//This work has been partly developed within the framework of the ESPRIT    //
//European Research Project  "MPEG-4 PC".                                   //
//////////////////////////////////////////////////////////////////////////////
//
// File: VisRend-CSELT-DD.h
//
// Author: Matthew Leditschke 
//
// Modified: Gianluca Di Cagno  gianluca.dicagno@cselt.it
//
// Description:
//   The definition of a class which performs visual rendering using
//   DirectDraw under Windows 95/NT.
//
// Created: August, 1997
//
//
///////////////////////////////////////////////////////////////////////////


#ifndef VIS_REND_CSELT_DD_H
#define VIS_REND_CSELT_DD_H


#include "VisRend.h"
#include "DirectDraw.h"


class VisualRendererDD : public VisualRenderer
{
	//
	// Public functions
	//
public:


	// Class constructor - given a pointer to the presenter and a handle
	// to the window to display in.
	VisualRendererDD(Presenter* pPresenter, HWND hWnd);

	// Class destructor.
	virtual ~VisualRendererDD();

	// Methods inherited from the VisualRenderer base class.
	virtual void Init();
	virtual void Terminate();
	virtual void Paint(void* pInfo = NULL);
	virtual void Clear(BRect const* pUpdateRect = NULL);
	virtual void GetCapabilities (Capabilities* pPresentationCaps);
	virtual void SetScreenSize(unsigned int newX,
							   unsigned int newY); 
	virtual void DisplayRGBAImage(SrcRGBAImage const& srcImage);
	virtual void DisplayYUV420AImage(SrcYUVAImage const& srcImage);
	virtual void DisplayCircle(BRect const& pClipBRect, PointF const* lpPoints,
							   Color3f color, int width, 
							   BOOL filled);
    virtual void DisplayCurve(BRect const& pClipBRect, 
							  PointF const* lpPoints,
							  int const* lpTypes,
							  Color3f color, 
							  int width, 
							  int length);
    virtual void DisplayLines(BRect const& pClipBRect, 
							  int const& startPos1, int const& startPos2,
							  int const& endPos1, int const& endPos2,
							  Color3f const& pixelColor, 
							  int width);
	virtual void DisplayPolygon(BRect const& clipBRect,
								PointF const* lpPoints, int numPoints,
								Color3f const& pixelColor,
								int width, BOOL filled);
	virtual void DisplayRectangle(BRect const& clipBRect,
								  BRect const& drawBRect,
								  Color3f const& pixelColor,
								  int width, BOOL filled,
								  float transparency);
	virtual void DisplayPoint(int xPos, int yPos, Color3f const* pPixelColor,
							  float transparency);
	virtual void* SetFont(LPCTSTR fontName, int fontSize, BOOL bold,
						  BOOL italics, BOOL underline);
	virtual void UnsetFont(void* pFont);
	virtual void GetTextSize(void* pFont, LPCTSTR string, int length,
							 unsigned int& width, unsigned int& height,unsigned int &ascent,unsigned int &descent);
	virtual void SetCharacterInterspace(void *  pFont,int space);
	virtual void PixelToLogical(void *  pFont,int& size );
	virtual void DisplayText(void* pFont, LPCTSTR pString, int length,
							 int x, int y, Color3f const& color,
							 BRect const* pClipBRect);

	virtual void GetScreenSize(unsigned int& currX,
							   unsigned int& currY) const
	{ currX = m_outputWidth; currY = m_outputHeight; }

	virtual unsigned int GetScreenWidth() const
	{ return m_outputWidth; }

	virtual unsigned int GetScreenHeight() const
	{ return m_outputHeight; }



	void PostError(HRESULT);
	
	void DisplayLogo();
	//
	// Private implementation details
	//
private:

	// Remember the handle to the window in which the drawing takes place
	HWND m_hWnd;

	// The DirectDraw object which accesses the DirectDraw libraries
	DirectDraw* m_pDirectDraw;

	// The dimensions of the output window
	unsigned int m_outputWidth;
	unsigned int m_outputHeight;
	BOOL m_bErrorDetected;
};


#endif // VIS_REND_CSELT_DD_H
