//This software module was originally developed by CSELT in the course of   //
//development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.             //
//This software module is an implementation of a part of one or more MPEG-4 //
//Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems        //
//(ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 Systems     //
//(ISO/IEC 14496-1) free license to this software module or modifications   //
//thereof for use in hardware or software products claiming conformance to  //
//the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use this software//
//module in hardware or software products are advised that its use may      //
//infringe existing patents.                                                //
//The original developer of this software module and his/her company, the   //
//subsequent editors and their companies, and ISO/IEC have no liability     //
//for use of this software module or modifications thereof in an            //
//implementation. Copyright is not released for non MPEG-4 Systems          //
//(ISO/IEC 14496-1) conforming products. CSELT S.pA. retains full right     //
//to use the code for his/her own purpose, assign or donate the code to a   //
//third party and to inhibit third parties from using the code for non      //
//MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright      //
//notice must be included in all copies or derivative works.                //
//Copyright  1999.                                                         //
//This work has been partly developed within the framework of the ESPRIT    //
//European Research Project  "MPEG-4 PC".                                   //
//////////////////////////////////////////////////////////////////////////////
//
// File: VisRend-CSELT-X11.h
//
// Author: Matthew Leditschke 
//
// Modified:
//
// Description:
//   The definition of a class which performs visual rendering using
//   the X11 libraries on unix.
//
// Created: September, 1997
//
//
///////////////////////////////////////////////////////////////////////////


#ifndef VIS_REND_CSELT_X11_H
#define VIS_REND_CSELT_X11_H


#include "VisRend.h"
#include <X11/Xlib.h>
#include <X11/Xutil.h>


// The different types of RGB displays that are supported
enum RGBpixelType { RGB32bpp, BGR32bpp };


class VisualRendererX11 : public VisualRenderer
{
	//
	// Public functions
	//
public:

	// Class constructor - given a pointer to the presenter and the name
	// of the display to open.
	VisualRendererX11(Presenter* pPresenter, const char* displayName);

	// Class destructor - calls terminate if it hasn't already been called
	~VisualRendererX11();

	// Methods inherited from the VisualRenderer base class.
	virtual void Init();
	virtual void Terminate();
	virtual void Paint(void* pInfo = NULL);
	virtual void Clear(BRect const* pUpdateRect = NULL);
	virtual void GetCapabilities (Capabilities* pPresentationCaps);
	virtual void SetScreenSize(unsigned int newX, unsigned int newY);
	virtual void DisplayRGBAImage(SrcRGBAImage const& srcImage);
	virtual void DisplayYUV420AImage(SrcYUVAImage const& srcImage);

	virtual void GetScreenSize(unsigned int& currX, unsigned int& currY) const
	{ currX = m_outputWidth; currY = m_outputHeight; }

	virtual unsigned int GetScreenWidth() const
	{ return m_outputWidth; }

	virtual unsigned int GetScreenHeight() const
	{ return m_outputHeight; }

	// Return the current display
	Display* GetDisplay() const { return m_display; }


	//
	// Private implementation details
	//
private:

	// Remember the name of the display being used
	char const* m_displayName;

	// The display being used
	Display* m_display;

	// The window being used
	Window m_window;

	// The Visual being used
	Visual *m_visual;

	// The RGB pixel format of the visual
	RGBpixelType m_RGBpixelFormat;

	// The graphics context being used
	GC m_gc;

	// The color depth of the window
	int m_depth;

	// Remember a pointer to the presenter that this renderer is part of
	Presenter* m_pPresenter;

	// The dimensions of the output window
	unsigned int m_outputWidth;
	unsigned int m_outputHeight;

	// The XImage structure which is drawn into
	XImage* m_ximage;

	// The memory associated with this XImage
	unsigned int* m_imageData;


	//
	// Private functions
	//
	Visual* FindFullColorVisual();
	Window CreateFullColorWindow (int x, int y, int w, int h);

};


#endif // VIS_REND_CSELT_X11_H


// Emacs uses these...
// Local Variables:
// mode: c++
// tab-width: 4
// End:
