//
// The main frame application for the IM1-2D player using X-windows.
//


#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <assert.h>
#include <X11/Xlib.h>
#include "defs.h"
#include "App.h"
#include "ztl.h"
#include "Executiv.h"
#include "Presenter.h"
#include "VisRend-CSELT-X11.h"
#include "trace.h"

extern char* sys_errlist[];


// ============================================================================

#ifdef sgi
#include <time.h>

//
// Define a usleep function for the SGI (doesn't seem to have one).
//

extern "C" int usleep(unsigned int useconds)
{
  struct timespec delay;
  delay.tv_sec = useconds / 1000000;
  delay.tv_nsec = (useconds % 1000000) * 1000;

  TRACE_PRINT1("In usleep(%d):\n", useconds);
  return nanosleep(&delay, NULL);
} /* usleep */

#endif


// ========================================================================

//
// The definition of the application class
//

class UnixApplication : public Application
{
  Executive* m_pExecutive;
  char* m_displayName;
  char* m_fileName;
  FILE* m_infile;
  BOOL m_finished;

public:

  UnixApplication(int argc, char* argv[]);
  virtual ~UnixApplication();

  void Run();

  void Finish(int nReason = 0);
};


// ========================================================================

//
// UnixApplication::UnixApplication
//
// Create the application, after first scanning the command line.
//

UnixApplication::UnixApplication(int argc, char* argv[])
{
  // Set the default display name to use
  m_displayName = NULL;
  m_displayName = "bitti:0.0";

  // Read in the command line
  if (argc != 2)
    {
      fprintf (stderr, "Usage: im1-2d filename\n");
      exit (-1);
    }

  m_fileName = argv[1];

#ifdef WRITE_TRACE
//  traceFile = fopen ("trace.log", "w");
//  traceFile = freopen ("trace.log", "w", stdout);
//  assert(traceFile != NULL);
  traceFile = stdout;
  TRACE_PRINT("Starting IM1Player:\n\n");
#endif

} /* UnixApplication::UnixApplication */


// ========================================================================

//
// UnixApplication::~UnixApplication
//
// Exit the application and shutdown everything.
//

UnixApplication::~UnixApplication()
{
  delete m_pExecutive;
} /* UnixApplication::~UnixApplication */


// ========================================================================

#ifdef SIMULATE_BIFS_DECODER
extern char* globalFileName;
#endif

//
// UnixApplication::Run
//
// Start the application running on the given file.
//

void UnixApplication::Run()
{
  // Open the given file for reading
  m_infile = fopen(m_fileName, "r");
  if (m_infile == NULL)
    {
      fprintf(stderr, "im1-2d: Error opening '%s' for reading. %s\n",
	      m_fileName, sys_errlist[errno]);
      exit (-1);
    }

#ifdef SIMULATE_BIFS_DECODER
  if (globalFileName != NULL)
    free(globalFileName);

  // Store the name of the file in a global variable, used when
  // simulating the BIFS decoder, so that it knows which scene
  // to create.
  // Get the base file name to decide which type of scene to create
  const char* tmpPtr = strrchr(m_fileName, '/');
  if (tmpPtr == NULL)
    tmpPtr = m_fileName;
  else
    tmpPtr++;
  globalFileName = strdup(tmpPtr);
#endif

  // Create the executive
  m_pExecutive = new Executive(this, (HWND)m_displayName);

  // Start the executive running on this file
  m_pExecutive->Start(m_infile);

  // Loop handling X-windows events until the end is signalled
  m_finished = FALSE;
  Display* display = ((VisualRendererX11*)(((Presenter*)GetPresenter()->m_pExecutive)->GetPresenter()->GetVisualRenderer()))->GetDisplay();
  while (!m_finished)
    {
      XEvent xev;
	
      XNextEvent(display, &xev);
    }

  // Stop the executive running
  m_pExecutive->Stop();

  // Close down the file
  fclose(m_infile);
  m_infile = NULL;
} /* UnixApplication::Run */


// ========================================================================

//
// UnixApplication::Finish
//
// Called by the Executive when the end of the file has been reached.
//

void UnixApplication::Finish(int /*nReason*/)
{
  m_finished = TRUE;
} /* UnixApplication::Finish */


// ========================================================================

int main (int argc, char* argv[])
{
  UnixApplication myApp(argc, argv);
  myApp.Run();



  return 0;
}


// ========================================================================
