///////////////////////////////////////////////////////////////////////////
//                                                                       //
// Copyright (c) 1997 ISO/IEC All Rights Reserved                        //
//                                                                       //
// This software module was developed by                                 //
//                                                                       //
// CSELT (Centro Studi e Laboratori Telecomunicazioni S.p.A)             //
//                                                                       //
// in the course of development of the MPEG-4 standard. This software    //
// module is an implementation of a part of one or more MPEG-4 tools as  //
// specified by the MPEG-4 standard.                                     //
//                                                                       //
// The copyright of this software belongs to ISO/IEC. ISO/IEC gives use  //
// of the MPEG-4 standard free license to use this  software module or   //
// modifications thereof for hardware or software products claiming      //
// conformance to the MPEG-4 standard.                                   //
//                                                                       //
// Those intending to use this software module in hardware or software   //
// products are advised that use may infringe existing  patents. The     //
// original developers of this software module and their companies, the  //
// subsequent editors and their companies, and ISO/IEC have no liability //
// and ISO/IEC have no liability for use of this software module or      //
// modification thereof in an implementation.                            //
//                                                                       //
///////////////////////////////////////////////////////////////////////////
//
// File: Defines.h
//
// Authors: Nuno Delgado, CSELT
//
// Description:
//   File with important constants and definitions relating to the DMIF
//   interface.
//
///////////////////////////////////////////////////////////////////////////


#ifndef _DEFINES_H
#define _DEFINES_H


#define DLLExport			__declspec(dllexport)



#define OK					1
#define ERROR				0



// Size of the buffer that receives the data from the network
#define DATA_BUFFER_SIZE			65536

// Size of Control Messages Exchanged
#define CONTROL_MSG_SIZE			256


// Messages exchanged over the DMIF control channel
#define CHANNEL_ADD_REQUEST			60
#define CHANNEL_ADD_RESPONSE		61
#define CHANNEL_READY_REQUEST		62
#define LIST_OF_CHANNELS			63
#define FMC_DESCRIPTOR				64
#define SESSION_DETACH_REQUEST		65
#define	UDP_PORT_CONFIRMATION		66
#define SESSION_DESTROY_REQUEST		67



// Type of channel
#define TCP_CHANNEL					1
#define UDP_CHANNEL					2



// Side of the connection
#define SERVER_SIDE					1
#define CLIENT_SIDE					2


// Constant that defines the max QoS value for TCP channels
#define MAX_QOS_VALUE_FOR_TCP_CHANNEL	5



/****************************************************
	
Following are the messages exchanged with the
DmifDaemon

****************************************************/


// Size of the messages
#define MSG_LEN					128


// Version of Winsock that we need. At least version 2.0
#define VERSION_MAJOR			2
#define VERSION_MINOR			0


// User defined messages
#define NETWORK_EVENT			(WM_USER + 10)


// Ports defined for the UDP and TCP  daemons
#define WELL_KNOWN_TCP_PORT		22000
#define WELL_KNOWN_UDP_PORT		22000


// Base port number for defined services
#define BASE_PORT_NUMBER_FOR_SERVICES	22100


// Base port number that is given to each client of the NetDaemon to use in their own
// application. This value is incremented in steps of 25 units by the NetDaemon each time
// it gives a range of ports to an application inside a peer

#define BASE_PORT_NUMBER		25000


// Messages exchanged over the Udp socket
#define REQUEST_CONNECT_TO_PEER		10
#define NEW_SERVICE_REGISTRATION	11
#define DMIF_PORT_NUMBER			12
#define UNREGIST_SERVICE			13
#define MSG_ERROR					20
#define MSG_OK						21

/* Descritpion of the messages exchanged over the UDP socket.
EVERY message begins with a 4 byte field containing the type of the message, i.e.,
or REQUEST_CONNECT_TO_PEER or NEW_SERVICE_REGISTRATION, etc. This field has 4 bytes.

  The REQUEST_CONNECT_TO_PEER msg is like this:
  TypeMsg (4 Bytes) + ServiceId (4 Bytes) + PeerAddr (string with variable length)

  The NEW_SERVICE_REGISTRATION msg is like this:
  TypeMsg (4 Bytes) + ServiceId (4 Bytes)

  The DMIF_PORT_NUMBER msg is like this:
  TypeMsg (4 Bytes) + PortNumber (2 bytes)

*/




#endif