//This software module was originally developed by CSELT in the course of   //
//development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.             //
//This software module is an implementation of a part of one or more MPEG-4 //
//Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems        //
//(ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 Systems     //
//(ISO/IEC 14496-1) free license to this software module or modifications   //
//thereof for use in hardware or software products claiming conformance to  //
//the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use this software//
//module in hardware or software products are advised that its use may      //
//infringe existing patents.                                                //
//The original developer of this software module and his/her company, the   //
//subsequent editors and their companies, and ISO/IEC have no liability     //
//for use of this software module or modifications thereof in an            //
//implementation. Copyright is not released for non MPEG-4 Systems          //
//(ISO/IEC 14496-1) conforming products. CSELT S.pA. retains full right     //
//to use the code for his/her own purpose, assign or donate the code to a   //
//third party and to inhibit third parties from using the code for non      //
//MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright      //
//notice must be included in all copies or derivative works.                //
//Copyright  1999.                                                         //
//This work has been partly developed within the framework of the ESPRIT    //
//European Research Project  "MPEG-4 PC".                                   //
//////////////////////////////////////////////////////////////////////////////
//
// File:  DrawableNode.h
//
// Author: Matthew Leditschke 
//
// Modified: Gianluca Di Cagno  gianluca.dicagno@cselt.it
//
// Description:
//   The definition of a class which encapsulates all of the features
//   of a node which can be drawn by the visual renderer.  I would have
//   liked to call this class Drawable, but X11 uses this name already.
//
// Created: August, 1997.
//
//
///////////////////////////////////////////////////////////////////////////


#ifndef DRAWABLE_NODE_H
#define DRAWABLE_NODE_H




#include "GroupNodeProxy.h"

#include "BRect.h"


class DrawableNode : public Traversable
{
	//
	// Public functions
	//
public:

	// The bounding rectangle used by the drawable node for this render

    BRect m_boundingRect;

	BOOL m_isBeingTracked;


	DrawableNode(MediaObject* pMO);
	// Class constructor
	
	~DrawableNode();
	


	virtual BOOL NeedsDrawing(DrawableContext * pContext)  =0;
	// Returns TRUE if this node needs to be drawn (because it has
	// changed since it was last drawn).

	virtual BOOL IsTransparent() const { return m_isTransparent; }
	// Return TRUE if this node is transparent.  ie. it does not
	// completely fill its bounding rectangle, either because not
	// every pixel is written and/or and alpha blending component
	// is used.  The default implementation returns TRUE if the
	// previous and current bounding boxes are different.  A derived
	// class which implements this method, must include the result
	// of this method from this class.

	virtual void Draw(BRect const* pClipBRect,DrawableContext * pDrawableContext) =0;
	// Draw the part of the node which is enclosed inside the given
	// bounding rectangle, or draw the entire object if no bounding
	// rectangle is given.
	//
	// The bounding rectange uses the BIFS coordinate system, with the
	// origin of the output display is in the middle of the window, with
	// increasing X and Y to the right and up.
/*
	virtual BOOL IsPointOver(int x, int y) { return GetBoundingRect()->IsPointIn(x, y); }
	// Return TRUE if the given point, in BIFS coordinates, is above the
	// current shape.  Return FALSE otherwise.  The default implementation
	// uses the bounding rectangle of the drawable.*/

	void GetPreviousBRectToRefresh(BRect *pRectToBeRefreshed);
	// Call this function to get the rect that needs to be refreshed, if any.
	// This is called to delete the previous rects of a node when a node changes
	// position.
	
	BOOL LookInPreviousBBoxList(BRect *);
	// Call this function to look in the previous Bounding Box list of a drawable
	// node if the argument BBox is found. If so, remove that BB from the previous
	// list.
	// All the BBox that will not be removed, will be cleaned during the rendering 
	// process
	virtual void ResetVisitsList(){m_nVisits=0;};

	void ResetCurrentBBoxList();

	virtual void UpdateBoundingRect(DrawableContext * pContext);
	// Call this function from the PreRender() method of a derived
	// class, once the m_boundingRect data member has been set.
	// This will combine the current bounding rectangle with the
	// previous.

    virtual  MediaObject * GetDrawableParent() const =0;
    //This returns a pointer to the Group or transform father.

    BOOL IsLeftTopObject() const {return m_bIsALeftTopImage;}
    
    // these two functions are useful when a drawable is deleted in the 
	// process events function.

	void SetDrawableOrder (int order) { m_orderInDrawableList=order;}
    
	int GetDrawableOrder () {return m_orderInDrawableList;}
	
	const int GetIdMediaObject() const { return ((MediaObject*) GetObject())-> GetID ();}
	//NOT USED
	// returns the id of the media object stored in the traversable mediaobject.
    

	//
	// Protected implementation details
	//
protected:
	

	
	
	// The bounding rectangle used by the drawable node for this render
	//	BRect m_boundingRect;

	DrawableContext * InitPreRender(Effects const& effects);
	// The method will store a copy of all of the effects parameters
	// locally in this node, and then register this node with the visual
	// renderer so that it is drawn.  If this node has been switched off,
	// FALSE is returned, in which case the calling function must return
	// immediately.

	

  
	//
	// Private implementation details
	//
private:
	BOOL m_isTransparent;
	static int m_numDrawablesCreated;
	int m_currentBRListSize ;
	BRect **m_previousBRList;
	BRect **m_currentBRList;
	
	
	// Order of this drawable in the drawable list after the qsort in drawScene.

    int m_orderInDrawableList;


	int m_nCurrentBBox;
	int m_nPreviousBBox;
	
protected:
	// This is need because Video and Text are drawn giving the left top point,
	// Geometric giving the center.
	
	BOOL m_bIsALeftTopImage;
	int m_nVisits;
};


#endif // DRAWABLE_NODE_H
