/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1997.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Mikael Bourges-Sevenier at mikael.bourgessevenier@cnet.francetelecom.fr.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Animation
FILE:       Anim.cpp
AUTHOR:     Mikael Bourges-Sevenier (France Telecom CNET/DIH/ATI)
OVERVIEW:   Implementation of my AnimationStream implementation
-------------------------------------------------------------------------*/

#include "defs.h"

#include "Anim.h"
#include "ODP.h"
#include "Executiv.h"
#include "AnimStreamDec.h"
#include "Presenter.h"

BOOL AnimationStreamImp :: Render (PresenterContext *pContext)
{
	/*-------------------------------------------------------------
	startTime / stopTime fields need to be offset by current time on first access
	-------------------------------------------------------------*/
	startTime.ApplyOffset (pContext);
	stopTime.ApplyOffset (pContext);

	/*-------------------------------------------------------------
	Code below is written so that startTime and stopTime are always accessed.
	This is required so that they get offset if necessary (the = operator on these
	fields ensures that)
	-------------------------------------------------------------*/
	double currentTime = pContext -> GetTime () / 1000.;
	bool bActive =
		 (startTime >= stopTime || currentTime < stopTime) && currentTime >= startTime;

  if (!m_bStart && bActive) {	
    m_bStart = TRUE;
    OpenStream ();
  }

  if (m_bStart && !bActive) {	
    CloseStream ();
    m_bStart = FALSE;
  }
  return TRUE;
}

/*-------------------------------------------------------------
Create a Animation decoder
ZL: This function is not in place. This files contains simulation of
rendering which shall not be part of the real players. MAnimationStream,
however, is part of every player.
-------------------------------------------------------------*/
Decoder *AnimationStreamImp :: CreateDecoder (
	DecoderConfigDescriptor *pDecoderConfig,
	QoS_Descriptor *,
	Decoder *
)
{
	//-------------------------------------------------------------
	//  ZL: need to check here that streamType is Animation
	//-------------------------------------------------------------
	if (pDecoderConfig -> streamType != ST_SceneDescriptionStream)
	{	GetExecutive () -> Error (INVALID_STREAM_TYPE);
		return NULL;
	}
	m_pDecoder = new AnimationStreamDecoder;
	m_pDecoder->SetConfig(pDecoderConfig);

	((Decoder *)m_pDecoder)->SetStream (NULL, GetExecutive (), GetRoot()  );
	return m_pDecoder;
}

BOOL AnimationStreamImp :: HasStreamFinished()
{
	if (!m_pDecoder) return FALSE;
	return m_pDecoder->m_bStreamFinished;
}
