/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

  SUBSYSTEM:  Decoder
  FILE:       AnimationStreamDec.cpp
  AUTHOR:     Mikael Bourges-Sevenier (France Telecom CNET/DIH/ATI)
  OVERVIEW:   Implementation of AnimationStreamDecoder
-------------------------------------------------------------------------*/
#include "defs.h"


#include "AnimStreamDec.h"

#include <assert.h>
//#include "nodes.h"
#include "Nodes.h" // SOLARIS PORTING - typo

#include "Stream.h"
#include "ODImp.h"
#include "Executiv.h"
#include "Except.h"


#ifdef ANIMATION

#define ANIM_START_CODE 0x00000018c
#define ANIM_END_CODE 0x00000018d
#define DEC_ERROR -1

int LOGLEVEL =0;

#endif


AnimationStreamDecoder :: AnimationStreamDecoder ()
{
#ifdef ANIMATION
    numNodes = 0;
	m_logfile = NULL;
	m_nbLogBits = 0;
	m_newAqp = true;

#endif
	m_bStreamFinished = FALSE;
	m_pBIFSComDec = NULL;
}

AnimationStreamDecoder :: ~AnimationStreamDecoder ()
{
#ifdef ANIMATION
	if (m_logfile) fclose(m_logfile);
#endif

	if (m_pBIFSComDec) delete m_pBIFSComDec;
}


/*-----------------------------------------------------------------
Setup decoder (parse BIFS Animation mask)
-----------------------------------------------------------------*/
BOOL AnimationStreamDecoder :: Setup (const BYTE *pSpecificInfo, int nSpecificInfoLength, int/* nESNumber*/ )
{
	if (!pSpecificInfo || nSpecificInfoLength <= 0 ) return FALSE;

	//reset current BIFS_Command config
	if (m_pBIFSComDec) delete m_pBIFSComDec;
	m_pBIFSComDec = NULL;

#ifdef ANIMATION
	//reset current BIFS_Anim config
	pNodeMask.Clear();
	nodeMask.Clear();
	numNodes = 0;
	frameID = 0;
#endif

	// parses specificInfo buffer
	setInputBuffer((unsigned char *) pSpecificInfo);
	setBufSize(nSpecificInfoLength);
	
	nodeIDbits=readBits(5);	
	routeIDbits=readBits(5);

#ifdef ANIMATION
	isCommandStream = readBit("isCommandStream");
#else
	isCommandStream = readBit();
#endif

	if(isCommandStream) {
		int oti = 0;
		if (m_pDecoderConfig) {
			oti = m_pDecoderConfig -> objectTypeIndication == 0xff ? 0 : m_pDecoderConfig -> objectTypeIndication;
		}
		m_pBIFSComDec = new BIFSDecoder(GetExecutive (), 
							GetRoot()->GetInline(), 
							GetExecutive () -> GetTraceFile (),
							"BIFS",
							GetExecutive () -> GetTraceFlags (),
							oti, 
							false
							);

		m_pBIFSComDec->Configure(pSpecificInfo, nSpecificInfoLength);
		return TRUE;
	} 

#ifdef ANIMATION

	if (!m_logfile) {
		// the name of the logfile will be dec_%fluxNumber%.log
		static int fluxNumber =0;
		char buffer[16];
		sprintf(buffer,"dec_%d.log",fluxNumber++); 
		m_logfile = fopen(buffer,"w");
	}
	randomAccess=readBit("randomAccess");

	GetExecutive () -> LockScene ();
	//FTRD - begin - 11/10/2001 
	bool hasNode = false;
	//FTRD - end - 11/10/2001 
	do {
		int nodeID=readBits(nodeIDbits,"nodeID");
		//FTRD - begin - 7/10/2001 
		char buffer[16];
		sprintf(buffer,"%d",nodeID);
		log(nodeID,nodeIDbits,"nodeID",buffer);
		//FTRD - begin - 7/10/2001 

		//fprintf(stderr,"[Mask] nodeID: %d\n",nodeID);
		MediaObject *node=GetRoot()->FindNode(nodeID);
		pNodeMask += node;
		nodeMask += TRUE;
		numNodes++;
		
		if (node) {
#ifdef TRACE_ENCODING
			ZString nodeName=node->GetName();
			//fprintf(stderr,"[Mask] name %s\n",(const char *) nodeName);
#endif
			// calls the node's parseAnimationMask method
			node->parseAnimationMask(this);

			//FTRD - begin - 11/10/2001 
			hasNode = true;
			//FTRD - end - 11/10/2001 
		}
	}

	//FTRD - begin - 7/10/2001 
	while(readBit("moreMasks"));
	//FTRD - begin - 7/10/2001 	
	
	GetExecutive () -> UnlockScene ();
	
	m_nbLogBits = 0;
	log("# BIFSAnim Track BIFSConfig");

	//FTRD - begin - 11/10/2001 
	return hasNode;
	//FTRD - end - 11/10/2001 


#else
	return FALSE;
#endif //ANIMATION

}

/*-----------------------------------------------------------------
Find a compatible format and initialize the decoder
-----------------------------------------------------------------*/
BOOL AnimationStreamDecoder :: SetFormat ( Capabilities &, DecoderParams &decoderParams )
{
	decoderParams.m_nStreamNumber = 0;
	decoderParams.m_nOptimalAUSize = 256;
	decoderParams.m_nDBSize = 256;
	decoderParams.m_nCBSize = 0;
	return TRUE;
}

void AnimationStreamDecoder :: Run ()
{
	LPBYTE pInData;
	int nInSize;
	DWORD dwTime;
	BOOL bRecurrent, bEOU;
	int nRecurrent = 0;

#ifdef ANIMATION	
	static int frameID;
#endif

	m_bStreamFinished = FALSE;

	while ((pInData =(LPBYTE) GetInputStream() -> Fetch (
						  nInSize,
						  dwTime,
						  bEOU,
						  //bRecurrent, // is it the same block as previously ?
						  nRecurrent, // is it the same block as previously ?
						  FALSE,
						  TRUE)
	  ) != END_OF_STREAM ) {

		if (IsActive() && !nRecurrent) {
			
			/*COMMAND STREAM*/
			if (isCommandStream) {
				while (GetExecutive()-> ReserveScene ()) {
					if (!IsActive ())
						break;
					GetExecutive()-> UnlockScene ();
					Sleep (3);
				}
				if (IsActive()) {
					try {
						((SysCoder *)m_pBIFSComDec)->ParseFrame(pInData, nInSize, dwTime);
					}
					catch (BifsException) {
					#ifdef PLAYER
						GetExecutive () -> Error (SCENE_PARSING_ERROR);
					#endif
					}
				}
				GetExecutive()->UnlockScene();

#ifdef ANIMATION	
				
			} else {
				/*ANIMATION STREAM*/

				//FTRD - begin - 7/10/2001 
				TRACEP(stderr,"\nAnimationFrame time %.3f\n",(float)dwTime/1000.0f);
				char buffer[64];
				static int m_numFrame =0;
				sprintf(buffer,"# BIFSAnim Frame %d",m_numFrame++);
				log(buffer);
				//FTRD - begin - 7/10/2001 

				// reads animation frames
				setInputBuffer(pInData);
				setBufSize(nInSize);
				
				// parses frame header
				
				//FTRD - begin - 7/10/2001 
				m_nbLogBits = 0;
				//FTRD - begin - 7/10/2001 
				
				readAFrame();
#endif
			
			}
		}
		GetInputStream() -> Release (TRUE);
		if (!IsActive()) break;
	}
	m_bStreamFinished = TRUE;
	
	//force notify EOS because restart of the stream may be set in the AnimationStream node
	GetExecutive()->InvalidateScene(NULL);

}


/*-----------------------------------------------------------------
Bit manipulation routines: we use a FIFO model for the bit buffer.
The floating-point parsing reads IEEE floats
-----------------------------------------------------------------*/

/*-----------------------------------------------------------------
Sets the input buffer
-----------------------------------------------------------------*/
void AnimationStreamDecoder::setInputBuffer(unsigned char *AUptr) {
  //fprintf(stderr,"setting input buffer to %ld\n",AUptr);
  startbuf=bufptr=AUptr;
  initBits();
}

/*-----------------------------------------------------------------
Initializes the bit counter and buffer
-----------------------------------------------------------------*/
void AnimationStreamDecoder::initBits() {
  incnt = 0;
  inused = 0;
  inbytecnt = 0;
}

/*-----------------------------------------------------------------
Reads n bits
-----------------------------------------------------------------*/
int AnimationStreamDecoder::readBits(int n) {
  int i;
  int ibuf = 0;

  if(n>32) {
    fprintf(stderr, "Cannot fetch more than 32 bits from readbits()\n");
    exit(-1);
  }

  for(i=0; i<n; i++) { /* MSB first */
    ibuf <<= 1;
    ibuf += readBit();
  }

  return ibuf;
}


/*-----------------------------------------------------------------
Reads a bit
-----------------------------------------------------------------*/
bool AnimationStreamDecoder::readBit() {
  int t;
 
  if(incnt==0) {
    if(bufptr-startbuf>=bufSize) 
      inbfr=0;
    else
      inbfr = *(bufptr++);
    incnt = 8;
    inbytecnt++;
  }
  t = (inbfr&0x80)>>7;
  inbfr <<= 1;
  incnt --;
  inused ++;
  return t != 0;
}


//Untill end of file: all BIFS-Anim specific
#ifdef ANIMATION


/*-----------------------------------------------------------------
Resets bit buffer
-----------------------------------------------------------------*/
void AnimationStreamDecoder::alignBits() {

  incnt = 0;
}


/*-----------------------------------------------------------------
This is used by the arithmetic decoder to find the byte boundary
after the decoding of a arithmetic coded stream.
-----------------------------------------------------------------*/
void AnimationStreamDecoder::adjustBits() {
  int bitin = 8 - incnt;
  
  if(bitin<=5) {
    //fseek(in_stream, -3L, SEEK_CUR);
    bufptr -= 3;
    incnt=0;
    readBits(bitin+2);
    }
  else {
    //fseek(in_stream, -2L, SEEK_CUR);
    bufptr -= 2;
    incnt=0;
    readBits(bitin-6);
  }
}

/*-----------------------------------------------------------------
Zero bit stuffing to next byte boundary
-----------------------------------------------------------------*/
void AnimationStreamDecoder::alignInputBits()
{
  if(incnt) {
    readBits(incnt); // incnt is reset in this call
  }
}

/*-----------------------------------------------------------------
Reads IEEE floating-point value
-----------------------------------------------------------------*/
float AnimationStreamDecoder :: readFloat ()
{	
#ifdef WIN32	
  unsigned char buf [4]="\0\0\0";
  for(int i=0;i<4;i++)
    buf[3-i]=readBits(8);
  
  return * (float *) buf;
#else
  return 0.0;
#endif
}
//FTRD - begin - 7/10/2001 
float AnimationStreamDecoder :: readGenericFloat (bool useEfficientCoding,int * bitcount)
{
  if(useEfficientCoding) return readEfficientFloat(bitcount);
  return readFloat();
}

float AnimationStreamDecoder :: readEfficientFloat (int * bitcount)
{	
	union
	{	
		float f;
		long l;
	} value;
	
	int mantissaLength=readBits(4, "mantissaLength");
	if (bitcount)
		(*bitcount)+=4;
	// PB efficient encoding fix according to 9.3.7.12.1 and 9.3.7.12.2
	if(mantissaLength) {
		int exponentLength=readBits(3, "exponentLength");
		if (bitcount)
			(*bitcount)+=3;
		int mantissaSign=readBits(1, "mantissaSign");
		if (bitcount)
			(*bitcount)+=1;
		int mantissa=readBits(mantissaLength-1, "mantissa");
		if (bitcount)
			(*bitcount)+=(mantissaLength-1);
		int exponentSign=0;
		int exponent=0;
		char exp;
		
		
		if(exponentLength) {
			exponentSign=readBits(1, "exponentSign");
			if (bitcount)
				(*bitcount)+=1;
			exponent=readBits(exponentLength-1, "exponent");
			if (bitcount)
				(*bitcount)+=(exponentLength-1);
			
			exp = (1-2*exponentSign)*((1<<(exponentLength-1))+exponent)+127;
			
			
			

		}
		else
			//FTRD - begin - 22/11/2001 
			exp = 127; // ML was 0
			//FTRD - begin - 22/11/2001 
		long val=mantissaSign<<31;
		val |= (exp & 0xff)<<23;
		val |= mantissa << 9; // PB was (23-mantissaLength) instead of 9
		value.l=val;
		
		return value.f;
		// PB end	
	}
	
	return 0.0;
}

//Bit manipulation routines
int AnimationStreamDecoder :: readBits(int n, char* szDesc, char* szComment,int logLevel)
{
	int val = readBits(n);
	if (m_logfile && szDesc)
		log(val,n,szDesc,szComment,logLevel);
	return val;
}

bool AnimationStreamDecoder :: readBit(char* szDesc, char* szComment,int logLevel)
{
	int val = readBit();
	if (m_logfile && szDesc)
		log(val,1,szDesc,szComment,logLevel);
	return val;
}

float AnimationStreamDecoder :: readFloat (char* szDesc, char* szComment,int logLevel)
{
	float val = readFloat();
	
	if (m_logfile && szDesc)
	{
		unsigned int f;
		memcpy(&f,&val,sizeof(unsigned int));
		log(f,32,szDesc,szComment,logLevel);
	}

	return val;
}
float AnimationStreamDecoder :: readGenericFloat (bool useEfficientCoding, char* szDesc, char* szComment,int logLevel)
{
	 if(useEfficientCoding) 
		 return readEfficientFloat();
	return readFloat(szDesc, szComment,logLevel);
}

void AnimationStreamDecoder :: logEncoded(int val, char* szDesc, char* szComment,int logLevel)
{
	extern int LOGLEVEL;
	if (logLevel > LOGLEVEL)
		return;
	if (m_logfile && szDesc)
		log(val,0,szDesc,szComment,logLevel);

}

void AnimationStreamDecoder :: log(int nValue, int nBits, char* szDesc, char* szComment,int logLevel)
{
	extern int LOGLEVEL;
	if (logLevel > LOGLEVEL)
		return;
	m_nbLogBits += nBits;
	fprintf (m_logfile,"BIFSAnim <>  (%3d,%10d)   %-40s%9X %s\n",
		nBits,
		m_nbLogBits,
		szDesc,
		nValue,
		szComment ? szComment : ""
		);
}

void AnimationStreamDecoder :: log(char* szComment,int logLevel)
{
	extern int LOGLEVEL;
	if (logLevel > LOGLEVEL)
		return;
	fprintf (m_logfile,"%s\n",szComment);
}


int AnimationStreamDecoder :: readAFrame()
{
  if(readFrameHeader()==DEC_ERROR) {
    fprintf(stderr,"Error in frame header\n");
    return DEC_ERROR;
  }
  readFrameData();
  return TRUE;
}

int AnimationStreamDecoder :: readFrameHeader()
{
  if(checkAnimStartCode()) 
  {
    //FTRD - begin - 7/10/2001 
	  isIntra=readBit("isIntra");
	//FTRD - begin - 7/10/2001 

    for (int i = 0; i < numNodes; i++) {
        //FTRD - begin - 7/10/2001 
		nodeMask[i] = readBit ("isActive");
		//FTRD - begin - 7/10/2001 
    }
    
    if(isIntra) {
	  //FTRD - begin - 7/10/2001 
      if(readBit("isFrameRate")) // is_frame_rate
	  	readFrameRate();
	  //FTRD - begin - 7/10/2001 

	  //FTRD - begin - 7/10/2001 
      if(readBit("isTimeCode")) //is_time_code
			readTimeCode();
	  //FTRD - begin - 7/10/2001 
    }
	//FTRD - begin - 7/10/2001 
    if(readBit("hasSkipFrames")) // is_skip_frames
      if(!readSkipFrames()) return DEC_ERROR;
	//FTRD - begin - 7/10/2001 
  }
  else 
    return DEC_ERROR;
  return 1; // OK
}

void AnimationStreamDecoder :: readFrameData()
{
	GetExecutive () -> LockScene ();
  //fprintf(stderr,"In readFrameData()\n");
  aad.setStream(this);
  
  if (!isIntra ) aad.reset(); // reset Adaptive Arithmetic Decoder 
                             //MBS: cause problem with FBA, 2D mesh
                             // is it an error in FDIS ?
  
		//FTRD - begin - 7/10/2001 
		  if (isIntra)
			  TRACEP(stderr,"\tINTRA \n");
		  else
			  TRACEP(stderr,"\tPRED \n");
		//FTRD - begin - 7/10/2001 


   for (int i=0; i<numNodes;i++) { // for all defined nodes in mask


	  if (nodeMask[i]) {// if node i is animated in this frame
		  
		  TRACEP(stderr,"\tNodeID (%d)\n",i);
		  
		  // calls the node's animate method
		  pNodeMask[i]->animate(this,isIntra); 
		  
		  
	  }
  }
  if (!(isIntra)) aad.adjustBits();
	GetExecutive () -> UnlockScene ();
}

bool AnimationStreamDecoder :: checkAnimStartCode()
{
  unsigned char *bufptrtmp=bufptr;
  
  int code=readBits(32);
  
	  

  if((code & 0xfffffe00)==0x0) {
		//FTRD - begin - 7/10/2001 
		//fprintf(stderr,"Start code %d found\n",code);
		incnt=0;
		log(code,32,"AnimationStartCode","");
		//FTRD - begin - 7/10/2001 
  }
  else 
  {
		//FTRD - begin - 7/10/2001 
		//fprintf(stderr,"Start code %d\n",code);
		bufptr=bufptrtmp;
		//FTRD - begin - 7/10/2001 
  }
  return TRUE;    
}

float AnimationStreamDecoder :: readFrameRate()
{
  int frame_rate,seconds;
  BOOL frequency_offset;
  
  //FTRD - begin - 7/10/2001 
  m_aTime.setFrameRate(frame_rate=readBits(8,"frameRate"));
  m_aTime.setFrameRateSeconds(seconds=readBits(4,"seconds"));
  m_aTime.setFrameRateOffset(frequency_offset=readBit("frequencyOffset"));    
  //FTRD - begin - 7/10/2001 
  
  return (float)1000/(1000+(frequency_offset ? 1 : 0))*
    (float)(frame_rate+(float)seconds/16);
}


void  AnimationStreamDecoder :: readTimeCode()
{
  m_aTime.setTimeCodeHours(readBits(5));
  m_aTime.setTimeCodeMinutes(readBits(6));
  m_aTime.setTimeCodeMarkerBit(readBit());
  m_aTime.setTimeCodeSeconds(readBits(6));
  
}

bool AnimationStreamDecoder :: readSkipFrames()
{
  int nfs,value;
  
  nfs = readBits(4,"SkipFrames");
  
  if(nfs<15) {
    m_aTime.setNumSkipFrames(nfs);
    return TRUE;
  }
  
  value = nfs;
  
  do {
    value = readBits(4,"SkipFrames");
    nfs+=value;
  }
  while (value == 15);
  
  if(nfs>254) {
    fprintf(stderr,"AnimationStreamDecoder: \n Maximum allowable frames skip is 254\n");
    return FALSE;
  }
  
  m_aTime.setNumSkipFrames(nfs);
  return TRUE;
}


bool AnimationStreamDecoder :: isNewAQP()
{
	return m_newAqp;
}

#endif //ANIMATION


