/*************************************************************************
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11


  SUBSYSTEM:  Decoder
  FILE:       AnimationQuantizer.cpp
  AUTHOR:     Mikael Bourges-Sevenier (France Telecom CNET/DIH/ATI)
	Bug fixes by Tony Walsh (Teltec)
  OVERVIEW:   Implementation of AnimationQuantizer
              Head file for animation streams decoding, quantization part

*************************************************************************/
#ifndef __ASQP_H__
#define __ASQP_H__

#include "defs.h"
#include "ztypes.h"
#include "Quantize.h"
#include "ArithModel.h"

//FTRD - begin - 7/10/2001 
#define TRACEP 
//FTRD - end - 7/10/2001 

class AnimationStreamDecoder;

class AnimationQuantizer : public Quantizer {
  float IMin[3]; // max 3 components
  float IMax[3];
  float PMin[3];
  float PMax[3];
  int INbBits,PNbBits;
  //FTRD - begin - 7/10/2001 
  float initial_IMin[3]; // max 3 components
  float initial_IMax[3];
  float initial_PMin[3];
  float initial_PMax[3];
  int initial_INbBits,initial_PNbBits;
 //FTRD - end - 7/10/2001 

  // the 15 quantizers...
  enum { AQ_NONE=0, AQ_POSITION3D, AQ_POSITION2D, AQ_RES1, 
	 AQ_COLOR, AQ_RES2, 
	 AQ_ANGLE, AQ_FLOAT, AQ_BOUNDFLOAT,
	 AQ_NORMAL, AQ_ROTATION, AQ_SIZE3D, AQ_SIZE2D, AQ_INT, 
	 AQ_RES3, AQ_RES4, AQ_LAST};

  static int nbBounds[];
  bool useDefault;

  int type; // quantizer used
  
  ArithModel *models; // Arithmetic predictive models (1 per coded component)
  ArithModel  bitModel; // for normals only
  int nbounds; // num of set of bounds
  int ncomp; // num of coded components
  float unQuantScale[4];  // The unQuantization scale indexed by aac component
  BOOL isUnitVector;  
  int     offset;  // The offset applied on Pmin for normals and rotations

  typedef struct _QVAL {
    int orientation;
    BOOL direction;
    BOOL inverse;
    float vq[4];		//LastUnQuantizedValue
	float vt[4];		//LastTrans
    void set(const struct _QVAL v) {
      orientation=v.orientation;
      direction=v.direction;
      inverse=v.inverse;
      for(int i=0;i<4;i++) 
		  vt[i]=v.vq[i];      
    }
    void add(const struct _QVAL v,int nbComp) {
      for(int i=0;i<nbComp;i++) 
	  {
		  vt[i]+=v.vt[i];
		  
		  //FTRD - begin - 7/10/2001 
		  TRACEP(stderr,"%0.0f ",vt[i]);
		  //FTRD - end - 7/10/2001 
	  }
		  
    }
  } QVAL;

  QVAL vq1,vDelta; // old quantized value & just decoded one

  // utility methods
  float unQuantize(int vq,int compIdx); // unquantize component compIdx
  //FTRD - begin - 7/10/2001 
  //void initModels();
  bool initModels();
  //FTRD - end - 7/10/2001 
  static void addDelta (QVAL &qVal1, QVAL &qDelta,int NbBits,int NBCOMP);
  static void quantToNat (QVAL &qv, float *natComp, int ncomp);
  void ParseAnimValue(AnimationStreamDecoder *pAS,BOOL isIntra);

public:
  AnimationQuantizer(int category=-1);
  ~AnimationQuantizer();
  void resetModels();
  void readInitialParameters(AnimationStreamDecoder *pAS,int category);
  void readSame(AnimationStreamDecoder *pAS);
  void setINbBits(int iNbBits);
  int decodeIComp(AnimationStreamDecoder *pAS);
  int decodePComp(AnimationStreamDecoder *pAS,ArithModel *models,int k);

  BOOL operator == (const AnimationQuantizer &qp) const;
  virtual int ParseInt (AnimationStreamDecoder *pAS,BOOL isIntra);
  virtual float ParseFloat (AnimationStreamDecoder *pAS,BOOL isIntra);
  virtual Vec3f ParseVec3f (AnimationStreamDecoder *pAS,BOOL isIntra);
  virtual Vec2f ParseVec2f (AnimationStreamDecoder *pAS,BOOL isIntra);
  virtual Color3f ParseColor (AnimationStreamDecoder *pAS,BOOL isIntra);
  virtual Rotation ParseRotation (AnimationStreamDecoder *pAS,BOOL isIntra);
  virtual Vec3f ParseNormal(AnimationStreamDecoder *pAS,BOOL isIntra);
};

#endif // __ASQP_H__
