// -*- c++ -*-

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

  SUBSYSTEM:  Decoder
  FILE:       AnimationStreamDec.h
  AUTHOR:     Mikael Bourges-Sevenier (France Telecom CNET/DIH/ATI)
  OVERVIEW:   Definition of ArithmeticDecoder
  ---------------------------------------------------------------------------
  
    The adaptative arithmetic decoder is derived from H263 Annex E. It can be
    instantiated with as many symbols as you want. This decoder is used by the
    FBA and animation stream decoders.
-------------------------------------------------------------------------*/

#ifndef __ARITHDEC_H_
#define __ARITHDEC_H_

#define CPLUSPLUS

#include "ArithModel.h"

class AnimationStreamDecoder;

class ArithmeticDecoder {
private:
  // Adapative Arithmetic Coding 
  unsigned int low, high, code_value, length, sacindex, cum;
  unsigned int bitcount;
  int nb_of_symbols, zerocount;
  unsigned int *freq,*cumul_freq;
  BOOL Bit;
  int  opposite_bits;

  //FTRD - begin - 7/10/2001 
  unsigned int m_lastvalue;
  //FTRD - end - 7/10/2001 

  AnimationStreamDecoder *pin;

public:
  ArithmeticDecoder(AnimationStreamDecoder *pAS=NULL);
  ~ArithmeticDecoder();
  void setStream(AnimationStreamDecoder *pAS) { pin=pAS; }

  void update(int symbol,ArithModel &model) { model.update(symbol); }
  void init(ArithModel &model) { model.init(); }

  int getBitCount() { return bitcount; }
  BOOL reset();
  int decode(ArithModel &model);
  int decodeAndUpdate(ArithModel &model);
  void adjustBits();

private:
  BOOL bit_out_psc_layer();
  
};

#endif // __ARITHDEC_H_
