/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

  SUBSYSTEM:  Decoder
  FILE:       ArithModel.h
  AUTHOR:     Mikael Bourges-Sevenier (France Telecom CNET/DIH/ATI)
  OVERVIEW:   Implementation of ArithModel
  ---------------------------------------------------------------------------
  
  The arithmetic model keeps track of all statistics to decode an
  animated field
-------------------------------------------------------------------------*/

#include "defs.h"
#include <stdio.h>
#include <stdlib.h>
#include "ArithModel.h"

ArithModel::ArithModel() :
  range_min(0),range_max(0),
  nb_of_symbols(256),
  cumul_freq(NULL),freq(NULL)
{
}

ArithModel::~ArithModel()
{
  if(cumul_freq) delete[] cumul_freq;
  if(freq) delete[] freq;
}

void ArithModel::init() 
{
  init(range_min,range_max);
}

void ArithModel::init(int nSymbols) 
{
  init(0,nSymbols-1);
}

void ArithModel::init(int min, int max) 
{
  range_min=min;
  range_max=max;
  if (nb_of_symbols==range_max-range_min+1 && freq!=NULL)
  {
  }
  else
  {
  nb_of_symbols = range_max-range_min+1;
  if(cumul_freq) delete[] cumul_freq;
  if(freq) delete[] freq;  
  freq=new int[nb_of_symbols];
  cumul_freq=new int[nb_of_symbols+1];
  }
  for(int i=0;i<nb_of_symbols;i++) {
    freq[i]=1;
    cumul_freq[i]=nb_of_symbols-i;
  }
  cumul_freq[nb_of_symbols]=0;
}

void ArithModel::update(int symbol)
{
  if (cumul_freq[0] == MAX_FREQUENCY) {
    int cum = 0;
    
    for(int i=nb_of_symbols-1; i>=0; i--) {
      freq[i] = (freq[i]+1)/2;
      cum += freq[i];
      cumul_freq[i] = cum;
    }
    cumul_freq[nb_of_symbols] = 0;
  }
  
  //assert(!(symbol < 0 || symbol >= nb_of_symbols));
  
  freq[symbol] ++;
  cumul_freq[symbol] ++;
  
  while(symbol>0)
    cumul_freq[--symbol] ++;
}
