// -*- c++ -*-

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

  SUBSYSTEM:  Decoder
  FILE:       ArithModel.h
  AUTHOR:     Mikael Bourges-Sevenier (France Telecom CNET/DIH/ATI)
  OVERVIEW:   Definition of ArithModel
  ---------------------------------------------------------------------------
  
  The arithmetic model keeps track of all statistics to decode an
  animated field
-------------------------------------------------------------------------*/

#if !defined(_arithmodel_h)	// Sentry, use file only if it's not already included.
#define _arithmodel_h

enum {
	Q1 = 16384,
	Q2 = 32768,
	Q3 = 49152,
	TOP = 65535,
	MAX_FREQUENCY = 16383,
//AC:20021009 - Arith Model Invalid Value -- begin
	ZEROMAX = 22,
//AC:20021009 - Arith Model Invalid Value -- end
	DEC_ERROR = -1,
	CODEVALNBBITS = 16 
};

class ArithModel { 
private:
  int range_min;
  int range_max;
  int nb_of_symbols;
  int *cumul_freq;     // cumulated frequency for each symbol
  int *freq;           // frequency of each symbol ... 
  
public:
  ArithModel();
  ~ArithModel();
  int getFreq(int idx) { return freq[idx]; }
  int getCumulFreq(int idx) { return cumul_freq[idx]; }
  void init();
  void init(int nSymbols);
  void init(int min, int max);
  void update(int symbol);  
};

#endif // _arithmodel_h
