/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  BIFS Decoder/ Compositor
FILE:       NodeFld.cpp
AUTHOR:     Mikael Bourges-Sevenier (France Telecom)
OVERVIEW:   Implementation of animation streams methods.
-------------------------------------------------------------------------*/
#include "defs.h"

#ifdef ANIMATION

#include "Nodes.h"
#include "NodeFld.h"
#include "MediaObj.h"
#include "BifsDec.h"
#include "Route.h"
#include "Root.h"
#include "Quantize.h"
#include "AnimStreamDec.h" // MBS: added


/*-----------------------------------------------------------------
The following functions are for parsing animation streams 
(mask and frames) resp.
added by MBS
-----------------------------------------------------------------*/
// dummy functions for NodeField
BOOL NodeField::animate(AnimationStreamDecoder *,BOOL)
{
  fprintf(stderr,"ERROR: in NodeField::parseAnimationMask\n");
  return FALSE;
}

void NodeField::parseAnimationMask(AnimationStreamDecoder *, int)
{
  fprintf(stderr,"ERROR: in NodeField::parseAnimationMask\n");
}

BOOL SF<int>::animate(AnimationStreamDecoder *pAS,BOOL intra)
{
  if(intra) {
	//FTRD - begin - 7/10/2001 
    if(pAS->readBit("hasQP")) // if we have new quantization parameters
      qp.readSame(pAS);
	//FTRD - end - 7/10/2001 
  }

  int val = qp.ParseInt(pAS,intra);
  BOOL ret = ! (val==m_value);
  m_value = val;
  return ret;
}

BOOL MF<int>::animate(AnimationStreamDecoder *pAS,BOOL intra)
{
  if(intra) {
	//FTRD - begin - 7/10/2001 
    if(pAS->readBit("hasQP")) // if we have new quantization parameters
      qp.readSame(pAS);
	//FTRD - end- 7/10/2001 
  }
	
  BOOL ret = FALSE;
	
  int val;

  if(isTotal) {
    for(int i=0;i<GetLength();i++) {
      val = qp.ParseInt(pAS,intra);
      if (!ret) ret = ! (m_array[i]==val);
	  m_array[i]=val;
    }
  }
  else {
    for(int i=0;i<idx.GetLength();i++) {
      val = qp.ParseInt(pAS,intra);
      if (!ret) ret = ! (m_array[idx[i]]==val);
	  m_array[idx[i]]=val;
    }
  }
  return ret;
}

void SF<int>::parseAnimationMask(AnimationStreamDecoder *pAS, int anim)
{
	//FTRD - begin -11/10/2001 
	/* not implemented */
	qp.readInitialParameters(pAS,anim);
  //FTRD - end- 11/10/2001 
}

void MF<int>::parseAnimationMask(AnimationStreamDecoder *pAS, int anim)
{
  /* not implemented */
	//FTRD - begin - 7/10/2001 
  isTotal=pAS->readBit("isTotal");

   if(!isTotal) 
  {
	  int nbBits = pAS->readBits(5,"indexListNbBits");
	  do {
		  idx += pAS->readBits(nbBits,"index");
	  } while(pAS->readBit("moreIndices"));
  }
   //FTRD - end - 7/10/2001 
  qp.readInitialParameters(pAS,anim);
}

BOOL SF<float>::animate(AnimationStreamDecoder *pAS,BOOL intra)
{
  if(intra) {
	//FTRD - begin - 7/10/2001 
    if(pAS->readBit("hasQP")) // if we have new quantization parameters
      qp.readSame(pAS);
	//FTRD - end- 7/10/2001 
  }
  float val = qp.ParseFloat(pAS,intra);
  BOOL ret = ! (m_value==val);
  m_value = val;
  return ret;
}

BOOL MF<float>::animate(AnimationStreamDecoder *pAS,BOOL intra)
{
  BOOL ret = FALSE;
  float val;
  if(intra) {
	//FTRD - begin - 7/10/2001 
    if(pAS->readBit("hasQP")) // if we have new quantization parameters
      qp.readSame(pAS);
	//FTRD - end - 7/10/2001 
  }

  if(isTotal) {
    for(int i=0;i<GetLength();i++) {
      val=qp.ParseFloat(pAS,intra);
	  if (!ret) ret = ! (m_array[i]==val);
	  m_array[i] = val;
    }
  }
  else {
    for(int i=0;i<idx.GetLength();i++) {
      
      val=qp.ParseFloat(pAS,intra);
	  if (!ret) ret = ! (m_array[idx[i]]==val);
	  m_array[idx[i]] = val;
    }
  }
  return ret;
}

void SF<float>::parseAnimationMask(AnimationStreamDecoder *pAS, int anim)
{
  qp.readInitialParameters(pAS,anim);
}

void MF<float>::parseAnimationMask(AnimationStreamDecoder *pAS, int anim)
{
  //FTRD - begin - 7/10/2001 
  isTotal=pAS->readBit("isTotal");

   if(!isTotal) 
  {
	  int nbBits = pAS->readBits(5,"indexListNbBits");
	  do {
		  idx += pAS->readBits(nbBits,"index");
	  } while(pAS->readBit("moreIndices"));
  }
   //FTRD - end - 7/10/2001 
  qp.readInitialParameters(pAS,anim);
}

BOOL SF<Bool>::animate(AnimationStreamDecoder *,BOOL)
{
  /* not implemented */
  return FALSE;
}

BOOL MF<Bool>::animate(AnimationStreamDecoder *,BOOL)
{
  /* not implemented */
  return FALSE;
}

void SF<Bool>::parseAnimationMask(AnimationStreamDecoder *, int)
{
  /* not implemented */
}

void MF<Bool>::parseAnimationMask(AnimationStreamDecoder *, int)
{
  /* not implemented */
}

BOOL SF<Vec2f>::animate(AnimationStreamDecoder *pAS,BOOL intra)
{
  if(intra) {
	//FTRD - begin - 7/10/2001 
    if(pAS->readBit("hasQP")) // if we have new quantization parameters
      qp.readSame(pAS);
	//FTRD - end - 7/10/2001 
  } 
  Vec2f val = qp.ParseVec2f(pAS,intra);
  BOOL ret = ! (m_value==val);
  m_value = val;
  return ret;
}

BOOL MF<Vec2f>::animate(AnimationStreamDecoder *pAS,BOOL intra)
{
  if(intra) {
	//FTRD - begin - 7/10/2001 
    if(pAS->readBit("hasQP")) // if we have new quantization parameters
      qp.readSame(pAS);
	//FTRD - end - 7/10/2001 
  }   
  Vec2f val;
  BOOL ret = FALSE;
  if(isTotal) {
    for(int i=0;i<GetLength();i++) {
      val=qp.ParseVec2f(pAS,intra);
	  if (!ret) ret = ! (m_array[i]==val);
	  m_array[i] = val;
    }
  }
  else {
    for(int i=0;i<idx.GetLength();i++) {
      val=qp.ParseVec2f(pAS,intra);
	  if (!ret) ret = ! (m_array[idx[i]]==val);
	  m_array[idx[i]] = val;
    }
  }
  return ret;
}

void SF<Vec2f>::parseAnimationMask(AnimationStreamDecoder *pAS, int anim)
{
  qp.readInitialParameters(pAS,anim);
}

void MF<Vec2f>::parseAnimationMask(AnimationStreamDecoder *pAS, int anim)
{
  //FTRD - begin - 7/10/2001 
  isTotal=pAS->readBit("isTotal");

   if(!isTotal) 
  {
	  int nbBits = pAS->readBits(5,"indexListNbBits");
	  do {
		  idx += pAS->readBits(nbBits,"index");
	  } while(pAS->readBit("moreIndices"));
  }
 //FTRD - end - 7/10/2001 

  qp.readInitialParameters(pAS,anim);
}

BOOL SF<Vec3f>::animate(AnimationStreamDecoder *pAS,BOOL intra)
{
  if(intra) {
	//FTRD - begin - 7/10/2001 
    if(pAS->readBit("hasQP")) // if we have new quantization parameters
      qp.readSame(pAS);
	//FTRD - end - 7/10/2001 
  }
  Vec3f val = qp.ParseVec3f(pAS,intra);
  BOOL ret = ! (m_value==val);
  m_value = val;
  return ret;
}

BOOL MF<Vec3f>::animate(AnimationStreamDecoder *pAS,BOOL intra)
{
  if(intra) {
	//FTRD - begin - 7/10/2001 
    if(pAS->readBit("hasQP")) // if we have new quantization parameters
      qp.readSame(pAS);
	//FTRD - end - 7/10/2001 
  }
  BOOL ret = FALSE;
  Vec3f val;
  if(isTotal) {
    for(int i=0;i<GetLength();i++) {
      val=qp.ParseVec3f(pAS,intra);
	  if (!ret) ret = ! (m_array[i]==val);
	  m_array[i]=val;
    }
  }
  else {
    for(int i=0;i<idx.GetLength();i++) {
      val=qp.ParseVec3f(pAS,intra);
	  if (!ret) ret = ! (m_array[idx[i]]==val);
	  m_array[idx[i]]=val;
    }
  }
  return ret;
}

void SF<Vec3f>::parseAnimationMask(AnimationStreamDecoder *pAS, int anim)
{
  qp.readInitialParameters(pAS,anim);
}

void MF<Vec3f>::parseAnimationMask(AnimationStreamDecoder *pAS, int anim)
{
 //FTRD - begin - 7/10/2001 
  isTotal=pAS->readBit("isTotal");
  if(!isTotal) 
  {
	  int nbBits = pAS->readBits(5,"indexListNbBits");
	  do {
		  idx += pAS->readBits(nbBits,"index");
	  } while(pAS->readBit("moreIndices"));
  }
  qp.readInitialParameters(pAS,anim);
  //FTRD - end - 7/10/2001 
}

void SF<Image>::parseAnimationMask(AnimationStreamDecoder *, int )
{
  /* not implemented */
}

BOOL SF<Image>::animate(AnimationStreamDecoder *,BOOL)
{
  /* not implemented */
  return FALSE;
}

BOOL SF<Time>::animate(AnimationStreamDecoder *,BOOL)
{
  /* not implemented */
  return FALSE;
}

BOOL MF<Time>::animate(AnimationStreamDecoder *,BOOL)
{
  /* not implemented */
  return FALSE;
}

BOOL MF<Image>::animate(AnimationStreamDecoder *,BOOL)
{
  /* not implemented */
  return FALSE;
}

void SF<Time>::parseAnimationMask(AnimationStreamDecoder *, int)
{
  /* not implemented */
}

void MF<Time>::parseAnimationMask(AnimationStreamDecoder *, int)
{
  /* not implemented */
}

void MF<Image>::parseAnimationMask(AnimationStreamDecoder *, int)
{
  /* not implemented */
}

BOOL SF<Rotation>::animate(AnimationStreamDecoder *pAS,BOOL intra)
{
  if(intra) {
	//FTRD - begin - 7/10/2001 
    if(pAS->readBit("hasQP")) // if we have new quantization parameters
      qp.readSame(pAS);
	//FTRD - end - 7/10/2001 
  }  
  Rotation val = qp.ParseRotation(pAS,intra);
  BOOL ret = ! (m_value==val);
  m_value = val;
  return ret;
}

BOOL MF<Rotation>::animate(AnimationStreamDecoder *pAS,BOOL intra)
{
  if(intra) {
	//FTRD - begin - 7/10/2001 
    if(pAS->readBit("hasQP")) // if we have new quantization parameters
      qp.readSame(pAS);
	//FTRD - end - 7/10/2001 
  }
  BOOL ret = FALSE;
  Rotation val;

  if(isTotal) {
    for(int i=0;i<GetLength();i++) {
      val=qp.ParseRotation(pAS,intra);
	  if (!ret) ret = ! (m_array[i]==val);
	  m_array[i] = val;
    }
  }
  else {
    for(int i=0;i<idx.GetLength();i++) {
      val=qp.ParseRotation(pAS,intra);
	  if (!ret) ret = ! (m_array[idx[i]]==val);
	  m_array[idx[i]] = val;
    }
  }
  return ret;
}

void SF<Rotation>::parseAnimationMask(AnimationStreamDecoder *pAS, int anim)
{
  qp.readInitialParameters(pAS,anim);
}

void MF<Rotation>::parseAnimationMask(AnimationStreamDecoder *pAS, int anim)
{
   //FTRD - begin - 7/10/2001 
  isTotal=pAS->readBit("isTotal");
    if(!isTotal) 
  {
	  int nbBits = pAS->readBits(5,"indexListNbBits");
	  do {
		  idx += pAS->readBits(nbBits,"index");
	  } while(pAS->readBit("moreIndices"));
  }
	//FTRD - end- 7/10/2001 
 
  qp.readInitialParameters(pAS,anim);
}

BOOL SF<Rotation2D>::animate(AnimationStreamDecoder *,BOOL)
{
  /* not implemented */
  return FALSE;
}

BOOL MF<Rotation2D>::animate(AnimationStreamDecoder *,BOOL)
{
  /* not implemented */
  return FALSE;
}

void SF<Rotation2D>::parseAnimationMask(AnimationStreamDecoder *, int)
{
  /* not implemented */
}

void MF<Rotation2D>::parseAnimationMask(AnimationStreamDecoder *, int)
{
  /* not implemented */
}

BOOL SF<Color3f>::animate(AnimationStreamDecoder *pAS,BOOL intra)
{
  if(intra) {
    if(pAS->readBit("hasQP")) // if we have new quantization parameters
     qp.readSame(pAS);
  } 
  Color3f val = qp.ParseColor(pAS,intra);
  BOOL ret = ! (val==m_value);
  m_value=val;
  return ret;
}

BOOL MF<Color3f>::animate(AnimationStreamDecoder *pAS,BOOL intra)
{
  if(intra) {
	//FTRD - begin - 7/10/2001 
    if(pAS->readBit("hasQP")) // if we have new quantization parameters
      qp.readSame(pAS);
	//FTRD - end- 7/10/2001 
  }
  BOOL ret = FALSE;
  Color3f val;

  if(isTotal) {
    for(int i=0;i<GetLength();i++) {
      val=qp.ParseColor(pAS,intra);
	  if (!ret) ret = ! (m_array[i]==val);
	  m_array[i] = val;
    }
  }
  else {
    for(int i=0;i<idx.GetLength();i++) {
      val=qp.ParseColor(pAS,intra);
	  if (!ret) ret = ! (m_array[idx[i]]==val);
	  m_array[idx[i]] = val;
    }
  }
  return ret;
}

void SF<Color3f>::parseAnimationMask(AnimationStreamDecoder *pAS, int anim)
{
  qp.readInitialParameters(pAS,anim);
}

void MF<Color3f>::parseAnimationMask(AnimationStreamDecoder *pAS, int anim)
{
  isTotal=pAS->readBit();
    if(!isTotal) 
  {
	  //FTRD - begin - 7/10/2001 
	  int nbBits = pAS->readBits(5,"indexListNbBits");
	  do {
		  idx += pAS->readBits(nbBits,"index");
	  } while(pAS->readBit("moreIndices"));
	  //FTRD - end- 7/10/2001 
  }
 
  qp.readInitialParameters(pAS,anim);
}

BOOL SF<ZString>::animate(AnimationStreamDecoder *,BOOL)
{
  /* not implemented */
  return FALSE;
}

BOOL MF<ZString>::animate(AnimationStreamDecoder *,BOOL)
{
  /* not implemented */
  return FALSE;
}

void SF<ZString>::parseAnimationMask(AnimationStreamDecoder *, int)
{
  /* not implemented */
}

void MF<ZString>::parseAnimationMask(AnimationStreamDecoder *, int)
{
  /* not implemented */
}

BOOL SF<Url>::animate(AnimationStreamDecoder *,BOOL)
{
  /* not implemented */
  return FALSE;
}

BOOL MF<Url>::animate(AnimationStreamDecoder *,BOOL)
{
  /* not implemented */
  return FALSE;
}

void SF<Url>::parseAnimationMask(AnimationStreamDecoder *, int)
{
  /* not implemented */
}

void MF<Url>::parseAnimationMask(AnimationStreamDecoder *, int)
{
  /* not implemented */
}

/*-------------------------------------------------------------
TO DO: Animation code for Vec4F
-------------------------------------------------------------*/
BOOL SF<Vec4f>::animate(AnimationStreamDecoder * /*pAS*/,BOOL /*intra*/)
{
  return FALSE;
}

void SF<Vec4f>::parseAnimationMask(AnimationStreamDecoder * /*pAS*/, int /*anim*/)
{
}

BOOL MF<Vec4f>::animate(AnimationStreamDecoder * /*pAS*/,BOOL /*intra*/)
{
  return FALSE;
}

void MF<Vec4f>::parseAnimationMask(AnimationStreamDecoder * /*pAS*/, int /*anim*/)
{
}

#endif ANIMATION
