/***********************************************************************
This software module was originally developed by Mikael Bourges-Sevenier
(France Telecom CNET) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. France Telecom CNET. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  BIFS Decoder/ Compositor
FILE:       Quantize.cpp
AUTHOR:     Mikael Bourges-Sevenier (France Telecom CNET)
OVERVIEW:   Implementation of Quantizer
-------------------------------------------------------------------------*/
//$Log: MQuantize.cpp,v $
//Revision 1.1.1.2  2006/04/14 14:11:03  hill
//no message
//
//Revision 1.1.1.1  2003/12/24 00:42:02  mikeseven
//Initial release of the CVS repository for MPEG-4 Systems code
//after the code was corrupted due to reorganization and human 
//errors.
//
//Revision 1.1  2002/07/18 00:59:44  mikeseven
//Initial revision
//
//Final
//Revision 1.x 2000/10/12  Pascal Belin
//QP bug fix
//Revision 1.2  2000/03/03 18:41:20  Alex Cotarmanac'h
//Version 1 -- Alexandre Cotarmanac_h
#include "defs.h"
#include <float.h>
#include "Quantize.h"
#include "BifsDec.h"
#include "ztl.h"
#include "NodeFld.h"
#include <math.h>

#define QPVAL(type, id) ((type) *(SF<type>*) m_pQP->GetField(id))

#define PI 3.1415927

BOOL Quantizer::HasEfficientFloat() 
{ 
  if(!m_pQP) return FALSE;    // PB uses this test instead of CanQuantize
  return QPVAL(Bool,EFLOAT);  // to check that Efficient coding is active
}

/*-----------------------------------------------------------------
Test is a quantizer is used
-----------------------------------------------------------------*/
BOOL Quantizer::CanQuantize () {
  if(!m_pQP) return FALSE;

  // PB removed if(QPVAL(Bool,EFLOAT)) return TRUE;

  switch(m_nQuantizationCode) {
  case Q_3D:
    return QPVAL(Bool,POS3DQ); 
  case Q_2D:
    return QPVAL(Bool,POS2DQ); 
  case Q_DRAWORDER:
    return QPVAL(Bool,DOQ); 
  case Q_COLOR:
    return QPVAL(Bool,COLORQ); 
  case Q_TEXTURECOORDINATE:
    return QPVAL(Bool,TCQ); 
  case Q_ANGLE:
    return QPVAL(Bool,ANGLEQ); 
  case Q_SCALE:
    return QPVAL(Bool,SCALEQ); 
  case Q_KEY:
    return QPVAL(Bool,KEYQ); 
  case Q_NORMAL:
  case Q_ROTATION:
    return QPVAL(Bool,NORMALQ); 
  case Q_SIZE3D:
    return QPVAL(Bool,SIZEQ); 
  case Q_SIZE2D:
    return QPVAL(Bool,SIZEQ); 
  case Q_LSQ:
    return TRUE; 
	//AC20000227
  case Q_COORDIDX:
	return TRUE;
	//AC20000227
  }
  return FALSE;
}

/*-----------------------------------------------------------------
Parse an float from the BIFS stream
-----------------------------------------------------------------*/
float Quantizer :: ParseFloat (BIFSDecoder *pBifs) {
  float nResult=0.0f;  
  // if(m_nQuantizationCode == Q_NONE) return nResult;   // PB : return nResult moved after the if/else
  // PB if(HasEfficientFloat()) was here

  if(CanQuantize()) {										// PB EfficientCoding fix starts
	  float mini=0,maxi=0,nBits = 0;
    
	  switch(m_nQuantizationCode) {
	  case Q_DRAWORDER:
		  //fprintf(stderr,"Dequant DrawOrder\n");
		  mini=max(m_pMin[0],QPVAL(float,DOMIN));		  
		  maxi=min(m_pMax[0],QPVAL(float,DOMAX));
		  nBits= QPVAL(int,DONB);
		  break;
	  case Q_COLOR:
		  //fprintf(stderr,"Dequant Color\n");
		  mini=max(m_pMin[0],QPVAL(float,COLORMIN));		  
		  maxi=min(m_pMax[0],QPVAL(float,COLORMAX));
		  nBits= QPVAL(int,COLORNB);
		  break;
	  case Q_ANGLE:
		  //fprintf(stderr,"Dequant Angle\n");
		  mini=max(m_pMin[0],QPVAL(float,ANGLEMIN));		  
		  maxi=min(m_pMax[0],QPVAL(float,ANGLEMAX));
		  nBits= QPVAL(int,ANGLENB);
		  break;
	  case Q_SCALE:
		  //fprintf(stderr,"Dequant Scale\n");
		  mini=max(m_pMin[0],QPVAL(float,SCALEMIN));		  
		  maxi=min(m_pMax[0],QPVAL(float,SCALEMAX));
		  nBits= QPVAL(int,SCALENB);
		  break;
	  case Q_KEY:
		  //fprintf(stderr,"Dequant Key\n");
		  mini=max(m_pMin[0],QPVAL(float,KEYMIN));		  
		  maxi=min(m_pMax[0],QPVAL(float,KEYMAX));
		  nBits= QPVAL(int,KEYNB);
		  break;
	  case Q_SIZE2D: 
	  case Q_SIZE3D: {
		  mini=max(m_pMin[0],QPVAL(float,SIZEMIN));
		  maxi=min(m_pMax[0],QPVAL(float,SIZEMAX));
		  nBits=QPVAL(int,SIZENB);
	      break;
					 }
	  case Q_LSQ:
		  //fprintf(stderr,"Dequant LSQ\n");
		  mini=0.f;
		  maxi=1<<m_nBits;
		  nBits=m_nBits;
		  break;
	  }
	  return InvQuantize(mini,maxi,nBits,pBifs -> ParseInt (nBits, "quantized value")); // PB logout change
  }
  else {
	  if(HasEfficientFloat())
		  return ParseEfficientFloat (pBifs);
  }
  
  return nResult;											// PB end

}

/*-----------------------------------------------------------------
Parse a Vec3f from the BIFS stream
-----------------------------------------------------------------*/
Vec3f Quantizer :: ParseVec3f (BIFSDecoder *pBifs) {
  Vec3f nResult;
  // if(m_nQuantizationCode == Q_NONE) return nResult;   // PB : return nResult moved after the if/else

  // PB if(HasEfficientFloat()) was here

  if(CanQuantize()) {										// PB EfficientCoding fix starts
	  float mini[3],maxi[3],nBits=0;
	
	  switch(m_nQuantizationCode) {
	  case Q_3D:
		  //fprintf(stderr,"Dequantizing a Vec3f\n");
		  mini[0]=max(m_pMin[0],QPVAL(Vec3f,POS3DMIN).x);
		  mini[1]=max(m_pMin[0],QPVAL(Vec3f,POS3DMIN).y);
		  mini[2]=max(m_pMin[0],QPVAL(Vec3f,POS3DMIN).z);
      
		  maxi[0]=min(m_pMax[0],QPVAL(Vec3f,POS3DMAX).x);
		  maxi[1]=min(m_pMax[0],QPVAL(Vec3f,POS3DMAX).y);
		  maxi[2]=min(m_pMax[0],QPVAL(Vec3f,POS3DMAX).z);
      
		  nBits= QPVAL(int,POS3DNB);
		  break;
	  case Q_NORMAL:
		  //fprintf(stderr,"Dequantizing a Normal\n");
		  nBits= QPVAL(int,NORMALNB);
		  return ParseNormal(pBifs,nBits);
		  break;
      case Q_SCALE: {
		  //fprintf(stderr,"Dequantizing a Scale3D\n");
		  for(int i=0;i<3;i++) {
			  mini[i]=max(m_pMin[0],QPVAL(float,SCALEMIN));
			  maxi[i]=min(m_pMax[0],QPVAL(float,SCALEMAX));
		  }
		  
		  nBits= QPVAL(int,SCALENB);
		  break;
					}
	  case Q_SIZE3D: {
		  //fprintf(stderr,"Dequantizing a Size3D\n");
		  for(int i=0;i<3;i++) {
			  mini[i]=max(m_pMin[0],QPVAL(float,SIZEMIN));
			  maxi[i]=min(m_pMax[0],QPVAL(float,SIZEMAX));
		  }
		  nBits=QPVAL(int,SIZENB);
		  break;
					 }
	  }
	  
	  nResult.x = InvQuantize(mini[0],maxi[0],nBits,pBifs -> ParseInt (nBits, "quantized value")); // PB logout change
	  nResult.y = InvQuantize(mini[1],maxi[1],nBits,pBifs -> ParseInt (nBits, "quantized value")); // PB logout change
	  nResult.z = InvQuantize(mini[2],maxi[2],nBits,pBifs -> ParseInt (nBits, "quantized value")); // PB logout change

	  return nResult;
  }
  else{
	  if(HasEfficientFloat()) {
		  nResult.x=ParseEfficientFloat (pBifs);
		  nResult.y=ParseEfficientFloat (pBifs);
		  nResult.z=ParseEfficientFloat (pBifs);
		  return nResult;
	  }
  }
  
  return nResult;								// PB end

}

/*-----------------------------------------------------------------
Parse a Vec2f from the BIFS stream
-----------------------------------------------------------------*/
Vec2f Quantizer :: ParseVec2f (BIFSDecoder *pBifs) {
  Vec2f nResult;
  // if(m_nQuantizationCode == Q_NONE) return nResult;   // PB : return nResult moved after the if/else
  
  // PB if(HasEfficientFloat()) was here
  
  if(CanQuantize()) {										// PB EfficientCoding fix starts
	  float mini[2],maxi[2];

	  int nBits = 0;

	  switch(m_nQuantizationCode) {
	  case Q_2D:
		  //fprintf(stderr,"Dequantizing a Vec2f\n");
		  mini[0]=max(m_pMin[0],QPVAL(Vec2f,POS2DMIN).x);
		  mini[1]=max(m_pMin[0],QPVAL(Vec2f,POS2DMIN).y);
		  maxi[0]=min(m_pMax[0],QPVAL(Vec2f,POS2DMAX).x);
		  maxi[1]=min(m_pMax[0],QPVAL(Vec2f,POS2DMAX).y);
		  nBits= QPVAL(int,POS2DNB);
		  break;
	  case Q_TEXTURECOORDINATE:
		  //fprintf(stderr,"Dequantizing a Texture Coordinate\n");
		  mini[0]=max(m_pMin[0],QPVAL(float,TCMIN));
		  mini[1]=max(m_pMin[0],QPVAL(float,TCMIN));
		  
		  maxi[0]=min(m_pMax[0],QPVAL(float,TCMAX));
		  maxi[1]=min(m_pMax[0],QPVAL(float,TCMAX));
		  nBits= QPVAL(int,TCNB);
		  break;
	  case Q_SCALE: {
		  //fprintf(stderr,"Dequantizing a Scale2D\n");
		  for(int i=0;i<2;i++) {
			  mini[i]=max(m_pMin[0],QPVAL(float,SCALEMIN));
			  maxi[i]=min(m_pMax[0],QPVAL(float,SCALEMAX));    
		  }      
		  nBits= QPVAL(int,SCALENB);
		  break;
					}
	  case Q_SIZE2D: {
		  //fprintf(stderr,"Dequantizing a Size2D\n");
		  for(int i=0;i<2;i++) {
			  mini[i]=max(m_pMin[0],QPVAL(float,SIZEMIN));	  
			  maxi[i]=min(m_pMax[0],QPVAL(float,SIZEMAX));
		  }
		  nBits=QPVAL(int,SIZENB);
		  break;
					 }
	  }

	  nResult.x = InvQuantize(mini[0],maxi[0],nBits,pBifs -> ParseInt (nBits, "quantized value")); // PB logout change
	  nResult.y = InvQuantize(mini[1],maxi[1],nBits,pBifs -> ParseInt (nBits, "quantized value")); // PB logout change
	  return nResult;
	    
  }

  else{											
	  if(HasEfficientFloat()) {
		  nResult.x=ParseEfficientFloat (pBifs);
		  nResult.y=ParseEfficientFloat (pBifs);
		  return nResult;
	  }
  }										// PB end

  return nResult;
  
}

/*-----------------------------------------------------------------
Parse a Rotation from the BIFS stream
-----------------------------------------------------------------*/
// MBS 110598: new FDIS verision
Rotation Quantizer :: ParseRotation (BIFSDecoder *pBifs) 
{
  Rotation nResult;
  // if(m_nQuantizationCode == Q_NONE) return nResult;   // PB : return nResult moved after the if/else

  // PB if(HasEfficientFloat()) should be moved at the end of the method
  
  if(CanQuantize()) {										// PB EfficientCoding fix starts
	  
	  float *comp=ParseOnUnitSphere(pBifs, QPVAL(int,NORMALNB), 3);
	  
	  // converts quaternion to rotation
	  nResult.theta=2*acos(comp[0]);
	  //AC20000227 BugFix : potential divide by zero if sin2==0
	  float epsilon=FLT_EPSILON;
	  float sin2= sin(nResult.theta/2);
	  if (fabs(sin2)<=epsilon) 
	  {
		  for(int i=1;i<4;i++)
			  comp[i] = 0;
		  comp[3]=1;
	  }
	  else
	  {
		  for(int i=1;i<4;i++)
			  comp[i] /= sin2;
	  }
	  //AC end 
	  nResult.x=comp[1];
	  nResult.y=comp[2];
	  nResult.z=comp[3];

	  return nResult;
  }
  else {
	  if(HasEfficientFloat()) {
		  nResult.x=ParseEfficientFloat (pBifs);
		  nResult.y=ParseEfficientFloat (pBifs);
		  nResult.z=ParseEfficientFloat (pBifs);
		  nResult.theta=ParseEfficientFloat (pBifs);
		  return nResult;
	  }
  }

  return nResult;

}

/* MBS 110598: old FCD N2201 version
Rotation Quantizer :: ParseRotation (BIFSDecoder *pBifs) {
  Rotation nResult;
  if(m_nQuantizationCode == Q_NONE) return nResult;
  //fprintf(stderr,"Dequant Rotation\n");

  int nBitsNormal= QPVAL(int,NORMALNB);
  int nBitsAngle = QPVAL(int,ANGLENB);

  Vec3f normal=ParseNormal(pBifs,nBitsNormal);

  float mini=max(m_pMin[0],QPVAL(float,ANGLEMIN));		  
  float maxi=min(m_pMax[0],QPVAL(float,ANGLEMAX));

  nResult.x = normal.x;
  nResult.y = normal.y;
  nResult.z = normal.z;
  nResult.theta = InvQuantize(mini,maxi,nBitsAngle,pBifs -> ParseInt (nBitsAngle));
  return nResult;
}
*/
/*-----------------------------------------------------------------
Parse a Rotation2D from the BIFS stream
-----------------------------------------------------------------*/
Rotation2D Quantizer :: ParseRotation2D (BIFSDecoder * /*pBifs*/) {
  Rotation2D nResult;
  if(m_nQuantizationCode == Q_NONE) return nResult;
  //fprintf(stderr,"Dequant Rotation2D\n");
  return nResult;
}

/*-----------------------------------------------------------------
Parse a Color3f from the BIFS stream
-----------------------------------------------------------------*/
Color3f Quantizer :: ParseColor3f (BIFSDecoder *pBifs) {
  Color3f nResult;
  // if(m_nQuantizationCode == Q_NONE) return nResult;   // PB : return nResult moved after the if/else
  // PB if(HasEfficientFloat()) was here

  if(CanQuantize()) {										// PB EfficientCoding fix starts
	  //fprintf(stderr,"Dequant Color\n");
	  float mini= max(m_pMin[0],QPVAL(float,COLORMIN));
	  float maxi= min(m_pMax[0],QPVAL(float,COLORMAX));
	  int nBits= QPVAL(int,COLORNB);

	  nResult.r = InvQuantize(mini,maxi,nBits,pBifs -> ParseInt (nBits, "quantizedRed")); // PB logout change
	  nResult.g = InvQuantize(mini,maxi,nBits,pBifs -> ParseInt (nBits, "quantizedGreen")); // PB logout change
	  nResult.b = InvQuantize(mini,maxi,nBits,pBifs -> ParseInt (nBits, "quantizedBlue")); // PB logout change
	  return nResult;
  }
  else {
	  if(HasEfficientFloat()) {						
		  nResult.r=ParseEfficientFloat (pBifs);
		  nResult.g=ParseEfficientFloat (pBifs);
		  nResult.b=ParseEfficientFloat (pBifs);
		  return nResult;
	  }										
  }

  return nResult;

}

/*-----------------------------------------------------------------
Parse an object on unit a sphere
-----------------------------------------------------------------*/
float *Quantizer :: ParseOnUnitSphere(BIFSDecoder *pBifs, int nbBits, int nbComp)
{
    int i,len=nbComp+1;
    float *comp=new float[len];

    float direction   = 1;												 // PB QP9 fix (9.3.7.25.1) + logout change
    if(nbComp==2)
      direction = (float)(1 - 2 * (pBifs -> ParseInt (1, "direction"))); //AC20000229 BugFix:direction in {-1,1}
	// I also have changed the encoding in MEncQuant (Animation)
	// so that it is like in the predictive encoding scheme of inverse.
    int orientation = pBifs -> ParseInt (2, "orientation");				// PB end


    for(i=0;i<nbComp;i++)
	{
		//AC 20000227: BugFix InvQuantize only handles positive values.( better say in min..max )
		int value=pBifs -> ParseInt (nbBits, "reduced component")-(1<<(nbBits-1)); // PB logout change
		int sgn = ((value>=0)?1:-1);
		comp[i] = sgn * InvQuantize(0, 1, nbBits-1,sgn*value );
	}

    float *tanv=new float[nbComp];
    float delta=1;
    for (i=0;i<nbComp;i++) {
      tanv[i] = tan( M_PI * comp[i] / 4);
      delta += tanv[i]*tanv[i];
    }
    delta = direction / sqrt(delta);
    comp[orientation] = delta;

    for (i=0;i<nbComp;i++) 
      comp[(orientation+i+1) % len] = tanv[i] * delta;  

    delete[] tanv;
    return comp;
}

/*-----------------------------------------------------------------
Parse a normal from the BIFS stream
-----------------------------------------------------------------*/
// MBS 110598: new FDIS version
Vec3f Quantizer :: ParseNormal (BIFSDecoder *pBifs, int nbBits) 
{
  float *comp=ParseOnUnitSphere(pBifs, nbBits, 2);

  // normalize to avoid round-off problems
  float delta=0;
  for(int i=0;i<3;i++)
    delta += comp[i]*comp[i];

  delta=sqrt(delta);
  if(delta > 0)
    for(int j=0;j<3;j++)
      comp[j] /= delta;

  return Vec3f(comp[0],comp[1],comp[2]);
}

/* MBS 110598: old FCD N2201 version
Vec3f Quantizer :: ParseNormal (BIFSDecoder *pBifs, int nbBits) {
  Vec3f nResult;
  if(m_nQuantizationCode == Q_NONE) return nResult;
  
  int octant=pBifs -> ParseInt (3);
  int triant=pBifs -> ParseInt (2);
  int aq=pBifs -> ParseInt (nbBits);
  int bq=pBifs -> ParseInt (nbBits);
  float a=(float) aq/(float) ((1<<nbBits)-1);
  float b=(float) bq/(float) ((1<<nbBits)-1);
  
  float tana=tan(a*M_PI/4.),tanb=tan(b*M_PI/4.);
  float delta=sqrt(1+tana*tana+tanb*tanb);
  switch(triant) {
  case 0: // x dominant
    nResult.x=1./delta;
    nResult.y=tana/delta;
    nResult.z=tanb/delta;
    break;
  case 1:
    nResult.x=tana/delta;
    nResult.y=1./delta;
    nResult.z=tanb/delta;
    break;
  case 2:
    nResult.x=tana/delta;
    nResult.y=tanb/delta;
    nResult.z=1./delta;
    break;
  }
  if((octant & 1)==1) nResult.x=-nResult.x;
  octant>>=1;
  if((octant & 1)==1) nResult.y=-nResult.y;
  octant>>=1;
  if((octant & 1)==1) nResult.z=-nResult.z;
  octant>>=1;

  return nResult;
}
*/

/*-----------------------------------------------------------------
Inverse Linear Scalar Quantizer
-----------------------------------------------------------------*/
float Quantizer::InvQuantize(float mini, float maxi, int nbits, int v) {
  if(v==0) return mini;
  if(v==(int) ((1<<nbits)-1)) return maxi;
  return (float) (mini+(maxi-mini)/((float) (1<<nbits)-1)*(float) v);
}

/*-----------------------------------------------------------------
The following is the efficient float parse function 
MBS 110698
-----------------------------------------------------------------*/
float Quantizer :: ParseEfficientFloat (BIFSDecoder *pBifs)
{	
  union
  {	
    float f;
    long l;
  } value;

  int mantissaLength=pBifs->ParseInt(4, "mantissaLength");
  // PB efficient encoding fix according to 9.3.7.12.1 and 9.3.7.12.2
  if(mantissaLength) {
    int exponentLength=pBifs->ParseInt(3, "exponentLength");
    int mantissaSign=pBifs->ParseInt(1, "mantissaSign");
	int mantissa=pBifs->ParseInt(mantissaLength-1, "mantissa");
    int exponentSign=0;
    int exponent=0;
    unsigned char exp;


    if(exponentLength) {
		exponentSign=pBifs->ParseInt(1, "exponentSign");
		exponent=pBifs->ParseInt(exponentLength-1, "exponent");
		exp = (1-2*exponentSign)*((1<<(exponentLength-1))+exponent)+127;
	}
	else
		//FTRD - begin - 22/11/2001 
		exp = 127; // ML was 0
		//FTRD - begin - 22/11/2001 

	long val=mantissaSign<<31;
    val |= (exp & 0xff)<<23;
    val |= mantissa << 9; // PB was (23-mantissaLength) instead of 9
    value.l=val;
	
    return value.f;
  // PB end	
  }

  return 0.0;
}

