


#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

#include "define.h"
#include "BifaError.h"

#include "XmlParser.h"
#include <dom/DOM.hpp>

#include "BifaQuantizer.h"
#include "BinaryWriter.h"
#include "AnimationFrame.h"
#include "AnimationFrameData.h"
#include "AnimQP.h"
#include "AnimationField.h"







AnimationField::AnimationField(AnimationFrameData * AFD, BifaQuantizer* bquant) : m_animationFrameData(AFD), m_bifaquantizer(bquant)
{
	m_maskID = 0;
	m_ID = 0;
	m_animQP = NULL;
	m_nbMField = 1;
	m_encodedValue = NULL;
	m_quantValue = NULL;
	m_shiftedValue = NULL;
	m_nbaacbits = NULL;

	m_nbQuantValue=0;

	AFD->addAnimationField(this);

}

AnimationField::~AnimationField()
{
	if (m_encodedValue)
		delete [] m_encodedValue;
	if (m_quantValue)
		delete [] m_quantValue;
	if (m_shiftedValue)
		delete [] m_shiftedValue;
	if (m_animQP)
		delete m_animQP;
	if (m_nbaacbits)
		delete [] m_nbaacbits;


}

	
void AnimationField::setMaskID(int maskid)
{
	m_maskID = maskid;
}


int	AnimationField::getMaskID()
{
	return m_maskID;
}


void AnimationField::setID(int ID)
{
	m_ID = ID;
}

int	AnimationField::getID()
{
	return m_ID;
}


void AnimationField::setAnimQP(AnimQP* aqp)
{
	if (!isIntra())
	{
		BifaError::error(19);
		return;
	}
	if (m_bifaquantizer && m_bifaquantizer->getAnimQP())
	{
		if (aqp->getType() != m_bifaquantizer->getAnimQP()->getType())
		{
			BifaError::error(2020);
		}
	}

	m_animQP = aqp;
}

AnimQP*	AnimationField::getAnimQP()
{
	return m_animQP;
}


void AnimationField::setNbMField(int nbfield)
{
	m_nbMField = nbfield;
}

int	AnimationField::getNbMField()
{
	return m_nbMField;
}


int	AnimationField::quantizeFloat(float min, float max, int nbBits, float val)
{
	if(val<=min)
	{
		if (val<min)
		{
				char buffer[32];
				sprintf(buffer,"%.3f ==> return 0",val);
				BifaError::error(118,buffer);
		}
		return 0;
				
	} 
	if(val>=max)
	{
		if (val>max)
		{
			char buffer[32];
			sprintf(buffer,"%.3f ==> return %d",val,(1<<nbBits)-1);
			BifaError::error(118,buffer);
		}
		return (1<<nbBits)-1;
				
	}
	return (int) floor( ((val-min)/(max-min)*((1<<nbBits)-1)) + 0.5);

}


float AnimationField::unQuantizeFloat(float IMin, float IMax, int INbBits,int vq) 
{
	int nbBits = INbBits;
	float span = IMax - IMin;
	float epsilon = 1e-8f;
	float unQuantScale=(span <= epsilon )?0:(
				(nbBits==0)?span*.5f: (float) span/ (float)((1<<nbBits) - 1)
		);
    return IMin + unQuantScale * (float)vq;
}


int	AnimationField::quantizeInt(int min, int max, int nbBits, int val)
{
	if(val<=min) return 0;
	if(val>=max) return (1<<nbBits)-1;
	return (int) floor( ((val-min)/(max-min)*((1<<nbBits)-1)) + 0.5);

}


int AnimationField::unQuantizeInt(int IMin, int IMax, int INbBits,int vq) 
{
	int nbBits = INbBits;
	float span = (float)IMax - IMin;
	float epsilon = 1e-8f;
	float unQuantScale=(span <= epsilon )?0:(
				(nbBits==0)?span*.5f: (float) span/ (float)((1<<nbBits) - 1)
		);
    return (int)(IMin + unQuantScale * vq);
}


float* AnimationField :: unQuantizeRotation (int vq[3],int nBits, int direction, int orientation) 
{
	
	int j;
	
	float delta=1;
	float val[3];
	val[0] = unQuantizeFloat(0.0f,1.0f,nBits,vq[0]);
	val[1] = unQuantizeFloat(0.0f,1.0f,nBits,vq[1]);
	val[2] = unQuantizeFloat(0.0f,1.0f,nBits,vq[2]);

	float* vt = new float[4];
	float natComp[4];
	for (j=0;j<3;j++) {
		float v = (float)val[j];
		vt[j]= (float) tan (v*M_PI/4);
		delta+= vt[j]*vt[j];
	}
	delta = (float)(direction/sqrt(delta));
	
	natComp[(orientation)%4] = delta;
	for (j=0;j<3;j++) {
		natComp[(orientation+j+1)%4] = vt[j]*delta;
	}
	
	
	float alpha2=(float)acos(natComp[0]);
	if(fabs(alpha2)<1e-5) 
	{
		vt[0] =0.0f;
		vt[1] =1.0f;
		vt[2] =0.0f;
		vt[3] =0.0f;

	}
	else
	{
		vt[0] =natComp[1]/(float)sin(alpha2);
		vt[1] =natComp[2]/(float)sin(alpha2);
		vt[2] =natComp[3]/(float)sin(alpha2);
		vt[3] =2*alpha2;
	}
	return vt;
}


float* AnimationField :: unQuantizeNormal(int vq[2],int nBits, int direction, int orientation) 
{
	
	int j;
	
	float delta=1;
	float val[2];
	val[0] = unQuantizeFloat(0.0f,1.0f,nBits,vq[0]);
	val[1] = unQuantizeFloat(0.0f,1.0f,nBits,vq[1]);
	
	float* vt = new float[4];
	float natComp[3];
	for (j=0;j<2;j++) {
		float v = (float)val[j];
		vt[j]= (float) tan (v*M_PI/4);
		delta+= vt[j]*vt[j];
	}
	delta = (float)(direction/sqrt(delta));
	
	natComp[(orientation)%3] = delta;
	for (j=0;j<2;j++) {
		natComp[(orientation+j+1)%3] = vt[j]*delta;
	}
	
	
	for (int i=0; i<3; i++)
		vt[i] = natComp[i];
	return vt;
}

bool AnimationField::isIntra()
{
	if (m_animationFrameData && m_animationFrameData->getAnimationFrame())
		return(m_animationFrameData->getAnimationFrame()->getIsIntra());
	return false;
}


int AnimationField::getNbBits(int k)
{
	int j=1;
	while (k >= (1<<j) ) 
		j++;
	return j;
}


int* AnimationField::getQuantValue() {return m_quantValue;}
int	 AnimationField::getNbQuantValue() { return m_nbQuantValue;}
int* AnimationField::getEncodedValue() {return m_encodedValue;}

 

void AnimationField::write(FILE * file) 
{
	float begin = m_animationFrameData->getAnimationFrame()->getBegin();
	int nodeID = m_animationFrameData->getMaskID();
	
	fprintf (file,"Time %0.3f ",begin);
	if (isIntra())
		fprintf (file,"\tI ");
	else
		fprintf (file,"\tP ");
	
	fprintf(file,"NodeID (%d) ",nodeID);

}


void AnimationField::write(BinaryWriter * binWriter)
{
	if (isIntra())
	{
		if (m_animQP)
		{
			binWriter->write(true,1,"hasQP");
			m_animQP->writeBifa(binWriter);
		}
		else
			binWriter->write(false,1,"hasQP");
	}


}


void AnimationField::nbBitsToWrite(int &bits)
{
	
	if (isIntra())
	{
		if (m_animQP)
		{
			
			bits += 1;
			
			m_animQP->nbBitsToWrite(bits);
		}
		else
			
			bits += 1;
	}
}



void AnimationField::parse(DOM_Element* node)
{
	DOMString str;
	str = node->getAttribute(XMTATTR_ID);
	if (str.length())
	{
		int id;
		printf("\t\t\t\t id(%s)\n",str.transcode());
		sscanf(str.transcode(),"%d",&id);
		setID(id);
	}
	
	str = node->getAttribute(XMTATTR_MASKID);
	if (str.length())
	{
		int maskID;
		printf("\t\t\t\t maskID(%s)\n",str.transcode());
		sscanf(str.transcode(),"%d",&maskID);
		setMaskID(maskID);
	}								
	

}
