


#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

#include "define.h"
#include "BifaError.h"

#include "XmlParser.h"
#include <dom/DOM.hpp>

#include "BinaryWriter.h"
#include "FrameRate.h"
#include "SkipFrame.h"
#include "AnimationFrameData.h"
#include "AnimationFrame.h"







AnimationFrame::AnimationFrame(float begin) : m_begin(begin)
{
	m_animationStarCode = -1;
	m_isIntra = true;
	m_skipFrame = NULL;
	m_frameRate = NULL;
	m_timecode = 0;
	
	
	memset(m_tabAFD,NULL,sizeof(AnimationFrameData*)*MAXOBJECT);
	m_nbAFD =0;
}
	

AnimationFrame::~AnimationFrame()
{
	if (m_tabAFD)
		for (int i=0;i< m_nbAFD; i++)
			delete m_tabAFD[i];
	if (m_skipFrame)
		delete m_skipFrame;
	if (m_frameRate)
		delete m_frameRate;
}


float AnimationFrame::getBegin()
{
	return m_begin;
}


void AnimationFrame::setAnimationStartCode(int animstartcode)
{
	m_animationStarCode = animstartcode;
}


int AnimationFrame::getAnimationStartCode()
{
	return m_animationStarCode;
}


void AnimationFrame::setIsIntra(bool isintra)
{
	m_isIntra = isintra;
}


bool AnimationFrame::getIsIntra()
{
	return m_isIntra;
}


void AnimationFrame::setTimeCode(int timecode)
{
	m_timecode = timecode;
}


int AnimationFrame::getTimeCode()
{
	return m_timecode;
}


int AnimationFrame::addAnimationFrameData(AnimationFrameData * afd)
{
	
	if (m_nbAFD > m_numNode)
	{
		char buffer[16];
		sprintf(buffer,"begin : %.3f",m_begin);
		BifaError::error(2012,buffer);
		return m_nbAFD;
	}
	m_tabAFD[m_nbAFD] = afd;
	return m_nbAFD++;
}


AnimationFrameData* AnimationFrame::getAnimationFrameData(int i)
{
	if (i>=0 && i<m_nbAFD)
		return m_tabAFD[i];

	
	return NULL;
}


int AnimationFrame::getNbAnimationFrameData()
{
	return m_nbAFD;
}


void AnimationFrame::setSkipFrame(SkipFrame* skipframe)
{
	m_skipFrame = skipframe;
}


SkipFrame* AnimationFrame::getSkipFrame()
{
	return m_skipFrame;
}


void AnimationFrame::setFrameRate(FrameRate* framerate)
{
	m_frameRate = framerate;
}


FrameRate* AnimationFrame::getFrameRate()
{
	return m_frameRate;
}


void AnimationFrame::setNumNode(int num)
{
	m_numNode = num;
}


int AnimationFrame::getNumNode()
{
	return m_numNode;
}


void AnimationFrame::write(FILE *file)
{
	
	for (int i=0;i<m_nbAFD;i++)
	{
		m_tabAFD[i]->write(file);
		fprintf(file,"--------------------\n");
	}
}


void AnimationFrame::write(BinaryWriter* binWriter)
{
	int bits =0;
	
	nbBitsToWrite(bits);
	
	bits=(int)ceil((double)bits/8.0);

	

	char buffer[64];
	static int m_numFrame =0;
	sprintf(buffer,"# BIFSAnim Frame %d",m_numFrame++);
	binWriter->log(buffer);
	
	binWriter->write((unsigned int)(m_begin*(1000.0f)),32);
	
	binWriter->write((unsigned int)bits,32);


	if (m_animationStarCode != -1)
		binWriter->write((unsigned int)m_animationStarCode,32,"AnimationStartCode");
	binWriter->write(m_isIntra,1,"isIntra");
	
	
	for (int i=0; i< m_numNode; i++)
	{
		bool writeFlag =false;
		for (int j=0; j<m_nbAFD;j++)
		{
			if (i == m_tabAFD[j]->getMaskID())
				writeFlag =true;
		}
		binWriter->write(writeFlag,1,"isActive");
	}

	if (m_isIntra)
	{
		if (m_frameRate)
		{
			binWriter->write(true,1,"isFrameRate");
			m_frameRate->write(binWriter);
		}
		else
			binWriter->write(false,1,"isFrameRate");
		if (m_timecode)
		{
			binWriter->write(true,1,"isTimeCode");
			binWriter->write((unsigned int)m_timecode,18);
		}
		else
			binWriter->write(false,1,"isTimeCode");
	}
	if (m_skipFrame)
	{
		binWriter->write(true,1,"hasSkipFrames");
		m_skipFrame->write(binWriter);
	}
	else
		binWriter->write(false,1,"hasSkipFrames");

	
	for (i=0; i< m_numNode; i++)
	{
		for (int j=0; j<m_nbAFD;j++)
		{		
			if (m_tabAFD[j] && (i == m_tabAFD[j]->getMaskID()))
				m_tabAFD[j]->write(binWriter);
		}
	}

	
	binWriter->flush();
	
}


void AnimationFrame::nbBitsToWrite(int &bits)
{
	if (m_animationStarCode != -1)
		
		bits += 32;
	
	bits+=1;
	
	for (int i=0; i< m_numNode; i++)
	{
		
		bits += 1;
	}

	if (m_isIntra)
	{
		if (m_frameRate)
		{
			
			bits+=1;
			
			m_frameRate->nbBitsToWrite(bits);
		}
		else
			
			bits +=1;
		if (m_timecode)
		{
			
			bits +=1;
			
			bits+=18;
		}
		else
			
			bits+=1;
	}
	if (m_skipFrame)
	{
		
		bits+=1;
		
		m_skipFrame->nbBitsToWrite(bits);
	}
	else
		
		bits+=1;


	
	for (i=0; i< m_numNode; i++)
	{
		for (int j=0; j<m_nbAFD;j++)
		{		
			if (m_tabAFD[j] && (i == m_tabAFD[j]->getMaskID()))
				m_tabAFD[j]->nbBitsToWrite(bits);

		}
	}
	
}


void AnimationFrame::parse(DOM_Element* node)
{
	DOMString str;
	
	str = node->getAttribute(XMTATTR_ANIMATIONSTARTCODE);
	if (str.length())
	{
		TRACEP("\t\t animationStartCode(%s)\n",str.transcode());
		int animstartcode;
		sscanf(str.transcode(),"%d",&animstartcode);
		setAnimationStartCode(animstartcode);
	}
	
	str = node->getAttribute(XMTATTR_ISINTRA);
	if (str.length())
	{
		TRACEP("\t\t isIntra(%s)\n",str.transcode());
		if (str.equals("true"))
			setIsIntra(true);
		else
			setIsIntra(false);
	}
	
	str = node->getAttribute(XMTATTR_TIMECODE);
	if (str.length())
	{
		TRACEP("\t\t TimeCode(%s)\n",str.transcode());
		int timeCode;
		sscanf(str.transcode(),"%d",&timeCode);
		setTimeCode(timeCode);
	}
	
	str = node->getAttribute(XMTATTR_SKIPFRAME);
	if (str.length())
	{
		TRACEP("\t\t SkipFrame(%s)\n",str.transcode());
		int skipframe;
		sscanf(str.transcode(),"%d",&skipframe);
		SkipFrame * tmpSkipFrame = new SkipFrame();
		tmpSkipFrame->setNbFrameToSkip(skipframe);
		setSkipFrame(tmpSkipFrame);
		
	}
	
}
