


#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "define.h"
#include "BifaError.h"

#include "XmlParser.h"
#include <dom/DOM.hpp>

#include "BinaryWriter.h"
#include "AnimationFrame.h"
#include "AnimationField.h"
#include "AnimationFrameData.h"





AnimationFrameData::AnimationFrameData( AnimationFrame * AF) : m_animationFrame(AF)
{
	m_maskID = 0;
	m_atNode = 0;
	
	memset(m_tabAFD, NULL, sizeof(AnimationField*)*MAXOBJECT);
	
	m_nbAFD =0;
	
	AF->addAnimationFrameData(this);
}


AnimationFrameData::~AnimationFrameData()
{
	if (m_tabAFD)
		for (int i=0; i<m_nbAFD;i++)
			delete m_tabAFD[i];
}


AnimationFrame* AnimationFrameData::getAnimationFrame()
{
	return m_animationFrame;
}


void AnimationFrameData::setMaskID(int maskid)
{
	
	if ( maskid >= m_animationFrame->getNumNode())
	{
		char buffer[16];
		sprintf(buffer,"maskID : %d",maskid);
		BifaError::error(2013,buffer);
		return;
	}
	
	for (int i=0; i<m_animationFrame->getNbAnimationFrameData();i++)
	{
		if ( (this != m_animationFrame->getAnimationFrameData(i))
			&& maskid == m_animationFrame->getAnimationFrameData(i)->getMaskID())
		{
			char buffer[16];
			sprintf(buffer,"maskID : %d",maskid);
			BifaError::error(2014,buffer);
			return;
		}
	}
	m_maskID = maskid;
}


int	AnimationFrameData::getMaskID()
{
	return m_maskID;
}


void AnimationFrameData::setAtNode(int atnod)
{
	m_atNode = atnod;
}


int	AnimationFrameData::getAtNode()
{
	return m_atNode;
}


int	AnimationFrameData::addAnimationField(AnimationField* afd)
{
	m_tabAFD[m_nbAFD]=afd;
	return m_nbAFD++;
}


AnimationField*	AnimationFrameData::getAnimationField(int i)
{
	if (i>=0 && i< m_nbAFD)
		return m_tabAFD[i];

	
	return NULL;
}


int	AnimationFrameData::getNbAnimationField()
{
	return m_nbAFD;
}


void AnimationFrameData::write(FILE * file)
{
	for (int i=0; i<m_nbAFD; i++)
		m_tabAFD[i]->write(file);
}


void AnimationFrameData::write(BinaryWriter * binWriter)
{
	
	for (int i=0; i<8; i++)
	{
		for (int j=0; j<m_nbAFD; j++)
		{
			if (i == m_tabAFD[j]->getMaskID())
				m_tabAFD[j]->write(binWriter);
		}
	}
	
}


void AnimationFrameData::nbBitsToWrite(int &bits)
{

	
	for (int i=0; i<8; i++)
	{
		for (int j=0; j<m_nbAFD; j++)
		{
			if (i == m_tabAFD[j]->getMaskID())
				m_tabAFD[j]->nbBitsToWrite(bits);
		}
	}
}


void AnimationFrameData::parse(DOM_Element* node)
{
	DOMString str;

	
	str = node->getAttribute(XMTATTR_MASKID);
	if (str.length())
	{
		TRACEP("\t\t\t maskID(%s)\n",str.transcode());
		int maskID;
		sscanf(str.transcode(),"%d",&maskID);
		setMaskID(maskID);
	}
	
	
	str = node->getAttribute(XMTATTR_ATNODE);
	if (str.length())
	{
		TRACEP("\t\t\t atNode(%s)\n",str.transcode());
		int atNode;
		sscanf(str.transcode(),"%d",&atNode);
		setAtNode(atNode);
	}
}
