


#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "define.h"

#include "ArithModel.h"
#include "ArithEncoder.h"
#include "AnimQP.h"
#include "AnimationField.h"
#include "Position3DAnimationField.h" 

#include "BifaQuantizer.h"





ArithEncoder BifaQuantizer::m_ArithEncoder;


BifaQuantizer::BifaQuantizer( int nodeID, int ID, int nbAnimationFrame):  m_nodeID(nodeID),  m_ID(ID), m_nbAnimationFrame(nbAnimationFrame)
{
	m_tabArithModel = NULL;
	
	m_nbArithModel	= 0;
	m_initialAnimQP = NULL;
	m_animQP		= NULL;
	m_type			= 0;
	m_indexList		= NULL;
	m_nbIndex		= 0;
	m_isTotal		= true;
	m_randomAccess	= false;
	m_tabAFD		= new AnimationField*[m_nbAnimationFrame];
	memset(m_tabAFD,NULL,sizeof(AnimationField*)*m_nbAnimationFrame);
	m_nbAFD			=0;

}


BifaQuantizer::~BifaQuantizer()
{
	if (m_tabAFD)
		delete [] m_tabAFD;
	for (int i=0; i<m_nbArithModel; i++)
		delete m_tabArithModel[i];

	if (m_tabArithModel)
		free( m_tabArithModel);
}


int	BifaQuantizer::getID()
{
	return m_ID;
}

int BifaQuantizer::getNodeID()
{
	return m_nodeID;
}


void BifaQuantizer::setInitialAnimQp(AnimQP* aqp)
{
	m_initialAnimQP = aqp;
	
	m_animQP =m_initialAnimQP; 
}


void BifaQuantizer::setAnimQp(AnimQP* aqp)
{
	
	m_animQP = aqp;
}


void BifaQuantizer::setType(int type)
{
	m_type = type;
}


void BifaQuantizer::setIndexList(int* indexlist, int nbindex)
{
	m_indexList = indexlist;
	m_nbIndex = nbindex;
}


void BifaQuantizer::setIsTotal(bool istotal)
{
	m_isTotal = istotal;
}


void BifaQuantizer::setRandomAccess(bool randomaccess)
{
	m_randomAccess = randomaccess;
}


int	BifaQuantizer::addAnimationField(AnimationField* afd)
{
	m_tabAFD[m_nbAFD] = afd;
	return m_nbAFD++;
}


AnimationField*	BifaQuantizer::getAnimationField(int i)
{
	if (i>=0 && i<m_nbAFD)
		return m_tabAFD[i];
	
	
	return NULL;
}


int BifaQuantizer::getNbAnimationField()
{
	return m_nbAFD;
}
	
int	BifaQuantizer::getAnimationType()
{
	if (m_animQP)
		return(m_animQP->getType());
	

	return 0;
}

AnimQP*	BifaQuantizer::getAnimQP()
{
	return m_animQP;
}



void BifaQuantizer::quantize()
{
	
	int indDebut =0;
	int indFin=0;
	
	
	bool stopFlag=false;
	
	int first =true;	

	while (!stopFlag && (indDebut < m_nbAFD))
	{
		if (!m_tabAFD[indDebut]->isIntra())
		{
			
			stopFlag = true;
			break;
		}
		indFin = indDebut+1;
		
		while(indFin<m_nbAFD && !m_tabAFD[indFin]->isIntra())
			indFin++;

		
		int * minValue=NULL;

		
		if (m_tabArithModel)
		{
			delete [] m_tabArithModel;
			m_tabArithModel =NULL;
		}
		
		if (m_tabAFD[indDebut]->getAnimQP())
		{
			
			m_animQP = m_tabAFD[indDebut]->getAnimQP();
			m_animQP->setType(m_initialAnimQP->getType());
			
			m_animQP->setPmin(m_initialAnimQP->getPmin(),m_initialAnimQP->getNbPmin());
			if ( (m_type != 6) && (m_type!=9) && (m_type!=10) && (m_type!=13) )
			{
				if (!m_animQP->getLowerBound())
					m_animQP->setLowerBound(m_initialAnimQP->getLowerBound(),m_initialAnimQP->getNbBound());
				if (!m_animQP->getUpperBound())
					m_animQP->setUpperBound(m_initialAnimQP->getUpperBound(),m_initialAnimQP->getNbBound());
			}
			if (!m_animQP->getInbBits())
				m_animQP->setInbBits(m_initialAnimQP->getInbBits());
			if (!m_animQP->getPnbBits())
				m_animQP->setPnbBits(m_initialAnimQP->getPnbBits());
			if (m_type == 13)
			{
				if (!m_animQP->getIminInt())
					m_animQP->setIminInt(m_initialAnimQP->getIminInt());
			}

		}

		
		m_tabAFD[indDebut]->quantizeValue(this);
		
		
		for (int i=indDebut+1; i<(indFin); i++)
		{
			
			m_tabAFD[i]->quantizeValue(this);
			
			m_tabAFD[i]->deltaValue(m_tabAFD[i-1]);
			
			m_tabAFD[i]->calcMinValue(&minValue);
			if (first)
			{
				
				m_animQP->setPmin(minValue,0);
				first=false;
			}

			
		}
		
		
		if (minValue)
		{
			m_animQP->updatePmin(minValue);
			m_initialAnimQP->updatePmin(minValue);
		}
	
		
		
		indDebut = indFin++;	
		
		
		if (m_randomAccess)
			m_animQP = m_initialAnimQP;
		
		delete [] minValue;

	}
	
	
	m_animQP = m_initialAnimQP;
	
	
	
	for (int j=0; j<(m_nbAFD); j++)
	{
		if (m_tabAFD[j]->isIntra() && m_tabAFD[j]->getAnimQP())
			m_tabAFD[j]->getAnimQP()->setPmin(m_initialAnimQP->getPmin(),0);
	}
	
	
	for (j=0; j<(m_nbAFD); j++)
	{
		
		if (m_tabAFD[j]->isIntra() && m_tabAFD[j]->getAnimQP())
			m_animQP=m_tabAFD[j]->getAnimQP();
		
		m_tabAFD[j]->shiftValue(this,m_animQP->getPmin());
		
		
		if (m_randomAccess)
			m_animQP = m_initialAnimQP;
	}
	
	
	m_animQP = m_initialAnimQP;

}


void BifaQuantizer::encode(int i,bool reset, bool flush)
{
	
	if (!m_tabAFD[i])
		return;
	
	if (!m_tabAFD[i]->isIntra())
	{
		
		if (reset)
			m_ArithEncoder.reset();
		m_tabAFD[i]->encodeValue(this,flush);
	}
	else
	{	
		
		int nbbits;
		
		if (m_tabAFD[i]->getAnimQP())
		{
			nbbits = m_tabAFD[i]->getAnimQP()->getPnbBits();
			for (int imod=0; imod<m_nbArithModel;imod++)
				m_tabArithModel[imod]->init(1<<nbbits);
		}
		
		else if (m_randomAccess)
		{
			nbbits = m_initialAnimQP->getPnbBits();
			for (int imod=0; imod<m_nbArithModel;imod++)
				m_tabArithModel[imod]->init(1<<nbbits);
		}
	}
}


int	BifaQuantizer::encodeAndUpdate(int val,int indexModel, int nbModel, int& bits, bool flush)
{
	
	if (!m_tabArithModel)
	{
		m_tabArithModel = (ArithModel**)malloc(sizeof(ArithModel*)*nbModel);
		m_nbArithModel = nbModel;

		for (int i=0; i<m_nbArithModel; i++)
		{
			m_tabArithModel[i] =new ArithModel;
			m_tabArithModel[i]->init(1<<m_animQP->getPnbBits());
		}
	}

	
	int	retval =0;
	
	m_ArithEncoder.encodeAndUpdate(val,*m_tabArithModel[indexModel],"encode",retval);
	
	
	if (flush)
		m_ArithEncoder.flush();
	
	m_ArithEncoder.getLastValue(retval,bits);
	
	return retval;
}





