


#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "define.h"
#include "BifaError.h"
#include "XmlParser.h"
#include <dom/DOM.hpp>

#include "BinaryWriter.h"
#include "AnimationMask.h"
#include "BifsConfig.h"






BifsConfig::BifsConfig()
{
	m_nodeIDbits = 0;
	m_routeIDbits = 0;

	
	m_isCommandStream = false;
	m_randomAcces = false;

	
	memset(m_tabAM,NULL,MAXOBJECT);
	
	m_nbAM = 0;

}

BifsConfig::~BifsConfig()
{
	for (int i=0; i<m_nbAM;i++)
		if (m_tabAM)
			delete m_tabAM[i];
	
}

void BifsConfig::setNodeIDbits( int bits)
{
	m_nodeIDbits = bits;
}

int BifsConfig::getNodeIDbits()
{
	return m_nodeIDbits;
}

void BifsConfig::setRouteIDbits( int bits)
{
	m_routeIDbits = bits;
}

int BifsConfig::getRouteIDbits()
{
	return m_routeIDbits;
}


void BifsConfig::setProtoIDbits( int bits)
{
	m_protoIDbits = bits;
}

int BifsConfig::getProtoIDbits()
{
	return m_protoIDbits;
}


void BifsConfig::setIsCommandStream(bool commandStream)
{
	m_isCommandStream = commandStream;
}

bool BifsConfig::getIsCommandStream()
{
	return m_isCommandStream;
}

void BifsConfig::setRandomAccess(bool randomAccess)
{
	m_randomAcces = randomAccess;
}

bool BifsConfig::getRandomAccess()
{
	return m_randomAcces;
}


int BifsConfig::addAnimationMask(AnimationMask * AM)
{
	
	if (m_nbAM>0)
	{
		BifaError::error(2000);
		return m_nbAM;
	}
	m_tabAM[m_nbAM] = AM;
	return m_nbAM++;
}


AnimationMask* BifsConfig::getAnimationMask(int i)
{
	if (i>=0 && i<m_nbAM)
		return m_tabAM[i];
	
	return NULL;
}


int BifsConfig::getNbAnimationMask()
{
	return m_nbAM;
}

void BifsConfig::setUse3DMeshEncoding(bool use)
{
	m_use3DMeshEncoding = use;
}

bool BifsConfig::getUse3DMeshEncoding()
{
	return m_use3DMeshEncoding;
}

void BifsConfig::setUsePredictiveField(bool use)
{
	m_usePredictiveField = use;
}


bool BifsConfig::getUsePredictiveField()
{
	return m_usePredictiveField;
}


void BifsConfig::write(BinaryWriter * binWriter)
{
	
	for (int i=0; i< m_nbAM; i++)
	{
		m_tabAM[i]->write(binWriter);
	}
}


void BifsConfig::parse(DOM_Element* node)
{

	DOMString str;
	
	str = node->DOM_Element::getAttribute(XMTATTR_NODEIDBITS);
	if (str.length())
	{
		TRACEP("\t NodeIDbits(%s)\n",str.transcode());
		int node;
		sscanf(str.transcode(),"%d",&node);
		setNodeIDbits(node);
	}
	
	str = node->getAttribute(XMTATTR_ROUTEIDBITS);
	if (str.length())
	{
		TRACEP("\t RouteIDbits(%s)\n",str.transcode());
		int route;
		sscanf(str.transcode(),"%d",&route);
		setRouteIDbits(route);
	}
	
	str = node->getAttribute(XMTATTR_PROTOIDBITS);
	if (str.length())
	{
		TRACEP("\t PROTOIDbits(%s)\n",str.transcode());
		int proto;
		sscanf(str.transcode(),"%d",&proto);
		setProtoIDbits(proto);
	}
	
	str = node->getAttribute(XMTATTR_RANDOMACCESS);
	if (str.length())
	{
		TRACEP("\t randomAccess(%s)\n",str.transcode());
		if (str.equals("true"))
			setRandomAccess(true);
		else
			setRandomAccess(false);
	}
	
	str = node->getAttribute(XMTATTR_USE3DMESHCODING);
	if (str.length())
	{
		TRACEP("\t use3dMeshEncoding(%s)\n",str.transcode());
		if (str.equals("true"))
			setUse3DMeshEncoding(true);
		else
			setUse3DMeshEncoding(false);
	}
	
	str = node->getAttribute(XMTATTR_USEPREDICTIVEMFFIELD);
	if (str.length())
	{
		TRACEP("\t usePredictiveMFField(%s)\n",str.transcode());
		if (str.equals("true"))
			setUsePredictiveField(true);
		else
			setUsePredictiveField(false);
	}
	
	
	
}
