


#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

#include "define.h"

#include "BinaryWriter.h"






BinaryWriter::BinaryWriter(FILE* file,FILE* logfile,int logLevel) : m_file(file),m_logfile(logfile),m_logLevel(logLevel)
{
	m_buffer = 0;
	m_nbBits =0;
	m_nbLogBits =0;
	

}


BinaryWriter::~BinaryWriter()
{
	
}


void BinaryWriter::write(unsigned int val, int nbBits, char* szDesc, char* szComment,int logLevel)
{
	
	if ((m_nbBits+nbBits)<=8)
	{
		m_buffer<<=nbBits;
		m_buffer |= val;
		m_nbBits+=nbBits;
	
		
		if (m_nbBits ==8)
		{
			fwrite((void*)&m_buffer,sizeof(m_buffer),1,m_file);
			m_buffer = 0;
			m_nbBits =0;
		}
	}
	
	else
	{
		
		int mank = 8-m_nbBits; 
		int val2 = val; 

		m_buffer<<=mank; 
		val2>>=(nbBits-mank); 
		int tmp=0;
		for (int i=0; i<mank;i++)
		{
			tmp+=(1<<i);
		}
		val2&=tmp;
		
		m_buffer |= val2;
		
		fwrite((void*)&m_buffer,sizeof(m_buffer),1,m_file); 
		m_buffer = 0;
		m_nbBits =0;
	
		
		write(val,nbBits-mank); 
	}

	
	if (m_logfile && szDesc)
		log(val,nbBits,szDesc,szComment,logLevel);

}


void BinaryWriter::write(bool val, int nbBits, char* szDesc, char* szComment,int logLevel)
{
	
	if (m_nbBits <8)
	{
		m_buffer<<=1;
		if (val)
			m_buffer |= 1;
		else
			m_buffer |= 0;

		m_nbBits++;
	}
	
	if (m_nbBits ==8)
	{
		fwrite((void*)&m_buffer,sizeof(m_buffer),1,m_file);
		m_buffer = 0;
		m_nbBits =0;
	}
	
	if (m_logfile && szDesc)
		log(val,1,szDesc,szComment,logLevel);
}


void BinaryWriter::write(float val, char* szDesc, char* szComment,int logLevel)
{
	
	unsigned int f;
	memcpy(&f,&val,sizeof(unsigned int));
	write(f,32);
	
	if (m_logfile && szDesc)
		log(f,32,szDesc,szComment,logLevel);

}

		

void BinaryWriter::writeEncoded(unsigned int val, int nbBits, char* szDesc, char* szComment,int logLevel)
{
#ifdef LOG_AACVAL
	write(val,nbBits,szDesc,szComment);
#else 
	for (int i=nbBits-1; i>=0;i--)
	{
		if ( (val>>i) & 0x1)
			write(true,1,"AAC","",1);
		else
			write(false,1,"AAC","",1);
	}
#endif 
}


void BinaryWriter::writeEfficientFloat(float val, char* szDesc, char* szComment,int logLevel)
{
#ifdef WIN32
	int bitcount=0;

	if(val==0) 
	{
		write((unsigned int)0,4,"mantissaLength");
		bitcount+=4;
		
		unsigned int f;
		memcpy(&f,&val,sizeof(unsigned int));
		if (m_logfile && szDesc)
			log((unsigned int)f,bitcount,szDesc,szComment,logLevel);
		return;
	}

	long lval;
	memcpy(&lval,&val,sizeof(long));

	int exponentLength = 7;
	int mantissaLength = 15;

	int mantissaSign = (lval>>31) & 0x1;
	int exponent= (lval>>23) & 0xFF;

	
	exponent-=127;
	int exponentSign = exponent>0?0:1;
	if (exponentSign)
		exponent = -exponent;
	
	while (!exponent<<(exponentLength) && exponentLength)
		exponentLength--;

	unsigned int mantissa=lval & 0x007FFFFF;
	
	
	mantissa>>=9;

	write((unsigned int)mantissaLength,4,"mantissaLength",NULL,logLevel);
	bitcount+=4;
	if(mantissaLength) 
	{
		write((unsigned int)exponentLength,3,"exponentLength",NULL,logLevel);
		bitcount+=3;
		write((unsigned int)mantissaSign,1,"mantissaSign",NULL,logLevel);
		bitcount+=1;
		write((unsigned int)mantissa, mantissaLength-1,"mantissa",NULL,logLevel);
		bitcount+=(mantissaLength-1);
		if(exponentLength) 
		{
			write((unsigned int)exponentSign,1,"exponentSign",NULL,logLevel);
			bitcount+=1;
			write((unsigned int)exponent,exponentLength-1,"exponent",NULL,logLevel);
			bitcount+=(exponentLength-1);
		}
	}

	unsigned int f;
	memcpy(&f,&val,sizeof(unsigned int));
	if (m_logfile && szDesc)
		log((unsigned int)f,bitcount,szDesc,szComment);

#endif 

}


int BinaryWriter::nbBitsEfficientFloat(float val)
{
#ifdef WIN32
	int bitcount =0;
	long lval;
	memcpy(&lval,&val,sizeof(long));

	int exponentLength = 7;
	int mantissaLength = 15;

	int mantissaSign = (lval>>31) & 0x1;
	int exponent= (lval>>23) & 0xFF;
	exponent-=127;
	exponent-=(1<<(exponentLength-1));
	
	int exponentSign = exponent>0?0:1;
	if (exponentSign)
		exponent = -exponent;
	unsigned int mantissa=lval & 0x007FFFFF;

	
	exponent>>=1;
	mantissa>>=9;

	
	bitcount+=4;
	if(mantissaLength) 
	{
		
		bitcount+=3;
		
		bitcount+=1;
		
		bitcount+=(mantissaLength-1);
		if(exponentLength) 
		{
			
			bitcount+=1;
			
			bitcount+=(exponentLength-1);
		}
	}

	return bitcount;

#endif 

	
}


void BinaryWriter::flush()
{
	m_buffer<<=(8-m_nbBits);
	if (m_nbBits)
		fwrite((void*)&m_buffer,sizeof(m_buffer),1,m_file); 

	m_buffer = 0;
	m_nbBits=0;
	m_nbLogBits =0;
}



void BinaryWriter::log(int nValue, int nBits, char* szDesc, char* szComment,int logLevel)
{
	if (logLevel>m_logLevel)
		return;

	m_nbLogBits += nBits;
	fprintf (m_logfile,"BIFSAnim <>  (%3d,%10d)   %-40s%9X %s\n",
		nBits,
		m_nbLogBits,
		szDesc,
		nValue,
		szComment ? szComment : ""
		);
}


void BinaryWriter::log(char* szComment,int logLevel)
{
	if (logLevel>m_logLevel)
		return;

	fprintf (m_logfile,"%s\n",szComment);
}