


#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "define.h"
#include "BifaError.h"

#include "XmlParser.h"
#include <dom/DOM.hpp>

#include "BinaryWriter.h"
#include "AnimQP.h"
#include "BifaQuantizer.h"

#include "ColorAnimationField.h"






ColorAnimationField::ColorAnimationField(AnimationFrameData* AFD, BifaQuantizer* bquant) : AnimationField(AFD,bquant)
{
	m_value = NULL;
}


ColorAnimationField::~ColorAnimationField()
{
	if (m_value)
		delete [] m_value;
}


void ColorAnimationField::setValue(float * values, int nbValue)
{
	
	if ((nbValue%3) != 0 )
	{
		char buffer[32];
		sprintf(buffer,"type : %d nbValue : %d",4,nbValue);
		BifaError::error(2017,buffer);
		return;
	}
	
	if (m_bifaquantizer && m_bifaquantizer->getAnimQP()
		&& m_bifaquantizer->getAnimQP()->getLowerBound()
		&& m_bifaquantizer->getAnimQP()->getUpperBound() )
	{
		float min = m_bifaquantizer->getAnimQP()->getLowerBound()[0];
		float max = m_bifaquantizer->getAnimQP()->getUpperBound()[0];
		
		for (int i=0; i<nbValue;i++)
		{ 
			if (values[i]<min || values[i]>max)
			{
				char buffer[16];
				sprintf(buffer,"type :%d : %.3f",4,values[i]);
				BifaError::error(2018,buffer);
				return;
			}
		}
	}
	
	m_nbMField = nbValue/3;


	m_value = new float[nbValue];
	memcpy(m_value,values,sizeof(float) * nbValue);
	m_nbValue = nbValue;

	TRACEP("\t\t\t\t");
	for (int i=0; i<m_nbValue; i++)
		TRACEP("%f ",m_value[i]);
	TRACEP("\n");
}


float* ColorAnimationField::getValue()
{
	return m_value;
}


int ColorAnimationField::getNbValue()
{
	return m_nbValue;
}


void ColorAnimationField::quantizeValue(BifaQuantizer * pBifaQuant)
{
	float *minVal =NULL;
	float *maxVal =NULL;
	
	m_nbBits = pBifaQuant->getAnimQP()->getInbBits();
	if (pBifaQuant->getAnimQP()->getUseDefault())
	{
		float bound[1] = { 0.0f };
		
		pBifaQuant->getAnimQP()->setLowerBound(bound,1);
		bound[0]=1.0f;
		pBifaQuant->getAnimQP()->setUpperBound(bound,1);
	}

	minVal = pBifaQuant->getAnimQP()->getLowerBound();
	maxVal = pBifaQuant->getAnimQP()->getUpperBound();
		

	m_quantValue = new int[m_nbValue];
	m_nbQuantValue = m_nbValue;
	for (int i=0; i<m_nbMField;i++)
	{
		
		m_quantValue[3*i] = quantizeFloat(minVal[0],maxVal[0],m_nbBits,m_value[3*i]);
		m_quantValue[3*i+1] = quantizeFloat(minVal[0],maxVal[0],m_nbBits,m_value[3*i+1]);
		m_quantValue[3*i+2] = quantizeFloat(minVal[0],maxVal[0],m_nbBits,m_value[3*i+2]);
		
		
		m_value[3*i] = unQuantizeFloat(minVal[0],maxVal[0],m_nbBits,m_quantValue[3*i]);
		m_value[3*i+1] = unQuantizeFloat(minVal[0],maxVal[0],m_nbBits,m_quantValue[3*i+1]);
		m_value[3*i+2] = unQuantizeFloat(minVal[0],maxVal[0],m_nbBits,m_quantValue[3*i+2]);

	}
	
}


void ColorAnimationField::deltaValue(AnimationField * pPrec)
{
	if (!m_shiftedValue)
		m_shiftedValue = new int[m_nbQuantValue];
	
	m_shiftedValue[0] = m_quantValue[0]-((ColorAnimationField*)pPrec)->getQuantValue()[3*(m_nbMField-1)];
	m_shiftedValue[1] = m_quantValue[1]-((ColorAnimationField*)pPrec)->getQuantValue()[3*(m_nbMField-1)+1];
	m_shiftedValue[2] = m_quantValue[2]-((ColorAnimationField*)pPrec)->getQuantValue()[3*(m_nbMField-1)+2];

	for (int i=1; i<m_nbMField; i++)
	{
		
		m_shiftedValue[3*i] = m_quantValue[3*i]-m_quantValue[3*(i-1)];
		m_shiftedValue[3*i+1] = m_quantValue[3*i+1]-m_quantValue[3*(i-1)+1];
		m_shiftedValue[3*i+2] = m_quantValue[3*i+2]-m_quantValue[3*(i-1)+2];
	}
}


void ColorAnimationField::calcMinValue(int ** minValue)
{
	
	if (!*minValue)
	{
		
		*minValue = new int[1];
		
		
		for (int i=0; i<m_nbMField;i++)
		{
			(*minValue)[0] = m_shiftedValue[3*i];
			(*minValue)[0] = m_shiftedValue[3*i+1]<(*minValue)[0]?m_shiftedValue[3*i+1]:(*minValue)[0];
			(*minValue)[0] = m_shiftedValue[3*i+2]<(*minValue)[0]?m_shiftedValue[3*i+2]:(*minValue)[0];
		}
	}
	else
	{
		
		for (int i=0; i<m_nbMField;i++)
		{
			(*minValue)[0] = m_shiftedValue[3*i]<(*minValue)[0]?m_shiftedValue[3*i]:(*minValue)[0];
			(*minValue)[0] = m_shiftedValue[3*i+1]<(*minValue)[0]?m_shiftedValue[3*i+1]:(*minValue)[0];
			(*minValue)[0] = m_shiftedValue[3*i+2]<(*minValue)[0]?m_shiftedValue[3*i+2]:(*minValue)[0];
		}
	}
}


void ColorAnimationField::shiftValue(BifaQuantizer * pBifaQuant, int* minvalue)
{
	if (!m_shiftedValue)
		m_shiftedValue = new int[m_nbQuantValue];
	
	
	for (int i=0; i<m_nbMField;i++)
	{
		m_shiftedValue[3*i] =m_shiftedValue[3*i]-minvalue[0];
		m_shiftedValue[3*i+1] =m_shiftedValue[3*i+1]-minvalue[0];
		m_shiftedValue[3*i+2] =m_shiftedValue[3*i+2]-minvalue[0];
		
		if (pBifaQuant->getAnimQP() 
			&& (pBifaQuant->getAnimQP()->getPnbBits() < getNbBits(m_shiftedValue[3*i])) )
		{
			char buffer[16];
			sprintf(buffer,"%d ==> %d",pBifaQuant->getAnimQP()->getPnbBits(),getNbBits(m_shiftedValue[3*i]));
			BifaError::error(1124,buffer);
			pBifaQuant->getAnimQP()->setPnbBits(getNbBits(m_shiftedValue[3*i]));
			return;
		}
		if (pBifaQuant->getAnimQP() 
			&& (pBifaQuant->getAnimQP()->getPnbBits() < getNbBits(m_shiftedValue[3*i+1])) )
		{
			char buffer[16];
			sprintf(buffer,"%d ==> %d",pBifaQuant->getAnimQP()->getPnbBits(),getNbBits(m_shiftedValue[3*i+1]));
			BifaError::error(1124,buffer);
			pBifaQuant->getAnimQP()->setPnbBits(getNbBits(m_shiftedValue[3*i+1]));
			return;
		}
		if (pBifaQuant->getAnimQP() 
			&& (pBifaQuant->getAnimQP()->getPnbBits() < getNbBits(m_shiftedValue[3*i+2])) )
		{
			char buffer[16];
			sprintf(buffer,"%d ==> %d",pBifaQuant->getAnimQP()->getPnbBits(),getNbBits(m_shiftedValue[3*i+2]));
			BifaError::error(1124,buffer);
			pBifaQuant->getAnimQP()->setPnbBits(getNbBits(m_shiftedValue[3*i+2]));
			return;
		}
	}
	
}


void ColorAnimationField::encodeValue(BifaQuantizer * pBifaQuant, bool flush)
{
	if (!m_encodedValue)
	{
		m_encodedValue = new int[m_nbQuantValue];
		m_nbaacbits  = new int[m_nbQuantValue];
	}
	
	for( int i=0; i< m_nbMField; i++)
	{
		int bits=0;
		m_encodedValue[3*i] = pBifaQuant->encodeAndUpdate(m_shiftedValue[3*i],0,1,bits);
		m_nbaacbits[3*i] = bits;
		m_encodedValue[3*i+1] = pBifaQuant->encodeAndUpdate(m_shiftedValue[3*i+1],0,1,bits);
		m_nbaacbits[3*i+1] = bits;
		
		m_encodedValue[3*i+2] = pBifaQuant->encodeAndUpdate(m_shiftedValue[3*i+2],0,1,bits,(i==(m_nbMField-1))&&flush);
		m_nbaacbits[3*i+2] = bits;
	}

}


void ColorAnimationField::write(FILE *file)
{
	fprintf(file,"Color3DAnim \t: maskId (%d)\t: Value : ",m_maskID);
	if (m_value)
	{
		for (int i =0; i<m_nbMField; i++)
			fprintf(file,"Field %d : %.3f %.3f %.3f \t: ",i,m_value[3*i],m_value[3*i+1],m_value[3*i+2]);
	}

	if (m_quantValue)
	{
		for (int i =0; i<m_nbMField; i++)
			fprintf(file,"Quant Val %d %d %d \t: ",m_quantValue[3*i],m_quantValue[3*i+1],m_quantValue[3*i+2]);
	}
	if (!isIntra())
	{
		
		if (m_shiftedValue)
		{
			for (int i =0; i<m_nbMField; i++)
				fprintf(file,"Shifted Val %d %d %d ",m_shiftedValue[3*i],m_shiftedValue[3*i+1],m_shiftedValue[3*i+2]);
		}
		
		if (m_encodedValue)
		{
			for (int i =0; i<m_nbMField; i++)
				fprintf(file,"Encoded Val %d(%d) %d(%d) %d(%d) ",m_encodedValue[3*i],m_nbaacbits[3*i] ,m_encodedValue[3*i+1],m_nbaacbits[3*i+1],m_encodedValue[3*i+2],m_nbaacbits[3*i+2],1);
		}
	}
	fprintf(file,"\n");

}


void ColorAnimationField::write(BinaryWriter * binWriter)
{

	AnimationField::write(binWriter);
	char buffer[64];

	if (isIntra())
	{
		for (int i=0;i<m_nbMField;i++)
		{
			
			binWriter->write((unsigned int)m_quantValue[3*i],m_nbBits,"quantValue");
			binWriter->write((unsigned int)m_quantValue[3*i+1],m_nbBits,"quantValue");
			binWriter->write((unsigned int)m_quantValue[3*i+2],m_nbBits,"quantValue");
		
			
			sprintf(buffer,"\t\t\t\t\t\t\t\t\t\t%.3f",m_value[3*i]);
			binWriter->log(buffer,1);
			sprintf(buffer,"\t\t\t\t\t\t\t\t\t\t%.3f",m_value[3*i+1]);
			binWriter->log(buffer,1);
			sprintf(buffer,"\t\t\t\t\t\t\t\t\t\t%.3f",m_value[3*i+2]);
			binWriter->log(buffer,1);


		}
	}
	else
	{
		for( int i=0; i< m_nbMField; i++)
		{
			if (m_encodedValue)
			{
				
				binWriter->writeEncoded((unsigned int)m_encodedValue[3*i],m_nbaacbits[3*i],"encodedValue","",1);
				binWriter->writeEncoded((unsigned int)m_encodedValue[3*i+1],m_nbaacbits[3*i+1],"encodedValue","",1);
				binWriter->writeEncoded((unsigned int)m_encodedValue[3*i+2],m_nbaacbits[3*i+2],"encodedValue","",1);
				
				
				sprintf(buffer,"\t\t\t\t\t\t\t\t\t\t%.3f",m_value[3*i]);
				binWriter->log(buffer,1);
				sprintf(buffer,"\t\t\t\t\t\t\t\t\t\t%.3f",m_value[3*i+1]);
				binWriter->log(buffer,1);
				sprintf(buffer,"\t\t\t\t\t\t\t\t\t\t%.3f",m_value[3*i+2]);
				binWriter->log(buffer,1);
			}
		}
	}

}


void ColorAnimationField::nbBitsToWrite(int &bits)
{
	AnimationField::nbBitsToWrite(bits);
	if (isIntra())
	{
		for (int i=0;i<m_nbQuantValue;i++)
		{
			
			bits+=m_nbBits;
		}
	}
	else
	{
		for( int i=0; i< m_nbMField; i++)
		{
			
			if (m_encodedValue)
			{
				bits+=m_nbaacbits[3*i];
				bits+=m_nbaacbits[3*i+1];
				bits+=m_nbaacbits[3*i+2];
			}
		}
	}
}


void ColorAnimationField::parse(DOM_Element* node)
{
	
}
