


#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "XmlParser.h"
#include <dom/DOM.hpp>
#include "define.h"
#include "BifaError.h"

#include "BinaryWriter.h"
#include "BifsConfig.h"
#include "AnimationMask.h"
#include "InitialFieldMask.h"
#include "ElementaryMask.h"






ElementaryMask::ElementaryMask( AnimationMask * AM) : m_animationMask(AM)
{
	m_atNode = 0;
	m_nodeID = 0;

	memset(m_tabIFM,NULL,sizeof(InitialFieldMask*)*MAXOBJECT);
	m_nbIFM =0;

	m_numDYNField = 10;

	AM->addElementaryMask(this);
}


ElementaryMask::~ElementaryMask()
{
	for (int i=0; i<m_nbIFM;i++)
		if (m_tabIFM)
			delete m_tabIFM[i];
}

void ElementaryMask::setAtNode(int atnod)
{
	
	if (atnod > (1<<m_animationMask->getBifsConfig()->getNodeIDbits()) )
	{
		char buffer[16];
		sprintf(buffer,"atNode : %d",atnod);
		BifaError::error(2001,buffer);
		return;
	}
	m_atNode = atnod;
}

int ElementaryMask::getAtNode()
{
	return m_atNode;
}

void ElementaryMask::setNodeID(int atnod)
{
	m_nodeID = atnod;
}

int ElementaryMask::getNodeID()
{
	return m_nodeID;
}



int ElementaryMask::addInitialFieldMask(InitialFieldMask * ifm)
{
	
	if (m_nbIFM > m_numDYNField)
	{
		char buffer[16];
		sprintf(buffer,"atNode : %d",m_atNode);
		BifaError::error(2002,buffer);
		return m_nbIFM;
	}
	m_tabIFM[m_nbIFM] = ifm;
	return m_nbIFM++;
}


InitialFieldMask* ElementaryMask::getInitialFieldMask(int i)
{
	if (i>=0 && i<m_nbIFM)
		return m_tabIFM[i];

	return NULL;
}


int ElementaryMask::getNbInitialFieldMask()
{
	return m_nbIFM;
}


void ElementaryMask::write(BinaryWriter * binWriter)
{
	char buffer[16];
	sprintf(buffer,"%d",m_atNode);
	binWriter->write((unsigned int)m_atNode,m_animationMask->getBifsConfig()->getNodeIDbits(),"nodeID",buffer);
	for (int i=0 ;i<m_numDYNField;i++)
	{
		bool writeFlag=false;
		for (int j=0; j<m_nbIFM; j++)
		{
			if (i == m_tabIFM[j]->getID() )
				writeFlag=true;
		}
		binWriter->write(writeFlag,1,"isAnimField");
	}

	for (i=0 ;i<m_numDYNField;i++)
	{
		
		for (int j=0; j<m_nbIFM; j++)
		{
			if (i == m_tabIFM[j]->getID() )
				m_tabIFM[j]->write(binWriter);			
		}
	}

}

void ElementaryMask::setNumDYNField(int num)
{
	m_numDYNField = num;
}

int ElementaryMask::getNumDYNField()
{
	return m_numDYNField;
}


void ElementaryMask::parse(DOM_Element * node)
{
	DOMString str;

	str = node->getAttribute(XMTATTR_ATNODE);
	if (str.length())
	{
		TRACEP("\t\t\t atNode(%s)\n",str.transcode());
		int atNode;
		sscanf(str.transcode(),"%d",&atNode);
		setAtNode(atNode);
	}
	
	str = node->getAttribute(XMTATTR_NUMDYNFIELDS);
	if (str.length())
	{
		
		TRACEP("\t\t\t numDynField(%s)\n",str.transcode());
		int num;
		sscanf(str.transcode(),"%d",&num);
		setNumDYNField(num);
	}
}
