


#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "define.h"
#include "BifaError.h"

#include "XmlParser.h"
#include <dom/DOM.hpp>

#include "BinaryWriter.h"
#include "AnimQP.h"
#include "BifaQuantizer.h"

#include "FloatAnimationField.h"






FloatAnimationField::FloatAnimationField(AnimationFrameData* AFD, BifaQuantizer* bquant) : AnimationField(AFD,bquant)
{
	m_value = NULL;
}


FloatAnimationField::~FloatAnimationField()
{
	if (m_value)
		delete [] m_value;
}


void FloatAnimationField::setValue(float * values, int nbValue)
{
	
	if (m_bifaquantizer && m_bifaquantizer->getAnimQP()
		&& m_bifaquantizer->getAnimQP()->getLowerBound()
		&& m_bifaquantizer->getAnimQP()->getUpperBound() )
	{
		float min = m_bifaquantizer->getAnimQP()->getLowerBound()[0];
		float max = m_bifaquantizer->getAnimQP()->getUpperBound()[0];
		
		for (int i=0; i<nbValue;i++)
		{ 
			if (values[i]<min || values[i]>max)
			{
				char buffer[16];
				sprintf(buffer,"%d : %.3f",7,values[i]);
				BifaError::error(2018,buffer);
				return;
			}
		}
	}

	
	m_nbMField = nbValue;

	m_value = new float[nbValue];
	memcpy(m_value,values,sizeof(float) * nbValue);
	m_nbValue = nbValue;

	TRACEP("\t\t\t\t");
	for (int i=0; i<m_nbValue; i++)
		TRACEP("%f ",m_value[i]);
	TRACEP("\n");
}


float* FloatAnimationField::getValue()
{
	return m_value;
}


int FloatAnimationField::getNbValue()
{
	return m_nbValue;
}


void FloatAnimationField::quantizeValue(BifaQuantizer * pBifaQuant)
{
	
	float *minVal = pBifaQuant->getAnimQP()->getLowerBound();
	float *maxVal = pBifaQuant->getAnimQP()->getUpperBound();

	
	m_nbBits = pBifaQuant->getAnimQP()->getInbBits();

	m_quantValue = new int[m_nbValue];
	m_nbQuantValue = m_nbValue;
	for (int i=0; i<m_nbMField;i++)
	{
		
		m_quantValue[i] = quantizeFloat(minVal[0],maxVal[0],m_nbBits,m_value[i]);

		
		m_value[i] = unQuantizeFloat(minVal[0],maxVal[0],m_nbBits,m_quantValue[i]);
		
	}
	
}


void FloatAnimationField::deltaValue(AnimationField * pPrec)
{
	if (!m_shiftedValue)
		m_shiftedValue = new int[m_nbQuantValue];
	
	m_shiftedValue[0] = m_quantValue[0]-((FloatAnimationField*)pPrec)->getQuantValue()[m_nbMField-1];

	for (int i=1; i<m_nbMField; i++)
		
		m_shiftedValue[i] = m_quantValue[i]-m_quantValue[i-1];
}


void FloatAnimationField::calcMinValue(int ** minValue)
{
	
	if (!*minValue)
	{
		
		*minValue = new int[1];
			
		(*minValue)[0] = m_shiftedValue[0];
		
		
		for (int i=1; i<m_nbMField;i++)
		{
			(*minValue)[0] = m_shiftedValue[i]<(*minValue)[0]?m_shiftedValue[i]:(*minValue)[0];
		}
	}
	else
	{
		
		for (int i=0; i<m_nbMField;i++)
		{
			(*minValue)[0] = m_shiftedValue[i]<(*minValue)[0]?m_shiftedValue[i]:(*minValue)[0];
		}
	}
}


void FloatAnimationField::shiftValue(BifaQuantizer * pBifaQuant, int* minvalue)
{
	if (!m_shiftedValue)
		m_shiftedValue = new int[m_nbQuantValue];
	
	
	for (int i=0; i<m_nbMField;i++)
	{
		m_shiftedValue[i] =m_shiftedValue[i]-minvalue[0];
		if (pBifaQuant->getAnimQP() 
			&& (pBifaQuant->getAnimQP()->getPnbBits() < getNbBits(m_shiftedValue[i])) )
		{
			char buffer[16];
			sprintf(buffer,"%d ==> %d",pBifaQuant->getAnimQP()->getPnbBits(),getNbBits(m_shiftedValue[i]));
			BifaError::error(1124,buffer);
			pBifaQuant->getAnimQP()->setPnbBits(getNbBits(m_shiftedValue[i]));
			return;
		}
	}
}


void FloatAnimationField::encodeValue(BifaQuantizer * pBifaQuant, bool flush)
{
	if (!m_encodedValue)
	{
		m_encodedValue = new int[m_nbQuantValue];
		m_nbaacbits  = new int[m_nbQuantValue];
	}
	
	
	for( int i=0; i< m_nbMField; i++)
	{
		int bits=0;
		
		m_encodedValue[i] = pBifaQuant->encodeAndUpdate(m_shiftedValue[i],0,1,bits,(i==(m_nbMField-1))&&flush);
		m_nbaacbits[i] = bits;
		
	}

}


void FloatAnimationField::write(FILE *file)
{
	fprintf(file,"FloatAnim \t: maskId (%d)\t: Value : ",m_maskID);
	if (m_value)
	{
		for (int i =0; i<m_nbMField; i++)
			fprintf(file,"Field %d : %.3f \t: ",i,m_value[i]);
	}

	if (m_quantValue)
	{
		for (int i =0; i<m_nbMField; i++)
			fprintf(file,"Quant Val %d \t: ",m_quantValue[i]);
	}
	if (!isIntra())
	{
		
		if (m_shiftedValue)
		{
			for (int i =0; i<m_nbMField; i++)
				fprintf(file,"Shifted Val %d ",m_shiftedValue[i]);
		}
		
		if (m_encodedValue)
		{
			for (int i =0; i<m_nbMField; i++)
				fprintf(file,"Encoded Val %d(%d) ",m_encodedValue[i],m_nbaacbits[i],1);
		}
	}
	fprintf(file,"\n");

}


void FloatAnimationField::write(BinaryWriter * binWriter)
{

	AnimationField::write(binWriter);

	char buffer[64];

	if (isIntra())
	{
		for (int i=0;i<m_nbMField;i++)
		{
			
			binWriter->write((unsigned int)m_quantValue[i],m_nbBits,"quantValue");
		
			
			sprintf(buffer,"\t\t\t\t\t\t\t\t\t\t%.3f",m_value[i]);
			binWriter->log(buffer,1);

		}
	}
	else
	{
		for( int i=0; i< m_nbMField; i++)
		{
			
			if (m_encodedValue)
				binWriter->writeEncoded((unsigned int)m_encodedValue[i],m_nbaacbits[i],"encodedValue","",1);

			
			sprintf(buffer,"\t\t\t\t\t\t\t\t\t\t%.3f",m_value[i]);
			binWriter->log(buffer,1);

		}
	}

}


void FloatAnimationField::nbBitsToWrite(int &bits)
{
	AnimationField::nbBitsToWrite(bits);
	if (isIntra())
	{
		for (int i=0;i<m_nbQuantValue;i++)
		{
			
			bits+=m_nbBits;
		}
	}
	else
	{
		for( int i=0; i< m_nbMField; i++)
		{
			
			if (m_encodedValue)
				bits+=m_nbaacbits[i];
		}
	}
}


void FloatAnimationField::parse(DOM_Element * node)
{
	
}
