


#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "define.h"
#include "BifaError.h"

#include "XmlParser.h"
#include <dom/DOM.hpp>

#include "BinaryWriter.h"
#include "ElementaryMask.h"
#include "AnimQP.h"
#include "InitialFieldMask.h"






InitialFieldMask::InitialFieldMask(ElementaryMask* EM) : m_elementaryMask(EM)
{
	m_ID = 0;
	m_isTotal = false;
	m_indexList = NULL;
	m_nbIndex = 0;
	m_initialAnimQP = NULL;
	
	EM->addInitialFieldMask(this);

}


InitialFieldMask::~InitialFieldMask()
{
	if (m_indexList)
		delete [] m_indexList;
	if (m_initialAnimQP)
		delete m_initialAnimQP;
	
}

void InitialFieldMask::setID(int ID)
{
	
	if (ID > m_elementaryMask->getNumDYNField())
	{
		char buffer[32];
		sprintf(buffer,"atNode %d FieldID %d",m_elementaryMask->getAtNode(),ID);
		BifaError::error(2003,buffer);
		return;
	}
	m_ID = ID;
}

int InitialFieldMask::getID()
{
	return m_ID;
}


void InitialFieldMask::setIsTotal(bool istotal)
{
	m_isTotal = istotal;
}

bool InitialFieldMask::getIsTotal()
{
	return m_isTotal;
}

void InitialFieldMask::setIndexList(int *indexlist, int nbindex)
{
	m_indexList = new int[nbindex];
	memcpy(m_indexList,indexlist,sizeof(int)*nbindex);
	m_nbIndex = nbindex;
}

int* InitialFieldMask::getIndexList()
{
	return m_indexList;
}

int InitialFieldMask::getNbIndex()
{
	return m_nbIndex;
}
	

void InitialFieldMask::setInitialAnimQP(AnimQP* aqp)
{
	m_initialAnimQP = aqp;
}

AnimQP* InitialFieldMask::getInitialAnimQP()
{
	return m_initialAnimQP;
}



void InitialFieldMask::write(BinaryWriter * binWriter)
{
	
	if (m_indexList || m_isTotal)
	{
		
		binWriter->write(m_isTotal,1,"isTotal");
		if (!m_isTotal)
		{
			
			int maxIndex=0;
			
			for (int i=0;i<m_nbIndex;i++)
			{
				if(m_indexList[i]>maxIndex)
					maxIndex=m_indexList[i];
			}
			
			int nbBit = getNbBits(maxIndex);
			binWriter->write((unsigned int)nbBit ,5,"indexListNbBits");
			
			
			for (i=0;i<m_nbIndex;i++)
			{
				binWriter->write((unsigned int)m_indexList[i],nbBit,"index");
				
				if (i<(m_nbIndex-1))
					binWriter->write(true,1,"moreIndices");
			}
			
			binWriter->write(false,1,"moreIndices");
		}
	}
	
	m_initialAnimQP->writeMask(binWriter);
	
}


int InitialFieldMask::getNbBits(int k)
{
	int j=1;
	while (k >= (1<<j) ) 
		j++;
	return j;
}


void InitialFieldMask::parse(DOM_Element* node)
{
	
	DOMString str;
	str = node->getAttribute(XMTATTR_ID);
	if (str.length())
	{
		TRACEP("\t\t\t id(%s)\n",str.transcode());
		int id;
		sscanf(str.transcode(),"%d",&id);
		setID(id);
	}
	
	str = node->getAttribute(XMTATTR_ISTOTAL);
	if (str.length())
	{
		TRACEP("\t\t\t isTotal(%s)\n",str.transcode());
		if (str.equals("true"))
			setIsTotal(true);
		else
			setIsTotal(false);
		
	}
	
	
	str = node->getAttribute(XMTATTR_INDEXLIST);
	if (str.length())
	{
		TRACEP("\t\t\t indexList(%s)\n",str.transcode());
		int nbElm=0;
		char * buffer =strdup(str.transcode());
		XmlParser::getCleanBuffer(buffer,nbElm);
		
		char * buff = buffer;
		
		int * indexList = new int[nbElm];
		char *tmp = buff;
		
		for (int i=0; i<nbElm; i++)
		{
			while(*tmp && (*tmp != ' ')) 
				tmp++;
			*tmp = '\0';
			sscanf(buff,"%d",&(indexList[i]));
			buff = ++tmp;
		}
		setIndexList(indexList,nbElm);
		delete buffer;
	}
}
