


#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "BifaError.h"
#include "define.h"

#include "XmlParser.h"
#include <dom/DOM.hpp>

#include "BinaryWriter.h"
#include "AnimQP.h"
#include "BifaQuantizer.h"

#include "IntegerAnimationField.h"






IntegerAnimationField::IntegerAnimationField(AnimationFrameData* AFD, BifaQuantizer* bquant) : AnimationField(AFD,bquant)
{
	m_value = NULL;
}


IntegerAnimationField::~IntegerAnimationField()
{
	if (m_value)
		delete [] m_value;
}



void IntegerAnimationField::setValue(int * values, int nbValue)
{
	if (m_bifaquantizer && m_bifaquantizer->getAnimQP())
	{
		int intMin = m_bifaquantizer->getAnimQP()->getIminInt();
		int intMax = (1<<(m_bifaquantizer->getAnimQP()->getInbBits()))-1+intMin;

		for (int i=0; i<nbValue;i++)
		{ 
			if (values[i]<intMin || values[i]>intMax)
			{
				char buffer[16];
				sprintf(buffer,"%d : %d",13,values[i]);
				BifaError::error(2018,buffer);
				return;
			}
		}
	}
	
	
	m_nbMField = nbValue;

	m_value = new int[nbValue];
	memcpy(m_value,values,sizeof(int) * nbValue);
	m_nbValue = nbValue;

	TRACEP("\t\t\t\t");
	for (int i=0; i<m_nbValue; i++)
		TRACEP("%d ",m_value[i]);
	TRACEP("\n");
}


int* IntegerAnimationField::getValue()
{
	return m_value;
}


int IntegerAnimationField::getNbValue()
{
	return m_nbValue;
}


void IntegerAnimationField::quantizeValue(BifaQuantizer * pBifaQuant)
{
	
	int intMin = pBifaQuant->getAnimQP()->getIminInt();
	m_nbBits = pBifaQuant->getAnimQP()->getInbBits();
	int intMax = (1<<m_nbBits)-1+intMin;

	
	m_quantValue = new int[m_nbValue];
	m_nbQuantValue = m_nbValue;

	for (int i=0; i<m_nbMField;i++)
	{
		
		if (m_value[i]<intMin)
		{
				char buffer[32];
				sprintf(buffer,"%d ==> return 0",m_value[i]);
				BifaError::error(18,buffer);
				m_value[i]=intMin;
		}
		else if  (m_value[i]>intMax)
		{
			char buffer[32];
			sprintf(buffer,"%d ==> return %d",m_value[i],intMax - intMin);
			BifaError::error(18,buffer);
			m_value[i]=intMax;
		}
		

		m_quantValue[i] = m_value[i] - intMin;
	}
	
}


void IntegerAnimationField::deltaValue(AnimationField * pPrec)
{
	if (!m_shiftedValue)
		m_shiftedValue = new int[m_nbQuantValue];
	
	m_shiftedValue[0] = m_quantValue[0]-((IntegerAnimationField*)pPrec)->getQuantValue()[m_nbMField-1];

	
	for (int i=1; i<m_nbMField; i++)
		m_shiftedValue[i] = m_quantValue[i]-m_quantValue[i-1];
}


void IntegerAnimationField::calcMinValue(int ** minValue)
{
	
	if (!*minValue)
	{
		
		*minValue = new int[1];
			
		(*minValue)[0] = m_shiftedValue[0];
		
		for (int i=1; i<m_nbMField;i++)
		{
			(*minValue)[0] = m_shiftedValue[i]<(*minValue)[0]?m_shiftedValue[i]:(*minValue)[0];
		}
	}
	else
	{
		
		for (int i=0; i<m_nbMField;i++)
		{
			(*minValue)[0] = m_shiftedValue[i]<(*minValue)[0]?m_shiftedValue[i]:(*minValue)[0];
		}
	}
}


void IntegerAnimationField::shiftValue(BifaQuantizer * pBifaQuant, int* minvalue)
{
	if (!m_shiftedValue)
		m_shiftedValue = new int[m_nbQuantValue];
	
	
	for (int i=0; i<m_nbMField;i++)
	{
		m_shiftedValue[i] =m_shiftedValue[i]-minvalue[0];
		if (pBifaQuant->getAnimQP() 
			&& (pBifaQuant->getAnimQP()->getPnbBits() < getNbBits(m_shiftedValue[i])) )
		{
			char buffer[16];
			sprintf(buffer,"%d ==> %d",pBifaQuant->getAnimQP()->getPnbBits(),getNbBits(m_shiftedValue[i]));
			BifaError::error(1124,buffer);
			pBifaQuant->getAnimQP()->setPnbBits(getNbBits(m_shiftedValue[i]));
			return;
		}
	}
}


void IntegerAnimationField::encodeValue(BifaQuantizer * pBifaQuant, bool flush)
{
	if (!m_encodedValue)
	{
		m_encodedValue = new int[m_nbQuantValue];
		m_nbaacbits  = new int[m_nbQuantValue];
	}

	
	for( int i=0; i< m_nbMField; i++)
	{
		int bits=0;
		m_encodedValue[i] = pBifaQuant->encodeAndUpdate(m_shiftedValue[i],0,1,bits,(i==(m_nbMField-1))&&flush);
		m_nbaacbits[i] = bits;
		
	}

}


void IntegerAnimationField::write(FILE *file)
{
	fprintf(file,"IntegerAnim \t: maskId (%d)\t: Value : ",m_maskID);
	if (m_value)
	{
		for (int i =0; i<m_nbMField; i++)
			fprintf(file,"Field %d : %d \t: ",i,m_value[i]);
	}

	if (m_quantValue)
	{
		for (int i =0; i<m_nbMField; i++)
			fprintf(file,"Quant Val %d \t: ",m_quantValue[i]);
	}
	if (!isIntra())
	{
		
		if (m_shiftedValue)
		{
			for (int i =0; i<m_nbMField; i++)
				fprintf(file,"Shifted Val %d ",m_shiftedValue[i]);
		}
		
		if (m_encodedValue)
		{
			for (int i =0; i<m_nbMField; i++)
				fprintf(file,"Encoded Val %d(%d) ",m_encodedValue[i],m_nbaacbits[i],1);
		}
	}
	fprintf(file,"\n");

}


void IntegerAnimationField::write(BinaryWriter * binWriter)
{

	AnimationField::write(binWriter);

	char buffer[64];


	if (isIntra())
	{
		
		for (int i=0;i<m_nbMField;i++)
		{
			
			binWriter->write((unsigned int)m_quantValue[i],m_nbBits,"quantValue");

			
			sprintf(buffer,"\t\t\t\t\t\t\t\t\t\t%d",m_value[i]);
			binWriter->log(buffer,1);

		}
	}
	else
	{
		for( int i=0; i< m_nbMField; i++)
		{
			
			if (m_encodedValue)
				binWriter->writeEncoded((unsigned int)m_encodedValue[i],m_nbaacbits[i],"encodedValue","",1);

			
			sprintf(buffer,"\t\t\t\t\t\t\t\t\t\t%d",m_value[i]);
			binWriter->log(buffer,1);

		}
	}
}


void IntegerAnimationField::nbBitsToWrite(int &bits)
{
	AnimationField::nbBitsToWrite(bits);
	if (isIntra())
	{
		
		for (int i=0;i<m_nbQuantValue;i++)
		{
			bits+=m_nbBits;
		}
	}
	else
	{
		for( int i=0; i< m_nbMField; i++)
		{
			
			if (m_encodedValue)
				bits+=m_nbaacbits[i];
		}
	}
}


void IntegerAnimationField::parse(DOM_Element* node)
{
	

}
