#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "XmlParser.h"
#include "BifaQuantizer.h"
#include "AnimationFrame.h"
#include "BifsConfig.h"
#include "BinaryWriter.h"

void usage(char** argv)
{
	printf("usage : %s <file.xmt>",argv[0]);
	exit(-1);
}



int main(int argc, char** argv)
{

	if ( argc < 2 )
	{
		usage(argv);
	}

	int len = strlen(argv[1]);
	char * buffer = new char[len+2];
	strcpy(buffer,argv[1]);

	
	int errorLevel =0;
	bool trace_flag =false;
	
	for (int i =2; i<argc;i++)
	{
		if (!strcmp(argv[i],"-o"))
		{
			i++;
			delete(buffer);
			len = strlen(argv[i]);
			buffer = new char[len+6];
			strcpy(buffer,argv[i]);
		}
		else if (!strcmp(argv[i],"-L"))
		{
			i++;
			sscanf(argv[i],"%d",&errorLevel);
		}
		else if (!strcmp(argv[i],"-trace"))
			trace_flag = true;
	}



	XmlParser * xmlParser = new XmlParser;
	xmlParser->setFilename(argv[1]);

	xmlParser->init();
	

	BifaQuantizer ** tabBifaQuantizer = NULL;
	AnimationFrame ** tabAnimationFrame = NULL; 
	BifsConfig	bifsConfig;

	int	nbBifaQuantizer;
	int	nbAnimationFrame;

	xmlParser->parse(&bifsConfig,&tabBifaQuantizer,nbBifaQuantizer,&tabAnimationFrame,nbAnimationFrame);

	for ( i=0; i<nbBifaQuantizer; i++)
		tabBifaQuantizer[i]->quantize();

	
	for (int frame=0; frame<nbAnimationFrame; frame++)
		for (int anim =0; anim<nbBifaQuantizer;anim++)
			tabBifaQuantizer[anim]->encode(frame,(anim==0),anim==(nbBifaQuantizer-1));




	char *point = strrchr(buffer,'.');
	if (!point)
		point = buffer+strlen(buffer);


	if (trace_flag)
	{
		strcpy(point,".trac");
		FILE * file = fopen(buffer,"w");
		if (file)
		{
			for (int j=0; j<nbAnimationFrame; j++)
				tabAnimationFrame[j]->write(file);
		}
		fclose(file);
	}

	strcpy(point,".logb");
	FILE* BinLogFile = fopen(buffer,"w");
	
	strcpy(point,".mask");
	FILE* maskFile = fopen(buffer,"wb");
	BinaryWriter* maskWriter = new BinaryWriter(maskFile,BinLogFile,errorLevel);

	strcpy(point,".bifa");
	FILE* bifaFile = fopen(buffer,"wb");
	BinaryWriter* bifaWriter = new BinaryWriter(bifaFile,BinLogFile,errorLevel);

	bifsConfig.write(maskWriter);
	maskWriter->flush();
	fclose(maskFile);



	for (int j=0; j<nbAnimationFrame; j++)
		tabAnimationFrame[j]->write(bifaWriter);

	
	bifaWriter->write((unsigned int)-1,32);
	bifaWriter->write((unsigned int)-1,32);

	fclose(BinLogFile);


	
	for (i=0;i<nbAnimationFrame;i++)
		delete  tabAnimationFrame[i];
	for (i=0;i<nbBifaQuantizer;i++)
		delete tabBifaQuantizer[i];

	delete [] tabAnimationFrame;
	delete [] tabBifaQuantizer;

	delete xmlParser;
	delete maskWriter;
	delete bifaWriter;
	delete buffer;
	return 0;

}
