


#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

#include "define.h"
#include "BifaError.h"

#include "XmlParser.h"
#include <dom/DOM.hpp>

#include "BinaryWriter.h"
#include "AnimQP.h"
#include "BifaQuantizer.h"
#include "ArithModel.h"
#include "ArithEncoder.h"

#include "NormalAnimationField.h"






NormalAnimationField::NormalAnimationField(AnimationFrameData* AFD, BifaQuantizer* bquant) : AnimationField(AFD,bquant)
{
	m_value = NULL;
	m_orientation = NULL;
	m_direction = NULL;;
	m_quat = NULL;
	m_inverseDir = NULL;
	m_nbBitsInv = NULL;

	
	m_arithModel = new ArithModel();
	m_arithModel->init(2);

}


NormalAnimationField::~NormalAnimationField()
{
	if (m_value)
		delete [] m_value;
	if (m_orientation)
		delete m_orientation;
	if (m_direction)
		delete m_direction;
	if (m_quat)
		delete m_quat;
	if (m_inverseDir)
		delete (m_inverseDir);
	if (m_nbBitsInv)
		delete (m_nbBitsInv);
	if (m_arithModel)
		delete m_arithModel;
}


void NormalAnimationField::setValue(float * values, int nbValue)
{

	
	if ((nbValue%3) != 0 )
	{
		char buffer[32];
		sprintf(buffer,"type : %d nbValue : %d",9,nbValue);
		BifaError::error(2017,buffer);
		return;
	}
	
	
	m_nbMField = nbValue/3;

	m_value = new float[nbValue];
	memcpy(m_value,values,sizeof(float) * nbValue);
	m_nbValue = nbValue;

	TRACEP("\t\t\t\t");
	for (int i=0; i<m_nbValue; i++)
		TRACEP("%f ",m_value[i]);
	TRACEP("\n");
}


float* NormalAnimationField::getValue()
{
	return m_value;
}


int NormalAnimationField::getNbValue()
{
	return m_nbValue;
}


int	NormalAnimationField::getDirection(int field)
{
	if (field < m_nbMField)
		return m_direction[field];
	
	return 0;
}


void NormalAnimationField::quantizeValue(BifaQuantizer * pBifaQuant)
{
	
	m_nbQuantValue = m_nbValue-m_nbMField;
	m_quantValue = new int[m_nbQuantValue];


	
	m_nbBits = pBifaQuant->getAnimQP()->getInbBits();
	m_quat = new float[m_nbMField*3];
	m_orientation = new int[m_nbMField];
	m_direction = new int[m_nbMField];

	
	for (int indField=0; indField<m_nbMField;indField++)
	{
		double m_x = m_value[3*indField+0];
		double m_y = m_value[3*indField+1];
		double m_z = m_value[3*indField+2];

		float normR = (float)sqrt(m_x*m_x + m_y*m_y + m_z*m_z);
		float scale = 1.0f/normR;

		m_quat[3*indField+0] = (float)m_x*scale;
		m_quat[3*indField+1] = (float)m_y*scale;
		m_quat[3*indField+2] = (float)m_z*scale;
		
		
		float maxTmp=m_quat[0];
		m_orientation[indField]=0;
		m_direction[indField]=m_quat[0]>0?1:-1;
		
		for (int i=0;i<3;i++) {
			if (fabs(m_quat[i])>=maxTmp) {
				maxTmp=(float)fabs(m_quat[i]);
				m_orientation[indField]=i;
				m_direction[indField]=m_quat[i]>0?1:-1;
			}
		}
				
		
		for (i=0;i<2;i++) 
		{
			float v=(float)( 4/M_PI * atan( (double)( m_quat[(m_orientation[indField]+i+1)%4] / m_quat[m_orientation[indField]]) ) );
			int tmp = (v>=0?1:-1)*quantizeFloat(0, 1, m_nbBits-1, (v>=0?v:-v));
			m_quantValue[2*indField+i]= (1<<(m_nbBits-1)) + tmp;
		}

		
		
		int vq[2];
		vq[0] = m_quantValue[2*indField]-(1<<(m_nbBits-1));
		vq[1] = m_quantValue[2*indField+1]-(1<<(m_nbBits-1));
		

		float * val = unQuantizeNormal (vq,m_nbBits-1, m_direction[indField], m_orientation[indField]);
		m_value[3*indField] = val[0];
		m_value[3*indField+1] = val[1];
		m_value[3*indField+2] = val[2];
		
		delete val;



	}
	
}


void NormalAnimationField::deltaValue(AnimationField * pPrec)
{
	if (!m_shiftedValue)
		m_shiftedValue = new int[m_nbQuantValue];
	if (!m_inverseDir)
		m_inverseDir = new int[m_nbMField];
	
	
	m_shiftedValue[0] = m_quantValue[0]-((NormalAnimationField*)pPrec)->getQuantValue()[2*(m_nbMField-1)];
	m_shiftedValue[1] = m_quantValue[1]-((NormalAnimationField*)pPrec)->getQuantValue()[2*(m_nbMField-1)+1];
	m_inverseDir[0] = ((NormalAnimationField*)pPrec)->getDirection(m_nbMField-1)==m_direction[0]?0:1;
	
	
	for (int i=1; i<m_nbMField; i++)
	{
		m_shiftedValue[2*i] = m_quantValue[2*i]-m_quantValue[2*(i-1)];
		m_shiftedValue[2*i+1] = m_quantValue[2*i+1]-m_quantValue[2*(i-1)+1];
		m_inverseDir[i]=m_direction[i]==m_direction[i-1]?0:1;
	}
}


void NormalAnimationField::calcMinValue(int ** minValue)
{
	
	if (!*minValue)
	{
		
		*minValue = new int[1];
		
		
		for (int i=0; i<m_nbMField;i++)
		{
			(*minValue)[0] = m_shiftedValue[2*i];
			(*minValue)[0] = m_shiftedValue[2*i+1]<(*minValue)[0]?m_shiftedValue[2*i+1]:(*minValue)[0];
		}
	}
	else
	{
		
		for (int i=0; i<m_nbMField;i++)
		{
			(*minValue)[0] = m_shiftedValue[2*i]<(*minValue)[0]?m_shiftedValue[2*i]:(*minValue)[0];
			(*minValue)[0] = m_shiftedValue[2*i+1]<(*minValue)[0]?m_shiftedValue[2*i+1]:(*minValue)[0];
		}
	}
}


void NormalAnimationField::shiftValue(BifaQuantizer * pBifaQuant, int* minvalue)
{
	if (!m_shiftedValue)
		m_shiftedValue = new int[m_nbQuantValue];
	
	
	for (int i=0; i<m_nbMField;i++)
	{
		m_shiftedValue[2*i] =m_shiftedValue[2*i]-minvalue[0];
		m_shiftedValue[2*i+1] =m_shiftedValue[2*i+1]-minvalue[0];

		if (pBifaQuant->getAnimQP() 
			&& (pBifaQuant->getAnimQP()->getPnbBits() < getNbBits(m_shiftedValue[2*i])) )
		{
			char buffer[16];
			sprintf(buffer,"%d ==> %d",pBifaQuant->getAnimQP()->getPnbBits(),getNbBits(m_shiftedValue[2*i]));
			BifaError::error(1124,buffer);
			pBifaQuant->getAnimQP()->setPnbBits(getNbBits(m_shiftedValue[2*i]));
			return;
		}
		
		if (pBifaQuant->getAnimQP() 
			&& (pBifaQuant->getAnimQP()->getPnbBits() < getNbBits(m_shiftedValue[2*i+1])) )
		{
			char buffer[16];
			sprintf(buffer,"%d ==> %d",pBifaQuant->getAnimQP()->getPnbBits(),getNbBits(m_shiftedValue[2*i+1]));
			BifaError::error(1124,buffer);
			pBifaQuant->getAnimQP()->setPnbBits(getNbBits(m_shiftedValue[2*i+1]));
			return;
		}
	}
	
	
}


void NormalAnimationField::encodeValue(BifaQuantizer * pBifaQuant, bool flush)
{
	if (!m_encodedValue)
	{
		m_encodedValue = new int[m_nbQuantValue];
		m_nbaacbits  = new int[m_nbQuantValue];
		
		m_nbBitsInv = new int[m_nbMField];
	}

	
	for( int i=0; i< m_nbMField; i++)
	{
		int bits=0;
		m_encodedValue[2*i] = pBifaQuant->encodeAndUpdate(m_shiftedValue[2*i],0,1,bits);
		m_nbaacbits[2*i] = bits;
		
		m_encodedValue[2*i+1] = pBifaQuant->encodeAndUpdate(m_shiftedValue[2*i+1],0,1,bits,(i==(m_nbMField-1))&&flush);
		m_nbaacbits[2*i+1] = bits;

		
		BifaQuantizer::m_ArithEncoder.encode(m_inverseDir[i],*m_arithModel);
		BifaQuantizer::m_ArithEncoder.getLastValue(m_inverseDir[i],m_nbBitsInv[i]);

	}

}


void NormalAnimationField::write(FILE *file)
{
	AnimationField::write(file);
	fprintf(file,"NormalAnim \tmaskId (%d) ",m_maskID);
	if (m_value)
	{
		for (int i =0; i<m_nbMField; i++)
			fprintf(file,"Values %.3f %.3f %.3f ",m_value[3*i],m_value[3*i+1],m_value[3*i+2]);
	}

	if (m_quantValue)
	{
		for (int i =0; i<m_nbMField; i++)
			fprintf(file,"\tQuant Val %d %d ",m_quantValue[2*i],m_quantValue[2*i+1]);
	}
	if (!isIntra())
	{
		
		if (m_shiftedValue)
		{
			for (int i =0; i<m_nbMField; i++)
				fprintf(file,"\tShifted Val %d %d ",m_shiftedValue[2*i],m_shiftedValue[2*i+1]);
		}
		
		if (m_encodedValue)
		{
			for (int i =0; i<m_nbMField; i++)
				fprintf(file,"\tEncoded Val %d(%d) %d(%d) ",m_encodedValue[2*i],m_nbaacbits[2*i] ,m_encodedValue[2*i+1],m_nbaacbits[2*i+1],1);
		}
	}
	fprintf(file,"\n");
	
}


void NormalAnimationField::write(BinaryWriter * binWriter)
{
	AnimationField::write(binWriter);
	
	char buffer[64];

	if (isIntra())
	{

		
		for (int i=0;i<m_nbMField;i++)
		{
			
			binWriter->write((unsigned int)(m_direction[i]>0?0:1),1,"direction");
			binWriter->write((unsigned int)m_orientation[i],2,"orientation");
			binWriter->write((unsigned int)m_quantValue[2*i],m_nbBits,"quantValue");
			binWriter->write((unsigned int)m_quantValue[2*i+1],m_nbBits,"quantValue");

			
			sprintf(buffer,"\t\t\t\t\t\t\t\t\t\t%.3f",m_value[3*i]);
			binWriter->log(buffer,1);
			sprintf(buffer,"\t\t\t\t\t\t\t\t\t\t%.3f",m_value[3*i+1]);
			binWriter->log(buffer,1);
			sprintf(buffer,"\t\t\t\t\t\t\t\t\t\t%.3f",m_value[3*i+2]);
			binWriter->log(buffer,1);
		}
	}
	else
	{
		for (int i=0;i<m_nbMField;i++)
		{
			binWriter->write((unsigned int)m_inverseDir[i],1,"encodedValue","",1);
			if (m_encodedValue)
			{
				
				binWriter->writeEncoded((unsigned int)m_encodedValue[2*i],m_nbaacbits[2*i],"encodedValue","",1);
				binWriter->writeEncoded((unsigned int)m_encodedValue[2*i+1],m_nbaacbits[2*i+1],"encodedValue","",1);
				
				
				sprintf(buffer,"\t\t\t\t\t\t\t\t\t\t%.3f",m_value[3*i]);
				binWriter->log(buffer,1);
				sprintf(buffer,"\t\t\t\t\t\t\t\t\t\t%.3f",m_value[3*i+1]);
				binWriter->log(buffer,1);
				sprintf(buffer,"\t\t\t\t\t\t\t\t\t\t%.3f",m_value[3*i+2]);
				binWriter->log(buffer,1);
			}
		}
	}
}


void NormalAnimationField::nbBitsToWrite(int &bits)
{
	AnimationField::nbBitsToWrite(bits);
	if (isIntra())
	{
		
		for (int i=0;i<m_nbMField;i++)
		{
			
			bits+=2;
			bits+=(2*m_nbBits);
		}
	}
	else
	{
		bits++;
		for (int i=0;i<m_nbMField;i++)
		{
			
			bits+=m_nbaacbits[2*i];
			bits+=m_nbaacbits[2*i+1];
		}
	}

}


void NormalAnimationField::parse(DOM_Element* node)
{
	
}
