


#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

#include "define.h"
#include "BifaError.h"

#include "XmlParser.h"
#include <dom/DOM.hpp>

#include "AnimQP.h"
#include "BifaQuantizer.h"
#include "BinaryWriter.h"
#include "RotationAnimationField.h"






RotationAnimationField::RotationAnimationField(AnimationFrameData* AFD, BifaQuantizer* bquant) : AnimationField(AFD,bquant)
{
	m_value = NULL;
	m_orientation = NULL;
	m_quat = NULL;
}


RotationAnimationField::~RotationAnimationField()
{
	if (m_value)
		delete [] m_value;
	if (m_orientation)
		delete m_orientation;
	if (m_quat)
		delete m_quat;

}


void RotationAnimationField::setValue(float * values, int nbValue)
{
	
	if ((nbValue%4) != 0 )
	{
		char buffer[32];
		sprintf(buffer,"type : %d nbValue : %d",10,nbValue);
		BifaError::error(2017,buffer);
		return;
	}
	
	
	
	m_nbMField = nbValue/4;

	m_value = new float[nbValue];
	memcpy(m_value,values,sizeof(float) * nbValue);
	m_nbValue = nbValue;

	for (int i=0; i<m_nbValue; i++)
		TRACEP("%f ",m_value[i]);
	TRACEP("\n");
}


float* RotationAnimationField::getValue()
{
	return m_value;
}


int RotationAnimationField::getNbValue()
{
	return m_nbValue;
}



void RotationAnimationField::quantizeValue(BifaQuantizer * pBifaQuant)
{
	
	m_nbQuantValue = m_nbValue-m_nbMField;
	m_quantValue = new int[m_nbQuantValue];

	
	m_nbBits = pBifaQuant->getAnimQP()->getInbBits();
	m_quat = new float[m_nbMField*4];
	m_orientation = new int[m_nbMField];

	
	for (int indField=0; indField<m_nbMField;indField++)
	{
		double m_x = m_value[4*indField+0];
		double m_y = m_value[4*indField+1];
		double m_z = m_value[4*indField+2];
		double m_angle = m_value[4*indField+3];
		

		float normR = (float)sqrt(m_x*m_x + m_y*m_y + m_z*m_z);
		
		m_quat[4*indField+0]=(float)cos(m_angle/2);
		float scale = (float)sin(m_angle/2)/normR;
		m_quat[4*indField+1] = (float)m_x*scale;
		m_quat[4*indField+2] = (float)m_y*scale;
		m_quat[4*indField+3] = (float)m_z*scale;
		
		float maxTmp=m_quat[0];
		m_orientation[indField]=0;
		
		for (int i=0;i<4;i++) {
			if (fabs(m_quat[i])>=maxTmp) {
				maxTmp=(float)fabs(m_quat[i]);
				m_orientation[indField]=i;
			}
		}
				
		
		for (i=0;i<3;i++) 
		{
			float v=(float)( 4/M_PI * atan( (double)( m_quat[(m_orientation[indField]+i+1)%4] / m_quat[m_orientation[indField]]) ) );
			int tmp = (v>=0?1:-1)*quantizeFloat(0, 1, m_nbBits-1, (v>=0?v:-v));
			m_quantValue[3*indField+i]= (1<<(m_nbBits-1)) + tmp;
		}

		
		int vq[3];
		vq[0] = m_quantValue[3*indField]-(1<<(m_nbBits-1));
		vq[1] = m_quantValue[3*indField+1]-(1<<(m_nbBits-1));
		vq[2] = m_quantValue[3*indField+2]-(1<<(m_nbBits-1));

		float * val = unQuantizeRotation (vq,m_nbBits-1, 1, m_orientation[indField]);
		m_value[4*indField] = val[0];
		m_value[4*indField+1] = val[1];
		m_value[4*indField+2] = val[2];
		m_value[4*indField+3] = val[3];
		delete val;



	}
	
}


void RotationAnimationField::deltaValue(AnimationField * pPrec)
{
	if (!m_shiftedValue)
		m_shiftedValue = new int[m_nbQuantValue];
	
	
	m_shiftedValue[0] = m_quantValue[0]-((RotationAnimationField*)pPrec)->getQuantValue()[3*(m_nbMField-1)];
	m_shiftedValue[1] = m_quantValue[1]-((RotationAnimationField*)pPrec)->getQuantValue()[3*(m_nbMField-1)+1];
	m_shiftedValue[2] = m_quantValue[2]-((RotationAnimationField*)pPrec)->getQuantValue()[3*(m_nbMField-1)+2];

	
	for (int i=1; i<m_nbMField; i++)
	{
		m_shiftedValue[3*i] = m_quantValue[3*i]-m_quantValue[3*(i-1)];
		m_shiftedValue[3*i+1] = m_quantValue[3*i+1]-m_quantValue[3*(i-1)+1];
		m_shiftedValue[3*i+2] = m_quantValue[3*i+2]-m_quantValue[3*(i-1)+2];
	}
}


void RotationAnimationField::calcMinValue(int ** minValue)
{
	
	if (!*minValue)
	{
		
		*minValue = new int[1];
		
		
		for (int i=0; i<m_nbMField;i++)
		{
			(*minValue)[0] = m_shiftedValue[3*i];
			(*minValue)[0] = m_shiftedValue[3*i+1]<(*minValue)[0]?m_shiftedValue[3*i+1]:(*minValue)[0];
			(*minValue)[0] = m_shiftedValue[3*i+2]<(*minValue)[0]?m_shiftedValue[3*i+2]:(*minValue)[0];
		}
	}
	else
	{
		
		for (int i=0; i<m_nbMField;i++)
		{
			(*minValue)[0] = m_shiftedValue[3*i]<(*minValue)[0]?m_shiftedValue[3*i]:(*minValue)[0];
			(*minValue)[0] = m_shiftedValue[3*i+1]<(*minValue)[0]?m_shiftedValue[3*i+1]:(*minValue)[0];
			(*minValue)[0] = m_shiftedValue[3*i+2]<(*minValue)[0]?m_shiftedValue[3*i+2]:(*minValue)[0];
		}
	}
}


void RotationAnimationField::shiftValue(BifaQuantizer * pBifaQuant, int* minvalue)
{
	if (!m_shiftedValue)
		m_shiftedValue = new int[m_nbQuantValue];

	
	for (int i=0; i<m_nbMField;i++)
	{
		m_shiftedValue[3*i] =m_shiftedValue[3*i]-minvalue[0];
		m_shiftedValue[3*i+1] =m_shiftedValue[3*i+1]-minvalue[0];
		m_shiftedValue[3*i+2] =m_shiftedValue[3*i+2]-minvalue[0];

		if (pBifaQuant->getAnimQP() 
			&& (pBifaQuant->getAnimQP()->getPnbBits() < getNbBits(m_shiftedValue[3*i])) )
		{
			char buffer[16];
			sprintf(buffer,"%d ==> %d",pBifaQuant->getAnimQP()->getPnbBits(),getNbBits(m_shiftedValue[3*i]));
			BifaError::error(1124,buffer);
			pBifaQuant->getAnimQP()->setPnbBits(getNbBits(m_shiftedValue[3*i]));
			return;
		}
		if (pBifaQuant->getAnimQP() 
			&& (pBifaQuant->getAnimQP()->getPnbBits() < getNbBits(m_shiftedValue[3*i+1])) )
		{
			char buffer[16];
			sprintf(buffer,"%d ==> %d",pBifaQuant->getAnimQP()->getPnbBits(),getNbBits(m_shiftedValue[3*i+1]));
			BifaError::error(1124,buffer);
			pBifaQuant->getAnimQP()->setPnbBits(getNbBits(m_shiftedValue[3*i+1]));
			return;
		}
		if (pBifaQuant->getAnimQP() 
			&& (pBifaQuant->getAnimQP()->getPnbBits() < getNbBits(m_shiftedValue[3*i+2])) )
		{
			char buffer[16];
			sprintf(buffer,"%d ==> %d",pBifaQuant->getAnimQP()->getPnbBits(),getNbBits(m_shiftedValue[3*i+2]));
			BifaError::error(1124,buffer);
			pBifaQuant->getAnimQP()->setPnbBits(getNbBits(m_shiftedValue[3*i+2]));
			return;
		}
	}
	
}


void RotationAnimationField::encodeValue(BifaQuantizer * pBifaQuant, bool flush)
{
	if (!m_encodedValue)
	{
		m_encodedValue = new int[m_nbQuantValue];
		m_nbaacbits  = new int[m_nbQuantValue];
	}
	for( int i=0; i< m_nbMField; i++)
	{
		
		int bits=0;
		m_encodedValue[3*i] = pBifaQuant->encodeAndUpdate(m_shiftedValue[3*i],0,1,bits);
		m_nbaacbits[3*i] = bits;
		m_encodedValue[3*i+1] = pBifaQuant->encodeAndUpdate(m_shiftedValue[3*i+1],0,1,bits);
		m_nbaacbits[3*i+1] = bits;
		
		m_encodedValue[3*i+2] = pBifaQuant->encodeAndUpdate(m_shiftedValue[3*i+2],0,1,bits,(i==(m_nbMField-1))&&flush);
		m_nbaacbits[3*i+2] = bits;
	}

}


void RotationAnimationField::write(FILE *file)
{
	AnimationField::write(file);
	fprintf(file,"RotationAnim \tmaskId (%d) ",m_maskID);
	if (m_value)
	{
		for (int i =0; i<m_nbMField; i++)
			fprintf(file,"Values %.3f %.3f %.3f %.3f ",m_value[4*i],m_value[4*i+1],m_value[4*i+2],m_value[4*i+3]);
	}

	if (m_quantValue)
	{
		for (int i =0; i<m_nbMField; i++)
			fprintf(file,"\tQuant Val %d %d %d ",m_quantValue[3*i],m_quantValue[3*i+1],m_quantValue[3*i+2]);
	}
	if (!isIntra())
	{
		
		if (m_shiftedValue)
		{
			for (int i =0; i<m_nbMField; i++)
				fprintf(file,"\tShifted Val %d %d %d ",m_shiftedValue[3*i],m_shiftedValue[3*i+1],m_shiftedValue[3*i+2]);
		}
		
		if (m_encodedValue)
		{
			for (int i =0; i<m_nbMField; i++)
				fprintf(file,"\tEncoded Val %d(%d) %d(%d) %d(%d) ",m_encodedValue[3*i],m_nbaacbits[3*i] ,m_encodedValue[3*i+1],m_nbaacbits[3*i+1],m_encodedValue[3*i+2],m_nbaacbits[3*i+2],1);
		}
	}
	fprintf(file,"\n");
	
}


void RotationAnimationField::write(BinaryWriter * binWriter)
{
	AnimationField::write(binWriter);
	
	char buffer[64];

	if (isIntra())
	{

		
		for (int i=0;i<m_nbMField;i++)
		{
			
			binWriter->write((unsigned int)m_orientation[i],2,"orientation");
			binWriter->write((unsigned int)m_quantValue[3*i],m_nbBits,"quantValue");
			binWriter->write((unsigned int)m_quantValue[3*i+1],m_nbBits,"quantValue");
			binWriter->write((unsigned int)m_quantValue[3*i+2],m_nbBits,"quantValue");

			
			sprintf(buffer,"\t\t\t\t\t\t\t\t\t\t%.3f",m_value[4*i]);
			binWriter->log(buffer,1);
			sprintf(buffer,"\t\t\t\t\t\t\t\t\t\t%.3f",m_value[4*i+1]);
			binWriter->log(buffer,1);
			sprintf(buffer,"\t\t\t\t\t\t\t\t\t\t%.3f",m_value[4*i+2]);
			binWriter->log(buffer,1);
			sprintf(buffer,"\t\t\t\t\t\t\t\t\t\t%.3f",m_value[4*i+3]);
			binWriter->log(buffer,1);
		}
	}
	else
	{
		for (int i=0;i<m_nbMField;i++)
		{
			if (m_encodedValue)
			{
				
				binWriter->writeEncoded((unsigned int)m_encodedValue[3*i],m_nbaacbits[3*i],"encodedValue","",1);
				binWriter->writeEncoded((unsigned int)m_encodedValue[3*i+1],m_nbaacbits[3*i+1],"encodedValue","",1);
				binWriter->writeEncoded((unsigned int)m_encodedValue[3*i+2],m_nbaacbits[3*i+2],"encodedValue","",1);
				
				
				sprintf(buffer,"\t\t\t\t\t\t\t\t\t\t%.3f",m_value[4*i]);
				binWriter->log(buffer,1);
				sprintf(buffer,"\t\t\t\t\t\t\t\t\t\t%.3f",m_value[4*i+1]);
				binWriter->log(buffer,1);
				sprintf(buffer,"\t\t\t\t\t\t\t\t\t\t%.3f",m_value[4*i+2]);
				binWriter->log(buffer,1);
				sprintf(buffer,"\t\t\t\t\t\t\t\t\t\t%.3f",m_value[4*i+3]);
				binWriter->log(buffer,1);
			}
		}
	}
}


void RotationAnimationField::nbBitsToWrite(int &bits)
{
	AnimationField::nbBitsToWrite(bits);
	if (isIntra())
	{
		
		for (int i=0;i<m_nbMField;i++)
		{
			
			bits+=2;
			bits+=(3*m_nbBits);
		}
	}
	else
	{
		for (int i=0;i<m_nbMField;i++)
		{
			
			bits+=m_nbaacbits[3*i];
			bits+=m_nbaacbits[3*i+1];
			bits+=m_nbaacbits[3*i+2];
		}
	}

}


void RotationAnimationField::parse(DOM_Element* node)
{
	
}
