


#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "define.h"
#include "BifaError.h"

#include "XmlParser.h"
#include <dom/DOM.hpp>

#include "BinaryWriter.h"
#include "AnimQP.h"
#include "BifaQuantizer.h"

#include "Size2DAnimationField.h"






Size2DAnimationField::Size2DAnimationField(AnimationFrameData* AFD, BifaQuantizer* bquant)  : AnimationField(AFD,bquant)
{
	m_value = NULL;
}


Size2DAnimationField::~Size2DAnimationField()
{
	if (m_value)
		delete [] m_value;
}


void Size2DAnimationField::setValue(float * values, int nbValue)
{
	
	if ((nbValue%2) != 0 )
	{
		char buffer[32];
		sprintf(buffer,"type : %d nbValue : %d",12,nbValue);
		BifaError::error(2017,buffer);
		return;
	}
	
	if (m_bifaquantizer && m_bifaquantizer->getAnimQP()
		&& m_bifaquantizer->getAnimQP()->getLowerBound()
		&& m_bifaquantizer->getAnimQP()->getUpperBound() )
	{
		float min = m_bifaquantizer->getAnimQP()->getLowerBound()[0];
		float max = m_bifaquantizer->getAnimQP()->getUpperBound()[0];
		
		for (int i=0; i<nbValue;i++)
		{ 
			if (values[i]<min || values[i]>max)
			{
				char buffer[16];
				sprintf(buffer,"type :%d : %.3f",12,values[i]);
				BifaError::error(2018,buffer);
				return;
			}
		}
	}
	
	m_nbMField = nbValue/2;


	m_value = new float[nbValue];
	memcpy(m_value,values,sizeof(float) * nbValue);
	m_nbValue = nbValue;

	TRACEP("\t\t\t\t");
	for (int i=0; i<m_nbValue; i++)
		TRACEP("%f ",m_value[i]);
	TRACEP("\n");
}


float* Size2DAnimationField::getValue()
{
	return m_value;
}


int Size2DAnimationField::getNbValue()
{
	return m_nbValue;
}


void Size2DAnimationField::quantizeValue(BifaQuantizer * pBifaQuant)
{
	
	float *minVal = pBifaQuant->getAnimQP()->getLowerBound();
	float *maxVal = pBifaQuant->getAnimQP()->getUpperBound();

	
	m_nbBits = pBifaQuant->getAnimQP()->getInbBits();

	m_quantValue = new int[m_nbValue];
	m_nbQuantValue = m_nbValue;
	
	for (int i=0; i<m_nbMField;i++)
	{
		m_quantValue[2*i] = quantizeFloat(minVal[0],maxVal[0],m_nbBits,m_value[2*i]);
		m_quantValue[2*i+1] = quantizeFloat(minVal[0],maxVal[0],m_nbBits,m_value[2*i+1]);
	
		
		m_value[2*i] = unQuantizeFloat(minVal[0],maxVal[0],m_nbBits,m_quantValue[2*i]);
		m_value[2*i+1] = unQuantizeFloat(minVal[0],maxVal[0],m_nbBits,m_quantValue[2*i+1]);

	}
	
}


void Size2DAnimationField::deltaValue(AnimationField * pPrec)
{
	if (!m_shiftedValue)
		m_shiftedValue = new int[m_nbQuantValue];
	
	
	m_shiftedValue[0] = m_quantValue[0]-((Size2DAnimationField*)pPrec)->getQuantValue()[2*(m_nbMField-1)];
	m_shiftedValue[1] = m_quantValue[1]-((Size2DAnimationField*)pPrec)->getQuantValue()[2*(m_nbMField-1)+1];

	
	for (int i=1; i<m_nbMField; i++)
	{
		m_shiftedValue[2*i] = m_quantValue[2*i]-m_quantValue[2*(i-1)];
		m_shiftedValue[2*i+1] = m_quantValue[2*i+1]-m_quantValue[2*(i-1)+1];
	}
}


void Size2DAnimationField::calcMinValue(int ** minValue)
{
	
	if (!*minValue)
	{
		
		*minValue = new int[1];
	
		
		for (int i=0; i<m_nbMField;i++)
		{
			(*minValue)[0] = m_shiftedValue[2*i];
			(*minValue)[0] = m_shiftedValue[2*i+1]<(*minValue)[0]?m_shiftedValue[2*i+1]:(*minValue)[0];
		}
	}
	else
	{
		
		for (int i=0; i<m_nbMField;i++)
		{
			(*minValue)[0] = m_shiftedValue[2*i]<(*minValue)[0]?m_shiftedValue[2*i]:(*minValue)[0];
			(*minValue)[0] = m_shiftedValue[2*i+1]<(*minValue)[0]?m_shiftedValue[2*i+1]:(*minValue)[0];
		}
	}
}


void Size2DAnimationField::shiftValue(BifaQuantizer * pBifaQuant, int* minvalue)
{
	if (!m_shiftedValue)
		m_shiftedValue = new int[m_nbQuantValue];

	
	for (int i=0; i<m_nbMField;i++)
	{
		m_shiftedValue[2*i] =m_shiftedValue[2*i]-minvalue[0];
		m_shiftedValue[2*i+1] =m_shiftedValue[2*i+1]-minvalue[0];
	
		if (pBifaQuant->getAnimQP() 
			&& (pBifaQuant->getAnimQP()->getPnbBits() < getNbBits(m_shiftedValue[2*i])) )
		{
			char buffer[16];
			sprintf(buffer,"%d ==> %d",pBifaQuant->getAnimQP()->getPnbBits(),getNbBits(m_shiftedValue[2*i]));
			BifaError::error(1124,buffer);
			pBifaQuant->getAnimQP()->setPnbBits(getNbBits(m_shiftedValue[2*i]));
			return;
		}
		if (pBifaQuant->getAnimQP() 
			&& (pBifaQuant->getAnimQP()->getPnbBits() < getNbBits(m_shiftedValue[2*i+1])) )
		{
			char buffer[16];
			sprintf(buffer,"%d ==> %d",pBifaQuant->getAnimQP()->getPnbBits(),getNbBits(m_shiftedValue[2*i+1]));
			BifaError::error(1124,buffer);
			pBifaQuant->getAnimQP()->setPnbBits(getNbBits(m_shiftedValue[2*i+1]));
			return;
		}
	}
	
}


void Size2DAnimationField::encodeValue(BifaQuantizer * pBifaQuant, bool flush)
{
	if (!m_encodedValue)
	{
		m_encodedValue = new int[m_nbQuantValue];
		m_nbaacbits  = new int[m_nbQuantValue];
	}
	
	for( int i=0; i< m_nbMField; i++)
	{
		int bits=0;
		m_encodedValue[2*i] = pBifaQuant->encodeAndUpdate(m_shiftedValue[2*i],0,1,bits);
		m_nbaacbits[2*i] = bits;
		
		m_encodedValue[2*i+1] = pBifaQuant->encodeAndUpdate(m_shiftedValue[2*i+1],0,1,bits,(i==(m_nbMField-1))&&flush);
		m_nbaacbits[2*i+1] = bits;
	}

}


void Size2DAnimationField::write(FILE *file)
{
	fprintf(file,"Size2DAnim \t: maskId (%d)\t: Value : ",m_maskID);
	if (m_value)
	{
		for (int i =0; i<m_nbMField; i++)
			fprintf(file,"Field %d : %.3f %.3f \t: ",i,m_value[2*i],m_value[2*i+1]);
	}

	if (m_quantValue)
	{
		for (int i =0; i<m_nbMField; i++)
			fprintf(file,"Quant Val %d %d \t: ",m_quantValue[2*i],m_quantValue[2*i+1]);
	}
	if (!isIntra())
	{
		
		if (m_shiftedValue)
		{
			for (int i =0; i<m_nbMField; i++)
				fprintf(file,"Shifted Val %d %d ",m_shiftedValue[2*i],m_shiftedValue[2*i+1]);
		}
		
		if (m_encodedValue)
		{
			for (int i =0; i<m_nbMField; i++)
				fprintf(file,"Encoded Val %d(%d) %d(%d) ",m_encodedValue[2*i],m_nbaacbits[2*i] ,m_encodedValue[2*i+1],m_nbaacbits[2*i+1],1);
		}
	}
	fprintf(file,"\n");

}


void Size2DAnimationField::write(BinaryWriter * binWriter)
{

	AnimationField::write(binWriter);

	char buffer[64];

	if (isIntra())
	{
		for (int i=0;i<m_nbMField;i++)
		{
			
			binWriter->write((unsigned int)m_quantValue[2*i],m_nbBits,"quantValue");
			binWriter->write((unsigned int)m_quantValue[2*i+1],m_nbBits,"quantValue");
			
			
			sprintf(buffer,"\t\t\t\t\t\t\t\t\t\t%.3f",m_value[2*i]);
			binWriter->log(buffer,1);
			sprintf(buffer,"\t\t\t\t\t\t\t\t\t\t%.3f",m_value[2*i+1]);
			binWriter->log(buffer,1);


		}
	}
	else
	{
		
		for( int i=0; i< m_nbMField; i++)
		{
			if (m_encodedValue)
			{
				
				binWriter->writeEncoded((unsigned int)m_encodedValue[2*i],m_nbaacbits[i],"encodedValue","",1);
				binWriter->writeEncoded((unsigned int)m_encodedValue[2*i+1],m_nbaacbits[i+1],"encodedValue","",1);
				
				
				sprintf(buffer,"\t\t\t\t\t\t\t\t\t\t%.3f",m_value[2*i]);
				binWriter->log(buffer,1);
				sprintf(buffer,"\t\t\t\t\t\t\t\t\t\t%.3f",m_value[2*i+1]);
				binWriter->log(buffer,1);


			}
		}
	}
		

}


void Size2DAnimationField::nbBitsToWrite(int &bits)
{
	AnimationField::nbBitsToWrite(bits);
	if (isIntra())
	{
		
		for (int i=0;i<m_nbQuantValue;i++)
		{
			bits+=m_nbBits;
		}
	}
	else
	{
		for( int i=0; i< m_nbMField; i++)
		{
			
			if (m_encodedValue)
			{
				bits+=m_nbaacbits[2*i];
				bits+=m_nbaacbits[2*i+1];
			}
		}
	}
}


void Size2DAnimationField::parse(DOM_Element* node)
{
	
}
