


#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <iostream.h>

#include "define.h"
#include "BifaError.h"


#include <util/PlatformUtils.hpp>
#include <util/XMLString.hpp>
#include <util/XMLUniDefs.hpp>
#include <framework/XMLFormatter.hpp>
#include <util/TranscodingException.hpp>
#include <dom/DOM_DOMException.hpp>
#include <parsers/DOMParser.hpp>
#include <dom/DOM.hpp>

#include "AnimationFrame.h"
#include "AnimationFrameData.h"
#include "AnimationMask.h"
#include "AnimQP.h"
#include "BifsConfig.h"
#include "ElementaryMask.h"
#include "FrameRate.h"
#include "InitialFieldMask.h"
#include "SkipFrame.h"
#include "BifaQuantizer.h"
#include "Position3DAnimationField.h"
#include "Position2DAnimationField.h"
#include "ColorAnimationField.h"
#include "AngleAnimationField.h"
#include "FloatAnimationField.h"
#include "GenericFloatAnimationField.h"
#include "NormalAnimationField.h"
#include "RotationAnimationField.h"
#include "Size3DAnimationField.h"
#include "Size2DAnimationField.h"
#include "IntegerAnimationField.h"

#include "XmlParser.h"








ostream& operator<< (ostream& target, const DOMString& s)
{
    char *p = s.transcode();
    target << p;
    delete [] p;
    return target;
}


XMLFormatter& operator<< (XMLFormatter& strm, const DOMString& s)
{
    unsigned int lent = s.length();

	if (lent <= 0)
		return strm;

    XMLCh*  buf = new XMLCh[lent + 1];
    XMLString::copyNString(buf, s.rawBuffer(), lent);
    buf[lent] = 0;
    strm << buf;
    delete [] buf;
    return strm;
}









XmlParser::XmlParser()
{
	m_filename = NULL;
}


XmlParser::~XmlParser()
{
	if (m_filename)
		free(m_filename);

}


int XmlParser::init()
{
	
	try
    {
        XMLPlatformUtils::Initialize();
    }

    catch(const XMLException& toCatch)
    {
        cerr << "Error during Xerces-c Initialization.\n"
             << "  Exception message:"
             << DOMString(toCatch.getMessage()) << endl;
        return 1;
    }

	return 0;
}


void XmlParser::setFilename( const char* filename)
{
	if (m_filename)
		free(m_filename);
	m_filename = strdup(filename);
}


void XmlParser::getNbElement(DOM_Element * elmNod, int& nbAnimationFrame, int& nbBifaQuantizer)
{
	
	
	if(elmNod->getNodeName().equals(XMTAG_PAR))
		nbAnimationFrame++;
	else if (elmNod->getNodeName().equals(XMTAG_INITIALFIELDSMASK))
		nbBifaQuantizer++;

	DOM_NodeList tmpList = elmNod->getChildNodes();
	
	
	for (unsigned int index = 0; index<tmpList.getLength() ;index++)
	{
		

		getNbElement((DOM_Element*)(&(tmpList.item(index))),nbAnimationFrame,nbBifaQuantizer);
	}
}
	


void XmlParser::parse( BifsConfig * pBifsConfig, BifaQuantizer *** pppBifaQuantizer , int &nbBifaQuantizer, AnimationFrame *** pppAnimationFrame, int &nbAnimationFrame)
{
	FILE *isOpen = NULL;
	if (m_filename && (isOpen =fopen(m_filename,"r")) )
	{
		fclose(isOpen);
		
		
		DOMParser *parser = new DOMParser();
		parser->setValidationScheme(DOMParser::Val_Auto);
		
		
		bool errorsOccured = false;
		try
		{
			parser->parse(__argv[1]);
			int errorCount = parser->getErrorCount();
			if (errorCount > 0)
				errorsOccured = true;
		}
		
		catch (const XMLException& e)
		{
			cerr << "An error occured during parsing\n   Message: " << DOMString(e.getMessage()) << endl;
			errorsOccured = true;
		}
		
		
		catch (const DOM_DOMException& e)
		{
			cerr << "A DOM error occured during parsing\n   DOMException code: "
				<< e.code << endl;
			errorsOccured = true;
		}
		
		catch (...)
		{
			cerr << "An error occured during parsing\n " << endl;
			errorsOccured = true;
		}
		
		if (!errorsOccured)
		{
			DOM_Node doc = parser->getDocument();
			DOM_Node aNode = doc.getFirstChild();
			
			
			while (aNode.getNodeType() != DOM_Node::ELEMENT_NODE)
			{
				short temp = aNode.getNodeType() ;
				aNode = aNode.getNextSibling();
			}
			
			nbBifaQuantizer = 0;
			nbAnimationFrame = 0;
			int	numNode = 0;
			
			int indexBifaQuantizer = 0;
			int indexAnimationFrame = 0;
			
			
			
			getNbElement((DOM_Element*)(&aNode),nbAnimationFrame,nbBifaQuantizer);
			
			DOM_NodeList nodList = aNode.getChildNodes();
			
			
			(*pppAnimationFrame) = new AnimationFrame*[nbAnimationFrame];
			
			(*pppBifaQuantizer) = new BifaQuantizer*[nbBifaQuantizer];
			
			for (unsigned int i = 0; i<nodList.getLength() ;i++)
			{
				
				DOM_Node dummyElmNod((nodList.item(i)));
				DOM_Element *elmNod = (DOM_Element *)(&dummyElmNod);
				
				
				if (elmNod->getNodeName().equals(XMTAG_BIFSCONFIG))
				{
					
					pBifsConfig->parse(elmNod);

					
					DOM_NodeList BCNodList = elmNod->getChildNodes();
					for (unsigned int BCIndex = 0; BCIndex<BCNodList.getLength() ;BCIndex++)
					{
						DOM_Node BCdummyElmNod((BCNodList.item(BCIndex)));
						DOM_Element *BCElmNod = (DOM_Element *)(&BCdummyElmNod);
						
						
						if (BCElmNod->getNodeName().equals(XMTAG_ANIMATIONMASK))
						{
							
							TRACEP("\tnew AnimationMask( BifsConfig )\n");
							AnimationMask* tmpAnimationMask = new AnimationMask(pBifsConfig);
							
							
							DOM_NodeList EMNodList = BCElmNod->getChildNodes();
							for (unsigned int EMIndex = 0; EMIndex<EMNodList.getLength() ;EMIndex++)
							{
								DOM_Node EMdummyElmNod((EMNodList.item(EMIndex)));
								DOM_Element *EMElmNod = (DOM_Element *)(&EMdummyElmNod);
								DOMString str;
								
								
								if (EMElmNod->getNodeName().equals(XMTAG_ELEMENTARYMASK))
								{
									
									TRACEP("\t\tnew ElementaryMask( AnimationMask )\n");
									ElementaryMask * tmpElementaryMask = new ElementaryMask(tmpAnimationMask);
									
									tmpElementaryMask->setNodeID(numNode++);
									
									tmpElementaryMask->parse(EMElmNod);
									
									
									DOM_NodeList IFMNodList = EMElmNod->getChildNodes();
									for (unsigned int IFMIndex = 0; IFMIndex<IFMNodList.getLength() ;IFMIndex++)
									{
										DOM_Node IFMdummyElmNod((IFMNodList.item(IFMIndex)));
										DOM_Element *IFMElmNod = (DOM_Element *)(&IFMdummyElmNod);
										DOMString str;
										
										if (IFMElmNod->getNodeName().equals(XMTAG_INITIALFIELDSMASK))
										{
											
											TRACEP("\t\t\tnew InitialFieldsMask( ElementaryMask )\n");
											InitialFieldMask* tmpInitialFielMask = new InitialFieldMask(tmpElementaryMask);
																					
											
											DOM_NodeList AQPMNodList = IFMElmNod->getChildNodes();
											for (unsigned int AQPMIndex = 0; AQPMIndex<AQPMNodList.getLength() ;AQPMIndex++)
											{
												DOM_Node AQPdummyElmNod((AQPMNodList.item(AQPMIndex)));
												DOM_Element *AQPMElmNod = (DOM_Element *)(&AQPdummyElmNod);
												DOMString str;
												
												if (AQPMElmNod->getNodeName().equals(XMTAG_INITIALANIMQP))
												{
													
													TRACEP("\t\t\tnew InitialAnimQP( InitialFieldMask)\n");
													AnimQP * tmpAnimQP = new AnimQP();
													tmpInitialFielMask->setInitialAnimQP(tmpAnimQP);
													
													tmpAnimQP->parse(AQPMElmNod);
												
												}
											}	
											
											tmpInitialFielMask->parse(IFMElmNod);
											
											
											(*pppBifaQuantizer)[indexBifaQuantizer] = new BifaQuantizer(
												tmpElementaryMask->getNodeID(),
												tmpInitialFielMask->getID(),
												nbAnimationFrame);
											
											(*pppBifaQuantizer)[indexBifaQuantizer]->setInitialAnimQp(tmpInitialFielMask->getInitialAnimQP());
											(*pppBifaQuantizer)[indexBifaQuantizer]->setIndexList(tmpInitialFielMask->getIndexList(),tmpInitialFielMask->getNbIndex());
											(*pppBifaQuantizer)[indexBifaQuantizer]->setIsTotal(tmpInitialFielMask->getIsTotal());
											(*pppBifaQuantizer)[indexBifaQuantizer]->setRandomAccess(pBifsConfig->getRandomAccess());
											(*pppBifaQuantizer)[indexBifaQuantizer]->setType(tmpInitialFielMask->getInitialAnimQP()->getType());
											
											
											indexBifaQuantizer++;

										}
									}
								}
							}
						}
					}
					
				}
					
				
				
				else if ((DOM_Element*)(&(nodList.item(i)))->getNodeName().equals(XMTAG_PAR))
				{
					TRACEP("new par\n");
					
					float begin = 0.0f;
					DOMString str = elmNod->getAttribute(XMTATTR_BEGIN);
					if (str.length())
					{
						TRACEP("\t begin(%s)\n",str.transcode());
						sscanf(str.transcode(),"%f",&begin);
					}
					
					
					DOM_NodeList AMNodList = elmNod->getChildNodes();
					
					
					
					for (int i=0; i<indexAnimationFrame;i++)
					{
						if (begin == (*pppAnimationFrame)[i]->getBegin())
						{
							char buffer[16];
							sprintf(buffer,"begin : %.3f",begin);
							BifaError::error(2011,buffer);
							return;
						}
					}

					for (unsigned int AMIndex = 0; AMIndex<AMNodList.getLength() ;AMIndex++)
					{
						DOM_Node AMdummyElmNod((AMNodList.item(AMIndex)));
						DOM_Element *AMElmNod = (DOM_Element *)(&AMdummyElmNod);
						DOMString str;
						
						
						if (AMElmNod->getNodeName().equals(XMTAG_ANIMATIONFRAME))
						{
							
							TRACEP("\tnew AnimationFrame( par )\n");
							(*pppAnimationFrame)[indexAnimationFrame] = new AnimationFrame(begin);
							(*pppAnimationFrame)[indexAnimationFrame]->setNumNode(numNode);
							
							
							(*pppAnimationFrame)[indexAnimationFrame]->parse(AMElmNod);
							
							
							DOM_NodeList FRNodList = AMElmNod->getChildNodes();
							for (unsigned int FRIndex = 0; FRIndex<FRNodList.getLength() ;FRIndex++)
							{
								DOM_Node FRdummyElmNod((FRNodList.item(FRIndex)));
								DOM_Element *FRElmNod = (DOM_Element *)(&FRdummyElmNod);
								DOMString str;
								
								
								if (FRElmNod->getNodeName().equals(XMTAG_FRAMERATE))
								{
									
									TRACEP("\t\tnew FrameRate( AnimationFrame)\n");
									FrameRate * tmpFrameRate = new FrameRate;
									
									tmpFrameRate->parse(FRElmNod);
									
									(*pppAnimationFrame)[indexAnimationFrame]->setFrameRate(tmpFrameRate);
									
								}
								
								
								if (FRElmNod->getNodeName().equals(XMTAG_ANIMATIONFRAMEDATA))
								{
									
									TRACEP("\t\tnew AnimationFrameData( AnimationFrame)\n");
									AnimationFrameData * tmpAnimationFrameData = new AnimationFrameData((*pppAnimationFrame)[indexAnimationFrame]);
									
									tmpAnimationFrameData->parse(FRElmNod);
									
									
									DOM_NodeList AFDNodList = FRElmNod->getChildNodes();
									for (unsigned int AFDIndex = 0; AFDIndex<AFDNodList.getLength() ;AFDIndex++)
									{
										DOM_Node AFDdummyElmNod((AFDNodList.item(AFDIndex)));
										DOM_Element *AFDElmNod = (DOM_Element *)(&AFDdummyElmNod);
										DOMString str;
										
										
										if (AFDElmNod->getNodeName().equals(XMTAG_ANIMATIONFIELD))
										{
											
											TRACEP("\t\t\tnew AnimationField( AnimationFrameData )\n");
											AnimationField * tmpAnimationField = NULL;
											
											
											
											int sovtype =0;
											int id = 0;
											int maskID;
											int	indexBifaQuantizer = 0;
											
											
											str = AFDElmNod->getAttribute(XMTATTR_ID);
											if (str.length())
											{
												TRACEP("\t\t\t\t id(%s)\n",str.transcode());
												sscanf(str.transcode(),"%d",&id);
											}
											
											
											str = AFDElmNod->getAttribute(XMTATTR_MASKID);
											if (str.length())
											{
												TRACEP("\t\t\t\t maskID(%s)\n",str.transcode());
												sscanf(str.transcode(),"%d",&maskID);
											}											
											
											
											for (int indBifa =0; indBifa<nbBifaQuantizer; indBifa++)
											{
												if ( ((*pppBifaQuantizer)[indBifa]->getNodeID() == tmpAnimationFrameData->getMaskID())
													&& ((*pppBifaQuantizer)[indBifa]->getID() == maskID) )
												{
													
													indexBifaQuantizer = indBifa;
													
													sovtype = (*pppBifaQuantizer)[indBifa]->getAnimationType();
													break;
												}
											}
											
											
											
											
											DOM_NodeList AQPMNodList = AFDElmNod->getChildNodes();
											AnimQP * tmpAnimQP = NULL;
											for (unsigned int AQPMIndex = 0; AQPMIndex<AQPMNodList.getLength() ;AQPMIndex++)
											{
												DOM_Node AQPdummyElmNod((AQPMNodList.item(AQPMIndex)));
												DOM_Element *AQPMElmNod = (DOM_Element *)(&AQPdummyElmNod);
												DOMString str;
												
												
												if (AQPMElmNod->getNodeName().equals(XMTAG_ANIMQP))
												{
													
													TRACEP("\t\t\t\tnew AnimQP( AnimationField)\n");
													tmpAnimQP = new AnimQP;
													
													tmpAnimQP->parse(AQPMElmNod);
													
													sovtype = tmpAnimQP->getType();
												}
												
											}
											
											
											
											str = AFDElmNod->getAttribute(XMTATTR_VALUES);
											int nbValues =0;
											char * buffer = NULL;
											char * buff = NULL;
											if (str.length())
											{
												TRACEP("\t\t\t\t values(%s)\n",str.transcode());
												buffer =strdup(str.transcode());
												XmlParser::getCleanBuffer(buffer,nbValues);
												
												buff = buffer;
												
											}
											
											switch( sovtype)
											{
											case 1: 
												{
													tmpAnimationField = new Position3DAnimationField(tmpAnimationFrameData,(*pppBifaQuantizer)[indBifa]);
													float * values = new float[nbValues];
													char *tmp = buff;
													for (int i=0; i<nbValues; i++)
													{
														while(*tmp && (*tmp != ' ')) 
															tmp++;
														*tmp = '\0';
														sscanf(buff,"%f",&(values[i]));
														buff = ++tmp;
													}
													((Position3DAnimationField*)tmpAnimationField)->setValue(values,nbValues);
													delete values;
													break;
												}
											case 2: 
												{
													tmpAnimationField = new Position2DAnimationField(tmpAnimationFrameData,(*pppBifaQuantizer)[indBifa]);
													float * values = new float[nbValues];
													char *tmp = buff;
													for (int i=0; i<nbValues; i++)
													{
														while(*tmp && (*tmp != ' ')) 
															tmp++;
														*tmp = '\0';
														sscanf(buff,"%f",&(values[i]));
														buff = ++tmp;
													}
													((Position2DAnimationField*)tmpAnimationField)->setValue(values,nbValues);
													delete values;
													break;
												}
											case 4: 
												{
													tmpAnimationField = new ColorAnimationField(tmpAnimationFrameData,(*pppBifaQuantizer)[indBifa]);
													float * values = new float[nbValues];
													char *tmp = buff;
													for (int i=0; i<nbValues; i++)
													{
														while(*tmp && (*tmp != ' ')) 
															tmp++;
														*tmp = '\0';
														sscanf(buff,"%f",&(values[i]));
														buff = ++tmp;
													}
													((ColorAnimationField*)tmpAnimationField)->setValue(values,nbValues);
													delete values;
													break;
												}
											case 6: 
												{
													tmpAnimationField = new AngleAnimationField(tmpAnimationFrameData,(*pppBifaQuantizer)[indBifa]);
													float * values = new float[nbValues];
													char *tmp = buff;
													for (int i=0; i<nbValues; i++)
													{
														while(*tmp && (*tmp != ' ')) 
															tmp++;
														*tmp = '\0';
														sscanf(buff,"%f",&(values[i]));
														buff = ++tmp;
													}
													((AngleAnimationField*)tmpAnimationField)->setValue(values,nbValues);
													delete values;
													break;
												}
											case 7: 
												{
													tmpAnimationField = new FloatAnimationField(tmpAnimationFrameData,(*pppBifaQuantizer)[indBifa]);
													float * values = new float[nbValues];
													char *tmp = buff;
													for (int i=0; i<nbValues; i++)
													{
														while(*tmp && (*tmp != ' ')) 
															tmp++;
														*tmp = '\0';
														sscanf(buff,"%f",&(values[i]));
														buff = ++tmp;
													}
													((FloatAnimationField*)tmpAnimationField)->setValue(values,nbValues);
													delete values;
													break;
												}
											case 8: 
												{
													tmpAnimationField = new GenericFloatAnimationField(tmpAnimationFrameData,(*pppBifaQuantizer)[indBifa]);
													float * values = new float[nbValues];
													char *tmp = buff;
													for (int i=0; i<nbValues; i++)
													{
														while(*tmp && (*tmp != ' ')) 
															tmp++;
														*tmp = '\0';
														sscanf(buff,"%f",&(values[i]));
														buff = ++tmp;
													}
													((GenericFloatAnimationField*)tmpAnimationField)->setValue(values,nbValues);
													delete values;
													break;
												}
											case 9: 
												{
													tmpAnimationField = new NormalAnimationField(tmpAnimationFrameData,(*pppBifaQuantizer)[indBifa]);
													float * values = new float[nbValues];
													char *tmp = buff;
													for (int i=0; i<nbValues; i++)
													{
														while(*tmp && (*tmp != ' ')) 
															tmp++;
														*tmp = '\0';
														sscanf(buff,"%f",&(values[i]));
														buff = ++tmp;
													}
													((NormalAnimationField*)tmpAnimationField)->setValue(values,nbValues);
													delete values;
													break;
												}
											case 10: 
												{
													tmpAnimationField = new RotationAnimationField(tmpAnimationFrameData,(*pppBifaQuantizer)[indBifa]);
													float * values = new float[nbValues];
													char *tmp = buff;
													for (int i=0; i<nbValues; i++)
													{
														while(*tmp && (*tmp != ' ')) 
															tmp++;
														*tmp = '\0';
														sscanf(buff,"%f",&(values[i]));
														buff = ++tmp;
													}
													((RotationAnimationField*)tmpAnimationField)->setValue(values,nbValues);
													delete values;
													break;
												}
											case 11: 
												{
													tmpAnimationField = new Size3DAnimationField(tmpAnimationFrameData,(*pppBifaQuantizer)[indBifa]);
													float * values = new float[nbValues];
													char *tmp = buff;
													for (int i=0; i<nbValues; i++)
													{
														while(*tmp && (*tmp != ' ')) 
															tmp++;
														*tmp = '\0';
														sscanf(buff,"%f",&(values[i]));
														buff = ++tmp;
													}
													((Size3DAnimationField*)tmpAnimationField)->setValue(values,nbValues);
													delete values;
													break;
												}
											case 12: 
												{
													tmpAnimationField = new Size2DAnimationField(tmpAnimationFrameData,(*pppBifaQuantizer)[indBifa]);
													float * values = new float[nbValues];
													char *tmp = buff;
													for (int i=0; i<nbValues; i++)
													{
														while(*tmp && (*tmp != ' ')) 
															tmp++;
														*tmp = '\0';
														sscanf(buff,"%f",&(values[i]));
														buff = ++tmp;
													}
													((Size2DAnimationField*)tmpAnimationField)->setValue(values,nbValues);
													delete values;
													break;
												}
											case 13: 
												{
													tmpAnimationField = new IntegerAnimationField(tmpAnimationFrameData,(*pppBifaQuantizer)[indBifa]);
													int * values = new int[nbValues];
													char *tmp = buff;
													for (int i=0; i<nbValues; i++)
													{
														while(*tmp && (*tmp != ' ')) 
															tmp++;
														*tmp = '\0';
														sscanf(buff,"%d",&(values[i]));
														buff = ++tmp;
													}
													((IntegerAnimationField*)tmpAnimationField)->setValue(values,nbValues);
													delete values;
													break;
												}
											default:
												{
													BifaError::error(2021);
												}
											}
											
											if (tmpAnimQP)
												tmpAnimationField->setAnimQP(tmpAnimQP);
											tmpAnimationField->setID(id);
											tmpAnimationField->setMaskID(maskID);
											
											(*pppBifaQuantizer)[indBifa]->addAnimationField(tmpAnimationField);
											free(buffer);
											
										}
									}
									
									
									
									
									int index = tmpAnimationFrameData->getMaskID();
									bool erreur = false;
									ElementaryMask * elemMask = pBifsConfig->getAnimationMask(0)->getElementaryMask(index);
									if (elemMask->getNbInitialFieldMask() != tmpAnimationFrameData->getNbAnimationField())
										erreur = true;
									if (!erreur)
									{
										for (int i=0; i<tmpAnimationFrameData->getNbAnimationField();i++)
										{
											if (tmpAnimationFrameData->getAnimationField(i)->getMaskID() != 
												elemMask->getInitialFieldMask(i)->getID())
												erreur=true;
										}
									}
									if (erreur)
									{
										char buffer[16];
										sprintf(buffer,"maskID : %d",tmpAnimationFrameData->getMaskID());
										BifaError::error(2015,buffer);
										return;
									}
									
								}
							}
							
							
							indexAnimationFrame++;
						}
					}
				}
			}
		}
		else
		{
			BifaError::error(2022);
		}
		delete parser;
		
	}
	else
	{
		
		nbAnimationFrame = 0;
		nbBifaQuantizer = 0;
		BifaError::error(2022);
	}
	
}


void XmlParser::getCleanBuffer( char * str, int& nbElm)
{
	char * buffer = strdup(str);
	
	nbElm = 0;

	
	while(*buffer == ' ')
		buffer++;

	
	char* p1=buffer;
	char* p2=buffer;

	while(*p1++)
	{
		if ( (*p1==' ') && (p1[1] ==' '))
			strcpy(p1,&(p1[1]));
	}

	
	p1-=2;
	
	if (*p1 == ' ')
		*p1 = '\0';

	
	p1=buffer;
	nbElm=1;
	while(*p1)
	{
		if (*p1++ == ' ')
			nbElm++;
	}
	
	strcpy(str,buffer);
	free(buffer);
}
