/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  BIFS Encoder
FILE:       BifsEnc.cpp
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Implementation of BIFSEncoder, the class that encodes date into
			bits.
-------------------------------------------------------------------------*/
//$Log: BifsEnc.cpp,v $
//Revision 1.1.1.2  2006/04/14 14:11:07  hill
//no message
//
//Revision 1.1.1.1  2003/12/24 00:42:04  mikeseven
//Initial release of the CVS repository for MPEG-4 Systems code
//after the code was corrupted due to reorganization and human 
//errors.
//
//Revision 1.4  2003/07/04 18:00:08  mikeseven
//no message
//
//Revision 1.3  2003/04/14 16:45:34  jean_lf
//RAP signal support, BIFSv2 encode fix
//
//Revision 1.2  2003/01/20 17:58:38  jean_lf
//no message
//
//Revision 1.1.1.1  2002/07/18 00:59:52  mikeseven
//Initial check in of July 17, 2002 version
//
//Revision 1.2.2.1  2000/03/10 15:59:30  Alex Cotarmanac'h
//deuxime envoi
//
//Revision 1.2  2000/03/03 18:41:20  Alex Cotarmanac'h
//Version 1 -- Alexandre Cotarmanac_h
//
#include "defs.h"
#include "BifsEnc.h"
#include "ODStrEnc.h"
#include "NodeFld.h"
#include "Token.h"
#include "bifs.h"
#include "Proto.h"
#include "SFScript.h"  // BF 043101

#include "Quantize.h" // MBS 110698: for EfficientFloat coding
#include "NodeCode.h" //AC
/*-----------------------------------------------------------------
BIFSEncoder constructor
-----------------------------------------------------------------*/
BIFSEncoder :: BIFSEncoder (
	FILE *fBifs,
	FILE *fList,
    FILE *fLogs, // mp conformance
	LPCSTR szType,
	int flags,
	ODStreamEncoder *pODEncoder,
	ODStreamEncoder *pOCIEncoder
) :
	SysCoder (fBifs, fList, szType, flags, 0, fLogs),
	m_scene (TRUE),
//AC 20000227 : Quantization 14 stuff.
		m_coordActive(false),
		m_coordStore(false),
		m_lengthCoord(false),
		m_IdToLength(),
		m_scriptCoding(false)
//AC end
{
	m_pODEncoder = pODEncoder;
	m_pOCIEncoder = pOCIEncoder;
	m_pProto = NULL;
//	m_bDontWrite = FALSE;
//AC 19991121 : setPredictive.
	setPredictive(flags);
//AC end

	m_pCurrentQP=NULL;
	m_pGlobalQP = NULL;
}

BIFSEncoder :: ~BIFSEncoder()
{
	while (m_globalQPList.GetLength()) {
		delete m_globalQPList[0];
		m_globalQPList.DeleteFrom(0);
	}
}


void BIFSEncoder :: SetCurrentQP(MediaObject *qp, BOOL bIsGlobal) 
{
	if (bIsGlobal) {
		m_pGlobalQP = qp; 
	} else {
		m_pCurrentQP = qp; 
	}
}
MediaObject *BIFSEncoder :: GetCurrentQP() 
{ 
	if (m_pCurrentQP) return m_pCurrentQP; 
	return m_pGlobalQP;
}

BOOL BIFSEncoder :: hasLocalQP() 
{ 
	//globalQP ignore their isLocal field since they're global
	return (m_pCurrentQP ? ((Bool) *((SF<Bool> *) m_pCurrentQP->GetField(0))) : FALSE) ; 
}

/*-----------------------------------------------------------------
Write the encoded data to another buffer
by MBS
-----------------------------------------------------------------*/
void BIFSEncoder :: Write (SysCoder *pCoder)
{
	Align();
	pCoder -> EncodeInt (m_buffer.GetLength (), 32, "value.length32");
	for (int i = 0; i < m_buffer.GetLength (); i++)
		pCoder -> EncodeInt (m_buffer[i], 8, "char"); // mp conformance
	m_buffer.Cut (0);
}


#ifdef ENCODER
/*-----------------------------------------------------------------
Read and encode a scene
-----------------------------------------------------------------*/
void BIFSEncoder :: Encode ()
{
	/*-------------------------------------------------------------
	The following overloading is required for encoding script nodes
	-------------------------------------------------------------*/
	OVERLOAD_NODE (ScriptImp);		// SCRIPT
	OVERLOAD_NODE (Ellipse); // MBS 062703: to void name clash with graphic API that defines Ellipse class.

	try
	{
		UpdateScene ();
	}
	catch (ExceptionEndOfFile)
	{
		nextToken.Error ("Unexpected end of file");
	}
}

/*-----------------------------------------------------------------
Encode Routes
-----------------------------------------------------------------*/
void BIFSEncoder :: EncodeRoutesDescription ()
{
	BOOL bMoreRoutes;

	EncodeInt (1, 1, "listDescription");
	do
	{
		EncodeRoute ();
		bMoreRoutes = nextToken == TID_ROUTE;
		EncodeInt (bMoreRoutes, 1, "moreRoutes");
	}
	while (bMoreRoutes);
}

/*-----------------------------------------------------------------
Encode one Route
-----------------------------------------------------------------*/
void BIFSEncoder :: EncodeRoute ()
{
	BOOL bIsUpdateable;

	if (nextToken == TID_DEF)
		bIsUpdateable = TRUE;
	else
	{	nextToken.Scan ();
		bIsUpdateable = nextToken == TID_DEF;
	}
	EncodeInt (bIsUpdateable, 1, "isUpdateable");
	if (bIsUpdateable)
	{	int nID = 0;
		nextToken.Scan ();
		if (nextToken == TID_NAME)
		{	nID = nextToken.GetDefinedRoute ();
			if (nID < 0)
				nID = nextToken.AddPredefinedRoute ();
		}
		else
			nextToken.Error ("Unexpected symbol");
		EncodeInt (nID, 10, "RouteID");
		if (GetFlags () & CODENAMES)
			EncodeString (nextToken.GetName ());
		nextToken.Scan ();
		if (nextToken == TID_ROUTE)
			nextToken.Scan ();
	}

	EncodeRouteDetails ();
}

/*-----------------------------------------------------------------
Encode update commands
-----------------------------------------------------------------*/
void BIFSEncoder :: UpdateScene ()
{
	/*-----------------------------------------------------------------
	Loop for each update frame
	-----------------------------------------------------------------*/
	while (nextToken != TID_EOF)
	{
		DWORD dwTS = 0;
		switch (nextToken) {
		case TID_FILE:
			nextToken.Scan ();
			if (nextToken != TID_STRING)
				nextToken.Error ("File name expected");
			else
			{	FILE *fNewOutput = fopen (nextToken.GetString (), "wb");
				if (fNewOutput == NULL)
					nextToken.Error ("Cannot open file");
				else
					ReplaceOutputFile (fNewOutput);
				nextToken.Scan ();
			}
			break;
		case TID_RAP:
			m_isRAP = TRUE;
			nextToken.Scan ();
			break;
		case TID_AT:
			nextToken.Scan ();
			if (nextToken != TID_INTEGER)
				nextToken.Error ("Time (in milliseconds) expected");
			dwTS = nextToken.GetInt ();
		default:
			EncodeAU (dwTS);
		}
	}
}

/*-----------------------------------------------------------------
Encode one access unit
-----------------------------------------------------------------*/
void BIFSEncoder :: EncodeAU (DWORD dwTS)
{
	if (GetFlags () & TRACE_VERBOSE)
		fprintf (m_fList, "------------AT %d-----------\n", dwTS);
	if (nextToken == TID_LEFT_BRACE)
	{	nextToken.Scan ();
		while (nextToken != TID_RIGHT_BRACE)
		{	if (nextToken == TID_EOF)
			{	nextToken.Error ("Unexpected end of file");
				break;
			}
			if (EncodeUpdateCommand ())
				EncodeInt (nextToken != TID_RIGHT_BRACE, 1, "continue");
		}
		nextToken.Scan ();
	}
	else if (EncodeUpdateCommand ())
		EncodeInt (0, 1, "continue");
	Flush (dwTS);
	if (m_pODEncoder && m_pODEncoder ->GetDataLength () > 0)
		m_pODEncoder -> Flush (dwTS);
	if (m_pOCIEncoder && m_pOCIEncoder ->GetDataLength () > 0)
		m_pOCIEncoder -> Flush (dwTS);
}

/*-----------------------------------------------------------------
Encode an INSERT command
-----------------------------------------------------------------*/
void BIFSEncoder :: InsertCommand ()
{
	nextToken.Scan ();
	EncodeInt (0, 2, "code", "InsertionCommand"); // mp conformance
	switch (nextToken) {
	case TID_AT:
		InsertToField (FALSE);
		break;
	case TID_ROUTE:
		EncodeInt (3, 2, "parameterType", "RouteInsertion"); // mp conformance
		EncodeRoute ();
		break;
	default:
		nextToken.Error ("Unexpected symbol");
		nextToken.Scan ();
	}
}

/*-----------------------------------------------------------------
Encode a replacement command
-----------------------------------------------------------------*/
void BIFSEncoder :: ReplaceCommand ()
{
	nextToken.Scan ();
	if (nextToken == TID_SCENE)
	{	nextToken.Scan ();
		if (nextToken != TID_BY)
			nextToken.Error ("BY expected");
		else
			nextToken.Scan ();
		SceneReplaceCommand ();
		return;
	}

	EncodeInt (2, 2, "code", "ReplacementCommand");	 // mp conformance
	if (nextToken == TID_ROUTE)
		ReplaceRoute ();
	else
	{	BOOL bLast = FALSE;
		if (nextToken == TID_LAST)
		{	bLast = TRUE;
			nextToken.Scan ();
		}
		MediaObject *pNode = ScanNode ();
		if (pNode)
		{	if (!bLast && nextToken == TID_BY)
			{	EncodeInt (0, 2, "parameterType", "NodeReplacement");
				EncodeInt (pNode -> GetID (), 10, "nodeID");
				nextToken.Scan ();
				SFWorldNode ().Encode (this);
			}
			else
				pNode -> FieldReplacement (this, bLast);
		}
	}
}

/*-----------------------------------------------------------------
Encode DELETE command
-----------------------------------------------------------------*/
void BIFSEncoder :: DeleteCommand ()
{
	nextToken.Scan ();
	EncodeInt (1, 2, "code", "DeletionCommand"); // mp conformance
	if (nextToken == TID_ROUTE)
	{	nextToken.Scan ();
		EncodeInt (3, 2, "parameterType", "RouteDeletion"); // mp conformance
		EncodeRouteID ();
	}
	else
	{	BOOL bLast = FALSE;
		if (nextToken == TID_LAST)
		{	bLast = TRUE;
			nextToken.Scan ();
		}
		MediaObject *pNode = ScanNode ();
		if (pNode) 
		{	if (nextToken != TID_PERIOD)
			{	EncodeInt (0, 2, "parameterType", "NodeDeletion"); // mp conformance
				EncodeInt (pNode -> GetID (), 10, "nodeID");
			}
			else
				pNode -> IndexedValueDeletion (this, bLast);
		}
	}
}

/*-----------------------------------------------------------------
Encode insertion of a value to an MF node field. If bAppend is TRUE the
value is appended to the end of the field, otherwise index is given
-----------------------------------------------------------------*/
void BIFSEncoder :: InsertToField (BOOL bAppend)
{
	nextToken.Scan ();
	MediaObject *pNode = ScanNode ();
	if (pNode)
		pNode -> InsertToField (this, bAppend);
}

/*-----------------------------------------------------------------
Scan a node name and instantiate the node (may be a PROTO)
-----------------------------------------------------------------*/
MediaObject * BIFSEncoder :: ScanNode ()
{   
	// BF simplified after Script modification
	MediaObject *pNode = NULL;
	int nNodeID = EncodeNodeID ();
	if (nNodeID >= 0)
	{	
		pNode = nextToken.GetInstance (nNodeID);
	}
	if (pNode)
		pNode -> SetID (nNodeID);
	return pNode;
}

/*-----------------------------------------------------------------
Encode and APPEND TO command
-----------------------------------------------------------------*/
void BIFSEncoder :: AppendCommand ()
{
	nextToken.Scan ();
	EncodeInt (0, 2, "code", "InsertionCommand"); // mp conformance
	if (nextToken != TID_TO)
		nextToken.Error ("TO expected");
	InsertToField (TRUE);
}

/*-----------------------------------------------------------------
Next token is route name. Translate it to route ID
-----------------------------------------------------------------*/
void BIFSEncoder :: EncodeRouteID ()
{
	int nID;

	if (nextToken == TID_NAME)
	{	nID = nextToken.GetDefinedRoute ();
		if (nID < 0)
			nID = nextToken.AddPredefinedRoute ();
		EncodeInt (nID, 10, "RouteID");
	}
	else
		nextToken.Error ("Unexpected symbol");
	nextToken.Scan ();
}

/*-----------------------------------------------------------------
Next token is node name. Translate it to node ID and return the ID
-----------------------------------------------------------------*/
int BIFSEncoder :: EncodeNodeID ()
{
	int nNodeID = -1;
	if (nextToken == TID_NAME)
	{	nNodeID = nextToken.GetDefinedName ();
		if (nNodeID < 0)
			nextToken.Error ("Undefined node name");
	}
	else
		nextToken.Error ("Unexpected symbol");
	nextToken.Scan ();
	return nNodeID;
}

/*-----------------------------------------------------------------
Encode a route (after encoding the DEF part
-----------------------------------------------------------------*/
void BIFSEncoder :: EncodeRouteDetails ()
{
	for (BOOL bIn = FALSE; bIn <= TRUE; ++bIn)
	{
		MediaObject *pNode = ScanNode ();
		if (pNode != NULL)
			pNode -> EncodeRoute (this, bIn);
		if (!bIn)
		{	if (nextToken == TID_TO)
				nextToken.Scan ();
			else
				nextToken.Error ("TO expected");
		}
	}
}

void BIFSEncoder :: SceneReplaceCommand ()
{
	m_isRAP = TRUE;
	EncodeInt (3, 2, "code", "SceneReplaceCommand"); // mp conformance
	EncodeScene ();
}

/*-----------------------------------------------------------------
Encode replace route command
-----------------------------------------------------------------*/
void BIFSEncoder :: ReplaceRoute ()
{	nextToken.Scan ();
	EncodeInt (3, 2, "parameterType", "RouteReplacement"); // mp conformance
	EncodeRouteID ();
	if (nextToken == TID_BY)
		nextToken.Scan ();
	else
		nextToken.Error ("BY expected");
	EncodeRouteDetails ();
}

/*-----------------------------------------------------------------
Encode an initial scene (or scene replacement
-----------------------------------------------------------------*/
void BIFSEncoder :: EncodeScene ()
{
	m_scene.Remove ();
	nextToken.Reset ();
	EncodeInt (0, 6, "reserved");
	EncodeInt ((GetFlags () & CODENAMES) != 0, 1, "useNames"); // mp conformance
	bool bMoreProtos = nextToken == TID_PROTO || nextToken == TID_EXTERNPROTO;
	EncodeInt (bMoreProtos, 1, "morePROTOs");
	while (bMoreProtos)
	{	Proto *pProto = new Proto;
		if (!pProto -> EncodeProtoDefinition (this))
			delete pProto;
		bMoreProtos = nextToken == TID_PROTO || nextToken == TID_EXTERNPROTO;
		EncodeInt (bMoreProtos, 1, "morePROTOs");
	}
	if (nextToken == TID_EOF)
		return;
	m_scene.Encode (this);
	BOOL bHasRoutes = nextToken == TID_ROUTE;
	EncodeInt (bHasRoutes, 1, "hasROUTEs"); // mp conformance
	if (bHasRoutes)
		EncodeRoutesDescription ();
}

/*-----------------------------------------------------------------
Encode one update command.
Return TRUE if the command is a BIFS command
-----------------------------------------------------------------*/
BOOL BIFSEncoder :: EncodeUpdateCommand ()
{
	switch (nextToken) {
	case TID_INSERT:
		InsertCommand ();
		break;
	case TID_REPLACE:
		ReplaceCommand ();
		break;
	case TID_DELETE:
		DeleteCommand ();
		break;
	case TID_APPEND:
		AppendCommand ();
		break;
	case TID_UPDATE:
	case TID_REMOVE:
		m_pODEncoder -> ODUpdate ();
		return FALSE;
	case TID_OCI_EVENT:
		m_pOCIEncoder -> OCIEvent ();
		return false;
	case TID_EXECUTE:
		m_pODEncoder ->ODExecute ();
		return false;
	case TID_GLOBALQUANT:
		EncodeGlobalQuant();
		break;
	case TID_INSERTPROTO:
		EncodeProtoInsert();
		break;
	case TID_DELETEPROTO:
		EncodeProtoDelete();
		break;
	case TID_DELETEEX:
		EncodeNodeDeleteEx();
		break;
	case TID_UPDATENODE:
		EncodeMultipleFieldReplace();
		break;
	case TID_MULTIPLEREPLACE:
		EncodeMultipleIndexedReplace();
		break;
	default:
		SceneReplaceCommand ();
		break;
	}
	return TRUE;
}

/*-------------------------------------------------------------
Encode a String value
-------------------------------------------------------------*/
void BIFSEncoder :: EncodeString (const ZString &string)
{
	for (int i = 0; i < string.GetLength (); i++)
	{
		unsigned char c = string [i];
		EncodeInt (c, 8, "char", (ZString ("value [") += c) += ']'); // mp conformance
	}
	EncodeInt (0, 8, "char", "end of string"); // mp conformance
}

#endif
