/***********************************************************************
This software module was originally developed by Jean Le Feuvre
(ENST.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. ENST retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  BIFS Encoder
FILE:       BifsEncEx.cpp
AUTHOR:     Jean Le Feuvre (ENST)
OVERVIEW:   Implementation of extended BIFS command encoding 
-------------------------------------------------------------------------*/

#include "defs.h"
#include "BifsEnc.h"
#include "ODStrEnc.h"
#include "NodeFld.h"
#include "Token.h"
#include "bifs.h"
#include "Proto.h"
#include "Quantize.h"


void BIFSEncoder::EncodeGlobalQuant()
{
	EncodeInt(0, 2, "code", "InsertionCommand");
	EncodeInt(1, 2, "parameterType", "ExtendedUpdate");
	EncodeInt(5, 8, "code", "GlobalQuantizationConfiguration");

	SFWorldNode quant;
	nextToken.Scan();
	quant.Encode(this);
	MediaObject *pQP = quant.GetNode();
	if (pQP && (pQP->GetNodeCode()==CODE_QuantizationParameter)) {
		SetCurrentQP(pQP, TRUE);
	} else {
		SetCurrentQP(pQP, TRUE);
	}
	SetCurrentQP(NULL, FALSE);
	//store the QP otherwise iy will be destroyed
	if (m_globalQPList.Find(pQP) < 0) {
		m_globalQPList += pQP;
		quant = NULL;
	}
}

void BIFSEncoder::EncodeProtoInsert()
{
	EncodeInt(0, 2, "code", "InsertionCommand");
	EncodeInt(1, 2, "parameterType", "ExtendedUpdate");
	EncodeInt(0, 8, "code", "PROTOlistInsertion");

	nextToken.Scan();
	if (nextToken != TID_LEFT_BRACE) {
		nextToken.Error ("left brace '{' expected");
		return;
	}

	nextToken.Scan();
	bool bMoreProtos = nextToken == TID_PROTO || nextToken == TID_EXTERNPROTO;
	EncodeInt (bMoreProtos, 1, "morePROTOs");
	while (bMoreProtos)
	{	Proto *pProto = new Proto;
		if (!pProto -> EncodeProtoDefinition (this))
			delete pProto;
		bMoreProtos = nextToken == TID_PROTO || nextToken == TID_EXTERNPROTO;
		EncodeInt (bMoreProtos, 1, "morePROTOs");
	}
	if (nextToken != TID_RIGHT_BRACE) {
		nextToken.Error ("right brace '}' expected");
		return;
	}
	nextToken.Scan();
}

void BIFSEncoder::EncodeProtoDelete()
{
	EncodeInt(0, 2, "code", "InsertionCommand");
	EncodeInt(1, 2, "parameterType", "ExtendedUpdate");

	nextToken.Scan();
	if (nextToken != TID_LEFT_BRACE) {
		nextToken.Error ("left brace '{' expected");
		return;
	}
	nextToken.Scan();
	if (nextToken == TID_RIGHT_BRACE) {
		EncodeInt(2, 8, "code", "PROTOdeletion");
		nextToken.Scan();
		return;
	}
	MediaObject *ptr;

	EncodeInt(1, 8, "code", "PROTOLISTdeletion");
	EncodeInt(1, 1, "listDescription");

	int bMoreProto = 1;
	while (bMoreProto) {
		EncodeInt(1, 1, "moreProto");
		int nProtoID = nextToken.GetDefinedProto (&ptr);
		if (nProtoID < 0) {
			nextToken.Error ("unknown protoID");
			break;
		} else {
			EncodeInt(nProtoID, 10, "protoID");
		}
		nextToken.Scan();
		if (nextToken == TID_RIGHT_BRACE) bMoreProto = 0;
	}
	EncodeInt(0, 1, "moreProto");

	
	if (nextToken != TID_RIGHT_BRACE) {
		nextToken.Error ("right brace '}' expected");
		return;
	}
	nextToken.Scan();
}

void BIFSEncoder::EncodeNodeDeleteEx()
{
	EncodeInt(0, 2, "code", "InsertionCommand");
	EncodeInt(1, 2, "parameterType", "ExtendedUpdate");
	EncodeInt(6, 8, "code", "NodeDeleteEx");

	nextToken.Scan ();
	if (nextToken != TID_NAME) {
		nextToken.Error ("Unknown nodeID");
		return;
	}
	int nNodeID = nextToken.GetDefinedName ();
	EncodeInt(nNodeID, 10, "nodeID");
	nextToken.Scan ();

}

void BIFSEncoder::EncodeMultipleFieldReplace()
{
	EncodeInt(0, 2, "code", "InsertionCommand");
	EncodeInt(1, 2, "parameterType", "ExtendedUpdate");
	EncodeInt(4, 8, "code", "MultipleFieldReplace");

	nextToken.Scan ();
	if (nextToken != TID_NAME) {
		nextToken.Error ("Unknown nodeID");
		return;
	}
	int nNodeID = nextToken.GetDefinedName ();
	EncodeInt(nNodeID, 10, "nodeID");
	MediaObject *pNode = nextToken.GetInstance(nNodeID);
	if (!pNode) {
		nextToken.Error ("Node is not defined");
		return;
	}
	nextToken.Scan ();
	EncodeInt (0, 1, "maskAccess");

	if (nextToken == TID_LEFT_BRACE) {
		nextToken.Scan ();
		while (nextToken != TID_RIGHT_BRACE)
			pNode->ParseField(this);
		nextToken.Scan ();
	}
	EncodeInt(1, 1, "endFlag");
}


#define GETNUMBITS(n, nbBits) {	\
	int j = n ; nbBits = 0;	\
	while (j > 0) {			\
		j >>= 1; nbBits ++;		\
	}	}


void BIFSEncoder::EncodeMultipleIndexedReplace()
{
	EncodeInt(0, 2, "code", "InsertionCommand");
	EncodeInt(1, 2, "parameterType", "ExtendedUpdate");
	EncodeInt(3, 8, "code", "MultipleFieldReplace");

	nextToken.Scan ();
	if (nextToken != TID_NAME) {
		nextToken.Error ("Unknown nodeID");
		return;
	}
	int nNodeID = nextToken.GetDefinedName ();
	EncodeInt(nNodeID, 10, "nodeID");
	MediaObject *pNode = nextToken.GetInstance(nNodeID);
	if (!pNode) {
		nextToken.Error ("Node is not defined");
		return;
	}
	nextToken.Scan ();
	if (nextToken != TID_PERIOD) {
		nextToken.Error(".field name expected");
		return;
	}
	nextToken.Scan ();
	if (nextToken != TID_NAME) {
		nextToken.Error(".field name expected");
		return;
	}
	const BifsFieldTable *pEntry = pNode->GetBifsEntry(nextToken.GetName());
	if (pEntry->m_inID<0) {
		nextToken.Error("not an 'in' field");
		return;
	}
	pNode->WriteINindex(this, pEntry);

	NodeField *pField = pNode->GetField(pEntry);

	nextToken.Scan ();
	if (nextToken != TID_LEFT_BRACE) {
		nextToken.Error("'{' expected");
		return;
	}
	nextToken.Scan ();
	if (nextToken != TID_INTEGER) {
		nextToken.Error("integer expected");
		return;
	}
	int num = nextToken.GetInt();
	int max = -1 + nextToken.GetInt();
	if (nextToken != TID_RIGHT_BRACE) {
		nextToken.Error("'}' expected");
		return;
	}
	int nbits, nbpos;
	GETNUMBITS(max, nbpos);
	GETNUMBITS(num, nbits);
	EncodeInt(nbpos, 5, "lenPosition");
	EncodeInt(nbits, 5, "lenNum");
	EncodeInt(num, nbits, "numPositions");
	
	nextToken.Scan ();
	if (nextToken != TID_LEFT_BRACKET) {
		nextToken.Error("'[' expected");
		return;
	}

	Quantizer quant(GetCurrentQP(),
			pEntry -> m_fMin,  
			pEntry -> m_fMax,  
			pEntry -> m_nDimension, 
			pEntry -> m_nQuantizationType);

	//to avoid compiler warning...
	nextToken.Scan();
	while (num) {
		if (nextToken != TID_INTEGER) {
			nextToken.Error("number expected");
			return;
		}
		int pos = nextToken.GetInt();
		EncodeInt(pos, nbpos, "position");
		pField->EncodeElement(this, &quant);
		num--;
		if (nextToken == TID_COMMA) nextToken.Scan();
	}
	if (num) {
		nextToken.Error("bad number of position specified");
		return;
	}
	if (nextToken != TID_RIGHT_BRACKET) {
		nextToken.Error("']' expected");
		return;
	}
	nextToken.Scan();

}

