/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  BIFS Encoder
FILE:       EncField.cpp
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Encoding of a NodeField object
-------------------------------------------------------------------------*/
//$Log: EncField.cpp,v $
//Revision 1.1.1.2  2006/04/14 14:11:09  hill
//no message
//
//Revision 1.1.1.1  2003/12/24 00:42:05  mikeseven
//Initial release of the CVS repository for MPEG-4 Systems code
//after the code was corrupted due to reorganization and human 
//errors.
//
//Revision 1.3  2002/10/30 12:23:09  jean_lf
//Nested Proto fixe
//
//Revision 1.2  2002/08/08 09:47:25  zvil
//Core 5.9 Release Candidate 9:
//- Handling of time fields
//- Generating BIFS from memory
//- Proper handling of BIFS tables
//- More IPMPX stuff
//- Fixes to IM1-2D for proper use of startTime / stopTime
//- Fixes to Im1Player for proper use of startTime / stopTime
//
//
//Revision 1.x 2000/10/12  Pascal Belin
//QP bug fix
//
//Revision 1.3.2.1  2000/03/10 15:59:31  Alex Cotarmanac'h
//deuxime envoi
//
//Revision 1.2  2000/03/03 18:41:20  Alex Cotarmanac'h
//Version 1 -- Alexandre Cotarmanac_h
//
//Modified 2001/05/08 Boris Felts boris.felts@polytechnique.org
//

#include "defs.h"
#include "NodeFld.h"
#include "Quantize.h"
#include "Token.h"
#include "CondEnc.h"
#include "Conditional.h"
#include "NodeCode.h"
#include "PredictiveEncoder.h"

/*-------------------------------------------------------------
Encode an SF field
-------------------------------------------------------------*/
BOOL NodeField :: Encode (BIFSEncoder *pBifs, Quantizer *pQuant)
{
	/*-------------------------------------------------------------
	Check if the value is enclosed in brackets, then call a specific function to
	parse the value itself
	-------------------------------------------------------------*/
	bool bBrackets = nextToken == TID_LEFT_BRACKET;
	if (bBrackets)
		nextToken.Scan ();
	BOOL bResult = EncodeElement (pBifs, pQuant);
	if (bBrackets)
	{	if (nextToken == TID_RIGHT_BRACKET)
			nextToken.Scan ();
		else
			nextToken.Error ("] expected");
	}
	return bResult;
}

/*-----------------------------------------------------------------
Encode an SFInt
-----------------------------------------------------------------*/
BOOL SF<int> :: EncodeElement (BIFSEncoder *pBifs,  Quantizer *pQuant)
{
	/*-----------------------------------------------------------------
	Look at the Quantizer to see if min-max values applied
	-----------------------------------------------------------------*/
	m_value = nextToken.GetInt ();
	if (!pQuant || !pQuant -> IsActive())
		pBifs -> EncodeInt (m_value, 32, "value");
	else
		pQuant -> EncodeInt (pBifs, m_value);
	return TRUE;
}

/*-----------------------------------------------------------------
Encode an SFBool
-----------------------------------------------------------------*/
BOOL SF<Bool> :: EncodeElement (BIFSEncoder *pBifs, Quantizer * /*pQuant*/)
{
	m_value = nextToken.GetBool ();
    if (!pBifs->isCodingScript()) {
	    pBifs -> EncodeInt (m_value, 1, "value");
    	return TRUE;
    } else {
        return m_value;
    }
}

/*-----------------------------------------------------------------
Encode and SFFloat
MBS: modified to take into account quantization parameters if any
-----------------------------------------------------------------*/
BOOL SF<float> :: EncodeElement (BIFSEncoder *pBifs,  Quantizer * pQuant)
{
	m_value = nextToken.GetReal ();
	if (!pQuant || !pQuant->IsActive())
	        pBifs -> EncodeFloat (m_value, "value");
	else
		pQuant -> EncodeFloat(pBifs,m_value);
	return TRUE;
}

/*-----------------------------------------------------------------
Encode an SFTime
-----------------------------------------------------------------*/
BOOL SF<Time> :: EncodeElement (BIFSEncoder *pBifs,  Quantizer * /*pQuant*/)
{
	m_value = nextToken.GetReal ();
	pBifs -> EncodeDouble (m_value, "value");
	return TRUE;
}

/*-----------------------------------------------------------------
Encode an SFVec3f
MBS: modified to take into account quantization parameters if any
-----------------------------------------------------------------*/
BOOL SF<Vec3f> :: EncodeElement (BIFSEncoder *pBifs, Quantizer * pQuant)
{
  m_value.x = nextToken.GetReal ();
  m_value.y = nextToken.GetReal ();
  m_value.z = nextToken.GetReal ();
  if(!pQuant || !pQuant->IsActive()) {
    pBifs -> EncodeFloat (m_value.x, "float"); //BF 043101 Conformance
	pBifs -> EncodeFloat (m_value.y, "float"); //BF 043101 Conformance
    pBifs -> EncodeFloat (m_value.z, "float"); //BF 043101 Conformance
  }
  else {
    pQuant -> EncodeVec3f (pBifs,m_value);
  }
  return TRUE;
}

/*-----------------------------------------------------------------
Encode an SFVec4f
MBS: modified to take into account quantization parameters if any
-----------------------------------------------------------------*/
BOOL SF<Vec4f> :: EncodeElement (BIFSEncoder *pBifs, Quantizer * pQuant)
{
  m_value.x = nextToken.GetReal ();
  m_value.y = nextToken.GetReal ();
  m_value.z = nextToken.GetReal ();
  m_value.q = nextToken.GetReal ();
  if(!pQuant || !pQuant->IsActive()) {
    pBifs -> EncodeFloat (m_value.x, "float"); //BF 043101 Conformance
	pBifs -> EncodeFloat (m_value.y, "float"); //BF 043101 Conformance
    pBifs -> EncodeFloat (m_value.z, "float"); //BF 043101 Conformance
    pBifs -> EncodeFloat (m_value.q, "float"); //BF 043101 Conformance
  }
  else {
    ;			// TO DO: handle Vec4f quantization
  }
  return TRUE;
}

/*-----------------------------------------------------------------
Encode an SFVec2f
MBS: modified to take into account quantization parameters if any
-----------------------------------------------------------------*/
BOOL SF<Vec2f> :: EncodeElement (BIFSEncoder *pBifs, Quantizer * pQuant)
{
  m_value.x = nextToken.GetReal ();
  m_value.y = nextToken.GetReal ();
  if(!pQuant || !pQuant->IsActive()) {
    pBifs -> EncodeFloat (m_value.x, "float"); //BF Conformance
	pBifs -> EncodeFloat (m_value.y, "float"); //BF Conformance
  }
  else {
    pQuant -> EncodeVec2f(pBifs,m_value);
  }
  return TRUE;
}

/*-----------------------------------------------------------------
Encode an SFRotation
MBS: modified to take into account quantization parameters if any
-----------------------------------------------------------------*/
BOOL SF<Rotation> :: EncodeElement (BIFSEncoder *pBifs, Quantizer * pQuant)
{
  m_value.x = nextToken.GetReal ();
  m_value.y = nextToken.GetReal ();
  m_value.z = nextToken.GetReal ();
  m_value.theta = nextToken.GetReal ();
  if(!pQuant || !pQuant->IsActive()) {
    pBifs -> EncodeFloat (m_value.x, "float"); //BF Conformance
    pBifs -> EncodeFloat (m_value.y, "float"); //BF Conformance
    pBifs -> EncodeFloat (m_value.z, "float"); //BF Conformance
    pBifs -> EncodeFloat (m_value.theta, "float");//BF Conformance
  }
  else {
    pQuant -> EncodeRotation (pBifs,m_value);
  }

  return TRUE;
}

/*-----------------------------------------------------------------
Encode an SFRotation2D
MBS: modified to take into account quantization parameters if any
-----------------------------------------------------------------*/
BOOL SF<Rotation2D> :: EncodeElement (BIFSEncoder *pBifs, Quantizer * pQuant)
{
  m_value.x = nextToken.GetReal ();
  m_value.y = nextToken.GetReal ();
  m_value.theta = nextToken.GetReal ();
  if(!pQuant || !pQuant->IsActive()) {
    pBifs -> EncodeFloat (m_value.x, "float"); //BF Conformance
    pBifs -> EncodeFloat (m_value.y, "float"); //BF Conformance
    pBifs -> EncodeFloat (m_value.theta, "float");
} //BF Conformance  
  else {
    pQuant -> EncodeRotation2D(pBifs,m_value);
  }
  return TRUE;
}

/*-----------------------------------------------------------------
Encode an SFColor
MBS: modified to take into account quantization parameters if any
-----------------------------------------------------------------*/
BOOL SF<Color3f> :: EncodeElement (BIFSEncoder *pBifs, Quantizer * pQuant)
{
  m_value.r = nextToken.GetReal ();
  m_value.g = nextToken.GetReal ();
  m_value.b = nextToken.GetReal ();
  if(!pQuant || !pQuant->IsActive()) {
    pBifs -> EncodeFloat (m_value.r, "float"); //BF Conformance
    pBifs -> EncodeFloat (m_value.g, "float"); //BF Conformance
    pBifs -> EncodeFloat (m_value.b, "float"); //BF Conformance
  }
  else {
    pQuant->EncodeColor3f(pBifs,m_value);
  }

  return TRUE;
}

/*-----------------------------------------------------------------
Need to define empty SF<ZString> :: Encode function
-----------------------------------------------------------------*/
BOOL SF<ZString> :: EncodeElement (BIFSEncoder *pBifs, Quantizer *)
{
	m_value = nextToken.GetString ();
	nextToken.Scan ();
	((SFString *) this) -> EncodeString (pBifs);
	return TRUE;
}

/*-----------------------------------------------------------------
Write the bits of the string after it has been parsed
MBS 110698: updated for UTF-8 encoding
WARNING: we suppose text file uses ISO-Latin1 character set from 0x00 to 0xFF
Zvi 021799: Undo this thing, because UTF-8 is already handled in
Token::Scan.
-----------------------------------------------------------------*/
void SFString :: EncodeString (BIFSEncoder *pBifs)
{
	pBifs ->EncodeSFString (m_value);
}

/*-----------------------------------------------------------------
Encode SFUrl
-----------------------------------------------------------------*/
BOOL SF<Url> :: EncodeElement (BIFSEncoder *pBifs, Quantizer * /*pQuant*/)
{
	BOOL isOD = nextToken == TID_INTEGER;
	pBifs -> EncodeInt (isOD, 1, "isOD");
	if (isOD)
	{	m_value.ODid = nextToken.GetInt ();
		pBifs -> EncodeInt (m_value.ODid, 10, "ODid");
	}
	else
	{	m_value.ODid = -1;
		m_value.urlValue.Encode (pBifs);
	}
	return TRUE;
}

/*-----------------------------------------------------------------
Encode SFImage
-----------------------------------------------------------------*/
BOOL SF<Image> :: EncodeElement (BIFSEncoder *pBifs, Quantizer * /*pQuant*/)
{
	m_value.width = nextToken.GetInt ();
	m_value.height = nextToken.GetInt ();
	m_value.numComponents = (int) nextToken.GetInt () - 1 ; //PAB+:
	///PAB-: m_value.numComponents = nextToken.GetInt ()  ;
	//Alvar 981223: Height and width are 12 bit encoded (10->12)
	pBifs -> EncodeInt (m_value.width, 12, "width");
	pBifs -> EncodeInt (m_value.height,12, "height");
	pBifs -> EncodeInt (m_value.numComponents, 2, "numComponents");

	/*-----------------------------------------------------------------
	The image is taken from a file
	//PAB: the image is either taken from a file, either directly
	//PAB: encoded as specified in VRML.
	-----------------------------------------------------------------*/
	nextToken.Push(); //PAB+:
	LPCSTR szFileName = nextToken.GetString ();
	nextToken.Pop(); //PAB+:
	FILE *fImage = fopen (szFileName, "rb");
	if (!fImage)
		//PAB: if file could not be opened,
		//PAB: assume inlining of the bytes (cf VRML spec)
	{
		for (
			//Alvar 981223: numComponents is in bytes (no *8)
			int i = m_value.width * m_value.height;
			--i >= 0;
		)
		{
			int nPixel = nextToken.GetInt();
			if (m_value.numComponents == 3)
				pBifs -> EncodeInt ( (nPixel>>24) & 0x00FF , 8, "pixel");
			if (m_value.numComponents >= 2)
				pBifs -> EncodeInt ( (nPixel>>16) & 0x00FF , 8, "pixel");
			if (m_value.numComponents >= 1)
				pBifs -> EncodeInt ( (nPixel>>8) & 0x00FF , 8, "pixel");
			if (m_value.numComponents >= 0)
				pBifs -> EncodeInt (nPixel & 0x00FF, 8, "pixel");
		}
//PAB-:		nextToken.Error ("File not found");
	}
	else
	{
		for (
			//Alvar 981223: numComponents is in bytes (no *8)
			int i = m_value.width * m_value.height * m_value.numComponents;
			--i >= 0;
		)
		{
			pBifs -> EncodeInt (fgetc (fImage), 8, "pixel");
		}
		fclose (fImage);
		nextToken.Scan ();
	}
	return TRUE;
}

/*-----------------------------------------------------------------
A dummy function
-----------------------------------------------------------------*/
BOOL SF<MediaObject *> :: EncodeElement (BIFSEncoder *, Quantizer *)
{
	return FALSE;
}

/*-----------------------------------------------------------------
SFGenericNode Destructor.  Deletes the node (unless it's a proto)
-----------------------------------------------------------------*/
void SFGenericNode :: Remove ()
{
	if (m_pNode && m_pNode ->GetNodeCode () >= 0)
		delete m_pNode;
	m_pNode = NULL;
}

/*-----------------------------------------------------------------
Encode a generic node
-----------------------------------------------------------------*/
BOOL SFGenericNode :: EncodeElement (BIFSEncoder *pBifs, Quantizer * /*pQuant*/)
{
	/*-------------------------------------------------------------
	Check if "NULL" case
	-------------------------------------------------------------*/
	if (nextToken == TID_NULL)
	{	nextToken.Scan ();
		pBifs ->EncodeInt (1, 1, "isReused");
		pBifs ->EncodeInt (0xffff, 10, "nodeID");
		return true;
	}

	/*-----------------------------------------------------------------
	Check if "USE" case
	-----------------------------------------------------------------*/
	BOOL bIsReused = nextToken == TID_USE;

	if (bIsReused)
	{	pBifs -> EncodeInt (bIsReused, 1, "isReused");
		nextToken.Scan ();
		if (nextToken == TID_NAME)
		{	int nNodeID = nextToken.GetDefinedName ();
			if (nNodeID >= 0)
			{
				pBifs -> EncodeInt (nNodeID, 10, "nodeID");

				MediaObject * qpNode = nextToken.GetInstance (nNodeID);		// PB fixes the encoding of USEd QPs
				if(qpNode ->GetNodeCode () == CODE_QuantizationParameter)
				{
					pBifs->SetCurrentQP(qpNode);
				}															// PB end

				//AC 20000227 QP14 stuff
				if (pBifs->isCoordStore())
				{
					pBifs->setCurrentId(nNodeID);
					printf("[QP14] Getting length from keyID=%d\n",nNodeID);
				}
				//end AC
			}
			else
				nextToken.Error ("Name not found");
			nextToken.Scan ();
			return TRUE;
		}
		else
		{	nextToken.Error ("Unexpected symbol");
			nextToken.Scan ();
			return FALSE;
		}
	}

	/*-----------------------------------------------------------------
	Check if "DEF" is used
	-----------------------------------------------------------------*/
	BOOL bIsUpdateable = nextToken == TID_DEF;
	int nID = -1;
	ZString name;

	if (bIsUpdateable)
	{	nextToken.Scan ();
		if (nextToken == TID_NAME)
		{	nID = nextToken.GetDefinedName ();
			if (nID < 0)
				nID = nextToken.AddPredefinedName ();
			else
				nID = -1;
			if (nID < 0)
			{	nextToken.Error ("Name already defined");
				nID = -1;
			}
			name = nextToken.GetName ();
			nextToken.Scan ();
		}
		else
		{	nextToken.Error ("Unexpected symbol");
			nextToken.Scan ();
		}
	}

	/*-----------------------------------------------------------------
	Determine the node type, instantiate the object, and let it
	Encode itself. The node may be a PROTO
	-----------------------------------------------------------------*/
	int nNodeType = INT_MAX;

	int nProtoID = nextToken.GetDefinedProto (&m_pNode);
	if (nProtoID < 0)
	{	m_pNode = CreateNode (nextToken.GetName (), nNodeType);
		if (!m_pNode)
		{	if (nextToken != TID_NAME)
				nextToken.Error ("Unexpected symbol");
			else
				nextToken.Error ("Unrecognized node name");
			nextToken.Scan ();
			return FALSE;
		}
		if (nNodeType == -1)
			nextToken.Error ("Unexpected node type");
	}

	if (nNodeType == -2)
	{	nNodeType = 1;
		if (nextToken != TID_LEFT_BRACE)
		{	nextToken.Error ("Unexpected symbol");
			nextToken.Scan ();
		}
	}
	else
		nextToken.Scan ();

	pBifs -> EncodeInt (0, 1, "isReused");

	/*-----------------------------------------------------------------
	Encode the rest of the field and the node
	-----------------------------------------------------------------*/
	char szTag [] = "localNodeType\0\0";
	for (int nVer = 1; nVer <= NodeField :: BIFS_VERSIONS; nVer++)
	{	int nVersionTableSize = GetNodeVersionSize (nVer);
		int nFieldWidth = BitFieldWidth (nVersionTableSize + 1 + (nVer == 2));
		if (nNodeType <= nVersionTableSize)
		{	pBifs ->EncodeInt (nNodeType + (nVer == 2), nFieldWidth, szTag);
			break;
		}
		else if (nProtoID >= 0 && nVer == 2)
		{	pBifs ->EncodeInt (1, nFieldWidth, "nodeType");
			pBifs ->EncodeInt (nProtoID, 10, "PROTOnodeType");
			nNodeType = 1;
			break;
		}
		else
		{	pBifs ->EncodeInt (0, nFieldWidth, szTag);
			nNodeType -= nVersionTableSize;
			szTag [13] = 'V';
			szTag [14] = '1' + nVer;
		}
	}

	pBifs -> EncodeInt (bIsUpdateable, 1, "isUpdateable");
	if (bIsUpdateable)
	{	pBifs -> EncodeInt (nID, 10, "nodeID");
		if (pBifs ->GetFlags () & CODENAMES)
			pBifs -> EncodeString (name);
		if (nID >= 0)
		{
			nextToken.SetNodeID (nID, m_pNode -> GetNodeCode (), m_pNode);
			m_pNode -> SetID(nID);	// SCRIPT
			m_pNode->setDefName(name); //AC 200003 SCRIPT

		}
	}

	// PB moved these lines from below the is(QP) (BenA had moved this statement from here to below if(isQP) in July 99)
	// QPs don't quantize themselves anymore

	//AC 20000227 Quantization Scheme 14
	// set Active if it's the right node.
	bool resetWhenFinished=false;
	switch (m_pNode ->GetNodeCode ())
	{
		// #GB
		case CODE_Curve2D:
		case CODE_PointSet:
		case CODE_PointSet2D:
		case CODE_IndexedFaceSet:
		case CODE_IndexedFaceSet2D:
		case CODE_IndexedLineSet:
		case CODE_IndexedLineSet2D:
			pBifs->setCoordActive();
			resetWhenFinished=true;
			break;
		default:
			// nothing to do.
			break;
	}
	//AC end

	BOOL bResult = m_pNode->Encode (pBifs);
	// PB end

	// MBS: sets BIFSEncoder to current QP node just parsed
	if(m_pNode ->GetNodeCode () == CODE_QuantizationParameter) {
	  pBifs->SetCurrentQP(m_pNode); // PB DEFined/USEd QP fix
	  m_pNode->SetID(nID);
	}
	else if (nNodeType == -1)
	{
		delete m_pNode;
	    m_pNode = NULL;
		return false;
	}
	// end MBS Q

	// PB moved the statement put here by BenA from here to above if(QP)

	//AC 20000227 Quantization Scheme14
	//do we have to record any data ?
	switch (m_pNode ->GetNodeCode ())
	{
		case CODE_Coordinate:
		case CODE_Coordinate2D:
			if(pBifs->isCoordActive()) // PB changed assert into if (used to check if this was active)
				if (nID>=0) pBifs->setIdToLength(nID,pBifs->getLengthCoord());
			break;
		default:
			// nothing to do.
			break;
	}
	//AC end

	//AC 20000227 Quantization Scheme 14
	if (resetWhenFinished) pBifs->resetCoord();
	//AC end
	return bResult;
}

/*-----------------------------------------------------------------
Create a node whose name is szNodeName. To do this try to instantiate
all nodes, and for each try if this is the required one.
Return the found nodeType in rNodeType.
-----------------------------------------------------------------*/
MediaObject *SFGenericNode :: CreateNode (
	LPCSTR szNodeName,
	int &rNodeType
)
{
	for (int nCode = 0; nCode < MPEG4_TOTAL_NUMBER_OF_NODES; nCode ++)
		if (m_bifsFactory [nCode] != NULL)
		{	MediaObject *pNode = m_bifsFactory [nCode] -> CreateNode ();
			if (!strcmp (pNode -> GetName (), szNodeName))
			{	rNodeType = nCode;
				return pNode;
			}
			delete pNode;
		}
	return NULL;
}

/*-----------------------------------------------------------------
Encode an MF field
-----------------------------------------------------------------*/
BOOL MField :: Encode (BIFSEncoder *pBifs, Quantizer *pQuant)
{
	// MBS 111098: must sets QP node explicitly
	if(pQuant)
	  pQuant->SetQPNode(pBifs->GetCurrentQP());
	// end MBS 111098
//AC 1999/11/20
#pragma message("MField :: Encode Predictive")
	if (pBifs->isPredictive())
	{
		if ( nextToken==TID_PREDICTIVE )
		{
			nextToken.Scan();
			pBifs->EncodeInt(1,1,"usePredictiveMFField");
			// Next Token has to be a "basename" for a file containing the predictive parameters.
			// basename.prd is the file where parameters can be found
			if (nextToken != TID_NAME)
			{
				nextToken.Error ("Unexpected symbol");
				nextToken.Scan ();
				return FALSE;
			}

			const char * cstbase = nextToken.GetName() ;
			char * basename = new char[strlen(cstbase)+1];
			strcpy(basename,cstbase);
			PredictiveEncoder::Encode(pBifs,pQuant,this, basename);

			delete [] basename;

			return TRUE;
		}
		else
		{
			pBifs->EncodeInt(0,1,"usePredictiveMFField");
		}
	}
//AC end



	pBifs -> EncodeInt (0, 1, "reserved");		// added for FDIS
	pBifs -> EncodeInt (1, 1, "isListDescription");



	if (nextToken == TID_LEFT_BRACKET)
	{	nextToken.Scan ();

		// MBS: local def of QP variables
		BOOL localQP=FALSE;
		MediaObject *qpOld = NULL;
		// end MBS

		// BenA (7/30/99): save quantization for this level
		MediaObject *qpForThisLevel = pBifs->GetCurrentQP();
		// end BenA

		int a_compteur = 0;	//AC
		while (nextToken != TID_RIGHT_BRACKET)
		{
			pBifs -> EncodeInt (0, 1, "endFlag");	// changed in the FDIS

			// MBS: saves old QP node
			if(!localQP) qpOld=pBifs->GetCurrentQP();
			// end MBS

			EncodeElement (pBifs, pQuant);
			a_compteur++;//AC

			// MBS: restores current QP

			if(pBifs->hasLocalQP()) {
			  if(localQP) {
			    localQP=FALSE;
			    pBifs->SetCurrentQP(qpOld);
			  }
			  else {
			    localQP=TRUE;
			  }
			}
			// end MBS


		if (nextToken == TID_COMMA)
			nextToken.Scan ();
		}

		//AC 20000227
		// store if needed number of points
		pBifs->setLengthCoord(a_compteur);
		//there's more than one point
		//end AC

		// BenA (7/30/99): restores the right QP to use -- end of MF field
		pBifs->SetCurrentQP(qpForThisLevel);
		// end BenA


		nextToken.Scan ();
	}
	else
	{	pBifs -> EncodeInt (0, 1, "endFlag");
		EncodeElement (pBifs,pQuant);
		//AC 20000227
		// store if needed number of points
		pBifs->setLengthCoord(1);
		//there's only one point
		//end AC
	}
	pBifs -> EncodeInt (1, 1, "endFlag");
	return TRUE;
}

/*-----------------------------------------------------------------
The destrcutor of MFGenericNode deletes all elements in the array (except the PROTOs)
-----------------------------------------------------------------*/
void MFGenericNode :: Remove ()
{
	for (int i = m_array.GetLength (); --i >= 0;) {
		MediaObject *ptr = m_array [i];
		if (ptr  &&  ptr -> GetNodeCode () >= 0)
			delete ptr ;
	}
	m_array.Clear ();
}

/*-----------------------------------------------------------------
Delete one element from MFGenericNode
-----------------------------------------------------------------*/
void MFGenericNode :: DeleteFrom (int index, BOOL)
{
	delete m_array [index];
	m_array.DeleteFrom (index);
}

/*-----------------------------------------------------------------
The default implementation of EncodeElement can be called only by
error
-----------------------------------------------------------------*/
BOOL NodeField :: EncodeElement (BIFSEncoder *, Quantizer *)
{
	nextToken.Error ("Not an MF field");
	return FALSE;
}

#ifndef MUX
/*-----------------------------------------------------------------
Encode the buffer of a Conditional node (which is written as an update
command
-----------------------------------------------------------------*/
BOOL SFConditionalString :: Encode (BIFSEncoder *pBifs, Quantizer *)
{
	ConditionalBIFSEncoder encoder (
		&m_value,
		pBifs -> GetListFile (),
		pBifs -> GetFlags ()
	);
	encoder.EncodeAU ();
	EncodeString (pBifs);
	return TRUE;
}
#endif		//	MUX
