/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1997.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  BIFS/OD Encoder
FILE:       EncOD.cpp
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Extensions to the Encoding of a objects in an OD stream.
-------------------------------------------------------------------------*/
#include "defs.h"
#include "EncOD.h"
#include "Token.h"
#include "ODEnc.h"
//#ifdef _SAIT_3DTG_MP4VIDEO_
#include "../IM1Decoders/MP4Video/MediaParser/SaitStream.h"
//#endif _SAIT_3DTG_MP4VIDEO_

void ObjectDescriptorUpdateEnc :: PreEncode (SDLBaseClass *)
{
	if (OD.GetLength () == 0)
		nextToken.Error ("Empty update command", GetName ());
}

void ES_DescriptorUpdateEnc :: PreEncode (SDLBaseClass *)
{
	if (objectDescriptorId == 0)
		nextToken.Error ("Missing objectDescriptorId", GetName ());
	if (esDescr.GetLength () == 0)
		nextToken.Error ("Empty update command", GetName ());
}

void ES_DescriptorRemoveEnc :: PreEncode (SDLBaseClass *)
{
	if (objectDescriptorId == 0)
		nextToken.Error ("Missing objectDescriptorId", GetName ());
}

void IPMP_DescriptorUpdateEnc :: PreEncode (SDLBaseClass *)
{
	if (ipmpDescr.GetLength () == 0)
		nextToken.Error ("Empty update command", GetName ());
}

void ObjectDescriptorEnc :: PreEncode (SDLBaseClass *pField)
{
	/*-------------------------------------------------------------
	If necessary, replace this OD by one from the script file
	-------------------------------------------------------------*/
	if (muxScript.GetLength () > 0)
	{	ObjectDescriptor *pOD = ReplaceByScriptOD ();
		if (pOD != NULL_DESC)
		{	delete this;
			*pField = pOD;
			return;
		}
	}

	if (URLstring.GetLength () > 0)
	{	URL_Flag = 1;
		if (esDescr.GetLength () > 0)
			nextToken.Error ("URL used, ES_Descriptors not expected", GetName ());
	}
	else if (esDescr.GetLength () == 0)
		nextToken.Error ("No ES_Descriptors", GetName ());
}

void InitialObjectDescriptorEnc :: PreEncode (SDLBaseClass *)
{
	reserved = 0xf;
	if (URLstring.GetLength () > 0)
	{	URL_Flag = 1;
		if (esDescr.GetLength () > 0)
			nextToken.Error ("URL used, ES_Descriptors not expected", GetName ());
	}
	else
	{	if (esDescr.GetLength () == 0)
			nextToken.Error ("No ES_Descriptors", GetName ());
		if (ODProfileLevelIndication == 0)
			nextToken.Error ("ODProfileLevelIndication required", GetName ());
		if (sceneProfileLevelIndication == 0)
			nextToken.Error ("sceneProfileLevelIndication required", GetName ());
		if (audioProfileLevelIndication == 0)
			nextToken.Error ("audioProfileLevelIndication required", GetName ());
		if (visualProfileLevelIndication == 0)
			nextToken.Error ("visualProfileLevelIndication required", GetName ());
		if (graphicsProfileLevelIndication == 0)
			nextToken.Error ("graphicsProfileLevelIndication required", GetName ());
	}
}

void ES_DescriptorEnc :: PreEncode (SDLBaseClass *)
{
	if (ES_ID == 0 || ES_ID == 0xffff)
		nextToken.Error ("ES_ID missing or has illegal value", GetName ());
	if (dependsOn_ES_ID > 0)
		streamDependenceFlag = 1;
	if (URLstring.GetLength () > 0)
		URL_Flag = 1;
	if (OCR_ES_Id > 0)
		OCRstreamFlag = 1;
	if (decConfigDescr == NULL_DESC)
		nextToken.Error ("Missing DecoderConfigDescriptor", GetName ());
	if (slConfigDescr == NULL_DESC)
		nextToken.Error ("Missing SLConfigDescriptor", GetName ());
	
//#ifdef _SAIT_3DTG_MP4VIDEO_
// by D.S.CHO //////////////////////////////////////////////////////////////////
	/*-----------------------------------------------------------------
	Get a updated ESD for the specified script file and try to parse a
	correspoing DecoderSpecificInfo in that file
	------------------------------------------------------------------*/
	if (muxInfo == NULL_DESC)
		nextToken.Error ("No Mux info for stream no %d\n", GetName ());
	else
	{	if (decConfigDescr -> decSpecificInfo == NULL_DESC &&
			((decConfigDescr -> objectTypeIndication == 0x20 && decConfigDescr ->streamType == 0x04) ||
			 (decConfigDescr -> objectTypeIndication == 0xC4 && decConfigDescr ->streamType == 0x05) )
		) 
		{	
//mykim for URL
			if (muxInfo -> fileName.GetLength () == 0) 
			{
				if(URL_Flag) {
					m_fStream = fopen(URLstring, "rb");
					if(!m_fStream)
						nextToken.Error ("Cannot open %s\n", (LPCSTR) URLstring);		
				}
				else 
					nextToken.Error ("No Mux info filename for stream no %d\n", GetName ());
			}else {
				m_fStream =  fopen (muxInfo -> fileName, "rb");
				if (!m_fStream)
					nextToken.Error ("Cannot open %s\n", (LPCSTR) muxInfo -> fileName);		
			}
			LPCSTR szStreamType = muxInfo -> streamFormat;
			for (int i = m_nFactoriesAvailable; --i >= 0;)
			{	if (m_pFactories [i] -> IsTypeOf (szStreamType))
				{	m_pStream = m_pFactories [i] -> Create (m_fStream);				
					break;
				}
			}	

			DecoderSpecificInfoStringEnc *p = new DecoderSpecificInfoStringEnc;
			unsigned char *pDecSpecificInfo = m_pStream -> GetDecSpecificInfo ();
			long nDecSpecificInfoLen = m_pStream -> GetDecSpecificInfoLen ();
			
			for (i=0; i<nDecSpecificInfoLen; i++) 
				p ->info.InsertElement (pDecSpecificInfo[i], i); 
				
			decConfigDescr -> decSpecificInfo = p;	
		}	
	}
//~by D.S.CHO //////////////////////////////////////////////////////////////////
//#endif _SAIT_3DTG_MP4VIDEO_
}

void DecoderConfigDescriptorEnc :: PreEncode (SDLBaseClass *)
{
  if ((streamType == ST_AudioStream || streamType == ST_VisualStream) &&
		(objectTypeIndication == 0 || objectTypeIndication == 0xff)
	)
		nextToken.Error ("objectTypeIndication missing or has illegal value", GetName ());
	if (bufferSizeDB == 0)
		nextToken.Error ("Missing bufferSizeDB", GetName ());
}

void ContentIdentificationDescriptorEnc :: PreEncode (SDLBaseClass *)
{
	if (contentType > 0)
		contentTypeFlag = 1;
	if (contentIdentifier.GetLength () > 0)
		contentIdentifierFlag = 1;
}

void KeyWordDescriptorEnc :: PreEncode (SDLBaseClass *)
{
	keyWordCount = keyWord.GetLength ();	
}

void ExpandedTextualDescriptorEnc :: PreEncode (SDLBaseClass *)
{
	itemCount = itemDescription.GetLength ();
	if (itemCount != itemText.GetLength ())
		nextToken.Error ("itemDescription and itemText mismatch", GetName ());
}

void ContentCreatorNameDescriptorEnc :: PreEncode (SDLBaseClass *)
{
	contentCreatorCount = contentCreatorName.GetLength ();	
}

void OCICreatorNameDescriptorEnc :: PreEncode (SDLBaseClass *)
{
	OCICreatorCount = OCICreatorName.GetLength ();
}

// NHK	990702
void SmpteCameraPositionDescriptorEnc :: PreEncode(SDLBaseClass *)
{
	parameterCount = parameter.GetLength ();
}

void SLConfigDescriptorEnc :: PreEncode (SDLBaseClass *)
{
	if (compositionUnitDuration > 0)
		durationFlag = 1;
}

//#ifdef _SAIT_3DTG_MP4VIDEO_
// by D.S.CHO
void DecoderSpecificInfoStringEnc :: PreEncode (SDLBaseClass *)
{
	if (info.GetLength()<0) 
	{
		nextToken.Error("NULL String of DecoderSpecificInfoStringEnc!");
	}
}	
//~by D.S.CHO
//#endif _SAIT_3DTG_MP4VIDEO_

void BIFSConfigEnc :: PreEncode (SDLBaseClass *)
{
	if (pixelWidth > 0 || pixelHeight > 0)
		hasSize = 1;
	if (pixelMetric > 0 || hasSize != 0)
		isCommandStream = 1;	
}

void BIFSv2ConfigEnc :: PreEncode (SDLBaseClass *)
{
	if (pixelWidth > 0 || pixelHeight > 0)
		hasSize = 1;
	if (pixelMetric > 0 || hasSize != 0)
		isCommandStream = 1;	
}

// AudioSpecificConfigEnc added by giorgio.zoia@epfl.ch  2000-01-20

void AudioSpecificConfigEnc :: PreEncode (SDLBaseClass *)
{
	switch (audioObjectType)
	{
		case 1 : case 2 : case 3 :
		case 4 : case 6 : case 7 :
		case 8 : case 13 : case 14 :
		case 15 :
			break ;

		default :

			nextToken.Error ("audioObjectType not supported", GetName ());
			break ;
	}

	if ((samplingFrequency) && (audioObjectType <= 7)) 
	{
		samplingFrequencyIndex = 0x0F ; // escape
	}
}

// GeneralAudioConfig added by gew@iis.fhg.de 990512

void GeneralAudioConfigEnc :: PreEncode (SDLBaseClass *)
{
	if (coreCoderDelay)
	{
		dependsOnCoreCoder = 1 ;
	}
}

/*-------------------------------------------------------------
Get a list of ODs for the specified script file and try to find a
correspoing OD in that file
-------------------------------------------------------------*/
ObjectDescriptor *ObjectDescriptorEnc :: ReplaceByScriptOD ()
{
	char szAbsName [300];
	LPCSTR pMuxScript;

	if (nextToken.GetRootFileName () &&
		ZExpandURL (nextToken.GetRootFileName (), muxScript, szAbsName, 300) == 0
	)
		pMuxScript = szAbsName;
	else
		pMuxScript = muxScript;
 	// SOLARIS PORTING - added args
 	//SDLClassArray<ObjectDescriptor> *pODs = ODEncoder :: GetScriptODs (pMuxScript);
 	SDLClassArray<ObjectDescriptor, 1, 255> *pODs = ODEncoder :: GetScriptODs (pMuxScript);
	if (pODs)
		for (int i = pODs -> GetLength (); --i >= 0;)
			if (ObjectDescriptorID == (*pODs) [i] -> ObjectDescriptorID)
				return (*pODs) [i];
	return NULL;
}

//#ifdef _SAIT_3DTG_MP4VIDEO_
// by D.S.CHO /////////////////////////////////////////////////////////////
/*-----------------------------------------------------------------
Array of factory objects to create proper Stream object
-----------------------------------------------------------------*/
StreamDsiFactoryBase *ES_DescriptorEnc :: m_pFactories [40];
int ES_DescriptorEnc :: m_nFactoriesAvailable;

/*-----------------------------------------------------------------
Register a StreamFactory object
-----------------------------------------------------------------*/
void ES_DescriptorEnc :: RegisterFactory (StreamDsiFactoryBase *pFactory)
{
	assert (m_nFactoriesAvailable < 40);
	m_pFactories [m_nFactoriesAvailable++] = pFactory;
}

/*-----------------------------------------------------------------
The StreamFactoryBase constructor registers a factory object
-----------------------------------------------------------------*/
StreamDsiFactoryBase :: StreamDsiFactoryBase (LPCSTR szType)
{
	m_szType = szType;
	ES_DescriptorEnc :: RegisterFactory (this);
}

/*-----------------------------------------------------------------
Check if a stream factory object is of a required type
-----------------------------------------------------------------*/
BOOL StreamDsiFactoryBase :: IsTypeOf (LPCSTR szType)
{
	return
		(	szType != NULL && m_szType != NULL &&
			!stricmp (szType, m_szType)
		);
}
//~by D.S.CHO ////////////////////////////////////////////////////////
//#endif _SAIT_3DTG_MP4VIDEO_
