/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  BIFS Encoder
FILE:       EncProto.cpp
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Encoding of a Proto object
-------------------------------------------------------------------------*/
#include "defs.h"
#include "Proto.h"
#include "Token.h"
#include "BifsEnc.h"

/*-------------------------------------------------------------
Encode a Proto
-------------------------------------------------------------*/
bool Proto :: EncodeProtoDefinition (BIFSEncoder *pCoder)
{
	bool isExtern = nextToken == TID_EXTERNPROTO;
	nextToken.Scan ();
	if (nextToken != TID_NAME)
	{	nextToken.Error ("Unexpected symbol");
		nextToken.Scan ();
		return false;
	}

	if (nextToken.GetDefinedProto () >= 0)
		nextToken.Error ("Name already defined");
	int nID = nextToken.AddPredefinedProto (this);
	pCoder -> EncodeInt (nID, 10, "id");
	if (pCoder ->GetFlags () & CODENAMES)
		pCoder ->EncodeString (nextToken.GetName ());
	nextToken.Scan ();
	
	/*-------------------------------------------------------------
	Encode the interface fields
	-------------------------------------------------------------*/
	if (nextToken == TID_LEFT_BRACKET)
	{	nextToken.Scan ();
		while (nextToken != TID_RIGHT_BRACKET)
		{	if (nextToken < TID_FIELD || nextToken > TID_EVENTOUT)
			{	nextToken.Error ("Event type expected");
				if (nextToken == TID_EOF)
					return true;
				nextToken.Scan ();
				continue;
			}
			int eventType;
			int fieldType;
			bool isSF;
			eventType = ET_FIELD + nextToken - TID_FIELD;
			nextToken.Scan ();
			if (nextToken >= TID_SFBOOL && nextToken <= TID_MFNODE)
			{	fieldType = FT_BOOL + nextToken - TID_SFBOOL;
				isSF = nextToken <= TID_SFNODE;
				if (!isSF)
					fieldType -= TID_MFBOOL - TID_SFBOOL;
			}
			else
			{	nextToken.Error ("Unexpected symbol");
				nextToken.Scan ();
				continue;
			}
			pCoder -> EncodeInt (1, 1, "moreFields");
			pCoder -> EncodeInt (eventType, 2, "eventType");
			pCoder -> EncodeInt (!isSF, 1, "isMF");
			pCoder -> EncodeInt (fieldType, 5, "fieldType");
			nextToken.Scan ();

			NodeField *pField = CreateInterfaceField (
				pCoder,
				eventType,
				fieldType,
				isSF,
				nextToken.GetName ()
			);
			if (nextToken == TID_NAME)
			{	if (pCoder ->GetFlags () & CODENAMES)
					pCoder -> EncodeString (nextToken.GetName ());
				nextToken.Scan ();
			}
			else
				nextToken.Error ("Missing field name");
			if (pField == NULL)
				continue;

			/*-------------------------------------------------------------
			Eencode the field value (if this is a field)
			-------------------------------------------------------------*/
			if (eventType <= ET_EXPOSED_FIELD)
				pField ->Encode (pCoder);
		}
		nextToken.Scan ();
	}
	pCoder -> EncodeInt (0, 1, "moreFields");
	m_bifsBounds.CalculateBounds ();

	/*-------------------------------------------------------------
	Encode the PROTO code
	-------------------------------------------------------------*/
	pCoder -> EncodeInt (isExtern, 1, "isExtern");
	if (isExtern)
		MF<Url>().Encode (pCoder);
	else
		EncodeProtoCode (pCoder);
	pCoder -> SetProto (NULL);
	pCoder -> EncodeInt (0, 1, "useQuant");
	pCoder -> EncodeInt (0, 1, "useAnim");
	return true;
}

/*-------------------------------------------------------------
Enoce the PROTO code
-------------------------------------------------------------*/
void Proto :: EncodeProtoCode (BIFSEncoder *pCoder)
{	
	if (nextToken == TID_LEFT_BRACE)
		nextToken.Scan ();
	else
		nextToken.Error ("{ expected");

	/*-------------------------------------------------------------
	Encode nested PROTOs if any
	-------------------------------------------------------------*/
	bool bMoreProtos = nextToken == TID_PROTO || nextToken == TID_EXTERNPROTO;
	pCoder -> EncodeInt (bMoreProtos, 1, "morePROTOs");
	while (bMoreProtos)
	{	Proto *pProto = new Proto;
		if (!pProto -> EncodeProtoDefinition (pCoder))
			delete pProto;
		bMoreProtos = nextToken == TID_PROTO || nextToken == TID_EXTERNPROTO;
		pCoder -> EncodeInt (bMoreProtos, 1, "morePROTOs");
	}

	/*-------------------------------------------------------------
	Encode the PROTO nodes
	-------------------------------------------------------------*/
	pCoder -> SetProto (this);
	bool moreNodes;
	do
	{	m_protoNodes.EncodeElement (pCoder);
		moreNodes = nextToken == TID_NAME || nextToken == TID_DEF || nextToken == TID_USE;
		pCoder -> EncodeInt (moreNodes, 1, "moreNodes");
	}
	while (moreNodes);

	/*-------------------------------------------------------------
	Encode the PROTO Routes
	-------------------------------------------------------------*/
	BOOL bHasRoutes = nextToken == TID_ROUTE;
	pCoder -> EncodeInt (bHasRoutes, 1, "hasROUTEs"); // mp conformance
	if (bHasRoutes)
		pCoder -> EncodeRoutesDescription ();

	if (nextToken == TID_RIGHT_BRACE)
		nextToken.Scan ();
	else
		nextToken.Error ("} expected");

}
