// This software module was originally developed by Wenlong Dong(Rockwell
// Science Center) and Wei Wu(Rockwell Science Center) in the course of
// development of the MPEG-4 Systems standard
// (ISO/IEC 14496-1).  This software module is an implementation of a part
// of one or more MPEG-4 Systems tools as specified by the MPEG-4 Systems
// standard.  ISO/IEC gives users of the MPEG-4 Systems standard free
// license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// For more information or to receive an up to date version of this module,
// you may call Wenlong Dong at wdong@usc.edu or Wei Wu at
// weiwu@jupiter.risc.rockwell.com.
//
// Copyright 1998.

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:	ScriptImp Node
FILE:       VScript.cpp
AUTHOR:     Wenlong Dong (Rockwell Science Center)
			Wei Wu       (Rockwell Science Center)
		    Yuval Fisher (Rockwell Science Center)
OVERVIEW:   Implementation of the ScriptImp class.
--------------------------------------------------------------------*/
//$Log: EncScript.cpp,v $
//Revision 1.1.1.2  2006/04/14 14:11:09  hill
//no message
//
//Revision 1.1.1.1  2003/12/24 00:42:05  mikeseven
//Initial release of the CVS repository for MPEG-4 Systems code
//after the code was corrupted due to reorganization and human 
//errors.
//
//Revision 1.2  2002/11/13 09:25:33  jean_lf
//fixed some memory leaks in script enc
//
//Revision 1.1.1.1  2002/07/18 00:59:55  mikeseven
//Initial check in of July 17, 2002 version
//
//Revision 1.1.1.1.2.1  2000/03/10 15:59:34  Alex Cotarmanac'h
//deuxime envoi
//
//Modified 2001/05/08 Boris Felts boris.felts@polytechnique.org
//

#include "defs.h"
#include "ScriptImp.h"
#include "VSEvents.h"
#include "NodeCode.h"

#include "Token.h"
#include "BifsEnc.h"
#include "CParser.h"
#include "Proto.h"



SCRIPT_EVENT_TYPE ScriptImp::Scan_eType()
{
	switch (nextToken)
	{
	case TID_FIELD:
		return SCRIPT_FIELD;
		break;
	case TID_EVENTIN:
		return SCRIPT_EVENTIN;
		break;
	case TID_EVENTOUT:
		return SCRIPT_EVENTOUT;
		break;
	default:
		//nextToken.Error("Unrecognized event type"); // this isn't an error
		//printf("[Script] Error bad fieldtype \n"); // mp this isn't an error
		return SCRIPT_UNDEFINED;
	}
}

/*-----------------------------------------------------------------
Parse a Script by using the field table. Overload MediaObject::Encode
-----------------------------------------------------------------*/
/*BOOL ScriptImp :: Encode (BIFSEncoder *pBifs)
{
	pBifs->setCodingScript();
	pBifs -> EncodeInt (0, 1, "maskAccess");
	if (nextToken == TID_LEFT_BRACE)
	{	nextToken.Scan ();
		while (nextToken != TID_RIGHT_BRACE)
			ParseField (pBifs);
		nextToken.Scan ();
	}
	pBifs -> EncodeInt (1, 1, "endFlag");
	pBifs->setNoCodingScript();
	return TRUE; 
}

BOOL ScriptImp :: EncodeScriptField(BIFSEncoder *pCoder)
{
	int eventType;
	if ((eventType = Scan_eType()) != SCRIPT_UNDEFINED) 	//Encode the interface fields
	{	
		pCoder -> EncodeInt (1, 1, "endFlag"); // ScriptFieldListDescription
		int fieldType;
		nextToken.Scan ();
		// encode a field 
		if (nextToken >= TID_SFBOOL && nextToken <= TID_MFNODE)
		{	
			fieldType = nextToken - TID_SFBOOL;
		}
		else
		{	nextToken.Error ("fielType expected\n");
			nextToken.Scan ();
			return false;
		}
		pCoder -> EncodeInt (eventType, 2, "eventType");
		pCoder -> EncodeInt (fieldType, 6, "fieldType");
		nextToken.Scan (); 
		// look for field name
		if (nextToken == TID_NAME)
		{	
			pCoder -> EncodeString (nextToken.GetName ());
		}
		else
		{
			nextToken.Error ("field name expected\n");
			nextToken.Scan ();
			return false;
		}	
		//create field instance
		NodeField *pField = CreateInterfaceField (
			pCoder,
			eventType,
			(FIELD_TYPE)fieldType,
			nextToken.GetName ()
			);
		addIdentifier(nextToken.GetName ());  // save the name in the list of identifiers
		
		if (eventType == SCRIPT_FIELD)
		{
			nextToken.Scan();  // BF: Hack, look for "=" for initial value	
			if(stricmp(nextToken.GetName(),"="))
			{
				pCoder -> EncodeInt (1, 1, "hasInitialValue");
				nextToken.Scan (); // look for initial value of the field
				if (pField == NULL)
				{
					nextToken.Error ("Field empty\n");
					nextToken.Scan ();
					return false;
				}
				else 
					pField ->Encode (pCoder);
			}
			else
			{
				pCoder -> EncodeInt (0, 1, "hasInitialValue");
			}
		}	
		nextToken.Scan ();
	}
	return true;
}
*/
/*-----------------------------------------------------------------
Encode the ScriptImp Node. Left problems:
(2) A field of type SFNode and MFNode is not implemented ???
-----------------------------------------------------------------*/

BOOL ScriptImp::Encode(BIFSEncoder *pCoder)
{
    urlState = URL_BEFORE; // mp
    fieldBuffer[0].fieldName = strdup("directOutput");
    fieldBuffer[0].isDefault = true;
    fieldBuffer[1].fieldName = strdup("mustEvaluate");
    fieldBuffer[1].isDefault = true;

	pCoder ->setScriptCoding(true);
	pCoder -> EncodeInt (1, 1, "maskAccess");

    if (nextToken == TID_LEFT_BRACE)
	{	
        nextToken.Scan ();
        if (nextToken == TID_RIGHT_BRACE) { // no field at all
            pCoder->EncodeInt(0, 1, "mask", "url field");
        }            
	    while (nextToken != TID_RIGHT_BRACE)
		    ParseField (pCoder);
        encodeRegularScriptFields(pCoder);
	    nextToken.Scan ();
	}


	pCoder ->setScriptCoding(false);
	return TRUE;
}


/*-----------------------------------------------------------------
Parse one node field and encode it
mp : there is a trick to encode the interface fields.
they have to follow each other just before the "url" field.
the "url" field is then hard coded. This is not entirely satisfying
but is the only way to make it work. ( given that script fields
cannot be linearly encoded while parsing a wrl file)
-----------------------------------------------------------------*/
void ScriptImp :: ParseField (BIFSEncoder *pCoder)
{	
    char fieldName[256];
    strcpy(fieldName,nextToken.GetName());
    // check for errors
	if (nextToken != TID_NAME &&
        nextToken != TID_EVENTIN &&
        nextToken != TID_EVENTOUT &&
        nextToken != TID_FIELD)
	{	nextToken.Error ("Unexpected symbol");
		nextToken.Scan ();
        return;
	}

	
	if (Scan_eType() != SCRIPT_UNDEFINED) 	//Encode the interface fields
	{	
        if (urlState == URL_AFTER) {
            nextToken.Error("interface fields must preceed url field");
            nextToken.Scan ();
            return;
        }
        if (urlState == URL_BEFORE) { // then we enter the "url" field encoding
            encodeUrlStart(pCoder);
            urlState = URL_INSIDE;
        }
        parseScriptField(pCoder, Scan_eType());
	}
    else if (stricmp(fieldName,"url")) { // other fields : mustevaluate or directoutput
        int fieldRef = 0;
        if (!stricmp(fieldName,fieldBuffer[0].fieldName)) {
            fieldRef = 0;
        } else if (!stricmp(fieldName,fieldBuffer[1].fieldName)) {
            fieldRef = 1;
        } else {
  			nextToken.Error ("unknown field");
            return;
        }
        if (!fieldBuffer[fieldRef].isDefault) { // already parsed this field !
            nextToken.Error("duplicated field");
            nextToken.Scan();
            return;
        }
        fieldBuffer[fieldRef].isDefault = false;

        nextToken.Scan();
        fieldBuffer[fieldRef].hasProto = false;
		Proto *pProto = pCoder -> GetProto ();
		if (pProto != NULL)
		{	
            fieldBuffer[fieldRef].hasProto = true;
            fieldBuffer[fieldRef].ISedField = false;
			if (nextToken == TID_IS)
			{	
                fieldBuffer[fieldRef].ISedField = true;

    			//pCoder -> EncodeInt (1, 1, "isedField");
				nextToken.Scan ();
                int nFieldID = 0;
				if (nextToken != TID_NAME)
					nextToken.Error ("Unexpected symbol");
				else if (pProto -> GetBifsEntry (nextToken.GetName (), &nFieldID) == NULL)
					nextToken.Error ("Unrecognized field name");
				fieldBuffer[fieldRef].protoNbBits = pProto ->GetBifsBounds () ->m_nBitsPerAll;
                fieldBuffer[fieldRef].ISedfieldRef = nFieldID;
				nextToken.Scan ();
                return;
            }
        }
      	const BifsFieldTable *pEntry = GetBifsEntry (fieldBuffer[fieldRef].fieldName);
		fieldBuffer[fieldRef].value = EncodeField (pCoder, pEntry); // tricks
        return;
    } else { // this is THE url field
        if (urlState == URL_AFTER) { // should never occur
            nextToken.Error("duplicate url fields declared in script");
            nextToken.Scan();
            return;
        }
        if (urlState == URL_BEFORE) // we had no interface fields
        { 
            encodeUrlStart(pCoder);            
        }
        encodeUrlField(pCoder);
	}
}

void ScriptImp::encodeRegularScriptFields (BIFSEncoder * pCoder) {
    for (int i = 0; i < 2; i++) {
        if (fieldBuffer[i].isDefault) {
            pCoder->EncodeInt(0, 1, "mask");
        } else {
            pCoder->EncodeInt(1, 1, "mask");
            if (fieldBuffer[i].hasProto) {
                if (fieldBuffer[i].ISedField) {
    			    pCoder -> EncodeInt (1, 1, "isedField");
				    pCoder -> EncodeInt (
					    fieldBuffer[i].ISedfieldRef,
					    fieldBuffer[i].protoNbBits,
					    "protoField");
                    continue;
                } else {
    			    pCoder -> EncodeInt (0, 1, "isedField");
                }
            }
            pCoder->EncodeInt(fieldBuffer[i].value,1, "value");
        }
    }
}

void ScriptImp::encodeUrlField(BIFSEncoder * pCoder) {
    pCoder -> EncodeInt(1, 1, "endFlag"); // end of interface fields
    pCoder -> EncodeInt(1, 1, "reserved");
    bool isMFField = false;
    nextToken.Scan();
    if(nextToken == TID_LEFT_BRACKET) { // BF TID instead of CTID
        isMFField = true;
        nextToken.Scan();
    }
    do {
		if (nextToken != TID_STRING)	
		{
			nextToken.Error("Not a url string");
			nextToken.Scan();
		}	
		ZString string = nextToken.GetString();
		int pos;
		ZString str = GetHeader(string, pos);
		if(!stricmp(str,"javascript:"))
		{
			string.Cut(0,pos);
		}
		// rajouter des conditions pour parser correctement le script (event d'abord etc...)

		url += string;
		nextToken.Scan();
		pCParser = new CParser((LPCSTR)url [0]);
		pCParser->SetSNode(this);	// NODEFIELD
		pCParser -> Encode(pCoder);
		delete pCParser;
    } while (isMFField && nextToken != TID_RIGHT_BRACKET); //BF TID instead of CTID
    if (nextToken == TID_RIGHT_BRACKET) //BF TID instead of CTID
	{
		nextToken.Scan();
		isMFField = false;
	}
    urlState = URL_AFTER;
    pCoder -> EncodeInt(1, 1, "endFlag"); // end of MFScript field
}

void ScriptImp::encodeUrlStart(BIFSEncoder * pCoder) {
    pCoder -> EncodeInt(1, 1, "mask", "url field");
    pCoder -> EncodeInt(0, 1, "reserved", "mffield");
    pCoder -> EncodeInt(1, 1, "isListDescription");
    pCoder -> EncodeInt(0, 1, "endFlag");
    pCoder -> EncodeInt(1, 1, "isListDescription", "scriptfields");
}

void ScriptImp::parseScriptField(BIFSEncoder *pCoder, int eventType) {
    pCoder -> EncodeInt(0, 1, "endFlag", "more fields"); // first field
	nextToken.Scan ();
	// encode a field 
	int fieldType = 0;
	if (nextToken >= TID_SFBOOL && nextToken <= TID_MFNODE)
	{	
		fieldType = nextToken - TID_SFBOOL;
		if (fieldType > SFNODE)
			fieldType = fieldType - SFNODE - 1 + MFBOOL; // BF 
	}
	else
	{	nextToken.Error ("fielType expected\n");
		nextToken.Scan ();
	}
	pCoder -> EncodeInt (eventType, 2, "eventType");
	pCoder -> EncodeInt (fieldType, 6, "fieldType");
	nextToken.Scan (); 
	// look for field name
	if (nextToken == TID_NAME)
	{	
		pCoder -> EncodeString (nextToken.GetName ());
	}
	else
	{
		nextToken.Error ("field name expected\n");
		nextToken.Scan ();
        return;
	}	
	//create field instance
    LPCSTR tokName = nextToken.GetName ();
	NodeField *pField = CreateInterfaceField (
		pCoder,
		eventType,
		(FIELD_TYPE)fieldType,
		tokName );
	addIdentifier(tokName);  // save the name in the list of identifiers


	if (eventType == SCRIPT_FIELD)
	{
        char * fieldName = strdup(tokName);
        nextToken.Scan();
        if ((nextToken == TID_NAME && 
            !(stricmp(nextToken.GetName(),"url") &&
              stricmp(nextToken.GetName(),"directOutput")&&
              stricmp(nextToken.GetName(),"mustEvaluate")))
            || nextToken == TID_EVENTIN
            || nextToken == TID_EVENTOUT
            || nextToken == TID_FIELD) {
			pCoder -> EncodeInt (0, 1, "hasInitialValue");
        } else { // we have a default value
			pCoder -> EncodeInt (1, 1, "hasInitialValue");
			if (pField == NULL)
			{
				nextToken.Error ("Field empty\n");
				nextToken.Scan ();
			}
            else
            {
                int nFieldID; // mp dummy
               	const BifsFieldTable *pEntry =GetBifsEntry (fieldName, &nFieldID);
	            pCoder ->setScriptCoding(false); // so that fields are actually encoded
				EncodeField (pCoder, pEntry);
               	pCoder ->setScriptCoding(true);
            }
        }
        free(fieldName);
    } else {
        nextToken.Scan();
    }
}


ZString ScriptImp::GetHeader(ZString string, int &pos)
{
	int pos1 = 0;
	ZString header;

	while((string[pos1]==' ')||(string[pos1]=='\n')||(string[pos1]=='\t'))
		pos1++;

	pos = pos1;
	for(int i=0; i<15; i++)
	{
		char c = string[pos];
		pos++;
		if(!isalpha(c))
			break;
	}

	header = string.Substr(pos1, pos - pos1);
	return header;
}