/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  BIFS/OD Encoder
FILE:       ODEnc.cpp
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Implementation of ODEncoder, the class that encodes OD
			update commands.
-------------------------------------------------------------------------*/
#include "defs.h"
#include "ODEnc.h"
#include "Token.h"
#include "EncOD.h"
#include "VrmlPars.h"
#include "XMTPars.h"

#include "IPMP_Messages.h"
#include "CrSdl.h"

/*-----------------------------------------------------------------
ODEncoder constructor
-----------------------------------------------------------------*/
ODEncoder :: ODEncoder (
	FILE *fOutput,
	FILE *fList,
	LPCSTR szType,
	int flags
) : ODCoder (fOutput, fList, szType, flags, 0)
{
	/*-------------------------------------------------------------
	Overload some OD classes so encoding can be done properly.
	-------------------------------------------------------------*/
	OVERLOAD_OD_CLASS (ObjectDescriptorUpdateEnc);
	OVERLOAD_OD_CLASS (ES_DescriptorUpdateEnc);
	OVERLOAD_OD_CLASS (ES_DescriptorRemoveEnc);
	OVERLOAD_OD_CLASS (IPMP_DescriptorUpdateEnc);
	OVERLOAD_OD_CLASS (ObjectDescriptorEnc);
	OVERLOAD_OD_CLASS (InitialObjectDescriptorEnc);
	OVERLOAD_OD_CLASS (ES_DescriptorEnc);
	OVERLOAD_OD_CLASS (DecoderConfigDescriptorEnc);
	OVERLOAD_OD_CLASS (ContentIdentificationDescriptorEnc);
	OVERLOAD_OD_CLASS (KeyWordDescriptorEnc);
	OVERLOAD_OD_CLASS (ExpandedTextualDescriptorEnc);
	OVERLOAD_OD_CLASS (ContentCreatorNameDescriptorEnc);
	OVERLOAD_OD_CLASS (OCICreatorNameDescriptorEnc);
	OVERLOAD_OD_CLASS (SmpteCameraPositionDescriptorEnc);		// NHK	990702
	OVERLOAD_OD_CLASS (SLConfigDescriptorEnc);
	OVERLOAD_OD_CLASS (BIFSConfigEnc);
	OVERLOAD_OD_CLASS (BIFSv2ConfigEnc);
	OVERLOAD_OD_CLASS (AudioSpecificConfigEnc); // gew@iis.fhg.de 990512
												// giorgio.zoia@epfl.ch 2000-01-20
	OVERLOAD_OD_CLASS (GeneralAudioConfigEnc);
}

/*-------------------------------------------------------------
Scan the list of parsed .scr file to see whether the specified
file has already been parsed. If not, parsed it now. Returns a pointer
to the list of parsed ODs
-------------------------------------------------------------*/
ZArray<ODEncoder :: ParseScriptDesc> ODEncoder :: m_parsedScriptODs; // define this static member

// SOLARIS PORTING - added args
//SDLClassArray<ObjectDescriptor> *ODEncoder :: GetScriptODs (LPCSTR szFile)
SDLClassArray<ObjectDescriptor, 1, 255> *ODEncoder :: GetScriptODs (LPCSTR szFile)
{
	/*-------------------------------------------------------------
	Scan the list
	-------------------------------------------------------------*/
	for (int i = m_parsedScriptODs.GetLength (); --i >= 0;)
		if (!strcmp (szFile, m_parsedScriptODs [i].m_fileName))
			return &m_parsedScriptODs [i].m_ods;

	/*-------------------------------------------------------------
	Open a new script file and parse it
	-------------------------------------------------------------*/
	Parser *pScriptParser;
	if (nextToken.GetFlags () & FORMAT_XMT)
		pScriptParser = new XMTParser;
	else
		pScriptParser = new VRMLParser;
	if (!pScriptParser ->Init (szFile))
	{	pScriptParser ->Shutdown ();
		delete pScriptParser;
		nextToken.Error ("cannot open file", szFile);
		return NULL;
	}
	ParseScriptDesc &rDesc = ++m_parsedScriptODs;
	rDesc.m_fileName = szFile;

	/*-------------------------------------------------------------
	Save the original value of nextToken, and reset it because we
	are going to parse a new file.
	-------------------------------------------------------------*/
	nextToken.Push ();
	nextToken.Init (pScriptParser);

	/*-------------------------------------------------------------
	Now do the parsing (the first descriptor might be an
	InitialObjectDescriptor
	-------------------------------------------------------------*/
	if (stricmp (nextToken.GetString (), "InitialObjectDescriptor") == 0)
 		// SOLARIS PORTING - added args
 		//SDLClass<InitialObjectDescriptor>().Scan ();
 		SDLClass<InitialObjectDescriptor, 1>().Scan ();
	while (nextToken != TID_EOF)
	{
 		// SOLARIS PORTING - added args
 		//SDLClass<ObjectDescriptor> od;
 		SDLClass<ObjectDescriptor, 1> od;
		od.Scan ();
		if (od != NULL_DESC)
		{	rDesc.m_ods += od;
			/*-------------------------------------------------------------
			This object descriptor will be used by object of types
			SDLClass and SDLClassArray, which destroy the descriptor in
			their constructor. To prevent this, we need to protect
			the object descriptor
			-------------------------------------------------------------*/
			od -> Protect (true);
		}
	}

	/*-------------------------------------------------------------
	Close the script file and restore the original token
	-------------------------------------------------------------*/
	pScriptParser ->Shutdown ();
	delete pScriptParser;
	nextToken.Pop ();
	return &rDesc.m_ods;
}

/*-------------------------------------------------------------
The desructor of the ParseScriptDesc structure marks all the contained
object descriptors as non-protected, so they will be deleted by the
SDLClassArray destructor.
-------------------------------------------------------------*/
ODEncoder :: ParseScriptDesc :: ~ParseScriptDesc ()
{
	for (int i = m_ods.GetLength (); --i >= 0;)
		m_ods [i] -> Protect (false);
}
