/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  BIFS/OD Encoder
FILE:       ODEnc.cpp
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Implementation of ODStreamEncoder, the class that encodes OD
			streams.
-------------------------------------------------------------------------*/
#include "defs.h"
#include "ODStrEnc.h"
#include "Token.h"
#include "EncOD.h"
#include "ODP3.h"
#include "IPMP_Messages.h"

/*-----------------------------------------------------------------
Encode OD-update commands
-----------------------------------------------------------------*/
void ODStreamEncoder :: ODUpdate ()
{
	BaseCommand *pCommand = NULL;

	switch (nextToken) {
	case TID_UPDATE:
		nextToken.Scan ();
		switch (nextToken) {
		case TID_OD:
			pCommand = UpdateOD ();
			break;
		case TID_ESD:
			pCommand = UpdateES ();
			break;
		case TID_IPMPD:
			pCommand = UpdateIPMPD ();
			break;
		case TID_IPMPDX:
			pCommand = UpdateIPMPDX ();
			break;
		default:
			nextToken.Error ("Unreconginzed command");
			nextToken.Scan ();
			return;
		}
		break;
	case TID_REMOVE:
		nextToken.Scan ();
		switch (nextToken) {
		case TID_OD:
			pCommand = RemoveOD ();
			break;
		case TID_ESD:
			pCommand = RemoveES ();
			break;
		case TID_IPMPD:
			pCommand = RemoveIPMPD ();
			break;
		default:
			nextToken.Error ("Unreconginzed command");
			nextToken.Scan ();
			return;
		}
		break;
	}
	if (pCommand != NULL_DESC)
	{
		pCommand -> PreEncode (NULL);
		pCommand -> EncDec (this, CALC_LENGTH, NULL);
		pCommand -> EncDec (this, ENCODE, NULL);
		delete pCommand;
	}
}

/*-----------------------------------------------------------------
Parse an ObjectDescriptorUpdate command
-----------------------------------------------------------------*/
BaseCommand *ODStreamEncoder :: UpdateOD ()
{
	/*-----------------------------------------------------------------
	Parse the list of ODs
	-----------------------------------------------------------------*/
	ObjectDescriptorUpdate *p = new ObjectDescriptorUpdateEnc;
	nextToken.Scan ();
	p -> OD.Scan ();
	return p;
}

/*-----------------------------------------------------------------
Parse an ObjectDescriptorRemove command
-----------------------------------------------------------------*/
BaseCommand *ODStreamEncoder :: RemoveOD ()
{
	/*-----------------------------------------------------------------
	Parse the list of ODids
	-----------------------------------------------------------------*/
	ObjectDescriptorRemove *p = new ObjectDescriptorRemove;
	nextToken.Scan ();
	p -> objectDescriptorId.Scan ();
	return p;
}

/*-----------------------------------------------------------------
Parse an ES_DescriptorRemove command
-----------------------------------------------------------------*/
BaseCommand *ODStreamEncoder :: RemoveES ()
{
	nextToken.Scan ();
	if (nextToken != TID_FROM)
		nextToken.Error ("FROM expected");
	nextToken.Scan ();
	if (nextToken != TID_INTEGER)
		nextToken.Error ("ObjectDescriptorID expected");

	/*-----------------------------------------------------------------
	Parse the list of ESids
	-----------------------------------------------------------------*/
	ES_DescriptorRemove *p = new ES_DescriptorRemoveEnc;
	p -> objectDescriptorId = (int) nextToken.GetInt ();
	p -> ES_ID.Scan ();
	return p;
}

/*-----------------------------------------------------------------
Parse an ES_DescriptorUpdate command
-----------------------------------------------------------------*/
BaseCommand *ODStreamEncoder :: UpdateES ()
{
	nextToken.Scan ();
	if (nextToken != TID_IN)
		nextToken.Error ("IN expected");
	nextToken.Scan ();
	if (nextToken != TID_INTEGER)
		nextToken.Error ("ObjectDescriptorID expected");

	/*-----------------------------------------------------------------
	Parse the list of ESs
	-----------------------------------------------------------------*/
	ES_DescriptorUpdate *p = new ES_DescriptorUpdateEnc;
	p -> objectDescriptorId = (int) nextToken.GetInt ();
	p -> esDescr.Scan ();
	return p;
}

/*-----------------------------------------------------------------
Parse an IPMP_DescriptorUpdate command
-----------------------------------------------------------------*/
BaseCommand *ODStreamEncoder :: UpdateIPMPD ()
{
	/*-----------------------------------------------------------------
	Parse the list of IPMPs
	-----------------------------------------------------------------*/
	IPMP_DescriptorUpdate *p = new IPMP_DescriptorUpdateEnc;
	nextToken.Scan ();
	p -> ipmpDescr.Scan ();
	return p;
}

/*-----------------------------------------------------------------
Parse an IPMP_ToolDescriptorUpdate command
-----------------------------------------------------------------*/
BaseCommand *ODStreamEncoder :: UpdateIPMPDX ()
{
	/*-----------------------------------------------------------------
	Parse the list of IPMPs
	-----------------------------------------------------------------*/
	IPMP_DescriptorUpdate *p = new IPMP_DescriptorUpdate;
	nextToken.Scan ();
	p -> ipmpDescr.Scan ();
	return p;
}

/*-----------------------------------------------------------------
Parse IPMP_DescriptorRemove command
-----------------------------------------------------------------*/
BaseCommand *ODStreamEncoder :: RemoveIPMPD ()
{
	/*-----------------------------------------------------------------
	Parse the list of IPMPDids
	-----------------------------------------------------------------*/
	IPMP_DescriptorRemove *p = new IPMP_DescriptorRemove;
	nextToken.Scan ();
	p -> IPMP_DescriptorID.Scan ();
	return p;
}

/*-------------------------------------------------------------
Parse an OCI event command
-------------------------------------------------------------*/
void ODStreamEncoder :: OCIEvent ()
{
	nextToken.Scan ();
	OCI_Event ociEvent;
	ociEvent.Scan ();
	ociEvent.DescEncDec (this, CALC_LENGTH);
	Align ();
	StoreDescLength (ociEvent.GetLength ());
	ociEvent.DescEncDec (this, ENCODE);
	Align ();
}

/*-------------------------------------------------------------
Parse an ODExecute command
-------------------------------------------------------------*/
void ODStreamEncoder :: ODExecute ()
{
	ObjectDescriptorExecute *pCommand = NULL;

	nextToken.Scan ();
	if (nextToken != TID_OD)
		nextToken.Error ("OD expected");
	else
		nextToken.Scan ();
	pCommand = new ObjectDescriptorExecute;
	pCommand -> objectDescriptorId.Scan ();
	if (pCommand != NULL_DESC)
	{
		pCommand -> PreEncode (NULL);
		pCommand -> EncDec (this, CALC_LENGTH, NULL);
		pCommand -> EncDec (this, ENCODE, NULL);
		delete pCommand;
	}
}

/*-----------------------------------------------------------------
Flushing the buffer - first make sure the file has been created, and if not, creat it.
Then call the base class Flush function to write the buffer.
-----------------------------------------------------------------*/
void ODStreamEncoder :: Flush (DWORD dwTS)
{
	if (!m_bInit)
	{	m_bInit = true;
		ZString stOutFile (nextToken.GetRootFileName ());
		stOutFile.Cut (strrchr (stOutFile, '.') - (LPCSTR) stOutFile + 1);
		stOutFile += m_szExt;
		if ((m_fOutput = fopen (stOutFile, "wb")) == NULL)
		{	nextToken.Error ("Cannot open file", stOutFile);
			return;
		}
	}
	ODEncoder :: Flush (dwTS);
}

/*-------------------------------------------------------------
ODStreamEncoder desctructor closes the output file
-------------------------------------------------------------*/
ODStreamEncoder :: ~ODStreamEncoder ()
{
	if (m_fOutput)
		fclose (m_fOutput);
}