/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Optibase Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Optibase Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1997.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

#if !defined(_Parser_h)	// Sentry, use file only if it's not already included.
#define _Parser_h

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  BIFS Encoder
FILE:       Parser.h
AUTHOR:     Zvi Lifshitz (Optibase Ltd.)
OVERVIEW:   Declaration of Parser, a base class for classes that parse tokens from input files.
-------------------------------------------------------------------------*/
#include "ztl.h"

typedef enum tagTokenCodes
{
	TID_UNKNOWN,
	TID_LEFT_BRACE,
	TID_RIGHT_BRACE,
	TID_LEFT_BRACKET,
	TID_RIGHT_BRACKET,
	TID_COMMA,
	TID_PERIOD,
	TID_STRING,
	TID_USE,
	TID_DEF,
	TID_FALSE,
	TID_TRUE,
	TID_ROUTE,
	TID_TO,
	TID_AT,
	/*JLF 03/03. added RAP indicator*/
	TID_RAP,
	TID_INSERT,
	TID_REPLACE,
	TID_DELETE,
	TID_APPEND,
	TID_LAST,
	TID_SCENE,
	TID_BY,
	/*extended commands*/
	TID_GLOBALQUANT,
	TID_INSERTPROTO,
	TID_DELETEPROTO,
	TID_DELETEEX,
	TID_UPDATENODE,
	TID_MULTIPLEREPLACE,
	/*end extended commands*/
	TID_UPDATE,
	TID_REMOVE,
	TID_OD,
	TID_ESD,
	TID_IN,
	TID_IPMPD,
	TID_IPMPDX,
	TID_FROM,
	TID_FILE,
	TID_PROTO,
	TID_EXTERNPROTO,
	TID_FIELD, TID_EXPOSEDFIELD, TID_EVENTIN, TID_EVENTOUT,
	TID_SFBOOL, TID_SFFLOAT, TID_SFTIME, TID_SFINT32, TID_SFSTRING, TID_SFVEC3F,
	TID_SFVEC2F, TID_SFCOLOR, TID_SFROTATION, TID_SFIMAGE, TID_SFNODE,
	TID_MFBOOL, TID_MFFLOAT, TID_MFTIME, TID_MFINT32, TID_MFSTRING, TID_MFVEC3F,
	TID_MFVEC2F, TID_MFCOLOR, TID_MFROTATION, TID_MFIMAGE, TID_MFNODE,
	TID_IS,
	TID_PREDICTIVE, //AC 1999/11/21
	TID_NULL,
	TID_OCI_EVENT,
	TID_EXECUTE,
	TID_NUMBER_OF_RESERVED,
	TID_NAME = TID_NUMBER_OF_RESERVED,
	TID_NUMBER,
	TID_INTEGER,
	TID_EOF
} TOKEN_CODE;

struct TokenInfo
{
	TOKEN_CODE m_code;
	ZString m_string;
	int m_nLine;
};

class Parser
{	
public:
	virtual bool Init (LPCSTR szFileName) = 0;
	virtual void Shutdown () = 0;
	virtual void GetToken (TokenInfo *pInfo) = 0;
	virtual LPCSTR GetLine () const = 0;
	virtual void GetString (ZString &str) = 0;

protected:
	static LPCSTR m_reserved [];
	TOKEN_CODE GetSymbol (LPCSTR szToken);
};

#endif                                      // _Parser_h sentry.
