/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  BIFS Encoder
FILE:       Token.cpp
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Implementation of Token, the class that handles text parsing.
-------------------------------------------------------------------------*/
//$Log: Token.cpp,v $
//Revision 1.2  2005/01/18 16:22:12  sony
//Son Tran Artemis: fix the bugs of parsing float number in the form ".0nn "
//
//Revision 1.1.1.1  2003/12/24 00:42:06  mikeseven
//Initial release of the CVS repository for MPEG-4 Systems code
//after the code was corrupted due to reorganization and human 
//errors.
//
//Revision 1.4  2003/04/03 04:29:01  mikeseven
//Initial release of Player3D and MPEG-J for all MPEG-4 reference softwares players with samples.
//For installation, follow instructions in docs/Player3D_setup.doc.
//
//Many thanks to Prashanth Gujjeti and Viswanathan Swaminathan from Sun Microsystems.
//
//-- Mikael Bourges-Sevenier (Mindego)
//
//Revision 1.3  2002/10/30 12:23:34  jean_lf
//Nested Proto fixe
//
//Revision 1.2  2002/08/08 09:47:25  zvil
//Core 5.9 Release Candidate 9:
//- Handling of time fields
//- Generating BIFS from memory
//- Proper handling of BIFS tables
//- More IPMPX stuff
//- Fixes to IM1-2D for proper use of startTime / stopTime
//- Fixes to Im1Player for proper use of startTime / stopTime
//
//
//Revision 1.x 2000/10/12  Pascal Belin
//QP bug fix
//
//Revision 1.2  2000/03/03 18:41:21  Alex Cotarmanac'h
//Version 1 -- Alexandre Cotarmanac_h
//
#include "defs.h"
#include "Token.h"
#include "MediaObj.h"
#include "Proto.h"

/*-------------------------------------------------------------
Display an error message at exit, if necessary.
-------------------------------------------------------------*/
Token :: ~Token ()
{
	if (m_bHadErrors)
	{	printf (
			"WARNING: Had parsing errors!\07\07\07\n"
			"Hit any key to continue\n"
		);
		getchar ();
	}
}

/*-----------------------------------------------------------------
Initiate the Token object
-----------------------------------------------------------------*/
void Token :: Init (Parser *pParser, LPCSTR szRootFile, FILE *fList)
{
	m_pParser = pParser;
	if (fList)
		m_fList = fList;
	if (szRootFile)
		m_pParentFile = szRootFile;
	m_nLineNumber = 0;
	Scan ();
}

/*-----------------------------------------------------------------
Translate the token to an int
-----------------------------------------------------------------*/
ZLargeInt<128> Token :: GetInt ()
{
	ZLargeInt<128> nResult;

	if (m_info.m_code == TID_FALSE)
	{	Scan ();
		return 0;
	}
	else if (m_info.m_code == TID_TRUE)
	{	Scan ();
		return 1;
	}
	else if (m_info.m_code != TID_INTEGER)
		Error ("Integer number expected");
	else
	{	int i = 0, nRadix = 10, c;
		int nSign = 1;
		switch (m_info.m_string [0]) {
		case '-':
			nSign = -1;
		case '+':
			++i;
		}
		if (m_info.m_string [i] == '0' && tolower (m_info.m_string [i + 1]) == 'x')
		{	nRadix = 16;
			i += 2;
		}
		for (;(c = tolower (m_info.m_string [i])) != 0; i++)
		{	if (c >= '0' && c <= '9')
				c -= '0';
			else if (nRadix == 16 && c >= 'a' && c <= 'f')
				c -= 'a' - 10;
			else
			{	Error ("Integer number expected");
				break;
			}
			nResult *= nRadix;
			nResult += c;
		}
		nResult *= nSign;
	}

	if (m_info.m_code == TID_INTEGER || m_info.m_code == TID_NUMBER)
		Scan ();
	return nResult;
}

/*-----------------------------------------------------------------
Translate TRUE/FALSE value
-----------------------------------------------------------------*/
bool Token :: GetBool ()
{
	bool bResult;

	switch (m_info.m_code) {
	case TID_TRUE:
		bResult = true;
		break;
	case TID_FALSE:
		bResult = false;
		break;
	case TID_NUMBER:
		Error ("Boolean expected");
		Scan ();
		return FALSE;
	default:
		Error ("Boolean expected");
		return FALSE;
	}
	Scan ();
	return bResult;
}

/*-----------------------------------------------------------------
Translate a real value
-----------------------------------------------------------------*/
double Token :: GetReal ()
{
	double dResult;
	// PB allows to write float like .125 for instance
	if (m_info.m_code == TID_PERIOD)
	{
		nextToken.Scan();
		if (m_info.m_code != TID_NUMBER && m_info.m_code != TID_INTEGER)
		{
			Error ("Number expected");
			return 0;
		}
		else
		{
		/*	//SON Tran Artemis
			//The following code segment has the bus while parsing float number presented in the form of ".[0]nnn"
			//for example .002 -> 0.2 

  
	
			dResult = atof (m_info.m_string);
			Scan ();
			// dResult *= 0.1; //mp ! funny stuff !
            while (dResult >= 1.0)
			dResult *= 0.1;
			return dResult;
			*/

		////SON Tran Artemis
			char Helper[16];  //a float number has maximum 16 decimal digits
			strcpy(Helper,(LPCSTR)(m_info.m_string));
			int Counter = strlen(Helper);
			dResult = atof (Helper);
			Scan ();
			while(Counter-- >0)
				dResult*= 0.1;
			return dResult;

		}
	}
	// PB end

	if (m_info.m_code != TID_NUMBER && m_info.m_code != TID_INTEGER)
	{	Error ("Number expected");
		return 0;
	}

	dResult = atof (m_info.m_string);
	Scan ();
	return dResult;
}

/*-----------------------------------------------------------------
If quotes are not used, read all characters till a white space 
-----------------------------------------------------------------*/
const ZString &Token :: GetString ()
{
	m_pParser -> GetString (m_string);
	return m_string;
}
	
/*-----------------------------------------------------------------
Scan a symbol and find its code
-----------------------------------------------------------------*/
void Token :: Scan ()
{
	if (m_info.m_code == TID_EOF)
		throw ExceptionEndOfFile ();

	for (;;)
	{	m_pParser -> GetToken (&m_info);

		/*-------------------------------------------------------------
		if the returned token has been parsed from a new source line, copy that
		line to the list file
		-------------------------------------------------------------*/
		if (m_info.m_nLine > m_nLineNumber)
		{	while (++m_nLineNumber < m_info.m_nLine)
				if (m_fList)
					putc ('\n', m_fList);
			if (m_fList)
				fputs (m_pParser -> GetLine (), m_fList);
		}

		/*-------------------------------------------------------------
		Return the parsed token or print an error coce if the token is not recognized
		-------------------------------------------------------------*/
		if (m_info.m_code != TID_UNKNOWN)
			return;
		Error (NULL);
	}
}

/*-----------------------------------------------------------------
Find a name in a list of defined name
-----------------------------------------------------------------*/
int Token :: GetDefinedName () const
{
	for (int i = m_defined.GetLength (); --i >= 0;)
		if (m_info.m_string == m_defined [i].m_name)
			return i;
	return -1;
}

/*-----------------------------------------------------------------
Find a name in a list of defined route
-----------------------------------------------------------------*/
int Token :: GetDefinedRoute () const
{
	for (int i = m_routes.GetLength (); --i >= 0;)
		if (m_info.m_string == m_routes [i])
			return i;
	return -1;
}

/*-----------------------------------------------------------------
Find a name in a list of defined name
-----------------------------------------------------------------*/
int Token :: GetDefinedProto (MediaObject **ppObject) const
{
	for (int i = m_protos.GetLength (); --i >= 0;)
		if (m_info.m_string == m_protos [i]->m_name) {
			if (ppObject)
				*ppObject = m_protos [i]->m_pObject;
			return i;
		}
	return -1;
}

/*-----------------------------------------------------------------
Display an error message in the list file
-----------------------------------------------------------------*/
void Token :: Error (LPCSTR szDesc, LPCSTR szItem)
{
	fprintf (
		m_fList,
		"ERROR: %s%s%s\n",
		szItem ? szItem : (LPCSTR) m_info.m_string,
		szDesc != NULL ? " - " : "",
		szDesc != NULL ? szDesc : ""
	);
	m_bHadErrors = TRUE;
}

/*-----------------------------------------------------------------
Add a name to the list of predefined name

MBS: modified to take into account defined nodeID in the script 
     in the forme text_nnn where nnn is the specified nodeID (nodeID < 1024)
Zvi (29/4/98) - Fixed the mess.
-----------------------------------------------------------------*/
int Token :: AddPredefinedName ()
{
	int nResult, nID;

	/*-------------------------------------------------------------
	Verify the a predefined slot is empty, or find an empty slot
	-------------------------------------------------------------*/
	if (!_strnicmp ("id_", m_info.m_string, 3) &&
		(nID = atoi ((LPCSTR) m_info.m_string + 3)) > 0
	)
	{	if (nID < m_defined.GetLength () && m_defined [nID].m_nCode != INT_MAX)
			return -1;
		nResult = nID;
	}
	else for (nResult = 0; nResult < m_defined.GetLength (); nResult++)
		if (m_defined [nResult].m_nCode == INT_MAX)
			break;

	m_defined.SetAt (nResult, DefinedNode ());
	m_defined [nResult].m_name = m_info.m_string;
	m_defined [nResult].m_nCode = 0;
	return nResult;
}

/*-------------------------------------------------------------
Add a name to the list of defined Protos
-------------------------------------------------------------*/
int Token :: AddPredefinedProto (Proto *pObject)
{
	int nResult = m_protos.GetLength ();
	DefinedProto *entry = new DefinedProto;
	entry->m_name = m_info.m_string;
	entry->m_pObject = pObject;
	m_protos += entry;
	return nResult;
}

/*-----------------------------------------------------------------
Add a name to the list of predefined routes
-----------------------------------------------------------------*/
int Token :: AddPredefinedRoute ()
{
	int nResult = m_routes.GetLength ();
	m_routes += m_info.m_string;
	return nResult;
}

/*-------------------------------------------------------------
Destructor for the DefinedProto structure
-------------------------------------------------------------*/
Token :: DefinedProto :: ~DefinedProto ()
{
// PORTING to CC 5.3
// Apparent problems with Proto :: ~Proto
// Right now commenting the stuff for Solaris - PENDING
#if defined(WIN32) || !defined(MUX)
	delete m_pObject;
#endif
}

/*-------------------------------------------------------------
Save old values of the token, and prepare it to parse a secondary file
-------------------------------------------------------------*/
void Token :: Push ()
{
	m_stack.m_pParser = m_pParser;
	m_stack.m_info = m_info;
	m_stack.m_nLineNumber = m_nLineNumber;
}

/*-------------------------------------------------------------
Restore old token values after parsing a secondary file
-------------------------------------------------------------*/
void Token :: Pop ()
{
	m_pParser = m_stack.m_pParser;
	m_info = m_stack.m_info;
	m_nLineNumber = m_stack.m_nLineNumber;
}

/*-------------------------------------------------------------
Reset the object for a new SceneReplace command
-------------------------------------------------------------*/
void Token :: Reset ()
{
	m_defined.Clear ();
	m_routes.Clear ();
	m_protos.Clear ();
}


// BF added for debugging

/*MediaObject * Token::GetInstance (LPCSTR scName)
{
	int i=0;
	int len=m_scriptArray.GetLength();
	while (i < len && !m_scriptArray[i]->Is(scName)) i++;
	if (i==len)
	{
		printf("[Token(Script)::GetInstance] Pas trouv : %s \n",scName);
		return NULL;
	}
	return m_scriptArray[i];
	
	scName = 0;
	return NULL;
}*/
/*void Token::AddPredefinedScript (ScriptInterface * scI)
{
	m_scriptArray+= scI;
}
//AC end
*/