/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1997.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

#if !defined(_Token_h)	// Sentry, use file only if it's not already included.
#define _Token_h

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  BIFS Encoder
FILE:       Token.h
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Implementation of Token, the class that handles text parsing.
-------------------------------------------------------------------------*/
//$Log: Token.h,v $
//Revision 1.1.1.2  2006/04/14 14:11:11  hill
//no message
//
//Revision 1.1.1.1  2003/12/24 00:42:06  mikeseven
//Initial release of the CVS repository for MPEG-4 Systems code
//after the code was corrupted due to reorganization and human 
//errors.
//
//Revision 1.4  2003/04/03 04:29:01  mikeseven
//Initial release of Player3D and MPEG-J for all MPEG-4 reference softwares players with samples.
//For installation, follow instructions in docs/Player3D_setup.doc.
//
//Many thanks to Prashanth Gujjeti and Viswanathan Swaminathan from Sun Microsystems.
//
//-- Mikael Bourges-Sevenier (Mindego)
//
//Revision 1.3  2002/10/30 12:23:50  jean_lf
//Nested Proto fixe
//
//Revision 1.2  2002/08/08 09:47:25  zvil
//Core 5.9 Release Candidate 9:
//- Handling of time fields
//- Generating BIFS from memory
//- Proper handling of BIFS tables
//- More IPMPX stuff
//- Fixes to IM1-2D for proper use of startTime / stopTime
//- Fixes to Im1Player for proper use of startTime / stopTime
//
//Revision 1.2.2.1  2000/03/10 15:59:31  Alex Cotarmanac'h
//deuxime envoi
//
#include "Parser.h"

class ScriptInterface;
class MediaObject;
class Proto;

class Token
{
	int m_nLineNumber, m_flags;
	FILE *m_fList;
	BOOL m_bHadErrors;
	LPCSTR m_pParentFile;
	TokenInfo m_info;
	Parser *m_pParser;
	ZString m_string;

// PORTING to CC 5.3
// Make these definitions public
#ifndef WIN32
public:
#endif
	struct DefinedNode
	{
		ZString m_name;
		int m_nCode;
		MediaObject *m_pInstance;
		BOOL operator == (const DefinedNode &) const {return FALSE;}
		DefinedNode () : m_nCode (INT_MAX), m_pInstance (NULL) {}
	};

	struct DefinedProto
	{
		ZString m_name;
		Proto *m_pObject;
		BOOL operator == (const DefinedNode &) const {return FALSE;}
		DefinedProto () : m_pObject (NULL) {}
		~DefinedProto ();
	};

	ZArray<DefinedNode> m_defined;
	ZArray<ZString> m_routes;
	ZArray<DefinedProto *> m_protos;

public:
	Token () : m_bHadErrors (FALSE), m_pParentFile (NULL) {}
	~Token ();
	void Reset ();
	void SetFlags (int flags) {m_flags = flags;}
	int GetFlags () const {return m_flags;}
	void Init (Parser *pParser, LPCSTR szRootFile = NULL, FILE *fList = NULL);
	operator int () {return m_info.m_code;}
	ZLargeInt<128> GetInt ();
	bool GetBool ();
	double GetReal ();
	const ZString &GetName () {return m_info.m_string;}
	const ZString &GetString ();
	void Scan ();
	int GetDefinedName () const;
	int GetDefinedRoute () const;
	int GetDefinedProto (MediaObject **ppObject = NULL) const;
	void Error (LPCSTR szDesc, LPCSTR szItem = NULL);
	int AddPredefinedName ();
	int AddPredefinedRoute ();
	int AddPredefinedProto (Proto *pObject);
	void SetNodeID (int nIndex, int nNodeCode, MediaObject *pInstance)
	{
		m_defined [nIndex].m_nCode = nNodeCode;
		m_defined [nIndex].m_pInstance = pInstance;
	}
	int GetNodeID (int nIndex) const {return m_defined [nIndex].m_nCode;}
	MediaObject *GetInstance (int nIndex) const {return m_defined [nIndex].m_pInstance;}
	LPCSTR GetRootFileName () const {return m_pParentFile;}
//AC 200003 Script
	/* Gets the instance of a scriptInterface*/
	//MediaObject *GetInstance (LPCSTR scName); 
	/* Stores the instance of a scriptInterface*/
	//void AddPredefinedScript (ScriptInterface *);
private:
	ZArray<ScriptInterface *> m_scriptArray;
//AC end

public:
	void Push ();
	void Pop ();

private:
	struct TokenStack
	{
		Parser *m_pParser;
		TokenInfo m_info;
		int m_nLineNumber;
	} m_stack;
};

class ExceptionEndOfFile {};

#ifndef MAIN
extern Token nextToken;
#endif	// MAIN

#endif                                      // _Token_h sentry.
