/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  BIFSEncoder
FILE:       main.cpp
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   main() function for the BIFS encoder.
-------------------------------------------------------------------------*/
//$Log: main.cpp,v $
//Revision 1.1.1.2  2006/04/14 14:11:10  hill
//no message
//
//Revision 1.1.1.1  2003/12/24 00:42:05  mikeseven
//Initial release of the CVS repository for MPEG-4 Systems code
//after the code was corrupted due to reorganization and human 
//errors.
//
//Revision 1.2  2003/04/14 16:45:34  jean_lf
//RAP signal support, BIFSv2 encode fix
//
//Revision 1.1.1.2  2003/01/30 14:45:19  payan
//no message
//
//Revision 1.1.1.1  2002/07/18 00:59:55  mikeseven
//Initial check in of July 17, 2002 version
//
//Revision 1.2  2000/03/03 18:41:21  Alex Cotarmanac'h
//Version 1 -- Alexandre Cotarmanac_h
//
#include "defs.h"
#include "BifsEnc.h"
#include "ODStrEnc.h"

#define MAIN
#include "Token.h"
#include "VrmlPars.h"
#include "XMTPars.h"

Token nextToken;


void printusage()
{
	printf(
		"BifsEnc in_filename [-option ...]\n"
		"\nImplemented options:\n"
		"v: trace verbose\n"
		"n: use codenames with DEF\n"
		"p: enable predictive MFField encoding\n"
		"r: use version2 of BIFS TS (binary output) to signal RAP AUs\n"
		"\nWritten by Zvi Lifshitz\n"
	);
}

int main (int argc, char **argv)
{
	FILE *fBifs, *fList, *fLogs;
	int nNameLength;
	int flags = TRACE_NONE;
	bool bForceVRML = false;
	bool bUseRAP = false;

	if (argc < 2)
	{	printf ("Missing file name\n");
		printusage();
		return -1;
	}
	ZString stFileName (*++argv);

	/*-------------------------------------------------------------
	Parse the flags
	-------------------------------------------------------------*/
	if (argc >= 3 && (*++argv)[0] == '-')
	{	int c;
		for (int i = 1; (c = (*argv) [i]) != 0; i++)
			switch (tolower (c)) {
			case 'v':
				flags |= TRACE_VERBOSE;
				break;
			case 'n':
				flags |= CODENAMES;
				break;
			//AC 19991117
			case 'p':
				flags |= PREDICTIVE_FLAG;
				break;
			//AC end
            //mp conformance output
			case 'c':
				flags |= TRACE_CONFORMANT;  
				break;
            //mp end
			case 'x':
				flags |= FORMAT_XMT;
				break;
			case 'w':
				bForceVRML = true;
				break;
			/*JLF: added support for RAP indication (-2 option)*/
			case 'r':
				bUseRAP = true;
				break;
			}
	}

	/*-----------------------------------------------------------------
	Figure out the source file name and the input format (VRML or XMT). The format can
	be forced by a flag in the command line argument (w-VRML, x-XMT). In this case the file
	name extension defaults are .txt and .xmt respectively. If a flag is not given, the
	format is determined by the file name extension: .txt or .wrl for VRML, .xmt or .xml for
	XMT. If the extension is none of these, the default format is VRML and the default
	extension is .txt
	-----------------------------------------------------------------*/
	nNameLength = stFileName.GetLength ();
	LPCSTR szExt = strrchr (stFileName, '.');
	if (szExt && !strchr (szExt, '\\'))
	{	nNameLength -= strlen (szExt);
		if (!bForceVRML && (!stricmp (szExt, ".xmt") || !stricmp (szExt, ".xml")))
			flags |= FORMAT_XMT;
	}
	else
		stFileName += flags & FORMAT_XMT ? ".xmt" : ".txt";

	nextToken.SetFlags (flags);
	Parser *pParser;
	if (flags & FORMAT_XMT)
		pParser = new XMTParser;
	else
		pParser = new VRMLParser;
	 
	if (!pParser ->Init (stFileName))
	{	printf ("Cannot open %s\n", (LPCSTR) stFileName);
		goto exit;
	}

	/*-----------------------------------------------------------------
	Output file name extension is "bif" and list file is "lst"
	-----------------------------------------------------------------*/
	stFileName.Cut (nNameLength);
	stFileName += ".bif";
	if ((fBifs = fopen (stFileName, "wb")) == NULL)
	{	printf ("Cannot open %s\n", (LPCSTR) stFileName);
		goto exit;
	}
	stFileName.Cut (nNameLength);
	stFileName += ".lst";
	if ((fList = fopen (stFileName, "w")) == NULL)
	{	printf ("Cannot open %s\n", (LPCSTR) stFileName);
		goto exit;
	}
	stFileName.Cut (nNameLength);
	stFileName += ".log";
	if ((fLogs = fopen (stFileName, "w")) == NULL)
	{	printf ("Cannot open %s\n", (LPCSTR) stFileName);
		goto exit;
	}

	{	/*-------------------------------------------------------------
		The encoding is performed here
		-------------------------------------------------------------*/
		nextToken.Init (pParser, stFileName, fList);
		ODStreamEncoder odEncoder ("od", fList, "OD", flags);
		odEncoder.m_bSignalRAP = bUseRAP;
		ODStreamEncoder ociEncoder ("oci", fList, "OCI", flags);
		ociEncoder.m_bSignalRAP = bUseRAP;
		BIFSEncoder encoder (fBifs, fList, fLogs, "BIFS", flags, &odEncoder, &ociEncoder);
		encoder.m_bSignalRAP = bUseRAP;
		encoder.Encode ();
	}
	fclose (fBifs);
	fclose (fList);
    fclose (fLogs); // mp conformance

	/*-------------------------------------------------------------
	Finish here
	-------------------------------------------------------------*/
exit:
	pParser ->Shutdown ();
	delete pParser;
	return 0;
}