/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) and edited by Mauro Rossi (CSELT), Guido Franceschini
(CSELT), George Tselikis (Xbind Inc.) and Michael Stepping (FTK)
in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Core
FILE:       AL.cpp
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Implementation of ALManager
-------------------------------------------------------------------------*/
#include <iostream.h>

#include "defs.h"
#include "AL.h"
#include "Channel.h"
#include "Executiv.h"
#include "StreamCo.h"
#include "ODImp.h"
#include "Stream.h"
#include "Inline.h"
#include "DAI.h"
#include "IPMPXinterface.h"

/*-----------------------------------------------------------------
Create a DataChannel object, configure it, and add it to a list
of channels waiting to be opened.
-----------------------------------------------------------------*/
//AC:20021009 - SL extension : AbstractSLConfig  -- begin
DataChannel *ALManager :: AttachStream (
	ES_DescriptorImp *pESD,
	MediaStreamImp *pStream,
	/*SLConfigDescriptor*/AbstractSLConfig *pConfigParams,
	int nExpectedAULength,
	bool bBackChannel
)
//AC:20021009 - SL extension : AbstractSLConfig  -- end
{
	int nESid = pESD -> ES_ID;
	DataChannel *pChannel = new DataChannel (this, nESid, pESD, bBackChannel);
	m_mtxChannels.P ();
	m_channels += pChannel;
	//AC:20021009 - SL extension -- begin
	//BUG: if multiple streams share the same ESID.
	//e.g: one stream is added, some time later deleted 
	//and a new stream is added later with the same ESID.
	struct ExtendedSLInfo slinfo;
	slinfo.ESID=nESid;
	slinfo.nMarkers=0;
	if (pConfigParams->GetTag() == TAG_ExtSLConfigDescr)
	{
			ExtSLConfigDescr *  extSL	= (ExtSLConfigDescr *) pConfigParams;
			int					nCount	= extSL->slextDescr.GetLength();
			for (int i=0;i<nCount;i++)
			{
				if (extSL->slextDescr[i]->GetTag()==TAG_MarkerDescriptor)
				{
					slinfo.nMarkers++;
				}
			}
	}
	if (slinfo.nMarkers>1)
	{
		fprintf(stderr,"[SL Extension] More than 1 marker\n");

		//should be a NON_FATAL_ERROR
	}
	m_slinfos +=slinfo; 	
	//AC:20021009 - SL extension -- end
	ClockReference *pClock = pESD -> GetClock ();
	/*-------------------------------------------------------------
	Attach the stream to the proper clock reference
	-------------------------------------------------------------*/
	if (pStream)
		pStream -> SetClockReference (pClock);
	pChannel -> Config (pStream, pConfigParams, nExpectedAULength, pClock);
	m_mtxChannels.V ();
	return pChannel;
}

/*-----------------------------------------------------------------
Request the opening of all the channels which are marked READY.
If service is not connected yet, only mark the channels as "required"
If service has been detached, reattach it.
-----------------------------------------------------------------*/
void ALManager :: RequestChannels ()
{
	bool bReadyChannels = false;
	m_mtxChannels.P ();
	for (int i = 0; i < m_channels.GetLength (); i++)
		bReadyChannels |= m_channels [i] -> Request ();
	m_mtxChannels.V ();
	if (bReadyChannels)
	{	if (m_state == SS_IDLE)
			Connect ();
		if (m_state == SS_CONNECTED)
			AddChannels ();
	}
}

/*-------------------------------------------------------------
Request the service to add all the channels that are marked "required"
-------------------------------------------------------------*/
void ALManager :: AddChannels ()
{
	DAIChannelAddParam inParam;
	
	m_mtxChannels.P();
	for (int i = 0; i < m_channels.GetLength(); i++)
	{
		if(m_channels[i] -> IsRequired())
		{
			DAIChannelAddElement &inEl = ++inParam;
			inEl.m_pData = m_channels [i] -> GetuuData ();
			inEl.m_nDataLen = m_channels [i] -> GetuuDataLength ();
			inEl.m_pQoSStream = m_channels [i] -> GetQoS() ;
			inEl.m_eDirection = m_channels [i] -> IsBackChannel () ? UPSTREAM : DOWNSTREAM;
			inEl.m_nTmpId	= (UINT) m_channels [i];
			inEl.m_pAppInfo = (BYTE*) m_channels [i] -> GetSLConfig ();
			//inEl.m_nAppInfoLen = sizeof SLConfigDesc;
 			inEl.m_nAppInfoLen = sizeof(SLConfigDesc); // SOLARIS PORTING - use sizeof()
		}
	}
	m_mtxChannels.V();

	if (inParam.GetLength() > 0)
		m_pExecutive -> ChannelAdd (m_nServiceID, inParam);
}

/*-----------------------------------------------------------------
When service is connected, request all pending channels and notify
the application.
-----------------------------------------------------------------*/
void ALManager :: OnServiceAttach (
	UINT nServiceID, 
	const BYTE* pUuData,
	int nUuDataLen,
	RESPONSE eResponse
)
{
	if (eResponse == RESPONSE_OK)
	{	
		m_nServiceID = nServiceID;
		m_state = SS_CONNECTED;
		AddChannels ();		
		/*-------------------------------------------------------------
		If this is a secondary session with remoteOD, notify the node
		that the session is established.
		-------------------------------------------------------------*/
		if (m_pOD)
			m_pOD -> OnConnect (pUuData, nUuDataLen);
	}
	else
	{	/*-------------------------------------------------------------
		Notifies the application and mark the service as idle if failed.
		-------------------------------------------------------------*/
		m_pExecutive -> Error (INVALID_URL, (LPCSTR) m_url);
		ChannelsClosed ();
		m_state = SS_UNAVAILABLE;
		m_pExecutive ->DeleteService (this);
	}
}

/*-------------------------------------------------------------
Close all channels when DMIF reports service detach
-------------------------------------------------------------*/
void ALManager :: OnServiceDetach (REASON reason)
{
	ChannelsClosed ();
	m_state = SS_IDLE;
	m_pExecutive ->DeleteService (this);
	if (reason != REASON_OK)
	{	m_pExecutive -> Error (UNEXPECTED_SERVICE_DETACH);
		m_state = SS_UNAVAILABLE;
	}
}

/*-----------------------------------------------------------------
When channels are added notify the DataChannel objects.
-----------------------------------------------------------------*/
void ALManager :: OnChannelAdd (const DAIChannelAddParam &cnfParam)
{
	for (int j=0; j < cnfParam.GetLength(); j++)
	{	DataChannel *pChannel = NULL;
		m_mtxChannels.P ();
		for (int i = m_channels.GetLength (); --i >= 0;)
		{
			if ((UINT) m_channels [i] == cnfParam [j].m_nTmpId)
			{	pChannel = m_channels [i];
				pChannel -> Lock ();
				break;
			}
		}
		m_mtxChannels.V ();
		if (pChannel != NULL) {
			DAIChannelAddElement *pEl = &cnfParam[j];
			if (pEl->m_nAppInfoLen) {
				const char *str = strstr((const char *) cnfParam [j].m_pAppInfo, "duration");
				if (str) {
					float dur;
					sscanf(str, "duration=%f\r\n", &dur);
					assert(dur>=0);
					pChannel->m_Duration = dur;
				}
			}
			pChannel -> OnChannelOpen (cnfParam [j].m_nChid, cnfParam [j].m_eResponse);
			pChannel -> Unlock ();
		}
	}
}

/*-------------------------------------------------------------
When service terminated notify all the channels to close
-------------------------------------------------------------*/
void ALManager :: ChannelsClosed ()
{
	m_mtxChannels.P ();
	for (int i = m_channels.GetLength (); --i >= 0;)
		m_channels [i] -> OnChannelDelete (REASON_OK);
	m_mtxChannels.V ();
}

/*-----------------------------------------------------------------
Connect to the given URL
-----------------------------------------------------------------*/
void ALManager :: Connect ()
{
	m_state = SS_CONNECTING;
	m_pExecutive -> ServiceAttach (this, m_url, m_nParentServiceID);
}

/*-----------------------------------------------------------------
Request a service detach
-----------------------------------------------------------------*/
void ALManager :: Disconnect ()
{
	/*-------------------------------------------------------------
	Remove all IPMPX descriptors
	-------------------------------------------------------------*/
	for (int i = m_IPMPDXs.GetLength (); --i >= 0;)
	{	if (m_pIPMP != NULL)
			m_pIPMP -> RemoveIPMPDescriptor (m_IPMPDXs [i]);
		delete m_IPMPDXs [i];
	}

	/*-------------------------------------------------------------
	Remove the IPMP system attached to this service
	-------------------------------------------------------------*/
	if (m_pIPMP)
		m_pExecutive ->GetIPMPSystem () -> RemoveIPMPServices (m_pIPMP);

	/*-------------------------------------------------------------
	Remove the service
	-------------------------------------------------------------*/
	bool bWasConnected = false;
	m_pExecutive ->DeleteService (this);
	InUse ();
	Lock ();
	if (m_state == SS_CONNECTING || m_state == SS_CONNECTED)
	{	m_state = SS_IDLE;
		bWasConnected = true;
	}
	Unlock ();
	if (bWasConnected)
		m_pExecutive -> ServiceDetach (this, REASON_OK);
}

/*-----------------------------------------------------------------
Make sure that the service is closed before the object is deleted.
-----------------------------------------------------------------*/
ALManager :: ~ALManager ()
{
	Disconnect ();
	while (m_nInUse) Sleep (5);
}

/*-------------------------------------------------------------
This function is called by DMIF in order to notify the application
about the result of the channel release request.
-------------------------------------------------------------*/
void ALManager :: OnChannelDeleteCnf (const DAIChannelDeleteParam& cnfParam)
{
	for (int j=0; j < cnfParam.GetLength(); j++)
		if (cnfParam [j].m_eResponse != RESPONSE_OK)
			m_pExecutive ->Error (CHANNEL_DELETE_FAILED);
}

/*-------------------------------------------------------------
Remove a channel from the list
-------------------------------------------------------------*/
void ALManager :: DeleteChannel (DataChannel *pChannel)
{
	m_mtxChannels.P ();
	m_channels -= pChannel;
	m_mtxChannels.V ();
}

static UINT nCommands = 0;
/*-------------------------------------------------------------
This function is called in order to send a user command,like PLAY,to the server.
This user command should be always referred to an already established channel.
For i.e in case of PLAY command this channel is going to receive the data.
-------------------------------------------------------------*/
void ALManager :: SendCommand (char *szCommand, UINT nChannelID)
{
	DAIUserCmdParam inParam;
	inParam.m_nCmdId = nCommands++;
	inParam.m_pChid = &nChannelID;
	inParam.m_nChLen = 1;
	inParam.m_nDataLen = strlen (szCommand) + 1;
	inParam.m_pData = szCommand;
	m_pExecutive -> SendCommand (GetServiceID (), inParam);
}
 
/*-------------------------------------------------------------
Request the DAI to delete a channel
-------------------------------------------------------------*/
void ALManager :: RequestChannelDelete (DataChannel *pChannel)
{
	m_pExecutive ->DeleteChannel (pChannel);
	DAIChannelDeleteParam param;
	DAIChannelDeleteElement &element = ++param;
	element.m_nChid = pChannel ->GetChannelID ();
	element.m_nTmpId = (UINT) pChannel;
	element.m_eResponse = RESPONSE_OK;
	element.m_eReason = REASON_OK;
	m_pExecutive ->ChannelDelete (m_nServiceID, param);
}

/*-------------------------------------------------------------
Display a message error in case user command rejected
-------------------------------------------------------------*/
void ALManager :: OnUserCommandCnf (const DAIUserCmdParam &outParam)
{
	if (outParam.m_eResponse != RESPONSE_OK)
	{	m_mtxChannels.P ();
		for (int j = outParam.m_nChLen; --j >= 0;)
		{	for (int i = m_channels.GetLength (); --i >= 0;)
				if (m_channels [i] ->GetChannelID () == outParam.m_pChid [j])
				{	m_channels [i] -> OnUserCommandReject (outParam.m_eResponse);
					break;
				}
		}
		m_mtxChannels.V ();
	}
}

/*-------------------------------------------------------------
Send data on a back channel
-------------------------------------------------------------*/
void ALManager :: DataSend (LPBYTE pData, int nLength, UINT nChannelID, int nAUnumber)
{
	m_pExecutive -> DataSend (m_nServiceID, nChannelID, pData, nLength, nAUnumber);
}

/*-------------------------------------------------------------
Generate and send a PLAY command (or PREROLL command if stream is not really playing yet)
-------------------------------------------------------------*/
void ALManager :: StartChannel (
	UINT nChannelID,
	double dStart,
	double dStop,
	double dSpeed,
	bool bPlay
)
{
	char szCommand [100];
	char szSpeed [20];
	char *szCmd = bPlay ? "PLAY" : "PREROLL";
	if (dStop == FLT_MAX)
		dStop = -1;
	if (dStart < 0) {
		if (dStop < 0)
			sprintf (szCommand, "%s ", szCmd);
		else
			sprintf (szCommand, "%s range=-%lf", szCmd, dStop);
	}	else
		if (dStop < 0)
			sprintf (szCommand, "%s range=%lf-", szCmd, dStart);
		else
			sprintf (szCommand, "%s range=%lf-%lf", szCmd, dStart, dStop);
	if (dSpeed != 1)
	{	sprintf (szSpeed, ";speed=%lf", dSpeed);
		strcat (szCommand, szSpeed);
	}
	SendCommand (szCommand, nChannelID);
}

/*-------------------------------------------------------------
Instantiate an IPMPX tool manager for this service
-------------------------------------------------------------*/
void ALManager :: SetIPMPSystem (IPMP_ToolListDescriptor *pToolList)
{
	if (m_pExecutive -> GetIPMPSystem ())
		m_pIPMP =
			m_pExecutive -> GetIPMPSystem () -> CreateIPMPServices (m_url, pToolList);
}

/*-------------------------------------------------------------
An IPMP_ToolDescriptorUpdate command was detected. Hand over each of the descriptor
to the associated IPMPX system
-------------------------------------------------------------*/
bool ALManager :: UpdateIPMPDescriptor (IPMP_Descriptor *pIPMPD)
{
	if (!m_pIPMP)
		return false;

	int ID = pIPMPD ->IPMP_DescriptorID;

	if (!m_pIPMP ->ProcessIPMPDescriptor (pIPMPD))
	{	if ( ID == 0xff )
			ID = pIPMPD -> IPMP_DescriptorIDEx;
		GetExecutive () -> Error (
			IPMP_CANNOT_PROCESS_IPMP_TOOL,
			ID
		);
		delete pIPMPD;
	}
	else
	{	/*-------------------------------------------------------------
		If an old descriptor with same ID exists replace it, otherwise add a new one
		to the list
		-------------------------------------------------------------*/
		for (int i = m_IPMPDXs.GetLength (); --i >= 0;)
			if (pIPMPD ->IPMP_DescriptorID == m_IPMPDXs [i] -> IPMP_DescriptorID
				&& pIPMPD ->IPMP_DescriptorIDEx == m_IPMPDXs [i] -> IPMP_DescriptorIDEx)
			{	delete m_IPMPDXs [i];
				m_IPMPDXs.SetAt (i, pIPMPD);
				return true;
			}
		m_IPMPDXs += pIPMPD;
	}
	return true;
}	

/*-------------------------------------------------------------
An IPMP_DescriptorRemove command was detected. Hand over each of the descriptor IDs
to the associated IPMPX system.
If an IPMPX system is not available return false so that the calling function will
handle it as an IPMP-hooks entity.
-------------------------------------------------------------*/
bool ALManager :: RemoveIPMPDescriptor (int nDescriptorID)
{
	if (!m_pIPMP)
		return false;

	for (int i = m_IPMPDXs.GetLength (); --i >= 0;)
		if (nDescriptorID == m_IPMPDXs [i] -> IPMP_DescriptorID)
		{	if (m_pIPMP != NULL && !m_pIPMP ->RemoveIPMPDescriptor (m_IPMPDXs [i]))
				GetExecutive () -> Error (IPMP_CANNOT_REMOVE_IPMP_TOOL, nDescriptorID);
			delete m_IPMPDXs [i];
			m_IPMPDXs.DeleteFrom (i);
			return true;
		}
	return true;
}	
