/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

#if !defined(_app_h)	// Sentry, use file only if it's not already included.
#define _app_h

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Application
FILE:       App.h
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Definition of Application
-------------------------------------------------------------------------*/

#include "Im1defs.h"
#include "DAI.h"
#include <stdarg.h>

enum SensorTypes
{
	SENSOR_NONE=0,
	SENSOR_TOUCH,
	SENSOR_DISC,
	SENSOR_PLANE,
	SENSOR_PROXIMITY,
	SENSOR_ANCHOR,
};

class Application
{
public:
	virtual void OnConnect (
		RESPONSE eResponse = RESPONSE_OK
	) = 0;
	virtual void OnFinish (
		RESPONSE eResponse = RESPONSE_END_OF_DATA
	) = 0;
	virtual void OnError (NON_FATAL_ERROR, va_list) = 0;

	//JLF: new callbacks
	virtual void OnFatalError (FATAL_ERROR, LPCSTR msg) = 0;
	//anchor clicked, application should browse to given URL. 
	//return non-0 if browsing possible, 0 otherwise. All URLs of the Anchor are called
	//sequentially until the app returns non-0
	virtual int Navigate(LPCSTR szAddress) = 0;
	//notify the app the cursor is over a sensor or has left it. If over, specify sensor type
	virtual void OverSensor(unsigned int sensorType) = 0;
	//notify app size should be changed
	virtual void SetSize(unsigned int width, unsigned int height) = 0;

	//enables external viewport settings. the app will dynamically send set_bound events
	virtual void ViewportStackChanged(class BindableStack *stack) = 0;
	
	//script support extension
	//returns name of the player
	virtual const char *GetApplicationName() = 0;
	virtual const char *GetApplicationVersion() = 0;
	virtual const char *GetWorldURL() = 0;
};

#endif                                      // _app_h sentry.
