/***********************************************************************
This software module was originally developed by Jean Le Feuvre
(ENST) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. ENST retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003.
************************************************************************


/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  BIFS Decoder
FILE:       BifsDecEx.cpp
AUTHOR:     Jean Le Feuvre, ENST
OVERVIEW:   Implementation of Extended BIFS Updates
-------------------------------------------------------------------------*/

#include "defs.h"
#include "BifsDec.h"
#include "Root.h"
//#include "except.h"
#include "Except.h" // SOLARIS PORTING - typo

/*for TRACE macro*/
#define pCoder this

void BIFSDecoder :: ParseExtendedUpdate()
{
	int nID;
	MediaObject *pNode;

	switch (ParseInt (8, "extendedType")) {
	/*proto list insertion*/
	case 0:
		m_pScene->ParseProtoList(this, TRUE);
		break;
	/*proto list deletion*/
	case 1:
		ParseProtoListDelete();
		break;
	/*all proto deletion*/
	case 2:
		m_pScene->RemoveProto(-1);
		break;
	/*multiple indexed field replacement*/
	case 3:
		nID = ParseInt(m_nNodeIDbits, "nodeID");
		pNode = m_pScene -> FindNode (nID);
		if (!pNode)
			throw BifsException (BIFX_NODEID_NOT_FOUND);
		pNode->MultipleReplaceIndexedValue(this);
		break;
	/*multiple field replacement*/
	case 4:
		nID = ParseInt(m_nNodeIDbits, "nodeID");
		pNode = m_pScene -> FindNode (nID);
		if (!pNode)
			throw BifsException (BIFX_NODEID_NOT_FOUND);
		/*simply call parse since the command has the same syntax as SFNode field encoding*/
		pNode->Parse(this);
		break;
	/*Global Quantization*/
	case 5:
	{
		MediaObject *pNode;
		SFWorldNode node;
		node.ParseNodeField(this, m_pScene, NULL, FALSE);
		pNode = node.GetNode();

		if (!pNode || (pNode->GetNodeCode() != CODE_QuantizationParameter)) {
			SetCurrentQP(NULL, TRUE);
		} else {
			SetCurrentQP(pNode, TRUE);
			//QP node
			node = NULL;
		}
	}
		break;
	/*extended node delete*/
	case 6:
		nID = ParseInt(m_nNodeIDbits, "nodeID");
		pNode = m_pScene -> FindNode (nID);
		if (!pNode)
			throw BifsException (BIFX_NODEID_NOT_FOUND);
		TRACEF ("Delete %s", pNode ->GetNameID ());
		pNode->Delete(TRUE);
		break;
	}
}


void BIFSDecoder::ParseProtoListDelete()
{
	int moreProto, protoID;
	if (ParseInt(1, "listDescription")) {
		moreProto = ParseInt(1, "moreProto");
		while (moreProto) {
			protoID = ParseInt(m_nProtoIDbits, "protoID");
			m_pScene->RemoveProto(protoID);
			moreProto = ParseInt(1, "moreProto");
		}
	} else {
		int len = ParseInt(5, "len");
		int count = ParseInt(len, "numPROTOs");
		for (int i=0; i<count;i++) {
			protoID = ParseInt(m_nProtoIDbits, "protoID");
			m_pScene->RemoveProto(protoID);
		}
	}
}
