/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Optibase Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Optibase Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2002.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  MUW
FILE:       BifsGen.cpp
AUTHOR:     Zvi Lifshitz (Optibase Ltd.)
OVERVIEW:   Implementation of BifsGenerator. A class that encodes scene memory image to BIFS
-------------------------------------------------------------------------*/
#include "defs.h"
#include "BifsGen.h"
#include "Root.h"
//#include "Bifs.h"
#include "bifs.h" // SOLARIS PORTING - typo

/*-------------------------------------------------------------
Constructor of an object that can encode BIFS from a scene memory image
-------------------------------------------------------------*/
BifsGenerator :: BifsGenerator (
	int nNodeIDbits,
	int nRouteIDbits,
	int nProtoIDbits,
	FILE *fList,
	LPCSTR szListTitle,
	int flags,
	int version,
    FILE * fLogs
) :
	SysCoder (NULL, fList, szListTitle, flags, version, fLogs),
	m_nNodeIDbits (nNodeIDbits),
	m_nRouteIDbits (nRouteIDbits),
	m_nProtoIDbits (nProtoIDbits)

{
	m_nEncodeCounter = 0;
	m_pProto = NULL;
}


/*-------------------------------------------------------------
Generate a scene replace command with pScene as the root of the scene
-------------------------------------------------------------*/
void BifsGenerator::GenerateSceneReplace (Root *pScene)
{
	++m_nEncodeCounter;
	EncodeInt (3, 2, "SceneReplaceCommand");
	EncodeInt (0, 6, "reserved");
	EncodeInt ((GetFlags () & CODENAMES) != 0, 1, "CODENAMES");
	pScene -> GenerateScene (this);
	Align ();
}

/*-------------------------------------------------------------
Encode a String value
-------------------------------------------------------------*/
void BifsGenerator :: EncodeString (const ZString &string)
{
	for (int i = 0; i < string.GetLength (); i++)
	{
		unsigned char c = string [i];
		EncodeInt (c, 8, (ZString ("value [") += c) += ']');  
	}
	EncodeInt (0, 8, "end of string");
}

/*-------------------------------------------------------------
Encode a route deletion command when BIFS is generated from a memory image
nID is the ID of the route to delete
-------------------------------------------------------------*/
void BifsGenerator :: GenerateRouteDeletion (int nID)
{
	EncodeInt (1, 2, "code", "DeletionCommand");
	EncodeInt (3, 2, "parameterType", "RouteDeletion");
	EncodeInt (nID, GetRouteIDBits (), "routeID");
}

/*-------------------------------------------------------------
Encode a node deletion command when BIFS is generated from a memory image
nID is the ID of the node to delete
-------------------------------------------------------------*/
void BifsGenerator :: GenerateNodeDeletion (int nID)
{
	EncodeInt (1, 2, "code", "DeletionCommand");
	EncodeInt (0, 2, "parameterType", "NodeDeletion");
	EncodeInt (nID, GetRouteIDBits (), "routeID");
	EncodeInt (nID, GetNodeIDBits (), "nodeID");
}
