/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) and edited by Mauro Rossi (CSELT) and Michael 
Stepping (FTK) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

#if !defined(_channel_h)	// Sentry, use file only if it's not already included.
#define _channel_h

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Adaptation Layer
FILE:       Channel.h
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Declarations of Channel and DataChannel
-------------------------------------------------------------------------*/
#include "ODDec.h"
#include "DAI.h"
#include "mpg4dmif.h"

class MediaStreamImp;
struct SLConfigDesc;
class SLConfigDescriptor;
//AC:20021009 - SL extension : AbstractSLConfig  -- begin
class AbstractSLConfig;
//AC:20021009 - SL extension : AbstractSLConfig  -- end
class ALManager;
class ClockReference;
class ES_Descriptor;
class QoS_Descriptor;

/*-------------------------------------------------------------
Interface for the Channel object. The object that handles stream related
events
-------------------------------------------------------------*/

class DataChannel
{
	ZMutex m_mtxLock;
	MediaStreamImp *m_pStream;
	ClockReference *m_pClock;
	ALManager *m_pAL;
	SLConfigDesc m_config;
//AC:20021009 - SL extension : AbstractSLConfig  -- begin
	/*SLConfigDescriptor*/ AbstractSLConfig *m_pConfig;
//AC:20021009 - SL extension : AbstractSLConfig  -- end
	DWORD m_dwCTS, m_dwPreCTS, m_dwDTS, m_dwPreDTS, m_dwTimeOffset;
	DWORD m_dwPreOCR, m_dwOCROffset;
	DWORD m_dwPDUSeqNumMask, m_dwAUSeqNumMask;
	int m_nAULength, m_nAUsequenceNumber, m_nPreSN, m_nAUnumber;
	int m_nExpectedAULength, m_nEncodedAULength;
	int m_nExpectedAUSequenceNumber, m_nExpectedPDUSequenceNumber;
	bool m_bWaitRandomAccessPoint, m_bNewAU, m_bIsOCRStream;
	LPBYTE m_pBuffer;
	int m_nESid;
	LPBYTE m_uuData;
	UINT m_uuLength;
	BYTE m_esidAsuuData [2];
	ZString m_qos, m_title;
	float m_fOCRResolutionFactor, m_fTSResolutionFactor;

	bool m_bStreamOpen, m_bOCRInit, m_bBackChannel, m_bPrerolled;

public:
	enum tagChannelState {
		CS_IDLE,		// state after instantiation
		CS_CONFIGURED,	// AttachStream has been called, but channel is not required yet
		CS_READY,		// Channel is required
		CS_REQUIRED,	// can be requested but service did not attach yet
		CS_REQUESTED,	// state when waiting to DMIF to add it
		CS_OPEN,		// state when used by the APP and open by DMIF
		CS_UNAVAILABLE	// could not be retreived by DMIF
	} m_state;
	ODDecoder m_ODDecoder;
	UINT m_nChannelID;
	FILE* m_pFile;	// Guido: log file for miscellanea

//BINDU ---  MPEGJ MERGE_3
public:
#ifdef SIA
	bool  enable;
	int error;
	int credits;
	int retrievedCredit;
#endif
//BINDU ---  MPEGJ MERGE_3

private:
	void CloseStream ();
	void SkipTillRAP ();

public:
	void Close ();

	bool IsOpen () const {return m_state == CS_OPEN;}
	void Play (BOOL bIsPreroll = FALSE);
	LPBYTE GetuuData () const
	{
		return m_uuData;
	}
	UINT GetuuDataLength () const {return m_uuLength;}
	void OnUserCommandReject (int);
	void Lock () {m_mtxLock.P ();}
	void Unlock () {m_mtxLock.V ();}
	void OnPacketReceived (
		const BYTE *pStreamData,
		int nStreamDataLength,
		const BYTE *pAppData,
		int nAppDataLength,
		int nErr
	);

	int DetermineFlags (int flags);
	LPCSTR DetermineTitle (int nESid);
	BOOL IsRequired ();
	bool Request ();
	void OnChannelDelete (REASON reason);
	void ChannelStop ();
	DataChannel (ALManager *pAL, int nESid, ES_Descriptor *pESD, bool bBackChannel);
	~DataChannel ();
	void SetESDParams (ES_Descriptor *pESD);
	int GetESid () const {return m_nESid;}
	UINT GetChannelID () const {return m_nChannelID;}
	LPCBYTE GetQoS () const {return m_qos;}
//AC:20021009 - SL extension : AbstractSLConfig  -- begin
	void Config (
		MediaStreamImp *pStream,
		/*SLConfigDescriptor*/AbstractSLConfig *pConfigParams,
		int nExpectedAULength,
		ClockReference *pClock
	);
	void ChannelReady ();
	void OnChannelOpen (UINT nChannelID, RESPONSE eResponse);
	const SLConfigDesc *GetSLConfig () const {return &m_config;}
	bool IsBackChannel () const {return m_bBackChannel;}
	void DataSend (LPBYTE pData, int nLength);

	/*set prerolled flag for next Play*/
	void SetPrerolled(bool bPreroll) { m_bPrerolled = bPreroll; }

	//length info for downstream whenever specified, -1 otherwise
	double m_Duration;

	float GetBufferOccupancy();
};

#endif                                      // _channel_h sentry.
