/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

#if !defined(_clock_h)	// Sentry, use file only if it's not already included.
#define _clock_h

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Buffer management
FILE:       Clock.h
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Definition of ClockReference.
-------------------------------------------------------------------------*/

#include "ztl.h"

#define NUM_TAPS	5
#define	MAX_DRIFT	.005
#define THREESOLD	100	// ms

class MediaControlImp;
class ObjectDescriptorImp;

class ClockReference
{
	DWORD m_dwRealStartTime;	// the "real" start time, used to estimate drift
	DWORD m_dwStartTime;		// this start time is recomputed each time the STB
								// is corrected, in order to avoid discontinuities
	static int m_nGranularity;
	int m_nOCRLength;
	int m_nAdjust, m_nFrozen;
	DWORD m_dwInitOCR, m_dwLastOCR, m_dwFreezeTime;	
	int m_nID, m_nOpenCounter;
	bool m_bReset, m_bHasOCR;
	double m_dFactor;
	double m_dDriftEstimate [NUM_TAPS];
	double m_dMediaTimeOffset;
	float m_fTransform, m_fSpeed, m_fCumulSpeed;
	/*pointer to inline if this clock is used by one of the streams in inline, NULL otherwise*/
	ObjectDescriptorImp *m_pInlineOD;
	MediaControlImp *m_pControl;

private:
	void CheckForInlineOD();


public:
	/*scale clock - used by flextime only*/
	void Scale (float fFactor);
	/*set clock speed (mediaControl)*/
	void SetSpeed(float fSpeed);
	bool IsActive () const {return m_nFrozen == 0;}
	bool HasOCR () const {return m_bHasOCR;}
	void ModifyClockSpeed (double dFactor) {m_dFactor = dFactor;}
	void Resume ();
	void Pause ();
	void Reset ();
	__int64 AdjustTimeOffset (DWORD dwTS, int nLength, float fResolutionFactor);
	ClockReference (int nID = -1) :
		m_nID (nID), m_nAdjust (50), m_dFactor (1.), m_bHasOCR (false), m_nOpenCounter (0),
		m_nFrozen (0), m_fTransform (1.), m_fSpeed(1.), m_fCumulSpeed(1.), m_dMediaTimeOffset (0), m_pInlineOD(NULL), m_pControl (NULL)
	{}
	~ClockReference ()
	{
		assert (m_nOpenCounter == 0);
	}
	bool IsOCRInit () {return !m_bReset;}
	int GetID () const {return m_nID;}
	operator int () const {return m_nID;}
	DWORD GetTime ();
	void SetTime (DWORD dwTime, int nOCRLength);
	bool WaitOn (ZSemaphore &rsmpControl, DWORD dwTime);
	int CompareTimes (DWORD dwTime, DWORD dwCompareTo = 0);
	static void SetGranularity (int nMS) {m_nGranularity = nMS;}
	void Adjust (DWORD dwTime = 0);
	void SetOCRFlag (bool bHasOCR) {m_bHasOCR = bHasOCR;}
	bool InitFromCTS (DWORD dwTime);
	void Start ();
	void Stop (bool bOCRStream);
	void MapMediaTime (double dMediaTimeOffset) {m_dMediaTimeOffset = dMediaTimeOffset;}
	double GetMediaTimeOffset () const {return m_dMediaTimeOffset;}
	bool IsUnitPlayable (double dTS);
	void SetMediaControl (MediaControlImp *pControl) {m_pControl = pControl;}
	MediaControlImp *GetMediaControl () const {return m_pControl;}

	void RegisterWithOD(ObjectDescriptorImp *od);
	void UnregisterWithOD(ObjectDescriptorImp *od);
	/*list of ODs depending on this clock (mediaControl support) - read only*/
	ZArray<ObjectDescriptorImp *> m_AttachedODs;
	/*returns inlineOD if any, otherwise NULL*/
	ObjectDescriptorImp *GetInlineOD() { return m_pInlineOD; }
};

/*-------------------------------------------------------------
A container for all clock references belonging to a single namespace
-------------------------------------------------------------*/
class ClockNamespace
{
public:
	void Resume ();
	void Pause ();
	~ClockNamespace ();
	ClockReference *AttachClock (int nClockID, int nHasOCR);
	ClockReference *GetClock (int nClockID);

private:
	ZArray<ClockReference*> m_clocks;
};

#endif                                      // _clock_h sentry.
