/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  BIFS Decoder
FILE:       Conditional.cpp
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Implementation of Conditional.
-------------------------------------------------------------------------*/

#include "defs.h"
//#include "Conditional.h" // MBS: no more needed
#include "Nodes.h" // MBS: Conditional is now declared in Nodes.h
#include "NodeCode.h"
#include "MemBifs.h"
#include "Executiv.h"
#include "Except.h"

/*-----------------------------------------------------------------
BIFS macros for Conditional
-----------------------------------------------------------------*/
// MBS: removed 'cause put in Nodes.cpp automatically
/*
BIFS_IMPLEMENT_NODE_START(Conditional)
	BIFS_EVENT(Conditional, activate, 0, 0, 0, 0)
	BIFS_EVENT(Conditional, reverseActivate, 1,  0,  0,  0)
	BIFS_FIELD(Conditional, buffer, 0, 1, 0, 1, 0, 0)
	BIFS_FIELD(Conditional, isActive, -1, -1, 1, -1, 0, 0)
BIFS_IMPLEMENT_NODE_END(Conditional)
*/
// end MBS

/*-----------------------------------------------------------------
Activate a conditional. Instantiate a version of BIFSDecoder which
gets its input from the buffer field
-----------------------------------------------------------------*/
void Conditional :: Onactivate ()
{
#ifdef BIFSDEC
	if ((Bool) activate )
	{
		#ifdef PLAYER
		GetExecutive () -> LockScene ();
		#endif

		isActive = TRUE;
		try
		{	MemBIFSDecoder (
				GetExecutive (),
				(const BYTE *) (LPCSTR) buffer,
				buffer.GetLength (),
				GetExecutive () -> GetTraceFile (),
				"Conditional",
				GetExecutive () -> GetTraceFlags (),
				0,
				GetRoot ()
			).ParseUpdateFrame ();
		}
		catch (BifsException)
		{
			#ifdef PLAYER
			GetExecutive () -> Error (SCENE_PARSING_ERROR);
			#endif
		}

		//JLF: found some (very rare) content using the isActive=TRUE from conditional. Currently this needs 
		//clarification in MPEG since isActive = TRUE then FALSE in a single route activation step, thus the isActive=TRUE is 
		//never seen...
		//VRML, 4.10.3 Execution model:
		//	"If a field is connected to another field via a ROUTE, an implementation shall 
		//	send only one event per ROUTE per timestamp"
		//
		//also note on eventOuts and scripts (VRML 4.12.9.3)
		//"Sending multiple values through an eventOut during a single script execution will result in the "last" 
		//event being sent, where "last" is determined by the semantics of the scripting language being used"
		//-> is conditional equivalent to "BIFS Scripting" ?  :)

//		isActive = FALSE;

		#ifdef PLAYER
		GetExecutive () -> UnlockScene ();
		GetExecutive () -> RequestChannels ();
		#endif
	}
#endif
}

void Conditional :: OnreverseActivate ()
{
#ifdef BIFSDEC
	if (! (Bool) reverseActivate)
	{
		#ifdef PLAYER
		GetExecutive () -> LockScene ();
		#endif

		isActive = TRUE;
		try
		{
			MemBIFSDecoder (
				GetExecutive (),
				(const BYTE *) (LPCSTR) buffer,
				buffer.GetLength (),
				GetExecutive () -> GetTraceFile (),
				"Conditional",
				GetExecutive () -> GetTraceFlags (),
				0,
				GetRoot ()
			).ParseUpdateFrame ();
		}
		catch (BifsException)
		{
			#ifdef PLAYER
			GetExecutive () -> Error (SCENE_PARSING_ERROR);
			#endif
		}
		isActive = FALSE;

		#ifdef PLAYER
		GetExecutive () -> UnlockScene ();
		GetExecutive () -> RequestChannels ();
		#endif
	}
#endif
}
	