/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1997.
************************************************************************

For more information or to receive an up to date version fo this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

#if !defined(_CrNodes_h)	// Sentry, use file only if it's not already included.
#define _CrNodes_h

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  BIFS Parser
FILE:       CrNodes.h
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Template function for creating nodes
-------------------------------------------------------------------------*/

#include "Except.h"

template <class T> MediaObject * SFNode<T> :: CreateNode (int nodeType)
{
	if (nodeType != 1)		// Always the case in FDIS
		throw BifsException (BIFX_PARSING_ERROR);
	return SFGenericNode :: CreateNode (T :: GetCode ());
}

template <class T> bool SFNode<T> :: IsTypeOK (MediaObject *pNode) const
{
	return pNode -> GetNodeCode () == T :: nodeCode;
}

/*-------------------------------------------------------------
Given a BIFS table ("version") number, determine the 'localNodeType' field width for that
version, if this node is the only node allowed in this place. The result is 1 if the T node
is of this version and 0 otherwise.
-------------------------------------------------------------*/
template <class T> int SFNode<T> :: GetNodeVersionSize (int nVersion) const
{
	return 
	(	T :: nodeCode >= m_nodeVersions [nVersion -1] &&
		(nVersion == BIFS_VERSIONS || T :: nodeCode < m_nodeVersions [nVersion])
	);
}

#ifdef BIFSENC

/*-------------------------------------------------------------
Create a node from name if node T is expected. If the name is not of any node type then
create T by default
-------------------------------------------------------------*/
template <class T> MediaObject *SFNode<T> :: CreateNode (
	LPCSTR szNodeName,
	int &nNodeType
)
{
	nNodeType = 1;
	MediaObject *pNode = new T;
	if (stricmp (pNode -> GetName (), szNodeName) != 0)
	{
		MediaObject *p = SFGenericNode :: CreateNode (szNodeName, nNodeType);
		if (p != NULL)
		{	nNodeType = -1;
			delete pNode;
			return p;
		}
		else
			nNodeType = -2;
	}
	return pNode;
}

/*-------------------------------------------------------------
Create a node from name if nodeType 'type' is expected. If the name if not of any
node type then create the first node of type 'type' from the list by default
-------------------------------------------------------------*/
template <NodeType type> MediaObject *SFNodeType<type> ::
CreateNode (LPCSTR szNodeName, int &rNodeType)
{
	MediaObject *pNode;

	for (int nCode = 0; nCode < GetNodeTableSize (); nCode ++)
	{	pNode = CreateNode (nCode + 1);
		if (pNode && !stricmp (pNode -> GetName (), szNodeName))
		{	rNodeType = nCode + 1;		// +1 added for FDIS
			return pNode;
		}
		delete pNode;
	}
	pNode = SFGenericNode:: CreateNode (szNodeName, rNodeType);
	if (pNode != NULL)
		rNodeType = -1;
	else
	{	pNode = CreateNode (1);
		rNodeType = -2;
	}
	return pNode;
}

#endif		// BIFSENC

#endif                                      // _CrNodes_h sentry.
