/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

#if !defined(_executiv_h)	// Sentry, use file only if it's not already included.
#define _executiv_h

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Executive
FILE:       Executiv.h
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Definition of Executive
-------------------------------------------------------------------------*/
#include "DAI.h"
#include "Im1defs.h"
#include "SysCoder.h"

#ifdef _DEBUG
#include "custom.h"
#else _DEBUG
#ifndef _SAIT_3DTG_MP4VIDEO_
#include "custom.h"
#else _SAIT_3DTG_MP4VIDEO_
#include "../player3d/custom.h"
#endif _SAIT_3DTG_MP4VIDEO_
#endif _DEBUG

#ifdef CreateService
#undef CreateService
#endif

class PresenterBase;
class ALManager;
class Application;
class MediaObject;
class ObjectDescriptorImp;
class InlineImp;
class ES_Descriptor;
class MFGenericNode;
class Root;
class StreamConsumer;
class DataChannel;
class Route;
class DeviceFilter;		//CC 010123
class IPMPSystem;

class Executive : Application_Sink
{
protected:
	DeviceFilter *m_pDeviceFilter;			// CC 010123
	DMIF_Filter *m_pDMIF;
	IPMPSystem *m_pIPMP;
	HINSTANCE m_hDMIF, m_hIPMP;
	PresenterBase *m_pPresenter;
	Application *m_pApp;
	InlineImp *m_pInline;
	ObjectDescriptorImp *m_pOD;
	Root *m_pRoot;
	HWND m_hWnd;
	ZSemaphore m_smpActive;
	ZMutex m_mtxScene, m_mtxServices, m_mtxChannels;
	int m_nReserved;
	bool m_bActive, m_bSceneInited, m_bPendingChannels;
	ZArray<ALManager*> m_services;
	ZMap<DataChannel*> m_channels;
	FILE *m_fTraceFile;
	int m_flags, m_nSceneLockedBy, m_nLocks, m_nStartLock, m_nRoutesLevel;

private:
	ZArray<MediaObject *> removedNodes;
	ZArray<Route *> removedRoutes;
	struct RouteList
	{	Route *m_pRoute;
		Root *m_pNameSpace;
		RouteList (Route *pRoute = NULL, Root *pNameSpace = NULL):
			m_pRoute (pRoute), m_pNameSpace (pNameSpace)
		{}
	};
	ZArray<RouteList> m_routes;
	int m_nActiveRoutes;

	ZArray<InlineImp *> restartInline;

private:
	// MPGJ - changed the below to virtual
	virtual ALManager *FindService (UINT nServiceId); // PA - made it virtual sothat we can access it from MPEGJDecoder
	DataChannel *FindChannel (UINT nChid);

	/*-------------------------------------------------------------
	Callback functions for Application_Sink
	-------------------------------------------------------------*/
	// Service Attach functions
	virtual void DAI_ServiceAttach_Ind(
		const char*		/* service_name */,
		const UINT		/* nServiceId */,
		const BYTE*		/* uuData */,
		const u_short	/* uuData_Length */
	) {}
	virtual void DAI_ServiceAttach_Cnf(
		const UINT		nTmpId,
		const UINT		nServiceId,
		const BYTE*		uuData,
		const u_short	uuData_Length,
		const RESPONSE	eResponse
	);

	// Service Detach functions
	virtual void DAI_ServiceDetach_Ind(
		const UINT		nServiceId,
		const REASON	eReason
	);
	virtual void DAI_ServiceDetach_Cnf(
		const UINT		nServiceId,
		const RESPONSE	eResponse
	);

	// Channel Add functions
	virtual void DAI_ChannelAdd_Ind(
		const UINT		/* nServiceId */,
		const DAIChannelAddParam& /* inParam */
	) {}
	virtual void DAI_ChannelAdd_Cnf(
		const UINT		nServiceId,
		const DAIChannelAddParam& outParam
	);

	// Channel Delete functions
	virtual void DAI_ChannelDelete_Ind(
		const UINT		nServiceId,
		const DAIChannelDeleteParam& inParam
	);
	virtual void DAI_ChannelDelete_Cnf(
		const UINT		nServiceId,
		const DAIChannelDeleteParam& outParam
	);

	// User Command function
	virtual void DAI_UserCommand_Ind(
		const UINT		/* nServiceId */,
		const DAIUserCmdParam& /* inParam */
	) {}
	virtual	void DAI_UserCommandAck_Ind(
		const UINT		/* nServiceId */,
		const DAIUserCmdParam& /* inParam */
	);
	virtual	void DAI_UserCommandAck_Cnf(
		const UINT		nServiceId,
		const DAIUserCmdParam& outParam
	);

	// Function for data reception
	virtual void DAI_DataReceive_Ind(
		const UINT		nChid,
		const BYTE*		pStreamData,
		const int		nStreamLength,
		const BYTE*		pAppData,
		const int		nAppLength,
		const int		nErrorFlag
	);

public:

// MPEGJ MERGE_2 - RLS
	InlineImp* GetInline() {
		ASSERT(m_pInline);
		return m_pInline; }
	// added to get inline to access its variables while throwing events- PA

	ALManager* alm_mpegj;
	ALManager* GetALManager() {
		if ( alm_mpegj == NULL )
			return FindService ( 0 );  // need to change this
		else
			return alm_mpegj;
	}
// MPEGJ MERGE_2 - RLS
	void Resume ();
	void Pause ();
	void RemoveSceneObjects ();
	void RemoveRoute (Route *pRoute);
	void RemoveNode (MediaObject *pNode);
	void DeleteChannel (DataChannel *pChannel);
	void DeleteService (ALManager *pService);
	void AddChannel (DataChannel *pChannel);
	void SendCommand (UINT nServiceID, const DAIUserCmdParam &inParam);
	void ServiceDetach (ALManager *pService, REASON reason);
	void ServiceAttach (ALManager *pService, LPCSTR szURL, UINT nParentServiceID);
	void ChannelAdd (UINT nServiceID, const DAIChannelAddParam &inParam);
	void ChannelDelete (UINT nServiceID, const DAIChannelDeleteParam &inParam);
	FILE *GetTraceFile () const {return m_fTraceFile;}
	int GetTraceFlags () const {return m_flags;}
	void SetClocksGranularity (int nMS);
	void SetPendingChannels () {m_bPendingChannels = TRUE;}
	void RequestChannels ();
	void CreateService (LPCSTR szURL, ObjectDescriptorImp *pNode, UINT nParentServiceID = 0);
	void ReplaceScene ();
	Root *GetScene () {return m_pRoot;}
	void Error (NON_FATAL_ERROR nError, ...);
	void DataSend (UINT nServiceID, UINT nChannelID, LPBYTE pData, int nLength, int nAUnumber);
	Executive (
		Application *pApp,
		HWND hWnd,
		FILE *fTraceFile = NULL,
		int flags = TRACE_NONE,
		DeviceFilter *pDeviceFilter = NULL
	);
	~Executive ();
	void Start (LPCSTR szAddress);
	void Stop ();
	void Paint (void *info);
	PresenterBase *GetPresenter () {return m_pPresenter;}
	//*** YT START (101)

	#ifndef MPGJ
	void LockScene (int nReserve = 0);
	void UnlockScene (int nReserve = 0);
	#else

	// make these methods accessible from mpegj.DLL
	virtual void LockScene (int nReserve = 0);
	virtual void UnlockScene (int nReserve = 0);
	#endif MPGJ
	//*** YT END (101)
	int ReserveScene (int nReserve = 0);
	void OnUserEvent  (USER_EVENT event, int x, int y, UINT uControlKeysStatus);
//BINDU --- MPEGJ MERGE_3 - RLS
	#ifdef SIA
	Application *getAppl();
	#endif
//BINDU --- MPEGJ MERGE_3 - RLS
	Route * GetRoute (int n) const {return m_routes[n].m_pRoute;}
	void UnregisterRoute (Route *pRoute);
	void RegisterRoute (Route *pRoute, Root *pNameSpace)
	{
		m_routes += RouteList (pRoute, pNameSpace);
	}
	void CloneRoutes (Root *pProto, Root *pCloned);
	Route *FindRoute  (int nID, Root *pNameSpace);
	int GetRouteListLength () const {return m_routes.GetLength ();}
	Route *GetMyRoute (int n, Root *pNameSpace) const
	{
		return m_routes [n].m_pNameSpace == pNameSpace ? m_routes[n].m_pRoute : NULL;
	}
	int MarkRoute (int n)
	{	m_nActiveRoutes += n;
		return m_nRoutesLevel + 1;
	}
	void ActivateRoutes ();
	IPMPSystem *GetIPMPSystem () const {return m_pIPMP;}

	DeviceFilter *GetDeviceFilter() { return m_pDeviceFilter; }

	//Get base application class
	Application *GetApplication() { return m_pApp; }
	//Notify something changed. It can be passed a pointer to the node which
	//changed or NULL for any other event
	void InvalidateScene(MediaObject *pObject);
	/*restarting an inline scene may destroy the graph owning the node asking for restart
	thus restart of inline is handled separately (cf StreamConsumer Stop and Start). This
	adds an inline scene to the list of scene to restart*/
	void AddInlineToRestart(InlineImp *pInline) { restartInline += pInline; }

	//this is called in PresBase, and restart inlines when needed, or by your presenter once a simulation
	//cycle is done
	void RestartInlines();

	CUSTOMIZE_EXECUTIVE_DECLARATION
};

#endif                                      // _executiv_h sentry.
