/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Optibase Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Optibase Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1997.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Presenter (simulation)
FILE:       FlxTime.cpp
AUTHOR:     Zvi Lifshitz (Optibase Ltd.)
OVERVIEW:   Implementation of FlexTime
-------------------------------------------------------------------------*/

#include "defs.h"
#include "FlxTime.h"
#include "Executiv.h"
#include "PresBase.h"
#include "Stream.h"
#include "Clock.h"

/*-------------------------------------------------------------
The constructor of the TemporalTransform implementation saves the instantiation time of the
node and other values required for calculation of time transformation
-------------------------------------------------------------*/
TemporalTransformImp :: TemporalTransformImp ()
{
	m_dwLastSwitched = GetClock ();
	m_dwTimeOffset = 0;
	m_fSpeed = 1;
	m_bUrlActive = true;
}

/*-------------------------------------------------------------
Calculate first transformation after parse
-------------------------------------------------------------*/
BOOL TemporalTransformImp :: Parse (BIFSDecoder *pBifs)
{
	BOOL bResult = StreamConsumer :: Parse (pBifs);
	Transform (NULL);
	return bResult;
}

/*-------------------------------------------------------------
The destructor of TemporalTransform gets the url out of its control
-------------------------------------------------------------*/
TemporalTransformImp :: ~TemporalTransformImp ()
{
	if (m_bUrlActive && GetStream ())
		GetStream () ->ModifyClock (m_fSpeed == 0 ? -1 : 1. / m_fSpeed);
}

/*-------------------------------------------------------------
Return true if the url controlled by this node is active
-------------------------------------------------------------*/
bool TemporalTransformImp :: IsUrlActive ()
{
	return GetMediaPosition (startTime * 1000, optimalDuration * 1000) != (DWORD) -1;
}

/*-------------------------------------------------------------
Return the optimal duration of the object
(need to fix this so if optimalDuration < 0 the duration will be retrieved from the url
-------------------------------------------------------------*/
double TemporalTransformImp :: GetDuration () const
{
	return optimalDuration;
}

/*-------------------------------------------------------------
Apply temporal transformation on the timebase of a TemporalTransform node and on its
URL (or segment).
Temporal transformation is enabled through the use of a context variable, with passes between
Render function during scene traversal
-------------------------------------------------------------*/
void TemporalTransformImp :: Transform (PresenterContext *pContext)
{
	bool bUrlActive = IsUrlActive ();
	if (bUrlActive != m_bUrlActive)
	{	if (bUrlActive)
			GetStream () ->ModifyClock (m_fSpeed);
		else if (GetStream ())
			GetStream () ->ModifyClock (m_fSpeed == 0 ? -1 : 1. / m_fSpeed);
		m_bUrlActive = bUrlActive;
	}
	 
	/*-------------------------------------------------------------
	If temporal parameters of the node have been changed, update values for calculating 
	the new transformation
	-------------------------------------------------------------*/
	float fSpeed = (Bool) active ? (float) speed : 0;
	if (fSpeed != m_fSpeed)
	{	DWORD dwNow = GetClock ();
		m_dwTimeOffset += (dwNow - m_dwLastSwitched) * (1 - m_fSpeed);
		m_dwLastSwitched = dwNow;
		if (bUrlActive)
		{	GetStream () ->ModifyClock (m_fSpeed == 0 ? -1 : 1. / m_fSpeed);
			GetStream () ->ModifyClock (fSpeed);
		}
		m_fSpeed = fSpeed;
		if ((Bool) active && GetDuration () >= 0)
			actualDuration = GetDuration () * m_fSpeed;
	}

	/*-------------------------------------------------------------
	Apply temporal transformation on the context variable
	-------------------------------------------------------------*/
	if (pContext)
		pContext -> Transform (GetClock (), m_dwTimeOffset, m_fSpeed, (Bool) active);
}

/*-------------------------------------------------------------
Apply termporal transformation on a PresenterContext object
-------------------------------------------------------------*/
void PresenterContext :: Transform (DWORD dwTime, DWORD dwOffset, float fSpeed, bool bActive)
{
	m_dwOffset += dwOffset;
	m_fSpeed *= fSpeed;
	m_bActive &= bActive;
	m_dwTime = (dwTime - m_dwOffset) * m_fSpeed;
}