/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Optibase Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Optibase Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2002.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  MUW
FILE:       GenField.cpp
AUTHOR:     Zvi Lifshitz (Optibase Ltd.)
OVERVIEW:   Encoding of a NodeField object
-------------------------------------------------------------------------*/
#include "defs.h"
#include "NodeFld.h"
//#include "CondEnc.h"
#include "Conditional.h"
#include "NodeCode.h"
#include "Root.h"
#include "Proto.h"
#include "BifsGen.h"

/*-----------------------------------------------------------------
Encode an SFInt while generating the carousel
-----------------------------------------------------------------*/
void SF<int> :: GenerateField (BifsGenerator *pBifs)
{
	pBifs -> EncodeInt (m_value, 32, "value");
}

/*-----------------------------------------------------------------
Encode an SFBool while generating the carousel
-----------------------------------------------------------------*/
void SF<Bool> :: GenerateField (BifsGenerator *pBifs)
{
	pBifs -> EncodeInt (m_value, 1, "value");
}

/*-----------------------------------------------------------------
Encode and SFFloat while generating the carousel
-----------------------------------------------------------------*/
void SF<float> :: GenerateField (BifsGenerator *pBifs)
{
    pBifs -> EncodeFloat (m_value, "value");
}

/*-----------------------------------------------------------------
Encode an SFTime while generating the carousel
-----------------------------------------------------------------*/
void SF<Time> :: GenerateField (BifsGenerator *pBifs)
{
	pBifs -> EncodeDouble (m_value, "value");
}

/*-----------------------------------------------------------------
Encode an SFVec3f while generating the carousel
-----------------------------------------------------------------*/
void SF<Vec3f> :: GenerateField (BifsGenerator *pBifs)
{
	pBifs -> EncodeFloat (m_value.x, "x");
	pBifs -> EncodeFloat (m_value.y, "y");
	pBifs -> EncodeFloat (m_value.z, "z");
}

/*-----------------------------------------------------------------
Encode an SFVec2f while generating the carousel
-----------------------------------------------------------------*/
void SF<Vec2f> :: GenerateField (BifsGenerator *pBifs)
{
	pBifs -> EncodeFloat (m_value.x, "x");
	pBifs -> EncodeFloat (m_value.y, "y");
}

/*-----------------------------------------------------------------
Encode an SFRotation while generating the carousel
-----------------------------------------------------------------*/
void SF<Rotation> :: GenerateField (BifsGenerator *pBifs)
{
	pBifs -> EncodeFloat (m_value.x, "x");
	pBifs -> EncodeFloat (m_value.y, "y");
	pBifs -> EncodeFloat (m_value.z, "z");
	pBifs -> EncodeFloat (m_value.theta, "theta");
}

/*-----------------------------------------------------------------
Encode an SFRotation2D while generating the carousel
-----------------------------------------------------------------*/
void SF<Rotation2D> :: GenerateField (BifsGenerator *pBifs)
{
	pBifs -> EncodeFloat (m_value.x, "x");
	pBifs -> EncodeFloat (m_value.y, "y");
	pBifs -> EncodeFloat (m_value.theta, "theta");
}

/*-----------------------------------------------------------------
Encode an SFColor while generating the carousel
-----------------------------------------------------------------*/
void SF<Color3f> :: GenerateField (BifsGenerator *pBifs)
{
	pBifs -> EncodeFloat (m_value.r, "r");
	pBifs -> EncodeFloat (m_value.g, "g");
	pBifs -> EncodeFloat (m_value.b, "b");
}

/*-----------------------------------------------------------------
Need to define empty SF<ZString> :: Generate function while generating the carousel
-----------------------------------------------------------------*/
void SF<ZString> :: GenerateField (BifsGenerator *pBifs)
{
	pBifs ->EncodeSFString (m_value);
}

/*-----------------------------------------------------------------
Encode SFUrl while generating the carousel
-----------------------------------------------------------------*/
void SF<Url> :: GenerateField (BifsGenerator *pBifs)
{
	bool isOD = m_value.ODid >= 0;
	pBifs -> EncodeInt (isOD, 1, "isOD");
	if (isOD)
		pBifs -> EncodeInt (m_value.ODid, 10, "ODid");
	else
		m_value.urlValue.GenerateField (pBifs);
}
		
/*-----------------------------------------------------------------
Encode SFImage while generating the carousel
-----------------------------------------------------------------*/
void SF<Image> :: GenerateField (BifsGenerator *pBifs)
{
	//Alvar 981223: Height and width are 12 bit encoded (10->12)
	pBifs -> EncodeInt (m_value.width, 12, "width");
	pBifs -> EncodeInt (m_value.height,12, "height");
	pBifs -> EncodeInt (m_value.numComponents, 2, "numComponents");

	for (int i = 0; i < m_value.width * m_value.height * m_value.numComponents; i++)
		pBifs -> EncodeInt (m_value.pixels [i], 8, "pixel");
}

/*-----------------------------------------------------------------
Encode an SFNode field while generating the carrousel
-----------------------------------------------------------------*/
void SFGenericNode :: GenerateField (BifsGenerator *pBifs)
{
	if (m_pNode == NULL)
		return;

	/*-----------------------------------------------------------------
	Check if the node has been already encoded in this iteration. If so, this is a used
	node.
	-----------------------------------------------------------------*/
	bool bIsReused = m_pNode -> GetEncodeCounter () == pBifs -> GetEncodeCounter ();
	pBifs -> EncodeInt (bIsReused, 1, "isReused");
	if (bIsReused)
	{	pBifs -> EncodeInt (m_pNode ->GetID (), pBifs -> GetNodeIDBits (), "nodeID");
		return;
	}

	/*-----------------------------------------------------------------
	Encode the rest of the field and the node
	-----------------------------------------------------------------*/
	int nProtoID = m_pNode -> GetProtoID ();
	int nNodeType = INT_MAX;
	char szTag [] = "localNodeType\0\0";
	if (nProtoID < 0)
		nNodeType = GetNodeType (m_pNode ->GetNodeCode ());

	for (int nVer = 1; nVer <= NodeField :: BIFS_VERSIONS; nVer++)
	{	int nVersionTableSize = GetNodeVersionSize (nVer);
		int nFieldWidth = BitFieldWidth (nVersionTableSize + 1 + (nVer == 2));
		if (nNodeType <= nVersionTableSize)
		{	pBifs ->EncodeInt (nNodeType + (nVer == 2), nFieldWidth, szTag);
			break;
		}
		else if (nProtoID >= 0 && nVer == 2)
		{	pBifs ->EncodeInt (1, nFieldWidth, "nodeType");
			pBifs ->EncodeInt (nProtoID, pBifs -> GetProtoIDBits (), "PROTOnodeType");
			break;
		}
		else
		{	pBifs ->EncodeInt (0, nFieldWidth, "localNodeTypeEscape");
			nNodeType -= nVersionTableSize;
			szTag [13] = 'V';
			szTag [14] = '1' + nVer;
		}
	}
	
	bool bIsUpdateable = m_pNode -> GetID () >= 0;
	pBifs -> EncodeInt (bIsUpdateable, 1, "isUpdateable");
	if (bIsUpdateable)
	{	pBifs -> EncodeInt (m_pNode -> GetID (), pBifs -> GetNodeIDBits (), "nodeID");
		if (pBifs ->GetFlags () & CODENAMES)
			pBifs -> EncodeString (m_pNode -> GetNameID ());
	}
	
	m_pNode -> GenerateNode (pBifs);
}

/*-----------------------------------------------------------------
Encode an MF field when generating the carrousel
-----------------------------------------------------------------*/
void MField :: GenerateField (BifsGenerator *pBifs)
{
	pBifs -> EncodeInt (0, 1, "reserved");		// added for FDIS
	pBifs -> EncodeInt (1, 1, "isListDescription");
	for (int i = 0; i < GetLength (); i++)
	{	pBifs -> EncodeInt (0, 1, "endFlag");	// changed in the FDIS
		GenerateElement (i, pBifs);
	}
	pBifs -> EncodeInt (1, 1, "endFlag");
}

/*-------------------------------------------------------------
Encode a PROTO interface field while generating the carrousel
-------------------------------------------------------------*/
void NodeField :: GenerateProtoFieldDefinition (BifsGenerator *pBifs, int eventType, LPCSTR szName)
{
	int nType = GetType ();
	pBifs -> EncodeInt (!(nType >> 5), 1, "isMF");
	pBifs -> EncodeInt (nType & 0x1f, 5, "fieldType");
	if (pBifs ->GetFlags () & CODENAMES)
		pBifs -> EncodeString (szName);

	/*-------------------------------------------------------------
	Eencode the field value (if this is a field)
	-------------------------------------------------------------*/
	if (eventType <= ET_EXPOSED_FIELD)
		GenerateField (pBifs);
}

/*-----------------------------------------------------------------
Encode an SFVec4f while generating the carousel
-----------------------------------------------------------------*/
void SF<Vec4f> :: GenerateField (BifsGenerator *pBifs)
{
	pBifs -> EncodeFloat (m_value.x, "x");
	pBifs -> EncodeFloat (m_value.y, "y");
	pBifs -> EncodeFloat (m_value.z, "z");
	pBifs -> EncodeFloat (m_value.q, "q");
}
