/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Optibase Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Optibase Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1997.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  BIFS Encoder
FILE:       GenNode.cpp
AUTHOR:     Zvi Lifshitz (Optibase Ltd.)
OVERVIEW:   Generating BIFS for a MediaObject object.
-------------------------------------------------------------------------*/
#include "defs.h"
#include "MediaObj.h"
#include "BifsGen.h"
#include "bifs.h"
#include "NodeFld.h"
#include "Proto.h"
#include "Route.h"


/*-------------------------------------------------------------
Encode a node when generating BIFS from a memory image
-------------------------------------------------------------*/
void MediaObject :: GenerateNode (BifsGenerator *pBifs)
{
	m_nEncodeCounter = pBifs -> GetEncodeCounter ();
	pBifs -> EncodeInt (0, 1, "maskAccess");

	const BifsFieldTable *pTable = GetFieldTable ();
	for (int i = 0; i < GetFieldCount (); i++)
	{	const BifsFieldTable *pEntry = pTable + i;
		NodeField *pField = GetField (pEntry);
		if (pField ->WasFieldUpdated ())
		{	pBifs -> EncodeInt (0, 1, "endFlag");
			Proto *pProto = pBifs -> GetProto ();
			bool isedField = false;
			if (pProto != NULL)
			{	int nFieldID = pField -> GetIsedFieldID ();
				isedField = nFieldID >= 0;
				pBifs -> EncodeInt (isedField, 1, "isedField");
				if (isedField)
				{	pBifs -> EncodeInt (i, GetBifsBounds () ->m_nBitsPerAll, "fieldRef");
					pBifs -> EncodeInt (
						nFieldID,
						pProto ->GetBifsBounds () ->m_nBitsPerAll,
						"protoField"
					);
				}
			}
			if (!isedField)
			{	pBifs -> EncodeInt (
					pEntry -> m_defID,
					GetBifsBounds () -> m_nBitsPerDef,
					"fieldRef"
				);
				pField -> GenerateField (pBifs);
			}
		}
	}
	pBifs -> EncodeInt (1, 1, "endFlag");
}

/*-------------------------------------------------------------
Used when encoding a Route for the carrousel. outID is the (out) index of the departure
field of the Route
-------------------------------------------------------------*/
void MediaObject :: GenerateRouteOutField (int outID, BifsGenerator *pBifs)
{
	pBifs -> EncodeInt (outID, GetBifsBounds () -> m_nBitsPerOut, "outFieldReference");
}

/*-------------------------------------------------------------
used when encoding a Route for the carrousel. index is the (absolute) index of the 
arrival field
-------------------------------------------------------------*/
void MediaObject :: GenerateRouteInField (int index, BifsGenerator *pBifs)
{
	pBifs -> EncodeInt (
		GetFieldTable () [index].m_inID,
		GetBifsBounds () -> m_nBitsPerIn,
		"inFieldRefernce"
	);
}

/*-------------------------------------------------------------
Encode an insert into an MF field command when the current node is the target node
nField is the index of the field within the node
pField contains the value to insert
nIndex is the point in the MF field to insert the new value. A value of -1 means append
at end
-------------------------------------------------------------*/
void MediaObject :: GenerateInsertionCommand (
	int nField,
	NodeField *pField,
	int nIndex,
	BifsGenerator *pBifs
)
{
	pBifs ->IncreaseCounter ();
	const BifsFieldTable *pEntry = GetFieldTable () + nField;
	pBifs -> EncodeInt (0, 2, "code", "InsertionCommand");
	if (pEntry -> m_bChildren)
	{	pBifs -> EncodeInt (0, 2, "parameterType", "NodeInsertion");
		pBifs -> EncodeInt (GetID (), pBifs ->GetNodeIDBits (), "nodeID");
	}
	else
	{	assert (pEntry -> m_inID >= 0);
		pBifs -> EncodeInt (2, 2, "parameterType", "IndexedValueInsertion");
		pBifs -> EncodeInt (GetID (), pBifs ->GetNodeIDBits (), "nodeID");
		pBifs -> EncodeInt (pEntry -> m_inID, GetBifsBounds () -> m_nBitsPerIn, "fieldID");
	}
	pBifs -> EncodeInt (
		nIndex > 0 ? 0 : nIndex == 0 ? 2 : 3,
		2,
		"position"
	);
	if (nIndex > 0)
		pBifs -> EncodeInt (
			nIndex,
			(2 - pEntry -> m_bChildren) * 8,
			"position"
		);
	pField ->GenerateField (pBifs);
}

/*-------------------------------------------------------------
Encode a node replacement command when generating BIFS from a memory image
pNode is the new node to replace the current node.
-------------------------------------------------------------*/
void MediaObject :: GenerateNodeReplacement (MediaObject *pNode, BifsGenerator *pBifs)
{
	pBifs ->IncreaseCounter ();
	pBifs -> EncodeInt (2, 2, "code", "ReplacementCommand");
	pBifs -> EncodeInt (0, 2, "parameterType", "NodeReplacement");;
	pBifs -> EncodeInt (GetID (), pBifs ->GetNodeIDBits (), "nodeID");
	SFWorldNode nodeField;
	nodeField.SetObject (pNode);
	nodeField.GenerateField (pBifs);
}

/*-------------------------------------------------------------
Encode a field replacement command when the current node is the target node
nField is the index of the field within the node
pField contains the value to insert
-------------------------------------------------------------*/
void MediaObject :: GenerateFieldReplacement (
	int nField,
	NodeField *pField,
	BifsGenerator *pBifs
)
{
	pBifs ->IncreaseCounter ();
	const BifsFieldTable *pEntry = GetFieldTable () + nField;
	pBifs -> EncodeInt (2, 2, "code", "ReplacementCommand");
	pBifs -> EncodeInt (1, 2, "parameterType", "FieldReplacement");
	pBifs -> EncodeInt (GetID (), pBifs ->GetNodeIDBits (), "nodeID");
	pBifs -> EncodeInt (pEntry -> m_inID, GetBifsBounds () -> m_nBitsPerIn, "fieldID");
	pField ->GenerateField (pBifs);
}

/*-------------------------------------------------------------
Encode an IndexedValueReplacement command when the current node is the target node
nField is the index of the field within the node
pField contains the value to insert
nIndex is the point in the MF field to insert the new value. A value of -1 means replace
the last element
-------------------------------------------------------------*/
void MediaObject :: GenerateIndexedValueReplacement (
	int nField,
	NodeField *pField,
	int nIndex,
	BifsGenerator *pBifs
)
{
	pBifs ->IncreaseCounter ();
	const BifsFieldTable *pEntry = GetFieldTable () + nField;
	pBifs -> EncodeInt (2, 2, "code", "ReplacementCommand");
	pBifs -> EncodeInt (2, 2, "parameterType", "IndexedValueReplacement");
	pBifs -> EncodeInt (GetID (), pBifs ->GetNodeIDBits (), "nodeID");
	pBifs -> EncodeInt (pEntry -> m_inID, GetBifsBounds () -> m_nBitsPerIn, "fieldID");
	pBifs -> EncodeInt (
		nIndex > 0 ? 0 : nIndex == 0 ? 2 : 3,
		2,
		"position"
	);
	if (nIndex > 0)
		pBifs -> EncodeInt (
			nIndex,
			(2 - pEntry -> m_bChildren) * 8,
			"position"
		);
	pField ->GenerateField (pBifs);
}

/*-------------------------------------------------------------
Encode an IndexedValueDeletion command when the current node is the target node
nField is the index of the field within the node
nIndex is the element in the MF field to delete. A value of -1 means delete
the last element
-------------------------------------------------------------*/
void MediaObject :: GenerateIndexedValueDeletion (
	int nField,
	int nIndex,
	BifsGenerator *pBifs
)
{
	const BifsFieldTable *pEntry = GetFieldTable () + nField;
	pBifs -> EncodeInt (1, 2, "code", "DeletionCommand");
	pBifs -> EncodeInt (2, 2, "parameterType", "IndexedValueDeletion");
	pBifs -> EncodeInt (GetID (), pBifs ->GetNodeIDBits (), "nodeID");
	pBifs -> EncodeInt (pEntry -> m_inID, GetBifsBounds () -> m_nBitsPerIn, "fieldID");
	pBifs -> EncodeInt (
		nIndex > 0 ? 0 : nIndex == 0 ? 2 : 3,
		2,
		"position"
	);
	if (nIndex > 0)
		pBifs -> EncodeInt (
			nIndex,
			(2 - pEntry -> m_bChildren) * 8,
			"position"
		);
}
