/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Optibase Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Optibase Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  MUW
FILE:       GenProto.cpp
AUTHOR:     Zvi Lifshitz (Optibase Ltd.)
OVERVIEW:   Encoding of a Proto object
-------------------------------------------------------------------------*/
#include "defs.h"
#include "Proto.h"
#include "BifsGen.h"

/*-------------------------------------------------------------
Encode a Proto while generating BIFS from a memory image
-------------------------------------------------------------*/
void Proto :: GenerateProtoDefinition (BifsGenerator *pCoder)
{
	int i;

	pCoder -> EncodeInt (GetID (), pCoder -> GetProtoIDBits (), "id");
	if (pCoder ->GetFlags () & CODENAMES)
		pCoder ->EncodeString (m_protoName);
	
	/*-------------------------------------------------------------
	Encode the interface fields
	-------------------------------------------------------------*/
	for (i = 0; i < m_fields.GetLength (); i++)
	{	pCoder -> EncodeInt (1, 1, "moreFields");
		int eventType;
		BifsFieldTable &rEntry = m_fieldTable [i];
		if (rEntry.m_defID >= 0)
		{	eventType = ET_FIELD;
			if (rEntry.m_inID >= 0)
				eventType = ET_EXPOSED_FIELD;
		}
		else if (rEntry.m_inID >= 0)
			eventType = ET_EVENT_IN;
		else
			eventType = ET_EVENT_OUT;

		pCoder -> EncodeInt (eventType, 2, "eventType");
		m_fields [i] -> GenerateProtoFieldDefinition (pCoder, eventType, rEntry.m_szName);
	}
	pCoder -> EncodeInt (0, 1, "moreFields");
	pCoder -> EncodeInt (m_nExternODID > 0, 1, "isExtern");
	if (m_nExternODID > 0)
		m_externUrl.GenerateField (pCoder);
	else
		GenerateProtoCode (pCoder);
	pCoder -> EncodeInt (0, 1, "useQuant");
	pCoder -> EncodeInt (0, 1, "useAnim");
}

/*-------------------------------------------------------------
Encode the nodes of the Proto when generating BIFS from a memory image
-------------------------------------------------------------*/
void Proto :: GenerateProtoCode (BifsGenerator *pCoder)
{
	/*-------------------------------------------------------------
	Encode nested PROTOs if any
	-------------------------------------------------------------*/
	GenerateProtos (pCoder);

	/*-------------------------------------------------------------
	Encode the PROTO nodes
	-------------------------------------------------------------*/
	pCoder ->SetProto (this);
	for (int i = 0; i < m_protoNodes.GetLength ();)
	{	m_protoNodes.GenerateElement (i, pCoder);
		bool moreNodes = ++i < m_protoNodes.GetLength ();
		pCoder -> EncodeInt (moreNodes, 1, "moreNodes");
	}
	pCoder ->SetProto (NULL);

	/*-------------------------------------------------------------
	Encode the PROTO Routes
	-------------------------------------------------------------*/
	GenerateRoutesDescription (pCoder);
}
