/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  BIFS Decoder/ Compositor / Presenter
FILE:       MediaObject.cpp
AUTHOR:     Stefano Batista (Edited by Zvi Lifshitz)
OVERVIEW:   Implementation of grouping properties of MediaObjects

Comment: now every MediaNode can be a grouping object
-------------------------------------------------------------------------*/

#include "defs.h"
#include "MediaObj.h"
#include "NodeFld.h"

/*-------------------------------------------------------------
ZL: Calculate the address of the 'children' field
-------------------------------------------------------------*/
MFGenericNode *MediaObject :: GetChildrenField () const
{
	if (GetChildrenOffset () < 0)
		return NULL;
	return (MFGenericNode *) (NodeField *) (((char *) this) + GetChildrenOffset ());
}

int MediaObject :: GetChildCount () const
{
	MFGenericNode *pChildren = GetChildrenField ();
	assert (pChildren);
	return pChildren -> GetLength();
}

// SB 99/03: added extra check on children length;
// fixes exception when trying to get element 0 from empty array
MediaObject * MediaObject :: GetFirstChild ()
{
	MFGenericNode *pChildren = GetChildrenField ();
	assert (pChildren);
	m_currentElement = 0;
	if ((*pChildren).GetLength() == 0)
		return NULL;
	else
		return (*pChildren) [m_currentElement++];
}

MediaObject * MediaObject :: GetNextSibling ()
{
	MFGenericNode *pChildren = GetChildrenField ();
	assert (pChildren);
	if ((*pChildren).GetLength() <= m_currentElement)
		return NULL;
	else
		return (*pChildren)[m_currentElement++];
}

/// Append a MediaObject to the list of Children
bool MediaObject :: AddChild (MediaObject * pObject)
{
	// put the child in the list of our children
	MFGenericNode *pChildren = GetChildrenField ();
	assert (pChildren);
	pChildren -> Append (pObject, this, true);
	{
		// here, we need to recompute the bounding box
		UpdateBBox();
		return true;
	}
}

/// Remove a MediaObject from the list of Children
bool MediaObject :: DeleteChild (MediaObject * pObject)
{
	// remove the child from the list of our children
	MFGenericNode *pChildren = GetChildrenField ();
	assert (pChildren);
	pObject -> RemoveContainer (pChildren);
	{
		// here, we need to recompute the bounding box
		UpdateBBox();
		return true;
	}
}
