/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1997.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

#if !defined(_Im1defs_h)	// Sentry, use file only if it's not already included.
#define _Im1defs_h

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  all
FILE:       Im1defs.h
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Constants and typedefs for Implementation 1
-------------------------------------------------------------------------*/
// SOLARIS PORTING - include defs.h
#include "defs.h"
#include "odtags.h"

typedef enum tagDefaultMPEG4Values
{
	MPEG4_DEFAULT_BIFS_BUFFER_SIZE = 500,
	MPEG4_DEFAULT_IPMP_BUFFER_SIZE = 500,
	MPEG4_OPTIMAL_IPMP_UNIT_SIZE = 128,
	MPEG4_TOTAL_NUMBER_OF_NODES = 180,
	MPEG4_NODE_TABLE_LENGTH = MPEG4_TOTAL_NUMBER_OF_NODES + 20,
	// by Liao Xuan, new descriptor classes and related tags are added
	//MPEG4_DESC_TABLE_LENGTH = 135
	//MPEG4_DESC_TABLE_LENGTH = 139
	// by Liao Xuan

	// mar 20061121: Replaced the "magic" number with
	// a more elegant approach (and easier to mantain)
	MPEG4_DESC_TABLE_LENGTH = CODE_NumberDescriptors
};

typedef struct tagES_Descriptor ES_DESCRIPTOR;
typedef struct tagESConfigParams ESCONFIGPARAMS;

typedef enum tagConnectionResult
{	
	CONNECTION_SUCCESS, FILE_NOT_FOUND, CANNOT_OPEN_FILE,
	IP_ADDRESS_NOT_FOUND, CANNOT_OPEN_SOCKET, MEMORY_PROBLEM,
	UNEXPECTED_EOF, SCENE_ACTIVE
}
CONNECTION_RESULT;

typedef enum tagServiceTermination
{	
	END_OF_SERVICE, HARDWARE_ERROR, COMMUNICATION_DISCONNECTED,
	REMOTE_TIME_OUT, BUFFER_ERROR
}
SERVICE_TERMINATION;

typedef enum tagNonFatalError
{
	IM1_OK, INVALID_STREAM_TYPE, STREAM_TYPE_COLLISION, NO_BIFS_STREAM,
	SCENE_PARSING_ERROR, OD_PARSING_ERROR, OD_UPDATE_NOT_SUPPORTED,
	ODID_COLLISION, UNKNOWN_ODID, UPDATE_ESD_NOT_SUPPORTED,
	REMOVE_ESD_NOT_SUPPORTED, UNKNOWN_DECODER_TYPE,
	INCOMPATIBLE_DECODER_TYPE, NO_STREAM, NO_STREAM_TYPE,
	REMOTE_OD_REQUIRES_URL, UNKNOWN_URL_PROTOCOL, BUFFER_TOO_SMALL,
	DECODER_SETUP_FAILURE, REMOTE_OD_REQUIRES_NEW_SESSION,
	REMOTE_OD_FLAG_REQUIRED, CHANNEL_ADD_FAILED, CHANNEL_DELETE_FAILED,
	DATA_TRANSMISSION_ERROR, INVALID_URL, IPMP_NOT_AVAILABLE,
	NO_ACCESS_TO_PROTECTED_STREAM, MISSING_DESCRIPTOR,
	TOO_MANY_DESCRIPTORS, INITIAL_OBJECT_DESCRIPTOR_NOT_FOUND,
	NO_SLCONFIG_INFO, ERROR_IN_PARSING_INITIAL_OBJECT_DESCRIPTOR,
	BIFS_UPDATE_REQUEST_DOES_NOT_MATCH_SCENE,
	MULTIPLE_ACCESS_OF_A_SINGLE_STREAM_NOT_PERMITTED,
	IPMP_STREAM_ALREADY_IN_USE, INVALID_BIFS_CONFIG,
	CANNOT_ALTER_REMOTE_OD, TRYING_TO_UPDATE_UNKNOWN_OD,
	TRYING_TO_REMOVE_UNKNOWN_ESD, OCI_PARSING_ERROR, SERVICE_DETACH_FAILED,
	SEND_COMMAND_FAILED, UNEXPECTED_SERVICE_DETACH, UNEXPECTED_CHANNEL_DELETE,
	DMIF_FILTER_NOT_FOUND, URL_MUST_REFER_TO_A_SINGLE_OD,
 	INVALID_UI_CONFIG, INVALID_UI_BUFFER,											// CC 010122
	//JLF: generic error
	VIDEO_RENDER_ERROR, AUDIO_RENDER_ERROR, SCRIPT_ERROR, SCRIPT_MESSAGE,
	ONLY_OD_ID_URL_ALLOWED, IPMP_CANNOT_PROCESS_OD, IPMP_CANNOT_DELETE_OD,
	IPMP_CANNOT_PROCESS_IPMP_TOOL, IPMP_CANNOT_REMOVE_IPMP_TOOL
}
NON_FATAL_ERROR;

typedef enum tagFatalError
{
	AUDIO_INIT_FAILED,
	VIDEO_INIT_FAILED,
	/*used to signal errors without varg which seems broken*/
	OTHER_ERROR
}
FATAL_ERROR;

typedef enum tagServiceType
{	
	FILE_SERVICE, REMOTE_SERVICE
}
SERVICE_TYPE;

typedef enum tagUserEvent
{
	EVENT_MOUSE_DOWN, EVENT_MOUSE_UP, EVENT_MOUSE_MOVE,
	EVENT_LEFT_BUTTON_DOWN, EVENT_LEFT_BUTTON_UP,
	EVENT_RIGHT_BUTTON_DOWN, EVENT_RIGHT_BUTTON_UP,
	EVENT_MIDDLE_BUTTON_DOWN, EVENT_MIDDLE_BUTTON_UP
} USER_EVENT;

typedef enum tagControlKey
{
	CK_CONTROL, CK_SHIFT, CK_ALT, CK_NONE
} CONTROL_KEY;

typedef struct
{
	unsigned short width;
	unsigned short height;
	unsigned short pitch;
	signed short offsetX;
	signed short offsetY;
} DecoderImage;

typedef struct
{
    DWORD samplesPerSecond;
    UCHAR bitsPerSample;
    UCHAR numChannels;
} DecoderAudioFormat;

#endif                                      // _Im1defs_h sentry.
