/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

#if !defined(_Inline_h)	// Sentry, use file only if it's not already included.
#define _Inline_h

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Executive
FILE:       Inline.h
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Definition of InlineImp, the objects that implements Inline
-------------------------------------------------------------------------*/
#include "Nodes.h"

// SOLARIS PORTING - To Core 5.9rc9 - Added for the SFNode Definition
// Solved the problem - but not quite sure
#include "CrNodes.h"

class ObjectDescriptorImp;
class ES_DescriptorImp;
class IPMP_DescriptorImp;
class ClockReference;
class DecoderImp;
class FlexTimeMgr;

class InlineImp : public Inline
{
	SFNode<Group> m_subScene;
	struct StreamAssociation
	{	int m_nODid;
		StreamConsumer *m_pNode;
		ObjectDescriptorImp *m_pOD;
		StreamAssociation (int nODid = -1) :
			m_nODid (nODid), m_pNode (NULL), m_pOD (NULL)
		{}
		~StreamAssociation ();
		operator int () const {return m_nODid;}
	};
	struct IPMPDAssociation
	{	int m_nID;
		IPMP_DescriptorImp *m_pIPMPD;
		ZArray<IPMPManager*> m_systems;
		IPMPDAssociation (int nID = 0) : m_nID (nID), m_pIPMPD (NULL) {}
		~IPMPDAssociation ();
		operator int () const {return m_nID;}
	};

#ifdef MPGJ
//MPEGJ MERGE  - RLS
public: //made this public to acess from other files to get the ODs - PA
//	ZArray<StreamAssociation> m_streamAssociation;
//MPEGJ MERGE - RLS
#endif
#ifndef MPGJ
private:
//	ZArray<StreamAssociation> m_streamAssociation;
#endif
	ZArray<StreamAssociation> m_streamAssociation;
	ZArray<IPMPDAssociation> m_IPMPDs;
	bool m_bCodeNames, m_bExternProto, m_bActive;
	ZMutex m_mtxStreams;
	DecoderImp *m_pBifsDecoder;
	FlexTimeMgr *m_pFlexTimeMgr;
	DWORD m_dwLastMediaTime;

	double m_dDuration;

private:
	virtual Decoder *CreateDecoder (
		DecoderConfigDescriptor *pDecoderConfig,
		QoS_Descriptor *pQoS,
		Decoder *
	);
	virtual BOOL Render (PresenterContext *pContext);
	virtual BOOL Parse (BIFSDecoder *pBifs);

public:
	virtual void ODRemoved ();
	void RemoveDecoder (DecoderImp *pDecoder);
	DWORD GetClock () const;
	void ConnectRemoteOD (ObjectDescriptorImp *pOD);
	void ParseUrl (BIFSDecoder *pCoder);
	InlineImp (bool bIsExternProto = false) :
		m_bExternProto (bIsExternProto),
		m_pBifsDecoder (NULL),
		m_bActive (true),
		m_pFlexTimeMgr(NULL),
		m_dDuration(-2),
		m_dwLastMediaTime(0)
	{}
	void SetCodeNamesFlags (bool flag) {m_bCodeNames = flag;}
	bool IsCodeNames () const {return m_bCodeNames;}
	void RemoveIPMP (int nID, IPMPManager *pIPMP);
	void AssociateIPMPSystem (int nID, IPMPManager *pIPMP);
	virtual void DeleteObject ();
	void Init ();
	Root *GetScene () const;
	IPMP_DescriptorImp *GetIPMP_Descriptor (int nIPMPid);
	void RemoveIPMPD (int nIPMPid);
	void UpdateIPMPD (IPMP_DescriptorImp *pIPMPD);
	void RemoveESD (int nODid, int nESDid);
	void UpdateESD (int nODid, ES_DescriptorImp *esd);
	void RemoveOD (int nODid);
	void UpdateOD (ObjectDescriptorImp *pOD);
	void ReplaceScene (Root *pScene);
	void RegisterStreamConsumer (StreamConsumer *pNode);
	void UnregisterStreamConsumer  (StreamConsumer *pNode);
	FlexTimeMgr *GetFlexTimeMgr () const {return m_pFlexTimeMgr;}
	void ExecuteOD (int nODid);

	/*overload set stream for re-initialization of subgraph*/
	virtual void SetStream (ObjectDescriptorImp *pOD);

	/*returns TRUE if inline scene is done playing*/
	bool IsFinished();

};

#endif                                      // _Inline_h sentry.
