/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Optibase Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Optibase Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1997.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

#if !defined(_MediaControl_h)	// Sentry, use file only if it's not already included.
#define _MediaControl_h

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:	Compositor (simulation)
FILE:       MediaControl.h
AUTHORS:    Zvi Lifshitz (Optibase Ltd.)
OVERVIEW:   Definitions of MediaControlImp - the node that implements MediaControl
-------------------------------------------------------------------------*/

#include "Nodes.h"

class MediaControlImp : public MediaControl
{
private:
	void SwitchControl (MediaControlImp *pNew);
	virtual void RemoveStream ();
	virtual void OnUpdate ();
	virtual void SetStream (ObjectDescriptorImp *pOD);
	void CalculatePlayInterval (
		double dSegmentStart,
		double dSegmentDuration,
		double &dStart,
		double &dStop
	) const;

public:
	void StreamStopped ();
	bool IsUnitPlayable (double dTS);
	void Play (int nChannel, BOOL bIsPreroll);
	MediaControlImp ();
	virtual BOOL Render (PresenterContext *context);
	/*used by media nodes to know whether the controled streams needs to be restarted*/
	void AddUser();
	void RemoveUser();

private:
	bool m_bStart, m_bChanged, m_saveMute, m_bPaused;
	ZArray<Url> m_saveUrl;
	double m_saveMediaStartTime, m_saveMediaStopTime, m_saveMediaSpeed;
	bool m_bEnabled, m_bPreRolled;
	unsigned int m_iNumUser;
	BOOL m_bActive, m_bPrevActive;

	void Pause();
	void Resume();
	void SetSpeed(double speed);
	void PreRoll();

};

#endif                                      // _MediaControl_h sentry.
