/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright ?1997.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  BIFS Decoder/ Compositor / Presenter
FILE:       MediaObj.cpp
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Implementation of base class for MediaObject objects.
-------------------------------------------------------------------------*/
//$Log: MediaObj.cpp,v $
//Revision 1.3  2006/10/20 14:14:53  granger
//MulltiResolution FootPrint Based Representation :
//Nodes, encoding, decoding, and visualization functionnalites on 3DPlayer addition
//
//Revision 1.2  2004/07/23 08:19:28  shinjun
//no message
//
//Revision 1.1.1.1  2003/12/24 00:42:10  mikeseven
//Initial release of the CVS repository for MPEG-4 Systems code
//after the code was corrupted due to reorganization and human 
//errors.
//
//Revision 1.14  2003/04/14 16:47:48  jean_lf
//MediaControl cleanup
//
//Revision 1.13  2003/04/03 04:29:02  mikeseven
//Initial release of Player3D and MPEG-J for all MPEG-4 reference softwares players with samples.
//For installation, follow instructions in docs/Player3D_setup.doc.
//
//Many thanks to Prashanth Gujjeti and Viswanathan Swaminathan from Sun Microsystems.
//
//-- Mikael Bourges-Sevenier (Mindego)
//
//Revision 1.12  2003/01/23 09:57:12  jean_lf
//no message
//
//Revision 1.11  2003/01/20 17:58:38  jean_lf
//no message
//
//Revision 1.10  2003/01/17 02:10:05  sowoo
//no message
//
//Revision 1.9  2003/01/15 12:32:33  sowoo
//BitWrapper / buffer version
//
//Revision 1.6  2002/11/18 17:17:18  jean_lf
//More Proto fixes and MediaSensor cleaned up
//
//Revision 1.5  2002/11/15 12:32:19  lmwang
//lmwang,have fixed mistaken entry bugs of ReplaceIndexedValue
//
//Revision 1.4  2002/10/30 12:27:30  jean_lf
//Diverse Proto fixes
//
//Revision 1.3  2002/10/25 09:32:32  jean_lf
//no message
//
//Revision 1.2  2002/08/08 09:47:26  zvil
//Core 5.9 Release Candidate 9:
//- Handling of time fields
//- Generating BIFS from memory
//- Proper handling of BIFS tables
//- More IPMPX stuff
//- Fixes to IM1-2D for proper use of startTime / stopTime
//- Fixes to Im1Player for proper use of startTime / stopTime
//
//Revision 1.2  2000/03/03 18:41:21  Alex Cotarmanac'h
//Version 1 -- Alexandre Cotarmanac_h
//
//Modified 2001/05/08 Boris Felts boris.felts@polytechnique.org

#include "defs.h"
#include <assert.h>
#include "MediaObj.h"
#include "Executiv.h"
#include "BifsDec.h"
#include "Route.h"
#include "Presenter.h"
#include "NodeFld.h"
#include "bifs.h"
#include "Root.h"
#include "Quantize.h"
#include "Nodes.h"
#include "RendObj.h"
#include "Except.h"
#include "Proto.h"
#include "Inline.h"
#include "NodeCode.h"  //BF Script
#include "ScriptImp.h" // BF Script
#include "../UI/UIStreamDec.h"

// SOLARIS PORTING - Merging with 2D player
#ifndef PLAYER3D
#ifndef OSMOSE
#include "../IM1Decoders/AFX/JWAFXStreamDecoder.h"	// SOWOO 20030109: BitWrapper "buffer" version
#endif
#endif //PLAYER3D

/*-----------------------------------------------------------------
MediaObject constructor
-----------------------------------------------------------------*/
MediaObject :: MediaObject ()
{
	m_pRoot = NULL;
	m_nUse = 0;
	m_pFirstRoute = NULL;
	m_nID = -1;
	m_pExecutive = NULL;
	m_pRenderingObject = NULL;
	m_bIsRoot = FALSE;
	m_pClone = NULL;
	m_bIsBeingParsed = false;

	/*-------------------------------------------------------------
	Grouping initialization is now part of every node
	-------------------------------------------------------------*/
#ifdef PLAYER
	CUSTOMIZE_GROUPNODE_INITIALIZATION
#endif
}

/*-------------------------------------------------------------
MediaObjects are deleted by a call to DeleteObject. This ensures that the rendering
object is deleted before the object itself.
-------------------------------------------------------------*/
void MediaObject :: DeleteObject ()
{
	/*-----------------------------------------------------------------
	Delete all routes which this is their arrival node
	-----------------------------------------------------------------*/
	while (m_pFirstRoute)
		m_pFirstRoute ->Remove ();  // this unlinks the route automatically

	/*-----------------------------------------------------------------
	If this node has an nID unregister it from the root
	-----------------------------------------------------------------*/
	if (m_pRoot && m_nID >= 0)
		m_pRoot -> UnregisterNode (this);

	/*-----------------------------------------------------------------
	Acknowledge all containers that this node does not exist
	-----------------------------------------------------------------*/
	for (int i = m_containers.GetLength (); --i >= 0;)
		if (m_containers [i] != NULL)
			if (m_containers [i].m_bISMF)
				((MFGenericNode *) m_containers [i].m_pContainer) ->
					RemoveNode (this);
			else
				((SFGenericNode *) m_containers [i].m_pContainer) ->
					RemoveNode ();
	if (m_pExecutive == NULL)
		delete this;
	else
		m_pExecutive ->RemoveNode (this);
}

/*-----------------------------------------------------------------
MediaObject destructor.
-----------------------------------------------------------------*/
MediaObject :: ~MediaObject ()
{
	delete m_pRenderingObject;
}

/*-----------------------------------------------------------------
Store the Executive pointer and perform object initialization
Create and attach a rendering object
-----------------------------------------------------------------*/
void MediaObject :: SetExecutive (Executive *pExecutive)
{
	m_pExecutive = pExecutive;
	if (pExecutive)
		m_pPresenter = (Presenter *) pExecutive -> GetPresenter ();
	RenderingObject *pObject = CreateRenderingObject (this);
	if (pObject)
	{	pObject -> Setup (this);
		m_pRenderingObject = pObject;
	}
}

/*-----------------------------------------------------------------
SetRoot stores a pointer to the root object of this object.
If this object has an ID, it registers it with the root.
-----------------------------------------------------------------*/
void MediaObject :: SetRoot (Root *pRoot)
{
	m_pRoot = pRoot;
	if (m_nID >= 0 && pRoot)
		pRoot -> RegisterNode (this);
}

/*-----------------------------------------------------------------
Set the node ID and register it with the node if necessary
-----------------------------------------------------------------*/
void MediaObject :: SetID (int nID)
{
	m_nID = nID;
	if (nID >= 0 && m_pRoot)
		m_pRoot -> RegisterNode (this);
}

/*-----------------------------------------------------------------
Parse a MediaObject by using the field table.
-----------------------------------------------------------------*/
BOOL MediaObject :: Parse (BIFSDecoder *pCoder)
{
	TRACEF ("%s {", GetName());
	TRACENEWLINE;
	TRACEINDENT (1);

	bool bResult = true, bProtoCode = IsProtoCode ();
	int nFieldCount = GetBifsBounds () -> m_nTotalDefs;
	if (bProtoCode)
		nFieldCount = GetBifsBounds () -> m_nTotalAll;

	/*-------------------------------------------------------------
	Set a flag to indicate that this node is now being parsed. This is necessary so that
	if this is a PROTO instance, its ISed time fields will be offset when necessary
	-------------------------------------------------------------*/
	m_bIsBeingParsed = true;

	/*-----------------------------------------------------------------
	A node may be described by a mask description or by a list
	description
	-----------------------------------------------------------------*/
	if (pCoder -> ParseInt (1, "maskAccess"))  // if MaskAccess //BF Conformance
	{	/*-------------------------------------------------------------
		A mask bit tell us whether it is followed by a field description
		If the defintion is part of Proto code the syntax is different because
		every field may be an IS. In this case we index into the entire structure,
		not just the DEFs
		-------------------------------------------------------------*/
		for (int i = 0; i < nFieldCount; i++)
		{
			if (pCoder -> ParseInt (1, "mask")) //BF Conformance
			{	if (!bProtoCode)
 					// SOLARIS PORTING - removed bResult because of assignment problem
 					//bResult = ParseField (pCoder, GetBifsDefEntry (i));
 					ParseField (pCoder, GetBifsDefEntry (i));
				else if (pCoder -> ParseInt (1, "isedField"))
					ParseISref (pCoder, i);
				else
 					// SOLARIS PORTING - removed bResult because of assignment problem
 					//bResult = ParseField (pCoder, GetFieldTable () + i);
 					ParseField (pCoder, GetFieldTable () + i);
			}
		}
	}
	else		// ListNodeDescription
	{
		/*-----------------------------------------------------------------
		Now loop for each field.
		-----------------------------------------------------------------*/
		while (!pCoder -> ParseInt (1, "endFlag"))	// while !endFlag
		{
			if (bProtoCode && pCoder -> ParseInt (1, "isedField"))
				ParseISref (
					pCoder,
					pCoder -> ParseInt (GetBifsBounds () -> m_nBitsPerAll, "fieldRef") //AC20011102 comment changed (swapped)
				);
			else
			{	int nIndex =
					pCoder -> ParseInt (GetBifsBounds () -> m_nBitsPerDef, "def"); // mp : conformance //AC20011102 comment changed
 				// SOLARIS PORTING - removed bResult because of assignment problem
 				//bResult = ParseField (pCoder, GetBifsDefEntry (nIndex));
 				ParseField (pCoder, GetBifsDefEntry (nIndex));
			}
		}
	}

// SOLARIS PORTING - Merging with 2D player
#ifndef PLAYER3D
#ifndef OSMOSE

	// SOWOO 20030107: Implementation of BitWrapper "buffer" version
	if(this->GetNodeCode() == CODE_BitWrapper)
	{
		// if "url" field has no data, there is data in "buffer" field.
		// if "buffer" field has data, this data is decoded here,
		// otherwise, the data is decoded using ES_DescriptorImp
		if(((BitWrapper*) this)->GetURL().GetLength() == 0)
		{
			if(((BitWrapper*) this)->buffer.GetLength() == 0)
				// there is no data to be decoded.
				return FALSE;

			JWAFXObjectStreamDecoder* pAFXCoder = 0;

			// create the decoder depending on the AFX object
			switch(((BitWrapper*) this)->node->GetNodeCode())
			{
			case CODE_IndexedFaceSet:
				pAFXCoder = new JWAFX3DMCDecoder;
				break;

			case CODE_MeshGrid:
				// insert the appropriate source code here.
				break;

			case CODE_WaveletSubdivisionSurface:
				// insert the appropriate source code here.
				break;

			case CODE_CoordinateInterpolator:
				pAFXCoder = new JWAFXCoordinateInterpolatorDecoder;
				break;

			case CODE_OrientationInterpolator:
				pAFXCoder = new JWAFXOrientationInterpolatorDecoder;
				break;

			case CODE_PositionInterpolator:
				pAFXCoder = new JWAFXPositionInterpolatorDecoder;
				break;

			case CODE_OctreeImage:
				pAFXCoder = new YAFXOctreeDecoder;
				break;

			// 2004-07-15, Shinjun {
			case CODE_PointTexture:
				pAFXCoder = new AFXPointTextureDecoder;
				break;
			// 2004-07-15, Shinjun }
			
			case CODE_FootPrintSet:
				// insert the appropriate source code here.
				break;

			default:
				// unsupported node appears.
				return FALSE;
				break;
			}

			// decode the buffer
			if(pAFXCoder != 0)
			{
				pAFXCoder->SetBuffer(
					(char*) (const char*) ((BitWrapper*) this)->buffer.GetValue(),
					((BitWrapper*) this)->buffer.GetLength()
					);

				// decode "buffer" and fill the "node" with the decoded data
				pAFXCoder->Decode();

#ifdef TRACE_ENCODING
				// write the decoded values into "trace.txt" file
				pAFXCoder->WriteIntoTracetxt(pCoder);
#endif

				delete pAFXCoder;
			}
		}
	}
	// END SOWOO

#endif	//#ifndef OSMOSE
#endif //PLAYER3D


	m_bIsBeingParsed = false;

	TRACEINDENT (-1);
	TRACEF ("}", 0);
	TRACENEWLINE;
	SetUpdateTime ();


#ifdef OSMOSE
	//osmose needs notification upon creation for timefields
	OnUpdate();
#endif
	return TRUE;
}

/*-----------------------------------------------------------------
Add a Route to the node (this is the arrival node)
-----------------------------------------------------------------*/
void MediaObject :: LinkRoute (Route *pRoute)
{
	pRoute -> LinkArrival (m_pFirstRoute);
}

/*-----------------------------------------------------------------
Unlink a Route from the list of routes (this is the arrival node)
-----------------------------------------------------------------*/
void MediaObject :: UnlinkRoute (Route *pRoute)
{
	pRoute -> UnlinkArrival (m_pFirstRoute);
}

/*-----------------------------------------------------------------
A route to this node was activated. If this is a field, update the
field and notify the node. If this is an eventIn, activate the event
handling function.
-----------------------------------------------------------------*/
void MediaObject :: RouteEvent (NodeField *pField, int nFieldIndex)
{
	if (nFieldIndex < 0 || nFieldIndex >= GetFieldCount ())
		throw BifsException (BIFX_PARSING_ERROR);

	BOOL bHasChanged = GetField(nFieldIndex)->Copy(*pField, this, false);
	UpdateField(GetFieldTable () + nFieldIndex, bHasChanged);
}

/*-----------------------------------------------------------------
Calls Presenter :: GetTime () to retrieve the renderer "time", and
stamp the node with this value.  Used by the renderer to identify new
scene updates.
-----------------------------------------------------------------*/
void MediaObject :: SetUpdateTime ()
{
	if (m_pExecutive) 
		m_dwUpdateTime = m_pExecutive -> GetPresenter () -> GetTime ();
}

/*-----------------------------------------------------------------
Called by BIFSDecoder to insert a value into one of the MFxxx fields
of the node.  The field ID, the new value and the exact insertion
position are parsed from the BIFS stream by this function. Returns TRUE
if successful.
-----------------------------------------------------------------*/
BOOL MediaObject :: InsertValue (BIFSDecoder *pCoder)
{
	int nPos;
	BOOL bResult = FALSE;
	int nIndex =
		pCoder -> ParseInt (GetBifsBounds () -> m_nBitsPerIn, "inID");
	const BifsFieldTable *pEntry = GetBifsInEntry (nIndex);
	if (pEntry == NULL)
		return FALSE;
	Quantizer quant (
		pEntry -> m_fMin,
		pEntry -> m_fMax,
		pEntry -> m_nDimension,
		pEntry -> m_nQuantizationType
	);
	TRACEF ("%s ", pEntry -> m_szName);

	MField *pField = (MField *) GetField (pEntry);
	switch (pCoder -> ParseInt (2, "type")) {
	case 0:				// insertion at a specified position
		nPos = pCoder -> ParseInt (16, "position");
		TRACEF ("[%d] ", nPos);
		bResult = pField -> InsertElement (pCoder, this, nPos, &quant);
		break;
	case 2: 			// insertion at the beginning of the field
		TRACEF ("[0] ", 0);
		bResult = pField -> InsertElement (pCoder, this, 0, &quant);
		break;
	case 3: 			// insertion at the end of the field
		TRACEF ("[end] ", 0);
		bResult = pField -> AppendElement (pCoder, this, &quant);
		break;
	}
	if (bResult)
	{	UpdateField (pEntry);
		pField -> OnFieldUpdate ();
	}
	return bResult;
}

/*-----------------------------------------------------------------
Called by BIFSDecoder to delete a value from one of the MFxxx fields
of the node.  The field ID and the element index are parsed from the
BIFS by this function.  Returns TRUE if successful.
-----------------------------------------------------------------*/
BOOL MediaObject :: DeleteValue (BIFSDecoder *pCoder)
{
	int nPos;
	BOOL bResult = FALSE;
	int nIndex =
		pCoder -> ParseInt (GetBifsBounds () -> m_nBitsPerIn, "inID");
	const BifsFieldTable *pEntry = GetBifsInEntry (nIndex);
	if (pEntry == NULL)
		return FALSE;
	TRACEF ("%s ", pEntry -> m_szName);

	MField *pField = (MField *) GetField (pEntry);
	switch (pCoder -> ParseInt (2, "type")) {
	case 0:				// deletion from a specified position
		nPos = pCoder -> ParseInt (16, "position");
		TRACEF ("[%d] ", nPos);
		if (nPos < pField -> GetLength ())
		{	pField -> DeleteFrom (nPos);
			bResult = TRUE;
		}
		break;
	case 2: 			// deletion from the beginning of the field
		TRACEF ("[0] ", 0);
		if (pField -> GetLength () > 0)
		{	pField -> DeleteFrom (0);
			bResult = TRUE;
		}
		break;
	case 3: 			// delete last element of field
		TRACEF ("[end] ", 0);
		if (pField -> GetLength () > 0)
		{	pField -> DeleteFrom (pField -> GetLength () - 1);
			bResult = TRUE;
		}
		break;
	}
	if (bResult)
		UpdateField (pEntry);
	return bResult;
}

/*-----------------------------------------------------------------
Called by BIFSDecoder to replace a value of one of the nodes fields.
The field ID and new value are parsed from the BIFS by this function.
Returns TRUE if successful.
-----------------------------------------------------------------*/
BOOL MediaObject :: ReplaceValue (BIFSDecoder *pCoder)
{
	int nIndex =
		pCoder -> ParseInt (GetBifsBounds () -> m_nBitsPerIn, "inID");
	const BifsFieldTable *pEntry = GetBifsInEntry (nIndex);
	if (pEntry == NULL)
		return FALSE;

	BOOL bHasChanged = TRUE;
	NodeField *pField = ParseField(pCoder, pEntry, &bHasChanged);
	if (pField)
	{
		UIStreamDecoder *pUIDec = (UIStreamDecoder *) pCoder->GetUICoder();
		if (pUIDec) {
			pUIDec->SetFields(pField, this, pEntry);
			return TRUE;
		}
		UpdateField (pEntry, bHasChanged);
		return TRUE;
	}
	return FALSE;
}

/*-----------------------------------------------------------------
Called by BIFSDecoder to replace a value of one of the elements of
one of the MFxxx fields of the node.  The field ID, the element index
and the new value are parsed from the BIFS by this function.  Returns
TRUE if successful.
-----------------------------------------------------------------*/
BOOL MediaObject :: ReplaceIndexedValue (BIFSDecoder *pCoder)
{
	int nPos;
	BOOL bResult = FALSE;
	int nIndex =
		pCoder -> ParseInt (GetBifsBounds () -> m_nBitsPerIn, "index");
	const BifsFieldTable *pEntry = GetBifsInEntry (nIndex);
	if (pEntry == NULL)
		return FALSE;

	// MBS: init quantizer with current QuantizationParameter node
	Quantizer quant(pCoder->GetCurrentQP(),
			pEntry -> m_fMin,
			pEntry -> m_fMax,
			pEntry -> m_nDimension,
			pEntry -> m_nQuantizationType);
	// end MBS

	TRACEF ("%s ", pEntry -> m_szName);

	MField *pField = (MField *) GetField (pEntry);
	switch (pCoder -> ParseInt (2, "type")) {
	case 0:				// replacement a specified position
		nPos = pCoder -> ParseInt (16, "position");
		TRACEF ("[%d] ", nPos);
		break;
	//limin wang fix,by std
	//case 1: 			// replacement at the beginning of the field
	case 2: 			// replacement at the beginning of the field
	//end
		nPos = 0;
		TRACEF ("[0] ", 0);
		break;
	//limin wang fix,by std
	//case 2: 			// Replacement at the end of the field
	case 3: 			// Replacement at the end of the field
	//end
		TRACEF ("[end] ", 0);
		if (pField -> GetLength () <= 0)
		{	GetExecutive () -> Error (BIFS_UPDATE_REQUEST_DOES_NOT_MATCH_SCENE);
			return false;
		}
		nPos = pField -> GetLength () - 1;
		break;
	default:
		GetExecutive () -> Error (SCENE_PARSING_ERROR);
		return false;
	}

	//bHasChanged isn't modified by most of the MF<> classes. Set it to TRUE (default)
	BOOL bHasChanged = TRUE;
	bResult = pField -> ReplaceElement (pCoder, this, nPos, bHasChanged, &quant);

	if (bResult)
	{
		UIStreamDecoder *pUIDec = (UIStreamDecoder *) pCoder->GetUICoder();
		if (pUIDec) {
			pUIDec->SetFields(pField, this, pEntry, nPos);
			return bResult;
		}
		//trigger routes
		pField -> OnFieldUpdate ();
		//and generate update on the field if changed
		UpdateField (pEntry, bHasChanged);
	}
	return bResult;
}

BOOL MediaObject :: MultipleReplaceIndexedValue (BIFSDecoder *pCoder)
{
	int nPos;
	BOOL bResult = FALSE;
	
	int nIndex = pCoder -> ParseInt (GetBifsBounds () -> m_nBitsPerIn, "index");
	const BifsFieldTable *pEntry = GetBifsInEntry (nIndex);
	if (pEntry == NULL) return FALSE;

	int lenPos = pCoder->ParseInt(5, "lenPosition");
	int lenNum = pCoder->ParseInt(5, "lenNum");
	int numPos = pCoder->ParseInt(lenNum, "numPositions");

	Quantizer quant(pCoder->GetCurrentQP(),
			pEntry -> m_fMin,
			pEntry -> m_fMax,
			pEntry -> m_nDimension,
			pEntry -> m_nQuantizationType);

	MField *pField = (MField *) GetField (pEntry);
	UIStreamDecoder *pUIDec = (UIStreamDecoder *) pCoder->GetUICoder();
	
	BOOL bHasChanged = TRUE;
	for (int i=0; i<numPos; i++) {
		nPos = pCoder -> ParseInt(lenPos, "position");
		TRACEF ("%s", pEntry -> m_szName);
		TRACEF ("[%d] ", nPos);
		bResult = pField -> ReplaceElement(pCoder, this, nPos, bHasChanged, &quant);
		if (!bResult) break;
		if (pUIDec) pUIDec->SetFields(pField, this, pEntry, nPos);
	}

	if (bResult) {
		//trigger routes
		pField -> OnFieldUpdate ();
		//and generate update on the field if changed
		UpdateField (pEntry, bHasChanged);
	}
	return bResult;
}

/*-----------------------------------------------------------------
Similar to Parse(), this function is called by BIFSDecoder to insert
one node into the children field of the node.  The information about
the new node and its exact position are parsed from the BIFS stream by
this function.   Returns TRUE if successful.
-----------------------------------------------------------------*/
BOOL MediaObject :: ParseChild (BIFSDecoder *pCoder)
{
	int nPos;
	BOOL bResult = FALSE;
	MFGenericNode *pChildren = GetChildrenField ();
	if (pChildren == NULL)
		return false;

	switch (pCoder -> ParseInt (2, "type")) {
	case 0:				// insertion at a specified position
		nPos = pCoder -> ParseInt (8, "position");
		TRACEF ("[%d] ", nPos);
		bResult = pChildren -> InsertElement (pCoder, this, nPos);
		break;
	case 2: 			// insertion at the beginning of the field
		TRACEF ("[0] ", 0);
		bResult = pChildren -> InsertElement (pCoder, this, 0);
		break;
	case 3: 			// insertion at the end of the field
		TRACEF ("[end] ", 0);
		bResult = pChildren -> AppendElement (pCoder, this);
		break;
	}
	if (bResult) {
		SetUpdateTime ();
		OnUpdate ();
		pChildren -> OnFieldUpdate ();
		m_pExecutive->InvalidateScene(this);
	}
	return bResult;
}

/*-----------------------------------------------------------------
Add a new Route whose departure field is one of the fields of this
node.  This departure field ID and arrival node ID are parsed from the
BIFS stream, then BindNode() is called in order to link the new Route
to the proper arrival field.  Returns TRUE if successful.
-----------------------------------------------------------------*/
BOOL MediaObject :: AddRoute (int nRouteID,  BIFSDecoder *pCoder, LPCSTR name)
{
	int nIndex =
		pCoder -> ParseInt (GetBifsBounds () -> m_nBitsPerOut, "outFieldRef"); //mp conformance
	nIndex = GetBifsOutEntry (nIndex);
	if (nIndex < 0)
		return FALSE;
	const BifsFieldTable *pEntry = GetFieldTable () + nIndex;
	TRACEF ("%s TO ", pEntry -> m_szName);
	Route *pRoute = new Route (nRouteID, GetField (pEntry), nIndex, this, name);

	int nNodeID = pCoder -> ParseInt (pCoder -> GetNodeIDbits (), "inNodeID"); //BF Conformance
	MediaObject *pNode = GetRoot () -> FindNode (nNodeID);
	if (pNode == NULL)
		return FALSE;
	TRACEF ("%s.", pNode ->GetNameID ());
	return pNode -> BindRoute (pRoute, pCoder);
}

/*-----------------------------------------------------------------
Having instantiated a new Route, and binding it to a departure field,
AddRoute() calls this function where it requests the node to bind the
Route to one of its fields as an arrival field.  The arrival field index
is parsed from the BIFS stream.  Returns TRUE if successful.
-----------------------------------------------------------------*/
BOOL MediaObject :: BindRoute (Route *pRoute, BIFSDecoder *pCoder)
{
	int nIndex =
		pCoder -> ParseInt (GetBifsBounds () -> m_nBitsPerIn, "inFieldRef"); //BF Conformance
	const BifsFieldTable *pTable = GetFieldTable ();

	for (int i = GetFieldCount (); --i >= 0;)
		if (nIndex == pTable [i].m_inID)
		{	TRACEF ("%s", pTable [i].m_szName);
			pRoute -> Bind (this, i);
			return TRUE;
		}

	return FALSE;
}

/*-----------------------------------------------------------------
This function is called after recalculation of the bounding-box.  The
function calls UpdateBBox() on all the nodes parents.
-----------------------------------------------------------------*/
void MediaObject :: OnBBoxUpdate ()
{
	for (int i = m_parents.GetLength (); --i >= 0;)
		m_parents [i] -> UpdateBBox ();
}

/*-----------------------------------------------------------------
Add the container field to the list of containers and the parent node
to the list of parents.
-----------------------------------------------------------------*/
void MediaObject :: SetContainer (
	NodeField *pContainer,
	BOOL bISMF,
	MediaObject *pParent
)
{
	if (pContainer == NULL)
		return;
	m_containers += Container (pContainer, bISMF);
	SetParent (pParent);
}

/*-----------------------------------------------------------------
Remove the container field from the list of containers and delete
the corresponding rendering object. If the node
is not contained any more in any other node, delete it.
-----------------------------------------------------------------*/
void MediaObject :: RemoveContainer (NodeField
	*pContainer,
	MediaObject *pParent
)
{
	if (!pContainer)
		return;
	int i = m_containers.Find (pContainer);
	if (i >= 0)
	{	RemoveParent (pParent);
		m_containers.DeleteFrom (i);
	}

	if (m_containers.GetLength () == 0)
		Remove ();
}

/*-----------------------------------------------------------------
Returns a pointer to the BIFS table entry for the field whose defID is
nID
-----------------------------------------------------------------*/
const BifsFieldTable * MediaObject :: GetBifsDefEntry (int nID)
{
	const BifsFieldTable *pTable = GetFieldTable ();
	for (int i = GetFieldCount (); --i >= 0;)
		if (nID == pTable [i].m_defID)
			return pTable + i;
	return NULL;
}

/*-----------------------------------------------------------------
Returns a pointer to the BIFS table entry for the field whose inID is
nID
-----------------------------------------------------------------*/
const BifsFieldTable * MediaObject :: GetBifsInEntry (int nID)
{
	const BifsFieldTable *pTable = GetFieldTable ();
	for (int i = GetFieldCount (); --i >= 0;)
		if (nID == pTable [i].m_inID)
			return pTable + i;
	return NULL;
}

/*-----------------------------------------------------------------
Returns a pointer to the BIFS table entry for the field whose outID is
nID
-----------------------------------------------------------------*/
int MediaObject :: GetBifsOutEntry (int nID)
{
	const BifsFieldTable *pTable = GetFieldTable ();
	for (int i = GetFieldCount (); --i >= 0;)
		if (nID == pTable [i].m_outID)
			return i;
	return -1;
}

/*-----------------------------------------------------------------
Returns a pointer to the BIFS table entry for the field whose dynID is
nID
-----------------------------------------------------------------*/
const BifsFieldTable * MediaObject :: GetBifsDynEntry (int nID)
{
	const BifsFieldTable *pTable = GetFieldTable ();
	for (int i = GetFieldCount (); --i >= 0;)
		if (nID == pTable [i].m_dynID)
			return pTable + i;
	return NULL;
}

/*-----------------------------------------------------------------
Parse a node field, when a pointer to the BIFS table entry describing
that field is given
-----------------------------------------------------------------*/
NodeField *MediaObject :: ParseField (
	BIFSDecoder *pCoder,
	const BifsFieldTable *pEntry,
	BOOL *bHasChanged
)
{
	if (pEntry == NULL)
		return FALSE;

	// MBS: init quantizer with current QuantizationParameter node
	Quantizer quant(pCoder->GetCurrentQP(),
			pEntry -> m_fMin,
			pEntry -> m_fMax,
			pEntry -> m_nDimension,
			pEntry -> m_nQuantizationType);
	if (pEntry->m_nQuantizationType == 13)	//AC20011109
	{
		int nBits=pEntry->m_quantParams;
		//printf("QP13 : setting nbBits to %d\n",nBits);
		quant.setQuantizationNbBits(nBits);
	}
	// end MBS
//#ifdef PREDICTIVE	//AC 20000227 QP 14  //BF
	if (pEntry->m_nQuantizationType == 14 )
	{
		if (pCoder->isCoordActive()) {
			int k=pCoder->getLengthCoord(); // nb fields
			int nbits=(int) ceil(log(k+1)/log(2)); //so as to be able to store -1.
			// Ceil in the spec is math::ceil(x+1)
			quant.setQuantizationNbBits(nbits);
		}
	}
//#endif	//AC end  //BF

	NodeField *pField = GetField (pEntry);

	// SOWOO 20030109: save the original flag information and add flag property
	int tempFlags = 0;
	if ((GetNodeCode() == CODE_BitWrapper) && (::strcmp(pEntry->m_szName, "buffer") == 0))
	{
		tempFlags = pCoder->GetFlags();
		pCoder->SetFlags(tempFlags | STRING_FROM_FILE);

		TRACEF("%s", "# The following \"buffer\" field contains non-printable character set.");
		TRACENEWLINE;
		TRACEF("%s", "# Those non-printable character set are converted to printable ones for printing out as follows.");
		TRACENEWLINE;
	}
	// SOWOO end

    if (GetNodeCode () != CODE_Script || pField -> GetType() != MFSTRING)
	{ // "url" field in scripts will be written once the script field decoded
	    TRACEF ("%s ", pEntry -> m_szName);  //BF write the name of the field
    }
    
	//JLF: detect field changes. If same value don't invalidate the scene
	if (pField) {
		if (pField -> Parse (pCoder, this, bHasChanged, &quant))
			pField ->OnFieldUpdate ();
	}

	// SOWOO 20030109: restore flags
	if ((GetNodeCode() == CODE_BitWrapper) && (::strcmp(pEntry->m_szName, "buffer") == 0))
		pCoder->SetFlags(tempFlags);
	// SOWOO end

	TRACENEWLINE;
	return pField;
}

/*-----------------------------------------------------------------
A node is replaced by replaceNode BIFS command.
-----------------------------------------------------------------*/
void MediaObject :: ReplaceNode (BIFSDecoder *pCoder)
{
	/*-----------------------------------------------------------------
	Parse a new node.
	-----------------------------------------------------------------*/
	SFWorldNode node;
	node.ParseNodeField (pCoder, this, NULL, FALSE);

	/*-----------------------------------------------------------------
	Go through all the containers and replace
	the pointer to the old node by the new one
	-----------------------------------------------------------------*/
	for (int i = m_containers.GetLength (); --i >= 0;)
		if (m_containers [i] != NULL)
		{	if (m_containers [i].m_bISMF)
				((MFGenericNode *) m_containers [i].m_pContainer) ->
					ReplaceNode (this, node.GetNode ());
			else
				((SFGenericNode *) m_containers [i].m_pContainer) ->
					ReplaceNode (node.GetNode ());
			m_containers [i].m_pContainer = NULL;
		}

	/*-----------------------------------------------------------------
	Call OnUpdate on the new node then delete this (the old) node
	-----------------------------------------------------------------*/
	if (node.GetNode()) {
		node.GetNode () -> SetUpdateTime ();
		node.GetNode () -> OnUpdate ();
	}
	Remove ();
}

/*-----------------------------------------------------------------
A node was deleted by a DeleteNode BIFS command
-----------------------------------------------------------------*/
void MediaObject :: Delete (BOOL bExtDelete)
{
	int pos = -1;
	/*-----------------------------------------------------------------
	Notify all the containers that the node was deleted
	-----------------------------------------------------------------*/
    // PORTING to CC 5.3
    int i = 0;
//	for (int i = m_containers.GetLength (); --i >= 0;)
	for ( i = m_containers.GetLength (); --i >= 0;)
		if (m_containers [i] != NULL)
		{	if (m_containers [i].m_bISMF) 
				pos = ((MFGenericNode *) m_containers [i].m_pContainer) ->
					RemoveNode (this);
			else
				((SFGenericNode *) m_containers [i].m_pContainer) ->
					RemoveNode ();
			NodeField *pField = m_containers [i].m_pContainer;
			pField -> OnFieldUpdate ();
			m_containers [i].m_pContainer = NULL;
		}

	/*-----------------------------------------------------------------
	Notify all parents that node was deleted
	-----------------------------------------------------------------*/
	for (i = m_parents.GetLength (); --i >= 0;) {
		if ( (pos >= 0) && bExtDelete && (m_parents[i]->GetNodeCode() == CODE_OrderedGroup)) {
			OrderedGroup *pOG = (OrderedGroup *) m_parents[i];
			if (pOG->order.GetLength()>=pos) pOG->order.DeleteFrom(pos);
		}
		m_parents [i] -> SetUpdateTime ();
		m_parents [i] -> OnUpdate ();
	}

	/*-----------------------------------------------------------------
	Delete the node
	-----------------------------------------------------------------*/
	Remove ();
}

/*-----------------------------------------------------------------
A field was updated as a result of a Route or BIFS update
pEntry points to the entry in the BIFS table describing the field
-----------------------------------------------------------------*/
void MediaObject :: UpdateField (const BifsFieldTable *pEntry, BOOL bHasChanged)
{
	EventHandler function = pEntry -> m_function;
	if (function == NULL) {
		OnUpdate();
		//JLF: notify the exectutive that the object has changed
		if (bHasChanged) {
			SetUpdateTime();
			m_pExecutive->InvalidateScene(this);
		}
	}
	else
		(*this.*function) ();
}

void MediaObject :: OnUpdate()
{
}


/*-------------------------------------------------------------
Parse the reference of an ISed field
-------------------------------------------------------------*/
void MediaObject :: ParseISref (BIFSDecoder *pCoder, int nField)
{
	const BifsFieldTable *pNodeEntry = GetFieldTable () + nField;
	TRACEF ("%s ", pNodeEntry -> m_szName);
	Proto *pProto = (Proto *) GetRoot ();
	int nBits = pProto -> GetBifsBounds () -> m_nBitsPerAll;
	int nArg = pCoder -> ParseInt (nBits, "protoField");
	const BifsFieldTable *pProtoEntry = pProto -> GetFieldTable () + nArg;
	TRACEF ("IS %s", pProtoEntry ->m_szName);
	TRACENEWLINE;

	/*-------------------------------------------------------------
	The field is ISed by linking it to the Proto's field through Routes.
	The Route will go from the Proto to the node if the Proto field is EventIn.
	The Route will go from the node to the Proto if the Proto field is EventOut.
	There will be 2 Routes going both ways if the Proto field is an ExposedField.
	In addition, if this is a Field or an ExposedField the value of the Proto field is copied
	to the node field
	-------------------------------------------------------------*/
	NodeField *pNodeField = GetField (pNodeEntry);
	NodeField *pProtoField = pProto -> GetField (nArg);
	if (pProtoEntry ->m_defID >= 0)
		pNodeField ->Copy (*pProtoField, this);
	if (pProtoEntry ->m_inID >= 0 || pProtoEntry ->m_defID >= 0)
	{	Route *pRoute = new Route (-2, pProtoField, nArg, pProto);
		pRoute ->Bind (this, nField);
	}
	if (pProtoEntry ->m_outID >= 0)
	{	Route *pRoute = new Route (-2, pNodeField, nField, this);
		pRoute -> Bind (pProto, nArg);
	}
}

/*-------------------------------------------------------------
Returns true if the definition of this node is part of Proto code
-------------------------------------------------------------*/
bool MediaObject :: IsProtoCode () const
{
	return m_pRoot && m_pRoot -> IsProto ();
}

/*-------------------------------------------------------------
Clone a node for the instantiation of a Proto
-------------------------------------------------------------*/
MediaObject * MediaObject :: Clone (Root *pScene)
{
	int NodeCode = GetNodeCode ();

	/*-------------------------------------------------------------
	if DEF/USE locate node instance in the current scene object
	-------------------------------------------------------------*/
	if (m_nID>=0) {
		MediaObject *pClone = pScene->FindNode(m_nID);
		if (pClone) 
			return pClone;
	}

	/*-------------------------------------------------------------
	Create a node of same type
	-------------------------------------------------------------*/
	MediaObject *pNode;
	if (NodeCode == -2) {
		pNode = ((Proto *)this)->Instantiate(pScene);
	} else {
		pNode = SFGenericNode ().CreateNode(NodeCode);
	}
	pNode ->SetExecutive (GetExecutive ());
	pNode ->SetRoot (pScene);

	if (m_nID>=0) {
		pNode -> SetID (m_nID);
	}
	//set current clone
	m_pClone = pNode;

	/*-------------------------------------------------------------
	Clone the values of all the fields
	-------------------------------------------------------------*/
	for (int i = 0; i < GetFieldCount (); i++)
		GetField (i) ->Clone (pNode ->GetField (i), pNode);
	pNode -> SetUpdateTime ();
	return pNode;
}

/*-------------------------------------------------------------
Remove all the fields of the node without deleting the node. This is done when
a node is removed from the scene but not deleted phisically
-------------------------------------------------------------*/
void MediaObject  :: RemoveFields ()
{
	for (int i = 0; i < GetFieldCount (); i++)
		GetField (i) -> Remove ();
}


/*-------------------------------------------------------------
Parse the name ID in case CODENAMES is active
-------------------------------------------------------------*/
void MediaObject ::ParseNameID (BIFSDecoder *pCoder)
{
	pCoder ->ParseString (m_name);
}

/*-------------------------------------------------------------
Return the DEF name of the node
-------------------------------------------------------------*/
LPCSTR MediaObject :: GetNameID () const
{
	if (m_name.GetLength () > 0)
		return m_name;
	if (m_nID < 0)
		return NULL;
	static char buf [20];
	sprintf (buf, "ID%d", m_nID);//AC 20000228 Vrml-like
	return buf;
}

#ifdef MPGJ		// SCENE API Integration
/*-------------------------------------------------------------
Inserts a route between a field in this node to a field in
the other node.
-------------------------------------------------------------*/
void MediaObject :: InsertRouteTo (int outID, MediaObject *toNode, int toID)
{
	Route *pRoute = new Route (-1, GetEventOut (outID), GetBifsOutEntry (outID), this);
	pRoute -> Bind ( toNode, toNode -> GetBifsInEntry (toID) - GetFieldTable () );
}

/*-------------------------------------------------------------
Deletes a route between a field in this node to a field in
the other node.
-------------------------------------------------------------*/
void MediaObject :: DeleteRouteTo (int outID, MediaObject *toNode, int toID)
{
//	for (int i = 0; i < m_pRoot -> GetRouteLength(); i++)
//	{
//		if (m_pRoot -> GetRoute (i) -> DoesLink (this, GetBifsOutEntry (outID), toNode, toNode -> GetBifsInEntry (toID) - GetFieldTable () ))
//		{
//			delete m_pRoot -> GetRoute (i);
//			break;
//		}
//	}
}
#endif // MPGJ

/*-------------------------------------------------------------
Delete a node physically
-------------------------------------------------------------*/
void MediaObject :: Remove()
{
	if (m_pRenderingObject)
		m_pRenderingObject ->DeleteObject ();
	DeleteObject ();
}

/*-------------------------------------------------------------
Get time of the scene this node belongs to
-------------------------------------------------------------*/
DWORD MediaObject :: GetClock () const
{
	if (GetRoot ())
		return GetRoot () -> GetSceneTime ();
	return 0;
}

/*-------------------------------------------------------------
Check the root of this object to see if it belongs to a sub-tree that is now being
parsed. Needed for correct parsing of ISed time fields in PROTO
-------------------------------------------------------------*/
bool MediaObject :: IsBeingParsed () const
{
	return m_pRoot && m_pRoot -> m_bIsBeingParsed;
}
