/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

#if !defined(_MediaObj_h)	// Sentry, use file only if it's not already included.
#define _MediaObj_h

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  BIFS Decoder/ Compositor / Presenter
FILE:       MediaObj.h
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Declarations of base classes for MediaObject objects.
-------------------------------------------------------------------------*/
#include "Im1defs.h"
#include "ztl.h"
#include "custom.h"

#ifdef BIFSENC	// SCRIPT
//#include "CParser.h"  //BF Script
class CParser;
#endif

class Executive;
class BIFSDecoder;
class Route;
class NodeField;
class MFGenericNode;
class Root;
class Presenter;
class AnimationStreamDecoder; // MBS: definition
class RenderingObject;
class PresenterContext;
class BifsGenerator;
struct BifsFieldTable;
struct BifsBounds;

#ifdef BIFSENC
class BIFSEncoder;
#endif

/*-------------------------------------------------------------------------
- MediaObject is the base class for constructing scene nodes.
- MediaObject objects can parse themselves from BIFS.
- MediaObject objects can render themselves.
- Each object should have default constructor, because during parsing,
the object is created first, then it loads itself.
-------------------------------------------------------------------------*/


/*-------------------------------------------------------------
MediaObject declaration
-------------------------------------------------------------*/
class MediaObject
{
	int m_nID;
	Executive *m_pExecutive;
	Presenter *m_pPresenter;
	Root *m_pRoot;
	Route *m_pFirstRoute;
	RenderingObject *m_pRenderingObject;
	MediaObject *m_pClone;
	DWORD m_dwUpdateTime;
	int m_nUse;
	ZArray<MediaObject *> m_parents, m_children;
	ZString m_name;
	int m_nEncodeCounter;

	struct Container
	{	NodeField *m_pContainer;
		BOOL m_bISMF;
		Container (NodeField *pContainer = NULL, BOOL bISMF = FALSE) :
			m_pContainer (pContainer),
			m_bISMF (bISMF)
		{}
		BOOL operator == (NodeField *pContainer) const
		{
			return pContainer == m_pContainer;
		}
		operator NodeField * () const {return m_pContainer;}
	};
	ZArray<Container> m_containers;

private:
	const BifsFieldTable * GetBifsDefEntry (int nID);
	const BifsFieldTable * GetBifsInEntry (int nID);
	int GetBifsOutEntry (int nID);
	const BifsFieldTable * GetBifsDynEntry (int nID);
	virtual const BifsBounds *GetBifsBounds () const = 0;
	virtual int GetChildrenOffset () const = 0;
	void ParseISref (BIFSDecoder *pCoder, int nField);
	bool IsProtoCode () const;

protected:
	virtual int GetFieldCount () const = 0;
	static int FindChildrenOffset (
		const BifsFieldTable *pFieldTable,
		int nFieldCount
	);

public:
	DWORD GetClock () const;
	void ParseNameID (BIFSDecoder *pBifs);
	NodeField *GetField (int nIndex); // MBS: added to access BIFS fields

#ifdef MPGJ		// SCENE API Integration
	int GetNumFields (); //--awm 991209 added for MPEGJ node creation to iterate over fields
#endif // MPGJ

	NodeField *GetField (const BifsFieldTable *pEntry);

#ifdef MPGJ		// SCENE API Integration
	NodeField *GetEventIn( int inID ) { return GetField(GetBifsInEntry(inID)); }
	NodeField *GetEventOut( int outID ) { return GetField(GetBifsOutEntry(outID)); }
	void InsertRouteTo( int outID, MediaObject *toNode, int toID );
	void DeleteRouteTo( int outID, MediaObject *toNode, int toID );
#endif // MPGJ
    int GetFieldID (const char * fieldName); // MP added for script purposes
	int GetID () {return m_nID;}
	LPCSTR GetNameID () const;
	MFGenericNode *GetChildrenField () const;
	virtual MediaObject *GetRenderingNode () {return this;}
	virtual int GetProtoID () const {return -1;}

	//public for access by Script engine
	virtual const BifsFieldTable *GetFieldTable () const = 0;

#ifdef BIFSDEC
public:
	//public for access by UI decoder, virtual for StreamConsumer overload
	virtual void UpdateField (const BifsFieldTable *pEntry, BOOL bHasChanged = TRUE);


private:
	NodeField *ParseField (BIFSDecoder *pBifs, const BifsFieldTable *pEntry, BOOL *bHasChanged = NULL);

protected:
	BOOL m_bIsRoot;
	bool m_bIsBeingParsed;

	void OnBBoxUpdate ();
	void SetCloneObject (MediaObject *pClone) {m_pClone = pClone;}

public:
	bool IsBeingParsed () const;
	int GetEncodeCounter () const {return m_nEncodeCounter;}
	void GenerateIndexedValueDeletion (int nField, int nIndex, BifsGenerator *pBifs);
	void GenerateIndexedValueReplacement (
		int nField,
		NodeField *pField,
		int nIndex,
		BifsGenerator *pBifs
	);
	void GenerateFieldReplacement (int nField, NodeField *pField, BifsGenerator *pBifs);
	void GenerateInsertionCommand (
		int nField,
		NodeField *pField,
		int nIndex,
		BifsGenerator *pBifs
	);
	void GenerateNodeReplacement (MediaObject *pNode, BifsGenerator *pBifs);
	void GenerateRouteOutField (int outID, BifsGenerator *pBifs);
	void GenerateRouteInField (int index, BifsGenerator *pBifs);
	void GenerateNode (BifsGenerator *pBifs);

	void Remove ();
	virtual void RemoveFields ();
	virtual MediaObject *Clone (Root *pScene);
	Presenter *GetPresenter () {return m_pPresenter;}
	BOOL IsRoot () {return m_bIsRoot;}
	/*if bExtDelete is set, the corresponding entry in the "order" field of a parent orderedGroup is removed */
	void Delete(BOOL bExtDelete = FALSE);
	void ReplaceNode (BIFSDecoder *pBifs);
	Executive *GetExecutive () {return m_pExecutive;}
	virtual void UpdateBBox () {}
	virtual BOOL BindRoute (Route *pRoute,  BIFSDecoder *pBifs);	// SCRIPT: changed to virtual
	virtual BOOL AddRoute (int nRouteID,  BIFSDecoder *pBifs, LPCSTR name);	// SCRIPT: changed to virtual
	BOOL ReplaceIndexedValue (BIFSDecoder *pBifsCoder);
	BOOL MultipleReplaceIndexedValue (BIFSDecoder *pBifsCoder);
	BOOL ReplaceValue (BIFSDecoder *pBifsCoder);
	BOOL DeleteValue (BIFSDecoder *pBifs);
	BOOL InsertValue (BIFSDecoder *pBifs);
	BOOL ParseChild (BIFSDecoder *pBifs);
	MediaObject ();
	virtual ~MediaObject ();
	virtual void DeleteObject ();
	void SetExecutive (Executive *pExecutive);
	void SetRoot (Root *pRoot);
	void SetID (int nID);
	virtual BOOL Parse (BIFSDecoder *pBifs);
	virtual BOOL Render (PresenterContext *) {return TRUE;}
	Root *GetRoot () const {return m_pRoot;}
	void LinkRoute (Route *pRoute);
	void UnlinkRoute (Route *pRoute);
	virtual void RouteEvent (NodeField *pField, int nFieldIndex);	// SCRIPT: changed to virtual
	void SetUpdateTime ();
	DWORD GetUpdateTime () {return m_dwUpdateTime;}
	MediaObject *GetClone () const {return m_pClone;}
	
	virtual void OnUpdate ();

	virtual BOOL Activate () {return FALSE;}
	void SetParent (MediaObject *pParent)
	{
		if (pParent)
		{	m_parents += pParent;
			pParent -> SetChild (this);
		}
	}
	MediaObject *GetParent(int index = 0) const
	{
		return index < m_parents.GetLength () ? m_parents [index] : NULL;
	}
	void RemoveParent (MediaObject *pParent)
	{
		if (pParent)
		{	m_parents -= pParent;
			pParent -> RemoveChild (this);
		}
	}
	int GetParentsCount () const {return m_parents.GetLength();};  // added by SB
	void SetChild (MediaObject *pChild)
	{
		if (pChild)
			m_children += pChild;
	}
	MediaObject *GetChild (int index = 0) const
	{
		return index < m_children.GetLength () ? m_children [index] : NULL;
	}
	void RemoveChild (MediaObject *pChild)
	{
		if (pChild)
			m_children -= pChild;
	}
	int GetChildrenCount () const {return m_children.GetLength();};  // added by SB
	void SetContainer (
		NodeField *pContainer,
		BOOL bISMF = FALSE,
		MediaObject *pParent = NULL
	);
	void RemoveContainer (NodeField *pContainer, MediaObject *pParent = NULL);
	virtual RenderingObject *CreateRenderingObject (MediaObject *)
	{
		return NULL;
	}
	RenderingObject *GetRenderingObject () const {return m_pRenderingObject;}
	void SetRenderingObject (RenderingObject *pRenderingObject)
	{
		m_pRenderingObject = pRenderingObject;
	}
#endif BIFSDEC

	//JLF 04/03: node name and field names are ALWAYS define (even without TRACE_ENCODING) for script
public:
	virtual char *GetName () {return NULL;}

#ifdef ANIMATION
private:
	/* MBS: for animation streams: this array sets which fields in a node are animated */
	ZArray<BOOL> animatedFields;

public:
	// MBS: parses animation mask for a node
	virtual void parseAnimationMask(AnimationStreamDecoder *pAS);

	// MBS: parses animation frame and updates fields accordingly
	virtual void animate(AnimationStreamDecoder *pAS,BOOL intra);
#endif // ANIMATION

#ifdef PLAYER
	/*-------------------------------------------------------------
	Methods and fields for grouping objects. Required by the 3D renderer
	-------------------------------------------------------------*/
public:
	int GetCurrentElement () const {return m_currentElement;}
	void SetCurrentElement (int iCurrent) {m_currentElement = iCurrent;}
	float GetDrawingOrder() const {return drawingOrder;}
	void SetDrawingOrder (float fDrawingOrder) {drawingOrder = fDrawingOrder;}
	int GetChildCount () const;
	MediaObject *GetFirstChild ();
	MediaObject *GetNextSibling ();
	bool AddChild (MediaObject * pObject);
	bool DeleteChild (MediaObject * pObject);

private:
	int m_currentElement;
	float drawingOrder;

	CUSTOMIZE_GROUPNODE_DECLARATION
#endif

        // MBS: moved this method here since we need it at BifsEnc & Player to check code of specific nodes (QP, FBA, Mesh ...)
public: virtual int GetNodeCode () const = 0;

#ifdef BIFSENC

//AC 200003 Script
private:
	ZString m_defName;
public:
	void setDefName(LPCSTR name){ m_defName=ZString(name); }
	LPCSTR getDefName() { return (LPCSTR)m_defName; }
//AC end
public:
	virtual ~MediaObject () {}
	void ParseField (BIFSEncoder *pBifs);
	virtual BOOL Encode (BIFSEncoder *pBifs);	// SCRIPT: changed to virtual
	virtual BOOL EncodeField (BIFSEncoder *pBifs, const BifsFieldTable *pEntry); //BF changed to virtual
	virtual void EncodeFieldElement (
		BIFSEncoder *pBifs,
		const BifsFieldTable *pEntry
	);

	void WriteINindex(BIFSEncoder *pBifs, const BifsFieldTable *pEntry);

	virtual void EncodeRoute (BIFSEncoder *pBifs, BOOL bIn);	// SCRIPT: changed to virtual
	void InsertToField (BIFSEncoder *pBifs, BOOL bAppend);
	void IndexedValueDeletion (BIFSEncoder *pBifs, BOOL bLast);
	void FieldReplacement (BIFSEncoder *pBifs, BOOL bLast);
	const BifsFieldTable *GetBifsEntry (LPCSTR szName, int *pFieldID = NULL);
//	friend void CParser :: ObjectMemberAccess (
//		BIFSEncoder *pBifs,
//		int startPos,
//		int opPosition,
//		int endPos
//	); // SCRIPT
	void SetID (int nID) {m_nID = nID;}

private:
	const BifsFieldTable *ParseMFElement (BOOL bLast, int &rIndex);
#endif		// BIFSENC

	CUSTOMIZE_MEDIAOBJECT_DECLARATION
};

#define NULL_OBJECT ((MediaObject *)NULL)

template <class T, int nodeType> class NodeFactory
{
	static int m_nodeType;

public:
	operator int () const {return m_nodeType;}
};

#endif                                      // _MediaObj_h sentry.
