/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Optibase Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Optibase Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1997.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:	Compositor (simulation)
FILE:       MediaSensor.cpp
AUTHORS:    Zvi Lifshitz (Optibase Ltd.)
OVERVIEW:   Implementation of MediaSensorImp - the node that implements MediaSensor
-------------------------------------------------------------------------*/

#include "defs.h"
#include "MediaSensor.h"
#include "Stream.h"
#include "ODImp.h"

/*-------------------------------------------------------------
The Render function examines the status of the media and update the fields accodringly
-------------------------------------------------------------*/
BOOL MediaSensorImp :: Render (PresenterContext *)
{
	/*-------------------------------------------------------------
	Loop through the URLs to see if the current media position is in one of them
	-------------------------------------------------------------*/
	bool bActive = false;
	double dTS = m_dMediaTime;
	DWORD dwTime;

	//if OD is not opened do nothing (OD duration is not valid if not opened)
	if (!GetOD()->IsOpen()) return TRUE;

	//if URL list not loaded do it
	if (m_bNeedsSegmentList) 
		if (!NextURL()) return TRUE;

	
	if (GetStream () != NULL) { 
		GetMediaTime(dwTime);
		/*do not update if empty*/
		if (dwTime) {
			dTS = dwTime / (double) 1000 + GetStream () ->GetClockReference () ->GetMediaTimeOffset ();

			BOOL bRun = TRUE;
			do
			{	/*-------------------------------------------------------------
				If media time precedes the end of previous segment restart from first URL
				-------------------------------------------------------------*/
				if (dTS < m_dEndOfLastSegment)
				{	ResetURL ();
					m_dEndOfLastSegment = 0;
				}
				/*-------------------------------------------------------------
				If media time precedes current segment go out of the loop
				-------------------------------------------------------------*/
				if (dTS < GetSegmentStart ())
					break;
				/*-------------------------------------------------------------
				If media time is within the segment go out of the loop with bActive = true
				-------------------------------------------------------------*/
				if (GetSegmentStart () < 0 || GetSegmentDuration () < 0 ||
					dTS < GetSegmentStart () + GetSegmentDuration ()
				)
				{	bActive = true;
					break;
				}
				/*-------------------------------------------------------------
				Media time exceeds the current segment. So try next URL and keep looping
				-------------------------------------------------------------*/
				m_dEndOfLastSegment = GetSegmentStart () + GetSegmentDuration ();
				bRun = NextURL();
			}
			while (bRun);

			//if we get here we assume we can reset (no more URLs)- this is needed because closeStream 
			//doesn't propagate to all nodes using the OD.
			if (!bRun) {
				ResetURL();
			}
		}
	}
		
	/*-------------------------------------------------------------
	Update node fields
	-------------------------------------------------------------*/
	LPCSTR szSegment;
	if (bActive != m_bActive) {
		isActive = m_bActive = bActive;

		//activate
		if (bActive) {
			double dur = GetSegmentDuration();
			if (dur==-1) 
				dur = GetOD()->GetDuration();
			if ((dur != -1) && (mediaDuration != dur)) mediaDuration = dur;
		}
	}
	//this means we've restarted the stream, reset
	if (dTS < m_dMediaTime) {
		ResetURL();
		m_dMediaTime = -1;
		m_dEndOfLastSegment = 0;
		m_dSegmentEnd = 0;
	}

	if (bActive) {
		if (dTS != m_dMediaTime)
			mediaCurrentTime = m_dMediaTime = dTS;

		if (m_dSegmentEnd >= 0 && (dTS < m_dSegmentStart || dTS >= m_dSegmentEnd)) {
			if (FindSegment (dTS, m_dSegmentStart, m_dSegmentEnd, szSegment)) {
				info.SetAt (0, szSegment);
				double dur = GetSegmentDuration();
				if (dur != -1) mediaDuration = dur;
				streamObjectStartTime = GetSegmentStart();
			}
		}
	}
	return true;
}
