/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1997.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/
//$Log: NodeFld.cpp,v $
//Revision 1.2  2003/12/25 14:54:36  iasalomi
//Update of the CVS repository for AFX Tools
//
//Revision 1.1.1.1  2003/12/24 00:42:11  mikeseven
//Initial release of the CVS repository for MPEG-4 Systems code
//after the code was corrupted due to reorganization and human 
//errors.
//
//Revision 1.13  2003/10/02 19:51:18  mikeseven
//node coding tables update per MPEG 65 (July 2003) meeting
//
//Revision 1.12  2003/04/14 16:47:48  jean_lf
//MediaControl cleanup
//
//Revision 1.11  2003/04/03 04:29:02  mikeseven
//Initial release of Player3D and MPEG-J for all MPEG-4 reference softwares players with samples.
//For installation, follow instructions in docs/Player3D_setup.doc.
//
//Many thanks to Prashanth Gujjeti and Viswanathan Swaminathan from Sun Microsystems.
//
//-- Mikael Bourges-Sevenier (Mindego)
//
//Revision 1.10  2003/01/23 09:57:12  jean_lf
//no message
//
//Revision 1.9  2003/01/20 17:58:39  jean_lf
//no message
//
//Revision 1.8  2003/01/13 11:30:03  sowoo
//no message
//
//Revision 1.7  2002/12/11 22:46:04  iverg
//no message
//
//Revision 1.6  2002/12/11 22:33:48  iverg
//added parts of the MU node decoding (not complete!!)
//
//Revision 1.5  2002/11/13 09:36:31  jean_lf
//fixed QP used in Proto
//
//Revision 1.4  2002/10/30 12:27:59  jean_lf
//Diverse Proto fixes
//
//Revision 1.3  2002/10/25 09:52:42  jean_lf
//no message
//
//Revision 1.2  2002/08/08 09:47:27  zvil
//Core 5.9 Release Candidate 9:
//- Handling of time fields
//- Generating BIFS from memory
//- Proper handling of BIFS tables
//- More IPMPX stuff
//- Fixes to IM1-2D for proper use of startTime / stopTime
//- Fixes to Im1Player for proper use of startTime / stopTime
//
//
//Revision 1.x 2000/10/12  Pascal Belin
//QP bug fix
//
//Revision 1.2  2000/03/03 18:41:21  Alex Cotarmanac'h
//Version 1 -- Alexandre Cotarmanac_h
//
//Modified 2001/05/08 Boris Felts boris.felts@polytechnique.org
//
/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  BIFS Decoder/ Compositor
FILE:       NodeFld.cpp
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Implementation of NodeField and derivations.
-------------------------------------------------------------------------*/
#include "defs.h"
#include "Nodes.h"
#include "NodeFld.h"
#include "MediaObj.h"
#include "BifsDec.h"
#include "Route.h"
#include "Root.h"
#include "Quantize.h"
#include "NodeCode.h"
#include "Proto.h"
#include "Inline.h"
#include "ScriptImp.h"
#include "PresBase.h"

//#if PREDICTIVE
#ifdef PREDICTIVE // SOLARIS PORTING - made ifdef
#include "Predictive.h"//AC: do not forget to add Predictive/Decoder to include path of this file
#endif


// SOLARIS PORTING - Commenting the stuff below by #ifdef
#ifdef BIFSDEC
//JLF: value returned in case parsing doesn't modify the SF field
//if FALSE this will prevent the field from modifying its update time
//while still allowing for routes cascades
#define RETURN_ON_SAME_VALUE	FALSE


/*-----------------------------------------------------------------
NodeField destructor. Delete all routes
-----------------------------------------------------------------*/
NodeField :: ~NodeField ()
{
	while (m_pFirstRoute)
		m_pFirstRoute ->Remove ();  // this unlinks the route automatically
#ifdef MPGJ		// SCENE API Integration
	delete m_listeners;
#endif MPGJ		// SCENE API Integration
}

/*-----------------------------------------------------------------
Add a Route to the nodeField
-----------------------------------------------------------------*/
void NodeField :: LinkRoute (Route *pRoute)
{
	pRoute -> Link (m_pFirstRoute);
}

/*-----------------------------------------------------------------
Unlink a Route from the list of routes
-----------------------------------------------------------------*/
void NodeField :: UnlinkRoute (Route *pRoute)
{
	pRoute -> Unlink (m_pFirstRoute);
}

/*-----------------------------------------------------------------
Activate attached routes on nodeField update
The paramter bFirst is used to eliminate loops.
-----------------------------------------------------------------*/
void NodeField :: OnFieldUpdate (bool bIsFirst)
{
	for (Route *pRoute = m_pFirstRoute;
		pRoute != NULL;
		pRoute = pRoute -> GetNext ()
	)
		pRoute -> MarkForActivation (bIsFirst);
#ifdef MPGJ		// SCENE API Integration
	if ( m_listeners )
	{
		int listenerCount = m_listeners->GetLength();
		for ( int i = 0; i < listenerCount; i++ )
		{
			Listener *l = m_listeners->ElementAt( i );
			l->notify( this );
		}
	}
  m_bFieldUpdated = true;
}

/*-----------------------------------------------------------------
Add an event out listener for this field
-----------------------------------------------------------------*/
void NodeField::AddListener( Listener *l )
{
	if ( m_listeners == 0 )
	{
		m_listeners = new ZArray<Listener*>;
	}
	*m_listeners += l;
}

/*-----------------------------------------------------------------
Remove an event out listener for this field
-----------------------------------------------------------------*/
void NodeField::RemoveListener( Listener *l )
{
	if ( m_listeners != 0 )
	{
		int listenerCount = m_listeners->GetLength();
		for ( int i = 0; i < listenerCount; i++ )
		{
			Listener *a = m_listeners->ElementAt( i );
			if ( l == a )
			{
				m_listeners->DeleteFrom( i );
				break;
			}
		}
	}
#endif MPGJ		// SCENE API Integration
}

/*-----------------------------------------------------------------
Parse an SFInt
-----------------------------------------------------------------*/
BOOL SF<int> :: Parse (BIFSDecoder *pCoder, MediaObject*, BOOL *bHasChanged, Quantizer *pQuant)
{
	int val;
	if(!pQuant || !pQuant -> IsActive ())
		val = pCoder -> ParseInt (32, "intValue");
	else
		val = pQuant -> ParseInt (pCoder);

	TRACEF ("%d", val);
	if (bHasChanged) *bHasChanged = (val==m_value) ? RETURN_ON_SAME_VALUE : TRUE;
	m_value = val;
	return TRUE;
}

/*-----------------------------------------------------------------
Parse an SFBool
-----------------------------------------------------------------*/
BOOL SF<Bool> :: Parse (
	BIFSDecoder *pCoder,
	MediaObject * /*pNode*/,
	BOOL *bHasChanged, 
	Quantizer * /*pQuant*/
)
{
	Bool val = pCoder -> ParseInt (1, "value") != 0; // mp : conformance
	TRACEF ("%s", val ? "TRUE" : "FALSE");
	if (bHasChanged) *bHasChanged = (val==m_value) ? RETURN_ON_SAME_VALUE : TRUE;
	m_value = val;
	return TRUE;
}

/*-----------------------------------------------------------------
Parse an SFFloat
MBS: modified to take into account quantization parameters if any
-----------------------------------------------------------------*/
BOOL SF<float> :: Parse (
	BIFSDecoder *pCoder,
	MediaObject * /*pNode*/,
	BOOL *bHasChanged, 
	Quantizer * pQuant
)
{
	float val;
	if(!pQuant || !pQuant -> IsActive ())
		val = pCoder -> ParseFloat ("value"); // mp conformance
	else
		val = pQuant->ParseFloat(pCoder);

	TRACEF ("%.2f", val);
	if (bHasChanged) *bHasChanged = (val==m_value) ? RETURN_ON_SAME_VALUE : TRUE;
	m_value = val;
	return TRUE;
}

/*-----------------------------------------------------------------
Parse an SFTime
-----------------------------------------------------------------*/
BOOL SF<Time> :: Parse (
	BIFSDecoder *pCoder,
	MediaObject * /*pNode*/,
	BOOL *bHasChanged, 
	Quantizer * /*pQuant*/
)
{
	Time val = pCoder -> ParseDouble("value");
	TRACEF ("%.2lf", val);
	if (bHasChanged) *bHasChanged = (val==m_value) ? RETURN_ON_SAME_VALUE : TRUE;
	m_value = val;
	return TRUE;
}

/*-------------------------------------------------------------
Parse startTime/stopTime field. It's the usual parse only that m_bOffset is reset to
false so that subsequence read from this field will add the time offset
-------------------------------------------------------------*/
BOOL SFTimeField :: Parse (
	BIFSDecoder *pCoder,
	MediaObject *pNode,
	BOOL *bHasChanged, 
	Quantizer *pQuant
)
{
	SF<Time> :: Parse (pCoder, pNode, bHasChanged, pQuant);
	m_bOffset = 1;
	//prevent route execution until offset is applied
	if (bHasChanged) *bHasChanged = FALSE;
	return TRUE;
}

/*-----------------------------------------------------------------
Parse an SFVec3f
MBS: modified to take into account quantization parameters if any
-----------------------------------------------------------------*/
BOOL SF<Vec3f> :: Parse (
	BIFSDecoder *pCoder,
	MediaObject * /*pNode*/,
	BOOL *bHasChanged, 
	Quantizer * pQuant
)
{
	Vec3f val;

	if(!pQuant || !pQuant->IsActive()) {
		val.x = pCoder -> ParseFloat("float"); //BF Conformance
		val.y = pCoder -> ParseFloat("float"); //BF Conformance
		val.z = pCoder -> ParseFloat("float"); //BF Conformance
	} else {
		val = pQuant -> ParseVec3f(pCoder);
	}
	TRACEF3 ("%.2f %.2f %.2f", val.x, val.y, val.z);
	if (bHasChanged) *bHasChanged = (val==m_value) ? RETURN_ON_SAME_VALUE : TRUE;
	m_value = val;
	return TRUE;
}

/*-----------------------------------------------------------------
Parse an SFVec2f
MBS: modified to take into account quantization parameters if any
-----------------------------------------------------------------*/
BOOL SF<Vec2f> :: Parse (
	BIFSDecoder *pCoder,
	MediaObject * /*pNode*/,
	BOOL *bHasChanged, 
	Quantizer * pQuant
)
{
	Vec2f val;

	if(!pQuant || !pQuant->IsActive())
	{	val.x = pCoder -> ParseFloat("float");  //BF Conformance
		val.y = pCoder -> ParseFloat("float");  //BF Conformance
	}
	else
		val = pQuant -> ParseVec2f(pCoder);
	
	TRACEF2 ("%.2f %.2f", val.x, val.y);
	if (bHasChanged) *bHasChanged = (val==m_value) ? RETURN_ON_SAME_VALUE : TRUE;
	m_value = val;
	return TRUE;
}

/*-----------------------------------------------------------------
Parse an SFRotation
MBS: modified to take into account quantization parameters if any
-----------------------------------------------------------------*/
BOOL SF<Rotation> :: Parse (
	BIFSDecoder *pCoder,
	MediaObject * /*pNode*/,
	BOOL *bHasChanged, 
	Quantizer * pQuant
)
{
	Rotation val;
	if(!pQuant || !pQuant->IsActive())
	{	val.x = pCoder -> ParseFloat("float");    //BF Conformance
		val.y = pCoder -> ParseFloat("float");    //BF Conformance
		val.z = pCoder -> ParseFloat("float");    //BF Conformance
		val.theta = pCoder -> ParseFloat("float");//BF Conformance
	}
	else
		val = pQuant -> ParseRotation(pCoder);

	TRACEF4 ("%.2f %.2f %.2f %.2f", val.x, val.y, val.z, val.theta);
	if (bHasChanged) *bHasChanged = (val==m_value) ? RETURN_ON_SAME_VALUE : TRUE;
	m_value = val;
	return TRUE;
}

/*-----------------------------------------------------------------
Parse an SFRotation2D
MBS: modified to take into account quantization parameters if any
-----------------------------------------------------------------*/
BOOL SF<Rotation2D> :: Parse (
	BIFSDecoder *pCoder,
	MediaObject * /*pNode*/,
	BOOL *bHasChanged, 
	Quantizer * pQuant
)
{
	Rotation2D val;
	
	if(!pQuant || !pQuant->IsActive()) {
		val.x = pCoder -> ParseFloat("float");  //BF Conformance
		val.y = pCoder -> ParseFloat("float");  //BF Conformance
		val.theta = pCoder -> ParseFloat("float");//BF Conformance
	} else
		val = pQuant -> ParseRotation2D(pCoder);

	TRACEF3 ("%2.f %.2f %.2f", val.x, val.y, val.theta);
	if (bHasChanged) *bHasChanged = (val==m_value) ? RETURN_ON_SAME_VALUE : TRUE;
	m_value = val;
	return TRUE;
}

/*-----------------------------------------------------------------
Parse an SFColor
MBS: modified to take into account quantization parameters if any
-----------------------------------------------------------------*/
BOOL SF<Color3f> :: Parse (
	BIFSDecoder *pCoder,
	MediaObject * /*pNode*/,
	BOOL *bHasChanged, 
	Quantizer * pQuant
)
{
	Color3f val;
	if(!pQuant || !pQuant->IsActive())
	{	val.r = pCoder -> ParseFloat("float");  //BF Conformance
		val.g = pCoder -> ParseFloat("float");  //BF Conformance
		val.b = pCoder -> ParseFloat("float");  //BF Conformance
	}
	else
		val = pQuant -> ParseColor3f(pCoder);

	TRACEF3 ("%.2f %.2f %.2f", val.r, val.g, val.b);
	if (bHasChanged) *bHasChanged = (val==m_value) ? RETURN_ON_SAME_VALUE : TRUE;
	m_value = val;
	return TRUE;
}




/*-----------------------------------------------------------------
Parse an SFObjectID
-----------------------------------------------------------------*/
BOOL SF<Url> :: Parse (
	BIFSDecoder *pCoder,
	MediaObject * pNode,
	BOOL *bHasChanged, 
	Quantizer * /*pQuant*/
)
{

	if (pCoder -> ParseInt (1, "isOD"))
	{	m_value.ODid = pCoder -> ParseInt (10, "ODid");
		TRACEF ("%d", m_value.ODid);
	}
	else
	{	m_value.ODid = -1;
		m_value.urlValue.Parse (pCoder,pNode, NULL); //BF Script 
	}
	if (bHasChanged) *bHasChanged = TRUE;
	return TRUE;
}

/*-----------------------------------------------------------------
Parse SFString
MBS 110698: updated for UTF-8 String
Zvi 021799: redo MBS change. The string length is given in bytes, not
in characters. The handling of UTF-8 coding shall be done by the compositor -
it is not the matter of the parser.
-----------------------------------------------------------------*/
BOOL SF<ZString> :: Parse (
	BIFSDecoder *pCoder,
	MediaObject * pNode,  
	BOOL *bHasChanged, 
	Quantizer * /*pQuant*/
)
{
	//note: because of script/conditional we don't check updates on ZString
	if (bHasChanged) *bHasChanged = TRUE;

	//if this is the first ZString in a script node then this is a SFScript 
	if ( (pNode->GetNodeCode()== CODE_Script) && !((ScriptImp *)pNode)->InScriptParsing() )
	{
		BOOL ret = ((ScriptImp *)pNode)->ParseSFScript(pCoder);
		m_value = ((ScriptImp *)pNode)->GetScriptText();
		TRACEF ("\"%s\"", (LPCSTR) m_value);
		return ret;
	} 
	pCoder ->ParseSFString (m_value);

	// SOWOO 20030109: Trace "buffer" field of BitWrapper node
	// As this "buffer" field contains non-printable character,
	// the following routine performs to convert these characters to pritable ones.
	if(pCoder->GetFlags() & STRING_FROM_FILE)
	{
		ZString string;
		char character[3];
		for(int i=0; i<m_value.GetLength(); i++)
		{
			// print out the one non-printable charactor 
			// using two printable character
			::sprintf(character, "%x%x", int(m_value[i] >> 4), int(m_value[i] & 0x000f));
			string += character;
		}

		TRACEF("\"%s\"", (LPCSTR) string);
	}
	else
	// SOWOO end
		TRACEF ("\"%s\"", (LPCSTR) m_value);

	return TRUE;
}

SFString::operator Url() const
{
	struct Url ret;

	LPCSTR string = (LPCSTR) m_value;
	if (!string) return ret;

// SOLARIS PORTING - problem with const char *
	char *tmp = strstr((LPSTR)string, "od:");
	if (tmp) {
		tmp += 3;
		ret.ODid = atoi(tmp);
		ret.urlValue.m_value = "";
		return ret;
	}
	//this is not really safe
	if (isdigit(string[0]) ) {
		ret.ODid = atoi(string);
		return ret;
	} 
	ret.urlValue.m_value = string;
	return ret;
}

/*-----------------------------------------------------------------
Parse SFImage
MBS 110498: this is a new type
BF  310401: fix the coding and the output
-----------------------------------------------------------------*/
BOOL SF<Image> :: Parse (
	BIFSDecoder *pCoder,
	MediaObject * /*pNode*/,
	BOOL *bHasChanged, 
	Quantizer * /*pQuant*/
)
{
	m_value.width = pCoder -> ParseInt (12, "width");
	m_value.height = pCoder -> ParseInt (12, "height");
	m_value.numComponents = pCoder -> ParseInt (2, "numComponents");
	
	if (bHasChanged) *bHasChanged = TRUE; 

	TRACEF3 (" %d %d %d", m_value.width, m_value.height, m_value.numComponents + 1);  // BF 010209: write pixel values
	TRACENEWLINE;
	//Alvar 981223: Added pixel array allocation and refcount stuff
	int nPixels = (m_value.numComponents + 1)* m_value.width * m_value.height;             // BF 010209: SFimage fix according to 9.3.7.23.1
	BYTE *p = m_value.pixels = new BYTE [nPixels];
	p;
	m_value.m_pRefCnt = new int;
	*(m_value.m_pRefCnt) = 1;
	//JLF: fix code for TRACE_ENCODING undef ..
	for (int i=0; i<nPixels; i++) {
		*p= pCoder -> ParseInt (8, "pixels");
		TRACEF("0x%x ", *p);
		p++;
	}
	TRACENEWLINE;

	//JLF: no filter on previous value for SFImage
	return TRUE;
}

/*-----------------------------------------------------------------
SFGenericNode Destructor.  Deletes the MediaObject, unless it is defined
by other node and only USEd here.
-----------------------------------------------------------------*/
void SFGenericNode :: Remove ()
{
	if (m_pNode)
	{	m_pNode -> RemoveContainer (this);
		m_pNode = NULL;
	}
}

/*-----------------------------------------------------------------
Assign a value into a generic-node. If the old value is not a copy
of another nodeField, delete the old node
-----------------------------------------------------------------*/
MediaObject *SFGenericNode :: SetObject (MediaObject *node, MediaObject *pParent, bool bIsFirst)
{
	if (m_pNode)
		m_pNode -> RemoveContainer (this);
	m_pNode = node;
	if (node)
		node -> SetContainer (this, false, pParent);
	OnFieldUpdate (bIsFirst);
	return node;
}

/*-----------------------------------------------------------------
Parse a node field which is a sub-node
-----------------------------------------------------------------*/
BOOL SFGenericNode :: Parse (
	BIFSDecoder *pCoder,
	MediaObject *pNode,
	BOOL *bHasChanged, 
	Quantizer * /*pQuant*/
)
{
	if (bHasChanged) *bHasChanged = TRUE;
	m_pParent = pNode;
	return ParseNodeField (pCoder, pNode, this, FALSE);
}

/*-------------------------------------------------------------
This function does the actual work of node parsing when the node
is either a scalar field in a parent node (in which case the function
is called from SFGenericNode :: Parse), an element in a vector field,
or when it is parsed in ReplaceNode command. The later two cases ParseNode
is called directly, not through Parse().
-------------------------------------------------------------*/
BOOL SFGenericNode :: ParseNodeField (
	BIFSDecoder *pCoder,
	MediaObject *pNode,			// the node this field belongs to
	NodeField *pContainer,		// a pointer to the container field
								// (this is not always 'this' because
								// a temporary NodeField object can be
								// used to parse the node, and then the
								// node is assigned or inserted in another
								// NodeField object.
	BOOL bISMF					// TRUE when the field is an MF field
)
{
//#ifdef PREDICTIVE	//AC 20000227 Quantization Scheme 14 Stuff  //BF
	bool resetWhenFinished=false;
	bool recordData=false;
//#endif	//AC end  //BF
	bool isUse=false;
	int nID=-1;

	MediaObject *pProtoQP = NULL;

	/*-------------------------------------------------------------
	m_pNode may already point to another node if the function is called
	during a ReplaceField command.
	-------------------------------------------------------------*/
	if (m_pNode)
	{	m_pNode -> RemoveContainer (pContainer, pNode);
		m_pNode = NULL;
	}

	if (pCoder -> ParseInt (1, "isReused"))	// if isReused
	{	nID = pCoder -> ParseInt (pCoder -> GetNodeIDbits (), "nodeID"); // mp : conformance

		/*-------------------------------------------------------------
		If the node ID is all ones, this actually means "null node"
		-------------------------------------------------------------*/
		if (nID == (1 << pCoder -> GetNodeIDbits ()) - 1)
		{	TRACEF ("NULL", 0);
			return true;
		}

		if (pNode && (m_pNode = pNode -> GetRoot () -> FindNode (nID)) != NULL ) {
			m_pNode -> SetContainer (pContainer, bISMF, pNode);
		} else
			return FALSE;

		/*if QP push it*/
		if(m_pNode->GetNodeCode() == QuantizationParameter :: GetCode())
		{
			pCoder->SetCurrentQP(m_pNode);
		}

		TRACEF ("USE %s", m_pNode ->GetNameID ());
		isUse=true; //AC 20000227
	}
	else
	{
		/*-----------------------------------------------------------------
		Get object type, instantiate the object, and let it parse itself
		-----------------------------------------------------------------*/
		nID = -1;
		int nNodeType = 0;
		int nProtoID = -1;
		char szTag [] = "localNodeType\0\0";

		for (int nVer = 1; nVer <= NodeField :: BIFS_VERSIONS; nVer++)
		{	int nVersionTableSize = GetNodeVersionSize (nVer);
			int nFieldWidth = BitFieldWidth (nVersionTableSize + 1 + (nVer == 2));
			int n = pCoder ->ParseInt (nFieldWidth, szTag);
			if (n == 0)
			{	nNodeType += nVersionTableSize;
				szTag [13] = 'V';
				szTag [14] = '1' + nVer;
			}
			else if (n == 1 && nVer == 2)
			{	nProtoID = pCoder -> ParseInt (pCoder -> GetProtoIDbits (), "PROTOnodeType");
				break;
			}
			else
			{	nNodeType += n - (nVer == 2);
				break;
			}
		}

		/*-----------------------------------------------------------------
		The root of the new object is the same as of the container
		object. If the container object is itslef a root, the root is
		the container object.
		-----------------------------------------------------------------*/
		Root *pRoot = pNode -> GetRoot ();
		if (!pRoot) {
			pRoot = (Root *) pNode;
			pRoot->Configure(FALSE, 0, 0, pCoder->GetNodeIDbits(), pCoder->GetRouteIDbits(), pCoder->GetProtoIDbits());
		}

		/*-------------------------------------------------------------
		Create a new node. If this is a proto, instantiate the proto
		-------------------------------------------------------------*/
		if (nProtoID < 0) {
			m_pNode = CreateNode (nNodeType);
		} else {
			/*-------------------------------------------------------------
			Locate the Proto, and check whether its first node is of appropriate type.
			-------------------------------------------------------------*/
			Proto *pProto = pRoot -> FindProto (nProtoID);
			//if proto not defined in the current root look in the scene root
			if (pProto == NULL)	{
				pProto = pCoder->GetScene()->FindProto(nProtoID);
				if (!pProto) return NULL;
			}

			m_pNode = pProto -> Instantiate (pRoot);
			if (m_pNode == NULL)
				return false;
			MediaObject *pFirst = ((Proto *)m_pNode) ->GetFirstNode ();
			if (pFirst && !IsTypeOK (pFirst))
			{	m_pNode -> Remove ();
				m_pNode = NULL;
			}
			if (pFirst && (pFirst->GetNodeCode() == CODE_QuantizationParameter) ) {
				pProtoQP = pFirst;
			}
		}
		if (!m_pNode)
			return FALSE;

		/*-------------------------------------------------------------
		Parse the DEF id, if any
		-------------------------------------------------------------*/
		if (pCoder -> ParseInt (1, "isUpdateable")) // if isUpdatable  //BF Conformance
		{
			nID = pCoder -> ParseInt (pCoder -> GetNodeIDbits (), "nodeID"); // mp : conformance
			if (pRoot -> GetInline () && pRoot -> GetInline () ->IsCodeNames ())
			{	m_pNode ->ParseNameID (pCoder);
				TRACEF ("DEF %s ", m_pNode ->GetNameID ());
			}
			else
				TRACEF ("DEF ID%d ", nID);//AC20000228 just to get it running in a Vrml Browser
		}

		m_pNode -> SetID (nID);
		m_pNode -> SetContainer (pContainer, bISMF, pNode);
		m_pNode -> SetRoot (pRoot);
		m_pNode -> SetExecutive (pCoder -> GetExecutive ());

		//***********************************
//#ifdef PREDICTIVE	//AC 20000227 QP14 Stuff  //BF

		switch (m_pNode->GetNodeCode())
		{
		case CODE_Coordinate:
		case CODE_Coordinate2D:
			pCoder->setCoordStore();
			if (nID>=0) recordData=true;
			break;
		case CODE_IndexedFaceSet:
		case CODE_IndexedFaceSet2D:
		case CODE_IndexedLineSet:
		case CODE_IndexedLineSet2D:
			pCoder->setCoordActive();
			resetWhenFinished=true;
			break;
		default:
			break;
		}
// #endif	//AC end  //BF

        //IG for MU nodes parse the Proto definitions
		// MBS removed MU nodes per July 2003 resolutions
        /*switch (m_pNode->GetNodeCode()) {
        case CODE_MUSession:
        case CODE_MUAvatar:
        case CODE_MUAvatar2D:
        case CODE_MUZone:
            {*/
                //parse proto definitions, assume empty for now
                /*int hasProtos =*/ //pCoder->ParseInt(1,"Has protos");
            /*}
            break;
        }*/

		/*-------------------------------------------------------------
		Parse the node
		-------------------------------------------------------------*/

		// PB Sept 1. 2000 - The following line used to be after the if
		//  According to the standard, QPs don't quantize themselves, but only their nodes.
		m_pNode -> Parse (pCoder);

		// MBS: sets current QP in decoder if we have a QP node
		if (m_pNode->GetNodeCode() == QuantizationParameter :: GetCode())
		{
			pCoder->SetCurrentQP(m_pNode);
		} else if (pProtoQP) {
			pCoder->SetCurrentQP(pProtoQP);
		}
		// end MBS

	    // PB: Cancelled the change made by BenA in july 99.

		// PB end
        //IG for MU nodes parse the Routes
		// MBS removed MU nodes per July 2003 resolutions
        /*switch (m_pNode->GetNodeCode()) {
        case CODE_MUSession:
        case CODE_MUAvatar:
        case CODE_MUAvatar2D:
        case CODE_MUZone:
            {*/
                //parse proto definitions, assume empty for now
                /*int hasRoutes =*/ //pCoder->ParseInt(1,"Has routes");
            /*}
            break;
        }*/
	}


	//***********************************
// #ifdef PREDICTIVE	//AC 20000227 Quantization Scheme 14  //BF
//	QP14 Use Case
	if (isUse)
	{
		//do we have to retrieve any data ?
		// we try and find when isCoordActive.
		switch (m_pNode->GetNodeCode())
		{
		case CODE_Coordinate:
		case CODE_Coordinate2D:
			pCoder->setCurrentId(nID);
		default:
			break;
		}
	}

	if (recordData)
	{
		if(pCoder->isCoordActive()) // PB put this into comment: assert (used to check this was active)
		if (nID>=0) pCoder->setIdToLength(nID,pCoder->getLengthCoord());
	}
	if (resetWhenFinished) pCoder->resetCoord();

//#endif	//AC end  //BF


	return TRUE;
}

/*-------------------------------------------------------------
Clone a node field when a Proto is instantiated
-------------------------------------------------------------*/
NodeField *SFGenericNode :: Clone (NodeField *pField, MediaObject *pParent)
{
	if (pField == NULL)
		pField = new SFGenericNode;
	if (m_pNode)
	{	((SFGenericNode*) pField) ->m_pNode =  m_pNode -> Clone (pParent -> GetRoot ());
		((SFGenericNode*) pField) -> m_pNode -> SetContainer (pField, false, pParent);
	}
	return pField;
}

/*-----------------------------------------------------------------
Parse an MF nodeField
-----------------------------------------------------------------*/
BOOL MField :: Parse (
	BIFSDecoder *pCoder,
	MediaObject *pNode,
	BOOL *bHasChanged,
	Quantizer *pQuant
)
{
	if (bHasChanged) *bHasChanged = TRUE;
	bool bFirst = true;
	if (pNode == NULL || pNode -> GetNodeCode () != CODE_Script || GetType() != MFSTRING) // mp
	{
	    TRACEF ("[", 0);
    }
	else
	{
        bFirst = false; // mp don't indent the first line
    }

	Remove ();

	/*-----------------------------------------------------------------
	MF nodeField may be coded as ListMFDescription or VectorMFDescription
	-----------------------------------------------------------------*/

	BOOL localQP=FALSE;
	MediaObject *qpOld = NULL;

	MediaObject *qpForThisLevel = pCoder->GetCurrentQP();

#ifdef PREDICTIVE	//AC 19991117
	/*-----------------------------------------------------------------
	MF nodeField may be coded also Predictive Mode
	-----------------------------------------------------------------*/

	// First check whether Predictive Mode is enabled
	if ( pCoder->UsePredictiveMFField() ) {
		if ( pCoder -> ParseInt(1,"usePredictive") )
		{
			if (bFirst) {
				TRACENEWLINE;
				TRACEINDENT (1);
				bFirst=false;
			}
			PredictiveMField a_PMField = PredictiveMField(pCoder, pNode, pQuant, this );
			return a_PMField.Parse ();
		}
	}
	// Disabled
#endif	//AC end

	
	if (!pCoder -> ParseInt (1, "reserved"))		// reserved, added in FDIS
	{	if (pCoder -> ParseInt (1, "isListDescription"))	// isListDescription
		{
			int nElements=0;

			while (pCoder -> ParseInt (1, "endFlag") == 0)	// changed for  FDIS // mp : conformance
			{
				if (bFirst)
				{	bFirst = false;
					TRACENEWLINE;
					TRACEINDENT (1);
				}
				// MBS: saves old QP
				if(!localQP) qpOld=pCoder->GetCurrentQP();
				// end MBS

				nElements++;

				if (!AppendElement (pCoder, pNode, pQuant))
					return FALSE;

				// MBS: restores old QP
				if(pCoder->hasLocalQP()) {
					if(localQP) {
						localQP=FALSE;
						pCoder->SetCurrentQP(qpOld);
					}
					else {
						localQP=TRUE;
					}
				}
				// end MBS

				TRACENEWLINE;
			}
			pCoder->setLengthCoord(nElements);
		}
		else
		{
			int nElements = pCoder -> ParseInt (pCoder -> ParseInt (5, "nbBits"), "numberOfFields"); // mp conformance
			pCoder->setLengthCoord(nElements);
			for (int i = nElements; --i >= 0;)
			{
				if (bFirst)
				{	bFirst = false;
					TRACENEWLINE;
					TRACEINDENT (1);
				}

				// MBS: saves old QP
				if(!localQP) qpOld=pCoder->GetCurrentQP();
				// end MBS

				if (!AppendElement (pCoder, pNode, pQuant))
					return FALSE;

				// MBS: restores old QP

				if(pCoder->hasLocalQP()) {
				  if(localQP) {
					localQP=FALSE;
					pCoder->SetCurrentQP(qpOld);
				  }
				  else {
					localQP=TRUE;
				  }
				}
				// end MBS

				TRACENEWLINE;
			}
		}

		// BenA (7/30/99): restore quantization for this level
		pCoder->SetCurrentQP(qpForThisLevel);
		// end BenA
	}

#ifdef TRACE_ENCODING
	if (!bFirst &&
		(pNode == NULL || pNode -> GetNodeCode () != CODE_Script || GetType () != MFSTRING)
	) // mp
		TRACEINDENT (-1);
	TRACEF ("]", 0);
	TRACENEWLINE;
#endif

	return TRUE;
}

/*-----------------------------------------------------------------
Deletes all nodes
-----------------------------------------------------------------*/
void MFGenericNode :: Remove ()
{
	for (int i = GetLength (); --i >= 0;)
		if (m_array [i])
			m_array [i] -> RemoveContainer (this, m_pParent);
	m_array.Clear ();
}

/*-----------------------------------------------------------------
Manipulation of MFGenericNode arrays
-----------------------------------------------------------------*/
void MFGenericNode :: InsertAt (int index, MediaObject *value, MediaObject *pParent, bool bRoute)
{
	m_array.InsertAt (index, value);
	value ->SetContainer (this, true, pParent);
	if (bRoute)
		OnFieldUpdate ();
}

void MFGenericNode :: Append (MediaObject *value, MediaObject *pParent, bool bRoute)
{
	if (value -> GetNodeCode () != CODE_Script)	// SCRIPT
	{	m_array += value;
		value ->SetContainer (this, true, pParent);
	}
	if (bRoute)
		OnFieldUpdate ();
}

void MFGenericNode :: SetAt (int index, MediaObject *value, MediaObject *pParent, bool bRoute)
{
	if (index < GetLength () && m_array [index] != NULL)
		m_array [index] -> RemoveContainer (this, m_pParent);
	m_array.SetAt (index, value);
	if (value != NULL)
		value ->SetContainer (this, true, pParent);
	if (bRoute)
		OnFieldUpdate ();
}

void MFGenericNode :: DeleteFrom (int index, BOOL bRoute)
{
	if (index < GetLength () && m_array [index] != NULL)
		m_array [index] -> RemoveContainer (this, m_pParent);
	m_array.DeleteFrom (index);
	if (bRoute)
		OnFieldUpdate ();
}

/*-----------------------------------------------------------------
Assigment operator activated in case of Routes. We assume that
the nodeField argument is always of the same type as this, therefor
allow for deliberate downcasting. This assumption is dangerous.
Commercial application should implement safe casting or proper
conversion mechanism.
-----------------------------------------------------------------*/
BOOL MFGenericNode :: Copy (NodeField &nodeField, MediaObject *pParent, bool bIsFirst)
{
	MFGenericNode &fld = (MFGenericNode &) nodeField;
    // PORTING to CC 5.3
    int i = 0;
//	for (int i = GetLength (); --i >= 0;)
	for ( i = GetLength (); --i >= 0;)
		DeleteFrom (i, false);
	for (i = 0; i < fld.GetLength (); i++)
	{	m_array += fld.m_array [i];
		fld.m_array [i] -> SetContainer (this, true, pParent);
	}
	OnFieldUpdate (bIsFirst);
	return TRUE;
}

/*-------------------------------------------------------------
The node in this field has been replaced by ReplaceNode command
-------------------------------------------------------------*/
void SFGenericNode :: ReplaceNode (MediaObject *pNew)
{
	m_pNode = pNew;
	pNew -> SetContainer (this, FALSE, m_pParent);
}

/*-------------------------------------------------------------
Node pOld in this MFxxxNode field has been replaced by a
ReplaceNode command.
-------------------------------------------------------------*/
void MFGenericNode :: ReplaceNode (MediaObject *pOld, MediaObject *pNew)
{
	for (int i = m_array.GetLength (); --i >= 0; i)
		if (m_array [i] == pOld)
		{	m_array [i] = pNew;
			if (pNew) pNew -> SetContainer (this, TRUE, m_pParent);
			return;
		}
}

/*-------------------------------------------------------------
Clone a vector node field when a Proto is instantiated
-------------------------------------------------------------*/
NodeField *MFGenericNode :: Clone (NodeField *pField, MediaObject *pParent)
{
	if (pField == NULL)
		pField = new MFGenericNode;
	for (int i = 0; i < m_array.GetLength (); i++)
	{	if (m_array [i])
		{	MediaObject *pNode = m_array [i] -> Clone (pParent -> GetRoot ());
			((MFGenericNode *) pField) -> m_array += pNode;
			pNode -> SetContainer (pField, true, pParent);
		}
	}
	return pField;
}

/*-------------------------------------------------------------
When a time field value is first read the current time shall be added as offset to its value
-------------------------------------------------------------*/
void SFTimeField :: ApplyOffset (const PresenterContext *pContext)
{
	if (m_bOffset==1) {
		m_bOffset = 2;
		//if value is -1 do not apply offset (this is a "disable" value for all time fields)
		if ((m_value!= -1) && pContext) m_value += pContext -> GetTime () / 1000.;
		//force route updates now that field is updated
		OnFieldUpdate();
	}
}

/*-------------------------------------------------------------
When a time field is written not by parsing set its offset flag to indicate value is already
offset
-------------------------------------------------------------*/
Time SFTimeField :: operator = (const Time value)
{
	m_bOffset += 1;
	return SF<Time> :: operator = (value);
}

/*-------------------------------------------------------------
When a time field gets its value from a Route set its offeset flag to indicate
no offset necessary
-------------------------------------------------------------*/
BOOL SFTimeField :: Copy (NodeField &nodeField, MediaObject *pParent, bool bIsFirst)
{
	BOOL ret;
	if (pParent && !pParent ->IsBeingParsed() ) {
		m_bOffset = 2;
	} else {
		m_bOffset = 1;
	}
	ret = SF<Time> :: Copy (nodeField, pParent, bIsFirst);
	//prevent route execution until offset is applied
	if (m_bOffset == 1) return FALSE;
	return ret;
}

/*-------------------------------------------------------------
This function is called when a PROTO is intantiated. It clones a time field properly
-------------------------------------------------------------*/
NodeField *SFTimeField :: Clone (NodeField *pField, MediaObject *pParent)
{
	if (pField == NULL)
		pField = new SFTimeField;
	return SF<Time> :: Clone (pField, pParent);
}

/*-----------------------------------------------------------------
Parse an SFVec4f
MBS: modified to take into account quantization parameters if any
-----------------------------------------------------------------*/
BOOL SF<Vec4f> :: Parse (
	BIFSDecoder *pCoder,
	MediaObject * /*pNode*/,
	BOOL *bHasChanged,
	Quantizer * pQuant
)
{
	Vec4f val;

	if(!pQuant || !pQuant->IsActive())
	{	val.x = pCoder -> ParseFloat("float"); //BF Conformance
		val.y = pCoder -> ParseFloat("float"); //BF Conformance
		val.z = pCoder -> ParseFloat("float"); //BF Conformance
		val.q = pCoder -> ParseFloat("float"); //BF Conformance
	}
	else
		;			// TO DO : qunatization for Vec4f

	TRACEF4 ("%.2f %.2f %.2f %.2f", val.x, val.y, val.z, val.q);
	if (bHasChanged) *bHasChanged = (val==m_value) ? RETURN_ON_SAME_VALUE : TRUE;
	m_value = val;
	return TRUE;
}




//MFUrl override for string to URL typecasting
BOOL MFUrl :: Copy (NodeField &nodeField, MediaObject *, bool bIsFirst) 
{
	m_array.Clear ();
	if (nodeField.GetType() == MFSTRING) {
		MF<ZString> &fld = (MF<ZString> &) nodeField;
		for (int i = 0; i < fld.GetLength (); i++) {
			m_array += (Url) ( (SFString) fld [i]);
		}
	} else {
		MF<Url> &fld = (MF<Url> &) nodeField;
		for (int i = 0; i < fld.GetLength (); i++)
			m_array += fld [i];
	}
	OnFieldUpdate (bIsFirst);

	return TRUE;
}
#endif // BIFSDEC



