/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1997.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/
//$Log: NodeFld.h,v $
//Revision 1.4  2004/06/24 10:21:34  kmueller
//MutliTexture functionality added
//
//Revision 1.3  2004/03/18 19:43:52  mikeseven
//Updated node coding tables per 68th meeting resolutions
//
//Revision 1.2  2003/12/25 14:54:36  iasalomi
//Update of the CVS repository for AFX Tools
//
//Revision 1.1.1.1  2003/12/24 00:42:11  mikeseven
//Initial release of the CVS repository for MPEG-4 Systems code
//after the code was corrupted due to reorganization and human 
//errors.
//
//Revision 1.12  2003/07/20 15:02:26  iasalomi
//Updated for MeshGrid tool
//
//Revision 1.11  2003/07/17 22:13:10  iasalomi
//Updated for MeshGrid tool
//
//Revision 1.10  2003/04/14 16:47:48  jean_lf
//MediaControl cleanup
//
//Revision 1.9  2003/04/03 04:29:02  mikeseven
//Initial release of Player3D and MPEG-J for all MPEG-4 reference softwares players with samples.
//For installation, follow instructions in docs/Player3D_setup.doc.
//
//Many thanks to Prashanth Gujjeti and Viswanathan Swaminathan from Sun Microsystems.
//
//-- Mikael Bourges-Sevenier (Mindego)
//
//Revision 1.8  2003/01/20 17:58:39  jean_lf
//no message
//
//Revision 1.7  2002/10/30 12:28:17  jean_lf
//Diverse Proto fixes
//
//Revision 1.6  2002/10/26 10:53:20  jean_lf
//no message
//
//Revision 1.5  2002/10/25 09:52:54  jean_lf
//no message
//
//Revision 1.4  2002/10/05 20:32:35  zvil
//Fix bug replace node by indexed value didn't delete old node
//
//Revision 1.3  2002/08/28 23:49:03  mikeseven
//Regenerated BIFS encoding and decoding tables for all node template versions.
//
//Revision 1.2  2002/08/08 09:47:27  zvil
//Core 5.9 Release Candidate 9:
//- Handling of time fields
//- Generating BIFS from memory
//- Proper handling of BIFS tables
//- More IPMPX stuff
//- Fixes to IM1-2D for proper use of startTime / stopTime
//- Fixes to Im1Player for proper use of startTime / stopTime
//
//Revision 1.2  2000/03/03 18:41:21  Alex Cotarmanac'h
//Version 1 -- Alexandre Cotarmanac_h
//
// -*- c++ -*-

#if !defined(_NodeFld_h)	// Sentry, use file only if it's not already included.
#define _NodeFld_h

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  BIFS Decoder/ Compositor
FILE:       NodeFld.h
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Declarations of NodeField and derivations.
-------------------------------------------------------------------------*/
#include "ztl.h"
#include "ztypes.h"
#include "MediaObj.h"
#include "FldTypes.h"
#include "NodeCode.h"

//#ifndef BIFSENC	// SCRIPT
//#include "ScriptDefs.h"
//#endif

class Quantizer;
class BifsGenerator;
class PresenterContext;

#ifdef ANIMATION
#include "AnimationQuantizer.h" // MBS: added
class AnimationStreamDecoder; // MBS: definition
#endif

#ifdef BIFSENC
class BIFSEncoder;
#endif

#ifdef MPGJ // SCENE API Integration
class Listener
{
public:
	virtual void notify( NodeField *source ) = 0;
	virtual ~Listener() {}
};
#endif MPGJ // SCENE API Integration

/*-------------------------------------------------------------------------
- NodeField is the base class all kind of node fields
- NodeField objects can parse themselves from BIFS.
- Each object must override the Parse virtual function.
-------------------------------------------------------------------------*/
class NodeField
{
	/*-------------------------------------------------------------
	Methods used by both decoder and encoder
	-------------------------------------------------------------*/
public:
	enum {BIFS_VERSIONS = 7};		// UPDATE HERE ON EACH NEW BIFS VERSION

	NodeField (FIELD_TYPE fieldType) :
		m_pFirstRoute (NULL), m_fieldType (fieldType), m_bFieldUpdated(false)
#ifdef MPGJ
	, m_listeners(NULL)
#endif MPGJ
	{}
	virtual void Remove () {}
	FIELD_TYPE GetType() const {return m_fieldType;}

	/*-------------------------------------------------------------
	Methods used by the decoder
	-------------------------------------------------------------*/
#ifdef ANIMATION
public:
    // added by MBS for animation streams
    virtual void parseAnimationMask(AnimationStreamDecoder *pAS,int anim);
    virtual BOOL animate(AnimationStreamDecoder *pAS,BOOL intra);

	//FTRD
	void setBounds(float *min, float*max) {qp.setBounds(min,max);}
	//FTRD

#endif //ANIMATION

#ifdef BIFSDEC
public:
	virtual ~NodeField ();
	virtual BOOL Parse (
		BIFSDecoder *pBifs,
		MediaObject *pNode,
		BOOL *bHasChanged = NULL,
		Quantizer *pQuant = NULL
	) = 0;
	virtual NodeField *Clone (NodeField *pField, MediaObject *pParent) = 0;
	virtual void Dequantize (Quantizer * /*pQuant*/) {}
	virtual void Clip (Quantizer * /*pQuant*/) {}
	//returns TRUE if field value has changed (default)
	virtual BOOL Copy (
		NodeField &nodeField,
		MediaObject *pParent,
		bool bIsFirst = true
	) = 0;
	void LinkRoute (Route *pRoute);
	void UnlinkRoute (Route *pRoute);
	void OnFieldUpdate (bool bIsFirst = true);
	virtual void GenerateField (BifsGenerator *pBifs) = 0;
	bool WasFieldUpdated () const {return m_bFieldUpdated;}
  void ResetFieldUpdate () { m_bFieldUpdated = false; };
	void SetIsedFieldID (int nID);
	int GetIsedFieldID () const {return m_nIsedFieldID;}
	void GenerateProtoFieldDefinition (BifsGenerator *pBifs, int eventType,  LPCSTR szName);

#ifdef MPGJ		// SCENE API Integration
	void AddListener( Listener *l );
	void RemoveListener( Listener *l );
#endif MPGJ		// SCENE API Integration
#endif // BIFSDEC

	/*-------------------------------------------------------------
	Methods used by the encoder
	-------------------------------------------------------------*/
#ifdef BIFSENC
public:
	virtual BOOL Encode (BIFSEncoder *pBifs, Quantizer *pQuant = NULL);
	virtual ~NodeField () {}
	virtual BOOL EncodeElement (BIFSEncoder *pBifs, Quantizer *pQuant = NULL) = 0;
#endif // BIFSENC

private:
	Route *m_pFirstRoute;
	bool m_bFieldUpdated;
	int m_nIsedFieldID;
#ifdef MPGJ // SCENE API Integration
	ZArray<Listener*> *m_listeners;
#endif MPGJ // SCENE API Integration

protected: //BF Script
	FIELD_TYPE m_fieldType;

#ifdef ANIMATION
protected:
    // MBS for animation streams
    AnimationQuantizer qp;
#endif

};

/*-----------------------------------------------------------------
A class template for all SF fields, except of SFNode (or specific type
SFNode fields).  T can be the classes Int, Float, Bool, Color, Time,
Rotation, ZString, Vec2f and Vec3f which are defined in ZTL.
E.g. SFInt  would be declared as SF<int>, and SFRotation as
SF<Rotation>.
-----------------------------------------------------------------*/
template <class T> class SF : public NodeField
{
public:
	SF<T> () : m_value (0), NodeField (s_fieldType) {}
	SF<T> (T value) : m_value (value), NodeField (s_fieldType) {}
	operator T () const {return m_value;}

#ifdef ANIMATION
    // added by MBS for animation streams
    virtual void parseAnimationMask(AnimationStreamDecoder *pAS,int anim);
    virtual BOOL animate(AnimationStreamDecoder *pAS,BOOL intra);
#endif

#ifdef BIFSDEC
	/*-----------------------------------------------------------------
	Copy is activated in case of Routes.
	-----------------------------------------------------------------*/
	virtual BOOL Copy (NodeField &nodeField, MediaObject *, bool bIsFirst)
	{
		BOOL ret = TRUE;
		T val = 0;

		switch (nodeField.GetType ()) {
		case SFBOOL:
			val = (bool) (Bool) (SF<Bool>&) nodeField;
			break;
		case SFINT32:
			val = (int) (SF<int>&) nodeField;
			break;
		case SFFLOAT:
			val = (float) (SF<float>&) nodeField;
			break;
		case SFTIME:
			val = (Time) (SF<Time>&) nodeField;
			break;
		default:
			if (GetType () == nodeField.GetType ())
				val = ((SF<T>&) nodeField).m_value;
			break;
		}
		if (m_value == val) ret = FALSE;
		m_value = val;
		OnFieldUpdate (bIsFirst);
		return ret;
	}
	virtual BOOL Parse (
		BIFSDecoder *pBifs,
		MediaObject *pNode,
		BOOL *bHasChanged = NULL,
		Quantizer *pQuant = NULL
	);
	virtual void GenerateField (BifsGenerator *pBifs);
	virtual NodeField *Clone (NodeField *pField, MediaObject *)
	{
		if (pField == NULL)
			pField = new SF<T>;
		((SF<T>*) pField) ->m_value = m_value;
		return pField;
	}
	T operator = (const T value)
	{
		m_value = value;
		//OnFieldUpdate (this);
		OnFieldUpdate ((bool)this); // SOLARIS PORTING
		return value;
	}
	T GetValue() {return m_value;} // SCRIPT
#endif

#ifdef BIFSENC
	virtual BOOL EncodeElement (
		BIFSEncoder *pBifs,
		Quantizer *pQuant = NULL
	);
#endif

#ifdef ENCODER
	T operator = (const T value) {return m_value = value;}
#endif

protected:
	T m_value;
	static FIELD_TYPE s_fieldType;
};

/*-----------------------------------------------------------------
SFBool is a synonym for SF<Bool>
-----------------------------------------------------------------*/
class SFBool : public SF<Bool>
{
public:
	SFBool (bool b = false) {m_value = b;}
	operator bool () const {return m_value;}
};

/*-----------------------------------------------------------------
SFString is an extenstion of SF<ZString>.
-----------------------------------------------------------------*/
struct Url;

class SFString : public SF<ZString>
{
public:
	SFString () {}
	SFString (LPCSTR szString) : SF<ZString> (szString) {}
	operator const ZString & () const {return m_value;}
	const ZString &operator = (const ZString &value)
	{
		SF<ZString> :: operator = (value);
		return value;
	}
	int GetLength () const {return m_value.GetLength ();}
	operator LPCSTR () const {return m_value;}
	operator Url () const;

#ifdef BIFSENC
	void EncodeString (BIFSEncoder *pBifs);
#endif

};

/*-----------------------------------------------------------------
Url
-----------------------------------------------------------------*/
struct Url
{
	int ODid;
	SFString urlValue;

	Url (int nODid = -1) : ODid (nODid) {}
	operator int () const {return ODid;}
	Url &operator = (const Url &url)
	{
		ODid = url.ODid;
		if (url.urlValue.GetLength() > 0)
			urlValue = url.urlValue;
		return *this;
	}
	operator SFString () const { return urlValue; }
	bool operator == (const Url &url) const
	{	
		return ODid == url.ODid && !strcmp(urlValue , url.urlValue);
	}

	bool operator != (const Url &url) const {return !(*this == url);}
};

/*-----------------------------------------------------------------
SFUrl stores either URL or objectID.
-----------------------------------------------------------------*/
class SFUrl : public SF<Url>
{
public:
	int operator = (const int value)
	{
		return m_value.ODid = value;
	}
	operator int () const {return m_value.ODid;}

	bool operator == (const SFUrl &url) const {return m_value == url;}

};


class BifsFactory
{
public:
	virtual MediaObject *CreateNode () = 0;
};

template <class T> class BifsNodeFactory : public BifsFactory
{
	virtual MediaObject *CreateNode () {return new T;}

public:
	BifsNodeFactory ()
	{
		SFGenericNode :: StoreFactory (T :: GetCode (), this);
	}
};

/*-----------------------------------------------------------------
This is a base class for all fields that are themselves nodes.  Some
fields may contain any node type, so they are defined as SFGenericNode,
but usually only a small amount of node types are allowed for a given
nodeField.  For this purpose, the nodes are classified according to Node
Data Types.  A Node Data Type is implemented by an object derived from
SFGenericNode.
-----------------------------------------------------------------*/
class SFGenericNode : public NodeField
{
private:
	virtual bool IsTypeOK (MediaObject *) const {return true;}
	virtual int GetNodeTableSize () const;
	virtual int GetNodeVersionSize (int ) const {return 0;}

public:
	SFGenericNode (BOOL bIsSceneRoot = FALSE) :
		NodeField (SFNODE),
		m_pNode (NULL),
		m_bIsSceneRoot (bIsSceneRoot)
	{}
	virtual ~SFGenericNode () {Remove ();}
	virtual void Remove ();
	virtual MediaObject *CreateNode (int nodeType);
	void SetParent (MediaObject *pNode) {m_pParent = pNode;}
	static void StoreFactory (int nCode, BifsFactory *pFactory);
	MediaObject *GetNode () const {return m_pNode;}
	operator MediaObject * () const {return m_pNode == NULL ? NULL : m_pNode -> GetRenderingNode ();}
	MediaObject *operator -> () const {return m_pNode == NULL ? NULL : m_pNode -> GetRenderingNode ();}

protected:
	static const int m_nodeVersions [];

#ifdef BIFSDEC
	/*-----------------------------------------------------------------
	Assigment operator activated in case of Routes. We assume that
	the nodeField argument is always of the same type as this, therefor
	allow for deliberate downcasting. This assumption is dangerous.
	Commercial application should implement safe casting or proper
	conversion mechanism.
	-----------------------------------------------------------------*/
public:
	MediaObject *SetObject (MediaObject *node, MediaObject *pParent = NULL, bool bIsFirst = true);
	void ReplaceNode (MediaObject *pNew);
	virtual BOOL Parse (
		BIFSDecoder *pBifs,
		MediaObject *pNode,
		BOOL *bHasChanged = NULL,
		Quantizer *pQuant = NULL
	);
	virtual NodeField *Clone (NodeField *pField, MediaObject *pParent);
	BOOL ParseNodeField (
		BIFSDecoder *pBifs,
		MediaObject *pNode,			// the node this field belongs to
		NodeField *pContainer,		// a pointer to the container field
		BOOL bISMF					// TRUE when the field is an MF field
	);
	void RemoveNode () {m_pNode = 0;}
	virtual void GenerateField (BifsGenerator *pBifs);
	virtual int GetNodeType (int) const {return 0;}

private:
	virtual BOOL Copy (NodeField &nodeField, MediaObject *pParent, bool bIsFirst)
	{
		SetObject (((SFGenericNode&) nodeField).GetNode (), pParent, bIsFirst);
		return TRUE;
	}

#endif	//	BIFSDEC

#ifdef BIFSENC
public:
	virtual BOOL EncodeElement (BIFSEncoder *pBifs, Quantizer *pQuant = NULL);

protected:
	virtual MediaObject *CreateNode (LPCSTR szNodeName, int &rNodeType);

#endif	// BIFSENC

#ifndef BIFSDEC
public:
	MediaObject *operator = (MediaObject *node) {return m_pNode = node;}
#endif	// BIFSDEC

private:
	BOOL m_bIsSceneRoot;
	static BifsFactory *m_bifsFactory [];
	MediaObject *m_pParent;

protected:
	MediaObject *m_pNode;
};

/*-----------------------------------------------------------------
A class template for all fields derived from SFGenericNode, when the
node is of a single known type.  E.g. SFAppearance2Dnode would be
declared as SFNode<Appearance2D>.
-----------------------------------------------------------------*/
template <class T> class SFNode : public SFGenericNode
{
	/*-------------------------------------------------------------
	Methods used by both the decoder and the encoder
	-------------------------------------------------------------*/
private:
	virtual int GetNodeTableSize () const {return 1;}
	virtual bool IsTypeOK (MediaObject *pNode) const;
	virtual int GetNodeVersionSize (int nVersion) const;

public:
	virtual MediaObject *CreateNode (int);
	T* operator -> () const
	{
		return (T*) (m_pNode == NULL ? NULL : m_pNode -> GetRenderingNode ());
	}
	operator T* () const
	{
		return (T*) (m_pNode == NULL ? NULL : m_pNode -> GetRenderingNode ());
	}

	/*-------------------------------------------------------------
	Methods used by the decoder
	-------------------------------------------------------------*/
#ifdef BIFSDEC
private:
	virtual NodeField *Clone (NodeField *pField, MediaObject *pParent)
	{
		if (pField == NULL)
			pField = new SFNode<T>;
		return SFGenericNode :: Clone (pField, pParent);
	}
	virtual int GetNodeType (int) const {return 1;}

public:
	T* ParseNode (BIFSDecoder *pBifs, MediaObject *pParent);
#endif

	/*-------------------------------------------------------------
	Methods used by the encoder
	-------------------------------------------------------------*/
#ifdef BIFSENC
public:
	MediaObject *operator = (MediaObject *node) {return m_pNode = node;}

private:
	virtual MediaObject *CreateNode (LPCSTR szNodeName, int &rNodeType);
#endif
};

/*-----------------------------------------------------------------
For every Node Data Type which includes more than one nodeType,
an SF...Node class is defined.
-----------------------------------------------------------------*/
//MBS 110498: added new node data types and remove old ones
//MBS 210601: added NT_TEMPORALNODE
typedef enum
{
  NT_2D = 0, NT_3D, NT_APPEARANCE, NT_AUDIO, NT_BACKGROUND2D, NT_BACKGROUND3D,
  NT_COLOR = 10, NT_COORDINATE, NT_COORDINATE2D, NT_EXPRESSION, NT_FAP, NT_FDP,
  NT_FIT = 20, NT_FACEDEFMESH, NT_FACEDEFTABLES, NT_FACEDEFTRANSFORM, NT_FOG,
  NT_FONTSTYLE = 30, NT_GEOMETRY, NT_LINEPROPERTIES, NT_MATERIAL, NT_NAVIGATIONINFO,
  NT_NORMAL = 40, NT_STREAMING, NT_TEXTURECOORDINATE, NT_TEXTURE,
  NT_TEXTURETRANSFORM = 50, NT_TOPNODE, NT_VIEWPOINT, NT_VISEME, NT_VIEWPORT,
  NT_MEDIANODE = 60,NT_TEMPORALNODE,NT_COLORNODE,NT_APPEARANCENODE,
  NT_DEPTHTEXTURENODE = 70, NT_PARTICLEINITIALIZERNODE, NT_INFLUENCENODE,
	NT_DEPTHIMAGENODE,
  NT_WORLDNODE = 80
} NodeType;

template <NodeType type> class SFNodeType : public SFGenericNode
{
	/*-------------------------------------------------------------
	Methods used by the decoder and the encoder
	-------------------------------------------------------------*/
private:
#ifdef MPGJ
public: //--awm 991208 added two public methods for MPEGJ
//	virtual int GetNodeType (int nodeCode) const; //--awm 991208 use this method in MPEGJ
	virtual MediaObject *CreateNode (int nodeType) //--awm 991208 use this method in MPEGJ

#endif MGPJ

#ifndef MPGJ
private:
	virtual MediaObject *CreateNode (int nodeType)
#endif
	{
		if (--nodeType < 0)			// -- added for FDIS
			return NULL;
		if (nodeType >= GetNodeTableSize ())
			return NULL;
		return SFGenericNode :: CreateNode (m_nodeTable [nodeType]);
	}
	virtual int GetNodeTableSize () const;
	virtual int GetNodeVersionSize (int nVersion) const
	{
		return m_nodeVersionLength [nVersion - 1];
	}

	virtual bool IsTypeOK (MediaObject *pNode) const
	{
		for (int i = GetNodeTableSize (); --i >= 0;)
			if (pNode -> GetNodeCode () == m_nodeTable [i])
				return true;
		return false;
	}

public:
	SFNodeType (BOOL bIsSceneRoot = FALSE) : SFGenericNode (bIsSceneRoot) {}

	/*-------------------------------------------------------------
	Methods used by the decoder
	-------------------------------------------------------------*/
#ifdef BIFSDEC
	virtual NodeField *Clone (NodeField *pField, MediaObject *pParent)
	{
		if (pField == NULL)
			pField = new SFNodeType<type>;
		return SFGenericNode :: Clone (pField, pParent);
	}

// MPEGJ integration
//private:
	/*-------------------------------------------------------------
	Return the index of the node whose code is 'nCode' in the node table of this type
	-------------------------------------------------------------*/
	virtual int GetNodeType (int nCode) const
	{
		for (int i = GetNodeTableSize (); --i >= 0;)
			if (m_nodeTable [i] == nCode)
				return i + 1;
		return 0;
	}
private:
#endif

	/*-------------------------------------------------------------
	Methods used by the encoder
	-------------------------------------------------------------*/
#ifdef BIFSENC
private:
	virtual MediaObject *CreateNode (LPCSTR szNodeName, int &rNodeType);
#endif	// BIFSENC

private:
	static const int m_nodeTable [];
	static const int m_nodeVersionLength [BIFS_VERSIONS];
};

//MBS 110498: removed old node data types, replaced by new ones
typedef SFNodeType<NT_2D> SF2DNode;
typedef SFNodeType<NT_3D> SF3DNode;
typedef SFNodeType<NT_AUDIO> SFAudioNode;
typedef SFNodeType<NT_GEOMETRY> SFGeometryNode;
typedef SFNodeType<NT_MATERIAL> SFMaterialNode;
typedef SFNodeType<NT_STREAMING> SFStreamingNode;
typedef SFNodeType<NT_TEXTURE> SFTextureNode;
typedef SFNodeType<NT_TEXTURETRANSFORM> SFTextureTransformNode;
typedef SFNodeType<NT_TOPNODE> SFTopNode;
typedef SFNodeType<NT_VIEWPORT> SFViewportNode;
typedef SFNodeType<NT_WORLDNODE> SFWorldNode;
typedef SFNodeType<NT_MEDIANODE> SFMediaNode;

//MBS 210601: added following new node types
typedef SFNodeType<NT_TEMPORALNODE> SFTemporalNode;
//ZL 190701: added following new node types
typedef SFNodeType<NT_COLORNODE> SFColorNode;
typedef SFNodeType<NT_APPEARANCENODE> SFAppearanceNode;
typedef SFNodeType<NT_DEPTHTEXTURENODE> SFDepthTextureNode;
typedef SFNodeType<NT_PARTICLEINITIALIZERNODE> SFParticleInitializerNode;
typedef SFNodeType<NT_INFLUENCENODE> SFInfluenceNode;
typedef SFNodeType<NT_DEPTHIMAGENODE> SFDepthImageNode;

//JLF 180902: modified for advanced text & graphics
typedef SFNodeType<NT_LINEPROPERTIES> SFLinePropertiesNode;
typedef SFNodeType<NT_FONTSTYLE> SFFontStyleNode;

// KM 060404: added node for textureCoords and multitextureCoords
typedef SFNodeType<NT_TEXTURECOORDINATE> SFTextureCoordinateNode;
/*-----------------------------------------------------------------
A base class for all MF... classes
-----------------------------------------------------------------*/
class MField : public NodeField
{
	/*-------------------------------------------------------------
	Methods used by the decoder and the encoder
	-------------------------------------------------------------*/
public:
	MField (FIELD_TYPE fieldType) : NodeField (fieldType) {}
	virtual int GetLength () const = 0;

	/*-------------------------------------------------------------
	Methods used by the decoder
	-------------------------------------------------------------*/
#ifdef BIFSDEC
	virtual BOOL Parse (
		BIFSDecoder *pBifs,
		MediaObject *pNode,
		BOOL *bHasChanged = NULL,
		Quantizer *pQuant = NULL
	);
	virtual void DeleteFrom (int index, BOOL bRoute = TRUE) = 0;
	virtual BOOL AppendElement (
		BIFSDecoder *pBifs,
		MediaObject *pNode,
		Quantizer *pQuant = NULL
	) = 0;
	virtual BOOL InsertElement (
		BIFSDecoder *pBifs,
		MediaObject *pNode,
		int nIndex,
		Quantizer *pQuant = NULL
	) = 0;
	virtual BOOL ReplaceElement (
		BIFSDecoder *pBifs,
		MediaObject *pNode,
		int nIndex,
		BOOL &bHasChanged,
		Quantizer *pQuant = NULL
	) = 0;
	virtual void GenerateField (BifsGenerator *pBifs);

private:
	virtual void GenerateElement (int index, BifsGenerator *pBifs) = 0;


#ifdef PREDICTIVE	//AC
public:
//Remark : just for debugging purposes
#ifdef _DEBUG
	virtual void addElement(int) { fprintf(stderr,"[ERROR] Should not be used(ndfld)\n");}
	virtual void addElement(float){ fprintf(stderr,"[ERROR] Should not be used(ndfld)\n");}
	virtual void addElement(Vec2f){ fprintf(stderr,"[ERROR] Should not be used(ndfld)\n");}
	virtual void addElement(Vec3f){ fprintf(stderr,"[ERROR] Should not be used(ndfld)\n");}
	virtual void addElement(Color3f){ fprintf(stderr,"[ERROR] Should not be used(ndfld)\n");}
	virtual void addElement(Rotation){ fprintf(stderr,"[ERROR] Should not be used(ndfld)\n");}
#else
	virtual void addElement(int) {}
	virtual void addElement(float){}
	virtual void addElement(Vec2f){}
	virtual void addElement(Vec3f){}
	virtual void addElement(Color3f){}
	virtual void addElement(Rotation){}
#endif

#endif	//AC end
#endif	//	BIFSDEC

	/*-------------------------------------------------------------
	Methods used by the encoder
	-------------------------------------------------------------*/
#ifdef BIFSENC
public:
	virtual BOOL Encode (
		BIFSEncoder *pBifs,
		Quantizer *pQuant = NULL
	);
	virtual BOOL EncodeElement (BIFSEncoder *pBifs, Quantizer *pQuant = NULL) = 0;
#endif		// BIFSINC

#ifdef ANIMATION
    /* added by MBS for animaion streams, special case of MField:
       isTotal: TRUE if all fields are animated
       idx    : else, we specify the animated indexes
       */
protected:
    BOOL isTotal;
    ZArray<int> idx;
#endif
};

/*-----------------------------------------------------------------
A class template for all MF fields, except of MFNode (or specific type
MFNode fields).  T can be int, float, BOOL, or the classes as Color,
Time, Rotation, ZString, Vec2f and Vec3f which are defined in ZTL.
-----------------------------------------------------------------*/
template <class T> class MF : public MField
{
	/*-------------------------------------------------------------
	Methods used by the decoder and the encoder
	-------------------------------------------------------------*/
public:
	MF<T> () : MField (s_fieldType) {}
	const T &operator [] (int index) const {return m_array [index];}
	virtual int GetLength () const {return m_array.GetLength ();}
	T operator += (T t) {return m_array += t;}
	BOOL operator -= (T t) {return m_array -= t;}
	virtual void Remove () {m_array.Clear();}
#ifdef PREDICTIVE	//AC
// see MField addElement
	void addElement(T t) { m_array+=t;}
#endif	//AC end
	/*-------------------------------------------------------------
	Methods used by the decoder
	-------------------------------------------------------------*/
#ifdef ANIMATION
public:
    // added by MBS for animation streams
    virtual void parseAnimationMask(AnimationStreamDecoder *pAS,int anim);
    virtual BOOL animate(AnimationStreamDecoder *pAS,BOOL intra);
#endif

public:
	BOOL UIReplaceElement(BIFSDecoder *pBifs, MediaObject *pNode, int nIndex) { 
		BOOL rien;
		return ReplaceElement(pBifs,pNode,nIndex,rien);
	}


#ifdef BIFSDEC
private:
	virtual BOOL AppendElement (
		BIFSDecoder *pBifs,
		MediaObject *pNode,
		Quantizer *pQuant = NULL
	)
	{
		SF<T> m_element;
		if (!m_element.Parse (pBifs, pNode, NULL, pQuant))
			return FALSE;
		m_array += m_element;
		return TRUE;
	}

	virtual BOOL InsertElement (
		BIFSDecoder *pBifs,
		MediaObject *pNode,
		int nIndex,
		Quantizer *pQuant = NULL
	)
	{
		SF<T> m_element;
		if (!m_element.Parse (pBifs, pNode, NULL, pQuant))
			return FALSE;
		m_array.InsertAt (nIndex, m_element);
		return TRUE;
	}

	virtual BOOL ReplaceElement (
		BIFSDecoder *pBifs,
		MediaObject *pNode,
		int nIndex,
		BOOL &bHasChanged,
		Quantizer *pQuant = NULL
	)
	{
		SF<T> m_element;
		if (!m_element.Parse (pBifs, pNode, &bHasChanged, pQuant))
			return FALSE;
		m_array.SetAt (nIndex, m_element);
		return TRUE;
	}

	virtual NodeField *Clone (NodeField *pField, MediaObject *)
	{
		if (pField == NULL)
			pField = new MF<T>;
		for (int i = 0; i < GetLength (); i++)
			((MF<T> *) pField) -> m_array += m_array [i];
		return pField;
	}
	virtual void GenerateElement (int index, BifsGenerator *pBifs)
	{
		SF<T> element;
		element = m_array [index];
		element.GenerateField (pBifs);
	}

protected:
	/*-----------------------------------------------------------------
	Assigment operator activated in case of Routes. We assume that
	the nodeField argument is always of the same type as this, therefor
	allow for deliberate downcasting. This assumption is dangerous.
	Commercial application should implement safe casting or proper
	conversion mechanism.
	-----------------------------------------------------------------*/
	virtual BOOL Copy (NodeField &nodeField, MediaObject *, bool bIsFirst)
	{
		m_array.Clear ();
		MF<T> &fld = (MF<T> &) nodeField;
		for (int i = 0; i < fld.GetLength (); i++)
			m_array += fld [i];
		OnFieldUpdate (bIsFirst);
		return TRUE;
	}

public:
  ZArray<T> &GetArray () { return m_array; }; // VUB
  void Expand (int size,  BOOL bRoute = FALSE) // VUB 
  {
    m_array.Expand(size);
		if (bRoute)
			OnFieldUpdate ();
  }
	void InsertAt (int index, T value, BOOL bRoute = TRUE)
	{
		m_array.InsertAt (index, value);
		if (bRoute)
			OnFieldUpdate ();
	}
	void Append (T value, BOOL bRoute = TRUE)
	{
		m_array += value;
		if (bRoute)
			OnFieldUpdate ();
	}
	void SetAt (int index, T value, BOOL bRoute = TRUE)
	{
		m_array.SetAt (index, value);
		if (bRoute)
			OnFieldUpdate ();
	}
	void DeleteFrom (int index, BOOL bRoute = TRUE)
	{
		m_array.DeleteFrom (index);
		if (bRoute)
			OnFieldUpdate ();
	}
#endif	//	BIFSDEC

	/*-------------------------------------------------------------
	Methods used by the encoder
	-------------------------------------------------------------*/
#ifdef BIFSENC
private:
	virtual BOOL EncodeElement (BIFSEncoder *pBifs, Quantizer *pQuant = NULL)
	{
		SF<T> m_element;
		return m_element.Encode (pBifs, pQuant);
	}
#endif		// BIFSENC

protected:
	ZArray<T> m_array;
	static FIELD_TYPE s_fieldType;
};


class MFUrl : public MF<Url>
{
protected:
	BOOL Copy (NodeField &nodeField, MediaObject *, bool bIsFirst) ;
};


/*-----------------------------------------------------------------
A base class for classes implementing MFNode (or specific type
MFNode fields).
-----------------------------------------------------------------*/
class MFGenericNode : public MField
{
	/*-------------------------------------------------------------
	Methods used by the decoder and the encoder
	-------------------------------------------------------------*/
public:
	MFGenericNode () : MField (MFNODE) {m_pParent = NULL;}
	virtual ~MFGenericNode () {Remove ();}
	virtual void Remove ();
	MediaObject *operator [] (int index)
	{
		MediaObject *pObject = m_array [index];
		return pObject == NULL ? NULL : pObject -> GetRenderingNode ();
	}
	void DeleteFrom (int index, BOOL bRoute = TRUE);
	virtual int GetLength () const {return m_array.GetLength ();}

private:

	/*-------------------------------------------------------------
	Methods used by the decoder
	-------------------------------------------------------------*/
#ifdef BIFSDEC
	friend class MediaObject;

private:
	virtual BOOL Copy (NodeField &nodeField, MediaObject *pParent, bool bIsFirst);
	int RemoveNode (MediaObject *pNode) 
	{
		int i = m_array.Find(pNode);
		m_array -= pNode;
		return i;
	}
	virtual void GenerateElement (int, BifsGenerator *) {}

protected:
	virtual NodeField *Clone (NodeField *pField, MediaObject *pParent);

public:
	MediaObject *GetObject (int index) {return m_array [index];}
	void ReplaceNode (MediaObject *pOld, MediaObject *pNew);
	void SetParent (MediaObject *pNode) {m_pParent = pNode;}
	void InsertAt (int index, MediaObject * value, MediaObject *pParent, bool bRoute = true);
	void Append (MediaObject * value, MediaObject *pParent, bool bRoute = true);
	void SetAt (int index, MediaObject * value, MediaObject *pParent, bool bRoute = true);
	virtual BOOL AppendElement (
		BIFSDecoder *pBifs,
		MediaObject *pNode,
		Quantizer * = NULL
	)
	{
		SFGenericNode m_element;
		if (!m_element.ParseNodeField (pBifs, pNode, this, TRUE))
			return FALSE;
		ASSERT (m_element.GetNode ());
		m_array += m_element.GetNode ();
		m_pParent = pNode;
		return TRUE;
	}

	virtual BOOL InsertElement (
		BIFSDecoder *pBifs,
		MediaObject *pNode,
		int nIndex,
		Quantizer * = NULL
	)
	{
		SFGenericNode m_element;
		if (!m_element.ParseNodeField (pBifs, pNode, this, TRUE))
			return FALSE;
		m_array.InsertAt (nIndex, m_element.GetNode ());
		m_pParent = pNode;
		return TRUE;
	}

	virtual BOOL ReplaceElement (
		BIFSDecoder *pBifs,
		MediaObject *pNode,
		int nIndex,
		BOOL &,
		Quantizer * = NULL
	)
	{
		SFGenericNode m_element;
		if (nIndex < GetLength () && m_array [nIndex] != NULL)
			m_array [nIndex] -> RemoveContainer (this, m_pParent);
		m_array.SetAt (nIndex, NULL);
		if (!m_element.ParseNodeField (pBifs, pNode, this, TRUE))
			return FALSE;
		m_array.SetAt (nIndex, m_element.GetNode ());
		m_pParent = pNode;
		return TRUE;
	}

#endif	//	BIFSDEC

	/*-------------------------------------------------------------
	Methods used by the encoder
	-------------------------------------------------------------*/
#ifdef BIFSENC
private:
	virtual BOOL EncodeElement (BIFSEncoder *pBifs, Quantizer *pQuant = NULL)
	{
		SFGenericNode m_element;
		if (m_element.Encode (pBifs, pQuant))
		{	m_array += m_element;
			m_element = NULL;
			return TRUE;
		}
		return FALSE;
	}
#endif		// BIFSENC

protected:
	ZArray<MediaObject *> m_array;
	MediaObject *m_pParent;
};

/*-----------------------------------------------------------------
A class template for all MFNode fields which allow nodes of a single
known type.  E.g. MFAppearance2Dnode would be declared as
MFNode<Appearance2D>.
-----------------------------------------------------------------*/
template <class T> class MFNode : public MFGenericNode
{
	/*-------------------------------------------------------------
	Methods used by the decoder and the encoder
	-------------------------------------------------------------*/
public:
	T *operator [] (int index)
	{
		MediaObject *pObject = m_array [index];
		return (T*) (pObject == NULL ? NULL : pObject -> GetRenderingNode ());
	}

	/*-------------------------------------------------------------
	Methods used by the decoder
	-------------------------------------------------------------*/
#ifdef BIFSDEC
public:
	T *ParseElement (BIFSDecoder *pBifs, MediaObject *pParent);

private:
	virtual BOOL AppendElement (
		BIFSDecoder *pBifs,
		MediaObject *pNode,
		Quantizer * = NULL
	)
	{
		SFNode<T> m_element;
		if (!m_element.ParseNodeField (pBifs, pNode, this, TRUE))
			return FALSE;
		ASSERT (m_element.GetNode ());
		// SOLARIS PORTING
		//m_array += m_element;
		m_array += (MediaObject*) m_element;
		m_pParent = pNode;
		return TRUE;
	}

	virtual BOOL InsertElement (
		BIFSDecoder *pBifs,
		MediaObject *pNode,
		int nIndex,
		Quantizer * = NULL
	)
	{
		SFNode<T> m_element;
		if (!m_element.ParseNodeField (pBifs, pNode, this, TRUE))
			return FALSE;
		// SOLARIS PORTING
		//m_array.InsertAt (nIndex, m_element);
		m_array.InsertAt (nIndex, (MediaObject*)m_element);
		m_pParent = pNode;
		return TRUE;
	}

	virtual BOOL ReplaceElement (
		BIFSDecoder *pBifs,
		MediaObject *pNode,
		int nIndex,
		BOOL &,
		Quantizer * = NULL
	)
	{
		SFNode<T> m_element;
		if (nIndex < GetLength () && m_array [nIndex] != NULL)
			m_array [nIndex] -> RemoveContainer (this, m_pParent);
		m_array.SetAt (nIndex, NULL);
		if (!m_element.ParseNodeField (pBifs, pNode, this, TRUE))
			return FALSE;
		// SOLARIS PORTING
		//m_array.SetAt (nIndex, m_element);
		m_array.SetAt (nIndex, (MediaObject*)m_element);
		m_pParent = pNode;
		return TRUE;
	}
	virtual NodeField *Clone (NodeField *pField, MediaObject *pParent)
	{
		if (pField == NULL)
			pField = new MFNode<T>;
		return MFGenericNode :: Clone (pField, pParent);
	}
	virtual void GenerateElement (int index, BifsGenerator *pBifs)
	{	
		SFNode<T> element;
		element.SetObject (m_array [index]);
		element.GenerateField (pBifs);
	}
#endif	//	BIFSDEC

	/*-------------------------------------------------------------
	Methods used by the encoder
	-------------------------------------------------------------*/
#ifdef BIFSENC
private:
	virtual BOOL EncodeElement (BIFSEncoder *pBifs, Quantizer *pQuant = NULL)
	{
		SFNode<T> m_element;
		if (m_element.Encode (pBifs, pQuant))
		// SOLARIS PORTING - added typecast
		//{	m_array += m_element;
		{
			m_array += (MediaObject *)m_element;
			#ifdef BIFSDEC
			((MediaObject*)m_element) -> SetContainer (this, TRUE);
			#endif
			m_element = NULL;
			return TRUE;
		}
		return FALSE;
	}
#endif		// BIFSENC

};

/*-----------------------------------------------------------------
A class template for all MFNode fields which allow more than one node
type.  T can be any of the SF...Node types. E.g. MF2DNode would be
declared as MFNodeType<SF2DNode>.
-----------------------------------------------------------------*/
template <class T> class MFNodeType : public MFGenericNode
{
	/*-------------------------------------------------------------
	Methods used by the decoder
	-------------------------------------------------------------*/
#ifdef BIFSDEC
public:
	virtual BOOL AppendElement (
		BIFSDecoder *pBifs,
		MediaObject *pNode,
		Quantizer * = NULL
	)
	{
		T m_element;
		if (!m_element.ParseNodeField (pBifs, pNode, this, TRUE))
			return FALSE;
		//ASSERT (m_element.GetNode ()); //AC20011102 : crashes when NULL.
		if (m_element.GetNode())
		m_array += m_element.GetNode ();
		m_pParent = pNode;
		return TRUE;
	}
	virtual void GenerateElement (int index, BifsGenerator *pBifs)
	{	
		T element;
		element.SetObject (m_array [index]);
		element.GenerateField (pBifs);
	}

private:
	virtual BOOL InsertElement (
		BIFSDecoder *pBifs,
		MediaObject *pNode,
		int nIndex,
		Quantizer * = NULL
	)
	{
		T m_element;
		if (!m_element.ParseNodeField (pBifs, pNode, this, TRUE))
			return FALSE;
		for (int i=m_array.GetLength()-1; i<nIndex;i++) m_array += NULL;
		m_array.InsertAt (nIndex, m_element.GetNode ());
		m_pParent = pNode;
		return TRUE;
	}

	virtual BOOL ReplaceElement (
		BIFSDecoder *pBifs,
		MediaObject *pNode,
		int nIndex,
		BOOL &,
		Quantizer * = NULL
	)
	{
		T m_element;
		if (nIndex < GetLength () && m_array [nIndex] != NULL)
			m_array [nIndex] -> RemoveContainer (this, m_pParent);
		m_array.SetAt (nIndex, NULL);
		if (!m_element.ParseNodeField (pBifs, pNode, this, TRUE))
			return FALSE;
		m_array.SetAt (nIndex, m_element.GetNode ());
		m_pParent = pNode;
		return TRUE;
	}
	virtual NodeField *Clone (NodeField *pField, MediaObject *pParent)
	{
		if (pField == NULL)
			pField = new MFNodeType<T>;
		return MFGenericNode :: Clone (pField, pParent);
	}
#endif	//	BIFSDEC

	/*-------------------------------------------------------------
	Methods used by the encoder
	-------------------------------------------------------------*/
#ifdef BIFSENC
public:
	virtual BOOL EncodeElement (BIFSEncoder *pBifs, Quantizer *pQuant = NULL)
	{
		T m_element;
		if (m_element.Encode (pBifs, pQuant))
		{	m_array += m_element;
			m_element = NULL;
			return TRUE;
		}
		return FALSE;
	}
#endif		// BIFSENC

};

typedef MFNodeType<SFWorldNode> MFWorldNode;

/*-------------------------------------------------------------
A derivation of SF<Time> field to handle the special behavior of
startTime/stopTime fields
-------------------------------------------------------------*/
class SFTimeField : public SF<Time>
{
private:
	//0: just constructed, 1: offset not set, 2: offset applied
	int m_bOffset;
		
public:
	SFTimeField (double value = 0) : SF<Time> (value), m_bOffset (0) {}

#ifdef BIFSDEC
private:
	virtual BOOL Parse (
		BIFSDecoder *pBifs,
		MediaObject *pNode,
		BOOL *bHasChanged = NULL, 
		Quantizer *pQuant = NULL
	);
	virtual BOOL Copy (NodeField &nodeField, MediaObject *pParent, bool bIsFirst);
	virtual NodeField *Clone (NodeField *pField, MediaObject *pParent);

public:
	operator Time () const {return m_value;}
	Time operator = (const Time value);
	//apply offset using presenter context
	void ApplyOffset (const PresenterContext *pContext = NULL);

#endif BIFSDEC
};

#endif                                      // _NodeFld_h sentry.

