/***********************************************************************
    This software module  was  originally  developed by Zvi  Lifshitz
    (Triton R&D Ltd.) and  Mikael Bourges-Sevenier (France Telecom CNET)
    in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)
    standard.  This software module is an implementation of a part of
    one or more MPEG-4 Systems (ISO/IEC 14496-1) tools  as  specified
    by  the  MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC gives
    users of the MPEG-4 Systems (ISO/IEC 14496-1)  free  license  to
    this software module or modifications thereof for use in hardware
    or software products claiming conformance to the  MPEG-4  Systems
    (ISO/IEC 14496-1). Those intending to use this software module in
    hardware or software  products  are  advised  that  its  use  may
    infringe   existing  patents.  The  original  developer  of  this
    software module and his/her company, the subsequent  editors  and
    their  companies,  and  ISO/IEC have no liability for use of this
    software module or modifications thereof  in  an  implementation.
    Copyright  is not released for non MPEG-4 Systems (ISO/IEC 14496-
    1) conforming  products.  Triton R&D Ltd. and France  Telecom  CNET
    retain  full  right to use the code for their own purpose, assign
    or donate the code to a third party and to inhibit third  parties
    from  using  the  code  for  non MPEG-4 Systems (ISO/IEC 14496-1)
    conforming products. This copyright notice must  be  included  in
    all copies or derivative works. Copyright  1997.
************************************************************************/

/**************************************************************************
 * Project Im1Player
 * INTERNATIONAL STANDARD ORGANIZATION
 * ISO/IEC JTC1/SC29/WG11
 * 
 * SUBSYSTEM: BIFS Encoder/Decoder
 * FILE:      Nodes.h
 * AUTHOR:    Mikael Bourges-Sevenier (Mindego Inc.)
 * OVERVIEW:  Declaration of nodes and fields.
 * 
 * Comment:   Automatically generated on Fri Apr 07 09:16:03 PDT 2006
 *            by parsing the MPEG-4 node templates files.
 *            Contact: mikael@mindego.com
 *
 **************************************************************************/
#include "float.h"
#include "StreamCo.h"
#include "NodeFld.h"
#include "bifs.h"
#include "Root.h"
#include "Conditional.h"

#if !defined(_Nodes_h)  // Sentry, use file only if it's not already included.
#define _Nodes_h

#ifndef PI
#define PI		3.14159265358979323846
#endif

#pragma warning(disable: 4305)  // truncation from 'const double' to 'const float'

/* Currently, we have 183 nodes defined. */

class Anchor : public StreamConsumer
{
         BIFS_DECLARE_NODE

         eventIn MFNodeType<SF3DNode> addChildren;
         eventIn MFNodeType<SF3DNode> removeChildren;
         exposedField MFNodeType<SF3DNode> children;
         exposedField SFString description;
         exposedField MF<ZString> parameter;
         eventIn SF<Bool> activate;
         void OnaddChildren();
         void OnremoveChildren();
         void Onactivate();

         public:
              Anchor() {
              }
};

class AnimationStream : public StreamConsumer
{
         BIFS_DECLARE_NODE

         exposedField SF<Bool> loop;
         exposedField SF<float> speed;
         exposedField SFTimeField startTime;
         exposedField SFTimeField stopTime;
         eventOut SF<Time> duration_changed;
         eventOut SF<Bool> isActive;

         public:
              AnimationStream() {
                   loop = FALSE;
                   speed = 1;
                   startTime = 0;
                   stopTime = 0;
              }
};

class Appearance : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SFMaterialNode material;
         exposedField SFTextureNode texture;
         exposedField SFTextureTransformNode textureTransform;

         public:
              Appearance() {
              }
};

class AudioBuffer : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<Bool> loop;
         exposedField SF<float> pitch;
         exposedField SFTimeField startTime;
         exposedField SFTimeField stopTime;
         exposedField MFNodeType<SFAudioNode> children;
         exposedField SF<int> numChan;
         exposedField MF<int> phaseGroup;
         exposedField SF<float> length;
         eventOut SF<Time> duration_changed;
         eventOut SF<Bool> isActive;

         public:
              AudioBuffer() {
                   loop = FALSE;
                   pitch = 1;
                   startTime = 0;
                   stopTime = 0;
                   numChan = 1;
                   phaseGroup += 1;
                   length = 0;
              }
};

class AudioClip : public StreamConsumer
{
         BIFS_DECLARE_NODE

         exposedField SFString description;
         exposedField SF<Bool> loop;
         exposedField SF<float> pitch;
         exposedField SFTimeField startTime;
         exposedField SFTimeField stopTime;
         eventOut SF<Time> duration_changed;
         eventOut SF<Bool> isActive;

         public:
              AudioClip() {
                   loop = FALSE;
                   pitch = 1;
                   startTime = 0;
                   stopTime = 0;
              }
};

class AudioDelay : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn MFNodeType<SFAudioNode> addChildren;
         eventIn MFNodeType<SFAudioNode> removeChildren;
         exposedField MFNodeType<SFAudioNode> children;
         exposedField SF<Time> delay;
         FIELD SF<int> numChan;
         FIELD MF<int> phaseGroup;
         void OnaddChildren();
         void OnremoveChildren();

         public:
              AudioDelay() {
                   delay = 0;
                   numChan = 1;
              }
};

class AudioFX : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn MFNodeType<SFAudioNode> addChildren;
         eventIn MFNodeType<SFAudioNode> removeChildren;
         exposedField MFNodeType<SFAudioNode> children;
         exposedField SFString orch;
         exposedField SFString score;
         exposedField MF<float> params;
         FIELD SF<int> numChan;
         FIELD MF<int> phaseGroup;
         void OnaddChildren();
         void OnremoveChildren();

         public:
              AudioFX() {
                   numChan = 1;
              }
};

class AudioMix : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn MFNodeType<SFAudioNode> addChildren;
         eventIn MFNodeType<SFAudioNode> removeChildren;
         exposedField MFNodeType<SFAudioNode> children;
         exposedField SF<int> numInputs;
         exposedField MF<float> matrix;
         FIELD SF<int> numChan;
         FIELD MF<int> phaseGroup;
         void OnaddChildren();
         void OnremoveChildren();

         public:
              AudioMix() {
                   numInputs = 1;
                   numChan = 1;
              }
};

class AudioSource : public StreamConsumer
{
         BIFS_DECLARE_NODE

         eventIn MFNodeType<SFAudioNode> addChildren;
         eventIn MFNodeType<SFAudioNode> removeChildren;
         exposedField MFNodeType<SFAudioNode> children;
         exposedField SF<float> pitch;
         exposedField SF<float> speed;
         exposedField SFTimeField startTime;
         exposedField SFTimeField stopTime;
         FIELD SF<int> numChan;
         FIELD MF<int> phaseGroup;
         void OnaddChildren();
         void OnremoveChildren();

         public:
              AudioSource() {
                   pitch = 1;
                   speed = 1;
                   startTime = 0;
                   stopTime = 0;
                   numChan = 1;
              }
};

class AudioSwitch : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn MFNodeType<SFAudioNode> addChildren;
         eventIn MFNodeType<SFAudioNode> removeChildren;
         exposedField MFNodeType<SFAudioNode> children;
         exposedField MF<int> whichChoice;
         FIELD SF<int> numChan;
         FIELD MF<int> phaseGroup;
         void OnaddChildren();
         void OnremoveChildren();

         public:
              AudioSwitch() {
                   numChan = 1;
              }
};

class Background : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn SF<Bool> set_bind;
         exposedField MF<float> groundAngle;
         exposedField MF<Color3f> groundColor;
         exposedField MF<Url> backUrl;
         exposedField MF<Url> bottomUrl;
         exposedField MF<Url> frontUrl;
         exposedField MF<Url> leftUrl;
         exposedField MF<Url> rightUrl;
         exposedField MF<Url> topUrl;
         exposedField MF<float> skyAngle;
         exposedField MF<Color3f> skyColor;
         eventOut SF<Bool> isBound;
         void Onset_bind();

         public:
              Background() {
                   skyColor += 0;
                   skyColor += 0;
                   skyColor += 0;
              }
};

class Background2D : public StreamConsumer
{
         BIFS_DECLARE_NODE

         eventIn SF<Bool> set_bind;
         exposedField SF<Color3f> backColor;
         eventOut SF<Bool> isBound;
         void Onset_bind();

         public:
              Background2D() {
                   backColor = Color3f(0, 0, 0);
              }
};

class Billboard : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn MFNodeType<SF3DNode> addChildren;
         eventIn MFNodeType<SF3DNode> removeChildren;
         exposedField MFNodeType<SF3DNode> children;
         exposedField SF<Vec3f> axisOfRotation;
         void OnaddChildren();
         void OnremoveChildren();

         public:
              Billboard() {
                   axisOfRotation = Vec3f(0, 1, 0);
              }
};

class Bitmap : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<Vec2f> scale;

         public:
              Bitmap() {
                   scale = Vec2f(-1, -1);
              }
};

class Box : public MediaObject
{
         BIFS_DECLARE_NODE

         FIELD SF<Vec3f> size;

         public:
              Box() {
                   size = Vec3f(2, 2, 2);
              }
};

class Circle : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<float> radius;

         public:
              Circle() {
                   radius = 1;
              }
};

class Collision : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn MFNodeType<SF3DNode> addChildren;
         eventIn MFNodeType<SF3DNode> removeChildren;
         exposedField MFNodeType<SF3DNode> children;
         exposedField SF<Bool> collide;
         FIELD SF3DNode proxy;
         eventOut SF<Time> collideTime;
         void OnaddChildren();
         void OnremoveChildren();

         public:
              Collision() {
                   collide = TRUE;
              }
};

class Color : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField MF<Color3f> color;

         public:
              Color() {
              }
};

class ColorInterpolator : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn SF<float> set_fraction;
         exposedField MF<float> key;
         exposedField MF<Color3f> keyValue;
         eventOut SF<Color3f> value_changed;
         void Onset_fraction();

         public:
              ColorInterpolator() {
              }
};

class CompositeTexture2D : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn MFNodeType<SF2DNode> addChildren;
         eventIn MFNodeType<SF2DNode> removeChildren;
         exposedField MFNodeType<SF2DNode> children;
         exposedField SF<int> pixelWidth;
         exposedField SF<int> pixelHeight;
         exposedField SFNode<class Background2D> background;
         exposedField SFNode<class Viewport> viewport;
         FIELD SF<int> repeatSandT;
         void OnaddChildren();
         void OnremoveChildren();

         public:
              CompositeTexture2D() {
                   pixelWidth = -1;
                   pixelHeight = -1;
                   repeatSandT = 3;
              }
};

class CompositeTexture3D : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn MFNodeType<SF3DNode> addChildren;
         eventIn MFNodeType<SF3DNode> removeChildren;
         exposedField MFNodeType<SF3DNode> children;
         exposedField SF<int> pixelWidth;
         exposedField SF<int> pixelHeight;
         exposedField SFNode<class Background> background;
         exposedField SFNode<class Fog> fog;
         exposedField SFNode<class NavigationInfo> navigationInfo;
         exposedField SFNode<class Viewpoint> viewpoint;
         FIELD SF<Bool> repeatS;
         FIELD SF<Bool> repeatT;
         void OnaddChildren();
         void OnremoveChildren();

         public:
              CompositeTexture3D() {
                   pixelWidth = -1;
                   pixelHeight = -1;
                   repeatS = TRUE;
                   repeatT = TRUE;
              }
};

class Conditional : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn SF<Bool> activate;
         eventIn SF<Bool> reverseActivate;
         exposedField SFConditionalString buffer;
         eventOut SF<Bool> isActive;
         void Onactivate();
         void OnreverseActivate();

         public:
              Conditional() {
                   reverseActivate = FALSE;
              }
};

class Cone : public MediaObject
{
         BIFS_DECLARE_NODE

         FIELD SF<float> bottomRadius;
         FIELD SF<float> height;
         FIELD SF<Bool> side;
         FIELD SF<Bool> bottom;

         public:
              Cone() {
                   bottomRadius = 1;
                   height = 2;
                   side = TRUE;
                   bottom = TRUE;
              }
};

class Coordinate : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField MF<Vec3f> point;

         public:
              Coordinate() {
              }
};

class Coordinate2D : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField MF<Vec2f> point;

         public:
              Coordinate2D() {
              }
};

class CoordinateInterpolator : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn SF<float> set_fraction;
         exposedField MF<float> key;
         exposedField MF<Vec3f> keyValue;
         eventOut MF<Vec3f> value_changed;
         void Onset_fraction();

         public:
              CoordinateInterpolator() {
              }
};

class CoordinateInterpolator2D : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn SF<float> set_fraction;
         exposedField MF<float> key;
         exposedField MF<Vec2f> keyValue;
         eventOut MF<Vec2f> value_changed;
         void Onset_fraction();

         public:
              CoordinateInterpolator2D() {
              }
};

class Curve2D : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SFNode<class Coordinate2D> point;
         exposedField SF<float> fineness;
         exposedField MF<int> type;

         public:
              Curve2D() {
                   fineness = 0.5;
              }
};

class Cylinder : public MediaObject
{
         BIFS_DECLARE_NODE

         FIELD SF<Bool> bottom;
         FIELD SF<float> height;
         FIELD SF<float> radius;
         FIELD SF<Bool> side;
         FIELD SF<Bool> top;

         public:
              Cylinder() {
                   bottom = TRUE;
                   height = 2;
                   radius = 1;
                   side = TRUE;
                   top = TRUE;
              }
};

class CylinderSensor : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<Bool> autoOffset;
         exposedField SF<float> diskAngle;
         exposedField SF<Bool> enabled;
         exposedField SF<float> maxAngle;
         exposedField SF<float> minAngle;
         exposedField SF<float> offset;
         eventOut SF<Bool> isActive;
         eventOut SF<Rotation> rotation_changed;
         eventOut SF<Vec3f> trackPoint_changed;

         public:
              CylinderSensor() {
                   autoOffset = TRUE;
                   diskAngle = 0.262;
                   enabled = TRUE;
                   maxAngle = -1;
                   minAngle = 0;
                   offset = 0;
              }
};

class DirectionalLight : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<float> ambientIntensity;
         exposedField SF<Color3f> color;
         exposedField SF<Vec3f> direction;
         exposedField SF<float> intensity;
         exposedField SF<Bool> on;

         public:
              DirectionalLight() {
                   ambientIntensity = 0;
                   color = Color3f(1, 1, 1);
                   direction = Vec3f(0, 0, -1);
                   intensity = 1;
                   on = TRUE;
              }
};

class DiscSensor : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<Bool> autoOffset;
         exposedField SF<Bool> enabled;
         exposedField SF<float> maxAngle;
         exposedField SF<float> minAngle;
         exposedField SF<float> offset;
         eventOut SF<Bool> isActive;
         eventOut SF<float> rotation_changed;
         eventOut SF<Vec2f> trackPoint_changed;

         public:
              DiscSensor() {
                   autoOffset = TRUE;
                   enabled = TRUE;
                   maxAngle = -1;
                   minAngle = 0;
                   offset = 0;
              }
};

class ElevationGrid : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn MF<float> set_height;
         exposedField SFNode<class Color> color;
         exposedField SFNode<class Normal> normal;
         exposedField SFTextureCoordinateNode texCoord;
         FIELD MF<float> height;
         FIELD SF<Bool> ccw;
         FIELD SF<Bool> colorPerVertex;
         FIELD SF<float> creaseAngle;
         FIELD SF<Bool> normalPerVertex;
         FIELD SF<Bool> solid;
         FIELD SF<int> xDimension;
         FIELD SF<float> xSpacing;
         FIELD SF<int> zDimension;
         FIELD SF<float> zSpacing;
         void Onset_height();

         public:
              ElevationGrid() {
                   ccw = TRUE;
                   colorPerVertex = TRUE;
                   creaseAngle = 0;
                   normalPerVertex = TRUE;
                   solid = TRUE;
                   xDimension = 0;
                   xSpacing = 1;
                   zDimension = 0;
                   zSpacing = 1;
              }
};

class Expression : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<int> expression_select1;
         exposedField SF<int> expression_intensity1;
         exposedField SF<int> expression_select2;
         exposedField SF<int> expression_intensity2;
         exposedField SF<Bool> init_face;
         exposedField SF<Bool> expression_def;

         public:
              Expression() {
                   expression_select1 = 0;
                   expression_intensity1 = 0;
                   expression_select2 = 0;
                   expression_intensity2 = 0;
                   init_face = FALSE;
                   expression_def = FALSE;
              }
};

class Extrusion : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn MF<Vec2f> set_crossSection;
         eventIn MF<Rotation> set_orientation;
         eventIn MF<Vec2f> set_scale;
         eventIn MF<Vec3f> set_spine;
         FIELD SF<Bool> beginCap;
         FIELD SF<Bool> ccw;
         FIELD SF<Bool> convex;
         FIELD SF<float> creaseAngle;
         FIELD MF<Vec2f> crossSection;
         FIELD SF<Bool> endCap;
         FIELD MF<Rotation> orientation;
         FIELD MF<Vec2f> scale;
         FIELD SF<Bool> solid;
         FIELD MF<Vec3f> spine;
         void Onset_crossSection();
         void Onset_orientation();
         void Onset_scale();
         void Onset_spine();

         public:
              Extrusion() {
                   beginCap = TRUE;
                   ccw = TRUE;
                   convex = TRUE;
                   creaseAngle = 0;
                   crossSection += 1;
                   crossSection += 1;
                   crossSection += -1;
                   crossSection += -1;
                   crossSection += -1;
                   crossSection += -1;
                   crossSection += 1;
                   crossSection += 1;
                   crossSection += 1;
                   crossSection += 1;
                   endCap = TRUE;
                   orientation += 0;
                   orientation += 0;
                   orientation += 1;
                   orientation += 0;
                   scale += 1;
                   scale += 1;
                   solid = TRUE;
                   spine += 0;
                   spine += 0;
                   spine += 0;
                   spine += 1;
                   spine += 1;
                   spine += 0;
              }
};

class Face : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SFNode<class FAP> fap;
         exposedField SFNode<class FDP> fdp;
         exposedField SFNode<class FIT> fit;
         exposedField SFAudioNode ttsSource;
         exposedField MFNodeType<SF3DNode> renderedFace;

         public:
              Face() {
              }
};

class FaceDefMesh : public MediaObject
{
         BIFS_DECLARE_NODE

         FIELD SF3DNode faceSceneGraphNode;
         FIELD MF<int> intervalBorders;
         FIELD MF<int> coordIndex;
         FIELD MF<Vec3f> displacements;

         public:
              FaceDefMesh() {
              }
};

class FaceDefTables : public MediaObject
{
         BIFS_DECLARE_NODE

         FIELD SF<int> fapID;
         FIELD SF<int> highLevelSelect;
         exposedField MFNode<class FaceDefMesh> faceDefMesh;
         exposedField MFNode<class FaceDefTransform> faceDefTransform;

         public:
              FaceDefTables() {
                   fapID = 1;
                   highLevelSelect = 1;
              }
};

class FaceDefTransform : public MediaObject
{
         BIFS_DECLARE_NODE

         FIELD SF3DNode faceSceneGraphNode;
         FIELD SF<int> fieldId;
         FIELD SF<Rotation> rotationDef;
         FIELD SF<Vec3f> scaleDef;
         FIELD SF<Vec3f> translationDef;

         public:
              FaceDefTransform() {
                   fieldId = 1;
                   rotationDef = Rotation(0, 0, 1, 0);
                   scaleDef = Vec3f(1, 1, 1);
                   translationDef = Vec3f(0, 0, 0);
              }
};

class FAP : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SFNode<class Viseme> viseme;
         exposedField SFNode<class Expression> expression;
         exposedField SF<int> open_jaw;
         exposedField SF<int> lower_t_midlip;
         exposedField SF<int> raise_b_midlip;
         exposedField SF<int> stretch_l_corner;
         exposedField SF<int> stretch_r_corner;
         exposedField SF<int> lower_t_lip_lm;
         exposedField SF<int> lower_t_lip_rm;
         exposedField SF<int> lower_b_lip_lm;
         exposedField SF<int> lower_b_lip_rm;
         exposedField SF<int> raise_l_cornerlip;
         exposedField SF<int> raise_r_cornerlip;
         exposedField SF<int> thrust_jaw;
         exposedField SF<int> shift_jaw;
         exposedField SF<int> push_b_lip;
         exposedField SF<int> push_t_lip;
         exposedField SF<int> depress_chin;
         exposedField SF<int> close_t_l_eyelid;
         exposedField SF<int> close_t_r_eyelid;
         exposedField SF<int> close_b_l_eyelid;
         exposedField SF<int> close_b_r_eyelid;
         exposedField SF<int> yaw_l_eyeball;
         exposedField SF<int> yaw_r_eyeball;
         exposedField SF<int> pitch_l_eyeball;
         exposedField SF<int> pitch_r_eyeball;
         exposedField SF<int> thrust_l_eyeball;
         exposedField SF<int> thrust_r_eyeball;
         exposedField SF<int> dilate_l_pupil;
         exposedField SF<int> dilate_r_pupil;
         exposedField SF<int> raise_l_i_eyebrow;
         exposedField SF<int> raise_r_i_eyebrow;
         exposedField SF<int> raise_l_m_eyebrow;
         exposedField SF<int> raise_r_m_eyebrow;
         exposedField SF<int> raise_l_o_eyebrow;
         exposedField SF<int> raise_r_o_eyebrow;
         exposedField SF<int> squeeze_l_eyebrow;
         exposedField SF<int> squeeze_r_eyebrow;
         exposedField SF<int> puff_l_cheek;
         exposedField SF<int> puff_r_cheek;
         exposedField SF<int> lift_l_cheek;
         exposedField SF<int> lift_r_cheek;
         exposedField SF<int> shift_tongue_tip;
         exposedField SF<int> raise_tongue_tip;
         exposedField SF<int> thrust_tongue_tip;
         exposedField SF<int> raise_tongue;
         exposedField SF<int> tongue_roll;
         exposedField SF<int> head_pitch;
         exposedField SF<int> head_yaw;
         exposedField SF<int> head_roll;
         exposedField SF<int> lower_t_midlip_o;
         exposedField SF<int> raise_b_midlip_o;
         exposedField SF<int> stretch_l_cornerlip;
         exposedField SF<int> stretch_r_cornerlip;
         exposedField SF<int> lower_t_lip_lm_o;
         exposedField SF<int> lower_t_lip_rm_o;
         exposedField SF<int> raise_b_lip_lm_o;
         exposedField SF<int> raise_b_lip_rm_o;
         exposedField SF<int> raise_l_cornerlip_o;
         exposedField SF<int> raise_r_cornerlip_o;
         exposedField SF<int> stretch_l_nose;
         exposedField SF<int> stretch_r_nose;
         exposedField SF<int> raise_nose;
         exposedField SF<int> bend_nose;
         exposedField SF<int> raise_l_ear;
         exposedField SF<int> raise_r_ear;
         exposedField SF<int> pull_l_ear;
         exposedField SF<int> pull_r_ear;

         public:
              FAP() {
                   open_jaw = +INT_MAX;
                   lower_t_midlip = +INT_MAX;
                   raise_b_midlip = +INT_MAX;
                   stretch_l_corner = +INT_MAX;
                   stretch_r_corner = +INT_MAX;
                   lower_t_lip_lm = +INT_MAX;
                   lower_t_lip_rm = +INT_MAX;
                   lower_b_lip_lm = +INT_MAX;
                   lower_b_lip_rm = +INT_MAX;
                   raise_l_cornerlip = +INT_MAX;
                   raise_r_cornerlip = +INT_MAX;
                   thrust_jaw = +INT_MAX;
                   shift_jaw = +INT_MAX;
                   push_b_lip = +INT_MAX;
                   push_t_lip = +INT_MAX;
                   depress_chin = +INT_MAX;
                   close_t_l_eyelid = +INT_MAX;
                   close_t_r_eyelid = +INT_MAX;
                   close_b_l_eyelid = +INT_MAX;
                   close_b_r_eyelid = +INT_MAX;
                   yaw_l_eyeball = +INT_MAX;
                   yaw_r_eyeball = +INT_MAX;
                   pitch_l_eyeball = +INT_MAX;
                   pitch_r_eyeball = +INT_MAX;
                   thrust_l_eyeball = +INT_MAX;
                   thrust_r_eyeball = +INT_MAX;
                   dilate_l_pupil = +INT_MAX;
                   dilate_r_pupil = +INT_MAX;
                   raise_l_i_eyebrow = +INT_MAX;
                   raise_r_i_eyebrow = +INT_MAX;
                   raise_l_m_eyebrow = +INT_MAX;
                   raise_r_m_eyebrow = +INT_MAX;
                   raise_l_o_eyebrow = +INT_MAX;
                   raise_r_o_eyebrow = +INT_MAX;
                   squeeze_l_eyebrow = +INT_MAX;
                   squeeze_r_eyebrow = +INT_MAX;
                   puff_l_cheek = +INT_MAX;
                   puff_r_cheek = +INT_MAX;
                   lift_l_cheek = +INT_MAX;
                   lift_r_cheek = +INT_MAX;
                   shift_tongue_tip = +INT_MAX;
                   raise_tongue_tip = +INT_MAX;
                   thrust_tongue_tip = +INT_MAX;
                   raise_tongue = +INT_MAX;
                   tongue_roll = +INT_MAX;
                   head_pitch = +INT_MAX;
                   head_yaw = +INT_MAX;
                   head_roll = +INT_MAX;
                   lower_t_midlip_o = +INT_MAX;
                   raise_b_midlip_o = +INT_MAX;
                   stretch_l_cornerlip = +INT_MAX;
                   stretch_r_cornerlip = +INT_MAX;
                   lower_t_lip_lm_o = +INT_MAX;
                   lower_t_lip_rm_o = +INT_MAX;
                   raise_b_lip_lm_o = +INT_MAX;
                   raise_b_lip_rm_o = +INT_MAX;
                   raise_l_cornerlip_o = +INT_MAX;
                   raise_r_cornerlip_o = +INT_MAX;
                   stretch_l_nose = +INT_MAX;
                   stretch_r_nose = +INT_MAX;
                   raise_nose = +INT_MAX;
                   bend_nose = +INT_MAX;
                   raise_l_ear = +INT_MAX;
                   raise_r_ear = +INT_MAX;
                   pull_l_ear = +INT_MAX;
                   pull_r_ear = +INT_MAX;
              }
};

class FDP : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SFNode<class Coordinate> featurePointsCoord;
         exposedField SFTextureCoordinateNode textureCoord;
         exposedField MFNode<class FaceDefTables> faceDefTables;
         exposedField MFNodeType<SF3DNode> faceSceneGraph;
         FIELD SF<Bool> useOrthoTexture;

         public:
              FDP() {
                   useOrthoTexture = FALSE;
              }
};

class FIT : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField MF<int> FAPs;
         exposedField MF<int> Graph;
         exposedField MF<int> numeratorExp;
         exposedField MF<int> denominatorExp;
         exposedField MF<int> numeratorImpulse;
         exposedField MF<int> numeratorTerms;
         exposedField MF<int> denominatorTerms;
         exposedField MF<float> numeratorCoefs;
         exposedField MF<float> denominatorCoefs;

         public:
              FIT() {
              }
};

class Fog : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<Color3f> color;
         exposedField SFString fogType;
         exposedField SF<float> visibilityRange;
         eventIn SF<Bool> set_bind;
         eventOut SF<Bool> isBound;
         void Onset_bind();

         public:
              Fog() {
                   color = Color3f(1, 1, 1);
                   fogType = ZString("LINEAR");
                   visibilityRange = 0;
              }
};

class FontStyle : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField MF<ZString> family;
         exposedField SF<Bool> horizontal;
         exposedField MF<ZString> justify;
         exposedField SFString language;
         exposedField SF<Bool> leftToRight;
         exposedField SF<float> size;
         exposedField SF<float> spacing;
         exposedField SFString style;
         exposedField SF<Bool> topToBottom;

         public:
              FontStyle() {
                   family += ZString("SERIF");
                   horizontal = TRUE;
                   justify += ZString("BEGIN");
                   leftToRight = TRUE;
                   size = 1;
                   spacing = 1;
                   style = ZString("PLAIN");
                   topToBottom = TRUE;
              }
};

class Form : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn MFNodeType<SF2DNode> addChildren;
         eventIn MFNodeType<SF2DNode> removeChildren;
         exposedField MFNodeType<SF2DNode> children;
         exposedField SF<Vec2f> size;
         exposedField MF<int> groups;
         exposedField MF<ZString> constraints;
         exposedField MF<int> groupsIndex;
         void OnaddChildren();
         void OnremoveChildren();

         public:
              Form() {
                   size = Vec2f(-1, -1);
              }
};

class Group : public Root
{
         BIFS_DECLARE_NODE

         eventIn MFNodeType<SF3DNode> addChildren;
         eventIn MFNodeType<SF3DNode> removeChildren;
         exposedField MFNodeType<SF3DNode> children;
         void OnaddChildren();
         void OnremoveChildren();

         public:
              Group() {
              }
};

class ImageTexture : public StreamConsumer
{
         BIFS_DECLARE_NODE

         FIELD SF<Bool> repeatS;
         FIELD SF<Bool> repeatT;

         public:
              ImageTexture() {
                   repeatS = TRUE;
                   repeatT = TRUE;
              }
};

class IndexedFaceSet : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn MF<int> set_colorIndex;
         eventIn MF<int> set_coordIndex;
         eventIn MF<int> set_normalIndex;
         eventIn MF<int> set_texCoordIndex;
         exposedField SFNode<class Color> color;
         exposedField SFNode<class Coordinate> coord;
         exposedField SFNode<class Normal> normal;
         exposedField SFTextureCoordinateNode texCoord;
         FIELD SF<Bool> ccw;
         FIELD MF<int> colorIndex;
         FIELD SF<Bool> colorPerVertex;
         FIELD SF<Bool> convex;
         FIELD MF<int> coordIndex;
         FIELD SF<float> creaseAngle;
         FIELD MF<int> normalIndex;
         FIELD SF<Bool> normalPerVertex;
         FIELD SF<Bool> solid;
         FIELD MF<int> texCoordIndex;
         void Onset_colorIndex();
         void Onset_coordIndex();
         void Onset_normalIndex();
         void Onset_texCoordIndex();

         public:
              IndexedFaceSet() {
                   ccw = TRUE;
                   colorPerVertex = TRUE;
                   convex = TRUE;
                   creaseAngle = 0;
                   normalPerVertex = TRUE;
                   solid = TRUE;
              }
};

class IndexedFaceSet2D : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn MF<int> set_colorIndex;
         eventIn MF<int> set_coordIndex;
         eventIn MF<int> set_texCoordIndex;
         exposedField SFNode<class Color> color;
         exposedField SFNode<class Coordinate2D> coord;
         exposedField SFTextureCoordinateNode texCoord;
         FIELD MF<int> colorIndex;
         FIELD SF<Bool> colorPerVertex;
         FIELD SF<Bool> convex;
         FIELD MF<int> coordIndex;
         FIELD MF<int> texCoordIndex;
         void Onset_colorIndex();
         void Onset_coordIndex();
         void Onset_texCoordIndex();

         public:
              IndexedFaceSet2D() {
                   colorPerVertex = TRUE;
                   convex = TRUE;
              }
};

class IndexedLineSet : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn MF<int> set_colorIndex;
         eventIn MF<int> set_coordIndex;
         exposedField SFNode<class Color> color;
         exposedField SFNode<class Coordinate> coord;
         FIELD MF<int> colorIndex;
         FIELD SF<Bool> colorPerVertex;
         FIELD MF<int> coordIndex;
         void Onset_colorIndex();
         void Onset_coordIndex();

         public:
              IndexedLineSet() {
                   colorPerVertex = TRUE;
              }
};

class IndexedLineSet2D : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn MF<int> set_colorIndex;
         eventIn MF<int> set_coordIndex;
         exposedField SFNode<class Color> color;
         exposedField SFNode<class Coordinate2D> coord;
         FIELD MF<int> colorIndex;
         FIELD SF<Bool> colorPerVertex;
         FIELD MF<int> coordIndex;
         void Onset_colorIndex();
         void Onset_coordIndex();

         public:
              IndexedLineSet2D() {
                   colorPerVertex = TRUE;
              }
};

class Inline : public StreamConsumer
{
         BIFS_DECLARE_NODE


         public:
              Inline() {
              }
};

class LOD : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField MFNodeType<SF3DNode> level;
         FIELD SF<Vec3f> center;
         FIELD MF<float> range;

         public:
              LOD() {
                   center = Vec3f(0, 0, 0);
              }
};

class Layer2D : public Root
{
         BIFS_DECLARE_NODE

         eventIn MFNodeType<SF2DNode> addChildren;
         eventIn MFNodeType<SF2DNode> removeChildren;
         exposedField MFNodeType<SF2DNode> children;
         exposedField SF<Vec2f> size;
         exposedField SFNode<class Background2D> background;
         exposedField SFNode<class Viewport> viewport;
         void OnaddChildren();
         void OnremoveChildren();

         public:
              Layer2D() {
                   size = Vec2f(-1, -1);
              }
};

class Layer3D : public Root
{
         BIFS_DECLARE_NODE

         eventIn MFNodeType<SF3DNode> addChildren;
         eventIn MFNodeType<SF3DNode> removeChildren;
         exposedField MFNodeType<SF3DNode> children;
         exposedField SF<Vec2f> size;
         exposedField SFNode<class Background> background;
         exposedField SFNode<class Fog> fog;
         exposedField SFNode<class NavigationInfo> navigationInfo;
         exposedField SFNode<class Viewpoint> viewpoint;
         void OnaddChildren();
         void OnremoveChildren();

         public:
              Layer3D() {
                   size = Vec2f(-1, -1);
              }
};

class Layout : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn MFNodeType<SF2DNode> addChildren;
         eventIn MFNodeType<SF2DNode> removeChildren;
         exposedField MFNodeType<SF2DNode> children;
         exposedField SF<Bool> wrap;
         exposedField SF<Vec2f> size;
         exposedField SF<Bool> horizontal;
         exposedField MF<ZString> justify;
         exposedField SF<Bool> leftToRight;
         exposedField SF<Bool> topToBottom;
         exposedField SF<float> spacing;
         exposedField SF<Bool> smoothScroll;
         exposedField SF<Bool> loop;
         exposedField SF<Bool> scrollVertical;
         exposedField SF<float> scrollRate;
         exposedField SF<int> scrollMode;
         void OnaddChildren();
         void OnremoveChildren();

         public:
              Layout() {
                   wrap = FALSE;
                   size = Vec2f(-1, -1);
                   horizontal = TRUE;
                   justify += ZString("BEGIN");
                   leftToRight = TRUE;
                   topToBottom = TRUE;
                   spacing = 1;
                   smoothScroll = FALSE;
                   loop = FALSE;
                   scrollVertical = TRUE;
                   scrollRate = 0;
                   scrollMode = 0;
              }
};

class LineProperties : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<Color3f> lineColor;
         exposedField SF<int> lineStyle;
         exposedField SF<float> width;

         public:
              LineProperties() {
                   lineColor = Color3f(0, 0, 0);
                   lineStyle = 0;
                   width = 1;
              }
};

class ListeningPoint : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn SF<Bool> set_bind;
         exposedField SF<Bool> jump;
         exposedField SF<Rotation> orientation;
         exposedField SF<Vec3f> position;
         FIELD SFString description;
         eventOut SF<Time> bindTime;
         eventOut SF<Bool> isBound;
         void Onset_bind();

         public:
              ListeningPoint() {
                   jump = TRUE;
                   orientation = Rotation(0, 0, 1, 0);
                   position = Vec3f(0, 0, 10);
              }
};

class Material : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<float> ambientIntensity;
         exposedField SF<Color3f> diffuseColor;
         exposedField SF<Color3f> emissiveColor;
         exposedField SF<float> shininess;
         exposedField SF<Color3f> specularColor;
         exposedField SF<float> transparency;

         public:
              Material() {
                   ambientIntensity = 0.2;
                   diffuseColor = Color3f(0.8, 0.8, 0.8);
                   emissiveColor = Color3f(0, 0, 0);
                   shininess = 0.2;
                   specularColor = Color3f(0, 0, 0);
                   transparency = 0;
              }
};

class Material2D : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<Color3f> emissiveColor;
         exposedField SF<Bool> filled;
         exposedField SFLinePropertiesNode lineProps;
         exposedField SF<float> transparency;

         public:
              Material2D() {
                   emissiveColor = Color3f(0.8, 0.8, 0.8);
                   filled = FALSE;
                   transparency = 0;
              }
};

class MovieTexture : public StreamConsumer
{
         BIFS_DECLARE_NODE

         exposedField SF<Bool> loop;
         exposedField SF<float> speed;
         exposedField SFTimeField startTime;
         exposedField SFTimeField stopTime;
         FIELD SF<Bool> repeatS;
         FIELD SF<Bool> repeatT;
         eventOut SF<Time> duration_changed;
         eventOut SF<Bool> isActive;

         public:
              MovieTexture() {
                   loop = FALSE;
                   speed = 1;
                   startTime = 0;
                   stopTime = 0;
                   repeatS = TRUE;
                   repeatT = TRUE;
              }
};

class NavigationInfo : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn SF<Bool> set_bind;
         exposedField MF<float> avatarSize;
         exposedField SF<Bool> headlight;
         exposedField SF<float> speed;
         exposedField MF<ZString> type;
         exposedField SF<float> visibilityLimit;
         eventOut SF<Bool> isBound;
         void Onset_bind();

         public:
              NavigationInfo() {
                   avatarSize += 0.25;
                   avatarSize += 0.75;
                   avatarSize += 0.75;
                   headlight = TRUE;
                   speed = 1;
                   type += ZString("WALK");
                   visibilityLimit = 0;
              }
};

class Normal : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField MF<Vec3f> vector;

         public:
              Normal() {
              }
};

class NormalInterpolator : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn SF<float> set_fraction;
         exposedField MF<float> key;
         exposedField MF<Vec3f> keyValue;
         eventOut MF<Vec3f> value_changed;
         void Onset_fraction();

         public:
              NormalInterpolator() {
              }
};

class OrderedGroup : public Root
{
         BIFS_DECLARE_NODE

         eventIn MFNodeType<SF3DNode> addChildren;
         eventIn MFNodeType<SF3DNode> removeChildren;
         exposedField MFNodeType<SF3DNode> children;
         exposedField MF<float> order;
         void OnaddChildren();
         void OnremoveChildren();

         public:
              OrderedGroup() {
              }
};

class OrientationInterpolator : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn SF<float> set_fraction;
         exposedField MF<float> key;
         exposedField MF<Rotation> keyValue;
         eventOut SF<Rotation> value_changed;
         void Onset_fraction();

         public:
              OrientationInterpolator() {
              }
};

class PixelTexture : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<Image> image;
         FIELD SF<Bool> repeatS;
         FIELD SF<Bool> repeatT;

         public:
              PixelTexture() {
                   image = Image(0, 0, 0);
                   repeatS = TRUE;
                   repeatT = TRUE;
              }
};

class PlaneSensor : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<Bool> autoOffset;
         exposedField SF<Bool> enabled;
         exposedField SF<Vec2f> maxPosition;
         exposedField SF<Vec2f> minPosition;
         exposedField SF<Vec3f> offset;
         eventOut SF<Bool> isActive;
         eventOut SF<Vec3f> trackPoint_changed;
         eventOut SF<Vec3f> translation_changed;

         public:
              PlaneSensor() {
                   autoOffset = TRUE;
                   enabled = TRUE;
                   maxPosition = Vec2f(-1, -1);
                   minPosition = Vec2f(0, 0);
                   offset = Vec3f(0, 0, 0);
              }
};

class PlaneSensor2D : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<Bool> autoOffset;
         exposedField SF<Bool> enabled;
         exposedField SF<Vec2f> maxPosition;
         exposedField SF<Vec2f> minPosition;
         exposedField SF<Vec2f> offset;
         eventOut SF<Bool> isActive;
         eventOut SF<Vec2f> trackPoint_changed;
         eventOut SF<Vec2f> translation_changed;

         public:
              PlaneSensor2D() {
                   autoOffset = TRUE;
                   enabled = TRUE;
                   maxPosition = Vec2f(0, 0);
                   minPosition = Vec2f(0, 0);
                   offset = Vec2f(0, 0);
              }
};

class PointLight : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<float> ambientIntensity;
         exposedField SF<Vec3f> attenuation;
         exposedField SF<Color3f> color;
         exposedField SF<float> intensity;
         exposedField SF<Vec3f> location;
         exposedField SF<Bool> on;
         exposedField SF<float> radius;

         public:
              PointLight() {
                   ambientIntensity = 0;
                   attenuation = Vec3f(1, 0, 0);
                   color = Color3f(1, 1, 1);
                   intensity = 1;
                   location = Vec3f(0, 0, 0);
                   on = TRUE;
                   radius = 100;
              }
};

class PointSet : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SFNode<class Color> color;
         exposedField SFNode<class Coordinate> coord;

         public:
              PointSet() {
              }
};

class PointSet2D : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SFNode<class Color> color;
         exposedField SFNode<class Coordinate2D> coord;

         public:
              PointSet2D() {
              }
};

class PositionInterpolator : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn SF<float> set_fraction;
         exposedField MF<float> key;
         exposedField MF<Vec3f> keyValue;
         eventOut SF<Vec3f> value_changed;
         void Onset_fraction();

         public:
              PositionInterpolator() {
              }
};

class PositionInterpolator2D : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn SF<float> set_fraction;
         exposedField MF<float> key;
         exposedField MF<Vec2f> keyValue;
         eventOut SF<Vec2f> value_changed;
         void Onset_fraction();

         public:
              PositionInterpolator2D() {
              }
};

class ProximitySensor2D : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<Vec2f> center;
         exposedField SF<Vec2f> size;
         exposedField SF<Bool> enabled;
         eventOut SF<Bool> isActive;
         eventOut SF<Vec2f> position_changed;
         eventOut SF<float> orientation_changed;
         eventOut SF<Time> enterTime;
         eventOut SF<Time> exitTime;

         public:
              ProximitySensor2D() {
                   center = Vec2f(0, 0);
                   size = Vec2f(0, 0);
                   enabled = TRUE;
              }
};

class ProximitySensor : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<Vec3f> center;
         exposedField SF<Vec3f> size;
         exposedField SF<Bool> enabled;
         eventOut SF<Bool> isActive;
         eventOut SF<Vec3f> position_changed;
         eventOut SF<Rotation> orientation_changed;
         eventOut SF<Time> enterTime;
         eventOut SF<Time> exitTime;

         public:
              ProximitySensor() {
                   center = Vec3f(0, 0, 0);
                   size = Vec3f(0, 0, 0);
                   enabled = TRUE;
              }
};

class QuantizationParameter : public MediaObject
{
         BIFS_DECLARE_NODE

         FIELD SF<Bool> isLocal;
         FIELD SF<Bool> position3DQuant;
         FIELD SF<Vec3f> position3DMin;
         FIELD SF<Vec3f> position3DMax;
         FIELD SF<int> position3DNbBits;
         FIELD SF<Bool> position2DQuant;
         FIELD SF<Vec2f> position2DMin;
         FIELD SF<Vec2f> position2DMax;
         FIELD SF<int> position2DNbBits;
         FIELD SF<Bool> drawOrderQuant;
         FIELD SF<float> drawOrderMin;
         FIELD SF<float> drawOrderMax;
         FIELD SF<int> drawOrderNbBits;
         FIELD SF<Bool> colorQuant;
         FIELD SF<float> colorMin;
         FIELD SF<float> colorMax;
         FIELD SF<int> colorNbBits;
         FIELD SF<Bool> textureCoordinateQuant;
         FIELD SF<float> textureCoordinateMin;
         FIELD SF<float> textureCoordinateMax;
         FIELD SF<int> textureCoordinateNbBits;
         FIELD SF<Bool> angleQuant;
         FIELD SF<float> angleMin;
         FIELD SF<float> angleMax;
         FIELD SF<int> angleNbBits;
         FIELD SF<Bool> scaleQuant;
         FIELD SF<float> scaleMin;
         FIELD SF<float> scaleMax;
         FIELD SF<int> scaleNbBits;
         FIELD SF<Bool> keyQuant;
         FIELD SF<float> keyMin;
         FIELD SF<float> keyMax;
         FIELD SF<int> keyNbBits;
         FIELD SF<Bool> normalQuant;
         FIELD SF<int> normalNbBits;
         FIELD SF<Bool> sizeQuant;
         FIELD SF<float> sizeMin;
         FIELD SF<float> sizeMax;
         FIELD SF<int> sizeNbBits;
         FIELD SF<Bool> useEfficientCoding;

         public:
              QuantizationParameter() {
                   isLocal = FALSE;
                   position3DQuant = FALSE;
                   position3DMin = Vec3f(-FLT_MAX, -FLT_MAX, -FLT_MAX);
                   position3DMax = Vec3f(+FLT_MAX, +FLT_MAX, +FLT_MAX);
                   position3DNbBits = 16;
                   position2DQuant = FALSE;
                   position2DMin = Vec2f(-FLT_MAX, -FLT_MAX);
                   position2DMax = Vec2f(+FLT_MAX, +FLT_MAX);
                   position2DNbBits = 16;
                   drawOrderQuant = FALSE;
                   drawOrderMin = -FLT_MAX;
                   drawOrderMax = +FLT_MAX;
                   drawOrderNbBits = 8;
                   colorQuant = TRUE;
                   colorMin = 0;
                   colorMax = 1;
                   colorNbBits = 8;
                   textureCoordinateQuant = TRUE;
                   textureCoordinateMin = 0;
                   textureCoordinateMax = 1;
                   textureCoordinateNbBits = 16;
                   angleQuant = TRUE;
                   angleMin = 0;
                   angleMax = 6.2831853;
                   angleNbBits = 16;
                   scaleQuant = FALSE;
                   scaleMin = 0;
                   scaleMax = +FLT_MAX;
                   scaleNbBits = 8;
                   keyQuant = TRUE;
                   keyMin = 0;
                   keyMax = 1;
                   keyNbBits = 8;
                   normalQuant = TRUE;
                   normalNbBits = 8;
                   sizeQuant = FALSE;
                   sizeMin = 0;
                   sizeMax = +FLT_MAX;
                   sizeNbBits = 8;
                   useEfficientCoding = FALSE;
              }
};

class Rectangle : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<Vec2f> size;

         public:
              Rectangle() {
                   size = Vec2f(2, 2);
              }
};

class ScalarInterpolator : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn SF<float> set_fraction;
         exposedField MF<float> key;
         exposedField MF<float> keyValue;
         eventOut SF<float> value_changed;
         void Onset_fraction();

         public:
              ScalarInterpolator() {
              }
};

class Script : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField MF<ZString> url;
         FIELD SF<Bool> directOutput;
         FIELD SF<Bool> mustEvaluate;

         public:
              Script() {
                   directOutput = FALSE;
                   mustEvaluate = FALSE;
              }
};

class Shape : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SFAppearanceNode appearance;
         exposedField SFGeometryNode geometry;

         public:
              Shape() {
              }
};

class Sound : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<Vec3f> direction;
         exposedField SF<float> intensity;
         exposedField SF<Vec3f> location;
         exposedField SF<float> maxBack;
         exposedField SF<float> maxFront;
         exposedField SF<float> minBack;
         exposedField SF<float> minFront;
         exposedField SF<float> priority;
         exposedField SFAudioNode source;
         FIELD SF<Bool> spatialize;

         public:
              Sound() {
                   direction = Vec3f(0, 0, 1);
                   intensity = 1;
                   location = Vec3f(0, 0, 0);
                   maxBack = 10;
                   maxFront = 10;
                   minBack = 1;
                   minFront = 1;
                   priority = 0;
                   spatialize = TRUE;
              }
};

class Sound2D : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<float> intensity;
         exposedField SF<Vec2f> location;
         exposedField SFAudioNode source;
         FIELD SF<Bool> spatialize;

         public:
              Sound2D() {
                   intensity = 1;
                   location = Vec2f(0, 0);
                   spatialize = TRUE;
              }
};

class Sphere : public MediaObject
{
         BIFS_DECLARE_NODE

         FIELD SF<float> radius;

         public:
              Sphere() {
                   radius = 1;
              }
};

class SphereSensor : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<Bool> autoOffset;
         exposedField SF<Bool> enabled;
         exposedField SF<Rotation> offset;
         eventOut SF<Bool> isActive;
         eventOut SF<Rotation> rotation_changed;
         eventOut SF<Vec3f> trackPoint_changed;

         public:
              SphereSensor() {
                   autoOffset = TRUE;
                   enabled = TRUE;
                   offset = Rotation(0, 1, 0, 0);
              }
};

class SpotLight : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<float> ambientIntensity;
         exposedField SF<Vec3f> attenuation;
         exposedField SF<float> beamWidth;
         exposedField SF<Color3f> color;
         exposedField SF<float> cutOffAngle;
         exposedField SF<Vec3f> direction;
         exposedField SF<float> intensity;
         exposedField SF<Vec3f> location;
         exposedField SF<Bool> on;
         exposedField SF<float> radius;

         public:
              SpotLight() {
                   ambientIntensity = 0;
                   attenuation = Vec3f(1, 0, 0);
                   beamWidth = 1.570796;
                   color = Color3f(1, 1, 1);
                   cutOffAngle = 0.785398;
                   direction = Vec3f(0, 0, -1);
                   intensity = 1;
                   location = Vec3f(0, 0, 0);
                   on = TRUE;
                   radius = 100;
              }
};

class Switch : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField MFNodeType<SF3DNode> choice;
         exposedField SF<int> whichChoice;

         public:
              Switch() {
                   whichChoice = -1;
              }
};

class TermCap : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn SF<Time> evaluate;
         exposedField SF<int> capability;
         eventOut SF<int> value;
         void Onevaluate();

         public:
              TermCap() {
                   capability = 0;
                   value = 0;
              }
};

class Text : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField MF<ZString> string;
         exposedField MF<float> length;
         exposedField SFFontStyleNode fontStyle;
         exposedField SF<float> maxExtent;

         public:
              Text() {
                   maxExtent = 0;
              }
};

class TextureCoordinate : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField MF<Vec2f> point;

         public:
              TextureCoordinate() {
              }
};

class TextureTransform : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<Vec2f> center;
         exposedField SF<float> rotation;
         exposedField SF<Vec2f> scale;
         exposedField SF<Vec2f> translation;

         public:
              TextureTransform() {
                   center = Vec2f(0, 0);
                   rotation = 0;
                   scale = Vec2f(1, 1);
                   translation = Vec2f(0, 0);
              }
};

class TimeSensor : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<Time> cycleInterval;
         exposedField SF<Bool> enabled;
         exposedField SF<Bool> loop;
         exposedField SFTimeField startTime;
         exposedField SFTimeField stopTime;
         eventOut SF<Time> cycleTime;
         eventOut SF<float> fraction_changed;
         eventOut SF<Bool> isActive;
         eventOut SF<Time> time;

         public:
              TimeSensor() {
                   cycleInterval = 1;
                   enabled = TRUE;
                   loop = FALSE;
                   startTime = 0;
                   stopTime = 0;
              }
};

class TouchSensor : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<Bool> enabled;
         eventOut SF<Vec3f> hitNormal_changed;
         eventOut SF<Vec3f> hitPoint_changed;
         eventOut SF<Vec2f> hitTexCoord_changed;
         eventOut SF<Bool> isActive;
         eventOut SF<Bool> isOver;
         eventOut SF<Time> touchTime;

         public:
              TouchSensor() {
                   enabled = TRUE;
              }
};

class Transform : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn MFNodeType<SF3DNode> addChildren;
         eventIn MFNodeType<SF3DNode> removeChildren;
         exposedField SF<Vec3f> center;
         exposedField MFNodeType<SF3DNode> children;
         exposedField SF<Rotation> rotation;
         exposedField SF<Vec3f> scale;
         exposedField SF<Rotation> scaleOrientation;
         exposedField SF<Vec3f> translation;
         void OnaddChildren();
         void OnremoveChildren();

         public:
              Transform() {
                   center = Vec3f(0, 0, 0);
                   rotation = Rotation(0, 0, 1, 0);
                   scale = Vec3f(1, 1, 1);
                   scaleOrientation = Rotation(0, 0, 1, 0);
                   translation = Vec3f(0, 0, 0);
              }
};

class Transform2D : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn MFNodeType<SF2DNode> addChildren;
         eventIn MFNodeType<SF2DNode> removeChildren;
         exposedField MFNodeType<SF2DNode> children;
         exposedField SF<Vec2f> center;
         exposedField SF<float> rotationAngle;
         exposedField SF<Vec2f> scale;
         exposedField SF<float> scaleOrientation;
         exposedField SF<Vec2f> translation;
         void OnaddChildren();
         void OnremoveChildren();

         public:
              Transform2D() {
                   center = Vec2f(0, 0);
                   rotationAngle = 0;
                   scale = Vec2f(1, 1);
                   scaleOrientation = 0;
                   translation = Vec2f(0, 0);
              }
};

class Valuator : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn SF<Bool> inSFBool;
         eventIn SF<Color3f> inSFColor;
         eventIn MF<Color3f> inMFColor;
         eventIn SF<float> inSFFloat;
         eventIn MF<float> inMFFloat;
         eventIn SF<int> inSFInt32;
         eventIn MF<int> inMFInt32;
         eventIn SF<Rotation> inSFRotation;
         eventIn MF<Rotation> inMFRotation;
         eventIn SFString inSFString;
         eventIn MF<ZString> inMFString;
         eventIn SF<Time> inSFTime;
         eventIn SF<Vec2f> inSFVec2f;
         eventIn MF<Vec2f> inMFVec2f;
         eventIn SF<Vec3f> inSFVec3f;
         eventIn MF<Vec3f> inMFVec3f;
         eventOut SF<Bool> outSFBool;
         eventOut SF<Color3f> outSFColor;
         eventOut MF<Color3f> outMFColor;
         eventOut SF<float> outSFFloat;
         eventOut MF<float> outMFFloat;
         eventOut SF<int> outSFInt32;
         eventOut MF<int> outMFInt32;
         eventOut SF<Rotation> outSFRotation;
         eventOut MF<Rotation> outMFRotation;
         eventOut SFString outSFString;
         eventOut MF<ZString> outMFString;
         eventOut SF<Time> outSFTime;
         eventOut SF<Vec2f> outSFVec2f;
         eventOut MF<Vec2f> outMFVec2f;
         eventOut SF<Vec3f> outSFVec3f;
         eventOut MF<Vec3f> outMFVec3f;
         exposedField SF<float> Factor1;
         exposedField SF<float> Factor2;
         exposedField SF<float> Factor3;
         exposedField SF<float> Factor4;
         exposedField SF<float> Offset1;
         exposedField SF<float> Offset2;
         exposedField SF<float> Offset3;
         exposedField SF<float> Offset4;
         exposedField SF<Bool> Sum;
         void OninSFBool();
         void OninSFColor();
         void OninMFColor();
         void OninSFFloat();
         void OninMFFloat();
         void OninSFInt32();
         void OninMFInt32();
         void OninSFRotation();
         void OninMFRotation();
         void OninSFString();
         void OninMFString();
         void OninSFTime();
         void OninSFVec2f();
         void OninMFVec2f();
         void OninSFVec3f();
         void OninMFVec3f();

         public:
              Valuator() {
                   Factor1 = 1;
                   Factor2 = 1;
                   Factor3 = 1;
                   Factor4 = 1;
                   Offset1 = 0;
                   Offset2 = 0;
                   Offset3 = 0;
                   Offset4 = 0;
                   Sum = FALSE;
              }
};

class Viewpoint : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn SF<Bool> set_bind;
         exposedField SF<float> fieldOfView;
         exposedField SF<Bool> jump;
         exposedField SF<Rotation> orientation;
         exposedField SF<Vec3f> position;
         FIELD SFString description;
         eventOut SF<Time> bindTime;
         eventOut SF<Bool> isBound;
         void Onset_bind();

         public:
              Viewpoint() {
                   fieldOfView = 0.785398;
                   jump = TRUE;
                   orientation = Rotation(0, 0, 1, 0);
                   position = Vec3f(0, 0, 10);
              }
};

class VisibilitySensor : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<Vec3f> center;
         exposedField SF<Bool> enabled;
         exposedField SF<Vec3f> size;
         eventOut SF<Time> enterTime;
         eventOut SF<Time> exitTime;
         eventOut SF<Bool> isActive;

         public:
              VisibilitySensor() {
                   center = Vec3f(0, 0, 0);
                   enabled = TRUE;
                   size = Vec3f(0, 0, 0);
              }
};

class Viseme : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<int> viseme_select1;
         exposedField SF<int> viseme_select2;
         exposedField SF<int> viseme_blend;
         exposedField SF<Bool> viseme_def;

         public:
              Viseme() {
                   viseme_select1 = 0;
                   viseme_select2 = 0;
                   viseme_blend = 0;
                   viseme_def = FALSE;
              }
};

class WorldInfo : public MediaObject
{
         BIFS_DECLARE_NODE

         FIELD MF<ZString> info;
         FIELD SFString title;

         public:
              WorldInfo() {
              }
};

class AcousticMaterial : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<float> ambientIntensity;
         exposedField SF<Color3f> diffuseColor;
         exposedField SF<Color3f> emissiveColor;
         exposedField SF<float> shininess;
         exposedField SF<Color3f> specularColor;
         exposedField SF<float> transparency;
         FIELD MF<float> reffunc;
         FIELD MF<float> transfunc;
         FIELD MF<float> refFrequency;
         FIELD MF<float> transFrequency;

         public:
              AcousticMaterial() {
                   ambientIntensity = 0.2;
                   diffuseColor = Color3f(0.8, 0.8, 0.8);
                   emissiveColor = Color3f(0, 0, 0);
                   shininess = 0.2;
                   specularColor = Color3f(0, 0, 0);
                   transparency = 0;
                   reffunc += 0;
                   transfunc += 1;
                   refFrequency += 0;
                   transFrequency += 0;
              }
};

class AcousticScene : public MediaObject
{
         BIFS_DECLARE_NODE

         FIELD SF<Vec3f> center;
         FIELD SF<Vec3f> Size;
         FIELD MF<Time> reverbTime;
         FIELD MF<float> reverbFreq;
         exposedField SF<float> reverbLevel;
         exposedField SF<Time> reverbDelay;

         public:
              AcousticScene() {
                   center = Vec3f(0, 0, 0);
                   Size = Vec3f(-1, -1, -1);
                   reverbTime += 0;
                   reverbFreq += 1000;
                   reverbLevel = 0.4;
                   reverbDelay = 0.5;
              }
};

class ApplicationWindow : public StreamConsumer
{
         BIFS_DECLARE_NODE

         exposedField SF<Bool> isActive;
         exposedField SFTimeField startTime;
         exposedField SFTimeField stopTime;
         exposedField SFString description;
         exposedField MF<ZString> parameter;
         exposedField SF<Vec2f> size;

         public:
              ApplicationWindow() {
                   isActive = FALSE;
                   startTime = 0;
                   stopTime = 0;
                   size = Vec2f(0, 0);
              }
};

class BAP : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<int> sacroiliac_tilt;
         exposedField SF<int> sacroiliac_torsion;
         exposedField SF<int> sacroiliac_roll;
         exposedField SF<int> l_hip_flexion;
         exposedField SF<int> r_hip_flexion;
         exposedField SF<int> l_hip_abduct;
         exposedField SF<int> r_hip_abduct;
         exposedField SF<int> l_hip_twisting;
         exposedField SF<int> r_hip_twisting;
         exposedField SF<int> l_knee_flexion;
         exposedField SF<int> r_knee_flexion;
         exposedField SF<int> l_knee_twisting;
         exposedField SF<int> r_knee_twisting;
         exposedField SF<int> l_ankle_flexion;
         exposedField SF<int> r_ankle_flexion;
         exposedField SF<int> l_ankle_twisting;
         exposedField SF<int> r_ankle_twisting;
         exposedField SF<int> l_subtalar_flexion;
         exposedField SF<int> r_subtalar_flexion;
         exposedField SF<int> l_midtarsal_flexion;
         exposedField SF<int> r_midtarsal_flexion;
         exposedField SF<int> l_metatarsal_flexion;
         exposedField SF<int> r_metatarsal_flexion;
         exposedField SF<int> l_sternoclavicular_abduct;
         exposedField SF<int> r_sternoclavicular_abduct;
         exposedField SF<int> l_sternoclavicular_rotate;
         exposedField SF<int> r_sternoclavicular_rotate;
         exposedField SF<int> l_acromioclavicular_abduct;
         exposedField SF<int> r_acromioclavicular_abduct;
         exposedField SF<int> l_acromioclavicular_rotate;
         exposedField SF<int> r_acromioclavicular_rotate;
         exposedField SF<int> l_shoulder_flexion;
         exposedField SF<int> r_shoulder_flexion;
         exposedField SF<int> l_shoulder_abduct;
         exposedField SF<int> r_shoulder_abduct;
         exposedField SF<int> l_shoulder_twisting;
         exposedField SF<int> r_shoulder_twisting;
         exposedField SF<int> l_elbow_flexion;
         exposedField SF<int> r_elbow_flexion;
         exposedField SF<int> l_elbow_twisting;
         exposedField SF<int> r_elbow_twisting;
         exposedField SF<int> l_wrist_flexion;
         exposedField SF<int> r_wrist_flexion;
         exposedField SF<int> l_wrist_pivot;
         exposedField SF<int> r_wrist_pivot;
         exposedField SF<int> l_wrist_twisting;
         exposedField SF<int> r_wrist_twisting;
         exposedField SF<int> skullbase_roll;
         exposedField SF<int> skullbase_torsion;
         exposedField SF<int> skullbase_tilt;
         exposedField SF<int> vc1roll;
         exposedField SF<int> vc1torsion;
         exposedField SF<int> vc1tilt;
         exposedField SF<int> vc2roll;
         exposedField SF<int> vc2torsion;
         exposedField SF<int> vc2tilt;
         exposedField SF<int> vc3roll;
         exposedField SF<int> vc3torsion;
         exposedField SF<int> vc3tilt;
         exposedField SF<int> vc4roll;
         exposedField SF<int> vc4torsion;
         exposedField SF<int> vc4tilt;
         exposedField SF<int> vc5roll;
         exposedField SF<int> vc5torsion;
         exposedField SF<int> vc5tilt;
         exposedField SF<int> vc6roll;
         exposedField SF<int> vc6torsion;
         exposedField SF<int> vc6tilt;
         exposedField SF<int> vc7roll;
         exposedField SF<int> vc7torsion;
         exposedField SF<int> vc7tilt;
         exposedField SF<int> vt1roll;
         exposedField SF<int> vt1torsion;
         exposedField SF<int> vt1tilt;
         exposedField SF<int> vt2roll;
         exposedField SF<int> vt2torsion;
         exposedField SF<int> vt2tilt;
         exposedField SF<int> vt3roll;
         exposedField SF<int> vt3torsion;
         exposedField SF<int> vt3tilt;
         exposedField SF<int> vt4roll;
         exposedField SF<int> vt4torsion;
         exposedField SF<int> vt4tilt;
         exposedField SF<int> vt5roll;
         exposedField SF<int> vt5torsion;
         exposedField SF<int> vt5tilt;
         exposedField SF<int> vt6roll;
         exposedField SF<int> vt6torsion;
         exposedField SF<int> vt6tilt;
         exposedField SF<int> vt7roll;
         exposedField SF<int> vt7torsion;
         exposedField SF<int> vt7tilt;
         exposedField SF<int> vt8roll;
         exposedField SF<int> vt8torsion;
         exposedField SF<int> vt8tilt;
         exposedField SF<int> vt9roll;
         exposedField SF<int> vt9torsion;
         exposedField SF<int> vt9tilt;
         exposedField SF<int> vt10roll;
         exposedField SF<int> vt10torsion;
         exposedField SF<int> vt10tilt;
         exposedField SF<int> vt11roll;
         exposedField SF<int> vt11torsion;
         exposedField SF<int> vt11tilt;
         exposedField SF<int> vt12roll;
         exposedField SF<int> vt12torsion;
         exposedField SF<int> vt12tilt;
         exposedField SF<int> vl1roll;
         exposedField SF<int> vl1torsion;
         exposedField SF<int> vl1tilt;
         exposedField SF<int> vl2roll;
         exposedField SF<int> vl2torsion;
         exposedField SF<int> vl2tilt;
         exposedField SF<int> vl3roll;
         exposedField SF<int> vl3torsion;
         exposedField SF<int> vl3tilt;
         exposedField SF<int> vl4roll;
         exposedField SF<int> vl4torsion;
         exposedField SF<int> vl4tilt;
         exposedField SF<int> vl5roll;
         exposedField SF<int> vl5torsion;
         exposedField SF<int> vl5tilt;
         exposedField SF<int> l_pinky0_flexion;
         exposedField SF<int> r_pinky0_flexion;
         exposedField SF<int> l_pinky1_flexion;
         exposedField SF<int> r_pinky1_flexion;
         exposedField SF<int> l_pinky1_pivot;
         exposedField SF<int> r_pinky1_pivot;
         exposedField SF<int> l_pinky1_twisting;
         exposedField SF<int> r_pinky1_twisting;
         exposedField SF<int> l_pinky2_flexion;
         exposedField SF<int> r_pinky2_flexion;
         exposedField SF<int> l_pinky3_flexion;
         exposedField SF<int> r_pinky3_flexion;
         exposedField SF<int> l_ring0_flexion;
         exposedField SF<int> r_ring0_flexion;
         exposedField SF<int> l_ring1_flexion;
         exposedField SF<int> r_ring1_flexion;
         exposedField SF<int> l_ring1_pivot;
         exposedField SF<int> r_ring1_pivot;
         exposedField SF<int> l_ring1_twisting;
         exposedField SF<int> r_ring1_twisting;
         exposedField SF<int> l_ring2_flexion;
         exposedField SF<int> r_ring2_flexion;
         exposedField SF<int> l_ring3_flexion;
         exposedField SF<int> r_ring3_flexion;
         exposedField SF<int> l_middle0_flexion;
         exposedField SF<int> r_middle0_flexion;
         exposedField SF<int> l_middle1_flexion;
         exposedField SF<int> r_middle1_flexion;
         exposedField SF<int> l_middle1_pivot;
         exposedField SF<int> r_middle1_pivot;
         exposedField SF<int> l_middle1_twisting;
         exposedField SF<int> r_middle1_twisting;
         exposedField SF<int> l_middle2_flexion;
         exposedField SF<int> r_middle2_flexion;
         exposedField SF<int> l_middle3_flexion;
         exposedField SF<int> r_middle3_flexion;
         exposedField SF<int> l_index0_flexion;
         exposedField SF<int> r_index0_flexion;
         exposedField SF<int> l_index1_flexion;
         exposedField SF<int> r_index1_flexion;
         exposedField SF<int> l_index1_pivot;
         exposedField SF<int> r_index1_pivot;
         exposedField SF<int> l_index1_twisting;
         exposedField SF<int> r_index1_twisting;
         exposedField SF<int> l_index2_flexion;
         exposedField SF<int> r_index2_flexion;
         exposedField SF<int> l_index3_flexion;
         exposedField SF<int> r_index3_flexion;
         exposedField SF<int> l_thumb1_flexion;
         exposedField SF<int> r_thumb1_flexion;
         exposedField SF<int> l_thumb1_pivot;
         exposedField SF<int> r_thumb1_pivot;
         exposedField SF<int> l_thumb1_twisting;
         exposedField SF<int> r_thumb1_twisting;
         exposedField SF<int> l_thumb2_flexion;
         exposedField SF<int> r_thumb2_flexion;
         exposedField SF<int> l_thumb3_flexion;
         exposedField SF<int> r_thumb3_flexion;
         exposedField SF<int> HumanoidRoot_tr_vertical;
         exposedField SF<int> HumanoidRoot_tr_lateral;
         exposedField SF<int> HumanoidRoot_tr_frontal;
         exposedField SF<int> HumanoidRoot_rt_body_turn;
         exposedField SF<int> HumanoidRoot_rt_body_roll;
         exposedField SF<int> HumanoidRoot_rt_body_tilt;
         exposedField SF<int> extensionBap187;
         exposedField SF<int> extensionBap188;
         exposedField SF<int> extensionBap189;
         exposedField SF<int> extensionBap190;
         exposedField SF<int> extensionBap191;
         exposedField SF<int> extensionBap192;
         exposedField SF<int> extensionBap193;
         exposedField SF<int> extensionBap194;
         exposedField SF<int> extensionBap195;
         exposedField SF<int> extensionBap196;
         exposedField SF<int> extensionBap197;
         exposedField SF<int> extensionBap198;
         exposedField SF<int> extensionBap199;
         exposedField SF<int> extensionBap200;
         exposedField SF<int> extensionBap201;
         exposedField SF<int> extensionBap202;
         exposedField SF<int> extensionBap203;
         exposedField SF<int> extensionBap204;
         exposedField SF<int> extensionBap205;
         exposedField SF<int> extensionBap206;
         exposedField SF<int> extensionBap207;
         exposedField SF<int> extensionBap208;
         exposedField SF<int> extensionBap209;
         exposedField SF<int> extensionBap210;
         exposedField SF<int> extensionBap211;
         exposedField SF<int> extensionBap212;
         exposedField SF<int> extensionBap213;
         exposedField SF<int> extensionBap214;
         exposedField SF<int> extensionBap215;
         exposedField SF<int> extensionBap216;
         exposedField SF<int> extensionBap217;
         exposedField SF<int> extensionBap218;
         exposedField SF<int> extensionBap219;
         exposedField SF<int> extensionBap220;
         exposedField SF<int> extensionBap221;
         exposedField SF<int> extensionBap222;
         exposedField SF<int> extensionBap223;
         exposedField SF<int> extensionBap224;
         exposedField SF<int> extensionBap225;
         exposedField SF<int> extensionBap226;
         exposedField SF<int> extensionBap227;
         exposedField SF<int> extensionBap228;
         exposedField SF<int> extensionBap229;
         exposedField SF<int> extensionBap230;
         exposedField SF<int> extensionBap231;
         exposedField SF<int> extensionBap232;
         exposedField SF<int> extensionBap233;
         exposedField SF<int> extensionBap234;
         exposedField SF<int> extensionBap235;
         exposedField SF<int> extensionBap236;
         exposedField SF<int> extensionBap237;
         exposedField SF<int> extensionBap238;
         exposedField SF<int> extensionBap239;
         exposedField SF<int> extensionBap240;
         exposedField SF<int> extensionBap241;
         exposedField SF<int> extensionBap242;
         exposedField SF<int> extensionBap243;
         exposedField SF<int> extensionBap244;
         exposedField SF<int> extensionBap245;
         exposedField SF<int> extensionBap246;
         exposedField SF<int> extensionBap247;
         exposedField SF<int> extensionBap248;
         exposedField SF<int> extensionBap249;
         exposedField SF<int> extensionBap250;
         exposedField SF<int> extensionBap251;
         exposedField SF<int> extensionBap252;
         exposedField SF<int> extensionBap253;
         exposedField SF<int> extensionBap254;
         exposedField SF<int> extensionBap255;
         exposedField SF<int> extensionBap256;
         exposedField SF<int> extensionBap257;
         exposedField SF<int> extensionBap258;
         exposedField SF<int> extensionBap259;
         exposedField SF<int> extensionBap260;
         exposedField SF<int> extensionBap261;
         exposedField SF<int> extensionBap262;
         exposedField SF<int> extensionBap263;
         exposedField SF<int> extensionBap264;
         exposedField SF<int> extensionBap265;
         exposedField SF<int> extensionBap266;
         exposedField SF<int> extensionBap267;
         exposedField SF<int> extensionBap268;
         exposedField SF<int> extensionBap269;
         exposedField SF<int> extensionBap270;
         exposedField SF<int> extensionBap271;
         exposedField SF<int> extensionBap272;
         exposedField SF<int> extensionBap273;
         exposedField SF<int> extensionBap274;
         exposedField SF<int> extensionBap275;
         exposedField SF<int> extensionBap276;
         exposedField SF<int> extensionBap277;
         exposedField SF<int> extensionBap278;
         exposedField SF<int> extensionBap279;
         exposedField SF<int> extensionBap280;
         exposedField SF<int> extensionBap281;
         exposedField SF<int> extensionBap282;
         exposedField SF<int> extensionBap283;
         exposedField SF<int> extensionBap284;
         exposedField SF<int> extensionBap285;
         exposedField SF<int> extensionBap286;
         exposedField SF<int> extensionBap287;
         exposedField SF<int> extensionBap288;
         exposedField SF<int> extensionBap289;
         exposedField SF<int> extensionBap290;
         exposedField SF<int> extensionBap291;
         exposedField SF<int> extensionBap292;
         exposedField SF<int> extensionBap293;
         exposedField SF<int> extensionBap294;
         exposedField SF<int> extensionBap295;
         exposedField SF<int> extensionBap296;

         public:
              BAP() {
                   sacroiliac_tilt = +INT_MAX;
                   sacroiliac_torsion = +INT_MAX;
                   sacroiliac_roll = +INT_MAX;
                   l_hip_flexion = +INT_MAX;
                   r_hip_flexion = +INT_MAX;
                   l_hip_abduct = +INT_MAX;
                   r_hip_abduct = +INT_MAX;
                   l_hip_twisting = +INT_MAX;
                   r_hip_twisting = +INT_MAX;
                   l_knee_flexion = +INT_MAX;
                   r_knee_flexion = +INT_MAX;
                   l_knee_twisting = +INT_MAX;
                   r_knee_twisting = +INT_MAX;
                   l_ankle_flexion = +INT_MAX;
                   r_ankle_flexion = +INT_MAX;
                   l_ankle_twisting = +INT_MAX;
                   r_ankle_twisting = +INT_MAX;
                   l_subtalar_flexion = +INT_MAX;
                   r_subtalar_flexion = +INT_MAX;
                   l_midtarsal_flexion = +INT_MAX;
                   r_midtarsal_flexion = +INT_MAX;
                   l_metatarsal_flexion = +INT_MAX;
                   r_metatarsal_flexion = +INT_MAX;
                   l_sternoclavicular_abduct = +INT_MAX;
                   r_sternoclavicular_abduct = +INT_MAX;
                   l_sternoclavicular_rotate = +INT_MAX;
                   r_sternoclavicular_rotate = +INT_MAX;
                   l_acromioclavicular_abduct = +INT_MAX;
                   r_acromioclavicular_abduct = +INT_MAX;
                   l_acromioclavicular_rotate = +INT_MAX;
                   r_acromioclavicular_rotate = +INT_MAX;
                   l_shoulder_flexion = +INT_MAX;
                   r_shoulder_flexion = +INT_MAX;
                   l_shoulder_abduct = +INT_MAX;
                   r_shoulder_abduct = +INT_MAX;
                   l_shoulder_twisting = +INT_MAX;
                   r_shoulder_twisting = +INT_MAX;
                   l_elbow_flexion = +INT_MAX;
                   r_elbow_flexion = +INT_MAX;
                   l_elbow_twisting = +INT_MAX;
                   r_elbow_twisting = +INT_MAX;
                   l_wrist_flexion = +INT_MAX;
                   r_wrist_flexion = +INT_MAX;
                   l_wrist_pivot = +INT_MAX;
                   r_wrist_pivot = +INT_MAX;
                   l_wrist_twisting = +INT_MAX;
                   r_wrist_twisting = +INT_MAX;
                   skullbase_roll = +INT_MAX;
                   skullbase_torsion = +INT_MAX;
                   skullbase_tilt = +INT_MAX;
                   vc1roll = +INT_MAX;
                   vc1torsion = +INT_MAX;
                   vc1tilt = +INT_MAX;
                   vc2roll = +INT_MAX;
                   vc2torsion = +INT_MAX;
                   vc2tilt = +INT_MAX;
                   vc3roll = +INT_MAX;
                   vc3torsion = +INT_MAX;
                   vc3tilt = +INT_MAX;
                   vc4roll = +INT_MAX;
                   vc4torsion = +INT_MAX;
                   vc4tilt = +INT_MAX;
                   vc5roll = +INT_MAX;
                   vc5torsion = +INT_MAX;
                   vc5tilt = +INT_MAX;
                   vc6roll = +INT_MAX;
                   vc6torsion = +INT_MAX;
                   vc6tilt = +INT_MAX;
                   vc7roll = +INT_MAX;
                   vc7torsion = +INT_MAX;
                   vc7tilt = +INT_MAX;
                   vt1roll = +INT_MAX;
                   vt1torsion = +INT_MAX;
                   vt1tilt = +INT_MAX;
                   vt2roll = +INT_MAX;
                   vt2torsion = +INT_MAX;
                   vt2tilt = +INT_MAX;
                   vt3roll = +INT_MAX;
                   vt3torsion = +INT_MAX;
                   vt3tilt = +INT_MAX;
                   vt4roll = +INT_MAX;
                   vt4torsion = +INT_MAX;
                   vt4tilt = +INT_MAX;
                   vt5roll = +INT_MAX;
                   vt5torsion = +INT_MAX;
                   vt5tilt = +INT_MAX;
                   vt6roll = +INT_MAX;
                   vt6torsion = +INT_MAX;
                   vt6tilt = +INT_MAX;
                   vt7roll = +INT_MAX;
                   vt7torsion = +INT_MAX;
                   vt7tilt = +INT_MAX;
                   vt8roll = +INT_MAX;
                   vt8torsion = +INT_MAX;
                   vt8tilt = +INT_MAX;
                   vt9roll = +INT_MAX;
                   vt9torsion = +INT_MAX;
                   vt9tilt = +INT_MAX;
                   vt10roll = +INT_MAX;
                   vt10torsion = +INT_MAX;
                   vt10tilt = +INT_MAX;
                   vt11roll = +INT_MAX;
                   vt11torsion = +INT_MAX;
                   vt11tilt = +INT_MAX;
                   vt12roll = +INT_MAX;
                   vt12torsion = +INT_MAX;
                   vt12tilt = +INT_MAX;
                   vl1roll = +INT_MAX;
                   vl1torsion = +INT_MAX;
                   vl1tilt = +INT_MAX;
                   vl2roll = +INT_MAX;
                   vl2torsion = +INT_MAX;
                   vl2tilt = +INT_MAX;
                   vl3roll = +INT_MAX;
                   vl3torsion = +INT_MAX;
                   vl3tilt = +INT_MAX;
                   vl4roll = +INT_MAX;
                   vl4torsion = +INT_MAX;
                   vl4tilt = +INT_MAX;
                   vl5roll = +INT_MAX;
                   vl5torsion = +INT_MAX;
                   vl5tilt = +INT_MAX;
                   l_pinky0_flexion = +INT_MAX;
                   r_pinky0_flexion = +INT_MAX;
                   l_pinky1_flexion = +INT_MAX;
                   r_pinky1_flexion = +INT_MAX;
                   l_pinky1_pivot = +INT_MAX;
                   r_pinky1_pivot = +INT_MAX;
                   l_pinky1_twisting = +INT_MAX;
                   r_pinky1_twisting = +INT_MAX;
                   l_pinky2_flexion = +INT_MAX;
                   r_pinky2_flexion = +INT_MAX;
                   l_pinky3_flexion = +INT_MAX;
                   r_pinky3_flexion = +INT_MAX;
                   l_ring0_flexion = +INT_MAX;
                   r_ring0_flexion = +INT_MAX;
                   l_ring1_flexion = +INT_MAX;
                   r_ring1_flexion = +INT_MAX;
                   l_ring1_pivot = +INT_MAX;
                   r_ring1_pivot = +INT_MAX;
                   l_ring1_twisting = +INT_MAX;
                   r_ring1_twisting = +INT_MAX;
                   l_ring2_flexion = +INT_MAX;
                   r_ring2_flexion = +INT_MAX;
                   l_ring3_flexion = +INT_MAX;
                   r_ring3_flexion = +INT_MAX;
                   l_middle0_flexion = +INT_MAX;
                   r_middle0_flexion = +INT_MAX;
                   l_middle1_flexion = +INT_MAX;
                   r_middle1_flexion = +INT_MAX;
                   l_middle1_pivot = +INT_MAX;
                   r_middle1_pivot = +INT_MAX;
                   l_middle1_twisting = +INT_MAX;
                   r_middle1_twisting = +INT_MAX;
                   l_middle2_flexion = +INT_MAX;
                   r_middle2_flexion = +INT_MAX;
                   l_middle3_flexion = +INT_MAX;
                   r_middle3_flexion = +INT_MAX;
                   l_index0_flexion = +INT_MAX;
                   r_index0_flexion = +INT_MAX;
                   l_index1_flexion = +INT_MAX;
                   r_index1_flexion = +INT_MAX;
                   l_index1_pivot = +INT_MAX;
                   r_index1_pivot = +INT_MAX;
                   l_index1_twisting = +INT_MAX;
                   r_index1_twisting = +INT_MAX;
                   l_index2_flexion = +INT_MAX;
                   r_index2_flexion = +INT_MAX;
                   l_index3_flexion = +INT_MAX;
                   r_index3_flexion = +INT_MAX;
                   l_thumb1_flexion = +INT_MAX;
                   r_thumb1_flexion = +INT_MAX;
                   l_thumb1_pivot = +INT_MAX;
                   r_thumb1_pivot = +INT_MAX;
                   l_thumb1_twisting = +INT_MAX;
                   r_thumb1_twisting = +INT_MAX;
                   l_thumb2_flexion = +INT_MAX;
                   r_thumb2_flexion = +INT_MAX;
                   l_thumb3_flexion = +INT_MAX;
                   r_thumb3_flexion = +INT_MAX;
                   HumanoidRoot_tr_vertical = +INT_MAX;
                   HumanoidRoot_tr_lateral = +INT_MAX;
                   HumanoidRoot_tr_frontal = +INT_MAX;
                   HumanoidRoot_rt_body_turn = +INT_MAX;
                   HumanoidRoot_rt_body_roll = +INT_MAX;
                   HumanoidRoot_rt_body_tilt = +INT_MAX;
                   extensionBap187 = +INT_MAX;
                   extensionBap188 = +INT_MAX;
                   extensionBap189 = +INT_MAX;
                   extensionBap190 = +INT_MAX;
                   extensionBap191 = +INT_MAX;
                   extensionBap192 = +INT_MAX;
                   extensionBap193 = +INT_MAX;
                   extensionBap194 = +INT_MAX;
                   extensionBap195 = +INT_MAX;
                   extensionBap196 = +INT_MAX;
                   extensionBap197 = +INT_MAX;
                   extensionBap198 = +INT_MAX;
                   extensionBap199 = +INT_MAX;
                   extensionBap200 = +INT_MAX;
                   extensionBap201 = +INT_MAX;
                   extensionBap202 = +INT_MAX;
                   extensionBap203 = +INT_MAX;
                   extensionBap204 = +INT_MAX;
                   extensionBap205 = +INT_MAX;
                   extensionBap206 = +INT_MAX;
                   extensionBap207 = +INT_MAX;
                   extensionBap208 = +INT_MAX;
                   extensionBap209 = +INT_MAX;
                   extensionBap210 = +INT_MAX;
                   extensionBap211 = +INT_MAX;
                   extensionBap212 = +INT_MAX;
                   extensionBap213 = +INT_MAX;
                   extensionBap214 = +INT_MAX;
                   extensionBap215 = +INT_MAX;
                   extensionBap216 = +INT_MAX;
                   extensionBap217 = +INT_MAX;
                   extensionBap218 = +INT_MAX;
                   extensionBap219 = +INT_MAX;
                   extensionBap220 = +INT_MAX;
                   extensionBap221 = +INT_MAX;
                   extensionBap222 = +INT_MAX;
                   extensionBap223 = +INT_MAX;
                   extensionBap224 = +INT_MAX;
                   extensionBap225 = +INT_MAX;
                   extensionBap226 = +INT_MAX;
                   extensionBap227 = +INT_MAX;
                   extensionBap228 = +INT_MAX;
                   extensionBap229 = +INT_MAX;
                   extensionBap230 = +INT_MAX;
                   extensionBap231 = +INT_MAX;
                   extensionBap232 = +INT_MAX;
                   extensionBap233 = +INT_MAX;
                   extensionBap234 = +INT_MAX;
                   extensionBap235 = +INT_MAX;
                   extensionBap236 = +INT_MAX;
                   extensionBap237 = +INT_MAX;
                   extensionBap238 = +INT_MAX;
                   extensionBap239 = +INT_MAX;
                   extensionBap240 = +INT_MAX;
                   extensionBap241 = +INT_MAX;
                   extensionBap242 = +INT_MAX;
                   extensionBap243 = +INT_MAX;
                   extensionBap244 = +INT_MAX;
                   extensionBap245 = +INT_MAX;
                   extensionBap246 = +INT_MAX;
                   extensionBap247 = +INT_MAX;
                   extensionBap248 = +INT_MAX;
                   extensionBap249 = +INT_MAX;
                   extensionBap250 = +INT_MAX;
                   extensionBap251 = +INT_MAX;
                   extensionBap252 = +INT_MAX;
                   extensionBap253 = +INT_MAX;
                   extensionBap254 = +INT_MAX;
                   extensionBap255 = +INT_MAX;
                   extensionBap256 = +INT_MAX;
                   extensionBap257 = +INT_MAX;
                   extensionBap258 = +INT_MAX;
                   extensionBap259 = +INT_MAX;
                   extensionBap260 = +INT_MAX;
                   extensionBap261 = +INT_MAX;
                   extensionBap262 = +INT_MAX;
                   extensionBap263 = +INT_MAX;
                   extensionBap264 = +INT_MAX;
                   extensionBap265 = +INT_MAX;
                   extensionBap266 = +INT_MAX;
                   extensionBap267 = +INT_MAX;
                   extensionBap268 = +INT_MAX;
                   extensionBap269 = +INT_MAX;
                   extensionBap270 = +INT_MAX;
                   extensionBap271 = +INT_MAX;
                   extensionBap272 = +INT_MAX;
                   extensionBap273 = +INT_MAX;
                   extensionBap274 = +INT_MAX;
                   extensionBap275 = +INT_MAX;
                   extensionBap276 = +INT_MAX;
                   extensionBap277 = +INT_MAX;
                   extensionBap278 = +INT_MAX;
                   extensionBap279 = +INT_MAX;
                   extensionBap280 = +INT_MAX;
                   extensionBap281 = +INT_MAX;
                   extensionBap282 = +INT_MAX;
                   extensionBap283 = +INT_MAX;
                   extensionBap284 = +INT_MAX;
                   extensionBap285 = +INT_MAX;
                   extensionBap286 = +INT_MAX;
                   extensionBap287 = +INT_MAX;
                   extensionBap288 = +INT_MAX;
                   extensionBap289 = +INT_MAX;
                   extensionBap290 = +INT_MAX;
                   extensionBap291 = +INT_MAX;
                   extensionBap292 = +INT_MAX;
                   extensionBap293 = +INT_MAX;
                   extensionBap294 = +INT_MAX;
                   extensionBap295 = +INT_MAX;
                   extensionBap296 = +INT_MAX;
              }
};

class BDP : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField MFNode<class BodyDefTable> bodyDefTables;
         exposedField MFNodeType<SF3DNode> bodySceneGraph;

         public:
              BDP() {
              }
};

class Body : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SFNode<class BDP> bdp;
         exposedField SFNode<class BAP> bap;
         exposedField MFNodeType<SF3DNode> renderedBody;

         public:
              Body() {
              }
};

class BodyDefTable : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SFString bodySceneGraphNodeName;
         exposedField MF<int> bapIDs;
         exposedField MF<int> vertexIds;
         exposedField MF<int> bapCombinations;
         exposedField MF<Vec3f> displacements;
         exposedField SF<int> numInterpolateKeys;

         public:
              BodyDefTable() {
                   numInterpolateKeys = 2;
              }
};

class BodySegmentConnectionHint : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SFString firstSegmentNodeName;
         exposedField SFString secondSegmentNodeName;
         exposedField MF<int> firstVertexIdList;
         exposedField MF<int> secondVertexIdList;

         public:
              BodySegmentConnectionHint() {
              }
};

class DirectiveSound : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<Vec3f> direction;
         exposedField SF<float> intensity;
         exposedField SF<Vec3f> location;
         exposedField SFAudioNode source;
         exposedField SFNode<class PerceptualParameters> perceptualParameters;
         exposedField SF<Bool> roomEffect;
         exposedField SF<Bool> spatialize;
         FIELD MF<float> directivity;
         FIELD MF<float> angles;
         FIELD MF<float> frequency;
         FIELD SF<float> speedOfSound;
         FIELD SF<float> distance;
         FIELD SF<Bool> useAirabs;

         public:
              DirectiveSound() {
                   direction = Vec3f(0, 0, -1);
                   intensity = 1;
                   location = Vec3f(0, 0, 0);
                   roomEffect = FALSE;
                   spatialize = TRUE;
                   directivity += 1;
                   angles += 1;
                   speedOfSound = 340;
                   distance = 100;
                   useAirabs = FALSE;
              }
};

class Hierarchical3DMesh : public StreamConsumer
{
         BIFS_DECLARE_NODE

         eventIn SF<int> triangleBudget;
         exposedField SF<float> level;
         eventOut SF<Bool> doneLoading;
         void OntriangleBudget();

         public:
              Hierarchical3DMesh() {
                   triangleBudget = 1000;
                   level = 1;
              }
};

class MaterialKey : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<Bool> isKeyed;
         exposedField SF<Bool> isRGB;
         exposedField SF<Color3f> keyColor;
         exposedField SF<float> lowThreshold;
         exposedField SF<float> highThreshold;
         exposedField SF<float> transparency;

         public:
              MaterialKey() {
                   isKeyed = TRUE;
                   isRGB = TRUE;
                   keyColor = Color3f(0, 0, 0);
                   lowThreshold = 0;
                   highThreshold = 0;
                   transparency = 0;
              }
};

class PerceptualParameters : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<float> sourcePresence;
         exposedField SF<float> sourceWarmth;
         exposedField SF<float> sourceBrilliance;
         exposedField SF<float> roomPresence;
         exposedField SF<float> runningReverberance;
         exposedField SF<float> envelopment;
         exposedField SF<float> lateReverberance;
         exposedField SF<float> heavyness;
         exposedField SF<float> liveness;
         exposedField MF<float> omniDirectivity;
         exposedField MF<float> directFilterGains;
         exposedField MF<float> inputFilterGains;
         exposedField SF<float> refDistance;
         exposedField SF<float> freqLow;
         exposedField SF<float> freqHigh;
         exposedField SF<Time> timeLimit1;
         exposedField SF<Time> timeLimit2;
         exposedField SF<Time> timeLimit3;
         exposedField SF<Time> modalDensity;

         public:
              PerceptualParameters() {
                   sourcePresence = 1;
                   sourceWarmth = 1;
                   sourceBrilliance = 1;
                   roomPresence = 1;
                   runningReverberance = 1;
                   envelopment = 0;
                   lateReverberance = 1;
                   heavyness = 1;
                   liveness = 1;
                   omniDirectivity += 1;
                   directFilterGains += 1;
                   directFilterGains += 1;
                   directFilterGains += 1;
                   inputFilterGains += 1;
                   inputFilterGains += 1;
                   inputFilterGains += 1;
                   refDistance = 1;
                   freqLow = 250;
                   freqHigh = 4000;
                   timeLimit1 = 0.02;
                   timeLimit2 = 0.04;
                   timeLimit3 = 0.1;
                   modalDensity = 0.8;
              }
};

class TemporalTransform : public StreamConsumer
{
         BIFS_DECLARE_NODE

         eventIn MFNodeType<SF3DNode> addChildren;
         eventIn MFNodeType<SF3DNode> removeChildren;
         exposedField MFNodeType<SF3DNode> children;
         exposedField SF<Time> startTime;
         exposedField SF<Time> optimalDuration;
         exposedField SF<Bool> active;
         exposedField SF<float> speed;
         exposedField SF<Vec2f> scalability;
         exposedField MF<int> stretchMode;
         exposedField MF<int> shrinkMode;
         exposedField SF<Time> maxDelay;
         eventOut SF<Time> actualDuration;
         void OnaddChildren();
         void OnremoveChildren();

         public:
              TemporalTransform() {
                   startTime = -1;
                   optimalDuration = -1;
                   active = FALSE;
                   speed = 1;
                   scalability = Vec2f(1, 1);
                   stretchMode += 0;
                   shrinkMode += 0;
                   maxDelay = 0;
              }
};

class TemporalGroup : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn MFNodeType<SFTemporalNode> addChildren;
         eventIn MFNodeType<SFTemporalNode> removeChildren;
         exposedField MFNodeType<SFTemporalNode> children;
         FIELD SF<Bool> costart;
         FIELD SF<Bool> coend;
         FIELD SF<Bool> meet;
         exposedField MF<float> priority;
         eventOut SF<Bool> isActive;
         eventOut SF<int> activeChild;
         void OnaddChildren();
         void OnremoveChildren();

         public:
              TemporalGroup() {
                   costart = TRUE;
                   coend = FALSE;
                   meet = FALSE;
              }
};

class ServerCommand : public StreamConsumer
{
         BIFS_DECLARE_NODE

         eventIn SF<Bool> trigger;
         exposedField SF<Bool> enable;
         exposedField SFString command;
         void Ontrigger();

         public:
              ServerCommand() {
                   trigger = FALSE;
                   enable = FALSE;
              }
};

class InputSensor : public StreamConsumer
{
         BIFS_DECLARE_NODE

         exposedField SF<Bool> enabled;
         exposedField SFConditionalString buffer;
         eventOut SF<Time> eventTime;

         public:
              InputSensor() {
                   enabled = TRUE;
              }
};

class MatteTexture : public MediaObject
{
         BIFS_DECLARE_NODE

         FIELD SFTextureNode surfaceA;
         FIELD SFTextureNode surfaceB;
         FIELD SFTextureNode alphaSurface;
         exposedField SFString operation;
         FIELD SF<Bool> overwrite;
         exposedField SF<float> fraction;
         exposedField MF<float> parameter;

         public:
              MatteTexture() {
                   overwrite = FALSE;
                   fraction = 0;
                   parameter += 0;
              }
};

class MediaBuffer : public StreamConsumer
{
         BIFS_DECLARE_NODE

         exposedField SF<float> bufferSize;
         exposedField SF<Time> mediaStartTime;
         exposedField SF<Time> mediaStopTime;
         eventOut SF<Bool> isBuffered;
         exposedField SF<Bool> enabled;

         public:
              MediaBuffer() {
                   bufferSize = 0;
                   mediaStartTime = -1;
                   mediaStopTime = +FLT_MAX;
                   enabled = TRUE;
              }
};

class MediaControl : public StreamConsumer
{
         BIFS_DECLARE_NODE

         exposedField SF<Time> mediaStartTime;
         exposedField SF<Time> mediaStopTime;
         exposedField SF<float> mediaSpeed;
         exposedField SF<Bool> loop;
         exposedField SF<Bool> preRoll;
         exposedField SF<Bool> mute;
         exposedField SF<Bool> enabled;
         eventOut SF<Bool> isPreRolled;

         public:
              MediaControl() {
                   mediaStartTime = -1;
                   mediaStopTime = +FLT_MAX;
                   mediaSpeed = 1;
                   loop = FALSE;
                   preRoll = TRUE;
                   mute = FALSE;
                   enabled = TRUE;
              }
};

class MediaSensor : public StreamConsumer
{
         BIFS_DECLARE_NODE

         eventOut SF<Time> mediaCurrentTime;
         eventOut SF<Time> streamObjectStartTime;
         eventOut SF<Time> mediaDuration;
         eventOut SF<Bool> isActive;
         eventOut MF<ZString> info;

         public:
              MediaSensor() {
              }
};

class BitWrapper : public StreamConsumer
{
         BIFS_DECLARE_NODE

         FIELD SFWorldNode node;
         FIELD SF<int> type;
         FIELD SFString buffer;

         public:
              BitWrapper() {
                   type = 0;
              }
};

class CoordinateInterpolator4D : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn SF<float> set_fraction;
         exposedField MF<float> key;
         exposedField MF<Vec4f> keyValue;
         eventOut MF<Vec4f> value_changed;
         void Onset_fraction();

         public:
              CoordinateInterpolator4D() {
              }
};

class DepthImage : public MediaObject
{
         BIFS_DECLARE_NODE

         FIELD SFDepthTextureNode diTexture;
         FIELD SF<float> farPlane;
         FIELD SF<Vec2f> fieldOfView;
         FIELD SF<float> nearPlane;
         FIELD SF<Rotation> orientation;
         FIELD SF<Bool> orthographic;
         FIELD SF<Vec3f> position;

         public:
              DepthImage() {
                   farPlane = 100;
                   fieldOfView = Vec2f(0.785398, 0.785398);
                   nearPlane = 10;
                   orientation = Rotation(0, 0, 1, 0);
                   orthographic = TRUE;
                   position = Vec3f(0, 0, 10);
              }
};

class FFD : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn MFNodeType<SF3DNode> addChildren;
         eventIn MFNodeType<SF3DNode> removeChildren;
         exposedField MFNodeType<SF3DNode> children;
         exposedField MF<Vec4f> controlPoint;
         FIELD SF<int> uDimension;
         FIELD MF<float> uKnot;
         FIELD SF<int> uOrder;
         FIELD SF<int> vDimension;
         FIELD MF<float> vKnot;
         FIELD SF<int> vOrder;
         FIELD SF<int> wDimension;
         FIELD MF<float> wKnot;
         FIELD SF<int> wOrder;
         void OnaddChildren();
         void OnremoveChildren();

         public:
              FFD() {
                   uDimension = 2;
                   uOrder = 2;
                   vDimension = 2;
                   vOrder = 2;
                   wDimension = 2;
                   wOrder = 2;
              }
};

class Implicit : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<Vec3f> bboxSize;
         exposedField MF<float> c;
         exposedField MF<int> densities;
         exposedField SF<Bool> dual;
         exposedField SF<Bool> solid;

         public:
              Implicit() {
                   bboxSize = Vec3f(2, 2, 2);
                   dual = FALSE;
                   solid = FALSE;
              }
};

class XXLFM_Appearance : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SFNode<class XXLFM_BlendList> blendList;
         exposedField MFNode<class XXLFM_LightMap> lightMapList;
         exposedField MFNodeType<SFTextureNode> tileList;
         exposedField SFNode<class XXLFM_FrameList> vertexFrameList;

         public:
              XXLFM_Appearance() {
              }
};

class XXLFM_BlendList : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField MF<int> blendMode;
         exposedField MF<int> lightMapIndex;

         public:
              XXLFM_BlendList() {
              }
};

class XXLFM_FrameList : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField MF<int> index;
         exposedField MF<Vec3f> frame;

         public:
              XXLFM_FrameList() {
                   index += -1;
                   frame += 1;
                   frame += 0;
                   frame += 0;
                   frame += 1;
                   frame += 1;
                   frame += 0;
                   frame += 0;
                   frame += 0;
                   frame += 1;
              }
};

class XXLFM_LightMap : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<Vec3f> biasRGB;
         exposedField SF<int> priorityLevel;
         exposedField SF<Vec3f> scaleRGB;
         exposedField SFNode<class XXLFM_SurfaceMapList> surfaceMapList;
         exposedField SFNode<class XXLFM_ViewMapList> viewMapList;

         public:
              XXLFM_LightMap() {
                   biasRGB = Vec3f(0, 0, 0);
                   priorityLevel = 0;
                   scaleRGB = Vec3f(1, 1, 1);
              }
};

class XXLFM_SurfaceMapList : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField MF<int> tileIndex;
         exposedField SFTextureCoordinateNode triangleCoordinate;
         exposedField MF<int> triangleIndex;
         exposedField MF<int> viewMapIndex;

         public:
              XXLFM_SurfaceMapList() {
              }
};

class XXLFM_ViewMapList : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SFTextureCoordinateNode textureOrigin;
         exposedField SFTextureCoordinateNode textureSize;
         exposedField MF<int> tileIndex;
         exposedField MF<int> vertexIndex;

         public:
              XXLFM_ViewMapList() {
              }
};

class MeshGrid : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn MF<int> set_colorIndex;
         eventIn MF<int> set_coordIndex;
         eventIn MF<int> set_normalIndex;
         eventIn MF<int> set_texCoordIndex;
         exposedField SFNode<class Color> color;
         exposedField SFNode<class Coordinate> coord;
         exposedField SF<int> displayLevel;
         exposedField SF<int> filterType;
         exposedField SFNode<class Coordinate> gridCoord;
         exposedField SF<int> hierarchicalLevel;
         exposedField MF<int> nLevels;
         exposedField SFNode<class Normal> normal;
         exposedField MF<int> nSlices;
         exposedField SFTextureCoordinateNode texCoord;
         exposedField MF<float> vertexOffset;
         exposedField MF<int> vertexLink;
         FIELD MF<int> colorIndex;
         FIELD MF<int> coordIndex;
         FIELD MF<int> normalIndex;
         FIELD SF<Bool> solid;
         FIELD MF<int> texCoordIndex;
         eventOut SF<Bool> isLoading;
         eventOut MF<int> nVertices;
         void Onset_colorIndex();
         void Onset_coordIndex();
         void Onset_normalIndex();
         void Onset_texCoordIndex();

         public:
              MeshGrid() {
                   displayLevel = 0;
                   filterType = 0;
                   hierarchicalLevel = 0;
                   solid = TRUE;
              }
};

class NonLinearDeformer : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<Vec3f> axis;
         exposedField MF<float> extend;
         exposedField SFGeometryNode geometry;
         exposedField SF<float> param;
         exposedField SF<int> type;

         public:
              NonLinearDeformer() {
                   axis = Vec3f(0, 0, 1);
                   param = 0;
                   type = 0;
              }
};

class NurbsCurve : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn MF<int> set_colorIndex;
         exposedField SFNode<class Color> color;
         exposedField MF<Vec4f> controlPoint;
         exposedField SF<int> tessellation;
         FIELD MF<int> colorIndex;
         FIELD SF<Bool> colorPerVertex;
         FIELD MF<float> knot;
         FIELD SF<int> order;
         void Onset_colorIndex();

         public:
              NurbsCurve() {
                   tessellation = 0;
                   colorPerVertex = TRUE;
                   order = 4;
              }
};

class NurbsCurve2D : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn MF<int> set_colorIndex;
         exposedField SFNode<class Color> color;
         exposedField MF<Vec3f> controlPoint;
         exposedField SF<int> tessellation;
         FIELD MF<int> colorIndex;
         FIELD SF<Bool> colorPerVertex;
         FIELD MF<float> knot;
         FIELD SF<int> order;
         void Onset_colorIndex();

         public:
              NurbsCurve2D() {
                   tessellation = 0;
                   colorPerVertex = TRUE;
                   order = 4;
              }
};

class NurbsSurface : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn MF<int> set_colorIndex;
         eventIn MF<int> set_texColorIndex;
         exposedField SFNode<class Color> color;
         exposedField MF<Vec4f> controlPoint;
         exposedField SFTextureCoordinateNode texCoord;
         exposedField SF<int> uTessellation;
         exposedField SF<int> vTessellation;
         FIELD SF<Bool> ccw;
         FIELD MF<int> colorIndex;
         FIELD SF<Bool> colorPerVertex;
         FIELD SF<Bool> solid;
         FIELD MF<int> texColorIndex;
         FIELD SF<int> uDimension;
         FIELD MF<float> uKnot;
         FIELD SF<int> uOrder;
         FIELD SF<int> vDimension;
         FIELD MF<float> vKnot;
         FIELD SF<int> vOrder;
         void Onset_colorIndex();
         void Onset_texColorIndex();

         public:
              NurbsSurface() {
                   uTessellation = 0;
                   vTessellation = 0;
                   ccw = TRUE;
                   colorPerVertex = TRUE;
                   solid = TRUE;
                   uDimension = 4;
                   uOrder = 4;
                   vDimension = 4;
                   vOrder = 4;
              }
};

class OctreeImage : public MediaObject
{
         BIFS_DECLARE_NODE

         FIELD MFNodeType<SFDepthImageNode> images;
         FIELD MF<int> octree;
         FIELD SF<int> octreeResolution;
         FIELD MF<int> voxelImageIndex;

         public:
              OctreeImage() {
                   octreeResolution = 256;
              }
};

class XXParticles : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<float> creationRate;
         exposedField SF<float> creationRateVariation;
         exposedField SF<float> emitAlpha;
         exposedField SF<Color3f> emitColor;
         exposedField SF<Color3f> emitColorVariation;
         exposedField SF<Vec3f> emitterPosition;
         exposedField SF<Vec3f> emitVelocity;
         exposedField SF<Vec3f> emitVelocityVariation;
         exposedField SF<Bool> enabled;
         exposedField SF<float> fadeAlpha;
         exposedField SF<Color3f> fadeColor;
         exposedField SF<float> fadeRate;
         exposedField SF<Vec3f> force;
         exposedField MFNodeType<SFInfluenceNode> influences;
         exposedField SFWorldNode init;
         exposedField SF<Time> maxLifeTime;
         exposedField SF<float> maxLifeTimeVariation;
         exposedField SF<int> maxParticles;
         exposedField SF<float> minRange;
         exposedField SF<float> maxRange;
         exposedField SFWorldNode primitive;
         exposedField SF<int> primitiveType;
         exposedField SF<float> particleRadius;
         exposedField SF<float> particleRadiusRate;
         exposedField SF<float> particleRadiusVariation;

         public:
              XXParticles() {
                   creationRate = 500;
                   creationRateVariation = 0;
                   emitAlpha = 1;
                   emitColor = Color3f(1, 1, 1);
                   emitColorVariation = Color3f(0, 0, 0);
                   emitterPosition = Vec3f(0, 3, 0);
                   emitVelocity = Vec3f(0, 0, 0);
                   emitVelocityVariation = Vec3f(1, 1, 1);
                   enabled = TRUE;
                   fadeAlpha = 1;
                   fadeColor = Color3f(0.25, 0.25, 0.25);
                   fadeRate = 0.25;
                   force = Vec3f(0, -9.8, 0);
                   maxLifeTime = 5;
                   maxLifeTimeVariation = 0;
                   maxParticles = 500;
                   minRange = 1;
                   maxRange = -1;
                   primitiveType = 2;
                   particleRadius = 0.1;
                   particleRadiusRate = 0;
                   particleRadiusVariation = 0;
              }
};

class XXParticleInitBox : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<float> falloff;
         exposedField SF<Vec3f> size;

         public:
              XXParticleInitBox() {
                   falloff = 0;
                   size = Vec3f(1, 1, 1);
              }
};

class XXPlanarObstacle : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<Vec3f> distance;
         exposedField SF<Vec3f> normal;
         exposedField SF<float> reflection;
         exposedField SF<float> absorption;

         public:
              XXPlanarObstacle() {
                   distance = Vec3f(0, 0, 0);
                   normal = Vec3f(0, 1, 0);
                   reflection = 0;
                   absorption = 0;
              }
};

class XXPointAttractor : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<float> innerRadius;
         exposedField SF<float> outerRadius;
         exposedField SF<Vec3f> position;
         exposedField SF<float> rate;

         public:
              XXPointAttractor() {
                   innerRadius = 10;
                   outerRadius = 100;
                   position = Vec3f(0, 0, 0);
                   rate = 1;
              }
};

class PointTexture : public MediaObject
{
         BIFS_DECLARE_NODE

         FIELD MF<Color3f> color;
         FIELD MF<int> depth;
         FIELD SF<int> depthNbBits;
         FIELD SF<int> height;
         FIELD SF<int> width;

         public:
              PointTexture() {
                   depthNbBits = 7;
                   height = 256;
                   width = 256;
              }
};

class PositionAnimator : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn SF<float> set_fraction;
         exposedField SF<Vec2f> fromTo;
         exposedField MF<float> key;
         exposedField MF<Rotation> keyOrientation;
         exposedField SF<int> keyType;
         exposedField MF<Vec2f> keySpline;
         exposedField MF<Vec3f> keyValue;
         exposedField SF<int> keyValueType;
         exposedField SF<Vec3f> offset;
         exposedField MF<float> weight;
         eventOut SF<Vec3f> endValue;
         eventOut SF<Rotation> rotation_changed;
         eventOut SF<Vec3f> value_changed;
         void Onset_fraction();

         public:
              PositionAnimator() {
                   fromTo = Vec2f(0, 1);
                   keyType = 0;
                   keySpline += 0;
                   keySpline += 0;
                   keySpline += 1;
                   keySpline += 1;
                   keyValueType = 0;
                   offset = Vec3f(0, 0, 0);
              }
};

class PositionAnimator2D : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn SF<float> set_fraction;
         exposedField SF<Vec2f> fromTo;
         exposedField MF<float> key;
         exposedField SF<int> keyOrientation;
         exposedField SF<int> keyType;
         exposedField MF<Vec2f> keySpline;
         exposedField MF<Vec2f> keyValue;
         exposedField SF<int> keyValueType;
         exposedField SF<Vec2f> offset;
         exposedField MF<float> weight;
         eventOut SF<Vec2f> endValue;
         eventOut SF<float> rotation_changed;
         eventOut SF<Vec2f> value_changed;
         void Onset_fraction();

         public:
              PositionAnimator2D() {
                   fromTo = Vec2f(0, 1);
                   keyOrientation = 0;
                   keyType = 0;
                   keySpline += 0;
                   keySpline += 0;
                   keySpline += 1;
                   keySpline += 1;
                   keyValueType = 0;
                   offset = Vec2f(0, 0);
              }
};

class PositionInterpolator4D : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn SF<float> set_fraction;
         exposedField MF<float> key;
         exposedField MF<Vec4f> keyValue;
         eventOut SF<Vec4f> value_changed;
         void Onset_fraction();

         public:
              PositionInterpolator4D() {
              }
};

class ProceduralTexture : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<Bool> aSmooth;
         exposedField MF<Vec2f> aWarpmap;
         exposedField MF<float> aWeights;
         exposedField SF<Bool> bSmooth;
         exposedField MF<Vec2f> bWarpmap;
         exposedField MF<float> bWeights;
         exposedField SF<int> cellWidth;
         exposedField SF<int> cellHeight;
         exposedField MF<Color3f> color;
         exposedField SF<float> distortion;
         exposedField SF<int> height;
         exposedField SF<int> roughness;
         exposedField SF<int> seed;
         exposedField SF<int> type;
         exposedField SF<Bool> xSmooth;
         exposedField MF<Vec2f> xWarpmap;
         exposedField SF<Bool> ySmooth;
         exposedField MF<Vec2f> yWarpmap;
         exposedField SF<int> width;
         eventOut SF<Image> image_changed;

         public:
              ProceduralTexture() {
                   aSmooth = FALSE;
                   aWarpmap += 0;
                   aWarpmap += 0;
                   aWarpmap += 1;
                   aWarpmap += 1;
                   aWeights += 0;
                   aWeights += 0;
                   aWeights += 0;
                   aWeights += 0;
                   aWeights += 0;
                   aWeights += 0;
                   aWeights += 0;
                   aWeights += 0;
                   aWeights += 0;
                   aWeights += 0;
                   aWeights += 0;
                   aWeights += 0;
                   aWeights += 1;
                   aWeights += 0;
                   aWeights += 0;
                   aWeights += 0;
                   bSmooth = FALSE;
                   bWarpmap += 0;
                   bWarpmap += 0;
                   bWarpmap += 1;
                   bWarpmap += 1;
                   bWeights += 0;
                   bWeights += 0;
                   bWeights += 0;
                   bWeights += 0;
                   bWeights += 0;
                   bWeights += 1;
                   bWeights += 0;
                   bWeights += 0;
                   bWeights += 0;
                   bWeights += 0;
                   bWeights += 0;
                   bWeights += 0;
                   bWeights += 0;
                   bWeights += 0;
                   bWeights += 0;
                   bWeights += 0;
                   cellWidth = 4;
                   cellHeight = 4;
                   color += 0.3;
                   color += 0.698;
                   color += 1;
                   color += 0.8;
                   color += 0.8;
                   color += 0.8;
                   color += 1;
                   color += 1;
                   color += 1;
                   color += 0;
                   color += 0;
                   color += 0;
                   distortion = 0;
                   height = 7;
                   roughness = 0;
                   seed = 129093;
                   type = 0;
                   xSmooth = FALSE;
                   ySmooth = FALSE;
                   width = 7;
              }
};

class Quadric : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<Vec3f> bboxSize;
         exposedField MF<int> densities;
         exposedField SF<Bool> dual;
         exposedField SF<Vec4f> P0;
         exposedField SF<Vec4f> P1;
         exposedField SF<Vec4f> P2;
         exposedField SF<Vec4f> P3;
         exposedField SF<Vec4f> P4;
         exposedField SF<Vec4f> P5;
         exposedField SF<Bool> solid;

         public:
              Quadric() {
                   bboxSize = Vec3f(2, 2, 2);
                   dual = FALSE;
                   P0 = Vec4f(-1, 0, 0, 1);
                   P1 = Vec4f(1, 0, 0, 1);
                   P2 = Vec4f(0, 1, 0, 0);
                   P3 = Vec4f(0, 0, 1, 0);
                   P4 = Vec4f(0, 1, 0, 1);
                   P5 = Vec4f(0, 0, 1, 1);
                   solid = FALSE;
              }
};

class SBBone : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn SF3DNode addChildren;
         eventIn SF3DNode removeChildren;
         exposedField SF<int> boneID;
         exposedField SF<Vec3f> center;
         exposedField MFNodeType<SF3DNode> children;
         exposedField SF<Vec3f> endpoint;
         exposedField SF<int> falloff;
         exposedField SF<int> ikChainPosition;
         exposedField MF<float> ikPitchLimit;
         exposedField MF<float> ikRollLimit;
         exposedField MF<float> ikTxLimit;
         exposedField MF<float> ikTyLimit;
         exposedField MF<float> ikTzLimit;
         exposedField MF<float> ikYawLimit;
         exposedField SF<Rotation> rotation;
         exposedField SF<int> rotationOrder;
         exposedField SF<Vec3f> scale;
         exposedField SF<Rotation> scaleOrientation;
         exposedField MF<float> sectionInner;
         exposedField MF<float> sectionOuter;
         exposedField MF<float> sectionPosition;
         exposedField MF<int> skinCoordIndex;
         exposedField MF<float> skinCoordWeight;
         exposedField SF<Vec3f> translation;
         void OnaddChildren();
         void OnremoveChildren();

         public:
              SBBone() {
                   boneID = 0;
                   center = Vec3f(0, 0, 0);
                   endpoint = Vec3f(0, 0, 1);
                   falloff = 1;
                   ikChainPosition = 0;
                   rotation = Rotation(0, 0, 1, 0);
                   rotationOrder = 0;
                   scale = Vec3f(1, 1, 1);
                   scaleOrientation = Rotation(0, 0, 1, 0);
                   translation = Vec3f(0, 0, 0);
              }
};

class SBMuscle : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<int> falloff;
         exposedField SFGeometryNode muscleCurve;
         exposedField SF<int> muscleID;
         exposedField SF<int> radius;
         exposedField MF<int> skinCoordIndex;
         exposedField MF<float> skinCoordWeight;

         public:
              SBMuscle() {
                   falloff = 1;
                   muscleID = 0;
                   radius = 1;
              }
};

class SBSegment : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn MFNodeType<SF3DNode> addChildren;
         eventIn MFNodeType<SF3DNode> removeChildren;
         exposedField SF<Vec3f> centerOfMass;
         exposedField MFNodeType<SF3DNode> children;
         exposedField SF<float> mass;
         exposedField MF<Vec3f> momentsOfInertia;
         exposedField SFString name;
         void OnaddChildren();
         void OnremoveChildren();

         public:
              SBSegment() {
                   centerOfMass = Vec3f(0, 0, 0);
                   mass = 0;
                   momentsOfInertia += 0;
                   momentsOfInertia += 0;
                   momentsOfInertia += 0;
                   momentsOfInertia += 0;
                   momentsOfInertia += 0;
                   momentsOfInertia += 0;
                   momentsOfInertia += 0;
                   momentsOfInertia += 0;
                   momentsOfInertia += 0;
              }
};

class SBSite : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn MFNodeType<SF3DNode> addChildren;
         eventIn MFNodeType<SF3DNode> removeChildren;
         exposedField SF<Vec3f> center;
         exposedField MFNodeType<SF3DNode> children;
         exposedField SFString name;
         exposedField SF<Rotation> rotation;
         exposedField SF<Vec3f> scale;
         exposedField SF<Rotation> scaleOrientation;
         exposedField SF<Vec3f> translation;
         void OnaddChildren();
         void OnremoveChildren();

         public:
              SBSite() {
                   center = Vec3f(0, 0, 0);
                   rotation = Rotation(0, 0, 1, 0);
                   scale = Vec3f(1, 1, 1);
                   scaleOrientation = Rotation(0, 0, 1, 0);
                   translation = Vec3f(0, 0, 0);
              }
};

class SBSkinnedModel : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField MFNode<class SBBone> bones;
         exposedField SF<Vec3f> center;
         exposedField MFNode<class SBMuscle> muscles;
         exposedField SFString name;
         exposedField SF<Rotation> rotation;
         exposedField MFNode<class SBSegment> segments;
         exposedField SF<Vec3f> scale;
         exposedField SF<Rotation> scaleOrientation;
         exposedField MFNode<class SBSite> sites;
         exposedField MFNodeType<SF3DNode> skeleton;
         exposedField MFNodeType<SF3DNode> skin;
         exposedField SFNode<class Coordinate> skinCoord;
         exposedField SFNode<class Normal> skinNormal;
         exposedField SF<Vec3f> translation;
         exposedField SF3DNode weighsComputationSkinCoord;

         public:
              SBSkinnedModel() {
                   center = Vec3f(0, 0, 0);
                   rotation = Rotation(0, 0, 1, 0);
                   scale = Vec3f(1, 1, 1);
                   scaleOrientation = Rotation(0, 0, 1, 0);
                   translation = Vec3f(0, 0, 0);
              }
};

class SBVCAnimation : public StreamConsumer
{
         BIFS_DECLARE_NODE

         exposedField MFNodeType<SF3DNode> virtualCharacters;

         public:
              SBVCAnimation() {
              }
};

class ScalarAnimator : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn SF<float> set_fraction;
         exposedField SF<Vec2f> fromTo;
         exposedField MF<float> key;
         exposedField SF<int> keyType;
         exposedField MF<Vec2f> keySpline;
         exposedField MF<float> keyValue;
         exposedField SF<int> keyValueType;
         exposedField SF<float> offset;
         exposedField MF<float> weight;
         eventOut SF<float> endValue;
         eventOut SF<float> value_changed;
         void Onset_fraction();

         public:
              ScalarAnimator() {
                   fromTo = Vec2f(0, 1);
                   keyType = 0;
                   keySpline += 0;
                   keySpline += 0;
                   keySpline += 1;
                   keySpline += 1;
                   keyValueType = 0;
                   offset = 0;
              }
};

class SimpleTexture : public MediaObject
{
         BIFS_DECLARE_NODE

         FIELD SFTextureNode depth;
         FIELD SFTextureNode texture;

         public:
              SimpleTexture() {
              }
};

class SolidRep : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<Vec3f> bboxSize;
         exposedField MF<int> densityList;
         exposedField SF3DNode solidTree;

         public:
              SolidRep() {
                   bboxSize = Vec3f(2, 2, 2);
              }
};

class SubdivisionSurface : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn MF<int> set_colorIndex;
         eventIn MF<int> set_coordIndex;
         eventIn MF<int> set_cornerVertexIndex;
         eventIn MF<int> set_creaseEdgeIndex;
         eventIn MF<int> set_creaseVertexIndex;
         eventIn MF<int> set_dartVertexIndex;
         eventIn MF<int> set_texCoordIndex;
         exposedField SFNode<class Color> color;
         exposedField SFNode<class Coordinate> coord;
         exposedField SFTextureCoordinateNode texCoord;
         exposedField MFNode<class SubdivSurfaceSector> sectors;
         exposedField SF<int> subdivisionLevel;
         exposedField SF<int> subdivisionType;
         exposedField SF<int> subdivisionSubType;
         FIELD MF<int> invisibleEdgeIndex;
         FIELD SF<Bool> ccw;
         FIELD MF<int> colorIndex;
         FIELD SF<Bool> colorPerVertex;
         FIELD SF<Bool> convex;
         FIELD MF<int> coordIndex;
         FIELD MF<int> cornerVertexIndex;
         FIELD MF<int> creaseEdgeIndex;
         FIELD MF<int> creaseVertexIndex;
         FIELD MF<int> dartVertexIndex;
         FIELD SF<Bool> solid;
         FIELD MF<int> texCoordIndex;
         void Onset_colorIndex();
         void Onset_coordIndex();
         void Onset_cornerVertexIndex();
         void Onset_creaseEdgeIndex();
         void Onset_creaseVertexIndex();
         void Onset_dartVertexIndex();
         void Onset_texCoordIndex();

         public:
              SubdivisionSurface() {
                   subdivisionLevel = 0;
                   subdivisionType = 0;
                   subdivisionSubType = 0;
                   ccw = TRUE;
                   colorPerVertex = TRUE;
                   convex = TRUE;
                   solid = TRUE;
              }
};

class SubdivSurfaceSector : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<float> flatness;
         exposedField SF<Vec3f> normal;
         exposedField SF<float> normalTension;
         exposedField SF<int> tag;
         exposedField SF<float> theta;
         FIELD SF<int> faceIndex;
         FIELD SF<int> vertexIndex;

         public:
              SubdivSurfaceSector() {
                   flatness = 0;
                   normal = Vec3f(0, 0, 0);
                   normalTension = 0;
                   tag = 0;
                   theta = 0;
                   faceIndex = 0;
                   vertexIndex = 0;
              }
};

class WaveletSubdivisionSurface : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SFGeometryNode baseMesh;
         exposedField SF<float> fieldOfView;
         exposedField SF<float> frequency;
         exposedField SF<int> quality;

         public:
              WaveletSubdivisionSurface() {
                   fieldOfView = 0.785398;
                   frequency = 1;
                   quality = 1;
              }
};

class Clipper2D : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn MFNodeType<SF2DNode> addChildren;
         eventIn MFNodeType<SF2DNode> removeChildren;
         exposedField MFNodeType<SF2DNode> children;
         exposedField SFGeometryNode geometry;
         exposedField SF<Bool> inside;
         exposedField SF2DNode transform;
         exposedField SF<Bool> XOR;
         void OnaddChildren();
         void OnremoveChildren();

         public:
              Clipper2D() {
                   inside = TRUE;
                   XOR = FALSE;
              }
};

class ColorTransform : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn MFNodeType<SF3DNode> addChildren;
         eventIn MFNodeType<SF3DNode> removeChildren;
         exposedField MFNodeType<SF3DNode> children;
         exposedField SF<float> mrr;
         exposedField SF<float> mrg;
         exposedField SF<float> mrb;
         exposedField SF<float> mra;
         exposedField SF<float> tr;
         exposedField SF<float> mgr;
         exposedField SF<float> mgg;
         exposedField SF<float> mgb;
         exposedField SF<float> mga;
         exposedField SF<float> tg;
         exposedField SF<float> mbr;
         exposedField SF<float> mbg;
         exposedField SF<float> mbb;
         exposedField SF<float> mba;
         exposedField SF<float> tb;
         exposedField SF<float> mar;
         exposedField SF<float> mag;
         exposedField SF<float> mab;
         exposedField SF<float> maa;
         exposedField SF<float> ta;
         void OnaddChildren();
         void OnremoveChildren();

         public:
              ColorTransform() {
                   mrr = 1;
                   mrg = 0;
                   mrb = 0;
                   mra = 0;
                   tr = 0;
                   mgr = 0;
                   mgg = 1;
                   mgb = 0;
                   mga = 0;
                   tg = 0;
                   mbr = 0;
                   mbg = 0;
                   mbb = 1;
                   mba = 0;
                   tb = 0;
                   mar = 0;
                   mag = 0;
                   mab = 0;
                   maa = 1;
                   ta = 0;
              }
};

class XEllipse : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<Vec2f> radius;

         public:
              XEllipse() {
                   radius = Vec2f(1, 1);
              }
};

class Ellipse : public XEllipse
{
  protected: virtual char *GetName () {return "Ellipse";}
};

class LinearGradient : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<Vec2f> endPoint;
         exposedField MF<float> key;
         exposedField MF<Color3f> keyValue;
         exposedField MF<float> opacity;
         exposedField SF<int> spreadMethod;
         exposedField SF<Vec2f> startPoint;
         exposedField SF3DNode transform;

         public:
              LinearGradient() {
                   endPoint = Vec2f(1, 0);
                   opacity += 1;
                   spreadMethod = 0;
                   startPoint = Vec2f(0, 0);
              }
};

class PathLayout : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn MFNodeType<SF2DNode> addChildren;
         eventIn MFNodeType<SF2DNode> removeChildren;
         exposedField MFNodeType<SF2DNode> children;
         exposedField SFGeometryNode geometry;
         exposedField MF<int> alignment;
         exposedField SF<float> pathOffset;
         exposedField SF<float> spacing;
         exposedField SF<Bool> reverseLayout;
         exposedField SF<int> wrapMode;
         exposedField SF<Bool> splitText;
         void OnaddChildren();
         void OnremoveChildren();

         public:
              PathLayout() {
                   alignment += 0;
                   alignment += 0;
                   pathOffset = 0;
                   spacing = 1;
                   reverseLayout = FALSE;
                   wrapMode = 0;
                   splitText = TRUE;
              }
};

class RadialGradient : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<Vec2f> center;
         exposedField SF<Vec2f> focalPoint;
         exposedField MF<float> key;
         exposedField MF<Color3f> keyValue;
         exposedField MF<float> opacity;
         exposedField SF<float> radius;
         exposedField SF<int> spreadMethod;
         exposedField SF3DNode transform;

         public:
              RadialGradient() {
                   center = Vec2f(0.5, 0.5);
                   focalPoint = Vec2f(0, 0);
                   opacity += 1;
                   radius = 0.5;
                   spreadMethod = 0;
              }
};

class SynthesizedTexture : public StreamConsumer
{
         BIFS_DECLARE_NODE

         exposedField MF<Vec3f> translation;
         exposedField MF<Rotation> rotation;
         exposedField SF<int> pixelWidth;
         exposedField SF<int> pixelHeight;
         exposedField SF<Bool> loop;
         exposedField SF<float> speed;
         exposedField SFTimeField startTime;
         exposedField SFTimeField stopTime;
         eventOut SF<Time> duration_changed;
         eventOut SF<Bool> isActive;

         public:
              SynthesizedTexture() {
                   pixelWidth = -1;
                   pixelHeight = -1;
                   loop = FALSE;
                   speed = 1;
                   startTime = 0;
                   stopTime = 0;
              }
};

class TransformMatrix2D : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn MFNodeType<SF2DNode> addChildren;
         eventIn MFNodeType<SF2DNode> removeChildren;
         exposedField MFNodeType<SF2DNode> children;
         exposedField SF<float> mxx;
         exposedField SF<float> mxy;
         exposedField SF<float> tx;
         exposedField SF<float> myx;
         exposedField SF<float> myy;
         exposedField SF<float> ty;
         void OnaddChildren();
         void OnremoveChildren();

         public:
              TransformMatrix2D() {
                   mxx = 1;
                   mxy = 0;
                   tx = 0;
                   myx = 0;
                   myy = 1;
                   ty = 0;
              }
};

class Viewport : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn SF<Bool> set_bind;
         exposedField SF<Vec2f> position;
         exposedField SF<Vec2f> size;
         exposedField SF<float> orientation;
         exposedField MF<int> alignment;
         exposedField SF<int> fit;
         FIELD SFString description;
         eventOut SF<Time> bindTime;
         eventOut SF<Bool> isBound;
         void Onset_bind();

         public:
              Viewport() {
                   position = Vec2f(0, 0);
                   size = Vec2f(-1, -1);
                   orientation = 0;
                   alignment += 0;
                   alignment += 0;
                   fit = 0;
              }
};

class XCurve2D : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SFNode<class Coordinate2D> point;
         exposedField SF<float> fineness;
         exposedField MF<int> type;

         public:
              XCurve2D() {
                   fineness = 0.5;
              }
};

class XFontStyle : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField MF<ZString> fontName;
         exposedField SF<Bool> horizontal;
         exposedField MF<ZString> justify;
         exposedField SFString language;
         exposedField SF<Bool> leftToRight;
         exposedField SF<float> size;
         exposedField SFString stretch;
         exposedField SF<float> letterSpacing;
         exposedField SF<float> wordSpacing;
         exposedField SF<int> weight;
         exposedField SF<Bool> fontKerning;
         exposedField SFString style;
         exposedField SF<Bool> topToBottom;
         exposedField MF<ZString> featureName;
         exposedField MF<int> featureStartOffset;
         exposedField MF<int> featureLength;
         exposedField MF<int> featureValue;

         public:
              XFontStyle() {
                   fontName += ZString("SERIF");
                   horizontal = TRUE;
                   justify += ZString("BEGIN");
                   leftToRight = TRUE;
                   size = 1;
                   stretch = ZString("NORMAL");
                   letterSpacing = 0;
                   wordSpacing = 0;
                   weight = 400;
                   fontKerning = TRUE;
                   style = ZString("PLAIN");
                   topToBottom = TRUE;
                   featureName += ZString("");
              }
};

class XLineProperties : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<Color3f> lineColor;
         exposedField SF<int> lineStyle;
         exposedField SF<Bool> isCenterAligned;
         exposedField SF<Bool> isScalable;
         exposedField SF<int> lineCap;
         exposedField SF<int> lineJoin;
         exposedField SF<float> miterLimit;
         exposedField SF<float> transparency;
         exposedField SF<float> width;
         exposedField SF<float> dashOffset;
         exposedField MF<float> dashes;
         exposedField SFTextureNode texture;
         exposedField SFTextureTransformNode textureTransform;

         public:
              XLineProperties() {
                   lineColor = Color3f(0, 0, 0);
                   lineStyle = 0;
                   isCenterAligned = TRUE;
                   isScalable = TRUE;
                   lineCap = 0;
                   lineJoin = 0;
                   miterLimit = 4;
                   transparency = 0;
                   width = 1;
                   dashOffset = 0;
              }
};

class AdvancedAudioBuffer : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn MFNodeType<SFAudioNode> addChildren;
         eventIn MFNodeType<SFAudioNode> removeChildren;
         exposedField MFNodeType<SFAudioNode> children;
         exposedField SF<Bool> loop;
         exposedField SF<float> pitch;
         exposedField SFTimeField startTime;
         exposedField SFTimeField stopTime;
         exposedField SF<Time> startLoadTime;
         exposedField SF<Time> stopLoadTime;
         exposedField SF<int> loadMode;
         exposedField SF<int> numAccumulatedBlocks;
         exposedField SF<int> deleteBlock;
         exposedField SF<int> playBlock;
         exposedField SF<float> length;
         FIELD SF<int> numChan;
         FIELD MF<int> phaseGroup;
         eventOut SF<Time> duration_changed;
         eventOut SF<Bool> isActive;
         void OnaddChildren();
         void OnremoveChildren();

         public:
              AdvancedAudioBuffer() {
                   loop = FALSE;
                   pitch = 1;
                   startTime = 0;
                   stopTime = 0;
                   startLoadTime = 0;
                   stopLoadTime = 0;
                   loadMode = 0;
                   numAccumulatedBlocks = 0;
                   deleteBlock = 0;
                   playBlock = 0;
                   length = 0;
                   numChan = 1;
              }
};

class AudioChannelConfig : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn MFNodeType<SFAudioNode> addChildren;
         eventIn MFNodeType<SFAudioNode> removeChildren;
         exposedField MFNodeType<SFAudioNode> children;
         exposedField SF<int> generalChannelFormat;
         exposedField SF<int> fixedPreset;
         exposedField SF<int> fixedPresetSubset;
         exposedField SF<int> fixedPresetAddInf;
         exposedField MF<int> channelCoordinateSystems;
         exposedField MF<float> channelSoundLocation;
         exposedField MF<int> channelDirectionalPattern;
         exposedField MF<Vec3f> channelDirection;
         exposedField SF<int> ambResolution2D;
         exposedField SF<int> ambResolution3D;
         exposedField SF<int> ambEncodingConvention;
         exposedField SF<float> ambNfcReferenceDistance;
         exposedField SF<float> ambSoundSpeed;
         exposedField SF<int> ambArrangementRule;
         exposedField SF<int> ambRecombinationPreset;
         exposedField MF<int> ambComponentIndex;
         exposedField MF<float> ambBackwardMatrix;
         exposedField MF<int> ambSoundfieldResolution;
         FIELD SF<int> numChannel;
         void OnaddChildren();
         void OnremoveChildren();

         public:
              AudioChannelConfig() {
                   generalChannelFormat = 0;
                   fixedPreset = 0;
                   fixedPresetSubset = 0;
                   fixedPresetAddInf = 0;
                   ambResolution2D = 1;
                   ambResolution3D = 0;
                   ambEncodingConvention = 0;
                   ambNfcReferenceDistance = 1.5;
                   ambSoundSpeed = 340;
                   ambArrangementRule = 0;
                   ambRecombinationPreset = 0;
                   numChannel = 0;
              }
};

class DepthImageV2 : public MediaObject
{
         BIFS_DECLARE_NODE

         FIELD SFDepthTextureNode diTexture;
         FIELD SF<float> farPlane;
         FIELD SF<Vec2f> fieldOfView;
         FIELD SF<float> nearPlane;
         FIELD SF<Rotation> orientation;
         FIELD SF<Bool> orthographic;
         FIELD SF<Vec3f> position;
         FIELD SF<Vec2f> splatMinMax;

         public:
              DepthImageV2() {
                   farPlane = 100;
                   fieldOfView = Vec2f(0.75, 0.75);
                   nearPlane = 10;
                   orientation = Rotation(0, 0, 1, 0);
                   orthographic = TRUE;
                   position = Vec3f(0, 0, 10);
                   splatMinMax = Vec2f(0.115, 0.975);
              }
};

class MorphShape : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF3DNode baseShape;
         exposedField SF<int> morphID;
         exposedField MFNodeType<SF3DNode> targetShapes;
         exposedField MF<float> weights;

         public:
              MorphShape() {
                   morphID = 0;
              }
};

class MultiTexture : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SF<float> alpha;
         exposedField SF<Color3f> color;
         exposedField MF<int> function;
         exposedField MF<int> mode;
         exposedField MF<int> source;
         exposedField MFNodeType<SFTextureNode> texture;
         exposedField MF<Vec3f> cameraVector;
         exposedField SF<Bool> transparent;

         public:
              MultiTexture() {
                   alpha = 1;
                   color = Color3f(1, 1, 1);
                   transparent = FALSE;
              }
};

class MultiTextureCoordinate : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField MFNodeType<SFTextureCoordinateNode> texCoord;

         public:
              MultiTextureCoordinate() {
              }
};

class PointTextureV2 : public MediaObject
{
         BIFS_DECLARE_NODE

         FIELD MF<Color3f> color;
         FIELD MF<int> depth;
         FIELD SF<int> depthNbBits;
         FIELD SF<int> height;
         FIELD SFNode<class Normal> normal;
         FIELD MF<Vec3f> splatU;
         FIELD MF<Vec3f> splatV;
         FIELD SF<int> width;

         public:
              PointTextureV2() {
                   depthNbBits = 7;
                   height = 256;
                   width = 256;
              }
};

class SBVCAnimationV2 : public StreamConsumer
{
         BIFS_DECLARE_NODE

         exposedField MF<int> activeUrlIndex;
         exposedField SF<Bool> loop;
         exposedField SF<float> speed;
         exposedField SFTimeField startTime;
         exposedField SFTimeField stopTime;
         exposedField SF<float> transitionTime;
         exposedField MFNodeType<SF3DNode> virtualCharacters;
         eventOut SF<Time> duration_changed;
         eventOut SF<Bool> isActive;

         public:
              SBVCAnimationV2() {
                   loop = FALSE;
                   speed = 1;
                   startTime = 0;
                   stopTime = 0;
                   transitionTime = 0;
              }
};

class SimpleTextureV2 : public MediaObject
{
         BIFS_DECLARE_NODE

         FIELD SFTextureNode depth;
         FIELD SFTextureNode normal;
         FIELD SFTextureNode splatU;
         FIELD SFTextureNode splatV;
         FIELD SFTextureNode texture;

         public:
              SimpleTextureV2() {
              }
};

class SurroundingSound : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SFAudioNode source;
         exposedField SF<float> intensity;
         exposedField SF<float> distance;
         exposedField SF<Vec3f> location;
         exposedField SF<float> distortionFactor;
         exposedField SF<Rotation> orientation;
         exposedField SF<Bool> isTransformable;

         public:
              SurroundingSound() {
                   intensity = 1;
                   distance = 0;
                   location = Vec3f(0, 0, 0);
                   distortionFactor = 0;
                   orientation = Rotation(0, 0, 1, 0);
                   isTransformable = TRUE;
              }
};

class Transform3DAudio : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn MFNodeType<SF3DNode> addChildren;
         eventIn MFNodeType<SF3DNode> removeChildren;
         exposedField MFNodeType<SF3DNode> children;
         exposedField SF<float> thirdCenterCoordinate;
         exposedField SF<Vec3f> rotationVector;
         exposedField SF<float> thirdScaleCoordinate;
         exposedField SF<Vec3f> scaleOrientationVector;
         exposedField SF<float> thirdTranslationCoordinate;
         exposedField SF<Rotation> coordinateTransform;
         void OnaddChildren();
         void OnremoveChildren();

         public:
              Transform3DAudio() {
                   thirdCenterCoordinate = 0;
                   rotationVector = Vec3f(0, 0, 1);
                   thirdScaleCoordinate = 0;
                   scaleOrientationVector = Vec3f(0, 0, 1);
                   thirdTranslationCoordinate = 0;
                   coordinateTransform = Rotation(1, 0, 0, PI/2);
              }
};

class WideSound : public MediaObject
{
         BIFS_DECLARE_NODE

         exposedField SFAudioNode source;
         exposedField SF<float> intensity;
         exposedField SF<Vec3f> location;
         exposedField SF<Bool> spatialize;
         exposedField SFNode<class PerceptualParameters> perceptualParameters;
         exposedField SF<Bool> roomEffect;
         exposedField SF<int> shape;
         exposedField MF<float> size;
         exposedField SF<Vec3f> direction;
         exposedField SF<float> density;
         exposedField SF<int> diffuseSelect;
         exposedField SF<float> decorrStrength;
         FIELD SF<float> speedOfSound;
         FIELD SF<float> distance;
         FIELD SF<Bool> useAirabs;

         public:
              WideSound() {
                   intensity = 1;
                   location = Vec3f(0, 0, 0);
                   spatialize = TRUE;
                   roomEffect = FALSE;
                   shape = 0;
                   size += 0;
                   direction = Vec3f(0, 1, 0);
                   density = 0.5;
                   diffuseSelect = 1;
                   decorrStrength = 1;
                   speedOfSound = 340;
                   distance = 1000;
                   useAirabs = FALSE;
              }
};


class FootPrintSet : public MediaObject
{
		BIFS_DECLARE_NODE

		eventIn MFNodeType<SF3DNode> addChildren;
		eventIn MFNodeType<SF3DNode> removeChildren;
		exposedField MFNodeType<SF3DNode> children;
		void OnaddChildren();
		void OnremoveChildren();

         public:
              FootPrintSet() {
              }
};

class FootPrint : public MediaObject
{
         BIFS_DECLARE_NODE

		 exposedField SF<int> index;
		 exposedField SFNode<class IndexedLineSet2D> footprint;

         public:
              FootPrint() {
                   index = -1;
              }
};

class BuildingPart : public MediaObject
{
		 BIFS_DECLARE_NODE

		 exposedField SF<int> index;
		 exposedField SFNode<class IndexedLineSet2D> footprint;
		 exposedField SF<int> buildingIndex;
		 exposedField SF<float> height;
		 exposedField SF<float> altitude;
		 exposedField MFNodeType<SFGeometryNode> alternativeGeometry;
		 exposedField MFNode<class Roof> roofs;
		 exposedField MFNode<class Facade> facades;

         public:
              BuildingPart() {
                   index = -1;
				   buildingIndex = -1;
				   height = 0.0f;
				   altitude = 0.0f;
              }

};

class Roof : public MediaObject
{
         BIFS_DECLARE_NODE

		 exposedField SF<int> Type;
		 exposedField SF<float> Height;
		 exposedField MF<float>	SlopeAngle;
		 exposedField SF<float>	EaveProjection;
		 exposedField SF<int> EdgeSupportIndex;
		 exposedField SF<Url> RoofTextureURL;
		 exposedField SF<Bool> IsGenericTexture;
		 exposedField SF<float> TextureXScale;
		 exposedField SF<float> TextureYScale;
		 exposedField SF<float> TextureXPosition;
		 exposedField SF<float> TextureYPosition;
		 exposedField SF<float> TextureRotation;


         public:
              Roof() {
					Type = 0;
					Height = 0.0;
					SlopeAngle += 0.0;
					EaveProjection = 0.0;
					EdgeSupportIndex = -1;
					IsGenericTexture = true;
					TextureXScale = 1.0;
					TextureYScale = 1.0;
					TextureXPosition = 0.0;
					TextureYPosition = 0.0;
					TextureRotation = 0.0;
              }
};


class Facade : public MediaObject
{
         BIFS_DECLARE_NODE

		 exposedField SF<float> WidthRatio;
		 exposedField SF<float> XScale;
		 exposedField SF<float>	YScale;
		 exposedField SF<float>	XPosition;
		 exposedField SF<float> YPosition;
		 exposedField SF<float> XRepeatInterval;
		 exposedField SF<float> YRepeatInterval;
		 exposedField SF<Bool> Repeat;
		 exposedField SFGeometryNode FacadePrimitive;
		 exposedField SF<int> NbStories;
		 exposedField MF<int> NbFacadeCellsByStorey;
		 exposedField MF<float> StoryHeight;
		 exposedField MFNode<class Facade> FacadeCellsArray;

         public:
              Facade() {
					WidthRatio = 1.0;
					XScale = 1.0;
					YScale = 1.0;
					XPosition = 0.0;
					YPosition = 0.0;
					XRepeatInterval = 0.0;
					YRepeatInterval = 0.0;
					Repeat = false;
					NbStories = 0;
              }
};

class Shadow : public MediaObject
{
         BIFS_DECLARE_NODE

         eventIn MFNodeType<SF3DNode> addChildren;
         eventIn MFNodeType<SF3DNode> removeChildren;
         exposedField MFNodeType<SF3DNode> children;
         exposedField SF<Bool> enabled;
         exposedField MF<Bool> cast;
         exposedField MF<Bool> receive;
         exposedField SF<float> penumbra;
         void OnaddChildren();
         void OnremoveChildren();

         public:
              Shadow() {
                   enabled = TRUE;
                   cast += TRUE;
                   receive += TRUE;
                   penumbra = 0;
              }
};


#endif // _Nodes_h sentry.
/* End of automatically generated file */
