/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  OD Decoder / Encoder
FILE:       ODCoder.cpp
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Implementation of ODCoder - the class for encoding/decoding
			of OD streams.
-------------------------------------------------------------------------*/

#include "defs.h"
#include "ODCoder.h"

/*-------------------------------------------------------------
nLength is a number of bits, which needs to be converted into bytes,
and stored in the special format for sizeOfClass (using escape bits)
-------------------------------------------------------------*/
void ODCoder :: StoreDescLength (int nLength)
{
	nLength = (nLength + 7) / 8;
	for (int i = 21; i > 0; i -= 7)
		if (nLength >= (1 << i))
		{	EncodeInt (1, 1, "nextByte");
			EncodeInt ((nLength >> i) & 0x7f, 7, "sizeByte");
		}
	EncodeInt (0, 1, "nextByte");
	EncodeInt (nLength & 0x7f, 7, "sizeByte");
}

/*-------------------------------------------------------------
Parse a sizeOfClass field and covert it to bits.
The total length of the field itself is stored in rnLengthFieldWidth.
-------------------------------------------------------------*/
int ODCoder :: ReadDescLength (int &rnLengthFieldWidth)
{
	int length = 0, sizeOfClass = 0, bNextByte;
	do
	{	bNextByte = ParseInt (1, "bNextByte");
		sizeOfClass <<= 7;
		sizeOfClass |= ParseInt (7, "size");
		length += 8;
	}
	while (bNextByte);
	rnLengthFieldWidth = length;
	return sizeOfClass * 8;
}
