/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  OD Decoder / Encoder
FILE:       ODDec.cpp
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Implementation of ODDecoder - the class for decoding
			of OD streams.
-------------------------------------------------------------------------*/

#include "defs.h"
#include "ODDec.h"
#include "Executiv.h"
#include "ODP.h"
#include "Except.h"

#define pCoder this

/*-------------------------------------------------------------
A constructor of ODDecoder which is used when we need to decode from
a memory buffer rather than a file
-------------------------------------------------------------*/
ODDecoder :: ODDecoder (
	Executive *pExecutive,
	LPCBYTE pData,
	int nBifsLen,
	FILE *fList,
	LPCSTR szListTitle,
	int flags,
	int version
) :
	ODCoder (fList, szListTitle, flags, version),
	m_pExecutive (pExecutive),
	m_pSource (NULL)
{
	Reset (pData, nBifsLen);
}

/*-----------------------------------------------------------------
Parse an OD stream
-----------------------------------------------------------------*/
void ODDecoder :: ParseFrame (LPCBYTE pData, int nLength, DWORD dwTS)
{
	Reset (pData, nLength);
	dwTS;
	TRACEF ("AT %d", dwTS);
	TRACENEWLINE;
	try
	{
		ParseODFrame ();
		m_pExecutive -> RequestChannels ();
	}
	catch (BifsException)
	{
		Error (OD_PARSING_ERROR);
	}
}

/*-----------------------------------------------------------------
Parse one OD-update AU
-----------------------------------------------------------------*/
void ODDecoder :: ParseODFrame ()
{
	TRACEF ("{", 0);
	TRACENEWLINE;
	TRACEINDENT (1);
	
	while (m_nSize > 0)
	{	
		int nTag = ParseInt (8, "tag");
		BaseCommand *pDesc = (BaseCommand *) BaseCommand :: CreateMe (nTag);
		if (pDesc != NULL)
		{	pDesc -> EncDec (this, DECODE, NULL);
			TRACENEWLINE;
			if (pDesc)
				pDesc -> Commit (m_pSource);
			delete pDesc;
		}
	}

	TRACEINDENT (-1);
	TRACEF ("}", 0);
	TRACENEWLINE;
}

/*-------------------------------------------------------------
Report an error to the executive
-------------------------------------------------------------*/
void ODDecoder :: Error (int nError)
{
	m_pExecutive -> Error ((NON_FATAL_ERROR) nError);
	TRACEF ("OD parsing error", 0);
	TRACENEWLINE;
}

/*-------------------------------------------------------------
This virtual function is called if the parser exausted the unit at the
middle of decoding
-------------------------------------------------------------*/
void ODDecoder :: EndOfUnit ()
{
	throw BifsException (BIFX_END_OF_STREAM);
}
