/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player/ Mux
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  IM1 Player
FILE:       ODImp.cpp
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Implementation of classes derived from standard OD descriptors.
			These extension are used to implement OD in the IM1 player.
-------------------------------------------------------------------------*/

#include "defs.h"
#include "ODImp.h"
//#include "InLine.h"
#include "Inline.h" // SOLARIS PORTING - typo
#include "Stream.h"
#include "IPMP.h"
//#include "Im1Defs.h"
#include "Im1defs.h" // SOLARIS PORTING - typo
#include "AL.h"
#include "Executiv.h"
#include "ODDec.h"
#include "Except.h"


/*-------------------------------------------------------------
Execute an ObjectDescriptorUpdate command
-------------------------------------------------------------*/
void ObjectDescriptorUpdateImp :: Commit (InlineImp *pSource)
{
	for (int i = 0; i < OD.GetLength (); i++)
	{	ObjectDescriptorImp *pOD = (ObjectDescriptorImp*) OD [i];
		OD.SetAt (i, NULL);		// this is necessary to prevent the 
								// deletion of the object
		pSource -> UpdateOD (pOD);
	}
}

/*-------------------------------------------------------------
Execute an ObjectDescriptorRemove command
-------------------------------------------------------------*/
void ObjectDescriptorRemoveImp :: Commit (InlineImp *pSource)
{
	for (int i = 0; i < objectDescriptorId.GetLength (); i++)
		pSource -> RemoveOD (objectDescriptorId [i]);
}

/*-------------------------------------------------------------
Execute an ES_DescriptorUpdate command
-------------------------------------------------------------*/
void ES_DescriptorUpdateImp :: Commit (InlineImp *pSource)
{
	for (int i = 0; i < esDescr.GetLength (); i++)
	{	ES_DescriptorImp *pESD = (ES_DescriptorImp *) esDescr [i];
		esDescr.SetAt (i, NULL);	// this is necessary to prevent the
								// deletion of the object
		pSource -> UpdateESD (objectDescriptorId, pESD);
	}
}	

/*-------------------------------------------------------------
Execute an ES_DescriptorRemove command
-------------------------------------------------------------*/
void ES_DescriptorRemoveImp :: Commit (InlineImp *pSource)
{
	for (int i = 0; i < ES_ID.GetLength (); i++)
		pSource -> RemoveESD (objectDescriptorId, ES_ID [i]);
}

/*-------------------------------------------------------------
Execute an IPMP_DescriptorUpdate command
-------------------------------------------------------------*/
void IPMP_DescriptorUpdateImp :: Commit (InlineImp *pSource)
{
	for (int i = 0; i < ipmpDescr.GetLength (); i++)
	{	pSource -> UpdateIPMPD ((IPMP_DescriptorImp *) ipmpDescr [i]);
		ipmpDescr.SetAt (i, NULL);
	}
}

/*-------------------------------------------------------------
Execute an IPMP_DescriptorRemove command
-------------------------------------------------------------*/
void IPMP_DescriptorRemoveImp :: Commit (InlineImp *pSource)
{
	for (int i = 0; i < IPMP_DescriptorID.GetLength (); i++)
		pSource -> RemoveIPMPD (IPMP_DescriptorID [i]);
}

/*-------------------------------------------------------------
Notify an IPMP system on an IPMP descriptor
-------------------------------------------------------------*/
int IPMP_DescriptorImp :: Notify (IPMPManager *pIPMP)
{	
	const BYTE *pData;
	int nDataSize;
	if (IPMPS_Type == 0)
	{	pData = URLString;
		nDataSize = URLString.GetLength ();
	}
	else
	{	pData = IPMP_data;
		nDataSize = IPMP_data.GetLength ();
	}
	return pIPMP -> SetDescriptor (
		IPMP_DescriptorID,
		IPMPS_Type,
		pData,
		nDataSize
	);
}

	
/*-------------------------------------------------------------
Decode an InitialObjectDescriptor. This is necessary because
ObjectDescriptorImp is used for both InitialObjectDescriptor and
ObjectDescriptor
-------------------------------------------------------------*/
void ObjectDescriptorImp :: DecInitialOD (ODCoder *pODCoder)
{
	/*-------------------------------------------------------------
	Make sure this is indeed an InitialObjectDescriptor object
	-------------------------------------------------------------*/
	try
	{	if (!InitialObjectDescriptor :: IsMyTag (pODCoder -> ParseInt (8, "tag")))
		{	pODCoder -> Error (INITIAL_OBJECT_DESCRIPTOR_NOT_FOUND);
			return;
		}
		reserved = 0xf;
		EncDec (pODCoder, DECODE);
	}
	catch (BifsException)
	{
		pODCoder -> Error (ERROR_IN_PARSING_INITIAL_OBJECT_DESCRIPTOR);
	}
}

void ObjectDescriptorExecuteImp :: Commit (InlineImp *pSource)
{
	for (int i = 0; i < objectDescriptorId.GetLength (); i++)
		pSource -> ExecuteOD (objectDescriptorId [i]);
}
