/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

#if !defined(_ODImp_h)	// Sentry, use file only if it's not already included.
#define _ODImp_h

/*-------------------------------------------------------------------------
Project Im1Player/ Mux
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  OD Decoder/ Encoder
FILE:       ODImp.h
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Declarations of classes derived from standard OD descriptors.
			These extension are used to implement OD in the IM1 player.
-------------------------------------------------------------------------*/

//#include "odp3.h"
#include "ODP3.h" // SOLARIS PORTING - typo
#include "odtags.h"
#include "Caps.h"
#include "Im1defs.h"
#include "Decoder.h"
#include "IPMP_Messages.h"

class ALManager;
class InlineImp;
class StreamConsumer;
class DataChannel;
class IPMPManager;
class MediaStreamImp;
class Executive;
class Decoder;
class Filter;
class ES_DescriptorImp;
class ClockReference;
class MediaControlImp;

#ifdef MPGJ
//*** YT START (12)
// I just wanted to use this class...
class DecoderImp;
//*** YT END (12)
#endif MPGJ

class ObjectDescriptorUpdateImp : public ObjectDescriptorUpdate
{
private:
	virtual void Commit (InlineImp *pSource);
};

class ObjectDescriptorRemoveImp : public ObjectDescriptorRemove
{
private:
	virtual void Commit (InlineImp *pSource);
};

class ES_DescriptorUpdateImp : public ES_DescriptorUpdate
{
private:
	virtual void Commit (InlineImp *pSource);
};

class ES_DescriptorRemoveImp : public ES_DescriptorRemove
{
private:
	virtual void Commit (InlineImp *pSource);
};

class IPMP_DescriptorUpdateImp : public IPMP_DescriptorUpdate
{
private:
	virtual void Commit (InlineImp *pSource);
};

class IPMP_DescriptorRemoveImp : public IPMP_DescriptorRemove
{
private:
	virtual void Commit (InlineImp *pSource);
};

class IPMP_DescriptorImp : public IPMP_Descriptor
{
public:
	int Notify (IPMPManager *pIPMP);
	~IPMP_DescriptorImp ()
	{
	}
};

class ObjectDescriptorExecuteImp : public ObjectDescriptorExecute
{
private:
	virtual void Commit (InlineImp *pSource);
};

class ObjectDescriptorImp : public InitialObjectDescriptor
{
public: static int GetCode () {return CODE_ObjectDescriptor;}

public:
	ObjectDescriptorImp () :
		m_pAL (NULL), m_pInline (NULL), m_pDecoder (NULL), m_pChannel (NULL), m_nUse (0),
		m_nInstance (0), m_nOpen (0), m_pRemoteAL (NULL), m_pRemoteOD (NULL),
		m_pControl (NULL), m_bPreRolled (false)
	{}
	~ObjectDescriptorImp ();
	ALManager *GetAL () const {return m_pAL;}
	void SetAL (ALManager *pAL) {m_pAL = pAL;}
	void SetRemoteAL (ALManager *pAL) {m_pRemoteAL = pAL;}
	void SetInline (InlineImp *pInline) {m_pInline = pInline;}
	InlineImp *GetInline () {return m_pInline;}
	void DecInitialOD (ODCoder *pODCoder);
	MediaStreamImp *SetStream (StreamConsumer *pNode);
	void Start ();
	void Stop ();
	void Detach ();
	Decoder *GetDecoder () {return m_pDecoder;}
	void SetDecoder (Decoder *pDecoder) {m_pDecoder = pDecoder;}
	void SetChannel (DataChannel *pChannel) {m_pChannel = pChannel;}
	void SetClocks ();
	void DataSend (LPBYTE pData, int nLength);
	int SetUseCount (int n) {return m_nUse += n;}
	void OnConnect (const BYTE *pInitOD, int nInitODLen);
	void AttachNode (StreamConsumer *pNode);
	bool UpdateESD (ES_DescriptorImp *pESD);
	void RemoveESD (int nESid);
	ES_DescriptorImp *FindESD (int nESid);
	bool IsPlaying () const {return m_nOpen > 0;}
	void Play (double dSpeed);
	void SetSpeed (double dSpeed);
	void PreRoll ();
	void Pause ();
	void Resume ();
	void Mute ();
	bool IsOpen();
	MediaControlImp *SetMediaControl (MediaControlImp *pControl)
	{
		MediaControlImp *p = m_pControl;
		m_pControl = pControl;
		return p;
	}
	/* gets max duration of associated input stream if known, -1 otherwise */
	double GetDuration();
	/* refreshes the segment description of the node */
	void GetSegmentList(StreamConsumer *pNode);
	
	/*remove all clock associations*/
	void RemoveClockLinks();
	/*update clock associations*/
	void UpdateClockLinks();

	ObjectDescriptorImp *GetRemoteOD() {return m_pRemoteOD;}

private:
	ALManager *m_pAL, *m_pRemoteAL;
	InlineImp *m_pInline;
	Decoder *m_pDecoder;
	DataChannel *m_pChannel;
	int m_nUse, m_nInstance, m_nOpen;
	ObjectDescriptorImp *m_pRemoteOD;
	MediaControlImp *m_pControl;
	bool m_bPreRolled;
};

class ES_DescriptorImp : public ES_Descriptor
{
public:
	ES_DescriptorImp ();
	MediaStreamImp *SetDecoder (
		ObjectDescriptorImp *pOD,
		StreamConsumer *pNode
	);
	void Start (bool wasPreRolled, bool setPreRolled);
	void Stop ();
	void Detach ();
	void Play (double dSpeed);
	void SetSpeed (double dSpeed);
	void Pause ();
	void Resume();
	void Mute ();
	Decoder *GetDecoder () const {return m_pDecoder;}
	ALManager *GetAL () const {return m_pAL;}
	MediaStreamImp *AddESD (ObjectDescriptorImp *pOD, StreamConsumer *pNode);
	bool IsMyStream (MediaStreamImp *pStream) const
	{
		return pStream == m_pInputStream || pStream == m_pOutputStream;
	}
	void SetClock (ALManager *pAL);
	ClockReference *GetClock () const {return m_pClock;}

private:
	Filter *LoadFilter (
		int nStreamType,
		int objectTypeIndication
	);
	DataChannel *OpenStream (
		int nBufferSize,
		int nExpectedAULength,
		bool bBackChannel = false,
		float fThreshold = 0
	);
	bool InvokeIPMP ();
	bool SetIPMP (ES_DescriptorImp *pES);
	bool HandleIPMPDescriptors (
		IPMP_DescriptorPointer * const * descPointers,
		int nDescPointersLength,
		InlineImp *pInline
	);
	void AttachIPMP (IPMPManager *pIPMP);
	void RemoveIPMPSystems (
		IPMPManager *pIPMP,
		IPMP_DescriptorPointer * const * descPointers,
		int nDescPointersLength,
		InlineImp *pInline
	);
	Decoder *CreateOCIDecoder (int version);
	#ifdef MPGJ
	//*** YT START (5)
	// utility function making MPEGJ decoder
	Decoder *CreateMPEGJDecoder();
	//*** YT END (5)
	#endif MPGJ

	#ifdef MPGJ
	//*** YT START (102)
public:
	virtual void AttachDecoder(DecoderImp* pDecoder);
	virtual DecoderImp* DetachDecoder();
	//*** YT END (102)
	#endif

	virtual bool GoNoGo (bool go);
	virtual bool SetFilter (Filter *filter, int controlPoint);
	virtual bool ReleaseFilter (Filter *filter);
	
private:
	ALManager *m_pAL;
	Executive *m_pExecutive;
	ObjectDescriptorImp *m_pOD;
	#ifndef MPGJ
	//*** YT START (103)
	// made it public see below

	MediaStreamImp *m_pOutputStream, *m_pInputStream;

//*** YT END (103)
	#endif !MPGJ
	Decoder *m_pDecoder;
	DataChannel *m_pChannel;
	bool m_bAttached, m_bGo;
	ZArray<IPMPManager*> m_ipmps;
	ClockReference *m_pClock;
	NON_FATAL_ERROR m_error;
	int m_nStart, m_nDecoderStart, m_nInstantiated;
	DecoderParams m_decoderParams;
	ZArray<Filter*> m_filters;
public:
	#ifdef MPGJ
	//*** YT START (103)
	// make these public
	// need to be accessed from MPDecoder class
	MediaStreamImp *m_pOutputStream, *m_pInputStream;
	//*** YT END (103)
	#endif MPGJ
};			

#endif                                      // _ODImp_h sentry.
