/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player/ Mux
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  OD Decoder/ Encoder
FILE:       odp.cpp
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Implementation of parsing/encoding of descriptors of OD stream
-------------------------------------------------------------------------*/

#include "defs.h"
//#include "odp.h"
#include "ODP.h" // SOLARIS PORTING - typo
#include "odtags.h"
#include "IPMP_Messages.h"
#include "CrSdl.h"

OD_IMPLEMENT_ABSTRACT_DESCRIPTOR_START (BaseCommand)
	OD_IMPLEMENT_EXTENSION (ObjectDescriptorUpdate)
	OD_IMPLEMENT_EXTENSION (ObjectDescriptorRemove)
	OD_IMPLEMENT_EXTENSION (ES_DescriptorUpdate)
	OD_IMPLEMENT_EXTENSION (ES_DescriptorRemove)
	OD_IMPLEMENT_EXTENSION (IPMP_DescriptorUpdate)
	OD_IMPLEMENT_EXTENSION (IPMP_DescriptorRemove)
OD_IMPLEMENT_ABSTRACT_DESCRIPTOR_END (BaseCommand)

OD_IMPLEMENT_DESCRIPTOR_START (ObjectDescriptorUpdate)
	OD_IMPLEMENT_FIELD (ObjectDescriptorUpdate, OD)
OD_IMPLEMENT_DESCRIPTOR_END (ObjectDescriptorUpdate)

OD_IMPLEMENT_DESCRIPTOR_START (ObjectDescriptorRemove)
	OD_IMPLEMENT_FIELD (ObjectDescriptorRemove, objectDescriptorId)
OD_IMPLEMENT_DESCRIPTOR_END (ObjectDescriptorRemove)

OD_IMPLEMENT_DESCRIPTOR_START (ES_DescriptorUpdate)
	OD_IMPLEMENT_FIELD (ES_DescriptorUpdate, objectDescriptorId)
	OD_IMPLEMENT_FIELD (ES_DescriptorUpdate, esDescr)
OD_IMPLEMENT_DESCRIPTOR_END (ES_DescriptorUpdate)

OD_IMPLEMENT_DESCRIPTOR_START (ES_DescriptorRemove)
	OD_IMPLEMENT_FIELD (ES_DescriptorRemove, objectDescriptorId)
	OD_IMPLEMENT_FIELD (ES_DescriptorRemove, align)
	OD_IMPLEMENT_FIELD (ES_DescriptorRemove, ES_ID)
OD_IMPLEMENT_DESCRIPTOR_END (ES_DescriptorRemove)

OD_IMPLEMENT_DESCRIPTOR_START (IPMP_DescriptorUpdate)
	OD_IMPLEMENT_FIELD (IPMP_DescriptorUpdate, ipmpDescr)
OD_IMPLEMENT_DESCRIPTOR_END (IPMP_DescriptorUpdate)

OD_IMPLEMENT_DESCRIPTOR_START (IPMP_DescriptorRemove)
	OD_IMPLEMENT_FIELD (IPMP_DescriptorRemove, IPMP_DescriptorID)
OD_IMPLEMENT_DESCRIPTOR_END (IPMP_DescriptorRemove)

OD_IMPLEMENT_DESCRIPTOR_START (ObjectDescriptor)
	OD_IMPLEMENT_FIELD (ObjectDescriptor, ObjectDescriptorID)
	OD_IMPLEMENT_FIELD (ObjectDescriptor, URL_Flag)
	OD_IMPLEMENT_FIELD (ObjectDescriptor, reserved)
	OD_IMPLEMENT_FIELD (ObjectDescriptor, URLstring)
	OD_IMPLEMENT_FIELD (ObjectDescriptor, esDescr)
	OD_IMPLEMENT_FIELD (ObjectDescriptor, ociDescr)
	OD_IMPLEMENT_FIELD (ObjectDescriptor, ipmpDescrPtr)
	OD_IMPLEMENT_FIELD (ObjectDescriptor, ipmpDescr)
	OD_IMPLEMENT_FIELD (ObjectDescriptor, extDescr)
	OD_IMPLEMENT_FIELD (ObjectDescriptor, muxScript)
OD_IMPLEMENT_DESCRIPTOR_END (ObjectDescriptor)

OD_IMPLEMENT_DESCRIPTOR_START (InitialObjectDescriptor)
	OD_IMPLEMENT_FIELD (InitialObjectDescriptor, ObjectDescriptorID)
	OD_IMPLEMENT_FIELD (InitialObjectDescriptor, URL_Flag)
	OD_IMPLEMENT_FIELD (InitialObjectDescriptor, includeInlineProfilesFlag)
	OD_IMPLEMENT_FIELD (InitialObjectDescriptor, reserved)
	OD_IMPLEMENT_FIELD (InitialObjectDescriptor, URLstring)
	OD_IMPLEMENT_FIELD (InitialObjectDescriptor, ODProfileLevelIndication)
	OD_IMPLEMENT_FIELD (InitialObjectDescriptor, sceneProfileLevelIndication)
	OD_IMPLEMENT_FIELD (InitialObjectDescriptor, audioProfileLevelIndication)
	OD_IMPLEMENT_FIELD (InitialObjectDescriptor, visualProfileLevelIndication)
	OD_IMPLEMENT_FIELD (InitialObjectDescriptor, graphicsProfileLevelIndication)
	OD_IMPLEMENT_FIELD (InitialObjectDescriptor, esDescr)
	OD_IMPLEMENT_FIELD (InitialObjectDescriptor, ociDescr)
	OD_IMPLEMENT_FIELD (InitialObjectDescriptor, ipmpDescrPtr)
	OD_IMPLEMENT_FIELD (InitialObjectDescriptor, ipmpDescr)
	OD_IMPLEMENT_FIELD (InitialObjectDescriptor, toolListDescr)	//added by jimmy, IPMPX stuff
	OD_IMPLEMENT_FIELD (InitialObjectDescriptor, extDescr)
OD_IMPLEMENT_DESCRIPTOR_END (InitialObjectDescriptor)

OD_IMPLEMENT_DESCRIPTOR_START (ES_Descriptor)
	OD_IMPLEMENT_FIELD (ES_Descriptor, ES_ID)
	OD_IMPLEMENT_FIELD (ES_Descriptor, streamDependenceFlag)
	OD_IMPLEMENT_FIELD (ES_Descriptor, URL_Flag)
	OD_IMPLEMENT_FIELD (ES_Descriptor, OCRstreamFlag)
	OD_IMPLEMENT_FIELD (ES_Descriptor, streamPriority)
	OD_IMPLEMENT_FIELD (ES_Descriptor, dependsOn_ES_ID)
	OD_IMPLEMENT_FIELD (ES_Descriptor, URLstring)
	OD_IMPLEMENT_FIELD (ES_Descriptor, OCR_ES_Id)
	OD_IMPLEMENT_FIELD (ES_Descriptor, decConfigDescr)
	OD_IMPLEMENT_FIELD (ES_Descriptor, slConfigDescr)
	OD_IMPLEMENT_FIELD (ES_Descriptor, ipiPtr)
	OD_IMPLEMENT_FIELD (ES_Descriptor, ipIDS)
	OD_IMPLEMENT_FIELD (ES_Descriptor, ipmpDescrPtr)
	OD_IMPLEMENT_FIELD (ES_Descriptor, langDescr)
	OD_IMPLEMENT_FIELD (ES_Descriptor, qosDescr)
	OD_IMPLEMENT_FIELD (ES_Descriptor, regDescr)
	OD_IMPLEMENT_FIELD (ES_Descriptor, extDescr)
	OD_IMPLEMENT_FIELD (ES_Descriptor, muxInfo)
	OD_IMPLEMENT_FIELD (ES_Descriptor, muxCodeTable)
OD_IMPLEMENT_DESCRIPTOR_END (ES_Descriptor)

OD_IMPLEMENT_DESCRIPTOR_START (DecoderConfigDescriptor)
	OD_IMPLEMENT_FIELD (DecoderConfigDescriptor, objectTypeIndication)
	OD_IMPLEMENT_FIELD (DecoderConfigDescriptor, streamType)
	OD_IMPLEMENT_FIELD (DecoderConfigDescriptor, upStream)
	OD_IMPLEMENT_FIELD (DecoderConfigDescriptor, reserved)
	OD_IMPLEMENT_FIELD (DecoderConfigDescriptor, bufferSizeDB)
	OD_IMPLEMENT_FIELD (DecoderConfigDescriptor, maxBitrate)
	OD_IMPLEMENT_FIELD (DecoderConfigDescriptor, avgBitrate)
	OD_IMPLEMENT_FIELD (DecoderConfigDescriptor, decSpecificInfo)
	// added by liao xuan 
	OD_IMPLEMENT_FIELD (DecoderConfigDescriptor, profileLevelIndicationIndexDescr)
	// added by liao xuan
OD_IMPLEMENT_DESCRIPTOR_END (DecoderConfigDescriptor)

OD_IMPLEMENT_DESCRIPTOR_START (profileLevelIndicationIndexDescriptor)
	OD_IMPLEMENT_FIELD (profileLevelIndicationIndexDescriptor, profileLevelIndicationIndex)
OD_IMPLEMENT_DESCRIPTOR_END (profileLevelIndicationIndexDescriptor)

OD_IMPLEMENT_ABSTRACT_DESCRIPTOR_START (DecoderSpecificInfo)
	OD_IMPLEMENT_EXTENSION (DecoderSpecificInfoString)
	OD_IMPLEMENT_EXTENSION (BIFSConfig)
	OD_IMPLEMENT_EXTENSION (BIFSv2Config)    //AC 19991117: added the BIFSv2Config
	OD_IMPLEMENT_EXTENSION (GeneralAudioConfig)
	OD_IMPLEMENT_EXTENSION (AudioSpecificConfig)
	OD_IMPLEMENT_EXTENSION (UIConfig)						// CC 010122
	OD_IMPLEMENT_EXTENSION (OCIDecoderConfiguration)
	//jimmy, IPMPX additions
	OD_IMPLEMENT_EXTENSION (IPMPToolES_DecoderConfig)
	OD_IMPLEMENT_EXTENSION (IPMPDecoderConfiguration)
	//end of IPMPX stuff
    OD_IMPLEMENT_EXTENSION (BIFSConfigEx)					// version 5
	OD_IMPLEMENT_EXTENSION (AFXConfig)
	OD_IMPLEMENT_EXTENSION (JPEG_DecoderConfig)				// mar 20061120
	OD_IMPLEMENT_EXTENSION (JPEG2000_DecoderConfig)			// mar 20061120
OD_IMPLEMENT_ABSTRACT_DESCRIPTOR_END (DecoderSpecificInfo)

OD_IMPLEMENT_DESCRIPTOR_START (DecoderSpecificInfoString)
	OD_IMPLEMENT_FIELD (DecoderSpecificInfoString, info)
OD_IMPLEMENT_DESCRIPTOR_END (DecoderSpecificInfoString)

OD_IMPLEMENT_DESCRIPTOR_START (OCIDecoderConfiguration)
	OD_IMPLEMENT_FIELD (OCIDecoderConfiguration, versionLabel)
OD_IMPLEMENT_DESCRIPTOR_END (OCIDecoderConfiguration)

OD_IMPLEMENT_ABSTRACT_DESCRIPTOR_START (IP_IdentificationDataSet)
	OD_IMPLEMENT_EXTENSION (ContentIdentificationDescriptor)
	OD_IMPLEMENT_EXTENSION (SupplementaryContentIdentificationDescriptor)
OD_IMPLEMENT_ABSTRACT_DESCRIPTOR_END (IP_IdentificationDataSet)

OD_IMPLEMENT_DESCRIPTOR_START (ContentIdentificationDescriptor)
	OD_IMPLEMENT_FIELD (ContentIdentificationDescriptor, compatibility)
	OD_IMPLEMENT_FIELD (ContentIdentificationDescriptor, contentTypeFlag)
	OD_IMPLEMENT_FIELD (ContentIdentificationDescriptor, contentIdentifierFlag)
	OD_IMPLEMENT_FIELD (ContentIdentificationDescriptor, protectedContent)
	OD_IMPLEMENT_FIELD (ContentIdentificationDescriptor, reserved)
	OD_IMPLEMENT_FIELD (ContentIdentificationDescriptor, contentType)
	OD_IMPLEMENT_FIELD (ContentIdentificationDescriptor, contentIdentifierType)
	OD_IMPLEMENT_FIELD (ContentIdentificationDescriptor, contentIdentifier)
OD_IMPLEMENT_DESCRIPTOR_END (ContentIdentificationDescriptor)

OD_IMPLEMENT_DESCRIPTOR_START (SupplementaryContentIdentificationDescriptor)
	OD_IMPLEMENT_FIELD (SupplementaryContentIdentificationDescriptor, languageCode)
	OD_IMPLEMENT_FIELD (SupplementaryContentIdentificationDescriptor, supplContentIdentifierTitle)
	OD_IMPLEMENT_FIELD (SupplementaryContentIdentificationDescriptor, supplContentIdentifierValue)
OD_IMPLEMENT_DESCRIPTOR_END (SupplementaryContentIdentificationDescriptor)

OD_IMPLEMENT_DESCRIPTOR_START (IPI_DescrPointer)
	OD_IMPLEMENT_FIELD (IPI_DescrPointer, IPI_ES_Id)
OD_IMPLEMENT_DESCRIPTOR_END (IPI_DescrPointer)

OD_IMPLEMENT_DESCRIPTOR_START (IPMP_DescriptorPointer)
	OD_IMPLEMENT_FIELD (IPMP_DescriptorPointer, IPMP_DescriptorID)
	OD_IMPLEMENT_FIELD (IPMP_DescriptorPointer, IPMP_DescriptorIDEx)
	OD_IMPLEMENT_FIELD (IPMP_DescriptorPointer, IPMP_ES_ID)
OD_IMPLEMENT_DESCRIPTOR_END (IPMP_DescriptorPointer)

OD_IMPLEMENT_ABSTRACT_DESCRIPTOR_START (IPMP_Data_BaseClass)
	OD_IMPLEMENT_EXTENSION (IPMP_OpaqueData)
	OD_IMPLEMENT_EXTENSION (IPMP_AudioWatermarkingInit)
	OD_IMPLEMENT_EXTENSION (IPMP_VideoWatermarkingInit)
	OD_IMPLEMENT_EXTENSION (IPMP_SelectiveDecryptionInit)
	OD_IMPLEMENT_EXTENSION (IPMP_KeyData)
	OD_IMPLEMENT_EXTENSION (IPMP_SendAudioWatermark)
	OD_IMPLEMENT_EXTENSION (IPMP_SendVideoWatermark)
	OD_IMPLEMENT_EXTENSION (IPMP_RightsData)
	OD_IMPLEMENT_EXTENSION (IPMP_Secure_Container)
	OD_IMPLEMENT_EXTENSION (IPMP_AddToolNotificationListener)
	OD_IMPLEMENT_EXTENSION (IPMP_RemoveToolNotificationListener)
	OD_IMPLEMENT_EXTENSION (IPMP_InitAuthentication)
	OD_IMPLEMENT_EXTENSION (IPMP_MutualAuthentication)
	OD_IMPLEMENT_EXTENSION (IPMP_UserQuery)
	OD_IMPLEMENT_EXTENSION (IPMP_UserQueryResponse)
	OD_IMPLEMENT_EXTENSION (IPMP_ToolParamCapabilitiesQuery)
	OD_IMPLEMENT_EXTENSION (IPMP_ToolParamCapabilitiesResponse)
	OD_IMPLEMENT_EXTENSION (IPMP_GetTools)
	OD_IMPLEMENT_EXTENSION (IPMP_GetToolsResponse)
	OD_IMPLEMENT_EXTENSION (IPMP_GetToolContext)
	OD_IMPLEMENT_EXTENSION (IPMP_GetToolContextResponse)
	OD_IMPLEMENT_EXTENSION (IPMP_ConnectTool)
	OD_IMPLEMENT_EXTENSION (IPMP_DisconnectTool)
	OD_IMPLEMENT_EXTENSION (IPMP_NotifyToolEvent)
	OD_IMPLEMENT_EXTENSION (IPMP_CanProcess)
	OD_IMPLEMENT_EXTENSION (User_Initialize)
	OD_IMPLEMENT_EXTENSION (User_Initialized)
OD_IMPLEMENT_ABSTRACT_DESCRIPTOR_END (IPMP_Data_BaseClass)

OD_IMPLEMENT_DESCRIPTOR_START (IPMP_Descriptor)
	OD_IMPLEMENT_FIELD (IPMP_Descriptor, IPMP_DescriptorID)
	OD_IMPLEMENT_FIELD (IPMP_Descriptor, IPMPS_Type)
	OD_IMPLEMENT_FIELD (IPMP_Descriptor, IPMP_DescriptorIDEx)
	OD_IMPLEMENT_FIELD (IPMP_Descriptor, IPMP_ToolID) 
	OD_IMPLEMENT_FIELD (IPMP_Descriptor, controlPointCode)
	OD_IMPLEMENT_FIELD (IPMP_Descriptor, sequenceCode)
	OD_IMPLEMENT_FIELD (IPMP_Descriptor, IPMPX_Data)
	OD_IMPLEMENT_FIELD (IPMP_Descriptor, URLString)
	OD_IMPLEMENT_FIELD (IPMP_Descriptor, IPMP_data)
OD_IMPLEMENT_DESCRIPTOR_END (IPMP_Descriptor)

OD_IMPLEMENT_DESCRIPTOR_START (QoS_Descriptor)
	OD_IMPLEMENT_FIELD (QoS_Descriptor, predefined)
	OD_IMPLEMENT_FIELD (QoS_Descriptor, qualifiers)
OD_IMPLEMENT_DESCRIPTOR_END (QoS_Descriptor)

OD_IMPLEMENT_ABSTRACT_DESCRIPTOR_START (QoS_Qualifier)
	OD_IMPLEMENT_EXTENSION (QoS_Qualifier_MAX_DELAY)
	OD_IMPLEMENT_EXTENSION (QoS_Qualifier_PREF_MAX_DELAY)
	OD_IMPLEMENT_EXTENSION (QoS_Qualifier_LOSS_PROB)
	OD_IMPLEMENT_EXTENSION (QoS_Qualifier_MAX_GAP_LOSS)
	OD_IMPLEMENT_EXTENSION (QoS_Qualifier_MAX_AU_SIZE)
	OD_IMPLEMENT_EXTENSION (QoS_Qualifier_AVG_AU_SIZE)
	OD_IMPLEMENT_EXTENSION (QoS_Qualifier_MAX_AU_RATE)
	OD_IMPLEMENT_EXTENSION (QoS_Qualifier_REBUFFERING_RATIO)
OD_IMPLEMENT_ABSTRACT_DESCRIPTOR_END (QoS_Qualifier)

OD_IMPLEMENT_DESCRIPTOR_START (QoS_Qualifier_MAX_DELAY)
	OD_IMPLEMENT_FIELD (QoS_Qualifier_MAX_DELAY, MAX_DELAY)
OD_IMPLEMENT_DESCRIPTOR_END (QoS_Qualifier_MAX_DELAY)

OD_IMPLEMENT_DESCRIPTOR_START (QoS_Qualifier_PREF_MAX_DELAY)
	OD_IMPLEMENT_FIELD (QoS_Qualifier_PREF_MAX_DELAY, PREF_MAX_DELAY)
OD_IMPLEMENT_DESCRIPTOR_END (QoS_Qualifier_PREF_MAX_DELAY)

OD_IMPLEMENT_DESCRIPTOR_START (QoS_Qualifier_LOSS_PROB)
	OD_IMPLEMENT_FIELD (QoS_Qualifier_LOSS_PROB, LOSS_PROB)
OD_IMPLEMENT_DESCRIPTOR_END (QoS_Qualifier_LOSS_PROB)

OD_IMPLEMENT_DESCRIPTOR_START (QoS_Qualifier_MAX_GAP_LOSS)
	OD_IMPLEMENT_FIELD (QoS_Qualifier_MAX_GAP_LOSS, MAX_GAP_LOSS)
OD_IMPLEMENT_DESCRIPTOR_END (QoS_Qualifier_MAX_GAP_LOSS)

OD_IMPLEMENT_DESCRIPTOR_START (QoS_Qualifier_MAX_AU_SIZE)
	OD_IMPLEMENT_FIELD (QoS_Qualifier_MAX_AU_SIZE, MAX_AU_SIZE)
OD_IMPLEMENT_DESCRIPTOR_END (QoS_Qualifier_MAX_AU_SIZE)

OD_IMPLEMENT_DESCRIPTOR_START (QoS_Qualifier_AVG_AU_SIZE)
	OD_IMPLEMENT_FIELD (QoS_Qualifier_AVG_AU_SIZE, AVG_AU_SIZE)
OD_IMPLEMENT_DESCRIPTOR_END (QoS_Qualifier_AVG_AU_SIZE)

OD_IMPLEMENT_DESCRIPTOR_START (QoS_Qualifier_MAX_AU_RATE)
	OD_IMPLEMENT_FIELD (QoS_Qualifier_MAX_AU_RATE, MAX_AU_RATE)
OD_IMPLEMENT_DESCRIPTOR_END (QoS_Qualifier_MAX_AU_RATE)

OD_IMPLEMENT_ABSTRACT_DESCRIPTOR_START (ExtensionDescriptor)
	0
OD_IMPLEMENT_ABSTRACT_DESCRIPTOR_END (ExtensionDescriptor)

OD_IMPLEMENT_DESCRIPTOR_START (RegistrationDescriptor)
	OD_IMPLEMENT_FIELD (RegistrationDescriptor, formatIdentifier)
	OD_IMPLEMENT_FIELD (RegistrationDescriptor, additionalIdentificationInfo)
OD_IMPLEMENT_DESCRIPTOR_END (RegistrationDescriptor)

OD_IMPLEMENT_ABSTRACT_DESCRIPTOR_START (OCI_Descriptor)
	OD_IMPLEMENT_EXTENSION (ContentClassificationDescriptor)
	OD_IMPLEMENT_EXTENSION (KeyWordDescriptor)
	OD_IMPLEMENT_EXTENSION (RatingDescriptor)
	OD_IMPLEMENT_EXTENSION (LanguageDescriptor)
	OD_IMPLEMENT_EXTENSION (ShortTextualDescriptor)
	OD_IMPLEMENT_EXTENSION (ExpandedTextualDescriptor)
	OD_IMPLEMENT_EXTENSION (ContentCreatorNameDescriptor)
	OD_IMPLEMENT_EXTENSION (ContentCreationDateDescriptor)
	OD_IMPLEMENT_EXTENSION (OCICreatorNameDescriptor)
	OD_IMPLEMENT_EXTENSION (OCICreationDateDescriptor)
	OD_IMPLEMENT_EXTENSION (SmpteCameraPositionDescriptor)	// NHK 990702
	OD_IMPLEMENT_EXTENSION (SegmentDescriptor)
	OD_IMPLEMENT_EXTENSION (MediaTimeDescriptor)
OD_IMPLEMENT_ABSTRACT_DESCRIPTOR_END (OCI_Descriptor)

OD_IMPLEMENT_DESCRIPTOR_START (ContentClassificationDescriptor)
	OD_IMPLEMENT_FIELD (ContentClassificationDescriptor, classificationEntity)
	OD_IMPLEMENT_FIELD (ContentClassificationDescriptor, classificationTable)
	OD_IMPLEMENT_FIELD (ContentClassificationDescriptor, contentClassificationData)
OD_IMPLEMENT_DESCRIPTOR_END (ContentClassificationDescriptor)

OD_IMPLEMENT_DESCRIPTOR_START (KeyWordDescriptor)
	OD_IMPLEMENT_FIELD (KeyWordDescriptor, languageCode)
	OD_IMPLEMENT_FIELD (KeyWordDescriptor, isUTF8_string)
	OD_IMPLEMENT_FIELD (KeyWordDescriptor, keyWordCount)
	OD_IMPLEMENT_FIELD (KeyWordDescriptor, keyWord)
OD_IMPLEMENT_DESCRIPTOR_END (KeyWordDescriptor)  

OD_IMPLEMENT_DESCRIPTOR_START (RatingDescriptor)
	OD_IMPLEMENT_FIELD (RatingDescriptor, ratingEntity)
	OD_IMPLEMENT_FIELD (RatingDescriptor, ratingCriteria)
	OD_IMPLEMENT_FIELD (RatingDescriptor, ratingInfo)
OD_IMPLEMENT_DESCRIPTOR_END (RatingDescriptor)

OD_IMPLEMENT_DESCRIPTOR_START (LanguageDescriptor)
	OD_IMPLEMENT_FIELD (LanguageDescriptor, languageCode)
OD_IMPLEMENT_DESCRIPTOR_END (LanguageDescriptor)

OD_IMPLEMENT_DESCRIPTOR_START (ShortTextualDescriptor)
	OD_IMPLEMENT_FIELD (ShortTextualDescriptor, languageCode)
	OD_IMPLEMENT_FIELD (ShortTextualDescriptor, isUTF8_string)
	OD_IMPLEMENT_FIELD (ShortTextualDescriptor, align)
	OD_IMPLEMENT_FIELD (ShortTextualDescriptor, eventName)
	OD_IMPLEMENT_FIELD (ShortTextualDescriptor, eventText)
OD_IMPLEMENT_DESCRIPTOR_END (ShortTextualDescriptor)

OD_IMPLEMENT_DESCRIPTOR_START (ExpandedTextualDescriptor)
	OD_IMPLEMENT_FIELD (ExpandedTextualDescriptor, languageCode)
	OD_IMPLEMENT_FIELD (ExpandedTextualDescriptor, isUTF8_string)
	OD_IMPLEMENT_FIELD (ExpandedTextualDescriptor, itemCount)
	OD_IMPLEMENT_FIELD (ExpandedTextualDescriptor, itemDescription)
	OD_IMPLEMENT_FIELD (ExpandedTextualDescriptor, itemText)
	OD_IMPLEMENT_FIELD (ExpandedTextualDescriptor, nonItemText)
OD_IMPLEMENT_DESCRIPTOR_END (ExpandedTextualDescriptor)  

OD_IMPLEMENT_DESCRIPTOR_START (ContentCreatorNameDescriptor)
	OD_IMPLEMENT_FIELD (ContentCreatorNameDescriptor, contentCreatorCount)
	OD_IMPLEMENT_FIELD (ContentCreatorNameDescriptor, languageCode)
	OD_IMPLEMENT_FIELD (ContentCreatorNameDescriptor, isUTF8_string)
	OD_IMPLEMENT_FIELD (ContentCreatorNameDescriptor, align)
	OD_IMPLEMENT_FIELD (ContentCreatorNameDescriptor, contentCreatorName)
OD_IMPLEMENT_DESCRIPTOR_END (ContentCreatorNameDescriptor)  

OD_IMPLEMENT_DESCRIPTOR_START (ContentCreationDateDescriptor)
	OD_IMPLEMENT_FIELD (ContentCreationDateDescriptor, contentCreationDate)
OD_IMPLEMENT_DESCRIPTOR_END (ContentCreationDateDescriptor)  

OD_IMPLEMENT_DESCRIPTOR_START (OCICreatorNameDescriptor)
	OD_IMPLEMENT_FIELD (OCICreatorNameDescriptor, OCICreatorCount)
	OD_IMPLEMENT_FIELD (OCICreatorNameDescriptor, languageCode)
	OD_IMPLEMENT_FIELD (OCICreatorNameDescriptor, isUTF8_string)
	OD_IMPLEMENT_FIELD (OCICreatorNameDescriptor, align)
	OD_IMPLEMENT_FIELD (OCICreatorNameDescriptor, OCICreatorName)
OD_IMPLEMENT_DESCRIPTOR_END (OCICreatorNameDescriptor)  

OD_IMPLEMENT_DESCRIPTOR_START (OCICreationDateDescriptor)
	OD_IMPLEMENT_FIELD (OCICreationDateDescriptor, OCICreationDate)
OD_IMPLEMENT_DESCRIPTOR_END (OCICreationDateDescriptor)  

// NHK 990702
OD_IMPLEMENT_DESCRIPTOR_START (MediaTimeDescriptor)
	OD_IMPLEMENT_FIELD (MediaTimeDescriptor, mediaTimeStamp)
OD_IMPLEMENT_DESCRIPTOR_END (MediaTimeDescriptor)

OD_IMPLEMENT_DESCRIPTOR_START (SmpteCameraPositionDescriptor)
	OD_IMPLEMENT_FIELD (SmpteCameraPositionDescriptor, cameraID)
	OD_IMPLEMENT_FIELD (SmpteCameraPositionDescriptor, parameterCount)
	OD_IMPLEMENT_FIELD (SmpteCameraPositionDescriptor, parameterID)
	OD_IMPLEMENT_FIELD (SmpteCameraPositionDescriptor, parameter)
OD_IMPLEMENT_DESCRIPTOR_END (SmpteCameraPositionDescriptor)

OD_IMPLEMENT_DESCRIPTOR_START (SLConfigDescriptor)
	OD_IMPLEMENT_FIELD (SLConfigDescriptor, predefined)
	OD_IMPLEMENT_FIELD (SLConfigDescriptor, useAccessUnitStartFlag)
	OD_IMPLEMENT_FIELD (SLConfigDescriptor, useAccessUnitEndFlag)
	OD_IMPLEMENT_FIELD (SLConfigDescriptor, useRandomAccessPointFlag)
	OD_IMPLEMENT_FIELD (SLConfigDescriptor, hasRandomAccessUnitsOnlyFlag)
	OD_IMPLEMENT_FIELD (SLConfigDescriptor, usePaddingFlag)
	OD_IMPLEMENT_FIELD (SLConfigDescriptor, useTimeStampsFlag)
	OD_IMPLEMENT_FIELD (SLConfigDescriptor, useIdleFlag)
	OD_IMPLEMENT_FIELD (SLConfigDescriptor, durationFlag)
	OD_IMPLEMENT_FIELD (SLConfigDescriptor, timeStampResolution)
	OD_IMPLEMENT_FIELD (SLConfigDescriptor, OCRResolution)
	OD_IMPLEMENT_FIELD (SLConfigDescriptor, timeStampLength)
	OD_IMPLEMENT_FIELD (SLConfigDescriptor, OCRLength)
	OD_IMPLEMENT_FIELD (SLConfigDescriptor, AU_Length)
	OD_IMPLEMENT_FIELD (SLConfigDescriptor, instantBitrateLength)
	OD_IMPLEMENT_FIELD (SLConfigDescriptor, degradationPriorityLength)
	OD_IMPLEMENT_FIELD (SLConfigDescriptor, AU_seqNumLength)
	OD_IMPLEMENT_FIELD (SLConfigDescriptor, packetSeqNumLength)
	OD_IMPLEMENT_FIELD (SLConfigDescriptor, reserved)
	OD_IMPLEMENT_FIELD (SLConfigDescriptor, timeScale)
	OD_IMPLEMENT_FIELD (SLConfigDescriptor, accessUnitDuration)
	OD_IMPLEMENT_FIELD (SLConfigDescriptor, compositionUnitDuration)
	OD_IMPLEMENT_FIELD (SLConfigDescriptor, startDecodingTimeStamp)
	OD_IMPLEMENT_FIELD (SLConfigDescriptor, startCompositionTimeStamp)
OD_IMPLEMENT_DESCRIPTOR_END (SLConfigDescriptor)

OD_IMPLEMENT_DESCRIPTOR_START (BIFSConfig)
	OD_IMPLEMENT_FIELD (BIFSConfig, nodeIDbits)
	OD_IMPLEMENT_FIELD (BIFSConfig, routeIDbits)
	OD_IMPLEMENT_FIELD (BIFSConfig, isCommandStream)
	OD_IMPLEMENT_FIELD (BIFSConfig, pixelMetric)
	OD_IMPLEMENT_FIELD (BIFSConfig, hasSize)
	OD_IMPLEMENT_FIELD (BIFSConfig, pixelWidth)
	OD_IMPLEMENT_FIELD (BIFSConfig, pixelHeight)
	OD_IMPLEMENT_FIELD (BIFSConfig, randomAccess)
	OD_IMPLEMENT_FIELD (BIFSConfig, animMask)
OD_IMPLEMENT_DESCRIPTOR_END (BIFSConfig)

OD_IMPLEMENT_DESCRIPTOR_START (BIFSv2Config)
	OD_IMPLEMENT_FIELD (BIFSv2Config, use3DMeshCoding)
	OD_IMPLEMENT_FIELD (BIFSv2Config, usePredictiveMFField)
	OD_IMPLEMENT_FIELD (BIFSv2Config, nodeIDbits)
	OD_IMPLEMENT_FIELD (BIFSv2Config, routeIDbits)
	OD_IMPLEMENT_FIELD (BIFSv2Config, protoIDbits)
	OD_IMPLEMENT_FIELD (BIFSv2Config, isCommandStream)
	OD_IMPLEMENT_FIELD (BIFSv2Config, pixelMetric)
	OD_IMPLEMENT_FIELD (BIFSv2Config, hasSize)
	OD_IMPLEMENT_FIELD (BIFSv2Config, pixelWidth)
	OD_IMPLEMENT_FIELD (BIFSv2Config, pixelHeight)
	OD_IMPLEMENT_FIELD (BIFSv2Config, randomAccess)
	OD_IMPLEMENT_FIELD (BIFSv2Config, animMask)
OD_IMPLEMENT_DESCRIPTOR_END (BIFSv2Config)

OD_IMPLEMENT_DESCRIPTOR_START (JPEG_DecoderConfig)
	OD_IMPLEMENT_FIELD (JPEG_DecoderConfig, headerLength)
	OD_IMPLEMENT_FIELD (JPEG_DecoderConfig, Xdensity)
	OD_IMPLEMENT_FIELD (JPEG_DecoderConfig, Ydensity)
	OD_IMPLEMENT_FIELD (JPEG_DecoderConfig, numComponents)
OD_IMPLEMENT_DESCRIPTOR_END (JPEG_DecoderConfig)

// SOWOO 20030715 begin
#if defined SAIT_CONFORMANCE_2003
OD_IMPLEMENT_DESCRIPTOR_START (PNG_DecoderConfig)
	OD_IMPLEMENT_FIELD (PNG_DecoderConfig, null)
OD_IMPLEMENT_DESCRIPTOR_END (PNG_DecoderConfig)
#endif SAIT_CONFORMANCE_2003
// SOWOO 20030715 end

OD_IMPLEMENT_DESCRIPTOR_START (JPEG2000_DecoderConfig)
	OD_IMPLEMENT_FIELD (JPEG2000_DecoderConfig, height)
	OD_IMPLEMENT_FIELD (JPEG2000_DecoderConfig, width)
	OD_IMPLEMENT_FIELD (JPEG2000_DecoderConfig, nc)
	OD_IMPLEMENT_FIELD (JPEG2000_DecoderConfig, BPC)
	OD_IMPLEMENT_FIELD (JPEG2000_DecoderConfig, C)
	OD_IMPLEMENT_FIELD (JPEG2000_DecoderConfig, UnkC)
	OD_IMPLEMENT_FIELD (JPEG2000_DecoderConfig, IPR)
OD_IMPLEMENT_DESCRIPTOR_END (JPEG2000_DecoderConfig)

// AudioSpecificConfig added by giorgio.zoia@epfl.ch 2000-01-20
// GeneralAudioConfig added by gew@iis.fhg.de 990512

OD_IMPLEMENT_DESCRIPTOR_START (AudioSpecificConfig)
	OD_IMPLEMENT_FIELD (AudioSpecificConfig, audioObjectType)
	OD_IMPLEMENT_FIELD (AudioSpecificConfig, samplingFrequencyIndex)
	OD_IMPLEMENT_FIELD (AudioSpecificConfig, samplingFrequency)
	OD_IMPLEMENT_FIELD (AudioSpecificConfig, channelConfiguration)
	OD_IMPLEMENT_FIELD (AudioSpecificConfig, GASpecificConfig)
	OD_IMPLEMENT_FIELD (AudioSpecificConfig, SASpecificConfig)
	OD_IMPLEMENT_FIELD (AudioSpecificConfig, CELPSpecificConfig)
OD_IMPLEMENT_DESCRIPTOR_END (AudioSpecificConfig)

OD_IMPLEMENT_DESCRIPTOR_START (GeneralAudioConfig)
	OD_IMPLEMENT_FIELD (GeneralAudioConfig, frameLengthFlag)
	OD_IMPLEMENT_FIELD (GeneralAudioConfig, dependsOnCoreCoder)
	OD_IMPLEMENT_FIELD (GeneralAudioConfig, coreCoderDelay)
	OD_IMPLEMENT_FIELD (GeneralAudioConfig, extensionFlag)
OD_IMPLEMENT_DESCRIPTOR_END (GeneralAudioConfig)

// SA and CELP SpecificConfig added by giorgio.zoia@epfl.ch 2000-01-20

OD_IMPLEMENT_DESCRIPTOR_START (StructuredAudioConfig)
	OD_IMPLEMENT_FIELD (StructuredAudioConfig, info)
OD_IMPLEMENT_DESCRIPTOR_END (StructuredAudioConfig)

OD_IMPLEMENT_DESCRIPTOR_START (CELP_DecoderConfig)
	OD_IMPLEMENT_FIELD (CELP_DecoderConfig, info)
OD_IMPLEMENT_DESCRIPTOR_END (CELP_DecoderConfig)

#ifdef TRACE_ENCODING
OD_IMPLEMENT_DESCRIPTOR_START (SL_PacketHeader)
	OD_IMPLEMENT_FIELD (SL_PacketHeader, accessUnitStartFlag)
	OD_IMPLEMENT_FIELD (SL_PacketHeader, accessUnitEndFlag)
	OD_IMPLEMENT_FIELD (SL_PacketHeader, OCRflag)
	OD_IMPLEMENT_FIELD (SL_PacketHeader, idleFlag)
	OD_IMPLEMENT_FIELD (SL_PacketHeader, paddingFlag)
	OD_IMPLEMENT_FIELD (SL_PacketHeader, paddingBits)
	OD_IMPLEMENT_FIELD (SL_PacketHeader, packetSequenceNumber)
	OD_IMPLEMENT_FIELD (SL_PacketHeader, DegPrioflag)
	OD_IMPLEMENT_FIELD (SL_PacketHeader, degradationPriority)
	OD_IMPLEMENT_FIELD (SL_PacketHeader, objectClockReference)
	OD_IMPLEMENT_FIELD (SL_PacketHeader, randomAccessPointFlag)
	OD_IMPLEMENT_FIELD (SL_PacketHeader, AU_sequenceNumber)
	OD_IMPLEMENT_FIELD (SL_PacketHeader, decodingTimeStampFlag)
	OD_IMPLEMENT_FIELD (SL_PacketHeader, compositionTimeStampFlag)
	OD_IMPLEMENT_FIELD (SL_PacketHeader, instantBitrateFlag)
	OD_IMPLEMENT_FIELD (SL_PacketHeader, decodingTimeStamp)
	OD_IMPLEMENT_FIELD (SL_PacketHeader, compositionTimeStamp)
	OD_IMPLEMENT_FIELD (SL_PacketHeader, accessUnitLength)
	OD_IMPLEMENT_FIELD (SL_PacketHeader, instantBitrate)
OD_IMPLEMENT_DESCRIPTOR_END (SL_PacketHeader)
#endif

/*-------------------------------------------------------------
A pseudo class used by the Mux tool
-------------------------------------------------------------*/
OD_IMPLEMENT_DESCRIPTOR_START (MuxInfo)
	OD_IMPLEMENT_FIELD (MuxInfo, fileName)
	OD_IMPLEMENT_FIELD (MuxInfo, streamFormat)
	OD_IMPLEMENT_FIELD (MuxInfo, streamAttr)
	OD_IMPLEMENT_FIELD (MuxInfo, repeatRate)
	OD_IMPLEMENT_FIELD (MuxInfo, sessionLength)
	OD_IMPLEMENT_FIELD (MuxInfo, maxAhead)
	OD_IMPLEMENT_FIELD (MuxInfo, startTime)
	OD_IMPLEMENT_FIELD (MuxInfo, muxCode)
	OD_IMPLEMENT_FIELD (MuxInfo, maxDeltaT)
	OD_IMPLEMENT_FIELD (MuxInfo, GroupID)
	OD_IMPLEMENT_FIELD (MuxInfo, UseDataReference)
OD_IMPLEMENT_DESCRIPTOR_END (MuxInfo)

OD_IMPLEMENT_DESCRIPTOR_START (MuxCodeSlot)
	OD_IMPLEMENT_FIELD (MuxCodeSlot, slotCount)
	OD_IMPLEMENT_FIELD (MuxCodeSlot, repetitionCount)
	OD_IMPLEMENT_FIELD (MuxCodeSlot, flexMuxChannel)
	OD_IMPLEMENT_FIELD (MuxCodeSlot, numberOfBytes)
OD_IMPLEMENT_DESCRIPTOR_END (MuxCodeSlot)

OD_IMPLEMENT_DESCRIPTOR_START (MuxCodeTableElement)
	OD_IMPLEMENT_FIELD (MuxCodeTableElement, length)
	OD_IMPLEMENT_FIELD (MuxCodeTableElement, muxCode)
	OD_IMPLEMENT_FIELD (MuxCodeTableElement, version)
	OD_IMPLEMENT_FIELD (MuxCodeTableElement, subStrucCount)
	OD_IMPLEMENT_FIELD (MuxCodeTableElement, slots)
OD_IMPLEMENT_DESCRIPTOR_END (MuxCodeTableElement)

OD_IMPLEMENT_DESCRIPTOR_START (MuxCodeTable)
	OD_IMPLEMENT_FIELD (MuxCodeTable, elements)
OD_IMPLEMENT_DESCRIPTOR_END (MuxCodeTable)

OD_IMPLEMENT_DESCRIPTOR_START (OCI_Event)
	OD_IMPLEMENT_FIELD (OCI_Event, eventID)
	OD_IMPLEMENT_FIELD (OCI_Event, absoluteTimeFlag)
	OD_IMPLEMENT_FIELD (OCI_Event, startingTime)
	OD_IMPLEMENT_FIELD (OCI_Event, duration)
	OD_IMPLEMENT_FIELD (OCI_Event, OCI_Descr)
OD_IMPLEMENT_DESCRIPTOR_END (OCI_Event)
