/*-------------------------------------------------------------------------
Project Im1Player/ Mux
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  OD Decoder/ Encoder
FILE:       ODP_SLExtension.cpp
AUTHOR:     Alexandre Cotarmanac'h Echevarria
			France Telecom R&D
E-MAIL:		alexandre.cotarmanach@francetelecom.com
OVERVIEW:   Implementation of SL Extension Descriptors.
-------------------------------------------------------------------------*/


#include "defs.h"
#include "IPMP_Messages.h"
//#include "ODIPMPX.h"
#include "odtags.h"

#include "ODP_SLExtension.h"
#include "CrSdl.h"

#include "ODCoder.h"										//pr DescEncDec


/*-------------------------------------------------------------------------
 AbstractSLConfig 
--------------------------------------------------------------------------*/
OD_IMPLEMENT_ABSTRACT_DESCRIPTOR_START (AbstractSLConfig)
	OD_IMPLEMENT_EXTENSION (SLConfigDescriptor)
	OD_IMPLEMENT_EXTENSION (ExtSLConfigDescr)
OD_IMPLEMENT_ABSTRACT_DESCRIPTOR_END (AbstractSLConfig)

/*------------------------------------------------------------------------*/


/*-------------------------------------------------------------------------
 SLExtensionDescriptor 
--------------------------------------------------------------------------*/
OD_IMPLEMENT_ABSTRACT_DESCRIPTOR_START (SLExtensionDescriptor)
	OD_IMPLEMENT_EXTENSION (DependencyPointer)
	OD_IMPLEMENT_EXTENSION (MarkerDescriptor)
OD_IMPLEMENT_ABSTRACT_DESCRIPTOR_END (SLExtensionDescriptor)


/*------------------------------------------------------------------------*/





/*-------------------------------------------------------------------------
 DependencyPointer  (SLExtensionDescriptor)

  class DependencyPointer extends SLExtensionDescriptor: bit(8) tag= DependencyPointerTag {
	bit(6) reserved;
	bit(1) mode;
	bit(1) hasESID;
	bit(8) dependencyLength;
	if (hasESID)
	{
		bit(16) ESID;
	} 
}
--------------------------------------------------------------------------*/
OD_IMPLEMENT_DESCRIPTOR_START (DependencyPointer)
	OD_IMPLEMENT_FIELD (DependencyPointer, reserved)
	OD_IMPLEMENT_FIELD (DependencyPointer, mode)
	OD_IMPLEMENT_FIELD (DependencyPointer, hasESID)
	OD_IMPLEMENT_FIELD (DependencyPointer, dependencyLength)
	OD_IMPLEMENT_FIELD (DependencyPointer, ESID)
OD_IMPLEMENT_DESCRIPTOR_END (DependencyPointer)

void DependencyPointer :: DescEncDec (ODCoder *pCoder, OPTYPE op)
{
	reserved.EncDec(pCoder, this, op);
	mode.EncDec(pCoder, this, op);
	hasESID.EncDec(pCoder, this, op);
	dependencyLength.EncDec(pCoder, this, op);
	if (hasESID)
	{
		ESID.EncDec(pCoder, this, op);
	}
}
/*------------------------------------------------------------------------*/



/*-------------------------------------------------------------------------
 MarkerDescriptor   (SLExtensionDescriptor)
	 class MarkerDescriptor extends SLExtensionDescriptor: bit(8) tag=DependencyMarkerTag {
		int(8) markerLength;
	}
--------------------------------------------------------------------------*/
OD_IMPLEMENT_DESCRIPTOR_START (MarkerDescriptor)
	OD_IMPLEMENT_FIELD (MarkerDescriptor, markerLength)
OD_IMPLEMENT_DESCRIPTOR_END (MarkerDescriptor)

void MarkerDescriptor :: DescEncDec (ODCoder *pCoder, OPTYPE op)
{
	markerLength.EncDec(pCoder, this, op);
}
/*------------------------------------------------------------------------*/




/*-------------------------------------------------------------------------
 ExtSLConfigDescr 
						TAG_ExtSLConfigDescrTag=0x64,


---------------------------------------------------------------------------

class ExtendedSLConfigDescriptor extends SLConfigDescriptor : bit(8) tag=ExtSLConfigDescrTag {
	SLExtensionDescriptor slextDescr[1..255];
}

-------------------------------------------------------------------------*/
//class ExtSLConfigDescr : public SLConfigDescriptor
//{
//	OD_DECLARE_DESCRIPTOR
//public:
//	SDLClassArray<class SLExtensionDescriptor, 0, 255> slextDescr; 
//
//private:
//	virtual void DescEncDec (ODCoder *pODCoder, OPTYPE op);
//};
OD_IMPLEMENT_DESCRIPTOR_START (ExtSLConfigDescr)
	OD_IMPLEMENT_FIELD (ExtSLConfigDescr, predefined)
	OD_IMPLEMENT_FIELD (ExtSLConfigDescr, useAccessUnitStartFlag)
	OD_IMPLEMENT_FIELD (ExtSLConfigDescr, useAccessUnitEndFlag)
	OD_IMPLEMENT_FIELD (ExtSLConfigDescr, useRandomAccessPointFlag)
	OD_IMPLEMENT_FIELD (ExtSLConfigDescr, hasRandomAccessUnitsOnlyFlag)
	OD_IMPLEMENT_FIELD (ExtSLConfigDescr, usePaddingFlag)
	OD_IMPLEMENT_FIELD (ExtSLConfigDescr, useTimeStampsFlag)
	OD_IMPLEMENT_FIELD (ExtSLConfigDescr, useIdleFlag)
	OD_IMPLEMENT_FIELD (ExtSLConfigDescr, durationFlag)
	OD_IMPLEMENT_FIELD (ExtSLConfigDescr, timeStampResolution)
	OD_IMPLEMENT_FIELD (ExtSLConfigDescr, OCRResolution)
	OD_IMPLEMENT_FIELD (ExtSLConfigDescr, timeStampLength)
	OD_IMPLEMENT_FIELD (ExtSLConfigDescr, OCRLength)
	OD_IMPLEMENT_FIELD (ExtSLConfigDescr, AU_Length)
	OD_IMPLEMENT_FIELD (ExtSLConfigDescr, instantBitrateLength)
	OD_IMPLEMENT_FIELD (ExtSLConfigDescr, degradationPriorityLength)
	OD_IMPLEMENT_FIELD (ExtSLConfigDescr, AU_seqNumLength)
	OD_IMPLEMENT_FIELD (ExtSLConfigDescr, packetSeqNumLength)
	OD_IMPLEMENT_FIELD (ExtSLConfigDescr, reserved)
	OD_IMPLEMENT_FIELD (ExtSLConfigDescr, timeScale)
	OD_IMPLEMENT_FIELD (ExtSLConfigDescr, accessUnitDuration)
	OD_IMPLEMENT_FIELD (ExtSLConfigDescr, compositionUnitDuration)
	OD_IMPLEMENT_FIELD (ExtSLConfigDescr, startDecodingTimeStamp)
	OD_IMPLEMENT_FIELD (ExtSLConfigDescr, startCompositionTimeStamp)
	OD_IMPLEMENT_FIELD (ExtSLConfigDescr, startCompositionTimeStamp)
	OD_IMPLEMENT_FIELD (ExtSLConfigDescr, slextDescr)
OD_IMPLEMENT_DESCRIPTOR_END (ExtSLConfigDescr)


void ExtSLConfigDescr :: DescEncDec (ODCoder *pCoder, OPTYPE op)
{
	if (pCoder->AcceptsSLExtension())
	{
		SLConfigDescriptor::DescEncDec(pCoder,op);
		slextDescr.EncDec(pCoder,this,op);
	}
	else
	{
		//-- error, ExtSL not allowed;
		throw;
	}
}

/*------------------------------------------------------------------------*/