#if !defined(__ODP_SLEXTENSION__H__)	// Sentry, use file only if it's not already included.
#define __ODP_SLEXTENSION__H__

/*-------------------------------------------------------------------------
Project Im1Player/ Mux
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  OD Decoder/ Encoder
FILE:       ODP_SLExtension.h
AUTHOR:     Alexandre Cotarmanac'h Echevarria
			France Telecom R&D
E-MAIL:		alexandre.cotarmanach@francetelecom.com
OVERVIEW:   Declarations of SL Extension Descriptors.
-------------------------------------------------------------------------*/

#include "sdl.h"
#include "sdldef.h"
//#include "odp.h"
#include "ODP.h" // SOLARIS PORTING - typo

/*-------------------------------------------------------------------------
 SL_EXTENSION_TAGS
Rque : requis pour l'implmentation des classes abstraites.
choix de l'intervalle complet
--------------------------------------------------------------------------*/
typedef enum
{
	
	TAGMIN_SLExtensionDescriptor	= 0x00,	
	TAGMIN_AbstractSLConfig			= 0x00,
	TAGMAX_SLExtensionDescriptor	= 0xFF,
	TAGMAX_AbstractSLConfig			= 0xFF
} SL_EXTENSION_TAGS;
/*------------------------------------------------------------------------*/


/*-------------------------------------------------------------------------
 SLExtensionDescriptor 
--------------------------------------------------------------------------*/
class SLExtensionDescriptor : public BaseDescriptor
{
	OD_DECLARE_ABSTRACT_DESCRIPTOR
public:
	virtual int GetMarkerLength()=0;
};
/*------------------------------------------------------------------------*/





/*-------------------------------------------------------------------------
 DependencyPointer  (SLExtensionDescriptor)

  class DependencyPointer extends SLExtensionDescriptor: bit(8) tag= DependencyPointerTag {
	bit(6) reserved;
	bit(1) mode;
	bit(1) hasESID;
	bit(8) dependencyLength;
	if (hasESID)
	{
		bit(16) ESID;
	} 
}
--------------------------------------------------------------------------*/
class DependencyPointer  : public SLExtensionDescriptor
{
	OD_DECLARE_DESCRIPTOR
public:
	SDLInt<6>	reserved;
	SDLInt<1>	mode;
	SDLInt<1>	hasESID;
	SDLInt<8>	dependencyLength;
	SDLInt<16>	ESID;
private:
	virtual void DescEncDec (ODCoder *pODCoder, OPTYPE op);
public :
	int GetMarkerLength() { return (int) dependencyLength; }
};
/*------------------------------------------------------------------------*/



/*-------------------------------------------------------------------------
 MarkerDescriptor   (SLExtensionDescriptor)
	 class MarkerDescriptor extends SLExtensionDescriptor: bit(8) tag=DependencyMarkerTag {
		int(8) markerLength;
	}
--------------------------------------------------------------------------*/
class MarkerDescriptor   : public SLExtensionDescriptor
{
	OD_DECLARE_DESCRIPTOR
public:
	SDLInt<8>	markerLength;
private:
	virtual void DescEncDec (ODCoder *pODCoder, OPTYPE op);
public :
	int GetMarkerLength() { return (int) markerLength; }
};

/*------------------------------------------------------------------------*/


/*-------------------------------------------------------------------------
 ExtSLConfigDescr 
						TAG_ExtSLConfigDescrTag=0x64,


---------------------------------------------------------------------------

class ExtendedSLConfigDescriptor extends SLConfigDescriptor : bit(8) tag=ExtSLConfigDescrTag {
	SLExtensionDescriptor slextDescr[1..255];
}

-------------------------------------------------------------------------*/
class ExtSLConfigDescr : public SLConfigDescriptor
{
	OD_DECLARE_DESCRIPTOR
public:
	SDLClassArray<class SLExtensionDescriptor, 0, 255> slextDescr; 

protected:
	virtual void DescEncDec (ODCoder *pODCoder, OPTYPE op);
};

#endif
