/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  BIFS Decoder/ Compositor / Presenter
FILE:       OpenStr.cpp
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Implementation of ObjectDescriptorImp and ESDescriptorImp
			funtions that handle opening/closing of streams.
-------------------------------------------------------------------------*/
#include "defs.h"
#include "ODImp.h"
#include "StreamCo.h"
#include "Executiv.h"
#include "Stream.h"
#include "AL.h"
#include "IPMP.h"
//#include "InLine.h"
#include "Inline.h" // SOLARIS PORTING - typo
#include "Channel.h"
#include "OCIDec.h"
#include "StrParser.h"
#include "ODP3.h"
#include "MediaControl.h"
#include "IPMPXinterface.h"

//#define REBUFFERING

// INT KM 2007 12 06
#include "..\IM1Decoders\AFX\AFX.h"
#include "..\IM1Decoders\AFX\KMFAMCDecoder.h"
// INT KM 2007 12 06


/*-------------------------------------------------------------
Destructor
-------------------------------------------------------------*/
ObjectDescriptorImp :: ~ObjectDescriptorImp ()
{
	RemoveClockLinks();

	//the service is deleted once all ODs are destroyed, so that clock namespace is still valid in ~delete
	delete m_pRemoteOD;

	delete m_pRemoteAL;
}

/*-------------------------------------------------------------
Create a ClockReference (or attach to an existing one) for each of the ESD in an OD
-------------------------------------------------------------*/
void ObjectDescriptorImp :: SetClocks ()
{
	for (int i = esDescr.GetLength (); --i >= 0;)
		((ES_DescriptorImp*) esDescr [i]) -> SetClock (m_pAL);
}

/*update clock associations*/
void ObjectDescriptorImp :: UpdateClockLinks()
{
	for (int i = esDescr.GetLength (); --i >= 0;)
		//link OD to clock
		((ES_DescriptorImp*) esDescr [i]) ->GetClock()->RegisterWithOD(this);
}

/*remove all clock associations*/
void ObjectDescriptorImp :: RemoveClockLinks()
{
	for (int i = esDescr.GetLength (); --i >= 0;) {
		ES_DescriptorImp*pESD = (ES_DescriptorImp*) esDescr [i];
		//unlink OD from clock
		pESD->GetClock()->UnregisterWithOD(this);
	}
}


/*-----------------------------------------------------------------
Go through the ESDescriptors contained in the OD, instantiate decoders
and IPMP systems, link them through MediaStream objects, and configure
all the input channels involved. The funtion returns a pointer to the
MediaStream object that provides the media to the compositor.
-----------------------------------------------------------------*/
MediaStreamImp *ObjectDescriptorImp :: SetStream (StreamConsumer *pNode)
{
	/*-------------------------------------------------------------
	If a remote OD has already been instantiated, use that one
	-------------------------------------------------------------*/
	if (m_pRemoteOD)
		return m_pRemoteOD ->SetStream (pNode);

	/*-------------------------------------------------------------
	Handle the remote OD case.
	-------------------------------------------------------------*/
	if (URL_Flag != 0) {
		if (m_nInstance++ == 0) {
			pNode -> GetExecutive () -> CreateService (URLstring, this, m_pAL -> GetServiceID ());
		}
		return NULL;
	}

	if (esDescr.GetLength () == 0)
	{	if (m_nInstance++ == 0)
			pNode -> GetExecutive () -> Error (NO_STREAM);
		return NULL;
	}

	/*-------------------------------------------------------------
	To do: reorder the ESDs according to their dependencies. 
	-------------------------------------------------------------*/

	/*-------------------------------------------------------------
	Notify an IPMPX system about this OD
	-------------------------------------------------------------*/
	if (m_nInstance++ == 0)
	{	m_pDecoder = NULL;
		IPMPServices *pIPMP = m_pAL ->GetIPMPServices ();
		if (pIPMP != NULL && !pIPMP ->ProcessObjectDescriptor (this))
		{	pNode ->GetExecutive () -> Error (
				IPMP_CANNOT_PROCESS_OD,
				(int) ObjectDescriptorID
			);
			return NULL;
		}
	}

	/*-------------------------------------------------------------
	Loop through the ESDs and set up decoder streams
	-------------------------------------------------------------*/
	MediaStreamImp *pResult = NULL;
	for (int i = 0; i < esDescr.GetLength (); i++)
	{	MediaStreamImp *pStream = ((ES_DescriptorImp *) esDescr [i]) ->
			SetDecoder (this, pNode);
		if (pResult == NULL)
			pResult = pStream;
	}
	return pResult;
}


void ObjectDescriptorImp :: GetSegmentList(StreamConsumer *pNode)
{
	/*-------------------------------------------------------------
	If the url of the node contains segment names translate them to time intervals according
	to the segmentDescriptor list
	-------------------------------------------------------------*/
	for (int i = ociDescr.GetLength (); --i >= 0;)
	{	if (ociDescr [i] -> GetTag () == TAG_SegmentDescriptor)
		{	SegmentDescriptor *pDesc = (SegmentDescriptor*) ociDescr [i];
			pNode -> SetSegment (pDesc ->segmentName, pDesc ->start, pDesc ->duration);
		}
	}
}

/*-------------------------------------------------------------
A session with remoteOD has been established
-------------------------------------------------------------*/
void ObjectDescriptorImp :: OnConnect (const BYTE *pInitOD, int nInitODLen)
{
	Executive *pExec = m_pInline -> GetExecutive ();
	ODDecoder odDec (
		pExec,
		pInitOD,
		nInitODLen,
		pExec -> GetTraceFile (),
		"IOD",
		pExec -> GetTraceFlags (),
		0
	);
	m_pRemoteOD = new ObjectDescriptorImp;
	m_pRemoteOD -> DecInitialOD (&odDec);
	m_pRemoteOD -> SetAL (m_pRemoteAL);
	m_pRemoteOD -> SetInline (m_pInline);
	m_pRemoteOD ->ObjectDescriptorID = ObjectDescriptorID;

	m_pRemoteOD -> SetClocks ();

	m_pRemoteAL -> SetIPMPSystem (m_pRemoteOD -> toolListDescr);
	m_pInline -> ConnectRemoteOD (m_pRemoteOD);

	//now that the real OD is associated to the inline, update clock links
	m_pRemoteOD->UpdateClockLinks();
}

/*-------------------------------------------------------------
When a new node is added on this OD, find the last OD in the remote OD chain and
attach that one to the node
-------------------------------------------------------------*/
void ObjectDescriptorImp :: AttachNode (StreamConsumer *pNode)
{
	if (m_pRemoteOD)
		m_pRemoteOD -> AttachNode (pNode);
	else
		pNode -> SetStream (this);
}

/*-------------------------------------------------------------
Start processing the streams described by this OD
-------------------------------------------------------------*/
void ObjectDescriptorImp :: Start ()
{
	/*-------------------------------------------------------------
	If no instance of the OD has opened the stream:	
	Loop through the ESDs and start them.
	If stream has prerolled no need to open the stream, just issue a play
	command.
	-------------------------------------------------------------*/
	if (m_nOpen++ == 0)
		for (int i = 0; i < esDescr.GetLength (); i++)
			((ES_DescriptorImp *) esDescr [i]) -> Start (m_bPreRolled, FALSE);


}

/*-------------------------------------------------------------
Stop processing the streams described by this OD
-------------------------------------------------------------*/
void ObjectDescriptorImp :: Stop ()
{
	/*-------------------------------------------------------------
	If this is the last instance using this node
	Loop through the ESDs and stop them.
	Then notify an associated MediaControl node, which will preroll the streams again, if
	necessary.
	-------------------------------------------------------------*/
	
	if (--m_nOpen == 0)
	{	for (int i = 0; i < esDescr.GetLength (); i++)
			((ES_DescriptorImp *) esDescr [i]) -> Stop ();
		m_bPreRolled = false;
		if (m_pControl)
			m_pControl -> StreamStopped ();
	}
}

/*-------------------------------------------------------------
Delete all the resources created by this OD
-------------------------------------------------------------*/
void  ObjectDescriptorImp:: Detach ()
{
	/*-------------------------------------------------------------
	if this is the last instance using the node
	Loop through the ESDs and detach them
	-------------------------------------------------------------*/
	if (--m_nInstance == 0)
	{	for (int i = 0; i < esDescr.GetLength (); i++)
		{	if (m_bPreRolled)
				((ES_DescriptorImp *) esDescr [i]) -> Stop ();
			((ES_DescriptorImp *) esDescr [i]) -> Detach ();
		}
		m_bPreRolled = false;

		/*-------------------------------------------------------------
		Notify an IPMPX system about the removal of this OD
		-------------------------------------------------------------*/
		IPMPServices *pIPMP = m_pAL ->GetIPMPServices ();
		if (pIPMP != NULL && !pIPMP ->RemoveObjectDescriptor (this))
		{	m_pAL -> GetExecutive () -> Error (
				IPMP_CANNOT_DELETE_OD,
				(int) ObjectDescriptorID
			);
		}
	}
}

/*-------------------------------------------------------------
Send data on a back channel
-------------------------------------------------------------*/
void ObjectDescriptorImp :: DataSend (LPBYTE pData, int nLength)
{
	if (m_pChannel)
		m_pChannel -> DataSend (pData, nLength);
}

double ObjectDescriptorImp :: GetDuration()
{ 
	return m_pChannel ? m_pChannel->m_Duration : -1;
}

bool ObjectDescriptorImp :: IsOpen() 
{
	return m_pChannel ? m_pChannel->IsOpen() : false; 
}

/*-------------------------------------------------------------
Find an ESD for removal
-------------------------------------------------------------*/
ES_DescriptorImp *ObjectDescriptorImp :: FindESD (int nESid)
{
	ES_DescriptorImp *pESD = NULL;
	if (m_pRemoteOD != NULL)
	{	m_pInline -> GetExecutive () -> Error (CANNOT_ALTER_REMOTE_OD);
		return NULL;
	}
	for (int i = esDescr.GetLength (); --i >= 0;)
	{	pESD = (ES_DescriptorImp *) esDescr [i];
		if (pESD -> ES_ID == nESid)
			return pESD;
	}
	m_pInline -> GetExecutive () -> Error (TRYING_TO_REMOVE_UNKNOWN_ESD);
	return NULL;
}

/*-------------------------------------------------------------
Find an ESD in this OD and remove it
-------------------------------------------------------------*/
void ObjectDescriptorImp :: RemoveESD (int nESid)
{
	ES_DescriptorImp *pESD;
	for (int i = esDescr.GetLength (); --i >= 0;)
	{	pESD = (ES_DescriptorImp *) esDescr [i];
		if (pESD -> ES_ID == nESid)
		{	esDescr.DeleteFrom (i);
			return;
		}
	}
}

/*-------------------------------------------------------------
Update an ESD for this OD
-------------------------------------------------------------*/
bool ObjectDescriptorImp :: UpdateESD (ES_DescriptorImp *pESD)
{
	if (m_pRemoteOD != NULL)
	{	m_pInline -> GetExecutive () -> Error (CANNOT_ALTER_REMOTE_OD);
		return false;
	}
	esDescr += pESD;
	return true;
}

/*-------------------------------------------------------------
Propagate a Play command to all streams
-------------------------------------------------------------*/
void ObjectDescriptorImp :: Play (double dSpeed)
{
	if (m_nOpen > 0)
		for (int i = 0; i < esDescr.GetLength (); i++)
			((ES_DescriptorImp *) esDescr [i]) -> Play (dSpeed);
}	

/*-------------------------------------------------------------
Preroll streams described by this OD (if not already started)
-------------------------------------------------------------*/
void ObjectDescriptorImp :: PreRoll ()
{
	/*-------------------------------------------------------------
	If no instance of the OD has opened the stream:	
	Loop through the ESDs and start them
	-------------------------------------------------------------*/
	if (m_nOpen == 0 && !m_bPreRolled)
	{	for (int i = 0; i < esDescr.GetLength (); i++)
			((ES_DescriptorImp *) esDescr [i]) -> Start (FALSE, TRUE);
		m_bPreRolled = true;
	}
}

/*-------------------------------------------------------------
Propagate a Pause command to all streams
-------------------------------------------------------------*/
void ObjectDescriptorImp :: Pause ()
{
	if (m_nOpen > 0)
		for (int i = 0; i < esDescr.GetLength (); i++)
			((ES_DescriptorImp *) esDescr [i]) -> Pause ();
}	

/*-------------------------------------------------------------
Propagate a Resume command to all streams
-------------------------------------------------------------*/
void ObjectDescriptorImp :: Resume()
{
	if (m_nOpen > 0)
		for (int i = 0; i < esDescr.GetLength (); i++)
			((ES_DescriptorImp *) esDescr [i]) -> Resume ();
}	


/*-------------------------------------------------------------
Propagate a Mute command to all streams
-------------------------------------------------------------*/
void ObjectDescriptorImp :: Mute ()
{
	if (m_nOpen > 0)
		for (int i = 0; i < esDescr.GetLength (); i++)
			((ES_DescriptorImp *) esDescr [i]) -> Mute ();
}

/*-------------------------------------------------------------
set speed of all in channels
-------------------------------------------------------------*/
void ObjectDescriptorImp :: SetSpeed(double speed)
{
	if (m_nOpen > 0 || m_bPreRolled)
		for (int i = 0; i < esDescr.GetLength (); i++)
			((ES_DescriptorImp *) esDescr [i]) -> SetSpeed(speed);
}	

/*-------------------------------------------------------------
ES_DescriptorImp constructor
-------------------------------------------------------------*/
ES_DescriptorImp :: ES_DescriptorImp ()
{
	m_pExecutive = NULL;
	m_pOutputStream = m_pInputStream = NULL;
	m_pDecoder = NULL;
	m_pChannel = NULL;
	m_pClock = NULL;
	m_bAttached = false;
	m_bGo = true;
	m_error = IM1_OK;
	m_nInstantiated = m_nStart = m_nDecoderStart = 0;
}

/*-------------------------------------------------------------
Create a ClockReference (or attach to an existing one) for this ESD
-------------------------------------------------------------*/
void ES_DescriptorImp :: SetClock (ALManager *pAL)
{
	if (slConfigDescr == NULL_DESC)
		return;
	
	/*-------------------------------------------------------------
	Find whether this is an OCR stream or it depends on another OCR stream.
	If it is an OCR stream decide whether real OCR are embedded (OCRLength > 0).
	Create a ClockReference or attach one, providing it this info
	-------------------------------------------------------------*/
	int nClockID = OCR_ES_Id;
	if (nClockID <= 0)
		nClockID = ES_ID;
	int nHasOCR = -1;
	if (nClockID == ES_ID)
		nHasOCR = slConfigDescr -> OCRLength > 0;
	m_pClock = pAL -> AttachClock (nClockID, nHasOCR);
}	

/*-------------------------------------------------------------
Instantiate a decoder and configure a channel for an elementary stream.
The function returns a pointer to the composition buffer.
-------------------------------------------------------------*/
MediaStreamImp *ES_DescriptorImp :: SetDecoder (
	ObjectDescriptorImp *pOD,
	StreamConsumer *pNode
)
{
	DecoderParams decoderParams;
	NON_FATAL_ERROR error = IM1_OK;


	/*-------------------------------------------------------------
	If a previous attempt to instantiate this decoder failed, return now
	-------------------------------------------------------------*/
	if (m_error != IM1_OK)
		return NULL;
	m_pOD = pOD;
	m_pAL = pOD -> GetAL ();
	m_pExecutive = m_pAL -> GetExecutive ();

	Decoder *pDecoder = NULL;
	if (decConfigDescr == NULL_DESC)
	{	m_pExecutive -> Error (m_error = NO_STREAM_TYPE);
		return NULL;
	}

	/*-------------------------------------------------------------
	OCR streams are opened without creating a data buffer
	-------------------------------------------------------------*/
	if (decConfigDescr -> streamType == ST_ClockReferenceStream)
	{	if (m_pChannel == NULL)
			m_pChannel = OpenStream (0, 0);
		return NULL;
	}

	/*-------------------------------------------------------------
	Igngore IPMP streams.
	Process IPMPX streams.
	-------------------------------------------------------------*/
	m_nInstantiated++;
	IPMPServices *pIPMP = m_pAL ->GetIPMPServices ();
	if (decConfigDescr -> streamType == ST_IPMPStream ||
		decConfigDescr -> streamType == ST_IPMPToolStream
	)
	{	if (pIPMP != NULL && m_nInstantiated == 1)
		{	int nBufferSize = decConfigDescr -> bufferSizeDB;
			if (nBufferSize == 0)
				nBufferSize = MPEG4_DEFAULT_IPMP_BUFFER_SIZE;
			m_pChannel = OpenStream (
				nBufferSize,
				MPEG4_OPTIMAL_IPMP_UNIT_SIZE
			);
			if (decConfigDescr -> streamType == ST_IPMPStream)
				pIPMP ->SetUpIPMPStream (pOD, this, m_pInputStream);
			else
				pIPMP ->ReceiveToolES (m_pInputStream, this); 
		}
		return NULL;
	}

	/*-------------------------------------------------------------
	Do the following only if a decoder has not been already instantiated
	-------------------------------------------------------------*/
	if (m_pDecoder == NULL)
	{	/*-------------------------------------------------------------
		If the stream depends on another stream find the decoder used by
		that stream. If no decoder is used, skip this stream as well.
		-------------------------------------------------------------*/
		if (dependsOn_ES_ID > 0) {
			for (int i = 0; i < pOD -> esDescr.GetLength (); i++) {
				if (dependsOn_ES_ID == pOD -> esDescr [i] -> ES_ID) {
					//regular dependency case
					if (pOD->esDescr[i]->decConfigDescr->streamType == decConfigDescr -> streamType) {
						pDecoder = ((ES_DescriptorImp *) pOD -> esDescr [i]) -> GetDecoder ();
						if (pDecoder == NULL) return NULL;
					}
					//other than "BIFS depends on OD"
					else if ((pOD->esDescr[i]->decConfigDescr->streamType != ST_ObjectDescriptorStream)
						|| (decConfigDescr->streamType != ST_SceneDescriptionStream)) {

						pDecoder = ((ES_DescriptorImp *) pOD -> esDescr [i]) -> GetDecoder ();
						if (pDecoder == NULL) return NULL;
					}
					break;
				}
			}
		}
						
		/*-------------------------------------------------------------
		First try if the node can create a decoder for this stream type.
		If the function returns the same decoder as the one used by stream
		whom this stream depends on, mark that this stream is attached to
		another decoder.
		-------------------------------------------------------------*/
		m_pDecoder =
			pNode -> CreateDecoder (decConfigDescr, qosDescr, pDecoder);
		m_bAttached = m_pDecoder != NULL && m_pDecoder == pDecoder;

		/*-------------------------------------------------------------
		ObjectContentInfoStream is a special case and we create a dedicated
		OCI parser for such stream
		-------------------------------------------------------------*/
		if (m_pDecoder == NULL && decConfigDescr -> streamType == ST_ObjectContentInfoStream)
			m_pDecoder = CreateOCIDecoder (decConfigDescr -> objectTypeIndication);
#ifdef MPGJ
	//*** YT - START (6)
	/*-------------------------------------------------------------
	MPEGJStream is a special case and we create a dedicated
	MPEGJ decoder for such stream
	-------------------------------------------------------------*/
	if (m_pDecoder == NULL && decConfigDescr -> streamType == ST_MPEGJStream)
		m_pDecoder = CreateMPEGJDecoder();
	//*** YT - END (6)
#endif MPGJ

		/*-------------------------------------------------------------
		If the node didn't create a decoder, create it now using the info
		in decConfigDescr. We skip this stream if the OD already has a
		decoder.
		-------------------------------------------------------------*/
		if (m_pDecoder == NULL)
		{
		#ifndef MPGJ
		if(pOD -> GetDecoder () != NULL)
			return NULL;
		#else
		//*** YT START (104)
		// Original IM1Core coder ignore elementary streams
		// of which OD has already setup a decoder for the
		// primary ES.
		// Since we want to switch decoder between ESs
		// we setup buffers and channels for secondary ESs also

		m_pDecoder = pOD -> GetDecoder ();
		if (m_pDecoder != NULL) {
			// here IM1COre used to return NULL;
			// but we setup channel
			m_pChannel = OpenStream (
				decoderParams.m_nDBSize,
				decoderParams.m_nOptimalAUSize
				);
			if (m_pChannel == NULL) {
				Detach ();
				return NULL;
			}

			// and setup buffers...
			if (decoderParams.m_nCBSize != 0)
				m_pOutputStream = new MediaStreamImp (
				m_pInputStream -> GetClockReference (),
				decoderParams.m_nCBSize + m_pDecoder -> GetOptimalOutputSize ()
				);

			// but this ES doesn't have decoder now
			m_pDecoder = NULL;

			// input buffer is attached to DataChannel
			// Open() will be called by DataChannel
			// Datachannel will call the input buffer attached to
			// the decoder
			// so make the buffer discard incoming AUs
			m_pInputStream->HalfClose(TRUE);
			m_pInputStream->Start();


			// output buffer is not attached to anybody
			// won't be called from anybody
			// so we don't call HalfClose() or Pause();
			m_pOutputStream->Open();
			m_pOutputStream->Start();

			//*** YT END (104)
			return NULL;
		}
		#endif
			m_pDecoder = (Decoder *) LoadFilter (
				decConfigDescr -> streamType,
				decConfigDescr -> objectTypeIndication
			);
		#ifdef MPGJ // MPEGJ - PA
		m_pDecoder ->decoderType = decConfigDescr -> streamType;
		#endif
		}

		/*-------------------------------------------------------------
		Setup the decoder
		-------------------------------------------------------------*/
		if (m_pDecoder != NULL)
		{	DecoderSpecificInfo *pSpecificInfo = decConfigDescr -> decSpecificInfo;

			LPCBYTE pInfo = NULL;
			int nInfoLength = 0;
			if (pSpecificInfo != NULL)
			{	pInfo = pSpecificInfo -> info;
				nInfoLength = pSpecificInfo -> info.GetLength ();
			}


// INT KM 2007 6 12 2007
			if (m_pDecoder->decoderType == 3) {// we test for AFX decoder
				if (pNode->GetNodeCode() == CODE_BitWrapper) {// BitWrapper.node
					NodeField* pNodeChild = (NodeField*) pNode->GetField(0);
					MediaObject* pMO = ((SFGenericNode*) pNodeChild)->GetNode();		
					if (pMO->GetNodeCode() == CODE_IndexedFaceSet) { // we read the first frame from the bit wrapper
						IndexedFaceSet * ifs = ( IndexedFaceSet *) pMO;
						((AFXDecoder * ) m_pDecoder)->SetIFS(ifs);
					}			
				}
			}
// INT KM 2007 6 12 2007



			if (!m_pDecoder -> Setup (pInfo, nInfoLength, 0))
				m_error = DECODER_SETUP_FAILURE;
			else {
				pOD -> SetDecoder (m_pDecoder);
			}

		}
	}

	/*-------------------------------------------------------------
	Get the rendering capabilities of the node.
	If there are no capabilities this is a non rendering node like MediaControl or
	MediaSensor
	-------------------------------------------------------------*/
	CapabilitiesImp caps;
	pNode -> GetCapabilities (&caps);
	bool bFormatOK = false;
	if (m_pDecoder)
		bFormatOK = m_pDecoder -> SetFormat (caps, m_decoderParams);

	if (caps.GetFormatCapabilityLength () > 0)
	{	/*-----------------------------------------------------------------
		Verify that there is a matching capability between the decoder and
		the node, and sets up the decoder format. If no matching
		capability exists, delete the decoder and ignore the stream
		-----------------------------------------------------------------*/
		if (m_pDecoder == NULL)
			m_error = UNKNOWN_DECODER_TYPE;
		else
		{	if (bFormatOK)
				pNode -> SetStreamFormat (m_decoderParams.m_format);
			else
				m_error = INCOMPATIBLE_DECODER_TYPE;
		}
	}

	if (m_error != IM1_OK)
	{	m_pExecutive -> Error (m_error);
		return NULL;
	}

	/*-------------------------------------------------------------
	Establish an input channel if hasn't been done already
	-------------------------------------------------------------*/
	if (m_pChannel == NULL)
	{	/*-------------------------------------------------------------
		Determine buffer threshold numbers. Use QoS_Qualifier_REBUFFERING_RATIO
		for this.
		-------------------------------------------------------------*/
		float fThreshold = 0;

		if (qosDescr != NULL_DESC)
		{	for (int i = qosDescr ->  qualifiers.GetLength (); --i >= 0;)
				if (qosDescr -> qualifiers [i] -> GetTag () == TAG_QoS_Qualifier_REBUFFERING_RATIO)
				{	fThreshold = 0.01 *
						((QoS_Qualifier_REBUFFERING_RATIO*) qosDescr -> qualifiers [i]) ->
						REBUFFERING_RATIO;
					break;
				}
		}

		/*-----------------------------------------------------------------
		Set up the channel
		-----------------------------------------------------------------*/
		m_pChannel = OpenStream (
						m_decoderParams.m_nDBSize,
						m_decoderParams.m_nOptimalAUSize,
						m_decoderParams.m_bBackChannel,
						fThreshold
					 );
		if (m_pChannel == NULL)
			return NULL;
		pOD -> SetChannel (m_pChannel);
	}

	/*-----------------------------------------------------------------
	Create the composition buffer.
	If decoderParams.m_nCBSize + m_pDecoder -> GetOptimalOutputSize () 
	is zero, it means no composition buffer is used, and the decoder
	manipulates the scene graph directly.
	(this is done only if a decoder has been instantiated and the output stream not yet)
	-----------------------------------------------------------------*/
	if (m_pOutputStream == NULL && m_pDecoder != NULL)
	{	if (m_decoderParams.m_nCBSize != 0)
			m_pOutputStream = new MediaStreamImp (
				m_pInputStream -> GetClockReference (),
				m_decoderParams.m_nCBSize + m_pDecoder -> GetOptimalOutputSize ()
			);
		m_pDecoder -> SetOutputStream (
			m_decoderParams.m_nStreamNumber,
			m_pOutputStream
		);
	}

	if (m_nInstantiated == 1)
	{	/*-------------------------------------------------------------
		if IPMPX system is not active and this is the first instance of this ESD,
		instantiate IPMP systems for this stream. If failed, dont use this stream
		-------------------------------------------------------------*/
		if (pIPMP == NULL)
		{	if (!InvokeIPMP ())
				m_error = IPMP_NOT_AVAILABLE;
		}
		
		/*-------------------------------------------------------------
		Set the input stream of the decoder and add it to the list of filters
		-------------------------------------------------------------*/
		if (m_pDecoder != NULL)
		{	m_pDecoder -> SetInputStream (0, m_pInputStream);
			m_filters += m_pDecoder;
		}

		/*-------------------------------------------------------------
		If an IPMPX system is active and this is the first instantiation of this
		ESD, process this ESD through the IPMPX interface.
		Otherwise skip IPMP streams
		-------------------------------------------------------------*/
		if (pIPMP != NULL)
		{	switch (pIPMP ->ProcessESDescriptor (this)) {
			case IPMPServices :: ALLOWED:
				m_bGo = true;
				break;
			case IPMPServices :: DENIED:
				m_error = IPMP_NOT_AVAILABLE;
				return NULL;
			case IPMPServices :: PROTECTED:
				m_bGo = false;
				break;
			}
		}
		if (m_error != IM1_OK)
		{	Detach ();
			return NULL;
		}
	}

	#ifdef MPGJ // MPEGJ - PA
	m_pDecoder -> SetExecutive(m_pExecutive);
	#endif

	/*-------------------------------------------------------------
	If decoderParams.m_nCBSize is zero
	and m_pDecoder -> GetOptimalOutputSize () is not zero,
	this is null decoder, i.e., stream is consumed directly by the 
	object.
	-------------------------------------------------------------*/
	if (m_pDecoder == NULL ||
		m_decoderParams.m_nCBSize == 0 && m_pDecoder -> GetOptimalOutputSize () != 0
	)
		return m_pInputStream;

	if (m_pOutputStream) {
		return m_pOutputStream;
	}
	return m_pInputStream;
}

/*-------------------------------------------------------------
Load the DLL and instantiate the Filter object for the given streamType
and objectTypeIndication. Returns NULL if failed.
-------------------------------------------------------------*/
Filter *ES_DescriptorImp :: LoadFilter (
	int nStreamType,
	int objectTypeIndication
)
{
	static ZRegistry registry("Decoders");
	HMODULE hDLL = NULL;

	/*-------------------------------------------------------------
	Construct the decoder type and asks the registry object to locate it,
	load its DLL, and instantiate it.
	-------------------------------------------------------------*/
	static LPCSTR szStreamType [] =
	{
		"",			// ObjectDescriptorStream = 1
		"",			// ClockReferenceStream = 2
		"BIFS",			// SceneDescriptionStream = 3
		"Visual",	// VisualStream = 4
		"Audio",	// AudioStream = 5
		"MPEG7",	// MPEG7Stream = 6
		"IPMP",		// IPMP = 7,
		"OCI",		// OCI = 8,
		"MPEGJ",	// MPEGJ = 9,
		"Interaction", //InteractionStream = 10,
		"IPMPTool"	// IPMPToolStream
	};
	enum {TABLE_LENGTH = sizeof szStreamType / sizeof (LPCSTR)};

	if (--nStreamType < 0 || nStreamType >= TABLE_LENGTH ||
		szStreamType [nStreamType] [0] == 0
	)
		return NULL;

	ZString szFilterType (szStreamType [nStreamType]);
	if (objectTypeIndication != 0 && objectTypeIndication != 0xff)
	{	char szBuffer [10];
		szFilterType += '-';
		szFilterType += itoa (objectTypeIndication, szBuffer, 16);
	}

	Filter *pFilter =
		(Filter *) registry.CreateInstance (szFilterType, hDLL);
	if (pFilter != NULL)
		pFilter -> SetLibrary (hDLL);
	return pFilter;
}

/*-------------------------------------------------------------
Perform the stream open procedure with this ES_Descriptor
-------------------------------------------------------------*/
DataChannel *ES_DescriptorImp :: OpenStream (
	int nBufferSize,
	int nExpectedAULength,
	bool bBackChannel,
	float fThreshold
)
{
	if (slConfigDescr == NULL_DESC)
	{	m_pExecutive -> Error (NO_SLCONFIG_INFO);
		return NULL;
	}

	/*-------------------------------------------------------------
	Open the stream, and attach it to a MediaStream object.
	If nBufferSize is zero, open the stream without attaching to
	a MediaStream, which means "get data and discard"
	-------------------------------------------------------------*/
	if (nBufferSize != 0)
	{
		int nThreshold = 0;

		if (nBufferSize > 0)
		{	if (decConfigDescr != NULL_DESC &&
				decConfigDescr -> bufferSizeDB != 0
			)
			{	nBufferSize = decConfigDescr -> bufferSizeDB;
				nThreshold = nBufferSize * fThreshold;
			}

			/*-------------------------------------------------------------
			The actual buffer size is calcualted as following:
			If nExpectedAULength > 0, we use twice the declared amount of
			buffer size, to compenstate for jitter delays, and solve buffer
			slicing (units need to be in contiguneous memory).
			If nExpectedAULength is zero, we assume buffer is required for
			just one AU.
			To this we have to add space for AU headers. If nExpectedAULength
			> 0, we take 3 x buffer-size/average unit-size. If 
			nExpectedAULength is 0, there is just one unit and one header.
			-------------------------------------------------------------*/
			if (nExpectedAULength > 0)
				nBufferSize =
					nBufferSize * 2 +
					3 * (sizeof (int) + sizeof (DWORD)) *
					(nBufferSize / nExpectedAULength + 1);
			else
			{	nExpectedAULength = nBufferSize;
				nBufferSize += sizeof (int) + sizeof (DWORD);
			}
		}

		/*this is to cope with some content with bad buffer size on BIFS/OD. It's ugly but we sure don't want the
		buffers to be dynamic*/
#ifdef OSMOSE
		if ( (decConfigDescr -> streamType == 1) || (decConfigDescr -> streamType == 3)) nBufferSize *= 2;
#endif

		/*-------------------------------------------------------------
		Create a MediaStream object with NULL clock reference. The
		clock reference will be set by ALManager::AttachStream()
		-------------------------------------------------------------*/
		m_pInputStream = new MediaStreamImp (NULL, nBufferSize, nThreshold);
	}


	/*-------------------------------------------------------------
	Open the input channel and link it to the proper MediaStream object
	-------------------------------------------------------------*/
	return m_pAL -> AttachStream (
		this,
		m_pInputStream,
		slConfigDescr,
		nExpectedAULength,
		bBackChannel
	);
}

/*-------------------------------------------------------------
Attach IPMP filters to an elementary stream. Return true if succeeded 
to setup all IPMP systems.
-------------------------------------------------------------*/
bool ES_DescriptorImp :: InvokeIPMP ()
{
	int i;

	/*-------------------------------------------------------------
	Loop through the ESDs to set up IPMP filters established by IPMP
	streams.
	-------------------------------------------------------------*/
	for (i = 0; i < m_pOD -> esDescr.GetLength (); i++)
		if (! ((ES_DescriptorImp *) m_pOD -> esDescr [i]) -> SetIPMP (this))
			return false;

	/*-------------------------------------------------------------
	Loop through the IPMP pointers and handle IPMP descriptors
	-------------------------------------------------------------*/
	if (!HandleIPMPDescriptors (
			ipmpDescrPtr,
			ipmpDescrPtr.GetLength (),
			m_pOD -> GetInline ()
		)
	)
		return false;

	/*-------------------------------------------------------------
	Loop through the IPMP pointers of the OD and handle IPMP descriptors
	-------------------------------------------------------------*/
	if (!HandleIPMPDescriptors (
			m_pOD -> ipmpDescrPtr,
			m_pOD -> ipmpDescrPtr.GetLength (),
			m_pOD -> GetInline ()
		)
	)
		return false;
	return true;
}

/*-------------------------------------------------------------
If the object is an IPMP stream, instantiate an IPMP system and attach
it to the given elementary stream
-------------------------------------------------------------*/
bool ES_DescriptorImp :: SetIPMP (ES_DescriptorImp *pES)
{
	if (decConfigDescr == NULL_DESC ||
		decConfigDescr -> streamType != ST_IPMPStream
	)
		return true;

	/*-------------------------------------------------------------
	This version doesn't allow the attaching of more than IPMP stream to
	a single data stream
	-------------------------------------------------------------*/
	if (m_pChannel != NULL)
	{	m_pExecutive -> Error (IPMP_STREAM_ALREADY_IN_USE);
		return false;
	}

	m_pAL = pES -> m_pAL;
	m_pExecutive = pES ->m_pExecutive;
	m_pOD = pES ->m_pOD;
	/*-------------------------------------------------------------
	Load the appropriate IPMP system
	-------------------------------------------------------------*/
	IPMPManager *pIPMP = (IPMPManager *) LoadFilter (
		ST_IPMPStream,
		decConfigDescr -> objectTypeIndication
	);
	if (!pIPMP)
	{	m_pExecutive -> Error (IPMP_NOT_AVAILABLE);
		return false;
	}
	else if (!pIPMP -> GetAccess ())
	{	m_pExecutive -> Error (NO_ACCESS_TO_PROTECTED_STREAM);
		return false;
	}

	//added by Andrea Varesio
	pIPMP -> SetAssociatedDecoder((void *) (pES -> m_pDecoder));	//the decoder is got from ES_DescriptorImp

	/*-------------------------------------------------------------
	Open the IPMP channel
	-------------------------------------------------------------*/
	int nBufferSize = decConfigDescr -> bufferSizeDB;
	if (nBufferSize == 0)
		nBufferSize = MPEG4_DEFAULT_IPMP_BUFFER_SIZE;
	m_pChannel = OpenStream (
		nBufferSize,
		MPEG4_OPTIMAL_IPMP_UNIT_SIZE
	);
	if (m_pChannel == NULL)
		return false;

	/*-------------------------------------------------------------
	Setup the IPMP system
	-------------------------------------------------------------*/
	DecoderSpecificInfo *pSpecificInfo = decConfigDescr -> decSpecificInfo;
	LPCBYTE pInfo = NULL;
	int nInfoLength = 0;
	if (pSpecificInfo != NULL)
	{	pInfo = pSpecificInfo -> info;
		nInfoLength = pSpecificInfo -> info.GetLength ();
	}
	pIPMP -> SetIPMPStream (
		m_pInputStream,
		decConfigDescr -> objectTypeIndication,
		pInfo,
		nInfoLength
	);

	/*-------------------------------------------------------------
	Attach the system to the elementary stream
	-------------------------------------------------------------*/
	pES -> AttachIPMP (pIPMP);
	return true;
}

/*-------------------------------------------------------------
Handle a set of IPMP descriptors linked to this stream.
Return true if all descriptors where processed successfully.
-------------------------------------------------------------*/
bool ES_DescriptorImp :: HandleIPMPDescriptors (
	IPMP_DescriptorPointer * const * descPointers,
	int nDescPointersLength,
	InlineImp *pInline
)
{
	/*-------------------------------------------------------------
	Loop through all the pointers and locate the descriptors they are
	pointing to
	-------------------------------------------------------------*/
	IPMPManager *pGenericIPMP = NULL;
	for (int i = 0; i < nDescPointersLength; i++)
	{
		int nID = descPointers [i] -> IPMP_DescriptorID;
		IPMPManager *pIPMP = NULL;
		IPMP_DescriptorImp *pIPMPD = pInline -> GetIPMP_Descriptor (nID);

		/*-------------------------------------------------------------
		If a descriptor pointed by the pointer already exists,
		walk through all the IPMP systems currently attached to this ES.
		-------------------------------------------------------------*/
		if (pIPMPD != NULL)
			for (int j = m_ipmps.GetLength (); --j >= 0;)
				if (m_ipmps [j] -> IsMyType (pIPMPD -> IPMPS_Type))
				{	pIPMP = m_ipmps [j];
					break;
				}
	
		/*-------------------------------------------------------------
		If no system can handle the descriptor, or if the descriptor is
		not known yet, instantiate a generic IPMP system, or use one if
		has already been instantiated before.
		-------------------------------------------------------------*/
		if (!pIPMP)
			pIPMP = pGenericIPMP;
		if (!pIPMP)
		{	pIPMP = pGenericIPMP =
				(IPMPManager *) LoadFilter (ST_IPMPStream, 0);
			if (!pIPMP)
			{	m_pExecutive -> Error (IPMP_NOT_AVAILABLE);
				return false;
			}
			AttachIPMP (pIPMP);
		}

		/*-------------------------------------------------------------
		If a descriptor exists pass it to the IPMP system
		-------------------------------------------------------------*/
		if (pIPMPD)
			switch (pIPMPD -> Notify (pIPMP)) {
			case 0:
				m_pExecutive -> Error (NO_ACCESS_TO_PROTECTED_STREAM);
				return false;
			case -1:
				m_pExecutive -> Error (IPMP_NOT_AVAILABLE);
				return false;
			}

		/*-------------------------------------------------------------
		Associate the IPMP system with the descriptor ID for future
		IPMPDescriptor updates
		-------------------------------------------------------------*/
		pInline -> AssociateIPMPSystem (nID, pIPMP);
	}

	return true;
}

/*-------------------------------------------------------------
Attach an IPMP filter to this elementary stream
-------------------------------------------------------------*/
void ES_DescriptorImp :: AttachIPMP (IPMPManager *pIPMP)
{
	/*-------------------------------------------------------------
	Determine whether this is a pre or post decoding IPMP system.
	Create a MediaStream object with the proper size, depending
	on previous question.
	Link before or after the decoder.
	-------------------------------------------------------------*/
	bool bPostDecoding = pIPMP -> IsPostDecoding ();
	if (bPostDecoding && m_pOutputStream == NULL)
		return;		// post decoding not applicable.
	MediaStreamImp *pStream = new MediaStreamImp (
		m_pInputStream -> GetClockReference (),
		(bPostDecoding ? m_pOutputStream : m_pInputStream) -> GetSize ()
	);
	if (bPostDecoding)
	{	pIPMP -> SetInputStream (m_pOutputStream);
		pIPMP -> SetOutputStream (pStream);
		m_pOutputStream = pStream;
	}
	else
	{	pIPMP -> SetInputStream (m_pInputStream);
		pIPMP -> SetOutputStream (pStream);
		m_pInputStream = pStream;
	}

	/*-------------------------------------------------------------
	Add to the list of IPMP systems
	-------------------------------------------------------------*/
	m_ipmps += pIPMP;
}

/*-------------------------------------------------------------
Start processing the stream described by this ES_Descriptor
-------------------------------------------------------------*/
void ES_DescriptorImp :: Start (bool wasPreRolled, bool setPreRolled)
{
	if (m_error != IM1_OK)
		return;

	/*-------------------------------------------------------------
	If stream has been prerolled only send a play command
	-------------------------------------------------------------*/
	if (wasPreRolled)
	{	if (m_pChannel)
			m_pChannel ->Play ();
		return;
	}

	/*restart the composition memory if any and desired*/
	if (m_pOutputStream) {
		m_pOutputStream->SetInitialBufferingOn();
		m_pOutputStream->Start();
	}

	/*-------------------------------------------------------------
	Start all filters connected to this ES_Descriptor, including its own decoder
	-------------------------------------------------------------*/
	if (m_pDecoder && !m_bAttached && m_nDecoderStart++ == 0)
		for (int i = 0; i < m_filters.GetLength (); i++)
			m_filters [i] ->Start ();

	if (m_nStart++ == 0)
	{	if (m_pChannel && m_bGo)
		{	m_pChannel -> ChannelReady ();
			/*set the channel preroll state*/
			m_pChannel->SetPrerolled(setPreRolled);
			m_pExecutive -> SetPendingChannels ();
		}

		/*-------------------------------------------------------------
		Start all IPMP systems
		-------------------------------------------------------------*/
		for (int i = 0; i < m_ipmps.GetLength (); i++)
			m_ipmps [i] -> Start ();
	}
}

/*-------------------------------------------------------------
Stop processing the stream described by this ES_Descriptor
-------------------------------------------------------------*/
void ES_DescriptorImp :: Stop ()
{

	/*-------------------------------------------------------------
	Stop all IPMP systems
	-------------------------------------------------------------*/
	if (--m_nStart == 0)
		for (int i = 0; i < m_ipmps.GetLength (); i++)
			m_ipmps [i] -> Stop ();


	/*also reset the composition memory if any - close once the input channel is stoped*/
	if (m_pOutputStream) m_pOutputStream->Terminate();

	if (m_pChannel && m_nStart == 0 && m_bGo)
		m_pChannel -> ChannelStop ();


	/*-------------------------------------------------------------
	Unlock scene before closing in case the decoder thread
	is blocking on scene lock. UnlockScene is called with nReserve>0,
	to prevent other threads from modifying the scene.
	-------------------------------------------------------------*/
	m_pExecutive -> UnlockScene (1);

	/*-------------------------------------------------------------
	Stop all the filters and close the channel
	-------------------------------------------------------------*/
	if (m_pDecoder && !m_bAttached && --m_nDecoderStart == 0) {
		for (int i = 0; i < m_filters.GetLength (); i++) {
			m_filters [i] ->Stop ();
		}
	}
	m_pExecutive -> LockScene (1);
}

/*-------------------------------------------------------------
Detach all the resources created by this ESD
-------------------------------------------------------------*/
void ES_DescriptorImp :: Detach ()
{
	#ifdef MPGJ
	//*** YT START (105)
	// when the ES is not attached to decoder
	// we need to shutdown buffer
	// this task used to be done by the decoder
	if(m_pDecoder == NULL) {
		m_pOutputStream->Terminate();
		m_pOutputStream->Close();
	}
	//*** YT END (105)
	#endif
	/*-------------------------------------------------------------
	Notify an IPMPX system about the removal of this ESD
	-------------------------------------------------------------*/
	IPMPServices *pIPMP = m_pAL ->GetIPMPServices ();
	if (m_nInstantiated && pIPMP)
	{	switch (decConfigDescr -> streamType) {
		case ST_IPMPStream:
			pIPMP ->RemoveIPMPStream (this);
			break;
		case ST_IPMPToolStream:
			pIPMP ->RemoveToolES (this);
			break;
		default:
			pIPMP ->RemoveESDescriptor (this);
			break;
		}
	}
	m_nInstantiated = 0;
	delete m_pOutputStream;
	m_pOutputStream = NULL;
	if (m_pDecoder && !m_bAttached)
	{	HMODULE hDLL = m_pDecoder -> GetLibrary ();
		delete m_pDecoder;
		m_pDecoder = NULL;
		m_bAttached = false;
		if (hDLL)
			FreeLibrary (hDLL);
	}	
	delete m_pChannel;
	m_pChannel = NULL;
	#ifdef MPGJ
	//*** YT START (105)
	if(m_pDecoder == NULL) {
		m_pInputStream->Terminate();
		m_pInputStream->Close();
	}
	//*** YT END (105)
	#endif MPGJ
	delete m_pInputStream;
	m_pInputStream = NULL;

	/*-------------------------------------------------------------
	Cleanup the filters list
	-------------------------------------------------------------*/
	m_filters.Clear ();

	/*-------------------------------------------------------------
	Delete all IPMP systems
	-------------------------------------------------------------*/
	for (int i = 0; i < m_ipmps.GetLength (); i++)
	{	IPMPManager *pIPMP = m_ipmps [i];
		/*-------------------------------------------------------------
		Loop through the IPMP pointers and disassociate the IPMP
		system from the descriptors
		-------------------------------------------------------------*/
		RemoveIPMPSystems (
			pIPMP,
			ipmpDescrPtr,
			ipmpDescrPtr.GetLength (),
			m_pOD -> GetInline ()
		);

		/*-------------------------------------------------------------
		Loop through the IPMP pointers of the OD and disassociate the IPMP
		system from the descriptors
		-------------------------------------------------------------*/
		RemoveIPMPSystems (
			pIPMP,
			m_pOD -> ipmpDescrPtr,
			m_pOD -> ipmpDescrPtr.GetLength (),
			m_pOD -> GetInline ()
		);

		delete pIPMP -> GetInputStream ();
		HMODULE hDLL = pIPMP -> GetLibrary ();
		delete pIPMP;
		if (hDLL)
			FreeLibrary (hDLL);
	}
	m_ipmps.Clear ();
}

/*-------------------------------------------------------------
Add a new ESD to an active OD
-------------------------------------------------------------*/
MediaStreamImp *ES_DescriptorImp :: AddESD (
	ObjectDescriptorImp *pOD,
	StreamConsumer *pNode
)
{
	return SetDecoder (pOD, pNode);
}

/*-------------------------------------------------------------
When an IPMP system is deleted, detach it from all the descriptors
it was linked to trough IPMP descriptor pointers.
-------------------------------------------------------------*/
void ES_DescriptorImp :: RemoveIPMPSystems (
	IPMPManager *pIPMP,
	IPMP_DescriptorPointer * const * descPointers,
	int nDescPointersLength,
	InlineImp *pInline
)
{
	for (int i = 0; i < nDescPointersLength; i++)
		pInline -> RemoveIPMP (
			descPointers [i] -> IPMP_DescriptorID,
			pIPMP
		);
}

/*-------------------------------------------------------------
Create an OCI decoder for an OCI stream
-------------------------------------------------------------*/
Decoder *ES_DescriptorImp :: CreateOCIDecoder (int version)
{
	SysCoder *pCoder;
	pCoder = new OCIDecoder (
		m_pExecutive,
		m_pExecutive -> GetTraceFile (),
		"OCI",
		m_pExecutive -> GetTraceFlags (),
		version
	);
	return new StreamParser (m_pExecutive, pCoder, NULL);
}

#ifdef MPGJ
//*** YT START (7)
/*-------------------------------------------------------------
Create an MPEGJ decoder for an MPEGJ stream
-------------------------------------------------------------*/
Decoder *ES_DescriptorImp :: CreateMPEGJDecoder ()
{
	Decoder* pDecoder = (Decoder *) LoadFilter (
			decConfigDescr -> streamType,
			decConfigDescr -> objectTypeIndication
		);
	pDecoder->decoderType = decConfigDescr -> streamType;
	return pDecoder;
}
//*** YT END (7)


/*-------------------------------------------------------------
Detach decoder from the elementary stream
-------------------------------------------------------------*/
//*** YT START (106)
// Detach decoder from ES
DecoderImp*
ES_DescriptorImp::DetachDecoder()
{
	// check decoder is already detached
	if(!m_pDecoder)
		return NULL;

	// paude input and output buffers
	((DecoderImp*)m_pDecoder)->Pause();
	Decoder* pDecoder = m_pDecoder;
	m_pDecoder = NULL;
	return (DecoderImp*)pDecoder;
}
//*** YT END (106)


/*-------------------------------------------------------------
Attach decoder from the elementary stream
-------------------------------------------------------------*/
//*** YT START (107)
// Attach decoder to ES
void ES_DescriptorImp::AttachDecoder(DecoderImp* pDecoder)
{
	// check decoder is already attached
	if(m_pDecoder)
		return;
	m_pDecoder = pDecoder;
	// change decoder assoicated with OD
	if(m_pOD->GetDecoder() != pDecoder)
		m_pOD->SetDecoder(m_pDecoder);
	// assume we are going to attach decoder between
	// MediaStreamImp already setup
	// also Decoder is already configured...

	// resume input and ouput buffers
	((DecoderImp*)m_pDecoder)->Resume();
}
//*** YT END (107)
#endif MPGJ
/*-------------------------------------------------------------
Send a Play command to the channel
-------------------------------------------------------------*/
void ES_DescriptorImp :: Play (double dSpeed)
{
	if (m_pChannel)
		GetAL () -> StartChannel (m_pChannel -> GetChannelID (), -1, -1, dSpeed);
}	

/*-------------------------------------------------------------
Propagate a Pause command to all streams
-------------------------------------------------------------*/
void ES_DescriptorImp :: Pause ()
{
	if (m_pChannel && m_pChannel -> IsOpen ())
		GetAL () -> PauseChannel (m_pChannel -> GetChannelID ());
}	

/*-------------------------------------------------------------
Propagate a Resume command to all streams
-------------------------------------------------------------*/
void ES_DescriptorImp :: Resume()
{
	if (m_pChannel && m_pChannel -> IsOpen ())
		GetAL () ->ResumeChannel (m_pChannel -> GetChannelID ());
}
	
/*-------------------------------------------------------------
Propagate a Mute command to all streams
-------------------------------------------------------------*/
void ES_DescriptorImp :: Mute ()
{
	if (m_pChannel && m_pChannel -> IsOpen ())
		GetAL () -> MuteChannel (m_pChannel -> GetChannelID ());
}

/*-------------------------------------------------------------
Propagate a SetSpeed command to all streams
-------------------------------------------------------------*/
void ES_DescriptorImp :: SetSpeed (double speed)
{
	if (m_pChannel && m_pChannel -> IsOpen ()) {
		//set clock speed
		GetClock()->SetSpeed(speed>0 ? speed : -speed);
		//send command
		char szCom[100];
		sprintf(szCom, "speed=%lf", speed);
		GetAL ()->SendCommand(szCom, m_pChannel -> GetChannelID ());
	}
}	

/*-------------------------------------------------------------
GoNoGo is called by the MR with a true if processing the stream 
may begin and a false if processing must stop.
@param go true indicates that processing on this elementary stream can proceed;
false indicates that processing must stop.
@return true if successful
-------------------------------------------------------------*/
bool ES_DescriptorImp :: GoNoGo (bool go)
{
	if (go != m_bGo)
	{	m_bGo = go;
		if (go)
		{	if (m_nStart && m_pChannel)
			{	m_pChannel -> ChannelReady ();
				m_pExecutive -> SetPendingChannels ();
			}
		}
		else if (m_nStart && m_pChannel)
			m_pChannel -> ChannelStop ();
	}
	return true;
}

/*-------------------------------------------------------------
Called by the MR to create a Filter in a control point.
@param filter the filter instance to be installed in the control point
@param controlPoint the position in the stream where the filter must be connected 
(0 is first filter in the chain, i.e. before the decoder)
@return true if successful
-------------------------------------------------------------*/
bool ES_DescriptorImp :: SetFilter (Filter *filter, int controlPoint)
{
	/*-------------------------------------------------------------
	Check that controlPoint is a valid value, and whether it is before or
	after the decoder
	-------------------------------------------------------------*/
	int i;
	bool bPostDecoding = false;
	if (m_pInputStream == NULL || controlPoint > m_filters.GetLength ())
		return false;
	for (i = 0; i < controlPoint; i++)
		if (m_filters [i] == m_pDecoder)
		{	bPostDecoding = true;
			break;
		}
	if (bPostDecoding && m_pOutputStream == NULL)
		return false;		// post decoding not applicable.

	/*-------------------------------------------------------------
	Create a MediaStream object and use it to connect the filter in the proper position
	-------------------------------------------------------------*/
	MediaStreamImp *pStream = new MediaStreamImp (
		m_pInputStream -> GetClockReference (),
		(bPostDecoding ? m_pOutputStream : m_pInputStream) -> GetSize ()
	);
	MediaStream *p;
	if (bPostDecoding)
	{	p = m_filters [controlPoint - 1] -> GetOutputStream ();
		filter -> SetInputStream (pStream);
		filter -> SetOutputStream (p);
		m_filters [controlPoint - 1] -> SetOutputStream (pStream);
	}
	else
	{	p = controlPoint == 0 ?
			m_pInputStream :
			m_filters [controlPoint - 1] -> GetOutputStream ();
		filter -> SetInputStream (p);
		filter -> SetOutputStream (pStream);
		m_filters [controlPoint] -> ReplaceInputStream (p, pStream);
	}

	/*-------------------------------------------------------------
	Insert in filter list
	-------------------------------------------------------------*/
	m_filters.InsertAt (controlPoint, filter);
	return true;
}

/*-------------------------------------------------------------
Called by the MR to remove a Filter from a control point
@param filter the filter to be removed
@return true if successful
-------------------------------------------------------------*/
bool ES_DescriptorImp :: ReleaseFilter (Filter *filter)
{
	/*-------------------------------------------------------------
	Lookup the filter in the list of connectd filter and determine whether this is 
	a pre or post decoder
	-------------------------------------------------------------*/
	int i;
	bool bPostDecoding = true;

	for (i = m_filters.GetLength (); --i >= 0;)
	{	if (m_filters [i] == m_pDecoder)
			bPostDecoding = false;
		else if (m_filters [i] == filter)
			break;
	}
	if (i < 0)
		return false;

	/*-------------------------------------------------------------
	Remove the filter from the list and delete its associated MediaStream object
	-------------------------------------------------------------*/
	MediaStream *pStream;
	if (bPostDecoding)
	{	pStream = filter ->GetInputStream ();
		m_filters [i - 1] -> SetOutputStream (filter ->GetOutputStream ());
	}
	else
	{	pStream = filter ->GetOutputStream ();
		m_filters [i + 1] -> ReplaceInputStream (pStream, filter ->GetInputStream ());
	}
	m_filters.DeleteFrom (i);
	delete pStream;
	return true;
}
