/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Presenter
FILE:       PresBase.cpp
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Implementation of PresenterBase
-------------------------------------------------------------------------*/
#include "defs.h"
#include "PresBase.h"

/*-----------------------------------------------------------------
Presenter constructor. Saves the arguments and creates the Presenter
thread.
-----------------------------------------------------------------*/
PresenterBase :: PresenterBase (Executive *pExecutive, HWND hWnd) :
	m_smpWaitToStart (1, 0),
	m_hWnd (hWnd)
{
	m_bDelete = m_bActive = FALSE;
	m_pExecutive = pExecutive;
}

/*-----------------------------------------------------------------
Presenter destructor. Wait till the thread finishes.
all linked objects.
-----------------------------------------------------------------*/
PresenterBase :: ~PresenterBase ()
{
	m_bDelete = TRUE;
	m_smpWaitToStart.V ();
	m_smpPresenterThread.P ();
}

/*-------------------------------------------------------------
Create the presenter thread. The thread will be blocked on a semaphore
until Start() is called
-------------------------------------------------------------*/
void PresenterBase :: Init ()
{
// PORTING to CC 5.3
  	if (ZThread<PresenterBase> (
//			PresenterThread,
			& PresenterBase :: PresenterThread,
			this,
#ifdef OSMOSE
			THREAD_PRIORITY_NORMAL
#else
			THREAD_PRIORITY_HIGHEST
#endif
			).Begin () == -1
	)
    	throw;
}

/*-----------------------------------------------------------------
Start() kicks of the PresenterBase thread
-----------------------------------------------------------------*/	
void PresenterBase :: Start ()
{
	m_bActive = TRUE;
	m_smpPresenterActive.P ();
	m_smpWaitToStart.V ();
}

/*-----------------------------------------------------------------
Terminates a session
-----------------------------------------------------------------*/
void PresenterBase :: Terminate ()
{
	m_bActive = FALSE;
	m_smpPresenterActive.P ();
	m_smpPresenterActive.V ();
}

/*-----------------------------------------------------------------
PresenterBaseThread() is the thread function of PresenteR
-----------------------------------------------------------------*/
void PresenterBase :: PresenterThread ()
{
	/*-----------------------------------------------------------------
	Mark the thread as running, then loop till object destructor performs
	-----------------------------------------------------------------*/
	ThreadSetup ();
	m_smpPresenterThread.P ();
	for (;;)
	{	
		/*-----------------------------------------------------------------
		Wait for Start command, then activate the BIFS decoder
		-----------------------------------------------------------------*/
		m_smpWaitToStart.P ();
		if (m_bDelete)
			break;

		while (m_bActive)
		{
			//ML - the 3D player doesn't use this rendering mechanism
#ifdef PLAYER3D
			Sleep(100);
#else
			Render ();
			m_pExecutive -> RequestChannels ();
#endif
			m_pExecutive -> RestartInlines();
		}
		m_smpPresenterActive.V ();
	}
	m_smpPresenterThread.V ();
}

void PresenterBase :: Reset() {} 
