/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

#if !defined(_PresBase_h)	// Sentry, use file only if it's not already included.
#define _PresBase_h

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Presenter
FILE:       PresBase.h
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Definition of PresenterBase, the base class for Presenters.
-------------------------------------------------------------------------*/
#include "Im1defs.h"
#include "ztl.h"
#include "Executiv.h"

class Capabilities;
class UserEventHandler;
class Root;
class FlexTimeMgr;

class PresenterBase
{
	ZSemaphore m_smpWaitToStart, m_smpPresenterThread, m_smpPresenterActive;
	Executive *m_pExecutive;
	BOOL m_bDelete;
	HWND m_hWnd;
	BOOL m_bActive;



private:
	void PresenterThread ();
	virtual void ThreadSetup () {};

public:
	PresenterBase (Executive *pExecutive, HWND hWnd);
	virtual ~PresenterBase ();
	Root *GetScene () {return m_pExecutive -> GetScene ();}
	Executive *GetExecutive () {return m_pExecutive;}
	virtual void GetCapabilities (Capabilities *pPresentationCapabilities) = 0;
	virtual void Start ();
	virtual void Init ();
	virtual void Terminate ();
	virtual void OnUserEvent (USER_EVENT event, int x, int y, UINT uControlKeysStatus) = 0;
	virtual DWORD GetTime () = 0;
	virtual int GetIterationInterval () = 0;
	virtual void Render () = 0;
	virtual void Paint (void *) {}
	virtual void RegisterEventHandler  (UserEventHandler *) {}
	virtual void UnregisterEventHandler  (UserEventHandler *) {}
	virtual void SetTargetFrameRate (float) {}
	virtual float GetTargetFrameRate () {return 0;}
	virtual float GetActualFrameRate () {return 0;}
	virtual FlexTimeMgr* CreateFlexTimeMgr () {return NULL;}
	virtual void DeleteFlexTimeMgr (FlexTimeMgr *) {}
	/*JLF: notify the presenter something changed. It can be passed a pointer to the node which
	changed or NULL for any other event*/
	virtual void InvalidateScene(MediaObject *) {}
	BOOL IsActive() { return m_bActive; }
	/*called when main scene is restarted due to clock restart*/
	virtual void Reset();

protected:
	HWND GetWindow () {return m_hWnd;}	
};

/*-------------------------------------------------------------
Context structure for the Render function
-------------------------------------------------------------*/
class PresenterContext
{	
public:
	PresenterContext () : m_bActive (true), m_dwOffset (0), m_fSpeed (1), m_dwTime (0) {}
	void Transform (DWORD dwTime, DWORD dwOffset, float fSpeed, bool bActive);
	void SetTime (DWORD dwTime)
	{
		m_dwTime = dwTime;
		m_dwOffset = 0;
		m_fSpeed = 1;
	}
	DWORD GetTime () const {return m_dwTime;}
	bool IsActive () const {return m_bActive;}
	float GetSpeed () const {return m_fSpeed;}

private:
	bool m_bActive;
	DWORD m_dwOffset, m_dwTime;
	float m_fSpeed;
};

#endif                                      // _PresBase_h sentry.
