/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  BIFS Decoder
FILE:       Proto.cpp
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Implementation of Proto
-------------------------------------------------------------------------*/

#include "defs.h"
#include "BifsDec.h"
#include "Proto.h"
#include "Route.h"
#include "Inline.h"
#include "Executiv.h"
/*-------------------------------------------------------------
Parse a Proto declaration
-------------------------------------------------------------*/
void Proto :: ParseProtoDeclaration (BIFSDecoder *pCoder)
{
	/*-------------------------------------------------------------
	PROTOinterfaceDefinition
	-------------------------------------------------------------*/
	SetID (pCoder -> ParseInt (pCoder -> GetProtoIDbits (), "id"));
	if (GetInline () ->IsCodeNames ())
	{	pCoder ->ParseString (m_protoName);
		TRACEF ("PROTO %s [", (LPCSTR) m_protoName);
	}
	else
		TRACEF ("PROTO %d [", GetID ());
	TRACENEWLINE;
	TRACEINDENT (1);
#ifdef TRACE_ENCODING
	int i = 0;
#endif
 	while (pCoder -> ParseInt (1, "moreFields"))
	{
 		int eventType = pCoder -> ParseInt (2, "eventType");
 		bool isSF = !pCoder -> ParseInt (1, "isMF");
 		int fieldType = pCoder -> ParseInt (5, "fieldType");
		ZString fieldName;
		if (GetInline () ->IsCodeNames ())
			pCoder ->ParseString (fieldName);
		else
		{
			#ifdef TRACE_ENCODING
			char szFieldName [15];
			sprintf (szFieldName, "Field%d", i++);
			fieldName = szFieldName;
			#endif				
		}
		NodeField *pField =
			CreateInterfaceField (pCoder, eventType, fieldType, isSF, fieldName);
		TRACEF ("\t%s", (LPCSTR) fieldName);
		if (pField && (eventType == ET_FIELD || eventType == ET_EXPOSED_FIELD))
		{	TRACEF ("\t", 0);
			pField -> Parse (pCoder, this, NULL);
		}
		TRACENEWLINE;
	}
	TRACEINDENT (-1);
	TRACEF ("]", 0);
	TRACENEWLINE;
	m_bifsBounds.CalculateBounds ();
	m_bFullyDefined = true;

	/*-------------------------------------------------------------
	Let the BifsParser parse the PROTOCode and the PROTOcodingTable .
	First see if this is an EXTERNPROTO
	-------------------------------------------------------------*/
	if (pCoder -> ParseInt (1, "isExtern"))
	{	/*-------------------------------------------------------------
		Put a special type of Inline node in the list of code nodes, parse the url
		field into it, and open the externproto stream
		-------------------------------------------------------------*/
		InlineImp *pInline = new InlineImp (true);
		pInline -> SetRoot (GetRoot ());
		pInline -> SetExecutive (GetExecutive ());
		pInline ->SetInline (GetInline ());
		pInline -> ParseUrl (pCoder);
		m_nExternODID = pInline ->GetObjectDescriptorID ();
		if (GetRoot () -> IsExternLibOpen (m_nExternODID))
			delete pInline;
		else
		{	m_protoNodes.Append (pInline, this, false);
			pInline -> OpenStream ();
		}
	}
	else
		pCoder -> ParseProtoCode (this);
	ParseProtoCodingTable (pCoder);
}

/*-------------------------------------------------------------
Parse the nodes making up the proto code
-------------------------------------------------------------*/
void Proto :: ParseNodes (BIFSDecoder *pCoder)
{
	//store current QP (for nested protos)
	MediaObject *qpOld = pCoder->GetCurrentQP();

	SetRoot (this);
	do
		m_protoNodes.AppendElement (pCoder, this, NULL);
	while (pCoder -> ParseInt (1, "moreNodes"));

	//restore original QP 
	pCoder->SetCurrentQP(qpOld);

}


/*-------------------------------------------------------------
Parse ParseProtoCodingTable
AC20011102 : modification
	 -> min and max
	 -> m_dynID and calculateBounds
Question to Zvi : is there a suggested method to report errors here ?
-------------------------------------------------------------*/
void Proto :: ParseProtoCodingTable (BIFSDecoder *pCoder)
{
	/*-------------------------------------------------------------
	Parse InterfaceCodingMask
	-------------------------------------------------------------*/
 	bool useQuant = pCoder	-> ParseInt (1, "useQuant");
 	bool useAnim = pCoder	-> ParseInt (1, "useAnim");

	/*-------------------------------------------------------------
	Parse InterfaceCodingParameters
	-------------------------------------------------------------*/
 	for (int i = 0; i < m_fields.GetLength (); i++)
	{
		if (useQuant)
		{
 			if (m_fieldTable [i].m_defID >= 0)
			{	int quantCategory = pCoder -> ParseInt (4, "quantCategory");
 				m_fieldTable [i].m_nQuantizationType = quantCategory;
 				if (quantCategory == 13)
 					m_fieldTable [i].m_quantParams =
						pCoder -> ParseInt (5, "nbBits");
 				if (pCoder -> ParseInt (1, "hasMinMax"))
				{	
					/*-------------------------------------------------------------
					Parse Min and Max
					-------------------------------------------------------------*/
					BifsFieldTable &rEntry=m_fieldTable[i];
					int fieldType=m_fields [i] -> GetType ();
					
					switch (fieldType) 
					{
					case SFTIME : case MFTIME : 
					case SFFLOAT: case MFFLOAT: 
					case SFINT32: case MFINT32: rEntry.m_nDimension = 1;break;
					case SFVEC3F: case MFVEC3F: case SFROTATION: case MFROTATION:
					case SFCOLOR: case MFCOLOR: rEntry.m_nDimension = 3; break;		//Note : check for rotation.
					case SFVEC2F: case MFVEC2F: rEntry.m_nDimension = 2; break;
					default:
						pCoder->GetExecutive()->Error(SCENE_PARSING_ERROR); //report error
 					}
					/*-------------------------------------------------------------
					minFieldValue.
					-------------------------------------------------------------*/
                    // PORTING to CC 5.3
                    int j = 0;
 //					for (int j=0; j<rEntry.m_nDimension;j++)
 					for ( j=0; j<rEntry.m_nDimension;j++)
					{						
							 if (fieldType==SFINT32 ||	fieldType==MFINT32)
						{
								rEntry.m_fMin[j]=pCoder->ParseInt(32,"minFieldValue");
						}
						else if (fieldType==SFTIME  ||	fieldType==MFTIME)
						{
								rEntry.m_fMin[j]=pCoder ->ParseDouble("minFieldValue");
						}
						else
						{
								rEntry.m_fMin[j] =pCoder ->ParseFloat("minFieldValue");								
						}
					}
					/*-------------------------------------------------------------
					maxFieldValue.
					-------------------------------------------------------------*/
					for (    j=0; j<rEntry.m_nDimension;j++)
					{						
							 if (fieldType==SFINT32 ||	fieldType==MFINT32)
						{
								rEntry.m_fMax[j]=pCoder->ParseInt(32,"maxFieldValue");
						}
						else if (fieldType==SFTIME  ||	fieldType==MFTIME)
						{
								rEntry.m_fMax[j] =pCoder ->ParseDouble("maxFieldValue");
						}
						else
						{
								rEntry.m_fMax[j] =pCoder ->ParseFloat("maxFieldValue");								
						}
					}
 				} //end hasMinMax.
 			}
		}
 		if (useAnim && m_fieldTable [i].m_inID >= 0 &&
			pCoder -> ParseInt (1, "isDyn")
		)
		{
			m_fieldTable [i].m_AnimationType = pCoder -> ParseInt (4, "animCategory");
			m_fieldTable [i].m_dynID=m_bifsBounds.m_nTotalDyns++;
		}
 	}//end main loop.
	m_bifsBounds.CalculateBounds();
}

/*-------------------------------------------------------------
Instantiate a Proto and returns a pointer to it.
This pointer will be inserted into the scene tree. The other nodes will be added
as children to the root Group node
-------------------------------------------------------------*/
MediaObject *Proto :: Instantiate (Root *pScene)
{
	int i;

	/*-------------------------------------------------------------
	Clone the Proto with all its fields
	-------------------------------------------------------------*/
	Proto *pInstance = new Proto;
	pInstance -> SetExecutive (pScene -> GetExecutive ());
	pInstance -> SetRoot (pInstance);
	pInstance ->Configure(FALSE, 0, 0, GetNodeIDbits(), GetRouteIDbits(), GetProtoIDbits());

	pInstance ->SetInline (pScene ->GetInline ());
	pInstance ->m_nDeclarationID = GetID ();
	pInstance ->m_protoName = m_protoName;

	for (i = 0; i < m_fields.GetLength (); i++)
	{	pInstance ->m_fields += m_fields [i] -> Clone (NULL, pInstance);
		pInstance ->m_fieldTable += m_fieldTable [i];
	}
	pInstance -> m_bifsBounds = m_bifsBounds;

	/*-------------------------------------------------------------
	No need to clone the code of an EXTERNPROTO
	-------------------------------------------------------------*/
	if (GetODID () >= 0)
		return pInstance;

	/*-------------------------------------------------------------
	set the current instanciation of this proto
	-------------------------------------------------------------*/
	SetCloneObject (pInstance);

	/*-------------------------------------------------------------
	Clone all the nodes. The first is the main proto node, the rest
	are appended to the scene root
	-------------------------------------------------------------*/
	for (i = 0; i < m_protoNodes.GetLength (); i++)
	{	MediaObject *pNode = m_protoNodes.GetObject (i) -> Clone (pInstance);
		pInstance -> m_protoNodes.Append (pNode, this, false);
		if (i == 0)
			pInstance -> m_pFirstNode = pNode;
		else
			pScene -> AddChild (pNode);
	}

	/*-------------------------------------------------------------
	Clone all the routes
	-------------------------------------------------------------*/
	GetExecutive () ->CloneRoutes (this, pInstance);

	return pInstance;
}

/*-------------------------------------------------------------
When a Proto is removed, delete all its fields
-------------------------------------------------------------*/
void Proto :: DeleteObject ()
{
	RemoveFields ();
	Root :: DeleteObject ();
}

/*-------------------------------------------------------------
Compose the name of a PROTO instance
-------------------------------------------------------------*/
#ifdef TRACE_ENCODING
char *Proto :: GetName ()
{
	if (m_protoName.GetLength () > 0)
		return (char *) (LPCSTR) m_protoName;

	static char szName [15];
	sprintf (szName, "PROTO %d", m_nDeclarationID);
	return szName;
}
#endif
