/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

#if !defined(_Proto_h)	// Sentry, use file only if it's not already included.
#define _Proto_h

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  BIFS Decoder
FILE:       Proto.h
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Declarations of Proto
-------------------------------------------------------------------------*/
#include "Root.h"
#include "bifs.h"

typedef enum {ET_FIELD, ET_EXPOSED_FIELD, ET_EVENT_IN, ET_EVENT_OUT} EventType;
typedef enum
{	FT_BOOL, FT_FLOAT, FT_TIME, FT_INT32, FT_STRING, FT_VEC3F,
	FT_VEC2F, FT_COLOR, FT_ROTATION, FT_IMAGE, FT_NODE
} FieldType;

class BIFSDecoder;
class SysCoder;

class Proto : public Root
{
	/*-------------------------------------------------------------
	methods used by the decoder
	-------------------------------------------------------------*/
#ifdef BIFSDEC
private:
	void ParseProtoCodingTable (BIFSDecoder *pBifs);
	virtual MediaObject *GetRenderingNode () {return m_pFirstNode;}
	void GenerateProtoCode (BifsGenerator *pBifs);

#ifdef TRACE_ENCODING
public:
	virtual char *GetName ();
#endif

public:
	virtual void DeleteObject ();
	MediaObject * Instantiate (Root *pScene);
	void ParseNodes (BIFSDecoder *pCoder);
	void ParseProtoDeclaration (BIFSDecoder *pCoder);
	MediaObject *GetFirstNode () const {return m_pFirstNode;}
	int GetODID () const {return m_nExternODID;}
	const ZString &GetProtoName () const {return m_protoName;}
	void SetODID (int nODID) {m_nExternODID = nODID;}
	void GenerateProtoDefinition (BifsGenerator *pCoder);
#endif

	/*-------------------------------------------------------------
	methods used by the encoder
	-------------------------------------------------------------*/
#ifdef ENCODER
public:
	bool EncodeProtoDefinition (BIFSEncoder *pCoder);

private:
	void EncodeProtoCode (BIFSEncoder *pCoder);
	bool readInterfaceCodingParameters(	int eventType,
										int fieldType, 
										int &quantCategory, 
										int &nbBits, 
										int & hasBounds, 
										float* pMin, 
										float* pMax, 
										int &animCategory);			//AC20011102
	void encodeInterfaceCodingParameters (BIFSEncoder *pCoder,
										  bool Quant, 
										  bool Anim);				//AC20011102
#endif

	/*-------------------------------------------------------------
	methods used by both decoder and encoder
	-------------------------------------------------------------*/
private:
	virtual int GetChildrenOffset () const
	{
		return
			(size_t) ((NodeField *) & ((Proto *)0) -> m_protoNodes) - 
			(size_t) (MediaObject *)((Proto *)0);
	}
	virtual int GetFieldCount () const {return m_fields.GetLength ();}
	virtual int GetNodeCode () const {return -1 - !m_bFullyDefined;}
	virtual void RemoveFields ();
	NodeField *CreateInterfaceField (
		SysCoder *pCoder,
		int eventType,
		int fieldType,
		bool isSF,
		LPCSTR szName = NULL
	);


public:
	Proto () :
		m_bifsBounds (NULL, 0), m_pFirstNode (NULL), m_bFullyDefined (false), m_nExternODID (-1)
	{}
	~Proto ();
	virtual const BifsBounds *GetBifsBounds () const {return &m_bifsBounds;}
	virtual const BifsFieldTable *GetFieldTable () const {return m_fieldTable;}
	NodeField *GetField (int index) {return m_fields [index];}
	virtual int GetProtoID () const {return m_nDeclarationID;}

private:
//AC
protected:
	ZArray<NodeField *> m_fields;
	ZArray<ZString> m_fieldNames;
	ZArray<BifsFieldTable> m_fieldTable;
	BifsBounds m_bifsBounds;
private:
//end AC
	MFNodeType<SFWorldNode> m_protoNodes;
	MediaObject *m_pFirstNode;
	int m_nDeclarationID, m_nExternODID;
	bool m_bFullyDefined;
	ZString m_protoName;
	MF<Url> m_externUrl;
};
	
#endif                                      // _Proto_h sentry.
