/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  BIFS Decoder/ Compositor
FILE:       Quantize.cpp
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
            Mikael Bourges-Sevenier (France Telecom CNET)
OVERVIEW:   Implementation of Quantizer
-------------------------------------------------------------------------*/
//$Log: Quantize.cpp,v $
//Revision 1.1.1.2  2006/04/14 14:11:31  hill
//no message
//
//Revision 1.1.1.1  2003/12/24 00:42:16  mikeseven
//Initial release of the CVS repository for MPEG-4 Systems code
//after the code was corrupted due to reorganization and human 
//errors.
//
//Revision 1.2  2002/12/02 10:23:41  jean_lf
//Bug fix for int quant
//
//Revision 1.1.1.1  2002/07/18 01:00:45  mikeseven
//Initial check in of July 17, 2002 version
//
//
//Revision 1.x 2000/10/12  Pascal Belin
//QP bug fix
//
//Revision 1.2.2.1  2000/03/10 15:59:32  Alex Cotarmanac'h
//deuxime envoi
//
//Revision 1.2  2000/03/03 18:41:21  Alex Cotarmanac'h
//Version 1 -- Alexandre Cotarmanac_h
//
//Modified 2001/05/08 Boris Felts boris.felts@polytechnique.org


#include "defs.h"
#include <float.h>
#include "Quantize.h"
#include "BifsDec.h"
#include "ztl.h"

// MBS
#define QPVAL(type, id) ((type) *(SF<type>*) m_pQP->GetField(id))
// end MBS

/*-----------------------------------------------------------------
Parse an int from the BIFS stream
-----------------------------------------------------------------*/
int Quantizer :: ParseInt (BIFSDecoder *pBifs)
{
  // MBS
  switch(m_nQuantizationCode) {
  case Q_DRAWORDER: {
    float mini=max(m_pMin[0],QPVAL(Vec3f,DOMIN).x);
//    float maxi=min(m_pMax[0],QPVAL(Vec3f,DOMAX).x);
    int nBits= QPVAL(int,DONB);
    return pBifs -> ParseInt (nBits, "quantized value") + mini; // PB  logout change
    break;
  }
//#ifdef PREDICTIVE	//AC 20000227 //BF
  case Q_COORDIDX:
	{
		int res=pBifs->ParseInt(m_nBits,"valueQP")-1;  // PB QP14 fix: 0 must stand for the minimum value, like in QP13
		// if (res == ((1<<m_nBits)-1) ) res=-1;
		return res;
		break;
	}
//#endif	//AC end  //BF
  case Q_NONE:
	return pBifs -> ParseInt (32, "not quantized value");

  default: 
	  {
    // end MBS
	/*-----------------------------------------------------------------
	Determine offset and number of bits
	-----------------------------------------------------------------*/
	int nOffset = 0, nBits = 32;
	if (m_nDimension == 1 &&
		m_pMin [0] != FLT_MIN && m_pMax [0] != FLT_MAX
	)
	{	int min = m_pMin [0];
		int max = m_pMax [0];
		nBits = BitFieldWidth (max - min + 1);
		nOffset = min;
	}
	return pBifs -> ParseInt (nBits, "quantized value") + nOffset;   // PB logout change
	// MBS
  }
  }
  // end MBS
}
