/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

// -*- c++ -*-

#if !defined(_Route_h)	// Sentry, use file only if it's not already included.
#define _Route_h

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  BIFS Decoder/ Compositor
FILE:       Route.h
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Declarations of Route
-------------------------------------------------------------------------*/

class MediaObject;
class NodeField;

class Route
{
	int m_nRouteID;
	ZString m_name;
	NodeField *m_pDepartureField;
	MediaObject *m_pArrivalNode, *m_pDepartureNode;
	int m_nDepartureField, m_nArrivalField;
	Route *m_pNext, *m_pNextArrival;
	static int m_nGlobalCounter;
#ifdef MU
        bool m_bGenerateRequest;
#endif
	int m_nCurrentCounter, m_nActivated;
	bool m_bFirst;

public:
	Route (
		int routeID,
		NodeField *pDepartureField,
		int nDepartureIndex,
		MediaObject *pDepartureNode,
		LPCSTR name = NULL
	);
	void Remove ();
	Route * Clone ();
	void Bind (MediaObject *pArrivalNode, int nArrivalField);
	void Activate ();
	void Link (Route *&rpListHead);
	void Unlink (Route *&rpListHead);
	void LinkArrival (Route *&rpListHead);
	void UnlinkArrival (Route *&rpListHead);
	Route *GetNext () {return m_pNext;}
	int GetID () const {return m_nRouteID;}
	LPCSTR GetName () const;
#ifdef MPGJ		// SCENE API Integration
	int DoesLink (MediaObject *fromNode, int fromField, MediaObject *toNode, int toField)
	{
	    return (fromNode == m_pDepartureNode && fromField == m_nDepartureField && toNode == m_pArrivalNode && toField == m_nArrivalField);
	}
#endif

	void MarkForActivation (bool bIsFirst);
	void Deactivate ();
	bool MarkedForActivation (int nLevel) const {return m_nActivated == nLevel;}
	void GenerateRoute (BifsGenerator *pBifs);
	void GenerateRouteInsertion (BifsGenerator *pBifs);
	void GenerateRouteReplace (BifsGenerator *pBifs);

private:
	void GenerateRouteDetails (BifsGenerator *pBifs);

#ifdef MU
        bool CheckForSharing ();
        void GenerateRequest();
#endif
};

#endif                                      // _Route_h sentry.
