/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  BIFS/OD Decoder
FILE:       StrParser.cpp
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Implementation of StreamParser - a class for decoding 
			Systems (OD, BIFS, OCI) streams.
-------------------------------------------------------------------------*/

#include "defs.h"
#include "StrParser.h"
#include "SysCoder.h"
#include "Stream.h"
#include "Executiv.h"
#include "Inline.h"
#include "Except.h"


/*-------------------------------------------------------------
StreamParser constructor
-------------------------------------------------------------*/
StreamParser :: StreamParser (Executive *pExecutive, SysCoder *pCoder, InlineImp *pInline) :
	m_pExecutive (pExecutive),
	m_pCoder (pCoder),
	m_pInline (pInline)
{}

/*-------------------------------------------------------------
StreamParser destructor deletes the coder object
-------------------------------------------------------------*/
StreamParser :: ~StreamParser ()
{
	if (m_pInline)
		m_pInline ->RemoveDecoder (this);
	delete m_pCoder;
}

/*-------------------------------------------------------------
Parse update commands from the stream and execute them
-------------------------------------------------------------*/
void StreamParser :: Run ()
{
	// SOLARIS PORTING - made bRecurrent
	//BOOL bRecurrent, bEOU;
	BOOL bEOU;
	int bRecurrent = 0;
	LPBYTE pData;
	int nSize;

	/*-----------------------------------------------------------------
	Loop till end of stream
	-----------------------------------------------------------------*/
	while (
		IsActive () &&
		(pData = (LPBYTE) GetInputStream () -> Fetch(
					nSize,
					m_dwLastTS,
					bEOU,
					bRecurrent,
					FALSE,
					TRUE
				)
		) != END_OF_STREAM
	) {
		if (IsActive () && pData != NULL) {	

			/*-------------------------------------------------------------
			Lock the scene before parsing the frame and processing it.
			If LockScene returns true the scene is not accessible. The
			scene was unlocked in this case only to release the thread
			in case the decoder is no longer active
			-------------------------------------------------------------*/
			while (m_pExecutive -> ReserveScene ()) {
				if (!IsActive ()) 
					break;
				m_pExecutive -> UnlockScene ();
				Sleep (3);
			}

			if (IsActive ()) {
				m_pCoder-> ParseFrame(pData, nSize, m_dwLastTS);
			}
			m_pExecutive -> UnlockScene ();
		}
		if (pData) GetInputStream () -> Release (TRUE);
	}
	TRACE("sys dec exit\n");
}

/*-------------------------------------------------------------
Configure the parser
-------------------------------------------------------------*/
BOOL StreamParser :: Setup (
	const BYTE *pSpecificInfo,
	int nSpecificInfoLength,
	int
)
{
	return m_pCoder -> Configure (pSpecificInfo, nSpecificInfoLength);
}

/*-----------------------------------------------------------------
Return some decoder attributes
-----------------------------------------------------------------*/
BOOL StreamParser :: SetFormat (
	Capabilities &,
	DecoderParams &decoderParams
)
{
	decoderParams.m_nStreamNumber = 0;
	decoderParams.m_nOptimalAUSize = 256;
	decoderParams.m_nDBSize = MPEG4_DEFAULT_BIFS_BUFFER_SIZE;
	decoderParams.m_nCBSize = 0;
	return TRUE;
}

/*-------------------------------------------------------------
Intercept SetInputStream in order to store the associated ClockReference object in the
associated decoder object
-------------------------------------------------------------*/
void StreamParser :: SetInputStream (int nStream, MediaStream *pStream)
{
	m_pCoder -> SetClock (((MediaStreamImp*) pStream) ->GetClockReference ());
	DecoderImp ::SetInputStream (nStream, pStream);
};
