/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright ?1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

#if !defined(_StreamCo_h)	// Sentry, use file only if it's not already included.
#define _StreamCo_h

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  BIFS Decoder/ Compositor / Presenter
FILE:       MediaObj.h
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Declarations of StreamConsumer
-------------------------------------------------------------------------*/
#include "MediaObj.h"
#include "NodeFld.h"
#include "Decoder.h"
#include "custom.h"
#include "ztl.h"

class MediaStreamImp;
class DataChannel;
class Service;
class ALManager;
class ObjectDescriptorImp;
class ES_DescriptorImp;
class InlineImp;
class DecoderFactoryBase;
class Capabilities;
class IPMPManager;
class DecoderConfigDescriptor;
class QoS_Descriptor;
class MediaControlImp;

class StreamConsumer : public MediaObject
{
protected:
	MFUrl url;		// changed to MF in FDIS

#ifdef PLAYER
private:
	MediaFormat m_fmtStreamFormat;
	int m_nODid, m_nURLIndex;
	InlineImp *m_pSource;
	MediaStreamImp *m_pStream;
	bool m_bStreamOpen;
	ObjectDescriptorImp *m_pOD;
	bool m_bActive;
	ZMutex m_mtxLock;
	DWORD m_dwMediaTime;
	double m_dStart, m_dDuration;
	bool m_bFullMediaControled;

protected:
	struct Segment
	{	ZString m_segmentName;
		double m_dStart, m_dDuration;
		Segment () : m_dStart (-1), m_dDuration (-1) {}
	};
	ZArray<Segment> m_segments;
	bool m_bNeedsSegmentList;

protected:
	virtual void RouteEvent (NodeField *pField, int nFieldIndex);
	virtual void RemoveStream ();
	virtual BOOL Parse (BIFSDecoder *pBifs);
	virtual MediaObject *Clone (Root *pScene);
	double GetSegmentStart () const {return m_dStart;}
	double GetSegmentDuration () const {return m_dDuration;}
	DWORD GetMediaPosition (long lStart, long lLength);
	void Register ();
	bool FindSegment(double dTS, double &rdSegmentStart, double &rdSegmentEnd, LPCSTR &rszSegment);

	/*get current media time , used for MediaSensor. returns TRUE if end of stream. If not running mediaTime 
	is 0 - Overriden by inline*/
	virtual bool GetMediaTime(DWORD &mediaTime);

public:
	bool NextURL ();
	MFUrl& GetURL() { return url; }		// SOWOO 20030109
	StreamConsumer ();
	virtual void DeleteObject ();
	virtual Decoder *CreateDecoder (
		DecoderConfigDescriptor *,
		QoS_Descriptor *,
		Decoder *pCurrentDecoder
	) {return pCurrentDecoder;}
	ObjectDescriptorImp *GetOD () const {return m_pOD;}
	void SetStreamFormat (MediaFormat fmtStreamFormat)
	{
		m_fmtStreamFormat = fmtStreamFormat;
	}
	virtual void GetCapabilities (Capabilities *pCaps);
	virtual void SetStream (ObjectDescriptorImp *pOD);
	virtual void ODRemoved ();
	void AddESD (ES_DescriptorImp *pESD);
	void RemoveESD (ES_DescriptorImp *pESD);
	void Reopen ();
	void OnConnect (ObjectDescriptorImp *pOD);
#ifndef MPGJ
	//*** YT START (115)
	// make it public

	MediaStreamImp *GetStream () { return m_pStream;}

	//*** YT END (115)
#endif !MPGJ
	MediaFormat GetStreamFormat () {return m_fmtStreamFormat;}
	int GetObjectDescriptorID () {return m_nODid;}

	void UpdateField (const BifsFieldTable *pEntry, BOOL bHasChanged);

	void OpenStream ();
	void CloseStream ();

	bool IsStreamOpen () {return m_bStreamOpen;}
	Decoder *GetDecoder () const;
	void SetInline (InlineImp *pSource) {m_pSource = pSource;}
	InlineImp *GetInline () const {return m_pSource;}
	void WriteToStream (LPBYTE pData, int nLength);
	void SetOD (ObjectDescriptorImp *pOD) {m_pOD = pOD;}
	void SetSegment (LPCSTR szName, double dStart, double dDuration);
	void ResetURL ();
	virtual LPVOID Fetch (
		int &rnSize,
		DWORD &rdwTime,
		BOOL &rbEOU,
		int &rnStatus,
		BOOL bSkip = TRUE,
		BOOL bWait = FALSE,
		long lDelta = 0,
		int nFlag = 0
	);
	virtual void Release (bool bDiscard = TRUE);
	/*stop with a check for clock dependencies*/
	virtual void Stop();
	/*start with a check for clock dependencies*/
	virtual void Start();

	/*sets the object speed if no media control associated, otherwise speed is handled by control*/
	void SetObjectSpeed(double speed);

	/*returns media control associated with clock - this can be a mediaControl NOT controling this object*/
	MediaControlImp *GetMediaControl();
	/*returns media control controling this object or NULL*/
	MediaControlImp *GetObjectMediaControl();

	CUSTOMIZE_STREAMCONSUMER_DECLARATION
#endif	// BIFSENC

public:
#ifdef MPGJ
	//*** YT START (115)
	// from above
	MediaStreamImp *GetStream () { return m_pStream;}
	//*** YT END (115)

	//*** YT START (116)
	void SetMediaStream(MediaStreamImp* pStream) {m_pStream = pStream;}
	//*** YT END (116)
#endif MPGJ
};

#endif                                      // _StreamCo_h sentry.
