/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright ?1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

#if !defined(_SysCoder_h)	// Sentry, use file only if it's not already included.
#define _SysCoder_h

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  BIFS Decoder
FILE:       SysCoder.h
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Declarations of SysCoder - the base class for encoding/decoding
			of Systems (OD, BIFS etc.) streams.
-------------------------------------------------------------------------*/

//Modified 2001/05/08 Boris Felts boris.felts@polytechnique.org

#include "ztl.h"

/*-------------------------------------------------------------
SysCoder flags
-------------------------------------------------------------*/
typedef enum
{	TRACE_NONE = 0,
	TRACE_PARSE = 1,
	TRACE_VERBOSE = 2,
	TRACE_HEADERS = 4,
	CODENAMES = 8,
	PREDICTIVE_FLAG = 16,
	FORMAT_XMT = 32,
	TRACE_CONFORMANT = 64, //BF Conformance
	STRING_FROM_FILE = 128	// SOWOO 20030109
} CODING_FLAGS;

class ClockReference;

class SysCoder
{
private:
	virtual void WriteByte ();
	virtual void EndOfUnit () {}

protected:
	virtual int GetBit ();
	void WriteBit (BOOL bit);
	void ReplaceOutputFile (FILE *fNewOutputFile);
	int GetVersion () const {return m_nVersion;}

public:
	SysCoder (FILE *fList, LPCSTR szListTitle, int flags, int version);
	SysCoder (FILE *fOutput, FILE *fList, LPCSTR szListTitle, int flags, int version, FILE *fLogs = NULL);
	virtual ~SysCoder () {}
	void Concatenate (SysCoder *pSource);
	virtual bool Configure (const BYTE *, int) {return true;}
	virtual void ParseFrame (LPCBYTE, int, DWORD) {}
	void Reset ();
	void Reset (LPCBYTE pData, int nLength);
	int Align ();
	int GetByte ();
	int ParseInt (int nBits, LPCSTR szName = NULL);
	ZLargeInt<128> ParseLongInt (int nBits, LPCSTR szName = NULL);
	float ParseFloat (LPCSTR szName);
	double ParseDouble (LPCSTR szName);
	void ParseSFString (ZString &string);
	void SkipBytes (int nLength);
	void EncodeInt (int nValue, int nBits, LPCSTR szDesc, LPCSTR szComment = NULL);
	void EncodeFloat (float fValue, LPCSTR szDesc);
	void EncodeDouble (double dValue, LPCSTR szDesc);
	void EncodeSFString (const ZString &string);
	void Write (bool bShort = false);
	virtual void Flush (DWORD dwTS = 0);
	virtual void Error (int) {}
	FILE *GetListFile () {return m_fList;}
	int GetFlags () {return m_flags;}
	void SetFlag (CODING_FLAGS flag) {m_flags |= flag;}
	void SetFlags (int flags) { m_flags = flags; }	// SOWOO 20030109
	int GetDataLength () const {return m_buffer.GetLength ();}
	LPCBYTE GetData () const {return m_buffer;}
	int GetOffset () const {return GetDataLength () * 8 + m_nLastBit;}
	void SetTraceFile (FILE *fList, LPCSTR szListTitle, int flags)
	{
		m_fList = fList;
		m_szType = szListTitle;
		m_flags = flags;
	}
	void SetClock (ClockReference *pClock) {m_pClock = pClock;}
	ClockReference *GetClock () const {return m_pClock;}

private:
	bool m_bDecode;
	LPCSTR m_szType;
	const BYTE *m_pData;
	int m_flags, m_nVersion; //BF Conformance
	ClockReference *m_pClock;

protected:
	ZString m_buffer;
	int m_nSize, m_nLastBit, m_nLastByte, m_nTotalBit; //BF Conformance
	FILE *m_fOutput, *m_fList, *m_fLogs;
	static int m_nIndent;
	BOOL m_bNewLine;

#ifdef TRACE_ENCODING
public:
	void Tracef (LPCSTR szFormat, ...);
	void TraceIndent (int nDirection);
	void TraceNewLine ();
#endif


	/*JLF 03/03. added RAP indicator, by default the first AU is RAP and the coder doesn't signal RAPs*/
#ifdef BIFSENC
public:
	BOOL m_bSignalRAP;
protected:
	BOOL m_isRAP;
#endif

};

#ifdef TRACE_ENCODING
#define TRACEF(format, text) pCoder -> Tracef (format, text)
#define TRACEF2(format, x, y) pCoder -> Tracef (format, x, y)
#define TRACEF3(format, x, y, z) pCoder -> Tracef (format, x, y, z)
#define TRACEF4(format, x, y, z, w) pCoder -> Tracef (format, x, y, z, w)
#define TRACEINDENT(direction) pCoder -> TraceIndent (direction)
#define TRACENEWLINE pCoder -> TraceNewLine ()
#else
#define TRACEF(format, text) {}
#define TRACEF2(format, x, y)
#define TRACEF3(format, x, y, z)
#define TRACEF4(format, x, y, z, w)
#define TRACEINDENT(direction)
// SOLARIS PORTING - commented pCoder below
//#define TRACENEWLINE pCoder
#define TRACENEWLINE // pCoder 
#endif	// TRACE_ENCODING

#endif                                      // _SysCoder_h sentry.
