/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  BIFS Decoder
FILE:       bifs.cpp
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Implementation of classes for pre-preparing of bifs tables.
-------------------------------------------------------------------------*/

#include "defs.h"
#include "bifs.h"

/*-----------------------------------------------------------------
BifsBounds constructor constructs the value of its fields out of a 
BIFS table
-----------------------------------------------------------------*/
BifsBounds :: BifsBounds (const BifsFieldTable *pTable, int nSize)
{
	m_nTotalDefs = m_nTotalIns = m_nTotalOuts = m_nTotalDyns = 0;
	m_nTotalAll = nSize;
	for (int i = nSize; --i >= 0;)
	{	if (pTable [i].m_defID >= 0)
			++m_nTotalDefs;
		if (pTable [i].m_inID >= 0)
			++m_nTotalIns;
		if (pTable [i].m_outID >= 0)
			++m_nTotalOuts;
		if (pTable [i].m_dynID >= 0)
			++m_nTotalDyns;
	}
	CalculateBounds ();
}

/*-------------------------------------------------------------
Once we know the range for each ID type we can calculate how much bits it requires
-------------------------------------------------------------*/
void BifsBounds :: CalculateBounds ()
{
	m_nBitsPerDef = BitFieldWidth (m_nTotalDefs);
	m_nBitsPerIn = BitFieldWidth (m_nTotalIns);
	m_nBitsPerOut = BitFieldWidth (m_nTotalOuts);
	m_nBitsPerDyn = BitFieldWidth (m_nTotalDyns);
	m_nBitsPerAll = BitFieldWidth (m_nTotalAll);
}
