/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

#if !defined(_bifs_h)	// Sentry, use file only if it's not already included.
#define _bifs_h

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  BIFS Decoder
FILE:       bifs.h
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Declarations of macros and objects for BIFS parsing
-------------------------------------------------------------------------*/

#include "MediaObj.h"
#include <float.h> // MBS: needed for FLT_MAX

/*-----------------------------------------------------------------
Each node declaration includes a static table with a row for each field.
This table is used to parse the contents of the field.
The table is an array of type BifsFieldTable.
The node includes virtual functions that return the table address and
its length.

MBS: BIFS_FIELD, BIFS_FIELD_ROOT, bifsFieldTable modifed to correctly
handle quantizationType and animationType.
-----------------------------------------------------------------*/
typedef void (MediaObject :: *EventHandler) ();

#define BIFS_FIELD_OFFSET(className, fieldName) \
	(WORD) ((size_t) ((NodeField *) & ((className *)0) -> fieldName) - \
	(size_t) (MediaObject *)((className *)0))

#ifdef PLAYER3D
#define PLAYER3D_CUSTOM_NODE_DEC \
	virtual RenderingObject *CreateRenderingObject (MediaObject *);
#define PLAYER3D_CUSTOM_NODE_IMP(className) \
	RenderingObject *className :: CreateRenderingObject (MediaObject *) \
		{ return GetPresenter()->Create##className##Proxy(this); }
#else
#define PLAYER3D_CUSTOM_NODE_DEC
#define PLAYER3D_CUSTOM_NODE_IMP(className)
#endif

#define BIFS_NODE_DECLARATION \
	PLAYER3D_CUSTOM_NODE_DEC \
	public: static int nodeCode; \
	public: static int GetCode () {return nodeCode;} \
	virtual int GetNodeCode () const; \
	private: \
	static const BifsFieldTable m_fieldTable []; \
	static const BifsBounds m_bifsBounds; \
	static int m_nChildrenOffset; \
	virtual int GetFieldCount () const; \
	virtual const BifsFieldTable *GetFieldTable () const \
		{return m_fieldTable;} \
	virtual const BifsBounds *GetBifsBounds () const \
		{return &m_bifsBounds;} \
	virtual int GetChildrenOffset () const {return m_nChildrenOffset;}

#define BIFS_NODE_START_IMPLEMENTATION(className) \
	PLAYER3D_CUSTOM_NODE_IMP(className) \
	int className :: nodeCode = CODE_##className; \
	int className :: GetNodeCode () const {return nodeCode;} \
	static BifsNodeFactory<class className> bifsFactory##className; \
	const BifsFieldTable className :: m_fieldTable [] = {

#define OVERLOAD_NODE(className) \
	static BifsNodeFactory<class className> bifsFactory##className

#define BIFS_NULL_ENTRY {0},

// MBS modif
#define BIFS_FIELD_ROOT(className, fieldName, event, \
	defID, inID, outID, dynID, \
	quantizationType, quantParams, animationType  \
        ) \
	BIFS_FIELD_OFFSET (className, fieldName), \
	event, \
	FALSE, \
	defID, \
	inID, \
	outID, \
	dynID, \
	quantizationType, \
	quantParams, \
    animationType

// end MBS modif

#ifdef BIFSDEC
#define EVENTHANDLER(fieldName) (EventHandler) On##fieldName
#else
#define EVENTHANDLER(fieldName) (EventHandler) NULL
#endif


//JLF 04/03: remove TRACE_ENCODING case and always define node::GetName and field name in bifs table for script

#define BIFS_DECLARE_NODE \
	BIFS_NODE_DECLARATION \
	protected: virtual char *GetName ();

#define BIFS_IMPLEMENT_NODE_START(className) \
	char *className :: GetName () {return #className;} \
	BIFS_NODE_START_IMPLEMENTATION (className)

/*-----------------------------------------------------------------
BIFS_FIELD inserts a row in the field table.
The row contains the offset of the field (casted to its NodeField base)
from the beginning of the node class (casted to MediaObject base class).
The debug version includes a field fieldName for each node field
-----------------------------------------------------------------*/
// MBS modif for QP 13
#define BIFS_FIELD(className, fieldName, defID, inID, outID, dynID, \
	quantizationType, animationType \
) \
	{ \
		BIFS_FIELD_ROOT ( \
			className, fieldName, NULL, defID, inID, outID, dynID, \
			quantizationType, -1, animationType \
		), \
		0, \
		{-FLT_MAX}, {FLT_MAX}, \
		#fieldName \
	},
// end MBS modif
// MBS: added macro for quantizer type 13
#define BIFS_FIELD_SQ(className, fieldName, defID, inID, outID, dynID, \
	quantizationType, quantParams, animationType \
) \
	{ \
		BIFS_FIELD_ROOT ( \
			className, fieldName, NULL, defID, inID, outID, dynID, \
			quantizationType, quantParams, animationType \
		), \
		0, \
		{-FLT_MAX}, {FLT_MAX}, \
		#fieldName \
	},
// end MBS
// MBS modif
#define BIFS_FIELD1(className, fieldName, defID, inID, outID, dynID, \
	quantizationType, animationType, min, max \
) \
	{ \
		BIFS_FIELD_ROOT ( \
			className, fieldName, NULL, defID, inID, outID, dynID, \
			quantizationType, -1, animationType \
		), \
		1, \
		{min}, {max}, \
		#fieldName \
	},
// end MBS modif
// MBS: added macro for quantizer type 13
#define BIFS_FIELD1_SQ(className, fieldName, defID, inID, outID, dynID, \
	quantizationType, quantParams, animationType, min, max \
) \
	{ \
		BIFS_FIELD_ROOT ( \
			className, fieldName, NULL, defID, inID, outID, dynID, \
			quantizationType, quantParams, animationType \
		), \
		1, \
		{min}, {max}, \
		#fieldName \
	},
// end MBS add
// MBS modif for QP 13
#define BIFS_FIELD2(className, fieldName, defID, inID, outID, dynID, \
	quantizationType, animationType, min0, min1, max0, max1 \
) \
	{ \
		BIFS_FIELD_ROOT ( \
			className, fieldName, NULL, defID, inID, outID, dynID, \
			quantizationType, -1, animationType \
		), \
		2, \
		{min0, min1}, {max0, max1}, \
		#fieldName \
	},

#define BIFS_FIELD3(className, fieldName, defID, inID, outID, dynID, \
	quantizationType, animationType, \
	min0,  min1,  min2,  max0,  max1,  max2 \
) \
	{ \
		BIFS_FIELD_ROOT ( \
			className, fieldName, NULL, defID, inID, outID, dynID, \
			quantizationType, -1, animationType \
		), \
		3, \
		{min0, min1, min2}, {max0, max1, max2}, \
		#fieldName \
	},

#define BIFS_FIELD4(className, fieldName, defID, inID, outID, dynID, \
	quantizationType, animationType, \
	min0,  min1,  min2,  min3,  max0,  max1,  max2,  max3 \
) \
	{ \
		BIFS_FIELD_ROOT ( \
			className, fieldName, NULL, defID, inID, outID, dynID, \
			quantizationType, -1, animationType \
		), \
		4, \
		{min0, min1, min2, min3}, {max0, max1, max2, max3}, \
		#fieldName \
	},

#define BIFS_EVENT(className, fieldName, inID, dynID, \
	quantizationType, animationType \
) \
	{ \
		BIFS_FIELD_ROOT ( \
			className, fieldName, EVENTHANDLER (fieldName), \
			-1, inID, -1, dynID, \
			quantizationType, -1, animationType \
		), \
		0, \
		{-FLT_MAX}, {FLT_MAX}, \
		#fieldName \
	},

#define BIFS_EVENT1(className, fieldName, inID, dynID, \
	quantizationType, animationType, min, max \
) \
	{ \
		BIFS_FIELD_ROOT ( \
			className, fieldName, EVENTHANDLER (fieldName), \
			-1, inID, -1, dynID, \
			quantizationType, -1, animationType \
		), \
		1, \
		{min}, {max}, \
		#fieldName \
	},

#define BIFS_EVENT2(className, fieldName, inID, dynID, \
	quantizationType, animationType, min0, min1, max0, max1 \
) \
	{ \
		BIFS_FIELD_ROOT ( \
			className, fieldName, EVENTHANDLER (fieldName), \
			-1, inID, -1, dynID, \
			quantizationType, -1, animationType \
		), \
		2, \
		{min0, min1}, {max0, max1}, \
		#fieldName \
	},

#define BIFS_EVENT3(className, fieldName, inID, dynID, \
	quantizationType, animationType, \
	min0,  min1,  min2,  max0,  max1,  max2 \
) \
	{ \
		BIFS_FIELD_ROOT ( \
			className, fieldName, EVENTHANDLER (fieldName), \
			-1, inID, -1, dynID, \
			quantizationType, -1, animationType \
		), \
		3, \
		{min0, min1, min2}, {max0, max1, max2}, \
		#fieldName \
	},

#define BIFS_EVENT4(className, fieldName, inID, dynID, \
	quantizationType, animationType, \
	min0,  min1,  min2,  min3,  max0,  max1,  max2,  max3 \
) \
	{ \
		BIFS_FIELD_ROOT ( \
			className, fieldName, EVENTHANDLER (fieldName), \
			-1, inID, -1, dynID, \
			quantizationType, -1, animationType \
		), \
		4, \
		{min0, min1, min2, min3}, {max0, max1, max2, max3}, \
		#fieldName \
	},

#define BIFS_CHILDREN(className, fieldName, defID, inID,  outID,  dynID) \
	{ \
		BIFS_FIELD_OFFSET (className, fieldName), \
		NULL, \
		TRUE, \
		defID, inID, outID, dynID, \
		-1, -1, \
	        -1, \
		0, \
		{-FLT_MAX}, {FLT_MAX}, \
		#fieldName \
	},
// end MBS modif for QP 13


#define BIFS_IMPLEMENT_NODE_END(className) \
	}; \
	int className :: GetFieldCount () const \
		{return sizeof m_fieldTable / sizeof (BifsFieldTable);} \
	const BifsBounds className :: m_bifsBounds ( \
		m_fieldTable, \
		sizeof m_fieldTable / sizeof (BifsFieldTable) \
	); \
	int className :: m_nChildrenOffset = \
		FindChildrenOffset ( \
			m_fieldTable, \
			sizeof m_fieldTable / sizeof (BifsFieldTable) \
		);

/*-----------------------------------------------------------------
The field table defines in nodes contains:
1. An offset of the field within the node class
2. Event handling function address for eventIns.
3. (Debug mode only) the field fieldName
-----------------------------------------------------------------*/
#ifdef WIN32
//#pragma pack (push, 2) // MBS modif for bigger stack
#pragma pack (push, 4)   // from 2 to 4
#endif

struct BifsFieldTable
{
	short m_offset;
	EventHandler m_function;
	BYTE m_bChildren;
	short m_defID, m_inID, m_outID, m_dynID;
	char m_nQuantizationType, m_quantParams;
    char m_AnimationType;
	BYTE m_nDimension;
	float m_fMin [3], m_fMax [3];
	LPCSTR m_szName;
};

struct BifsBounds
{
	int m_nTotalDefs, m_nTotalIns, m_nTotalOuts, m_nTotalDyns, m_nTotalAll;
	int m_nBitsPerDef, m_nBitsPerIn, m_nBitsPerOut, m_nBitsPerDyn, m_nBitsPerAll;
	BifsBounds (const BifsFieldTable *pTable, int nSize);
	BifsBounds () :
		m_nTotalDefs (0), m_nTotalIns (0), m_nTotalOuts (0),
		m_nTotalDyns (0), m_nTotalAll (0)
	{}
	void CalculateBounds ();
};

#ifdef WIN32
#pragma pack (pop)
#endif

#endif                                      // _bifs_h sentry.
