/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

#if !defined(_sdldef_h)	// Sentry, use file only if it's not already included.
#define _sdldef_h

/*-------------------------------------------------------------------------
Project Im1Player/ Mux
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  OD Decoder/ Encoder
FILE:       sdldef.h
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Macros for SDL processing
-------------------------------------------------------------------------*/

/*-------------------------------------------------------------
We will need to know descriptor names for tracing the parse and
for encoding.
-------------------------------------------------------------*/
#if (defined(TRACE_ENCODING) || defined(BIFSENC)) && !defined(USE_OD_NAME)
#define USE_OD_NAME
#endif

#ifdef USE_OD_NAME
#define OD_NAME_DECLARATION virtual LPCSTR GetName ();
#define OD_NAME_DEFINITION(className) \
	LPCSTR className :: GetName () {return #className;}
#define OD_FIELD_NAME(fieldName) #fieldName
#else
#define OD_NAME_DECLARATION
#define OD_NAME_DEFINITION(className)
#define OD_FIELD_NAME(fieldName) NULL
#endif

/*-------------------------------------------------------------
This macro is included in the declaration of each non-abstract descriptor
-------------------------------------------------------------*/
#define OD_DECLARE_DESCRIPTOR \
	public: virtual int GetTag (); \
	public: static int GetCode (); \
	static bool IsMyTag (int nTag); \
	static BaseDescriptor *CreateMe (int nTag); \
	static BaseDescriptor *CreateMe (LPCSTR szClassName); \
	OD_NAME_DECLARATION \
	private: \
	static const ODFieldTable m_fieldTable []; \
	virtual int GetFieldCount () const; \
	virtual const ODFieldTable *GetFieldTable () const \
		{return m_fieldTable;}

/*-------------------------------------------------------------
This macro is included in the declaration of each abstract descriptor
-------------------------------------------------------------*/
#define OD_DECLARE_ABSTRACT_DESCRIPTOR \
	public: static bool IsMyTag (int nTag); \
	static BaseDescriptor *CreateMe (int nTag); \
	static BaseDescriptor *CreateMe (LPCSTR szClassName); \
	OD_NAME_DECLARATION \
	private: \
	static const ODExtensionTable m_extensionTable [];

/*-------------------------------------------------------------
This macro is included in the definition of every non-abstract descriptor
-------------------------------------------------------------*/
#define OD_IMPLEMENT_START(className) \
	int className :: GetTag () {return TAG_##className;} \
	int className :: GetCode () {return CODE_##className;} \
	OD_NAME_DEFINITION(className) \
	static ODFactory<class className> odFactory##className; \
	const ODFieldTable className :: m_fieldTable [] = {

/*-------------------------------------------------------------
This macro starts the implementation of non-abstract descriptors
-------------------------------------------------------------*/
#define OD_IMPLEMENT_DESCRIPTOR_START(className) \
	bool className :: IsMyTag (int nTag) {return nTag == TAG_##className;} \
	BaseDescriptor *className :: CreateMe (int) \
	{return SDLField :: CreateClass (className :: GetCode ());} \
	BaseDescriptor *className :: CreateMe (LPCSTR) \
	{return SDLField :: CreateClass (className :: GetCode ());} \
	OD_IMPLEMENT_START (className)

/*-------------------------------------------------------------
This macro starts the implementation of abstract descriptors
-------------------------------------------------------------*/
#define OD_IMPLEMENT_ABSTRACT_DESCRIPTOR_START(className) \
	bool className :: IsMyTag (int nTag) \
	{return nTag >= TAGMIN_##className && nTag <= TAGMAX_##className;} \
	OD_NAME_DEFINITION(className) \
	const ODExtensionTable className :: m_extensionTable [] = {

/*-------------------------------------------------------------
Calcualte the offset of a field in a descriptor
-------------------------------------------------------------*/
#define OD_FIELD_OFFSET(className, fieldName) \
	(WORD) ((size_t) ((SDLField *) & ((className *)0) -> fieldName) - \
	(size_t) (BaseDescriptor *)((className *)0))

/*-------------------------------------------------------------
This macro defines a field entry in a descriptor field table
-------------------------------------------------------------*/
#define OD_IMPLEMENT_FIELD(className, fieldName) \
	{ \
		OD_FIELD_OFFSET (className, fieldName), \
		OD_FIELD_NAME (fieldName) \
	},

/*-------------------------------------------------------------
This macro defines an extension entry in an abstarct descriptor extensions table
-------------------------------------------------------------*/
#define OD_IMPLEMENT_EXTENSION(extension) \
	{ \
		TAG_##extension, \
		CODE_##extension \
	},

/*-------------------------------------------------------------
This macro terminates the implementation of each non-abstract descriptor
-------------------------------------------------------------*/
#define OD_IMPLEMENT_DESCRIPTOR_END(className) \
	}; \
	int className :: GetFieldCount () const \
		{return sizeof m_fieldTable / sizeof (ODFieldTable);}

/*-------------------------------------------------------------
This macro terminates the implementation of each abstract descriptor
-------------------------------------------------------------*/
#define OD_IMPLEMENT_ABSTRACT_DESCRIPTOR_END(className) \
	}; \
	BaseDescriptor *className :: CreateMe (int nTag) \
	{	return CreateExtension ( \
			nTag, \
			m_extensionTable, \
			sizeof m_extensionTable / sizeof (ODExtensionTable) \
		); \
	} \
	BaseDescriptor *className :: CreateMe (LPCSTR szClassName) \
	{	return CreateExtension ( \
			szClassName, \
			m_extensionTable, \
			sizeof m_extensionTable / sizeof (ODExtensionTable) \
		); \
	}

/*-------------------------------------------------------------
A macro for overloading generic OD classes with implementation specific
extensions
-------------------------------------------------------------*/
#define OVERLOAD_OD_CLASS(className) \
	static ODFactory<class className> odFactory##className

#endif                                      // _sdldef_h sentry.
