/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Compositor
FILE:       ztype.cpp
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Implementation of types of node fields
-------------------------------------------------------------------------*/

#include "defs.h"
#include "ztypes.h"
#include <math.h>

/*-----------------------------------------------------------------
Image
-----------------------------------------------------------------*/
//MBS 110498: added constructor Image(w,h,components)
//Alvar 981223: Added pixel array allocation and refcount stuff

Image :: Image (int w, int h, int components) :      // Mike B-S ctor
	width (w), 
	height (h), 
	numComponents (components), 
	pixels (NULL) ,
	m_pRefCnt (NULL)
{}

Image :: Image (int) :                           // Zvi ctor
	width (0), 
	height (0),
	numComponents (0),
	pixels (NULL),
	m_pRefCnt (NULL) 
{}
	
Image :: Image (const Image &i)                      // copy ctor
{
	width         = i.width;
	height        = i.height;
	numComponents = i.numComponents;
	pixels        = i.pixels;
	m_pRefCnt     = i.m_pRefCnt;
	if( m_pRefCnt )
		++*m_pRefCnt;
}

Image& Image :: operator = (const Image &i)           // assignment
{
	if( m_pRefCnt ) 
		if( --*m_pRefCnt == 0 )
		{	delete [] pixels;
			delete m_pRefCnt;
		}
	width         = i.width;
	height        = i.height;
	numComponents = i.numComponents;
	pixels        = i.pixels;
	m_pRefCnt     = i.m_pRefCnt;
	if( m_pRefCnt )
		++*m_pRefCnt;
	return *this;
}

Image :: ~Image () 
{ 
	if (m_pRefCnt && --*m_pRefCnt == 0)
	{	delete [] pixels;
		delete m_pRefCnt;
	}
}

void Color3f::toHSV()
{
	float _max = max(r, max(g, b));
	float _min = min(r, min(g, b));
	float h, s;

	s = ((_max == 0.0) ? 0.0 : ((_max-_min)/_max));
	if (s != 0.0) {
		float rl((_max - r) / (_max-_min));
		float gl((_max - g) / (_max-_min));
		float bl((_max - b) / (_max-_min));
		if (_max == r) {
			if (_min == g) h = 60.0*(5.0+bl);
			else h = 60.0*(1.0-gl);
		} else if (_max == g) {
			if (_min == b) h = 60.0*(1.0+rl);
			else h = 60.0*(3.0-bl);
		} else {
			if (_min == r) h = 60.0*(3.0+gl);
			else h = 60.0*(5.0-rl);
		}
	} else {
		h = 0.0;
	}
	r = h;
	g = s;
	b = _max;
}

void Color3f::fromHSV()
{
	int hh = (int) (r/360.0);
	float hue = r - hh * 360.0;
	float h6 = hue/60.0;
	int i = (int) h6;
	float f = h6-i;
	float p1 = b * (1.0 - g);
	float p2 = b * (1.0 - (g*f));
	float p3 = b * (1.0 - g * (1-f) );
	float _r, _g, _b;
	switch(i) {
	case 0:
		_r = b;	
		_g = p3;	
		_b = p1;
		break;
	case 1:
		_r = p2;
		_g = b;
		_b = p1;
		break;
	case 2:
		_r = p1;
		_g = b;
		_b = p3;
		break;
	case 3:
		_r = p1;       
		_g = p2;
		_b = b;
		break;
	case 4:
		_r = p3;
		_g = p1;
		_b = b;
		break;
	case 5:
		_r = b;
		_g = p1;
		_b = p2;
		break;
	default:
		_r = _g = _b = 0;
	}
	r = _r;
	g = _g;
	b = _b;
}


// SB @@@@ from Karl's code >>>>
void Rotation :: Normalize ()
{
	float mag = sqrt (x * x + y * y + z * z);
	if (mag == 0.0)
		mag = 1.0;
	x /= mag;
	y /= mag;
	z /= mag;
}

void Rotation::makeQuat()
{
    const float sintd2 = sin(theta * 0.5);
    const float len = sqrt(x*x + y*y + z*z);
    const float f = sintd2 / len;
    theta = cos(theta * 0.5);
    x = x * f;
    y = y * f;
    z = z * f;
}
void Rotation::unmakeQuat()
{
    double sina2 = sqrt(x*x + y*y + z*z);
    double angle = (2.0 * atan2(sina2, theta));

    if (sina2 >= 1e-8) {
		sina2 = 1.0/sina2;
        x *= sina2;
        y *= sina2;
        z *= sina2;
        theta = angle;
    } else {
		x = 0;
		y = 1;
		z = 0;
		theta = 0;
    }
}


Rotation Rotation::Slerp(Rotation r, float t)
{
	Rotation from = *this;
	Rotation to = r;
	from.makeQuat();
	to.makeQuat();

	float cosom = from * r;

	// adjust signs (if necessary)
	Rotation quat;
	if (cosom < 0.0) {
		cosom = -cosom;
		quat.x = -r.x;   // Reverse all signs
		quat.y = -r.y;
		quat.z = -r.z;
		quat.theta = -r.theta;
	} else { 
		quat = to;
	}

	// Calculate coefficients
	float sclp, sclq;
	// 0.0001 -> some epsillon
	if ((1.0 - cosom) > 0.0001) {
		// Standard case (slerp)
		float omega, sinom;
		omega = acos( cosom ); 
		sinom = sin(omega);
		sclp  = sin( (1.0 - t) * omega ) / sinom;
		sclq  = sin( t * omega ) / sinom;
	} 
	// Very close, do linear interp
	else {
		sclp = 1.0 - t;
		sclq = t;
	}
	Rotation res;
	res.x = sclp * from.x + sclq * quat.x;
	res.y = sclp * from.y + sclq * quat.y;
	res.z = sclp * from.z + sclq * quat.z;
	res.theta = sclp * from.theta + sclq * quat.theta;

	res.unmakeQuat();
	return res;
}

void Rotation::operator*=(const Vec3f& v){

	float qx, qy, qz, qw;
	qx = x;
	qy = y;
	qz = z;
	qw = theta;
	x = qw * v.x            + qy * v.z - qz * v.y;
	y = qw * v.y - qx * v.z            + qz * v.x;
	z = qw * v.z + qx * v.y - qy * v.x;
	theta =          - qx * v.x - qy * v.y - qz * v.z;
}

void Vec3f::operator*=(const Rotation& r)
{ //Transforms the vector instance by a quaternion.
	Rotation q =r;
	q.makeQuat();
	Rotation temp(r.theta, -r.x, -r.y, -r.z);
	temp.makeQuat();
	temp *= *this;
	temp *= q;
	x = temp.x;
	y = temp.y;
	z = temp.z;
}

void Vec3f::operator*=(const float m_elements[16])
{
  float ox = x;
  float oy = y;
  float oz = z;

  x = m_elements[0]*ox + m_elements[4]*oy + m_elements[8]*oz+m_elements[12];
  y = m_elements[1]*ox + m_elements[5]*oy + m_elements[9]*oz+m_elements[13];
  z = m_elements[2]*ox + m_elements[6]*oy + m_elements[10]*oz+m_elements[14];
}


