/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  BIFS Decoder/ Compositor / Presenter
FILE:       OpenStr.cpp
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Implementation of ObjectDescriptorImp and ESDescriptorImp
			funtions that handle opening/closing of streams.
-------------------------------------------------------------------------*/
#include "defs.h"
#include "ODImp.h"
#include "StreamCo.h"
#include "Executiv.h"
#include "Stream.h"
#include "AL.h"
#include "IPMP.h"
#include "InLine.h"
#include "Channel.h"
#include "OCIDec.h"
#include "StrParser.h"
#include "ODP3.h"

//#define REBUFFERING

/*-------------------------------------------------------------
Destructor
-------------------------------------------------------------*/
ObjectDescriptorImp :: ~ObjectDescriptorImp ()
{
	delete m_pRemoteAL;
	delete m_pRemoteOD;
}

/*-------------------------------------------------------------
Create a ClockReference (or attach to an existing one) for each of the ESD in an OD
-------------------------------------------------------------*/
void ObjectDescriptorImp :: SetClocks ()
{
	for (int i = esDescr.GetLength (); --i >= 0;)
		((ES_DescriptorImp*) esDescr [i]) -> SetClock (m_pAL);
}

/*-----------------------------------------------------------------
Go through the ESDescriptors contained in the OD, instantiate decoders
and IPMP systems, link them through MediaStream objects, and configure
all the input channels involved. The funtion returns a pointer to the
MediaStream object that provides the media to the compositor.
-----------------------------------------------------------------*/
MediaStreamImp *ObjectDescriptorImp :: SetStream (StreamConsumer *pNode)
{
	/*-------------------------------------------------------------
	If a remote OD has already been instantiated, use that one
	-------------------------------------------------------------*/
	if (m_pRemoteOD)
		return m_pRemoteOD ->SetStream (pNode);

	/*-------------------------------------------------------------
	Handle the remote OD case.
	-------------------------------------------------------------*/
	if (URL_Flag != 0)
	{	if (m_nInstance++ == 0)
			pNode -> GetExecutive () -> CreateService (URLstring, this, m_pAL -> GetServiceID ());
		return NULL;
	}

	if (esDescr.GetLength () == 0)
	{	if (m_nInstance++ == 0)
			pNode -> GetExecutive () -> Error (NO_STREAM);
		return NULL;
	}

	/*-------------------------------------------------------------
	To do: reorder the ESDs according to their dependencies. 
	-------------------------------------------------------------*/

	/*-------------------------------------------------------------
	Loop through the ESDs and set up decoder streams
	-------------------------------------------------------------*/
	if (m_nInstance++ == 0)
		m_pDecoder = NULL;
	MediaStreamImp *pResult = NULL;
	for (int i = 0; i < esDescr.GetLength (); i++)
	{	MediaStreamImp *pStream = ((ES_DescriptorImp *) esDescr [i]) ->
			SetDecoder (this, pNode);
		if (pResult == NULL)
			pResult = pStream;
	}
	
	/*-------------------------------------------------------------
	If the url of the node contains a segment name translate it to time interval according
	to the segmentDescriptor list
	-------------------------------------------------------------*/
	pNode ->SetSegment (-1, -1);
	if (pNode -> GetSegment () != NULL)
	{	for (i = ociDescr.GetLength (); --i >= 0;)
		{	if (ociDescr [i] -> GetTag () == TAG_SegmentDescriptor)
			{	SegmentDescriptor *pDesc = (SegmentDescriptor*) ociDescr [i];
				if (!strcmp (pDesc ->segmentName, pNode -> GetSegment ()))
				{	pNode -> SetSegment (pDesc ->start, pDesc ->duration);
					break;
				}
			}
		}
	}

	return pResult;
}

/*-------------------------------------------------------------
A session with remoteOD has been established
-------------------------------------------------------------*/
void ObjectDescriptorImp :: OnConnect (const BYTE *pInitOD, int nInitODLen)
{
	ODDecoder odDec (
		m_pInline -> GetExecutive (),
		pInitOD,
		nInitODLen,
		m_pInline -> GetExecutive () -> GetTraceFile (),
		"IOD",
		m_pInline -> GetExecutive () -> GetTraceFlags (),
		0
	);
	m_pRemoteOD = new ObjectDescriptorImp;
	m_pRemoteOD -> DecInitialOD (&odDec);
	m_pRemoteOD -> SetAL (m_pRemoteAL);
	m_pRemoteOD -> SetInline (m_pInline);
	m_pRemoteOD -> SetClocks ();
	m_pRemoteOD ->ObjectDescriptorID = ObjectDescriptorID;
	m_pInline -> ConnectRemoteOD (m_pRemoteOD);
}

/*-------------------------------------------------------------
When a new node is added on this OD, find the last OD in the remote OD chain and
attach that one to the node
-------------------------------------------------------------*/
void ObjectDescriptorImp :: AttachNode (StreamConsumer *pNode)
{
	if (m_pRemoteOD)
		m_pRemoteOD -> AttachNode (pNode);
	else
		pNode -> SetStream (this);
}

/*-------------------------------------------------------------
Start processing the streams described by this OD
-------------------------------------------------------------*/
void ObjectDescriptorImp :: Start ()
{
	/*-------------------------------------------------------------
	If no instance of the OD has opened the stream:	
	Loop through the ESDs and start them
	-------------------------------------------------------------*/
	if (m_nOpen++ == 0)
		for (int i = 0; i < esDescr.GetLength (); i++)
			((ES_DescriptorImp *) esDescr [i]) -> Start ();
}

/*-------------------------------------------------------------
Stop processing the streams described by this OD
-------------------------------------------------------------*/
void ObjectDescriptorImp :: Stop ()
{
	/*-------------------------------------------------------------
	If this is the last instance using this node
	Loop through the ESDs and stop them
	-------------------------------------------------------------*/
	if (--m_nOpen == 0)
		for (int i = 0; i < esDescr.GetLength (); i++)
			((ES_DescriptorImp *) esDescr [i]) -> Stop ();
}

/*-------------------------------------------------------------
Delete all the resources created by this OD
-------------------------------------------------------------*/
void  ObjectDescriptorImp:: Detach ()
{
	/*-------------------------------------------------------------
	if this is the last instance using the node
	Loop through the ESDs and detach them
	-------------------------------------------------------------*/
	if (--m_nInstance == 0)
		for (int i = 0; i < esDescr.GetLength (); i++)
			((ES_DescriptorImp *) esDescr [i]) -> Detach ();
}

/*-------------------------------------------------------------
Send data on a back channel
-------------------------------------------------------------*/
void ObjectDescriptorImp :: DataSend (LPBYTE pData, int nLength)
{
	if (m_pChannel)
		m_pChannel -> DataSend (pData, nLength);
}

/*-------------------------------------------------------------
Find an ESD for removal
-------------------------------------------------------------*/
ES_DescriptorImp *ObjectDescriptorImp :: FindESD (int nESid)
{
	ES_DescriptorImp *pESD = NULL;
	if (m_pRemoteOD != NULL)
	{	m_pInline -> GetExecutive () -> Error (CANNOT_ALTER_REMOTE_OD);
		return NULL;
	}
	for (int i = esDescr.GetLength (); --i >= 0;)
	{	pESD = (ES_DescriptorImp *) esDescr [i];
		if (pESD -> ES_ID == nESid)
			return pESD;
	}
	m_pInline -> GetExecutive () -> Error (TRYING_TO_REMOVE_UNKNOWN_ESD);
	return NULL;
}

/*-------------------------------------------------------------
Find an ESD in this OD and remove it
-------------------------------------------------------------*/
void ObjectDescriptorImp :: RemoveESD (int nESid)
{
	ES_DescriptorImp *pESD;
	for (int i = esDescr.GetLength (); --i >= 0;)
	{	pESD = (ES_DescriptorImp *) esDescr [i];
		if (pESD -> ES_ID == nESid)
		{	esDescr.DeleteFrom (i);
			return;
		}
	}
}

/*-------------------------------------------------------------
Update an ESD for this OD
-------------------------------------------------------------*/
bool ObjectDescriptorImp :: UpdateESD (ES_DescriptorImp *pESD)
{
	if (m_pRemoteOD != NULL)
	{	m_pInline -> GetExecutive () -> Error (CANNOT_ALTER_REMOTE_OD);
		return false;
	}
	esDescr += pESD;
	return true;
}

/*-------------------------------------------------------------
ES_DescriptorImp constructor
-------------------------------------------------------------*/
ES_DescriptorImp :: ES_DescriptorImp ()
{
	m_pExecutive = NULL;
	m_pOutputStream = m_pInputStream = NULL;
	m_pDecoder = NULL;
	m_pChannel = NULL;
	m_pClock = NULL;
	m_bAttached = false;
	m_error = IM1_OK;
	m_nInstantiated = m_nStart = m_nDecoderStart = 0;
}

/*-------------------------------------------------------------
Create a ClockReference (or attach to an existing one) for this ESD
-------------------------------------------------------------*/
void ES_DescriptorImp :: SetClock (ALManager *pAL)
{
	if (slConfigDescr == NULL_DESC)
		return;
	
	/*-------------------------------------------------------------
	Find whether this is an OCR stream or it depends on another OCR stream.
	If it is an OCR stream decide whether real OCR are embedded (OCRLength > 0).
	Create a ClockReference or attach one, providing it this info
	-------------------------------------------------------------*/
	int nClockID = OCR_ES_Id;
	if (nClockID <= 0)
		nClockID = ES_ID;
	int nHasOCR = -1;
	if (nClockID == ES_ID)
		nHasOCR = slConfigDescr -> OCRLength > 0;
	m_pClock = pAL -> AttachClock (nClockID, nHasOCR);
}	

/*-------------------------------------------------------------
Instantiate a decoder and configure a channel for an elementary stream.
The function returns a pointer to the composition buffer.
-------------------------------------------------------------*/
MediaStreamImp *ES_DescriptorImp :: SetDecoder (
	ObjectDescriptorImp *pOD,
	StreamConsumer *pNode
)
{
	/*-------------------------------------------------------------
	If a previous attempt to instantiate this decoder failed, return now
	-------------------------------------------------------------*/
	if (m_error != IM1_OK)
		return NULL;

	Decoder *pDecoder = NULL;
	m_pExecutive = pNode -> GetExecutive ();
	m_pOD = pOD;
	if (decConfigDescr == NULL_DESC)
	{	m_pExecutive -> Error (m_error = NO_STREAM_TYPE);
		return NULL;
	}

	/*-------------------------------------------------------------
	Skip IPMP streams
	-------------------------------------------------------------*/
	if (decConfigDescr -> streamType == ST_IPMPStream)
		return NULL;

	/*-------------------------------------------------------------
	OCR streams are opened without creating a data buffer
	-------------------------------------------------------------*/
	if (decConfigDescr -> streamType == ST_ClockReferenceStream)
	{	if (m_pChannel == NULL)
			m_pChannel = OpenStream (pOD, 0, 0);
		return NULL;
	}

	/*-------------------------------------------------------------
	Do the following only if a decoder has not been already instantiated
	-------------------------------------------------------------*/
	if (m_pDecoder == NULL)
	{	/*-------------------------------------------------------------
		If the stream depends on another stream find the decoder used by
		that stream. If no decoder is used, skip this stream as well.
		-------------------------------------------------------------*/
		if (dependsOn_ES_ID > 0) {
			for (int i = 0; i < pOD -> esDescr.GetLength (); i++) {
				if (dependsOn_ES_ID == pOD -> esDescr [i] -> ES_ID) {
					//regular dependency case
					if (pOD->esDescr[i]->decConfigDescr->streamType == decConfigDescr -> streamType) {
						pDecoder = ((ES_DescriptorImp *) pOD -> esDescr [i]) -> GetDecoder ();
						if (pDecoder == NULL) return NULL;
					}
					//other than "BIFS depends on OD"
					else if ((pOD->esDescr[i]->decConfigDescr->streamType != ST_ObjectDescriptorStream)
						|| (decConfigDescr->streamType != ST_SceneDescriptionStream)) {

						pDecoder = ((ES_DescriptorImp *) pOD -> esDescr [i]) -> GetDecoder ();
						if (pDecoder == NULL) return NULL;
					}
					break;
				}
			}
		}
						
		/*-------------------------------------------------------------
		First try if the node can create a decoder for this stream type.
		If the function returns the same decoder as the one used by stream
		whom this stream depends on, mark that this stream is attached to
		another decoder.
		-------------------------------------------------------------*/
		m_pDecoder =
			pNode -> CreateDecoder (decConfigDescr, qosDescr, pDecoder);
		m_bAttached = m_pDecoder != NULL && m_pDecoder == pDecoder;

		/*-------------------------------------------------------------
		ObjectContentInfoStream is a special case and we create a dedicated
		OCI parser for such stream
		-------------------------------------------------------------*/
		if (m_pDecoder == NULL && decConfigDescr -> streamType == ST_ObjectContentInfoStream)
			m_pDecoder = CreateOCIDecoder (decConfigDescr -> objectTypeIndication);

		/*-------------------------------------------------------------
		If the node didn't create a decoder, create it now using the info
		in decConfigDescr. We skip this stream if the OD already has a
		decoder.
		-------------------------------------------------------------*/
		if (m_pDecoder == NULL)
		{	if (pOD -> GetDecoder () != NULL)
				return NULL;
			m_pDecoder = (Decoder *) LoadFilter (
				decConfigDescr -> streamType,
				decConfigDescr -> objectTypeIndication
			);
		}

		/*-------------------------------------------------------------
		Setup the decoder
		-------------------------------------------------------------*/
		if (m_pDecoder != NULL)
		{	DecoderSpecificInfo *pSpecificInfo = decConfigDescr -> decSpecificInfo;
			LPCBYTE pInfo = NULL;
			int nInfoLength = 0;
			if (pSpecificInfo != NULL)
			{	pInfo = pSpecificInfo -> info;
				nInfoLength = pSpecificInfo -> info.GetLength ();
			}
			if (!m_pDecoder -> Setup (pInfo, nInfoLength, 0))
				m_error = DECODER_SETUP_FAILURE;
			else
			{	pOD -> SetDecoder (m_pDecoder);
				if (m_pInputStream != NULL)
					m_pDecoder -> SetInputStream (0, m_pInputStream);
			}
		}
	}

	/*-------------------------------------------------------------
	Get the rendering capabilities of the node.
	If there are no capabilities this is a non rendering node like MediaControl or
	MediaSensor
	-------------------------------------------------------------*/
	CapabilitiesImp caps;
	pNode -> GetCapabilities (&caps);
	bool bFormatOK = false;
	if (m_pDecoder)
		bFormatOK = m_pDecoder -> SetFormat (caps, m_decoderParams);

	if (caps.GetFormatCapabilityLength () > 0)
	{	/*-----------------------------------------------------------------
		Verify that there is a matching capability between the decoder and
		the node, and sets up the decoder format. If no matching
		capability exists, delete the decoder and ignore the stream
		-----------------------------------------------------------------*/
		if (m_pDecoder == NULL)
			m_error = UNKNOWN_DECODER_TYPE;
		else
		{	if (bFormatOK)
				pNode -> SetStreamFormat (m_decoderParams.m_format);
			else
				m_error = INCOMPATIBLE_DECODER_TYPE;
		}
	}

	if (m_error != IM1_OK)
	{	m_pExecutive -> Error (m_error);
		return NULL;
	}

	/*-------------------------------------------------------------
	Establish an input channel if hasn't been done already
	-------------------------------------------------------------*/
	if (m_pChannel == NULL)
	{	/*-------------------------------------------------------------
		Determine buffer threshold numbers. In this version we're using predefined
		numbers according to stream type
		-------------------------------------------------------------*/
		float fThreshold = 0;

#ifdef REBUFFERING
		switch (decConfigDescr -> streamType) {
		case ST_AudioStream:
			fThreshold = 0.8;
			break;
		case ST_VisualStream:
			if (decConfigDescr -> objectTypeIndication != 0x6c &&
				decConfigDescr -> objectTypeIndication != 0xc3
			)
				fThreshold = 0.6;
			break;
		}
#endif

		/*-----------------------------------------------------------------
		Set up the channel
		-----------------------------------------------------------------*/
		m_pChannel = OpenStream (
						pOD,
						m_decoderParams.m_nDBSize,
						m_decoderParams.m_nOptimalAUSize,
						m_decoderParams.m_bBackChannel,
						fThreshold
					 );
		if (m_pChannel == NULL)
			return NULL;
		pOD -> SetChannel (m_pChannel);
	}

	/*-----------------------------------------------------------------
	Create the composition buffer.
	If decoderParams.m_nCBSize + m_pDecoder -> GetOptimalOutputSize () 
	is zero, it means no composition buffer is used, and the decoder
	manipulates the scene graph directly.
	(this is done only if a decoder has been instantiated and the output stream not yet)
	-----------------------------------------------------------------*/
	if (m_pOutputStream == NULL && m_pDecoder != NULL)
	{	if (m_decoderParams.m_nCBSize != 0)
			m_pOutputStream = new MediaStreamImp (
				m_pInputStream -> GetClockReference (),
				m_decoderParams.m_nCBSize + m_pDecoder -> GetOptimalOutputSize ()
			);
		m_pDecoder -> SetOutputStream (
			m_decoderParams.m_nStreamNumber,
			m_pOutputStream
		);
	}

	/*-------------------------------------------------------------
	Instantiate IPMP systems for this stream. If failed, dont use this
	stream
	-------------------------------------------------------------*/
	if (m_nInstantiated++ == 0)
	{	if (!InvokeIPMP (pOD))
		{	m_error = IPMP_NOT_AVAILABLE;
			return NULL;
		}
		if (m_pDecoder != NULL)
			m_pDecoder -> SetInputStream (0, m_pInputStream);
	}

	/*-------------------------------------------------------------
	If decoderParams.m_nCBSize is zero
	and m_pDecoder -> GetOptimalOutputSize () is not zero,
	this is null decoder, i.e., stream is consumed directly by the 
	object.
	-------------------------------------------------------------*/
	if (m_pDecoder == NULL ||
		m_decoderParams.m_nCBSize == 0 && m_pDecoder -> GetOptimalOutputSize () != 0
	)
		return m_pInputStream;
	return m_pOutputStream;
}

/*-------------------------------------------------------------
Load the DLL and instantiate the Filter object for the given streamType
and objectTypeIndication. Returns NULL if failed.
-------------------------------------------------------------*/
Filter *ES_DescriptorImp :: LoadFilter (
	int nStreamType,
	int objectTypeIndication
)
{
	static ZRegistry registry("Decoders");
	HMODULE hDLL = NULL;

	/*-------------------------------------------------------------
	Construct the decoder type and asks the registry object to locate it,
	load its DLL, and instantiate it.
	-------------------------------------------------------------*/
	static LPCSTR szStreamType [] =
	{
		"",			// ObjectDescriptorStream = 1
		"",			// ClockReferenceStream = 2
		"",			// SceneDescriptionStream = 3
		"Visual",	// VisualStream = 4
		"Audio",	// AudioStream = 5
		"MPEG7",	// MPEG7Stream = 6
		"IPMP"		// IPMP = 7
	};
	enum {TABLE_LENGTH = sizeof szStreamType / sizeof (LPCSTR)};

	if (--nStreamType < 0 || nStreamType >= TABLE_LENGTH ||
		szStreamType [nStreamType] [0] == 0
	)
		return NULL;

	ZString szFilterType (szStreamType [nStreamType]);
	if (objectTypeIndication != 0 && objectTypeIndication != 0xff)
	{	char szBuffer [10];
		szFilterType += '-';
		szFilterType += itoa (objectTypeIndication, szBuffer, 16);
	}

	Filter *pFilter =
		(Filter *) registry.CreateInstance (szFilterType, hDLL);
	if (pFilter != NULL)
		pFilter -> SetLibrary (hDLL);
	return pFilter;
}

/*-------------------------------------------------------------
Perform the stream open procedure with this ES_Descriptor
-------------------------------------------------------------*/
DataChannel *ES_DescriptorImp :: OpenStream (
	ObjectDescriptorImp *pOD,
	int nBufferSize,
	int nExpectedAULength,
	bool bBackChannel,
	float fThreshold
)
{
	m_pAL = pOD -> GetAL ();
	if (slConfigDescr == NULL_DESC)
	{	m_pExecutive -> Error (NO_SLCONFIG_INFO);
		return NULL;
	}

	/*-------------------------------------------------------------
	Open the stream, and attach it to a MediaStream object.
	If nBufferSize is zero, open the stream without attaching to
	a MediaStream, which means "get data and discard"
	-------------------------------------------------------------*/
	if (nBufferSize != 0)
	{
		int nThreshold = 0;

		if (nBufferSize > 0)
		{	if (decConfigDescr != NULL_DESC &&
				decConfigDescr -> bufferSizeDB != 0
			)
			{	nBufferSize = decConfigDescr -> bufferSizeDB;
				nThreshold = nBufferSize * fThreshold;
			}

			/*-------------------------------------------------------------
			The actual buffer size is calcualted as following:
			If nExpectedAULength > 0, we use twice the declared amount of
			buffer size, to compenstate for jitter delays, and solve buffer
			slicing (units need to be in contiguneous memory).
			If nExpectedAULength is zero, we assume buffer is required for
			just one AU.
			To this we have to add space for AU headers. If nExpectedAULength
			> 0, we take 3 x buffer-size/average unit-size. If 
			nExpectedAULength is 0, there is just one unit and one header.
			-------------------------------------------------------------*/
			if (nExpectedAULength > 0)
				nBufferSize =
					nBufferSize * 2 +
					3 * (sizeof (int) + sizeof (DWORD)) *
					(nBufferSize / nExpectedAULength + 1);
			else
			{	nExpectedAULength = nBufferSize;
				nBufferSize += sizeof (int) + sizeof (DWORD);
			}
		}

		/*-------------------------------------------------------------
		Create a MediaStream object with NULL clock reference. The
		clock reference will be set by ALManager::AttachStream()
		-------------------------------------------------------------*/
		m_pInputStream = new MediaStreamImp (NULL, nBufferSize, nThreshold);
#ifdef STREAM_DELAY //PAB+:
		m_pInputStream->SetESD(this); 
#endif //STREAM_DELAY

	}


	/*-------------------------------------------------------------
	Open the input channel and link it to the proper MediaStream object
	-------------------------------------------------------------*/
	return m_pAL -> AttachStream (
		this,
		m_pInputStream,
		slConfigDescr,
		nExpectedAULength,
		bBackChannel
	);
}

/*-------------------------------------------------------------
Attach IPMP filters to an elementary stream. Return true if succeeded 
to setup all IPMP systems.
-------------------------------------------------------------*/
bool ES_DescriptorImp :: InvokeIPMP (ObjectDescriptorImp *pOD)
{
	int i;

	/*-------------------------------------------------------------
	Loop through the ESDs to set up IPMP filters established by IPMP
	streams.
	-------------------------------------------------------------*/
	for (i = 0; i < pOD -> esDescr.GetLength (); i++)
		if (! ((ES_DescriptorImp *) pOD -> esDescr [i]) -> SetIPMP (this, pOD))
			return false;

	/*-------------------------------------------------------------
	Loop through the IPMP pointers and handle IPMP descriptors
	-------------------------------------------------------------*/
	if (!HandleIPMPDescriptors (
			ipmpDescrPtr,
			ipmpDescrPtr.GetLength (),
			pOD -> GetInline ()
		)
	)
		return false;

	/*-------------------------------------------------------------
	Loop through the IPMP pointers of the OD and handle IPMP descriptors
	-------------------------------------------------------------*/
	if (!HandleIPMPDescriptors (
			pOD -> ipmpDescrPtr,
			pOD -> ipmpDescrPtr.GetLength (),
			pOD -> GetInline ()
		)
	)
		return false;
	return true;
}

/*-------------------------------------------------------------
If the object is an IPMP stream, instantiate an IPMP system and attach
it to the given elementary stream
-------------------------------------------------------------*/
bool ES_DescriptorImp :: SetIPMP (
	ES_DescriptorImp *pES,
	ObjectDescriptorImp *pOD
)
{
	if (decConfigDescr == NULL_DESC ||
		decConfigDescr -> streamType != ST_IPMPStream
	)
		return true;

	/*-------------------------------------------------------------
	This version doesn't allow the attaching of more than IPMP stream to
	a single data stream
	-------------------------------------------------------------*/
	m_pAL = pOD -> GetAL ();
	m_pExecutive = m_pAL -> GetExecutive ();

	if (m_pChannel != NULL)
	{	m_pExecutive -> Error (IPMP_STREAM_ALREADY_IN_USE);
		return false;
	}

	/*-------------------------------------------------------------
	Load the appropriate IPMP system
	-------------------------------------------------------------*/
	IPMPManager *pIPMP = (IPMPManager *) LoadFilter (
		ST_IPMPStream,
		decConfigDescr -> objectTypeIndication
	);
	if (!pIPMP)
	{	m_pExecutive -> Error (IPMP_NOT_AVAILABLE);
		return false;
	}
	else if (!pIPMP -> GetAccess ())
	{	m_pExecutive -> Error (NO_ACCESS_TO_PROTECTED_STREAM);
		return false;
	}

	//added by Andrea Varesio
	pIPMP -> SetAssociatedDecoder((void *) (pES -> m_pDecoder));	//the decoder is got from ES_DescriptorImp

	/*-------------------------------------------------------------
	Open the IPMP channel
	-------------------------------------------------------------*/
	int nBufferSize = decConfigDescr -> bufferSizeDB;
	if (nBufferSize == 0)
		nBufferSize = MPEG4_DEFAULT_IPMP_BUFFER_SIZE;
	m_pChannel = OpenStream (
		pOD,
		nBufferSize,
		MPEG4_OPTIMAL_IPMP_UNIT_SIZE
	);
	if (m_pChannel == NULL)
		return false;

	/*-------------------------------------------------------------
	Setup the IPMP system
	-------------------------------------------------------------*/
	DecoderSpecificInfo *pSpecificInfo = decConfigDescr -> decSpecificInfo;
	LPCBYTE pInfo = NULL;
	int nInfoLength = 0;
	if (pSpecificInfo != NULL)
	{	pInfo = pSpecificInfo -> info;
		nInfoLength = pSpecificInfo -> info.GetLength ();
	}
	pIPMP -> SetIPMPStream (
		m_pInputStream,
		decConfigDescr -> objectTypeIndication,
		pInfo,
		nInfoLength
	);

	/*-------------------------------------------------------------
	Attach the system to the elementary stream
	-------------------------------------------------------------*/
	pES -> AttachIPMP (pIPMP);
	return true;
}

/*-------------------------------------------------------------
Handle a set of IPMP descriptors linked to this stream.
Return true if all descriptors where processed successfully.
-------------------------------------------------------------*/
bool ES_DescriptorImp :: HandleIPMPDescriptors (
	IPMP_DescriptorPointer * const * descPointers,
	int nDescPointersLength,
	InlineImp *pInline
)
{
	/*-------------------------------------------------------------
	Loop through all the pointers and locate the descriptors they are
	pointing to
	-------------------------------------------------------------*/
	IPMPManager *pGenericIPMP = NULL;
	for (int i = 0; i < nDescPointersLength; i++)
	{
		int nID = descPointers [i] -> IPMP_DescriptorID;
		IPMPManager *pIPMP = NULL;
		IPMP_DescriptorImp *pIPMPD = pInline -> GetIPMP_Descriptor (nID);

		/*-------------------------------------------------------------
		If a descriptor pointed by the pointer already exists,
		walk through all the IPMP systems currently attached to this ES.
		-------------------------------------------------------------*/
		if (pIPMPD != NULL)
			for (int j = m_ipmps.GetLength (); --j >= 0;)
				if (m_ipmps [j] -> IsMyType (pIPMPD -> IPMPS_Type))
				{	pIPMP = m_ipmps [j];
					break;
				}
	
		/*-------------------------------------------------------------
		If no system can handle the descriptor, or if the descriptor is
		not known yet, instantiate a generic IPMP system, or use one if
		has already been instantiated before.
		-------------------------------------------------------------*/
		if (!pIPMP)
			pIPMP = pGenericIPMP;
		if (!pIPMP)
		{	pIPMP = pGenericIPMP =
				(IPMPManager *) LoadFilter (ST_IPMPStream, 0);
			if (!pIPMP)
			{	m_pExecutive -> Error (IPMP_NOT_AVAILABLE);
				return false;
			}
			AttachIPMP (pIPMP);
		}

		/*-------------------------------------------------------------
		If a descriptor exists pass it to the IPMP system
		-------------------------------------------------------------*/
		if (pIPMPD)
			switch (pIPMPD -> Notify (pIPMP)) {
			case 0:
				m_pExecutive -> Error (NO_ACCESS_TO_PROTECTED_STREAM);
				return false;
			case -1:
				m_pExecutive -> Error (IPMP_NOT_AVAILABLE);
				return false;
			}

		/*-------------------------------------------------------------
		Associate the IPMP system with the descriptor ID for future
		IPMPDescriptor updates
		-------------------------------------------------------------*/
		pInline -> AssociateIPMPSystem (nID, pIPMP);
	}

	return true;
}

/*-------------------------------------------------------------
Attach an IPMP filter to this elementary stream
-------------------------------------------------------------*/
void ES_DescriptorImp :: AttachIPMP (IPMPManager *pIPMP)
{
	/*-------------------------------------------------------------
	Determine whether this is a pre or post decoding IPMP system.
	Create a MediaStream object with the proper size, depending
	on previous question.
	Link before or after the decoder.
	-------------------------------------------------------------*/
	bool bPostDecoding = pIPMP -> IsPostDecoding ();
	if (bPostDecoding && m_pOutputStream == NULL)
		return;		// post decoding not applicable.
	MediaStreamImp *pStream = new MediaStreamImp (
		m_pInputStream -> GetClockReference (),
		(bPostDecoding ? m_pOutputStream : m_pInputStream) -> GetSize ()
	);
#ifdef STREAM_DELAY //PAB+:
	pStream->SetESD(this); //PAB+:
#endif //STREAM_DELAY

	if (bPostDecoding)
	{	pIPMP -> SetInputStream (m_pOutputStream);
		pIPMP -> SetOutputStream (pStream);
		m_pOutputStream = pStream;
	}
	else
	{	pIPMP -> SetInputStream (m_pInputStream);
		pIPMP -> SetOutputStream (pStream);
		m_pInputStream = pStream;
	}

	/*-------------------------------------------------------------
	Add to the list of IPMP systems
	-------------------------------------------------------------*/
	m_ipmps += pIPMP;
}

/*-------------------------------------------------------------
Start processing the stream described by this ES_Descriptor
-------------------------------------------------------------*/
void ES_DescriptorImp :: Start ()
{
	if (m_error != IM1_OK)
		return;

	if (m_pDecoder && !m_bAttached && m_nDecoderStart++ == 0)
		m_pDecoder -> Start ();

	if (m_nStart++ == 0)
	{	if (m_pChannel)
		{	m_pChannel -> ChannelReady ();
			m_pExecutive -> SetPendingChannels ();
		}

		/*-------------------------------------------------------------
		Start all IPMP systems
		-------------------------------------------------------------*/
		for (int i = 0; i < m_ipmps.GetLength (); i++)
			m_ipmps [i] -> Start ();
	}
}

/*-------------------------------------------------------------
Stop processing the stream described by this ES_Descriptor
-------------------------------------------------------------*/
void ES_DescriptorImp :: Stop ()
{
	/*-------------------------------------------------------------
	Stop all IPMP systems
	-------------------------------------------------------------*/
	if (--m_nStart == 0)
		for (int i = 0; i < m_ipmps.GetLength (); i++)
			m_ipmps [i] -> Stop ();

	/*-------------------------------------------------------------
	Unlock the scene before closing for the case the decoder thread
	is blocking on scene lock. UnlockScene is called with nReserve>0,
	to prevent other threads from modifying the scene.
	-------------------------------------------------------------*/
	m_pExecutive -> UnlockScene (1);

	/*-------------------------------------------------------------
	Stop the decoder and close the channel
	-------------------------------------------------------------*/
	if (m_pDecoder && !m_bAttached && --m_nDecoderStart == 0)
		m_pDecoder -> Stop ();
	if (m_pChannel && m_nStart == 0)
		m_pChannel -> ChannelStop ();

	m_pExecutive -> LockScene (1);
}

/*-------------------------------------------------------------
Detach all the resources created by this ESD
-------------------------------------------------------------*/
void ES_DescriptorImp :: Detach ()
{
	m_nInstantiated = 0;
	delete m_pOutputStream;
	m_pOutputStream = NULL;
	if (m_pDecoder && !m_bAttached)
	{	HMODULE hDLL = m_pDecoder -> GetLibrary ();
		delete m_pDecoder;
		m_pDecoder = NULL;
		m_bAttached = false;
		if (hDLL)
			FreeLibrary (hDLL);
	}	
	delete m_pChannel;
	m_pChannel = NULL;
	delete m_pInputStream;
	m_pInputStream = NULL;

	/*-------------------------------------------------------------
	Delete all IPMP systems
	-------------------------------------------------------------*/
	for (int i = 0; i < m_ipmps.GetLength (); i++)
	{	IPMPManager *pIPMP = m_ipmps [i];
		/*-------------------------------------------------------------
		Loop through the IPMP pointers and disassociate the IPMP
		system from the descriptors
		-------------------------------------------------------------*/
		RemoveIPMPSystems (
			pIPMP,
			ipmpDescrPtr,
			ipmpDescrPtr.GetLength (),
			m_pOD -> GetInline ()
		);

		/*-------------------------------------------------------------
		Loop through the IPMP pointers of the OD and disassociate the IPMP
		system from the descriptors
		-------------------------------------------------------------*/
		RemoveIPMPSystems (
			pIPMP,
			m_pOD -> ipmpDescrPtr,
			m_pOD -> ipmpDescrPtr.GetLength (),
			m_pOD -> GetInline ()
		);

		delete pIPMP -> GetInputStream ();
		HMODULE hDLL = pIPMP -> GetLibrary ();
		delete pIPMP;
		if (hDLL)
			FreeLibrary (hDLL);
	}
	m_ipmps.Clear ();
}

/*-------------------------------------------------------------
Add a new ESD to an active OD
-------------------------------------------------------------*/
MediaStreamImp *ES_DescriptorImp :: AddESD (
	ObjectDescriptorImp *pOD,
	StreamConsumer *pNode
)
{
	return SetDecoder (pOD, pNode);
}

/*-------------------------------------------------------------
When an IPMP system is deleted, detach it from all the descriptors
it was linked to trough IPMP descriptor pointers.
-------------------------------------------------------------*/
void ES_DescriptorImp :: RemoveIPMPSystems (
	IPMPManager *pIPMP,
	IPMP_DescriptorPointer * const * descPointers,
	int nDescPointersLength,
	InlineImp *pInline
)
{
	for (int i = 0; i < nDescPointersLength; i++)
		pInline -> RemoveIPMP (
			descPointers [i] -> IPMP_DescriptorID,
			pIPMP
		);
}

/*-------------------------------------------------------------
Create an OCI decoder for an OCI stream
-------------------------------------------------------------*/
Decoder *ES_DescriptorImp :: CreateOCIDecoder (int version)
{
	SysCoder *pCoder;
	pCoder = new OCIDecoder (
		m_pExecutive,
		m_pExecutive -> GetTraceFile (),
		"OCI",
		m_pExecutive -> GetTraceFlags (),
		version
	);
	return new StreamParser (m_pExecutive, pCoder, NULL);
}