/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

#if !defined(_stream_h)	// Sentry, use file only if it's not already included.
#define _stream_h

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Buffer management
FILE:       Stream.h
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Definition of MediaStream.
-------------------------------------------------------------------------*/
#include <stdio.h>
#include "ztl.h"
#include "Clock.h"
#include "MediaStream.h"

#ifdef STREAM_DELAY //PAB+:
class ES_DescriptorImp; //PAB+:
#endif //STREAM_DELAY

class MediaStreamImp : public MediaStream
{
	int m_nBufferSize, m_nUsedBy;
	LPBYTE m_pBuffer;
	int m_nAllocatedLength, m_nFetched;
	int m_nFreeStart, m_nTakenStart, m_nNextFetch, m_nThreshold;
	ZMutex m_mtxLockStream;
	ZSemaphore m_smpLockOutBlock, m_smpWaitForMemory, m_smpWaitForData;
	ZSemaphore m_smpSending, m_smpReceiving, m_smpTimer;
	ZSemaphore m_smpInUse;
	bool m_bWaitingForMemory, m_bReallocInProgress, m_bWaitingForData;
	bool m_bEndOfStream, m_bFirstRetrieved;
	bool m_bOpenToSend;
	int m_nOpenToReceive, m_nWaiting;
	bool m_bTrace, m_bDynamic, m_bClockActive;
	BOOL m_bWait, m_bShove;
	ClockReference *m_pClock;

	static int m_nStreamCount;
	static FILE *m_fTraceFile;
	static ZMutex m_mtxLockTraceFile;
	int m_nSN;
#ifdef STREAM_DELAY //PAB+:
	bool m_bFirstPass; //PAB+: for my delaying testing hack:
	ES_DescriptorImp *m_pESD; //PAB+:
#endif //STREAM_DELAY

private:
	void CheckThreshold ();
	GetBufferFullness () const;
	void Reset ();
	void Discard ();
	int GetBlockLength (int nOffset)
	{
		return LOADINT (m_pBuffer + nOffset);
	}
	DWORD GetTimeStamp (int nOffset)
		{return LOADDWORD (m_pBuffer + nOffset + sizeof (int));}
	int GetNext (int nOffset);
	void Trace (int nOffset, LPCSTR szAction);

	/*-------------------------------------------------------------
	Interface functions
	-------------------------------------------------------------*/
public:
	int GetSize () const {return m_nBufferSize;}
	virtual LPVOID Allocate (int nSize, BOOL bWait = TRUE, BOOL bShove = FALSE);
	virtual LPVOID Reallocate (int nSize);
	virtual BOOL Dispatch (int nSize, DWORD dwTime, BOOL bEOU = TRUE);
	virtual LPVOID FetchNext (
		int &rnSize,
		DWORD &rdwTime,
		BOOL &rbEOU,
		BOOL bWait = TRUE
	);
	virtual LPVOID Fetch (
		int &rnSize,
		DWORD &rdwTime,
		BOOL &rbEOU,
		int &rnStatus,
		BOOL bSkip = TRUE,
		BOOL bWait = FALSE,
		long lDelta = 0,
		int nFlag = 0
	);
	virtual void Release (BOOL bDiscard = TRUE);
	virtual void Close ();
	virtual void Terminate ();
	virtual void Open ();
	virtual void Start ();
	virtual void Abort () {m_smpTimer.V ();}
	
public:
	void ModifyClock (float fFactor);
	MediaStreamImp (ClockReference *pClock, int nBufferSize, int nThreshold = 0);
	~MediaStreamImp ();
	void SetClockReference (ClockReference *pClock) {m_pClock = pClock;}
	void AdjustClock (DWORD dwTime = 0) {m_pClock -> Adjust (dwTime);}
	ClockReference *GetClockReference () {return m_pClock;}
	DWORD GetClock () {return m_pClock -> GetTime ();}
	void SetTraceFlag (bool bFlag) {m_bTrace = bFlag;}
#ifdef STREAM_DELAY //PAB+:
	ES_DescriptorImp * GetESD() {return m_pESD;} //PAB+:
	ES_DescriptorImp * SetESD(ES_DescriptorImp *pESD) {return m_pESD = pESD;} //PAB+:
#endif //STREAM_DELAY
};

#endif                                      // _stream_h sentry.
