/***********************************************************************
    This software module  was  originally  developed by Zvi  Lifshitz
    (Triton R&D Ltd.), Mauro Rossi (CSELT) and Guido Franceschini
    (CSELT), and edited by Michael Stepping (FTK), Reinhard Bayer
    (GMD Fokus) and Carsten Herpel(THOMSON multimedia).
    in the course of development of the MPEG-4 DMIF (ISO/IEC 14496-6)
    standard.  This software module is an implementation of a part of
    one or more MPEG-4  DMIF  (ISO/IEC 14496-6)  tools  as  specified
    by  the  MPEG-4 DMIF (ISO/IEC 14496-6)  standard.  ISO/IEC  gives
    users of  the  MPEG-4  DMIF  (ISO/IEC 14496-6)  free  license  to
    this software module or modifications thereof for use in hardware
    or software products claiming  conformance  to  the  MPEG-4  DMIF
    (ISO/IEC 14496-6). Those intending to use this software module in
    hardware or software  products  are  advised  that  its  use  may
    infringe   existing  patents.  The  original  developer  of  this
    software module and his/her company, the subsequent  editors  and
    their  companies,  and  ISO/IEC have no liability for use of this
    software module or modifications thereof  in  an  implementation.
    Copyright  is not released for non MPEG-4 DMIF  (ISO/IEC 14496-6)
    conforming  products. Triton R&D Ltd. and CSELT
    retain  full  right to use the code for their own purpose, assign
    or donate the code to a third party and to inhibit third  parties
    from  using  the  code  for  non  MPEG-4  DMIF  (ISO/IEC 14496-6)
    conforming products. This copyright notice must  be  included  in
    all copies or derivative works. Copyright  1998.
************************************************************************/

// Contact: guido.franceschini@cselt.it

/**************************************************************************
 * Project Im1Player
 * INTERNATIONAL STANDARD ORGANIZATION
 * ISO/IEC JTC1/SC29/WG11
 *
 * SUBSYSTEM: DMIF
 * FILE:      DMIFClientFilter.cpp
 * AUTHOR:    Mauro Rossi (CSELT)
 * OVERVIEW:  Implementation of DMIFClientFilter, the class that implements the
 *			  generic layer of DMIF.
 *
 **************************************************************************/

#include "defs.h"

// no longer valid for VC7 and gcc3.2
//#include <iostream.h>

#include "DMIFClientFilter.h"
#ifdef __UNIX
#include "unistd.h"
#endif // __UNIX
//#include <fstream.h>
//ofstream	logBase,log0,log1,log2,log3;

/*-------------------------------------------------------------
The exported function that instantiates the proper filter
-------------------------------------------------------------*/
extern "C" DMIF_Filter* CreateInstance ()
{
	return new DMIFClientFilter();
}

DMIFClientFilter :: DMIFClientFilter () :
	m_nChannelId(0)
{
	m_nServiceId = 1;

//	logBase.attach(stdout->_file);//screen log
//	log0.clear();
//	log1.clear();
//	log2.clear();
//	log3.clear();
//	log0.attach(logBase.fd());
//	log1.attach(logBase.fd());
//	log2.attach(logBase.fd());
//	log3.attach(logBase.fd());
}

/*-------------------------------------------------------------
DMIFClientFilter destructor. Delete the assoicated Service, and all FlexDemux
objects (DataChannels are deleted by the application)
-------------------------------------------------------------*/
DMIFClientFilter :: ~DMIFClientFilter ()
{
	int i;
	int n = 0;
	struct serviceDesc* pServiceDesc;

	while (m_loadedServices.GetLength ()) {
		if (n++)
			Sleep(1000);
		m_smpServices.P();
		for (i = m_loadedServices.GetLength (); --i >= 0;)

		{
			pServiceDesc = m_loadedServices[i];
			pServiceDesc->m_mtxService.P();
			if (pServiceDesc->m_nUse == 0) {
				if (pServiceDesc->m_szAbsURL)
					delete (pServiceDesc->m_szAbsURL);
				pServiceDesc->m_szAbsURL = NULL;
				m_loadedServices.DeleteFrom(i);
				m_smpServices.V ();

				delete pServiceDesc->m_pService;
#ifdef WIN32
				if (pServiceDesc->m_hDLL)
					FreeLibrary (pServiceDesc->m_hDLL);
#endif
				delete pServiceDesc;
				m_smpServices.P ();
			} else {
				pServiceDesc->m_mtxService.V();
			}
		}
		m_smpServices.V ();
	}
}

struct serviceDesc* DMIFClientFilter :: LockService (const Service* pService)
{
	int i;
	struct serviceDesc* pServiceDesc=NULL;

	m_smpServices.P();
	for (i = m_loadedServices.GetLength (); --i >= 0;)
	{
		if (m_loadedServices[i]->m_pService == pService)
		{
			pServiceDesc = m_loadedServices[i];
			pServiceDesc->m_mtxService.P();
			pServiceDesc->m_nUse++;
			pServiceDesc->m_mtxService.V();
			m_smpServices.V();
			break;
		}
	}
	if (pServiceDesc==NULL)
		m_smpServices.V();
	return (pServiceDesc);
}

struct serviceDesc* DMIFClientFilter :: LockService (const UINT nServiceId)
{
	int i;
	struct serviceDesc* pServiceDesc=NULL;

	m_smpServices.P();
	for (i = m_loadedServices.GetLength (); --i >= 0;)
	{
		if (m_loadedServices[i]->m_nServiceId == nServiceId)
		{
			pServiceDesc = m_loadedServices[i];
			pServiceDesc->m_mtxService.P();
			pServiceDesc->m_nUse++;
			pServiceDesc->m_mtxService.V();
			m_smpServices.V();
			break;
		}
	}
	if (pServiceDesc==NULL)
		m_smpServices.V();
	return (pServiceDesc);
}

void DMIFClientFilter :: UnlockService (struct serviceDesc* pServiceDesc)
{
	m_smpServices.P();
	pServiceDesc->m_mtxService.P();
	pServiceDesc->m_nUse--;
	pServiceDesc->m_mtxService.V();
	m_smpServices.V();
}

#ifdef __UNIX
// forward declaration to avoid including the whole MP4-Stuff
/*extern "C" Service *CreateMP4FileReaderInstance(); */
extern "C" void *CreateMP4FileReaderInstance();
#endif

/*-------------------------------------------------------------
ServiceAttach creates a proper Service object and calls its
ServiceAttach method
-------------------------------------------------------------*/
void DMIFClientFilter::DAI_ServiceAttach_Req(
	const UINT nTmpId,
	const UINT nParentServiceId,
	const char* szRelURL,
	const BYTE* uuData,
	const u_short uuData_length)
{
	int i,nSize;
	char szBuffer [100];
	Service* pService = NULL;
	HMODULE	hDLL = NULL;
	RESPONSE eResponse = RESPONSE_SERVICEDLL_NOT_FOUND;

	/*-------------------------------------------------------------
	If a parentServiceId is specified, possibly expand the URL
	-------------------------------------------------------------*/
	char* szAbsURL = NULL;
	if (nParentServiceId)
	{
		serviceDesc* pServiceDesc=LockService(nParentServiceId);
		nSize = strlen(szRelURL) + strlen(pServiceDesc->m_szAbsURL) + 1;
		szAbsURL = new char[nSize];
		ZExpandURL (pServiceDesc->m_szAbsURL, szRelURL, szAbsURL, nSize);
		UnlockService(pServiceDesc);
	}
	else
	{
		szAbsURL = new char[strlen(szRelURL) + 1];
		strcpy(szAbsURL, szRelURL);
	}

	m_smpServices.P ();
	/*-------------------------------------------------------------
	Try all registered services.
	-------------------------------------------------------------*/
#ifdef WIN32
	static ZRegistry registry("DMIF\\Instances");
	for (i = 0; registry.Enumerate (i, szBuffer, sizeof szBuffer); ++i)
	{
		pService = (Service*) registry.CreateInstance (szBuffer, hDLL);
		if (pService)
		{
			if (pService -> IsYourURL (szAbsURL))
				break;
			else
			{
				delete pService;
				FreeLibrary (hDLL);
				pService = NULL;
				hDLL = NULL;
			}
		}
		else
		{
			eResponse = RESPONSE_SERVICEDLL_ERROR;
		}
	}
#endif
#ifdef __UNIX
// this "CreateInstance" is hard coded to load the mp4 library.
		pService = (Service*) CreateMP4FileReaderInstance() ;
		if (pService)
		{
			if (pService -> IsYourURL (szAbsURL))
				;
			else
			{
				delete pService;
				pService = NULL;
				eResponse = RESPONSE_SERVICEDLL_ERROR;
			}
		}
		else
		{
				eResponse = RESPONSE_SERVICEDLL_ERROR;
		}
#endif


	/*-------------------------------------------------------------
	Add the new service to the service list, and call its ServiceAttach
	method. Or, if service could not be instantiated, report to the
	application ServiceDetach.
	-------------------------------------------------------------*/
	if (pService)
	{
		struct serviceDesc* pServiceDesc = new(struct serviceDesc);
		pServiceDesc->m_eStatus = 0;	// SERVICE_ACTIVE
		pServiceDesc->m_nUse = 0;
		pServiceDesc->m_hDLL = hDLL;
		pServiceDesc->m_pService = pService;
		pServiceDesc->m_nTmpId = nTmpId;
		pServiceDesc->m_nServiceId = m_nServiceId++;
		pServiceDesc->m_szAbsURL = new char[strlen(szAbsURL) + 1];
		strcpy(pServiceDesc->m_szAbsURL, szAbsURL);
		m_loadedServices += pServiceDesc;
	}
	m_smpServices.V ();
	if (pService)
	{
		pService -> SetSink (this);
		pService -> DPI_ServiceAttach_Req(szAbsURL, uuData, uuData_length);
	}
	else
	{
		char s[_MAX_PATH + 64];
		strcpy(s,"Couldn't find the DLL managing the service: \n");
		strcat(s,szAbsURL);
#ifdef WIN32
		// done by Jean
//		MessageBox(NULL,s,"DMIF Error",MB_OK | MB_ICONERROR | MB_SETFOREGROUND);
#endif
#ifdef __UNIX
		printf("%s", s);
#endif

//		log0<<"ERROR: DMIFClientFilter::DAI_ServiceAttach_Req - Service not found!"<<endl;
		GetSink () -> DAI_ServiceAttach_Cnf(nTmpId,0,NULL,0,eResponse);
	}
	if(szAbsURL)
	{
		delete szAbsURL;
		szAbsURL=NULL;
	}
	return;
}

/*-------------------------------------------------------------
Pass a ServiceDetach call to the service
-------------------------------------------------------------*/
void DMIFClientFilter::DAI_ServiceDetach_Req(
	const UINT nServiceId,
	const REASON reason)
{
	serviceDesc* pServiceDesc=LockService(nServiceId);

	if(pServiceDesc && pServiceDesc->m_pService)
	{
		pServiceDesc->m_pService -> DPI_ServiceDetach_Req(reason);
	}
	else
	{
		GetSink () -> DAI_ServiceDetach_Cnf(nServiceId,RESPONSE_SERVICE_DOES_NOT_EXIST);
	}
	if(pServiceDesc)
		UnlockService(pServiceDesc);
	return;
}

/*-------------------------------------------------------------
Pass a ChannelAdd call to the service
-------------------------------------------------------------*/
void DMIFClientFilter::DAI_ChannelAdd_Req(
	const UINT nServiceId,
	const DAIChannelAddParam& reqParam)
{
	serviceDesc* pServiceDesc=LockService(nServiceId);

	if(pServiceDesc && pServiceDesc->m_pService)
	{
		for(int i=0;i<reqParam.GetLength();i++)
		{
			reqParam[i].m_nChid = NewChannelId();  //(UINT)new MyChannel(GetSink());
		}
		// this local parameter loop is passed to specific service
		pServiceDesc->m_pService -> DPI_ChannelAdd_Req(reqParam);
		// after returning, acces to this loop is no longer allowed
	}
	else
	{
		DAIChannelAddParam cnfParam = reqParam;
		for(int i=0;i<cnfParam.GetLength();i++)
		{
			cnfParam[i].m_nChid = 0;
			cnfParam[i].m_eResponse = RESPONSE_SERVICE_DOES_NOT_EXIST;
		}
		GetSink () -> DAI_ChannelAdd_Cnf(nServiceId,cnfParam);
		cnfParam.Clear();
	}
	if(pServiceDesc)
		UnlockService(pServiceDesc);
	return;
}

/*-------------------------------------------------------------
Pass a ChannelDelete call to the service
-------------------------------------------------------------*/
void DMIFClientFilter::DAI_ChannelDelete_Req(
	const UINT nServiceId,
	const DAIChannelDeleteParam& reqParam)
{
	int i;
	DAIChannelDeleteParam cnfParam,dpiReqParam;

	serviceDesc* pServiceDesc=LockService(nServiceId);

	if(pServiceDesc && pServiceDesc->m_pService)
	{
		for(i=0;i<reqParam.GetLength();i++)
		{
			dpiReqParam += reqParam[i];
		}
		if(dpiReqParam.GetLength() > 0) {
			pServiceDesc->m_pService->DPI_ChannelDelete_Req(dpiReqParam);
			dpiReqParam.Clear();
		}
	}
	else
	{
		for(i=0;i<reqParam.GetLength();i++)
		{
			cnfParam += reqParam[i];
			cnfParam[i].m_eResponse = RESPONSE_SERVICE_DOES_NOT_EXIST;
		}
		GetSink () -> DAI_ChannelDelete_Cnf(nServiceId,cnfParam);
		cnfParam.Clear();
	}
	if(pServiceDesc)
		UnlockService(pServiceDesc);
	return;
}

/*-------------------------------------------------------------
Pass a UserCommand call to the service
-------------------------------------------------------------*/
void DMIFClientFilter::DAI_UserCommand_Req(const UINT nServiceId, const DAIUserCmdParam& inParam)
{
	serviceDesc* pServiceDesc=LockService(nServiceId);

	if(pServiceDesc && pServiceDesc->m_pService)
	{
		pServiceDesc->m_pService->DPI_UserCommand_Req(inParam);
	}
	if(pServiceDesc)
		UnlockService(pServiceDesc);
	return;
}

/*-------------------------------------------------------------
Pass a UserCommandAck call to the service
-------------------------------------------------------------*/
void DMIFClientFilter::DAI_UserCommandAck_Req(const UINT nServiceId, const DAIUserCmdParam& inParam)
{
	serviceDesc* pServiceDesc=LockService(nServiceId);

	if(pServiceDesc && pServiceDesc->m_pService)
	{
		pServiceDesc->m_pService->DPI_UserCommandAck_Req(inParam);
	}
	else
	{
		DAIUserCmdParam outParam;
		outParam.m_nChLen=0;
		outParam.m_pChid=NULL;
		outParam.m_pData=NULL;
		outParam.m_nDataLen=0;
		outParam.m_nCmdId=inParam.m_nCmdId;
		outParam.m_eResponse=RESPONSE_SERVICEDLL_NOT_FOUND;
		GetSink () -> DAI_UserCommandAck_Cnf(nServiceId,outParam);
	}
	if(pServiceDesc)
		UnlockService(pServiceDesc);
	return;
}

void DMIFClientFilter::DAI_DataSend_Req(
	const UINT				nServiceId,
	const unsigned int		nChid,
	const BYTE*				pStreamData,
	const int				nStreamDataLength,
	const BYTE*				pAppData,
	const int				nAppDataLength)
{
	serviceDesc* pServiceDesc=LockService(nServiceId);

	if(pServiceDesc && pServiceDesc->m_pService)
	{
		pServiceDesc->m_pService->DPI_DataSend_Req(nChid,pStreamData,nStreamDataLength,pAppData,nAppDataLength);
	}
	if(pServiceDesc)
		UnlockService(pServiceDesc);
	return;
}

////
void DMIFClientFilter::DAI_ServiceAttach_Rsp(
		const UINT		nServiceId,
		const BYTE*		uuData,
		const u_short	uuData_Length,
		const RESPONSE	eResponse)
{
	nServiceId; uuData; uuData_Length; eResponse;
}

void DMIFClientFilter::DAI_ServiceDetach_Rsp(
		const UINT		nServiceId,
		const RESPONSE	eResponse)
{
	serviceDesc* pServiceDesc = LockService(nServiceId);

	pServiceDesc->m_pService -> DPI_ServiceDetach_Rsp(eResponse);

	UnlockService(pServiceDesc);
	return;
}

void DMIFClientFilter::DAI_ChannelAdd_Rsp(
		const UINT		nServiceId,
		const DAIChannelAddParam& outParam)
{
	nServiceId; outParam;
}

void DMIFClientFilter::DAI_TransmuxAdd_Rsp(
		const UINT		nServiceId,
		const DAIChannelAddParam& outParam)
{
	nServiceId; outParam;
}

void DMIFClientFilter::DAI_ChannelDelete_Rsp(
		const UINT		nServiceId,
		const DAIChannelDeleteParam& outParam)
{
	nServiceId; outParam;
}

void DMIFClientFilter::DAI_UserCommandAck_Rsp(
		const UINT		nServiceId,
		const DAIUserCmdParam& outParam)
{
	serviceDesc* pServiceDesc = LockService(nServiceId);

	if (pServiceDesc && pServiceDesc->m_pService) {
		pServiceDesc->m_pService->DPI_UserCommandAck_Rsp(outParam);
		UnlockService(pServiceDesc);
	}
}


/*-------------------------------------------------------------
Pass ServiceAttach callback to the application
-------------------------------------------------------------*/
void DMIFClientFilter :: DPI_ServiceAttach_Cnf (
	const Service* pService,
	const BYTE *pUuData,
	const int nUuDataLen,
	const RESPONSE eResponse
)
{
	serviceDesc* pServiceDesc = LockService(pService);

	if (pServiceDesc)
	{
		GetSink () -> DAI_ServiceAttach_Cnf(pServiceDesc->m_nTmpId,pServiceDesc->m_nServiceId,pUuData,nUuDataLen,eResponse);
		UnlockService(pServiceDesc);
	}
	return;
}

/*-------------------------------------------------------------
Pass ServiceDetach callback to the application
-------------------------------------------------------------*/
void DMIFClientFilter :: DPI_ServiceDetach_Cnf (
	const Service* pService,
	const RESPONSE eResponse)
{
	serviceDesc* pServiceDesc=LockService(pService);

	if (pServiceDesc)
	{
		GetSink () -> DAI_ServiceDetach_Cnf(pServiceDesc->m_nServiceId,eResponse);
		UnlockService(pServiceDesc);
	}
	return;
}

/*-------------------------------------------------------------
A FlexMux channel is added to an existing TransMux channel
-------------------------------------------------------------*/
void DMIFClientFilter :: DPI_ChannelAdd_Cnf (
	const Service* pService,
	const StreamMapElement &outParam,
	const RESPONSE eResponse
)
{
	serviceDesc* pServiceDesc=LockService(pService);

	if (pServiceDesc)
	{
		DAIChannelAddParam cnfParam;
		DAIChannelAddElement* pCnfEl = new DAIChannelAddElement();
		pCnfEl->m_nChid = outParam.m_nChid;
		pCnfEl->m_nTmpId = outParam.m_nTmpId;
		pCnfEl->m_eResponse = eResponse;
		pCnfEl->m_pQoSStream = NULL;		// unused field
		pCnfEl->m_eDirection = UNSPECIFIED;	// useless field; just avoid random values
		pCnfEl->m_nDataLen = outParam.m_nDataLen;
		if(pCnfEl->m_nDataLen > 0)
		{
			pCnfEl->m_pData = new BYTE[pCnfEl->m_nDataLen];
			memcpy(pCnfEl->m_pData,outParam.m_pData,pCnfEl->m_nDataLen);
		}
		else
			pCnfEl->m_pData = NULL;

		pCnfEl->m_nAppInfoLen = outParam.m_nAppInfoLen;
		if(pCnfEl->m_nAppInfoLen > 0)
		{
			pCnfEl->m_pAppInfo = new BYTE[pCnfEl->m_nAppInfoLen];
			memcpy(pCnfEl->m_pAppInfo, outParam.m_pAppInfo, pCnfEl->m_nAppInfoLen);
		}
		else
			pCnfEl->m_pAppInfo = NULL;

		cnfParam += *pCnfEl;

		if(pCnfEl->m_pData)
		{
			delete[] pCnfEl->m_pData;
			pCnfEl->m_pData = NULL;
		}
		if(pCnfEl->m_pAppInfo)
		{
			delete[] pCnfEl->m_pAppInfo;
			pCnfEl->m_pAppInfo = NULL;
		}
		delete pCnfEl;
		pCnfEl = NULL;

		GetSink() -> DAI_ChannelAdd_Cnf(pServiceDesc->m_nServiceId,cnfParam);

		for(int i=0;i<cnfParam.GetLength();i++)
		{
			if(cnfParam[i].m_nDataLen > 0)
				delete[] cnfParam[i].m_pData;
			if(cnfParam[i].m_nAppInfoLen > 0)
				delete[] cnfParam[i].m_pAppInfo;
		}

		UnlockService(pServiceDesc);
	}
	return;
}

/*-------------------------------------------------------------
A FlexMux channel is removed from an existing TransMux channel
-------------------------------------------------------------*/
void DMIFClientFilter :: DPI_ChannelDelete_Cnf (
	const Service* pService,
	const UINT		nChid,
	const RESPONSE eResponse
)
{
	serviceDesc* pServiceDesc=LockService(pService);

	if (pServiceDesc)
	{
		DAIChannelDeleteParam cnfParam;
		DAIChannelDeleteElement cnfEl;

		cnfEl.m_eReason   = REASON_UNKNOWN_ERROR ; // useless field; just avoid random values
		cnfEl.m_nChid	  = nChid;
		cnfEl.m_eResponse = eResponse;
		// MS 20020223
		cnfEl.m_nTmpId = 0 ;

		cnfParam += cnfEl;

		GetSink () -> DAI_ChannelDelete_Cnf(pServiceDesc->m_nServiceId,cnfParam);
		UnlockService(pServiceDesc);

		cnfParam.Clear();
	}
	return;
}

/*-------------------------------------------------------------
The answer to the UserCommand is received
-------------------------------------------------------------*/
void DMIFClientFilter :: DPI_UserCommandAck_Cnf(
	const Service* pService,
	const DAIUserCmdParam& outParam)
{
	serviceDesc* pServiceDesc=LockService(pService);

	if (pServiceDesc)
	{
		GetSink()->DAI_UserCommandAck_Cnf(pServiceDesc->m_nServiceId,outParam);
		UnlockService(pServiceDesc);
	}
	return;
}



/*-----------------------------------------------------------------
Pass a received data packet to the proper channel object
-----------------------------------------------------------------*/
void DMIFClientFilter :: DPI_DataReceive_Ind (
	const Service*	pService,
	const UINT		m_nChid,
	const BYTE*		pStreamData,
	const int		nStreamDataLength,
	const BYTE*		pAppData,
	const int		nAppDataLength,
	const int		nErr
)
{
	serviceDesc* pServiceDesc=LockService(pService);

	if (pServiceDesc)
	{
		//[Michael Stepping 2001] to allow also DATA_EOF
		//int eFlag = nErr ? DATA_CORRUPTED : DATA_OK;
		const int eFlag = (DATA_STATUS) nErr ;

		GetSink()->DAI_DataReceive_Ind(m_nChid,pStreamData,nStreamDataLength,pAppData,nAppDataLength,eFlag);
		UnlockService(pServiceDesc);
	}
	return;
}


void DMIFClientFilter :: DPI_ServiceAttach_Ind(
		const Service*		pService,
		const char*			serviceParam,
		const BYTE*			pUuDataInBuffer,
		const int			nUuDataInLen)
{
	pService; serviceParam; pUuDataInBuffer; nUuDataInLen;
}

void DMIFClientFilter :: DPI_ServiceDetach_Ind(
		const Service*		pService,
		const REASON		eReason)
{
	serviceDesc* pServiceDesc=LockService(pService);

	if (pServiceDesc)
	{
		GetSink() -> DAI_ServiceDetach_Ind(pServiceDesc->m_nServiceId, eReason);
		UnlockService(pServiceDesc);
	}
	return;
}

void DMIFClientFilter :: DPI_ChannelAdd_Ind(
		const Service*		pService,
		const DAIChannelAddParam& inParam)
{
	pService; inParam;
}

void DMIFClientFilter :: DPI_ChannelDelete_Ind(
		const Service*		pService,
		const DAIChannelDeleteParam& inParam)
{
	pService; inParam;
}

void DMIFClientFilter :: DPI_UserCommand_Ind(
		const Service*		pService,
		const DAIUserCmdParam&	inParam)
{
	serviceDesc* pServiceDesc=LockService(pService);

	if (pServiceDesc)
	{
		GetSink() -> DAI_UserCommand_Ind(pServiceDesc->m_nServiceId,inParam);
		UnlockService(pServiceDesc);
	}
	return;
}

void DMIFClientFilter :: DPI_UserCommandAck_Ind(
		const Service*		pService,
		const DAIUserCmdParam&	inParam)
{
	serviceDesc* pServiceDesc=LockService(pService);

	if (pServiceDesc)
	{
		GetSink() -> DAI_UserCommandAck_Ind(pServiceDesc->m_nServiceId,inParam);
		UnlockService(pServiceDesc);
	}
	return;
}


///////////////////////////


UINT DMIFClientFilter::NewChannelId()
{
	return ++m_nChannelId;
}

///////////////////////////////////////////
/// UTILITY FUNCTIONs
///////////////////////////////////////////

/*
Michael Stepping:
all the log stuff has to be removed from functions below,
leads to problems in VC7, because streams are now in STL
instead of CLIB
*/
/*
int GetQOSSTREAMLength(QOSSTREAM qos)
{
	int ret;

	if( qos == NULL)
	{
		log2<<"qos NULL"<<endl;
		return 0;
	}

	if( qos[0] != 0xC )
	{
		log0<<"qos[0] != 0xC"<<endl;
		return -1;  // It is not a QOS descriptor;
	}

	if( qos[1] & 0x80 )
	{
		log0<<"qos TOO long"<<endl;
		return -1; // Qos length greater than 127+2 bytes. Not yet supported.
	}

	ret = 2 + (int)( qos[1] & 0x7F );

	return ret;
}

void PrintQOSSTREAMString(QOSSTREAM qos)
{
	int length = GetQOSSTREAMLength(qos);
	int i;

	if(qos==NULL)
	{
		log2<<" EMPTY (QOS == NULL)"<<endl;
		return;
	}

	if(length == -1)
	{
		log2<<" EMPTY (NOT QOS DESCRIPTOR)"<<endl;
		return;
	}
	if(length==0)
	{
		log2<<" EMPTY (LENGTH == 0)"<<endl;
		return;
	}
	log2<<hex;

	for(i=0;i<length;i++) // qos[0] and qos[1] = 2bytes
	{
		log2<<(unsigned int) (qos[i])<<".";
	}
	log2<<dec<<endl;
	for(i=3;i<length;i++) // qos[0] and qos[1] = 2bytes
	{
		//log2 << i<< endl;;
		//log2 << "qos = " << (unsigned int) qos[i] << endl;
		switch (qos[i])
		{
		case 0x01:{
				log2 << "MAX_DELAY = ";
				union vv {
					BYTE		 bvalue[4];
					unsigned int ivalue;
				}v;
				v.bvalue[0] = 0x00;
				v.ivalue = 0;
				int l = qos[++i];
				for (int t = 0; t<l; t++)
					v.bvalue[l-1-t] = qos[i+1+t];
				i += l;
				log2 << "*"<< (unsigned int )v.ivalue << "*"<< endl;
				//for (t = 0; t<4; t++)
				//	log2 << (unsigned int )v.bvalue[t] << "-";
				break;}
		case 0x02:{
				log2 << "AVG_DELAY = ";
				union vv {
					BYTE		 bvalue[4];
					unsigned int ivalue;
				}v;
				v.bvalue[0] = 0x00;
				v.ivalue = 0;
				int l = qos[++i];
				for (int t = 0; t<l; t++)
					v.bvalue[l-1-t] = qos[i+1+t];
				i += l;
				log2 << "*"<< (unsigned int )v.ivalue << "*"<< endl;
				break;}
		case 0x03:{
				log2 << "LOSS_PROB = ";
				union vv {
					BYTE	 bvalue[4];
					float	 ivalue;
				}v;
				v.bvalue[0] = 0x00;
				v.ivalue = 0;
				int l = qos[++i];
				for (int t = 0; t<l; t++)
					v.bvalue[l-1-t] = qos[i+1+t];
				i += l;
				log2 << "*"<< (double )v.ivalue << "*"<< endl;
				break;}
		case 0x04:{
				log2 << "MAX_GAP_LOSS = ";
				union vv {
					BYTE		 bvalue[4];
					unsigned int ivalue;
				}v;
				v.bvalue[0] = 0x00;
				v.ivalue = 0;
				int l = qos[++i];
				for (int t = 0; t<l; t++)
					v.bvalue[l-1-t] = qos[i+1+t];
				i += l;
				log2 << "*"<< (unsigned int )v.ivalue << "*"<< endl;
				break;}
		case 0x41:{
				log2 << "MAX_AU_SIZE = ";
				union vv {
					BYTE		 bvalue[4];
					unsigned int ivalue;
				}v;

				v.bvalue[0] = 0x00;
				v.ivalue = 0;
				int l = qos[++i];
				for (int t = 0; t<l; t++)
					v.bvalue[l-1-t] = qos[i+1+t];
				i += l;
				log2 << "*"<< (unsigned int )v.ivalue << "*"<< endl;
				break;}
		case 0x42:{
				log2 << "AVG_AU_SIZE = ";
				union vv {
					BYTE		 bvalue[4];
					unsigned int ivalue;
				}v;
				v.bvalue[0] = 0x00;
				v.ivalue = 0;
				int l = qos[++i];
				for (int t = 0; t<l; t++)
					v.bvalue[l-1-t] = qos[i+1+t];
				i += l;
				log2 << "*"<< (unsigned int )v.ivalue << "*"<< endl;
				break;}
		case 0x43:{
				log2 << "MAX_AU_RATE = ";
				union vv {
					BYTE		 bvalue[4];
					unsigned int ivalue;
				}v;
				v.bvalue[0] = 0x00;
				v.ivalue = 0;
				int l = qos[++i];
				for (int t = 0; t<l; t++)
					v.bvalue[l-1-t] = qos[i+1+t];
				i += l;
				log2 << "*"<< (unsigned int )v.ivalue << "*"<< endl;
				break;}
		default:{
				log2 << "UNKNOWN QUALIFIER " << endl;
				int l = qos[++i];
				i = i+l;
				break;}
		}
	}
}


*/

