/***********************************************************************
    This software module  was  originally  developed by Zvi  Lifshitz
    (Triton R&D Ltd.), Mauro Rossi (CSELT) and Guido Franceschini
    (CSELT), and edited by Michael Stepping (FTK)
    in the course of development of the MPEG-4 DMIF (ISO/IEC 14496-6)
    standard.  This software module is an implementation of a part of
    one or more MPEG-4  DMIF  (ISO/IEC 14496-6)  tools  as  specified
    by  the  MPEG-4 DMIF (ISO/IEC 14496-6)  standard.  ISO/IEC  gives
    users of  the  MPEG-4  DMIF  (ISO/IEC 14496-6)  free  license  to
    this software module or modifications thereof for use in hardware
    or software products claiming  conformance  to  the  MPEG-4  DMIF 
    (ISO/IEC 14496-6). Those intending to use this software module in
    hardware or software  products  are  advised  that  its  use  may
    infringe   existing  patents.  The  original  developer  of  this
    software module and his/her company, the subsequent  editors  and
    their  companies,  and  ISO/IEC have no liability for use of this
    software module or modifications thereof  in  an  implementation.
    Copyright  is not released for non MPEG-4 DMIF  (ISO/IEC 14496-6)
    conforming  products. Triton R&D Ltd. and CSELT
    retain  full  right to use the code for their own purpose, assign
    or donate the code to a third party and to inhibit third  parties
    from  using  the  code  for  non  MPEG-4  DMIF  (ISO/IEC 14496-6)
    conforming products. This copyright notice must  be  included  in
    all copies or derivative works. Copyright  1998.
************************************************************************/

// Contact: guido.franceschini@cselt.it

/**************************************************************************
 * Project Im1Player
 * INTERNATIONAL STANDARD ORGANIZATION
 * ISO/IEC JTC1/SC29/WG11
 * 
 * SUBSYSTEM: DMIF
 * FILE:      DMIFClientFilter.h
 * AUTHOR:    Mauro Rossi (CSELT)
 * OVERVIEW:  Definitions of DMIFClientFilter, the class that implements
 *			  the generic layer of DMIF.
 * 
 **************************************************************************/

#if !defined(_DMIFClientFilter_h)	// Sentry, use file only if it's not already included.
#define _DMIFClientFilter_h

#include "DAI.h"
#include "DPI.h"
#include "MList.h"

extern int GetQOSSTREAMLength(QOSSTREAM qos);


struct serviceDesc
{
	UINT				m_nServiceId;
	UINT				m_nUse;
	UINT				m_nTmpId;
	char*				m_szAbsURL;
	Service*			m_pService;
	HMODULE				m_hDLL;
	ZMutex				m_mtxService;
	UINT				m_eStatus;
};

class DMIFClientFilter : public DMIF_Filter, public ServiceSink
{
public:
	DMIFClientFilter ();
	virtual ~DMIFClientFilter ();

	//DMIF_Filter - client side
	virtual void DAI_ServiceAttach_Req(
		const UINT		nTmpId,
		const UINT		nParentServiceId,
		const char*		szAbsURL,
		const BYTE*		uuData,
		const u_short	uuData_length
	);
	virtual void DAI_ServiceDetach_Req(
		const UINT		nServiceId,
		const REASON	eReason
	);
	virtual void DAI_ChannelAdd_Req(
		const UINT		nServiceId,
		const DAIChannelAddParam& inParam
	);
	virtual void DAI_ChannelDelete_Req(
		const UINT		nServiceId,
		const DAIChannelDeleteParam& inParam
	);
	virtual void DAI_UserCommand_Req(
		const UINT		nServiceId,
		const DAIUserCmdParam& inParam
	);
	virtual	void DAI_UserCommandAck_Req(
		const UINT		nServiceId,
		const DAIUserCmdParam& inParam
	);
	virtual void DAI_DataSend_Req(
		const UINT				nServiceId,
		const unsigned int		nChid,
		const BYTE*				pStreamData,
		const int				nStreamDataLength,
		const BYTE*				pAppData,
		const int				nAppDataLength
	);
	////
	virtual void DAI_ServiceAttach_Rsp(
		const UINT		nServiceId,
		const BYTE*		uuData,
		const u_short	uuData_Length,
		const RESPONSE	eResponse
	);
	virtual void DAI_ServiceDetach_Rsp(
		const UINT		nServiceId,
		const RESPONSE	eResponse
	);
	virtual void DAI_ChannelAdd_Rsp(
		const UINT		nServiceId,
		const DAIChannelAddParam& outParam
	);
	virtual void DAI_TransmuxAdd_Rsp(
		const UINT		nServiceId,
		const DAIChannelAddParam& outParam
	);
	virtual void DAI_ChannelDelete_Rsp(
		const UINT		nServiceId,
		const DAIChannelDeleteParam& outParam
	);
	virtual	void DAI_UserCommandAck_Rsp(
		const UINT		nServiceId,
		const DAIUserCmdParam& outParam
	);

	//ServiceClientSink (DPI)
	virtual void DPI_ServiceAttach_Cnf (
		const Service*			pService,
		const BYTE*				pUuData,
		const int				nUuDataLen,
		const RESPONSE			eResponse
	);
	virtual void DPI_ServiceDetach_Cnf (
		const Service*			pService,
		const RESPONSE			eResponse
	);
	virtual void DPI_ChannelAdd_Cnf (
		const Service*			pService,
		const StreamMapElement	&outParam,
		const RESPONSE			eResponse
	);
	virtual void DPI_ChannelDelete_Cnf (
		const Service*			pService,
		const UINT				nChid,
		const RESPONSE			eResponse
	);
	virtual void DPI_UserCommandAck_Cnf (
		const Service*			pService,
		const DAIUserCmdParam&	outParam
	);
	virtual void DPI_DataReceive_Ind (
		const Service*			pService,
		const UINT				m_nChid,
		const BYTE*				pStreamData,
		const int				nStreamDataLength,
		const BYTE*				pAppData,
		const int				nAppDataLength,
		const int				nErr
	);
	virtual void DPI_ServiceAttach_Ind(
		const Service*		pService,
		const char*				serviceParam, 
		const BYTE*				pUuDataInBuffer,
		const int				nUuDataInLen
	);
	virtual void DPI_ServiceDetach_Ind(
		const Service*		pService,
		const REASON			eReason
	);
	virtual void DPI_ChannelAdd_Ind(
		const Service*		pService,
		const DAIChannelAddParam& inParam
	);
	virtual void DPI_ChannelDelete_Ind(
		const Service*		pService,
		const DAIChannelDeleteParam& inParam
	);
	virtual void DPI_UserCommand_Ind(
		const Service*		pService,
		const DAIUserCmdParam&	inParam
	);
	virtual void DPI_UserCommandAck_Ind(
		const Service*		pService,
		const DAIUserCmdParam&	inParam
	);

private:

	UINT		m_nServiceId;
	ZSemaphore	m_smpServices;
	MList<struct serviceDesc> m_loadedServices;
	struct serviceDesc* LockService(const Service* pService);
	struct serviceDesc* LockService(const UINT nServiceId);
	void UnlockService(struct serviceDesc* pServiceDesc);
	UINT	NewChannelId();
	UINT	m_nChannelId;

};

#endif
