/***********************************************************************
    This software module  was  originally  developed by Zvi  Lifshitz
    (Triton R&D Ltd.), Mauro Rossi (CSELT) and Guido Franceschini
    (CSELT), and edited by Michael Stepping (FTK)
    in the course of development of the MPEG-4 DMIF (ISO/IEC 14496-6)
    standard.  This software module is an implementation of a part of
    one or more MPEG-4  DMIF  (ISO/IEC 14496-6)  tools  as  specified
    by  the  MPEG-4 DMIF (ISO/IEC 14496-6)  standard.  ISO/IEC  gives
    users of  the  MPEG-4  DMIF  (ISO/IEC 14496-6)  free  license  to
    this software module or modifications thereof for use in hardware
    or software products claiming  conformance  to  the  MPEG-4  DMIF
    (ISO/IEC 14496-6). Those intending to use this software module in
    hardware or software  products  are  advised  that  its  use  may
    infringe   existing  patents.  The  original  developer  of  this
    software module and his/her company, the subsequent  editors  and
    their  companies,  and  ISO/IEC have no liability for use of this
    software module or modifications thereof  in  an  implementation.
    Copyright  is not released for non MPEG-4 DMIF  (ISO/IEC 14496-6)
    conforming  products. Triton R&D Ltd. and CSELT
    retain  full  right to use the code for their own purpose, assign
    or donate the code to a third party and to inhibit third  parties
    from  using  the  code  for  non  MPEG-4  DMIF  (ISO/IEC 14496-6)
    conforming products. This copyright notice must  be  included  in
    all copies or derivative works. Copyright  1998.
************************************************************************/

/**************************************************************************
 * Project Im1Player
 * INTERNATIONAL STANDARD ORGANIZATION
 * ISO/IEC JTC1/SC29/WG11
 *
 * SUBSYSTEM: DMIF
 * FILE:      DPI.h
 * OVERVIEW:  Definitions of DMIF-Plugin Interface.
 *
 **************************************************************************/

#ifndef _DPI_h_
#define _DPI_h_

#include "defs.h"
#include "ztl.h"
#include "DAItypes.h"
#include "DAItools.h"
#include "FlexMuxTool.h"

/*-------------------------------------------------------------
Struct for passing OUT parameters in :
ServiceClientSink::DPI_ChannelAdd_Cnf
ServiceClientSink::DPI_TransmuxAdd_Cnf
-------------------------------------------------------------*/
struct StreamMapElement
{
	int			m_nFmxInfo;
	UINT		m_nChid;
	UINT		m_nTmpId;
	QOSSTREAM	m_pQoSStream;
	// SLConfigDescriptor is provisionally exposed as a separate parameter
	BYTE*		m_pAppInfo;
	USHORT		m_nAppInfoLen;
	BYTE*		m_pData;
	int			m_nDataLen;

  public: // methods
	BOOL operator == (StreamMapElement const &el) const
	{
		return (m_nChid == el.m_nChid);
	}

	StreamMapElement() : m_nFmxInfo( -1 ), m_nChid( 0 ), m_nTmpId( 0 ),
		m_pQoSStream( NULL ),
		m_pAppInfo( NULL ),
		m_nAppInfoLen( 0 ),
		m_pData( NULL ),
		m_nDataLen( 0 )
	{}

	StreamMapElement(const StreamMapElement& o)//copy constructor
	{
		m_nFmxInfo	= o.m_nFmxInfo;
		m_nChid		= o.m_nChid;
		m_nTmpId		= o.m_nTmpId;
		m_pQoSStream= o.m_pQoSStream;
		m_pAppInfo	= o.m_pAppInfo;
		m_nAppInfoLen= o.m_nAppInfoLen;
		m_nDataLen	= o.m_nDataLen;
		if ( m_nDataLen > 0 )
		{
			m_pData		= new BYTE[o.m_nDataLen];
			memcpy(m_pData,o.m_pData,o.m_nDataLen);
		}
		else
			m_pData = NULL ;
	}

	StreamMapElement& operator=(const StreamMapElement& r)
	{
		m_nFmxInfo	= r.m_nFmxInfo;
		m_nChid		= r.m_nChid;
		m_nTmpId	= r.m_nTmpId;
		m_pQoSStream= r.m_pQoSStream;
		m_pAppInfo	= r.m_pAppInfo;
		m_nAppInfoLen= r.m_nAppInfoLen;
		m_nDataLen	= r.m_nDataLen;
		if ( m_pData )
			delete [] m_pData ;
		if ( m_nDataLen > 0 )
		{
			m_pData		= new BYTE[r.m_nDataLen];
			memcpy(m_pData,r.m_pData,m_nDataLen);
		}
		else
			m_pData = NULL ;
		return ( *this ) ;
	}

	virtual ~StreamMapElement()
	{
		delete[] m_pData ;
	}
};

typedef ZArray<StreamMapElement>		DPI_TransmuxAdd_CnfParam;
typedef ZArray<RESPONSE>				DPI_ChannelsDelete_CnfParam;


//--------------------------

struct OnTransmuxAddstruct
{
	int			m_nFmxInfo;
	WORD		m_nESid;
	UINT		m_nChid;
	QOSSTREAM	m_sQosStream;
	RESPONSE	m_eResponse;
	BOOL operator == (OnTransmuxAddstruct const &el) const
		{return el.m_nESid == m_nESid;}
};

typedef ZArray<OnTransmuxAddstruct> OnTransmuxAddArray;

struct OnTransmuxAddElem
{
	HANDLE				m_hTAT;
//	FLEXMUX_TYPE		m_eFmxType;
	WORD				m_eFmxType;//FLEXMUX_TYPE is "used internally of DMIF" see FlexMuxTool.h
	QOSSTREAM			m_sQosStream;
	DIRECTION			m_eDirection;
	OnTransmuxAddArray*	m_pInfo;
	BOOL operator == (OnTransmuxAddElem const &/*el*/) const
		{return FALSE;}
};

struct OnChannelsDeleteElem
{
	HANDLE		m_hTAT;
	RESPONSE	m_eResponse;
	BOOL operator == (OnChannelsDeleteElem const &el) const
		{return(m_hTAT == el.m_hTAT );}
};

struct OnChannelsAddElem
{
	UINT		m_nChid;
	WORD		m_nESid;
	QOSSTREAM	m_sQosStream;
	BYTE*		m_pAppInfo;
	UINT		m_nAppInfoLen;
	BOOL operator == (OnChannelsAddElem const &el) const
		{return(m_nChid == el.m_nChid);}
};

typedef ZArray<OnTransmuxAddElem> OnTransmuxAddParam;
typedef ZArray<OnChannelsAddElem> OnChannelsAddParam;
typedef ZArray<OnChannelsDeleteElem> OnChannelsDeleteParam;


class Service;

/*-------------------------------------------------------------
Interface for DMIF callbacks sink object (equivalent to	DAI callbacks,
except of Channel related events, which are reported directly to the
Channel object.
Inteface for a service sink object
-------------------------------------------------------------*/
class ServiceSink
{
public:
	virtual ~ServiceSink () {};
	virtual void DPI_ServiceAttach_Cnf (
		const Service*			pService,
		const BYTE*				pUuData,
		const int				nUuDataLen,
		const RESPONSE			Response
	) = 0;
	virtual void DPI_ServiceDetach_Cnf (
		const Service*			pService,
		const RESPONSE			Response
	) = 0;
	virtual void DPI_ChannelAdd_Cnf (
		const Service*			pService,
		const StreamMapElement	&outParam,
		const RESPONSE			eResponse
	) = 0;
	virtual void DPI_ChannelDelete_Cnf (
		const Service*			pService,
		const UINT				nChid,
		const RESPONSE			Response
	) = 0;
	virtual void DPI_DataReceive_Ind (
		const Service*			pService,
		const UINT				m_nChid,
		const BYTE*				pStreamData,
		const int				nStreamDataLength,
		const BYTE*				pAppInfo,
		const int				nAppInfoLength,
		const int				nErr
	) = 0;
	virtual void DPI_UserCommandAck_Cnf (
		const Service*			pService,
		const DAIUserCmdParam&	outParam
	) = 0;
/////
	virtual void DPI_ServiceAttach_Ind(
		const Service*		pService,
		const char*				serviceParam,
		const BYTE*				pUuDataInBuffer,
		const int				nUuDataInLen
	) = 0;
	virtual void DPI_ServiceDetach_Ind(
		const Service*		pService,
		const REASON			eReason
	) = 0;
	virtual void DPI_ChannelAdd_Ind(
		const Service*		pService,
		const DAIChannelAddParam& inParam
	) = 0;
	virtual void DPI_ChannelDelete_Ind(
		const Service*		pService,
		const DAIChannelDeleteParam& inParam
	) = 0;
	virtual void DPI_UserCommand_Ind(
		const Service*		pService,
		const DAIUserCmdParam&	inParam
	) = 0;
	virtual void DPI_UserCommandAck_Ind(
		const Service*		pService,
		const DAIUserCmdParam&	inParam
	) = 0;

};

/*-------------------------------------------------------------
Interface for the DMIF object
Interface for a service handler
-------------------------------------------------------------*/
class Service
{
private:
	ServiceSink* m_pSink;
public:
	// Management functions
	Service () { m_pSink = NULL; }
	virtual ~Service () {}
	void SetSink (ServiceSink* pSink) { m_pSink = pSink; }
	ServiceSink* GetSink () { return m_pSink; }
	virtual BOOL IsYourURL (LPCSTR szURL) = 0;
	virtual LPCSTR GetModuleID () = 0;

	virtual void QueryInterface (
		const char* sTag,
		const void* pSink,
		void*& r_pSink
		) {	sTag; pSink; r_pSink = (void*) NULL; }
	virtual const char* GetInfo() { return NULL; }

	// Service Attach functions
	virtual void DPI_ServiceAttach_Req(
		const LPCSTR	szURL,
		const BYTE*		pUuData,
		const int		nUuDataLen
	) = 0;

	// Service Detach functions
	virtual void DPI_ServiceDetach_Req(
		const REASON	Reason
	) = 0;

	// Channel Add functions
	virtual void DPI_ChannelAdd_Req(
		const DAIChannelAddParam &inParam
	) = 0;

	// Channel Delete functions
	virtual void DPI_ChannelDelete_Req(
		const DAIChannelDeleteParam &inParam
	) = 0;

	// User Command function
	virtual void DPI_UserCommand_Req(
		const DAIUserCmdParam& inParam
	) = 0;
	virtual	void DPI_UserCommandAck_Req(
		const DAIUserCmdParam& outParam
	) = 0;
	virtual BOOL DPI_DataSend_Req(
		const unsigned int		nChid,
		const BYTE*				pStreamData,
		const int				nStreamDataLength,
		const BYTE*				pAppData,
		const int				nAppDataLength
	) = 0;
/////
	virtual void DPI_ServiceAttach_Rsp(
		const BYTE*				pUuDataOutBuffer,
		const int				nUuDataOutLen,
		const RESPONSE			eResponse
	) = 0;
	virtual void DPI_ServiceDetach_Rsp(
		const RESPONSE			eResponse
	) = 0;
	virtual FlexMuxDataParam* DPI_ChannelAdd_Rsp(
		const OnChannelsAddParam& outParam
	) = 0;
	virtual FlexMuxDataParam* DPI_TransmuxAdd_Rsp(
		const OnTransmuxAddParam& outParam
	) = 0;
	virtual void DPI_ChannelDelete_Rsp(
		const OnChannelsDeleteParam& outParam
	) = 0;
	virtual void DPI_UserCommandAck_Rsp(
		const DAIUserCmdParam&	outParam
	) = 0;

};

#endif //_DPI_h_
