/***********************************************************************
    This software module  was  originally  developed by Zvi  Lifshitz
    (Triton R&D Ltd.), Mauro Rossi (CSELT) and Guido Franceschini
    (CSELT), and edited by Michael Stepping (FTK)
    in the course of development of the MPEG-4 DMIF (ISO/IEC 14496-6)
    standard.  This software module is an implementation of a part of
    one or more MPEG-4  DMIF  (ISO/IEC 14496-6)  tools  as  specified
    by  the  MPEG-4 DMIF (ISO/IEC 14496-6)  standard.  ISO/IEC  gives
    users of  the  MPEG-4  DMIF  (ISO/IEC 14496-6)  free  license  to
    this software module or modifications thereof for use in hardware
    or software products claiming  conformance  to  the  MPEG-4  DMIF 
    (ISO/IEC 14496-6). Those intending to use this software module in
    hardware or software  products  are  advised  that  its  use  may
    infringe   existing  patents.  The  original  developer  of  this
    software module and his/her company, the subsequent  editors  and
    their  companies,  and  ISO/IEC have no liability for use of this
    software module or modifications thereof  in  an  implementation.
    Copyright  is not released for non MPEG-4 DMIF  (ISO/IEC 14496-6)
    conforming  products. Triton R&D Ltd. and CSELT
    retain  full  right to use the code for their own purpose, assign
    or donate the code to a third party and to inhibit third  parties
    from  using  the  code  for  non  MPEG-4  DMIF  (ISO/IEC 14496-6)
    conforming products. This copyright notice must  be  included  in
    all copies or derivative works. Copyright  1998.
************************************************************************/

/**************************************************************************
 * Project Im1Player
 * INTERNATIONAL STANDARD ORGANIZATION
 * ISO/IEC JTC1/SC29/WG11
 * 
 * SUBSYSTEM: DMIF
 * FILE:      FlexMuxTool.h
 * OVERVIEW:  Definitions of FlexMux and MuxCode tools related structures
 * 
 **************************************************************************/

#ifndef _FlexMuxTool_h_
#define _FlexMuxTool_h_


class MPXChannel;

#define MAX_CHANNELS_PER_FLEXMUX 256

// enumeration of FlexMuxTypes is used internally of DMIF
typedef enum tagFlexmuxType
{ 
	UNDEFINED_FLEXMUX	= 0x00,
	NULL_FLEXMUX		= 0x01, 
	MPEG4_FLEXMUX		= 0x02
} FLEXMUX_TYPE;


/*-------------------------------------------------------------
Structs for passing MuxCodeTable entries:
ServiceClientSink::DPI_MuxCodeTableReceived_Ind
FlexDemux::AddMuxCodeTableElements
-------------------------------------------------------------*/
struct McteSlot
{
	int flexMuxChannel;
	int numberOfBytes;
public:
	McteSlot() : flexMuxChannel(-1), numberOfBytes(-1)
	{}
};
typedef ZArray<McteSlot> McteSlotTable;

struct LinearMuxCodeTableElement
{
	int version;
	McteSlotTable slots;
public:
	LinearMuxCodeTableElement() : version(-1), slots()
	{}
};
typedef	ZArray<LinearMuxCodeTableElement> LinearMuxCodeTable;

struct StreamMapElement2
{   
	int m_nFmxInfo;
	UINT m_nChid;
	WORD m_nESid;
	BOOL operator == (const StreamMapElement2 &el) const {return el.m_nChid == m_nChid;}
};

typedef ZArray<StreamMapElement2> StreamTable;

class FlexMuxDataElement
{	
public:
	HANDLE				hTAT;
	FLEXMUX_TYPE		eFmxType;
	StreamTable*		pSMEs;
public:
	FlexMuxDataElement():
	  hTAT((HANDLE)0),
	  eFmxType(UNDEFINED_FLEXMUX),
	  pSMEs(NULL)
	{}
/*This dtor crashes the application.
	~FlexMuxDataElement()
	{
		if(pSMEs)
		{
			pSMEs->Clear();
			delete pSMEs;
		}
	}
	FlexMuxDataElement(const FlexMuxDataElement& o)//copy constructor
	{
		hTAT = o.hTAT;
		eFmxType = o.eFmxType;
		pSMEs = new StreamTable(*(o.pSMEs));
	}
*/
	BOOL operator == (const FlexMuxDataElement &rElement) const
    {
        return (hTAT == rElement.hTAT);
    }
};

typedef ZArray<FlexMuxDataElement> FlexMuxDataParam;

#endif //_FlexMuxTool_h_
