/*------------------------------------------------------------------
	 1999 CSELT - Centro Studi e Laboratori Telecomunicazioni.
	All rights reserved.
------------------------------------------------------------------*/

#ifndef _MLIST_H_
#define _MLIST_H_

#ifndef NULL
#define NULL 0
#endif

//Element of the list that holds the pointer to the info.
template <class T>
class MListElement
{
private:
	bool delOnDstr;
public:
	T*				info;
	MListElement*	next;
public:
	//Constructor - Sets the flag that tells if delete the info on exit.
	MListElement(T* pEl,bool d) :
		info(pEl),
		next(NULL),
		delOnDstr(d)
		{}

	//Destructor - Eventually deletes the info.
	~MListElement()
	{
		if(delOnDstr)
		{
			if(info)
			{
				delete info;
				info = NULL;
			}
		}
		next = NULL;
	}
};


template <class T>
class MList
{
private:
	int length;
protected:
	MListElement<T>* head;
	bool deleteOnDestruction;
public:

	//Constructor - default: do not calls the destructor of the elements. (it deletes only the skeleton)
	MList(bool d=false) :
	  head(NULL),
	  length(0),
	  deleteOnDestruction(d)
	  {};

	~MList()
	{
		Clear();
	}

	//Returns the position in the list of the element. -1 if not found.
	int Find(T* pEl)
	{
		int k = -1;
		MListElement<T>* p = head;
		while(p != NULL)
		{
			++k;
			if(*(p->info) == *pEl)
				break;
			p = p->next;
		}
		if(p)
			return k;
		else
			return -1;
	}

	//Returns the pointer to the i-th element. NULL if out of bounds.
	T* operator[] (int i)
	{
		MListElement<T>* p = GetElem(i);
		if(p)
			return p->info;
		else
			return NULL;
	}

	//Adds the pointer to the new element at the end of the list.
	int operator+= (T* pEl)
	{
		MListElement<T>* p = head;
		if(p==NULL)
		{
			head = new MListElement<T>(pEl,deleteOnDestruction);
		}
		else
		{
			while(p->next)
				p = p->next;
			p->next = new MListElement<T>(pEl,deleteOnDestruction);
		}
		++length;
		return length;
	}

	//Delete the element from the list.
	int operator-= (T* pEl)
	{
		MListElement<T>* dl = NULL;
		MListElement<T>* p = head;
		if(p!=NULL)//empty list
		{
			if(*(p->info) == *pEl)//first element matches
			{
				head = p->next;
				delete p;
				--length;
			}
			else
			{
				while(p->next)//check next elements
				{
					if(*(p->next->info) == *pEl)
					{
						dl = p->next;
						p->next = p->next->next;
						delete dl;
						--length;
						break;
					}
					p = p->next;
				}
			}
		}
		return length;
	}

	//Deletes the i-th element (starting from number 0)
	int DeleteFrom(int i)
	{
		MListElement<T>* prev = NULL;
		MListElement<T>* p = NULL;

		if(i<0 || i>=length)
			return -1;

		if(i==0)
		{
			p = head;
			head = p->next;
			delete p;
			--length;
			return 0;
		}
		prev = GetElem(i-1);
		p = GetElem(i);
		prev->next = p->next;
		delete p;
		--length;
		return 0;
	}

	//Deletes the element
	int Delete(T* pEl)
	{
		MListElement<T>* prev = head;
		MListElement<T>* p = NULL;

		if(prev->info == pEl)
		{
			head = prev->next;
			delete prev;
			--length;
			return 0;
		}

		p = prev->next;
		while(p)
		{
			if(p->info == pEl)
			{
				prev->next = p->next;
				delete p;
				--length;
				return 0;
			}
			prev = p;
			p = p->next;
		}

		return -1;//not found.
	}


	//Clears the list.
	void Clear()
	{
		MListElement<T>* n = NULL;
		MListElement<T>* p = head;
		while(p)
		{
			n = p->next;
			delete p;
			p = n;
		}
		head = NULL;
		length = 0;
	}

	//Returns the number of elements in the list.
	int GetLength()
	{
		return length;
	}

private:
	MListElement<T>* GetElem(int i)
	{
		MListElement<T>* p = head;

		if(i<0 || i>=length)
			return NULL;

		while(i>0)
		{
			p = p->next;
			--i;
		}
		return p;
	}


};

#endif //_MLIST_H_

