/*
This software module was originally developed by Apple Computer, Inc.
and edited by Guido Franceschini (CSELT) in the course of development of MPEG-4.
This software module is an implementation of a part of one or
more MPEG-4 tools as specified by MPEG-4.
ISO/IEC gives users of MPEG-4 free license to this
software module or modifications thereof for use in hardware
or software products claiming conformance to MPEG-4.
Those intending to use this software module in hardware or software
products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company,
the subsequent editors and their companies, and ISO/IEC have no
liability for use of this software module or modifications thereof
in an implementation.
Copyright is not released for non MPEG-4 conforming
products. Apple Computer, Inc. retains full right to use the code for its own
purpose, assign or donate the code to a third party and to
inhibit third parties from using the code for non
MPEG-4 conforming products.
This copyright notice must be included in all copies or
derivative works. Copyright (c) 1999.
*/
#include "defs.h"

#ifdef __UNIX
#include <netinet/in.h>
#endif

#include "FileMP4Instance.h"
#include "MP4TrackReaderClient.h"

LPCSTR FileMP4Instance::s_moduleID = "FileMP4Instance";
LPCSTR FileMP4Instance::s_myURLScheme = "file:";

class MP4FileState
{
	public:
		MP4Movie movie;
};

#ifdef WIN32
extern "C" Service *CreateInstance()
{
	return new FileMP4Instance;
}
#endif
#ifdef __UNIX
extern "C" Service *CreateMP4FileReaderInstance()
{
	return new FileMP4Instance;
}
#endif

LPCSTR FileMP4Instance::GetModuleID()
{
	return s_moduleID;
}

FileMP4Instance::FileMP4Instance()
{
		m_movie = 0;
	m_iod = 0;
		m_trackCount = 0;
	m_szURL = NULL;
}

FileMP4Instance::~FileMP4Instance()
{
	MP4TrackReaderClient* client = 0;
	while ( m_readers.GetLength() )
	{
		client = m_readers[ 0 ];
		client->WaitForThread ();
		m_readers -= client;
		delete client;
	}
}

void FileMP4Instance::ExtractIOD(LPCSTR serviceURL, BYTE*& initialOD, int* pIodLength)
{
	try
	{
		BOOL isU = FALSE;
// SOLARIS PORTING
#ifdef WIN32
		if (IsTextUnicode((void *) serviceURL, 1+strlen(serviceURL), NULL)) {
			isU = TRUE;
		}
#endif
		
		MP4Err err = MP4OpenMovieFile( &m_movie, serviceURL, MP4OpenMovieInPlace);
		MP4Exception::test( err );
		err = MP4GetMovieTrackCount( m_movie, &m_trackCount );
		MP4Exception::test( err );
		err = MP4NewHandle( 0, &m_iod );
		MP4Exception::test( err );
		err = MP4GetMovieInitialObjectDescriptor( m_movie, m_iod );
		MP4Exception::test( err );
		err = MP4GetHandleSize( m_iod, (unsigned int *)pIodLength );
		MP4Exception::test( err );
		initialOD = (BYTE *) *m_iod;
	}
	catch ( MP4Exception& /*e*/ )
	{
		if ( m_movie )
			MP4DisposeMovie( m_movie );
		m_movie = 0;
		initialOD = NULL;
		*pIodLength = -1;
	}
	return;
}

// SOLARIS PORTING - Added const
//void FileMP4Instance::DPI_ServiceAttach_Req(LPCSTR szURL, const BYTE*, int)
void FileMP4Instance::DPI_ServiceAttach_Req(const LPCSTR szURL, const BYTE*, const int)
{
	BYTE* initialOD=NULL;
	int iodLength;
	m_szURL = new char[strlen(szURL)+1];
	strcpy(m_szURL,szURL);

	/*------------------------------------------
	   Extract the IOD
	-------------------------------------------*/
	ExtractIOD(szURL,initialOD,&iodLength);
	if (iodLength <0) {
		GetSink()->DPI_ServiceAttach_Cnf(this, NULL, 0, RESPONSE_URL_NOT_FOUND);
	} else {
	/*-------------------------------------------
	   Return the IOD to the application
	-------------------------------------------*/
	GetSink()->DPI_ServiceAttach_Cnf (
		this,
		initialOD,
		iodLength,
		RESPONSE_OK
	);
	}
	return;
}

// SOLARIS PORTING - Added const
//void FileMP4Instance::DPI_ServiceDetach_Req( REASON )
void FileMP4Instance::DPI_ServiceDetach_Req(const REASON )
{
	if ( m_movie )
		MP4DisposeMovie( m_movie );
	m_movie = 0;
	if (m_iod)
		MP4DisposeHandle(m_iod);
	m_iod = 0;

	GetSink()->DPI_ServiceDetach_Cnf( this, RESPONSE_OK );

	if (m_szURL) delete m_szURL;
	m_szURL = NULL;
	return;
}

void FileMP4Instance::DPI_ChannelAdd_Req( const DAIChannelAddParam &inParam )
{
	for ( int i = 0; i < inParam.GetLength(); i++ )
	{
		RESPONSE eResponse = RESPONSE_INVALID_DIRECTION;
		const DAIChannelAddElement &element = inParam[ i ];

		MP4TrackReaderClient *client = NULL;
		if(element.m_eDirection == DOWNSTREAM) {
			// the data is either a 16-bit elementary stream ID or a URL
			if ( element.m_nDataLen == 2 ) {
				// ES id. These map directly to mp4 track ids
				// Take care that are encoded in network byte order

				MP4Track theTrack;
				unsigned short trackID = ntohs(*(unsigned short*) element.m_pData);
				MP4Err err = MP4GetMovieTrack( m_movie, trackID, &theTrack );
				if ( err == MP4NoErr ) {
					assert (element.m_nAppInfoLen == sizeof (struct SLConfigDesc));
					client = new MP4TrackReaderClient( theTrack, trackID, element.m_nChid, element.m_nTmpId, (struct SLConfigDesc *)element.m_pAppInfo, GetSink(), this );
					if (client->m_reader) {
						m_readers += client;
						client->SendAddCnf();
					} else {
						client->SendAddCnf();
						delete client;
					}
					continue;
				}
			}
		}

		StreamMapElement rMapElement;
		rMapElement.m_nChid = element.m_nChid;
		rMapElement.m_nTmpId = element.m_nTmpId;
		rMapElement.m_pData = NULL;
		rMapElement.m_nDataLen = 0;

		rMapElement.m_nFmxInfo = 0;		// unused field
		GetSink()->DPI_ChannelAdd_Cnf( this, rMapElement, eResponse );
	}
	return;
}

MP4TrackReaderClient* FileMP4Instance::getClientForChID( int chID )
{
	MP4TrackReaderClient* ret = 0;
	int readerCount = m_readers.GetLength();

	for ( int i = 0; i < readerCount; i++ )
	{
		MP4TrackReaderClient *aClient = m_readers[ i ];
		if ( aClient && (aClient->getchID() == chID) )
		{
			ret = aClient;
			break;
		}
	}
	return ret;
}

MP4TrackReaderClient* FileMP4Instance::getClientForTmpID( int TmpID )
{
	MP4TrackReaderClient* ret = 0;
	int readerCount = m_readers.GetLength();
	for ( int i = 0; i < readerCount; i++ )
	{
		MP4TrackReaderClient *aClient = m_readers[ i ];
		if ( aClient && (aClient->getTmpID() == TmpID) )
		{
			ret = aClient;
			break;
		}
	}
	return ret;
}

void FileMP4Instance::DPI_ChannelDelete_Req( const DAIChannelDeleteParam &inParam )
{
	int parameterCount = inParam.GetLength();

	for ( int i = 0; i < parameterCount; i++ )
	{
		int chID = inParam[i].m_nChid;
		MP4TrackReaderClient *client = getClientForChID( chID );
		if ( client )
		{
			if ( client->getStatus() != MP4TrackReaderClient::finished )
				client->requestStop();
			else
				; // in case the reader has finished exactly now
			client->WaitForThread ();
			m_readers -= client;
			delete client;
		}
	}
	return;
}

void FileMP4Instance::DPI_UserCommand_Req( const DAIUserCmdParam& inParam )
{
	int parameterCount = inParam.m_nChLen;

	for ( int i = 0; (i < parameterCount); i++ )
	{
		int chID = inParam.m_pChid[i];
		MP4TrackReaderClient *client = getClientForChID( chID );

		if ( client )
		{
			if (strncmp(inParam.m_pData, "PLAY", 4)==0) {
				client->requestPlay();
			} else if (strcmp(inParam.m_pData, "PAUSE")==0) {
				client->requestPause();
			} else {
				;
			}
		}
	}
}

void FileMP4Instance::DPI_UserCommandAck_Req(const DAIUserCmdParam& inParam )
{
	bool bFound = true;
	int parameterCount = inParam.m_nChLen;
	DAIUserCmdParam outParam;
	char *rg;
	double begin, end;

	for ( int i = 0; (i < parameterCount); i++ )
	{
		int chID = inParam.m_pChid[i];
		MP4TrackReaderClient *client = getClientForChID( chID );

		if ( client )
		{
			if ( (strncmp(inParam.m_pData, "PLAY", 4)==0) || (strncmp(inParam.m_pData, "PREROLL", 4)==0) ) {
				rg = strstr(inParam.m_pData, "range=");
				if (rg) {
					int res = sscanf(rg, "range=%lf-%lf", &begin, &end);
					if (res==2) {
						client->SetRange(begin, end);
					} else {
						res = sscanf(rg, "range=%lf-", &begin);
						if (res==1) {
							client->SetRange(begin, -1);
						} else {
							res = sscanf(rg, "range=-%lf", &end);
							if (res==1) {
								client->SetRange(0, end);
							}
						}
					}
				}
				rg = strstr(inParam.m_pData, "speed");
				begin = 1.0;
				if (rg) {
					sscanf(rg, "speed=%lf", &begin);
					client->SetSpeed(begin);
					
				}
				client->requestPlay();

				if (strncmp(inParam.m_pData, "PREROLL", 4)==0) client->m_bIsPrerolling = TRUE;

			} else if (strcmp(inParam.m_pData, "PAUSE")==0) {
				client->requestPause();
			} else if (strcmp(inParam.m_pData, "STOP")==0) {
				client->requestPause(TRUE);
			} else if ((rg = strstr(inParam.m_pData, "speed")) != NULL) {
				sscanf(rg, "speed=%lf", &begin);
				client->SetSpeed(begin);
			} else if (strcmp(inParam.m_pData, "RESUME")==0) {
				client->requestPlay();
			}
		}
		else
			bFound = false;
	}
	const char *string=(bFound==false || parameterCount==0) ? "ERROR" : "OK";
	outParam.m_nDataLen=strlen(string)+1;
	outParam.m_pData=new char[outParam.m_nDataLen];
	strcpy(outParam.m_pData,string);
	outParam.m_nChLen=inParam.m_nChLen;
	outParam.m_pChid=inParam.m_pChid;
	outParam.m_nCmdId=inParam.m_nCmdId;
	outParam.m_eResponse=RESPONSE_OK;
	GetSink () -> DPI_UserCommandAck_Cnf(this,outParam);
	delete outParam.m_pData;
}

///////////
void FileMP4Instance::DPI_ServiceAttach_Rsp(
	const BYTE*				pUuDataOutBuffer,
	const int				nUuDataOutLen,
	const RESPONSE			eResponse)
{
	pUuDataOutBuffer; nUuDataOutLen; eResponse;
}

void FileMP4Instance::DPI_ServiceDetach_Rsp(
	const RESPONSE			eResponse)
{
	eResponse;
}

FlexMuxDataParam* FileMP4Instance::DPI_ChannelAdd_Rsp(
	const OnChannelsAddParam& outParam)
{
	outParam; return NULL;
}

FlexMuxDataParam* FileMP4Instance::DPI_TransmuxAdd_Rsp(
	const OnTransmuxAddParam& outParam)
{
	outParam; return NULL;
}

void FileMP4Instance::DPI_ChannelDelete_Rsp(
	const OnChannelsDeleteParam& outParam)
{
	outParam;
}

void FileMP4Instance :: DPI_UserCommandAck_Rsp(const DAIUserCmdParam&	outParam)
{
	if (outParam.m_eResponse != RESPONSE_OK) return;

	for (int i=0; i<outParam.m_nChLen; i++) {
		int chID = outParam.m_pChid[i];
		MP4TrackReaderClient *client = getClientForChID( chID );
		if (!client) continue;
		if (outParam.m_pData && strstr(outParam.m_pData, "BUFFERSPACE")) {
			sscanf(outParam.m_pData, "BUFFERSPACE=%f", &client->m_bufferOccupancy);
		}
	}
}



/*-------------------------------------------------------------
This is my URL if it starts with the correct scheme or
is a valid file name

**INTERIM trick to allow IM1 to work with TRIF files as well**
in the case of no scheme present, the file extension .trif
is not recognized as my URL
-------------------------------------------------------------*/
BOOL FileMP4Instance::IsYourURL( LPCSTR szURL )
{
	return !_strnicmp( szURL, s_myURLScheme, strlen(s_myURLScheme) ) ||
		(ZFileExists( szURL )
		&& _strnicmp( szURL + strlen(szURL) - strlen (".trif"), ".trif", strlen(".trif")));
}
