/*
This software module was originally developed by Apple Computer, Inc.
and edited by Guido Franceschini (CSELT) in the course of development of MPEG-4. 
This software module is an implementation of a part of one or 
more MPEG-4 tools as specified by MPEG-4. 
ISO/IEC gives users of MPEG-4 free license to this
software module or modifications thereof for use in hardware 
or software products claiming conformance to MPEG-4.
Those intending to use this software module in hardware or software
products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company,
the subsequent editors and their companies, and ISO/IEC have no
liability for use of this software module or modifications thereof
in an implementation.
Copyright is not released for non MPEG-4 conforming
products. Apple Computer, Inc. retains full right to use the code for its own
purpose, assign or donate the code to a third party and to
inhibit third parties from using the code for non 
MPEG-4 conforming products.
This copyright notice must be included in all copies or
derivative works. Copyright (c) 1999.
*/
/*
	$Id: FileMP4Instance.h,v 1.1.1.2 2006/04/14 14:12:02 hill Exp $
*/

#if !defined(_FileMP4Instance_h)	// Sentry, use file only if it's not already included.
#define _FileMP4Instance_h

#include "DPI.h"
#include "MP4Movies.h"
#include "mpg4dmif.h"

class MP4TrackReaderClient;

class FileMP4Instance : public Service
{
public:
	FileMP4Instance();
	virtual ~FileMP4Instance();
	
private:
	virtual BOOL IsYourURL( LPCSTR szURL );
	virtual LPCSTR GetModuleID();

	virtual void DPI_ServiceAttach_Req(
		const LPCSTR	szURL, 
		const BYTE*		pUuData,
		const int		nUuDataLen
		);
	virtual void DPI_ServiceDetach_Req(
		const REASON	eReason
		);
	virtual void DPI_ChannelAdd_Req(
		const DAIChannelAddParam	&inParam
		);
	virtual void DPI_ChannelDelete_Req(
		const DAIChannelDeleteParam	&inParam
		);
	virtual void DPI_UserCommand_Req(
		const DAIUserCmdParam&	inParam
		);
	virtual	void DPI_UserCommandAck_Req(
		const DAIUserCmdParam&	outParam
		);
	virtual BOOL DPI_DataSend_Req(
		const unsigned int		nChid,
		const BYTE*				pStreamData,
		const int				nStreamDataLength,
		const BYTE*				pAppData,
		const int				nAppDataLength
		) {nChid;pStreamData;nStreamDataLength;pAppData;nAppDataLength;return FALSE;};
///
	virtual void DPI_ServiceAttach_Rsp(
		const BYTE*				pUuDataOutBuffer, 
		const int				nUuDataOutLen,
		const RESPONSE			eResponse
		);
	virtual void DPI_ServiceDetach_Rsp(
		const RESPONSE			eResponse
		);
	virtual FlexMuxDataParam* DPI_ChannelAdd_Rsp(
		const OnChannelsAddParam& outParam
		);
	virtual FlexMuxDataParam* DPI_TransmuxAdd_Rsp(
		const OnTransmuxAddParam& outParam
		);
	virtual void DPI_ChannelDelete_Rsp(
		const OnChannelsDeleteParam& outParam
		);
	virtual void DPI_UserCommandAck_Rsp(
		const DAIUserCmdParam&	outParam
		);
///
private:
	void ExtractIOD(LPCSTR serviceURL, BYTE*& initialOD, int* pIodLength);
	MP4TrackReaderClient *getClientForChID( int chID );
	MP4TrackReaderClient *getClientForTmpID( int TmpID );
	
	MP4Movie  m_movie;
	unsigned int m_trackCount;
	MP4Handle m_iod;
	ZArray<MP4TrackReaderClient*> m_readers;
	char* m_szURL;

	static LPCSTR s_moduleID;
	static LPCSTR s_myURLScheme;
};

class MP4Exception
{
public:
	inline MP4Exception( MP4Err error );
	static inline void test( MP4Err error );
	static inline void testMalloc( void* ptr );
	inline MP4Err getErrorCode() const;
private:
	int m_err;
};

inline MP4Exception::MP4Exception( MP4Err error )
{
	m_err = error;
}

inline MP4Err MP4Exception::getErrorCode() const
{
	return m_err;
}

inline void MP4Exception::test( MP4Err error )
{
	if ( error != MP4NoErr )
		throw MP4Exception( error );
}

inline void MP4Exception::testMalloc( void* ptr )
{
	if ( ptr == 0 )
		throw MP4Exception( MP4NoMemoryErr );
}

#endif
