/*
This software module was originally developed by Apple Computer, Inc.
and edited by Guido Franceschini (CSELT) in the course of development of MPEG-4.
This software module is an implementation of a part of one or
more MPEG-4 tools as specified by MPEG-4.
ISO/IEC gives users of MPEG-4 free license to this
software module or modifications thereof for use in hardware
or software products claiming conformance to MPEG-4.
Those intending to use this software module in hardware or software
products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company,
the subsequent editors and their companies, and ISO/IEC have no
liability for use of this software module or modifications thereof
in an implementation.
Copyright is not released for non MPEG-4 conforming
products. Apple Computer, Inc. retains full right to use the code for its own
purpose, assign or donate the code to a third party and to
inhibit third parties from using the code for non
MPEG-4 conforming products.
This copyright notice must be included in all copies or
derivative works. Copyright (c) 1999.
*/
/*
	$Id: MP4TrackReaderClient.cpp,v 1.1.1.2 2006/04/14 14:12:02 hill Exp $
*/
#include "FileMP4Instance.h"
#include "MP4TrackReaderClient.h"
#include "ztl.h"


#define AU_BUFFER_MSEC	300
//when speed>1 for audio the buffer is increased by this size
#define AUDIO_BUFFER_SCALE	5


MP4TrackReaderClient::MP4TrackReaderClient( MP4Track theTrack, int ESID, int chID, int nTmpId, struct SLConfigDesc* pslcd, ServiceSink *sink, FileMP4Instance *pService )
: m_track(theTrack), m_id(ESID), m_chID(chID), m_nTmpId(nTmpId), m_slcd(*pslcd),
m_sink(sink), m_pService (pService), m_sampleH(0), m_mode(pause), m_status (starting),
m_smpMode(1,0), m_readerThread (&MP4TrackReaderClient::readerLoop, this )
{
	m_reader = NULL;
	RESPONSE eResponse = RESPONSE_REMOTE_ERROR;
	MP4Err err = MP4NoErr;
	err = MP4CreateTrackReader( m_track, &m_reader );
	if ( err == MP4NoErr )
	{
		err = MP4NewHandle( 4096, &m_sampleH );
	}

	MP4Media media;
	MP4GetTrackMedia(theTrack, &media);

	u64 tmp;
	MP4GetTrackDuration(theTrack, &tmp);
	m_Duration = tmp;

	u32 ts;
	MP4Movie mov;
	MP4GetTrackMovie(theTrack, &mov);
	MP4GetMovieTimeScale(mov, &ts);
	m_Duration /= ts;
	m_Start = 0;
	m_End = m_Duration;
	
	m_bIsPrerolling = FALSE;

	m_bIsSeeking = 0;
	m_speed = 1.0;

	MP4GetMediaSampleCount(media, &m_sampleCount);
	MP4GetMediaTimeScale(media, &m_TimeScale);
	MP4GetMediaDuration(media, &tmp);

	float sampDur = tmp / m_sampleCount;
	sampDur /= m_TimeScale;
	sampDur *= 1000;
	if (sampDur) 
		m_AUBufferSize = 1000 / sampDur;
	else
		m_AUBufferSize = 5;

	MP4GetMediaHandlerDescription(media, &ts, NULL);
	m_bTrackIsAudio = (ts==MP4AudioHandlerType) ? TRUE : FALSE;
	//no more than 200 ms buffer for video
	if (ts==MP4VisualHandlerType) m_AUBufferSize /= 5;

	m_bReset = TRUE;

	if ( err == MP4NoErr ) {
		int result = StartReaderThread();
		if ( result == -1 ) {
			err = MP4NoMemoryErr;
			eResponse = RESPONSE_RESOURCES_EXHAUSTED;
		}
	}	
	if ( err != MP4NoErr ) 
		m_status = finished;
	else {
		while (m_status == starting) Sleep(1);
	}
}

void MP4TrackReaderClient::SendAddCnf()
{
	StreamMapElement mapElement;
	mapElement.m_nFmxInfo = 0;
	mapElement.m_nChid    = m_chID;
	mapElement.m_nTmpId    = m_nTmpId;
	DPI_TransmuxAdd_CnfParam smt;
	StreamMapElement &rMapElement = smt.ElementAt( 0 );
	rMapElement.m_nChid = m_chID;
	rMapElement.m_nTmpId = m_nTmpId;
	rMapElement.m_pData = new unsigned char[2];

	// use network byte order for the user data representing the ES_ID !!!
	*((unsigned short*)rMapElement.m_pData) = htons(m_id);
	rMapElement.m_nDataLen = 2;

	char str[1024];
	double dur = GetDuration();

	sprintf(str, "duration=%f\r\n", dur);
	rMapElement.m_pAppInfo = (unsigned char *)strdup(str);
	rMapElement.m_nAppInfoLen = strlen(str)+1;
	m_sink->DPI_ChannelAdd_Cnf( m_pService, rMapElement, (m_status == finished) ? RESPONSE_REMOTE_ERROR : RESPONSE_OK);
	if (rMapElement.m_pAppInfo) free(rMapElement.m_pAppInfo);

}


MP4TrackReaderClient::status MP4TrackReaderClient::getStatus()
{
	return m_status;
}

void MP4TrackReaderClient::requestPlay()
{
	assert(m_mode != stop);
	if (!m_bIsPrerolling && (m_mode != pause) ) m_bReset = TRUE;
	m_bIsPrerolling = FALSE;
	if (m_mode != play) {
		assert(m_status!=starting);
		m_mode = play;
#ifdef _DEBUG
		assert(m_smpMode.V());
#else
		m_smpMode.V();
#endif
	}
}

void MP4TrackReaderClient::requestPause(BOOL bReset)
{
	m_mode = pause;
	m_bReset = bReset;
}

void MP4TrackReaderClient::requestStop()
{
	//signal semaphore if we're paused
	if (m_mode==pause) {
		m_mode = stop;
#ifdef _DEBUG
		assert(m_smpMode.V());
#else
		m_smpMode.V();
#endif
	} else {
		m_mode = stop;
	}
}

MP4TrackReaderClient::~MP4TrackReaderClient()
{
	if ( m_sampleH )
		MP4DisposeHandle( m_sampleH );
	if ( m_reader )
		MP4DisposeTrackReader( m_reader );
}

/*
this provides some basic control of data flow so that we don't send AUs ahead of time
in order to do this we use DMIF command request on the channel to get decoding buffer 
occupancy. if over threshold wait
*/
void MP4TrackReaderClient::WaitForSpace()
{
	DAIUserCmdParam par;
	int retry = 10;
	par.AddChid(m_chID);
	m_bufferOccupancy = 0;
	par.SetData(strlen("GETBUFFERSPACE")+1, (unsigned char *) "GETBUFFERSPACE");

	while (retry) {
		m_sink->DPI_UserCommandAck_Ind(m_pService, par);
		if (par.m_eResponse == RESPONSE_CHANNEL_DELETED) break;
		if (m_bufferOccupancy<0.8) break;
		Sleep(10);
		retry--;
	}
	par.Reset();
}


void MP4TrackReaderClient::readerLoop()
{
	MP4Err err;
	struct SLHeader slph;
	u32 lastDTS = 0;
	u32 trackID;
	u32 AUSize = 0;
	u32 AUFlags, cts, dts;
	MP4GetTrackID(m_track, &trackID);

	memset(&slph, 0, sizeof (struct SLHeader));
	//init default values
	slph.accessUnitEndFlag = slph.accessUnitStartFlag = 1;
	slph.compositionTimeStampFlag = 1;
	slph.decodingTimeStampFlag = 0;

	slph.decodingTimeStamp = 0;
	BOOL bIsFirst = TRUE;
	
	double prevSpeed = m_speed;
	lastDTS = 0;
	m_status = idle;
	m_smpMode.P();	

	int buffer = m_AUBufferSize;

	do
	{
		err = MP4NoErr;
		if (m_mode==play) {
			//if playing and end of file reached or speed is 0, move to pause mode
			if ((m_speed==0.0) || (m_status == eof)) m_mode = pause;
		}

		if (m_mode == pause) {
			m_mode = pause;
			m_smpMode.P();		
		}
		if ( m_mode == stop ) {
			break;
		}

		if (m_bReset) {
			m_bReset = FALSE;
			err = MP4DisposeTrackReader( m_reader );
			err = MP4CreateTrackReader( m_track, &m_reader );

			memset(&slph, 0, sizeof (struct SLHeader));
			//init default values
			slph.accessUnitEndFlag = slph.accessUnitStartFlag = 1;
			slph.compositionTimeStampFlag = 1;
			bIsFirst = TRUE;
			lastDTS = 0;
			buffer = m_AUBufferSize;
			prevSpeed = m_speed;

			/*there is a seek operation so setup reader*/
			if (m_bIsSeeking && (m_speed>0)) {
#ifndef _SAIT_3DTG_MP4VIDEO_
				MP4TrackReaderSeekToSyncSample(m_reader, m_Start*m_TimeScale);
#endif _SAIT_3DTG_MP4VIDEO_
				//disable the sleep in seek mode (cf below)
				bIsFirst = FALSE;
			}
			assert(m_Start>=0);

			if (m_speed<0) {
				if (m_bIsSeeking) {
					lastDTS = m_End * m_TimeScale;
				} else {
					lastDTS = m_Duration*m_TimeScale;
				}
			}
		}

		/*keep track of the timeline (using DTS - always increasing) and update at each speed change*/
		if (prevSpeed != m_speed) {
			prevSpeed = m_speed;
			//reset buffer
			buffer = m_AUBufferSize;
			//buffer a lot on audio if speed>0 due to SR conversion (more CUs consumed to fill the audio board)
			if (m_bTrackIsAudio && m_speed>1) buffer*=AUDIO_BUFFER_SCALE;
			if (m_bTrackIsAudio && m_speed<-1) buffer*=4*AUDIO_BUFFER_SCALE;
		}

		AUSize = AUFlags = cts = dts = 0;

		if (!m_sampleCount) {
			err = MP4EOF;
		} else if (m_speed>0) {
			m_status = reading;
			err = MP4TrackReaderGetNextAccessUnit(m_reader, m_sampleH, &AUSize, &AUFlags, &cts, &dts);
		} else {
			m_status = reading;
			u32 off = 20;
			err = MP4EOF;
			while (lastDTS>off) {
				MP4TrackReaderSeekToSyncSample(m_reader, lastDTS-off);
				err = MP4TrackReaderGetNextAccessUnit(m_reader, m_sampleH, &AUSize, &AUFlags, &cts, &dts);
				if (err!= MP4NoErr) break;
				if ( ((AUFlags & MP4MediaSampleNotSync) == 0) && (dts!=lastDTS) )break;
				off+=20;
				err = MP4EOF;
			}
		}
		
		switch (err) {
		case MP4NoErr:
			if (m_bIsSeeking) {
				//not reached yet (only forward)
				if ((m_speed > 0) && (m_Start > 0) && (dts < m_Start*m_TimeScale)) {
					dts = cts = m_Start*m_TimeScale;
					/*turn off buffering*/
					buffer++;
				} else {
					m_bIsSeeking = FALSE;
				}
			}

			m_status = writing;

			if (m_speed > 0) {
				slph.compositionTimeStamp = cts;
				if (AUFlags & MP4MediaSampleHasCTSOffset) {
					slph.decodingTimeStamp = dts;
					slph.decodingTimeStampFlag = 1;
				} else {
					slph.decodingTimeStamp = 0;
					slph.decodingTimeStampFlag = 0;
				}
			} else {
				if (slph.AU_sequenceNumber) {
					slph.compositionTimeStamp = m_Duration*m_TimeScale - cts;
				} else {
					slph.compositionTimeStamp = 0;
				}
				slph.decodingTimeStamp = 0;
				slph.decodingTimeStampFlag = 0;
			}

			slph.accessUnitLength = AUSize;
			slph.AU_sequenceNumber ++;
			slph.packetSequenceNumber++;
			//seq num is hardcoded to 5 bits in ESD headers generated by libiso 
			slph.packetSequenceNumber &= 0x1F;
			slph.randomAccessPointFlag = ( (AUFlags & MP4MediaSampleNotSync) == 0 ) ? 1 : 0;

			if (bIsFirst) {
				bIsFirst = FALSE;
				lastDTS = dts;
			}

			if (!buffer) {
				WaitForSpace();
				buffer = m_AUBufferSize;
			}
			buffer--;
 			lastDTS = dts;
			m_status = idle;

			m_sink->DPI_DataReceive_Ind(m_pService, m_chID, (const BYTE *) *m_sampleH, AUSize, (const BYTE *) &slph, sizeof (struct SLHeader), 0 );
			break;

		//track reader with edit list is broken in lib iso, so we handle bad param
		//as end of file
		case MP4BadParamErr:
		case MP4EOF:
			m_status = eof;
			Sleep(100);
			//signal no data with error to signal end of stream
			m_sink->DPI_DataReceive_Ind(m_pService, m_chID, NULL, 0, NULL, 0, -1);

			err = MP4DisposeTrackReader( m_reader );
			err = MP4CreateTrackReader( m_track, &m_reader );
			break;
		}
	}
	while (err == MP4NoErr );

	RESPONSE eResponse = RESPONSE_REMOTE_ERROR;
	if ( err == MP4NoErr )
		eResponse = RESPONSE_OK;
	err = MP4DisposeTrackReader( m_reader );
	m_reader = NULL;
	m_status = finished;
	m_sink->DPI_ChannelDelete_Cnf( m_pService, m_chID, eResponse );
}
