/*
This software module was originally developed by Apple Computer, Inc.
and edited by Guido Franceschini (CSELT) in the course of development of MPEG-4. 
This software module is an implementation of a part of one or 
more MPEG-4 tools as specified by MPEG-4. 
ISO/IEC gives users of MPEG-4 free license to this
software module or modifications thereof for use in hardware 
or software products claiming conformance to MPEG-4.
Those intending to use this software module in hardware or software
products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company,
the subsequent editors and their companies, and ISO/IEC have no
liability for use of this software module or modifications thereof
in an implementation.
Copyright is not released for non MPEG-4 conforming
products. Apple Computer, Inc. retains full right to use the code for its own
purpose, assign or donate the code to a third party and to
inhibit third parties from using the code for non 
MPEG-4 conforming products.
This copyright notice must be included in all copies or
derivative works. Copyright (c) 1999.
*/
/*
	$Id: MP4TrackReaderClient.h,v 1.1.1.2 2006/04/14 14:12:02 hill Exp $
*/

#if !defined(_MP4TrackreaderClient_h)	// Sentry, use file only if it's not already included.
#define _MP4TrackreaderClient_h

#include "MP4Movies.h"
#include "ztl.h"

class MP4TrackReaderClient
{
	public:
		enum status
		{
			starting,
			idle,
			reading,
			writing,
			buffering,
			finished,
			eof
		};
		enum mode
		{
			play,
			pause,
			stop
		};

		MP4TrackReaderClient( MP4Track theTrack, int ESID, int chID, int nTmpId, struct SLConfigDesc* pslcd, ServiceSink *sink, FileMP4Instance *pService );
		virtual	~MP4TrackReaderClient();
		
		void SendAddCnf();

		status	getStatus();
		void	requestPlay();
		void	requestPause(BOOL bReset = FALSE);
		void	requestStop();
		int		getchID() { return m_chID; }
		int		getTmpID() { return m_nTmpId; }
		
		int		StartReaderThread () {return m_readerThread.Begin ();}
		void	WaitForThread () {m_readerThread.Wait ();}

		void SetRange(double start, double end)
		{
			if (m_bIsPrerolling) return;
			m_Start = start;
			m_End = (end>0) ? end : m_Duration;
			m_bIsSeeking = TRUE;
			m_bReset = TRUE;
		}
		void SetSpeed(double speed) 
		{
			m_speed = speed;
			if (m_mode == pause) {
#ifdef _DEBUG
				assert(m_smpMode.V());
#else
				m_smpMode.V();
#endif
			}
		}

		MP4TrackReader	m_reader;
		double GetDuration() {return m_Duration;}

		float m_bufferOccupancy;

		BOOL m_bIsPrerolling;

	private:
		void readerLoop();
		void WaitForSpace();

		int				m_id;
		MP4Track		m_track;
		int				m_chID;
		int				m_nTmpId;
		ServiceSink*	m_sink;
		FileMP4Instance*		m_pService;
		MP4Handle		m_sampleH;
		status			m_status;
		mode			m_mode;
		BOOL m_bReset;
		struct SLConfigDesc	m_slcd;
		ZSemaphore		m_smpMode;
		ZThread<MP4TrackReaderClient>	m_readerThread;

		u32 m_TimeScale;
		double m_Duration, m_speed;
		double m_Start, m_End;
		BOOL m_bIsSeeking;
		u32 m_AUBufferSize;
		u32 m_sampleCount;
		BOOL m_bTrackIsAudio;

};

#endif