/***********************************************************************
    This software module  was  originally  developed by Zvi  Lifshitz
    (Triton R&D Ltd.) and edited by Mauro Rossi (CSELT),
    Guido Franceschini (CSELT) and Michael Stepping (FTK)
    in the course of development of the MPEG-4 DMIF (ISO/IEC 14496-6)
    standard.  This software module is an implementation of a part of
    one or more MPEG-4  DMIF  (ISO/IEC 14496-6)  tools  as  specified
    by  the  MPEG-4 DMIF (ISO/IEC 14496-6)  standard.  ISO/IEC  gives
    users of  the  MPEG-4  DMIF  (ISO/IEC 14496-6)  free  license  to
    this software module or modifications thereof for use in hardware
    or software products claiming  conformance  to  the  MPEG-4  DMIF 
    (ISO/IEC 14496-6). Those intending to use this software module in
    hardware or software  products  are  advised  that  its  use  may
    infringe   existing  patents.  The  original  developer  of  this
    software module and his/her company, the subsequent  editors  and
    their  companies,  and  ISO/IEC have no liability for use of this
    software module or modifications thereof  in  an  implementation.
    Copyright  is not released for non MPEG-4 DMIF  (ISO/IEC 14496-6)
    conforming  products. Triton R&D Ltd.
    retains full  right to use the code for  its  own purpose, assign
    or donate the code to a third party and to inhibit third  parties
    from  using  the  code  for  non  MPEG-4  DMIF  (ISO/IEC 14496-6)
    conforming products. This copyright notice must  be  included  in
    all copies or derivative works. Copyright  1998.
************************************************************************/

// Contact: guido.franceschini@cselt.it

/**************************************************************************
 * Project Im1Player
 * INTERNATIONAL STANDARD ORGANIZATION
 * ISO/IEC JTC1/SC29/WG11
 * 
 * SUBSYSTEM: DMIF
 * FILE:      FileTrifInstance.h
 * AUTHOR:    Mauro Rossi (CSELT)
 * OVERVIEW:  Definitions of FileTrifInstance, the class that implements
 *			  the DMIF instance for local storage with Trifial File Format.
 * 
 **************************************************************************/

#if !defined(_FileTrifInstance_h)	// Sentry, use file only if it's not already included.
#define _FileTrifInstance_h

#include "DPI.h"
#include "FlexMuxTool.h"

class FileTrifInstance : public Service
{
public:
	FileTrifInstance(FileTrifInstance* pMain = NULL, int nID = 0);
	virtual ~FileTrifInstance();

private:
	void RewindFile ();
	virtual BOOL IsYourURL( LPCSTR szURL );
	virtual LPCSTR GetModuleID();

	virtual void DPI_ServiceAttach_Req(
		const LPCSTR	szURL, 
		const BYTE*		pUuData,
		const int		nUuDataLen
		);
	virtual void DPI_ServiceDetach_Req(
		const REASON	eReason
		);
	virtual void DPI_ChannelAdd_Req(
		const DAIChannelAddParam	&inParam
		);
	virtual void DPI_ChannelDelete_Req(
		const DAIChannelDeleteParam	&inParam
		);
	virtual void DPI_UserCommand_Req(
		const DAIUserCmdParam&	inParam
		);
	virtual	void DPI_UserCommandAck_Req(
		const DAIUserCmdParam&	outParam
		);
	virtual BOOL DPI_DataSend_Req(
		const unsigned int		nChid,
		const BYTE*				pStreamData,
		const int				nStreamDataLength,
		const BYTE*				pAppData,
		const int				nAppDataLength
		) {nChid;pStreamData;nStreamDataLength;pAppData;nAppDataLength;return FALSE;};
///
	virtual void DPI_ServiceAttach_Rsp(
		const BYTE*				pUuDataOutBuffer, 
		const int				nUuDataOutLen,
		const RESPONSE			eResponse
		);
	virtual void DPI_ServiceDetach_Rsp(
		const RESPONSE			eResponse
		);
	virtual FlexMuxDataParam* DPI_ChannelAdd_Rsp(
		const OnChannelsAddParam& outParam
		);
	virtual FlexMuxDataParam* DPI_TransmuxAdd_Rsp(
		const OnTransmuxAddParam& outParam
		);
	virtual void DPI_ChannelDelete_Rsp(
		const OnChannelsDeleteParam& outParam
		);
	virtual void DPI_UserCommandAck_Rsp(
		const DAIUserCmdParam&	outParam
		);
///

	RESPONSE Connect (LPCSTR szURL, ZString *initialOD = NULL);
	void Disconnect ();
	void OpenChannel (BYTE* pData, int nDataLen, UINT nChid, UINT nTmpId);
	bool IsMyFile (LPCSTR szFileName);
	bool CloseChannels ( const DAIChannelDeleteParam &inParam ) ;


	void PlayChannel (int nTag);
	void Rewind ();
	void StopChannel (int nTag, RESPONSE nTerminationCode);
	void FileThread ();
	 int GetByte ();
	 int EsidFromChid(UINT chid);
	void ChidEsidStore(UINT chid,int esid);
	void ChidEsidRemove(UINT chid);
	 int TagFromChid(UINT chid);
	 int TagFromData(BYTE *pData, int nDataLen);
	void ReadMuxCodeTableFromFile(LPCSTR szFile,LinearMuxCodeTable &mcte);

private:				// member variables.
	ZString m_url;
	FileTrifInstance* m_pMain;
	ZArray<FileTrifInstance*> m_auxFiles;
	int m_nID;

	FILE *m_fInput;
	BOOL m_bActive, m_bRewinding, m_bTerminate;
	ZMutex m_mtxChannels;
	int m_nChannelWaitedFor, m_nNotReady;
	ZSemaphore m_smpWaitForChannel, m_smpActive;

	struct ChannelAssociation
	{	bool m_bReady, m_bNeedRewind, m_bInProcess;
		BYTE *m_pData;
		int m_nDataLen;
		int m_nESid;
		UINT m_nChid;
		ChannelAssociation (int nESid = 0) :
			m_nESid (nESid), m_bReady (false), m_bNeedRewind (false),
			m_bInProcess (false), m_nChid (0), m_nDataLen(0), m_pData(NULL)
		{
			if(nESid)
			{	m_pData = new BYTE[2];
// SOLARIS PORTING - Changing the memory copy operation - due to LITTLE/BIG endian trouble
// The changed code should be Platform independent
				
				// change back by Liao Xuan, to fix Error"Open Stream Failed"
				memcpy(m_pData,&m_nESid,2);
				//m_pData[0] = HIBYTE(m_nESid);
				//m_pData[1] = LOBYTE(m_nESid);
				// change back by Liao Xuan, to fix Error"Open Stream Failed"
				m_nDataLen = 2;
			}
		}
		~ChannelAssociation ()
		{
			if(m_pData)
				delete[] m_pData;
		}
		operator int () const {return m_nESid;}
	
		ChannelAssociation(const ChannelAssociation& o)//copy constructor
		{
			m_bReady		= o.m_bReady;
			m_bNeedRewind	= o.m_bNeedRewind;
			m_nESid			= o.m_nESid;
			m_nChid			= o.m_nChid;
			m_nDataLen		= o.m_nDataLen;
			if ( m_nDataLen > 0 )
			{
				m_pData		= new BYTE[o.m_nDataLen];
				memcpy(m_pData,o.m_pData,o.m_nDataLen);
			}
			else
				m_pData = NULL ;
		}

		ChannelAssociation& operator=(const ChannelAssociation& r)
		{
			m_bReady		= r.m_bReady;
			m_bNeedRewind	= r.m_bNeedRewind;
			m_nESid			= r.m_nESid;
			m_nChid			= r.m_nChid;
			m_nDataLen		= r.m_nDataLen;
			if ( m_pData )
				delete [] m_pData ;
			if ( m_nDataLen > 0 )
			{
				m_pData		= new BYTE[r.m_nDataLen];
				memcpy(m_pData,r.m_pData,m_nDataLen);
			}
			else
				m_pData = NULL ;
			return ( *this ) ;
		}
	};
	ZArray<ChannelAssociation> m_channelMap;
	LinearMuxCodeTable m_muxCodeTable;
	ZThread<FileTrifInstance> m_fileThread;

	static LPCSTR s_moduleID;
	static LPCSTR s_myURLScheme;
	static LPCSTR s_myFileExtension;
};

/*-------------------------------------------------------------
Exception object
-------------------------------------------------------------*/
struct FileException
{
	enum ErrorCodes {END_OF_FILE, MEMORY_ERROR} m_nCode;
	FileException (enum ErrorCodes nCode = END_OF_FILE) : m_nCode (nCode) {}
};

#endif                                      // _FileService_h sentry.
