/******************************************************************************
*  SOFTWARE LICENSE AND CONDITIONS
*
*  This software module was originally developed by Yasser Pourmohammadi and 
*  Kambiz Asrar Haghighi (University of British Columbia) in the course of 
*  development of  the streaming functionality for MPEG-4 using the DMIF
*  (ISO/IEC 14496-6) standard. This software module is an implementation 
*  of a part of one or more MPEG-4 tools as specified by MPEG-4. 
*  
*  Users of the MPEG-4 may use the software source code or modifications 
*  thereof only for hardware or software products related to MPEG-4 evaluation
*  and testing.  The use of the source code for any purpose other than 
*  evaluation and testing, including the use of derivative works for the 
*  generation of commercial products, is strictly prohibited.
*  
*  The original developer of this software module and his/her company, 
*  the subsequent editors and their companies, and ISO/IEC have no 
*  liability for use of this software module or modifications thereof 
*  in an implementation. Copyright is not released for non MPEG-4 
*  conforming products. UBC retains full right to use the code for 
*  his/her own purpose, assign or donate the code to a third party and 
*  to inhibit third parties from using the code for non MPEG-4 conforming 
*  products. This copyright notice must be included in all copies or 
*  derivative works.
*
*  Contact:
*
*  Kambiz Asrar Haghighi                 email: kambiz@interchange.ubc.ca
*  Yasser Pourmohammadi                  email: yasserp@ece.ubc.ca
*  Professor Hussein M. Alnuweiri
*  UBC Signal Processing and Multimedia Group
*  http://spmg.ece.ubc.ca/research/MPEG4
*  Vancouver BC Canada V6T1Z4                  
*
*  Copyright  2000  University of British Columbia, Canada
********************************************************************************/

#include "Messages.h"

// ----------------------------------------------------------------------------------------------
// Common functions used by entire DMIF signalling functions
// ----------------------------------------------------------------------------------------------

// Change called address to a usable format
SOCKADDR_IN SetDestAddress(LPCSTR calledAddress)
{
   PHOSTENT phostent = NULL;           // Points to the HOSTENT structure
   char *start, *stopstring;
   int colon = ':';
   int startpoint = 0, value;
   TCHAR szError[100];                 // Error message string
   SOCKADDR_IN destination_sin;
   char ipAddress[15];
   char *port;

   
   // Fill out the server socket's address information.
   destination_sin.sin_family = AF_INET;

   // Retrieve the host information corresponding to the target host name.
   start = strchr (calledAddress, colon);
   if(start == NULL)  
   {
       MessageBox (NULL, TEXT("Invalid Destination Address! Exiting!"), TEXT("Error"), MB_OK);
       exit(1);
   }

   value = start - calledAddress;

   // write value of calledAddress into ipAddress & ensure that last char is NULL
   strncpy (ipAddress, calledAddress, value);
   ipAddress[value]=NULL;

   // find port number
   port = start + 1;
   
   if ((phostent = gethostbyname (ipAddress)) == NULL) 
   {
      wsprintf (szError, TEXT("Unable to get the host name. Error: %d IP address: %s"), 
              WSAGetLastError (), ipAddress);
      MessageBox (NULL, szError, TEXT("Error"), MB_OK);
      WSACleanup ();
      exit(1);
   }

   // Assign the socket IP address.
   memcpy ((char FAR *)&(destination_sin.sin_addr), 
          phostent->h_addr, 
          phostent->h_length);

   // Convert to network ordering.
   destination_sin.sin_port = htons((unsigned short)strtol(port, &stopstring, 10));

   return destination_sin;
}

__int16 bindSockettoPort(SOCKET Socket)
{
   static SOCKADDR_IN local_sin;
   static int local_port = 4800;
   //------------------------------------------------------------------------------------  
   // Associating a local address with ServerSock, so that it can listen to the remote system 
   // on thwe local address
   // binding ServerSock to a local address  HAS NO IMPACT ON SENDTO FUNCION. 
   // Fill out the local socket's address information.
   local_sin.sin_family = AF_INET;
   local_sin.sin_port = htons (local_port);  
   local_sin.sin_addr.s_addr = htonl (INADDR_ANY);
 
   // Associate the local address with WinSocket.
   while ((bind (Socket,(struct sockaddr *) &local_sin,sizeof (local_sin)) == SOCKET_ERROR) && (local_port < 5000)) 
   {
      local_sin.sin_port = htons (++local_port);  
   }
   if (local_port >= 5000)
      return 0;
   else
   {
      local_port++;
      return (local_port-1);
   }
}



///////////////////////////////////////////////////////////////////////////////////////////////////
// Auxiliary functions for DS_SessionSetupConfirmMessage_type & DS_SessionSetupRequestMessage_type
///////////////////////////////////////////////////////////////////////////////////////////////////
void DS_SessionSetupRequestMessage_type::getStream (char * stream, int * length)
{
   char * locationptr;
   int len;
   __int16 dummy=0;

   // data size
   char* messagelenPosition = stream + 10; // starting from 0, the + 10 points to the 11th 
   locationptr = stream;
   // Building stream:
      
   dsmccMessageHeader.appendStream(locationptr,&len);
   locationptr = stream + len;
   // append the other fields to stream,
#ifdef _WIN32
   memcpy(locationptr, &dummy,2);locationptr += 2;
   memcpy(locationptr, &networkSessionId,8);locationptr += 8;
#else
   memcpy(locationptr, &networkSessionId,10);locationptr += 10;
#endif
   memcpy(locationptr, &compatibilityDescriptor,1);locationptr += 1;
   
   *length = (int)locationptr - (int)stream;
   dsmccMessageHeader.messageLength = *length - 12;
   //Correcting the messagelen
   memcpy(messagelenPosition, &(dsmccMessageHeader.messageLength),2);
   return ;
}


void DS_SessionSetupConfirmMessage_type::createObjFromStream(char* stream)
{
   char * locationptr;
   locationptr = stream;  
   dsmccMessageHeader.getHeader( stream);
   locationptr += 12;
   memcpy(&response, locationptr, 2); locationptr += 2;
   memcpy(&compatibilityDescriptor, locationptr, 1); 
}
   
///////////////////////////////////////////////////////////////////////////////////////////////////////
// Auxiliary functions for DS_SessionReleaseRequestMessage_type & DS_SessionReleaseConfirmMessage_type
///////////////////////////////////////////////////////////////////////////////////////////////////////
void DS_SessionReleaseRequestMessage_type::getStream (char * stream, int * length)
{
   char * locationptr;
   int len;
   __int16 dummy=0;

   // data size
   char* messagelenPosition = stream + 10; // starting from 0, the + 10 points to the 11th 
   locationptr = stream;

   // Building stream:
   dsmccMessageHeader.appendStream(locationptr,&len);
   locationptr = stream + len;
   // append the other fields to stream,
#ifdef _WIN32
   memcpy(locationptr, &dummy,2);locationptr += 2;
   memcpy(locationptr, &networkSessionId,8);locationptr += 8;
#else
   memcpy(locationptr, &networkSessionId,10);locationptr += 10;
#endif
   memcpy(locationptr, &reason,2);locationptr += 2;

   *length = (int)locationptr - (int)stream;
   dsmccMessageHeader.messageLength = *length - 12;
   //Correcting the messagelen
   memcpy(messagelenPosition, &(dsmccMessageHeader.messageLength),2);
   return ;
}


void DS_SessionReleaseConfirmMessage_type::createObjFromStream(char* stream)
{
   char * locationptr;
   locationptr = stream;  
   dsmccMessageHeader.getHeader( stream);
   locationptr += 12;
   memcpy(&response, locationptr, 2); 
}

////////////////////////////////////////////////////////////////////////////////////////////////////
// Auxiliary functions for DS_ServiceAttachRequestMessage_type & DS_ServiceAttachConfirmMessage_type
////////////////////////////////////////////////////////////////////////////////////////////////////
void DS_ServiceAttachRequestMessage_type::getStream (char * stream, int * length)
{
   char * locationptr;
   int len;
   __int16 dummy=0;
   int ddDatalength;  
   char* messagelenPosition = stream + 10; // starting from 0, the + 10 points to the 11th 
   
   ddDatalength = ddData->getSize();

   locationptr = stream;
   // Building stream:
      
   dsmccMessageHeader.appendStream(locationptr,&len);
   locationptr = stream + len;
   // append the other fields to stream,
#ifdef _WIN32
   memcpy(locationptr, &dummy,2);locationptr += 2;
   memcpy(locationptr, &networkSessionId,8);locationptr += 8;
#else
   memcpy(locationptr, &networkSessionId,10);locationptr += 10;
#endif
   memcpy(locationptr, &serviceId,2);locationptr += 2;
   memcpy(locationptr, &serviceNameLen,1);locationptr += 1;
   memcpy(locationptr, serviceName,serviceNameLen);locationptr += serviceNameLen;
   
   // Append ddData
   appendDDdata(locationptr, ddData, &len);
   locationptr += len;

   *length = (int)locationptr - (int)stream;

   dsmccMessageHeader.messageLength = *length - 12;
   //Correcting the messagelen
   memcpy(messagelenPosition, &(dsmccMessageHeader.messageLength),2);
   return;
}

void DS_ServiceAttachConfirmMessage_type::createObjFromStream(char* stream)
{
   int len;
   char * locationptr;
   locationptr = stream;  
   dsmccMessageHeader.getHeader( stream);
   locationptr += 12;
   memcpy(&response, locationptr, 2); locationptr += 2;
  
   if( !getDDdata(ddData, locationptr, &len) ){
      MessageBox (NULL, TEXT("ddData corrupted, repeat request ..."), TEXT(" DATA ERROR "), MB_ICONERROR);
      return;
   };
   locationptr += len;
   return;

}


////////////////////////////////////////////////////////////////////////////////////////////////////
// Auxiliary functions for DS_ServiceDetachRequestMessage_type & DS_ServiceDetachConfirmMessage_type
////////////////////////////////////////////////////////////////////////////////////////////////////
void DS_ServiceDetachRequestMessage_type::getStream (char * stream, int * length)
{
   char * locationptr;
   int len;
   __int16 dummy=0;

   // data size
   char* messagelenPosition = stream + 10; // starting from 0, the + 10 points to the 11th 


   locationptr = stream;
   // Building stream:
      
//   memcpy(locationptr, dsmccMessageHeader.getStream(&len), 12);
   dsmccMessageHeader.appendStream(locationptr,&len);
   locationptr = stream + len;
   // append the other fields to stream,
#ifdef _WIN32
   memcpy(locationptr, &dummy,2);locationptr += 2;
   memcpy(locationptr, &networkSessionId,8);locationptr += 8;
#else
   memcpy(locationptr, &networkSessionId,10);locationptr += 10;
#endif
   memcpy(locationptr, &serviceId,2);locationptr += 2;
   memcpy(locationptr, &reason,2);locationptr += 2;

   *length = (int)locationptr - (int)stream;
   dsmccMessageHeader.messageLength = *length - 12;
   //Correcting the messagelen
   memcpy(messagelenPosition, &(dsmccMessageHeader.messageLength),2);
   return;
}


void DS_ServiceDetachConfirmMessage_type::createObjFromStream(char* stream)
{
   char * locationptr;
   locationptr = stream;  
   dsmccMessageHeader.getHeader( stream);
   locationptr += 12;
   memcpy(&response, locationptr, 2); 
}
   

////////////////////////////////////////////////////////////////////////////////////////////////////
// Auxiliary functions for DS_TransMuxSetupRequestMessage_type & DS_TransMuxSetupConfirmMessage_type
////////////////////////////////////////////////////////////////////////////////////////////////////
void DS_TransmuxSetupRequestMessage_type::getStream (char * stream, int * length)
{
   char * locationptr;
   int len;
   __int16 dummy=0;
   __int8 DescCount = 0;
   char* messagelenPosition = stream + 10; // starting from 0, the + 10 points to the 11th 

   locationptr = stream;
   // Building stream:      
   dsmccMessageHeader.appendStream(locationptr,&len);
   locationptr = stream + len;
   // append the other fields to stream,
#ifdef _WIN32
   memcpy(locationptr, &dummy,2);locationptr += 2;
   memcpy(locationptr, &networkSessionId,8);locationptr += 8;
#else
   memcpy(locationptr, &networkSessionId,10);locationptr += 10;
#endif
   memcpy(locationptr, &LoopCount,1);locationptr += 1;
 
   // Append qosDescriptor data
   for (int j = 0; j < LoopCount; j++) {
      memcpy(locationptr,&(p_TransmuxReqLoop->operator %(j)->TAT),2);locationptr+=2;
      memcpy(locationptr,&(p_TransmuxReqLoop->operator %(j)->direction),1);locationptr+=1;
      DescCount = p_TransmuxReqLoop->operator %(j)->qosDescriptor.getDescCount();
      memcpy(locationptr,&(DescCount),1);locationptr+=1;
      for( int k =0; k< p_TransmuxReqLoop->operator %(j)->qosDescriptor.getDescCount(); k++){
            /*
            for(i=0; iI<QoS_QualifierCount; i++) {	Count=4
            QoS_QualifierTag	1
            QoS_QualifierDataLength	1
            QoS_QualifierData	QoS_QualifierDataLength
            */
         memcpy(locationptr, &(p_TransmuxReqLoop->operator %(j)->qosDescriptor.DAPtr->operator %(k)->Type),1);locationptr+=1;
         memcpy(locationptr, &(p_TransmuxReqLoop->operator %(j)->qosDescriptor.DAPtr->operator %(k)->Len),1);locationptr+=1;
         memcpy(locationptr,&(p_TransmuxReqLoop->operator %(j)->qosDescriptor.DAPtr->operator %(k)->DataFields),
                p_TransmuxReqLoop->operator %(j)->qosDescriptor.DAPtr->operator %(k)->Len);
         locationptr += p_TransmuxReqLoop->operator %(j)->qosDescriptor.DAPtr->operator %(k)->Len;
      }
      /* ResourceDescriptorType
         ResourceLength
         ResourceDataFieldCount

      */
      memcpy(locationptr,&(p_TransmuxReqLoop->operator %(j)->resourceDescriptor.ResourceDescriptorCount),2);locationptr+=2;
      memcpy(locationptr,
         &(p_TransmuxReqLoop->operator %(j)->resourceDescriptor.IPresourceDescriptor.commonDescriptorHeader.resourceDescriptorType),2);locationptr+=2;
      memcpy(locationptr,&(p_TransmuxReqLoop->operator %(j)->resourceDescriptor.IPresourceDescriptor.commonDescriptorHeader.resourceLength),2);locationptr+=2;
      memcpy(locationptr,&(p_TransmuxReqLoop->operator %(j)->resourceDescriptor.IPresourceDescriptor.commonDescriptorHeader.resourceDataFieldCount),2);locationptr+=2;

      memcpy(locationptr,
         &(p_TransmuxReqLoop->operator %(j)->resourceDescriptor.IPresourceDescriptor.resourceDescriptorDataFields.destinationIpAddress),4);locationptr+=4;
      memcpy(locationptr,
         &(p_TransmuxReqLoop->operator %(j)->resourceDescriptor.IPresourceDescriptor.resourceDescriptorDataFields.destinationIpPort),2);locationptr+=2;
      memcpy(locationptr,
         &(p_TransmuxReqLoop->operator %(j)->resourceDescriptor.IPresourceDescriptor.resourceDescriptorDataFields.ipProtocol),2);locationptr+=2;
      memcpy(locationptr,
         &(p_TransmuxReqLoop->operator %(j)->resourceDescriptor.IPresourceDescriptor.resourceDescriptorDataFields.sourceIpAddress),4);locationptr+=4;
      memcpy(locationptr,
         &(p_TransmuxReqLoop->operator %(j)->resourceDescriptor.IPresourceDescriptor.resourceDescriptorDataFields.sourceIpPort),2);locationptr+=2;
      
   }
      
   *length = (int)locationptr - (int)stream;
   dsmccMessageHeader.messageLength = *length - 12;
   //Correcting the messagelen
   memcpy(messagelenPosition, &(dsmccMessageHeader.messageLength),2);
   return;
}


void DS_TransmuxSetupConfirmMessage_type::createObjFromStream(char* stream)
{
/*    dsmccMessageHeader()	
      count	1B
      loop(count) {	
	      response	2B
	      resources() 	
         }
*/	

   char * locationptr;
   locationptr = stream;  
   dsmccMessageHeader.getHeader( stream);
   locationptr += 12;
   memcpy (&LoopCount, locationptr, 1); locationptr += 1;
   p_TransmuxConfLoop->setLoopsize(LoopCount);
   for(int i=0;i < LoopCount;i++){
      memcpy(&(p_TransmuxConfLoop->operator %(i)->response),locationptr, 2); locationptr += 2;
      memcpy(&(p_TransmuxConfLoop->operator %(i)->resourceDescriptor.ResourceDescriptorCount),locationptr,2);locationptr += 2;
      /* DEV */ // Since there is only one resource (IP) , the "resources" field of the loop contains only one descriptor.
      memcpy(&(p_TransmuxConfLoop->operator %(i)->resourceDescriptor.IPresourceDescriptor.commonDescriptorHeader.resourceDescriptorType),locationptr,2);locationptr += 2;
      memcpy(&(p_TransmuxConfLoop->operator %(i)->resourceDescriptor.IPresourceDescriptor.commonDescriptorHeader.resourceLength),locationptr,2);locationptr += 2;
      memcpy(&(p_TransmuxConfLoop->operator %(i)->resourceDescriptor.IPresourceDescriptor.commonDescriptorHeader.resourceDataFieldCount),locationptr,2);locationptr += 2;
      memcpy(&(p_TransmuxConfLoop->operator %(i)->resourceDescriptor.IPresourceDescriptor.resourceDescriptorDataFields.destinationIpAddress),locationptr,4);locationptr+=4;
      memcpy(&(p_TransmuxConfLoop->operator %(i)->resourceDescriptor.IPresourceDescriptor.resourceDescriptorDataFields.destinationIpPort),locationptr,2);locationptr+=2;
      memcpy(&(p_TransmuxConfLoop->operator %(i)->resourceDescriptor.IPresourceDescriptor.resourceDescriptorDataFields.ipProtocol),locationptr,2);locationptr+=2;
      memcpy(&(p_TransmuxConfLoop->operator %(i)->resourceDescriptor.IPresourceDescriptor.resourceDescriptorDataFields.sourceIpAddress),locationptr,4);locationptr+=4;
      memcpy(&(p_TransmuxConfLoop->operator %(i)->resourceDescriptor.IPresourceDescriptor.resourceDescriptorDataFields.sourceIpPort),locationptr,2);locationptr+=2;
   }
}


////////////////////////////////////////////////////////////////////////////////////////////////////
// Auxiliary functions for DS_TransMuxReleaseRequestMessage_type & DS_TransMuxReleaseConfirmMessage_type
////////////////////////////////////////////////////////////////////////////////////////////////////
void DS_TransmuxReleaseRequestMessage_type::getStream (char * stream, int * length)
{
   char * locationptr;
   int len;
   __int16 dummy=0;
   __int8 DescCount = 0;
   char* messagelenPosition = stream + 10; // starting from 0, the + 10 points to the 11th 

   locationptr = stream;
   // Building stream:      
   dsmccMessageHeader.appendStream(locationptr,&len);
   locationptr = stream + len;
   // append the other fields to stream,
#ifdef _WIN32
   memcpy(locationptr, &dummy,2);locationptr += 2;
   memcpy(locationptr, &networkSessionId,8);locationptr += 8;
#else
   memcpy(locationptr, &networkSessionId,10);locationptr += 10;
#endif
   memcpy(locationptr, &LoopCount,1);locationptr += 1;
 
   // Append qosDescriptor data
   for (int j = 0; j < LoopCount; j++) {
      memcpy(locationptr,&(p_TransmuxReqLoop->operator %(j)->TAT),2);locationptr+=2;
     }
    
   *length = (int)locationptr - (int)stream;
   dsmccMessageHeader.messageLength = *length - 12;
   //Correcting the messagelen
   memcpy(messagelenPosition, &(dsmccMessageHeader.messageLength),2);
   return;
}


void DS_TransmuxReleaseConfirmMessage_type::createObjFromStream(char* stream)
{
/*    dsmccMessageHeader()	
      count	1B
      loop(count) {	
	      response	2B
         }
*/	

   char * locationptr;
   locationptr = stream;  
   dsmccMessageHeader.getHeader( stream);
   locationptr += 12;
   memcpy (&LoopCount, locationptr, 1); locationptr += 1;
   p_TransmuxConfLoop->setLoopsize(LoopCount);
   for(int i=0;i < LoopCount;i++){
      memcpy(&(p_TransmuxConfLoop->operator %(i)->response),locationptr, 2); locationptr += 2;
   }
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Auxiliary functions for DS_TransmuxConfigRequestMessage_type & DS_TransmuxConfigConfirmMessage_type
////////////////////////////////////////////////////////////////////////////////////////////////
void DS_TransmuxConfigRequestMessage_type::getStream (char * stream, int * length)
{
   char * locationptr;
   int len;
   __int16 dummy=0;

   char* messagelenPosition = stream + 10;// starting from 0 , the + 10 points to the 11th 
   //................................... now DS_Message is ready to used for building the stream
   locationptr = stream;
   
   // Building stream:
      
   dsmccMessageHeader.appendStream(locationptr,&len);// Message Len is wrong now but will be corrected later in the end of this func.
   locationptr = stream + len;
   // append the other fields to stream,
#ifdef _WIN32
   memcpy(locationptr, &dummy,2);locationptr += 2;
   memcpy(locationptr, &networkSessionId,8);locationptr += 8;
#else
   memcpy(locationptr, &networkSessionId,10);locationptr += 10;
#endif

   memcpy(locationptr, &LoopCount,1);locationptr += 1;

   for (int j = 0; j < LoopCount; j++) { // Loop(count)
      memcpy(locationptr, &(p_TransmuxReqLoop->operator %(j)->TAT),2);locationptr += 2;
                  
      // Append ddData
      appendDDdata(locationptr, &(p_TransmuxReqLoop->operator %(j)->ddData), &len);
      locationptr += len;
   }

   *length = (int)locationptr - (int)stream;
   dsmccMessageHeader.messageLength = *length - 12;
   //Correcting the messagelen
   memcpy(messagelenPosition, &(dsmccMessageHeader.messageLength),2);
   return;
}


void DS_TransmuxConfigConfirmMessage_type::createObjFromStream(char* stream)
{ // Changes to * ConfLoop are directly applied to the response loop in RemoteInstance (since the GenericMsgLoop 
  // pointer in this message is pointing to that loop;

   char * locationptr;
   locationptr = stream;
   dsmccMessageHeader.getHeader(stream);
   locationptr += 12;
   memcpy (&LoopCount,locationptr, 1); locationptr += 1;
   p_TransmuxConfLoop->setLoopsize(LoopCount);
   for(int i=0;i< LoopCount;i++){
      memcpy(&(p_TransmuxConfLoop->operator %(i)->response),locationptr,2); locationptr += 2;
   }
   return;
} 



////////////////////////////////////////////////////////////////////////////////////////////////
// Auxiliary functions for DS_ChannelAddRequestMessage_type & DS_ChannelAddConfirmMessage_type
////////////////////////////////////////////////////////////////////////////////////////////////
void DS_ChannelAddRequestMessage_type::getStream (char * stream, int * length)
{
   char * locationptr;
   int len;
   __int16 dummy=0;
   //int channelDescLength, ddDatalength;
      

   //dsmccMessageHeader.messageLength = 10 + 2 + 1 + (3*count) + ddDatalength + channelDescLength;
   // The message Length Can be calculated after making up the stream -> MsgLen = StreamLen - 12;  Therefore its address is saved for correction
   char* messagelenPosition = stream + 10;// starting from 0 , the + 10 points to the 11th 

   //................................... now DS_Message is ready to used for building the stream
   locationptr = stream;
   
   // Building stream:
      
   dsmccMessageHeader.appendStream(locationptr,&len);// Message Len is wrong now but will be corrected later in the end of this func.
   locationptr = stream + len;
   // append the other fields to stream,
#ifdef _WIN32
   memcpy(locationptr, &dummy,2);locationptr += 2;
   memcpy(locationptr, &networkSessionId,8);locationptr += 8;
#else
   memcpy(locationptr, &networkSessionId,10);locationptr += 10;
#endif
   memcpy(locationptr, &serviceId,2);locationptr += 2;
   memcpy(locationptr, &LoopCount,1);locationptr += 1;

   for (int j = 0; j < LoopCount; j++) { // Loop(count)
      memcpy(locationptr, &(p_CHReqLoop->operator %(j)->CAT),2);locationptr += 2;
      memcpy(locationptr, &(p_CHReqLoop->operator %(j)->direction),1);locationptr += 1;
      //Append channelDescriptor Data      
      __int8 dummyDescriptorCount = 0;
      memcpy(locationptr, &dummyDescriptorCount/*&(p_CHReqLoop->operator %(j)->channelDescriptor.DescriptorCount)*/,1);locationptr +=1;  
      /* YK */ // Channel descriptor count is assumed to be 0 , means there is no channel descriptor.
      /* DEV */ 

      // Append ddData
      appendDDdata(locationptr, &(p_CHReqLoop->operator %(j)->ddData), &len);
      locationptr += len;
   }

   *length = (int)locationptr - (int)stream;
   dsmccMessageHeader.messageLength = *length - 12;
   //Correcting the messagelen
   memcpy(messagelenPosition, &(dsmccMessageHeader.messageLength),2);
   return;
}


void DS_ChannelAddConfirmMessage_type::createObjFromStream(char* stream)
{ // Changes to * ConfLoop are directly applied to the response loop in RemoteInstance (since the GenericMsgLoop 
  // pointer in this message is pointing to that loop;

   int len;
   char * locationptr;
   locationptr = stream;
   dsmccMessageHeader.getHeader(stream);
   locationptr += 12;
   memcpy (&LoopCount,locationptr, 1); locationptr += 1;
   p_CHCnfLoop->setLoopsize(LoopCount);
   for(int i=0;i< LoopCount;i++){
      memcpy(&(p_CHCnfLoop->operator %(i)->response),locationptr,2); locationptr += 2;
      memcpy(&(p_CHCnfLoop->operator %(i)->TAT),locationptr,2); locationptr += 2;
      // Extracting ddData
      if (!getDDdata(&(p_CHCnfLoop->operator %(i)->ddData), locationptr, &len) ){
         MessageBox (NULL, TEXT("ddData corrupted, repeat request ..."), TEXT(" DATA ERROR !"), MB_ICONERROR);
         break;
      }
      locationptr += len;
   }
   
   return;
}


////////////////////////////////////////////////////////////////////////////////////////////////
// Auxiliary functions for DS_ChannelDeleteRequestMessage_type & DS_ChannelDeleteConfirnmMessage_type
////////////////////////////////////////////////////////////////////////////////////////////////
void DS_ChannelDeleteRequestMessage_type::getStream (char * stream, int * length)
{
   char * locationptr;
   int len;
   __int16 dummy=0;
   
   char* messagelenPosition = stream + 10;// starting from 0 , the + 10 points to the 11th 
   //................................... now DS_Message is ready to used for building the stream
   locationptr = stream;
  
   // Building stream:
      
   dsmccMessageHeader.appendStream(locationptr,&len);// Message Len is wrong now but will be corrected later in the end of this func.
   locationptr = stream + len;
   // append the other fields to stream,
#ifdef _WIN32
   memcpy(locationptr, &dummy,2);locationptr += 2;
   memcpy(locationptr, &networkSessionId,8);locationptr += 8;
#else
   memcpy(locationptr, &networkSessionId,10);locationptr += 10;
#endif

   memcpy(locationptr, &LoopCount,1);locationptr += 1;

   for (int j = 0; j < LoopCount; j++) { // Loop(count)
      memcpy(locationptr, &(p_CHReqLoop->operator %(j)->CAT),2);locationptr += 2;
      memcpy(locationptr, &(p_CHReqLoop->operator %(j)->reason),2);locationptr += 2;
   }
   *length = (int)locationptr - (int)stream;
   dsmccMessageHeader.messageLength = *length - 12;
   //Correcting the messagelen
   memcpy(messagelenPosition, &(dsmccMessageHeader.messageLength),2);
   return;
}


void DS_ChannelDeleteConfirmMessage_type::createObjFromStream(char* stream)
{ // Changes to * ConfLoop are directly applied to the response loop in RemoteInstance (since the GenericMsgLoop 
  // pointer in this message is pointing to that loop;

   char * locationptr;
   locationptr = stream;
   dsmccMessageHeader.getHeader(stream);
   locationptr += 12;
   memcpy (&LoopCount,locationptr, 1); locationptr += 1;
   p_CHCnfLoop->setLoopsize(LoopCount);
   for(int i=0;i< LoopCount;i++){
      memcpy(&(p_CHCnfLoop->operator %(i)->response),locationptr,2); locationptr += 2;
   }
 
   return;
}

////////////////////////////////////////////////////////////////////////////////////////////////
// Auxiliary functions for DS_ChannelAddedRequestMessage_type & DS_ChannelAddedConfirmMessage_type
////////////////////////////////////////////////////////////////////////////////////////////////
void DS_ChannelAddedRequestMessage_type::getStream (char * stream, int * length)
{
   char * locationptr;
   int len;
   __int16 dummy=0;
   char* messagelenPosition = stream + 10;// starting from 0 , the + 10 points to the 11th 
   //................................... now DS_Message is ready to used for building the stream
   locationptr = stream;
   // Building stream:
   dsmccMessageHeader.appendStream(locationptr,&len);// Message Len is wrong now but will be corrected later in the end of this func.
   locationptr = stream + len;
   // append the other fields to stream,
#ifdef _WIN32
   memcpy(locationptr, &dummy,2);locationptr += 2;
   memcpy(locationptr, &networkSessionId,8);locationptr += 8;
#else
   memcpy(locationptr, &networkSessionId,10);locationptr += 10;
#endif
   memcpy(locationptr, &serviceId,2);locationptr += 2;
   memcpy(locationptr, &LoopCount,1);locationptr += 1;

   for (int j = 0; j < LoopCount; j++) { // Loop(count)
      memcpy(locationptr, &(p_CHReqLoop->operator %(j)->CAT),2);locationptr += 2;
      memcpy(locationptr, &(p_CHReqLoop->operator %(j)->direction),1);locationptr += 1;
      //Append channelDescriptor Data      
      __int8 dummyDescriptorCount = 0;
      memcpy(locationptr, &dummyDescriptorCount/*&(p_CHReqLoop->operator %(j)->channelDescriptor.DescriptorCount)*/,1);locationptr +=1;  
      /* YK */ // Channel descriptor count is assumed to be 0 , means there is no channel descriptor.
      /* DEV */ 
    
      // ADD TAT 
      memcpy(locationptr, &(p_CHReqLoop->operator %(j)->TAT),2);locationptr += 2;

      // Append ddData
      appendDDdata(locationptr, &(p_CHReqLoop->operator %(j)->ddData), &len);
      locationptr += len;
   }

   *length = (int)locationptr - (int)stream;
   dsmccMessageHeader.messageLength = *length - 12;
   //Correcting the messagelen
   memcpy(messagelenPosition, &(dsmccMessageHeader.messageLength),2);
   return;
}


void DS_ChannelAddedConfirmMessage_type::createObjFromStream(char* stream)
{ // Changes to * ConfLoop are directly applied to the response loop in RemoteInstance (since the GenericMsgLoop 
  // pointer in this message is pointing to that loop;
/*
   DS_ChannelAddedConfirm(){	
      dsmccMessageHeader()	
      Count	1
      loop(count) {	
	      Response	2
	      ddData()	
      }	
   }	
*/
   int len;
   char * locationptr;
   locationptr = stream;
   dsmccMessageHeader.getHeader(stream);
   locationptr += 12;
   memcpy (&LoopCount,locationptr, 1); locationptr += 1;
   p_CHCnfLoop->setLoopsize(LoopCount);
   for(int i=0;i< LoopCount;i++){
      memcpy(&(p_CHCnfLoop->operator %(i)->response),locationptr,2); locationptr += 2;
      // Extracting ddData
      if (!getDDdata(&(p_CHCnfLoop->operator %(i)->ddData), locationptr, &len) ){
         MessageBox (NULL, TEXT("ddData corrupted, repeat request ..."), TEXT(" DATA ERROR !"), MB_ICONERROR);
         break;
      }
      locationptr += len;
   }
   
   return;
}

/////////////////////////////////////////////////////////////////////////////////////////////
// Auxiliary functions for DS_UserCommandMessage_type
////////////////////////////////////////////////////////////////////////////////////////////////
void DS_UserCommandMessage_type::getStream (char * stream, int * length)
{
   char * locationptr;
   int len;
   __int16 dummy=0;

   char* messagelenPosition = stream + 10;// starting from 0 , the + 10 points to the 11th 

   //................................... now DS_Message is ready to used for building the stream
   locationptr = stream;
   
   // Building stream:
      
   dsmccMessageHeader.appendStream(locationptr,&len);// Message Len is wrong now but will be corrected later in the end of this func.
   locationptr = stream + len;
   // append the other fields to stream,
#ifdef _WIN32
   memcpy(locationptr, &dummy,2);locationptr += 2;
   memcpy(locationptr, &networkSessionId,8);locationptr += 8;
#else
   memcpy(locationptr, &networkSessionId,10);locationptr += 10;
#endif

   // Append ddData   
   appendDDdata(locationptr, ddData, &len);
   locationptr += len;

   // Append CATloop
   memcpy(locationptr, &LoopCount,1);locationptr += 1;

   for (int j = 0; j < LoopCount; j++) { // Loop(count)
      memcpy(locationptr, &(p_CATLoop->operator %(j)->CAT),2);locationptr += 2;
   }

   *length = (int)locationptr - (int)stream;
   dsmccMessageHeader.messageLength = *length - 12;
   //Correcting the messagelen
   memcpy(messagelenPosition, &(dsmccMessageHeader.messageLength),2);
   return;
}

/////////////////////////////////////////////////////////////////////////////////////////////
// Auxiliary functions for DS_UserCommandAckConfirmMessage_type
////////////////////////////////////////////////////////////////////////////////////////////////
void DS_UserCommandAckConfirmMessage_type::createObjFromStream(char * stream)
{
   /*
   DS_UserCommandAckConfirm(){	
      dsmccMessageHeader()	
      networkSessionId	10
      response	2
      ddData()	
   }	
   */
   int len;
   char * locationptr;
   locationptr = stream;
   dsmccMessageHeader.getHeader(stream);
   locationptr += 12;

#ifdef _WIN32
   locationptr += 2;
   memcpy(&networkSessionId, locationptr,8);locationptr += 8;
#else
   memcpy(&networkSessionId, locationptr,10);locationptr += 10;
#endif
   
   memcpy(&response, locationptr,2);locationptr += 2;
   // Extracting ddData
   if (!getDDdata(ddData, locationptr, &len) ){
      MessageBox (NULL, TEXT("ddData corrupted, repeat request ..."), TEXT(" DATA ERROR !"), MB_ICONERROR);
   }
   locationptr += len;

   return;
}
