/******************************************************************************
*  SOFTWARE LICENSE AND CONDITIONS
*
*  This software module was originally developed by Yasser Pourmohammadi and 
*  Kambiz Asrar Haghighi (University of British Columbia) in the course of 
*  development of  the streaming functionality for MPEG-4 using the DMIF
*  (ISO/IEC 14496-6) standard. This software module is an implementation 
*  of a part of one or more MPEG-4 tools as specified by MPEG-4. 
*  
*  Users of the MPEG-4 may use the software source code or modifications 
*  thereof only for hardware or software products related to MPEG-4 evaluation
*  and testing.  The use of the source code for any purpose other than 
*  evaluation and testing, including the use of derivative works for the 
*  generation of commercial products, is strictly prohibited.
*  
*  The original developer of this software module and his/her company, 
*  the subsequent editors and their companies, and ISO/IEC have no 
*  liability for use of this software module or modifications thereof 
*  in an implementation. Copyright is not released for non MPEG-4 
*  conforming products. UBC retains full right to use the code for 
*  his/her own purpose, assign or donate the code to a third party and 
*  to inhibit third parties from using the code for non MPEG-4 conforming 
*  products. This copyright notice must be included in all copies or 
*  derivative works.
*
*  Contact:
*
*  Kambiz Asrar Haghighi                 email: kambiz@interchange.ubc.ca
*  Yasser Pourmohammadi                  email: yasserp@ece.ubc.ca
*  Professor Hussein M. Alnuweiri
*  UBC Signal Processing and Multimedia Group
*  http://spmg.ece.ubc.ca/research/MPEG4
*  Vancouver BC Canada V6T1Z4                  
*
*  Copyright  2000  University of British Columbia, Canada
********************************************************************************/

#include "..\remoteinstance.h"
#include "DN_udp.h"
#include "DPI.h"
//------------------------------------------------   D N I ---------------------------------------------

// globals

NetSessionListClass NetSessionList; // Declaring the global NetworkSession record list. Information about
                                    // the bound socket and destination address is kept in a record in this list.
//BOOL LISTENTHREAD = TRUE;

void YKTRACE(const char * a, ...)
{
	va_list marker;
	char buf[1024];

	va_start(marker, a);
	vsprintf(buf, a, marker);
	OutputDebugString(buf);
	va_end(marker);
}


// Function to setup Session
BOOL DN_UDP_CLASS::DN_SessionSetup( long double networkSessionId,LPCSTR calledAddress,LPCSTR callingAddress,
                     __int8 compatibilityDescriptorIn, __int16 *response, __int8 *compatibilityDescriptorOut)
{  
   char buf[MAX_MESSAGE_LENGTH];
   SOCKADDR_IN destination_sin;
   SOCKET sigSocket = INVALID_SOCKET;
   // local variables 
   DS_SessionSetupRequestMessage_type DSSessionSetupReq;
   DS_SessionSetupConfirmMessage_type DSSessionSetupCnf;
   int accept_sin_len,iReturn;
   int bufferLength;


   // Create a socket and associate it with networkSessionId;
   if ((sigSocket = socket (AF_INET, SOCK_DGRAM, 0)) == INVALID_SOCKET)
   {
      MessageBox (NULL, TEXT("Allocating socket failed."), TEXT("Error"), MB_OK);
      return FALSE;
   }
   accept_sin_len = sizeof (accept_sin);

   if (!bindSockettoPort(sigSocket))
   {
      MessageBox (NULL, TEXT("Allocating socket failed."), TEXT("Error"), MB_OK);
      return FALSE;
   }

   // change calling address to a usable format
   destination_sin=SetDestAddress(calledAddress);
   
   // make up message data
   DSSessionSetupReq.dsmccMessageHeader.protocolDiscriminator = 0x11;
   DSSessionSetupReq.dsmccMessageHeader.dsmccType = 0x06;
   DSSessionSetupReq.dsmccMessageHeader.reserved = (__int8)0xFF;
   DSSessionSetupReq.dsmccMessageHeader.adaptationLength = 0x00;
   DSSessionSetupReq.dsmccMessageHeader.messageId = SESSIONSETUPREQUEST;
   DSSessionSetupReq.dsmccMessageHeader.transactionId = ++transactionId;
   DSSessionSetupReq.networkSessionId = networkSessionId;
   DSSessionSetupReq.compatibilityDescriptor = (__int8)0xFF;

   
   DSSessionSetupReq.getStream(buf,&bufferLength);
   /* YK */
   // what happens to the memory spaces allocated to "buf". free 
   // Answer: Because the structure that getstream belongs to, is a local structure to "this" fucntion, when DN... returns that 
   // memory space is freed.
   
   // send session setup message
   if( sendto (sigSocket, buf,bufferLength, NULL, (PSOCKADDR) &destination_sin, 
                  sizeof (destination_sin)) == SOCKET_ERROR)  
     {
          MessageBox (NULL, TEXT("Sending data failed."), TEXT("Error"), MB_OK);
          exit(1);
     }
   // blocking call
   // Receive data from the client.
   /* YK */
   // need a type of timeout to retry sending data (assuming send has failed)
   iReturn=recvfrom(sigSocket,buf,MAX_MESSAGE_LENGTH,NULL,(struct sockaddr *) &accept_sin, 
                     (int *) &accept_sin_len);

  // Check if there is any data received. If there is, display it.

   if (iReturn == SOCKET_ERROR)
   {
      MessageBox (NULL, TEXT("No data is received, recvfrom failed."), TEXT("Error"), MB_OK);
   }
   else if (iReturn == 0)
   {
      MessageBox (NULL, TEXT("Connection closed gracefully"), TEXT("Server"),MB_OK);
   }
   else
   {
      DSSessionSetupCnf.createObjFromStream(buf);
      
      // Parse received message
      if ((SESSIONSETUPCONFIRM == DSSessionSetupCnf.dsmccMessageHeader.messageId) &&
          (DSSessionSetupReq.dsmccMessageHeader.transactionId == DSSessionSetupCnf.dsmccMessageHeader.transactionId))
      {
         *response = DSSessionSetupCnf.response;

         // The triplet made up of networkSessionId, its associated sigSocket and destination address is ready 
         // now: Put it in the list
         NetSessionList.addElementstoSessionList(networkSessionId,sigSocket,destination_sin);

         return TRUE;
      }
      else
         return FALSE;
   }
   return FALSE;
} // End of DN_UDP_CLASS::DN_SessionSetup


// DN_UDP_CLASS::DN_SessionRelease

BOOL DN_UDP_CLASS::DN_SessionRelease(long double networkSessionId, __int16 reason, __int16 *response/*out*/)
{
   char buf[MAX_MESSAGE_LENGTH];
   int bufferLength;
   int iReturn;
   SOCKET sigSocket;
   int accept_sin_len;
   DS_SessionReleaseRequestMessage_type DSSessionReleaseReq;
   DS_SessionReleaseConfirmMessage_type DSSessionReleaseCnf;

   accept_sin_len = sizeof (accept_sin);

   // make up message data
   DSSessionReleaseReq.dsmccMessageHeader.protocolDiscriminator = 0x11;
   DSSessionReleaseReq.dsmccMessageHeader.dsmccType = 0x06;
   DSSessionReleaseReq.dsmccMessageHeader.reserved = (__int8)0xFF;
   DSSessionReleaseReq.dsmccMessageHeader.adaptationLength = 0x00;
   DSSessionReleaseReq.dsmccMessageHeader.messageId = SESSIONRELEASEREQUEST;
   DSSessionReleaseReq.dsmccMessageHeader.transactionId = ++transactionId;
   DSSessionReleaseReq.networkSessionId = networkSessionId;
   DSSessionReleaseReq.reason = reason;

   // Revise the following lines:
   DSSessionReleaseReq.getStream(buf,&bufferLength);

   // MessageBox (NULL, TEXT("Session Release"), TEXT("Server"),MB_OK);
   sigSocket = NetSessionList.findSocketfromNSID(networkSessionId);
   // send session release message
   if( sendto (sigSocket, buf,bufferLength, NULL, 
               (PSOCKADDR) &(NetSessionList.findDest_SinfromNSID(networkSessionId)), 
                        sizeof (NetSessionList.findDest_SinfromNSID(networkSessionId))) == SOCKET_ERROR)  
     {
          MessageBox (NULL, TEXT("Sending data failed."), TEXT("Error"), MB_OK);
          exit(1);
     }

   // blocking call
   // Receive data from the client.
   /* YK */
   // need a kind of timeout to retry sending data (assuming send has failed)
   iReturn=recvfrom(sigSocket,buf,MAX_MESSAGE_LENGTH,NULL,(struct sockaddr *) &accept_sin, 
                 (int *) &accept_sin_len);

   // Check if there is any data received. If there is, display it.

   if (iReturn == SOCKET_ERROR)
   {
      TCHAR szError[100];                 // Error message string
      wsprintf (szError, TEXT("recvfrom failed. Error: %d"),WSAGetLastError ());
      MessageBox (NULL, szError, TEXT("Error"), MB_OK);
   }
   else if (iReturn == 0)
   {
      MessageBox (NULL, TEXT("Connection closed gracefully"), TEXT("Server"),MB_OK);
   }
   else
   {
      DSSessionReleaseCnf.createObjFromStream(buf);
      // Parse received message
      if ((SESSIONRELEASECONFIRM == DSSessionReleaseCnf.dsmccMessageHeader.messageId) &&
          (DSSessionReleaseReq.dsmccMessageHeader.transactionId == DSSessionReleaseCnf.dsmccMessageHeader.transactionId))
      {
         *response = DSSessionReleaseCnf.response; 
         if (*response == RESPONSE_GOOD) {
            if (closesocket(sigSocket) == SOCKET_ERROR)
            {
               MessageBox (NULL, TEXT("Closing socket failed."), TEXT("Error"), MB_OK);
               return FALSE;
            }
            NetSessionList.deleteElementfromSessionList(networkSessionId);
            return TRUE;
         }
      }
      else
         return FALSE;
   }
   return FALSE;
} // End of DN_UDP_CLASS::DN_SessionRelease

//------------------------------------------------------------------------------------------------
BOOL DN_UDP_CLASS::DN_ServiceAttach(long double networkSessionId, __int16 serviceId, const char *serviceName,
                     ddDataClass * ddDataIn,__int16 *response/*out*/, ddDataClass * ddDataOut /*out*/)

{
   char buf[MAX_MESSAGE_LENGTH];
   int iReturn;
   int bufferLength;
   int accept_sin_len;
   DS_ServiceAttachRequestMessage_type DSServiceAttachReq;
   DS_ServiceAttachConfirmMessage_type DSServiceAttachCnf;

   // make up message data
   DSServiceAttachReq.networkSessionId = networkSessionId;
   DSServiceAttachReq.dsmccMessageHeader.protocolDiscriminator = 0x11;
   DSServiceAttachReq.dsmccMessageHeader.dsmccType = 0x06;
   DSServiceAttachReq.dsmccMessageHeader.reserved = (__int8)0xFF;
   DSServiceAttachReq.dsmccMessageHeader.adaptationLength = 0x00;
   DSServiceAttachReq.dsmccMessageHeader.messageId = SERVICEATTACHREQUEST;
   DSServiceAttachReq.dsmccMessageHeader.transactionId = ++transactionId;

   // Revise the following lines:
   DSServiceAttachReq.serviceId = serviceId; 
   DSServiceAttachReq.serviceNameLen = strlen(serviceName);
   DSServiceAttachReq.serviceName = (char*)malloc(DSServiceAttachReq.serviceNameLen);
   if (DSServiceAttachReq.serviceName == NULL)
      exit(-11);
   strcpy (DSServiceAttachReq.serviceName , serviceName);
   DSServiceAttachReq.ddData = ddDataIn;


   DSServiceAttachReq.getStream(buf,&bufferLength);

   accept_sin_len = sizeof (accept_sin);

   SOCKET sigSocket = NetSessionList.findSocketfromNSID(networkSessionId);
   // send service attach message
   if( sendto (sigSocket, buf, bufferLength, NULL, (PSOCKADDR) &NetSessionList.findDest_SinfromNSID(networkSessionId), 
                  sizeof (NetSessionList.findDest_SinfromNSID(networkSessionId))) == SOCKET_ERROR)  
     {
          MessageBox (NULL, TEXT("Sending data failed."), TEXT("Error"), MB_OK);
          exit(1);
     }
   // blocking call
   // Receive data from the client.
   /* YK */
   // need a kind of timeout to retry sending data (assuming send has failed)
   iReturn=recvfrom(sigSocket,buf,MAX_MESSAGE_LENGTH,NULL,(struct sockaddr *) &accept_sin, 
                 (int *) &accept_sin_len);

   // Check if there is any data received. If there is, display it.

   if (iReturn == SOCKET_ERROR)
   {
      TCHAR szError[100];                 // Error message string
      wsprintf (szError, TEXT("recvfrom failed. Error: %d"), 
              WSAGetLastError ());
      MessageBox (NULL, szError, TEXT("Error"), MB_OK);
   }
   else if (iReturn == 0)
   {
      MessageBox (NULL, TEXT("Connection closed gracefully"), TEXT("Server"),MB_OK);
   }
   else
   {
      DSServiceAttachCnf.ddData = ddDataOut;
      DSServiceAttachCnf.createObjFromStream(buf);

      // Parse received message
      if ((SERVICEATTACHCONFIRM == DSServiceAttachCnf.dsmccMessageHeader.messageId) &&
          (DSServiceAttachReq.dsmccMessageHeader.transactionId == DSServiceAttachCnf.dsmccMessageHeader.transactionId))
      {
         *response = DSServiceAttachCnf.response;
         return TRUE;
      }
      else
         return FALSE;
   }
   return FALSE;
} // End of DN_UDP_CLASS::DN_ServiceAttach


// DN_UDP_CLASS::DN_ServiceDetach
BOOL DN_UDP_CLASS::DN_ServiceDetach(long double networkSessionId, __int16 serviceId, __int16 reason, __int16 *response)
{
   char buf[MAX_MESSAGE_LENGTH];
   int bufferLength;
   int iReturn;
   int accept_sin_len;
   DS_ServiceDetachRequestMessage_type DSServiceDetachReq;
   DS_ServiceDetachConfirmMessage_type DSServiceDetachCnf;
   
   // make up message data
   DSServiceDetachReq.dsmccMessageHeader.protocolDiscriminator = 0x11;
   DSServiceDetachReq.dsmccMessageHeader.dsmccType = 0x06;
   DSServiceDetachReq.dsmccMessageHeader.reserved = (__int8)0xFF;
   DSServiceDetachReq.dsmccMessageHeader.adaptationLength = 0x00;
   DSServiceDetachReq.networkSessionId = networkSessionId;
   DSServiceDetachReq.dsmccMessageHeader.messageId = SERVICEDETACHREQUEST;
   DSServiceDetachReq.dsmccMessageHeader.transactionId = ++transactionId;

   // Revise the following lines:
   DSServiceDetachReq.serviceId = serviceId; 
   DSServiceDetachReq.reason = reason;
   
   accept_sin_len = sizeof (accept_sin);

   DSServiceDetachReq.getStream(buf,&bufferLength);

   //MessageBox (NULL, TEXT("Service Detach"), TEXT("Server"),MB_OK);

   SOCKET sigSocket = NetSessionList.findSocketfromNSID(networkSessionId);
   // send service Detach message
   if( sendto (sigSocket, buf, bufferLength, NULL, (PSOCKADDR) &NetSessionList.findDest_SinfromNSID(networkSessionId), 
                  sizeof (NetSessionList.findDest_SinfromNSID(networkSessionId))) == SOCKET_ERROR)  
     {
          MessageBox (NULL, TEXT("Sending data failed."), TEXT("Error"), MB_OK);
          exit(1);
     }

   // blocking call to receive response from server
   iReturn=recvfrom(sigSocket,buf,MAX_MESSAGE_LENGTH,NULL,(struct sockaddr *) &accept_sin, 
                 (int *) &accept_sin_len);

   // Check if there is any data received. If there is, display it.

   if (iReturn == SOCKET_ERROR)
   {
      TCHAR szError[100];                 // Error message string
      wsprintf (szError, TEXT("recvfrom failed. Error: %d"), 
              WSAGetLastError ());
      MessageBox (NULL, szError, TEXT("Error"), MB_OK);
   }
   else if (iReturn == 0)
   {
      MessageBox (NULL, TEXT("Connection closed gracefully"), TEXT("Server"),MB_OK);
   }
   else
   {
      DSServiceDetachCnf.createObjFromStream(buf);
      // Parse received message
      if ((SERVICEDETACHCONFIRM == DSServiceDetachCnf.dsmccMessageHeader.messageId) &&
          (DSServiceDetachReq.dsmccMessageHeader.transactionId == DSServiceDetachCnf.dsmccMessageHeader.transactionId))
      {
         *response = DSServiceDetachCnf.response;
         return TRUE;
      }
      else
         return FALSE;
   }
   return FALSE;
} // ServiceDetach


//------------------------------------------------------------------------------------------------
// TransmuxSetup
BOOL DN_UDP_CLASS::DN_TransMuxSetup(long double networkSessionId, GenericMsgLoop * TMReqLoop, GenericMsgLoop * TMConfLoop)
{
   SOCKET TransmuxSocket = INVALID_SOCKET;

   char buf[MAX_MESSAGE_LENGTH];
   int iReturn,i;
   int bufferLength;
   int accept_sin_len;
   // determine the calling address
   DWORD callingAddress;
   __int16 callingPort;
   accept_sin_len = sizeof (accept_sin);
   struct hostent *h;
   int x = 0;

   if ((h=gethostbyname("localhost")) == NULL) {  /* get the host info */
      x = WSAGetLastError ();
      exit(1);
   }

   if ((h=gethostbyname(h->h_name)) == NULL) {  /* get the host info */
      x = WSAGetLastError ();
      exit(1);
   }

   callingAddress = inet_addr( inet_ntoa(*((struct in_addr *)h->h_addr)) );
   

   DS_TransmuxSetupRequestMessage_type DSTransmuxSetupReq;
   DS_TransmuxSetupConfirmMessage_type DSTransmuxSetupCnf;

   // make up message data
   DSTransmuxSetupReq.networkSessionId = networkSessionId;
   DSTransmuxSetupReq.dsmccMessageHeader.protocolDiscriminator = 0x11;
   DSTransmuxSetupReq.dsmccMessageHeader.dsmccType = 0x06;
   DSTransmuxSetupReq.dsmccMessageHeader.reserved = (__int8)0xFF;
   DSTransmuxSetupReq.dsmccMessageHeader.adaptationLength = 0x00;
   DSTransmuxSetupReq.dsmccMessageHeader.messageId = TRANSMUXSETUPREQUEST;

   DSTransmuxSetupReq.LoopCount = TMReqLoop->LoopCount;
   DSTransmuxSetupReq.dsmccMessageHeader.transactionId = ++transactionId;

   /* YK */
   /* LOOP? ; ONLY ONE DS MESSAGE SHOULD BE SENT : What's the LOOP for? */
   //  IF THERE IS ONLY ONE TAT IN EACH NETWORK SESSION ID; DSTransmux..LOOP WILL HAVE NO MEANING 
   // SOCKETS THAT ARE CREATED HERE ARE FORGOTTEN LATER (ONLY THE FIRST ONE WHICH IS ASSOCIATED WITH A TAT IS REMEMBERED.
   
   DSTransmuxSetupReq.p_TransmuxReqLoop = TMReqLoop;
   
   // THE FOLLOWING LOOP IS RUN ONLY ONCE.
   for(i=0;i < 1 /**//*TMReqLoop->LoopCount */; i++)
   {      
      // the originating peer creates a UDP socket and binds it to some port number
      // and generates a Transmux Association Tag
      /* YK */
      // Create a socket and associate it with networkSessionId;
            
      if ((TransmuxSocket = socket (AF_INET, SOCK_DGRAM, 0)) == INVALID_SOCKET)
      {
         MessageBox (NULL, TEXT("Allocating socket failed."), TEXT("Error"), MB_OK);
         return FALSE;
      }   
      
      if ((callingPort = bindSockettoPort(TransmuxSocket)) == 0 )
      {
         MessageBox (NULL, TEXT("Allocating socket failed."), TEXT("Error"), MB_OK);
         return FALSE;
      }
      
      DSTransmuxSetupReq.p_TransmuxReqLoop->operator %(i)->TAT = callingPort;
      
      // the originating peer creates an IP resource descriptor, setting the sourceIpAddress, sourceIpPort and ipProtocol fields.
      DSTransmuxSetupReq.p_TransmuxReqLoop->operator %(i)->resourceDescriptor.ResourceDescriptorCount = 1; // For IP resource descriptor ONLY.
      DSTransmuxSetupReq.p_TransmuxReqLoop->operator %(i)->resourceDescriptor.IPresourceDescriptor.commonDescriptorHeader.resourceDescriptorType=IPDESCRIPTOR_TYPE;
      DSTransmuxSetupReq.p_TransmuxReqLoop->operator %(i)->resourceDescriptor.IPresourceDescriptor.commonDescriptorHeader.resourceLength=14;
      DSTransmuxSetupReq.p_TransmuxReqLoop->operator %(i)->resourceDescriptor.IPresourceDescriptor.commonDescriptorHeader.resourceDataFieldCount=1;
      
      DSTransmuxSetupReq.p_TransmuxReqLoop->operator %(i)->resourceDescriptor.IPresourceDescriptor.resourceDescriptorDataFields.ipProtocol = UDPPROTOCOL_TYPE;
      DSTransmuxSetupReq.p_TransmuxReqLoop->operator %(i)->resourceDescriptor.IPresourceDescriptor.resourceDescriptorDataFields.sourceIpAddress = callingAddress;
      DSTransmuxSetupReq.p_TransmuxReqLoop->operator %(i)->resourceDescriptor.IPresourceDescriptor.resourceDescriptorDataFields.sourceIpPort= htons(callingPort);
      
      
   }
   // the originating peer  sends a DS_TransMuxSetupRequest message containing the above TAT and resource.
   // Revise the following lines:
   
   DSTransmuxSetupReq.getStream(buf,&bufferLength);
   
   SOCKET sigSocket = NetSessionList.findSocketfromNSID(networkSessionId);
   // send service attach message
   if( sendto (sigSocket, buf, bufferLength, NULL, (PSOCKADDR) &NetSessionList.findDest_SinfromNSID(networkSessionId), 
                  sizeof (NetSessionList.findDest_SinfromNSID(networkSessionId))) == SOCKET_ERROR)  
     {
          MessageBox (NULL, TEXT("Sending data failed."), TEXT("Error"), MB_OK);
          exit(1);
     }
         
   //the originating peer associates the socket to the Transmux Association Tag carried in the DS_TransMuxSetup messages.
   //NetSessionList.AddTAT
         
   accept_sin_len = sizeof (accept_sin);   
   iReturn=recvfrom(sigSocket,buf,MAX_MESSAGE_LENGTH,NULL,(struct sockaddr *) &accept_sin, 
                 (int *) &accept_sin_len);
   
   // Check if there is any data received. If there is, display it.
   
   if (iReturn == SOCKET_ERROR)
   {
      TCHAR szError[100];                 // Error message string
      wsprintf (szError, TEXT("recvfrom failed. Error: %d"), 
              WSAGetLastError ());
      MessageBox (NULL, szError, TEXT("Error"), MB_OK);
   }
   else if (iReturn == 0)
   {
      MessageBox (NULL, TEXT("Connection closed gracefully"), TEXT("Server"),MB_OK);
   }
   else
   {  
      DSTransmuxSetupCnf.p_TransmuxConfLoop = TMConfLoop; // DS_message internal pointer to a loop is now pointing to the
                                                         // TMCnfloop; therefore all changes in DS_message apply to TMCnfLoop too.
      DSTransmuxSetupCnf.createObjFromStream(buf);
      //the originating peer receives the DS_TransMuxSetupConfirm message and retrieves the address and port number of the peer.
      // Revise the following lines:
   
      // Parse received message
      if ((TRANSMUXSETUPCONFIRM == DSTransmuxSetupCnf.dsmccMessageHeader.messageId) &&
          (DSTransmuxSetupReq.dsmccMessageHeader.transactionId == DSTransmuxSetupCnf.dsmccMessageHeader.transactionId))
      {
         // THE FOLLOWING LOOP IS RUN ONLY ONCE.      
         for(i=0;i< 1  /* YK */ /*DSTransmuxSetupCnf.count*/ ;i++){
            DWORD DestIPAddress = DSTransmuxSetupCnf.p_TransmuxConfLoop->operator %(i)->resourceDescriptor.IPresourceDescriptor.resourceDescriptorDataFields.destinationIpAddress;
            __int16 DestPort = ntohs(DSTransmuxSetupCnf.p_TransmuxConfLoop->operator %(i)->resourceDescriptor.IPresourceDescriptor.resourceDescriptorDataFields.destinationIpPort);
            NetSessionList.addTATtoNetSessionRecord(networkSessionId,TransmuxSocket,callingPort /*TAT*/,
                                                  TMReqLoop->operator %(i)->direction ,DestIPAddress, DestPort);
         } /* YK */ // This addTATtoNetSes... function overwrites the old TAT value if there are more than one TAT in each net session
         // if the response for creating a particular TAT was NEGATIVE, then the socket has to be closed
            /* KNOWN BUG */ /* DEV */
         if(DSTransmuxSetupCnf.p_TransmuxConfLoop->operator %(i)->response == RESPONSE_NOTSOGOOD)
         {
            closesocket(TransmuxSocket);
            // Delete socket from List
            NetSessionList.deleteTATfromNetSessionRecord(networkSessionId,callingPort);
         }
         else
         {

            // this layer just passes the information to the higher layer and the upper layer starts the thread
            // Start a thread to listen to User data stream sent by the server.
            /* KNOWN BUG */ /* YK */ /* PLEASE ... */
            //_beginthread(userPlaneThreadProc,NULL,&(TMConfLoop->operator %(i)->TAT));
         }

         return TRUE;
      }
      else
         return FALSE;
   }// else
   return FALSE;
} // End of DN_UDP_CLASS::DN_TransmuxSetup

//----------------------------------------------------------------------------------------------------------------------
BOOL DN_UDP_CLASS::DN_TransMuxRelease(long double networkSessionId,GenericMsgLoop *TATLoop, GenericMsgLoop * RespLoop)
{
   SOCKET TransmuxSocket = INVALID_SOCKET;

   char buf[MAX_MESSAGE_LENGTH];
   int iReturn,i;
   int bufferLength;
   int accept_sin_len;

   DS_TransmuxReleaseRequestMessage_type DSTransmuxReleaseReq;
   DS_TransmuxReleaseConfirmMessage_type DSTransmuxReleaseCnf;


   // the originating peer sends a DS_TransMuxReleaseRequest message containing the TAT associated to the socket being closed.

   // make up message data
   DSTransmuxReleaseReq.networkSessionId = networkSessionId;
   DSTransmuxReleaseReq.dsmccMessageHeader.protocolDiscriminator = 0x11;
   DSTransmuxReleaseReq.dsmccMessageHeader.dsmccType = 0x06;
   DSTransmuxReleaseReq.dsmccMessageHeader.reserved = (__int8)0xFF;
   DSTransmuxReleaseReq.dsmccMessageHeader.adaptationLength = 0x00;
   
   DSTransmuxReleaseReq.dsmccMessageHeader.messageId = TRANSMUXRELEASEREQUEST;
   DSTransmuxReleaseReq.dsmccMessageHeader.transactionId = ++transactionId;

   DSTransmuxReleaseReq.LoopCount = TATLoop->LoopCount;
   DSTransmuxReleaseReq.p_TransmuxReqLoop = TATLoop;
   
   DSTransmuxReleaseReq.getStream(buf,&bufferLength);
   
  // MessageBox (NULL, TEXT("TransmuxRelease"), TEXT("Server"),MB_OK);

   SOCKET sigSocket = NetSessionList.findSocketfromNSID(networkSessionId);
   // send service attach message
   if( sendto (sigSocket, buf, bufferLength, NULL, (PSOCKADDR) &NetSessionList.findDest_SinfromNSID(networkSessionId), 
                  sizeof (NetSessionList.findDest_SinfromNSID(networkSessionId))) == SOCKET_ERROR)  
     {
          MessageBox (NULL, TEXT("Sending data failed."), TEXT("Error"), MB_OK);
          exit(1);
     }

   // the target peer closes the UDP socket corresponding to the above TAT.
   // the target peer sends a DS_TransMuxReleaseConfirm message.
   accept_sin_len = sizeof (accept_sin);   
   iReturn=recvfrom(sigSocket,buf,MAX_MESSAGE_LENGTH,NULL,(struct sockaddr *) &accept_sin, 
                 (int *) &accept_sin_len);
   
   // Check if there is any data received. If there is, display it.
   
   if (iReturn == SOCKET_ERROR)
   {
      TCHAR szError[100];                 // Error message string
      wsprintf (szError, TEXT("recvfrom failed. Error: %d"), 
              WSAGetLastError ());
      MessageBox (NULL, szError, TEXT("Error"), MB_OK);
   }
   else if (iReturn == 0)
   {
      MessageBox (NULL, TEXT("Connection closed gracefully"), TEXT("Server"),MB_OK);
   }
   else
   {  
      DSTransmuxReleaseCnf.p_TransmuxConfLoop = RespLoop; // DS_message internal pointer to a loop is now pointing to the
                                                         // Resploop; therefore all changes in DS_message apply to RespLoop too.
      DSTransmuxReleaseCnf.createObjFromStream(buf);
      //the originating peer receives the DS_TransmuxReleaseConfirm message and retrieves the address and port number of the peer.
      // Revise the following lines:
   
      // Parse received message
      if ((TRANSMUXRELEASECONFIRM == DSTransmuxReleaseCnf.dsmccMessageHeader.messageId) &&
          (DSTransmuxReleaseReq.dsmccMessageHeader.transactionId == DSTransmuxReleaseCnf.dsmccMessageHeader.transactionId))
      {
         // the originating peer closes the UDP socket corresponding to the above TAT.
         // THE FOLLOWING LOOP IS RUN ONLY ONCE = There is only one socket in UDP case.      
         for(i=0;i < DSTransmuxReleaseCnf.LoopCount ;i++){
            if (DSTransmuxReleaseCnf.p_TransmuxConfLoop->operator %(i)->response == RESPONSE_OK) {
               SOCKET UPSocket = NetSessionList.deleteTATfromNetSessionRecord(networkSessionId,TATLoop->operator %(i)->TAT); 
               //LISTENTHREAD = FALSE; // kills listening thread
               
               closesocket(UPSocket); // need to kill all threads if more than one exists
            }
         }
         return TRUE;
      }
      else
         return FALSE;
   }// else
   return FALSE;
} // End of DN_UDP_CLASS::DN_TransmuxRelease

//----------------------------------------------------------------------------------------------------------------------
BOOL DN_UDP_CLASS::DN_TransMuxConfig(long double networkSessionId, GenericMsgLoop * TMReqLoop, GenericMsgLoop * TMCnfLoop)
{
   SOCKET TransmuxSocket = INVALID_SOCKET;

   char buf[MAX_MESSAGE_LENGTH];
   int iReturn,i;
   int bufferLength;
   int accept_sin_len;
   accept_sin_len = sizeof (accept_sin);

   DS_TransmuxConfigRequestMessage_type DSTransmuxConfigReq;
   DS_TransmuxConfigConfirmMessage_type DSTransmuxConfigCnf;

   // make up message data
   DSTransmuxConfigReq.networkSessionId = networkSessionId;
   DSTransmuxConfigReq.dsmccMessageHeader.protocolDiscriminator = 0x11;
   DSTransmuxConfigReq.dsmccMessageHeader.dsmccType = 0x06;
   DSTransmuxConfigReq.dsmccMessageHeader.reserved = (__int8)0xFF;
   DSTransmuxConfigReq.dsmccMessageHeader.adaptationLength = 0x00;
   
   DSTransmuxConfigReq.dsmccMessageHeader.messageId = TRANSMUXCONFIGREQUEST;
   DSTransmuxConfigReq.dsmccMessageHeader.transactionId = ++transactionId;

   DSTransmuxConfigReq.LoopCount = TMReqLoop->LoopCount;
   DSTransmuxConfigReq.p_TransmuxReqLoop = TMReqLoop;
   
   DSTransmuxConfigReq.getStream(buf,&bufferLength);
   
   SOCKET sigSocket = NetSessionList.findSocketfromNSID(networkSessionId);
   // send service attach message
   if( sendto (sigSocket, buf, bufferLength, NULL, (PSOCKADDR) &NetSessionList.findDest_SinfromNSID(networkSessionId), 
                  sizeof (NetSessionList.findDest_SinfromNSID(networkSessionId))) == SOCKET_ERROR)  
     {
          MessageBox (NULL, TEXT("Sending data failed."), TEXT("Error"), MB_OK);
          exit(1);
     }

   iReturn=recvfrom(sigSocket,buf,MAX_MESSAGE_LENGTH,NULL,(struct sockaddr *) &accept_sin, 
                 (int *) &accept_sin_len);
   
   // Check if there is any data received. If there is, display it.
   
   if (iReturn == SOCKET_ERROR)
   {
      TCHAR szError[100];                 // Error message string
      wsprintf (szError, TEXT("recvfrom failed. Error: %d"), 
              WSAGetLastError ());
      MessageBox (NULL, szError, TEXT("Error"), MB_OK);
   }
   else if (iReturn == 0)
   {
      MessageBox (NULL, TEXT("Connection closed gracefully"), TEXT("Server"),MB_OK);
   }
   else
   {  
         
      DSTransmuxConfigCnf.p_TransmuxConfLoop = TMCnfLoop; // DS_message internal pointer to a loop is now pointing to the
                                                         // Resploop; therefore all changes in DS_message apply to RespLoop too.
      DSTransmuxConfigCnf.createObjFromStream(buf);

      // What is this function supposed to do???!!! /* YK */
      
      // Parse received message
      if ((TRANSMUXCONFIGCONFIRM == DSTransmuxConfigCnf.dsmccMessageHeader.messageId) &&
          (DSTransmuxConfigReq.dsmccMessageHeader.transactionId == DSTransmuxConfigCnf.dsmccMessageHeader.transactionId))
      {

         // THE FOLLOWING LOOP IS RUN ONLY ONCE = There is only one socket in UDP case.      
         for(i=0;i< 1  /* YK */ /*DSTransmuxConfigCnf.LoopCount*/ ;i++){

            //What to do with RESPONSEs.

         } /* YK */ // This addTATtoNetSes... function overwrites the old TAT value if there are h net session
            /* KNOWN BUG */ /* DEV */
         return TRUE;
      }
      else
         return FALSE;
   }// else
   return FALSE;
} // End of DN_UDP_CLASS::DN_TransmuxConfig

//------------------------------------------------------------------------------------------------
// ChannelAdd

BOOL DN_UDP_CLASS::DN_ChannelAdd(long double networkSessionId,__int16 serviceId,GenericMsgLoop * ReqLoop ,/*loop(CAT, channelDescriptor, direction, ddDataIn());*/
                 GenericMsgLoop * CnfLoop /*loop(response, TAT, ddDataOut())*/)
{

   char buf[MAX_MESSAGE_LENGTH];
   int iReturn;
   int bufferLength;
   int accept_sin_len = sizeof (accept_sin);
   

   DS_ChannelAddRequestMessage_type DSChannelAddReq;
   DS_ChannelAddConfirmMessage_type DSChannelAddCnf;

   // make up message data
   DSChannelAddReq.dsmccMessageHeader.protocolDiscriminator = 0x11;
   DSChannelAddReq.dsmccMessageHeader.dsmccType = 0x06;
   DSChannelAddReq.dsmccMessageHeader.reserved = (__int8)0xFF;
   DSChannelAddReq.dsmccMessageHeader.adaptationLength = 0x00;
   DSChannelAddReq.dsmccMessageHeader.messageId = CHANNELADDREQUEST;

   DSChannelAddReq.networkSessionId= networkSessionId;
   DSChannelAddReq.serviceId = serviceId;
   DSChannelAddReq.LoopCount = ReqLoop->LoopCount;

   DSChannelAddReq.p_CHReqLoop = ReqLoop; /* YK */ // IMPORTANT  >:o(

   DSChannelAddReq.dsmccMessageHeader.transactionId = ++transactionId;

   DSChannelAddReq.getStream(buf,&bufferLength);      
   
   SOCKET sigSocket = NetSessionList.findSocketfromNSID(networkSessionId);
   // send service attach message
   if( sendto (sigSocket, buf, bufferLength, NULL, (PSOCKADDR) &NetSessionList.findDest_SinfromNSID(networkSessionId), 
                  sizeof (NetSessionList.findDest_SinfromNSID(networkSessionId))) == SOCKET_ERROR)  
     {
          MessageBox (NULL, TEXT("Sending data failed."), TEXT("Error"), MB_OK);
          exit(1);
     }
         
   iReturn=recvfrom(sigSocket,buf,MAX_MESSAGE_LENGTH,NULL,(struct sockaddr *) &accept_sin, 
                 (int *) &accept_sin_len);
   
   // Check if there is any data received. If there is, display it.
   
   if (iReturn == SOCKET_ERROR)
   {
      TCHAR szError[100];                 // Error message string
      wsprintf (szError, TEXT("recvfrom failed. Error: %d"), 
              WSAGetLastError ());
      MessageBox (NULL, szError, TEXT("Error"), MB_OK);
   }
   else if (iReturn == 0)
   {
      MessageBox (NULL, TEXT("Connection closed gracefully"), TEXT("Server"),MB_OK);
   }
   else
   {  
      DSChannelAddCnf.p_CHCnfLoop = CnfLoop; // DS_message internal pointer to a loop is now pointing to the
                                             // Cnfloop; therefore all changes in DS_message apply to CnfLoop too.
      DSChannelAddCnf.createObjFromStream(buf);
      
      // What happens to dsmccMessageHeader : Probably it is discarded !!! (not passed to the upper layer)
      // Parse received message /* YK */
      if ((CHANNELADDCONFIRM == DSChannelAddCnf.dsmccMessageHeader.messageId) &&
          (DSChannelAddReq.dsmccMessageHeader.transactionId == DSChannelAddCnf.dsmccMessageHeader.transactionId))
      {
         return TRUE;
      }
      else
         return FALSE;
   }
   return FALSE;
} // End of Channel_ADD


//----------------------------------------------------------------------------------------------------------------------
BOOL DN_UDP_CLASS::DN_ChannelAdded(long double networkSessionId,__int16 serviceId, GenericMsgLoop * ReqLoop /* loop(CAT, channelDescriptor, direction, TAT, ddDataIn())*/,
                    GenericMsgLoop * CnfLoop /* OUT: loop(response, ddDataOut())*/ )
{
   char buf[MAX_MESSAGE_LENGTH];
   int iReturn;
   int bufferLength;
   int accept_sin_len = sizeof (accept_sin);
   

   DS_ChannelAddedRequestMessage_type DSChannelAddedReq;
   DS_ChannelAddedConfirmMessage_type DSChannelAddedCnf;

   // make up message data
   DSChannelAddedReq.dsmccMessageHeader.protocolDiscriminator = 0x11;
   DSChannelAddedReq.dsmccMessageHeader.dsmccType = 0x06;
   DSChannelAddedReq.dsmccMessageHeader.reserved = (__int8)0xFF;
   DSChannelAddedReq.dsmccMessageHeader.adaptationLength = 0x00;
   DSChannelAddedReq.dsmccMessageHeader.messageId = CHANNELADDEDREQUEST;
   DSChannelAddedReq.dsmccMessageHeader.transactionId = ++transactionId;

   DSChannelAddedReq.networkSessionId= networkSessionId;
   DSChannelAddedReq.serviceId = serviceId;
   
   DSChannelAddedReq.LoopCount = ReqLoop->LoopCount;
   DSChannelAddedReq.p_CHReqLoop = ReqLoop; 


   DSChannelAddedReq.getStream(buf,&bufferLength);      
   
   SOCKET sigSocket = NetSessionList.findSocketfromNSID(networkSessionId);
   // send service attach message
   if( sendto (sigSocket, buf, bufferLength, NULL, (PSOCKADDR) &NetSessionList.findDest_SinfromNSID(networkSessionId), 
                  sizeof (NetSessionList.findDest_SinfromNSID(networkSessionId))) == SOCKET_ERROR)  
     {
          MessageBox (NULL, TEXT("Sending data failed."), TEXT("Error"), MB_OK);
          exit(1);
     }
         
   iReturn=recvfrom(sigSocket,buf,MAX_MESSAGE_LENGTH,NULL,(struct sockaddr *) &accept_sin, 
                 (int *) &accept_sin_len);
   
   // Check if there is any data received. If there is, display it.
   
   if (iReturn == SOCKET_ERROR)
   {
      TCHAR szError[100];                 // Error message string
      wsprintf (szError, TEXT("recvfrom failed. Error: %d"), 
              WSAGetLastError ());
      MessageBox (NULL, szError, TEXT("Error"), MB_OK);
   }
   else if (iReturn == 0)
   {
      MessageBox (NULL, TEXT("Connection closed gracefully"), TEXT("Server"),MB_OK);
   }
   else
   {  
      DSChannelAddedCnf.p_CHCnfLoop = CnfLoop; // DS_message internal pointer to a loop is now pointing to the
                                             // Cnfloop; therefore all changes in DS_message apply to CnfLoop too.
      DSChannelAddedCnf.createObjFromStream(buf);
      
      // What happens to dsmccMessageHeader : Probably it is discarded !!! (not passed to the upper layer)
      // Parse received message
      if ((CHANNELADDEDCONFIRM == DSChannelAddedCnf.dsmccMessageHeader.messageId) &&
          (DSChannelAddedReq.dsmccMessageHeader.transactionId == DSChannelAddedCnf.dsmccMessageHeader.transactionId))
      {
         return TRUE;
      }
      else
         return FALSE;
   }
   return FALSE;
} // End of Channel_Added


//----------------------------------------------------------------------------------------------------------------------
BOOL DN_UDP_CLASS::DN_ChannelDelete(long double networkSessionId,GenericMsgLoop * CATReasonLoop, GenericMsgLoop * Resploop)
{
   char buf[MAX_MESSAGE_LENGTH];
   int iReturn;
   int bufferLength;
   int accept_sin_len = sizeof (accept_sin);
   

   DS_ChannelDeleteRequestMessage_type DSChannelDeleteReq;
   DS_ChannelDeleteConfirmMessage_type DSChannelDeleteCnf;

   // make up message data
   DSChannelDeleteReq.dsmccMessageHeader.protocolDiscriminator = 0x11;
   DSChannelDeleteReq.dsmccMessageHeader.dsmccType = 0x06;
   DSChannelDeleteReq.dsmccMessageHeader.reserved = (__int8)0xFF;
   DSChannelDeleteReq.dsmccMessageHeader.adaptationLength = 0x00;
   
   DSChannelDeleteReq.dsmccMessageHeader.messageId = CHANNELDELETEREQUEST;
   DSChannelDeleteReq.dsmccMessageHeader.transactionId = ++transactionId;

   DSChannelDeleteReq.networkSessionId= networkSessionId;

   DSChannelDeleteReq.LoopCount = CATReasonLoop->LoopCount;
   DSChannelDeleteReq.p_CHReqLoop = CATReasonLoop; 


   DSChannelDeleteReq.getStream(buf,&bufferLength);      
   
   SOCKET sigSocket = NetSessionList.findSocketfromNSID(networkSessionId);
   // send service attach message
   if( sendto (sigSocket, buf, bufferLength, NULL, (PSOCKADDR) &NetSessionList.findDest_SinfromNSID(networkSessionId), 
                  sizeof (NetSessionList.findDest_SinfromNSID(networkSessionId))) == SOCKET_ERROR)  
     {
          MessageBox (NULL, TEXT("Sending data failed."), TEXT("Error"), MB_OK);
          exit(1);
     }
         
   iReturn=recvfrom(sigSocket,buf,MAX_MESSAGE_LENGTH,NULL,(struct sockaddr *) &accept_sin, 
                 (int *) &accept_sin_len);
   
   // Check if there is any data received. If there is, display it.
   
   if (iReturn == SOCKET_ERROR)
   {
      TCHAR szError[100];                 // Error message string
      wsprintf (szError, TEXT("recvfrom failed. Error: %d"), 
              WSAGetLastError ());
      MessageBox (NULL, szError, TEXT("Error"), MB_OK);
   }
   else if (iReturn == 0)
   {
      MessageBox (NULL, TEXT("Connection closed gracefully"), TEXT("Server"),MB_OK);
   }
   else
   {  
      DSChannelDeleteCnf.p_CHCnfLoop = Resploop; // DS_message internal pointer to a loop is now pointing to the
                                             // Resploop; therefore all changes in DS_message apply to Resploop too.
      DSChannelDeleteCnf.createObjFromStream(buf);
      
      // What happens to dsmccMessageHeader : Probably it is discarded !!! (not passed to the upper layer)
      // Parse received message
      if ((CHANNELDELETECONFIRM == DSChannelDeleteCnf.dsmccMessageHeader.messageId) &&
          (DSChannelDeleteReq.dsmccMessageHeader.transactionId == DSChannelDeleteCnf.dsmccMessageHeader.transactionId))
      {
         return TRUE;
      }
      else
         return FALSE;
   }
   return FALSE;
} // End of Channel_Delete


//----------------------------------------------------------------------------------------------------------------------
BOOL DN_UDP_CLASS::DN_UserCommand(long double networkSessionId, ddDataClass *ddDataIn, GenericMsgLoop * CATLoop)
{
   char buf[MAX_MESSAGE_LENGTH];
   int bufferLength;
   DS_UserCommandMessage_type DSUserCommand;

   // make up message data
   DSUserCommand.networkSessionId = networkSessionId;
   DSUserCommand.dsmccMessageHeader.protocolDiscriminator = 0x11;
   DSUserCommand.dsmccMessageHeader.dsmccType = 0x06;
   DSUserCommand.dsmccMessageHeader.reserved = (__int8)0xFF;
   DSUserCommand.dsmccMessageHeader.adaptationLength = 0x00;
   DSUserCommand.dsmccMessageHeader.messageId = USERCOMMAND;
   DSUserCommand.dsmccMessageHeader.transactionId = ++transactionId;

   // Revise the following lines:
   DSUserCommand.ddData = ddDataIn;
   DSUserCommand.p_CATLoop = CATLoop;
   DSUserCommand.LoopCount = CATLoop->LoopCount;

   DSUserCommand.getStream(buf,&bufferLength);

   SOCKET sigSocket = NetSessionList.findSocketfromNSID(networkSessionId);
   // send usercommand message
   if( sendto (sigSocket, buf, bufferLength, NULL, (PSOCKADDR) &NetSessionList.findDest_SinfromNSID(networkSessionId), 
                  sizeof (NetSessionList.findDest_SinfromNSID(networkSessionId))) == SOCKET_ERROR)  
     {
          MessageBox (NULL, TEXT("Sending data failed."), TEXT("Error"), MB_OK);
          exit(1);
     }
   
  return TRUE;
} // End of DN_UDP_CLASS::DN_UserCommand


BOOL DN_UDP_CLASS::DN_UserCommandAck(long double networkSessionId, ddDataClass *ddDataIn, GenericMsgLoop * CATLoop,
                       __int16 *response, ddDataClass * ddDataOut)
{
   char buf[MAX_MESSAGE_LENGTH];
   int iReturn;
   int bufferLength;
   DS_UserCommandMessage_type DSUserCommand;
   DS_UserCommandAckConfirmMessage_type DSUserCommandAckCnf;
   int accept_sin_len = sizeof (accept_sin);
   
   // make up message data
   DSUserCommand.networkSessionId = networkSessionId;
   DSUserCommand.dsmccMessageHeader.protocolDiscriminator = 0x11;
   DSUserCommand.dsmccMessageHeader.dsmccType = 0x06;
   DSUserCommand.dsmccMessageHeader.reserved = (__int8)0xFF;
   DSUserCommand.dsmccMessageHeader.adaptationLength = 0x00;
   DSUserCommand.dsmccMessageHeader.messageId = USERCOMMANDACKREQUEST;
   DSUserCommand.dsmccMessageHeader.transactionId = ++transactionId;

   // Revise the following lines:
   DSUserCommand.ddData = ddDataIn;
   DSUserCommand.p_CATLoop = CATLoop;
   DSUserCommand.LoopCount = CATLoop->LoopCount;

   DSUserCommand.getStream(buf,&bufferLength);

   SOCKET sigSocket = NetSessionList.findSocketfromNSID(networkSessionId);
   // send usercommand message
   if( sendto (sigSocket, buf, bufferLength, NULL, (PSOCKADDR) &NetSessionList.findDest_SinfromNSID(networkSessionId), 
                  sizeof (NetSessionList.findDest_SinfromNSID(networkSessionId))) == SOCKET_ERROR)  
     {
          MessageBox (NULL, TEXT("Sending data failed."), TEXT("Error"), MB_OK);
          exit(1);
     }
   
   iReturn=recvfrom(sigSocket,buf,MAX_MESSAGE_LENGTH,NULL,(struct sockaddr *) &accept_sin, 
                 (int *) &accept_sin_len);
   
   // Check if there is any data received. If there is, display it.
   
   if (iReturn == SOCKET_ERROR)
   {
      TCHAR szError[100];                 // Error message string
      wsprintf (szError, TEXT("recvfrom failed. Error: %d"), 
              WSAGetLastError ());
      MessageBox (NULL, szError, TEXT("Error"), MB_OK);
   }
   else if (iReturn == 0)
   {
      MessageBox (NULL, TEXT("Connection closed gracefully"), TEXT("Server"),MB_OK);
   }
   else
   {
      DSUserCommandAckCnf.ddData = ddDataOut;
      DSUserCommandAckCnf.createObjFromStream(buf);

      // Parse received message
      if ((USERCOMMANDACKCONFIRM == DSUserCommandAckCnf.dsmccMessageHeader.messageId) &&
          (DSUserCommand.dsmccMessageHeader.transactionId == DSUserCommandAckCnf.dsmccMessageHeader.transactionId))
      {
         *response = DSUserCommandAckCnf.response;
         return TRUE;
      }
      else
         return FALSE;
   }// else
   return FALSE;
} // End of DN_UDP_CLASS::DN_UserCommandAck

/*
#if 0
// This Function should redirect each listenning thread to a separate object,so that different threads 
// don't run in the same function simultaneously

void userPlaneThreadProc(void *valueofPtr)
{
   
   Upmesg  * upmessage;
   upmessage = (struct Upmesg *)(valueofPtr);

   ListenerClass * pListenerObj = createListenerObj();
   pListenerObj->begin(upmessage->pService, upmessage->TAT);
   delete valueofPtr;
}


void ListenerClass::begin(RemoteInstance * pService, __int16 TAT)
{
   char buf[30000];
   int iReturn;
   __int16 destPort;
   DWORD destIP;
   SOCKET UPSocket;
   SOCKADDR_IN accept_sin;
   int accept_sin_len = sizeof(accept_sin);
   // MessageBox (NULL, TEXT("You Are Here Listener Class"), TEXT("Info"), MB_OK);

   NetSessionList.getUPInfo(TAT,&destIP,&destPort,&UPSocket);   

   accept_sin.sin_family = AF_INET;
   accept_sin.sin_addr.s_addr = htonl(INADDR_ANY);
   accept_sin.sin_port = htons(TAT);

   while(LISTENTHREAD)
   {
      iReturn = recvfrom(UPSocket,buf,30000,NULL,(struct sockaddr *) &accept_sin, 
                 (int*)&accept_sin_len);
      if (iReturn == SOCKET_ERROR)
      {
         if (WSAGetLastError() != 10038) { // added for graceful closing of socket
            TCHAR szError[100];                 // Error message string
            wsprintf (szError, TEXT("recvfrom failed. Error: %d"), 
                    WSAGetLastError ());
            MessageBox (NULL, szError, TEXT("Error"), MB_OK);
         }
      }
      else if (iReturn == 0)
      {
         MessageBox (NULL, TEXT("Connection closed gracefully"), TEXT("Server"),MB_OK);
      }
      else
      {  
         YKTRACE("***********iReturn: %d********** Channel Id: %d***************\n",iReturn,(__int16)*buf);
         pService->DNI_DataReceive_Ind(buf);
      }
         
   }

}

ListenerClass * createListenerObj(){
      return new ListenerClass;
   }

#endif   */