/******************************************************************************
*  SOFTWARE LICENSE AND CONDITIONS
*
*  This software module was originally developed by Kambiz Asrar Haghighi 
*  and Yasser Pourmohammadi (University of British Columbia) in the course of 
*  development of  the streaming functionality for MPEG-4 using the DMIF
*  (ISO/IEC 14496-6) standard. This software module is an implementation 
*  of a part of one or more MPEG-4 tools as specified by MPEG-4. 
*  
*  Users of the MPEG-4 may use the software source code or modifications 
*  thereof only for hardware or software products related to MPEG-4 evaluation
*  and testing.  The use of the source code for any purpose other than 
*  evaluation and testing, including the use of derivative works for the 
*  generation of commercial products, is strictly prohibited.
*  
*  The original developer of this software module and his/her company, 
*  the subsequent editors and their companies, and ISO/IEC have no 
*  liability for use of this software module or modifications thereof 
*  in an implementation. Copyright is not released for non MPEG-4 
*  conforming products. UBC retains full right to use the code for 
*  his/her own purpose, assign or donate the code to a third party and 
*  to inhibit third parties from using the code for non MPEG-4 conforming 
*  products. This copyright notice must be included in all copies or 
*  derivative works.
*
*  Contact:
*
*  Kambiz Asrar Haghighi                 email: kambiz@interchange.ubc.ca
*  Yasser Pourmohammadi                  email: yasserp@ece.ubc.ca
*  Professor Hussein M. Alnuweiri
*  UBC Signal Processing and Multimedia Group
*  http://spmg.ece.ubc.ca/research/MPEG4
*  Vancouver BC Canada V6T1Z4                  
*
*  Copyright  2000  University of British Columbia, Canada
********************************************************************************/
#ifndef _DATADESC_H
#define _DATADESC_H

#include <windows.h>

// ddData Types
struct commonDescriptorHeaderType{
   __int16 DmifDescriptorType; // 0x1 for uuData
   __int16 DmifDescriptorLen; // length of the following data field.
};


class dmifDescriptorClass{
public:
   ~dmifDescriptorClass(){ 
      free(descriptorDataFields);}
   commonDescriptorHeaderType commonDescriptorHeader;
   char* descriptorDataFields; // uuData
   BOOL copyToDataFields(int len , char * source)
   {
      if( (source != NULL) && (len>=0)) {
         descriptorDataFields = (char*)malloc(len);
         memcpy(descriptorDataFields,source,len);
         commonDescriptorHeader.DmifDescriptorLen = len;
         return TRUE;
      }
      return FALSE;
   }
      
};

class DArray {

public:
   dmifDescriptorClass * dsptr;
   DArray(int count){dsptr = new dmifDescriptorClass[count];}
   ~DArray(){
      delete [] dsptr;
   }

   dmifDescriptorClass * operator % (int j) {
      return (dmifDescriptorClass * )((int)dsptr + j*sizeof(dmifDescriptorClass));
   }
};

class ddDataClass{
public:
   ddDataClass(int count){
      DAPtr = new DArray(count);
      DmifDescriptorCount = count;
   };
   ddDataClass(){
      DAPtr = new DArray(0);
      DmifDescriptorCount = 0;
   }

   ~ddDataClass(){
      delete DAPtr;};  
   // copy constructor
   ddDataClass(ddDataClass const &other)
   {    
      DmifDescriptorCount = other.DmifDescriptorCount;
      DAPtr = new DArray(DmifDescriptorCount);
      for(int i=0;i<DmifDescriptorCount;i++)
      {
         DAPtr->operator %(i)->commonDescriptorHeader.DmifDescriptorType 
            = other.DAPtr->operator %(i)->commonDescriptorHeader.DmifDescriptorType;
         DAPtr->operator %(i)->commonDescriptorHeader.DmifDescriptorLen 
            = other.DAPtr->operator %(i)->commonDescriptorHeader.DmifDescriptorLen ;
         DAPtr->operator %(i)->copyToDataFields(DAPtr->operator %(i)->commonDescriptorHeader.DmifDescriptorLen,
                                                 other.DAPtr->operator %(i)->descriptorDataFields);
      }
   }
    
   void setArraySize(int count)
   {
      delete DAPtr;
      DAPtr = new DArray(count);
      DmifDescriptorCount = count;
   }
   
   /*  This function receives a ddDataClass as input , resizes the "this" class and copies the input to the place that
       its ddDataClass pointer points to :  *ddData = ddDataInTemp*/
   void BeFilledBy(const ddDataClass ddDataInTemp)
   {
            
      DmifDescriptorCount = ddDataInTemp.DmifDescriptorCount;
      setArraySize(DmifDescriptorCount);
      for(int i=0;i<DmifDescriptorCount;i++){
         DAPtr->operator %(i)->commonDescriptorHeader.DmifDescriptorType = ddDataInTemp.DAPtr->operator %(i)->commonDescriptorHeader.DmifDescriptorType;
         DAPtr->operator %(i)->commonDescriptorHeader.DmifDescriptorLen = ddDataInTemp.DAPtr->operator %(i)->commonDescriptorHeader.DmifDescriptorLen;
         DAPtr->operator %(i)->copyToDataFields(DAPtr->operator %(i)->commonDescriptorHeader.DmifDescriptorLen,
                                                   ddDataInTemp.DAPtr->operator %(i)->descriptorDataFields);
      }
   }

   //Return the size of class (real data fields in Class)
   int getSize()
   {
      int count = DmifDescriptorCount;
      int ddDatalength = 2; // DmifDescriptorCount size
   
      for (int i = 0; i < count; i++) {
         ddDatalength += DAPtr->operator %(i)->commonDescriptorHeader.DmifDescriptorLen;
         ddDatalength += 4;
      }
      return ddDatalength;
   }
   __int16 getDescCount() {
      return DmifDescriptorCount;
   }

   DArray * DAPtr;
private:
   __int16 DmifDescriptorCount;
};

/*iiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiiii*/
#define PRIORITY	   0x01	// Priority for the stream
#define MAX_AU_SIZE	0x41	// Maximum size of a PDU, as delivered over the transmux
#define AVG_BITRATE	0x42	// Average bit rate measured over an observation time window
#define MAX_BITRATE	0x43	// Maximum bit rate measured over an observation time window


class channelQualifierClass{
public:
   __int8 Type; // 0x1 for uuData
   __int8 Len; // length of the following data field.
   ~channelQualifierClass(){ 
      free(DataFields);}
   char* DataFields; // uuData
   BOOL copyToDataFields(int size , char * source)
   {
      if( (source != NULL) && (size>=0)) {
         DataFields = (char*)malloc(size);
         memcpy(DataFields,source,size);
         return TRUE;
      }
      return FALSE;
   }
      
};

class ChQualifierArray {

public:
   channelQualifierClass * dsptr;
   ChQualifierArray(int count){dsptr = new channelQualifierClass[count];}
   ~ChQualifierArray(){
      delete [] dsptr;
   }

   channelQualifierClass * operator % (int j) {
      return (channelQualifierClass * )((int)dsptr + j*sizeof(channelQualifierClass));
   }
};

class channelDescriptorClass {
public:
   channelDescriptorClass(int count){
      DAPtr = new ChQualifierArray(count);
      DescriptorCount = count;
   };
   channelDescriptorClass(){
      DAPtr = new ChQualifierArray(0);
      DescriptorCount = 0;
   }

   ~channelDescriptorClass(){
      delete DAPtr;};  
   // copy constructor
   channelDescriptorClass(channelDescriptorClass const &other)
   {    
      DescriptorCount = other.DescriptorCount;
      DAPtr = new ChQualifierArray(DescriptorCount);
      for(int i=0;i<DescriptorCount;i++)
      {
         DAPtr->operator %(i)->Type 
            = other.DAPtr->operator %(i)->Type;
         DAPtr->operator %(i)->Len 
            = other.DAPtr->operator %(i)->Len ;
         DAPtr->operator %(i)->copyToDataFields(DAPtr->operator %(i)->Len,
                                                 other.DAPtr->operator %(i)->DataFields);
      }
   }
    
   void setArraySize(int count)
   {
      delete DAPtr;
      DAPtr = new ChQualifierArray(count);
      DescriptorCount = count;
   }
   
   void BeFilledBy(const channelDescriptorClass DescriptorInTemp)
   {
            
      DescriptorCount = DescriptorInTemp.DescriptorCount;
      setArraySize(DescriptorCount);
      for(int i=0;i<DescriptorCount;i++){
         DAPtr->operator %(i)->Type = DescriptorInTemp.DAPtr->operator %(i)->Type;
         DAPtr->operator %(i)->Len = DescriptorInTemp.DAPtr->operator %(i)->Len;
         DAPtr->operator %(i)->copyToDataFields(DAPtr->operator %(i)->Len,
                                                   DescriptorInTemp.DAPtr->operator %(i)->DataFields);
      }
   }
   int getSize()
   {
      int count = DescriptorCount;
      int channelDescLength = 1; // DmifDescriptorCount size
   
      for (int i = 0; i < count; i++) {
         channelDescLength += DAPtr->operator %(i)->Len;
         channelDescLength += 2;
      }
      return channelDescLength;
   }
   __int8 getDescCount() {
      return DescriptorCount;
   }


   ChQualifierArray * DAPtr;
private:
   __int8 DescriptorCount;
};

/* Resource Descriptor Class */

// resources typedef
typedef struct {
   __int32 sourceIpAddress;
   __int16 sourceIpPort;
   __int32 destinationIpAddress;
   __int16 destinationIpPort;
   __int16 ipProtocol;
} resourceDescriptorDataFields_type;


typedef struct {
   __int16 resourceDescriptorType;
   __int16 resourceLength;
   __int16 resourceDataFieldCount;
} commonDescriptorHeader_type;

typedef struct {
   commonDescriptorHeader_type commonDescriptorHeader;
   resourceDescriptorDataFields_type resourceDescriptorDataFields;
} resourceDescriptor_type;


class  resourceDescriptorClass{
public:
   __int16 ResourceDescriptorCount;
   resourceDescriptor_type IPresourceDescriptor;     // Shall be an IP resource Descriptor
};



#endif // _DATADESC_H

