/******************************************************************************
*  SOFTWARE LICENSE AND CONDITIONS
*
*  This software module was originally developed by Kambiz Asrar Haghighi 
*  and Yasser Pourmohammadi (University of British Columbia) in the course of 
*  development of  the streaming functionality for MPEG-4 using the DMIF
*  (ISO/IEC 14496-6) standard. This software module is an implementation 
*  of a part of one or more MPEG-4 tools as specified by MPEG-4. 
*  
*  Users of the MPEG-4 may use the software source code or modifications 
*  thereof only for hardware or software products related to MPEG-4 evaluation
*  and testing.  The use of the source code for any purpose other than 
*  evaluation and testing, including the use of derivative works for the 
*  generation of commercial products, is strictly prohibited.
*  
*  The original developer of this software module and his/her company, 
*  the subsequent editors and their companies, and ISO/IEC have no 
*  liability for use of this software module or modifications thereof 
*  in an implementation. Copyright is not released for non MPEG-4 
*  conforming products. UBC retains full right to use the code for 
*  his/her own purpose, assign or donate the code to a third party and 
*  to inhibit third parties from using the code for non MPEG-4 conforming 
*  products. This copyright notice must be included in all copies or 
*  derivative works.
*
*  Contact:
*
*  Kambiz Asrar Haghighi                 email: kambiz@interchange.ubc.ca
*  Yasser Pourmohammadi                  email: yasserp@ece.ubc.ca
*  Professor Hussein M. Alnuweiri
*  UBC Signal Processing and Multimedia Group
*  http://spmg.ece.ubc.ca/research/MPEG4
*  Vancouver BC Canada V6T1Z4                  
*
*  Copyright  2000  University of British Columbia, Canada
********************************************************************************/

#ifndef _MESSAGES_H
#define _MESSAGES_H

#include "DN_definitions.h"
#include "DataDesc.h"



///////////////////////////////////////////////////////////////////////////////
// DSMcc message Header type:  
///////////////////////////////////////////////////////////////////////////////


typedef struct{
   __int8   protocolDiscriminator;
   __int8   dsmccType;
   __int16 messageId;
   __int32 transactionId;
   __int8 reserved;
   __int8 adaptationLength;
   __int16 messageLength;
   void appendStream(char *,int * );
   void getHeader(char *);
}  dsmccMessageHeader_type;

typedef struct{
   void appendDDdata(char * locationptr, ddDataClass * source, int * len);
   BOOL getDDdata(ddDataClass * target, char * source, int * len);
} base_message;

///////////////////////////////////////////////////////////////////////////////
// Session Setup Request and Confirm message types and their initialization:  /
///////////////////////////////////////////////////////////////////////////////
typedef struct{
      
   dsmccMessageHeader_type dsmccMessageHeader;
   long double networkSessionId;
   __int8 compatibilityDescriptor;  // this part is permanently ignored 
   void getStream (char *,int*);    // client
   void createObjFromStream(char*); // server
}DS_SessionSetupRequestMessage_type;


typedef struct {
   dsmccMessageHeader_type dsmccMessageHeader;
   __int16 response;
   __int8 compatibilityDescriptor;
   void getStream (char *,int*);    // client
   void createObjFromStream(char*); // client
}DS_SessionSetupConfirmMessage_type;


///////////////////////////////////////////////////////////////////////////////
// Session Release Request and Confirm message types and their initialization:/
///////////////////////////////////////////////////////////////////////////////
typedef struct {
   dsmccMessageHeader_type dsmccMessageHeader;
   long double networkSessionId;
   __int16 reason;
   void getStream (char *,int*);   
   void createObjFromStream(char*);
}DS_SessionReleaseRequestMessage_type;

typedef struct {
   dsmccMessageHeader_type dsmccMessageHeader;
   __int16 response;
   void getStream (char *,int*);   
   void createObjFromStream(char*);
}DS_SessionReleaseConfirmMessage_type;


///////////////////////////////////////////////////////////////////////////////
// Service Attach Request and Confirm message types and their initialization: /
///////////////////////////////////////////////////////////////////////////////
typedef struct  : public base_message{
   dsmccMessageHeader_type dsmccMessageHeader;
   long double networkSessionId; 
   __int16 serviceId;
   __int8 serviceNameLen;
   char * serviceName;
   ddDataClass * ddData; 
   void getStream (char*,int*);         // client   
   void createObjFromStream(char*);   //server
}DS_ServiceAttachRequestMessage_type;


typedef struct  : public base_message{
   dsmccMessageHeader_type dsmccMessageHeader;
   __int16 response;
   ddDataClass * ddData; 
   void getStream (char *,int*);             //server
   void createObjFromStream(char*);   //client
}DS_ServiceAttachConfirmMessage_type;


///////////////////////////////////////////////////////////////////////////////
// Service Detach Request and Confirm message types and their initialization: /
///////////////////////////////////////////////////////////////////////////////
typedef struct {
   dsmccMessageHeader_type dsmccMessageHeader;
   long double networkSessionId; 
   __int16 serviceId;
   __int16 reason;
   void getStream (char *,int*);    
   void createObjFromStream(char*);
}DS_ServiceDetachRequestMessage_type;

typedef struct {
   dsmccMessageHeader_type dsmccMessageHeader;
   __int16 response;
   void getStream (char *,int*);   
   void createObjFromStream(char*);
}DS_ServiceDetachConfirmMessage_type;


////////////////////////////////////////////////////////////////////////////
// Channel Add Request and Confirm message types and their initialization: /
////////////////////////////////////////////////////////////////////////////

/* This class includes all the data fileds that are required for assembling Channel ADD/DELETE/ADDED messages.
   In each case some of these data fields are ignored (in getstream functions).
*/

class channelElementsClass{
public:
   __int16 CAT; //2 Bytes
   __int16 TAT; //2 Bytes
   __int16 response;//2 Bytes
   __int8 direction; //1 Byte
   __int16 reason; //2 Bytes
   channelDescriptorClass channelDescriptor;
   ddDataClass ddData;
   channelDescriptorClass qosDescriptor;
   resourceDescriptorClass resourceDescriptor;

   ~channelElementsClass(){
   }
};

class GenericMsgLoop {

public:
   __int8 LoopCount;
   channelElementsClass * ElementPtr;
   GenericMsgLoop(int count){
      ElementPtr = new channelElementsClass[count];
      LoopCount = count;
   }
   GenericMsgLoop(){
      ElementPtr = new channelElementsClass[0];
      LoopCount = 0;
   }

   ~GenericMsgLoop(){
      delete [] ElementPtr;
   }

   void setLoopsize(int size)
   {
      delete [] ElementPtr;
      ElementPtr = new channelElementsClass[size];
      LoopCount = size;
   }

   channelElementsClass * operator % (int j) {
      return (channelElementsClass * )((int)ElementPtr + j*sizeof(channelElementsClass));
   }
};


 

typedef struct  : public base_message{
   dsmccMessageHeader_type dsmccMessageHeader;
   long double networkSessionId; 
   __int16 serviceId;
   __int8 LoopCount;
   GenericMsgLoop * p_CHReqLoop;
   void getStream (char *,int*);   
   void createObjFromStream(char*);
}DS_ChannelAddRequestMessage_type;

typedef struct  : public base_message{
   dsmccMessageHeader_type dsmccMessageHeader;
   __int8 LoopCount;
   GenericMsgLoop * p_CHCnfLoop;
   /* others */
   void getStream (char *,int*);   
   void createObjFromStream(char*);
}DS_ChannelAddConfirmMessage_type;



typedef struct  : public base_message{
   dsmccMessageHeader_type dsmccMessageHeader;
   long double networkSessionId; 
   __int8 LoopCount;
   GenericMsgLoop * p_CHReqLoop; //Loop(CAT,Reason)
   void getStream (char *,int*);   
   void createObjFromStream(char*);
}DS_ChannelDeleteRequestMessage_type;

typedef struct  : public base_message{
   dsmccMessageHeader_type dsmccMessageHeader;
   __int8 LoopCount;
   GenericMsgLoop * p_CHCnfLoop; // Loop (Response)
   /* others */
   void getStream (char *,int*);   
   void createObjFromStream(char*);
}DS_ChannelDeleteConfirmMessage_type;



typedef struct  : public base_message{
   dsmccMessageHeader_type dsmccMessageHeader;
   long double networkSessionId; 
   __int16 serviceId;
   __int8 LoopCount;
   GenericMsgLoop * p_CHReqLoop;
   void getStream (char *,int*);   
   void createObjFromStream(char*);
}DS_ChannelAddedRequestMessage_type;

typedef struct  : public base_message{
   dsmccMessageHeader_type dsmccMessageHeader;
   __int8 LoopCount;
   GenericMsgLoop * p_CHCnfLoop;
   /* others */
   void getStream (char *,int*);   
   void createObjFromStream(char*);
}DS_ChannelAddedConfirmMessage_type;

///////////////////////////////////////////////////////////////////////////////
// Resources and TransmuxSetup Request and Confirm message types:             /
///////////////////////////////////////////////////////////////////////////////

typedef struct {
   dsmccMessageHeader_type dsmccMessageHeader;
   long double networkSessionId;
   __int8 LoopCount;
   GenericMsgLoop * p_TransmuxReqLoop;
   void getStream (char *,int*);   
   void createObjFromStream(char*);
} DS_TransmuxSetupRequestMessage_type;

typedef struct {
   dsmccMessageHeader_type dsmccMessageHeader;
   __int8 LoopCount;
   GenericMsgLoop * p_TransmuxConfLoop;
   void getStream (char *,int*);   
   void createObjFromStream(char*);
} DS_TransmuxSetupConfirmMessage_type;



typedef struct {
   dsmccMessageHeader_type dsmccMessageHeader;
   long double networkSessionId;
   __int8 LoopCount;
   GenericMsgLoop * p_TransmuxReqLoop; // The loop only contains TAT .
   void getStream (char *,int*);   
   void createObjFromStream(char*);
} DS_TransmuxReleaseRequestMessage_type;

typedef struct {
   dsmccMessageHeader_type dsmccMessageHeader;
   __int8 LoopCount;
   GenericMsgLoop * p_TransmuxConfLoop; // Loop(response)
   void getStream (char *,int*);   
   void createObjFromStream(char*);
} DS_TransmuxReleaseConfirmMessage_type;


typedef struct : public base_message{
   dsmccMessageHeader_type dsmccMessageHeader;
   long double networkSessionId;
   __int8 LoopCount; //1 Byte
   GenericMsgLoop * p_TransmuxReqLoop; // The loop only contains TAT .
   void getStream (char *,int*);   
   void createObjFromStream(char*);
} DS_TransmuxConfigRequestMessage_type;

typedef struct {
   dsmccMessageHeader_type dsmccMessageHeader;
   __int8 LoopCount; //1 Byte
   GenericMsgLoop * p_TransmuxConfLoop; // Loop(response)
   void getStream (char *,int*);   
   void createObjFromStream(char*);
} DS_TransmuxConfigConfirmMessage_type;

///////////////////////////////////////
// UserCommand message type:          / 
///////////////////////////////////////

typedef struct : public base_message{
   dsmccMessageHeader_type dsmccMessageHeader;
   long double networkSessionId;
   ddDataClass * ddData;
   GenericMsgLoop * p_CATLoop;
   __int8 LoopCount;
   void getStream (char *,int*);   
   void createObjFromStream(char*);
} DS_UserCommandMessage_type;

typedef struct : public base_message{
   dsmccMessageHeader_type dsmccMessageHeader;
   long double networkSessionId;
   __int16 response;
   ddDataClass * ddData;
   void getStream (char *,int*);   
   void createObjFromStream(char*);
} DS_UserCommandAckConfirmMessage_type;




#endif // #ifndef _MESSAGES_H