/******************************************************************************
*  SOFTWARE LICENSE AND CONDITIONS
*
*  This software module was originally developed by Kambiz Asrar Haghighi and 
*  Yasser Pourmohammadi (University of British Columbia) in the course of 
*  development of  the streaming functionality for MPEG-4 using the DMIF
*  (ISO/IEC 14496-6) standard. This software module is an implementation 
*  of a part of one or more MPEG-4 tools as specified by MPEG-4. 
*  
*  Users of the MPEG-4 may use the software source code or modifications 
*  thereof only for hardware or software products related to MPEG-4 evaluation
*  and testing.  The use of the source code for any purpose other than 
*  evaluation and testing, including the use of derivative works for the 
*  generation of commercial products, is strictly prohibited.
*  
*  The original developer of this software module and his/her company, 
*  the subsequent editors and their companies, and ISO/IEC have no 
*  liability for use of this software module or modifications thereof 
*  in an implementation. Copyright is not released for non MPEG-4 
*  conforming products. UBC retains full right to use the code for 
*  his/her own purpose, assign or donate the code to a third party and 
*  to inhibit third parties from using the code for non MPEG-4 conforming 
*  products. This copyright notice must be included in all copies or 
*  derivative works.
*
*  Contact:
*
*  Kambiz Asrar Haghighi                 email: kambiz@interchange.ubc.ca
*  Yasser Pourmohammadi                  email: yasserp@ece.ubc.ca
*  Professor Hussein M. Alnuweiri
*  UBC Signal Processing and Multimedia Group
*  http://spmg.ece.ubc.ca/research/MPEG4
*  Vancouver BC Canada V6T1Z4                  
*
*  Copyright  2000  University of British Columbia, Canada
********************************************************************************/

#include "common.h"
#include "messages.h"

///////////////////////////////////////////////////////////////////
// Shared Functions between client and server
///////////////////////////////////////////////////////////////////
void dsmccMessageHeader_type::appendStream(char *tempstr, int * len)
{
   memcpy(tempstr,&protocolDiscriminator,1);
   memcpy(tempstr+1,&dsmccType,1);
   memcpy(tempstr+2,&messageId,2);
   memcpy(tempstr+4,&transactionId,4);
   memcpy(tempstr+8,&reserved,1);
   memcpy(tempstr+9,&adaptationLength,1);
   memcpy(tempstr+10,&messageLength,2);
   *len=12;
}

void dsmccMessageHeader_type::getHeader(char * stream)
{
   memcpy(&protocolDiscriminator, stream, 1);
   memcpy(&dsmccType, (stream +1), 1);
   memcpy(&messageId, (stream +2), 2);;
   memcpy(&transactionId, (stream + 4), 4);;
   memcpy(&reserved, (stream + 8), 1);
   memcpy(&adaptationLength, (stream + 9), 1);
   memcpy(&messageLength, (stream + 10), 2);
} 

void base_message::appendDDdata(char * locationptr, ddDataClass * tempddPtr, int * len)
{
   // Append ddData

   *len = 0;
   __int16 DMIFcount = tempddPtr->getDescCount();
   // if the DMIFcount is negative, assume that data has not been filled and set to zero
   if (DMIFcount < 0)
      DMIFcount = 0; /* DEV */ // should we give an error message?
   memcpy(locationptr,&(DMIFcount),2);locationptr+=2;*len += 2;
   for(int k=0; k < DMIFcount ; k++){ 
      dmifDescriptorClass * tempDescPtr = tempddPtr->DAPtr->operator %(k);
      memcpy(locationptr,&(tempDescPtr->commonDescriptorHeader.DmifDescriptorType),2);locationptr+=2;*len += 2;
      memcpy(locationptr,&(tempDescPtr->commonDescriptorHeader.DmifDescriptorLen),2);locationptr+=2;*len += 2;
      memcpy(locationptr,tempDescPtr->descriptorDataFields,tempDescPtr->commonDescriptorHeader.DmifDescriptorLen);
      locationptr+=tempDescPtr->commonDescriptorHeader.DmifDescriptorLen; *len += tempDescPtr->commonDescriptorHeader.DmifDescriptorLen;
   }

}

BOOL base_message::getDDdata(ddDataClass * tempPtr, char* locationptr, int *len)
{
   __int16 ddDataIntCount;
   *len = 0;
   memcpy(&ddDataIntCount,locationptr,2); locationptr += 2;*len+=2;

   tempPtr->setArraySize(ddDataIntCount);
   for (int k = 0; k < ddDataIntCount; k++) {
      memcpy(&(tempPtr->DAPtr->operator %(k)->commonDescriptorHeader.DmifDescriptorType), locationptr, 2);locationptr+=2;*len+=2;
      if( !(tempPtr->DAPtr->operator %(k)->commonDescriptorHeader.DmifDescriptorType > 0) || 
          !(tempPtr->DAPtr->operator %(k)->commonDescriptorHeader.DmifDescriptorType < 5)   )
          return FALSE;
         
      memcpy(&(tempPtr->DAPtr->operator %(k)->commonDescriptorHeader.DmifDescriptorLen), locationptr, 2);locationptr+=2;*len+=2;
      tempPtr->DAPtr->operator %(k)->copyToDataFields(tempPtr->DAPtr->operator %(k)->commonDescriptorHeader.DmifDescriptorLen
                                                       ,locationptr);  
      locationptr+=tempPtr->DAPtr->operator %(k)->commonDescriptorHeader.DmifDescriptorLen;
      *len+=tempPtr->DAPtr->operator %(k)->commonDescriptorHeader.DmifDescriptorLen;
   }
   return TRUE;
}


/*--------------------------------------------------------------------------------------------------------------------------------
NetSessionListClass functions: 
--------------------------------------------------------------------------------------------------------------------------------*/
/*   LinkedList for binding networkSessionId, sigsocket ... */
NetSessionListClass::NetSessionListClass()
{
   //List = new NetSessionListStruct;
   List = NULL;
}

NetSessionListClass::~NetSessionListClass()
{
   NetSessionRecordStruct* currentElement = this->List;
   NetSessionRecordStruct* prevElement;

   while(currentElement != NULL)
   {
      prevElement = currentElement;
      currentElement = currentElement->nextElement;
      delete prevElement;
   }
}

BOOL NetSessionListClass::addElementstoSessionList(long double networkSessionId, SOCKET sigSocket, SOCKADDR_IN destination_sin)
{
   NetSessionRecordStruct* tempList = this->List;

   NetSessionRecordStruct* netSessListItem = new NetSessionRecordStruct;
   netSessListItem->networkSessionId = networkSessionId;
   netSessListItem->sigSocket = sigSocket;
   netSessListItem->destination_sin = destination_sin;
   netSessListItem->nextElement = NULL;
   netSessListItem->initializeUPList();
   if (tempList == NULL)
      this->List = netSessListItem ;
   else
   {   
      while (tempList->nextElement != NULL)
         tempList = tempList->nextElement;
      tempList->nextElement = netSessListItem;
   }   
   return TRUE;
}

#ifdef _SERVER
BOOL NetSessionListClass::addElementstoSessionList(long double networkSessionId,NetworkSessionClass * objPtr, 
                                                   SOCKET sigSocket, SOCKADDR_IN destination_sin)
{
   NetSessionRecordStruct* tempList = this->List;

   NetSessionRecordStruct* netSessListItem = new NetSessionRecordStruct;
   netSessListItem->p_ServingObject = objPtr;
   netSessListItem->networkSessionId = networkSessionId;
   netSessListItem->sigSocket = sigSocket;
   netSessListItem->destination_sin = destination_sin;
   netSessListItem->nextElement = NULL;
   netSessListItem->initializeUPList();
   if (tempList == NULL)
      this->List = netSessListItem ;
   else
   {   
      while (tempList->nextElement != NULL)
         tempList = tempList->nextElement;
      tempList->nextElement = netSessListItem;
   }   
   return TRUE;
}


BOOL NetSessionListClass::addElementstoSessionList(long double networkSessionId,NetworkSessionClass * objPtr)
{
   NetSessionRecordStruct* tempList = this->List;

   NetSessionRecordStruct* netSessListItem = new NetSessionRecordStruct;
   netSessListItem->p_ServingObject = objPtr;
   netSessListItem->networkSessionId = networkSessionId;
   netSessListItem->nextElement = NULL;
   netSessListItem->initializeUPList();
   if (tempList == NULL)
      this->List = netSessListItem ;
   else
   {   
      while (tempList->nextElement != NULL)
         tempList = tempList->nextElement;
      tempList->nextElement = netSessListItem;
   }   
   return TRUE;
}
#endif


SOCKET NetSessionListClass::findSocketfromNSID(long double networkSessionId)
{
   NetSessionRecordStruct* tempList = this->List;
   
   while(tempList != NULL )
   {
      if(tempList->networkSessionId == networkSessionId)
         return tempList->sigSocket;
      tempList = tempList->nextElement;
   }
   return INVALID_SOCKET;

}

SOCKADDR_IN NetSessionListClass::findDest_SinfromNSID(long double networkSessionId)
{
   SOCKADDR_IN alaki_sin;
   alaki_sin.sin_family=AF_INET;
   
   NetSessionRecordStruct* tempList = this->List;
      
   while(tempList != NULL )
   {
      if (tempList->networkSessionId == networkSessionId)
         return tempList->destination_sin;
      tempList = tempList->nextElement;     
   }
   return alaki_sin;
}



BOOL NetSessionListClass::deleteElementfromSessionList(long double networkSessionId)
{
   // Find the element
   NetSessionRecordStruct* currentElement = this->List;
   NetSessionRecordStruct* prevElement;

   if (currentElement == NULL)
      return FALSE;
 
   if (currentElement->networkSessionId == networkSessionId)
      List = List->nextElement;
   else {
      while(currentElement->networkSessionId != networkSessionId )
      {
         prevElement = currentElement;
         currentElement = currentElement->nextElement;      
      }
      prevElement->nextElement = currentElement->nextElement;
   }
   delete currentElement;
   return TRUE;
   
}

void NetSessionListClass::addTATtoNetSessionRecord(long double networkSessionId,SOCKET TMSocket,
                                                    __int16 TAT,__int8 direction,DWORD destIPAddress,__int16 destPort)
{
   NetSessionRecordStruct* tempList = List;
   
   while(tempList != NULL )
   {
      if(tempList->networkSessionId == networkSessionId){
         tempList->addRecordtoUPList(TMSocket,TAT,direction,destIPAddress,destPort);
      }         
      tempList = tempList->nextElement;
   }
   return;
}

SOCKET NetSessionListClass::deleteTATfromNetSessionRecord(long double networkSessionId,__int16 TAT)
{
   SOCKET tempUPSocket;
   DWORD dummy;
   __int16 dummy2;
   NetSessionRecordStruct* tempList = List;
   
   while(tempList != NULL )
   {
      if(tempList->networkSessionId == networkSessionId){
         tempList->getElements(TAT,&dummy,&dummy2,&tempUPSocket);
         tempList->deleteRecordfromUPList(TAT);
      }         
      tempList = tempList->nextElement;
   }
   return tempUPSocket;
}
#ifdef _SERVER
NetworkSessionClass * NetSessionListClass::getNetSessionObj(long double NSID)
{
   
   NetSessionRecordStruct* tempList = this->List;
      
   while(tempList != NULL )
   {
      if (tempList->networkSessionId == NSID)
         return tempList->p_ServingObject;
      tempList = tempList->nextElement;     
   }
   return NULL;
}
#endif
void NetSessionListClass::getUPInfo(__int16 TAT, DWORD * destIP,__int16 *destport,SOCKET *UPsocket)
{
     
   NetSessionRecordStruct* tempList = this->List;
      
   while(tempList != NULL )
   {

      if (tempList->getTAT() == TAT)
      { 
         List->getElements(TAT,destIP,destport,UPsocket);
         break;
      }
      tempList = tempList->nextElement;     
   }

}
//----------------------------------------------------------- Networ Session Record Class functions

void NetSessionRecordStruct::initializeUPList()
{

   UserPlaneList = NULL;
}




void NetSessionRecordStruct::addRecordtoUPList(SOCKET TransmuxSocket,__int16 TAT, 
                                               __int8 direction,DWORD destIP,__int16 destPort)
{

   UserPlaneRecord_Type* tempList = UserPlaneList;

   UserPlaneRecord_Type* uPlaneItem = new UserPlaneRecord_Type;
   uPlaneItem->TAT = TAT;
   uPlaneItem->TransmuxSocket = TransmuxSocket;
   uPlaneItem->direction = direction;
   uPlaneItem->dest_Transmux_tuple.IPAddress=destIP;
   uPlaneItem->dest_Transmux_tuple.Port = destPort;
   uPlaneItem->nextUPRecord= NULL;   
   if (tempList == NULL)
      UserPlaneList = uPlaneItem ;
   else
   {   
      while (tempList->nextUPRecord!= NULL)
         tempList = tempList->nextUPRecord;
      tempList->nextUPRecord= uPlaneItem;
   }
   return ;
}


BOOL NetSessionRecordStruct::deleteRecordfromUPList(__int16 TAT)
{
   // Find the element
   UserPlaneRecord_Type* currentElement = this->UserPlaneList;
   UserPlaneRecord_Type* prevElement;

   if (currentElement == NULL)
      return FALSE;
 
   if (currentElement->TAT == TAT)
      UserPlaneList = UserPlaneList->nextUPRecord;
   else {
      while(currentElement != NULL && currentElement->TAT != TAT)
      {
         prevElement = currentElement;
         currentElement = currentElement->nextUPRecord;
      }
      if (currentElement != NULL)
         prevElement->nextUPRecord = currentElement->nextUPRecord;
   }
   delete currentElement;
   return TRUE;
 
};      

void NetSessionRecordStruct::getElements(__int16 TAT,DWORD * destIP,__int16 *destport,SOCKET *UPsocket)
{
   UserPlaneRecord_Type* tempList = this->UserPlaneList;
   
   while(tempList != NULL )
   {
      if(tempList->TAT == TAT ){
         *destIP = tempList->dest_Transmux_tuple.IPAddress;
         *destport = tempList->dest_Transmux_tuple.Port;
         *UPsocket = tempList->TransmuxSocket;
         return;
      }
      tempList = tempList->nextUPRecord;
   }
   return;
};

