/******************************************************************************
*  SOFTWARE LICENSE AND CONDITIONS
*
*  This software module was originally developed by Yasser Pourmohammadi and 
*  Kambiz Asrar Haghighi (University of British Columbia) in the course of 
*  development of  the streaming functionality for MPEG-4 using the DMIF
*  (ISO/IEC 14496-6) standard. This software module is an implementation 
*  of a part of one or more MPEG-4 tools as specified by MPEG-4. 
*  
*  Users of the MPEG-4 may use the software source code or modifications 
*  thereof only for hardware or software products related to MPEG-4 evaluation
*  and testing.  The use of the source code for any purpose other than 
*  evaluation and testing, including the use of derivative works for the 
*  generation of commercial products, is strictly prohibited.
*  
*  The original developer of this software module and his/her company, 
*  the subsequent editors and their companies, and ISO/IEC have no 
*  liability for use of this software module or modifications thereof 
*  in an implementation. Copyright is not released for non MPEG-4 
*  conforming products. UBC retains full right to use the code for 
*  his/her own purpose, assign or donate the code to a third party and 
*  to inhibit third parties from using the code for non MPEG-4 conforming 
*  products. This copyright notice must be included in all copies or 
*  derivative works.
*
*  Contact:
*
*  Kambiz Asrar Haghighi                 email: kambiz@interchange.ubc.ca
*  Yasser Pourmohammadi                  email: yasserp@ece.ubc.ca
*  Professor Hussein M. Alnuweiri
*  UBC Signal Processing and Multimedia Group
*  http://spmg.ece.ubc.ca/research/MPEG4
*  Vancouver BC Canada V6T1Z4                  
*
*  Copyright  2000  University of British Columbia, Canada
********************************************************************************/

//Common.h
#if !defined (COMMON_H)
#define COMMON_H

#include <stdlib.h>
#ifdef _SERVER
#include "DN_server_udp.h"
#include "Server_DN_Package.h"
#endif

#include <winsock.h>


//----------------------------------------- Network Session List / Manager Class -----------------------------------------------
struct UserPlaneRecord_Type // The TAT is the unique identifier in this record. 
{
   __int16 TAT;            // LOCAL PORT TO RECEIVE ELEMENTARY STREAM THROUGH.
   SOCKET TransmuxSocket;  // UDP Instance only uses one transmux Socket for each Network Session
   __int8 direction;
   struct {
      DWORD IPAddress;
      __int16 Port;
   } dest_Transmux_tuple;
   UserPlaneRecord_Type * nextUPRecord;
};

struct NetSessionRecordStruct
{
   long double networkSessionId;
   SOCKET sigSocket;
   SOCKADDR_IN destination_sin;
#ifdef _SERVER
   NetworkSessionClass * p_ServingObject;
#endif

   NetSessionRecordStruct* nextElement;

   void initializeUPList();
   void addRecordtoUPList(SOCKET TransmuxSocket,__int16 TAT, __int8 direction,DWORD destIP,__int16 destPort);
   BOOL deleteRecordfromUPList(__int16 TAT);     
   void getElements(__int16 TAT,DWORD * destIP,__int16 *detport,SOCKET *UPsocket);
   __int16 getTAT(){
      return UserPlaneList->TAT;
   };
private :
   UserPlaneRecord_Type * UserPlaneList;
};

class NetSessionListClass
{
public:
   NetSessionListClass();
   ~NetSessionListClass();
#ifdef _SERVER
   // Only used by server :
   NetworkSessionClass * createNetSessionInstance(){
      return new NetworkSessionClass;}
   BOOL addElementstoSessionList(long double networkSessionId,NetworkSessionClass *, SOCKET sigSocket, SOCKADDR_IN destination_sin);
   NetworkSessionClass * getNetSessionObj(long double NSID);
   BOOL addElementstoSessionList( long double networkSessionId, NetworkSessionClass *);
#endif
   // common
   BOOL deleteElementfromSessionList(long double networkSessionId);
   // Only used by client      
   BOOL addElementstoSessionList(long double networkSessionId, SOCKET sigSocket, SOCKADDR_IN destination_sin);
   SOCKET findSocketfromNSID(long double networkSessionId);
   SOCKADDR_IN findDest_SinfromNSID(long double networkSessionId);
   void addTATtoNetSessionRecord(long double networkSessionId,SOCKET TMsocket,__int16 TAT,__int8 direction, DWORD,__int16);
   SOCKET deleteTATfromNetSessionRecord(long double networkSessionId,__int16 TAT);
   //      void addChannelInfotoNetSessionRecord(long double networkSessionId,__int16 TAT,__int16 CAT,__int8 direction);
   void getUPInfo(__int16 TAT, DWORD * destIP,__int16 *destport,SOCKET *UPsocket);
   __int16 getTAT(){
      return List->getTAT() ;};
private:
   NetSessionRecordStruct * List;
};

#endif COMMON_H