/******************************************************************************
*  SOFTWARE LICENSE AND CONDITIONS
*
*  This software module was originally developed by Yasser Pourmohammadi and 
*  Kambiz Asrar Haghighi (University of British Columbia) in the course of 
*  development of  the streaming functionality for MPEG-4 using the DMIF
*  (ISO/IEC 14496-6) standard. This software module is an implementation 
*  of a part of one or more MPEG-4 tools as specified by MPEG-4. 
*  
*  Users of the MPEG-4 may use the software source code or modifications 
*  thereof only for hardware or software products related to MPEG-4 evaluation
*  and testing.  The use of the source code for any purpose other than 
*  evaluation and testing, including the use of derivative works for the 
*  generation of commercial products, is strictly prohibited.
*  
*  The original developer of this software module and his/her company, 
*  the subsequent editors and their companies, and ISO/IEC have no 
*  liability for use of this software module or modifications thereof 
*  in an implementation. Copyright is not released for non MPEG-4 
*  conforming products. UBC retains full right to use the code for 
*  his/her own purpose, assign or donate the code to a third party and 
*  to inhibit third parties from using the code for non MPEG-4 conforming 
*  products. This copyright notice must be included in all copies or 
*  derivative works.
*
*  Contact:
*
*  Kambiz Asrar Haghighi                 email: kambiz@interchange.ubc.ca
*  Yasser Pourmohammadi                  email: yasserp@ece.ubc.ca
*  Professor Hussein M. Alnuweiri
*  UBC Signal Processing and Multimedia Group
*  http://spmg.ece.ubc.ca/research/MPEG4
*  Vancouver BC Canada V6T1Z4                  
*
*  Copyright  2000  University of British Columbia, Canada
********************************************************************************/

#include "RemoteInstance.h"
#include "DN_udp.h"
#include "DN_tcp.h"
#include "mpg4dmif.h"

// Global parameters that must be known to (and shared amongst) all objects of RemoteInstance class:
// Local to DMIF;

// Service Record List for all Services
ServiceRecordClassList SRCList;


UINT serviceId;
UINT ssid;


LPCSTR RemoteInstance::s_moduleID = "RemoteInstance";
LPCSTR RemoteInstance::s_myUDPURLScheme = "x-dudp:";
LPCSTR RemoteInstance::s_myTCPURLScheme = "x-dtcp:";

extern "C" Service *CreateInstance()
{
	return new RemoteInstance;
}

LPCSTR RemoteInstance::GetModuleID() 
{
	return s_moduleID;
}

RemoteInstance::RemoteInstance()
{
   m_sink = NULL;
   userThread = NULL;
}


RemoteInstance::~RemoteInstance()
{
}


void RemoteInstance :: AddInterface (const char* sTag,const void* pSink,void*& r_pSink)
{
	//No extra interfaces
	sTag;pSink;
	r_pSink = (void*) NULL;
}


void RemoteInstance::DPI_ServiceAttach_Req(LPCSTR szURL, const BYTE* uuDataInBuffer, int uuDataInLen)
{
   char   myname[256];
   LPCSTR calledAddress;
   LPCSTR callingAddress;
   char serviceName[20];
   __int8 compatibilityDescriptorIn;
   __int8 compatibilityDescriptorOut;
   long double networkSessionId;
   __int16 serviceId;
   __int16 response;
   ddDataClass ddDataIn(0), ddDataOut(0);


   // find out whether we need TCP or UDP object and create it
   if (_strnicmp( szURL, s_myUDPURLScheme, strlen(s_myUDPURLScheme) ) == 0)
      DN_Object = new DN_UDP_CLASS;
   if (_strnicmp( szURL, s_myTCPURLScheme, strlen(s_myTCPURLScheme) ) == 0)
      DN_Object = new DN_TCP_CLASS;

   // temp variables for determining calledAddress
   int atSign = '@';
   int endOfAdd = '/';
   char *start, *end;
   char tmp[100] = "";
   int startpoint, endpoint, value;

   // temp variables for determining calling address
   struct hostent *h;
   int x = 0;
   BOOL existingNSID = FALSE;


   // determine called address from URL
   szURL = szURL + 9;
   // check whether entered URL is valid
   if (checkURLValidity(szURL) == FALSE) {
      GetSink()->DPI_ServiceAttach_Cnf(this, NULL, 0, RESPONSE_URL_NOT_RECOGNIZED);
      return;
   }

   // parse URL
   start = strchr (szURL, atSign);
   if (start == NULL) {
      GetSink()->DPI_ServiceAttach_Cnf(this, NULL, 0, RESPONSE_URL_NOT_RECOGNIZED);
      return;
   }

   startpoint = start - szURL + 1;

   end = strchr (szURL, endOfAdd);
   if (end == NULL) {
      GetSink()->DPI_ServiceAttach_Cnf(this, NULL, 0, RESPONSE_URL_NOT_RECOGNIZED);
      return;
   }
   endpoint = end - szURL + 1;

   value = endpoint - startpoint;
   if (value > 0)
      strncpy (tmp, start, value);
   else {
      GetSink()->DPI_ServiceAttach_Cnf(this, NULL, 0, RESPONSE_URL_NOT_RECOGNIZED);
      return;
   }
   calledAddress = tmp + 1;

   // determine serviceName
   strncpy (serviceName,(end+1),(strlen(szURL)-endpoint+1));

   // determine the calling address
   gethostname(myname, sizeof(myname));        /* who are we? */
   if ((h=gethostbyname(myname)) == NULL) {  /* get the host info */
      x = WSAGetLastError ();
      exit(1);
   }

   callingAddress = inet_ntoa(*((struct in_addr *)h->h_addr));


   // set compatibility descriptor in to be null (not used currently)
   compatibilityDescriptorIn = (__int8)0xFF;

   // check if there is no network session active, then create one
   if (SRCList.serviceExists((__int32)this, &serviceId) == FALSE)
   {
      // determine if a networkSession exists to destination URL
      existingNSID = SRCList.findNSIDfromURL(szURL, &networkSessionId);
      SRCList.addNewRecordtoList((__int32)this, serviceId, networkSessionId, szURL);
   }
   else
   {
      networkSessionId = SRCList.getNetworkSessionId((__int32)this);
      existingNSID = TRUE;
   }


   // if there is no uuData to transmit, do not make a ddData object
   if (uuDataInBuffer != NULL)
   {
      ddDataIn.setArraySize(1);
      ddDataIn.DAPtr->operator %(0)->commonDescriptorHeader.DmifDescriptorType = UUDATADESCRIPTOR;
      ddDataIn.DAPtr->operator %(0)->copyToDataFields(uuDataInLen, (char*)uuDataInBuffer);
   }
   if (existingNSID == TRUE)
   {
      // use existing service
      // service_attach
      DN_Object->DN_ServiceAttach(networkSessionId, serviceId, serviceName, 
                       &ddDataIn, &response/*out*/, &ddDataOut /*out*/);

   }
   else
   {
      // create new service
      // session_setup & service_attach
      DN_Object->DN_SessionSetup(networkSessionId, calledAddress, callingAddress, 
                      compatibilityDescriptorIn, &response, &compatibilityDescriptorOut);
      DN_Object->DN_ServiceAttach(networkSessionId,serviceId, serviceName, 
                       &ddDataIn, &response/*out*/, &ddDataOut /*out*/);
   }



   
   /*------------------------------------------
	   Extract the IOD -- from ddDataOut
   -------------------------------------------*/
   int iodLength=0;
   for(int i=0;i<ddDataOut.getDescCount();i++){
      if(ddDataOut.DAPtr->operator %(i)->commonDescriptorHeader.DmifDescriptorType == UUDATADESCRIPTOR)
          iodLength+=ddDataOut.DAPtr->operator %(i)->commonDescriptorHeader.DmifDescriptorLen;
   }
   BYTE* initialOD = (BYTE*)malloc(iodLength); /* KNOWN BUGS */ 
   // memory space allocated to initialOd must be freed eventually, but doesn't seem so
   BYTE* locationptr = initialOD;
   for(i=0;i<ddDataOut.getDescCount();i++){
      if(ddDataOut.DAPtr->operator %(i)->commonDescriptorHeader.DmifDescriptorType == UUDATADESCRIPTOR){
         memcpy(locationptr,ddDataOut.DAPtr->operator %(i)->descriptorDataFields,
         ddDataOut.DAPtr->operator %(i)->commonDescriptorHeader.DmifDescriptorLen);
         locationptr += ddDataOut.DAPtr->operator %(i)->commonDescriptorHeader.DmifDescriptorLen;
      }
   }

   /*-------------------------------------------
		   Return the IOD to the application
	-------------------------------------------*/

   GetSink()->DPI_ServiceAttach_Cnf (
	this,  // this is a pointer to pService ; DPI_ServiceAttach is defined in DMIFClientFilter; commented below 
	initialOD,             // DPI_ServiceAttach then returns the ssId by passing its nTmpId AND nSeriveId to DAI_cnf
	iodLength,
	(tagResponseParameter)RESPONSE_OK
	);
}


void RemoteInstance::DPI_ServiceDetach_Req( REASON )
{
   __int16 serviceId;
   long double networkSessionId;
   __int16 response;

   //MessageBox (NULL, TEXT("Service Detach."), TEXT("Error"), MB_OK);

   networkSessionId = SRCList.getNetworkSessionId((__int32)this);
   serviceId = SRCList.getServiceId((__int32)this);

   // send service detach request to server
   DN_Object->DN_ServiceDetach(networkSessionId, serviceId, REASON_OK, &response);

   // if response is OK, check to see if we can release network session
   if (response == RESPONSE_OK) {
      if (SRCList.deleteRecordfromList(serviceId))
      {
         // Check to see if there are any services left in the network
         // session, if there aren't, delete the network session record from SRCList
         if (SRCList.sessionExists(networkSessionId) == FALSE) {
            DN_Object->DN_SessionRelease(networkSessionId, REASON_OK, &response);
         }
         GetSink()->DPI_ServiceDetach_Cnf( this, (tagResponseParameter)response );
      }
      else
         MessageBox (NULL, TEXT("Could not find serviceId."), TEXT("Error"), MB_OK);
   }
   else
      GetSink()->DPI_ServiceDetach_Cnf(this, RESPONSE_SERVICE_DETACH_ERROR);
   return;
}



void RemoteInstance::DPI_ChannelAdd_Req( const DAIChannelAddParam &inParam )
{
   __int16 serviceId;
   int i;
   Upmesg *pumesg;
   pumesg = new Upmesg;
   long double networkSessionId;
   RESPONSE eResponse = RESPONSE_INVALID_DIRECTION;
   __int8 NumberOfChannels = (__int8)inParam.GetLength();
   GenericMsgLoop ReqLoop(NumberOfChannels); // Loop size must be set now, setLoopSize be called later.
   GenericMsgLoop RespLoop(NumberOfChannels);

   networkSessionId = SRCList.getNetworkSessionId((__int32)this);
   serviceId = SRCList.getServiceId((__int32)this);
  
   if ( SRCList.getTAT(serviceId) == NULL ){
      GenericMsgLoop TMReqLoop(1); /* DEV */ // For UDP instance only one TAT is needed per service
      GenericMsgLoop TMRespLoop(1);
      TMReqLoop.operator %(0)->direction = DIRECTION_UPSTREAM;

      // Set-up transmux channel between client and server
      DN_Object->DN_TransMuxSetup(networkSessionId,&TMReqLoop,&TMRespLoop);
   
      for(int j=0;j < TMRespLoop.LoopCount /* assume UDP only has 1-to-1 relationship with TAT */;j++){
         SRCList.putTAT((__int32)this,TMReqLoop.operator %(j)->TAT,TMReqLoop.operator %(j)->direction);
         // this layer just passes the information to the higher layer and the upper layer starts the thread
         // Start a thread to listen to User data stream sent by the server.
         pumesg->pService = this;
         pumesg->TAT = TMReqLoop.operator %(j)->TAT;
         userThread = (void*)_beginthread(userPlaneThreadProc,NULL,(void*)pumesg);
      }

   }

   for ( i = 0; i < NumberOfChannels; i++ )
   {  
		const DAIChannelAddElement &element = inParam[ i ]; 
      ReqLoop.operator %(i)->CAT = element.m_nChid;
      ReqLoop.operator %(i)->TAT = SRCList.getTAT(serviceId)->TAT;
      ReqLoop.operator %(i)->direction = element.m_eDirection;
      /* YK */ // AppInfo (SLConfigDesc) is not meaningful by the application and is ignored at the server
      if( element.m_nDataLen != 0 ){
         if( element.m_nAppInfoLen != 0)
         {
            ReqLoop.operator %(i)->ddData.setArraySize(2);
            ReqLoop.operator %(i)->ddData.DAPtr->operator %(0)->commonDescriptorHeader.DmifDescriptorType = UUDATADESCRIPTOR;
            ReqLoop.operator %(i)->ddData.DAPtr->operator %(0)->copyToDataFields(element.m_nDataLen,(char*)element.m_pData);
            ReqLoop.operator %(i)->ddData.DAPtr->operator %(1)->commonDescriptorHeader.DmifDescriptorType = UUDATADESCRIPTOR;
            ReqLoop.operator %(i)->ddData.DAPtr->operator %(1)->copyToDataFields(element.m_nAppInfoLen,(char*)element.m_pAppInfo);
         }
         else
         {
            ReqLoop.operator %(i)->ddData.setArraySize(1);
            ReqLoop.operator %(i)->ddData.DAPtr->operator %(0)->commonDescriptorHeader.DmifDescriptorType = UUDATADESCRIPTOR;
            ReqLoop.operator %(i)->ddData.DAPtr->operator %(0)->copyToDataFields(element.m_nDataLen,(char*)element.m_pData);
         }
      }

   	assert (element.m_nAppInfoLen == sizeof (struct SLConfigDesc));
      // OPTIONAL ChannelDescriptor parameters to be set NOT USED
      ReqLoop.operator %(i)->channelDescriptor.setArraySize(0);
   }
   
   // inform server that the channels have been added
   DN_Object->DN_ChannelAdded(networkSessionId,serviceId, &ReqLoop, &RespLoop);
	
   for ( i = 0; i < NumberOfChannels; i++ )
   {
   	const DAIChannelAddElement &element = inParam[ i ]; 
      SRCList.putCAT((__int32)this,(__int16)element.m_nChid,element.m_eDirection,RespLoop.operator %(i)->TAT);
      // uudataOut
		StreamMapElement rMapElement;
		rMapElement.m_nChid = element.m_nChid;
		rMapElement.m_nTmpId = element.m_nTmpId;
		rMapElement.m_nDataLen = RespLoop.operator %(i)->ddData.DAPtr->operator %(0)->commonDescriptorHeader.DmifDescriptorLen;
      if (rMapElement.m_nDataLen > 0)
      {
         rMapElement.m_pData = new unsigned char[rMapElement.m_nDataLen];
         memcpy(rMapElement.m_pData, RespLoop.operator %(i)->ddData.DAPtr->operator %(0)->descriptorDataFields,rMapElement.m_nDataLen);
      }
      else
         rMapElement.m_pData = NULL;
		rMapElement.m_nFmxInfo = 0;		// unused field
      eResponse = (tagResponseParameter)RespLoop.operator %(i)->response;
		GetSink()->DPI_ChannelAdd_Cnf( this, rMapElement, eResponse );
	}

	return;

}

void RemoteInstance::DPI_ChannelDelete_Req( const DAIChannelDeleteParam &inParam )
{
   __int16 serviceId;
   long double networkSessionId;
   RESPONSE eResponse = RESPONSE_INVALID_DIRECTION;

   __int8 NumberOfChannels = (__int8)inParam.GetLength();
   GenericMsgLoop ReqLoop(NumberOfChannels); // Loop size must be set now, setLoopSize be called later.
   GenericMsgLoop RespLoop(NumberOfChannels);
   networkSessionId = SRCList.getNetworkSessionId((__int32)this);
   serviceId = SRCList.getServiceId((__int32)this);

   for ( int i = 0; i < NumberOfChannels; i++ )
   {  
		const DAIChannelDeleteElement &element = inParam[ i ]; 
      ReqLoop.operator %(i)->CAT = element.m_nChid;
      ReqLoop.operator %(i)->reason = element.m_eReason;
   }
   
   DN_Object->DN_ChannelDelete(networkSessionId, &ReqLoop, &RespLoop);

   for ( i = 0; i < NumberOfChannels; i++ )
   {
   	const DAIChannelDeleteElement &element = inParam[ i ]; 
      SRCList.deleteCAT(serviceId,(__int16)element.m_nChid);

      eResponse = (tagResponseParameter)RespLoop.operator %(i)->response;
		GetSink()->DPI_ChannelDelete_Cnf( this, element.m_nChid, eResponse );
	}

   if ( (SRCList.getTAT(serviceId) != NULL) && (SRCList.getCAT(serviceId) == NULL) ){
      GenericMsgLoop TMReqLoop(1); /* DEV */ // For UDP instance only one TAT is needed per service
      GenericMsgLoop TMRespLoop(1);
      TMReqLoop.operator %(0)->TAT = SRCList.getTAT(serviceId)->TAT;

      DN_Object->DN_TransMuxRelease(networkSessionId, &TMReqLoop, &TMRespLoop);

      // Remove TAT from list and call end userListener Thread
      // can keep global handle for this thread in object to do this
      for(int j=0;j < TMRespLoop.LoopCount /* assume UDP only has 1-to-1 relationship with TAT */;j++){
         SRCList.deleteTAT(serviceId, TMReqLoop.operator %(0)->TAT);
      }
      CloseHandle(userThread);
   }

	return;
}			

void RemoteInstance::DPI_UserCommand_Req( const DAIUserCmdParam& inParam )
{
   long double networkSessionId;
	int parameterCount = inParam.m_nChLen;
   GenericMsgLoop CATLoop(parameterCount);
   ddDataClass ddData(0);

	for ( int i = 0; i < parameterCount; i++ )
	{
		int chID = inParam.m_pChid[i];
      CATLoop.operator %(i)->CAT = chID;
   }
   networkSessionId = SRCList.getNetworkSessionId((__int32)this);
   DN_Object->DN_UserCommand(networkSessionId, &ddData, &CATLoop);
}

void RemoteInstance::DPI_UserCommandAck_Req(const DAIUserCmdParam& inParam ) 
{
	bool bFound = true;
	DAIUserCmdParam outParam;
   long double networkSessionId;
	int parameterCount = inParam.m_nChLen;
   GenericMsgLoop CATLoop(parameterCount);
   ddDataClass ddData(0);
   __int16 response;
   ddDataClass ddDataOut(0);

	for ( int i = 0; i < parameterCount; i++ )
	{
		int chID = inParam.m_pChid[i];
      CATLoop.operator %(i)->CAT = chID;
   }

   if (inParam.m_nDataLen > 0)
   {
      ddData.setArraySize(1);
      ddData.DAPtr->operator %(0)->commonDescriptorHeader.DmifDescriptorType = UUDATADESCRIPTOR;
      ddData.DAPtr->operator %(0)->copyToDataFields(inParam.m_nDataLen, inParam.m_pData);
   }

   networkSessionId = SRCList.getNetworkSessionId((__int32)this);
   serviceId = SRCList.getServiceId((__int32)this);

   DN_Object->DN_UserCommandAck(networkSessionId, &ddData, &CATLoop, &response, &ddDataOut);

   if (response == RESPONSE_GOOD)
   {
	   char *string=(bFound==false || parameterCount==0) ? "ERROR" : "OK";
	   outParam.m_nDataLen=strlen(string)+1;
	   outParam.m_pData=new char[outParam.m_nDataLen];
	   strcpy(outParam.m_pData,string);
	   outParam.m_eResponse=RESPONSE_OK;

	   outParam.m_nChLen=inParam.m_nChLen;
	   outParam.m_pChid=inParam.m_pChid;
	   outParam.m_nCmdId=inParam.m_nCmdId;

      GetSink () -> DPI_UserCommandAck_Cnf(this,outParam);
   }
}

void RemoteInstance::DNI_DataReceive_Ind(char * pRecvData)
{
   char * locationptr = pRecvData;

   //SL_Depacketize (&slcd, &slph, (const BYTE *) *m_sampleH, packetSize, &nHeaderLen);
	__int16 nServiceId = SRCList.getServiceId((__int32)this);
   unsigned __int8 index ;    // FlexMuxChannelId = Channel Id in our case
   unsigned __int8 length;    // = nStreamDataLength + nAppDataLength;   // length of SL PDU
	BYTE*				pStreamData;
   // Get pAppData & length
	struct SLConfigDesc	slcd;
	struct SLHeader		slph;

	slcd. predefined = 0;
	slcd. useAccessUnitStartFlag = TRUE;
	slcd. useAccessUnitEndFlag = TRUE;
	slcd. useRandomAccessPointFlag = TRUE;
	slcd. hasRandomAccessUnitsOnlyFlag = FALSE;
	slcd. usePaddingFlag = FALSE;
	slcd. useTimeStampsFlag = TRUE;
	slcd. useIdleFlag = FALSE;
	slcd. durationFlag = FALSE;
	slcd. timeStampResolution = 1000;
	slcd. OCRResolution = 1000;
	slcd. timeStampLength = 32;	// must be  64
	slcd. OCRLength = 0;		// must be  64
	slcd. AU_Length = 0;		// must be  32
	slcd. instantBitrateLength = 0;	// must be ?????
	slcd. degradationPriorityLength = 0;
	slcd. AU_seqNumLength = 0; // must be  16
	slcd. packetSeqNumLength = 5; // must be  16
	slcd. timeScale = 1000;
	int nHeaderLen = 0;

//   MessageBox(NULL,"Receiving Data", TEXT("INFO"), MB_OK);
   locationptr = pRecvData;
   memcpy(&index,locationptr,sizeof(index));locationptr+=sizeof(index);
   memcpy(&length,locationptr,sizeof(length));locationptr+=sizeof(length);
   pStreamData = (BYTE*)malloc(length);
   memcpy(pStreamData,locationptr,length);locationptr+=length;

// For future development:
   // Looking into SL Header is not allowed in this layer:
   // However the following function is not looking into the SL header and is just reformating the received message,
   // we prefer to change the DataReceive_Ind instead, and keep this layer free from Sync Layer matters. 
   m_smpMode.P();
      SL_Depacketize(&slcd, &slph, pStreamData, length, &nHeaderLen);
   m_smpMode.V();
   
   GetSink()->DPI_DataReceive_Ind( this, index, pStreamData + nHeaderLen, length - nHeaderLen, (const BYTE *) &slph, sizeof (struct SLHeader), 0);
}

///////////
void RemoteInstance::DPI_ServiceAttach_Rsp(
	const BYTE*				pUuDataOutBuffer, 
	const int				nUuDataOutLen,
	const RESPONSE			eResponse)
{
	pUuDataOutBuffer; nUuDataOutLen; eResponse;
}

void RemoteInstance::DPI_ServiceDetach_Rsp(
	const RESPONSE			eResponse)
{
	eResponse;
}

FlexMuxDataParam* RemoteInstance::DPI_ChannelAdd_Rsp(
	const OnChannelsAddParam& outParam)
{
	outParam; return NULL;
}

FlexMuxDataParam* RemoteInstance::DPI_TransmuxAdd_Rsp(
	const OnTransmuxAddParam& outParam)
{
	outParam; return NULL;
}

void RemoteInstance::DPI_ChannelDelete_Rsp(
	const OnChannelsDeleteParam& outParam)
{
	outParam;
}

void RemoteInstance :: DPI_UserCommandAck_Rsp(
	const DAIUserCmdParam&	outParam)
{
	outParam;
}
///////////

/*-------------------------------------------------------------
This is my URL if it starts with the correct scheme or
is a valid file name
-------------------------------------------------------------*/
BOOL RemoteInstance::IsYourURL( LPCSTR szURL )
{
   if (_strnicmp( szURL, s_myUDPURLScheme, strlen(s_myUDPURLScheme) ) == 0) {
      return TRUE;
   }
   if (_strnicmp( szURL, s_myTCPURLScheme, strlen(s_myTCPURLScheme) ) == 0) {
      return TRUE;
   }
   return FALSE;
}

BOOL checkURLValidity(LPCSTR url)
{
   int atSign = '@';
   int colon = ':';
   int backslash = '/';
   char *tempURL;

   if (strchr(url,colon) == NULL)
      return FALSE;
   if ((tempURL = strchr(url,atSign)) == NULL)
      return FALSE;
   if (strchr(tempURL,colon) == NULL)
      return FALSE;
   if (strchr(url,backslash) == NULL)
      return FALSE;

   return TRUE;
}



/************************************************************************************/
YKSemaphore :: YKSemaphore (long lMaxCount, long lInitialCount)
{
	if (lInitialCount < 0)
		lInitialCount = lMaxCount;
	m_hSemaph = CreateSemaphore (NULL, lInitialCount, lMaxCount, NULL);
}
