/******************************************************************************
*  SOFTWARE LICENSE AND CONDITIONS
*
*  This software module was originally developed by Kambiz Asrar Haghighi 
*  and Yasser Pourmohammadi (University of British Columbia) in the course of 
*  development of  the streaming functionality for MPEG-4 using the DMIF
*  (ISO/IEC 14496-6) standard. This software module is an implementation 
*  of a part of one or more MPEG-4 tools as specified by MPEG-4. 
*  
*  Users of the MPEG-4 may use the software source code or modifications 
*  thereof only for hardware or software products related to MPEG-4 evaluation
*  and testing.  The use of the source code for any purpose other than 
*  evaluation and testing, including the use of derivative works for the 
*  generation of commercial products, is strictly prohibited.
*  
*  The original developer of this software module and his/her company, 
*  the subsequent editors and their companies, and ISO/IEC have no 
*  liability for use of this software module or modifications thereof 
*  in an implementation. Copyright is not released for non MPEG-4 
*  conforming products. UBC retains full right to use the code for 
*  his/her own purpose, assign or donate the code to a third party and 
*  to inhibit third parties from using the code for non MPEG-4 conforming 
*  products. This copyright notice must be included in all copies or 
*  derivative works.
*
*  Contact:
*
*  Kambiz Asrar Haghighi                 email: kambiz@interchange.ubc.ca
*  Yasser Pourmohammadi                  email: yasserp@ece.ubc.ca
*  Professor Hussein M. Alnuweiri
*  UBC Signal Processing and Multimedia Group
*  http://spmg.ece.ubc.ca/research/MPEG4
*  Vancouver BC Canada V6T1Z4                  
*
*  Copyright  2000  University of British Columbia, Canada
********************************************************************************/

#if !defined(_RemoteInstance_h)	// Sentry, use file only if it's not already included.
#define _RemoteInstance_h


#include "datastr.h"
#include "DataDesc.h"
#include "DPI.h"


class YKSemaphore
{
	HANDLE m_hSemaph;

public:
	YKSemaphore (long lMaxCount = 1, long lInitialCount = -1);
	~YKSemaphore () {CloseHandle (m_hSemaph);}
	DWORD P (UINT uTimeout = INFINITE)
		{return WaitForSingleObject (m_hSemaph, uTimeout);}
	BOOL V () {return ReleaseSemaphore (m_hSemaph, 1, NULL);}
};


// DN_functions
class DN_CLASS;

class RemoteInstance : public Service
{
public:
	RemoteInstance();
	virtual ~RemoteInstance();
	
   void DNI_DataReceive_Ind(char * recvbuffer); // called by the DN package. YK defined function

private:
	virtual BOOL IsYourURL( LPCSTR szURL );
	virtual LPCSTR GetModuleID();
	
	virtual void AddInterface (const char* sTag,const void* pSink,void*& r_pSink);
	
	virtual void DPI_ServiceAttach_Req(
		const LPCSTR	szURL, 
		const BYTE*		pUuData,
		const int		nUuDataLen
		);
	virtual void DPI_ServiceDetach_Req(
		const REASON	eReason
		);
	virtual void DPI_ChannelAdd_Req(
		const DAIChannelAddParam	&inParam
		);
	virtual void DPI_ChannelDelete_Req(
		const DAIChannelDeleteParam	&inParam
		);
	virtual void DPI_UserCommand_Req(
		const DAIUserCmdParam&	inParam
		);
	virtual	void DPI_UserCommandAck_Req(
		const DAIUserCmdParam&	outParam
		);
	virtual BOOL DPI_DataSend_Req(
		const unsigned int	nChid,
		const BYTE*				pStreamData,
		const int				nStreamDataLength,
		const BYTE*				pAppData,
		const int				nAppDataLength
		) {nChid;pStreamData;nStreamDataLength;pAppData;nAppDataLength;return FALSE;};
	virtual void DPI_ServiceAttach_Rsp(
		const BYTE*				pUuDataOutBuffer, 
		const int				nUuDataOutLen,
		const RESPONSE			eResponse
		);
	virtual void DPI_ServiceDetach_Rsp(
		const RESPONSE			eResponse
		);
	virtual FlexMuxDataParam* DPI_ChannelAdd_Rsp(
		const OnChannelsAddParam& outParam
		);
	virtual FlexMuxDataParam* DPI_TransmuxAdd_Rsp(
		const OnTransmuxAddParam& outParam
		);
	virtual void DPI_ChannelDelete_Rsp(
		const OnChannelsDeleteParam& outParam
		);
	virtual void DPI_UserCommandAck_Rsp(
		const DAIUserCmdParam&	outParam
		);
///
private:
   DN_CLASS * DN_Object;
	static LPCSTR s_moduleID;
	static LPCSTR s_myUDPURLScheme;
	static LPCSTR s_myTCPURLScheme;
   ServiceSink * m_sink;
   YKSemaphore	  m_smpMode;
   HANDLE userThread;
};

BOOL checkURLValidity(LPCSTR url);


#endif
