/******************************************************************************
*  SOFTWARE LICENSE AND CONDITIONS
*
*  This software module was originally developed by Kambiz Asrar Haghighi 
*  and Yasser Pourmohammadi (University of British Columbia) in the course of 
*  development of  the streaming functionality for MPEG-4 using the DMIF
*  (ISO/IEC 14496-6) standard. This software module is an implementation 
*  of a part of one or more MPEG-4 tools as specified by MPEG-4. 
*  
*  Users of the MPEG-4 may use the software source code or modifications 
*  thereof only for hardware or software products related to MPEG-4 evaluation
*  and testing.  The use of the source code for any purpose other than 
*  evaluation and testing, including the use of derivative works for the 
*  generation of commercial products, is strictly prohibited.
*  
*  The original developer of this software module and his/her company, 
*  the subsequent editors and their companies, and ISO/IEC have no 
*  liability for use of this software module or modifications thereof 
*  in an implementation. Copyright is not released for non MPEG-4 
*  conforming products. UBC retains full right to use the code for 
*  his/her own purpose, assign or donate the code to a third party and 
*  to inhibit third parties from using the code for non MPEG-4 conforming 
*  products. This copyright notice must be included in all copies or 
*  derivative works.
*
*  Contact:
*
*  Kambiz Asrar Haghighi                 email: kambiz@interchange.ubc.ca
*  Yasser Pourmohammadi                  email: yasserp@ece.ubc.ca
*  Professor Hussein M. Alnuweiri
*  UBC Signal Processing and Multimedia Group
*  http://spmg.ece.ubc.ca/research/MPEG4
*  Vancouver BC Canada V6T1Z4                  
*
*  Copyright  2000  University of British Columbia, Canada
********************************************************************************/

// datastruct.cpp : Defines the entry point for the application.
//

#include <stdlib.h>
#include "datastr.h"



ServiceRecordClass::ServiceRecordClass()
{
   pCAT = NULL;
   pTAT = NULL;
}

ServiceRecordClass::~ServiceRecordClass()
{
   free(absURL);
   /* YK */ // delete linked list of CAT & TAT
}

int ServiceRecordClass::addCATtoList(__int16 CAT,__int8 direction, __int16 TAT)
{
   CAT_TYPE* tempCAT;
   CAT_TYPE* pnCAT = new CAT_TYPE;
   pnCAT->CAT = CAT;
   pnCAT->direction = direction;
   pnCAT->AssociatedTAT = TAT;
   pnCAT->nextCAT = NULL;
   tempCAT = this->pCAT;
   if (tempCAT == NULL)
      this->pCAT = pnCAT;
   else {
      while (tempCAT->nextCAT != NULL)
         tempCAT = tempCAT->nextCAT;
      tempCAT->nextCAT = pnCAT;
   }
   return 1;
}

CAT_TYPE* ServiceRecordClass::getCATfromList()
{
   return this->pCAT;
}

BOOL ServiceRecordClass::deleteCATfromList(__int16 cat)
{
   // Find the element
   CAT_TYPE* currentElement = this->pCAT;
   CAT_TYPE* prevElement;

   if (currentElement == NULL)
      return FALSE;
 
   if (currentElement->CAT == cat)
      pCAT = pCAT->nextCAT;
   else {
      while(currentElement != NULL && currentElement->CAT != cat)
      {
         prevElement = currentElement;
         currentElement = currentElement->nextCAT;
      }
      if (currentElement != NULL)
         prevElement->nextCAT = currentElement->nextCAT;
   }
   delete currentElement;
   return TRUE;
}

int ServiceRecordClass::addTATtoList(__int16 TAT,__int8 direction)
{
   TAT_TYPE* tempTAT;
   TAT_TYPE* pnTAT = new TAT_TYPE;
   pnTAT->TAT = TAT;
   pnTAT->direction = direction;
   pnTAT->nextTAT = NULL;
   tempTAT = this->pTAT;
   if (tempTAT == NULL)
      this->pTAT = pnTAT;
   else {
      while (tempTAT->nextTAT != NULL)
         tempTAT = tempTAT->nextTAT;
      tempTAT->nextTAT = pnTAT;
   }
   return 1;
}

TAT_TYPE* ServiceRecordClass::getTATfromList()
{
   return this->pTAT;
}

BOOL ServiceRecordClass::deleteTATfromList(__int16 tat)
{
   // Find the element
   TAT_TYPE* currentElement = this->pTAT;
   TAT_TYPE* prevElement;

   if (currentElement == NULL)
      return FALSE;
 
   if (currentElement->TAT == tat)
      pTAT = pTAT->nextTAT;
   else {
      while(currentElement != NULL && currentElement->TAT != tat)
      {
         prevElement = currentElement;
         currentElement = currentElement->nextTAT;
      }
      if (currentElement != NULL)
         prevElement->nextTAT = currentElement->nextTAT;
   }
   delete currentElement;
   return TRUE;
}


ServiceRecordClassList::ServiceRecordClassList()
{
   WSADATA WSAData;                    // Contains details of the 
   TCHAR szError[100];                 // Error message string
   // Initialize Winsocket. 
   if (WSAStartup (MAKEWORD(1,1), &WSAData) != 0) 
   {
    wsprintf (szError, TEXT("WSAStartup failed. Error: %d"), 
              WSAGetLastError ());
    MessageBox (NULL, szError, TEXT("Error"), MB_OK);
    exit (1);
   }

   ListLen=0;
   return;
}

ServiceRecordClassList::~ServiceRecordClassList()
{
   int i;
   for(i=0;i < ListLen ;i++)
         delete serviceRecord[i];
   return;
}

void ServiceRecordClassList::addNewRecordtoList(__int32 servicePtr,__int16 serviceId, long double networkSessionId, LPCSTR url)
{
   serviceRecord[ListLen] = new ServiceRecordClass;
   serviceRecord[ListLen]->serviceId = serviceId;
   serviceRecord[ListLen]->pCAT = NULL;
   serviceRecord[ListLen]->networkSessionId = networkSessionId;
   serviceRecord[ListLen]->servicePtr = servicePtr;
   serviceRecord[ListLen]->copyAbsURL(url);
   serviceRecord[ListLen]->pTAT = NULL;
   ListLen++;
   
   return;
}

BOOL ServiceRecordClassList::deleteRecordfromList(__int16 serviceId)
{
   for (int i = 0; i < ListLen; i++) 
   {
      if (serviceRecord[i]->serviceId == serviceId) {
         serviceRecord[i]->absURL = serviceRecord[ListLen-1]->absURL;
         serviceRecord[i]->networkSessionId = serviceRecord[ListLen-1]->networkSessionId;
         serviceRecord[i]->pCAT = serviceRecord[ListLen-1]->pCAT;
         serviceRecord[i]->pTAT = serviceRecord[ListLen-1]->pTAT;
         serviceRecord[i]->serviceId = serviceRecord[ListLen-1]->serviceId;
         serviceRecord[i]->servicePtr = serviceRecord[ListLen-1]->servicePtr;
         delete serviceRecord[ListLen-1];
         serviceRecord[ListLen-1] = NULL;
         ListLen--;
         return TRUE;
      }
   }
   return FALSE;
}


long double ServiceRecordClassList::getNetworkSessionId(__int32 nServicePtr)
{
   int i;
   long double networkSessionId=0;
   for(i=0;i < ListLen ;i++)
      if( serviceRecord[i]->servicePtr == nServicePtr)
         return serviceRecord[i]->networkSessionId;
   return -1;
}

__int16 ServiceRecordClassList::getServiceId(__int32 nServicePtr)
{
   int i;
   __int16 serviceId=0;
   for(i=0;i < ListLen ;i++)
      if( serviceRecord[i]->servicePtr == nServicePtr)
         return serviceRecord[i]->serviceId;
   return -1;
}


// Determine whether a record for this service has been previously created
// Creates a new serviceId if non exists
BOOL ServiceRecordClassList::serviceExists(__int32 servicePtr,__int16 *serviceId/*out*/)
{
   int i = 0;
   BOOL loop;
   
   for (i=0;i < ListLen; i++)
      if (serviceRecord[i]->servicePtr == servicePtr)
         return TRUE;
   
   do
   {
      loop = FALSE;
      srand((unsigned int)GetTickCount());
      *serviceId = rand();
      for (i=0; i<ListLen; i++)
      {
         if (*serviceId == serviceRecord[i]->serviceId)
            loop = TRUE;
      }
   }while(loop);      

      
   return FALSE;
}

BOOL ServiceRecordClassList::sessionExists(long double networkSessionId)
{
   for (int i=0; i < ListLen; i++)
   {
      if (serviceRecord[i]->networkSessionId == networkSessionId)
         return TRUE;
   }
   return FALSE;
}


// Find a networksSessionId for a given URL, if none exists, create a new
// networkSessionId
BOOL ServiceRecordClassList::findNSIDfromURL(LPCSTR url, long double *networkSessionId)
{
   int i = 0;
   BOOL loop = FALSE;

   for (i=0;i < ListLen; i++)
   {
      if (strcmp(serviceRecord[i]->absURL, url)) 
      {
         *networkSessionId = serviceRecord[i]->networkSessionId;
         return TRUE;
      }
   }
   
   // generating a new networkSessionId
   do
   {
      loop = FALSE;
      srand((unsigned int)GetTickCount());
      *networkSessionId = rand();
      for (i=0; i<ListLen; i++)
      {
         if (*networkSessionId == serviceRecord[i]->networkSessionId)
            loop = TRUE;
      }
   }while(loop);

   return FALSE;
}

// Find the CAT based on the seviceId
CAT_TYPE* ServiceRecordClassList::getCAT(__int16 nServiceId)
{
   int i;
   int CAT=0;
   for(i=0;i < ListLen ;i++)
      if( serviceRecord[i]->serviceId == nServiceId)
         return serviceRecord[i]->getCATfromList();
   return NULL;
}

void ServiceRecordClassList::deleteCAT(__int32 nServiceId, __int16 CAT)
{
   int i;
   for(i=0;i < ListLen ;i++)
      if( serviceRecord[i]->serviceId == nServiceId)
         serviceRecord[i]->deleteCATfromList(CAT);
   return;
}

void ServiceRecordClassList::putCAT(__int32 servicePtr, __int16 CAT, __int8 direction, __int16 TAT)
{
   for(int i=0;i<ListLen;i++)
   {
      if ( serviceRecord[i]->servicePtr == servicePtr)
      {
           serviceRecord[i]->addCATtoList(CAT,direction,TAT);
           break;
      }
   }
   return;
}

TAT_TYPE* ServiceRecordClassList::getTAT(__int16 nServiceId)
{
   int i;
   int TAT=0;
   for(i=0;i < ListLen ;i++)
      if( serviceRecord[i]->serviceId == nServiceId)
         return serviceRecord[i]->pTAT; // Only returns pointer to  the first element in TAT linked list.
   return NULL;
}

void ServiceRecordClassList::putTAT(__int32 servicePtr, __int16 TAT, __int8 direction)
{
   for(int i=0;i<ListLen;i++)
   {
      if ( serviceRecord[i]->servicePtr == servicePtr)
      {
           serviceRecord[i]->addTATtoList(TAT,direction);
           break;
      }
   }
   return;
}

void ServiceRecordClassList::deleteTAT(__int32 nServiceId, __int16 TAT)
{
   int i;
   for(i=0;i < ListLen ;i++)
      if( serviceRecord[i]->serviceId == nServiceId)
         serviceRecord[i]->deleteTATfromList(TAT);
   return;
}
