/******************************************************************************
*  SOFTWARE LICENSE AND CONDITIONS
*
*  This software module was originally developed by Yasser Pourmohammadi and 
*  Kambiz Asrar Haghighi (University of British Columbia) in the course of 
*  development of  the streaming functionality for MPEG-4 using the DMIF
*  (ISO/IEC 14496-6) standard. This software module is an implementation 
*  of a part of one or more MPEG-4 tools as specified by MPEG-4. 
*  
*  Users of the MPEG-4 may use the software source code or modifications 
*  thereof only for hardware or software products related to MPEG-4 evaluation
*  and testing.  The use of the source code for any purpose other than 
*  evaluation and testing, including the use of derivative works for the 
*  generation of commercial products, is strictly prohibited.
*  
*  The original developer of this software module and his/her company, 
*  the subsequent editors and their companies, and ISO/IEC have no 
*  liability for use of this software module or modifications thereof 
*  in an implementation. Copyright is not released for non MPEG-4 
*  conforming products. UBC retains full right to use the code for 
*  his/her own purpose, assign or donate the code to a third party and 
*  to inhibit third parties from using the code for non MPEG-4 conforming 
*  products. This copyright notice must be included in all copies or 
*  derivative works.
*
*  Contact:
*
*  Kambiz Asrar Haghighi                 email: kambiz@interchange.ubc.ca
*  Yasser Pourmohammadi                  email: yasserp@ece.ubc.ca
*  Professor Hussein M. Alnuweiri
*  UBC Signal Processing and Multimedia Group
*  http://spmg.ece.ubc.ca/research/MPEG4
*  Vancouver BC Canada V6T1Z4                  
*
*  Copyright  2000  University of British Columbia, Canada
********************************************************************************/

// A new class which contains the 
#ifndef _datastr_h
#define _datastr_h

#define MAX_NUM_SERVICES 10 /* YK */

#include <afxwin.h>
#include <winsock2.h>

typedef struct CAT_TYPE
{
   __int16 CAT;
   __int8 direction;
   __int16 AssociatedTAT; /* DEV */ // One of the TAT's in the following list, we could also point to
                                    // that object over here instead
   CAT_TYPE* nextCAT;
} CAT_TYPE;

typedef struct TAT_TYPE
{
   __int16 TAT;
   __int8 direction;
   TAT_TYPE* nextTAT;

} TAT_TYPE;

class ServiceRecordClass
{
public:
   ServiceRecordClass();
   ~ServiceRecordClass();
   int addCATtoList(__int16 cat,__int8 dir, __int16 TAT);
   CAT_TYPE* getCATfromList();
   BOOL deleteCATfromList(__int16 cat);
   int addTATtoList(__int16 cat,__int8 dir);
   TAT_TYPE* getTATfromList();
   BOOL deleteTATfromList(__int16 tat);

   __int32 servicePtr;
   long double networkSessionId;
   LPSTR absURL;
   CAT_TYPE* pCAT;
   TAT_TYPE* pTAT;
   __int16 serviceId;
   void copyAbsURL(LPCSTR url) {
      absURL = (char*)malloc(strlen(url));
      memcpy(absURL,url,strlen(url));
   }
};


class ServiceRecordClassList{
public:
   ServiceRecordClassList();
   //ServiceRecordList(int,long double,int,int);
   ~ServiceRecordClassList();
   long double getNetworkSessionId(__int32 servicePtr);
   __int16 getServiceId(__int32 nServicePtr);
   CAT_TYPE* getCAT(__int16);
   void putCAT(__int32 servicePtr, __int16 CAT, __int8 direction, __int16 TAT);
   void deleteCAT(__int32 serviceId, __int16 CAT);
   void putTAT(__int32 servicePtr, __int16 TAT, __int8 direction);
   TAT_TYPE * getTAT(__int16 serviceId);
   void deleteTAT(__int32 serviceId, __int16 TAT);
   void addNewRecordtoList(__int32,__int16 ,long double, LPCSTR);
   BOOL deleteRecordfromList(__int16 serviceId);
   BOOL serviceExists(__int32,__int16*);
   BOOL sessionExists(long double networkSessionId);
   BOOL findNSIDfromURL(LPCSTR url, long double *networkSessionId);
private:
   int ListLen;
   ServiceRecordClass*  serviceRecord[MAX_NUM_SERVICES];
   /* YK */ // would be useful to make a linked list to avoid
   // specifying a fixed size for the arrays & simplify deconstructor

};



#endif