/***********************************************************************
    This software module  was  originally  developed by Guido
	Franceschini (CSELT)
    in the course of development of the MPEG-4 DMIF (ISO/IEC 14496-6)
    standard.  This software module is an implementation of a part of
    one or more MPEG-4  DMIF  (ISO/IEC 14496-6)  tools  as  specified
    by  the  MPEG-4 DMIF (ISO/IEC 14496-6)  standard.  ISO/IEC  gives
    users of  the  MPEG-4  DMIF  (ISO/IEC 14496-6)  free  license  to
    this software module or modifications thereof for use in hardware
    or software products claiming  conformance  to  the  MPEG-4  DMIF
    (ISO/IEC 14496-6). Those intending to use this software module in
    hardware or software  products  are  advised  that  its  use  may
    infringe   existing  patents.  The  original  developer  of  this
    software module and his/her company, the subsequent  editors  and
    their  companies,  and  ISO/IEC have no liability for use of this
    software module or modifications thereof  in  an  implementation.
    Copyright  is not released for non MPEG-4 DMIF  (ISO/IEC 14496-6)
    conforming  products. CSELT
    retain  full  right to use the code for their own purpose, assign
    or donate the code to a third party and to inhibit third  parties
    from  using  the  code  for  non  MPEG-4  DMIF  (ISO/IEC 14496-6)
    conforming products. This copyright notice must  be  included  in
    all copies or derivative works. Copyright  1998.
************************************************************************/

// Contact: guido.franceschini@cselt.it

/**************************************************************************
 * Project Im1Player
 * INTERNATIONAL STANDARD ORGANIZATION
 * ISO/IEC JTC1/SC29/WG11
 *
 * SUBSYSTEM: DMIF
 * FILE:      sl_mpeg4.cpp
 * AUTHOR:    Guido Franceschini (CSELT)
 * OVERVIEW:  Implementation of the Sync Layer packetizer and depacketizer
 *            for the MPEG4 Sync Layer.
 *
 **************************************************************************/

#include "defs.h"
#include <assert.h>
#include "mpg4dmif.h"

//Michael Stepping
#ifdef WIN32
	// INLINE
	// some methods can be auto-inlined in _this_ file.
	#ifdef NDEBUG
	#pragma auto_inline(on)
	// warning C4711: function 'xxx' selected for automatic inline expansion
	#pragma warning( disable: 4711 )
	#endif
#endif



class SLCoder
{
public:
	SLCoder (const BYTE* pBuf, const int nSize);
	int GetByte ();
	int GetBit ();
	// SOLARIS PORTING - Core 5.9
	//int GetBits (BYTE* pBuf, int nBits);
	int GetBits (BYTE* pBuf, int nBits, int nBytes = 0);
	int GetLength () const
	{	return (m_nRead); }
	void WriteByte ();
	void WriteBit (BOOL bit);
	// SOLARIS PORTING - Core 5.9
	//void WriteBits (BYTE* pBuf, int nBits);
	void WriteBits (BYTE* pBuf, int nBits, int nBytes = 0);
	int WriteAlign ();
	void Set()
	{	m_nLastByte = GetByte ();	}

protected:
	BYTE* m_pBuf;
	int m_nSize, m_nLastBit, m_nLastByte, m_nWritten, m_nRead;
};

SLCoder :: SLCoder (const BYTE* pBuf = NULL, const int nSize = 0 ) :
	m_pBuf ((BYTE *)pBuf),
	m_nSize ((int)nSize),
	m_nLastBit (0),
	m_nLastByte (0),
	m_nWritten (0),
	m_nRead (1)
{}

int SLCoder :: GetByte ()
{
//	if (m_nSize == 0)
//		EndOfUnit ();
	return (int) *m_pBuf++;
}

int SLCoder :: GetBit ()
{
	if (m_nLastBit >= 8)
	{
	 	m_nLastByte = GetByte ();
		m_nLastBit = 0;
		m_nSize--;
		m_nRead++;
	}

	m_nLastByte <<= 1;
	++m_nLastBit;
	return (m_nLastByte & 0x100) >> 8;
}

// SOLARIS PORTING - Core 5.9
// Added the third parameter to know the number of bytes involved.
// We can safely do this because this is the only class to use
// SLCoder
//int SLCoder :: GetBits (BYTE* pBuf, int nBits)
int SLCoder :: GetBits (BYTE* pBuf, int nBits, int nBytes)
{
	int n = (64 - nBits)%8;
	// SOLARIS PORTING
	// MAde changes for to work with SPARC
	BYTE b = 0;
#if defined (WIN32) || defined (__LINUX)
	pBuf += ((nBits-1)/8);

	while (--nBits >= 0)
	{
		b <<= 1;
		b |= GetBit ();
		if (++n == 8) {
			n = 0;
			*pBuf-- = b;
			b = 0;
		}
	}
	// WIN-SOL release
	nBytes; // Avoid Compiler warning
#endif
#ifdef __SOLARIS
	pBuf += nBytes - 1 - ((nBits-1)/8);

	while (--nBits >= 0)
	{
		b <<= 1;
		b |= GetBit ();
		if (++n == 8) {
			n = 0;
			*pBuf++ = b;
			b = 0;
		}
	}
#endif
	// SOLARIS PORTING - END of changes
	return 0;
}



void SLCoder :: WriteByte ()
{
	*m_pBuf++ = m_nLastByte;
	m_nWritten ++;
}

void SLCoder :: WriteBit (BOOL bit)
{
	m_nLastByte <<= 1;
	m_nLastByte |= bit;
	if (++m_nLastBit == 8)
	{	m_nLastBit = 0;
		WriteByte ();
		m_nLastByte = 0;
	}
}

// SOLARIS PORTING - Core 5.9
// Added the third parameter to know the number of bytes involved.
// We can safely do this because this is the only class to use
// SLCoder
//void SLCoder :: WriteBits (BYTE* pBuf, int nBits)
void SLCoder :: WriteBits (BYTE* pBuf, int nBits, int nBytes)
{
	int n = (64 - nBits)%8;

#if defined (WIN32) || defined (__LINUX)
	pBuf += ((nBits-1)/8);
	BYTE b = *pBuf;

	if (n != 8)
		b <<= n;
	while (--nBits >= 0)
	{	WriteBit (b > 127);
		b <<= 1;
		if (++n == 8) {
			n = 0;
			b = *(--pBuf);
		}
	}
	// WIN-SOL release
	nBytes; // Avoid Compiler warning
#endif
#ifdef __SOLARIS
	pBuf += nBytes - 1 - ((nBits-1)/8);
	BYTE b = *pBuf;
	if (n != 8)
		b <<= n;
	while (--nBits >= 0)
	{	WriteBit (b > 127);
		b <<= 1;
		if (++n == 8) {
			n = 0;
			b = *(++pBuf);
		}
	}
#endif
}

int SLCoder :: WriteAlign ()
{
	if (m_nLastBit)
	{
		m_nLastByte <<= 8 - m_nLastBit;
		WriteByte ();
		m_nLastByte = 0;
	}
	return (m_nWritten);
}

// SOLARIS PORTING - The changes made to readBits and writeBits affect the
// follwing code
void
SL_Packetize (const struct SLConfigDesc* pConfig, const struct SLHeader* pHeader, const BYTE* pPDU, const int nPDULen, int* pnHeaderLen)
{
	SLCoder Coder(pPDU, nPDULen);
	if (pConfig -> useAccessUnitStartFlag)
		Coder.WriteBits ( (BYTE*) &(pHeader -> accessUnitStartFlag),1, sizeof(pHeader -> accessUnitStartFlag));
	if (pConfig -> useAccessUnitEndFlag)
		Coder.WriteBits ( (BYTE*) &(pHeader -> accessUnitEndFlag),1, sizeof(pHeader -> accessUnitEndFlag));
	if (pConfig -> OCRLength>0)
		Coder.WriteBits ( (BYTE*) &(pHeader -> OCRflag),1, sizeof(pHeader -> OCRflag));
	if (pConfig -> useIdleFlag)
		Coder.WriteBits ( (BYTE*) &(pHeader -> idleFlag),1, sizeof(pHeader -> idleFlag));
	if (pConfig -> usePaddingFlag)
	{
		Coder.WriteBits ( (BYTE*) &(pHeader -> paddingFlag),1, sizeof(pHeader -> paddingFlag));
		if (pHeader -> paddingFlag)
			Coder.WriteBits ( (BYTE*) &(pHeader -> paddingBits),3, sizeof(pHeader -> paddingBits));
	}
	if (!pHeader -> idleFlag && (!pHeader -> paddingFlag || pHeader -> paddingBits!=0))
	{
		if (pConfig -> packetSeqNumLength>0)
			Coder.WriteBits ( (BYTE*) &(pHeader -> packetSequenceNumber), pConfig -> packetSeqNumLength, sizeof(pHeader -> packetSequenceNumber));
		if (pConfig -> degradationPriorityLength>0)
		{	Coder.WriteBits ( (BYTE*) &(pHeader -> degradationPriorityFlag), 1, sizeof(pHeader -> degradationPriorityFlag));
			if (pHeader -> degradationPriorityFlag)
				Coder.WriteBits ( (BYTE*) &(pHeader -> degradationPriority), pConfig -> degradationPriorityLength, sizeof(pHeader -> degradationPriority));
		}
		if (pHeader -> OCRflag)
		{
//			m_nOCROffset = pCoder.GetOffset ();
			Coder.WriteBits ( (BYTE*) &(pHeader -> objectClockReference), pConfig -> OCRLength, sizeof(pHeader -> objectClockReference));
		}
		if (pHeader -> accessUnitStartFlag)
		{
			if (pConfig -> useRandomAccessPointFlag)
				Coder.WriteBits ( (BYTE*) &(pHeader -> randomAccessPointFlag),1, sizeof(pHeader -> randomAccessPointFlag));
			if (pConfig -> AU_seqNumLength > 0)
				Coder.WriteBits ( (BYTE*) &(pHeader -> AU_sequenceNumber), pConfig -> AU_seqNumLength, sizeof(pHeader -> AU_sequenceNumber));
			if (pConfig -> useTimeStampsFlag)
			{
				Coder.WriteBits ( (BYTE*) &(pHeader -> decodingTimeStampFlag),1, sizeof(pHeader -> decodingTimeStampFlag));
				Coder.WriteBits ( (BYTE*) &(pHeader -> compositionTimeStampFlag),1, sizeof(pHeader -> compositionTimeStampFlag));
			}
			if (pConfig -> instantBitrateLength>0)
				Coder.WriteBits ( (BYTE*) &(pHeader -> instantBitrateFlag),1, sizeof(pHeader -> instantBitrateFlag));
			if (pHeader -> decodingTimeStampFlag)
				Coder.WriteBits ( (BYTE*) &(pHeader -> decodingTimeStamp), pConfig -> timeStampLength, sizeof(pHeader -> decodingTimeStamp));
			if (pHeader -> compositionTimeStampFlag)
				Coder.WriteBits ( (BYTE*) &(pHeader -> compositionTimeStamp), pConfig -> timeStampLength, sizeof(pHeader -> compositionTimeStamp));
			if (pConfig -> AU_Length > 0)
				Coder.WriteBits ( (BYTE*) &(pHeader -> accessUnitLength), pConfig -> AU_Length, sizeof(pHeader -> accessUnitLength));
			if (pHeader -> instantBitrateFlag)
				Coder.WriteBits ( (BYTE*) &(pHeader -> instantBitrate), pConfig -> instantBitrateLength, sizeof(pHeader -> instantBitrate));
		}
	}
	*pnHeaderLen = Coder.WriteAlign ();
}

int
SL_Depacketize (const struct SLConfigDesc* pConfig, const struct SLHeader* pHeader, const BYTE* pPDU, const int nPDULen, int* pnHeaderLen)
{
	SLCoder Coder(pPDU, nPDULen);
	Coder.Set ();
	memset ((void *)pHeader, 0, sizeof(*pHeader));
	if (pConfig -> useAccessUnitStartFlag)
		Coder.GetBits ( (BYTE*) &(pHeader -> accessUnitStartFlag),1, sizeof(pHeader -> accessUnitStartFlag) );
	if (pConfig -> useAccessUnitEndFlag)
		Coder.GetBits ( (BYTE*) &(pHeader -> accessUnitEndFlag),1, sizeof(pHeader -> accessUnitEndFlag) );
	if ((!pConfig -> useAccessUnitStartFlag) && (!pConfig -> useAccessUnitEndFlag))
	{
		((struct SLHeader*)(pHeader)) -> accessUnitStartFlag = TRUE;
		((struct SLHeader*)(pHeader)) -> accessUnitEndFlag = TRUE;
	}
	if (pConfig -> OCRLength>0)
		Coder.GetBits ( (BYTE*) &(pHeader -> OCRflag),1, sizeof(pHeader -> OCRflag) );
	if (pConfig -> useIdleFlag)
		Coder.GetBits ( (BYTE*) &(pHeader -> idleFlag),1, sizeof(pHeader -> idleFlag) );
	if (pConfig -> usePaddingFlag)
	{
		Coder.GetBits ( (BYTE*) &(pHeader -> paddingFlag),1, sizeof(pHeader -> paddingFlag) );
		if (pHeader -> paddingFlag)
			Coder.GetBits ( (BYTE*) &(pHeader -> paddingBits),3, sizeof(pHeader -> paddingBits) );
	}
	if (!pHeader -> idleFlag && (!pHeader -> paddingFlag || pHeader -> paddingBits!=0))
	{
		if (pConfig -> packetSeqNumLength>0)
			Coder.GetBits ( (BYTE*) &(pHeader -> packetSequenceNumber),pConfig -> packetSeqNumLength, sizeof(pHeader -> packetSequenceNumber) );
		if (pConfig -> degradationPriorityLength>0)
		{	Coder.GetBits ( (BYTE*) &(pHeader -> degradationPriorityFlag), 1, sizeof(pHeader -> degradationPriorityFlag) );
			if (pHeader -> degradationPriorityFlag)
				Coder.GetBits ( (BYTE*) &(pHeader -> degradationPriority), pConfig -> degradationPriorityLength, sizeof(pHeader -> degradationPriority) );
		}
		if (pHeader -> OCRflag)
		{
//			m_nOCROffset = pCoder.GetOffset ();
			Coder.GetBits ( (BYTE*) &(pHeader -> objectClockReference), pConfig -> OCRLength, sizeof(pHeader -> objectClockReference) );
		}
		if (pHeader -> accessUnitStartFlag)
		{
			if (pConfig -> useRandomAccessPointFlag)
				Coder.GetBits ( (BYTE*) &(pHeader -> randomAccessPointFlag),1, sizeof(pHeader -> randomAccessPointFlag) );
			if (pConfig -> AU_seqNumLength > 0)
				Coder.GetBits ( (BYTE*) &(pHeader -> AU_sequenceNumber), pConfig -> AU_seqNumLength, sizeof(pHeader -> AU_sequenceNumber) );
			if (pConfig -> useTimeStampsFlag)
			{
				Coder.GetBits ( (BYTE*) &(pHeader -> decodingTimeStampFlag),1, sizeof(pHeader -> decodingTimeStampFlag) );
				Coder.GetBits ( (BYTE*) &(pHeader -> compositionTimeStampFlag),1, sizeof(pHeader -> compositionTimeStampFlag) );
			}
			if (pConfig -> instantBitrateLength>0)
				Coder.GetBits ( (BYTE*) &(pHeader -> instantBitrateFlag),1, sizeof(pHeader -> instantBitrateFlag) );
			if (pHeader -> decodingTimeStampFlag)
				Coder.GetBits ( (BYTE*) &(pHeader -> decodingTimeStamp), pConfig -> timeStampLength, sizeof(pHeader -> decodingTimeStamp) );
			if (pHeader -> compositionTimeStampFlag)
				Coder.GetBits ( (BYTE*) &(pHeader -> compositionTimeStamp), pConfig -> timeStampLength, sizeof(pHeader -> compositionTimeStamp) );
			if (pConfig -> AU_Length > 0)
				Coder.GetBits ( (BYTE*) &(pHeader -> accessUnitLength), pConfig -> AU_Length, sizeof(pHeader -> accessUnitLength) );
			if (pHeader -> instantBitrateFlag)
				Coder.GetBits ( (BYTE*) &(pHeader -> instantBitrate), pConfig -> instantBitrateLength, sizeof(pHeader -> instantBitrate) );
		}
	}
	*pnHeaderLen = Coder.GetLength ();
	return 0;
}


/*
main()
{
struct SLConfigDescriptor	slcd;
struct SL_PacketHeader		slph;
struct SL_PacketHeader		slph2;

	slcd. predefined = 0;
	slcd. useAccessUnitStartFlag = TRUE;
	slcd. useAccessUnitEndFlag = TRUE;
	slcd. useRandomAccessPointFlag = FALSE;
	slcd. hasRandomAccessUnitsOnlyFlag = TRUE;
	slcd. usePaddingFlag = FALSE;
	slcd. useTimeStampsFlag = TRUE;
	slcd. useIdleFlag = FALSE;
	slcd. durationFlag = FALSE;
	slcd. timeStampResolution = 1000;
	slcd. OCRResolution = 1000;
	slcd. timeStampLength = 15;	// must be  64
	slcd. OCRLength = 0;		// must be  64
	slcd. AU_Length = 8;		// must be  32
	slcd. instantBitrateLength = 0;	// must be ?????
	slcd. degradationPriorityLength = 0;
	slcd. AU_seqNumLength = 4; // must be  16
	slcd. packetSeqNumLength = 0; // must be  16
//	slcd. reserved;
	slcd. timeScale = 1000;
//	slcd. accessUnitDuration;
//	slcd. compositionUnitDuration;
//	slcd. startDecodingTimeStamp;
//	slcd. startCompositionTimeStamp;
//	SDLAlignedInt<1> OCRstreamFlag;
//	slcd. reserved_2;
//	SDLInt<16> OCR_ES_Id;

	slph. accessUnitStartFlag = TRUE;
	slph. accessUnitEndFlag = FALSE;
	slph. OCRflag = FALSE;
	slph. idleFlag = FALSE;
	slph. paddingFlag = FALSE;
//	slph. paddingBits;
	slph. packetSequenceNumber = 0xF00F;
//	slph. objectClockReference;
	slph. randomAccessPointFlag = TRUE;
	slph. AU_sequenceNumber = 0x11;
	slph. decodingTimeStampFlag = TRUE;
	slph. compositionTimeStampFlag = TRUE;
	slph. instantBitrateFlag = FALSE;
	slph. decodingTimeStamp = 0x1234;
	slph. compositionTimeStamp = 0x4321;
	slph. accessUnitLength = 0xff;
//	slph. instantBitrate;	// instantBitRateLength is ?????
//	slph. degradationPriority;

	int nPDULen = 1000;
	BYTE* pPDU = new (BYTE[nPDULen]);
	int nHeaderLen = 0;

	SL_Packetize (&slcd, &slph, pPDU, nPDULen, &nHeaderLen);
	SL_Depacketize (&slcd, &slph2, pPDU, nPDULen, &nHeaderLen);
}
*/
