/**************************************************************************
 * Project Im1Player
 * INTERNATIONAL STANDARD ORGANIZATION
 * ISO/IEC JTC1/SC29/WG11
 * 
 * SUBSYSTEM: DMIF
 * FILE:      DMXChannel.cpp
 * AUTHOR:    Mauro Rossi (CSELT)
 * OVERVIEW:  Implementation of DMXChannel.
 * 
 **************************************************************************/

#include "ztl.h"
#include "MList.h"
#include "FlexDemux.h"
#include "TDxI_Sink.h"
#include "mpg4dmif.h"
#include "DMXChannel.h"

extern int GetQOSSTREAMLength(QOSSTREAM);
#include <fstream.h>

DMXChannel::DMXChannel(TDxI_Sink* pSnk,UINT nChid) :
	m_pSink(pSnk),
	m_nFmxChNum(0),
	m_bPlayingChannel(true),
	m_bUseSlcd(false),
	m_sQosStream(NULL),
	m_nChId(nChid),
	m_nTotalPayloadBytes(0),
	m_nTotalFmxPackets(0)
{}

DMXChannel::~DMXChannel()
{
	if(m_sQosStream)
		delete[] (BYTE*)m_sQosStream;
}

//--------------------------------------------------------------------

void DMXChannel::DPIC_ChannelDelete_Cnf(const RESPONSE eResponse)
{
	eResponse;
	return;
}

void DMXChannel::DPIC_PacketReceived_Ind(
	const BYTE*	pStreamData,
	int			nStreamDataLength,
	const BOOL	bErr)
{
	struct SLHeader HeaderData;
	int nHeaderDataLength = 0;
	int nAppDataLength = 0;
	BYTE* pAppData = NULL;

	if(!bErr)
	{
		if (m_bUseSlcd)
		{
			SL_Depacketize	(&m_slcd, &HeaderData, pStreamData, nStreamDataLength, &nHeaderDataLength);
			pAppData = (BYTE *) &HeaderData;
			nAppDataLength = sizeof(HeaderData);
		}
		m_nTotalPayloadBytes += nStreamDataLength - nHeaderDataLength;
		m_nTotalFmxPackets ++ ;
		if(m_bPlayingChannel)
			m_pSink->TDxI_ForwardData(m_nChId, pStreamData + nHeaderDataLength, nStreamDataLength - nHeaderDataLength, pAppData, nAppDataLength, bErr);
	}
}


void DMXChannel::SetChNum(int nFmxChNum)
{
	m_nFmxChNum = nFmxChNum;
}

int DMXChannel::GetChNum()
{
	return m_nFmxChNum;
}

void DMXChannel::SetChId(UINT nChid)
{
	m_nChId = nChid;
}

UINT DMXChannel::GetChId()
{
	return m_nChId;
}

unsigned __int64 DMXChannel :: GetTotalPayload() const
{
	return m_nTotalPayloadBytes;
}

DWORD DMXChannel :: GetTotalFmxPackets() const
{
	return m_nTotalFmxPackets;
}

/*-------------------------------------------------------------
//Copies the QosStream.
-------------------------------------------------------------*/
int DMXChannel::SetQos(QOSSTREAM sQos)
{
	int length = GetQOSSTREAMLength(sQos);
	if(length > 0)
	{
		m_sQosStream = new BYTE[length];
		memcpy((char*)m_sQosStream,(char*)sQos,length);
		return 0;
	}
	else
	{
		m_sQosStream = (QOSSTREAM)NULL;
		return 1;
	}
}

/*-------------------------------------------------------------
//Gets a pointer to the Qos.
-------------------------------------------------------------*/
QOSSTREAM DMXChannel::GetQos() const
{
	return m_sQosStream;
}

/*-------------------------------------------------------------
Enables the channel
-------------------------------------------------------------*/
void DMXChannel::PlayChannel()
{
	m_bPlayingChannel = true;
}

/*-------------------------------------------------------------
Disables the channel
-------------------------------------------------------------*/
void DMXChannel::PauseChannel()
{
	m_bPlayingChannel = false;
}

