/**************************************************************************
 * Project Im1Player
 * INTERNATIONAL STANDARD ORGANIZATION
 * ISO/IEC JTC1/SC29/WG11
 * 
 * SUBSYSTEM: DMIF
 * FILE:      FilterChannel.cpp
 * AUTHOR:    Mauro Rossi (CSELT)
 * OVERVIEW:  Implementation of FilterChannel.
 * 
 **************************************************************************/

#include "porting.h"
#include "ztl.h"
#include "MList.h"
#include "Transmux.h"
#include "FlexDemux.h"
#include "TDxI_Sink.h"
#include "mpg4dmif.h"
#include "FilterChannel.h"

extern int GetQOSSTREAMLength(QOSSTREAM);
#include <fstream.h>

FilterChannel::FilterChannel(TDxI_Sink* pSnk,UINT nChid) :
	m_pSink(pSnk),
	m_nFmxChNum(0),
	m_bPlayingChannel(true),
	m_bUseSlcd(false),
	m_sQosStream(NULL),
	m_nChId(nChid)
{}

FilterChannel::~FilterChannel()
{
	if(m_sQosStream)
		delete[] (BYTE*)m_sQosStream;
}

//--------------------------------------------------------------------

void FilterChannel::DPIC_ChannelDelete_Cnf(const RESPONSE eResponse)
{
	eResponse;
	return;
}

void FilterChannel::DPIC_PacketReceived_Ind(
	const BYTE*	pStreamData,
	int			nStreamDataLength,
	const BOOL	bErr)
{
	struct SLHeader HeaderData;
	int nHeaderDataLength = 0;
	int nAppDataLength = 0;
	BYTE* pAppData = NULL;

	if (m_bUseSlcd)
	{
		SL_Depacketize	(&m_slcd, &HeaderData, pStreamData, nStreamDataLength, &nHeaderDataLength);
		pAppData = (BYTE *) &HeaderData;
		nAppDataLength = sizeof(HeaderData);
	}
	if(m_bPlayingChannel)
		m_pSink->TDxI_ForwardData(m_nChId, pStreamData + nHeaderDataLength, nStreamDataLength - nHeaderDataLength, pAppData, nAppDataLength, bErr);
}


void FilterChannel::SetChNum(int nFmxChNum)
{
	m_nFmxChNum = nFmxChNum;
}

int FilterChannel::GetChNum()
{
	return m_nFmxChNum;
}

void FilterChannel::SetChId(UINT nChid)
{
	m_nChId = nChid;
}

UINT FilterChannel::GetChId()
{
	return m_nChId;
}


/*-------------------------------------------------------------
//Copies the QosStream.
-------------------------------------------------------------*/
int FilterChannel::SetQos(QOSSTREAM sQos)
{
	int length = GetQOSSTREAMLength(sQos);
	if(length > 0)
	{
		m_sQosStream = new BYTE[length];
		memcpy((char*)m_sQosStream,(char*)sQos,length);
		return 0;
	}
	else
	{
		m_sQosStream = (QOSSTREAM)NULL;
		return 1;
	}
}

/*-------------------------------------------------------------
//Gets a pointer to the Qos.
-------------------------------------------------------------*/
QOSSTREAM FilterChannel::GetQos() const
{
	return m_sQosStream;
}

/*-------------------------------------------------------------
Enables the channel
-------------------------------------------------------------*/
void FilterChannel::PlayChannel()
{
	m_bPlayingChannel = true;
}

/*-------------------------------------------------------------
Disables the channel
-------------------------------------------------------------*/
void FilterChannel::PauseChannel()
{
	m_bPlayingChannel = false;
}

