/*------------------------------------------------------------------
	 2000 CSELT - Centro Studi e Laboratori Telecomunicazioni.
	All rights reserved.
------------------------------------------------------------------*/

typedef const unsigned char *LPCBYTE;
typedef LPCBYTE QOSSTREAM ;

#include <windows.h>
#include <iostream.h>

///////////////////////////////////////////
/// UTILITY FUNCTIONs
///////////////////////////////////////////

int GetQOSSTREAMLength(QOSSTREAM qos)
{
	int ret;

	if( qos == NULL)
	{
		cout<<"qos NULL"<<endl;
		return 0;
	}

	if( qos[0] != 0xC )
	{
		cout<<"qos[0] != 0xC"<<endl;
		return -1;  // It is not a QOS descriptor;
	}

	if( qos[1] & 0x80 ) 
	{
		cout<<"qos TOO long"<<endl;
		return -1; // Qos length greater than 127+2 bytes. Not yet supported.
	}

	ret = 2 + (int)( qos[1] & 0x7F );

	return ret;
}

void PrintQOSSTREAMString(QOSSTREAM qos)
{
	int length = GetQOSSTREAMLength(qos);
	int i;

	if(qos==NULL)
	{
		cout<<" EMPTY (QOS == NULL)"<<endl;
		return;
	}

	if(length == -1)
	{
		cout<<" EMPTY (NOT QOS DESCRIPTOR)"<<endl;
		return;
	}
	if(length==0)
	{
		cout<<" EMPTY (LENGTH == 0)"<<endl;
		return;
	}
	cout<<hex;

	for(i=0;i<length;i++) // qos[0] and qos[1] = 2bytes
	{
		cout<<(unsigned int) (qos[i])<<".";
	}
	cout<<dec<<endl;
	for(i=3;i<length;i++) // qos[0] and qos[1] = 2bytes
	{
/*		cout << i<< endl;;
		cout << "qos = " << (unsigned int) qos[i] << endl;*/
		switch (qos[i])
		{
		case 0x01:{
				cout << "MAX_DELAY = ";
				union vv {
					BYTE		 bvalue[4];
					unsigned int ivalue;
				}v;
				v.bvalue[0] = 0x00;
				v.ivalue = 0;
				int l = qos[++i];
				for (int t = 0; t<l; t++)
					v.bvalue[l-1-t] = qos[i+1+t];
				i += l;
				cout << "*"<< (unsigned int )v.ivalue << "*"<< endl;
/*				for (t = 0; t<4; t++)
					cout << (unsigned int )v.bvalue[t] << "-";*/
				break;}
		case 0x02:{
				cout << "AVG_DELAY = ";
				union vv {
					BYTE		 bvalue[4];
					unsigned int ivalue;
				}v;
				v.bvalue[0] = 0x00;
				v.ivalue = 0;
				int l = qos[++i];
				for (int t = 0; t<l; t++)
					v.bvalue[l-1-t] = qos[i+1+t];
				i += l;
				cout << "*"<< (unsigned int )v.ivalue << "*"<< endl;
				break;}
		case 0x03:{
				cout << "LOSS_PROB = ";
				union vv {
					BYTE	 bvalue[4];
					float	 ivalue;
				}v;
				v.bvalue[0] = 0x00;
				v.ivalue = 0;
				int l = qos[++i];
				for (int t = 0; t<l; t++)
					v.bvalue[l-1-t] = qos[i+1+t];
				i += l;
				cout << "*"<< (double )v.ivalue << "*"<< endl;
				break;}
		case 0x04:{
				cout << "MAX_GAP_LOSS = ";
				union vv {
					BYTE		 bvalue[4];
					unsigned int ivalue;
				}v;
				v.bvalue[0] = 0x00;
				v.ivalue = 0;
				int l = qos[++i];
				for (int t = 0; t<l; t++)
					v.bvalue[l-1-t] = qos[i+1+t];
				i += l;
				cout << "*"<< (unsigned int )v.ivalue << "*"<< endl;
				break;}
		case 0x41:{
				cout << "MAX_AU_SIZE = ";
				union vv {
					BYTE		 bvalue[4];
					unsigned int ivalue;
				}v;
				v.bvalue[0] = 0x00;
				v.ivalue = 0;
				int l = qos[++i];
				for (int t = 0; t<l; t++)
					v.bvalue[l-1-t] = qos[i+1+t];
				i += l;
				cout << "*"<< (unsigned int )v.ivalue << "*"<< endl;
				break;}
		case 0x42:{
				cout << "AVG_AU_SIZE = ";
				union vv {
					BYTE		 bvalue[4];
					unsigned int ivalue;
				}v;
				v.bvalue[0] = 0x00;
				v.ivalue = 0;
				int l = qos[++i];
				for (int t = 0; t<l; t++)
					v.bvalue[l-1-t] = qos[i+1+t];
				i += l;
				cout << "*"<< (unsigned int )v.ivalue << "*"<< endl;
				break;}
		case 0x43:{
				cout << "MAX_AU_RATE = ";
				union vv {
					BYTE		 bvalue[4];
					unsigned int ivalue;
				}v;
				v.bvalue[0] = 0x00;
				v.ivalue = 0;
				int l = qos[++i];
				for (int t = 0; t<l; t++)
					v.bvalue[l-1-t] = qos[i+1+t];
				i += l;
				cout << "*"<< (unsigned int )v.ivalue << "*"<< endl;
				break;}
		default:{
				cout << "UNKNOWN QUALIFIER " << endl;
				int l = qos[++i];
				i = i+l;
				break;}
		}
	}
}


