/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Decoder
FILE:       Decoder.cpp
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Implementation of Decoder - base class for all decoders.
-------------------------------------------------------------------------*/
#include "defs.h"
#include "DecoderImp.h"
#include "MediaStream.h"

#ifdef MPGJ

#include "../core/ODImp.h"			// MPEGJ MERGE - RLS
#include "../core/Inline.h"			// MPEGJ MERGE - RLS
#include "../mpegj-player/MPDecoderInterface.h"	// MPEGJ MERGE - RLS

static int isStopped;		// MPEGJ MERGE - RLS
#endif

/*-----------------------------------------------------------------
Decoder constructor
-----------------------------------------------------------------*/
DecoderImp :: DecoderImp () :
// PORTING to CC 5.3
// Passing the address of the function
//	m_decoderThread ( DecoderThread, this)
	m_decoderThread ( & DecoderImp :: DecoderThread, this)
{
	m_pOutStream = m_pInStream = NULL;
	m_bActive = false;
	#ifdef MPGJ
	// MPEGJ MERGE - RLS
	// load the function pointer table to throw events - PA
	di = MPJGetTerminalDispatchTable();
	isStopped = 0;
	// MPEGJ MERGE - RLS
	#endif
}

/*-----------------------------------------------------------------
Decoder destructor
-----------------------------------------------------------------*/
DecoderImp :: ~DecoderImp ()
{
	/*-----------------------------------------------------------------
	Make sure the decoder thread terminates before the object is deleted.
	-----------------------------------------------------------------*/
	ASSERT (m_decoderThread.GetThread () == NULL);
}

/*-----------------------------------------------------------------
The default implementation of Stop() aborts any "wait for time" access
on the input stream and signals the decoder thread to stop.
-----------------------------------------------------------------*/
void DecoderImp :: Stop ()
{
	m_bActive = false;

	if (m_pInStream)
	{	m_pInStream -> Abort ();
		m_pInStream -> Terminate ();
	}

	if (m_pOutStream) {
		m_pOutStream -> Close ();
	}

	WaitForThread ();
	ASSERT (m_decoderThread.GetThread () == NULL);
}

#ifdef MPGJ
//*** YT START (117)
// pause and resume decoder
// 1. pause or resume input and output buffer
// 2. if going resume state - start decoder thread
void DecoderImp :: Pause ()
{
	m_bActive = FALSE;

	if(m_pInStream)
		m_pInStream->HalfClose(TRUE);
	if(m_pOutStream)
		m_pOutStream->Pause(TRUE);
}

void DecoderImp :: Resume ()
{
	m_bActive = TRUE;

	if(m_pInStream)
		m_pInStream->HalfClose(FALSE);
	if(m_pOutStream)
		m_pOutStream->Pause(FALSE);

	if (StartDecoderThread () == -1)
		throw;
}


//*** YT END (117)
#endif MPGJ
/*-----------------------------------------------------------------
Start() resets the input stream and open both input and output stream
thread.
-----------------------------------------------------------------*/
void DecoderImp :: Start ()
{
	#ifdef MPGJ
	isStopped = 0; //PA
	#endif

	ASSERT (m_decoderThread.GetThread () == NULL);
	m_bActive = true;
	if (m_pInStream)
		m_pInStream -> Start ();
	if (m_pOutStream)
		m_pOutStream -> Open ();
	if (StartDecoderThread () == -1) {
		throw;
	}
}

/*-----------------------------------------------------------------
Run() retrieves units from the input stream, decode them, and dispatch
units to the output stream, till the input stream's end is exceeded.
-----------------------------------------------------------------*/
void DecoderImp :: Run ()
{
	LPBYTE pInData, pOutData;
	int nInSize, nOutSize;
	DWORD dwTime, prevTime;
	BOOL bEOU;


	#ifdef MPGJ
	// MPEGJ MERGE_3 - RLS
	// to get the decoder OD ID to throw the event on to the corresponding decoder on java side - PA
    InlineImp* pInline;
	Executive *myExecutive;
	Decoder* dec;
	myExecutive = GetExecutive();
	pInline = myExecutive->GetInline();
	int len = pInline->m_streamAssociation.GetLength();
	//*** YT START (118)
	int dec_found = 0;

	// associate the decoder with ESID also, not only OD
	for( int i=0; i < len; i++ ) {
		for(int j = pInline->m_streamAssociation[i].m_pOD->esDescr.GetLength()-1;j>=0;j--) {
			dec =  ((ES_DescriptorImp *)(pInline->m_streamAssociation[i].m_pOD->esDescr[j]))->GetDecoder();
			if( this == dec ) {
				ESID = pInline->m_streamAssociation[i].m_pOD->esDescr[j]->ES_ID;
				ODID = pInline->m_streamAssociation[i].m_pOD->ObjectDescriptorID;
				dec_found = 1;
				break;
			}
		}
		if( dec_found )
		{
			break;
		}
	}
	//*** YT END (118)

     /**** get the instance of the function pointer table. - PA ****/
    di = MPJGetTerminalDispatchTable();
    // set the status variable
   	if( di != NULL ) {
    	//STREAM_START
	    // go till end of list and add the new event there
	    decoderQueue* temp;
		temp = &(di->dEQ);
		if( *temp == NULL )
			//*** YT START (119)
			// (*temp) = new decoderEventQueue( 4, ODID);
			(*temp) = new decoderEventQueue( 4, ODID, ESID);
			//*** YT END (119)
		else {
			while ( (*temp)->next != NULL )
				*temp = (*temp)->next;
			//*** YT START (119)
			// (*temp) = new decoderEventQueue( 4, ODID);
			(*temp)->next = new decoderEventQueue( 4, ODID, ESID);
			//*** YT END (119)
		}

	    if( di->isStartedDispatching == 1) {
		   // signal the thread to throw the event
// SOLARIS PORTING
#ifdef WIN32
		   	count = ResumeThread(di->hThrow);
#else
			count = thr_continue( *(thread_t *)(di->hThrow) );
#endif
		}
	}
   	/****** end of thread resume code ****/
	//END MPEGJ MERGE_3 - RLS
	#endif

	prevTime = 0;
	while (m_bActive) {
		pInData = (LPBYTE) m_pInStream -> FetchNext (nInSize, dwTime, bEOU, TRUE);
		assert(pInData);
		if (pInData == END_OF_STREAM) break;

		if (m_bActive) {
			nOutSize = GetOptimalOutputSize ();


			pOutData = (LPBYTE) m_pOutStream -> Allocate (nOutSize, TRUE, FALSE);
			if (pOutData == NULL) {
				throw;
			}
			/*output has been closed while we were waiting for data, just return*/			
			if (pOutData == END_OF_STREAM) {
				return;
			}
			while (stop == 1 || !Decode (pInData, nInSize, pOutData, nOutSize))
			{	pOutData = (LPBYTE) m_pOutStream -> Reallocate (nOutSize);

				if (pOutData == NULL) {
					throw;
				}
				if (pOutData == END_OF_STREAM)
					return;
// WIN - SOL Release - Stop the CPU consumption
#ifdef WIN32
				if(stop == 1)
					Sleep( 100 );
#endif
			}
			/*if we have the same time we're in a seek phase (cf file reader), don't dispatch
			note this shall be modified for scalable codecs
			*/
			if (dwTime==prevTime) nOutSize = 0;
			/*also try to drop frames depending on clock speed / renderer framerate so that we don't fill 
			the composition buffer for nothing*/
			else if (prevTime && CanSkipOutput()) {
				if (m_pInStream->CompareTime(dwTime, prevTime) <= 0) {
					nOutSize = 0;
				}
			}
			if (m_bActive) m_pOutStream -> Dispatch (nOutSize, dwTime);
			if (nOutSize) prevTime = dwTime;
		}
		m_pInStream -> Release (TRUE);
	}
	m_pOutStream -> Close ();
	#ifdef MPGJ
// MPEGJ MERGE_7 - RLS
	/***** THROW STREAM END EVENT - PA */
	// set the status varible
	di = MPJGetTerminalDispatchTable();
	// go till end of list and add the new event there
	decoderQueue* temp;
	temp = &(di->dEQ);
	if( *temp == NULL )
		//*** YT START (119)
		//(*temp) = new decoderEventQueue( 5, ODID);
		(*temp) = new decoderEventQueue( 5, ODID, ESID);
		//*** YT END (119)
	else {
		while ( (*temp)->next != NULL )
			*temp = (*temp)->next;
		//*** YT START (119)
		//(*temp)->next = new decoderEventQueue( 5, ODID);
		(*temp)->next = new decoderEventQueue( 5, ODID, ESID);
		//*** YT END (119)
	}
	if( di->isStartedDispatching == 1) {
		// signal the thread to throw the event
		if ( di->isJVMStarted == 1 ) {
			count = 0;
// SOLARIS PORTING
#ifdef WIN32
			count = ResumeThread(di->hThrow);
#else
			count = thr_continue( *(thread_t *)(di->hThrow) );
#endif
		}
	}

	/***** end of thread resume code  */
	//END  MPEGJ MERGE_7 - RLS
	#endif //MPGJ
}

/*-----------------------------------------------------------------
The Decoder thread function calls the virtual Run() function and
release the flag semaphore when finishes.
-----------------------------------------------------------------*/
void DecoderImp :: DecoderThread ()
{
	if (Init ())
		Run ();

	Terminate ();
}

/*-----------------------------------------------------------------
Default implementation of SetFormat, picks up the first supplied
format and returns some default values.
-----------------------------------------------------------------*/
BOOL DecoderImp :: SetFormat (
	Capabilities &formatOptions,
	DecoderParams &decoderParams
)
{
	if (formatOptions.GetFormatCapabilityLength () == 0)
		return FALSE;
	decoderParams.m_nStreamNumber = 0;
	decoderParams.m_format = formatOptions.GetFormatCapability (0);
	decoderParams.m_nOptimalAUSize = 256;
	decoderParams.m_nDBSize = 1024;
	decoderParams.m_nCBSize = 1024;
	decoderParams.m_bRequiresFullAUs = TRUE;
	return TRUE;
}
