

/*------------------------------------------------------------------------------*/
/*										*/
/*										*/
/*  This software module was originally developed by                            */
/*        Prem Kalra, MIRALab, University of Geneva                             */
/*  and edited by                                                               */
/*        Igor Pandzic, MIRALab, University of Geneva                           */
/*        Marc Escher, MIRALab, University of Geneva                            */
/*        Pierre Beylot, MIRALab, University of Geneva                          */
/*  in the course of development of the MPEG-4 Video (ISO/IEC 14496-2).	        */
/*  This software module is an implementation of a part of one or more	        */
/*  MPEG-4 Video (ISO/IEC 14496-2) tools as specified by the MPEG-4 Video       */
/*  (ISO/IEC 14496-2). ISO/IEC gives users of the                               */
/*  MPEG-4 Video (ISO/IEC 14496-2) free license to this software module or	*/
/*  modifications thereof for use in hardware or software products claiming 	*/
/*  conformance to the MPEG-4 Video (ISO/IEC 14496-2). Those intending to use	*/
/*  this software module in hardware or software products are advised that	*/
/*  its use may infringe existing patents. The original developer of this	*/
/*  software module and his/her company, the subsequent editors and their	*/
/*  companies, and ISO/IEC have no liability for use of this software		*/
/*  module or modifications thereof. Copyright is not released for non		*/
/*  MPEG-4 Video (ISO/IEC 14496-2) conforming products.                         */
/*  MIRALab, University of Geneva retains full right to use the code for its    */
/*  own purpose, assign or donate the code to a third party and to inhibit      */
/*  third parties from using the code for non MPEG-4 Video (ISO/IEC 14496-2)    */
/*  conforming products. This copyright notice must be included in all copies   */
/*  or derivative works. Copyright (c) (1997).			      		*/
/*										*/
/*										*/
/*------------------------------------------------------------------------------*/




#include "DefMan.h"
#include "assert.h"

#define SURFACE_MAX_COORD  10000.0
#define SURFACE_MIN_COORD  -10000.0
#define BOX_MARGIN 15.0

#pragma warning(disable: 4131)  // uses old-style declarator


/*---------------------------------------------------------*/
/* Procedures to get the max min values for x, y, z        */
/*                                                         */

float
SMGD_Box_get_maxcoord(the_surface,index)

SM_Surface_ptr the_surface;
int index;

/*
**   index = X_INDEX -> max_x
**   index = Y_INDEX -> max_y
**   index = Z_INDEX -> max_z
*/

{
  float max_coord;

  max_coord = SURFACE_MIN_COORD;
  SM_FOR_EACH_VERTEX(the_surface)
    max_coord = (max_coord > vertex_ptr->point[index]) ? max_coord : vertex_ptr->point[index];
  SM_END_FOR_EACH

  return(max_coord); 
} 
  

float
SMGD_Box_get_mincoord(the_surface,index)

SM_Surface_ptr the_surface;
int index;

/*
**   index = X_INDEX -> min_x
**   index = Y_INDEX -> min_y
**   index = Z_INDEX -> min_z
*/

{
  float min_coord;

  min_coord = SURFACE_MAX_COORD;
  SM_FOR_EACH_VERTEX(the_surface)
    min_coord = (min_coord < vertex_ptr->point[index]) ? min_coord : vertex_ptr->point[index];
  SM_END_FOR_EACH
 
  return(min_coord); 
} 

/*------------------------------------------------------------*/

void
SMGD_Box_set_vertex(the_box,index,point)

SM_Surface_Controlbox_ptr the_box;
int index;
VECTOR_3D point;

{
  int i;

  i = index;
  
  V3D_COPY(the_box->vertex_list[i].point,point);
}

/*------------------------------------------------------------*/

void
SMGD_Box_get_vertex(the_box,index,point)

SM_Surface_Controlbox_ptr the_box;
int index;
VECTOR_3D point;

{
  int i;

  i = index;
  V3D_COPY(point,the_box->vertex_list[i].point);
  
}

/*----------------------------------------------------------*/


/*---------------------------------------------------------*/

void
SMGD_Box_find_quadrilateral(center,delta,j,p1,p2,p3,p4)

VECTOR_3D center;
float delta;
int j;
VECTOR_3D p1,p2,p3,p4;

{

int i1,i2;

      switch (j) {
        case 0 : 
                i1 = Y_INDEX;
                i2 = Z_INDEX;
               break;
        case 1 : 
                i1 = Z_INDEX;
                i2 = X_INDEX;
               break;
        case 2 : 
                i1 = X_INDEX;
                i2 = Y_INDEX;
               break;
        default : break;
      }
        V3D_COPY(p1,center);
        p1[i1] = center[i1] - delta/2.0;
        p1[i2] = center[i2] - delta/2.0;
        V3D_COPY(p2,p1);
        p2[i1] = p1[i1] + delta;
        V3D_COPY(p3,p2);
        p3[i2] = p2[i2] + delta;
        V3D_COPY(p4,p3);
        p4[i1] = p3[i1] - delta;
}

/*-----------------------------------------------------------------*/

void
SMGD_box_maxmin_point_get(the_box,max_point,min_point) 
SM_Surface_Controlbox_ptr the_box;
VECTOR_3D max_point,min_point;
{
  int i,j,k,ind,index;

  for (ind=X_INDEX; ind<=Z_INDEX; ind++) {
    min_point[ind] = SURFACE_MAX_COORD;
    max_point[ind] = SURFACE_MIN_COORD;
  } 
  index = 0;
  for (k=0; k<the_box->n; k++) {
    for (j=0; j<the_box->m; j++) {
      for (i=0; i<the_box->l; i++) {
        for (ind=X_INDEX; ind<=Z_INDEX; ind++) {
         min_point[ind] = 
                  (min_point[ind] < the_box->vertex_list[index].point[ind]) ?
                   min_point[ind] : the_box->vertex_list[index].point[ind];
         max_point[ind] = 
                  (max_point[ind] > the_box->vertex_list[index].point[ind]) ?
                    max_point[ind] : the_box->vertex_list[index].point[ind];
        }
        index++;
       }
      }
    }
}

/*-------------------------------------------------------------------*/

void
SMGD_controlbox_vertex_indices_get(the_box,the_index,index_i,index_j,index_k)
SM_Surface_Controlbox_ptr the_box;
int the_index, *index_i, *index_j, *index_k;

{
      int l,m,n;
      int num, new_num;

      l = the_box->l; m = the_box->m; n = the_box->n; 

      num = the_index/(l*m);
      if (num > 0) { 
        *index_k = num;
        num = (int) fmod((double) the_index, (double) (l*m)); 
      }
      else {
        *index_k = 0;
        num = the_index;
      }

      new_num = num/l;
      if (new_num > 0) { 
        *index_j = new_num;
        *index_i = (int) fmod((double) num, (double) l); 
      }
      else {
        *index_j = 0;
        *index_i = (int) fmod((double) num, (double) l); 
      }
}

/*-----------------------------------------------------------------*/

void
SMGD_surface_controlbox_recreate(box_ptr1,box_ptr2,l,m,n)
SM_Surface_Controlbox_ptr *box_ptr1,*box_ptr2;

int l,m,n;
{
    int number_of_points;
       
    SM_Surface_Controlbox_ptr the_box,f_box;    

 
    the_box = *box_ptr1;
    f_box = *box_ptr2;

    number_of_points = l * m * n;
       
    if (the_box != NULL) {
     free(the_box);
     the_box = (SM_Surface_Controlbox_ptr) 
                              calloc(1, sizeof(SM_Surface_Controlbox));
     the_box->vertex_list = (SM_Surface_Controlbox_vertex_ptr) 
          calloc(number_of_points, sizeof(SM_Surface_Controlbox_vertex));

     the_box->l = l;
     the_box->m = m;
     the_box->n = n;
     the_box->vertex_list_size = number_of_points;
    }
    
    if (f_box != NULL) {
     free(f_box);
     f_box = (SM_Surface_Controlbox_ptr) 
                    calloc(1, sizeof(SM_Surface_Controlbox));
     f_box->vertex_list = (SM_Surface_Controlbox_vertex_ptr) 
          calloc(number_of_points, sizeof(SM_Surface_Controlbox_vertex));

     f_box->l = l;
     f_box->m = m;
     f_box->n = n;
     f_box->vertex_list_size = number_of_points;
    }
    
    *box_ptr1 = the_box;
    *box_ptr2 = f_box;

 
}
       
/*----------------------------------------------------------------------*/
/*
**  The procedure for creating the controlbox
*/

void
SMGD_surface_box_create(the_surface,the_box,f_box,l,m,n) 

SM_Surface_ptr the_surface;
SM_Surface_Controlbox_ptr the_box, f_box;
int l,m,n;
{

  VECTOR_3D point1, point2, difference;
  int i,j,k,index;
  float dx, dy, dz;


  SMGD_surface_controlbox_recreate(&the_box,&f_box,l,m,n);
  for (i=X_INDEX; i<=Z_INDEX; i++) {
   point1[i] = SMGD_Box_get_mincoord(the_surface,i) - BOX_MARGIN/10.0;
   point2[i] = SMGD_Box_get_maxcoord(the_surface,i) + BOX_MARGIN/10.0;
   difference[i] = point2[i] - point1[i];
  }
  dx = difference[X_INDEX]/(the_box->l - 1);
  dy = difference[Y_INDEX]/(the_box->m - 1);
  dz = difference[Z_INDEX]/(the_box->n - 1);
  index = 0;
  for (k=0; k<the_box->n; k++) {
    for (j=0; j<the_box->m; j++) {
      for (i=0; i<the_box->l; i++) {
         point2[X_INDEX] = point1[X_INDEX] + i*dx;
         point2[Y_INDEX] = point1[Y_INDEX] + j*dy;
         point2[Z_INDEX] = point1[Z_INDEX] + k*dz;
         the_box->vertex_list[index].weight = 1.0;
         the_box->vertex_list[index].i = i;
         the_box->vertex_list[index].j = j;
         the_box->vertex_list[index].k = k;
         V3D_COPY(the_box->vertex_list[index].point,point2); 
         index++;
      }
    }
  }

 }

/*-----------------------------------------------------------------*/

void
SMGD_region_box_create(the_surface,the_region_list,region_current_name,
                       the_region_controlbox,the_region_forcebox,l,m,n)
SM_Surface_ptr the_surface;
SM_Region_List_ptr the_region_list;
SM_Region_Name region_current_name;
SM_Region_Controlbox_ptr the_region_controlbox,the_region_forcebox;

{
    int the_index;

    the_index = SM_region_index_get(the_region_list,region_current_name);
    if (the_index != SM_INDEX_ERROR) {

     SM_region_controlbox_recreate(the_region_controlbox,l,m,n);
     SM_region_controlbox_recreate(the_region_forcebox,l,m,n);
     SM_region_controlbox_get(the_surface,the_region_list,
                              the_index,the_region_controlbox); 
/*
     SM_region_controlcylinder_recreate(the_region_controlbox,l,m,n);
     SM_region_controlcylinder_recreate(the_region_forcebox,l,m,n);
     SM_region_controlcylinder_get(the_surface,the_region_list,
                              the_index,the_region_controlbox); 
*/

    }
}


/*--------------------------------------------------------------*/
BOOL
SMGD_control_cylinder_index_get(the_cylinder,i,j,k,index)
SM_Region_Controlcylinder_ptr the_cylinder;
short i,j,k;
int *index;

{
    SM_FOR_CONTROL_CYLINDER(the_cylinder)
      if ( (i == cylinder_vertex_ptr->i) && 
           (j == cylinder_vertex_ptr->j) && 
           (k == cylinder_vertex_ptr->k) ) {
        *index = cylinder_vertex_index;
        return(YES);
      }
    SM_END_FOR_EACH
    return(NO);
}


void
SMGD_controlbox_origin_get(the_controlbox,origin)
SM_Surface_Controlbox_ptr the_controlbox;
VECTOR_3D origin;
{
  VECTOR_3D max_point,min_point;
  int i;

     SMGD_box_maxmin_point_get(the_controlbox,max_point,min_point);
      for (i=X_INDEX; i<=Z_INDEX; i++) 
        origin[i] = (max_point[i] + min_point[i])/2.0;

}
/*---------------------------------------------------------*/
/* procedure for scaling the object                        */

void
SMGD_scale_point(old_point,s_vect,new_point) 
VECTOR_3D old_point,s_vect,new_point;

{
  int i;

      for(i=X_INDEX; i<=Z_INDEX; i++)
        new_point[i] = old_point[i] * s_vect[i];
}

/**
**/

void
SMGD_scale_surface(the_surface,s_vect)

SM_Surface_ptr the_surface;
VECTOR_3D s_vect;

{
   VECTOR_3D old_point,new_point;

   SM_FOR_EACH_VERTEX(the_surface)
      SM_vertex_point_get(the_surface,vertex_index,old_point);
      SMGD_scale_point(old_point,s_vect,new_point);
      SM_vertex_point_set(the_surface,vertex_index,new_point);
   SM_END_FOR_EACH
}


void
SMGD_scale_face_region(the_surface,the_face,the_region_list,current_region,s_vect)
SM_Surface_ptr the_surface, the_face;
SM_Region_List_ptr the_region_list;
SM_Region_Name current_region;
VECTOR_3D s_vect;

{
   SM_Region_ptr the_region;
   SM_Region_Controlbox_ptr the_box;
   int the_index,v_index,l,m,n;
   VECTOR_3D old_point,new_point,origin_point;
   BOOL anchor;

    the_index = SM_region_index_get(the_region_list,current_region); 
    if (the_index != SM_INDEX_ERROR) {
     l = 2; m = 2; n = 2;
     the_box = SM_region_controlbox_new(l,m,n);
     SM_region_controlbox_recreate(the_box,l,m,n); 
     SM_region_controlbox_get(the_face,the_region_list,the_index,the_box);
     SMGD_controlbox_origin_get(the_box,origin_point);
     the_region = &the_region_list->region_list[the_index];
     SM_FOR_EACH_REGION_VERTEX(the_region) 
      v_index = region_vertex_ptr->id;
      anchor = region_vertex_ptr->anchor;
      if ((SM_vertex_index_valid(the_face,v_index)) && (!anchor) ) {
        SM_vertex_point_get(the_face,v_index,old_point);
        V3D_SUB(old_point,old_point,origin_point);
        SMGD_scale_point(old_point,s_vect,new_point);
        V3D_ADD(new_point,new_point,origin_point);
        SM_vertex_point_set(the_surface,v_index,new_point);
      }
     SM_END_FOR_EACH
     SM_region_controlbox_dispose(the_box);
    }
}

/** 
 * 
 */


void
SMGD_scale_region(the_surface,the_region_list,current_region,s_vect)
SM_Surface_ptr the_surface;
SM_Region_List_ptr the_region_list;
SM_Region_Name current_region;
VECTOR_3D s_vect;

{
   SM_Region_ptr the_region;
   SM_Region_Controlbox_ptr the_box;
   int the_index,v_index,l,m,n;
   VECTOR_3D old_point,new_point,origin_point;
   BOOL anchor;

    the_index = SM_region_index_get(the_region_list,current_region); 
    if (the_index != SM_INDEX_ERROR) {
     l = 2; m = 2; n = 2;
     the_box = SM_region_controlbox_new(l,m,n);
     SM_region_controlbox_recreate(the_box,l,m,n); 
     SM_region_controlbox_get(the_surface,the_region_list,the_index,the_box);
     SMGD_controlbox_origin_get(the_box,origin_point);
     the_region = &the_region_list->region_list[the_index];
     SM_FOR_EACH_REGION_VERTEX(the_region) 
      v_index = region_vertex_ptr->id;
      anchor = region_vertex_ptr->anchor;
      if ((SM_vertex_index_valid(the_surface,v_index)) && (!anchor) ) {
        SM_vertex_point_get(the_surface,v_index,old_point);
        V3D_SUB(old_point,old_point,origin_point);
        SMGD_scale_point(old_point,s_vect,new_point);
        V3D_ADD(new_point,new_point,origin_point);
        SM_vertex_point_set(the_surface,v_index,new_point);
      }
     SM_END_FOR_EACH
     SM_region_controlbox_dispose(the_box);
    }
}

/*
 * 
 */


void
SMGD_scale_object_controlbox(the_controlbox,s_vect)
SM_Surface_Controlbox_ptr the_controlbox;
VECTOR_3D s_vect;

{
      int i,number_of_points;
      VECTOR_3D old_point, new_point, max_point, min_point, origin_point;

      number_of_points = the_controlbox->l * the_controlbox->m * 
                         the_controlbox->n;
  
      SMGD_box_maxmin_point_get(the_controlbox,max_point,min_point);
      for (i=X_INDEX; i<=Z_INDEX; i++) 
        origin_point[i] = (max_point[i] + min_point[i])/2.0;

/* test 
      factor = s_vect[0] * s_vect[1] * s_vect[2];
      V3D_INIT(s_vect,1.0,1.0,1.0); 
*/
      for (i=0; i<number_of_points; i++) { 
       V3D_COPY(old_point,the_controlbox->vertex_list[i].point); 
       V3D_SUB(old_point,old_point,origin_point);
       SMGD_scale_point(old_point,s_vect,new_point);
       V3D_ADD(new_point,new_point,origin_point);
       V3D_COPY(the_controlbox->vertex_list[i].point,new_point); 
      }

/* test  
       the_controlbox->vertex_list[21].weight *= factor;
*/
     
}
       
/**
**/

void
SMGD_scale_object_control_plane(the_controlbox,index,s_vect,plane_dir)
SM_Surface_Controlbox_ptr the_controlbox;
int index;
VECTOR_3D s_vect;
int plane_dir;

{

     int i,j,k,ind_i,ind_j,ind_k,l,m,n,the_index;
     VECTOR_3D old_point, new_point, max_point, min_point, origin_point;

     SMGD_box_maxmin_point_get(the_controlbox,max_point,min_point);
      for (i=X_INDEX; i<=Z_INDEX; i++) 
        origin_point[i] = (max_point[i] + min_point[i])/2.0;

     l = the_controlbox->l;
     m = the_controlbox->m;
     n = the_controlbox->n;
     SMGD_controlbox_vertex_indices_get(the_controlbox,index,&ind_i,&ind_j,&ind_k);
     switch (plane_dir) {
        case 0 : 
         i = ind_i;
         for (k=0; k<n; k++) {        
          for (j=0; j<m; j++) {        
            the_index = k*(l*m) + j*l + i;
            V3D_COPY(old_point,the_controlbox->vertex_list[the_index].point); 
            V3D_SUB(old_point,old_point,origin_point);
            SMGD_scale_point(old_point,s_vect,new_point);
            V3D_ADD(new_point,new_point,origin_point);
            V3D_COPY(the_controlbox->vertex_list[the_index].point,new_point); 
          }
         }
         break;
        case 1 : 
         j = ind_j;
         for (k=0; k<n; k++) {        
          for (i=0; i<l; i++) {        
            the_index = k*(l*m) + j*l + i;
            V3D_COPY(old_point,the_controlbox->vertex_list[the_index].point); 
            V3D_SUB(old_point,old_point,origin_point);
            SMGD_scale_point(old_point,s_vect,new_point);
            V3D_ADD(new_point,new_point,origin_point);
            V3D_COPY(the_controlbox->vertex_list[the_index].point,new_point); 
          }
         }
         break;
        case 2 : 
         k = ind_k;
         for (j=0; j<m; j++) {        
          for (i=0; i<l; i++) {        
            the_index = k*(l*m) + j*l + i;
            V3D_COPY(old_point,the_controlbox->vertex_list[the_index].point); 
            V3D_SUB(old_point,old_point,origin_point);
            SMGD_scale_point(old_point,s_vect,new_point);
            V3D_ADD(new_point,new_point,origin_point);
            V3D_COPY(the_controlbox->vertex_list[the_index].point,new_point); 
          }
         }
         break;
        default :
        break;
      } 
}

void
SMGD_scale_object_control_point(the_controlbox,index,s_vect,origin)
SM_Surface_Controlbox_ptr the_controlbox;
int index;
VECTOR_3D s_vect,origin;

{

     VECTOR_3D old_point, new_point;

     V3D_COPY(old_point,the_controlbox->vertex_list[index].point); 
     V3D_SUB(old_point,old_point,origin);
     SMGD_scale_point(old_point,s_vect,new_point);
     V3D_ADD(new_point,new_point,origin);
     V3D_COPY(the_controlbox->vertex_list[index].point,new_point); 
}

/*---------------------------------------------------------*/
/* procedure for translating the object                    */

void
SMGD_translate_point(old_point,t_vect,new_point) 
VECTOR_3D old_point,t_vect,new_point;

{
  int i;

      for(i=X_INDEX; i<=Z_INDEX; i++) 
        new_point[i] = old_point[i] + t_vect[i];
}

/**
**/

void
SMGD_translate_surface(the_surface,t_vect)

SM_Surface_ptr the_surface;
VECTOR_3D t_vect;

{
   VECTOR_3D old_point,new_point;

   SM_FOR_EACH_VERTEX(the_surface)
      SM_vertex_point_get(the_surface,vertex_index,old_point);
      SMGD_translate_point(old_point,t_vect,new_point);
      SM_vertex_point_set(the_surface,vertex_index,new_point);
   SM_END_FOR_EACH
}

void
SMGD_translate_face_region(the_surface,the_face,the_region_list,current_region,t_vect)
SM_Surface_ptr the_surface, the_face;
SM_Region_List_ptr the_region_list;
SM_Region_Name current_region;
VECTOR_3D t_vect;

{
   SM_Region_ptr the_region;
   SM_Region_Controlbox_ptr the_box;
   int the_index,v_index,l,m,n;
   VECTOR_3D old_point,new_point,origin_point;
   BOOL anchor;

    the_index = SM_region_index_get(the_region_list,current_region); 
    if (the_index != SM_INDEX_ERROR) {
     l = 2; m = 2; n = 2;
     the_box = SM_region_controlbox_new(l,m,n);
     SM_region_controlbox_recreate(the_box,l,m,n); 
     SM_region_controlbox_get(the_face,the_region_list,the_index,the_box);
     SMGD_controlbox_origin_get(the_box,origin_point);
     the_region = &the_region_list->region_list[the_index];
     SM_FOR_EACH_REGION_VERTEX(the_region) 
      v_index = region_vertex_ptr->id;
      anchor = region_vertex_ptr->anchor;
      if ((SM_vertex_index_valid(the_face,v_index)) && (!anchor) ) {
        SM_vertex_point_get(the_face,v_index,old_point);
        V3D_SUB(old_point,old_point,origin_point);
        SMGD_translate_point(old_point,t_vect,new_point);
        V3D_ADD(new_point,new_point,origin_point);
        SM_vertex_point_set(the_surface,v_index,new_point);
      }
     SM_END_FOR_EACH
     SM_region_controlbox_dispose(the_box);
    }
}

/** 
 * 
 */


void
SMGD_translate_region(the_surface,the_region_list,current_region,t_vect)
SM_Surface_ptr the_surface;
SM_Region_List_ptr the_region_list;
SM_Region_Name current_region;
VECTOR_3D t_vect;

{
   SM_Region_ptr the_region;
   SM_Region_Controlbox_ptr the_box;
   int the_index,v_index,l,m,n;
   VECTOR_3D old_point,new_point,origin_point;
   BOOL anchor;

    the_index = SM_region_index_get(the_region_list,current_region); 
    if (the_index != SM_INDEX_ERROR) {
     l = 2; m = 2; n = 2;
     the_box = SM_region_controlbox_new(l,m,n);
     SM_region_controlbox_recreate(the_box,l,m,n); 
     SM_region_controlbox_get(the_surface,the_region_list,the_index,the_box);
     SMGD_controlbox_origin_get(the_box,origin_point);
     the_region = &the_region_list->region_list[the_index];
     SM_FOR_EACH_REGION_VERTEX(the_region) 
      v_index = region_vertex_ptr->id;
      anchor = region_vertex_ptr->anchor;
      if ((SM_vertex_index_valid(the_surface,v_index)) && (!anchor) ) {
        SM_vertex_point_get(the_surface,v_index,old_point);
        V3D_SUB(old_point,old_point,origin_point);
        SMGD_translate_point(old_point,t_vect,new_point);
        V3D_ADD(new_point,new_point,origin_point);
        SM_vertex_point_set(the_surface,v_index,new_point);
      }
     SM_END_FOR_EACH
     SM_region_controlbox_dispose(the_box);
    }
}

/*
**/

void
SMGD_translate_object_controlbox(the_controlbox,t_vect)
SM_Surface_Controlbox_ptr the_controlbox;
VECTOR_3D t_vect;

{
      int i,number_of_points;
      VECTOR_3D old_point, new_point, max_point, min_point, origin_point;

      number_of_points = the_controlbox->l * the_controlbox->m * 
                         the_controlbox->n;
  
      SMGD_box_maxmin_point_get(the_controlbox,max_point,min_point);
      for (i=X_INDEX; i<=Z_INDEX; i++) 
        origin_point[i] = (max_point[i] + min_point[i])/2.0;

      for (i=0; i<number_of_points; i++) { 
       V3D_COPY(old_point,the_controlbox->vertex_list[i].point); 
       V3D_SUB(old_point,old_point,origin_point);
       SMGD_translate_point(old_point,t_vect,new_point);
       V3D_ADD(new_point,new_point,origin_point);
       V3D_COPY(the_controlbox->vertex_list[i].point,new_point); 
      }
}
       

/**
**/

void
SMGD_translate_object_control_plane(the_controlbox,index,t_vect,plane_dir)
SM_Surface_Controlbox_ptr the_controlbox;
int index;
VECTOR_3D t_vect;
int plane_dir;

{

     int i,j,k,ind_i,ind_j,ind_k,l,m,n,the_index;
     VECTOR_3D old_point, new_point, max_point, min_point, origin_point;

     SMGD_box_maxmin_point_get(the_controlbox,max_point,min_point);
      for (i=X_INDEX; i<=Z_INDEX; i++) 
        origin_point[i] = (max_point[i] + min_point[i])/2.0;

     l = the_controlbox->l;
     m = the_controlbox->m;
     n = the_controlbox->n;
     SMGD_controlbox_vertex_indices_get(the_controlbox,index,&ind_i,&ind_j,&ind_k);
     switch (plane_dir) {
        case 0 : 
         i = ind_i;
         for (k=0; k<n; k++) {        
          for (j=0; j<m; j++) {        
            the_index = k*(l*m) + j*l + i;
            V3D_COPY(old_point,the_controlbox->vertex_list[the_index].point); 
            V3D_SUB(old_point,old_point,origin_point);
            SMGD_translate_point(old_point,t_vect,new_point);
            V3D_ADD(new_point,new_point,origin_point);
            V3D_COPY(the_controlbox->vertex_list[the_index].point,new_point); 
          }
         }
         break;
        case 1 : 
         j = ind_j;
         for (k=0; k<n; k++) {        
          for (i=0; i<l; i++) {        
            the_index = k*(l*m) + j*l + i;
            V3D_COPY(old_point,the_controlbox->vertex_list[the_index].point); 
            V3D_SUB(old_point,old_point,origin_point);
            SMGD_translate_point(old_point,t_vect,new_point);
            V3D_ADD(new_point,new_point,origin_point);
            V3D_COPY(the_controlbox->vertex_list[the_index].point,new_point); 
          }
         }
         break;
        case 2 : 
         k = ind_k;
         for (j=0; j<m; j++) {        
          for (i=0; i<l; i++) {        
            the_index = k*(l*m) + j*l + i;
            V3D_COPY(old_point,the_controlbox->vertex_list[the_index].point); 
            V3D_SUB(old_point,old_point,origin_point);
            SMGD_translate_point(old_point,t_vect,new_point);
            V3D_ADD(new_point,new_point,origin_point);
            V3D_COPY(the_controlbox->vertex_list[the_index].point,new_point); 
          }
         }
         break;
        default :
        break;
      } 
}

void
SMGD_translate_object_control_point(the_controlbox,index,t_vect,origin)
SM_Surface_Controlbox_ptr the_controlbox;
int index;
VECTOR_3D t_vect,origin;

{

     VECTOR_3D old_point, new_point;

     V3D_COPY(old_point,the_controlbox->vertex_list[index].point); 
     V3D_SUB(old_point,old_point,origin);
     SMGD_translate_point(old_point,t_vect,new_point);
     V3D_ADD(new_point,new_point,origin);
     V3D_COPY(the_controlbox->vertex_list[index].point,new_point); 
}

/*---------------------------------------------------------*/
/* procedure for rotating the object                       */

void
SMGD_rotate_point(old_point,axis_flag,theta,new_point) 
VECTOR_3D old_point,new_point;
int axis_flag;
float theta;

{
      switch (axis_flag) {
        case 0 :
          new_point[X_INDEX] = old_point[X_INDEX];
          new_point[Y_INDEX] = old_point[Y_INDEX]*cos(theta) - 
                               old_point[Z_INDEX]*sin(theta);
          new_point[Z_INDEX] = old_point[Y_INDEX]*sin(theta) + 
                               old_point[Z_INDEX]*cos(theta);
        break;
        case 1 :
          new_point[X_INDEX] = old_point[X_INDEX]*cos(theta) + 
                               old_point[Z_INDEX]*sin(theta);
          new_point[Y_INDEX] = old_point[Y_INDEX];
          new_point[Z_INDEX] = -old_point[X_INDEX]*sin(theta) + 
                               old_point[Z_INDEX]*cos(theta);
        break;
        case 2 :
          new_point[X_INDEX] = old_point[X_INDEX]*cos(theta) - 
                               old_point[Y_INDEX]*sin(theta);
          new_point[Y_INDEX] = old_point[X_INDEX]*sin(theta) + 
                               old_point[Y_INDEX]*cos(theta);
          new_point[Z_INDEX] = old_point[Z_INDEX];
        break;
        default :
        break;
      }
}

/**
**/

void
SMGD_rotate_surface(the_surface,axis_flag,theta)

SM_Surface_ptr the_surface;
int axis_flag;
float theta;

{
   VECTOR_3D old_point,new_point;

   SM_FOR_EACH_VERTEX(the_surface)
      SM_vertex_point_get(the_surface,vertex_index,old_point);
      SMGD_rotate_point(old_point,axis_flag,theta,new_point);
      SM_vertex_point_set(the_surface,vertex_index,new_point);
   SM_END_FOR_EACH
}

void
SMGD_rotate_face_region(the_surface,origin_point, the_region_list,current_region,
                    axis_flag,theta)
SM_Surface_ptr the_surface;
VECTOR_3D origin_point;
SM_Region_List_ptr the_region_list;
SM_Region_Name current_region;
int axis_flag;
float theta;

{
   SM_Region_ptr the_region;
   int the_index,v_index;
   VECTOR_3D old_point,new_point;
   BOOL anchor;

    the_index = SM_region_index_get(the_region_list,current_region); 
    if (the_index != SM_INDEX_ERROR) {
     the_region = &the_region_list->region_list[the_index];
     SM_FOR_EACH_REGION_VERTEX(the_region) 
      v_index = region_vertex_ptr->id;
      anchor = region_vertex_ptr->anchor;
      if ((SM_vertex_index_valid(the_surface,v_index)) && (!anchor) ) {
        SM_vertex_point_get(the_surface,v_index,old_point);
        V3D_SUB(old_point,old_point,origin_point);
        SMGD_rotate_point(old_point,axis_flag,theta,new_point);
        V3D_ADD(new_point,new_point,origin_point);
        SM_vertex_point_set(the_surface,v_index,new_point);
      }
     SM_END_FOR_EACH
    }
}

/*
 * A small change is done here the amount of rotation is considered as
 * absolute value and added to the old value coming from the the_face.
 * From the eralier where the relative amount was used, commands are
 * commented.
 *
 */
void
SMGD_rotate_try_face_region(the_surface, the_face, origin_point, the_region_list,current_region,
                    axis_flag,theta)
SM_Surface_ptr the_surface, the_face;
VECTOR_3D origin_point;
SM_Region_List_ptr the_region_list;
SM_Region_Name current_region;
int axis_flag;
float theta;

{
   SM_Region_ptr the_region;
   int the_index,v_index;
   VECTOR_3D old_point,new_point, the_surface_point, deform, the_face_point;
   BOOL anchor;

    the_index = SM_region_index_get(the_region_list,current_region); 

    if (the_index != SM_INDEX_ERROR) {
     the_region = &the_region_list->region_list[the_index];
     SM_FOR_EACH_REGION_VERTEX(the_region) 
      v_index = region_vertex_ptr->id;
      anchor = region_vertex_ptr->anchor;
      if ((SM_vertex_index_valid(the_surface,v_index)) && (!anchor) ) {
        SM_vertex_point_get(the_face,v_index,old_point);
        V3D_COPY(the_face_point,old_point);
        SM_vertex_point_get(the_surface,v_index,the_surface_point);
        V3D_SUB(old_point,old_point,origin_point);
        SMGD_rotate_point(old_point,axis_flag,theta,new_point);
        V3D_SUB(deform,new_point,old_point);
        V3D_ADD(new_point,deform,the_face_point);
	/*
        V3D_ADD(new_point,deform,the_surface_point);

        V3D_ADD(new_point,new_point,origin_point);
	*/	
        SM_vertex_point_set(the_surface,v_index,new_point);
      }
     SM_END_FOR_EACH
    }
}

/** 
 * 
 */


void
SMGD_rotate_region(the_surface,the_region_list,current_region,
                    axis_flag,theta)
SM_Surface_ptr the_surface;
SM_Region_List_ptr the_region_list;
SM_Region_Name current_region;
int axis_flag;
float theta;

{
   SM_Region_ptr the_region;
   SM_Region_Controlbox_ptr the_box;
   int the_index,v_index,l,m,n;
   VECTOR_3D old_point,new_point,origin_point;
   BOOL anchor;

    the_index = SM_region_index_get(the_region_list,current_region); 
    if (the_index != SM_INDEX_ERROR) {
     l = 2; m = 2; n = 2;
     the_box = SM_region_controlbox_new(l,m,n);
     SM_region_controlbox_recreate(the_box,l,m,n); 
     SM_region_controlbox_get(the_surface,the_region_list,the_index,the_box);
     SMGD_controlbox_origin_get(the_box,origin_point);
     the_region = &the_region_list->region_list[the_index];
     SM_FOR_EACH_REGION_VERTEX(the_region) 
      v_index = region_vertex_ptr->id;
      anchor = region_vertex_ptr->anchor;
      if ((SM_vertex_index_valid(the_surface,v_index)) && (!anchor) ) {
        SM_vertex_point_get(the_surface,v_index,old_point);
        V3D_SUB(old_point,old_point,origin_point);
        SMGD_rotate_point(old_point,axis_flag,theta,new_point);
        V3D_ADD(new_point,new_point,origin_point);
        SM_vertex_point_set(the_surface,v_index,new_point);
      }
     SM_END_FOR_EACH
     SM_region_controlbox_dispose(the_box);
    }
    
}

/*
**/

void
SMGD_rotate_object_controlbox(the_controlbox,axis_flag,theta)
SM_Surface_Controlbox_ptr the_controlbox;
int axis_flag;
float theta;
{
      int i,number_of_points;
      VECTOR_3D old_point, new_point, max_point, min_point, origin_point;

      number_of_points = the_controlbox->l * the_controlbox->m * 
                         the_controlbox->n;
  
      SMGD_box_maxmin_point_get(the_controlbox,max_point,min_point);
      for (i=X_INDEX; i<=Z_INDEX; i++) 
        origin_point[i] = (max_point[i] + min_point[i])/2.0;
      for (i=0; i<number_of_points; i++) { 
       V3D_COPY(old_point,the_controlbox->vertex_list[i].point); 
       V3D_SUB(old_point,old_point,origin_point);
       SMGD_rotate_point(old_point,axis_flag,theta,new_point);
       V3D_ADD(new_point,new_point,origin_point);
       V3D_COPY(the_controlbox->vertex_list[i].point,new_point); 
      }
}
/**
**/

void
SMGD_rotate_object_control_plane(the_controlbox,index,axis_flag,theta,plane_dir)
SM_Surface_Controlbox_ptr the_controlbox;
int index;
int axis_flag;
float theta;
int plane_dir;

{

     int i,j,k,ind_i,ind_j,ind_k,l,m,n,the_index;
     VECTOR_3D old_point, new_point, max_point, min_point, origin_point;

     SMGD_box_maxmin_point_get(the_controlbox,max_point,min_point);
      for (i=X_INDEX; i<=Z_INDEX; i++) 
        origin_point[i] = (max_point[i] + min_point[i])/2.0;

     l = the_controlbox->l;
     m = the_controlbox->m;
     n = the_controlbox->n;
     SMGD_controlbox_vertex_indices_get(the_controlbox,index,&ind_i,&ind_j,&ind_k);
     switch (plane_dir) {
        case 0 : 
         i = ind_i;
         for (k=0; k<n; k++) {        
          for (j=0; j<m; j++) {        
            the_index = k*(l*m) + j*l + i;
            V3D_COPY(old_point,the_controlbox->vertex_list[the_index].point); 
            V3D_SUB(old_point,old_point,origin_point);
            SMGD_rotate_point(old_point,axis_flag,theta,new_point);
            V3D_ADD(new_point,new_point,origin_point);
            V3D_COPY(the_controlbox->vertex_list[the_index].point,new_point); 
          }
         }
         break;
        case 1 : 
         j = ind_j;
         for (k=0; k<n; k++) {        
          for (i=0; i<l; i++) {        
            the_index = k*(l*m) + j*l + i;
            V3D_COPY(old_point,the_controlbox->vertex_list[the_index].point); 
            V3D_SUB(old_point,old_point,origin_point);
            SMGD_rotate_point(old_point,axis_flag,theta,new_point);
            V3D_ADD(new_point,new_point,origin_point);
            V3D_COPY(the_controlbox->vertex_list[the_index].point,new_point); 
          }
         }
         break;
        case 2 : 
         k = ind_k;
         for (j=0; j<m; j++) {        
          for (i=0; i<l; i++) {        
            the_index = k*(l*m) + j*l + i;
            V3D_COPY(old_point,the_controlbox->vertex_list[the_index].point); 
            V3D_SUB(old_point,old_point,origin_point);
            SMGD_rotate_point(old_point,axis_flag,theta,new_point);
            V3D_ADD(new_point,new_point,origin_point);
            V3D_COPY(the_controlbox->vertex_list[the_index].point,new_point); 
          }
         }
         break;
        default :
        break;
      } 
}

void
SMGD_rotate_object_control_point(the_controlbox,index,axis_flag,theta,origin)
SM_Surface_Controlbox_ptr the_controlbox;
int index;
int axis_flag;
float theta;
VECTOR_3D origin;
{

     VECTOR_3D old_point, new_point;

     V3D_COPY(old_point,the_controlbox->vertex_list[index].point); 
     V3D_SUB(old_point,old_point,origin);
     SMGD_rotate_point(old_point,axis_flag,theta,new_point);
     V3D_ADD(new_point,new_point,origin);
     V3D_COPY(the_controlbox->vertex_list[index].point,new_point); 
}
/*---------------------------------------------------------*/
void
SMGD_rotate_arbaxis_matrix(v_point,v_dir,theta,R)
VECTOR_3D v_point, v_dir;
float theta;
MATRIX_H3D R;
{
     MATRIX_H3D T, T_1, Rx, Rx_1, Ry, Ry_1, Rz, Rt;
     float x1, y1, z1;
     float A,B,C,V,L;

     x1 = v_point[0]; y1 = v_point[1]; z1 = v_point[2];
     A = v_dir[0]; B = v_dir[1]; C = v_dir[2];
     V = sqrt(B*B + C*C);
     L = sqrt(A*A + B*B + C*C);

     MH3D_INIT(T, 1.0, 0.0, 0.0, 0.0,
                  0.0, 1.0, 0.0, 0.0,
                  0.0, 0.0, 1.0, 0.0,
                  -x1, -y1, -z1, 1.0);

     MH3D_INIT(T_1, 1.0, 0.0, 0.0, 0.0,
                    0.0, 1.0, 0.0, 0.0,
                    0.0, 0.0, 1.0, 0.0,
                     x1,  y1,  z1, 1.0);

     if (V < SM_EPSILON ) {
       MH3D_IDENTITY(Rx);
       MH3D_IDENTITY(Rx_1);
     }
     else {
      MH3D_INIT(Rx, 1.0, 0.0, 0.0, 0.0,
                  0.0, C/V, B/V, 0.0,
                  0.0,-B/V, C/V, 0.0,
                  0.0, 0.0, 0.0, 1.0);

      MH3D_INIT(Rx_1, 1.0, 0.0, 0.0, 0.0,
                  0.0, C/V, -B/V, 0.0,
                  0.0, B/V, C/V, 0.0,
                  0.0, 0.0, 0.0, 1.0);
     }

     if (L < SM_EPSILON) {
       MH3D_IDENTITY(Ry);
       MH3D_IDENTITY(Ry_1);
     }
     else {
      MH3D_INIT(Ry, V/L, 0.0, A/L, 0.0,
                  0.0, 1.0, 0.0, 0.0,
                  -A/L,0.0, V/L, 0.0,
                  0.0, 0.0, 0.0, 1.0);

      MH3D_INIT(Ry_1, V/L, 0.0, -A/L, 0.0,
                  0.0, 1.0, 0.0, 0.0,
                  A/L,0.0, V/L, 0.0,
                  0.0, 0.0, 0.0, 1.0);
     }

     MH3D_INIT(Rz, cos(theta), sin(theta), 0.0, 0.0,
                  -sin(theta), cos(theta), 0.0, 0.0,
                  0.0, 0.0, 1.0, 0.0,
                  0.0, 0.0, 0.0, 1.0);

     MH3D_mul(Rt,Rx_1,T_1);
     MH3D_mul(Rt,Ry_1,Rt);
     MH3D_mul(Rt,Rz,Rt);
     MH3D_mul(Rt,Ry,Rt);
     MH3D_mul(Rt,Rx,Rt);
     MH3D_mul(Rt,T,Rt);
     
     MH3D_COPY(R,Rt);

} 

void
SMGD_rotate_arbaxis_point(old_point,matrix,new_point)
VECTOR_3D old_point,new_point;
MATRIX_H3D matrix;
{
     VECTOR_H3D point_h;

     VH3D_INIT(point_h,old_point[0],old_point[1],old_point[2]);
     MH3D_transform_point_H3D(point_h,matrix,point_h);
     V3D_FROM_VH3D(new_point,point_h);
}

/*---------------------------------------------------------*/
/* procedure for circular transformation                   */

void
SMGD_circular_point(old_point,axis_flag,rad,new_point) 
VECTOR_3D old_point,new_point;
int axis_flag;
float rad;

{

 float theta;
 
      switch (axis_flag) {
        case 0 : /* y-z plane */
	  theta = atan2(old_point[Z_INDEX], old_point[Y_INDEX]);
	  if (fabs(old_point[Y_INDEX]) <= SM_EPSILON) {
	   if (fabs(old_point[Z_INDEX]) <= SM_EPSILON) {
	       V3D_COPY(new_point, old_point);
	       break;
	   } else {
	    if (old_point[Z_INDEX] < 0.0) 
	     theta = -M_PI/2.0;
	    else
	     theta = M_PI/2.0; 
	   }	  	  
	  }
	  new_point[Y_INDEX] = rad * cos(theta);
	  new_point[Z_INDEX] = rad * sin(theta);
	  new_point[X_INDEX] = old_point[X_INDEX];
        break;
        case 1 : /* x-z plane */
	  theta = atan2(old_point[X_INDEX], old_point[Z_INDEX]);
	  if (fabs(old_point[Z_INDEX]) <= SM_EPSILON) {
	   if (fabs(old_point[X_INDEX]) <= SM_EPSILON) {
	    V3D_COPY(new_point, old_point);
	    break;
	   } else {
	    if (old_point[X_INDEX] < 0.0) 
	     theta = -M_PI/2.0;
	    else
	     theta = M_PI/2.0; 
	   }
	  }
	  new_point[Z_INDEX] = rad * cos(theta);
	  new_point[X_INDEX] = rad * sin(theta);	  	  
	  new_point[Y_INDEX] = old_point[Y_INDEX];	  	  
        break;
        case 2 : /* x-y plane */
	  theta = atan2(old_point[Y_INDEX], old_point[X_INDEX]);
	  if (fabs(old_point[X_INDEX]) <= SM_EPSILON) {
	   if (fabs(old_point[Y_INDEX]) <= SM_EPSILON) {
	      V3D_COPY(new_point, old_point);
	      break;
	   } else {
	    if (old_point[Y_INDEX] < 0.0) 
	     theta = -M_PI/2.0;
	    else
	     theta = M_PI/2.0; 
	   }
	  }
	  new_point[X_INDEX] = rad * cos(theta);
	  new_point[Y_INDEX] = rad * sin(theta);
	  new_point[Z_INDEX] = old_point[Z_INDEX];	  	  
        break;
        default :
        break;
      }
}

/*------------------------------------------------------------------*/

void
SMGD_circular_object_control_plane(the_controlbox,index,axis_flag,value)
SM_Surface_Controlbox_ptr the_controlbox;
int index;
int axis_flag;
float value;

{

     int i,j,k,ind_i,ind_j,ind_k,l,m,n,the_index;
     VECTOR_3D old_point, new_point, max_point, min_point, origin_point;
     VECTOR_3D temp;
     float rad;
     
     SMGD_box_maxmin_point_get(the_controlbox,max_point,min_point);
      for (i=X_INDEX; i<=Z_INDEX; i++) 
        origin_point[i] = (max_point[i] + min_point[i])/2.0;
     
     l = the_controlbox->l;
     m = the_controlbox->m;
     n = the_controlbox->n;
     SMGD_controlbox_vertex_indices_get(the_controlbox,index,&ind_i,&ind_j,&ind_k);
     switch (axis_flag) {
        case 0 : 
	 V3D_INIT(temp, the_controlbox->vertex_list[index].point[0], max_point[1], max_point[2]);
	 origin_point[0] =  the_controlbox->vertex_list[index].point[0];
	 V3D_SUB(temp, temp, origin_point);
	 rad = V3D_NORM(temp);
         i = ind_i;
         for (k=0; k<n; k++) {        
          for (j=0; j<m; j++) {        
            the_index = k*(l*m) + j*l + i;
            V3D_COPY(old_point,the_controlbox->vertex_list[the_index].point); 
            V3D_SUB(old_point,old_point,origin_point);
            SMGD_circular_point(old_point,axis_flag,rad,new_point);
            V3D_ADD(new_point,new_point,origin_point);
            V3D_COPY(the_controlbox->vertex_list[the_index].point,new_point); 
          }
         }
         break;
        case 1 : 
	 V3D_INIT(temp, max_point[0], the_controlbox->vertex_list[index].point[1], max_point[2]);
	 origin_point[1] =  the_controlbox->vertex_list[index].point[1];
	 V3D_SUB(temp, temp, origin_point);
	 rad = V3D_NORM(temp);
         j = ind_j;
         for (k=0; k<n; k++) {        
          for (i=0; i<l; i++) {        
            the_index = k*(l*m) + j*l + i;
            V3D_COPY(old_point,the_controlbox->vertex_list[the_index].point); 
            V3D_SUB(old_point,old_point,origin_point);
            SMGD_circular_point(old_point,axis_flag,rad,new_point);
            V3D_ADD(new_point,new_point,origin_point);
            V3D_COPY(the_controlbox->vertex_list[the_index].point,new_point); 
          }
         }
         break;
        case 2 : 
	 V3D_INIT(temp, max_point[0], max_point[1], the_controlbox->vertex_list[index].point[2]);
	 origin_point[2] =  the_controlbox->vertex_list[index].point[2];
	 V3D_SUB(temp, temp, origin_point);
	 rad = V3D_NORM(temp);
         k = ind_k;
         for (j=0; j<m; j++) {        
          for (i=0; i<l; i++) {        
            the_index = k*(l*m) + j*l + i;
            V3D_COPY(old_point,the_controlbox->vertex_list[the_index].point); 
            V3D_SUB(old_point,old_point,origin_point);
            SMGD_circular_point(old_point,axis_flag,rad,new_point);
            V3D_ADD(new_point,new_point,origin_point);
            V3D_COPY(the_controlbox->vertex_list[the_index].point,new_point); 
          }
         }
         break;
        default :
        break;
      } 
}


/*---------------------------------------------------------*/


/* procedure for shearing the object                       */

void
SMGD_shear_point(old_point,s_value,axis_flag,new_point) 
VECTOR_3D old_point;
float s_value;
int axis_flag;
VECTOR_3D new_point;

{
        V3D_COPY(new_point,old_point);
        switch (axis_flag) {
         case 0 : 
           new_point[0] = old_point[0] + (s_value) * old_point[1];
         break;
         case 1 : 
           new_point[1] = old_point[1] + (s_value) * old_point[2];
         break;
         case 2 : 
           new_point[2] = old_point[2] + (s_value) * old_point[0];
         break;
        }
}

/**
**/

void
SMGD_shear_surface(the_surface,s_value,axis_flag)

SM_Surface_ptr the_surface;
float s_value;
int axis_flag;

{
   VECTOR_3D old_point,new_point;

   SM_FOR_EACH_VERTEX(the_surface)
      SM_vertex_point_get(the_surface,vertex_index,old_point);
      SMGD_shear_point(old_point,s_value,axis_flag,new_point);
      SM_vertex_point_set(the_surface,vertex_index,new_point);
   SM_END_FOR_EACH
}


void
SMGD_shear_face_region(the_surface,the_face, the_region_list,current_region,s_value,
                   axis_flag)
SM_Surface_ptr the_surface, the_face;
SM_Region_List_ptr the_region_list;
SM_Region_Name current_region;
float s_value;
int axis_flag;

{
   SM_Region_ptr the_region;
   SM_Region_Controlbox_ptr the_box;
   int the_index,v_index,l,m,n;
   VECTOR_3D old_point,new_point,origin_point;
   BOOL anchor;

    the_index = SM_region_index_get(the_region_list,current_region); 
    if (the_index != SM_INDEX_ERROR) {
     l = 2; m = 2; n = 2;
     the_box = SM_region_controlbox_new(l,m,n);
     SM_region_controlbox_recreate(the_box,l,m,n); 
     SM_region_controlbox_get(the_face,the_region_list,the_index,the_box);
     SMGD_controlbox_origin_get(the_box,origin_point);
     the_region = &the_region_list->region_list[the_index];
     SM_FOR_EACH_REGION_VERTEX(the_region) 
      v_index = region_vertex_ptr->id;
      anchor = region_vertex_ptr->anchor;
      if ((SM_vertex_index_valid(the_face,v_index)) && (!anchor) ) {
        SM_vertex_point_get(the_face,v_index,old_point);
        V3D_SUB(old_point,old_point,origin_point);
        SMGD_shear_point(old_point,s_value,axis_flag,new_point);
        V3D_ADD(new_point,new_point,origin_point);
        SM_vertex_point_set(the_surface,v_index,new_point);
      }
     SM_END_FOR_EACH
     SM_region_controlbox_dispose(the_box);
    }
}

/** 
 * 
 */


void
SMGD_shear_region(the_surface,the_region_list,current_region,s_value,
                   axis_flag)
SM_Surface_ptr the_surface;
SM_Region_List_ptr the_region_list;
SM_Region_Name current_region;
float s_value;
int axis_flag;

{
   SM_Region_ptr the_region;
   SM_Region_Controlbox_ptr the_box;
   int the_index,v_index,l,m,n;
   VECTOR_3D old_point,new_point,origin_point;
   BOOL anchor;

    the_index = SM_region_index_get(the_region_list,current_region); 
    if (the_index != SM_INDEX_ERROR) {
     l = 2; m = 2; n = 2;
     the_box = SM_region_controlbox_new(l,m,n);
     SM_region_controlbox_recreate(the_box,l,m,n); 
     SM_region_controlbox_get(the_surface,the_region_list,the_index,the_box);
     SMGD_controlbox_origin_get(the_box,origin_point);
     the_region = &the_region_list->region_list[the_index];
     SM_FOR_EACH_REGION_VERTEX(the_region) 
      v_index = region_vertex_ptr->id;
      anchor = region_vertex_ptr->anchor;
      if ((SM_vertex_index_valid(the_surface,v_index)) && (!anchor) ) {
        SM_vertex_point_get(the_surface,v_index,old_point);
        V3D_SUB(old_point,old_point,origin_point);
        SMGD_shear_point(old_point,s_value,axis_flag,new_point);
        V3D_ADD(new_point,new_point,origin_point);
        SM_vertex_point_set(the_surface,v_index,new_point);
      }
     SM_END_FOR_EACH
     SM_region_controlbox_dispose(the_box);
    }
}

/*
**/

void
SMGD_shear_object_controlbox(the_controlbox,s_value,axis_flag)
SM_Surface_Controlbox_ptr the_controlbox;
float s_value;
int axis_flag;

{
      int i,number_of_points;
      VECTOR_3D old_point, new_point, max_point, min_point, origin_point;

      number_of_points = the_controlbox->l * the_controlbox->m * 
                         the_controlbox->n;
  
      SMGD_box_maxmin_point_get(the_controlbox,max_point,min_point);
      for (i=X_INDEX; i<=Z_INDEX; i++) 
        origin_point[i] = (max_point[i] + min_point[i])/2.0;
      for (i=0; i<number_of_points; i++) { 
       V3D_COPY(old_point,the_controlbox->vertex_list[i].point); 
       V3D_SUB(old_point,old_point,origin_point);
       SMGD_shear_point(old_point,s_value,axis_flag,new_point);
       V3D_ADD(new_point,new_point,origin_point);
       V3D_COPY(the_controlbox->vertex_list[i].point,new_point); 
      }
}

void
SMGD_shearmin_face_region(the_surface,the_face,the_region_list,current_region,s_value,axis_flag)
SM_Surface_ptr the_surface, the_face;
SM_Region_List_ptr the_region_list;
SM_Region_Name current_region;
float s_value;
int axis_flag;

{
   SM_Region_ptr the_region;
   SM_Region_Controlbox_ptr the_box;
   int the_index,v_index,i,l,m,n;
   BOOL anchor;
   VECTOR_3D old_point, new_point, max_point, min_point, origin_point;
     
    the_index = SM_region_index_get(the_region_list,current_region); 
    if (the_index != SM_INDEX_ERROR) {
     l = 2; m = 2; n = 2;
     the_box = SM_region_controlbox_new(l,m,n);
     SM_region_controlbox_recreate(the_box,l,m,n); 
     SM_region_controlbox_get(the_face,the_region_list,the_index,the_box);
     SMGD_box_maxmin_point_get(the_box,max_point,min_point);
     for (i=X_INDEX; i<=Z_INDEX; i++) 
        origin_point[i] = (max_point[i] + min_point[i])/2.0;
     switch (axis_flag) {
         case 0 : 
           origin_point[1] = min_point[1];
         break;
         case 1 : 
           origin_point[2] = min_point[2];
         break;
         case 2 : 
           origin_point[0] = min_point[0];
         break;
      }
     the_region = &the_region_list->region_list[the_index];
     SM_FOR_EACH_REGION_VERTEX(the_region) 
      v_index = region_vertex_ptr->id;
      anchor = region_vertex_ptr->anchor;
      if ((SM_vertex_index_valid(the_face,v_index)) && (!anchor) ) {
        SM_vertex_point_get(the_face,v_index,old_point);
        V3D_SUB(old_point,old_point,origin_point);
        SMGD_shear_point(old_point,s_value,axis_flag,new_point);
        V3D_ADD(new_point,new_point,origin_point);
        SM_vertex_point_set(the_surface,v_index,new_point);
      }
     SM_END_FOR_EACH
     SM_region_controlbox_dispose(the_box);
    }
}

/**
 * 
 */


void
SMGD_shearmin_region(the_surface,the_region_list,current_region,s_value,axis_flag)
SM_Surface_ptr the_surface;
SM_Region_List_ptr the_region_list;
SM_Region_Name current_region;
float s_value;
int axis_flag;

{
   SM_Region_ptr the_region;
   SM_Region_Controlbox_ptr the_box;
   int the_index,v_index,i,l,m,n;
   BOOL anchor;
   VECTOR_3D old_point, new_point, max_point, min_point, origin_point;
     
    the_index = SM_region_index_get(the_region_list,current_region); 
    if (the_index != SM_INDEX_ERROR) {
     l = 2; m = 2; n = 2;
     the_box = SM_region_controlbox_new(l,m,n);
     SM_region_controlbox_recreate(the_box,l,m,n); 
     SM_region_controlbox_get(the_surface,the_region_list,the_index,the_box);
     SMGD_box_maxmin_point_get(the_box,max_point,min_point);
     for (i=X_INDEX; i<=Z_INDEX; i++) 
        origin_point[i] = (max_point[i] + min_point[i])/2.0;
     switch (axis_flag) {
         case 0 : 
           origin_point[1] = min_point[1];
         break;
         case 1 : 
           origin_point[2] = min_point[2];
         break;
         case 2 : 
           origin_point[0] = min_point[0];
         break;
      }
     the_region = &the_region_list->region_list[the_index];
     SM_FOR_EACH_REGION_VERTEX(the_region) 
      v_index = region_vertex_ptr->id;
      anchor = region_vertex_ptr->anchor;
      if ((SM_vertex_index_valid(the_surface,v_index)) && (!anchor) ) {
        SM_vertex_point_get(the_surface,v_index,old_point);
        V3D_SUB(old_point,old_point,origin_point);
        SMGD_shear_point(old_point,s_value,axis_flag,new_point);
        V3D_ADD(new_point,new_point,origin_point);
        SM_vertex_point_set(the_surface,v_index,new_point);
      }
     SM_END_FOR_EACH
     SM_region_controlbox_dispose(the_box);
    }
}

/*
**/

void
SMGD_shearmax_region(the_surface,the_region_list,current_region,s_value,axis_flag)
SM_Surface_ptr the_surface;
SM_Region_List_ptr the_region_list;
SM_Region_Name current_region;
float s_value;
int axis_flag;

{
   SM_Region_ptr the_region;
   SM_Region_Controlbox_ptr the_box;
   int the_index,v_index,i,l,m,n;
   BOOL anchor;
   VECTOR_3D old_point, new_point, max_point, min_point, origin_point;
     
    the_index = SM_region_index_get(the_region_list,current_region); 
    if (the_index != SM_INDEX_ERROR) {
     l = 2; m = 2; n = 2;
     the_box = SM_region_controlbox_new(l,m,n);
     SM_region_controlbox_recreate(the_box,l,m,n); 
     SM_region_controlbox_get(the_surface,the_region_list,the_index,the_box);
     SMGD_box_maxmin_point_get(the_box,max_point,min_point);
     for (i=X_INDEX; i<=Z_INDEX; i++) 
        origin_point[i] = (max_point[i] + min_point[i])/2.0;
     switch (axis_flag) {
         case 0 : 
           origin_point[1] = max_point[1];
         break;
         case 1 : 
           origin_point[2] = max_point[2];
         break;
         case 2 : 
           origin_point[0] = max_point[0];
         break;
      }
     the_region = &the_region_list->region_list[the_index];
     SM_FOR_EACH_REGION_VERTEX(the_region) 
      v_index = region_vertex_ptr->id;
      anchor = region_vertex_ptr->anchor;
      if ((SM_vertex_index_valid(the_surface,v_index)) && (!anchor) ) {
        SM_vertex_point_get(the_surface,v_index,old_point);
        V3D_SUB(old_point,old_point,origin_point);
        SMGD_shear_point(old_point,s_value,axis_flag,new_point);
        V3D_ADD(new_point,new_point,origin_point);
        SM_vertex_point_set(the_surface,v_index,new_point);
      }
     SM_END_FOR_EACH
     SM_region_controlbox_dispose(the_box);
    }
}

/**
**/

void
SMGD_shearcenter_point(old_point,s_value,axis_flag,new_point,max_point,min_point) 
VECTOR_3D old_point;
float s_value;
int axis_flag;
VECTOR_3D new_point,max_point,min_point;

{
   float delta1,delta2;

        V3D_COPY(new_point,old_point);
        switch (axis_flag) {
         case 0 : 
           delta1 = max_point[1] - min_point[1];
           delta2 = max_point[2] - min_point[2];
           new_point[0] = old_point[0] + (s_value) * (delta1/2.0 - fabs(old_point[1]))
                                       + (s_value) * (delta2/2.0 - fabs(old_point[2]));
         break;
         case 1 : 
           delta1 = max_point[0] - min_point[0];
           delta2 = max_point[2] - min_point[2];
           new_point[1] = old_point[1] + (s_value) * (delta1/2.0 - fabs(old_point[0]))
                                       + (s_value) * (delta2/2.0 - fabs(old_point[2]));
         break;
         case 2 : 
           delta1 = max_point[0] - min_point[0];
           delta2 = max_point[1] - min_point[1];
           new_point[2] = old_point[2] + (s_value) * (delta1/2.0 - fabs(old_point[0]))
                                       + (s_value) * (delta2/2.0 - fabs(old_point[1]));

         break;
        }
}

void
SMGD_shearcenter_face_region(the_surface,the_face,the_region_list,current_region,s_value,axis_flag)
SM_Surface_ptr the_surface, the_face;
SM_Region_List_ptr the_region_list;
SM_Region_Name current_region;
float s_value;
int axis_flag;

{
   SM_Region_ptr the_region;
   SM_Region_Controlbox_ptr the_box;
   int the_index,v_index,i,l,m,n;
   BOOL anchor;
   VECTOR_3D old_point, new_point, max_point, min_point, origin_point;

    the_index = SM_region_index_get(the_region_list,current_region); 
    if (the_index != SM_INDEX_ERROR) {
     l = 2; m = 2; n = 2;
     the_box = SM_region_controlbox_new(l,m,n);
     SM_region_controlbox_recreate(the_box,l,m,n); 
     SM_region_controlbox_get(the_face,the_region_list,the_index,the_box);
     SMGD_box_maxmin_point_get(the_box,max_point,min_point);
     for (i=X_INDEX; i<=Z_INDEX; i++) 
       origin_point[i] = (max_point[i] + min_point[i])/2.0;
     the_region = &the_region_list->region_list[the_index];
     SM_FOR_EACH_REGION_VERTEX(the_region) 
      v_index = region_vertex_ptr->id;
      anchor = region_vertex_ptr->anchor;
      if ((SM_vertex_index_valid(the_face,v_index)) && (!anchor) ) {
        SM_vertex_point_get(the_face,v_index,old_point);
        V3D_SUB(old_point,old_point,origin_point);
        SMGD_shearcenter_point(old_point,s_value,axis_flag,new_point,max_point,min_point);
        V3D_ADD(new_point,new_point,origin_point);
        SM_vertex_point_set(the_surface,v_index,new_point);
      }
     SM_END_FOR_EACH
     SM_region_controlbox_dispose(the_box);
    }
}

/** 
 * 
 */


void
SMGD_shearcenter_region(the_surface,the_region_list,current_region,s_value,axis_flag)
SM_Surface_ptr the_surface;
SM_Region_List_ptr the_region_list;
SM_Region_Name current_region;
float s_value;
int axis_flag;

{
   SM_Region_ptr the_region;
   SM_Region_Controlbox_ptr the_box;
   int the_index,v_index,i,l,m,n;
   BOOL anchor;
   VECTOR_3D old_point, new_point, max_point, min_point, origin_point;

    the_index = SM_region_index_get(the_region_list,current_region); 
    if (the_index != SM_INDEX_ERROR) {
     l = 2; m = 2; n = 2;
     the_box = SM_region_controlbox_new(l,m,n);
     SM_region_controlbox_recreate(the_box,l,m,n); 
     SM_region_controlbox_get(the_surface,the_region_list,the_index,the_box);
     SMGD_box_maxmin_point_get(the_box,max_point,min_point);
     for (i=X_INDEX; i<=Z_INDEX; i++) 
       origin_point[i] = (max_point[i] + min_point[i])/2.0;
     the_region = &the_region_list->region_list[the_index];
     SM_FOR_EACH_REGION_VERTEX(the_region) 
      v_index = region_vertex_ptr->id;
      anchor = region_vertex_ptr->anchor;
      if ((SM_vertex_index_valid(the_surface,v_index)) && (!anchor) ) {
        SM_vertex_point_get(the_surface,v_index,old_point);
        V3D_SUB(old_point,old_point,origin_point);
        SMGD_shearcenter_point(old_point,s_value,axis_flag,new_point,max_point,min_point);
        V3D_ADD(new_point,new_point,origin_point);
        SM_vertex_point_set(the_surface,v_index,new_point);
      }
     SM_END_FOR_EACH
     SM_region_controlbox_dispose(the_box);
    }
}

/*
**/

void
SMGD_shearmin_object_controlbox(the_controlbox,s_value,axis_flag)
SM_Surface_Controlbox_ptr the_controlbox;
float s_value;
int axis_flag;

{
      int i,number_of_points;
      VECTOR_3D old_point, new_point, max_point, min_point, origin_point;

      number_of_points = the_controlbox->l * the_controlbox->m * 
                         the_controlbox->n;
  
      SMGD_box_maxmin_point_get(the_controlbox,max_point,min_point);
      for (i=X_INDEX; i<=Z_INDEX; i++) 
        origin_point[i] = (max_point[i] + min_point[i])/2.0;

      /* shift the origin point to the min value in the manner shear
         operation takes place                                    */

      switch (axis_flag) {
         case 0 : 
           origin_point[1] = min_point[1];
         break;
         case 1 : 
           origin_point[2] = min_point[2];
         break;
         case 2 : 
           origin_point[0] = min_point[0];
         break;
      }

      for (i=0; i<number_of_points; i++) { 
       V3D_COPY(old_point,the_controlbox->vertex_list[i].point); 
       V3D_SUB(old_point,old_point,origin_point);
       SMGD_shear_point(old_point,s_value,axis_flag,new_point);
       V3D_ADD(new_point,new_point,origin_point);
       V3D_COPY(the_controlbox->vertex_list[i].point,new_point); 
      }
}

/**
**/

void
SMGD_shear_object_control_plane(the_controlbox,index,s_value,axis_flag,plane_dir)
SM_Surface_Controlbox_ptr the_controlbox;
int index;
float s_value;
int axis_flag;
int plane_dir;

{

     int i,j,k,ind_i,ind_j,ind_k,l,m,n,the_index;
     VECTOR_3D old_point, new_point, max_point, min_point, origin_point;

     SMGD_box_maxmin_point_get(the_controlbox,max_point,min_point);
      for (i=X_INDEX; i<=Z_INDEX; i++) 
        origin_point[i] = (max_point[i] + min_point[i])/2.0;

     l = the_controlbox->l;
     m = the_controlbox->m;
     n = the_controlbox->n;
     SMGD_controlbox_vertex_indices_get(the_controlbox,index,&ind_i,&ind_j,&ind_k);
     switch (plane_dir) {
        case 0 : 
         i = ind_i;
         for (k=0; k<n; k++) {        
          for (j=0; j<m; j++) {        
            the_index = k*(l*m) + j*l + i;
            V3D_COPY(old_point,the_controlbox->vertex_list[the_index].point); 
            V3D_SUB(old_point,old_point,origin_point);
            SMGD_shear_point(old_point,s_value,axis_flag,new_point);
            V3D_ADD(new_point,new_point,origin_point);
            V3D_COPY(the_controlbox->vertex_list[the_index].point,new_point); 
          }
         }
         break;
        case 1 : 
         j = ind_j;
         for (k=0; k<n; k++) {        
          for (i=0; i<l; i++) {        
            the_index = k*(l*m) + j*l + i;
            V3D_COPY(old_point,the_controlbox->vertex_list[the_index].point); 
            V3D_SUB(old_point,old_point,origin_point);
            SMGD_shear_point(old_point,s_value,axis_flag,new_point);
            V3D_ADD(new_point,new_point,origin_point);
            V3D_COPY(the_controlbox->vertex_list[the_index].point,new_point); 
          }
         }
         break;
        case 2 : 
         k = ind_k;
         for (j=0; j<m; j++) {        
          for (i=0; i<l; i++) {        
            the_index = k*(l*m) + j*l + i;
            V3D_COPY(old_point,the_controlbox->vertex_list[the_index].point); 
            V3D_SUB(old_point,old_point,origin_point);
            SMGD_shear_point(old_point,s_value,axis_flag,new_point);
            V3D_ADD(new_point,new_point,origin_point);
            V3D_COPY(the_controlbox->vertex_list[the_index].point,new_point); 
          }
         }
         break;
        default :
        break;
      } 
}

void
SMGD_shear_object_control_point(the_controlbox,index,s_value,axis_flag,origin)
SM_Surface_Controlbox_ptr the_controlbox;
int index;
float s_value;
int axis_flag;
VECTOR_3D origin;
{

     VECTOR_3D old_point, new_point;

     V3D_COPY(old_point,the_controlbox->vertex_list[index].point); 
     V3D_SUB(old_point,old_point,origin);
     SMGD_shear_point(old_point,s_value,axis_flag,new_point);
     V3D_ADD(new_point,new_point,origin);
     V3D_COPY(the_controlbox->vertex_list[index].point,new_point); 
}
/*---------------------------------------------------------*/
/* procedure for shifting the object                       */

void
SMGD_shift_point(old_point,s_value,axis_flag,new_point) 
VECTOR_3D old_point;
float s_value;
int axis_flag;
VECTOR_3D new_point;

{
        V3D_COPY(new_point,old_point);
        switch (axis_flag) {
         case 0 : 
            if (old_point[0] < 0) 
            new_point[0] = old_point[0] - (s_value) * old_point[0];
            else
            new_point[0] = old_point[0] + (s_value) * old_point[0];
         break;
         case 1 : 
            if (old_point[1] < 0) 
            new_point[1] = old_point[1] - (s_value) * old_point[1];
            else
            new_point[1] = old_point[1] + (s_value) * old_point[1];
         break;
         case 2 : 
            if (old_point[2] < 0) 
            new_point[2] = old_point[2] - (s_value) * old_point[2];
            else
            new_point[2] = old_point[2] + (s_value) * old_point[2];
         break;
        }

}

/**
**/

void
SMGD_shift_surface(the_surface,s_value,axis_flag)

SM_Surface_ptr the_surface;
float s_value;
int axis_flag;

{
   VECTOR_3D old_point,new_point;

   SM_FOR_EACH_VERTEX(the_surface)
      SM_vertex_point_get(the_surface,vertex_index,old_point);
      SMGD_shift_point(old_point,s_value,axis_flag,new_point);
      SM_vertex_point_set(the_surface,vertex_index,new_point);
   SM_END_FOR_EACH
}

/*
   long start_time;
   start_time = clock();
*/
/*
printf("shift time : %f ms.\n",(float)(clock() - start_time)/1000.0);
*/

void
SMGD_shift_face_region(the_surface,the_face,the_region_list,current_region,s_value,
                   axis_flag)
SM_Surface_ptr the_surface, the_face;
SM_Region_List_ptr the_region_list;
SM_Region_Name current_region;
float s_value;
int axis_flag;

{
   SM_Region_ptr the_region;
   SM_Region_Controlbox_ptr the_box;
   int the_index,v_index,l,m,n;
   VECTOR_3D old_point,new_point;
   BOOL anchor;
   VECTOR_3D origin_point;
   
   the_index = SM_region_index_get(the_region_list,current_region); 
    if (the_index != SM_INDEX_ERROR) {
     l = 2; m = 2; n = 2;
     the_box = SM_region_controlbox_new(l,m,n);
     SM_region_controlbox_recreate(the_box,l,m,n); 
     SM_region_controlbox_get(the_face,the_region_list,the_index,the_box);
     SMGD_controlbox_origin_get(the_box,origin_point);
     the_region = &the_region_list->region_list[the_index];
     SM_FOR_EACH_REGION_VERTEX(the_region) 
      v_index = region_vertex_ptr->id;
      anchor = region_vertex_ptr->anchor;
      if ((SM_vertex_index_valid(the_face,v_index)) && (!anchor) ) {
        SM_vertex_point_get(the_face,v_index,old_point);
        V3D_SUB(old_point,old_point,origin_point);
        SMGD_shift_point(old_point,s_value,axis_flag,new_point);
        V3D_ADD(new_point,new_point,origin_point);
        SM_vertex_point_set(the_surface,v_index,new_point);
      }
     SM_END_FOR_EACH
     SM_region_controlbox_dispose(the_box);
    }

}
 
/** 
 *
 */

void
SMGD_shift_region(the_surface,the_region_list,current_region,s_value,
                   axis_flag)
SM_Surface_ptr the_surface;
SM_Region_List_ptr the_region_list;
SM_Region_Name current_region;
float s_value;
int axis_flag;

{
   SM_Region_ptr the_region;
   SM_Region_Controlbox_ptr the_box;
   int the_index,v_index,l,m,n;
   VECTOR_3D old_point,new_point;
   BOOL anchor;
   VECTOR_3D origin_point;
   the_index = SM_region_index_get(the_region_list,current_region); 
    if (the_index != SM_INDEX_ERROR) {
     l = 2; m = 2; n = 2;
     the_box = SM_region_controlbox_new(l,m,n);
     SM_region_controlbox_recreate(the_box,l,m,n); 
     SM_region_controlbox_get(the_surface,the_region_list,the_index,the_box);
     SMGD_controlbox_origin_get(the_box,origin_point);
     the_region = &the_region_list->region_list[the_index];
     SM_FOR_EACH_REGION_VERTEX(the_region) 
      v_index = region_vertex_ptr->id;
      anchor = region_vertex_ptr->anchor;
      if ((SM_vertex_index_valid(the_surface,v_index)) && (!anchor) ) {
        SM_vertex_point_get(the_surface,v_index,old_point);
        V3D_SUB(old_point,old_point,origin_point);
        SMGD_shift_point(old_point,s_value,axis_flag,new_point);
        V3D_ADD(new_point,new_point,origin_point);
        SM_vertex_point_set(the_surface,v_index,new_point);
      }
     SM_END_FOR_EACH
     SM_region_controlbox_dispose(the_box);
    }

}

/*
 * 
 */

void
SMGD_shift_object_controlbox(the_controlbox,s_value,axis_flag)
SM_Surface_Controlbox_ptr the_controlbox;
float s_value;
int axis_flag;

{
      int i,number_of_points;
      VECTOR_3D old_point, new_point, max_point, min_point, origin_point;

      number_of_points = the_controlbox->l * the_controlbox->m * 
                         the_controlbox->n;
  
      SMGD_box_maxmin_point_get(the_controlbox,max_point,min_point);
      for (i=X_INDEX; i<=Z_INDEX; i++) 
        origin_point[i] = (max_point[i] + min_point[i])/2.0;
      for (i=0; i<number_of_points; i++) { 
       V3D_COPY(old_point,the_controlbox->vertex_list[i].point); 
       V3D_SUB(old_point,old_point,origin_point);
       SMGD_shift_point(old_point,s_value,axis_flag,new_point);
       V3D_ADD(new_point,new_point,origin_point);
       V3D_COPY(the_controlbox->vertex_list[i].point,new_point); 
      }
}
       
/**
**/

void
SMGD_shift_object_control_plane(the_controlbox,index,s_value,axis_flag,plane_dir)
SM_Surface_Controlbox_ptr the_controlbox;
int index;
float s_value;
int axis_flag;
int plane_dir;

{

     int i,j,k,ind_i,ind_j,ind_k,l,m,n,the_index;
     VECTOR_3D old_point, new_point, max_point, min_point, origin_point;

     SMGD_box_maxmin_point_get(the_controlbox,max_point,min_point);
      for (i=X_INDEX; i<=Z_INDEX; i++) 
        origin_point[i] = (max_point[i] + min_point[i])/2.0;

     l = the_controlbox->l;
     m = the_controlbox->m;
     n = the_controlbox->n;
     SMGD_controlbox_vertex_indices_get(the_controlbox,index,&ind_i,&ind_j,&ind_k);
     switch (plane_dir) {
        case 0 : 
         i = ind_i;
         for (k=0; k<n; k++) {        
          for (j=0; j<m; j++) {        
            the_index = k*(l*m) + j*l + i;
            V3D_COPY(old_point,the_controlbox->vertex_list[the_index].point); 
            V3D_SUB(old_point,old_point,origin_point);
            SMGD_shift_point(old_point,s_value,axis_flag,new_point);
            V3D_ADD(new_point,new_point,origin_point);
            V3D_COPY(the_controlbox->vertex_list[the_index].point,new_point); 
          }
         }
         break;
        case 1 : 
         j = ind_j;
         for (k=0; k<n; k++) {        
          for (i=0; i<l; i++) {        
            the_index = k*(l*m) + j*l + i;
            V3D_COPY(old_point,the_controlbox->vertex_list[the_index].point); 
            V3D_SUB(old_point,old_point,origin_point);
            SMGD_shift_point(old_point,s_value,axis_flag,new_point);
            V3D_ADD(new_point,new_point,origin_point);
            V3D_COPY(the_controlbox->vertex_list[the_index].point,new_point); 
          }
         }
         break;
        case 2 : 
         k = ind_k;
         for (j=0; j<m; j++) {        
          for (i=0; i<l; i++) {        
            the_index = k*(l*m) + j*l + i;
            V3D_COPY(old_point,the_controlbox->vertex_list[the_index].point); 
            V3D_SUB(old_point,old_point,origin_point);
            SMGD_shift_point(old_point,s_value,axis_flag,new_point);
            V3D_ADD(new_point,new_point,origin_point);
            V3D_COPY(the_controlbox->vertex_list[the_index].point,new_point); 
          }
         }
         break;
        default :
        break;
      } 
}

void
SMGD_shift_object_control_point(the_controlbox,index,s_value,axis_flag,origin)
SM_Surface_Controlbox_ptr the_controlbox;
int index;
float s_value;
int axis_flag;
VECTOR_3D origin;
{

     VECTOR_3D old_point, new_point;

     V3D_COPY(old_point,the_controlbox->vertex_list[index].point); 
     V3D_SUB(old_point,old_point,origin);
     SMGD_shift_point(old_point,s_value,axis_flag,new_point);
     V3D_ADD(new_point,new_point,origin);
     V3D_COPY(the_controlbox->vertex_list[index].point,new_point); 
}
/*------------------------------------------------------------*/
/* procedure for tapering the object                          */

void
SMGD_taper_point(old_point,min_value,max_value,
                 axis_flag,the_function,new_point)

VECTOR_3D old_point;
float min_value, max_value;
int axis_flag;
float  (*the_function) (float,float,float);
VECTOR_3D new_point;

{
  float x,y,z,r;

    switch (axis_flag) {
       case 0 : /*  for x */
               x = old_point[X_INDEX];
               y = old_point[Y_INDEX];
               z = old_point[Z_INDEX];
               r = (*the_function)(x,max_value,min_value);
               new_point[X_INDEX] = x;
               new_point[Y_INDEX] = r * y;
               new_point[Z_INDEX] = r * z;
              break;

       case 1 : /*  for y */
               x = old_point[X_INDEX];
               y = old_point[Y_INDEX];
               z = old_point[Z_INDEX];
               r = (*the_function)(y,max_value,min_value);
               new_point[X_INDEX] = r * x;
               new_point[Y_INDEX] = y;
               new_point[Z_INDEX] = r * z;
              break;

       case 2 : /*  for z */
               x = old_point[X_INDEX];
               y = old_point[Y_INDEX];
               z = old_point[Z_INDEX];
               r = (*the_function)(z,max_value,min_value);
               new_point[X_INDEX] = r * x;
               new_point[Y_INDEX] = r * y;
               new_point[Z_INDEX] = z;
              break;

       default : 
              break;
        } /* switch */

}

/**
**/

void
SMGD_taper_surface(the_surface,axis_flag,the_function) 
SM_Surface_ptr the_surface;
int axis_flag;
float (*the_function) (float,float,float);

{
 float min_value, max_value;
 VECTOR_3D old_point, new_point;

    min_value = SMGD_Box_get_mincoord(the_surface,axis_flag);
    max_value = SMGD_Box_get_maxcoord(the_surface,axis_flag);

   SM_FOR_EACH_VERTEX(the_surface)
      SM_vertex_point_get(the_surface,vertex_index,old_point);
      SMGD_taper_point(old_point,min_value,max_value,
                 axis_flag,the_function,new_point);
      SM_vertex_point_set(the_surface,vertex_index,new_point);
   SM_END_FOR_EACH
}

/**
**/

void
SMGD_taper_region(the_surface,the_region_list,current_region,
                              axis_flag,the_function)
SM_Surface_ptr the_surface;
SM_Region_List_ptr the_region_list;
SM_Region_Name current_region;
int axis_flag;
float (*the_function) (float,float,float);

{
   SM_Region_ptr the_region;
   SM_Region_Controlbox_ptr the_box;
   int the_index,v_index,l,m,n;
   VECTOR_3D old_point,new_point,origin_point;
   float min_value,max_value;
   BOOL anchor;

    the_index = SM_region_index_get(the_region_list,current_region); 
    if (the_index != SM_INDEX_ERROR) {
     SM_region_maxcoord_get(the_surface,the_region_list,
                            the_index,axis_flag,&max_value);
     SM_region_mincoord_get(the_surface,the_region_list,
                            the_index,axis_flag,&min_value);
     l = 2; m = 2; n = 2;
     the_box = SM_region_controlbox_new(l,m,n);
     SM_region_controlbox_recreate(the_box,l,m,n); 
     SM_region_controlbox_get(the_surface,the_region_list,the_index,the_box);
     SMGD_controlbox_origin_get(the_box,origin_point);
     the_region = &the_region_list->region_list[the_index];
     SM_FOR_EACH_REGION_VERTEX(the_region) 
      v_index = region_vertex_ptr->id;
      anchor = region_vertex_ptr->anchor;
      if ((SM_vertex_index_valid(the_surface,v_index)) && (!anchor) ) {
        SM_vertex_point_get(the_surface,v_index,old_point);
        V3D_SUB(old_point,old_point,origin_point);
        SMGD_taper_point(old_point,min_value,max_value,
                 axis_flag,the_function,new_point);
        V3D_ADD(new_point,new_point,origin_point);
        SM_vertex_point_set(the_surface,v_index,new_point);
      }
     SM_END_FOR_EACH
     SM_region_controlbox_dispose(the_box);
    }
}

/**
**/

void
SMGD_taper_object_controlbox(the_controlbox,axis_flag,the_function)
SM_Surface_Controlbox_ptr the_controlbox;
int axis_flag;
float  (*the_function) (float,float,float);

{
      int i,number_of_points;
      VECTOR_3D old_point, new_point,
                max_point, min_point, origin_point;
      float min_value, max_value;

      number_of_points = the_controlbox->l * the_controlbox->m * 
                         the_controlbox->n;
      SMGD_box_maxmin_point_get(the_controlbox,max_point,min_point);
      for (i=X_INDEX; i<=Z_INDEX; i++) 
        origin_point[i] = (max_point[i] + min_point[i])/2.0;
      min_value = min_point[axis_flag] - origin_point[axis_flag]; 
      max_value = max_point[axis_flag] - origin_point[axis_flag]; 
      for (i=0; i<number_of_points; i++) { 
       V3D_COPY(old_point,the_controlbox->vertex_list[i].point); 
       V3D_SUB(old_point,old_point,origin_point);
       SMGD_taper_point(old_point,min_value,max_value,axis_flag,
                        the_function,new_point);
       V3D_ADD(new_point,new_point,origin_point);
       V3D_COPY(the_controlbox->vertex_list[i].point,new_point); 
      }
}
       
/**
**/

void
SMGD_taper_object_control_plane(the_controlbox,index,axis_flag,the_function,plane_dir)
SM_Surface_Controlbox_ptr the_controlbox;
int index;
int axis_flag;
float (*the_function) (float,float,float);
int plane_dir;

{

     int i,j,k,ind_i,ind_j,ind_k,l,m,n,the_index;
     VECTOR_3D old_point, new_point, max_point, min_point, origin_point;
     float min_value, max_value;

     SMGD_box_maxmin_point_get(the_controlbox,max_point,min_point);
      for (i=X_INDEX; i<=Z_INDEX; i++) 
        origin_point[i] = (max_point[i] + min_point[i])/2.0;

     min_value = min_point[axis_flag] - origin_point[axis_flag]; 
     max_value = max_point[axis_flag] - origin_point[axis_flag]; 
     l = the_controlbox->l;
     m = the_controlbox->m;
     n = the_controlbox->n;
     SMGD_controlbox_vertex_indices_get(the_controlbox,index,&ind_i,&ind_j,&ind_k);
     switch (plane_dir) {
        case 0 : 
         i = ind_i;
         for (k=0; k<n; k++) {        
          for (j=0; j<m; j++) {        
            the_index = k*(l*m) + j*l + i;
            V3D_COPY(old_point,the_controlbox->vertex_list[the_index].point); 
            V3D_SUB(old_point,old_point,origin_point);
            SMGD_taper_point(old_point,min_value,max_value,axis_flag,
                        the_function,new_point);
            V3D_ADD(new_point,new_point,origin_point);
            V3D_COPY(the_controlbox->vertex_list[the_index].point,new_point); 
          }
         }
         break;
        case 1 : 
         j = ind_j;
         for (k=0; k<n; k++) {        
          for (i=0; i<l; i++) {        
            the_index = k*(l*m) + j*l + i;
            V3D_COPY(old_point,the_controlbox->vertex_list[the_index].point); 
            V3D_SUB(old_point,old_point,origin_point);
            SMGD_taper_point(old_point,min_value,max_value,axis_flag,
                        the_function,new_point);
            V3D_ADD(new_point,new_point,origin_point);
            V3D_COPY(the_controlbox->vertex_list[the_index].point,new_point); 
          }
         }
         break;
        case 2 : 
         k = ind_k;
         for (j=0; j<m; j++) {        
          for (i=0; i<l; i++) {        
            the_index = k*(l*m) + j*l + i;
            V3D_COPY(old_point,the_controlbox->vertex_list[the_index].point); 
            V3D_SUB(old_point,old_point,origin_point);
            SMGD_taper_point(old_point,min_value,max_value,axis_flag,
                        the_function,new_point);
            V3D_ADD(new_point,new_point,origin_point);
            V3D_COPY(the_controlbox->vertex_list[the_index].point,new_point); 
          }
         }
         break;
        default :
        break;
      } 
}


void
SMGD_taper_object_control_point(the_controlbox,index,axis_flag,the_function,origin)
SM_Surface_Controlbox_ptr the_controlbox;
int index;
int axis_flag;
float (*the_function) (float,float,float);
VECTOR_3D origin;
{

     VECTOR_3D old_point, new_point;
     float min_value = 0.0, max_value = 0.0;

     V3D_COPY(old_point,the_controlbox->vertex_list[index].point); 
     V3D_SUB(old_point,old_point,origin);
     SMGD_taper_point(old_point,min_value,max_value,axis_flag,
                 the_function,new_point);
     V3D_ADD(new_point,new_point,origin);
     V3D_COPY(the_controlbox->vertex_list[index].point,new_point); 
}
/*------------------------------------------------------------*/
/* procedure for twisting the object                          */


void
SMGD_twist_point(old_point,min_value,max_value,axis_flag,the_function,new_point)

VECTOR_3D old_point, new_point;
int axis_flag;
float (*the_function) (float,float,float);
float min_value,max_value;

{
  float x,y,z,r;
  float C_theta,S_theta;

    switch (axis_flag) {
       case 0 : /*  for x */
                x = old_point[X_INDEX];
                y = old_point[Y_INDEX];
                z = old_point[Z_INDEX];
                r = (*the_function)(x,max_value,min_value);
                C_theta = cos(r);
                S_theta = sin(r);
                new_point[X_INDEX] = x;
                new_point[Y_INDEX] = y * C_theta - z * S_theta;
                new_point[Z_INDEX] = y * S_theta + z * S_theta;
               break;

       case 1 : /*  for y */
                x = old_point[X_INDEX];
                y = old_point[Y_INDEX];
                z = old_point[Z_INDEX];
                r = (*the_function)(y,max_value,min_value);
                C_theta = cos(r);
                S_theta = sin(r);
                new_point[X_INDEX] = x * C_theta + z * S_theta;
                new_point[Y_INDEX] = y;
                new_point[Z_INDEX] = -x * S_theta + z * C_theta;
               break;

       case 2 : /*  for z */
                x = old_point[X_INDEX];
                y = old_point[Y_INDEX];
                z = old_point[Z_INDEX];
                r = (*the_function)(z,max_value,min_value);
                C_theta = cos(r);
                S_theta = sin(r);
                new_point[X_INDEX] = x * C_theta - y * S_theta;
                new_point[Y_INDEX] = x * S_theta + y * C_theta;
                new_point[Z_INDEX] = z;
               break;

       default : 
               break;
        } /* switch */

}

void
SMGD_twist_surface(the_surface,axis_flag,the_function)

SM_Surface_ptr the_surface;
int axis_flag;
float (*the_function) (float,float,float);
  
{
 float min_value, max_value;
 VECTOR_3D old_point, new_point;

    min_value = SMGD_Box_get_mincoord(the_surface,axis_flag);
    max_value = SMGD_Box_get_maxcoord(the_surface,axis_flag);

   SM_FOR_EACH_VERTEX(the_surface)
      SM_vertex_point_get(the_surface,vertex_index,old_point);
      SMGD_twist_point(old_point,min_value,max_value,
                 axis_flag,the_function,new_point);
      SM_vertex_point_set(the_surface,vertex_index,new_point);
   SM_END_FOR_EACH
}

void
SMGD_twist_region(the_surface,the_region_list,current_region,
                              axis_flag,the_function)
SM_Surface_ptr the_surface;
SM_Region_List_ptr the_region_list;
SM_Region_Name current_region;
int axis_flag;
float (*the_function) (float,float,float);

{
   SM_Region_ptr the_region;
   SM_Region_Controlbox_ptr the_box;
   int the_index,v_index,l,m,n;
   VECTOR_3D old_point,new_point,origin_point;
   float min_value,max_value;
   BOOL anchor;

    the_index = SM_region_index_get(the_region_list,current_region); 
    if (the_index != SM_INDEX_ERROR) {
     SM_region_maxcoord_get(the_surface,the_region_list,
                            the_index,axis_flag,&max_value);
     SM_region_mincoord_get(the_surface,the_region_list,
                            the_index,axis_flag,&min_value);
     l = 2; m = 2; n = 2;
     the_box = SM_region_controlbox_new(l,m,n);
     SM_region_controlbox_recreate(the_box,l,m,n); 
     SM_region_controlbox_get(the_surface,the_region_list,the_index,the_box);
     SMGD_controlbox_origin_get(the_box,origin_point);
     the_region = &the_region_list->region_list[the_index];
     SM_FOR_EACH_REGION_VERTEX(the_region) 
      v_index = region_vertex_ptr->id;
      anchor = region_vertex_ptr->anchor;
      if ((SM_vertex_index_valid(the_surface,v_index)) && (!anchor) ) {
        SM_vertex_point_get(the_surface,v_index,old_point);
        V3D_SUB(old_point,old_point,origin_point);
        SMGD_twist_point(old_point,min_value,max_value,
                 axis_flag,the_function,new_point);
        V3D_ADD(new_point,new_point,origin_point);
        SM_vertex_point_set(the_surface,v_index,new_point);
      }
     SM_END_FOR_EACH
     SM_region_controlbox_dispose(the_box);
    }
}

void
SMGD_twist_object_controlbox(the_controlbox,axis_flag,the_function)
SM_Surface_Controlbox_ptr the_controlbox;
int axis_flag;
float (*the_function) (float,float,float);

{
      int i,number_of_points;
      VECTOR_3D old_point, new_point,
                max_point, min_point, origin_point;
      float min_value, max_value;

      number_of_points = the_controlbox->l * the_controlbox->m * 
                         the_controlbox->n;
      SMGD_box_maxmin_point_get(the_controlbox,max_point,min_point);
      for (i=X_INDEX; i<=Z_INDEX; i++) 
        origin_point[i] = (max_point[i] + min_point[i])/2.0;
      min_value = min_point[axis_flag] - origin_point[axis_flag]; 
      max_value = max_point[axis_flag] - origin_point[axis_flag]; 
      for (i=0; i<number_of_points; i++) { 
       V3D_COPY(old_point,the_controlbox->vertex_list[i].point); 
       V3D_SUB(old_point,old_point,origin_point);
       SMGD_twist_point(old_point,min_value,max_value,axis_flag,
                        the_function,new_point);
       V3D_ADD(new_point,new_point,origin_point);
       V3D_COPY(the_controlbox->vertex_list[i].point,new_point); 
      }
}
       
/**
**/

void
SMGD_twist_object_control_plane(the_controlbox,index,axis_flag,the_function,plane_dir)
SM_Surface_Controlbox_ptr the_controlbox;
int index;
int axis_flag;
float (*the_function) (float,float,float);
int plane_dir;

{

     int i,j,k,ind_i,ind_j,ind_k,l,m,n,the_index;
     VECTOR_3D old_point, new_point, max_point, min_point, origin_point;
     float min_value, max_value;

     SMGD_box_maxmin_point_get(the_controlbox,max_point,min_point);
      for (i=X_INDEX; i<=Z_INDEX; i++) 
        origin_point[i] = (max_point[i] + min_point[i])/2.0;
      min_value = min_point[axis_flag] - origin_point[axis_flag]; 
      max_value = max_point[axis_flag] - origin_point[axis_flag]; 

     l = the_controlbox->l;
     m = the_controlbox->m;
     n = the_controlbox->n;
     SMGD_controlbox_vertex_indices_get(the_controlbox,index,&ind_i,&ind_j,&ind_k);
     switch (plane_dir) {
        case 0 : 
         i = ind_i;
         for (k=0; k<n; k++) {        
          for (j=0; j<m; j++) {        
            the_index = k*(l*m) + j*l + i;
            V3D_COPY(old_point,the_controlbox->vertex_list[the_index].point); 
            V3D_SUB(old_point,old_point,origin_point);
            SMGD_twist_point(old_point,min_value,max_value,axis_flag,
                        the_function,new_point);
            V3D_ADD(new_point,new_point,origin_point);
            V3D_COPY(the_controlbox->vertex_list[the_index].point,new_point); 
          }
         }
         break;
        case 1 : 
         j = ind_j;
         for (k=0; k<n; k++) {        
          for (i=0; i<l; i++) {        
            the_index = k*(l*m) + j*l + i;
            V3D_COPY(old_point,the_controlbox->vertex_list[the_index].point); 
            V3D_SUB(old_point,old_point,origin_point);
            SMGD_twist_point(old_point,min_value,max_value,axis_flag,
                        the_function,new_point);
            V3D_ADD(new_point,new_point,origin_point);
            V3D_COPY(the_controlbox->vertex_list[the_index].point,new_point); 
          }
         }
         break;
        case 2 : 
         k = ind_k;
         for (j=0; j<m; j++) {        
          for (i=0; i<l; i++) {        
            the_index = k*(l*m) + j*l + i;
            V3D_COPY(old_point,the_controlbox->vertex_list[the_index].point); 
            V3D_SUB(old_point,old_point,origin_point);
            SMGD_twist_point(old_point,min_value,max_value,axis_flag,
                        the_function,new_point);
            V3D_ADD(new_point,new_point,origin_point);
            V3D_COPY(the_controlbox->vertex_list[the_index].point,new_point); 
          }
         }
         break;
        default :
        break;
      }
}

void
SMGD_twist_object_control_point(the_controlbox,index,axis_flag,the_function,origin)
SM_Surface_Controlbox_ptr the_controlbox;
int index;
int axis_flag;
float (*the_function) (float,float,float);
VECTOR_3D origin;
{

     VECTOR_3D old_point, new_point;
     float min_value, max_value;

     V3D_COPY(old_point,the_controlbox->vertex_list[index].point); 
     V3D_SUB(old_point,old_point,origin);
     SMGD_twist_point(old_point,min_value,max_value,axis_flag,
                 the_function,new_point);
     V3D_ADD(new_point,new_point,origin);
     V3D_COPY(the_controlbox->vertex_list[index].point,new_point); 
}
/*----------------------------------------------------------------*/

void
SMGD_get_bending_parameters(mn_value,mx_value,bend_angle,pt1,pt2,
                            min_value,max_value,curvature)
float mx_value, mn_value, bend_angle;
VECTOR_2D pt1,pt2;
float *max_value,
      *min_value,
      *curvature;
{
   float fmin, fmax, local_EPSILON;

    fmin = pt1[0]*(mx_value-mn_value) + mn_value;
    fmax = pt2[0]*(mx_value-mn_value) + mn_value;
    local_EPSILON = 0.0001 * fabs(mx_value-mn_value);

    local_EPSILON = 0.0001 * fabs(mx_value-mn_value);
    if  ( (fabs(fmax-fmin) < local_EPSILON) || 
          (fabs(bend_angle) < SM_EPSILON))  
     *curvature = 0.0001;
    else
     *curvature = bend_angle/fabs(fmax - fmin); 

    *min_value = fmin;
    *max_value = fmax;
}  

/*-------------------------------------------------------------*/

void
SMGD_bend_point(old_point,min_value,max_value,axis_flag,curvature,
                new_point)

VECTOR_3D old_point,new_point;
float min_value,max_value,curvature;
int axis_flag;

{

  float x,y,z;
  float k,k_cap,y0,y_cap,x0,
        x_cap,z0,z_cap,theta,
        C_theta,S_theta;
  int s_flag;

    k = curvature;

    switch (axis_flag) {
         case 0 :  /* for x */
                  x = old_point[X_INDEX];
                  y = old_point[Y_INDEX];
                  z = old_point[Z_INDEX];
                   x_cap = x;
                   k_cap = k;
                   x0 = (max_value - min_value)/2.0;
                   s_flag = 1;
                   if (x < min_value) {
                     x_cap = min_value;
                     k_cap = 0;
                     s_flag = 2;
                    }
                   if (x > max_value) {
                     x_cap = max_value;
                     k_cap = 0;
                     s_flag = 3;
                    }
                    theta = k * (x_cap - x0);
                    C_theta = cos(theta);
                    S_theta = sin(theta);
                    switch (s_flag) {
                       case 1 :
                              new_point[X_INDEX] = -S_theta*(y-1/k) + x0;
                              new_point[Y_INDEX] = C_theta*(y-1/k) + 1/k;
                              new_point[Z_INDEX] = z;
                              break;
                       case 2 :
                              new_point[X_INDEX] = -S_theta*(y-1/k) + x0 +
                                                   C_theta*(x-min_value);
                              new_point[Y_INDEX] = C_theta*(y-1/k) + 1/k +
                                                   S_theta*(x-min_value);
                              new_point[Z_INDEX] = z;
                              break;
                       case 3 :
                              new_point[X_INDEX] = -S_theta*(y-1/k) + x0 +
                                                   C_theta*(x-max_value);
                              new_point[Y_INDEX] = C_theta*(y-1/k) + 1/k +
                                                   S_theta*(x-max_value);
                              new_point[Z_INDEX] = z;
                              break;
                        default :  
                              break;
                      } /* switch */
                   break;
         case 1 :  /* for y */
                  x = old_point[X_INDEX];
                  y = old_point[Y_INDEX];
                  z = old_point[Z_INDEX];
                   y_cap = y;
                   k_cap = k;
                   y0 = (max_value - min_value)/2.0;
                   s_flag = 1;
                   if (y < min_value) {
                     y_cap = min_value;
                     k_cap = 0;
                     s_flag = 2;
                    }
                   if (y > max_value) {
                     y_cap = max_value;
                     k_cap = 0;
                     s_flag = 3;
                    }
                    theta = k * (y_cap - y0);
                    C_theta = cos(theta);
                    S_theta = sin(theta);
                    switch (s_flag) {
                       case 1 :
                              new_point[X_INDEX] = x;
                              new_point[Y_INDEX] = -S_theta*(z-1/k) + y0;
                              new_point[Z_INDEX] = C_theta*(z-1/k) + 1/k;
                              break;
                       case 2 :
                              new_point[X_INDEX] = x;
                              new_point[Y_INDEX] = -S_theta*(z-1/k) + y0 +
                                                   C_theta*(y-min_value);
                              new_point[Z_INDEX] = C_theta*(z-1/k) + 1/k +
                                                   S_theta*(y-min_value);
                              break;
                       case 3 :
                              new_point[X_INDEX] = x;
                              new_point[Y_INDEX] = -S_theta*(z-1/k) + y0 +
                                                   C_theta*(y-max_value);
                              new_point[Z_INDEX] = C_theta*(z-1/k) + 1/k +
                                                   S_theta*(y-max_value);
                              break;
                        default :  
                              break;
                      } /* switch */
                     break;
         case 2 :  /* for z */
                  x = old_point[X_INDEX];
                  y = old_point[Y_INDEX];
                  z = old_point[Z_INDEX];
                   z_cap = z;
                   k_cap = k;
                   z0 = (max_value - min_value)/2.0;
                   s_flag = 1;
                   if (z < min_value) {
                     z_cap = min_value;
                     k_cap = 0;
                     s_flag = 2;
                    }
                   if (z > max_value) {
                     z_cap = max_value;
                     k_cap = 0;
                     s_flag = 3;
                    }
                    theta = k * (z_cap - z0);
                    C_theta = cos(theta);
                    S_theta = sin(theta);
                    switch (s_flag) {
                       case 1 :
                              new_point[X_INDEX] = C_theta*(x-1/k) + 1/k;
                              new_point[Y_INDEX] = y;
                              new_point[Z_INDEX] = -S_theta*(x-1/k) + z0;
                              break;
                       case 2 :
                              new_point[X_INDEX] = C_theta*(x-1/k) + 1/k + 
                                                   S_theta*(z-min_value);
                              new_point[Y_INDEX] = y;
                              new_point[Z_INDEX] = -S_theta*(x-1/k) + z0 +
                                                   C_theta*(z-min_value);
                              break;
                       case 3 :
                              new_point[X_INDEX] = C_theta*(x-1/k) + 1/k + 
                                                   S_theta*(z-max_value);
                              new_point[Y_INDEX] = y;
                              new_point[Z_INDEX] = -S_theta*(x-1/k) + z0 +
                                                   C_theta*(z-max_value);
                              break;
                        default :  
                              break;
                      } /* switch */
                     break;
                    default :  
                          break;
                   } /* switch outer */

}

void
SMGD_bend_surface(the_surface,axis_flag,bend_angle,pt1,pt2)
SM_Surface_ptr the_surface;
int axis_flag;
float bend_angle;
VECTOR_2D pt1,pt2;

{
  VECTOR_3D old_point,new_point;
  float mn_value,mx_value,min_value,max_value,curvature;

  mn_value = SMGD_Box_get_mincoord(the_surface,axis_flag);
  mx_value = SMGD_Box_get_maxcoord(the_surface,axis_flag);

  SMGD_get_bending_parameters(mn_value,mx_value,bend_angle,pt1,pt2,
                              &min_value,&max_value,&curvature);
  
   SM_FOR_EACH_VERTEX(the_surface)
      SM_vertex_point_get(the_surface,vertex_index,old_point);
      SMGD_bend_point(old_point,min_value,max_value,
                 axis_flag,curvature,new_point);
      SM_vertex_point_set(the_surface,vertex_index,new_point);
   SM_END_FOR_EACH
}


void
SMGD_bend_region(the_surface,the_region_list,current_region,
                              axis_flag,bend_angle,pt1,pt2)
SM_Surface_ptr the_surface;
SM_Region_List_ptr the_region_list;
SM_Region_Name current_region;
int axis_flag;
float bend_angle;
VECTOR_2D pt1, pt2;

{
   SM_Region_ptr the_region;
   SM_Region_Controlbox_ptr the_box;
   int the_index,v_index,l,m,n;
   VECTOR_3D old_point,new_point,origin_point;
   float mn_value,mx_value,min_value,max_value,curvature;
   BOOL anchor;

    the_index = SM_region_index_get(the_region_list,current_region); 
    if (the_index != SM_INDEX_ERROR) {
     SM_region_maxcoord_get(the_surface,the_region_list,
                            the_index,axis_flag,&mx_value);
     SM_region_mincoord_get(the_surface,the_region_list,
                            the_index,axis_flag,&mn_value);
     l = 2; m = 2; n = 2;
     the_box = SM_region_controlbox_new(l,m,n);
     SM_region_controlbox_recreate(the_box,l,m,n); 
     SM_region_controlbox_get(the_surface,the_region_list,the_index,the_box);
     SMGD_controlbox_origin_get(the_box,origin_point);
     the_region = &the_region_list->region_list[the_index];
     SMGD_get_bending_parameters(mn_value,mx_value,bend_angle,pt1,pt2,
                              &min_value,&max_value,&curvature);
     SM_FOR_EACH_REGION_VERTEX(the_region) 
      v_index = region_vertex_ptr->id;
      anchor = region_vertex_ptr->anchor;
      if ((SM_vertex_index_valid(the_surface,v_index)) && (!anchor) ) {
        SM_vertex_point_get(the_surface,v_index,old_point);
        V3D_SUB(old_point,old_point,origin_point);
        SMGD_bend_point(old_point,min_value,max_value,
                 axis_flag,curvature,new_point);
        V3D_ADD(new_point,new_point,origin_point);
        SM_vertex_point_set(the_surface,v_index,new_point);
      }
     SM_END_FOR_EACH
     SM_region_controlbox_dispose(the_box);
    }
}

void
SMGD_bend_object_controlbox(the_controlbox,axis_flag,bend_angle,pt1,pt2)
SM_Surface_Controlbox_ptr the_controlbox;
int axis_flag;
float bend_angle;
VECTOR_2D pt1,pt2;

{
      int i,number_of_points;
      VECTOR_3D old_point, new_point,
                max_point, min_point, origin_point;
      float mn_value,mx_value,min_value, max_value,curvature;

      number_of_points = the_controlbox->l * the_controlbox->m * 
                         the_controlbox->n;
      SMGD_box_maxmin_point_get(the_controlbox,max_point,min_point);
      for (i=X_INDEX; i<=Z_INDEX; i++) 
        origin_point[i] = (max_point[i] + min_point[i])/2.0;
      mn_value = min_point[axis_flag] - origin_point[axis_flag]; 
      mx_value = max_point[axis_flag] - origin_point[axis_flag]; 

      SMGD_get_bending_parameters(mn_value,mx_value,bend_angle,pt1,pt2,
                              &min_value,&max_value,&curvature);
      for (i=0; i<number_of_points; i++) { 
       V3D_COPY(old_point,the_controlbox->vertex_list[i].point); 
       V3D_SUB(old_point,old_point,origin_point);
       SMGD_bend_point(old_point,min_value,max_value,axis_flag,
                        curvature,new_point);
       V3D_ADD(new_point,new_point,origin_point);
       V3D_COPY(the_controlbox->vertex_list[i].point,new_point); 
      }
}
       
/**
**/

void
SMGD_bend_object_control_plane(the_controlbox,index,axis_flag,
                               bend_angle,pt1,pt2,plane_dir)
SM_Surface_Controlbox_ptr the_controlbox;
int index;
int axis_flag;
float bend_angle;
VECTOR_2D pt1, pt2;
int plane_dir;

{

     int i,j,k,ind_i,ind_j,ind_k,l,m,n,the_index;
     VECTOR_3D old_point, new_point, max_point, min_point, origin_point;
     float mn_value,mx_value,min_value, max_value,curvature;

     SMGD_box_maxmin_point_get(the_controlbox,max_point,min_point);
      for (i=X_INDEX; i<=Z_INDEX; i++) 
        origin_point[i] = (max_point[i] + min_point[i])/2.0;

     mn_value = min_point[axis_flag] - origin_point[axis_flag]; 
     mx_value = max_point[axis_flag] - origin_point[axis_flag]; 
     SMGD_get_bending_parameters(mn_value,mx_value,bend_angle,pt1,pt2,
                              &min_value,&max_value,&curvature);
     l = the_controlbox->l;
     m = the_controlbox->m;
     n = the_controlbox->n;
     SMGD_controlbox_vertex_indices_get(the_controlbox,index,&ind_i,&ind_j,&ind_k);
     switch (plane_dir) {
        case 0 : 
         i = ind_i;
         for (k=0; k<n; k++) {        
          for (j=0; j<m; j++) {        
            the_index = k*(l*m) + j*l + i;
            V3D_COPY(old_point,the_controlbox->vertex_list[the_index].point); 
            V3D_SUB(old_point,old_point,origin_point);
            SMGD_bend_point(old_point,min_value,max_value,axis_flag,
                        curvature,new_point);
            V3D_ADD(new_point,new_point,origin_point);
            V3D_COPY(the_controlbox->vertex_list[the_index].point,new_point); 
          }
         }
         break;
        case 1 : 
         j = ind_j;
         for (k=0; k<n; k++) {        
          for (i=0; i<l; i++) {        
            the_index = k*(l*m) + j*l + i;
            V3D_COPY(old_point,the_controlbox->vertex_list[the_index].point); 
            V3D_SUB(old_point,old_point,origin_point);
            SMGD_bend_point(old_point,min_value,max_value,axis_flag,
                        curvature,new_point);
            V3D_ADD(new_point,new_point,origin_point);
            V3D_COPY(the_controlbox->vertex_list[the_index].point,new_point); 
          }
         }
         break;
        case 2 : 
         k = ind_k;
         for (j=0; j<m; j++) {        
          for (i=0; i<l; i++) {        
            the_index = k*(l*m) + j*l + i;
            V3D_COPY(old_point,the_controlbox->vertex_list[the_index].point); 
            V3D_SUB(old_point,old_point,origin_point);
            SMGD_bend_point(old_point,min_value,max_value,axis_flag,
                        curvature,new_point);
            V3D_ADD(new_point,new_point,origin_point);
            V3D_COPY(the_controlbox->vertex_list[the_index].point,new_point); 
          }
         }
         break;
        default :
        break;
      }
}

void
SMGD_bend_object_control_point(the_controlbox,index,axis_flag,
                               bend_angle,pt1,pt2,origin)
SM_Surface_Controlbox_ptr the_controlbox;
int index;
int axis_flag;
float bend_angle;
VECTOR_2D pt1, pt2;
VECTOR_3D origin;
{

     int i;
     VECTOR_3D old_point, new_point, max_point, min_point;
     float mn_value,mx_value,min_value,max_value,curvature;

     SMGD_box_maxmin_point_get(the_controlbox,max_point,min_point);
      for (i=X_INDEX; i<=Z_INDEX; i++) 
        origin[i] = (max_point[i] + min_point[i])/2.0;

     mn_value = min_point[axis_flag] - origin[axis_flag]; 
     mx_value = max_point[axis_flag] - origin[axis_flag]; 
     SMGD_get_bending_parameters(mn_value,mx_value,bend_angle,pt1,pt2,
                              &min_value,&max_value,&curvature);

     V3D_COPY(old_point,the_controlbox->vertex_list[index].point); 
     V3D_SUB(old_point,old_point,origin);
     SMGD_bend_point(old_point,min_value,max_value,axis_flag,
                 curvature,new_point);
     V3D_ADD(new_point,new_point,origin);
     V3D_COPY(the_controlbox->vertex_list[index].point,new_point); 
}

/*   stretch */

void
SMGD_stretch_point(old_point,s_value,axis_flag,new_point)
VECTOR_3D old_point; 
float s_value;
int axis_flag;
VECTOR_3D new_point;
{
     V3D_COPY(new_point,old_point);
     new_point[axis_flag] = old_point[axis_flag] + 
                            fabs(s_value) * old_point[axis_flag]; 

}

/**
**/

void
SMGD_stretch_surface(the_surface,s_value,axis_flag)

SM_Surface_ptr the_surface;
float s_value;
int axis_flag;

{
   VECTOR_3D old_point,new_point;

   SM_FOR_EACH_VERTEX(the_surface)
      SM_vertex_point_get(the_surface,vertex_index,old_point);
      SMGD_stretch_point(old_point,s_value,axis_flag,new_point);
      SM_vertex_point_set(the_surface,vertex_index,new_point);
   SM_END_FOR_EACH
}

/**
**/

void
SMGD_stretch_region(the_surface,the_region_list,current_region,s_value,
                   axis_flag)
SM_Surface_ptr the_surface;
SM_Region_List_ptr the_region_list;
SM_Region_Name current_region;
float s_value;
int axis_flag;

{
   SM_Region_ptr the_region;
   SM_Region_Controlbox_ptr the_box;
   int the_index,v_index,l,m,n;
   VECTOR_3D old_point,new_point,origin_point;
   BOOL anchor;

    the_index = SM_region_index_get(the_region_list,current_region); 
    if (the_index != SM_INDEX_ERROR) {
     l = 2; m = 2; n = 2;
     the_box = SM_region_controlbox_new(l,m,n);
     SM_region_controlbox_recreate(the_box,l,m,n); 
     SM_region_controlbox_get(the_surface,the_region_list,the_index,the_box);
     SMGD_controlbox_origin_get(the_box,origin_point);
     the_region = &the_region_list->region_list[the_index];
     SM_FOR_EACH_REGION_VERTEX(the_region) 
      v_index = region_vertex_ptr->id;
      anchor = region_vertex_ptr->anchor;
      if ((SM_vertex_index_valid(the_surface,v_index)) && (!anchor) ) {
        SM_vertex_point_get(the_surface,v_index,old_point);
        V3D_SUB(old_point,old_point,origin_point);
        SMGD_stretch_point(old_point,s_value,axis_flag,new_point);
        V3D_ADD(new_point,new_point,origin_point);
        SM_vertex_point_set(the_surface,v_index,new_point);
      }
     SM_END_FOR_EACH
     SM_region_controlbox_dispose(the_box);
    }
}

/**
**/

void
SMGD_stretch_object_controlbox(the_controlbox,s_value,axis_flag)
SM_Surface_Controlbox_ptr the_controlbox;
float s_value;
int axis_flag;

{
      int i,number_of_points;
      VECTOR_3D old_point, new_point, max_point, min_point, origin_point;

      number_of_points = the_controlbox->l * the_controlbox->m * 
                         the_controlbox->n;
  
      SMGD_box_maxmin_point_get(the_controlbox,max_point,min_point);
      for (i=X_INDEX; i<=Z_INDEX; i++) {
        if (s_value > 0) 
          origin_point[i] = min_point[i];
        else
          origin_point[i] = max_point[i];
      }
      for (i=0; i<number_of_points; i++) { 
       V3D_COPY(old_point,the_controlbox->vertex_list[i].point); 
       V3D_SUB(old_point,old_point,origin_point);
       SMGD_stretch_point(old_point,s_value,axis_flag,new_point);
       V3D_ADD(new_point,new_point,origin_point);
       V3D_COPY(the_controlbox->vertex_list[i].point,new_point); 
      }
}
       
/**
**/

void
SMGD_stretch_object_control_plane(the_controlbox,index,s_value,axis_flag,plane_dir)
SM_Surface_Controlbox_ptr the_controlbox;
int index;
float s_value;
int axis_flag;
int plane_dir;

{

     int i,j,k,ind_i,ind_j,ind_k,l,m,n,the_index;
     VECTOR_3D old_point, new_point, max_point, min_point, origin_point;

     SMGD_box_maxmin_point_get(the_controlbox,max_point,min_point);
      for (i=X_INDEX; i<=Z_INDEX; i++) {
        if (s_value > 0) 
          origin_point[i] = min_point[i];
        else
          origin_point[i] = max_point[i];
      }

     l = the_controlbox->l;
     m = the_controlbox->m;
     n = the_controlbox->n;
     SMGD_controlbox_vertex_indices_get(the_controlbox,index,&ind_i,&ind_j,&ind_k);
     switch (plane_dir) {
        case 0 : 
         i = ind_i;
         for (k=0; k<n; k++) {        
          for (j=0; j<m; j++) {        
            the_index = k*(l*m) + j*l + i;
            V3D_COPY(old_point,the_controlbox->vertex_list[the_index].point); 
            V3D_SUB(old_point,old_point,origin_point);
            SMGD_stretch_point(old_point,s_value,axis_flag,new_point);
            V3D_ADD(new_point,new_point,origin_point);
            V3D_COPY(the_controlbox->vertex_list[the_index].point,new_point); 
          }
         }
         break;
        case 1 : 
         j = ind_j;
         for (k=0; k<n; k++) {        
          for (i=0; i<l; i++) {        
            the_index = k*(l*m) + j*l + i;
            V3D_COPY(old_point,the_controlbox->vertex_list[the_index].point); 
            V3D_SUB(old_point,old_point,origin_point);
            SMGD_stretch_point(old_point,s_value,axis_flag,new_point);
            V3D_ADD(new_point,new_point,origin_point);
            V3D_COPY(the_controlbox->vertex_list[the_index].point,new_point); 
          }
         }
         break;
        case 2 : 
         k = ind_k;
         for (j=0; j<m; j++) {        
          for (i=0; i<l; i++) {        
            the_index = k*(l*m) + j*l + i;
            V3D_COPY(old_point,the_controlbox->vertex_list[the_index].point); 
            V3D_SUB(old_point,old_point,origin_point);
            SMGD_stretch_point(old_point,s_value,axis_flag,new_point);
            V3D_ADD(new_point,new_point,origin_point);
            V3D_COPY(the_controlbox->vertex_list[the_index].point,new_point); 
          }
         }
         break;
        default :
        break;
      } 
}

/*   compress */

void
SMGD_compress_point(old_point,s_value,axis_flag,new_point)
VECTOR_3D old_point; 
float s_value;
int axis_flag;
VECTOR_3D new_point;
{
     V3D_COPY(new_point,old_point);
     new_point[axis_flag] = old_point[axis_flag] - 
                            fabs(s_value) * old_point[axis_flag]; 

}


/**
**/

void
SMGD_compress_surface(the_surface,s_value,axis_flag)

SM_Surface_ptr the_surface;
float s_value;
int axis_flag;

{
   VECTOR_3D old_point,new_point;

   SM_FOR_EACH_VERTEX(the_surface)
      SM_vertex_point_get(the_surface,vertex_index,old_point);
      SMGD_compress_point(old_point,s_value,axis_flag,new_point);
      SM_vertex_point_set(the_surface,vertex_index,new_point);
   SM_END_FOR_EACH
}

/**
**/

void
SMGD_compress_region(the_surface,the_region_list,current_region,s_value,
                   axis_flag)
SM_Surface_ptr the_surface;
SM_Region_List_ptr the_region_list;
SM_Region_Name current_region;
float s_value;
int axis_flag;

{
   SM_Region_ptr the_region;
   SM_Region_Controlbox_ptr the_box;
   int the_index,v_index,l,m,n;
   VECTOR_3D old_point,new_point,origin_point;
   BOOL anchor;

    the_index = SM_region_index_get(the_region_list,current_region); 
    if (the_index != SM_INDEX_ERROR) {
     l = 2; m = 2; n = 2;
     the_box = SM_region_controlbox_new(l,m,n);
     SM_region_controlbox_recreate(the_box,l,m,n); 
     SM_region_controlbox_get(the_surface,the_region_list,the_index,the_box);
     SMGD_controlbox_origin_get(the_box,origin_point);
     the_region = &the_region_list->region_list[the_index];
     SM_FOR_EACH_REGION_VERTEX(the_region) 
      v_index = region_vertex_ptr->id;
      anchor = region_vertex_ptr->anchor;
      if ((SM_vertex_index_valid(the_surface,v_index)) && (!anchor) ) {
        SM_vertex_point_get(the_surface,v_index,old_point);
        V3D_SUB(old_point,old_point,origin_point);
        SMGD_compress_point(old_point,s_value,axis_flag,new_point);
        V3D_ADD(new_point,new_point,origin_point);
        SM_vertex_point_set(the_surface,v_index,new_point);
      }
     SM_END_FOR_EACH
     SM_region_controlbox_dispose(the_box);
    }
}

/**
**/

void
SMGD_compress_object_controlbox(the_controlbox,s_value,axis_flag)
SM_Surface_Controlbox_ptr the_controlbox;
float s_value;
int axis_flag;

{
      int i,number_of_points;
      VECTOR_3D old_point, new_point, max_point, min_point, origin_point;

      number_of_points = the_controlbox->l * the_controlbox->m * 
                         the_controlbox->n;
  
      SMGD_box_maxmin_point_get(the_controlbox,max_point,min_point);
      for (i=X_INDEX; i<=Z_INDEX; i++) {
        if (s_value > 0) 
          origin_point[i] = min_point[i];
        else
          origin_point[i] = max_point[i];
      }
      for (i=0; i<number_of_points; i++) { 
       V3D_COPY(old_point,the_controlbox->vertex_list[i].point); 
       V3D_SUB(old_point,old_point,origin_point);
       SMGD_compress_point(old_point,s_value,axis_flag,new_point);
       V3D_ADD(new_point,new_point,origin_point);
       V3D_COPY(the_controlbox->vertex_list[i].point,new_point); 
      }
}
       
/**
**/

void
SMGD_compress_object_control_plane(the_controlbox,index,s_value,axis_flag,plane_dir)
SM_Surface_Controlbox_ptr the_controlbox;
int index;
float s_value;
int axis_flag;
int plane_dir;

{

     int i,j,k,ind_i,ind_j,ind_k,l,m,n,the_index;
     VECTOR_3D old_point, new_point, max_point, min_point, origin_point;

     SMGD_box_maxmin_point_get(the_controlbox,max_point,min_point);
     for (i=X_INDEX; i<=Z_INDEX; i++) {
        if (s_value > 0) 
          origin_point[i] = min_point[i];
        else
          origin_point[i] = max_point[i];
      }
     l = the_controlbox->l;
     m = the_controlbox->m;
     n = the_controlbox->n;
     SMGD_controlbox_vertex_indices_get(the_controlbox,index,&ind_i,&ind_j,&ind_k);
     switch (plane_dir) {
        case 0 : 
         i = ind_i;
         for (k=0; k<n; k++) {        
          for (j=0; j<m; j++) {        
            the_index = k*(l*m) + j*l + i;
            V3D_COPY(old_point,the_controlbox->vertex_list[the_index].point); 
            V3D_SUB(old_point,old_point,origin_point);
            SMGD_compress_point(old_point,s_value,axis_flag,new_point);
            V3D_ADD(new_point,new_point,origin_point);
            V3D_COPY(the_controlbox->vertex_list[the_index].point,new_point); 
          }
         }
         break;
        case 1 : 
         j = ind_j;
         for (k=0; k<n; k++) {        
          for (i=0; i<l; i++) {        
            the_index = k*(l*m) + j*l + i;
            V3D_COPY(old_point,the_controlbox->vertex_list[the_index].point); 
            V3D_SUB(old_point,old_point,origin_point);
            SMGD_compress_point(old_point,s_value,axis_flag,new_point);
            V3D_ADD(new_point,new_point,origin_point);
            V3D_COPY(the_controlbox->vertex_list[the_index].point,new_point); 
          }
         }
         break;
        case 2 : 
         k = ind_k;
         for (j=0; j<m; j++) {        
          for (i=0; i<l; i++) {        
            the_index = k*(l*m) + j*l + i;
            V3D_COPY(old_point,the_controlbox->vertex_list[the_index].point); 
            V3D_SUB(old_point,old_point,origin_point);
            SMGD_compress_point(old_point,s_value,axis_flag,new_point);
            V3D_ADD(new_point,new_point,origin_point);
            V3D_COPY(the_controlbox->vertex_list[the_index].point,new_point); 
          }
         }
         break;
        default :
        break;
      } 
}
/*-----------------------------------------------------------------------*/
/*                                                                       */
/*    Free form deformations using control 3D volume                     */
/*   Procedures dealing with Sederberg's deformations                    */
/*                                                                       */
/*-----------------------------------------------------------------------*/

# define MAX_CACHE 20

static float f_cache[MAX_CACHE+1];
static float b_cache[MAX_CACHE+1][MAX_CACHE+1];
static int cache_initialized = 0;

static void init_cache(void);

static void init_cache()
{
   if (!cache_initialized) {
     int i, j;

     f_cache[0] = 1.0;
     for (i=1; i<=MAX_CACHE; i++) {
	 f_cache[i] = f_cache[i-1] * (float)i;
     }

     for (i=0; i<=MAX_CACHE; i++) {
	 for (j=0; j<=i; j++) {
	     b_cache[i][j] = (f_cache[i]/(f_cache[i-j] * f_cache[j]));
	 }
     }
     cache_initialized = 1;
   }

}

/*-------------------------------------------------------------*/

int SMGD_factorial(n)
int n;
{

   assert((n>=0) && (n<= MAX_CACHE));

   if (!cache_initialized) init_cache();

   return((int)f_cache[n]);
}



/*-------------------------------------------------------------*/

float SMGD_power_int(value,n)
   float  value;
   int     n;
   /* This algorithm was taken straight from D.E.Knuth "The Art of
    * Computer Programming", vol.2, Seminumerical Algorithms,
    * section 4.6.3 (pg 398), Addison-Wesley, 1969. */
{
# define SQUARE(X) ((X)*(X))
# define ABS(X)    fabs(X)

  register float acc, pow_x; /* add "register"s for efficiency */
  register unsigned int odd, N;

  /* a couple, but not all, boundary conditions... */
   assert (!((value == 0.0) && (n <0)));

/*
   assert (!((value == 0.0) && (n == 0)));
*/
   switch (n)
     {
    case 0: return 1.0;
    case 1: return value;
    case 2: return SQUARE(value);
    case 3: return SQUARE(value)*value;
    case 4: return acc = SQUARE(value), SQUARE(acc);
    case 5: return acc = SQUARE(value), SQUARE(acc)*value;
    case 6: return acc = SQUARE(value), SQUARE(acc)*acc;
    case 7: return acc = SQUARE(value), SQUARE(acc)*acc*value;
    case 8: return acc = SQUARE(value), acc = SQUARE(acc), SQUARE(acc);
    /* include here other common cases in your application. */
    default: break;
     }
   acc = 1.0, pow_x = value, N = ABS(n); /* wrong if n == -MININT */
   for (;;) 
     {
      odd = N&1, N >>= 1;
      if (odd) { acc *= pow_x; if (N == 0) return n < 0 ? 1.0/acc : acc;}
      pow_x *= pow_x;
     }
 /*NOTREACHED*/

# undef ABS
# undef SQUARE
}

/*-------------------------------------------------------------*/

float SMGD_burnstein(n,i,t)

int n,i;
float t;

{
   assert((n>=0) && (n<=MAX_CACHE) && (i>=0) && (i<=MAX_CACHE));

   if (!cache_initialized) init_cache();

   return b_cache[n][i] * SMGD_power_int(t, i) * SMGD_power_int(1.0-t, n-i);
}

#define SMGD_BURNSTEIN_INIT \
  if (!cache_initialized) init_cache()
   
#define SMGD_BURNSTEIN(N, I, T) \
   (b_cache[(N)][(I)] *SMGD_power_int((T),(I)) *SMGD_power_int(1.0-(T),(N)-(I)))



/*--------------------------------------------------------------*/

void
SMGD_surface_freeform_apply(the_surface,globalDeform,the_controlbox)

SM_Surface_ptr the_surface;
SM_ffd_parameters globalDeform;
SM_Surface_Controlbox_ptr the_controlbox;

{

   float s,t,u;
   int i,j,k,l,m,n,index;
   float *Bl, *Bn, *Bm, Blmn;
   VECTOR_3D ffd,X_vect,temp;
   float nume,denom;

   l = the_controlbox->l; m = the_controlbox->m; n = the_controlbox->n;

   SMGD_BURNSTEIN_INIT;
   
   Bl = (float *) calloc(l, sizeof(float));
   Bm = (float *) calloc(m, sizeof(float));
   Bn = (float *) calloc(n, sizeof(float));
    
   SM_FOR_EACH_VERTEX(the_surface) 
    V3D_SUB(X_vect,vertex_ptr->point,globalDeform.X0);
    V3D_CROSS(temp,globalDeform.T_vect,globalDeform.U_vect);
    nume = V3D_DOT(temp,X_vect);
    denom = V3D_DOT(temp,globalDeform.S_vect);
    s = nume/denom;

    V3D_CROSS(temp,globalDeform.S_vect,globalDeform.U_vect);
    nume = V3D_DOT(temp,X_vect);
    denom = V3D_DOT(temp,globalDeform.T_vect);
    t = nume/denom;
 
    V3D_CROSS(temp,globalDeform.S_vect,globalDeform.T_vect);
    nume = V3D_DOT(temp,X_vect);
    denom = V3D_DOT(temp,globalDeform.U_vect);
    u = nume/denom;
 
    V3D_INIT(ffd, 0.0, 0.0, 0.0);
    
    for (k=0; k<n; k++)
      Bn[k] = SMGD_BURNSTEIN(n-1,k,u);
    for (j=0; j<m; j++)
      Bm[j] = SMGD_BURNSTEIN(m-1,j,t);
    for (i=0; i<l; i++)
      Bl[i] = SMGD_BURNSTEIN(l-1,i,s);

    index = 0;
    for (k=0; k<n; k++)
     for (j=0; j<m; j++)
      for (i=0; i<l; i++) {
        Blmn = Bl[i] * Bm[j] * Bn[k];
        V3D_SCALE(temp,Blmn,the_controlbox->vertex_list[index].point);
        V3D_ADD(ffd,ffd,temp);
        index++;
       }

    SM_vertex_point_set(the_surface,vertex_index,ffd);
   SM_END_FOR_EACH
   
   free(Bl); free(Bm); free(Bn);
   
} /* end of procedure */

/*-----------------------------------------------------------------*/

typedef struct {
    float s, t, u;
    int index;
}STU, *STU_ptr;
static STU_ptr surface_stu_list;

void
SMGD_ffd_init(the_box,ffd_parm) 
SM_Region_Controlbox_ptr the_box;
SM_ffd_parameters *ffd_parm;
{
     VECTOR_3D point1, point2, difference;

     SMGD_box_maxmin_point_get(the_box,point2,point1);
     V3D_SUB(difference,point2,point1);
     V3D_COPY(ffd_parm->X0,point1);
     V3D_INIT(ffd_parm->S_vect,difference[X_INDEX],0.0,0.0); 
     V3D_INIT(ffd_parm->T_vect,0.0,difference[Y_INDEX],0.0);
     V3D_INIT(ffd_parm->U_vect,0.0,0.0,difference[Z_INDEX]);
}

void
Shape_neutral_face_stu_get(the_surface, l, m, n)
SM_Surface_ptr the_surface;
int l, m, n;
{


   float s,t,u;
   VECTOR_3D X_vect,temp;
   float nume,denom;
   STU_ptr stu_ptr;
   SM_Surface_Controlbox_ptr the_box, dummy_box;
   SM_ffd_parameters globalDeform;
   
    surface_stu_list = (STU_ptr) calloc(the_surface->vert_list_size, sizeof(STU));
    the_box = (SM_Surface_Controlbox_ptr) calloc(1, sizeof(SM_Surface_Controlbox));
    the_box->vertex_list = (SM_Surface_Controlbox_vertex_ptr) 
          calloc(l*m*n, sizeof(SM_Surface_Controlbox_vertex));
    dummy_box = (SM_Surface_Controlbox_ptr) calloc(1, sizeof(SM_Surface_Controlbox));
    dummy_box->vertex_list = (SM_Surface_Controlbox_vertex_ptr) 
          calloc(l*m*n, sizeof(SM_Surface_Controlbox_vertex));
    SMGD_surface_box_create(the_surface, the_box, dummy_box, l, m, n); 
    SMGD_ffd_init(the_box,&globalDeform); 

    stu_ptr = surface_stu_list;
    SM_FOR_EACH_VERTEX(the_surface) 
      V3D_SUB(X_vect,vertex_ptr->point,globalDeform.X0);
      V3D_CROSS(temp,globalDeform.T_vect,globalDeform.U_vect);
      nume = V3D_DOT(temp,X_vect);
      denom = V3D_DOT(temp,globalDeform.S_vect);
      s = nume/denom;

      V3D_CROSS(temp,globalDeform.S_vect,globalDeform.U_vect);
      nume = V3D_DOT(temp,X_vect);
      denom = V3D_DOT(temp,globalDeform.T_vect);
      t = nume/denom;
 
      V3D_CROSS(temp,globalDeform.S_vect,globalDeform.T_vect);
      nume = V3D_DOT(temp,X_vect);
      denom = V3D_DOT(temp,globalDeform.U_vect);
      u = nume/denom;
      stu_ptr->s = s;
      stu_ptr->u = u;
      stu_ptr->t = t;
      stu_ptr->index = vertex_index;
      stu_ptr++;
    SM_END_FOR_EACH

    SM_region_controlbox_dispose(the_box);
    SM_region_controlbox_dispose(dummy_box);
}

void
Shape_surface_freeform_apply(the_surface,the_controlbox)
SM_Surface_ptr the_surface;
SM_Surface_Controlbox_ptr the_controlbox;
{

   float s,t,u;
   int i,j,k,l,m,n,index;
   float *Bl, *Bn, *Bm, Blmn;
   VECTOR_3D ffd, temp;
   STU_ptr ptr;

     l = the_controlbox->l; m = the_controlbox->m; n = the_controlbox->n;
     ptr = surface_stu_list;
     
     SMGD_BURNSTEIN_INIT;
     
     Bl = (float *) calloc(l, sizeof(float));
     Bm = (float *) calloc(m, sizeof(float));
     Bn = (float *) calloc(n, sizeof(float));

     SM_FOR_EACH_VERTEX(the_surface) 
     s = ptr->s; t = ptr->t; u = ptr->u; 
     V3D_INIT(ffd, 0.0, 0.0, 0.0);
     index = 0;


    for (k=0; k<n; k++)
      Bn[k] = SMGD_BURNSTEIN(n-1,k,u);
    for (j=0; j<m; j++)
      Bm[j] = SMGD_BURNSTEIN(m-1,j,t);
    for (i=0; i<l; i++)
      Bl[i] = SMGD_BURNSTEIN(l-1,i,s);
     
     for (k=0; k<n; k++) {
      for (j=0; j<m; j++) {
       for (i=0; i<l; i++) {
         Blmn = Bl[i] * Bm[j] * Bn[k];
         V3D_SCALE(temp,Blmn,the_controlbox->vertex_list[index].point);
         V3D_ADD(ffd,ffd,temp);
         index++;
        }
       }
      }
     SM_vertex_point_set(the_surface,vertex_index,ffd);
     ptr++;
    SM_END_FOR_EACH

   free(Bl); free(Bm); free(Bn);
  
}


/*----------------------------------------------------------------*/

void
SMGD_region_freeform_apply(the_surface,the_region_list,
                            region_current_name, globalDeform,the_controlbox)
SM_Surface_ptr the_surface;
SM_Region_List_ptr the_region_list;
SM_Region_Name region_current_name;
SM_ffd_parameters globalDeform;
SM_Region_Controlbox_ptr the_controlbox;

{

   float s,t,u;
   int i,j,k,l,m,n,index;
   float *Bl, *Bn, *Bm, Blmn;
   VECTOR_3D ffd,X_vect,temp,the_point;
   float nume,denom;
   SM_Region_ptr the_region_ptr;
   int the_index, v_index;
   BOOL anchor;
   float weight, stiff_factor;
   VECTOR_3D old_point, new_point;
/*
   long start_time;
   start_time = clock();
*/
   l = the_controlbox->l; m = the_controlbox->m; n = the_controlbox->n;

   the_index = SM_region_index_get(the_region_list,region_current_name);
   the_region_ptr = &the_region_list->region_list[the_index];


   SMGD_BURNSTEIN_INIT;
   
   Bl = (float *) calloc(l, sizeof(float));
   Bm = (float *) calloc(m, sizeof(float));
   Bn = (float *) calloc(n, sizeof(float));
   
   SM_FOR_EACH_REGION_VERTEX(the_region_ptr) 
    v_index = region_vertex_ptr->id;
    anchor = region_vertex_ptr->anchor;
    stiff_factor = region_vertex_ptr->properties.stiffness;
/*
    stiff_factor = 1.0;
*/
    if ((SM_vertex_index_valid(the_surface,v_index)) && (!anchor) ) {
     SM_vertex_point_get(the_surface,v_index,the_point);
     
/* test */
     V3D_COPY(old_point,the_point);
/* */
     V3D_SUB(X_vect,the_point,globalDeform.X0);
     V3D_CROSS(temp,globalDeform.T_vect,globalDeform.U_vect);
     nume  = V3D_DOT(temp,X_vect);
     denom = V3D_DOT(temp,globalDeform.S_vect);
     s = nume/denom;

     V3D_CROSS(temp,globalDeform.S_vect,globalDeform.U_vect);
     nume  = V3D_DOT(temp,X_vect);
     denom = V3D_DOT(temp,globalDeform.T_vect);
     t = nume/denom;
 
     V3D_CROSS(temp,globalDeform.S_vect,globalDeform.T_vect);
     nume  = V3D_DOT(temp,X_vect);
     denom = V3D_DOT(temp,globalDeform.U_vect);
     u = nume/denom;
 
     V3D_INIT(ffd, 0.0, 0.0, 0.0);

     index  = 0;
     weight = 0.0;
     

     for (k=0; k<n; k++)
      Bn[k] = SMGD_BURNSTEIN(n-1,k,u);
     for (j=0; j<m; j++)
      Bm[j] = SMGD_BURNSTEIN(m-1,j,t);
     for (i=0; i<l; i++)
      Bl[i] = SMGD_BURNSTEIN(l-1,i,s);

     for (k=0; k<n; k++) {
      for (j=0; j<m; j++) {
       for (i=0; i<l; i++) {
         Blmn = Bl[i] * Bm[j] * Bn[k] * the_controlbox->vertex_list[index].weight; 
         weight += Blmn;
         V3D_SCALE(temp,Blmn,the_controlbox->vertex_list[index].point);
         V3D_ADD(ffd,ffd,temp);
         index++;
        }
       }
      }
      
     if (fabs(weight) > SM_EPSILON) {
      V3D_SCALE(ffd,1.0/weight,ffd);
      V3D_SUB(new_point,ffd,old_point);
      V3D_SCALE(new_point,1.0/stiff_factor,new_point);
      V3D_ADD(new_point,new_point,old_point);
      SM_vertex_point_set(the_surface,v_index,new_point);
     }
    }
    SM_END_FOR_EACH
    
   free(Bl); free(Bm); free(Bn);
/*
 printf("ffd time : %f ms.\n",(float)(clock() - start_time)/1000.0);
*/  

} /* end of procedure */
/*---------------------------------------------------------------*/


void
SMGD_region_face_freeform_apply(the_surface,the_face,the_region_list,
                            region_current_name, globalDeform,the_controlbox)
SM_Surface_ptr the_surface, the_face;
SM_Region_List_ptr the_region_list;
SM_Region_Name region_current_name;
SM_ffd_parameters globalDeform;
SM_Region_Controlbox_ptr the_controlbox;

{

   float s,t,u;
   int i,j,k,l,m,n,index;
   float *Bl, *Bn, *Bm, Blmn;
   VECTOR_3D ffd,X_vect,temp,the_point;
   float nume,denom;
   SM_Region_ptr the_region_ptr;
   int the_index, v_index;
   BOOL anchor;
   float weight, stiff_factor;
   VECTOR_3D old_point, new_point, the_surface_point;
/*
   long start_time;
   start_time = clock();
*/
   l = the_controlbox->l; m = the_controlbox->m; n = the_controlbox->n;

   the_index = SM_region_index_get(the_region_list,region_current_name);
   the_region_ptr = &the_region_list->region_list[the_index];


   SMGD_BURNSTEIN_INIT;
   
   Bl = (float *) calloc(l, sizeof(float));
   Bm = (float *) calloc(m, sizeof(float));
   Bn = (float *) calloc(n, sizeof(float));
   
   SM_FOR_EACH_REGION_VERTEX(the_region_ptr) 
    v_index = region_vertex_ptr->id;
    anchor = region_vertex_ptr->anchor;
    stiff_factor = region_vertex_ptr->properties.stiffness;
/*
    stiff_factor = 1.0;
*/
    if ((SM_vertex_index_valid(the_face,v_index)) && (!anchor) ) {
     SM_vertex_point_get(the_face,v_index,the_point);
     SM_vertex_point_get(the_surface,v_index,the_surface_point);
     
/* test */
     V3D_COPY(old_point,the_point);
/* */
     V3D_SUB(X_vect,the_point,globalDeform.X0);
     V3D_CROSS(temp,globalDeform.T_vect,globalDeform.U_vect);
     nume  = V3D_DOT(temp,X_vect);
     denom = V3D_DOT(temp,globalDeform.S_vect);
     s = nume/denom;

     V3D_CROSS(temp,globalDeform.S_vect,globalDeform.U_vect);
     nume  = V3D_DOT(temp,X_vect);
     denom = V3D_DOT(temp,globalDeform.T_vect);
     t = nume/denom;
 
     V3D_CROSS(temp,globalDeform.S_vect,globalDeform.T_vect);
     nume  = V3D_DOT(temp,X_vect);
     denom = V3D_DOT(temp,globalDeform.U_vect);
     u = nume/denom;
 
     V3D_INIT(ffd, 0.0, 0.0, 0.0);

     index  = 0;
     weight = 0.0;
     

     for (k=0; k<n; k++)
      Bn[k] = SMGD_BURNSTEIN(n-1,k,u);
     for (j=0; j<m; j++)
      Bm[j] = SMGD_BURNSTEIN(m-1,j,t);
     for (i=0; i<l; i++)
      Bl[i] = SMGD_BURNSTEIN(l-1,i,s);

     for (k=0; k<n; k++) {
      for (j=0; j<m; j++) {
       for (i=0; i<l; i++) {
         Blmn = Bl[i] * Bm[j] * Bn[k] * the_controlbox->vertex_list[index].weight; 
         weight += Blmn;
         V3D_SCALE(temp,Blmn,the_controlbox->vertex_list[index].point);
         V3D_ADD(ffd,ffd,temp);
         index++;
        }
       }
      }
      
     if (fabs(weight) > SM_EPSILON) {
      V3D_SCALE(ffd,1.0/weight,ffd);
      V3D_SUB(new_point,ffd,old_point);
      V3D_SCALE(new_point,1.0/stiff_factor,new_point);
      V3D_ADD(new_point,new_point,the_surface_point);
      SM_vertex_point_set(the_surface,v_index,new_point);
     }
    }
    SM_END_FOR_EACH
    
   free(Bl); free(Bm); free(Bn);
/*
 printf("ffd time : %f ms.\n",(float)(clock() - start_time)/1000.0);
*/  

} /* end of procedure */
/*---------------------------------------------------------------*/

void
SMGD_region_index_freeform_apply(the_surface, the_N_face, the_region_list,
                            region_index, globalDeform,the_controlbox)
SM_Surface_ptr the_surface, the_N_face;
SM_Region_List_ptr the_region_list;
int region_index;
SM_ffd_parameters globalDeform;
SM_Region_Controlbox_ptr the_controlbox;

{

   float s,t,u;
   int i,j,k,l,m,n,index;
   float *Bl, *Bn, *Bm, Blmn;
   VECTOR_3D ffd,X_vect,temp,the_point;
   float nume,denom;
   SM_Region_ptr the_region_ptr;
   int v_index;
   BOOL anchor;
   float weight, stiff_factor;
   VECTOR_3D old_point, new_point;
/*
   long start_time;
   start_time = clock();
*/
   l = the_controlbox->l; m = the_controlbox->m; n = the_controlbox->n;

   the_region_ptr = &the_region_list->region_list[region_index];


   SMGD_BURNSTEIN_INIT;
   
   Bl = (float *) calloc(l, sizeof(float));
   Bm = (float *) calloc(m, sizeof(float));
   Bn = (float *) calloc(n, sizeof(float));

   SM_FOR_EACH_REGION_VERTEX(the_region_ptr) 
    v_index = region_vertex_ptr->id;
    anchor = region_vertex_ptr->anchor;
/* test
    stiff_factor = region_vertex_ptr->properties.stiffness;
*/
    stiff_factor = 1.0;
    if ((SM_vertex_index_valid(the_N_face,v_index)) && (!anchor) ) {
     SM_vertex_point_get(the_N_face,v_index,the_point);
     
/* test */
     V3D_COPY(old_point,the_point);
/* */
     V3D_SUB(X_vect,the_point,globalDeform.X0);
     V3D_CROSS(temp,globalDeform.T_vect,globalDeform.U_vect);
     nume  = V3D_DOT(temp,X_vect);
     denom = V3D_DOT(temp,globalDeform.S_vect);
     s = nume/denom;

     V3D_CROSS(temp,globalDeform.S_vect,globalDeform.U_vect);
     nume  = V3D_DOT(temp,X_vect);
     denom = V3D_DOT(temp,globalDeform.T_vect);
     t = nume/denom;
 
     V3D_CROSS(temp,globalDeform.S_vect,globalDeform.T_vect);
     nume  = V3D_DOT(temp,X_vect);
     denom = V3D_DOT(temp,globalDeform.U_vect);
     u = nume/denom;
 
     V3D_INIT(ffd, 0.0, 0.0, 0.0);

     index  = 0;
     weight = 0.0;
     
     for (k=0; k<n; k++)
      Bn[k] = SMGD_BURNSTEIN(n-1,k,u);
     for (j=0; j<m; j++)
      Bm[j] = SMGD_BURNSTEIN(m-1,j,t);
     for (i=0; i<l; i++)
      Bl[i] = SMGD_BURNSTEIN(l-1,i,s);

     for (k=0; k<n; k++) {
      for (j=0; j<m; j++) {
       for (i=0; i<l; i++) {
         Blmn = Bl[i] * Bm[j] * Bn[k] * the_controlbox->vertex_list[index].weight; 
         weight += Blmn;
         V3D_SCALE(temp,Blmn,the_controlbox->vertex_list[index].point);
         V3D_ADD(ffd,ffd,temp);
         index++;
        }
       }
      }

     if (fabs(weight) > SM_EPSILON) {
      V3D_SCALE(ffd,1.0/weight,ffd);
      V3D_SUB(new_point,ffd,old_point);
      V3D_SCALE(new_point,1.0/stiff_factor,new_point);
      V3D_ADD(new_point,new_point,old_point);
      SM_vertex_point_set(the_surface,v_index,new_point);
     }
    }
    SM_END_FOR_EACH
    
   free(Bl); free(Bm); free(Bn);
/*
 printf("ffd time : %f ms.\n",(float)(clock() - start_time)/1000.0);
*/  

} /* end of procedure */
/*---------------------------------------------------------------*/

void
SMGD_region_cylinder_freeform_apply(the_surface,the_region_list,
                            region_current_name, cylinderDeform,the_cylinder)
SM_Surface_ptr the_surface;
SM_Region_List_ptr the_region_list;
SM_Region_Name region_current_name;
SM_effd_cylinder cylinderDeform;
SM_Region_Controlcylinder_ptr the_cylinder;

{

   float s,t,u;
   int i,j,k,l,m,n,index;
   float Bl,Bn,Bm,Blmn;
   VECTOR_3D ffd,X_vect,temp,the_point;
   SM_Region_ptr the_region_ptr;
   int the_index, v_index;
   BOOL anchor;
   float weight, stiff_factor;
   VECTOR_3D old_point, new_point;

   float dr,dt,dz;


   l = the_cylinder->l; m = the_cylinder->m; n = the_cylinder->n;

   the_index = SM_region_index_get(the_region_list,region_current_name);
   the_region_ptr = &the_region_list->region_list[the_index];

   SMGD_BURNSTEIN_INIT;
   SM_FOR_EACH_REGION_VERTEX(the_region_ptr) 
    v_index = region_vertex_ptr->id;
    anchor = region_vertex_ptr->anchor;
    stiff_factor = 1.0;
    if ((SM_vertex_index_valid(the_surface,v_index)) && (!anchor) ) {
     SM_vertex_point_get(the_surface,v_index,the_point);

     SM_region_cartesian_to_polar(the_point,cylinderDeform.center,X_vect);
     s = X_vect[0]/cylinderDeform.radius; 
     t = X_vect[1]/(2*M_PI);
     u = X_vect[2]/cylinderDeform.height;
     V3D_COPY(old_point,the_point); 
     V3D_INIT(ffd, 0.0, 0.0, 0.0);


     index  = 0;
     weight = 0.0;

     dr = cylinderDeform.radius/(the_cylinder->l - 1);
     dt = 2*M_PI/(the_cylinder->m - 1);
     dz = cylinderDeform.height/(the_cylinder->n - 1);

     for (k=0; k<n; k++) {
      Bn = SMGD_BURNSTEIN(n-1,k,u);
      for (j=0; j<m; j++) {
       Bm = SMGD_BURNSTEIN(m-1,j,t);
       for (i=0; i<l; i++) {
         Bl = SMGD_BURNSTEIN(l-1,i,s);
         Blmn = Bl * Bm * Bn * the_cylinder->vertex_list[index].weight;
         weight += Blmn;

         SM_region_cartesian_to_polar(the_cylinder->vertex_list[index].point,
                                    cylinderDeform.center,X_vect);
         if ((i == 0) || (j == m-1))
          V3D_INIT(X_vect,i*dr,j*dt,k*dz);

         V3D_SCALE(temp,Blmn,X_vect);
         V3D_ADD(ffd,ffd,temp);
         index++;
        }
       }
      }
     if (fabs(weight) > SM_EPSILON) {
      V3D_SCALE(ffd,1.0/weight,ffd);
      SM_region_polar_to_cartesian(ffd,cylinderDeform.center,X_vect);
      V3D_SUB(new_point,X_vect,old_point);
      V3D_SCALE(new_point,1.0/stiff_factor,new_point);
      V3D_ADD(new_point,new_point,old_point);
      SM_vertex_point_set(the_surface,v_index,new_point);
     }
    }
    SM_END_FOR_EACH

    

} /* end of procedure */
/*---------------------------------------------------------------*/


void
SMGD_surface_force_apply(the_surface,the_box,the_forcebox)

SM_Surface_ptr the_surface;
SM_Surface_Controlbox_ptr the_box, the_forcebox;

{
  int i,j,k,l,m,n,index;
  float dx, dy, dz, dist;
  float stiffness;
  VECTOR_3D new_point, net_force;

   l = the_box->l; m = the_box->m; n = the_box->n;
   stiffness = 5.0;
   SM_FOR_EACH_VERTEX(the_surface)
    index = 0;
    for (k=0; k<n; k++) {
     for (j=0; j<m; j++) {
      for (i=0; i<l; i++) {
       dx = vertex_ptr->point[X_INDEX] - 
            the_box->vertex_list[index].point[X_INDEX];
       dy = vertex_ptr->point[Y_INDEX] - 
            the_box->vertex_list[index].point[Y_INDEX];
       dz = vertex_ptr->point[Z_INDEX] - 
            the_box->vertex_list[index].point[Z_INDEX];
       /* min dist == 1 */
       dist = stiffness/(1.0 + sqrt(dx*dx + dy*dy + dz*dz));
       V3D_SUB(net_force,the_box->vertex_list[index].point,
                 the_forcebox->vertex_list[index].point);
       V3D_SCALE(new_point, dist, net_force); 
       V3D_ADD(new_point, vertex_ptr->point, new_point);
       SM_vertex_point_set(the_surface, vertex_index, new_point);
       index++;
       }
      }
     }
    SM_END_FOR_EACH
}

/*--------------------------------------------------------------*/

void
SMGD_region_force_apply(the_surface,the_region_list,
                        region_current_name,the_box,the_region_forcebox)

SM_Surface_ptr the_surface;
SM_Region_List_ptr the_region_list;
SM_Region_Name region_current_name;
SM_Region_Controlbox_ptr the_box, the_region_forcebox;

{
  int i,j,k,l,m,n,index;
  float dx, dy, dz, dist;
  float stiffness;
  VECTOR_3D new_point, net_force, the_point;
  SM_Region_ptr the_region_ptr;
  int v_index,the_index;
  BOOL anchor;

   l = the_box->l; m = the_box->m; n = the_box->n;
   stiffness = 5.0;
   the_index = SM_region_index_get(the_region_list,region_current_name);
   the_region_ptr = &the_region_list->region_list[the_index];
   SM_FOR_EACH_REGION_VERTEX(the_region_ptr)
    v_index = region_vertex_ptr->id;
    anchor = region_vertex_ptr->anchor;
    if ((SM_vertex_index_valid(the_surface,v_index)) && (!anchor) ) {
    SM_vertex_point_get(the_surface,v_index,the_point);
    index = 0;
    for (k=0; k<n; k++) {
     for (j=0; j<m; j++) {
      for (i=0; i<l; i++) {
       dx = the_point[X_INDEX] - 
            the_box->vertex_list[index].point[X_INDEX];
       dy = the_point[Y_INDEX] - 
            the_box->vertex_list[index].point[Y_INDEX];
       dz = the_point[Z_INDEX] - 
            the_box->vertex_list[index].point[Z_INDEX];
       /* min dist == 1 */
       dist = stiffness/(1.0 + sqrt(dx*dx + dy*dy + dz*dz));
       V3D_SUB(net_force,the_box->vertex_list[index].point,
                 the_region_forcebox->vertex_list[index].point);
       V3D_SCALE(new_point, dist, net_force); 
       V3D_ADD(new_point, the_point, new_point);
       SM_vertex_point_set(the_surface, v_index, new_point);
       index++;
       }
      }
     }
    }
    SM_END_FOR_EACH
}

/*---------------------------------------------------------------*/
BOOL 
SMGD_get_perpendicular_distance(the_point,line_point,line_direction,dist)

VECTOR_3D the_point, line_point, line_direction;
float *dist;

{
   VECTOR_3D a_point;
   float mag_dot,mag_distance,mag_direction,mag_result;

   V3D_SUB(a_point,line_point,the_point);
   mag_distance = V3D_DOT(a_point,a_point); 
   mag_dot = V3D_DOT(a_point,line_direction); 
   mag_dot = mag_dot * mag_dot;
   mag_direction = V3D_DOT(line_direction,line_direction);

   if ( !(fabs(mag_direction) < SM_EPSILON) )
   {
    mag_dot = mag_dot/mag_direction;
    mag_result = mag_distance - mag_dot;
    *dist = sqrt(mag_result);
    return(YES);
   }
   else
    return(NO);   
}
/*----------------------------------------------------------------*/

void
SMGD_radialforce_apply(the_surface,the_box,the_forcebox)

SM_Surface_ptr the_surface;
SM_Surface_Controlbox_ptr the_box, the_forcebox;

{
  int i,j,k,l,m,n,index;
  float dx, dy, dz, dist;
  float angular_stiffness,radial_stiffness,R_max,
        scale_factor_angular, scale_factor_radial,
        norm_diff_vect, norm_net_force,cos_angle,
        radial_disp,angular_disp;
  VECTOR_3D old_point, new_point, net_force,
            new_vertexpoint, diff_vect;

   l = the_box->l; m = the_box->m; n = the_box->n;
   /* for present computation the R_max is considered as the
   maximum distance between the two extreme distant points in
   the control box (before deformation) */
   index = l*m*n - 1;
   dx = the_forcebox->vertex_list[index].point[X_INDEX] -
        the_forcebox->vertex_list[0].point[X_INDEX];
   dy = the_forcebox->vertex_list[index].point[Y_INDEX] -
        the_forcebox->vertex_list[0].point[Y_INDEX];
   dz = the_forcebox->vertex_list[index].point[Z_INDEX] -
        the_forcebox->vertex_list[0].point[Z_INDEX];
   R_max = 0.5 * sqrt(dx*dx + dy*dy + dz*dz);
   radial_stiffness = 0.5;
   angular_stiffness = 0.5;
   scale_factor_angular = 1.0;
   scale_factor_radial = 1.0;

   SM_FOR_EACH_VERTEX(the_surface)
    index = 0;
    for (k=0; k<n; k++) {
     for (j=0; j<m; j++) {
      for (i=0; i<l; i++) {
       /* find the radial diatance of the point from the point
       in the control box and the angle between the two vectors
       for further computation */
       if (the_box->vertex_list[index].picked) {

       V3D_COPY(old_point,the_forcebox->vertex_list[index].point);
       V3D_COPY(new_point,the_box->vertex_list[index].point);
       V3D_SUB(net_force,new_point,old_point);
       if (SMGD_get_perpendicular_distance(vertex_ptr->point,
                                       old_point,net_force,&dist))
        {
         radial_disp = radial_stiffness * cos((dist/R_max)*M_PI/2);
         radial_disp = scale_factor_radial * radial_disp;
        }
       else
         radial_disp = 0;
       V3D_SCALE(new_vertexpoint, radial_disp, net_force); 
       V3D_ADD(new_vertexpoint, vertex_ptr->point, new_vertexpoint);
       V3D_SUB(diff_vect,new_point,new_vertexpoint);
       cos_angle = V3D_DOT(net_force,diff_vect);
       norm_net_force = V3D_NORM(net_force);
       norm_diff_vect = V3D_NORM(diff_vect);
       if (!(fabs(norm_net_force) < SM_EPSILON || 
             fabs(norm_diff_vect) < SM_EPSILON)) {
        cos_angle = cos_angle/(norm_net_force * norm_diff_vect);
        angular_disp = angular_stiffness * cos_angle/
                      (norm_diff_vect * norm_diff_vect);
        angular_disp = scale_factor_angular * angular_disp;
       }
       else
        angular_disp = 0;
       V3D_SCALE(diff_vect, angular_disp, diff_vect);
       V3D_ADD(new_vertexpoint, new_vertexpoint, diff_vect);
       SM_vertex_point_set(the_surface, vertex_index, new_vertexpoint);
       } /* if thr box vertex picked */
       index++;
       }
      }
     }
    SM_END_FOR_EACH
}

/*----------------------------------------------------------------------*/
void
SMGD_control_box_pick_set(the_controlbox,value)
SM_Region_Controlbox_ptr the_controlbox;
BOOL value;
{
  int i;
  
  for (i=0; i<the_controlbox->vertex_list_size; i++) 
    the_controlbox->vertex_list[i].picked = value;
}

/*-----------------------------------------------------------------------*/

