
/*------------------------------------------------------------------------------*/
/*										*/
/*										*/
/*  This software module was originally developed by                            */
/*        Prem Kalra, MIRALab, University of Geneva                             */
/*  and edited by                                                               */
/*        Igor Pandzic, MIRALab, University of Geneva                           */
/*        Marc Escher, MIRALab, University of Geneva                            */
/*        Pierre Beylot, MIRALab, University of Geneva                          */
/*  in the course of development of the MPEG-4 Video (ISO/IEC 14496-2).	        */
/*  This software module is an implementation of a part of one or more	        */
/*  MPEG-4 Video (ISO/IEC 14496-2) tools as specified by the MPEG-4 Video       */
/*  (ISO/IEC 14496-2). ISO/IEC gives users of the                               */
/*  MPEG-4 Video (ISO/IEC 14496-2) free license to this software module or	*/
/*  modifications thereof for use in hardware or software products claiming 	*/
/*  conformance to the MPEG-4 Video (ISO/IEC 14496-2). Those intending to use	*/
/*  this software module in hardware or software products are advised that	*/
/*  its use may infringe existing patents. The original developer of this	*/
/*  software module and his/her company, the subsequent editors and their	*/
/*  companies, and ISO/IEC have no liability for use of this software		*/
/*  module or modifications thereof. Copyright is not released for non		*/
/*  MPEG-4 Video (ISO/IEC 14496-2) conforming products.                         */
/*  MIRALab, University of Geneva retains full right to use the code for its    */
/*  own purpose, assign or donate the code to a third party and to inhibit      */
/*  third parties from using the code for non MPEG-4 Video (ISO/IEC 14496-2)    */
/*  conforming products. This copyright notice must be included in all copies   */
/*  or derivative works. Copyright (c) (1997).			      		*/
/*										*/
/*										*/
/*------------------------------------------------------------------------------*/


#include "FaceMan.h"

static float AU_mouth_compexp_scale = 1;
static VECTOR_3D Neutral_head_center;

void
AU_neutral_face_set()
{
    AU_mouth_compexp_scale = 1; 
    V3D_NULL(Neutral_head_center);
}

AU_pointlist_ptr
AU_pointlist_new()
{
     AU_pointlist_ptr the_list;
     
     the_list = (AU_pointlist_ptr) calloc(1,sizeof(AU_pointlist));
     the_list->size = 0;
     the_list->vlist = NULL;
     return(the_list);
}

void
AU_pointlist_dispose(the_list)
AU_pointlist_ptr the_list;
{
  if (the_list != NULL) {
     if (the_list->vlist != NULL)
       free(the_list->vlist);
    free(the_list);
  }
}

void
AU_set_pointlist(the_surface,the_region_list,the_index,the_list)
SM_Surface_ptr the_surface;
SM_Region_List_ptr the_region_list;
int the_index;
AU_pointlist_ptr the_list;
{
   SM_Region_ptr the_region;
   int v_index,vsize;
   VECTOR_3D the_point;
   AU_point_ptr v_ptr;
   
   the_region = &the_region_list->region_list[the_index];
   vsize = the_region->region_vertex_list_size;
   the_list->vlist = (AU_point_ptr) calloc(vsize,sizeof(AU_point));
   the_list->size = vsize;
   v_ptr = the_list->vlist;
   SM_FOR_EACH_REGION_VERTEX(the_region)
     v_index = region_vertex_ptr->id;
     if (SM_vertex_index_valid(the_surface,v_index)) {
        SM_vertex_point_get(the_surface,v_index,the_point);
        V3D_COPY(v_ptr->point,the_point);
        v_ptr->index = v_index;
        v_ptr++;
     }
   SM_END_FOR_EACH

}

BOOL
AU_get_apoint(the_list,the_index,point)
AU_pointlist_ptr the_list;
int the_index;
VECTOR_3D point;

{
     int i;
     AU_point_ptr v_ptr;

     for (i=0; i<the_list->size; i++) {
         v_ptr = &the_list->vlist[i];
         if (the_index == v_ptr->index) {
          V3D_COPY(point,v_ptr->point);
          return(YES);
         }
     }
     return(NO);
}

void
AU_face_ffd_init(the_box,ffd_parm) 
SM_Region_Controlbox_ptr the_box;
SM_ffd_parameters *ffd_parm;
{
     VECTOR_3D point1, point2, difference;

     SMGD_box_maxmin_point_get(the_box,point2,point1);
     V3D_SUB(difference,point2,point1);
     V3D_COPY(ffd_parm->X0,point1);
     V3D_INIT(ffd_parm->S_vect,difference[X_INDEX],0.0,0.0); 
     V3D_INIT(ffd_parm->T_vect,0.0,difference[Y_INDEX],0.0);
     V3D_INIT(ffd_parm->U_vect,0.0,0.0,difference[Z_INDEX]);
}

void
AU_face_region_reset(the_surface,the_face,the_region_list,the_index)
SM_Surface_ptr the_surface;
SM_Surface_ptr the_face;
SM_Region_List_ptr the_region_list;
int the_index;
{
   SM_Region_ptr the_region;
   int v_index;
   VECTOR_3D old_point;

   the_region = &the_region_list->region_list[the_index];
   SM_FOR_EACH_REGION_VERTEX(the_region)
     v_index = region_vertex_ptr->id;
     if (SM_vertex_index_valid(the_surface,v_index)) {
        SM_vertex_point_get(the_face,v_index,old_point);
        SM_vertex_point_set(the_surface,v_index,old_point);
     }
   SM_END_FOR_EACH

}

/*--------------------------------------------------------------------*/

BOOL 
AU_eye_hole_center_get(the_surface, the_region_list, the_index, center_point)
SM_Surface_ptr the_surface;
SM_Region_List_ptr the_region_list;
int the_index;
VECTOR_3D center_point;
{

         SM_Region_ptr reg_ptr;
	 int e_index,  count, v1, v2;
	 BOOL bool_val;
	 int *temp_ptr, i;
	 VECTOR_3D the_point;
	 
	 for(i=0;i<3;i++)
	   center_point[i] = 0.0;

         reg_ptr = &the_region_list->region_list[the_index];
	 temp_ptr = (int *) calloc(500,sizeof(int)); 
         count = 0;
         SM_FOR_EACH_REGION_EDGE(reg_ptr)
          e_index = *region_edge_ptr;
          if (SM_edge_hole_get(the_surface,e_index,&bool_val) && bool_val) {
              if (SM_edge_vertices_get(the_surface,e_index,&v1,&v2)) {
                  temp_ptr[count++] = v1; 
                  temp_ptr[count++] = v2; 
              }
           }
         SM_END_FOR_EACH
	 
	 if (count == 0) {
	 	 free(temp_ptr);
	          return(NO);
	 }

         for (i=0; i<count; i++) {
           SM_vertex_point_get(the_surface,temp_ptr[i],the_point);
	   V3D_ADD(center_point, center_point, the_point);
	 }

	 free(temp_ptr);

	 V3D_SCALE(center_point, 1.0/(float) count, center_point);
	 return(YES);
}

/*--------------------------------------------------------------------*/
void
AU_eye_origin_get(the_surface, the_region_list, the_index, origin_point)
SM_Surface_ptr the_surface;
SM_Region_List_ptr the_region_list;
int the_index;
VECTOR_3D origin_point;

{

   if (!AU_eye_hole_center_get(the_surface, the_region_list, the_index, origin_point))
     SM_region_center_get(the_surface, the_region_list, the_index, origin_point);
     

  
/*
   SM_Region_Controlbox_ptr the_box;
   int l,m,n;

    V3D_NULL(origin_point); 
    l = 2; m = 2; n = 2;
    the_box = SM_region_controlbox_new(l,m,n);
    SM_region_controlbox_recreate(the_box,l,m,n); 
    SM_region_controlbox_get(the_surface,the_region_list,the_index,the_box);
    SMGD_controlbox_origin_get(the_box,origin_point);
    free(the_box);
    */
}

/*--------------------------------------------------------------------*/
void
AU_eye_dilate_pupil(the_surface, the_region_list,the_name,the_value)
SM_Surface_ptr the_surface;
SM_Region_List_ptr the_region_list;
SM_Region_Name the_name;
float the_value;

{
    SM_Region_Controlbox_ptr the_box;
    int the_index,l,m,n;
    SM_ffd_parameters ffd_parm;
    VECTOR_3D s_vect;

    the_index = SM_region_index_get(the_region_list,the_name);
    l = 3; m = 3; n = 3;
    the_value = 1.0 + the_value * 0.3;
    if (the_index != SM_INDEX_ERROR) {
       the_box = SM_region_controlbox_new(l,m,n);
       SM_region_controlbox_recreate(the_box,l,m,n); 
       SM_region_controlbox_get(the_surface,the_region_list,the_index,the_box);
       AU_face_ffd_init(the_box,&ffd_parm); 
       V3D_INIT(s_vect,the_value,the_value,the_value);
       SMGD_scale_object_controlbox(the_box,s_vect);
       SMGD_region_freeform_apply(the_surface,the_region_list,the_name,
                                  ffd_parm,the_box);
       SM_region_controlbox_dispose(the_box);
    }
}

/*--------------------------------------------------------------------*/
void
AU_eye_move_horizontal(the_surface, the_region_list,the_name,the_value)
SM_Surface_ptr the_surface;
SM_Region_List_ptr the_region_list;
SM_Region_Name the_name;
float the_value;

{
    int the_index;
    VECTOR_3D origin_point;
    
    the_index = SM_region_index_get(the_region_list,the_name);
    if (the_index != SM_INDEX_ERROR) {
       AU_eye_origin_get(the_surface, the_region_list, the_index, origin_point);
       SMGD_rotate_face_region(the_surface, origin_point, the_region_list,the_name,
                                  Y_INDEX,the_value * 0.6);
    }
}

/*--------------------------------------------------------------------*/
void
AU_eye_move_vertical(the_surface,the_region_list,the_name,the_value)
SM_Surface_ptr the_surface;
SM_Region_List_ptr the_region_list;
SM_Region_Name the_name;
float the_value;

{
     int the_index;
     VECTOR_3D origin_point;

     the_index = SM_region_index_get(the_region_list,the_name);
     if (the_index != SM_INDEX_ERROR) {
       AU_eye_origin_get(the_surface, the_region_list, the_index, origin_point);
       SMGD_rotate_face_region(the_surface,origin_point,the_region_list,the_name,
                                  X_INDEX,the_value);
    }
}

/*--------------------------------------------------------------------*/
void
AU_eye_track_pan(the_surface, the_face,the_region_list,the_name,the_value)
SM_Surface_ptr the_surface, the_face;
SM_Region_List_ptr the_region_list;
SM_Region_Name the_name;
float the_value;

{
    int the_index;
    VECTOR_3D origin_point;
    
    the_index = SM_region_index_get(the_region_list,the_name);
    if (the_index != SM_INDEX_ERROR) {
       AU_eye_origin_get(the_face, the_region_list, the_index, origin_point);
       SMGD_rotate_try_face_region(the_surface, the_face, origin_point, the_region_list,the_name,
                                  Y_INDEX,the_value * 0.6);
    }
}

/*--------------------------------------------------------------------*/
void
AU_eye_track_tilt(the_surface,the_face,the_region_list,the_name,the_value)
SM_Surface_ptr the_surface,the_face;
SM_Region_List_ptr the_region_list;
SM_Region_Name the_name;
float the_value;

{
     int the_index;
     VECTOR_3D origin_point;

     the_index = SM_region_index_get(the_region_list,the_name);
     if (the_index != SM_INDEX_ERROR) {
       AU_eye_origin_get(the_face, the_region_list, the_index, origin_point);
       SMGD_rotate_try_face_region(the_surface,the_face,origin_point,the_region_list,the_name,
                                  X_INDEX,the_value);
    }
}

/*--------------------------------------------------------------------*/

void
AU_eye_track(the_surface,the_face,the_region_list,the_name,track_vec)
SM_Surface_ptr the_surface;
SM_Surface_ptr the_face;
SM_Region_List_ptr the_region_list;
SM_Region_Name the_name;
VECTOR_3D track_vec;

{
    float pan,tilt;
    VECTOR_3D origin_point;
    int the_index;

    pan = AU_gaze_point_pan_angle_get(track_vec,Neutral_head_center);
    tilt = AU_gaze_point_tilt_angle_get(track_vec,Neutral_head_center); 
    
    the_index = SM_region_index_get(the_region_list,the_name);
    V3D_NULL(origin_point);
    if (the_index != SM_INDEX_ERROR) 
       AU_eye_origin_get(the_face, the_region_list, the_index, origin_point);

    SMGD_rotate_try_face_region(the_surface,the_face,origin_point,the_region_list,the_name,Y_INDEX,pan * 0.15);
    SMGD_rotate_try_face_region(the_surface,the_surface,origin_point,the_region_list,the_name,X_INDEX,tilt * 0.15);

}

/*--------------------------------------------------------------------*/

void
AU_eye_move_out(the_surface,the_region_list,the_name,the_value)
SM_Surface_ptr the_surface;
SM_Region_List_ptr the_region_list;
SM_Region_Name the_name;
float the_value;

{
    int the_index;
    VECTOR_3D t_vect;

    the_index = SM_region_index_get(the_region_list,the_name);
    the_value *= 5.0;
    V3D_INIT(t_vect,0.0,0.0,the_value);
    if (the_index != SM_INDEX_ERROR) {
       SMGD_translate_region(the_surface,the_region_list,the_name,
                                  t_vect);
    }
}

/*--------------------------------------------------------------------*/

/******
void
AU_eye_move_horizontal(the_surface,the_region_list,the_name,the_value)
SM_Surface_ptr the_surface;
SM_Region_List_ptr the_region_list;
SM_Region_Name the_name;
float the_value;

{
    SM_Region_Controlbox_ptr the_box;
    int number_of_points,the_index,l,m,n;
    SM_ffd_parameters ffd_parm;

    the_index = SM_region_index_get(the_region_list,the_name);
    l = 3; m = 3; n = 3;
    if (the_index != SM_INDEX_ERROR) {
       the_box = SM_region_controlbox_new(l,m,n);
       SM_region_controlbox_recreate(the_box,l,m,n); 
       SM_region_controlbox_get(the_surface,the_region_list,the_index,the_box);
       AU_face_ffd_init(the_box,&ffd_parm); 
       SMGD_rotate_object_controlbox(the_box,Y_INDEX,the_value * 0.6);
       SMGD_region_freeform_apply(the_surface,the_region_list,the_name,
                                  ffd_parm,the_box);
       SM_region_controlbox_dispose(the_box);
    }
}

void
AU_eye_move_horizontal(the_surface,the_neutral_face,the_region_list,the_name,the_value)
SM_Surface_ptr the_surface;
SM_Surface_ptr the_neutral_face;
SM_Region_List_ptr the_region_list;
SM_Region_Name the_name;
float the_value;

{
    SM_Region_Controlbox_ptr the_box, the_N_box;
    int number_of_points,the_index,l,m,n;
    SM_ffd_parameters ffd_parm;

    the_index = SM_region_index_get(the_region_list,the_name);
    l = 3; m = 3; n = 3;
    if (the_index != SM_INDEX_ERROR) {
       the_box = SM_region_controlbox_new(l,m,n);
       the_N_box = SM_region_controlbox_new(l,m,n);
       SM_region_controlbox_recreate(the_box,l,m,n); 
       SM_region_controlbox_recreate(the_N_box,l,m,n); 
       SM_region_controlbox_get(the_surface,the_region_list,the_index,the_box);
       SM_region_controlbox_get(the_neutral_face,the_region_list,the_index,the_N_box);
       AU_face_ffd_init(the_N_box,&ffd_parm); 
       SMGD_rotate_object_controlbox(the_box,Y_INDEX,the_value * 0.6);
       SMGD_region_index_freeform_apply(the_surface,the_neutral_face,the_region_list,the_index,
                                  ffd_parm,the_box);
       SM_region_controlbox_dispose(the_box);
       SM_region_controlbox_dispose(the_N_box);
    }
}

void
AU_eye_move_vertical(the_surface,the_region_list,the_name,the_value)
SM_Surface_ptr the_surface;
SM_Region_List_ptr the_region_list;
SM_Region_Name the_name;
float the_value;

{
    SM_Region_Controlbox_ptr the_box;
    int the_index,l,m,n;
    SM_ffd_parameters ffd_parm;

    the_index = SM_region_index_get(the_region_list,the_name);
    l = 3; m = 3; n = 3;
    if (the_index != SM_INDEX_ERROR) {
       the_box = SM_region_controlbox_new(l,m,n);
       SM_region_controlbox_recreate(the_box,l,m,n); 
       SM_region_controlbox_get(the_surface,the_region_list,the_index,the_box);
       AU_face_ffd_init(the_box,&ffd_parm); 
       SMGD_rotate_object_controlbox(the_box,X_INDEX,the_value);
       SMGD_region_freeform_apply(the_surface,the_region_list,the_name,
                                  ffd_parm,the_box);
       SM_region_controlbox_dispose(the_box);
    }
}

void
AU_eye_move_out(the_surface,the_region_list,the_name,the_value)
SM_Surface_ptr the_surface;
SM_Region_List_ptr the_region_list;
SM_Region_Name the_name;
float the_value;

{
    SM_Region_Controlbox_ptr the_box;
    int the_index,l,m,n;
    SM_ffd_parameters ffd_parm;
    VECTOR_3D t_vect;

    the_index = SM_region_index_get(the_region_list,the_name);
    l = 3; m = 3; n = 3;
    the_value *= 5.0;
    V3D_INIT(t_vect,0.0,0.0,the_value);
    if (the_index != SM_INDEX_ERROR) {
       the_box = SM_region_controlbox_new(l,m,n);
       SM_region_controlbox_recreate(the_box,l,m,n); 
       SM_region_controlbox_get(the_surface,the_region_list,the_index,the_box);
       AU_face_ffd_init(the_box,&ffd_parm); 
       SMGD_translate_object_controlbox(the_box,t_vect);
       SMGD_region_freeform_apply(the_surface,the_region_list,the_name,
                                  ffd_parm,the_box);
       SM_region_controlbox_dispose(the_box);
    }
}

*****/
/*----------------------------------------------------------------------*/
void
AU_eye_brow_raise(the_surface,the_face,the_region_list,the_name,which,the_value)
SM_Surface_ptr the_surface, the_face;
SM_Region_List_ptr the_region_list;
SM_Region_Name the_name;
int which;
float the_value;

{
    SM_Region_Controlbox_ptr the_box;
    int the_index,l,m,n;
    SM_ffd_parameters ffd_parm;
    BOOL plane_on;
    int vert_index,plane_dir;
    VECTOR_3D t_vect;

    the_index = SM_region_index_get(the_region_list,the_name);
    l = 3; m = 3; n = 3;
    plane_on = NO;
    V3D_INIT(t_vect,0.0,the_value*1.2,0.0);
     switch (which) {
      case 0 : /* all    */
             plane_on = NO;
      break;
      case 1 : /* left   */
               vert_index = 0;
               plane_dir = X_INDEX;
               plane_on = YES;
      break;
      case 2 : /* middle */
               vert_index = 1;
               plane_dir = X_INDEX;
               plane_on = YES;
      break;
      case 3 : /* right  */
               vert_index = 2;
               plane_dir = X_INDEX;
               plane_on = YES;
      break;
      default :
      break;
    }
    if (the_index != SM_INDEX_ERROR) {
       the_box = SM_region_controlbox_new(l,m,n);
       SM_region_controlbox_recreate(the_box,l,m,n); 
       SM_region_controlbox_get(the_face,the_region_list,the_index,the_box);
       AU_face_ffd_init(the_box,&ffd_parm); 
       if (plane_on) 
        SMGD_translate_object_control_plane(the_box,vert_index,t_vect,plane_dir);
       else
        SMGD_translate_object_controlbox(the_box,t_vect);
       SMGD_region_face_freeform_apply(the_surface,the_face,the_region_list,the_name,
                                  ffd_parm,the_box);
       SM_region_controlbox_dispose(the_box);
    }
    /*
    if (the_index != SM_INDEX_ERROR) {
       the_box = SM_region_controlbox_new(l,m,n);
       SM_region_controlbox_recreate(the_box,l,m,n); 
       SM_region_controlbox_get(the_surface,the_region_list,the_index,the_box);
       AU_face_ffd_init(the_box,&ffd_parm); 
       if (plane_on) 
        SMGD_shift_object_control_plane(the_box,vert_index,the_value,Y_INDEX,plane_dir);
       else
        SMGD_shift_object_controlbox(the_box,the_value,Y_INDEX);
       SMGD_region_freeform_apply(the_surface,the_region_list,the_name,
                                  ffd_parm,the_box);
       SM_region_controlbox_dispose(the_box);
    }
    */
}


/*---------------------------------------------------------------------*/

void
AU_eye_brow_left_hori(the_surface,the_face,the_region_list,the_name,the_value)
SM_Surface_ptr the_surface, the_face;
SM_Region_List_ptr the_region_list;
SM_Region_Name the_name;
double the_value;

{
    SM_Region_Controlbox_ptr the_box;
    int the_index,l,m,n;
    SM_ffd_parameters ffd_parm;
    int vert_index,plane_dir;
    VECTOR_3D t_vect;

    the_index = SM_region_index_get(the_region_list,the_name);
    l = 3; m = 3; n = 3;
    V3D_INIT(t_vect,the_value,0.0,0.0);
    plane_dir = X_INDEX;
    if (the_index != SM_INDEX_ERROR) {
       the_box = SM_region_controlbox_new(l,m,n);
       SM_region_controlbox_recreate(the_box,l,m,n); 
       SM_region_controlbox_get(the_face,the_region_list,the_index,the_box);
       AU_face_ffd_init(the_box,&ffd_parm); 
       vert_index = 1;
       SMGD_translate_object_control_plane(the_box,vert_index,t_vect,plane_dir);
       vert_index = 2;
       SMGD_translate_object_control_plane(the_box,vert_index,t_vect,plane_dir);
       /*
       SMGD_shift_object_controlbox(the_box,my_value * 0.2,X_INDEX);
       */
       SMGD_region_face_freeform_apply(the_surface,the_face,the_region_list,the_name,
                                  ffd_parm,the_box);
       SM_region_controlbox_dispose(the_box);
    }
}

/*---------------------------------------------------------------------*/

void
AU_eye_brow_right_hori(the_surface,the_face,the_region_list,the_name,the_value)
SM_Surface_ptr the_surface, the_face;
SM_Region_List_ptr the_region_list;
SM_Region_Name the_name;
double the_value;

{
    SM_Region_Controlbox_ptr the_box;
    int the_index,l,m,n;
    SM_ffd_parameters ffd_parm;
    int vert_index,plane_dir;
    VECTOR_3D t_vect;

    the_index = SM_region_index_get(the_region_list,the_name);
    l = 3; m = 3; n = 3;
    V3D_INIT(t_vect,the_value,0.0,0.0);
    plane_dir = X_INDEX;
    if (the_index != SM_INDEX_ERROR) {
       the_box = SM_region_controlbox_new(l,m,n);
       SM_region_controlbox_recreate(the_box,l,m,n); 
       SM_region_controlbox_get(the_face,the_region_list,the_index,the_box);
       AU_face_ffd_init(the_box,&ffd_parm); 
       vert_index = 1;
       SMGD_translate_object_control_plane(the_box,vert_index,t_vect,plane_dir);
       vert_index = 0;
       SMGD_translate_object_control_plane(the_box,vert_index,t_vect,plane_dir);
       /*
       SMGD_shift_object_controlbox(the_box,my_value * 0.2,X_INDEX);
       */
       SMGD_region_face_freeform_apply(the_surface,the_face,the_region_list,the_name,
                                  ffd_parm,the_box);
       SM_region_controlbox_dispose(the_box);
    }
}

/*---------------------------------------------------------------------*/


void
AU_eye_brow_hori(the_surface,the_face,the_region_list,the_name,the_value)
SM_Surface_ptr the_surface, the_face;
SM_Region_List_ptr the_region_list;
SM_Region_Name the_name;
float the_value;

{
    if (strcmp(the_name,"right_eyebrow") == 0) 
     AU_eye_brow_right_hori(the_surface,the_face,the_region_list,the_name,-the_value);
    else
     AU_eye_brow_left_hori(the_surface,the_face,the_region_list,the_name,the_value);
}

/*-----------------------------------------------------------------------*/
BOOL
AU_jaw_lower_postproc(the_surface,the_region_list,llip_index,the_pointlist,
                      old_center,new_center)
SM_Surface_ptr the_surface;
SM_Region_List_ptr the_region_list;
int llip_index;
AU_pointlist_ptr the_pointlist;
VECTOR_3D old_center,new_center;

{
   SM_Region_ptr the_region;
   int v_index,index1,index2;
   VECTOR_3D old_point, line_dir, X1, X2, the_point;
   float mag,dist,span,factor;
   float value1,value2;

   the_region = &the_region_list->region_list[llip_index];


   SM_region_mincoord_get(the_surface,the_region_list,
                          llip_index,X_INDEX,&value1); 
   SM_region_maxcoord_get(the_surface,the_region_list,
                          llip_index,X_INDEX,&value2); 

   SM_region_mincoord_index_get(the_surface,the_region_list,
                          llip_index,X_INDEX,&index1); 
   SM_region_maxcoord_index_get(the_surface,the_region_list,
                          llip_index,X_INDEX,&index2); 

   if (SM_vertex_index_valid(the_surface,index1)) 
     SM_vertex_point_get(the_surface,index1,X1);
   else
     return(NO);


   if (SM_vertex_index_valid(the_surface,index2)) 
     SM_vertex_point_get(the_surface,index2,X2);
   else
     return(NO);


   V3D_SUB(X2,X2,X1);
   span = V3D_NORM(X2);
   span = span/2.0;
   V3D_SUB(line_dir,new_center,old_center);
   mag = V3D_NORM(line_dir);

   if ( (mag < SM_EPSILON) || (span < SM_EPSILON) )
     return(NO);
   else
     V3D_NORMALIZE(line_dir,line_dir);

   SM_FOR_EACH_REGION_VERTEX(the_region)
     v_index = region_vertex_ptr->id;
     if (SM_vertex_index_valid(the_surface,v_index)) {
       AU_get_apoint(the_pointlist,v_index,old_point);  
       SM_vertex_point_get(the_surface,v_index,the_point);
       V3D_SUB(the_point,the_point,old_point);      
       if (SMGD_get_perpendicular_distance(old_point,old_center,line_dir,&dist))
       {
        factor = 1 - (dist * dist) / (span * span);
        V3D_SCALE(the_point,factor,the_point);
        V3D_ADD(the_point,the_point,old_point);
        SM_vertex_point_set(the_surface,v_index,the_point);
       }
     }
   SM_END_FOR_EACH

   return(YES);
}

/*****
void
AU_jaw_lower_postproc(the_surface,the_face,the_region_list,llip_index)
SM_Surface_ptr the_surface;
SM_Surface_ptr the_face;
SM_Region_List_ptr the_region_list;
int llip_index;

{
   SM_Region_ptr the_region;
   int v_index;
   VECTOR_3D old_center,new_center,old_point;
   float num,deno,mean_x;
   float value1,value2,delta_x,delta_y,old_x,old_y,new_y;

   the_region = &the_region_list->region_list[llip_index];
   SM_region_center_get(the_surface,the_region_list,llip_index,new_center); 
   SM_region_center_get(the_face,the_region_list,llip_index,old_center); 
   delta_y = fabs(new_center[1] - old_center[1]);
   SM_region_mincoord_get(the_face,the_region_list,
                          llip_index,X_INDEX,&value1); 
   SM_region_maxcoord_get(the_face,the_region_list,
                          llip_index,X_INDEX,&value2); 
   delta_x = fabs(value2 - value1)/2.0;
   mean_x = fabs(value2 + value1)/2.0;
   SM_FOR_EACH_REGION_VERTEX(the_region)
     v_index = region_vertex_ptr->id;
     if (SM_vertex_index_valid(the_surface,v_index)) {
        SM_vertex_point_get(the_face,v_index,old_point);
        old_x = old_point[0]; old_y = old_point[1];
        num = (old_x-mean_x) * (old_x-mean_x);
        deno = delta_x * delta_x;
        new_y = old_y - delta_y * (1 - num/deno); 
        old_point[1] = new_y;
        SM_vertex_point_set(the_surface,v_index,old_point);
     }
   SM_END_FOR_EACH
}
****/
/*-----------------------------------------------------------------------*/

void
AU_jaw_move_lower(the_surface,the_face,the_region_list,the_name,the_value)
SM_Surface_ptr the_surface, the_face;
SM_Region_List_ptr the_region_list;
SM_Region_Name the_name;
float the_value;

{
    SM_Region_Controlbox_ptr the_box;
    int the_index,l,m,n,llip_index;
    SM_ffd_parameters ffd_parm;
    SM_Region_Name llip_name;
    AU_pointlist_ptr reg_copy;
    VECTOR_3D old_center, new_center;

    the_index = SM_region_index_get(the_region_list,the_name);
    l = 3; m = 3; n = 3;
    the_value = -the_value*0.04;
    if (the_index != SM_INDEX_ERROR) {
       the_box = SM_region_controlbox_new(l,m,n);
       SM_region_controlbox_recreate(the_box,l,m,n); 
       SM_region_controlbox_get(the_face,the_region_list,the_index,the_box);
       AU_face_ffd_init(the_box,&ffd_parm); 
       SMGD_shearmin_object_controlbox(the_box,the_value,Y_INDEX);
       SMGD_rotate_object_controlbox(the_box,X_INDEX,-the_value * 0.5);
       strcpy(llip_name,"lower_lip");
       llip_index = SM_region_index_get(the_region_list,llip_name);
       SM_region_center_get(the_surface,the_region_list,llip_index,old_center); 
       reg_copy = AU_pointlist_new();
       AU_set_pointlist(the_surface,the_region_list,the_index,reg_copy);   
       SMGD_region_face_freeform_apply(the_surface,the_face,the_region_list,the_name,
                                  ffd_parm,the_box);

       SM_region_controlbox_dispose(the_box);
       SM_region_center_get(the_surface,the_region_list,llip_index,new_center); 
       AU_jaw_lower_postproc(the_surface,the_region_list,llip_index,reg_copy,
                             old_center,new_center);
       AU_pointlist_dispose(reg_copy);
    }
}

/*-----------------------------------------------------------------------*/

void
AU_jaw_move_hori(the_surface,the_face,the_region_list,the_name,the_value)
SM_Surface_ptr the_surface, the_face;
SM_Region_List_ptr the_region_list;
SM_Region_Name the_name;
float the_value;

{
    SM_Region_Controlbox_ptr the_box;
    int the_index,l,m,n;
    SM_ffd_parameters ffd_parm;
    int vert_index, plane_dir;

    the_index = SM_region_index_get(the_region_list,the_name);
    l = 3; m = 3; n = 3;
    the_value = the_value*0.054;
    if (the_index != SM_INDEX_ERROR) {
       the_box = SM_region_controlbox_new(l,m,n);
       SM_region_controlbox_recreate(the_box,l,m,n); 
       SM_region_controlbox_get(the_face,the_region_list,the_index,the_box);
       AU_face_ffd_init(the_box,&ffd_parm); 
       plane_dir = Y_INDEX;
       vert_index = 0;
       SMGD_shear_object_control_plane(the_box,vert_index,the_value,X_INDEX,plane_dir);
       SMGD_region_face_freeform_apply(the_surface,the_face,the_region_list,the_name,
                                  ffd_parm,the_box);
       SM_region_controlbox_dispose(the_box);
    }
}

/*-----------------------------------------------------------------------*/
void
AU_jaw_thrust(the_surface,the_face,the_region_list,the_name,the_value)
SM_Surface_ptr the_surface, the_face;
SM_Region_List_ptr the_region_list;
SM_Region_Name the_name;
float the_value;

{
    SM_Region_Controlbox_ptr the_box;
    int the_index,l,m,n;
    SM_ffd_parameters ffd_parm;

    the_index = SM_region_index_get(the_region_list,the_name);
    l = 3; m = 3; n = 3;
    the_value = the_value*0.12;
    if (the_index != SM_INDEX_ERROR) {
       the_box = SM_region_controlbox_new(l,m,n);
       SM_region_controlbox_recreate(the_box,l,m,n); 
       SM_region_controlbox_get(the_face,the_region_list,the_index,the_box);
       AU_face_ffd_init(the_box,&ffd_parm); 
       SMGD_shift_object_controlbox(the_box,the_value,Z_INDEX);
       SMGD_region_face_freeform_apply(the_surface,the_face,the_region_list,the_name,
                                  ffd_parm,the_box);
       SM_region_controlbox_dispose(the_box);
    }
}

/*-----------------------------------------------------------------------*/

void
AU_nose_comp_exp(the_surface,the_face,the_region_list,the_name,the_value)
SM_Surface_ptr the_surface, the_face;
SM_Region_List_ptr the_region_list;
SM_Region_Name the_name;
float the_value;

{
    SM_Region_Controlbox_ptr the_box;
    int the_index,l,m,n;
    SM_ffd_parameters ffd_parm;
    VECTOR_3D s_vect;

    the_index = SM_region_index_get(the_region_list,the_name);
    l = 3; m = 3; n = 3;
    the_value = 1.0 + the_value * 0.15;
    if (the_index != SM_INDEX_ERROR) {
       the_box = SM_region_controlbox_new(l,m,n);
       SM_region_controlbox_recreate(the_box,l,m,n); 
       SM_region_controlbox_get(the_face,the_region_list,the_index,the_box);
       AU_face_ffd_init(the_box,&ffd_parm); 
       V3D_INIT(s_vect,the_value,1.0,1.0);
       SMGD_scale_object_controlbox(the_box,s_vect);
       SMGD_region_face_freeform_apply(the_surface,the_face,the_region_list,the_name,
                                  ffd_parm,the_box);
       SM_region_controlbox_dispose(the_box);
    }
}

/*-----------------------------------------------------------------------*/
void
AU_nose_r_comp_exp(the_surface,the_face,the_region_list,the_name,the_value)
SM_Surface_ptr the_surface, the_face;
SM_Region_List_ptr the_region_list;
SM_Region_Name the_name;
float the_value;

{
    SM_Region_Controlbox_ptr the_box;
    int the_index,l,m,n;
    SM_ffd_parameters ffd_parm;

    the_index = SM_region_index_get(the_region_list,the_name);
    l = 3; m = 3; n = 3;
    the_value = the_value * 0.15;
    if (the_index != SM_INDEX_ERROR) {
       the_box = SM_region_controlbox_new(l,m,n);
       SM_region_controlbox_recreate(the_box,l,m,n); 
       SM_region_controlbox_get(the_face,the_region_list,the_index,the_box);
       AU_face_ffd_init(the_box,&ffd_parm); 
       SMGD_shift_object_control_plane(the_box,8,the_value,X_INDEX,X_INDEX);
       SMGD_region_face_freeform_apply(the_surface,the_face,the_region_list,the_name,
                                  ffd_parm,the_box);
       SM_region_controlbox_dispose(the_box);
    }
}

/*-----------------------------------------------------------------------*/
void
AU_nose_l_comp_exp(the_surface,the_face,the_region_list,the_name,the_value)
SM_Surface_ptr the_surface, the_face;
SM_Region_List_ptr the_region_list;
SM_Region_Name the_name;
float the_value;

{
    SM_Region_Controlbox_ptr the_box;
    int the_index,l,m,n;
    SM_ffd_parameters ffd_parm;

    the_index = SM_region_index_get(the_region_list,the_name);
    l = 3; m = 3; n = 3;
    the_value = -the_value * 0.15;
    if (the_index != SM_INDEX_ERROR) {
       the_box = SM_region_controlbox_new(l,m,n);
       SM_region_controlbox_recreate(the_box,l,m,n); 
       SM_region_controlbox_get(the_face,the_region_list,the_index,the_box);
       AU_face_ffd_init(the_box,&ffd_parm); 
       SMGD_shift_object_control_plane(the_box,0,the_value,X_INDEX,X_INDEX);
       SMGD_region_face_freeform_apply(the_surface,the_face,the_region_list,the_name,
                                  ffd_parm,the_box);
       SM_region_controlbox_dispose(the_box);
    }
}

/*------------------------------------------------------------------------*/

void
AU_nose_raise(the_surface,the_face,the_region_list,the_name,the_value)
SM_Surface_ptr the_surface, the_face;
SM_Region_List_ptr the_region_list;
SM_Region_Name the_name;
float the_value;

{
    SM_Region_Controlbox_ptr the_box;
    int the_index,l,m,n;
    SM_ffd_parameters ffd_parm;

    the_index = SM_region_index_get(the_region_list,the_name);
    l = 3; m = 3; n = 3;
    the_value = the_value * 0.15;
    if (the_index != SM_INDEX_ERROR) {
       the_box = SM_region_controlbox_new(l,m,n);
       SM_region_controlbox_recreate(the_box,l,m,n); 
       SM_region_controlbox_get(the_face,the_region_list,the_index,the_box);
       AU_face_ffd_init(the_box,&ffd_parm); 
       SMGD_shift_object_controlbox(the_box,the_value,Y_INDEX);
       SMGD_region_face_freeform_apply(the_surface,the_face, the_region_list,the_name,
                                  ffd_parm,the_box);
       SM_region_controlbox_dispose(the_box);
    }
}

/*------------------------------------------------------------------------*/
void
AU_nose_bend(the_surface,the_face,the_region_list,the_name,the_value)
SM_Surface_ptr the_surface, the_face;
SM_Region_List_ptr the_region_list;
SM_Region_Name the_name;
float the_value;

{
    SM_Region_Controlbox_ptr the_box;
    int the_index,l,m,n;
    SM_ffd_parameters ffd_parm;
    VECTOR_3D origin;
    

    the_index = SM_region_index_get(the_region_list,the_name);
    l = 3; m = 3; n = 3;
    the_value = the_value * 0.3;
    if (the_index != SM_INDEX_ERROR) {
       the_box = SM_region_controlbox_new(l,m,n);
       SM_region_controlbox_recreate(the_box,l,m,n); 
       SM_region_controlbox_get(the_face,the_region_list,the_index,the_box);
       AU_face_ffd_init(the_box,&ffd_parm); 
/*        SMGD_rotate_object_controlbox(the_box,Y_INDEX,the_value); */
       SMGD_controlbox_origin_get(the_box,origin);
       SMGD_rotate_face_region(the_surface,origin,the_region_list,the_name,Y_INDEX,the_value);
       /*            SMGD_region_face_freeform_apply(the_surface,the_face, the_region_list,the_name,
                                  ffd_parm,the_box); */
       SM_region_controlbox_dispose(the_box);
    }
}

/*------------------------------------------------------------------------*/

void
AU_mouth_comp_exp(the_surface,the_face,the_region_list,the_name,which,the_value) 
SM_Surface_ptr the_surface, the_face;
SM_Region_List_ptr the_region_list;
SM_Region_Name the_name;
int which;
float the_value;

{
    SM_Region_Controlbox_ptr the_box;
    int the_index,l,m,n;
    SM_ffd_parameters ffd_parm;
    VECTOR_3D origin;
    int vert_index;
    int k;    
    
    /*
    the_value = 1.0 + 0.4*the_value;
    */

    the_value = 0.18*the_value;
    the_index = -1;
    the_index = SM_region_index_get(the_region_list,the_name);
    
    l = 3; m = 3; n = 3;
     
    if (the_index != SM_INDEX_ERROR) {
       the_box = SM_region_controlbox_new(l,m,n);
       SM_region_controlbox_recreate(the_box,l,m,n); 
       SM_region_controlbox_get(the_face,the_region_list,the_index,the_box);
       SMGD_controlbox_origin_get(the_box,origin);
       AU_face_ffd_init(the_box,&ffd_parm); 
       switch (which) {
       	 case 0 : /* both left and right */
          for (k=0; k<n; k++) {
           vert_index = l + k*(m*n);
           SMGD_shift_object_control_point(the_box,vert_index,-the_value,X_INDEX,origin); 
           vert_index = l + k*(m*n) + l - 1;
           SMGD_shift_object_control_point(the_box,vert_index,the_value,X_INDEX,origin);
          }           
       	 break;
     	 case 1 : /* left   */
          for (k=0; k<n; k++) {
           vert_index = l + k*(m*n);
           SMGD_shift_object_control_point(the_box,vert_index,-the_value,X_INDEX,origin); 
          }
      	 break;
      	 case 2 : /* right */
          for (k=0; k<n; k++) {
           vert_index = l + k*(m*n) + l - 1;
           SMGD_shift_object_control_point(the_box,vert_index,the_value,X_INDEX,origin); 
          }
      	 break;
 	 default : /* both left and right */
          for (k=0; k<n; k++) {
           vert_index = l + k*(m*n);
           SMGD_shift_object_control_point(the_box,vert_index,-the_value,X_INDEX,origin); 
           vert_index = l + k*(m*n) + l - 1;
           SMGD_shift_object_control_point(the_box,vert_index,the_value,X_INDEX,origin); 
          }           
     	 break;
       }
       SMGD_region_face_freeform_apply(the_surface,the_face,the_region_list,the_name,
                                  ffd_parm,the_box);
       SM_region_controlbox_dispose(the_box);
    }
}

/*------------------------------------------------------------------------*/
/*** old version 
void
AU_mouth_comp_exp(the_surface,the_region_list,the_name,the_value) 
SM_Surface_ptr the_surface;
SM_Region_List_ptr the_region_list;
SM_Region_Name the_name;
float the_value;

{
    SM_Region_Controlbox_ptr the_box;
    int the_index,l,m,n;
    SM_ffd_parameters ffd_parm;
    VECTOR_3D s_vect;
    int vert_index, plane_dir;
    float local_scale_value, global_scale_value;

    the_index = SM_region_index_get(the_region_list,the_name);
    
    if (the_value > 0.0)
     local_scale_value = 1.0 + (1.25 - 1.0)*the_value;
    else
     local_scale_value = 1.0 + (1.0 - 1.0/1.25)*the_value;
     
	   
    global_scale_value = local_scale_value / AU_mouth_compexp_scale;
    AU_mouth_compexp_scale = local_scale_value;
 
    l = 3; m = 3; n = 3;
    if (the_index != SM_INDEX_ERROR) {
       the_box = SM_region_controlbox_new(l,m,n);
       SM_region_controlbox_recreate(the_box,l,m,n); 
       SM_region_controlbox_get(the_surface,the_region_list,the_index,the_box);
       AU_face_ffd_init(the_box,&ffd_parm); 
       V3D_INIT(s_vect,global_scale_value,1.0,1.0);
       vert_index = (l*m*n - 1)/2;
       plane_dir = Z_INDEX;
       SMGD_scale_object_control_plane(the_box,vert_index,s_vect,plane_dir);
       SMGD_region_freeform_apply(the_surface,the_region_list,the_name,
                                  ffd_parm,the_box);
       SM_region_controlbox_dispose(the_box);
    }
}
****/
/*------------------------------------------------------------------------*/

void
AU_mouth_beak(the_surface,the_face,the_region_list,the_name,the_value) 
SM_Surface_ptr the_surface, the_face;
SM_Region_List_ptr the_region_list;
SM_Region_Name the_name;
float the_value;

{
    SM_Region_Controlbox_ptr the_box;
    int the_index,l,m,n;
    SM_ffd_parameters ffd_parm;
    VECTOR_3D s_vect;
    int vert_index, plane_dir;
    float scalex_val,scaley_val,shiftz_val;

    the_index = SM_region_index_get(the_region_list,the_name);
    scalex_val = 1.0 - the_value * 0.4;
    scaley_val = 1.0 + the_value * 0.4;
    shiftz_val = the_value * 0.4;
    l = 3; m = 3; n = 3;
    if (the_index != SM_INDEX_ERROR) {
       the_box = SM_region_controlbox_new(l,m,n);
       SM_region_controlbox_recreate(the_box,l,m,n); 
       SM_region_controlbox_get(the_face,the_region_list,the_index,the_box);
       AU_face_ffd_init(the_box,&ffd_parm); 
       vert_index = (l*m*n - 1)/2;
       plane_dir = Z_INDEX;
       V3D_INIT(s_vect,scalex_val,1.0,1.0);
       SMGD_scale_object_control_plane(the_box,vert_index,s_vect,plane_dir);
       plane_dir = X_INDEX;
       V3D_INIT(s_vect,1.0,scaley_val,1.0);
       SMGD_scale_object_control_plane(the_box,vert_index,s_vect,plane_dir);
       SMGD_shift_object_control_plane(the_box,vert_index,
                                       shiftz_val,Z_INDEX,plane_dir);
       SMGD_region_face_freeform_apply(the_surface,the_face,the_region_list,the_name,
                                  ffd_parm,the_box);
       SM_region_controlbox_dispose(the_box);
    }
}

/*-----------------------------------------------------------------------*/

void
AU_lip_lower_raiser(the_surface,the_face,the_region_list,the_name,the_value) 
SM_Surface_ptr the_surface, the_face;
SM_Region_List_ptr the_region_list;
SM_Region_Name the_name;
float the_value;

{
    SM_Region_Controlbox_ptr the_box;
    int the_index,l,m,n;
    SM_ffd_parameters ffd_parm;
    int vert_index, plane_dir;

    the_index = SM_region_index_get(the_region_list,the_name);
    l = 3; m = 3; n = 3;
    the_value = - the_value;
    if (the_index != SM_INDEX_ERROR) {
       the_box = SM_region_controlbox_new(l,m,n);
       SM_region_controlbox_recreate(the_box,l,m,n); 
       SM_region_controlbox_get(the_face,the_region_list,the_index,the_box);
       AU_face_ffd_init(the_box,&ffd_parm); 
       vert_index = (l*m*n - 1)/2;
       plane_dir = X_INDEX;
       SMGD_shift_object_control_plane(the_box,vert_index,
                                       the_value * 0.8,Y_INDEX,plane_dir);
       SMGD_region_face_freeform_apply(the_surface,the_face,the_region_list,the_name,
                                  ffd_parm,the_box);
       SM_region_controlbox_dispose(the_box);
    }

}

/*-------------------------------------------------------------------------*/

void
AU_cheek_puff_squeeze(the_surface,the_face,the_region_list,the_name,the_value)
SM_Surface_ptr the_surface, the_face;
SM_Region_List_ptr the_region_list;
SM_Region_Name the_name;
float the_value;

{
    SM_Region_Controlbox_ptr the_box;
    int the_index,l,m,n;
    SM_ffd_parameters ffd_parm;

    the_index = SM_region_index_get(the_region_list,the_name);
    l = 3; m = 3; n = 3;
    if (strcmp(the_name,"left_cheek") == 0) 
     the_value = -the_value;

    if (the_index != SM_INDEX_ERROR) {
       the_box = SM_region_controlbox_new(l,m,n);
       SM_region_controlbox_recreate(the_box,l,m,n); 
       SM_region_controlbox_get(the_face,the_region_list,the_index,the_box);
       AU_face_ffd_init(the_box,&ffd_parm); 
       SMGD_shift_object_controlbox(the_box,the_value * 0.2,X_INDEX);
       SMGD_region_face_freeform_apply(the_surface,the_face,the_region_list,the_name,
                                  ffd_parm,the_box);
       SM_region_controlbox_dispose(the_box);
    }

}

/*-------------------------------------------------------------------------*/

void
AU_cheek_find_dispvect(the_box,the_value,side)
SM_Region_Controlbox_ptr the_box;
float the_value;
int side;

{

     VECTOR_3D point1,point2,disp_vect,apply_vect;
     int i,j,k,l;
     int left,right;

     l = 3; i = 10; left = 1; right = 0; 
     if (side == right) { 
       j = i+2*l+1;
       k = i-1;
     }
     else {
       j = i+2*l-1; 
       k = i+1;
     }
     V3D_COPY(point1,the_box->vertex_list[k].point);
     V3D_COPY(point2,the_box->vertex_list[j].point);
     V3D_SUB(disp_vect,point2,point1);
     V3D_SCALE(disp_vect,0.5,disp_vect);
     V3D_SCALE(apply_vect,the_value*0.5,disp_vect);

     V3D_ADD(the_box->vertex_list[i].point,the_box->vertex_list[i].point,
             apply_vect);

     if (side == right) {
       V3D_ADD(the_box->vertex_list[i+1].point,the_box->vertex_list[i+1].point,
             apply_vect);
       V3D_ADD(the_box->vertex_list[i+4].point,
             the_box->vertex_list[i+4].point,apply_vect);
     }
     else { 
       V3D_ADD(the_box->vertex_list[i-1].point,the_box->vertex_list[i-1].point,
             apply_vect);
       V3D_ADD(the_box->vertex_list[i+2].point,
             the_box->vertex_list[i+2].point,apply_vect);
     }

     V3D_ADD(the_box->vertex_list[i+6].point,the_box->vertex_list[i+6].point,
             apply_vect);
     V3D_ADD(the_box->vertex_list[j].point,the_box->vertex_list[j].point,
             apply_vect);

     V3D_SCALE(apply_vect,the_value,disp_vect);
     V3D_ADD(the_box->vertex_list[i+3].point,the_box->vertex_list[i+3].point,
             apply_vect);
}


void
AU_cheek_zygo(the_surface,the_face,the_region_list,the_name,the_value) 
SM_Surface_ptr the_surface, the_face;
SM_Region_List_ptr the_region_list;
SM_Region_Name the_name;
float the_value;

{
    SM_Region_Controlbox_ptr the_box;
    int the_index,l,m,n;
    SM_ffd_parameters ffd_parm;
    int side;
     
    the_index = SM_region_index_get(the_region_list,the_name);
    l = 3; m = 3; n = 3;
    if (strcmp(the_name,"left_zygo") == 0) side = 1;
    if (strcmp(the_name,"right_zygo") == 0) side = 0;
    if (the_index != SM_INDEX_ERROR) {
       the_box = SM_region_controlbox_new(l,m,n);
       SM_region_controlbox_recreate(the_box,l,m,n); 
       SM_region_controlbox_get(the_face,the_region_list,the_index,the_box);
       AU_face_ffd_init(the_box,&ffd_parm); 
       AU_cheek_find_dispvect(the_box,the_value * 0.4,side);    
       SMGD_region_face_freeform_apply(the_surface,the_face,the_region_list,the_name,
                                  ffd_parm,the_box);
       SM_region_controlbox_dispose(the_box);
    }

}

/*-------------------------------------------------------------------------*/

void
AU_cheek_lift(the_surface,the_face,the_region_list,the_name,the_value)
SM_Surface_ptr the_surface, the_face;
SM_Region_List_ptr the_region_list;
SM_Region_Name the_name;
float the_value;

{
    SM_Region_Controlbox_ptr the_box;
    int the_index,l,m,n;
    SM_ffd_parameters ffd_parm;
    float x_value,y_value;

    the_index = SM_region_index_get(the_region_list,the_name);
    l = 3; m = 3; n = 3;
    y_value = the_value * 0.20;
    if (strcmp(the_name,"left_cheek") == 0)
     x_value = - the_value * 0.1;
    else
     x_value = the_value * 0.1;
    if (the_index != SM_INDEX_ERROR) {
       the_box = SM_region_controlbox_new(l,m,n);
       SM_region_controlbox_recreate(the_box,l,m,n); 
       SM_region_controlbox_get(the_face,the_region_list,the_index,the_box);
       AU_face_ffd_init(the_box,&ffd_parm); 
       SMGD_shift_object_controlbox(the_box,x_value,X_INDEX);
       SMGD_shift_object_controlbox(the_box,y_value,Y_INDEX);
       SMGD_region_face_freeform_apply(the_surface,the_face,the_region_list,the_name,
                                  ffd_parm,the_box);
       SM_region_controlbox_dispose(the_box);
    }

}

/*-------------------------------------------------------------------------*/

void
AU_cheek_lower_cornerlip(the_surface,the_face,the_region_list,the_name,the_value)
SM_Surface_ptr the_surface, the_face;
SM_Region_List_ptr the_region_list;
SM_Region_Name the_name;
float the_value;

{
    SM_Region_Controlbox_ptr the_box;
    int the_index,l,m,n;
    SM_ffd_parameters ffd_parm;
    float left_value,mid_value,right_value;
    int plane_dir;

    the_index = SM_region_index_get(the_region_list,the_name);
    l = 3; m = 3; n = 3;
    plane_dir = X_INDEX;
    if (strcmp(the_name,"left_zygo") == 0) {
     left_value  = - the_value * 0.17;
     mid_value   = - the_value * 0.17;
     right_value = the_value * 0.;
    }     
    else {
     left_value  = the_value * 0.;
     mid_value   = - the_value * 0.17;
     right_value = - the_value * 0.17;
    }
    if (the_index != SM_INDEX_ERROR) {
       the_box = SM_region_controlbox_new(l,m,n);
       SM_region_controlbox_recreate(the_box,l,m,n); 
       SM_region_controlbox_get(the_face,the_region_list,the_index,the_box);
       AU_face_ffd_init(the_box,&ffd_parm); 
       SMGD_shift_object_control_plane(the_box,0,left_value,Y_INDEX,plane_dir);
       SMGD_shift_object_control_plane(the_box,1,mid_value,Y_INDEX,plane_dir);
       SMGD_shift_object_control_plane(the_box,2,right_value,Y_INDEX,plane_dir);
       SMGD_region_face_freeform_apply(the_surface,the_face,the_region_list,the_name,
                                  ffd_parm,the_box);
       SM_region_controlbox_dispose(the_box);
    }

}

/*-------------------------------------------------------------------------*/
void
AU_mouth_apply_dispvect(the_box,the_value,index1,index2)
SM_Region_Controlbox_ptr the_box;
float the_value;
int index1,index2;
{
   VECTOR_3D point1, point2, disp_vect, apply_vect;

       V3D_COPY(point1,the_box->vertex_list[index1].point);
       V3D_COPY(point2,the_box->vertex_list[index2].point);
       V3D_SUB(disp_vect,point2,point1);
       V3D_SCALE(apply_vect,the_value*0.5,disp_vect);
       V3D_ADD(the_box->vertex_list[index2].point,
               the_box->vertex_list[index2].point,apply_vect);
}

void
AU_mouth_find_dispvect(the_box,the_value,side)
SM_Region_Controlbox_ptr the_box;
float the_value;
int side;

{
     int i,j,l;
     int upper,lower;

     l = 3; i = 13; upper = 1; lower = 0; 
     if (side == upper) { 
       j = i+1;
       AU_mouth_apply_dispvect(the_box,the_value*0.8,i,j);
       j = i+l+1;
       AU_mouth_apply_dispvect(the_box,the_value,i,j);
       j = i+l;
       AU_mouth_apply_dispvect(the_box,the_value,i,j);
       j = i+l-1;
       AU_mouth_apply_dispvect(the_box,the_value,i,j);
       j = i-1;
       AU_mouth_apply_dispvect(the_box,the_value*0.8,i,j);
       j = i-l;
       AU_mouth_apply_dispvect(the_box,-the_value,i,j);
     }
     else {
       j = i+1;
       AU_mouth_apply_dispvect(the_box,the_value*0.8,i,j);
       j = i-l+1;
       AU_mouth_apply_dispvect(the_box,the_value,i,j);
       j = i-l;
       AU_mouth_apply_dispvect(the_box,the_value,i,j);
       j = i-l-1;
       AU_mouth_apply_dispvect(the_box,the_value,i,j);
       j = i-1;
       AU_mouth_apply_dispvect(the_box,the_value*0.8,i,j);
       j = i+l;
       AU_mouth_apply_dispvect(the_box,-the_value,i,j);
     }
}

/*------------------------------------------------------------------*/

void
AU_mouth_risorius(the_surface,the_face,the_region_list,the_name,the_value) 
SM_Surface_ptr the_surface, the_face;
SM_Region_List_ptr the_region_list;
SM_Region_Name the_name;
float the_value;

{
    SM_Region_Controlbox_ptr the_box;
    int the_index,l,m,n;
    SM_ffd_parameters ffd_parm;
    int side;
     
    the_index = SM_region_index_get(the_region_list,the_name);
    l = 3; m = 3; n = 3;
    if (strcmp(the_name,"upper_lip") == 0) side = 1;
    if (strcmp(the_name,"lower_lip") == 0) side = 0;
    if (the_index != SM_INDEX_ERROR) {
       the_box = SM_region_controlbox_new(l,m,n);
       SM_region_controlbox_recreate(the_box,l,m,n); 
       SM_region_controlbox_get(the_face,the_region_list,the_index,the_box);
       AU_face_ffd_init(the_box,&ffd_parm); 
       AU_mouth_find_dispvect(the_box,the_value * 0.5,side);    
       SMGD_region_face_freeform_apply(the_surface,the_face,the_region_list,the_name,
                                  ffd_parm,the_box);
       SM_region_controlbox_dispose(the_box);
    }

}

/*-------------------------------------------------------------------------*/
void
AU_eyelid_lower_updown(the_surface,the_face,the_region_list,the_name,the_value)
SM_Surface_ptr the_surface, the_face;
SM_Region_List_ptr the_region_list;
SM_Region_Name the_name;
float the_value;

{
    SM_Region_Controlbox_ptr the_box;
    int the_index,l,m,n;
    SM_ffd_parameters ffd_parm;
    VECTOR_3D t_vect;
     
    the_index = SM_region_index_get(the_region_list,the_name);
    l = 3; m = 3; n = 3;
    if (the_index != SM_INDEX_ERROR) {
       the_box = SM_region_controlbox_new(l,m,n);
       SM_region_controlbox_recreate(the_box,l,m,n); 
       SM_region_controlbox_get(the_face,the_region_list,the_index,the_box);
       AU_face_ffd_init(the_box,&ffd_parm); 
/*
       V3D_INIT(t_vect, 0.0, the_value * 0.2, -the_value * 0.1);
*/
       V3D_INIT(t_vect, 0.0, the_value * 0.7, -the_value * 0.2);
       SMGD_translate_object_controlbox(the_box,t_vect);
/*
       SMGD_shift_object_controlbox(the_box,the_value * 0.2,Y_INDEX);
       SMGD_shift_object_controlbox(the_box,-the_value*0.5*0.2,Z_INDEX);
*/
/*
       SMGD_rotate_object_controlbox(the_box,X_INDEX,-the_value*0.1);
*/
       SMGD_region_face_freeform_apply(the_surface,the_face,the_region_list,the_name,
                                  ffd_parm,the_box);
       SM_region_controlbox_dispose(the_box);
    }
}
/*-----------------------------------------------------------------------*/
BOOL
AU_find_inside_triangle(the_surface,t_index,the_point)
SM_Surface_ptr the_surface;
int t_index;
VECTOR_3D the_point;
{
  VECTOR_3D p1,p2,p3,vect1,vect2,res;
  int v1,v2,v3;
  float u,v,w,sum;
  float Au,Av,Aw,A;
  
  SM_triangle_vertex_index_get(the_surface, t_index, &v1, &v2, &v3); 
  SM_vertex_point_get(the_surface, v1, p1);
  SM_vertex_point_get(the_surface, v2, p2);
  SM_vertex_point_get(the_surface, v3, p3);
  /* area A */
  V3D_SUB(vect1,p2,p1);
  V3D_SUB(vect2,p3,p1);
  V3D_CROSS(res,vect1,vect2);
  A = V3D_NORM(res);    
  /* area Au */
  V3D_SUB(vect2,the_point,p1);
  V3D_CROSS(res,vect1,vect2);
  Au = V3D_NORM(res);    
  /* area Av */ 
  V3D_SUB(vect1,p3,p1);
  V3D_CROSS(res,vect1,vect2);
  Av = V3D_NORM(res);    
  /* area Aw */
  V3D_SUB(vect1,p3,p2);
  V3D_SUB(vect2,the_point,p2);
  V3D_CROSS(res,vect1,vect2);
  Aw = V3D_NORM(res);    
  
  if (A < SM_EPSILON) return(NO);
  u = Au/A; v = Av/A; w = Aw/A;
  
  sum = u + v + w;
  if (fabs(sum-1.0) < SM_EPSILON) {
    if (  ((u >= 0) && (u <= 1))
	&& ((v >= 0) && (v <= 1))
	&& ((w >= 0) && (w <= 1)) ) {
      return(YES);
    }
  }
  return(NO);
}


/*-----------------------------------------------------------------*/
BOOL
AU_find_ray_intersection(the_surface,ray_o,ray_d,tri_index,point_int)
SM_Surface_ptr the_surface;
VECTOR_3D ray_o,ray_d;
int tri_index;
VECTOR_3D point_int;

{

float t_min,d,t_deno,t_nume,t;
VECTOR_3D point,normal,the_point,dir;
int index1, index2, index3;
BOOL intersection;
float mag;

    t_min = 100000;
    V3D_COPY(point_int,ray_o);
    intersection = NO;
    mag = V3D_NORM(ray_d);
    if (mag < SM_EPSILON) return(NO);
    V3D_NORMALIZE(ray_d,ray_d);
    SM_triangle_normal_get(the_surface, tri_index, normal);
    SM_triangle_vertex_index_get(the_surface, tri_index, 
                                         &index1, &index2, &index3);
    SM_vertex_point_get(the_surface, index1, point);
     d = - V3D_DOT(normal, point);
     t_deno = V3D_DOT(normal, ray_d);
     t_nume = V3D_DOT(normal, ray_o);
      if (fabs(t_deno) >= SM_EPSILON) {
	    t_nume = - (t_nume + d);
	    t = t_nume/t_deno;
	    if ((t > 0) && (t <= mag)) {
                t_min = t; 
                V3D_SCALE(dir,t,ray_d);
                V3D_ADD(the_point,ray_o,dir);
                if (AU_find_inside_triangle(the_surface,tri_index,the_point))
                {
                 V3D_COPY(point_int,the_point);
                 printf("intersection yes \n");
                 intersection = YES;
                }
	    } 
	}
    return(intersection);
}
/*--------------------------------------------------------------------*/
BOOL
AU_eye_intersection(the_surface,the_region_list,new_point,old_point,with_name)
SM_Surface_ptr the_surface;
SM_Region_List_ptr the_region_list;
VECTOR_3D old_point, new_point;
SM_Region_Name with_name;
{
    SM_Region_ptr r_ptr;
    VECTOR_3D ray_o, ray_d, point_int;
    int t, index;
    BOOL stop;
       
    index = SM_region_index_get(the_region_list,with_name);
    r_ptr = &the_region_list->region_list[index];
    V3D_COPY(ray_o,old_point);
    V3D_SUB(ray_d,new_point,old_point);
    stop = NO;
    SM_FOR_EACH_REGION_TRIANGLE(r_ptr)
    if (!stop) {
     t = *region_tri_ptr;
     printf(" the triangle index %d \n",t);
     if (AU_find_ray_intersection(the_surface,ray_o,ray_d,t,point_int)) { 
      V3D_COPY(new_point,point_int); 
      stop = YES;
     }
    }
    SM_END_FOR_EACH
    return(stop);
}
       
/*-----------------------------------------------------------------------*/

void
AU_region_freeform_apply(the_surface,the_region_list,
         region_current_name, with_name, globalDeform,the_controlbox)
SM_Surface_ptr the_surface;
SM_Region_List_ptr the_region_list;
SM_Region_Name region_current_name, with_name;
SM_ffd_parameters globalDeform;
SM_Region_Controlbox_ptr the_controlbox;

{

   float s,t,u;
   int i,j,k,l,m,n,index;
   float Bl,Bn,Bm,Blmn;
   VECTOR_3D ffd,X_vect,temp,the_point;
   float nume,denom;
   SM_Region_ptr the_region_ptr;
   int the_index, v_index;
   BOOL anchor;
   VECTOR_3D old_point, new_point;

   l = the_controlbox->l; m = the_controlbox->m; n = the_controlbox->n;

   the_index = SM_region_index_get(the_region_list,region_current_name);
   the_region_ptr = &the_region_list->region_list[the_index];
   SM_FOR_EACH_REGION_VERTEX(the_region_ptr) 
    v_index = region_vertex_ptr->id;
    anchor = region_vertex_ptr->anchor;
    if ((SM_vertex_index_valid(the_surface,v_index)) && (!anchor) ) {
     SM_vertex_point_get(the_surface,v_index,the_point);
     V3D_SUB(X_vect,the_point,globalDeform.X0);
     V3D_CROSS(temp,globalDeform.T_vect,globalDeform.U_vect);
     nume = V3D_DOT(temp,X_vect);
     denom = V3D_DOT(temp,globalDeform.S_vect);
     s = nume/denom;

     V3D_CROSS(temp,globalDeform.S_vect,globalDeform.U_vect);
     nume = V3D_DOT(temp,X_vect);
     denom = V3D_DOT(temp,globalDeform.T_vect);
     t = nume/denom;
 

     V3D_CROSS(temp,globalDeform.S_vect,globalDeform.T_vect);
     nume = V3D_DOT(temp,X_vect);
     denom = V3D_DOT(temp,globalDeform.U_vect);
     u = nume/denom;
 

     V3D_INIT(ffd, 0.0, 0.0, 0.0);

     V3D_COPY(old_point,the_point);
     index = 0;
     for (k=0; k<n; k++) {
      for (j=0; j<m; j++) {
       for (i=0; i<l; i++) {
         Bl = SMGD_burnstein(l-1,i,s);
         Bm = SMGD_burnstein(m-1,j,t);
         Bn = SMGD_burnstein(n-1,k,u);
         Blmn = Bl * Bm * Bn;
         V3D_SCALE(temp,Blmn,the_controlbox->vertex_list[index].point);
         V3D_ADD(ffd,ffd,temp);
         index++;
        }
       }
      }
     V3D_COPY(new_point,ffd);
     if (AU_eye_intersection(the_surface,the_region_list,new_point,
                          old_point,with_name))
      SM_vertex_point_set(the_surface,v_index,new_point);
     else
      SM_vertex_point_set(the_surface,v_index,ffd);
    }
    SM_END_FOR_EACH
   

} /* end of procedure */

/*---------------------------------------------------------------------*/
void
AU_region_outline_set(the_surface,the_region_list,the_index)
SM_Surface_ptr the_surface;
SM_Region_List_ptr the_region_list;
int the_index;

{
         int *temp_ptr, i, count, point1_ind, point2_ind;
         SM_Region_ptr reg_ptr;
         int e_index, v1_new, v1_old, v2_new, v2_old;
         BOOL bool_val;
         VECTOR_3D point,the_point, point1, point2;
         VECTOR_3D the_center, eye_center;
         double x_point, x_center;
         float max_z;

         temp_ptr = (int *) calloc(50,sizeof(int)); 
         count = 0;
         v1_new = -1; v1_old = -1;
         v2_new = -1; v2_old = -1;
         reg_ptr = &the_region_list->region_list[the_index];
         SM_FOR_EACH_REGION_EDGE(reg_ptr)
          e_index = *region_edge_ptr;
          if (SM_edge_hole_get(the_surface,e_index,&bool_val)) {
            if (bool_val) {
              if (SM_edge_vertices_get(the_surface,e_index,&v1_new,&v2_new)){
                if ( (v1_new != v1_old) && (v1_new != v2_old) ) {
                  temp_ptr[count] = v1_new; 
                  count++;
                }
                if ( (v2_new != v1_old) && (v2_new != v2_old) ) {
                  temp_ptr[count] = v2_new; 
                  count++;
                }
                v1_old = v1_new; v2_old = v2_new;
              }
             }
           }
         SM_END_FOR_EACH
         V3D_NULL(point);
         max_z = 0;
         for (i=0; i<count; i++) {
           SM_vertex_point_get(the_surface,temp_ptr[i],the_point);
           V3D_ADD(point,point,the_point);
           max_z = (max_z > the_point[2]) ? max_z : the_point[2];
         } 
         V3D_SCALE(point,1.0/(float) count,point);
         V3D_COPY(the_center,point);
         x_center = the_center[0];
         SM_FOR_EACH_REGION_VERTEX (reg_ptr)
          if (region_vertex_ptr->flag) {
            SM_vertex_point_get(the_surface,region_vertex_ptr->id,the_point);
            x_point = the_point[0];
            if (fabs(x_point) > fabs(x_center) ) {
                point1_ind = region_vertex_ptr->id;
                V3D_COPY(point1,the_point);
            }
            else {
                point2_ind = region_vertex_ptr->id;
                V3D_COPY(point2,the_point);
            }
          }
         SM_END_FOR_EACH
         V3D_COPY(eye_center,point1);
         V3D_ADD(eye_center,eye_center,point2);
         V3D_SCALE(eye_center,0.5,eye_center);
         point1[2] = eye_center[2]; point2[2] = eye_center[2];
         for (i=0; i<count; i++) {
           SM_vertex_point_get(the_surface,temp_ptr[i],the_point);
           the_point[2] = eye_center[2];
           SM_vertex_point_set(the_surface,temp_ptr[i],the_point);
         } 
/*
         SM_vertex_point_set(the_surface,point1_ind,point1);
         SM_vertex_point_set(the_surface,point2_ind,point2);
*/

}

/*---------------------------------------------------------------------*/

void
find_determinant(c1,c2,c3,value)
VECTOR_3D c1, c2, c3;
float *value;
{
      float sol;
      sol = c1[0] * ((c2[1] * c3[2]) - (c2[2] * c3[1])) -
            c2[0] * ((c1[1] * c3[2]) - (c1[2] * c3[1])) +
            c3[0] * ((c1[1] * c2[2]) - (c1[2] * c2[1])); 
      *value = sol;
}

BOOL
cramer_solver(x,y,z,k,a,b,c)

VECTOR_3D x,y,z,k;
float *a,*b,*c;
{
    VECTOR_3D col1, col2, col3;
    float value1, value2;

    V3D_COPY(col1,x);
    V3D_COPY(col2,y);
    V3D_COPY(col3,z);
    find_determinant(col1,col2,col3,&value1);
    if (fabs(value1) < SM_EPSILON) return(NO); 
    V3D_COPY(col1,y);
    V3D_COPY(col2,z);
    V3D_COPY(col3,k);
    find_determinant(col1,col2,col3,&value2);
    *a = value2/value1;
    V3D_COPY(col1,x);
    V3D_COPY(col2,z);
    V3D_COPY(col3,k);
    find_determinant(col1,col2,col3,&value2);
    *b = -value2/value1;
    V3D_COPY(col1,x);
    V3D_COPY(col2,y);
    V3D_COPY(col3,k);
    find_determinant(col1,col2,col3,&value2);
    *c = value2/value1;
    return(YES);
}

BOOL
get_sphere_center(p1,p2,p3,p4,center,radius)
VECTOR_3D p1,p2,p3,p4,center;
float *radius;
{
          VECTOR_3D c1, c2, c3, k, p12, p14, p13;
          float a,b,c;
          
          V3D_SUB(p12,p2,p1);
          V3D_SUB(p13,p3,p1);
          V3D_SUB(p14,p4,p1);
          V3D_INIT(c1,p12[0],p13[0],p14[0]);
          V3D_INIT(c2,p12[1],p13[1],p14[1]);
          V3D_INIT(c3,p12[2],p13[2],p14[2]);
          k[0] = (V3D_NORM(p2) - V3D_NORM(p1))/2.0;
          k[1] = (V3D_NORM(p3) - V3D_NORM(p1))/2.0;
          k[2] = (V3D_NORM(p4) - V3D_NORM(p1))/2.0;
        
          if (cramer_solver(c1,c2,c3,k,&a,&b,&c)) {
            V3D_INIT(center,a,b,c);
            V3D_SUB(p12,p1,center);
            *radius = V3D_NORM(p12);
            return(YES);
          }
          else
          {
            return(NO);
          }
}

/*---------------------------------------------------------------*/

void
AU_eyelid_upper_updown(the_surface,the_region_list,the_name,the_value,P1,P2)
SM_Surface_ptr the_surface;
SM_Region_List_ptr the_region_list;
SM_Region_Name the_name;
float the_value;
VECTOR_3D P1,P2;

{
    int the_index,l,m,n;
    SM_Region_Name with_name;
    VECTOR_3D p1,p2,center,direction,old_point,new_point;
    float radius;
    SM_Region_ptr reg_ptr, r1_ptr;
    int v1_new,v1_old,v2_new,v2_old,*temp_ptr,count;
    MATRIX_H3D mat;
    int v_index;
    float my_value, stiffness_factor,res;
    int  p1_index, p2_index, r1_index;
    BOOL p1_set, p2_set, my_flag;
    VECTOR_3D my_vector,my_center;

    the_index = SM_region_index_get(the_region_list,the_name);
    l = 3; m = 3; n = 3;
    strcpy(with_name,"left_loweyelid");
    temp_ptr = (int *) calloc(100,sizeof(int)); 
    count = 0;
    v1_new = -1; v1_old = -1;
    v2_new = -1; v2_old = -1;
    reg_ptr = &the_region_list->region_list[the_index];
    
    p1_set = NO; p2_set = NO;
    
    if (strcmp(the_name,"left_upeyelid") == 0) {
      strcpy(with_name,"left_loweyelid");
      r1_index = SM_region_index_get(the_region_list,with_name);
      r1_ptr = &the_region_list->region_list[r1_index]; 
       
      SM_FOR_EACH_REGION_VERTEX(r1_ptr)
        if ( fabs(region_vertex_ptr->properties.young_modulus + 999.0) <= SM_EPSILON) {
	 p1_index = region_vertex_ptr->id;
	 p1_set = YES;
	 SM_vertex_point_get(the_surface, p1_index, p1);
	}
        if ( fabs(region_vertex_ptr->properties.young_modulus - 999.0) <= SM_EPSILON) {
	 p2_index = region_vertex_ptr->id;
	 p2_set = YES; 
	 SM_vertex_point_get(the_surface, p2_index, p2);
	}
      SM_END_FOR_EACH  
            
      if  (! ((p1_set) && (p2_set)) )  {        
       V3D_COPY(p1,P1);
       V3D_COPY(p2,P2);
       V3D_INIT(direction,-0.973447,0.020480,-0.227993);
      }  else {
       V3D_SUB(direction, p1, p2);
       V3D_NORMALIZE(direction, direction);
      }
      
      V3D_ADD(p1,p1,p2);
      V3D_SCALE(center,0.5,p1);
      V3D_SUB(p1,p1,center);
      radius = V3D_NORM(p1);
      my_value = -the_value/2.0;
    } else {
      strcpy(with_name,"right_loweyelid");
      r1_index = SM_region_index_get(the_region_list,with_name);
      r1_ptr = &the_region_list->region_list[r1_index];  
      SM_FOR_EACH_REGION_VERTEX(r1_ptr)
        if ( fabs(region_vertex_ptr->properties.young_modulus + 999.0) <= SM_EPSILON) {
	 p2_index = region_vertex_ptr->id;
	 p2_set = YES;
	 SM_vertex_point_get(the_surface, p2_index, p2);
	}
        if ( fabs(region_vertex_ptr->properties.young_modulus - 999.0) <= SM_EPSILON) {
	 p1_index = region_vertex_ptr->id;
	 p1_set = YES; 
	 SM_vertex_point_get(the_surface, p1_index, p1);
	}
      SM_END_FOR_EACH  
            
      if  (! ((p1_set) && (p2_set)) )  {        
       V3D_COPY(p1,P1);
       p1[0] *= -1;
       V3D_COPY(p2,P2);
       p2[0] *= -1;
       V3D_INIT(direction,0.982149,0.024068,-0.186558);
      } else {       
       V3D_SUB(direction, p1, p2);
       V3D_NORMALIZE(direction, direction);
      }

      V3D_ADD(p1,p1,p2);
      V3D_SCALE(center,0.5,p1);
      V3D_SUB(p1,p1,center);
      radius = V3D_NORM(p1);
      my_value = the_value/2.0;
    }
        
    my_flag=YES; 
    V3D_COPY(my_vector,direction);
    V3D_COPY(my_center,center);
    V3D_SCALE(my_vector,2.0,my_vector);
    V3D_ADD(my_vector,my_center,my_vector);
    if (the_index != SM_INDEX_ERROR) {
           SM_FOR_EACH_REGION_VERTEX(reg_ptr)
            v_index = region_vertex_ptr->id;
            if (!region_vertex_ptr->anchor) {
             SM_vertex_point_get(the_surface,v_index,old_point);
             V3D_COPY(p2,direction);
             V3D_SUB(p1,old_point,center);
             res = V3D_DOT(p1,p2);
             if (res < 0.0) 
              V3D_INVERT(p2,p2);
             res = V3D_DOT(p1,p2);
             stiffness_factor = 1.0;
             my_value = my_value / stiffness_factor;
             SMGD_rotate_arbaxis_matrix(center,direction,my_value,mat);
             SMGD_rotate_arbaxis_point(old_point,mat,new_point);
             SM_vertex_point_set(the_surface,v_index,new_point);
            }
           SM_END_FOR_EACH 
     }
    

      free(temp_ptr);
}

/*--------------------------------------------------------------------*/
/*    without ffd                                                     */
/*--------------------------------------------------------------------*/

void
AU_head_center_set(the_surface, the_region_list)
SM_Surface_ptr the_surface;
SM_Region_List_ptr the_region_list;
{
   SM_Region_Controlbox_ptr the_box;
   int the_index,l,m,n;
   VECTOR_3D origin_point;
   SM_Region_Name current_region;

    strcpy(current_region, "head");
    the_index = SM_region_index_get(the_region_list,current_region);
    V3D_NULL(origin_point); 
    if (the_index != SM_INDEX_ERROR) {
     l = 2; m = 2; n = 2;
     the_box = SM_region_controlbox_new(l,m,n);
     SM_region_controlbox_recreate(the_box,l,m,n); 
     SM_region_controlbox_get(the_surface,the_region_list,the_index,the_box);
     SMGD_controlbox_origin_get(the_box,origin_point);
     free(the_box);
    }
    V3D_COPY(Neutral_head_center, origin_point);
}
/*-----------------------------------------------------------------------*/

void
AU_head_nod(the_surface,the_region_list,the_name,the_value)
SM_Surface_ptr the_surface;
SM_Region_List_ptr the_region_list;
SM_Region_Name the_name;
float the_value;

{
    int the_index;
     
    the_index = SM_region_index_get(the_region_list,the_name);
    if (the_index != SM_INDEX_ERROR) {
       SMGD_rotate_face_region(the_surface,Neutral_head_center,the_region_list,the_name,X_INDEX,the_value * 0.15);
    }
}
/*-----------------------------------------------------------------------*/
void
AU_head_turn(the_surface,the_region_list,the_name,the_value)
SM_Surface_ptr the_surface;
SM_Region_List_ptr the_region_list;
SM_Region_Name the_name;
float the_value;

{
    int the_index;
     
    the_index = SM_region_index_get(the_region_list,the_name);
    if (the_index != SM_INDEX_ERROR) {
       SMGD_rotate_face_region(the_surface,Neutral_head_center,the_region_list,the_name,Y_INDEX,the_value * 0.15);
    }
}
/*-----------------------------------------------------------------------*/
void
AU_head_roll(the_surface,the_region_list,the_name,the_value)
SM_Surface_ptr the_surface;
SM_Region_List_ptr the_region_list;
SM_Region_Name the_name;
float the_value;

{
    int the_index;
     
    the_index = SM_region_index_get(the_region_list,the_name);
    if (the_index != SM_INDEX_ERROR) {
       SMGD_rotate_face_region(the_surface,Neutral_head_center,the_region_list,the_name,Z_INDEX,the_value * 0.15);
    }
}
/*-----------------------------------------------------------------------*/
void
AU_head_track_tilt(the_surface,the_face,the_region_list,the_name,the_value)
SM_Surface_ptr the_surface,the_face;
SM_Region_List_ptr the_region_list;
SM_Region_Name the_name;
float the_value;

{
    int the_index;
     
    the_index = SM_region_index_get(the_region_list,the_name);
    if (the_index != SM_INDEX_ERROR) {
       SMGD_rotate_try_face_region(the_surface,the_face,Neutral_head_center,the_region_list,the_name,X_INDEX,the_value * 0.15);
    }
}
/*-----------------------------------------------------------------------*/
void
AU_head_track_pan(the_surface,the_face,the_region_list,the_name,the_value)
SM_Surface_ptr the_surface,the_face;
SM_Region_List_ptr the_region_list;
SM_Region_Name the_name;
float the_value;

{
    int the_index;
     
    the_index = SM_region_index_get(the_region_list,the_name);
    if (the_index != SM_INDEX_ERROR) {
       SMGD_rotate_try_face_region(the_surface,the_face,Neutral_head_center,the_region_list,the_name,Y_INDEX,the_value * 0.15);
    }
}

/*-----------------------------------------------------------------------*/

void
AU_head_track(the_surface,the_face,the_region_list,the_name,track_vec)
SM_Surface_ptr the_surface;
SM_Surface_ptr the_face;
SM_Region_List_ptr the_region_list;
SM_Region_Name the_name;
VECTOR_3D track_vec;

{
    float pan,tilt;

    pan = AU_gaze_point_pan_angle_get(track_vec,Neutral_head_center);
    tilt = AU_gaze_point_tilt_angle_get(track_vec,Neutral_head_center);
    SMGD_rotate_try_face_region(the_surface,the_face,Neutral_head_center,the_region_list,the_name,Y_INDEX,pan * 0.15);
    SMGD_rotate_try_face_region(the_surface,the_surface,Neutral_head_center,the_region_list,the_name,X_INDEX,tilt * 0.15);

}




/*-----------------------------------------------------------------------*/
/*          with ffd                                                    */
/*----------------------------------------------------------------------*/

/************************
void
AU_head_nod(the_surface,the_region_list,the_name,the_value)
SM_Surface_ptr the_surface;
SM_Region_List_ptr the_region_list;
SM_Region_Name the_name;
float the_value;

{
    SM_Region_Controlbox_ptr the_box;
    int the_index,l,m,n;
    SM_ffd_parameters ffd_parm;
    VECTOR_3D s_vect;
     
    the_index = SM_region_index_get(the_region_list,the_name);
    l = 3; m = 3; n = 3;
    if (the_index != SM_INDEX_ERROR) {
       the_box = SM_region_controlbox_new(l,m,n);
       SM_region_controlbox_recreate(the_box,l,m,n); 
       SM_region_controlbox_get(the_surface,the_region_list,the_index,the_box);
       AU_face_ffd_init(the_box,&ffd_parm); 
       SMGD_rotate_object_controlbox(the_box,X_INDEX,the_value * 0.2);
       SMGD_region_freeform_apply(the_surface,the_region_list,the_name,
                                  ffd_parm,the_box);
       SM_region_controlbox_dispose(the_box);
    }
}

void
AU_head_turn(the_surface,the_region_list,the_name,the_value)
SM_Surface_ptr the_surface;
SM_Region_List_ptr the_region_list;
SM_Region_Name the_name;
float the_value;

{
    SM_Region_Controlbox_ptr the_box;
    int the_index,l,m,n;
    SM_ffd_parameters ffd_parm;
    VECTOR_3D s_vect;
     
    the_index = SM_region_index_get(the_region_list,the_name);
    l = 3; m = 3; n = 3;
    if (the_index != SM_INDEX_ERROR) {
       the_box = SM_region_controlbox_new(l,m,n);
       SM_region_controlbox_recreate(the_box,l,m,n); 
       SM_region_controlbox_get(the_surface,the_region_list,the_index,the_box);
       AU_face_ffd_init(the_box,&ffd_parm); 
       SMGD_rotate_object_controlbox(the_box,Y_INDEX,the_value * 0.1);
       SMGD_region_freeform_apply(the_surface,the_region_list,the_name,
                                  ffd_parm,the_box);
       SM_region_controlbox_dispose(the_box);
    }
}


void
AU_head_roll(the_surface,the_region_list,the_name,the_value)
SM_Surface_ptr the_surface;
SM_Region_List_ptr the_region_list;
SM_Region_Name the_name;
float the_value;

{
    SM_Region_Controlbox_ptr the_box;
    int the_index,l,m,n;
    SM_ffd_parameters ffd_parm;
    VECTOR_3D s_vect;
     
    the_index = SM_region_index_get(the_region_list,the_name);
    l = 3; m = 3; n = 3;
    if (the_index != SM_INDEX_ERROR) {
       the_box = SM_region_controlbox_new(l,m,n);
       SM_region_controlbox_recreate(the_box,l,m,n); 
       SM_region_controlbox_get(the_surface,the_region_list,the_index,the_box);
       AU_face_ffd_init(the_box,&ffd_parm); 
       SMGD_rotate_object_controlbox(the_box,Z_INDEX,the_value * 0.15);
       SMGD_region_freeform_apply(the_surface,the_region_list,the_name,
                                  ffd_parm,the_box);
       SM_region_controlbox_dispose(the_box);
    }
}

*************/
/*-----------------------------------------------------------------------*/

void
AU_tongue_lift(the_surface,the_face,the_region_list,the_name,the_value)
SM_Surface_ptr the_surface, the_face;
SM_Region_List_ptr the_region_list;
SM_Region_Name the_name;
float the_value;

{
    SM_Region_Controlbox_ptr the_box;
    int the_index,l,m,n;
    SM_ffd_parameters ffd_parm;
    int vert_index,plane_dir;
    VECTOR_3D t_vect;

    the_index = SM_region_index_get(the_region_list,the_name);
    l = 3; m = 3; n = 3;
    plane_dir = Z_INDEX;
    vert_index = 26;
    
    V3D_INIT(t_vect, 0.0, the_value*1.45, 0.0);
    if (the_index != SM_INDEX_ERROR) {
       the_box = SM_region_controlbox_new(l,m,n);
       SM_region_controlbox_recreate(the_box,l,m,n); 
       SM_region_controlbox_get(the_face,the_region_list,the_index,the_box);
       AU_face_ffd_init(the_box,&ffd_parm); 
       /*
       SMGD_shear_object_control_plane(the_box,vert_index,the_value,Y_INDEX,plane_dir);
       SMGD_rotate_object_control_plane(the_box,vert_index,X_INDEX,the_value*0.7,plane_dir);
       */
       SMGD_translate_object_control_plane(the_box,vert_index,t_vect,plane_dir);
       SMGD_region_face_freeform_apply(the_surface,the_face,the_region_list,the_name,
                                  ffd_parm,the_box);
       SM_region_controlbox_dispose(the_box);
    }
}

/*-----------------------------------------------------------------------*/
void
AU_tongue_lift_tip(the_surface,the_face,the_region_list,the_name,the_value)
SM_Surface_ptr the_surface, the_face;
SM_Region_List_ptr the_region_list;
SM_Region_Name the_name;
float the_value;

{
    SM_Region_Controlbox_ptr the_box;
    int the_index,l,m,n;
    SM_ffd_parameters ffd_parm;
    int vert_index,plane_dir;
    VECTOR_3D t_vect;

    the_index = SM_region_index_get(the_region_list,the_name);
    l = 5; m = 5; n = 5;
    plane_dir = Z_INDEX;
    vert_index = 124;
    
    V3D_INIT(t_vect, 0.0, the_value*1.45, 0.0);
    if (the_index != SM_INDEX_ERROR) {
       the_box = SM_region_controlbox_new(l,m,n);
       SM_region_controlbox_recreate(the_box,l,m,n); 
       SM_region_controlbox_get(the_face,the_region_list,the_index,the_box);
       AU_face_ffd_init(the_box,&ffd_parm); 
       /*
       SMGD_shear_object_control_plane(the_box,vert_index,the_value,Y_INDEX,plane_dir);
       SMGD_rotate_object_control_plane(the_box,vert_index,X_INDEX,the_value*0.7,plane_dir);
       */
       SMGD_translate_object_control_plane(the_box,vert_index,t_vect,plane_dir);
       SMGD_region_face_freeform_apply(the_surface,the_face,the_region_list,the_name,
                                  ffd_parm,the_box);
       SM_region_controlbox_dispose(the_box);
    }
}

/*-----------------------------------------------------------------------*/
void
AU_tongue_roll(the_surface,the_face,the_region_list,the_name,the_value)
SM_Surface_ptr the_surface, the_face;
SM_Region_List_ptr the_region_list;
SM_Region_Name the_name;
float the_value;

{
    SM_Region_Controlbox_ptr the_box;
    int the_index,l,m,n;
    SM_ffd_parameters ffd_parm;
    int plane_dir;
    VECTOR_3D t_vect;

    the_index = SM_region_index_get(the_region_list,the_name);
    l = 3; m = 3; n = 3;
    plane_dir = X_INDEX;
    
    if (the_index != SM_INDEX_ERROR) {
       the_box = SM_region_controlbox_new(l,m,n);
       SM_region_controlbox_recreate(the_box,l,m,n); 
       SM_region_controlbox_get(the_face,the_region_list,the_index,the_box);
       AU_face_ffd_init(the_box,&ffd_parm); 
       V3D_INIT(t_vect, 0.0, the_value*1.45, 0.0);
       SMGD_translate_object_control_plane(the_box,26,t_vect,plane_dir);
       SMGD_translate_object_control_plane(the_box,0,t_vect,plane_dir);
       V3D_INIT(t_vect, 0.0, -the_value*1.45, 0.0);
       SMGD_translate_object_control_plane(the_box,13,t_vect,plane_dir);
       SMGD_region_face_freeform_apply(the_surface,the_face,the_region_list,the_name,
                                  ffd_parm,the_box);
       SM_region_controlbox_dispose(the_box);
    }
}

/*--------------------------------------------------------------------------*/

void
AU_tongue_pan(the_surface,the_face,the_region_list,the_name,the_value)
SM_Surface_ptr the_surface, the_face;
SM_Region_List_ptr the_region_list;
SM_Region_Name the_name;
float the_value;

{
    SM_Region_Controlbox_ptr the_box;
    int the_index,l,m,n;
    SM_ffd_parameters ffd_parm;
    int vert_index,plane_dir;
    VECTOR_3D t_vect;
    
    the_index = SM_region_index_get(the_region_list,the_name);
    l = 3; m = 3; n = 3;
    plane_dir = Z_INDEX;
    vert_index = 26;
    V3D_INIT(t_vect, the_value*1.45, 0.0, 0.0);
    if (the_index != SM_INDEX_ERROR) {
       the_box = SM_region_controlbox_new(l,m,n);
       SM_region_controlbox_recreate(the_box,l,m,n); 
       SM_region_controlbox_get(the_face,the_region_list,the_index,the_box);
       AU_face_ffd_init(the_box,&ffd_parm); 
       /*
       SMGD_shift_object_control_plane(the_box,vert_index,the_value,X_INDEX,plane_dir);
       */
       SMGD_translate_object_control_plane(the_box,vert_index,t_vect,plane_dir);
       SMGD_region_face_freeform_apply(the_surface,the_face,the_region_list,the_name,
                                  ffd_parm,the_box);
       SM_region_controlbox_dispose(the_box);
    }
}

/*--------------------------------------------------------------------------*/

void
AU_tongue_out(the_surface,the_face,the_region_list,the_name,the_value)
SM_Surface_ptr the_surface, the_face;
SM_Region_List_ptr the_region_list;
SM_Region_Name the_name;
float the_value;

{
    SM_Region_Controlbox_ptr the_box;
    int the_index,l,m,n;
    SM_ffd_parameters ffd_parm;
    int vert_index,plane_dir;
    VECTOR_3D t_vect;
    
    the_index = SM_region_index_get(the_region_list,the_name);
    l = 3; m = 3; n = 3;
    plane_dir = Z_INDEX;
    vert_index = 26;
    V3D_INIT(t_vect, 0.0, 0.0, the_value*1.45);
    if (the_index != SM_INDEX_ERROR) {
       the_box = SM_region_controlbox_new(l,m,n);
       SM_region_controlbox_recreate(the_box,l,m,n); 
       SM_region_controlbox_get(the_face,the_region_list,the_index,the_box);
       AU_face_ffd_init(the_box,&ffd_parm); 
       /*
       SMGD_shift_object_control_plane(the_box,vert_index,the_value,X_INDEX,plane_dir);
       */
       SMGD_translate_object_control_plane(the_box,vert_index,t_vect,plane_dir);
       SMGD_region_face_freeform_apply(the_surface,the_face,the_region_list,the_name,
                                  ffd_parm,the_box);
       SM_region_controlbox_dispose(the_box);
    }
}

/*--------------------------------------------------------------------------*/
void
AU_raise_ear(the_surface,the_face,the_region_list,the_name,the_value)
SM_Surface_ptr the_surface, the_face;
SM_Region_List_ptr the_region_list;
SM_Region_Name the_name;
float the_value;

{
    SM_Region_Controlbox_ptr the_box;
    int the_index,l,m,n;
    SM_ffd_parameters ffd_parm;
    int vert_index,plane_dir;
    VECTOR_3D t_vect;
    
    the_index = SM_region_index_get(the_region_list,the_name);
    l = 3; m = 3; n = 3;
    plane_dir = X_INDEX;
    if(strstr(the_name,"left"))
      vert_index = 0;
    else
      vert_index = 8;
    V3D_INIT(t_vect, 0.0, the_value*3.45, 0.0);
    if (the_index != SM_INDEX_ERROR) {
       the_box = SM_region_controlbox_new(l,m,n);
       SM_region_controlbox_recreate(the_box,l,m,n); 
       SM_region_controlbox_get(the_face,the_region_list,the_index,the_box);
       AU_face_ffd_init(the_box,&ffd_parm); 
       /*
       SMGD_shift_object_control_plane(the_box,vert_index,the_value,X_INDEX,plane_dir);
       */
       SMGD_translate_object_control_plane(the_box,vert_index,t_vect,plane_dir);
       SMGD_region_face_freeform_apply(the_surface,the_face,the_region_list,the_name,
                                  ffd_parm,the_box);
       SM_region_controlbox_dispose(the_box);
    }
}

/*--------------------------------------------------------------------------*/
void
AU_pull_ear(the_surface,the_face,the_region_list,the_name,the_value)
SM_Surface_ptr the_surface, the_face;
SM_Region_List_ptr the_region_list;
SM_Region_Name the_name;
float the_value;

{
    SM_Region_Controlbox_ptr the_box;
    int the_index,l,m,n;
    SM_ffd_parameters ffd_parm;
    int vert_index,plane_dir;
    VECTOR_3D t_vect;
    
    the_index = SM_region_index_get(the_region_list,the_name);
    l = 3; m = 3; n = 3;
    plane_dir = Z_INDEX;
    vert_index = 0;
    if(strstr(the_name,"left"))
      {
	V3D_INIT(t_vect, -the_value*4.45, 0.0, 0.0);
      }
    else
      {
	V3D_INIT(t_vect, the_value*4.45, 0.0, 0.0);
      }

    if (the_index != SM_INDEX_ERROR) {
       the_box = SM_region_controlbox_new(l,m,n);
       SM_region_controlbox_recreate(the_box,l,m,n); 
       SM_region_controlbox_get(the_face,the_region_list,the_index,the_box);
       AU_face_ffd_init(the_box,&ffd_parm); 
       SMGD_translate_object_control_plane(the_box,vert_index,t_vect,plane_dir);
       SMGD_region_face_freeform_apply(the_surface,the_face,the_region_list,the_name,
                                  ffd_parm,the_box);
       SM_region_controlbox_dispose(the_box);
    }
}

/*--------------------------------------------------------------------------*/

void
AU_chin_depress(the_surface,the_face,the_region_list,the_name,the_value)
SM_Surface_ptr the_surface,  the_face;
SM_Region_List_ptr the_region_list;
SM_Region_Name the_name;
float the_value;

{
    SM_Region_Controlbox_ptr the_box;
    int the_index,l,m,n;
    SM_ffd_parameters ffd_parm;
    int vert_index;
    VECTOR_3D t_vect, origin;
    
    the_index = SM_region_index_get(the_region_list,the_name);
    l = 3; m = 3; n = 3;
    if (the_index != SM_INDEX_ERROR) {
       the_box = SM_region_controlbox_new(l,m,n);
       SM_region_controlbox_recreate(the_box,l,m,n); 
       SM_region_controlbox_get(the_face,the_region_list,the_index,the_box);
       SMGD_controlbox_origin_get(the_box,origin);
       AU_face_ffd_init(the_box,&ffd_parm);        
       SMGD_shearmin_object_controlbox(the_box,-the_value*0.05,Y_INDEX);
       V3D_INIT(t_vect, -the_value*0.5, 0.0, 0.0);
       vert_index = 12;
       SMGD_translate_object_control_point(the_box, vert_index, t_vect, origin); 
       V3D_INIT(t_vect, the_value*0.5, 0.0, 0.0);
       vert_index = 14;
       SMGD_translate_object_control_point(the_box, vert_index, t_vect, origin); 
       V3D_INIT(t_vect, -the_value*0.5, the_value*0.005, 0.0);
       vert_index = 15;
       SMGD_translate_object_control_point(the_box, vert_index, t_vect, origin); 
       V3D_INIT(t_vect, the_value*0.5, 0.0, 0.0);
       vert_index = 17;
       SMGD_translate_object_control_point(the_box, vert_index, t_vect, origin); 
       
       SMGD_region_face_freeform_apply(the_surface,the_face,the_region_list,the_name,
                                  ffd_parm,the_box);
       SM_region_controlbox_dispose(the_box);
    }
            
}

void
AU_lip_lower_side_depress(the_surface,the_face,the_region_list,the_name,the_value)
SM_Surface_ptr the_surface,  the_face;
SM_Region_List_ptr the_region_list;
SM_Region_Name the_name;
float the_value;

{
    SM_Region_Controlbox_ptr the_box;
    int the_index,l,m,n;
    SM_ffd_parameters ffd_parm;
    int vert_index;
    VECTOR_3D  t_vect, origin;
    
    the_index = SM_region_index_get(the_region_list,the_name);
    l = 3; m = 3; n = 3;
    if (the_index != SM_INDEX_ERROR) {
       the_box = SM_region_controlbox_new(l,m,n);
       SM_region_controlbox_recreate(the_box,l,m,n); 
       SM_region_controlbox_get(the_face,the_region_list,the_index,the_box);
       SMGD_controlbox_origin_get(the_box,origin);
       AU_face_ffd_init(the_box,&ffd_parm); 
       V3D_INIT(t_vect, 0.0, -the_value*0.5, 0.0);       
       for (vert_index=9; vert_index<=17; vert_index++)
         SMGD_translate_object_control_point(the_box, vert_index, t_vect, origin);                
       V3D_INIT(t_vect, 0.0, 0.0, the_value*0.5);       
       for (vert_index=12; vert_index<=17; vert_index++)
         SMGD_translate_object_control_point(the_box, vert_index, t_vect, origin);                
       SMGD_region_face_freeform_apply(the_surface,the_face,the_region_list,the_name,
                                  ffd_parm,the_box);
       SM_region_controlbox_dispose(the_box);
    }
            
}



void
AU_lip_lower_depress(the_surface,the_face,the_region_list,the_value)
SM_Surface_ptr the_surface,  the_face;
SM_Region_List_ptr the_region_list;
float the_value;

{
    
    int vert_index;
    SM_Region_Name the_name;
    
    strcpy(the_name, "lower_outer_l_lip");
    AU_lip_lower_side_depress(the_surface,the_face,the_region_list,the_name,the_value);    
    strcpy(the_name, "lower_outer_r_lip");
    AU_lip_lower_side_depress(the_surface,the_face,the_region_list,the_name,the_value);    
            
}

void
AU_mouth_squeeze_part_cornerlip(the_surface,the_face,the_region_list,the_name,which,the_value)
SM_Surface_ptr the_surface,  the_face;
SM_Region_List_ptr the_region_list;
SM_Region_Name the_name;
int which;
double the_value;

{
    SM_Region_Controlbox_ptr	the_box;
    int				the_index,k,l,m,n;
    SM_ffd_parameters		ffd_parm;
    VECTOR_3D			t_vect,origin;
    
    the_index = -1;
    the_index = SM_region_index_get(the_region_list,the_name);
    
    l = 3; m = 3; n = 3;
     
    if (the_index != SM_INDEX_ERROR) {
       the_box = SM_region_controlbox_new(l,m,n);
       SM_region_controlbox_recreate(the_box,l,m,n); 
       SM_region_controlbox_get(the_face,the_region_list,the_index,the_box);
       SMGD_controlbox_origin_get(the_box,origin);
       AU_face_ffd_init(the_box,&ffd_parm);        
       switch (which) {
       	 case 0 :  /* both left and right */
	  V3D_INIT(t_vect, -the_value, 0.0, -the_value);
          for (k=9; k<=15; k+=3) 
            SMGD_translate_object_control_point(the_box, k, t_vect, origin); 
	  V3D_INIT(t_vect, the_value, 0.0, -the_value);
          for (k=11; k<=17; k+=3) 
            SMGD_translate_object_control_point(the_box, k, t_vect, origin);                
	  V3D_INIT(t_vect, 1.0, 1.0, 1.0 + the_value*0.25);
          for (k=19; k<=25; k+=3) 
            SMGD_scale_object_control_point(the_box, k, t_vect, origin);                
       	 break;
     	 case 1 :  /* left    */
	  V3D_INIT(t_vect, -the_value, 0.0, -the_value);
          for (k=9; k<=15; k+=3) 
            SMGD_translate_object_control_point(the_box, k, t_vect, origin);                
	  V3D_INIT(t_vect, 1.0, 1.0, 1.0 + the_value*0.25);
          for (k=19; k<=25; k+=3) 
            SMGD_scale_object_control_point(the_box, k, t_vect, origin);                
      	 break;
      	 case 2 :  /* right   */
	  V3D_INIT(t_vect, the_value, 0.0, -the_value);
          for (k=11; k<=17; k+=3) 
            SMGD_translate_object_control_point(the_box, k, t_vect, origin);                
	  V3D_INIT(t_vect, 1.0, 1.0, 1.0 + the_value*0.25);
          for (k=19; k<=25; k+=3) 
            SMGD_scale_object_control_point(the_box, k, t_vect, origin);                
      	 break;
 	 default : /* both left and right */
	  V3D_INIT(t_vect, -the_value, 0.0, -the_value);
          for (k=9; k<=15; k+=3) 
            SMGD_translate_object_control_point(the_box, k, t_vect, origin);                
	  V3D_INIT(t_vect, the_value, 0.0, -the_value);
          for (k=11; k<=17; k+=3) 
            SMGD_translate_object_control_point(the_box, k, t_vect, origin);                
	  V3D_INIT(t_vect, 1.0, 1.0, 1.0 + the_value*0.25);
          for (k=19; k<=25; k+=3) 
            SMGD_scale_object_control_point(the_box, k, t_vect, origin);                
     	 break;
       }
       SMGD_region_face_freeform_apply(the_surface,the_face,the_region_list,the_name,
                                  ffd_parm,the_box);
       SM_region_controlbox_dispose(the_box);
    }
            
}

void
AU_mouth_squeeze_cornerlip(the_surface,the_face,the_region_list,which,the_value)
SM_Surface_ptr the_surface,  the_face;
SM_Region_List_ptr the_region_list;
int which;
float the_value;
{
   SM_Region_Name	the_name;
   
   strcpy(the_name, "lower_lip");
   AU_mouth_squeeze_part_cornerlip(the_surface,the_face,the_region_list,the_name,which,the_value);
   strcpy(the_name, "upper_lip");
   AU_mouth_squeeze_part_cornerlip(the_surface,the_face,the_region_list,the_name,which,the_value);

   
}
void
AU_lip_upper_tighten(the_surface,the_face,the_region_list,the_name,the_value)
SM_Surface_ptr the_surface,  the_face;
SM_Region_List_ptr the_region_list;
SM_Region_Name the_name;
float the_value;

{
    
    
    
}

void
AU_lip_lower_tighten(the_surface,the_face,the_region_list,the_name,the_value)
SM_Surface_ptr the_surface,  the_face;
SM_Region_List_ptr the_region_list;
SM_Region_Name the_name;
float the_value;

{
    
    
    
}

void
AU_lip_upper_press(the_surface,the_face,the_region_list,the_name,the_value)
SM_Surface_ptr the_surface,  the_face;
SM_Region_List_ptr the_region_list;
SM_Region_Name the_name;
float the_value;

{
    
    
    
}

void
AU_lip_lower_press(the_surface,the_face,the_region_list,the_name,the_value)
SM_Surface_ptr the_surface,  the_face;
SM_Region_List_ptr the_region_list;
SM_Region_Name the_name;
float the_value;

{
    
    
    
}

void
AU_lip_upper_suck(the_surface,the_face,the_region_list,the_name,the_value)
SM_Surface_ptr the_surface,  the_face;
SM_Region_List_ptr the_region_list;
SM_Region_Name the_name;
float the_value;

{
    
    SM_Region_Controlbox_ptr	the_box;
    int				the_index,l,m,n;
    SM_ffd_parameters		ffd_parm;
    
    the_index = SM_region_index_get(the_region_list,the_name);
    l = 3; m = 3; n = 3;
    if (the_index != SM_INDEX_ERROR) {
       the_box = SM_region_controlbox_new(l,m,n);
       SM_region_controlbox_recreate(the_box,l,m,n); 
       SM_region_controlbox_get(the_face,the_region_list,the_index,the_box);
       AU_face_ffd_init(the_box,&ffd_parm); 
       SMGD_shift_object_controlbox(the_box,-the_value*0.05,Y_INDEX);
       SMGD_shift_object_controlbox(the_box,-the_value*0.1,Z_INDEX);
       SMGD_shearmin_object_controlbox(the_box,-the_value*0.05,Y_INDEX);
       SMGD_region_face_freeform_apply(the_surface,the_face,the_region_list,the_name,
                                  ffd_parm,the_box);
       SM_region_controlbox_dispose(the_box);
    }
    
    
}

void
AU_lip_lower_suck(the_surface,the_face,the_region_list,the_name,the_value)
SM_Surface_ptr the_surface,  the_face;
SM_Region_List_ptr the_region_list;
SM_Region_Name the_name;
float the_value;

{
        
    SM_Region_Controlbox_ptr	the_box;
    int				the_index,l,m,n;
    SM_ffd_parameters		ffd_parm;
    
    the_index = SM_region_index_get(the_region_list,the_name);
    l = 3; m = 3; n = 3;
    if (the_index != SM_INDEX_ERROR) {
       the_box = SM_region_controlbox_new(l,m,n);
       SM_region_controlbox_recreate(the_box,l,m,n); 
       SM_region_controlbox_get(the_face,the_region_list,the_index,the_box);
       AU_face_ffd_init(the_box,&ffd_parm); 
       SMGD_shift_object_controlbox(the_box,the_value*0.05,Y_INDEX);
       SMGD_shift_object_controlbox(the_box,-the_value*0.1,Z_INDEX);
       SMGD_shearmin_object_controlbox(the_box,the_value*0.05,Y_INDEX);
       SMGD_region_face_freeform_apply(the_surface,the_face,the_region_list,the_name,
                                  ffd_parm,the_box);
       SM_region_controlbox_dispose(the_box);
    }
            
}

void
AU_lip_suck(the_surface,the_face,the_region_list,the_value)
SM_Surface_ptr the_surface,  the_face;
SM_Region_List_ptr the_region_list;
float the_value;

{
    SM_Region_Name the_name;
    strcpy(the_name, "upper_lip");
    AU_lip_upper_suck(the_surface,the_face,the_region_list,the_name,the_value);
    strcpy(the_name, "lower_lip");
    AU_lip_lower_suck(the_surface,the_face,the_region_list,the_name,the_value);
    
}

/* A vision plane is defined as three points,
 * the center which is calculated from p1, p2, p3, the unit
 * normal and the distance from the origin d,  which is the z if the
 * plane is perpendicular to z axis. The unit normal can be
 * calculated from the points but it may be useful to store it.
 */
 
typedef struct {
    VECTOR_3D p1, p2, p3, c, n;
    float d;
}VisionPlane, *VisionPlane_ptr;


void
AU_gaze_vplane_set(V_plane, d, p1, p2, p3)
VisionPlane_ptr V_plane;
float d;
VECTOR_3D p1, p2, p3;
{
  VECTOR_3D p, temp;
  
    V3D_COPY(V_plane->p1, p1);
    V3D_COPY(V_plane->p2, p2);
    V3D_COPY(V_plane->p3, p3);
    V_plane->d = d;
    V3D_SUB(p, p2, p1);
    V3D_SCALE(p, 0.5, p);
    V3D_ADD(p, p, p1);
    V3D_SUB(temp, p3, p1);
    V3D_SCALE(temp, 0.5, temp);
    V3D_ADD(V_plane->c, temp, p);
    V3D_SUB(p, p2, p1);
    V3D_CROSS(V_plane->n, p, temp);
    V3D_NORMALIZE(V_plane->n, V_plane->n);
}

void
AU_gaze_vplane_point_projection_get(V_plane, p)
VisionPlane_ptr V_plane;
VECTOR_3D p;
{
    
}

/*
 * The tilt angle corresponds to the angle subtended by the projection of
 * the point on Y-plane (the up vector plane).
 * p: the point, o: origin.
 */
float
AU_gaze_point_tilt_angle_get(p,o)
VECTOR_3D p,o;

{
   VECTOR_3D temp, up;
   float mag;
   
   V3D_INIT(up, 0,1,0);
   V3D_SUB(temp, p, o);
   mag = V3D_DOT(temp, up);
   if (p[2] < 0) mag = 0;
   return(mag);
}

/*
 * The pan angle corresponds to the angle subtended by the projection of
 * the point on X-plane (the side vector plane).
 * p: the point, o: origin.
 */
float
AU_gaze_point_pan_angle_get(p,o)
VECTOR_3D p,o;

{
   VECTOR_3D temp, side;
   float mag;
   
   V3D_INIT(side, 1,0,0);
   V3D_SUB(temp, p, o);
   mag = V3D_DOT(temp, side);
   if (p[2] < 0) mag = 0;
   return(mag);
}

