

/*------------------------------------------------------------------------------*/
/*										*/
/*										*/
/*  This software module was originally developed by                            */
/*        Prem Kalra, MIRALab, University of Geneva                             */
/*  and edited by                                                               */
/*        Igor Pandzic, MIRALab, University of Geneva                           */
/*        Marc Escher, MIRALab, University of Geneva                            */
/*        Pierre Beylot, MIRALab, University of Geneva                          */
/*  in the course of development of the MPEG-4 Video (ISO/IEC 14496-2).	        */
/*  This software module is an implementation of a part of one or more	        */
/*  MPEG-4 Video (ISO/IEC 14496-2) tools as specified by the MPEG-4 Video       */
/*  (ISO/IEC 14496-2). ISO/IEC gives users of the                               */
/*  MPEG-4 Video (ISO/IEC 14496-2) free license to this software module or	*/
/*  modifications thereof for use in hardware or software products claiming 	*/
/*  conformance to the MPEG-4 Video (ISO/IEC 14496-2). Those intending to use	*/
/*  this software module in hardware or software products are advised that	*/
/*  its use may infringe existing patents. The original developer of this	*/
/*  software module and his/her company, the subsequent editors and their	*/
/*  companies, and ISO/IEC have no liability for use of this software		*/
/*  module or modifications thereof. Copyright is not released for non		*/
/*  MPEG-4 Video (ISO/IEC 14496-2) conforming products.                         */
/*  MIRALab, University of Geneva retains full right to use the code for its    */
/*  own purpose, assign or donate the code to a third party and to inhibit      */
/*  third parties from using the code for non MPEG-4 Video (ISO/IEC 14496-2)    */
/*  conforming products. This copyright notice must be included in all copies   */
/*  or derivative works. Copyright (c) (1997).			      		*/
/*										*/
/*										*/
/*------------------------------------------------------------------------------*/

#include "Muscle.h"


static SM_Surface_Controlbox_ptr the_neutral_box;
static VECTOR_3D the_neutral_box_origin;
 
MPA_Muscle_List_ptr
MPA_muscle_list_new()
{
    MPA_Muscle_List_ptr the_mpa_muscle_list;
    the_mpa_muscle_list = (MPA_Muscle_List_ptr) calloc(1,sizeof(MPA_Muscle_List));
    the_mpa_muscle_list->muscle_list_size = 0;
    the_mpa_muscle_list->muscle_list = NULL;
    return(the_mpa_muscle_list);
}

BOOL
MPA_muscle_list_dispose(the_mpa_muscle_list)
MPA_Muscle_List_ptr the_mpa_muscle_list;
/*
** To dispose the muscle list
*/
{

   if (the_mpa_muscle_list == NULL) return(NO);

   MPA_FOR_EACH_MUSCLE(the_mpa_muscle_list)
    MPA_muscle_remove_ref(the_mpa_muscle_list,muscle_index);
   MPA_END_FOR_EACH

   if (the_mpa_muscle_list->muscle_list_size > 0) 
    free(the_mpa_muscle_list->muscle_list);
   the_mpa_muscle_list->muscle_list_size = 0;
   the_mpa_muscle_list->muscle_list = NULL;
   
   free(the_mpa_muscle_list);
   return(YES);
}

BOOL 
MPA_muscle_remove_ref(the_mpa_muscle_list,index)
MPA_Muscle_List_ptr the_mpa_muscle_list;
int index;
/*
** To remove the reference for a muscle
*/
{
   MPA_Muscle_ptr the_muscle;
   
   the_muscle = &the_mpa_muscle_list->muscle_list[index];
   if (the_muscle == NULL) return(NO);

   if (the_muscle->point_list_size > 0) {
     the_muscle->point_list_size = 0;
     free(the_muscle->point_list);
   }
   return(YES);
}

void
MPA_muscle_list_print(the_mpa_muscle_list)
MPA_Muscle_List_ptr the_mpa_muscle_list;
{
    if (the_mpa_muscle_list != NULL) {
      printf(" the total number of muscles %d \n",
                 the_mpa_muscle_list->muscle_list_size);
      MPA_FOR_EACH_MUSCLE(the_mpa_muscle_list)
        printf(" the muscle ptr %d \n",muscle_ptr);
        printf(" the muscle name %s \n",muscle_ptr->name);
        printf(" the muscle l m n %d %d %d \n",muscle_ptr->l,
                                               muscle_ptr->m,
                                               muscle_ptr->n);
        printf(" the muscle max_sen %f \n",muscle_ptr->max_sensitivity);
        printf(" the muscle min_sen %f \n",muscle_ptr->min_sensitivity);
        printf(" the muscle point list size %d \n",muscle_ptr->point_list_size);
      MPA_END_FOR_EACH
    }

}

void
MPA_muscle_point_print(the_mpa_muscle_list,index)
MPA_Muscle_List_ptr the_mpa_muscle_list;
int index;
{
    MPA_Muscle_ptr the_muscle;

    the_muscle = &the_mpa_muscle_list->muscle_list[index];
    MPA_FOR_EACH_MUSCLE_POINT(the_muscle)
      printf(" index %d \n",point_ptr->index);
      printf(" weight %f \n",point_ptr->weight);
      printf(" deform list size %d \n",point_ptr->deform_list_size);
      MPA_FOR_EACH_MUSCLE_POINT_DEFORM(point_ptr)
        printf("type axis value %d %d %f\n",deform_ptr->deform_type,
                                            deform_ptr->axis,
                                            deform_ptr->value);
      MPA_END_FOR_EACH
    MPA_END_FOR_EACH 
}

void
MPA_muscle_point_write_file(infile,the_muscle)
FILE *infile;
MPA_Muscle_ptr the_muscle;
{
 
    MPA_FOR_EACH_MUSCLE_POINT(the_muscle)
      fprintf(infile,"[ %d %f %d\n",point_ptr->index,
                                  point_ptr->weight,
                                  point_ptr->deform_list_size);
      MPA_FOR_EACH_MUSCLE_POINT_DEFORM(point_ptr)
        fprintf(infile,"(%d %d %f) ",deform_ptr->deform_type,
                                     deform_ptr->axis,
                                     deform_ptr->value);
      MPA_END_FOR_EACH
      fprintf(infile,"\n");
      fprintf(infile,"] \n");
    MPA_END_FOR_EACH 
}

void
MPA_muscle_list_write_file(infile,the_mpa_muscle_list)
FILE *infile;
MPA_Muscle_List_ptr the_mpa_muscle_list;
{
    if (the_mpa_muscle_list != NULL) {
     fprintf(infile,"%d \n",the_mpa_muscle_list->muscle_list_size);      
      MPA_FOR_EACH_MUSCLE(the_mpa_muscle_list)
        fprintf(infile,"%s %d %d %d %f %f %d\n",muscle_ptr->name,
                                                muscle_ptr->l,
                                                muscle_ptr->m,
                                                muscle_ptr->n,
                                                muscle_ptr->max_sensitivity,
                                                muscle_ptr->min_sensitivity,
                                                muscle_ptr->point_list_size);
        MPA_muscle_point_write_file(infile,muscle_ptr);
      MPA_END_FOR_EACH
    }

}

BOOL
MPA_muscle_list_write(filename,the_mpa_muscle_list)
char *filename;
MPA_Muscle_List_ptr the_mpa_muscle_list;
{
  FILE *infile; 
    
  if (infile = fopen(filename, "w")) {
    MPA_muscle_list_write_file(infile,the_mpa_muscle_list);
    fclose(infile);
    return(YES);
  } else return(NO);

}

void
MPA_muscle_point_read_file(infile,the_muscle)
FILE *infile;
MPA_Muscle_ptr the_muscle;
{

   char garb_c;

    MPA_FOR_EACH_MUSCLE_POINT(the_muscle)
     fscanf(infile,"%c %hd %f %d\n",&garb_c,
                                   &point_ptr->index,
                                   &point_ptr->weight,
                                   &point_ptr->deform_list_size);
      point_ptr->deform_list = (Muscle_Deform_ptr)
        calloc(point_ptr->deform_list_size,sizeof(Muscle_Deform));
       MPA_FOR_EACH_MUSCLE_POINT_DEFORM(point_ptr)
       fscanf(infile,"%c%hd %hd %f%c ",&garb_c,
                           &deform_ptr->deform_type,
                           &deform_ptr->axis,
                           &deform_ptr->value,
                           &garb_c);
      MPA_END_FOR_EACH
      fscanf(infile,"\n");
      fscanf(infile,"%c \n",&garb_c);
    MPA_END_FOR_EACH 
}

void
MPA_muscle_list_read_file(infile,the_mpa_muscle_list)
FILE *infile;
MPA_Muscle_List_ptr the_mpa_muscle_list;
{
    if (the_mpa_muscle_list != NULL) {
     fscanf(infile,"%d \n",&the_mpa_muscle_list->muscle_list_size);   
     the_mpa_muscle_list->muscle_list = (MPA_Muscle_ptr) 
        calloc(the_mpa_muscle_list->muscle_list_size,sizeof(MPA_Muscle));   
      MPA_FOR_EACH_MUSCLE(the_mpa_muscle_list)
        fscanf(infile,"%s %hd %hd %hd %f %f %d\n",muscle_ptr->name,
                                                &(muscle_ptr->l),
                                                &(muscle_ptr->m),
                                                &(muscle_ptr->n),
                                                &(muscle_ptr->max_sensitivity),
                                                &(muscle_ptr->min_sensitivity),
                                                &(muscle_ptr->point_list_size));
        muscle_ptr->point_list = (Muscle_Point_ptr) 
            calloc(muscle_ptr->point_list_size,sizeof(Muscle_Point));
        MPA_muscle_point_read_file(infile,muscle_ptr);
      MPA_END_FOR_EACH
    }

}

BOOL
MPA_muscle_list_read(filename,the_mpa_muscle_list)
char *filename;
MPA_Muscle_List_ptr the_mpa_muscle_list;
{
  FILE *infile; 

  if (infile = fopen(filename, "r")) {
    MPA_muscle_list_read_file(infile,the_mpa_muscle_list);
    fclose(infile);
    return(YES);
  } else return(NO);

}


BOOL
MPA_muscle_name_valid(the_mpa_muscle_list,name)
MPA_Muscle_List_ptr the_mpa_muscle_list;
MPA_Muscle_Name name;
{
    MPA_FOR_EACH_MUSCLE(the_mpa_muscle_list)
     if (strcmp(name,muscle_ptr->name) == 0) return(YES);
    MPA_END_FOR_EACH
    return(NO);
}


BOOL
MPA_muscle_index_get(the_mpa_muscle_list,name,index)
MPA_Muscle_List_ptr the_mpa_muscle_list;
MPA_Muscle_Name name;
int *index;
{
    MPA_FOR_EACH_MUSCLE(the_mpa_muscle_list)
     if (strcmp(name,muscle_ptr->name) == 0)  {
       *index = muscle_index;
       return(YES);
     }
    MPA_END_FOR_EACH
    return(NO);
}

BOOL
MPA_muscle_add(the_mpa_muscle_list,name)
MPA_Muscle_List_ptr the_mpa_muscle_list;
MPA_Muscle_Name name;
{

   MPA_Muscle_ptr new_muscle_list,ptr1,ptr2,the_muscle;
   int new_muscle_index,i;

   if (MPA_muscle_name_valid(the_mpa_muscle_list,name))
     return(NO);

   new_muscle_index = the_mpa_muscle_list->muscle_list_size;

   new_muscle_list = (MPA_Muscle_ptr) calloc
                     ((the_mpa_muscle_list->muscle_list_size + 1),
                                         sizeof(MPA_Muscle));
   
   if (new_muscle_index != 0) {
    ptr1 = the_mpa_muscle_list->muscle_list;
    ptr2 = new_muscle_list;
    for (i=0; i<the_mpa_muscle_list->muscle_list_size; i++)
     *ptr2++ = *ptr1++;
    free(the_mpa_muscle_list->muscle_list);
   }

   the_mpa_muscle_list->muscle_list_size += 1;
   the_mpa_muscle_list->muscle_list = new_muscle_list;
   the_muscle = &the_mpa_muscle_list->muscle_list[new_muscle_index];

   strcpy(the_muscle->name,name);
   the_muscle->l = 3;
   the_muscle->m = 3;
   the_muscle->n = 3;
   the_muscle->max_sensitivity = 1.0;
   the_muscle->min_sensitivity = 1.0;
   the_muscle->point_list_size = 0;
   the_muscle->point_list = NULL;

   return(YES);

}

BOOL
MPA_muscle_dispose(the_mpa_muscle_list,name)
MPA_Muscle_List_ptr the_mpa_muscle_list;
MPA_Muscle_Name name;
{
   MPA_Muscle_ptr new_muscle_list,ptr1,ptr2,the_muscle;
   int new_muscle_index,i,the_index,new_muscle_list_size;

   if (!MPA_muscle_index_get(the_mpa_muscle_list,name,&the_index))
     return(NO);
  
   if (!MPA_muscle_remove_ref(the_mpa_muscle_list,the_index))
     return(NO);

   new_muscle_list_size = the_mpa_muscle_list->muscle_list_size - 1;
   if (new_muscle_list_size == 0) {
     the_mpa_muscle_list->muscle_list = NULL;
     the_mpa_muscle_list->muscle_list_size = 0;
     return(YES);
   }
   new_muscle_list = (MPA_Muscle_ptr) calloc
                     ((the_mpa_muscle_list->muscle_list_size - 1),
                       sizeof(MPA_Muscle));
   
   the_muscle = &the_mpa_muscle_list->muscle_list[the_index];
   ptr1 = the_mpa_muscle_list->muscle_list;
   ptr2 = new_muscle_list;

   for (i=0; i<the_mpa_muscle_list->muscle_list_size; i++) {
     if (i != the_index) 
       *ptr2++ = *ptr1++;
     else
       ptr1++;
   }

   free(the_mpa_muscle_list->muscle_list);
   
   the_mpa_muscle_list->muscle_list_size -= 1;
   the_mpa_muscle_list->muscle_list = new_muscle_list;

   return(YES);
}

BOOL
MPA_muscle_rename(the_mpa_muscle_list,old_name,new_name)
MPA_Muscle_List_ptr the_mpa_muscle_list;
MPA_Muscle_Name old_name,new_name;
{

   MPA_Muscle_ptr the_muscle;
   int the_index;

   if (MPA_muscle_name_valid(the_mpa_muscle_list,new_name))
     return(NO);

   if (!MPA_muscle_index_get(the_mpa_muscle_list,old_name,&the_index))
     return(NO);

   the_muscle = &the_mpa_muscle_list->muscle_list[the_index];
   strcpy(the_muscle->name,new_name);
   return(YES);
}


BOOL
MPA_muscle_pointlist_add(the_mpa_muscle_list,name)
MPA_Muscle_List_ptr the_mpa_muscle_list;
MPA_Muscle_Name name;
{
   int the_index,i,j,k;
   MPA_Muscle_ptr the_muscle;
   int index;
   Muscle_Point_ptr ptr;

    if (!MPA_muscle_index_get(the_mpa_muscle_list,name,&the_index))
     return(NO);

    the_muscle = &the_mpa_muscle_list->muscle_list[the_index];
    the_muscle->point_list_size = the_muscle->l * the_muscle->m * the_muscle->n;
    the_muscle->point_list = (Muscle_Point_ptr) calloc((the_muscle->point_list_size),
                                                       sizeof(Muscle_Point));
    ptr = the_muscle->point_list;
    index = 0;
    for (k=0; k<the_muscle->n; k++) {
     for (j=0; j<the_muscle->m; j++) {
      for (i=0; i<the_muscle->l; i++) {
        ptr->index = index;
        ptr->weight = 1.0;
        ptr->deform_list_size = 0;
        ptr->deform_list = NULL;
        ptr++;
        index++;
      }
     }
    }

    return(YES);
}



BOOL
MPA_muscle_sensitivity_set(the_mpa_muscle_list,name,type,value)
MPA_Muscle_List_ptr the_mpa_muscle_list;
MPA_Muscle_Name name;
int type;
float value;
{
   MPA_Muscle_ptr the_muscle;
   int the_index;

    if (!MPA_muscle_index_get(the_mpa_muscle_list,name,&the_index))
     return(NO);

    the_muscle = &the_mpa_muscle_list->muscle_list[the_index];
    switch (type) {
     case MUSCLE_SENSITIVITY_MAX :
       the_muscle->max_sensitivity = value;
     break;
     case MUSCLE_SENSITIVITY_MIN :
       the_muscle->min_sensitivity = value;
     break;
     default :
     break;
    }
    return(YES);
}

BOOL
MPA_muscle_sensitivity_get(the_mpa_muscle_list,name,type,value)
MPA_Muscle_List_ptr the_mpa_muscle_list;
MPA_Muscle_Name name;
int type;
float *value;
{
   MPA_Muscle_ptr the_muscle;
   int the_index;

    if (!MPA_muscle_index_get(the_mpa_muscle_list,name,&the_index))
     return(NO);

    the_muscle = &the_mpa_muscle_list->muscle_list[the_index];
    switch (type) {
     case MUSCLE_SENSITIVITY_MAX :
       *value = the_muscle->max_sensitivity;
     break;
     case MUSCLE_SENSITIVITY_MIN :
       *value = the_muscle->min_sensitivity;
     break;
     default :
     break;
    }
    return(YES);
}

BOOL
MPA_muscle_precesion_set(the_mpa_muscle_list,name,type,value)
MPA_Muscle_List_ptr the_mpa_muscle_list;
MPA_Muscle_Name name;
int type;
int value;
{
   MPA_Muscle_ptr the_muscle;
   int the_index;

    if (!MPA_muscle_index_get(the_mpa_muscle_list,name,&the_index))
     return(NO);

    the_muscle = &the_mpa_muscle_list->muscle_list[the_index];
    switch (type) {
     case MUSCLE_PRECESION_L :
       the_muscle->l = value;
     break;
     case MUSCLE_PRECESION_M :
       the_muscle->m = value;
     break;
     case MUSCLE_PRECESION_N :
       the_muscle->n = value;
     break;
     default :
     break;
    }
    return(YES);
}

BOOL
MPA_muscle_precesion_get(the_mpa_muscle_list,name,type,value)
MPA_Muscle_List_ptr the_mpa_muscle_list;
MPA_Muscle_Name name;
int type;
int *value;
{
   MPA_Muscle_ptr the_muscle;
   int the_index;

    if (!MPA_muscle_index_get(the_mpa_muscle_list,name,&the_index))
     return(NO);

    the_muscle = &the_mpa_muscle_list->muscle_list[the_index];
    switch (type) {
     case MUSCLE_PRECESION_L :
       *value = the_muscle->l;
     break;
     case MUSCLE_PRECESION_M :
       *value = the_muscle->m;
     break;
     case MUSCLE_PRECESION_N :
       *value = the_muscle->n;
     break;
     default :
     break;
    }
    return(YES);
}

void
MPA_muscle_point_weight_set(the_muscle,index,value)
MPA_Muscle_ptr the_muscle;
int index;
float value;
{
   Muscle_Point_ptr the_point;
  
   the_point = &the_muscle->point_list[index];
   the_point->weight = value;    
}

void
MPA_muscle_point_weight_get(the_muscle,index,value)
MPA_Muscle_ptr the_muscle;
int index;
float *value;
{
   Muscle_Point_ptr the_point;
  
   the_point = &the_muscle->point_list[index];
   *value = the_point->weight;    
}


BOOL
MPA_muscle_point_deform_add(the_muscle,point_index,deform)
MPA_Muscle_ptr the_muscle;
int point_index;
Muscle_Deform_ptr deform;
{
    Muscle_Point_ptr the_point;
    Muscle_Deform_ptr ptr1,ptr2,new_deform_list,the_deform;
    int new_index,new_size,i;
    int id_value;

    if ( (point_index < 0) || (point_index > the_muscle->point_list_size) )
     return(NO);

    the_point = &the_muscle->point_list[point_index];  
  
    if (MPA_muscle_point_deform_id_get(the_muscle,point_index,
              deform->deform_type,deform->axis,&id_value)) {
      the_deform = &the_point->deform_list[id_value];
      the_deform->value += deform->value;
      return(YES);
    }

    new_index = the_point->deform_list_size;
    new_size = the_point->deform_list_size + 1;
    
    new_deform_list = (Muscle_Deform_ptr) calloc((new_size),
                                                 sizeof(Muscle_Deform));
    ptr2 = new_deform_list;
    ptr1 = the_point->deform_list;

    for (i=0; i<the_point->deform_list_size; i++) 
      *ptr2++ = *ptr1++;
    free(the_point->deform_list);
    the_point->deform_list = new_deform_list;
    the_point->deform_list_size = new_size;
    the_deform = &the_point->deform_list[new_index];
    the_deform->deform_type = deform->deform_type;
    the_deform->axis  = deform->axis;
    the_deform->value = deform->value;
    
    ptr2 = the_point->deform_list;
    for (i=0; i<the_point->deform_list_size; i++) {
      ptr2->id = i;
      ptr2++;
    }
    return(YES);
}

BOOL
MPA_muscle_point_deform_dispose(the_muscle,point_index,deform_index)
MPA_Muscle_ptr the_muscle;
int point_index;
int deform_index;
{
    Muscle_Point_ptr the_point;
    Muscle_Deform_ptr ptr1,ptr2,new_deform_list,the_deform;
    int new_index,new_size,i;

    if ( (point_index < 0) || (point_index > the_muscle->point_list_size) )
     return(NO);

    the_point = &the_muscle->point_list[point_index];    
    new_size = the_point->deform_list_size - 1;
    
    new_deform_list = (Muscle_Deform_ptr) calloc((new_size),
                                                 sizeof(Muscle_Deform));
    ptr2 = new_deform_list;
    ptr1 = the_point->deform_list;

    for (i=0; i<the_point->deform_list_size; i++) {
     if (i != deform_index)
      *ptr2++ = *ptr1++;
     else
      ptr1++;
    }
    free(the_point->deform_list);
    the_point->deform_list = new_deform_list;
    the_point->deform_list_size = new_size;

    ptr2 = the_point->deform_list;
    for (i=0; i<the_point->deform_list_size; i++) {
      ptr2->id = i;
      ptr2++;
    }
    return(YES);
}

BOOL
MPA_muscle_point_deform_dispose_all(the_muscle,point_index)
MPA_Muscle_ptr the_muscle;
int point_index;
{
    Muscle_Point_ptr the_point;

    if ( (point_index < 0) || (point_index > the_muscle->point_list_size) )
     return(NO);

    the_point = &the_muscle->point_list[point_index];

    if (the_point->deform_list_size > 0) {
      the_point->deform_list_size = 0;
      free(the_point->deform_list);
    }

    return(YES);    

}
    
BOOL
MPA_muscle_point_deform_id_get(the_muscle,point_index,type,axis,id)
MPA_Muscle_ptr the_muscle;
int point_index;
int type,axis;
int *id;
{
    Muscle_Point_ptr the_point;

    if ( (point_index < 0) || (point_index > the_muscle->point_list_size) )
     return(NO);
    the_point = &the_muscle->point_list[point_index];
    if (the_point->deform_list_size <= 0) 
      return(NO);

    MPA_FOR_EACH_MUSCLE_POINT_DEFORM(the_point)
      if ( (deform_ptr->deform_type == type) && (deform_ptr->axis == axis) ) {
        *id = deform_ptr->id;
         return(YES);   
      }
    MPA_END_FOR_EACH

    return(NO);

}

MPA_Table_ptr
MPA_table_new()
{
    MPA_Table_ptr the_table;
    
    the_table = (MPA_Table_ptr) calloc(1,sizeof(MPA_Table));
    the_table->mpa_table_size = 0;
    the_table->mpa_table = NULL;
    return(the_table);
}

BOOL
MPA_table_dispose(the_mpa_table)
MPA_Table_ptr the_mpa_table;
{
   if (the_mpa_table == NULL) return(NO);

   MPA_FOR_EACH_TABLE_ENTRY(the_mpa_table)
    MPA_table_remove_ref(the_mpa_table,table_index);
   MPA_END_FOR_EACH

   if (the_mpa_table->mpa_table_size > 0) 
    free(the_mpa_table->mpa_table);
   the_mpa_table->mpa_table_size = 0;
   the_mpa_table->mpa_table = NULL;
   
   free(the_mpa_table);
   return(YES);
   

}

BOOL
MPA_table_remove_ref(the_mpa_table,table_index)
MPA_Table_ptr the_mpa_table;
int table_index;
{
   MPA_Muscle_Bind_Entry_ptr  the_entry;

   the_entry = &the_mpa_table->mpa_table[table_index];
   if (the_entry == NULL) return(NO);
   if (the_entry->bind_list_size > 0) 
    free(the_entry->bind_list);
   return(YES);
}

int
MPA_table_index_get(the_mpa_table,name)
MPA_Table_ptr the_mpa_table;
MPA_Name name;
{
    MPA_FOR_EACH_TABLE_ENTRY(the_mpa_table)
     if (strcmp(name,table_ptr->name) == 0)  {
       return(table_index);
     }
    MPA_END_FOR_EACH
    return(SM_INDEX_ERROR);
}

BOOL
MPA_table_add_entry(the_mpa_table,name)
MPA_Table_ptr the_mpa_table;
MPA_Name name;
{
  
   MPA_Muscle_Bind_Entry_ptr new_table,ptr1,ptr2,the_entry;
   int new_table_index,i;

   if (MPA_table_index_get(the_mpa_table,name) != SM_INDEX_ERROR)
    return(NO);

   new_table_index = the_mpa_table->mpa_table_size;

   new_table = (MPA_Muscle_Bind_Entry_ptr) calloc
                     ((the_mpa_table->mpa_table_size + 1),
                                         sizeof(MPA_Muscle_Bind_Entry));
   
   if (new_table_index != 0) {
    ptr1 = the_mpa_table->mpa_table;
    ptr2 = new_table;
    for (i=0; i<the_mpa_table->mpa_table_size; i++)
     *ptr2++ = *ptr1++;
    free(the_mpa_table->mpa_table);
   }

   the_mpa_table->mpa_table_size += 1;
   the_mpa_table->mpa_table = new_table;
   the_entry = &the_mpa_table->mpa_table[new_table_index];

   strcpy(the_entry->name,name);
   the_entry->bind_list_size = 0;
   the_entry->bind_list = NULL;

   return(YES);
    

}

BOOL
MPA_table_remove_entry(the_mpa_table,name)
MPA_Table_ptr the_mpa_table;
MPA_Name name;
{
   MPA_Muscle_Bind_Entry_ptr new_table,ptr1,ptr2,the_entry;
   int the_index,i,new_size;

   the_index = MPA_table_index_get(the_mpa_table,name);

   if (the_index == SM_INDEX_ERROR)
    return(NO);

   if (!MPA_table_remove_ref(the_mpa_table,the_index))
    return(NO);

   new_table = (MPA_Muscle_Bind_Entry_ptr) calloc
                     ((the_mpa_table->mpa_table_size - 1),
                                         sizeof(MPA_Muscle_Bind_Entry));
   
   new_size = the_mpa_table->mpa_table_size - 1;

   ptr1 = the_mpa_table->mpa_table;
   ptr2 = new_table;
   for (i=0; i<the_mpa_table->mpa_table_size; i++) {
    if (i != the_index)
     *ptr2++ = *ptr1++;
    else
     ptr1++;
   }

   free(the_mpa_table->mpa_table);
   the_mpa_table->mpa_table_size -= 1;
   the_mpa_table->mpa_table = new_table;

   return(YES);
}

BOOL
MPA_table_entry_bind_index_get(the_mpa_table,entry,reg,the_index)
MPA_Table_ptr the_mpa_table;
int entry;
int reg;
int *the_index;
{
    MPA_Muscle_Bind_Entry_ptr the_entry;
    
    *the_index = SM_INDEX_ERROR;
    the_entry = &the_mpa_table->mpa_table[entry];
    
    MPA_FOR_EACH_TABLE_ENTRY_BIND(the_entry)
      if (bind_ptr->region_index == reg) {
       *the_index = bind_index;
       return(YES);
      }
    MPA_END_FOR_EACH

    return(NO);
}


BOOL
MPA_table_entry_insert_tuple(the_mpa_table,entry,reg,muscle)
MPA_Table_ptr the_mpa_table;
int entry,reg;
MPA_Muscle_Tuple muscle;
{
  int i,the_index;
  MPA_Muscle_Bind_Entry_ptr the_entry;
  MPA_Muscle_Bind_ptr the_bind;
  MPA_Muscle_Tuple_ptr ptr1,ptr2,new_tuple_list;

   MPA_table_entry_bind_index_get(the_mpa_table,entry,reg,&the_index);
   if (the_index == SM_INDEX_ERROR) return(NO);
   
   the_entry = &the_mpa_table->mpa_table[entry];
   the_bind = &the_entry->bind_list[the_index];

   new_tuple_list = (MPA_Muscle_Tuple_ptr) calloc
                     ((the_bind->muscle_tuple_list_size + 1),
                                         sizeof(MPA_Muscle_Tuple));
   
   ptr1 = the_bind->muscle_tuple_list;
   ptr2 = new_tuple_list;
   for (i=0; i<the_bind->muscle_tuple_list_size; i++) {
     *ptr2++ = *ptr1++;
   }
   
   ptr2->index = muscle.index;
   ptr2->intensity = muscle.intensity;

   free(the_bind->muscle_tuple_list);
   the_bind->muscle_tuple_list = new_tuple_list;
   the_bind->muscle_tuple_list_size += 1;

   return(NO);
}

void
MPA_table_entry_insert_region(the_mpa_table,entry,reg)
MPA_Table_ptr the_mpa_table;
int entry,reg;
{
  int i,the_index;
  MPA_Muscle_Bind_Entry_ptr the_entry;
  MPA_Muscle_Bind_ptr new_bind_list, ptr1, ptr2;

   
   the_entry = &the_mpa_table->mpa_table[entry];

   new_bind_list = (MPA_Muscle_Bind_ptr) calloc
                     ((the_entry->bind_list_size + 1),
                                         sizeof(MPA_Muscle_Bind));
   
   ptr1 = the_entry->bind_list;
   ptr2 = new_bind_list;
   for (i=0; i<the_entry->bind_list_size; i++) {
     *ptr2++ = *ptr1++;
   }
   
   ptr2->region_index = reg;
   ptr2->muscle_tuple_list_size = 0;
   ptr2->muscle_tuple_list = NULL;
   
   if (the_entry->bind_list)
    free(the_entry->bind_list);
   the_entry->bind_list = new_bind_list;
   the_entry->bind_list_size += 1;

}

BOOL
MPA_table_write(filename, the_mpa_table)
char *filename;
MPA_Table_ptr the_mpa_table;
{
  FILE *infile;
  int i;
  
    if (infile = fopen(filename, "w")) {
      fprintf(infile, "%d\n", the_mpa_table->mpa_table_size);
      MPA_FOR_EACH_TABLE_ENTRY(the_mpa_table)
        fprintf(infile, "%s %d\n", table_ptr->name, table_ptr->bind_list_size);
	MPA_FOR_EACH_TABLE_ENTRY_BIND(table_ptr)
	  fprintf(infile, "%d %d ", bind_ptr->region_index, 
	                            bind_ptr->muscle_tuple_list_size);
	  for (i=0; i<bind_ptr->muscle_tuple_list_size; i++)
	   fprintf(infile, "( %d %f )", bind_ptr->muscle_tuple_list[i].index,
	                                bind_ptr->muscle_tuple_list[i].intensity); 
	  fprintf(infile, "\n");
	MPA_END_FOR_EACH
      MPA_END_FOR_EACH
      fclose(infile);
      return(YES);
    } else
      return(NO);
    
}


BOOL
MPA_table_read_file(infile, the_mpa_table)
FILE *infile;
MPA_Table_ptr the_mpa_table;
{
  int i;
  char garbage;
    
    if (the_mpa_table != NULL) {
      fscanf(infile, "%d\n",&the_mpa_table->mpa_table_size);
      the_mpa_table->mpa_table = (MPA_Muscle_Bind_Entry_ptr) 
        calloc(the_mpa_table->mpa_table_size,sizeof(MPA_Muscle_Bind_Entry));   
      MPA_FOR_EACH_TABLE_ENTRY(the_mpa_table)
        fscanf(infile, "%s %d\n", table_ptr->name, &table_ptr->bind_list_size);
	table_ptr->bind_list = (MPA_Muscle_Bind_ptr) 
        calloc(table_ptr->bind_list_size,sizeof(MPA_Muscle_Bind));   
	MPA_FOR_EACH_TABLE_ENTRY_BIND(table_ptr)
	  fscanf(infile, "%d %d ", &bind_ptr->region_index, 
	                           &bind_ptr->muscle_tuple_list_size);
	  bind_ptr->muscle_tuple_list = (MPA_Muscle_Tuple_ptr) 
          calloc(bind_ptr->muscle_tuple_list_size,sizeof(MPA_Muscle_Tuple));   
	  for (i=0; i<bind_ptr->muscle_tuple_list_size; i++)
	   fscanf(infile, "%c %d %f %c", &garbage, &bind_ptr->muscle_tuple_list[i].index,
	                                &bind_ptr->muscle_tuple_list[i].intensity, &garbage); 
	  fscanf(infile, "\n");
	MPA_END_FOR_EACH
      MPA_END_FOR_EACH
      fclose(infile);
      return(YES);
    } else
      return(NO);
     
}

BOOL
MPA_table_read(filename,the_mpa_table)
char *filename;
MPA_Table_ptr the_mpa_table;
{
  FILE *infile; 

  if (infile = fopen(filename, "r")) {
    MPA_table_read_file(infile,the_mpa_table);
    fclose(infile);
    return(YES);
  } else return(NO);

}

void
MPA_deform_a_vect(out_vect, in_vect, type, axis, g_value, p_value, c_value)
VECTOR_3D out_vect, in_vect;
int type, axis;
float g_value, p_value, c_value;
{

    VECTOR_3D t_vect;
    float numerator, denominator, value;
    
    switch (type) {
	case MUSCLE_DEFORM_ROTATE:
	/*
	    value = c_value - p_value;
	    fprintf(stderr, "value %f\n", value);
	*/
	
	    value = c_value;	
	    SMGD_rotate_point(in_vect, axis, value * g_value, out_vect);	    
	break;
	case MUSCLE_DEFORM_MOVE:
	    V3D_NULL(t_vect);
	    value = c_value - p_value;
	    t_vect[axis] += value * g_value;
	    SMGD_translate_point(in_vect, t_vect,  out_vect);	    
	break;
	case MUSCLE_DEFORM_SHIFT:
	    value = c_value - p_value;
	    SMGD_shift_point(in_vect, value * g_value, axis, out_vect);
	break;
	case MUSCLE_DEFORM_SHEAR:
	    value = c_value - p_value;
	    SMGD_shear_point(in_vect, value * g_value, axis, out_vect);
	break;
	case MUSCLE_DEFORM_SQUASH:
	
	/*	
	    if (c_value > 0.0) 
		numerator = (1.0 + (g_value - 1.0) * c_value);
	    else
		numerator = (1.0 + (1.0/g_value - 1.0) * (-c_value));
		    
	    if (p_value > 0.0) 
		denominator = (1.0 + (g_value - 1.0) * p_value);
	    else
		denominator = (1.0 + (1.0/g_value - 1.0) * p_value);
			    
	    value = numerator/denominator;
	
	    if ( (c_value - p_value) > 0.0) 
		numerator = (1.0 + (g_value - 1.0) * (c_value - p_value));
	    else
		numerator = (1.0 + (1.0/g_value - 1.0) * (-(c_value - p_value)));       
	    value = numerator;
	*/
	    numerator = (1.0 + (g_value - 1.0) * c_value);
	    denominator = (1.0 + (g_value - 1.0) * p_value);
			    
	    value = numerator/denominator;
	    V3D_INIT(t_vect, 1.0, 1.0, 1.0);
	    t_vect[axis] *= value;
	    SMGD_scale_point(in_vect, t_vect, out_vect);
	break;
	default:
	break;
    }
}

void
MPA_control_box_set(the_box, origin, the_muscle, p_value, c_value)
SM_Region_Controlbox_ptr the_box;
VECTOR_3D origin;
MPA_Muscle_ptr the_muscle;
float p_value, c_value;
{
  VECTOR_3D in_vect, out_vect;
/*  
    SMGD_controlbox_origin_get(the_box, origin);
 */
    MPA_FOR_EACH_MUSCLE_POINT(the_muscle)
      V3D_COPY(in_vect, the_box->vertex_list[point_index].point);
      V3D_COPY(out_vect, the_box->vertex_list[point_index].point);
      MPA_FOR_EACH_MUSCLE_POINT_DEFORM(point_ptr)
        V3D_SUB(in_vect, in_vect, origin);
	MPA_deform_a_vect(out_vect, in_vect, deform_ptr->deform_type, 
	                  deform_ptr->axis, deform_ptr->value, p_value, c_value);
	V3D_ADD(out_vect, out_vect, origin);
	V3D_COPY(in_vect, out_vect);
      MPA_END_FOR_EACH
      V3D_COPY(the_box->vertex_list[point_index].point, out_vect);
    MPA_END_FOR_EACH
}


void MPA_general_apply(the_mpa_table, mpa_index, the_mpa_muscle_list, the_region_list,
the_surface, the_neutral_face, pre_value, cur_value)
MPA_Table_ptr the_mpa_table;
int  mpa_index;
MPA_Muscle_List_ptr the_mpa_muscle_list;
SM_Region_List_ptr the_region_list;
SM_Surface_ptr the_surface,the_neutral_face;
double pre_value, cur_value;

{
    MPA_Muscle_Bind_Entry_ptr the_mpa;
    MPA_Muscle_ptr the_muscle;
    SM_Region_Controlbox_ptr the_box, the_N_box;
    int m_index, the_index, i, l, m, n;
    SM_ffd_parameters ffd_parm;
    VECTOR_3D origin;

    the_mpa = &the_mpa_table->mpa_table[mpa_index];
    MPA_FOR_EACH_TABLE_ENTRY_BIND(the_mpa)
       for(i=0; i<bind_ptr->muscle_tuple_list_size; i++) {
	   m_index = bind_ptr->muscle_tuple_list[i].index;
           the_muscle = &the_mpa_muscle_list->muscle_list[m_index];
	   l = the_muscle->l; m = the_muscle->m; n = the_muscle->n;
           the_box = SM_region_controlbox_new(l,m,n);
           the_N_box = SM_region_controlbox_new(l,m,n);
           SM_region_controlbox_recreate(the_box,l,m,n); 
           SM_region_controlbox_recreate(the_N_box,l,m,n); 
           SM_region_controlbox_get(the_surface,the_region_list,bind_ptr->region_index,the_box);
           SM_region_controlbox_get(the_neutral_face,the_region_list,bind_ptr->region_index,the_N_box);
           AU_face_ffd_init(the_N_box,&ffd_parm); 
           SMGD_controlbox_origin_get(the_N_box, origin);
	   MPA_control_box_set(the_box, origin, the_muscle, pre_value, cur_value);
	   SMGD_region_index_freeform_apply(the_surface, the_neutral_face, the_region_list, bind_ptr->region_index, 
					    ffd_parm, the_box);
	   SM_region_controlbox_dispose(the_box);
	   SM_region_controlbox_dispose(the_N_box);
       }
    MPA_END_FOR_EACH
    
}


/*----------------------------------------------------------------------*/

/*
 * This routine is similar to MPA_muscle_list_read, this is renamed for
 * convenience
 */
Shape_Muscle_List_ptr
Shape_muscle_list_new()
{
    Shape_Muscle_List_ptr the_shape_list;
    
    the_shape_list = (Shape_Muscle_List_ptr) calloc(1,sizeof(Shape_Muscle_List));
    the_shape_list->muscle_list_size = 0;
    the_shape_list->muscle_list = NULL;
    return(the_shape_list);
}



BOOL
Shape_muscle_list_read(filename,the_shape_muscle_list)
char *filename;
Shape_Muscle_List_ptr the_shape_muscle_list;
{
  FILE *infile; 

  if (infile = fopen(filename, "r")) {
    MPA_muscle_list_read_file(infile,the_shape_muscle_list);
    fclose(infile);
    return(YES);
  } else return(NO);

}

void
Shape_neutral_face_box_get(the_shape_list, index, the_surface)
Shape_Muscle_List_ptr the_shape_list;
int index;
SM_Surface_ptr the_surface;
{
    Shape_Muscle_ptr the_shape;
    int the_index, i, l, m, n;

    the_shape = &the_shape_list->muscle_list[index];
    l = the_shape->l; m = the_shape->m; n = the_shape->n;
    the_neutral_box = (SM_Surface_Controlbox_ptr) calloc(1, sizeof(SM_Surface_Controlbox));
    the_neutral_box->vertex_list = (SM_Surface_Controlbox_vertex_ptr) 
          calloc(l*m*n, sizeof(SM_Surface_Controlbox_vertex));
    SMGD_surface_box_create(the_surface, the_neutral_box, NULL, l, m, n); 
    SMGD_controlbox_origin_get(the_neutral_box, the_neutral_box_origin);

}

BOOL
Shape_controlbox_copy(from_box, to_box)
SM_Surface_Controlbox_ptr from_box, to_box;
{
       SM_Region_Controlbox_Vertex_ptr ptr1,ptr2;
       int i;
       
       if (from_box->vertex_list_size != to_box->vertex_list_size)
          return(NO);
	  
       ptr1 = from_box->vertex_list;
       ptr2 = to_box->vertex_list;
              
       for (i=0; i<from_box->vertex_list_size; i++) 
	    *ptr2++ = *ptr1++;
	    
	return(YES);
}

void
Shape_print_box(filename, the_box)
char *filename;
SM_Surface_Controlbox_ptr the_box;
{
    FILE *infile;
    int i;
    
    fprintf(stderr, "%d\n", the_box->vertex_list_size);
    if (infile = fopen(filename, "w")) {
	for (i=0; i<the_box->vertex_list_size; i++)
	  fprintf(infile, "%f %f %f \n", the_box->vertex_list[i].point[0], 
					 the_box->vertex_list[i].point[1],
					 the_box->vertex_list[i].point[2]);
    }
    fclose(infile);
    
}

void
Shape_general_apply(the_shape_list, index, the_surface, prev_value, curr_value)
Shape_Muscle_List_ptr the_shape_list;
int index;
SM_Surface_ptr the_surface;
float prev_value, curr_value;

{
    SM_Surface_Controlbox_ptr the_box, dummy_box;
    int the_index, i, l, m, n;
    SM_ffd_parameters ffd_parm;
    Shape_Muscle_ptr the_shape;
    VECTOR_3D origin;

    
    the_shape = &the_shape_list->muscle_list[index];
/*    
    l = the_shape->l; m = the_shape->m; n = the_shape->n;
    the_box = (SM_Surface_Controlbox_ptr) calloc(1, sizeof(SM_Surface_Controlbox));
    the_box->vertex_list = (SM_Surface_Controlbox_vertex_ptr) 
          calloc(l*m*n, sizeof(SM_Surface_Controlbox_vertex));
    dummy_box = (SM_Surface_Controlbox_ptr) calloc(1, sizeof(SM_Surface_Controlbox));
    dummy_box->vertex_list = (SM_Surface_Controlbox_vertex_ptr) 
          calloc(l*m*n, sizeof(SM_Surface_Controlbox_vertex));
    SMGD_surface_box_create(the_surface, the_box, dummy_box, l, m, n); 
  
    if (Shape_controlbox_copy(the_neutral_box, the_box)) {
 */
     V3D_COPY(origin, the_neutral_box_origin);
     MPA_control_box_set(the_neutral_box, origin, the_shape, prev_value, curr_value); 
     Shape_surface_freeform_apply(the_surface, the_neutral_box);
 /*
    }
    
    
    SM_region_controlbox_dispose(the_box);
    SM_region_controlbox_dispose(dummy_box);
 */
    
}
