
/*------------------------------------------------------------------------------*/
/*										*/
/*										*/
/*  This software module was originally developed by                            */
/*        Prem Kalra, MIRALab, University of Geneva                             */
/*  and edited by                                                               */
/*        Igor Pandzic, MIRALab, University of Geneva                           */
/*        Marc Escher, MIRALab, University of Geneva                            */
/*        Pierre Beylot, MIRALab, University of Geneva                          */
/*  in the course of development of the MPEG-4 Video (ISO/IEC 14496-2).	        */
/*  This software module is an implementation of a part of one or more	        */
/*  MPEG-4 Video (ISO/IEC 14496-2) tools as specified by the MPEG-4 Video       */
/*  (ISO/IEC 14496-2). ISO/IEC gives users of the                               */
/*  MPEG-4 Video (ISO/IEC 14496-2) free license to this software module or	*/
/*  modifications thereof for use in hardware or software products claiming 	*/
/*  conformance to the MPEG-4 Video (ISO/IEC 14496-2). Those intending to use	*/
/*  this software module in hardware or software products are advised that	*/
/*  its use may infringe existing patents. The original developer of this	*/
/*  software module and his/her company, the subsequent editors and their	*/
/*  companies, and ISO/IEC have no liability for use of this software		*/
/*  module or modifications thereof. Copyright is not released for non		*/
/*  MPEG-4 Video (ISO/IEC 14496-2) conforming products.                         */
/*  MIRALab, University of Geneva retains full right to use the code for its    */
/*  own purpose, assign or donate the code to a third party and to inhibit      */
/*  third parties from using the code for non MPEG-4 Video (ISO/IEC 14496-2)    */
/*  conforming products. This copyright notice must be included in all copies   */
/*  or derivative works. Copyright (c) (1997).			      		*/
/*										*/
/*										*/
/*------------------------------------------------------------------------------*/



#include "script.h"

typedef char MPA_Muscle_Name[50];

typedef struct {
 short id;
 short deform_type;
 short axis;
 float value;
} Muscle_Deform, *Muscle_Deform_ptr, 
  Shape_Deform, *Shape_Deform_ptr;

typedef struct {
 short index;
 float weight;
 Muscle_Deform_ptr deform_list;
 int deform_list_size;
} Muscle_Point, *Muscle_Point_ptr, 
  Shape_Point, *Shape_Point_ptr;

typedef struct {
 MPA_Muscle_Name name;
 short l,m,n;
 float max_sensitivity;
 float min_sensitivity;
 Muscle_Point_ptr point_list;
 int point_list_size;
} MPA_Muscle, *MPA_Muscle_ptr, 
  Shape_Muscle, *Shape_Muscle_ptr;

typedef struct {
 MPA_Muscle_ptr muscle_list;
 int muscle_list_size;
} MPA_Muscle_List, *MPA_Muscle_List_ptr, 
  Shape_Muscle_List, *Shape_Muscle_List_ptr;

typedef struct {
  int index;
  float intensity;
} MPA_Muscle_Tuple, *MPA_Muscle_Tuple_ptr;

typedef struct {
  int region_index;
  int muscle_tuple_list_size;
  MPA_Muscle_Tuple_ptr muscle_tuple_list;
} MPA_Muscle_Bind, *MPA_Muscle_Bind_ptr;


typedef struct {
  MPA_Name  name;
  int bind_list_size;
  MPA_Muscle_Bind_ptr  bind_list;
} MPA_Muscle_Bind_Entry, *MPA_Muscle_Bind_Entry_ptr;

typedef struct {
  int mpa_table_size;
  MPA_Muscle_Bind_Entry_ptr mpa_table;
} MPA_Table, *MPA_Table_ptr;

#define MUSCLE_DEFORM_NULL   -1
#define MUSCLE_DEFORM_ROTATE  0
#define MUSCLE_DEFORM_MOVE    1
#define MUSCLE_DEFORM_SHIFT   2
#define MUSCLE_DEFORM_SHEAR   3
#define MUSCLE_DEFORM_SQUASH  4

#define MUSCLE_SENSITIVITY_MAX 0
#define MUSCLE_SENSITIVITY_MIN 1

#define MUSCLE_PRECESION_L 0
#define MUSCLE_PRECESION_M 1
#define MUSCLE_PRECESION_N 2

#define MPA_FOR_EACH_MUSCLE(the_mpa_muscle_list) \
    { \
       MPA_Muscle_ptr muscle_ptr; \
       register int muscle_index; \
       for ( muscle_index = 0, \
             muscle_ptr = the_mpa_muscle_list->muscle_list; \
             muscle_index < the_mpa_muscle_list->muscle_list_size; \
             muscle_index++, \
             muscle_ptr++ ) \
             {

#define MPA_FOR_EACH_MUSCLE_POINT(the_muscle_ptr) \
    { \
       Muscle_Point_ptr point_ptr; \
       register int point_index; \
       for ( point_index = 0, \
             point_ptr = the_muscle_ptr->point_list; \
             point_index < the_muscle_ptr->point_list_size; \
             point_index++, \
             point_ptr++ ) \
             {

#define MPA_FOR_EACH_MUSCLE_POINT_DEFORM(the_point_ptr) \
    { \
       Muscle_Deform_ptr deform_ptr; \
       register int deform_index; \
       for ( deform_index = 0, \
             deform_ptr = the_point_ptr->deform_list; \
             deform_index < the_point_ptr->deform_list_size; \
             deform_index++, \
             deform_ptr++ ) \
             {

#define MPA_FOR_EACH_TABLE_ENTRY(the_table_ptr) \
    { \
       MPA_Muscle_Bind_Entry_ptr table_ptr; \
       register int table_index; \
       for ( table_index = 0, \
             table_ptr = the_table_ptr->mpa_table; \
             table_index < the_table_ptr->mpa_table_size; \
             table_index++, \
             table_ptr++ ) \
             {

#define MPA_FOR_EACH_TABLE_ENTRY_BIND(the_bind_ptr) \
    { \
       MPA_Muscle_Bind_ptr bind_ptr; \
       register int bind_index; \
       for ( bind_index = 0, \
             bind_ptr = the_bind_ptr->bind_list; \
             bind_index < the_bind_ptr->bind_list_size; \
             bind_index++, \
             bind_ptr++ ) \
             {


#define MPA_END_FOR_EACH }}

extern MPA_Muscle_List_ptr
MPA_muscle_list_new();

extern BOOL
MPA_muscle_list_dispose(MPA_Muscle_List_ptr);

extern BOOL 
MPA_muscle_remove_ref(MPA_Muscle_List_ptr,int);

extern void
MPA_muscle_list_print(MPA_Muscle_List_ptr);

extern void
MPA_muscle_point_print(MPA_Muscle_List_ptr,int);

extern BOOL
MPA_muscle_name_valid(MPA_Muscle_List_ptr,MPA_Muscle_Name);

extern BOOL
MPA_muscle_index_get(MPA_Muscle_List_ptr,MPA_Muscle_Name,int *);

extern BOOL
MPA_muscle_add(MPA_Muscle_List_ptr,MPA_Muscle_Name);

extern BOOL
MPA_muscle_dispose(MPA_Muscle_List_ptr,MPA_Muscle_Name);

extern BOOL
MPA_muscle_rename(MPA_Muscle_List_ptr,MPA_Muscle_Name,MPA_Muscle_Name);

extern BOOL
MPA_muscle_pointlist_add(MPA_Muscle_List_ptr,MPA_Muscle_Name);

extern BOOL
MPA_muscle_sensitivity_set(MPA_Muscle_List_ptr,MPA_Muscle_Name,int,double);

extern BOOL
MPA_muscle_sensitivity_get(MPA_Muscle_List_ptr,MPA_Muscle_Name,int,float *);

extern BOOL
MPA_muscle_precesion_set(MPA_Muscle_List_ptr,MPA_Muscle_Name,int,int);

extern BOOL
MPA_muscle_precesion_get(MPA_Muscle_List_ptr,MPA_Muscle_Name,int,int *);

extern void
MPA_muscle_point_weight_set(MPA_Muscle_ptr,int,double);

extern void
MPA_muscle_point_weight_get(MPA_Muscle_ptr,int,float *);

extern BOOL
MPA_muscle_point_deform_add(MPA_Muscle_ptr,int,Muscle_Deform_ptr);

extern BOOL
MPA_muscle_point_deform_dispose(MPA_Muscle_ptr,int,int);

extern BOOL
MPA_muscle_point_deform_dispose_all(MPA_Muscle_ptr,int);

extern BOOL
MPA_muscle_point_deform_id_get(MPA_Muscle_ptr,int,int,int,int *);

extern BOOL
MPA_muscle_list_read(char *,MPA_Muscle_List_ptr);

extern void
MPA_muscle_list_read_file(FILE *,MPA_Muscle_List_ptr);

extern void
MPA_muscle_point_read_file(FILE *,MPA_Muscle_ptr);

extern BOOL
MPA_muscle_list_write(char *,MPA_Muscle_List_ptr);

extern void
MPA_muscle_list_write_file(FILE *,MPA_Muscle_List_ptr);

extern void
MPA_muscle_point_write_file(FILE *,MPA_Muscle_ptr);

extern MPA_Table_ptr
MPA_table_new();

extern BOOL
MPA_table_dispose(MPA_Table_ptr);

extern int
MPA_table_index_get(MPA_Table_ptr,MPA_Name);

extern BOOL 
MPA_table_add_entry(MPA_Table_ptr,MPA_Name);

extern BOOL 
MPA_table_remove_entry(MPA_Table_ptr,MPA_Name);

extern BOOL
MPA_table_remove_ref(MPA_Table_ptr,int);

extern BOOL
MPA_table_entry_bind_index_get(MPA_Table_ptr,int,int,int *);

extern BOOL
MPA_table_entry_insert_tuple(MPA_Table_ptr,int,int,MPA_Muscle_Tuple);

extern void
MPA_table_entry_insert_region(MPA_Table_ptr,int,int);

extern BOOL
MPA_table_write(char *, MPA_Table_ptr);

extern BOOL
MPA_table_read_file(FILE *, MPA_Table_ptr);

extern BOOL
MPA_table_read(char *, MPA_Table_ptr);

extern void
MPA_deform_a_vect(VECTOR_3D, VECTOR_3D, int, int, double, double, double);

extern void
MPA_control_box_set(SM_Region_Controlbox_ptr, VECTOR_3D, MPA_Muscle_ptr, double, double);

extern void
MPA_general_apply(MPA_Table_ptr, int, MPA_Muscle_List_ptr, 
SM_Region_List_ptr, SM_Surface_ptr, SM_Surface_ptr, double, double);


/*--------------------------------------------------------*/
extern Shape_Muscle_List_ptr
Shape_muscle_list_new();

extern BOOL
Shape_muscle_list_read(char *, Shape_Muscle_List_ptr);

extern void
Shape_general_apply(Shape_Muscle_List_ptr, int, SM_Surface_ptr, double, double);

extern void
Shape_neutral_face_box_get(Shape_Muscle_List_ptr, int,  SM_Surface_ptr);

extern BOOL
Shape_controlbox_copy(SM_Surface_Controlbox_ptr, SM_Surface_Controlbox_ptr);

