
/*------------------------------------------------------------------------------*/
/*										*/
/*										*/
/*  This software module was originally developed by                            */
/*        Prem Kalra, MIRALab, University of Geneva                             */
/*  and edited by                                                               */
/*        Igor Pandzic, MIRALab, University of Geneva                           */
/*        Marc Escher, MIRALab, University of Geneva                            */
/*        Pierre Beylot, MIRALab, University of Geneva                          */
/*  in the course of development of the MPEG-4 Video (ISO/IEC 14496-2).	        */
/*  This software module is an implementation of a part of one or more	        */
/*  MPEG-4 Video (ISO/IEC 14496-2) tools as specified by the MPEG-4 Video       */
/*  (ISO/IEC 14496-2). ISO/IEC gives users of the                               */
/*  MPEG-4 Video (ISO/IEC 14496-2) free license to this software module or	*/
/*  modifications thereof for use in hardware or software products claiming 	*/
/*  conformance to the MPEG-4 Video (ISO/IEC 14496-2). Those intending to use	*/
/*  this software module in hardware or software products are advised that	*/
/*  its use may infringe existing patents. The original developer of this	*/
/*  software module and his/her company, the subsequent editors and their	*/
/*  companies, and ISO/IEC have no liability for use of this software		*/
/*  module or modifications thereof. Copyright is not released for non		*/
/*  MPEG-4 Video (ISO/IEC 14496-2) conforming products.                         */
/*  MIRALab, University of Geneva retains full right to use the code for its    */
/*  own purpose, assign or donate the code to a third party and to inhibit      */
/*  third parties from using the code for non MPEG-4 Video (ISO/IEC 14496-2)    */
/*  conforming products. This copyright notice must be included in all copies   */
/*  or derivative works. Copyright (c) (1997).			      		*/
/*										*/
/*										*/
/*------------------------------------------------------------------------------*/

#include "SurfMan.h"
#include "RegMan.h"

long SM_color_region_triangle = 0xff00ff00;
long SM_color_region_edge = 0xff000000;
long SM_color_region_vertex = 0xff00ffff;
long SM_color_region_vertex_stiffness = 0xfff0f0ff;
 

SM_Region_List_ptr
SM_region_list_new(the_surface)
SM_Surface_ptr the_surface;
/*
** Allocates / assignes a new list of regions, empty region
*/
{
   SM_Region_List_ptr the_region_list;

   the_region_list = (SM_Region_List_ptr) calloc(1,sizeof(SM_Region_List));
   the_region_list->struct_used = YES;
   the_region_list->region_list_size = 0;
   the_region_list->region_list = NULL;
   the_region_list->the_surface = the_surface;
   return(the_region_list);
}

void
SM_region_list_dispose(the_region_list, the_surface)
SM_Region_List_ptr the_region_list;
SM_Surface_ptr the_surface;
/*
** To destroy the region list of the surface loaded
*/
{
   SM_FOR_EACH_REGION(the_region_list)
    SM_region_remove_ref(the_region_list,region_index, the_surface);
    free(region_ptr);
   SM_END_FOR_EACH

   free(the_region_list);
}

void
SM_region_dispose_all(the_region_list, the_surface)
SM_Region_List_ptr the_region_list;
SM_Surface_ptr the_surface;
/*
** To destroy the region list of the surface loaded
*/
{
   SM_FOR_EACH_REGION(the_region_list)
    SM_region_remove_ref(the_region_list,region_index, the_surface);
   SM_END_FOR_EACH

   if (the_region_list->region_list_size > 0) 
    free(the_region_list->region_list);
   the_region_list->region_list_size = 0;
   the_region_list->region_list = NULL;
}

BOOL
SM_region_print(the_region_list,the_name)

SM_Region_List_ptr the_region_list;
SM_Region_Name the_name;
{
      SM_FOR_EACH_REGION(the_region_list)
       if (strcmp(region_ptr->name,the_name) == 0) {
         printf("the region name %s \n",region_ptr->name);
         printf("the region id %d \n",region_ptr->region_id);
         printf("the vertices %d \n",region_ptr->region_vertex_list_size);
         printf("the edge %d \n",region_ptr->region_edge_list_size);
         printf("the triangle %d \n",region_ptr->region_tri_list_size);
         return(YES);
       }
      SM_END_FOR_EACH
      return(NO); 
}

void
SM_region_list_print(the_region_list)
SM_Region_List_ptr the_region_list;
{
   if (the_region_list != NULL) {
      SM_FOR_EACH_REGION(the_region_list) 
         printf(" the region ptr %d \n",region_ptr);
         printf("the region name %s \n",region_ptr->name);
         printf("the region id %d \n",region_ptr->region_id);
         printf("the vertices %d \n",region_ptr->region_vertex_list_size);
         printf("the edge %d \n",region_ptr->region_edge_list_size);
         printf("the triangle %d \n",region_ptr->region_tri_list_size);
      SM_END_FOR_EACH
   }
}

void
SM_region_read_file(the_region_list, the_index, the_surface, the_file)
SM_Region_List_ptr the_region_list;
int the_index;
SM_Surface_ptr the_surface;
FILE *the_file;

{
    SM_Region_ptr the_region_ptr;
    SM_Region_Vertex_ptr v_ptr;
    int i, e1, e2, e3;
    char garbage_char,garbage_string[20];
    int garbage_int;
    int compteur;
    
    the_region_ptr = &the_region_list->region_list[the_index];
    if (the_region_ptr->region_edge_list != NULL) free(the_region_ptr->region_edge_list);
    if (the_region_ptr->region_tri_list != NULL) free(the_region_ptr->region_tri_list);
    if (the_region_ptr->region_vertex_list != NULL) free(the_region_ptr->region_vertex_list);
    fscanf(the_file,"%c\n",&garbage_char);
    fscanf(the_file,"%s\n",the_region_ptr->name);
    fscanf(the_file,"%d\n",&the_region_ptr->region_id);
    fscanf(the_file,"%d %s\n%c\n",&the_region_ptr->region_vertex_list_size,
                                  garbage_string,&garbage_char);
    the_region_ptr->region_vertex_list = (SM_Region_Vertex_ptr)
    calloc((the_region_ptr->region_vertex_list_size), sizeof(SM_Region_Vertex));
    v_ptr = the_region_ptr->region_vertex_list;
    for (i=0; i<the_region_ptr->region_vertex_list_size; i++){ 
     fscanf(the_file,"%c %d %f %f %f %d %c",&garbage_char,
                &v_ptr->id,
                &v_ptr->properties.stiffness,
                &v_ptr->properties.young_modulus,
                &v_ptr->properties.poisson_ratio,
                &v_ptr->anchor,&garbage_char); 
     v_ptr++;
    }
    fscanf(the_file,"%c\n",&garbage_char);
    fscanf(the_file,"%d %s\n%c\n",&the_region_ptr->region_edge_list_size,
                                  garbage_string,&garbage_char);
/*
    the_region_ptr->region_edge_list = (SM_Region_Edge_ptr)
    calloc((the_region_ptr->region_edge_list_size), sizeof(SM_Region_Edge));
*/
    for (i=0; i<the_region_ptr->region_edge_list_size; i++) 
     fscanf(the_file,"%d ",&garbage_int); 
    fscanf(the_file,"%c\n",&garbage_char);
    fscanf(the_file,"%d %s\n%c\n",&the_region_ptr->region_tri_list_size,
                                  garbage_string,&garbage_char);
    the_region_ptr->region_tri_list = (SM_Region_Triangle_ptr)
    calloc((the_region_ptr->region_tri_list_size), sizeof(SM_Region_Triangle));
    for (i=0; i<the_region_ptr->region_tri_list_size; i++) {
     fscanf(the_file,"%d ",&the_region_ptr->region_tri_list[i]); 
     SM_triangle_region_add(the_surface, the_region_ptr->region_tri_list[i], the_index);
    }
    fscanf(the_file,"%c\n",&garbage_char);
    fscanf(the_file,"%c\n",&garbage_char);
    SM_edge_picked_push(the_surface);
    compteur = 0;
    SM_FOR_EACH_REGION_TRIANGLE(the_region_ptr)
	SM_triangle_edge_index_get(the_surface, *region_tri_ptr, &e1, &e2, &e3);
	if (! (BOOL) IS_YES(the_surface->edge_list[e1].picked)) {
	    SM_edge_picked_set(the_surface, e1, YES);
	    compteur += 1;
	}
	if (! (BOOL) IS_YES(the_surface->edge_list[e2].picked)) {
	    SM_edge_picked_set(the_surface, e2, YES);
	    compteur += 1;
	}
	if (! (BOOL) IS_YES(the_surface->edge_list[e3].picked)) {
	    SM_edge_picked_set(the_surface, e3, YES);
	    compteur += 1;
	}
    SM_END_FOR_EACH
    the_region_ptr->region_edge_list = (SM_Region_Edge_ptr)malloc(compteur * sizeof(SM_Region_Edge));
    the_region_ptr->region_edge_list_size = 0;
    SM_FOR_EACH_EDGE(the_surface)
	if ((BOOL) IS_YES(edge_ptr->picked)) {
	     the_region_ptr->region_edge_list[the_region_ptr->region_edge_list_size] = edge_index;
	     the_region_ptr->region_edge_list_size += 1;
	}
    SM_END_FOR_EACH
    SM_edge_picked_pop(the_surface);
}

void
SM_region_list_read_file(the_region_list,the_surface,the_file)
SM_Region_List_ptr the_region_list;
SM_Surface_ptr the_surface;
FILE *the_file;
{
    SM_Region_Name name_string;
    SM_Region_ptr R_ptr;
    int i;

    if (the_region_list != NULL) {
      fscanf(the_file,"%s\n",name_string);
      fscanf(the_file,"%d\n",&(the_region_list->region_list_size));
      the_region_list->region_list = (SM_Region_ptr)
         calloc( (the_region_list->region_list_size), sizeof(SM_Region)); 
      R_ptr = the_region_list->region_list;
      for (i=0,R_ptr=the_region_list->region_list; 
           i<the_region_list->region_list_size; i++,R_ptr++)
        R_ptr->struct_used = YES;
      SM_FOR_EACH_REGION(the_region_list)
       SM_region_read_file(the_region_list,region_index,the_surface,the_file); 
      SM_END_FOR_EACH
     }
}

BOOL
SM_region_list_read(the_region_list, the_surface, the_file_name)
SM_Region_List_ptr the_region_list;
SM_Surface_ptr the_surface;
char *the_file_name;
{
  FILE *the_file; 

  the_surface->the_region_list = (struct SM_Region_List*)the_region_list;
  the_region_list->the_surface = the_surface;    
  if (the_file = fopen(the_file_name, "r")) {
    SM_region_list_read_file(the_region_list, the_surface, the_file);
    fclose(the_file);
    return(YES);
  } else return(NO);
   
}

void
SM_region_write_file(the_region_list,the_index,the_file) 
SM_Region_List_ptr the_region_list;
int the_index;
FILE *the_file;
{
    SM_Region_ptr the_region_ptr;
    SM_Region_Vertex_ptr v_ptr;
    int i;

    the_region_ptr = &the_region_list->region_list[the_index];
    fprintf(the_file,"[\n");
    fprintf(the_file,"%s\n",the_region_ptr->name);
    fprintf(the_file,"%d\n",the_region_ptr->region_id);
    fprintf(the_file,"%d vertices\n{\n",the_region_ptr->region_vertex_list_size);
    v_ptr = the_region_ptr->region_vertex_list;
    for (i=0; i<the_region_ptr->region_vertex_list_size; i++){ 
     fprintf(the_file,"( %d %f %f %f %d )",
                v_ptr->id,
                v_ptr->properties.stiffness,
                v_ptr->properties.young_modulus,
                v_ptr->properties.poisson_ratio,
                v_ptr->anchor); 
            /*    v_ptr->flag); */ 
     v_ptr++;
    }
    fprintf(the_file,"}\n");
    fprintf(the_file,"%d edges\n{\n",the_region_ptr->region_edge_list_size);
    for (i=0; i<the_region_ptr->region_edge_list_size; i++) 
     fprintf(the_file,"%d ",the_region_ptr->region_edge_list[i]); 
    fprintf(the_file,"}\n");
    fprintf(the_file,"%d triangles\n{\n",the_region_ptr->region_tri_list_size);
    for (i=0; i<the_region_ptr->region_tri_list_size; i++) 
     fprintf(the_file,"%d ",the_region_ptr->region_tri_list[i]); 
    fprintf(the_file,"}\n");
    fprintf(the_file,"]\n");
}

void
SM_region_list_write_file(the_region_list,the_file)
SM_Region_List_ptr the_region_list;
FILE *the_file;
{
    SM_Region_Name list_name;
    
    strcpy(list_name,"region_list");
    if (the_region_list != NULL) {
      fprintf(the_file,"%s\n",list_name);
      fprintf(the_file,"%d\n",the_region_list->region_list_size);
      SM_FOR_EACH_REGION(the_region_list)
       SM_region_write_file(the_region_list,region_index,the_file); 
      SM_END_FOR_EACH
     }
}
 
BOOL
SM_region_list_write(the_region_list,the_file_name)
SM_Region_List_ptr the_region_list;
char *the_file_name;

{
  FILE *the_file; 
    
  if (the_file = fopen(the_file_name, "w")) {
    SM_region_list_write_file(the_region_list,the_file);
    fclose(the_file);
    return(YES);
  }
  else return(NO);
   
}

BOOL
SM_region_valid_name(the_region_list,the_name)
SM_Region_List_ptr the_region_list;
SM_Region_Name the_name;

{
   SM_FOR_EACH_REGION(the_region_list)
     if (strcmp(the_name,region_ptr->name) == 0) return(YES);
   SM_END_FOR_EACH
   return(NO);
}

BOOL
SM_region_triangle_valid(the_region_list,r_index,t_index)
SM_Region_List_ptr the_region_list;
int r_index,t_index;

{
       SM_Region_ptr r_ptr;

       r_ptr = &the_region_list->region_list[r_index];
       SM_FOR_EACH_REGION_TRIANGLE(r_ptr)
        if (t_index == *region_tri_ptr) 
          return(YES); 
       SM_END_FOR_EACH

       return(NO);
}


int
SM_region_add(the_region_list,the_name)
SM_Region_List_ptr the_region_list;
SM_Region_Name the_name;
/* 
** To add the region in the list of the regions 
*/
{
   SM_Region_ptr new_region, new_region_list, ptr1, ptr2;
   int i, new_region_index;
   
   /* check if the name is a valid name i.e. already exists */
   if (SM_region_valid_name(the_region_list,the_name))
     return(SM_INDEX_ERROR);

   new_region_index = 0;
    while (new_region_index < the_region_list->region_list_size &&
             the_region_list->region_list[new_region_index].struct_used)
          new_region_index++;

    new_region_list = (SM_Region_ptr) 
                       calloc ( (the_region_list->region_list_size 
                                 + 1), sizeof(SM_Region));
   
    if (new_region_index != 0) {
      ptr1 = the_region_list->region_list;
      ptr2 = new_region_list;
      for (i=1; i<=the_region_list->region_list_size; i++)
        *ptr2++ = *ptr1++;
      free(the_region_list->region_list);
     }
     the_region_list->region_list_size += 1;
     the_region_list->region_list = new_region_list; 
     new_region = &the_region_list->region_list[new_region_index];
 
     /*
     ** Initializes the new region 
     */

     new_region->struct_used = YES;
     strcpy(new_region->name,the_name);
     new_region->region_id = new_region_index;
       
     /*
     ** Associate the vertices, edges, and the triangles
     */
     new_region->region_vertex_list_size = 0;
     new_region->region_vertex_list = NULL;
     new_region->region_edge_list_size = 0;
     new_region->region_edge_list = NULL;
     new_region->region_tri_list_size = 0;
     new_region->region_tri_list = NULL;
     
    return(new_region_index);
}

int
SM_region_index_get(the_region_list,the_name)
SM_Region_List_ptr the_region_list;
SM_Region_Name the_name;

{
  SM_FOR_EACH_REGION(the_region_list)
   if (strcmp(region_ptr->name,the_name) == 0)
      return(region_ptr->region_id);
  SM_END_FOR_EACH

  return(SM_INDEX_ERROR);
}

BOOL
SM_region_name_get(the_region_list,the_index,the_name) 
SM_Region_List_ptr the_region_list;
int the_index;
SM_Region_Name the_name;

{
   SM_Region_ptr the_region_ptr;

   the_region_ptr = &the_region_list->region_list[the_index];  
   strcpy(the_name,the_region_ptr->name);
   return(YES);
}

BOOL
SM_region_dispose(the_region_list,the_name, the_surface) 
SM_Region_List_ptr the_region_list;
SM_Region_Name the_name;
SM_Surface_ptr the_surface;
{

   int i, the_index, new_region_list_size;
   SM_Region_ptr the_region, new_region_list, ptr1, ptr2;

   /* check for the valid name */
   if (!SM_region_valid_name(the_region_list,the_name))
     return(NO);
   
   the_index = SM_region_index_get(the_region_list,the_name);
   if (the_index == SM_INDEX_ERROR)
     return(NO);

   /* 
   ** De-associate the vertices, edges and the triangles
   */
   SM_region_remove_ref(the_region_list,the_index, the_surface);

    new_region_list_size = the_region_list->region_list_size - 1;
    if (new_region_list_size == 0) {
      the_region_list->region_list_size = 0;
      the_region_list->region_list = NULL;
      return(YES);
    }
    new_region_list = (SM_Region_ptr) 
                       calloc ( (the_region_list->region_list_size)-1, 
                                 sizeof(SM_Region));
    the_region = &the_region_list->region_list[the_index];
    the_region->struct_used = NO;
    ptr1 = the_region_list->region_list;
    ptr2 = new_region_list;
    
    for (i=1; i<=the_region_list->region_list_size; i++){
      if (ptr1->struct_used) {
       *ptr2 = *ptr1;
       ptr1++; ptr2++;
      }
      else
        ptr1++;
    }
    free(the_region_list->region_list);
    the_region_list->region_list = new_region_list; 
    the_region_list->region_list_size = new_region_list_size; 
 
    ptr2 = new_region_list;
    /* update the index */
    for (i=0; i<the_region_list->region_list_size; i++){
     ptr2->region_id = i;
     ptr2++;
    }
       
    return(YES);
}
 
BOOL
SM_region_check_eye(the_region_name)
SM_Region_Name the_region_name;
{
      if ( (strcmp(the_region_name,"left_eye") == 0) || 
           (strcmp(the_region_name,"right_eye") == 0) ) 
       return(YES);
      else
       return(NO);
}

BOOL
SM_region_set_eye(the_surface,index)
SM_Surface_ptr the_surface;
int index;
{
  int e1,e2,e3,n1,n2,n3;
  BOOL bool_val; 

    if (!SM_triangle_edge_index_get(the_surface,index,&e1,&e2,&e3))
     return(NO);
    SM_triangle_picked_set(the_surface,index,YES);
    n1 = SM_triangle_neighbor_get(the_surface,index,e1);
    n2 = SM_triangle_neighbor_get(the_surface,index,e2);
    n3 = SM_triangle_neighbor_get(the_surface,index,e3);
    if ( (SM_triangle_picked_get(the_surface,n1,&bool_val) && bool_val) && 
         (SM_triangle_picked_get(the_surface,n2,&bool_val) && bool_val) &&
         (SM_triangle_picked_get(the_surface,n3,&bool_val) && bool_val)) 
    return(YES);

    if (!(SM_triangle_picked_get(the_surface,n1,&bool_val) && bool_val)) 
      SM_region_set_eye(the_surface,n1); 
    if (!(SM_triangle_picked_get(the_surface,n2,&bool_val) && bool_val)) 
      SM_region_set_eye(the_surface,n2); 
    if (!(SM_triangle_picked_get(the_surface,n3,&bool_val) && bool_val)) 
     SM_region_set_eye(the_surface,n3); 

    return(YES);
}

BOOL
SM_region_make_eye(the_surface) 
SM_Surface_ptr the_surface;

{
    int index;

      SM_FOR_EACH_TRIANGLE(the_surface)
       if (triangle_ptr->picked) 
           index = triangle_index;
      SM_END_FOR_EACH

      if (SM_region_set_eye(the_surface,index)) return(YES);
      else return(NO);
}

BOOL
SM_region_remove_ref(the_region_list,the_region_index, the_surface) 
SM_Region_List_ptr the_region_list;
int the_region_index;
SM_Surface_ptr the_surface;

{
   SM_Region_ptr the_region;

   the_region = &the_region_list->region_list[the_region_index];
   if (the_region == NULL) return(NO);
   if (the_region->region_vertex_list_size > 0) {
    the_region->region_vertex_list_size = 0;
    free(the_region->region_vertex_list);
   }
   if (the_region->region_edge_list_size > 0) {
    the_region->region_edge_list_size = 0;
    free(the_region->region_edge_list);
   }
   if (the_region->region_tri_list_size > 0) {
     SM_FOR_EACH_REGION_TRIANGLE(the_region)
      SM_triangle_region_rem(the_surface, *region_tri_ptr, the_region_index);
     SM_END_FOR_EACH
    the_region->region_tri_list_size = 0;
    free(the_region->region_tri_list);
   }
   return(YES);
}


BOOL
SM_region_add_ref(the_surface,the_region_list,the_region_index)
SM_Surface_ptr the_surface;
SM_Region_List_ptr the_region_list;
int the_region_index;

{

   SM_Region_ptr the_region;
   SM_Region_Vertex_ptr the_vertex_list;
   SM_Region_Edge_ptr the_edge_list;
   SM_Region_Triangle_ptr the_tri_list;
   int count_tri, count_vert, count_edge, v1, v2, v3, e1, e2, e3;

   the_region = &the_region_list->region_list[the_region_index];
   count_tri = 0;
   count_vert = 0;
   count_edge = 0;
 
   SM_vertex_picked_push(the_surface);
   SM_FOR_EACH_VERTEX(the_surface)
     SM_vertex_picked_set(the_surface,vertex_index,NO);
   SM_END_FOR_EACH

   SM_edge_picked_push(the_surface);
   SM_FOR_EACH_EDGE(the_surface)
     SM_edge_picked_set(the_surface,edge_index,NO);
   SM_END_FOR_EACH

   SM_FOR_EACH_TRIANGLE(the_surface)
    if ((BOOL) IS_YES(triangle_ptr->picked)) {
      count_tri++;
      SM_triangle_vertex_index_get(the_surface,triangle_index,&v1,&v2,&v3);
      SM_vertex_picked_set(the_surface,v1,YES);
      SM_vertex_picked_set(the_surface,v2,YES);
      SM_vertex_picked_set(the_surface,v3,YES);
      SM_triangle_edge_index_get(the_surface,triangle_index,&e1,&e2,&e3);
      SM_edge_picked_set(the_surface,e1,YES);
      SM_edge_picked_set(the_surface,e2,YES);
      SM_edge_picked_set(the_surface,e3,YES);
      SM_triangle_region_add(the_surface, triangle_index, the_region_index);
    }
   SM_END_FOR_EACH 

   if (count_tri == 0) return(NO);
    
   the_tri_list = (SM_Region_Triangle_ptr) 
                   calloc((count_tri), sizeof(SM_Region_Triangle));
   the_region->region_tri_list = the_tri_list;
   the_region->region_tri_list_size = count_tri;
   SM_FOR_EACH_TRIANGLE(the_surface)
    if ((BOOL) IS_YES(triangle_ptr->picked)) {
      *the_tri_list = triangle_index;
      the_tri_list++;
     }
   SM_END_FOR_EACH

   SM_FOR_EACH_VERTEX(the_surface)
    if ((BOOL) IS_YES(vertex_ptr->picked)) 
     count_vert++;
   SM_END_FOR_EACH
   the_vertex_list = (SM_Region_Vertex_ptr) calloc((count_vert), 
                                                  sizeof(SM_Region_Vertex));

   the_region->region_vertex_list = the_vertex_list;
   the_region->region_vertex_list_size = count_vert;
   SM_FOR_EACH_VERTEX(the_surface)
    if ((BOOL) IS_YES(vertex_ptr->picked)) {
      the_vertex_list->id = vertex_index;
      the_vertex_list->properties.stiffness = SM_REGION_DEFAULT_STIFFNESS;
      the_vertex_list->properties.young_modulus = SM_REGION_DEFAULT_YOUNGMOD;
      the_vertex_list->properties.poisson_ratio = SM_REGION_DEFAULT_POISSON;
      the_vertex_list->anchor = NO;
      the_vertex_list->flag = NO;
      the_vertex_list++;
     }
   SM_END_FOR_EACH

   SM_FOR_EACH_EDGE(the_surface)
    if ((BOOL) IS_YES(edge_ptr->picked)) 
     count_edge++;
   SM_END_FOR_EACH
   the_edge_list = (SM_Region_Edge_ptr) calloc((count_edge), 
                                                  sizeof(SM_Region_Edge));

   the_region->region_edge_list = the_edge_list;
   the_region->region_edge_list_size = count_edge;
   SM_FOR_EACH_EDGE(the_surface)
    if ((BOOL) IS_YES(edge_ptr->picked)) {
      *the_edge_list = edge_index;
      the_edge_list++;
     }
   SM_END_FOR_EACH
   SM_vertex_picked_pop(the_surface);
   SM_edge_picked_pop(the_surface);
   return(YES);
}

BOOL
SM_region_modify_ref(the_surface,the_region_list,the_region_index,what)
SM_Surface_ptr the_surface;
SM_Region_List_ptr the_region_list;
int the_region_index;
int what;

{

   SM_Region_ptr the_region;
   SM_Region_Vertex_ptr the_vertex_list, v_ptr;
   SM_Region_Edge_ptr the_edge_list;
   SM_Region_Triangle_ptr the_tri_list, new_tri_list;
   SM_Region_Triangle_ptr ptr1, ptr2, temp_tri_list;
   int count_tri, count_vert, count_edge, v1, v2, v3, e1, e2, e3;
   int i, incr_tri, tri_index, old_count_tri;
   BOOL bool_val;

   if (the_region_index == SM_INDEX_ERROR) return(NO);
   the_region = &the_region_list->region_list[the_region_index];
   count_tri = the_region->region_tri_list_size;
   the_tri_list = the_region->region_tri_list;
   old_count_tri = count_tri;

   count_vert = 0;
   count_edge = 0;
 
   SM_vertex_picked_push(the_surface);
   SM_FOR_EACH_VERTEX(the_surface)
     SM_vertex_picked_set(the_surface,vertex_index,NO);
   SM_END_FOR_EACH

   SM_edge_picked_push(the_surface);
   SM_FOR_EACH_EDGE(the_surface)
     SM_edge_picked_set(the_surface,edge_index,NO);
   SM_END_FOR_EACH

   incr_tri = 0;
   
   SM_FOR_EACH_TRIANGLE(the_surface)
    if ((BOOL) IS_YES(triangle_ptr->picked)) {
    switch (what) {
     case REGION_ADD_REF :
     if (!SM_region_triangle_valid(the_region_list,
               the_region_index,triangle_index)) {
	SM_triangle_region_add(the_surface, triangle_index,the_region_index);
        incr_tri++;
     }
     else
       SM_triangle_picked_set(the_surface,triangle_index,NO);
     break;
     case REGION_REM_REF :
     if (SM_region_triangle_valid(the_region_list,
               the_region_index,triangle_index)) {
	SM_triangle_region_rem(the_surface, triangle_index, the_region_index);
        incr_tri++;
     }
     else
       SM_triangle_picked_set(the_surface,triangle_index,NO);
     break;
     default :
     break;
    }
    }
   SM_END_FOR_EACH 

   if (incr_tri == 0) return(NO);

   temp_tri_list = (SM_Region_Triangle_ptr) 
                   calloc((incr_tri), sizeof(SM_Region_Triangle));
   ptr1 = temp_tri_list;
   SM_FOR_EACH_TRIANGLE(the_surface)
    if ((BOOL) IS_YES(triangle_ptr->picked)) {
      *ptr1 = triangle_index;
       ptr1++;
    }
   SM_END_FOR_EACH 

   switch (what) {
     case REGION_ADD_REF :
        count_tri += incr_tri;
     break;
     case REGION_REM_REF :
        count_tri -= incr_tri;
     break;
     default :
     break;
   }

   new_tri_list = (SM_Region_Triangle_ptr) 
                   calloc((count_tri), sizeof(SM_Region_Triangle));
   ptr1 = new_tri_list;
   ptr2 = the_tri_list;
   if (what == REGION_ADD_REF) {
    for (i=0; i<old_count_tri; i++) {
     *ptr1 = *ptr2;
      ptr1++; ptr2++;
    } 
    ptr2 = temp_tri_list;
    for (i=0; i<incr_tri; i++) {
      *ptr1 = *ptr2;
      ptr1++; ptr2++;
    } 
   }

   ptr1 = new_tri_list;
   ptr2 = the_tri_list;
   if (what == REGION_REM_REF) {
   for (i=0; i<old_count_tri; i++) {
    tri_index = *ptr2;
    if (SM_triangle_picked_get(the_surface,tri_index,&bool_val) && bool_val) 
      ptr2++;
    else {
      *ptr1 = *ptr2;
      ptr1++; ptr2++;
     } 
    }
   }

   free(the_region->region_tri_list);
   if (incr_tri > 0) 
    free(temp_tri_list);

   the_region->region_tri_list = new_tri_list;
   the_region->region_tri_list_size = count_tri;
   ptr1 = new_tri_list;
   for (i=0; i<count_tri; i++) {
     tri_index = *ptr1;
     SM_triangle_vertex_index_get(the_surface,tri_index,&v1,&v2,&v3);
     SM_vertex_picked_set(the_surface,v1,YES);
     SM_vertex_picked_set(the_surface,v2,YES);
     SM_vertex_picked_set(the_surface,v3,YES);
     SM_triangle_edge_index_get(the_surface,tri_index,&e1,&e2,&e3);
     SM_edge_picked_set(the_surface,e1,YES);
     SM_edge_picked_set(the_surface,e2,YES);
     SM_edge_picked_set(the_surface,e3,YES);
     ptr1++;
   }

   SM_FOR_EACH_VERTEX(the_surface)
    if ((BOOL) IS_YES(vertex_ptr->picked)) 
     count_vert++;
   SM_END_FOR_EACH
   the_vertex_list = (SM_Region_Vertex_ptr) calloc((count_vert), 
                                                  sizeof(SM_Region_Vertex));

   v_ptr = the_vertex_list;
   SM_FOR_EACH_VERTEX(the_surface)
    if (vertex_ptr->picked) {
      v_ptr->id = vertex_index;
      v_ptr->properties.stiffness = SM_REGION_DEFAULT_STIFFNESS;
      v_ptr->properties.young_modulus = SM_REGION_DEFAULT_YOUNGMOD;
      v_ptr->properties.poisson_ratio = SM_REGION_DEFAULT_POISSON;
      v_ptr->anchor = NO;
      v_ptr->flag = NO;
      SM_FOR_EACH_REGION_VERTEX(the_region)
       if (region_vertex_ptr->id == v_ptr->id) {
        v_ptr->properties.stiffness = region_vertex_ptr->properties.stiffness;
        v_ptr->properties.young_modulus = region_vertex_ptr->properties.young_modulus;
        v_ptr->properties.poisson_ratio = region_vertex_ptr->properties.poisson_ratio;
        v_ptr->anchor = region_vertex_ptr->anchor;
        v_ptr->flag = region_vertex_ptr->flag;
       }
      SM_END_FOR_EACH
     v_ptr++;
     }
   SM_END_FOR_EACH

   free(the_region->region_vertex_list);
   the_region->region_vertex_list = the_vertex_list;
   the_region->region_vertex_list_size = count_vert;


   SM_FOR_EACH_EDGE(the_surface)
    if ((BOOL) IS_YES(edge_ptr->picked)) 
     count_edge++;
   SM_END_FOR_EACH
   the_edge_list = (SM_Region_Edge_ptr) calloc((count_edge), 
                                                  sizeof(SM_Region_Edge));
   free(the_region->region_edge_list);
   the_region->region_edge_list = the_edge_list;
   the_region->region_edge_list_size = count_edge;

   SM_FOR_EACH_EDGE(the_surface)
    if ((BOOL) IS_YES(edge_ptr->picked)) {
      *the_edge_list = edge_index;
      the_edge_list++;
     }
   SM_END_FOR_EACH
   SM_vertex_picked_pop(the_surface);
   SM_edge_picked_pop(the_surface);
   return(YES);
}

void
SM_region_copy(the_surface,the_region_list,region1,region2)
SM_Surface_ptr the_surface;
SM_Region_List_ptr the_region_list;
int region1,region2;
{
     SM_Region_ptr r1,r2;
     SM_Region_Vertex_ptr the_vertex_list;
     SM_Region_Edge_ptr the_edge_list;
     SM_Region_Triangle_ptr the_tri_list;
     int count_tri, count_vert, count_edge;

     r1 = &the_region_list->region_list[region1];
     r2 = &the_region_list->region_list[region2];
     count_tri = r1->region_tri_list_size;
     the_tri_list = (SM_Region_Triangle_ptr) 
                   calloc((count_tri), sizeof(SM_Region_Triangle));
     r2->region_tri_list = the_tri_list;
     r2->region_tri_list_size = count_tri;
     SM_FOR_EACH_REGION_TRIANGLE(r1)
      *the_tri_list = *region_tri_ptr;
      SM_triangle_region_add(the_surface, *region_tri_ptr, region2);
      the_tri_list++;
     SM_END_FOR_EACH

     count_vert = r1->region_vertex_list_size;
     the_vertex_list = (SM_Region_Vertex_ptr) 
                   calloc((count_vert), sizeof(SM_Region_Vertex));
     r2->region_vertex_list = the_vertex_list;
     r2->region_vertex_list_size = count_vert;
     SM_FOR_EACH_REGION_VERTEX(r1)
      *the_vertex_list = *region_vertex_ptr;
      the_vertex_list++;
     SM_END_FOR_EACH

     count_edge = r1->region_edge_list_size;
     the_edge_list = (SM_Region_Edge_ptr) 
                   calloc((count_edge), sizeof(SM_Region_Edge));
     r2->region_edge_list = the_edge_list;
     r2->region_edge_list_size = count_edge;
     SM_FOR_EACH_REGION_EDGE(r1)
      *the_edge_list = *region_edge_ptr;
      the_edge_list++;
     SM_END_FOR_EACH

}

BOOL
Find_inside_triangle(the_surface,t_index,the_point)
SM_Surface_ptr the_surface;
int t_index;
VECTOR_3D the_point;
{
   VECTOR_3D p1,p2,p3,vect1,vect2,res;
   int v1,v2,v3;
   float u,v,w,sum;
   float Au,Av,Aw,A;

   SM_triangle_vertex_index_get(the_surface, t_index, &v1, &v2, &v3); 
   SM_vertex_point_get(the_surface, v1, p1);
   SM_vertex_point_get(the_surface, v2, p2);
   SM_vertex_point_get(the_surface, v3, p3);
   /* area A */
   V3D_SUB(vect1,p2,p1);
   V3D_SUB(vect2,p3,p1);
   V3D_CROSS(res,vect1,vect2);
   A = V3D_NORM(res);    
   /* area Au */
   V3D_SUB(vect2,the_point,p1);
   V3D_CROSS(res,vect1,vect2);
   Au = V3D_NORM(res);    
   /* area Av */ 
   V3D_SUB(vect1,p3,p1);
   V3D_CROSS(res,vect1,vect2);
   Av = V3D_NORM(res);    
   /* area Aw */
   V3D_SUB(vect1,p3,p2);
   V3D_SUB(vect2,the_point,p2);
   V3D_CROSS(res,vect1,vect2);
   Aw = V3D_NORM(res);    

   if (A < SM_EPSILON) return(NO);
   u = Au/A; v = Av/A; w = Aw/A;

   sum = u + v + w;
   if (fabs(sum-1.0) < SM_EPSILON) {
   if (  ((u >= 0) && (u <= 1))
      && ((v >= 0) && (v <= 1))
      && ((w >= 0) && (w <= 1)) ) {
      return(YES);
    }
   }
   return(NO);
}


/*
 *  To obtain a region flipped from another. The normal for the symmetry plane
 *  shold have the right direction. i.e. out of the two choices of normals outwards
 *  or inwards and should be normalized.
 * 
 */
 
 

BOOL
SM_region_flip(the_surface,the_region_list,region1,region2,plane_norm, plane_d)
SM_Surface_ptr the_surface;
SM_Region_List_ptr the_region_list;
int region1,region2;
VECTOR_3D plane_norm;
float plane_d;
{
     SM_Region_ptr r1;
     int count_tri;
     VECTOR_3D center_source, center_target, 
               normal, point, ray_o, ray_d, dir, the_point;
     int index1, index2, index3, t_index;
     float d, t, t_deno, t_nume, d1_perp, d2_perp, dot_product;
      
     r1 = &the_region_list->region_list[region1];
     count_tri = r1->region_tri_list_size;
     
     count_tri = 0;
     SM_FOR_EACH_REGION_TRIANGLE(r1)
      t_index = *region_tri_ptr;
      SM_triangle_angular_bisector_get(the_surface, t_index, center_source);
      V3D_COPY(ray_o, center_source);
      V3D_COPY(ray_d, plane_norm);
      dot_product = V3D_DOT(plane_norm, center_source);
      if (dot_product > 0.0) { 
       fprintf(stderr, "could not flip check your inputs\n"); 
       return(NO); 
      }
      d1_perp =  fabs(dot_product) + plane_d;
      SM_FOR_EACH_TRIANGLE(the_surface) 
       if (!triangle_ptr->picked) {
        SM_triangle_angular_bisector_get(the_surface, triangle_index, center_target);
	dot_product = V3D_DOT(plane_norm, center_target);
	d2_perp =  dot_product + plane_d;
	if ( (dot_product >= 0.0) && (d2_perp >= 0.0) ) {
	    SM_triangle_normal_get(the_surface, triangle_index, normal);
	    SM_triangle_vertex_index_get(the_surface, triangle_index, 
                                         &index1, &index2, &index3);
	    SM_vertex_point_get(the_surface, index1, point);
	    d = - V3D_DOT(normal, point);
	    t_deno = V3D_DOT(normal, ray_d);
	    t_nume = V3D_DOT(normal, ray_o);
	    if (fabs(t_deno) >= SM_EPSILON) {
	       t_nume = - (t_nume + d);
	       t = t_nume/t_deno;
	       if (t > 0.0) {
	         V3D_SCALE(dir,t,ray_d);
                 V3D_ADD(the_point,ray_o,dir);
	         dot_product = V3D_DOT(plane_norm, the_point);
	         d2_perp =  dot_product + plane_d;
                if  (   (Find_inside_triangle(the_surface,triangle_index,the_point))
		        && (fabs(d1_perp - d2_perp) <= 0.001)  ) {		   		   
	             triangle_ptr->picked = YES;
                     count_tri++;
		     /*
		     fprintf(stderr, "triangle_index %d\n", triangle_index);
		     */
		     if (count_tri >= r1->region_tri_list_size) {
                      SM_region_add_ref(the_surface,the_region_list,region2);
		      return(YES);
		     }
		   }
	       }
           }
	}
       }	
      SM_END_FOR_EACH
      SM_END_FOR_EACH
      
      SM_region_add_ref(the_surface,the_region_list,region2);
      return(YES);
}

 /*   version with inverted looping
  * 
  */
  
  
  /*
BOOL
SM_region_flip(the_surface,the_region_list,region1,region2,plane_norm, plane_d)
SM_Surface_ptr the_surface;
SM_Region_List_ptr the_region_list;
int region1,region2;
VECTOR_3D plane_norm;
float plane_d;
{
     SM_Region_ptr r1;
     int count_tri;
     VECTOR_3D center_source, center_target, 
               normal, point, ray_o, ray_d, dir, the_point;
     int index1, index2, index3, t_index;
     float d, t, t_deno, t_nume, d1_perp, d2_perp, dot_product, d_diff;
      
     r1 = &the_region_list->region_list[region1];
     count_tri = r1->region_tri_list_size;
     
     count_tri = 0;
     SM_FOR_EACH_TRIANGLE(the_surface) 
       if (!triangle_ptr->picked) {
        SM_triangle_angular_bisector_get(the_surface, triangle_index, center_source);
	dot_product = V3D_DOT(plane_norm, center_source);
	d1_perp =  dot_product + plane_d;
	
	if ( (dot_product >= 0.0) && (d1_perp >= 0.0) ) {	
          V3D_COPY(ray_o, center_source);
          V3D_COPY(ray_d, plane_norm);
	  V3D_INVERT(ray_d, ray_d);
	  
          SM_FOR_EACH_REGION_TRIANGLE(r1)
            t_index = *region_tri_ptr;
            SM_triangle_angular_bisector_get(the_surface, t_index, center_target);		
	    SM_triangle_normal_get(the_surface, t_index, normal);
	    SM_triangle_vertex_index_get(the_surface, t_index, 
                                         &index1, &index2, &index3);
	    SM_vertex_point_get(the_surface, index1, point);
	    d = - V3D_DOT(normal, point);
	    t_deno = V3D_DOT(normal, ray_d);
	    t_nume = V3D_DOT(normal, ray_o);
	    
	    if (fabs(t_deno) >= SM_EPSILON) {
	       t_nume = - (t_nume + d);
	       t = t_nume/t_deno;
	       
	       if (t > 0.0) {
	         V3D_SCALE(dir,t,ray_d);
                 V3D_ADD(the_point,ray_o,dir);
	         dot_product = V3D_DOT(plane_norm, the_point);
		 
		 if (dot_product <= 0.0) {
	          d2_perp =  fabs(dot_product) + plane_d;
		  printf(" ....\n");
                   if  (   (Find_inside_triangle(the_surface,t_index,the_point))
		        && (fabs(d1_perp - d2_perp) <= SM_EPSILON)  ) {		   		   
	             triangle_ptr->picked = YES;
                     count_tri++;
		     fprintf(stderr, "triangle_index %d\n", triangle_index);
		     
		     if (count_tri >= r1->region_tri_list_size) {
                      SM_region_add_ref(the_surface,the_region_list,region2);
		      return(YES);
		     }
		     
		   }
		   
		 }
		 
	       }
            }
            SM_END_FOR_EACH
         }
       }
      SM_END_FOR_EACH
      
      SM_region_add_ref(the_surface,the_region_list,region2);
      return(YES);
}

*/


void
SM_region_merge(the_surface,the_region_list,region1,region2,region)
SM_Surface_ptr the_surface;
SM_Region_List_ptr the_region_list;
int region1,region2,region;
{
     SM_Region_ptr r1,r2,r;
     SM_Region_Vertex_ptr the_vertex_list;
     SM_Region_Edge_ptr the_edge_list;
     SM_Region_Triangle_ptr the_tri_list;
     int count_tri, count_vert, count_edge;

     r1 = &the_region_list->region_list[region1];
     r2 = &the_region_list->region_list[region2];
     r = &the_region_list->region_list[region];
     count_tri = r1->region_tri_list_size + r2->region_tri_list_size;
     the_tri_list = (SM_Region_Triangle_ptr) 
                   calloc((count_tri), sizeof(SM_Region_Triangle));
     r->region_tri_list = the_tri_list;
     r->region_tri_list_size = count_tri;
     SM_FOR_EACH_REGION_TRIANGLE(r1)
      *the_tri_list = *region_tri_ptr;
      SM_triangle_region_add(the_surface, *region_tri_ptr, region);
      the_tri_list++;
     SM_END_FOR_EACH
     SM_FOR_EACH_REGION_TRIANGLE(r2)
      *the_tri_list = *region_tri_ptr;
       SM_triangle_region_add(the_surface, *region_tri_ptr, region);
     the_tri_list++;
     SM_END_FOR_EACH

     count_vert = r1->region_vertex_list_size + r2->region_vertex_list_size;
     the_vertex_list = (SM_Region_Vertex_ptr) 
                   calloc((count_vert), sizeof(SM_Region_Vertex));
     r->region_vertex_list = the_vertex_list;
     r->region_vertex_list_size = count_vert;
     SM_FOR_EACH_REGION_VERTEX(r1)
      *the_vertex_list = *region_vertex_ptr;
      the_vertex_list++;
     SM_END_FOR_EACH
     SM_FOR_EACH_REGION_VERTEX(r2)
      *the_vertex_list = *region_vertex_ptr;
      the_vertex_list++;
     SM_END_FOR_EACH

     count_edge = r1->region_edge_list_size + r2->region_edge_list_size;
     the_edge_list = (SM_Region_Edge_ptr) 
                   calloc((count_edge), sizeof(SM_Region_Edge));
     r->region_edge_list = the_edge_list;
     r->region_edge_list_size = count_edge;
     SM_FOR_EACH_REGION_EDGE(r1)
      *the_edge_list = *region_edge_ptr;
      the_edge_list++;
     SM_END_FOR_EACH
     SM_FOR_EACH_REGION_EDGE(r2)
      *the_edge_list = *region_edge_ptr;
      the_edge_list++;
     SM_END_FOR_EACH

}

void
SM_region_vertex_anchor_set(the_region_list,the_index,vert_index,value)
SM_Region_List_ptr the_region_list;
int the_index;
int vert_index;
BOOL value;

{
   SM_Region_ptr the_region_ptr;

   the_region_ptr = &the_region_list->region_list[the_index];
   SM_FOR_EACH_REGION_VERTEX(the_region_ptr)
     if (region_vertex_ptr->id == vert_index)
       region_vertex_ptr->anchor = value;
   SM_END_FOR_EACH

}

void
SM_region_vertex_anchor_get(the_region_list,the_index,vert_index,value)
SM_Region_List_ptr the_region_list;
int the_index;
int vert_index;
BOOL *value;

{
   SM_Region_ptr the_region_ptr;

   the_region_ptr = &the_region_list->region_list[the_index];
   SM_FOR_EACH_REGION_VERTEX(the_region_ptr)
     if (region_vertex_ptr->id == vert_index)
       *value = region_vertex_ptr->anchor;
   SM_END_FOR_EACH

}
 

void
SM_region_vertex_property_set(the_region_list,the_index,
                              vert_index,the_property,value)
SM_Region_List_ptr the_region_list;
int the_index;
int vert_index;
int the_property;
float value;
{
   SM_Region_ptr the_region_ptr;

   the_region_ptr = &the_region_list->region_list[the_index];
   SM_FOR_EACH_REGION_VERTEX(the_region_ptr)
   if (region_vertex_ptr->id == vert_index) {
     switch (the_property) {
      case 0 : 
             region_vertex_ptr->properties.stiffness = value;
             break;
      case 1 : 
             region_vertex_ptr->properties.young_modulus = value;
             break;
      case 2 : 
             region_vertex_ptr->properties.poisson_ratio = value;
             break;
      default :
             break;
     }
    }
   SM_END_FOR_EACH
}

void
SM_region_vertex_property_get(the_region_list,the_index,vert_index,the_property,value)
SM_Region_List_ptr the_region_list;
int the_index,vert_index,the_property;
float *value;

{
   SM_Region_ptr the_region_ptr;

   the_region_ptr = &the_region_list->region_list[the_index];
   SM_FOR_EACH_REGION_VERTEX(the_region_ptr)
   if (region_vertex_ptr->id == vert_index) {
    switch (the_property) {
      case 0 : /* the stiffness */
             *value = region_vertex_ptr->properties.stiffness;
             break;
      case 1 : /* the young modulus */
             *value = region_vertex_ptr->properties.young_modulus;
             break;
      case 2 : /* the poisson ratio */
             *value = region_vertex_ptr->properties.poisson_ratio;
             break;
      default :
             break;
    }
   }
   SM_END_FOR_EACH
}

BOOL
SM_region_center_get(the_surface,the_region_list,the_index,center) 
SM_Surface_ptr the_surface;
SM_Region_List_ptr the_region_list;
int the_index;
VECTOR_3D center;

{
      SM_Region_ptr the_region_ptr;
      VECTOR_3D the_point;
      int v_index,count;

      V3D_NULL(center);
      count = 0;
      the_region_ptr = &the_region_list->region_list[the_index];
      SM_FOR_EACH_REGION_VERTEX(the_region_ptr)
       v_index = region_vertex_ptr->id; 
       if (SM_vertex_index_valid(the_surface,v_index)) {
        SM_vertex_point_get(the_surface,v_index,the_point);     
        V3D_ADD(center,center,the_point);  
        count++;
       }
      SM_END_FOR_EACH
      if (count == 0) return(NO);
      V3D_SCALE(center,1.0/(float) count, center);
      return(YES);
}
 
void
SM_region_maxcoord_index_get(the_surface,the_region_list,the_index,axis_flag,index)
SM_Surface_ptr the_surface;
SM_Region_List_ptr the_region_list;
int the_index,axis_flag;
int *index;
{
     SM_Region_ptr the_region_ptr;
     float max_value;
     VECTOR_3D the_point;
     int v_index;
     
     the_region_ptr = &the_region_list->region_list[the_index];
     max_value = SM_REGION_MIN_COORD;
     *index = SM_INDEX_ERROR;
     SM_FOR_EACH_REGION_VERTEX(the_region_ptr)
       v_index = region_vertex_ptr->id;
       if (SM_vertex_index_valid(the_surface,v_index)) {
         SM_vertex_point_get(the_surface,v_index,the_point);
         if (max_value > the_point[axis_flag]) 
            max_value = max_value;
         else {
             max_value = the_point[axis_flag];
             *index = v_index;
         }
       }
     SM_END_FOR_EACH
}

void
SM_region_mincoord_index_get(the_surface,the_region_list,the_index,axis_flag,index)
SM_Surface_ptr the_surface;
SM_Region_List_ptr the_region_list;
int the_index,axis_flag;
int *index;
{
     SM_Region_ptr the_region_ptr;
     float min_value;
     VECTOR_3D the_point;
     int v_index;
     
     the_region_ptr = &the_region_list->region_list[the_index];
     min_value = SM_REGION_MAX_COORD;
     *index = SM_INDEX_ERROR;
     SM_FOR_EACH_REGION_VERTEX(the_region_ptr)
       v_index = region_vertex_ptr->id;
       if (SM_vertex_index_valid(the_surface,v_index)) {
         SM_vertex_point_get(the_surface,v_index,the_point);
         if (min_value < the_point[axis_flag]) 
            min_value = min_value;
         else {
             min_value = the_point[axis_flag];
             *index = v_index;
         }
       }
     SM_END_FOR_EACH
}

void
SM_region_maxcoord_get(the_surface,the_region_list,the_index,axis_flag,value)
SM_Surface_ptr the_surface;
SM_Region_List_ptr the_region_list;
int the_index,axis_flag;
float *value;
{
     SM_Region_ptr the_region_ptr;
     float max_value;
     VECTOR_3D the_point;
     int v_index;
     
     the_region_ptr = &the_region_list->region_list[the_index];
     max_value = SM_REGION_MIN_COORD;
     SM_FOR_EACH_REGION_VERTEX(the_region_ptr)
       v_index = region_vertex_ptr->id;
       if (SM_vertex_index_valid(the_surface,v_index)) {
         SM_vertex_point_get(the_surface,v_index,the_point);
         max_value = (max_value > the_point[axis_flag]) ? 
                      max_value : the_point[axis_flag];
       }
     SM_END_FOR_EACH
     *value = max_value;
}


void
SM_region_mincoord_get(the_surface,the_region_list,the_index,axis_flag,value)
SM_Surface_ptr the_surface;
SM_Region_List_ptr the_region_list;
int the_index,axis_flag;
float *value;
{
     SM_Region_ptr the_region_ptr;
     float min_value;
     VECTOR_3D the_point;
     int v_index;
     
     the_region_ptr = &the_region_list->region_list[the_index];
     min_value = SM_REGION_MAX_COORD;
     SM_FOR_EACH_REGION_VERTEX(the_region_ptr)
       v_index = region_vertex_ptr->id;
       if (SM_vertex_index_valid(the_surface,v_index)) {
         SM_vertex_point_get(the_surface,v_index,the_point);
         min_value = (min_value < the_point[axis_flag]) ? 
                      min_value : the_point[axis_flag];
       }
     SM_END_FOR_EACH
     *value = min_value;
}

void
SM_region_controlcenter_get(max_point,min_point,point) 
VECTOR_3D max_point, min_point;
VECTOR_3D point;

{
       int i;

       for(i=X_INDEX; i<=Z_INDEX; i++) 
        point[i] = (max_point[i] + min_point[i])/2.0;
}

void
SM_region_controlradius_get(max_point,min_point,radius)
VECTOR_3D max_point, min_point;
float *radius;

{
       float dx, dy, dz;

       dx = max_point[X_INDEX] - min_point[X_INDEX];
       dy = max_point[Y_INDEX] - min_point[Y_INDEX];
       dz = max_point[Z_INDEX] - min_point[Z_INDEX];

       *radius = sqrt(dx * dx + dy * dy + dz * dz)/2.0 
                 + SM_REGION_SPHERE_MARGIN; 
      
}


void
SM_region_controlaxis_get(max_point,min_point,axis)
VECTOR_3D max_point, min_point;
int *axis;
{
    
       float dx, dy, dz;

       dx = max_point[X_INDEX] - min_point[X_INDEX];
       dy = max_point[Y_INDEX] - min_point[Y_INDEX];
       dz = max_point[Z_INDEX] - min_point[Z_INDEX];

       if (dx > dy) {
         if (dx > dz) 
           *axis = SM_REGION_AXIS_X;
         else
           *axis = SM_REGION_AXIS_Z;
       }
       else {
         if (dy > dz)
           *axis = SM_REGION_AXIS_Y;
         else 
           *axis = SM_REGION_AXIS_Z;
       }
}

SM_Region_Controlbox_ptr
SM_region_controlbox_new(l,m,n)
int l,m,n;
{
    SM_Region_Controlbox_ptr the_controlbox;
 
    the_controlbox = (SM_Region_Controlbox_ptr) 
                      calloc(1,sizeof(SM_Region_Controlbox));
    the_controlbox->struct_used = YES;
    the_controlbox->l = l;
    the_controlbox->m = m;
    the_controlbox->n = n;
    the_controlbox->vertex_list_size = 0;
    the_controlbox->vertex_list = NULL;

    return(the_controlbox);
}

BOOL
SM_region_controlbox_valid(the_box)
SM_Region_Controlbox_ptr the_box;
{
      if (the_box == NULL) return(NO);
      if (the_box->struct_used) return(YES);
      return(NO);
}

BOOL
SM_region_controlbox_vertexlist_set(the_box)   
SM_Region_Controlbox_ptr the_box;
{
       int number_of_points;

       if (!SM_region_controlbox_valid(the_box)) return(NO);
       number_of_points = (the_box->l) * (the_box->m) * (the_box->n);
       the_box->vertex_list_size = number_of_points;
       the_box->vertex_list = (SM_Region_Controlbox_Vertex_ptr)        
         calloc(number_of_points, sizeof(SM_Region_Controlbox_Vertex));
       return(YES);
}

void
SM_region_controlbox_vertexlist_dispose(the_box)   
SM_Region_Controlbox_ptr the_box;
{

          if (the_box->vertex_list != NULL) {
           free(the_box->vertex_list);
           the_box->vertex_list_size = 0;
           the_box->vertex_list = NULL;
          }
}

void
SM_region_controlbox_dispose(the_box)
SM_Region_Controlbox_ptr the_box;
{
       if (SM_region_controlbox_valid(the_box)) { 
        SM_region_controlbox_vertexlist_dispose(the_box);
        free(the_box);
       }
}

void
SM_region_controlbox_create(the_box,l,m,n)
SM_Region_Controlbox_ptr the_box;
int l,m,n;
{
  int L, M, N;
  
       L = the_box->l;
       M = the_box->m;
       N = the_box->n;
       the_box->l = l;
       the_box->m = m;
       the_box->n = n;
       if (!SM_region_controlbox_vertexlist_set(the_box)) {
        the_box->l = L;
        the_box->m = M;
        the_box->n = N;
      }  
}

void
SM_region_controlbox_recreate(the_box,l,m,n)
SM_Region_Controlbox_ptr the_box;
int l,m,n;
{
       SM_region_controlbox_vertexlist_dispose(the_box);
       SM_region_controlbox_create(the_box,l,m,n);
}

void
SM_region_controlbox_make(min_point,max_point,the_box)
SM_Region_Controlbox_ptr the_box;
VECTOR_3D min_point, max_point;
{
      VECTOR_3D point1, point2, difference;
      float dx, dy, dz;
      int i, j, k, index;

      for(i=X_INDEX; i<=Z_INDEX; i++) {
        point1[i] = min_point[i] - SM_REGION_BOX_MARGIN;
        point2[i] = max_point[i] + SM_REGION_BOX_MARGIN;
        difference[i] = point2[i] - point1[i];
      }
      dx = difference[X_INDEX]/(the_box->l - 1);
      dy = difference[Y_INDEX]/(the_box->m - 1);
      dz = difference[Z_INDEX]/(the_box->n - 1);
      index = 0;
      for (k=0; k<the_box->n; k++) {
       for (j=0; j<the_box->m; j++) {
        for (i=0; i<the_box->l; i++) {
          point2[X_INDEX] = point1[X_INDEX] + i*dx;
          point2[Y_INDEX] = point1[Y_INDEX] + j*dy;
          point2[Z_INDEX] = point1[Z_INDEX] + k*dz;
          the_box->vertex_list[index].struct_used = YES;
          the_box->vertex_list[index].picked = NO;
          the_box->vertex_list[index].weight = 1.0;
          the_box->vertex_list[index].i = i;
          the_box->vertex_list[index].j = j;
          the_box->vertex_list[index].k = k;
          V3D_COPY(the_box->vertex_list[index].point,point2); 
          index++;
         }
        }
       }
}

  
void
SM_region_controlbox_get(the_surface,the_region_list,the_index,the_box)
SM_Surface_ptr the_surface;
SM_Region_List_ptr the_region_list;
int the_index;
SM_Region_Controlbox_ptr the_box;

{
     VECTOR_3D min_point, max_point;
     float max_value, min_value;
     int i;

     for (i=0; i<3; i++) {  
     SM_region_maxcoord_get(the_surface,the_region_list,the_index,i,&max_value);
     max_point[i] = max_value;
     SM_region_mincoord_get(the_surface,the_region_list,the_index,i,&min_value);
     min_point[i] = min_value;
     }
     SM_region_controlbox_make(min_point,max_point,the_box);
}

SM_Region_Controlsphere_ptr
SM_region_controlsphere_new()

{
    SM_Region_Controlsphere_ptr the_controlsphere;
 
    the_controlsphere = (SM_Region_Controlsphere_ptr) 
                      calloc(1,sizeof(SM_Region_Controlsphere));
    the_controlsphere->struct_used = YES;
    the_controlsphere->latitude_prec = SM_REGION_SPHERE_DEFAULT_PREC;
    the_controlsphere->longitude_prec = SM_REGION_SPHERE_DEFAULT_PREC;
    the_controlsphere->vertex_list_size = 0;
    the_controlsphere->vertex_list = NULL;

    return(the_controlsphere);
}


BOOL
SM_region_controlsphere_valid(c_sphere)
SM_Region_Controlsphere_ptr c_sphere;
{
      if (c_sphere == NULL) return(NO);
      if (c_sphere->struct_used) return(YES);
      return(NO);
}

BOOL
SM_region_controlsphere_prec_set(c_sphere,which_one,value)   
SM_Region_Controlsphere_ptr c_sphere;
int which_one,value;
{
   if (!SM_region_controlsphere_valid(c_sphere)) return(NO);
   switch (which_one) {
     case 0 : /* latitude */ 
              c_sphere->latitude_prec = value;
              break;
     case 1 : /* longitude */ 
              c_sphere->longitude_prec = value;
              break;
     default : 
              break;
    }
   return(YES);
}


BOOL
SM_region_controlsphere_vertexlist_set(c_sphere)   
SM_Region_Controlsphere_ptr c_sphere;
{
       int number_of_points;

       if (!SM_region_controlsphere_valid(c_sphere)) return(NO);
       number_of_points = (c_sphere->longitude_prec - 2) * 
                          (c_sphere->latitude_prec) + 2;
       c_sphere->vertex_list_size = number_of_points;
       c_sphere->vertex_list = (SM_Region_Controlsphere_Vertex_ptr)        
         calloc(number_of_points, sizeof(SM_Region_Controlsphere_Vertex));
       return(YES);
}

void
SM_region_controlsphere_vertexlist_dispose(c_sphere)   
SM_Region_Controlsphere_ptr c_sphere;
{

       if (SM_region_controlsphere_valid(c_sphere)) {
          free(c_sphere->vertex_list);
          c_sphere->vertex_list_size = 0;
          c_sphere->vertex_list = NULL;
       }
}

void
SM_region_controlsphere_dispose(c_sphere)
SM_Region_Controlsphere_ptr c_sphere;
{
       SM_region_controlsphere_vertexlist_dispose(c_sphere);
       free(c_sphere);
}

void
SM_region_controlsphere_create(c_sphere)
SM_Region_Controlsphere_ptr c_sphere;
{
       c_sphere = SM_region_controlsphere_new();
       SM_region_controlsphere_vertexlist_set(c_sphere);
}

void
SM_region_controlsphere_recreate(c_sphere)
SM_Region_Controlsphere_ptr c_sphere;
{
       SM_region_controlsphere_dispose(c_sphere);
       SM_region_controlsphere_create(c_sphere);
}


void
SM_region_controlsphere_make(center,radius,c_sphere)
VECTOR_3D center;
float radius;
SM_Region_Controlsphere_ptr c_sphere;
{

      VECTOR_3D point1;
      int i, j, res_long, res_lat, index;
      float theta, delta_theta, phi, delta_phi;

      res_long = c_sphere->longitude_prec;
      res_lat = c_sphere->latitude_prec;
 
      phi = 0.0; delta_theta = 2 * M_PI/res_lat; delta_phi = M_PI/res_long;
      V3D_INIT(point1, (float) center[X_INDEX], (float) center[Y_INDEX], 
               (float) (center[Z_INDEX]-radius) );
      index = 0;
      c_sphere->vertex_list[index].struct_used = YES;
      c_sphere->vertex_list[index].picked = NO;
      V3D_COPY(c_sphere->vertex_list[index].point, point1);
      index++;
      for (i=1; i<res_long; i++) {
        theta = 0.0;
        phi += delta_phi;
        for (j=0; j<res_lat; j++) {
          V3D_INIT(point1,(float) radius * sin(phi) * cos(theta),
                          (float) radius * sin(phi) * sin(theta),
                          (float) radius * cos(phi) );
          V3D_COPY(c_sphere->vertex_list[index].point, point1);
          theta += delta_theta;
          index++;
        }
      }
      V3D_INIT(point1, (float) center[X_INDEX], (float) center[Y_INDEX], 
               (float) (center[Z_INDEX]+radius) );
      c_sphere->vertex_list[index].struct_used = YES;
      c_sphere->vertex_list[index].picked = NO;
      V3D_COPY(c_sphere->vertex_list[index].point, point1);
}

void
SM_region_controlsphere_get(the_surface,the_region_list,the_index,c_sphere)
SM_Surface_ptr the_surface;
SM_Region_List_ptr the_region_list;
int the_index;
SM_Region_Controlsphere_ptr c_sphere;

{
    VECTOR_3D max_point, min_point, center;
    float radius;
    float max_value, min_value;
    int i;

    for (i=0; i<3; i++) {
     SM_region_maxcoord_get(the_surface,the_region_list,the_index,i,&max_value);
     max_point[i] = max_value;
     SM_region_mincoord_get(the_surface,the_region_list,the_index,i,&min_value);
     min_point[i] = min_value;
    }
    SM_region_controlcenter_get(max_point,min_point,center); 
    SM_region_controlradius_get(max_point,min_point,&radius);
    SM_region_controlsphere_make(center,radius,c_sphere);
}

/*------------------------------------------------------------------*/

SM_Region_Controlcylinder_ptr
SM_region_controlcylinder_new(l,m,n)

{
    SM_Region_Controlcylinder_ptr the_controlcylinder;
 
    the_controlcylinder = (SM_Region_Controlcylinder_ptr) 
                      calloc(1,sizeof(SM_Region_Controlcylinder));
    the_controlcylinder->struct_used = YES;
    the_controlcylinder->l = l;
    the_controlcylinder->m = m;
    the_controlcylinder->n = n;
    the_controlcylinder->vertex_list_size = 0;
    the_controlcylinder->vertex_list = NULL;

    return(the_controlcylinder);
}

BOOL
SM_region_controlcylinder_valid(the_cylinder)
SM_Region_Controlcylinder_ptr the_cylinder;
{
      if (the_cylinder == NULL) return(NO);
      if (the_cylinder->struct_used) return(YES);
      return(NO);
}

BOOL
SM_region_controlcylinder_vertexlist_set(the_cylinder)   
SM_Region_Controlcylinder_ptr the_cylinder;
{
       int number_of_points;

       if (!SM_region_controlcylinder_valid(the_cylinder)) return(NO);
       number_of_points = (the_cylinder->l) * (the_cylinder->m) * (the_cylinder->n);
       the_cylinder->vertex_list_size = number_of_points;
       the_cylinder->vertex_list = (SM_Region_Controlcylinder_Vertex_ptr)        
         calloc(number_of_points, sizeof(SM_Region_Controlcylinder_Vertex));
       return(YES);
}

void
SM_region_controlcylinder_vertexlist_dispose(the_cylinder)   
SM_Region_Controlcylinder_ptr the_cylinder;
{

          if (the_cylinder->vertex_list != NULL) {
           free(the_cylinder->vertex_list);
           the_cylinder->vertex_list_size = 0;
           the_cylinder->vertex_list = NULL;
          }
}

void
SM_region_controlcylinder_dispose(the_cylinder)
SM_Region_Controlcylinder_ptr the_cylinder;
{
       if (SM_region_controlcylinder_valid(the_cylinder)) { 
       SM_region_controlcylinder_vertexlist_dispose(the_cylinder);
       free(the_cylinder);
       }
}

void
SM_region_controlcylinder_create(the_cylinder,l,m,n)
SM_Region_Controlcylinder_ptr the_cylinder;
int l,m,n;
{
       the_cylinder = SM_region_controlcylinder_new(l,m,n);
       SM_region_controlcylinder_vertexlist_set(the_cylinder);
}

void
SM_region_controlcylinder_recreate(the_cylinder,l,m,n)
SM_Region_Controlcylinder_ptr the_cylinder;
int l,m,n;
{
       SM_region_controlcylinder_dispose(the_cylinder);
       SM_region_controlcylinder_create(the_cylinder,l,m,n);
}

void
SM_region_polar_to_cartesian(point1,center,point2)
VECTOR_3D point1,center,point2;
{
  float r,t;

      r = point1[0]; t = point1[1];
      point2[X_INDEX] = center[X_INDEX] + r*cos(t);
      point2[Y_INDEX] = center[Y_INDEX] + point1[2];
      point2[Z_INDEX] = center[Z_INDEX] + r*sin(t);

}

void
SM_region_cartesian_to_polar(point1,center,point2)
VECTOR_3D point1,center,point2;
{
  float r,t,z;

      r = sqrt((point1[0] - center[0]) * (point1[0] - center[0]) + 
               (point1[2] - center[2]) * (point1[2] - center[2]));
      t = atan2((point1[2] - center[2]),(point1[0] - center[0]));
      if ( t < 0.0) t += 2*M_PI;
      if ( ( t < SM_EPSILON) && ((point1[2] - center[2]) < 0.0) ) t += 2*M_PI; 
      z = point1[Y_INDEX] - center[Y_INDEX];
      V3D_INIT(point2,r,t,z);
}

void
SM_region_controlcylinder_make(center,radius,height,the_cylinder)
SM_Region_Controlcylinder_ptr the_cylinder;
VECTOR_3D center;
float  radius, height;
{
      VECTOR_3D point1, point2, polar_center;
      float dr, dt, dz;
      int i, j, k, index;

      dr = radius/(the_cylinder->l - 1);
      dt = 2*M_PI/(the_cylinder->m - 1);
      dz = height/(the_cylinder->n - 1);
      index = 0;
printf("radius height %f %f \n",radius,height);
      V3D_NULL(polar_center);
      for (k=0; k<the_cylinder->n; k++) {
       for (j=0; j<the_cylinder->m; j++) {
        for (i=0; i<the_cylinder->l; i++) {
          point1[0] = polar_center[0] + i*dr;
          point1[1] = polar_center[1] + j*dt;
          point1[2] = polar_center[2] + k*dz;
          the_cylinder->vertex_list[index].struct_used = YES;
          the_cylinder->vertex_list[index].picked = NO;
          the_cylinder->vertex_list[index].weight = 1.0;
          the_cylinder->vertex_list[index].i = i;
          the_cylinder->vertex_list[index].j = j;
          the_cylinder->vertex_list[index].k = k;
          SM_region_polar_to_cartesian(point1,center,point2);
          V3D_COPY(the_cylinder->vertex_list[index].point,point2); 
          index++;
         }
        }
       }
}


void
SM_region_controlcylinder_get(the_surface,the_region_list,the_index,the_cylinder)
SM_Surface_ptr the_surface;
SM_Region_List_ptr the_region_list;
int the_index;
SM_Region_Controlcylinder_ptr the_cylinder;

{
    VECTOR_3D max_point, min_point, center;
    float radius;
    float max_value, min_value, height;
    int i;

    for (i=0; i<3; i++) {
     SM_region_maxcoord_get(the_surface,the_region_list,the_index,i,&max_value);
     max_point[i] = max_value;
     SM_region_mincoord_get(the_surface,the_region_list,the_index,i,&min_value);
     min_point[i] = min_value;
    }
    SM_region_controlcenter_get(max_point,min_point,center);
    /* the axis is presently fixed as Y */
    center[Y_INDEX] = min_point[Y_INDEX] - SM_REGION_BOX_MARGIN; 
    height = max_point[Y_INDEX] - min_point[Y_INDEX] + 2*SM_REGION_BOX_MARGIN;
    SM_region_controlradius_get(max_point,min_point,&radius);
    SM_region_controlcylinder_make(center,radius,height,the_cylinder);
}
/*------------------------------------------------------------------*/
/* These are addional routines added by Laurent Moccozet in order
to be able to manipulate the data structure for some applications
like 'simplification' of surfaces.

P kalra                                           oct '92
*/
 
/*
* Returns the actual number of vertices of a region
*/
int 
SM_region_vertex_count_get(the_region_list, the_name)
SM_Region_List_ptr the_region_list;
SM_Region_Name     the_name;
{
    int           the_index;
    SM_Region_ptr region_ptr;
    
    the_index = SM_region_index_get(the_region_list, the_name);
    if (the_index == -1) return(SM_INDEX_ERROR);

    region_ptr = &the_region_list->region_list[the_index];
    return(region_ptr->region_vertex_list_size);
}

/*
* Returns the actual number of edges of a region
*/
int 
SM_region_edge_count_get(the_region_list, the_name)
SM_Region_List_ptr the_region_list;
SM_Region_Name     the_name;
{
    int           the_index;
    SM_Region_ptr region_ptr;

    the_index = SM_region_index_get(the_region_list, the_name);
    if (the_index == -1) return(SM_INDEX_ERROR);
    
    region_ptr = &the_region_list->region_list[the_index];
    return(region_ptr->region_edge_list_size);
}

/*
* Returns the actual number of triangles of a region
*/
int 
SM_region_triangle_count_get(the_region_list, the_name)
SM_Region_List_ptr the_region_list;
SM_Region_Name     the_name;
{
    int           the_index;
    SM_Region_ptr region_ptr;

    the_index = SM_region_index_get(the_region_list, the_name);
    if (the_index == -1) return(SM_INDEX_ERROR);
     
    region_ptr = &the_region_list->region_list[the_index];
    return(region_ptr->region_tri_list_size);
}

/*
 * Given a list of regions,  the name of a region and the index of an edge,  check
 * if the edge belongs to the region.
 * 
 */
BOOL
SM_region_edge_valid(the_region_list, the_name, edge_index)
SM_Region_List_ptr the_region_list;
SM_Region_Name     the_name;
int                edge_index; 
{
    int the_index;
    SM_Region_ptr region_ptr;
    
    the_index = SM_region_index_get(the_region_list, the_name);
    if (the_index == -1) return(NO);
    region_ptr = &the_region_list->region_list[the_index];
    SM_FOR_EACH_REGION_EDGE(region_ptr)
	if (edge_index == *region_edge_ptr) {
	    return(YES);
	}
    SM_END_FOR_EACH 
    return(NO);   
}

/*
 * Given a list of regions,  the name of a region and the index of an vertex,  check
 * if the vertex belongs to the region.
 * The index is assumed to refer to the surface,  not to the region.
 * 
 */

BOOL
SM_region_vertex_valid(the_region_list, the_name, vertex_index)
SM_Region_List_ptr the_region_list;
SM_Region_Name     the_name;
int                vertex_index;
{
    int           the_index;
    SM_Region_ptr region_ptr;
    
    the_index = SM_region_index_get(the_region_list, the_name);
    if (the_index == -1) return(NO);
    region_ptr = &the_region_list->region_list[the_index];
    SM_FOR_EACH_REGION_VERTEX(region_ptr)
	if (vertex_index == region_vertex_ptr->id) {
	    return(YES);
	}
    SM_END_FOR_EACH
    return(NO);
}

/*
 * Given a surface, a list of region,  the name of a region (in the list,  and
 * the index of an edge in the surface, return the indices of the two endpoints of
 * the edge in the region.
 * 
 * ERRORS :the index of the edge is not valid for the surface.
 *         the index of the edge is not valid for the region.
 *         the region is not valid.
 */
BOOL 
SM_region_edge_vertices_get(the_surface, the_region_list, the_name, edge_index, v1, v2)
SM_Surface_ptr     the_surface;
SM_Region_List_ptr the_region_list;
SM_Region_Name     the_name;
int                edge_index, *v1, *v2;
{
    int           p1, p2, the_index;
    SM_Region_ptr region_ptr;
     
    the_index = SM_region_index_get(the_region_list, the_name);
    if (the_index == -1) return(NO);  
    if ( ! SM_edge_index_valid(the_surface, edge_index) ) return (NO);
    if ( ! SM_region_edge_valid(the_region_list, the_name, edge_index) ) return (NO);
    region_ptr = &the_region_list->region_list[the_index];
    SM_edge_vertices_get(the_surface, edge_index, &p1, &p2);
    SM_FOR_EACH_REGION_VERTEX(region_ptr)
	if ( region_vertex_ptr->id == p1 )  {
	    *v1 = region_vertex_index;
	}
	if ( region_vertex_ptr->id == p2 ) {
	    *v2 = region_vertex_index;
	}
    SM_END_FOR_EACH
    return(YES);
}
 
/*
 * Given a surface, a list of regions, the name of a region, the index of a triangle and
 * three pointers to integers,  return the indices in the region of the three vertices 
 * of the triangle.
 * 
 * ERRORS : the region is not valid
 *          the index of the triangle is not valid for the surface.
 *          the index of the triangle is not valid for the region.
 */
 
BOOL 
SM_region_triangle_vertices_get(the_surface, the_region_list, the_name, triangle_index, v1, v2, v3)
SM_Surface_ptr     the_surface;
SM_Region_List_ptr the_region_list;
SM_Region_Name     the_name;
int                triangle_index, *v1, *v2, *v3;
 {
    int           p1, p2, p3, the_index;
    SM_Region_ptr region_ptr;
 
     the_index = SM_region_index_get(the_region_list, the_name);
     if (the_index == -1) return(NO);
   
     if ( ! SM_triangle_index_valid(the_surface, triangle_index) ) return (NO);
     if ( ! SM_region_triangle_valid(the_region_list, the_index, triangle_index) ) return(NO);

     region_ptr = &(the_region_list->region_list[the_index]);
     SM_triangle_vertex_index_get(the_surface, triangle_index, &p1, &p2, &p3);
     SM_FOR_EACH_REGION_VERTEX(region_ptr)
	if ( region_vertex_ptr->id == p1 )  {
	    *v1 = region_vertex_index;
	}
	if ( region_vertex_ptr->id == p2 ) {
	    *v2 = region_vertex_index;
	}
	if ( region_vertex_ptr->id == p3 ) {
	    *v3 = region_vertex_index;
	}
     SM_END_FOR_EACH
     return(YES);
 }

/*
 * Given a surface, the index of a vertex, return in an array the list of the indexes of
 * the incident edges to the vertex.
 * 
 * ERRORS : the index of the vertex is not valid for the surface.
 */
 
BOOL 
SM_region_vertex_incident_edges(the_surface, vertex_index, list_edges, nb_edges)
SM_Surface_ptr the_surface;
int            vertex_index;
int            **list_edges;
int            *nb_edges;
{
    SM_Vertex_ptr      vertex_ptr;
    int                *list_ptr,indice;
    SM_Region_List_ptr temp_list;
    int                temp_region;
    SM_Region_ptr      temp_region_ptr = NULL;
    
    if (! SM_vertex_index_valid(the_surface, vertex_index)) return(NO);
    vertex_ptr = &(the_surface->vert_list[vertex_index]);    
    temp_list = SM_region_list_new(the_surface);
    temp_region = SM_region_add(temp_list,"temp");
    SM_FOR_EACH_NORMAL(vertex_ptr)
 	    for (   indice = 0,
	    	    list_ptr = normal_ptr->tri_ref_list;
	    	    indice < normal_ptr->tri_ref_list_occ;
	    	    indice++,
	    	    list_ptr++	) {
			SM_triangle_picked_set(the_surface, *list_ptr, YES);
		    }
    SM_END_FOR_EACH
    SM_region_add_ref(the_surface, temp_list, temp_region);
    temp_region_ptr = &(temp_list->region_list[temp_region]);    
    SM_FOR_EACH_REGION_TRIANGLE(temp_region_ptr)
	SM_triangle_picked_set(the_surface, *region_tri_ptr, NO);
    SM_END_FOR_EACH
    *nb_edges = 0;
    *list_edges = (int *)calloc(1, sizeof(int));
    SM_FOR_EACH_REGION_EDGE(temp_region_ptr)
	(*nb_edges)++;
	*list_edges = (int *)realloc(*list_edges, (*nb_edges)*sizeof(int));
	(*list_edges)[*nb_edges] = *region_edge_ptr;
    SM_END_FOR_EACH    
    SM_region_dispose_all(temp_list, the_surface);
    SM_region_list_dispose(temp_list, the_surface);
    return(YES);
}

/*------------------------------------------------------------------*/
