
/*------------------------------------------------------------------------------*/
/*										*/
/*										*/
/*  This software module was originally developed by                            */
/*        Prem Kalra, MIRALab, University of Geneva                             */
/*  and edited by                                                               */
/*        Igor Pandzic, MIRALab, University of Geneva                           */
/*        Marc Escher, MIRALab, University of Geneva                            */
/*        Pierre Beylot, MIRALab, University of Geneva                          */
/*  in the course of development of the MPEG-4 Video (ISO/IEC 14496-2).	        */
/*  This software module is an implementation of a part of one or more	        */
/*  MPEG-4 Video (ISO/IEC 14496-2) tools as specified by the MPEG-4 Video       */
/*  (ISO/IEC 14496-2). ISO/IEC gives users of the                               */
/*  MPEG-4 Video (ISO/IEC 14496-2) free license to this software module or	*/
/*  modifications thereof for use in hardware or software products claiming 	*/
/*  conformance to the MPEG-4 Video (ISO/IEC 14496-2). Those intending to use	*/
/*  this software module in hardware or software products are advised that	*/
/*  its use may infringe existing patents. The original developer of this	*/
/*  software module and his/her company, the subsequent editors and their	*/
/*  companies, and ISO/IEC have no liability for use of this software		*/
/*  module or modifications thereof. Copyright is not released for non		*/
/*  MPEG-4 Video (ISO/IEC 14496-2) conforming products.                         */
/*  MIRALab, University of Geneva retains full right to use the code for its    */
/*  own purpose, assign or donate the code to a third party and to inhibit      */
/*  third parties from using the code for non MPEG-4 Video (ISO/IEC 14496-2)    */
/*  conforming products. This copyright notice must be included in all copies   */
/*  or derivative works. Copyright (c) (1997).			      		*/
/*										*/
/*										*/
/*------------------------------------------------------------------------------*/


#ifndef REGION_MANAGER
#define REGION_MANAGER

#include "SurfMan.h"

#define SM_REGION_NAME_LENGTH 50
#define SM_REGION_DEFAULT_STIFFNESS 1.0 
#define SM_REGION_DEFAULT_YOUNGMOD 0.0 
#define SM_REGION_DEFAULT_POISSON 0.0 
#define SM_REGION_BOX_DEFAULT_LMN 3
#define SM_REGION_BOX_MIN_LMN 2
#define SM_REGION_BOX_MARGIN 2.0
#define SM_REGION_SPHERE_MARGIN 2.0
#define SM_REGION_SPHERE_DEFAULT_PREC 2.0
#define SM_REGION_MIN_COORD -10000.0 
#define SM_REGION_MAX_COORD 10000.0 
#define SM_REGION_AXIS_X 0 
#define SM_REGION_AXIS_Y 0 
#define SM_REGION_AXIS_Z 0 

#define REGION_ADD_REF 0
#define REGION_REM_REF 1

#define REGION_SHOW  (unsigned int) (1)

#define REGION_SHOW_NULL   (unsigned int) (0)
#define REGION_SHOW_REGION REGION_SHOW
#define REGION_SHOW_ANCHOR REGION_SHOW << 1
#define REGION_SHOW_STIFF  REGION_SHOW << 2

typedef char SM_Region_Name[SM_REGION_NAME_LENGTH];

typedef struct {
 float stiffness;
 float young_modulus;
 float poisson_ratio;
} SM_Vertex_Properties, *SM_Vertex_Properties_ptr;

typedef struct {
  int id;
  SM_Vertex_Properties properties;
  BOOL anchor;
  BOOL flag;
} SM_Region_Vertex, *SM_Region_Vertex_ptr;

typedef int SM_Region_Edge, *SM_Region_Edge_ptr;
typedef int SM_Region_Triangle, *SM_Region_Triangle_ptr;

typedef struct {
 BOOL struct_used;
 SM_Region_Name name;
 SM_Id region_id;
 int region_vertex_list_size;
 SM_Region_Vertex_ptr region_vertex_list;
 int region_edge_list_size;
 SM_Region_Edge_ptr region_edge_list;
 int region_tri_list_size;
 SM_Region_Triangle_ptr region_tri_list;
} SM_Region, *SM_Region_ptr; 
 
typedef struct {
 BOOL struct_used;
 int region_list_size;
 SM_Region_ptr region_list;
 SM_Surface_ptr the_surface;
} SM_Region_List, *SM_Region_List_ptr;

typedef struct {
  int vertex_id;
  float weight;
  BOOL struct_used;
  BOOL picked;
  short i,j,k;
  VECTOR_3D point;
} SM_Region_Controlbox_Vertex, *SM_Region_Controlbox_Vertex_ptr,
  SM_Region_Controlsphere_Vertex, *SM_Region_Controlsphere_Vertex_ptr,
  SM_Region_Controlcylinder_Vertex, *SM_Region_Controlcylinder_Vertex_ptr,
  SM_Surface_Controlbox_vertex, *SM_Surface_Controlbox_vertex_ptr;

typedef struct {
  BOOL struct_used;
  int l, m, n;
  int vertex_list_size;
  SM_Region_Controlbox_Vertex_ptr vertex_list;
} SM_Region_Controlbox, *SM_Region_Controlbox_ptr,
  SM_Region_Controlcylinder, *SM_Region_Controlcylinder_ptr,
  SM_Surface_Controlbox, *SM_Surface_Controlbox_ptr;
 
typedef struct {
  BOOL struct_used;
  int latitude_prec, longitude_prec;
  int vertex_list_size;
  SM_Region_Controlbox_Vertex_ptr vertex_list;
} SM_Region_Controlsphere, *SM_Region_Controlsphere_ptr;

/*
typedef struct {
  BOOL struct_used;
  int latitude_prec, longitude_prec;
  int vertex_list_size;
  SM_Region_Controlbox_Vertex_ptr vertex_list;
} SM_Region_Controlcylinder, *SM_Region_Controlcylinder_ptr;
*/

#define SM_FOR_EACH_REGION(the_region_list) \
    { \
       SM_Region_ptr region_ptr; \
       register int region_index; \
       for ( region_index = 0, \
             region_ptr = the_region_list->region_list; \
             region_index < the_region_list->region_list_size; \
             region_index++, \
             region_ptr++ ) \
             if (region_ptr->struct_used) {
 
#define SM_FOR_EACH_REGION_VERTEX(the_region_ptr) \
    { \
       SM_Region_Vertex_ptr region_vertex_ptr; \
       register int region_vertex_index; \
       for ( region_vertex_index = 0, \
             region_vertex_ptr = the_region_ptr->region_vertex_list; \
             region_vertex_index < the_region_ptr->region_vertex_list_size; \
             region_vertex_index++, \
             region_vertex_ptr++ ) \
             {

#define SM_FOR_EACH_REGION_TRIANGLE(the_region_ptr) \
    { \
       SM_Region_Triangle_ptr region_tri_ptr; \
       register int region_tri_index; \
       for ( region_tri_index = 0, \
             region_tri_ptr = the_region_ptr->region_tri_list; \
             region_tri_index < the_region_ptr->region_tri_list_size; \
             region_tri_index++, \
             region_tri_ptr++ ) \
             {

#define SM_FOR_EACH_REGION_EDGE(the_region_ptr) \
    { \
       SM_Region_Edge_ptr region_edge_ptr; \
       register int region_edge_index; \
       for ( region_edge_index = 0, \
             region_edge_ptr = the_region_ptr->region_edge_list; \
             region_edge_index < the_region_ptr->region_edge_list_size; \
             region_edge_index++, \
             region_edge_ptr++ ) \
             {

#define SM_FOR_EACH_CTRLPOINT(the_controlbox_ptr) \
    { \
       SM_Surface_Controlbox_vertex_ptr ctrlpoint_ptr; \
       register int ctrlpoint_index; \
       register int list_size; \
       list_size = the_controlbox_ptr->l*the_controlbox_ptr->m*the_controlbox_ptr->n; \
       for ( ctrlpoint_index = 0, \
             ctrlpoint_ptr = the_controlbox_ptr->vertex_list; \
             ctrlpoint_index < list_size; \
             ctrlpoint_index++, \
             ctrlpoint_ptr++ ) \
             {

#define SM_FOR_CONTROL_CYLINDER(the_cylinder_ptr) \
    { \
       SM_Region_Controlcylinder_Vertex_ptr cylinder_vertex_ptr; \
       register int cylinder_vertex_index; \
       for ( cylinder_vertex_index = 0, \
             cylinder_vertex_ptr = the_cylinder_ptr->vertex_list; \
             cylinder_vertex_index < the_cylinder_ptr->vertex_list_size; \
             cylinder_vertex_index++, \
             cylinder_vertex_ptr++ ) \
             {

/*
** The Routines 
*/

extern SM_Region_List_ptr
SM_region_list_new(SM_Surface_ptr);
/* allocates memory for a new empty region list */
 
extern void
SM_region_list_dispose(SM_Region_List_ptr, SM_Surface_ptr);
/* diposes (frees) the region list */

extern void
SM_region_dispose_all(SM_Region_List_ptr, SM_Surface_ptr);
/* empties the list of regions */

extern void
SM_region_list_print(SM_Region_List_ptr);
/* for debugging */

extern BOOL
SM_region_print(SM_Region_List_ptr,SM_Region_Name);
/* for debugging */

extern BOOL
SM_region_list_read(SM_Region_List_ptr, SM_Surface_ptr, char *);
/* to read the list of region from a file */

extern void
SM_region_list_read_file(SM_Region_List_ptr, SM_Surface_ptr, FILE *);
/* internal routine to read from a file given by file pointer */

extern void
SM_region_read_file(SM_Region_List_ptr,int, SM_Surface_ptr, FILE * );
/* internal routine to read a region from a file given by a poniter */

extern BOOL
SM_region_list_write(SM_Region_List_ptr,char *);
/* to write the list of regions into a file */

extern void
SM_region_list_write_file(SM_Region_List_ptr,FILE *);
/* internal routine to write to file given by file pointer */

extern void
SM_region_write_file(SM_Region_List_ptr,int,FILE * );
/* internal to write a regin into a file given by file pointer */

extern BOOL
SM_region_triangle_valid(SM_Region_List_ptr,int,int);
/* to check if the triangle exit in a region, the arguments are 
the_region_list, region_index, triangle_index. It returns YES if
it exists */

extern BOOL
SM_region_valid_name(SM_Region_List_ptr,SM_Region_Name);
/* to check the name of the region if it already exists, in case 
it does exist it return NO as region with this name can be added 
into the list */

extern BOOL
SM_region_check_eye(SM_Region_Name);

extern BOOL
SM_region_make_eye(SM_Surface_ptr);

extern BOOL
SM_region_set_eye(SM_Surface_ptr,int);

extern int
SM_region_add(SM_Region_List_ptr,SM_Region_Name);
/* to add a region into the list of regions */

extern int
SM_region_index_get(SM_Region_List_ptr,SM_Region_Name);
/* to get the index of the region given by its name, it returns
the index, if region does not exist it return -1 */

extern BOOL
SM_region_name_get(SM_Region_List_ptr,int,SM_Region_Name);
/* the inverse of the previous procedure, provides the name of
the region given its index */

extern BOOL
SM_region_dispose(SM_Region_List_ptr,SM_Region_Name, SM_Surface_ptr);
/* to delete a rgion from the list of regions */

extern BOOL
SM_region_remove_ref(SM_Region_List_ptr,int, SM_Surface_ptr);
/* removes the references of region being deleted, internal
routine */

extern BOOL
SM_region_add_ref(SM_Surface_ptr,SM_Region_List_ptr,int);
/* adds reference to the triangles picked, internal routine 
while adding a region, region is given by its index */

extern BOOL
SM_region_modify_ref(SM_Surface_ptr,SM_Region_List_ptr,int,int);
/* modifies a region i.e. adds triangles or removes triangles from
the respective list in a region */

extern void
SM_region_copy(SM_Surface_ptr,SM_Region_List_ptr,int,int);
/* copies a region to another region provided by indices */

extern void
SM_region_merge(SM_Surface_ptr,SM_Region_List_ptr,int,int,int);
/* merges a region into another region provided by indices */

extern BOOL
SM_region_flip(SM_Surface_ptr,SM_Region_List_ptr,int,int,VECTOR_3D, double);

extern void
SM_region_vertex_anchor_set(SM_Region_List_ptr,int,int,BOOL);
/* sets the anchor flag of a point to YES or NO. the arguments are
the_region_list, region_index, vertex_index and boolean flag */

extern void
SM_region_vertex_anchor_get(SM_Region_List_ptr,int,int,BOOL *);
/* to get the anchorness flag of a point */

extern void
SM_region_vertex_property_set(SM_Region_List_ptr,int,int,int,double); 
/* to set other properties of the point, not available from 
the interface, may be later version have it */

extern void
SM_region_vertex_property_get(SM_Region_List_ptr,int,int,int,float *); 
/* not in use at present */

extern BOOL
SM_region_center_get(SM_Surface_ptr,SM_Region_List_ptr,int,VECTOR_3D);
/* gives the center of region given by its index, the point return is
the positional center of the region */

extern void
SM_region_maxcoord_index_get(SM_Surface_ptr,SM_Region_List_ptr,int,int,int *);

extern void
SM_region_mincoord_index_get(SM_Surface_ptr,SM_Region_List_ptr,int,int,int *);

extern void
SM_region_maxcoord_get(SM_Surface_ptr,SM_Region_List_ptr,int,int,float *);
/* to get the max extremities, the arguments are 
the_surface,the_region_list,region_index,axis_index (0:X,1:Y,2:Z)
it is useful to obtain a bounding box*/

extern void
SM_region_mincoord_get(SM_Surface_ptr,SM_Region_List_ptr,int,int,float *);
/* to get the min extermities */

extern void
SM_region_controlcenter_get(VECTOR_3D,VECTOR_3D,VECTOR_3D);
/* gives the center when given the two extreme points */

extern void
SM_region_controlradius_get(VECTOR_3D,VECTOR_3D,float *);
/* gives the radius for contro cylinder, not available by
the interface at present */

extern void
SM_region_controlaxis_get(VECTOR_3D,VECTOR_3D,int *);
/* prvides the axis (X,Y,Z) for the cylinder, not available by
the interface at present */

extern SM_Region_Controlbox_ptr
SM_region_controlbox_new(int,int,int);
/* allocates for a new control box given by precision (resolution)
l,m,n */

extern BOOL
SM_region_controlbox_valid(SM_Region_Controlbox_ptr);
/* to check if the proper allocation has been done */

extern BOOL
SM_region_controlbox_vertexlist_set(SM_Region_Controlbox_ptr);
/* to define the vertex list for the control box, internal
routine while creating the control box */

extern void
SM_region_controlbox_vertexlist_dispose(SM_Region_Controlbox_ptr);
/* to dispose the vertex list while freeing the control box,
internal routine */

extern void
SM_region_controlbox_dispose(SM_Region_Controlbox_ptr);
/* to dispose the control box */

extern void
SM_region_controlbox_create(SM_Region_Controlbox_ptr,int,int,int);
/* to create the control box with the precision, uses 'new' and
vertexlist_set internal routine */

extern void
SM_region_controlbox_recreate(SM_Region_Controlbox_ptr,int,int,int);
/* combination of dispose and create used when the precision has
changed */

extern void
SM_region_controlbox_make(VECTOR_3D, VECTOR_3D, SM_Region_Controlbox_ptr);
/* internal routine to make a controlbox when given the extreme points */

extern void
SM_region_controlbox_get(SM_Surface_ptr, SM_Region_List_ptr, int, SM_Region_Controlbox_ptr);
/* to obtain the control box for a region given by its index */

extern SM_Region_Controlsphere_ptr
SM_region_controlsphere_new(void);
/* not presently available by the interface */

extern BOOL
SM_region_controlsphere_valid(SM_Region_Controlsphere_ptr);
/* not presently available by the interface */

extern BOOL
SM_region_controlsphere_prec_set(SM_Region_Controlsphere_ptr,int,int);
/* not presently available by the interface */

extern BOOL
SM_region_controlsphere_vertexlist_set(SM_Region_Controlsphere_ptr);
/* not presently available by the interface */

extern void
SM_region_controlsphere_vertexlist_dispose(SM_Region_Controlsphere_ptr);
/* not presently available by the interface */

extern void
SM_region_controlsphere_dispose(SM_Region_Controlsphere_ptr);
/* not presently available by the interface */

extern void
SM_region_controlsphere_create(SM_Region_Controlsphere_ptr);
/* not presently available by the interface */

extern void
SM_region_controlsphere_recreate(SM_Region_Controlsphere_ptr);
/* not presently available by the interface */

extern void
SM_region_controlsphere_make(VECTOR_3D, double, SM_Region_Controlsphere_ptr);
/* not presently available by the interface */

extern void
SM_region_controlsphere_get(SM_Surface_ptr, SM_Region_List_ptr, int, SM_Region_Controlsphere_ptr);
/* not presently available by the interface */

extern SM_Region_Controlcylinder_ptr
SM_region_controlcylinder_new(int,int,int);
/* not presently available by the interface */

extern BOOL
SM_region_controlcylinder_valid(SM_Region_Controlcylinder_ptr);
/* not presently available by the interface */

extern BOOL
SM_region_controlcylinder_vertexlist_set(SM_Region_Controlcylinder_ptr);   
/* not presently available by the interface */

extern void
SM_region_controlcylinder_vertexlist_dispose(SM_Region_Controlcylinder_ptr);
/* not presently available by the interface */

extern void
SM_region_controlcylinder_dispose(SM_Region_Controlcylinder_ptr);
/* not presently available by the interface */

extern void
SM_region_controlcylinder_create(SM_Region_Controlcylinder_ptr,int,int,int);
/* not presently available by the interface */

extern void
SM_region_controlcylinder_recreate(SM_Region_Controlcylinder_ptr,int,int,int);
/* not presently available by the interface */

extern void
SM_region_polar_to_cartesian(VECTOR_3D,VECTOR_3D,VECTOR_3D);
/* not presently available by the interface */

extern void
SM_region_cartesian_to_polar(VECTOR_3D,VECTOR_3D,VECTOR_3D);
/* not presently available by the interface */

extern void
SM_region_controlcylinder_make(VECTOR_3D,double,double,SM_Region_Controlcylinder_ptr);
/* not presently available by the interface */

extern void
SM_region_controlcylinder_get(SM_Surface_ptr,SM_Region_List_ptr,int,SM_Region_Controlcylinder_ptr);
/* not presently available by the interface */

extern BOOL
SM_region_draw(SM_Surface_ptr, SM_Region_List_ptr, SM_Region_Name, unsigned int);
/* to draw a region given by its name */

extern void
SM_region_draw_all(SM_Surface_ptr, SM_Region_List_ptr, unsigned int);
/* to draw all the regions */

extern int 
SM_region_vertex_count_get(SM_Region_List_ptr, SM_Region_Name);
/* to get the number of vertices in a region directly  LM */

extern int 
SM_region_edge_count_get(SM_Region_List_ptr, SM_Region_Name);
/* to get the number of edges in a region directly  LM */

extern int 
SM_region_triangle_count_get(SM_Region_List_ptr, SM_Region_Name);
/* to get the number of triangles in a region directly  LM */

extern BOOL
SM_region_edge_valid(SM_Region_List_ptr, SM_Region_Name, int);
/* to verify if the edge belongs to the region  LM */

extern BOOL
SM_region_vertex_valid(SM_Region_List_ptr, SM_Region_Name, int);
/* to verify if the vertex belongs to the region  LM */

extern BOOL 
SM_region_edge_vertices_get(SM_Surface_ptr,SM_Region_List_ptr, SM_Region_Name, int, int *, int *);
/* to get the vertices belonging to the edge in a region  LM */

extern BOOL 
SM_region_triangle_vertices_get(SM_Surface_ptr,SM_Region_List_ptr, SM_Region_Name, int, int *, int *, int *);
/* to get the vertices belonging to the triangle in a region  LM */

extern BOOL 
SM_region_vertex_incident_edges(SM_Surface_ptr, int, int **, int *);

extern void
SM_surface_region_list_set(SM_Surface_ptr, SM_Region_List_ptr);

extern BOOL
Find_inside_triangle(SM_Surface_ptr,int,VECTOR_3D);

#endif
