
/*------------------------------------------------------------------------------*/
/*										*/
/*										*/
/*  This software module was originally developed by                            */
/*        Prem Kalra, MIRALab, University of Geneva                             */
/*  and edited by                                                               */
/*        Igor Pandzic, MIRALab, University of Geneva                           */
/*        Marc Escher, MIRALab, University of Geneva                            */
/*        Pierre Beylot, MIRALab, University of Geneva                          */
/*  in the course of development of the MPEG-4 Video (ISO/IEC 14496-2).	        */
/*  This software module is an implementation of a part of one or more	        */
/*  MPEG-4 Video (ISO/IEC 14496-2) tools as specified by the MPEG-4 Video       */
/*  (ISO/IEC 14496-2). ISO/IEC gives users of the                               */
/*  MPEG-4 Video (ISO/IEC 14496-2) free license to this software module or	*/
/*  modifications thereof for use in hardware or software products claiming 	*/
/*  conformance to the MPEG-4 Video (ISO/IEC 14496-2). Those intending to use	*/
/*  this software module in hardware or software products are advised that	*/
/*  its use may infringe existing patents. The original developer of this	*/
/*  software module and his/her company, the subsequent editors and their	*/
/*  companies, and ISO/IEC have no liability for use of this software		*/
/*  module or modifications thereof. Copyright is not released for non		*/
/*  MPEG-4 Video (ISO/IEC 14496-2) conforming products.                         */
/*  MIRALab, University of Geneva retains full right to use the code for its    */
/*  own purpose, assign or donate the code to a third party and to inhibit      */
/*  third parties from using the code for non MPEG-4 Video (ISO/IEC 14496-2)    */
/*  conforming products. This copyright notice must be included in all copies   */
/*  or derivative works. Copyright (c) (1997).			      		*/
/*										*/
/*										*/
/*------------------------------------------------------------------------------*/


#ifdef WIN32
#include <winsock.h>
#endif

#include "SurfMan.h"

char SM_version[20] = "SM 93.05.20";

/*
**  Memory allocation increments for lists.
*/

#define SM_TRI_REF_LIST_INC 5

#define SM_VERT_LIST_INIT   200		/* initial list sizes */
#define SM_TRI_LIST_INIT    200
#define SM_EDGE_LIST_INIT   200
#define SM_MAT_LIST_INIT    20

#define SM_VERT_LIST_INC    1.2		/* increment factors on lists */
#define SM_TRI_LIST_INC	    1.2
#define SM_EDGE_LIST_INC    1.2
#define SM_MAT_LIST_INC	    1.5

#define SM_NORM_LIST_INC    3		/* constant increments for vertex normal lists */

#define SM_DEBUG NO


/*
**  RGBA color/coverage variables with default values.
*/

long SM_color_tri_picked	= 0xff0000f0;
long SM_color_tri_not_picked	= 0xff0000c0;
long SM_color_tri_normal	= 0xff7733ff;
long SM_color_tri_shadow	= 0xff111111;

long SM_color_vert_const	= 0xffccaaff;
long SM_color_vert_picked	= 0xff0000f0;
long SM_color_vert_not_picked   = 0xff0000c0;
long SM_color_vert_bind		= 0xffff77ff;
long SM_color_vert_not_bind	= 0xffff77ff;
long SM_color_vert_normal	= 0xffaa33ff;
long SM_color_vert_shadow	= 0xff111111;

long SM_color_edge_const	= 0xaaccffff;
long SM_color_edge_picked	= 0xff0000f0;
long SM_color_edge_not_picked	= 0xff0000c0;
long SM_color_edge_holes	= 0xff77aaff;
long SM_color_edge_not_holes	= 0xff77aaff;
long SM_color_edge_shadow	= 0xff111111;


static int *material_link;
/*
**  Vertices widths and normal lengths when drawing.
*/

float SM_vert_draw_width	    = 5.0;
float SM_vert_normal_draw_length    = 5.0;
float SM_tri_normal_draw_length	    = 5.0;


VECTOR_3D SM_origin = { 0.0, 0.0, 0.0, };

void Stack_Init(the_surface, type)
SM_Surface_ptr the_surface;
BOOL type;
{
  if (type == SM_VERTEX) {
    the_surface->free_vert_list.free_ptr = NULL;
    the_surface->free_vert_list.counter = -1;
  }
  else {
    if (type == SM_EDGE) {
      the_surface->free_edge_list.free_ptr = NULL;
      the_surface->free_edge_list.counter = -1;    
    }
    else {
      the_surface->free_tri_list.free_ptr = NULL;
      the_surface->free_tri_list.counter = -1;   
    }
  }
}

void Stack_Free(the_surface)
SM_Surface_ptr the_surface;
{
  free(the_surface->free_vert_list.free_ptr);
  free(the_surface->free_edge_list.free_ptr);
  free(the_surface->free_tri_list.free_ptr);
}

int Get_From_Stack(the_surface, type)
SM_Surface_ptr the_surface;
BOOL type;
{
  
  if (type == SM_VERTEX) {
    if (the_surface->free_vert_list.counter == -1) return SM_INDEX_ERROR;
    the_surface->free_vert_list.counter -= 1;
    return(the_surface->free_vert_list.free_ptr[the_surface->free_vert_list.counter+1]);
  }
  else if (type == SM_EDGE) {
    if (the_surface->free_edge_list.counter == -1) return SM_INDEX_ERROR;
    the_surface->free_edge_list.counter -= 1;
    return(the_surface->free_edge_list.free_ptr[the_surface->free_edge_list.counter+1]);	
	 } else {
    if (the_surface->free_tri_list.counter == -1) return SM_INDEX_ERROR;
    the_surface->free_tri_list.counter -= 1;
    return(the_surface->free_tri_list.free_ptr[the_surface->free_tri_list.counter+1]);
   }
}

#define SM_STACK_INC 10

void Put_In_Stack(SM_Surface_ptr,int,BOOL);

void Put_In_Stack(the_surface, add, type)
SM_Surface_ptr the_surface;
int            add;
BOOL           type;
{
  int *tempo;
  
  if (type == SM_VERTEX) {
    if (the_surface->free_vert_list.counter == -1) {
      the_surface->free_vert_list.free_ptr = (int *)calloc(SM_STACK_INC, sizeof(int));
      the_surface->free_vert_list.counter += 1;
      the_surface->free_vert_list.free_ptr[the_surface->free_vert_list.counter] = add;
    }
    else {
      if (((the_surface->free_vert_list.counter % (SM_STACK_INC-1)) == 0) && (the_surface->free_edge_list.counter != 0)) {
        the_surface->free_vert_list.free_ptr = (int *)realloc(the_surface->free_vert_list.free_ptr,(the_surface->free_vert_list.counter+SM_STACK_INC)*sizeof(int));
        the_surface->free_vert_list.counter += 1;
        the_surface->free_vert_list.free_ptr[the_surface->free_vert_list.counter] = add;
      }
      else {
        the_surface->free_vert_list.counter += 1;
        the_surface->free_vert_list.free_ptr[the_surface->free_vert_list.counter] = add;		
      }
    }
  }
  else if (type == SM_EDGE) {
    if (the_surface->free_edge_list.counter == -1) {
      the_surface->free_edge_list.free_ptr = (int *)calloc(SM_STACK_INC, sizeof(int));
      the_surface->free_edge_list.counter += 1;
      the_surface->free_edge_list.free_ptr[the_surface->free_edge_list.counter] = add;
    }
    else {
      if (((the_surface->free_edge_list.counter % (SM_STACK_INC-1)) == 0) && (the_surface->free_edge_list.counter != 0)){
        the_surface->free_edge_list.free_ptr = (int *)realloc(the_surface->free_edge_list.free_ptr,(the_surface->free_edge_list.counter+SM_STACK_INC)*sizeof(int));
        the_surface->free_edge_list.counter += 1;
        the_surface->free_edge_list.free_ptr[the_surface->free_edge_list.counter] = add;
      }
      else {
        the_surface->free_edge_list.counter += 1;
        the_surface->free_edge_list.free_ptr[the_surface->free_edge_list.counter] = add;		
      }
    }
    
  } else {
    if (the_surface->free_tri_list.counter == -1) {
      the_surface->free_tri_list.free_ptr = (int *)calloc(SM_STACK_INC, sizeof(int));
      the_surface->free_tri_list.counter += 1;
      the_surface->free_tri_list.free_ptr[the_surface->free_tri_list.counter] = add;
    }
    else {
      if (((the_surface->free_tri_list.counter % (SM_STACK_INC-1)) == 0) && (the_surface->free_edge_list.counter != 0)){
        tempo = (int *)realloc(the_surface->free_tri_list.free_ptr,(the_surface->free_tri_list.counter+SM_STACK_INC)*sizeof(int));
        the_surface->free_tri_list.free_ptr = tempo;
        the_surface->free_tri_list.counter += 1;
        the_surface->free_tri_list.free_ptr[the_surface->free_tri_list.counter] = add;
      }
      else {
        the_surface->free_tri_list.counter += 1;
        the_surface->free_tri_list.free_ptr[the_surface->free_tri_list.counter] = add;		
      }
    }
  }
}

/*
------------------------------   SM_Material  ---------------------------------
*/


#define SM_EMISSION	0
#define SM_AMBIENT	4
#define SM_DIFFUSE	8
#define SM_SPECULAR	12
#define SM_SHININESS	16
#define SM_ALPHA	18

SM_Mat SM_material_default = {
  YES, NO, NO, -1, 0xffc8c8c8, "default",
  {
    EMISSION, 0.0, 0.0, 0.0,
      AMBIENT, 0.2, 0.2, 0.2,
      DIFFUSE, 0.8, 0.8, 0.8,
      SPECULAR, 0.0, 0.0, 0.0,
      SHININESS, 0.0,
      ALPHA, 1.0,
      0.0,
  }, 0,
};

SM_Material the_materials = {
  0, -2, -2, NULL,
};






/*
------------------------------   SM_VECTOR_3D   --------------------------------
*/


void
SM_VECTOR_3D_write(fil, V)
FILE *fil;
VECTOR_3D V;
/*
**  Writes the components of the vector V, in a list format, on the unix 
**  file *fil.
*/
{
  fprintf(fil,"%f,%f,%f",V[X_INDEX],V[Y_INDEX],V[Z_INDEX]);
}


BOOL
SM_VECTOR_3D_read(fil, the_vector, id)
Parser_File *fil;
VECTOR_3D the_vector;
Parser_List_Level id;
/*
**  Returns YES if all the components of the vector exists.
**  Reading is done by the "parser.o" module for list manipulations,
**  so *fil must be initialized accordingly.
*/
{
  BOOL ok;
  
  if (parser_read_float(fil, id, &the_vector[X_INDEX])) {
    ok = (parser_read_float(fil, id, &the_vector[Y_INDEX]) &&
	     parser_read_float(fil, id, &the_vector[Z_INDEX]));	    
    return(ok);
  }
  return(NO);
}














/*
-------------------------------   SM_VERTEX   ---------------------------------
*/

BOOL
SM_vertex_index_valid(the_surface, vertex_index)
SM_Surface_ptr the_surface;
int vertex_index;
/*
**  Returns YES if vertex_index is a valid vertex index.
*/
{
  return( vertex_index >= 0				&& 
    vertex_index < the_surface->vert_list_size	&&
    the_surface->vert_list[vertex_index].struct_used );
}



BOOL
SM_vertex_read(fil, the_surface)
Parser_File *fil;
SM_Surface_ptr the_surface;
/*
**  Returns the vertex_index if the first component of the vertex was found
**  in the fil.  
**  If the first field of the vertex was not found, returns SM_INDEX_ERROR.
**  Reading is done by the "parser.o" module for list manipulations,
**  so *fil must be initialized accordingly.
*/
{   
  Parser_List_Level id1;
  int new_vertex_index, picked;
  
  parser_bgn_list(fil, &id1);
  /*
  **  Test if another SM_Vertex exists by testing for first component.
  */
  if (parser_read_int(fil, id1, &picked)) {
    new_vertex_index = SM_vertex_new(the_surface);
    the_surface->vert_list[new_vertex_index].picked = (BOOL_STACK) picked;
    SM_VECTOR_3D_read(fil, the_surface->vert_list[new_vertex_index].point, id1);
  } else
    new_vertex_index = SM_INDEX_ERROR;
  parser_end_list(fil, id1);
  return(new_vertex_index != SM_INDEX_ERROR);
}



void
SM_vertex_write(fil, the_surface, vertex_index) 
FILE *fil;
SM_Surface_ptr the_surface;
int vertex_index;
/*
**  Writes the content of the the_vertex, in a list format, on the unix
**  file *fil.
*/
{
  SM_Vertex_ptr the_vertex;
  
  the_vertex = &the_surface->vert_list[vertex_index];
  
  fprintf(fil, "(");
  SM_VECTOR_3D_write(fil, the_vertex->point);
  fprintf(fil, ")\n");
}

void
SM_vertex_allocate(SM_Surface_ptr);

void
SM_vertex_allocate(the_surface)
SM_Surface_ptr the_surface;
/*
**  Adds (allocates) new memory for storing vertices.
*/
{
  register SM_Vertex_ptr ptr_1, ptr_2;
  int total_size;
  
  /*
  **	If the list exists, increase its size.  If not, allocate an initial list size.
  */
  if (the_surface->vert_list) { 
    total_size = the_surface->vert_list_size * SM_VERT_LIST_INC;
    the_surface->vert_list = (SM_Vertex_ptr) 
      realloc(the_surface->vert_list, total_size * sizeof(SM_Vertex));
  } else { 
    total_size = SM_VERT_LIST_INIT;
    the_surface->vert_list = (SM_Vertex_ptr)
      malloc(total_size * sizeof(SM_Vertex));
  }
  
  /*
  **	Mark unused structures as unused.
  */
  ptr_1 = the_surface->vert_list + the_surface->vert_list_size;
  ptr_2 = the_surface->vert_list + total_size - 1;
  do { ptr_1->struct_used = NO; } while (ptr_1++ != ptr_2);
  
  the_surface->vert_list_size = total_size;
}


int
SM_vertex_new(the_surface)
SM_Surface_ptr the_surface;
/*
**  Adds a new vertex in "the_surface" and assigns default values.
**  Allocates new memory if needed.
**  Returns the index of the new vertex.
*/
{
  SM_Vertex_ptr new_vertex;
  int new_vertex_index;
  
  new_vertex_index = Get_From_Stack(the_surface, SM_VERTEX);
  if (new_vertex_index == SM_INDEX_ERROR) {
    new_vertex_index = the_surface->current_vert_list_size;
  }
  /* 
  **	Allocate new memory if needed. 
  */
  if (new_vertex_index == the_surface->vert_list_size) 
    SM_vertex_allocate(the_surface);
  
  new_vertex = &the_surface->vert_list[new_vertex_index];
  
  /* 
  **	Initialize new vertex to default values. 
  */
  new_vertex->struct_used = YES;
  new_vertex->picked = STACK_NO;
  new_vertex->vertex_id = new_vertex_index;
  new_vertex->norm_list = NULL;
  new_vertex->norm_list_size = 0;
  new_vertex->func_result = 0.0;
  SM_vertex_point_set(the_surface, new_vertex_index, SM_origin);
  the_surface->current_vert_list_size += 1;
  return(new_vertex_index);
}


BOOL
SM_vertex_dispose(the_surface, vertex_index)
SM_Surface_ptr the_surface;
int vertex_index;
/*
**  Removes the vertex from the_surface.  Deletes all triangles
**  using this vertex.
*/
{
  SM_Vertex_ptr vertex_ptr;
  register int j;
  int * list_ptr;
  
  /*
  **	Parameter validation.
  **	Next 4 lines stops the triangles from counter modifying
  **	the_vertex->tri_ref_list when they are being destroyed.
  */
  if (!SM_vertex_index_valid(the_surface, vertex_index))  return(NO);
  
  vertex_ptr = &the_surface->vert_list[vertex_index];
  
  vertex_ptr->struct_used = NO;
  
  SM_FOR_EACH_NORMAL(vertex_ptr)
    /* Delete all triangles. */
    for (   j = 0,
      list_ptr = normal_ptr->tri_ref_list;
    j < normal_ptr->tri_ref_list_occ;
    j++,
      list_ptr++   )
      
      SM_triangle_dispose(the_surface, *list_ptr);
    
    if (normal_ptr->tri_ref_list) free(normal_ptr->tri_ref_list);
    SM_END_FOR_EACH
      Put_In_Stack(the_surface, vertex_index, SM_VERTEX);
    the_surface->current_vert_list_size -= 1;
    if (vertex_ptr->norm_list) free(vertex_ptr->norm_list);
    return(YES);
}




/*
**  point field.
*/

BOOL
SM_vertex_point_set(the_surface, vertex_index, the_value)
SM_Surface_ptr the_surface;
int vertex_index;
VECTOR_3D the_value;
/*
**  Copies the vector value "the_value" into the "point" field of
**  the_vertex.  The normals of all triangles using this vertex are
**  not good anymore.
*/
{
  SM_Vertex_ptr vertex_ptr;
  int * list_ptr;
  register int j;
  
  /*
  **	Parameter validation.
  */
  if (!SM_vertex_index_valid(the_surface, vertex_index))  return(NO);
  
  vertex_ptr = &the_surface->vert_list[vertex_index];
  
  V3D_COPY(vertex_ptr->point, the_value);
  
  SM_FOR_EACH_NORMAL(vertex_ptr)
    normal_ptr->normal_good = NO;
  
  /* Triangles' normals not good also. */
  for (   j = 0,
    list_ptr = normal_ptr->tri_ref_list;
  j < normal_ptr->tri_ref_list_occ;
  j++,
    list_ptr++   )
    
    if (SM_triangle_index_valid(the_surface, *list_ptr))
		    the_surface->tri_list[*list_ptr].normal_good = NO;
    SM_END_FOR_EACH
      
      return(YES);
}


BOOL
SM_vertex_point_get(the_surface, vertex_index, the_value)
SM_Surface_ptr the_surface;
int vertex_index;
VECTOR_3D the_value;
/*
**  Copies the vector value "vertex_index" into the "the_value" field.
*/
{
  if (!SM_vertex_index_valid(the_surface, vertex_index))  return(NO);
  
  V3D_COPY(the_value, the_surface->vert_list[vertex_index].point);
  return(YES);
}

int
SM_vertex_index_point(the_surface, a_point)
SM_Surface_ptr the_surface;
VECTOR_3D a_point;
/*
**  Returns the index of a vertex located at "a_point".
*/
{
  register SM_Vertex_ptr vertex_ptr;
  register int vertex_index;
  register float *f, x, y, z;
  
  if (the_surface != NULL && the_surface->vert_list_size <= 0)  return(SM_INDEX_ERROR);
  
  x = a_point[X_INDEX];
  y = a_point[Y_INDEX];
  z = a_point[Z_INDEX];
  vertex_index = the_surface->vert_list_size;
  vertex_ptr = &(the_surface->vert_list[vertex_index - 1]);    
  do {
    if (vertex_ptr->struct_used) {
      f = vertex_ptr->point;
      if (*f - x < SM_EPSILON && *f++ - x > -SM_EPSILON &&
        *f - y < SM_EPSILON && *f++ - y > -SM_EPSILON &&
        *f - z < SM_EPSILON && *f - z > -SM_EPSILON)  return(vertex_index - 1);
    }
    vertex_ptr--;
    vertex_index--;
  } while (vertex_index);
  
  return(SM_INDEX_ERROR);
}





/*
**  normal field.
*/
BOOL
SM_vertex_normal_set(the_surface, vertex_index, normal_index, new_normal)
SM_Surface_ptr the_surface;
int vertex_index, normal_index;
VECTOR_3D new_normal;
/*
**  Sets the normal at this vertex.  The normal will not be good anymore if this
**  or any ajoining vertices are moved or destroyed.
*/
{
  if (!SM_vertex_normal_index_valid(the_surface, vertex_index, normal_index))  
    return(NO);
  
  V3D_COPY(the_surface->vert_list[vertex_index].norm_list[normal_index].normal,
    new_normal);
  the_surface->vert_list[vertex_index].norm_list[normal_index].normal_good = YES;
  return(YES);
}



BOOL
SM_vertex_normal_get(the_surface, vertex_index, normal_index, returned_normal)
SM_Surface_ptr the_surface;
int vertex_index, normal_index;
VECTOR_3D returned_normal;
/*
**  Returns a copy of the surface normal at the_vertex, as the mean sum of
**  surrounding triangles.
**  Returns YES if normal is good.
**  If returns NO, does not affect the value of "returned_normal".
*/
{
/*
**	Parameter validation.
  */
  if (!SM_vertex_normal_index_valid(the_surface, vertex_index, normal_index))  
    return(NO);
  
  return(SM_vertex_normal_sm_get(the_surface, vertex_index, normal_index, returned_normal));
}

BOOL
SM_vertex_normal_sm_get(the_surface, vertex_index, normal_index, returned_normal)
SM_Surface_ptr the_surface;
int vertex_index, normal_index;
VECTOR_3D returned_normal;
/*
**  For INTERNAL use: Makes NO parameter validation.
**  Returns a copy of the surface normal at the_vertex, as the mean sum of
**  surrounding triangles.
**  Returns YES if normal is good.
**  If returns NO, does not affect the value of "returned_normal".
*/
{
  SM_Vertex_normal_ptr the_normal;
  static VECTOR_3D triangle_normal, vertex_normal;
  register int i;
  int * list_ptr;
  
  the_normal = &the_surface->vert_list[vertex_index].norm_list[normal_index];
  
  /* 
  **	the vertex must have at least one triangle to have a normal. 
  **  if tri_ref_list_occ == 0  =>  struct_used = NO...
  */
  if (the_normal->tri_ref_list_occ <= 0)  return(NO);
  
  if (the_normal->normal_good) {
    V3D_COPY(returned_normal, the_normal->normal);
    return(YES);
  }
  
  /* 
  **	Calculate the normal from surrounding triangles and store it. 
  */
  V3D_NULL(vertex_normal);
  
  for (   i = 0,
    list_ptr = the_normal->tri_ref_list;
  i < the_normal->tri_ref_list_occ;
  i++,
    list_ptr++	) 
    
    if (SM_triangle_normal_get(the_surface, *list_ptr, triangle_normal))	
      V3D_ADD(vertex_normal, vertex_normal, triangle_normal)
      else return(NO);
    
    V3D_SCALE(	the_normal->normal, 
      1.0 / (float) the_normal->tri_ref_list_occ, 
      vertex_normal  );
    
    the_normal->normal_good = YES;
    V3D_COPY(returned_normal, the_normal->normal);
    return(YES);
}




/*
**  picked field.
*/

BOOL
SM_vertex_picked_set(the_surface, vertex_index, value)
SM_Surface_ptr the_surface;
int vertex_index;
BOOL value;
/*
**  Assigns the first bit of the picked field
*/
{
  if (!SM_vertex_index_valid(the_surface, vertex_index))  return(NO);
  the_surface->vert_list[vertex_index].picked = 
    ((the_surface->vert_list[vertex_index].picked >> 1) << 1) | (BOOL_STACK) value;
  return(YES);
}

BOOL
SM_vertex_picked_get(the_surface, vertex_index, value)
SM_Surface_ptr the_surface;
int vertex_index;
BOOL * value;
/*
**  Returns YES in value if vertex_index is picked.
*/
{
  if (!SM_vertex_index_valid(the_surface, vertex_index))  return(NO);
  *value = (BOOL) IS_YES(the_surface->vert_list[vertex_index].picked);
  return(YES);
}

void
SM_vertex_picked_push(the_surface)
SM_Surface_ptr the_surface;
/*
**  Shifts the picked register one bit to the left and copies the first value.
*/
{
  SM_FOR_EACH_VERTEX(the_surface)
    vertex_ptr->picked = (vertex_ptr->picked << 1) | IS_YES(vertex_ptr->picked);
  SM_END_FOR_EACH
}

void
SM_vertex_picked_pop(the_surface)
SM_Surface_ptr the_surface;
/*
**  Shifts the picked register to the right.
*/
{
  SM_FOR_EACH_VERTEX(the_surface)
    vertex_ptr->picked = (vertex_ptr->picked >> 1);
  SM_END_FOR_EACH
}

void
SM_vertex_picked_center(the_surface, the_center)
SM_Surface_ptr the_surface;
VECTOR_3D the_center;
/*
**  Returns the center of all picked points.
*/
{
  int number_picked_vert;
  
  V3D_NULL(the_center);
  if (number_picked_vert = SM_vertex_picked_count(the_surface)) {
    
    SM_FOR_EACH_VERTEX(the_surface)
      if ( (BOOL) IS_YES(vertex_ptr->picked))
        V3D_ADD(the_center, the_center, vertex_ptr->point);
      SM_END_FOR_EACH
        
        V3D_SCALE(the_center, 1.0 / (float) number_picked_vert, the_center);
  }
}


int 
SM_vertex_picked_count(the_surface)
SM_Surface_ptr the_surface;
/*
**  Counts and returns the number of picked vertices in "the_surface".
*/
{
  int number_picked = 0;
  
  SM_FOR_EACH_VERTEX(the_surface)
    number_picked += (BOOL) IS_YES(vertex_ptr->picked);
  SM_END_FOR_EACH
    
    return( number_picked );
}





/*
**  normal and tri_ref_list fields.
*/

BOOL
SM_vertex_normal_index_valid(the_surface, vertex_index, normal_index)
SM_Surface_ptr the_surface;
int vertex_index, normal_index;
/*
**  Returns YES if normal_index is a valid normal index of vertex_index.
*/
{
  static SM_Vertex_ptr vertex_ptr;
  
  if (vertex_index < 0 || vertex_index >= the_surface->vert_list_size) return(NO);
  
  vertex_ptr = &the_surface->vert_list[vertex_index];
  if (!vertex_ptr->struct_used) return(NO);
  
  if (normal_index < 0 || normal_index >= vertex_ptr->norm_list_size) return(NO);
  return(vertex_ptr->norm_list[normal_index].struct_used);
}


void
SM_triangle_ref_vert_normal(the_surface, triangle_index, vertex_index, new_normal)
SM_Surface_ptr the_surface;
int triangle_index, vertex_index, new_normal;
/*
**  LOW LEVEL called by SM_vertex_normal_tri_ref level functions.
**  Reassigns the normal_index pointer in triangle_index.  
**  This is used when an edge is squared.
*/
{
  SM_Triangle_ptr the_triangle;
  
  the_triangle = &the_surface->tri_list[triangle_index];
  
  if (the_triangle->vertex1 == vertex_index)
    the_triangle->normal1 = new_normal;
  else if (the_triangle->vertex2 == vertex_index)
    the_triangle->normal2 = new_normal;
  else if (the_triangle->vertex3 == vertex_index)
    the_triangle->normal3 = new_normal;
}




BOOL
SM_vertex_normal_tri_ref_add(the_surface, 
                             vertex_index, 
                             normal_index, 
                             triangle_index) 
                             SM_Surface_ptr the_surface;
int vertex_index, normal_index, triangle_index;
/*
**  LOW LEVEL routine called by SM_vertex_normal level functions.
**  Adds a tri_ref of triangle_index having the normal normal_index to vertex_index.
**  Corresponds to adding triangle_index at the end of the normal_index's
**  tri_ref_list.
**  Counter-references the triangle.
*/
{
  SM_Vertex_normal_ptr the_normal;
  int dummy1, dummy2;
  
  if (!SM_vertex_normal_index_valid(the_surface, vertex_index, normal_index))  
    return(NO);
  
  the_normal = &the_surface->vert_list[vertex_index].norm_list[normal_index];
  
  /*
  **	Check if it already exists.
  */
  if (SM_vertex_normal_index_get( the_surface, 
    vertex_index,
    triangle_index,
    &dummy1, &dummy2 )) return(YES);
  
    /*
    **	Allocate new tri_list memory if necessary.
  */
  if (the_normal->tri_ref_list_occ == the_normal->tri_ref_list_size) {
    the_normal->tri_ref_list_size += SM_TRI_REF_LIST_INC;
    if (the_normal->tri_ref_list) 
      the_normal->tri_ref_list = (int *) 
      realloc(the_normal->tri_ref_list,
      the_normal->tri_ref_list_size * sizeof(int));
    else
      the_normal->tri_ref_list = (int *) malloc(the_normal->tri_ref_list_size * sizeof(int));
  }
  
  /*
  **	Add the reference.
  */
  the_normal->tri_ref_list[the_normal->tri_ref_list_occ] = triangle_index;
  the_normal->tri_ref_list_occ++;
  the_normal->normal_good = NO;
  
  /*
  **	Counter reference the new triangle that uses this normal.
  */
  SM_triangle_ref_vert_normal(the_surface, 
				triangle_index,
        vertex_index, 
        normal_index);
  return(YES);
}



BOOL
SM_vertex_normal_tri_ref_remove(the_surface, 
                                vertex_index, 
                                normal_index,
                                list_index)
                                SM_Surface_ptr the_surface;
int vertex_index, normal_index, list_index;
/*
**  LOW LEVEL routine called by SM_vertex_normal level functions.
**  Removes the list_index tri_ref from the normal_index in vertex_index.
**  If the normal list is not used anymore, deletes it.
**  Does not de_counter_reference the triangles (no need to).
*/
{
  register int i;
  SM_Vertex_ptr the_vertex;
  SM_Vertex_normal_ptr the_normal;
  
  if (!SM_vertex_normal_index_valid(the_surface, vertex_index, normal_index))
    return(NO);
  
  the_vertex = &the_surface->vert_list[vertex_index];
  the_normal = &the_vertex->norm_list[normal_index];
  
  /*
  **	The triangle reference does not exist in the list.
  */
  if ( list_index < 0 || list_index >= the_normal->tri_ref_list_occ )
    return(NO);
  
    /*
    **	Deallocate the list if no references exists to this normal.
  */
  if ( the_normal->tri_ref_list_occ == 1 ) {
    free(the_normal->tri_ref_list);
    
    /*
    **  This normal struct is not used anymore.
    */
    the_normal->tri_ref_list_occ = 0;
    the_normal->tri_ref_list_size = 0;
    the_vertex->norm_list[normal_index].struct_used = NO;
    the_vertex->norm_list[normal_index].tri_ref_list = NULL;
    return(YES);
  }
  
  /*
  **	Slide the list down from list_index on up.
  */
  for (i = list_index; i < the_normal->tri_ref_list_occ - 1; i++)
    the_normal->tri_ref_list[i] = the_normal->tri_ref_list[i + 1];
  
  the_normal->tri_ref_list_occ -= 1;
  the_normal->normal_good = NO;
  
  return(YES);
}



int
SM_vertex_normal_tri_ref_merge(the_surface, vertex_index, normal1, normal2)
SM_Surface_ptr the_surface;
int vertex_index, normal1, normal2;
/*
**  Merges two tri_ref_list.  This is done when an edge is unsquared
**  or two surfaces sharing the same point are joined by a middle
**  triangle.
**  Returns the list_index of the merged list.
**  THERE IS NO NEED TO DE-COUNTER-REFERENCE WHEN DELETING.
*/
{
  SM_Vertex_ptr the_vertex;
  int triangle_ref;
  
  if (!SM_vertex_normal_index_valid(the_surface, vertex_index, normal1) ||
    !SM_vertex_normal_index_valid(the_surface, vertex_index, normal2))  
    return(SM_INDEX_ERROR);
  
  the_vertex = &the_surface->vert_list[vertex_index];
  
  /*
  **	We add to normal1 until the normal2 at the left
  **	has disapeared..
  */
  while (the_vertex->norm_list[normal2].struct_used) {
    triangle_ref = the_vertex->norm_list[normal2].tri_ref_list[0];
    SM_vertex_normal_tri_ref_remove(the_surface,
      vertex_index,
      normal2, 0);
      /*
      **  This will do the counter-referencing.
    */
    SM_vertex_normal_tri_ref_add(the_surface, 
      vertex_index, 
      normal1,
      triangle_ref);
  }
  return(normal1);
}



BOOL
SM_vertex_normal_split(	the_surface,
                       vertex_index,
                       normal_index,
                       edge_index )
                       SM_Surface_ptr the_surface;
int vertex_index, normal_index, edge_index;
/*
**  Splits the tri_ref_list of vertex_index at edge_index into two lists
**  and counter-references the change to the triangles.
**  This edge must have two_triangles to be split.
*/
{
  int dummy, new_normal, list_index, edge1, edge2, triangle_index, next_normal;
  
  if (!SM_vertex_normal_index_valid(the_surface, vertex_index, normal_index) ||
    !SM_edge_index_valid(the_surface, edge_index) ||
    !the_surface->edge_list[edge_index].two_triangles)
    return(NO);
  
  new_normal = SM_vertex_normal_new(the_surface, vertex_index);
  
  /*
  **	Start with one triangle of this edge.
  */
  triangle_index = the_surface->edge_list[edge_index].triangle1;
  
  /*
  **	Loop around until there is no more triangle.
  */
  while (triangle_index != SM_INDEX_ERROR) {
  /*
  **  Remove from old list and add this triangle to new list.
    */
    if (SM_vertex_normal_index_get( the_surface, 
      vertex_index, 
      triangle_index, 
      &dummy, &list_index)    &&
      SM_vertex_normal_tri_ref_remove(the_surface, 
      vertex_index,
      normal_index,
      list_index))
      
      SM_vertex_normal_tri_ref_add(the_surface,
					 vertex_index,
           new_normal,
           triangle_index);
    
    else return(NO);
    
    /*
    **  The next edge will be different from the last.
    */
    SM_edge_two_sides(the_surface, vertex_index, triangle_index, &edge1, &edge2);
    
    edge_index = ((edge1 == edge_index) ? edge2 : edge1);
    
    /*
    **  IN the case where the normal is first split, there is no
    **  division on patches, so check for this case.
    */
    if (edge_index == SM_INDEX_ERROR)   return(YES);
    
    /*
    **  Find the new triangle neighbor with this edge, if it exists.
    **  This triangle must be in the same patch.
    */
    triangle_index = SM_triangle_neighbor_get(the_surface, 
      triangle_index, 
      edge_index);
    
    if (SM_vertex_normal_index_get( the_surface,
      vertex_index,
      triangle_index,
      &next_normal, &dummy) &&
      next_normal != normal_index)
      
      return(YES);
  }
  return(YES);
}




BOOL
SM_vertex_normal_index_get( the_surface, 
                           vertex_index, 
                           triangle_index, 
                           normal_index,
                           list_index)
                           SM_Surface_ptr the_surface;
int vertex_index, triangle_index, *normal_index, *list_index;
/*
**  Returns the normal and tri_ref_list index in vertex_index where triangle_index is.
*/
{
  SM_Vertex_ptr the_vertex;
  register int i, j;
  
  if (!SM_vertex_index_valid(the_surface, vertex_index))  return(NO);
  
  the_vertex = &the_surface->vert_list[vertex_index];
  
  for(i = 0; i < the_vertex->norm_list_size; i++)
    if (the_vertex->norm_list[i].struct_used) 
      for(j = 0; j < the_vertex->norm_list[i].tri_ref_list_occ; j++)
        if (the_vertex->norm_list[i].tri_ref_list[j] == triangle_index) {
          *normal_index = i;
          *list_index = j;
          return(YES);
        }
        return(NO);
}



BOOL
SM_vertex_normal_edge_index_get(the_surface, vertex_index, edge_index, n1, i1, n2, i2)
SM_Surface_ptr the_surface;
int vertex_index, edge_index, *n1, *i1, *n2, *i2;
/*
**  Finds and returns both normal indexes of the edge.
*/
{
  SM_Edge_ptr the_edge;
  
  if (!SM_edge_index_valid(the_surface, edge_index))  return(NO);
  
  the_edge = &the_surface->edge_list[edge_index];
  
  if (!the_edge->two_triangles)  return(NO);
  
  if (!SM_vertex_normal_index_get(the_surface, vertex_index, 
    the_edge->triangle1, n1, i1)) return(NO);
  
  if (!SM_vertex_normal_index_get(the_surface, vertex_index, 
    the_edge->triangle2, n2, i2)) return(NO);
  return(YES);
}



int
SM_vertex_normal_count(the_surface, vertex_index)
SM_Surface_ptr the_surface;
int vertex_index;
/*
**  Counts and returns the number of normals assigned to this vertex.
*/
{
  register int count, i;
  SM_Vertex_ptr the_vertex;
  
  the_vertex = &the_surface->vert_list[vertex_index];
  
  for ( i = 0, count = 0; i < the_vertex->norm_list_size; i++ )
    count += the_vertex->norm_list[i].struct_used;
  return(count);
}


BOOL
SM_vertex_normal_get_first(the_surface, vertex_index, returned_normal)
SM_Surface_ptr the_surface;
int vertex_index;
VECTOR_3D returned_normal;
/*
**  This is only used for displaying shaded edges and vertices 
**  (return the first normal of vertex_index).
*/
{
  SM_Vertex_ptr the_vertex;
  
  if (!SM_vertex_index_valid(the_surface, vertex_index))  return(NO);
  
  the_vertex = &the_surface->vert_list[vertex_index];
  
  SM_FOR_EACH_NORMAL(the_vertex)
    if (SM_vertex_normal_sm_get(the_surface, 
      vertex_index, 
      normal_index, 
      returned_normal))  return(YES);
    SM_END_FOR_EACH
      return(NO);
}



BOOL
SM_vertex_triangle_get_first(the_surface, vertex_index, returned_triangle)
SM_Surface_ptr the_surface;
int vertex_index, * returned_triangle;
/*
**  This is only used for displaying shaded edges and vertices 
**  (return the first triangle of vertex_index).
*/
{
  SM_Vertex_ptr the_vertex;
  
  if (!SM_vertex_index_valid(the_surface, vertex_index))  return(NO);
  
  the_vertex = &the_surface->vert_list[vertex_index];
  
  SM_FOR_EACH_NORMAL(the_vertex)
    *returned_triangle = normal_ptr->tri_ref_list[0];
  return(YES);
  SM_END_FOR_EACH
    return(NO);
}



BOOL
SM_vertex_normal_contiguous( the_surface, vertex_index )
SM_Surface_ptr the_surface;
int vertex_index;
/*
**  Returns YES if the_surface at this point has no holes and is
**  not squared.
*/
{
  int first_triangle, triangle_index, edge1, edge2, edge_index;
  SM_Vertex_normal_ptr the_normal;
  
  if (!SM_vertex_normal_count(the_surface, vertex_index) == 1)
    return(NO);
  
    /*
    **	Find the used normal.
  */
  the_normal = the_surface->vert_list[vertex_index].norm_list;
  while (!the_normal->struct_used)  the_normal++;
  
  /*
  **	This normal must have at least 3 triangles.
  */
  if (the_normal->tri_ref_list_occ <= 2)  return(NO);
  
  /*
  **	Arbitrarily take the first triangle.
  */
  first_triangle = the_normal->tri_ref_list[0];
  triangle_index = first_triangle;
  edge_index = SM_INDEX_ERROR;
  
  /*
  **	Loop until around until there is no more triangle or the first
  **	triangle has been found again.
  */
  while (triangle_index != SM_INDEX_ERROR) {
  /*
  **  The next edge will be different from the last.
    */
    SM_edge_two_sides(the_surface, vertex_index, triangle_index, &edge1, &edge2);
    edge_index = ((edge1 == edge_index) ? edge2 : edge1);
    
    if (edge_index == SM_INDEX_ERROR)   return(NO);
    
    /*
    **  Find the new triangle neighbor with this edge.
    */
    triangle_index = SM_triangle_neighbor_get(the_surface, 
      triangle_index, 
      edge_index);
    if (triangle_index == first_triangle)
      return(YES);
  }
  return(NO);
}



int
SM_vertex_normal_new( the_surface, vertex_index )
SM_Surface_ptr the_surface;
int vertex_index;
/*
**  Returns a new normal structure index.  Allocates if necessary.
*/
{
  SM_Vertex_ptr the_vertex;
  SM_Vertex_normal_ptr ptr_1, ptr_2;
  int new_normal_index, total_size;
  
  if (!SM_vertex_index_valid(the_surface, vertex_index))  return(SM_INDEX_ERROR);
  the_vertex = &the_surface->vert_list[vertex_index];
  
  
  /* Find end of the list (dichotomic search - just a simple solution) */
  {
    int mid, lo, up;
    
    lo = 0;
    up = the_vertex->norm_list_size-1;
    if (up < 0 || (!the_vertex->norm_list[lo].struct_used)) {
      new_normal_index = lo; 
    } else if (the_vertex->norm_list[up].struct_used) {
      new_normal_index = up+1;
    } else {
      while (up != lo+1) {
        mid = lo + ((up-lo) >> 1);
        if (!the_vertex->norm_list[mid].struct_used) {
          up = mid;
        } else {
          lo = mid;
        }
      }
      new_normal_index = up;
    }
  }
  
  /*
  **	Allocate if needed.
  */
  if ( new_normal_index == the_vertex->norm_list_size ) {
    
  /*
  **	If the list exists, increase its size.  If not, allocate an initial list size.
    */
    if (the_vertex->norm_list) { 
      total_size = the_vertex->norm_list_size + SM_NORM_LIST_INC;
      the_vertex->norm_list = (SM_Vertex_normal_ptr) 
				    realloc(the_vertex->norm_list, total_size * sizeof(SM_Vertex_normal));
    } else { 
      total_size = SM_NORM_LIST_INC;
      the_vertex->norm_list = (SM_Vertex_normal_ptr)
				    malloc(total_size * sizeof(SM_Vertex_normal));
    }
    
    /*
    **	Mark unused structures as unused.
    */
    ptr_1 = the_vertex->norm_list + the_vertex->norm_list_size;
    ptr_2 = the_vertex->norm_list + total_size - 1;
    do { ptr_1->struct_used = NO; } while (ptr_1++ != ptr_2);
    
    the_vertex->norm_list_size = total_size;
    
  }
  
  the_vertex->norm_list[new_normal_index].struct_used = YES;
  the_vertex->norm_list[new_normal_index].normal_good = NO;
  the_vertex->norm_list[new_normal_index].tri_ref_list_size = 0;
  the_vertex->norm_list[new_normal_index].tri_ref_list_occ = 0;
  the_vertex->norm_list[new_normal_index].tri_ref_list = NULL;
  return(new_normal_index);
}			



int
SM_vertex_normal_ref_add(the_surface, 
                         vertex_index, 
                         triangle_index, 
                         right_edge, 
                         left_edge)
                         SM_Surface_ptr the_surface;
int vertex_index, triangle_index, right_edge, left_edge;
/*
**  This is called when a new triangle is created.
**  Adds a reference vertex_index -> triangle_index and a counter-reference.
**  Returns the new normal_index in the vertex vertex_index.
**  This function supposes that the edges have just been created.
*/
{
  int new_normal_index, dummy;
  int right_triangle, right_norm_index, left_triangle, left_norm_index;
  
  /*
  **	Parameter validation.
  */
  if (!SM_vertex_index_valid(the_surface, vertex_index) ||
    !SM_edge_index_valid(the_surface, right_edge) ||
    !SM_edge_index_valid(the_surface, left_edge))  {
    return(SM_INDEX_ERROR);
  }
  right_triangle = SM_triangle_neighbor_get( the_surface, 
    triangle_index, 
    right_edge );
  left_triangle = SM_triangle_neighbor_get(  the_surface,
    triangle_index,
    left_edge );
    /*
    **  We look to see what edges are not holes.
    **  If right_edge is not a hole anymore, then add this triangle_index
    **  reference to the same normal_struct as it's right neighbor,
    **  to continue the surface.
  */
  if (right_triangle != SM_INDEX_ERROR) {
    if (!SM_vertex_normal_index_get(the_surface,
      vertex_index,
      right_triangle,
      &right_norm_index,
      &dummy))  return(SM_INDEX_ERROR);
    SM_vertex_normal_tri_ref_add(the_surface,
      vertex_index, 
      right_norm_index, 
      triangle_index);
      /*
      **	If the other edge is also not a hole anymore, we have to merge
      **	these two tri_ref_list because this new triangle joins them together.
    */
    if (left_triangle != SM_INDEX_ERROR) {
      if (!SM_vertex_normal_index_get(the_surface,
        vertex_index,
        left_triangle,
        &left_norm_index,
        &dummy))  {
					   return(SM_INDEX_ERROR);
      }
      /*
      **	There is no need to merge if it is already the same normal_list
      **	(when the patch completely surrounds the vertex.)
      */
      if (left_norm_index != right_norm_index) {
        SM_vertex_normal_tri_ref_merge( the_surface,
          vertex_index, 
          right_norm_index, 
          left_norm_index );
      }
    }
    return(right_norm_index);
  } else 
  /*
  **  If left_edge is not a hole anymore but the right_edge is one,
  **  add this triangle_index reference to the same normal_struct as
  **  it's left neighbor.
  */
  if (left_triangle != SM_INDEX_ERROR) {
    if (!SM_vertex_normal_index_get(the_surface,
      vertex_index,
      left_triangle,
      &left_norm_index,
      &dummy))  return(SM_INDEX_ERROR);
    SM_vertex_normal_tri_ref_add(the_surface,
					 vertex_index,
           left_norm_index,
           triangle_index);
    return(left_norm_index);
  } else {
  /*
  **  Just create a new normal struct and
  **  reference triangle_index.  This means that both the left and right
  **  edges are holes.
    */
    new_normal_index = SM_vertex_normal_new(the_surface, vertex_index);
    SM_vertex_normal_tri_ref_add(the_surface,
					 vertex_index,
           new_normal_index,
           triangle_index);
    return(new_normal_index);
  }
  
}



BOOL
SM_vertex_ref_triangle_remove(the_surface, 
                              vertex_index, 
                              triangle_index)
                              SM_Surface_ptr the_surface;
int vertex_index, triangle_index;
/*
**  Removes the reference to triangle_index, if it exists.
**  Removing this reference may split a patch, if it is neither
**  at the end nor part of a contiguous vertex.
**  Supposes that the edges of triangle_index still exist.
*/
{
  SM_Vertex_ptr the_vertex;
  int list_index, normal_index;
  int right_triangle, left_triangle, left_edge, right_edge;
  BOOL was_contiguous;
  
  /*
  **	When a vertex (vertex_index) is being destroyed, the struct_used test
  **	also stops de-referencing from the triangle being destroyed.
  */
  if (!SM_vertex_index_valid(the_surface, vertex_index) ||
    !SM_triangle_index_valid(the_surface, triangle_index))
    return(NO);
  
  the_vertex = &the_surface->vert_list[vertex_index];
  
  if (!SM_edge_two_sides( the_surface, 
    vertex_index, 
    triangle_index, 
    &left_edge, 
    &right_edge))  return(NO);
  
  right_triangle = SM_triangle_neighbor_get( the_surface, 
    triangle_index, 
    right_edge );
  left_triangle = SM_triangle_neighbor_get(  the_surface,
    triangle_index,
    left_edge );
  
  if (!SM_vertex_normal_index_get( the_surface,
    vertex_index,
    triangle_index,
    &normal_index,
    &list_index ))  return(NO);
  
  was_contiguous = SM_vertex_normal_contiguous(the_surface, vertex_index);
  
  /*
  **	We now see if removing this triangle creates a hole/split.
  **	A triangle removed at the left or right edges (begining or end) does not
  **	create a new hole.  A square edge delimits also at the end of a patch.
  **	Do not split if the surface was continuous.
  **	A test for struct_used is necessary here because the normal (patch) may
  **	have been deleted (no one uses it anymore).
  */
  if (!the_vertex->norm_list[normal_index].struct_used)  return(NO);
  
  if (was_contiguous || 
    right_triangle == SM_INDEX_ERROR ||
    left_triangle == SM_INDEX_ERROR) {
    
    return(SM_vertex_normal_tri_ref_remove( the_surface,
						vertex_index,
            normal_index,
            list_index));
  }
  
  /*
  **	Else, we have to do two splits split.
  */
  SM_vertex_normal_split( the_surface, 
    vertex_index, 
    normal_index, 
    right_edge);
  
    /*
    **	The last split created a new normal, so find where our
    **	triangle's normal is.
  */
  SM_vertex_normal_index_get(	the_surface,
				vertex_index,
        triangle_index,
        &normal_index,
        &list_index);
  SM_vertex_normal_split( the_surface, 
    vertex_index, 
    normal_index, 
    left_edge	);
  
    /*
    **	The last split created a new normal, so find where our
    **	triangle's normal is.
  */
  SM_vertex_normal_index_get(	the_surface,
				vertex_index,
        triangle_index,
        &normal_index,
        &list_index);
  if (!SM_vertex_normal_tri_ref_remove(the_surface,
    vertex_index,
    normal_index,
    list_index))  return(NO);
  
  return(YES);
}



















/*
-------------------------------   SM_TRIANGLE  ---------------------------------
*/

BOOL
SM_triangle_index_valid(the_surface, triangle_index)
SM_Surface_ptr the_surface;
int triangle_index;
/*
**  Returns YES if triangle_index is a valid triangle index.
*/
{
  return( triangle_index >= 0				&& 
    triangle_index < the_surface->tri_list_size	&&
    the_surface->tri_list[triangle_index].struct_used );
}



void
SM_triangle_write(fil, the_surface, triangle_index) 
FILE *fil;
SM_Surface_ptr the_surface;
int triangle_index;
/*
**  Writes the content of the_triangle, in a list format, on the unix
**  file *fil.  Before doing this the first time, all 'vertex_id' fields must be
**  renumbered with SM_vertex_id_rename.
*/
{
  SM_Triangle_ptr the_triangle;
  the_triangle = &the_surface->tri_list[triangle_index];
  
  fprintf(fil, "(%d,%d,%d,%d", 
    the_surface->vert_list[the_triangle->vertex1].vertex_id,
    the_surface->vert_list[the_triangle->vertex2].vertex_id,
    the_surface->vert_list[the_triangle->vertex3].vertex_id,
    the_materials.mat_list[the_triangle->material_index].material_id
    
    );
  fprintf(fil,")\n");	
  
}




BOOL
SM_triangle_ref_vertex(the_surface, triangle_index, v1_index, v2_index, v3_index)
SM_Surface_ptr the_surface;
int triangle_index, v1_index, v2_index, v3_index;
/*
**  Assigns the three vertices and three edges to triangle_index's triangle.
**  Makes sure that the surface normales across the edges are coherent.
**  Returns YES if all reference was added.
*/
{
  int edge1, edge2, edge3;
  SM_Triangle_ptr the_triangle;
  
  if (!SM_triangle_index_valid(the_surface, triangle_index))  return(NO);
  
  the_triangle = &the_surface->tri_list[triangle_index];
  
  if (!SM_vertex_index_valid(the_surface, v1_index) ||
    !SM_vertex_index_valid(the_surface, v2_index) ||
    !SM_vertex_index_valid(the_surface, v3_index))	return(NO);
  
    /*
    **	Create (or reuse existing) edges first.
  */
  if ((edge1 = SM_edge_new(the_surface, triangle_index, v1_index, v2_index)) ==
    SM_INDEX_ERROR)	return(NO);
  
  if ((edge2 = SM_edge_new(the_surface, triangle_index, v2_index, v3_index)) ==
    SM_INDEX_ERROR) {
    SM_edge_ref_triangle_remove(the_surface, edge1, triangle_index);
    return(NO);
  }
  
  if ((edge3 = SM_edge_new(the_surface, triangle_index, v3_index, v1_index)) ==
    SM_INDEX_ERROR) {
    SM_edge_ref_triangle_remove(the_surface, edge1, triangle_index);
    SM_edge_ref_triangle_remove(the_surface, edge2, triangle_index);
    return(NO);
  }
  
  the_triangle->vertex1 = v1_index;
  the_triangle->vertex2 = v2_index;
  the_triangle->vertex3 = v3_index;
  
  the_triangle->edge1 = edge1;
  the_triangle->edge2 = edge2;
  the_triangle->edge3 = edge3;
  
  /*
  **	Tell the vertices that I'm using them and find out what normals	
  **	this triangle is using.
  **  Since, by the last line, this triangle is now part of a new coherent patch,
  **	we also can give the edges in ccw order for each vertex.
  */
  the_triangle->normal1 = 
    SM_vertex_normal_ref_add(the_surface, v1_index, triangle_index, edge1, edge3);
  the_triangle->normal2 = 
    SM_vertex_normal_ref_add(the_surface, v2_index, triangle_index, edge2, edge1);
  the_triangle->normal3 = 
    SM_vertex_normal_ref_add(the_surface, v3_index, triangle_index, edge3, edge2);
  
    /*
    **	Make sure that the connecting patches created by the creation of
    **	this triangle are coherent.
  */
  SM_triangle_normal_coherent(the_surface, triangle_index);
  return(YES);
}


void
SM_triangle_allocate(the_surface)
SM_Surface_ptr the_surface;
/*
**  Adds (allocates) more memory to store triangles.
*/
{
  register SM_Triangle_ptr ptr_1, ptr_2;
  int total_size;
  
  /*
  **	If the list exists, increase its size.  If not, allocate an initial list size.
  */
  if (the_surface->tri_list) { 
    total_size = the_surface->tri_list_size * SM_TRI_LIST_INC;
    the_surface->tri_list = (SM_Triangle_ptr) 
      realloc(the_surface->tri_list, total_size * sizeof(SM_Triangle));
  } else { 
    total_size = SM_TRI_LIST_INIT;
    the_surface->tri_list = (SM_Triangle_ptr)
      malloc(total_size * sizeof(SM_Triangle));
  }
  
  /*
  **	Mark unused structures as unused.
  */
  ptr_1 = the_surface->tri_list + the_surface->tri_list_size;
  ptr_2 = the_surface->tri_list + total_size - 1;
  do { ptr_1->struct_used = NO; } while (ptr_1++ != ptr_2);
  
  the_surface->tri_list_size = total_size;
}



int
SM_triangle_new(the_surface, v1_index, v2_index, v3_index)
SM_Surface_ptr the_surface;
int v1_index, v2_index, v3_index;
/*
**  Creates a new triangles made out of vertices v1_index, v2_index and v3_index.
**  If v1_index, v2_index and v3_index are all SM_INDEX_ERROR, they must be 
**  set immediately after this function return with SM_triangle_ref_vertex.  
**  This is useful when reading a triangle from a file.
**  Returns the index to the newly created triangle.
*/
{
  SM_Triangle_ptr new_triangle;
  int new_triangle_index;
  BOOL t1, t2, t3;
  int index1, index2, index3;
  
  new_triangle_index = Get_From_Stack(the_surface, SM_TRIANGLE);
  if (new_triangle_index == SM_INDEX_ERROR) {
    new_triangle_index = the_surface->current_tri_list_size;
  }
  
  /* 
  **	Allocate new memory if necessary. 
  */
  if (new_triangle_index == the_surface->tri_list_size) 
    SM_triangle_allocate(the_surface);
  
  new_triangle = &the_surface->tri_list[new_triangle_index];
  
  /* 
  **	initialise new triangle. 
  */
  new_triangle->struct_used = YES;
  
  new_triangle->vertex1 = SM_INDEX_ERROR;
  new_triangle->vertex2 = SM_INDEX_ERROR;
  new_triangle->vertex3 = SM_INDEX_ERROR;
  
  new_triangle->normal1 = SM_INDEX_ERROR;
  new_triangle->normal2 = SM_INDEX_ERROR;
  new_triangle->normal3 = SM_INDEX_ERROR;
  
  new_triangle->edge1 = SM_INDEX_ERROR;
  new_triangle->edge2 = SM_INDEX_ERROR;
  new_triangle->edge3 = SM_INDEX_ERROR;
  
  /*
  **	If the vertex indices are == SM_INDEX_ERROR, we assume that the
  **	caller will call this function again to give valid index values.
  */
  if (v1_index != SM_INDEX_ERROR &&
    v2_index != SM_INDEX_ERROR &&
    v3_index != SM_INDEX_ERROR)
    
    /*
    **	If vertex assignments were not good, delete this new triangle
    **	and return an error.
    */
    if (!SM_triangle_ref_vertex(the_surface, 
      new_triangle_index, 
      v1_index, 
      v2_index, 
      v3_index)) {
      
      SM_triangle_dispose(the_surface, new_triangle_index);
      return(SM_INDEX_ERROR);
    }
    
    new_triangle->normal_good = NO;
    new_triangle->picked = STACK_NO;
    new_triangle->sm_flag = NO;
    new_triangle->user_flag = NO;
    new_triangle->triangle_id = new_triangle_index;
    new_triangle->region_number = 0;
    new_triangle->region_list = NULL;
    
    /*
    **	Looks across its 3 edges to see which material is the most
    **	popular among its neighbors, and take this as its own.  If no material is found,
    **	by default, the gl default material 0 will automatically be used.
    */
    new_triangle->material_index = SM_INDEX_ERROR;
    
    t1 = SM_triangle_material_get(the_surface,
      SM_triangle_neighbor_get(the_surface, new_triangle_index, new_triangle->edge1),
      &index1);
    t2 = SM_triangle_material_get(the_surface,
      SM_triangle_neighbor_get(the_surface, new_triangle_index, new_triangle->edge2),
      &index2);
    t3 = SM_triangle_material_get(the_surface,
      SM_triangle_neighbor_get(the_surface, new_triangle_index, new_triangle->edge3),
      &index3);
    
    if (t1 && t2 && (index1 == index2)) 
      SM_triangle_material_set(the_surface, new_triangle_index, index1);
    else
      if (t1 && t3 && (index1 == index3)) 
        SM_triangle_material_set(the_surface, new_triangle_index, index1);
      else
        if (t2 && t3 && (index2 == index3))
          SM_triangle_material_set(the_surface, new_triangle_index, index2);
        else 
          if (t1) SM_triangle_material_set(the_surface, new_triangle_index, index1);
          else 
            if (t2) SM_triangle_material_set(the_surface, new_triangle_index, index2);
            else 
              if (t3) SM_triangle_material_set(the_surface, new_triangle_index, index3);
              the_surface->current_tri_list_size += 1;
              return(new_triangle_index);
}


BOOL
SM_triangle_dispose(the_surface, triangle_index)
SM_Surface_ptr the_surface;
int triangle_index;
/*
**  Removes the triangle from the_surface, including all
**  back references from its 3 vertices.
*/
{
  SM_Triangle_ptr the_triangle;
  extern void SM_region_list_triangle_rem(SM_Surface_ptr, int);
  
  /*
  **	Parameter validation.
  */
  if (!SM_triangle_index_valid(the_surface, triangle_index))  return(NO);
  
  the_triangle = &the_surface->tri_list[triangle_index];
  if (the_triangle->region_number != 0) {
    SM_region_list_triangle_rem(the_surface, triangle_index);
    free(the_triangle->region_list);
  }
  SM_vertex_ref_triangle_remove(  the_surface, 
    the_triangle->vertex1, 
    triangle_index);
  SM_vertex_ref_triangle_remove(  the_surface, 
    the_triangle->vertex2, 
    triangle_index);
  SM_vertex_ref_triangle_remove(  the_surface,
    the_triangle->vertex3, 
    triangle_index);
  
  SM_edge_ref_triangle_remove(the_surface, the_triangle->edge1, triangle_index);
  SM_edge_ref_triangle_remove(the_surface, the_triangle->edge2, triangle_index);
  SM_edge_ref_triangle_remove(the_surface, the_triangle->edge3, triangle_index);
  
  the_triangle->struct_used = NO;
  Put_In_Stack(the_surface, triangle_index, SM_TRIANGLE);
  the_surface->current_tri_list_size -= 1;
  return(YES);
}

BOOL
SM_triangle_vertex_index_get(the_surface, triangle_index, v1, v2, v3)
SM_Surface_ptr the_surface;
int triangle_index, *v1, *v2, *v3;
{
  if (!SM_triangle_index_valid(the_surface, triangle_index))	return(NO);
  
  *v1 = the_surface->tri_list[triangle_index].vertex1;
  *v2 = the_surface->tri_list[triangle_index].vertex2;
  *v3 = the_surface->tri_list[triangle_index].vertex3;
  return(YES);
}

#define REGION_TRIANGLE_INCREMENT 10
void SM_triangle_region_add(the_surface,  triangle_index, the_region_index)
SM_Surface_ptr the_surface;
int            triangle_index;
int            the_region_index;
{
  SM_Triangle_ptr the_triangle;
  
  the_triangle = &(the_surface->tri_list[triangle_index]);
  if (the_triangle->region_number == 0) {
    the_triangle->region_list = (int *)malloc(REGION_TRIANGLE_INCREMENT*sizeof(int));
  }
  else {
    if ((the_triangle->region_number % REGION_TRIANGLE_INCREMENT) == 0) {
      the_triangle->region_list = (int *)realloc(the_triangle->region_list, (the_triangle->region_number + REGION_TRIANGLE_INCREMENT)*sizeof(int));
    }
  }
  the_triangle->region_list[the_triangle->region_number] = the_region_index;
  the_triangle->region_number += 1;
}

void SM_triangle_region_rem(the_surface,  triangle_index, the_region_index)
SM_Surface_ptr     the_surface;
int                triangle_index;
int                the_region_index;
{
  SM_Triangle_ptr      the_triangle;
  int                  i, j;
  
  the_triangle = &(the_surface->tri_list[triangle_index]);
  for (i=0; i<the_triangle->region_number; i++) {
    if (the_triangle->region_list[i] == the_region_index) {
      the_triangle->region_number -= 1;
      for (j=i; j<the_triangle->region_number; j++) {
        the_triangle->region_list[j] = the_triangle->region_list[j+1];
      }
      return;
    }
  }
}

void SM_region_triangle_rem(the_surface,  triangle_index, the_region_index)
SM_Surface_ptr     the_surface;
int                triangle_index;
int                the_region_index;
{
  SM_Triangle_ptr      the_triangle;
  int                  i, j;
  int                  v1, v2, v3, e1, e2, e3, voisin1, voisin2, voisin3;
  SM_Region_Edge_ptr   temp_edge_list, new_edge_list;
  int                  temp_edge_list_size;
  SM_Region_ptr        the_region;
  SM_Region_Vertex_ptr temp_vertex_list, new_vertex_list;
  int                  temp_vertex_list_size;
  SM_Vertex_ptr        vertex1, vertex2, vertex3;
  int                  *list_ptr;
  BOOL                 Clean1, Clean2, Clean3, Edge1, Edge2, Edge3;
  SM_Region_List_ptr   the_region_list;
  
  the_region_list = (SM_Region_List_ptr)the_surface->the_region_list;
  the_triangle = &(the_surface->tri_list[triangle_index]);
  the_region = &(the_region_list->region_list[the_region_index]);
  SM_triangle_edge_index_get(the_surface, triangle_index, &e1, &e2, &e3);
  voisin1 = SM_triangle_neighbor_get(the_surface, triangle_index, e1);
  voisin2 = SM_triangle_neighbor_get(the_surface, triangle_index, e2);
  voisin3 = SM_triangle_neighbor_get(the_surface, triangle_index, e3);
  Edge1 = SM_region_triangle_valid(the_region_list, the_region_index, voisin1);
  Edge2 = SM_region_triangle_valid(the_region_list, the_region_index, voisin2);
  Edge3 = SM_region_triangle_valid(the_region_list, the_region_index, voisin3);
  if (!Edge1 || !Edge2 || !Edge3) {
    new_edge_list = temp_edge_list = (SM_Region_Edge_ptr)malloc(the_region->region_edge_list_size*sizeof(SM_Region_Edge));
    temp_edge_list_size = the_region->region_edge_list_size;
    SM_FOR_EACH_REGION_EDGE(the_region)
      if ((*region_edge_ptr != e1) && 
        (*region_edge_ptr != e2) && 
        (*region_edge_ptr != e3) ) {
        *temp_edge_list = *region_edge_ptr;
        temp_edge_list++;
      }
      else
        if ((*region_edge_ptr == e1) && (Edge1)) {		    
          *temp_edge_list = *region_edge_ptr;
          temp_edge_list++;
        }
        else
          if ((*region_edge_ptr == e2) && (Edge2)) {		    
            *temp_edge_list = *region_edge_ptr;
            temp_edge_list++;
          }
          else
            if ((*region_edge_ptr == e3) && (Edge3)) {		    
              *temp_edge_list = *region_edge_ptr;
              temp_edge_list++;
            }
            else {
              temp_edge_list_size -= 1;
            }
            SM_END_FOR_EACH
              free(the_region->region_edge_list);
            the_region->region_edge_list = new_edge_list;
            the_region->region_edge_list_size = temp_edge_list_size;
  }
  SM_triangle_vertex_index_get(the_surface, triangle_index, &v1, &v2, &v3);
  vertex1 = &(the_surface->vert_list[v1]);
  vertex2 = &(the_surface->vert_list[v2]);
  vertex3 = &(the_surface->vert_list[v3]);
  Clean1 = YES;
  SM_FOR_EACH_NORMAL(vertex1)
    for (   j = 0,
      list_ptr = normal_ptr->tri_ref_list;
    j < normal_ptr->tri_ref_list_occ;
    j++,
      list_ptr++   ) {
      if (*list_ptr != triangle_index) {
        if (SM_region_triangle_valid(the_region_list, the_region_index, *list_ptr)) {
          Clean1 = NO;
          break;
        }
      }
    }
    SM_END_FOR_EACH
      Clean2 = YES;
    SM_FOR_EACH_NORMAL(vertex2)
      for (   j = 0,
        list_ptr = normal_ptr->tri_ref_list;
      j < normal_ptr->tri_ref_list_occ;
      j++,
        list_ptr++   ) {
        if (*list_ptr != triangle_index) {
          if (SM_region_triangle_valid(the_region_list, the_region_index, *list_ptr)) {
            Clean2 = NO;
            break;
          }
        }
      }
      SM_END_FOR_EACH
        Clean3 = YES;
      SM_FOR_EACH_NORMAL(vertex3)
        for (   j = 0,
          list_ptr = normal_ptr->tri_ref_list;
        j < normal_ptr->tri_ref_list_occ;
        j++,
          list_ptr++   ) {
          if (*list_ptr != triangle_index) {
            if (SM_region_triangle_valid(the_region_list, the_region_index, *list_ptr)) {
              Clean3 = NO;
              break;
            }
          }
        }
        SM_END_FOR_EACH
          if (Clean1 || Clean2 || Clean3) {
            new_vertex_list = temp_vertex_list = (SM_Region_Vertex_ptr)malloc(the_region->region_vertex_list_size*sizeof(SM_Region_Vertex));
            temp_vertex_list_size = the_region->region_vertex_list_size;
            SM_FOR_EACH_REGION_VERTEX(the_region)
              if ((region_vertex_ptr->id != v1) && (region_vertex_ptr->id != v2) && (region_vertex_ptr->id != v3)) {
                *temp_vertex_list = *region_vertex_ptr;
                temp_vertex_list++;
              }
              else 
                if ((region_vertex_ptr->id == v1) && !(Clean1)) {
                  *temp_vertex_list = *region_vertex_ptr;
                  temp_vertex_list++;
                }
                else
                  if ((region_vertex_ptr->id == v2) && !(Clean2)) {
                    *temp_vertex_list = *region_vertex_ptr;
                    temp_vertex_list++;
                  }
                  else 
                    if ((region_vertex_ptr->id == v3) && !(Clean3)) {
                      *temp_vertex_list = *region_vertex_ptr;
                      temp_vertex_list++;
                    }
                    else temp_vertex_list_size -= 1;
                    SM_END_FOR_EACH
                      free(the_region->region_vertex_list);
                    the_region->region_vertex_list = new_vertex_list;
                    the_region->region_vertex_list_size = temp_vertex_list_size;
          }
          for (i=0; i<the_region->region_tri_list_size; i++) {
            if (the_region->region_tri_list[i] == triangle_index) {
              the_region->region_tri_list_size -= 1;
              for (j=i; j<the_region->region_tri_list_size; j++) {
                the_region->region_tri_list[j] = the_region->region_tri_list[j+1];
              }
              return;
            }
          }
}

void SM_region_list_triangle_rem(the_surface, triangle_index)
SM_Surface_ptr     the_surface;
int                triangle_index;
{
  SM_Triangle_ptr the_triangle;
  int             i;
  
  if (the_surface->the_region_list == NULL) return;
  the_triangle = &(the_surface->tri_list[triangle_index]);
  for (i=0; i<the_triangle->region_number; i++) {
    SM_region_triangle_rem(the_surface, triangle_index, the_triangle->region_list[i]);
  }    
}

BOOL
SM_triangle_region_number_set(the_surface, triangle_index, mode)
SM_Surface_ptr the_surface;
int            triangle_index;
int            mode;
{
  if (!SM_triangle_index_valid(the_surface, triangle_index)) {
    return(NO);
  }
  
  if (mode == ADD) {
    the_surface->tri_list[triangle_index].region_number += 1;
  }
  else {
    the_surface->tri_list[triangle_index].region_number -= 1;
  }
  return(YES);   
}

int
SM_triangle_region_number_get(the_surface, triangle_index)
SM_Surface_ptr the_surface;
int            triangle_index;
{
  return(the_surface->tri_list[triangle_index].region_number);    
}


BOOL
SM_triangle_normal_index_get(the_surface, triangle_index, v1, v2, v3)
SM_Surface_ptr the_surface;
int triangle_index, *v1, *v2, *v3;
{
  if (!SM_triangle_index_valid(the_surface, triangle_index))	return(NO);
  
  *v1 = the_surface->tri_list[triangle_index].normal1;
  *v2 = the_surface->tri_list[triangle_index].normal2;
  *v3 = the_surface->tri_list[triangle_index].normal3;
  return(YES);
}

BOOL
SM_triangle_edge_index_get(the_surface, triangle_index, v1, v2, v3)
SM_Surface_ptr the_surface;
int triangle_index, *v1, *v2, *v3;
{
  if (!SM_triangle_index_valid(the_surface, triangle_index))	return(NO);
  
  *v1 = the_surface->tri_list[triangle_index].edge1;
  *v2 = the_surface->tri_list[triangle_index].edge2;
  *v3 = the_surface->tri_list[triangle_index].edge3;
  return(YES);
}



/*
**  Flag field.
*/

BOOL
SM_triangle_flag_set(the_surface, triangle_index, value)
SM_Surface_ptr the_surface;
int triangle_index;
BOOL value;
/*
**  This sets the value of the "all purpose" flag of triangle_index.
*/
{
  if (!SM_triangle_index_valid(the_surface, triangle_index))	return(NO);
  the_surface->tri_list[triangle_index].user_flag = value;
  return(YES);
}

BOOL
SM_triangle_flag_get(the_surface, triangle_index, value)
SM_Surface_ptr the_surface;
int triangle_index;
BOOL * value;
/*
**  Get the value of the "all purpose" flag of triangle_index.
*/
{
  if (!SM_triangle_index_valid(the_surface, triangle_index))	return(NO);
  *value = the_surface->tri_list[triangle_index].user_flag;
  return(YES);
}


/*
**  Picked field.
*/

BOOL
SM_triangle_picked_set(the_surface, triangle_index, value)
SM_Surface_ptr the_surface;
int triangle_index;
BOOL value;
{
  if (!SM_triangle_index_valid(the_surface, triangle_index))	return(NO);
  the_surface->tri_list[triangle_index].picked = 
    ((the_surface->tri_list[triangle_index].picked >> 1) << 1) | (BOOL_STACK) value;
  return(YES);
}

BOOL
SM_triangle_picked_get(the_surface, triangle_index, value)
SM_Surface_ptr the_surface;
int triangle_index;
BOOL * value;
/*
**  Returns YES if triangle_index is picked.
*/
{
  if (!SM_triangle_index_valid(the_surface, triangle_index))	return(NO);
  *value = (BOOL) IS_YES(the_surface->tri_list[triangle_index].picked);
  return(YES);
}


void
SM_triangle_picked_push(the_surface)
SM_Surface_ptr the_surface;
{
  SM_FOR_EACH_TRIANGLE(the_surface)
    triangle_ptr->picked = (triangle_ptr->picked << 1) | 
    IS_YES(triangle_ptr->picked);
  SM_END_FOR_EACH
}


void
SM_triangle_picked_pop(the_surface)
SM_Surface_ptr the_surface;
{
  SM_FOR_EACH_TRIANGLE(the_surface)
    triangle_ptr->picked = triangle_ptr->picked >> 1;
  SM_END_FOR_EACH
}


BOOL
SM_triangle_picked_triangulate(the_surface)
SM_Surface_ptr the_surface;
/*
**  Triangulates the picked points in the_surface.
**  FOR NOW, works only for 3 points.
**  Returns YES if successful.
*/
{
  SM_Vertex_ptr list_ptr;
  int vert[3], vert_count, vert_number;
  
  if (SM_vertex_picked_count(the_surface) != 3) return(NO);
  
  vert_count = 0;
  vert_number = 0;
  list_ptr = the_surface->vert_list; 
  while (vert_count < 3) {
    if (list_ptr->struct_used && (BOOL) IS_YES(list_ptr->picked)) 
      vert[vert_count++] = vert_number;
    list_ptr++;
    vert_number++;
  }
  return(SM_triangle_new(the_surface, vert[0], vert[1], vert[2]) != SM_INDEX_ERROR);
}



/*
**  Material properties field.
*/

BOOL
SM_triangle_material_set(the_surface, triangle_index, index)
SM_Surface_ptr the_surface;
int triangle_index;
int index;
/*
**  Sets the material name of triangle_index.  
**  Sets the index reference
**  pointing to the material list of the_surface.
*/
{
  if (!SM_triangle_index_valid(the_surface, triangle_index))	return(NO);
  
  the_surface->tri_list[triangle_index].material_index = index;
  return(YES);
}

BOOL
SM_triangle_material_get(the_surface, triangle_index, index)
SM_Surface_ptr the_surface;
int triangle_index;
int *index;
/*
**  Gets the material name of triangle_index.  
*/
{
  if (!SM_triangle_index_valid(the_surface, triangle_index))	return(NO);
  *index = the_surface->tri_list[triangle_index].material_index;
  return(YES);
}

int
SM_triangle_neighbor_get(the_surface, triangle_index, edge_index)
SM_Surface_ptr the_surface;
int triangle_index, edge_index;
/*
**  Returns the other triangle that uses edge_index.
*/
{
  SM_Edge_ptr the_edge;
  
  if (!SM_triangle_index_valid(the_surface, triangle_index) ||
    !SM_edge_index_valid(the_surface, edge_index))  return(SM_INDEX_ERROR);
  
  the_edge = &the_surface->edge_list[edge_index];
  
  if (!the_edge->two_triangles)  return(SM_INDEX_ERROR);
  
  if (the_edge->triangle1 == triangle_index)  return(the_edge->triangle2);
  if (the_edge->triangle2 == triangle_index)  return(the_edge->triangle1);
  return(SM_INDEX_ERROR);
}




/*
**  normal field.
*/

BOOL
SM_triangle_normal_set(the_surface, triangle_index, new_normal)
SM_Surface_ptr the_surface;
int triangle_index;
VECTOR_3D new_normal;
/*
**  Override the default normal calculation of the triangle and set the 
**  value of the normal.
**  This value must be reset if one of its vertices is moved.
*/
{
  if (!SM_triangle_index_valid(the_surface, triangle_index))  return(NO);
  
  V3D_COPY(the_surface->tri_list[triangle_index].normal, new_normal);
  the_surface->tri_list[triangle_index].normal_good = YES;
  return(YES);
}


BOOL
SM_triangle_normal_get(the_surface, triangle_index, returned_normal)
SM_Surface_ptr the_surface;
int triangle_index;
VECTOR_3D returned_normal;
/*
**  Returns a copy of the normal of the_triangle.
**  Returns YES if all is ok.
*/
{
  SM_Triangle_ptr the_triangle;
  VECTOR_3D v1, v2;
  
  /*
  **	Parameter validation.
  */
  if (!SM_triangle_index_valid(the_surface, triangle_index))  return(NO);
  
  the_triangle = &the_surface->tri_list[triangle_index];
  
  if (the_triangle->normal_good) {
    V3D_COPY(returned_normal, the_triangle->normal);
    return(YES);
  }
  
  /*
  **	Calculate the normal and store it.
  */
  V3D_SUB(v1, 
    the_surface->vert_list[the_triangle->vertex1].point, 
    the_surface->vert_list[the_triangle->vertex2].point);
  V3D_SUB(v2, 
    the_surface->vert_list[the_triangle->vertex3].point, 
    the_surface->vert_list[the_triangle->vertex2].point);
  V3D_CROSS(returned_normal, v2, v1);
  if (V3D_NORM(returned_normal) < SM_EPSILON) 
    return(NO);
  V3D_NORMALIZE(returned_normal, returned_normal);
  V3D_COPY(the_triangle->normal, returned_normal);
  the_triangle->normal_good = YES;
  return(YES);
}



BOOL
SM_triangle_normal_flip(the_surface, triangle_index)
SM_Surface_ptr the_surface;
int triangle_index;
/*
**  Flips the normal (it just reorients the vector and edge pointers).
**  Sets normal_good to NO.
**  It sets the user_flag to YES if triangle_index's normal has been flipped.
*/
{
  SM_Triangle_ptr tri_ptr;
  int temp, tri_neighbor;
  
  if (!SM_triangle_index_valid(the_surface, triangle_index))  return(NO);
  tri_ptr = &the_surface->tri_list[triangle_index];
  
  /*
  **	Flip this normal.
  */
  temp = tri_ptr->vertex1;  tri_ptr->vertex1 = tri_ptr->vertex3;  
  tri_ptr->vertex3 = temp;
  temp = tri_ptr->normal1;  tri_ptr->normal1 = tri_ptr->normal3;  
  tri_ptr->normal3 = temp;
  temp = tri_ptr->edge1;    tri_ptr->edge1 = tri_ptr->edge2;      
  tri_ptr->edge2 = temp;
  
  /*
  **	Vertex normals are not good anymore.
  */
  tri_ptr->normal_good = NO;
  the_surface->vert_list[tri_ptr->vertex1].
    norm_list[tri_ptr->normal1].normal_good = NO;
  the_surface->vert_list[tri_ptr->vertex2].
    norm_list[tri_ptr->normal2].normal_good = NO;
  the_surface->vert_list[tri_ptr->vertex3].
    norm_list[tri_ptr->normal3].normal_good = NO;
  
  tri_ptr->user_flag = YES;
  
  /*
  **	Check all three neighbors' normals.
  */
  tri_neighbor = SM_edge_neighbor_get(the_surface, tri_ptr->edge1, triangle_index);
  if (tri_neighbor != SM_INDEX_ERROR && 
    !SM_edge_normal_contiguous(the_surface, tri_ptr->edge1))
    
    SM_triangle_normal_flip(the_surface, tri_neighbor);
  
  tri_neighbor = SM_edge_neighbor_get(the_surface, tri_ptr->edge2, triangle_index);
  if (tri_neighbor != SM_INDEX_ERROR && 
    !SM_edge_normal_contiguous(the_surface, tri_ptr->edge2))
    
    SM_triangle_normal_flip(the_surface, tri_neighbor);
  
  tri_neighbor = SM_edge_neighbor_get(the_surface, tri_ptr->edge3, triangle_index);
  if (tri_neighbor != SM_INDEX_ERROR && 
    !SM_edge_normal_contiguous(the_surface, tri_ptr->edge3))
    
    SM_triangle_normal_flip(the_surface, tri_neighbor);
  return(YES);
}


void
SM_triangle_normal_direct(the_surface, triangle_index, direction, count_YES, count_NO)
SM_Surface_ptr the_surface;
int triangle_index, *count_YES, *count_NO;
BOOL direction;
/*
**  Passes through all the triangles to count the normals that are in
**  the direction == YES:
**
**	truth table		direction   contiguous	 next direction
**	===========		    0		0		1
**				    0		1		0
**				    1		0		0
**				    1		1		1
*/
{
  SM_Triangle_ptr tri_ptr;
  
  if (!SM_triangle_index_valid(the_surface, triangle_index))	return;
  
  tri_ptr = &the_surface->tri_list[triangle_index];
  
  /*
  **	If this triangle already checked.
  */
  if (tri_ptr->sm_flag)  return;
  tri_ptr->sm_flag = YES;
  *count_YES += direction;
  *count_NO += !direction;
  
  SM_triangle_normal_direct(	the_surface,
				SM_edge_neighbor_get(the_surface, 
        tri_ptr->edge1, 
        triangle_index),
        direction == SM_edge_normal_contiguous(the_surface, 
        tri_ptr->edge1), 
        count_YES, count_NO );
  
  SM_triangle_normal_direct(	the_surface,
				SM_edge_neighbor_get(the_surface, 
        tri_ptr->edge2, 
        triangle_index),
        direction == SM_edge_normal_contiguous(the_surface, 
        tri_ptr->edge2),
        count_YES, count_NO );
  
  SM_triangle_normal_direct(	the_surface,
				SM_edge_neighbor_get(the_surface, 
        tri_ptr->edge3, 
        triangle_index),
        direction == SM_edge_normal_contiguous(the_surface, 
        tri_ptr->edge3),
        count_YES, count_NO );
}


BOOL
SM_triangle_normal_coherent(the_surface, triangle_index)
SM_Surface_ptr the_surface;
int triangle_index;
/*
**  Orients the normals of this triangle and all it's neighbors in
**  the same fashion.
**  Works only with triangle and edge information, no need for the
**  vertices.
*/
{
  SM_Triangle_ptr tri_ptr;
  int count_same, count_different;
  
  if (!SM_triangle_index_valid(the_surface, triangle_index))  return(NO);
  
  /*
  **	Clear all triangle sm_flags.
  */
  SM_FOR_EACH_TRIANGLE(the_surface)
    triangle_ptr->sm_flag = NO;
  SM_END_FOR_EACH
    
  /*	
  **	See which direction is most popular (the number "count_same" of triangles
  **	oriented the same as this one).
  */
  
  count_same = 0;
  count_different = 0;
  SM_triangle_normal_direct(	the_surface, 
				triangle_index, 
        YES,	
        &count_same, 
        &count_different    );
  
        /*
        **	If triangle_index's normal is different from the majority of its
        **	neighbors, flip it.
        **	Otherwise, see if some of its 3 neighbors have different normals from
        **	triangle_normal, caused by the gaps filled by the adjuction of it.
  */
  
  if (count_same <= count_different)
    SM_triangle_normal_flip(the_surface, triangle_index);
  else {
    tri_ptr = &the_surface->tri_list[triangle_index];
    
    if (!SM_edge_normal_contiguous(the_surface, tri_ptr->edge1))
      SM_triangle_normal_flip(the_surface, 
      SM_edge_neighbor_get(the_surface, 
      tri_ptr->edge1,
      triangle_index));
    
    if (!SM_edge_normal_contiguous(the_surface, tri_ptr->edge2))
      SM_triangle_normal_flip(the_surface, 
      SM_edge_neighbor_get(the_surface, 
      tri_ptr->edge2,
      triangle_index));
    
    if (!SM_edge_normal_contiguous(the_surface, tri_ptr->edge3))
      SM_triangle_normal_flip(the_surface, 
      SM_edge_neighbor_get(the_surface, 
      tri_ptr->edge3,
      triangle_index));
    
  }
  return(YES);
}



/*
**  Triangle center.
*/

BOOL
SM_triangle_center_get(the_surface, triangle_index, returned_center)
SM_Surface_ptr the_surface;
int triangle_index;
VECTOR_3D returned_center;
/*
**  Returns in returned_center the geometric center of the triangle.
*/
{
  SM_Triangle_ptr the_triangle;
  
  /*
  **	Parameter validation.
  */
  if (!SM_triangle_index_valid(the_surface, triangle_index))  return(NO);
  
  the_triangle = &the_surface->tri_list[triangle_index];
  
  V3D_ADD(returned_center,
    the_surface->vert_list[the_triangle->vertex1].point,
    the_surface->vert_list[the_triangle->vertex2].point);
  V3D_ADD(returned_center,
    returned_center,
    the_surface->vert_list[the_triangle->vertex3].point);
  V3D_SCALE(returned_center, 1.0 / 3.0, returned_center);
  return(YES);
}





/*
**  Triangle center : the angular bisector
*/

BOOL
SM_triangle_angular_bisector_get(the_surface, triangle_index, returned_center)
SM_Surface_ptr the_surface;
int triangle_index;
VECTOR_3D returned_center;
/*
**  Returns in returned_center the angular bisector of the triangle.
*/
{
  SM_Triangle_ptr the_triangle;
  VECTOR_3D temp_vector;
  float u, v, w, sum;
  
  /*
  **	Parameter validation.
  */
  if (!SM_triangle_index_valid(the_surface, triangle_index))  return(NO);
  
  the_triangle = &the_surface->tri_list[triangle_index];
  
  V3D_SUB(temp_vector,
    the_surface->vert_list[the_triangle->vertex3].point,
    the_surface->vert_list[the_triangle->vertex2].point);
  u = V3D_NORM(temp_vector);
  
  V3D_SUB(temp_vector,
    the_surface->vert_list[the_triangle->vertex1].point,
    the_surface->vert_list[the_triangle->vertex3].point);
  v = V3D_NORM(temp_vector);
  
  V3D_SUB(temp_vector,
    the_surface->vert_list[the_triangle->vertex2].point,
    the_surface->vert_list[the_triangle->vertex1].point);
  w = V3D_NORM(temp_vector);
  
  sum = u + v + w;
  if (fabs(sum)  < SM_EPSILON ) return(NO);
  u = u/sum; v = v/sum; w = w/sum;
  
  V3D_SCALE(returned_center, u, the_surface->vert_list[the_triangle->vertex1].point);
  V3D_SCALE(temp_vector, v, the_surface->vert_list[the_triangle->vertex2].point);
  V3D_ADD(returned_center,returned_center,temp_vector);
  V3D_SCALE(temp_vector, w, the_surface->vert_list[the_triangle->vertex3].point);
  V3D_ADD(returned_center,returned_center,temp_vector);
  return(YES);
}


















/*
---------------------------------   SM_EDGE  ------------------------------------
*/

BOOL
SM_edge_index_valid(the_surface, edge_index)
SM_Surface_ptr the_surface;
int edge_index;
/*
**  Returns YES if edge_index is a valid edge index.
*/
{
  return( edge_index >= 0				&& 
    edge_index < the_surface->edge_list_size	&&
    the_surface->edge_list[edge_index].struct_used );
}


void
SM_edge_allocate(the_surface)
SM_Surface_ptr the_surface;
/*
**  For internal use only.
**  Extends the capacity (memory) used for storing edges.
*/
{
  register SM_Edge_ptr ptr_1, ptr_2;
  int total_size;
  
  /*
  **	If the list exists, increase its size.  If not, allocate an initial list size.
  */
  if (the_surface->edge_list) { 
    total_size = the_surface->edge_list_size * SM_EDGE_LIST_INC;
    the_surface->edge_list = (SM_Edge_ptr) 
      realloc(the_surface->edge_list, total_size * sizeof(SM_Edge));
  } else { 
    total_size = SM_EDGE_LIST_INIT;
    the_surface->edge_list = (SM_Edge_ptr)
      malloc(total_size * sizeof(SM_Edge));
  }
  
  /*
  **	Mark unused structures as unused.
  */
  ptr_1 = the_surface->edge_list + the_surface->edge_list_size;
  ptr_2 = the_surface->edge_list + total_size - 1;
  do { ptr_1->struct_used = NO; } while (ptr_1++ != ptr_2);
  
  the_surface->edge_list_size = total_size;
}


int
SM_edge_new(the_surface, triangle_index, v1_index, v2_index)
SM_Surface_ptr the_surface;
int triangle_index, v1_index, v2_index;
/*
**  Adds a new edge defined by vertices v1 and v2, 
**  if not already existing.
**  Returns SM_INDEX_ERROR if more than two triangles uses this
**  edge.
**  Otherwise, returns the index of the edge.
*/
{
  register SM_Edge_ptr edge_ptr;
  int new_edge_index;
  
  /*
  **	Gross parameter validation.
  */
  if (triangle_index < SM_INDEX_ERROR || triangle_index >= the_surface->tri_list_size ||
    !SM_vertex_index_valid(the_surface, v1_index) ||
    !SM_vertex_index_valid(the_surface, v2_index))
    
    return(SM_INDEX_ERROR);
  
    /*
    **	Find if edge (v1_index, v2_index) already exists.
  */
  new_edge_index = 0;
  edge_ptr = the_surface->edge_list;
  while (new_edge_index < the_surface->edge_list_size) {
    if (edge_ptr->struct_used &&
      (edge_ptr->vertex1 == v1_index && edge_ptr->vertex2 == v2_index ||
	     edge_ptr->vertex1 == v2_index && edge_ptr->vertex2 == v1_index)
       ) {
       /*
       **	Error if there are already two triangles using this edge.
      */
      if (edge_ptr->two_triangles)
        return(SM_INDEX_ERROR);
      if (edge_ptr->triangle1 == SM_INDEX_ERROR) {
        edge_ptr->triangle1 = triangle_index;
        return(new_edge_index);
      }
      
      if (triangle_index == SM_INDEX_ERROR)
        return(new_edge_index);
      
        /*
        **	Add the second triangle.
      */
      edge_ptr->two_triangles = YES;
      edge_ptr->triangle2 = triangle_index;
      return(new_edge_index);
    }
    new_edge_index++;
    edge_ptr++;
  }
  
  
  new_edge_index = Get_From_Stack(the_surface, SM_EDGE);
  if (new_edge_index == SM_INDEX_ERROR) {
    new_edge_index = the_surface->current_edge_list_size;
  }
  
  /* 
  **	Allocate new memory if necessary. 
  */
  if (new_edge_index >= the_surface->edge_list_size) 
    SM_edge_allocate(the_surface);
  
  edge_ptr = &the_surface->edge_list[new_edge_index];
  
  /* 
  **	initialise new edge. 
  */
  edge_ptr->struct_used = YES;
  edge_ptr->two_triangles = NO;
  edge_ptr->picked = STACK_NO;
  edge_ptr->edge_id = new_edge_index;
  edge_ptr->vertex1 = v1_index;
  edge_ptr->vertex2 = v2_index;
  edge_ptr->triangle1 = triangle_index;
  edge_ptr->triangle2 = SM_INDEX_ERROR;
  the_surface->current_edge_list_size += 1;
  return(new_edge_index);
}



BOOL
SM_edge_dispose(the_surface, edge_index)
SM_Surface_ptr the_surface;
int edge_index;
/*
**  Deletes the edge and the 1 or 2 triangles using it.
*/
{
  SM_Edge_ptr the_edge;
  
  if (!SM_edge_index_valid(the_surface, edge_index))	return(NO);
  
  the_edge = &the_surface->edge_list[edge_index];
  the_edge->struct_used = NO;
  if (the_edge->two_triangles) 
    SM_triangle_dispose(the_surface, the_edge->triangle2);
  SM_triangle_dispose(the_surface, the_edge->triangle1);
  Put_In_Stack(the_surface, edge_index, SM_EDGE);
  the_surface->current_edge_list_size -= 1;
  return(YES);
}



BOOL
SM_edge_ref_triangle_remove(the_surface, edge_index, triangle_index)
SM_Surface_ptr the_surface;
int edge_index, triangle_index;
/*
**  Deletes the reference to triangle_index.
**  If no one references this edge anymore, destroys it.
*/
{
  SM_Edge_ptr the_edge;
  
  if (!SM_edge_index_valid(the_surface, edge_index) ||
    !SM_triangle_index_valid(the_surface, triangle_index))  return(NO);
  
  the_edge = &the_surface->edge_list[edge_index];
  
  if (triangle_index != the_edge->triangle1  &&
    the_edge->two_triangles &&
    triangle_index != the_edge->triangle2)	return(NO);
  
  if (the_edge->two_triangles) {
    the_edge->two_triangles = NO;
    
    /*
    **  If we remove triangle1's reference, bring triangle2 to the
    **  triangle1 variable.
    */
    if (triangle_index == the_edge->triangle1) {
      the_edge->triangle1 = the_edge->triangle2;
      the_edge->triangle2 = SM_INDEX_ERROR;
    }
  } else {
    the_edge->struct_used = NO;
    Put_In_Stack(the_surface, edge_index, SM_EDGE);
    the_surface->current_edge_list_size -= 1;
  }
  return(YES);
}



BOOL
SM_edge_two_sides(the_surface, vertex_index, triangle_index, left_edge, right_edge)
SM_Surface_ptr the_surface;
int vertex_index, triangle_index, * left_edge, * right_edge;
/*
**  Returns the two edges having vertex_index in triangle_index.
*/
{
  SM_Triangle_ptr the_triangle; 
  
  if (!SM_vertex_index_valid(the_surface, vertex_index) ||
    !SM_triangle_index_valid(the_surface, triangle_index))  return(NO);
  
  the_triangle = &the_surface->tri_list[triangle_index];
  
  if (the_triangle->vertex1 == vertex_index) {
    *left_edge = the_triangle->edge3;
    *right_edge = the_triangle->edge1;
    return(YES);
  }
  if (the_triangle->vertex2 == vertex_index) {
    *left_edge = the_triangle->edge1;
    *right_edge = the_triangle->edge2;
    return(YES);
  }
  if (the_triangle->vertex3 == vertex_index) {
    *left_edge = the_triangle->edge2;
    *right_edge = the_triangle->edge3;
    return(YES);
  }
  return(NO);
}



int
SM_edge_neighbor_get(the_surface, edge_index, triangle_index)
SM_Surface_ptr the_surface;
int triangle_index, edge_index;
/*
**  This is only used for triangle normal coherence throughout the surface.
**  Returns the other triangle that uses edge_index.
*/
{
  SM_Edge_ptr the_edge;
  
  if (!SM_triangle_index_valid(the_surface, triangle_index) ||
    !SM_edge_index_valid(the_surface, edge_index))  return(SM_INDEX_ERROR);
  
  the_edge = &the_surface->edge_list[edge_index];
  
  if (!the_edge->two_triangles)  return(SM_INDEX_ERROR);
  
  if (the_edge->triangle1 == triangle_index)  return(the_edge->triangle2);
  if (the_edge->triangle2 == triangle_index)  return(the_edge->triangle1);
  return(SM_INDEX_ERROR);
}



int
SM_edge_common_triangles_get(the_surface, triangle1, triangle2)
SM_Surface_ptr the_surface;
int triangle1, triangle2;
/*
**  Returns the index of the edge common to triangle1 and triangle2.
**  Returns SM_INDEX_ERROR if there is none.
*/
{
  SM_Triangle_ptr tri_ptr1, tri_ptr2;
  
  if (!SM_triangle_index_valid(the_surface, triangle1) ||
    !SM_triangle_index_valid(the_surface, triangle2))  return(SM_INDEX_ERROR);
  
  tri_ptr1 = &the_surface->tri_list[triangle1];
  tri_ptr2 = &the_surface->tri_list[triangle2];
  
  if (tri_ptr1->edge1 == tri_ptr2->edge1  || 
    tri_ptr1->edge1 == tri_ptr2->edge2  ||
    tri_ptr1->edge1 == tri_ptr2->edge3)  return(tri_ptr1->edge1);
  
  if (tri_ptr1->edge2 == tri_ptr2->edge1  || 
    tri_ptr1->edge2 == tri_ptr2->edge2  ||
    tri_ptr1->edge2 == tri_ptr2->edge3)  return(tri_ptr1->edge2);
  
  if (tri_ptr1->edge3 == tri_ptr2->edge1  || 
    tri_ptr1->edge3 == tri_ptr2->edge2  ||
    tri_ptr1->edge3 == tri_ptr2->edge3)  return(tri_ptr1->edge3);
  
  return(SM_INDEX_ERROR);
}



BOOL
SM_edge_normal_contiguous(the_surface, edge_index)
SM_Surface_ptr the_surface;
int edge_index;
/*
**  Returns YES if the normals of the two triangles occupying this edge
**  form a contiguous surface.
*/
{
  SM_Edge_ptr edge_ptr;
  SM_Triangle_ptr tri_ptr1, tri_ptr2;
  int v11, v12, v21, v22;
  
  if (!SM_edge_index_valid(the_surface, edge_index))  return(NO);
  
  edge_ptr = &the_surface->edge_list[edge_index];
  
  if (!edge_ptr->two_triangles)  return(NO);
  
  /*
  **	An edge is contiguous if the vertices of its two triangles
  **	meet in cw and ccw order.
  */
  tri_ptr1 = &the_surface->tri_list[edge_ptr->triangle1];
  tri_ptr2 = &the_surface->tri_list[edge_ptr->triangle2];
  
  /*
  **	Find the order v11, v12 of the edge in triangle1.
  */
  if (tri_ptr1->edge1 == edge_index)  { 
    v11 = tri_ptr1->vertex1;  v12 =	tri_ptr1->vertex2;
  } else 
    if (tri_ptr1->edge2 == edge_index)  { 
      v11 = tri_ptr1->vertex2;  v12 =	tri_ptr1->vertex3;
    } else 
      if (tri_ptr1->edge3 == edge_index)  { 
        v11 = tri_ptr1->vertex3;  v12 =	tri_ptr1->vertex1;
      }
      
      /*
      **	Find the order v21, v22 of the edge in triangle2.
      */
      if (tri_ptr2->edge1 == edge_index)  { 
        v21 = tri_ptr2->vertex1;  v22 =	tri_ptr2->vertex2;
      } else 
        if (tri_ptr2->edge2 == edge_index)  { 
          v21 = tri_ptr2->vertex2;  v22 =	tri_ptr2->vertex3;
        } else 
          if (tri_ptr2->edge3 == edge_index)  { 
            v21 = tri_ptr2->vertex3;  v22 =	tri_ptr2->vertex1;
          }
          
          return( v11 == v22  &&  v12 == v21 );
}



/*
**  Two_triangles field.
*/

BOOL
SM_edge_hole_get(the_surface, edge_index, value)
SM_Surface_ptr the_surface;
int edge_index;
BOOL * value;
{
  if (!SM_edge_index_valid(the_surface, edge_index))	return(NO);
  *value = !the_surface->edge_list[edge_index].two_triangles;
  return(YES);
}



/*
**  Picked field.
*/

BOOL
SM_edge_picked_set(the_surface, edge_index, value)
SM_Surface_ptr the_surface;
int edge_index;
BOOL value;
{
  if (!SM_edge_index_valid(the_surface, edge_index))	return(NO);
  the_surface->edge_list[edge_index].picked = 
    ((the_surface->edge_list[edge_index].picked >> 1) << 1) | (BOOL_STACK) value;
  return(YES);
}


BOOL
SM_edge_picked_get(the_surface, edge_index, value)
SM_Surface_ptr the_surface;
int edge_index;
BOOL * value;
{
  if (!SM_edge_index_valid(the_surface, edge_index))	return(NO);
  *value = (BOOL) IS_YES(the_surface->edge_list[edge_index].picked);
  return(YES);
}

void
SM_edge_picked_push(the_surface)
SM_Surface_ptr the_surface;
{
  SM_FOR_EACH_EDGE(the_surface)
    edge_ptr->picked = (edge_ptr->picked << 1) | IS_YES(edge_ptr->picked);
  SM_END_FOR_EACH
}

void
SM_edge_picked_pop(the_surface)
SM_Surface_ptr the_surface;
{
  SM_FOR_EACH_EDGE(the_surface)
    edge_ptr->picked = edge_ptr->picked >> 1;
  SM_END_FOR_EACH
}



/*
**  Triangle fields.
*/

BOOL 
SM_edge_triangles_get(the_surface, edge_index, t1, t2)
SM_Surface_ptr the_surface;
int edge_index, * t1, * t2;
{
  if (!SM_edge_index_valid(the_surface, edge_index))	return(NO);
  *t1 = the_surface->edge_list[edge_index].triangle1;
  *t2 = ( the_surface->edge_list[edge_index].two_triangles ?
    the_surface->edge_list[edge_index].triangle2     :
  SM_INDEX_ERROR );
  return(YES);
}



/*
**  Vertex fields.
*/

BOOL 
SM_edge_vertices_get(the_surface, edge_index, v1, v2)
SM_Surface_ptr the_surface;
int edge_index, * v1, * v2;
{
  if (!SM_edge_index_valid(the_surface, edge_index))	return(NO);
  *v1 = the_surface->edge_list[edge_index].vertex1;
  *v2 = the_surface->edge_list[edge_index].vertex2;
  return(YES);
}















/*
-------------------------------  MATERIALS  ------------------------------
*/


void
SM_material_dispose()
/*
**  Deletes the material database.
*/
{
  if (the_materials.mat_list) free(the_materials.mat_list);
  the_materials.mat_list = NULL;
  the_materials.mat_list_size = 0;
  the_materials.glbind = SM_INDEX_ERROR;
  the_materials.glcpack = SM_INDEX_ERROR;
}


BOOL
SM_material_name_valid(name)
SM_Material_Name name;
/*
**  Returns YES if name is a valid material name in the_materials.
*/
{
  SM_FOR_EACH_MATERIAL
    if (strcmp(name, material_ptr->name) == 0) return(YES);
    SM_END_FOR_EACH
      return(NO);
}

BOOL
SM_material_index_valid(index)
int index;
{
  if (index > the_materials.mat_list_size) return(NO);
  if (the_materials.mat_list[index].struct_used == NO) return(NO);
  return(YES);
}

int
SM_material_index_get(name)
SM_Material_Name name;
/*
**  Returns the index of the material "name".
*/
{
  SM_FOR_EACH_MATERIAL
    if (strcmp(name, material_ptr->name) == 0) return(material_index);
    SM_END_FOR_EACH
      return(SM_INDEX_ERROR);
}

char 
*SM_material_name_get(index)
int index;
{
  return(the_materials.mat_list[index].name);
}

void
SM_material_read_fil(fil)
Parser_File * fil;
/*
**  Reads a list of materials from a file and simply adds them to the
**  existing material list the_materials.
**  If a material identifier already exists in the_materials, it
**  ignores it.
**  Reading is done by the "parser.o" module, for list manipulations,
**  so *fil must be initialized accordingly.
*/
{
  Parser_List_Level id1, id2;
  float property, r, g, b, sa;    
  SM_Material_Name name;
  int index, new_mat_index;
  
  parser_bgn_list(fil, &id1);
  /*
  **	Test if another material exists.
  */
  index = 0;
  material_link = (int *)calloc(10, sizeof(int));
  while (!parser_end_of_list(fil, id1)) {
    parser_bgn_list(fil, &id2);
    if (parser_read_string(fil, id2, name) &&
      ((new_mat_index = SM_material_add(name)) != SM_INDEX_ERROR)) {
      /*
      **  Test if another property exists.
      */
      if ((index != 0) && (index % 9 == 0)) {
        material_link = (int *)realloc(material_link, (index+10)*sizeof(int));
      }
      material_link[index] = new_mat_index;
      index += 1;
      while (parser_read_float(fil, id2, &property)) {
        if (property == EMISSION || property == AMBIENT || 
          property == DIFFUSE || property == SPECULAR) {
          parser_read_float(fil, id2, &r);
          parser_read_float(fil, id2, &g);
          parser_read_float(fil, id2, &b);
          SM_material_rgb_set(name, property, r, g, b);
        }
        else
          if (property == SHININESS || property == ALPHA) {
            parser_read_float(fil, id2, &sa);
            SM_material_value_set(name, property, sa);
          } else
            if (property == REFLECTIVITY) {
              parser_read_float(fil, id2, &sa);
              SM_material_value_set(name, property, sa);
            }
      }
    }
    else if (new_mat_index == SM_INDEX_ERROR) {
      new_mat_index = SM_material_index_get(name);
      if ((index != 0) && (index % 9 == 0)) {
        material_link = (int *)realloc(material_link, (index+10)*sizeof(int));
      }
      material_link[index] = new_mat_index;
      index += 1;
    }
    parser_end_list(fil, id2);
  }
  parser_end_list(fil, id1);
}


BOOL
SM_material_read(name)
char * name;
/*
**  Opens and reads the materials from a Parser_file.
**  Returns YES if the file was open.
*/
{
  Parser_File * fil;
  
  if (fil = parser_open_file(name)) {
    SM_material_read_fil(fil);
    return(YES);
  }
  parser_close_file(fil);
  return(NO);
}




void
SM_material_write_fil(fil)
FILE *fil;
{
  register int i;
  
  fprintf(fil,"materials\n{\n");
  SM_FOR_EACH_MATERIAL
    fprintf(fil,"{\"%s\",",material_ptr->name);
  for (i = 0; i < SM_MAT_LIST_LENGTH; i++)  
    fprintf(fil,"%g,",material_ptr->gl_properties[i]);
  fprintf(fil,"%d,%g},\n",REFLECTIVITY,material_ptr->reflectivity);
  SM_END_FOR_EACH
    fprintf(fil,"}\n");
}


void
SM_material_write(name)
char * name;
/*
**  Opens and writes the material database in the file "name".
*/
{
  FILE * fil;
  
  if (fil = fopen(name, "w"))
    SM_material_write_fil(fil);
  fclose(fil);
}



BOOL
SM_material_add(name)
SM_Material_Name name;
/*
**  Creates room for a new surface property and returns its index.
**  Assigns default values.
**  Returns NO if name already exists, YES otherwise.
*/
{
  SM_Mat_ptr ptr_1, ptr_2;
  int new_mat_index, total_size;
  
  /*
  **	Security mesure.
  */
  name[SM_MATERIAL_NAME_LENGTH - 1] = '\0';
  
  /*
  **	Find if material already exists.
  */
  if (SM_material_name_valid(name))	return(SM_INDEX_ERROR);
  
  /* 
  **	Find unused structure, if there is still room in the list. 
  */
  new_mat_index = 0;
  while ( new_mat_index < the_materials.mat_list_size &&
    the_materials.mat_list[new_mat_index].struct_used) new_mat_index++;
  
    /* 
    **	Allocate new memory if needed. 
  */
  if (new_mat_index == the_materials.mat_list_size) {
  /*
  **	If the list exists, increase its size.  If not, allocate an initial list size.
    */
    if (the_materials.mat_list) { 
      total_size = the_materials.mat_list_size * SM_MAT_LIST_INC;
      the_materials.mat_list = (SM_Mat_ptr) 
				    realloc(the_materials.mat_list, total_size * sizeof(SM_Mat));
    } else { 
      total_size = SM_MAT_LIST_INIT;
      the_materials.mat_list = (SM_Mat_ptr)
				    malloc(total_size * sizeof(SM_Mat));
    }
    
    /*
    **	Mark unused structures as unused.
    */
    ptr_1 = the_materials.mat_list + the_materials.mat_list_size;
    ptr_2 = the_materials.mat_list + total_size - 1;
    do { ptr_1->struct_used = NO; } while (ptr_1++ != ptr_2);
    
    the_materials.mat_list_size = total_size;
    
  }
  
  /* 
  **	Initialize new prop to default values. 
  */
  the_materials.mat_list[new_mat_index] = SM_material_default;
  the_materials.glbind = SM_INDEX_ERROR;
  the_materials.glcpack = SM_INDEX_ERROR;
  strcpy(the_materials.mat_list[new_mat_index].name, name);
  the_materials.mat_list[new_mat_index].reflectivity = 0.0;
  return(new_mat_index);
}

BOOL
SM_material_remove(name)
SM_Material_Name name;
/*
**  Deletes the material property mat_index.
*/
{
  int mat_index;
  
  if (!SM_material_name_valid(name))	return(NO);
  
  mat_index = SM_material_index_get(name);
  the_materials.mat_list[mat_index].struct_used = NO;
  the_materials.glbind = SM_INDEX_ERROR;
  the_materials.glcpack = SM_INDEX_ERROR;
  return(YES);
}



BOOL
SM_material_rgb_set(SM_Material_Name name,int gl_property,float r,float g,float b)
{
/*
**  Sets r,g and b gl_properties of mat_index. 
**  "property" can be EMISSION, AMBIENT, DIFFUSE or SPECULAR.
  */
  int SM_property, mat_index;
  SM_Mat_ptr the_prop;
  
  if (!SM_material_name_valid(name)) return(NO);
  
  mat_index = SM_material_index_get(name);
  
  /*
  **	Find the index of property in the property list.
  */
  switch (gl_property) {
  case EMISSION:	SM_property = SM_EMISSION;	break;
  case AMBIENT:	SM_property = SM_AMBIENT;	break;
  case DIFFUSE:	SM_property = SM_DIFFUSE;	break;
  case SPECULAR:	SM_property = SM_SPECULAR;	break;	
  default:	return(NO);
  }
  
  the_prop = &the_materials.mat_list[mat_index];
  
  the_prop->gl_properties[SM_property + 1] = r;
  the_prop->gl_properties[SM_property + 2] = g;
  the_prop->gl_properties[SM_property + 3] = b;
  
  if (SM_property == SM_DIFFUSE) 
    the_prop->rgba = 
    (unsigned long) (r * 255.0) |
    (unsigned long) (g * 255.0)<<8 |
    (unsigned long) (b * 255.0)<<16 |
    (unsigned long) (the_prop->gl_properties[SM_ALPHA + 1] * 255.0)<<24;
  
    /*
    **	This materials has to be lmdef again.  Last bind and cpack indexes may not
    **	be valid anymore.
  */
  the_prop->gldef = NO;
  the_materials.glbind = SM_INDEX_ERROR;
  the_materials.glcpack = SM_INDEX_ERROR;
  return(YES);
}



BOOL
SM_material_rgb_get(SM_Material_Name name,int gl_property,float * r,float * g,float * b)
/*
**  Returns r, g and b gl_properties of mat_index. 
**  "property" can be EMISSION, AMBIENT, DIFFUSE or SPECULAR.
*/
{
  int SM_property, mat_index;
  SM_Mat_ptr the_prop;
  
  if (!SM_material_name_valid(name)) return(NO);
  
  mat_index = SM_material_index_get(name);
  
  /*
  **	Find the index of property in the property list.
  */
  switch (gl_property) {
  case EMISSION:	SM_property = SM_EMISSION;	break;
  case AMBIENT:	SM_property = SM_AMBIENT;	break;
  case DIFFUSE:	SM_property = SM_DIFFUSE;	break;
  case SPECULAR:	SM_property = SM_SPECULAR;	break;	
  default:	return(NO);
  }
  
  the_prop = &the_materials.mat_list[mat_index];
  
  *r = the_prop->gl_properties[SM_property + 1];
  *g = the_prop->gl_properties[SM_property + 2];
  *b = the_prop->gl_properties[SM_property + 3];
  return(YES);
}



BOOL
SM_material_value_set(SM_Material_Name name,int gl_property,float value)
/*
**  Sets the "value" property of mat_index.
**  Here, property can be either SHININESS or ALPHA.
*/
{
  int SM_property, mat_index;
  SM_Mat_ptr the_prop;
  
  if (!SM_material_name_valid(name)) return(NO);
  
  mat_index = SM_material_index_get(name);
  
  /*
  **	Find the index of property in the property list.
  */
  switch (gl_property) {
  case SHININESS:	SM_property = SM_SHININESS;  break;
  case ALPHA:	SM_property = SM_ALPHA;	  break;
  case REFLECTIVITY:
    the_materials.mat_list[mat_index].reflectivity = value;
    return(YES);
  default:	return(NO);
  }
  
  the_prop = &the_materials.mat_list[mat_index];
  
  the_prop->gl_properties[SM_property + 1] = value;
  
  if (SM_property == SM_ALPHA) 
    the_prop->rgba = 
    (unsigned long) (the_prop->gl_properties[SM_DIFFUSE + 1] * 255.0) |
    (unsigned long) (the_prop->gl_properties[SM_DIFFUSE + 2] * 255.0)<<8 |
    (unsigned long) (the_prop->gl_properties[SM_DIFFUSE + 3] * 255.0)<<16 |
    (unsigned long) (value * 255.0)<<24;
  
    /*
    **	This materials has to be lmdef again.  Last bind and cpack indexes may not
    **	be valid anymore.
  */
  the_prop->gldef = NO;
  the_materials.glbind = SM_INDEX_ERROR;
  the_materials.glcpack = SM_INDEX_ERROR;
  return(YES);
}



BOOL
SM_material_value_get(name, gl_property, value)
SM_Material_Name name;
int gl_property;
float * value;
/*
**  Sets the "value" property of mat_index.
**  Here, property can be either SM_SHININESS or SM_ALPHA.
*/
{
  int SM_property, mat_index;
  
  if (!SM_material_name_valid(name)) return(NO);
  
  mat_index = SM_material_index_get(name);
  
  /*
  **	Find the index of property in the property list.
  */
  switch (gl_property) {
  case SHININESS:	SM_property = SM_SHININESS;  break;
  case ALPHA:	SM_property = SM_ALPHA;	  break;
  case REFLECTIVITY: 
    *value = the_materials.mat_list[mat_index].reflectivity;
    return(YES);
  default:	return(NO);
  }
  
  *value = the_materials.mat_list[mat_index].gl_properties[SM_property + 1];
  return(YES);
}
















/*
-------------------------------  SM_MATRIX  --------------------------------
*/

void
SM_matrix_delta(the_surface, delta)
SM_Surface_ptr the_surface;
MATRIX_H3D delta;
{
  MH3D_mul(the_surface->model_transform, the_surface->model_transform, delta);
}


void
SM_matrix_get(the_surface, the_matrix)
SM_Surface_ptr the_surface;
MATRIX_H3D the_matrix;
{
  MH3D_COPY(the_matrix, the_surface->model_transform);
}

void
SM_matrix_set(the_surface, the_matrix)
SM_Surface_ptr the_surface;
MATRIX_H3D the_matrix;
{
  MH3D_COPY(the_surface->model_transform, the_matrix);
}


















/*
--------------------------  SM_SURFACE manipulations  -------------------------
*/

SM_Surface_ptr
SM_surface_new()
/*
**  Allocates a new, empty surface.
*/
{
  SM_Surface_ptr the_surface;
  
  the_surface = (SM_Surface_ptr) calloc(1, sizeof(SM_Surface));
  the_surface->struct_used = YES;
  the_surface->vert_list_size = 0;
  the_surface->current_vert_list_size = 0;
  the_surface->vert_list = NULL;
  the_surface->tri_list_size = 0;
  the_surface->current_tri_list_size = 0;
  the_surface->tri_list = NULL;
  the_surface->edge_list_size = 0;
  the_surface->current_edge_list_size = 0;
  the_surface->edge_list = NULL;
  Stack_Init(the_surface, SM_VERTEX);
  Stack_Init(the_surface, SM_EDGE);
  Stack_Init(the_surface, SM_TRIANGLE);
  MH3D_IDENTITY(the_surface->model_transform);
  the_surface->the_region_list = NULL;
  return(the_surface);
}




void
SM_surface_dispose(the_surface)
SM_Surface_ptr the_surface;
/*
**  Destroys the whole surface by eliminating all vertices.
**  Then, deallocates the lists.
*/
{
  SM_FOR_EACH_VERTEX(the_surface)
  /*
  SM_vertex_dispose(the_surface, vertex_index);
  */
  SM_FOR_EACH_NORMAL(vertex_ptr)
  if (normal_ptr->tri_ref_list) free(normal_ptr->tri_ref_list);
  SM_END_FOR_EACH
    if (vertex_ptr->norm_list) free(vertex_ptr->norm_list);
    SM_END_FOR_EACH
      
      SM_FOR_EACH_TRIANGLE(the_surface)
      if (triangle_ptr->region_list) free(triangle_ptr->region_list);
      SM_END_FOR_EACH
        
        if (the_surface->vert_list_size) {
          free(the_surface->vert_list);
          the_surface->vert_list_size = 0;
          the_surface->vert_list = NULL;
        }
        if (the_surface->tri_list_size) {
          free(the_surface->tri_list);
          the_surface->tri_list_size = 0;
          the_surface->tri_list = NULL;
        }
        if (the_surface->edge_list_size) {
          free(the_surface->edge_list);
          the_surface->edge_list_size = 0;
          the_surface->edge_list = NULL;
        }
        Stack_Free(the_surface);
        free(the_surface);
}

void
SM_surface_region_list_set(the_surface, the_region_list)
SM_Surface_ptr the_surface;
SM_Region_List_ptr the_region_list;
{
  if (the_region_list == NULL) return;
  the_surface->the_region_list = (struct SM_Region_List*)the_region_list;
  the_region_list->the_surface = the_surface;
  SM_FOR_EACH_TRIANGLE(the_surface)
    if (triangle_ptr->region_number != 0) {
      free(triangle_ptr->region_list);
      triangle_ptr->region_number = 0;
    }
    SM_END_FOR_EACH
      SM_FOR_EACH_REGION(the_region_list)
      SM_FOR_EACH_REGION_TRIANGLE(region_ptr)
      SM_triangle_region_add(the_surface,  *region_tri_ptr, region_index);
    SM_END_FOR_EACH
      SM_END_FOR_EACH
}

int
SM_surface_memory_used(the_surface)
SM_Surface_ptr the_surface;
/*
**  Returns the amount of memory used in Kbytes.
*/
{
  int mem_used = 0;
  
  if (!the_surface) return(0);
  
  SM_FOR_EACH_VERTEX(the_surface)
    mem_used += sizeof(SM_Vertex_normal) * vertex_ptr->norm_list_size;
  SM_END_FOR_EACH
    
    mem_used += sizeof(SM_Vertex) * the_surface->vert_list_size;
  mem_used += sizeof(SM_Triangle) * the_surface->tri_list_size;
  mem_used += sizeof(SM_Edge) * the_surface->edge_list_size;
  mem_used += sizeof(SM_Surface);
  
  return((int) (mem_used / 1024.0));
}


void
SM_surface_material_write_fil(the_surface, fil)
SM_Surface_ptr the_surface;
FILE * fil;
/*
**  Writes, in the unix text file fil, the materials that the_surface
**  uses.
*/
{
  int i;
  
  /*
  **	The flags will be used to see which materials are used
  **	in the_surface->material_list.
  */
  SM_FOR_EACH_MATERIAL
    material_ptr->flag = NO;
  SM_END_FOR_EACH
    
    SM_FOR_EACH_TRIANGLE(the_surface)
    if (triangle_ptr->material_index != SM_INDEX_ERROR)
      the_materials.mat_list[triangle_ptr->material_index].flag = YES;
    SM_END_FOR_EACH
      
      fprintf(fil,"materials\n{\n");
    SM_FOR_EACH_MATERIAL
      if (material_ptr->flag) {
        fprintf(fil,"{\"%s\",",material_ptr->name);
        for (i = 0; i < SM_MAT_LIST_LENGTH; i++)  
          fprintf(fil,"%g,",material_ptr->gl_properties[i]);
        fprintf(fil, "%d,%g}\n",REFLECTIVITY,material_ptr->reflectivity);
      }
      SM_END_FOR_EACH
        fprintf(fil,"}\n");
}


void
SM_surface_material_write(the_surface, name)
SM_Surface_ptr the_surface;
char * name;
/*
**  Opens a unix file "name" and writes the materials that the_surface uses.
*/
{
  FILE * fil;
  
  if (fil = fopen(name, "w")) 
    SM_surface_material_write_fil(the_surface, fil);
  fclose(fil);
}

SM_Surface_ptr
SM_surface_read(name)
char *name;
{
  typedef struct SM_incident_edge {
    int edge_index;
    int endpoint_index;
    struct SM_incident_edge *next;
  }SM_Incident_edge, *SM_Incident_edge_ptr;
  
  typedef struct SM_temp_vertex {
    SM_Incident_edge_ptr first;
    SM_Incident_edge_ptr last;
  }SM_Temp_vertex, *SM_Temp_vertex_ptr;
  
  FILE             *fil;
  int              i;
  int              number_vertices, number_triangles, number_materials;
  float            dummy;    
  int              new_mat_index;
  
  int              j,index;
  SM_Surface_ptr   the_surface;
  SM_Vertex_ptr    new_vertex;
  SM_Edge_ptr      edge_ptr;
  VECTOR_3D        point;
  char             filename[100];
  SM_Material_Name mat_name;
  short            mat_index;
  SM_Triangle_ptr  tri_ptr;
  
  SM_Temp_vertex_ptr SM_Temp_vertex_list;
  SM_Incident_edge_ptr current_edge;
  short index_vertex[3];
  int existing_edge1, existing_edge2;
  SM_Incident_edge_ptr tempo, current;
  
  if (!(strstr(name, ".sm"))) {
    sprintf(filename, "%s.sm", name);
    fil = fopen(filename, "rb");
  } else
    fil = fopen(name, "rb");
  if (fil) {
    the_surface = SM_surface_new();
    /*
    * read the number of vertices.
    */
    fread(&number_vertices, sizeof(int), 1, fil);
#ifdef LITTLE_ENDIAN
    number_vertices = ntohl(number_vertices);
#endif
    /*
    * read the vertices.
    */
    index = -1;
    for (i=0; i<number_vertices; i++) {
      index++;
      fread(point, sizeof(float), 3, fil);
#ifdef LITTLE_ENDIAN
      *((unsigned long*) &point[0]) = ntohl(*((unsigned long*) &point[0]));
      *((unsigned long*) &point[1]) = ntohl(*((unsigned long*) &point[1]));
      *((unsigned long*) &point[2]) = ntohl(*((unsigned long*) &point[2]));
#endif
      if (index >= the_surface->vert_list_size)   
        SM_vertex_allocate(the_surface);
      new_vertex = &the_surface->vert_list[index];
      new_vertex->struct_used = YES;
      new_vertex->picked = (BOOL_STACK) NO;
      new_vertex->vertex_id = index;
      new_vertex->norm_list = NULL;
      new_vertex->norm_list_size = 0;
      new_vertex->func_result = 0.0;
      SM_vertex_point_set(the_surface, index, point);
      the_surface->current_vert_list_size += 1;	
    }
    SM_Temp_vertex_list = (SM_Temp_vertex_ptr)calloc(the_surface->current_vert_list_size, sizeof(SM_Temp_vertex));
    for (i=0;i<the_surface->current_vert_list_size; i++) {
      SM_Temp_vertex_list[i].first = NULL;
      SM_Temp_vertex_list[i].last = NULL;
    }
    /*
    * read number of materials.
    */
    fread(&number_materials, sizeof(int), 1, fil);
#ifdef LITTLE_ENDIAN
    number_materials = ntohl(number_materials);
#endif
    /*
    * read materials.
    */
    
    index = 0;
    material_link = (int *)calloc(10, sizeof(int));
    for (i=0; i<number_materials; i++) {
      fread(mat_name, sizeof(char), SM_MATERIAL_NAME_LENGTH, fil);
      if ((new_mat_index = SM_material_add(mat_name)) != SM_INDEX_ERROR) {
        if ((index != 0) && (index % 9 == 0)) {
          material_link = (int *)realloc(material_link, (index+10)*sizeof(int));
        }
        material_link[index] = new_mat_index;
        index += 1;
        fread(the_materials.mat_list[new_mat_index].gl_properties, sizeof(float), SM_MAT_LIST_LENGTH, fil);
#ifdef LITTLE_ENDIAN
        for (j = 0; j < SM_MAT_LIST_LENGTH; j++)
          *((unsigned long*) &the_materials.mat_list[new_mat_index].gl_properties[j]) = ntohl(*((unsigned long*) &the_materials.mat_list[new_mat_index].gl_properties[j]));
#endif
        fread(&(the_materials.mat_list[new_mat_index].reflectivity), sizeof(float), 1, fil);
#ifdef LITTLE_ENDIAN
        *((unsigned long*) &the_materials.mat_list[new_mat_index].reflectivity) = ntohl(*((unsigned long*) &the_materials.mat_list[new_mat_index].reflectivity));
#endif
        the_materials.mat_list[new_mat_index].rgba = 	    
          (unsigned long) (the_materials.mat_list[new_mat_index].gl_properties[SM_DIFFUSE+1] * 255.0) |
          (unsigned long) (the_materials.mat_list[new_mat_index].gl_properties[SM_DIFFUSE+2] * 255.0)<<8 |
          (unsigned long) (the_materials.mat_list[new_mat_index].gl_properties[SM_DIFFUSE+3] * 255.0)<<16 |
          (unsigned long) (the_materials.mat_list[new_mat_index].gl_properties[SM_ALPHA + 1] * 255.0)<<24;
        the_materials.mat_list[new_mat_index].gldef =NO;
        the_materials.glbind = SM_INDEX_ERROR;
        the_materials.glcpack = SM_INDEX_ERROR;
      }
      else if (new_mat_index == SM_INDEX_ERROR) {
        new_mat_index = SM_material_index_get(mat_name);
        if ((index != 0) && (index % 9 == 0)) {
          material_link = (int *)realloc(material_link, (index+10)*sizeof(int));
        }
        for (j = 0; j < SM_MAT_LIST_LENGTH; j++)  
          fread(&dummy, sizeof(float), 1, fil);
        fread(&dummy, sizeof(float), 1, fil);
        material_link[index] = new_mat_index;
        index += 1;
      }
    }
    /*
    * read number of triangles.
    */
    fread(&number_triangles, sizeof(int), 1, fil);
#ifdef LITTLE_ENDIAN
    number_triangles = ntohl(number_triangles);
#endif
    /*
    * read triangles and reconstruct edges.
    */
    index = -1;
    for (j=0; j<number_triangles; j++) {
      index++;
      if (index >= the_surface->tri_list_size) 
        SM_triangle_allocate(the_surface);
      tri_ptr = &the_surface->tri_list[index];
      the_surface->current_tri_list_size += 1;
      tri_ptr->struct_used = YES;
      fread(index_vertex, sizeof(short), 3, fil);
#ifdef LITTLE_ENDIAN
      index_vertex[0] = ntohs(index_vertex[0]);
      index_vertex[1] = ntohs(index_vertex[1]);
      index_vertex[2] = ntohs(index_vertex[2]);
#endif
      fread(&mat_index, sizeof(short), 1, fil);
#ifdef LITTLE_ENDIAN
      mat_index = ntohs(mat_index);
#endif
      tri_ptr->vertex1 = index_vertex[0];
      tri_ptr->vertex2 = index_vertex[1];
      tri_ptr->vertex3 = index_vertex[2];
      tri_ptr->region_number = 0;
      tri_ptr->region_list = (int *)NULL;
      tri_ptr->edge1 = tri_ptr->edge2 = tri_ptr->edge3 = -1;
      for (i=0; i<3; i++) {
        if (SM_Temp_vertex_list[(index_vertex[i])].first != NULL) {
          existing_edge1 = -1;
          existing_edge2 = -1;
          current_edge = SM_Temp_vertex_list[(index_vertex[i])].first;
          while ( current_edge != (SM_Incident_edge_ptr)NULL) {
            if (current_edge->endpoint_index == index_vertex[(i+1)%3]) {
              existing_edge1 = current_edge->edge_index;
              if (existing_edge2 != -1) break;
            }
            if (current_edge->endpoint_index == index_vertex[(i+2)%3]) {
              existing_edge2 = current_edge->edge_index;
              if (existing_edge1 != -1) break;
            }
            current_edge = current_edge->next;
          }
          if (existing_edge1 != -1) {
          /*
          * Il existe deja une arete entre i et i+1
            */ 
            if (the_surface->edge_list[existing_edge1].two_triangles == NO) {
              if (the_surface->edge_list[existing_edge1].triangle1 != index) {
                the_surface->edge_list[existing_edge1].triangle2 = index;
                the_surface->edge_list[existing_edge1].two_triangles = YES;
              }
            }	    
          }
          else {
          /*
          * Sinon il faut en creer une 
            */
            if (the_surface->current_edge_list_size >= the_surface->edge_list_size) {
              SM_edge_allocate(the_surface);
            }
            edge_ptr = &the_surface->edge_list[the_surface->current_edge_list_size];
            the_surface->current_edge_list_size += 1;
            edge_ptr->struct_used = YES;
            edge_ptr->two_triangles = NO;
            edge_ptr->picked = STACK_NO;
            edge_ptr->edge_id = existing_edge1 = the_surface->current_edge_list_size-1;
            edge_ptr->triangle1 = index;
            edge_ptr->triangle2 = SM_INDEX_ERROR;
            edge_ptr->vertex1 = index_vertex[i];
            edge_ptr->vertex2 = index_vertex[(i+1)%3];
            tempo = SM_Temp_vertex_list[index_vertex[i]].last;
            if (tempo == NULL) {
              SM_Temp_vertex_list[index_vertex[i]].first = (SM_Incident_edge_ptr)calloc(1,sizeof(SM_Incident_edge));
              SM_Temp_vertex_list[index_vertex[i]].last = SM_Temp_vertex_list[index_vertex[i]].first;
              SM_Temp_vertex_list[index_vertex[i]].first->endpoint_index = index_vertex[(i+1)%3];
              SM_Temp_vertex_list[index_vertex[i]].first->edge_index = existing_edge1;
              SM_Temp_vertex_list[index_vertex[i]].first->next = (SM_Incident_edge_ptr)NULL;    
            }
            else {
              SM_Temp_vertex_list[index_vertex[i]].last = (SM_Incident_edge_ptr)calloc(1,sizeof(SM_Incident_edge));
              SM_Temp_vertex_list[index_vertex[i]].last->endpoint_index = index_vertex[(i+1)%3];
              SM_Temp_vertex_list[index_vertex[i]].last->edge_index = existing_edge1;
              SM_Temp_vertex_list[index_vertex[i]].last->next = (SM_Incident_edge_ptr)NULL;
              tempo->next = SM_Temp_vertex_list[index_vertex[i]].last;
              SM_Temp_vertex_list[index_vertex[i]].last->next = NULL;
            }	
            tempo = SM_Temp_vertex_list[index_vertex[(i+1)%3]].last;
            if (tempo == NULL) {
              SM_Temp_vertex_list[index_vertex[(i+1)%3]].first = (SM_Incident_edge_ptr)calloc(1,sizeof(SM_Incident_edge));
              SM_Temp_vertex_list[index_vertex[(i+1)%3]].last = SM_Temp_vertex_list[index_vertex[(i+1)%3]].first;
              SM_Temp_vertex_list[index_vertex[(i+1)%3]].first->endpoint_index = index_vertex[i];
              SM_Temp_vertex_list[index_vertex[(i+1)%3]].first->edge_index = existing_edge1;
              SM_Temp_vertex_list[index_vertex[(i+1)%3]].first->next = (SM_Incident_edge_ptr)NULL;    
            }
            else {
              SM_Temp_vertex_list[index_vertex[(i+1)%3]].last = (SM_Incident_edge_ptr)calloc(1,sizeof(SM_Incident_edge));
              SM_Temp_vertex_list[index_vertex[(i+1)%3]].last->endpoint_index = index_vertex[i];
              SM_Temp_vertex_list[index_vertex[(i+1)%3]].last->edge_index = existing_edge1;
              SM_Temp_vertex_list[index_vertex[(i+1)%3]].last->next = (SM_Incident_edge_ptr)NULL;
              tempo->next = SM_Temp_vertex_list[index_vertex[(i+1)%3]].last;
              SM_Temp_vertex_list[index_vertex[(i+1)%3]].last->next = NULL;
            }	
          }
          if (existing_edge2 != -1) {
          /*
          * Il existe deja une arete entre i et i+2
            */ 
            if (the_surface->edge_list[existing_edge2].two_triangles == NO) {
              if (the_surface->edge_list[existing_edge2].triangle1 != index) {
                the_surface->edge_list[existing_edge2].triangle2 = index;
                the_surface->edge_list[existing_edge2].two_triangles = YES;
              }
            }	    
          }
          else {
            if (the_surface->current_edge_list_size >= the_surface->edge_list_size) { 
              SM_edge_allocate(the_surface);
            }
            edge_ptr = &the_surface->edge_list[the_surface->current_edge_list_size];
            the_surface->current_edge_list_size += 1;
            edge_ptr->struct_used = YES;
            edge_ptr->two_triangles = NO;
            edge_ptr->picked = STACK_NO;
            edge_ptr->edge_id = existing_edge2 = the_surface->current_edge_list_size-1;
            edge_ptr->triangle1 = index;
            edge_ptr->triangle2 = SM_INDEX_ERROR;
            edge_ptr->vertex1 = index_vertex[i];
            edge_ptr->vertex2 = index_vertex[(i+2)%3];
            tempo = SM_Temp_vertex_list[index_vertex[i]].last;
            if (tempo == NULL) {
              SM_Temp_vertex_list[index_vertex[i]].first = (SM_Incident_edge_ptr)calloc(1,sizeof(SM_Incident_edge));
              SM_Temp_vertex_list[index_vertex[i]].last = SM_Temp_vertex_list[index_vertex[i]].first;
              SM_Temp_vertex_list[index_vertex[i]].first->endpoint_index = index_vertex[(i+2)%3];
              SM_Temp_vertex_list[index_vertex[i]].first->edge_index = existing_edge2;
              SM_Temp_vertex_list[index_vertex[i]].first->next = (SM_Incident_edge_ptr)NULL;   		    
            }
            else {		    
              SM_Temp_vertex_list[index_vertex[i]].last = (SM_Incident_edge_ptr)calloc(1,sizeof(SM_Incident_edge));
              SM_Temp_vertex_list[index_vertex[i]].last->endpoint_index = index_vertex[(i+2)%3];
              SM_Temp_vertex_list[index_vertex[i]].last->edge_index = existing_edge2;
              SM_Temp_vertex_list[index_vertex[i]].last->next = (SM_Incident_edge_ptr)NULL;
              tempo->next = SM_Temp_vertex_list[index_vertex[i]].last;
              SM_Temp_vertex_list[index_vertex[i]].last->next = NULL;
            }	
            tempo = (SM_Incident_edge_ptr)(SM_Temp_vertex_list[index_vertex[(i+2)%3]].last);
            if (tempo == NULL) {
              SM_Temp_vertex_list[index_vertex[(i+2)%3]].first = (SM_Incident_edge_ptr)calloc(1,sizeof(SM_Incident_edge));
              SM_Temp_vertex_list[index_vertex[(i+2)%3]].last = SM_Temp_vertex_list[index_vertex[(i+2)%3]].first;
              SM_Temp_vertex_list[index_vertex[(i+2)%3]].first->endpoint_index = index_vertex[i];
              SM_Temp_vertex_list[index_vertex[(i+2)%3]].first->edge_index = existing_edge2;
              SM_Temp_vertex_list[index_vertex[(i+2)%3]].first->next = (SM_Incident_edge_ptr)NULL;   		    
            }
            else {		    
              SM_Temp_vertex_list[index_vertex[(i+2)%3]].last = (SM_Incident_edge_ptr)calloc(1,sizeof(SM_Incident_edge));
              SM_Temp_vertex_list[index_vertex[(i+2)%3]].last->endpoint_index = index_vertex[i];
              SM_Temp_vertex_list[index_vertex[(i+2)%3]].last->edge_index = existing_edge2;
              SM_Temp_vertex_list[index_vertex[(i+2)%3]].last->next = (SM_Incident_edge_ptr)NULL;
              tempo->next = SM_Temp_vertex_list[index_vertex[(i+2)%3]].last;
              SM_Temp_vertex_list[index_vertex[(i+2)%3]].last->next = NULL;
            }	
          }
        }
        else {	
          if (the_surface->current_edge_list_size >= the_surface->edge_list_size) {
            SM_edge_allocate(the_surface);
          }
          edge_ptr = &the_surface->edge_list[the_surface->current_edge_list_size];
          the_surface->current_edge_list_size += 1;
          edge_ptr->struct_used = YES;
          edge_ptr->two_triangles = NO;
          edge_ptr->picked = STACK_NO;
          edge_ptr->edge_id = existing_edge1 = the_surface->current_edge_list_size-1;
          edge_ptr->triangle1 = index;
          edge_ptr->triangle2 = SM_INDEX_ERROR;
          edge_ptr->vertex1 = index_vertex[i];
          edge_ptr->vertex2 = index_vertex[(i+1)%3];
          if (the_surface->current_edge_list_size >= the_surface->edge_list_size) {
            SM_edge_allocate(the_surface);
          }
          edge_ptr = &the_surface->edge_list[the_surface->current_edge_list_size];
          the_surface->current_edge_list_size += 1;
          edge_ptr->struct_used = YES;
          edge_ptr->two_triangles = NO;
          edge_ptr->picked = STACK_NO;
          edge_ptr->edge_id = existing_edge2 = the_surface->current_edge_list_size-1;
          edge_ptr->triangle1 = index;
          edge_ptr->triangle2 = SM_INDEX_ERROR;
          edge_ptr->vertex1 = index_vertex[i];
          edge_ptr->vertex2 = index_vertex[(i+2)%3];
          
          SM_Temp_vertex_list[index_vertex[i]].first = (SM_Incident_edge_ptr)calloc(1,sizeof(SM_Incident_edge));
          SM_Temp_vertex_list[index_vertex[i]].last = SM_Temp_vertex_list[index_vertex[i]].first;
          SM_Temp_vertex_list[index_vertex[i]].first->endpoint_index = index_vertex[(i+1)%3];
          SM_Temp_vertex_list[index_vertex[i]].first->edge_index = existing_edge1;
          SM_Temp_vertex_list[index_vertex[i]].first->next = (SM_Incident_edge_ptr)NULL;
          
          tempo = SM_Temp_vertex_list[index_vertex[i]].last;
          SM_Temp_vertex_list[index_vertex[i]].last = (SM_Incident_edge_ptr)calloc(1,sizeof(SM_Incident_edge));
          SM_Temp_vertex_list[index_vertex[i]].last->endpoint_index = index_vertex[(i+2)%3];
          SM_Temp_vertex_list[index_vertex[i]].last->edge_index = existing_edge2;
          SM_Temp_vertex_list[index_vertex[i]].last->next = (SM_Incident_edge_ptr)NULL;
          tempo->next = SM_Temp_vertex_list[index_vertex[i]].last;
          SM_Temp_vertex_list[index_vertex[i]].last->next = NULL;
          
          tempo = SM_Temp_vertex_list[index_vertex[(i+1)%3]].last;
          if (tempo == NULL) {
            SM_Temp_vertex_list[index_vertex[(i+1)%3]].first = (SM_Incident_edge_ptr)calloc(1,sizeof(SM_Incident_edge));
            SM_Temp_vertex_list[index_vertex[(i+1)%3]].last = SM_Temp_vertex_list[index_vertex[(i+1)%3]].first;
            SM_Temp_vertex_list[index_vertex[(i+1)%3]].first->endpoint_index = index_vertex[i];
            SM_Temp_vertex_list[index_vertex[(i+1)%3]].first->edge_index = existing_edge1;
            SM_Temp_vertex_list[index_vertex[(i+1)%3]].first->next = (SM_Incident_edge_ptr)NULL;    
          }
          else {
            SM_Temp_vertex_list[index_vertex[(i+1)%3]].last = (SM_Incident_edge_ptr)calloc(1,sizeof(SM_Incident_edge));
            SM_Temp_vertex_list[index_vertex[(i+1)%3]].last->endpoint_index = index_vertex[i];
            SM_Temp_vertex_list[index_vertex[(i+1)%3]].last->edge_index = existing_edge1;
            SM_Temp_vertex_list[index_vertex[(i+1)%3]].last->next = (SM_Incident_edge_ptr)NULL;
            tempo->next = SM_Temp_vertex_list[index_vertex[(i+1)%3]].last;
            SM_Temp_vertex_list[index_vertex[(i+1)%3]].last->next = NULL;
          }	
          tempo = (SM_Incident_edge_ptr)(SM_Temp_vertex_list[index_vertex[(i+2)%3]].last);
          if (tempo == NULL) {
            SM_Temp_vertex_list[index_vertex[(i+2)%3]].first = (SM_Incident_edge_ptr)calloc(1,sizeof(SM_Incident_edge));
            SM_Temp_vertex_list[index_vertex[(i+2)%3]].last = SM_Temp_vertex_list[index_vertex[(i+2)%3]].first;
            SM_Temp_vertex_list[index_vertex[(i+2)%3]].first->endpoint_index = index_vertex[i];
            SM_Temp_vertex_list[index_vertex[(i+2)%3]].first->edge_index = existing_edge2;
            SM_Temp_vertex_list[index_vertex[(i+2)%3]].first->next = (SM_Incident_edge_ptr)NULL;   		    
          }
          else {		    
            SM_Temp_vertex_list[index_vertex[(i+2)%3]].last = (SM_Incident_edge_ptr)calloc(1,sizeof(SM_Incident_edge));
            SM_Temp_vertex_list[index_vertex[(i+2)%3]].last->endpoint_index = index_vertex[i];
            SM_Temp_vertex_list[index_vertex[(i+2)%3]].last->edge_index = existing_edge2;
            SM_Temp_vertex_list[index_vertex[(i+2)%3]].last->next = (SM_Incident_edge_ptr)NULL;
            tempo->next = SM_Temp_vertex_list[index_vertex[(i+2)%3]].last;
            SM_Temp_vertex_list[index_vertex[(i+2)%3]].last->next = NULL;
          }	
        }
        if (i==0) {
          tri_ptr->edge1 = existing_edge1;
        }
        else if (i==1) {
          tri_ptr->edge2 = existing_edge1;
        }
        else {
          tri_ptr->edge3 = existing_edge1;
        }
    }
    tri_ptr->normal_good = NO;
    tri_ptr->picked = (BOOL_STACK) NO;
    tri_ptr->sm_flag = NO;
    tri_ptr->user_flag = NO;
    tri_ptr->triangle_id = index;
    SM_triangle_material_set(the_surface, index, material_link[mat_index]);
    tri_ptr->normal1 = 
      SM_vertex_normal_ref_add(the_surface, tri_ptr->vertex1, index, tri_ptr->edge1, tri_ptr->edge3);
    tri_ptr->normal2 = 
      SM_vertex_normal_ref_add(the_surface, tri_ptr->vertex2, index, tri_ptr->edge2, tri_ptr->edge1);
    tri_ptr->normal3 = 
      SM_vertex_normal_ref_add(the_surface, tri_ptr->vertex3, index, tri_ptr->edge3, tri_ptr->edge2);
  }
  free(material_link);
  fread(the_surface->model_transform, sizeof(float), (W_INDEX+1)*(W_INDEX+1), fil);
  for (i=0; i<the_surface->current_vert_list_size; i++) {
    tempo = current = SM_Temp_vertex_list[i].first;
    while (current != NULL) {
      current = current->next;
      free(tempo);
      tempo = current;
    }
  }
  free(SM_Temp_vertex_list);
  fclose(fil);
  return(the_surface);
    }
    
    return(NULL);
}


SM_Surface_ptr
SM_surface_readtxt(name)
char *name;
/*
**  Reads the contents of a stored surface from file 'name'
**  the_materials is the application's material list.
**  and returns a pointer to the new structure.
**  Returns NULL if name is not a valid and existing filename.
*/
{
/*
* Local types used to build edges list from vertices and triangles.
  */
  typedef struct SM_incident_edge {
    int edge_index;
    int endpoint_index;
    struct SM_incident_edge *next;
  }SM_Incident_edge, *SM_Incident_edge_ptr;
  
  typedef struct SM_temp_vertex {
    SM_Incident_edge_ptr first;
    SM_Incident_edge_ptr last;
  }SM_Temp_vertex, *SM_Temp_vertex_ptr;
  
  Parser_File *fil;
  Parser_List_Level id1, id2, id3;
  int i,j,index,v1_index;
  SM_Surface_ptr the_surface;
  SM_Vertex_ptr new_vertex;
  SM_Edge_ptr edge_ptr;
  VECTOR_3D point;
  char filename[100];
  int local_vertex2;
  int local_vertex3;
  int mat_index;
  SM_Triangle_ptr tri_ptr;
  
  SM_Temp_vertex_ptr SM_Temp_vertex_list;
  SM_Incident_edge_ptr current_edge;
  int index_vertex[3];
  int existing_edge1, existing_edge2;
  SM_Incident_edge_ptr tempo, current;
  
  /*
  **	Add .sm terminator if not present in the name.
  */
  if (!(strstr(name, ".sm"))) {
    sprintf(filename, "%s.sm", name);
    fil = parser_open_file(filename);
  } else
    fil = parser_open_file(name);
  
  if (fil) {
    the_surface = SM_surface_new();
    
    parser_bgn_list(fil, &id1);
    
    /*
    **  SM_Vertex.
    */
    parser_bgn_list(fil, &id2);
    index = -1;
    while (!parser_end_of_list(fil, id2)) { 
      parser_bgn_list(fil, &id3);
		    /*
        **  Test if another SM_Vertex exists by testing for first component.
		    */
		    if (SM_VECTOR_3D_read(fil, point, id3)) {
          index++;
          if (index >= the_surface->vert_list_size)   
            SM_vertex_allocate(the_surface);
          
          new_vertex = &the_surface->vert_list[index];
          new_vertex->struct_used = YES;
          new_vertex->picked = (BOOL_STACK) NO;
          new_vertex->vertex_id = index;
          new_vertex->norm_list = NULL;
          new_vertex->norm_list_size = 0;
          new_vertex->func_result = 0.0;
          SM_vertex_point_set(the_surface, index, point);
          the_surface->current_vert_list_size += 1;
        }
        parser_end_list(fil, id3);
    }
    parser_end_list(fil, id2);
    SM_Temp_vertex_list = (SM_Temp_vertex_ptr)calloc(the_surface->current_vert_list_size, sizeof(SM_Temp_vertex));
    for (i=0;i<the_surface->current_vert_list_size; i++) {
      SM_Temp_vertex_list[i].first = NULL;
      SM_Temp_vertex_list[i].last = NULL;
    }
    /*
    **  SM_Mat.
    */
    SM_material_read_fil(fil);
    /*
    **  SM_Triangle.
    **  And construction of SM_Edge
    */
    parser_bgn_list(fil, &id2);
    
    
    index = -1;
    while (!parser_end_of_list(fil, id2)) { 
      parser_bgn_list(fil, &id3);
      
      if (parser_read_int(fil, id3, &v1_index)) {
        index++;
        if (index >= the_surface->tri_list_size) 
          SM_triangle_allocate(the_surface);
        
        tri_ptr = &the_surface->tri_list[index];
        the_surface->current_tri_list_size += 1;
        tri_ptr->struct_used = YES;
        parser_read_int(fil, id3, &local_vertex2);
        parser_read_int(fil, id3, &local_vertex3);
        parser_read_int(fil, id3, &mat_index);
        tri_ptr->vertex1 = v1_index;
        tri_ptr->vertex2 = local_vertex2;
        tri_ptr->vertex3 = local_vertex3;
        tri_ptr->edge1 = tri_ptr->edge2 = tri_ptr->edge3 = -1;
        tri_ptr->region_number = 0;
        tri_ptr->region_list = NULL;
        V3D_INIT(index_vertex, v1_index, local_vertex2, local_vertex3);
        for (i=0; i<3; i++) {
          if (SM_Temp_vertex_list[(index_vertex[i])].first != NULL) {
            existing_edge1 = -1;
            existing_edge2 = -1;
            current_edge = SM_Temp_vertex_list[(index_vertex[i])].first;
            while ( current_edge != (SM_Incident_edge_ptr)NULL) {
              if (current_edge->endpoint_index == index_vertex[(i+1)%3]) {
                existing_edge1 = current_edge->edge_index;
                if (existing_edge2 != -1) break;
              }
              if (current_edge->endpoint_index == index_vertex[(i+2)%3]) {
                existing_edge2 = current_edge->edge_index;
                if (existing_edge1 != -1) break;
              }
              current_edge = current_edge->next;
            }
            if (existing_edge1 != -1) {
            /*
            * Il existe deja une arete entre i et i+1
              */ 
              if (the_surface->edge_list[existing_edge1].two_triangles == NO) {
                if (the_surface->edge_list[existing_edge1].triangle1 != index) {
                  the_surface->edge_list[existing_edge1].triangle2 = index;
                  the_surface->edge_list[existing_edge1].two_triangles = YES;
                }
              }	    
            }
            else {
            /*
            * Sinon il faut en creer une 
              */
              if (the_surface->current_edge_list_size >= the_surface->edge_list_size) {
                SM_edge_allocate(the_surface);
              }
              edge_ptr = &the_surface->edge_list[the_surface->current_edge_list_size];
              the_surface->current_edge_list_size += 1;
              edge_ptr->struct_used = YES;
              edge_ptr->two_triangles = NO;
              edge_ptr->picked = STACK_NO;
              edge_ptr->edge_id = existing_edge1 = the_surface->current_edge_list_size-1;
              edge_ptr->triangle1 = index;
              edge_ptr->triangle2 = SM_INDEX_ERROR;
              edge_ptr->vertex1 = index_vertex[i];
              edge_ptr->vertex2 = index_vertex[(i+1)%3];
              tempo = SM_Temp_vertex_list[index_vertex[i]].last;
              if (tempo == NULL) {
                SM_Temp_vertex_list[index_vertex[i]].first = (SM_Incident_edge_ptr)calloc(1,sizeof(SM_Incident_edge));
                SM_Temp_vertex_list[index_vertex[i]].last = SM_Temp_vertex_list[index_vertex[i]].first;
                SM_Temp_vertex_list[index_vertex[i]].first->endpoint_index = index_vertex[(i+1)%3];
                SM_Temp_vertex_list[index_vertex[i]].first->edge_index = existing_edge1;
                SM_Temp_vertex_list[index_vertex[i]].first->next = (SM_Incident_edge_ptr)NULL;    
              }
              else {
                SM_Temp_vertex_list[index_vertex[i]].last = (SM_Incident_edge_ptr)calloc(1,sizeof(SM_Incident_edge));
                SM_Temp_vertex_list[index_vertex[i]].last->endpoint_index = index_vertex[(i+1)%3];
                SM_Temp_vertex_list[index_vertex[i]].last->edge_index = existing_edge1;
                SM_Temp_vertex_list[index_vertex[i]].last->next = (SM_Incident_edge_ptr)NULL;
                tempo->next = SM_Temp_vertex_list[index_vertex[i]].last;
                SM_Temp_vertex_list[index_vertex[i]].last->next = NULL;
              }	
              tempo = SM_Temp_vertex_list[index_vertex[(i+1)%3]].last;
              if (tempo == NULL) {
                SM_Temp_vertex_list[index_vertex[(i+1)%3]].first = (SM_Incident_edge_ptr)calloc(1,sizeof(SM_Incident_edge));
                SM_Temp_vertex_list[index_vertex[(i+1)%3]].last = SM_Temp_vertex_list[index_vertex[(i+1)%3]].first;
                SM_Temp_vertex_list[index_vertex[(i+1)%3]].first->endpoint_index = index_vertex[i];
                SM_Temp_vertex_list[index_vertex[(i+1)%3]].first->edge_index = existing_edge1;
                SM_Temp_vertex_list[index_vertex[(i+1)%3]].first->next = (SM_Incident_edge_ptr)NULL;    
              }
              else {
                SM_Temp_vertex_list[index_vertex[(i+1)%3]].last = (SM_Incident_edge_ptr)calloc(1,sizeof(SM_Incident_edge));
                SM_Temp_vertex_list[index_vertex[(i+1)%3]].last->endpoint_index = index_vertex[i];
                SM_Temp_vertex_list[index_vertex[(i+1)%3]].last->edge_index = existing_edge1;
                SM_Temp_vertex_list[index_vertex[(i+1)%3]].last->next = (SM_Incident_edge_ptr)NULL;
                tempo->next = SM_Temp_vertex_list[index_vertex[(i+1)%3]].last;
                SM_Temp_vertex_list[index_vertex[(i+1)%3]].last->next = NULL;
              }	
            }
            if (existing_edge2 != -1) {
            /*
            * Il existe deja une arete entre i et i+2
              */ 
              if (the_surface->edge_list[existing_edge2].two_triangles == NO) {
                if (the_surface->edge_list[existing_edge2].triangle1 != index) {
                  the_surface->edge_list[existing_edge2].triangle2 = index;
                  the_surface->edge_list[existing_edge2].two_triangles = YES;
                }
              }	    
            }
            else {
              if (the_surface->current_edge_list_size >= the_surface->edge_list_size) { 
                SM_edge_allocate(the_surface);
              }
              edge_ptr = &the_surface->edge_list[the_surface->current_edge_list_size];
              the_surface->current_edge_list_size += 1;
              edge_ptr->struct_used = YES;
              edge_ptr->two_triangles = NO;
              edge_ptr->picked = STACK_NO;
              edge_ptr->edge_id = existing_edge2 = the_surface->current_edge_list_size-1;
              edge_ptr->triangle1 = index;
              edge_ptr->triangle2 = SM_INDEX_ERROR;
              edge_ptr->vertex1 = index_vertex[i];
              edge_ptr->vertex2 = index_vertex[(i+2)%3];
              tempo = SM_Temp_vertex_list[index_vertex[i]].last;
              if (tempo == NULL) {
                SM_Temp_vertex_list[index_vertex[i]].first = (SM_Incident_edge_ptr)calloc(1,sizeof(SM_Incident_edge));
                SM_Temp_vertex_list[index_vertex[i]].last = SM_Temp_vertex_list[index_vertex[i]].first;
                SM_Temp_vertex_list[index_vertex[i]].first->endpoint_index = index_vertex[(i+2)%3];
                SM_Temp_vertex_list[index_vertex[i]].first->edge_index = existing_edge2;
                SM_Temp_vertex_list[index_vertex[i]].first->next = (SM_Incident_edge_ptr)NULL;   		    
              }
              else {		    
                SM_Temp_vertex_list[index_vertex[i]].last = (SM_Incident_edge_ptr)calloc(1,sizeof(SM_Incident_edge));
                SM_Temp_vertex_list[index_vertex[i]].last->endpoint_index = index_vertex[(i+2)%3];
                SM_Temp_vertex_list[index_vertex[i]].last->edge_index = existing_edge2;
                SM_Temp_vertex_list[index_vertex[i]].last->next = (SM_Incident_edge_ptr)NULL;
                tempo->next = SM_Temp_vertex_list[index_vertex[i]].last;
                SM_Temp_vertex_list[index_vertex[i]].last->next = NULL;
              }	
              tempo = (SM_Incident_edge_ptr)(SM_Temp_vertex_list[index_vertex[(i+2)%3]].last);
              if (tempo == NULL) {
                SM_Temp_vertex_list[index_vertex[(i+2)%3]].first = (SM_Incident_edge_ptr)calloc(1,sizeof(SM_Incident_edge));
                SM_Temp_vertex_list[index_vertex[(i+2)%3]].last = SM_Temp_vertex_list[index_vertex[(i+2)%3]].first;
                SM_Temp_vertex_list[index_vertex[(i+2)%3]].first->endpoint_index = index_vertex[i];
                SM_Temp_vertex_list[index_vertex[(i+2)%3]].first->edge_index = existing_edge2;
                SM_Temp_vertex_list[index_vertex[(i+2)%3]].first->next = (SM_Incident_edge_ptr)NULL;   		    
              }
              else {		    
                SM_Temp_vertex_list[index_vertex[(i+2)%3]].last = (SM_Incident_edge_ptr)calloc(1,sizeof(SM_Incident_edge));
                SM_Temp_vertex_list[index_vertex[(i+2)%3]].last->endpoint_index = index_vertex[i];
                SM_Temp_vertex_list[index_vertex[(i+2)%3]].last->edge_index = existing_edge2;
                SM_Temp_vertex_list[index_vertex[(i+2)%3]].last->next = (SM_Incident_edge_ptr)NULL;
                tempo->next = SM_Temp_vertex_list[index_vertex[(i+2)%3]].last;
                SM_Temp_vertex_list[index_vertex[(i+2)%3]].last->next = NULL;
              }	
            }
      }
      else {	
				    if (the_surface->current_edge_list_size >= the_surface->edge_list_size) {
              SM_edge_allocate(the_surface);
            }
            edge_ptr = &the_surface->edge_list[the_surface->current_edge_list_size];
            the_surface->current_edge_list_size += 1;
            edge_ptr->struct_used = YES;
            edge_ptr->two_triangles = NO;
            edge_ptr->picked = STACK_NO;
            edge_ptr->edge_id = existing_edge1 = the_surface->current_edge_list_size-1;
            edge_ptr->triangle1 = index;
            edge_ptr->triangle2 = SM_INDEX_ERROR;
            edge_ptr->vertex1 = index_vertex[i];
            edge_ptr->vertex2 = index_vertex[(i+1)%3];
            if (the_surface->current_edge_list_size >= the_surface->edge_list_size) {
              SM_edge_allocate(the_surface);
            }
            edge_ptr = &the_surface->edge_list[the_surface->current_edge_list_size];
            the_surface->current_edge_list_size += 1;
            edge_ptr->struct_used = YES;
            edge_ptr->two_triangles = NO;
            edge_ptr->picked = STACK_NO;
            edge_ptr->edge_id = existing_edge2 = the_surface->current_edge_list_size-1;
            edge_ptr->triangle1 = index;
            edge_ptr->triangle2 = SM_INDEX_ERROR;
            edge_ptr->vertex1 = index_vertex[i];
            edge_ptr->vertex2 = index_vertex[(i+2)%3];
            
            SM_Temp_vertex_list[index_vertex[i]].first = (SM_Incident_edge_ptr)calloc(1,sizeof(SM_Incident_edge));
            SM_Temp_vertex_list[index_vertex[i]].last = SM_Temp_vertex_list[index_vertex[i]].first;
            SM_Temp_vertex_list[index_vertex[i]].first->endpoint_index = index_vertex[(i+1)%3];
            SM_Temp_vertex_list[index_vertex[i]].first->edge_index = existing_edge1;
            SM_Temp_vertex_list[index_vertex[i]].first->next = (SM_Incident_edge_ptr)NULL;
            
            tempo = SM_Temp_vertex_list[index_vertex[i]].last;
            SM_Temp_vertex_list[index_vertex[i]].last = (SM_Incident_edge_ptr)calloc(1,sizeof(SM_Incident_edge));
            SM_Temp_vertex_list[index_vertex[i]].last->endpoint_index = index_vertex[(i+2)%3];
            SM_Temp_vertex_list[index_vertex[i]].last->edge_index = existing_edge2;
            SM_Temp_vertex_list[index_vertex[i]].last->next = (SM_Incident_edge_ptr)NULL;
            tempo->next = SM_Temp_vertex_list[index_vertex[i]].last;
            SM_Temp_vertex_list[index_vertex[i]].last->next = NULL;
            
            tempo = SM_Temp_vertex_list[index_vertex[(i+1)%3]].last;
            if (tempo == NULL) {
              SM_Temp_vertex_list[index_vertex[(i+1)%3]].first = (SM_Incident_edge_ptr)calloc(1,sizeof(SM_Incident_edge));
              SM_Temp_vertex_list[index_vertex[(i+1)%3]].last = SM_Temp_vertex_list[index_vertex[(i+1)%3]].first;
              SM_Temp_vertex_list[index_vertex[(i+1)%3]].first->endpoint_index = index_vertex[i];
              SM_Temp_vertex_list[index_vertex[(i+1)%3]].first->edge_index = existing_edge1;
              SM_Temp_vertex_list[index_vertex[(i+1)%3]].first->next = (SM_Incident_edge_ptr)NULL;    
            }
            else {
              SM_Temp_vertex_list[index_vertex[(i+1)%3]].last = (SM_Incident_edge_ptr)calloc(1,sizeof(SM_Incident_edge));
              SM_Temp_vertex_list[index_vertex[(i+1)%3]].last->endpoint_index = index_vertex[i];
              SM_Temp_vertex_list[index_vertex[(i+1)%3]].last->edge_index = existing_edge1;
              SM_Temp_vertex_list[index_vertex[(i+1)%3]].last->next = (SM_Incident_edge_ptr)NULL;
              tempo->next = SM_Temp_vertex_list[index_vertex[(i+1)%3]].last;
              SM_Temp_vertex_list[index_vertex[(i+1)%3]].last->next = NULL;
            }	
            tempo = (SM_Incident_edge_ptr)(SM_Temp_vertex_list[index_vertex[(i+2)%3]].last);
            if (tempo == NULL) {
              SM_Temp_vertex_list[index_vertex[(i+2)%3]].first = (SM_Incident_edge_ptr)calloc(1,sizeof(SM_Incident_edge));
              SM_Temp_vertex_list[index_vertex[(i+2)%3]].last = SM_Temp_vertex_list[index_vertex[(i+2)%3]].first;
              SM_Temp_vertex_list[index_vertex[(i+2)%3]].first->endpoint_index = index_vertex[i];
              SM_Temp_vertex_list[index_vertex[(i+2)%3]].first->edge_index = existing_edge2;
              SM_Temp_vertex_list[index_vertex[(i+2)%3]].first->next = (SM_Incident_edge_ptr)NULL;   		    
            }
            else {		    
              SM_Temp_vertex_list[index_vertex[(i+2)%3]].last = (SM_Incident_edge_ptr)calloc(1,sizeof(SM_Incident_edge));
              SM_Temp_vertex_list[index_vertex[(i+2)%3]].last->endpoint_index = index_vertex[i];
              SM_Temp_vertex_list[index_vertex[(i+2)%3]].last->edge_index = existing_edge2;
              SM_Temp_vertex_list[index_vertex[(i+2)%3]].last->next = (SM_Incident_edge_ptr)NULL;
              tempo->next = SM_Temp_vertex_list[index_vertex[(i+2)%3]].last;
              SM_Temp_vertex_list[index_vertex[(i+2)%3]].last->next = NULL;
            }	
      }
      if (i==0) {
        tri_ptr->edge1 = existing_edge1;
      }
      else if (i==1) {
        tri_ptr->edge2 = existing_edge1;
      }
      else {
        tri_ptr->edge3 = existing_edge1;
      }
        }
        tri_ptr->normal_good = NO;
        tri_ptr->picked = (BOOL_STACK) NO;
        tri_ptr->sm_flag = NO;
        tri_ptr->user_flag = NO;
        tri_ptr->triangle_id = index;
        SM_triangle_material_set(the_surface, index, material_link[mat_index]);
        tri_ptr->normal1 = 
          SM_vertex_normal_ref_add(the_surface, tri_ptr->vertex1, index, tri_ptr->edge1, tri_ptr->edge3);
        tri_ptr->normal2 = 
          SM_vertex_normal_ref_add(the_surface, tri_ptr->vertex2, index, tri_ptr->edge2, tri_ptr->edge1);
        tri_ptr->normal3 = 
          SM_vertex_normal_ref_add(the_surface, tri_ptr->vertex3, index, tri_ptr->edge3, tri_ptr->edge2);
     } 
     parser_end_list(fil, id3);
            }
            parser_end_list(fil, id2);
            free(material_link);
            /*
            **  Matrix.
            */
            parser_bgn_list(fil, &id2);
            for (i = X_INDEX; i <= W_INDEX; i++)
              for (j = X_INDEX; j <= W_INDEX; j++) 
                parser_read_float(fil, id2, &the_surface->model_transform[i][j]);
              parser_end_list(fil, id2);
              parser_end_list(fil, id1);
              parser_close_file(fil);
              for (i=0; i<the_surface->current_vert_list_size; i++) {
                tempo = current = SM_Temp_vertex_list[i].first;
                while (current != NULL) {
                  current = current->next;
                  free(tempo);
                  tempo = current;
                }
              }
              free(SM_Temp_vertex_list);
              return(the_surface);
    }
    return(NULL);
}

BOOL
SM_surface_write(the_surface, name)
char *name;
SM_Surface_ptr the_surface;
/*
**  Saves the contents of the_surface in the file 'name'.
**  Returns YES if filename is good.
*/
{
  FILE *fil;
  register int new_id;
  char filename[100];
  int material_number;
  
  /*
  **	Add .sm terminator if not present in the name.
  */
  if (!(strstr(name, ".sm"))) {
    sprintf(filename, "%s.sm", name);
    fil = fopen(filename, "w");
  } else
    fil = fopen(name, "w");
  new_id = 0;
  SM_FOR_EACH_VERTEX(the_surface)
    vertex_ptr->vertex_id = new_id++;
  SM_END_FOR_EACH
    new_id = 0;
  if (fil) {
    fwrite(&(the_surface->current_vert_list_size), sizeof(int), 1, fil);
    SM_FOR_EACH_VERTEX(the_surface)
      fwrite(vertex_ptr->point, sizeof(float), 3, fil);
    SM_END_FOR_EACH
      material_number = 0;
    SM_FOR_EACH_MATERIAL
      material_ptr->flag = NO;
    SM_END_FOR_EACH
      
      SM_FOR_EACH_TRIANGLE(the_surface)
      if (triangle_ptr->material_index != SM_INDEX_ERROR) {
        the_materials.mat_list[triangle_ptr->material_index].flag = YES;
      }
      SM_END_FOR_EACH
        SM_FOR_EACH_MATERIAL
        if (material_ptr->flag)
          material_number++;
        SM_END_FOR_EACH
          fwrite(&material_number, sizeof(int), 1, fil);
        SM_FOR_EACH_MATERIAL
          if (material_ptr->flag) {
            fwrite(material_ptr->name, sizeof(char), SM_MATERIAL_NAME_LENGTH, fil);
            fwrite(material_ptr->gl_properties, sizeof(float), SM_MAT_LIST_LENGTH, fil);
            fwrite(&material_ptr->reflectivity, sizeof(float), 1, fil);
          }
          SM_END_FOR_EACH
            SM_FOR_EACH_MATERIAL
            if (material_ptr->flag)
              material_ptr->material_id = new_id++;
            SM_END_FOR_EACH
              fwrite(&the_surface->current_tri_list_size, sizeof(int), 1, fil);
            SM_FOR_EACH_TRIANGLE(the_surface)
              fwrite(&(the_surface->vert_list[triangle_ptr->vertex1].vertex_id), sizeof(short), 1, fil);		
            fwrite(&(the_surface->vert_list[triangle_ptr->vertex2].vertex_id), sizeof(short), 1, fil);		
            fwrite(&(the_surface->vert_list[triangle_ptr->vertex3].vertex_id), sizeof(short), 1, fil);		
            fwrite(&(the_materials.mat_list[triangle_ptr->material_index].material_id), sizeof(short), 1, fil);		
            SM_END_FOR_EACH
              fwrite(the_surface->model_transform, sizeof(float), (W_INDEX+1)*(W_INDEX+1), fil);
            fclose(fil);
            return(YES);
  }
  else return(NO);
}

BOOL
SM_surface_writetxt(the_surface, name)
char *name;
SM_Surface_ptr the_surface;
/*
**  Saves the contents of the_surface in the file 'name'.
**  Returns YES if filename is good.
*/
{
  FILE *fil;
  register int i, j, new_id;
  char filename[100];
  
  /*
  **	Add .sm terminator if not present in the name.
  */
  if (!(strstr(name, ".sm"))) {
    sprintf(filename, "%s.sm", name);
    fil = fopen(filename, "w");
  } else
    fil = fopen(name, "w");
  new_id = 0;
  SM_FOR_EACH_VERTEX(the_surface)
    vertex_ptr->vertex_id = new_id++;
  SM_END_FOR_EACH
    new_id = 0;
  if (fil) {
    fprintf(fil,"%s\n[\n", SM_version);
    
    /*
    **  SM_Vertex.
    */
    fprintf(fil,"vertices %d\n{\n", the_surface->current_vert_list_size);
    SM_FOR_EACH_VERTEX(the_surface)
      SM_vertex_write(fil, the_surface, vertex_index);
    SM_END_FOR_EACH
      fprintf(fil,"}\n");
      /*
      **  SM_Material.
    */
    SM_surface_material_write_fil(the_surface, fil);
    SM_FOR_EACH_MATERIAL
      if (material_ptr->flag)
        material_ptr->material_id = new_id++;
      SM_END_FOR_EACH
        
      /*
      **  SM_Triangles.
      */
      fprintf(fil,"triangles %d\n{\n", the_surface->current_tri_list_size);
      SM_FOR_EACH_TRIANGLE(the_surface)
        SM_triangle_write(fil, the_surface, triangle_index);
      SM_END_FOR_EACH
        fprintf(fil,"}\n");
      
        /*
        **  Matrice.
      */
      fprintf(fil,"matrix\n(\n");
      for (i = X_INDEX; i <= W_INDEX; i++) {
        for (j = X_INDEX; j <= W_INDEX; j++)
          fprintf(fil,"%g,",the_surface->model_transform[i][j]);
        fprintf(fil,"\n");
      }
      fprintf(fil,")\n");
      
      fprintf(fil,"]\n");
      fclose(fil);
      return(YES);
  }
  else return(NO);
}

void
SM_surface_fuse(surface1, surface2, merge_vertices)
SM_Surface_ptr surface1, surface2;
int merge_vertices;
/*
**  Fuses surface2 into surface1.
*/
{
  int i;
  float *vector, diff;
  
  SM_FOR_EACH_VERTEX(surface2)
    i = SM_INDEX_ERROR;
  
    /*
    **  If the same vertex exists in surface1, use it.
  */
  if (merge_vertices) {
    vector = vertex_ptr->point;
    SM_FOR_EACH_VERTEX(surface1)
      diff = vertex_ptr->point[0] - vector[0];
    if (diff > -SM_EPSILON && diff < SM_EPSILON) {
		    diff = vertex_ptr->point[1] - vector[1];
        if (diff > -SM_EPSILON && diff < SM_EPSILON) {
          diff = vertex_ptr->point[2] - vector[2];
          if (diff > -SM_EPSILON && diff < SM_EPSILON) { 
            i = vertex_index;
          }
        }
    }
    SM_END_FOR_EACH
  }
  if (i == SM_INDEX_ERROR) {
    vertex_ptr->vertex_id = SM_vertex_new(surface1);
    if (vertex_ptr->vertex_id != SM_INDEX_ERROR) 
      SM_vertex_point_set(surface1, vertex_ptr->vertex_id, vertex_ptr->point);
  } else {
    vertex_ptr->vertex_id = i;
  }
  SM_END_FOR_EACH
    
    SM_FOR_EACH_TRIANGLE(surface2)
    i = SM_triangle_new(surface1, 
    surface2->vert_list[triangle_ptr->vertex1].vertex_id,
    surface2->vert_list[triangle_ptr->vertex2].vertex_id,
    surface2->vert_list[triangle_ptr->vertex3].vertex_id);
  triangle_ptr->triangle_id = i;
  
  SM_triangle_material_set(surface1, i, triangle_ptr->material_index);
  SM_END_FOR_EACH
}


static VECTOR_3D old_amp;

BOOL
SM_surface_function_2D_set(the_surface, 
                           the_function,
                           origin_point)
                           SM_Surface_ptr the_surface;
float (* the_function)(float);
VECTOR_3D origin_point;
/*
**  This is used to specify the current radially symetric function to apply
**  on the currently picked vertices.
**  "the_function" is a 2D function with the domain [0,1[.
**  origin_point is a point in 3D space where the origin of the
**  function will be.
*/
{
  VECTOR_3D difference;
  BOOL ok = YES;
  float distance, max_dist = 0.0;
  
  /*
  **	Else, find the farthest point from the origin.  This will 
  **	determine the scope of the function.
  */
  SM_FOR_EACH_VERTEX(the_surface)
    if ((BOOL) IS_YES(vertex_ptr->picked)) {
      V3D_SUB(difference, vertex_ptr->point, origin_point);
      distance = V3D_NORM(difference);
      if (distance > max_dist)
        max_dist = distance;
    }
    
    SM_END_FOR_EACH
      
    /*
    **	The function result, when applied with SM_surface_function_apply,
    **	will be between 0 and |new_amp|.
    */
    if (max_dist < 1.0) max_dist = 1.0;
    
    SM_FOR_EACH_VERTEX(the_surface)
      if ((BOOL) IS_YES(vertex_ptr->picked)) {
        V3D_SUB(difference, vertex_ptr->point, origin_point);
        vertex_ptr->func_result = 
          (* the_function)(V3D_NORM(difference)/max_dist);
      }
      SM_END_FOR_EACH
        V3D_NULL(old_amp);
      return(ok);	
}


void
SM_surface_function_apply(the_surface, new_amp)
SM_Surface_ptr the_surface;
VECTOR_3D new_amp;
/*
**  After the function has been set for the current picked vertices 
**  configuration, the function is applied by giving with aplitude
**  and direction "new_amp".
**
**  newV = oldV - old_amp * f + new_amp * f
**  newV = oldV - f * (old_amp - new_amp)
*/
{
  register int j, * list_ptr;
  
  SM_FOR_EACH_VERTEX(the_surface)
    if ((BOOL) IS_YES(vertex_ptr->picked)) {
      vertex_ptr->point[X_INDEX] -= vertex_ptr->func_result * 
        (old_amp[X_INDEX] - new_amp[X_INDEX]);
      vertex_ptr->point[Y_INDEX] -= vertex_ptr->func_result * 
        (old_amp[Y_INDEX] - new_amp[Y_INDEX]);
      vertex_ptr->point[Z_INDEX] -= vertex_ptr->func_result * 
        (old_amp[Z_INDEX] - new_amp[Z_INDEX]);
      
        /*
        **	This is a sped up version of
        **	SM_vertex_point_set(the_surface, vertex_index, new_point);
      */
      SM_FOR_EACH_NORMAL(vertex_ptr)
        normal_ptr->normal_good = NO;
      
      /* Triangles' normals not good also. */
      for (   j = 0,
        list_ptr = normal_ptr->tri_ref_list;
      j < normal_ptr->tri_ref_list_occ;
      j++,
        list_ptr++   )
        
        the_surface->tri_list[*list_ptr].normal_good = NO;
      
      SM_END_FOR_EACH
    }
    SM_END_FOR_EACH
      V3D_COPY(old_amp, new_amp);
}

/* These are addional routines added by Laurent Moccozet in order
to be able to manipulate the data structure for some applications
like 'simplification' of surfaces.

  P kalra                                           oct '92
*/

/*
* Returns the actual number of vertices in the surface.
*/

int 
SM_vertex_count_get(the_surface)
SM_Surface_ptr the_surface;
{
  int count = 0;
  
  SM_FOR_EACH_VERTEX(the_surface)
    count++;
  SM_END_FOR_EACH
    return(count);
}

/*
* Returns the actual number of edges in the surface.
*/
int
SM_edge_count_get(the_surface)
SM_Surface_ptr the_surface;
{
  int count = 0;
  
  SM_FOR_EACH_EDGE(the_surface)
    count++;
  SM_END_FOR_EACH
    return(count);
}

/*
* Returns the actual number of triangles in the surface.
*/
int 
SM_triangle_count_get(the_surface)
SM_Surface_ptr the_surface;
{
  int count = 0;
  
  SM_FOR_EACH_TRIANGLE(the_surface)
    count++;
  SM_END_FOR_EACH
    return(count);
}


BOOL 
SM_vertex_incident_triangles(the_surface, vertex_index, list_tri, nb_tri)
SM_Surface_ptr the_surface;
int            vertex_index;
int            **list_tri;
int            *nb_tri;
{
  SM_Vertex_ptr vertex_ptr;
  int           *list_ptr,indice;
  
  if (! SM_vertex_index_valid(the_surface, vertex_index) ) return(NO);
  *nb_tri = 0;
  *list_tri = (int *)calloc(1, sizeof(int));    
  vertex_ptr = &(the_surface->vert_list[vertex_index]);    
  SM_FOR_EACH_NORMAL(vertex_ptr)
    for (   indice = 0,
      list_ptr = normal_ptr->tri_ref_list;
    indice < normal_ptr->tri_ref_list_occ;
    indice++,
      list_ptr++	) {
      (*nb_tri)++;
      *list_tri = (int *)realloc(*list_tri,(*nb_tri)*sizeof(int));
      (*list_tri)[*nb_tri] = *list_ptr;    
    }
    SM_END_FOR_EACH
      return(YES);
}

/*
*  given a surface and the index of a vertex,  return the BOOL value YES if at least
*  one of its incident edges is on the boundary of a hole.
*  return the index of the first found edge belonging to a hole.
* 
*  ERROR : the index of the vertex is not valid for the surface.
*/

BOOL 
SM_vertex_hole_get(the_surface, vertex_index, e, hole)
SM_Surface_ptr the_surface;
int            vertex_index;
int            *e;
BOOL           *hole;
{
  SM_Vertex_ptr vertex_ptr;
  int           *list_ptr,indice, i;
  int           ed[3];
  int	          s1, s2;
  BOOL          IsHole;
  
  if (! SM_vertex_index_valid(the_surface, vertex_index) ) return(NO);
  vertex_ptr = &(the_surface->vert_list[vertex_index]);    
  SM_FOR_EACH_NORMAL(vertex_ptr)
    for (   indice = 0,
      list_ptr = normal_ptr->tri_ref_list;
    indice < normal_ptr->tri_ref_list_occ;
    indice++,
      list_ptr++	) {
      SM_triangle_edge_index_get(the_surface, *list_ptr, &ed[0], &ed[1], &ed[2]);
      for (i=0; i<3; i++) {
        SM_edge_vertices_get(the_surface, ed[i], &s1, &s2);
        if ( (s1 == vertex_index) || (s2 == vertex_index) ) {
          SM_edge_hole_get(the_surface, ed[i], &IsHole);
          if ( IsHole ) {
            *e = ed[i];
            *hole = YES;
            return(YES);
          }
        }
      }
    }
    SM_END_FOR_EACH
      *hole = NO;
    return(YES);
}

/*
* Unpick all vertices, edges, triangles of the surface
*/

void
SM_surface_unpick_all(the_surface)
SM_Surface_ptr the_surface;
{
  SM_FOR_EACH_VERTEX(the_surface)
    SM_vertex_picked_set(the_surface, vertex_index, NO);
  SM_END_FOR_EACH
    SM_FOR_EACH_EDGE(the_surface)
    SM_edge_picked_set(the_surface, edge_index, NO);
  SM_END_FOR_EACH
    SM_FOR_EACH_TRIANGLE(the_surface)
    SM_triangle_picked_set(the_surface, triangle_index, NO);
  SM_END_FOR_EACH
}


/*
* Pick all vertices, edges, triangles of the surface
*/

void
SM_surface_pick_all(the_surface)
SM_Surface_ptr the_surface;
{
  SM_FOR_EACH_VERTEX(the_surface)
    SM_vertex_picked_set(the_surface, vertex_index, YES);
  SM_END_FOR_EACH
    SM_FOR_EACH_EDGE(the_surface)
    SM_edge_picked_set(the_surface, edge_index, YES);
  SM_END_FOR_EACH
    SM_FOR_EACH_TRIANGLE(the_surface)
    SM_triangle_picked_set(the_surface, triangle_index, YES);
  SM_END_FOR_EACH
}

/*
* Invert the picked flag for all the triangles of the surface.
*/

void
SM_triangle_picked_invert(the_surface)
SM_Surface_ptr the_surface;
{
  BOOL picked;
  
  SM_FOR_EACH_TRIANGLE(the_surface)
    SM_triangle_picked_get(the_surface, triangle_index, &picked);
  SM_triangle_picked_set(the_surface, triangle_index, !picked);
  SM_END_FOR_EACH
}
/*----------------------------------------------------------------------*/

