
/*------------------------------------------------------------------------------*/
/*										*/
/*										*/
/*  This software module was originally developed by                            */
/*        Prem Kalra, MIRALab, University of Geneva                             */
/*  and edited by                                                               */
/*        Igor Pandzic, MIRALab, University of Geneva                           */
/*        Marc Escher, MIRALab, University of Geneva                            */
/*        Pierre Beylot, MIRALab, University of Geneva                          */
/*  in the course of development of the MPEG-4 Video (ISO/IEC 14496-2).	        */
/*  This software module is an implementation of a part of one or more	        */
/*  MPEG-4 Video (ISO/IEC 14496-2) tools as specified by the MPEG-4 Video       */
/*  (ISO/IEC 14496-2). ISO/IEC gives users of the                               */
/*  MPEG-4 Video (ISO/IEC 14496-2) free license to this software module or	*/
/*  modifications thereof for use in hardware or software products claiming 	*/
/*  conformance to the MPEG-4 Video (ISO/IEC 14496-2). Those intending to use	*/
/*  this software module in hardware or software products are advised that	*/
/*  its use may infringe existing patents. The original developer of this	*/
/*  software module and his/her company, the subsequent editors and their	*/
/*  companies, and ISO/IEC have no liability for use of this software		*/
/*  module or modifications thereof. Copyright is not released for non		*/
/*  MPEG-4 Video (ISO/IEC 14496-2) conforming products.                         */
/*  MIRALab, University of Geneva retains full right to use the code for its    */
/*  own purpose, assign or donate the code to a third party and to inhibit      */
/*  third parties from using the code for non MPEG-4 Video (ISO/IEC 14496-2)    */
/*  conforming products. This copyright notice must be included in all copies   */
/*  or derivative works. Copyright (c) (1997).			      		*/
/*										*/
/*										*/
/*------------------------------------------------------------------------------*/


#ifndef SURFACE_MANAGER
#define SURFACE_MANAGER

#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include "vector_h.h"
#include "matrix_h.h"
#include "parser.h"

#define SM_UI	(unsigned int) (1)
#define SM_UL	(unsigned long) (1)

#define EMISSION		1
#define AMBIENT			2
#define DIFFUSE			3
#define SPECULAR		4
#define SHININESS		5
#define ALPHA			7


/*  
**  Geometry bits 0-2	
*/
#define SM_TRIANGLE		SM_UL
#define SM_VERTEX		SM_UL << 1
#define SM_EDGE			SM_UL << 2

/*
**  Coloring bits 3-5	
*/
#define SM_CONSTANT		SM_UL << 3	
#define SM_LIGHTED		SM_UL << 4
#define SM_SHADED		SM_UL << 5
#define SM_SHADOW		SM_UL << 16

/*
**  Normals bits 6	
*/
#define SM_NORMALS		SM_UL << 6

/*
**  Boolean attributs bits 7-14 
*/
#define SM_PICKED		SM_UL << 7
#define SM_NOT_PICKED		SM_UL << 8
#define SM_BIND			SM_UL << 9
#define SM_NOT_BIND		SM_UL << 10
#define SM_HOLES		SM_UL << 11
#define SM_NOT_HOLES		SM_UL << 12
#define SM_SQUARE		SM_UL << 13
#define SM_NOT_SQUARE		SM_UL << 14
#define SM_TO_PICK		SM_UL << 15

#ifndef M_PI
#define M_PI 3.14159265358979323846
#endif

#define NO 0
#define YES 1

#define ADD 0
#define SUB 1
#ifndef BOOL
#define BOOL int
#endif
#define PACKED_BOOL unsigned char

/*
**  Picking stack boolean values.
*/

#define BOOL_STACK unsigned int
#define STACK_NO (BOOL_STACK) (0)
#define STACK_YES (BOOL_STACK) (1)
#define IS_YES(stack_bool_value) (stack_bool_value & STACK_YES)


#define SM_INDEX_ERROR -1
#define SM_EPSILON 1e-06


/*
**  This needs a variable vertex_ptr pointing to a valid vertex.
*/
#define SM_FOR_EACH_NORMAL(vertex_ptr) \
    { \
	SM_Vertex_normal_ptr normal_ptr; \
	register int normal_index; \
	for (	normal_index = 0, \
		normal_ptr = vertex_ptr->norm_list; \
		normal_index < vertex_ptr->norm_list_size; \
		normal_index++, \
		normal_ptr++	) \
	    if (normal_ptr->struct_used) { 

/*
**  This needs the_surface pointing to a valid surface.
*/
#define SM_FOR_EACH_VERTEX(the_surface) \
    { \
	SM_Vertex_ptr vertex_ptr; \
	register int vertex_index; \
	for (	vertex_index = 0, \
		vertex_ptr = the_surface->vert_list; \
		vertex_index < the_surface->vert_list_size; \
		vertex_index++, \
		vertex_ptr++	) \
	    if (vertex_ptr->struct_used) { 

/*
**  This needs the_surface pointing to a valid surface.
*/
#define SM_FOR_EACH_TRIANGLE(the_surface) \
    { \
	SM_Triangle_ptr triangle_ptr; \
	register int triangle_index; \
	for (	triangle_index = 0, \
		triangle_ptr = the_surface->tri_list; \
		triangle_index < the_surface->tri_list_size; \
		triangle_index++, \
		triangle_ptr++	) \
	    if (triangle_ptr->struct_used) {

/*
**  This needs the_surface pointing to a valid surface.
*/
#define SM_FOR_EACH_EDGE(the_surface) \
    { \
	SM_Edge_ptr edge_ptr; \
	register int edge_index; \
	for (	edge_index = 0, \
		edge_ptr = the_surface->edge_list; \
		edge_index < the_surface->edge_list_size; \
		edge_index++, \
		edge_ptr++	) \
	    if (edge_ptr->struct_used) {


#define SM_FOR_EACH_MATERIAL \
    { \
	SM_Mat_ptr material_ptr; \
	register int material_index; \
	SM_Material_Name material_name; \
	for (	material_index = 0, \
		material_ptr = the_materials.mat_list; \
		material_index < the_materials.mat_list_size; \
		material_index++, \
		material_ptr++	) \
	    if (material_ptr->struct_used) { \
		strcpy(material_name, material_ptr->name);

#define SM_END_FOR_EACH  }}





/*
------------------------------   SM_Material  ---------------------------------
*/

#define SM_MATERIAL_NAME_LENGTH 24
#define SM_MAT_LIST_LENGTH 21

#define SM_EMISSION	0
#define SM_AMBIENT	4
#define SM_DIFFUSE	8
#define SM_SPECULAR	12
#define SM_SHININESS	16
#define SM_ALPHA	18

#define REFLECTIVITY	100

typedef char SM_Material_Name[SM_MATERIAL_NAME_LENGTH];

typedef struct {
    BOOL struct_used, flag, gldef;
    short material_id;	
    unsigned long rgba;
    SM_Material_Name name;
    float gl_properties[SM_MAT_LIST_LENGTH], 
	  reflectivity;
} SM_Mat, * SM_Mat_ptr;

typedef struct {
    int mat_list_size, glbind, glcpack;
    SM_Mat_ptr mat_list;
} SM_Material, * SM_Material_ptr;




/*
------------------------------  SM_Identifier  --------------------------------
*/

typedef int SM_Id;  /* not used anymore. */




/*
-------------------------------   SM_Vertex  ----------------------------------
*/

typedef struct SM_Vertex_normal{
    BOOL struct_used;
    PACKED_BOOL normal_good;
    VECTOR_3D normal;
    short tri_ref_list_size, tri_ref_list_occ;
    int * tri_ref_list;
} SM_Vertex_normal, * SM_Vertex_normal_ptr;

typedef struct SM_Vertex{
    BOOL struct_used;
    PACKED_BOOL flag;
    BOOL_STACK picked;
    short vertex_id;
    SM_Vertex_normal_ptr norm_list;
    short norm_list_size;
    float func_result;
    VECTOR_3D point; 
} SM_Vertex, * SM_Vertex_ptr;




/*
-------------------------------   SM_Triangle  ---------------------------------
*/
typedef struct SM_Triangle{
    BOOL struct_used;
    PACKED_BOOL normal_good, sm_flag, user_flag;
    BOOL_STACK picked;
    short region_number;
    int   *region_list;
    short triangle_id;
    short vertex1, vertex2, vertex3;
    short normal1, normal2, normal3;
    short edge1,   edge2,   edge3;
    short material_index;
    VECTOR_3D normal;
} SM_Triangle, * SM_Triangle_ptr;




/*
---------------------------------   SM_Edge  ------------------------------------
*/

typedef struct SM_Edge{
    BOOL struct_used;
    PACKED_BOOL flag, two_triangles;
    BOOL_STACK picked;
    short edge_id;
    short   vertex1, vertex2;
    short triangle1, triangle2; /* TEST ok */
} SM_Edge, * SM_Edge_ptr;


/*
------------------------------   SM_Surface  ---------------------------------
*/

typedef struct {
    int *free_ptr;
    int counter;  
}SM_Free_stack, *SM_Free_stack_ptr;

typedef struct {
    BOOL struct_used;

    int vert_list_size;
    int current_vert_list_size;
    SM_Vertex_ptr vert_list;
    SM_Free_stack free_vert_list;

    int tri_list_size;
    int current_tri_list_size;
    SM_Triangle_ptr tri_list;
    SM_Free_stack free_tri_list;

    int edge_list_size;
    int current_edge_list_size;
    SM_Edge_ptr edge_list;
    SM_Free_stack free_edge_list;

#ifdef CPLUSPLUS
    void *the_region_list;
#else
    struct SM_Region_List *the_region_list;
#endif
    MATRIX_H3D model_transform;
} SM_Surface, * SM_Surface_ptr;

#include "RegMan.h"

#ifdef CPLUSPLUS
extern "C" {
#endif

extern SM_Material the_materials;

/*
**  SurfMan routines.
*/

extern int
SM_vertex_new(SM_Surface_ptr);

extern BOOL
SM_vertex_dispose(SM_Surface_ptr, int);

extern BOOL
SM_vertex_index_valid(SM_Surface_ptr, int);

extern int
SM_vertex_index_point(SM_Surface_ptr, VECTOR_3D);

extern BOOL
SM_vertex_point_set(SM_Surface_ptr, int, VECTOR_3D);

extern BOOL
SM_vertex_point_get(SM_Surface_ptr, int, VECTOR_3D);

extern BOOL
SM_vertex_normal_index_valid(SM_Surface_ptr, int, int);

extern BOOL
SM_vertex_normal_set(SM_Surface_ptr, int, int, VECTOR_3D);

extern BOOL
SM_vertex_normal_get(SM_Surface_ptr, int, int, VECTOR_3D);

extern BOOL
SM_vertex_picked_set(SM_Surface_ptr, int, BOOL);

extern BOOL
SM_vertex_picked_get(SM_Surface_ptr, int, BOOL *);

extern void
SM_vertex_picked_push(SM_Surface_ptr);

extern void
SM_vertex_picked_pop(SM_Surface_ptr);

extern void
SM_vertex_picked_center(SM_Surface_ptr, VECTOR_3D);

extern int
SM_vertex_picked_count(SM_Surface_ptr);





extern int
SM_triangle_new(SM_Surface_ptr, int, int, int);

extern BOOL
SM_triangle_dispose(SM_Surface_ptr, int);

extern BOOL
SM_triangle_index_valid(SM_Surface_ptr, int);

extern BOOL
SM_triangle_vertex_index_get(SM_Surface_ptr, int, int *, int *, int *);

extern BOOL
SM_triangle_edge_index_get(SM_Surface_ptr, int, int *, int *, int *);

extern BOOL
SM_triangle_picked_set(SM_Surface_ptr, int, BOOL);

extern BOOL
SM_triangle_picked_get(SM_Surface_ptr, int, BOOL *);

extern void
SM_triangle_picked_push(SM_Surface_ptr);

extern void
SM_triangle_picked_pop(SM_Surface_ptr);

extern BOOL
SM_triangle_picked_triangulate(SM_Surface_ptr);

extern int
SM_triangle_neighbor_get(SM_Surface_ptr, int, int);

extern BOOL
SM_triangle_normal_set(SM_Surface_ptr, int, VECTOR_3D);

extern BOOL
SM_triangle_normal_get(SM_Surface_ptr, int, VECTOR_3D);

extern BOOL
SM_triangle_normal_flip(SM_Surface_ptr, int);

extern BOOL
SM_triangle_normal_index_get(SM_Surface_ptr, int, int *, int *, int *);

extern BOOL
SM_triangle_center_get(SM_Surface_ptr, int, VECTOR_3D);

extern BOOL
SM_triangle_angular_bisector_get(SM_Surface_ptr, int, VECTOR_3D);

extern BOOL
SM_triangle_material_set(SM_Surface_ptr, int, int);

extern BOOL
SM_triangle_material_get(SM_Surface_ptr, int, int *);

extern void
SM_triangle_material_check(SM_Surface_ptr);




extern BOOL
SM_edge_dispose(SM_Surface_ptr, int);

extern BOOL
SM_edge_index_valid(SM_Surface_ptr, int);

extern BOOL
SM_edge_square_set(SM_Surface_ptr, int, BOOL);

extern BOOL
SM_edge_square_get(SM_Surface_ptr, int, BOOL *);

extern BOOL
SM_edge_hole_get(SM_Surface_ptr, int, BOOL *);

extern BOOL
SM_edge_picked_set(SM_Surface_ptr, int, BOOL);

extern BOOL
SM_edge_picked_get(SM_Surface_ptr, int, BOOL *);

extern void
SM_edge_picked_push(SM_Surface_ptr);

extern void
SM_edge_picked_pop(SM_Surface_ptr);

extern BOOL
SM_edge_triangles_get(SM_Surface_ptr, int, int *, int *);

extern BOOL
SM_edge_vertices_get(SM_Surface_ptr, int, int *, int *);

extern int
SM_edge_common_triangles_get(SM_Surface_ptr, int, int);




extern void
SM_matrix_delta(SM_Surface_ptr, MATRIX_H3D);

extern void
SM_matrix_get(SM_Surface_ptr, MATRIX_H3D);

extern void
SM_matrix_set(SM_Surface_ptr, MATRIX_H3D);





extern void
SM_star_draw(VECTOR_3D);

extern void
SM_dot_draw(VECTOR_3D);

extern void
SM_VECTOR_3D_draw(VECTOR_3D, VECTOR_3D, float);

extern SM_Surface_ptr
SM_surface_new(void);

extern void
SM_surface_dispose(SM_Surface_ptr);

extern int
SM_surface_memory_used(SM_Surface_ptr);

extern void
SM_surface_material_write(SM_Surface_ptr, char *);

extern SM_Surface_ptr
SM_surface_read(char *);
extern SM_Surface_ptr
SM_surface_readtxt(char *);

extern BOOL
SM_surface_writetxt(SM_Surface_ptr, char *);
extern BOOL
SM_surface_write(SM_Surface_ptr, char *);

extern void
SM_surface_fuse(SM_Surface_ptr, SM_Surface_ptr, int);

extern BOOL
SM_surface_function_2D_set(SM_Surface_ptr, float (*)(float), VECTOR_3D);

extern void
SM_surface_function_apply(SM_Surface_ptr, VECTOR_3D);

extern long
SM_surface_prim_color_get(unsigned long);

extern void
SM_surface_prim_color_set(unsigned long, long);

extern void
SM_surface_quick_draw(SM_Surface_ptr);

extern void
SM_surface_draw(SM_Surface_ptr, unsigned long);



extern char 
*SM_material_name_get(int);

extern BOOL
SM_material_name_valid(SM_Material_Name);

extern BOOL
SM_material_read(char *);

extern void
SM_material_write(char *);

extern BOOL
SM_material_add(SM_Material_Name);

extern BOOL
SM_material_remove(SM_Material_Name);

extern BOOL
SM_material_rgb_set(SM_Material_Name, int, float, float, float);

extern BOOL
SM_material_rgb_get(SM_Material_Name, int, float *, float *, float *);

extern BOOL
SM_material_value_set(SM_Material_Name, int, float);

extern BOOL
SM_material_value_get(SM_Material_Name, int, float *);





extern int 
SM_vertex_count_get(SM_Surface_ptr);

extern int 
SM_edge_count_get(SM_Surface_ptr);

extern int 
SM_triangle_count_get(SM_Surface_ptr);

extern BOOL 
SM_vertex_incident_triangles(SM_Surface_ptr, int, int **, int *);

extern BOOL 
SM_vertex_hole_get(SM_Surface_ptr, int, int *, int *);

extern void
SM_surface_pick_all(SM_Surface_ptr);

extern void
SM_surface_unpick_all(SM_Surface_ptr);

extern void
SM_triangle_picked_invert(SM_Surface_ptr);

extern BOOL
SM_VECTOR_3D_read(Parser_File*,VECTOR_3D,Parser_List_Level);

extern void
SM_VECTOR_3D_write(FILE*,VECTOR_3D);

extern int Get_From_Stack(SM_Surface_ptr,BOOL);

extern void Put_In_Stack(SM_Surface_ptr,int,BOOL);

extern void
SM_vertex_allocate(SM_Surface_ptr);

extern BOOL
SM_vertex_normal_sm_get(SM_Surface_ptr,int,int,VECTOR_3D);

extern BOOL
SM_vertex_normal_index_get(SM_Surface_ptr,int,int,int*,int*);

extern void
SM_triangle_ref_vert_normal(SM_Surface_ptr,int,int,int);


extern BOOL
SM_vertex_normal_tri_ref_remove(SM_Surface_ptr,int,int,int);

extern BOOL
SM_vertex_normal_tri_ref_add(SM_Surface_ptr,int,int,int);

extern int
SM_vertex_normal_new( SM_Surface_ptr,int );

extern BOOL
SM_edge_two_sides(SM_Surface_ptr,int,int,int*,int*);

extern int
SM_vertex_normal_count (SM_Surface_ptr,int );

extern int
SM_vertex_normal_tri_ref_merge(SM_Surface_ptr,int,int,int);

extern BOOL
SM_vertex_normal_contiguous(SM_Surface_ptr,int );

extern BOOL
SM_vertex_normal_split(SM_Surface_ptr,int,int,int);

extern int
SM_edge_new(SM_Surface_ptr,int,int,int);

extern BOOL
SM_edge_ref_triangle_remove(SM_Surface_ptr,int,int);

extern int
SM_vertex_normal_ref_add(SM_Surface_ptr,int,int,int,int);

extern BOOL
SM_triangle_normal_coherent(SM_Surface_ptr,int);

extern BOOL
SM_triangle_ref_vertex(SM_Surface_ptr,int,int,int,int);


extern BOOL
SM_vertex_ref_triangle_remove(SM_Surface_ptr,int,int);

extern void SM_region_triangle_rem(SM_Surface_ptr,int,int);

extern BOOL
SM_edge_normal_contiguous(SM_Surface_ptr,int);

extern int
SM_edge_neighbor_get(SM_Surface_ptr,int,int);

extern void
SM_triangle_normal_direct(SM_Surface_ptr,int,BOOL,int*,int*);

extern int
SM_material_index_get(SM_Material_Name);

extern void
SM_material_read_fil(Parser_File *);

extern void
SM_material_write_fil(FILE *);

extern int
SM_material_index_get(SM_Material_Name);


extern void
SM_triangle_allocate(SM_Surface_ptr);

extern void
SM_edge_allocate(SM_Surface_ptr);

extern void Stack_Init(SM_Surface_ptr,BOOL);

extern void Stack_Free(SM_Surface_ptr);

extern void SM_triangle_region_add(SM_Surface_ptr,int,int);

extern void SM_triangle_region_rem(SM_Surface_ptr,int,int);

extern void
SM_surface_material_write_fil(SM_Surface_ptr,FILE *);

extern SM_Surface_ptr
SM_surface_new();

extern void
SM_vertex_write(FILE*,SM_Surface_ptr,int);

extern void
SM_triangle_write(FILE*,SM_Surface_ptr,int);

#ifdef CPLUSPLUS
}
#endif

#endif
