
/*------------------------------------------------------------------------------*/
/*										*/
/*										*/
/*  This software module was originally developed by                            */
/*        Prem Kalra, MIRALab, University of Geneva                             */
/*  and edited by                                                               */
/*        Igor Pandzic, MIRALab, University of Geneva                           */
/*        Marc Escher, MIRALab, University of Geneva                            */
/*        Pierre Beylot, MIRALab, University of Geneva                          */
/*  in the course of development of the MPEG-4 Video (ISO/IEC 14496-2).	        */
/*  This software module is an implementation of a part of one or more	        */
/*  MPEG-4 Video (ISO/IEC 14496-2) tools as specified by the MPEG-4 Video       */
/*  (ISO/IEC 14496-2). ISO/IEC gives users of the                               */
/*  MPEG-4 Video (ISO/IEC 14496-2) free license to this software module or	*/
/*  modifications thereof for use in hardware or software products claiming 	*/
/*  conformance to the MPEG-4 Video (ISO/IEC 14496-2). Those intending to use	*/
/*  this software module in hardware or software products are advised that	*/
/*  its use may infringe existing patents. The original developer of this	*/
/*  software module and his/her company, the subsequent editors and their	*/
/*  companies, and ISO/IEC have no liability for use of this software		*/
/*  module or modifications thereof. Copyright is not released for non		*/
/*  MPEG-4 Video (ISO/IEC 14496-2) conforming products.                         */
/*  MIRALab, University of Geneva retains full right to use the code for its    */
/*  own purpose, assign or donate the code to a third party and to inhibit      */
/*  third parties from using the code for non MPEG-4 Video (ISO/IEC 14496-2)    */
/*  conforming products. This copyright notice must be included in all copies   */
/*  or derivative works. Copyright (c) (1997).			      		*/
/*										*/
/*										*/
/*------------------------------------------------------------------------------*/

#include <assert.h>
#include <stdio.h>
#include <string.h>

#include "fap_dec.h"
#include "SurfMan.h"



static Fap_def fap_def[NB_FAP];
static Mpa_def mpa_def[NB_MPA];
static FILE *fap_file;

extern void
run_mpas(int *, float *, SM_Surface_ptr,SM_Surface_ptr,SM_Region_List_ptr);

int
init_fap_converter(char *conv_filename)
{
  init_faps(fap_def);
  init_mpas(mpa_def);
  return read_mpa_ranges(conv_filename);
}


int 
read_mpa_ranges(char *infilename)
{
  FILE *infile;
  char line[2000], str[100], mpa_str[200];
  int i, mpa_index, coord_index;
  FCoord *coord;
  float units[7];


  if (!(infile = fopen(infilename, "r"))) {
    printf("can't read file %s\n", infilename);
    return 0;
  }


  fscanf(infile, "%f%f%f%f%f", 
	 &(units[UNIT_ES]), &(units[UNIT_ENS]), 
	 &(units[UNIT_MNS]), &(units[UNIT_MW]),
	 &(units[UNIT_IRISD]));


  units[UNIT_ES] /= 1024.0;
  units[UNIT_ENS] /= 1024.0;
  units[UNIT_MNS] /= 1024.0;
  units[UNIT_MW] /= 1024.0;
  units[UNIT_IRISD] /= 1024.0;
  units[UNIT_AU] = 10E-5;


  for (i = 0; i < NB_MPA; i++) 
    mpa_def[i].defined = 0;    


  do {
    fscanf(infile, "%s", mpa_str);
 
    mpa_index = get_mpa_index(mpa_str);

    if (mpa_index == -1) {
      printf("%s doesn't exist\n", mpa_str);
      return 0;
    }

    mpa_def[mpa_index].defined = 1;

    fscanf(infile, "%*s%s", str);

    while (str[0] != ']') {
      sscanf(str, "%d", &coord_index);
      coord = &(mpa_def[mpa_index].coords[0]);

      while (coord->which != coord_index && coord->which != END_COORD_INDEX) {
	coord++;
      }

      if (coord->which == END_COORD_INDEX) {
	printf("can't read mpa: %s\n", mpa_str);
	return 0;
      }
 
      fscanf(infile, "%f%f%f", &(coord->min), &(coord->middle), &(coord->max));
      coord->min /= units[coord->unit];
      coord->middle /= units[coord->unit];
      coord->max /= units[coord->unit];

      fscanf(infile, "%s", str);
    }
  } while (!feof(infile) && line != NULL);

  fclose(infile);
  return 1;
}


#define SKIP_COMMENTS(infile) 					\
while (fgets(line, 2000, infile) && line[0] == '#')


int
open_fap_file(char *filename)
{
  char line[2000];
  int nb_frame;

  if (!(fap_file = fopen(filename, "r"))) {
    printf("can't read file %s\n", filename);
    return -1;
  }

  printf("reading FAP frames from %s...", filename);

  SKIP_COMMENTS(fap_file);
  sscanf(line, "%*f%*s%*d%d", &nb_frame);
  return nb_frame;
}





int
get_mpa_index(const char *mpa_str)
{
  int i;

  for (i = 0; i < NB_MPA; i++)
    if (!strcasecmp(mpa_str, tab_mpa_str[i])) return i;

  return -1;
}

float
faps_to_face_coordinate(int *index, int nb_fap, int *mask, float *faps)
{
  int i, count;
  float f = 0.0;

  for (i = count = 0; i < nb_fap; i++) {
    if (mask[index[i]] != 0) {
      f += faps[index[i]];
      count ++;
    }
  }

  return count == 0 ? 0.0 : f / (float)count;
}



void
face_coordinate_to_mpa(float value, float *mpas, int coord_index, int mpa_index)
{
  int interval;
  float min, middle, max, xo, yo, a, y;
  FCoord *coord;



  if (!mpa_def[mpa_index].defined) {
    return;
  }


  interval = mpa_def[mpa_index].interval;
  assert(interval != INTERV_NOT_DEFINED);


  coord = &(mpa_def[mpa_index].coords[0]);
  while (coord->which != coord_index && coord->which != END_COORD_INDEX)
    coord++;

  if (coord->which == END_COORD_INDEX) {
    printf("%s\n", tab_mpa_str[mpa_index]);
  }

  assert(coord->which != END_COORD_INDEX);
  min = coord->min;
  middle = coord->middle;
  max = coord->max;


  xo = middle;
  yo = interval == INTERV_UNI ? 0.5 : 0.0;

  if (max > min)
    a = 1.0 / (value < xo ? middle - min : max - middle);
  else
    a = 1.0 / (value < xo ? max - middle : middle - min);

  if (interval == INTERV_UNI) a /= 2.;


  y = yo + a * (value - xo);


  if (y > 1) y = 1.0;
  else if (interval == INTERV_UNI) {if (y < 0.0) y = 0.0;}
  else if (y < -1.0) y = -1.0;

  mpas[mpa_index] = y;
}


float
mpa_to_face_coordinate(float *mpas, int mpa_index, int coord_index)
{
  int interval;
  float min, middle, max, xo, yo, a;
  float value = mpas[mpa_index];
  FCoord *coord;


  if (!mpa_def[mpa_index].defined) return 0.0;


  interval = mpa_def[mpa_index].interval;
  assert(interval != INTERV_NOT_DEFINED);


  coord = &(mpa_def[mpa_index].coords[0]);
  while (coord->which != coord_index && coord->which != END_COORD_INDEX)
    coord++;

  assert(coord->which != END_COORD_INDEX);
  min = coord->min;
  middle = coord->middle;
  max = coord->max;

  yo = middle;
  xo = interval == INTERV_UNI ? 0.5 : 0.0;

  a = value < xo ? middle - min : max - middle;
  if (interval == INTERV_UNI) a *= 2.;

  return yo + a * (value - xo);
}

void
read_fap_frame(int *mask, float *faps)
{
  int i;


  for (i = 0; i < NB_FAP; i++) {
    fscanf(fap_file, "%d", &(mask[i]));
  }


  fscanf(fap_file, "%*d");

  for (i = 0; i < NB_FAP; i++) {
    if (mask[i]) {
      fscanf(fap_file, "%f", &(faps[i]));
    }
    else 
      faps[i] = 0.0;
  }
}

leftrightinterpolation(int *fapmask, int *faps, int startfapnum, int endfapnum) {
  int fapnum;

  for(fapnum=startfapnum;fapnum <= endfapnum;fapnum+=2)
    if(fapmask[fapnum] && (!fapmask[fapnum+1])) {
        faps[fapnum+1] = faps[fapnum];
        fapmask[fapnum+1] = 1;
    }
    else {
      if(fapmask[fapnum+1] && (!fapmask[fapnum])) {
          faps[fapnum] = faps[fapnum+1];
          fapmask[fapnum] = 1;
      }
    }
}

void
run_faps(int *mask, int *ifaps,SM_Surface_ptr the_surface,SM_Surface_ptr the_face,SM_Region_List_ptr the_region_list)
{
  int i, index_fap[10];
  float f, mpas[NB_MPA];
  float faps[NB_FAP];
  int mpas_mask[NB_MPA];

#if 0
  leftrightinterpolation(mask,ifaps, 6, 12);/*inner lips*/
  leftrightinterpolation(mask,ifaps, 53, 61);/*outer lips,nose*/
  leftrightinterpolation(mask,ifaps, 19, 41);/*eyelids,eyebrows,eyeballs,pupils,cheeks*/
  leftrightinterpolation(mask,ifaps, 65, 67);/*ears*/
#endif
  leftrightinterpolation(mask,ifaps, 5, 11);/*inner lips*/
  leftrightinterpolation(mask,ifaps, 52, 60);/*outer lips,nose*/
  leftrightinterpolation(mask,ifaps, 18, 40);/*eyelids,eyebrows,eyeballs,pupils,cheeks*/
  leftrightinterpolation(mask,ifaps, 64, 66);/*ears*/

/*
  if (mask[FAP_PUFF_L_CHEEK] || mask[FAP_STRETCH_L_NOSE] || 
      mask[FAP_STRETCH_R_NOSE] || mask[FAP_BEND_NOSE]) {
    getchar();
  }
*/

  for(i=0;i<NB_FAP;i++) {

/*
    if(mask[i])
      fprintf(stderr,"FAP%d = %d\n",i + 1,ifaps[i]);

*/
    if (fap_def[i].direction == FAP_DIR_DOWN || 
	fap_def[i].direction == FAP_DIR_RIGHT)
      faps[i] = -ifaps[i];
    else
      faps[i] = ifaps[i];
  }  

  for (i = 0; i < NB_MPA; i++) {
    mpas[i] = 0.0;
    mpas_mask[i] = mpa_def[i].defined;
  }



  f = mask[FAP_THRUST_JAW] ? faps[FAP_THRUST_JAW] : 0.0;
  face_coordinate_to_mpa(f, mpas, COORD_JAW_F, JAW_THRUST);
  
  index_fap[0] = FAP_PUSH_B_LIP;
  index_fap[1] = FAP_PUSH_T_LIP;

  f = faps_to_face_coordinate(index_fap, 2, mask, faps);  
  f -= mpa_to_face_coordinate(mpas, JAW_THRUST, COORD_MOUTH_F);
  face_coordinate_to_mpa(f, mpas, COORD_MOUTH_F, PULL_MIDLIPS);  

  f = mask[FAP_BEND_NOSE] ? faps[FAP_BEND_NOSE] : 0.0;
  face_coordinate_to_mpa(f, mpas, COORD_NOSE_TIP_H, NOSE_BEND);

  f = mask[FAP_RAISE_NOSE] ? faps[FAP_RAISE_NOSE] : 0.0;
  face_coordinate_to_mpa(f, mpas, COORD_NOSE_V, RAISE_NOSE);

  f = mask[FAP_STRETCH_L_NOSE] ? faps[FAP_STRETCH_L_NOSE] : 0.0;
  face_coordinate_to_mpa(f, mpas, COORD_NOSE_H_RIGHT, STRETCH_R_NOSE);

  f = mask[FAP_STRETCH_R_NOSE] ? faps[FAP_STRETCH_R_NOSE] : 0.0;
  face_coordinate_to_mpa(f, mpas, COORD_NOSE_H_LEFT, STRETCH_L_NOSE);

  f = mask[FAP_SHIFT_JAW] ? faps[FAP_SHIFT_JAW] : 0.0;
  face_coordinate_to_mpa(f, mpas, COORD_JAW_H, MOVE_HORI_JAW);

  f = mask[FAP_DEPRESS_CHIN] ? faps[FAP_DEPRESS_CHIN] : 0.0;  
  f -= mpa_to_face_coordinate(mpas, PULL_MIDLIPS, COORD_CHIN_F);
  face_coordinate_to_mpa(f, mpas, COORD_CHIN_F, DEPRESS_CHIN);

  f = mask[FAP_OPEN_JAW] ? faps[FAP_OPEN_JAW] : 0.0;
  f -= mpa_to_face_coordinate(mpas, DEPRESS_CHIN, COORD_JAW_V);
  face_coordinate_to_mpa(f, mpas, COORD_JAW_V, OPEN_JAW);

  index_fap[0] = FAP_RAISE_L_CORNERLIP_O;
  index_fap[1] = FAP_RAISE_L_CORNERLIP;
  f = faps_to_face_coordinate(index_fap, 2, mask, faps);

  if (f > 0.0) {
    face_coordinate_to_mpa(f, mpas, COORD_MOUTH_V_RIGHT, RAISE_R_CORNERLIP);
  }
  else {
    face_coordinate_to_mpa(f, mpas, COORD_MOUTH_V_RIGHT, LOWER_R_CORNERLIP);
  }


#if 0
  face_coordinate_to_mpa(f, mpas, COORD_MOUTH_V_RIGHT, LOWER_R_CORNERLIP);

  if (mpas[LOWER_R_CORNERLIP] == 1.0 || mpas[LOWER_R_CORNERLIP] == -1.0) {

    if (mpas[LOWER_R_CORNERLIP] == 1.0) {assert(f < 0.0);}
    if (mpas[LOWER_R_CORNERLIP] == -1.0) {assert(f > 0.0);}    
    f -= mpa_to_face_coordinate(mpas, LOWER_R_CORNERLIP, COORD_MOUTH_V_RIGHT);
    if (mpas[LOWER_R_CORNERLIP] == 1.0) {assert(f < 0.0);}
    if (mpas[LOWER_R_CORNERLIP] == -1.0) {assert(f > 0.0);}

    face_coordinate_to_mpa(f, mpas, COORD_MOUTH_V_RIGHT, RAISE_R_CORNERLIP);
  }
#endif


  index_fap[0] = FAP_RAISE_R_CORNERLIP_O;
  index_fap[1] = FAP_RAISE_R_CORNERLIP;
  f = faps_to_face_coordinate(index_fap, 2, mask, faps);

  if (f > 0.0) {
    face_coordinate_to_mpa(f, mpas, COORD_MOUTH_V_LEFT, RAISE_L_CORNERLIP);
  }
  else {
    face_coordinate_to_mpa(f, mpas, COORD_MOUTH_V_LEFT, LOWER_L_CORNERLIP);
  }

#if 0
  face_coordinate_to_mpa(f, mpas, COORD_MOUTH_V_LEFT, LOWER_L_CORNERLIP);


  if (mpas[LOWER_L_CORNERLIP] == 1.0 || 
      mpas[LOWER_L_CORNERLIP] == -1.0) {


    if (mpas[LOWER_L_CORNERLIP] == 1.0) {assert(f < 0.0);}
    if (mpas[LOWER_L_CORNERLIP] == -1.0) {assert(f > 0.0);}
    f -= mpa_to_face_coordinate(mpas, LOWER_L_CORNERLIP, COORD_MOUTH_V_LEFT);
    if (mpas[LOWER_L_CORNERLIP] == 1.0) {assert(f < 0.0);}
    if (mpas[LOWER_L_CORNERLIP] == -1.0) {assert(f > 0.0);}

    face_coordinate_to_mpa(f, mpas, COORD_MOUTH_V_LEFT, RAISE_L_CORNERLIP);
  }
#endif


  index_fap[0] = FAP_STRETCH_L_CORNERLIP_O;
  index_fap[1] = FAP_STRETCH_L_CORNERLIP;
  f = faps_to_face_coordinate(index_fap, 2, mask, faps);
  f -= mpa_to_face_coordinate(mpas, RAISE_R_CORNERLIP, COORD_MOUTH_H_RIGHT);
  f -= mpa_to_face_coordinate(mpas, PULL_MIDLIPS, COORD_MOUTH_H_RIGHT);
  face_coordinate_to_mpa(f, mpas, COORD_MOUTH_H_RIGHT, STRETCH_R_CORNERLIP);

  index_fap[0] = FAP_STRETCH_R_CORNERLIP_O;
  index_fap[1] = FAP_STRETCH_R_CORNERLIP;
  f = faps_to_face_coordinate(index_fap, 2, mask, faps);
  f -= mpa_to_face_coordinate(mpas, RAISE_L_CORNERLIP, COORD_MOUTH_H_LEFT);
  f -= mpa_to_face_coordinate(mpas, PULL_MIDLIPS, COORD_MOUTH_H_LEFT);
  face_coordinate_to_mpa(f, mpas, COORD_MOUTH_H_LEFT, STRETCH_L_CORNERLIP);

  index_fap[0] = FAP_LOWER_T_MIDLIP_O;
  index_fap[1] = FAP_LOWER_T_MIDLIP;
  index_fap[2] = FAP_LOWER_T_LIP_LM_O;
  index_fap[3] = FAP_LOWER_T_LIP_LM;
  index_fap[4] = FAP_LOWER_T_LIP_RM_O;
  index_fap[5] = FAP_LOWER_T_LIP_RM;
  f = faps_to_face_coordinate(index_fap, 6, mask, faps);
  f -= mpa_to_face_coordinate(mpas, RAISE_L_CORNERLIP, COORD_MOUTH_V_TOP);
  f -= mpa_to_face_coordinate(mpas, RAISE_R_CORNERLIP, COORD_MOUTH_V_TOP);
  f -= mpa_to_face_coordinate(mpas, LOWER_L_CORNERLIP, COORD_MOUTH_V_TOP);
  f -= mpa_to_face_coordinate(mpas, LOWER_R_CORNERLIP, COORD_MOUTH_V_TOP);
  f -= mpa_to_face_coordinate(mpas, PULL_MIDLIPS, COORD_MOUTH_V_TOP);
  f -= mpa_to_face_coordinate(mpas, RAISE_NOSE, COORD_MOUTH_V_TOP);
  face_coordinate_to_mpa(f, mpas, COORD_MOUTH_V_TOP, RAISE_UPPERLIP);


  if (mpas[RAISE_UPPERLIP] == 1.0) {
    f -= mpa_to_face_coordinate(mpas, RAISE_UPPERLIP, COORD_MOUTH_V_TOP);
    face_coordinate_to_mpa(f, mpas, COORD_MOUTH_V_TOP, RAISE_U_MIDLIP);
  }

  index_fap[0] = FAP_RAISE_B_MIDLIP_O;
  index_fap[1] = FAP_RAISE_B_MIDLIP;
  index_fap[2] = FAP_RAISE_B_LIP_LM_O;
  index_fap[3] = FAP_RAISE_B_LIP_LM;
  index_fap[4] = FAP_RAISE_B_LIP_RM_O;
  index_fap[5] = FAP_RAISE_B_LIP_RM;
  f = faps_to_face_coordinate(index_fap, 6, mask, faps);
  f -= mpa_to_face_coordinate(mpas, RAISE_L_CORNERLIP, COORD_MOUTH_V_BOTTOM);
  f -= mpa_to_face_coordinate(mpas, RAISE_R_CORNERLIP, COORD_MOUTH_V_BOTTOM);
  f -= mpa_to_face_coordinate(mpas, LOWER_L_CORNERLIP, COORD_MOUTH_V_BOTTOM);
  f -= mpa_to_face_coordinate(mpas, LOWER_R_CORNERLIP, COORD_MOUTH_V_BOTTOM);
  f -= mpa_to_face_coordinate(mpas, PULL_MIDLIPS, COORD_MOUTH_V_BOTTOM);
  f -= mpa_to_face_coordinate(mpas, OPEN_JAW, COORD_MOUTH_V_BOTTOM);
  face_coordinate_to_mpa(f, mpas, COORD_MOUTH_V_BOTTOM, LOWER_LOWERLIP);

  if (mpas[LOWER_LOWERLIP] == 1.0) {
    f -= mpa_to_face_coordinate(mpas, LOWER_LOWERLIP, COORD_MOUTH_V_BOTTOM);
    face_coordinate_to_mpa(f, mpas, COORD_MOUTH_V_BOTTOM, RAISE_L_MIDLIP);
  }

  if (mask[FAP_LIFT_L_CHEEK]) {
    f = mask[FAP_LIFT_L_CHEEK] ? faps[FAP_LIFT_L_CHEEK] : 0.0;
    f -= mpa_to_face_coordinate(mpas, LOWER_R_CORNERLIP, COORD_CHEEK_V_RIGHT);
    f -= mpa_to_face_coordinate(mpas, RAISE_R_CORNERLIP, COORD_CHEEK_V_RIGHT);
    face_coordinate_to_mpa(f, mpas, COORD_CHEEK_V_RIGHT, LIFT_R_CHEEK);
  }
  else {
    mpas[LIFT_R_CHEEK] = 0.0;
  }

  if (mask[FAP_LIFT_R_CHEEK]) {
    f = mask[FAP_LIFT_R_CHEEK] ? faps[FAP_LIFT_R_CHEEK] : 0.0;
    f -= mpa_to_face_coordinate(mpas, LOWER_L_CORNERLIP, COORD_CHEEK_V_LEFT);
    f -= mpa_to_face_coordinate(mpas, RAISE_L_CORNERLIP, COORD_CHEEK_V_LEFT);
    face_coordinate_to_mpa(f, mpas, COORD_CHEEK_V_LEFT, LIFT_L_CHEEK);
  }
  else 
    mpas[LIFT_L_CHEEK] = 0.0;

  if (mask[FAP_PUFF_L_CHEEK]) {
    f = mask[FAP_PUFF_L_CHEEK] ? faps[FAP_PUFF_L_CHEEK] : 0.0;
    f -= mpa_to_face_coordinate(mpas, LIFT_R_CHEEK, COORD_CHEEK_H_RIGHT);
    f -= mpa_to_face_coordinate(mpas, RAISE_R_CORNERLIP, COORD_CHEEK_H_RIGHT);
    face_coordinate_to_mpa(f, mpas, COORD_CHEEK_H_RIGHT, PUFF_R_CHEEK);
  }
  else 
    mpas[PUFF_R_CHEEK] = 0.0;

  if (mask[FAP_PUFF_R_CHEEK]) {
    f = mask[FAP_PUFF_R_CHEEK] ? faps[FAP_PUFF_R_CHEEK] : 0.0;
    f -= mpa_to_face_coordinate(mpas, LIFT_L_CHEEK, COORD_CHEEK_H_LEFT);
    f -= mpa_to_face_coordinate(mpas, RAISE_L_CORNERLIP, COORD_CHEEK_H_LEFT);
    face_coordinate_to_mpa(f, mpas, COORD_CHEEK_H_LEFT, PUFF_L_CHEEK);
  }
  else 
    mpas[PUFF_L_CHEEK] = 0.0;

  f = mask[FAP_CLOSE_T_L_EYELID] ? faps[FAP_CLOSE_T_L_EYELID] : 0.0;
  face_coordinate_to_mpa(f, mpas, COORD_EYELID_V_RIGHT_TOP, CLOSE_UPPER_R_EYELID);

  f = mask[FAP_CLOSE_T_R_EYELID] ? faps[FAP_CLOSE_T_R_EYELID] : 0.0;
  face_coordinate_to_mpa(f, mpas, COORD_EYELID_V_LEFT_TOP, CLOSE_UPPER_L_EYELID);

  f = mask[FAP_CLOSE_B_L_EYELID] ? faps[FAP_CLOSE_B_L_EYELID] : 0.0;
  face_coordinate_to_mpa(f, mpas, COORD_EYELID_V_RIGHT_BOTTOM, CLOSE_LOWER_R_EYELID);

  f = mask[FAP_CLOSE_B_R_EYELID] ? faps[FAP_CLOSE_B_R_EYELID] : 0.0;
  face_coordinate_to_mpa(f, mpas, COORD_EYELID_V_LEFT_BOTTOM, CLOSE_LOWER_L_EYELID);

  f = mask[FAP_PITCH_L_EYEBALL] ? faps[FAP_PITCH_L_EYEBALL] : 0.0;
  face_coordinate_to_mpa(f, mpas, COORD_EYE_V_RIGHT, MOVE_V_R_EYEBALL);

  f = mask[FAP_YAW_L_EYEBALL] ? faps[FAP_YAW_L_EYEBALL] : 0.0;
  face_coordinate_to_mpa(f, mpas, COORD_EYE_H_RIGHT, MOVE_H_R_EYEBALL);

  f = mask[FAP_THRUST_L_EYEBALL] ? faps[FAP_THRUST_L_EYEBALL] : 0.0; 
  face_coordinate_to_mpa(f, mpas, COORD_EYE_F_RIGHT, MOVE_O_R_EYEBALL);

  f = mask[FAP_PITCH_R_EYEBALL] ? faps[FAP_PITCH_R_EYEBALL] : 0.0;
  face_coordinate_to_mpa(f, mpas, COORD_EYE_V_LEFT, MOVE_V_L_EYEBALL);

  f = mask[FAP_YAW_R_EYEBALL] ? faps[FAP_YAW_R_EYEBALL] : 0.0;
  face_coordinate_to_mpa(f, mpas, COORD_EYE_H_LEFT, MOVE_H_L_EYEBALL);

  f = mask[FAP_THRUST_R_EYEBALL] ? faps[FAP_THRUST_R_EYEBALL] : 0.0; 
  face_coordinate_to_mpa(f, mpas, COORD_EYE_F_LEFT, MOVE_O_L_EYEBALL);

  f = mask[FAP_DILATE_L_PUPIL] ? faps[FAP_DILATE_L_PUPIL] : 0.0;
  face_coordinate_to_mpa(f, mpas, COORD_PUPIL_RIGHT, EYE_DILATE_R_PUPIL);

  f = mask[FAP_DILATE_R_PUPIL] ? faps[FAP_DILATE_R_PUPIL] : 0.0;
  face_coordinate_to_mpa(f, mpas, COORD_PUPIL_LEFT, EYE_DILATE_L_PUPIL);

  f = mask[FAP_SQUEEZE_L_EYEBROW] ? faps[FAP_SQUEEZE_L_EYEBROW] : 0.0;
  face_coordinate_to_mpa(f, mpas, COORD_EYEBROW_H_RIGHT, SQUEEZE_R_EYEBROW);

  f = mask[FAP_SQUEEZE_R_EYEBROW] ? faps[FAP_SQUEEZE_R_EYEBROW] : 0.0;
  face_coordinate_to_mpa(f, mpas, COORD_EYEBROW_H_LEFT, SQUEEZE_L_EYEBROW);

  f = mask[FAP_RAISE_L_M_EYEBROW] ? faps[FAP_RAISE_L_M_EYEBROW] : 0.0;
  face_coordinate_to_mpa(f, mpas, COORD_EYEBROW_V_RIGHT_MIDDLE, RAISE_R_EYEBROW_M);

  f = mask[FAP_RAISE_R_M_EYEBROW] ? faps[FAP_RAISE_R_M_EYEBROW] : 0.0;
  face_coordinate_to_mpa(f, mpas, COORD_EYEBROW_V_LEFT_MIDDLE, RAISE_L_EYEBROW_M);

  f = mask[FAP_RAISE_L_O_EYEBROW] ? faps[FAP_RAISE_L_O_EYEBROW] : 0.0;
  f -= mpa_to_face_coordinate(mpas, RAISE_R_EYEBROW_M, COORD_EYEBROW_V_RIGHT_RIGHT);
  face_coordinate_to_mpa(f, mpas, COORD_EYEBROW_V_RIGHT_RIGHT, RAISE_R_EYEBROW_R);

  f = mask[FAP_RAISE_L_I_EYEBROW] ? faps[FAP_RAISE_L_I_EYEBROW] : 0.0;
  f -= mpa_to_face_coordinate(mpas, RAISE_R_EYEBROW_M, COORD_EYEBROW_V_RIGHT_LEFT);
  face_coordinate_to_mpa(f, mpas, COORD_EYEBROW_V_RIGHT_LEFT, RAISE_R_EYEBROW_L);

  f = mask[FAP_RAISE_R_O_EYEBROW] ? faps[FAP_RAISE_R_O_EYEBROW] : 0.0;
  f -= mpa_to_face_coordinate(mpas, RAISE_L_EYEBROW_M, COORD_EYEBROW_V_LEFT_LEFT);
  face_coordinate_to_mpa(f, mpas, COORD_EYEBROW_V_LEFT_LEFT, RAISE_L_EYEBROW_L);


  f = mask[FAP_RAISE_R_I_EYEBROW] ? faps[FAP_RAISE_R_I_EYEBROW] : 0.0;
  f -= mpa_to_face_coordinate(mpas, RAISE_L_EYEBROW_M, COORD_EYEBROW_V_LEFT_RIGHT);
  face_coordinate_to_mpa(f, mpas, COORD_EYEBROW_V_LEFT_RIGHT, RAISE_L_EYEBROW_R);

  f = mask[FAP_RAISE_L_EAR] ? faps[FAP_RAISE_L_EAR] : 0.0;
  face_coordinate_to_mpa(f, mpas, COORD_EAR_V_RIGHT, RAISE_R_EAR);

  f = mask[FAP_RAISE_R_EAR] ? faps[FAP_RAISE_R_EAR] : 0.0;
  face_coordinate_to_mpa(f, mpas, COORD_EAR_V_LEFT, RAISE_L_EAR);

  f = mask[FAP_PULL_L_EAR] ? faps[FAP_PULL_L_EAR] : 0.0;
  face_coordinate_to_mpa(f, mpas, COORD_EAR_H_RIGHT, PULL_R_EAR);

  f = mask[FAP_PULL_R_EAR] ? faps[FAP_PULL_R_EAR] : 0.0;
  face_coordinate_to_mpa(f, mpas, COORD_EAR_H_LEFT, PULL_L_EAR);

  f = mask[FAP_RAISE_TONGUE] ? faps[FAP_RAISE_TONGUE] : 0.0;
  face_coordinate_to_mpa(f, mpas, COORD_TONGUE_V, TONGUE_LIFT);

  f = mask[FAP_SHIFT_TONGUE_TIP] ? faps[FAP_SHIFT_TONGUE_TIP] : 0.0;
  face_coordinate_to_mpa(f, mpas, COORD_TONGUE_TIP_H, TONGUE_PAN);

  f = mask[FAP_THRUST_TONGUE_TIP] ? faps[FAP_THRUST_TONGUE_TIP] : 0.0;
  face_coordinate_to_mpa(f, mpas, COORD_TONGUE_TIP_F, TONGUE_OUT);

  f = mask[FAP_RAISE_TONGUE_TIP] ? faps[FAP_RAISE_TONGUE_TIP] : 0.0;
  face_coordinate_to_mpa(f, mpas, COORD_TONGUE_TIP_V, TONGUE_LIFT_TIP);

  f = mask[FAP_TONGUE_ROLL] ? faps[FAP_TONGUE_ROLL] : 0.0;
  face_coordinate_to_mpa(f, mpas, COORD_TONGUE_ROLL, TONGUE_ROLL);

  f = mask[FAP_HEAD_PITCH] ? faps[FAP_HEAD_PITCH] : 0.0;
  face_coordinate_to_mpa(f, mpas, COORD_HEAD_NOD, NOD_HEAD);

  f = mask[FAP_HEAD_ROLL] ? faps[FAP_HEAD_ROLL] : 0.0;
  face_coordinate_to_mpa(f, mpas, COORD_HEAD_ROLL, ROLL_HEAD);

  f = mask[FAP_HEAD_YAW] ? faps[FAP_HEAD_YAW] : 0.0;
  face_coordinate_to_mpa(f, mpas, COORD_HEAD_TURN, TURN_HEAD);

  run_mpas(mpas_mask, mpas,the_surface,the_face,the_region_list);
}


void init_mpas(Mpa_def tab[])
{
  tab[MOVE_H_L_EYEBALL].interval = INTERV_BI;
  tab[MOVE_H_L_EYEBALL].coords[0].which = COORD_EYE_H_LEFT;
  tab[MOVE_H_L_EYEBALL].coords[0].unit = UNIT_AU;
  tab[MOVE_H_L_EYEBALL].coords[1].which = END_COORD_INDEX;

  tab[MOVE_H_R_EYEBALL].interval = INTERV_BI;
  tab[MOVE_H_R_EYEBALL].coords[0].which = COORD_EYE_H_RIGHT;
  tab[MOVE_H_R_EYEBALL].coords[0].unit = UNIT_AU;
  tab[MOVE_H_R_EYEBALL].coords[1].which = END_COORD_INDEX;

  tab[MOVE_V_L_EYEBALL].interval = INTERV_BI;
  tab[MOVE_V_L_EYEBALL].coords[0].which = COORD_EYE_V_LEFT;
  tab[MOVE_V_L_EYEBALL].coords[0].unit = UNIT_AU;
  tab[MOVE_V_L_EYEBALL].coords[1].which = END_COORD_INDEX;

  tab[MOVE_V_R_EYEBALL].interval = INTERV_BI;
  tab[MOVE_V_R_EYEBALL].coords[0].which = COORD_EYE_V_RIGHT;
  tab[MOVE_V_R_EYEBALL].coords[0].unit = UNIT_AU;
  tab[MOVE_V_R_EYEBALL].coords[1].which = END_COORD_INDEX;

  tab[CLOSE_UPPER_L_EYELID].interval = INTERV_BI;
  tab[CLOSE_UPPER_L_EYELID].coords[0].which = COORD_EYELID_V_LEFT_TOP;
  tab[CLOSE_UPPER_L_EYELID].coords[0].unit = UNIT_IRISD;
  tab[CLOSE_UPPER_L_EYELID].coords[1].which = END_COORD_INDEX;

  tab[CLOSE_UPPER_R_EYELID].interval = INTERV_BI;
  tab[CLOSE_UPPER_R_EYELID].coords[0].which = COORD_EYELID_V_RIGHT_TOP;
  tab[CLOSE_UPPER_R_EYELID].coords[0].unit = UNIT_IRISD;
  tab[CLOSE_UPPER_R_EYELID].coords[1].which = END_COORD_INDEX;

  tab[CLOSE_LOWER_L_EYELID].interval = INTERV_BI;
  tab[CLOSE_LOWER_L_EYELID].coords[0].which = COORD_EYELID_V_LEFT_BOTTOM;
  tab[CLOSE_LOWER_L_EYELID].coords[0].unit = UNIT_IRISD;
  tab[CLOSE_LOWER_L_EYELID].coords[1].which = END_COORD_INDEX;

  tab[CLOSE_LOWER_R_EYELID].interval = INTERV_BI;
  tab[CLOSE_LOWER_R_EYELID].coords[0].which = COORD_EYELID_V_RIGHT_BOTTOM;
  tab[CLOSE_LOWER_R_EYELID].coords[0].unit = UNIT_IRISD;
  tab[CLOSE_LOWER_R_EYELID].coords[1].which = END_COORD_INDEX;

  tab[RAISE_L_EYEBROW_L].interval = INTERV_BI;
  tab[RAISE_L_EYEBROW_L].coords[0].which = COORD_EYEBROW_V_LEFT_LEFT;
  tab[RAISE_L_EYEBROW_L].coords[0].unit = UNIT_ENS;
  tab[RAISE_L_EYEBROW_L].coords[1].which = END_COORD_INDEX;

  tab[RAISE_R_EYEBROW_L].interval = INTERV_BI;
  tab[RAISE_R_EYEBROW_L].coords[0].which = COORD_EYEBROW_V_RIGHT_LEFT;
  tab[RAISE_R_EYEBROW_L].coords[0].unit = UNIT_ENS;
  tab[RAISE_R_EYEBROW_L].coords[1].which = END_COORD_INDEX;

  tab[RAISE_L_EYEBROW_M].interval = INTERV_BI;
  tab[RAISE_L_EYEBROW_M].coords[0].which = COORD_EYEBROW_V_LEFT_MIDDLE;
  tab[RAISE_L_EYEBROW_M].coords[0].unit = UNIT_ENS;
  tab[RAISE_L_EYEBROW_M].coords[1].which = COORD_EYEBROW_V_LEFT_LEFT;
  tab[RAISE_L_EYEBROW_M].coords[1].unit = UNIT_ENS;
  tab[RAISE_L_EYEBROW_M].coords[2].which = COORD_EYEBROW_V_LEFT_RIGHT;
  tab[RAISE_L_EYEBROW_M].coords[2].unit = UNIT_ENS;
  tab[RAISE_L_EYEBROW_M].coords[3].which = END_COORD_INDEX;

  tab[RAISE_R_EYEBROW_M].interval = INTERV_BI;
  tab[RAISE_R_EYEBROW_M].coords[0].which = COORD_EYEBROW_V_RIGHT_MIDDLE;
  tab[RAISE_R_EYEBROW_M].coords[0].unit = UNIT_ENS;
  tab[RAISE_R_EYEBROW_M].coords[1].which = COORD_EYEBROW_V_RIGHT_LEFT;
  tab[RAISE_R_EYEBROW_M].coords[1].unit = UNIT_ENS;
  tab[RAISE_R_EYEBROW_M].coords[2].which = COORD_EYEBROW_V_RIGHT_RIGHT;
  tab[RAISE_R_EYEBROW_M].coords[2].unit = UNIT_ENS;
  tab[RAISE_R_EYEBROW_M].coords[3].which = END_COORD_INDEX;

  tab[RAISE_L_EYEBROW_R].interval = INTERV_BI;
  tab[RAISE_L_EYEBROW_R].coords[0].which = COORD_EYEBROW_V_LEFT_RIGHT;
  tab[RAISE_L_EYEBROW_R].coords[0].unit = UNIT_ENS;
  tab[RAISE_L_EYEBROW_R].coords[1].which = END_COORD_INDEX;

  tab[RAISE_R_EYEBROW_R].interval = INTERV_BI;
  tab[RAISE_R_EYEBROW_R].coords[0].which = COORD_EYEBROW_V_RIGHT_RIGHT;
  tab[RAISE_R_EYEBROW_R].coords[0].unit = UNIT_ENS;
  tab[RAISE_R_EYEBROW_R].coords[1].which = END_COORD_INDEX;

  tab[SQUEEZE_L_EYEBROW].interval = INTERV_UNI;
  tab[SQUEEZE_L_EYEBROW].coords[0].which = COORD_EYEBROW_H_LEFT;
  tab[SQUEEZE_L_EYEBROW].coords[0].unit = UNIT_ES;
  tab[SQUEEZE_L_EYEBROW].coords[1].which = END_COORD_INDEX;

  tab[SQUEEZE_R_EYEBROW].interval = INTERV_UNI;
  tab[SQUEEZE_R_EYEBROW].coords[0].which = COORD_EYEBROW_H_RIGHT;
  tab[SQUEEZE_R_EYEBROW].coords[0].unit = UNIT_ES;
  tab[SQUEEZE_R_EYEBROW].coords[1].which = END_COORD_INDEX;

  tab[MOVE_O_L_EYEBALL].interval = INTERV_BI;
  tab[MOVE_O_L_EYEBALL].coords[0].which = COORD_EYE_F_LEFT;
  tab[MOVE_O_L_EYEBALL].coords[0].unit = UNIT_IRISD; 
  tab[MOVE_O_L_EYEBALL].coords[1].which = END_COORD_INDEX;

  tab[MOVE_O_R_EYEBALL].interval = INTERV_BI;
  tab[MOVE_O_R_EYEBALL].coords[0].which = COORD_EYE_F_RIGHT;
  tab[MOVE_O_R_EYEBALL].coords[0].unit = UNIT_IRISD; 
  tab[MOVE_O_R_EYEBALL].coords[1].which = END_COORD_INDEX;

  tab[OPEN_JAW].interval = INTERV_BI;
  tab[OPEN_JAW].coords[0].which = COORD_JAW_V;
  tab[OPEN_JAW].coords[0].unit = UNIT_MNS;
  tab[OPEN_JAW].coords[1].which = COORD_MOUTH_V_BOTTOM;
  tab[OPEN_JAW].coords[1].unit = UNIT_MNS;
  tab[OPEN_JAW].coords[2].which = END_COORD_INDEX;

  tab[MOVE_HORI_JAW].interval = INTERV_BI;
  tab[MOVE_HORI_JAW].coords[0].which = COORD_JAW_H;
  tab[MOVE_HORI_JAW].coords[0].unit = UNIT_MNS;
  tab[MOVE_HORI_JAW].coords[1].which = END_COORD_INDEX;

  tab[DEPRESS_CHIN].interval = INTERV_BI;
  tab[DEPRESS_CHIN].coords[0].which = COORD_CHIN_F;
  tab[DEPRESS_CHIN].coords[0].unit = UNIT_MNS;
  tab[DEPRESS_CHIN].coords[1].which = COORD_JAW_V;
  tab[DEPRESS_CHIN].coords[1].unit = UNIT_MNS;
  tab[DEPRESS_CHIN].coords[2].which = END_COORD_INDEX;

  tab[RAISE_L_CORNERLIP].interval = INTERV_BI;
  tab[RAISE_L_CORNERLIP].coords[0].which = COORD_MOUTH_V_LEFT;
  tab[RAISE_L_CORNERLIP].coords[0].unit = UNIT_MNS;
  tab[RAISE_L_CORNERLIP].coords[1].which = COORD_MOUTH_H_LEFT;
  tab[RAISE_L_CORNERLIP].coords[1].unit = UNIT_MW;
  tab[RAISE_L_CORNERLIP].coords[2].which = COORD_MOUTH_V_TOP;
  tab[RAISE_L_CORNERLIP].coords[2].unit = UNIT_MNS;
  tab[RAISE_L_CORNERLIP].coords[3].which = COORD_MOUTH_V_BOTTOM;
  tab[RAISE_L_CORNERLIP].coords[3].unit = UNIT_MNS;
  tab[RAISE_L_CORNERLIP].coords[4].which = COORD_CHEEK_V_LEFT;
  tab[RAISE_L_CORNERLIP].coords[4].unit = UNIT_ENS;
  tab[RAISE_L_CORNERLIP].coords[5].which = COORD_CHEEK_H_LEFT;
  tab[RAISE_L_CORNERLIP].coords[5].unit = UNIT_ES;
  tab[RAISE_L_CORNERLIP].coords[6].which = END_COORD_INDEX;

  tab[RAISE_R_CORNERLIP].interval = INTERV_BI;
  tab[RAISE_R_CORNERLIP].coords[0].which = COORD_MOUTH_V_RIGHT;
  tab[RAISE_R_CORNERLIP].coords[0].unit = UNIT_MNS;
  tab[RAISE_R_CORNERLIP].coords[1].which = COORD_MOUTH_H_RIGHT;
  tab[RAISE_R_CORNERLIP].coords[1].unit = UNIT_MW;
  tab[RAISE_R_CORNERLIP].coords[2].which = COORD_MOUTH_V_TOP;
  tab[RAISE_R_CORNERLIP].coords[2].unit = UNIT_MNS;
  tab[RAISE_R_CORNERLIP].coords[3].which = COORD_MOUTH_V_BOTTOM;
  tab[RAISE_R_CORNERLIP].coords[3].unit = UNIT_MNS;
  tab[RAISE_R_CORNERLIP].coords[4].which = COORD_CHEEK_V_RIGHT;
  tab[RAISE_R_CORNERLIP].coords[4].unit = UNIT_ENS;
  tab[RAISE_R_CORNERLIP].coords[5].which = COORD_CHEEK_H_RIGHT;
  tab[RAISE_R_CORNERLIP].coords[5].unit = UNIT_ES;
  tab[RAISE_R_CORNERLIP].coords[6].which = END_COORD_INDEX;

  tab[PUFF_L_CHEEK].interval = INTERV_BI;
  tab[PUFF_L_CHEEK].coords[0].which = COORD_CHEEK_H_LEFT;
  tab[PUFF_L_CHEEK].coords[0].unit = UNIT_ES;
  tab[PUFF_L_CHEEK].coords[1].which = END_COORD_INDEX;

  tab[PUFF_R_CHEEK].interval = INTERV_BI;
  tab[PUFF_R_CHEEK].coords[0].which = COORD_CHEEK_H_RIGHT;
  tab[PUFF_R_CHEEK].coords[0].unit = UNIT_ES;
  tab[PUFF_R_CHEEK].coords[1].which = END_COORD_INDEX;

  tab[LIFT_L_CHEEK].interval = INTERV_BI;
  tab[LIFT_L_CHEEK].coords[0].which = COORD_CHEEK_V_LEFT;
  tab[LIFT_L_CHEEK].coords[0].unit = UNIT_ENS;
  tab[LIFT_L_CHEEK].coords[1].which = COORD_CHEEK_H_LEFT;
  tab[LIFT_L_CHEEK].coords[1].unit = UNIT_ES;
  tab[LIFT_L_CHEEK].coords[2].which = END_COORD_INDEX;

  tab[LIFT_R_CHEEK].interval = INTERV_BI;
  tab[LIFT_R_CHEEK].coords[0].which = COORD_CHEEK_V_RIGHT;
  tab[LIFT_R_CHEEK].coords[0].unit = UNIT_ENS;
  tab[LIFT_R_CHEEK].coords[1].which = COORD_CHEEK_H_RIGHT;
  tab[LIFT_R_CHEEK].coords[1].unit = UNIT_ES;
  tab[LIFT_R_CHEEK].coords[2].which = END_COORD_INDEX;

  tab[LOWER_L_CORNERLIP].interval = INTERV_BI;
  tab[LOWER_L_CORNERLIP].coords[0].which = COORD_MOUTH_V_LEFT;
  tab[LOWER_L_CORNERLIP].coords[0].unit = UNIT_MNS;
  tab[LOWER_L_CORNERLIP].coords[1].which = COORD_MOUTH_V_TOP;
  tab[LOWER_L_CORNERLIP].coords[1].unit = UNIT_MNS;
  tab[LOWER_L_CORNERLIP].coords[2].which = COORD_MOUTH_V_BOTTOM;
  tab[LOWER_L_CORNERLIP].coords[2].unit = UNIT_MNS;
  tab[LOWER_L_CORNERLIP].coords[3].which = COORD_CHEEK_V_LEFT;
  tab[LOWER_L_CORNERLIP].coords[3].unit = UNIT_ENS;
  tab[LOWER_L_CORNERLIP].coords[4].which = END_COORD_INDEX;

  tab[LOWER_R_CORNERLIP].interval = INTERV_BI;
  tab[LOWER_R_CORNERLIP].coords[0].which = COORD_MOUTH_V_RIGHT;
  tab[LOWER_R_CORNERLIP].coords[0].unit = UNIT_MNS;
  tab[LOWER_R_CORNERLIP].coords[1].which = COORD_MOUTH_V_TOP;
  tab[LOWER_R_CORNERLIP].coords[1].unit = UNIT_MNS;
  tab[LOWER_R_CORNERLIP].coords[2].which = COORD_MOUTH_V_BOTTOM;
  tab[LOWER_R_CORNERLIP].coords[2].unit = UNIT_MNS;
  tab[LOWER_R_CORNERLIP].coords[3].which = COORD_CHEEK_V_RIGHT;
  tab[LOWER_R_CORNERLIP].coords[3].unit = UNIT_ENS;
  tab[LOWER_R_CORNERLIP].coords[4].which = END_COORD_INDEX;

  tab[RAISE_UPPERLIP].interval = INTERV_BI;
  tab[RAISE_UPPERLIP].coords[0].which = COORD_MOUTH_V_TOP;
  tab[RAISE_UPPERLIP].coords[0].unit = UNIT_MNS;
  tab[RAISE_UPPERLIP].coords[1].which = END_COORD_INDEX;

  tab[LOWER_LOWERLIP].interval = INTERV_BI;
  tab[LOWER_LOWERLIP].coords[0].which = COORD_MOUTH_V_BOTTOM;
  tab[LOWER_LOWERLIP].coords[0].unit = UNIT_MNS;
  tab[LOWER_LOWERLIP].coords[1].which = END_COORD_INDEX;

  tab[RAISE_U_MIDLIP].interval = INTERV_UNI;
  tab[RAISE_U_MIDLIP].coords[0].which = COORD_MOUTH_V_TOP;
  tab[RAISE_U_MIDLIP].coords[0].unit = UNIT_MNS;
  tab[RAISE_U_MIDLIP].coords[1].which = END_COORD_INDEX;

  tab[RAISE_L_MIDLIP].interval = INTERV_UNI;
  tab[RAISE_L_MIDLIP].coords[0].which = COORD_MOUTH_V_BOTTOM;
  tab[RAISE_L_MIDLIP].coords[0].unit = UNIT_MNS;
  tab[RAISE_L_MIDLIP].coords[1].which = END_COORD_INDEX;

  tab[PULL_MIDLIPS].interval = INTERV_BI;
  tab[PULL_MIDLIPS].coords[0].which = COORD_MOUTH_F;
  tab[PULL_MIDLIPS].coords[0].unit = UNIT_MNS;
  tab[PULL_MIDLIPS].coords[1].which = COORD_CHIN_F;
  tab[PULL_MIDLIPS].coords[1].unit = UNIT_MNS;
  tab[PULL_MIDLIPS].coords[2].which = COORD_MOUTH_H_LEFT;
  tab[PULL_MIDLIPS].coords[2].unit = UNIT_MW;
  tab[PULL_MIDLIPS].coords[3].which = COORD_MOUTH_H_RIGHT;
  tab[PULL_MIDLIPS].coords[3].unit = UNIT_MW;
  tab[PULL_MIDLIPS].coords[4].which = COORD_MOUTH_V_TOP;
  tab[PULL_MIDLIPS].coords[4].unit = UNIT_MNS;
  tab[PULL_MIDLIPS].coords[5].which = COORD_MOUTH_V_BOTTOM;
  tab[PULL_MIDLIPS].coords[5].unit = UNIT_MNS;
  tab[PULL_MIDLIPS].coords[6].which = END_COORD_INDEX;

  tab[STRETCH_L_CORNERLIP].interval = INTERV_BI;
  tab[STRETCH_L_CORNERLIP].coords[0].which = COORD_MOUTH_H_LEFT;
  tab[STRETCH_L_CORNERLIP].coords[0].unit = UNIT_MW;
  tab[STRETCH_L_CORNERLIP].coords[1].which = END_COORD_INDEX;

  tab[STRETCH_R_CORNERLIP].interval = INTERV_BI;
  tab[STRETCH_R_CORNERLIP].coords[0].which = COORD_MOUTH_H_RIGHT;
  tab[STRETCH_R_CORNERLIP].coords[0].unit = UNIT_MW;
  tab[STRETCH_R_CORNERLIP].coords[1].which = END_COORD_INDEX;

  tab[RAISE_NOSE].interval = INTERV_UNI;
  tab[RAISE_NOSE].coords[0].which = COORD_NOSE_V;
  tab[RAISE_NOSE].coords[0].unit = UNIT_ENS;
  tab[RAISE_NOSE].coords[1].which = COORD_MOUTH_V_TOP;
  tab[RAISE_NOSE].coords[1].unit = UNIT_MNS;
  tab[RAISE_NOSE].coords[2].which = END_COORD_INDEX;

  tab[TURN_HEAD].interval = INTERV_BI;
  tab[TURN_HEAD].coords[0].which = COORD_HEAD_TURN;
  tab[TURN_HEAD].coords[0].unit = UNIT_AU;
  tab[TURN_HEAD].coords[1].which = END_COORD_INDEX;

  tab[NOD_HEAD].interval = INTERV_BI;
  tab[NOD_HEAD].coords[0].which = COORD_HEAD_NOD;
  tab[NOD_HEAD].coords[0].unit = UNIT_AU;
  tab[NOD_HEAD].coords[1].which = END_COORD_INDEX;

  tab[ROLL_HEAD].interval = INTERV_BI;
  tab[ROLL_HEAD].coords[0].which = COORD_HEAD_ROLL;
  tab[ROLL_HEAD].coords[0].unit = UNIT_AU;
  tab[ROLL_HEAD].coords[1].which = END_COORD_INDEX;

  tab[TONGUE_LIFT].interval = INTERV_BI;
  tab[TONGUE_LIFT].coords[0].which = COORD_TONGUE_V;
  tab[TONGUE_LIFT].coords[0].unit = UNIT_MW;
  tab[TONGUE_LIFT].coords[1].which = END_COORD_INDEX; 

  tab[TONGUE_PAN].interval = INTERV_BI;
  tab[TONGUE_PAN].coords[0].which = COORD_TONGUE_TIP_H;
  tab[TONGUE_PAN].coords[0].unit = UNIT_MW;
  tab[TONGUE_PAN].coords[1].which = END_COORD_INDEX; 

  tab[TONGUE_OUT].interval = INTERV_BI;
  tab[TONGUE_OUT].coords[0].which = COORD_TONGUE_TIP_F;
  tab[TONGUE_OUT].coords[0].unit = UNIT_MW;
  tab[TONGUE_OUT].coords[1].which = END_COORD_INDEX;

  tab[TONGUE_LIFT_TIP].interval = INTERV_BI;
  tab[TONGUE_LIFT_TIP].coords[0].which = COORD_TONGUE_TIP_V;
  tab[TONGUE_LIFT_TIP].coords[0].unit = UNIT_MW;
  tab[TONGUE_LIFT_TIP].coords[1].which = END_COORD_INDEX;

  tab[TONGUE_ROLL].interval = INTERV_BI;
  tab[TONGUE_ROLL].coords[0].which = COORD_TONGUE_ROLL;
  tab[TONGUE_ROLL].coords[0].unit = UNIT_AU;
  tab[TONGUE_ROLL].coords[1].which = END_COORD_INDEX;

  tab[JAW_THRUST].interval = INTERV_BI;
  tab[JAW_THRUST].coords[0].which = COORD_JAW_F;
  tab[JAW_THRUST].coords[0].unit = UNIT_MNS;
  tab[JAW_THRUST].coords[1].which = COORD_MOUTH_F;
  tab[JAW_THRUST].coords[1].unit = UNIT_MNS;
  tab[JAW_THRUST].coords[2].which = END_COORD_INDEX;

  tab[EYE_DILATE_L_PUPIL].interval = INTERV_BI;
  tab[EYE_DILATE_L_PUPIL].coords[0].which = COORD_PUPIL_LEFT;
  tab[EYE_DILATE_L_PUPIL].coords[0].unit = UNIT_IRISD;
  tab[EYE_DILATE_L_PUPIL].coords[1].which = END_COORD_INDEX;

  tab[EYE_DILATE_R_PUPIL].interval = INTERV_BI;
  tab[EYE_DILATE_R_PUPIL].coords[0].which = COORD_PUPIL_RIGHT;
  tab[EYE_DILATE_R_PUPIL].coords[0].unit = UNIT_IRISD;
  tab[EYE_DILATE_R_PUPIL].coords[1].which = END_COORD_INDEX;

  tab[NOSE_BEND].interval = INTERV_BI;
  tab[NOSE_BEND].coords[0].which = COORD_NOSE_TIP_H;
  tab[NOSE_BEND].coords[0].unit = UNIT_ENS;
  tab[NOSE_BEND].coords[1].which = END_COORD_INDEX; 

  tab[STRETCH_L_NOSE].interval = INTERV_BI;
  tab[STRETCH_L_NOSE].coords[0].which = COORD_NOSE_H_LEFT;
  tab[STRETCH_L_NOSE].coords[0].unit = UNIT_ENS;
  tab[STRETCH_L_NOSE].coords[1].which = END_COORD_INDEX;

  tab[STRETCH_R_NOSE].interval = INTERV_BI;
  tab[STRETCH_R_NOSE].coords[0].which = COORD_NOSE_H_RIGHT;
  tab[STRETCH_R_NOSE].coords[0].unit = UNIT_ENS;
  tab[STRETCH_R_NOSE].coords[1].which = END_COORD_INDEX;

  tab[RAISE_L_EAR].interval = INTERV_BI;
  tab[RAISE_L_EAR].coords[0].which = COORD_EAR_V_LEFT;
  tab[RAISE_L_EAR].coords[0].unit = UNIT_ENS;
  tab[RAISE_L_EAR].coords[1].which = END_COORD_INDEX; 

  tab[RAISE_R_EAR].interval = INTERV_BI;
  tab[RAISE_R_EAR].coords[0].which = COORD_EAR_V_RIGHT;
  tab[RAISE_R_EAR].coords[0].unit = UNIT_ENS;
  tab[RAISE_R_EAR].coords[1].which = END_COORD_INDEX; 

  tab[PULL_L_EAR].interval = INTERV_BI;
  tab[PULL_L_EAR].coords[0].which = COORD_EAR_H_LEFT;
  tab[PULL_L_EAR].coords[0].unit = UNIT_ENS;
  tab[PULL_L_EAR].coords[1].which = END_COORD_INDEX; 

  tab[PULL_R_EAR].interval = INTERV_BI;
  tab[PULL_R_EAR].coords[0].which = COORD_EAR_H_RIGHT;
  tab[PULL_R_EAR].coords[0].unit = UNIT_ENS;
  tab[PULL_R_EAR].coords[1].which = END_COORD_INDEX; 
}


void init_faps(Fap_def tab[])
{
  tab[FAP_VISEME].group = FAP_GROUP_VISEME_AND_EXPRESSION;
  tab[FAP_VISEME].unit = UNIT_NOT_DEFINED;
  tab[FAP_VISEME].interval = INTERV_NOT_DEFINED;
  tab[FAP_VISEME].direction = FAP_DIR_NOT_DEFINED;

  tab[FAP_EXPRESSION].group = FAP_GROUP_VISEME_AND_EXPRESSION;
  tab[FAP_EXPRESSION].unit = UNIT_NOT_DEFINED;
  tab[FAP_EXPRESSION].interval = INTERV_NOT_DEFINED;
  tab[FAP_EXPRESSION].direction = FAP_DIR_NOT_DEFINED;

  tab[FAP_OPEN_JAW].group = FAP_GROUP_INNERLIPS_AND_JAW;
  tab[FAP_OPEN_JAW].unit = UNIT_MNS;
  tab[FAP_OPEN_JAW].interval = INTERV_UNI;
  tab[FAP_OPEN_JAW].direction = FAP_DIR_DOWN;

  tab[FAP_LOWER_T_MIDLIP].group = FAP_GROUP_INNERLIPS_AND_JAW;
  tab[FAP_LOWER_T_MIDLIP].unit = UNIT_MNS;
  tab[FAP_LOWER_T_MIDLIP].interval = INTERV_BI;
  tab[FAP_LOWER_T_MIDLIP].direction = FAP_DIR_DOWN;

  tab[FAP_RAISE_B_MIDLIP].group = FAP_GROUP_INNERLIPS_AND_JAW;
  tab[FAP_RAISE_B_MIDLIP].unit = UNIT_MNS;
  tab[FAP_RAISE_B_MIDLIP].interval = INTERV_BI;
  tab[FAP_RAISE_B_MIDLIP].direction = FAP_DIR_UP;

  tab[FAP_STRETCH_L_CORNERLIP].group = FAP_GROUP_INNERLIPS_AND_JAW;
  tab[FAP_STRETCH_L_CORNERLIP].unit = UNIT_MW;
  tab[FAP_STRETCH_L_CORNERLIP].interval = INTERV_BI;
  tab[FAP_STRETCH_L_CORNERLIP].direction = FAP_DIR_LEFT;

  tab[FAP_STRETCH_R_CORNERLIP].group = FAP_GROUP_INNERLIPS_AND_JAW;
  tab[FAP_STRETCH_R_CORNERLIP].unit = UNIT_MW;
  tab[FAP_STRETCH_R_CORNERLIP].interval = INTERV_BI;
  tab[FAP_STRETCH_R_CORNERLIP].direction = FAP_DIR_RIGHT;

  tab[FAP_LOWER_T_LIP_LM].group = FAP_GROUP_INNERLIPS_AND_JAW;
  tab[FAP_LOWER_T_LIP_LM].unit = UNIT_MNS;
  tab[FAP_LOWER_T_LIP_LM].interval = INTERV_BI;
  tab[FAP_LOWER_T_LIP_LM].direction = FAP_DIR_DOWN;

  tab[FAP_LOWER_T_LIP_RM].group = FAP_GROUP_INNERLIPS_AND_JAW;
  tab[FAP_LOWER_T_LIP_RM].unit = UNIT_MNS;
  tab[FAP_LOWER_T_LIP_RM].interval = INTERV_BI;
  tab[FAP_LOWER_T_LIP_RM].direction = FAP_DIR_DOWN;

  tab[FAP_RAISE_B_LIP_LM].group = FAP_GROUP_INNERLIPS_AND_JAW;
  tab[FAP_RAISE_B_LIP_LM].unit = UNIT_MNS;
  tab[FAP_RAISE_B_LIP_LM].interval = INTERV_BI;
  tab[FAP_RAISE_B_LIP_LM].direction = FAP_DIR_UP;

  tab[FAP_RAISE_B_LIP_RM].group = FAP_GROUP_INNERLIPS_AND_JAW;
  tab[FAP_RAISE_B_LIP_RM].unit = UNIT_MNS;
  tab[FAP_RAISE_B_LIP_RM].interval = INTERV_BI;
  tab[FAP_RAISE_B_LIP_RM].direction = FAP_DIR_UP;

  tab[FAP_RAISE_L_CORNERLIP].group = FAP_GROUP_INNERLIPS_AND_JAW;
  tab[FAP_RAISE_L_CORNERLIP].unit = UNIT_MNS;
  tab[FAP_RAISE_L_CORNERLIP].interval = INTERV_BI;
  tab[FAP_RAISE_L_CORNERLIP].direction = FAP_DIR_UP;

  tab[FAP_RAISE_R_CORNERLIP].group = FAP_GROUP_INNERLIPS_AND_JAW;
  tab[FAP_RAISE_R_CORNERLIP].unit = UNIT_MNS;
  tab[FAP_RAISE_R_CORNERLIP].interval = INTERV_BI;
  tab[FAP_RAISE_R_CORNERLIP].direction = FAP_DIR_UP;

  tab[FAP_THRUST_JAW].group = FAP_GROUP_INNERLIPS_AND_JAW;
  tab[FAP_THRUST_JAW].unit = UNIT_MNS;
  tab[FAP_THRUST_JAW].interval = INTERV_UNI;
  tab[FAP_THRUST_JAW].direction = FAP_DIR_FORWARD;

  tab[FAP_SHIFT_JAW].group = FAP_GROUP_INNERLIPS_AND_JAW;
  tab[FAP_SHIFT_JAW].unit = UNIT_MNS;
  tab[FAP_SHIFT_JAW].interval = INTERV_BI;
  tab[FAP_SHIFT_JAW].direction = FAP_DIR_RIGHT;

  tab[FAP_PUSH_B_LIP].group = FAP_GROUP_INNERLIPS_AND_JAW;
  tab[FAP_PUSH_B_LIP].unit = UNIT_MNS;
  tab[FAP_PUSH_B_LIP].interval = INTERV_BI;
  tab[FAP_PUSH_B_LIP].direction = FAP_DIR_FORWARD;

  tab[FAP_PUSH_T_LIP].group = FAP_GROUP_INNERLIPS_AND_JAW;
  tab[FAP_PUSH_T_LIP].unit = UNIT_MNS;
  tab[FAP_PUSH_T_LIP].interval = INTERV_BI;
  tab[FAP_PUSH_T_LIP].direction = FAP_DIR_FORWARD;

  tab[FAP_DEPRESS_CHIN].group = FAP_GROUP_INNERLIPS_AND_JAW;
  tab[FAP_DEPRESS_CHIN].unit = UNIT_MNS;
  tab[FAP_DEPRESS_CHIN].interval = INTERV_UNI;
  tab[FAP_DEPRESS_CHIN].direction = FAP_DIR_UP;

  tab[FAP_CLOSE_T_L_EYELID].group = FAP_GROUP_EYES;
  tab[FAP_CLOSE_T_L_EYELID].unit = UNIT_IRISD;
  tab[FAP_CLOSE_T_L_EYELID].interval = INTERV_BI;
  tab[FAP_CLOSE_T_L_EYELID].direction = FAP_DIR_DOWN;

  tab[FAP_CLOSE_T_R_EYELID].group = FAP_GROUP_EYES;
  tab[FAP_CLOSE_T_R_EYELID].unit = UNIT_IRISD;
  tab[FAP_CLOSE_T_R_EYELID].interval = INTERV_BI;
  tab[FAP_CLOSE_T_R_EYELID].direction = FAP_DIR_DOWN;

  tab[FAP_CLOSE_B_L_EYELID].group = FAP_GROUP_EYES;
  tab[FAP_CLOSE_B_L_EYELID].unit = UNIT_IRISD;
  tab[FAP_CLOSE_B_L_EYELID].interval = INTERV_BI;
  tab[FAP_CLOSE_B_L_EYELID].direction = FAP_DIR_UP;

  tab[FAP_CLOSE_B_R_EYELID].group = FAP_GROUP_EYES;
  tab[FAP_CLOSE_B_R_EYELID].unit = UNIT_IRISD;
  tab[FAP_CLOSE_B_R_EYELID].interval = INTERV_BI;
  tab[FAP_CLOSE_B_R_EYELID].direction = FAP_DIR_UP;

  tab[FAP_YAW_L_EYEBALL].group = FAP_GROUP_EYES;
  tab[FAP_YAW_L_EYEBALL].unit = UNIT_AU;
  tab[FAP_YAW_L_EYEBALL].interval = INTERV_BI;
  tab[FAP_YAW_L_EYEBALL].direction = FAP_DIR_LEFT;

  tab[FAP_YAW_R_EYEBALL].group = FAP_GROUP_EYES;
  tab[FAP_YAW_R_EYEBALL].unit = UNIT_AU;
  tab[FAP_YAW_R_EYEBALL].interval = INTERV_BI;
  tab[FAP_YAW_R_EYEBALL].direction = FAP_DIR_LEFT;

  tab[FAP_PITCH_L_EYEBALL].group = FAP_GROUP_EYES;
  tab[FAP_PITCH_L_EYEBALL].unit = UNIT_AU;
  tab[FAP_PITCH_L_EYEBALL].interval = INTERV_BI;
  tab[FAP_PITCH_L_EYEBALL].direction = FAP_DIR_DOWN;

  tab[FAP_PITCH_R_EYEBALL].group = FAP_GROUP_EYES;
  tab[FAP_PITCH_R_EYEBALL].unit = UNIT_AU;
  tab[FAP_PITCH_R_EYEBALL].interval = INTERV_BI;
  tab[FAP_PITCH_R_EYEBALL].direction = FAP_DIR_DOWN;

  tab[FAP_THRUST_L_EYEBALL].group = FAP_GROUP_EYES;
  tab[FAP_THRUST_L_EYEBALL].unit = UNIT_IRISD;
  tab[FAP_THRUST_L_EYEBALL].interval = INTERV_BI;
  tab[FAP_THRUST_L_EYEBALL].direction = FAP_DIR_FORWARD;

  tab[FAP_THRUST_R_EYEBALL].group = FAP_GROUP_EYES;
  tab[FAP_THRUST_R_EYEBALL].unit = UNIT_IRISD;
  tab[FAP_THRUST_R_EYEBALL].interval = INTERV_BI;
  tab[FAP_THRUST_R_EYEBALL].direction = FAP_DIR_FORWARD;

  tab[FAP_DILATE_L_PUPIL].group = FAP_GROUP_EYES;
  tab[FAP_DILATE_L_PUPIL].unit = UNIT_IRISD;
  tab[FAP_DILATE_L_PUPIL].interval = INTERV_UNI;
  tab[FAP_DILATE_L_PUPIL].direction = FAP_DIR_NOT_DEFINED;

  tab[FAP_DILATE_R_PUPIL].group = FAP_GROUP_EYES;
  tab[FAP_DILATE_R_PUPIL].unit = UNIT_IRISD;
  tab[FAP_DILATE_R_PUPIL].interval = INTERV_UNI;
  tab[FAP_DILATE_R_PUPIL].direction = FAP_DIR_NOT_DEFINED;

  tab[FAP_RAISE_L_I_EYEBROW].group = FAP_GROUP_EYEBROWS;
  tab[FAP_RAISE_L_I_EYEBROW].unit = UNIT_ENS;
  tab[FAP_RAISE_L_I_EYEBROW].interval = INTERV_BI;
  tab[FAP_RAISE_L_I_EYEBROW].direction = FAP_DIR_UP;

  tab[FAP_RAISE_R_I_EYEBROW].group = FAP_GROUP_EYEBROWS;
  tab[FAP_RAISE_R_I_EYEBROW].unit = UNIT_ENS;
  tab[FAP_RAISE_R_I_EYEBROW].interval = INTERV_BI;
  tab[FAP_RAISE_R_I_EYEBROW].direction = FAP_DIR_UP;

  tab[FAP_RAISE_L_M_EYEBROW].group = FAP_GROUP_EYEBROWS;
  tab[FAP_RAISE_L_M_EYEBROW].unit = UNIT_ENS;
  tab[FAP_RAISE_L_M_EYEBROW].interval = INTERV_BI;
  tab[FAP_RAISE_L_M_EYEBROW].direction = FAP_DIR_UP;

  tab[FAP_RAISE_R_M_EYEBROW].group = FAP_GROUP_EYEBROWS;
  tab[FAP_RAISE_R_M_EYEBROW].unit = UNIT_ENS;
  tab[FAP_RAISE_R_M_EYEBROW].interval = INTERV_BI;
  tab[FAP_RAISE_R_M_EYEBROW].direction = FAP_DIR_UP;

  tab[FAP_RAISE_L_O_EYEBROW].group = FAP_GROUP_EYEBROWS;
  tab[FAP_RAISE_L_O_EYEBROW].unit = UNIT_ENS;
  tab[FAP_RAISE_L_O_EYEBROW].interval = INTERV_BI;
  tab[FAP_RAISE_L_O_EYEBROW].direction = FAP_DIR_UP;

  tab[FAP_RAISE_R_O_EYEBROW].group = FAP_GROUP_EYEBROWS;
  tab[FAP_RAISE_R_O_EYEBROW].unit = UNIT_ENS;
  tab[FAP_RAISE_R_O_EYEBROW].interval = INTERV_BI;
  tab[FAP_RAISE_R_O_EYEBROW].direction = FAP_DIR_UP;

  tab[FAP_SQUEEZE_L_EYEBROW].group = FAP_GROUP_EYEBROWS;
  tab[FAP_SQUEEZE_L_EYEBROW].unit = UNIT_ES;
  tab[FAP_SQUEEZE_L_EYEBROW].interval = INTERV_BI;
  tab[FAP_SQUEEZE_L_EYEBROW].direction = FAP_DIR_RIGHT;

  tab[FAP_SQUEEZE_R_EYEBROW].group = FAP_GROUP_EYEBROWS;
  tab[FAP_SQUEEZE_R_EYEBROW].unit = UNIT_ES;
  tab[FAP_SQUEEZE_R_EYEBROW].interval = INTERV_BI;
  tab[FAP_SQUEEZE_R_EYEBROW].direction = FAP_DIR_LEFT;

  tab[FAP_PUFF_L_CHEEK].group = FAP_GROUP_CHEEKS;
  tab[FAP_PUFF_L_CHEEK].unit = UNIT_ES;
  tab[FAP_PUFF_L_CHEEK].interval = INTERV_BI;
  tab[FAP_PUFF_L_CHEEK].direction = FAP_DIR_LEFT;

  tab[FAP_PUFF_R_CHEEK].group = FAP_GROUP_CHEEKS;
  tab[FAP_PUFF_R_CHEEK].unit = UNIT_ES;
  tab[FAP_PUFF_R_CHEEK].interval = INTERV_BI;
  tab[FAP_PUFF_R_CHEEK].direction = FAP_DIR_RIGHT;

  tab[FAP_LIFT_L_CHEEK].group = FAP_GROUP_CHEEKS;
  tab[FAP_LIFT_L_CHEEK].unit = UNIT_ENS;
  tab[FAP_LIFT_L_CHEEK].interval = INTERV_UNI;
  tab[FAP_LIFT_L_CHEEK].direction = FAP_DIR_UP;

  tab[FAP_LIFT_R_CHEEK].group = FAP_GROUP_CHEEKS;
  tab[FAP_LIFT_R_CHEEK].unit = UNIT_ENS;
  tab[FAP_LIFT_R_CHEEK].interval = INTERV_UNI;
  tab[FAP_LIFT_R_CHEEK].direction = FAP_DIR_UP;

  tab[FAP_SHIFT_TONGUE_TIP].group = FAP_GROUP_TONGUE;
  tab[FAP_SHIFT_TONGUE_TIP].unit = UNIT_MW;
  tab[FAP_SHIFT_TONGUE_TIP].interval = INTERV_BI;
  tab[FAP_SHIFT_TONGUE_TIP].direction = FAP_DIR_RIGHT;

  tab[FAP_RAISE_TONGUE_TIP].group = FAP_GROUP_TONGUE;
  tab[FAP_RAISE_TONGUE_TIP].unit = UNIT_MW;
  tab[FAP_RAISE_TONGUE_TIP].interval = INTERV_BI;
  tab[FAP_RAISE_TONGUE_TIP].direction = FAP_DIR_UP;

  tab[FAP_THRUST_TONGUE_TIP].group = FAP_GROUP_TONGUE;
  tab[FAP_THRUST_TONGUE_TIP].unit = UNIT_MW;
  tab[FAP_THRUST_TONGUE_TIP].interval = INTERV_BI;
  tab[FAP_THRUST_TONGUE_TIP].direction = FAP_DIR_FORWARD;

  tab[FAP_RAISE_TONGUE].group = FAP_GROUP_TONGUE;
  tab[FAP_RAISE_TONGUE].unit = UNIT_MW;
  tab[FAP_RAISE_TONGUE].interval = INTERV_BI;
  tab[FAP_RAISE_TONGUE].direction = FAP_DIR_UP;

  tab[FAP_TONGUE_ROLL].group = FAP_GROUP_TONGUE;
  tab[FAP_TONGUE_ROLL].unit = UNIT_AU;
  tab[FAP_TONGUE_ROLL].interval = INTERV_UNI;
  tab[FAP_TONGUE_ROLL].direction = FAP_DIR_FORWARD;

  tab[FAP_HEAD_PITCH].group = FAP_GROUP_HEAD;
  tab[FAP_HEAD_PITCH].unit = UNIT_AU;
  tab[FAP_HEAD_PITCH].interval = INTERV_BI;
  tab[FAP_HEAD_PITCH].direction = FAP_DIR_DOWN;

  tab[FAP_HEAD_YAW].group = FAP_GROUP_HEAD;
  tab[FAP_HEAD_YAW].unit = UNIT_AU;
  tab[FAP_HEAD_YAW].interval = INTERV_BI;
  tab[FAP_HEAD_YAW].direction = FAP_DIR_LEFT;

  tab[FAP_HEAD_ROLL].group = FAP_GROUP_HEAD;
  tab[FAP_HEAD_ROLL].unit = UNIT_AU;
  tab[FAP_HEAD_ROLL].interval = INTERV_BI;
  tab[FAP_HEAD_ROLL].direction = FAP_DIR_RIGHT;

  tab[FAP_LOWER_T_MIDLIP_O].group = FAP_GROUP_OUTERLIPS;
  tab[FAP_LOWER_T_MIDLIP_O].unit = UNIT_MNS;
  tab[FAP_LOWER_T_MIDLIP_O].interval = INTERV_BI;
  tab[FAP_LOWER_T_MIDLIP_O].direction = FAP_DIR_DOWN;

  tab[FAP_RAISE_B_MIDLIP_O].group = FAP_GROUP_OUTERLIPS;
  tab[FAP_RAISE_B_MIDLIP_O].unit = UNIT_MNS;
  tab[FAP_RAISE_B_MIDLIP_O].interval = INTERV_BI;
  tab[FAP_RAISE_B_MIDLIP_O].direction = FAP_DIR_UP;

  tab[FAP_STRETCH_L_CORNERLIP_O].group = FAP_GROUP_OUTERLIPS;
  tab[FAP_STRETCH_L_CORNERLIP_O].unit = UNIT_MW;
  tab[FAP_STRETCH_L_CORNERLIP_O].interval = INTERV_BI;
  tab[FAP_STRETCH_L_CORNERLIP_O].direction = FAP_DIR_LEFT;

  tab[FAP_STRETCH_R_CORNERLIP_O].group = FAP_GROUP_OUTERLIPS;
  tab[FAP_STRETCH_R_CORNERLIP_O].unit = UNIT_MW;
  tab[FAP_STRETCH_R_CORNERLIP_O].interval = INTERV_BI;
  tab[FAP_STRETCH_R_CORNERLIP_O].direction = FAP_DIR_RIGHT;

  tab[FAP_LOWER_T_LIP_LM_O].group = FAP_GROUP_OUTERLIPS;
  tab[FAP_LOWER_T_LIP_LM_O].unit = UNIT_MNS;
  tab[FAP_LOWER_T_LIP_LM_O].interval = INTERV_BI;
  tab[FAP_LOWER_T_LIP_LM_O].direction = FAP_DIR_DOWN;

  tab[FAP_LOWER_T_LIP_RM_O].group = FAP_GROUP_OUTERLIPS;
  tab[FAP_LOWER_T_LIP_RM_O].unit = UNIT_MNS;
  tab[FAP_LOWER_T_LIP_RM_O].interval = INTERV_BI;
  tab[FAP_LOWER_T_LIP_RM_O].direction = FAP_DIR_DOWN;

  tab[FAP_RAISE_B_LIP_LM_O].group = FAP_GROUP_OUTERLIPS;
  tab[FAP_RAISE_B_LIP_LM_O].unit = UNIT_MNS;
  tab[FAP_RAISE_B_LIP_LM_O].interval = INTERV_BI;
  tab[FAP_RAISE_B_LIP_LM_O].direction = FAP_DIR_UP;

  tab[FAP_RAISE_B_LIP_RM_O].group = FAP_GROUP_OUTERLIPS;
  tab[FAP_RAISE_B_LIP_RM_O].unit = UNIT_MNS;
  tab[FAP_RAISE_B_LIP_RM_O].interval = INTERV_BI;
  tab[FAP_RAISE_B_LIP_RM_O].direction = FAP_DIR_UP;

  tab[FAP_RAISE_L_CORNERLIP_O].group = FAP_GROUP_OUTERLIPS;
  tab[FAP_RAISE_L_CORNERLIP_O].unit = UNIT_MNS;
  tab[FAP_RAISE_L_CORNERLIP_O].interval = INTERV_BI;
  tab[FAP_RAISE_L_CORNERLIP_O].direction = FAP_DIR_UP;

  tab[FAP_RAISE_R_CORNERLIP_O].group = FAP_GROUP_OUTERLIPS;
  tab[FAP_RAISE_R_CORNERLIP_O].unit = UNIT_MNS;
  tab[FAP_RAISE_R_CORNERLIP_O].interval = INTERV_BI;
  tab[FAP_RAISE_R_CORNERLIP_O].direction = FAP_DIR_UP;

  tab[FAP_STRETCH_L_NOSE].group = FAP_GROUP_NOSE;
  tab[FAP_STRETCH_L_NOSE].unit = UNIT_ENS;
  tab[FAP_STRETCH_L_NOSE].interval = INTERV_BI;
  tab[FAP_STRETCH_L_NOSE].direction = FAP_DIR_LEFT;

  tab[FAP_STRETCH_R_NOSE].group = FAP_GROUP_NOSE;
  tab[FAP_STRETCH_R_NOSE].unit = UNIT_ENS;
  tab[FAP_STRETCH_R_NOSE].interval = INTERV_BI;
  tab[FAP_STRETCH_R_NOSE].direction = FAP_DIR_RIGHT;

  tab[FAP_RAISE_NOSE].group = FAP_GROUP_NOSE;
  tab[FAP_RAISE_NOSE].unit = UNIT_ENS;
  tab[FAP_RAISE_NOSE].interval = INTERV_BI;
  tab[FAP_RAISE_NOSE].direction = FAP_DIR_UP;

  tab[FAP_BEND_NOSE].group = FAP_GROUP_NOSE;
  tab[FAP_BEND_NOSE].unit = UNIT_ENS;
  tab[FAP_BEND_NOSE].interval = INTERV_BI;
  tab[FAP_BEND_NOSE].direction = FAP_DIR_RIGHT;

  tab[FAP_RAISE_L_EAR].group = FAP_GROUP_EARS;
  tab[FAP_RAISE_L_EAR].unit = UNIT_ENS;
  tab[FAP_RAISE_L_EAR].interval = INTERV_BI;
  tab[FAP_RAISE_L_EAR].direction = FAP_DIR_UP;

  tab[FAP_RAISE_R_EAR].group = FAP_GROUP_EARS;
  tab[FAP_RAISE_R_EAR].unit = UNIT_ENS;
  tab[FAP_RAISE_R_EAR].interval = INTERV_BI;
  tab[FAP_RAISE_R_EAR].direction = FAP_DIR_UP;

  tab[FAP_PULL_L_EAR].group = FAP_GROUP_EARS;
  tab[FAP_PULL_L_EAR].unit = UNIT_ENS;
  tab[FAP_PULL_L_EAR].interval = INTERV_BI;
  tab[FAP_PULL_L_EAR].direction = FAP_DIR_LEFT;

  tab[FAP_PULL_R_EAR].group = FAP_GROUP_EARS;
  tab[FAP_PULL_R_EAR].unit = UNIT_ENS;
  tab[FAP_PULL_R_EAR].interval = INTERV_BI;
  tab[FAP_PULL_R_EAR].direction = FAP_DIR_RIGHT;
}


