
/*------------------------------------------------------------------------------*/
/*										*/
/*										*/
/*  This software module was originally developed by                            */
/*        Prem Kalra, MIRALab, University of Geneva                             */
/*  and edited by                                                               */
/*        Igor Pandzic, MIRALab, University of Geneva                           */
/*        Marc Escher, MIRALab, University of Geneva                            */
/*        Pierre Beylot, MIRALab, University of Geneva                          */
/*  in the course of development of the MPEG-4 Video (ISO/IEC 14496-2).	        */
/*  This software module is an implementation of a part of one or more	        */
/*  MPEG-4 Video (ISO/IEC 14496-2) tools as specified by the MPEG-4 Video       */
/*  (ISO/IEC 14496-2). ISO/IEC gives users of the                               */
/*  MPEG-4 Video (ISO/IEC 14496-2) free license to this software module or	*/
/*  modifications thereof for use in hardware or software products claiming 	*/
/*  conformance to the MPEG-4 Video (ISO/IEC 14496-2). Those intending to use	*/
/*  this software module in hardware or software products are advised that	*/
/*  its use may infringe existing patents. The original developer of this	*/
/*  software module and his/her company, the subsequent editors and their	*/
/*  companies, and ISO/IEC have no liability for use of this software		*/
/*  module or modifications thereof. Copyright is not released for non		*/
/*  MPEG-4 Video (ISO/IEC 14496-2) conforming products.                         */
/*  MIRALab, University of Geneva retains full right to use the code for its    */
/*  own purpose, assign or donate the code to a third party and to inhibit      */
/*  third parties from using the code for non MPEG-4 Video (ISO/IEC 14496-2)    */
/*  conforming products. This copyright notice must be included in all copies   */
/*  or derivative works. Copyright (c) (1997).			      		*/
/*										*/
/*										*/
/*------------------------------------------------------------------------------*/


#ifndef _FAP_
#define _FAP_

#include "mpa.h"
#include "SurfMan.h"


/* FAP */
#define FAP_VISEME			0
#define FAP_EXPRESSION			1

#define FAP_OPEN_JAW			2
#define FAP_LOWER_T_MIDLIP		3
#define FAP_RAISE_B_MIDLIP		4
#define FAP_STRETCH_L_CORNERLIP		5
#define FAP_STRETCH_R_CORNERLIP		6
#define FAP_LOWER_T_LIP_LM		7 
#define FAP_LOWER_T_LIP_RM		8 
#define FAP_RAISE_B_LIP_LM		9 
#define FAP_RAISE_B_LIP_RM		10 
#define FAP_RAISE_L_CORNERLIP		11
#define FAP_RAISE_R_CORNERLIP		12
#define FAP_THRUST_JAW			13
#define FAP_SHIFT_JAW			14
#define FAP_PUSH_B_LIP			15
#define FAP_PUSH_T_LIP			16
#define FAP_DEPRESS_CHIN		17

#define FAP_CLOSE_T_L_EYELID		18
#define FAP_CLOSE_T_R_EYELID		19
#define FAP_CLOSE_B_L_EYELID		20
#define FAP_CLOSE_B_R_EYELID		21
#define FAP_YAW_L_EYEBALL		22
#define FAP_YAW_R_EYEBALL		23
#define FAP_PITCH_L_EYEBALL		24
#define FAP_PITCH_R_EYEBALL		25
#define FAP_THRUST_L_EYEBALL		26
#define FAP_THRUST_R_EYEBALL		27
#define FAP_DILATE_L_PUPIL		28
#define FAP_DILATE_R_PUPIL		29

#define FAP_RAISE_L_I_EYEBROW		30
#define FAP_RAISE_R_I_EYEBROW		31
#define FAP_RAISE_L_M_EYEBROW		32
#define FAP_RAISE_R_M_EYEBROW		33
#define FAP_RAISE_L_O_EYEBROW		34
#define FAP_RAISE_R_O_EYEBROW		35
#define FAP_SQUEEZE_L_EYEBROW		36
#define FAP_SQUEEZE_R_EYEBROW		37

#define FAP_PUFF_L_CHEEK		38
#define FAP_PUFF_R_CHEEK		39
#define FAP_LIFT_L_CHEEK		40
#define FAP_LIFT_R_CHEEK		41

#define FAP_SHIFT_TONGUE_TIP		42
#define FAP_RAISE_TONGUE_TIP		43
#define FAP_THRUST_TONGUE_TIP		44
#define FAP_RAISE_TONGUE		45
#define FAP_TONGUE_ROLL			46

#define FAP_HEAD_PITCH			47
#define FAP_HEAD_YAW			48
#define FAP_HEAD_ROLL			49

#define FAP_LOWER_T_MIDLIP_O		50
#define FAP_RAISE_B_MIDLIP_O		51
#define FAP_STRETCH_L_CORNERLIP_O	52
#define FAP_STRETCH_R_CORNERLIP_O	53
#define FAP_LOWER_T_LIP_LM_O		54 
#define FAP_LOWER_T_LIP_RM_O		55 
#define FAP_RAISE_B_LIP_LM_O		56 
#define FAP_RAISE_B_LIP_RM_O		57 
#define FAP_RAISE_L_CORNERLIP_O		58
#define FAP_RAISE_R_CORNERLIP_O		59

#define FAP_STRETCH_L_NOSE		60
#define FAP_STRETCH_R_NOSE		61
#define FAP_RAISE_NOSE			62
#define FAP_BEND_NOSE			63

#define FAP_RAISE_L_EAR			64
#define FAP_RAISE_R_EAR			65
#define FAP_PULL_L_EAR			66
#define FAP_PULL_R_EAR			67

#define NB_FAP				68

/* FAP GROUP */
#define FAP_GROUP_VISEME_AND_EXPRESSION	0
#define FAP_GROUP_INNERLIPS_AND_JAW	1
#define FAP_GROUP_EYES			2
#define FAP_GROUP_EYEBROWS		3
#define FAP_GROUP_CHEEKS		4
#define FAP_GROUP_TONGUE		5
#define FAP_GROUP_HEAD			6
#define FAP_GROUP_OUTERLIPS		7
#define FAP_GROUP_NOSE			8
#define FAP_GROUP_EARS			9

/* FAP DIRECTION */
#define FAP_DIR_NOT_DEFINED		0
#define FAP_DIR_UP			1
#define FAP_DIR_RIGHT			2
#define FAP_DIR_DOWN			3
#define FAP_DIR_LEFT			4
#define FAP_DIR_FORWARD			5




/* INTERVAL */
#define INTERV_NOT_DEFINED         	0
#define INTERV_UNI			1	/* unidirectionnal */
#define INTERV_BI			2	/* bidirectionnal */

/* UNITS */
#define UNIT_NOT_DEFINED   		0
#define UNIT_ES    			1	/* eye to eye */
#define UNIT_ENS			2	/* nose to eye */
#define UNIT_MNS			3	/* mouth to nose */
#define UNIT_MW		       		4	/* mouth's width */
#define UNIT_IRISD		       	5	/* eye height */
#define UNIT_AU		       		6

/* COORDINATES */
#define END_COORD_INDEX      		-1
#define COORD_MOUTH_V_TOP      		0
#define COORD_MOUTH_V_RIGHT		1
#define COORD_MOUTH_V_BOTTOM		2
#define COORD_MOUTH_V_LEFT	       	3
#define COORD_MOUTH_H_RIGHT		4
#define COORD_MOUTH_H_LEFT	       	5
#define COORD_MOUTH_F		     	6
#define COORD_JAW_V		     	7
#define COORD_JAW_H			8
#define COORD_JAW_F			9
#define COORD_CHIN_F		     	10
#define COORD_CHEEK_V_LEFT	     	11
#define COORD_CHEEK_V_RIGHT	     	12
#define COORD_CHEEK_H_LEFT	     	13
#define COORD_CHEEK_H_RIGHT	     	14
#define COORD_EYELID_V_LEFT_TOP	     	15
#define COORD_EYELID_V_LEFT_BOTTOM     	16
#define COORD_EYELID_V_RIGHT_TOP     	17
#define COORD_EYELID_V_RIGHT_BOTTOM    	18
#define COORD_EYE_V_LEFT	     	19
#define COORD_EYE_V_RIGHT		20
#define COORD_EYE_H_LEFT		21
#define COORD_EYE_H_RIGHT	     	22
#define COORD_EYE_F_LEFT		23
#define COORD_EYE_F_RIGHT		24
#define COORD_PUPIL_LEFT       		25
#define COORD_PUPIL_RIGHT		26
#define COORD_EYEBROW_H_LEFT		27
#define COORD_EYEBROW_H_RIGHT		28
#define COORD_EYEBROW_V_LEFT_LEFT	29
#define COORD_EYEBROW_V_LEFT_MIDDLE	30
#define COORD_EYEBROW_V_LEFT_RIGHT	31
#define COORD_EYEBROW_V_RIGHT_LEFT	32
#define COORD_EYEBROW_V_RIGHT_MIDDLE	33
#define COORD_EYEBROW_V_RIGHT_RIGHT	34
#define COORD_NOSE_H_LEFT		35
#define COORD_NOSE_H_RIGHT		36
#define COORD_NOSE_TIP_H		37
#define COORD_NOSE_V			38
#define COORD_EAR_H_LEFT		39
#define COORD_EAR_H_RIGHT		40
#define COORD_EAR_V_LEFT		41
#define COORD_EAR_V_RIGHT		42
#define COORD_TONGUE_V			43
#define COORD_TONGUE_ROLL		44
#define COORD_TONGUE_TIP_H		45
#define COORD_TONGUE_TIP_V		46
#define COORD_TONGUE_TIP_F		47
#define COORD_HEAD_TURN       		48
#define COORD_HEAD_NOD			49
#define COORD_HEAD_ROLL			50

typedef struct {
  int	group;
  int	unit;
  int	interval;
  int	direction;
} Fap_def;

typedef struct {
  int	which;
  int	unit;
  float	min, middle, max;
} FCoord;

typedef struct {
  int	defined;
  int	interval;

  FCoord coords[10];      
} Mpa_def;


extern void init_faps(Fap_def*);
extern void init_mpas(Mpa_def*);
extern int read_mpa_ranges(char *);
extern int get_mpa_index(const char *);
extern int init_fap_converter(char *conv_filename);
extern int open_fap_file(char *filename);
extern void read_fap_frame(int *mask, float *faps);
extern void run_faps(int *mask, int *ifaps,SM_Surface_ptr the_surface,SM_Surface_ptr the_face,SM_Region_List_ptr the_region_list);




#endif /* _FAP_ */
