

/*------------------------------------------------------------------------------*/
/*										*/
/*										*/
/*  This software module was originally developed by                            */
/*        Prem Kalra, MIRALab, University of Geneva                             */
/*  and edited by                                                               */
/*        Igor Pandzic, MIRALab, University of Geneva                           */
/*        Marc Escher, MIRALab, University of Geneva                            */
/*        Pierre Beylot, MIRALab, University of Geneva                          */
/*  in the course of development of the MPEG-4 Video (ISO/IEC 14496-2).	        */
/*  This software module is an implementation of a part of one or more	        */
/*  MPEG-4 Video (ISO/IEC 14496-2) tools as specified by the MPEG-4 Video       */
/*  (ISO/IEC 14496-2). ISO/IEC gives users of the                               */
/*  MPEG-4 Video (ISO/IEC 14496-2) free license to this software module or	*/
/*  modifications thereof for use in hardware or software products claiming 	*/
/*  conformance to the MPEG-4 Video (ISO/IEC 14496-2). Those intending to use	*/
/*  this software module in hardware or software products are advised that	*/
/*  its use may infringe existing patents. The original developer of this	*/
/*  software module and his/her company, the subsequent editors and their	*/
/*  companies, and ISO/IEC have no liability for use of this software		*/
/*  module or modifications thereof. Copyright is not released for non		*/
/*  MPEG-4 Video (ISO/IEC 14496-2) conforming products.                         */
/*  MIRALab, University of Geneva retains full right to use the code for its    */
/*  own purpose, assign or donate the code to a third party and to inhibit      */
/*  third parties from using the code for non MPEG-4 Video (ISO/IEC 14496-2)    */
/*  conforming products. This copyright notice must be included in all copies   */
/*  or derivative works. Copyright (c) (1997).			      		*/
/*										*/
/*										*/
/*------------------------------------------------------------------------------*/

#ifndef lint
static char SccsId[] = "@(#)matrix_h.c    V1.3    7/5/92";
#endif
/*@#-----------------------------------------------------------------------
    FICHIER         : matrix_handling.c
    SYSTEME         : UNIX V 3.1C / SG 4D
    LANGAGE         : C
    -----------------------------------------------------------------------
    MODULE          : matrix_handling
    PROJET          :
    DIRECTEUR       : D. THALMANN
    UNITE           : EPFL/LIG
    -----------------------------------------------------------------------
    BUT             : Manipulation matrices homogenes 3D.
                      ATTENTION:
                       - Les coordonnees sont declarees du type FLOAT.
                       - Les cas d'erreurs ne sont pas traites
    MOT-CLES        : "vecteurs"
    PREFIXE         : MATRIX, MH3D
    -----------------------------------------------------------------------
    MODULES         : standards, standards_types
    BIBLIOTHEQUES   : math, stdio
    PORTABILITE     :
    -----------------------------------------------------------------------
    CREATION        : 06/07/89 Enrico Gobbetti
    -----------------------------------------------------------------------#@*/
# include "mooc.h"
# include "matrix_h.h"
# include <math.h>



/*@#-----------------------------------------------------------------------
    FONCTION        : MH3D_add
    BUT             : Addition de matrices homogenes
    MOT-CLES        : "Matrices"
    -----------------------------------------------------------------------
    OUT    res      : resultat
    IN     left     : 
    OUT    right    :
    VALEUR          : void.
    -----------------------------------------------------------------------#@*/

void
MH3D_add(res, left, right)
  MATRIX_H3D res;
  MATRIX_H3D left, right;
{
  register int i, j;

  for (i = X_INDEX; i <= W_INDEX; ++i) {
    for (j = X_INDEX; j<= W_INDEX; ++j) {
      res[i][j] = left[i][j] + right[i][j];
    }
  }  
} 



/*@#-----------------------------------------------------------------------
    FONCTION        : MH3D_sub
    BUT             : Soustraction de matrices homogenes
    MOT-CLES        : "Matrices"
    -----------------------------------------------------------------------
    OUT    res      : resultat
    IN     left     : 
    OUT    right    :
    VALEUR          : void.
    -----------------------------------------------------------------------#@*/

void
MH3D_sub(res, left, right)
  MATRIX_H3D res;
  MATRIX_H3D left, right;
{
  register int i, j;

  for (i = X_INDEX; i <= W_INDEX; ++i) {
    for (j = X_INDEX; j<= W_INDEX; ++j) {
      res[i][j] = left[i][j] - right[i][j];
    }
  }  
} 



/*@#-----------------------------------------------------------------------
    FONCTION        : MH3D_scale
    BUT             : Multiplication d'une matrice homogene par un scalaire
    MOT-CLES        : "Matrices"
    -----------------------------------------------------------------------
    OUT    res      : resultat
    IN     scalar   : 
    OUT    matrix   :
    VALEUR          : void.
    -----------------------------------------------------------------------#@*/

void
MH3D_scale(res, factor, matrix)
  MATRIX_H3D res;
  double      factor;
  MATRIX_H3D matrix;
{
  register int i, j;

  for (i = X_INDEX; i <= W_INDEX; ++i) {
    for (j = X_INDEX; j<= W_INDEX; ++j) {
      res[i][j] = factor * matrix[i][j];
    }
  }  
} 



/*@#-----------------------------------------------------------------------
    FONCTION        : MH3D_mul
    BUT             : Multiplication de matrices homogenes
    MOT-CLES        : "Matrices"
    -----------------------------------------------------------------------
    OUT    res      : resultat
    IN     left     : 
    OUT    right    :
    VALEUR          : void.
    -----------------------------------------------------------------------#@*/

void
MH3D_mul(res, left, right)
  MATRIX_H3D res;
  MATRIX_H3D left, right;
{
  register int i, j, k;
  MATRIX_H3D product;

  for (i = X_INDEX; i <= W_INDEX; ++i) {
    for (j = X_INDEX; j<= W_INDEX; ++j) {
      product[i][j] = 0.0;
      for (k = X_INDEX; k<= W_INDEX; ++k) {
	product[i][j] += left[i][k]*right[k][j];  
      }
    }
  }

  MH3D_COPY(res, product);
} 



/*@#-----------------------------------------------------------------------
    FONCTION        : MH3D_determinant
    BUT             : Calcul du determinant d'une matrice homogene
    MOT-CLES        : "Matrices"
    -----------------------------------------------------------------------
    IN     matrix   : 
    VALEUR          : determinant.
    -----------------------------------------------------------------------#@*/

float
MH3D_determinant(matrix)
  MATRIX_H3D matrix;
{

  float det;

  det =   
    matrix[X_INDEX][X_INDEX] * 
    ( matrix[Y_INDEX][Y_INDEX] * 
      matrix[Z_INDEX][Z_INDEX] * 
      matrix[W_INDEX][W_INDEX]
    + matrix[Y_INDEX][Z_INDEX] * 
      matrix[Z_INDEX][W_INDEX] * 
      matrix[W_INDEX][Y_INDEX]
    + matrix[Y_INDEX][W_INDEX] * 
      matrix[Z_INDEX][Y_INDEX] * 
      matrix[W_INDEX][Z_INDEX]
    - matrix[W_INDEX][Y_INDEX] * 
      matrix[Z_INDEX][Z_INDEX] * 
      matrix[Y_INDEX][W_INDEX]
    - matrix[W_INDEX][Z_INDEX] * 
      matrix[Z_INDEX][W_INDEX] * 
      matrix[Y_INDEX][Y_INDEX]
    - matrix[W_INDEX][W_INDEX] *
      matrix[Z_INDEX][Y_INDEX] *
      matrix[Y_INDEX][Z_INDEX] ) -
    matrix[X_INDEX][Y_INDEX] * 
    ( matrix[Y_INDEX][X_INDEX] *
      matrix[Z_INDEX][Z_INDEX] *
      matrix[W_INDEX][W_INDEX]
    + matrix[Y_INDEX][Z_INDEX] *
      matrix[Z_INDEX][W_INDEX] *
      matrix[W_INDEX][X_INDEX]
    + matrix[Y_INDEX][W_INDEX] *
      matrix[Z_INDEX][X_INDEX] *
      matrix[W_INDEX][Z_INDEX]
    - matrix[W_INDEX][X_INDEX] *
      matrix[Z_INDEX][Z_INDEX] *
      matrix[Y_INDEX][W_INDEX]
    - matrix[W_INDEX][Z_INDEX] *
      matrix[Z_INDEX][W_INDEX] *
      matrix[Y_INDEX][X_INDEX]
    - matrix[W_INDEX][W_INDEX] *
      matrix[Z_INDEX][X_INDEX] *
      matrix[Y_INDEX][Z_INDEX] ) +
    matrix[X_INDEX][Z_INDEX] * 
    ( matrix[Y_INDEX][X_INDEX] *
      matrix[Z_INDEX][Y_INDEX] *
      matrix[W_INDEX][W_INDEX]
    + matrix[Y_INDEX][Y_INDEX] *
      matrix[Z_INDEX][W_INDEX] *
      matrix[W_INDEX][X_INDEX]
    + matrix[Y_INDEX][W_INDEX] *
      matrix[Z_INDEX][X_INDEX] *
      matrix[W_INDEX][Y_INDEX]
    - matrix[W_INDEX][X_INDEX] *
      matrix[Z_INDEX][Y_INDEX] *
      matrix[Y_INDEX][W_INDEX]
    - matrix[W_INDEX][Y_INDEX] *
      matrix[Z_INDEX][W_INDEX] *
      matrix[Y_INDEX][X_INDEX]
    - matrix[W_INDEX][W_INDEX] *
      matrix[Z_INDEX][X_INDEX] *
      matrix[Y_INDEX][Y_INDEX] ) -
    matrix[X_INDEX][W_INDEX] * 
    ( matrix[Y_INDEX][X_INDEX] *
      matrix[Z_INDEX][Y_INDEX] *
      matrix[W_INDEX][Z_INDEX]
    + matrix[Y_INDEX][Y_INDEX] *
      matrix[Z_INDEX][Z_INDEX] *
      matrix[W_INDEX][X_INDEX]
    + matrix[Y_INDEX][Z_INDEX] *
      matrix[Z_INDEX][X_INDEX] *
      matrix[W_INDEX][Y_INDEX]
    - matrix[W_INDEX][X_INDEX] *
      matrix[Z_INDEX][Y_INDEX] *
      matrix[Y_INDEX][Z_INDEX]
    - matrix[W_INDEX][Y_INDEX] * 
      matrix[Z_INDEX][Z_INDEX] *
      matrix[Y_INDEX][X_INDEX]
    - matrix[W_INDEX][Z_INDEX] *
      matrix[Z_INDEX][X_INDEX] *
      matrix[Y_INDEX][Y_INDEX] );

  return det;  
}



/*@#-----------------------------------------------------------------------
    FONCTION        : MH3D_cofactor
    BUT             : Calcul d'un cofacteur d'une matrice homogene
    MOT-CLES        : "Matrices"
    -----------------------------------------------------------------------
    IN     matrix   :
    IN     i, j     : 
    VALEUR          : cofacteur.
    -----------------------------------------------------------------------#@*/

float
MH3D_cofactor(matrix, i, j)
  MATRIX_H3D matrix;
  int        i, j;
{
  MATRIX_H3D      mij;
  int             i1, j1, i2, j2;
  static float    factorsign[] = { 1.0, -1.0, 1.0, -1.0 };
      
  MH3D_IDENTITY(mij);

  i1 = X_INDEX;
  for (i2 = X_INDEX; i2 < W_INDEX; ++i2) {
    if (i1 == i) ++i1;

    j1 = X_INDEX;
    for (j2 = X_INDEX; j2 < W_INDEX; ++j2) {
      if (j1 == j) ++j1;

      mij[i2][j2] = matrix[i1][j1];
      ++j1;
    }
    ++i1;   
  }

  return (factorsign[i] * factorsign[j] * MH3D_determinant(mij)); 
} 

    

/*@#-----------------------------------------------------------------------
    FONCTION        : MH3D_invert
    BUT             : Calcul de l'inverse d'une matrice homogene
    MOT-CLES        : "Matrices"
    -----------------------------------------------------------------------
    OUT    res      : Matrice inversee.
    IN   determinant: Determinant de la matrice a inverser.
    IN     source   : Matrice a inverser.
    VALEUR          : void.
    -----------------------------------------------------------------------#@*/
  /*
  ** -- Attention: la matrice est supposee inversible!
  */

void
MH3D_invert(res, determinant, source)
  MATRIX_H3D res;
  double      determinant;
  MATRIX_H3D source;
{
  MATRIX_H3D   inv;
  int          i, j;
  float        scale;

  scale = 1.0 / determinant;

  for (i = X_INDEX; i <= W_INDEX; ++i) {
    for (j = X_INDEX; j <= W_INDEX; ++j) {
      inv[i][j] = MH3D_cofactor(source, j, i) * scale;
    }
  }

  MH3D_COPY(res, inv);

}


/*@#-----------------------------------------------------------------------
    FONCTION        : MH3D_transpose
    BUT             : Calcul de la transposee d'une matrice homogene
    MOT-CLES        : "Matrices"
    -----------------------------------------------------------------------
    OUT    res      :
    IN     source   : 
    VALEUR          : void.
    -----------------------------------------------------------------------#@*/

void
MH3D_transpose(res, source)
  MATRIX_H3D res;
  MATRIX_H3D source;
{
  MATRIX_H3D trans;
  int        i, j;
  
  for (i = X_INDEX; i <= W_INDEX; ++i) {
    for (j = X_INDEX; j <= W_INDEX; ++j) {
      trans[j][i] = source[i][j]; 
    } 
  }

  MH3D_COPY(res, trans);
}



/*@#-----------------------------------------------------------------------
    FONCTION        : MH3D_transform_point_H3D
    BUT             : 
    MOT-CLES        : "Matrices"
    -----------------------------------------------------------------------
    OUT    res      :
    IN     matrix   : 
    IN     vector   :
    VALEUR          : void.
    -----------------------------------------------------------------------#@*/

void
MH3D_transform_point_H3D(res, matrix, vector)
  VECTOR_H3D res;
  MATRIX_H3D matrix;
  VECTOR_H3D vector;
{
  VECTOR_H3D localv;

  localv[X_INDEX] = matrix[X_INDEX][X_INDEX] * vector[X_INDEX] +
                   matrix[Y_INDEX][X_INDEX] * vector[Y_INDEX] +
                   matrix[Z_INDEX][X_INDEX] * vector[Z_INDEX] +
                   matrix[W_INDEX][X_INDEX] * vector[W_INDEX];

  localv[Y_INDEX] = matrix[X_INDEX][Y_INDEX] * vector[X_INDEX] +
                   matrix[Y_INDEX][Y_INDEX] * vector[Y_INDEX] +
                   matrix[Z_INDEX][Y_INDEX] * vector[Z_INDEX] +
                   matrix[W_INDEX][Y_INDEX] * vector[W_INDEX];

  localv[Z_INDEX] = matrix[X_INDEX][Z_INDEX] * vector[X_INDEX] +
                   matrix[Y_INDEX][Z_INDEX] * vector[Y_INDEX] +
                   matrix[Z_INDEX][Z_INDEX] * vector[Z_INDEX] +
                   matrix[W_INDEX][Z_INDEX] * vector[W_INDEX];

  localv[W_INDEX] = matrix[X_INDEX][W_INDEX] * vector[X_INDEX] +
                   matrix[Y_INDEX][W_INDEX] * vector[Y_INDEX] +
                   matrix[Z_INDEX][W_INDEX] * vector[Z_INDEX] +
                   matrix[W_INDEX][W_INDEX] * vector[W_INDEX];

  VH3D_COPY(res, localv);
}



/*@#-----------------------------------------------------------------------
    FONCTION        : MH3D_transform_vector_H3D
    BUT             : 
    MOT-CLES        : "Matrices"
    -----------------------------------------------------------------------
    OUT    res      :
    IN     matrix   : 
    IN     vector   :
    VALEUR          : void.
    -----------------------------------------------------------------------#@*/

void
MH3D_transform_vector_H3D(res, matrix, vector)
  VECTOR_H3D res;
  MATRIX_H3D matrix;
  VECTOR_H3D vector;
{
  VECTOR_H3D localv;

  localv[X_INDEX] = matrix[X_INDEX][X_INDEX] * vector[X_INDEX] +
                   matrix[Y_INDEX][X_INDEX] * vector[Y_INDEX] +
                   matrix[Z_INDEX][X_INDEX] * vector[Z_INDEX] +
                                              vector[W_INDEX];

  localv[Y_INDEX] = matrix[X_INDEX][Y_INDEX] * vector[X_INDEX] +
                   matrix[Y_INDEX][Y_INDEX] * vector[Y_INDEX] +
                   matrix[Z_INDEX][Y_INDEX] * vector[Z_INDEX] +
                                              vector[W_INDEX];

  localv[Z_INDEX] = matrix[X_INDEX][Z_INDEX] * vector[X_INDEX] +
                   matrix[Y_INDEX][Z_INDEX] * vector[Y_INDEX] +
                   matrix[Z_INDEX][Z_INDEX] * vector[Z_INDEX] +
                                              vector[W_INDEX];

  localv[W_INDEX] =                            vector[W_INDEX];

  VH3D_COPY(res, localv);
}



/*@#-----------------------------------------------------------------------
    FONCTION        : MH3D_transform_point_3D
    BUT             : 
    MOT-CLES        : "Matrices"
    -----------------------------------------------------------------------
    OUT    res      :
    IN     matrix   : 
    IN     vector   :
    VALEUR          : void.
    -----------------------------------------------------------------------#@*/

void
MH3D_transform_point_3D(res, matrix, vector)
  VECTOR_3D  res;
  MATRIX_H3D matrix;
  VECTOR_3D  vector;
{
  VECTOR_H3D localv;

  VH3D_FROM_V3D(localv, vector);
  MH3D_transform_point_H3D(localv, matrix, localv);
  V3D_FROM_VH3D(res, localv); 
}



/*@#-----------------------------------------------------------------------
    FONCTION        : MH3D_transform_vector_3D
    BUT             : 
    MOT-CLES        : "Matrices"
    -----------------------------------------------------------------------
    OUT    res      :
    IN     matrix   : 
    IN     vector   :
    VALEUR          : void.
    -----------------------------------------------------------------------#@*/

void
MH3D_transform_vector_3D(res, matrix, vector)
  VECTOR_3D  res;
  MATRIX_H3D matrix;
  VECTOR_3D  vector;
{
  VECTOR_H3D localv;

  VH3D_FROM_V3D(localv, vector);
  MH3D_transform_vector_H3D(localv, matrix, localv);
  V3D_FROM_VH3D(res, localv); 
}

#define EPSILON 1e-4

void
MH3D_from_axis_angle(mout, rv)
  MATRIX_H3D mout;
  VECTOR_3D  rv;
{
   VECTOR_3D k;
   float ang;                  /* scalar angle of rotation */
   float c, s, v;              /* cosine, sine, versine of ang */

   MH3D_IDENTITY(mout);

   ang = V3D_NORM(rv);         /* these equations are set up for cw rot. */
   if (ang < EPSILON)
      return;

   V3D_NORMALIZE(k, rv);
   c = cos( ang );             /* in right-handed, ccw rot in left-handed */
   s = sin( ang );
   v = 1.0 - c;                /* versine(x) = 1 - cos(x) */

   mout[0][0] = k[0] * k[0] * v + c;
   mout[0][1] = k[0] * k[1] * v + k[2] * s;
   mout[0][2] = k[0] * k[2] * v - k[1] * s;

   mout[1][0] = k[1] * k[0] * v - k[2] * s;
   mout[1][1] = k[1] * k[1] * v + c;
   mout[1][2] = k[1] * k[2] * v + k[0] * s;

   mout[2][0] = k[2] * k[0] * v + k[1] * s;
   mout[2][1] = k[2] * k[1] * v - k[0] * s;
   mout[2][2] = k[2] * k[2] * v + c;
}



void
MH3D_to_axis_angle(axis, aMatrix)
  VECTOR_3D  axis;
  MATRIX_H3D aMatrix;

{
   float angle, norm, ww, xx, yy, four_w, two_x;
   VECTOR_H3D  value;

   ww = 0.25 * ( 1.0 + aMatrix[X_INDEX][X_INDEX] +
                       aMatrix[Y_INDEX][Y_INDEX] +
                       aMatrix[Z_INDEX][Z_INDEX]);

   if (ww > EPSILON)
      {
      value[W_INDEX] = sqrt(ww);
      four_w  = 4.0 * value[W_INDEX];
      value[X_INDEX] =
          (aMatrix[Y_INDEX][Z_INDEX] - aMatrix[Z_INDEX][Y_INDEX]) / four_w;

      value[Y_INDEX] =
          (aMatrix[Z_INDEX][X_INDEX] - aMatrix[X_INDEX][Z_INDEX]) / four_w;

      value[Z_INDEX] =
          (aMatrix[X_INDEX][Y_INDEX] - aMatrix[Y_INDEX][X_INDEX]) / four_w;
      }
   else
      {
      value[W_INDEX] = 0.0;
      xx = -0.5 * (aMatrix[Y_INDEX][Y_INDEX] + aMatrix[Z_INDEX][Z_INDEX]);

      if (xx > EPSILON)
         {
         value[X_INDEX] = sqrt(xx);
         two_x = value[X_INDEX] * 2.0;
         value[Y_INDEX] = aMatrix[X_INDEX][Y_INDEX] / two_x;
         value[Z_INDEX] = aMatrix[X_INDEX][Z_INDEX] / two_x;
         }
      else
         {
         value[X_INDEX] = 0.0;
         yy = 0.5 * (1.0 - aMatrix[Z_INDEX][Z_INDEX]);

         if (yy > EPSILON)
            {
            value[Y_INDEX] = sqrt(yy);
            value[Z_INDEX] = aMatrix[Y_INDEX][Z_INDEX] /
                                   (2.0 * value[Y_INDEX]);
            }
         else
            {
            value[Y_INDEX] = 0.0;
            value[Z_INDEX] = 1.0;
            }
         }
      }


   /* normalize the quaternion */
   norm = sqrt(
      value[X_INDEX] * value[X_INDEX] +
      value[Y_INDEX] * value[Y_INDEX] +
      value[Z_INDEX] * value[Z_INDEX] +
      value[W_INDEX] * value[W_INDEX]);

   if (norm > EPSILON)
      {
      value[X_INDEX] /= norm;
      value[Y_INDEX] /= norm;
      value[Z_INDEX] /= norm;
      value[W_INDEX] /= norm;
      }
   else
      {
      value[X_INDEX] = 0.0;
      value[Y_INDEX] = 0.0;
      value[Z_INDEX] = 0.0;
      value[W_INDEX] = 1.0;
      }

   /* calculate angle */
   angle = 2.0 * acos(value[W_INDEX]);

   axis[X_INDEX] = value[X_INDEX];
   axis[Y_INDEX] = value[Y_INDEX];
   axis[Z_INDEX] = value[Z_INDEX];

   norm = V3D_NORM(axis);
   if (norm > EPSILON)
      {
      V3D_SCALE(axis, angle/norm, axis);
      }
   else
      {
      V3D_INIT(axis, angle, 0.0, 0.0);
      }
}
