
/*------------------------------------------------------------------------------*/
/*										*/
/*										*/
/*  This software module was originally developed by                            */
/*        Prem Kalra, MIRALab, University of Geneva                             */
/*  and edited by                                                               */
/*        Igor Pandzic, MIRALab, University of Geneva                           */
/*        Marc Escher, MIRALab, University of Geneva                            */
/*        Pierre Beylot, MIRALab, University of Geneva                          */
/*  in the course of development of the MPEG-4 Video (ISO/IEC 14496-2).	        */
/*  This software module is an implementation of a part of one or more	        */
/*  MPEG-4 Video (ISO/IEC 14496-2) tools as specified by the MPEG-4 Video       */
/*  (ISO/IEC 14496-2). ISO/IEC gives users of the                               */
/*  MPEG-4 Video (ISO/IEC 14496-2) free license to this software module or	*/
/*  modifications thereof for use in hardware or software products claiming 	*/
/*  conformance to the MPEG-4 Video (ISO/IEC 14496-2). Those intending to use	*/
/*  this software module in hardware or software products are advised that	*/
/*  its use may infringe existing patents. The original developer of this	*/
/*  software module and his/her company, the subsequent editors and their	*/
/*  companies, and ISO/IEC have no liability for use of this software		*/
/*  module or modifications thereof. Copyright is not released for non		*/
/*  MPEG-4 Video (ISO/IEC 14496-2) conforming products.                         */
/*  MIRALab, University of Geneva retains full right to use the code for its    */
/*  own purpose, assign or donate the code to a third party and to inhibit      */
/*  third parties from using the code for non MPEG-4 Video (ISO/IEC 14496-2)    */
/*  conforming products. This copyright notice must be included in all copies   */
/*  or derivative works. Copyright (c) (1997).			      		*/
/*										*/
/*										*/
/*------------------------------------------------------------------------------*/


# ifndef MODULE_MATRIX_HANDLING
# define MODULE_MATRIX_HANDLING

# include "vector_h.h"
# include <math.h>
# include <stdio.h>

/*
** ------------------------------------
** -- Declaration des types exportes --
** ------------------------------------
*/

typedef float MATRIX_H3D[1 + W_INDEX][1 + W_INDEX];

/*
** ----------------------------
** -- Macros de manipulation --
** ----------------------------
*/

# define MH3D_DUMP(FILE, M) \
  {\
    VH3D_DUMP(FILE, M[X_INDEX]); \
    VH3D_DUMP(FILE, M[Y_INDEX]); \
    VH3D_DUMP(FILE, M[Z_INDEX]); \
    VH3D_DUMP(FILE, M[W_INDEX]); \
  }

# define MH3D_INIT(RES, XX, XY, XZ, XW, YX, YY, YZ, YW, ZX, ZY, ZZ, ZW, WX, WY, WZ, WW)\
    {\
      RES[X_INDEX][X_INDEX] = XX; \
      RES[X_INDEX][Y_INDEX] = XY; \
      RES[X_INDEX][Z_INDEX] = XZ; \
      RES[X_INDEX][W_INDEX] = XW; \
     \
      RES[Y_INDEX][X_INDEX] = YX; \
      RES[Y_INDEX][Y_INDEX] = YY; \
      RES[Y_INDEX][Z_INDEX] = YZ; \
      RES[Y_INDEX][W_INDEX] = YW; \
     \
      RES[Z_INDEX][X_INDEX] = ZX; \
      RES[Z_INDEX][Y_INDEX] = ZY; \
      RES[Z_INDEX][Z_INDEX] = ZZ; \
      RES[Z_INDEX][W_INDEX] = ZW; \
     \
      RES[W_INDEX][X_INDEX] = WX; \
      RES[W_INDEX][Y_INDEX] = WY; \
      RES[W_INDEX][Z_INDEX] = WZ; \
      RES[W_INDEX][W_INDEX] = WW; \
    }


# define MH3D_COPY(RES, SOURCE) \
    {\
      MH3D_INIT(RES, SOURCE[X_INDEX][X_INDEX], \
                     SOURCE[X_INDEX][Y_INDEX], \
                     SOURCE[X_INDEX][Z_INDEX], \
                     SOURCE[X_INDEX][W_INDEX], \
                    \
                     SOURCE[Y_INDEX][X_INDEX], \
                     SOURCE[Y_INDEX][Y_INDEX], \
                     SOURCE[Y_INDEX][Z_INDEX], \
                     SOURCE[Y_INDEX][W_INDEX], \
                    \
                     SOURCE[Z_INDEX][X_INDEX], \
                     SOURCE[Z_INDEX][Y_INDEX], \
                     SOURCE[Z_INDEX][Z_INDEX], \
                     SOURCE[Z_INDEX][W_INDEX], \
                    \
                     SOURCE[W_INDEX][X_INDEX], \
                     SOURCE[W_INDEX][Y_INDEX], \
                     SOURCE[W_INDEX][Z_INDEX], \
                     SOURCE[W_INDEX][W_INDEX]  \
      );\
    }


# define MH3D_NULL(RES) \
    {\
      MH3D_INIT(RES, 0.0, 0.0, 0.0, 0.0, \
                     0.0, 0.0, 0.0, 0.0, \
                     0.0, 0.0, 0.0, 0.0, \
                     0.0, 0.0, 0.0, 0.0);\
    }


# define MH3D_IDENTITY(RES) \
    {\
      MH3D_INIT(RES, 1.0, 0.0, 0.0, 0.0, \
                     0.0, 1.0, 0.0, 0.0, \
                     0.0, 0.0, 1.0, 0.0, \
                     0.0, 0.0, 0.0, 1.0);\
    }

# define MH3D_EQUAL(M1, M2) \
	(\
	M1[X_INDEX][X_INDEX] == M2[X_INDEX][X_INDEX] &&\
	M1[X_INDEX][Y_INDEX] == M2[X_INDEX][Y_INDEX] &&\
	M1[X_INDEX][Z_INDEX] == M2[X_INDEX][Z_INDEX] &&\
	M1[X_INDEX][W_INDEX] == M2[X_INDEX][W_INDEX] &&\
	\
	M1[Y_INDEX][X_INDEX] == M2[Y_INDEX][X_INDEX] &&\
	M1[Y_INDEX][Y_INDEX] == M2[Y_INDEX][Y_INDEX] &&\
	M1[Y_INDEX][Z_INDEX] == M2[Y_INDEX][Z_INDEX] &&\
	M1[Y_INDEX][W_INDEX] == M2[Y_INDEX][W_INDEX] &&\
	\
	M1[Z_INDEX][X_INDEX] == M2[Z_INDEX][X_INDEX] &&\
	M1[Z_INDEX][Y_INDEX] == M2[Z_INDEX][Y_INDEX] &&\
	M1[Z_INDEX][Z_INDEX] == M2[Z_INDEX][Z_INDEX] &&\
	M1[Z_INDEX][W_INDEX] == M2[Z_INDEX][W_INDEX] &&\
	\
	M1[W_INDEX][X_INDEX] == M2[W_INDEX][X_INDEX] &&\
	M1[W_INDEX][Y_INDEX] == M2[W_INDEX][Y_INDEX] &&\
	M1[W_INDEX][Z_INDEX] == M2[W_INDEX][Z_INDEX] &&\
	M1[W_INDEX][W_INDEX] == M2[W_INDEX][W_INDEX] &&\
	)

#define MH3D_TRANSLATE(RES, VEC) \
{ \
   MH3D_IDENTITY(RES); \
   RES[W_INDEX][X_INDEX] = VEC[X_INDEX]; \
   RES[W_INDEX][Y_INDEX] = VEC[Y_INDEX]; \
   RES[W_INDEX][Z_INDEX] = VEC[Z_INDEX]; \
}

#define MH3D_ROTATE(RES, THETA, AXIS) \
{ \
   MH3D_IDENTITY(RES); \
   RES[(AXIS+1) % 3][(AXIS+1) % 3] = cos(THETA); \
   RES[(AXIS+1) % 3][(AXIS+2) % 3] = sin(THETA); \
   RES[(AXIS+2) % 3][(AXIS+2) % 3] = cos(THETA); \
   RES[(AXIS+2) % 3][(AXIS+1) % 3] = -sin(THETA); \
}

#define MH3D_ROTPART(RES, A) \
{ \
   MH3D_COPY(RES, A); \
   RES[W_INDEX][X_INDEX] = 0.0; \
   RES[W_INDEX][Y_INDEX] = 0.0; \
   RES[W_INDEX][Z_INDEX] = 0.0; \
}

#define MH3D_TRANSPART(RES, A) \
{ \
   MH3D_IDENTITY(RES); \
   RES[W_INDEX][X_INDEX] = A[W_INDEX][0]; \
   RES[W_INDEX][Y_INDEX] = A[W_INDEX][Y_INDEX]; \
   RES[W_INDEX][Z_INDEX] = A[W_INDEX][Z_INDEX]; \
}



extern void 
MH3D_add(MATRIX_H3D, MATRIX_H3D, MATRIX_H3D);

extern void 
MH3D_sub(MATRIX_H3D, MATRIX_H3D, MATRIX_H3D);

extern void 
MH3D_scale(MATRIX_H3D, double, MATRIX_H3D);

extern void 
MH3D_mul(MATRIX_H3D, MATRIX_H3D, MATRIX_H3D);

extern float
MH3D_determinant(MATRIX_H3D);

extern float
MH3D_cofactor(MATRIX_H3D, int, int);

extern void
MH3D_invert(MATRIX_H3D, double, MATRIX_H3D);

extern void 
MH3D_transpose(MATRIX_H3D, MATRIX_H3D);

extern void
MH3D_transform_point_H3D(VECTOR_H3D, MATRIX_H3D, VECTOR_H3D);

extern void
MH3D_transform_vector_H3D(VECTOR_H3D, MATRIX_H3D, VECTOR_H3D);

extern void
MH3D_transform_point_3D(VECTOR_3D, MATRIX_H3D, VECTOR_3D);

extern void
MH3D_transform_vector_3D(VECTOR_3D, MATRIX_H3D, VECTOR_3D);

# endif
