

/*------------------------------------------------------------------------------*/
/*										*/
/*										*/
/*  This software module was originally developed by                            */
/*        Prem Kalra, MIRALab, University of Geneva                             */
/*  and edited by                                                               */
/*        Igor Pandzic, MIRALab, University of Geneva                           */
/*        Marc Escher, MIRALab, University of Geneva                            */
/*        Pierre Beylot, MIRALab, University of Geneva                          */
/*  in the course of development of the MPEG-4 Video (ISO/IEC 14496-2).	        */
/*  This software module is an implementation of a part of one or more	        */
/*  MPEG-4 Video (ISO/IEC 14496-2) tools as specified by the MPEG-4 Video       */
/*  (ISO/IEC 14496-2). ISO/IEC gives users of the                               */
/*  MPEG-4 Video (ISO/IEC 14496-2) free license to this software module or	*/
/*  modifications thereof for use in hardware or software products claiming 	*/
/*  conformance to the MPEG-4 Video (ISO/IEC 14496-2). Those intending to use	*/
/*  this software module in hardware or software products are advised that	*/
/*  its use may infringe existing patents. The original developer of this	*/
/*  software module and his/her company, the subsequent editors and their	*/
/*  companies, and ISO/IEC have no liability for use of this software		*/
/*  module or modifications thereof. Copyright is not released for non		*/
/*  MPEG-4 Video (ISO/IEC 14496-2) conforming products.                         */
/*  MIRALab, University of Geneva retains full right to use the code for its    */
/*  own purpose, assign or donate the code to a third party and to inhibit      */
/*  third parties from using the code for non MPEG-4 Video (ISO/IEC 14496-2)    */
/*  conforming products. This copyright notice must be included in all copies   */
/*  or derivative works. Copyright (c) (1997).			      		*/
/*										*/
/*										*/
/*------------------------------------------------------------------------------*/

char *sm_version = "Face 1.01";


#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include <signal.h>
#ifdef unix
#include <sys/time.h>
#endif

#include "SurfMan.h"
#include "fap_dec.h"



#include "script.h"
#include "face_interface.h"
#include "script_interface.h"
#include "SurfMan.h"


static char *WriteFileHeader(FILE *file, char* descr)
{
  char *file_name = strdup(descr);
  char *init_name = strtok(descr,"/");
  char *name, *buf_mem;

#ifdef unix
  struct timeval s1;
  struct timezone s2;

  gettimeofday(&s1,&s2);
#endif

  buf_mem = strtok(NULL,"/");
  if (buf_mem == NULL)
     name = init_name;
  else
    while(buf_mem != NULL)
      {
	name = buf_mem;
	buf_mem = strtok(NULL,"/");
      }

  name = strtok(name,".");

  fprintf(file,"#VRML V1.0 ascii\n\n");

  fprintf(file,"# ------------------------------------------------\n");
  fprintf(file,"# ---          SmToVRML1 Convertor v1.0         ---\n");
  fprintf(file,"# ---            CopyRight LIG/EPFL            ---\n");
  fprintf(file,"# --- Sm File Converted :\n#     %s\n",file_name);
#ifdef unix
  fprintf(file,"# --- Conversion Time :\n#     %s",asctime(localtime(&(s1.tv_sec))));
#endif
  fprintf(file,"# ------------------------------------------------\n\n");
  
  fprintf(file," Separator { # --- Shape : %s ---\n",name);
  fprintf(file,"   renderCulling ON\n");
  fprintf(file,"   ShapeHints {\n");
  fprintf(file,"      vertexOrdering COUNTERCLOCKWISE\n");
  fprintf(file,"      shapeType      SOLID\n");
  fprintf(file,"   }\n");

  return name;
}


void writewrl(the_surface,the_face,the_region_list)
SM_Surface_ptr the_surface, the_face;
SM_Region_List_ptr the_region_list;
{
  FILE *file = NULL;
  static int fcount = -1;
  char name[100];

  SM_Triangle_ptr triangle_ptr = NULL;
  SM_Mat_ptr      material_ptr = NULL;
  VECTOR_3D vect3D;
  
  float r,g,b,shininess;

  unsigned short *index_vert,*normal_index;
  int i, j, index;
  int material_index, triangle_index; 
  int num_tr, num_pt;

  fcount++;
  sprintf(name,"out%d.debug.vrml",fcount);

  printf("Writing %s\n",name);

  if ((file = fopen(name,"w")) == NULL)
    {
      fprintf(stderr,"VRML1 Output File %s could not be opened - exiting...\n",name);
      exit(1);
    }
  WriteFileHeader(file,name);
  
  for (material_index = 0, material_ptr = the_materials.mat_list;
       material_index < the_materials.mat_list_size;
       material_index++, material_ptr++ ) 
    {
      if (! material_ptr->struct_used)
	continue;
      
      fprintf(file,"   Separator { # --- for %s region\n",material_ptr->name);

      /* --- Initialize */
      num_pt = 0;
      num_tr = 0;

      /* --- Materiel Def. */
      fprintf(file,"               Material {\n");
      SM_material_rgb_get(material_ptr->name, DIFFUSE,&r,&g,&b);
      fprintf(file,"                  diffuseColor %f %f %f\n",r,g,b);
      
      SM_material_rgb_get(material_ptr->name, AMBIENT,&r,&g,&b);
      fprintf(file,"                  ambientColor %f %f %f\n",r,g,b);
      
      SM_material_rgb_get(material_ptr->name, SPECULAR,&r,&g,&b);
      fprintf(file,"                  specularColor %f %f %f\n",r,g,b);
      
      fprintf(file,"                  emissiveColor 0 0 0\n");
      
      SM_material_value_get(material_ptr->name, SHININESS,&shininess);
      fprintf(file,"                  shininess %f\n",shininess);
      
      fprintf(file,"               } # End of Material\n"); /* End Of Material */
      
      index = 0;
      index_vert = (unsigned short *)malloc(3*the_surface->current_tri_list_size*sizeof(unsigned short));
      normal_index = (unsigned short *)malloc(3*the_surface->current_tri_list_size*sizeof(unsigned short));
      
      for (triangle_index = 0, triangle_ptr = the_surface->tri_list;
	   triangle_index < the_surface->current_tri_list_size;
	   triangle_index++, triangle_ptr++) 
	if (triangle_ptr->struct_used) 
	  {
	    /* test on triangle material */
	    if(triangle_ptr->material_index == material_index) 
	      {
		num_tr++;
		
		index_vert[index]   = triangle_ptr->vertex1; 
		index_vert[index+1] = triangle_ptr->vertex2;
		index_vert[index+2] = triangle_ptr->vertex3;
		
		normal_index[index] = triangle_ptr->normal1;
		normal_index[index+1] = triangle_ptr->normal2;
		normal_index[index+2] = triangle_ptr->normal3;
		
		index+=3;
	      }
	  }
      
      if ( index != 0 )
	{
	  fprintf(file,"         Coordinate3 {\n");
	  fprintf(file,"            point [ ");
	  
	  j = 0;
	  for(i=0;i<index;i+=3)
	    {
	      SM_vertex_point_get(the_surface,index_vert[i],vect3D);
	      fprintf(file,"            %f %f %f,\n",
		      vect3D[0],
		      vect3D[1],
		      vect3D[2]);
	      
	      SM_vertex_point_get(the_surface,index_vert[i+1],vect3D);
	      fprintf(file,"            %f %f %f,\n",
		      vect3D[0],
		      vect3D[1],
		      vect3D[2]);
	      
	      SM_vertex_point_get(the_surface,index_vert[i+2],vect3D);
	      fprintf(file,"            %f %f %f",
		      vect3D[0],
		      vect3D[1],
		      vect3D[2]);
	      
	      if (i+3 < index)
		fprintf(file,",\n");
	    }
	  fprintf(file,"           ] # --- End of Point\n");
	  
	  fprintf(file,"         } # --- End of Coordinate\n");
	  
	  /* Normal */
	  fprintf(file,"      Normal {\n");
	  fprintf(file,"         vector [ ");
	  
	  for(i=0;i<index;i+=3)
	    {
	      SM_vertex_normal_get(the_surface,index_vert[i],normal_index[i],vect3D);	      
	      fprintf(file,"            %f %f %f,\n",
		      vect3D[0],
		      vect3D[1],
		      vect3D[2]);
	      
	      SM_vertex_normal_get(the_surface,index_vert[i+1],normal_index[i+1],vect3D);	      
	      fprintf(file,"            %f %f %f,\n",
		      vect3D[0],
		      vect3D[1],
		      vect3D[2]);
	      
	      SM_vertex_normal_get(the_surface,index_vert[i+2],normal_index[i+2],vect3D);	      
	      fprintf(file,"            %f %f %f",
		      vect3D[0],
		      vect3D[1],
		      vect3D[2]);
	      
	      if (i+3 < index)
		fprintf(file,",\n");
	    }
	  
	  fprintf(file,"           ] # --- End of Vector\n");
	  fprintf(file,"      } # --- End of Normal\n");
	  
	  /* Normal Index */
	  fprintf(file,"      NormalBinding{ \n");
	  fprintf(file,"         value PER_VERTEX_INDEXED\n");
	  fprintf(file,"      }\n");
	  
	  /* Coord Index */
	  fprintf(file,"      IndexedFaceSet {\n");
	  fprintf(file,"      coordIndex [ ");
	  
	  for(i=0;i<index;i+=3)
	    {
	      fprintf(file,"      %d, %d, %d, -1",i,i+1,i+2);
	      
	      if (i+3 < index)
		fprintf(file,",\n");
	    }
	  
	  fprintf(file,"           ] # --- End of coordIndex\n");
	  
	  /* Normal Index */
	  fprintf(file,"      normalIndex [ ");
	  
	  for(i=0;i<index;i+=3)
	    {
	      fprintf(file,"      %d, %d, %d, -1",i,i+1,i+2);
	      
	      if (i+3 < index)
		fprintf(file,",\n");
	    }
	  fprintf(file,"           ] # --- End Of normalIndex\n");
	  
	  
	  fprintf(file,"      } # --- End Of IndexedFaceSet\n");
	}
      
      fprintf(file,"     } # --- End Of Group for %s region\n",material_ptr->name);
      
    } /* End of Material */
  
  free(index_vert);
  free(normal_index);
  
  fprintf(file," } # --- End Of Separator for %s\n",name);
  
  fflush(file);
  fclose(file);
}



void
Face_initialize_parameter(SM_Surface_ptr,SM_Surface_ptr,SM_Region_List_ptr);

void
Face_initialize_parameter(SM_Surface_ptr the_surface,SM_Surface_ptr the_face,SM_Region_List_ptr the_region_list)
{
    Face_set_intiale_parameters();
    Face_put_actor_maxmin_values();
    Face_set_maxmin_limits();
    AU_neutral_face_set();
    AU_head_center_set(the_surface, the_region_list);
}

void
Face_operation_neutralize(SM_Surface_ptr the_surface,SM_Surface_ptr the_face,SM_Region_List_ptr the_region_list)
{
  SM_Surface_ptr temp_surface;

  if (temp_surface = SM_surface_read("mirastdface.sm")) {
    SM_surface_dispose(the_surface);
    the_surface = temp_surface;
    SM_surface_region_list_set(the_surface, the_region_list);
  }
  
  Face_initialize_parameter(the_surface,the_face,the_region_list);
}

void
run_mpas(int *mask, float *mpas,SM_Surface_ptr the_surface,SM_Surface_ptr the_face,SM_Region_List_ptr the_region_list)
{
  
  int i;
  int mpa_no;
  MPA_Name mpa_name;
  static float previous_mpas[NB_MPA];
  static MPA_Name_List_ptr mpa_name_list;
  static int init = 1;
  SM_Surface_ptr *the_surface_p,*the_face_p;
  SM_Region_List_ptr *the_region_list_p;
  
  if(init)
  {
    init = 0;
    for(i = 0; i < NB_MPA; i++)
      previous_mpas[i] = 0.0;
    
    mpa_name_list = (MPA_Name_List_ptr) calloc(1,sizeof(MPA_Name_List));
    mpa_name_list->size = SCRIPT_TOTAL_MPAS;
    
    MPA_script_mpalist_define(mpa_name_list);
  }
  for (i = 0; i < NB_MPA; i++) {
    if (mask[i] && mpas[i] != previous_mpas[i]) {
      strcpy(mpa_name, tab_mpa_str[i]);
      if (!MPA_script_mpa_id_get(mpa_name, &mpa_no, mpa_name_list)) {
        fprintf(stderr, "the mpa %s is not right \n", tab_mpa_str[i]);
      }
      
      Script_apply_mpa(mpa_no, mpas[i],the_surface,the_face,the_region_list);
      previous_mpas[i] = mpas[i];
    }
  }
  
#ifdef DRAW
  viewer_redraw_all(the_surface);
#endif
#ifdef DUMP_VRML
  writewrl(the_surface,the_face,the_region_list);
#endif

}


void init_face(the_surface_p,the_face_p,the_region_list_p)
SM_Surface_ptr *the_surface_p, *the_face_p;
SM_Region_List_ptr *the_region_list_p;
{
    SM_Surface_ptr the_surface, the_face;
    SM_Region_List_ptr the_region_list;
    
    SM_Surface_ptr temp_surface;

    the_surface = SM_surface_new();

    the_region_list = SM_region_list_new(the_surface);    
    Face_initialize_parameter(the_surface,the_face,the_region_list);  
                

    if (temp_surface = SM_surface_read("mirastdface.sm")) {
      SM_surface_dispose(the_surface);
      the_surface = temp_surface;
      SM_surface_region_list_set(the_surface, the_region_list);
    }

    if (the_face = SM_surface_read("mirastdface.sm")) {
      SM_surface_region_list_set(the_face, the_region_list);
      SM_region_list_read(the_region_list,the_surface,"mirastdface.rg");	    
    }

    if (!init_fap_converter("mirastdconv.txt")) 
      exit(1);

#ifdef DRAW
    viewer_open(the_surface);
    viewer_redraw_all(the_surface);
#endif

    (*the_surface_p) = the_surface;
    (*the_face_p) = the_face;
    (*the_region_list_p) = the_region_list;

#if 0
    /* This part of the code reads the fap file format and is now
       not used */
    nb_frame = open_fap_file(argv[1]);
    if (nb_frame == -1) exit(2);

    for (i = 0; i < nb_frame; i++) {
      read_fap_frame(mask, faps);
      run_faps(mask, faps);
    }
#endif
}

