
/*------------------------------------------------------------------------------*/
/*										*/
/*										*/
/*  This software module was originally developed by                            */
/*        Prem Kalra, MIRALab, University of Geneva                             */
/*  and edited by                                                               */
/*        Igor Pandzic, MIRALab, University of Geneva                           */
/*        Marc Escher, MIRALab, University of Geneva                            */
/*        Pierre Beylot, MIRALab, University of Geneva                          */
/*  in the course of development of the MPEG-4 Video (ISO/IEC 14496-2).	        */
/*  This software module is an implementation of a part of one or more	        */
/*  MPEG-4 Video (ISO/IEC 14496-2) tools as specified by the MPEG-4 Video       */
/*  (ISO/IEC 14496-2). ISO/IEC gives users of the                               */
/*  MPEG-4 Video (ISO/IEC 14496-2) free license to this software module or	*/
/*  modifications thereof for use in hardware or software products claiming 	*/
/*  conformance to the MPEG-4 Video (ISO/IEC 14496-2). Those intending to use	*/
/*  this software module in hardware or software products are advised that	*/
/*  its use may infringe existing patents. The original developer of this	*/
/*  software module and his/her company, the subsequent editors and their	*/
/*  companies, and ISO/IEC have no liability for use of this software		*/
/*  module or modifications thereof. Copyright is not released for non		*/
/*  MPEG-4 Video (ISO/IEC 14496-2) conforming products.                         */
/*  MIRALab, University of Geneva retains full right to use the code for its    */
/*  own purpose, assign or donate the code to a third party and to inhibit      */
/*  third parties from using the code for non MPEG-4 Video (ISO/IEC 14496-2)    */
/*  conforming products. This copyright notice must be included in all copies   */
/*  or derivative works. Copyright (c) (1997).			      		*/
/*										*/
/*										*/
/*------------------------------------------------------------------------------*/

#include "parser.h"
#include "math.h"


#define NB_SET_DELIMIT 1
char SetDelimit[NB_SET_DELIMIT][2]={'"','"'};










/*
=========================  Boolean token type tests  ==========================
*/

static BOOL
is_begin_list(c)
char c;
{
    return( c == '(' || c == '{' || c == '[' || c == '<' );
}

static BOOL
is_end_list(int);

static BOOL
is_end_list(c)
char c;
{
    return( c == ')' || c == '}' || c == ']' || c == '>' );
}

static BOOL
is_separator(c)
char c;
{
    return( c == ' ' || c == ',' );
}

static int
is_set(lead_char)
char lead_char;
{
    int i;

    for (i = 0; i < NB_SET_DELIMIT; i++) 
	if (SetDelimit[i][0] == lead_char) return(i + 1);
    return(0);
}

static BOOL
endset(lead_char, end_char)
char lead_char, end_char;
{
    int i;

    if (i = is_set(lead_char))
	return( SetDelimit[i-1][1] == end_char );
    return(NO);
}

static BOOL
is_alpha(c)
char c;
{
    return( c >= 'a' && c <= 'z' || c >= 'A' && c <= 'Z' );
}


static BOOL
is_digit(c)
char c;
{
    return( c >= '0' && c <= '9' );
}


static BOOL
is_num(c)
char c;
{
    return( is_digit(c) || c == '-' || c == '.' );
}

static BOOL
parser_end_token(lead_char, c)
char lead_char, c;
{
    if (is_alpha(lead_char) && (is_alpha(c) || is_digit(c) || c == '_')) return(NO);
    if (is_num(lead_char) && (is_num(c))) return(NO);
    if (is_set(lead_char)) 
	if (! endset(lead_char,c)) return(NO);
    return(YES);
}











/*
=============================  Token generation  =============================
*/

BOOL
parser_next_token(fil, s, max_length)
Parser_File *fil;
char s[];
int max_length;
/*
**	Retourne le prochain token du fichier fil->unix_file sous forme 
**	de chaine de caracteres dans s[max_length].
**	Tokens possibles:
**		identificateur: Alpha + AlphaNumerique
**		numerique: Numerique
**		caractere special: autrement.
**		set: "..."  (sans le dernier char)
**	Retourne YES si la fin du fichier pas rencontree.  Retourne NO
**	autrement avec possibilite d'avoir qqch dans s.
**	NOTE: max_length doit etre >= 2, pour contenir au moins un char.
*/
{
    char lead_char;
    int c, i;
    FILE *unix_file;

    if (max_length >= 2) {
	unix_file = fil->unix_file;
	while ( is_separator( (char) (c = getc(unix_file)) ) );  

   	lead_char = c;
	if (lead_char != (char)EOF) {
	    i = 0;
 	    s[i++] = lead_char;
	    while ( (c = getc(unix_file)) != EOF && 
		    !parser_end_token(lead_char, c) ) {
		s[i] = c;
		if (i < max_length - 1) i++;
	    }
	    s[i] = '\0';
	    if (!is_set(lead_char)) ungetc(c,unix_file);

	    /*
	    **	Gestion automatique de niveaux de listes.
	    */
	    if (is_begin_list(s[0])) fil->list_level_count++;
	    else if (is_end_list(s[0])) fil->list_level_count--;
	    if (c != EOF) return(YES);
	} else s[0] = '\0';
    }
    fil->end_of_file = YES;
    return(NO);
}













/*
============================   Type conversions   ============================
*/

static double 
char_to_double(s)
char s[];
{
    double val=0.0,neg=1.0;
    int dec=0;

    while (*s) {
	if (*s >= '0'  &&  *s <= '9') {
		val = val * 10.0 + (double) (*s) - (double) '0';
		if (dec) dec++;
	}
	else if (*s == '.') dec = 1;
	else if (*s == '-') neg = -neg;
	s++;
    }
    if (dec) return( neg * val/pow(10.0,(double) (dec-1)) );
    else return(neg * val);
}











/*
============================  List manipulations  ===========================
*/

BOOL
parser_end_of_file(fil)
Parser_File *fil;
{
    return( fil->end_of_file );
}



BOOL
parser_end_of_list(fil, list_level)
Parser_File *fil;
Parser_List_Level list_level;
{
    return( list_level > fil->list_level_count );
}



BOOL
parser_correct_list(fil, list_level)
Parser_File *fil;
Parser_List_Level list_level;
/*
**  Returns YES if list_level is the current list level in fil.
*/
{
    return( list_level == fil->list_level_count );
}





/*
============================  Typed data reading  ===========================
*/

BOOL
parser_read_identifier(fil, list_level, return_value)
Parser_File *fil;
Parser_List_Level list_level;
char * return_value;
/*
**  Asks to read an identifier from the list_level wanted.  
**  If list_level is superior to the current file's, *return_value is an empty string.
**  If list_level is inferior, scans the file to list_level and reads.
**  Returns YES if a identifier value was there to be read.
*/
{
    static char s[100];
    BOOL ident_found = NO;

    return_value[0] = '\0';
    while (!parser_end_of_list(fil, list_level) && !ident_found) {
	parser_next_token(fil, s, 100);
	if (parser_correct_list(fil, list_level) && is_alpha(s[0])) {
	    ident_found = YES;
	    strcpy(return_value, s);
	}
    }
    return(ident_found);
}
    



BOOL
parser_read_string(fil, list_level, return_value)
Parser_File *fil;
Parser_List_Level list_level;
char * return_value;
/*
**  Asks to read a string from the list_level wanted.  
**  If list_level is superior to the current file's, *return_value is an empty string.
**  If list_level is inferior, scans the file to list_level and reads.
**  Returns YES if a new string value was there to be read.
*/
{
    static char s[100];
    BOOL ident_found = NO;

    return_value[0] = '\0';
    while (!parser_end_of_list(fil, list_level) && !ident_found) {
	parser_next_token(fil, s, 100);
	if (parser_correct_list(fil, list_level) && is_set(s[0])) {
	    ident_found = YES;

	    /*
	    **	Remove first ".
	    */
	    strcpy(return_value, &s[1]);
	}
    }
    return(ident_found);
}
    



BOOL
parser_read_float(fil, list_level, return_value)
Parser_File *fil;
Parser_List_Level list_level;
float *return_value;
/*
**  Asks to read a float from the list_level wanted.  
**  If list_level is superior to the current file's, *return_value 0.0.
**  If list_level is inferior, scans the file to list_level and reads.
**  Returns YES if a new float value was there to be read.
*/
{
    static char s[100];
    BOOL float_found = NO;
    
    *return_value = 0.0;
    while ( !parser_end_of_list(fil, list_level) && !float_found ) {
	parser_next_token(fil, s, 100);
	if (parser_correct_list(fil, list_level) && is_num(s[0]) ) {
	    float_found = YES;	
	    *return_value = (float) char_to_double(s);
	}
    }
    return( float_found );
}



BOOL
parser_read_int(fil, list_level, return_value)
Parser_File *fil;
Parser_List_Level list_level;
int *return_value;
/*
**  Asks to read an int from the list_level wanted.  
**  If list_level is superior to the current file's, *return_value 0.
**  If list_level is inferior, scans the file to list_level and reads.
**  Returns YES if a new int value was there to be read.
*/
{
    float float_value;

    BOOL ok = parser_read_float(fil, list_level, &float_value);
    *return_value = (int) float_value;
    return(ok);
}


void
parser_bgn_list(fil, list_level)
Parser_File *fil;
Parser_List_Level *list_level;
/*
**  Advances the fil->unix_file pointer till a new, superior list
**  level is encountered.
*/
{
    static char s[2];
    Parser_List_Level last_level;

    last_level = fil->list_level_count;
    while( !parser_end_of_list(fil, last_level) && 
	   !parser_correct_list(fil, last_level + 1)) {
	parser_next_token(fil, s, 2);
    }
    *list_level = last_level + 1;
}



void
parser_end_list(fil, list_level)
Parser_File *fil;
Parser_List_Level list_level;
/*
**  Advances the fil->unix_file pointer till the "parser_end_of_list" is
**  encountered.
*/
{
    static char s[2];

    while (!parser_end_of_list(fil, list_level))
	parser_next_token(fil, s, 2);
}



Parser_File *
parser_open_file(name)
char *name;
{
    Parser_File *fil;
    FILE *unix_fil;

    if (unix_fil = fopen(name,"r")) {
	fil = (Parser_File *) malloc(sizeof(Parser_File));
	fil->unix_file = unix_fil;
	fil->end_of_file = NO;
	fil->list_level_count = 0;
	return(fil);
    }
    else return(NULL);
}



void
parser_close_file(fil)
Parser_File *fil;
{
    (void) fclose(fil->unix_file);
    (void) free(fil);
}



