

/*------------------------------------------------------------------------------*/
/*										*/
/*										*/
/*  This software module was originally developed by                            */
/*        Prem Kalra, MIRALab, University of Geneva                             */
/*  and edited by                                                               */
/*        Igor Pandzic, MIRALab, University of Geneva                           */
/*        Marc Escher, MIRALab, University of Geneva                            */
/*        Pierre Beylot, MIRALab, University of Geneva                          */
/*  in the course of development of the MPEG-4 Video (ISO/IEC 14496-2).	        */
/*  This software module is an implementation of a part of one or more	        */
/*  MPEG-4 Video (ISO/IEC 14496-2) tools as specified by the MPEG-4 Video       */
/*  (ISO/IEC 14496-2). ISO/IEC gives users of the                               */
/*  MPEG-4 Video (ISO/IEC 14496-2) free license to this software module or	*/
/*  modifications thereof for use in hardware or software products claiming 	*/
/*  conformance to the MPEG-4 Video (ISO/IEC 14496-2). Those intending to use	*/
/*  this software module in hardware or software products are advised that	*/
/*  its use may infringe existing patents. The original developer of this	*/
/*  software module and his/her company, the subsequent editors and their	*/
/*  companies, and ISO/IEC have no liability for use of this software		*/
/*  module or modifications thereof. Copyright is not released for non		*/
/*  MPEG-4 Video (ISO/IEC 14496-2) conforming products.                         */
/*  MIRALab, University of Geneva retains full right to use the code for its    */
/*  own purpose, assign or donate the code to a third party and to inhibit      */
/*  third parties from using the code for non MPEG-4 Video (ISO/IEC 14496-2)    */
/*  conforming products. This copyright notice must be included in all copies   */
/*  or derivative works. Copyright (c) (1997).			      		*/
/*										*/
/*										*/
/*------------------------------------------------------------------------------*/

#include "script.h"

MPA_Script_ptr
MPA_script_new()
/*
** Allocates the memory for the new list of script
*/
{
   MPA_Script_ptr the_script;

   the_script = (MPA_Script_ptr) calloc(1,sizeof(MPA_Script));
   the_script->script_list = NULL;
   the_script->script_list_size = 0;
   return(the_script);
}


void
MPA_script_dispose(the_script)
MPA_Script_ptr the_script;
/*
** Disposes the memory for the script
*/
{
/*
#ifdef OLD_VERSION

   Frame_ptr the_frame;

   MPA_FOR_EACH_FRAME(the_script)
    the_frame = &the_script->script_list[frame_index];
    if (the_frame->MPA_list_size > 0) {
       the_frame->MPA_list_size = 0;
       free(the_frame->MPA_list); 
     }
     free(the_frame);
   MPA_END_FOR_EACH
   
   free(the_script);


#endif
*/


   Frame_ptr the_frame;

   MPA_FOR_EACH_FRAME(the_script)
    the_frame = &the_script->script_list[frame_index];
    if (the_frame->MPA_list_size > 0) {
       the_frame->MPA_list_size = 0;
       free(the_frame->MPA_list); 
     }
   MPA_END_FOR_EACH
   
   free(the_script->script_list);
   free(the_script);


}


BOOL 
MPA_script_frame_exists(the_script,f_index,frame)
MPA_Script_ptr the_script;
int f_index;
float frame;
{
   Frame_ptr the_frame;
   
   the_frame = &the_script->script_list[f_index];
  
   if (the_frame->frame_no == frame) return(YES);
   else return(NO);

}

void
MPA_script_add_mpa(the_script,frame_index,mpa,intensity,name) 
MPA_Script_ptr the_script;
int frame_index, mpa;
float intensity;
MPA_Name name;
/* 
** Add mpa to a frame
*/
{
MPA_ptr new_mpa, ptr1, ptr2, new_mpa_list;
Frame_ptr the_frame;
int new_mpa_index,i;

   the_frame = &the_script->script_list[frame_index];
   new_mpa_index = 0;
   while (new_mpa_index < the_frame->MPA_list_size)
    new_mpa_index++;
   new_mpa_list = (MPA_ptr) 
                  calloc((the_frame->MPA_list_size + 1), sizeof(MPA));

   if (new_mpa_index != 0) {
     ptr1 = the_frame->MPA_list;
     ptr2 = new_mpa_list;
     for (i=1; i <= the_frame->MPA_list_size; i++) 
      *ptr2++ = *ptr1++;
     free(the_frame->MPA_list);
   }
   the_frame->MPA_list_size += 1;
   the_frame->MPA_list = new_mpa_list;
  
   new_mpa = &the_frame->MPA_list[new_mpa_index];
   new_mpa->MPA_no = mpa;
   new_mpa->MPA_intensity = intensity;
   strcpy(new_mpa->MPA_name,name);
   new_mpa->MPA_id = new_mpa_index;
   new_mpa->MPA_used = YES;

}

void
MPA_script_delete_mpa(the_script,frame_index,mpa_index) 
MPA_Script_ptr the_script;
int frame_index, mpa_index;
/* 
** Delete mpa from a frame
*/
{
MPA_ptr ptr1, ptr2, new_mpa_list, the_mpa;
Frame_ptr the_frame;
int i;

   the_frame = &the_script->script_list[frame_index];

   new_mpa_list = (MPA_ptr) 
                  calloc((the_frame->MPA_list_size - 1), sizeof(MPA));

   the_mpa = &the_frame->MPA_list[mpa_index];
   the_mpa->MPA_used = NO; 
   ptr1 = the_frame->MPA_list;
   ptr2 = new_mpa_list;
   for (i=1; i <= the_frame->MPA_list_size; i++) {
    if (ptr1->MPA_used) {
      *ptr2++ = *ptr1++;
    } else 
       ptr1++;
   }

   free(the_frame->MPA_list);
 
   the_frame->MPA_list_size -= 1;
   the_frame->MPA_list = new_mpa_list;

   ptr2 = new_mpa_list;
   for (i=0; i<the_frame->MPA_list_size; i++) {
    ptr2->MPA_id = i;
    ptr2++;
   }

}


void
MPA_script_delete_notused_mpa(the_script,frame_index)
MPA_Script_ptr the_script;
int frame_index;
/* 
** Delete mpa from a frame
*/
{
MPA_ptr ptr1, ptr2, new_mpa_list;
Frame_ptr the_frame;
int i,n;

   the_frame = &the_script->script_list[frame_index];

   ptr1 = the_frame->MPA_list;
   n = 0;
   for (i=1; i <= the_frame->MPA_list_size; i++) {
     if (ptr1->MPA_used) 
       n++;
     ptr1++;
   }

   new_mpa_list = (MPA_ptr) 
                  calloc(n, sizeof(MPA));

   ptr1 = the_frame->MPA_list;
   ptr2 = new_mpa_list;
   for (i=1; i <= the_frame->MPA_list_size; i++) {
    if (ptr1->MPA_used) {
      *ptr2++ = *ptr1++;
    } else 
       ptr1++;
   }

   free(the_frame->MPA_list);
 
   the_frame->MPA_list_size = n;
   the_frame->MPA_list = new_mpa_list;

   ptr2 = new_mpa_list;
   for (i=0; i<the_frame->MPA_list_size; i++) {
    ptr2->MPA_id = i;
    ptr2++;
   }

}



void
MPA_script_add_frame(the_script,mpa,frame,intensity,name)
MPA_Script_ptr the_script;
int mpa;
float frame, intensity;
MPA_Name name;
/*
** Add a frame to the script
*/
{
Frame_ptr new_script_list, ptr1, ptr2, the_frame;
int new_frame_index, i;
MPA_ptr mpa_list;
float *frame_array, temp;
int *index_array, temp_index, j, n;

   new_frame_index = 0;
   while (new_frame_index < the_script->script_list_size)
     new_frame_index++;
   new_script_list = (Frame_ptr) calloc((the_script->script_list_size + 1),
                                         sizeof(Frame));

   if (new_frame_index != 0) {
     ptr1 = the_script->script_list;
     ptr2 = new_script_list;
     for (i=1; i <= the_script->script_list_size; i++)
      *ptr2++ = *ptr1++;
     free(the_script->script_list);
   }

   the_script->script_list_size += 1;
   the_script->script_list = new_script_list;
   the_frame = &the_script->script_list[new_frame_index];

   mpa_list = (MPA_ptr) calloc(1, sizeof(MPA));
   mpa_list->MPA_no = mpa;
   mpa_list->MPA_intensity = intensity;
   mpa_list->MPA_id = 0;
   mpa_list->MPA_used = YES;
   strcpy(mpa_list->MPA_name,name);
   the_frame->MPA_list = mpa_list;   
   the_frame->MPA_list_size = 1;   
   the_frame->frame_no = frame;
   the_frame->frame_id = new_frame_index;
   the_frame->executed = NO;
     

   frame_array = (float *) calloc(the_script->script_list_size,sizeof(float));
   index_array = (int *) calloc(the_script->script_list_size,sizeof(int));

   i = 0;
   MPA_FOR_EACH_FRAME(the_script)
     frame_array[i] = frame_ptr->frame_no;
     index_array[i] = frame_ptr->frame_id;
     i++;
   MPA_END_FOR_EACH 


   temp_index = index_array[0];
   temp = frame_array[0];
   n = the_script->script_list_size;
   for (i=0; i<n-1; i++) {
     for (j=i+1; j<n; j++) {
       if (frame_array[i] > frame_array[j]) {
         temp = frame_array[i];
         frame_array[i] = frame_array[j];
         frame_array[j] = temp;
         temp_index = index_array[i];
         index_array[i] = index_array[j];
         index_array[j] = temp_index;
       }
     }
   }

   new_script_list = (Frame_ptr) calloc((the_script->script_list_size),
                                         sizeof(Frame));

   ptr2 = new_script_list;
   
   for (i=0; i < the_script->script_list_size; i++) {
     temp_index = index_array[i];
     ptr1 = &the_script->script_list[temp_index];
     *ptr2++ = *ptr1;
   }
   free(the_script->script_list);
   the_script->script_list = new_script_list;
   for (i=0; i < the_script->script_list_size; i++) { 
     the_frame = &the_script->script_list[i];
     the_frame->frame_id = i;
   }

   free(frame_array);
   free(index_array);
}

void
MPA_script_add_entry(the_script,mpa,frame,intensity,name) 
MPA_Script_ptr the_script;
int mpa;
float frame, intensity;
MPA_Name name;
/*
** Add an entry to the script list
*/
{
  BOOL add_mpa;
  int index;

   add_mpa = NO;
   MPA_FOR_EACH_FRAME(the_script)
     if (MPA_script_frame_exists(the_script,frame_index,frame)) {
       add_mpa = YES;
       index = frame_index;
     }
   MPA_END_FOR_EACH

   if (add_mpa) 
     MPA_script_add_mpa(the_script,index,mpa,intensity,name); 
   else 
     MPA_script_add_frame(the_script,mpa,frame,intensity,name);
}

void
MPA_script_mpalist_define(mpa_name_list)
MPA_Name_List_ptr mpa_name_list;

{

    strcpy(mpa_name_list->list[0].name,"move_h_l_eyeball");
    mpa_name_list->list[0].id = 101;
    strcpy(mpa_name_list->list[1].name,"move_h_r_eyeball");
    mpa_name_list->list[1].id = 102;
    strcpy(mpa_name_list->list[2].name,"move_h_eyeballs");
    mpa_name_list->list[2].id = 103;
    strcpy(mpa_name_list->list[3].name,"move_v_l_eyeball");
    mpa_name_list->list[3].id = 111;
    strcpy(mpa_name_list->list[4].name,"move_v_r_eyeball");
    mpa_name_list->list[4].id = 112;
    strcpy(mpa_name_list->list[5].name,"move_v_eyeballs");
    mpa_name_list->list[5].id = 113;
    strcpy(mpa_name_list->list[6].name,"close_upper_l_eyelid");
    mpa_name_list->list[6].id = 121;
    strcpy(mpa_name_list->list[7].name,"close_upper_r_eyelid");
    mpa_name_list->list[7].id = 122;
    strcpy(mpa_name_list->list[8].name,"close_upper_eyelids");
    mpa_name_list->list[8].id = 123;
    strcpy(mpa_name_list->list[9].name,"close_lower_l_eyelid");
    mpa_name_list->list[9].id = 131;
    strcpy(mpa_name_list->list[10].name,"close_lower_r_eyelid");
    mpa_name_list->list[10].id = 132;
    strcpy(mpa_name_list->list[11].name,"close_lower_eyelids");
    mpa_name_list->list[11].id = 133;
    strcpy(mpa_name_list->list[12].name,"raise_l_eyebrow");
    mpa_name_list->list[12].id = 141;
    strcpy(mpa_name_list->list[13].name,"raise_r_eyebrow");
    mpa_name_list->list[13].id = 142;
    strcpy(mpa_name_list->list[14].name,"raise_eyebrows");
    mpa_name_list->list[14].id = 143;
    strcpy(mpa_name_list->list[15].name,"squeeze_l_eyebrow");
    mpa_name_list->list[15].id = 151;
    strcpy(mpa_name_list->list[16].name,"squeeze_r_eyebrow");
    mpa_name_list->list[16].id = 152;
    strcpy(mpa_name_list->list[17].name,"squeeze_eyebrows");
    mpa_name_list->list[17].id = 153;
    strcpy(mpa_name_list->list[18].name,"open_jaw");
    mpa_name_list->list[18].id = 201;
    strcpy(mpa_name_list->list[19].name,"raise_l_cornerlip");
    mpa_name_list->list[19].id = 301;
    strcpy(mpa_name_list->list[20].name,"raise_r_cornerlip");
    mpa_name_list->list[20].id = 302;
    strcpy(mpa_name_list->list[21].name,"raise_cornerlips");
    mpa_name_list->list[21].id = 303;
    strcpy(mpa_name_list->list[22].name,"puff_l_cheek");
    mpa_name_list->list[22].id = 311;
    strcpy(mpa_name_list->list[23].name,"puff_r_cheek");   
    mpa_name_list->list[23].id = 312;
    strcpy(mpa_name_list->list[24].name,"puff_cheeks");
    mpa_name_list->list[24].id = 313;
    strcpy(mpa_name_list->list[25].name,"raise_upperlip");
    mpa_name_list->list[25].id = 401;
    strcpy(mpa_name_list->list[26].name,"lower_lowerlip");
    mpa_name_list->list[26].id = 402;
    strcpy(mpa_name_list->list[27].name,"raise_u_midlip");
    mpa_name_list->list[27].id = 411;
    strcpy(mpa_name_list->list[28].name,"raise_l_midlip");
    mpa_name_list->list[28].id = 412;
    strcpy(mpa_name_list->list[29].name,"raise_midlips");
    mpa_name_list->list[29].id = 413;
    strcpy(mpa_name_list->list[30].name,"pull_midlips");
    mpa_name_list->list[30].id = 421;
    strcpy(mpa_name_list->list[31].name,"stretch_cornerlips");
    mpa_name_list->list[31].id = 431;
    strcpy(mpa_name_list->list[32].name,"stretch_nose");
    mpa_name_list->list[32].id = 501;
    strcpy(mpa_name_list->list[33].name,"raise_nose");
    mpa_name_list->list[33].id = 511;
    strcpy(mpa_name_list->list[34].name,"turn_head");
    mpa_name_list->list[34].id = 601;
    strcpy(mpa_name_list->list[35].name,"nod_head");
    mpa_name_list->list[35].id = 611;
    strcpy(mpa_name_list->list[36].name,"roll_head");
    mpa_name_list->list[36].id = 621;

    strcpy(mpa_name_list->list[37].name,"move_hori_jaw");
    mpa_name_list->list[37].id = 202;

    strcpy(mpa_name_list->list[38].name,"move_o_l_eyeball");
    mpa_name_list->list[38].id = 161;
    strcpy(mpa_name_list->list[39].name,"move_o_r_eyeball");
    mpa_name_list->list[39].id = 162;
    strcpy(mpa_name_list->list[40].name,"move_o_eyeballs");
    mpa_name_list->list[40].id = 163;

    strcpy(mpa_name_list->list[41].name,"raise_l_eyebrow_l");
    mpa_name_list->list[41].id = 1441;
    strcpy(mpa_name_list->list[42].name,"raise_r_eyebrow_l");
    mpa_name_list->list[42].id = 1442;
    strcpy(mpa_name_list->list[43].name,"raise_eyebrows_l");
    mpa_name_list->list[43].id = 1443;

    strcpy(mpa_name_list->list[44].name,"raise_l_eyebrow_m");
    mpa_name_list->list[44].id = 1451;
    strcpy(mpa_name_list->list[45].name,"raise_r_eyebrow_m");
    mpa_name_list->list[45].id = 1452;
    strcpy(mpa_name_list->list[46].name,"raise_eyebrows_m");
    mpa_name_list->list[46].id = 1453;

    strcpy(mpa_name_list->list[47].name,"raise_l_eyebrow_r");
    mpa_name_list->list[47].id = 1461;
    strcpy(mpa_name_list->list[48].name,"raise_r_eyebrow_r");
    mpa_name_list->list[48].id = 1462;
    strcpy(mpa_name_list->list[49].name,"raise_eyebrows_r");
    mpa_name_list->list[49].id = 1463;

    strcpy(mpa_name_list->list[50].name,"lift_l_cheek");
    mpa_name_list->list[50].id = 321;
    strcpy(mpa_name_list->list[51].name,"lift_r_cheek");
    mpa_name_list->list[51].id = 322;
    strcpy(mpa_name_list->list[52].name,"lift_cheeks");
    mpa_name_list->list[52].id = 323;

    strcpy(mpa_name_list->list[53].name,"lower_l_cornerlip");
    mpa_name_list->list[53].id = 331;
    strcpy(mpa_name_list->list[54].name,"lower_r_cornerlip");
    mpa_name_list->list[54].id = 332;
    strcpy(mpa_name_list->list[55].name,"lower_cornerlips");
    mpa_name_list->list[55].id = 333;

    strcpy(mpa_name_list->list[56].name,"depress_chin");
    mpa_name_list->list[56].id = 203;

    strcpy(mpa_name_list->list[57].name,"suck_lips");
    mpa_name_list->list[57].id = 441;
    strcpy(mpa_name_list->list[58].name,"squeeze_cornerlips");
    mpa_name_list->list[58].id = 451;
    strcpy(mpa_name_list->list[59].name,"squeeze_l_cornerlip");
    mpa_name_list->list[59].id = 4511;
    strcpy(mpa_name_list->list[60].name,"squeeze_r_cornerlip");
    mpa_name_list->list[60].id = 4512;

    strcpy(mpa_name_list->list[61].name,"stretch_l_cornerlip");
    mpa_name_list->list[61].id = 4311;
    strcpy(mpa_name_list->list[62].name,"stretch_r_cornerlip");
    mpa_name_list->list[62].id = 4312;
   
    strcpy(mpa_name_list->list[63].name,"tongue_lift");
    mpa_name_list->list[63].id = 701;
    strcpy(mpa_name_list->list[64].name,"tongue_pan");
    mpa_name_list->list[64].id = 702;
    strcpy(mpa_name_list->list[65].name,"tongue_out");
    mpa_name_list->list[65].id = 703;
    strcpy(mpa_name_list->list[66].name,"tongue_lift_tip");
    mpa_name_list->list[66].id = 704;
    strcpy(mpa_name_list->list[67].name,"tongue_roll");
    mpa_name_list->list[67].id = 705;

    strcpy(mpa_name_list->list[68].name,"jaw_thrust");
    mpa_name_list->list[68].id = 204;

    strcpy(mpa_name_list->list[69].name,"eye_dilate_l_pupil");
    mpa_name_list->list[69].id = 171;
    strcpy(mpa_name_list->list[70].name,"eye_dilate_r_pupil");
    mpa_name_list->list[70].id = 172;
    strcpy(mpa_name_list->list[71].name,"eye_dilate_pupils");
    mpa_name_list->list[71].id = 173;

    strcpy(mpa_name_list->list[72].name,"nose_bend");
    mpa_name_list->list[72].id = 521;
    strcpy(mpa_name_list->list[73].name,"stretch_l_nose");
    mpa_name_list->list[73].id = 502;
    strcpy(mpa_name_list->list[74].name,"stretch_r_nose");
    mpa_name_list->list[74].id = 503;

    strcpy(mpa_name_list->list[75].name,"raise_l_ear");
    mpa_name_list->list[75].id = 801;
    strcpy(mpa_name_list->list[76].name,"raise_r_ear");
    mpa_name_list->list[76].id = 802;
    strcpy(mpa_name_list->list[77].name,"raise_ears");
    mpa_name_list->list[77].id = 803;
    strcpy(mpa_name_list->list[78].name,"pull_l_ear");
    mpa_name_list->list[78].id = 811;
    strcpy(mpa_name_list->list[79].name,"pull_r_ear");
    mpa_name_list->list[79].id = 812;
    strcpy(mpa_name_list->list[80].name,"pull_ears");
    mpa_name_list->list[80].id = 813;

}

BOOL
MPA_script_mpa_id_get(mpa_name,mp,mpa_name_list)
MPA_Name mpa_name;
int *mp;
MPA_Name_List_ptr mpa_name_list;
{
  int i;

   for (i=0; i<mpa_name_list->size; i++) {
    if (strcmp(mpa_name,mpa_name_list->list[i].name) == 0) {
      *mp = mpa_name_list->list[i].id;
      return(YES);
    }
   }
   return(NO);
}

BOOL
MPA_script_read(the_script,filename,mpa_name_list,script_mpa_list)
MPA_Script_ptr the_script;
char *filename;
MPA_Name_List_ptr mpa_name_list;
Script_Mpa_List_ptr script_mpa_list;
/*
** Read the file from HLSS
*/
{
   FILE *infile;
   int *mpas, start_mpa, frame_index;
   int *mpa_ptr;
   float *frame_ptr, *int_ptr;
   float *frames, *intensity;
   char garb1[50],garb2[50],garb3[50], text_input[80];
   MPA_Name mpa_name;
   int mp;
   float fm,intens,prev_frame,EPSILON=0.00001;
   Script_Mpa_ptr script_mpa_ptr;
   BOOL skip=NO;
  
   if (infile = fopen(filename, "r")) {
   /** reading starts **/
    start_mpa = -1; frame_index = 0;
    mpas = (int *) calloc(MAX_FRAMES,sizeof(int));
    frames = (float *) calloc(MAX_FRAMES,sizeof(float));
    intensity = (float *) calloc(MAX_FRAMES,sizeof(float));
    mpa_ptr = mpas; frame_ptr = frames; int_ptr = intensity;

    script_mpa_ptr = &script_mpa_list->mpa_list[0];
    script_mpa_list->list_size = 0;
    while (!feof(infile)) {
      fgets(text_input,80,infile);
      switch (text_input[0]) {
        case '[' : /* beginnig of the mpa */
          sscanf(text_input,"%s %s %s\n",garb1,garb2,mpa_name);
          if (MPA_script_mpa_id_get(mpa_name,&mp,mpa_name_list)) {
           *script_mpa_ptr = mp;
           script_mpa_ptr++;
           script_mpa_list->list_size += 1;
           prev_frame = -1;
           skip = NO;
          } else
           skip = YES;
        break;
        case ']' : /* skip a line */
          fgets(text_input,80,infile);
          mpa_ptr = mpas;
          frame_ptr = frames;
          int_ptr = intensity;
        break;
        default :
            sscanf(text_input,"%f %f %s %s %s\n",&fm,&intens,garb1,garb2,garb3); 
            *mpa_ptr = mp; *frame_ptr = fm; *int_ptr = intens;
            if (!skip && (fabs(prev_frame - fm) > EPSILON))
             MPA_script_add_entry(the_script,*mpa_ptr,*frame_ptr,*int_ptr,mpa_name);
            mpa_ptr++; frame_ptr++; int_ptr++;
            prev_frame = fm;
        break;
       }
     }
    fclose(infile);
    free(mpas);
    free(frames);
    free(intensity);
    return(YES);
   }
   else
    return(NO);
}

void
MPA_script_print(the_script,filename)
MPA_Script_ptr the_script;
char *filename;
/*
** Prints the script into a file
*/
{
   FILE *outfile;
   int i, j;
   Frame_ptr the_frame;
   MPA_ptr the_mpa;
   
   if (outfile = fopen(filename,"w")) {
     fprintf(outfile," the script_list_size %d\n",the_script->script_list_size);
     printf(" the script_list_size %d\n",the_script->script_list_size);
     fprintf(outfile,"\n");
     for (i=0; i<the_script->script_list_size; i++) {
       the_frame = &the_script->script_list[i];
       fprintf(outfile,"frame information\n");
       fprintf(outfile,"%d %f %d\n",
          the_frame->frame_id,the_frame->frame_no,the_frame->MPA_list_size);
       fprintf(outfile,"about the mpas of the frame\n");
        for (j=0; j<the_frame->MPA_list_size; j++) {
         the_mpa = &the_frame->MPA_list[j];
         fprintf(outfile,"( %d %d %f ) ",
                  the_mpa->MPA_id,the_mpa->MPA_no,the_mpa->MPA_intensity); 
        }
       fprintf(outfile,"\n");
      }
    fclose(outfile);
   }
}

BOOL
MPA_script_get_frame(the_script,frame,mpa_no,left,right,mpa_l_index,mpa_r_index)
MPA_Script_ptr the_script;
float frame;
int mpa_no, *left, *right, *mpa_l_index, *mpa_r_index;

{
    BOOL right_stop=NO, left_stop=NO;
    int i,j,k,end;
    float left_frame, right_frame;
    Frame_ptr left_frame_ptr, right_frame_ptr;
    float EPSILON=0.001;
    MPA_ptr the_mpa;

    end = the_script->script_list_size - 1;
 
    for (i=0; i<the_script->script_list_size; i++) {
     left_frame_ptr = &the_script->script_list[i];
     left_frame = left_frame_ptr->frame_no;
     for (k=0; k<left_frame_ptr->MPA_list_size; k++) {
      the_mpa = &left_frame_ptr->MPA_list[k];
      if (the_mpa->MPA_no == mpa_no) {    
       if (fabs(left_frame - frame) < EPSILON) {
        *left = i; *right = i; *mpa_l_index = k;
         return(YES);
        } 
       if (!left_stop) {
        if (left_frame > frame)  {
         *right = i; left_stop = YES; *mpa_r_index = k;
        }
       }
      }
     }
    }

    for (j=end; j>=0; j--) {
     right_frame_ptr = &the_script->script_list[j];
     right_frame = right_frame_ptr->frame_no;
     for (k=0; k<right_frame_ptr->MPA_list_size; k++) {
      the_mpa = &right_frame_ptr->MPA_list[k];
      if (the_mpa->MPA_no == mpa_no) {    
       if (!right_stop) {
        if (right_frame < frame)  {
         *left = j; right_stop = YES; *mpa_l_index = k;
        }
       }
      }
     }
    }

    if ( (!right_stop) && (left_stop) ) {
       *left = -1; 
       right_stop = YES;
     }   

    if ( (!left_stop) && (right_stop) ) {
       *right = -1; 
       left_stop = YES;
     }   
    
    if ((right_stop) && (left_stop)) 
    {
      return(YES);
    }
    else {
      printf(" there is no left/right at frame %f\n",frame);
      return(NO);
    }     
     
}

void
MPA_script_delete_repeated_mpa(the_script,f_index)
MPA_Script_ptr the_script;
int f_index;
{
  Frame_ptr the_frame;
  int i ,j, n;
  MPA_ptr ptr1, ptr2;

  the_frame = &the_script->script_list[f_index];

  n = the_frame->MPA_list_size;
  for (i=0; i<n-1; i++) {
    ptr1 = &the_frame->MPA_list[i];
    for (j=i+1; j<n; j++) {
     ptr2 = &the_frame->MPA_list[j];
      if (ptr1->MPA_no == ptr2->MPA_no) {
        ptr1->MPA_intensity += ptr2->MPA_intensity;
        if ( (ptr1->MPA_intensity > 1) || (ptr1->MPA_intensity < -1) ) {
         printf(" the value has exceeded max limits \n");
         printf(" the frame index %d frame num %f\n",f_index,the_frame->frame_no); 
         printf(" the mpa %d intensity %f\n",ptr1->MPA_no, ptr1->MPA_intensity);
        }
        ptr2->MPA_used = NO;
      }
     }
   }

   for (i=0; i<n; i++) 
     MPA_script_delete_notused_mpa(the_script,f_index);

}

void
MPA_script_run_interpolate(the_script,script_mpa_list)
MPA_Script_ptr the_script;
Script_Mpa_List_ptr script_mpa_list;
{
Frame_ptr new_script_list, the_frame, new_frame, left_frame, right_frame;
int new_frame_index, the_start, the_end, i, j, number_of_frames, index;
MPA_ptr old_mpa_list, new_mpa_list, ptr1, ptr2, left_mpa, right_mpa;
float fr_no, t;
int left, right;
int mpa_count, k, the_mpa_no, mpa_l_index, mpa_r_index;
BOOL *script_mpa_flag;

   new_frame_index = 0;
   the_frame = &the_script->script_list[new_frame_index];
   the_start = floor(the_frame->frame_no);
   while (new_frame_index < the_script->script_list_size)
     new_frame_index++;

   the_frame = &the_script->script_list[new_frame_index-1];
   the_end = ceil(the_frame->frame_no);

   number_of_frames = the_end - the_start + 1; 
   new_script_list = (Frame_ptr) calloc((number_of_frames),
                                         sizeof(Frame));
   script_mpa_flag = (BOOL *) calloc(script_mpa_list->list_size, sizeof(BOOL));
   for (i=0; i<script_mpa_list->list_size; i++)
     *script_mpa_flag = NO;
   new_frame = new_script_list;
   index = 0;
   for (i=the_start; i<=the_end; i++) {
     fr_no = i;
     new_frame->frame_no = fr_no;
     old_mpa_list = (MPA_ptr) calloc(script_mpa_list->list_size, sizeof(MPA));
     ptr1 = old_mpa_list;
     mpa_count = 0;
     for (k=0; k<script_mpa_list->list_size; k++) {
      the_mpa_no = script_mpa_list->mpa_list[k];
      MPA_script_get_frame(the_script,fr_no,the_mpa_no,&left,&right,&mpa_l_index,&mpa_r_index);
      if (left == right) {
       left_frame = &the_script->script_list[left];
       left_mpa = &left_frame->MPA_list[mpa_l_index];
       *ptr1 = *left_mpa;
        ptr1++; mpa_count++;
      } else
      if (left == -1) {
      } else
      if (right == -1) {
       if (!script_mpa_flag[k]) {
       left_frame = &the_script->script_list[left];
       left_mpa = &left_frame->MPA_list[mpa_l_index];
       *ptr1 = *left_mpa;
        ptr1++; mpa_count++; 
        script_mpa_flag[k] = YES;
       }
       else{
       }      
      }
      else {
       left_frame = &the_script->script_list[left];
       right_frame = &the_script->script_list[right];
       left_mpa = &left_frame->MPA_list[mpa_l_index];
       right_mpa = &right_frame->MPA_list[mpa_r_index];
       t = (new_frame->frame_no - left_frame->frame_no)/
          (right_frame->frame_no - left_frame->frame_no);
       ptr1->MPA_no = left_mpa->MPA_no; 
       ptr1->MPA_intensity = left_mpa->MPA_intensity * (1 - t) + right_mpa->MPA_intensity * (t); 
       strcpy(ptr1->MPA_name,left_mpa->MPA_name);
       ptr1++; mpa_count++;
      } 
     }
     new_mpa_list = (MPA_ptr) calloc(mpa_count, sizeof(MPA));
     new_frame->MPA_list_size = mpa_count;
     ptr2 = new_mpa_list; ptr1 = old_mpa_list;
     for (j=0; j<new_frame->MPA_list_size; j++) { 
      ptr2->MPA_no = ptr1->MPA_no;
      ptr2->MPA_intensity = ptr1->MPA_intensity;
      strcpy(ptr2->MPA_name,ptr1->MPA_name);
      ptr2->MPA_id = j;
      ptr2->MPA_used = YES; 
      ptr2++; ptr1++;
     }
     free(old_mpa_list);
     new_frame->MPA_list = new_mpa_list;
     new_frame->frame_id = index;
     index++;
     new_frame++;
   }

   free(script_mpa_flag);

   free(the_script->script_list);
   the_script->script_list_size = number_of_frames;
   the_script->script_list = new_script_list;

   printf(" the number of frames %d \n",the_script->script_list_size);

}

Expression_List_ptr
Script_explist_new()
/*
** Allocates the memory for the new list of exp 
*/
{
   Expression_List_ptr the_exp_list;

   the_exp_list = (Expression_List_ptr) calloc(1,sizeof(Expression_List));
   the_exp_list->exp_list = NULL;
   the_exp_list->exp_list_size = 0;
   return(the_exp_list);
}

void
Script_explist_dispose(the_exp_list)
Expression_List_ptr the_exp_list;
/*
** Disposes the memory for the explist 
*/
{
   if (the_exp_list->exp_list_size > 0) {
    free(the_exp_list->exp_list);
    the_exp_list->exp_list_size = 0;
   }
}

BOOL
Script_exp_read(the_exp_list,filename,mpa_name_list)
Expression_List_ptr the_exp_list;
char *filename;
MPA_Name_List_ptr mpa_name_list;

{   

   FILE *infile;
   char text_input[80], garb1[50], garb2[50]; 
   MPA_Name mpa_name;
   Expression_Name exp_name;
   float intensity;
   int mpa_no;

   if (infile = fopen(filename, "r")) {
    while (!feof(infile)) {
      fgets(text_input,80,infile);
      switch (text_input[0]) {
        case '[' : 
          switch (text_input[1]) {
           case 'e' :
            sscanf(text_input,"%s %s\n",garb1,exp_name);
            Script_exp_add_init(the_exp_list,exp_name,1.0,1.0);
           break;
           case 'm' :
            sscanf(text_input,"%s %s %s %f\n",garb1,mpa_name,garb2,&intensity);
            MPA_script_mpa_id_get(mpa_name,&mpa_no,mpa_name_list);
            Script_exp_add_mpa(the_exp_list,exp_name,mpa_name,mpa_no,intensity);
           break;
           default :
           break;
          }
        break;
        case ']' : /* skip a line */
          fgets(text_input,80,infile);
        break;
        default :
        break;
       }
     }
    fclose(infile);
    return(YES);
    }

    return(NO); 
}

void
Script_exp_single_init(the_exp,name,intensity,duration)
Expression_ptr the_exp;
Expression_Name name;
float intensity;
float duration;
{   

   the_exp->MPA_list = NULL;   
   the_exp->MPA_list_size = 0;   
   strcpy(the_exp->name,name);
   the_exp->id = 0;
   the_exp->intensity = intensity;
   the_exp->duration = duration;

}

BOOL
Script_exp_single_add_mpa(the_exp,exp_name,mpa_name,mpa_no,intensity)
Expression_ptr the_exp;
Expression_Name exp_name;
MPA_Name mpa_name;
int mpa_no;
float intensity;
{

MPA_ptr new_mpa, ptr1, ptr2, new_mpa_list;
int new_mpa_index,i;

    for (i=0; i<the_exp->MPA_list_size; i++) {
	ptr1 = &the_exp->MPA_list[i];
	if (ptr1->MPA_no == mpa_no) {
	  ptr1->MPA_intensity = intensity;
	  return (YES);
	}
    }
          
    new_mpa_index = 0;
    while (new_mpa_index < the_exp->MPA_list_size)
     new_mpa_index++;
         
    new_mpa_list = (MPA_ptr) 
                  calloc((the_exp->MPA_list_size + 1), sizeof(MPA));

    if (new_mpa_index != 0) {
     ptr1 = the_exp->MPA_list;
     ptr2 = new_mpa_list;
     for (i=1; i <= the_exp->MPA_list_size; i++) 
      *ptr2++ = *ptr1++;
     free(the_exp->MPA_list);
    }
    the_exp->MPA_list_size += 1;
    the_exp->MPA_list = new_mpa_list;
  
    new_mpa = &the_exp->MPA_list[new_mpa_index];
    new_mpa->MPA_no = mpa_no;
    new_mpa->MPA_intensity = intensity;
    strcpy(new_mpa->MPA_name,mpa_name);
    new_mpa->MPA_id = new_mpa_index;
    new_mpa->MPA_used = YES;
    
    return(YES);
}

BOOL
Script_exp_single_read(the_exp,filename,mpa_name_list)
Expression_ptr the_exp;
char *filename;
MPA_Name_List_ptr mpa_name_list;

{   

   FILE *infile;
   char text_input[80], garb1[50], garb2[50]; 
   MPA_Name mpa_name;
   Expression_Name exp_name;
   float intensity;
   int mpa_no;

   if (infile = fopen(filename, "r")) {
    while (!feof(infile)) {
      fgets(text_input,80,infile);
      switch (text_input[0]) {
        case '[' : 
          switch (text_input[1]) {
           case 'e' :
            sscanf(text_input,"%s %s\n",garb1,exp_name);
            Script_exp_single_init(the_exp,exp_name,1.0,1.0);
           break;
           case 'm' :
            sscanf(text_input,"%s %s %s %f\n",garb1,mpa_name,garb2,&intensity);
            MPA_script_mpa_id_get(mpa_name,&mpa_no,mpa_name_list);
            Script_exp_single_add_mpa(the_exp,exp_name,mpa_name,mpa_no,intensity);
           break;
           default :
           break;
          }
        break;
        case ']' : /* skip a line */
          fgets(text_input,80,infile);
        break;
        default :
        break;
       }
     }
    fclose(infile);
    return(YES);
    }

    return(NO); 
}


BOOL
Script_exp_ipc_read(the_exp_list,filename,mpa_name_list)
Expression_List_ptr the_exp_list;
char *filename;
MPA_Name_List_ptr mpa_name_list;

{   

   FILE *infile;
   char text_input[80], garb1[50], garb2[50]; 
   MPA_Name mpa_name;
   Expression_Name exp_name;
   float intensity,duration;
   int mpa_no;

   if (infile = fopen(filename, "r")) {
    Script_explist_dispose(the_exp_list);
    the_exp_list = Script_explist_new();
    while (!feof(infile)) {
      fgets(text_input,80,infile);
      switch (text_input[0]) {
        case '[' : 
          switch (text_input[1]) {
           case 'e' :
            sscanf(text_input,"%s %s %f %f\n",garb1,exp_name,&intensity,&duration);
            Script_exp_add_init(the_exp_list,exp_name,intensity,duration);
           break;
           case 'm' :
            sscanf(text_input,"%s %s %s %f\n",garb1,mpa_name,garb2,&intensity);
            MPA_script_mpa_id_get(mpa_name,&mpa_no,mpa_name_list);
            Script_exp_add_mpa(the_exp_list,exp_name,mpa_name,mpa_no,intensity);
           break;
           default :
           break;
          }
        break;
        case ']' : /* skip a line */
          fgets(text_input,80,infile);
        break;
        default :
        break;
       }
     }
    fclose(infile);
    return(YES);
    }

    return(NO); 
}

BOOL
Script_exp_write(the_exp_list,filename)
Expression_List_ptr the_exp_list;
char *filename;

{
   FILE *infile;
   MPA_Name mpa_name;
   float intensity;

   if (infile = fopen(filename, "w")) {
    MPA_FOR_EACH_EXPRESSION(the_exp_list) 
      fprintf(infile,"[expression %s \n",exp_ptr->name);
      fprintf(infile,"\n");
      MPA_FOR_EACH_MPA_OF_EXP(exp_ptr)
       intensity = mpa_ptr->MPA_intensity;
       strcpy(mpa_name,mpa_ptr->MPA_name);
       fprintf(infile,"[mpa %s intensity %f ]\n",mpa_name,intensity);
      MPA_END_FOR_EACH
      fprintf(infile,"\n");
      fprintf(infile,"]\n");
      fprintf(infile,"\n");
     MPA_END_FOR_EACH  
    fclose(infile);
    return(YES);
    }
     
    return(NO); 
}

BOOL
Script_exp_ipc_write(the_exp_list,filename)
Expression_List_ptr the_exp_list;
char *filename;

{
   FILE *infile;
   MPA_Name mpa_name;
   float intensity;

   if (infile = fopen(filename, "w")) {
    MPA_FOR_EACH_EXPRESSION(the_exp_list) 
      fprintf(infile,"[expression %s %f\n",exp_ptr->name,exp_ptr->intensity);
      fprintf(infile,"\n");
      MPA_FOR_EACH_MPA_OF_EXP(exp_ptr)
       intensity = mpa_ptr->MPA_intensity;
       strcpy(mpa_name,mpa_ptr->MPA_name);
       fprintf(infile,"[mpa %s intensity %f ]\n",mpa_name,intensity);
      MPA_END_FOR_EACH
      fprintf(infile,"\n");
      fprintf(infile,"]\n");
      fprintf(infile,"\n");
     MPA_END_FOR_EACH  
    fclose(infile);
    return(YES);
    }
     
    return(NO); 
}

BOOL 
Script_exp_name_exist(the_exp_list,name) 
Expression_List_ptr the_exp_list;
Expression_Name name;
{
    MPA_FOR_EACH_EXPRESSION(the_exp_list)
      if (strcmp(exp_ptr->name,name) == 0)
        return(YES);
    MPA_END_FOR_EACH

    return(NO); 
}

void
Script_exp_add_init(the_exp_list,name,intensity,duration)
Expression_List_ptr the_exp_list;
Expression_Name name;
float intensity;
float duration;
{   

   int new_exp_index, i;
   Expression_ptr ptr1,ptr2,the_exp,new_exp_list;

   new_exp_index = 0;
   while (new_exp_index < the_exp_list->exp_list_size)
     new_exp_index++;
   new_exp_list = (Expression_ptr) calloc((the_exp_list->exp_list_size + 1),
                                         sizeof(Expression));

   if (new_exp_index != 0) {
     ptr1 = the_exp_list->exp_list;
     ptr2 = new_exp_list;
     for (i=1; i <= the_exp_list->exp_list_size; i++)
      *ptr2++ = *ptr1++;
     free(the_exp_list->exp_list);
   }

   the_exp_list->exp_list_size += 1;
   the_exp_list->exp_list = new_exp_list;
   the_exp = &the_exp_list->exp_list[new_exp_index];

   the_exp->MPA_list = NULL;   
   the_exp->MPA_list_size = 0;   
   strcpy(the_exp->name,name);
   the_exp->id = new_exp_index;
   the_exp->intensity = intensity;
   the_exp->duration = duration;

}

BOOL
Script_exp_add_mpa(the_exp_list,exp_name,mpa_name,mpa_no,intensity)
Expression_List_ptr the_exp_list;
Expression_Name exp_name;
MPA_Name mpa_name;
int mpa_no;
float intensity;
{

MPA_ptr new_mpa, ptr1, ptr2, new_mpa_list;
Expression_ptr the_exp;
int new_mpa_index,i, the_index;

    the_index = -1;

    MPA_FOR_EACH_EXPRESSION(the_exp_list)
     if (strcmp(exp_ptr->name,exp_name) == 0) 
        the_index = exp_index;
    MPA_END_FOR_EACH

    if (the_index == -1) return(NO);
  
    the_exp = &the_exp_list->exp_list[the_index];

   
    for (i=0; i<the_exp->MPA_list_size; i++) {
	ptr1 = &the_exp->MPA_list[i];
	if (ptr1->MPA_no == mpa_no) {
	  ptr1->MPA_intensity = intensity;
	  return (YES);
	}
    }
          
    new_mpa_index = 0;
    while (new_mpa_index < the_exp->MPA_list_size)
     new_mpa_index++;
         
    new_mpa_list = (MPA_ptr) 
                  calloc((the_exp->MPA_list_size + 1), sizeof(MPA));

    if (new_mpa_index != 0) {
     ptr1 = the_exp->MPA_list;
     ptr2 = new_mpa_list;
     for (i=1; i <= the_exp->MPA_list_size; i++) 
      *ptr2++ = *ptr1++;
     free(the_exp->MPA_list);
    }
    the_exp->MPA_list_size += 1;
    the_exp->MPA_list = new_mpa_list;
  
    new_mpa = &the_exp->MPA_list[new_mpa_index];
    new_mpa->MPA_no = mpa_no;
    new_mpa->MPA_intensity = intensity;
    strcpy(new_mpa->MPA_name,mpa_name);
    new_mpa->MPA_id = new_mpa_index;
    new_mpa->MPA_used = YES;
    
    return(YES);
}

BOOL
Script_exp_delete_mpa(the_exp_list,exp_index,mpa_no)
Expression_List_ptr the_exp_list;
int exp_index, mpa_no;
{

MPA_ptr the_mpa, ptr1, ptr2, new_mpa_list;
Expression_ptr the_exp;
int i;

BOOL OK;

    if (exp_index == -1) return(NO);
    the_exp = &the_exp_list->exp_list[exp_index];
    if (the_exp->MPA_list_size <= 0) return(NO);
    
    OK = NO;
    MPA_FOR_EACH_MPA_OF_EXP(the_exp)
     if (mpa_ptr->MPA_no == mpa_no) {
       OK = YES;
     }
    MPA_END_FOR_EACH
    
    if (!OK) return(NO);
    
    new_mpa_list = (MPA_ptr) 
                  calloc((the_exp->MPA_list_size - 1), sizeof(MPA));

     ptr1 = the_exp->MPA_list;
     ptr2 = new_mpa_list;
     for (i=1; i <= the_exp->MPA_list_size; i++) {
      the_mpa = &the_exp->MPA_list[i-1];
      if (the_mpa->MPA_no == mpa_no) {
       ptr1++;
      }
      else
      *ptr2++ = *ptr1++;
     }
     free(the_exp->MPA_list);

     the_exp->MPA_list_size -= 1;
     the_exp->MPA_list = new_mpa_list;
  
    return(YES);
   
}

void
Script_exp_delete(the_exp_list,exp_name)
Expression_List_ptr the_exp_list;
Expression_Name exp_name;
{
  
   Expression_ptr the_exp, new_exp_list, ptr1, ptr2, exp_p;
   int i, the_index, new_exp_list_size, exp_id;

   the_index = -1;

   MPA_FOR_EACH_EXPRESSION(the_exp_list)
     if (strcmp(exp_ptr->name,exp_name) == 0) 
        the_index = exp_index;
   MPA_END_FOR_EACH

   if (the_index != -1) {
    the_exp = &the_exp_list->exp_list[the_index];
    free(the_exp->MPA_list);
    the_exp->MPA_list_size = 0;
    new_exp_list = (Expression_ptr) 
                  calloc((the_exp_list->exp_list_size - 1), sizeof(Expression));
    new_exp_list_size =  the_exp_list->exp_list_size - 1;  
    ptr1 = the_exp_list->exp_list;
    ptr2 = new_exp_list;
    for (i=1; i <= the_exp_list->exp_list_size; i++) {
      exp_p = &the_exp_list->exp_list[i-1];
      exp_id = exp_p->id;
      if (exp_id != the_index) 
       *ptr2++ = *ptr1++;
      else
       ptr1++;
    }
    free(the_exp_list->exp_list);
    
    the_exp_list->exp_list_size = new_exp_list_size;
    the_exp_list->exp_list = new_exp_list;
    ptr2 = new_exp_list;  
    for (i=0; i<the_exp_list->exp_list_size; i++) {
      ptr2->id = i;
      ptr2++;
    }
   }
 
}

void
Script_exp_copy(exp1,exp2)
Expression_ptr exp1,exp2;
{
    Expression_ptr new_exp;
    MPA_ptr ptr1,ptr2;
    int i;

    if (!exp2) {
     if (!exp2->MPA_list) free(exp2->MPA_list);
     free(exp2);
    }

    new_exp = (Expression_ptr) calloc(1,sizeof(Expression));
    new_exp->MPA_list_size = exp1->MPA_list_size;
    new_exp->MPA_list = (MPA_ptr) calloc(new_exp->MPA_list_size,sizeof(MPA));
    strcpy(new_exp->name,"dummy");
    new_exp->intensity = exp1->intensity;
    ptr1 = new_exp->MPA_list;
    ptr2 = exp1->MPA_list;
    for (i=0; i<new_exp->MPA_list_size; i++) 
      *ptr1++ = *ptr2++;
     
    exp2 = new_exp;
}

void
Script_exp_index_get(the_exp_list,name,index)
Expression_List_ptr the_exp_list;
Expression_Name name;
int *index;
{
      *index = -1;
      MPA_FOR_EACH_EXPRESSION(the_exp_list)
        if (strcmp(name,exp_ptr->name) == 0)
          *index = exp_index;
      MPA_END_FOR_EACH 
}

/* To obtain the instant intensity of the expression, with the
following assumptions :

 1. it is linear over time like :
          i(peak)
          /\
         /  \
        /    \------ i (initial)
       t0    tf

 2. i(initial) = 0;
 3. the peak occurs at t/2
*/


void
Script_ipc_exp_intensity_get(the_exp,start,current,rate,exp_intensity)
Expression_ptr the_exp;
float start,current,rate;
float *exp_intensity;
{

  float elapse_time;
  float exp_duration;
  float slope,final;

  elapse_time = current - start;
  exp_duration = the_exp->duration;
  final = start + the_exp->duration;

  if (elapse_time <= exp_duration/2.0) {
    slope = (the_exp->intensity - 0.0)/(the_exp->duration/2.0 - start);
    *exp_intensity = (elapse_time * slope) + 0.0;
  }
  else {
    slope = (the_exp->intensity - 0.0)/(final - the_exp->duration/2.0);
    *exp_intensity = (exp_duration - elapse_time) * slope + 0.0;
  }
  
}


/*---------------------------------------------------------------------*/
BOOL
Script_ipc_mpaqueue_mpa_exist(the_frame,mpa_no,the_index)
Frame_ptr the_frame;
int mpa_no;
int *the_index;
{
     int i;
     MPA_ptr the_mpa;

     *the_index = -1;
     for (i=0; i<the_frame->MPA_list_size; i++) {
       the_mpa = &the_frame->MPA_list[i];
       if (mpa_no == the_mpa->MPA_no) {
        *the_index = i;
        return(YES);   
       }      
     }
     return(NO);
}

/*---------------------------------------------------------------------*/
void
Script_ipc_mpaqueue_add(the_frame,mpa_no,mpa_intensity)
Frame_ptr the_frame;
int mpa_no;
float mpa_intensity;
{
    int i,new_index;
    MPA_ptr ptr1,ptr2,new_list,new_mpa;

    new_index = the_frame->MPA_list_size;
    new_list = (MPA_ptr) 
               calloc((the_frame->MPA_list_size + 1), sizeof(MPA));

    if (new_index != 0) {
     ptr1 = the_frame->MPA_list;
     ptr2 = new_list;
     for (i=0; i<the_frame->MPA_list_size; i++) 
      *ptr2++ = *ptr1++;
     if (the_frame->MPA_list != NULL)
      free(the_frame->MPA_list);
    }

   if (the_frame->MPA_list != NULL)
     free(the_frame->MPA_list);

   the_frame->MPA_list_size += 1;
   the_frame->MPA_list = new_list;
  
   new_mpa = &the_frame->MPA_list[new_index];
   new_mpa->MPA_no = mpa_no;
   new_mpa->MPA_intensity = mpa_intensity;
   new_mpa->MPA_id = new_index;
   new_mpa->MPA_used = YES;

}

/*---------------------------------------------------------------------*/

void
Script_ipc_mpaqueue_modify(the_mpa_queue,frame_id,mpa_no,mpa_intensity)
MPA_Script_ptr the_mpa_queue;
int frame_id;
int mpa_no;
float mpa_intensity;
{
    Frame_ptr the_frame;
    int the_index;
    MPA_ptr the_mpa;
    float the_sum,t,intensity;

    the_frame = &the_mpa_queue->script_list[frame_id];
    if (Script_ipc_mpaqueue_mpa_exist(the_frame,mpa_no,&the_index)) {
      the_mpa = &the_frame->MPA_list[the_index];
      the_sum = the_mpa->MPA_intensity + mpa_intensity;
/*
      t = fabs(the_sum)/2.0;
      intensity = t*t*(3.0 - 2.0*t);
*/
      t = (fabs(the_sum)/2.0 - 0.0) * (M_PI/2.0 - 0.0);
      intensity = sin(t);
      if (the_sum < 0.0) 
       intensity = -intensity;
        the_mpa->MPA_intensity = intensity;
    } else 
    {
      Script_ipc_mpaqueue_add(the_frame,mpa_no,mpa_intensity);
    }
}

/*---------------------------------------------------------------------*/

void
Script_ipc_frame_add(the_mpa_queue)
MPA_Script_ptr the_mpa_queue;
{
    int i,new_index;
    Frame_ptr new_list,ptr1,ptr2,the_frame;

    new_index = the_mpa_queue->script_list_size;
    new_list = (Frame_ptr) calloc((the_mpa_queue->script_list_size + 1),
                                   sizeof(Frame));
    if (new_index != 0) {
     ptr2 = new_list;
     ptr1 = the_mpa_queue->script_list;
     for (i=0; i<the_mpa_queue->script_list_size; i++) 
        *ptr2++ = *ptr1++; 
     if (the_mpa_queue->script_list)   
      free(the_mpa_queue->script_list);
    }

    the_mpa_queue->script_list = new_list;
    the_mpa_queue->script_list_size += 1;
    the_frame = &the_mpa_queue->script_list[new_index];
    the_frame->frame_id = new_index;

}

/*----------------------------------------------------------------------*/

BOOL
Script_ipc_mpaqueue_executed_frames_delete(the_mpa_queue)
MPA_Script_ptr the_mpa_queue;
{
    Frame_ptr ptr1,ptr2,new_list;
    int i,count;

    count = 0;
    MPA_FOR_EACH_FRAME(the_mpa_queue)
      if (!frame_ptr->executed) count++;
    MPA_END_FOR_EACH

    if (count == 0) {
      if (the_mpa_queue->script_list)
       free(the_mpa_queue->script_list);
      the_mpa_queue->script_list_size = 0;
      return(YES);
    } 
    
    new_list = (Frame_ptr) 
               calloc((count), sizeof(Frame));
    ptr1 = the_mpa_queue->script_list;
    ptr2 = new_list;
    for (i=0; i<the_mpa_queue->script_list_size; i++)  {
      if (!ptr1->executed)
       *ptr2++ = *ptr1++;
      else
        ptr1++;
    }
    if (the_mpa_queue->script_list)
     free(the_mpa_queue->script_list);
    the_mpa_queue->script_list = new_list;
    the_mpa_queue->script_list_size = count;
    ptr1 = the_mpa_queue->script_list;
    for (i=0; i<the_mpa_queue->script_list_size; i++) {
      ptr1->frame_id = i;
      ptr1++;
    }
    return(YES);
   
}


/*-------------------------------------------------------------------*/
BOOL
Script_ipc_expqueue_update(the_exp_list,exp_index,exp_dur,exp_step,exp_int,
                           the_mpa_queue,frame_index,next)
Expression_List_ptr the_exp_list;
int exp_index;
float exp_dur, exp_step, exp_int;
MPA_Script_ptr the_mpa_queue;
int frame_index;
int *next;

{
     float start,current,step,shift_factor,exp_intensity,mpa_intensity;
     Expression_ptr the_exp;
     int current_exp_frame,frames_per_exp;


     the_exp = &the_exp_list->exp_list[exp_index];
     
     if (frame_index > the_mpa_queue->script_list_size) return(NO);     

     start = 0.0;
     step = exp_step;

     the_exp->duration = exp_dur;
     the_exp->intensity = exp_int;
     frames_per_exp = floor( ( (double) (the_exp->duration)/step) + 0.5) + 1;

     if (frames_per_exp < 3) return(NO);

     if (frames_per_exp%2 == 0)
       shift_factor = step/2.0;
     else
       shift_factor = 0.0;
     
     /* for the first frame */
     if (frame_index == the_mpa_queue->script_list_size) {
        Script_ipc_frame_add(the_mpa_queue);
     }
     current = start;
     current_exp_frame = 0;
     Script_ipc_exp_intensity_get(the_exp,start,current,step,&exp_intensity);
     MPA_FOR_EACH_MPA_OF_EXP(the_exp)
      mpa_intensity = exp_intensity * (mpa_ptr->MPA_intensity);
      Script_ipc_mpaqueue_modify(the_mpa_queue,frame_index,mpa_ptr->MPA_no,mpa_intensity);
     MPA_END_FOR_EACH

     frame_index++;
     current = current + step - shift_factor;
     current_exp_frame++;

     /* for the intermediate frames */
     while (current_exp_frame < (frames_per_exp - 1)) {
      if (frame_index == the_mpa_queue->script_list_size) 
        Script_ipc_frame_add(the_mpa_queue);
      Script_ipc_exp_intensity_get(the_exp,start,current,step,&exp_intensity);
      MPA_FOR_EACH_MPA_OF_EXP(the_exp)
       mpa_intensity = exp_intensity * (mpa_ptr->MPA_intensity);
       Script_ipc_mpaqueue_modify(the_mpa_queue,frame_index,mpa_ptr->MPA_no,mpa_intensity);
      MPA_END_FOR_EACH
      frame_index++;
      current = current + step;
      current_exp_frame++;
     }

     /* for the last frame */
     if (frame_index == the_mpa_queue->script_list_size) 
        Script_ipc_frame_add(the_mpa_queue);
     current = start;
     Script_ipc_exp_intensity_get(the_exp,start,current,step,&exp_intensity);
     MPA_FOR_EACH_MPA_OF_EXP(the_exp)
      mpa_intensity = exp_intensity * (mpa_ptr->MPA_intensity);
      Script_ipc_mpaqueue_modify(the_mpa_queue,frame_index,mpa_ptr->MPA_no,mpa_intensity);
     MPA_END_FOR_EACH

     *next = frame_index;
     return(YES);     

}    
