
/*------------------------------------------------------------------------------*/
/*										*/
/*										*/
/*  This software module was originally developed by                            */
/*        Prem Kalra, MIRALab, University of Geneva                             */
/*  and edited by                                                               */
/*        Igor Pandzic, MIRALab, University of Geneva                           */
/*        Marc Escher, MIRALab, University of Geneva                            */
/*        Pierre Beylot, MIRALab, University of Geneva                          */
/*  in the course of development of the MPEG-4 Video (ISO/IEC 14496-2).	        */
/*  This software module is an implementation of a part of one or more	        */
/*  MPEG-4 Video (ISO/IEC 14496-2) tools as specified by the MPEG-4 Video       */
/*  (ISO/IEC 14496-2). ISO/IEC gives users of the                               */
/*  MPEG-4 Video (ISO/IEC 14496-2) free license to this software module or	*/
/*  modifications thereof for use in hardware or software products claiming 	*/
/*  conformance to the MPEG-4 Video (ISO/IEC 14496-2). Those intending to use	*/
/*  this software module in hardware or software products are advised that	*/
/*  its use may infringe existing patents. The original developer of this	*/
/*  software module and his/her company, the subsequent editors and their	*/
/*  companies, and ISO/IEC have no liability for use of this software		*/
/*  module or modifications thereof. Copyright is not released for non		*/
/*  MPEG-4 Video (ISO/IEC 14496-2) conforming products.                         */
/*  MIRALab, University of Geneva retains full right to use the code for its    */
/*  own purpose, assign or donate the code to a third party and to inhibit      */
/*  third parties from using the code for non MPEG-4 Video (ISO/IEC 14496-2)    */
/*  conforming products. This copyright notice must be included in all copies   */
/*  or derivative works. Copyright (c) (1997).			      		*/
/*										*/
/*										*/
/*------------------------------------------------------------------------------*/

#include "FaceMan.h"

/*----- type declarations -----------*/

typedef char MPA_Name[50];

typedef struct {
   int      MPA_no;
   float    MPA_intensity;
   MPA_Name MPA_name;
   int      MPA_id;
   BOOL     MPA_used;
} MPA, *MPA_ptr;

typedef struct {
   MPA_ptr MPA_list;
   int     MPA_list_size;
   float   frame_no;
   int     frame_id;
   BOOL    executed;
} Frame, *Frame_ptr;

typedef struct {
   Frame_ptr  script_list;
   int        script_list_size;
} MPA_Script, *MPA_Script_ptr;

typedef struct {
    char   name[50];
    short  id;
} MPA_Name_No;

typedef struct {
 MPA_Name_No list[100];
 int         size;
} MPA_Name_List, *MPA_Name_List_ptr;

typedef int Script_Mpa, *Script_Mpa_ptr;

typedef struct {
 Script_Mpa_ptr mpa_list;
 int            list_size;
} Script_Mpa_List, *Script_Mpa_List_ptr;

typedef char Expression_Name[50];

typedef struct {
 MPA_ptr         MPA_list;
 int             MPA_list_size;
 Expression_Name name;
 float           intensity;
 float           duration;
 short           id;
} Expression, *Expression_ptr;

typedef struct {
 Expression_ptr exp_list;
 int exp_list_size;
} Expression_List, *Expression_List_ptr;


/*---- macros    ---------------------*/

#define MPA_FOR_EACH_FRAME(the_script) \
 { \
    Frame_ptr frame_ptr; \
    register int frame_index; \
    for (frame_index = 0, \
         frame_ptr = the_script->script_list; \
         frame_index < the_script->script_list_size; \
         frame_index++, \
         frame_ptr++) \
         { 

#define MPA_END_FOR_EACH }}

#define MPA_FOR_EACH_MPA(the_frame) \
 { \
     MPA_ptr mpa_ptr; \
     register int mpa_index; \
     for (mpa_index = 0, \
          mpa_ptr = the_frame->MPA_list; \
          mpa_index < the_frame->MPA_list_size; \
          mpa_index++, \
          mpa_ptr++) \
          {

#define MPA_FOR_EACH_EXPRESSION(the_exp_list) \
 { \
    Expression_ptr exp_ptr; \
    register int exp_index; \
    for (exp_index = 0, \
         exp_ptr = the_exp_list->exp_list; \
         exp_index < the_exp_list->exp_list_size; \
         exp_index++, \
         exp_ptr++) \
         { 

#define MPA_FOR_EACH_MPA_OF_EXP(the_exp) \
 { \
     MPA_ptr mpa_ptr; \
     register int mpa_index; \
     for (mpa_index = 0, \
          mpa_ptr = the_exp->MPA_list; \
          mpa_index < the_exp->MPA_list_size; \
          mpa_index++, \
          mpa_ptr++) \
          {

#define MPA_FOR_EACH_MPA_OF_QUEUE(the_mpa_queue) \
 { \
     MPA_ptr mpa_ptr; \
     register int mpa_index; \
     for (mpa_index = 0, \
          mpa_ptr = the_mpa_queue->MPA_list; \
          mpa_index < the_mpa_queue->MPA_list_size; \
          mpa_index++, \
          mpa_ptr++) \
          {

#define MPA_END_FOR_EACH }}

/*---- constants  --------------------*/   

#define MAX_FRAMES 3000
#define MAX_MPAS 100

/*---- functions ---------------------*/

extern MPA_Script_ptr
MPA_script_new(void);

extern void
MPA_script_dispose(MPA_Script_ptr);

extern BOOL
MPA_script_frame_exists(MPA_Script_ptr,int,double);

extern void
MPA_script_add_mpa(MPA_Script_ptr,int,int,double,MPA_Name);

extern void
MPA_script_delete_mpa(MPA_Script_ptr,int,int);

extern void
MPA_script_delete_notused_mpa(MPA_Script_ptr,int);

extern void
MPA_script_add_frame(MPA_Script_ptr,int,double,double,MPA_Name);

extern void
MPA_script_add_entry(MPA_Script_ptr,int,double,double,MPA_Name);

extern void
MPA_script_mpalist_define(MPA_Name_List_ptr);

extern BOOL
MPA_script_mpa_id_get(MPA_Name,int *, MPA_Name_List_ptr);

extern BOOL
MPA_script_read(MPA_Script_ptr,char *,MPA_Name_List_ptr, Script_Mpa_List_ptr);

extern void
MPA_script_print(MPA_Script_ptr,char *);

extern BOOL
MPA_script_get_frame(MPA_Script_ptr,double,int , int *, int *, int *, int *);

extern void
MPA_script_delete_repeated_mpa(MPA_Script_ptr,int);

extern void
MPA_script_run_interpolate(MPA_Script_ptr, Script_Mpa_List_ptr);

/*-----------------------------------------------------------------------*/
extern Expression_List_ptr
Script_explist_new(void);

extern void
Script_explist_dispose(Expression_List_ptr);

extern BOOL
Script_exp_read(Expression_List_ptr,char *,MPA_Name_List_ptr);

extern BOOL
Script_exp_ipc_read(Expression_List_ptr,char *,MPA_Name_List_ptr);

extern BOOL
Script_exp_write(Expression_List_ptr,char *);

extern BOOL
Script_exp_ipc_write(Expression_List_ptr,char *);

extern BOOL
Script_exp_name_exist(Expression_List_ptr,Expression_Name);

extern void
Script_exp_add_init(Expression_List_ptr,Expression_Name,double,double);

extern BOOL
Script_exp_add_mpa(Expression_List_ptr,Expression_Name,MPA_Name,int,double);

extern BOOL
Script_exp_delete_mpa(Expression_List_ptr,int,int);

extern void
Script_exp_delete(Expression_List_ptr,Expression_Name);

extern void
Script_exp_copy(Expression_ptr,Expression_ptr);

extern void
Script_exp_index_get(Expression_List_ptr,Expression_Name,int *);

extern BOOL
Script_exp_single_read(Expression_ptr,char *,MPA_Name_List_ptr);

extern BOOL
Script_exp_single_add_mpa(Expression_ptr,Expression_Name,MPA_Name,int,double);

extern void
Script_exp_single_init(Expression_ptr,Expression_Name,double,double);

/*-------------------------------------------------------------------------*/
extern void
Script_ipc_exp_intensity_get(Expression_ptr,double,double,double,float *);

extern BOOL
Script_ipc_mpaqueue_mpa_exist(Frame_ptr,int,int *);

extern void
Script_ipc_mpaqueue_add(Frame_ptr,int,double);

extern void
Script_ipc_mpaqueue_modify(MPA_Script_ptr,int,int,double);

extern void
Script_ipc_frame_add(MPA_Script_ptr);

extern BOOL
Script_ipc_mpaqueue_executed_frames_delete(MPA_Script_ptr);

extern BOOL
Script_ipc_expqueue_update(Expression_List_ptr,int,double,double,double,MPA_Script_ptr,int,int *);

/*----*/
