
/*------------------------------------------------------------------------------*/
/*										*/
/*										*/
/*  This software module was originally developed by                            */
/*        Prem Kalra, MIRALab, University of Geneva                             */
/*  and edited by                                                               */
/*        Igor Pandzic, MIRALab, University of Geneva                           */
/*        Marc Escher, MIRALab, University of Geneva                            */
/*        Pierre Beylot, MIRALab, University of Geneva                          */
/*  in the course of development of the MPEG-4 Video (ISO/IEC 14496-2).	        */
/*  This software module is an implementation of a part of one or more	        */
/*  MPEG-4 Video (ISO/IEC 14496-2) tools as specified by the MPEG-4 Video       */
/*  (ISO/IEC 14496-2). ISO/IEC gives users of the                               */
/*  MPEG-4 Video (ISO/IEC 14496-2) free license to this software module or	*/
/*  modifications thereof for use in hardware or software products claiming 	*/
/*  conformance to the MPEG-4 Video (ISO/IEC 14496-2). Those intending to use	*/
/*  this software module in hardware or software products are advised that	*/
/*  its use may infringe existing patents. The original developer of this	*/
/*  software module and his/her company, the subsequent editors and their	*/
/*  companies, and ISO/IEC have no liability for use of this software		*/
/*  module or modifications thereof. Copyright is not released for non		*/
/*  MPEG-4 Video (ISO/IEC 14496-2) conforming products.                         */
/*  MIRALab, University of Geneva retains full right to use the code for its    */
/*  own purpose, assign or donate the code to a third party and to inhibit      */
/*  third parties from using the code for non MPEG-4 Video (ISO/IEC 14496-2)    */
/*  conforming products. This copyright notice must be included in all copies   */
/*  or derivative works. Copyright (c) (1997).			      		*/
/*										*/
/*										*/
/*------------------------------------------------------------------------------*/


# ifndef MODULE_VECTOR_HANDLING
# define MODULE_VECTOR_HANDLING


# include <math.h>
# include <stdio.h>

/*
** ------------------------------------
** -- Declaration des types exportes --
** ------------------------------------
*/

# define X_INDEX     0
# define Y_INDEX     1
# define Z_INDEX     2
# define W_INDEX     3


typedef float VECTOR_2D [1 + Y_INDEX];
typedef float VECTOR_3D [1 + Z_INDEX];
typedef float VECTOR_H3D[1 + W_INDEX];



/*
** ------------------------------------
** -- Operations sur les vecteurs 2D --
** ------------------------------------
*/

# define V2D_INIT(RES, X_VALUE, Y_VALUE)\
    {\
      RES[X_INDEX] = X_VALUE;\
      RES[Y_INDEX] = Y_VALUE;\
    }


# define V2D_NULL(RES)\
    {\
      V2D_INIT(RES, 0.0, 0.0);\
    }


# define V2D_COPY(RES, U)\
    {\
      RES[X_INDEX] = U[X_INDEX];\
      RES[Y_INDEX] = U[Y_INDEX];\
    }


# define V2D_DUMP(FILE, U)\
    {\
      (void)fprintf(FILE, "U = (%f, %f)\n", U[X_INDEX], U[Y_INDEX]);\
    }


# define V2D_NORM(U)\
    ( sqrt( U[X_INDEX] * U[X_INDEX] + \
            U[Y_INDEX] * U[Y_INDEX]))


# define V2D_NORMALIZE(RES, U)\
    {\
      register float _norm;\
      _norm = V2D_NORM(U);\
      V2D_INIT(RES, U[X_INDEX]/_norm, U[Y_INDEX]/_norm);\
    }


# define V2D_ADD(RES, LEFT, RIGHT)\
    {\
      RES[X_INDEX] = LEFT[X_INDEX] + RIGHT[X_INDEX]; \
      RES[Y_INDEX] = LEFT[Y_INDEX] + RIGHT[Y_INDEX]; \
    }


# define V2D_SUB(RES, LEFT, RIGHT)\
    {\
      RES[X_INDEX] = LEFT[X_INDEX] - RIGHT[X_INDEX]; \
      RES[Y_INDEX] = LEFT[Y_INDEX] - RIGHT[Y_INDEX]; \
    }


# define V2D_SCALE(RES, SCALAR, RIGHT)\
    {\
      RES[X_INDEX] = (SCALAR) * RIGHT[X_INDEX]; \
      RES[Y_INDEX] = (SCALAR) * RIGHT[Y_INDEX]; \
    }


# define V2D_INVERT(RES, U)\
    {\
      RES[X_INDEX] = - U[X_INDEX]; \
      RES[Y_INDEX] = - U[Y_INDEX]; \
    }


# define V2D_DOT(LEFT, RIGHT)\
    (LEFT[X_INDEX] * RIGHT[X_INDEX] +\
     LEFT[Y_INDEX] * RIGHT[Y_INDEX])



/*
** ------------------------------------
** -- Operations sur les vecteurs 3D --
** ------------------------------------
*/

# define V3D_INIT(RES, X_VALUE, Y_VALUE, Z_VALUE)\
    {\
      RES[X_INDEX] = X_VALUE;\
      RES[Y_INDEX] = Y_VALUE;\
      RES[Z_INDEX] = Z_VALUE;\
    }


# define V3D_NULL(RES)\
    {\
      V3D_INIT(RES, 0.0, 0.0, 0.0);\
    }


# define V3D_COPY(RES, U)\
    {\
      RES[X_INDEX] = U[X_INDEX];\
      RES[Y_INDEX] = U[Y_INDEX];\
      RES[Z_INDEX] = U[Z_INDEX];\
    }


# define V3D_DUMP(FILE, U)\
    {\
      (void)fprintf(FILE, "U = (%f, %f, %f)\n", U[X_INDEX], \
                                                U[Y_INDEX], \
                                                U[Z_INDEX]);\
    }


# define V3D_NORM(U)\
    ( sqrt( U[X_INDEX] * U[X_INDEX] + \
            U[Y_INDEX] * U[Y_INDEX] + \
            U[Z_INDEX] * U[Z_INDEX]))


# define V3D_NORMALIZE(RES, U)\
    {\
      register float _norm;\
      _norm = V3D_NORM(U);\
      V3D_INIT(RES, U[X_INDEX]/_norm, \
                    U[Y_INDEX]/_norm, \
                    U[Z_INDEX]/_norm);\
    }


# define V3D_ADD(RES, LEFT, RIGHT)\
    {\
      RES[X_INDEX] = LEFT[X_INDEX] + RIGHT[X_INDEX]; \
      RES[Y_INDEX] = LEFT[Y_INDEX] + RIGHT[Y_INDEX]; \
      RES[Z_INDEX] = LEFT[Z_INDEX] + RIGHT[Z_INDEX]; \
    }


# define V3D_SUB(RES, LEFT, RIGHT)\
    {\
      RES[X_INDEX] = LEFT[X_INDEX] - RIGHT[X_INDEX]; \
      RES[Y_INDEX] = LEFT[Y_INDEX] - RIGHT[Y_INDEX]; \
      RES[Z_INDEX] = LEFT[Z_INDEX] - RIGHT[Z_INDEX]; \
    }


# define V3D_SCALE(RES, SCALAR, RIGHT)\
    {\
      RES[X_INDEX] = (SCALAR) * RIGHT[X_INDEX]; \
      RES[Y_INDEX] = (SCALAR) * RIGHT[Y_INDEX]; \
      RES[Z_INDEX] = (SCALAR) * RIGHT[Z_INDEX]; \
    }

# define V3D_ADD_SCALE(RES, LEFT, SCALAR, RIGHT)\
    {\
      RES[X_INDEX] = LEFT[X_INDEX] + (SCALAR) * RIGHT[X_INDEX]; \
      RES[Y_INDEX] = LEFT[Y_INDEX] + (SCALAR) * RIGHT[Y_INDEX]; \
      RES[Z_INDEX] = LEFT[Z_INDEX] + (SCALAR) * RIGHT[Z_INDEX]; \
    }


# define V3D_INVERT(RES, U)\
    {\
      RES[X_INDEX] = - U[X_INDEX]; \
      RES[Y_INDEX] = - U[Y_INDEX]; \
      RES[Z_INDEX] = - U[Z_INDEX]; \
    }


# define V3D_DOT(LEFT, RIGHT)\
    (LEFT[X_INDEX] * RIGHT[X_INDEX] + \
     LEFT[Y_INDEX] * RIGHT[Y_INDEX] + \
     LEFT[Z_INDEX] * RIGHT[Z_INDEX])


# define V3D_CROSS(RES, LEFT, RIGHT)\
    {\
      RES[X_INDEX] = LEFT[Y_INDEX] * RIGHT[Z_INDEX] - \
                     LEFT[Z_INDEX] * RIGHT[Y_INDEX];  \
      RES[Y_INDEX] = LEFT[Z_INDEX] * RIGHT[X_INDEX] - \
                     LEFT[X_INDEX] * RIGHT[Z_INDEX];  \
      RES[Z_INDEX] = LEFT[X_INDEX] * RIGHT[Y_INDEX] - \
                     LEFT[Y_INDEX] * RIGHT[X_INDEX];  \
    }


/*
** ----------------------------
** -- Conversions 3D <-> H3D --
** ----------------------------
*/

# define VH3D_FROM_V3D(VH3D_OUT, V3D_IN)\
    {\
      VH3D_OUT[X_INDEX] = V3D_IN[X_INDEX];\
      VH3D_OUT[Y_INDEX] = V3D_IN[Y_INDEX];\
      VH3D_OUT[Z_INDEX] = V3D_IN[Z_INDEX];\
      VH3D_OUT[W_INDEX] = 1.0;\
    }


# define V3D_FROM_VH3D(V3D_OUT, VH3D_IN)\
    {\
      V3D_OUT[X_INDEX] = VH3D_IN[X_INDEX] / VH3D_IN[W_INDEX];\
      V3D_OUT[Y_INDEX] = VH3D_IN[Y_INDEX] / VH3D_IN[W_INDEX];\
      V3D_OUT[Z_INDEX] = VH3D_IN[Z_INDEX] / VH3D_IN[W_INDEX];\
    }


/*
** ----------------------------------------------
** -- Operations sur les vecteurs homogenes 3D --
** ----------------------------------------------
*/

# define VH3D_INIT(RES, X_VALUE, Y_VALUE, Z_VALUE)\
    {\
      RES[X_INDEX] = X_VALUE;\
      RES[Y_INDEX] = Y_VALUE;\
      RES[Z_INDEX] = Z_VALUE;\
      RES[W_INDEX] = 1.0;\
    }


# define VH3D_NULL(RES)\
    {\
      VH3D_INIT(RES, 0.0, 0.0, 0.0);\
    }


# define VH3D_COPY(RES, U)\
    {\
      RES[X_INDEX] = U[X_INDEX];\
      RES[Y_INDEX] = U[Y_INDEX];\
      RES[Z_INDEX] = U[Z_INDEX];\
      RES[W_INDEX] = U[W_INDEX];\
    }


# define VH3D_AT_INFINITY(U)\
    (_ABS(U[W_INDEX]) < 1.0E-7)


# define VH3D_DUMP(FILE, U)\
    {\
      (void)fprintf(FILE, "U = (%f, %f, %f, %f)\n", U[X_INDEX], \
                                                    U[Y_INDEX], \
                                                    U[Z_INDEX], \
                                                    U[W_INDEX]);\
    }



# endif
