/*
This software module was originally developed by 
Image Processing Software and Technology Research Department 5 of AT&T
in the course of development of the MPEG-4. 
This software module is an implementation of a part of one or 
more MPEG-4 tools as specified by the MPEG-4. 
ISO/IEC gives users of the MPEG-4 standard free license to this
software module or modifications thereof for use in hardware 
or software products claiming conformance to the MPEG-4 standard.
Those intending to use this software module in hardware or software
products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company,
the subsequent editors and their companies, and ISO/IEC have no
liability for use of this software module or modifications thereof
in an implementation.
Copyright is not released for non MPEG-4 standard conforming
products. AT&T retains full right to use the code for his/her own
purpose, assign or donate the code to a third party and to
inhibit third parties from using the code for non 
MPEG-4 standard conforming products.
This copyright notice must be included in all copies or
derivative works. Copyright (c) 1997, AT&T.
*/

#include <stdlib.h>
#include "PartInfo_mpeg.h"

// SOLARIS PORTING - removed the const
void PartInfo::setName( int name)
{
  _name = name;
}



TransformInfo::TransformInfo()
{
  node = new Transform;
}

TransformInfo::~TransformInfo(){}

void TransformInfo::setNode(Transform *transformNode)
{
  node = transformNode;
}

void TransformInfo::setNeutralValues(const TransformSTable *transformTable)
{
  fieldRef = transformTable->getFieldRef();
  if(fieldRef ==1){
    // SB ####
    neutralRotation = (Rotation) transformTable->getRotation();
  }
  else{
    // SB ####
    neutralTranslationOrScalefactor = (Vec3f) transformTable->getTranslationOrScalefactor();
  }
}

void TransformInfo::setTransformValue(const float value)
{
  float angle;
  const Rotation& rotation = (Rotation) neutralRotation;
  const Vec3f& tOrSfactor = (Vec3f) neutralTranslationOrScalefactor;
  switch(fieldRef){
  case 1:
    angle = value*1E-5;
    updated_rotation = Rotation(rotation.x, rotation.y, rotation.z,rotation.theta*angle); 
    node->rotation /* SetRotation */ = (Rotation) (updated_rotation);
    break;
  case 2:
    updated_translationOrScalefactor = Vec3f(value*tOrSfactor.x,value*tOrSfactor.y,value*tOrSfactor.z);
    node->scale /* SetScale */ = (Vec3f) (updated_translationOrScalefactor);
    break;
  case 3:
    updated_translationOrScalefactor = Vec3f(value*tOrSfactor.x,value*tOrSfactor.y,value*tOrSfactor.z);
    node->translation /* SetTranslation */ = (Vec3f) (updated_translationOrScalefactor);
    break;
  }
}

Transform *TransformInfo::getNode()
{
  return node;
}

IndexedFaceSetInfo::IndexedFaceSetInfo()
{
  numLocalVertices=0;
  localVertexPositions = NULL;
  neutralLocalVertexPositions=NULL;
  updateMask=NULL;
  local2Global = NULL; 
}  

IndexedFaceSetInfo::~IndexedFaceSetInfo()
{
  delete[] localIndices;
  delete[] localVertexPositions;
  delete[] neutralLocalVertexPositions;
  delete[] updateMask;
  delete[] local2Global;
}


void IndexedFaceSetInfo::setNode(Coordinate *coordNode){
  partCoordNode = coordNode;  
}

void IndexedFaceSetInfo::setNeutralValues(){
  if(partCoordNode->point /* GetPoint() */ .GetLength()==0){
    numLocalVertices = 0;
  } else{
    numLocalVertices = partCoordNode->point /* GetPoint() */ .GetLength();
    delete[] neutralLocalVertexPositions;
    delete[] localVertexPositions;
    neutralLocalVertexPositions = new float[numLocalVertices][3];
    localVertexPositions = new float[numLocalVertices][3];
    int i;
    for (i=0; i<numLocalVertices;i++){
      neutralLocalVertexPositions[i][0] = partCoordNode->point /* GetPoint() */ [i].x;
      neutralLocalVertexPositions[i][1] = partCoordNode->point /* GetPoint() */ [i].y;
      neutralLocalVertexPositions[i][2] = partCoordNode->point /* GetPoint() */ [i].z;
    }
  }
}

void IndexedFaceSetInfo::makeUpdateMask(){
  updateMask = new int[numLocalVertices];
  initUpdateMask();
}

void IndexedFaceSetInfo::initUpdateMask(){
  for(int i=0; i<numLocalVertices;i++){
    updateMask[i]=FALSE;
  }
}

void IndexedFaceSetInfo::setLocalVerticesValues(){
  MF<Vec3f> local;
  for (int i=0; i<numLocalVertices;i++){
    Vec3f localtmp;
    localtmp.x=localVertexPositions[i][0];
    localtmp.y=localVertexPositions[i][1];
    localtmp.z=localVertexPositions[i][2];
    local.InsertAt(i,localtmp);
  }
  partCoordNode->point = (local);
}



void IndexedFaceSetInfo::initLocalVertexPositions(){
  for (int i=0; i<numLocalVertices;i++){
    for (int j=0;j<3;j++){
      localVertexPositions[i][j]=neutralLocalVertexPositions[i][j];
    }
  }
}

void IndexedFaceSetInfo::addVertexDisplacement(const int index, const float displacement[3]){
  if(updateMask[index]==FALSE){ 
    for(int j=0; j<3; j++){
      localVertexPositions[index][j]=neutralLocalVertexPositions[index][j] + displacement[j];
    }
    updateMask[index]=TRUE;
  }else{
    for(int j=0; j<3; j++){
      localVertexPositions[index][j]+=displacement[j];
    }    
  }
}


Coordinate *IndexedFaceSetInfo::getNode()
{
  return partCoordNode;
}
