/*
This software module was originally developed by 
Image Processing Software and Technology Research Department 5 of AT&T
in the course of development of the MPEG-4. 
This software module is an implementation of a part of one or 
more MPEG-4 tools as specified by the MPEG-4. 
ISO/IEC gives users of the MPEG-4 standard free license to this
software module or modifications thereof for use in hardware 
or software products claiming conformance to the MPEG-4 standard.
Those intending to use this software module in hardware or software
products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company,
the subsequent editors and their companies, and ISO/IEC have no
liability for use of this software module or modifications thereof
in an implementation.
Copyright is not released for non MPEG-4 standard conforming
products. AT&T retains full right to use the code for his/her own
purpose, assign or donate the code to a third party and to
inhibit third parties from using the code for non 
MPEG-4 standard conforming products.
This copyright notice must be included in all copies or
derivative works. Copyright (c) 1997, AT&T.
*/
#ifndef PARTINFO_H
#define PARTINFO_H

/*Class:         PartInfo, TransformInfo, IndexedFaceSetInfo                        
  Description:   classes to manipulate Transforms and IndexedFaceSets nodes
                 in a scene graph */

// #include <inttypes.h>

#include "Presenter.h"

// #include "group/Transform.h"
// #include "geometry/IndexedFaceSet.h"
#include "Nodes.h"

#include "TableIndexedFaceSet_mpeg.h"

class PartInfo //base class for TransformInfo and IndexedFaceSetInfo
{
public:

  PartInfo(){};
  ~PartInfo(){};
  void setName(int name); //sets the name of the node

  int getName(){return _name;}; //get the name of the node

private:

  int _name; //name of the node

};

class TransformInfo : public virtual PartInfo //class for the manipulation of a Transform node
{

public:
  TransformInfo();
  ~TransformInfo();

  void setNode(Transform *transformNode); //set the internal pointer to transformNode
  void setNeutralValues(const TransformSTable *transformTable); //set neutral values for the Transform node
  void setTransformValue(const float transformValue); //set a value in the Transform node; fieldRef specifies which field of the Transform node is accessed
  Transform *getNode(); //get a pointer to the Transform node, to which this class is pointing

private:

  Transform *node; //pointer to the Transform node
  int fieldRef;
  SF<Rotation> neutralRotation;
  SF<Rotation> updated_rotation;
  SF<Vec3f> neutralTranslationOrScalefactor; //neutral scale factor or neutral translation vector
  SF<Vec3f> updated_translationOrScalefactor;

};


class IndexedFaceSetInfo : public virtual PartInfo //class for the manipulation of an Coordinate node of an IndexedFaceSet
{

public:

  IndexedFaceSetInfo();
  ~IndexedFaceSetInfo();
//  void setName(const int name);
//  void setValues(const int numGlobalVertices, 
//		 const SoMFVec3f *globalVertexPositions, 
//		 const int numIndices, const int32_t *globalIndices); //uses globalVertexPositions and globalIndices to determine the relationship between globalIndices and localIndices (local2Global) and set the coordinates for the local vertices in localVertexPositions 
  void setNode(Coordinate *coordNode); //set the internal Coordinate pointer to coordNode
  void setNeutralValues(); //pass the coordinates of the partCoordNode to neutralLocalVertexPositions
  void makeUpdateMask(); //make a mask for all vertices in localVertexPositions and set all mask values FALSE; if a displacement is added to a vertex in addVertexDisplacement, then the mask for this vertex is set TRUE
  void initUpdateMask(); //set all values for updateMask to FALSE
  void setLocalVerticesValues(); //pass the values from localVertexPositions to partCoordNode
  void initLocalVertexPositions(); //pass the values in neutralLocalVertexPositions to localVertexPositions
  void addVertexDisplacement(const int index, const float displacement[3]); //add a displacents to the vertex in localVertexPositions referenced by index
//  void setGlobalVertexPositions(SoMFVec3f *globalVertexPositions, int & no_vert); //pass the values of partCoordNode to globalVertexPositions; the arrays globalVertexPositions and local2Global are enlarged if necessary
//  void updateGlobalVertexPositions(float (*globalVertexPositions)[3]); //pass the values of partCoordNode to globalVertexPositions; the arrays globalVertexPositions and local2Global are not modified
//  void readAlias(const char *filename, const char *partName, 
//		 const int no_vert, SoMFVec3f *globalVertexPositions); //old

//  int32_t *getLocalIndices(); //returns pointer to localIndices
//  float **getLocalVertexPositions();
//  void update(const SoMFVec3f *globalVertexPositions); //pass the values in globalVertexPositions to partCoordNode
//  int getNumLocalVertices();
  Coordinate *getNode(); //returns pointer to partCoordNode

private:

  
  int numLocalVerticesInit;//number of vertices in partCoordNode after setValues has been called
  int numLocalVertices; //actual number of vertices in partCoordNode; may be changed in setNeutralValues
  int *localIndices; //all global indices for vertices stored in partCoordNode are stored here
  float (*localVertexPositions)[3]; //array for the vertex positions of the IndexedFaceSet
  float (*neutralLocalVertexPositions)[3]; //array for the neutral vertex positions of the IndexedFaceSet
  int *updateMask; //mask for each vertex in localVertexPositions, which says whether a displacement has been added to a specific mask
  int *local2Global; //for each vertex stored in localVertexPositions, the global index is stored here
  Coordinate *partCoordNode; //pointer to the Coordinate node of the IndexedFaceSet
};

#endif
