/*
This software module was originally developed by 
Image Processing Software and Technology Research Department 5 of AT&T
in the course of development of the MPEG-4. 
This software module is an implementation of a part of one or 
more MPEG-4 tools as specified by the MPEG-4. 
ISO/IEC gives users of the MPEG-4 standard free license to this
software module or modifications thereof for use in hardware 
or software products claiming conformance to the MPEG-4 standard.
Those intending to use this software module in hardware or software
products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company,
the subsequent editors and their companies, and ISO/IEC have no
liability for use of this software module or modifications thereof
in an implementation.
Copyright is not released for non MPEG-4 standard conforming
products. AT&T retains full right to use the code for his/her own
purpose, assign or donate the code to a third party and to
inhibit third parties from using the code for non 
MPEG-4 standard conforming products.
This copyright notice must be included in all copies or
derivative works. Copyright (c) 1997, AT&T.
*/
#include "Presenter.h"
#include "TableIndexedFaceSet_mpeg.h"
#include <stdio.h>

void VertexDisplacementsInfo::setValues(const int vertexIndex, 
				    const int nIntervals, 
				    const MF<Vec3f> displacements)
{
  _nIntervals=nIntervals;
  _vertexIndex = vertexIndex;
  int i;
  for(i=0;i<nIntervals;i++){
	   _displacements.InsertAt(i,displacements[i]);
	   
  }
}

int VertexDisplacementsInfo::getVertexIndex() const{
  return _vertexIndex;
}

const Vec3f VertexDisplacementsInfo::getValues(const int index) const{
  return _displacements[index];
}

VertexDisplacementsInfo & VertexDisplacementsInfo::operator = (const VertexDisplacementsInfo &v){
  _vertexIndex = v._vertexIndex;
  _nIntervals = v._nIntervals;
  int i;
  for(i=0;i<_nIntervals;i++){
    _displacements.InsertAt(i,v._displacements[i]);
  }
  return *this;
}

/*TransformSTable & TransformSTable::operator = (const TransformSTable &t)
{
  setName(t.getName());
  _fieldRef = t._fieldRef;

  if (_fieldRef==2){    //AnimDefTransform defines rotation
     SbVec3f axis;
     float angle;
     t._rotation.getValue(axis, angle);
     _rotation.setValue(axis, angle);
  }
  else {   //AnimDefTransform defines translation or scaling
     _translationOrScalefactor.setValue(t._translationOrScalefactor.getValue());
  }

  return *this;
}*/


void TransformSTable::setTranslationOrScalefactor(const SF<Vec3f> translationOrScalefactor)
{
  _translationOrScalefactor=translationOrScalefactor;
}
  
IndexedFaceSetSTable::IndexedFaceSetSTable()
{
  _intervalBorders = NULL;
  _vertexDisplacementsTable = NULL;
  _nVertices = _nIntervals = 0;
}

IndexedFaceSetSTable::IndexedFaceSetSTable(const int nVertices, const int nIntervals, int *intervalBorders)
{
  _nVertices = nVertices;
  _nIntervals = nIntervals;
  _intervalBorders = new int[nIntervals+1];
  _vertexDisplacementsTable = new VertexDisplacementsInfo[nVertices];
  for(int i=0; i<nIntervals+1;i++){
    _intervalBorders[i]= intervalBorders[i];
  }
}

IndexedFaceSetSTable & IndexedFaceSetSTable::operator = (const IndexedFaceSetSTable &t)
{
  setName(t.getName());
  delete[] _intervalBorders;
  delete[] _vertexDisplacementsTable;
  _nVertices = t._nVertices;
  _nIntervals = t._nIntervals;
  _intervalBorders = new int[_nIntervals+1];
  _vertexDisplacementsTable = new VertexDisplacementsInfo[_nVertices];
  int i;
  for(i=0; i<_nIntervals+1;i++){
    _intervalBorders[i]= t._intervalBorders[i];
  }
  for(i=0;i<_nVertices;i++){
    _vertexDisplacementsTable[i] = t._vertexDisplacementsTable[i];
  }
  return *this;
}

/*void TableIndexedFaceSet::setIntervalBorders(const int *intervalBorders)
{
  int i;
  for (i=0; i<nIntervals+1;i++){
    _intervalBorders[i]= intervalBorders[i];
  }
}*/

void IndexedFaceSetSTable::set1VertexDisplacements(const int index, const VertexDisplacementsInfo & displacements)
{
  _vertexDisplacementsTable[index] = displacements;
}
  
int IndexedFaceSetSTable::getNumVertices() const
{
  return _nVertices;
}

int IndexedFaceSetSTable::getNumIntervals() const
{
  return _nIntervals;
}

const int *IndexedFaceSetSTable::getIntervalBorders() const
{
  return _intervalBorders;
}

const VertexDisplacementsInfo *IndexedFaceSetSTable::getValue(const int index) const{
  return _vertexDisplacementsTable+index;
}

IndexedFaceSetSTable::~IndexedFaceSetSTable()
{
  delete[] _intervalBorders;
  delete[] _vertexDisplacementsTable;
}

void AnimationDefinitionTable:: set1Value(const int index, const IndexedFaceSetSTable  & newValue)
{
  if (index > (maxNum-1)){
    allocValues(index+1);
  }
  tables[index]=newValue;
}

// SB 98/12/18 ????
void AnimationDefinitionTable:: set1Value(
    const int index, /* const */ TransformSTable  & newValue)
{
  if (index > (maxNumTransformTables-1)){
    allocValuesTransformTables(index+1);
  }
  transformTables[index]=newValue;
}

const IndexedFaceSetSTable *AnimationDefinitionTable:: getIndexedFaceSetSTable(const int index) const
{
  return tables+index;
}

const TransformSTable *AnimationDefinitionTable:: getTransformSTable(const int index) const
{
  return transformTables+index;
}

void AnimationDefinitionTable::allocValues(const int num)
{
  IndexedFaceSetSTable *tablesOld = tables;
  tables = new IndexedFaceSetSTable[num];
  for (int i=0; i<maxNum;i++) tables[i]=tablesOld[i];
  delete[] tablesOld;
  maxNum = num;
  nNodes = maxNum + maxNumTransformTables;
}

void AnimationDefinitionTable::allocValuesTransformTables(const int num)
{
  TransformSTable *tablesOld = transformTables;
  transformTables = new TransformSTable[num];
  for (int i=0; i<maxNumTransformTables;i++) transformTables[i]=tablesOld[i];
  delete[] tablesOld;
  maxNumTransformTables = num;
  nNodes = maxNum + maxNumTransformTables;
}






