/*
This software module was originally developed by 
Image Processing Software and Technology Research Department 5 of AT&T
in the course of development of the MPEG-4. 
This software module is an implementation of a part of one or 
more MPEG-4 tools as specified by the MPEG-4. 
ISO/IEC gives users of the MPEG-4 standard free license to this
software module or modifications thereof for use in hardware 
or software products claiming conformance to the MPEG-4 standard.
Those intending to use this software module in hardware or software
products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company,
the subsequent editors and their companies, and ISO/IEC have no
liability for use of this software module or modifications thereof
in an implementation.
Copyright is not released for non MPEG-4 standard conforming
products. AT&T retains full right to use the code for his/her own
purpose, assign or donate the code to a third party and to
inhibit third parties from using the code for non 
MPEG-4 standard conforming products.
This copyright notice must be included in all copies or
derivative works. Copyright (c) 1997, AT&T.
*/
#ifndef TableIndexedFaceSet_H
#define TableIndexedFaceSet_H

/* Class:   VetexDisplacementsInfo, STable, TransformSTable, 
            IndexedFaceSetSTable, AnimationDefinitionTable    */                 

#include <string.h>

class VertexDisplacementsInfo { //class for storing the displacements for a vertex

public:

  VertexDisplacementsInfo(){ _vertexIndex = 0; _nIntervals = 0;};
  //sets the index and the displacements for the vertex; nIntervals is the number of intervals
  void setValues(const int vertexIndex, const int nIntervals, const MF<Vec3f> displacements); 
  //sets the index and the displacements for the vertex; nIntervals is the number of intervals
  int getVertexIndex() const; //return the index of the vertex
  const Vec3f getValues(const int index) const; //returns a pointer into the array of displacements in the field, starting at index index
  VertexDisplacementsInfo & operator = (const VertexDisplacementsInfo &v); //assignment operator
  ~VertexDisplacementsInfo(){}

private:
  int _vertexIndex; //index of the vertex
  int _nIntervals; //number of intervals for the displacements
  MF<Vec3f> _displacements; //array of displacements
};


class STable //base class for TransformsTable and IndexedFaceSetSTable
{
public:
  void setName(const int NodeId) {nodeId=NodeId;}; //set the name
  int getName() const {return nodeId;}; //get the name
  STable & operator = (const STable &t){nodeId=t.nodeId;
                                        return *this;}; //assignment operator
private:
  
  int nodeId; //name of the table
};

class TransformSTable : public STable //class for AnimationDefTransform
{

public:
  TransformSTable() {_fieldRef = -1;};
 // TransformSTable &operator = (const TransformSTable &t); //assignment operator
  void setFieldRef(const int fieldRef){_fieldRef = fieldRef;}; //set the field reference
  void setRotation(const SF<Rotation> rotation) {_rotation=rotation;}		
  void setTranslationOrScalefactor(const SF<Vec3f> TranslationOrScalefactor);		   
  int getFieldRef() const{return _fieldRef;}; //get the field reference
  const SF<Vec3f>& getTranslationOrScalefactor() const {return _translationOrScalefactor;};
  const SF<Rotation>& getRotation() const {return _rotation;};
  ~TransformSTable(){};
  
private:
  int _fieldRef;
  SF<Rotation> _rotation;
  SF<Vec3f> _translationOrScalefactor;
};
    

class IndexedFaceSetSTable : public STable //class for AnimationDefIndexedFaceSet
{

public:

  IndexedFaceSetSTable(const int nVertices, const int nIntervals, int *intervalBorders);
  IndexedFaceSetSTable();
  IndexedFaceSetSTable & operator = (const IndexedFaceSetSTable &t); //assignment operator
  //  void setIntervalBorders(const int *intervalBorders);
  void set1VertexDisplacements(const int index, const VertexDisplacementsInfo & displacements); //set the indexth value to the given displacements
  int getNumVertices() const; //get the number of vertices, for which displacements are stored
  int getNumIntervals() const; //get the number of intervals
  const VertexDisplacementsInfo *getValue(const int index) const; //get the value of the indexth displacement
  const int *getIntervalBorders() const; //get the interval borders
  ~IndexedFaceSetSTable();


private:

  int _nVertices; //number of vertices, for which displacements are given
  int _nIntervals; //number of intervals
  int *_intervalBorders; //interval borders
  VertexDisplacementsInfo *_vertexDisplacementsTable; //vertex displacments

};

class AnimationDefinitionTable //class for AnimationDefinitionTable
{

public:
  AnimationDefinitionTable(){tables = NULL; maxNum = 0; transformTables=NULL;
  maxNumTransformTables = 0; nNodes = 0;};
  //  void setValues(int start, int num, const IndexedFaceSetSTable *newValues);
  void set1Value(const int index, const IndexedFaceSetSTable & newValue); //set the indexth IndexFaceSetTable to have the given newValue
  void set1Value(const int index, /* const */ TransformSTable &newValue); //set the indexth TransformTable to have the indexth newValue
  const TransformSTable *getTransformSTable(const int index) const; //return a pointer into the array of TransformDefTables, starting at index
  const IndexedFaceSetSTable *getIndexedFaceSetSTable(const int index) const; //return a pointer into the array of IndexedFaceSetDefTables starting at index
  ~AnimationDefinitionTable(){delete[] tables; delete[] transformTables;};
  int getNumNodes() const {return nNodes;}; //get total number of TransformDefTables and IndexedFaceSetDefTables
  int getNumTransformTables()const { return maxNumTransformTables;}; //
  int getNumIndexedFaceSetTables()const{return maxNum;};

private:

  IndexedFaceSetSTable *tables; //array of IndexedFaceSetDefTables
  TransformSTable *transformTables; //array of TransformDefTables
  int maxNum; //number of IndexedFaceSetDefTables
  int maxNumTransformTables; //number TransformDefTables
  void allocValuesTransformTables(const int num); //allocate resources for num TransformDefTables
  void allocValues(const int num); //allocate resources for num IndexedFaceSetDefTables
  int nNodes; //total number of TransformDefTables and IndexedFaceSetDefTables
};



#endif
