// This software module was originally developed by Igor Pandzic (MIRALab,
// University of Geneva) and Karl Anders ygard (Telenor R&D) in the course
// of development of the MPEG-4 Systems standard (ISO/IEC 14496-1).  This
// software module is an implementation of a part of one or more MPEG-4
// Systems tools as specified by the MPEG-4 Systems standard.  ISO/IEC
// gives users of the MPEG-4 Systems standard free license to this software
// module or modifications thereof for use in hardware or software products
// claiming conformance to the MPEG-4 Systems standard.  Those intending to
// use this software module in hardware or software products are advised
// that its use may infringe existing patents.  The original developer of
// this software module and his/her company, the subsequent editors and
// their companies, and ISO/IEC have no liability for use of this software
// module or modifications thereof in an implementation.  Copyright is not
// released for non MPEG-4 Systems conforming products.  The original
// developer retains full right to use the code for his/her own purpose,
// assign or donate the code to a third party and to inhibit third parties
// from using the code for non MPEG-4 Systems conforming products.  This
// copyright notice must be included in all copies or derivative works.
//
// For more information or to receive an up to date version of this module,
// mail karlo@nta.no or check <http://televr.fou.telenor.no/compositor/>.
//
// Copyright 1997.

#include <assert.h>

#include "defs.h"

#include "FbaClass.h"
#include "thread_utils.h"

/*******************************************************************************
the entry point to the decoder DLL
*******************************************************************************/

void * CreateInstance()
{
  TRACE("[FBA] CreateInstance \n");
  return new FbaClass;
}

/*-----------------------------------------------------------------
Find a compatible format and initialize the decoder
-----------------------------------------------------------------*/

MediaFormat FbaClass :: SetFormat (Capabilities *)
{
  return MediaFormat :: FBA;
}

/*-----------------------------------------------------------------
Calculate the output size in bytes
-----------------------------------------------------------------*/

int FbaClass :: GetOptimalOutputSize ()
{
	return sizeof(FOP_DATA);
}

/* virtual */
BOOL FbaClass::Init()
{
  TRACE("[FBA::Init] \n");

  init();

  return TRUE;
}

/* virtual */
void FbaClass::Run()
{
  TRACE("[FBA::Run] \n");

  //SetThreadPriority(GetCurrentThread(), THREAD_PRIORITY_ABOVE_NORMAL);
  Set_Priority( THREAD_PRIORITY_ABOVE_NORMAL);
  DecoderImp::Run();
}

BOOL FbaClass :: Decode (
		LPBYTE pucInput,
		int iInputLength,
		LPBYTE pucOutput,
		int & riOutputLength)
{
  TRACE("[FBA::Decode] buffer %08x \n",
    pucInput);

	unsigned int uiPts = 0;

  fba_decoder(
    (unsigned char*) pucInput, 
    0, 
    iInputLength, 
    &moFopData, 
    &moBopData, 
    sizeof(FOP_DATA) + sizeof(BOP_DATA), 
    &uiPts);

  TRACE("[FBA::Decode] decoded %4d bytes with PTS %8d \n",
    iInputLength, uiPts);

	memcpy(pucOutput, &moFopData, GetOptimalOutputSize ());
	riOutputLength = GetOptimalOutputSize (); 
	return TRUE;
}

