/*************************************************************************

This software module was originally developed by

        Wei Wu (weiwu@stallion.risc.rockwell.com),  Rockwell Science Center

in the course of development of the MPEG-4 Video (ISO/IEC 14496-2).
This software module is an implementation of a part of one or more MPEG-4
Video tools as specified by the MPEG-4 Video. ISO/IEC gives users of the
MPEG-4 Video free license to this software module or modifications thereof
for use in hardware or software products claiming conformance to the MPEG-4
Video. Those intending to use this software module in hardware or software
products are advised that its use may infringe existing patents. The original
developer of this software module and his/her company, the subsequent editors
and their companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation. Copyright is not
released for non MPEG-4 Video conforming products. Rockwell Science Center
retain full right to use the code for his/her own purpose, assign
or donate  the code to a third party and to inhibit third parties from using the
code  for non <MPEG standard> conforming products. This copyright notice must be

included in all copies or derivative works. Copyright (c) 1998.

*************************************************************************/


#include <math.h>
#include "sim.h"

#define SHORT_MIN	-32768
#define SHORT_MAX	32767
#define PI		3.14159265358979323846

static float DCTQuantSteps[NFAP] = { 1, 1, 7.5, 7.5, 7.5, 7.5, 7.5, 7.5, 7.5, 7.5,
                                     7.5, 7.5, 7.5, 15, 15, 15, 15, 5, 10,10,
                                     10, 10, 425, 425, 425, 425, 5, 5, 5, 5,
                                     7.5, 7.5, 7.5, 7.5, 7.5, 7.5, 7.5, 7.5, 20, 20,
                                     20, 20, 10, 10, 10, 10, 255, 170, 255, 255,
                                     7.5, 7.5, 7.5, 7.5, 7.5, 7.5, 7.5, 7.5, 7.5, 7.5,
                                     15, 15, 15, 15, 10, 10, 10, 10 };
static int fap_scale[32] = { 1, 1, 2, 3, 5, 7, 8, 10, 12, 15, 18, 21, 25, 30, 35, 42,
                      50, 60, 72, 87, 105, 128, 156, 191, 234, 288, 355, 439, 543,
                                          674, 836, 1039 };

void
DCT_initialize() {
    int i;

    fop.fo.dctQP = (float *) malloc(NFAP*sizeof(float));
    init_dctQP();
    fop.fo.faps = (float **) malloc(NFAP*sizeof(float*));
    for (i=0; i<NFAP; i++) {
        fop.fo.faps[i] = (float *) malloc(SEG_LENGTH*sizeof(float));
    }
    fop.fo.visemes = (int **) malloc(4*sizeof(int*));
    for (i=0; i<4; i++) {
        fop.fo.visemes[i] = (int*) malloc((SEG_LENGTH+1)*sizeof(int));
    }
    fop.fo.expressions = (int **) malloc(6*sizeof(int*));
    for (i=0; i<6; i++) {
        fop.fo.expressions[i] = (int*) malloc((SEG_LENGTH+1)*sizeof(int));
    }
    fop.fo.dcPred = (float*) malloc(NFAP*sizeof(float));
    init_htables();
    fop.fo.idctMatrix = (float**) malloc(SEG_LENGTH*sizeof(float*));
    for (i=0; i<SEG_LENGTH; i++) {
        fop.fo.idctMatrix[i] = (float*) malloc(SEG_LENGTH*sizeof(float));
    }
    init_dct();
}

void
init_dctQP() {
  int i;
  int quant = fop.fo.fap_quant;

  for (i=0; i<NFAP; i++) {
	fop.fo.dctQP[i] = fap_scale[quant]*DCTQuantSteps[i];
  }
}

void
init_htables() {
  int i, j, n, m;
  FILE *ifp;

  ifp = fopen("dc.table", "r");
  fscanf(ifp, "%d", &n);
  fop.fo.dc_table_size = n;
  fop.fo.dc_table = (int**) malloc(n*sizeof(int*));
  for (i=0; i<n; i++) {
	fscanf(ifp, "%d", &m);
	fop.fo.dc_table[i] = (int*) malloc((m+1)*sizeof(int));
	fop.fo.dc_table[i][0] = m;
	for (j=1; j<=m; j++) {
		fscanf(ifp, "%d", &fop.fo.dc_table[i][j]);
	}
  }
  fclose(ifp);

  ifp = fopen("runlength.table", "r");
  fscanf(ifp, "%d", &n);
  fop.fo.runlength_table_size = n;
  fop.fo.runlength_table = (int**) malloc(n*sizeof(int*));
  for (i=0; i<n; i++) {
	fscanf(ifp, "%d", &m);
	fop.fo.runlength_table[i] = (int*) malloc((m+1)*sizeof(int));
	fop.fo.runlength_table[i][0] = m;
	for (j=1; j<=m; j++) {
		fscanf(ifp, "%d", &fop.fo.runlength_table[i][j]);
	}
  }
  fclose(ifp);

  ifp = fopen("ac.table", "r");
  fscanf(ifp, "%d", &n);
  fop.fo.ac_table_size = n;
  fop.fo.ac_table = (int**) malloc(n*sizeof(int*));
  for (i=0; i<n; i++) {
	fscanf(ifp, "%d", &m);
	fop.fo.ac_table[i] = (int*) malloc((m+1)*sizeof(int));
	fop.fo.ac_table[i][0] = m;
	for (j=1; j<=m; j++) {
		fscanf(ifp, "%d", &fop.fo.ac_table[i][j]);
	}
  }
  fclose(ifp);

  ifp = fopen("v_sel.table", "r");
  fscanf(ifp, "%d", &n);
  fop.fo.v_sel_table_size = n;
  fop.fo.v_sel_table = (int**) malloc(n*sizeof(int*));
  for (i=0; i<n; i++) {
	fscanf(ifp, "%d", &m);
	fop.fo.v_sel_table[i] = (int*) malloc((m+1)*sizeof(int));
	fop.fo.v_sel_table[i][0] = m;
	for (j=1; j<=m; j++) {
		fscanf(ifp, "%d", &fop.fo.v_sel_table[i][j]);
	}
  }
  fclose(ifp);

  ifp = fopen("v_blend.table", "r");
  fscanf(ifp, "%d", &n);
  fop.fo.v_blend_table_size = n;
  fop.fo.v_blend_table = (int**) malloc(n*sizeof(int*));
  for (i=0; i<n; i++) {
	fscanf(ifp, "%d", &m);
	fop.fo.v_blend_table[i] = (int*) malloc((m+1)*sizeof(int));
	fop.fo.v_blend_table[i][0] = m;
	for (j=1; j<=m; j++) {
		fscanf(ifp, "%d", &fop.fo.v_blend_table[i][j]);
	}
  }
  fclose(ifp);

  ifp = fopen("e_sel.table", "r");
  fscanf(ifp, "%d", &n);
  fop.fo.e_sel_table_size = n;
  fop.fo.e_sel_table = (int**) malloc(n*sizeof(int*));
  for (i=0; i<n; i++) {
	fscanf(ifp, "%d", &m);
	fop.fo.e_sel_table[i] = (int*) malloc((m+1)*sizeof(int));
	fop.fo.e_sel_table[i][0] = m;
	for (j=1; j<=m; j++) {
		fscanf(ifp, "%d", &fop.fo.e_sel_table[i][j]);
	}
  }
  fclose(ifp);

  ifp = fopen("e_intensity.table", "r");
  fscanf(ifp, "%d", &n);
  fop.fo.e_intensity_table_size = n;
  fop.fo.e_intensity_table = (int**) malloc(n*sizeof(int*));
  for (i=0; i<n; i++) {
	fscanf(ifp, "%d", &m);
	fop.fo.e_intensity_table[i] = (int*) malloc((m+1)*sizeof(int));
	fop.fo.e_intensity_table[i][0] = m;
	for (j=1; j<=m; j++) {
		fscanf(ifp, "%d", &fop.fo.e_intensity_table[i][j]);
	}
  }
  fclose(ifp);
}

void
init_dct() {
  int i,j;
  float c;
 
  for(i=0; i<SEG_LENGTH; i++) {
    if(i==0) c = (float)(1.0/sqrt(2));
    else c = 1;
    for(j=0; j<SEG_LENGTH; j++) {
	fop.fo.idctMatrix[j][i] = c * (float)(sqrt(2.0/SEG_LENGTH))
				* (float)(cos(PI*(float)(2*j+1)*i/2.0/SEG_LENGTH));
    }
  }

}

void
idctTrans(float *src, float *dest) {
  int i, j;

  for (i=0; i<SEG_LENGTH; i++) {
    dest[i] = 0;
    for (j=0; j<SEG_LENGTH; j++) {
	dest[i] += fop.fo.idctMatrix[i][j]*src[j];
    }
  }
}

void
dctdequant(int *src, float *dest) {
  int i;

  dest[0] = src[0] * (fop.fo.dctQP[0]/3.0);

  for (i=1; i<SEG_LENGTH; i++) {
	dest[i] = src[i] * fop.fo.dctQP[i];
  }
}

int
getIDC() {
  int symbol;
  symbol = getbits(16);
  if (symbol == SHORT_MAX+1) {
	symbol = getbits(32);
  } else if (symbol > SHORT_MAX) {
		symbol -= 65536;
  }
  return symbol;
}

int
getPDC() {
  int symbol;
  symbol = huffman_decode(fop.fo.dc_table, fop.fo.dc_table_size);
  if (symbol==0) {
	symbol = getIDC();
  } else {
	symbol -= 256;
  }
  return symbol;
}

void
getAC(int *coef) {
  int i, n;
  int symbol;

  for (i=1; i<SEG_LENGTH; ) {
	symbol = huffman_decode(fop.fo.runlength_table, fop.fo.runlength_table_size);
	if (symbol==0) {
		for(n=i; n<SEG_LENGTH; n++) {
			coef[n] = 0;
		}
		return;
	}
	for (n=0; n<symbol-1; n++) {
		coef[i++] = 0;
	}
	symbol = huffman_decode(fop.fo.ac_table, fop.fo.ac_table_size);
	if (symbol==256) {
		coef[i++] = getIDC();
	} else {
		coef[i++] = symbol-256;
	}
  }
}

int
huffman_decode(int **table, int size) {
  int i, n;
  int bit;
  char *valid;

  valid = (char *) malloc(size);
  for (i=0; i<size; i++) {
	valid[i] = 1;
  }
  
  n = 1;
  while(n<=30) {
    bit = getbits(1);
    for (i=0; i<size; i++) {
      if(valid[i]) {
  	  if (bit == table[i][n]) {
		if (table[i][0] == n) {
			return i;
		}
	  } else {
		valid[i] = 0;
	  }
      }
    }
    n++;
  }
  fprintf(stderr, "Error: 30 bits has been read and huffman decoder \
still cannot find a symbol\n");
  exit(-1);
}

void 
write1Segment(int curFrame, FILE *fapfile) {
  int i, j;

  for (i=0; i<SEG_LENGTH; i++) {
    for (j=0; j<NFAP; j++) {
	fprintf(fapfile, "%d ", fop.fo.fap_mask[j]);
    }
    fprintf(fapfile, "\n");
    fprintf(fapfile, "%d ", curFrame);
    if(fop.fo.fap_mask[0]) {
	fprintf(fapfile, "%d %d %d %d ", fop.fo.visemes[0][i],
		fop.fo.visemes[1][i], fop.fo.visemes[2][i],
		fop.fo.visemes[3][i]);
    }
    if(fop.fo.fap_mask[1]) {
	fprintf(fapfile, "%d %d %d %d %d %d ", fop.fo.expressions[0][i],
		fop.fo.expressions[1][i], fop.fo.expressions[2][i],
		fop.fo.expressions[3][i], fop.fo.expressions[4][i],
		fop.fo.expressions[5][i]);
    }
    for (j=2; j<NFAP; j++) {
	if(fop.fo.fap_mask[j]) {
		fprintf(fapfile, "%d ", (int) fop.fo.faps[j][i]);
	}
    }
    fprintf(fapfile, "\n");
    curFrame++;
  }
}
