/*************************************************************************

This software module was originally developed by

        Wei Wu (weiwu@stallion.risc.rockwell.com),  Rockwell Science Center

in the course of development of the MPEG-4 Video (ISO/IEC 14496-2).
This software module is an implementation of a part of one or more MPEG-4
Video tools as specified by the MPEG-4 Video. ISO/IEC gives users of the
MPEG-4 Video free license to this software module or modifications thereof
for use in hardware or software products claiming conformance to the MPEG-4
Video. Those intending to use this software module in hardware or software
products are advised that its use may infringe existing patents. The original
developer of this software module and his/her company, the subsequent editors
and their companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation. Copyright is not
released for non MPEG-4 Video conforming products. Rockwell Science Center
retain full right to use the code for his/her own purpose, assign
or donate  the code to a third party and to inhibit third parties from using the
code  for non <MPEG standard> conforming products. This copyright notice must be

included in all copies or derivative works. Copyright (c) 1998.

*************************************************************************/

#include "sim.h"

void decode_i_segment()
{

  int i;
  float dctCoef[SEG_LENGTH];
  int dctQCoef[SEG_LENGTH];

  if (fop.fo.fap_mask[0])
	decode_visemes_segment(1);
  if (fop.fo.fap_mask[1])
	decode_expressions_segment(1);
  for (i=2; i<NFAP; i++) {
	if (fop.fo.fap_mask[i]) {
		dctQCoef[0] = getIDC();	
		getAC(dctQCoef);
		dctdequant(dctQCoef, dctCoef);
		fop.fo.dcPred[i] = dctCoef[0];
		idctTrans(dctCoef, fop.fo.faps[i]);
	}
  }
}

void decode_p_segment()
{

  int i;
  float dctCoef[SEG_LENGTH];
  int dctQCoef[SEG_LENGTH];

  if (fop.fo.fap_mask[0])
	decode_visemes_segment(0);
  if (fop.fo.fap_mask[1])
	decode_expressions_segment(0);
  for (i=2; i<NFAP; i++) {
	if (fop.fo.fap_mask[i]) {
		dctQCoef[0] = getPDC();
		getAC(dctQCoef);
		dctdequant(dctQCoef, dctCoef);
		dctCoef[0] += fop.fo.dcPred[i];
		fop.fo.dcPred[i] = dctCoef[0];
		idctTrans(dctCoef, fop.fo.faps[i]);
	}
  }
}

void decode_visemes_segment(int is_intra)
{
  int i;
  int symbol;

  if (is_intra) {
	fop.fo.visemes[0][0] = getbits(4);
	fop.fo.visemes[1][0] = getbits(4);
	fop.fo.visemes[2][0] = getbits(6);
	fop.fo.visemes[3][0] = getbits(1);
  } else {
	symbol = huffman_decode(fop.fo.v_sel_table, fop.fo.v_sel_table_size);
	fop.fo.visemes[0][0] = symbol + fop.fo.visemes[0][SEG_LENGTH]-14;
	symbol = huffman_decode(fop.fo.v_sel_table, fop.fo.v_sel_table_size);
	fop.fo.visemes[1][0] = symbol + fop.fo.visemes[1][SEG_LENGTH]-14;
	symbol = huffman_decode(fop.fo.v_blend_table, fop.fo.v_blend_table_size);
	fop.fo.visemes[2][0] = symbol + fop.fo.visemes[2][SEG_LENGTH]-63;
	fop.fo.visemes[3][0] = getbits(1);
  }

  for (i=1; i<SEG_LENGTH; i++) {
	symbol = huffman_decode(fop.fo.v_sel_table, fop.fo.v_sel_table_size);
	fop.fo.visemes[0][i] = symbol + fop.fo.visemes[0][i-1]-14;
	symbol = huffman_decode(fop.fo.v_sel_table, fop.fo.v_sel_table_size);
	fop.fo.visemes[1][i] = symbol + fop.fo.visemes[1][i-1]-14;
	symbol = huffman_decode(fop.fo.v_blend_table, fop.fo.v_blend_table_size);
	fop.fo.visemes[2][i] = symbol + fop.fo.visemes[2][i-1]-63;
	fop.fo.visemes[3][i] = getbits(1);
  }

	fop.fo.visemes[0][SEG_LENGTH] = fop.fo.visemes[0][SEG_LENGTH-1];
	fop.fo.visemes[1][SEG_LENGTH] = fop.fo.visemes[1][SEG_LENGTH-1];
	fop.fo.visemes[2][SEG_LENGTH] = fop.fo.visemes[2][SEG_LENGTH-1];
}

void decode_expressions_segment(int is_intra)
{
  int i;
  int symbol;

  if (is_intra) {
	fop.fo.expressions[0][0] = getbits(4);
	fop.fo.expressions[1][0] = getbits(4);
	fop.fo.expressions[2][0] = getbits(6);
	fop.fo.expressions[3][0] = getbits(6);
	fop.fo.expressions[4][0] = getbits(1);
	fop.fo.expressions[5][0] = getbits(1);
  } else {
	symbol = huffman_decode(fop.fo.e_sel_table, fop.fo.e_sel_table_size);
	fop.fo.expressions[0][0] = symbol + fop.fo.expressions[0][SEG_LENGTH]-6;
	symbol = huffman_decode(fop.fo.e_sel_table, fop.fo.e_sel_table_size);
	fop.fo.expressions[1][0] = symbol + fop.fo.expressions[1][SEG_LENGTH]-6;
	symbol = huffman_decode(fop.fo.e_intensity_table, fop.fo.e_intensity_table_size);
	fop.fo.expressions[2][0] = symbol + fop.fo.expressions[2][SEG_LENGTH]-63;
	symbol = huffman_decode(fop.fo.e_intensity_table, fop.fo.e_intensity_table_size);
	fop.fo.expressions[3][0] = symbol + fop.fo.expressions[3][SEG_LENGTH]-63;
	fop.fo.expressions[4][0] = getbits(1);
	fop.fo.expressions[5][0] = getbits(1);
  }

  for (i=1; i<SEG_LENGTH; i++) {
	symbol = huffman_decode(fop.fo.e_sel_table, fop.fo.e_sel_table_size);
	fop.fo.expressions[0][i] = symbol + fop.fo.expressions[0][i-1]-6;
	symbol = huffman_decode(fop.fo.e_sel_table, fop.fo.e_sel_table_size);
	fop.fo.expressions[1][i] = symbol + fop.fo.expressions[1][i-1]-6;
	symbol = huffman_decode(fop.fo.e_intensity_table, fop.fo.e_intensity_table_size);
	fop.fo.expressions[2][i] = symbol + fop.fo.expressions[2][i-1]-63;
	symbol = huffman_decode(fop.fo.e_intensity_table, fop.fo.e_intensity_table_size);
	fop.fo.expressions[3][i] = symbol + fop.fo.expressions[3][i-1]-63;
	fop.fo.expressions[4][i] = getbits(1);
	fop.fo.expressions[5][i] = getbits(1);
  }

	fop.fo.expressions[0][SEG_LENGTH] = fop.fo.expressions[0][SEG_LENGTH-1];
	fop.fo.expressions[1][SEG_LENGTH] = fop.fo.expressions[1][SEG_LENGTH-1];
	fop.fo.expressions[2][SEG_LENGTH] = fop.fo.expressions[2][SEG_LENGTH-1];
	fop.fo.expressions[3][SEG_LENGTH] = fop.fo.expressions[3][SEG_LENGTH-1];
}
