/*************************************************************************

This software module was originally developed by 
	Wei Wu (weiwu@stallion.risc.rockwell.com),  Rockwell Science Center 
        Tolga K. Capin (capin@lig.di.epfl.ch), EPFL-LIG,
        Joaquim Esmerado (jle@lig.di.epfl.ch), EPFL-LIG.
and modified by
        Igor Pandzic (Igor.Pandzic@cui.unige.ch), MIRALab, University of Geneva

in the course of development of the MPEG-4 Video (ISO/IEC 14496-2). 
This software module is an implementation of a part of one or more MPEG-4 Video
tools as specified by the MPEG-4 Video. ISO/IEC gives users of the MPEG-4 Video
free license to this software module or modifications thereof for use in 
hardware or software products claiming conformance to the MPEG-4 Video. Those 
intending to use this software module in hardware or software products are advised 
that its use may infringe existing patents. The original developer of this software 
module and his/her company, the subsequent editors and their companies, and ISO/IEC 
have no liability for use of this software module or modifications thereof in an 
implementation. Copyright is not released for non MPEG-4 Video conforming 
products. Rockwell Science Center retains full right to use the code for his/her own 
purpose, assign or donate the code to a third party and to inhibit third parties from 
using the code for non <MPEG standard> conforming products. This copyright notice 
must be included in all copies or derivative works. Copyright (c) 1997.

Module Name:    fba_decoder.c

Abstract:       main module for FAP decoding

*************************************************************************/

#include <assert.h>
#include "sim.h"

fba_object_plane_t fop;

/*viseme/expression LUT variables -edp*/
vbdptr **vsptr;/*8 bit vsptr array index is viseme_1(4 msbs)and viseme_2(4 lsbs)*/
int vsptrind,vbdptrind,expvals;/*constructed array indices and expvals*/
explink *curexplinkptr,*lastexplinkptr,*newexplinkptr;

int initialised = 0;
int coding_type;

static FILE *infp, *fapfp, *bapfp;

void init( void);

#ifndef MUX

void
main(int argc, char **argv) {

  int i, n=0, n_bap=0, inb;
  int sc;
  int fapflag=0, bapflag=0;
  unsigned int PTS=0;
  FOP_DATA *fop_data;
  FILE *infp, *fapfp, *bapfp;

  if(argc<2) {
	fprintf(stderr, "Usage: %s streamfile [fapfile] [bapfile]\n", argv[0]);
	exit(-1);
  }

  init();

  /* read whole stream from a file into the buffer */
  infp = fopen(argv[1], "rb");
  /*i = 0;
  while(getc(infp) != EOF) i++;-edp*/
  fseek(infp,0, SEEK_END);/*-edp*/
  i = ftell(infp);/*-edp*/

  fop.stream.buf = (unsigned char*)malloc(i);
  fop.stream.p = 0;
  fop.stream.incnt = 0;
  fop.stream.bitcnt = 0;
  i = 0;
  rewind(infp);
  while((inb = getc(infp)) != EOF) fop.stream.buf[i++] = inb;
  fclose(infp);

  if(argc==3) {
	fapflag=1;
  	fapfp = fopen(argv[2], "w");
  }
  if(argc==4) {
	fapflag=1;
  	fapfp = fopen(argv[2], "w");
	bapflag=1;
  	bapfp = fopen(argv[3], "w");
  }

  fop.fo.fop_data = (FOP_DATA*) malloc(sizeof(FOP_DATA));
  fop.bo.bop_data = (BOP_DATA*) malloc(sizeof(BOP_DATA));


  initbits();
  sc = getbits(32);
  assert(sc==FBA_OBJECT_START_CODE);

  do {
    fba_object_plane_header(fop.fo.fop_data, fop.bo.bop_data, &PTS);
    fba_object_plane_data(fop.fo.fop_data, fop.bo.bop_data, &PTS);
    if(fapflag && (fop.fba_paramset & FAP_PARAMSET_MASK)) {
      if(fop.fo.coding_type == PREDICTIVE_CODING_TYPE) {
	for(i=0; i<NFAP; i++) {
	  fprintf(fapfp, "%d ", fop.fo.fop_data->fap_mask[i]);
	}
	fprintf(fapfp, "\n%d ", n);
	if(fop.fo.fop_data->fap_mask[0]) {
	  fprintf(fapfp, "%d %d %d %d ", fop.fo.fop_data->viseme_1,
		  fop.fo.fop_data->viseme_2, fop.fo.fop_data->viseme_blend,
		  fop.fo.fop_data->viseme_def);
	}
	if(fop.fo.fop_data->fap_mask[1]) {
	  fprintf(fapfp, "%d %d %d %d %d %d ", 
		  fop.fo.fop_data->expression_1,
		  fop.fo.fop_data->expression_1_intensity,
		  fop.fo.fop_data->expression_2,
		  fop.fo.fop_data->expression_2_intensity,
		  fop.fo.fop_data->init_face,
		  fop.fo.fop_data->expression_def);
	}
	for(i=2; i<NFAP; i++) {
	  if(fop.fo.fop_data->fap_mask[i]) {
	    fprintf(fapfp, "%d ", fop.fo.fop_data->fap[i]);
	  }
	}
	fprintf(fapfp, "\n");
	n++;
      } else {
	write1Segment(n, fapfp);
	n += SEG_LENGTH;
      }
    }
    if(bapflag && (fop.fba_paramset & BAP_PARAMSET_MASK)) {
      for(i=0; i<NUM_BAPS; i++) {
	fprintf(bapfp, "%d ", fop.bo.bop_data->bap_mask[i]);
      }
      fprintf(bapfp, "\n%d ", n_bap);
      for(i=0; i<NUM_BAPS; i++) {
	if(fop.bo.bop_data->bap_mask[i]) {
	  fprintf(bapfp, "%d ", fop.bo.bop_data->bap[i]);
	}
      }
      fprintf(bapfp, "\n");
      n_bap++;
    }
    
  } while(nextbits_bytealigned()!=FBA_OBJECT_END_CODE);

  if (fop.fo.coding_type == PREDICTIVE_CODING_TYPE) {
    for(i=0; i<7; i++) {
      free(fop.fo.Ifreq_ve[i]);
      free(fop.fo.Icumul_ve[i]);
      free(fop.fo.freq_ve[i]);
      free(fop.fo.cumul_ve[i]);
    }
    for(i=0; i<NFAP; i++) {
      free(fop.fo.Ifreq[i]);
      free(fop.fo.Icumul_freq[i]);
      free(fop.fo.freq[i]);
      free(fop.fo.cumul_freq[i]);
    }
    for(i=0; i<NUM_BAPS; i++) {
      free(fop.bo.Ifreq[i]);
      free(fop.bo.Icumul_freq[i]);
      free(fop.bo.freq[i]);
      free(fop.bo.cumul_freq[i]);
    }
  }
  else { /* DCT coding type */
    free(fop.fo.dctQP);
    for (i=0; i<NFAP; i++) {
	free(fop.fo.faps[i]);
    }
    free(fop.fo.faps);
    for (i=0; i<4; i++) {
	free(fop.fo.visemes[i]);
    }
    free(fop.fo.visemes);
    for (i=0; i<6; i++) {
	free(fop.fo.expressions[i]);
    }
    free(fop.fo.expressions);
    free(fop.fo.dcPred);
    for (i=0; i<SEG_LENGTH; i++) {
	free(fop.fo.idctMatrix[i]);
    }
    free(fop.fo.idctMatrix);
  }
  if(fapflag) {
  	fclose(fapfp);
  }
  if(bapflag) {
  	fclose(bapfp);
  }
}

#endif // ! MUX

void
init(void)
{
  int i,j;
  /*
  ** Right now QP and IQP are the same. We use IQP just in case we decide
  ** to use different QP values for intra and inter frames later
  */
  int temp_mask_index[11] = {0, 2, 18, 30, 38, 42, 47, 50, 60, 64, 68};

  int temp_QP[NFAP] = {1, 1, 4, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 1, 1, 1, 1, 1,
                1, 1, 1, 1, 128, 128, 128, 128, 1, 1, 1, 1, 2, 2,
                2, 2, 2, 2, 1, 1, 2, 2, 2, 2, 1, 1, 1, 1, 512, 128,
                128, 128, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 1, 1, 1, 1,
                1, 1, 1, 1};
  int temp_IQP[NFAP] = {1, 1, 4, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 1, 1, 1, 1, 1,
                1, 1, 1, 1, 128, 128, 128, 128, 1, 1, 1, 1, 2, 2,
                2, 2, 2, 2, 1, 1, 2, 2, 2, 2, 1, 1, 1, 1, 512, 128,
                128, 128, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 1, 1, 1, 1,
                1, 1, 1, 1};

  int  bap_mask_index[BAP_NUM_GROUPS][MAX_BAPS]={
                   {1,2,3},
                   {4,6,10,14},
                   {5,7,11,15},
                   {8,12,16,18,20,22},
                   {9,13,17,19,21,23},
                   {32,34,36,38,42},
                   {33,35,37,39,43},
                   {24,26,28,30,40,44,46},
                   {25,27,29,31,41,45,47},
                   {48,49,50,60,61,62,87,88,89,114,115,116},
                   {54,55,56,72,73,74,99,100,101,108,109,110,120,121,122},
                   {57,58,59,66,67,68,81,82,83,93,94,95,105,106,107,117,118,119},
                   {63,64,65,69,70,71,75,76,77,90,91,92,102,103,104,111,112,113},
                   {51,52,53,78,79,80,84,85,86,96,97,98},
                   {125,131,133,137,143,145,149,155,157,161,167,169,171,173,177,179},
                   {126,132,134,138,144,146,150,156,158,162,168,170,172,174,178,180},
                   {123,127,129,135,139,141,147,151,153,159,163,165,175},
                   {124,128,130,136,140,142,148,152,154,160,164,166,176},
                   {181,182,183,184,185,186}
  };
  int baps_in_group[BAP_NUM_GROUPS] = {
    3, 4, 4, 6, 6, 5, 5, 7, 7, 12, 15, 18, 18, 12, 16, 16, 13, 13, 6
  };

static int temp_BQP[NUM_BAPS] =
   {256,256,256,256,256,256,256,256,256,256,
    256,256,256,256,256,256,256,256,256,256,
    256,256,256,256,256,256,256,256,256,256,
    256,256,256,256,256,256,256,256,256,256,
    256,256,256,256,256,256,256,256,256,256,
    256,256,256,256,256,256,256,256,256,256,
    256,256,256,256,256,256,256,256,256,256,
    256,256,256,256,256,256,256,256,256,256,
    256,256,256,256,256,256,256,256,256,256,
    256,256,256,256,256,256,256,256,256,256,
    256,256,256,256,256,256,256,256,256,256,
    256,256,256,256,256,256,256,256,256,256,
    256,256,256,256,256,256,256,256,256,256,
    256,256,256,256,256,256,256,256,256,256,
    256,256,256,256,256,256,256,256,256,256,
    256,256,256,256,256,256,256,256,256,256,
    256,256,256,256,256,256,256,256,256,256,
    256,256,256,256,256,256,256,256,256,256,
    256,256,256,256,256,256};

static int temp_IBQP[NUM_BAPS] =
   {256,256,256,256,256,256,256,256,256,256,
    256,256,256,256,256,256,256,256,256,256,
    256,256,256,256,256,256,256,256,256,256,
    256,256,256,256,256,256,256,256,256,256,
    256,256,256,256,256,256,256,256,256,256,
    256,256,256,256,256,256,256,256,256,256,
    256,256,256,256,256,256,256,256,256,256,
    256,256,256,256,256,256,256,256,256,256,
    256,256,256,256,256,256,256,256,256,256,
    256,256,256,256,256,256,256,256,256,256,
    256,256,256,256,256,256,256,256,256,256,
    256,256,256,256,256,256,256,256,256,256,
    256,256,256,256,256,256,256,256,256,256,
    256,256,256,256,256,256,256,256,256,256,
    256,256,256,256,256,256,256,256,256,256,
    256,256,256,256,256,256,256,256,256,256,
    256,256,256,256,256,256,256,256,256,256,
    256,256,256,256,256,256,256,256,256,256,
    256,256,256,256,256,256};


  int temp_Irange_ve_max[7]={14,14,63,6,63,6,63};
  int temp_range_ve_min[7]={-14,-14,-63,-6,-63,-6,-63};

  for (i=0; i<11; i++)
    fop.fo.mask_index[i] = temp_mask_index[i];
  for (i=0; i<NFAP; i++)
    fop.fo.QP[i] = temp_QP[i];
  for (i=0; i<NFAP; i++)
    fop.fo.IQP[i] = temp_IQP[i];

  for (i=0; i<NUM_BAPS; i++)
    fop.bo.BQP[i] = temp_BQP[i];
  for (i=0; i<NUM_BAPS; i++)
    fop.bo.IBQP[i] = temp_IBQP[i];                                  

  for(i=0; i<7; i++) {
	fop.fo.Ifreq_ve[i] = NULL;
	fop.fo.Icumul_ve[i] = NULL;
	fop.fo.freq_ve[i] = NULL;
	fop.fo.cumul_ve[i] = NULL;
  }

  for(i=0; i<NFAP; i++) {
        fop.fo.Ifreq[i] = NULL;
        fop.fo.Icumul_freq[i] = NULL;
        fop.fo.freq[i] = NULL;
        fop.fo.cumul_freq[i] = NULL;
  }

  for(i=0; i<NUM_BAPS; i++) {
        fop.bo.Ifreq[i] = NULL;
        fop.bo.Icumul_freq[i] = NULL;
        fop.bo.freq[i] = NULL;
        fop.bo.cumul_freq[i] = NULL;
  }

  for (i=0; i<7; i++) {
    fop.fo.range_ve_max[i] =
      fop.fo.Irange_ve_max[i] = temp_Irange_ve_max[i];
    fop.fo.Irange_ve_min[i] = 0;
    fop.fo.range_ve_min[i] = temp_range_ve_min[i];
  }

  for (i=0; i<11; i++) {
    fop.fo.mask_index[i] = temp_mask_index[i];
  }

  for (i=0; i<BAP_NUM_GROUPS; i++) {
    fop.bo.baps_in_group[i] = baps_in_group[i];
    for (j=0; j<baps_in_group[i]; j++) {
      fop.bo.bap_mask_index[i][j] = bap_mask_index[i][j]-1;
    }
  }

  fop.fo.Idefault_min[2] = -1080; fop.fo.Idefault_max[2] = 1080;
  fop.fo.default_min[2] = -360; fop.fo.default_max[2] = 360;
  fop.fo.Idefault_min[3] = -600; fop.fo.Idefault_max[3] = 600;
  fop.fo.default_min[3] = -180; fop.fo.default_max[3] = 180;
  fop.fo.Idefault_min[4] = -1860; fop.fo.Idefault_max[4] = 1860;
  fop.fo.default_min[4] = -600; fop.fo.default_max[4] = 600;
  for(i=5;i<9;i++) {
    fop.fo.Idefault_min[i] = -600; fop.fo.Idefault_max[i] = 600;
    fop.fo.default_min[i] = -180; fop.fo.default_max[i] = 180;
  }
  for(i=9;i<11;i++) {
    fop.fo.Idefault_min[i] = -1860; fop.fo.Idefault_max[i] = 1860;
    fop.fo.default_min[i] = -600; fop.fo.default_max[i] = 600;
  }
  for(i=11;i<14;i++) {
    fop.fo.Idefault_min[i] = -600; fop.fo.Idefault_max[i] = 600;
    fop.fo.default_min[i] = -180; fop.fo.default_max[i] = 180;
  }
  for(i=14;i<17;i++) {
    fop.fo.Idefault_min[i] = -1080; fop.fo.Idefault_max[i] = 1080;
    fop.fo.default_min[i] = -360; fop.fo.default_max[i] = 360;
  }
  fop.fo.Idefault_min[17] = -420; fop.fo.Idefault_max[17] = 420;
  fop.fo.default_min[17] = -180; fop.fo.default_max[17] = 180;
  for(i=18;i<20;i++) {
    fop.fo.Idefault_min[i] = -1080; fop.fo.Idefault_max[i] = 1080;
    fop.fo.default_min[i] = -600; fop.fo.default_max[i] = 600;
  }
  for(i=20;i<22;i++) {
    fop.fo.Idefault_min[i] = -600; fop.fo.Idefault_max[i] = 600;
    fop.fo.default_min[i] = -240; fop.fo.default_max[i] = 240;
  }
  for(i=22;i<24;i++) {
    fop.fo.Idefault_min[i] = -1200; fop.fo.Idefault_max[i] = 1200;
    fop.fo.default_min[i] = -420; fop.fo.default_max[i] = 420;
  }
  for(i=24;i<26;i++) {
    fop.fo.Idefault_min[i] = -900; fop.fo.Idefault_max[i] = 900;
    fop.fo.default_min[i] = -300; fop.fo.default_max[i] = 300;
  }
  for(i=26;i<28;i++) {
    fop.fo.Idefault_min[i] = -600; fop.fo.Idefault_max[i] = 600;
    fop.fo.default_min[i] = -180; fop.fo.default_max[i] = 180;
  }
  for(i=28;i<30;i++) {
    fop.fo.Idefault_min[i] = -420; fop.fo.Idefault_max[i] = 420;
    fop.fo.default_min[i] = -120; fop.fo.default_max[i] = 120;
  }
  for(i=30;i<36;i++) {
    fop.fo.Idefault_min[i] = -900; fop.fo.Idefault_max[i] = 900;
    fop.fo.default_min[i] = -360; fop.fo.default_max[i] = 360;
  }
  for(i=36;i<40;i++) {
    fop.fo.Idefault_min[i] = -900; fop.fo.Idefault_max[i] = 900;
    fop.fo.default_min[i] = -300; fop.fo.default_max[i] = 300;
  }
  for(i=40;i<42;i++) {
    fop.fo.Idefault_min[i] = -600; fop.fo.Idefault_max[i] = 600;
    fop.fo.default_min[i] = -180; fop.fo.default_max[i] = 180;
  }
  for(i=42;i<46;i++) {
    fop.fo.Idefault_min[i] = -1080; fop.fo.Idefault_max[i] = 1080;
    fop.fo.default_min[i] = -420; fop.fo.default_max[i] = 420;
  }
  fop.fo.Idefault_min[46] = -1080; fop.fo.Idefault_max[46] = 1080;
  fop.fo.default_min[46] = -420; fop.fo.default_max[46] = 420;
  for(i=47;i<50;i++) {
    fop.fo.Idefault_min[i] = -1860; fop.fo.Idefault_max[i] = 1860;
    fop.fo.default_min[i] = -600; fop.fo.default_max[i] = 600;
  }
  fop.fo.Idefault_min[50] = -600; fop.fo.Idefault_max[50] = 600;
  fop.fo.default_min[50] = -180; fop.fo.default_max[50] = 180;
  fop.fo.Idefault_min[51] = -1860; fop.fo.Idefault_max[51] = 1860;
  fop.fo.default_min[51] = -600; fop.fo.default_max[51] = 600;
  for(i=52;i<56;i++) {
    fop.fo.Idefault_min[i] = -600; fop.fo.Idefault_max[i] = 600;
    fop.fo.default_min[i] = -180; fop.fo.default_max[i] = 180;
  }
  for(i=56;i<58;i++) {
    fop.fo.Idefault_min[i] = -1860; fop.fo.Idefault_max[i] = 1860;
    fop.fo.default_min[i] = -600; fop.fo.default_max[i] = 600;
  }
  for(i=58;i<60;i++) {
    fop.fo.Idefault_min[i] = -600; fop.fo.Idefault_max[i] = 600;
    fop.fo.default_min[i] = -180; fop.fo.default_max[i] = 180;
  }
  for(i=60;i<62;i++) {
    fop.fo.Idefault_min[i] = -540; fop.fo.Idefault_max[i] = 540;
    fop.fo.default_min[i] = -120; fop.fo.default_max[i] = 120;
  }
  fop.fo.Idefault_min[62] = -680; fop.fo.Idefault_max[62] = 680;
  fop.fo.default_min[62] = -180; fop.fo.default_max[62] = 180;
  fop.fo.Idefault_min[63] = -900; fop.fo.Idefault_max[63] = 900;
  fop.fo.default_min[63] = -180; fop.fo.default_max[63] = 180;
  for(i=64;i<66;i++) {
    fop.fo.Idefault_min[i] = -900; fop.fo.Idefault_max[i] = 900;
    fop.fo.default_min[i] = -240; fop.fo.default_max[i] = 240;
  }
  for(i=66;i<68;i++) {
    fop.fo.Idefault_min[i] = -900; fop.fo.Idefault_max[i] = 900;
    fop.fo.default_min[i] = -300; fop.fo.default_max[i] = 300;
  }

  /*init viseme/expression variables -edp had to move this after "while(getc(infp) != EOF) i++;" because of buffer overflow*/
  vsptr = (vbdptr **)malloc(256*sizeof(vbdptr *));
  for(i=0;i < 256;i++) vsptr[i] = 0;

}


/* Decode a frame of FBA data; return number of bits read from the
   AUptr stream; return 0 if end of FBA stream is encountered */
int
fba_decoder(unsigned char *AUptr, int bitoffset, int AUbitcount,
        FOP_DATA *fop_data, BOP_DATA *bop_data, int fba_bytecount, unsigned int *PTS) 
{
  int bits = 0;

  /* copy the stream data into the buffer */
  fop.stream.buf = AUptr;
  fop.stream.bufsize = AUbitcount;
  fop.stream.p = 0;
  fop.stream.incnt = 0;
  fop.stream.bitcnt = 0;

  initbits();
  getbits(bitoffset);

  if (!initialised) {
	int sc;

	init();

	initialised = 1;

	sc = getbits(32);
	assert(sc==FBA_OBJECT_START_CODE);
  }

  fba_object_plane_header(fop_data, bop_data, PTS);
  fba_object_plane_data(fop_data, bop_data, PTS);

  /*  for(i=0; i<7; i++) {
	free(fop.fo.Ifreq_ve[i]);
	free(fop.fo.Icumul_ve[i]);
	free(fop.fo.freq_ve[i]);
	free(fop.fo.cumul_ve[i]);
  }
  for(i=0; i<NFAP; i++) {
        free(fop.fo.Ifreq[i]);
        free(fop.fo.Icumul_freq[i]);
        free(fop.fo.freq[i]);
        free(fop.fo.cumul_freq[i]);
	} */

  return(cntbits() - bitoffset);
}

