/*************************************************************************

This software module was originally developed by 
	Wei Wu (weiwu@stallion.risc.rockwell.com),  Rockwell Science Center 
        Tolga K. Capin (capin@lig.di.epfl.ch), EPFL-LIG,
        Joaquim Esmerado (jle@lig.di.epfl.ch), EPFL-LIG.
	Eric Petajan (edp@bell-labs.com), Lucent Technologies

in the course of development of the MPEG-4 Video (ISO/IEC 14496-2). 
This software module is an implementation of a part of one or more MPEG-4 Video
tools as specified by the MPEG-4 Video. ISO/IEC gives users of the MPEG-4 Video
free license to this software module or modifications thereof for use in 
hardware or software products claiming conformance to the MPEG-4 Video. Those 
intending to use this software module in hardware or software products are advised 
that its use may infringe existing patents. The original developer of this software 
module and his/her company, the subsequent editors and their companies, and ISO/IEC 
have no liability for use of this software module or modifications thereof in an 
implementation. Copyright is not released for non MPEG-4 Video conforming 
products. Rockwell Science Center and EPFL-LIG retain full right to use the code
for his/her own purpose, assign or donate the code to a third party and to inhibit
third parties from using the code for non <MPEG standard> conforming products. This
copyright notice  must be included in all copies or derivative works. Copyright (c) 1997.

Module Name:    fba_object_plane.c
Modifications: added gender bit and fixed frame_rate calculation -edp


*************************************************************************/

#include <assert.h>
#include "sim.h"

#define DEF_FRAME_RATE	30.0
/* #define DEBUG 1 */
/*-edp*/
extern vbdptr **vsptr;/*8 bit vsptr array index is viseme_1(4 msbs)and viseme_2(4 lsbs)*/
extern int vsptrind,vbdptrind,expvals;/*constructed array indices and expvals*/
extern explink *curexplinkptr,*lastexplinkptr,*newexplinkptr;

void bap_decode_skip_frames(void);
void decode_frame_rate(void);
void bap_decode_frame_rate(void);
void decode_time_code(void);
void bap_decode_time_code(void);
void decode_skip_frames(void);
void decode_fap_group_mask(int);
void bap_decode_group_mask(int);
void decode_new_minmax(void);
void bap_decode_new_minmax(void);
void decode_ifap(FOP_DATA*), decode_pfap(FOP_DATA*);
void decode_ibap(BOP_DATA*), decode_pbap(BOP_DATA*);
void bap_update(int symbol, int index);
void print_fba_header(void);

int
fba_object_plane_header(FOP_DATA* fop_data, BOP_DATA *bop_data, unsigned int *PTS) {

  int sc;
  static int suggested_gender=0;
  static int coding_type=0; /* 3-3-98 */
  static int first_intra_segment=1; /* 6-26-98 */
  int is_frame_rate = 0;
  int is_time_code = 0;
  int skip_frames = 0;
  static double real_frame_rate = DEF_FRAME_RATE;
  static double NEXT_PTS=0.0;
  int minn,maxn;/*-edp*/

  resetbits();
  if(nextbits_bytealigned() == FBA_OBJECT_PLANE_START_CODE) {
    next_start_code();
    sc = getbits(32);
    assert(sc==FBA_OBJECT_PLANE_START_CODE);
  }
  fop.is_intra = getbits(1);
  fop.fba_paramset = getbits(2);

  if (fop.is_intra) {
    if (fop.fba_paramset & FAP_PARAMSET_MASK) {
      is_frame_rate = getbits(1);
      if(is_frame_rate) {
	decode_frame_rate();
	if(fop.fo.frequency_offset)
	  real_frame_rate = ((double)fop.fo.frame_rate +\
			     ((double)fop.fo.seconds/16))*(1000/1001);
	else
	  real_frame_rate = (double)fop.fo.frame_rate + ((double)fop.fo.seconds/16);

      }
      fop.fo.is_time_code = getbits(1);
      if(fop.fo.is_time_code) {
	decode_time_code();
      }
      skip_frames = getbits(1);
      if(skip_frames) {
	decode_skip_frames();
      } else {
	fop.fo.number_of_frames_to_skip = 0;
      }
    }
  
    if (fop.fba_paramset & BAP_PARAMSET_MASK) {
      fop.bo.is_frame_rate = getbits(1);
      if(fop.bo.is_frame_rate) {
	bap_decode_frame_rate();
	real_frame_rate = fop.bo.frame_rate;
      }
      fop.bo.is_time_code = getbits(1);
      if(fop.bo.is_time_code) {
	bap_decode_time_code();
      }
      fop.bo.skip_frames = getbits(1);
      if(fop.bo.skip_frames) {
	bap_decode_skip_frames();
      } else {
	fop.bo.number_of_frames_to_skip = 0;
      }
    }
  }
  return 0;
}
  


int
fba_object_plane_data(FOP_DATA* fop_data, BOP_DATA *bop_data, unsigned int *PTS) {

  int i, j;
  int marker;
  int hquant;
  int bitcnt;
  int sc;
  static int suggested_gender=0;
  static int coding_type=0; /* 3-3-98 */
  static int first_intra_segment=1; /* 6-26-98 */
  static int real_frame_rate = DEF_FRAME_RATE;
  static double NEXT_PTS=0.0;
  static int first_action=1;
  static int fr_no=0;
  int minn,maxn;/*-edp*/
  int extendable_bap_id;

  static int no_frames=0;
  printf("Frame %d\n", no_frames++);

  /*  if (first_action) {
    print_fba_header();
    first_action = 0;
  }
  */

  if (fop.fba_paramset & FAP_PARAMSET_MASK) {

	if(fop.is_intra) {
	  fop.fo.fap_quant = getbits(5);
	  hquant = fop.fo.fap_quant/2;

	  if(fop.fo.Irange_max[3] == 0 && fop.fo.Irange_min[3] == 0) {
	    /*init min/max values if not set before -edp*/
	    if(fop.fo.fap_quant)
	      for(i=2;i<7;i+=2) {
		fop.fo.Irange_ve_max[i] = (63+hquant)/fop.fo.fap_quant;
		fop.fo.range_ve_max[i] = (63+hquant)/fop.fo.fap_quant;
		fop.fo.range_ve_min[i] = -(63+hquant)/fop.fo.fap_quant;
	      }
	    for(i=2;i<NFAP;i++) {/*set init minmax values -edp*/
	      maxn = (fop.fo.Idefault_max[i]+RANGE_DELTA-1)/RANGE_DELTA;
	      maxn *= RANGE_DELTA;
	      if(fop.fo.fap_quant)
		fop.fo.Irange_max[i] = maxn/fop.fo.fap_quant;
	      else fop.fo.Irange_max[i] = maxn;
		    
	      minn = (fop.fo.Idefault_min[i]-RANGE_DELTA+1)/RANGE_DELTA;
	      minn *= RANGE_DELTA;
	      if(fop.fo.fap_quant)
		fop.fo.Irange_min[i] = minn/fop.fo.fap_quant;
	      else fop.fo.Irange_min[i] = minn;
			  
	      maxn = (fop.fo.default_max[i]+RANGE_DELTA-1)/RANGE_DELTA;
	      maxn *= RANGE_DELTA;
	      if(fop.fo.fap_quant)
		fop.fo.range_max[i] = maxn/fop.fo.fap_quant;
	      else fop.fo.range_max[i] = maxn;
	      
	      minn = (fop.fo.default_min[i]-RANGE_DELTA+1)/RANGE_DELTA;
	      minn *= RANGE_DELTA;
	      if(fop.fo.fap_quant)
		fop.fo.range_min[i] = minn/fop.fo.fap_quant;
	      else fop.fo.range_min[i] = minn;
	    }
	  }

	  for(i=0; i<N_FAP_GROUP; i++) {
	    marker = getbits(1);
	    assert(marker);
	    fop.fo.fap_mask_type[i] = getbits(2);
	    if(fop.fo.fap_mask_type[i]==1 || fop.fo.fap_mask_type[i]==2) {
	      decode_fap_group_mask(i);
	    } else if(fop.fo.fap_mask_type[i]==0) {
	      for(j=fop.fo.mask_index[i]; j<fop.fo.mask_index[i+1]; j++) {
		fop.fo.fap_mask[j] = 0;
	      }
	    } else {
	      for(j=fop.fo.mask_index[i]; j<fop.fo.mask_index[i+1]; j++) {
		fop.fo.fap_mask[j] = 1;
	      }
	    }
	  }

	  suggested_gender = getbits(1);
	  coding_type = getbits(1);/*changed from 2 to 1 -edp*/
	  fop.fo.coding_type = coding_type;
	  assert(coding_type==PREDICTIVE_CODING_TYPE || coding_type==DCT_CODING_TYPE);
	  
	  if (coding_type == PREDICTIVE_CODING_TYPE) {

	    fop.fo.is_i_new_max = getbits(1);
	    fop.fo.is_i_new_min = getbits(1);
	    fop.fo.is_p_new_max = getbits(1);
	    fop.fo.is_p_new_min = getbits(1);
	    
	    decode_new_minmax();
	    
	    if(fop.fo.fap_quant) {
	      fop.fo.Irange_ve_max[2] = (63+hquant)/fop.fo.fap_quant;
	      fop.fo.Irange_ve_max[4] = (63+hquant)/fop.fo.fap_quant;
	      fop.fo.Irange_ve_max[6] = (63+hquant)/fop.fo.fap_quant;
	      fop.fo.range_ve_max[2] = (63+hquant)/fop.fo.fap_quant;
	      fop.fo.range_ve_max[4] = (63+hquant)/fop.fo.fap_quant;
	      fop.fo.range_ve_max[6] = (63+hquant)/fop.fo.fap_quant;
	      fop.fo.range_ve_min[2] = -(63+hquant)/fop.fo.fap_quant;
	      fop.fo.range_ve_min[4] = -(63+hquant)/fop.fo.fap_quant;
	      fop.fo.range_ve_min[6] = -(63+hquant)/fop.fo.fap_quant;
	    }

	    init_arith(/*fop.fo.fap_mask*/);
	    for(i=0; i<NFAP; i++) {
	      fop_data->fap_mask[i] = fop.fo.fap_mask[i];
	    }
	    decode_ifap(fop_data);
	  } else {
	    if (first_intra_segment) {
	      DCT_initialize();
	      first_intra_segment = 0;
	    }
	    decode_i_segment();
	  }
	} else {
	  if (coding_type == PREDICTIVE_CODING_TYPE) {
	    for(i=0; i<NFAP; i++) {
	      fop_data->fap_mask[i] = fop.fo.fap_mask[i];
	    }
	    decode_pfap(fop_data);
	  }  else {
	    decode_p_segment();
	  }
	}

	if (coding_type == PREDICTIVE_CODING_TYPE) { /* 6-26-98 */
	  adjust_bits(); /* get ready to code next frame */
	  
	  if(fop.fo.is_time_code) {
	    *PTS = (3600*fop.fo.time_code_hours+60*fop.fo.time_code_minutes+fop.fo.time_code_seconds)*1000;
	  } else {
	    *PTS = (int) NEXT_PTS;
	  }
	  /* Modified on 6/24/98
	     NEXT_PTS += (fop.fo.number_of_frames_to_skip+1)*1000.0/(double) real_frame_rate;
	     */
	  NEXT_PTS += (fop.fo.number_of_frames_to_skip+1)*1000.0/(double) real_frame_rate;
	}
	bitcnt = cntbits();
  }

  if (fop.fba_paramset & BAP_PARAMSET_MASK) {
        if(fop.is_intra) {
                fop.bo.bap_quant = getbits(5);
                for(i=0; i<BAP_NUM_GROUPS; i++) {
                        marker = getbits(1);
                        assert(marker);
                        fop.bo.bap_mask_type[i] = getbits(2);
                        if(fop.bo.bap_mask_type[i]==1) {
			  bap_decode_group_mask(i);
                        } 
			else if(fop.bo.bap_mask_type[i]==3 || fop.bo.bap_mask_type[i]==0) {
			  int j;
			  int temp_index;

			  for(j=0;j<fop.bo.baps_in_group[i]; j++) {
			    temp_index = fop.bo.bap_mask_index[i][j];
			    if(fop.bo.bap_mask_type[i]==3)
			       fop.bo.bap_mask[temp_index] = 1;
			    else
			       fop.bo.bap_mask[temp_index] = 0;
			  }
			}
			else {
			  fprintf( stderr, "Error:\n");
			  fprintf( stderr,
				   "stream contains wrong BAP mask type for group %d\n", i);
			  exit(0);
			}

                }
		fop.bo.suggested_gender = getbits(1);
		fop.bo.body_object_coding_type = getbits(1);
                fop.bo.is_i_new_max = getbits(1);
                fop.bo.is_i_new_min = getbits(1);
                fop.bo.is_p_new_max = getbits(1);
                fop.bo.is_p_new_min = getbits(1);

		fop.bo.has_extendable_baps = getbits(1);
		if (fop.bo.has_extendable_baps) {
		  marker = getbits(1);
		  assert(marker);
		  fop.bo.num_extendable_baps = getbits(8);
		  for (i=0; i<fop.bo.num_extendable_baps; i++) {
		    marker = getbits(1);
		    assert(marker);
		    extendable_bap_id = getbits(8);
		    marker = getbits(1);
		    assert(marker);
		    fop.bo.extendable_baps[extendable_bap_id] = getbits(18);
		  }
		}

                if(fop.bo.is_i_new_max||fop.bo.is_i_new_min||
		   fop.bo.is_p_new_max||fop.bo.is_p_new_min) {
		  bap_decode_new_minmax();
                }
                bap_init_arith();
                for(i=0; i<NUM_BAPS; i++) {
		  bop_data->bap_mask[i] = fop.bo.bap_mask[i];
                }
                decode_ibap(bop_data);
        } else {
	  for(i=0; i<NUM_BAPS; i++) {
	    bop_data->bap_mask[i] = fop.bo.bap_mask[i];
	  }
	  decode_pbap(bop_data);
        }

	if (coding_type == PREDICTIVE_CODING_TYPE) { /* 6-26-98 */
	  adjust_bits(); /* get ready to code next frame */

	  if(fop.bo.is_time_code) {
	    *PTS = (3600*fop.bo.time_code_hours+60*fop.bo.time_code_minutes+fop.bo.time_code_seconds)*1000;
	  } else {
	    *PTS = (int) NEXT_PTS;
	  }
	  NEXT_PTS += (fop.bo.number_of_frames_to_skip+1)*1000.0/
	    (double) real_frame_rate;
	}
	bitcnt = cntbits();
  }

  /* Moved the below code to "face only" part above
  if (coding_type == PREDICTIVE_CODING_TYPE) { 
    adjust_bits(); 

    if(fop.fo.is_time_code) {
      *PTS = (3600*fop.fo.time_code_hours+60*fop.fo.time_code_minutes+fop.fo.time_code_seconds)*1000;
    } else {
      *PTS = (int) NEXT_PTS;
    }
    NEXT_PTS += (fop.fo.number_of_frames_to_skip+1)*1000.0/(double) real_frame_rate;
  }
  bitcnt = cntbits();
*/
  return bitcnt;
}

void
decode_frame_rate() {

  fop.fo.frame_rate = getbits(8);
  fop.fo.seconds = getbits(4);
  fop.fo.frequency_offset = getbits(1);
}

void
decode_time_code() {

  fop.fo.time_code_hours = getbits(5);
  fop.fo.time_code_minutes = getbits(6);
  fop.fo.marker_bit = getbits(1);
  fop.fo.time_code_seconds = getbits(6);
}

void
decode_skip_frames(void) {
 
  int ivalue;
 
  fop.fo.number_of_frames_to_skip = 0;
  do {
        ivalue = getbits(4);
        fop.fo.number_of_frames_to_skip += ivalue;
  } while (ivalue==15);
}

void
bap_decode_frame_rate() {

  fop.bo.frame_rate = getbits(8);
  fop.bo.seconds = getbits(4);
  fop.bo.frequency_offset = getbits(1);
}

void
bap_decode_time_code() {

  fop.bo.time_code_hours = getbits(5);
  fop.bo.time_code_minutes = getbits(6);
  fop.bo.marker_bit = getbits(1);
  fop.bo.time_code_seconds = getbits(6);
}

void
bap_decode_skip_frames(void) {

  int ivalue;

  fop.bo.number_of_frames_to_skip = 0;
  do {
        ivalue = getbits(4);
        fop.bo.number_of_frames_to_skip += ivalue;
  } while (ivalue==15);
}

void
decode_fap_group_mask(int n) {
  int i;

  for(i=fop.fo.mask_index[n]; i<fop.fo.mask_index[n+1]; i++) {
	fop.fo.fap_mask[i] = getbits(1);
  }
}

void
bap_decode_group_mask(int n) {
  int i;
  int temp_index;

  for(i=0;i<fop.bo.baps_in_group[n]; i++) {
    temp_index = fop.bo.bap_mask_index[n][i];
    fop.bo.bap_mask[temp_index] = getbits(1);
  }

}

void
decode_new_minmax() {
  int i, j;
  int minmax;
  int marker;
  int scaler;

  if (fop.fo.fap_quant) scaler = fop.fo.fap_quant;
  else scaler = 1;
  
  if(fop.fo.is_i_new_max) {
    for(i=2, j=0; i<NFAP; i++, j++) {
      if(i==18 || i==30 || i==38 || i== 42 || 
	 i==47 || i==50 || i==60 || i== 64) {
	j=0;
      }
      if(!(j&3)) {
	marker = getbits(1);
	assert(marker);
      }
      if(fop.fo.fap_mask[i]) {
	minmax = getbits(5);
	fop.fo.Irange_max[i] = minmax*RANGE_DELTA/scaler;
      }
    }
  }
  if(fop.fo.is_i_new_min) {
    for(i=2, j=0; i<NFAP; i++, j++) {
      if(i==18 || i==30 || i==38 || i== 42 || 
	 i==47 || i==50 || i==60 || i== 64) {
	j=0;
      }
      if(!(j&3)) {
	marker = getbits(1);
	assert(marker);
      }
      if(fop.fo.fap_mask[i]) {
	minmax = getbits(5);
	fop.fo.Irange_min[i] = -minmax*RANGE_DELTA/scaler;
      }
    }
  }
  if(fop.fo.is_p_new_max) {
    for(i=2, j=0; i<NFAP; i++, j++) {
      if(i==18 || i==30 || i==38 || i== 42 || 
	 i==47 || i==50 || i==60 || i== 64) {
	j=0;
      }
      if(!(j&3)) {
	marker = getbits(1);
	assert(marker);
      }
      if(fop.fo.fap_mask[i]) {
	minmax = getbits(5);
	fop.fo.range_max[i] = minmax*RANGE_DELTA/scaler;
      }
    }
  }
  if(fop.fo.is_p_new_min) {
    for(i=2, j=0; i<NFAP; i++, j++) {
      if(i==18 || i==30 || i==38 || i== 42 || 
	 i==47 || i==50 || i==60 || i== 64) {
	j=0;
      }
      if(!(j&3)) {
	marker = getbits(1);
	assert(marker);
      }
      if(fop.fo.fap_mask[i]) {
	minmax = getbits(5);
	fop.fo.range_min[i] = -minmax*RANGE_DELTA/scaler;
      }
    }
  }
  
}


void
bap_decode_new_minmax() {
  int i, j;
  int minmax;
  int marker;
  int scaler;
  int temp_index;


  if (fop.bo.bap_quant) scaler = fop.bo.bap_quant;
  else scaler = 1;
  
  if(fop.bo.is_i_new_max) {
    for (i=0; i<BAP_NUM_GROUPS; i++) {
        for(j=0;j<fop.bo.baps_in_group[i]; j++) {
          temp_index = fop.bo.bap_mask_index[i][j];
		if(!(j&3)) {
			marker = getbits(1);
			assert(marker);
		}
                if(fop.bo.bap_mask[temp_index]) {
                        minmax = getbits(5);
                        fop.bo.Irange_max[temp_index] = minmax*RANGE_DELTA/scaler;
                }
        }
    }
  }
  if(fop.bo.is_i_new_min) {
    for (i=0; i<BAP_NUM_GROUPS; i++) {
        for(j=0;j<fop.bo.baps_in_group[i]; j++) {
          temp_index = fop.bo.bap_mask_index[i][j];

		if(!(j&3)) {
			marker = getbits(1);
			assert(marker);
		}
                if(fop.bo.bap_mask[temp_index]) {
                        minmax = getbits(5);
                        fop.bo.Irange_min[temp_index] = -minmax*RANGE_DELTA/scaler;
                }
        }
    }
  }
  if(fop.bo.is_p_new_max) {
    for (i=0; i<BAP_NUM_GROUPS; i++) {
        for(j=0;j<fop.bo.baps_in_group[i]; j++) {
          temp_index = fop.bo.bap_mask_index[i][j];

		if(!(j&3)) {
			marker = getbits(1);
			assert(marker);
		}
                if(fop.bo.bap_mask[temp_index]) {
                        minmax = getbits(5);
                        fop.bo.range_max[temp_index] = minmax*RANGE_DELTA/scaler;
                }
        }
    }
  }
  if(fop.bo.is_p_new_min) {
    for (i=0; i<BAP_NUM_GROUPS; i++) {
        for(j=0;j<fop.bo.baps_in_group[i]; j++) {
          temp_index = fop.bo.bap_mask_index[i][j];

		if(!(j&3)) {
			marker = getbits(1);
			assert(marker);
		}
                if(fop.bo.bap_mask[temp_index]) {
                        minmax = getbits(5);
                        fop.bo.range_min[temp_index] = -minmax*RANGE_DELTA/scaler;
                }
        }
    }
  }

}

void
decode_ibap(BOP_DATA *bop_data) {
  int i, j, k;
  int symbol, qstep;
  int dequant(int, int);

  decoder_reset();

  for (k=0; k<BAP_NUM_GROUPS; k++) {
    for(j=0;j<fop.bo.baps_in_group[k]; j++) {
	  i = fop.bo.bap_mask_index[k][j];
	if(fop.bo.bap_mask[i]) {
		symbol = decode_a_symbol(fop.bo.Icumul_freq[i]);
		fop.bo.bap_Q[i] = symbol + fop.bo.Irange_min[i];
		qstep = fop.bo.IBQP[i]*fop.bo.bap_quant;
		fop.bo.bap_prev[i] = dequant(fop.bo.bap_Q[i], qstep);
		bop_data->bap[i] = fop.bo.bap_prev[i];
	}
    }
  }

}


void
procvisexp(FOP_DATA *fop_data) {/*-edp*/
  int i;
  static int numnew;  
  /*if def bits set then update viseme/expression LUT else lookup vis/exp in LUT and replace interp FAPs for this frame -edp*/
  /*form packed indices and expressiondata*/
  if(fop_data->fap_mask[0]) {
    vsptrind = fop_data->viseme_2;
    vsptrind |= fop_data->viseme_1 <<4;
    vbdptrind = fop_data->viseme_blend;
  }
  else {
    vsptrind=0;vbdptrind=0;
  }
  if(fop_data->fap_mask[1]) {
    expvals = fop_data->init_face;
    expvals |= fop_data->expression_2_intensity<<1;
    expvals |= fop_data->expression_2 <<7;
    expvals |= fop_data->expression_1_intensity <<10;
    expvals |= fop_data->expression_1 <<13;
  }
  else {
    expvals = 0;
  }
  /*compare vis/exp to LUT. create new entries if needed and if one or both defbits are set*/
  lastexplinkptr = 0;/*0 indicates only one link*/
  curexplinkptr = 0;

  if(vsptr[vsptrind]) {/*test existance of viseme select pair in LUT*/
    if(vsptr[vsptrind][vbdptrind]) {/*test existance of viseme_blend and viseme_def values in LUT*/
      curexplinkptr = vsptr[vsptrind][vbdptrind];
      do {
	if(curexplinkptr->expvals == expvals) {/*test existance of expression values*/
	  break;
	}
	if(!(curexplinkptr->next)) {
	  if(fop_data->expression_def ||fop_data->viseme_def) {
	    newexplinkptr = (explink *)malloc(sizeof(explink));
#if DEBUG
	    printf("numnew1 = %d\n",numnew++);
#endif
	    if(expvals < curexplinkptr->expvals) {
	      if(!lastexplinkptr) {/*insert before first link*/
		vsptr[vsptrind][vbdptrind] = newexplinkptr;
		newexplinkptr->next = curexplinkptr;
		curexplinkptr = newexplinkptr;
		break;
	      }
	      else {
		lastexplinkptr->next = newexplinkptr;
		newexplinkptr->next = curexplinkptr;
		curexplinkptr = newexplinkptr;
		break;
	      }
	    }
	    else {
	      curexplinkptr->next = newexplinkptr;
	      newexplinkptr->next = 0;
	      lastexplinkptr = curexplinkptr;
	      curexplinkptr = newexplinkptr;
	      break;
	    }
	  }
	  else {
	    curexplinkptr = 0;/*no LUT entry found*/
	    break;
	  }
	}
	else {
	  if(fop_data->expression_def ||fop_data->viseme_def) {/*is next, def set*/
            if(expvals < curexplinkptr->expvals) {
	      newexplinkptr = (explink *)malloc(sizeof(explink));
#if DEBUG
	      printf("numnew2 = %d\n",numnew++);
#endif
              if(!lastexplinkptr) {/*insert before first link*/
                vsptr[vsptrind][vbdptrind] = newexplinkptr;
                newexplinkptr->next = curexplinkptr;
                curexplinkptr = newexplinkptr;
                break;
              }
	      else {
                lastexplinkptr->next = newexplinkptr;
                newexplinkptr->next = curexplinkptr;
                curexplinkptr = newexplinkptr;
                break;
              }
	    }
	    else {
	      lastexplinkptr = curexplinkptr;
	      curexplinkptr = curexplinkptr->next;
	    }
	  }
	  else {/*is next,no def*/
	    if(expvals < curexplinkptr->expvals) {
	      curexplinkptr = 0;/*no LUT entry found*/
	      break;
	    }
	    else {
	      lastexplinkptr = curexplinkptr;
	      curexplinkptr = curexplinkptr->next;
	    }
	  }
	}
      } while(1);
    }
    else if(fop_data->expression_def ||fop_data->viseme_def) {
      vsptr[vsptrind][vbdptrind] = (explink *)malloc(sizeof(explink));
#if DEBUG
      printf("numnew3 = %d\n",numnew++);
#endif
      curexplinkptr = vsptr[vsptrind][vbdptrind];
      curexplinkptr->next = 0;
    }
  }
  else if(fop_data->expression_def ||fop_data->viseme_def){
    vsptr[vsptrind] = (vbdptr *)malloc(64*sizeof(explink *));
    for(i = 0;i < 64;i++) vsptr[vsptrind][i] = 0;
    vsptr[vsptrind][vbdptrind] = (explink *)malloc(sizeof(explink));
#if DEBUG
    printf("numnew4 = %d\n",numnew++);
#endif
    curexplinkptr = vsptr[vsptrind][vbdptrind];
    curexplinkptr->next = 0;
  }
  /*assume the curexplinkptr references the either the correct LUT entry or a new empty LUT entry*/
  if(curexplinkptr) {
    if(fop_data->viseme_def && !(fop_data->expression_def)) {/*replace LUT entry only for mouth related FAPs*/
      curexplinkptr->expvals = expvals;
      for(i=0; i<NFAP; i++)
	if(fop_data->fap_mask[i] && ((i>1 && i<18) ||(i>37 && i<47) || (i>49 && i <60))) curexplinkptr->faps[i] = fop_data->fap[i];
	else curexplinkptr->faps[i] = MAXINT;/*set to interpolate*/
    }
    else if(fop_data->expression_def) {/*replace LUT entry for all but headrot FAPs*/
      curexplinkptr->expvals = expvals;
      for(i=0; i<NFAP; i++)
	if(fop_data->fap_mask[i] && ((i>1 && i<47) || (i>49 && i <NFAP))) curexplinkptr->faps[i] = fop_data->fap[i];
	else curexplinkptr->faps[i] = MAXINT;/*set to interpolate*/
    }
    else if(!(fop_data->viseme_def) && !(fop_data->expression_def)) {
      for(i=0; i<NFAP; i++) if(!fop_data->fap_mask[i] && (curexplinkptr->faps[i] != MAXINT)) {/*this needs to change when MAXINT is accepted to mean interpolate by face models*/
	fop_data->fap[i] = curexplinkptr->faps[i];/*update interp faps with LUT faps*/
	fop_data->fap_mask[i] = 1;
      }
    }  
  }
}

void
decode_ifap(FOP_DATA *fop_data) {
  int i;
  int symbol, qstep;
  void decode_viseme(FOP_DATA*, int);
  void decode_expression(FOP_DATA*, int);
  int dequant(int, int);

  if(fop.fo.fap_mask[0]) {
	decoder_reset();
	decode_viseme(fop_data, 1);
  }
  if(fop.fo.fap_mask[1]) {
	decoder_reset();
	decode_expression(fop_data, 1);
  }

  decoder_reset();
  for(i=2; i<NFAP; i++) {
	if(fop.fo.fap_mask[i]) {
		symbol = decode_a_symbol(fop.fo.Icumul_freq[i]);
		fop.fo.fap_Q[i] = symbol + fop.fo.Irange_min[i];
		qstep = fop.fo.IQP[i]*fop.fo.fap_quant;
		fop.fo.fap_prev[i] = dequant(fop.fo.fap_Q[i], qstep);
		fop_data->fap[i] = fop.fo.fap_prev[i];
	}
  }
  procvisexp(fop_data);/*-edp*/
}

void
decode_pbap(BOP_DATA *bop_data) {
  int i, j, k;
  int skip_frames; 
  int symbol, qstep;
  int dequant(int, int);
  int recon(int, int);

  
  skip_frames = getbits(1);
  if(skip_frames) {
	bap_decode_skip_frames();
  } else {
	fop.bo.number_of_frames_to_skip = 0;
  }
  

  decoder_reset();

  for (k=0; k<BAP_NUM_GROUPS; k++) {
    for(j=0;j<fop.bo.baps_in_group[k]; j++) {
	  i = fop.bo.bap_mask_index[k][j];
	  if(fop.bo.bap_mask[i]) {
	    symbol = decode_a_symbol(fop.bo.cumul_freq[i]);
	    bap_update(symbol, i);
	    fop.bo.bap_Q[i] = symbol + fop.bo.range_min[i];
	    qstep = fop.bo.BQP[i]*fop.bo.bap_quant;
	    fop.bo.bap_diff[i] = dequant(fop.bo.bap_Q[i], qstep);
	    fop.bo.bap_prev[i] = recon(fop.bo.bap_diff[i], fop.bo.bap_prev[i]);
	    bop_data->bap[i] = fop.bo.bap_prev[i];
	  }
    }
  }	
}

void
decode_pfap(FOP_DATA *fop_data) {
  int i;
  int skip_frames;
  int symbol, qstep;
  void decode_viseme(FOP_DATA*, int);
  void decode_expression(FOP_DATA*, int);
  int dequant(int, int);
  int recon(int, int);

  skip_frames = getbits(1);
  if(skip_frames) {
	decode_skip_frames();
  } else {
	fop.fo.number_of_frames_to_skip = 0;
  }
  if(fop.fo.fap_mask[0]) {
	decoder_reset();
	decode_viseme(fop_data, 0);
  }
  if(fop.fo.fap_mask[1]) {
	decoder_reset();
	decode_expression(fop_data, 0);
  }

  decoder_reset();
  for(i=2; i<NFAP; i++) {
	if(fop.fo.fap_mask[i]) {
		symbol = decode_a_symbol(fop.fo.cumul_freq[i]);
		update(symbol, i);
		fop.fo.fap_Q[i] = symbol + fop.fo.range_min[i];
		qstep = fop.fo.QP[i]*fop.fo.fap_quant;
		fop.fo.fap_diff[i] = dequant(fop.fo.fap_Q[i], qstep);
		fop.fo.fap_prev[i] = recon(fop.fo.fap_diff[i], fop.fo.fap_prev[i]);
		fop_data->fap[i] = fop.fo.fap_prev[i];
	}
  }
  procvisexp(fop_data);

}

void
decode_viseme(FOP_DATA *fop_data, int is_intra) {
  int symbol;

  if(is_intra) {
    symbol = decode_a_symbol(fop.fo.Icumul_ve[0]);
    symbol += fop.fo.Irange_ve_min[0];
    fop.fo.ve_prev[0] = symbol;
    fop_data->viseme_1 = fop.fo.ve_prev[0];
    
    symbol = decode_a_symbol(fop.fo.Icumul_ve[1]);
    symbol += fop.fo.Irange_ve_min[1];
    fop.fo.ve_prev[1] = symbol;
    fop_data->viseme_2 = fop.fo.ve_prev[1];

    symbol = decode_a_symbol(fop.fo.Icumul_ve[2]);
    fop.fo.viseme_blend_Q = symbol + fop.fo.Irange_ve_min[2];
    fop.fo.ve_prev[2] = dequant(fop.fo.viseme_blend_Q, fop.fo.fap_quant);
    fop_data->viseme_blend = fop.fo.ve_prev[2];

    adjust_bits();
    fop_data->viseme_def = getbits(1);
  } else {
    symbol = decode_a_symbol(fop.fo.cumul_ve[0]);
    update_viseme_expression(symbol, 0);
    fop.fo.ve_diff[0] = symbol + fop.fo.range_ve_min[0];
    fop.fo.ve_prev[0] = recon(fop.fo.ve_diff[0], fop.fo.ve_prev[0]);
    fop_data->viseme_1 = fop.fo.ve_prev[0];
    
    symbol = decode_a_symbol(fop.fo.cumul_ve[1]);
    update_viseme_expression(symbol, 1);
    fop.fo.ve_diff[1] = symbol + fop.fo.range_ve_min[1];
    fop.fo.ve_prev[1] = recon(fop.fo.ve_diff[1], fop.fo.ve_prev[1]);
    fop_data->viseme_2 = fop.fo.ve_prev[1];
    
    symbol = decode_a_symbol(fop.fo.cumul_ve[2]);
    update_viseme_expression(symbol, 2);
    fop.fo.viseme_blend_Q = symbol + fop.fo.range_ve_min[2];
    fop.fo.ve_diff[2] = dequant(fop.fo.viseme_blend_Q, fop.fo.fap_quant);
    fop.fo.ve_prev[2] = recon(fop.fo.ve_diff[2], fop.fo.ve_prev[2]);
    fop_data->viseme_blend = fop.fo.ve_prev[2];
    
    adjust_bits();
    fop_data->viseme_def = getbits(1);
  }

}

void
decode_expression(FOP_DATA *fop_data, int is_intra) {
  int symbol;
  int dequant(int, int);
  int recon(int, int);

  if(is_intra) {
    symbol = decode_a_symbol(fop.fo.Icumul_ve[3]);
    symbol += fop.fo.Irange_ve_min[3];
    fop.fo.ve_prev[3] = symbol;
    fop_data->expression_1 = fop.fo.ve_prev[3];
    
    symbol = decode_a_symbol(fop.fo.Icumul_ve[4]);
    fop.fo.expression_intensity1_Q = symbol + fop.fo.Irange_ve_min[4];
    fop.fo.ve_prev[4] = dequant(fop.fo.expression_intensity1_Q, fop.fo.fap_quant);
    fop_data->expression_1_intensity = fop.fo.ve_prev[4];

    symbol = decode_a_symbol(fop.fo.Icumul_ve[5]);
    symbol += fop.fo.Irange_ve_min[5];
    fop.fo.ve_prev[5] = symbol;
    fop_data->expression_2 = fop.fo.ve_prev[5];
    
    symbol = decode_a_symbol(fop.fo.Icumul_ve[6]);
    fop.fo.expression_intensity2_Q = symbol + fop.fo.Irange_ve_min[6];
    fop.fo.ve_prev[6] = dequant(fop.fo.expression_intensity2_Q, fop.fo.fap_quant);
    fop_data->expression_2_intensity = fop.fo.ve_prev[6];
    
    adjust_bits();
    fop_data->init_face = getbits(1);
    fop_data->expression_def = getbits(1);
  } else {
    symbol = decode_a_symbol(fop.fo.cumul_ve[3]);
    update_viseme_expression(symbol, 3);
    fop.fo.ve_diff[3] = symbol + fop.fo.range_ve_min[3];
    fop.fo.ve_prev[3] = recon(fop.fo.ve_diff[3], fop.fo.ve_prev[3]);
    fop_data->expression_1 = fop.fo.ve_prev[3];
    
    symbol = decode_a_symbol(fop.fo.cumul_ve[4]);
    update_viseme_expression(symbol, 4);
    fop.fo.expression_intensity1_Q = symbol + fop.fo.range_ve_min[4];
    fop.fo.ve_diff[4] = dequant(fop.fo.expression_intensity1_Q, fop.fo.fap_quant);
    fop.fo.ve_prev[4] = recon(fop.fo.ve_diff[4], fop.fo.ve_prev[4]);
    fop_data->expression_1_intensity = fop.fo.ve_prev[4];
    
    symbol = decode_a_symbol(fop.fo.cumul_ve[5]);
    update_viseme_expression(symbol, 5);
    fop.fo.ve_diff[5] = symbol + fop.fo.range_ve_min[5];
    fop.fo.ve_prev[5] = recon(fop.fo.ve_diff[5], fop.fo.ve_prev[5]);
    fop_data->expression_2 = fop.fo.ve_prev[5];
    
    symbol = decode_a_symbol(fop.fo.cumul_ve[6]);
    update_viseme_expression(symbol, 6);
    fop.fo.expression_intensity2_Q = symbol + fop.fo.range_ve_min[6];
    fop.fo.ve_diff[6] = dequant(fop.fo.expression_intensity2_Q, fop.fo.fap_quant);
    fop.fo.ve_prev[6] = recon(fop.fo.ve_diff[6], fop.fo.ve_prev[6]);
    fop_data->expression_2_intensity = fop.fo.ve_prev[6];
    
    adjust_bits();
    fop_data->init_face = getbits(1);
    fop_data->expression_def = getbits(1);
  }
  
}

int
recon(int fap_diff, int fap_prev) {
  int fap_rec;

  fap_rec = fap_diff + fap_prev;
  return fap_rec;
}

int
dequant(int qfap, int qstep) {


  if(qstep) {
        return qfap*qstep;
  } else {
        return qfap;
  }      
}


/*
void print_fba_header()
{
  if ( bapfp )
     fprintf ( bapfp, "3.1 Figaro %d -1\n", fop.bo.frame_rate );
}
*/
