/*************************************************************************

This software module was originally developed by 
	Wei Wu (weiwu@stallion.risc.rockwell.com),  Rockwell Science Center 
in the course of development of the MPEG-4 Video (ISO/IEC 14496-2). 
This software module is an implementation of a part of one or more MPEG-4 Video
tools as specified by the MPEG-4 Video. ISO/IEC gives users of the MPEG-4 Video
free license to this software module or modifications thereof for use in 
hardware or software products claiming conformance to the MPEG-4 Video. Those 
intending to use this software module in hardware or software products are advised 
that its use may infringe existing patents. The original developer of this software 
module and his/her company, the subsequent editors and their companies, and ISO/IEC 
have no liability for use of this software module or modifications thereof in an 
implementation. Copyright is not released for non MPEG-4 Video conforming 
products. Rockwell Science Center retains full right to use the code for his/her own 
purpose, assign or donate the code to a third party and to inhibit third parties from 
using the code for non <MPEG standard> conforming products. This copyright notice 
must be included in all copies or derivative works. Copyright (c) 1997.

Module Name:    getbits.c

*************************************************************************/

#include "sim.h"
#include <assert.h>

void
initbits() {
  fop.stream.incnt = 0;
}

void
resetbits() {
  fop.stream.bitcnt = 0;
}

int
cntbits() {
  return fop.stream.p * 8 - fop.stream.incnt;
}

/* This function should only be called by getbits() */
int
getbit() {
  int t;
 
  if(fop.stream.incnt==0) {
        if (fop.stream.p >= fop.stream.bufsize) {
	  fprintf(stderr, "Warning: buffer underflowing: %d\n", fop.stream.p);
	  return 0;
	}

	fop.stream.inbfr = fop.stream.buf[fop.stream.p++];
	fop.stream.incnt = 8;
  }
  t = (fop.stream.inbfr&0x80)>>7;
  fop.stream.inbfr <<= 1;
  fop.stream.incnt -= 1;
  return t;
}


int
getbits(int n) {
  int i;
  int ibuf = 0;

  if(n>32) {
        fprintf(stderr, "Cannot fetch more than 32 bits from getbits()\n");
        exit(-1);
  }

  for(i=0; i<n; i++) { /* MSB first */
        ibuf <<= 1;
        ibuf += getbit();
  }

  fop.stream.bitcnt += n;

  return ibuf;
}
 
 
/*
void
alignbits(void) {
 
  fop.stream.bitcnt += fop.stream.incnt;
  fop.stream.incnt = 0;
}
*/

void
next_start_code(void) {
  if(fop.stream.incnt==0) {
	getbits(8);
  } else {
	getbits(fop.stream.incnt);
  }
}

int
nextbits_bytealigned(void) {
  int incnt_save, inbfr_save;
  int sc;
  int flag=0;

  incnt_save = fop.stream.incnt;
  inbfr_save = fop.stream.inbfr;
  if(fop.stream.incnt==0) {
	getbits(8);
	flag = 1;
  }
  getbits(fop.stream.incnt);
  sc = getbits(32);
  fop.stream.incnt = incnt_save;
  fop.stream.inbfr = inbfr_save;
  if(flag==1) {
    fop.stream.p -= 5;
    /*fseek(fop.streamfp, -5, SEEK_CUR);*/
  } else {
    fop.stream.p -= 4;
    /*fseek(fop.streamfp, -4, SEEK_CUR);*/
  }
  return sc;
}


/*
   This function is called right before fix length string is about to
   be outputed after arithmetic coding
*/
void
adjust_bits() {
  int incnt_save;

  fop.stream.p -= 3;
  /*fseek(fop.streamfp, -3, SEEK_CUR);*/
  incnt_save = fop.stream.incnt;
  fop.stream.incnt = 0;
  getbits(8-incnt_save+2);
  fop.stream.bitcnt -= (8-incnt_save+2);
  fop.stream.bitcnt -= 14; /* 16-2 */
}
