/*************************************************************************

This software module was originally developed by 
	Wei Wu (weiwu@stallion.risc.rockwell.com),  Rockwell Science Center 
and modified by
        Tolga K. Capin (capin@lig.di.epfl.ch), EPFL-LIG,
        Joaquim Esmerado (jle@lig.di.epfl.ch), EPFL-LIG.
in the course of development of the MPEG-4 Video (ISO/IEC 14496-2). 
This software module is an implementation of a part of one or more MPEG-4 Video
tools as specified by the MPEG-4 Video. ISO/IEC gives users of the MPEG-4 Video
free license to this software module or modifications thereof for use in 
hardware or software products claiming conformance to the MPEG-4 Video. Those 
intending to use this software module in hardware or software products are advised 
that its use may infringe existing patents. The original developer of this software 
module and his/her company, the subsequent editors and their companies, and ISO/IEC 
have no liability for use of this software module or modifications thereof in an 
implementation. Copyright is not released for non MPEG-4 Video conforming 
products. Rockwell Science Center retains full right to use the code for his/her own 
purpose, assign or donate the code to a third party and to inhibit third parties from 
using the code for non <MPEG standard> conforming products. This copyright notice 
must be included in all copies or derivative works. Copyright (c) 1997.

Module Name:	sim.h

Abstract:	header file for FBA coding

Remarks:

*************************************************************************/

#ifndef __SIM_H_
#define __SIM_H_

#include <stdio.h>
#include <stdlib.h>
#include "common.h"

#ifdef CPLUSPLUS
extern "C" {
#endif

#define FDP_ID	0
#define	FAP_ID	1
#define BAP_ID	2
#define BDP_ID	3

#define NFAP	68
#define N_FAP_GROUP  10

#define RANGE_DELTA	60

#define FAP_PARAMSET_MASK       1
#define BAP_PARAMSET_MASK       2

#define FBA_OBJECT_START_CODE		257
#define FBA_OBJECT_PLANE_START_CODE     259
#define FBA_OBJECT_END_CODE		258

#define PREDICTIVE_CODING_TYPE          0 
#define DCT_CODING_TYPE                   1 

#define MAX_BAPS          30        /* Max no of BAPs that belong to any group */
#define BAP_NUM_GROUPS    19        /* Total no of BAP groups */
#define NUM_BAPS          186       /* Total no of BAPs */

#define SEG_LENGTH        16

#define MAX_NUM_EXTENDABLE_BAPS 256  /* 2^8 */

typedef struct {
  unsigned char fap_mask[68];
  int viseme_1, viseme_2, viseme_blend, viseme_def;
  int expression_1, expression_1_intensity;
  int expression_2, expression_2_intensity;
  int init_face, expression_def;
  int fap[68];
} FOP_DATA;

typedef struct {
  unsigned char bap_mask[NUM_BAPS];
  int bap[NUM_BAPS];
} BOP_DATA;

typedef struct 
{
  FOP_DATA *fop_data;
  bool_t is_time_code;
  int frame_rate, seconds, frequency_offset;
  int time_code_hours, time_code_minutes, marker_bit, time_code_seconds;
  int number_of_frames_to_skip;
  int fap_quant;
  int is_i_new_max, is_i_new_min, is_p_new_max, is_p_new_min;
  int fap_mask[NFAP];
  int fap_mask_type[N_FAP_GROUP];
  int fap_orig[NFAP], fap_Q[NFAP], fap_prev[NFAP], fap_diff[NFAP];
  int ve_orig[7], ve_prev[7], ve_diff[7];
  int viseme_blend_Q, expression_intensity1_Q, expression_intensity2_Q;
  int mask_index[11];
  int QP[NFAP];
  int IQP[NFAP];
  int Idefault_min[NFAP],Idefault_max[NFAP];
  int default_min[NFAP],default_max[NFAP];
  int  Irange_ve_max[7], Irange_ve_min[7];
  int  range_ve_max[7], range_ve_min[7];
  int *Ifreq_ve[7], *Icumul_ve[7];
  int *freq_ve[7], *cumul_ve[7];
  int  Irange_max[NFAP], Irange_min[NFAP];
  int  range_max[NFAP], range_min[NFAP];
  int *freq[NFAP], *cumul_freq[NFAP];
  int *Ifreq[NFAP], *Icumul_freq[NFAP];
  int suggested_gender;
  int coding_type;
  /* DCT part */
  float *dctQP;
  float ** faps;
  int **visemes;
  int **expressions;
  float *dcPred;
  int **dc_table;
  int **ac_table;
  int **runlength_table;
  int **v_sel_table;
  int **v_blend_table;
  int **e_sel_table;
  int **e_intensity_table;
  int dc_table_size;
  int ac_table_size;
  int runlength_table_size;
  int v_sel_table_size;
  int v_blend_table_size;
  int e_sel_table_size;
  int e_intensity_table_size;
  float **idctMatrix;
} face_object_t;

typedef struct 
{
  
  BOP_DATA *bop_data;
  bool_t is_time_code;
  bool_t is_frame_rate;
  bool_t skip_frames;
  int frame_rate, seconds, frequency_offset;
  int time_code_hours, time_code_minutes, marker_bit, time_code_seconds;
  int number_of_frames_to_skip;
  int bap_quant;
  int is_i_new_max, is_i_new_min, is_p_new_max, is_p_new_min;
  int bap_mask[NUM_BAPS];
  int bap_orig[NUM_BAPS], bap_Q[NUM_BAPS],
      bap_prev[NUM_BAPS], bap_diff[NUM_BAPS];
  int bap_mask_type[BAP_NUM_GROUPS];
  
  int bap_mask_index[BAP_NUM_GROUPS][MAX_BAPS];
  int baps_in_group[BAP_NUM_GROUPS];

  int BQP[NUM_BAPS];
  int IBQP[NUM_BAPS];

  int  Irange_max[NUM_BAPS], Irange_min[NUM_BAPS];
  int  range_max[NUM_BAPS], range_min[NUM_BAPS];
  int *freq[NUM_BAPS], *cumul_freq[NUM_BAPS];
  int *Ifreq[NUM_BAPS], *Icumul_freq[NUM_BAPS];

  int suggested_gender;
  int body_object_coding_type;

  int has_extendable_baps;
  int num_extendable_baps;
  int extendable_baps[MAX_NUM_EXTENDABLE_BAPS];

} body_object_t;

typedef struct
{
  unsigned char *buf;
  unsigned char inbfr;
  int p;
  int incnt;
  int bitcnt;
  int bufsize;
} fba_in_buffer;

typedef struct 
{
  bool_t is_intra;
  int fba_paramset;
  fba_in_buffer stream;
  face_object_t fo;
  body_object_t bo;
} fba_object_plane_t;

extern fba_object_plane_t fop;

typedef struct nextexplink explink;
typedef explink *vbdptr;/*7 bit vbdptr array index is viseme_blend*/

extern int vsptrind,vbdptrind,expvals;/*constructed array indices and expvals*/
extern vbdptr **vsptr;/*8 bit vsptr array index is viseme_1(4 msbs)and viseme_2(4 lsbs)*/
extern explink *curexplinkptr,*lastexplinkptr,*newexplinkptr;

/* Adaptive Arithmetic Coding */
#define	Max_frequency	16383
#define sign(x) 	((x) < 0 ? -1 : 1)

void init(void);

int fba_decoder(
    unsigned char *AUptr, 
    int bitoffset, 
    int AUbitcount,
		FOP_DATA *fop_data, 
    BOP_DATA *bop_data,
		int fba_bytecount, 
    unsigned int *PTS);  

int decode_a_symbol(int*);
void decoder_reset(void); 
void bit_out_psc_layer(void);
void update_viseme_expression(int, int);
void update(int, int);
void init_arith(void);

int bap_decode_a_symbol(int*);
void bap_decoder_reset(void); 
void bap_bit_out_psc_layer(void);
void bap_init_arith(void);
void bap_update(int, int);

void initbits(void);
void resetbits(void);
int getbits(int);
int getbit(void);
void next_start_code(void);
int nextbits_bytealigned(void);
void adjust_bits(void);
int cntbits(void);

/* decode exp */
void decode_viseme(FOP_DATA *fop_data, int is_intra);
void decode_expression(FOP_DATA *fop_data, int is_intra);
int  recon(int fap_diff, int fap_prev) ;
int  dequant(int qfap, int qstep) ;

/*viseme/expression LUT variables -edp*/
typedef struct nextexplink explink;

struct nextexplink {
  explink *next;
  int expvals;/*lower 19 bits are packed with the following from MSB to LSB: expression1(3bits),expression_1_intensity(6bits),expression_2(3bits), expression_2_intensity(6bits),init_face(1bit)*/
  int faps[68];/*faps[0] is fap1*/
};

typedef explink *vbdptr;/*7 bit vbdptr array index is viseme_blend*/
#define MAXINT 0x7FFFFFFF /*value to indicate interpolate fap*/


/* DCT related functions */
void DCT_initialize();
void write1Segment(int curFrame, FILE *fapfile);
void init_dctQP();
void init_htables();
void init_dct();
void decode_i_segment();
void decode_p_segment();
void decode_visemes_segment(int);
void decode_expressions_segment(int);
int huffman_decode(int **table, int size);
void idctTrans(float*, float*);
void dctdequant(int*, float*);
int getIDC();
int getPDC();
void getAC(int*);

int  fba_object_plane_data(FOP_DATA* fop_data,BOP_DATA *bop_data, unsigned int *PTS) ;
int  fba_object_plane_header(FOP_DATA* fop_data, BOP_DATA *bop_data, unsigned int *PTS);

#ifdef CPLUSPLUS
}
#endif

#endif
