//=============================================================================
//  This software module was originally developed by IBM Research (T.J. Watson)
//  in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1) 
//  standard. This software module is an implementation of a part of one or 
//  more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 
//  Systems (ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 
//  Systems (ISO/IEC 14496-1) free license to this software module or 
//  modifications thereof for use in hardware or software products claiming 
//  conformance to the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use
//  this software module in hardware or software products are advised that its
//  use may infringe existing patents. The original developer of this software 
//  module and his/her company, the subsequent editors and their companies, and
//  ISO/IEC have no liability for use of this software module or modifications 
//  thereof in an implementation. Copyright is not released for non MPEG-4 
//  Systems (ISO/IEC 14496-1) conforming products. IBM retain full right to use
//  the code for their own purpose, assign or donate the code to a third party 
//  and to inhibit third parties from using the code for non MPEG-4 Systems 
//  (ISO/IEC 14496-1) conforming products. This copyright notice must be 
//  included in all copies or derivative works. Copyright  1999-2000.
//=============================================================================
//  IBM  TJ Watson Research, USA
//-----------------------------------------------------------------------------
//  FLEXTIMECONNECTION.H
//
//  Contact   :
//    Dr. Michelle Kim ( mykim@us.ibm.com )
//
//  Overview  :
//    Declaration of FlexTimeConnection
//
//=============================================================================  


#ifndef FLEXTIMECONNECTION_H__INCLUDED
#define FLEXTIMECONNECTION_H__INCLUDED

#include "ztl.h"
#include "FlexTimeBase.h"

//PAB-: class FlexTimeBase;
class FlexTimeMgr;
class FlexTimeObject;

class FlexTimeConnection : public FlexTimeBase
{
private:
  int     m_iCommands;
  bool    m_bFirstEvalTimeNotSet;
  double  m_dFirstEvalTime;
  double  m_dLastEvalTime;
  bool    m_bHasSwitched;

  // 2 ZArrays to store the starting and the ending objects...
  ZArray<FlexTimeObject*> startingObjects;
  ZArray<FlexTimeObject*> endingObjects;
  
  //  The folowing integer does not have to be unique as far as the class is
  //  concerned, but will be so in the FlexTimeMgr
  int     m_nId; 

  //  Evaluate whether this Connection should switch-over by ending the
  //  "ending" objects and starting the "starting" objects.
  bool evaluateSwitchOver(double, double &dActualSwitchOverTime);

  //  Execute the switch-over for this Connection 
  void executeSwitchOver(double);

  void miniTrace();
  
public:
  FlexTimeConnection(int nId);
  ~FlexTimeConnection();

  //  Returns the ID of a Flextime connection. This connection is used to link
  //  FlextimeObjects together
  int GetId() {return m_nId;}

  //  Add objects to the current connection.
  void AddStartingObject(FlexTimeObject*);
  void AddEndingObject(FlexTimeObject*); 

  //  Delete objects to the current connection.
  void DeleteStartingObject(FlexTimeObject*);
  void DeleteEndingObject(FlexTimeObject*); 


  // Accessors to the ZArrays
  int  GetNumStartingObjects();
  FlexTimeObject *GetStartingObject(int);
  
  // Accessors to the ZArrays
  int  GetNumEndingObjects();
  FlexTimeObject *GetEndingObject(int);
  
  //  Evaluate the switch-over for this Connection and execute it 
  //  if appropriate
  bool evaluate(double);

  //  Added in order to force the switchOver
  void ForceSwitchOver(double); 
  void trace();

  bool IsLinked() const { return (startingObjects.GetLength()>0)&&(endingObjects.GetLength()>0); }

  bool IsClass(const char *pszClassName) const;

};


#define NOT_CONNECTED -1
#endif // FLEXTIMECONNECTION_H__INCLUDED