//=============================================================================
//  This software module was originally developed by IBM Research (T.J. Watson)
//  in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1) 
//  standard. This software module is an implementation of a part of one or 
//  more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 
//  Systems (ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 
//  Systems (ISO/IEC 14496-1) free license to this software module or 
//  modifications thereof for use in hardware or software products claiming 
//  conformance to the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use
//  this software module in hardware or software products are advised that its
//  use may infringe existing patents. The original developer of this software 
//  module and his/her company, the subsequent editors and their companies, and
//  ISO/IEC have no liability for use of this software module or modifications 
//  thereof in an implementation. Copyright is not released for non MPEG-4 
//  Systems (ISO/IEC 14496-1) conforming products. IBM retain full right to use
//  the code for their own purpose, assign or donate the code to a third party 
//  and to inhibit third parties from using the code for non MPEG-4 Systems 
//  (ISO/IEC 14496-1) conforming products. This copyright notice must be 
//  included in all copies or derivative works. Copyright  1999-2000.
//=============================================================================
//  IBM  TJ Watson Research, USA
//-----------------------------------------------------------------------------
//  FLEXTIMEMGR.CPP
//
//  Contact   :
//    Dr. Michelle Kim ( mykim@us.ibm.com )
//
//  Overview  :
//    The implementation of the proxy object used by the presenter to handle 
//    the TemporalForm node.
//
//=============================================================================  


#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include "defs.h"

#include "FlexTimeMgr.h"
// SOLARIS PORTING - typo in next 2 lines
//#include "Spring.h"
//#include "TCNSimple.h"
#include "spring.h"
#include "TCNsimple.h"
#include "FlexTimeBase.h"

#include "TemporalGroupImpFlex.h"
#include "TemporalTransformImpFlex.h"
#include "FlxTime.h"
#include "FlexTimeObject.h"
#include "FlexTimeConnection.h"


// needed to get the system time in the Evaluate() function
#include <sys/timeb.h>
#include <time.h>

#define FT_TRACE true

//  Initialization of the static member of the FlexTimeManager Object
FlexTimeMgr *FlexTimeMgr::s_pftmRoot = NULL;


//-----------------------------------------------------------------------------
//  Constructor to create a Manager with a TemporalForm node.
//  @param node Pointer to the node.
//  @param pSource Inline node that constructs this manager.
//-----------------------------------------------------------------------------
FlexTimeMgr::FlexTimeMgr()
{
  m_bSceneStarted = false;

  //PAB_NL: starts the counter from 0 (NOT_CONNECTED meaning not connected yet)
  m_iConnectionId = 0;
  m_pTCN = new TCNSimple();

  //  Insert this new FlexTime object set the root as the newly create object
  SetFlexNext(GetFlexRoot());
  SetFlexRoot(this);
}


//-----------------------------------------------------------------------------
//  Destructor
//-----------------------------------------------------------------------------
FlexTimeMgr::~FlexTimeMgr()
{
  FlexTimeMgr *pftm = GetFlexRoot();

  // the object we want to destroy is not the root
  if(pftm!=this) {
    while(pftm) {
      if(pftm->GetFlexNext()==this) {
        pftm->SetFlexNext(GetFlexNext());
        goto exit;
      }
      pftm = pftm->GetFlexNext();
	}
  
	// This FlexTimeManager should have been chained...
	  assert(true==false);
  }
  else
    SetFlexRoot(GetFlexNext());

exit:
  delete m_pTCN;
}


//-----------------------------------------------------------------------------
//  Get the current system time in seconds (is time since 1 jan 1970)
//-----------------------------------------------------------------------------
double FlexTimeMgr::GetSysTime()
{
#ifdef WIN32
  struct _timeb timebuffer;
  _ftime(&timebuffer);
  return (double) timebuffer.time + (double) timebuffer.millitm / 1000.0;
#else
// SOLARIS PORTING
  struct timeval loc_time;
  ::gettimeofday( &loc_time, NULL );
  return (double) loc_time.tv_sec + (double) loc_time.tv_usec/1000000.0;
#endif
}


//-----------------------------------------------------------------------------
//  Evaluate the situation to see if connections need to be switched.
//  This method should be called by something ike the renderer between render 
//  cycles to asses the situation and possibly update it.
//-----------------------------------------------------------------------------
bool FlexTimeMgr::Evaluate()
{
  Lock();

  //  not started yet: evaluation need not be done yet
  if(!m_bSceneStarted)
  {
    StartScene();
    UnLock();
    return false;
  }

  //  get current system time in milliseconds (is time since 1 jan 1970)
  double dCurrentTime = GetSysTime();

  //  evaluate all connections
  bool ret = FALSE;
  int iCount = GetFTConnectionCount();
  for(int iNum=0; iNum<iCount; iNum++) 
  {
    //  evaluate if this connection switches over
    //  returns true if it has executed the switchover
    if(m_zaFTConnections[iNum]->evaluate(dCurrentTime)) 
    {
		ret = true;

      TRACE(  "FlexTimeMgr>    Switched connection %d at time = %.3fs \n",
              m_zaFTConnections[iNum]->GetId(), dCurrentTime-m_dStartTime);
      // remove all ending FTObjects
      //PAB_NL: for now I ll keep them (for the current TCN quick algo )
      //PAB_NL: this would not work for long presentations
      //PAB_NL:TRACE("FlexTimeMgr>      removing ending %d FTObjects:\n",
      //  connections[iNum]->getiNumEndingObjects());
      //PAB_NL: 
      //for(int obj=0; obj<m_zaFTConnections[iNum]->getiNumEndingObjects(); obj++) {
      //PAB_NL: object = m_zaFTConnections[iNum]->getEndingObject(obj);
      //PAB_NL: object->trace();
      //PAB_NL: DeleteFTObject(object);
      //}
      // remove the connection itself
      //PAB_NL: DeleteFTConnection(iNum--);
      // re-evaluate durations
    }
  }
  TRACE(".");

  UnLock();
  return ret;
}


//-----------------------------------------------------------------------------
//  Deletes the connection whose adress is pFTCon and remove it from our list
//-----------------------------------------------------------------------------
bool FlexTimeMgr::DeleteFTConnection(FlexTimeConnection *pftc)
{
  //  First find it in the table
  for(int i=0; i<GetFTConnectionCount(); i++)
  {
    if(m_zaFTConnections[i] == pftc)
    {
      m_zaFTConnections.DeleteFrom(i);
      return true;
    }
  }
  return false;
}


//-----------------------------------------------------------------------------
//  Instantiate a new connection to be used new connection should only 
//  be instantiated through that.
//  ensures that each connection has a unique Id number
//-----------------------------------------------------------------------------
FlexTimeConnection *FlexTimeMgr::CreateFTConnection()
{
  FlexTimeConnection *pNewCon;

  // Connection array management
  pNewCon = new FlexTimeConnection(m_iConnectionId++);
  pNewCon->SetFlexTimeMgr(this);

  assert(pNewCon->GetId()>=0);
  m_zaFTConnections += pNewCon;
  return pNewCon;
}


//-----------------------------------------------------------------------------
//  Merge the 2 connections into the first one, deletes the obsolete one
//  returns true if OK
//-----------------------------------------------------------------------------
bool FlexTimeMgr::MergeFTConnection(FlexTimeConnection *pftcA,
                                    FlexTimeConnection *pftcB)
{
  //  I could/should  move part of this in a 
  //  FlexTimeConnection::MergeInto(pFTCon);
  
  if(pftcA==pftcB)
    return true;

  //  If one (and only one) of the 2 connections does not exist, we re done
  if((pftcA==NULL) ^ (pftcB==NULL))
    return true;
  
  if(pftcA==NULL && pftcB==NULL)
    return false;
  
  //  if the 2 connections exist, merge the second one in the first
  for(int i=0; i<pftcB->GetNumStartingObjects(); i++) 
    pftcA->AddStartingObject(pftcB->GetStartingObject(i));
  
  for(i=0; i<pftcB->GetNumEndingObjects(); i++) 
    pftcA->AddEndingObject(pftcB->GetEndingObject(i));
  
  //  the FlexTimeConnection B is not used anymore -> we have to delete it
  return DeleteFTConnection(pftcB);
}


//-----------------------------------------------------------------------------
//  Remove a FlexTimeObject from the list. If the object cannot be
//  found in the list this method doesn't do anything.
//  @param object FlexTimeObject into the list.
//-----------------------------------------------------------------------------
void FlexTimeMgr::DeleteFTObject(FlexTimeObject *pFTObject)
{
  // first find it
  int iCount = GetFTObjectCount();
  for(int iNum=0; iNum<iCount; iNum++)
  {
    if(m_zaFTObjects[iNum] == pFTObject)
    {
      // the object is found -> delete it
      m_zaFTObjects.DeleteFrom(iNum);
      delete pFTObject;
      return;
    }
  }
  // the object is not found
}


//-----------------------------------------------------------------------------
//  Print the contents.
//-----------------------------------------------------------------------------
void FlexTimeMgr::trace()
{
  int iCount = GetFTObjectCount();
  TRACE("\n\nObjects (count=%d):\n",iCount);
  
  for(int iNum = 0; iNum < iCount; iNum++)
    m_zaFTObjects[iNum]->trace();
  
  iCount = GetFTConnectionCount();
  TRACE("\nConnections (count=%d):\n",iCount);
  
  for(iNum=0; iNum<iCount; iNum++)
  {
    TRACE("  Connection %d:\n",iNum);
    m_zaFTConnections[iNum]->trace();
  }
  TRACE("\n");
}


//-----------------------------------------------------------------------------
//  Mutex protection management
//-----------------------------------------------------------------------------
bool FlexTimeMgr::Lock()
{
  m_mtxLockFTMgr.P();
  return true;
}

bool FlexTimeMgr::UnLock()
{
  m_mtxLockFTMgr.V();
  return true;
}



//-----------------------------------------------------------------------------
//  Called when the scene parser parses a temporalGroup
//  adds the TemporalGroup in the constraint network
//-----------------------------------------------------------------------------
void FlexTimeMgr::AddTemporalGroup(TemporalGroupImpFlex *pTG)
{
  //  Return if there is no child in this TemporalGroup
  if(pTG->GetChildCount() == 0)
    return;

  //  Create a flexTimeObject for the TemporalGroup
  //  (this should be the only time we parse it)
  assert(pTG->GetFTObject() == NULL);

  FlexTimeObject *pFTObject = NULL;
 
  //  add this TG as a TemporalParent of its children:
  int iPriorityCount = pTG->GetPriorityCount();
  int iChildCount = pTG->GetChildCount();
  for(int i=0;i<iChildCount;i++) 
  {
    //  We make a 'safe' dynamic cast from a MediaObject to a TemporalNode
    TemporalNode *pTNChild = pTG->GetTemporalNode(i);

    //  The TemporalGroup children field specifies the list of TTransform OR
    //  TGroup. So this pointer should never be NULL !
    assert(pTNChild);
    pTNChild->SetFlexTimeMgr(this);
    pTNChild->AddTemporalParent(pTG);
    pTNChild->SetTempPriority( (i<iPriorityCount) ? pTG->priority[i] : 0 );
  }

  //  The TG itself should not have any preexisting StartVertex& EndVertex,
  //  because this is the first time we parse this TG
  FlexTimeObject *pFTGroup = GetFTObject(pTG);
  assert(pFTGroup->GetStartConnection() == NULL);
  assert(pFTGroup->GetEndConnection() == NULL);

  // Boolean caching
  bool bCostart = (Bool) pTG->costart;
  bool bCoend   = (Bool) pTG->coend;

  double dOptimal = 0;

  FlexTimeConnection *pftcStart = NULL;
  FlexTimeConnection *pftcEnd   = NULL;

  //  Add a CoStart/CoEnd TemporalGroup ---------------------------------------
  if(bCostart || bCoend)
  {
    //  This can still be more optimized to avoid create born-to-be-merged
    //  connection:
    for(int i=0;i<pTG->GetChildCount();i++)
    {
      //  Check that the new one is either a TT or TG (a TemporalNode)
      pFTObject = GetFTObject( pTG->GetTemporalNode(i));

      // Just to be sure that the pointer is valid
      assert(pFTObject);

      //  the optimal time for a Costart/Coend TemporalGroup will be set to the 
      //  highest optTimes of its children:
      if (pFTObject->GetOptDuration() > dOptimal)
        dOptimal = pFTObject->GetOptDuration();

      //  This test could be outside of the loop, but it avoids looping twice
      //  (once for costart, once for coend) on the childrenField.
      if(bCostart)
        AddCoStart(pftcStart, pFTObject);
      else if(!pFTObject->GetStartConnection())
        // if the startvertex doesn't exits, we have to create a new one
        CreateFTConnection()->AddStartingObject(pFTObject);

      if(bCoend)
        AddCoEnd(pftcEnd, pFTObject);
      else if(pFTObject->GetEndConnection() == NULL)
        // if the startvertex doesn't exits, we have to create a new one
        CreateFTConnection()->AddEndingObject(pFTObject);
    }

    // Connect the FlexTimeObject representing the Group itself...
    (pftcStart?pftcStart:CreateFTConnection())->AddStartingObject(pFTGroup);
    (pftcEnd?pftcEnd:CreateFTConnection())->AddEndingObject(pFTGroup);
  }
  //  Add a Meet TemporalGroup ------------------------------------------------
  else if((Bool)pTG->meet)
  { 
    for(int i=0;i<pTG->GetChildCount();i++)
    {
      //  Check that the new one is either a TT or TG (a spring)
      pFTObject = GetFTObject( pTG->GetTemporalNode(i) );
      assert(pFTObject);

      //  the optimal time for a Meet TEmporalGroup will be set to the sum of
      //  optTimes of its child:
      dOptimal += pFTObject->GetOptDuration();

      AddMeet(pftcStart,pftcEnd,pFTObject);
      pftcStart = pftcEnd;
      pftcEnd = NULL;
    }

    //  Init the endVertex of the group
    if (pFTObject) {//PAB+:
      pFTObject->GetEndConnection()->AddEndingObject(pFTGroup);
    } //PAB+:
    
    //  and the StartVertex of the group
    pFTObject = GetFTObject( pTG->GetTemporalNode(0) );
    pFTObject->GetStartConnection()->AddStartingObject(pFTGroup);

  }

  pFTGroup->SetOptDuration(dOptimal);
 
  if(m_bSceneStarted)
  {
    //  This is the case of a TemporalGroup added once the scene has begun
    //  need to update all the TCN

    int iSpringCount;
    Spring **psa = GetSpringArray(&iSpringCount);
    m_pTCN->Reset(psa,iSpringCount);
    delete psa;

    bool bConsistent = m_pTCN->Evaluate();
    if(!bConsistent)
      TRACE("INCONSISTENT TCN !!\n");
  }
}



//-----------------------------------------------------------------------------
//  This method is used to get a clean array of spring pointers from the array 
//  of FlexTime objects
//-----------------------------------------------------------------------------
Spring **FlexTimeMgr::GetSpringArray(int *piCount)
{
  int iCount = GetFTObjectCount();
  if(!iCount)
    return NULL;

  int iSpringCount = 0;

  Spring** psa = new  Spring*[iCount];

  for(int i=0;i<iCount;i++)
  {
    // this way, we can remove non constraint objects (such as temporal groups
    // from the TCN calculation
    if(!m_zaFTObjects.ElementAt(i)->GetTemporalNode()->IsClass("TemporalGroupImpFlex"))
    {
        psa[iSpringCount++] = (Spring*) m_zaFTObjects.ElementAt(i);
    }
  }

  if(piCount)
    *piCount=iSpringCount;

  return psa;
}


//-----------------------------------------------------------------------------
//  Some of the durations have been updated, reCompute the TCN,
//  and update the FeasibleMins and Maxs.
//-----------------------------------------------------------------------------
void FlexTimeMgr::UpdateFeasibleDurations()
{
  bool bNetworkConsistent = m_pTCN->Evaluate();
  
  if(!bNetworkConsistent)
  {
    //  This is a very light error management :-(
    TRACE ("FTMgr: the Temporal Constraints can not be fully respected any more\n");
    trace();
    m_pTCN->DumpMatrix();
//SOLARIS PORTING
#ifdef WIN32
    _asm int 3;
#endif
  }
  m_pTCN->UpdateFeasibleLengths();
}


//-----------------------------------------------------------------------------
//  Start the scene
//  returns true if we actually started it, 
//  false otherwise ( no TemporalNodes yet, inconsistent network,
//  scene already started...)
//-----------------------------------------------------------------------------
bool FlexTimeMgr::StartScene()
{
  //  returns if scene has already started
  if(m_bSceneStarted) 
    return false;
  
  //  returns if there is no TemporalNodes have been parsed
  if(GetFTConnectionCount() == 0) 
    return false;
  
  //  load the TCN & minimize it 
  int iSpringCount;
  Spring **psa = GetSpringArray(&iSpringCount);
  
  bool bRc = m_pTCN->Reset(psa,iSpringCount);
  delete psa;
  
  if(!bRc)
    return false;
  
  if(m_pTCN->Evaluate() == false)
  {
    TRACE ( " INCONSISTENT TCN \n");
    return false;
  }
  
  //  determine the first connection, start it ...
  FlexTimeConnection * pFirstCon = FindFTConnection( m_pTCN->FindFirstConnectionId() );
  
  if(pFirstCon == NULL)
    return false;
  
  m_pTCN->UpdateFeasibleLengths();
  
  m_dStartTime = GetSysTime();
  
  TRACE("FlexTimeMgr--> StartScene\n");
  
  //  with the current spec, I have to start it right away
  pFirstCon->ForceSwitchOver( m_dStartTime);//GetSysTime() );
  
  m_bSceneStarted = true;
  
  trace();
  return true;
}


//-----------------------------------------------------------------------------
//  GetFTObject() : Get the FlexTime Object for this TemporalNode,
//  creates it if necessary + pointer management.
//-----------------------------------------------------------------------------
FlexTimeObject *FlexTimeMgr::GetFTObject(TemporalNode *pTemporalNode)
{
  assert(pTemporalNode != NULL);
  //PAB_NL:  if CreateFTObject() is called somewhere else, 
  // I won't store any pointer to the FTObject here !!
  FlexTimeObject * pFTObject = pTemporalNode->GetFTObject();
  
  if(pFTObject == NULL)
  {
    pTemporalNode->SetFlexTimeMgr(this);
    pFTObject = pTemporalNode->CreateFTObject();
    m_zaFTObjects+= pFTObject ;
  }
  pFTObject->SetFlexTimeMgr(this);
  return pFTObject;
}


//-----------------------------------------------------------------------------
//  Find a Connection From its Id #
//  I could also have only pointers to Connections instead of Id
//  (that would speed up the thing)
//  I made the move , so this one should not be needed any more
//  I keep it 'cause I have not made the move to pointer in the TCN
//-----------------------------------------------------------------------------
FlexTimeConnection *FlexTimeMgr::FindFTConnection(int iId)
{
  int iCount = GetFTConnectionCount();
  for(int i=0; i<iCount; i++)
  {
    if(m_zaFTConnections[i]->GetId() == iId)
      return m_zaFTConnections[i];
  }
  return NULL;
}


//-----------------------------------------------------------------------------
//  creates a new flextimeManager and insert it at the begining of a built-in
//  chained list
//-----------------------------------------------------------------------------
FlexTimeMgr *GetFlexTimeManagerRoot()
{
  return FlexTimeMgr::s_pftmRoot;
}



//-----------------------------------------------------------------------------
// Add a meet temporal constraint between two vertices and a FT object
//-----------------------------------------------------------------------------
void FlexTimeMgr::AddMeet( FlexTimeConnection *&pftcStart,
                           FlexTimeConnection *&pftcEnd,
                           FlexTimeObject *pfto)
{
  AddCoStart(pftcStart, pfto);
  AddCoEnd(pftcEnd,     pfto);
}


//-----------------------------------------------------------------------------
// Add a costart temporal constraint between the start vertices and a FT object
//-----------------------------------------------------------------------------
void FlexTimeMgr::AddCoStart(FlexTimeConnection *&pftc,FlexTimeObject *pfto)
{
  if(pftc==NULL)
  {
    if(!pfto->GetStartConnection())
    {
      //  let's create a new connection if the start connection does not exists
      pftc = CreateFTConnection();
      pftc->AddStartingObject(pfto);
    }
    else
    {
      //  this child alreay has a EndVertex
      pftc = pfto->GetStartConnection();
    }
  }
  else
  {
    if(!pfto->GetStartConnection())
    {
      //  this child does not have a EndVertex -> we just add it to the Ending
      //  object list of the connection
      pftc->AddStartingObject(pfto);
    }
    else
    {
      // both the FlexTimeObject's EndVertex and the FlexTimeConnection 
      MergeFTConnection(pftc,pfto->GetStartConnection());
    }
  }
}


//-----------------------------------------------------------------------------
// Add a coend temporal constraint between the start vertices and a FT object
//-----------------------------------------------------------------------------
void FlexTimeMgr::AddCoEnd(FlexTimeConnection *&pftc,FlexTimeObject *pfto)
{
  if(pftc==NULL)
  {
    if(!pfto->GetEndConnection())
    {
      //  let's create a new connection if the end connection does not exists
      pftc = CreateFTConnection();
      pftc->AddEndingObject(pfto);
    }
    else
    {
      //  this child alreay has a EndVertex
      pftc = pfto->GetEndConnection();
    }
  }
  else
  {
    if(!pfto->GetEndConnection())
    {
      //  this child does not have a EndVertex -> we just add it to the Ending
      //  object list of the connection
      pftc->AddEndingObject(pfto);
    }
    else
    {
      // both the FlexTimeObject's EndVertex and the FlexTimeConnection 
      MergeFTConnection(pftc,pfto->GetEndConnection());
    }
  }
}