//=============================================================================
//  This software module was originally developed by IBM Research (T.J. Watson)
//  in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1) 
//  standard. This software module is an implementation of a part of one or 
//  more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 
//  Systems (ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 
//  Systems (ISO/IEC 14496-1) free license to this software module or 
//  modifications thereof for use in hardware or software products claiming 
//  conformance to the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use
//  this software module in hardware or software products are advised that its
//  use may infringe existing patents. The original developer of this software 
//  module and his/her company, the subsequent editors and their companies, and
//  ISO/IEC have no liability for use of this software module or modifications 
//  thereof in an implementation. Copyright is not released for non MPEG-4 
//  Systems (ISO/IEC 14496-1) conforming products. IBM retain full right to use
//  the code for their own purpose, assign or donate the code to a third party 
//  and to inhibit third parties from using the code for non MPEG-4 Systems 
//  (ISO/IEC 14496-1) conforming products. This copyright notice must be 
//  included in all copies or derivative works. Copyright  1999-2000.
//=============================================================================
//  IBM  TJ Watson Research, USA
//-----------------------------------------------------------------------------
//  FLEXTIMEMGR.H
//
//  Contact   :
//    Dr. Michelle Kim ( mykim@us.ibm.com )
//
//  Overview  :
//      Declaration of the FlexTimeMgr which implements the functions of the 
//      TemporalForm BIFS node.
//
//=============================================================================  


#ifndef FLEXTIMEMGR_H__INCLUDED
#define FLEXTIMEMGR_H__INCLUDED


#include "../Include/ztl.h"

class TemporalGroupImpFlex;
class TemporalTransformImpFlex;
class FlexObjectImp ;
class MediaObject;

class FlexTimeConnection;
class FlexTimeObject;
class TemporalNode;
class TCNSimple;
class Spring;



//=============================================================================
//  FlexTimeMgr
//=============================================================================
class FlexTimeMgr
{
private:
  //  the stuff here is the implementation of a clean chained-list
  //  implementation
  static FlexTimeMgr *s_pftmRoot;
  FlexTimeMgr *m_pftmNext;

  //  this 'friendly' global function returns the root of our chained list
  friend FlexTimeMgr *GetFlexTimeManagerRoot();
  
  //  returns the root of the chained-list
  FlexTimeMgr *GetFlexRoot() const    { return s_pftmRoot; }
  
  //  these methods are accessors for the chained-list
  // MPEGJ porting - As dummy InlineImp s are created, we check
  // to see if s_pftmRoot is NULL or not.
  void SetFlexRoot(FlexTimeMgr *pftm) { if( s_pftmRoot == NULL ) { s_pftmRoot=pftm; } }
  void SetFlexNext(FlexTimeMgr *pftm) { m_pftmNext=pftm; }
  
  //  these methods control the ZMutex m_mtxLockFTMgr.
  ZMutex m_mtxLockFTMgr;
  bool Lock();
  bool UnLock();

  //  The TCNSimple : it is used to check the network consistency.
  TCNSimple * m_pTCN;

  //  these two arrays contains the whole structure of your TCN
  ZArray<FlexTimeConnection*> m_zaFTConnections;
  ZArray<FlexTimeObject*>     m_zaFTObjects;

  //  we need this boolean to check if we need to recompute the consistency of
  //  the TCN network
  bool m_bSceneStarted;  //PAB_NL:

  // m_dStartTime will be the time reference for the FlexTimeManager.
  double m_dStartTime;

  //PAB_NL: new stuff post_NL
  //  we uses this integer to generate new connections id
  int m_iConnectionId;

  //  turns the internal ZArray of FlexTimeObject into an Array of Spring 
  //  (basically just a cast)
  Spring** GetSpringArray(int *piCount=NULL);

  bool StartScene();
  FlexTimeConnection * FindFTConnection(int iId);

  void AddCoStart(FlexTimeConnection *&pftc,FlexTimeObject *pfto);
  void AddCoEnd(FlexTimeConnection *&pftc,FlexTimeObject *pfto);
  void AddMeet( FlexTimeConnection *&pftcStart,FlexTimeConnection *&pftcEnd,
                FlexTimeObject *pfto);

public:
  FlexTimeMgr();
  ~FlexTimeMgr();

  void UpdateFeasibleDurations();
  //JLF: now returns true if scene state has changed
  bool Evaluate();


  // interface definition to modify the TCN
  void AddTemporalGroup(TemporalGroupImpFlex *pTG);
  


  // FlexTime Object
  FlexTimeObject * GetFTObject(TemporalNode*);
  void DeleteFTObject(FlexTimeObject*);
  //  returns the number of Flextime Objects
  int  GetFTObjectCount() const  { return m_zaFTObjects.GetLength(); }


  // FlexTime Connection
  FlexTimeConnection * CreateFTConnection();
  bool MergeFTConnection(FlexTimeConnection *pftcA, FlexTimeConnection *pftcB);
  bool DeleteFTConnection(FlexTimeConnection *pftc);
  //  returns the number of Connections
  int  GetFTConnectionCount() const  { return m_zaFTConnections.GetLength(); }


  //  returns the current system time (in second)
  double GetSysTime();

  void trace();

  //  return the next FlexTimeManager of the chained list
  FlexTimeMgr *GetFlexNext() const  {return m_pftmNext;}

};


#endif // FLEXTIMEMGR_H__INCLUDED