//=============================================================================
//  This software module was originally developed by IBM Research (T.J. Watson)
//  in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1) 
//  standard. This software module is an implementation of a part of one or 
//  more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 
//  Systems (ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 
//  Systems (ISO/IEC 14496-1) free license to this software module or 
//  modifications thereof for use in hardware or software products claiming 
//  conformance to the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use
//  this software module in hardware or software products are advised that its
//  use may infringe existing patents. The original developer of this software 
//  module and his/her company, the subsequent editors and their companies, and
//  ISO/IEC have no liability for use of this software module or modifications 
//  thereof in an implementation. Copyright is not released for non MPEG-4 
//  Systems (ISO/IEC 14496-1) conforming products. IBM retain full right to use
//  the code for their own purpose, assign or donate the code to a third party 
//  and to inhibit third parties from using the code for non MPEG-4 Systems 
//  (ISO/IEC 14496-1) conforming products. This copyright notice must be 
//  included in all copies or derivative works. Copyright  1999-2000.
//=============================================================================
//  IBM  TJ Watson Research, USA
//-----------------------------------------------------------------------------
//  FLEXTIMEOBJECT.H
//
//  Contact   :
//    Dr. Michelle Kim ( mykim@us.ibm.com )
//
//  Overview  :
//    FlexTime Object structure
//
//=============================================================================  


#ifndef FLEXTIMEOBJECT_H__INCLUDED
#define FLEXTIMEOBJECT_H__INCLUDED

class FlexTimeConnection;
class TemporalNode;

// SOLARIS PORTING - typo
//#include "Spring.h" //PAB+:
#include "spring.h" //PAB+:
#include "FlexTimeBase.h" //PAB+:

#define MAX_DURATION 1E6

class FlexTimeObject: public Spring, public FlexTimeBase
{
private:

  double m_dEndTime;
  double m_dStartTime;
  double optDuration;
  double m_dActualDuration;
  
  bool   m_bActive;
  bool   m_bStartAuReceived;
  bool   m_bEndAuReceived;
  
  FlexTimeConnection * m_pStartFTConnection;
  FlexTimeConnection * m_pEndFTConnection;

  // the priority of the current object
  float m_fPriority;
  
  
  //bool   streamReady;
  
  int   odId;
  
  int   stretchMode;
  int   shrinkMode;
  TemporalNode *m_pTemporalNode;
  
  
  double getMinEndTime();
  double getOptEndTime();
  double getMaxEndTime();
  
#ifdef _DEBUG
  int m_nId;
  static int s_nIdCounter;
#endif
  
public:
  FlexTimeObject(double dMin, double dMax, double dOpt,
    float fPriority, TemporalNode *pTemporalNode);
  ~FlexTimeObject();
  
  void SetStartConnection(FlexTimeConnection *pFTCon); 
  FlexTimeConnection *GetStartConnection() { return m_pStartFTConnection; }
  
  void SetEndConnection(FlexTimeConnection *pFTCon);
  FlexTimeConnection *GetEndConnection() { return m_pEndFTConnection; }
  
  bool IsReadyToStart();
  bool IsReadyToEnd(double dTime);
  bool IsActive() {return m_bActive;}
  bool OptTimeReached(double dTime, double &dActualOptTime);
  bool MaxTimeReached(double dTime, double &dActualMaxTime);
  
  void  SetPriority(float fPriority);
  float GetPriority() const { return m_fPriority; }
  
  double SetOptDuration(double dOptimal) { return optDuration=dOptimal;}
  double GetOptDuration() { return optDuration; }
  double GetActualDuration() { return m_dActualDuration; }
  
  TemporalNode* GetTemporalNode() const { return m_pTemporalNode; }
  
  void   putStartAu();
  void   putEndAu();
  
  void   Start(double dTime);
  void   End(double dTime);
  
  void   trace();
  
  // used to know when the TemporalNode has started
  void SetStartTime(double dStartTime) { m_dStartTime = dStartTime; }
  double GetStartTime() const { return m_dStartTime; }
  
  // return true if the maxdelay of an object has been reach
  bool MaxDelayHasExpired();


  virtual bool IsClass(const char *pszClassName) const;
};

#endif // FLEXTIMEOBJECT_H__INCLUDED
