//=============================================================================
//  This software module was originally developed by IBM Research (T.J. Watson)
//  in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1) 
//  standard. This software module is an implementation of a part of one or 
//  more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 
//  Systems (ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 
//  Systems (ISO/IEC 14496-1) free license to this software module or 
//  modifications thereof for use in hardware or software products claiming 
//  conformance to the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use
//  this software module in hardware or software products are advised that its
//  use may infringe existing patents. The original developer of this software 
//  module and his/her company, the subsequent editors and their companies, and
//  ISO/IEC have no liability for use of this software module or modifications 
//  thereof in an implementation. Copyright is not released for non MPEG-4 
//  Systems (ISO/IEC 14496-1) conforming products. IBM retain full right to use
//  the code for their own purpose, assign or donate the code to a third party 
//  and to inhibit third parties from using the code for non MPEG-4 Systems 
//  (ISO/IEC 14496-1) conforming products. This copyright notice must be 
//  included in all copies or derivative works. Copyright  1999-2000.
//=============================================================================
//  IBM  TJ Watson Research, USA
//-----------------------------------------------------------------------------
//  PRESENTERFLEX.CPP
//
//  Contact   :
//    Dr. Michelle Kim ( mykim@us.ibm.com )
//
//  Overview  :
//    Implementation of a FlexTime Presenter
//    Players implementing FlexTime should derive their Presenter from this one
//    rather than directly from PresenterBase.
//
//=============================================================================  

#include "defs.h"
#include "PresenterFlex.h"
#include "FlexTimeMgr.h"

#define FTMGR_CYCLE 1 //PAB+: the FlexTime Mgr Evaluate() will be called 
                      //PAB+: every FTMGR_CYCLE Renderer cycle(s)

PresenterFlex::PresenterFlex(Executive* pExecutive, HWND hWnd)
  : PresenterBase (pExecutive, hWnd)
{
  m_iCycleCount = 0;
}

PresenterFlex::~PresenterFlex()
{
}

//-----------------------------------------------------------------------------
//  Regroups the calls that have to be driven by the render cycle.
//-----------------------------------------------------------------------------
bool PresenterFlex::PresenterDrivenEvents()
{
	bool res = FALSE;
	if (m_iCycleCount++ > FTMGR_CYCLE) { 
		m_iCycleCount = 0; //PAB+: 
		GetExecutive()->LockScene();
		
		// evaluate the FlexTimeManager (one for each inline)
		FlexTimeMgr * pFlexTimeMgr = GetFlexTimeManagerRoot();
		while(pFlexTimeMgr) {
			if (pFlexTimeMgr->Evaluate()) res = true;
			pFlexTimeMgr = pFlexTimeMgr->GetFlexNext();
		}
		GetExecutive()->UnlockScene();
	} 
	return res;
}

/*-------------------------------------------------------------
ZL: create and delete FlexTimeMgrs here
-------------------------------------------------------------*/
FlexTimeMgr* PresenterFlex :: CreateFlexTimeMgr ()
{
	return new FlexTimeMgr();
}

void PresenterFlex :: DeleteFlexTimeMgr (FlexTimeMgr *pFlexTimeMgr)
{
	delete pFlexTimeMgr;
}
