//=============================================================================
//  This software module was originally developed by IBM Research (T.J. Watson)
//  in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1) 
//  standard. This software module is an implementation of a part of one or 
//  more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 
//  Systems (ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 
//  Systems (ISO/IEC 14496-1) free license to this software module or 
//  modifications thereof for use in hardware or software products claiming 
//  conformance to the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use
//  this software module in hardware or software products are advised that its
//  use may infringe existing patents. The original developer of this software 
//  module and his/her company, the subsequent editors and their companies, and
//  ISO/IEC have no liability for use of this software module or modifications 
//  thereof in an implementation. Copyright is not released for non MPEG-4 
//  Systems (ISO/IEC 14496-1) conforming products. IBM retain full right to use
//  the code for their own purpose, assign or donate the code to a third party 
//  and to inhibit third parties from using the code for non MPEG-4 Systems 
//  (ISO/IEC 14496-1) conforming products. This copyright notice must be 
//  included in all copies or derivative works. Copyright  1999-2000.
//=============================================================================
//  IBM  TJ Watson Research, USA
//-----------------------------------------------------------------------------
//  TCNSIMPLE.H
//
//  Contact   :
//    Dr. Michelle Kim ( mykim@us.ibm.com )
//
//  Overview  :
//    Declaration of the Constraint Network
//
//  Comment   :
//    Port to C++ of Michelle Kim's TCNSimple
//
//=============================================================================  


#ifndef TCNSIMPLE_H__INCLUDED
#define TCNSIMPLE_H__INCLUDED

class Spring;

//  The maximum number of vertices in the Constraint Network
#define VERTEX_MAX_NUMBER 500

//  Time boundary
#define MAX_BOUNDARY      1e9

class TCNSimple 
{ 
private:
  //  This array is initialised each time we have to do calculation
  Spring **m_pSpringArray;
  int m_nSpringCount;

	int VertexIdMap[VERTEX_MAX_NUMBER];
	int m_nVertexCount;

	double m_dConstraintNetwork[VERTEX_MAX_NUMBER][VERTEX_MAX_NUMBER];

  //  The TCNSimple fills the Arcs (variable length arcs = Springs)
  //  with their feasibleMin and feasibleMax each time we evaluate.
  //  The calculation is based on those feasibleMin and feasibleMax.
  //  Once an Arc has finished playback. Its feasibleMin and feasibleMax
  //  should be set to their actual playback duration (wallclock).
public:
	TCNSimple() ;
	~TCNSimple();
  
  //  Reset with a new set of Arcs (maybe should be in the constructor)
	bool Reset(Spring** pSpringArray, int nArrayLength) ;

	//  Reload the whole durations in case some have changed
  bool Evaluate(); 

  // update the spring with the values calculated in the matrix
	void UpdateFeasibleLengths();

  //  return the index of the first connection
	int FindFirstConnectionId();

  void DumpMatrix();

private:
	void ResetConstraintNetwork();
	bool AddConstraintToNetwork(int iVertexIdA,int iVertexIdB,double dBoundary);
	bool MinimizeNetwork();
	void ComputeShortestPath();
	bool NegativeCycle();
	bool FindLongestPair(int &nStartVertexId,int &nEndVertexId);
	int  FindNetworkIndex(int nVertexId);
};

#endif // TCNSIMPLE_H__INCLUDED