//=============================================================================
//  This software module was originally developed by IBM Research (T.J. Watson)
//  in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1) 
//  standard. This software module is an implementation of a part of one or 
//  more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 
//  Systems (ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 
//  Systems (ISO/IEC 14496-1) free license to this software module or 
//  modifications thereof for use in hardware or software products claiming 
//  conformance to the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use
//  this software module in hardware or software products are advised that its
//  use may infringe existing patents. The original developer of this software 
//  module and his/her company, the subsequent editors and their companies, and
//  ISO/IEC have no liability for use of this software module or modifications 
//  thereof in an implementation. Copyright is not released for non MPEG-4 
//  Systems (ISO/IEC 14496-1) conforming products. IBM retain full right to use
//  the code for their own purpose, assign or donate the code to a third party 
//  and to inhibit third parties from using the code for non MPEG-4 Systems 
//  (ISO/IEC 14496-1) conforming products. This copyright notice must be 
//  included in all copies or derivative works. Copyright  1999-2000.
//=============================================================================
//  IBM  TJ Watson Research, USA
//-----------------------------------------------------------------------------
//  TEMPORALGROUPIMPFLEX.CPP
//
//  Contact   :
//    Dr. Michelle Kim ( mykim@us.ibm.com )
//
//  Overview  :
//    The implementation of the TemporalGroupImpFlex which implements the functions
//    of the TemporalGroup BIFS node.
//
//=============================================================================  


#include "defs.h"
#include "trace.h"
#include "ODDec.h"
//#include "InlineImpFlex.h"		// ZL: I'm doing it differently
#include "Inline.h"

#include <string.h>

//  FlexTime stuff
#include "FlexTimeMgr.h"
//#include "Spring.h"
#include "spring.h" // SOLARIS PORTING - typo
#include "FlexTimeBase.h"
#include "TemporalGroupImpFlex.h"
#include "FlxTime.h"
#include "FlexTimeObject.h"


//=============================================================================
//  Constructor & Destructor
//=============================================================================
TemporalGroupImpFlex::TemporalGroupImpFlex(): TemporalGroup() 
{
}

TemporalGroupImpFlex::~TemporalGroupImpFlex()
{
}


//=============================================================================
//  Parse: overloads the MediaObject one to add the Registration to FTMgr
//=============================================================================
BOOL TemporalGroupImpFlex::Parse(BIFSDecoder *pCoder)
{
  BOOL bResult = MediaObject::Parse(pCoder);
  if(bResult)
    RegisterToFlexTimeMgr();
  return bResult;
}


//=============================================================================
//  As a TemporalNode, the TemporalGroup is able to generate a FTObject
//  which will represent the TG in the TemporalGraph of the FlexTimeMgr
//=============================================================================
FlexTimeObject * TemporalGroupImpFlex::CreateFTObject()
{
  TemporalNode::CreateFTObject();
  
  if(m_pFTObject)
  {
    //  FlexTimeObjects representing TemporalGroup don't wait for any streams:
    m_pFTObject->putStartAu();
    m_pFTObject->putEndAu();
  }
  return m_pFTObject;
}



//=============================================================================
//  Called by a child when it gets activated by the FlexTimeMgr.
//  generates the activeChild eventOut
//=============================================================================
bool TemporalGroupImpFlex::ChildActivated(TemporalNode *pChild)
{
  if( (Bool)meet )
  {
    //  Find which child this is by looping in the children list
    for(int i=0;i<GetChildrenCount();i++)
    {
      if( pChild == GetTemporalNode(i))
      {
        //  Generates the eventOut thanks to the overload of the equal operator
        activeChild=i;
        return true;
      }
    }
  }
  return false;
}


//=============================================================================
//  TemporalGroupImpFlex::RegisterToFlexTimeMgr()
//  Add the current Object to the flextime manager.
//=============================================================================
bool TemporalGroupImpFlex::RegisterToFlexTimeMgr()
{
  InlineImp *pInline = ( GetRoot()->GetInline() );		// ZL: changed to the origianl InlineImp
  if(pInline == NULL)
    return false;
  
  FlexTimeMgr *pFTMgr = pInline->GetFlexTimeMgr();
  if(pFTMgr == NULL)
    return false;
  
  pFTMgr->AddTemporalGroup(this);
  return true; 
}

//=============================================================================
// Sets the isActive flag of the TemporalGroup object
//=============================================================================
void TemporalGroupImpFlex::SetActive(bool bActive)
{
  if( !bActive && !(Bool)coend )
    return;
  
  if( bActive && !(Bool)costart )
    return;
  
  isActive = bActive;
}



//=============================================================================
//  returns true if the classname matches
//=============================================================================
bool TemporalGroupImpFlex::IsClass(const char *pszClassName) const
{
  return ::stricmp("TemporalGroupImpFlex",pszClassName)==0;
}


//=============================================================================
//
//=============================================================================
TemporalNode *TemporalGroupImpFlex::GetTemporalNode(int i)
{
  if(i<0 || i>=children.GetLength())
    return NULL;
  else
    return dynamic_cast<TemporalNode*>( children[i] );
}