//=============================================================================
//  This software module was originally developed by IBM Research (T.J. Watson)
//  in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1) 
//  standard. This software module is an implementation of a part of one or 
//  more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 
//  Systems (ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 
//  Systems (ISO/IEC 14496-1) free license to this software module or 
//  modifications thereof for use in hardware or software products claiming 
//  conformance to the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use
//  this software module in hardware or software products are advised that its
//  use may infringe existing patents. The original developer of this software 
//  module and his/her company, the subsequent editors and their companies, and
//  ISO/IEC have no liability for use of this software module or modifications 
//  thereof in an implementation. Copyright is not released for non MPEG-4 
//  Systems (ISO/IEC 14496-1) conforming products. IBM retain full right to use
//  the code for their own purpose, assign or donate the code to a third party 
//  and to inhibit third parties from using the code for non MPEG-4 Systems 
//  (ISO/IEC 14496-1) conforming products. This copyright notice must be 
//  included in all copies or derivative works. Copyright  1999-2000.
//=============================================================================
//  IBM  TJ Watson Research, USA
//-----------------------------------------------------------------------------
//  TEMPORALGROUPIMP.H
//
//  Contact   :
//    Dr. Michelle Kim ( mykim@us.ibm.com )
//
//  Overview  :
//    The definition of the TemporalGroupImp which implements the functions of 
//    the TemporalGroup BIFS node.
//
//=============================================================================  


#ifndef TEMPORALGROUP_H__INCLUDED
#define TEMPORALGROUP_H__INCLUDED

// SOLARIS PORTING - changed "\" to "/"
//#include "..\core\BifsDec.h"
#include "../core/BifsDec.h"

//  FlexTime stuff
#include "Nodes.h"
#include "TemporalNode.h"

//=============================================================================
//  TEMPORALGROUPIMP
//=============================================================================
//
//  Implements the TemporalGroup object
//
//=============================================================================
class TemporalGroupImpFlex: public TemporalGroup, public TemporalNode
{
private:
  //  Add the current Object to the flextime manager.
  bool RegisterToFlexTimeMgr();
  
public:
  //  constructor & destructor
  TemporalGroupImpFlex();
  ~TemporalGroupImpFlex();
  
  // Called when data have to be parsed...
  virtual BOOL Parse(BIFSDecoder *pCoder);
  // Creates a FlexTimeObject (use by the TCN to check graph consistency)
  virtual FlexTimeObject *CreateFTObject();
  
  //  Called by a child when it gets activated (+activeChild eventOut)
  bool ChildActivated(TemporalNode * pChild);
  
  //  A way to simplify the way the boolean active is set... 
  //  (isActive for TG & Active for TT)
  virtual void SetActive(bool bActive=true);
  
  //  This an accesor to the number of items present in the priority field
  int GetPriorityCount() const { return priority.GetLength(); }
  
  
  //  returns the temporalNode (children)
  TemporalNode *GetTemporalNode(int i);
  
  //  TemporalBase Overload
  bool IsClass(const char *pszClassName) const;
  
  virtual TemporalGroupImpFlex* GetTG() { return this; };
};


#endif // TEMPORALGROUP_H__INCLUDED