//=============================================================================
//  This software module was originally developed by IBM Research (T.J. Watson)
//  in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1) 
//  standard. This software module is an implementation of a part of one or 
//  more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 
//  Systems (ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 
//  Systems (ISO/IEC 14496-1) free license to this software module or 
//  modifications thereof for use in hardware or software products claiming 
//  conformance to the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use
//  this software module in hardware or software products are advised that its
//  use may infringe existing patents. The original developer of this software 
//  module and his/her company, the subsequent editors and their companies, and
//  ISO/IEC have no liability for use of this software module or modifications 
//  thereof in an implementation. Copyright is not released for non MPEG-4 
//  Systems (ISO/IEC 14496-1) conforming products. IBM retain full right to use
//  the code for their own purpose, assign or donate the code to a third party 
//  and to inhibit third parties from using the code for non MPEG-4 Systems 
//  (ISO/IEC 14496-1) conforming products. This copyright notice must be 
//  included in all copies or derivative works. Copyright  1999-2000.
//=============================================================================
//  IBM  TJ Watson Research, USA
//-----------------------------------------------------------------------------
//  TEMPORALNODE.CPP
//
//  Contact   :
//    Dr. Michelle Kim ( mykim@us.ibm.com )
//
//  Overview  :
//    Basic Implementation of a TemporalNode (Base class fo TemporalGroup and 
//    TemporalTransform, which can instantiate a FlexTimeObject to represent 
//    them in the temporal graph)
//
//=============================================================================  


#include "defs.h"
#include "FlexTimeMgr.h"
#include "FlexTimeBase.h"
#include "TemporalNode.h"
#include "Spring.h"
#include "FlexTimeObject.h" 
#include "TemporalGroupImpFlex.h"


//-----------------------------------------------------------------------------
//  Constructor / Destructor
//-----------------------------------------------------------------------------
TemporalNode::TemporalNode()
{
  m_pFTObject=NULL;
}

TemporalNode::~TemporalNode()
{
  if(m_pFTObject)
   delete m_pFTObject;
}


//-----------------------------------------------------------------------------
// This method will create a new FT object with default values
//-----------------------------------------------------------------------------
FlexTimeObject *TemporalNode::CreateFTObject()
{
#ifndef BIFSENC
  assert(!m_pFTObject);
  //  Create a new FlexTimeObject
  //  optimal Duration: sum of its children optDurations ?
  //  low priority default for TemporalGroups
  m_pFTObject = new FlexTimeObject( 0,
                                    MAX_DURATION,
                                    MAX_DURATION,
                                    -1,
                                    this );
  
  if(m_pFTObject)
    //  If the object is successfully created, it must be registered as soon as
    //  possible
    m_pFTObject->SetFlexTimeMgr(GetFlexTimeMgr());

  return m_pFTObject;

#else //  BIFSENC
  return NULL;
#endif // BIFSENC
}


//-----------------------------------------------------------------------------
//  Starts a Node with the speed fSpeed. This object notifies its parents
//  that it has been started
//-----------------------------------------------------------------------------
void TemporalNode::Start(float/*fSpeed*/)
{
  SetActive(true);

  //  signal that this child has been activated to all its TemporalGroup 
  //  parents
  int iCount = m_zaTGParents.GetLength();
  for(int i=0;i<iCount;i++)
    m_zaTGParents[i]->ChildActivated(this);
}


//-----------------------------------------------------------------------------
//  Stops a Node
//-----------------------------------------------------------------------------
void TemporalNode::End()
{
  SetActive(false);
}

//-----------------------------------------------------------------------------
//  Add a temporal parent
//-----------------------------------------------------------------------------
void TemporalNode::AddTemporalParent(TemporalGroupImpFlex *ptgParent)
{
  m_zaTGParents += ptgParent;
}


//-----------------------------------------------------------------------------
//  Remove a temporal parent
//-----------------------------------------------------------------------------
void TemporalNode::RemoveTemporalParent(TemporalGroupImpFlex *ptgParent)
{
  m_zaTGParents -= ptgParent;
}

/*
//-----------------------------------------------------------------------------
//  returns a temporalnode
//-----------------------------------------------------------------------------
TemporalNode *TemporalNode::GetTemporalNode(int i)
{
}
*/