//=============================================================================
//  This software module was originally developed by IBM Research (T.J. Watson)
//  in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1) 
//  standard. This software module is an implementation of a part of one or 
//  more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 
//  Systems (ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 
//  Systems (ISO/IEC 14496-1) free license to this software module or 
//  modifications thereof for use in hardware or software products claiming 
//  conformance to the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use
//  this software module in hardware or software products are advised that its
//  use may infringe existing patents. The original developer of this software 
//  module and his/her company, the subsequent editors and their companies, and
//  ISO/IEC have no liability for use of this software module or modifications 
//  thereof in an implementation. Copyright is not released for non MPEG-4 
//  Systems (ISO/IEC 14496-1) conforming products. IBM retain full right to use
//  the code for their own purpose, assign or donate the code to a third party 
//  and to inhibit third parties from using the code for non MPEG-4 Systems 
//  (ISO/IEC 14496-1) conforming products. This copyright notice must be 
//  included in all copies or derivative works. Copyright  1999-2000.
//=============================================================================
//  IBM  TJ Watson Research, USA
//-----------------------------------------------------------------------------
//  TEMPORALNODE.H
//
//  Contact   :
//    Dr. Michelle Kim ( mykim@us.ibm.com )
//
//  Overview  :
//    Declaration of a TemporalNode (Base class fo TemporalGroup and 
//    TemporalTransform, which can instantiate a FlexTimeObject to represent 
//    them in the temporal graph)
//
//=============================================================================  


#ifndef TEMPORALNODE_H__INCLUDED
#define TEMPORALNODE_H__INCLUDED

#include "ztl.h"
#include "FlexTimeBase.h"

class FlexTimeObject;
class TemporalGroupImpFlex;

//=============================================================================
//  Mother class of TemporalTransform and TemporalGroup
//=============================================================================
class TemporalNode : public FlexTimeBase
{
protected:
  //  The parents of the current object.
  ZArray<TemporalGroupImpFlex*> m_zaTGParents;

  //  Link this object with a FlexTimeobject that is used to compute the TCN
  FlexTimeObject *m_pFTObject;

  float  m_fTempPriority;

public:
  TemporalNode();
  virtual ~TemporalNode();

  FlexTimeObject *GetFTObject() const { return m_pFTObject; }
  virtual FlexTimeObject *CreateFTObject();

  // These 2 methods are called by FlexTime Mgr to start and stop this object
  virtual void Start(float fSpeed=1.0);
  virtual void End();

  void AddTemporalParent(TemporalGroupImpFlex   *ptgParent);
  void RemoveTemporalParent(TemporalGroupImpFlex *ptgParent);
  
  //  These methods are overloaded in TT and TG
  virtual void SetActive(bool)      {};
  virtual bool IsStreamActive() { return true; }

  // used to cache the priority of as member variable.
  void  SetTempPriority(float fPriority)  { m_fTempPriority=fPriority; }
  float GetTempPriority() const           { return m_fTempPriority; }

};

#endif // TEMPORALNODE_H__INCLUDED