//=============================================================================
//  This software module was originally developed by IBM Research (T.J. Watson)
//  in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1) 
//  standard. This software module is an implementation of a part of one or 
//  more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 
//  Systems (ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 
//  Systems (ISO/IEC 14496-1) free license to this software module or 
//  modifications thereof for use in hardware or software products claiming 
//  conformance to the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use
//  this software module in hardware or software products are advised that its
//  use may infringe existing patents. The original developer of this software 
//  module and his/her company, the subsequent editors and their companies, and
//  ISO/IEC have no liability for use of this software module or modifications 
//  thereof in an implementation. Copyright is not released for non MPEG-4 
//  Systems (ISO/IEC 14496-1) conforming products. IBM retain full right to use
//  the code for their own purpose, assign or donate the code to a third party 
//  and to inhibit third parties from using the code for non MPEG-4 Systems 
//  (ISO/IEC 14496-1) conforming products. This copyright notice must be 
//  included in all copies or derivative works. Copyright  1999-2000.
//=============================================================================
//  IBM  TJ Watson Research, USA
//-----------------------------------------------------------------------------
//  TEMPORALTRANSFORMIMPFLEX.CPP
//
//  Contact   :
//    Dr. Michelle Kim ( mykim@us.ibm.com )
//
//  Overview  :
//    The implementation of the TemporalTransformImpFlex which implements the functions
//    of the TemporalGroup BIFS node.
//
//=============================================================================  

#include "defs.h"
//PAB:? precompiled headers stuff...
#include "FlexTimeMgr.h" 
#include "Spring.h" 
#include "FlexTimeObject.h" 
#include "TemporalTransformImpFlex.h"


//-----------------------------------------------------------------------------
//  constructor
//-----------------------------------------------------------------------------
TemporalTransformImpFlex::TemporalTransformImpFlex()
{
}

//-----------------------------------------------------------------------------
//  Start : starts the TemporalTransform Object with the correct speed ratio
//-----------------------------------------------------------------------------
void TemporalTransformImpFlex::Start(float fSpeed)
{
  TemporalNode::Start(fSpeed);
  
  //if default playback mode is used (no linear change of the speed, do not 
  //change the speed.		
  if( (shrinkMode[0]==0 && fSpeed<1.0) || (stretchMode[0]==0 && fSpeed>1.0) )
		fSpeed=1.0;

  //else the TemporalTransform speed is changed to the new computed speed
  speed = fSpeed;
}

//-----------------------------------------------------------------------------
//  End : ends the TemporalTransform Object 
//-----------------------------------------------------------------------------
void TemporalTransformImpFlex::End()
{
  TemporalNode::End();
  //this will also set the active field to false.
}

//-----------------------------------------------------------------------------
//PAB:  CreateFTObject()
//-----------------------------------------------------------------------------
FlexTimeObject * TemporalTransformImpFlex::CreateFTObject() 
{
  TemporalNode::CreateFTObject();
  
  if(!m_pFTObject)
    return NULL;
  
  int nODId = -1;
  //if there is no url, then consider that the FlexTimeObject is ready to start
  for(int i=0; i<url.GetLength(); i++)
  {
    if(url[i]>=0)
    {
      nODId = url[i];
      break;
    }
  }

  if(nODId < 0)
    m_pFTObject->putStartAu();
  
  double dOptDur  = optimalDuration;
  double dScalMin = ((Vec2f)scalability).x;
  double dScalMax = ((Vec2f)scalability).y;
  
  //  If the optimal duration is negative if scalability factors are outside
  //  their bounds, the optimal duration is not available.
  if( dOptDur<=0.0 || dScalMin>1.0 || dScalMax<1.0 )
    // it's not available ! -> reset it
    dOptDur = 0.0;
  else  //if(dOptDur>0.0)
  {
    dScalMin *= dOptDur;
    dScalMax *= dOptDur;
  }
  
  m_pFTObject->SetOptDuration(dOptDur);
  m_pFTObject->SetFeasibleMin(dScalMin);
  m_pFTObject->SetFeasibleMax(dScalMax);

  //  //PAB: default: an FTObject actually have several priorities  depending
  //  in which relation it is !!!
  //  the highest value will here be used.
  m_pFTObject->SetPriority(GetTempPriority()); 
  
  return m_pFTObject;	
}

//=============================================================================
//  returns true if the Url is Active (TemporalTansformImp)
//=============================================================================
bool TemporalTransformImpFlex::IsStreamActive() 
{
	return IsUrlActive();
}

//=============================================================================
// Sets the active flag of the Transform object
//=============================================================================
void TemporalTransformImpFlex::SetActive(bool bActive)
{
  active = bActive;
}



//=============================================================================
//  returns true if the classname matches TemporalTransformImpFlex
//=============================================================================
  //  TemporalBase Overload
bool TemporalTransformImpFlex::IsClass(const char *pszClassName) const
{
  return ::stricmp("TemporalTransformImpFlex",pszClassName)==0;
}

