/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Optibase Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Optibase Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1997.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Presenter
FILE:       SimPres.cpp
AUTHOR:     Zvi Lifshitz (Optibase Ltd.)
OVERVIEW:   Implementation of Presenter (simulation).
-------------------------------------------------------------------------*/
#include "defs.h"
#include "Presenter.h"
#include "Caps.h"
#include "MediaObj.h"
#include "Executiv.h"
#include "Clock.h"
#include "ZNodes.h"
#include "ZFlxTime.h"

#define MIKE

#ifdef MIKE
#include "MikeBS/MNodes.h"
#endif

Presenter *g_pPresenter;

/*-----------------------------------------------------------------
Presenter constructor. Saves the arguments and creates the Presenter
thread.
-----------------------------------------------------------------*/
Presenter :: Presenter (Executive *pExecutive, HWND hWnd) :
	PresenterFlex (pExecutive, hWnd)
{
	//MBS 110498: removed nodes that no more exist
	OVERLOAD_NODE (ZGroup);
	OVERLOAD_NODE (ZTransform);
	OVERLOAD_NODE (ZTransform2D);
	OVERLOAD_NODE (ZShape);
	OVERLOAD_NODE (ZCircle);
	OVERLOAD_NODE (ZText);
	OVERLOAD_NODE (ZMovieTexture);
	OVERLOAD_NODE (ZAppearance);
	OVERLOAD_NODE (ZMaterial2D);
	OVERLOAD_NODE (ZMaterial);
	OVERLOAD_NODE (ZAudioSource);
	OVERLOAD_NODE (ZAudioClip);
	OVERLOAD_NODE (ZSound2D);
	OVERLOAD_NODE (ZImageTexture);
	OVERLOAD_NODE (ZSound);
	OVERLOAD_NODE (ZFontStyle);
	OVERLOAD_NODE (ZTimeSensor);
	OVERLOAD_NODE (ZTemporalTransform);

#ifdef AUDIOBIFSV3
	OVERLOAD_NODE (ZAdvancedAudioBuffer);
	OVERLOAD_NODE (ZAudioChannelConfig);
	OVERLOAD_NODE (ZSurroundingSound);
	OVERLOAD_NODE (ZTransform3DAudio);
	OVERLOAD_NODE (ZWideSound);
#endif

#ifdef MIKE
	MikeOverloads ();
#endif
		
	fprintf(stderr,"Im1Player with FlexTime support 5.1 (release candidate 2) - Copyright (c) Optibase Ltd.\n");
	m_dwIterations = 1;
	m_fLogFile = fopen ("log.txt", "w");
	g_pPresenter = this;
	m_bClockActive = FALSE;
	m_dwOffset = timeGetTime ();
}

/*-----------------------------------------------------------------
Presenter destructor.
-----------------------------------------------------------------*/
Presenter :: ~Presenter ()
{
	fclose (m_fLogFile);
}

/*-----------------------------------------------------------------
The capabilities of the Presenter combines the capabilities of the 
visual and audio renderers.
-----------------------------------------------------------------*/
void Presenter :: GetCapabilities (Capabilities *pCaps)
{
	pCaps -> AddFormatCapability (MediaFormat (MediaFormat :: RGB), 0);
	pCaps -> AddFormatCapability (MediaFormat (MediaFormat :: PCM), 1);
}

/*-----------------------------------------------------------------
Init() kicks of the presenter thread
-----------------------------------------------------------------*/	
void Presenter :: Start ()
{
	PresenterBase :: Start ();
	m_dwStartTime = timeGetTime ();
}

/*-----------------------------------------------------------------
Render() is called in a loop from the Presenter thread
-----------------------------------------------------------------*/
void Presenter :: Render ()
{
	/*-----------------------------------------------------------------
	Render the scene every 30 ms
	-----------------------------------------------------------------*/
	Sleep (30);
	PresenterDrivenEvents ();
	GetExecutive () -> LockScene ();
	if (GetScene () != NULL)
	{	ZPresenterContext context (m_dwIterations);
		context.SetTime (GetScene () ->GetSceneTime ());
		GetScene () -> Render (&context);
	}
	++m_dwIterations;
	if (m_dwIterations == 0)		// some nodes require that m_dwIterations
		++m_dwIterations;			// would never be zero.
	GetExecutive () -> UnlockScene ();
}

/*-----------------------------------------------------------------
Log the reception of a frame to the log file
-----------------------------------------------------------------*/
void Presenter :: LogStream (
	DWORD dwSceneTime,
	LPCSTR szDesc,
	int nodeID,
	int nID,
	int nUnitSize,
	DWORD dwTime
)
{
	Logf (
		dwSceneTime,
		"%s, ID: %d, ODid: %d, CU size: %d, Time stamp: %lu\n",
		szDesc,
		nodeID,
		nID,
		nUnitSize,
		dwTime
	);
}

/*-----------------------------------------------------------------
Format a string into the log file
-----------------------------------------------------------------*/
void Presenter :: Logf (DWORD dwSceneTime, LPCSTR szFormat, ...)
{
	fprintf (
		m_fLogFile,
		"Presenter time: %6lu, Scene time: %6lu ",
		GetClockTime (),
		dwSceneTime
	);
	va_list argptr;
	va_start (argptr, szFormat);
	vfprintf (m_fLogFile, szFormat, argptr);
	va_end (argptr);
}

/*-----------------------------------------------------------------
User events
-----------------------------------------------------------------*/
void Presenter :: OnUserEvent  (USER_EVENT, int, int, UINT) {}

/*-----------------------------------------------------------------
My implementation of GetTime
-----------------------------------------------------------------*/
DWORD Presenter :: GetTime ()
{
	return m_dwIterations;
}

/*-----------------------------------------------------------------
Save system time as an offset, and set the clock on.
-----------------------------------------------------------------*/
void Presenter :: SetClock (BOOL bOn)
{
	if (bOn)
		m_dwOffset = timeGetTime ();
	m_bClockActive = bOn;
}

/*-----------------------------------------------------------------
Return the sysmem time minus the clocko offset of the Presenter's clock
-----------------------------------------------------------------*/
DWORD Presenter :: GetClockTime ()
{
	return timeGetTime () - m_dwOffset;
}
