/******************************************************************************
 * This software module was originally developed by Agfa Monotype Corporation
 * in the course of development of the ISO/IEC 14996-18 standard for reference 
 * purposes and its performance may not have been optimized. This software 
 * module is an implementation of font compression technology as specified 
 * by the ISO/IEC 14996-18 standard.
 * 
 * ISO/IEC gives users a free copyright license to this software module or
 * modifications thereof for use in products claiming conformance to 
 * ISO/IEC 14996-18 International Standard. ISO/IEC gives users the same free
 * license to this software module or modifications thereof for research 
 * purposes and further ISO/IEC standardization.
 * 
 * Those intending to use this software module in products are advised that
 * its use may infringe existing patents and that this license does not grant 
 * rights under any patents. ISO/IEC have no liability for use of this 
 * software module or modifications thereof.
 * 
 * No license is granted and copyright is not released for products that do 
 * not conform to ISO/IEC 14996-18 International Standard.
 * 
 * Agfa Monotype Corporation retains full right to modify and use the code for
 * its own purpose, assign or donate the code to a third party and to inhibit 
 * third parties from using the code for products that do not conform to
 * audiovisual and image-coding related ITU recommendations and 
 * ISO/IEC International Standards. All rights not expressly granted to the 
 * software module or modifications thereof are reserved by 
 * Agfa Monotype Corporation.
 * 
 * This copyright notice must be included in all copies or derivative works.
 * Copyright (c) ISO/IEC 2003. All rights reserved.
 ******************************************************************************/

/*
 * File:						bitio.h
 * Revision history
 * $Log: BITIO.H,v $
 * Revision 1.2  2004/03/01 21:48:03  vladlev
 * minor optimization for performance
 *
 * Revision 1.1  2004/02/10 22:01:08  vladlev
 * Initial version
 *
 */
#include "MEMORY.H"

#ifdef __cplusplus
extern "C" {
#endif

typedef struct {
	BYTE *array;	/* the array of BYTES */
	LONG index;		/* next available bit index */
	LONG max;		/* maximum bit index */
	LONG high;		/* high water mark - for written arrays */
	MTX_MemHandler *mem;
	} BITIO;

/* read a value from the bitarray */
void MTX_BITIO_WriteValue( BITIO *t, ULONG value, LONG numberOfBits );
/* byte aligned special cases */
void MTX_BITIO_Write32( BITIO *t, ULONG value);
void MTX_BITIO_Write8( BITIO *t, ULONG value );
void MTX_BITIO_output_bit(BITIO *t,int bit);

/* write a value to the bitarray */
ULONG MTX_BITIO_ReadValue( BITIO *t, LONG numberOfBits );
/* byte aligned special cases */
ULONG MTX_BITIO_Read32( BITIO *t);
ULONG MTX_BITIO_Read8( BITIO *t);

void MTX_BITIO_flush_bits(BITIO *t);
BYTE *MTX_BITIO_GetMemoryPointer( BITIO *t );
LONG MTX_BITIO_GetBytesOut( BITIO *t );

#if 0
ULONG MTX_BITIO_ftell( register BITIO *t );
void MTX_BITIO_fseek( register BITIO *t, ULONG pos );
#else
#define MTX_BITIO_ftell(t) (t)->index
#define MTX_BITIO_fseek(t,pos) (t)->index = (pos)

#endif
BITIO *MTX_BITIO_Create( MTX_MemHandler *mem, void *memPtr, LONG memSize, const CHAR param );
void MTX_BITIO_Destroy(BITIO *t);




#ifdef __cplusplus
}
#endif
