/******************************************************************************
 * This software module was originally developed by Agfa Monotype Corporation
 * in the course of development of the ISO/IEC 14996-18 standard for reference 
 * purposes and its performance may not have been optimized. This software 
 * module is an implementation of font compression technology as specified 
 * by the ISO/IEC 14996-18 standard.
 * 
 * ISO/IEC gives users a free copyright license to this software module or
 * modifications thereof for use in products claiming conformance to 
 * ISO/IEC 14996-18 International Standard. ISO/IEC gives users the same free
 * license to this software module or modifications thereof for research 
 * purposes and further ISO/IEC standardization.
 * 
 * Those intending to use this software module in products are advised that
 * its use may infringe existing patents and that this license does not grant 
 * rights under any patents. ISO/IEC have no liability for use of this 
 * software module or modifications thereof.
 * 
 * No license is granted and copyright is not released for products that do 
 * not conform to ISO/IEC 14996-18 International Standard.
 * 
 * Agfa Monotype Corporation retains full right to modify and use the code for
 * its own purpose, assign or donate the code to a third party and to inhibit 
 * third parties from using the code for products that do not conform to
 * audiovisual and image-coding related ITU recommendations and 
 * ISO/IEC International Standards. All rights not expressly granted to the 
 * software module or modifications thereof are reserved by 
 * Agfa Monotype Corporation.
 * 
 * This copyright notice must be included in all copies or derivative works.
 * Copyright (c) ISO/IEC 2003. All rights reserved.
 ******************************************************************************/

/*
 * File:						main.c
 * Revision history
 * $Log: Main.c,v $
 * Revision 1.2  2004/02/19 19:36:42  vladlev
 * Added new functionality to write uncompressed OpenType TrueType/CFF
 * files in output MP4 file.
 *
 * Revision 1.1  2004/02/10 22:01:09  vladlev
 * Initial version
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <assert.h>
#include <string.h>

#include "ERRCODES.H"
#include "CONFIG.h"
#include "sfnt.h"
#include "MEMORY.H"
#include "BITIO.H"
#include "STRCTR.H"
#include "RADECOMP.H"
#include "RA_COMP.H"
#include "tablecmp.h"
#include "TTF_CONV.H"
#include "RA_TTF.H"
#include "RA_TEST.H"
#include "RA_UTIL.H"

#define WRITE_MP4		0
#define WRITE_MTX_MP4	1
#define TEST_MTX		2

FILE *actlog;
int inTTC;
CHAR *outExt;

/* for MP4 file output */
CHAR fontName[128];
void setFontDataPtr (BYTE *mem, LONG size, BYTE format);
int createMP4file( char *filename );


/* Simple function for mapping eg. TIMES.TTF to TIMES.CTF */
static void ReplaceExtension( CHAR *inName, CHAR *outName, CHAR *extension )
{
	int i, len = strlen( inName );
	
	for ( i = 0; i < len; i++ ) {
		CHAR c = inName[i];
		if ( c == '.' ) break; /*****/
		outName[i]		= c;
	}
	outName[i++] 	= '.';
	strcpy( &outName[i], extension );
}

/*----------------------------- TTFTEST -----------------------------------------*/
static int TTFTEST(char* nameIn)
{
	CHAR nameOut[256];
	
    ReplaceExtension( nameIn, nameOut, outExt );
	printf("Starting test of \"%s\"\n", nameIn);
	return MTX_RA_TTF_Test( nameIn, nameOut );
}
/*----------------------------- TTFtoCTF -----------------------------------------*/
static int TTFtoCTF(CHAR* nameIn, CHAR*nameOut)
{
	ReplaceExtension( nameIn, nameOut, "CTF" );
	printf("Starting TTF->CTF-translation of \"%s\" to \"%s\"\n", nameIn, nameOut);
	return MTX_TT_Converter( nameIn, nameOut, true );
}


extern CHAR *GetTablePtr( CHAR *sfnt, int fontOff, sfnt_TableTag tag );

/* Get <fontName> and store it for future use in MP4 file */
static void getFontName ( CHAR *sfnt, int fontOff )
{
	CHAR *name = GetTablePtr ( sfnt, fontOff, tag_NamingTable );
	USHORT nameRecordCount = SWAPENDS_U16(GET_name_recordCount ( name ));
	USHORT stringOffset = SWAPENDS_U16(GET_name_stringOffset( name ));
	TTF_Name_Record *nameRecord;
	CHAR *nameString;
	USHORT platformID;
	USHORT encodingID;
	USHORT languageID;
	USHORT nameID;
	USHORT stringLength;
	int i, j, k;

//	printf ("Count = %x, String Offset = %x\n", nameRecordCount, stringOffset);

	for (i=0; i<nameRecordCount; i++) {
		nameRecord = GET_nameRecord_entryPtr ( name, i );
		platformID = SWAPENDS_U16(nameRecord->platformID);
		encodingID = SWAPENDS_U16(nameRecord->encodingID);
		languageID = SWAPENDS_U16(nameRecord->languageID);
		if ((platformID == MSFT_PLATFORM) && 
			(encodingID == UNICODE1_1) && 
			((languageID & 0x00FF) == 0x0009 /* English */) ) {
			/* I am using Microsoft platform ID with Unicode encoding
			 * and any variant of English language, this combination
			 * is likely to exist in any font
			 */
			if ((nameID = SWAPENDS_U16(nameRecord->nameID)) == 4) {  /* unique font name */
				nameString = name + stringOffset + SWAPENDS_U16(nameRecord->offset);
				stringLength = SWAPENDS_U16(nameRecord->length);
				for (j=k=0; j<stringLength; j++) {
					if (*(nameString)) {
						fontName[k++] = *nameString;
					}
					nameString++;
				}
				fontName[k] = 0;
				printf ("PlatformID=%d, EncodingID=%d, LanguageID=0x%x\n",
					platformID, encodingID, languageID);
				printf ("NameID=%d, Name=<%s>\n", nameID, fontName);
			}
		}
	}
}


static SHORT getFontType(CHAR *nameIn)
{
    MTX_MemHandler *mem = MTX_mem_Create( malloc, realloc, free);
    if (mem)
    {
        LONG sizeIn;
        CHAR *sfnt;
        sfnt_TableTag tag; 
        int fontOff;
    	CHAR *head;
        USHORT fontType;
        
        sfnt = (CHAR *)RA_ReadFileIntoMemory( mem, nameIn, &sizeIn );
        if(!sfnt)
        {
            printf("RA_ReadFileIntoMemory() failed\n");
            fprintf(actlog, "RA_ReadFileIntoMemory() failed\n");
	        MTX_mem_CloseMemory( mem );
	        if(mem->errorCode)
	        {
	            printf("******* Error: errorCode %d\n", mem->errorCode);
	            fprintf(actlog, "******* Error: errorCode %d\n", mem->errorCode);
	        }
	        MTX_mem_Destroy( mem );
            return false;
        }

        /* Process TrueType font file pointed at by data to get fontType */
        tag = *((sfnt_TableTag *)((CHAR *)sfnt + TTC_TAG));
        if ( SWAPENDS_32(tag) == tag_TTCollection ) 
        {
            inTTC = true;
            fontOff = SWAPENDS_32( GET_ttc_font_offset( sfnt, 0 ) ); /* offset to first font */
        }
        else
        {
            inTTC = false;
            fontOff = 0;
        }

		getFontName (sfnt, fontOff);

        head = GetTablePtr( sfnt, fontOff, tag_FontHeader );
        fontType = SWAPENDS_U16(GET_head_glyphDataFormat( head ));

		if (fontType) /* shoudl be '0' for TTF fonts */
		{
            printf("******* Error: invalid font format\n");
            fprintf(actlog, "******* Error: invalid font format\n");
	        MTX_mem_free( mem, sfnt );
	        MTX_mem_CloseMemory( mem );
	        if(mem->errorCode)
	        {
	            printf("******* Error: errorCode %d\n", mem->errorCode);
	            fprintf(actlog, "******* Error: errorCode %d\n", mem->errorCode);
	        }
	        MTX_mem_Destroy( mem );
            return false;
        }

        MTX_mem_free( mem, sfnt );
        
        MTX_mem_CloseMemory( mem );
        if(mem->errorCode)
        {
            printf("******* Error: errorCode %d\n", mem->errorCode);
            fprintf(actlog, "******* Error: errorCode %d\n", mem->errorCode);
        }
        MTX_mem_Destroy( mem );
    }
    else
    {
        printf("MTX_mem_Create() failed.\n");
        fprintf(actlog, "MTX_mem_Create() failed.\n");
        return false;
	}

	if(inTTC)
		outExt = "a3b";
	else 
		outExt = "a3a";
    return true;
}


void main(int argc, CHAR *argv[])
{
    int bSuccess = false;
    CHAR nameIn[256], nameOut[256];
    
    FILE *verbose = NULL;
    int mode = -1;
    BYTE DSIfontformat = 0;
    
    printf("Welcome to the MicroType(R) Express font compressor!\n" );
    printf("Application built: %s, %s\n", __TIME__ , __DATE__ );
    
	if (argc == 3)
	{
		if (!strcmp (argv[1], "-t"))
		{
			mode = TEST_MTX;
			strcpy(nameIn,argv[2]);
		}
		else if (!strcmp (argv[1], "-w"))
		{
			mode = WRITE_MP4;
			strcpy(nameIn,argv[2]);
		}
		else
		{
			printf ("\tInvalid option: [%s]\n", argv[1]);
			printf ("\tUsage:\n");
			printf ("\tFontCompress [-t][-w] <path\\font_file_name>\n");
			printf ("\t[-t] - test MTX compressed file\n");
			printf ("\t[-w] - write input (TTF or OTF) font file in MP4 file format\n");
			printf ("\t[none] - compress input TTF file and write in MP4 file format\n");
			exit (0);
		}
	}
	else if (argc == 2)
	{
		mode = WRITE_MTX_MP4;
		strcpy(nameIn,argv[1]);
	}
	else
	{
		printf ("\tUsage:\n");
		printf ("\tFontCompress [-t][-w] <path\\font_file_name>\n");
		printf ("\t[-t] - test MTX compressed file\n");
		printf ("\t[-w] - write input (TTF or OTF) font file in MP4 file format\n");
		printf ("\t[none] - compress input TTF file and write in MP4 file format\n");
		exit (0);
	}

	/* Check input file */
	verbose = fopen(nameIn,"r");
	if (verbose)
	{
		fclose(verbose);
		verbose = NULL;
		/* the output file name is the one that used to write data to MP4 file */
		/* if we don't compress the font, the output name should be the same
		/* as input file name */
		strcpy(nameOut, nameIn);
	}
	else
	{
	    printf("Can't open the file %s\n", nameIn);
	    exit(0);
	}

	printf ("\nInput file: %s\n", nameIn);

	switch (mode) {
	case TEST_MTX:
	    /* Open log file */
	    ReplaceExtension( nameIn, nameOut, "log" );
	    actlog = fopen(nameOut, "w" );
	    if(!actlog)
	    {
	        printf("Can't open log file %s\n", nameOut);
	        exit(0);
	    }
	    fprintf(actlog, "Input file: %s\n", nameIn);
		fprintf(actlog, "\nTTF testing an 'act3' format file\n");
		if (getFontType(nameIn))
		{
			/* Check second input file */
			ReplaceExtension (nameIn, nameOut, outExt);
			verbose = fopen(nameOut,"r");
			if (verbose)
			{
				fclose(verbose);
				verbose = NULL;
			}
			else
			{
		        printf("Can't open the file %s\n", nameOut);
		        exit(0);
		    }
		}

		bSuccess = TTFTEST( nameIn );
        fprintf(actlog, "*** %s",(bSuccess) ? "Success\n" : "Failure\n");
        fclose(actlog);
		break;
	case WRITE_MTX_MP4:
	    /* Open log file */
	    ReplaceExtension( nameIn, nameOut, "log" );
	    actlog = fopen(nameOut, "w" );
	    if(!actlog)
	    {
	        printf("Can't open log file %s\n", nameOut);
	        exit(0);
	    }
	    fprintf(actlog, "Input file: %s\n", nameIn);

		if (getFontType(nameIn))
	    {	/* compress font file */
	        CHAR nameCTF[256];

			DSIfontformat = 2; /* compressed OpenType/TrueType */
			bSuccess = TTFtoCTF (nameIn, nameCTF);
			if(bSuccess)
			{
		        ReplaceExtension( nameIn, nameOut, outExt );
		        printf("\n\nStarting table compression\n");
		        bSuccess = MTX_TABLE_COMPRESS_Compress(nameCTF, nameOut);
		        if (bSuccess)
				{
					bSuccess = TTFTEST( nameIn );
					if(!bSuccess)
					{
						fprintf(actlog, "****** TTFTEST failed *****\n");
						printf("****** TTFTEST failed *****\n");
					}
				}
				else
		        {
		            fprintf(actlog, "****** MTX_TABLE_COMPRESS_Compress() failed *****\n");
		            printf("****** MTX_TABLE_COMPRESS_Compress() failed *****\n");
		        }
			}
			else
			{
				fprintf(actlog, "****** TTF to CTF conversion failed *****\n");
				printf("****** TTF to CTF conversion failed *****\n");
			}
	    }
        fprintf(actlog, "*** %s",(bSuccess) ? "Success\n" : "Failure\n");
        fclose(actlog);

		if (!bSuccess) break;
		/* otherwise continue to write data in MP4 file */
	case WRITE_MP4:
	default:
	    {	/* write MP4 file */
			int err;
		    MTX_MemHandler *fileMem = MTX_mem_Create( malloc, realloc, free);
			BYTE *fontMem = NULL;
			LONG fontSize;

			if (fileMem) {
		        if (fontMem = RA_ReadFileIntoMemory ( fileMem, nameOut, &fontSize ))
				{
					if (mode == WRITE_MP4)
					{
						/* we are here just to write input font file in MP4 format */
						/* so we need to get font name and type first */
						int fontOff = 0;
						BYTE *tablePtr = (BYTE *)GetTablePtr( fontMem, 0, tag_CFF );

						/* is it OpenType CFF ? */
						if (tablePtr == fontMem) {
							/* CFF table is not present, we have TTF data */
					        sfnt_TableTag tag = *((sfnt_TableTag *)((CHAR *)fontMem + TTC_TAG)); 
							if ( SWAPENDS_32(tag) == tag_TTCollection ) 
								/* if TTC, find offset to the first font */
							    fontOff = SWAPENDS_32( GET_ttc_font_offset( fontMem, 0 ) );
							DSIfontformat = 1; /* OpenType/TrueType */
						}
						else
							DSIfontformat = 3; /* OpenType CFF */
						getFontName (fontMem, fontOff);
					}
					setFontDataPtr (fontMem, fontSize, DSIfontformat);
					outExt = "mp4";
			        ReplaceExtension( nameIn, nameOut, outExt );
					if (err = createMP4file ( nameOut ))
						printf("****** createMP4file failed *****\n");
					else
						bSuccess = true;
	
					MTX_mem_free( fileMem, fontMem );
				}
				else
					printf("Writing MP4: RA_ReadFileIntoMemory() failed\n");

		        MTX_mem_CloseMemory( fileMem );
		        MTX_mem_Destroy( fileMem );
			}
		}
	}
	printf("*** %s",(bSuccess) ? "Success\n" : "Failure\n");
}


