/******************************************************************************
 * This software module was originally developed by Agfa Monotype Corporation
 * in the course of development of the ISO/IEC 14996-18 standard for reference 
 * purposes and its performance may not have been optimized. This software 
 * module is an implementation of font compression technology as specified 
 * by the ISO/IEC 14996-18 standard.
 * 
 * ISO/IEC gives users a free copyright license to this software module or
 * modifications thereof for use in products claiming conformance to 
 * ISO/IEC 14996-18 International Standard. ISO/IEC gives users the same free
 * license to this software module or modifications thereof for research 
 * purposes and further ISO/IEC standardization.
 * 
 * Those intending to use this software module in products are advised that
 * its use may infringe existing patents and that this license does not grant 
 * rights under any patents. ISO/IEC have no liability for use of this 
 * software module or modifications thereof.
 * 
 * No license is granted and copyright is not released for products that do 
 * not conform to ISO/IEC 14996-18 International Standard.
 * 
 * Agfa Monotype Corporation retains full right to modify and use the code for
 * its own purpose, assign or donate the code to a third party and to inhibit 
 * third parties from using the code for products that do not conform to
 * audiovisual and image-coding related ITU recommendations and 
 * ISO/IEC International Standards. All rights not expressly granted to the 
 * software module or modifications thereof are reserved by 
 * Agfa Monotype Corporation.
 * 
 * This copyright notice must be included in all copies or derivative works.
 * Copyright (c) ISO/IEC 2003. All rights reserved.
 ******************************************************************************/

/*
 * File:						RA_TEST.C
 * Revision history
 * $Log: RA_TEST.C,v $
 * Revision 1.1  2004/02/10 22:01:09  vladlev
 * Initial version
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <assert.h>
#include <string.h>

#include "ERRCODES.H"
#include "CONFIG.h"


#include "sfnt.h"
#include "MEMORY.H"
#include "BITIO.H"
#include "STRCTR.H"
#include "RADECOMP.H"
#include "RA_COMP.H"
#include "tablecmp.h"
#include "TTF_CONV.H"
#include "RA_TTF.H"
#include "RA_TEST.H"
#include "RA_UTIL.H"


extern CHAR *GetTablePtr( CHAR *sfnt, int fontOff, sfnt_TableTag tag );

/* This function does some testing to ensure that the random access decompressor iw working. */
int MTX_RA_Test( MTX_MemHandler *mem, char *inDataName, char *compressedDataName )
{
	int bSuccess = true;
	unsigned char *inData;
	long inDataLength;
    clock_t t1, t2;
    double dur1 = 0.0;
    unsigned char *compressedData;
    long compressedDataLength;

		

	printf("Begin TEST\n");
	
	compressedData = RA_ReadFileIntoMemory( mem, compressedDataName, &compressedDataLength );
	if (!compressedData)
		return false;
    {
    	char *data;
    	long outLen, i;
    	MTX_RA_DECOMP *decomp;
    	
    	
    	decomp = MTX_RA_DECOMP_Create( mem, compressedData );
		if (!decomp)
			return false;
        
 		t2 			= clock();
    	data = MTX_RA_GetEverythingInFragmentPtr( decomp, &outLen );
		if (!data)
			return false;

		t1			= clock(); 
		dur1		= ((difftime(t1,t2))/ (CLOCKS_PER_SEC));
		/* printf("MTX_RA_GetEverythingInFragmentPtr() time = %f\n", (double)dur1); /* */
    	
    	
 		inData		= RA_ReadFileIntoMemory( mem, inDataName, &inDataLength );
		printf("Original Size = \"%d\"\n", inDataLength );
		if ( !inData || outLen != inDataLength )
		{
			printf("*** ERROR *** out_size = %d\n", outLen);
			bSuccess = false;
		}
  	
	if ( bSuccess ) printf("Length check is OK !!!!\n");

    	for ( i = 0; i < outLen; i++ ) 
		{
    		/* assert( data[i] == (char)inData[i] ); */
	  		if ( data[i] != (char)inData[i] )
			{
				printf("*** ERROR *** at byte i=%d was %02x sb %02x\n",i, data[i],inData[i]);
				bSuccess = false;
			}
		}
		if ( bSuccess ) printf("Sequential data read check is OK !!!!\n");
    	
	    MTX_RA_ReleaseFragment( decomp, data );

     	
      	outLen = MTX_RA_GetDataSize( decomp );
     	printf("Starting Random Access Check\n");
     	
 		t2 			= clock();
    	for ( i = 0; i < outLen && bSuccess; i += 100 ) {
     		long k, byteCount = 150;
     		if ( outLen - i < byteCount ) byteCount = outLen - i;
     		data = MTX_RA_GetFragmentPtr( decomp, i, &byteCount, BYTE_CACHE_INTERVAL );
			if (!data)
				return false;

     		for ( k = 0; k < byteCount; k++ ) {
     			if ( data[k] != (char)inData[i + k] ) {
     				printf("*** ERROR *** Bad Bytes at %ld [%d] [%d]\n", i, (int)data[k], (int)inData[i + k] );
					bSuccess = false;
					break; /*****/
    			}
       			/* assert( data[k] == (char)inData[i + k] ); */
       		}
       		MTX_RA_ReleaseFragment( decomp, data );
		}
		t1			= clock(); 
		dur1		= ((difftime(t1,t2))/ (CLOCKS_PER_SEC));
		/* printf("SUM MTX_RA_GetFragmentPtr() time = %f\n", (double)dur1); /* */
		if ( bSuccess ) printf("Random Access data read check is OK !!!!\n");
		printf("End Random Access Check\n");
     	
  		MTX_RA_DECOMP_Destroy( decomp );
    }
    
	MTX_mem_free( mem, compressedData );
   	MTX_mem_free( mem, inData );
	printf("End TEST\n");
	return bSuccess;
}
 

/*
 * MTX_RA_TTF_Test( mem, "times.ttf", "times.NDE", "times.STR", "times.BTS");
 * The function does some testing to ensure that we can get back the
 * TTF data from the compressed CTF data
 */
int MTX_RA_TTF_Test( char *ttfName, char *compressedDataName )
{
	long i, j;
	MTX_MemHandler *mem;
	int ierrorNumber;
	int bSuccess = false;
	long byteCount;
	char *sfnt1, *sfnt2;
	long length1, length2;
	long numOffsets1, numOffsets2;
	unsigned char *compressedData;
	long compressedDataLength;
	MTX_RA_TT_Decomp *ttDecomp;
	clock_t t1, t2;
	double dur1 = 0.0;

	printf("Begin TTF TEST\n");
	mem = MTX_mem_Create( malloc, realloc, free);
	if ( mem == NULL ) {
		ierrorNumber = ERR_mem_Create_Failed;
	}
	/* try */
    if ( mem != NULL) {
        long numGlyphs = -1;
        long indexToLocFormat1 = -1, indexToLocFormat2 = -1;
        long IndexToLocOffset1 = -1, IndexToLocOffset2 = -1;
        long IndexToLocLength1 = -1, IndexToLocLength2 = -1;
        long glyfOffset1 = -1, glyfOffset2 = -1;
        long glyfLength1 = -1, glyfLength2 = -1;
        int fontOff1, fontOff2;
        int numInTTC, idxTTC;
        sfnt_TableTag tag; 
        int isTTC;
        char *p1, *p2;
        
        compressedData = RA_ReadFileIntoMemory( mem, compressedDataName, &compressedDataLength );
        if (!compressedData)
        {
            ierrorNumber = mem->errorCode;
            goto bailOut; /*****/
        }
        
        t2 			= clock();
        ttDecomp	= MTX_RA_TT_Create( mem, compressedData );
        if (!ttDecomp)
        {
            ierrorNumber = mem->errorCode;
            goto bailOut; /*****/
        }
        
        t1			= clock(); 
        dur1		= ((difftime(t1,t2))/ (CLOCKS_PER_SEC));
        
        printf("MTX_RA_TT_Create() time = %f\n", (double)dur1);
        
        sfnt1 = (char *)RA_ReadFileIntoMemory( mem, ttfName, &length1 );
        if (!sfnt1)
        {
            ierrorNumber = mem->errorCode;
            goto bailOut; /*****/
        }
        
        length2 = MTX_RA_Get_TTF_DataSize( ttDecomp );
        printf("TTF len = %ld, CTF len = %ld\n", length1, length2 );
        
        /* Check if the uncompressed font is a TrueTypeCollection */
        tag = *((sfnt_TableTag *)((CHAR *)sfnt1 + TTC_TAG));
        if ( SWAPENDS_32(tag) == tag_TTCollection ) 
        {
            numInTTC = SWAPENDS_32(*((int *)((CHAR *)sfnt1 + TTC_NUMFONTS)));
            isTTC = 1;
        }
        else
        {
            numInTTC = 1;
            isTTC = 0;
        }
        /* Check if the compressed font is a TrueTypeCollection */
        byteCount = OFFSETTABLESIZE;  /* 12 bytes in the offset table */
        sfnt2 = MTX_RA_Get_TTF_FragmentPtr(ttDecomp, 0, &byteCount);
        if(!sfnt2)
        {
            ierrorNumber = ttDecomp->mem->errorCode;
            goto bailOut; /*****/
        }
        tag = *((sfnt_TableTag *)((CHAR *)sfnt2 + TTC_TAG));
        /* Make sure both fonts are or aren't TTCs and if so; have the same number of fonts */
        if ( SWAPENDS_32(tag) == tag_TTCollection ) 
        {
            if(!isTTC)
            {
                printf("***** ERROR Unompressed file is not a TTC and compressed file is.\n");
                ierrorNumber = 999;
                goto bailOut;
            }
            /* get # of fonts in TTC file */
            if(numInTTC != SWAPENDS_32(*((ULONG *)((CHAR *)sfnt2 + TTC_NUMFONTS))))
            {
                printf("***** ERROR TrueType collections have differnent number of fonts\n");
                ierrorNumber = 999;
                goto bailOut;
            }
        }
        else if(isTTC)
        {
            printf("***** ERROR Compressed file is not a TTC and uncompressed file is.\n");
            ierrorNumber = 999;
            goto bailOut;
        }
        MTX_RA_ReleaseTTF_Fragment(ttDecomp,sfnt2);
        
        /* Run tests for each font in the TTC */        
        for(idxTTC=0; idxTTC<numInTTC; idxTTC++ )
        {
            printf("==============Testing font %d =====================\n", idxTTC);

            if(isTTC)   /* get indices to fonts in TTC */
            {
                /* ttc offset for uncompressed font */
                fontOff1 = SWAPENDS_32( GET_ttc_font_offset( sfnt1, idxTTC ) );
                /* ttc offset for compressed font */
                byteCount = 12 + numInTTC*4;
                p2 = MTX_RA_Get_TTF_FragmentPtr(ttDecomp,0,(long *)&byteCount);
                if(!p2)
                {
                    ierrorNumber = ttDecomp->mem->errorCode;
                    goto bailOut; /*****/
                }
                fontOff2 = SWAPENDS_32(GET_ttc_font_offset(p2, idxTTC));
                MTX_RA_ReleaseTTF_Fragment(ttDecomp,p2);
            }
            else
            {
                fontOff1 = 0;
                fontOff2 = 0;
            }

            numOffsets1 = SWAPENDS_U16( GET_sfnt_numOffsets( sfnt1 + fontOff1 ) );
            byteCount = OFFSETTABLESIZE;
            p2 = MTX_RA_Get_TTF_FragmentPtr( ttDecomp, fontOff2, &byteCount );
            if (!p2 && byteCount > 0)
            {
                ierrorNumber = ttDecomp->mem->errorCode;
                goto bailOut; /*****/
            }

            numOffsets2 = SWAPENDS_U16( GET_sfnt_numOffsets( p2 ) );
            MTX_RA_ReleaseTTF_Fragment( ttDecomp, p2 );
            
            /* TEST 1 */
            if( (!idxTTC && (numOffsets1 != numOffsets2-1 /* one extra 'act3' table */))
                || (idxTTC && (numOffsets1 != numOffsets2 )))
            {
                printf("***** ERROR number of tables in fonts are different\n");
                printf("   TTC index %d", idxTTC);
                ierrorNumber = ERR_fonts_are_different;
                goto bailOut; /*****/
            }
            printf("numOffsets1 == numOffsets2 !\n");
            
            /* TEST 2 */
            byteCount = OFFSETTABLESIZE + numOffsets1 * DirectoryEntrySize;
            sfnt2 = MTX_RA_Get_TTF_FragmentPtr( ttDecomp, fontOff2, &byteCount );
            if (!sfnt2 && byteCount > 0)
            {
                ierrorNumber = ttDecomp->mem->errorCode;
                goto bailOut; /*****/
            }
            
            for ( i = 0; i < numOffsets1; i++ ) {
                long offset1, length1;
                long offset2, length2, tag2;
                
                offset2 = SWAPENDS_U32( GET_sfnt_table_offset( sfnt2, i ) );
                length2 = SWAPENDS_U32( GET_sfnt_table_length( sfnt2, i ) ) ;
                tag2	= SWAPENDS_32( GET_sfnt_table_tag( sfnt2, i ) );
                if(tag2 == tag_ACT)
                    continue;
                printf("%ld: Checking table %c%c%c%c\n", i, (char)(tag2>>24)&0xff, (char)(tag2>>16)&0xff, (char)(tag2>>8)&0xff, (char)(tag2>>0)&0xff );
                /* printf("CTF offset = %ld, end = %ld\n", offset2, offset2 + length2 ); */
                
                offset1 = length1 = -1;
                for ( j = 0; j < numOffsets1; j++ ) {
                    if ( tag2 == SWAPENDS_32( GET_sfnt_table_tag( sfnt1 + fontOff1, j ) ) )
                    {
                        offset1 = SWAPENDS_U32( GET_sfnt_table_offset( sfnt1 + fontOff1, j ) );
                        length1 = SWAPENDS_U32( GET_sfnt_table_length( sfnt1 + fontOff1, j ) );
                        break;
                    }
                }
                if ( offset1 < 0 ) {
                    printf("***** missing table\n");
                    ierrorNumber = ERR_fonts_are_different;
                    goto bailOut; /*****/
                }
                /* tag_IndexToLoc tag_FontHeader */
                if ( tag2 == tag_GlyphData ) {
                    glyfOffset1 = offset1;
                    glyfOffset2 = offset2;
                    glyfLength1 = length1;
                    glyfLength2 = length2;
                    printf("postponing...\n");
                } else {
                    long minLen = length1 < length2 ? length1 : length2;
                    
                    if(tag2 != tag_IndexToLoc)
                    {
                        if (((length1+3) & (~3)) != ((length2+3) & (~3))) {
                        printf("length1(%ld) != length2(%ld)\n", length1, length2);
                        ierrorNumber = ERR_fonts_are_different;
                        goto bailOut; /*****/
                        }
                    }
                    p1 = sfnt1 + offset1;
                    byteCount = length2;
                    p2 = MTX_RA_Get_TTF_FragmentPtr( ttDecomp, offset2, &byteCount );
                    if (!p2 && byteCount > 0)
                    {
                        ierrorNumber = ttDecomp->mem->errorCode;
                        goto bailOut; /*****/
                    }
                    
                    if ( tag2 == tag_MaxProfile ) {
                        numGlyphs = SWAPENDS_U16( GET_maxp_numGlyphs( p1 ) );
                    } else if ( tag2 == tag_FontHeader ) {
                        indexToLocFormat1 = SWAPENDS_16( GET_head_indexToLocFormat( p1 ) );
                        indexToLocFormat2 = SWAPENDS_16( GET_head_indexToLocFormat( p2 ) );
                    }
                    
                    if ( tag2 == tag_IndexToLoc ) {
                        IndexToLocOffset1 = offset1;
                        IndexToLocOffset2 = offset2;
                        IndexToLocLength1 = length1;
                        IndexToLocLength2 = length2;
                        printf("postponing...\n");
                    } else {
                        for ( j = 0; j < minLen; j++ ) {
                            /***** skip the checkSumAdjustment, and indexToLocFormat ******/
                            if ( tag_FontHeader == tag2 && ((j >= 8 && j < 12) || (j >= 50 && j < 52)) ) continue;  /*****/
                            if ( p1[j] != p2[j] ) {
                                printf("*****Byte Difference\n");
                                ierrorNumber = ERR_fonts_are_different;
                                goto bailOut; /*****/
                            }
                        }
                    }
                    MTX_RA_ReleaseTTF_Fragment( ttDecomp, p2 );
                }
            }
            /* TEST 3 */
            printf("Checking glyf && loca tables\n");
            {
                long 				glyphIndex;
                unsigned long		*longIndexToLoc1, *longIndexToLoc2;
                unsigned short		*shortIndexToLoc1, *shortIndexToLoc2;
                long				glyphDataOffset, glyphDataLength;
                char				*spline1, *spline2;
                long				byteCount1, byteCount2, ok;
                
                assert( indexToLocFormat1 != -1 );
                assert( IndexToLocOffset1 > 0 && IndexToLocLength1 > 0 );
                longIndexToLoc1		= (unsigned long *)(sfnt1 + IndexToLocOffset1);
                shortIndexToLoc1	= (unsigned short *)longIndexToLoc1;
                
                assert( indexToLocFormat2 != -1 );
                assert( IndexToLocOffset2 > 0 && IndexToLocLength2 > 0 );
                byteCount			= IndexToLocLength2;
                longIndexToLoc2		= (unsigned long *)MTX_RA_Get_TTF_FragmentPtr( ttDecomp, IndexToLocOffset2, &byteCount );
                if (!longIndexToLoc2 && byteCount > 0)
                {
                    ierrorNumber = ttDecomp->mem->errorCode;
                    goto bailOut; /*****/
                }
                shortIndexToLoc2	= (unsigned short *)longIndexToLoc2;
                
                ok = 0;
                printf("Checking %ld glyphs......\n", numGlyphs );
                for ( glyphIndex = 0; glyphIndex < numGlyphs; glyphIndex++ ) {
                    if ( indexToLocFormat1 == SHORT_INDEX_TO_LOC_FORMAT ) {
                        glyphDataOffset = SWAPENDS_U16(shortIndexToLoc1[glyphIndex]);
                        glyphDataLength = SWAPENDS_U16(shortIndexToLoc1[glyphIndex+1]) - glyphDataOffset;
                        glyphDataOffset += glyphDataOffset; /* times 2 */
                        glyphDataLength += glyphDataLength;
                    } else {
                        glyphDataOffset = SWAPENDS_U32(longIndexToLoc1[glyphIndex]);
                        glyphDataLength = SWAPENDS_U32(longIndexToLoc1[glyphIndex+1]) - glyphDataOffset;
                    }
                    byteCount1 = glyphDataLength;
                    spline1 = sfnt1 + glyfOffset1 + glyphDataOffset;
                    
                    if ( indexToLocFormat2 == SHORT_INDEX_TO_LOC_FORMAT ) {
                        glyphDataOffset = SWAPENDS_U16(shortIndexToLoc2[glyphIndex]);
                        glyphDataLength = SWAPENDS_U16(shortIndexToLoc2[glyphIndex+1]) - glyphDataOffset;
                        glyphDataOffset += glyphDataOffset; /* times 2 */
                        glyphDataLength += glyphDataLength;
                    } else {
                        glyphDataOffset = SWAPENDS_U32(longIndexToLoc2[glyphIndex]);
                        glyphDataLength = SWAPENDS_U32(longIndexToLoc2[glyphIndex+1]) - glyphDataOffset;
                    }
                    byteCount2 = glyphDataLength;
                    spline2 = MTX_RA_Get_TTF_FragmentPtr( ttDecomp, glyfOffset2 + glyphDataOffset, &byteCount2 );
                    if (!spline2 && byteCount2 > 0)
                    {
                        ierrorNumber = ttDecomp->mem->errorCode;
                        goto bailOut; /*****/
                    }
                    /* assert( byteCount2 == byteCount1 || byteCount2 == byteCount1 - 1 ); */				
                    if ( MTX_TTF_GlyphsAreEqual(mem, spline1, byteCount1, spline2, byteCount2 ) ) {
                        ok++;
                    } else {
                        printf("\n*****Glyphs are not equal.\n");
                        ierrorNumber = ERR_fonts_are_different;
                        goto bailOut; /*****/
                    }
                    MTX_RA_ReleaseTTF_Fragment( ttDecomp, spline2 );
                }
                MTX_RA_ReleaseTTF_Fragment( ttDecomp, longIndexToLoc2 );
                printf("%ld glyphs tested and equal !!!\n", ok );
                printf("Done!\n", numGlyphs );
            }
            MTX_RA_ReleaseTTF_Fragment( ttDecomp, sfnt2 );
        } /* for each font in TTC */
        bSuccess = true;
        MTX_RA_TT_Destroy( ttDecomp );
        
        MTX_mem_free( mem, compressedData );
        MTX_mem_free( mem, sfnt1 );
        
        MTX_mem_CloseMemory( mem );
        MTX_mem_Destroy( mem );
    }
	/* catch(const int ierrorNumber) */
	/* this was old "setjmp" error-recovery - how to reimplement? */
	else {
bailOut:
		bSuccess = false;
		if ( mem != NULL ) {
			MTX_mem_FreeAllMemory(mem);
			MTX_mem_Destroy( mem );
		}
		RA_DisplayMessage( ierrorNumber );
	}
	printf("End TTF TEST\n");
	return bSuccess;
}





