/******************************************************************************
 * This software module was originally developed by Agfa Monotype Corporation
 * in the course of development of the ISO/IEC 14996-18 standard for reference 
 * purposes and its performance may not have been optimized. This software 
 * module is an implementation of font compression technology as specified 
 * by the ISO/IEC 14996-18 standard.
 * 
 * ISO/IEC gives users a free copyright license to this software module or
 * modifications thereof for use in products claiming conformance to 
 * ISO/IEC 14996-18 International Standard. ISO/IEC gives users the same free
 * license to this software module or modifications thereof for research 
 * purposes and further ISO/IEC standardization.
 * 
 * Those intending to use this software module in products are advised that
 * its use may infringe existing patents and that this license does not grant 
 * rights under any patents. ISO/IEC have no liability for use of this 
 * software module or modifications thereof.
 * 
 * No license is granted and copyright is not released for products that do 
 * not conform to ISO/IEC 14996-18 International Standard.
 * 
 * Agfa Monotype Corporation retains full right to modify and use the code for
 * its own purpose, assign or donate the code to a third party and to inhibit 
 * third parties from using the code for products that do not conform to
 * audiovisual and image-coding related ITU recommendations and 
 * ISO/IEC International Standards. All rights not expressly granted to the 
 * software module or modifications thereof are reserved by 
 * Agfa Monotype Corporation.
 * 
 * This copyright notice must be included in all copies or derivative works.
 * Copyright (c) ISO/IEC 2003. All rights reserved.
 ******************************************************************************/

/*
 * File:						RA_TTF.C
 * Revision history
 * $Log: RA_TTF.C,v $
 * Revision 1.1  2004/02/10 22:01:09  vladlev
 * Initial version
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <assert.h>
#include <string.h>

#include "ERRCODES.H"
#include "CONFIG.h"
#include "MEMORY.H"
#include "BITIO.H"
#include "STRCTR.H"
#include "RADECOMP.H"
#include "SFNT.H"
#include "TABLECMP.H"
#include "RA_COMP.H"
#include "TTF_CONV.H"
#include "RA_TTF.H"
#include "RA_TEST.H"
#include "RA_UTIL.H"

/*
 * If TT_TO_CT is true then this function converts from TTF to CTF format,
 * otherwise it converts from CTF to TTF format.
 */
int MTX_TT_Converter( CHAR *nameIn, CHAR *nameOut, int TT_TO_CT )
{
	int bSuccess = false;
	int ierrorNumber = 0;

	MTX_MemHandler *mem = MTX_mem_Create( malloc, realloc, free);
	/* try */
	if ( mem != NULL )
	{
		LONG sizeOut, sizeIn;
		TTF_Converter *conv;
		CHAR *dataOut, *dataIn;
		
		if (dataIn = (CHAR *)RA_ReadFileIntoMemory( mem, nameIn, &sizeIn ))
		{
			printf("Size IN = %ld\n", sizeIn );
			if (conv = MTX_TTC_Create( mem, 0 ))
			{
				if ( TT_TO_CT ) 
				{
					dataOut = (CHAR *)MTX_TTC_TTF_To_CTF( conv, (BYTE *)dataIn, sizeIn, &sizeOut );
				}
				else
				{
					dataOut = (CHAR *)MTX_TTC_CTF_To_TTF( conv, (BYTE *)dataIn, sizeIn, &sizeOut );
				}
				if (dataOut)
				{
					MTX_TTC__Destroy( conv );
					MTX_mem_free( mem, dataIn );
					RA_WriteDataToFile( mem, nameOut, dataOut, sizeOut );
					MTX_mem_free( mem, dataOut );

					printf("Size OUT = %ld\n", sizeOut );

					MTX_mem_CloseMemory( mem );
					if(mem->errorCode)
						printf("******* Error: errorCode %d\n", mem->errorCode);
					MTX_mem_Destroy( mem );
					bSuccess = true;
				}
			}
		}
		if (!bSuccess) ierrorNumber = mem->errorCode;
	}
	else ierrorNumber = ERR_mem_Create_Failed;
	/* catch(const int ierrorNumber) */
	if (ierrorNumber)
	{
		if ( mem != NULL ) {
			MTX_mem_FreeAllMemory(mem);
			MTX_mem_Destroy( mem );
		}
		RA_DisplayMessage( ierrorNumber );
	}
	return bSuccess;
}

/*===============================================================*/
/* The constructor. */

static MTX_RA_TT_Decomp *create_act3( MTX_MemHandler *mem, BYTE *compressedData)
{
	MTX_RA_TT_Decomp *t;
    MTX_TABLE_COMPRESS *tc;
    MTX_ROM_BLOCK *rb;
    SHORT numEntries, i;
    CHAR *sfnt;
    sfnt_TableTag tag; 
    ULONG fontOff, tableOffset;
    USHORT numTables;
    
    t = (MTX_RA_TT_Decomp *)MTX_mem_malloc( mem, sizeof( MTX_RA_TT_Decomp ) );
	if (!t && mem->errorCode)
		return NULL;
	t->mem = mem;
    t->tc = MTX_TABLE_COMPRESS_Create(mem);
    if(!t->tc)
        return NULL;

    /* Find ACT table */
    sfnt = compressedData;
    /* check for TrueTypeCollection, set flag & #-fonts-in-file if so */
    tag = *((sfnt_TableTag *)((CHAR *)sfnt + TTC_TAG));
    if ( SWAPENDS_32(tag) == tag_TTCollection ) 
        fontOff = SWAPENDS_32( GET_ttc_font_offset( sfnt, 0 ) );  /* directory of first font */
    else
        fontOff = 0;
    numTables = SWAPENDS_U16(GET_sfnt_numOffsets( sfnt + fontOff ));

    for ( i = 0; i < numTables; i++ )
    {
        tag = SWAPENDS_32(GET_sfnt_table_tag( sfnt + fontOff, i ));
        if(tag == tag_ACT)
        {
            tableOffset = SWAPENDS_U32(GET_sfnt_table_offset( sfnt + fontOff, i ));
            break;
        }
    }
    if(i == numTables)
        return NULL;

    if(!MTX_RA_TT_ReadRom(t, &sfnt[tableOffset]))
        return NULL;
    tc = t->tc;
    rb = t->tc->rb;
    numEntries = t->tc->numEntries;
    t->cmpStart = sfnt;
    t->cmpEnd   = sfnt + t->cmpSize;
    for(i=0; i<numEntries; i++)
    {
        if(rb[i].isCompressed)
        {
            rb[i].decomp = MTX_RA_DECOMP_Create( mem, sfnt + rb[i].cmpStart);
            if(!rb[i].decomp)
                return NULL;
        }
    }
	MTX_TTC_InitCoordEncoding();
	
	return t; /******/
}

MTX_RA_TT_Decomp *MTX_RA_TT_Create( MTX_MemHandler *mem, BYTE *compressedData)
{
	return create_act3(mem, compressedData);
}

ULONG MTX_RA_TT_getUncompressedFileSize(MTX_RA_TT_Decomp *t)
{
    return t->uncmpSize;
}
/*====================================================================================*/

/* 
 * Returns a pointer to the requested data in the ctf data.
 * The caller sets *length to the requested length, and if we return more
 * then we reset *length to the actual length.
 * For reads within the glyph table the function assumes that the client
 * only request entire glyphs. No multi-glyph or partial glyph reads are allowed.
 * CTF data is translated to TTF format on the fly.
 */
static CHAR *get_act3( MTX_RA_TT_Decomp *t, ULONG position, LONG *byteCountPtr )
{
    MTX_ROM_BLOCK *rb;
    SHORT  i;
	register CHAR *data;

    if ( *byteCountPtr == 0 ) {
		return NULL; /*****/
	}
    rb = t->tc->rb;
    for(i=t->tc->numEntries-1; i>=0; i--) /* start at end */
    {
        if(position >= rb[i].start)
            break;
    }
    if(rb[i].isCompressed)
        data = MTX_RA_GetFragmentPtr( rb[i].decomp, position-rb[i].start, byteCountPtr, rb[i].isCompressed );
    else
        data = t->cmpStart + rb[i].cmpStart + position - rb[i].start;

	if (!data)
		return NULL;
    
    /* if a glyph read then expand the ctf spline data to ttf spline data */
    if ( position >= t->glyf_OffsetStart && position < t->glyf_OffsetEnd ) {
    	register CHAR *ttf_data;
    	
    	ttf_data = MTX_TTC_GetTTFGlyphMemory( t->mem, data, *byteCountPtr, byteCountPtr);
		MTX_RA_ReleaseTTF_Fragment( t, data );
    	data = ttf_data;
    }
    
    return data; /*****/
}

CHAR *MTX_RA_Get_TTF_FragmentPtr( MTX_RA_TT_Decomp *t, ULONG position, LONG *byteCountPtr )
{
	return get_act3(t, position, byteCountPtr);
}
/*====================================================================================*/

/*
 * Releases the memory previously allocated by MTX_RA_Get_TTF_FragmentPtr,
 */
static void release_act3( MTX_RA_TT_Decomp *t, void *data )
{
    /* Only free ptrs outside of the compressed font. Pointers inside are "ROM pointers" */
    if((CHAR*)data<t->cmpStart || t->cmpEnd < (CHAR*)data)
    	MTX_mem_free( t->mem, data );
}

void MTX_RA_ReleaseTTF_Fragment( MTX_RA_TT_Decomp *t, void *data )
{
	release_act3( t, data );
}
/*====================================================================================*/

/* Returns the length of the CTF data */
LONG MTX_RA_Get_TTF_DataSize( MTX_RA_TT_Decomp *t )
{
	return (LONG)(t->cmpEnd - t->cmpStart);
}

/* The destructor. */
static void destroy_act3(MTX_RA_TT_Decomp *t )
{
    MTX_ROM_BLOCK *rb;
    SHORT numEntries, i;

    rb = t->tc->rb;
    numEntries = t->tc->numEntries;
    for(i=0; i<numEntries; i++)
        if(rb[i].isCompressed)
            MTX_RA_DECOMP_Destroy(rb[i].decomp);
    MTX_TABLE_COMPRESS_Destroy(t->tc);
	MTX_mem_free( t->mem, t );
}

void MTX_RA_TT_Destroy(MTX_RA_TT_Decomp *t )
{
	destroy_act3(t);
}
/*===============================================================*/






