/******************************************************************************
 * This software module was originally developed by Agfa Monotype Corporation
 * in the course of development of the ISO/IEC 14996-18 standard for reference 
 * purposes and its performance may not have been optimized. This software 
 * module is an implementation of font compression technology as specified 
 * by the ISO/IEC 14996-18 standard.
 * 
 * ISO/IEC gives users a free copyright license to this software module or
 * modifications thereof for use in products claiming conformance to 
 * ISO/IEC 14996-18 International Standard. ISO/IEC gives users the same free
 * license to this software module or modifications thereof for research 
 * purposes and further ISO/IEC standardization.
 * 
 * Those intending to use this software module in products are advised that
 * its use may infringe existing patents and that this license does not grant 
 * rights under any patents. ISO/IEC have no liability for use of this 
 * software module or modifications thereof.
 * 
 * No license is granted and copyright is not released for products that do 
 * not conform to ISO/IEC 14996-18 International Standard.
 * 
 * Agfa Monotype Corporation retains full right to modify and use the code for
 * its own purpose, assign or donate the code to a third party and to inhibit 
 * third parties from using the code for products that do not conform to
 * audiovisual and image-coding related ITU recommendations and 
 * ISO/IEC International Standards. All rights not expressly granted to the 
 * software module or modifications thereof are reserved by 
 * Agfa Monotype Corporation.
 * 
 * This copyright notice must be included in all copies or derivative works.
 * Copyright (c) ISO/IEC 2003. All rights reserved.
 ******************************************************************************/

/*
 * File:						RA_TTF.H
 * Revision history
 * $Log: RA_TTF.H,v $
 * Revision 1.1  2004/02/10 22:01:09  vladlev
 * Initial version
 *
 */

/*
 * If TT_TO_CT is true then this function converts from TTF to CTF format,
 * otherwise it converts from CTF to TTF format.
 */
int MTX_TT_Converter( CHAR *nameIn, CHAR *nameOut, int TT_TO_CT );



typedef struct {
	MTX_MemHandler *mem; 		/* The memory handler. */
    ULONG glyf_OffsetStart;		/* Offset to first byte. */
    ULONG glyf_OffsetEnd;		/* One past the last byte .*/
    /* act3 specific */
    SHORT headerLength;
    ULONG uncmpSize;
    ULONG cmpSize;
    MTX_TABLE_COMPRESS *tc;
	CHAR *cmpStart;               /* Compressed font start (after the header) */
    CHAR *cmpEnd;                 /* just past the last byte of compressed data */
    /* end act3 specific */
    MTX_RA_DECOMP *decomp;
} MTX_RA_TT_Decomp;

/* The constructor. */
MTX_RA_TT_Decomp *MTX_RA_TT_Create ( MTX_MemHandler *mem, BYTE *compressedData );
MTX_RA_TT_Decomp *MTX_RA_TT_ReadRom( MTX_RA_TT_Decomp *t, CHAR* data );
ULONG MTX_RA_TT_getUncompressedFileSize(MTX_RA_TT_Decomp *t);
	
/* 
 * Returns a pointer to the requested data in the ctf data.
 * The caller sets *length to the requested length, and if we return more
 * then we reset *length to the actual length.
 * For reads within the glyph table the function assumes that the client
 * only request entire glyphs. No multi-glyph or partial glyph reads are allowed.
 * CTF data is translated to TTF format on the fly.
 */
CHAR *MTX_RA_Get_TTF_FragmentPtr( MTX_RA_TT_Decomp *t, ULONG position, LONG *byteCountPtr );

/*
 * Releases the memory previously allocated by MTX_RA_Get_TTF_FragmentPtr,
 */
void MTX_RA_ReleaseTTF_Fragment( MTX_RA_TT_Decomp *t, void *data );

/* Returns the length of the CTF data */
LONG MTX_RA_Get_TTF_DataSize( MTX_RA_TT_Decomp *t );

/* The destructor. */
void MTX_RA_TT_Destroy(MTX_RA_TT_Decomp *t);

