/******************************************************************************
 * This software module was originally developed by Agfa Monotype Corporation
 * in the course of development of the ISO/IEC 14996-18 standard for reference 
 * purposes and its performance may not have been optimized. This software 
 * module is an implementation of font compression technology as specified 
 * by the ISO/IEC 14996-18 standard.
 * 
 * ISO/IEC gives users a free copyright license to this software module or
 * modifications thereof for use in products claiming conformance to 
 * ISO/IEC 14996-18 International Standard. ISO/IEC gives users the same free
 * license to this software module or modifications thereof for research 
 * purposes and further ISO/IEC standardization.
 * 
 * Those intending to use this software module in products are advised that
 * its use may infringe existing patents and that this license does not grant 
 * rights under any patents. ISO/IEC have no liability for use of this 
 * software module or modifications thereof.
 * 
 * No license is granted and copyright is not released for products that do 
 * not conform to ISO/IEC 14996-18 International Standard.
 * 
 * Agfa Monotype Corporation retains full right to modify and use the code for
 * its own purpose, assign or donate the code to a third party and to inhibit 
 * third parties from using the code for products that do not conform to
 * audiovisual and image-coding related ITU recommendations and 
 * ISO/IEC International Standards. All rights not expressly granted to the 
 * software module or modifications thereof are reserved by 
 * Agfa Monotype Corporation.
 * 
 * This copyright notice must be included in all copies or derivative works.
 * Copyright (c) ISO/IEC 2003. All rights reserved.
 ******************************************************************************/

/*
 * File:						RA_COMP.H
 * Revision history
 * $Log: Ra_comp.h,v $
 * Revision 1.1  2004/02/10 22:01:10  vladlev
 * Initial version
 *
 */

typedef struct {
		MTX_MemHandler *mem;		/* The memory handler. */
		
		BYTE *dataIn;				/* The input data. */
		CHAR *taken;				/* Indicates if a byte has already been 'taken' by a symbol. */
		LONG lengthIn;				/* The input data length. */
		LONG *grade_vector;			/* sorted strings */
		CHAR *match_length;			/* match_length[i] == length of leading match between grade_vector[i] and grade_vector[i+1] */
		LONG *matches;				/* indices of match */
		LONG symbolCount;			/* The current symbol count. */
		LONG MaxSymbolCount;		/* The maximum allowed symbolcount.*/
		CHAR **dictSymbol;			/* The array of symbol string pointers. */		
		LONG *dictSymbolLength;		/* The array of the symbol string lengths. */
		LONG *offsets;				/* Contains information on how to find the symbol string in the string store */
		
		LONG *count, *parent, *child_0, *child_1; /* Needed for the Huffman tree, 2 * symbolCount LONG */
		LONG totalCount; 			/* The total sum of all symbol event counts. */
		
		/*** Begin The String Store ***/
		BYTE *stringStore;			/* The string store. */
		LONG stringStoreLength; 	/* Actual length of the string store. */
		LONG maxStringStoreLength; 	/* Max length of the string store. */
		LONG nodeLength;			/* The lenght of the 'node' area approximation .*/
		/*** End The String Store ***/
		
		int fileType;				/* the file type */
		int offset_length;			/* to allow explicit or implicit offsets */
} MTX_RA_COMP;

/* The Constructor. */
MTX_RA_COMP *MTX_COMP_Create( MTX_MemHandler *mem );

/* The Destructor. */
void MTX_COMP_Destroy( MTX_RA_COMP *t );

/* The function packs the file 'name' using MTX_RA_COMP */
int MTX_RA_Pack( CHAR *nameIn, CHAR *nameOut, FILE *verbose );
int MTX_RA_PackToRAM( MTX_MemHandler *mem, LONG inSize, CHAR *inData,
                            LONG *compressedSize, CHAR ** compressedData);
