/******************************************************************************
 * This software module was originally developed by Agfa Monotype Corporation
 * in the course of development of the ISO/IEC 14996-18 standard for reference 
 * purposes and its performance may not have been optimized. This software 
 * module is an implementation of font compression technology as specified 
 * by the ISO/IEC 14996-18 standard.
 * 
 * ISO/IEC gives users a free copyright license to this software module or
 * modifications thereof for use in products claiming conformance to 
 * ISO/IEC 14996-18 International Standard. ISO/IEC gives users the same free
 * license to this software module or modifications thereof for research 
 * purposes and further ISO/IEC standardization.
 * 
 * Those intending to use this software module in products are advised that
 * its use may infringe existing patents and that this license does not grant 
 * rights under any patents. ISO/IEC have no liability for use of this 
 * software module or modifications thereof.
 * 
 * No license is granted and copyright is not released for products that do 
 * not conform to ISO/IEC 14996-18 International Standard.
 * 
 * Agfa Monotype Corporation retains full right to modify and use the code for
 * its own purpose, assign or donate the code to a third party and to inhibit 
 * third parties from using the code for products that do not conform to
 * audiovisual and image-coding related ITU recommendations and 
 * ISO/IEC International Standards. All rights not expressly granted to the 
 * software module or modifications thereof are reserved by 
 * Agfa Monotype Corporation.
 * 
 * This copyright notice must be included in all copies or derivative works.
 * Copyright (c) ISO/IEC 2003. All rights reserved.
 ******************************************************************************/

/*
 * File:						RA_UTIL.C
 * Revision history
 * $Log: Ra_util.c,v $
 * Revision 1.1  2004/02/10 22:01:10  vladlev
 * Initial version
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <assert.h>
#include <string.h>

#include "ERRCODES.H"
#include "CONFIG.h"
#include "MEMORY.H"
#include "RA_UTIL.H"

#ifdef ENABLE_DISPLAY
/* Display a simple error message. */
void RA_DisplayMessage(const int ierrorNumber)
{
    printf("\n*****ERROR*****ALERT*****ERROR*****ALERT*****\n");
    printf(  "** THE CODE ENCOUNTERED AN ERROR CONDITION **\n");
    printf(  "The Error Exception is = \"%d\"\n", ierrorNumber );
    printf(  "*****ERROR*****ALERT*****ERROR*****ALERT*****\n");
}
#else
void RA_DisplayMessage(const int ierrorNumber)
{
	ierrorNumber;
}
#endif


/* Simple function to read a file into memory, with complete error checking. */
BYTE *RA_ReadFileIntoMemory( MTX_MemHandler *mem, const CHAR *fname, LONG *size )
{
	FILE *fp;
	int err;
	LONG size_In;
	size_t count;
	BYTE *memArea;
	
	assert( fname != NULL );
	/* Open the file. */
	fp = fopen(fname, "rb"); 
	if ( fp == NULL )

		{
#ifdef DEBUG
		extern int errno;
		/***/ printf("errno = %d\n",errno);
		/***/ fflush(stdout);
#endif
	 	mem->errorCode = ERR_fopen;
		return NULL;
	}
	err = fseek( fp, 0L, SEEK_END ); /* Go to the end */
	if ( err != 0 ) 
	{
	 	mem->errorCode = ERR_fseek;
		return NULL;
	}
	size_In = ftell( fp );
	
	if ( size_In <= 0 ) 
	{
	 	mem->errorCode = ERR_file_len_lessthan_0;
		return NULL;
	}
	if ( ferror(fp) != 0 ) 
	{
	 	mem->errorCode = ERR_ftell;
		return NULL;
	}

	err = fseek( fp, 0L, SEEK_SET ); /* Go to the beginning. */
	if ( err != 0 ) 
	{
	 	mem->errorCode = ERR_fseek;
		return NULL;
	}
	
	/* Allocate the memory. */
	memArea = (BYTE *)MTX_mem_malloc( mem, MAX_SYMBOL_LENGTH + size_In );
	if (!memArea && mem->errorCode)
		return NULL;
	
	/* Read in the data */
	count = fread( memArea, sizeof( CHAR ), size_In, fp );
	if ( count != (size_t)size_In || ferror(fp) != 0 ) 
	{
	 	mem->errorCode = ERR_fread;
		return NULL;
	}
	/* Close the file. */
	err = fclose( fp ); if ( err != 0 ) 
	{
	 	mem->errorCode = ERR_fclose;
		return NULL;
	}
	
	*size = size_In;
	return memArea; /******/
}

/* Simple function, with error checking, to write a memory area to a file */
void RA_WriteDataToFile( MTX_MemHandler *mem, const CHAR *fname, void *dataIn, LONG sizeIn)
{
	FILE *fp;
	int err;
	size_t count;
		
	fp = fopen( fname, "wb" ); 
	if ( fp == NULL ) 
	{
	 	mem->errorCode = ERR_fopen;
		printf("RA_WriteDataToFile error = %d\n", ERR_fopen);
		/*** exit(-11); ***/
	}
	count = fwrite( dataIn, sizeof( CHAR ), sizeIn, fp );
	if ( ferror(fp) != 0 || count != (size_t)sizeIn) 
	{
	 	mem->errorCode = ERR_fwrite;
		printf("RA_WriteDataToFile error = %d\n", ERR_fwrite);
		/*** exit(-12); ***/
	}
	err = fclose( fp ); 
	if ( err != 0 ) 
	{
	 	mem->errorCode = ERR_fclose;
		printf("RA_WriteDataToFile error = %d\n", ERR_fclose);
		/*** exit(-13); ***/
	}
}
