/******************************************************************************
 * This software module was originally developed by Agfa Monotype Corporation
 * in the course of development of the ISO/IEC 14996-18 standard for reference 
 * purposes and its performance may not have been optimized. This software 
 * module is an implementation of font compression technology as specified 
 * by the ISO/IEC 14996-18 standard.
 * 
 * ISO/IEC gives users a free copyright license to this software module or
 * modifications thereof for use in products claiming conformance to 
 * ISO/IEC 14996-18 International Standard. ISO/IEC gives users the same free
 * license to this software module or modifications thereof for research 
 * purposes and further ISO/IEC standardization.
 * 
 * Those intending to use this software module in products are advised that
 * its use may infringe existing patents and that this license does not grant 
 * rights under any patents. ISO/IEC have no liability for use of this 
 * software module or modifications thereof.
 * 
 * No license is granted and copyright is not released for products that do 
 * not conform to ISO/IEC 14996-18 International Standard.
 * 
 * Agfa Monotype Corporation retains full right to modify and use the code for
 * its own purpose, assign or donate the code to a third party and to inhibit 
 * third parties from using the code for products that do not conform to
 * audiovisual and image-coding related ITU recommendations and 
 * ISO/IEC International Standards. All rights not expressly granted to the 
 * software module or modifications thereof are reserved by 
 * Agfa Monotype Corporation.
 * 
 * This copyright notice must be included in all copies or derivative works.
 * Copyright (c) ISO/IEC 2003. All rights reserved.
 ******************************************************************************/

/*
 * File:						RADECOMP.C
 * Revision history
 * $Log: Radecomp.c,v $
 * Revision 1.1  2004/02/10 22:01:10  vladlev
 * Initial version
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <string.h>

#include "CONFIG.H"
#include "ERRCODES.H"
#include "BITIO.H"
#include "RADECOMP.H"

/* externals */
extern void print_code(ULONG,int);

#ifdef DEBUG
void show_string (BYTE *p, int len)
	{
	int i;

	printf("%d ",len);
	putchar('"');
	for (i=0; i<len; i++)
		putchar((p[i]>=' ' && p[i]<='~') ? p[i] : '~');
	printf("\" <");
	for (i=0; i<len; i++)
		printf("%02x ",p[i]);
	printf(">\n");
	}
#endif

/* Returns the cost of storing n in bytes in the "ULong" format. */
LONG MTX_RA_ULongStorageCost( register ULONG n )
{
	register LONG cost = 0;
	register BYTE byte;
	
	do {
		byte = (BYTE)(n & 0x7f);
		n >>= 7;
		if ( n > 0 ) byte |= 0x80;
		cost++;
	} while ( byte & 0x80 ); /* Loop if there are more bits. */
	return cost; /*****/
}

/* Stores n in the "ULong" format to a file. */
void MTX_RA_StoreULong( FILE *fp, ULONG n )
{
	BYTE byte;
	
	do {
		byte = (BYTE)(n & 0x7f);
		n >>= 7;
		if ( n > 0 ) byte |= 0x80;
		putc( byte, fp);
	} while ( byte & 0x80 ); /* Loop if there are more bits. */
}


/* Reads a number in the "ULong" format from memory. */
static ULONG ReadULongMem( BYTE **ptrHandle )
{
	ULONG shift = 0, tmp, n = 0;
	BYTE byte;
	BYTE *ptr = *ptrHandle;
	
	do {
		byte = *ptr++;
		tmp = byte & 0x7f;
		n |= tmp << shift;
		shift += 7;
	} while ( byte & 0x80 ); /* Loop if there are more bits. */
	*ptrHandle = ptr;
	return n; /*****/
}

/* Reads an unsigned character from memory. */
static BYTE ReadChar( BYTE **ptrHandle )
{
	BYTE byte;
	BYTE *ptr = *ptrHandle;
	
	byte = *ptr++;
	*ptrHandle = ptr;
	return byte; /*****/
}


/* Stores a single node to the disk */
static void StoreNodeToDisk( FILE *fp, FILE *strings, MTX_RA_nodeType *node, LONG offset, LONG count, LONG index )
{
	register LONG i;
	BYTE flags;

	flags = (BYTE)(offset >= 0 ? 0x80 : 0); /* Indicate how we can find the string .*/
	if ( count < 64 ) flags |= 0x40; /* The count is less than or equal to 63. */
	flags |= count % 64;
	putc( flags,  fp );
	if ( !(flags & 0x40) ) {
		/* The count is greater than 63 */
		MTX_RA_StoreULong( fp, count/64 );
	}
	if ( index >= 256 ) {
		if ( flags & 0x80 ) {
			/* We store the offset to a string already existing in the string store. */
			putc( offset % 256,  fp );
			MTX_RA_StoreULong( fp, offset/256 );
		} else {
			/* We put the string in the string store. */
			for ( i = 0; i < node->len; i++ ) {
				putc( node->p[i],  strings );
			}
		}
	}
}


/* node-len should already be set */
/* Reads a single node from memory */
static void ReadNodeFromMemory( BYTE **ptrHandle, CHAR *strings, LONG *pos, MTX_RA_nodeType *node, LONG *count, LONG index )
{
	BYTE flags;
	
	flags = ReadChar( ptrHandle );
	count[ index ] = flags % 64;
	if ( !(flags & 0x40) ) {
		/* The count is greater than 63. */
		count[ index ] |= ReadULongMem( ptrHandle ) << 6;
	}
	/* Only enter here for symbols of length greater than 1. */
	/* 0..255 are all single byte symbols. */
	if ( index >= 256 ) {
		if ( flags & 0x80 ) {
			/* First read the offset so that we can fin the string. */
			LONG offset;
			offset		= ReadChar( ptrHandle );
			offset     |= ReadULongMem( ptrHandle ) << 8;
			node->p		= &strings[offset];
		} else {
			/* The string is at the current position. */
			node->p		= &strings[*pos];
			*pos		= *pos + node->len;
		}
	}
}


/* Stores the node/symbol information to disk. */
void MTX_RA_StoreAllNodesToDisk( FILE *fp, FILE *strings, MTX_RA_nodeType *list, LONG offsets[], LONG *count, LONG n )
{
	register LONG i, j;
	register LONG rLen;

	/* MTX_RA_StoreULong( fp, CURRENT_VERSION_NUMBER ); */
	MTX_RA_StoreULong( fp, n );	/* n == count */
	/* *** Do the length fields *** */
	for ( i = 0; i < n; ) {
		assert( list[i].len <= 255 );
		assert( list[i].len >  0);
		putc(list[i].len,fp);	/* Store the length. */
		for ( rLen = 0, j = i; list[i].len == list[j].len && j < n; j++ )
			rLen++;				/* Compute the run length. */
		MTX_RA_StoreULong( fp, rLen );
		i = j;
	}			
	putc( 0, fp); /* Store 0 to indicate the end of the length fields. */
	/* Do the rest (symbol counts and symbol string locations). */
	for ( i = 0; i < n; i++ ) {
		StoreNodeToDisk( fp, strings, &list[i], offsets[i], count[i], i );
	}
}

/*
 * arrays are 2*n LONG, n == symbolCount
 * input:  0..n-1 are set in the count array to >= 0
 * output : parent[], and count[], child_0[], child_1[],  The child contents start at n !
 * The function has been speed-optimized.
 * It is OK for parent to be == NULL.
 * The time is O(n*log(n)) !
 */
/**************************************************************/
/* quicksort <indices> so that vector[indices[i]] is ascending */
/* a subroutine for <MTX_RA_BuildHuffmanTree> 
*/

void quick_index_sort(LONG *vector, LONG *indices, LONG left, LONG right)
{
	LONG i,j,median,temp,v,n;

	n = 1 + right - left;

	if (n==1) /* already sorted */
		return;	

	if (n <= 9)	/* shell_sort little guys */
		{
		LONG gap;

		indices += left;

		/* this sequence of gap's suggested by Sedgewick */
		for (gap=1,temp=n/9; gap<=temp; gap=1+3*gap) {};
		for (/***/; gap>0; gap/=3)
			{
			for (i=gap; i<n; i++)
				{
				for (j=i-gap; j>=0 && vector[indices[j]] > vector[indices[j+gap]]; j-=gap)
					{
					temp = indices[j];
					indices[j] = indices[j+gap];
					indices[j+gap] = temp;
					}
				}
			}
		}
	else
		{
		/* median of 3 elements partition */
		median = (left + right)>>1;
		if (vector[indices[left]] > vector[indices[median]])
			{
			temp = indices[median];
			indices[median] = indices[left];
			indices[left] = temp;
			}
		if (vector[indices[median]] > vector[indices[right]])
			{
			temp = indices[median];
			indices[median] = indices[right];
			indices[right] = temp;
			}
		if (vector[indices[left]] > vector[indices[median]])
			{
			temp = indices[median];
			indices[median] = indices[left];
			indices[left] = temp;
			}

		/* they're now in ascending order ... swap median and right */
		temp = indices[median];
		indices[median] = indices[right];
		indices[right] = temp;

		i = left-1;
		j = right;
		v = vector[indices[right]];
		/* vector[indices[median]] >= v... and vector[indices[left]] <= v; so the following */
		/* while loops will terminate with i<=median and j>=left */

		/* form the partition around element <v> */
		for (;;)
			{
			while (vector[indices[++i]] < v) {};
			while (vector[indices[--j]] > v) {};
			if (i >= j) break;
			temp = indices[i];
			indices[i] = indices[j];
			indices[j] = temp;
			}
		temp = indices[i];
		indices[i] = indices[right];
		indices[right] = temp;

		/* sort the pieces */
		quick_index_sort(vector,indices,left,i-1);
		quick_index_sort(vector,indices,i+1,right);
		}
}

/**************************************************************/
LONG MTX_RA_BuildHuffmanTree(MTX_MemHandler *mem, LONG n, register LONG *count, LONG *parent, LONG *child_0, LONG *child_1 )
{
	LONG *indices,i,a,b;
	LONG *leaf_ptr,num_leaves,num_parents,parent_ptr,next_parent;

	/* make an index vector so that counts[indices[i]] are ascending */
	/* then counts[*indices] is the smallest no-parent leaf count */
	/* and indices++; removes this count from consideration */
	leaf_ptr = indices = (LONG *)mem->malloc(n * sizeof(LONG));
	for (i=0; i<n; i++)
		indices[i] = i;
	quick_index_sort(count,indices,0,n-1);
	num_leaves = n;	
	

	/* since the parents are generated with increasing count, we */
	/* have counts[parent_ptr] is the smallest no-parent parent count */
	/* and parent_ptr++; removes this count from consideration */
	next_parent = parent_ptr = n;

	num_parents = 0;
	for (;;)
		{
		/* done when exactly one parent-less parent */
		if (num_leaves==0 && num_parents==1)
			break;

		/* find smallest count and remove it from contention */
		if (num_leaves>0 && (num_parents==0 || (count[*leaf_ptr] <= count[parent_ptr])))
			{
			a = *leaf_ptr++;
			num_leaves--;
			}
		else if (num_parents>0 && (num_leaves==0 || (count[parent_ptr] <= count[*leaf_ptr])))
			{
			a = parent_ptr++;
			num_parents--;
			}

		/* find (next) smallest count and remove it from contention */
		if (num_leaves>0 && (num_parents==0 || (count[*leaf_ptr] <= count[parent_ptr])))
			{
			b = *leaf_ptr++;
			num_leaves--;
			}
		else if (num_parents>0 && (num_leaves==0 || (count[parent_ptr] <= count[*leaf_ptr])))
			{
			b = parent_ptr++;
			num_parents--;
			}

		/* (maybe) note the new parent of <a> and <b> */
		if (parent)
			{
			parent[a] = next_parent;
			parent[b] = next_parent;
			}

		/* next parent has <a> and <b> as children and sums their counts */
		child_0[next_parent] = a;
		child_1[next_parent] = b;
		count[next_parent] = count[a] + count[b];
		next_parent++;
		num_parents++;
		}

	mem->free(indices);
	return next_parent-1;

}

/**************************************************************/

/* Contains all 256 different bytes 0..255 */
CHAR MTX_RA_AllCharArray[256];


/* we want to use the version of the functions
* <MTX_RA_GetFragmentPtr> and <MTX_RA_GetEverythingInFragmentPtr>
* which call <ReadSymbol> as defined in the file <canonical.c>
*/
CHAR * ReadSymbol( register MTX_RA_DECOMP *t, LONG *len );

/* Returns the length of the uncompressed data. */
LONG MTX_RA_GetDataSize( MTX_RA_DECOMP *t )
{
	return t->unCompressedByteCount; /*****/
}




/* Decompresses everything and returns the pointer to the decompressed data */
CHAR *MTX_RA_GetEverythingInFragmentPtr( MTX_RA_DECOMP *t, LONG *outLen )
{
	CHAR *data;
	LONG byteCount;
	register LONG i, j;
	register CHAR *p;
	LONG len;
	
	byteCount = t->unCompressedByteCount;
	MTX_BITIO_fseek(t->bio,t->token_offset);
	t->next = MTX_BITIO_ReadValue(t->bio, TABLE_BITS);
	data = (CHAR *)MTX_mem_malloc( t->mem, byteCount );
	if (!data && t->mem->errorCode)
		return NULL;
	
	for ( i = 0; i < byteCount; ) {
		p = ReadSymbol( t, &len );
		j = len;
		for ( ;j > 0; j-- ) {
			data[i++] = *p++;
		}
	}
	assert( i == byteCount );

	
	*outLen = byteCount;
	return data; /*****/
}

/* 
 * Returns a pointer to the requested data.
 * The caller sets *length to the requested length, and if we return more
 * then we reset *length to the actual length.
 */
CHAR *MTX_RA_GetFragmentPtr( MTX_RA_DECOMP *t, ULONG position, LONG *byteCountPtr, SHORT byteCacheInterval )
{
	register CHAR *p, *data;
	register LONG i, j, byteCount = *byteCountPtr;
	LONG len; /* not a register variable since we need &len */

#ifdef DEBUG
	printf("**********\nMTX_RA_GetFragmentPtr(pos=%d,len=%d)\n",position,byteCount);
#endif

	assert( byteCount > 0 );
	data		=  (CHAR *)MTX_mem_malloc( t->mem, byteCount );
	if (!data && t->mem->errorCode)
		return NULL;
	
	j = position / byteCacheInterval;
	i = j * byteCacheInterval - t->deltaBytes[j]; /* The current position. */
	assert( position >= (ULONG)i );

	/* access bitmark[j] */
	{
		ULONG  bitMark;
		BYTE *p = &t->bitMarks[4*j];
		bitMark = *p++;
		bitMark = (bitMark<<8) | *p++;
		bitMark = (bitMark<<8) | *p++;
		bitMark = (bitMark<<8) | *p++;
	
		MTX_BITIO_fseek( t->bio, bitMark );	/* Go to a position on or before "position". */
	}
	/* gotta prime the pump for ReadSymbol() */
	t->next = MTX_BITIO_ReadValue(t->bio, TABLE_BITS);

#ifdef DEBUG
	printf("first %d bits: ",t->bits[t->num_groups-1]);
	print_code(t->next,t->bits[t->num_groups-1]);
	printf("\n");
#endif /* DEBUG */

	/* Loop until we straddle position */
	for(;;i += j) {
		p = ReadSymbol( t, &len );
		j = len;
		if ( (ULONG)(i + j) > position ) break; /*****/
	}
	assert( j >= 1 ); /* j == len */
	assert( (ULONG)i <= position );
	assert( (ULONG)(i + j) > position );
	
	/* Skip extra data in the beginning of the first symbol. */
	i 	 = position - i;
	p 	+= i;
	j 	-= i; /* j == len */
	assert( j > 0 );

	/* First read the remainder of the first partial symbol, and then */
	/* read all intermediate symbols and the last partial symbol. */
	for (i = 0;;) {
		if ( i + j < byteCount ) {
			/* Copy the intermediate (or first) symbol. */
			do {
				data[i++] = *p++;
			} while ( --j > 0 );
		} else {
			/* Copy the last partial symbol. */
			j = byteCount - i;
			do {
				data[i++] = *p++;
			} while ( --j > 0 );
			assert( i == byteCount );
			return data; /***** DONE! *****/
		}
		p = ReadSymbol( t, &len );
		j = len;
	}
	/* We never get here so no return is needed. */
}



/*
 * Releases the memory previously allocated by MTX_RA_GetFragmentPtr,
 * or MTX_RA_GetEverythingInFragmentPtr.
 */
void MTX_RA_ReleaseFragment( MTX_RA_DECOMP *t, void *data )
{
	MTX_mem_free( t->mem, data );
}



