/******************************************************************************
 * This software module was originally developed by Agfa Monotype Corporation
 * in the course of development of the ISO/IEC 14996-18 standard for reference 
 * purposes and its performance may not have been optimized. This software 
 * module is an implementation of font compression technology as specified 
 * by the ISO/IEC 14996-18 standard.
 * 
 * ISO/IEC gives users a free copyright license to this software module or
 * modifications thereof for use in products claiming conformance to 
 * ISO/IEC 14996-18 International Standard. ISO/IEC gives users the same free
 * license to this software module or modifications thereof for research 
 * purposes and further ISO/IEC standardization.
 * 
 * Those intending to use this software module in products are advised that
 * its use may infringe existing patents and that this license does not grant 
 * rights under any patents. ISO/IEC have no liability for use of this 
 * software module or modifications thereof.
 * 
 * No license is granted and copyright is not released for products that do 
 * not conform to ISO/IEC 14996-18 International Standard.
 * 
 * Agfa Monotype Corporation retains full right to modify and use the code for
 * its own purpose, assign or donate the code to a third party and to inhibit 
 * third parties from using the code for products that do not conform to
 * audiovisual and image-coding related ITU recommendations and 
 * ISO/IEC International Standards. All rights not expressly granted to the 
 * software module or modifications thereof are reserved by 
 * Agfa Monotype Corporation.
 * 
 * This copyright notice must be included in all copies or derivative works.
 * Copyright (c) ISO/IEC 2003. All rights reserved.
 ******************************************************************************/

/*
 * File:						RADECOMP.H
 * Revision history
 * $Log: Radecomp.h,v $
 * Revision 1.1  2004/02/10 22:01:10  vladlev
 * Initial version
 *
 */

#define FILE_TYPE_ANY 50
#define FILE_TYPE_TTF 51
#define FILE_TYPE_CTF 52


#ifdef DEBUG
extern void show_string(BYTE *, int);
#endif


extern void MTX_RA_StoreULong( FILE *fp, ULONG n );

/* Returns the cost of storing n in bytes in the "ULong" format. */
LONG MTX_RA_ULongStorageCost( register ULONG n );

/* 
 * Returns the total length of the compressed data, and finds the pointers and
 * and lengths of the 3 separate chuncks of data within the merged file.
 */
int MTX_RA_GetThe3Pieces(	MTX_MemHandler *mem, BYTE *compressedData,
							BYTE **nodes, LONG *nodeSize,
							BYTE **strings, LONG *stringSize,
							BYTE **bitsStream, LONG *bitsStreamLength );


typedef struct {
	BYTE len;	/* symbol length	*/
	CHAR *p;	/* The string		*/
} MTX_RA_nodeType;

/* Stores the node/symbol information to disk. */
extern void MTX_RA_StoreAllNodesToDisk( FILE *fp, FILE *strings,
			MTX_RA_nodeType *list, LONG offsets[], LONG *count, LONG n );

/* Contains all 256 different bytes 0..255 */
extern CHAR MTX_RA_AllCharArray[];

/*
 * arrays are 2*n LONG, n == symbolCount
 * input:  0..n-1 are set in the count array to >= 0
 * output : parent[], and count[], child_0[], child_1[],  The child contents start at n !
 * The function has been speed-optimized.
 * It is OK for parent to be == NULL.
 * The time is O(n*log(n)) !
 */
LONG MTX_RA_BuildHuffmanTree( MTX_MemHandler *mem, LONG n, LONG *count, LONG *parent, LONG *child_0, LONG *child_1 );

#define BYTE_COUNT_LENGTH		28
#define BYTE_CACHE_INTERVAL		128L

#define CURRENT_VERSION_NUMBER 3

#define MAGIC_NUMBER			0x5a

typedef struct {
	MTX_MemHandler *mem;
	BITIO *bio;
	LONG unCompressedByteCount;	/* original file size */

	/* dictionary stuff */
	LONG num_tokens;	/* number of tokens in dictionary */
	LONG string_len;	/* length of <string> */
	BYTE *string;		/* BYTES for all tokens in dictionary */
	LONG *offsets;		/* token <i> begins at starts + offsets[i] */
	BYTE *lengths;		/* token <i> is lengths[i] bytes LONG */

	/* canonical huffman code stuff */
	LONG num_groups;	/* number of groups (codes with same number of bits) */
	ULONG *firsts;		/* first huffman code in each group - left justified */
	ULONG *lasts;		/* last huffman code in each group - left justified */
	BYTE *bits;			/* number of significant bits in each group */
	LONG *tokens;		/* first token number in each group */
	ULONG next;			/* next group of bits ... maintained by <get_next_token> */
	ULONG next_mask;	/* to toss excess bits on left */
    LONG  slowGroup;    /* first group not completely in table */
/* a table of frequently used huffman codes for quick decoding */
    LONG token_num_table[TABLE_SIZE];
    BYTE token_bits_table[TABLE_SIZE];
	/* navigation */
	BYTE *bitMarks;     /* point to ROM and decode the ULONGs on the fly */
	
	BYTE *deltaBytes;	/* error in the anchor position compared to the ideal position */
	ULONG token_offset;	/* position of first token ... ie: the encoded data */
	} MTX_RA_DECOMP;


/* The Constructor. */
MTX_RA_DECOMP *MTX_RA_DECOMP_Create(MTX_MemHandler *mem, BYTE *compressedData );

/* Returns the length of the uncompressed data. */
LONG MTX_RA_GetDataSize( MTX_RA_DECOMP *t );

/* Decompresses everything and returns the pointer to the decompressed data */
CHAR *MTX_RA_GetEverythingInFragmentPtr( MTX_RA_DECOMP *t, LONG *outLen );
								   
/* 
 * Returns a pointer to the requested data.
 * The caller sets *byteCountPtr to the requested length, and if we return more
 * then we reset *byteCountPtr to the actual length.
 */
CHAR *MTX_RA_GetFragmentPtr( MTX_RA_DECOMP *t, ULONG position, LONG *byteCountPtr, SHORT byteCacheInterval );

/*
 * Releases the memory previously allocated by MTX_RA_GetFragmentPtr,
 * or MTX_RA_GetEverythingInFragmentPtr.
 */
void MTX_RA_ReleaseFragment( MTX_RA_DECOMP *t, void *data );

/* The Destructor. */
void MTX_RA_DECOMP_Destroy(MTX_RA_DECOMP *t);



