/******************************************************************************
 * This software module was originally developed by Agfa Monotype Corporation
 * in the course of development of the ISO/IEC 14996-18 standard for reference 
 * purposes and its performance may not have been optimized. This software 
 * module is an implementation of font compression technology as specified 
 * by the ISO/IEC 14996-18 standard.
 * 
 * ISO/IEC gives users a free copyright license to this software module or
 * modifications thereof for use in products claiming conformance to 
 * ISO/IEC 14996-18 International Standard. ISO/IEC gives users the same free
 * license to this software module or modifications thereof for research 
 * purposes and further ISO/IEC standardization.
 * 
 * Those intending to use this software module in products are advised that
 * its use may infringe existing patents and that this license does not grant 
 * rights under any patents. ISO/IEC have no liability for use of this 
 * software module or modifications thereof.
 * 
 * No license is granted and copyright is not released for products that do 
 * not conform to ISO/IEC 14996-18 International Standard.
 * 
 * Agfa Monotype Corporation retains full right to modify and use the code for
 * its own purpose, assign or donate the code to a third party and to inhibit 
 * third parties from using the code for products that do not conform to
 * audiovisual and image-coding related ITU recommendations and 
 * ISO/IEC International Standards. All rights not expressly granted to the 
 * software module or modifications thereof are reserved by 
 * Agfa Monotype Corporation.
 * 
 * This copyright notice must be included in all copies or derivative works.
 * Copyright (c) ISO/IEC 2003. All rights reserved.
 ******************************************************************************/

/*
 * File						SFNT.H
 * Revision history
 * $Log: SFNT.H,v $
 * Revision 1.3  2004/03/01 22:33:01  vladlev
 * Added tag definitions for Apple's TT GX
 *
 * Revision 1.2  2004/02/19 19:36:43  vladlev
 * Added new functionality to write uncompressed OpenType TrueType/CFF
 * files in output MP4 file.
 *
 * Revision 1.1  2004/02/10 22:01:10  vladlev
 * Initial version
 *
 */

#define FWord SHORT
#define uFWord USHORT

#ifndef Fixed
#define Fixed LONG
#endif

#define tag_ACT                 0x61637433        /* 'act3' */

#define tag_TTCollection		0x74746366		  /* 'ttcf' */

#define tag_FontHeader          0x68656164        /* 'head' */
#define tag_HoriHeader          0x68686561        /* 'hhea' */
#define tag_IndexToLoc          0x6c6f6361        /* 'loca' */
#define tag_MaxProfile          0x6d617870        /* 'maxp' */
#define tag_ControlValue        0x63767420        /* 'cvt ' */
#define tag_MetricValue         0x6d767420        /* 'mvt ' */
#define tag_PreProgram          0x70726570        /* 'prep' */
#define tag_GlyphData           0x676c7966        /* 'glyf' */
#define tag_HorizontalMetrics   0x686d7478        /* 'hmtx' */
#define tag_CharToIndexMap      0x636d6170        /* 'cmap' */
#define tag_Kerning             0x6b65726e        /* 'kern' */
#define tag_HoriDeviceMetrics   0x68646d78        /* 'hdmx' */
#define tag_Encryption          0x63727970        /* 'cryp' */
#define tag_NamingTable         0x6e616d65        /* 'name' */
#define tag_FontProgram         0x6670676d        /* 'fpgm' */

#define tag_VertDeviceMetrics   0x56444d58        /* 'VDMX' */

#define tag_EBLC				0x45424C43		  /* 'EBLC' */
#define tag_EBDT				0x45424454		  /* 'EBDT' */

#define tag_CFF					0x43464620		  /* 'CFF ' */

#define tag_EBLC2				0x626c6f63		  /* 'bloc' Apple's TT GX version */
#define tag_EBDT2				0x62646174		  /* 'bdat' Apple's TT GX version */

#define SHORT_INDEX_TO_LOC_FORMAT		0
#define LONG_INDEX_TO_LOC_FORMAT		1
#define GLYPH_DATA_FORMAT				0

#define TTC_TAG			0
#define TTC_NUMFONTS	8
#define TTC_OFFSET		12

typedef long sfnt_TableTag;

#define OFFSETTABLESIZE 12L
#define DirectoryEntrySize 16L

#define GET_sfnt_numOffsets( sfnt ) ( *((USHORT *)((char *)(sfnt) + 4)) )
#define GET_sfnt_table_tag( sfnt, i ) 				( *((sfnt_TableTag *)((char *)(sfnt) + OFFSETTABLESIZE + (i) * DirectoryEntrySize)) )
#define GET_sfnt_table_checkSum( sfnt, i )			( *((ULONG *)((char *)(sfnt) + OFFSETTABLESIZE + (i) * DirectoryEntrySize + 4)) )
#define SET_sfnt_table_checkSum( sfnt, i, value )	( *((ULONG *)((char *)(sfnt) + OFFSETTABLESIZE + (i) * DirectoryEntrySize + 4)) = value )
#define GET_sfnt_table_offset( sfnt, i )			( *((ULONG *)((char *)(sfnt) + OFFSETTABLESIZE + (i) * DirectoryEntrySize + 8)) )
#define SET_sfnt_table_offset( sfnt, i, value )		( *((ULONG *)((char *)(sfnt) + OFFSETTABLESIZE + (i) * DirectoryEntrySize + 8)) = (value) )
#define GET_sfnt_table_length( sfnt, i )			( *((ULONG *)((char *)(sfnt) + OFFSETTABLESIZE + (i) * DirectoryEntrySize + 12)) )
#define SET_sfnt_table_length( sfnt, i, value )		( *((ULONG *)((char *)(sfnt) + OFFSETTABLESIZE + (i) * DirectoryEntrySize + 12)) = (value) )

#define GET_ttc_font_offset( sfnt, i )	(( *((int *)((char *)(sfnt) + TTC_OFFSET + i*4)) ))
#define SET_ttc_font_offset( sfnt, i, value )	( *((int *)((char *)(sfnt) +  TTC_OFFSET + i*4)) = (value) )

#define GET_head_magicNumber( head ) ( *((ULONG *)((char *)(head) + 12)) )
#define GET_head_unitsPerEm( head ) ( *((USHORT *)((char *)(head) + 18)) )
#define GET_head_indexToLocFormat( head ) ( *((SHORT *)((char *)(head) + 50)) )
#define GET_head_glyphDataFormat( head ) ( *((SHORT *)((char *)(head) + 52)) )

#define SET_head_checkSumAdjustment( head, value ) ( *((ULONG *)((char *)(head) + 8)) = (value) )
#define SET_head_indexToLocFormat( head, value ) ( *((SHORT *)((char *)(head) + 50)) = value )

#define GET_maxp_numGlyphs( maxp ) ( *((USHORT *)((char *)(maxp) + 4)) )

/*
 * UNPACKING Constants
*/
#define ONCURVE  			0x01
#define XSHORT   			0x02
#define YSHORT   			0x04
#define REPEAT_FLAGS    	0x08 /* repeat flag n times */
#define SHORT_X_IS_POS   	0x10 /* the short vector is positive */
#define NEXT_X_IS_ZERO   	0x10 /* the relative x coordinate is zero */
#define SHORT_Y_IS_POS   	0x20 /* the short vector is positive */
#define NEXT_Y_IS_ZERO   	0x20 /* the relative y coordinate is zero */


/*
 * Composite glyph constants
 */
#define COMPONENTCTRCOUNT 			-1		/* ctrCount == -1 for composite */
#define ARG_1_AND_2_ARE_WORDS		0x0001	/* if set args are words otherwise they are bytes */
#define ARGS_ARE_XY_VALUES			0x0002	/* if set args are xy values, otherwise they are points */
#define ROUND_XY_TO_GRID			0x0004	/* for the xy values if above is true */
#define WE_HAVE_A_SCALE				0x0008	/* Sx = Sy, otherwise scale == 1.0 */
#define NON_OVERLAPPING				0x0010	/* set to same value for all components */
#define MORE_COMPONENTS				0x0020	/* indicates at least one more glyph after this one */
#define WE_HAVE_AN_X_AND_Y_SCALE	0x0040	/* Sx, Sy */
#define WE_HAVE_A_TWO_BY_TWO		0x0080	/* t00, t01, t10, t11 */
#define WE_HAVE_INSTRUCTIONS		0x0100	/* instructions follow */
#define USE_MY_METRICS				0x0200	/* */


#define GET_hhea_numberOf_LongHorMetrics( hhea ) ( *((USHORT *)((hhea) + 34)) )


#define HorizontalMetricsSize 4
#define GET_hmtx_advanceWidth_Ptr( hmtx, i ) 	( (USHORT *)((char *)(hmtx) + (i)*HorizontalMetricsSize + 0))
#define GET_hmtx_leftSideBearing_Ptr( hmtx, i ) ( (SHORT *)((char *)(hmtx) + (i)*HorizontalMetricsSize + 2))
#define GET_hmtx_FirstSoloLsb_Ptr( hmtx, numEntries ) ( (USHORT *)((char *)(hmtx) + (numEntries)*HorizontalMetricsSize))



/*	Begin hdmx 
 *	Each record is n+2 bytes, padded to long word alignment.
 *	First byte is ppem, second is maxWidth, rest are widths for each glyph
 */

#define sfnt_devMetricsSize 8
#define GET_hdmx_version_Ptr( hdmx )	((SHORT *)((char *)(hdmx) + 0))
#define GET_hdmx_numRecords_Ptr( hdmx )	((SHORT *)((char *)(hdmx) + 2))
#define GET_hdmx_recordSize_Ptr( hdmx )	((LONG *)((char *)(hdmx) + 4))


#define GET_hdmxSubRecord_ppem_Ptr( devSubRecord )		((BYTE *)((BYTE *)(devSubRecord) + 0))
#define GET_hdmxSubRecord_maxWidth_Ptr( devSubRecord )	((BYTE *)((BYTE *)(devSubRecord) + 1))
#define GET_hdmxSubRecord_width_Ptr( devSubRecord, i )	((BYTE *)((BYTE *)(devSubRecord) + 2 + (i) ))

/* End hdmx */

/*** Begin VDMX ***/

#define vdmxGroupSize 4
#define vdmxTableSize 6

#define GET_vdmxGroup_recs_Ptr( groups )		((USHORT *)((char *)(groups) + 0))
#define GET_vdmxGroup_startSize_Ptr( groups )	((BYTE *)( (char *)(groups) + 2))
#define GET_vdmxGroup_endSize_Ptr( groups )		((BYTE *)( (char *)(groups) + 3))

#define GET_vdmxGroup_entry_yPelHeight_Ptr( groups, i ) ((USHORT *)( (char *)(groups) + 4 + (i) * vdmxTableSize + 0))
#define GET_vdmxGroup_entry_yMax_Ptr( groups, i ) ((USHORT *)( (char *)(groups) + 4 + (i) * vdmxTableSize + 2))
#define GET_vdmxGroup_entry_yMin_Ptr( groups, i ) ((USHORT *)( (char *)(groups) + 4 + (i) * vdmxTableSize + 4))


#define sfnt_vdmxRatio_Size 4
/* sfnt_vdmxFormatSize includes one sfnt_vdmxRatio_Size */
#define sfnt_vdmxFormatSize 10

/*** End VDMX ***/

/*** Begin 'name' ***/
#define NAMERECORDSIZE 12
#define MSFT_PLATFORM   3
#define UNICODE1_1      1

#define GET_name_recordCount( name ) ( *((USHORT *)((char *)(name) + 2)) )
#define GET_name_stringOffset( name ) ( *((USHORT *)((char *)(name) + 4)) )

typedef struct {
	USHORT platformID;
	USHORT encodingID;
	USHORT languageID;
	USHORT nameID;
	USHORT length;
	USHORT offset;
} TTF_Name_Record;

#define GET_nameRecord_entryPtr( name, i ) ( (TTF_Name_Record *)((char *)(name) + 6 + (i) * NAMERECORDSIZE) )

/*** End 'name' ***/
