/******************************************************************************
 * This software module was originally developed by Agfa Monotype Corporation
 * in the course of development of the ISO/IEC 14996-18 standard for reference 
 * purposes and its performance may not have been optimized. This software 
 * module is an implementation of font compression technology as specified 
 * by the ISO/IEC 14996-18 standard.
 * 
 * ISO/IEC gives users a free copyright license to this software module or
 * modifications thereof for use in products claiming conformance to 
 * ISO/IEC 14996-18 International Standard. ISO/IEC gives users the same free
 * license to this software module or modifications thereof for research 
 * purposes and further ISO/IEC standardization.
 * 
 * Those intending to use this software module in products are advised that
 * its use may infringe existing patents and that this license does not grant 
 * rights under any patents. ISO/IEC have no liability for use of this 
 * software module or modifications thereof.
 * 
 * No license is granted and copyright is not released for products that do 
 * not conform to ISO/IEC 14996-18 International Standard.
 * 
 * Agfa Monotype Corporation retains full right to modify and use the code for
 * its own purpose, assign or donate the code to a third party and to inhibit 
 * third parties from using the code for products that do not conform to
 * audiovisual and image-coding related ITU recommendations and 
 * ISO/IEC International Standards. All rights not expressly granted to the 
 * software module or modifications thereof are reserved by 
 * Agfa Monotype Corporation.
 * 
 * This copyright notice must be included in all copies or derivative works.
 * Copyright (c) ISO/IEC 2003. All rights reserved.
 ******************************************************************************/

/*
 * File:						STRCTR.H
 * Revision history
 * $Log: Strctr.h,v $
 * Revision 1.1  2004/02/10 22:01:10  vladlev
 * Initial version
 *
 */

typedef struct {
	MTX_MemHandler *mem; 		/* The memory handler. */
	LONG strLength;				/* The length of the strings we are working on. */
	LONG hashTableSize;			/* The length of our hastable. */
	CHAR **ht_string;			/* The string pointers in the hashtable .*/
	BYTE *ht_xor;				/* byte wide exclusive or of the string */
	USHORT *ht_count;			/* The event counts for the hastable. */
			
	LONG maxNodeIndex;			/* The index of the biggest member of ht_count. */
	USHORT maxCount;			/* The maximum count from ht_count. */ 
} MTX_RA_StringCounter;

/* The constructor. */
MTX_RA_StringCounter *MTX_STRCTR_Create( MTX_MemHandler *mem, LONG strLength, LONG hashTableSize );

/* Call to reset the object */
void MTX_STRCTR_ClearHashTable( MTX_RA_StringCounter *t, LONG strLength );


/*  First call this with all strings */
/* The function increments the event count for string p */
void MTX_STRCTR_Increment( MTX_RA_StringCounter *t, CHAR *p );
/* The function zeroes the event count for string p */
void MTX_STRCTR_Clear( MTX_RA_StringCounter *t, CHAR *p );
/* Call this to get the most frequent string (highest event count). */
CHAR *MTX_STRCTR_GetMaxString( MTX_RA_StringCounter *t, USHORT *eventCount );
/* Returns the n strings with the largest event counts in descending order. */
void MTX_STRCTR_GetNMaxStrings( MTX_RA_StringCounter *t, LONG n, CHAR **arr, LONG *eventCounts );
/* Sorts the two arrays according to eventCounts. */
void MTX_STRCTR_SortNMaxStrings( LONG n, CHAR **arr, LONG *eventCounts );
/* Sorts the three arrays according to eventCounts. */
void MTX_STRCTR_SortNMaxStrings2( LONG n, CHAR **arr, LONG *eventCounts, LONG *arr2 );


/* The destructor. */
void MTX_STRCTR_Destroy(MTX_RA_StringCounter *t);
