/******************************************************************************
 * This software module was originally developed by Agfa Monotype Corporation
 * in the course of development of the ISO/IEC 14996-18 standard for reference 
 * purposes and its performance may not have been optimized. This software 
 * module is an implementation of font compression technology as specified 
 * by the ISO/IEC 14996-18 standard.
 * 
 * ISO/IEC gives users a free copyright license to this software module or
 * modifications thereof for use in products claiming conformance to 
 * ISO/IEC 14996-18 International Standard. ISO/IEC gives users the same free
 * license to this software module or modifications thereof for research 
 * purposes and further ISO/IEC standardization.
 * 
 * Those intending to use this software module in products are advised that
 * its use may infringe existing patents and that this license does not grant 
 * rights under any patents. ISO/IEC have no liability for use of this 
 * software module or modifications thereof.
 * 
 * No license is granted and copyright is not released for products that do 
 * not conform to ISO/IEC 14996-18 International Standard.
 * 
 * Agfa Monotype Corporation retains full right to modify and use the code for
 * its own purpose, assign or donate the code to a third party and to inhibit 
 * third parties from using the code for products that do not conform to
 * audiovisual and image-coding related ITU recommendations and 
 * ISO/IEC International Standards. All rights not expressly granted to the 
 * software module or modifications thereof are reserved by 
 * Agfa Monotype Corporation.
 * 
 * This copyright notice must be included in all copies or derivative works.
 * Copyright (c) ISO/IEC 2003. All rights reserved.
 ******************************************************************************/

/*
 * File:								TTF_CONV.H
 * Revision history
 * $Log: TTF_CONV.H,v $
 * Revision 1.1  2004/02/10 22:01:10  vladlev
 * Initial version
 *
 */
#include "MEMORY.H"

#ifdef __cplusplus
extern "C" {
#endif

typedef struct {
	BYTE *ctf1; /* The ctf data  */
	LONG ctfSize1;
} ctfDescriptor;


typedef struct {
	CHAR *ttf;				/* The TrueType data */
	LONG ttfSize;			/* The TrueType size */

	CHAR *tableStart;		/* The start of the input table */
	CHAR *tableEnd;			/* Points at the first byte past the input table */
	
	ctfDescriptor ctfData;	/* ctf data descriptor */
	
	SHORT convertFormat;
	SHORT indexToLocFormat;
	
	MTX_MemHandler *mem;
} TTF_Converter;

/* public interface routines */

/* It asssumes that spline does point at the beginning of glyph data (IsSpline() == true). */
/* Expands a glyph in the 'glyf' table from the CTF format to the native TTF format on the fly. */
CHAR *MTX_TTC_GetTTFGlyphMemory( MTX_MemHandler *mem, CHAR *spline, LONG oldLength, LONG *newLength);
		
/* Converts the TTF data to CTF data. */
BYTE *MTX_TTC_TTF_To_CTF(	TTF_Converter *t, BYTE *ttfData, LONG ttfSize, LONG *size1 );
/* Converts the CTF data to TTF data. */
BYTE *MTX_TTC_CTF_To_TTF(	TTF_Converter *t, BYTE *ctf1, LONG size1, LONG *ttSize);
/* Constructor */
TTF_Converter *MTX_TTC_Create( MTX_MemHandler *mem, SHORT version );
/* Destructor */
void MTX_TTC__Destroy( TTF_Converter *t );

/* Returns true if they are equal. */
int MTX_TTF_GlyphsAreEqual( MTX_MemHandler *mem, CHAR *spline1, LONG len1, CHAR *spline2, LONG len2 );


/* Initializes the relative coordinate encoding technique used */
/* to store the coordinates in the CTF format. */
void MTX_TTC_InitCoordEncoding( void );
int CheckForRepeatedTable( CHAR *sfnt0, CHAR *sfnt1, int matchIndex, int matchTTC );


#ifdef BIG_ENDIAN

#define SWAPENDS_16( w ) (w)
#define SWAPENDS_U16( w ) (w)
#define SWAPENDS_32( w ) (w)
#define SWAPENDS_U32( w ) (w)

#else

SHORT SWAPENDS_16( register SHORT w );
USHORT SWAPENDS_U16( register USHORT w );
LONG SWAPENDS_32( register LONG ww );
ULONG SWAPENDS_U32( register ULONG ww );

#endif /* BIG_ENDIAN */

SHORT READWORD_INC( CHAR * *pRef );
ULONG READLONG_INC( BYTE * *pRef );
void WRITEWORD_INC( CHAR * *pRef, SHORT x );
void WRITELONG_INC( CHAR * *p, LONG x );

#ifdef __cplusplus
}
#endif
