/******************************************************************************
 * This software module was originally developed by Agfa Monotype Corporation
 * in the course of development of the ISO/IEC 14996-18 standard for reference 
 * purposes and its performance may not have been optimized. This software 
 * module is an implementation of font compression technology as specified 
 * by the ISO/IEC 14996-18 standard.
 * 
 * ISO/IEC gives users a free copyright license to this software module or
 * modifications thereof for use in products claiming conformance to 
 * ISO/IEC 14996-18 International Standard. ISO/IEC gives users the same free
 * license to this software module or modifications thereof for research 
 * purposes and further ISO/IEC standardization.
 * 
 * Those intending to use this software module in products are advised that
 * its use may infringe existing patents and that this license does not grant 
 * rights under any patents. ISO/IEC have no liability for use of this 
 * software module or modifications thereof.
 * 
 * No license is granted and copyright is not released for products that do 
 * not conform to ISO/IEC 14996-18 International Standard.
 * 
 * Agfa Monotype Corporation retains full right to modify and use the code for
 * its own purpose, assign or donate the code to a third party and to inhibit 
 * third parties from using the code for products that do not conform to
 * audiovisual and image-coding related ITU recommendations and 
 * ISO/IEC International Standards. All rights not expressly granted to the 
 * software module or modifications thereof are reserved by 
 * Agfa Monotype Corporation.
 * 
 * This copyright notice must be included in all copies or derivative works.
 * Copyright (c) ISO/IEC 2003. All rights reserved.
 ******************************************************************************/

/*
 * File:						fixed.h
 * Revision history
 * $Log: fixed.h,v $
 * Revision 1.1  2004/03/04 21:10:45  vladlev
 * IType font engine API - Initial version
 *
 *
 */

#ifndef __FIXED__
#define __FIXED__

/* AMT supplies assembly code for MULDIV, VARMUL and VARDIV 
* for the Microsoft Visual C compiler on x86 processors
*/
#if defined(_WIN32) && defined(_M_IX86) && !defined  (__MARM_ARMI__)
#define HAS_ASM
#endif

#ifdef __cplusplus
extern "C" {
#endif


#ifndef LO_WORD
#define HI_WORD(x) ((ULONG)(x) >> 16)
#define LO_WORD(x) ((x) & 0xFFFF)
#endif

#define FIXED_ONE (1L << 16)
#define FRACT_ONE (1L << 30)
#define SHORTFRACT_ONE (1 << 14)

#define FIXED_ONEHALF (1L << 15)
#define FIXED_ONEFOURTH (1L << 14)
#define FRACT_ONEHALF (1L << 29)
#define SHORTFRACT_ONEHALF (1 << 13)

#define MYINFINITY  2147483647L


/* smallest half integer >= y */
#define ABOVE(y) (((y + FIXED_ONEHALF - 1) & (-FIXED_ONE)) + FIXED_ONEHALF)

/* largest half integer < y */
#define BELOW(y) (((y - FIXED_ONEHALF - 1) & (-FIXED_ONE)) + FIXED_ONEHALF)

#define FIXED_TRUNC(x)	(SHORT)((x)>>16)
#define FIXED_ROUND(x)	FIXED_TRUNC((x)+FIXED_ONEHALF)

/* 64 bit integers in 32 bit ANSI C*/
LONG varmul(LONG a, LONG b, int n);		/* (a*b)>>n */
LONG vardiv(LONG a, LONG b, int n);		/* (a<<n)/b */
LONG muldiv(LONG a, LONG b, LONG c);	/* (a*b)/c */

/* 64 bit integers with compiler support */
#ifdef HAS_FS_INT64
LONG varmul_64(LONG a, LONG b, int n);
LONG vardiv_64(LONG a, LONG b, int n);
LONG muldiv_64(LONG a, LONG b, LONG c);
#endif

/* user supplied assembly code */
#ifdef HAS_ASM
LONG varmul_asm(LONG a, LONG b, int n);
LONG vardiv_asm(LONG a, LONG b, int n);
LONG muldiv_asm(LONG a, LONG b, LONG c);
#endif

/* normalize a fixed point vector */
VOID fixed_norm(FIXED_VECTOR *p);

/* sqrt of 2.30 number */
FRACT FracSqrt(FRACT x);				
#define ShortFracDot(a,b) (ShortFrac)(((LONG)a*b)>>14)

/* select the proper variant of MULDIV, VARMUL, and VARDIV */
#if SIZEOF_LONG==8
/* first: native 64 bit ints */
#define VarMul(a,b,c) varmul_64(a,b,c)
#define VarDiv(a,b,c) vardiv_64(a,b,c)
#define FixMul(a,b) (FS_FIXED)varmul_64(a,b,16)
#define FixDiv(a,b) (FS_FIXED)vardiv_64(a,b,16)
#define Mul26Dot6(a,b) (F26DOT6)varmul_64(a,b,6)
#define Div26Dot6(a,b) (F26DOT6)vardiv_64(a,b,6)
#define LongMulDiv(a,b,c) muldiv_64(a,b,c)
#define ShortMulDiv(a,b,c) muldiv_64(a,(LONG)b,(LONG)c)
#define FracMul(a,b) (FRACT)varmul_64(a,b,30)
#define FracDiv(a,b) (FRACT)vardiv_64(a,b,30)
#define ShortFracMul(a,b) (F26DOT6)varmul_64(a,(LONG)b,14)

#elif defined(HAS_ASM)
/* second: user supplied assembly code */
#define VarMul(a,b,c) varmul_asm(a,b,c)
#define VarDiv(a,b,c) vardiv_asm(a,b,c)
#define FixMul(a,b) (FS_FIXED)varmul_asm(a,b,16)
#define FixDiv(a,b) (FS_FIXED)vardiv_asm(a,b,16)
#define Mul26Dot6(a,b) (F26DOT6)varmul_asm(a,b,6)
#define Div26Dot6(a,b) (F26DOT6)vardiv_asm(a,b,6)
#define LongMulDiv(a,b,c) muldiv_asm(a,b,c)
#define ShortMulDiv(a,b,c) muldiv_asm(a,(LONG)b,(LONG)c)
#define FracMul(a,b) (FRACT)varmul_asm(a,b,30)
#define FracDiv(a,b) (FRACT)vardiv_asm(a,b,30)
#define ShortFracMul(a,b) (F26DOT6)varmul_asm(a,(LONG)b,14)

#elif defined(HAS_FS_INT64)
/* third: compiler supported 64 bit integers */
#define VarMul(a,b,c) varmul_64(a,b,c)
#define VarDiv(a,b,c) vardiv_64(a,b,c)
#define FixMul(a,b) (FS_FIXED)varmul_64(a,b,16)
#define FixDiv(a,b) (FS_FIXED)vardiv_64(a,b,16)
#define Mul26Dot6(a,b) (F26DOT6)varmul_64(a,b,6)
#define Div26Dot6(a,b) (F26DOT6)vardiv_64(a,b,6)
#define LongMulDiv(a,b,c) muldiv_64(a,b,c)
#define ShortMulDiv(a,b,c) muldiv_64(a,(LONG)b,(LONG)c)
#define FracMul(a,b) (FRACT)varmul_64(a,b,30)
#define FracDiv(a,b) (FRACT)vardiv_64(a,b,30)
#define ShortFracMul(a,b) (F26DOT6)varmul_64(a,(LONG)b,14)

#else
/* otherwise 32 bit ANSI C */
#define VarMul(a,b,c) varmul(a,b,c)
#define VarDiv(a,b,c) vardiv(a,b,c)
#define FixMul(a,b) (FS_FIXED)varmul(a,b,16)
#define FixDiv(a,b) (FS_FIXED)vardiv(a,b,16)
#define Mul26Dot6(a,b) (F26DOT6)varmul(a,b,6)
#define Div26Dot6(a,b) (F26DOT6)vardiv(a,b,6)
#define LongMulDiv(a,b,c) muldiv(a,b,c)
#define ShortMulDiv(a,b,c) muldiv(a,(LONG)b,(LONG)c)
#define FracMul(a,b) (FRACT)varmul(a,b,30)
#define FracDiv(a,b) (FRACT)vardiv(a,b,30)
#define ShortFracMul(a,b) (F26DOT6)varmul(a,(LONG)b,14)
#endif /* SIZEOF_LONG */

#ifdef __cplusplus
	}
#endif


#endif /* __FIXED__ */
