/******************************************************************************
 * This software module was originally developed by Agfa Monotype Corporation
 * in the course of development of the ISO/IEC 14996-18 standard for reference 
 * purposes and its performance may not have been optimized. This software 
 * module is an implementation of font compression technology as specified 
 * by the ISO/IEC 14996-18 standard.
 * 
 * ISO/IEC gives users a free copyright license to this software module or
 * modifications thereof for use in products claiming conformance to 
 * ISO/IEC 14996-18 International Standard. ISO/IEC gives users the same free
 * license to this software module or modifications thereof for research 
 * purposes and further ISO/IEC standardization.
 * 
 * Those intending to use this software module in products are advised that
 * its use may infringe existing patents and that this license does not grant 
 * rights under any patents. ISO/IEC have no liability for use of this 
 * software module or modifications thereof.
 * 
 * No license is granted and copyright is not released for products that do 
 * not conform to ISO/IEC 14996-18 International Standard.
 * 
 * Agfa Monotype Corporation retains full right to modify and use the code for
 * its own purpose, assign or donate the code to a third party and to inhibit 
 * third parties from using the code for products that do not conform to
 * audiovisual and image-coding related ITU recommendations and 
 * ISO/IEC International Standards. All rights not expressly granted to the 
 * software module or modifications thereof are reserved by 
 * Agfa Monotype Corporation.
 * 
 * This copyright notice must be included in all copies or derivative works.
 * Copyright (c) ISO/IEC 2003. All rights reserved.
 ******************************************************************************/

/*
 * File:						makeMP4file.c
 * Revision history
 * $Log: makeMP4file.c,v $
 * Revision 1.2  2004/02/19 19:36:42  vladlev
 * Added new functionality to write uncompressed OpenType TrueType/CFF
 * files in output MP4 file.
 *
 * Revision 1.1  2004/02/10 22:01:11  vladlev
 * Initial version
 *
 */

#include "MP4Movies.h"

extern char fontName[];

static u8 fontDecoderSpecificInfo[128];
static u8 DSIfontFormat;
static u8 *fontDataMem;
static u32 fontDataSize;


void setFontDataPtr (u8 *mem, u32 size, u8 DSIformat)
{
	fontDataMem = mem;
	fontDataSize = size;
	DSIfontFormat = DSIformat;
}

MP4Err createDecoderSpecificInfo (u32 *decoderSpecificInfoSize)
{
	MP4Err err;
	u8 fontNameLength;
	u8 *p = fontDecoderSpecificInfo;

	err = MP4NoErr;
	*p = (DSIfontFormat << 1); p++; /* font format 0x02 - compressed TTF, no font storage */
	fontNameLength = strlen (fontName);
	if (fontNameLength > 124)
		/* it's VERY inlikely to occur, but if it is, */
		/* let's clip it so that we stay within the limit */
		/* of allocated buffer size (124 + 4 = 128) */
		fontNameLength = 124;
	*p = fontNameLength; p++;
	strncpy (p, fontName, fontNameLength);
	p += fontNameLength;
	*p = 0x01; p++;/* fontSubsetID = 0, reserved = 1 */
	*decoderSpecificInfoSize = p - fontDecoderSpecificInfo;
	return err;
}

void dumpDecoderSpecificInfo (u32 decoderSpecificInfoSize)
{
	u8 fontNameLength;
	u8 font_name[128];
	u8 *p = fontDecoderSpecificInfo;

	printf ("\nDecoderSpecificInfo:\n");
	printf ("fontFormat(bit(7)) = 0x%x", (*p >> 1));
	switch (*p >> 1) {
	case 0:
		printf (" - Forbidden!!!\n");
		break;
	case 1:
		printf (" - Uncompressed OpenType TTF\n");
		break;
	case 2:
		printf (" - Compressed OpenType TTF\n");
		break;
	case 3:
		printf (" - OpenType CFF\n");
		break;
	default:
		printf (" - Undefined\n");
	}
	printf ("fontStore(bit(1))  = %x\n", (*p & 0x01)); p++;
	fontNameLength = *p++;
	printf ("fontNameLength(bit(8)) = %d\n", fontNameLength);
	strncpy (font_name, p, fontNameLength);
	p += fontNameLength;
	font_name[fontNameLength] = 0;
	printf ("fontName = <%s>\n", font_name);
	printf ("fontSubsetID(bit(7)) = %d\n", (*p >> 1));
	printf ("reserved(bit(1)) = %d\n", (*p & 0x01)); p++;
	printf ("decoderSpecificInfoSize=%d", decoderSpecificInfoSize);
	if (decoderSpecificInfoSize == (u32)(p - fontDecoderSpecificInfo))
		printf (" and matched!\n");
	else printf (" doesn't match!\n");
}

MP4Err getDecoderSpecificInfo( MP4Handle decoderSpecificInfoH )
{
	MP4Err err;
	u8 *p;
	u32 i, frameLength;
	
	err = MP4NoErr;

	err = createDecoderSpecificInfo (&frameLength); if (err) goto bail;
	dumpDecoderSpecificInfo (frameLength); /* for debug */

	err = MP4SetHandleSize( decoderSpecificInfoH, frameLength+2 ); if (err) goto bail;

	p = (u8*) *decoderSpecificInfoH;
	*p++ = 0x05; /* DecSpecificInfoTag, 14496-1, Sec 8.2.2.2, Table 1*/
	*p++ = frameLength; /* sizeOfInstance, 14496-1, Sec 12.3.3 */
	for (i=0; i<frameLength; i++)
		*p++ = fontDecoderSpecificInfo[i];
bail:
	return err;
}


MP4Err addFontData ( MP4Track trak, MP4Media media )
{
	MP4Err err;
	MP4Handle sampleEntryH;
	MP4Handle sampleDataH;
	MP4Handle sampleDurationH;
	MP4Handle sampleSizeH;
	MP4Handle decoderSpecificInfoH;
 	u32 objectTypeIndication;
	u32 streamType;
	u32 decoderBufferSize;
	u32 maxBitrate;
	u32 avgBitrate;
	u8 *p;
	
	err = MP4NoErr;
	err = MP4SetMediaLanguage( media, "und" ); if (err) goto bail;

	objectTypeIndication = 0x06; /* mpeg-4 font data */
	streamType = 0x0C; /* Font Data stream */
	decoderBufferSize = fontDataSize; /* size of a font AU */
	maxBitrate = 128000;
	avgBitrate = 128000;
	err = MP4NewHandle( 0, &decoderSpecificInfoH ); if (err) goto bail;
	err = getDecoderSpecificInfo( decoderSpecificInfoH ); if (err) goto bail;

	err = MP4NewHandle( 0, &sampleEntryH ); if (err) goto bail;
	err = MP4NewSampleDescription( trak, sampleEntryH,
		1,
		objectTypeIndication,
		streamType,
		decoderBufferSize,
		maxBitrate,
		avgBitrate,
		decoderSpecificInfoH ); if (err) goto bail;

	err = MP4NewHandle( sizeof(u32), &sampleDurationH ); if (err) goto bail;
	*((u32*) *sampleDurationH) = 1000; /* 10 sec? */

	err = MP4NewHandle( 0, &sampleDataH ); if (err) goto bail;

	err = MP4NewHandle( sizeof(u32), &sampleSizeH ); if (err) goto bail;

	err = MP4SetHandleSize( sampleDataH, fontDataSize ); if (err) goto bail;
	p = (u8*) *sampleDataH;
	memcpy ( p, fontDataMem, fontDataSize);

	err = MP4GetHandleSize( sampleDataH, (u32*) *sampleSizeH ); if (err) goto bail;
	
	err = MP4AddMediaSamples( media, sampleDataH, 1, sampleDurationH, sampleSizeH, sampleEntryH, NULL, NULL ); if (err) goto bail;
	if ( sampleEntryH )
	{
		err = MP4DisposeHandle( sampleEntryH ); if (err) goto bail;
		sampleEntryH = NULL;
	}	

bail:
	return err;
}


MP4Err createMP4file( char *filename )
{
	MP4Err err;
	MP4Movie moov;
	MP4Track trak;
	MP4Media media;
	u8 OD_profileAndLevel;
	u8 scene_profileAndLevel;
	u8 audio_profileAndLevel;
	u8 visual_profileAndLevel;
	u8 graphics_profileAndLevel;
	u32 initialObjectDescriptorID;
	u32 timeScale;
	u64 mediaDuration;
	
	err = MP4NoErr;
	initialObjectDescriptorID = 1;
	OD_profileAndLevel = 0xff; /* none required */
	scene_profileAndLevel = 0xff; /* none required */
	audio_profileAndLevel = 0xff; /* none required */
	visual_profileAndLevel = 0xff; /* none required */
	graphics_profileAndLevel = 0xff; /* none required */
	timeScale = 10; /* 1/10 of a second is precise enough for a font lifetime */

	err = MP4NewMovie( &moov,
		initialObjectDescriptorID, 
		OD_profileAndLevel,
		scene_profileAndLevel,
		audio_profileAndLevel,
		visual_profileAndLevel,
		graphics_profileAndLevel ); if (err) goto bail;

	err = MP4NewMovieTrack( moov, 0, &trak ); if (err) goto bail;

	if (initialObjectDescriptorID != 0) {
		err = MP4AddTrackToMovieIOD( trak ); if (err) goto bail;
	}
	
	err = MP4NewTrackMedia( trak, 
		&media, 
		MP4SceneDescriptionHandlerType, /* we  need to define new font handler type ! */
		timeScale, NULL ); if (err) goto bail;

	err = MP4BeginMediaEdits( media ); if (err) goto bail;
	
	err = addFontData( trak, media ); if (err) goto bail;
	
	err = MP4EndMediaEdits( media ); if (err) goto bail;

	err = MP4GetMediaDuration( media, &mediaDuration ); if (err) goto bail;

	err = MP4InsertMediaIntoTrack( trak, 0, 0, mediaDuration, 1 ); if (err) goto bail;

	err = MP4WriteMovieToFile( moov, filename ); if (err) goto bail;

	err = MP4DisposeMovie( moov );
bail:
	return err;
}

