/******************************************************************************
 * This software module was originally developed by Agfa Monotype Corporation
 * in the course of development of the ISO/IEC 14996-18 standard for reference 
 * purposes and its performance may not have been optimized. This software 
 * module is an implementation of font compression technology as specified 
 * by the ISO/IEC 14996-18 standard.
 * 
 * ISO/IEC gives users a free copyright license to this software module or
 * modifications thereof for use in products claiming conformance to 
 * ISO/IEC 14996-18 International Standard. ISO/IEC gives users the same free
 * license to this software module or modifications thereof for research 
 * purposes and further ISO/IEC standardization.
 * 
 * Those intending to use this software module in products are advised that
 * its use may infringe existing patents and that this license does not grant 
 * rights under any patents. ISO/IEC have no liability for use of this 
 * software module or modifications thereof.
 * 
 * No license is granted and copyright is not released for products that do 
 * not conform to ISO/IEC 14996-18 International Standard.
 * 
 * Agfa Monotype Corporation retains full right to modify and use the code for
 * its own purpose, assign or donate the code to a third party and to inhibit 
 * third parties from using the code for products that do not conform to
 * audiovisual and image-coding related ITU recommendations and 
 * ISO/IEC International Standards. All rights not expressly granted to the 
 * software module or modifications thereof are reserved by 
 * Agfa Monotype Corporation.
 * 
 * This copyright notice must be included in all copies or derivative works.
 * Copyright (c) ISO/IEC 2003. All rights reserved.
 ******************************************************************************/

/*
 * File:						object.h
 * Revision history
 * $Log: object.h,v $
 * Revision 1.1  2004/03/04 21:10:45  vladlev
 * IType font engine API - Initial version
 *
 *
 */

#ifndef __OBJECTS__
#define __OBJECTS__

#include "port.h"

#include "bitio.h"

#include <stdio.h>		/* FILE, fopen() ... fclose() */
#include <stdlib.h>		/* malloc(), free(), realloc() */
#include <string.h>		/* memcpy(), memset(), and strcmp() */

typedef LONG FS_FIXED;			/* as 16.16 */
typedef LONG FRACT;				/* as 2.30 */
typedef LONG F26DOT6;			/* as 26.6 */
typedef SHORT SHORTFRACT;		/* as 2.14 */
typedef BYTE BOOLEAN;



/* 2.14 UNIT VECTOR */
typedef struct {
	SHORTFRACT x,y;
	} SHORT_UNIT_VECTOR;

/* 2.30 UNIT VECTOR */
typedef struct {
	FRACT x,y;
	} UNIT_VECTOR;

typedef struct {
	F26DOT6 x,y;
	} F26DOT6_VECTOR;

typedef struct {
	FS_FIXED x,y;
	} FIXED_VECTOR;

typedef struct {
	FS_FIXED        transform[3][3];
	} transMatrix;

/* some useful macros -- watch out for side effects */
#ifndef ABS
#define ABS(x)      (((x)>0)?(x):-(x))
#endif

#ifndef MAX
#define MAX(a,b)    (((a)>(b))?(a):(b))
#endif

#ifndef MIN
#define MIN(a,b)    (((a)<(b))?(a):(b))
#endif

/* byte swapping macros  */
#if BYTE_ORDER == LITTLE_ENDIAN
#define NEED_TO_SWAP 1
#define SWAPW(x) swapw(x)
#define SWAPL(x) swapl(x)

#else /* BYTEORDER==BIG_ENDIAN */
#define NEED_TO_SWAP 0
#define SWAPW(x) (x)
#define SWAPL(x) (x)
#endif /* BYTEORDER */


/* move <p> to the next possible location of a (VOID *) */
#define FS_ALIGN(p) p += (sizeof(VOID *) - 1); p &= ~(sizeof(VOID *) - 1)

#ifdef INT_MEM
typedef struct mem {
#ifdef FS_DEBUG
LONG tag;
#endif /* FS_DEBUG */
struct mem *next;
LONG units;
	} MEM;
#endif /* INT_MEM */




/* outline element types */
#define FS_MOVETO 0
#define FS_LINETO 1
#define FS_QUADTO 2
#define FS_CUBETO 3

#ifdef STATS
extern ULONG made_outl;
extern ULONG cached_outl;
extern ULONG made_bmap;
extern ULONG cached_bmap;
extern ULONG made_gmap;
extern ULONG embedded_bmap;
extern ULONG cached_gmap;
extern ULONG uncached_too_big;
#endif

/********************* rasterization ****************************/
/* transition array node */
typedef struct {
	FS_FIXED value;		/* coordinate of this node in singley linked list */
	LONG next;			/* index of next T_NODE in the singley linked list */
	} TNODE;

/* transition array */
typedef struct { 
	SHORT max;			/* y coordinate of slot[0] */
	SHORT num;			/* number of slots */
	LONG stop_index;	/* 1+maximum index into <nodes> */
	LONG next_index;	/* next available index into <nodes> */
	LONG *indices;		/* starting index into <nodes> for each coordinate */
	TNODE *nodes;		/* the singly linked list of values */
	} TLIST;




/************* TrueType Fonts ***********************************/
#define STUBCONTROL 0x10000L
#define NODOCONTROL 0x20000L

/* unpacking constants */
#define ONCURVE  			0x01
#define XSHORT   			0x02
#define YSHORT   			0x04
#define REPEAT_FLAGS    	0x08
#define SHORT_X_IS_POS   	0x10  /* flags & XSHORT */
#define NEXT_X_IS_ZERO   	0x10
#define SHORT_Y_IS_POS   	0x20  /* flags & YSHORT */
#define NEXT_Y_IS_ZERO   	0x20


/* composite glyph constants */
#define ARG_1_AND_2_ARE_WORDS		0x0001	/* if set args are words otherwise they are bytes */
#define ARGS_ARE_XY_VALUES			0x0002	/* if set args are xy values, otherwise they are points */
#define ROUND_XY_TO_GRID			0x0004	/* for the xy values if above is true */
#define WE_HAVE_A_SCALE				0x0008	/* Sx = Sy, otherwise scale == 1.0 */
#define NON_OVERLAPPING				0x0010	/* set to same value for all components */
#define MORE_COMPONENTS				0x0020	/* indicates at least one more glyph after this one */
#define WE_HAVE_AN_X_AND_Y_SCALE	0x0040	/* Sx, Sy */
#define WE_HAVE_A_TWO_BY_TWO		0x0080	/* t00, t01, t10, t11 */
#define WE_HAVE_INSTRUCTIONS		0x0100	/* instructions follow */
#define USE_MY_METRICS				0x0200	/* apply these metrics to parent glyph */

#define TTF_MAGIC 0x5F0F3CF5L

/* table tags in Motorola Byte order */
#define MAKE_TAG(a,b,c,d)	(((ULONG)a<<24)|((ULONG)b<<16)|((ULONG)c<<8)|d)

/* if your compiler is decent these will become simple 32 bit constants */
#define TAG_ttcf	MAKE_TAG('t','t','c','f')
#define TAG_head	MAKE_TAG('h','e','a','d')
#define TAG_hhea	MAKE_TAG('h','h','e','a')
#define TAG_hmtx	MAKE_TAG('h','m','t','x')
#define TAG_maxp	MAKE_TAG('m','a','x','p')
#define TAG_loca	MAKE_TAG('l','o','c','a')
#define TAG_glyf	MAKE_TAG('g','l','y','f')
#define TAG_cmap	MAKE_TAG('c','m','a','p')
#define TAG_cvt		MAKE_TAG('c','v','t',' ')
#define TAG_fpgm	MAKE_TAG('f','p','g','m')
#define TAG_prep	MAKE_TAG('p','r','e','p')
#define TAG_EBLC	MAKE_TAG('E','B','L','C')
#define TAG_EBDT	MAKE_TAG('E','B','D','T')
#define TAG_kern	MAKE_TAG('k','e','r','n')
#define TAG_act3	MAKE_TAG('a','c','t','3')	
#define TAG_nstk	MAKE_TAG('n','s','t','k')	

/* additional tags for inquiry fn */
#define TAG_name	MAKE_TAG('n','a','m','e')
#define TAG_post	MAKE_TAG('p','o','s','t')
#define TAG_OS2		MAKE_TAG('O','S','/','2')
#define TAG_EBSC	MAKE_TAG('E','B','S','C')
#define TAG_gasp	MAKE_TAG('g','a','s','p')
#define TAG_hdmx	MAKE_TAG('h','d','m','x')
#define TAG_LTSH	MAKE_TAG('L','T','S','H')
#define TAG_PCLT	MAKE_TAG('P','C','L','T')
#define TAG_VDMX	MAKE_TAG('V','D','M','X')
#define TAG_vhea	MAKE_TAG('v','h','e','a')
#define TAG_vmtx	MAKE_TAG('v','m','t','x')

#define TAG_GSUB	MAKE_TAG('G','S','U','B')
#define TAG_GPOS	MAKE_TAG('G','P','O','S')
#define TAG_BASE	MAKE_TAG('B','A','S','E')
#define TAG_JSTF	MAKE_TAG('J','S','T','F')
#define TAG_GDEF	MAKE_TAG('G','D','E','F')

typedef struct {
	ULONG			tag;			/* eg: 'cmap' */
	ULONG			checkSum;		/* for table */
	ULONG			offset;			/* offset from beginning of font */
	ULONG			length;			/* length (bytes) */
	} TTF_DIR;

typedef struct {
	FS_FIXED		version;		/* 0x10000 (1.0) */
	USHORT			numTables;		/* number of tables */
	USHORT			searchRange;	/* (max2 <= numOffsets)*16 */
	USHORT			entrySelector;	/* log2(max2 <= numOffsets) */
	USHORT			rangeShift;		/* numOffsets*16-searchRange*/
	TTF_DIR			tables[1];		/* table[numOffsets] */
	} TTF_HEADER;

typedef struct {
	ULONG			tag;			/* should be 'ttcf' */
	FS_FIXED		version;		/* 0x10000 (1.0) */
	ULONG			numFonts;		/* number of fonts in TTC file */
	ULONG			offsets[1]; 	/* offsets from beginning of file to table directories */
	} TTC_HEADER;

typedef struct {
	FS_FIXED	version;			/* for this table, set to 1.0 */
	FS_FIXED	fontRevision;		/* For Font Manufacturer */
	ULONG		checkSumAdjustment;
	ULONG		magicNumber; 		/* signature, should always be 0x5F0F3CF5  == MAGIC */
	USHORT		flags;
	USHORT		unitsPerEm;			/* Specifies how many in Font Units we have per EM */
	ULONG		created[2];
	ULONG		modified[2];
	SHORT		xMin;				/* fontwide bbox */
	SHORT		yMin;
	SHORT		xMax;
	SHORT		yMax;
	USHORT		macStyle;			/* macintosh style word */
	USHORT		lowestRecPPEM; 		/* lowest recommended pixels per Em */
	SHORT		fontDirectionHint;	/* 0==mixed, 1==L-R or T-B, -1==R-L or B-T , 2==like 1 + neutrals, -2 like (-1) + neutrals */
	SHORT		indexToLocFormat;	/* 0== short, 1==long */
	SHORT		glyphDataFormat;	/* better be 0 */
	} TTF_HEAD;

typedef struct {
	FS_FIXED	version;				/* for this table, set to 1.0 */
	SHORT		yAscender;
	SHORT		yDescender;
	SHORT		yLineGap;				/* Recommended linespacing = ascender - descender + linegap */
	USHORT		advanceWidthMax;	
	SHORT		minLeftSideBearing;
	SHORT		minRightSideBearing;
	SHORT		xMaxExtent;				/* Max of ( LSBi + (XMAXi - XMINi) ), i loops through all glyphs */
	SHORT		horizontalCaretSlopeNumerator;
	SHORT		horizontalCaretSlopeDenominator;
	SHORT		caretOffset;
	USHORT		reserved1;
	USHORT		reserved2;
	USHORT		reserved3;
	USHORT		reserved4;
	SHORT		metricDataFormat;			/* set to 0 for current format */
	USHORT		numberOf_LongHorMetrics;	/* if format == 0 */
	} TTF_HHEA;

typedef struct {
	FS_FIXED	version;				/* for this table, set to 1.0 */
	USHORT		numGlyphs;				/* number of glyphs in the file ... loca is one larger */
	USHORT		maxPoints;				/* in an individual glyph */
	USHORT		maxContours;			/* in an individual glyph */
	USHORT		maxCompositePoints;		/* in an composite glyph */
	USHORT		maxCompositeContours;	/* in an composite glyph */
	USHORT		maxElements;			/* set to 2, or 1 if no twilightzone points */
	USHORT		maxTwilightPoints;		/* max points in element zero */
	USHORT		maxStorage;				/* max number of storage locations */
	USHORT		maxFunctionDefs;		/* max number of FDEFs in any preprogram */
	USHORT		maxInstructionDefs;		/* max number of IDEFs in any preprogram */
	USHORT		maxStackElements;		/* max number of stack elements for any individual glyph */
	USHORT		maxSizeOfInstructions;	/* max size in bytes for any individual glyph */
	USHORT		maxComponentElements;	/* number of glyphs referenced at top level */
	USHORT		maxComponentDepth;		/* levels of recursion, 1 for simple components */
	} TTF_MAXP;

/* buffer size limits for TTF_CMAP and TTF_NAME structs */
#define MAX_MAPPINGS_LEN			20
#define MAX_FONT_NAME_LEN			50
#define MAX_FONT_FAMILY_NAME_LEN	50
#define MAX_COPYRIGHT_LEN			200

typedef struct {
	USHORT platform;
	USHORT encoding;
	ULONG offset;
	} CMAP_TAB;	/* extracted from CMAP.C */

typedef struct {
	USHORT numEncodingTables;
	CMAP_TAB mappings[MAX_MAPPINGS_LEN];
	BOOLEAN mappings_too_long;
	} TTF_CMAP;		/* abbreviated contents */

/* "name" table offsets */
#define NAME_NUM_REC		2
#define NAME_OFF_TO_STRS	4
#define NAME_NAMERECS		6
#define NAME_SIZE_NAMEREC	12
#define NAME_TAB_PLATID		0
#define NAME_TAB_SPECID		2
#define NAME_TAB_LANGID		4
#define NAME_TAB_NAMEID		6
#define NAME_TAB_STRLEN		8
#define NAME_TAB_STROFF		10

#define MSFT_ENC           3    /* platform ID */
#define US_ENGL       0x0409    /* language ID in name table */

typedef enum {
	name_Copyright,
	name_Family,
	name_Subfamily,
	name_UniqueName,
	name_FullName,
	name_Version,
	name_Postscript,
	name_Trademark
} sfnt_NameIndex;

typedef struct {
	CHAR font_name[MAX_FONT_NAME_LEN];
	BOOLEAN font_name_too_long;
	CHAR font_family_name[MAX_FONT_FAMILY_NAME_LEN];
	BOOLEAN font_family_name_too_long;
	CHAR copyright[MAX_COPYRIGHT_LEN];
	BOOLEAN copyright_too_long;
	} TTF_NAME;		/* abbreviated contents */

typedef struct {
	USHORT version;
	SHORT xAvgCharWidth;
	USHORT usWeightClass;
	USHORT usWidthClass;
	SHORT fsType;
	SHORT ySubscriptXSize;
	SHORT ySubscriptYSize;
	SHORT ySubscriptXOffset;
	SHORT ySubscriptYOffset;
	SHORT ySuperscriptXSize;
	SHORT ySuperscriptYSize;
	SHORT ySuperscriptXOffset;
	SHORT ySuperscriptYOffset;
	SHORT yStrikeoutSize;
	SHORT yStrikeoutPosition;
	SHORT sFamilyClass;
	BYTE panose[10];
	ULONG ulUnicodeRange1;	/* Bits 0-31 */
	ULONG ulUnicodeRange2;	/* Bits 32-63 */
	ULONG ulUnicodeRange3;	/* Bits 64-95 */
	ULONG ulUnicodeRange4;	/* Bits 96-127 */
	CHAR achVendID[4];
	USHORT fsSelection;
	USHORT usFirstCharIndex;
	USHORT usLastCharIndex;
	SHORT sTypoAscender;
	SHORT sTypoDescender;
	SHORT sTypoLineGap;
	USHORT usWinAscent;
	USHORT usWinDescent;
	ULONG ulCodePageRange1;	/* Bits 0-31 */
	ULONG ulCodePageRange2;	/* Bits 32-63 */
	} TTF_OS2;

#ifdef CCC
typedef struct {
	SHORT numBits_LOCA;  /* number of bits used for LOCA */
	SHORT minVal_LSB, numBits_HMTX;	/* mimium value for LSB and number of bits for HMTX */
	ULONG numBits_GLYF;				/* number of parts for GLYF */
} CCC_INFO;
#endif


typedef struct {
	FILECHAR			*path;					/* if a disk file .. the path */
	BYTE				*memptr;				/* else ROM/RAM location */
	FILE				*fp;					/* fp = FS_open(path) for disk file */
	TTC_HEADER			*ttc_header;			/* NULL if vanilla TTF */
	ULONG				ttc_which;				/* current subfont of TTC */
	ULONG				ttf_offset;				/* offset to TTF in TTC */
	TTF_HEADER			*ttf_header;			/* for current (sub) font */
	TTF_HEAD			*head;					/* for current (sub) font */
	TTF_HHEA			*hhea;					/* for current (sub) font */
	TTF_MAXP			*maxp;					/* for current (sub) font */
	BYTE				*prep;					/* "prep" table */
	LONG				num_prep;
	BYTE				*fpgm;					/* "fpgm" table */
	LONG				num_fpgm;
	SHORT				*cvt;					/* "cvt" table */
	LONG				num_cvt;
	ULONG				*loca;					/* loca table converted to long format */
	ULONG				loca_offset;
	ULONG				glyf_size;
	ULONG				glyf_offset;			/* offset to "glyf" table */
	ULONG				eblc_offset;			/* offset to "EBLC" table */
	ULONG				kern_offset;			/* offset to 'kern' table */
	ULONG				hmtx_offset;			/* offset to "hmtx" table */

	TTF_OS2				*os2;					/* for current (sub) font */
	TTF_CMAP			*cmap;					/* for current (sub) font - (abbreviated) */
	TTF_NAME			*name;					/* for current (sub) font - (abbreviated) */
	
	VOID				*decomp;				/* (for ACT3 fonts) MTX_RA_TT_Decomp */
	USHORT				*nstk;					/* array of non-stik indices */
#ifdef CCC
	CCC_INFO		ccc_info;
#endif
	} TTF;


/************* loaded/available fonts ***************************/
typedef struct _lfnt {
	CHAR *name;							/* the proper name of the font */
	FILECHAR *path;						/* if on DISK ... the complete path name */
	BYTE *memptr;						/* else if in RAM/ROM ... the address */
	ULONG index;						/* index of <name> in a TT collection */
	TTF *ttf;							/* if loaded ... here's the essential data */
	USHORT platform;					/* platform for the cmap ...0xFFFF==none */
	USHORT encoding;					/* encoding for the cmap ...0xFFFF==none */
	BYTE *cmap;							/* copied cmap, for act and disk-based font */
	ULONG cmap_offset;					/* referenced cmap, for rom/ram-based non act font */
	struct _lfnt *prev,*next;			/* linked list goo */
	} LFNT;

/************* scaled font enviromnent **************************/
typedef struct {
	FS_FIXED user_scale[4];				/* scale matrix in 1/65536-the of PIXELS */
	USHORT xppm;						/* meaningful only for 'vanilla' characters */
	USHORT yppm;						/* used to match sizes for embedded bitmaps */ 
	USHORT lpm;							/* rotation invarient measure of character size */
	SHORT vanilla;						/* ? no rotation, skewing, or mirroring */
#ifdef STIK
	FS_FIXED stroke_pct;
	SHORT stroke_width;
	FS_FIXED cap_round;					/* for stik font autohinter */
	FS_FIXED cap_square;
	FS_FIXED x_round;
	FS_FIXED x_square;
	FS_FIXED base_round;
	FS_FIXED base_square;
#endif
#ifdef PSEUDO_BOLD
	FS_FIXED bold_pct;					/* percent of em to embolden */
	SHORT bold_width;
#endif
	VOID * ttkey;						/* the TT scale information */
	} SENV;

/************* scaled font instance *****************************/
typedef struct _sfnt {
	LFNT *lfnt;							/* the loaded font */
	SENV *senv;							/* scale enviromnent */
	ULONG cached_count;					/* number of things in cache */
	struct _sfnt *prev,*next;			/* linked list goo */
	} SFNT;

/*************** cache ******************************************/
#define CACHE_MOD 127
/* the largest 6 prime numbers < 32768  (Knuth V2 p390) */
#define CACHE_OUTLINE	(USHORT)(32768L - 19)
#define CACHE_GRAYMAP	(USHORT)(32768L - 49)
#define CACHE_BITMAP	(USHORT)(32768L - 51)
#define CACHE_PHASE0	(USHORT)(32768L - 55)
#define CACHE_PHASE1	(USHORT)(32768L - 61)
#define CACHE_PHASE2	(USHORT)(32768L - 75)
#define CACHE_PHASE3	(USHORT)(32768L - 81)

typedef struct _cache_entry {
	USHORT		type;		/* CACHE_OUTLINE, CACHE_BITMAP, CACHE_GRAYMAP */
	USHORT		index;		/* glyph index of the <data> */
	ULONG		flags;		/* user accessible flags */
	BYTE		*data;		/* the FS_OUTLINE*, FS_BITMAP*, or FS_GRAYMAP* */
	SFNT		*sfnt;		/* the scaled font it came from */
#ifdef STIK
	SHORT		stroke_width;
#endif /* STIK */

#ifdef PSEUDO_BOLD
	SHORT		bold_width;
#endif /* PSEUDO_BOLD */

	struct _cache_entry *prev,*next;
	ULONG ref_counter;
	} CACHE_ENTRY;


/************* the global state *********************************/
/* internal flags */
#define FLAGS_GRAY_LIMIT	(ULONG)0x00000001
#define FLAGS_STIK			(ULONG)0x00000002
#define FLAGS_ACT3			(ULONG)0x00000004
#define FLAGS_CCC			(ULONG)0x00000008
/* accessible interpreter flags */
#define FLAG_DO_DROPOUTS	(ULONG)0x00000100
#define FLAGS_DROPOUTS_ON	FLAG_DO_DROPOUTS
#define FLAGS_DROPOUTS_OFF  (ULONG)(~FLAGS_DROPOUTS_ON)
#define FLAGS_NO_HINTS		(ULONG)0x00000200
#define FLAGS_HINTS_OFF		FLAGS_NO_HINTS
#define FLAGS_HINTS_ON		(ULONG)(~FLAGS_HINTS_OFF)
/* accessible bitmap flags */
#define FLAGS_EMBOSSED		(ULONG)0x00000400
#define FLAGS_ENGRAVED		(ULONG)0x00000800
#define FLAGS_OUTLINED		(ULONG)0x00001000
#define FLAGS_NO_EFFECT		(ULONG)(~(FLAGS_EMBOSSED | FLAGS_ENGRAVED | FLAGS_OUTLINED))
/* accessible graymap flags */
#define FLAGS_SOFTENED      (ULONG)0x00002000
#define FLAGS_SOFTENED_ON   (ULONG)0x00002000
#define FLAGS_SOFTENED_OFF  (ULONG)(~FLAGS_SOFTENED_ON)
#define FLAGS_PHASED		(ULONG)0x00004000
#define FLAGS_PHASED_ON		FLAGS_PHASED
#define FLAGS_PHASED_OFF	(ULONG)(~FLAGS_PHASED_ON)
#define FLAGS_NO_GRAYMAP_EFFECT (ULONG)(~(FLAGS_SOFTENED_ON | FLAGS_PHASED_ON))
/* Major and Minor versions */
#define MAJOR_VERSION 200
#define MINOR_VERSION 100
/* accessible subpixel flags */
#define FLAGS_SUBPIXEL		(ULONG)0x00008000
#define FLAGS_SUBPIXEL_ON	FLAGS_SUBPIXEL
#define FLAGS_SUBPIXEL_OFF	(ULONG)(~FLAGS_SUBPIXEL_ON)

/* all user accessible flags */
#define FLAGS_FIX_DROPOUTS	  (ULONG)0x00000010
#define FLAGS_USER (FLAGS_EMBOSSED | FLAGS_ENGRAVED | FLAGS_OUTLINED \
	| FLAGS_SOFTENED_ON | FLAGS_PHASED_ON | FLAGS_SUBPIXEL_ON \
	| FLAGS_HINTS_OFF | FLAGS_DROPOUTS_ON) /* Fariborz */
/* different private values for ttf->head->glyphDataFormat */
#define STIK_FORMAT (SHORT)(0x9654)
#define STIK_FORMAT_CCC (SHORT)(0x0200)
#define TTF_FORMAT_CCC  (SHORT)(0x0400)

/* When we're processing a STIK font, we use bit 7 of the
* first TrueType glyph flag byte to say that this character
* REALLY is an outline character. It also appears in the byte
* FS_OUTLINE->type[0] with the same meaning */
#define OUTLINE_CHAR		0x80

/*** bitmaps -- 1 bit per pixel ***/
typedef struct {
	CACHE_ENTRY *cache_ptr;	
	LONG size;			/* allocated size in bytes */
	SHORT lo_x;			/* pixel coordinate of left column of pixels */
	SHORT hi_y;			/* pixel coordinate of top row of pixels */
	SHORT i_dx;			/* integer x increment to move character origin - 0 if rotated or skewed */
	SHORT i_dy;			/* integer y increment to move character origin - 0 if rotated or skewed */
	FS_FIXED dx;		/* x increment to move character origin */
	FS_FIXED dy;		/* y increment to move character origin */
	SHORT width;		/* width of the bitmap in pixels */
	SHORT height;		/* height of the bitmap in pixels */
	SHORT bpl;			/* bytes per line ... USUALLY (7+width)/8 ... but not always */
	BYTE  bits[1];		/* the bits in top to bottom, left to right order. 0==WHITE */
	/* note the actual size of the <bits> array is bpl*height */
	} FS_BITMAP;



/*** graymaps -- 4 pits per pixel ***/
typedef struct {
	CACHE_ENTRY *cache_ptr;	
	LONG  size;			/* allocated size in bytes */
	SHORT lo_x;			/* pixel coordinate of left column of pixels */
	SHORT hi_y;			/* pixel coordinate of top row of pixels */
	SHORT i_dx;			/* integer x increment to move character origin - 0 if rotated or skewed */
	SHORT i_dy;			/* integer y increment to move character origin - 0 if rotated or skewed */
	FS_FIXED dx;		/* hi resolution x increment to move character origin */
	FS_FIXED dy;		/* hi resolution y increment to move character origin */
	SHORT width;		/* width of the bitmap in pixels */
	SHORT height;		/* height of the bitmap in pixels */
	SHORT bpl;			/* bytes per line ... USUALLY (1+width)/2 ... but not always */
	BYTE  bits[1];		/* the bits in top to bottom, left to right order, 4 bits per pixel, 0==WHITE */
	/* note the actual size of the <bits> array is bpl*height */
	} FS_GRAYMAP;


/**************** outlines **************************************/
typedef struct {
	CACHE_ENTRY *cache_ptr;	
	LONG  size;			/* allocation size in bytes */
	FS_FIXED lo_x;		/* smallest x coordinate */
	FS_FIXED hi_x;		/* largest x coordinate */
	FS_FIXED lo_y;		/* smallest y coordinate */
	FS_FIXED hi_y;		/* largest y coordinate */
	SHORT i_dx;			/* integer x increment to move character origin - 0 if rotated or skewed */
	SHORT i_dy;			/* integer y increment to move character origin - 0 if rotated or skewed */
	FS_FIXED dx;		/* hi resolution x increment to move character origin */
	FS_FIXED dy;		/* hi resolution y increment to move character origin */
	SHORT num;			/* number of elements */
	BYTE *type;			/* type of the elements */
	FS_FIXED *x;		/* x coordinates for the pieces */
	FS_FIXED *y;		/* y coordinates of the pieces */
	/* note: OUTLINEs are allocated in one contiguous piece, so you can delete
	* them with <FS_free_char>. The pointers are set in creation or by <copy_outline>
	*/
	} FS_OUTLINE;

typedef struct FS_STATE {
	SFNT *scaled_fonts;					/* all scaled fonts */
	LFNT *loaded_fonts;					/* all available/loaded fonts */
	LFNT *cur_lfnt;						/* the current named LFNT */
	SFNT *cur_sfnt;						/* the scaled font -- ready for character generation */
	ULONG error;						/* the most recent error code */
	TLIST *tlist;						/* the current transition list -- for the rasterizer */
	TLIST *drop;						/* the current dropout list -- for the rasterizer */
#ifdef STIK
	FS_BITMAP *bmap;					/* for direct stik font rendering */
	FS_FIXED stroke_pct;				/* stroke width as percent of an em */
	FS_FIXED pixel_aspect;				/* displays aspect ratio: (xdpi << 16) / ydpi */
#endif /* STIK */
#ifdef PSEUDO_BOLD
	FS_FIXED bold_pct;					/* percent of em to embolden */
#endif
#ifdef REENTRANT
	VOID (*line)(struct FS_STATE*, FS_FIXED,FS_FIXED,FS_FIXED,FS_FIXED);
#else
	VOID (*line)(FS_FIXED,FS_FIXED,FS_FIXED,FS_FIXED);
#endif /* REENTRANT */
	USHORT lpm;							/* the LinesPerEm of the transformation -- rasterization */
	ULONG flags;
	CACHE_ENTRY *cache[CACHE_MOD];		/* cached things for all SFNT's */
	
	ULONG heap_size;		/* amount of memory we have */
	ULONG allocated;		/* high water mark of memory allocation */

#ifdef INT_MEM
	MEM base;				/* empty to get things rolling */
	MEM *allocp;			/* the last allocated block */
	MEM *memory;			/* allocated in FS_init() */
#endif /* INT_MEM */
	ULONG version;
	} FS_STATE;

/*********** fonts available at init time ***********************/
typedef struct {
	CHAR *name;
	FILECHAR *path;
	BYTE *memptr;
	ULONG index;
	} MY_FONT;

/************** font metrics information ************************/
typedef struct {
	SHORT xMin;
	SHORT yMin;
	SHORT xMax;
	SHORT yMax;
	} BBOX;

/* bit values for "font_type" field */
#define FM_FLAGS_TTF	0x0001
#define FM_FLAGS_STIK	0x0002
#define FM_FLAGS_CCC	0x0004
#define FM_FLAGS_ACT3	0x0008
#define FM_FLAGS_TTC	0x0010

typedef struct {
	USHORT unitsPerEm;								/* from "head" table */
	SHORT os2_ascent;								/* from "OS/2" table */
	SHORT os2_descent;								/* from "OS/2" table */
	USHORT os2_leading;								/* from "OS/2" table */
	SHORT hhea_ascent;								/* from "hhea" table */
	SHORT hhea_descent;								/* from "hhea" table */
	USHORT hhea_leading;							/* from "hhea" table */
	USHORT embedding_bits;							/* from "OS/2" table */
	BBOX font_bbox_exact;
	BBOX font_bbox_with_fluff;
	USHORT font_type;
		/* Bit 0: TTF
		 * Bit 1: STF
		 * Bit 2: CCC
		 * Bit 3: AC3
		 * Bit 4: TTC
		 * Bits 5-15: Reserved */
	USHORT num_cmap_tables;							/* from "cmap" table */
	CMAP_TAB mappings[MAX_MAPPINGS_LEN];			/* from "cmap" table */
	CHAR font_name[MAX_FONT_NAME_LEN];
	CHAR font_family_name[MAX_FONT_FAMILY_NAME_LEN];
	CHAR copyright[MAX_COPYRIGHT_LEN];
	} FONT_METRICS;

/****************************************************************/
/*
* if REENTRANT is defined, the calling program must
* declare and allocate the state pointer using the	
* name "FS_state_ptr".  Then this  pointer will be
* passed by the _PS_ and _PS0_ macros. for example
*
*  main()
*	{
*	// other local variables 
*	#ifdef REENTRANT
*	FS_STATE *FS_state_ptr;
*	FS_state_ptr = (FS_STATE *) malloc(sizeof(FS_STATE));
*	#endif	
*	
*	FS_init(_PS_ heap_SIZE);
*	// other code
*   }
* if REENTRANT is undefined, the state is kept in the
* static variable FS_state, defined in the file api.c
* the address of this static variable will be passed
* by the _PS_ and _PS0_ macros
*
* In this way you can write code which does not care
* whether REENTRANT is defined or not. For example here
* are sample prototypes of two functions that work either way:
*   int foo(_DS0_);
*   int goo(_DS_ arg);
*
* And the sample calls of these functions that work either way:
*   r = foo(_PS0_);
*   r = goo(_PS_ arg);
*/


#ifdef REENTRANT
#define STATE (*FS_state_ptr)			/* dereference passed pointer */
#define _DS0_ FS_STATE *FS_state_ptr	/* define state pointer as the only argument */
#define _DS_  FS_STATE *FS_state_ptr,	/* define state pointer as the first argument */
#define _PS0_ FS_state_ptr				/* pass state pointer as the only argument */
#define _PS_  FS_state_ptr,				/* pass state pointer as the first argument */

#else
#define STATE FS_state					/* just reference the global variable */
#define _DS0_ VOID_ARGS					/* define no arguments */
#define _DS_							/* define nothing */
#define _PS0_							/* pass nothing */
#define _PS_							/* pass nothing */

#endif /* REENTRANT */


#ifdef ACT3
#include "config_it.h"
#include "sfnt.h"
#include "radecomp_it.h"
#include "tablecmp_it.h"
#include "ttf_conv_it.h"
#include "errcodes.h"
#include "ra_ttf_it.h"
#endif


/* where do FS_STATE info and "my_fonts" live ?	*/
#ifdef __cplusplus
#ifndef REENTRANT
extern "C" {extern FS_STATE FS_state;}
#endif /* REENTRANT */
#ifdef  USE_MY_FONTS 
extern "C" { __declspec(dllimport) MY_FONT my_fonts[];}
#endif /* USE_MY_FONTS */

#else
#ifndef REENTRANT
extern FS_STATE FS_state;
#endif /* REENTRANT */
#ifdef USE_MY_FONTS
extern MY_FONT my_fonts[];
#endif /* USE_MY_FONTS */
#endif /* __cplusplus */

#endif /* __OBJECTS__ */
