/******************************************************************************
 * This software module was originally developed by Agfa Monotype Corporation
 * in the course of development of the ISO/IEC 14996-18 standard for reference 
 * purposes and its performance may not have been optimized. This software 
 * module is an implementation of font compression technology as specified 
 * by the ISO/IEC 14996-18 standard.
 * 
 * ISO/IEC gives users a free copyright license to this software module or
 * modifications thereof for use in products claiming conformance to 
 * ISO/IEC 14996-18 International Standard. ISO/IEC gives users the same free
 * license to this software module or modifications thereof for research 
 * purposes and further ISO/IEC standardization.
 * 
 * Those intending to use this software module in products are advised that
 * its use may infringe existing patents and that this license does not grant 
 * rights under any patents. ISO/IEC have no liability for use of this 
 * software module or modifications thereof.
 * 
 * No license is granted and copyright is not released for products that do 
 * not conform to ISO/IEC 14996-18 International Standard.
 * 
 * Agfa Monotype Corporation retains full right to modify and use the code for
 * its own purpose, assign or donate the code to a third party and to inhibit 
 * third parties from using the code for products that do not conform to
 * audiovisual and image-coding related ITU recommendations and 
 * ISO/IEC International Standards. All rights not expressly granted to the 
 * software module or modifications thereof are reserved by 
 * Agfa Monotype Corporation.
 * 
 * This copyright notice must be included in all copies or derivative works.
 * Copyright (c) ISO/IEC 2003. All rights reserved.
 ******************************************************************************/

/*
 * File:						port.h
 * Revision history
 * $Log: port.h,v $
 * Revision 1.1  2004/03/04 21:10:45  vladlev
 * IType font engine API - Initial version
 *
 *
 */

/*
 * this file contains almost all the things you should need
 * to change to get the AMT font scaler up and running on
 * your machine; plus all the optional settings. Please
 * make sure that all the entries in here make sense for
 * your machine, compiler and environment.
 *
 * We assume ANSI standard include files. If your system
 * is not compliant you may need to make additional changes.
 */

#ifndef __PORT__
#define __PORT__

/****************************************************************/
/***** the basic data types -- make SURE these are correct ******/
/****************************************************************/
typedef char CHAR;                 /* signed 8 bit integer      */
typedef unsigned char BYTE;        /* unsigned 8 bit integer    */

typedef short SHORT;               /* signed 16 bit integer     */
typedef unsigned short USHORT;     /* unsigned 16 bit integer   */

typedef long LONG;                 /* signed 32 bit integer     */
typedef unsigned long ULONG;       /* unsigned 32 bit integer   */

#define SIZEOF_LONG 4              /* in bytes ... sizeof(long) */

/* Use this datatype where you need to match the system-defined "char" type
(i.e. to pass to system I/O or string functions) - if you use this datatype
rather than an unadorned "char", the semantics are clearer. */

typedef char FILECHAR;
		 
/****************************************************************/
/**** please set the following options for your installation ****/
/****************************************************************/

/* define to use ACT3 compressed fonts */
#define ACT3
/* define to use algorithmicly compressed fonts */
#undef CCC


/* define to use STIK fonts */
#undef STIK

/* define to get lots of extra error checking, messages, etc. */
#undef FS_DEBUG

/* define to use the dump_bitmap, dump_graymap, etc.. functions */
#define FS_DUMP

/* define if using our local buffer [malloc/free] code */
/* leave undefined to use the operating systems [malloc/free] */
#define INT_MEM

/* define to cache each of OUTLINES, BITMAPS, or GRAYMAPS */
#define CACHE_OUTLINES
#define CACHE_BITMAPS
#define CACHE_GRAYMAPS
#define CACHE_PHASED

/* maximum size (in bytes) of any single cache data entry */
/* eg: an NxM graymap is approximately 30 + (N*M)/2 bytes */
/* so a 50x50 graymap is about 1280 bytes */
/* eg: an NxM bitmap is approximately 30 + (N*M)/8 bytes */
/* so a 100x100 bitmap is about 1280 bytes */
#define CACHE_MAX_SIZE 16384

/* define to get statistics from the cache */
#undef STATS

/* Define for pseudo bold bitmaps and graymaps */
#define PSEUDO_BOLD

/* define to use 'phased' grayscale FS_get_phased() */
#define PHASED

/* define this if you want to use the my_fonts[] array to specify initial fonts */
/* if you do not want to use the my_fonts[] array, #undef USE_MY_FONTS */
#undef USE_MY_FONTS

/* to enable subpixel rendering, define SUBPIXEL as one of */
/* SUBPIXEL_BITMAPS, SUBPIXEL_GRAYMAPS_1 or SUBPIXEL_GRAYMAPS_2 */
/* If SUBPIXEL is undefined, you won't support subpixel rendering */
#define SUBPIXEL_BITMAPS 1		/* bitmaps, filter on the fly : smaller but slower */
#define SUBPIXEL_GRAYMAPS_1 2	/* pre-filtered bitmaps : larger but faster */
#define SUBPIXEL_GRAYMAPS_2 3	/* filtered graymaps : largest, slowest, best looking */

#undef SUBPIXEL
/* #define SUBPIXEL SUBPIXEL_BITMAPS */
/* #define SUBPIXEL SUBPIXEL_GRAYMAPS_1 */
/* #define SUBPIXEL SUBPIXEL_GRAYMAPS_2 */


/* defined to get reentrant / multi-user code (no static read/write data) */
#define REENTRANT

/* define if you have provided assembly code for FIXMUL, FIXDIV, and MULDIV */
#undef HAS_ASM

/* define for embedded bitmap support (EBLC, EBDT data tables) */
#define EMBEDDED_BITMAP

/* define if your system needs to be prevented from LF-CR translation -- like MSDOS */
#define BINARY

/* please define BYTE_ORDER as either BIG_ENDIAN or LITTLE_ENDIAN */
#define BIG_ENDIAN            0	               /* Motorola 680X0, etc */
#define LITTLE_ENDIAN         1	               /* Intel 80X86, etc    */
#define BYTE_ORDER      LITTLE_ENDIAN

/* if you don't support 'const', comment out the lowercase 'const' */
#define CONST const		
	
/* if you don't support 'inline', comment out the lowercase 'inline' */
#define INLINE /* inline */	

/* VOID_ARGS is for functions that have no arguments eg: int foo(VOID_ARGS) */
/* if you don't support 'void', define as nothing, ie: "#define VOID_ARGS" */
#define VOID_ARGS void		

/* VOID is for functions without return values, eg: "VOID foo(int)" and */
/* for functions which return generic pointers, eg: "VOID *foo(int)" and */
/* functions which take a generic pointer as argument, eg: foo(VOID *) */
/* if you don't support 'void', define as SOME type, ie: "#define VOID int" */
#define VOID void	

/****************************************************************/
/*********** compiler support for size of a heap item ***********/
/****************************************************************/

/* if using operating system malloc/free ... say how you get the size */
/* of a heap item. if you can't ... leave FREE_SIZE_DEFINED undefined */
#undef FREE_SIZE_DEFINED

/* Tru64 Unix on the ALPHA has the ANSI call */
#if defined(__digital__) && defined(__unix__)
#define FREE_SIZE_DEFINED
#define FREE_SIZE(p) mallocblksize(p)
ULONG mallocblksize(void *p);
#endif 

/* Silicon Graphics MIPS does too */
#if defined (sgi)
#define FREE_SIZE_DEFINED
#define FREE_SIZE(p) mallocblksize(p)
ULONG mallocblksize(void *p);
#endif

/* Windows likes to be different */
#ifdef _WIN32
#define FREE_SIZE_DEFINED
#define FREE_SIZE(p) _msize(p)
ULONG _msize(void *p);
#endif 



/****************************************************************/
/****** processor/compiler support for 64 bit integers **********/
/****************************************************************/
/* of course we'd prefer to use native 64 bit integers */
#if SIZEOF_LONG==8
#define HAS_FS_INT64
typedef long FS_INT64;
#else

/* if your compiler has a way to support 64 bit integers on 32
* bit platforms add the definition here; using the same format
* as the entries below.  The conditional should uniquely specify
* your compiler */

/* Microsoft Visual C/C++ */
#if defined(_WIN32)
#define HAS_FS_INT64
typedef __int64 FS_INT64;
#endif

/* the GNU C/C++ compiler */
#if defined(__GNUC__) || defined(__GNUG__)
#define HAS_FS_INT64
typedef long long FS_INT64;
#endif

/* Silicon Graphics IRIX compiler on MIPS */
#if defined(sgi)
#define HAS_FS_INT64
typedef long long FS_INT64;
#endif

#endif /* SIZEOF_LONG==8 */





	



#endif /* __PORT__ */
