/******************************************************************************
 * This software module was originally developed by Agfa Monotype Corporation
 * in the course of development of the ISO/IEC 14996-18 standard for reference 
 * purposes and its performance may not have been optimized. This software 
 * module is an implementation of font compression technology as specified 
 * by the ISO/IEC 14996-18 standard.
 * 
 * ISO/IEC gives users a free copyright license to this software module or
 * modifications thereof for use in products claiming conformance to 
 * ISO/IEC 14996-18 International Standard. ISO/IEC gives users the same free
 * license to this software module or modifications thereof for research 
 * purposes and further ISO/IEC standardization.
 * 
 * Those intending to use this software module in products are advised that
 * its use may infringe existing patents and that this license does not grant 
 * rights under any patents. ISO/IEC have no liability for use of this 
 * software module or modifications thereof.
 * 
 * No license is granted and copyright is not released for products that do 
 * not conform to ISO/IEC 14996-18 International Standard.
 * 
 * Agfa Monotype Corporation retains full right to modify and use the code for
 * its own purpose, assign or donate the code to a third party and to inhibit 
 * third parties from using the code for products that do not conform to
 * audiovisual and image-coding related ITU recommendations and 
 * ISO/IEC International Standards. All rights not expressly granted to the 
 * software module or modifications thereof are reserved by 
 * Agfa Monotype Corporation.
 * 
 * This copyright notice must be included in all copies or derivative works.
 * Copyright (c) ISO/IEC 2003. All rights reserved.
 ******************************************************************************/

/*
 * File:						radecomp_it.h
 * Revision history
 * $Log: radecomp_it.h,v $
 * Revision 1.1  2004/03/04 21:10:45  vladlev
 * IType font engine API - Initial version
 *
 *
 */


#define BYTE_CACHE_INTERVAL		128L

#define CURRENT_VERSION_NUMBER	3

#define MAGIC_NUMBER			0x5a

/* how many bits to use in the table -- 10 seems about right */
#define TABLE_BITS 10
#define TABLE_SIZE (1<<TABLE_BITS)

typedef struct {
	BITIO *bio;
	LONG unCompressedByteCount;	/* original file size */

	/* dictionary stuff */
	LONG num_tokens;	/* number of tokens in dictionary */
	LONG string_len;	/* length of <string> */
	BYTE *string;		/* BYTES for all tokens in dictionary */
	ULONG *offsets;		/* token <i> begins at starts + offsets[i] */
	BYTE *lengths;		/* token <i> is lengths[i] bytes LONG */

	/* canonical huffman code stuff */
	LONG num_groups;	/* number of groups (codes with same number of bits) */
	ULONG *firsts;		/* first huffman code in each group - left justified */
	ULONG *lasts;		/* last huffman code in each group - left justified */
	BYTE *bits;			/* number of significant bits in each group */
	ULONG *tokens;		/* first token number in each group */
	ULONG next;			/* next group of bits ... maintained by <get_next_token> */
	ULONG next_mask;	/* to toss excess bits on left */
    LONG  slowGroup;    /* first group not completely in table */
	
	/* navigation */
	BYTE *bitMarks;     /* point to ROM and decode the ULONGs on the fly */
	
	BYTE *deltaBytes;	/* error in the anchor position compared to the ideal position */
	ULONG token_offset;	/* position of first token ... ie: the encoded data */
    /* a table of frequently used huffman codes for quick decoding */
    LONG token_num_table[TABLE_SIZE];
    BYTE token_bits_table[TABLE_SIZE];
	} MTX_RA_DECOMP;


/* prototypes */
MTX_RA_DECOMP *MTX_RA_DECOMP_Create( _DS_ BYTE *data );
BYTE *MTX_RA_GetFragmentPtr( _DS_ MTX_RA_DECOMP *t, ULONG position, ULONG *byteCountPtr );
BYTE *MTX_TTC_GetTTFGlyphMemory( _DS_ BYTE *spline, ULONG oldLength, ULONG *newLength);
VOID MTX_RA_ReleaseFragment( _DS_ MTX_RA_DECOMP *t, VOID *data );
VOID MTX_RA_DECOMP_Destroy(_DS_ MTX_RA_DECOMP *t);
