/******************************************************************************
 * This software module was originally developed by Agfa Monotype Corporation
 * in the course of development of the ISO/IEC 14996-18 standard for reference 
 * purposes and its performance may not have been optimized. This software 
 * module is an implementation of font compression technology as specified 
 * by the ISO/IEC 14996-18 standard.
 * 
 * ISO/IEC gives users a free copyright license to this software module or
 * modifications thereof for use in products claiming conformance to 
 * ISO/IEC 14996-18 International Standard. ISO/IEC gives users the same free
 * license to this software module or modifications thereof for research 
 * purposes and further ISO/IEC standardization.
 * 
 * Those intending to use this software module in products are advised that
 * its use may infringe existing patents and that this license does not grant 
 * rights under any patents. ISO/IEC have no liability for use of this 
 * software module or modifications thereof.
 * 
 * No license is granted and copyright is not released for products that do 
 * not conform to ISO/IEC 14996-18 International Standard.
 * 
 * Agfa Monotype Corporation retains full right to modify and use the code for
 * its own purpose, assign or donate the code to a third party and to inhibit 
 * third parties from using the code for products that do not conform to
 * audiovisual and image-coding related ITU recommendations and 
 * ISO/IEC International Standards. All rights not expressly granted to the 
 * software module or modifications thereof are reserved by 
 * Agfa Monotype Corporation.
 * 
 * This copyright notice must be included in all copies or derivative works.
 * Copyright (c) ISO/IEC 2003. All rights reserved.
 ******************************************************************************/

/*
 * File:				tablecmp.h
 * Revision history
 * $Log: tablecmp.h,v $
 * Revision 1.1  2004/02/10 22:01:11  vladlev
 * Initial version
 *
 */

#include "MEMORY.H"

#ifdef __cplusplus
extern "C" {
#endif

typedef struct
{
    ULONG           start;         /* uncompressed "start"                */
    ULONG           length;        /* uncompressed length                 */
    ULONG           cmpStart;      /* actual compressed start             */
    SHORT           isCompressed;  /* non zero if rom block is compressed */
    MTX_RA_DECOMP * decomp;        /* decompressor if block is compressed */
} MTX_ROM_BLOCK;
typedef struct
{
    MTX_MemHandler *mem;
    int numTags;
    sfnt_TableTag *tagsToCompress;
    ULONG glyf_OffsetStart;		/* Offset to first byte. */
    ULONG glyf_OffsetEnd;		/* One past the last byte .*/
    CHAR *sfnt0;
    LONG sizeIn;
    CHAR *sfnt1;
    LONG sizeOut;
    SHORT numSlots;    /* total number of slots in rb[] */
    SHORT numEntries;  /* number of slots used in rb[]  */
    MTX_ROM_BLOCK *rb;
} MTX_TABLE_COMPRESS;

MTX_TABLE_COMPRESS *MTX_TABLE_COMPRESS_Create           (MTX_MemHandler *mem);
                    MTX_TABLE_COMPRESS_Destroy          (MTX_TABLE_COMPRESS *t);
int                 MTX_TABLE_COMPRESS_ShouldCompress   (MTX_TABLE_COMPRESS *t, sfnt_TableTag tag);
MTX_TABLE_COMPRESS *MTX_TABLE_COMPRESS_Add              (MTX_TABLE_COMPRESS *t, ULONG start, ULONG length,
                                                           ULONG cmpStart, SHORT isCompressed);
void                MTX_TABLE_COMPRESS_SortAndCombine   (MTX_TABLE_COMPRESS *t);
SHORT               MTX_TABLE_COMPRESS_WriteRom         (MTX_TABLE_COMPRESS *t, CHAR *data );
int                 MTX_TABLE_COMPRESS_Compress         (CHAR* nameIn, CHAR* nameOut);
#ifdef __cplusplus
}
#endif
