/******************************************************************************
 * This software module was originally developed by Agfa Monotype Corporation
 * in the course of development of the ISO/IEC 14996-18 standard for reference 
 * purposes and its performance may not have been optimized. This software 
 * module is an implementation of font compression technology as specified 
 * by the ISO/IEC 14996-18 standard.
 * 
 * ISO/IEC gives users a free copyright license to this software module or
 * modifications thereof for use in products claiming conformance to 
 * ISO/IEC 14996-18 International Standard. ISO/IEC gives users the same free
 * license to this software module or modifications thereof for research 
 * purposes and further ISO/IEC standardization.
 * 
 * Those intending to use this software module in products are advised that
 * its use may infringe existing patents and that this license does not grant 
 * rights under any patents. ISO/IEC have no liability for use of this 
 * software module or modifications thereof.
 * 
 * No license is granted and copyright is not released for products that do 
 * not conform to ISO/IEC 14996-18 International Standard.
 * 
 * Agfa Monotype Corporation retains full right to modify and use the code for
 * its own purpose, assign or donate the code to a third party and to inhibit 
 * third parties from using the code for products that do not conform to
 * audiovisual and image-coding related ITU recommendations and 
 * ISO/IEC International Standards. All rights not expressly granted to the 
 * software module or modifications thereof are reserved by 
 * Agfa Monotype Corporation.
 * 
 * This copyright notice must be included in all copies or derivative works.
 * Copyright (c) ISO/IEC 2003. All rights reserved.
 ******************************************************************************/

/*
 * File:						ttf_conv_it.h
 * Revision history
 * $Log: ttf_conv_it.h,v $
 * Revision 1.1  2004/03/04 21:10:46  vladlev
 * IType font engine API - Initial version
 *
 *
 */

#ifdef __cplusplus
extern "C" {
#endif

typedef struct {
	BYTE *ctf1;				/* The ctf data  */
	LONG ctfSize1;
	} ctfDescriptor;


typedef struct {
	BYTE *ttf;				/* The TrueType data */
	LONG ttfSize;			/* The TrueType size */

	BYTE *tableStart;		/* The start of the input table */
	BYTE *tableEnd;			/* Points at the first byte past the input table */
	
	ctfDescriptor ctfData;	/* ctf data descriptor */
	
	SHORT convertFormat;
	SHORT indexToLocFormat;
	} TTF_Converter;

typedef struct {
	SHORT			byteCount;
	SHORT			xBits;
	SHORT			yBits;
	SHORT			dxPlus;
	SHORT			dyPlus;
	BYTE 			xIsNegative; /* No meaning if xBits == 0 */
	BYTE 			yIsNegative; /* No meaning if yBits == 0 */
} coordEncodingType;

/* This data structure is read only, except when it is initialized once */
/* by InitCoordEncoding(), this means it can be shared in a preemptive */
/* multi-tasking environment. */

const static coordEncodingType coordEncoding[128]=
{
{2, 0, 8, 0, 0, 0, 1 },
{2, 0, 8, 0, 0, 0, 0 },
{2, 0, 8, 0, 256, 0, 1 },
{2, 0, 8, 0, 256, 0, 0 },
{2, 0, 8, 0, 512, 0, 1 },
{2, 0, 8, 0, 512, 0, 0 },
{2, 0, 8, 0, 768, 0, 1 },
{2, 0, 8, 0, 768, 0, 0 },
{2, 0, 8, 0, 1024, 0, 1 },
{2, 0, 8, 0, 1024, 0, 0 },
{2, 8, 0, 0, 0, 1, 0 },
{2, 8, 0, 0, 0, 0, 0 },
{2, 8, 0, 256, 0, 1, 0 },
{2, 8, 0, 256, 0, 0, 0 },
{2, 8, 0, 512, 0, 1, 0 },
{2, 8, 0, 512, 0, 0, 0 },
{2, 8, 0, 768, 0, 1, 0 },
{2, 8, 0, 768, 0, 0, 0 },
{2, 8, 0, 1024, 0, 1, 0 },
{2, 8, 0, 1024, 0, 0, 0 },
{2, 4, 4, 1, 1, 1, 1  },
{2, 4, 4, 1, 1, 0, 1  },
{2, 4, 4, 1, 1, 1, 0  },
{2, 4, 4, 1, 1, 0, 0  },
{2, 4, 4, 1, 17, 1, 1 },
{2, 4, 4, 1, 17, 0, 1 },
{2, 4, 4, 1, 17, 1, 0 },
{2, 4, 4, 1, 17, 0, 0 },
{2, 4, 4, 1, 33, 1, 1 },
{2, 4, 4, 1, 33, 0, 1 },
{2, 4, 4, 1, 33, 1, 0 },
{2, 4, 4, 1, 33, 0, 0 },
{2, 4, 4, 1, 49, 1, 1 },
{2, 4, 4, 1, 49, 0, 1 },
{2, 4, 4, 1, 49, 1, 0 },
{2, 4, 4, 1, 49, 0, 0 },
{2, 4, 4, 17, 1, 1, 1 },
{2, 4, 4, 17, 1, 0, 1 },
{2, 4, 4, 17, 1, 1, 0 },
{2, 4, 4, 17, 1, 0, 0 },
{2, 4, 4, 17, 17, 1, 1 },
{2, 4, 4, 17, 17, 0, 1 },
{2, 4, 4, 17, 17, 1, 0 },
{2, 4, 4, 17, 17, 0, 0 },
{2, 4, 4, 17, 33, 1, 1 },
{2, 4, 4, 17, 33, 0, 1 },
{2, 4, 4, 17, 33, 1, 0 },
{2, 4, 4, 17, 33, 0, 0 },
{2, 4, 4, 17, 49, 1, 1 },
{2, 4, 4, 17, 49, 0, 1 },
{2, 4, 4, 17, 49, 1, 0 },
{2, 4, 4, 17, 49, 0, 0 },
{2, 4, 4, 33, 1, 1, 1 },
{2, 4, 4, 33, 1, 0, 1 },
{2, 4, 4, 33, 1, 1, 0 },
{2, 4, 4, 33, 1, 0, 0 },
{2, 4, 4, 33, 17, 1, 1 },
{2, 4, 4, 33, 17, 0, 1 },
{2, 4, 4, 33, 17, 1, 0 },
{2, 4, 4, 33, 17, 0, 0 },
{2, 4, 4, 33, 33, 1, 1 },
{2, 4, 4, 33, 33, 0, 1 },
{2, 4, 4, 33, 33, 1, 0 },
{2, 4, 4, 33, 33, 0, 0 },
{2, 4, 4, 33, 49, 1, 1 },
{2, 4, 4, 33, 49, 0, 1 },
{2, 4, 4, 33, 49, 1, 0 },
{2, 4, 4, 33, 49, 0, 0 },
{2, 4, 4, 49, 1, 1, 1 },
{2, 4, 4, 49, 1, 0, 1 },
{2, 4, 4, 49, 1, 1, 0 },
{2, 4, 4, 49, 1, 0, 0 },
{2, 4, 4, 49, 17, 1, 1 },
{2, 4, 4, 49, 17, 0, 1 },
{2, 4, 4, 49, 17, 1, 0 },
{2, 4, 4, 49, 17, 0, 0 },
{2, 4, 4, 49, 33, 1, 1 },
{2, 4, 4, 49, 33, 0, 1 },
{2, 4, 4, 49, 33, 1, 0 },
{2, 4, 4, 49, 33, 0, 0 },
{2, 4, 4, 49, 49, 1, 1 },
{2, 4, 4, 49, 49, 0, 1 },
{2, 4, 4, 49, 49, 1, 0 },
{2, 4, 4, 49, 49, 0, 0 },
{3, 8, 8, 1, 1, 1, 1 },
{3, 8, 8, 1, 1, 0, 1 },
{3, 8, 8, 1, 1, 1, 0 },
{3, 8, 8, 1, 1, 0, 0 },
{3, 8, 8, 1, 257, 1, 1 },
{3, 8, 8, 1, 257, 0, 1 },
{3, 8, 8, 1, 257, 1, 0 },
{3, 8, 8, 1, 257, 0, 0 },
{3, 8, 8, 1, 513, 1, 1 },
{3, 8, 8, 1, 513, 0, 1 },
{3, 8, 8, 1, 513, 1, 0 },
{3, 8, 8, 1, 513, 0, 0 },
{3, 8, 8, 257, 1, 1, 1 },
{3, 8, 8, 257, 1, 0, 1 },
{3, 8, 8, 257, 1, 1, 0 },
{3, 8, 8, 257, 1, 0, 0 },
{3, 8, 8, 257, 257, 1, 1 },
{3, 8, 8, 257, 257, 0, 1 },
{3, 8, 8, 257, 257, 1, 0 },
{3, 8, 8, 257, 257, 0, 0 },
{3, 8, 8, 257, 513, 1, 1 },
{3, 8, 8, 257, 513, 0, 1 },
{3, 8, 8, 257, 513, 1, 0 },
{3, 8, 8, 257, 513, 0, 0 },
{3, 8, 8, 513, 1, 1, 1 },
{3, 8, 8, 513, 1, 0, 1 },
{3, 8, 8, 513, 1, 1, 0 },
{3, 8, 8, 513, 1, 0, 0 },
{3, 8, 8, 513, 257, 1, 1 },
{3, 8, 8, 513, 257, 0, 1 },
{3, 8, 8, 513, 257, 1, 0 },
{3, 8, 8, 513, 257, 0, 0 },
{3, 8, 8, 513, 513, 1, 1 },
{3, 8, 8, 513, 513, 0, 1 },
{3, 8, 8, 513, 513, 1, 0 },
{3, 8, 8, 513, 513, 0, 0 },
{4, 12, 12, 0, 0, 1, 1 },
{4, 12, 12, 0, 0, 0, 1 },
{4, 12, 12, 0, 0, 1, 0 },
{4, 12, 12, 0, 0, 0, 0 },
{5, 16, 16, 0, 0, 1, 1 },
{5, 16, 16, 0, 0, 0, 1 },
{5, 16, 16, 0, 0, 1, 0 },
{5, 16, 16, 0, 0, 0, 0 }
};			
/* Destructor */
VOID MTX_TTC__Destroy( TTF_Converter *t );


/* Initializes the relative coordinate encoding technique used */
/* to store the coordinates in the CTF format. */
/*VOID MTX_TTC_InitCoordEncoding( VOID_ARGS ); */


SHORT READWORD_INC( BYTE * *pRef );
ULONG READLONG_INC( BYTE * *pRef );

#ifdef __cplusplus
}
#endif
