#include "H263Class.H"
//#include "thread_utils.h"

extern "C"
{
}

/*******************************************************************************
the entry point to the decoder DLL
*******************************************************************************/

void * CreateInstance()
{
  return new H263Class;
}

/*******************************************************************************
*******************************************************************************/

// static members

/* static */ struct H263Class::yuvconvtable H263Class::convtab[256];
/* static */ int H263Class::cliptab[768];

H263Class::H263Class()
{
  mbInitialised = FALSE;
  mpoDecoder = new H263Dec;
  mpucBuffer = new unsigned char [38016];
  mpoInbuf = new bitiob;
  miHeight = 288;
  miWidth = 352;

  // Initialise Yuv to Rgb conversion tables
  {
	// PORTING to CC 5.3
	int i = 0;
    //for (int i = 0; i < 256; ++i) {
    for (i = 0; i < 256; ++i) {
      convtab[i].u1 = int(float(i - 128) * 1.37f);
      convtab[i].u2 = int(float(i - 128) * 0.698f);
      convtab[i].v1 = int(float(i - 128) * 0.336f);
      convtab[i].v2 = int(float(i - 128) * 1.73f);
    }
    for (i = 0; i < 256; ++i)
      cliptab[i] = 0;
    for (; i < 512; ++i)
      cliptab[i] = i - 256;
    for (; i < 768; ++i)
      cliptab[i] = 255;
  }

}

H263Class::~H263Class()
{
  delete mpoInbuf;
  delete[] mpucBuffer;
  if (mpoDecoder != NULL)
    delete mpoDecoder;
}

/*******************************************************************************
*******************************************************************************/

/* virtual */
// WIN-SOL release
bool H263Class::Init()
{
  /***
  mpoDecoder->initdecoder();
  mpoDecoder->first = 0;
  ***/

#ifdef H263DUMP
  mpoFileDump = fopen("H263Dump.Yuv", "wb");
#endif // H263DUMP
  return TRUE;
}

/* virtual */
void H263Class::Run()
{
  TRACE("[H263::Run] \n");

  Init();

//  SetThreadPriority(GetCurrentThread(), THREAD_PRIORITY_ABOVE_NORMAL);
// WIN-SOL release
#ifdef WIN32
  SetThreadPriority(GetCurrentThread(), THREAD_PRIORITY_TIME_CRITICAL);
#else
  Set_Priority(THREAD_PRIORITY_TIME_CRITICAL);
#endif // WIN32

  DecoderImp::Run();
}

/* virtual */
BOOL H263Class::Decode(
  unsigned char * pucInput,
  int iInputLength,
  unsigned char * pucOutput,
  int & riOutputLength)
{
  //int iFlag;

  TRACE("[H263::Decode] \n");
  TRACE("H263Class::Decode: pInput %08x, pOutput %08x \n", pucInput, pucOutput);

  if (riOutputLength < GetOptimalOutputSize())
  {
    riOutputLength = GetOptimalOutputSize();
    return FALSE;
  }

  mpoInbuf->init(pucInput, 0, iInputLength);

  if (mpoDecoder->GetHeader(*mpoInbuf))
  {
    if (mpoDecoder->first) {
      mpoDecoder->initdecoder();
      miHeight = mpoDecoder->vertical_size;
      miWidth = mpoDecoder->horizontal_size;
      mpoDecoder->first = 0;
    }

    getpicture(*mpoInbuf, &(mpoDecoder->framenum), mapucFrame, mpoDecoder);
    (mpoDecoder->framenum)++;
    TRACE("[H263] Frame Decoded %4d \n", mpoDecoder->framenum);
    // put Width & Height in the first 4 bytes...
    pucOutput[0] = (miWidth >> 8) & 0xff;
    pucOutput[1] = (miWidth) & 0xff;
    pucOutput[2] = (miHeight >> 8) & 0xff;
    pucOutput[3] = (miHeight) & 0xff;
    // copy Y, Cb, Cr
    Yuv422ToRgb(
      pucOutput + 4, // dest.
      mapucFrame[0], // source Y
      mapucFrame[1], // source Cb
      mapucFrame[2], // source Cr
      miWidth, miHeight);
#ifdef H263DUMP
#endif
    return TRUE;
  }
  else
  {
    return FALSE;
  }
}

void H263Class::Yuv422ToRgb(
  unsigned char* rgbdata,
  unsigned char* yptr,
  unsigned char* uptr,
  unsigned char* vptr,
  int lumcols, int lumrows)
{
  // row loop
  
  for (int j = lumrows >> 1; j > 0; --j) {
    unsigned char* tuptr = uptr;
    unsigned char* tvptr = vptr;
    int i;
    
    // loop chrominance columns 
    
    for (i = lumcols >> 1; i > 0; --i) {
      int u = *tuptr++;
      int v = *tvptr++;
      
      // horizontal downsampling
      
      for (int k = 2; k > 0; --k) { 
        int y = *yptr++;
        
        *rgbdata++ = (cliptab + 256)[y                 + convtab[v].v2]; // r
        *rgbdata++ = (cliptab + 256)[y - convtab[u].u2 - convtab[v].v1]; // g
        *rgbdata++ = (cliptab + 256)[y + convtab[u].u1                ]; // b
      }
    }
    
    for (i = lumcols >> 1; i > 0; --i) {
      // calculate intermediate values that can be recycled
      
      int u = *uptr++;
      int v = *vptr++;
      
      // horizontal downsampling   
      
      for (int k = 2; k > 0; --k) { 
        int y = *yptr++;
        
        *rgbdata++ = (cliptab + 256)[y                 + convtab[v].v2]; // r
        *rgbdata++ = (cliptab + 256)[y - convtab[u].u2 - convtab[v].v1]; // g
        *rgbdata++ = (cliptab + 256)[y + convtab[u].u1                ]; // b
      }
    }
  }
}

