/************************************************************************
 *
 *  H263Decoder.h, TMN specific defines for libh263 (H.263 decoder)
 *  Copyright (C) 1995, 1996, 1997  Telenor R&D, Norway
 *        Karl Olav Lillevold <Karl.Lillevold@nta.no>
 *        Robert Danielsen <Robert.Danielsen@nta.no>
 *        Karl Anders ygard <Karl.Oygard@nta.no>
 *  
 *  Contacts: 
 *  Robert Danielsen                  <Robert.Danielsen@nta.no>
 *
 *  Telenor Research and Development  http://www.nta.no/brukere/DVC/
 *  P.O.Box 83                        tel.:   +47 63 84 84 00
 *  N-2007 Kjeller, Norway            fax.:   +47 63 81 00 76
 *  
 ************************************************************************/

/*
 * Disclaimer of Warranty
 *
 * These software programs are available to the user without any license
 * fee or royalty on an "as is" basis.  Telenor Research and Development
 * disclaims any and all warranties, whether express, implied, or statuary,
 * including any implied warranties or merchantability or of fitness for a
 * particular purpose.  In no event shall the copyright-holder be liable
 * for any incidental, punitive, or consequential damages of any kind
 * whatsoever arising from the use of these programs.
 *
 * This disclaimer of warranty extends to the user of these programs and
 * user's customers, employees, agents, transferees, successors, and
 * assigns.
 *
 * Telenor Research and Development does not represent or warrant that the
 * programs furnished hereunder are free of infringement of any third-party
 * patents.
 *
 * Commercial implementations of H.263, including shareware, are subject to
 * royalty fees to patent holders.  Many of these patents are general
 * enough such that they are unavoidable regardless of implementation
 * design.
 * */

/*
 * based on mpeg2decode, (C) 1994, MPEG Software Simulation Group
 * and mpeg2play, (C) 1994 Stefan Eckart
 *                         <stefan@lis.e-technik.tu-muenchen.de>
 *
 */

#ifndef _H263DEC_H_
#define _H263DEC_H_

/* From sim.h */
#define PSC        1
#define PSC_LENGTH        17
#define SE_CODE                         31

#define MODE_INTER                      0
#define MODE_INTER_Q                    1
#define MODE_INTER4V                    2
#define MODE_INTRA                      3
#define MODE_INTRA_Q                    4

#define PBMODE_NORMAL                   0
#define PBMODE_MVDB                     1
#define PBMODE_CBPB_MVDB                2

#define ESCAPE                          7167
#define ESCAPE_INDEX                    102

#define PCT_INTER                       1
#define PCT_INTRA                       0
#define ON                              1
#define OFF                             0

#define SF_SQCIF                        1  /* 001 */
#define SF_QCIF                         2  /* 010 */
#define SF_CIF                          3  /* 011 */
#define SF_4CIF                         4  /* 100 */
#define SF_16CIF                        5  /* 101 */


/* this is necessary for the max resolution 16CIF */
#define MBC                             88
#define MBR                             72

#define NO_VEC                          999

class bitiob;

const int OM[5][8][8] = {
	{
		{4,5,5,5,5,5,5,4},
		{5,5,5,5,5,5,5,5},
		{5,5,6,6,6,6,5,5},
		{5,5,6,6,6,6,5,5},
		{5,5,6,6,6,6,5,5},
		{5,5,6,6,6,6,5,5},
		{5,5,5,5,5,5,5,5},
		{4,5,5,5,5,5,5,4},
	},
	{
		{2,2,2,2,2,2,2,2},
		{1,1,2,2,2,2,1,1},
		{1,1,1,1,1,1,1,1},
		{1,1,1,1,1,1,1,1},
		{0,0,0,0,0,0,0,0},
		{0,0,0,0,0,0,0,0},
		{0,0,0,0,0,0,0,0},
		{0,0,0,0,0,0,0,0},
	},
	{
		{0,0,0,0,0,0,0,0},
		{0,0,0,0,0,0,0,0},
		{0,0,0,0,0,0,0,0},
		{0,0,0,0,0,0,0,0},
		{1,1,1,1,1,1,1,1},
		{1,1,1,1,1,1,1,1},
		{1,1,2,2,2,2,1,1},
		{2,2,2,2,2,2,2,2},
	},
	{
		{0,0,0,0,1,1,1,2},
		{0,0,0,0,1,1,2,2},
		{0,0,0,0,1,1,2,2},
		{0,0,0,0,1,1,2,2},
		{0,0,0,0,1,1,2,2},
		{0,0,0,0,1,1,2,2},
		{0,0,0,0,1,1,2,2},
		{0,0,0,0,1,1,1,2},
	},
	{
		{2,1,1,1,0,0,0,0},
		{2,2,1,1,0,0,0,0},
		{2,2,1,1,0,0,0,0},
		{2,2,1,1,0,0,0,0},
		{2,2,1,1,0,0,0,0},
		{2,2,1,1,0,0,0,0},
		{2,2,1,1,0,0,0,0},
		{2,1,1,1,0,0,0,0},
	}
};

const int T_YUV = 0;
const int T_SIF = 1;
const int T_TGA = 2;
const int T_PPM = 3;
const int T_YUV_CONC = 5;

class H263Dec
{
public:

  // constructor

			H263Dec();
			~H263Dec();

  int			Decode(bitiob& inbuf, unsigned char *frame[3]);

  int			first;
  int			framenum;

  static const unsigned char zig_zag_scan[64];
  static const int	convmat[8][4];
  static const int	bscan_tab[4];
  static const int	bquant_tab[4];
  static const int	roundtab[16];

  int			quiet;
  int 			trace;
  char 			errortext[256];
  unsigned char *	refframe[3],*oldrefframe[3],*bframe[3],*newframe[3];
  unsigned char *	edgeframe[3], *edgeframeorig[3], *exnewframe[3];
  int 			MV[2][5][MBR+1][MBC+2];
  int 			modemap[MBR+1][MBC+2];
  unsigned char *       clp;
  int 			horizontal_size,vertical_size,mb_width,mb_height;
  int 			coded_picture_width, coded_picture_height;
  int			chrom_width,chrom_height,blk_cnt;
  int			pict_type,newgob;
  int			mv_outside_frame,syntax_arith_coding,adv_pred_mode,pb_frame;
  int			long_vectors;
  int			fault,expand;
  int			verbose;
  int			refidct;
  int			matrix_coefficients;
  int			temp_ref, prev_temp_ref, quant, source_format;

  int			trd, trb, bscan, bquant;
	
  /* output 
  char *		outputname;
  int			outtype; */

  int			GetHeader(bitiob& inbuf);
  void			GetPictureHeader(bitiob& inbuf);
  int 			FindStartcode(bitiob& inbuf);

  short			block[12][64];

public:
  void			initdecoder();

private:

  void			options(int *argcp, char **argvp[]);
  int			getval(char *argv[]);
};

#endif // _H263DEC_H_
