/**********************************************************************
*
* Copyright  1995 Telenor Research and Development
* All rights reserved
*
**********************************************************************/
/**********************************************************************
*
*	Name:		bitio.h
*	Description:	class definitions for bitwise i/o to buffers
*			and files
*
*	Input:		
*	Returns:	
*	Side effects:	
*
*	Date: 		28.09.95
*	Author: 	Robert Inge Danielsen  <rid@nta.no>
*	Last modified:	20.10.95 Merged bitiob.h and bitiof.h files
*
**********************************************************************/
#ifndef BITIO_H
#define BITIO_H

#include <fstream.h>

/* Class for bitwise buffer i/o */

class bitiob {
	unsigned int current;
	unsigned int next;
	union {
		unsigned char *bufch;
		unsigned int *bufi;
	};
	int pos;
	int buflen;
	int count;
	int bitcount;
	int in_or_out;
public:
	bitiob () {};
	bitiob (unsigned char *, int in_or_out, int);
	void init (unsigned char *, int in_or_out, int);
	//void getw (int n, unsigned int&);
	unsigned int get(int n);
	//void peekw (int n, unsigned int&);
	unsigned int peek(int n);
	void put (int n, const unsigned int);
	void flush ();
	int len () {return bitcount;}
	int eof (); 
	void reset (int in_or_ou);
};


/* Class for bitwise file i/o */

class bitiof {
	unsigned int current;
	unsigned int next;
	union {
		unsigned char *bufch;
		unsigned int *bufi;
	};
	int pos;			/* bitposition in current */
	int buflen;			/* length in bytes */
	int count;			/* count in ints */
	int bitcount;			/* total bitcount */
	int buflim;			/* for flushing when buffer not filled */
	int end_of_file;
	ifstream ifile;
	ofstream ofile;
public:
	bitiof () {};
	~bitiof();
	bitiof (char *, int, int = 2048);
	void init (char *, int, int = 2048);
	void getbuf ();
	void get (int n, unsigned int&);
	void peek (int n, unsigned int&);
	void put (int n, const unsigned int);
	void flush ();
	void finish ();
	int len () {return bitcount;}
	int reof (); 
};

#endif /* BITIO_H */
