//
// This code has been taken from that written by Mike Coleman and
// presented at Stockholm.  The changes have been to remove
// references to the OS class, add the code which writes the
// audio data format into the media stream buffer, and remove the
// Run() method, using the base class's method.
//
// Matthew Leditschke
// CSELT
//

#include "defs.h"
#include "AACDecoder.h"
#include <assert.h>
#include <limits.h>
#include <math.h>

#include "MediaStream.h"

/*-------------------------------------------------------------
The exported function that instantiates the proper decoder
-------------------------------------------------------------*/
void *CreateInstance ()
{
	return new AACDecoder;
}

FILE *fp;

AACDecoder::AACDecoder( )
  : DecoderImp(), m_targetBitrate(64000), m_targetSampleRate(44100)
//  : DecoderImp(), m_targetBitrate(158000), m_targetSampleRate(48000)
{

	fp=fopen("AACDecoder.txt","w");

	fprintf(fp,"AAC decoder is called\n");

	//Son: As I check from the code, the decoded data of aacTool is always 1024 frame of 2 bytes
	m_FrameLen = 1024;
	ChannelNo = 0;

	m_firstDecode = TRUE;
	m_PUlen = m_FrameLen * sizeof(short);  
	m_timeStampIncrement = 1.0e6 * ((double)m_FrameLen/ m_targetSampleRate);
	m_timeStamp = 0.0;
	m_lastTimeStamp=0;
	const char* path;
	path = "aacTool.dll";

	m_pluginHandle = ::LoadLibrary( path );
	assert(m_pluginHandle != 0);

	
	m_decodeFrame     = (long(*)(void*, void*, int, void*, long))  ::GetProcAddress( (HMODULE)m_pluginHandle, "decodeFrame" );
	m_initInstance    = (int (*)(void**)) ::GetProcAddress( (HMODULE)m_pluginHandle, "initInstance" );
	m_releaseInstance = (int (*)(void*))  ::GetProcAddress( (HMODULE)m_pluginHandle, "releaseInstance" );
	
	fprintf(fp,"m_pluginInstanceData %d\n",m_pluginInstanceData);
//SON I call it in setup method
 //  	int res = (*m_initInstance)( &m_pluginInstanceData );

}

AACDecoder::~AACDecoder()
{
	if ( m_releaseInstance != 0 )
		(*m_releaseInstance)( m_pluginInstanceData );
	//m_smpActive.P ();
	if (!IsActive ())
		if(m_pluginHandle!=0)
			FreeLibrary((HINSTANCE )m_pluginHandle);
	
	fflush(fp);	
	fclose(fp);

	//m_smpActive.V ();
}

int AACDecoder::GetOptimalInputSize()
{
	fprintf(fp,"GetOptimalInputSize is called \n");
	return (m_targetBitrate * m_FrameLen) / m_targetSampleRate;
}

int AACDecoder::GetOptimalOutputSize()
{
	return m_PUlen;
}

/*int AACDecoder::GetRecommendedInputBufferSize()
{
	return 2 * GetOptimalInputSize();
}

int AACDecoder::GetRecommendedOutputBufferSize()
{
	return 480 * GetOptimalOutputSize();
}*/

BOOL AACDecoder :: SetFormat (	Capabilities &formatOptions,DecoderParams &decoderParams)
{
		int i;
	BOOL bResult = FALSE;
	fprintf(fp,"SetFormat is called \n");
	for (i = 0; !bResult; i++)
	{	if (i >= formatOptions.GetFormatCapabilityLength ())
			return FALSE;
	    
	    if(formatOptions.GetFormatCapability (i).m_code==MediaFormat :: PCM ) 
			break;
		continue;
	}

	fprintf(fp,"Format type %d\n",formatOptions.GetFormatCapability (i).m_code);
		
    decoderParams.m_nStreamNumber = 0;
	decoderParams.m_format = formatOptions.GetFormatCapability (i--);
	decoderParams.m_nOptimalAUSize = 256;
	decoderParams.m_nDBSize = 20 * GetOptimalInputSize();//was 2*
	decoderParams.m_nCBSize = 20*GetOptimalOutputSize();//was 4*
	decoderParams.m_bRequiresFullAUs = TRUE;
	return TRUE;
}		

//#define SILENCE_PLEASE

BOOL AACDecoder::Decode( LPBYTE AUptr, int AUlen, LPBYTE PUptr, int &PUlen )
{
	if ( PUlen < m_PUlen )
	{
		PUlen = m_PUlen;
		return FALSE;
	}

//  int headerSize = 0;
/*	if (m_firstDecode)
	{
		m_firstDecode = FALSE;

		DecoderAudioFormat* pFmt = (DecoderAudioFormat*)PUptr;
		pFmt->samplesPerSecond = 44100;
		pFmt->bitsPerSample	   = 16;
		pFmt->numChannels	   = 1;
		headerSize = sizeof(DecoderAudioFormat);
	}*/

#ifdef SILENCE_PLEASE
	::memset( PUptr, m_PUlen, 0 );
	PUlen = m_PUlen;
#else

	PUlen = (*m_decodeFrame)( m_pluginInstanceData, AUptr, AUlen, PUptr, PUlen);

	if ( PUlen == 0 )
	{
		::memset( PUptr, m_PUlen, 0 );
		PUlen = m_PUlen;
	}
	else
		PUlen;
#endif
	return PUlen;
}



/*-------------------------------------------------------------
In this decoder, attributes are fixed for all units
-------------------------------------------------------------*/
DWORD AACDecoder :: GetAttributes (
	ATTRIBUTE_CODE code, 
	const LPBYTE
)
{
	fprintf(fp,"GetAttributes is called: %d\n",code);
	switch (code) {
	case AUDIO_FREQUENCY:
		return  m_targetSampleRate;
	case BITS_PER_SAMPLE :
		return 16;
	case NUM_CHANNELS:
		return ChannelNo;
	default:
		return 0;
	}
}


void AACDecoder :: Run ()
{
	
	LPBYTE pInData, pOutData,pDecodedFrame;
	int nInSize, nOutSize;
	DWORD dwTime, RenderTime=0;
	DWORD FrameTime = (int)floor(m_FrameLen * 1000 / (double)m_targetSampleRate + 0.5);
	BOOL bEOU;


	fprintf(fp,"AACDecoder :: Run new start\n");

	
	nOutSize = GetOptimalOutputSize ();
	
    //SON Header consist of 
	//ChannelNo, WindowSize,SamplingRate,SymbolSize

	int HeaderSize = 4*sizeof(int);
	pDecodedFrame  = (LPBYTE)malloc(nOutSize); 
	if(pDecodedFrame == NULL)
		throw;

	while (IsActive ())
	{
		if ( 		(pInData =
			(LPBYTE) GetInputStream ()-> FetchNext (nInSize, dwTime, bEOU, TRUE)
		) !=END_OF_STREAM )
		{
			nOutSize = GetOptimalOutputSize ();
			while (!Decode (pInData, nInSize, pDecodedFrame, nOutSize))
			{
				pDecodedFrame = (LPBYTE) GetOutputStream ()  -> Reallocate (nOutSize);
				if (pDecodedFrame == NULL)
					throw;
				if (pDecodedFrame == END_OF_STREAM)
				{
					fprintf(fp,"AACDecoder :: output stream end of stream: after decode\n");
					free(pDecodedFrame);
					return;
				}
			}


			pOutData = (LPBYTE) GetOutputStream () -> Allocate (nOutSize+HeaderSize, TRUE, FALSE);
			if (pOutData == NULL)
				throw;
			if (pOutData == END_OF_STREAM)
			{
				fprintf(fp,"AACDecoder :: output stream end of stream after allocate\n");
				free(pDecodedFrame);
				return;
			}

			((int*)pOutData)[0] = ChannelNo;
			((int*)pOutData)[1] = 1024;
			((int*)pOutData)[2] = m_targetSampleRate;
			((int*)pOutData)[3] = 16;

			// copy decoded data into buffer to dispatch
			memcpy(pOutData+HeaderSize , pDecodedFrame, nOutSize);


			GetOutputStream () -> Dispatch (nOutSize+HeaderSize, RenderTime);
			RenderTime+=FrameTime;
			m_lastTimeStamp=dwTime;

			GetInputStream () -> Release (TRUE);

		}
		else
		{
			fprintf(fp,"AACDecoder :: end of stream: after FetchNext\n");
			break;
		}
	}
	fprintf(fp,"AACDecoder :: Run stop\n");
	free(pDecodedFrame);

}

int SamplingRates[16] = {
	96000, 88200, 64000, 48000, 44100, 32000, 24000, 22050, 
	16000, 12000, 11025, 8000, 7350, 0, 0, 0
};


BOOL AACDecoder :: Setup (
	const BYTE *pSpecificInfo,
	int nSpecificInfoLength,
	int
)
{
	m_FrameLen=0;
	m_targetSampleRate = 0;

	BYTE Data;
	Data = pSpecificInfo[0] >> 3;  // AudioObjectType (5 bits)
	if ( 	Data == 1 || Data == 2 ||
		Data == 3 || Data == 4 ||
		Data == 6 || Data == 7 ) //General Audio
	{
		Data = ((pSpecificInfo[0] >> 3) & 0x0F); //Channel config See Mpeg-4 audio subpart 1: 1.6.3.4
		ChannelNo = 0;
		if((Data>0) &&(Data<8))
			ChannelNo = Data;

		Data = ((pSpecificInfo[0] << 1) & 0x0F) | (pSpecificInfo[1] >> 7);
		if(Data != 0xf)  //I deal with valid index for Sampling only 
		{
			m_targetSampleRate = SamplingRates[Data];

			Data = (pSpecificInfo[1] >> 2) & 0x1;
			//The followings are true only if window_sequence is not EIGHT_SHORT_SEQUENCE or 
			//"low delay coding tool" . See Mpeg-4 audio subpart 4
			if(!Data)
				m_FrameLen=1024;
			else
				m_FrameLen=960;


		}

	}

	fprintf(fp,"Frame %d Sampling %d\n",m_FrameLen, m_targetSampleRate);

	//SON I pass the actual sampling rate to the decoder
	m_pluginInstanceData = (void*)(&m_targetSampleRate);
	int res = (*m_initInstance)( &m_pluginInstanceData );
	return TRUE;

}