#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include "cifsdec.h"
#include <vector>
#include "MPG_IfsQ_E.h"	// SOWOO 20021210
using namespace std;	//MJH 010911

CIFSDecoder::CIFSDecoder():
	pCIFS(new MPG_IfsQ_E), cifs(*pCIFS)	// SOWOO 20021210
{ 
	pbuf=NULL;
	pCoordIdx = NULL;
	pCoord = NULL;
	pColorIndex = NULL;
	pColor = NULL;
	pNormalIndex = NULL;
	pNormal = NULL;
	pTexCoordIndex = NULL;
	pTexCoord = NULL;
}

CIFSDecoder::~CIFSDecoder() { 
//	free(pbuf);
	if(pCoordIdx!=NULL)       delete pCoordIdx;
	if(pCoord!=NULL)          delete pCoord;
	if(pColorIndex!=NULL)     delete pColorIndex;
	if(pColor!=NULL)          delete pColor;
	if(pNormalIndex!=NULL)    delete pNormalIndex;
	if(pNormal!=NULL)         delete pNormal;
	if(pTexCoordIndex!=NULL)  delete pTexCoordIndex;
	if(pTexCoord!=NULL)      delete pTexCoord;
	if(pCIFS!=NULL)			delete pCIFS;	// SOWOO 20021210
}

unsigned char* CIFSDecoder::Decode(int &m_nLastBytes, int &m_nLastBits, int &m_nlength, int tsize)
{
	bool saveWrlLevels = false;
	bool success=true;
	unsigned char* pt=NULL;

	if(pbuf == NULL) 
		error("Input Buffer is empty!!\n");
	int validbyte = m_nLastBytes >> (m_nLastBits);
	validbyte &= 0xff;

	MPG_InputStream istrm;
	success = istrm.open(pbuf, validbyte, m_nLastBits, m_nlength, tsize);
	if(success) success = pCIFS->expand(istrm,saveWrlLevels);
	if(success) pt = istrm.close(pbuf, m_nLastBytes, m_nLastBits, m_nlength, tsize);

	if(!success || pt==NULL) {
		error("reading and/or expanding input file");
		return NULL;
	}
	else
		return pt; 
}

bool CIFSDecoder::Decode(char *urlname)
{
	bool saveWrlLevels=true;	// SOWOO 20021210
	int success;

	MPG_InputStream istrm;

	success = istrm.open(urlname);
	if(success) success = pCIFS->expand(istrm, saveWrlLevels);
	if(success) success = istrm.close();
	
	if(!success) {
		error("reading and/or expanding input file");
		return false;
	}
	return true;
}

// SOWOO 20021210
bool CIFSDecoder::DecodeWithBuffer(char *pBuffer, int nLength)
{
	bool saveWrlLevels=true;	// SOWOO 20021210
	int success;

	MPG_InputStream istrm;

	success = istrm.open(nLength, (unsigned char*) pBuffer);
	if(success) success = pCIFS->expand(istrm, saveWrlLevels);
	if(success) success = istrm.close();
	
	if(!success) {
		error("reading and/or expanding input file");
		return false;
	}
	return true;
}


void CIFSDecoder::error(const char *msg)
{
//  FILE* logfp = stderr;
//  fprintf(logfp,"error : %s\n",(msg)?msg:"");
//  exit(-1);
}

/////////////////////////////////////////////////////////////////////////
// extract the field value of IndexedFaceSet Node ///////////////////////
/////////////////////////////////////////////////////////////////////////
bool CIFSDecoder::GetCcw()
{
	return pCIFS->getCcw(); 
}

bool CIFSDecoder::GetConvex()
{ 
	return pCIFS->getConvex(); 
}

float CIFSDecoder::GetCreaseAngle()
{
	return pCIFS->getCreaseAngle(); 
}

bool CIFSDecoder::GetSolid()
{
	return pCIFS->getSolid(); 
}

int CIFSDecoder::GetCoordIdx()
{
	vector<int>& pt = pCIFS->getCoordIndex();
	int size = pt.size();
	pCoordIdx = new int[size];
	for(int i=0; i<size; i++)
		pCoordIdx[i] = pt[i];

	return size;
}

int CIFSDecoder::GetCoord()
{
	vector<float>& pt = pCIFS->getCoord();
	int size = pt.size();
	pCoord = new float[size];
	for(int i=0; i<size; i++)
		pCoord[i] = pt[i];

	return size;
}

bool CIFSDecoder::GetColorPerVertex()
{
	return pCIFS->getColorPerVertex();
}

int CIFSDecoder::GetColorIndex()
{
	vector<int>& pt = pCIFS->getColorIndex();
	int size = pt.size();
	pColorIndex = new int[size];
	for(int i=0; i<size; i++)
		pColorIndex[i] = pt[i];
	
	return size;
}

int CIFSDecoder::GetColor()
{
	vector<float>& pt = pCIFS->getColor();
	int size = pt.size();
	pColor = new float[size];
	for(int i=0; i<size; i++)
		pColor[i] = pt[i];

	return size;
}

bool CIFSDecoder::GetNormalPerVertex()
{
	return pCIFS->getNormalPerVertex();
}

int CIFSDecoder::GetNormalIndex()
{
	vector<int>& pt = pCIFS->getNormalIndex();
	int size = pt.size();
	pNormalIndex = new int[size];
	for(int i=0; i<size; i++)
		pNormalIndex[i] = pt[i];

	return size;
}

int CIFSDecoder::GetNormal()
{
	vector<float>& pt = pCIFS->getNormal();
	int size = pt.size();
	pNormal = new float[size];
	for(int i=0; i<size; i++)
		pNormal[i] = pt[i];

	return size;
}

int CIFSDecoder::GetTexCoordIndex()
{
	vector<int>& pt = pCIFS->getTexCoordIndex();
	int size = pt.size();
	pTexCoordIndex = new int[size];
	for(int i=0; i<size; i++)
		pTexCoordIndex[i] = pt[i];

	return size;
}

int CIFSDecoder::GetTexCoord()
{
	vector<float>& pt = pCIFS->getTexCoord();
	int size = pt.size();
	pTexCoord = new float[size];
	for(int i=0; i<size; i++)
		pTexCoord[i] = pt[i];

	return size;
}




#if defined SAIT_CONFORMANCE_2003
void CIFSDecoder::setValues(unsigned char *pData)
{

	// 3DMC  ũ⸦ .
	//  ũ⸦ rnOutputLength   rnOutputLength ,
	// rnOutputLength  ũ⸦  !
	// , return FALSE
	int i = 0;
	int j = 0;
	int   nLengthOfTotal = 0;
//	Vec3f fdummy;
		
	// ccw
	bool* ccw = (bool*) pData;
	//SF<Bool>* pData_ccw = (SF<Bool>*) pIndexedFaceSet->GetField(8);
	//*pData_ccw = ccw[0];
	pCIFS->setCcw(ccw[0]);
	nLengthOfTotal += sizeof(bool); 

	// convex
	bool* convex = (bool*) (pData + nLengthOfTotal);
	//SF<Bool>* pData_convex = (SF<Bool>*) pIndexedFaceSet->GetField(11);
	//*pData_convex = convex[0];
	pCIFS->setConvex(convex[0]);
	nLengthOfTotal += sizeof(bool); 
		
	// creaseAngle
	float* creaseAngle = (float *) (pData + nLengthOfTotal);
	//SF<float>* pData_creaseAngle = (SF<float>*) pIndexedFaceSet->GetField(13);
	//*pData_creaseAngle = creaseAngle[0];
	pCIFS->setCreaseAngle(creaseAngle[0]);
	nLengthOfTotal += sizeof(float); 

	// solid
	bool* solid = (bool *) ( pData + nLengthOfTotal);
	//SF<Bool>* pData_solid = (SF<Bool>*) pIndexedFaceSet->GetField(16);
	//*pData_solid = solid[0];
	pCIFS->setSolid(solid[0]);
	nLengthOfTotal += sizeof(bool); 

	// Size of Coord
	int* pData_SizeCoord = (int*) (pData + nLengthOfTotal);
	int nSizeOfCoord = pData_SizeCoord[0];
	nLengthOfTotal += sizeof(int);

	// Coord
	float* pData_coord = (float*) (pData + nLengthOfTotal);
	//SFNode<class Coordinate>* pCoordinate = (SFNode<class Coordinate>*) pIndexedFaceSet->GetField(5);
	//MF<Vec3f>* pPoint = (MF<Vec3f>*) pCoordinate->GetNode()->GetField(0);

	vector<float>& pt = pCIFS->getCoord();

	 for(i = 0; i<nSizeOfCoord; i++)
		 pt.push_back(pData_coord[i]);

	 nLengthOfTotal += sizeof(float)*nSizeOfCoord; 
 
	// Size of coordIndex
	int* pData_SizeCoordIndex = (int*) (pData + nLengthOfTotal);
	int nSizeOfCoordIndex = pData_SizeCoordIndex[0];
	nLengthOfTotal += sizeof(int); 

	// coordIndex
	int* pData_coordIndex = (int*) (pData + nLengthOfTotal);
	//MF<int>* pCoordIndex = (MF<int>*) pIndexedFaceSet->GetField(12);

	vector<int>& ci = pCIFS->getCoordIndex();

	for(i = 0; i< nSizeOfCoordIndex; i++)
		 ci.push_back(pData_coordIndex[i]);

	nLengthOfTotal += sizeof(int)*nSizeOfCoordIndex; 

	// Size of Color
	int* pData_SizeColor = (int*) (pData + nLengthOfTotal);
	int nSizeOfColor = pData_SizeColor[0];
	nLengthOfTotal += sizeof(int); 

	if(nSizeOfColor > 0)
	{
		// colorPerVertex:  True or False
		bool* colorPerVertex = (bool*) ( pData + nLengthOfTotal);
		//SF<Bool>* pData_colorPerVertex = (SF<Bool>*) pIndexedFaceSet->GetField(10);
		//*pData_colorPerVertex = colorPerVertex[0];
		pCIFS->setColorPerVertex(colorPerVertex[0]);
		nLengthOfTotal += sizeof(bool); 

		// Color
		float* pData_color = (float*) (pData + nLengthOfTotal);
		//SFColorNode* pColor = (SFColorNode*) pIndexedFaceSet->GetField(4);
		//MF<Vec3f>* pValue = (MF<Vec3f>*) pColor->GetNode()->GetField(0);

		vector<float>& cl = pCIFS->getColor();
		for(i = 0; i<nSizeOfColor; i++)
			cl.push_back(pData_color[i]);

		nLengthOfTotal += sizeof(float)*nSizeOfColor; 

		// Size of Color Index
		int* pData_SizeColorIndex = (int*) (pData + nLengthOfTotal);
		int nSizeOfColorIndex = pData_SizeColorIndex[0];
		nLengthOfTotal += sizeof(int); 

		int* pData_colorIndex = (int*) (pData + nLengthOfTotal);

		// Color Index
		if(nSizeOfColorIndex > 0)
		{
			//MF<int>* pColorIndex = (MF<int>*) pIndexedFaceSet->GetField(9);
		
			vector<int>& cli = pCIFS->getColorIndex();

			for(i = 0; i< nSizeOfColorIndex; i++)
				cli.push_back(pData_colorIndex[i]);
			nLengthOfTotal += sizeof(int)*nSizeOfColorIndex; 
		}
		else
		{
			if(colorPerVertex[0])
			{
				//MF<int>* pColorIndex = (MF<int>*) pIndexedFaceSet->GetField(9);

				vector<int>& cli = pCIFS->getColorIndex();
				for(i = 0; i< nSizeOfCoordIndex; i++)
					cli.push_back(pData_colorIndex[i]);
			}
		}
	}


	// Size of Normal
	int* pData_SizeNormal = (int*) (pData + nLengthOfTotal);
	int nSizeOfNormal = pData_SizeNormal[0];
	nLengthOfTotal += sizeof(int); 

	if(nSizeOfNormal > 0) // Normals
	{
		// NormalPerVertex:  True or False
		bool* normalPerVertex = (bool*) ( pData + nLengthOfTotal);
		//SF<Bool>* pData_normalPerVertex = (SF<Bool>*) pIndexedFaceSet->GetField(15);
		//*pData_normalPerVertex = normalPerVertex[0];
		pCIFS->setNormalPerVertex(normalPerVertex[0]);
		nLengthOfTotal += sizeof(bool); 
		
		// Normal
		float* pData_normal = (float*) (pData + nLengthOfTotal);
		//SFNode<class Normal>* pNormal = (SFNode<class Normal>*) pIndexedFaceSet->GetField(6);
		//MF<Vec3f>* pValue = (MF<Vec3f>*) pNormal->GetNode()->GetField(0);

		vector<float>& nm = pCIFS->getNormal();

		for(i = 0; i<nSizeOfNormal; i++)
			nm.push_back(pData_normal[i]);
		nLengthOfTotal += sizeof(float)*nSizeOfNormal; 

		// Size of Normal Index
		int* pData_SizeNormalIndex = (int*) (pData + nLengthOfTotal);
		int nSizeOfNormalIndex = pData_SizeNormalIndex[0];
		nLengthOfTotal += sizeof(int); 

		int* pData_normalIndex = (int*) (pData + nLengthOfTotal);

		// NormalIndex
		if(nSizeOfNormalIndex > 0) 
		{
			//MF<int>* pNormalIndex = (MF<int>*) pIndexedFaceSet->GetField(14);
		
			vector<int>& nmi = pCIFS->getNormalIndex();
			for(i = 0; i< nSizeOfNormalIndex; i++)
				nmi.push_back(pData_normalIndex[i]);

			nLengthOfTotal += sizeof(int)*nSizeOfNormalIndex; 
		}
		else  
		{
			if(normalPerVertex[0]) 
			{
				//MF<int>* pNormalIndex = (MF<int>*) pIndexedFaceSet->GetField(14);
				vector<int>& nmi = pCIFS->getNormalIndex();
				for(i = 0; i< nSizeOfCoordIndex; i++)
					nmi.push_back(pData_normalIndex[i]);
			}
		}
	}

	// Size of TexCoord
	int* pData_SizeTexCoord = (int*) (pData + nLengthOfTotal);
	int nSizeOfTexCoord = pData_SizeTexCoord[0];
	nLengthOfTotal += sizeof(int); 


	if(nSizeOfTexCoord > 0)
	{
		float* pData_texcoord = (float*) (pData + nLengthOfTotal);
		// TexCoord
		//SFNode<class TextureCoordinate>* pTexCoord = (SFNode<class TextureCoordinate>*) pIndexedFaceSet->GetField(7);
		//MF<Vec2f>* pValue = (MF<Vec2f>*) pTexCoord->GetNode()->GetField(0);

		vector<float>& tc = pCIFS->getTexCoord();

		 for(i = 0; i<nSizeOfTexCoord; i++)
			 tc.push_back(pData_texcoord[i]);
		nLengthOfTotal += sizeof(float)*nSizeOfTexCoord; 

		// Size of TexCoordIndex
		int* pData_SizeTexCoordIndex = (int*) (pData + nLengthOfTotal);
		int nSizeOfTexCoordIndex = pData_SizeTexCoordIndex[0];
		nLengthOfTotal += sizeof(int); 

		// TexCoordIndex
		if(nSizeOfTexCoordIndex > 0)
		{
			int* pData_texcoordIndex = (int*) (pData + nLengthOfTotal);
			//MF<int>* pTexCoordIndex = (MF<int>*) pIndexedFaceSet->GetField(17);
		
			vector<int>& tci = pCIFS->getTexCoordIndex();

			for(i = 0; i< nSizeOfTexCoordIndex; i++)
				tci.push_back(pData_texcoordIndex[i]);
			nLengthOfTotal += sizeof(int)*nSizeOfTexCoordIndex; 
		}
	}
}


void CIFSDecoder::write(char* fileName, int ID)
{
	pCIFS->write(fileName, ID);
}

#endif SAIT_CONFORMANCE_2003