//-------------------------------------------------------------------------
// 
// Copyright (C) IBM Corporation & Samsung Electronics Co. LTD., 1996, 1997, 1998, 1999
// Licensed Materials, Program Property of IBM & Samsung.
// 
// This software module is an implementation of a part of one or more
// MPEG-4 tools as specified by the MPEG-4 standard (ISO/IEC 14496).
// ISO/IEC gives users of the MPEG-4 standard (ISO/IEC 14496) free
// license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4
// standard (ISO/IEC 14496).  Those intending to use this software module
// in hardware or software products are advised that its use may infringe
// existing patents.  The original developer of this software module and
// his/her company, the subsequent editors and their companies, and
// ISO/IEC have no liability for use of this software module or
// modifications thereof in an implementation.  Copyright is not released
// for non MPEG-4 standard (ISO/IEC 14496) conforming products.  IBM
// Corp. & Samsung Electronics Co. LTD retains full right to use the code 
// for his/her own purpose, assign or donate the code to a third party and
// to inhibit third parties from using the code for non MPEG-4 standard 
// (ISO/IEC 14496) conforming products. This copyright notice must be included
// in all copies or derivative works.
// 
//  AUTHORS:
//    Gabriel Taubin
//    Andre Gueziec
//    Claudio Silva
//    Ioana Martin
//  
//  IBM T. J. Watson Research Center
//  P.O.Box 704
//  Yorktown Heights, New York 10598
//  
//  BASED ON EARLIER IMPLEMENTATION OF
//  VRML COMPRESSED BINARY FORMAT PROPOSAL BY:
//    Gabriel Taubin
//    Bill Horn
//    Francis Lazarus
//
//  Functionality : Error Resilience
//
//  Sungjin Kim  (sunjin@sait.samsung.co.kr)
//  Mahn-Jin Han  (manjini@sait.samsung.co.kr)
//  Mun-Sup Song (mssong@sait.samsung.co.kr)  
//
//  Signal Processing Lab. Samsung A.I.T. 
//-------------------------------------------------------------------------

// MPG_BranchTree.h: interface for the MPG_BranchTree class.
//
//////////////////////////////////////////////////////////////////////

#ifndef _MPG_BranchTree_h_
#define _MPG_BranchTree_h_

#if !defined(AFX_MPG_BRANCHTREE_H__3D8CDBE0_2E71_11D3_8CB4_00105A6B59A3__INCLUDED_)
#define AFX_MPG_BRANCHTREE_H__3D8CDBE0_2E71_11D3_8CB4_00105A6B59A3__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#define LEFT_BRANCH 1
#define RIGHT_BRANCH 0

class MPG_BranchTree;

class MPG_EachBranchTree
{
public:
	MPG_EachBranchTree();
	virtual ~MPG_EachBranchTree();

	int	face_num;
	int corner_num;
	MPG_BranchTree *next;

};

class MPG_BranchTree  
{
public:
	MPG_BranchTree();
	virtual ~MPG_BranchTree();

	MPG_EachBranchTree ebt[2];
	MPG_BranchTree *parent;
	int cur_dir;

	void InsertInfo(int dir, int f, int c);
	void GetInfo(int dir, int& f, int& c);
	void MakeBranch(int dir);
	MPG_BranchTree* GetBranch(int dir);
	bool Empty(int dir);
	MPG_BranchTree* GetParent();
	void SetCurDir(int dir);
	int GetCurDir();
};

class MPG_BranchForest
{
public:
	MPG_BranchForest();
	virtual ~MPG_BranchForest();

	MPG_BranchTree* root_bt;
	MPG_BranchForest* next;

	void MakeNextBTree();
	MPG_BranchForest* GetNextBTree();
	MPG_BranchTree* GetRootBTree(); 
};
#endif // !defined(AFX_MPG_BRANCHTREE_H__3D8CDBE0_2E71_11D3_8CB4_00105A6B59A3__INCLUDED_)

#endif // _MPG_BranchTree_h_
