//-------------------------------------------------------------------------
// 
// Copyright (C) IBM Corporation & Samsung Electronics Co. LTD., 1996, 1997, 1998, 1999
// Licensed Materials, Program Property of IBM & Samsung.
// 
// This software module is an implementation of a part of one or more
// MPEG-4 tools as specified by the MPEG-4 standard (ISO/IEC 14496).
// ISO/IEC gives users of the MPEG-4 standard (ISO/IEC 14496) free
// license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4
// standard (ISO/IEC 14496).  Those intending to use this software module
// in hardware or software products are advised that its use may infringe
// existing patents.  The original developer of this software module and
// his/her company, the subsequent editors and their companies, and
// ISO/IEC have no liability for use of this software module or
// modifications thereof in an implementation.  Copyright is not released
// for non MPEG-4 standard (ISO/IEC 14496) conforming products.  IBM
// Corp. & Samsung Electronics Co. LTD retains full right to use the code 
// for his/her own purpose, assign or donate the code to a third party and
// to inhibit third parties from using the code for non MPEG-4 standard 
// (ISO/IEC 14496) conforming products. This copyright notice must be included
// in all copies or derivative works.
// 
//  AUTHORS:
//    Gabriel Taubin
//    Andre Gueziec
//    Claudio Silva
//    Ioana Martin
//  
//  IBM T. J. Watson Research Center
//  P.O.Box 704
//  Yorktown Heights, New York 10598
//  
//  BASED ON EARLIER IMPLEMENTATION OF
//  VRML COMPRESSED BINARY FORMAT PROPOSAL BY:
//    Gabriel Taubin
//    Bill Horn
//    Francis Lazarus
//
//  Functionality : Error Resilience
//
//  Sungjin Kim  (sunjin@sait.samsung.co.kr)
//  Mahn-Jin Han  (manjini@sait.samsung.co.kr)
//  Mun-Sup Song (mssong@sait.samsung.co.kr)  
//
//  Signal Processing Lab. Samsung A.I.T. 
//-------------------------------------------------------------------------


#include <stdio.h>
#include "MPG_ER.h"
#include "MPG_Global.h"

void MPG_ER::setDirection()
{
	int r,i,j;
	int pair[2];
	int nR = getNumberOfRuns();

	for(r=0; r<nR; r++) {
		if(getLeaf(r)==0) {
			for(j=0, i=0; i<nR; i++) {
				if(r==getFather(i)) {
					pair[j]=i;
					j++;
				}
			} 
			if(j==2) {
				if(getLength(pair[0]) <= getLength(pair[1])) {
					putDirect(1);  //left_first
					putFirst(pair[0]);
					putSecond(pair[1]);
				} else {
					putDirect(0);  //right_first
					putFirst(pair[1]);
					putSecond(pair[0]);
				}
			} else {
				printf("\n Error! incorrect tree structure\n");
				exit(1);
			}
		} else {
			putDirect(-1);  //in case of Leaf
			putFirst(-1);
			putSecond(-1);
		}
	}
}

void MPG_ER::printDirection()
{
	int i;
	
	printf("run, start, branch, leaf, father, length, direct, first, second\n");
	for(i=0; i<getNumberOfRuns(); i++) {
		printf("%3d, %6d, %6d, %3d, %6d, %7d, %6d, %6d, %6d\n",
			getRun(i), getStart(i),getBranch(i),getLeaf(i), 
			getFather(i), getLength(i), getDirect(i), getFirst(i), getSecond(i));
	}
	printf("\n");
}

bool MPG_PacketQ_C::vg_StartCode(
					MPG_OutputStream& vgostrm,
					MPG_OutputStream& ostrm, 
					MPG_OutputBitStream& vgobstrm,
					MPG_OutputBitStream& obstrm,
					MPG_QFac& vgcodec, 
					MPG_QFac& codec,
					int &vg_mobl_bits) 
{
	updateQFac(obstrm,vgobstrm,codec,vgcodec); 
	// write 3D_MOBL_start_code
	if(vg_sc_enable) {
		vg_mobl_bits=totalbits;
		if(vg_pt0_enable) {
			if(vgostrm.putStartCode(MPG_Global::MOBL0_start_code)==false) 
				return false;
		} else {
			if(vgostrm.putStartCode(MPG_Global::MOBL1_start_code)==false) 
				return false;
		}
		vgobstrm.init();  
		if(vgobstrm.putMBitInt(8,mobl_id)==false) return false;
		if(vgobstrm.get_i() != 8) {
			totalbits += 32;
			restbits -= 32;
		} else {
			totalbits += 24;
			restbits -= 24;
		}
		vgobstrm.flush(); 
		vgcodec.start_encode();
		MPG_Global::vg_reset(); 
		MPG_Global::vg_store();
		vg_mobl_bits=totalbits-vg_mobl_bits;
	} 
	return true;
}

void MPG_PacketQ_C::vg_packetization(
					MPG_OutputStream& vgostrm,
					MPG_OutputStream& ostrm, 
					MPG_OutputBitStream& vgobstrm,
					MPG_OutputBitStream& obstrm,
					MPG_QFac& vgcodec, 
					MPG_QFac& codec,
					int icc,
					MPG_ER2& ccbit)
{
	if((icc+1==ccbit.next) && (!vg_pt0_enable)) {
		if(vgcodec.rbits < 24+bdry_loopbits*2+next_spbits) { 
//			int rbytes=stuffing(vgostrm,vgobstrm,vgcodec);
			vgobstrm.init();
			totalbits=0;
			restbits=packetSize;
			vgcodec.tbits=totalbits;
			vgcodec.rbits=restbits;
		}
	}
	writeStream(vgostrm,ostrm);
	updateQFac(vgobstrm,obstrm,vgcodec,codec);
	MPG_Global::vg_store(); 
	nvg++;
	if(vg_sc_enable) 
	  mobl_id++;
}

bool MPG_PacketQ_C::sp_StartCode(
					MPG_OutputStream& spostrm,
					MPG_OutputStream& ostrm, 
					MPG_OutputBitStream& spobstrm,
					MPG_OutputBitStream& obstrm,
					MPG_QFac& spcodec, 
					MPG_QFac& codec,
					int &sp_mobl_bits)
{
	updateQFac(obstrm,spobstrm,codec,spcodec);
	// read 3D_MOBL_start_code
	if(!sp_sc_enable) {
		if(vg_sc_enable) {
			MPG_Global::sp_reset();
			MPG_Global::sp_store();
		}
	} else {
		sp_mobl_bits=totalbits;
		if(spostrm.putStartCode(MPG_Global::MOBL2_start_code)==false) 
			return false;
		spobstrm.init(); 
		if(spobstrm.putMBitInt(8,mobl_id)==false) return false;
		if(spobstrm.get_i() != 8) {
			totalbits += 32;
			restbits -= 32;
		} else {
			totalbits += 24;
			restbits -= 24;
		}
		spobstrm.flush();
		spcodec.start_encode();
		MPG_Global::sp_reset();
		
		spcodec.init_bits(totalbits, restbits);

		if(!vg_pt0_enable && vg_id_len>0)   
			spcodec.encodePT2_Zero(vg_id_len, sp_vg_id);
		spcodec.encodePT2_Zero(bdry_loopbits,sp_left_bloop_idx);
		spcodec.encodePT2_Zero(bdry_loopbits,sp_right_bloop_idx);
		spcodec.encode(sp_bdry_pred);
		
		MPG_Global::sp_store();
		totalbits=spcodec.tbits;
		restbits=spcodec.rbits;
		sp_mobl_bits=totalbits-sp_mobl_bits;
	}
	return true;
}

void MPG_PacketQ_C::sp_packetization(
					MPG_OutputStream& vgostrm,
					MPG_OutputStream& spostrm,
					MPG_OutputStream& ostrm, 
					MPG_OutputBitStream& vgobstrm,
					MPG_OutputBitStream& spobstrm,
					MPG_OutputBitStream& obstrm,
					MPG_QFac& vgcodec, 
					MPG_QFac& spcodec, 
					MPG_QFac& codec,
					int icc,
					int nCC,
					int &next,
					int Gdepth,
					int &sptbits,
					int &sprbits,
					MPG_BranchTree*& cur_bt,	
					MPG_BranchTree*& tmp_bt,	
					MPG_ER2& ccbit)
{
	int lastComponent; //, rbytes=0;
	
	switch(erMode) {
	case 0:  
		lastComponent = (icc==nCC-1) ? 1: 0;
		spcodec.encode(lastComponent,MPG_Global::last_component);
		if(lastComponent) {
			spcodec.cbits=0;
			spcodec.finish_encode();
		};
		writeStream(spostrm,ostrm);
		updateQFac(spobstrm,obstrm,spcodec,codec);
		MPG_Global::sp_store();
		totalbits=spcodec.tbits;
		restbits=spcodec.rbits;
		vg_sc_enable=0;
		next=1; 
		break;
	case 1: 
		if(sp_pass0) { 
			//pt0 and pt2 without data partition
			int write_enable=1;
			if(sp_pt2_enable) {
				if(!sp_partitioning) {
					//pt2 without data partition
					spcodec.finish_encode();
					if(ccbit.next > icc+1) {
						if(spcodec.rbits < 24+bdry_loopbits*2+next_spbits) {  
//							rbytes=stuffing(spostrm,spobstrm,spcodec);
							spobstrm.init();
							totalbits=0;
							restbits=packetSize;
							spcodec.tbits=totalbits;
							spcodec.rbits=restbits;
						}
					}
				} else {
					//only pt2 with data partition
					totalbits=sptbits;   
					restbits=sprbits;
					sp_pass0=0;
					next=0;  
					write_enable=0;  
					sp_partitioning=0;  
					spostrm.clearOStream();
					cur_bt = tmp_bt;	
				}
			} else {
				//only pt0
				if(icc+1==ccbit.next) {
					spcodec.encode(1, MPG_Global::last_component);
					spcodec.finish_encode();
				} else {
					spcodec.encode(0, MPG_Global::last_component);
				}
			}
			if(write_enable) {
				writeStream(spostrm,ostrm);
				updateQFac(spobstrm,obstrm,spcodec,codec);
				MPG_Global::sp_store();     
				if(sp_sc_enable) mobl_id++;  
				totalbits=spcodec.tbits;
				restbits=spcodec.rbits;
				sptbits=spcodec.tbits;  
				sprbits=spcodec.rbits;
				if(Gdepth==-1) 
					next=1;
				else  
					next=0;
			} 
		} else { //only pt2 with data partition
			spcodec.finish_encode();
			if(spcodec.rbits < 24+bdry_loopbits*2+next_spbits) { 
				// insert a sp with dummy bits into the current packet
//				rbytes=stuffing(spostrm,spobstrm,spcodec);
				spobstrm.init();
				totalbits=0;
				restbits=packetSize;
				spcodec.tbits=totalbits;
				spcodec.rbits=restbits;
			} else {
				// insert a sp into the current packet
				totalbits=spcodec.tbits;
				restbits=spcodec.rbits;
			}
			writeStream(spostrm,ostrm);
			updateQFac(spobstrm,obstrm,spcodec,codec);
			MPG_Global::sp_store();
			if(sp_sc_enable) mobl_id++;  
			sptbits=totalbits;  
			sprbits=restbits;
			sp_pass0=1;
			sp_partitioning=0;
			next=0;
		}; 
		break;
	default: break;
	}
				  
	if(erMode>0 && next && Gdepth==-1)
	  sp_vg_id++;
}

void MPG_PacketQ_C::writeStream(MPG_OutputStream& s1, 
					            MPG_OutputStream& d1) 
{
	d1.insertOStream(s1);     //ostrm <- tmpostrm
	s1.clearOStream();        //clear the tmpostrm._buff
}

void MPG_PacketQ_C::updateQFac(MPG_OutputBitStream& s1,
					     MPG_OutputBitStream& d1,
					     MPG_QFac& s2,
					     MPG_QFac& d2) 
{
	d1.updateOBitStream(s1);  //obstrm(_b,_i) <- tmpobstrm(_b,_i)
	d2.updateQF(s2);          //codec <- tmpcodec 
}
/*
// MJH 001218 : removed
// (this function added 0xff until the packet is full)
int MPG_PacketQ_C::stuffing(MPG_OutputStream& s1,
						 MPG_OutputBitStream& s2,
						 MPG_QFac& s3) 
{
	s2.init();
	int restbits = s3.rbits % packetSize;
    int rbytes = (restbits>0) ? restbits/8: 0;
//	for(int k=0; k<rbytes; k++) s1.putUChar(0xff);
		
	return rbytes;
}
*/

bool MPG_ER2::determine_PT(int icc, int nCC, MPG_PacketQ_C& codap, MPG_OutputStream& s1, 
						   MPG_OutputBitStream& s2, MPG_QFac& s3) 
{
	curr=next=icc;
	int pbits=0;

	partition_type=0;
	estimate_packet(0, nCC, codap, pbits); //estimate the total bits for pt0
	if(curr==next) { 
		partition_type=1;
		estimate_packet(1, nCC, codap, pbits); //estimate the total bits for pt1
		if(curr==next) {  //assign a new packet
//			int rbytes=codap.stuffing(s1,s2,s3);
			s2.init();
			codap.totalbits=0;
			int tmp = (getVGbits(next)+90)/codap.packetSize;
			if((getVGbits(next)+90) % codap.packetSize >= 0)
				tmp++;
			codap.restbits= codap.packetSize * tmp;
			s3.tbits=codap.totalbits;
			s3.rbits=codap.restbits;
			if(tmp==1) {  
				partition_type=0;
				estimate_packet(0, nCC, codap, pbits);
				if(curr==next) {
					estimate_packet(1, nCC, codap, pbits);
					partition_type=1;
				} 
			} else {
				estimate_packet(1, nCC, codap, pbits);
				partition_type=1;
			}
		}
	} 

	return true;
}

void MPG_ER2::estimate_packet(int type, int nCC, MPG_PacketQ_C& codap, int& pbits) 
{
	pbits=0;
	int tmp=0, pflag=1;
	do {  
		if(type==0)  //estimate the total bits for pt0
			tmp=getVGbits(next)+getTTbits(next)+getTDbits(next);
		else         //estimate the total bits for pt1
			tmp=getVGbits(next);
		pbits+=tmp;
		if(codap.restbits > pbits+90) 
			next++;
		else {
			pbits-=tmp; 
			pflag=0;
		}
	} while(pflag && (next < nCC)); 
}
