//-------------------------------------------------------------------------
// 
// Copyright (C) IBM Corporation & Samsung Electronics Co. LTD., 1996, 1997, 1998, 1999
// Licensed Materials, Program Property of IBM & Samsung.
// 
// This software module is an implementation of a part of one or more
// MPEG-4 tools as specified by the MPEG-4 standard (ISO/IEC 14496).
// ISO/IEC gives users of the MPEG-4 standard (ISO/IEC 14496) free
// license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4
// standard (ISO/IEC 14496).  Those intending to use this software module
// in hardware or software products are advised that its use may infringe
// existing patents.  The original developer of this software module and
// his/her company, the subsequent editors and their companies, and
// ISO/IEC have no liability for use of this software module or
// modifications thereof in an implementation.  Copyright is not released
// for non MPEG-4 standard (ISO/IEC 14496) conforming products.  IBM
// Corp. & Samsung Electronics Co. LTD retains full right to use the code 
// for his/her own purpose, assign or donate the code to a third party and
// to inhibit third parties from using the code for non MPEG-4 standard 
// (ISO/IEC 14496) conforming products. This copyright notice must be included
// in all copies or derivative works.
// 
//  AUTHORS:
//    Gabriel Taubin
//    Andre Gueziec
//    Claudio Silva
//    Ioana Martin
//  
//  IBM T. J. Watson Research Center
//  P.O.Box 704
//  Yorktown Heights, New York 10598
//  
//  BASED ON EARLIER IMPLEMENTATION OF
//  VRML COMPRESSED BINARY FORMAT PROPOSAL BY:
//    Gabriel Taubin
//    Bill Horn
//    Francis Lazarus
//
//  Functionality : Error Resilience
//
//  Sungjin Kim  (sunjin@sait.samsung.co.kr)
//  Mahn-Jin Han  (manjini@sait.samsung.co.kr)
//  Mun-Sup Song (mssong@sait.samsung.co.kr)  
//
//  Signal Processing Lab. Samsung A.I.T. 
//-------------------------------------------------------------------------


#include <stdio.h>
#include <stdlib.h>
#include "MPG_IfsEdit.h"

//---------------------------------------------------------------------------
//
//   MPG_IfsEdit::_duplicatePerVertexProperties
//
//---------------------------------------------------------------------------

bool MPG_IfsEdit::_duplicatePerVertexProperties(vector<int>& v_stitch)
{
  MPG_Ifs& ifs  = _ifsMesh.getIfs();

  int nV    = ifs.getNumberOfCoord();
  int nVnew = v_stitch.size();

  int i,j,k;

  // check for errors in v_stitch array
  for(i=0;i<v_stitch.size();i++)
    if(v_stitch[i]>=nV)
      return false;

  if(ifs.getCoordBinding()==PB_PER_VERTEX) // should always be true
    {
      vector<float>& coord = ifs.getCoord();
      vector<float>  newCoord;
      for(i=0;i<v_stitch.size();i++)
	if(v_stitch[i]>=0)
	  {
	    j = v_stitch[i];
	    for(k=0;k<3;k++)
	      newCoord.push_back(coord[3*j+k]);
	  }
      coord.swap(newCoord);
    }
  if(ifs.getNormalBinding()==PB_PER_VERTEX)
    {
      vector<int>& normalIndex = ifs.getNormalIndex();
      if(normalIndex.size()>0)
	{
	  // PB_PER_VERTEX_INDEXED
	  vector<int>  newNormalIndex;
	  for(i=0;i<v_stitch.size();i++)
	    if(v_stitch[i]>=0)
	      newNormalIndex.push_back(normalIndex[v_stitch[i]]);
	  normalIndex.swap(newNormalIndex);
	}
      else
	{
	  vector<float>& normal = ifs.getNormal();
	  vector<float>  newNormal;
	  for(i=0;i<v_stitch.size();i++)
	    if(v_stitch[i]>=0)
	      {
		j = v_stitch[i];
		for(k=0;k<3;k++)
		  newNormal.push_back(normal[3*j+k]);
	      }
	  normal.swap(newNormal);
	}
    }
  if(ifs.getColorBinding()==PB_PER_VERTEX)
    {
      vector<int>& colorIndex = ifs.getColorIndex();
      if(colorIndex.size()>0)
	{
	  // PB_PER_VERTEX_INDEXED
	  vector<int>  newColorIndex;
	  for(i=0;i<v_stitch.size();i++)
	    if(v_stitch[i]>=0)
	      newColorIndex.push_back(colorIndex[v_stitch[i]]);
	  colorIndex.swap(newColorIndex);
	}
      else
	{
	  vector<float>& color = ifs.getColor();
	  vector<float>  newColor;
	  for(i=0;i<v_stitch.size();i++)
	    if(v_stitch[i]>=0)
	      {
		j = v_stitch[i];
		for(k=0;k<3;k++)
		  newColor.push_back(color[3*j+k]);
	      }
	  color.swap(newColor);
	}
    }
  if(ifs.getTexCoordBinding()==PB_PER_VERTEX)
    {
      vector<int>& texCoordIndex = ifs.getTexCoordIndex();
      if(texCoordIndex.size()>0)
	{
	  // PB_PER_VERTEX_INDEXED
	  vector<int>  newTexCoordIndex;
	  for(i=0;i<v_stitch.size();i++)
	    if(v_stitch[i]>=0)
	      newTexCoordIndex.push_back(texCoordIndex[v_stitch[i]]);
	  texCoordIndex.swap(newTexCoordIndex);
	}
      else
	{
	  vector<float>& texCoord = ifs.getTexCoord();
	  vector<float>  newTexCoord;
	  for(i=0;i<v_stitch.size();i++)
	    if(v_stitch[i]>=0)
	      {
		j = v_stitch[i];
		for(k=0;k<2;k++)
		  newTexCoord.push_back(texCoord[2*j+k]);
	      }
	  texCoord.swap(newTexCoord);
	}
    }
  return true;
}

//---------------------------------------------------------------------------
//
//   MPG_IfsEdit::removeIsolatedVertices
//
//---------------------------------------------------------------------------

bool MPG_IfsEdit::removeIsolatedVertices()
{
  MPG_Ifs& ifs  = _ifsMesh.getIfs();
  int nV = ifs.getNumberOfCoord();
  vector<bool> used((size_t)nV,false);
  vector<int>& coordIndex = ifs.getCoordIndex();
  int i;
  for(i=0;i<coordIndex.size();i++)
    if(coordIndex[i]>=0)
      {
	if(coordIndex[i]>=nV) return false;
	used[coordIndex[i]] = true;
      }

  vector<int> mapsTo((size_t)nV,-1);
  vector<int> invMapsTo((size_t)nV,-1);
  int j;
  for(j=i=0;i<nV;i++)
    if(used[i]==true)
      {	invMapsTo[j] = i; mapsTo[i] = j++; }

  for(i=0;i<coordIndex.size();i++)
    if(coordIndex[i]>=0)
      coordIndex[i] = mapsTo[coordIndex[i]];

  return (j==nV)?true:_duplicatePerVertexProperties(invMapsTo);
}
