//-------------------------------------------------------------------------
// 
// Copyright (C) IBM Corporation & Samsung Electronics Co. LTD., 1996, 1997, 1998, 1999
// Licensed Materials, Program Property of IBM & Samsung.
// 
// This software module is an implementation of a part of one or more
// MPEG-4 tools as specified by the MPEG-4 standard (ISO/IEC 14496).
// ISO/IEC gives users of the MPEG-4 standard (ISO/IEC 14496) free
// license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4
// standard (ISO/IEC 14496).  Those intending to use this software module
// in hardware or software products are advised that its use may infringe
// existing patents.  The original developer of this software module and
// his/her company, the subsequent editors and their companies, and
// ISO/IEC have no liability for use of this software module or
// modifications thereof in an implementation.  Copyright is not released
// for non MPEG-4 standard (ISO/IEC 14496) conforming products.  IBM
// Corp. & Samsung Electronics Co. LTD retains full right to use the code 
// for his/her own purpose, assign or donate the code to a third party and
// to inhibit third parties from using the code for non MPEG-4 standard 
// (ISO/IEC 14496) conforming products. This copyright notice must be included
// in all copies or derivative works.
// 
//  AUTHORS:
//    Gabriel Taubin
//    Andre Gueziec
//    Claudio Silva
//    Ioana Martin
//  
//  IBM T. J. Watson Research Center
//  P.O.Box 704
//  Yorktown Heights, New York 10598
//  
//  BASED ON EARLIER IMPLEMENTATION OF
//  VRML COMPRESSED BINARY FORMAT PROPOSAL BY:
//    Gabriel Taubin
//    Bill Horn
//    Francis Lazarus
//
//  Functionality : Error Resilience
//
//  Sungjin Kim  (sunjin@sait.samsung.co.kr)
//  Mahn-Jin Han  (manjini@sait.samsung.co.kr)
//  Mun-Sup Song (mssong@sait.samsung.co.kr)  
//
//  Signal Processing Lab. Samsung A.I.T. 
//-------------------------------------------------------------------------


#include <stdio.h>
#include <stdlib.h>
#include "MPG_Ifs.h"

#define VRML_HEADER "#VRML V2.0 utf8"

//---------------------------------------------------------------------------
//
//   MPG_Ifs::write
//
//---------------------------------------------------------------------------

bool MPG_Ifs::write(char* fname)
{
  FILE* fp = fopen(fname,"w");
  if(fp==(FILE*)0) return false;

  fprintf(fp,"#VRML V2.0 utf8\n");
  fprintf(fp,"Shape {\n");
  fprintf(fp," appearance Appearance {\n");
  fprintf(fp,"  material Material {  # grey\n");
  fprintf(fp,"   ambientIntensity  0.5\n");
  fprintf(fp,"   diffuseColor      0.7 0.7 0.7\n");
  fprintf(fp,"  }\n");
  fprintf(fp," }\n");

  fprintf(fp," geometry IndexedFaceSet {\n");

  if(getCcw()   ==false) fprintf(fp,"  ccw FALSE\n");
  if(getConvex()==false) fprintf(fp,"  convex FALSE\n");
  if(getSolid() ==false) fprintf(fp,"  solid FALSE\n");
  float creaseAngle = getCreaseAngle();
  if(creaseAngle>0.0)    fprintf(fp,"  creaseAngle %8.4f\n",creaseAngle);

  vector<float>& coord           = getCoord();
  vector<int>&   coordIndex      = getCoordIndex();
  {
    int i;
    fprintf(fp,"  coord Coordinate {\n");
    fprintf(fp,"   point [\n");
    for(i=0;i<coord.size();i++)
      {
        fprintf(fp,"%8.4f ",coord[i]);
        if(i%3==2) fprintf(fp,"\n");
      }
    fprintf(fp,"    ]\n");
    fprintf(fp,"  }\n");
    fprintf(fp,"  coordIndex [\n");
    for(i=0;i<coordIndex.size();i++)
      {
		if(coordIndex[i] == -2)
        fprintf(fp,"%d ",0);
		else {
        fprintf(fp,"%d ",coordIndex[i]);
        if(coordIndex[i]<0) fprintf(fp,"\n");
		}	//MJH 062600 : added bracket here
      }
    fprintf(fp,"    ]\n");
  }

  // if(color.size()==0)
  //   NO_COLORS
  // else if(colorPerVertex==FALSE)
  //   if(colorIndex.size()>0)
  //     COLOR_PER_FACE_INDEXED;
  //     colorIndex.size()==getNumberOfFaces()
  //   else
  //     COLOR_PER_FACE;
  //     color.size()/3==getNumberOfFaces()
  // else // if(colorPerVertex==TRUE)
  //   if(colorIndex.size()>0)
  //     COLOR_PER_CORNER_INDEXED;
  //     colorIndex.size()==coordIndex.size()
  //   else
  //     COLOR_PER_VERTEX;
  //     color.size()/3==coord.size()/3

  bool           colorPerVertex  = getColorPerVertex();
  vector<float>& color           = getColor();
  vector<int>&   colorIndex      = getColorIndex();
  if(color.size()>0)
    {
      int i;
      fprintf(fp,"  colorPerVertex %s\n",
              (colorPerVertex==true)?"TRUE":"FALSE\n");

      fprintf(fp,"  color Color {\n");
      fprintf(fp,"   color [\n");
      for(i=0;i<color.size();i++)
        {
          float ci = color[i];
          if(ci<0.0) ci=0.0;
          if(ci>1.0) ci=1.0;
          fprintf(fp,"%8.4f ",ci);
          if(i%3==2) fprintf(fp,"\n");
        }
      fprintf(fp,"    ]\n");
      fprintf(fp,"  }\n");

      if(colorIndex.size()>0)
        {
          fprintf(fp,"  colorIndex [\n");
          for(i=0;i<colorIndex.size();i++)
            {
              fprintf(fp,"%d ",colorIndex[i]);
              if(colorIndex[i]<0) fprintf(fp,"\n");
            }
          fprintf(fp,"    ]\n");
        }
    }

  // if(normal.size()==0)
  //   NO_NORMALS
  // else if(normalPerVertex==FALSE)
  //   if(normalIndex.size()>0)
  //     NORMAL_PER_FACE_INDEXED;
  //     normalIndex.size()==getNumberOfFaces()
  //   else
  //     NORMAL_PER_FACE;
  //     normal.size()/3==getNumberOfFaces()
  // else // if(normalPerVertex==TRUE)
  //   if(normalIndex.size()>0)
  //     NORMAL_PER_CORNER_INDEXED;
  //     normalIndex.size()==coordIndex.size()
  //   else
  //     NORMAL_PER_VERTEX;
  //     normal.size()/3==coord.size()/3

  bool           normalPerVertex = getNormalPerVertex();
  vector<float>& normal          = getNormal();
  vector<int>&   normalIndex     = getNormalIndex();
  if(normal.size()>0)
    {
      int i;
      fprintf(fp,"  normalPerVertex %s\n",
              (normalPerVertex==true)?"TRUE":"FALSE\n");

      fprintf(fp,"  normal Normal {\n");
      fprintf(fp,"   vector [\n");
      for(i=0;i<normal.size();i++)
        {
          fprintf(fp,"%8.4f ",normal[i]);
          if(i%3==2) fprintf(fp,"\n");
        }
      fprintf(fp,"    ]\n");
      fprintf(fp,"  }\n");

      if(normalIndex.size()>0)
        {
          fprintf(fp,"  normalIndex [\n");
          for(i=0;i<normalIndex.size();i++)
            {
              fprintf(fp,"%d ",normalIndex[i]);
              if(normalIndex[i]<0) fprintf(fp,"\n");
            }
          fprintf(fp,"    ]\n");
        }
    }

  // if(texCoord.size()==0)
  //   NO_TEX_COORD
  // else if(texCoordIndex.size()>0)
  //   TEX_COORD_PER_CORNER_INDEXED;
  //   texCoordIndex.size()==coordIndex.size()
  // else
  //   TEX_COORD_PER_VERTEX;
  //   texCoord.size()/3==coord.size()/3

  vector<float>& texCoord        = getTexCoord();
  vector<int>&   texCoordIndex   = getTexCoordIndex();
  if(texCoord.size()>0)
    {
      int i;

      fprintf(fp,"  texCoord TextureCoordinate {\n");
      fprintf(fp,"   point [\n");
      for(i=0;i<texCoord.size();i++)
        {
          fprintf(fp,"%8.4f ",texCoord[i]);
          if(i%2==1) fprintf(fp,"\n");
        }
      fprintf(fp,"    ]\n");
      fprintf(fp,"  }\n");

      if(texCoordIndex.size()>0)
        {
          fprintf(fp,"  texCoordIndex [\n");
          for(i=0;i<texCoordIndex.size();i++)
            {
              fprintf(fp,"%d ",texCoordIndex[i]);
              if(texCoordIndex[i]<0) fprintf(fp,"\n");
            }
          fprintf(fp,"    ]\n");
        }
    }

  fprintf(fp," }\n");		// IndexedFaceSet
  fprintf(fp,"}\n");		// Shape
  fclose(fp);

  return true;
}
