//-------------------------------------------------------------------------
// 
// Copyright (C) IBM Corporation & Samsung Electronics Co. LTD., 1996, 1997, 1998, 1999
// Licensed Materials, Program Property of IBM & Samsung.
// 
// This software module is an implementation of a part of one or more
// MPEG-4 tools as specified by the MPEG-4 standard (ISO/IEC 14496).
// ISO/IEC gives users of the MPEG-4 standard (ISO/IEC 14496) free
// license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4
// standard (ISO/IEC 14496).  Those intending to use this software module
// in hardware or software products are advised that its use may infringe
// existing patents.  The original developer of this software module and
// his/her company, the subsequent editors and their companies, and
// ISO/IEC have no liability for use of this software module or
// modifications thereof in an implementation.  Copyright is not released
// for non MPEG-4 standard (ISO/IEC 14496) conforming products.  IBM
// Corp. & Samsung Electronics Co. LTD retains full right to use the code 
// for his/her own purpose, assign or donate the code to a third party and
// to inhibit third parties from using the code for non MPEG-4 standard 
// (ISO/IEC 14496) conforming products. This copyright notice must be included
// in all copies or derivative works.
// 
//  AUTHORS:
//    Gabriel Taubin
//    Andre Gueziec
//    Claudio Silva
//    Ioana Martin
//  
//  IBM T. J. Watson Research Center
//  P.O.Box 704
//  Yorktown Heights, New York 10598
//  
//  BASED ON EARLIER IMPLEMENTATION OF
//  VRML COMPRESSED BINARY FORMAT PROPOSAL BY:
//    Gabriel Taubin
//    Bill Horn
//    Francis Lazarus
//
//  Functionality : Error Resilience
//
//  Sungjin Kim  (sunjin@sait.samsung.co.kr)
//  Mahn-Jin Han  (manjini@sait.samsung.co.kr)
//  Mun-Sup Song (mssong@sait.samsung.co.kr)  
//
//  Signal Processing Lab. Samsung A.I.T. 
//-------------------------------------------------------------------------


#ifndef _MPG_Lex_h_
#define _MPG_Lex_h_

// implement with templates instead !!!

//
// used in conjunction with MPG_Heap to sort lexicographically
//
//  given vector<float> p; with p.size()==nP*nD
//
//  int i,j;
//  MPG_Lex::D = nD;
//  MPG_Heap h(MPG_Lex::LT_F,MPG_Lex::EQ_F,nP);
//  for(i=0;i<nP;i++)
//    h.add((void*)(p.begin()+i*nD));
//
//  float* pj = NULL;
//  for(j=0;j<nP;j++)
//    {
//      i  = h.delMin((void*)pj);
//      // here pj == &(p[nD*i])
//      ...
//
//    }
//

#ifndef __GNUC__
#include <bool.h>
#endif // __GNUC__

class MPG_Lex {
public:

  static int  D;

  // compare (float)pp and (float)qq as floats
  // D is ignored
  static bool LT(void* pp, void* qq);
  static bool EQ(void* pp, void* qq);


  // compare (float*)pp and (float*)qq as float arrays
  static bool LT_F(void* pp, void* qq);
  static bool EQ_F(void* pp, void* qq);

  // compare (int*)pp and (int*)qq as int arrays
  static bool LT_I(void* pp, void* qq);
  static bool EQ_I(void* pp, void* qq);

};

#endif // _MPG_Lex_h_

