//-------------------------------------------------------------------------
// 
// Copyright (C) IBM Corporation & Samsung Electronics Co. LTD., 1996, 1997, 1998, 1999
// Licensed Materials, Program Property of IBM & Samsung.
// 
// This software module is an implementation of a part of one or more
// MPEG-4 tools as specified by the MPEG-4 standard (ISO/IEC 14496).
// ISO/IEC gives users of the MPEG-4 standard (ISO/IEC 14496) free
// license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4
// standard (ISO/IEC 14496).  Those intending to use this software module
// in hardware or software products are advised that its use may infringe
// existing patents.  The original developer of this software module and
// his/her company, the subsequent editors and their companies, and
// ISO/IEC have no liability for use of this software module or
// modifications thereof in an implementation.  Copyright is not released
// for non MPEG-4 standard (ISO/IEC 14496) conforming products.  IBM
// Corp. & Samsung Electronics Co. LTD retains full right to use the code 
// for his/her own purpose, assign or donate the code to a third party and
// to inhibit third parties from using the code for non MPEG-4 standard 
// (ISO/IEC 14496) conforming products. This copyright notice must be included
// in all copies or derivative works.
// 
//  AUTHORS:
//    Gabriel Taubin
//    Andre Gueziec
//    Claudio Silva
//    Ioana Martin
//  
//  IBM T. J. Watson Research Center
//  P.O.Box 704
//  Yorktown Heights, New York 10598
//  
//  BASED ON EARLIER IMPLEMENTATION OF
//  VRML COMPRESSED BINARY FORMAT PROPOSAL BY:
//    Gabriel Taubin
//    Bill Horn
//    Francis Lazarus
//
//  Functionality : Error Resilience
//
//  Sungjin Kim  (sunjin@sait.samsung.co.kr)
//  Mahn-Jin Han  (manjini@sait.samsung.co.kr)
//  Mun-Sup Song (mssong@sait.samsung.co.kr)  
//
//  Signal Processing Lab. Samsung A.I.T. 
//-------------------------------------------------------------------------

#include "MPG_Property.h"

//---------------------------------------------------------------------------
//
//   MPG_Property::decode
//
//---------------------------------------------------------------------------

bool MPG_Property::decode(MPG_InputBitStream& ibstrm)
{
  bool success = false;
	
  // check for incompatible arguments
  if(_type==PT_NONE) return false;
	
  int dimValue = getDimensionValue();
	
  vector<float> qv;
	
  // bbox
  // if(bbox) 
  //   min[0..dimValue-1]
  //   size
  // 
  // quant
  // pred_type
  // if(pred_type==2) 
  //   nlambda
  //   for(i=1;i<nlambda;i++)
  //   lambda[i]
  // 
	
  bool bbox = false;
  if(ibstrm.getBit(bbox)==false) return false;
	
  bool bbox_quantization = (_type!=PT_NORMAL)?true:false;
  if(bbox_quantization)
	{
		// decode bbox
		float* minCorner   = new float[dimValue];
		memset(minCorner,'\0',dimValue*sizeof(float));
		float side         = 1.0;
		if(bbox)
		{
			int i;
			for(i=0;i<dimValue;i++)
			{ if(ibstrm.get32BitFloat(minCorner[i])==false) return false; }
			if(ibstrm.get32BitFloat(side)==false) return false;
		}
		
		// decode precision
		int precision = 0;
		if(ibstrm.getMBitInt(5,precision)==false) return false;
		int bitsPerCoord = precision;
		if(bitsPerCoord<=0) return false;
		
		if(_bbq==(MPG_BBoxQ*)0)
			_bbq = new MPG_BBoxQ(dimValue,bitsPerCoord);
		
		_bbq->setDimension(dimValue);
		_bbq->setBitsPerCoordinate(bitsPerCoord);
		_bbq->setLowerCorner(minCorner);
		_bbq->setSideLength(side);
		
		setBBoxQ(_bbq);
		
		// binding determines the prediction scheme
		int prediction = 0;
		if(ibstrm.getMBitInt(2,prediction)==false) return false;
		_prediction = (prediction==0)?PP_NONE:
		(prediction==1)?PP_TREE:
		(prediction==2)?PP_PARALLELOGRAM:
		PP_RESERVED;
		if(_prediction==PP_RESERVED) return false;
		
		// get lambdas
		int nLambda = 0;
		if(_prediction==PP_TREE ||_prediction==PP_PARALLELOGRAM)
		{
			if(ibstrm.getMBitInt(2,nLambda)==false) return false;
			if(nLambda==0) return false;
			if(_prediction==PP_PARALLELOGRAM && nLambda!=3) return false;
			
			_lambda.insert(_lambda.end(),(size_t)nLambda,0.0);
			
			// decode quantized lambdas
			int i;
			_lambda[nLambda-1] = 1.0;
			for(i=0;i<nLambda-1;i++)
			{
				int iLambda = 0;
				if(ibstrm.getMBitSInt(3+precision,iLambda)==false) return false;
				_lambda[i] = iLambda;
				_lambda[i] = ldexp(_lambda[i],-bitsPerCoord);
				_lambda[nLambda-1] -= _lambda[i];
			}
		}
		success = true;
	}
  else // if(normal quantization)
	{
		if(bbox) return false;
		
		// decode number of subdivision levels
		int subdivLevels = 0;
		if(ibstrm.getMBitInt(5,subdivLevels)==false) return false;
		if(subdivLevels<0) return false;
		
		int bitsPerNormal = 3+2*subdivLevels;
		
		if(_nrq==(MPG_NormalQ*)0)
			_nrq = new MPG_NormalQ(bitsPerNormal);
		_nrq->setBitsPerNormal(bitsPerNormal);
		
		// decode prediction type
		int prediction = 0;
		if(ibstrm.getMBitInt(2,prediction)==false) return false;
		if(prediction!=0) return false;
		_prediction = PP_NONE;	// no other prediction ??????
		
		success = true;
	}
	
  return success;
}
