//-------------------------------------------------------------------------
// 
// Copyright (C) IBM Corporation & Samsung Electronics Co. LTD., 1996, 1997, 1998, 1999
// Licensed Materials, Program Property of IBM & Samsung.
// 
// This software module is an implementation of a part of one or more
// MPEG-4 tools as specified by the MPEG-4 standard (ISO/IEC 14496).
// ISO/IEC gives users of the MPEG-4 standard (ISO/IEC 14496) free
// license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4
// standard (ISO/IEC 14496).  Those intending to use this software module
// in hardware or software products are advised that its use may infringe
// existing patents.  The original developer of this software module and
// his/her company, the subsequent editors and their companies, and
// ISO/IEC have no liability for use of this software module or
// modifications thereof in an implementation.  Copyright is not released
// for non MPEG-4 standard (ISO/IEC 14496) conforming products.  IBM
// Corp. & Samsung Electronics Co. LTD retains full right to use the code 
// for his/her own purpose, assign or donate the code to a third party and
// to inhibit third parties from using the code for non MPEG-4 standard 
// (ISO/IEC 14496) conforming products. This copyright notice must be included
// in all copies or derivative works.
// 
//  AUTHORS:
//    Gabriel Taubin
//    Andre Gueziec
//    Claudio Silva
//    Ioana Martin
//  
//  IBM T. J. Watson Research Center
//  P.O.Box 704
//  Yorktown Heights, New York 10598
//  
//  BASED ON EARLIER IMPLEMENTATION OF
//  VRML COMPRESSED BINARY FORMAT PROPOSAL BY:
//    Gabriel Taubin
//    Bill Horn
//    Francis Lazarus
//
//  Functionality : Error Resilience
//
//  Sungjin Kim  (sunjin@sait.samsung.co.kr)
//  Mahn-Jin Han  (manjini@sait.samsung.co.kr)
//  Mun-Sup Song (mssong@sait.samsung.co.kr)  
//
//  Signal Processing Lab. Samsung A.I.T. 
//-------------------------------------------------------------------------

#ifndef _MPG_QFac_H_
#define _MPG_QFac_H_

#include "MPG_Stream.h"
#include "MPG_QFpm.h"

// XXX: Interface still needs work with respect to counting all the
// bits that actually get outputed.

class MPG_QFac {
protected:

  unsigned             C;
  unsigned             A;
  unsigned             C_high;
  int                  count;
  int                  first;
  int                  bits;

  int                 _ostrm_bits;
  int                 _istrm_bits;
  MPG_OutputStream   *_ostrm;
  MPG_InputStream    *_istrm;

protected:

  int  get_byte();
  void put_byte(int);

  void hw_renorm_encode();
  void hw_renorm_decode();
  int sw_renorm_decode();

  void hw_start_encode();
  void hw_encode(int, MPG_QFpm &);
  void hw_encode(int);
  void hw_finish_encode();

  void hw_start_decode();
  int  hw_decode(MPG_QFpm &);
  void hw_finish_decode();

  void sw_start_decode();
  int  sw_decode(MPG_QFpm &);
  int  sw_decode();
  void sw_finish_decode();

public:
	int rbits;  //#remain bits per a packet 
	int cbits;  //#current bits per a packet
	int tbits;  //#total bits per a packet 
	int erMode;
	
	int getcount() { return count; } 
	
	void init_bits(int a, int b) {
		tbits=a;
		rbits=b; 
		cbits=0; 
	} 
	
	inline bool Bitcnt() {
		cbits++;
		tbits++;
		if(erMode) rbits--;
		return true;
	}
	
	void updateQF(MPG_QFac &a) {
		C = a.C;
		A = a.A;
		C_high = a.C_high;
		count = a.count;
		first = a.first;
		bits = a.bits;
		
		tbits = a.tbits;
		rbits = a.rbits;
		cbits = a.cbits;
	}
	
	void encodePT2(int len, int val, MPG_QFpm &qfPM) {
		int i, bit=0;
		for(i=0; i<len; i++) {
			bit = (val >> (len-i-1)) & 1;
			encode(bit, qfPM);
		} 
	}

	void encodePT2_Zero(int len, int val) {
		int i, bit=0;
		for(i=0; i<len; i++) {
			bit = (val >> (len-i-1)) & 1;
			encode(bit);
		} 
	}

  // construct encoder
  inline MPG_QFac(MPG_OutputStream &o, int e)  
    : _istrm(NULL), _ostrm(&o), _ostrm_bits(0), _istrm_bits(0)
  { erMode = e; }  

  // construct encoder
  inline MPG_QFac(MPG_InputStream &i)
    : _istrm(&i), _ostrm(NULL), _ostrm_bits(0), _istrm_bits(0)
  {}

  inline ~MPG_QFac()
  {}

  inline int getOBits()
  { return _ostrm_bits; }

  inline int getIBits()
  { return _istrm_bits-32; }

  inline void encode(int bit)
  { bits = 0; hw_encode(bit); }


  inline void encode(int bit, MPG_QFpm &qfPM)
  { bits = 0; hw_encode(bit, qfPM); }

  inline int decode()
  { return sw_decode(); }

  inline int decode(MPG_QFpm &qfPM)
  { return sw_decode(qfPM); }

  void start_encode(void)
  { hw_start_encode(); }

  void finish_encode(void)
  { hw_finish_encode(); }

  void start_decode(void)
  { sw_start_decode(); }

  void finish_decode(void)
  { sw_finish_decode(); }
};

#endif
