//-------------------------------------------------------------------------
// 
// Copyright (C) IBM Corporation & Samsung Electronics Co. LTD., 1996, 1997, 1998, 1999
// Licensed Materials, Program Property of IBM & Samsung.
// 
// This software module is an implementation of a part of one or more
// MPEG-4 tools as specified by the MPEG-4 standard (ISO/IEC 14496).
// ISO/IEC gives users of the MPEG-4 standard (ISO/IEC 14496) free
// license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4
// standard (ISO/IEC 14496).  Those intending to use this software module
// in hardware or software products are advised that its use may infringe
// existing patents.  The original developer of this software module and
// his/her company, the subsequent editors and their companies, and
// ISO/IEC have no liability for use of this software module or
// modifications thereof in an implementation.  Copyright is not released
// for non MPEG-4 standard (ISO/IEC 14496) conforming products.  IBM
// Corp. & Samsung Electronics Co. LTD retains full right to use the code 
// for his/her own purpose, assign or donate the code to a third party and
// to inhibit third parties from using the code for non MPEG-4 standard 
// (ISO/IEC 14496) conforming products. This copyright notice must be included
// in all copies or derivative works.
// 
//  AUTHORS:
//    Gabriel Taubin
//    Andre Gueziec
//    Claudio Silva
//    Ioana Martin
//  
//  IBM T. J. Watson Research Center
//  P.O.Box 704
//  Yorktown Heights, New York 10598
//  
//  BASED ON EARLIER IMPLEMENTATION OF
//  VRML COMPRESSED BINARY FORMAT PROPOSAL BY:
//    Gabriel Taubin
//    Bill Horn
//    Francis Lazarus
//
//  Functionality : Error Resilience
//
//  Sungjin Kim  (sunjin@sait.samsung.co.kr)
//  Mahn-Jin Han  (manjini@sait.samsung.co.kr)
//  Mun-Sup Song (mssong@sait.samsung.co.kr)  
//
//  Signal Processing Lab. Samsung A.I.T. 
//-------------------------------------------------------------------------


#ifndef _MPG_Stitcher_h_
#define _MPG_Stitcher_h_

#include <vector>
#include "MPG_Stitches.h"

using namespace std;	//MJH 010911

class MPG_Stitcher {

private:
  
  // the following vectors are persistent 
  vector <int>  _s_length ;
  vector <int>  _stitch_to;
  vector <bool> _rev_stitch;

  vector <int>  _v_cluster;  // clustering without re-enumerating clustered vertices

  int           _nClustered; // number of vertices after clustering 

public:

  inline MPG_Stitcher();

  ~MPG_Stitcher()
  {}

  // go from labels to stitches 
  bool represent(// inputs
		 const vector<int>&      nVcc, // vertices per conn, component
		 const vector<int>&      v_father,
		       vector<int>&      v_cluster,
		 // output
		 MPG_Stitches&           v_stitch);
  
  // go from stitches to cluster
  bool use(// inputs
	   int first_vertex,
	   const vector<int>&      v_father,
	   MPG_Stitches&           v_stitch,
	   // output
	   vector<int>&            v_cluster);

};

inline MPG_Stitcher::MPG_Stitcher():_nClustered(0){
    _s_length.erase(_s_length.begin(),_s_length.end());
    _stitch_to.erase(_stitch_to.begin(),_stitch_to.end());
    _rev_stitch.erase(_rev_stitch.begin(),_rev_stitch.end());
  }


#endif // _MPG_Stitcher_h_

