// FixCoordInterp.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include "FixCoordInterp.h"
//#include <stdio.h>
//#include <stdlib.h>
//#include <string.h>

FILE* logfp     = stderr;

#define not(b) ((b)==true)?false:true


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// The one and only application object

CWinApp theApp;

using namespace std;

void usage()
{
  static char *mm[] = {
    " -----------------------------------------------------------------------",
    "  USAGE: FixCI [options] indexFileName baseFileName ",
    "",
    "    -h           for help",
    "    -id<inDir>   tries to read input files from 'inDir' directory",
    "    -od<outDir>  places output files in 'outDir' directory",
    " -----------------------------------------------------------------------",
    NULL
  };
  for(char **m=mm;*m;m++)
    printf("%s\n",*m);
  exit(1);
}

void error(const char *msg)
{
  fprintf(logfp,"error : %s\n",(msg)?msg:"");
  exit(-1);
}


int _tmain(int argc, TCHAR* argv[], TCHAR* envp[])
{
	int nRetCode = 0;

	// initialize MFC and print and error on failure
	if (!AfxWinInit(::GetModuleHandle(NULL), NULL, ::GetCommandLine(), 0))
	{
		// TODO: change error code to suit your needs
		cerr << _T("Fatal Error: MFC initialization failed") << endl;
		nRetCode = 1;
	}
	else
	{
		// TODO: code your application's behavior here.

		char* baseFname = NULL;
		char* inExt     = NULL;
		char* inDir     = NULL;
		char* outDir    = NULL;
		char* inFname   = NULL;
		char* outFname  = NULL;
		char* indexFname= NULL;

		int i,j;
		if(argc==1)
			usage();
		else for(i=1;i<argc;i++)
		{
			if(!strcmp(argv[i],"-h") || !strcmp(argv[i],"-help"))
				usage();
			else if(!strncmp(argv[i],"-id",3))
				inDir = strdup(argv[i]+3);
			else if(!strncmp(argv[i],"-od",3))
				outDir = strdup(argv[i]+3);
			else if(argv[i][0]=='-')
				error("unknown option");
			else if(!indexFname)
				indexFname = strdup(argv[i]);
			else if(!baseFname)
				baseFname = strdup(argv[i]);
			else
				error("too many input files");
		}
		
		if(!indexFname)
			error("no index file");
		if(!baseFname)
			error("no input file");
		
		for(j=strlen(baseFname)-1;j>=0;j--)
			if(baseFname[j]=='/') 
			{ baseFname[j]='\0'; j++; break; }
		if(!inDir)
			inDir = (j>=0)?baseFname:strdup(".");
		else if(j>0 && strcmp(inDir,baseFname)!=0)
			error("inDir");
		if(j>0) baseFname += j;
		for(j=strlen(baseFname)-1;j>0;j--)
			if(baseFname[j]=='.') 
			{ baseFname[j]='\0'; inExt = baseFname+j+1; break; }
		if(!baseFname)
			error("no input file");
		if(!outDir) outDir = strdup(inDir);
		if(!inExt)  inExt  = strdup("");
				
		// read from index file
		int n = strlen(inDir)+strlen(indexFname)+9;
		char *fullIndexFname = new char[n];
		if(!strcmp(inDir,"."))
			sprintf(fullIndexFname,"%s",indexFname);
		else
			sprintf(fullIndexFname,"%s\\%s",inDir,indexFname);

		int	vertNum;
		int *oldIndex;

		FILE* fp = fopen(fullIndexFname,"r");
		if(fp!=(FILE*)0)
		{
			fscanf(fp,"%d",&vertNum);
			oldIndex = new int[vertNum];
			for (i=0; i<vertNum; i++)
				fscanf(fp,"%d",oldIndex+i);
		}
		else
			error("Error reading Index File");

		fclose(fp);
		delete [] fullIndexFname;

		// read from input file
		int		frameNum;
		float	*ci;
		char	tmpchar;

		n = strlen(inDir)+strlen(baseFname)+9;
		inFname = new char[n];
		if(!strcmp(inDir,"."))
			sprintf(inFname,"%s.ci",baseFname);
		else
			sprintf(inFname,"%s\\%s.ci",inDir,baseFname);
		
		fp = fopen(inFname,"r");
		if(fp!=(FILE*)0)
		{
			fscanf(fp,"%d",&frameNum);
			do {
				i = fscanf(fp,"%c",&tmpchar);
			} while (tmpchar != '[' && i != EOF);
			if (i==EOF)
				error("Error while reading Input file : no starting bracket");

			ci = new float[3*vertNum*frameNum];
			for (i=0; i<3*vertNum*frameNum; i=i+3) {
				fscanf(fp,"%f",ci+i);
				fscanf(fp,"%f",ci+i+1);
				fscanf(fp,"%f",ci+i+2);
				do {
					fscanf(fp,"%c",&tmpchar);
				} while (tmpchar != ',' && tmpchar !='\n');
			}

			do {
				i = fscanf(fp,"%c",&tmpchar);
			} while (tmpchar != ']' && i != 0);
			if (i==EOF)
				error("Error while reading Input file : no ending bracket");
		}
		else
			error("Error reading Index File");

		fclose(fp);
		delete [] inFname;

		//write to output file
		float	*newci = new float[3*vertNum*frameNum];

		int curpos, oldpos;
		for (i=0; i<frameNum; i++)
			for (j=0; j<vertNum; j++)
			{
				curpos = 3*((i*vertNum)+j);
				oldpos = 3*((i*vertNum)+oldIndex[j]);
				newci[curpos] = ci[oldpos];
				newci[curpos+1] = ci[oldpos+1];
				newci[curpos+2] = ci[oldpos+2];
			}

		n = strlen(outDir)+strlen(baseFname)+8;
		outFname = new char[n];
		if(!strcmp(outDir,"."))
			sprintf(outFname,"%s.nci",baseFname);
		else
			sprintf(outFname,"%s/%s.nci",outDir,baseFname);
		
		fp = fopen(outFname,"w");
		if(fp!=(FILE*)0)
		{
			fprintf(fp, "[\n");
			for (i=0; i<3*vertNum*frameNum; i=i+3) {
//				fprintf(fp,"%f %f,\n", newci[i], newci[i+1]);
				fprintf(fp,"%f %f %f,\n", newci[i], newci[i+1], newci[i+2]);
			}
			fprintf(fp, "]\n");
		}
		else
			error("Error reading Index File");

		fclose(fp);
		delete [] outFname;

		delete [] oldIndex;
		delete [] ci;
		delete [] newci;
		printf("done!!!\n");
	}

	return nRetCode;
}


