//-------------------------------------------------------------------------
// 
// Copyright (C) IBM Corporation & Samsung Electronics Co. LTD., 1996, 1997, 1998, 1999
// Licensed Materials, Program Property of IBM & Samsung.
// 
// This software module is an implementation of a part of one or more
// MPEG-4 tools as specified by the MPEG-4 standard (ISO/IEC 14496).
// ISO/IEC gives users of the MPEG-4 standard (ISO/IEC 14496) free
// license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4
// standard (ISO/IEC 14496).  Those intending to use this software module
// in hardware or software products are advised that its use may infringe
// existing patents.  The original developer of this software module and
// his/her company, the subsequent editors and their companies, and
// ISO/IEC have no liability for use of this software module or
// modifications thereof in an implementation.  Copyright is not released
// for non MPEG-4 standard (ISO/IEC 14496) conforming products.  IBM
// Corp. & Samsung Electronics Co. LTD retains full right to use the code 
// for his/her own purpose, assign or donate the code to a third party and
// to inhibit third parties from using the code for non MPEG-4 standard 
// (ISO/IEC 14496) conforming products. This copyright notice must be included
// in all copies or derivative works.
// 
//  AUTHORS:
//    Gabriel Taubin
//    Andre Gueziec
//    Claudio Silva
//    Ioana Martin
//  
//  IBM T. J. Watson Research Center
//  P.O.Box 704
//  Yorktown Heights, New York 10598
//  
//  BASED ON EARLIER IMPLEMENTATION OF
//  VRML COMPRESSED BINARY FORMAT PROPOSAL BY:
//    Gabriel Taubin
//    Bill Horn
//    Francis Lazarus
//
//  Functionality : Error Resilience
//
//  Sungjin Kim  (sunjin@sait.samsung.co.kr)
//  Mahn-Jin Han  (manjini@sait.samsung.co.kr)
//  Mun-Sup Song (mssong@sait.samsung.co.kr)  
//
//  Signal Processing Lab. Samsung A.I.T. 
//-------------------------------------------------------------------------

#ifndef _MPG_Loops_h_
#define _MPG_Loops_h_

#include <vector>

using namespace std;	//MJH 010911

class MPG_Loops {			// single linked list

protected:
  vector<int> _value;
  vector<int> _next;
  vector<int> _stack;
  vector<int> _start;
  int         _curr;

public:
  ~MPG_Loops() {}
   MPG_Loops() {}

  MPG_Loops(int ivR, int ivL)
  { init(ivR,ivL); }

  void insert(int ivO)
  {
    int L = _curr;
    int O = _next.size();
    _next.push_back(_next[_curr]);
    _next[L] = O;
    _value.push_back(ivO);
  }

  void start(int ivR)
  {
    int i = _value.size();
    // [i  ] : ivR -> [i]
    _value.push_back(ivR);
    _next.push_back(i);
    _curr = i;
    _start.push_back(i);
  }

  void start(int ivR, int ivL)
  {
    int i = _value.size();
    // [i  ] : ivR -> [i+1]
    // [i+1] : ivL -> [i  ]
    _value.push_back(ivR);
    _next.push_back(i+1);
    _value.push_back(ivL);
    _next.push_back(i);
    _curr = i+1;
    _start.push_back(i);
  }

  void clear()
  {
    _value.erase(_value.begin(),_value.end());
    _next.erase(_next.begin(),_next.end());
    _stack.erase(_stack.begin(),_stack.end());
    _start.erase(_start.begin(),_start.end());
  }

  void init(int ivR)
  {
    // [0] : ivR -> [0]
    clear();
    start(ivR);
  }

  void init(int ivR, int ivL)
  {
    // [0] : ivR -> [1]
    // [1] : ivL -> [0]
    clear();
    start(ivR,ivL);
  }

  int  getNumberOfLoops()
  { return _start.size(); }
  int  getLoopStart(int iLoop)
  { return (iLoop>=0&&iLoop<_start.size())?_start[iLoop]:-1; }
  int  getLoop(int iLoop, vector<int>& indx)
  {
    int nLoop = 0;
    if(iLoop>=0&&iLoop<_start.size())
      {
	int i = _start[iLoop];
	do {
	  indx.push_back(_value[i]); i = _next[i]; nLoop++;
	} while (i!=_start[iLoop]);
      }
    return nLoop;
  }
  void getAllLoops(vector<int>& face, vector<int>& indx)
  {
    face.erase(face.begin(),face.end());
    indx.erase(indx.begin(),indx.end());

    int nTotal = 0;
    face.push_back(nTotal);

    int nLoops = getNumberOfLoops();
    for(int iLoop=0;iLoop<nLoops;iLoop++)
      {
	nTotal += getLoop(iLoop,indx);
	face.push_back(nTotal);
      }
  }

  int  getCurrValue()
  { return _value[_curr]; }
  int  getNextValue()
  { return _value[_next[_curr]]; }

  int  getCurr()
  { return _curr; }
  int  getNext()
  { return _next[_curr]; }
  void setCurr(int loopCurr)
  { _curr = loopCurr; }
  void advance()
  { _curr = _next[_curr]; }
  int size()
  { return _value.size(); }
  void pushCurr()
  { _stack.push_back(_curr); }
  void pushNext()
  { _stack.push_back(_next[_curr]); }
  bool pop(int& valR, int& valL)
  {
    if(_stack.size()>0)
      {
	_curr = _stack.back(); _stack.pop_back();
	valL = _value[_curr];
	valR = _value[_next[_curr]];
	return true;
      }
    return false;
  }

  bool isNext(int curr, int next)
  { return (curr>=0&&curr<_next.size()&&_next[curr]==next)?true:false; }

  bool canReach(int to, int from)
  {
    int curr = from;
    do {
      if(curr==to) return true;
      curr = _next[curr];
    } while(curr!=from);
    return false;
  }

  void invert(int from)
  {
    int prev, curr, next;

    prev = from; curr = _next[prev];
    do {
      next = _next[curr]; _next[curr] = prev; prev = curr; curr = next;
    } while(curr!=from);
    _next[from] = prev;
  }

  bool swap(int from, int to, int invOr)
  {
    if(from<0 || from>=_next.size() || to<0 || to>=_next.size() || from==to)
      return false;
    else if(!invOr)
      {
	// if(canReach(to,from)==true)
	// loop is cut into two loops
	// --> need to push an index into the _stack() !!!
	int next_to = _next[to];
	_next[to] = _next[from];
	_next[from] = next_to;
	return true;
      }
    else
      {
	// it 'to' and 'from' are in the same loop
	// cut the loop into two loops
	if(canReach(to,from)==true)
	  swap(from,to,false);
	// now 'to' and 'from' are in different loops
	// 'to' loop is inverted
	// (it would be more efficient to mantain the length
	//  of the loops and invert the shortest one)
	to = _next[to];
	invert(to);
	// and both loops joined into a single one
	// swap(from,to,false);
	int next_to = _next[to];
	_next[to] = _next[from];
	_next[from] = next_to;
	// --> need to remove an index from the _stack() !!!
	return true;
      }
  }

  void mapTo(vector<int>& map)
  {
    for(int i=0;i<_value.size();i++)
      {
	int vi = _value[i];
	_value[i] = map[vi];
      }
  }
};

typedef MPG_Loops* MPG_pLoop;
#ifndef __GNUC__
// to make the compiler happy
inline void destroy(MPG_Loops* pointer) {}
inline void destroy(MPG_pLoop* pointer) {}
#endif // __GNUC__

#endif
