//-------------------------------------------------------------------------
// 
// Copyright (C) IBM Corporation & Samsung Electronics Co. LTD., 1996, 1997, 1998, 1999
// Licensed Materials, Program Property of IBM & Samsung.
// 
// This software module is an implementation of a part of one or more
// MPEG-4 tools as specified by the MPEG-4 standard (ISO/IEC 14496).
// ISO/IEC gives users of the MPEG-4 standard (ISO/IEC 14496) free
// license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4
// standard (ISO/IEC 14496).  Those intending to use this software module
// in hardware or software products are advised that its use may infringe
// existing patents.  The original developer of this software module and
// his/her company, the subsequent editors and their companies, and
// ISO/IEC have no liability for use of this software module or
// modifications thereof in an implementation.  Copyright is not released
// for non MPEG-4 standard (ISO/IEC 14496) conforming products.  IBM
// Corp. & Samsung Electronics Co. LTD retains full right to use the code 
// for his/her own purpose, assign or donate the code to a third party and
// to inhibit third parties from using the code for non MPEG-4 standard 
// (ISO/IEC 14496) conforming products. This copyright notice must be included
// in all copies or derivative works.
// 
//  AUTHORS:
//    Gabriel Taubin
//    Andre Gueziec
//    Claudio Silva
//    Ioana Martin
//  
//  IBM T. J. Watson Research Center
//  P.O.Box 704
//  Yorktown Heights, New York 10598
//  
//  BASED ON EARLIER IMPLEMENTATION OF
//  VRML COMPRESSED BINARY FORMAT PROPOSAL BY:
//    Gabriel Taubin
//    Bill Horn
//    Francis Lazarus
//
//  Functionality : Error Resilience
//
//  Sungjin Kim  (sunjin@sait.samsung.co.kr)
//  Mahn-Jin Han  (manjini@sait.samsung.co.kr)
//  Mun-Sup Song (mssong@sait.samsung.co.kr)  
//
//  Signal Processing Lab. Samsung A.I.T. 
//-------------------------------------------------------------------------


#ifndef _MPG_MeshFs_h_
#define _MPG_MeshFs_h_

#include "MPG_Mesh.h"

typedef vector<int>* MPG_pVecInt;
typedef vector<MPG_pVecInt> vector_pVecInt;

class MPG_MeshFs : public MPG_Mesh {

protected:

        bool _isForest(vector<bool>& emk, MPG_Partition& vp);
        void _buildLoop(vector<bool>& e_mark,
			vector<bool>& v_is_bdry,
			vector<int>& v_order,
			int nVtree,
			int rootV, MPG_Graph::Edge& rootE, int rootF,
			vector<int>& loop,
			vector<bool>& v_is_free,
			vector<int>& v_old,
			vector<int>& new_corner);

  static vector<int> _emptyCoordIndex;

public:

  inline       MPG_MeshFs(vector<int>& coordIndex);
               MPG_MeshFs(vector<int>& coordIndex0, vector<int>& map,
                          vector<int>& coordIndex1);
  inline      ~MPG_MeshFs();

         bool  representForestSplit(vector<int>&           map,
                                    vector<bool>&          emk2,
                                    vector<MPG_pSPolygon>& vp_sp,
                                    MPG_Property&          pCoord,
                                    MPG_Property&          pNormal,
                                    MPG_Property&          pColor,
                                    MPG_Property&          pTexCoord,
                                    vector<int>&           qCoord,
                                    vector<int>&           qNormal,
                                    vector<int>&           qColor,
                                    vector<int>&           qTexCoord,
                                    vector<int>&           triData,
                                    vector<int>&           v_traversal,
                                    vector<int>&           t_traversal,
                                    vector<MPG_pVecInt>&   vp_v_loop,
                                    vector<MPG_pVecInt>&   vp_vcf_loop);


         bool  decodeForestSplit(vector<bool>& edge_is_in_forest,
			 vector<MPG_pVecInt>&   vp_coordIndex,
			 vector<int>&           cmap,
			 vector<MPG_pVecInt>&   vp_vcf_loop);
  

  // vertex clustering
         bool  isValidMap(vector<int>& map);
         bool  isCollapsedFace(int i, vector<int>& map);
  inline void  partitionCollapsedFaces(vector<int>& map,
				       vector<int>& f_remaining,
				       vector<int>& f_collapsed);
         bool  permuteFaces(vector<int>& f_traversal);
         bool  isConnectivityPreservingMap(vector<int>& map);
};

inline MPG_MeshFs::MPG_MeshFs(vector<int>& coordIndex):
  MPG_Mesh(coordIndex)
{ }

inline MPG_MeshFs::~MPG_MeshFs()
{ }

inline void MPG_MeshFs::partitionCollapsedFaces(vector<int> &map,
						vector<int>& f_remaining,
						vector<int>& f_collapsed)
{
  f_remaining.erase(f_remaining.begin(),f_remaining.end());
  f_collapsed.erase(f_collapsed.begin(),f_collapsed.end());
  if(isValidMap(map)==true)
    for(int i=0;i<getNumberOfFaces();i++)
      if(isCollapsedFace(i,map)==true)
	f_collapsed.push_back(i);
      else
	f_remaining.push_back(i);
}

#endif // _MPG_MeshFs_h_
