//-------------------------------------------------------------------------
// 
// Copyright (C) IBM Corporation & Samsung Electronics Co. LTD., 1996, 1997, 1998, 1999
// Licensed Materials, Program Property of IBM & Samsung.
// 
// This software module is an implementation of a part of one or more
// MPEG-4 tools as specified by the MPEG-4 standard (ISO/IEC 14496).
// ISO/IEC gives users of the MPEG-4 standard (ISO/IEC 14496) free
// license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4
// standard (ISO/IEC 14496).  Those intending to use this software module
// in hardware or software products are advised that its use may infringe
// existing patents.  The original developer of this software module and
// his/her company, the subsequent editors and their companies, and
// ISO/IEC have no liability for use of this software module or
// modifications thereof in an implementation.  Copyright is not released
// for non MPEG-4 standard (ISO/IEC 14496) conforming products.  IBM
// Corp. & Samsung Electronics Co. LTD retains full right to use the code 
// for his/her own purpose, assign or donate the code to a third party and
// to inhibit third parties from using the code for non MPEG-4 standard 
// (ISO/IEC 14496) conforming products. This copyright notice must be included
// in all copies or derivative works.
// 
//  AUTHORS:
//    Gabriel Taubin
//    Andre Gueziec
//    Claudio Silva
//    Ioana Martin
//  
//  IBM T. J. Watson Research Center
//  P.O.Box 704
//  Yorktown Heights, New York 10598
//  
//  BASED ON EARLIER IMPLEMENTATION OF
//  VRML COMPRESSED BINARY FORMAT PROPOSAL BY:
//    Gabriel Taubin
//    Bill Horn
//    Francis Lazarus
//
//  Functionality : Error Resilience
//
//  Sungjin Kim  (sunjin@sait.samsung.co.kr)
//  Mahn-Jin Han  (manjini@sait.samsung.co.kr)
//  Mun-Sup Song (mssong@sait.samsung.co.kr)  
//
//  Signal Processing Lab. Samsung A.I.T. 
//-------------------------------------------------------------------------

#ifndef _MPG_Partition_h_
#define _MPG_Partition_h_

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

class MPG_Partition {
 private:

  void _zero() { memset((char*)this,'\0',sizeof(MPG_Partition)); }
  void _init(int n, int *father=(int*)0, int *size=(int*)0, int *order=(int*)0);

  MPG_Partition& operator= (const MPG_Partition& rhs);
  MPG_Partition (const MPG_Partition& x);

 public:
  ~MPG_Partition()
    {
      if(!_ext_arrays) delete [] _father;
      delete [] _size;
      if(_partFirst) delete [] _partFirst;
      _zero();
    }
  MPG_Partition(int n, int *order=(int*)0):_ext_arrays(0)
    { _init(n,(int*)0,(int*)0,order);}
  MPG_Partition(int n, int *father, int *size):_ext_arrays(1)
    { _init(n,father,size); }
  MPG_Partition(int n, int *father, int *size, int *order):_ext_arrays(1)
    { _init(n,father,size,order); }

  void  restart(int n=0);
  int   find(int i);
  int   join(int i, int j);
  int   join(int i, int j, int *order);
  int   size(int i) const { return _size[i]; }
  int   father(int i) { return(_father[i]); }  
  const int parts()   { return(_parts); }
  const int n()       { return(_n); }

  // gt @ Mon Dec 15 1997
  void  end();
  // gt @ Wed Dec 17 1997
  void  makeParts();
  // gt @ Wed Dec 17 1997
  void  getPart(int i, int& n, int*& part);

private:
  const int _ext_arrays;

  int _n_max;
  int _n;
  int _parts;
  int *_father;
  int *_size;
  int *_order;

  // gt @ Wed Dec 17 1997
  int *_partFirst;
  int *_partIndex;
};

#endif
