//-------------------------------------------------------------------------
// 
// Copyright (C) IBM Corporation & Samsung Electronics Co. LTD., 1996, 1997, 1998, 1999
// Licensed Materials, Program Property of IBM & Samsung.
// 
// This software module is an implementation of a part of one or more
// MPEG-4 tools as specified by the MPEG-4 standard (ISO/IEC 14496).
// ISO/IEC gives users of the MPEG-4 standard (ISO/IEC 14496) free
// license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4
// standard (ISO/IEC 14496).  Those intending to use this software module
// in hardware or software products are advised that its use may infringe
// existing patents.  The original developer of this software module and
// his/her company, the subsequent editors and their companies, and
// ISO/IEC have no liability for use of this software module or
// modifications thereof in an implementation.  Copyright is not released
// for non MPEG-4 standard (ISO/IEC 14496) conforming products.  IBM
// Corp. & Samsung Electronics Co. LTD retains full right to use the code 
// for his/her own purpose, assign or donate the code to a third party and
// to inhibit third parties from using the code for non MPEG-4 standard 
// (ISO/IEC 14496) conforming products. This copyright notice must be included
// in all copies or derivative works.
// 
//  AUTHORS:
//    Gabriel Taubin
//    Andre Gueziec
//    Claudio Silva
//    Ioana Martin
//  
//  IBM T. J. Watson Research Center
//  P.O.Box 704
//  Yorktown Heights, New York 10598
//  
//  BASED ON EARLIER IMPLEMENTATION OF
//  VRML COMPRESSED BINARY FORMAT PROPOSAL BY:
//    Gabriel Taubin
//    Bill Horn
//    Francis Lazarus
//
//  Functionality : Error Resilience
//
//  Sungjin Kim  (sunjin@sait.samsung.co.kr)
//  Mahn-Jin Han  (manjini@sait.samsung.co.kr)
//  Mun-Sup Song (mssong@sait.samsung.co.kr)  
//
//  Signal Processing Lab. Samsung A.I.T. 
//-------------------------------------------------------------------------


#ifndef _MPG_Stitches_h_
#define _MPG_Stitches_h_

//MJH 010911 : changed for removal of stl3.13
#include <vector>	
#include <set>
//#include <vector.h>	
//#include <set.h>
//#include <multiset.h>
//MJH 019011 : end

using namespace std;	//MJH 010911

#include "MPG_Stream.h"
#include "MPG_QFac.h"

class MPG_Stitches {

public:

enum CommandType {
  ST_NONE = 0 ,// integers 1, 2, ... are default
  ST_PUSH,         
  ST_GET,
  ST_POP         
};


typedef vector <CommandType> vector_CT;	//MJH 019011 : added for removal of stl3.13

protected:

//MJH 019011 : changed for removal of stl3.13
	vector_CT						_type;
//  vector <CommandType>              _type;
//MJH 019011 : end

  vector <int>                      _length;
  vector <int>                      _stitchTo;
  vector <bool>                     _push;
  vector <bool>                     _reverse;


public:

  inline               ~MPG_Stitches();
  inline                MPG_Stitches();
  inline                MPG_Stitches(int nV);
  
  inline const int      size();

  inline void           reset(int nV);

  inline bool           getCommandType(int i, CommandType& t);
  inline bool           getLength(int i, int& length);
  inline bool           setLength(int i, int length);

  inline bool           get(int i, CommandType& t, int& index,
			    int& length, bool& push, bool& reverse);
  inline bool           set(int i, CommandType t, int index,
			    int length, bool push, bool reverse);

  inline void           push_back(CommandType t, int index,
				  int length, bool push, bool reverse);

  bool                  encode(int nV0, int nV1, vector <int>&stk, 
			       MPG_QFac &codec);

  bool                  decode(int nV0, int nV1, vector <int>&stk, 
			       MPG_QFac &codec);
};

typedef MPG_Stitches* MPG_pStitches;
#ifndef __GNUC__
// to make the xlC compiler happy
inline void destroy(MPG_Stitches* pointer) {}
inline void destroy(MPG_pStitches* pointer) {}
inline void destroy(MPG_Stitches::CommandType command) {}
inline void destroy(MPG_Stitches::CommandType* pointer) {}
#endif // __GNUC__

inline MPG_Stitches::~MPG_Stitches() // default destructor 
{}

inline MPG_Stitches::MPG_Stitches() // default constructor
{}

inline MPG_Stitches::MPG_Stitches(int nV)
{
  reset(nV);
}

inline const int MPG_Stitches::size()
{ return _type.size(); }

inline void MPG_Stitches:: reset(int nV)
{
  _type.erase(_type.begin(),_type.end());
  _type.insert(_type.begin(), (size_t)nV, ST_NONE);
  
  _length.erase(_length.begin(),_length.end());
  _length.insert(_length.begin(), (size_t)nV, 0);
  
  _push.erase(_push.begin(),_push.end());
  _push.insert(_push.begin(), (size_t)nV, false);
  
  _reverse.erase(_reverse.begin(),_reverse.end());
  _reverse.insert(_reverse.begin(), (size_t)nV, false);
  
  _stitchTo.erase(_stitchTo.begin(),_stitchTo.end());
  _stitchTo.insert(_stitchTo.begin(), (size_t)nV, -1);
}

inline bool MPG_Stitches::getCommandType(int i, CommandType& t)
{
  if(i<0 || i>=_type.size()) return false;
  t       = _type[i];
  return true;
}

inline bool MPG_Stitches::getLength(int i, int& length)
{
  if(i<0 || i>=_type.size()) return false;
  length = _length[i];
  return true;
}

inline bool MPG_Stitches::setLength(int i, int length)
{
  if(i<0 || i>=_type.size()) return false;
  _length[i] = length;
  return true;
}

inline bool MPG_Stitches::get(int i, CommandType& t, int& index,
			      int& length, bool& push, bool& reverse)
{
  if(i<0 || i>=_type.size()) return false;

  t       = _type[i];
  length  = _length[i];
  index   = _stitchTo[i];
  push    = _push[i];
  reverse = _reverse[i];
  return true;
}

inline bool MPG_Stitches::set(int i, CommandType t, int index,
			      int length, bool push, bool reverse)
{
  if(i<0 || i>=_type.size()) return false;
  if(i>=_type.size())
    {
      // we have to insert i - _type.size() elements in each array
      int how_many = i - _type.size()+1;
      _type.insert(_type.end(),         (size_t)how_many, ST_NONE);
      _length.insert(_length.end(),     (size_t)how_many, 0);
      _push.insert(_push.end(),         (size_t)how_many, false);
      _reverse.insert(_reverse.end(),         (size_t)how_many, false);
      _stitchTo.insert(_stitchTo.end(), (size_t)how_many, -1);
    }

  _type[i]      = t;
  _length[i]    = length;
  _stitchTo[i]  = index;
  _push[i]      = push;
  _reverse[i]   = reverse;

  return true;
}

inline void MPG_Stitches::push_back(CommandType t, int index,
				    int length, bool push, bool reverse)
{
  _type.push_back(t);
  _length.push_back(length);
  _stitchTo.push_back(index);
  _push.push_back(push);
  _reverse.push_back(reverse);
}


class MPG_PotentialStitch
{
 private:
  
  int   _v_origin;
  int   _v_dest;
  int   _length;
  bool  _direction;

 public:

  inline int origin()     const { return _v_origin; }
  inline int dest()       const { return _v_dest; }
  inline int length()     const { return _length; }
  inline bool direction() const { return _direction; }
  
  MPG_PotentialStitch(int v_origin, int v_dest, int length, bool direction)
    : _v_origin(v_origin), _v_dest(v_dest), _length(length), 
      _direction(direction)
    {}
  ~MPG_PotentialStitch() {}

  bool operator<(const MPG_PotentialStitch &a) const
    {
      return _length < a._length;
    }
};


#endif // _MPG_Stitches_h_
