//-------------------------------------------------------------------------
// 
// Copyright (C) IBM Corporation & Samsung Electronics Co. LTD., 1996, 1997, 1998, 1999
// Licensed Materials, Program Property of IBM & Samsung.
// 
// This software module is an implementation of a part of one or more
// MPEG-4 tools as specified by the MPEG-4 standard (ISO/IEC 14496).
// ISO/IEC gives users of the MPEG-4 standard (ISO/IEC 14496) free
// license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4
// standard (ISO/IEC 14496).  Those intending to use this software module
// in hardware or software products are advised that its use may infringe
// existing patents.  The original developer of this software module and
// his/her company, the subsequent editors and their companies, and
// ISO/IEC have no liability for use of this software module or
// modifications thereof in an implementation.  Copyright is not released
// for non MPEG-4 standard (ISO/IEC 14496) conforming products.  IBM
// Corp. & Samsung Electronics Co. LTD retains full right to use the code 
// for his/her own purpose, assign or donate the code to a third party and
// to inhibit third parties from using the code for non MPEG-4 standard 
// (ISO/IEC 14496) conforming products. This copyright notice must be included
// in all copies or derivative works.
// 
//  AUTHORS:
//    Gabriel Taubin
//    Andre Gueziec
//    Claudio Silva
//    Ioana Martin
//  
//  IBM T. J. Watson Research Center
//  P.O.Box 704
//  Yorktown Heights, New York 10598
//  
//  BASED ON EARLIER IMPLEMENTATION OF
//  VRML COMPRESSED BINARY FORMAT PROPOSAL BY:
//    Gabriel Taubin
//    Bill Horn
//    Francis Lazarus
//
//  Functionality : Error Resilience
//
//  Sungjin Kim  (sunjin@sait.samsung.co.kr)
//  Mahn-Jin Han  (manjini@sait.samsung.co.kr)
//  Mun-Sup Song (mssong@sait.samsung.co.kr)  
//
//  Signal Processing Lab. Samsung A.I.T. 
//-------------------------------------------------------------------------


#include "MPG_Global.h"

// start codes
const char MPG_Global::MO_start_code   = 0x20;
const char MPG_Global::MOL_start_code  = 0x30;
const char MPG_Global::MOBL_start_code = 0x31;
const char MPG_Global::MOBL0_start_code = 0x31;	
const char MPG_Global::MOBL1_start_code = 0x33; 
const char MPG_Global::MOBL2_start_code = 0x34;	
const char MPG_Global::MORL_start_code = 0x32;
// connected components
MPG_QFpm MPG_Global::last_component;
// triangle tree
MPG_QFpm MPG_Global::tt_run_length;
MPG_QFpm MPG_Global::tt_leaf;
MPG_QFpm MPG_Global::tt_triangulated;
// triangle data
MPG_QFpm MPG_Global::td_marching_pattern[2];
MPG_QFpm MPG_Global::td_polygon_edge[2];
MPG_QFpm MPG_Global::td_coord_leading_bit[96];
MPG_QFpm MPG_Global::td_normal_leading_bit[32];
MPG_QFpm MPG_Global::td_color_leading_bit[96];
MPG_QFpm MPG_Global::td_texCoord_leading_bit[64];
// vertex graph
MPG_QFpm MPG_Global::vg_simple;
MPG_QFpm MPG_Global::vg_last;
MPG_QFpm MPG_Global::vg_forward_run;
MPG_QFpm MPG_Global::vg_loop_index;
MPG_QFpm MPG_Global::vg_run_length;
MPG_QFpm MPG_Global::vg_leaf;
MPG_QFpm MPG_Global::vg_loop;

// stitches
MPG_QFpm MPG_Global::stitch_has;
MPG_QFpm MPG_Global::stitch_none;
MPG_QFpm MPG_Global::stitch_command[2];
MPG_QFpm MPG_Global::stitch_length;
MPG_QFpm MPG_Global::stitch_length_diff;
MPG_QFpm MPG_Global::stitch_depth;
MPG_QFpm MPG_Global::stitch_sign;
MPG_QFpm MPG_Global::stitch_push;
MPG_QFpm MPG_Global::stitch_reverse;

// partition_type
MPG_QFpm MPG_Global::codap_last_vg_context;
MPG_QFpm MPG_Global::codap_branch_len_context;
MPG_QFpm MPG_Global::td_orientation_context;

// pfs
MPG_QFpm MPG_Global::pfs_sharp_edges;
MPG_QFpm MPG_Global::pfs_fixed_vertices;
MPG_QFpm MPG_Global::pfs_forest_edge;
MPG_QFpm MPG_Global::pfs_other_update;

//---------------------------------------------------------------------------
//
//   MPG_Global::reset
//
//---------------------------------------------------------------------------

void MPG_Global::reset()
{
  int i;

  last_component.reset();
  // sp
  tt_run_length.reset();
  tt_leaf.reset();
  tt_triangulated.reset();
  for(i=0;i<2;i++)
    td_marching_pattern[i].reset();
  for(i=0;i<2;i++)
    td_polygon_edge[i].reset();
  for(i=0;i<96;i++)
    td_coord_leading_bit[i].reset();
  for(i=0;i<32;i++)
    td_normal_leading_bit[i].reset();
  for(i=0;i<96;i++)
    td_color_leading_bit[i].reset();
  for(i=0;i<64;i++)
    td_texCoord_leading_bit[i].reset();
  // vp
  vg_simple.reset();
  vg_last.reset();
  vg_forward_run.reset();
  vg_loop_index.reset();
  vg_run_length.reset();
  vg_leaf.reset();
  vg_loop.reset();

  // stitches
  stitch_has.reset();
  stitch_none.reset();
  for(i=0;i<2;i++)
    stitch_command[i].reset();
  stitch_length.reset();
  stitch_length_diff.reset();
  stitch_depth.reset();
  stitch_sign.reset();
  stitch_push.reset();
  stitch_reverse.reset();

  // partition
  codap_last_vg_context.reset();
  codap_branch_len_context.reset();	
  td_orientation_context.reset();

  // pfs
  pfs_sharp_edges.reset();
  pfs_fixed_vertices.reset();
  pfs_forest_edge.reset();
  pfs_other_update.reset();
}

void MPG_Global::vg_reset()
{
	int i;
	// vp
	vg_simple.reset();
	vg_last.reset();
	vg_forward_run.reset();
	vg_loop_index.reset();
	vg_run_length.reset();
	vg_leaf.reset();
	vg_loop.reset();
	codap_last_vg_context.reset();  

	// stitches
	stitch_has.reset();
	stitch_none.reset();
	for(i=0;i<2;i++)
		stitch_command[i].reset();
	stitch_length.reset();
	stitch_length_diff.reset();
	stitch_depth.reset();
	stitch_sign.reset();
	stitch_push.reset();
	stitch_reverse.reset();
}

void MPG_Global::vg_store()
{
	int i;
	// vp
	vg_simple.store();
	vg_last.store();
	vg_forward_run.store();
	vg_loop_index.store();
	vg_run_length.store();
	vg_leaf.store();
	vg_loop.store();
	codap_last_vg_context.store();

	// stitches
	stitch_has.store();
	stitch_none.store();
	for(i=0;i<2;i++)
		stitch_command[i].store();
	stitch_length.store();
	stitch_length_diff.store();
	stitch_depth.store();
	stitch_sign.store();
	stitch_push.store();
	stitch_reverse.store();
}

void MPG_Global::vg_restore()
{
	int i;
	// vp
	vg_simple.restore();
	vg_last.restore();
	vg_forward_run.restore();
	vg_loop_index.restore();
	vg_run_length.restore();
	vg_leaf.restore();
	vg_loop.restore();
	codap_last_vg_context.restore();
	
	// stitches
	stitch_has.restore();
	stitch_none.restore();
	for(i=0;i<2;i++)
		stitch_command[i].restore();
	stitch_length.restore();
	stitch_length_diff.restore();
	stitch_depth.restore();
	stitch_sign.restore();
	stitch_push.restore();
	stitch_reverse.restore();
}

void MPG_Global::sp_reset()
{
	int i;
	
	last_component.reset();
	// sp
	tt_run_length.reset();
	tt_leaf.reset();
	tt_triangulated.reset();
	for(i=0;i<2;i++)
		td_marching_pattern[i].reset();
	for(i=0;i<2;i++)
		td_polygon_edge[i].reset();
	for(i=0;i<96;i++)
		td_coord_leading_bit[i].reset();
	for(i=0;i<32;i++)
		td_normal_leading_bit[i].reset();
	for(i=0;i<96;i++)
		td_color_leading_bit[i].reset();
	for(i=0;i<64;i++)
		td_texCoord_leading_bit[i].reset();

	//codap
	codap_branch_len_context.reset();	
	td_orientation_context.reset();
}

void MPG_Global::sp_store()
{
	int i;
	
	last_component.store();
	// sp
	tt_run_length.store();
	tt_leaf.store();
	tt_triangulated.store();
	for(i=0;i<2;i++)
		td_marching_pattern[i].store();
	for(i=0;i<2;i++)
		td_polygon_edge[i].store();
	for(i=0;i<96;i++)
		td_coord_leading_bit[i].store();
	for(i=0;i<32;i++)
		td_normal_leading_bit[i].store();
	for(i=0;i<96;i++)
		td_color_leading_bit[i].store();
	for(i=0;i<64;i++)
		td_texCoord_leading_bit[i].store();
	
	//codap
	codap_branch_len_context.store();	
	td_orientation_context.store();
}

void MPG_Global::sp_restore()
{
	int i;
	
	last_component.restore();
	// sp
	tt_run_length.restore();
	tt_leaf.restore();
	tt_triangulated.restore();
	for(i=0;i<2;i++)
		td_marching_pattern[i].restore();
	for(i=0;i<2;i++)
		td_polygon_edge[i].restore();
	for(i=0;i<96;i++)
		td_coord_leading_bit[i].restore();
	for(i=0;i<32;i++)
		td_normal_leading_bit[i].restore();
	for(i=0;i<96;i++)
		td_color_leading_bit[i].restore();
	for(i=0;i<64;i++)
		td_texCoord_leading_bit[i].restore();
	
	//codap
	codap_branch_len_context.restore();	
	td_orientation_context.restore();
}

void MPG_Global::pfs_reset() 
{
	// pfs
	pfs_sharp_edges.reset();
	pfs_fixed_vertices.reset();
	pfs_forest_edge.reset();
	pfs_other_update.reset();
}

void MPG_Global::pfs_store() 
{
	// pfs
	pfs_sharp_edges.store();
	pfs_fixed_vertices.store();
	pfs_forest_edge.store();
	pfs_other_update.store();
}

void MPG_Global::pfs_restore() 
{
	// pfs
	pfs_sharp_edges.restore();
	pfs_fixed_vertices.restore();
	pfs_forest_edge.restore();
	pfs_other_update.restore();
}
