//-------------------------------------------------------------------------
// 
// Copyright (C) IBM Corporation & Samsung Electronics Co. LTD., 1996, 1997, 1998, 1999
// Licensed Materials, Program Property of IBM & Samsung.
// 
// This software module is an implementation of a part of one or more
// MPEG-4 tools as specified by the MPEG-4 standard (ISO/IEC 14496).
// ISO/IEC gives users of the MPEG-4 standard (ISO/IEC 14496) free
// license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4
// standard (ISO/IEC 14496).  Those intending to use this software module
// in hardware or software products are advised that its use may infringe
// existing patents.  The original developer of this software module and
// his/her company, the subsequent editors and their companies, and
// ISO/IEC have no liability for use of this software module or
// modifications thereof in an implementation.  Copyright is not released
// for non MPEG-4 standard (ISO/IEC 14496) conforming products.  IBM
// Corp. & Samsung Electronics Co. LTD retains full right to use the code 
// for his/her own purpose, assign or donate the code to a third party and
// to inhibit third parties from using the code for non MPEG-4 standard 
// (ISO/IEC 14496) conforming products. This copyright notice must be included
// in all copies or derivative works.
// 
//  AUTHORS:
//    Gabriel Taubin
//    Andre Gueziec
//    Claudio Silva
//    Ioana Martin
//  
//  IBM T. J. Watson Research Center
//  P.O.Box 704
//  Yorktown Heights, New York 10598
//  
//  BASED ON EARLIER IMPLEMENTATION OF
//  VRML COMPRESSED BINARY FORMAT PROPOSAL BY:
//    Gabriel Taubin
//    Bill Horn
//    Francis Lazarus
//
//  Functionality : Error Resilience
//
//  Sungjin Kim  (sunjin@sait.samsung.co.kr)
//  Mahn-Jin Han  (manjini@sait.samsung.co.kr)
//  Mun-Sup Song (mssong@sait.samsung.co.kr)  
//
//  Signal Processing Lab. Samsung A.I.T. 
//-------------------------------------------------------------------------

#include <stdio.h>
#include <stdlib.h>
#include "MPG_Graph.h"

//---------------------------------------------------------------------------
//
//   MPG_Graph::MPG_Graph
//
//---------------------------------------------------------------------------

MPG_Graph::MPG_Graph(vector<int>& coordIndex)
{
  if(coordIndex.size()==0) return;
  if(coordIndex.back()>=0) coordIndex.push_back(-1);

  int i,nfi,*fi,j,k,h;

  int nV = -1;
  int nF =  0;
  for(i=0;i<coordIndex.size();i++)
    if(coordIndex[i]>=0)
      {
	if(nV<0 || coordIndex[i]>nV) nV = coordIndex[i];
      }
    else
      nF++;
  nV++;

  _isOriented = false;
  _isConst    = false;
  _oneEdge.insert(_oneEdge.end(),(size_t) nV,(pEdge)0);
  _firstEdge.insert(_firstEdge.end(),(size_t) nV,-1);

  nfi = 0;
  fi  = NULL;
  for(i=k=0;k<coordIndex.size();k++)
    if(coordIndex[k]>=0)
      {
	if(k==0 || coordIndex[k-1]<0)
	  { fi = coordIndex.begin()+k; nfi = 0; }
	nfi++;
      }
    else
      {

        insertFace(nfi,fi); i++;
      }

  enumerateEdges();

  int nE = getNumberOfEdges();
  _edgeF.reserve(nE+1);
  _edgeF.insert(_edgeF.end(),(size_t) nE+1,0);

  // count number of incident faces
  nfi = 0;
  fi  = NULL;
  for(k=0;k<coordIndex.size();k++)
    if(coordIndex[k]>=0)
      {
	if(k==0 || coordIndex[k-1]<0)
	  { fi = coordIndex.begin()+k; nfi = 0; }
	nfi++;
      }
    else
      {
	for(j=0;j<nfi;j++)
	  {
	    h = (j+1)%nfi;
	    Edge* e = getEdge(fi[j],fi[h]);
	    int  ie = e->getIndex();
	    _edgeF[ie+1]++;
	  }
      }

  for(i=0;i<nE;i++)
    _edgeF[i+1] += _edgeF[i];
  int nEF = _edgeF[nE];

  _edgeI.reserve(nEF);
  _edgeI.insert(_edgeI.end(),(size_t) nEF,-1);

  vector<int> nFe((size_t) nE,0);
  nfi = 0;
  fi  = NULL;
  for(i=k=0;k<coordIndex.size();k++)
    if(coordIndex[k]>=0)
      {
	if(k==0 || coordIndex[k-1]<0)
	  { fi = coordIndex.begin()+k; nfi = 0; }
	nfi++;
      }
    else
      {

	for(j=0;j<nfi;j++)
	  {
	    h = (j+1)%nfi;
	    Edge* e = getEdge(fi[j],fi[h]);
	    int  ie = e->getIndex();
	    _edgeI[_edgeF[ie]+nFe[ie]] = h;
	    nFe[ie]++;
	  }
	i++;
      }
}

//---------------------------------------------------------------------------
//
//   MPG_Graph::getEdge
//
//---------------------------------------------------------------------------

MPG_Graph::pEdge MPG_Graph::getEdge(int i, int j)
{
  pEdge e = (pEdge)0;

  int nV = getNumberOfVertices();
  if(0<=i&&i<nV&&0<=j&&j<nV&&i!=j)
    {
      if(_isOriented==false && j<i) { int k=j; j=i; i=k; }
      if(_firstEdge[i]>=0)
	{
	  pEdge ei = (pEdge)0;
	  int h;
	  for(h=_firstEdge[i];h>=0;h=ei->getNext())
	    {
	      ei = (pEdge)(_edge.begin()+h);
	      if((*ei)[1]==j)
		{ e = ei; break; }
	    }
	}
    }

  return e;
}

//---------------------------------------------------------------------------
//
//   MPG_Graph::insertEdge
//
//---------------------------------------------------------------------------

void MPG_Graph::insertEdge(int i, int j, int indx)
{
  if(_isConst==false && getEdge(i,j)==(pEdge)0)
    {
      if(_isOriented==false && j<i) { int k=j; j=i; i=k; }

      int pos = _pushBackEdge(i,j,indx);

      if(pos>=0)
	{
	  pEdge e_pos = _getPosEdge(pos);

	  if(_oneEdge[i]==(pEdge)0) _oneEdge[i] = e_pos;
	  if(_oneEdge[j]==(pEdge)0) _oneEdge[j] = e_pos;

	  // lexicographic insert in i-th. list
	  int h = _firstEdge[i];
	  if(h<0)
	    _firstEdge[i] = pos;
	  else
	    {
	      pEdge e_h = _getPosEdge(h);
	      if(j<(*e_h)[1])
		{ e_pos->_next = h; _firstEdge[i] = pos; }
	      else
		while(1)
		  {
		    if(e_h->_next<0)
		      { e_h->_next = pos; break; }
		    else
		      {
			int k = e_h->_next;
			pEdge e_k = _getPosEdge(k);
			if(j<(*e_k)[1])
			  { e_h->_next = pos; e_pos->_next = k; break; }
			else
			  { e_h = e_k; }
		      }
		  }
	    }
	}
    }
}

//---------------------------------------------------------------------------
//
//   MPG_Graph::smooth
//
//---------------------------------------------------------------------------

bool MPG_Graph::smooth(vector<float>& x,
		       vector<float>& sx,
		       float a, float b, int n,
		       vector<bool>*  e_sharp,
		       vector<bool>*  v_fixed)
{
  bool success = false;

  int nV  = getNumberOfVertices();
  int nD  = x.size()/nV;
  if(nD<1) return false;

  int nE  = getNumberOfEdges();
  int nVD = nV*nD;

  // make shure edge and vertex arrays are long enough;
  // if passed as arguments
  if(e_sharp!=NULL && e_sharp->size()<nE)
    {
      int n = nE-e_sharp->size();
      e_sharp->insert(e_sharp->end(),(size_t)n,false);
    }
  if(v_fixed!=NULL && v_fixed->size()<nV)
    {
      int n = nV-v_fixed->size();
      v_fixed->insert(v_fixed->end(),(size_t)n,false);
    }

  // determine sharp vertices from sharp edges
  vector<bool> v_sharp((size_t)nV,false);
  if(e_sharp!=NULL)
    {
      MPG_Graph::pEdge e;
      int k;
      for(k=0;k<nV;k++)
	for(e=getFirstEdge(k);e!=(MPG_Graph::pEdge)0;e=getNextEdge(e))
	  if((*e_sharp)[e->getIndex()]==true)
	    {
	      v_sharp[e->getVertex(0)] = true;
	      v_sharp[e->getVertex(1)] = true;
	    }
    }

  // if not smoothing in place, initialize sx to a copy of x
  if(&sx!=&x)
    {
      sx.erase(sx.begin(),sx.end());
      sx.reserve((size_t)nVD);
      int h;
      for(h=0;h<nVD;h++)
	sx.push_back(x[h]);
    }

  if(n>0)
    {
      int h,i,j,k,ie;
      float w_e;
      bool  vi_is_fixed,vj_is_fixed,e_is_sharp;

      float* sx_ji = new float[nD];

      // allocate auxiliary displacements array
      vector<float> dx((size_t)nVD,0.0);

      // allocate auxiliary normalization array
      vector<float> nx((size_t)nV,0.0);

      // do the work
      if(n%2) n--;		// n must be even
      float lambda = a;
      while(n-->0)
	{
	  // initialize
	  for(h=0;h<nVD;h++)
	    dx[h] = 0.0;
	  for(h=0;h<nV;h++)
	    nx[h] = 0.0;

	  // dx = multiply graph matrix L by vector sx
	  // L  = I-W ; W=diag(w)*E
	  MPG_Graph::pEdge e;
	  for(k=0;k<nV;k++)
	    for(e=getFirstEdge(k);e!=(MPG_Graph::pEdge)0;e=getNextEdge(e))
	      {
		ie          = e->getIndex();
		w_e         = 1.0;
		e_is_sharp  = (e_sharp!=NULL )?(*e_sharp )[ie]:false;

		i = e->getVertex(0);
		vi_is_fixed = (v_fixed!=NULL)?(*v_fixed)[i]:false;

		j = e->getVertex(1);
		vj_is_fixed = (v_fixed!=NULL)?(*v_fixed)[j]:false;

		for(h=0;h<nD;h++)
		  sx_ji[h] = sx[j*nD+h]-sx[i*nD+h];

		if(vi_is_fixed==false &&
		   (e_is_sharp==true || v_sharp[i]==false))
		  for(nx[i]+=w_e,h=0;h<nD;h++)
		    dx[i*nD+h] += w_e*sx_ji[h];

		if(vj_is_fixed==false &&
		   (e_is_sharp==true || v_sharp[j]==false))
		  for(nx[j]+=w_e,h=0;h<nD;h++)
		    dx[j*nD+h] -= w_e*sx_ji[h];
	      }

	  // normalize
	  for(k=0;k<nV;k++)
	    if(nx[i]>0.0)
	      for(h=0;h<nD;h++)
		dx[k*nD+h] /= nx[k];

	  // accumulate
	  for(h=0;h<nVD;h++)
	    sx[h] += lambda*dx[h];

	  // advance
	  lambda = (lambda==a)?b:a;
	}
      success = true;
    }

  return success;
}
