//-------------------------------------------------------------------------
// 
// Copyright (C) IBM Corporation & Samsung Electronics Co. LTD., 1996, 1997, 1998, 1999
// Licensed Materials, Program Property of IBM & Samsung.
// 
// This software module is an implementation of a part of one or more
// MPEG-4 tools as specified by the MPEG-4 standard (ISO/IEC 14496).
// ISO/IEC gives users of the MPEG-4 standard (ISO/IEC 14496) free
// license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4
// standard (ISO/IEC 14496).  Those intending to use this software module
// in hardware or software products are advised that its use may infringe
// existing patents.  The original developer of this software module and
// his/her company, the subsequent editors and their companies, and
// ISO/IEC have no liability for use of this software module or
// modifications thereof in an implementation.  Copyright is not released
// for non MPEG-4 standard (ISO/IEC 14496) conforming products.  IBM
// Corp. & Samsung Electronics Co. LTD retains full right to use the code 
// for his/her own purpose, assign or donate the code to a third party and
// to inhibit third parties from using the code for non MPEG-4 standard 
// (ISO/IEC 14496) conforming products. This copyright notice must be included
// in all copies or derivative works.
// 
//  AUTHORS:
//    Gabriel Taubin
//    Andre Gueziec
//    Claudio Silva
//    Ioana Martin
//  
//  IBM T. J. Watson Research Center
//  P.O.Box 704
//  Yorktown Heights, New York 10598
//  
//  BASED ON EARLIER IMPLEMENTATION OF
//  VRML COMPRESSED BINARY FORMAT PROPOSAL BY:
//    Gabriel Taubin
//    Bill Horn
//    Francis Lazarus
//
//  Functionality : Error Resilience
//
//  Sungjin Kim  (sunjin@sait.samsung.co.kr)
//  Mahn-Jin Han  (manjini@sait.samsung.co.kr)
//  Mun-Sup Song (mssong@sait.samsung.co.kr)  
//
//  Signal Processing Lab. Samsung A.I.T. 
//-------------------------------------------------------------------------


#include "MPG_Lex.h"

int MPG_Lex::D = 0;

//---------------------------------------------------------------------------
//
//   MPG_Lex::LT
//
//---------------------------------------------------------------------------

// for compatibility with old MRS_Heap
bool MPG_Lex::LT(void* pp, void* qq)
{
  union { void* k; float f; } p,q;

  p.k = pp;
  q.k = qq;

  return (p.f<p.f)?true:false;
}

//---------------------------------------------------------------------------
//
//   MPG_Lex::EQ
//
//---------------------------------------------------------------------------

// for compatibility with old MRS_Heap
bool MPG_Lex::EQ(void* pp, void* qq)
{
  union { void* k; float f; } p,q;

  p.k = pp;
  q.k = qq;

  return (p.f==p.f)?true:false;
}

//---------------------------------------------------------------------------
//
//   MPG_Lex::LT_F
//
//---------------------------------------------------------------------------

bool MPG_Lex::LT_F(void* pp, void* qq)
{
  float* p = (float*)pp;
  float* q = (float*)qq;

  int i;
  bool p_leq_q = true;
  for(i=0;i<D;i++)
    if(p[i]<q[i])
      break;
    else if(p[i]>q[i])
      { p_leq_q = false; break; };
  return p_leq_q;
}

//---------------------------------------------------------------------------
//
//   MPG_Lex::EQ_F
//
//---------------------------------------------------------------------------

bool MPG_Lex::EQ_F(void* pp, void* qq)
{
  float* p = (float*)pp;
  float* q = (float*)qq;

  bool p_eq_q = true;

  int i;
  for(i=0;i<D;i++)
    if(p[i]!=q[i])
      { p_eq_q = false; break; };

  return p_eq_q;
}

//---------------------------------------------------------------------------
//
//   MPG_Lex::LT_I
//
//---------------------------------------------------------------------------

bool MPG_Lex::LT_I(void* pp, void* qq)
{
  int* p = (int*)pp;
  int* q = (int*)qq;

  int i;
  bool p_leq_q = true;
  for(i=0;i<D;i++)
    if(p[i]<q[i])
      break;
    else if(p[i]>q[i])
      { p_leq_q = false; break; };
  return p_leq_q;
}

//---------------------------------------------------------------------------
//
//   MPG_Lex::EQ_I
//
//---------------------------------------------------------------------------

bool MPG_Lex::EQ_I(void* pp, void* qq)
{
  int* p = (int*)pp;
  int* q = (int*)qq;

  bool p_eq_q = true;

  int i;
  for(i=0;i<D;i++)
    if(p[i]!=q[i])
      { p_eq_q = false; break; };

  return p_eq_q;
}

