//-------------------------------------------------------------------------
// 
// Copyright (C) IBM Corporation & Samsung Electronics Co. LTD., 1996, 1997, 1998, 1999
// Licensed Materials, Program Property of IBM & Samsung.
// 
// This software module is an implementation of a part of one or more
// MPEG-4 tools as specified by the MPEG-4 standard (ISO/IEC 14496).
// ISO/IEC gives users of the MPEG-4 standard (ISO/IEC 14496) free
// license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4
// standard (ISO/IEC 14496).  Those intending to use this software module
// in hardware or software products are advised that its use may infringe
// existing patents.  The original developer of this software module and
// his/her company, the subsequent editors and their companies, and
// ISO/IEC have no liability for use of this software module or
// modifications thereof in an implementation.  Copyright is not released
// for non MPEG-4 standard (ISO/IEC 14496) conforming products.  IBM
// Corp. & Samsung Electronics Co. LTD retains full right to use the code 
// for his/her own purpose, assign or donate the code to a third party and
// to inhibit third parties from using the code for non MPEG-4 standard 
// (ISO/IEC 14496) conforming products. This copyright notice must be included
// in all copies or derivative works.
// 
//  AUTHORS:
//    Gabriel Taubin
//    Andre Gueziec
//    Claudio Silva
//    Ioana Martin
//  
//  IBM T. J. Watson Research Center
//  P.O.Box 704
//  Yorktown Heights, New York 10598
//  
//  BASED ON EARLIER IMPLEMENTATION OF
//  VRML COMPRESSED BINARY FORMAT PROPOSAL BY:
//    Gabriel Taubin
//    Bill Horn
//    Francis Lazarus
//
//  Functionality : Error Resilience
//
//  Sungjin Kim  (sunjin@sait.samsung.co.kr)
//  Mahn-Jin Han  (manjini@sait.samsung.co.kr)
//  Mun-Sup Song (mssong@sait.samsung.co.kr)  
//
//  Signal Processing Lab. Samsung A.I.T. 
//-------------------------------------------------------------------------


#include <stdio.h>
#include <stdlib.h>
#include <vector>
#include "MPG_QFac.h"

using namespace std;	//MJH 010911

void MPG_QFac::hw_start_encode() 
{
  C = 0x0;
  C_high = 0x0;
  A = 0x8000;
  count = 12;
  first = 1;
}

void MPG_QFac::hw_renorm_encode() 
{
  do {
    C <<= 1;
    A <<= 1;
    count--;
    if (count == 0) {
      if (C_high == 0xff) {
      l1:
	put_byte(0xff);
	C_high = C >> 20;
	C &= 0x000fffff;
	count = 7;
      }
      else {
	if (C < 0x08000000) {
	l2:
	  put_byte(C_high);
	  C_high = C >> 19;
	  C &= 0x0007ffff;
	  count = 8;
	}
	else {
	  C_high++;
	  if (C_high == 0xff) {
	    C &= 0x07ffffff;
	    goto l1;
	  }
	  else
	    goto l2;
	}
      }
    }
    bits++;
	Bitcnt();  
  } while (!(A & 0x8000));
}

void MPG_QFac::hw_encode(int symbol, MPG_QFpm &state) 
{
  int cond_exchg = A < 2*state.Qe;
  
  if (symbol == (state.MPS ^ cond_exchg)) {
    C += state.Qe;
    A -= state.Qe;
  }
  else
    A = state.Qe;
  
  if (!(A & 0x8000)) {
    state.update(symbol);
    hw_renorm_encode();
  }
}

void MPG_QFac::hw_encode(int symbol) 
{
  int cond_exchg = A < 0xac02;
  
  if (symbol == cond_exchg) {
    C += 0x5601;
    A -= 0x5601;
  }
  else
    A = 0x5601;
  
  if (!(A & 0x8000)) {
    hw_renorm_encode();
  }
}

void MPG_QFac::hw_finish_encode() 
{
  C += 0x4000;
  C &= 0xffffc000;

  int added_byte=(count>3) ? 0: 1;  

  A >>= 15;
  hw_renorm_encode();

  A >>= 3;
  hw_renorm_encode();
  put_byte(C_high);
  put_byte(0x00);

  int byte_align=(tbits%8!=0) ? (8-(tbits%8)): 0;
  if(added_byte) {
	  cbits += 16;
	  tbits += byte_align+16;
	  if(erMode) rbits -= (byte_align+16);
  } else {
	  cbits += 8;
	  tbits += byte_align+8;
	  if(erMode) rbits -= (byte_align+8);
  }
}

void MPG_QFac::hw_start_decode() 
{
  int b = get_byte() ^ 0xff;

  C = b << 23;
  b = get_byte() ^ 0xff;

  C += b << 15;
  if ((b + 1) >> 8) {
    C += (get_byte() ^ 0xff) << 8;
    count = 8;
  }
  else
    count = 1;
  A = 0x80000000;
}

void MPG_QFac::hw_renorm_decode() 
{
  do {
    C <<= 1;
    if (--count == 0) {
      int b = get_byte() ^ 0xff;
      C += b << 8;
      count = 8 - ((b + 1) >> 8);
    }
    A <<= 1;
  } while (!(A & 0x80000000));
}

int MPG_QFac::hw_decode(MPG_QFpm &state) 
{
  int cond_exchg = A < (state.Qe << 17);
  int symbol = state.MPS ^ cond_exchg;
  
  if (C >= (state.Qe << 16)) {
    C -= state.Qe << 16;
    A -= state.Qe << 16;
  }
  else {
    symbol ^= 1;
    A = state.Qe << 16;
  }

  if (!(A & 0x80000000)) {
    state.update(symbol);
    hw_renorm_decode();
  }
  
  return symbol;
}

void MPG_QFac::hw_finish_decode() 
{ }

void MPG_QFac::sw_start_decode() 
{
  A = 0x00010000;
  C = 0x00010000;
  count = 0;
  sw_renorm_decode();
}

int MPG_QFac::sw_renorm_decode() 
{
  do {
    if (count == 0) {
      int b = get_byte();
      if (b < 0)
	return -1;
      C -= 0xff << 8;
      C += b << 8;
      count = 8 - ((0x100-b) >> 8);
    }
    count--;
    C = C << 1;
    A = A << 1;
  } while (!(A & 0x80000000));

  return 0;
}

int MPG_QFac::sw_decode(MPG_QFpm &state) 
{
  int symbol = state.MPS;

  A -= state.Qe;

  if (C <= A) {
    if (A & 0x80000000)
      return symbol;
    symbol ^= A < state.Qe;
  }
  else {
    symbol ^= A >= state.Qe;
    C = C - A;
    A = state.Qe;
  }

  state.update(symbol);
  symbol = (sw_renorm_decode()) ? -1 : symbol;

  return symbol;
}

int MPG_QFac::sw_decode() 
{
  int symbol = 0;
  A -= 0x56010000;

  if (C <= A) {
    if (A & 0x80000000)
      return symbol;
    symbol = A < 0x56010000;
  }
  else {
    symbol = A >= 0x56010000;
    C = C - A;
    A = 0x56010000;
  }

  symbol = (sw_renorm_decode()) ? -1 : symbol;

  return symbol;
}

void MPG_QFac::sw_finish_decode() 
{
}

void MPG_QFac::put_byte(int b) 
{
  if (first)
    first = 0;
  else {
    _ostrm->putUChar(b ^ 0xff);
  }
}

int MPG_QFac::get_byte() 
{
  unsigned char c;
  _istrm->getUChar(c);
  return c;
}

