//-------------------------------------------------------------------------
// 
// Copyright (C) IBM Corporation & Samsung Electronics Co. LTD., 1996, 1997, 1998, 1999
// Licensed Materials, Program Property of IBM & Samsung.
// 
// This software module is an implementation of a part of one or more
// MPEG-4 tools as specified by the MPEG-4 standard (ISO/IEC 14496).
// ISO/IEC gives users of the MPEG-4 standard (ISO/IEC 14496) free
// license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4
// standard (ISO/IEC 14496).  Those intending to use this software module
// in hardware or software products are advised that its use may infringe
// existing patents.  The original developer of this software module and
// his/her company, the subsequent editors and their companies, and
// ISO/IEC have no liability for use of this software module or
// modifications thereof in an implementation.  Copyright is not released
// for non MPEG-4 standard (ISO/IEC 14496) conforming products.  IBM
// Corp. & Samsung Electronics Co. LTD retains full right to use the code 
// for his/her own purpose, assign or donate the code to a third party and
// to inhibit third parties from using the code for non MPEG-4 standard 
// (ISO/IEC 14496) conforming products. This copyright notice must be included
// in all copies or derivative works.
// 
//  AUTHORS:
//    Gabriel Taubin
//    Andre Gueziec
//    Claudio Silva
//    Ioana Martin
//  
//  IBM T. J. Watson Research Center
//  P.O.Box 704
//  Yorktown Heights, New York 10598
//  
//  BASED ON EARLIER IMPLEMENTATION OF
//  VRML COMPRESSED BINARY FORMAT PROPOSAL BY:
//    Gabriel Taubin
//    Bill Horn
//    Francis Lazarus
//
//  Functionality : Error Resilience
//
//  Sungjin Kim  (sunjin@sait.samsung.co.kr)
//  Mahn-Jin Han  (manjini@sait.samsung.co.kr)
//  Mun-Sup Song (mssong@sait.samsung.co.kr)  
//
//  Signal Processing Lab. Samsung A.I.T. 
//-------------------------------------------------------------------------

#include <stdio.h>
#include <stdlib.h>
#include "MPG_Global.h"
#include "MPG_SPolygon.h"

void MPG_SPolygon::DetermineDirection()
{
	int nT     = getNumberOfTriangles();
	int nRuns  = getNumberOfRuns();       //leaf+branch
	vector<int>  run_father((size_t)nRuns,-1);
	vector<int>  run_length((size_t)nRuns,0);
	vector<bool> run_ends_in_leaf((size_t)nRuns,false);
	vector<int>  stk((size_t) 1,-1);

	int iT,iRun, depth=0, preiT=-1;
	tdirect.clear();
	for(iT=iRun=0;iT<nT;iT++)
	{
		run_length[iRun]++;
		switch(getTri(iT))
		{
		case sp_LEAF:
			run_father[iRun] = stk.back(); 
			stk.pop_back();
			run_ends_in_leaf[iRun] = true;
			tdirect.putRun(iRun);
			tdirect.putLeaf(1);
			if(iT==0) 
				tdirect.putStart(iT);
			else 
				tdirect.putStart(preiT+1);
			tdirect.putBranch(iT);
			tdirect.putFather(run_father[iRun]);
			preiT=iT;
			depth--;
			iRun++;
			break;
		case sp_BRANCH:
			run_father[iRun] = stk.back(); 
			stk.pop_back();
			run_ends_in_leaf[iRun] = false;
			stk.push_back(iRun);
			stk.push_back(iRun);
			tdirect.putRun(iRun);
			tdirect.putLeaf(0);
			if(iT==0) 
				tdirect.putStart(iT);
			else 
				tdirect.putStart(preiT+1);
			tdirect.putBranch(iT);
			tdirect.putFather(run_father[iRun]);
			preiT=iT;
			depth++;
			iRun++;
			break;
		case sp_LEFT:
		case sp_RIGHT:
		default:
			break;
		}
	}

	// determine the L->R length of each run
	for(iRun=nRuns-1;iRun>0;iRun--)	{
		if(run_ends_in_leaf[iRun]==true)
			run_length[iRun]++;
		else
			run_length[iRun]--;
		run_length[run_father[iRun]] += run_length[iRun];
	}
	run_length[iRun]--;		// iRun==0
	for(iRun=0; iRun<nRuns; iRun++) tdirect.putLength(run_length[iRun]);

	tdirect.setDirection();

#if 0
	tdirect.printDirection();
#endif
}

//---------------------------------------------------------------------------
//
//   MPG_SPolygon::encodeTriangleTree
//
//---------------------------------------------------------------------------

bool MPG_SPolygon::encodeTriangleTree(MPG_QFac &codec)
{
  MPG_SPolygon::TriType tt;

  int nTsp          = getNumberOfTriangles();
  int run_length    = 0;
  int iT;
  for(iT=0;iT<nTsp;iT++)
    {
      tt = (MPG_SPolygon::TriType) getTri(iT);

      run_length++;
      switch(tt)
	{
	  // end of run
	case MPG_SPolygon::sp_LEAF:
	case MPG_SPolygon::sp_BRANCH:
	  {
	    // encode run_length
	    int j;
	    for(j=0;j<run_length-1;j++)
	      codec.encode(0,MPG_Global::tt_run_length);
	    codec.encode(1,MPG_Global::tt_run_length);
	    run_length = 0;

	    // encode ends_in_leaf
	    switch(tt)
	      {
	      case MPG_SPolygon::sp_LEAF:
		codec.encode(true,MPG_Global::tt_leaf);
		break;

	      case MPG_SPolygon::sp_BRANCH:
		codec.encode(false,MPG_Global::tt_leaf);
		break;
	      }
	  }
          break;

	default:
	  break;
	}
    }

  return true;
}

// triangle_tree() {
//	  depth = 0
//	  ntriangles = 0
//	  branch_position = -2
//	  do {
//		 qf_decode(tt_run_length, tt_run_length_context)
//		 ntriangles += tt_run_length
//		 qf_decode(tt_leaf, tt_leaf_context)
//		 if (tt_leaf == '1') {
//			depth--
//		 } else {
//			branch_position = ntriangles
//			depth++
//		 }
//	  } while (depth >= 0)
//	  if (3D_MOBL_start_code == "partition_type_2")
//		 if (codap_right_bloop_idx - codap_left_bloop_idx - 1 > ntriangles) {
//          if (branch_position == ntriangles - 2) {
//             qf_decode(codap_branch_len, codap_branch_len_context)
//             ntriangles -= 2
//          } else
//		       ntriangles--
//       }
// }
bool MPG_SPolygon::encodeTriangleTree(MPG_QFac &codec,
									  vector<int>&   Grstk, 
									  int            &Gdepth,
									  MPG_PacketQ_C& codap
									  )
{
  MPG_SPolygon::TriType tt;

  int nTsp          = getNumberOfTriangles();
  int run_length    = 0;
  int iT, iRun=0, ldepth=0, iS=0; 
  int gdepth=Gdepth;
  vector<int> stk((size_t) 1,-1);    
  iS = codap.start_tri;
  iRun = codap.start_run;
  
  for(iT=0;iT<nTsp;iT++)
  {
      tt = (MPG_SPolygon::TriType) getTri(iS);  
      run_length++;
      switch(tt)
	  {
		  // end of run
	  case MPG_SPolygon::sp_LEAF:
		  codec.encode(1, MPG_Global::tt_run_length);
		  codec.encode(true,MPG_Global::tt_leaf);
		  if(ldepth==1) {
			  iRun = stk.back(); 
			  stk.pop_back();
			  iS = tdirect.getStart(iRun);
		  } else {
			  iS++;
			  iRun++;
		  };
		  ldepth--;
		  gdepth--;
		  break;
	  case MPG_SPolygon::sp_BRANCH:
		  codec.encode(1, MPG_Global::tt_run_length);
		  codec.encode(false, MPG_Global::tt_leaf);
		  
		  //changes the traversal order by td_orientation: left first -> right first
		  if(ldepth==0) { 
			  if(codap.sp_pt2_enable) { 
				  stk.push_back(tdirect.getSecond(iRun));
				  iRun = tdirect.getFirst(iRun);
			  } else {                 //pt0
				  if(tdirect.getDirect(iRun)) {  
					  stk.push_back(tdirect.getSecond(iRun));
					  iRun = tdirect.getFirst(iRun);
				  } else {
					  stk.push_back(tdirect.getFirst(iRun));
					  iRun = tdirect.getSecond(iRun);
				  }
			  }
			  iS = tdirect.getStart(iRun);
		  } else { 
			  iS++;
			  iRun++;
		  };
		  ldepth++;
		  gdepth++;
		  break;
	  default:
		  codec.encode(0, MPG_Global::tt_run_length);
		  iS++;  
		  break;
	  }

	  int kk;
	  if(codap.erMode>0) {
	  	  if(!codap.sp_pass0) {
		  	  if(iS==codap.next_tri) { 
				  switch(tt) {
				  case MPG_SPolygon::sp_BRANCH:  
					  codap.sp_sub_pt_enable=1;
					  if(Grstk.size()==1) codap.sp_bdry_pred=0;
					  kk=stk.back();
					  Grstk.push_back(kk);
					  codec.encode(1, MPG_Global::tt_run_length);
					  codec.encode(true, MPG_Global::tt_leaf);
					  codec.encode(1, MPG_Global::tt_run_length);
					  codec.encode(true, MPG_Global::tt_leaf);
					  codap.sp_branch_len = tdirect.getLength(iRun)-1;
					  break;
				  default : 
					  if(Grstk.size()>1) codap.sp_bdry_pred=1;
					  codap.sp_sub_pt_enable=0;
					  codec.encode(1, MPG_Global::tt_run_length);
					  codec.encode(true, MPG_Global::tt_leaf);
					  break;
				  }
				  break;
			  }
		  } else {
			  if(ldepth==-1) 
				  break;
		  }
	  }
    }
	
	return true;
}

//---------------------------------------------------------------------------
//
//   MPG_SPolygon::encodeTriangleData
//
//---------------------------------------------------------------------------

// don't we need to transform v_cluster
// to decoder order of traversal
// before entering this method ???

// AG 1/29/99 I think that we are OK on that

// I did not see where the prediction takes place
// decodeTriangleData uses prediction

bool MPG_SPolygon::encodeTriangleData(MPG_QFac&     codec,
				      int           nV0,
				      vector<int>&  v_loop_sp,
				      vector<int>&  v_visited,
				      vector<int>&  v_cluster,
				      MPG_Property& pCoord,
				      MPG_Property& pNormal,
				      MPG_Property& pColor,
				      MPG_Property& pTexCoord,
				      int*& tData)
{
  MPG_Binding coordBinding    = pCoord.getBinding();
  MPG_Binding normalBinding   = pNormal.getBinding();
  MPG_Binding colorBinding    = pColor.getBinding();
  MPG_Binding texCoordBinding = pTexCoord.getBinding();
    
  int bitsPerCoord    = (pCoord.getBBoxQ()!=(MPG_BBoxQ*)0)?
    pCoord.getBBoxQ()->getBitsPerCoordinate():0;
  int bitsPerNormal   = (pNormal.getNormalQ()!=(MPG_NormalQ*)0)?
    pNormal.getNormalQ()->getBitsPerNormal():0;
  int bitsPerColor    = (pColor.getBBoxQ()!=(MPG_BBoxQ*)0)?
    pColor.getBBoxQ()->getBitsPerCoordinate():0;
  int bitsPerTexCoord = (pTexCoord.getBBoxQ()!=(MPG_BBoxQ*)0)?
    pTexCoord.getBBoxQ()->getBitsPerCoordinate():0;
  
  // determine if SPolygon has properties per vertex
  
  bool has_per_vertex = (pCoord.getBinding()    == PB_PER_VERTEX ||
			 pNormal.getBinding()   == PB_PER_VERTEX ||
			 pColor.getBinding()    == PB_PER_VERTEX ||
			 pTexCoord.getBinding() == PB_PER_VERTEX)?true:false;

  // v_loop_sp, v_visited and v_clustered
  // are not refered to if has_per_vertex==false

  //
  // triangle_data() {
  //   triangulated
  //   root_triangle()
  //   for(i=1;i<ntriangles;i++)
  //     triangle()
  // }
  //

  int prev_marching_pattern = 0;
  int prev_polygon_edge     = 1;

  int triangulated = (hasPBits()==false)?1:0;
  codec.encode(triangulated,MPG_Global::tt_triangulated);

  int nT     = getNumberOfTriangles();
  int nVloop = getNumberOfVertices();
  if(has_per_vertex==true && nVloop!=v_loop_sp.size()) return false;
  int nRuns  = getNumberOfRuns();

  // determine number of vertices of vertex graph
  // from vertex loop look-up table
  // int nV     = 0;
  // int i;
  // for(i=0;i<v_loop_sp.size();i++)
  //  if(v_loop_sp[i]>nV)
  //    nV = v_loop_sp[i];
  // nV++;

  // face tree array
  vector<int>  f_father;

  // corner tree arrays
  vector<int>  c_traversal;
  vector<int>  c_father;

 
  // triangle tree arrays
  vector<int>  run_father((size_t)nRuns,-1);
  vector<int>  run_length((size_t)nRuns,0);
  vector<bool> run_ends_in_leaf((size_t)nRuns,false);

  // traversal stack
  vector<int>  stk((size_t) 1,-1);

  int nPE    = 0;    //      number of {non_root polygon_edge==0}
  int nF     = nT;   // - nPE
  int nC     = 3*nT; // - 2*nPE

  // construct the father array
  int iT,iRun;
  for(iT=iRun=0;iT<nT;iT++)
    {
      run_length[iRun]++;
      switch(getTri(iT))
	{
	case sp_LEAF:
	  run_father[iRun] = stk.back(); stk.pop_back();
	  run_ends_in_leaf[iRun] = true;
	  iRun++;
	  break;

	case sp_BRANCH:
	  run_father[iRun] = stk.back(); stk.pop_back();
	  run_ends_in_leaf[iRun] = false;
	  stk.push_back(iRun);
	  stk.push_back(iRun);
	  iRun++;
	  break;

	case sp_LEFT:
	case sp_RIGHT:
	default:
	  break;
	}
    }

  // determine the L->R length of each run
  for(iRun=nRuns-1;iRun>0;iRun--)
    {
      if(run_ends_in_leaf[iRun]==true)
	run_length[iRun]++;
      else
	run_length[iRun]--;
      run_length[run_father[iRun]] += run_length[iRun];
    }
  run_length[iRun]--;		// iRun==0

  int iP = -1;
  int iR =  0;
  int iL =  1;
  int iO = -1;

  vector<int> faceStack;
  int iF  = f_father.size();	// == 0
  faceStack.push_back(iF);
  int iFp = iF;

  int nC0  =  c_father.size();	// next free corner index
  int iC   = nC0;
  int cRp  =  iC;
  int cR   =  iC;
  int cLp  =  iC+1;
  int cL   =  iC+1;
  int cO   = -1;

  c_father.insert(c_father.end(),(size_t)nC,-1);

  MPG_Loops loop;
  MPG_Loops c_loop;

  for(iT=0,iRun=0;iT<nT;iT++)
    {
      // if(iT==0)
      //   root_triangle()
      // else
      //   triangle()

      // get marching bit
      int marching = -1;
      switch(getTri(iT))
	{
	case sp_LEFT:  marching = 1; break;
	case sp_RIGHT: marching = 0; break;
	default:                     break;
	}

      if(marching>=0)
	{
	  codec.encode(marching,
		       MPG_Global::td_marching_pattern[prev_marching_pattern]);
	  if(marching != prev_marching_pattern)
	    prev_marching_pattern = !prev_marching_pattern;
	}

      // this is the place to encode
      // the polygon edge according to the syntax !!!

      // if neither root not triangulated get polygon_edge bit
      int polygon_edge = 1;
      if(iT>0&&triangulated==0)
	{
	  polygon_edge = getPolyBit(iT);
	  codec.encode(polygon_edge,
		       MPG_Global::td_polygon_edge[prev_polygon_edge]);
	}
      prev_polygon_edge = polygon_edge;
      if(iT>0 && polygon_edge==0) nPE++;

      // if necessary start a new face
      // iT==0||triangulated==1 => polygon_edge==1
      if(polygon_edge==1)
	{
	  loop.start(iR,iL);

	  // add face to the face tree
	  iF = f_father.size();
	  f_father.push_back(iFp);

	  // add corners to the corner tree
	  cR = iC++; c_father[cR] = cRp; c_traversal.push_back(cR);
	  cL = iC++; c_father[cL] = cLp; c_traversal.push_back(cL);
	  c_loop.start(cR,cL);
	}

      // add corner to the corner tree
      cO = iC++; c_father[cO] = cL; c_traversal.push_back(cO);

      // corners
      //
      // if(polygon_edge==1) {
      //
      //         cO          c_father[cR] = cRp
      //        /   \        c_father[cL] = cLp
      //      /       \      c_father[cO] = cL
      //    cL         cR
      //    -------------
      //    cLP        cRp
      //
      // } else {
      //
      //         cO          c_father[cO] = cL
      //        /   \
                                      //      /       \
                                      //    cL         cR
                                      //    .............
                                      //    cL         cR
                                      //
                                      // }

                                      // determine iO
                                      switch(getTri(iT))
                                        {
                                        case sp_LEAF:   iO =  iL+1;                                 break;
                                        case sp_LEFT:   iO =  (iR+nVloop-1)%nVloop;                 break;
                                        case sp_RIGHT:  iO =  iL+1;                                 break;
                                        case sp_BRANCH: iO = (iR+nVloop-run_length[iRun+1])%nVloop; break;
                                        }
                                      loop.insert(iO);

                                      // encode properties

                                      if(iT == 0)
                                        {
                                          // root_triangle()
                                          //
                                          //       O
                                          //     / F  \
                                          //   /   ^   \
                                          //  L ---|--- R
                                          //       Fp

                                          // encode root_coord()
                                          //
                                          // root_coord() {
                                          //   root_coord_sample()    <--- R
                                          //   coord_sample()         <--- L
                                          //   coord_sample()         <--- O
                                          // }
                                          //
                                          if(pCoord.getBinding() == PB_PER_VERTEX)
                                            {
                                              // int* nR = &(qCoord[3*v_loop_sp[iR]]);
                                              // int* nL = &(qCoord[3*v_loop_sp[iL]]);
                                              // int* nO = &(qCoord[3*v_loop_sp[iO]]);
	      
                                              int* nR;
                                              int* nL;
                                              int* nO;
	      
                                              if(v_visited[v_cluster[nV0+v_loop_sp[iR]]] == false)
                                                {
                                                  nR = tData; tData+=3;
                                                  encodeRootCoordSample(codec,bitsPerCoord,nR);
                                                }
                                              if(v_visited[v_cluster[nV0+v_loop_sp[iL]]] == false)
                                                {
                                                  nL = tData; tData+=3;
                                                  encodeCoordSample(codec,bitsPerCoord,nL);
                                                }
                                              if(v_visited[v_cluster[nV0+v_loop_sp[iO]]] == false)
                                                {
                                                  nO = tData; tData+=3;
                                                  encodeCoordSample(codec,bitsPerCoord,nO);
                                                }
                                            }

                                          // encode root_normal()
                                          //
                                          // root_normal() {
                                          //   if(normal_binding!=0) {   <--- PB_NONE
                                          //     root_normal_sample()
                                          //     if(normal_binding!=2) { <--- PB_PER_FACE
                                          //       normal_sample()
                                          //       normal_sample()
                                          //     }
                                          //   }
                                          // }
                                          //
                                          if(normalBinding == PB_PER_VERTEX)
                                            {
                                              // int* nR = & qNormal[v_loop_sp[iR]];
                                              // int* nL = & qNormal[v_loop_sp[iL]];
                                              // int* nO = & qNormal[v_loop_sp[iO]];

                                              int* nR;
                                              int* nL;
                                              int* nO;

                                              if(v_visited[v_cluster[nV0+v_loop_sp[iR]]] == false)
                                                {
                                                  nR = tData; tData+=1;
                                                  encodeRootNormalSample(codec,bitsPerNormal,nR);
                                                }
                                              if(v_visited[v_cluster[nV0+v_loop_sp[iL]]] == false)
                                                {
                                                  nL = tData; tData+=1;
                                                  encodeNormalSample(codec,bitsPerNormal,nL);
                                                }
                                              if(v_visited[v_cluster[nV0+v_loop_sp[iO]]] == false)
                                                {
                                                  nO = tData; tData+=1;
                                                  encodeNormalSample(codec,bitsPerNormal,nO);
                                                }
                                            }
                                          else if(normalBinding == PB_PER_FACE)
                                            {
                                              // int* nF = & qNormal[iF];
                                              int* nF = tData; tData+=1;
                                              encodeRootNormalSample(codec,bitsPerNormal,nF);
                                            }
                                          else if(normalBinding == PB_PER_CORNER)
                                            {
                                              // int* nR = & qNormal[cR];
                                              // int* nL = & qNormal[cL];
                                              // int* nO = & qNormal[cO];

                                              int* nR = tData; tData+=1;
                                              int* nL = tData; tData+=1;
                                              int* nO = tData; tData+=1;

                                              encodeRootNormalSample(codec,bitsPerNormal,nR);
                                              encodeNormalSample(codec,bitsPerNormal,nL);
                                              encodeNormalSample(codec,bitsPerNormal,nO);
                                            }

                                          // encode root_color()
                                          //
                                          // root_color() {
                                          //   if(color_binding!=0) {   <--- PB_NONE
                                          //     root_color_sample()
                                          //     if(color_binding!=2) { <--- PB_PER_FACE
                                          //       color_sample()
                                          //       color_sample()
                                          //     }
                                          //   }
                                          // }
                                          //
                                          if(colorBinding == PB_PER_VERTEX)
                                            {
                                              // int* nR = & qColor[3*v_loop_sp[iR]];
                                              // int* nL = & qColor[3*v_loop_sp[iL]];
                                              // int* nO = & qColor[3*v_loop_sp[iO]];

                                              int* nR;
                                              int* nL;
                                              int* nO;

                                              if(v_visited[v_cluster[nV0+v_loop_sp[iR]]] == false)
                                                {
                                                  nR = tData; tData+=3;
                                                  encodeRootColorSample(codec,bitsPerColor,nR);
                                                }
                                              if(v_visited[v_cluster[nV0+v_loop_sp[iL]]] == false)
                                                {
                                                  nL = tData; tData+=3;
                                                  encodeColorSample(codec,bitsPerColor,nL);
                                                }
                                              if(v_visited[v_cluster[nV0+v_loop_sp[iO]]] == false)
                                                {
                                                  nO = tData; tData+=3;
                                                  encodeColorSample(codec,bitsPerColor,nO);
                                                }
                                            }
                                          else if(colorBinding == PB_PER_FACE)
                                            {
                                              // int* nF = & qColor[iF];

                                              int* nF = tData; tData+=3;

                                              encodeRootColorSample(codec,bitsPerColor,nF);
                                            }
                                          else if(colorBinding == PB_PER_CORNER)
                                            {
                                              // int* nR = & qColor[3*cR];
                                              // int* nL = & qColor[3*cL];
                                              // int* nO = & qColor[3*cO];

                                              int* nR = tData; tData+=3;
                                              int* nL = tData; tData+=3;
                                              int* nO = tData; tData+=3;

                                              encodeRootColorSample(codec,bitsPerColor,nR);
                                              encodeColorSample(codec,bitsPerColor,nL);
                                              encodeColorSample(codec,bitsPerColor,nO);
                                            }

                                          // encode root_texCoord()
                                          //
                                          // root_texCoord() {
                                          //   if(texCoord_binding!=0) {   <--- PB_NONE
                                          //     root_texCoord_sample()
                                          //     if(texCoord_binding!=2) { <--- PB_PER_FACE
                                          //       texCoord_sample()
                                          //       texCoord_sample()
                                          //     }
                                          //   }
                                          // }
                                          //
                                          if(texCoordBinding == PB_PER_VERTEX)
                                            {
	      
                                              int* nR;
                                              int* nL;
                                              int* nO;

                                              if(v_visited[v_cluster[nV0+v_loop_sp[iR]]] == false)
                                                {
                                                  nR = tData; tData+=2;
                                                  encodeRootTexCoordSample(codec,bitsPerTexCoord,nR);
                                                }
                                              if(v_visited[v_cluster[nV0+v_loop_sp[iL]]] == false)
                                                {
                                                  nL = tData; tData+=2;
                                                  encodeTexCoordSample(codec,bitsPerTexCoord,nL);
                                                }
                                              if(v_visited[v_cluster[nV0+v_loop_sp[iO]]] == false)
                                                {
                                                  nO = tData; tData+=2;
                                                  encodeTexCoordSample(codec,bitsPerTexCoord,nO);
                                                }
                                            }
                                          else if(texCoordBinding == PB_PER_CORNER)
                                            {
                                              // int* nR = & qTexCoord[2*cR];
                                              // int* nL = & qTexCoord[2*cL];
                                              // int* nO = & qTexCoord[2*cO];

                                              int* nR = tData; tData+=2;
                                              int* nL = tData; tData+=2;
                                              int* nO = tData; tData+=2;

                                              encodeRootTexCoordSample(codec,bitsPerTexCoord,nR);
                                              encodeTexCoordSample(codec,bitsPerTexCoord,nL);
                                              encodeTexCoordSample(codec,bitsPerTexCoord,nO);
                                            }

                                          if(has_per_vertex==true)
                                            {
                                              v_visited[v_cluster[nV0+v_loop_sp[iR]]] = true;
                                              v_visited[v_cluster[nV0+v_loop_sp[iL]]] = true;
                                              v_visited[v_cluster[nV0+v_loop_sp[iO]]] = true;
                                            }
                                        }
                                      else
                                        {
                                          // triangle()
                                          //

                                          //       O
                                          //     / F  \
                                          //   /   ^   \
                                          //  L ---|--- R
                                          //   \   Fp  /
                                          //     \   /
                                          //       P

                                          // coord() {
                                          //   if(!visited)
                                          //     coord_sample()
                                          // }
                                          //

                                          // coord_sample() per vertex (always)
                                          if(pCoord.getBinding() == PB_PER_VERTEX)
                                            {
                                              if(v_visited[v_cluster[nV0+v_loop_sp[iO]]] == false)
                                                {
                                                  int* nO = tData; tData+=3;
                                                  encodeCoordSample(codec,bitsPerCoord,nO);
                                                }
                                            }
                                          // normal_sample()
                                          //
                                          if(normalBinding == PB_PER_VERTEX)
                                            {
                                              if(v_visited[v_cluster[nV0+v_loop_sp[iO]]] == false)
                                                {
                                                  int* nO = tData; tData+=1;
                                                  encodeNormalSample(codec,bitsPerNormal,nO);
                                                }
                                            }
                                          else if(normalBinding == PB_PER_FACE)
                                            {
                                              if(polygon_edge==1)
                                                {
                                                  // int* nF  = & qNormal[iF];
                                                  int* nF = tData; tData+=1;
                                                  encodeNormalSample(codec,bitsPerNormal,nF);
                                                }
                                            }
                                          else if(normalBinding == PB_PER_CORNER)
                                            {
                                              if(polygon_edge==1)
                                                {
                                                  // int* nR = & qNormal[cR];
                                                  int* nR = tData; tData+=1;
                                                  encodeNormalSample(codec,bitsPerNormal,nR);

                                                  // int* nL = & qNormal[cL];
                                                  int* nL = tData; tData+=1;
                                                  encodeNormalSample(codec,bitsPerNormal,nL);
                                                }

                                              // int* nO = & qNormal[cO];
                                              int* nO = tData; tData+=1;
                                              encodeNormalSample(codec,bitsPerNormal,nO);
                                            }

                                          // color_sample()
                                          //
                                          if(colorBinding == PB_PER_VERTEX)
                                            {
                                              if(v_visited[v_cluster[nV0+v_loop_sp[iO]]] == false)
                                                {
                                                  int* nO = tData; tData+=3;
                                                  encodeColorSample(codec,bitsPerColor,nO);
                                                }
                                            }
                                          else if(colorBinding == PB_PER_FACE)
                                            {
                                              if(polygon_edge==1)
                                                {
                                                  // int* nF  = & qColor[iF];
                                                  int* nF = tData; tData+=3;
                                                  encodeColorSample(codec,bitsPerColor,nF);
                                                }
                                            }
                                          else if(colorBinding == PB_PER_CORNER)
                                            {
                                              if(polygon_edge==1)
                                                {
                                                  // int* nR = & qColor[3*cR];
                                                  int* nR = tData; tData+=3;
                                                  encodeColorSample(codec,bitsPerColor,nR);

                                                  // int* nL = & qColor[3*cL];
                                                  int* nL = tData; tData+=3;
                                                  encodeColorSample(codec,bitsPerColor,nL);
                                                }

                                              // int* nO = & qColor[3*cO];
                                              int* nO = tData; tData+=3;
                                              encodeColorSample(codec,bitsPerColor,nO);
                                            }

                                          // texCoord_sample()
                                          //
                                          if(texCoordBinding == PB_PER_VERTEX)
                                            {
                                              if(v_visited[v_cluster[nV0+v_loop_sp[iO]]] == false)
                                                {
                                                  int* nO = tData; tData+=2;
                                                  encodeTexCoordSample(codec,bitsPerTexCoord,nO);
                                                }
                                            }
                                          else if(texCoordBinding == PB_PER_CORNER)
                                            {
                                              if(polygon_edge==1)
                                                {
                                                  // int* nR = & qTexCoord[2*cR];
                                                  int* nR = tData; tData+=2;
                                                  encodeTexCoordSample(codec,bitsPerTexCoord,nR);

                                                  // int* nL = & qTexCoord[2*cL];
                                                  int* nL = tData; tData+=2;
                                                  encodeTexCoordSample(codec,bitsPerTexCoord,nL);
                                                }

                                              // int* nO = & qTexCoord[2*cO];
                                              int* nO = tData; tData+=2;
                                              encodeTexCoordSample(codec,bitsPerTexCoord,nO);
                                            }

                                          if(has_per_vertex==true)
                                            v_visited[v_cluster[nV0+v_loop_sp[iO]]] = true;
                                        }

                                      // advance to next triangle
                                      switch(getTri(iT))
                                        {

                                        case sp_LEAF:

                                          // pop an edge from traversal stack
                                          //       ^
                                          //  L ---|--- R
                                          //   \       /
                                          //     \   /
                                          //       P

                                          loop.pop(iR,iL);
                                          c_loop.insert(cO);
                                          c_loop.pop(cR,cL);

                                          cRp  = stk.back(); stk.pop_back();
                                          cLp  = stk.back(); stk.pop_back();
                                          iP   = stk.back(); stk.pop_back();
                                          iFp  = stk.back(); stk.pop_back();

                                          iRun++;		// end of run
                                          break;

                                        case sp_LEFT:

                                          // turn left
                                          //
                                          //       O = R-1
                                          //     /   \
                                          //   /   ^   \
                                          //  L ---|--- R
                                          //   \       /
                                          //     \   /
                                          //       P

                                          iP  = iR;
                                          iR  = iO;
                                          iFp = iF;
                                          c_loop.insert(cO);
                                          cRp = cO;
                                          cLp = cL;
                                          cR  = cRp;
                                          cL  = cLp;
                                          break;

                                        case sp_RIGHT:

                                          // turn right
                                          //
                                          //       O = L+1
                                          //     /   \
                                          //   /   ^   \
                                          //  L ---|--- R
                                          //   \       /
                                          //     \   /
                                          //       P

                                          loop.advance();
                                          iP  = iL;
                                          iL  = iO;
                                          iFp = iF;
                                          c_loop.insert(cO);
                                          c_loop.advance();
                                          cRp = cR;
                                          cLp = cO;
                                          cR  = cRp;
                                          cL  = cLp;
                                          break;

                                        case sp_BRANCH:

                                          //       O
                                          //     /   \
                                          //   /   ^   \
                                          //  L ---|--- R
                                          //   \       /
                                          //     \   /
                                          //       P

                                          // push left edge onto traversal stack
                                          //       ^
                                          //  L ---|--- O
                                          //   \       /
                                          //     \   /
                                          //       R


                                          loop.pushCurr();
                                          stk.push_back(iF);
                                          //original:  stk.push_back(iP);
                                          stk.push_back(iR);

                                          // advance to right edge
                                          //       ^
                                          //  O ---|--- R
                                          //   \       /
                                          //     \   /
                                          //       L

                                          loop.advance();

                                          iFp = iF;
                                          iP = iL;
                                          iL = iO;

                                          c_loop.insert(cO);
                                          c_loop.pushCurr();
                                          c_loop.advance();

                                          stk.push_back(cL);
                                          stk.push_back(cO);

                                          cRp = cR;
                                          cLp = cO;
                                          cR  = cRp;
                                          cL  = cLp;
                                          iRun++;		// end of run
                                          break;
                                        }
    }

  return 1;
}

bool MPG_SPolygon::encodeTriangleData(MPG_QFac&     codec,
				      int           nV0,
				      vector<int>&  v_loop_sp,
				      vector<int>&  v_visited,
				      vector<int>&  v_cluster,
				      MPG_Property& pCoord,
				      MPG_Property& pNormal,
				      MPG_Property& pColor,
				      MPG_Property& pTexCoord,
					  vector<int>&           qCoord,
					  vector<int>&           qNormal,
					  vector<int>&           qColor,
					  vector<int>&           qTexCoord,
					  vector<int>&           v_org_idx,
					  vector<int>&           f_index,	
					  vector<int>&           c_index,	
					  MPG_BranchTree*&		 cur_bt,	
					  vector<int>&           Grstk,
					  vector<int>&           Gistk,
					  int                    &Gdepth,
					  int					 &GiC,
					  int					 &GiF,
					  int					 &prevCR,	//MJH 001013 : added in case of partitioning at non-polygon edge
					  int					 &prevCL,	//MJH 001013 : added in case of partitioning at non-polygon edge
					  int                    availbits, 
					  MPG_PacketQ_C&         codap
					  )
{
  MPG_Binding coordBinding    = pCoord.getBinding();
  MPG_Binding normalBinding   = pNormal.getBinding();
  MPG_Binding colorBinding    = pColor.getBinding();
  MPG_Binding texCoordBinding = pTexCoord.getBinding();
  int bitsPerCoord    = (pCoord.getBBoxQ()!=(MPG_BBoxQ*)0)?
    pCoord.getBBoxQ()->getBitsPerCoordinate():0;
  int bitsPerNormal   = (pNormal.getNormalQ()!=(MPG_NormalQ*)0)?
    pNormal.getNormalQ()->getBitsPerNormal():0;
  int bitsPerColor    = (pColor.getBBoxQ()!=(MPG_BBoxQ*)0)?
    pColor.getBBoxQ()->getBitsPerCoordinate():0;
  int bitsPerTexCoord = (pTexCoord.getBBoxQ()!=(MPG_BBoxQ*)0)?
    pTexCoord.getBBoxQ()->getBitsPerCoordinate():0;
  
  // determine if SPolygon has properties per vertex
  bool has_per_vertex = (pCoord.getBinding()    == PB_PER_VERTEX ||
			 pNormal.getBinding()   == PB_PER_VERTEX ||
			 pColor.getBinding()    == PB_PER_VERTEX ||
			 pTexCoord.getBinding() == PB_PER_VERTEX)?true:false;

  //  triangle_data(i) {
  //	 qf_decode(triangulated, triangulated_context)
  //	 depth=0
  //	 root_triangle()
  //	 for (i=1; i<ntriangles; i++)
  //		 triangle(i)
  // }
  int nT     = getNumberOfTriangles();
  int nVloop = getNumberOfVertices();
  if (has_per_vertex==true && nVloop!=v_loop_sp.size()) return false;
  int nRuns  = getNumberOfRuns();
  int iP = -1;
  int iR =  0;
  int iL =  1;
  int iO = -1;
  int iT=0, iRun=0, ldepth=0, iS=0;
  int tdbits=0, sub_bits=0, sub_ptcount=0;
  int overflow=0, v_update=0;
  int gdepth=Gdepth;
  vector<int>  stk_direct((size_t)1,-1);
  vector<int>  visited_idx2;
  vector<int>  visited_idx1;
  
  iS = codap.start_tri;
  iRun = codap.start_run;
  iL = codap.sp_left_bloop_idx;
  iR = codap.sp_right_bloop_idx;
  int p_run=iRun, p_tri=iS;
  int return_run=iS, return_tri=iRun;
  int p_branch=0, p_leaf=0;
  int next_pt=0;
  int prev_marching_pattern = 0;
  int prev_polygon_edge     = 1;
  int triangulated = (hasDirectPBits(codap)==false) ? 1:0;   
  codec.encode(triangulated,MPG_Global::tt_triangulated);

  // face tree array
  vector<int>  f_father;
  // corner tree arrays
  vector<int>  c_traversal;
  vector<int>  c_father;

  // traversal stack
  vector<int>  stk((size_t) 1,-1);
  int nPE    = 0;    // number of {non_root polygon_edge==0}
  int nF     = nT;   // - nPE
  int nC     = 3*nT; // - 2*nPE
  int iF  = f_father.size();	//==0
  
  if (GiF>=0) //MJH 001011 : added for per face
	iF += GiF;                  
  else {
	iF = -1;
  }
  int iFp = iF+1;

  int nC0  =  c_father.size(); // next free corner index
  nC0 += GiC;                  
  int iC   = nC0;
  int cRp  =  iC;
  int cR   =  iC;
  int cLp  =  iC; //MJH 062600 : removed "+1";
  int cL   =  iC+1;
  int cO   = -1;
  
  f_father.insert(f_father.end(),(size_t)(iFp+nF),-1);
  c_father.insert(c_father.end(),(size_t)(nC0+nC),-1);
  MPG_Loops loop;
  MPG_Loops c_loop;
  for(iT=0 ;iT<nT;iT++) 
  {
      // if(iT==0) {
	  //	if (marching_triangle)
	  //	    qf_decode(marching_pattern,marching_pattern_context[marching_pattern])
	  //	else {
	  //	    if (3D_MOBL_start_code == "partition_type_2")
	  //		    if (tt_leaf == '0' && depth==0)
	  //			    qf_decode(td_orientation, td_orientation_context)
	  //	    if (tt_leaf == '0')
	  //		    depth++
	  //	    else
	  //		    depth--
	  //	}
	  //	root_triangle() {
	  //	    if (3D_MOBL_start_code == "partition_type_2")
	  //		    if (triangulated == '0')
	  //			    qf_decode(polygon_edge,polygon_edge_context[polygon_edge])
	  //    }
      // } else {
      //    triangle() {
	  //	    if (triangulated == '0')
	  //	       qf_decode(polygon_edge,polygon_edge_context[polygon_edge])
	  //    }
	  // }
	  if(iT!=0) tdbits = codec.cbits; 
	  int td_orientation=1;  //left_first
	  
      // get marching bit
      int marching = -1;
      switch(getTri(iS))     
	  {
	  case sp_LEFT:  marching = 1; break;
	  case sp_RIGHT: marching = 0; break;
	  default:                     break;
	  }
	  	  
	  if(marching>=0)	{
		  codec.encode(marching,
			  MPG_Global::td_marching_pattern[prev_marching_pattern]);
		  if(marching != prev_marching_pattern)
			  prev_marching_pattern = !prev_marching_pattern;
	  } else {  
		  if(getTri(iS)==sp_BRANCH && ldepth==0) { 
			  td_orientation = tdirect.getDirect(iRun);
			  if(codap.sp_pt2_enable)  
				  codec.encode(td_orientation,MPG_Global::td_orientation_context);
		  }
	  }
	  
      // this is the place to encode
      // the polygon edge according to the syntax !!!
      // if neither root not triangulated get polygon_edge bit
      int polygon_edge = 1;
	  if(triangulated==0 && (iT>0 || (iT==0 && codap.sp_pt2_enable))) {  
		  polygon_edge = getPolyBit(iS);  
		  codec.encode(polygon_edge, MPG_Global::td_polygon_edge[prev_polygon_edge]);
	  }
	  prev_polygon_edge = polygon_edge;
	  if(polygon_edge==0) nPE++;	//MJH 000111 : moved to else below
	  	  
      // if necessary start a new face
      if(polygon_edge==1 || iT==0)      
	  {
		  loop.start(iR,iL);
		  // add face to the face tree
		  if (polygon_edge==0 && iS) //MJH 001011 : added for partitioning at non-polygon edge
			  iFp--;
		  else
			  iF++;	//MJH 001005 : added for polygon models
		  f_father[iF] = iFp;       

		  // add corners to the corner tree
		  if (iS && polygon_edge==0) //MJH 001011 : added for partitioning at non-polygon edge
		  {
			cRp = prevCR;
			cLp = prevCR;
			cR = prevCR; 
			cL = prevCL; 
			c_father[cR] = cRp; c_traversal.push_back(cR);
			c_father[cL] = cLp; c_traversal.push_back(cL);
			c_loop.start(cR,cL);
		  }
		  else
		  {
			cR = iC++; c_father[cR] = cRp; c_traversal.push_back(cR);
			cL = iC++; c_father[cL] = cLp; c_traversal.push_back(cL);
			c_loop.start(cR,cL);
		  }
	  }
	  else { //MJH 001005 : added for polygon models
			iF = iFp;
//			nPE++;
	  }
	  
      // add corner to the corner tree
      cO = iC++; c_father[cO] = cL; c_traversal.push_back(cO);

      // corners
      // if(polygon_edge==1) {
      //
      //         cO          c_father[cR] = cRp
      //        /   \        c_father[cL] = cLp
      //      /       \      c_father[cO] = cL
      //    cL         cR
      //    -------------
      //    cLP        cRp
      //
      // } else {
      //
      //         cO          c_father[cO] = cL
      //        /   \
	  //      /       \
	  //    cL         cR
	  //    .............
	  //    cL         cR
	  //
	  // }
	  
	  // determine iO
	  switch(getTri(iS))
	  {
	  case sp_LEAF:   iO =  iL+1;                                 break;
	  case sp_LEFT:   iO =  (iR+nVloop-1)%nVloop;                 break;
	  case sp_RIGHT:  iO =  iL+1;                                 break;
	  case sp_BRANCH: 
		  if(codap.sp_pt2_enable) {  
			  if(td_orientation) {
				  iO = (iR+nVloop-tdirect.getLength(iRun+1))%nVloop;
			  } else {
				  int r = tdirect.getFirst(iRun);
				  iO = (iL+tdirect.getLength(r))%nVloop;
			  }
		  } else {                 //pt0
			  iO = (iR+nVloop-tdirect.getLength(iRun+1))%nVloop;
		  };  
		  break;
	  }
	  loop.insert(iO);
	  
	  // encode properties
	  if(iT == 0)
	  {
		  // root_coord()
		  // root_normal()
		  // root_color()
		  // root_texCoord()
		  //
		  //       O
		  //     / F  \
		  //   /   ^   \
		  //  L ---|--- R
		  //       Fp
		  // root_coord() {
		  //	if (3D_MOBL_start_code == "partition_type_2") {
		  //		if (visited[vertex_index] == 0) {
		  //			root_coord_sample()
		  //		    if (visited[vertex_index] == 0) {
		  //				coord_sample()
		  //				coord_sample()
		  //		    } 
		  //		}
		  //	} else {
		  //		root_coord_sample() <--- R
		  //		coord_sample()      <--- L
		  //		coord_sample()      <--- O
		  //	}
		  // }
		  {
			  int* nR = &(qCoord[3*v_org_idx[nV0+v_loop_sp[iR]]]);
			  int* nL = &(qCoord[3*v_org_idx[nV0+v_loop_sp[iL]]]);
			  int* nO = &(qCoord[3*v_org_idx[nV0+v_loop_sp[iO]]]);
			  if(v_visited[v_cluster[nV0+v_loop_sp[iR]]] == AVAIL_PRED ||
				  (v_visited[v_cluster[nV0+v_loop_sp[iR]]] == ALREADY_CODED && codap.sp_bdry_pred)){  
				  encodeRootCoordSample(codec,bitsPerCoord,nR);
				  if(v_visited[v_cluster[nV0+v_loop_sp[iL]]] == AVAIL_PRED ||
					  (v_visited[v_cluster[nV0+v_loop_sp[iL]]] == ALREADY_CODED && codap.sp_bdry_pred)){  
					  pCoord.predictC(nR,nR,nR,nL);
					  encodeCoordSample(codec,bitsPerCoord,nL);
					  pCoord.predictE(nR,nR,nR,nL);	
					  pCoord.predictC(nR,nR,nL,nO);
					  encodeCoordSample(codec,bitsPerCoord,nO);
					  pCoord.predictE(nR,nR,nL,nO);	
				  }
			  }
			  else if(v_visited[v_cluster[nV0+v_loop_sp[iO]]] == AVAIL_PRED ||
				  (v_visited[v_cluster[nV0+v_loop_sp[iO]]] == ALREADY_CODED && codap.sp_bdry_pred)){  
				  encodeRootCoordSample(codec,bitsPerCoord,nO);
			  }
		  }

		  // root_normal() {
		  //	if (normal_binding != "not_bound")  <--- PB_NONE
		  //	   if (3D_MOBL_start_code == "partition_type_2") {
		  //	      if (normal_binding != "bound_per_vertex" ||	visited[vertex_index] == 0) {
		  //			 root_normal_sample()
		  //		     if (normal_binding != "bound_per_face" && 
		  //                 (normal_binding != "bound_per_vertex" || visited[vertex_index] == 0)) {
		  //			    normal_sample()
		  //			    normal_sample()
		  //			 }
		  //		} else {
		  //			 root_normal_sample() 
		  //			 if (normal_binding != "bound_per_face") { <--- PB_PER_FACE
		  //			    normal_sample()
		  //			    normal_sample()
		  //			 }
		  //		}
		  //    }
		  // }
		  if(normalBinding == PB_PER_VERTEX)
		  {
			  int* nR = & qNormal[v_org_idx[nV0+v_loop_sp[iR]]];
			  int* nL = & qNormal[v_org_idx[nV0+v_loop_sp[iL]]];
			  int* nO = & qNormal[v_org_idx[nV0+v_loop_sp[iO]]];
			  if(v_visited[v_cluster[nV0+v_loop_sp[iR]]] == AVAIL_PRED ||
				  (v_visited[v_cluster[nV0+v_loop_sp[iR]]] == ALREADY_CODED && codap.sp_bdry_pred)){  
				  encodeRootNormalSample(codec,bitsPerNormal,nR);
				  if(v_visited[v_cluster[nV0+v_loop_sp[iL]]] == AVAIL_PRED ||
					  (v_visited[v_cluster[nV0+v_loop_sp[iL]]] == ALREADY_CODED && codap.sp_bdry_pred)){  
					  pNormal.predictC(nR,nR,nR,nL);
					  encodeNormalSample(codec,bitsPerNormal,nL);
					  pNormal.predictE(nR,nR,nR,nL);
					  pNormal.predictC(nR,nR,nL,nO);
					  encodeNormalSample(codec,bitsPerNormal,nO);
					  pNormal.predictE(nR,nR,nL,nO);
				  }
			  }
			  else if(v_visited[v_cluster[nV0+v_loop_sp[iO]]] == AVAIL_PRED ||
				  (v_visited[v_cluster[nV0+v_loop_sp[iO]]] == ALREADY_CODED && codap.sp_bdry_pred)){ 
				  encodeRootNormalSample(codec,bitsPerNormal,nO);
			  }
		  }
		  else if(normalBinding == PB_PER_FACE)
		  {
			  int* nF = & qNormal[f_index[iF]];	
			  encodeRootNormalSample(codec,bitsPerNormal,nF);
		  }
		  else if(normalBinding == PB_PER_CORNER)
		  {
			  //MJH 062600 : swapped L and R
			  int* nR = & qNormal[c_index[cR]];	 
			  int* nL = & qNormal[c_index[cL]];	 
			  int* nO = & qNormal[c_index[cO]];	 
			  encodeRootNormalSample(codec,bitsPerNormal,nR);
			  encodeNormalSample(codec,bitsPerNormal,nL);
			  encodeNormalSample(codec,bitsPerNormal,nO);
		  }
		  
		  // root_color() {
		  //	  if (color_binding != "not_bound")  <--- PB_NONE
		  //	 	 if (3D_MOBL_start_code == "partition_type_2") {
		  //	        if (color_binding != "bound_per_vertex" || visited[vertex_index] == 0) {
		  //				root_color_sample()
		  //			    if (color_binding != "bound_per_face" && 
		  //		            (color_binding != "bound_per_vertex" || visited[vertex_index] == 0)) {
		  //				    color_sample()
		  //				    color_sample()
		  //			    }
		  //		    } else {
		  //			    root_color_sample() 
		  //			    if (color_binding != "bound_per_face") { <--- PB_PER_FACE
		  //				    color_sample()
		  //				    color_sample()
		  //			    }
		  //		    }
		  //      }
		  // }
		  if(colorBinding == PB_PER_VERTEX)
		  {
			  int* nR = & qColor[3*v_org_idx[nV0+v_loop_sp[iR]]];
			  int* nL = & qColor[3*v_org_idx[nV0+v_loop_sp[iL]]];
			  int* nO = & qColor[3*v_org_idx[nV0+v_loop_sp[iO]]];
			  if(v_visited[v_cluster[nV0+v_loop_sp[iR]]] == AVAIL_PRED ||
				  (v_visited[v_cluster[nV0+v_loop_sp[iR]]] == ALREADY_CODED && codap.sp_bdry_pred)){  
				  encodeRootColorSample(codec,bitsPerColor,nR);
				  if(v_visited[v_cluster[nV0+v_loop_sp[iL]]] == AVAIL_PRED ||
					  (v_visited[v_cluster[nV0+v_loop_sp[iL]]] == ALREADY_CODED && codap.sp_bdry_pred)){  
					  pColor.predictC(nR,nR,nR,nL);
					  encodeColorSample(codec,bitsPerColor,nL);
					  pColor.predictE(nR,nR,nR,nL);
					  pColor.predictC(nR,nR,nL,nO);
					  encodeColorSample(codec,bitsPerColor,nO);
					  pColor.predictE(nR,nR,nL,nO);
				  }
			  }
			  else if(v_visited[v_cluster[nV0+v_loop_sp[iO]]] == AVAIL_PRED ||
				  (v_visited[v_cluster[nV0+v_loop_sp[iO]]] == ALREADY_CODED && codap.sp_bdry_pred)){  
				  encodeRootColorSample(codec,bitsPerColor,nO);
			  }
		  }
		  else if(colorBinding == PB_PER_FACE)
		  {
			  int* nF = & qColor[3*f_index[iF]];	
			  encodeRootColorSample(codec,bitsPerColor,nF);
		  }
		  else if(colorBinding == PB_PER_CORNER)
		  {
			  //MJH 062600 : swapped L and R
			  int* nR = & qColor[3*c_index[cR]];
			  int nL[3], nO[3];
			  encodeRootColorSample(codec,bitsPerColor,nR);
			  pColor.predictC(c_father,qColor,cL,nL,c_index);
			  encodeColorSample(codec,bitsPerColor,nL);
			  pColor.predictC(c_father,qColor,cO,nO,c_index); 
			  encodeColorSample(codec,bitsPerColor,nO);
		  }
		  
		  // root_texCoord() {
		  //	  if (texCoord_binding != "not_bound")  <--- PB_NONE
		  //		 if (3D_MOBL_start_code == "partition_type_2") {
		  //	         if (texCoord_binding != "bound_per_vertex" || 
		  //		         visited[vertex_index] == 0) {
		  //				root_texCoord_sample()
		  //			    if (texCoord_binding!= "bound_per_vertex" || 
		  //		            visited[vertex_index] == 0) {  <--- PB_PER_FACE
		  //					texCoord_sample()
		  //					texCoord_sample()
		  //				}
		  //		     } else {
		  //			    root_texCoord_sample() 
		  //			    texCoord_sample()
		  //			    texCoord_sample()
		  //		     }
		  //      }
		  // }
		  if(texCoordBinding == PB_PER_VERTEX)
		  {
			  int* nR = & qTexCoord[2*v_org_idx[nV0+v_loop_sp[iR]]];
			  int* nL = & qTexCoord[2*v_org_idx[nV0+v_loop_sp[iL]]];
			  int* nO = & qTexCoord[2*v_org_idx[nV0+v_loop_sp[iO]]];
			  if(v_visited[v_cluster[nV0+v_loop_sp[iR]]] == AVAIL_PRED ||
				  (v_visited[v_cluster[nV0+v_loop_sp[iR]]] == ALREADY_CODED && codap.sp_bdry_pred)){  
				  encodeRootTexCoordSample(codec,bitsPerTexCoord,nR);
				  if(v_visited[v_cluster[nV0+v_loop_sp[iL]]] == AVAIL_PRED ||
					  (v_visited[v_cluster[nV0+v_loop_sp[iL]]] == ALREADY_CODED && codap.sp_bdry_pred)){  
					  pTexCoord.predictC(nR,nR,nR,nL);
					  encodeTexCoordSample(codec,bitsPerTexCoord,nL);
					  pTexCoord.predictE(nR,nR,nR,nL);
					  pTexCoord.predictC(nR,nR,nL,nO);
					  encodeTexCoordSample(codec,bitsPerTexCoord,nO);
					  pTexCoord.predictE(nR,nR,nL,nO);
				  }
			  }
			  else if(v_visited[v_cluster[nV0+v_loop_sp[iO]]] == AVAIL_PRED ||
				  (v_visited[v_cluster[nV0+v_loop_sp[iO]]] == ALREADY_CODED && codap.sp_bdry_pred)){  
				  encodeRootTexCoordSample(codec,bitsPerTexCoord,nO);
			  }
		  }
		  else if(texCoordBinding == PB_PER_CORNER)
		  {
			  //MJH 062600 : swapped L and R
			  int* nR = & qTexCoord[2*c_index[cR]];
			  int nL[2], nO[2];
			  encodeRootTexCoordSample(codec,bitsPerTexCoord,nR);
			  pTexCoord.predictC(c_father,qTexCoord,cL,nL,c_index);	 
			  encodeTexCoordSample(codec,bitsPerTexCoord,nL);
			  pTexCoord.predictC(c_father,qTexCoord,cO,nO,c_index);	
			  encodeTexCoordSample(codec,bitsPerTexCoord,nO);
		  }
		  if(v_visited[v_cluster[nV0+v_loop_sp[iR]]]==AVAIL_PRED) {
			  visited_idx2.push_back(v_cluster[nV0+v_loop_sp[iR]]);
			  v_visited[v_cluster[nV0+v_loop_sp[iR]]] |= ALREADY_CODED + AVAIL_PRED;  
		  }
		  if(v_visited[v_cluster[nV0+v_loop_sp[iL]]]==AVAIL_PRED) {
			  visited_idx2.push_back(v_cluster[nV0+v_loop_sp[iL]]);
			  v_visited[v_cluster[nV0+v_loop_sp[iL]]] |= ALREADY_CODED + AVAIL_PRED;  
		  }
		  if(v_visited[v_cluster[nV0+v_loop_sp[iO]]]==AVAIL_PRED) {
			  visited_idx2.push_back(v_cluster[nV0+v_loop_sp[iO]]);
			  v_visited[v_cluster[nV0+v_loop_sp[iO]]] |= ALREADY_CODED + AVAIL_PRED;  
		  }
		  if(codap.sp_bdry_pred) {
			  if(v_visited[v_cluster[nV0+v_loop_sp[iR]]]==ALREADY_CODED) {
				  visited_idx1.push_back(v_cluster[nV0+v_loop_sp[iR]]);
				  v_visited[v_cluster[nV0+v_loop_sp[iR]]] |= ALREADY_CODED + AVAIL_PRED;  
			  }
			  if(v_visited[v_cluster[nV0+v_loop_sp[iL]]]==ALREADY_CODED) {
				  visited_idx1.push_back(v_cluster[nV0+v_loop_sp[iL]]);
				  v_visited[v_cluster[nV0+v_loop_sp[iL]]] |= ALREADY_CODED + AVAIL_PRED;  
			  }
			  if(v_visited[v_cluster[nV0+v_loop_sp[iO]]]==ALREADY_CODED) {
				  visited_idx1.push_back(v_cluster[nV0+v_loop_sp[iO]]);
				  v_visited[v_cluster[nV0+v_loop_sp[iO]]] |= ALREADY_CODED + AVAIL_PRED;  
			  }
		  } 
      }	else {
		  // coord()
		  // normal()
		  // color()
		  // texCoord()
		  //
		  //       O
		  //     / F  \
		  //   /   ^   \
		  //  L ---|--- R
		  //   \   Fp  /
		  //     \   /
		  //       P
		  //
		  // coord() {
		  //	  if (3D_MOBL_start_code == "partition_type_2") {
		  //		 if (visited[vertex_index] == 0)
		  //			 if (no_ancestors)
		  //				root_coord_sample()
		  //			 else
		  //				coord_sample()
		  //	  } else {
		  //		 if (visited[vertex_index] == 0)
		  //			 coord_sample()   <- per vertex (always)
		  //	  }
		  // }
		  // coord_sample() per vertex (always)
		  int iPavail = ((v_visited[v_cluster[nV0+v_loop_sp[iP]]] & (AVAIL_PRED+ALREADY_CODED)) == AVAIL_PRED+ALREADY_CODED);
		  int iRavail = ((v_visited[v_cluster[nV0+v_loop_sp[iR]]] & (AVAIL_PRED+ALREADY_CODED)) == AVAIL_PRED+ALREADY_CODED);
		  int iLavail = ((v_visited[v_cluster[nV0+v_loop_sp[iL]]] & (AVAIL_PRED+ALREADY_CODED)) == AVAIL_PRED+ALREADY_CODED);
		  if(pCoord.getBinding() == PB_PER_VERTEX)
		  {
			  if((v_visited[v_cluster[nV0+v_loop_sp[iO]]] == AVAIL_PRED) ||
				  (v_visited[v_cluster[nV0+v_loop_sp[iO]]] == ALREADY_CODED && codap.sp_bdry_pred)){  
				  int* nO = &(qCoord[3*v_org_idx[nV0+v_loop_sp[iO]]]);
				  if (iPavail + iRavail + iLavail == 0) {
					  encodeRootCoordSample(codec,bitsPerCoord,nO);
				  } else {
					  if (iPavail && iRavail && iLavail) {
						  int* nP = &(qCoord[3*v_org_idx[nV0+v_loop_sp[iP]]]);
						  int* nR = &(qCoord[3*v_org_idx[nV0+v_loop_sp[iR]]]);
						  int* nL = &(qCoord[3*v_org_idx[nV0+v_loop_sp[iL]]]);
						  pCoord.predictC(nP,nL,nR,nO);
						  encodeCoordSample(codec,bitsPerCoord,nO);
						  pCoord.predictE(nP,nL,nR,nO);
					  }
					  else if (iPavail+iRavail+iLavail == 2)
					  {
						  if (iPavail) {
							  int* nRL;
							  if (iRavail)
								  nRL = &(qCoord[3*v_org_idx[nV0+v_loop_sp[iR]]]);
							  else
								  nRL = &(qCoord[3*v_org_idx[nV0+v_loop_sp[iL]]]);
							  pCoord.predictC(nRL,nRL,nRL,nO);
							  encodeCoordSample(codec,bitsPerCoord,nO);
							  pCoord.predictE(nRL,nRL,nRL,nO);
						  } else {
							  int* nR = &(qCoord[3*v_org_idx[nV0+v_loop_sp[iR]]]);
							  int* nL = &(qCoord[3*v_org_idx[nV0+v_loop_sp[iL]]]);
							  pCoord.predictC2(nR,nL,nO);
							  encodeCoordSample(codec,bitsPerCoord,nO);
							  pCoord.predictE2(nR,nL,nO);
						  }
					  } else {
						  int* nPRL;
						  if (iPavail)
							  nPRL = &(qCoord[3*v_org_idx[nV0+v_loop_sp[iP]]]);
						  else if (iRavail)
							  nPRL = &(qCoord[3*v_org_idx[nV0+v_loop_sp[iR]]]);
						  else
							  nPRL = &(qCoord[3*v_org_idx[nV0+v_loop_sp[iL]]]);
						  pCoord.predictC(nPRL,nPRL,nPRL,nO);
						  encodeCoordSample(codec,bitsPerCoord,nO);
						  pCoord.predictE(nPRL,nPRL,nPRL,nO);
					  }
				  }
			  }
		  }

		  // normal() {
		  //	  if (normal_binding == "bound_per_vertex") {
		  //		 if (3D_MOBL_start_code == "partition_type_2") {
		  //			if (visited[vertex_index] == 0)
		  //			   if (no_ancestors)
		  //			  	  root_normal_sample()
		  //			   else
		  //			 	  normal_sample()
		  //		 } else {
		  //			if (visited[vertex_index] == 0)
		  //				normal_sample()
		  //		 }
		  //	  } else if (normal_binding == "bound_per_face") {
		  //		 if (triangulated == '1' || polygon_edge == '1')
		  //			normal_sample()
		  //	  } else if (normal_binding == "bound_per_corner") {
		  //		 if (triangulated == '1' || polygon_edge == '1') {
		  //			normal_sample()
		  //			normal_sample()
		  //		 }
		  //		 normal_sample()
		  //	  }
		  // }
		  if(normalBinding == PB_PER_VERTEX)
		  {
			  if((v_visited[v_cluster[nV0+v_loop_sp[iO]]] == AVAIL_PRED) ||
				  (v_visited[v_cluster[nV0+v_loop_sp[iO]]] == ALREADY_CODED && codap.sp_bdry_pred)){  
				  int* nO = & qNormal[v_org_idx[nV0+v_loop_sp[iO]]];
 				  if (iPavail + iRavail + iLavail == 0) {
					  encodeRootNormalSample(codec,bitsPerNormal,nO);
				  } else {
					  if (iPavail && iRavail && iLavail) {
						  int* nP = & qNormal[v_org_idx[nV0+v_loop_sp[iP]]];
						  int* nR = & qNormal[v_org_idx[nV0+v_loop_sp[iR]]];
						  int* nL = & qNormal[v_org_idx[nV0+v_loop_sp[iL]]];
						  pNormal.predictC(nP,nL,nR,nO);
						  encodeNormalSample(codec,bitsPerNormal,nO);
						  pNormal.predictE(nP,nL,nR,nO);
					  }
					  else if (iPavail+iRavail+iLavail == 2)
					  {
						  if (iPavail) {
							  int* nRL;
							  if (iRavail)
								  nRL = & qNormal[v_org_idx[nV0+v_loop_sp[iR]]];
							  else
								  nRL = & qNormal[v_org_idx[nV0+v_loop_sp[iL]]];
							  pNormal.predictC(nRL,nRL,nRL,nO);
							  encodeNormalSample(codec,bitsPerNormal,nO);
							  pNormal.predictE(nRL,nRL,nRL,nO);
						  } else {
							  int* nR = & qNormal[v_org_idx[nV0+v_loop_sp[iR]]];
							  int* nL = & qNormal[v_org_idx[nV0+v_loop_sp[iL]]];
							  pNormal.predictC2(nR,nL,nO);
							  encodeNormalSample(codec,bitsPerNormal,nO);
							  pNormal.predictE2(nR,nL,nO);
						  }
					  } else {
						  int* nPRL;
						  if (iPavail)
							  nPRL = & qNormal[v_org_idx[nV0+v_loop_sp[iP]]];
						  else if (iRavail)
							  nPRL = & qNormal[v_org_idx[nV0+v_loop_sp[iR]]];
						  else
							  nPRL = & qNormal[v_org_idx[nV0+v_loop_sp[iL]]];
						  pNormal.predictC(nPRL,nPRL,nPRL,nO);
						  encodeNormalSample(codec,bitsPerNormal,nO);
						  pNormal.predictE(nPRL,nPRL,nPRL,nO);
					  }
				  }
			  }
		  }
		  else if(normalBinding == PB_PER_FACE)
		  {
			  if(polygon_edge==1)
			  {
				  int* nF = & qNormal[f_index[iF]];	
				  encodeNormalSample(codec,bitsPerNormal,nF);
			  }
		  }
		  else if(normalBinding == PB_PER_CORNER)
		  {
			  if(polygon_edge==1)
			  {
				  //MJH 062600 : swapped L and R
				  int* nR = & qNormal[c_index[cR]];	 
				  encodeNormalSample(codec,bitsPerNormal,nR);
				  int* nL = & qNormal[c_index[cL]];	 
				  encodeNormalSample(codec,bitsPerNormal,nL);
			  }
			  int* nO = & qNormal[c_index[cO]];	 
			  encodeNormalSample(codec,bitsPerNormal,nO);
		  }
		  
		  // color() {
		  //	 if (color_binding == "bound_per_vertex") {
		  //		if (3D_MOBL_start_code == "partition_type_2") {
		  //		   if (visited[vertex_index] == 0)
		  //			  if (no_ancestors)
		  //				 root_color_sample()
		  //			  else
		  //				 color_sample()
		  //		} else {
		  //			if (visited[vertex_index] == 0)
		  //				color_sample()
		  //		}
		  //	 } else if (color_binding == "bound_per_face") {
		  //		if (triangulated == '1' || polygon_edge == '1')
		  //			color_sample()
		  //	 } else if (color_binding == "bound_per_corner") {
		  //		if (triangulated == '1' || polygon_edge == '1') {
		  //			color_sample()
		  //			color_sample()
		  //		}
		  //		color_sample()
		  //	 }
		  // }
		  if(colorBinding == PB_PER_VERTEX)
		  {
			  if((v_visited[v_cluster[nV0+v_loop_sp[iO]]] == AVAIL_PRED) ||
				  (v_visited[v_cluster[nV0+v_loop_sp[iO]]] == ALREADY_CODED && codap.sp_bdry_pred)){  
				  int* nO = & qColor[3*v_org_idx[nV0+v_loop_sp[iO]]];
				  if (iPavail + iRavail + iLavail == 0) {
					  encodeRootColorSample(codec,bitsPerColor,nO);
				  } else {
					  if (iPavail && iRavail && iLavail) {
						  int* nP = & qColor[3*v_org_idx[nV0+v_loop_sp[iP]]];
						  int* nR = & qColor[3*v_org_idx[nV0+v_loop_sp[iR]]];
						  int* nL = & qColor[3*v_org_idx[nV0+v_loop_sp[iL]]];
						  pColor.predictC(nP,nL,nR,nO);
						  encodeColorSample(codec,bitsPerColor,nO);
						  pColor.predictE(nP,nL,nR,nO);
					  }
					  else if (iPavail+iRavail+iLavail == 2)
					  {
						  if (iPavail) {
							  int* nRL;
							  if (iRavail)
								  nRL = & qColor[3*v_org_idx[nV0+v_loop_sp[iR]]];
							  else
								  nRL = & qColor[3*v_org_idx[nV0+v_loop_sp[iL]]];
							  pColor.predictC(nRL,nRL,nRL,nO);
							  encodeColorSample(codec,bitsPerColor,nO);
							  pColor.predictE(nRL,nRL,nRL,nO);
						  } else {
							  int* nR = & qColor[3*v_org_idx[nV0+v_loop_sp[iR]]];
							  int* nL = & qColor[3*v_org_idx[nV0+v_loop_sp[iL]]];
							  pColor.predictC2(nR,nL,nO);
							  encodeColorSample(codec,bitsPerColor,nO);
							  pColor.predictE2(nR,nL,nO);
						  }
					  } else {
						  int* nPRL;
						  if (iPavail)
							  nPRL = & qColor[3*v_org_idx[nV0+v_loop_sp[iP]]];
						  else if (iRavail)
							  nPRL = & qColor[3*v_org_idx[nV0+v_loop_sp[iR]]];
						  else
							  nPRL = & qColor[3*v_org_idx[nV0+v_loop_sp[iL]]];
						  pColor.predictC(nPRL,nPRL,nPRL,nO);
						  encodeColorSample(codec,bitsPerColor,nO);
						  pColor.predictE(nPRL,nPRL,nPRL,nO);
					  }
				  }
			  }
		  }
		  else if(colorBinding == PB_PER_FACE)
		  {
			  if(polygon_edge==1)
			  {
				  int nF[3];	                            
				  pColor.predictC(f_father,qColor,iF,nF,f_index);   
				  encodeColorSample(codec,bitsPerColor,nF);
			  }
		  }
		  else if(colorBinding == PB_PER_CORNER)
		  {
			  if(polygon_edge==1)
			  {
				  //MJH 062600 : swapped L and R
				  int nR[3];  
				  pColor.predictC(c_father,qColor,cR,nR,c_index);	
				  encodeColorSample(codec,bitsPerColor,nR);

				  int nL[3];  
				  pColor.predictC(c_father,qColor,cL,nL,c_index);	 
				  encodeColorSample(codec,bitsPerColor,nL);
			  }
			  int nO[3];                              
			  pColor.predictC(c_father,qColor,cO,nO,c_index); 
			  encodeColorSample(codec,bitsPerColor,nO);
		  }
		  
		  // texCoord() {
		  //	  if (texCoord_binding == "bound_per_vertex") {
		  //		 if (3D_MOBL_start_code == "partition_type_2") {
		  //			if (visited[vertex_index] == 0)
		  //				if (no_ancestors)
		  //					root_texCoord_sample()
		  //				else
		  //					texCoord_sample()
		  //		}
		  //		else {
		  //			if (visited[vertex_index] == 0)
		  //				texCoord_sample()
		  //		}
		  // 	 } else if (texCoord_binding == "bound_per_corner") {
		  //		if (triangulated == '1' || polygon_edge == '1') {
		  //			texCoord_sample()
		  //			texCoord_sample()
		  //		}
		  //		texCoord_sample()
		  //	 }
		  // }
		  if(texCoordBinding == PB_PER_VERTEX)
		  {
			  if((v_visited[v_cluster[nV0+v_loop_sp[iO]]] == AVAIL_PRED) ||
				  (v_visited[v_cluster[nV0+v_loop_sp[iO]]] == ALREADY_CODED && codap.sp_bdry_pred)){  
				  int* nO = & qTexCoord[2*v_org_idx[nV0+v_loop_sp[iO]]];
				  if (iPavail + iRavail + iLavail == 0) {
					  encodeRootTexCoordSample(codec,bitsPerTexCoord,nO);
				  } else {
					  if (iPavail && iRavail && iLavail) {
						  int* nP = & qTexCoord[2*v_org_idx[nV0+v_loop_sp[iP]]];
						  int* nR = & qTexCoord[2*v_org_idx[nV0+v_loop_sp[iR]]];
						  int* nL = & qTexCoord[2*v_org_idx[nV0+v_loop_sp[iL]]];
						  pTexCoord.predictC(nP,nL,nR,nO);
						  encodeTexCoordSample(codec,bitsPerTexCoord,nO);
						  pTexCoord.predictE(nP,nL,nR,nO);
					  }
					  else if (iPavail+iRavail+iLavail == 2)
					  {
						  if (iPavail) {
							  int* nRL;
							  if (iRavail)
								  nRL = & qTexCoord[2*v_org_idx[nV0+v_loop_sp[iR]]];
							  else
								  nRL = & qTexCoord[2*v_org_idx[nV0+v_loop_sp[iL]]];
							  pTexCoord.predictC(nRL,nRL,nRL,nO);
							  encodeTexCoordSample(codec,bitsPerTexCoord,nO);
							  pTexCoord.predictE(nRL,nRL,nRL,nO);
						  } else {
							  int* nR = & qTexCoord[2*v_org_idx[nV0+v_loop_sp[iR]]];
							  int* nL = & qTexCoord[2*v_org_idx[nV0+v_loop_sp[iL]]];
							  pTexCoord.predictC2(nR,nL,nO);
							  encodeTexCoordSample(codec,bitsPerTexCoord,nO);
							  pTexCoord.predictE2(nR,nL,nO);
						  }
					  } else {
						  int* nPRL;
						  if (iPavail)
							  nPRL = & qTexCoord[2*v_org_idx[nV0+v_loop_sp[iP]]];
						  else if (iRavail)
							  nPRL = & qTexCoord[2*v_org_idx[nV0+v_loop_sp[iR]]];
						  else
							  nPRL = & qTexCoord[2*v_org_idx[nV0+v_loop_sp[iL]]];
						  pTexCoord.predictC(nPRL,nPRL,nPRL,nO);
						  encodeTexCoordSample(codec,bitsPerTexCoord,nO);
						  pTexCoord.predictE(nPRL,nPRL,nPRL,nO);
					  }
				  }
			  }
		  }
		  else if(texCoordBinding == PB_PER_CORNER)
		  {
			  if(polygon_edge==1)
			  {
				  //MJH 062600 : swapped L and R
				  int nR[3];   
				  pTexCoord.predictC(c_father,qTexCoord,cR,nR,c_index);	
				  encodeTexCoordSample(codec,bitsPerTexCoord,nR);

				  int nL[3];  
				  pTexCoord.predictC(c_father,qTexCoord,cL,nL,c_index);	
				  encodeTexCoordSample(codec,bitsPerTexCoord,nL);
			  }
			  int nO[3];	                                    
			  pTexCoord.predictC(c_father,qTexCoord,cO,nO,c_index);	 
			  encodeTexCoordSample(codec,bitsPerTexCoord,nO);
		  }
		  if(v_visited[v_cluster[nV0+v_loop_sp[iO]]]==AVAIL_PRED) {
			  visited_idx2.push_back(v_cluster[nV0+v_loop_sp[iO]]);
			  v_visited[v_cluster[nV0+v_loop_sp[iO]]] |= ALREADY_CODED + AVAIL_PRED;  
		  }
		  if(codap.sp_bdry_pred && v_visited[v_cluster[nV0+v_loop_sp[iO]]]==ALREADY_CODED) {
			  visited_idx1.push_back(v_cluster[nV0+v_loop_sp[iO]]);
			  v_visited[v_cluster[nV0+v_loop_sp[iO]]] |= ALREADY_CODED + AVAIL_PRED;  
		  }
      }
	  tdbits = codec.cbits-tdbits;
	  int tt = getTri(iS); 
	  if(codap.sp_pass0 && codap.sp_pt2_enable) {
		  //determine partition position
		  Calc_partition_pos(tt, iS, iRun, ldepth, td_orientation, codap.sp_pt2_enable, //codap.sp_sc_enable
							p_branch, p_leaf, p_run, p_tri);	
		  //check overflow
		  if(Calc_Overflow(tt, iT, ldepth, td_orientation,
							p_run, p_tri, tdbits, codec.cbits, availbits, 
							sub_bits, overflow, next_pt,
							v_update, sub_ptcount, return_tri,
							return_run, codap)==true) 
							break;
	  }
	  
	  // advance to next triangle
	  switch(getTri(iS))  
	  {
	  case sp_LEAF:
		  
		  // pop an edge from traversal stack
		  //       ^
		  //  L ---|--- R
		  //   \       /
		  //     \   /
		  //       P
		  loop.pop(iR,iL);
		  c_loop.insert(cO);
		  c_loop.pop(cR,cL);

//MJH 20010221 : moved to below (Crash in Win NT and W2K when stk is empty) 
//		  cRp  = stk.back(); stk.pop_back();
//		  cLp  = stk.back(); stk.pop_back();
//		  iP   = stk.back(); stk.pop_back();
//		  iFp  = stk.back(); stk.pop_back();

		  if(ldepth==1) {
			  iRun = stk_direct.back(); 
			  stk_direct.pop_back();
			  iS = tdirect.getStart(iRun);
		  } else {
			  iS++;
			  iRun++;
		  };
		  ldepth--;  
		  gdepth--;

		  //MJH 20010221 : moved from above and if condition added
		  if (ldepth>=0) {
			  cRp  = stk.back(); 
			  stk.pop_back();
			  cLp  = stk.back(); 
			  stk.pop_back();
			  iP   = stk.back(); 
			  stk.pop_back();
			  iFp  = stk.back(); 
			  stk.pop_back();
		  }

		  while (cur_bt->GetCurDir() > 1)
			  cur_bt = cur_bt->GetParent();
		  if (cur_bt->GetCurDir() == LEFT_BRANCH) {
			  cur_bt->GetInfo(RIGHT_BRANCH, iF, iC); 
			  cur_bt->SetCurDir(RIGHT_BRANCH+2);
		  }
		  else { 
			  cur_bt->GetInfo(LEFT_BRANCH, iF, iC); 
			  cur_bt->SetCurDir(LEFT_BRANCH+2);
		  };
		  break;
	  case sp_LEFT:
		  // turn left
		  //
		  //       O = R-1
		  //     /   \
		  //   /   ^   \
		  //  L ---|--- R
		  //   \       /
		  //     \   /
		  //       P
		  iP  = iR;
		  iR  = iO;
		  iFp = iF;
		  c_loop.insert(cO);
		  cRp = cO;
		  cLp = cL;
		  cR  = cRp;
		  cL  = cLp;
		  iS++; 
//MJH 001005 : removed for polygon models
//		  if(polygon_edge==1 || iT==0)      
//			  iF++;	

		  break;
	  case sp_RIGHT:
		  // turn right
		  //
		  //       O = L+1
		  //     /   \
		  //   /   ^   \
		  //  L ---|--- R
		  //   \       /
		  //     \   /
		  //       P
		  loop.advance();
		  iP  = iL;
		  iL  = iO;
		  iFp = iF;
		  c_loop.insert(cO);
		  c_loop.advance();
		  cRp = cR;
		  cLp = cO;
		  cR  = cRp;
		  cL  = cLp;
		  iS++; 

//MJH 001005 : removed for polygon models
//		  if(polygon_edge==1 || iT==0)      
//			  iF++;	

		  break;
	  case sp_BRANCH:
		  //       O
		  //     /   \
		  //   /   ^   \
		  //  L ---|--- R
		  //   \       /
		  //     \   /
		  //       P
		  // push left edge onto traversal stack
		  //       ^
		  //  L ---|--- O
		  //   \       /
		  //     \   /
		  //       R
		  stk.push_back(iF);
		  if(codap.sp_pt2_enable) {  //pt2
			  if(td_orientation) {  //left_first
				  loop.pushCurr();   
				  stk.push_back(iR);
				  loop.advance();	   
			  } else {
				  loop.pushNext();   
				  stk.push_back(iL);
			  }
		  } else {                  //pt0
			  loop.pushCurr();   
			  stk.push_back(iR);
			  loop.advance();	   
		  }
		  iFp = iF;
		  c_loop.insert(cO);
		  if (cur_bt->GetCurDir() < 2)
			  cur_bt = cur_bt->GetBranch(cur_bt->GetCurDir());	
		  else
			  cur_bt = cur_bt->GetBranch(cur_bt->GetCurDir()-2);
		  if(codap.sp_pt2_enable) {
			  if (td_orientation) {
				  iP = iL;
				  iL = iO;
				  c_loop.pushCurr();
				  c_loop.advance();
				  stk.push_back(cL);
				  stk.push_back(cO);
				  cRp = cR;
				  cLp = cO;
				  cur_bt->GetInfo(LEFT_BRANCH, iF, iC); 	
				  cur_bt->SetCurDir(LEFT_BRANCH);
			  } else {	
				  iP = iR;
				  iR = iO;
				  c_loop.pushNext();
				  stk.push_back(cO);
				  stk.push_back(cR);
				  cRp = cO;
				  cLp = cL;
				  cur_bt->GetInfo(RIGHT_BRANCH, iF, iC); 	
				  cur_bt->SetCurDir(RIGHT_BRANCH);
			  }
		  } else {
			  iP = iL;
			  iL = iO;
			  c_loop.pushCurr();
			  c_loop.advance();
			  stk.push_back(cL);
			  stk.push_back(cO);
			  cRp = cR;
			  cLp = cO;
			  cur_bt->GetInfo(LEFT_BRANCH, iF, iC); 
			  cur_bt->SetCurDir(LEFT_BRANCH);
		  }
		  cR  = cRp;
		  cL  = cLp;
		  if(ldepth==0) {
			  if(codap.sp_pt2_enable) { //pt2
				  stk_direct.push_back(tdirect.getSecond(iRun));
				  iRun = tdirect.getFirst(iRun);
			  } else {                 //pt0
				  if(td_orientation) {  
					  stk_direct.push_back(tdirect.getSecond(iRun));
					  iRun = tdirect.getFirst(iRun);
				  } else {
					  stk_direct.push_back(tdirect.getFirst(iRun));
					  iRun = tdirect.getSecond(iRun);
				  }
			  }
			  iS = tdirect.getStart(iRun);
		  } else { 
			  iS++;
			  iRun++;
		  };
		  ldepth++;
		  gdepth++;
		  break;
       }
	   if(codap.erMode>0) {
		   if(updateData(iS, iRun, iL, iR, iP, tt, td_orientation, 
						iF, iC, cR, cL, gdepth, ldepth, next_pt, codap,
						v_update, Gistk, Grstk, 
						GiF, GiC, Gdepth,
						prevCR, prevCL)==true) 
						break;
	   } else {
		   GiF=iF;
		   GiC=iC;
	   }
   }

   int k;
   for(k=0; k<visited_idx2.size(); k++) { 
	   if(v_visited[visited_idx2[k]]==ALREADY_CODED+AVAIL_PRED)
		   v_visited[visited_idx2[k]]=(v_update) ? AVAIL_PRED: ALREADY_CODED;  
   }
   for(k=0; k<visited_idx1.size(); k++) {
	   if(v_visited[visited_idx1[k]]==ALREADY_CODED+AVAIL_PRED)
		   v_visited[visited_idx1[k]]=ALREADY_CODED;
   }
   if(overflow==0 && sub_ptcount==0 && codap.sp_pass0) {
	   codap.next_tri = getNumberOfTriangles();
	   codap.next_run = getNumberOfRuns();
   }
	
	return 1;
}
// end of encodeTriangleData

inline void MPG_SPolygon::Calc_partition_pos(int tri, int iS, int iRun, 
									  int ldepth,
									  int td_orientation,
									  int sp_pt2_enable, 
									  int &p_branch, int &p_leaf,
									  int &p_run, int &p_tri) 
{
	int first, second;
	//determine partition position
	switch(tri) {
	case sp_BRANCH:
		if(ldepth==0) {
			if(sp_pt2_enable) {
				first=tdirect.getFirst(iRun);
				second=tdirect.getSecond(iRun);
			} else {
				if(td_orientation) {
					first=tdirect.getFirst(iRun);
					second=tdirect.getSecond(iRun);
				} else {
					first=tdirect.getSecond(iRun);
					second=tdirect.getFirst(iRun);
				}
			}
			if(getTri(tdirect.getStart(first))==sp_LEAF ||
				getTri(tdirect.getStart(second))==sp_LEAF) { 
				p_branch++;
			} else {
				if(p_branch>0 || p_leaf>0)  
					p_branch++;
				else 
					p_branch=0;
			}
			if(p_branch<2) {
				p_run=iRun;
				p_tri=iS;
			}
		} else { 
			p_branch=0;
		};
		break;
	case sp_LEAF:
		if(ldepth==0) {
			p_leaf=0;
		} else {
			if(p_branch>0 && tdirect.getLength(iRun)<3)  
				p_leaf++;
			else 
				p_leaf=0;
		};
		break;
	default:
		if(ldepth==0) {
			if(p_branch>0 && p_leaf>0) {
				if(iS!=tdirect.getStart(iRun)) {
					p_run=iRun;
					p_tri=iS;
				} 
			} else {
				p_run=iRun;
				p_tri=iS;
			}
		};
		p_branch=0;
		p_leaf=0;
		break;
	}
}

inline bool MPG_SPolygon::Calc_Overflow(int tri,
								 int iT,
								 int ldepth,
								 int td_orientation,
								 int p_run, 
								 int p_tri,
								 int tdbits,
								 int cbits,
								 int availbits, 
								 int &sub_bits,
								 int &overflow, 
								 int &next_pt,
								 int &v_update,
								 int &sub_ptcount,
								 int &return_tri,
								 int &return_run,
								 MPG_PacketQ_C &codap) 
{
	int first, second, curr_bits;
	curr_bits=24+codap.bdry_loopbits*2+cbits+(int)(tt_avgbits*iT+0.5)+48;
	if(availbits <= curr_bits) {  
		overflow=1;
		if(ldepth==0 || sub_ptcount>1) { 
			if(ldepth==0 && sub_ptcount<2) { 
				//data partitioning at main stem
				if(sub_ptcount==0) {
					codap.next_run=p_run;
					codap.next_tri=p_tri;
				} else {  
					//sub_ptcount=1
					codap.next_run=return_run;
					codap.next_tri=return_tri;
				}
			} else { 
				//sun_ptcount>1: subtre partitioning 
				if(codap.sp_pt2_enable) {
					codap.next_run=tdirect.getFirst(return_run);
				} else {
					if(tdirect.getDirect(return_run))
						codap.next_run=tdirect.getFirst(return_run);
					else 
						codap.next_run=tdirect.getSecond(return_run);
				}
				codap.next_tri=tdirect.getStart(codap.next_run);
			}
			v_update=1;
			codap.sp_partitioning=1;
			if(codap.vg_sc_enable && !codap.sp_pt2_enable) {
				if(codap.next_tri==0 && codap.next_run==0)  
					codap.sp_pt2_enable=1;
				return true;
			} else {
				if(codap.start_tri==codap.next_tri 
					&& codap.start_run==codap.next_run) {
					if(!next_pt) {
						if(sub_ptcount==1) {
							if(codap.sp_pt2_enable) {
								first= tdirect.getFirst(codap.next_run);
								if(tdirect.getLength(first)>2) {
									codap.next_tri=tdirect.getStart(first);
									codap.next_run=first;
									return true;
								} else {
									if(ldepth==0) {
										sub_ptcount=0;
										sub_bits=0;
									}
									next_pt=1;
								}
							} else {
								if(tdirect.getDirect(codap.next_run)) { 
									first= tdirect.getFirst(codap.next_run);
									second= tdirect.getSecond(codap.next_run);
								} else {
									first= tdirect.getSecond(codap.next_run);
									second= tdirect.getFirst(codap.next_run);
								}
								if(tdirect.getLength(second)>2 &&  
									tdirect.getLength(first)>2) {
									codap.next_tri=tdirect.getStart(first);
									codap.next_run=first;
									return true;
								} else {
									if(ldepth==0) {
										sub_ptcount=0;
										sub_bits=0;
									}
									next_pt=1;
								}
							}
						} else {
							sub_bits=0;
							next_pt=1;
						}
					} else {
						if(ldepth==0) {
							sub_ptcount=0;
							sub_bits=0;
						}
					}
				} else {
					if(next_pt==1) next_pt=0; 
					return true;
				}
			}
		} else { 
			//determine subtree partitioning
			if(next_pt) {
				if(ldepth==0 && tri==sp_BRANCH) 
					sub_bits=0;
				else 
					sub_bits += tdbits;
			} else
				sub_bits += tdbits;
			if(sub_ptcount==0) {
				sub_ptcount=1;
				return_tri = p_tri;
				return_run = p_run;
			} else if(availbits <= 24+codap.bdry_loopbits*3+sub_bits+(int)(tt_avgbits*iT+0.5)+48)  
				sub_ptcount++;
		}
	} else {  
		if(ldepth==0) {
			if(tri==sp_BRANCH) 
				sub_bits=0;
		}
		else 
			sub_bits += tdbits;
	}
	return false;
}

inline bool MPG_SPolygon::updateData(int iS, int iRun, int iL, int iR, int iP, 
							   int tri, int td_orientation, 
							   int iF, int iC, 
							   int cR, int cL,	//MJH 001013 : added in case of partitioning at non-polygon edge
							   int gdepth,
							   int ldepth, int next_pt,
							   MPG_PacketQ_C& codap,
							   int &v_update,
							   vector<int>& Gistk, 
							   vector<int>& Grstk, 
							   int &GiF, int &GiC, int &Gdepth,
							   int &prevCR, int &prevCL	//MJH 001013 : added in case of partitioning at non-polygon edge
							   )
{
	if(!codap.sp_pass0) {  
		if(iS==codap.next_tri) { 
			codap.sp_left_bloop_idx = iL;
			codap.sp_right_bloop_idx = iR;
			codap.start_tri=codap.next_tri;
			codap.start_run=codap.next_run;
			codap.sp_sub_pt_enable=0;
			if(tri==sp_BRANCH) {
				codap.sp_bdry_pred=1;
				if(codap.sp_sc_enable) {
					if(td_orientation) {
						Gistk.push_back(iP);  //iL
						Gistk.push_back(iL);  //iR
					} else {
						Gistk.push_back(iR);  //iL
						Gistk.push_back(iP);  //iR
					}
				} else {
					Gistk.push_back(iP);  //iL
					Gistk.push_back(iL);  //iR
				}
			}
			GiF=iF;
			GiC=iC;
			Gdepth = gdepth;
			prevCR = cR;	//MJH 001013
			prevCL = cL;	//MJH 001013
			return true;
		}
	} else {
		if(ldepth==-1) {
			if(next_pt) {
				codap.sp_partitioning=0;
				v_update=0;
				codap.next_tri = iS;  
				codap.next_run = iRun;
			}
			if(Grstk.size()>1) {
				codap.start_run=Grstk.back();
				Grstk.pop_back();
				codap.start_tri=tdirect.getStart(codap.start_run);
				codap.sp_sub_pt_enable=0;
				codap.sp_bdry_pred=(Grstk.size()>1) ? 1:0;
				codap.sp_right_bloop_idx = Gistk.back();
				Gistk.pop_back();
				codap.sp_left_bloop_idx = Gistk.back();
				Gistk.pop_back();
			}
			GiF=iF;
			GiC=iC;
			Gdepth = gdepth;
			prevCR = cR;	//MJH 001013
			prevCL = cL;	//MJH 001013
			return true;
		}
	}
	return false;
}
