//-------------------------------------------------------------------------
// 
// Copyright (C) IBM Corporation & Samsung Electronics Co. LTD., 1996, 1997, 1998, 1999
// Licensed Materials, Program Property of IBM & Samsung.
// 
// This software module is an implementation of a part of one or more
// MPEG-4 tools as specified by the MPEG-4 standard (ISO/IEC 14496).
// ISO/IEC gives users of the MPEG-4 standard (ISO/IEC 14496) free
// license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4
// standard (ISO/IEC 14496).  Those intending to use this software module
// in hardware or software products are advised that its use may infringe
// existing patents.  The original developer of this software module and
// his/her company, the subsequent editors and their companies, and
// ISO/IEC have no liability for use of this software module or
// modifications thereof in an implementation.  Copyright is not released
// for non MPEG-4 standard (ISO/IEC 14496) conforming products.  IBM
// Corp. & Samsung Electronics Co. LTD retains full right to use the code 
// for his/her own purpose, assign or donate the code to a third party and
// to inhibit third parties from using the code for non MPEG-4 standard 
// (ISO/IEC 14496) conforming products. This copyright notice must be included
// in all copies or derivative works.
// 
//  AUTHORS:
//    Gabriel Taubin
//    Andre Gueziec
//    Claudio Silva
//    Ioana Martin
//  
//  IBM T. J. Watson Research Center
//  P.O.Box 704
//  Yorktown Heights, New York 10598
//  
//  BASED ON EARLIER IMPLEMENTATION OF
//  VRML COMPRESSED BINARY FORMAT PROPOSAL BY:
//    Gabriel Taubin
//    Bill Horn
//    Francis Lazarus
//
//  Functionality : Error Resilience
//
//  Sungjin Kim  (sunjin@sait.samsung.co.kr)
//  Mahn-Jin Han  (manjini@sait.samsung.co.kr)
//  Mun-Sup Song (mssong@sait.samsung.co.kr)  
//
//  Signal Processing Lab. Samsung A.I.T. 
//-------------------------------------------------------------------------


#include <stdio.h>
#include <stdlib.h>
#include "MPG_Global.h"
#include "MPG_Sample.h"

//---------------------------------------------------------------------------
//
//   MPG_Sample::_encodeRootSample
//
//---------------------------------------------------------------------------

void MPG_Sample::_encodeRootSample(MPG_QFac& codec,
				     int       bitsPerX,
				     int*      iX,
				     int       nD)
{
  int j,k,iX_jk;
  for(j=0;j<nD;j++)
    {
      for(k=0;k<bitsPerX;k++)
	{
	  // msb ... lsb
	  iX_jk = (iX[j]>>(bitsPerX-k-1))&0x1;
	  codec.encode(iX_jk);
	}
    }
}

//---------------------------------------------------------------------------
//
//   MPG_Sample::_encodeSample
//
//---------------------------------------------------------------------------

void MPG_Sample::_encodeSample(MPG_QFac& codec,
				 int       bitsPerX,
				 int*      iX,
				 int       nD,
				 int       p)
{
  int b, j;
  for(j=0; j < nD; j++)
    {
      int iXj   = iX[j];
      for(b = bitsPerX-1;b>=0;b--)
	{
	  int val_ij = (iXj>=0)?iXj:-iXj;
	  int sgn_ij = (iXj>=0)?0:1;
	  {
	    val_ij >>= b;
	    int bit = val_ij&0x1;

	    if(val_ij<=1)
	      {
		if(p==0)
		  codec.encode(bit,MPG_Global::td_coord_leading_bit[32*j+b]);
		else if(p==1)
		  codec.encode(bit,MPG_Global::td_normal_leading_bit[32*j+b]);
		else if(p==2)
		  codec.encode(bit,MPG_Global::td_color_leading_bit[32*j+b]);
		else if(p==3)
		  codec.encode(bit,MPG_Global::td_texCoord_leading_bit[32*j+b]);
	      }
	    else
	      codec.encode(bit);

	    // if first significant bit
	    if(val_ij==1)
	      {
		codec.encode(sgn_ij);
	      }
	  }
	}
    }
}

