//-------------------------------------------------------------------------
// 
// Copyright (C) IBM Corporation & Samsung Electronics Co. LTD., 1996, 1997, 1998, 1999
// Licensed Materials, Program Property of IBM & Samsung.
// 
// This software module is an implementation of a part of one or more
// MPEG-4 tools as specified by the MPEG-4 standard (ISO/IEC 14496).
// ISO/IEC gives users of the MPEG-4 standard (ISO/IEC 14496) free
// license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4
// standard (ISO/IEC 14496).  Those intending to use this software module
// in hardware or software products are advised that its use may infringe
// existing patents.  The original developer of this software module and
// his/her company, the subsequent editors and their companies, and
// ISO/IEC have no liability for use of this software module or
// modifications thereof in an implementation.  Copyright is not released
// for non MPEG-4 standard (ISO/IEC 14496) conforming products.  IBM
// Corp. & Samsung Electronics Co. LTD retains full right to use the code 
// for his/her own purpose, assign or donate the code to a third party and
// to inhibit third parties from using the code for non MPEG-4 standard 
// (ISO/IEC 14496) conforming products. This copyright notice must be included
// in all copies or derivative works.
// 
//  AUTHORS:
//    Gabriel Taubin
//    Andre Gueziec
//    Claudio Silva
//    Ioana Martin
//  
//  IBM T. J. Watson Research Center
//  P.O.Box 704
//  Yorktown Heights, New York 10598
//  
//  BASED ON EARLIER IMPLEMENTATION OF
//  VRML COMPRESSED BINARY FORMAT PROPOSAL BY:
//    Gabriel Taubin
//    Bill Horn
//    Francis Lazarus
//
//  Functionality : Error Resilience
//
//  Sungjin Kim  (sunjin@sait.samsung.co.kr)
//  Mahn-Jin Han  (manjini@sait.samsung.co.kr)
//  Mun-Sup Song (mssong@sait.samsung.co.kr)  
//
//  Signal Processing Lab. Samsung A.I.T. 
//-------------------------------------------------------------------------

#include "MPG_Global.h"
#include "MPG_Sample.h"

//---------------------------------------------------------------------------
//
//   MPG_Sample::_decodeRootSample
//
//---------------------------------------------------------------------------

void MPG_Sample::_decodeRootSample(MPG_QFac& codec,
				     int       bitsPerX,
				     int*      iX,
				     int       nD)
{
  int j,k,iX_jk;
  for(j=0;j<nD;j++)
    {
      iX[j] = 0;
      for(k=0;k<bitsPerX;k++)
	{
	  iX_jk = codec.decode();
	  iX[j] = iX[j] + (iX_jk<<(bitsPerX-k-1));
	}
    }
}

//---------------------------------------------------------------------------
//
//   MPG_Sample::_decodeSample
//
//---------------------------------------------------------------------------

void MPG_Sample::_decodeSample(MPG_QFac& codec,
				 int       bitsPerX,
				 int*      iX,
				 int       nD,
				 int       p)
{
  int b, j;
  for(j=0;j<nD;j++)
    {
      int* ipvi = &(iX[0]);
      ipvi[j] = 0;

      b = bitsPerX-1;
      int msb = 0;
      do {
	msb =
	  (p==0)? codec.decode(MPG_Global::td_coord_leading_bit[32*j+b]):
	  (p==1)? codec.decode(MPG_Global::td_normal_leading_bit[32*j+b]):
	  (p==2)? codec.decode(MPG_Global::td_color_leading_bit[32*j+b]):
	  (p==3)? codec.decode(MPG_Global::td_texCoord_leading_bit[32*j+b]):0;
	msb = msb << b;
	b--;
      } while (msb == 0 && b >=0);

      int sgn = 0;
      int rest = 0;
      if(msb != 0)
	{
	  sgn = codec.decode();
	  rest = 0;
	  while (b>=0) {
	    rest |= (codec.decode() << b);
	    b--;
	  }
	}

      if(sgn)
	ipvi[j] = -(msb+rest);
      else
	ipvi[j] = (msb+rest);
    }
}
