//-------------------------------------------------------------------------
// 
// Copyright (C) IBM Corporation & Samsung Electronics Co. LTD., 1996, 1997, 1998, 1999
// Licensed Materials, Program Property of IBM & Samsung.
// 
// This software module is an implementation of a part of one or more
// MPEG-4 tools as specified by the MPEG-4 standard (ISO/IEC 14496).
// ISO/IEC gives users of the MPEG-4 standard (ISO/IEC 14496) free
// license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4
// standard (ISO/IEC 14496).  Those intending to use this software module
// in hardware or software products are advised that its use may infringe
// existing patents.  The original developer of this software module and
// his/her company, the subsequent editors and their companies, and
// ISO/IEC have no liability for use of this software module or
// modifications thereof in an implementation.  Copyright is not released
// for non MPEG-4 standard (ISO/IEC 14496) conforming products.  IBM
// Corp. & Samsung Electronics Co. LTD retains full right to use the code 
// for his/her own purpose, assign or donate the code to a third party and
// to inhibit third parties from using the code for non MPEG-4 standard 
// (ISO/IEC 14496) conforming products. This copyright notice must be included
// in all copies or derivative works.
// 
//  AUTHORS:
//    Gabriel Taubin
//    Andre Gueziec
//    Claudio Silva
//    Ioana Martin
//  
//  IBM T. J. Watson Research Center
//  P.O.Box 704
//  Yorktown Heights, New York 10598
//  
//  BASED ON EARLIER IMPLEMENTATION OF
//  VRML COMPRESSED BINARY FORMAT PROPOSAL BY:
//    Gabriel Taubin
//    Bill Horn
//    Francis Lazarus
//
//  Functionality : Error Resilience
//
//  Sungjin Kim  (sunjin@sait.samsung.co.kr)
//  Mahn-Jin Han  (manjini@sait.samsung.co.kr)
//  Mun-Sup Song (mssong@sait.samsung.co.kr)  
//
//  Signal Processing Lab. Samsung A.I.T. 
//-------------------------------------------------------------------------
#pragma warning(disable:4786)

#include <stdio.h>
#include <stdlib.h>
#include "MPG_Global.h"
#include "MPG_QFac.h"
#include "MPG_Stitches.h"

#ifndef MPGELIB
extern FILE* logfp;
#endif // MPGELIB

//  this file contains implementations of the encode and decode methods
//  for stitches
        
//---------------------------------------------------------------------------
//
//   MPG_Stitches::encode
//
//---------------------------------------------------------------------------
// stitches() {
//	  for each vertex in connected_component {
//	 	 qf_decode(stitch_cmd, stitch_cmd_context)
//		 if (stitch_cmd) {
//			qf_decode(stitch_pop_or_get, stitch_pop_or_get_context)
//			if (stitch_pop_or_get == '1') {
//				qf_decode(stitch_pop, stitch_pop_context)
//				qf_decode(	stitch_stack_index, stitch_stack_index_context)
//				qf_decode(stitch_incr_length,stitch_incr_length_context)
//				if (stitch_incr_length != 0)
//			        qf_decode(stitch_incr_length_sign,stitch_incr_length_sign_context)
//				qf_decode(stitch_push, stitch_push_context)
//				if (total length >0)
//					qf_decode(stitch_reverse, stitch_reverse_context)
//			} else
//				qf_decode(stitch_length, stitch_length_context)
//		 }
//	  }
// }


bool MPG_Stitches::encode(// inputs
			  int nV0,
			  int nV1,
			  vector <int> &stk, // stack for stitch anchors
			  MPG_QFac &codec)
{
  int i;
  bool has_stitches = false;
  {
    CommandType t;
    for(i=nV0;i<nV1;i++) 
	{
      if(getCommandType(i,t)==false)
			return false;
      else if(t!=ST_NONE)
		{ has_stitches = true; break; }
	}
  }
  codec.encode(has_stitches,MPG_Global::stitch_has);
		
  if(has_stitches==true)
    {
      CommandType t;		//MJH comment : this should be removed in order for stitch to work
      int                       index;
      int                       length;
      bool                      push;
      bool                      reverse;

      for(i=nV0;i<nV1;i++) {
	if(get(i,t,index,length,push,reverse)==false)
	  return false;
	else {

	  /* AG: encode the variable length code first so that we can
	     change the codes associated with the commands more easily */

	  if (t==ST_NONE) codec.encode(0,MPG_Global::stitch_none);
	  else {
	    codec.encode(1,MPG_Global::stitch_none);
	    if(t==ST_PUSH) codec.encode(0,MPG_Global::stitch_command[0]); 
	    // AG 3/5/99 push and pop can be exchanged
	    else {
	      codec.encode(1,MPG_Global::stitch_command[0]);
	      if(t==ST_GET) codec.encode(0,MPG_Global::stitch_command[1]);
	      else // if (t==ST_POP)
		codec.encode(1,MPG_Global::stitch_command[1]);
	    }
	  }
	    
	  // then encode the rest
	
	if (t!=ST_NONE)
	  {

	    int h;
	    if(t==ST_PUSH)
	      {
		// encode 'length' as an unsigned int
		for(h=0;h<length;h++)
		  codec.encode(0,MPG_Global::stitch_length);
		codec.encode(1,MPG_Global::stitch_length);
			  
		stk.push_back(i);
	      }
	    else
	      {

		// have to find index in stk
		int k = index;
			  
		// determine depth of stack
		int nS=0; // AG 3/1/99 after suggestion from FB
		for(h=0;h<stk.size();h++)
		  if(stk[h]>=0)
		    nS++;
			  
		// find index of k in stack
		int depth = 0;
		for(h=0;h<stk.size();h++)
		  if(stk[h]>=0)
		    {
		      if(stk[h]==k) break;
		      depth++;
		    }
		// k should have been found in the stack !!!
			  
		// AG: 1/27/99 making sure k is always found
#ifndef MPGELIB
		if (h >= stk.size())
		  fprintf(logfp,"for GET or POP %d index %d wasn't in stack of depth %d\n",i,k,nS);
#endif // MPGELIB
			  
		if (t==ST_POP)
		  {
		    // remove k from stack;
		    if(h==stk.size()-1)
		      stk.pop_back();
		    else
		      stk[h] = -1;
		  }
			  
		// if "push" bit set to true add the current vertex as anchor in the stack
		if(push==true)
		  stk.push_back(i);

		// encode 'depth' as an unsigned int
		for(h=0;h<=depth;h++)
		  {
		    if(h==nS-1) break;
		    bool val=(h<depth)?false:true;
		    codec.encode(val,MPG_Global::stitch_depth);
		  }
			     
		// encode 'length' as a signed int
		int length_abs = (length<0)?-length:length;
		for(h=0;h<length_abs;h++)
		  codec.encode(0,MPG_Global::stitch_length_diff);
		codec.encode(1,MPG_Global::stitch_length_diff);
		// encode 'sign' bit only if length!=0
		if(length!=0)
		  {
		    bool sign = (length<0)?true:false;
		    codec.encode(sign,MPG_Global::stitch_sign);
		  }
			  
		// encode 'push' bit
		codec.encode(push,MPG_Global::stitch_push);

		  
		// encode 'reverse' bit only if true length!=0
		int length_k = 0;
		if(getLength(k,length_k)==false)
		  return false;
		if(length+length_k!=0)
		  codec.encode(reverse,MPG_Global::stitch_reverse);

		// AG: in this way the length will be correct
		// for the purpose of encoding the "reverse" bit
		if (push) setLength(i, length + length_k);
			
	      }
	  } // if  (t!=ST_NONE) 
	} // else if get (stitch) 
      } // end for 
    } // if (has_stitches == true)
  return true;

}

//---------------------------------------------------------------------------
//
//   MPG_Stitches::decode
//
//---------------------------------------------------------------------------
bool MPG_Stitches::decode(// inputs
			  int nV0,
			  int nV1,
			  vector <int> &stk, // stack for stitch anchors
			  MPG_QFac &codec)
{
  bool has_stitches = (codec.decode(MPG_Global::stitch_has)==1)?true:false;

  int i;
  for(i=nV0;i<nV1;i++)
    {
      CommandType t                      = ST_NONE;
      int                       index    = -1;
      int                       length   =  0;
      int                       length_k =  0;
      bool                      push     =  false;
      bool                      reverse  =  false;
      if(has_stitches==true) // AG 1/27/99 bug: was false
	{
	  /* AG: decode the variable length code first so that we can
	     change the codes associated with the commands more easily */

          bool bit = (codec.decode(MPG_Global::stitch_none)==1)?true:false;
	  if (bit==false) t = ST_NONE;
	  else {
            bit = (codec.decode(MPG_Global::stitch_command[0])==1)?true:false;
	    if (bit==false)
              t = ST_PUSH;// AG 3/5/99 push and pop can be exchanged
            
	    else {
              bit = (codec.decode(MPG_Global::stitch_command[1])==1)?true:false;
	      t   =(bit==false)?ST_GET:ST_POP;
	    }
	  }
          
	  if (t != ST_NONE)
	    {
	      if (t == ST_PUSH){
                
		// decode 'length' as an unsigned int
		while(codec.decode(MPG_Global::stitch_length)
		      ==false)
		  length++;
                
		stk.push_back(i);
	      }
	      else // ST_GET or ST_POP
		{
		  int h;
		  
		  // determine depth of stack
		  int nS=0; // AG 3/1/99 after suggestion from FB
		  for(h=0;h<stk.size();h++)
		    if(stk[h]>=0)
		      nS++;
			      
		  // decode depth
		  int depth = 0;
		  while (1)
		    {
		      if(depth==nS-1) break;
		      if(codec.decode(MPG_Global::stitch_depth))
			break;
		      depth++;
		    }
		  // find the depth-th. element >=0 in stack
		  int ix = depth;
		  for(h=0;h<stk.size();h++)
		    if(stk[h]>=0)
		      {
			if(ix==0) break;
			ix--;
		      }
		  index = stk[h];

		  if(t==ST_POP)
		    {
		      // remove index from stack;
		      if(h==stk.size()-1)
			stk.pop_back();
		      else
			stk[h] = -1;
		    }
			  
		  // decode 'length' as a signed int
		  while(codec.decode(MPG_Global::stitch_length_diff)
			==false)
		    length++;
		  if(length!=0 &&
		     codec.decode(MPG_Global::stitch_sign)==1)
		    length = -length;
			       
		  // decode 'push' bit
		  push = (codec.decode(MPG_Global::stitch_push)==1)?true:false;
			      
		  // AG: make sure to push in stack  after decoding
		  // the push bit

		  if(push==true)
		    {
		      stk.push_back(i);
		    }
			      
		  // decode 'reverse' bit only if true length!=0
		  int k = index;
		  if(getLength(k,length_k)==false) {
		    //return false;
		  }
		  if(length+length_k!=0)
		    reverse =
			(codec.decode(MPG_Global::stitch_reverse)==1)?true:false;
		} //  ST_GET or ST_POP
	      // else => use default values
	    } // (t != ST_NONE)
	  // AG 1/27/99 and 3/3/99
	  // in this way the length will correct for the stitches and
	  // for the decoding of the reverse bit.
	  push_back(t,index,length+length_k, push, reverse);
	} // if(has_stitches==true)
      // AG 1/27/99
      else
	{
	  // no stitch (default)
	  push_back(ST_NONE,-1,0, false, false);
	}
    } 
  return true;
}

//---------------------------------------------------------------------------
//
//   bug fixes
//
//---------------------------------------------------------------------------

/*

  3/3/99  (pb discoverted by FB) length+length_k is not correct
           and thus the reverse bit is encoded too often

  encoder: if (push) setLength(length+length_k) (added setLength() method)
  decoder:
  push_back(length+length_k)
  


 */
