//--------------------------------------------------------------------------
//
//  ANG_Arcball.h
//
//  Copyright (C) IBM Corporation, 1999
//  Program Property of IBM
//
//  AUTHORS: 
//		Ioana M. Martin (ioana@us.ibm.com)
//
//  IBM T. J. Watson Research Center
//  30 Saw Mill River Road
//  Hawthorne, New York 10532
//
//  Reference: Ken Shoemake, "Arcball Rotation Control", Graphics Gems iv
//--------------------------------------------------------------------------

#ifndef _arcball_h_
#define _arcball_h_

/* vector in world coordinates */

typedef struct{
   float x, y, z;
} WVECTOR;

/* quaternion: pair (vector, scalar) */

typedef struct {
   WVECTOR v;
   float w;
} QUAT;

// arcball sphere 
class Arcball
{		
	friend class CProgressiveExpandApp;

	WVECTOR center;
	float radius;
	WVECTOR v1;

	WVECTOR vdown;			        /* where rotation starts */
	WVECTOR rotAxis;				/* rotation axis */
	float rotAngle;				    /* rotation angle */
	float sLeft;					/* sphere system: left coord */
	float sBottom;					/* sphere system: bottom coord */
	float sTop;						/* sphere system: top coord */
	float sRight;					/* sphere system: right coord */
	QUAT qOne;						/* scalar 1 */
	WVECTOR vZero;					/* zero vector */

public:

	Arcball();
	void ballBeginDrag(WVECTOR);
	void ballEndDrag();
	void dotProd(WVECTOR, WVECTOR, float&);
	void vectProd(WVECTOR, WVECTOR, WVECTOR&);
	void worldToBall(float, float, WVECTOR&);
	void vectToQuat(WVECTOR, QUAT&);
	float GetRotAngle(QUAT);
	void SetRotAngle(float);
	float GetRotAngle();
	float GetXRotAxis();
	float GetYRotAxis();
	float GetZRotAxis();
	void GetRotAxis(QUAT, WVECTOR&);
	void SetRotAxis(WVECTOR);
	void ballPlace(WVECTOR, float);
	void quatInv(QUAT, QUAT&);
	void quatMult(QUAT, QUAT, QUAT&);
	float GetSLeft();
	float GetSRight();
	float GetSTop();
	float GetSBottom();

	WVECTOR v0;
	QUAT qDown, qNow;
	int dragging;
};

#endif

