// ProgressiveExpand.cpp : Defines the class behaviors for the application.
//

#include "stdafx.h"
#include "ProgressiveExpand.h"

#include "MainFrm.h"
#include "ProgressiveExpandDoc.h"
#include "ProgressiveExpandView.h"

#include <GL/gl.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CProgressiveExpandApp

BEGIN_MESSAGE_MAP(CProgressiveExpandApp, CWinApp)
	//{{AFX_MSG_MAP(CProgressiveExpandApp)
	ON_COMMAND(ID_APP_ABOUT, OnAppAbout)
		// NOTE - the ClassWizard will add and remove mapping macros here.
		//    DO NOT EDIT what you see in these blocks of generated code!
	//}}AFX_MSG_MAP
	// Standard file based document commands
	ON_COMMAND(ID_FILE_NEW, CWinApp::OnFileNew)
	ON_COMMAND(ID_FILE_OPEN, CWinApp::OnFileOpen)
	// Standard print setup command
	ON_COMMAND(ID_FILE_PRINT_SETUP, CWinApp::OnFilePrintSetup)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CProgressiveExpandApp construction

CProgressiveExpandApp::CProgressiveExpandApp()
{
	// TODO: add construction code here,
	// Place all significant initialization in InitInstance
	_ifs = NULL;
	_nTriangles = 50;
	_timerON = 0;
	_polygonMode = GL_FILL;
	_width = _height = _depth = 10.0;
	_cx = _cy = _cz = 0.0;
}

/////////////////////////////////////////////////////////////////////////////
// The one and only CProgressiveExpandApp object

CProgressiveExpandApp theApp;

/////////////////////////////////////////////////////////////////////////////
// Command-line processing; override ParseParam in CCommandLineInfo class --

void CProgressiveExpandApp::ParseCommandLine(CCommandLineInfo& rCmdInfo)
{
	for (int i = 1; i < __argc; i++)
	{
		LPCTSTR pszParam = __targv[i];
		BOOL bFlag = FALSE;
		BOOL bLast = ((i + 1) == __argc);
		if (pszParam[0] == '-' || pszParam[0] == '/')
		{
			// remove flag specifier
			bFlag = TRUE;
			++pszParam;
		}
		switch(i)
		{
			case 1:
				_modelName = pszParam+6;
				break;
			case 2:
				_nTriangles = atoi(pszParam+10);
				break;
			case 3:
				_width = atof(pszParam+6);
				break;
			case 4:
				_height = atof(pszParam+7);
				break;
			case 5:
				_depth = atof(pszParam+6);
			case 6:
				_cx = atof(pszParam+3);
				break;
			case 7:
				_cy = atof(pszParam+3);
				break;
			case 8:
				_cz = atof(pszParam+3);
				break;
		}
		//rCmdInfo.ParseParam(pszParam, bFlag, bLast);
	}
}

/////////////////////////////////////////////////////////////////////////////
// CProgressiveExpandApp initialization

BOOL CProgressiveExpandApp::InitInstance()
{
	AfxEnableControlContainer();

	// Standard initialization
	// If you are not using these features and wish to reduce the size
	//  of your final executable, you should remove from the following
	//  the specific initialization routines you do not need.

#ifdef _AFXDLL
	Enable3dControls();			// Call this when using MFC in a shared DLL
#else
	Enable3dControlsStatic();	// Call this when linking to MFC statically
#endif

	// Change the registry key under which our settings are stored.
	// TODO: You should modify this string to be something appropriate
	// such as the name of your company or organization.
	SetRegistryKey(_T("Local AppWizard-Generated Applications"));

	LoadStdProfileSettings();  // Load standard INI file options (including MRU)

	// Register the application's document templates.  Document templates
	//  serve as the connection between documents, frame windows and views.

	CSingleDocTemplate* pDocTemplate;
	pDocTemplate = new CSingleDocTemplate(
		IDR_MAINFRAME,
		RUNTIME_CLASS(CProgressiveExpandDoc),
		RUNTIME_CLASS(CMainFrame),       // main SDI frame window
		RUNTIME_CLASS(CProgressiveExpandView));
	AddDocTemplate(pDocTemplate);

	// Parse command line for standard shell commands, DDE, file open
	CCommandLineInfo cmdInfo;
	ParseCommandLine(cmdInfo);

	// If no model was specified, popup error mesage
	if(!strlen(_modelName))
	{
		MessageBox(NULL, "Please specify an input file name", "Error", MB_OK);
		exit(1);
	}
	else
	{
		_istrm = new MPG_InputStream;
		bool success = _istrm->open(_modelName.GetBuffer(_modelName.GetLength())); 
		if(!success)
		{	
			MessageBox(NULL, "Cannot open specified file", "Error", MB_OK);
			exit(1);
		}
	}

	// initialize arcball
	Arcball *ball = new Arcball();
	SetArcball(*ball);

	// Dispatch commands specified on the command line
	if (!ProcessShellCommand(cmdInfo))
		return FALSE;

	// The one and only window has been initialized, so show and update it.
	m_pMainWnd->ShowWindow(SW_SHOW);
	m_pMainWnd->UpdateWindow();

	return TRUE;
}


/////////////////////////////////////////////////////////////////////////////
// CAboutDlg dialog used for App About

class CAboutDlg : public CDialog
{
public:
	CAboutDlg();

// Dialog Data
	//{{AFX_DATA(CAboutDlg)
	enum { IDD = IDD_ABOUTBOX };
	//}}AFX_DATA

	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CAboutDlg)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	//}}AFX_VIRTUAL

// Implementation
protected:
	//{{AFX_MSG(CAboutDlg)
		// No message handlers
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
{
	//{{AFX_DATA_INIT(CAboutDlg)
	//}}AFX_DATA_INIT
}

void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAboutDlg)
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
	//{{AFX_MSG_MAP(CAboutDlg)
		// No message handlers
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

// App command to run the dialog
void CProgressiveExpandApp::OnAppAbout()
{
	CAboutDlg aboutDlg;
	aboutDlg.DoModal();
}

// Set model name
void CProgressiveExpandApp::SetModelName(CString name)
{
	_modelName = name;
}

// Retrieve model name
CString& CProgressiveExpandApp::GetModelName()
{
	return _modelName;
}

// Set polygon mode
void CProgressiveExpandApp::SetPolygonMode(GLenum pm)
{
	_polygonMode = pm;
}

// Retrieve polygon mode
GLenum CProgressiveExpandApp::GetPolygonMode()
{
	return _polygonMode;
}

// Retrieve # of triangles to be expanded at a time
int CProgressiveExpandApp::GetNTriangles()
{
	return _nTriangles;
}

// Display model 
void CProgressiveExpandApp::DrawOpenGL()
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT); 
	if(_ifs)
		_ifs->Render();
}

void CProgressiveExpandApp::FillInIfs(P_MPG_IfsQ_E* ifs)
{
//	if(_ifs)
//		delete _ifs;
	_ifs = ifs;
}

Arcball& CProgressiveExpandApp::GetArcball()
{
	return _ball;
}

void CProgressiveExpandApp::SetArcball(Arcball& newBall)
{
	_ball.center = newBall.center;
	_ball.radius = newBall.radius;
	_ball.v0 = newBall.v0;
	_ball.v1 = newBall.v1;
	_ball.dragging = newBall.dragging;

	_ball.vdown = newBall.vdown;			        
	_ball.rotAxis = newBall.rotAxis;	
	_rotAxis[0] = newBall.rotAxis.x;
	_rotAxis[1] = newBall.rotAxis.y;
	_rotAxis[2] = newBall.rotAxis.z;
	_rotAngle = _ball.rotAngle = newBall.rotAngle;

	_ball.qDown = newBall.qDown;
	_ball.qNow = newBall.qNow;
}

void CProgressiveExpandApp::SetRotAxis(float a, float b, float c)
{
	_rotAxis[0] = a;
	_rotAxis[1] = b;
	_rotAxis[2] = c;
}

float *CProgressiveExpandApp::GetRotAxis()
{
	return _rotAxis;
}

void CProgressiveExpandApp::SetRotAngle(float ra)
{
	_rotAngle = ra;
}

float CProgressiveExpandApp::GetRotAngle()
{
	return _rotAngle;
}
/////////////////////////////////////////////////////////////////////////////
// CProgressiveExpandApp message handlers

