// ProgressiveExpandDoc.cpp : implementation of the CProgressiveExpandDoc class
//

#include "stdafx.h"
#include "ProgressiveExpand.h"

#include "ProgressiveExpandDoc.h"
#include "P_MPG_IfsQ_E.h"
#include "MPG_State.h"

#include <GL/gl.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CProgressiveExpandDoc

IMPLEMENT_DYNCREATE(CProgressiveExpandDoc, CDocument)

BEGIN_MESSAGE_MAP(CProgressiveExpandDoc, CDocument)
	//{{AFX_MSG_MAP(CProgressiveExpandDoc)
	ON_COMMAND(ID_EXPAND, OnExpandModel)
	ON_COMMAND(ID_EXPAND_TRI, OnExpandTriangles)
	ON_COMMAND(ID_EXPAND_AUTO, OnExpandAuto)
	ON_COMMAND(ID_WIRE, OnToggleWire)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CProgressiveExpandDoc construction/destruction

CProgressiveExpandDoc::CProgressiveExpandDoc()
{
	// TODO: add one-time construction code here

}

CProgressiveExpandDoc::~CProgressiveExpandDoc()
{
}

BOOL CProgressiveExpandDoc::OnNewDocument()
{
	if (!CDocument::OnNewDocument())
		return FALSE;

	// TODO: add reinitialization code here
	// (SDI documents will reuse this document)

	return TRUE;
}



/////////////////////////////////////////////////////////////////////////////
// CProgressiveExpandDoc serialization

void CProgressiveExpandDoc::Serialize(CArchive& ar)
{
	if (ar.IsStoring())
	{
		// TODO: add storing code here
	}
	else
	{
		// TODO: add loading code here
	}
}

/////////////////////////////////////////////////////////////////////////////
// CProgressiveExpandDoc diagnostics

#ifdef _DEBUG
void CProgressiveExpandDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CProgressiveExpandDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CProgressiveExpandDoc commands

extern CProgressiveExpandApp theApp;
void CProgressiveExpandDoc::OnExpandModel() 
{
	P_MPG_IfsQ_E* ifs = theApp.GetIfs();
	if(!ifs)
		ifs = new P_MPG_IfsQ_E;

	static MPG_State *state = new MPG_State(theApp.GetNTriangles());

	// expand header first, then the rest
	// create vectors to store quantized values
	static vector <int> qCoord;
	static vector <int> qNormal;
	static vector <int> qColor;
	static vector <int> qTexCoord;

	// create proper headers and quantizers
	// if not yet created by calling method
	static MPG_Property pCoord(PT_COORD,PB_NONE);
	static MPG_Property pNormal(PT_NORMAL,PB_NONE);
	static MPG_Property pColor(PT_COLOR,PB_NONE);
	static MPG_Property pTexCoord(PT_TEXCOORD,PB_NONE);
	static vector<int>     v_visited;
	static vector<int>     v_cluster;
	static vector<int> v_father;
	static vector<int> stk;
	static MPG_Stitcher stitcher;
	static MPG_Stitches v_stitches;
	static int             nV0    = 0;  // number of vertices so far without current cc
	static int             nV1    = 0;  // number of vertices so far with current cc
	static int             nVcls0 = 0;  // number of vertices after clustering
	static int             nVcls1 = 0;  // number of vertices after clustering
	static expandHeaderDone = false;

	static MPG_InputStream* istrm = theApp.GetInputStream();

	if(!expandHeaderDone)
	{
		ifs->expandHeader(*istrm, *state, pCoord, pNormal,
			pColor, pTexCoord, qCoord, qNormal, qColor, qTexCoord,
			v_visited, v_cluster, stitcher, v_father, v_stitches, stk);

		// expand rest example: uncomment the following lines
		// ifs->expandRest(*theApp.GetInputStream(), pCoord, pNormal,
		//	pColor, pTexCoord, qCoord, qNormal, qColor, qTexCoord,
		//	v_visited, v_cluster, stitcher, v_father, v_stitches, stk,
		//	nV0, nV1, nVcls0, nVcls1);

		expandHeaderDone = true;
	}

	static bool expandDone = false;
	static bool newMol = true;
	static bool newMobl = true;
	static bool newCC = true;
	static int mol_id = 0;
	static MPG_QFac codec(*istrm);


	if(!expandDone)
	{
		ifs->expandCC(*istrm, pCoord, pNormal,
			pColor, pTexCoord, qCoord, qNormal, qColor, qTexCoord,
			v_visited, v_cluster, 
			stitcher, v_father, v_stitches, stk,
			nV0, nV1, nVcls0, nVcls1, 
			newMol, newMobl, newCC,
			mol_id, codec,
			expandDone);

		ifs->ComputeNormals();

		theApp.FillInIfs(ifs);

		UpdateAllViews(NULL);	
	}
	else
		MessageBox(NULL, "Entire model retrieved", "Message", MB_OK);
	
}

void CProgressiveExpandDoc::OnExpandTriangles() 
{
	P_MPG_IfsQ_E* ifs = theApp.GetIfs();
	if(!ifs)
		ifs = new P_MPG_IfsQ_E;

	static MPG_State *state = new MPG_State(theApp.GetNTriangles());

	// expand header first, then the rest
	// create vectors to store quantized values
	static vector <int> qCoord;
	static vector <int> qNormal;
	static vector <int> qColor;
	static vector <int> qTexCoord;

	// create proper headers and quantizers
	// if not yet created by calling method
	static MPG_Property pCoord(PT_COORD,PB_NONE);
	static MPG_Property pNormal(PT_NORMAL,PB_NONE);
	static MPG_Property pColor(PT_COLOR,PB_NONE);
	static MPG_Property pTexCoord(PT_TEXCOORD,PB_NONE);
	static vector<int>     v_visited;
	static vector<int>     v_cluster;
	static vector<int> v_father;
	static vector<int> stk;
	static MPG_Stitcher stitcher;
	static MPG_Stitches v_stitches;
	static expandHeaderDone = false;

	static MPG_InputStream* istrm = theApp.GetInputStream();

	if(!expandHeaderDone)
	{
		ifs->expandHeader(*istrm, *state, pCoord, pNormal,
			pColor, pTexCoord, qCoord, qNormal, qColor, qTexCoord,
			v_visited, v_cluster, stitcher, v_father, v_stitches, stk);

		expandHeaderDone = true;
	}

	static bool expandDone = false;
	static MPG_QFac codec(*istrm);

	static vector<int> coordIndex;
	static vector<float> coord;
	static vector<float> normal;
	static vector<float> color;
	static vector<float> texCoord;
	static vector<int> f_father;
	static vector<int> c_traversal;
	static vector<int> c_father;
	static vector<int> run_father;
	static vector<int> run_length;
	static vector<bool> run_ends_in_leaf;
	static vector<int> faceStack;
	static MPG_SPolygon sp;
	static vector<int> v_loop_cc;
	static MPG_VGraph vg;
	static MPG_Loops loop; 
	static MPG_Loops c_loop;

	if(!expandDone)
	{
		ifs->expandTriangles(*istrm, *state, pCoord, pNormal,
			pColor, pTexCoord, qCoord, qNormal, qColor, qTexCoord,
			v_visited, v_cluster, stitcher, v_father, v_stitches,
			codec, coordIndex, coord, normal, color, texCoord,
			f_father, c_traversal, c_father, run_father, run_length,
			run_ends_in_leaf, stk, faceStack, sp, vg, v_loop_cc, loop, c_loop,	
			expandDone);
		
		ifs->ComputeNormals();

		theApp.FillInIfs(ifs);

		UpdateAllViews(NULL);	
	}
	else
		MessageBox(NULL, "Entire model retrieved", "Message", MB_OK);
}

void CProgressiveExpandDoc::OnExpandAuto() 
{
	theApp._timerON = 1 - theApp._timerON;
}

void CProgressiveExpandDoc::OnToggleWire() 
{
	GLenum pm = theApp.GetPolygonMode();
	if(pm == GL_LINE)
		theApp.SetPolygonMode(GL_FILL);
	else
		theApp.SetPolygonMode(GL_LINE);
}
