// ProgressiveExpandView.cpp : implementation of the CProgressiveExpandView class
//

#include "stdafx.h"
#include "ProgressiveExpand.h"

#include "ProgressiveExpandDoc.h"
#include "ProgressiveExpandView.h"

#include <GL/gl.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

int *freq;

/////////////////////////////////////////////////////////////////////////////
// CProgressiveExpandView

IMPLEMENT_DYNCREATE(CProgressiveExpandView, CView)

BEGIN_MESSAGE_MAP(CProgressiveExpandView, CView)
	//{{AFX_MSG_MAP(CProgressiveExpandView)
	ON_WM_CREATE()
	ON_WM_MOUSEMOVE()
	ON_WM_RBUTTONDOWN()
	ON_WM_RBUTTONUP()
	ON_WM_TIMER()
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CProgressiveExpandView construction/destruction

CProgressiveExpandView::CProgressiveExpandView()
{
	// TODO: add construction code here

}

CProgressiveExpandView::~CProgressiveExpandView()
{
}

BOOL CProgressiveExpandView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	// required or OpenGL will fail to initialize
	cs.style |= (WS_DISABLED | WS_CLIPCHILDREN | WS_CLIPSIBLINGS);

	return CView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CProgressiveExpandView drawing
extern CProgressiveExpandApp theApp;
void CProgressiveExpandView::OnDraw(CDC* pDC)
{
	CProgressiveExpandDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	// TODO: add draw code for native data here

	TRACE("\n\n--------------------- CProgressiveExpandView::OnDraw() ---------------\t");

	// Viewport rectangle
	RECT rect;
	GetClientRect(&rect);
	long w = rect.right;
	long h = rect.bottom;

	float width = 10.0, height = 10.0, depth = 10.0;
	float cx = 0.0, cy = 0.0, cz = 0.0;
	float aRatio = 1.0;

	float ra = 0.0;
	float *rotAxis = NULL;

	// Set current OpenGL device context to viewport
	wglMakeCurrent(pDC->m_hDC, m_hRC);
	
	// Create some default settings for the scene

//	glClearColor(0.8f, 0.8f, 0.8f, 1.0f);
	glClearColor(1.0f, 1.0f, 1.0f, 1.0f);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glEnable(GL_DEPTH_TEST);

	GLenum pmode = theApp.GetPolygonMode();
	glPolygonMode(GL_FRONT_AND_BACK, pmode);
	if(pmode == GL_FILL)
	{
		glEnable(GL_LIGHTING);
		glEnable(GL_LIGHT0);
		glEnable(GL_NORMALIZE);
		const float material_diffuse[] = { 1.0f, 0.6f, 0.4f, 1.0f };
		const float material_specular[] = { 1.0f, 1.0f, 1.0f, 1.0f };
		const float material_shininess[] = { 10.0f };
		const float lightpos[] = { 1000.0f, 1000.0f, 1000.0f };
		glLightfv(GL_LIGHT0, GL_POSITION, lightpos);
		glMaterialfv( GL_FRONT_AND_BACK, GL_SPECULAR, material_specular );
		glMaterialfv( GL_FRONT_AND_BACK, GL_SHININESS, material_shininess );
		glMaterialfv( GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, material_diffuse );
	}
	else
	{
		glDisable(GL_LIGHTING);
		glDisable(GL_LIGHT0);
		glDisable(GL_NORMALIZE);
		glColor3f(0.0, 0.0, 1.0);
	}

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glViewport(0, 0, w, h);

	P_MPG_IfsQ_E* ifs = theApp.GetIfs();
	if(ifs)
	{
		//ifs->GetBBox(width, height, depth, cx, cy, cz);
		theApp.GetViewParams(width, height, depth, cx, cy, cz);
		TRACE("\nwidth=%f, height=%f, depth=%f, cx=%f, cy=%f, cz=%f",
			width, height, depth, cx, cy, cz);
		aRatio = height / width;

		ra = theApp.GetRotAngle();
		rotAxis = theApp.GetRotAxis();
	}
	glOrtho(-width*aRatio, width*aRatio, -height, height, -depth, depth);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	glPushMatrix();
		if(rotAxis)
			glRotatef(ra, rotAxis[0], rotAxis[1], rotAxis[2]);
		glPushMatrix();
			glTranslatef(-cx, -cy, -cz);
			theApp.DrawOpenGL();
		glPopMatrix();
	glPopMatrix();

	SwapBuffers(pDC->m_hDC);
	
	// Emit the scene
	glFinish();

	// Set current OpenGL device context to NULL
    wglMakeCurrent(NULL, NULL);

	// enable (window was created disabled)
	CView::EnableWindow(true);           
}

/////////////////////////////////////////////////////////////////////////////
// CProgressiveExpandView printing

BOOL CProgressiveExpandView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CProgressiveExpandView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add extra initialization before printing
}

void CProgressiveExpandView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}

/////////////////////////////////////////////////////////////////////////////
// CProgressiveExpandView diagnostics

#ifdef _DEBUG
void CProgressiveExpandView::AssertValid() const
{
	CView::AssertValid();
}

void CProgressiveExpandView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CProgressiveExpandDoc* CProgressiveExpandView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CProgressiveExpandDoc)));
	return (CProgressiveExpandDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CProgressiveExpandView message handlers

int CProgressiveExpandView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{	
		PIXELFORMATDESCRIPTOR pfd = {
		sizeof(PIXELFORMATDESCRIPTOR),   // size of this pfd
		1,                     // version number
		PFD_DRAW_TO_WINDOW |   // Property Flags
			PFD_SUPPORT_OPENGL
		 | PFD_DOUBLEBUFFER,
		PFD_TYPE_RGBA,         // RGBA type
		24,                    // 24-bit color depth
		0, 0, 0, 0, 0, 0,      // color bits ignored
		0,                     // no alpha buffer
		0,                     // shift bit ignored

		0,                     // no accumulation buffer
		0, 0, 0, 0,            // accum bits ignored
		32,                    // 32-bit z-buffer
		0,                     // no stencil buffer
		0,                     // no auxiliary buffer
		PFD_MAIN_PLANE,        // main layer
		0,                     // reserved
		0, 0, 0                // layer masks ignored
	};


    CClientDC clientDC(this);

    int pixelFormat =
        ChoosePixelFormat(clientDC.m_hDC, &pfd);

	ASSERT(pixelFormat != 0);

    BOOL success =
        SetPixelFormat(clientDC.m_hDC, pixelFormat, &pfd);

	ASSERT(success);

	int retval = DescribePixelFormat(clientDC.m_hDC, pixelFormat, sizeof(pfd), &pfd);
	ASSERT(retval != 0);

    m_hRC = wglCreateContext(clientDC.m_hDC);

	ASSERT(m_hRC != NULL);

	m_uTimerID = SetTimer(0x451, 5, NULL);
	freq = new int[100000];
	memset(freq, 0, 100000);

	if (CView::OnCreate(lpCreateStruct) == -1)
		return -1;

	return 0;
}

void CProgressiveExpandView::OnMouseMove(UINT nFlags, CPoint point) 
{
		int sx, sy;
	float wx, wy;
	CRect window;
	float sleft, sright, sbottom, stop;
	WVECTOR v, vp, rotAxis;
	float dp;
	QUAT qdrag;
	const double PI = 3.1415926535;

	if(theApp.GetArcball().dragging)
	{
		sleft = theApp.GetArcball().GetSLeft();
		sright = theApp.GetArcball().GetSRight();
		stop = theApp.GetArcball().GetSTop();
		sbottom = theApp.GetArcball().GetSBottom();

		this->GetTopLevelFrame()->GetClientRect(window);
		sx = point.x; sy = point.y;

		// normalize screen coordinates between -1 and 1 
		wx = sleft + (sright-sleft)*sx / (float)window.Width();
		wy = stop  + (sbottom-stop)*sy / (float)window.Height();

		theApp.GetArcball().worldToBall(wx, wy, v);

		// compute quaternion that defines current rotation 
		theApp.GetArcball().vectProd(theApp.GetArcball().v0, v, vp);
		theApp.GetArcball().dotProd(theApp.GetArcball().v0, v, dp);
		qdrag.v.x = vp.x; qdrag.v.y = vp.y; qdrag.v.z = vp.z;
		qdrag.w = dp;

		// compose current rotation with the old one 
		theApp.GetArcball().quatMult(qdrag, theApp.GetArcball().qDown, theApp.GetArcball().qNow);
		Arcball a = theApp.GetArcball();
		theApp.SetArcball(a);

		// find rotation angle and axis 
		float RotAngle = 180.0 * theApp.GetArcball().GetRotAngle(theApp.GetArcball().qNow)/PI;
		theApp.GetArcball().SetRotAngle(RotAngle);

		theApp.GetArcball().GetRotAxis(theApp.GetArcball().qNow, rotAxis);
		theApp.GetArcball().SetRotAxis(rotAxis);
		a = theApp.GetArcball();
		theApp.SetArcball(a);

		// draw rotated image 
		CClientDC dc(this);
		CProgressiveExpandView::OnDraw(&dc);
	}
	
	CView::OnMouseMove(nFlags, point);
}

void CProgressiveExpandView::OnRButtonDown(UINT nFlags, CPoint point) 
{
	int sx, sy;
	float wx, wy;
	float sleft, sright, sbottom, stop;
	WVECTOR vdown;
	CRect window;

	sleft = theApp.GetArcball().GetSLeft();
	sright = theApp.GetArcball().GetSRight();
	stop = theApp.GetArcball().GetSTop();
	sbottom = theApp.GetArcball().GetSBottom();

	this->GetTopLevelFrame()->GetClientRect(window);
	TRACE("\n\t>> window: (%d, %d)", window.Width(), window.Height());

	SetCapture();
	TRACE("\n\t>> button down: (%d, %d); start rotation", point.x, point.y);

	// screen coordinates where the mouse was pressed 
    sx = point.x; sy = point.y;

    // normalize screen coordinates between -1 and 1 
	wx = sleft + (sright-sleft)*sx / (float)window.Width();
    wy = stop  + (sbottom-stop)*sy / (float)window.Height();

	TRACE("\n\t>> begin drag: wx=%f, wy=%f\t", wx, wy);

    // compute and save sphere coordinates where mouse was pressed 
    theApp.GetArcball().worldToBall(wx, wy, vdown);
    theApp.GetArcball().ballBeginDrag(vdown);
	Arcball a = theApp.GetArcball();
	theApp.SetArcball(a);
	
	CView::OnRButtonDown(nFlags, point);
}

void CProgressiveExpandView::OnRButtonUp(UINT nFlags, CPoint point) 
{
	TRACE("\n\t>> button up: (%d, %d)", point.x, point.y);
	theApp.GetArcball().ballEndDrag();
	Arcball a = theApp.GetArcball();
	theApp.SetArcball(a);

	ReleaseCapture();
	
	CView::OnRButtonUp(nFlags, point);
}

void CProgressiveExpandView::OnTimer(UINT nIDEvent) 
{
	if(theApp._timerON)
	{
		P_MPG_IfsQ_E* ifs = theApp.GetIfs();
		if(!ifs)
			ifs = new P_MPG_IfsQ_E;

		static MPG_State *state = new MPG_State(theApp.GetNTriangles());

		// expand header first, then the rest
		// create vectors to store quantized values
		static vector <int> qCoord;
		static vector <int> qNormal;
		static vector <int> qColor;
		static vector <int> qTexCoord;

		// create proper headers and quantizers
		// if not yet created by calling method
		static MPG_Property pCoord(PT_COORD,PB_NONE);
		static MPG_Property pNormal(PT_NORMAL,PB_NONE);
		static MPG_Property pColor(PT_COLOR,PB_NONE);
		static MPG_Property pTexCoord(PT_TEXCOORD,PB_NONE);
		static vector<int>     v_visited;
		static vector<int>     v_cluster;
		static vector<int> v_father;
		static vector<int> stk;
		static MPG_Stitcher stitcher;
		static MPG_Stitches v_stitches;
		static expandHeaderDone = false;
		static bool saveWRL = false;

		static MPG_InputStream* istrm = theApp.GetInputStream();

		if(!expandHeaderDone)
		{
			ifs->expandHeader(*istrm, *state, pCoord, pNormal,
				pColor, pTexCoord, qCoord, qNormal, qColor, qTexCoord,
				v_visited, v_cluster, stitcher, v_father, v_stitches, stk);

			expandHeaderDone = true;
		}

		static bool expandDone = false;
		static MPG_QFac codec(*istrm);

		static vector<int> coordIndex;
		static vector<float> coord;
		static vector<float> normal;
		static vector<float> color;
		static vector<float> texCoord;
		static vector<int> f_father;
		static vector<int> c_traversal;
		static vector<int> c_father;
		static vector<int> run_father;
		static vector<int> run_length;
		static vector<bool> run_ends_in_leaf;
		//static vector<int> stk;
		static vector<int> faceStack;
		static MPG_SPolygon sp;
		static vector<int> v_loop_cc;
		static MPG_VGraph vg;
		static MPG_Loops loop; 
		static MPG_Loops c_loop;

		if(!expandDone)
		{
			ifs->expandTriangles(*istrm, *state, pCoord, pNormal,
				pColor, pTexCoord, qCoord, qNormal, qColor, qTexCoord,
				v_visited, v_cluster, stitcher, v_father, v_stitches,
				codec, coordIndex, coord, normal, color, texCoord,
				f_father, c_traversal, c_father, run_father, run_length,
				run_ends_in_leaf, stk, faceStack, sp, vg, v_loop_cc, loop, c_loop,	
				expandDone);

			ifs->ComputeNormals();
			theApp.FillInIfs(ifs);

			InvalidateRect(0, FALSE);
			UpdateWindow();
		}
		else
		{
			theApp._timerON = 0;
			KillTimer(m_uTimerID);
			theApp.GetMainWnd()->MessageBox("Entire model retrieved", "Message", MB_OK);

			/*FILE *f = fopen("freq.out", "w");
			int nv1 = ifs->getCoord().size()/3;
			for(int i=0;i<nv1;i++)
				fprintf(f, "\n%d %d", i, freq[i]);
			fclose(f);*/
			delete [] freq;
			
			//exit(0);
		}

	}
	
	CView::OnTimer(nIDEvent);

	// remove extra timer messages
	MSG msg;
	while(::PeekMessage(&msg, m_hWnd, WM_TIMER, WM_TIMER, PM_REMOVE));
}
