//-------------------------------------------------------------------------
// 
// Copyright (C) IBM Corporation & Samsung Electronics Co. LTD., 1996, 1997, 1998, 1999
// Licensed Materials, Program Property of IBM & Samsung.
// 
// This software module is an implementation of a part of one or more
// MPEG-4 tools as specified by the MPEG-4 standard (ISO/IEC 14496).
// ISO/IEC gives users of the MPEG-4 standard (ISO/IEC 14496) free
// license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4
// standard (ISO/IEC 14496).  Those intending to use this software module
// in hardware or software products are advised that its use may infringe
// existing patents.  The original developer of this software module and
// his/her company, the subsequent editors and their companies, and
// ISO/IEC have no liability for use of this software module or
// modifications thereof in an implementation.  Copyright is not released
// for non MPEG-4 standard (ISO/IEC 14496) conforming products.  IBM
// Corp. & Samsung Electronics Co. LTD retains full right to use the code 
// for his/her own purpose, assign or donate the code to a third party and
// to inhibit third parties from using the code for non MPEG-4 standard 
// (ISO/IEC 14496) conforming products. This copyright notice must be included
// in all copies or derivative works.
// 
//  AUTHORS:
//    Gabriel Taubin
//    Andre Gueziec
//    Claudio Silva
//    Ioana Martin
//  
//  IBM T. J. Watson Research Center
//  P.O.Box 704
//  Yorktown Heights, New York 10598
//  
//  BASED ON EARLIER IMPLEMENTATION OF
//  VRML COMPRESSED BINARY FORMAT PROPOSAL BY:
//    Gabriel Taubin
//    Bill Horn
//    Francis Lazarus
//
//  Functionality : Error Resilience
//
//  Sungjin Kim  (sunjin@sait.samsung.co.kr)
//  Mahn-Jin Han  (manjini@sait.samsung.co.kr)
//  Mun-Sup Song (mssong@sait.samsung.co.kr)  
//
//  Signal Processing Lab. Samsung A.I.T. 
//-------------------------------------------------------------------------

#ifndef _MPG_BBoxQ_EXT_h_
#define _MPG_BBoxQ_EXT_h_

#include <vector>
#include "MPG_Stream.h"

using namespace std;

class MPG_BBoxQ_EXT {

protected:

  vector<float> _lowerCorner;
  vector<float> _sideLength;
  float         _avgError;
  float         _maxError;
  

//DMLAB (200501) - ADD
public:
	int         _bitsPerCoordinate;

	int			delta_flag;
	int 		delta_u;
	int			delta_v;
//~DMLAB (200501)

protected:

         void  _update(vector<float>& value);
         void  _update(int nValues, float* value);

public:
  static const float _epsilon;
  enum {_max_n_lambda = 4};   // 2 bits are dedicated to the prediction
  // parameters

  inline       ~MPG_BBoxQ_EXT();
  MPG_BBoxQ_EXT(int nD=1, int nBitsPerCoordinate=10);

  inline void   clear();

  inline int    getDimension();
  inline void   setDimension(int dimension=1);
  inline float* getLowerCorner();
  inline void   setLowerCorner(float* lowerCorner);
  inline void   setLowerCorner(float lowerCorner);
  inline void   setLowerCorner(int i, float lowerCorner_i);

  inline float  getSideLength();
  inline void   setSideLength(float sideLength);
  inline int    getBitsPerCoordinate();
  inline void   setBitsPerCoordinate(int bitsPerCoordinate=10);

  inline void   setBBox(int nValue, float* value);
  inline void   setBBox(vector<float>& value);

         bool   quantize(float* value, int* iqValue, int nValues=1);
         bool   quantize(vector<float>& value, vector<int>& iqValue);
         bool   quantize(vector<float>& value, vector<int>& index, vector<int>& iqValue);
	 bool   unQuantize(int* iqValue, float* value, int nValues=1);
	 bool   unQuantize(vector<int>& iqValue, vector<float>& value);


 //DMLAB (200501) - ADD
	 bool   texquantize(float* value, int* iqValue, int nValues=1,int img_flag=false, int img_u=0, int img_v=0);
     bool   texquantize(vector<float>& value, vector<int>& iqValue,int img_flag, int img_u, int img_v);
     bool   texquantize(vector<float>& value, vector<int>& index, vector<int>& iqValue, int img_flag,int img_u, int img_v);

	 bool   texunQuantize(int* iqValue, float* value, int nValues=1);
	 bool   texunQuantize(vector<int>& iqValue, vector<float>& value);
 //~DMLAB (200501)

  inline float  getAvgError();
  inline float  getMaxError();

         bool   encode(MPG_OutputBitStream_EXT& obstrm);
         bool   decode(MPG_InputBitStream_EXT& ibstrm);

};

inline MPG_BBoxQ_EXT::~MPG_BBoxQ_EXT()
{}

inline void MPG_BBoxQ_EXT::clear()
{
  _lowerCorner.erase(_lowerCorner.begin(),_lowerCorner.end());
  setSideLength(0.0);

  _bitsPerCoordinate = 0;
}

inline int    MPG_BBoxQ_EXT::getDimension()
{ return _lowerCorner.size(); }

inline void   MPG_BBoxQ_EXT::setDimension(int dimension)
{
  // clear(); ???
  while(_lowerCorner.size()<dimension)
    _lowerCorner.push_back(0.0);
  while(_lowerCorner.size()>dimension)
    _lowerCorner.pop_back();
}

inline float* MPG_BBoxQ_EXT::getLowerCorner()
{ return _lowerCorner.begin(); }

inline void   MPG_BBoxQ_EXT::setLowerCorner(float* lowerCorner)
{
  for(int i=0;i<getDimension();i++)
    _lowerCorner[i] = lowerCorner[i];
}

inline void   MPG_BBoxQ_EXT::setLowerCorner(float lowerCorner)
{
  for(int i=0;i<getDimension();i++)
    _lowerCorner[i] = lowerCorner;
}

inline void   MPG_BBoxQ_EXT::setLowerCorner(int i, float lowerCorner_i)
{ _lowerCorner[i] = lowerCorner_i; }

inline float  MPG_BBoxQ_EXT::getSideLength()
{
  float maxSideLength = 0;
  int i;
  for(i=0;i<_sideLength.size();i++)
    if(_sideLength[i]>maxSideLength)
      maxSideLength = _sideLength[i];
  return maxSideLength;
}

inline void   MPG_BBoxQ_EXT::setSideLength(float sideLength)
{
  fill(_sideLength.begin(),_sideLength.end(),sideLength);
}

inline int    MPG_BBoxQ_EXT::getBitsPerCoordinate()
{ return _bitsPerCoordinate; }

inline void   MPG_BBoxQ_EXT::setBitsPerCoordinate(int bitsPerCoordinate)
{ _bitsPerCoordinate = bitsPerCoordinate; }

inline void   MPG_BBoxQ_EXT::setBBox(int nV, float* v)
{
  if(nV>0 && v!=(float*)0)
    {
      int nD = getDimension();
      setLowerCorner(v);
      _sideLength.reserve(nD);
      _sideLength.insert(_sideLength.begin(),nD,0.0);
      _update(nV,v);
    }
}

inline void   MPG_BBoxQ_EXT::setBBox(vector<float>& vec)
{
  int   nV = vec.size()/getDimension();
  float *v = vec.begin();
  setBBox(nV,v);
}

// should be called after MPG_BBoxQ_EXT::quantize()
// which computes this value
inline float  MPG_BBoxQ_EXT::getAvgError()
{ return _avgError; }

// should be called after MPG_BBoxQ_EXT::quantize()
// which computes this value
inline float  MPG_BBoxQ_EXT::getMaxError()
{ return _maxError; }

#endif // _MPG_BBoxQ_EXT_h_

