//-------------------------------------------------------------------------
// 
// Copyright (C) IBM Corporation & Samsung Electronics Co. LTD., 1996, 1997, 1998, 1999
// Licensed Materials, Program Property of IBM & Samsung.
// 
// This software module is an implementation of a part of one or more
// MPEG-4 tools as specified by the MPEG-4 standard (ISO/IEC 14496).
// ISO/IEC gives users of the MPEG-4 standard (ISO/IEC 14496) free
// license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4
// standard (ISO/IEC 14496).  Those intending to use this software module
// in hardware or software products are advised that its use may infringe
// existing patents.  The original developer of this software module and
// his/her company, the subsequent editors and their companies, and
// ISO/IEC have no liability for use of this software module or
// modifications thereof in an implementation.  Copyright is not released
// for non MPEG-4 standard (ISO/IEC 14496) conforming products.  IBM
// Corp. & Samsung Electronics Co. LTD retains full right to use the code 
// for his/her own purpose, assign or donate the code to a third party and
// to inhibit third parties from using the code for non MPEG-4 standard 
// (ISO/IEC 14496) conforming products. This copyright notice must be included
// in all copies or derivative works.
// 
//  AUTHORS:
//    Gabriel Taubin
//    Andre Gueziec
//    Claudio Silva
//    Ioana Martin
//  
//  IBM T. J. Watson Research Center
//  P.O.Box 704
//  Yorktown Heights, New York 10598
//  
//  BASED ON EARLIER IMPLEMENTATION OF
//  VRML COMPRESSED BINARY FORMAT PROPOSAL BY:
//    Gabriel Taubin
//    Bill Horn
//    Francis Lazarus
//
//  Functionality : Error Resilience
//
//  Sungjin Kim  (sunjin@sait.samsung.co.kr)
//  Mahn-Jin Han  (manjini@sait.samsung.co.kr)
//  Mun-Sup Song (mssong@sait.samsung.co.kr)  
//
//  Signal Processing Lab. Samsung A.I.T. 
//-------------------------------------------------------------------------

#ifndef _MPG_ClustSurf_h_
#define _MPG_ClustSurf_h_

#include "MPG_Mesh.h"
#include "MPG_IfsQ_EXT.h"

class MPG_ClustSurf {

protected:

  int                   _nD;	// vertex dimension

  // Ifs variables
  bool                  _ccw;
  bool                  _convex;
  float                 _creaseAngle;
  bool                  _solid;

  // connectivity
  vector<int>           _coordIndex;

  // clustering maps
  vector<int>           _coordMap;

  // property bindings

  MPG_Binding           _coordBinding;
  MPG_Binding           _normalBinding;
  MPG_Binding           _colorBinding;
  MPG_Binding           _texCoordBinding;

  // property values

  vector<float>          _coord;
  vector<float>          _normal;
  vector<float>          _color;
  vector<float>          _texCoord;

  vector<int>            _coordFirst;
  vector<int>            _normalFirst;
  vector<int>            _colorFirst;
  vector<int>            _texCoordFirst;

  // quantization
  int                    _bitsPerCoord;
  int                    _bitsPerNormal;
  int                    _bitsPerColor;
  int                    _bitsPerTexCoord;

  MPG_BBoxQ*             _bbqCoord;
  MPG_NormalQ*           _nrqNormal;
  MPG_BBoxQ*             _bbqColor;
  MPG_BBoxQ*             _bbqTexCoord;

public:

  inline                ~MPG_ClustSurf();
                         MPG_ClustSurf(int nD=3);
         MPG_ClustSurf&  operator=(MPG_Ifs& ifs);
         MPG_ClustSurf&  operator=(MPG_IfsQ_EXT& ifs);
  inline                 MPG_ClustSurf(MPG_Ifs& ifs);
  inline                 MPG_ClustSurf(MPG_IfsQ_EXT& ifs);
         void            clear();

  // I/O

  // I/O
  int                     readFaces(FILE *fp);
  int                     readTriangles(FILE *fp);
  int                     writeFaces(FILE *fp);
  int                     writeTriangles(FILE *fp);

         bool            read(char *fname=(char*)0);
         bool            write(char *fname=(char*)0);
         bool            write(int level=0, const char *baseName=(char*)0);

  inline int             getNumberOfLevels();
  inline int             getDimension();
  inline void            setDimension(int nD);

  // level management

         bool         prependLevel(vector<int>&   coordIndex,
				   vector<int>&   coordMap,
				   vector<float>& coord,
				   vector<float>& normal,
				   vector<float>& color,
				   vector<float>& texCoord);

         void         closeLevel();

         void         appendLevel(vector<int>&   connInfo,
				  vector<float>& coord,
				  vector<float>& normal,
				  vector<float>& color,
				  vector<float>& texCoord);

         void         appendLevel(vector<int>&   connInfo,
				  int            nCoord,
				  int            nNormal,
				  int            nColor,
				  int            nTexCoord);

         void         deleteLastLevel();

  // clustering maps

  inline int*         getMap(int level);
  inline bool         getMap(int level, vector<int>& cmap);
  inline int          getMap(int level, int i);
  inline void         setMap(int level, int i, int map_i);

  // Mesh

  inline void         setCoordIndex(vector<int>& coordIndex);
  bool                getCoordIndex(int level, vector<int>& coordIndex);

  // Ifs

         bool         getIfs(MPG_Ifs& ifs, int level);

  // Ifs variables

  inline void         setCcw(bool ccw);
  inline void         setConvex(bool convex);
  inline void         setCreaseAngle(float creaseAngle);
  inline void         setSolid(bool solid);

  inline bool         getCcw();
  inline bool         getConvex();
  inline float        getCreaseAngle();
  inline bool         getSolid();

  // quantization

  inline void         setBitsPerCoord(int n);
  inline void         setBitsPerNormal(int n);
  inline void         setBitsPerColor(int n);
  inline void         setBitsPerTexCoord(int n);

  inline int          getBitsPerCoord();
  inline int          getBitsPerNormal();
  inline int          getBitsPerColor();
  inline int          getBitsPerTexCoord();

  inline MPG_BBoxQ*   getCoordQuantizer(int bitsPerCoord);
  inline MPG_BBoxQ*   getCoordQuantizer();
  inline void                 setCoordQuantizer(MPG_BBoxQ* bbq);
  inline MPG_NormalQ* getNormalQuantizer(int bitsPerNormal);
  inline MPG_NormalQ* getNormalQuantizer();
  inline void                 setNormalQuantizer(MPG_NormalQ* nrq);
  inline MPG_BBoxQ*   getColorQuantizer(int bitsPerColor);
  inline MPG_BBoxQ*   getColorQuantizer();
  inline void                 setColorQuantizer(MPG_BBoxQ* bbq);
  inline MPG_BBoxQ*   getTexCoordQuantizer(int bitsPerCoord);
  inline MPG_BBoxQ*   getTexCoordQuantizer();
  inline void                 setTexCoordQuantizer(MPG_BBoxQ* bbq);

  // properties

  inline MPG_Binding  getCoordBinding();
  inline MPG_Binding  getNormalBinding();
  inline MPG_Binding  getColorBinding();
  inline MPG_Binding  getTexCoordBinding();

  inline bool         setCoordBinding(MPG_Binding coordBinding);
  inline bool         setNormalBinding(MPG_Binding normalBinding);
  inline bool         setColorBinding(MPG_Binding colorBinding);
  inline bool         setTexCoordBinding(MPG_Binding texCoordBinding);

  inline int          getNumberOfCoord(int level=0);
  inline int          getNumberOfNormal(int level=0);
  inline int          getNumberOfColor(int level=0);
  inline int          getNumberOfTexCoord(int level=0);

  inline vector<float>& getCoord();
  inline vector<float>& getNormal();
  inline vector<float>& getColor();
  inline vector<float>& getTexCoord();

  inline float*       getCoord(int level, int indx=0);
  inline float*       getNormal(int level, int indx=0);
  inline float*       getColor(int level, int indx=0);
  inline float*       getTexCoord(int level, int indx=0);

  inline float        getCoord(int level, int indx, int j);
  inline float        getNormal(int level, int indx, int j);
  inline float        getColor(int level, int indx, int j);
  inline float        getTexCoord(int level, int indx, int j);

  inline bool         getCoord(int level, vector<float>& coord);
  inline bool         getNormal(int level, vector<float>& normal);
  inline bool         getColor(int level, vector<float>& color);
  inline bool         getTexCoord(int level, vector<float>& texCoord);

  inline void         deleteNormal();
  inline void         deleteColor();
  inline void         deleteTexCoord();

  // pfs simplification
         bool         pfsGenerate(int move_vertices);

protected:

         void        _error(char *msg);
};

// MPG_ClustSurf::~MPG_ClustSurf ---------------------------------------------

inline MPG_ClustSurf::~MPG_ClustSurf()
{}

inline MPG_ClustSurf::MPG_ClustSurf(MPG_Ifs& ifs):
  _coordIndex(ifs.getCoordIndex())
{
  clear();
  *this = ifs;
}

inline MPG_ClustSurf::MPG_ClustSurf(MPG_IfsQ_EXT& ifs):
  _coordIndex(ifs.getCoordIndex())
{
  clear();
  *this = ifs;
}

inline void MPG_ClustSurf::setCoordIndex(vector<int>& coordIndex)
{
  clear();
  _coordIndex = coordIndex;
}

inline int MPG_ClustSurf::getNumberOfLevels()
{ return _coordFirst.size()-1; }

inline int MPG_ClustSurf::getDimension()
{ return _nD; }

inline void MPG_ClustSurf::setDimension(int nD)
{
  if(getNumberOfCoord(0)>0) clear();
  _nD = nD;
}

inline int* MPG_ClustSurf::getMap(int level)
{
  if(level<0 || level>=getNumberOfLevels() ||
     _coordFirst.size()<=1) return NULL;
  return &(_coordMap[_coordFirst[level]]);
}

inline bool MPG_ClustSurf::getMap(int level, vector<int>& cmap)
{
  if(level<0 || level>=getNumberOfLevels()) return false;
  cmap.erase(cmap.begin(),cmap.end());
  int i;
  for(i=_coordFirst[level];i<_coordFirst[level+1];i++)
    cmap.push_back(_coordMap[i]);
  return true;
}

inline int MPG_ClustSurf::getMap(int level, int indx)
{
  if(getNumberOfLevels()<=1) return -1;
  return _coordMap[_coordFirst[level]+indx];
}

inline void MPG_ClustSurf::setMap(int level, int indx, int map_i)
{
  if(getNumberOfLevels()<=1) return;
  _coordMap[_coordFirst[level]+indx] = map_i;
}

inline void MPG_ClustSurf::setBitsPerCoord(int n)
{ _bitsPerCoord = n; }

inline void MPG_ClustSurf::setBitsPerNormal(int n)
{ _bitsPerNormal = n; }

inline void MPG_ClustSurf::setBitsPerColor(int n)
{ _bitsPerColor = n; }

inline void MPG_ClustSurf::setBitsPerTexCoord(int n)
{ _bitsPerTexCoord = n; }

inline int MPG_ClustSurf::getBitsPerCoord()
{ return _bitsPerCoord; }

inline int MPG_ClustSurf::getBitsPerNormal()
{ return _bitsPerNormal; }

inline int MPG_ClustSurf::getBitsPerColor()
{ return _bitsPerColor; }

inline int MPG_ClustSurf::getBitsPerTexCoord()
{ return _bitsPerTexCoord; }

// Ifs variables

inline void MPG_ClustSurf::setCcw(bool ccw)
{ _ccw = ccw; }

inline void MPG_ClustSurf::setConvex(bool convex)
{ _convex = convex; }

inline void MPG_ClustSurf::setCreaseAngle(float creaseAngle)
{ _creaseAngle = creaseAngle; }

inline void MPG_ClustSurf::setSolid(bool solid)
{ _solid = solid; }

inline bool MPG_ClustSurf::getCcw()
{ return _ccw; }

inline bool MPG_ClustSurf::getConvex()
{ return _convex; }

inline float MPG_ClustSurf::getCreaseAngle()
{ return _creaseAngle; }

inline bool MPG_ClustSurf::getSolid()
{ return _solid; }

// properties

inline MPG_Binding MPG_ClustSurf::getCoordBinding()
{ return _coordBinding; }

inline MPG_Binding MPG_ClustSurf::getNormalBinding()
{ return _normalBinding; }

inline MPG_Binding MPG_ClustSurf::getColorBinding()
{ return _colorBinding; }

inline MPG_Binding MPG_ClustSurf::getTexCoordBinding()
{ return _texCoordBinding; }

inline bool MPG_ClustSurf::setCoordBinding(MPG_Binding coordBinding)
{
  if(_coord.size()>0) return false;
  _coordBinding = coordBinding;
  return true;
}

inline bool MPG_ClustSurf::setNormalBinding(MPG_Binding normalBinding)
{
  if(_coord.size()>0) return false;
  _normalBinding = normalBinding;
  return true;
}

inline bool MPG_ClustSurf::setColorBinding(MPG_Binding colorBinding)
{
  if(_coord.size()>0) return false;
  _colorBinding = colorBinding;
  return true;
}

inline bool MPG_ClustSurf::setTexCoordBinding(MPG_Binding texCoordBinding)
{
  if(_coord.size()>0) return false;
  _texCoordBinding = texCoordBinding;
  return true;
}

inline int MPG_ClustSurf::getNumberOfCoord(int level)
{
  if(_coordFirst.size()<=1) return 0;
  return _coordFirst[level+1]-_coordFirst[level];
}

inline int MPG_ClustSurf::getNumberOfNormal(int level)
{
  if(_normalFirst.size()<=1) return 0;
  return _normalFirst[level+1]-_normalFirst[level];
}

inline int MPG_ClustSurf::getNumberOfColor(int level)
{
  if(_colorFirst.size()<=1) return 0;
  return _colorFirst[level+1]-_colorFirst[level];
}

inline int MPG_ClustSurf::getNumberOfTexCoord(int level)
{
  if(_texCoordFirst.size()<=1) return 0;
  return _texCoordFirst[level+1]-_texCoordFirst[level];
}

inline vector<float>& MPG_ClustSurf::getCoord()
{ return _coord; }

inline vector<float>& MPG_ClustSurf::getNormal()
{ return _normal; }

inline vector<float>& MPG_ClustSurf::getColor()
{ return _color; }

inline vector<float>& MPG_ClustSurf::getTexCoord()
{ return _texCoord; }

inline float* MPG_ClustSurf::getCoord(int level, int indx)
{
  if(_coordFirst.size()<=1) return NULL;
  return &(_coord[_nD*(_coordFirst[level]+indx)]);
}

inline float* MPG_ClustSurf::getNormal(int level, int indx)
{
  if(_normalFirst.size()<=1) return NULL;
  return &(_normal[3*(_normalFirst[level]+indx)]);
  return NULL;
}

inline float* MPG_ClustSurf::getColor(int level, int indx)
{
  if(_colorFirst.size()<=1) return NULL;
  return &(_color[3*(_colorFirst[level]+indx)]);
  return NULL;
}

inline float* MPG_ClustSurf::getTexCoord(int level, int indx)
{
  if(_texCoordFirst.size()<=1) return NULL;
  return &(_texCoord[2*(_texCoordFirst[level]+indx)]);
}

inline float MPG_ClustSurf::getCoord(int level, int indx, int j)
{ return getCoord(level,indx)[j]; }

inline float MPG_ClustSurf::getNormal(int level, int indx, int j)
{ return getNormal(level,indx)[j]; }

inline float MPG_ClustSurf::getColor(int level, int indx, int j)
{ return getColor(level,indx)[j]; }

inline float MPG_ClustSurf::getTexCoord(int level, int indx, int j)
{ return getTexCoord(level,indx)[j]; }

inline bool MPG_ClustSurf::getCoord(int level, vector<float>& coord)
{
  coord.erase(coord.begin(),coord.end());
  int nL = getNumberOfLevels();
  if(level<0 || level>=nL) return false;
  if(_coordFirst[level]<_coordFirst[level+1])
    coord.insert(coord.end(),
		 _coord.begin()+_nD*_coordFirst[level],
		 _coord.begin()+_nD*_coordFirst[level+1]);
  return true;
}

inline bool MPG_ClustSurf::getNormal(int level, vector<float>& normal)
{
  normal.erase(normal.begin(),normal.end());
  int nL = getNumberOfLevels();
  if(level<0 || level>=nL) return false;
  if(_normalFirst[level]<_normalFirst[level+1])
    normal.insert(normal.end(),
		  _normal.begin()+3*_normalFirst[level],
		  _normal.begin()+3*_normalFirst[level+1]);
  return true;
}

inline bool MPG_ClustSurf::getColor(int level, vector<float>& color)
{
  color.erase(color.begin(),color.end());
  int nL = getNumberOfLevels();
  if(level<0 || level>=nL) return false;
  if(_colorFirst[level]<_colorFirst[level+1])
    color.insert(color.end(),
		 _color.begin()+2*_colorFirst[level],
		 _color.begin()+2*_colorFirst[level+1]);
  return true;
}

inline bool MPG_ClustSurf::getTexCoord(int level, vector<float>& texCoord)
{
  texCoord.erase(texCoord.begin(),texCoord.end());
  int nL = getNumberOfLevels();
  if(level<0 || level>=nL) return false;
  if(_texCoordFirst[level]<_texCoordFirst[level+1])
    texCoord.insert(texCoord.end(),
		    _texCoord.begin()+2*_texCoordFirst[level],
		    _texCoord.begin()+2*_texCoordFirst[level+1]);
  return true;
}

inline void MPG_ClustSurf::deleteNormal()
{
  _normalBinding = PB_NONE;
  _normal.erase(_normal.begin(),_normal.end());
  int i;
  for(i=0;i<_normal.size();i++)
    _normalFirst[i] = 0;
}

inline void MPG_ClustSurf::deleteColor()
{
  _colorBinding = PB_NONE;
  _color.erase(_color.begin(),_color.end());
  int i;
  for(i=0;i<_color.size();i++)
    _colorFirst[i] = 0;
}

inline void MPG_ClustSurf::deleteTexCoord()
{
  _texCoordBinding = PB_NONE;
  _texCoord.erase(_texCoord.begin(),_texCoord.end());
  int i;
  for(i=0;i<_texCoord.size();i++)
    _texCoordFirst[i] = 0;
}

inline MPG_BBoxQ* MPG_ClustSurf::getCoordQuantizer(int bitsPerCoord)
{
  if(bitsPerCoord>0)
    {
      if(_bbqCoord==(MPG_BBoxQ*)0)
	{
	  _bbqCoord = new MPG_BBoxQ(3,bitsPerCoord);
	  _bbqCoord->setBBox(_coord);
	}
      else if(_bbqCoord->getBitsPerCoordinate()!=bitsPerCoord)
	_bbqCoord->setBitsPerCoordinate(bitsPerCoord);
    }
  // else just return current quantizer
  return _bbqCoord;
}

inline MPG_BBoxQ* MPG_ClustSurf::getCoordQuantizer()
{ return getCoordQuantizer(_bitsPerCoord); }

inline MPG_NormalQ* MPG_ClustSurf::getNormalQuantizer(int bitsPerNormal)
{
  if(bitsPerNormal>0)
    {
      if(_nrqNormal==(MPG_NormalQ*)0)
	_nrqNormal   = new MPG_NormalQ(bitsPerNormal);
      else if(_nrqNormal->getBitsPerNormal()!=bitsPerNormal)
	_nrqNormal->setBitsPerNormal(bitsPerNormal);
    }
  // else just return current quantizer
  return _nrqNormal;
}

inline MPG_NormalQ* MPG_ClustSurf::getNormalQuantizer()
{ return getNormalQuantizer(_bitsPerNormal); }

inline MPG_BBoxQ* MPG_ClustSurf::getColorQuantizer(int bitsPerColor)
{
  if(bitsPerColor>0)
    {
      if(_bbqColor==(MPG_BBoxQ*)0)
	{
	  _bbqColor = new MPG_BBoxQ(3,bitsPerColor);
	  _bbqColor->setBBox(_color);
	}
      else if(_bbqColor->getBitsPerCoordinate()!=bitsPerColor)
	_bbqColor->setBitsPerCoordinate(bitsPerColor);
    }
  // else just return current quantizer
  return _bbqColor;
}

inline MPG_BBoxQ* MPG_ClustSurf::getColorQuantizer()
{ return getColorQuantizer(_bitsPerColor); }

inline MPG_BBoxQ* MPG_ClustSurf::getTexCoordQuantizer(int bitsPerTexCoord)
{
  if(bitsPerTexCoord>0)
    {
      if(_bbqTexCoord==(MPG_BBoxQ*)0)
	{
	  _bbqTexCoord = new MPG_BBoxQ(2,bitsPerTexCoord);
	  _bbqTexCoord->setBBox(_texCoord);
	}
      else if(_bbqTexCoord->getBitsPerCoordinate()!=bitsPerTexCoord)
	_bbqTexCoord->setBitsPerCoordinate(bitsPerTexCoord);
    }
  // else just return current quantizer
  return _bbqTexCoord;
}

inline MPG_BBoxQ* MPG_ClustSurf::getTexCoordQuantizer()
{ return getTexCoordQuantizer(_bitsPerTexCoord); }

inline void MPG_ClustSurf::setCoordQuantizer(MPG_BBoxQ* bbq)
{
  // if current quantizer is no longer referenced to elsewhere,
  // it should be deleted before replacing it with this
  // method
  // if(_bbqCoord==(MPG_BBoxQ*)0) delete _bbqCoord;
  _bbqCoord = bbq;
}

inline void MPG_ClustSurf::setNormalQuantizer(MPG_NormalQ* nrq)
{
  // if current quantizer is no longer referenced to elsewhere,
  // it should be deleted before replacing it with this
  // method
  // if(_bbqNormal==(MPG_BBoxQ*)0) delete _bbqNormal;
  _nrqNormal = nrq;
}

inline void MPG_ClustSurf::setColorQuantizer(MPG_BBoxQ* bbq)
{
  // if current quantizer is no longer referenced to elsewhere,
  // it should be deleted before replacing it with this
  // method
  // if(_bbqColor==(MPG_BBoxQ*)0) delete _bbqColor;
  _bbqColor = bbq;
}

inline void MPG_ClustSurf::setTexCoordQuantizer(MPG_BBoxQ* bbq)
{
  // if current quantizer is no longer referenced to elsewhere,
  // it should be deleted before replacing it with this
  // method
  // if(_bbqTexCoord==(MPG_BBoxQ*)0) delete _bbqTexCoord;
  _bbqTexCoord = bbq;
}


#endif // _MPG_ClustSurf_h_
