//-------------------------------------------------------------------------
// 
// Copyright (C) IBM Corporation & Samsung Electronics Co. LTD., 1996, 1997, 1998, 1999
// Licensed Materials, Program Property of IBM & Samsung.
// 
// This software module is an implementation of a part of one or more
// MPEG-4 tools as specified by the MPEG-4 standard (ISO/IEC 14496).
// ISO/IEC gives users of the MPEG-4 standard (ISO/IEC 14496) free
// license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4
// standard (ISO/IEC 14496).  Those intending to use this software module
// in hardware or software products are advised that its use may infringe
// existing patents.  The original developer of this software module and
// his/her company, the subsequent editors and their companies, and
// ISO/IEC have no liability for use of this software module or
// modifications thereof in an implementation.  Copyright is not released
// for non MPEG-4 standard (ISO/IEC 14496) conforming products.  IBM
// Corp. & Samsung Electronics Co. LTD retains full right to use the code 
// for his/her own purpose, assign or donate the code to a third party and
// to inhibit third parties from using the code for non MPEG-4 standard 
// (ISO/IEC 14496) conforming products. This copyright notice must be included
// in all copies or derivative works.
// 
//  AUTHORS:
//    Gabriel Taubin
//    Andre Gueziec
//    Claudio Silva
//    Ioana Martin
//  
//  IBM T. J. Watson Research Center
//  P.O.Box 704
//  Yorktown Heights, New York 10598
//  
//  BASED ON EARLIER IMPLEMENTATION OF
//  VRML COMPRESSED BINARY FORMAT PROPOSAL BY:
//    Gabriel Taubin
//    Bill Horn
//    Francis Lazarus
//
//  Functionality : Error Resilience
//
//  Sungjin Kim  (sunjin@sait.samsung.co.kr)
//  Mahn-Jin Han  (manjini@sait.samsung.co.kr)
//  Mun-Sup Song (mssong@sait.samsung.co.kr)  
//
//  Signal Processing Lab. Samsung A.I.T. 
//-------------------------------------------------------------------------


#ifndef _MPG_ER_EXT_h_
#define _MPG_ER_EXT_h_

#include <vector>
#include "MPG_Stream.h"
#include "MPG_QFac.h"
#include "MPG_BranchTree.h"

using namespace std;

inline int slog2(int x) {
	int n = 0;
	while (x >= 1) {
		n++;
		x >>= 1;
	}
	return n;
}

class MPG_ER_EXT {
private:
	vector<int> run;
	vector<int> start_pos;
	vector<int> branch_pos;
	vector<int> direct;
	vector<int> first;
	vector<int> second;
	vector<int> run_father;
	vector<int> run_length;
	vector<int> run_leaf;

public:
	~MPG_ER_EXT() {}
	MPG_ER_EXT() {}

	inline void clear() {
		run.erase(run.begin(), run.end());
		start_pos.erase(start_pos.begin(), start_pos.end());
		branch_pos.erase(branch_pos.begin(), branch_pos.end());
		direct.erase(direct.begin(), direct.end());
		first.erase(first.begin(), first.end());
		second.erase(second.begin(), second.end());
		run_father.erase(run_father.begin(), run_father.end());
		run_length.erase(run_length.begin(), run_length.end());
		run_leaf.erase(run_leaf.begin(), run_leaf.end());
	}

	inline int getRun(int irun) { return run[irun]; }
	inline int getStart(int irun) { return start_pos[irun]; }
	inline int getBranch(int irun) { return branch_pos[irun]; }
	inline int getDirect(int irun) { return direct[irun]; }
	inline int getFirst(int irun) { return first[irun]; }
	inline int getSecond(int irun) { return second[irun]; }
	inline int getFather(int irun) { return run_father[irun]; }
	inline int getLength(int irun) { return run_length[irun]; }
	inline int getLeaf(int irun) { return run_leaf[irun]; }
	
	inline void putRun(int i) { run.push_back(i); }
	inline void putStart(int i) { start_pos.push_back(i); }
	inline void putBranch(int i) { branch_pos.push_back(i); }
	inline void putDirect(int i) { direct.push_back(i); }
	inline void putFirst(int i) { first.push_back(i); }
	inline void putSecond(int i) { second.push_back(i); }
	inline void putFather(int i) { run_father.push_back(i); }
	inline void putLength(int i) { run_length.push_back(i); }
	inline void putLeaf(int i) { run_leaf.push_back(i); }

	inline int getNumberOfRuns() { return run.size(); }

	void setDirection();
	void printDirection();
};

class MPG_ER_EXT2;

class MPG_PacketQ_C_EXT {
public:
	int vg_sc_enable;
	int vg_pt0_enable;

	int sp_sc_enable;
	int sp_pt2_enable;
	int sp_pass0;
	int sp_partitioning;
	int sp_branch_len;
	int sp_left_bloop_idx;
	int sp_right_bloop_idx;
	int sp_bdry_pred;
	int sp_vg_id;
	int vg_id_len;
	int sp_sub_pt_enable;
	int start_run, start_tri;
	int next_run, next_tri;
	int nvg;

	int next_spbits;
	int bdry_loopbits;
	int totalbits;
	int restbits;
	int erMode;
	int packetSize;
	int mobl_id;

public:
	~MPG_PacketQ_C_EXT() {}
	MPG_PacketQ_C_EXT(int er, int ps, int total, int rest) {
		erMode=er;
		vg_pt0_enable=1;   
		vg_sc_enable= 1;
		packetSize=ps*8;
		totalbits=total;
		restbits=rest;
		mobl_id=0;
	}

	void vg_init(int ptype) {
		vg_pt0_enable = (ptype) ? 0:1;
		nvg=0;
	}

	void sp_init(int vloop, int root, int tri) {
		sp_pt2_enable=!vg_pt0_enable;
		sp_sc_enable=sp_pt2_enable;
		sp_branch_len=0;
	    sp_left_bloop_idx=1;
	    sp_right_bloop_idx=0;
		sp_sub_pt_enable=0;
		sp_bdry_pred=0;
		sp_pass0=1;
		sp_partitioning=0;

		start_run=0;
		start_tri=0;
	    next_run=0;
		next_tri=0;

		//tt(8)+bl+td(root+tri)+last(1)+flush(40)
		bdry_loopbits = slog2(vloop);
		next_spbits = 49+bdry_loopbits+root+tri*2; 
		if(next_spbits % 8 != 0) 
			next_spbits += (8-(next_spbits%8));
	}

	bool vg_StartCode(
					MPG_OutputStream_EXT& vgostrm,
					MPG_OutputStream_EXT& ostrm, 
					MPG_OutputBitStream_EXT& vgobstrm,
					MPG_OutputBitStream_EXT& obstrm,
					MPG_QFac_EXT& vgcodec, 
					MPG_QFac_EXT& codec,
					int &vg_mobl_bits);

	void vg_packetization(
					MPG_OutputStream_EXT& vgostrm,
					MPG_OutputStream_EXT& ostrm, 
					MPG_OutputBitStream_EXT& vgobstrm,
					MPG_OutputBitStream_EXT& obstrm,
					MPG_QFac_EXT& vgcodec, 
					MPG_QFac_EXT& codec,
					int icc,
					MPG_ER_EXT2& ccbit);

	bool sp_StartCode(
					MPG_OutputStream_EXT& spostrm,
					MPG_OutputStream_EXT& ostrm, 
					MPG_OutputBitStream_EXT& spobstrm,
					MPG_OutputBitStream_EXT& obstrm,
					MPG_QFac_EXT& spcodec, 
					MPG_QFac_EXT& codec,
					int &sp_mobl_bits);

	void sp_packetization(
					MPG_OutputStream_EXT& vgostrm,
					MPG_OutputStream_EXT& spostrm,
					MPG_OutputStream_EXT& ostrm, 
					MPG_OutputBitStream_EXT& vgobstrm,
					MPG_OutputBitStream_EXT& spobstrm,
					MPG_OutputBitStream_EXT& obstrm,
					MPG_QFac_EXT& vgcodec, 
					MPG_QFac_EXT& spcodec, 
					MPG_QFac_EXT& codec,
					int icc,
					int nCC,
					int &next,
					int Gdepth,
					int &sptbits,
					int &sprbits,
					MPG_BranchTree_EXT*& cur_bt,
					MPG_BranchTree_EXT*& tmp_bt,
					MPG_ER_EXT2& ccbit
					);

	void writeStream(MPG_OutputStream_EXT& s1, 
					        MPG_OutputStream_EXT& d1);

    void updateQFac(MPG_OutputBitStream_EXT& s1,
					     MPG_OutputBitStream_EXT& d1,
					     MPG_QFac_EXT& s2,
					     MPG_QFac_EXT& d2);

	//MJH 001218 : removed
    //int stuffing(MPG_OutputStream_EXT& s1,
	//					MPG_OutputBitStream_EXT& s2,
	//					MPG_QFac_EXT& s3);
};

class MPG_ER_EXT2 {
public:
	int curr;
	int next;
	int partition_type;
	vector<int> vgbits;
	vector<int> ttbits;
	vector<int> tdbits;

public:
	~MPG_ER_EXT2() {}
	MPG_ER_EXT2() 
	{
		curr=0;
		next=0;
		partition_type=0;
	}

	void clear() {
		vgbits.erase(vgbits.begin(), vgbits.end());
		ttbits.erase(ttbits.begin(), ttbits.end());
		tdbits.erase(tdbits.begin(), tdbits.end());
	}

	int  getVGbits(int irun) { return vgbits[irun]; }
	int  getTTbits(int irun) { return ttbits[irun]; }
	int  getTDbits(int irun) { return tdbits[irun]; }
	void putVGbits(int i) { vgbits.push_back(i); }
	void putTTbits(int i) { ttbits.push_back(i); }
	void putTDbits(int i) { tdbits.push_back(i); }

	bool determine_PT(int icc, int nCC, MPG_PacketQ_C_EXT& codap, 
		MPG_OutputStream_EXT& s1, MPG_OutputBitStream_EXT& s2, MPG_QFac_EXT& s3);  
	void estimate_packet(int type, int nCC, MPG_PacketQ_C_EXT& codap, int& pbits); 
};

#endif // _MPG_ER_EXT_h_
