//-------------------------------------------------------------------------
// 
// Copyright (C) IBM Corporation & Samsung Electronics Co. LTD., 1996, 1997, 1998, 1999
// Licensed Materials, Program Property of IBM & Samsung.
// 
// This software module is an implementation of a part of one or more
// MPEG-4 tools as specified by the MPEG-4 standard (ISO/IEC 14496).
// ISO/IEC gives users of the MPEG-4 standard (ISO/IEC 14496) free
// license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4
// standard (ISO/IEC 14496).  Those intending to use this software module
// in hardware or software products are advised that its use may infringe
// existing patents.  The original developer of this software module and
// his/her company, the subsequent editors and their companies, and
// ISO/IEC have no liability for use of this software module or
// modifications thereof in an implementation.  Copyright is not released
// for non MPEG-4 standard (ISO/IEC 14496) conforming products.  IBM
// Corp. & Samsung Electronics Co. LTD retains full right to use the code 
// for his/her own purpose, assign or donate the code to a third party and
// to inhibit third parties from using the code for non MPEG-4 standard 
// (ISO/IEC 14496) conforming products. This copyright notice must be included
// in all copies or derivative works.
// 
//  AUTHORS:
//    Gabriel Taubin
//    Andre Gueziec
//    Claudio Silva
//    Ioana Martin
//  
//  IBM T. J. Watson Research Center
//  P.O.Box 704
//  Yorktown Heights, New York 10598
//  
//  BASED ON EARLIER IMPLEMENTATION OF
//  VRML COMPRESSED BINARY FORMAT PROPOSAL BY:
//    Gabriel Taubin
//    Bill Horn
//    Francis Lazarus
//
//  Functionality : Error Resilience
//
//  Sungjin Kim  (sunjin@sait.samsung.co.kr)
//  Mahn-Jin Han  (manjini@sait.samsung.co.kr)
//  Mun-Sup Song (mssong@sait.samsung.co.kr)  
//
//  Signal Processing Lab. Samsung A.I.T. 
//-------------------------------------------------------------------------

#ifndef _MPG_Global_EXT_H_
#define _MPG_Global_EXT_H_

#include "MPG_QFpm.h"

class MPG_Global_EXT {

public:

  // start codes
  static const char MO_start_code;
  static const char MOL_start_code;
  static const char MOBL_start_code; 
  static const char MOBL0_start_code;	
  static const char MOBL1_start_code;	
  static const char MOBL2_start_code;	
  static const char MORL_start_code;

  // connected components
  static MPG_QFpm_EXT last_component;
  // triangle tree
  static MPG_QFpm_EXT tt_run_length;
  static MPG_QFpm_EXT tt_leaf;
  static MPG_QFpm_EXT tt_triangulated;
  // triangle data
  static MPG_QFpm_EXT td_marching_pattern[2];
  static MPG_QFpm_EXT td_polygon_edge[2];
  static MPG_QFpm_EXT td_coord_leading_bit[96];
  static MPG_QFpm_EXT td_normal_leading_bit[32];
  static MPG_QFpm_EXT td_color_leading_bit[96];
  static MPG_QFpm_EXT td_texCoord_leading_bit[64];
  // vertex graph
  static MPG_QFpm_EXT vg_simple;
  static MPG_QFpm_EXT vg_last;
  static MPG_QFpm_EXT vg_forward_run;
  static MPG_QFpm_EXT vg_loop_index;
  static MPG_QFpm_EXT vg_run_length;
  static MPG_QFpm_EXT vg_leaf;
  static MPG_QFpm_EXT vg_loop;

  static MPG_QFpm_EXT codap_last_vg_context;
  static MPG_QFpm_EXT codap_branch_len_context;	
  static MPG_QFpm_EXT td_orientation_context;
   
  // stitches
  static MPG_QFpm_EXT stitch_has;
  static MPG_QFpm_EXT stitch_none;
  static MPG_QFpm_EXT stitch_command[2];
  static MPG_QFpm_EXT stitch_length;
  static MPG_QFpm_EXT stitch_length_diff;
  static MPG_QFpm_EXT stitch_depth;
  static MPG_QFpm_EXT stitch_sign;
  static MPG_QFpm_EXT stitch_push;
  static MPG_QFpm_EXT stitch_reverse;

  // pfs
  static MPG_QFpm_EXT pfs_sharp_edges;
  static MPG_QFpm_EXT pfs_fixed_vertices;
  static MPG_QFpm_EXT pfs_forest_edge;
  static MPG_QFpm_EXT pfs_other_update;

  inline ~MPG_Global_EXT()
  {}

  inline MPG_Global_EXT()
  {}

  static void reset();
  static void vg_reset();
  static void sp_reset();
  static void pfs_reset();
  
  static void vg_store();
  static void sp_store();
  static void pfs_store();
  
  static void vg_restore();
  static void sp_restore();
  static void pfs_restore();
};

#endif
