//-------------------------------------------------------------------------
// 
// Copyright (C) IBM Corporation & Samsung Electronics Co. LTD., 1996, 1997, 1998, 1999
// Licensed Materials, Program Property of IBM & Samsung.
// 
// This software module is an implementation of a part of one or more
// MPEG-4 tools as specified by the MPEG-4 standard (ISO/IEC 14496).
// ISO/IEC gives users of the MPEG-4 standard (ISO/IEC 14496) free
// license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4
// standard (ISO/IEC 14496).  Those intending to use this software module
// in hardware or software products are advised that its use may infringe
// existing patents.  The original developer of this software module and
// his/her company, the subsequent editors and their companies, and
// ISO/IEC have no liability for use of this software module or
// modifications thereof in an implementation.  Copyright is not released
// for non MPEG-4 standard (ISO/IEC 14496) conforming products.  IBM
// Corp. & Samsung Electronics Co. LTD retains full right to use the code 
// for his/her own purpose, assign or donate the code to a third party and
// to inhibit third parties from using the code for non MPEG-4 standard 
// (ISO/IEC 14496) conforming products. This copyright notice must be included
// in all copies or derivative works.
// 
//  AUTHORS:
//    Gabriel Taubin
//    Andre Gueziec
//    Claudio Silva
//    Ioana Martin
//  
//  IBM T. J. Watson Research Center
//  P.O.Box 704
//  Yorktown Heights, New York 10598
//  
//  BASED ON EARLIER IMPLEMENTATION OF
//  VRML COMPRESSED BINARY FORMAT PROPOSAL BY:
//    Gabriel Taubin
//    Bill Horn
//    Francis Lazarus
//
//  Functionality : Error Resilience
//
//  Sungjin Kim  (sunjin@sait.samsung.co.kr)
//  Mahn-Jin Han  (manjini@sait.samsung.co.kr)
//  Mun-Sup Song (mssong@sait.samsung.co.kr)  
//
//  Signal Processing Lab. Samsung A.I.T. 
//-------------------------------------------------------------------------


#ifndef _MPG_Graph_EXT_h_
#define _MPG_Graph_EXT_h_

#include <vector>
using namespace std;

class MPG_Graph_EXT {

public:

  class Edge {

    friend MPG_Graph_EXT;

  private:

    int   _v[2];
    int   _indx;
    int   _next;

  public:

    inline     ~Edge();
    inline      Edge(int v0=-1, int v1=-1);
    inline int  getVertex(int i=0);
    inline int  getOtherVertex(int vi);
    inline bool isVertex(int iv);
    inline int  operator[](int i);
    inline int  getIndex();
    inline int  getNext();
  };

  typedef Edge* pEdge;

private:

  bool              _isConst;
  bool              _isOriented;
  vector<int>       _edge;
  vector<int>	    _firstEdge;
  vector<pEdge>	    _oneEdge;
  vector<int>	    _edgeF;
  vector<int>	    _edgeI;

public:

  inline            ~MPG_Graph_EXT();
  inline             MPG_Graph_EXT(int nV, bool isOriented=false);
                     MPG_Graph_EXT(vector<int>& coordIndex);

  inline int         getNumberOfVertices();
  inline int         getNumberOfEdges();
  inline const bool  getIsOriented();
  inline const bool  isConst();
  pEdge              getEdge(int i, int j);
  inline pEdge       getInverseEdge(pEdge e);
  inline bool        hasInverseEdge(pEdge e);
  inline pEdge       getFirstEdge(int i=0);
  inline pEdge       getNextEdge(pEdge e);
  inline pEdge       getOneEdge(int i=0);
  void               insertEdge(int i, int j, int indx=-1);
  inline void        insertEdge(int e[2]);
  inline void        insertFace(int nf, int f[]);
  inline void        insertTriangle(int i, int j, int k);
  inline void        insertTriangle(int t[3]);

  inline void        enumerateEdges();

  inline void        setEdgeFaces(vector<int>& edgeF, vector<int>& edgeI);
  inline int         getNumberOfEdgeFaces(int eIndx);
  inline int         getNumberOfEdgeFaces(Edge& e);
  inline int         getEdgeFace(int eIndx, int i);
  inline int         getEdgeFace(int eIndx);
  inline int         getEdgeFace(Edge& e, int i);
  inline int         getEdgeFace(Edge& e);
  inline int         getOtherEdgeFace(int eIndx, int fi);
  inline int         getOtherEdgeFace(Edge& e, int fi);
  inline bool        isEdgeFace(int eIndx, int fi);
  inline bool        isEdgeFace(Edge& e, int fi);
  inline int         getEdgeVertex(Edge& e, int i);
  inline bool        isEdgeVertex(Edge& e, int vi);

  inline const bool  isUnknownEdge(int eIndx);
  inline const bool  isUnknownEdge(Edge& e);
  inline const bool  isIsolatedEdge(int eIndx);
  inline const bool  isIsolatedEdge(Edge& e);
  inline const bool  isBoundaryEdge(int eIndx);
  inline const bool  isBoundaryEdge(Edge& e);
  inline const bool  isRegularEdge(int eIndx);
  inline const bool  isRegularEdge(Edge& e);
  inline const bool  isManifoldEdge(int eIndx);
  inline const bool  isManifoldEdge(Edge& e);
  inline const bool  isSingularEdge(int eIndx);
  inline const bool  isSingularEdge(Edge& e);

protected:

  inline pEdge      _getPosEdge(int pos);
  inline int        _pushBackEdge(int i, int j, int indx=-1);

public:

         bool        smooth(vector<float>& x,
			    vector<float>& sx,
			    float          a,
			    float          b,
			    int            n,
			    vector<bool>*  e_sharp = NULL,
			    vector<bool>*  v_fixed = NULL);

};

#ifndef __GNUC__
inline void destroy(MPG_Graph_EXT::Edge* pointer) {}
inline void destroy(MPG_Graph_EXT::pEdge* pointer) {}
#endif // __GNUC__

typedef MPG_Graph_EXT* pMPG_Graph_EXT;
#ifndef __GNUC__
inline void destroy(MPG_Graph_EXT* pointer) {}
inline void destroy(pMPG_Graph_EXT* pointer) {}
#endif // __GNUC__

// MPG_Graph_EXT::Edge::~Edge ---------------------------------------------------

inline MPG_Graph_EXT::Edge::~Edge()
{ _v[0]=-1; _v[1]=-1; }

// MPG_Graph_EXT::Edge::Edge ----------------------------------------------------

inline MPG_Graph_EXT::Edge::Edge(int v0, int v1)
{ _v[0]=v0; _v[1]=v1; }

// MPG_Graph_EXT::Edge::getVertex -----------------------------------------------

inline int MPG_Graph_EXT::Edge::getVertex(int i)
{ return (i==0)?_v[0]:_v[1]; }

// MPG_Graph_EXT::Edge::operator[] ----------------------------------------------

inline int MPG_Graph_EXT::Edge::operator[](int i)
{ return getVertex(i); }

// MPG_Graph_EXT::Edge::getIndex ------------------------------------------------

inline int MPG_Graph_EXT::Edge::getIndex()
{ return _indx; }

// MPG_Graph_EXT::Edge::getNext -------------------------------------------------

inline int MPG_Graph_EXT::Edge::getNext()
{ return _next; }

inline int MPG_Graph_EXT::Edge::getOtherVertex(int vi)
{ return (_v[0]==vi)?_v[1]:(_v[1]==vi)?_v[0]:-1; }

inline bool MPG_Graph_EXT::Edge::isVertex(int vi)
{ return (_v[0]==vi||_v[1]==vi)?true:false; }

// MPG_Graph_EXT::~MPG_Graph_EXT ----------------------------------------------------

inline MPG_Graph_EXT::~MPG_Graph_EXT()
{ }

// MPG_Graph_EXT::MPG_Graph_EXT -----------------------------------------------------

inline MPG_Graph_EXT::MPG_Graph_EXT(int nV, bool isOriented)
{
  _isOriented = isOriented;
  _isConst    = false;
  _oneEdge.insert(_oneEdge.end(),(size_t) nV,(pEdge)0);
  _firstEdge.insert(_firstEdge.end(),(size_t) nV,-1);
}

// MPG_Graph_EXT::getNumberOfVertices -------------------------------------------

inline int MPG_Graph_EXT::getNumberOfVertices()
{ return _firstEdge.size(); }

// MPG_Graph_EXT::getNumberOfEdges ----------------------------------------------

inline int MPG_Graph_EXT::getNumberOfEdges()
{ return _edge.size()/4; }

// MPG_Graph_EXT::getIsOriented -------------------------------------------------

inline const bool MPG_Graph_EXT::getIsOriented()
{ return _isOriented; }

// MPG_Graph_EXT::isConst -------------------------------------------------------

inline const bool MPG_Graph_EXT::isConst()
{ return _isConst; }

// MPG_Graph_EXT::getInverseEdge ------------------------------------------------

inline MPG_Graph_EXT::pEdge MPG_Graph_EXT::getInverseEdge(MPG_Graph_EXT::pEdge e)
{
  MPG_Graph_EXT::Edge* eInv = (_isOriented==false)?e:
                          (e!=(pEdge)0       )?getEdge((*e)[1],(*e)[0]):
                                               (pEdge)0;
  return eInv;
}

// MPG_Graph_EXT::hasInverseEdge ------------------------------------------------

inline bool MPG_Graph_EXT::hasInverseEdge(MPG_Graph_EXT::pEdge e)
{ return (getInverseEdge(e)==(pEdge)0)?false:true; }

// MPG_Graph_EXT::getFirstEdge --------------------------------------------------

inline MPG_Graph_EXT::pEdge MPG_Graph_EXT::getFirstEdge(int i)
{
  int pos = _firstEdge[i];
  return (pos<0)?(pEdge)0:_getPosEdge(pos);
}

// MPG_Graph_EXT::getNextEdge ---------------------------------------------------

inline MPG_Graph_EXT::pEdge MPG_Graph_EXT::getNextEdge(pEdge e)
{
  int pos = e->getNext();
  return (pos<0)?(pEdge)0:_getPosEdge(pos);
}

// MPG_Graph_EXT::getOneEdge ----------------------------------------------------

inline MPG_Graph_EXT::pEdge MPG_Graph_EXT::getOneEdge(int i)
{
  return _oneEdge[i];
}

// MPG_Graph_EXT::_getPosEdge ---------------------------------------------------

inline MPG_Graph_EXT::pEdge MPG_Graph_EXT::_getPosEdge(int pos)
{ return (pEdge)(_edge.begin()+pos); }

// MPG_Graph_EXT::_pushBackEdge --------------------------------------------------

inline int MPG_Graph_EXT::_pushBackEdge(int i, int j, int indx)
{
  int pos = -1;
  int nV = getNumberOfVertices();
  if(0<=i&&i<nV&&0<=j&&j<nV&&i!=j)
    {
      pos = _edge.size();
      _edge.push_back( i);
      _edge.push_back( j);
      _edge.push_back(indx);
      _edge.push_back(-1);
    }
  return pos;
}

// MPG_Graph_EXT::insertEdge ----------------------------------------------------

inline void MPG_Graph_EXT::insertEdge(int e[2])
{ if(_isConst==false) insertEdge(e[0],e[1]); }

// MPG_Graph_EXT::insertFace ----------------------------------------------------

inline void MPG_Graph_EXT::insertFace(int nf, int f[])
{
  if(_isConst==false && nf>0)
    for(int i=0;i<nf;i++)
      {
	int j = (i+1)%nf;
	insertEdge(f[i],f[j]);
      }
}

// MPG_Graph_EXT::insertTriangle ------------------------------------------------

inline void MPG_Graph_EXT::insertTriangle(int i, int j, int k)
{
  if(_isConst==false)
    {
      insertEdge(i,j);
      insertEdge(j,k);
      insertEdge(k,i);
    }
}

// MPG_Graph_EXT::insertTriangle ------------------------------------------------

inline void MPG_Graph_EXT::insertTriangle(int t[3])
{
  if(_isConst==false)
    insertTriangle(t[0],t[1],t[2]);
}

// MPG_Graph_EXT::enumerateEdges ------------------------------------------------

inline void MPG_Graph_EXT::enumerateEdges()
{
  if(_isConst==false)
    {
      Edge *e;
      int nV = getNumberOfVertices();
      for(int indx=0,i=0;i<nV;i++)
	for(e=getFirstEdge(i);e!=(Edge*)0;e=getNextEdge(e))
	  e->_indx = indx++;
      _isConst = true;
    }
}

// MPG_Graph_EXT::setEdgeFaces --------------------------------------------------

inline void MPG_Graph_EXT::setEdgeFaces(vector<int>& edgeF, vector<int>& edgeI)
{
  _edgeF = edgeF;
  _edgeI = edgeI;
}

// MPG_Graph_EXT::getNumberOfEdgeFaces ------------------------------------------

inline int MPG_Graph_EXT::getNumberOfEdgeFaces(int eIndx)
{ return _edgeF[eIndx+1]-_edgeF[eIndx]; }

// MPG_Graph_EXT::getNumberOfEdgeFaces ------------------------------------------

inline int MPG_Graph_EXT::getNumberOfEdgeFaces(Edge& e)
{ return getNumberOfEdgeFaces(e.getIndex()); }

// MPG_Graph_EXT::getEdgeFace ---------------------------------------------------

inline int MPG_Graph_EXT::getEdgeFace(int eIndx, int i)
{ return _edgeI[_edgeF[eIndx]+i]; }

// MPG_Graph_EXT::getEdgeFace ---------------------------------------------------

inline int MPG_Graph_EXT::getEdgeFace(int eIndx)
{
  int fi = -1;
  int nf = getNumberOfEdgeFaces(eIndx);
  if(nf==1) nf = 2;
  for(int i=0;i<nf;i++)
    if(_edgeI[_edgeF[eIndx]+i]>=0)
      { fi = _edgeI[_edgeF[eIndx]+i]; break; }
  return fi;
}

// MPG_Graph_EXT::getEdgeFace ---------------------------------------------------

inline int MPG_Graph_EXT::getEdgeFace(Edge& e, int i)
{ return getEdgeFace(e.getIndex(),i); }

// MPG_Graph_EXT::getEdgeFace ---------------------------------------------------

inline int MPG_Graph_EXT::getEdgeFace(Edge& e)
{ return getEdgeFace(e.getIndex()); }

// MPG_Graph_EXT::getOtherEdgeFace ----------------------------------------------

inline int MPG_Graph_EXT::getOtherEdgeFace(int eIndx, int fi)
{
  if(_edgeF[eIndx]+1<_edgeF[eIndx+1])
    for(int j=_edgeF[eIndx];j<_edgeF[eIndx+1];j++)
      if(_edgeI[j]==fi)
	return (j>_edgeF[eIndx])?_edgeI[_edgeF[eIndx]]:_edgeI[_edgeF[eIndx]+1];
  return -1;
}

// MPG_Graph_EXT::getOtherEdgeFace ----------------------------------------------

inline int MPG_Graph_EXT::getOtherEdgeFace(Edge& e, int fi)
{ return getOtherEdgeFace(e.getIndex(),fi); }

// MPG_Graph_EXT::isEdgeFace ----------------------------------------------------

inline bool MPG_Graph_EXT::isEdgeFace(int eIndx, int fi)
{
  bool is_edge_face = false;
  for(int j=_edgeF[eIndx];j<_edgeF[eIndx+1];j++)
    if(_edgeI[j]==fi)
      { is_edge_face=true; break; }
  return is_edge_face;
}

// MPG_Graph_EXT::isEdgeFace ----------------------------------------------------

inline bool MPG_Graph_EXT::isEdgeFace(Edge& e, int fi)
{ return isEdgeFace(e.getIndex(),fi); }

// MPG_Graph_EXT::getEdgeVertex -------------------------------------------------

inline int MPG_Graph_EXT::getEdgeVertex(Edge& e, int i)
{ return e.getVertex(i); }

// MPG_Graph_EXT::isEdgeVertex --------------------------------------------------

inline bool MPG_Graph_EXT::isEdgeVertex(Edge& e, int vi)
{ return e.isVertex(vi); }

// MPG_Graph_EXT::isXXXEdge -----------------------------------------------------

inline const bool MPG_Graph_EXT::isUnknownEdge(int eIndx)
{ return (getNumberOfEdgeFaces(eIndx) <0)?true:false; }

inline const bool MPG_Graph_EXT::isUnknownEdge(Edge& e)
{ return isUnknownEdge(e.getIndex()); }

inline const bool MPG_Graph_EXT::isIsolatedEdge(int eIndx)
{ return (getNumberOfEdgeFaces(eIndx)==0)?true:false; }

inline const bool MPG_Graph_EXT::isIsolatedEdge(Edge& e)
{ return isIsolatedEdge(e.getIndex()); }

inline const bool MPG_Graph_EXT::isBoundaryEdge(int eIndx)
{ return (getNumberOfEdgeFaces(eIndx)==1)?true:false; }

inline const bool MPG_Graph_EXT::isBoundaryEdge(Edge& e)
{ return isBoundaryEdge(e.getIndex()); }

inline const bool MPG_Graph_EXT::isRegularEdge(int eIndx)
{ return (getNumberOfEdgeFaces(eIndx)==2)?true:false; }

inline const bool MPG_Graph_EXT::isRegularEdge(Edge& e)
{ return isRegularEdge(e.getIndex()); }

inline const bool MPG_Graph_EXT::isManifoldEdge(int eIndx)
{ return ((getNumberOfEdgeFaces(eIndx)==1)||
	  (getNumberOfEdgeFaces(eIndx)==2))?true:false; }

inline const bool MPG_Graph_EXT::isManifoldEdge(Edge& e)
{ return isManifoldEdge(e.getIndex()); }

inline const bool MPG_Graph_EXT::isSingularEdge(int eIndx)
{ return (getNumberOfEdgeFaces(eIndx)>=3)?true:false; }

inline const bool MPG_Graph_EXT::isSingularEdge(Edge& e)
{ return isSingularEdge(e.getIndex()); }

#endif // _MPG_Graph_EXT_h_
