//-------------------------------------------------------------------------
// 
// Copyright (C) IBM Corporation & Samsung Electronics Co. LTD., 1996, 1997, 1998, 1999
// Licensed Materials, Program Property of IBM & Samsung.
// 
// This software module is an implementation of a part of one or more
// MPEG-4 tools as specified by the MPEG-4 standard (ISO/IEC 14496).
// ISO/IEC gives users of the MPEG-4 standard (ISO/IEC 14496) free
// license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4
// standard (ISO/IEC 14496).  Those intending to use this software module
// in hardware or software products are advised that its use may infringe
// existing patents.  The original developer of this software module and
// his/her company, the subsequent editors and their companies, and
// ISO/IEC have no liability for use of this software module or
// modifications thereof in an implementation.  Copyright is not released
// for non MPEG-4 standard (ISO/IEC 14496) conforming products.  IBM
// Corp. & Samsung Electronics Co. LTD retains full right to use the code 
// for his/her own purpose, assign or donate the code to a third party and
// to inhibit third parties from using the code for non MPEG-4 standard 
// (ISO/IEC 14496) conforming products. This copyright notice must be included
// in all copies or derivative works.
// 
//  AUTHORS:
//    Gabriel Taubin
//    Andre Gueziec
//    Claudio Silva
//    Ioana Martin
//  
//  IBM T. J. Watson Research Center
//  P.O.Box 704
//  Yorktown Heights, New York 10598
//  
//  BASED ON EARLIER IMPLEMENTATION OF
//  VRML COMPRESSED BINARY FORMAT PROPOSAL BY:
//    Gabriel Taubin
//    Bill Horn
//    Francis Lazarus
//
//  Functionality : Error Resilience
//
//  Sungjin Kim  (sunjin@sait.samsung.co.kr)
//  Mahn-Jin Han  (manjini@sait.samsung.co.kr)
//  Mun-Sup Song (mssong@sait.samsung.co.kr)  
//
//  Signal Processing Lab. Samsung A.I.T. 
//-------------------------------------------------------------------------


#ifndef _MPG_Heap_EXT_h_
#define _MPG_Heap_EXT_h_

#include <vector>
using namespace std;

#ifdef _AIX
inline void destroy(void*) {}
inline void destroy(void**) {}
#endif // _AIX

//
// used in conjunction with MPG_Lex_EXT to sort lexicographically
//
//  given vector<float> p; with p.size()==nP*nD
//
//  int i,j;
//  MPG_Lex_EXT::D = nD;
//  MPG_Heap_EXT h(MPG_Lex_EXT::LT_F,MPG_Lex_EXT::EQ_F,nP);
//  for(i=0;i<nP;i++)
//    h.add((void*)(p.begin()+i*nD));
//
//  float* pj = NULL;
//  for(j=0;j<nP;j++)
//    {
//      i  = h.delMin((void*)pj);
//      // here pj == &(p[nD*i])
//      ...
//
//    }
//

class MPG_Heap_EXT {

public:

  typedef void* Key;
  typedef bool  (*KeyOp)(Key, Key);

protected:

  KeyOp         _LT;
  KeyOp         _EQ;

  int           _last;
  vector<Key>	_key;
  vector<int>	_perm;
  vector<int>	_invp;

protected:

  void _switch(int i, int j);
  void _down(int father);
  void _up();

  void _del(int i)
  {
    if(0<=i && i<_last)
      { _last--; _switch(i,_last); _down(i); }
  }

  int _n()
  { return _perm.size(); }

  int _less(int i, int j)
  {
    // return (_key[i]<_key[j] || (_key[i]==_key[j] && _invp[i]<_invp[j]))?1:0;
    return (_LT(_key[i],_key[j]) ||
	    (_EQ(_key[i],_key[j]) && _invp[i]<_invp[j]))?1:0;
  }

public:

   MPG_Heap_EXT(KeyOp LT, KeyOp EQ, int n=0);

  ~MPG_Heap_EXT()
  { }

  int length()
  { return _n(); }

  void del(int i)
  { if(0<=i && i<_n()) _del(_perm[i]); }

  int delMin()
  {
    if(_last<=0) return -1;
    _del(0);
    return _invp[_last];
  }

  int delMin(Key& key)
  {
    if(_last<=0) return -1;
    _del(0);
    key = _key[_last];
    return _invp[_last];
  }

  int getMin()
  {
    return _invp[0];
  }

  int getMin(Key& key)
  {
    key = _key[_last];
    return _invp[0];
  }

  int add(Key key);
  
  void build()
  {
    int i;
    for(i=0;i<_n();i++)
      _invp[i] = _perm[i] = i;
    for(_last=0;++_last<_n();_up());
  }

  void sort()
  {
    build();
    while(delMin()>=0);
  }
};

#endif

