//-------------------------------------------------------------------------
// 
// Copyright (C) IBM Corporation & Samsung Electronics Co. LTD., 1996, 1997, 1998, 1999
// Licensed Materials, Program Property of IBM & Samsung.
// 
// This software module is an implementation of a part of one or more
// MPEG-4 tools as specified by the MPEG-4 standard (ISO/IEC 14496).
// ISO/IEC gives users of the MPEG-4 standard (ISO/IEC 14496) free
// license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4
// standard (ISO/IEC 14496).  Those intending to use this software module
// in hardware or software products are advised that its use may infringe
// existing patents.  The original developer of this software module and
// his/her company, the subsequent editors and their companies, and
// ISO/IEC have no liability for use of this software module or
// modifications thereof in an implementation.  Copyright is not released
// for non MPEG-4 standard (ISO/IEC 14496) conforming products.  IBM
// Corp. & Samsung Electronics Co. LTD retains full right to use the code 
// for his/her own purpose, assign or donate the code to a third party and
// to inhibit third parties from using the code for non MPEG-4 standard 
// (ISO/IEC 14496) conforming products. This copyright notice must be included
// in all copies or derivative works.
// 
//  AUTHORS:
//    Gabriel Taubin
//    Andre Gueziec
//    Claudio Silva
//    Ioana Martin
//  
//  IBM T. J. Watson Research Center
//  P.O.Box 704
//  Yorktown Heights, New York 10598
//  
//  BASED ON EARLIER IMPLEMENTATION OF
//  VRML COMPRESSED BINARY FORMAT PROPOSAL BY:
//    Gabriel Taubin
//    Bill Horn
//    Francis Lazarus
//
//  Functionality : Error Resilience
//
//  Sungjin Kim  (sunjin@sait.samsung.co.kr)
//  Mahn-Jin Han  (manjini@sait.samsung.co.kr)
//  Mun-Sup Song (mssong@sait.samsung.co.kr)  
//
//  Signal Processing Lab. Samsung A.I.T. 
//-------------------------------------------------------------------------

#ifndef _MPG_Ifs_EXT_h_
#define _MPG_Ifs_EXT_h_

// IndexedFaceSet {
//   eventIn       MFInt32 set_colorIndex
//   eventIn       MFInt32 set_coordIndex
//   eventIn       MFInt32 set_normalIndex
//   eventIn       MFInt32 set_texCoordIndex
//   exposedField  SFNode  color             NULL
//   exposedField  SFNode  coord             NULL
//   exposedField  SFNode  normal            NULL
//   exposedField  SFNode  texCoord          NULL
//   field         SFBool  ccw               TRUE
//   field         MFInt32 colorIndex        []        # [-1,)
//   field         SFBool  colorPerVertex    TRUE
//   field         SFBool  convex            TRUE
//   field         MFInt32 coordIndex        []        # [-1,)
//   field         SFFloat creaseAngle       0         # [ 0,)
//   field         MFInt32 normalIndex       []        # [-1,)
//   field         SFBool  normalPerVertex   TRUE
//   field         SFBool  solid             TRUE
//   field         MFInt32 texCoordIndex     []        # [-1,)
// }

#include <vector>
#include "MPG_Stream.h"
#include "MPG_Property.h"

using namespace std;

#define ORDER_MODE 0
#define ADAPTIVE_QUANT_TEXCOORD_MODE 1
#define ESCAPE_MODE 15

class MPG_Ifs_EXT {

protected:

  bool                       _ccw;
  bool                       _convex;
  float                      _creaseAngle;
  bool                       _solid;
  bool                       _3DMC_extension;
  int						vertex_order_flag;
  int						face_order_flag;
  int						vertex_order_per_CC_flag;
  int						face_order_per_CC_flag;

  vector<float>              _coord;
  vector<int>                _coordIndex;

  bool                       _normalPerVertex;
  vector<float>              _normal;
  vector<int>                _normalIndex;

  bool                       _colorPerVertex;
  vector<float>              _color;
  vector<int>                _colorIndex;

  vector<float>              _texCoord;
  vector<int>                _texCoordIndex;

  vector<int>			qVertexIndex;
  vector<int>			VertexIndex;
  vector<int>			qFaceIndex;
  vector<int>			FaceIndex;
  //dmlab 2006.6.26
  vector<int>			vertexStitch;
  int					numOfvertexStitching;
  vector<int>			stitchChangeList;
  vector<int>			stitchDellist;

public:

  inline                     ~MPG_Ifs_EXT();
  inline                      MPG_Ifs_EXT();

  inline void                 clear();

  inline void                 setCcw(bool ccw);
  inline void                 setConvex(bool convex);
  inline void                 setCreaseAngle(float creaseAngle);
  inline void                 setSolid(bool solid);

  inline bool                 getCcw();
  inline bool                 getConvex();
  inline float                getCreaseAngle();
  inline bool                 getSolid();

  inline vector<float>&       getCoord();
  inline vector<int>&         getCoordIndex();
  inline MPG_Binding          getCoordBinding();
  inline int                  getNumberOfCoord();

  inline void                 setNormalPerVertex(bool normalPerVertex);
  inline bool                 getNormalPerVertex();
  inline vector<float>&       getNormal();
  inline vector<int>&         getNormalIndex();
  inline MPG_Binding          getNormalBinding();
  inline int                  getNumberOfNormal();

  inline void                 setColorPerVertex(bool colorPerVertex);
  inline bool                 getColorPerVertex();
  inline vector<float>&       getColor();
  inline vector<int>&         getColorIndex();
  inline MPG_Binding          getColorBinding();
  inline int                  getNumberOfColor();

  inline vector<float>&       getTexCoord();
  inline vector<int>&         getTexCoordIndex();
  inline MPG_Binding          getTexCoordBinding();
  inline int                  getNumberOfTexCoord();
  void							stitchAnalysis();

public:

         bool                 read(char* fname);
#if !defined _3DMCEXT_CONFORMANCE_
         bool                 write(char* fname);
#else
		 bool				  write(char* fileName, int defID = -1);
#endif _3DMCEXT_CONFORMANCE_

		 bool                 writeOrders(char* fname);

		 //MJH 000922 : added for library generation
	     bool                 read(FILE* fp); 

};

inline MPG_Ifs_EXT::~MPG_Ifs_EXT()
{}

inline MPG_Ifs_EXT::MPG_Ifs_EXT():
  _ccw(true),
  _convex(true),
  _creaseAngle(0),
  _solid(true),
  _3DMC_extension(false),
  _normalPerVertex(true),
  _colorPerVertex(true)
{
}

inline void MPG_Ifs_EXT::clear()
{
  _ccw             = true;
  _convex          = true;
  _creaseAngle     = 0.0;
  _solid           = true;
  _3DMC_extension        = false;

  _normalPerVertex = true;
  _colorPerVertex  = true;

  _coord.erase(_coord.begin(),_coord.end());
  _coordIndex.erase(_coordIndex.begin(),_coordIndex.end());
  _normal.erase(_normal.begin(),_normal.end());
  _normalIndex.erase(_normalIndex.begin(),_normalIndex.end());
  _color.erase(_color.begin(),_color.end());
  _colorIndex.erase(_colorIndex.begin(),_colorIndex.end());
  _texCoord.erase(_texCoord.begin(),_texCoord.end());
  _texCoordIndex.erase(_texCoordIndex.begin(),_texCoordIndex.end());
}

inline void MPG_Ifs_EXT::setCcw(bool ccw)
{ _ccw = ccw; }

inline void MPG_Ifs_EXT::setConvex(bool convex)
{ _convex = convex; }

inline void MPG_Ifs_EXT::setCreaseAngle(float creaseAngle)
{ _creaseAngle = creaseAngle; }

inline void MPG_Ifs_EXT::setSolid(bool solid)
{ _solid = solid; }

inline bool MPG_Ifs_EXT::getCcw()
{ return _ccw; }

inline bool MPG_Ifs_EXT::getConvex()
{ return _convex; }

inline float MPG_Ifs_EXT::getCreaseAngle()
{ return _creaseAngle; }

inline bool MPG_Ifs_EXT::getSolid()
{ return _solid; }

inline vector<float>& MPG_Ifs_EXT::getCoord()
{ return _coord; }

inline vector<int>& MPG_Ifs_EXT::getCoordIndex()
{ return _coordIndex; }

inline void MPG_Ifs_EXT::setNormalPerVertex(bool normalPerVertex)
{ _normalPerVertex = normalPerVertex; }

inline bool MPG_Ifs_EXT::getNormalPerVertex()
{ return _normalPerVertex; }

inline vector<float>& MPG_Ifs_EXT::getNormal()
{ return _normal; }

inline vector<int>& MPG_Ifs_EXT::getNormalIndex()
{ return _normalIndex; }

inline void MPG_Ifs_EXT::setColorPerVertex(bool colorPerVertex)
{ _colorPerVertex = colorPerVertex; }

inline bool MPG_Ifs_EXT::getColorPerVertex()
{ return _colorPerVertex; }

inline vector<float>& MPG_Ifs_EXT::getColor()
{ return _color; }

inline vector<int>& MPG_Ifs_EXT::getColorIndex()
{ return _colorIndex; }

inline vector<float>& MPG_Ifs_EXT::getTexCoord()
{ return _texCoord; }

inline vector<int>& MPG_Ifs_EXT::getTexCoordIndex()
{ return _texCoordIndex; }

inline MPG_Binding MPG_Ifs_EXT::getCoordBinding()
{ return PB_PER_VERTEX; }

inline MPG_Binding MPG_Ifs_EXT::getNormalBinding()
{
  // if(normal.size()==0)
  //   NO_NORMALS
  // else if(normalPerVertex==FALSE)
  //   if(normalIndex.size()>0)
  //     NORMAL_PER_FACE_INDEXED;
  //     normalIndex.size()==getNumberOfFaces()
  //   else
  //     NORMAL_PER_FACE;
  //     normal.size()/3==getNumberOfFaces()
  // else // if(normalPerVertex==TRUE)
  //   if(normalIndex.size()>0)
  //     NORMAL_PER_CORNER_INDEXED;
  //     normalIndex.size()==coordIndex.size()
  //   else
  //     NORMAL_PER_VERTEX;
  //     normal.size()/3==coord.size()/3

  return (_normal.size()==0      )?PB_NONE:
         (_normalPerVertex==false)?PB_PER_FACE:
         (_normalIndex.size()>0  )?PB_PER_CORNER:
                                   PB_PER_VERTEX;
}

inline MPG_Binding MPG_Ifs_EXT::getColorBinding()
{
  // if(color.size()==0)
  //   NO_COLORS
  // else if(colorPerVertex==FALSE)
  //   if(colorIndex.size()>0)
  //     COLOR_PER_FACE_INDEXED;
  //     colorIndex.size()==getNumberOfFaces()
  //   else
  //     COLOR_PER_FACE;
  //     color.size()/3==getNumberOfFaces()
  // else // if(colorPerVertex==TRUE)
  //   if(colorIndex.size()>0)
  //     COLOR_PER_CORNER_INDEXED;
  //     colorIndex.size()==coordIndex.size()
  //   else
  //     COLOR_PER_VERTEX;
  //     color.size()/3==coord.size()/3

  return (_color.size()==0      )?PB_NONE:
         (_colorPerVertex==false)?PB_PER_FACE:
         (_colorIndex.size()>0  )?PB_PER_CORNER:
                                  PB_PER_VERTEX;
}

inline MPG_Binding MPG_Ifs_EXT::getTexCoordBinding()
{
  // if(texCoord.size()==0)
  //   NO_TEX_COORD
  // else if(texCoordIndex.size()>0)
  //   TEX_COORD_PER_CORNER_INDEXED;
  //   texCoordIndex.size()==coordIndex.size()
  // else
  //   TEX_COORD_PER_VERTEX;
  //   texCoord.size()/3==coord.size()/3

  return (_texCoord.size()==0    )?PB_NONE:
         (_texCoordIndex.size()>0)?PB_PER_CORNER:
                                   PB_PER_VERTEX;
}

inline int MPG_Ifs_EXT::getNumberOfCoord()
{ return _coord.size()/3; }

inline int MPG_Ifs_EXT::getNumberOfNormal()
{ return _normal.size()/3; }

inline int MPG_Ifs_EXT::getNumberOfColor()
{ return _color.size()/3; }

inline int MPG_Ifs_EXT::getNumberOfTexCoord()
{ return _texCoord.size()/2; }

#endif // _MPG_Ifs_EXT_h_

