//-------------------------------------------------------------------------
// 
// Copyright (C) IBM Corporation & Samsung Electronics Co. LTD., 1996, 1997, 1998, 1999
// Licensed Materials, Program Property of IBM & Samsung.
// 
// This software module is an implementation of a part of one or more
// MPEG-4 tools as specified by the MPEG-4 standard (ISO/IEC 14496).
// ISO/IEC gives users of the MPEG-4 standard (ISO/IEC 14496) free
// license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4
// standard (ISO/IEC 14496).  Those intending to use this software module
// in hardware or software products are advised that its use may infringe
// existing patents.  The original developer of this software module and
// his/her company, the subsequent editors and their companies, and
// ISO/IEC have no liability for use of this software module or
// modifications thereof in an implementation.  Copyright is not released
// for non MPEG-4 standard (ISO/IEC 14496) conforming products.  IBM
// Corp. & Samsung Electronics Co. LTD retains full right to use the code 
// for his/her own purpose, assign or donate the code to a third party and
// to inhibit third parties from using the code for non MPEG-4 standard 
// (ISO/IEC 14496) conforming products. This copyright notice must be included
// in all copies or derivative works.
// 
//  AUTHORS:
//    Gabriel Taubin
//    Andre Gueziec
//    Claudio Silva
//    Ioana Martin
//  
//  IBM T. J. Watson Research Center
//  P.O.Box 704
//  Yorktown Heights, New York 10598
//  
//  BASED ON EARLIER IMPLEMENTATION OF
//  VRML COMPRESSED BINARY FORMAT PROPOSAL BY:
//    Gabriel Taubin
//    Bill Horn
//    Francis Lazarus
//
//  Functionality : Error Resilience
//
//  Sungjin Kim  (sunjin@sait.samsung.co.kr)
//  Mahn-Jin Han  (manjini@sait.samsung.co.kr)
//  Mun-Sup Song (mssong@sait.samsung.co.kr)  
//
//  Signal Processing Lab. Samsung A.I.T. 
//-------------------------------------------------------------------------


#ifndef _MPG_IfsEdit_EXT_h_
#define _MPG_IfsEdit_EXT_h_

#include "MPG_IfsMesh.h"
#include "MPG_MeshEdit.h"

class MPG_IfsEdit_EXT {

protected:

  MPG_IfsMesh_EXT& _ifsMesh;

protected:

  // used after MPG_Mesh_EXT::_cut() to fix the property arrays

         bool            _duplicatePerVertexProperties(vector<int>& v_stitch);

public:

  inline                  MPG_IfsEdit_EXT(MPG_IfsMesh_EXT& ifsMesh);

  inline void             setFaceCorner(int i, int j, int fij);

  // cut through marked edges
  inline bool             cutThroughMarkedEdges(vector<bool>& e_mark,
					       vector<int>& v_stitch);
  // cut through singular and non-oriented regular edges, and singular vertices
  inline bool             convertToOrientedManifold(vector<int>& v_stitch);
  // cut through singular edges, and singular vertices
  inline bool             convertToManifold(vector<int>& v_stitch);

         bool             removeIsolatedVertices();
};

inline MPG_IfsEdit_EXT::MPG_IfsEdit_EXT(MPG_IfsMesh_EXT& ifsMesh):
  _ifsMesh(ifsMesh)
{}

inline void MPG_IfsEdit_EXT::setFaceCorner(int i, int j, int fij)
{
  int* fi = _ifsMesh.getFace(i);
  fi[j] = fij;
 }

inline bool MPG_IfsEdit_EXT::cutThroughMarkedEdges(vector<bool>& e_mark,
					       vector<int>& v_stitch)
{
  int nVbefore = _ifsMesh.getNumberOfVertices();
  MPG_MeshEdit_EXT& mesh = (MPG_MeshEdit_EXT&)_ifsMesh;
  if(mesh.cutThroughMarkedEdges(e_mark,v_stitch)==false) return false;
  int nVafter = _ifsMesh.getNumberOfVertices();
  if(nVafter>nVbefore)
    {
      // recompute edges and conected components
      _ifsMesh.reset();
      // duplicate PER_VERTEX properties
      if(_duplicatePerVertexProperties(v_stitch)==false) return false;
    }
  return true;
}

inline bool MPG_IfsEdit_EXT::convertToOrientedManifold(vector<int>& v_stitch)
{
  int nVbefore = _ifsMesh.getNumberOfVertices();
  printf("vertices number before increasing vertices : %d\n", nVbefore);
  MPG_MeshEdit_EXT& mesh = (MPG_MeshEdit_EXT&)_ifsMesh;
  if(mesh.convertToOrientedManifold(v_stitch)==false) return false;
  int nVafter = _ifsMesh.getNumberOfVertices();
  if(nVafter>nVbefore)
    {
      // recompute edges and conected components
      _ifsMesh.reset();
      // duplicate PER_VERTEX properties
      if(_duplicatePerVertexProperties(v_stitch)==false) return false;
    }
  return true;
}

inline bool MPG_IfsEdit_EXT::convertToManifold(vector<int>& v_stitch)
{
  int nVbefore = _ifsMesh.getNumberOfVertices();
  MPG_MeshEdit_EXT& mesh = (MPG_MeshEdit_EXT&)_ifsMesh;
  if(mesh.convertToManifold(v_stitch)==false) return false;
  int nVafter = _ifsMesh.getNumberOfVertices();
  if(nVafter>nVbefore)
    {
      // recompute edges and conected components
      _ifsMesh.reset();
      // duplicate PER_VERTEX properties
      if(_duplicatePerVertexProperties(v_stitch)==false) return false;
    }
  return true;
}

#endif // _MPG_Mesh_EXT_h_
