//-------------------------------------------------------------------------
// 
// Copyright (C) IBM Corporation & Samsung Electronics Co. LTD., 1996, 1997, 1998, 1999
// Licensed Materials, Program Property of IBM & Samsung.
// 
// This software module is an implementation of a part of one or more
// MPEG-4 tools as specified by the MPEG-4 standard (ISO/IEC 14496).
// ISO/IEC gives users of the MPEG-4 standard (ISO/IEC 14496) free
// license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4
// standard (ISO/IEC 14496).  Those intending to use this software module
// in hardware or software products are advised that its use may infringe
// existing patents.  The original developer of this software module and
// his/her company, the subsequent editors and their companies, and
// ISO/IEC have no liability for use of this software module or
// modifications thereof in an implementation.  Copyright is not released
// for non MPEG-4 standard (ISO/IEC 14496) conforming products.  IBM
// Corp. & Samsung Electronics Co. LTD retains full right to use the code 
// for his/her own purpose, assign or donate the code to a third party and
// to inhibit third parties from using the code for non MPEG-4 standard 
// (ISO/IEC 14496) conforming products. This copyright notice must be included
// in all copies or derivative works.
// 
//  AUTHORS:
//    Gabriel Taubin
//    Andre Gueziec
//    Claudio Silva
//    Ioana Martin
//  
//  IBM T. J. Watson Research Center
//  P.O.Box 704
//  Yorktown Heights, New York 10598
//  
//  BASED ON EARLIER IMPLEMENTATION OF
//  VRML COMPRESSED BINARY FORMAT PROPOSAL BY:
//    Gabriel Taubin
//    Bill Horn
//    Francis Lazarus
//
//  Functionality : Error Resilience
//
//  Sungjin Kim  (sunjin@sait.samsung.co.kr)
//  Mahn-Jin Han  (manjini@sait.samsung.co.kr)
//  Mun-Sup Song (mssong@sait.samsung.co.kr)  
//
//  Signal Processing Lab. Samsung A.I.T. 
//-------------------------------------------------------------------------

#ifndef _MPG_IfsQ_EXT_h_
#define _MPG_IfsQ_EXT_h_

#include "MPG_Ifs.h"
#include "MPG_Property.h"
#include "MPG_BBoxQ.h"
#include "MPG_NormalQ.h"

class MPG_IfsQ_EXT : public MPG_Ifs_EXT {

protected:
  MPG_BBoxQ_EXT*                 _bbqCoord;
  MPG_NormalQ_EXT*               _nrqNormal;
  MPG_BBoxQ_EXT*                 _bbqColor;
  MPG_BBoxQ_EXT*                 _bbqTexCoord;

public:

  inline                     ~MPG_IfsQ_EXT();
  inline                      MPG_IfsQ_EXT();

  inline void                 clear();

  inline MPG_BBoxQ_EXT*           getCoordQuantizer(int bitsPerCoord=0);
  inline void                 setCoordQuantizer(MPG_BBoxQ_EXT* bbq);

  inline MPG_NormalQ_EXT*         getNormalQuantizer(int bitsPerNormal=0);
  inline void                 setNormalQuantizer(MPG_NormalQ_EXT* nrq);

  inline MPG_BBoxQ_EXT*           getColorQuantizer(int bitsPerColor=0);
  inline void                 setColorQuantizer(MPG_BBoxQ_EXT* bbq);

  inline MPG_BBoxQ_EXT*           getTexCoordQuantizer(int bitsPerCoord=0);
  inline void                 setTexCoordQuantizer(MPG_BBoxQ_EXT* bbq);
};

inline MPG_IfsQ_EXT::~MPG_IfsQ_EXT()
{}

inline MPG_IfsQ_EXT::MPG_IfsQ_EXT()
{
  _bbqCoord    = (MPG_BBoxQ_EXT*)0;
  _nrqNormal   = (MPG_NormalQ_EXT*)0;
  _bbqColor    = (MPG_BBoxQ_EXT*)0;
  _bbqTexCoord = (MPG_BBoxQ_EXT*)0;
}

inline void MPG_IfsQ_EXT::clear()
{
  MPG_Ifs_EXT::clear();
  if(_bbqCoord!=(MPG_BBoxQ_EXT*)0)
    { delete _bbqCoord;    _bbqCoord   = ((MPG_BBoxQ_EXT*)0);   }
  if(_nrqNormal!=(MPG_NormalQ_EXT*)0)
    { delete _nrqNormal;   _nrqNormal  = ((MPG_NormalQ_EXT*)0); }
  if(_bbqColor!=(MPG_BBoxQ_EXT*)0)
    { delete _bbqColor;    _bbqColor   = ((MPG_BBoxQ_EXT*)0);   }
  if(_bbqTexCoord!=(MPG_BBoxQ_EXT*)0)
    { delete _bbqTexCoord; _bbqTexCoord= ((MPG_BBoxQ_EXT*)0);   }
}

inline MPG_BBoxQ_EXT* MPG_IfsQ_EXT::getCoordQuantizer(int bitsPerCoord)
{
  if(bitsPerCoord>0)
    {
      if(_bbqCoord==(MPG_BBoxQ_EXT*)0)
	{
	  _bbqCoord = new MPG_BBoxQ_EXT(3,bitsPerCoord);
	  _bbqCoord->setBBox(_coord);
	}
      else if(_bbqCoord->getBitsPerCoordinate()!=bitsPerCoord)
	_bbqCoord->setBitsPerCoordinate(bitsPerCoord);
    }
  // else just return current quantizer
  return _bbqCoord;
}

inline MPG_NormalQ_EXT* MPG_IfsQ_EXT::getNormalQuantizer(int bitsPerNormal)
{
  if(bitsPerNormal>0)
    {
      if(_nrqNormal==(MPG_NormalQ_EXT*)0)
	_nrqNormal   = new MPG_NormalQ_EXT(bitsPerNormal);
      else if(_nrqNormal->getBitsPerNormal()!=bitsPerNormal)
	_nrqNormal->setBitsPerNormal(bitsPerNormal);
    }
  // else just return current quantizer
  return _nrqNormal;
}

inline MPG_BBoxQ_EXT* MPG_IfsQ_EXT::getColorQuantizer(int bitsPerColor)
{
  if(bitsPerColor>0)
    {
      if(_bbqColor==(MPG_BBoxQ_EXT*)0)
	{
	  _bbqColor = new MPG_BBoxQ_EXT(3,bitsPerColor);
	  _bbqColor->setBBox(_color);
	}
      else if(_bbqColor->getBitsPerCoordinate()!=bitsPerColor)
	_bbqColor->setBitsPerCoordinate(bitsPerColor);
    }
  // else just return current quantizer
  return _bbqColor;
}

inline MPG_BBoxQ_EXT* MPG_IfsQ_EXT::getTexCoordQuantizer(int bitsPerTexCoord)
{
  if(bitsPerTexCoord>0)
    {
      if(_bbqTexCoord==(MPG_BBoxQ_EXT*)0)
	{
	  _bbqTexCoord = new MPG_BBoxQ_EXT(2,bitsPerTexCoord);
	  _bbqTexCoord->setBBox(_texCoord);
	}
      else if(_bbqTexCoord->getBitsPerCoordinate()!=bitsPerTexCoord)
	_bbqTexCoord->setBitsPerCoordinate(bitsPerTexCoord);
    }
  // else just return current quantizer
  return _bbqTexCoord;
}

inline void MPG_IfsQ_EXT::setCoordQuantizer(MPG_BBoxQ_EXT* bbq)
{
  // if current quantizer is no longer referenced to elsewhere,
  // it should be deleted before replacing it with this
  // method
  // if(_bbqCoord==(MPG_BBoxQ_EXT*)0) delete _bbqCoord;
  _bbqCoord = bbq;
}

inline void MPG_IfsQ_EXT::setNormalQuantizer(MPG_NormalQ_EXT* nrq)
{
  // if current quantizer is no longer referenced to elsewhere,
  // it should be deleted before replacing it with this
  // method
  // if(_bbqNormal==(MPG_BBoxQ_EXT*)0) delete _bbqNormal;
  _nrqNormal = nrq;
}

inline void MPG_IfsQ_EXT::setColorQuantizer(MPG_BBoxQ_EXT* bbq)
{
  // if current quantizer is no longer referenced to elsewhere,
  // it should be deleted before replacing it with this
  // method
  // if(_bbqColor==(MPG_BBoxQ_EXT*)0) delete _bbqColor;
  _bbqColor = bbq;
}

inline void MPG_IfsQ_EXT::setTexCoordQuantizer(MPG_BBoxQ_EXT* bbq)
{
  // if current quantizer is no longer referenced to elsewhere,
  // it should be deleted before replacing it with this
  // method
  // if(_bbqTexCoord==(MPG_BBoxQ_EXT*)0) delete _bbqTexCoord;
  _bbqTexCoord = bbq;
}

#endif // _MPG_IfsQ_EXT_h_

