//-------------------------------------------------------------------------
//  
//  Copyright (C) IBM Corporation, 1996, 1997, 1998, 1999
//  Licensed Materials, Program Property of IBM
//  
//  This software module is an implementation of a part of one or more
//  MPEG-4 tools as specified by the MPEG-4 standard (ISO/IEC 14492).
//  ISO/IEC gives users of the MPEG-4 standard free license to this
//  software module or modifications thereof for use in hardware or
//  software products claiming conformance to the MPEG-4 standard.  Those
//  intending to use this software module in hardware or software products
//  are advised that its use may infringe existing patents.  The original
//  developer of this software module and his/her company, the subsequent
//  editors and their companies, and ISO/IEC have no liability for use of
//  this software module or modifications thereof in an implementation.
//  Copyright is not released for non MPEG-4 conforming products.  IBM
//  Corp. retains full right to use the code for his/her own purpose,
//  assign or donate the code to a third party and to inhibit third
//  parties from using the code for non MPEG-4 conforming products.  This
//  copyright notice must be included in all copies or derivative works.
//  
//  AUTHORS:
//    Gabriel Taubin
//    Andre Gueziec
//    Claudio Silva
//    Ioana Martin
//  
//  IBM T. J. Watson Research Center
//  P.O.Box 704
//  Yorktown Heights, New York 10598
//  
//  BASED ON EARLIER IMPLEMENTATION OF
//  VRML COMPRESSED BINARY FORMAT PROPOSAL BY:
//    Gabriel Taubin
//    Bill Horn
//    Francis Lazarus
// 
//-------------------------------------------------------------------------

#ifndef _MPG_IfsQ_E_EXT_h_
#define _MPG_IfsQ_E_EXT_h_

#include "MPG_IfsQ.h"

#ifdef _SINGLE_RESOLUTION_PROGRESSIVE

#endif // _SINGLE_RESOLUTION_PROGRESSIVE

class MPG_IfsQ_E_EXT : public MPG_IfsQ_EXT {

public:

	inline      ~MPG_IfsQ_E_EXT();
	inline       MPG_IfsQ_E_EXT();

	// functions to expand input stream

	bool  expand(MPG_InputStream_EXT& istrm, bool saveWrlLevels);
	
#ifdef _SINGLE_RESOLUTION_PROGRESSIVE	
	// expand just the header 
	bool  expandHeader(MPG_InputStream_EXT& istrm, MPG_State_EXT& state,
		MPG_Property_EXT& pCoord, MPG_Property_EXT& pNormal,
		MPG_Property_EXT& pColor, MPG_Property_EXT& pTexCoord,
		vector<int>& qCoord, vector<int>& qNormal,
		vector<int>& qColor, vector<int>& qTexCoord,
		vector<int>& v_visited, vector<int>& v_cluster,
		MPG_EXT_Stitcher& stitcher, vector<int>& v_father,
		MPG_EXT_Stitches& v_stitches, vector<int>& stk);
	
	// expand the remaining stream (after header was expanded)
	bool  expandRest(MPG_InputStream_EXT& istrm,
		MPG_Property_EXT& pCoord, MPG_Property_EXT& pNormal,
		MPG_Property_EXT& pColor, MPG_Property_EXT& pTexCoord,
		vector<int>& qCoord, vector<int>& qNormal,
		vector<int>& qColor, vector<int>& qTexCoord,
		vector<int>& v_visited, vector<int>& v_cluster, 
		MPG_EXT_Stitcher& stitcher, vector<int>& v_father,
		MPG_EXT_Stitches& v_stitches, vector<int>& stk,
		int& nV0, int& nV1, int& nVcls0, int& nVcls);

	// expand stream one connected component at a time
	bool  expandCC(MPG_InputStream_EXT& istrm, 
		MPG_Property_EXT& pCoord, MPG_Property_EXT& pNormal,
		MPG_Property_EXT& pColor, MPG_Property_EXT& pTexCoord,
		vector<int>& qCoord, vector<int>& qNormal,
		vector<int>& qColor, vector<int>& qTexCoord,
		vector<int>& v_visited, vector<int>& v_cluster, 
		MPG_EXT_Stitcher& stitcher, vector<int>& v_father,
		MPG_EXT_Stitches& v_stitches, vector<int>& stk,
		int& nV0, int& nV1, int& nVcls0, int& nVcls1,
		bool& newMol, bool& newMobl, bool& newCC,
		int& mol_id, MPG_QFac_EXT& codec, 
		bool& expandDone);
	
	// expand stream n triangles at a time
	bool expandTriangles(MPG_InputStream_EXT& istrm, MPG_State_EXT& state, 
		MPG_Property_EXT& pCoord, MPG_Property_EXT& pNormal,
		MPG_Property_EXT& pColor, MPG_Property_EXT& pTexCoord,
		vector<int>& qCoord, vector<int>& qNormal,
		vector<int>& qColor, vector<int>& qTexCoord,
		vector<int>& v_visited, vector<int>& v_cluster,  
		MPG_EXT_Stitcher& stitcher, vector<int>& v_father,
		MPG_EXT_Stitches& v_stitches, 
		MPG_QFac_EXT& codec, 
		vector<int>&   coordIndex,
		vector<float>& coord,
		vector<float>& normal,
		vector<float>& color,
		vector<float>& texCoord,
		vector<int>& f_father,
		vector<int>& c_traversal,
		vector<int>& c_father,
		vector<int>& run_father,
		vector<int>& run_length,
		vector<bool>& run_ends_in_leaf,
		vector<int>& stk, 
		vector<int>& faceStack,
		MPG_SPolygon_EXT& sp, MPG_VGraph_EXT& vg,
		vector<int>& v_loop_cc, 
		MPG_Loops_EXT& loop, MPG_Loops_EXT& c_loop,
		bool& expandDone);
#endif // _SINGLE_RESOLUTION_PROGRESSIVE
};

inline MPG_IfsQ_E_EXT::~MPG_IfsQ_E_EXT()
{}

inline MPG_IfsQ_E_EXT::MPG_IfsQ_E_EXT():MPG_IfsQ_EXT()
{}

#endif // _MPG_IfsQ_E_EXT_h_
