//-------------------------------------------------------------------------
// 
// Copyright (C) IBM Corporation & Samsung Electronics Co. LTD., 1996, 1997, 1998, 1999
// Licensed Materials, Program Property of IBM & Samsung.
// 
// This software module is an implementation of a part of one or more
// MPEG-4 tools as specified by the MPEG-4 standard (ISO/IEC 14496).
// ISO/IEC gives users of the MPEG-4 standard (ISO/IEC 14496) free
// license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4
// standard (ISO/IEC 14496).  Those intending to use this software module
// in hardware or software products are advised that its use may infringe
// existing patents.  The original developer of this software module and
// his/her company, the subsequent editors and their companies, and
// ISO/IEC have no liability for use of this software module or
// modifications thereof in an implementation.  Copyright is not released
// for non MPEG-4 standard (ISO/IEC 14496) conforming products.  IBM
// Corp. & Samsung Electronics Co. LTD retains full right to use the code 
// for his/her own purpose, assign or donate the code to a third party and
// to inhibit third parties from using the code for non MPEG-4 standard 
// (ISO/IEC 14496) conforming products. This copyright notice must be included
// in all copies or derivative works.
// 
//  AUTHORS:
//    Gabriel Taubin
//    Andre Gueziec
//    Claudio Silva
//    Ioana Martin
//  
//  IBM T. J. Watson Research Center
//  P.O.Box 704
//  Yorktown Heights, New York 10598
//  
//  BASED ON EARLIER IMPLEMENTATION OF
//  VRML COMPRESSED BINARY FORMAT PROPOSAL BY:
//    Gabriel Taubin
//    Bill Horn
//    Francis Lazarus
//
//  Functionality : Error Resilience
//
//  Sungjin Kim  (sunjin@sait.samsung.co.kr)
//  Mahn-Jin Han  (manjini@sait.samsung.co.kr)
//  Mun-Sup Song (mssong@sait.samsung.co.kr)  
//
//  Signal Processing Lab. Samsung A.I.T. 
//-------------------------------------------------------------------------

#ifndef _MPG_MeshTs_h_
#define _MPG_MeshTs_h_

#include "MPG_IfsMesh_EXT.h"

class MPG_MeshTs : public MPG_Mesh {

private:

  vector<bool>           _is_vtree_edge;
  vector<bool>           _is_ftree_edge;

public:

  inline                  MPG_MeshTs(vector<int>& coordIndex);
  inline                 ~MPG_MeshTs();

  inline void             reset();
  inline bool             isInVtree(int ie);
  inline bool             isInVtree(MPG_Graph::Edge *e);
  inline bool             isInFtree(int ie);
  inline bool             isInFtree(MPG_Graph::Edge *e);
  inline bool             isJumpEdge(int ie);
  inline bool             isJumpEdge(MPG_Graph::Edge *e);
  inline int              getNumberOfVtreeEdges();
  inline int              getNumberOfFtreeEdges();
  inline int              getNumberOfJumpEdges();
  inline vector<bool>&    getVtree();
  inline vector<bool>&    getFtree();

protected:

         bool            _constructTrees();

public:

         bool representConnectivity(vector<MPG_pSPolygon>& vpsp,
                                    vector<MPG_pVGraph>&   vpvg,
                                    MPG_Property&          pCoord,
                                    MPG_Property&          pNormal,
                                    MPG_Property&          pColor,
                                    MPG_Property&          pTexCoord,
                                    vector<int>&           qCoord,
                                    vector<int>&           qNormal,
                                    vector<int>&           qColor,
                                    vector<int>&           qTexCoord,
                                    vector<int>&           v_visited,
                                    vector<int>&           v_cluster,
                                    vector<int>&           v_traversal,
                                    vector<int>&           v_father,
                                    vector<int>&           f_traversal,
									vector<int>&           f_index,
									vector<int>&           c_index,
									MPG_BranchForest*	    root_btf,
									int*				   f_array_cnt
									);
};

inline MPG_MeshTs::MPG_MeshTs(vector<int>& coordIndex):
  MPG_Mesh(coordIndex)
{
  int nE = getNumberOfEdges();
  _is_vtree_edge.insert(_is_vtree_edge.end(),(size_t)nE,false);
  _is_ftree_edge.insert(_is_ftree_edge.end(),(size_t)nE,false);
  _constructTrees();
}

inline void MPG_MeshTs::reset()
{
  MPG_Mesh::reset();
  int nE = getNumberOfEdges();
  _is_vtree_edge.insert(_is_vtree_edge.end(),(size_t)nE,false);
  _is_ftree_edge.insert(_is_ftree_edge.end(),(size_t)nE,false);
  _constructTrees();
}

inline MPG_MeshTs::~MPG_MeshTs()
{}

inline bool MPG_MeshTs::isInVtree(int ie)
{ return _is_vtree_edge[ie]; }

inline bool MPG_MeshTs::isInVtree(MPG_Graph::Edge *e)
{ return (e!=(MPG_Graph::Edge*)0)?isInVtree(e->getIndex()):false; }

inline bool MPG_MeshTs::isInFtree(int ie)
{ return _is_ftree_edge[ie]; }

inline bool MPG_MeshTs::isInFtree(MPG_Graph::Edge *e)
{ return (e!=(MPG_Graph::Edge*)0)?isInFtree(e->getIndex()):false; }

inline bool MPG_MeshTs::isJumpEdge(int ie)
  { return (isInVtree(ie)==false&&isInFtree(ie)==false)?true:false; }

inline bool MPG_MeshTs::isJumpEdge(MPG_Graph::Edge *e)
{ return (e!=(MPG_Graph::Edge*)0)?isJumpEdge(e->getIndex()):false; }

inline int MPG_MeshTs::getNumberOfVtreeEdges()
{
  int nE_vtree = 0;
  int i;
  for(i=0;i<_is_vtree_edge.size();i++)
    if(_is_vtree_edge[i] == true)
      nE_vtree++;
  return nE_vtree;
}

inline int MPG_MeshTs::getNumberOfFtreeEdges()
{
  int nE_ftree = 0;
  int i;
  for(i=0;i<_is_ftree_edge.size();i++)
    if(_is_ftree_edge[i] == true)
      nE_ftree++;
  return nE_ftree;
}

inline int MPG_MeshTs::getNumberOfJumpEdges()
{
  int nE_jump =
    getNumberOfEdges()-getNumberOfVtreeEdges()-getNumberOfFtreeEdges();
  return(nE_jump);
}

inline vector<bool>& MPG_MeshTs::getVtree()
{ return _is_vtree_edge; }

inline vector<bool>& MPG_MeshTs::getFtree()
{ return _is_ftree_edge; }

#endif // _MPG_MeshTs_h_
