//-------------------------------------------------------------------------
// 
// Copyright (C) IBM Corporation & Samsung Electronics Co. LTD., 1996, 1997, 1998, 1999
// Licensed Materials, Program Property of IBM & Samsung.
// 
// This software module is an implementation of a part of one or more
// MPEG-4 tools as specified by the MPEG-4 standard (ISO/IEC 14496).
// ISO/IEC gives users of the MPEG-4 standard (ISO/IEC 14496) free
// license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4
// standard (ISO/IEC 14496).  Those intending to use this software module
// in hardware or software products are advised that its use may infringe
// existing patents.  The original developer of this software module and
// his/her company, the subsequent editors and their companies, and
// ISO/IEC have no liability for use of this software module or
// modifications thereof in an implementation.  Copyright is not released
// for non MPEG-4 standard (ISO/IEC 14496) conforming products.  IBM
// Corp. & Samsung Electronics Co. LTD retains full right to use the code 
// for his/her own purpose, assign or donate the code to a third party and
// to inhibit third parties from using the code for non MPEG-4 standard 
// (ISO/IEC 14496) conforming products. This copyright notice must be included
// in all copies or derivative works.
// 
//  AUTHORS:
//    Gabriel Taubin
//    Andre Gueziec
//    Claudio Silva
//    Ioana Martin
//  
//  IBM T. J. Watson Research Center
//  P.O.Box 704
//  Yorktown Heights, New York 10598
//  
//  BASED ON EARLIER IMPLEMENTATION OF
//  VRML COMPRESSED BINARY FORMAT PROPOSAL BY:
//    Gabriel Taubin
//    Bill Horn
//    Francis Lazarus
//
//  Functionality : Error Resilience
//
//  Sungjin Kim  (sunjin@sait.samsung.co.kr)
//  Mahn-Jin Han  (manjini@sait.samsung.co.kr)
//  Mun-Sup Song (mssong@sait.samsung.co.kr)  
//
//  Signal Processing Lab. Samsung A.I.T. 
//-------------------------------------------------------------------------


#ifndef _MPG_NormalQ_EXT_h_
#define _MPG_NormalQ_EXT_h_

#include <vector>

using namespace std;

class MPG_NormalQ_EXT {

protected:

  int           _nSubdivisionLevels;
  int           _nBitsPerNormal;      // 3+2*_nSubdivisionLevels
  float         _avgError;
  float         _maxError;

private:

  unsigned long _normal2Code(const float *normal);
  void          _code2Normal(const unsigned long code, float *normal);

public:

  inline       ~MPG_NormalQ_EXT();
  inline        MPG_NormalQ_EXT(int nBitsPerNormal=9);

  inline int    getBitsPerNormal();
  inline void   setBitsPerNormal(int bitsPerNormal);
  inline void   clear();

         bool   quantize(float* value, int* iqValue, int nValues=1);
         bool   quantize(vector<float>& value, vector<int>& iqValue);
         bool   quantize(vector<float>& n, vector<int>& indx, vector<int>& iqn);
	     bool   unQuantize(int* iqValue, float* value, int nValues=1);
         bool   unQuantize(vector<int>& iqValue, vector<float>& value);

  inline float  getAvgError();
  inline float  getMaxError();
};

inline MPG_NormalQ_EXT::MPG_NormalQ_EXT(int nBitsPerNormal)
{
  _nSubdivisionLevels = (nBitsPerNormal-3)/2;
  _nBitsPerNormal     = (_nSubdivisionLevels>=0)?3+2*_nSubdivisionLevels:3;
  _avgError           = 0.0;
  _maxError           = 0.0;
}

inline MPG_NormalQ_EXT::~MPG_NormalQ_EXT()
{}

inline void MPG_NormalQ_EXT::clear()
{ _nBitsPerNormal = 0; _nSubdivisionLevels = 0; }

inline int    MPG_NormalQ_EXT::getBitsPerNormal()
{ return _nBitsPerNormal; }

inline void   MPG_NormalQ_EXT::setBitsPerNormal(int bitsPerNormal)
{
  _nBitsPerNormal     = bitsPerNormal;
  _nSubdivisionLevels = (_nBitsPerNormal-3)/2;
}

// should be called after MPG_NormalQ_EXT::quantize()
// which computes this value
inline float  MPG_NormalQ_EXT::getAvgError()
{ return _avgError; }

// should be called after MPG_NormalQ_EXT::quantize()
// which computes this value
inline float  MPG_NormalQ_EXT::getMaxError()
{ return _maxError; }

#endif // _MPG_NormalQ_EXT_h_

