//-------------------------------------------------------------------------
// 
// Copyright (C) IBM Corporation & Samsung Electronics Co. LTD., 1996, 1997, 1998, 1999
// Licensed Materials, Program Property of IBM & Samsung.
// 
// This software module is an implementation of a part of one or more
// MPEG-4 tools as specified by the MPEG-4 standard (ISO/IEC 14496).
// ISO/IEC gives users of the MPEG-4 standard (ISO/IEC 14496) free
// license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4
// standard (ISO/IEC 14496).  Those intending to use this software module
// in hardware or software products are advised that its use may infringe
// existing patents.  The original developer of this software module and
// his/her company, the subsequent editors and their companies, and
// ISO/IEC have no liability for use of this software module or
// modifications thereof in an implementation.  Copyright is not released
// for non MPEG-4 standard (ISO/IEC 14496) conforming products.  IBM
// Corp. & Samsung Electronics Co. LTD retains full right to use the code 
// for his/her own purpose, assign or donate the code to a third party and
// to inhibit third parties from using the code for non MPEG-4 standard 
// (ISO/IEC 14496) conforming products. This copyright notice must be included
// in all copies or derivative works.
// 
//  AUTHORS:
//    Gabriel Taubin
//    Andre Gueziec
//    Claudio Silva
//    Ioana Martin
//  
//  IBM T. J. Watson Research Center
//  P.O.Box 704
//  Yorktown Heights, New York 10598
//  
//  BASED ON EARLIER IMPLEMENTATION OF
//  VRML COMPRESSED BINARY FORMAT PROPOSAL BY:
//    Gabriel Taubin
//    Bill Horn
//    Francis Lazarus
//
//  Functionality : Error Resilience
//
//  Sungjin Kim  (sunjin@sait.samsung.co.kr)
//  Mahn-Jin Han  (manjini@sait.samsung.co.kr)
//  Mun-Sup Song (mssong@sait.samsung.co.kr)  
//
//  Signal Processing Lab. Samsung A.I.T. 
//-------------------------------------------------------------------------


#ifndef _MPG_QFpm_EXT_H_
#define _MPG_QFpm_EXT_H_

#include <math.h>

struct MPG_QEntry_EXT {
  unsigned short Qe;
  short          nMPS;
  short          nLPS;
  short          sMPS;
};

class MPG_QFpm_EXT {
public:
  static struct MPG_QEntry_EXT model[];
  static int _shift;

  unsigned       Qe;
  short          index;
  short          MPS;
  unsigned       preQe;
  short		   preindex;
  short		   preMPS;

  inline ~MPG_QFpm_EXT()
  {}

  inline MPG_QFpm_EXT()
  { reset(); }

  inline void reset()
  { index = 0; MPS = 0; Qe = model[index].Qe << _shift; }

  void update(int);

  inline void store() {
	  preindex = index;
	  preMPS = MPS;
	  preQe = Qe;
  }
  
  inline void restore() {
	  index = preindex;
	  MPS = preMPS;
	  Qe = preQe;
  }
};

#endif
