//-------------------------------------------------------------------------
//  
//  Copyright (C) IBM Corporation, 1996, 1997, 1998, 1999
//  Licensed Materials, Program Property of IBM
//  
//  This software module is an implementation of a part of one or more
//  MPEG-4 tools as specified by the MPEG-4 standard (ISO/IEC 14492).
//  ISO/IEC gives users of the MPEG-4 standard free license to this
//  software module or modifications thereof for use in hardware or
//  software products claiming conformance to the MPEG-4 standard.  Those
//  intending to use this software module in hardware or software products
//  are advised that its use may infringe existing patents.  The original
//  developer of this software module and his/her company, the subsequent
//  editors and their companies, and ISO/IEC have no liability for use of
//  this software module or modifications thereof in an implementation.
//  Copyright is not released for non MPEG-4 conforming products.  IBM
//  Corp. retains full right to use the code for his/her own purpose,
//  assign or donate the code to a third party and to inhibit third
//  parties from using the code for non MPEG-4 conforming products.  This
//  copyright notice must be included in all copies or derivative works.
//  
//  AUTHORS:
//    Gabriel Taubin
//    Andre Gueziec
//    Claudio Silva
//    Ioana Martin
//  
//  IBM T. J. Watson Research Center
//  P.O.Box 704
//  Yorktown Heights, New York 10598
//  
//  BASED ON EARLIER IMPLEMENTATION OF
//  VRML COMPRESSED BINARY FORMAT PROPOSAL BY:
//    Gabriel Taubin
//    Bill Horn
//    Francis Lazarus
// 
//-------------------------------------------------------------------------

#ifndef _MPG_SPolygon_EXT_h_
#define _MPG_SPolygon_EXT_h_

#include "MPG_Sample.h"
#include "MPG_Loops.h"
#include "MPG_Property.h"
#include "MPG_ER.h"      

#define ALREADY_CODED	0x01
#define AVAIL_PRED		0X02
#define AVAIL_RECON		0x04

class MPG_SPolygon_EXT : public MPG_Sample_EXT {

protected:

  vector<int>  _tri;

public:

  ~MPG_SPolygon_EXT() {}
   MPG_SPolygon_EXT() {}

  enum TriType {
    sp_LEAF   = 0,
    sp_LEFT   = 1,
    sp_RIGHT  = 2,
    sp_BRANCH = 3,
    sp_UNDET  = 4
  };

	MPG_ER_EXT tdirect;
	float tt_avgbits;
	int codap_left_bloop_idx;	
	int codap_right_bloop_idx; 
	int codap_bdry_pred;
	int codap_branch_len;
	int ntriangles;
	int poly_part;	

	inline vector<int>& getTriData() { return _tri; } 
	inline void Calc_partition_pos(int tri, int iS, int iRun,
		                          int ldepth, int td_orientation,
								  int sp_pt2_enable,
								  int &p_branch, int &p_leaf,
								  int &p_run, int &p_tri);

	inline bool Calc_Overflow(int tri,
								int iT,
								 int ldepth,
								 int td_orientation,
								 int p_run, 
								 int p_tri,
								 int tdbits,
								 int cbits,
								 int availbits, 
								 int &sub_bits,
								 int &overflow, 
								 int &next_pt,
								 int &v_update,
								 int &sub_ptcount,
								 int &return_tri,
								 int &return_run,
								 MPG_PacketQ_C_EXT &codap);
	
	inline bool updateData(int iS, int iRun, int iL, int iR, int iP, 
							int tri, int td_orientation, 
							int iF, int iC, 
							int cR, int cL,	//MJH 001013 : added in case of partitioning at non-polygon edge
							int gdepth,
							int ldepth, int next_pt,
							MPG_PacketQ_C_EXT& codap,
							int &v_update,
							vector<int>& Gistk, 
							vector<int>& Grstk, 
							int &GiF, int &GiC, int &Gdepth,
							int &prevCR, int &prevCL	//MJH 001013 : added in case of partitioning at non-polygon edge
							);

  inline void      clear();
  inline const int getNumberOfTriangles();
  inline const int getNumberOfVertices();
  inline const int getNumberOfBoundaryEdges();
  inline const int getNumberOfInternalEdges();

  inline const int getNumberOfRuns();
  inline const int getNumberOfLeafNodes();
  inline const int getNumberOfBranchNodes();

  inline void      setTri(int i, TriType t, bool pBit=true);
  inline void      pushTri(TriType t, bool pBit=true);
  inline void      setTri(int i, int ttpb);
  inline void      pushTri(int ttpb);
  inline int       getTri(int i);
  inline void	   popTri();	         
  inline void      setPolyBit(int i, bool pBit=true);
  inline int       getPolyBit(int i);
  inline int       getTriPolyBit(int i);
  inline int       get(int i, TriType& t, bool& pBit);
  inline bool      hasPBits();
  inline bool      hasDirectPBits(MPG_PacketQ_C_EXT codap);   

  // compress methods
  void      DetermineDirection();                      
  bool      encodeTriangleTree(MPG_QFac_EXT     &codec, 
							vector<int>&   Grstk, 
							int            &Gdepth,   
							MPG_PacketQ_C_EXT& codap
							);
  
  bool      encodeTriangleData(MPG_QFac_EXT&      codec,
							int            nV0,
							vector<int>&   v_loop,
							vector<int>&   v_visited,
							vector<int>&   v_cluster,
							MPG_Property_EXT&  pCoord,
							MPG_Property_EXT&  pNormal,
							MPG_Property_EXT&  pColor,
							MPG_Property_EXT&  pTexCoord,
							vector<int>&           qCoord,
							vector<int>&           qNormal,
							vector<int>&           qColor,
							vector<int>&           qTexCoord,
							vector<int>&           v_org_idx,  
							vector<int>&           f_index,	
							vector<int>&           c_index,	
							MPG_BranchTree_EXT*&	   cur_bt,	
							vector<int>&           Grstk,
							vector<int>&           Gistk,
							int                    &Gdepth,
							int					   &GiC,
							int					   &GiF,
							int					   &prevCR, //MJH 001013 : added in case of partitioning at non-polygon edge
							int					   &prevCL,	//MJH 001013 : added in case of partitioning at non-polygon edge
							int                    availbits, 
							MPG_PacketQ_C_EXT&         codap,
// DMLAB (200503)
							int*					TCbits
//~DMLAB (200503)

							);

         bool      encodeTriangleTree(MPG_QFac_EXT&      codec);

         bool      encodeTriangleData(MPG_QFac_EXT&      codec,
				      int            nV0,
				      vector<int>&   v_loop,
				      vector<int>&   v_visited,
				      vector<int>&   v_cluster,
				      MPG_Property_EXT&  pCoord,
				      MPG_Property_EXT&  pNormal,
				      MPG_Property_EXT&  pColor,
				      MPG_Property_EXT&  pTexCoord,
				      int*&          tData);
  // expand methods

         bool      decodeTriangleTree(MPG_QFac_EXT& codec,
						int partition_type=0, int v_loop_cc_size=0); 

         bool      decodeTriangleData(MPG_QFac_EXT&      codec,
				      int            nVtree,
				      int            nV0,
				      vector<int>&   v_loop,
				      vector<int>&   v_visited,
				      vector<int>&   v_cluster,
				      vector<int>&   coordIndex,
				      MPG_Property_EXT&  pCoord,
				      MPG_Property_EXT&  pNormal,
				      MPG_Property_EXT&  pColor,
				      MPG_Property_EXT&  pTexCoord,
				      vector <int>&  qCoord,
				      vector <int>&  qNormal,
				      vector <int>&  qColor,
				      vector <int>&  qTexCoord,
#ifdef MPGELIB
//MJH 082100 : used for initialization of face and corner tree array at the start of each IFS
					  bool&			 newIFS,		
#endif
					  int			 partition_type=0   
					  );

#ifdef _SINGLE_RESOLUTION_PROGRESSIVE
  // progressive expand methods

		 bool      decodeSingleTriangle(int iT, int& iRun,
					  MPG_QFac_EXT&      codec,
				      int&           nVtree,
				      int&           nV0,
					  int&		     nVloop,
					  MPG_Loops_EXT&	 loop,
					  MPG_Loops_EXT&	 c_loop,
				      vector<int>&   v_loop_cc,
				      vector<int>&   v_visited,
				      vector<int>&   v_cluster,
				      vector<int>&   coordIndex,
				      MPG_Property_EXT&  pCoord,
				      MPG_Property_EXT&  pNormal,
				      MPG_Property_EXT&  pColor,
				      MPG_Property_EXT&  pTexCoord,
				      vector <int>&  qCoord,
				      vector <int>&  qNormal,
				      vector <int>&  qColor,
				      vector <int>&  qTexCoord,
					  int& prev_marching_pattern, 
					  int& prev_polygon_edge, 
					  int& triangulated,
					  vector<int>& f_father,
					  vector<int>& c_traversal,
					  vector<int>& c_father,
					  vector<int>& run_father,
					  vector<int>& run_length,
					  vector<bool>& run_ends_in_leaf,
					  vector<int>& stk,
					  int& nPE, int& nF, int& nC,
					  int& nCoord, int& nNormal, int& nColor, int& nTexCoord,
					  int& iP, int& iR, int& iL, int& iO,
					  int& iR1, int& iL1, int& iO1,
					  vector<int>& faceStack, int& iF, int& iFp, int& iC,
					  int& cRp, int& cR, int& cLp, int& cL, int& cO);

		 bool      preprocessTriangleData(
					  MPG_QFac_EXT&      codec,
				      int            nVtree,
				      int            nV0,
				      vector<int>&   v_loop_cc,
				      vector<int>&   v_visited,
				      vector<int>&   v_cluster,
				      vector<int>&   coordIndex,
				      MPG_Property_EXT&  pCoord,
				      MPG_Property_EXT&  pNormal,
				      MPG_Property_EXT&  pColor,
				      MPG_Property_EXT&  pTexCoord,
				      vector <int>&  qCoord,
				      vector <int>&  qNormal,
				      vector <int>&  qColor,
				      vector <int>&  qTexCoord,
					  int& prev_marching_pattern, 
					  int& prev_polygon_edge, 
					  int& triangulated,
					  vector<int>& f_father,
					  vector<int>& c_traversal,
					  vector<int>& c_father,
					  vector<int>& run_father,
					  vector<int>& run_length,
					  vector<bool>& run_ends_in_leaf,
					  vector<int>& stk,
					  int& nPE, int& nF, int& nC,
					  int& nCoord, int& nNormal, int& nColor, int nTexCoord,
					  int& iP, int& iR, int& iL, int& iO,
					  vector<int>& faceStack, int& iF, int& iFp, int& iC,
					  int& cRp, int& cR, int& cLp, int& cL, int& cO);

		 bool		postprocessTriangleData(
					  MPG_Loops_EXT&	 loop,
					  MPG_Loops_EXT&	 c_loop,
					  vector<int>&   v_loop_sp,
					  vector<int>&   coordIndex,
					  MPG_Property_EXT&  pCoord,
					  MPG_Property_EXT&  pNormal,
					  MPG_Property_EXT&  pColor,
					  MPG_Property_EXT&  pTexCoord,
					  vector <int>&  qCoord,
					  vector <int>&  qNormal,
					  vector <int>&  qColor,
					  vector <int>&  qTexCoord,
					  int& nPE, int& nF, int& nC);
#endif // _SINGLE_RESOLUTION_PROGRESSIVE

};

typedef MPG_SPolygon_EXT* MPG_EXT_pSPolygon;
#ifndef __GNUC__
// to make the compiler happy
inline void destroy(MPG_SPolygon_EXT* pointer) {}
inline void destroy(MPG_EXT_pSPolygon* pointer) {}
#endif // __GNUC__

inline void MPG_SPolygon_EXT::clear()
{ _tri.erase(_tri.begin(),_tri.end()); }

inline const int MPG_SPolygon_EXT::getNumberOfTriangles()
{ return _tri.size(); }

inline const int MPG_SPolygon_EXT::getNumberOfVertices()
{ return (getNumberOfTriangles()>0)?getNumberOfTriangles()+2:0; }

inline const int MPG_SPolygon_EXT::getNumberOfBoundaryEdges()
{ return getNumberOfVertices(); }

inline const int MPG_SPolygon_EXT::getNumberOfInternalEdges()
{ return (getNumberOfTriangles()>0)?getNumberOfTriangles()-1:0; }

inline const int MPG_SPolygon_EXT::getNumberOfRuns()
{
  int n_runs = 0;
  vector<int>::iterator t;
  for(t=_tri.begin();t!=_tri.end();t++)
    if(((*t)&0x7)==sp_LEAF || ((*t)&0x7)==sp_BRANCH)
      n_runs++;
  return n_runs;
}

inline const int MPG_SPolygon_EXT::getNumberOfLeafNodes()
{
  int n_leaf = 0;
  vector<int>::iterator t;
  for(t=_tri.begin();t!=_tri.end();t++)
    if(((*t)&0x7)==sp_LEAF)
      n_leaf++;
  return n_leaf;
}

inline const int MPG_SPolygon_EXT::getNumberOfBranchNodes()
{
  int n_branch = 0;
  vector<int>::iterator t;
  for(t=_tri.begin();t!=_tri.end();t++)
    if(((*t)&0x7)==sp_BRANCH)
      n_branch++;
  return n_branch;
}

inline void MPG_SPolygon_EXT::setTri(int i, TriType tt, bool pBit)
{ _tri[i] = (pBit==true)?(0x8|(tt&0x7)):(tt&0x7); }

inline void MPG_SPolygon_EXT::pushTri(TriType tt, bool pBit)
{
  int t = (pBit==true)?(0x8|(tt&0x7)):(tt&0x7);
 _tri.push_back(t);
}

inline void MPG_SPolygon_EXT::setTri(int i, int ttpb)
{ _tri[i] = (ttpb&0x7); }

inline void MPG_SPolygon_EXT::pushTri(int ttpb)
{ _tri.push_back(ttpb&0x7); }

inline void MPG_SPolygon_EXT::popTri()	
{ _tri.pop_back(); }

inline int MPG_SPolygon_EXT::getTri(int i)
{ return (0<=i&&i<_tri.size())?(_tri[i]&0x7):-1; }

inline void MPG_SPolygon_EXT::setPolyBit(int i, bool pBit)
{
  if(pBit)
    _tri[i] |= 0x8;
  else
    _tri[i] &= 0x7;
}

inline int MPG_SPolygon_EXT::getPolyBit(int i)
{ return (0<=i&&i<_tri.size())?(_tri[i]&0x8)?1:0:-1; }

inline int MPG_SPolygon_EXT::getTriPolyBit(int i)
{ return (0<=i&&i<_tri.size())?(_tri[i]&0x7):-1; }

inline int MPG_SPolygon_EXT::get(int i, TriType& tt, bool& pBit)
{
  if(0<=i&&i<_tri.size())
    {
      tt   = (TriType)(_tri[i]&0x7);
      pBit = (_tri[i]&0x8)?1:0;
      return 0;
    }
  return -1;
}

inline bool MPG_SPolygon_EXT::hasPBits()
{
  bool hasPBits = false;
  for(int i=0;i<_tri.size();i++)
    if(getPolyBit(i)==0)
      { hasPBits = true; break; }
  return hasPBits;
}

inline bool MPG_SPolygon_EXT::hasDirectPBits(MPG_PacketQ_C_EXT codap)  
{
	bool hasPBits = false;

	int end = 0;
	int i = codap.start_tri;
	int j = codap.start_run;
	int depth=0;
	vector<int>  stk((size_t)1, -1);
	do {
		if(getPolyBit(i)==0) {
			hasPBits = true; 
			break; 
		}
		switch(getTri(i)) {
		case sp_LEAF: 
			if(depth==1) {
				j = stk.back();
				stk.pop_back();
				i = tdirect.getStart(j);
			} else {
				i++;
				j++;
			};
			depth--; 
			break;
		case sp_BRANCH: 
			if(depth==0) { 
				if(codap.sp_pt2_enable) { 
					stk.push_back(tdirect.getSecond(j));
					j = tdirect.getFirst(j);
				} else {                 //pt0
					if(tdirect.getDirect(j)) {  
						stk.push_back(tdirect.getSecond(j));
						j = tdirect.getFirst(j);
					} else {
						stk.push_back(tdirect.getFirst(j));
						j = tdirect.getSecond(j);
					}
				}
				i = tdirect.getStart(j);
			} else { 
				i++;
				j++;
			};
			depth++; 
			break;
		default: 
			i++;
			break;
		}

		if(!codap.sp_pass0) {
			if(i==codap.next_tri) 
				end=1;  
		} else {
			if(depth==-1) 
				end=1;
		}
	} while(!end);  

	return hasPBits;
}

#endif // _MPG_SPolygon_EXT_h_

