//-------------------------------------------------------------------------
// 
// Copyright (C) IBM Corporation & Samsung Electronics Co. LTD., 1996, 1997, 1998, 1999
// Licensed Materials, Program Property of IBM & Samsung.
// 
// This software module is an implementation of a part of one or more
// MPEG-4 tools as specified by the MPEG-4 standard (ISO/IEC 14496).
// ISO/IEC gives users of the MPEG-4 standard (ISO/IEC 14496) free
// license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4
// standard (ISO/IEC 14496).  Those intending to use this software module
// in hardware or software products are advised that its use may infringe
// existing patents.  The original developer of this software module and
// his/her company, the subsequent editors and their companies, and
// ISO/IEC have no liability for use of this software module or
// modifications thereof in an implementation.  Copyright is not released
// for non MPEG-4 standard (ISO/IEC 14496) conforming products.  IBM
// Corp. & Samsung Electronics Co. LTD retains full right to use the code 
// for his/her own purpose, assign or donate the code to a third party and
// to inhibit third parties from using the code for non MPEG-4 standard 
// (ISO/IEC 14496) conforming products. This copyright notice must be included
// in all copies or derivative works.
// 
//  AUTHORS:
//    Gabriel Taubin
//    Andre Gueziec
//    Claudio Silva
//    Ioana Martin
//  
//  IBM T. J. Watson Research Center
//  P.O.Box 704
//  Yorktown Heights, New York 10598
//  
//  BASED ON EARLIER IMPLEMENTATION OF
//  VRML COMPRESSED BINARY FORMAT PROPOSAL BY:
//    Gabriel Taubin
//    Bill Horn
//    Francis Lazarus
//
//  Functionality : Error Resilience
//
//  Sungjin Kim  (sunjin@sait.samsung.co.kr)
//  Mahn-Jin Han  (manjini@sait.samsung.co.kr)
//  Mun-Sup Song (mssong@sait.samsung.co.kr)  
//
//  Signal Processing Lab. Samsung A.I.T. 
//-------------------------------------------------------------------------

#ifndef _MPG_Sample_EXT_h_
#define _MPG_Sample_EXT_h_

#include <vector>
#include "MPG_Stream.h"
#include "MPG_QFac.h"
using namespace std;

class MPG_Sample_EXT {

protected:
  
  static void
  _encodeRootSample(MPG_QFac_EXT& codec, int bitsPerX, int* iX, int nD);
  
public:

  inline static void
  encodeRootCoordSample(MPG_QFac_EXT& codec, int bitsPerCoord,
			int* iCoord);
  
  inline static void
  encodeRootNormalSample(MPG_QFac_EXT& codec, int bitsPerNormal,
			 int* iNormal);
  
  inline static void
  encodeRootColorSample(MPG_QFac_EXT& codec, int bitsPerColor,
			int* iColor);
  
  inline static void
  encodeRootTexCoordSample(MPG_QFac_EXT& codec, int bitsPerTexCoord,
			   int* iTexCoord);

protected:
  
  static void
  _encodeSample(MPG_QFac_EXT& codec, int bitsPerX, int* iX, int nD, int p);
  
public:

  inline static void
  encodeCoordSample(MPG_QFac_EXT& codec, int bitsPerCoord,
		    int* iCoord);
  
  inline static void
  encodeNormalSample(MPG_QFac_EXT& codec, int bitsPerNormal,
		     int* iNormal);
  
  inline static void
  encodeColorSample(MPG_QFac_EXT& codec, int bitsPerColor,
		    int* iColor);
  
  inline static void
  encodeTexCoordSample(MPG_QFac_EXT& codec, int bitsPerTexCoord,
		       int* iTexCoord);

  inline static void
  encodeCoordSamples(MPG_QFac_EXT& codec, int bitsPerCoord,
			 vector<int> iCoord);
  
  inline static void
  encodeNormalSamples(MPG_QFac_EXT& codec, int bitsPerNormal,
			  vector<int> iNormal);
  
  inline static void
  encodeColorSamples(MPG_QFac_EXT& codec, int bitsPerColor,
			 vector<int> iColor);
  
  inline static void
  encodeTexCoordSamples(MPG_QFac_EXT& codec, int bitsPerTexCoord,
			    vector<int> iTexCoord);

protected:

  static void
  _decodeRootSample(MPG_QFac_EXT& codec, int bitsPerX, int* iX, int nD);
  
public:

  inline static void
  decodeRootCoordSample(MPG_QFac_EXT& codec, int bitsPerCoord,
			int* iCoord);
  
  inline static void
  decodeRootNormalSample(MPG_QFac_EXT& codec, int bitsPerNormal,
			 int* iNormal);
  
  inline static void
  decodeRootColorSample(MPG_QFac_EXT& codec, int bitsPerColor,
			int* iColor);
  
  inline static void
  decodeRootTexCoordSample(MPG_QFac_EXT& codec, int bitsPerTexCoord,
			   int* iTexCoord);
  
protected:
  
  static void
  _decodeSample(MPG_QFac_EXT& codec, int bitsPerX, int* iX, int nD, int p);

public:

  inline static void
  decodeCoordSample(MPG_QFac_EXT& codec, int bitsPerCoord,
		    int* iCoord);
  
  inline static void
  decodeNormalSample(MPG_QFac_EXT& codec, int bitsPerNormal,
		     int* iNormal);
  
  inline static void
  decodeColorSample(MPG_QFac_EXT& codec, int bitsPerColor,
		    int* iColor);
  
  inline static void
  decodeTexCoordSample(MPG_QFac_EXT& codec, int bitsPerTexCoord,
		       int* iTexCoord);

  inline static void
  decodeCoordSamples(MPG_QFac_EXT& codec, int bitsPerCoord,
		     vector<int>& iCoord, int n=0);
  
  inline static void
  decodeNormalSamples(MPG_QFac_EXT& codec, int bitsPerNormal,
		      vector<int>& iNormal, int n=0);
  
  inline static void
  decodeColorSamples(MPG_QFac_EXT& codec, int bitsPerColor,
		     vector<int>& iColor, int n=0);
  
  inline static void
  decodeTexCoordSamples(MPG_QFac_EXT& codec, int bitsPerTexCoord,
			vector<int>& iTexCoord, int n=0);
  
};

inline void MPG_Sample_EXT::
encodeRootCoordSample(MPG_QFac_EXT& codec, int bitsPerCoord, int* iCoord)
{ _encodeRootSample(codec, bitsPerCoord, iCoord, 3); }

inline void MPG_Sample_EXT::
encodeRootColorSample(MPG_QFac_EXT& codec, int bitsPerColor, int* iColor)
{ _encodeRootSample(codec, bitsPerColor, iColor, 3); }

inline void MPG_Sample_EXT::
encodeRootNormalSample(MPG_QFac_EXT& codec, int bitsPerNormal, int* iNormal)
{ _encodeRootSample(codec, bitsPerNormal, iNormal, 1); }

inline void MPG_Sample_EXT::
encodeRootTexCoordSample(MPG_QFac_EXT& codec, int bitsPerTexCoord, int* iTexCoord)
{ _encodeRootSample(codec, bitsPerTexCoord, iTexCoord, 2); }

inline void MPG_Sample_EXT::
encodeCoordSample(MPG_QFac_EXT& codec, int bitsPerCoord, int* iCoord)
{ _encodeSample(codec, bitsPerCoord, iCoord, 3, 0); }

inline void MPG_Sample_EXT::
encodeColorSample(MPG_QFac_EXT& codec, int bitsPerColor, int* iColor)
{ _encodeSample(codec, bitsPerColor, iColor, 3, 2); }

inline void MPG_Sample_EXT::
encodeNormalSample(MPG_QFac_EXT& codec, int bitsPerNormal, int* iNormal)
{ _encodeSample(codec, bitsPerNormal, iNormal, 1, 1); }

inline void MPG_Sample_EXT::
encodeTexCoordSample(MPG_QFac_EXT& codec, int bitsPerTexCoord, int* iTexCoord)
{ _encodeSample(codec, bitsPerTexCoord, iTexCoord, 2, 3); }

inline void MPG_Sample_EXT::
encodeCoordSamples(MPG_QFac_EXT& codec, int bitsPerCoord,
		   vector<int> iCoord)
{
  int n = iCoord.size()/3;
  int i;
  for(i=0;i<n;i++)
    encodeCoordSample(codec,bitsPerCoord, iCoord.begin()+3*i);
}

inline void MPG_Sample_EXT::
encodeColorSamples(MPG_QFac_EXT& codec, int bitsPerColor,
		   vector<int> iColor)
{
  int n = iColor.size()/3;
  int i;
  for(i=0;i<n;i++)
    encodeColorSample(codec, bitsPerColor, iColor.begin()+3*i);
}

inline void MPG_Sample_EXT::
encodeNormalSamples(MPG_QFac_EXT& codec, int bitsPerNormal,
		    vector<int> iNormal)
{
  int n = iNormal.size();
  int i;
  for(i=0;i<n;i++)
    encodeNormalSample(codec, bitsPerNormal, iNormal.begin()+i);
}

inline void MPG_Sample_EXT::
encodeTexCoordSamples(MPG_QFac_EXT& codec, int bitsPerTexCoord,
		      vector<int> iTexCoord)
{
  int n = iTexCoord.size()/2;
  int i;
  for(i=0;i<n;i++)
    encodeTexCoordSample(codec, bitsPerTexCoord, iTexCoord.begin()+2*i);
}

inline void MPG_Sample_EXT::
decodeRootCoordSample(MPG_QFac_EXT& codec, int bitsPerCoord, int* iCoord)
{ _decodeRootSample(codec, bitsPerCoord, iCoord, 3); }

inline void MPG_Sample_EXT::
decodeRootColorSample(MPG_QFac_EXT& codec, int bitsPerColor, int* iColor)
{ _decodeRootSample(codec, bitsPerColor, iColor, 3); }

inline void MPG_Sample_EXT::
decodeRootNormalSample(MPG_QFac_EXT& codec, int bitsPerNormal, int* iNormal)
{ _decodeRootSample(codec, bitsPerNormal, iNormal, 1); }

inline void MPG_Sample_EXT::
decodeRootTexCoordSample(MPG_QFac_EXT& codec, int bitsPerTexCoord, int* iTexCoord)
{ _decodeRootSample(codec, bitsPerTexCoord, iTexCoord, 2); }

inline void MPG_Sample_EXT::
decodeCoordSample(MPG_QFac_EXT& codec, int bitsPerCoord, int* iCoord)
{ _decodeSample(codec, bitsPerCoord, iCoord, 3, 0); }

inline void MPG_Sample_EXT::
decodeColorSample(MPG_QFac_EXT& codec, int bitsPerColor, int* iColor)
{ _decodeSample(codec, bitsPerColor, iColor, 3, 2); }

inline void MPG_Sample_EXT::
decodeNormalSample(MPG_QFac_EXT& codec, int bitsPerNormal, int* iNormal)
{ _decodeSample(codec, bitsPerNormal, iNormal, 1, 1); }

inline void MPG_Sample_EXT::
decodeTexCoordSample(MPG_QFac_EXT& codec, int bitsPerTexCoord, int* iTexCoord)
{ _decodeSample(codec, bitsPerTexCoord, iTexCoord, 2, 3); }

inline void MPG_Sample_EXT::
decodeCoordSamples(MPG_QFac_EXT& codec, int bitsPerCoord,
		   vector<int>& iCoord, int n)
{
  if(n>0)
    {
      iCoord.erase(iCoord.begin(),iCoord.end());
      iCoord.insert(iCoord.begin(),(size_t)(3*n),0);
    }
  else
    n = iCoord.size()/3;
  int i;
  for(i=0;i<n;i++)
    decodeCoordSample(codec,bitsPerCoord, iCoord.begin()+3*i);
}

inline void MPG_Sample_EXT::
decodeColorSamples(MPG_QFac_EXT& codec, int bitsPerColor,
		   vector<int>& iColor, int n)
{
  if(n>0)
    {
      iColor.erase(iColor.begin(),iColor.end());
      iColor.insert(iColor.begin(),(size_t)(3*n),0);
    }
  else
    n = iColor.size()/3;
  int i;
  for(i=0;i<n;i++)
    decodeColorSample(codec, bitsPerColor, iColor.begin()+3*i);
}

inline void MPG_Sample_EXT::
decodeNormalSamples(MPG_QFac_EXT& codec, int bitsPerNormal,
		    vector<int>& iNormal, int n)
{
  if(n>0)
    {
      iNormal.erase(iNormal.begin(),iNormal.end());
      iNormal.insert(iNormal.begin(),(size_t)n,0);
    }
  else
    n = iNormal.size();
  int i;
  for(i=0;i<n;i++)
    decodeNormalSample(codec, bitsPerNormal, iNormal.begin()+i);
}

inline void MPG_Sample_EXT::
decodeTexCoordSamples(MPG_QFac_EXT& codec, int bitsPerTexCoord,
		      vector<int>& iTexCoord, int n)
{
  if(n>0)
    {
      iTexCoord.erase(iTexCoord.begin(),iTexCoord.end());
      iTexCoord.insert(iTexCoord.begin(),(size_t)(2*n),0);
    }
  else
    n = iTexCoord.size()/2;
  int i;
  for(i=0;i<n;i++)
    decodeTexCoordSample(codec, bitsPerTexCoord, iTexCoord.begin()+2*i);
}

#endif // _MPG_Sample_EXT_h_

