//-------------------------------------------------------------------------
//  
//  Copyright (C) IBM Corporation, 1996, 1997, 1998, 1999
//  Licensed Materials, Program Property of IBM
//  
//  This software module is an implementation of a part of one or more
//  MPEG-4 tools as specified by the MPEG-4 standard (ISO/IEC 14492).
//  ISO/IEC gives users of the MPEG-4 standard free license to this
//  software module or modifications thereof for use in hardware or
//  software products claiming conformance to the MPEG-4 standard.  Those
//  intending to use this software module in hardware or software products
//  are advised that its use may infringe existing patents.  The original
//  developer of this software module and his/her company, the subsequent
//  editors and their companies, and ISO/IEC have no liability for use of
//  this software module or modifications thereof in an implementation.
//  Copyright is not released for non MPEG-4 conforming products.  IBM
//  Corp. retains full right to use the code for his/her own purpose,
//  assign or donate the code to a third party and to inhibit third
//  parties from using the code for non MPEG-4 conforming products.  This
//  copyright notice must be included in all copies or derivative works.
//  
//  AUTHORS:
//    Gabriel Taubin
//    Andre Gueziec
//    Claudio Silva
//    Ioana Martin
//  
//  IBM T. J. Watson Research Center
//  P.O.Box 704
//  Yorktown Heights, New York 10598
//  
//  BASED ON EARLIER IMPLEMENTATION OF
//  VRML COMPRESSED BINARY FORMAT PROPOSAL BY:
//    Gabriel Taubin
//    Bill Horn
//    Francis Lazarus
// 
//-------------------------------------------------------------------------

#ifndef _MPG_State_EXT_h
#define _MPG_State_EXT_h

#include "MPG_Loops.h"
#include "MPG_SPolygon.h"
#include "MPG_VGraph.h"

class MPG_State_EXT
{
public:
	MPG_State_EXT(int nTriangles = 0);
	void SetState(int nTriangles, 
		int nV0, int nV1, int nVcls0, int nVcls1,
		bool newMol, bool newMobl, bool newCC, bool newTri,
		int mol_id, int lastTriangle, int prev_marching_pattern, 
		int prev_polygon_edge, int triangulated, int nPE, int nF,
		int nC, int nCoord, int nNormal, int nColor, int nTexCoord,
		int iP, int iR, int iL, int iO, int iF, int iFp, int iC,
		int cRp, int cR, int cLp, int cL, int cO, int nVcc, int iRun);
	void GetState(int& nTriangles, 
		int& nV0, int& nV1, int& nVcls0, int& nVcls1,
		bool& newMol, bool& newMobl, bool& newCC, bool& newTri,
		int& mol_id, int& lastTriangle, int& prev_marching_pattern, 
		int& prev_polygon_edge, int& triangulated, int& nPE, int& nF,
		int& nC, int& nCoord, int& nNormal, int& nColor, int& nTexCoord,
		int& iP, int& iR, int& iL, int& iO, int& iF, int& iFp, int& iC,
		int& cRp, int& cR, int& cLp, int& cL, int& cO, int& nVcc, int& iRun);
	void SetNVertices(int nV0, int nV1, int nVcls0, int nVcls1);
	void GetNVertices(int& nV0, int& nV1, int& nVcls0, int& nVcls1);

private:
	int _nTriangles;
	int _nV0;		// number of vertices so far without current cc
	int _nV1;		// number of vertices so far with current cc
	int _nVcls0;	// number of vertices before clustering
	int _nVcls1;    // number of vertices after clustering

	bool _newMol;
	bool _newMobl;
	bool _newCC;
	bool _newTri;
	int  _mol_id;
	int  _lastTriangle;

	int  _prev_marching_pattern; 
	int  _prev_polygon_edge;
	int  _triangulated;

	int  _nPE; 
	int  _nF; 
	int  _nC;
	int  _nCoord; 
	int  _nNormal; 
	int  _nColor; 
	int  _nTexCoord;
	int  _iP; 
	int  _iR; 
	int  _iL; 
	int  _iO;
	int  _iF; 
	int  _iFp; 
	int  _iC; 
	int  _cRp; 
	int  _cR; 
	int  _cLp;
	int  _cL; 
	int  _cO;
	int  _nVcc;
	int _iRun;
};

#endif