//-------------------------------------------------------------------------
// 
// Copyright (C) IBM Corporation & Samsung Electronics Co. LTD., 1996, 1997, 1998, 1999
// Licensed Materials, Program Property of IBM & Samsung.
// 
// This software module is an implementation of a part of one or more
// MPEG-4 tools as specified by the MPEG-4 standard (ISO/IEC 14496).
// ISO/IEC gives users of the MPEG-4 standard (ISO/IEC 14496) free
// license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4
// standard (ISO/IEC 14496).  Those intending to use this software module
// in hardware or software products are advised that its use may infringe
// existing patents.  The original developer of this software module and
// his/her company, the subsequent editors and their companies, and
// ISO/IEC have no liability for use of this software module or
// modifications thereof in an implementation.  Copyright is not released
// for non MPEG-4 standard (ISO/IEC 14496) conforming products.  IBM
// Corp. & Samsung Electronics Co. LTD retains full right to use the code 
// for his/her own purpose, assign or donate the code to a third party and
// to inhibit third parties from using the code for non MPEG-4 standard 
// (ISO/IEC 14496) conforming products. This copyright notice must be included
// in all copies or derivative works.
// 
//  AUTHORS:
//    Gabriel Taubin
//    Andre Gueziec
//    Claudio Silva
//    Ioana Martin
//  
//  IBM T. J. Watson Research Center
//  P.O.Box 704
//  Yorktown Heights, New York 10598
//  
//  BASED ON EARLIER IMPLEMENTATION OF
//  VRML COMPRESSED BINARY FORMAT PROPOSAL BY:
//    Gabriel Taubin
//    Bill Horn
//    Francis Lazarus
//
//  Functionality : Error Resilience
//
//  Sungjin Kim  (sunjin@sait.samsung.co.kr)
//  Mahn-Jin Han  (manjini@sait.samsung.co.kr)
//  Mun-Sup Song (mssong@sait.samsung.co.kr)  
//
//  Signal Processing Lab. Samsung A.I.T. 
//-------------------------------------------------------------------------

#ifndef _SYS_Stream_EXT_h_
#define _SYS_Stream_EXT_h_

#include <stdio.h>
#include <stdlib.h>

#ifdef _AIX
#include <sys/vnode.h>
#endif
#include <sys/stat.h>

#include "MPG_Global.h"	
#include <vector>

#define STREAM_INIT_SIZE 128

using namespace std;

// so far only IO to/from vector<char>

class MPG_Stream_EXT {
protected:
  vector<unsigned char>       _buff;
  int                _next;		// only used by InputStream
  char*              _fname;
public:
  inline              MPG_Stream_EXT();
  inline             ~MPG_Stream_EXT();
  inline const int    size();
  inline const unsigned char*  begin();
};

class MPG_InputStream_EXT : public MPG_Stream_EXT {
public:
  inline             MPG_InputStream_EXT();
  inline             ~MPG_InputStream_EXT();
  inline const unsigned char* begin();
  inline const int   next();
  inline const int   size();
  inline bool        getUChar(unsigned char& c);
  inline bool        getInt(int& i);
  inline bool        getFloat(float& f);
  inline bool        getBytes(int nBytes, unsigned char* bytes);
  inline bool        move(int nBytes);
  inline bool        open(char* fname);
  inline bool        open(int nBytes, unsigned char* bytes);
  inline bool        close();
  inline bool        getStartCode(const unsigned char sc);
  inline bool        lookingAtStartCode(unsigned char sc);
  inline unsigned char  getMOBLStartCode();	
  inline bool		 getVisualStartCode(const unsigned char sc); 
  inline const int   Bsize(); 

//MJH 000925 : used for CIFS library
#ifdef MPGELIB
  inline bool		 open(unsigned char* pbuf, int &m_nLastBytes, 
							int &m_nLastBits, int &m_nlength, int tsize); 
  inline unsigned char* close(unsigned char* pbuf, int &m_nLastBytes, int &m_nLastBits, int &m_nlength, int tsize);
#endif
};

class MPG_OutputStream_EXT : public MPG_Stream_EXT {
public:
  inline             MPG_OutputStream_EXT() {}
  inline            ~MPG_OutputStream_EXT() {}
  inline bool        putUChar(unsigned char c);
  inline bool        putInt(int i);
  inline bool        putFloat(float f);
  inline bool        putBytes(int nBytes, const unsigned char* bytes);
  inline bool        open(char* fname);
  inline bool        close();
  inline bool        putStartCode(const unsigned char sc);
  inline const int   size() { return _buff.size(); } 
  inline void        insertOStream(MPG_OutputStream_EXT& a);  
  inline void        clearOStream() { _buff.clear(); };
  inline void        replaceStartCode(int e, const unsigned char sc);
  inline void        insertStartCode(int e, const unsigned char sc);
  inline void        eraseOutputStream(int e);
  inline bool		 putVisualStartCode(const unsigned char sc);	

#ifdef MPGCLIB
  //MJH 000922 : added for library generation
  inline unsigned char*		 GetBufferUChar(void); 
#endif
};

class MPG_InputBitStream_EXT {
protected:
  MPG_InputStream_EXT*  _istrm;
  unsigned char              _b;
  int               _i;
  inline bool       _getb();
public:
  inline             MPG_InputBitStream_EXT(MPG_InputStream_EXT& istrm);
  inline            ~MPG_InputBitStream_EXT();
  inline bool        getBit(bool& bit);
  inline bool        getMBitInt(int nbits, int& mbi);
  inline bool        getMBitUInt(int nbits, unsigned int& mbui);
  inline bool        getMBitSInt(int nbits, int& mbi);
  inline bool        get32BitFloat(float& x);
  inline bool        move(int nBytes);
  inline bool        flush();
  inline bool        getMOBit(bool& bit);
  inline bool        getMOBitInt(int nbits, int& mbi);
  inline bool        getMOStartCodeBitInt(const unsigned char sc);
};

class MPG_OutputBitStream_EXT {
protected:
  MPG_OutputStream_EXT* _ostrm;
  unsigned char              _b;
  int               _i;
  inline bool       _putb();
public:
  inline             MPG_OutputBitStream_EXT(MPG_OutputStream_EXT& istrm);
  inline            ~MPG_OutputBitStream_EXT();
  inline bool        putBit(bool bit);
  inline bool        putMBitInt(int nbits, int mbi);
  inline bool        putMBitUInt(int nbits, unsigned int mbui);
  inline bool        putMBitSInt(int nbits, int mbi);
  inline bool        put32BitFloat(float x);
  inline bool        flush();
  inline bool        putMOBit(bool bit);
  inline bool        putMOBitInt(int nbits, int mbi);
  inline bool        putMOStartCodeBitInt(const unsigned char sc); 
  inline int         get_i() { return _i; } 
  inline void        init() { _b=0x00; _i=8; } 
  inline void        updateOBitStream(MPG_OutputBitStream_EXT& a) {
	  _b=a._b; a._b=0x00;
	  _i=a._i; a._i=8;
  }
  inline MPG_OutputStream_EXT* getOutputStream(){
	  return _ostrm;
  }
};

// MPG_Stream_EXT -----------------------------------------------------------------

MPG_Stream_EXT::MPG_Stream_EXT(): _next(0), _fname(NULL)
{ _buff.reserve(STREAM_INIT_SIZE); }

MPG_Stream_EXT::~MPG_Stream_EXT()
//{ if(_fname!=NULL) delete [] _fname; }
{ 
	if(_fname!=NULL) 
		free (_fname); 
}

const int MPG_Stream_EXT::size()
{ return _buff.size(); }

const unsigned char* MPG_Stream_EXT::begin()
{ return _buff.begin(); }

// MPG_InputStream_EXT ------------------------------------------------------------

MPG_InputStream_EXT::MPG_InputStream_EXT()
{ }

MPG_InputStream_EXT::~MPG_InputStream_EXT()
{ }

const unsigned char* MPG_InputStream_EXT::begin()
{ return _buff.begin(); }

const int MPG_InputStream_EXT::next()
{ return _next; }

const int MPG_InputStream_EXT::size()
{ return _buff.size()-_next; }

const int MPG_InputStream_EXT::Bsize()
{ return _buff.size(); }

bool MPG_InputStream_EXT::getUChar(unsigned char& c)
{
  c = (_next<_buff.size())?_buff[_next]:0;
  _next++;
  return true;
}
bool MPG_InputStream_EXT::getInt(int& i)
{
  if(_next+sizeof(int)<=_buff.size())
    {
      for(int j=0;j<sizeof(int);j++)
	((unsigned char*)&i)[j] = _buff[_next++];
      return true;
    }
  else
    return false;
}

bool MPG_InputStream_EXT::getFloat(float& f)
{
  if(_next+sizeof(float)<=_buff.size())
    {
      for(int j=0;j<sizeof(float);j++)
	((unsigned char*)&f)[j] = _buff[_next++];
      return true;
    }
  else
    return false;
}

bool MPG_InputStream_EXT::move(int nBytes) {
  _next += nBytes;
  return true;
}

bool MPG_InputStream_EXT::getBytes(int nBytes, unsigned char* bytes)
{
  if(nBytes>0 && bytes!=NULL && _next+nBytes<=_buff.size())
    { while(nBytes-->0) *bytes++ = _buff[_next++]; return true; }
  else
    return false;
}

bool MPG_InputStream_EXT::getStartCode(const unsigned char sc)
{
  // CS 021799
  bool success = false;

  if(_next+2<=_buff.size() &&
     _buff[_next  ]==0x00  && _buff[_next+1]==sc)
    { success = true; }
  _next+=2;
  return success;
}

bool MPG_InputStream_EXT::getVisualStartCode(const unsigned char sc)
{
  bool success = false;

  while(_next+4<=_buff.size()) 
  {
	  if( _buff[_next  ]==0x00  && _buff[_next+1]==0x00 &&
		  _buff[_next+2]==0x01  && _buff[_next+3]==sc) 
	  {
		  _next += 4;
		  return true; 
	  } else {
		  _next++;
	  }
  } 
  return success;
}

bool MPG_InputStream_EXT::lookingAtStartCode(unsigned char sc)
{
  // CS 021799
  bool success = false;

  if(_next+2<=_buff.size() &&
     _buff[_next  ]==0x00 &&
     _buff[_next+1]==sc) return true;
  else
    return false;
}

unsigned char MPG_InputStream_EXT::getMOBLStartCode()
{
  int tmp = _next;  
  while (_next+2<=_buff.size()) {
	  if (_buff[_next]==0x00 && _buff[_next+1]!=0x00 && _buff[_next+1] < 0x60) {
		  if (_buff[_next+1]==MPG_Global_EXT::MOBL0_start_code ||
			  _buff[_next+1]==MPG_Global_EXT::MOBL1_start_code ||
			  _buff[_next+1]==MPG_Global_EXT::MOBL2_start_code) {
	  		  _next+=2;
			  return _buff[_next-1];
		  }
		  else {
			  _next=tmp;  
			  return 0;
		  }
	  }
	  else
		  _next++;
  }
  _next=tmp;  //MJH 001218
  return 0;
}
#ifdef MPGELIB
//MJH 000925 : added for CIFS decoder library
inline bool	MPG_InputStream_EXT::open(unsigned char* pbuf, int &m_nLastBytes, 
								  int &m_nLastBits, int &m_nlength, int tsize)
{
	_buff.erase(_buff.begin(),_buff.end());
	_buff.insert(_buff.begin(),&pbuf[0],&pbuf[tsize-m_nlength]);
    _next = 0;		

	if(m_nLastBits !=0) {
//		if (_buff[_next]==0x00 && _buff[_next+1]==0x00 && 
//			_buff[_next+2]==0x00 && _buff[_next+3]==0x20 ) {
//			_next +=2;
		if (_buff[_next]==0x00 && _buff[_next+1]==0x20 ) {
			if(_next >= tsize)
				return false;
			m_nLastBits = 8;
			m_nLastBytes = 0x00;
		} else 
			return false;
	} else {
		m_nLastBits = 8;
		m_nLastBytes = 0x00;
	}
	
	return true;
}

inline unsigned char* MPG_InputStream_EXT::close(unsigned char* pbuf, int &m_nLastBytes, int &m_nLastBits, int &m_nlength, int tsize)
{
	m_nlength = (tsize - _next)-m_nlength;
	if(m_nlength < 0) 
		return false;
	m_nLastBits = 8;
	m_nLastBytes = 0x00;

	return (pbuf + _next);
}
//MJH end
#endif
void MPG_OutputStream_EXT::replaceStartCode(int e, const unsigned char sc) {
	_buff[e+1] = sc;
}

void MPG_OutputStream_EXT::insertStartCode(int e, const unsigned char sc) {
	unsigned char b[2];
	b[0] = 0x00;
	b[1] = sc;

	vector<unsigned char>::iterator t;
	t=_buff.begin();

	_buff.insert(t+e, b[0]);
	_buff.insert(t+e+1, b[1]);
}

void MPG_OutputStream_EXT::eraseOutputStream(int e) {
	vector<unsigned char>::iterator t;
	t=_buff.begin();
	_buff.erase(t+e, _buff.end());
}

void MPG_OutputStream_EXT::insertOStream(MPG_OutputStream_EXT& a) {
	if(a._buff.size()!=0) 
		_buff.insert(_buff.end(), a._buff.begin(), a._buff.end());
}

inline bool MPG_InputStream_EXT::open(char* fname)
{
  bool success = false;
  _buff.erase(_buff.begin(),_buff.end());
  _next = 0;
  if(_fname) { delete [] _fname; _fname = NULL; }
  if(fname!=NULL)
    {
      FILE *fp = fopen(fname,"rb");
      if(fp!=(FILE*)0)
	{
	  _fname = strdup(fname);
	  struct stat struct_stat;
	  if(stat(fname,&struct_stat)>=0 && struct_stat.st_size>0)
	    {
	      int nBytesStat = struct_stat.st_size;
	      _buff.insert(_buff.end(),(size_t)nBytesStat,'\0');
	      int nBytesRead = fread(_buff.begin(),1,nBytesStat,fp);
	      if(nBytesStat==nBytesRead) success = true;
	    }
	  fclose(fp);
	}
    }
  return success;
}

inline bool MPG_InputStream_EXT::open(int nBytes, unsigned char* bytes)
{
  bool success = true;
  _buff.erase(_buff.begin(),_buff.end());
  _buff.insert(_buff.end(),(size_t)nBytes, '\0');
  memcpy(_buff.begin(), bytes, nBytes);

  return success;
}

inline bool MPG_InputStream_EXT::close()
{ return true; }

// MPG_OutputStream_EXT -----------------------------------------------------------

bool MPG_OutputStream_EXT::putUChar(unsigned char c)
{
  _buff.push_back(c);
  return true;
}

bool MPG_OutputStream_EXT::putInt(int i)
{
  for(int j=0;j<sizeof(int);j++)
    _buff.push_back(((unsigned char*)&i)[j]);
  return true;
}

bool MPG_OutputStream_EXT::putFloat(float f)
{
  for(int j=0;j<sizeof(float);j++)
    _buff.push_back(((unsigned char*)&f)[j]);
  return true;
}

bool MPG_OutputStream_EXT::putBytes(int nBytes, const unsigned char* bytes)
{
  if(nBytes>0 && bytes!=NULL)
    { while(nBytes-->0)	_buff.push_back(*bytes++); return true; }
  return false;
}

bool MPG_OutputStream_EXT::putStartCode(const unsigned char sc)
{
  // CS 021799
  unsigned char b[2];
  b[0] = 0x00;
  b[1] = sc;
  return putBytes(2,b);
}

bool MPG_OutputStream_EXT::putVisualStartCode(const unsigned char sc)
{
  unsigned char b[4];
  b[0] = 0x00;
  b[1] = 0x00;
  b[2] = 0x01;
  b[3] = sc;
  return putBytes(4,b);
}

inline bool MPG_OutputStream_EXT::open(char* fname)
{
  if(fname!=NULL)
    {
      _fname = strdup(fname);
      return true;
    }
  return false;
}

inline bool MPG_OutputStream_EXT::close()
{
  bool success = false;
  if(_fname!=NULL)
    {
      FILE *fp = fopen(_fname,"wb");
      if(fp!=(FILE*)0)
	{
	  int  nBytes = _buff.size();
	  unsigned char *bytes = _buff.begin();
	  int  nBytesWritten = fwrite(bytes,1,nBytes,fp);
	  success = (nBytesWritten<nBytes)?false:true;
	  fclose(fp);
	}
    }
  return success;
}
#ifdef MPGCLIB
//MJH 000922 : added for library generation
inline unsigned char* MPG_OutputStream_EXT::GetBufferUChar(void)
{
	int bsize = _buff.size();
	unsigned char* bf = (unsigned char*)malloc(sizeof(char)*bsize);
	memset(bf,0x00,bsize);
	memcpy(bf, _buff.begin(), bsize);

    return bf;
}
#endif
// MPG_InputBitStream_EXT ---------------------------------------------------------

MPG_InputBitStream_EXT::MPG_InputBitStream_EXT(MPG_InputStream_EXT& istrm)
{
  _istrm = &istrm;
  _b = 0x00;
  _i = 8;
}

bool MPG_InputBitStream_EXT::_getb()
{
  bool success = false;
  if(_istrm)
    success = _istrm->getUChar(_b);
  return success;
}

MPG_InputBitStream_EXT::~MPG_InputBitStream_EXT()
{ }

bool MPG_InputBitStream_EXT::getBit(bool& bit)
{
  bool success = true;
  if(_i==8) success = _getb();
  if(!(success==false))
    {
      bit = (_b&(1<<--_i))?true:false; // MSB first, FJB 7/5/98
      if(_i==0) 
	{
	  _i=8;
	  // CS 021799
	  if(_b == 0x00) {
	    _getb();	    
	    _i=7;
	  }
	}
    }
  return success;
}

bool MPG_InputBitStream_EXT::getMBitInt(int nbits, int& mbi)
{
  // Most significant bit first
  mbi = 0;
  bool bit = false;
  for(int i=0;i<nbits;i++)
    {
      if(getBit(bit)==false) return false;
      mbi <<= 1;
      if(bit!=false) mbi |= 1;
    }
  return true;
}

bool MPG_InputBitStream_EXT::getMBitUInt(int nbits, unsigned int& mbui)
{
  return getMBitInt(nbits,(int&)mbui);
}

bool MPG_InputBitStream_EXT::getMBitSInt(int nbits, int& mbi)
{
  bool sgn = 0;
  if(getBit(sgn)==false) return false;
  if(getMBitInt(nbits-1,mbi)==false) return false;
  mbi |= -(int)(sgn)<<(nbits-1);
  return true;
}

bool MPG_InputBitStream_EXT::get32BitFloat(float& x)
{
  int& ix = *(int*)(&x);
  return getMBitInt(32,ix);
}

bool MPG_InputBitStream_EXT::move(int nbytes)
{
  bool success = _istrm->MPG_InputStream_EXT::move(nbytes-1);
  if(success) success = _getb();
  return success;
}

bool MPG_InputBitStream_EXT::flush()
{
  if(_i!=8)
    {
      _i = 8;
    }
  return true;
}

bool MPG_InputBitStream_EXT::getMOBit(bool& bit)
{
  bool success = true;
  if(_i==8) success = _getb();
  if(!(success==false))
  {
      bit = (_b&(1<<--_i))?true:false; // MSB first, FJB 7/5/98
      if(_i==0) 
		  _i=8;
  }
  return success;
}

bool MPG_InputBitStream_EXT::getMOBitInt(int nbits, int& mbi)
{
  // Most significant bit first
  mbi = 0;
  bool bit = false;
  for(int i=0;i<nbits;i++)
    {
      if(getMOBit(bit)==false) return false;
      mbi <<= 1;
      if(bit!=false) mbi |= 1;
    }
  return true;
}

bool MPG_InputBitStream_EXT::getMOStartCodeBitInt(const unsigned char sc)
{
  bool success = false;
  int a, b;
  if(_istrm->next()+2 <=_istrm->Bsize()) {
	  if(getMOBitInt(8, a)==false) return false;
	  if(getMOBitInt(8, b)==false) return false;
	  if(a==0x00 && b==sc) 
		  success = true;
  }
  return success;
}

// MPG_OutputBitStream_EXT --------------------------------------------------------

MPG_OutputBitStream_EXT::MPG_OutputBitStream_EXT(MPG_OutputStream_EXT& ostrm)
  : _ostrm(&ostrm), _b(0x00), _i(8)
{}

bool MPG_OutputBitStream_EXT::_putb()
{
  bool success = false;
  if(_ostrm)
    success = _ostrm->putUChar(_b);
  return success;
}

MPG_OutputBitStream_EXT::~MPG_OutputBitStream_EXT()
{ flush(); }

bool MPG_OutputBitStream_EXT::putBit(bool bit)
{
  --_i;
  if(bit!=false) _b |= (1<<_i); // MSB first FJB 7/5/98
  if(_i==0)
    {
      if(_putb()==false) return false;
      // CS 021799
      if(_b != 0x00)
	{
	  _b = 0x00;
	  _i = 8;
	}
      else
	{
	  _b = 0x80;
	  _i = 7;	  	  
	}
    }
  return true;
}

bool MPG_OutputBitStream_EXT::putMBitInt(int nbits, int mbi)
{
  // Most significant bit first
  bool bit = false;
  for(int i=0;i<nbits;i++)
    {
      bit = (mbi & (1<<(nbits-i-1)))?true:false;
      if(putBit(bit)==false) return false;
    }
  return true;
}

bool MPG_OutputBitStream_EXT::putMBitUInt(int nbits, unsigned int mbui)
{
  return putMBitInt(nbits,(int)mbui);
}

bool MPG_OutputBitStream_EXT::putMBitSInt(int nbits, int mbi)
{ return putMBitInt(nbits,mbi); }


bool MPG_OutputBitStream_EXT::put32BitFloat(float x)
{
  int ix = *(int*)(&x);
  return putMBitInt(32,ix);;
}

bool MPG_OutputBitStream_EXT::flush()
{
  if(_i!=8)
    {
      if(_putb()==false) return false;
      _b = 0x00;
      _i = 8;
    }
  return true;
}
//ory DMLAB 2007 01 04
#ifndef _bitsToRepresent_
#define _bitsToRepresent_
static int bitsToRepresent(int n)
{
  if(n<0) return -1;
  //ory DMLAB 2006 05
  if(n==1) return 1; 
  int nBits = 0;
  while((1<<nBits)<=(n-1) && nBits<32) nBits++;
  //~ory DMLAB
  return nBits;
}
#endif
//~ory DMLAB
bool MPG_OutputBitStream_EXT::putMOBit(bool bit)
{
  --_i;
  if(bit!=false) _b |= (1<<_i); // MSB first FJB 7/5/98
  if(_i==0)
  {
      if(_putb()==false) return false;
      _b = 0x00;
	  _i = 8;
  }
  return true;
}

bool MPG_OutputBitStream_EXT::putMOBitInt(int nbits, int mbi)
{
  // Most significant bit first
  bool bit = false;
  for(int i=0;i<nbits;i++)
  {
	  bit = (mbi & (1<<(nbits-i-1)))?true:false;
	  if(putMOBit(bit)==false) return false;
  }
  return true;
}

bool MPG_OutputBitStream_EXT::putMOStartCodeBitInt(const unsigned char sc)
{
  unsigned char b;
  b = 0x00;
  if(putMOBitInt(8,b)==false) return false;
  b = sc;
  if(putMOBitInt(8,b)==false) return false;
  return true;
}

#endif // _MPG_Stream_EXT_h_
