//-------------------------------------------------------------------------
// 
// Copyright (C) IBM Corporation & Samsung Electronics Co. LTD., 1996, 1997, 1998, 1999
// Licensed Materials, Program Property of IBM & Samsung.
// 
// This software module is an implementation of a part of one or more
// MPEG-4 tools as specified by the MPEG-4 standard (ISO/IEC 14496).
// ISO/IEC gives users of the MPEG-4 standard (ISO/IEC 14496) free
// license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4
// standard (ISO/IEC 14496).  Those intending to use this software module
// in hardware or software products are advised that its use may infringe
// existing patents.  The original developer of this software module and
// his/her company, the subsequent editors and their companies, and
// ISO/IEC have no liability for use of this software module or
// modifications thereof in an implementation.  Copyright is not released
// for non MPEG-4 standard (ISO/IEC 14496) conforming products.  IBM
// Corp. & Samsung Electronics Co. LTD retains full right to use the code 
// for his/her own purpose, assign or donate the code to a third party and
// to inhibit third parties from using the code for non MPEG-4 standard 
// (ISO/IEC 14496) conforming products. This copyright notice must be included
// in all copies or derivative works.
// 
//  AUTHORS:
//    Gabriel Taubin
//    Andre Gueziec
//    Claudio Silva
//    Ioana Martin
//  
//  IBM T. J. Watson Research Center
//  P.O.Box 704
//  Yorktown Heights, New York 10598
//  
//  BASED ON EARLIER IMPLEMENTATION OF
//  VRML COMPRESSED BINARY FORMAT PROPOSAL BY:
//    Gabriel Taubin
//    Bill Horn
//    Francis Lazarus
//
//  Functionality : Error Resilience
//
//  Sungjin Kim  (sunjin@sait.samsung.co.kr)
//  Mahn-Jin Han  (manjini@sait.samsung.co.kr)
//  Mun-Sup Song (mssong@sait.samsung.co.kr)  
//
//  Signal Processing Lab. Samsung A.I.T. 
//-------------------------------------------------------------------------


#ifndef _MPG_Vector_EXT_h_
#define _MPG_Vector_EXT_h_

#include <math.h>
#include <vector>

using namespace std;

#define vector_erase(x) x.erase(x.begin(),x.end())

class MPG_Vector_EXT {
public:

       static void setAverageVector(int nD, float* v0, vector<float*>& v);
       static void setAverageNormal(float* n0, vector<float*>& n);
inline static void setAverageColor(float* n0, vector<float*>& n);
inline static void setAverageTexCoord(float* n0, vector<float*>& n);

inline static void normalizeNormal(float* n);

       static void computeNormal(float* p0, float* p1, float* p2,
				 float* n, bool normalize);

       static void computeFaceCentroid(vector<float>& coord,
				       int nfi, int* fi, float* fc);

       static void computeFaceNormal(vector<float>& coord,
				     int nfi, int* fi, float* fn,
				     bool normalize);
       static void clusterPropertyValues(vector<int>& cmap, vector<float>& p,
					 int nD, bool normalize);

  // wrong place ???
       static bool indexLabel(int          nV,
			      vector<int>& v_label,
			      vector<int>& v_labelIndex,
			      vector<int>& v_labelValue,
			      vector<int>* v_sortOrder = NULL,
			      vector<int>* v_partStart = NULL);

       static bool cluster(vector<int>& v_order,
			   vector<int>& v_partStart, // cMap
			   vector<float>& coord,
			   vector<float>& coordCluster,
			   bool closest);

       static bool clusterLabel(int          nV,
				vector<int>& v_label,
				vector<int>& v_labelIndex, // cmap
				vector<float>& coord,
				vector<float>& coordClustered,
				bool closest = true);

       static void makeColorPalette(int nColor, vector<float>& color);

};

inline void MPG_Vector_EXT::setAverageColor(float* n0, vector<float*>& n)
{ setAverageVector(3,n0,n); }

inline void MPG_Vector_EXT::setAverageTexCoord(float* n0, vector<float*>& n)
{ setAverageVector(2,n0,n); }

void MPG_Vector_EXT::normalizeNormal(float* n)
{
  float n_length = sqrt(n[0]*n[0] + n[1]*n[1] + n[2]*n[2]);
  if(n_length > 0.0)
    {
      n[0] = n[0]/n_length;
      n[1] = n[1]/n_length;
      n[2] = n[2]/n_length;
    }
}

#endif // _MPG_Vector_EXT_h_


