//-------------------------------------------------------------------------
// 
// Copyright (C) IBM Corporation & Samsung Electronics Co. LTD., 1996, 1997, 1998, 1999
// Licensed Materials, Program Property of IBM & Samsung.
// 
// This software module is an implementation of a part of one or more
// MPEG-4 tools as specified by the MPEG-4 standard (ISO/IEC 14496).
// ISO/IEC gives users of the MPEG-4 standard (ISO/IEC 14496) free
// license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4
// standard (ISO/IEC 14496).  Those intending to use this software module
// in hardware or software products are advised that its use may infringe
// existing patents.  The original developer of this software module and
// his/her company, the subsequent editors and their companies, and
// ISO/IEC have no liability for use of this software module or
// modifications thereof in an implementation.  Copyright is not released
// for non MPEG-4 standard (ISO/IEC 14496) conforming products.  IBM
// Corp. & Samsung Electronics Co. LTD retains full right to use the code 
// for his/her own purpose, assign or donate the code to a third party and
// to inhibit third parties from using the code for non MPEG-4 standard 
// (ISO/IEC 14496) conforming products. This copyright notice must be included
// in all copies or derivative works.
// 
//  AUTHORS:
//    Gabriel Taubin
//    Andre Gueziec
//    Claudio Silva
//    Ioana Martin
//  
//  IBM T. J. Watson Research Center
//  P.O.Box 704
//  Yorktown Heights, New York 10598
//  
//  BASED ON EARLIER IMPLEMENTATION OF
//  VRML COMPRESSED BINARY FORMAT PROPOSAL BY:
//    Gabriel Taubin
//    Bill Horn
//    Francis Lazarus
//
//  Functionality : Error Resilience
//
//  Sungjin Kim  (sunjin@sait.samsung.co.kr)
//  Mahn-Jin Han  (manjini@sait.samsung.co.kr)
//  Mun-Sup Song (mssong@sait.samsung.co.kr)  
//
//  Signal Processing Lab. Samsung A.I.T. 
//-------------------------------------------------------------------------
// MPG_BranchTree_EXT.cpp: implementation of the MPG_BranchTree_EXT class.
//
//////////////////////////////////////////////////////////////////////

#include "MPG_BranchTree.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

MPG_EachBranchTree_EXT::MPG_EachBranchTree_EXT()
{
	face_num = -1;
	corner_num = -1;
	next = 0;
}

MPG_EachBranchTree_EXT::~MPG_EachBranchTree_EXT()
{
	if (next)
		delete next;
	next=0;
}

MPG_BranchTree_EXT::MPG_BranchTree_EXT()
{
	parent = 0;
	cur_dir = LEFT_BRANCH;
}

MPG_BranchTree_EXT::~MPG_BranchTree_EXT()
{
}

void MPG_BranchTree_EXT::InsertInfo(int dir, int f, int c)
{
	ebt[dir].face_num = f;
	ebt[dir].corner_num = c;
}

void MPG_BranchTree_EXT::GetInfo(int dir, int& f, int& c)
{
	f = ebt[dir].face_num;
	c = ebt[dir].corner_num;
}

void MPG_BranchTree_EXT::MakeBranch(int dir)
{
	ebt[dir].next = new MPG_BranchTree_EXT;
	ebt[dir].next->parent = this;
}

MPG_BranchTree_EXT* MPG_BranchTree_EXT::GetBranch(int dir)
{
	return ebt[dir].next;
}

bool MPG_BranchTree_EXT::Empty(int dir)
{
	if (ebt[dir].next)
		return false;
	else
		return true;
}

void MPG_BranchTree_EXT::SetCurDir(int dir)
{
	cur_dir = dir;
}

int MPG_BranchTree_EXT::GetCurDir()
{
	return cur_dir;
}


MPG_BranchTree_EXT* MPG_BranchTree_EXT::GetParent()
{
	return parent;
}

MPG_BranchForest_EXT::MPG_BranchForest_EXT()
{
	root_bt = new MPG_BranchTree_EXT;
	next = 0;
}

MPG_BranchForest_EXT::~MPG_BranchForest_EXT()
{
	delete root_bt;
	if (next)
		delete next;
}

void MPG_BranchForest_EXT::MakeNextBTree()
{
	if (!next)
		next = new MPG_BranchForest_EXT;
}

MPG_BranchForest_EXT* MPG_BranchForest_EXT::GetNextBTree()
{
	return next;
}

MPG_BranchTree_EXT* MPG_BranchForest_EXT::GetRootBTree()
{
	return root_bt;
}


