//-------------------------------------------------------------------------
//  
//  Copyright (C) IBM Corporation, 1996, 1997, 1998, 1999
//  Licensed Materials, Program Property of IBM
//  
//  This software module is an implementation of a part of one or more
//  MPEG-4 tools as specified by the MPEG-4 standard (ISO/IEC 14492).
//  ISO/IEC gives users of the MPEG-4 standard free license to this
//  software module or modifications thereof for use in hardware or
//  software products claiming conformance to the MPEG-4 standard.  Those
//  intending to use this software module in hardware or software products
//  are advised that its use may infringe existing patents.  The original
//  developer of this software module and his/her company, the subsequent
//  editors and their companies, and ISO/IEC have no liability for use of
//  this software module or modifications thereof in an implementation.
//  Copyright is not released for non MPEG-4 conforming products.  IBM
//  Corp. retains full right to use the code for his/her own purpose,
//  assign or donate the code to a third party and to inhibit third
//  parties from using the code for non MPEG-4 conforming products.  This
//  copyright notice must be included in all copies or derivative works.
//  
//  AUTHORS:
//    Gabriel Taubin
//    Andre Gueziec
//    Claudio Silva
//    Ioana Martin
//  
//  IBM T. J. Watson Research Center
//  P.O.Box 704
//  Yorktown Heights, New York 10598
//  
//  BASED ON EARLIER IMPLEMENTATION OF
//  VRML COMPRESSED BINARY FORMAT PROPOSAL BY:
//    Gabriel Taubin
//    Bill Horn
//    Francis Lazarus
// 
//-------------------------------------------------------------------------

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "MPG_IfsQ_EXT_C.h"
#include "MPG_IfsEdit_EXT.h"

FILE* logfp     = stderr;

//---------------------------------------------------------------------------
//
//   usage
//
//---------------------------------------------------------------------------

void usage()
{
  static char *mm[] = {
    " -----------------------------------------------------------------------",
    "  Copyright (C) IBM Corporation, 1996, 1997, 1998 ",
    "  Licensed Materials, Program Property of IBM ",
    " ",
    "  This software module is an implementation of a part of one or more ",
    "  MPEG-4 tools as specified by the MPEG-4.  ISO/IEC gives users of the ",
    "  MPEG-4 free license to this software module or modifications thereof ",
    "  for use in hardware or software products claiming conformance to the ",
    "  MPEG-4.  Those intending to use this software module in hardware or ",
    "  software products are advised that its use may infringe existing ",
    "  patents.  The original developer of this software module and his/her ",
    "  company, the subsequent editors and their companies, and ISO/IEC have ",
    "  no liability for use of this software module or modifications thereof ",
    "  in an implementation.  Copyright is not released for non MPEG-4 ",
    "  conforming products.  IBM Corp. retains full right to use the code for ",
    "  his/her own purpose, assign or donate the code to a third party and to ",
    "  inhibit third parties from using the code for non MPEG-4 conforming ",
    "  products.  This copyright notice must be included in all copies or ",
    "  derivative works. ",
    " ",
    "  AUTHORS: ",
    " ",
    "  Gabriel Taubin ",
    "  Bill Horn ",
    "  Andre Gueziec ",
    "  Claudio Silva ",
    "  Francis Lazarus ",
    " ",
    "  IBM T. J. Watson Research Center ",
    "  P.O.Box 704 ",
    "  Yorktown Heights, New York 10598 ",
    "",
    " -----------------------------------------------------------------------",
    " ",
    "  USAGE: break_model [options] baseFname ",
    "",
    "    -h           for help",
    "    -id<inDir>   tries to read input files from 'inDir' directory",
    "    -od<outDir>  places output files in 'outDir' directory",
    "    -x<n>        number of partitions on the x axis    (default= 4)",
    "    -y<n>        number of partitions on the y axis    (default= 4)",
    "    -z<n>        number of partitions on the z axis    (default= 4)",
    "    -n<n>        number of partitions on the sphere (normals)      ",
    "",
    " -----------------------------------------------------------------------",
    NULL
  };
  for(char **m=mm;*m;m++)
    fprintf(stderr,"%s\n",*m);
  exit(1);
}

//---------------------------------------------------------------------------
//
//   error
//
//---------------------------------------------------------------------------

void error(const char *msg)
{
  fprintf(logfp,"error : %s\n",(msg)?msg:"");
  exit(-1);
}

//---------------------------------------------------------------------------
//
//   main
//
//---------------------------------------------------------------------------

int main(int argc, char **argv)
{
  char* baseFname = NULL;
  char* inExt     = NULL;
  char* inDir     = NULL;
  char* outDir    = NULL;
  char* inFname   = NULL;
  char* outFname  = NULL;

  int bitsPerCoord       = 10;
  int bitsPerNormal      =  9;
  int bitsPerColor       =  6;
  int bitsPerTexCoord    = 10;

  int coordPrediction    =  2;
  int normalPrediction   =  0;
  int colorPrediction    =  1;
  int texCoordPrediction =  1;

  int x_band  = 0;
  int y_band  = 0;
  int z_band  = 0;
  int n_band  = 0;

  int i,j,n,success;
  if(argc==1)
    usage();
  else for(i=1;i<argc;i++)
    {
      int d;
      if(!strcmp(argv[i],"-h") || !strcmp(argv[i],"-help"))
	usage();
      else if(!strncmp(argv[i],"-id",3))
        inDir = strdup(argv[i]+3);
      else if(!strncmp(argv[i],"-od",3))
        outDir = strdup(argv[i]+3);
      else if(sscanf(argv[i],"-x%d",&d)>0)
	x_band = d;
      else if(sscanf(argv[i],"-y%d",&d)>0)
	y_band = d;
      else if(sscanf(argv[i],"-z%d",&d)>0)
	z_band = d;
      else if(sscanf(argv[i],"-n%d",&d)>0)
	n_band = d;
      else if(argv[i][0]=='-')
	error("unknown option");
      else if(!baseFname)
	baseFname = strdup(argv[i]);
      else
	error("too many input files");
    }

  // check arguments
  if(!baseFname)
    error("no input file");

  // remove directory baseFname
  for(j=strlen(baseFname)-1;j>=0;j--)
    if(baseFname[j]=='/') { baseFname[j]='\0'; j++; break; }
  if(!inDir)
    inDir = (j>=0)?baseFname:strdup(".");
  else if(j>0 && strcmp(inDir,baseFname)!=0)
    error("inDir");
  if(j>0) baseFname += j;
  // remove extension
  for(j=strlen(baseFname)-1;j>0;j--)
    if(baseFname[j]=='.') { baseFname[j]='\0'; inExt = baseFname+j+1; break; }
  if(!baseFname)
    error("no input file");
  if(!outDir) outDir = strdup(inDir);
  if(!inExt)  inExt  = strdup("");

  MPG_IfsQ_EXT_C ifs;

  // read from input file
  {
    int n = strlen(inDir)+strlen(baseFname)+8;
    inFname = new char[n];
    // construct input file name
    if(!strcmp(inDir,"."))
      sprintf(inFname,"%s.wrl",baseFname);
    else
      sprintf(inFname,"%s/%s.wrl",inDir,baseFname);

    success = ifs.read(inFname);

    delete [] inFname;
  }

  if(!success) error("parsing input file");

  // operate
  {

    MPG_IfsMesh_EXT mesh(ifs);
    MPG_IfsEdit_EXT ifsEdit(mesh);
    vector<bool> e_marks;

    mesh.partitionVertices(3,
			   ifs.getCoord(),
			   ifs.getCoordIndex(),
			   x_band,
			   y_band,
			   z_band,
			   n_band,
			   e_marks);
    vector<int> v_stitch2;
    success = ifsEdit.cutThroughMarkedEdges(e_marks, v_stitch2);
  }

  // write to output file
  {
    int n = strlen(outDir)+strlen(baseFname)+12;
    outFname = new char[n];
    if(!strcmp(outDir,"."))
      sprintf(outFname,"%s-B.wrl",baseFname);
    else
      sprintf(outFname,"%s/%s-B.wrl",outDir,baseFname);

    success = ifs.write(outFname);

    delete [] outFname;
  }

  if(!success) error("compressing and saving output file");

  return 0;
}
