//-------------------------------------------------------------------------
// 
// Copyright (C) IBM Corporation & Samsung Electronics Co. LTD., 1996, 1997, 1998, 1999, 2000
// Licensed Materials, Program Property of IBM & Samsung.
// 
// This software module is an implementation of a part of one or more
// MPEG-4 tools as specified by the MPEG-4 standard (ISO/IEC 14496).
// ISO/IEC gives users of the MPEG-4 standard (ISO/IEC 14496) free
// license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4
// standard (ISO/IEC 14496).  Those intending to use this software module
// in hardware or software products are advised that its use may infringe
// existing patents.  The original developer of this software module and
// his/her company, the subsequent editors and their companies, and
// ISO/IEC have no liability for use of this software module or
// modifications thereof in an implementation.  Copyright is not released
// for non MPEG-4 standard (ISO/IEC 14496) conforming products.  IBM
// Corp. & Samsung Electronics Co. LTD retains full right to use the code 
// for his/her own purpose, assign or donate the code to a third party and
// to inhibit third parties from using the code for non MPEG-4 standard 
// (ISO/IEC 14496) conforming products. This copyright notice must be included
// in all copies or derivative works.
// 
//  AUTHORS:
//    Gabriel Taubin
//    Andre Gueziec
//    Claudio Silva
//    Ioana Martin
//  
//  IBM T. J. Watson Research Center
//  P.O.Box 704
//  Yorktown Heights, New York 10598
//  
//  BASED ON EARLIER IMPLEMENTATION OF
//  VRML COMPRESSED BINARY FORMAT PROPOSAL BY:
//    Gabriel Taubin
//    Bill Horn
//    Francis Lazarus
//
//  Functionality : Error Resilience
//
//  Sungjin Kim  (sunjin@sait.samsung.co.kr)
//  Mahn-Jin Han  (manjini@sait.samsung.co.kr)
//  Mun-Sup Song (mssong@sait.samsung.co.kr)  
//
//  Signal Processing Lab. Samsung A.I.T. 
//-------------------------------------------------------------------------

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "MPG_IfsQ_EXT_C.h"

FILE* logfp     = stderr;

#define not(b) ((b)==true)?false:true

char* baseFname = NULL;
char* inExt     = NULL;
char* inDir     = NULL;
char* outDir    = NULL;
char* inFname   = NULL;
char* outFname  = NULL;
char* logFname  = NULL;

//---------------------------------------------------------------------------
//
//   usage
//
//---------------------------------------------------------------------------

void usage()
{
  static char *mm[] = {
    " -----------------------------------------------------------------------",
    "  Copyright (C) IBM Corporation & Samsung Electronics Co. LTD., 1996, 1997, 1998, 1999, 2000 ",
    "  Licensed Materials, Program Property of IBM & Samsung.",
    " ",
    "  This software module is an implementation of a part of one or more ",
    "  MPEG-4 tools as specified by the MPEG-4.  ISO/IEC gives users of the ",
    "  MPEG-4 free license to this software module or modifications thereof ",
    "  for use in hardware or software products claiming conformance to the ",
    "  MPEG-4.  Those intending to use this software module in hardware or ",
    "  software products are advised that its use may infringe existing ",
    "  patents.  The original developer of this software module and his/her ",
    "  company, the subsequent editors and their companies, and ISO/IEC have ",
    "  no liability for use of this software module or modifications thereof ",
    "  in an implementation.  Copyright is not released for non MPEG-4 ",
    "  conforming products.  IBM Corp. & Samsung Electronics Co. LTD retains ",
	"  full right to use the code for his/her own purpose, assign or donate ",
	"  the code to a third party and to inhibit third parties from using ",
	"  the code for non MPEG-4 conforming products. This copyright notice must ",
	"  be included in all copies or derivative works. ",
    " ",
    "  AUTHORS: ",
    " ",
    "  Gabriel Taubin ",
    "  Bill Horn ",
    "  Andre Gueziec ",
    "  Claudio Silva ",
    "  Francis Lazarus ",
    " ",
    "  IBM T. J. Watson Research Center ",
    "  P.O.Box 704 ",
    "  Yorktown Heights, New York 10598 ",
    "",
	"  Functionality : Error Resilience ",
    "  Sungjin Kim  (sunjin@sait.samsung.co.kr)",
    "  Mahn-Jin Han (manjini@sait.samsung.co.kr)",
    "  Mun-Sup Song (mssong@sait.samsung.co.kr)",  
    " ",
    "  Signal Processing Lab. Samsung A.I.T.",
    " -----------------------------------------------------------------------",
    " ",
    "  USAGE: mpgc [options] baseFname ",
    "",
    "    -h           for help",
    "    -id<inDir>   tries to read input files from 'inDir' directory",
    "    -od<outDir>  places output files in 'outDir' directory",
    "",
    "    -bpv<bpv>    number of bits per vertex coordinate    (default= 10)",
    "    -bpn<bpn>    number of bits per normal               (default=  9)",
    "    -bpc<bpc>    number of bits per color component      (default=  6)",
    "    -bpt<bpt>    number of bits per texture coordinate   (default= 10)",
    "",
    "    -ppv<ppv>    prediction mode for vertex coordinates  (default=  2)",
    "    -ppn<ppn>    prediction mode for normals             (default=  0)",
    "    -ppc<ppc>    prediction mode for color components    (default=  1)",
    "    -ppt<ppt>    prediction mode for texture coordinates (default=  1)",
    "",
	"    -er<bpv>     Error Resilience mode                   (default=0)",
	"                  (0:pt=0, 1:pt=0+1+2)",
	"    -ps<bpv>     #bits per Packet                        (default=180)",
	"    -bp<bpv>     boundary predicition type               (default=0)",
	"                  (0: restricted predition, 1:extended prediction)",
	"",
    "    -hm          turns on hierarchical mode              (default=off)",
    "    -cls         save cls file   [ignored if hm=off]     (default=off)",
    "    -wrl         save wrl levels [ignored if hm=off]     (default=off)",
//DMLAB (200501) - ADD
    "    -u           image size for u direction              (default=off)",
    "    -v           image size for v direction              (default=off)",
//~DMLAB (200501)
    "",
    " -----------------------------------------------------------------------",
    NULL
  };
  for(char **m=mm;*m;m++)
    printf("%s\n",*m);
  exit(1);
}

//---------------------------------------------------------------------------
//
//   error
//
//---------------------------------------------------------------------------

void error(const char *msg)
{
  fprintf(logfp,"error : %s\n",(msg)?msg:"");
  exit(-1);
}

//---------------------------------------------------------------------------
//
//   main
//
//---------------------------------------------------------------------------

int main(int argc, char **argv)
{
	int bitsPerCoord       = 10;
	int bitsPerNormal      =  9;
	int bitsPerColor       =  6;
	int bitsPerTexCoord    = 10;
	
//DMLAB (200501) - ADD
	int img_flag = false;
	int img_u = 0;
	int img_v = 0;
//~DMLAB (200501)
	
	int coordPrediction    =  2;
	int normalPrediction   =  0;
	int colorPrediction    =  1;
	int texCoordPrediction =  1;
	
	int erMode = 0;            
	int packetSize=180;
	int boundaryPrediction=0;  
	
	bool hierarchicalMode  = false;
	bool saveClsFile       = false;
	bool saveWrlLevels     = false;
	//DK
	bool vertex_order_flag  =  0;
	bool face_order_flag    =  0;
	bool face_order_per_CC_flag = 0;
	bool vertex_order_per_CC_flag = 0;
	//~DK
	clock_t start, stop; //DK
	double duration; //DK
	
	int i,j,success;
	if(argc==1)
		usage();
	else for(i=1;i<argc;i++)
    {
		int d;
		if(!strcmp(argv[i],"-h") || !strcmp(argv[i],"-help"))
			usage();
		else if(!strncmp(argv[i],"-id",3))
			inDir = strdup(argv[i]+3);
		else if(!strncmp(argv[i],"-od",3))
			outDir = strdup(argv[i]+3);
		else if(sscanf(argv[i],"-bpv%d",&d)>0)
			bitsPerCoord = d;
		else if(sscanf(argv[i],"-bpn%d",&d)>0)
			bitsPerNormal = d;
		else if(sscanf(argv[i],"-bpc%d",&d)>0)
			bitsPerColor = d;
		else if(sscanf(argv[i],"-bpt%d",&d)>0)
			bitsPerTexCoord = d;
		else if(sscanf(argv[i],"-ppv%d",&d)>0)
			coordPrediction = d;
		else if(sscanf(argv[i],"-ppn%d",&d)>0)
			normalPrediction = d;
		else if(sscanf(argv[i],"-ppc%d",&d)>0)
			colorPrediction = d;
		else if(sscanf(argv[i],"-ppt%d",&d)>0)
			texCoordPrediction = d;
		else if(sscanf(argv[i],"-er%d",&d)>0)
			erMode=d;              
		else if(sscanf(argv[i],"-ps%d",&d)>0)
			packetSize=d;          
		else if(sscanf(argv[i],"-bp%d",&d)>0)
			boundaryPrediction=d;  
		//DK
		else if(!strncmp(argv[i],"-vo",3))
			vertex_order_flag=1;
		else if(!strncmp(argv[i],"-fo",3))
			face_order_flag=1;
		else if(!strncmp(argv[i],"-cf",3))
			face_order_per_CC_flag=1;
		else if(!strncmp(argv[i],"-cv",3))
			vertex_order_per_CC_flag=1;
			
		//~DK
		else if(!strcmp(argv[i],"-hm") ||
			!strcmp(argv[i],"-hierarchicalMode"))
			hierarchicalMode=not(hierarchicalMode);
		else if(!strcmp(argv[i],"-cls") ||
			!strcmp(argv[i],"-saveClsFile"))
			saveClsFile=not(saveClsFile);
		else if(!strcmp(argv[i],"-wrl") ||
			!strcmp(argv[i],"-saveWrlLevels"))
			saveWrlLevels=not(saveWrlLevels);
//DMLAB (200501) - ADD
		else if(sscanf(argv[i],"-u%d",&d)>0)
			img_u = d;
		else if(sscanf(argv[i],"-v%d",&d)>0)
			img_v = d;
//~DMLAB (200501)
		else if(argv[i][0]=='-')
			error("unknown option");
		else if(!baseFname)
			baseFname = strdup(argv[i]);
		else
			error("too many input files");
    }
	
	
//DMLAB (200501) - ADD
		if((img_u > 0) && (img_v > 0))
			img_flag = true;
//~DMLAB (200501)

	if(bitsPerCoord<1       || bitsPerCoord>31)      error("bitsPerCoord");
	if(bitsPerNormal<3      || bitsPerNormal>31)     error("bitsPerNormal");
	if(bitsPerColor<1       || bitsPerColor>31)      error("bitsPerColor");
	if(bitsPerTexCoord<1    || bitsPerTexCoord>31)   error("bitsPerTexCoord");
	
	if(coordPrediction<0    || coordPrediction>2)    error("coordPrediction");
	if(normalPrediction<0   || normalPrediction>2)   error("normalPrediction");
	if(colorPrediction<0    || colorPrediction>2)    error("colorPrediction");
	if(texCoordPrediction<0 || texCoordPrediction>2) error("texCoordPrediction");
	
	if(erMode < 0 || erMode >1) error("error resilience mode");  
	if(packetSize<0) error("Packet size");
	if(boundaryPrediction<0 || boundaryPrediction>1) error("boundary prediction"); 
	
	if(!baseFname)
		error("no input file");
	
	for(j=strlen(baseFname)-1;j>=0;j--)
		if(baseFname[j]=='/') 
		{ baseFname[j]='\0'; j++; break; }
	if(!inDir)
		inDir = (j>=0)?baseFname:strdup(".");
	else if(j>0 && strcmp(inDir,baseFname)!=0)
		error("inDir");
	if(j>0) baseFname += j;
	for(j=strlen(baseFname)-1;j>0;j--)
		if(baseFname[j]=='.') 
		{ baseFname[j]='\0'; inExt = baseFname+j+1; break; }
	if(!baseFname)
		error("no input file");
	if(!outDir) outDir = strdup(inDir);
	if(!inExt)  inExt  = strdup("");
	MPG_IfsQ_EXT_C ifs;
			
	// read from input file
	{
		int n = strlen(inDir)+strlen(baseFname)+8;
		inFname = new char[n];
		if(!strcmp(inDir,"."))
			sprintf(inFname,"%s.wrl",baseFname);
		else
			sprintf(inFname,"%s/%s.wrl",inDir,baseFname);
		success = ifs.read(inFname);
		delete [] inFname;
	}
	if(!success) error("parsing input file");
		
	// write to output file
	{
		int n = strlen(outDir)+strlen(baseFname)+12;
		outFname = new char[n];
		if(!strcmp(outDir,"."))
			sprintf(outFname,"%s.m3d",baseFname);
		else
			sprintf(outFname,"%s/%s.m3d",outDir,baseFname);
				
		MPG_OutputStream ostrm;
		success = ostrm.open(outFname);
		if(success==false) error("opening output file");
				
		MPG_OutputStream t_ostrm;     
		start = clock();
		printf("%s\n", "start compressing");
//DMLAB (200606) - UPDATE
		success = ifs.compress(ostrm, t_ostrm,                           
						bitsPerCoord, coordPrediction,
						bitsPerNormal, normalPrediction,
						bitsPerColor, colorPrediction,
						bitsPerTexCoord, texCoordPrediction,
						erMode, packetSize, boundaryPrediction, 
						hierarchicalMode, saveClsFile,
						saveWrlLevels,img_flag, img_u, img_v,
						vertex_order_flag, face_order_flag,face_order_per_CC_flag,vertex_order_per_CC_flag);
//~DMLAB (200606)
		stop= clock();
		duration = ((double)(stop-start))/CLK_TCK;
		printf("running time = %f\n", duration);

		if(success==false) error("compressing");
		success = ostrm.close();
		if(success==false) error("clossing output file");
		delete [] outFname;
	}
	if(!success) error("compressing and saving output file");
	printf("done!!!\n");
			
	return 0;
}
