//-------------------------------------------------------------------------
// 
// Copyright (C) IBM Corporation & Samsung Electronics Co. LTD., 1996, 1997, 1998, 1999
// Licensed Materials, Program Property of IBM & Samsung.
// 
// This software module is an implementation of a part of one or more
// MPEG-4 tools as specified by the MPEG-4 standard (ISO/IEC 14496).
// ISO/IEC gives users of the MPEG-4 standard (ISO/IEC 14496) free
// license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4
// standard (ISO/IEC 14496).  Those intending to use this software module
// in hardware or software products are advised that its use may infringe
// existing patents.  The original developer of this software module and
// his/her company, the subsequent editors and their companies, and
// ISO/IEC have no liability for use of this software module or
// modifications thereof in an implementation.  Copyright is not released
// for non MPEG-4 standard (ISO/IEC 14496) conforming products.  IBM
// Corp. & Samsung Electronics Co. LTD retains full right to use the code 
// for his/her own purpose, assign or donate the code to a third party and
// to inhibit third parties from using the code for non MPEG-4 standard 
// (ISO/IEC 14496) conforming products. This copyright notice must be included
// in all copies or derivative works.
// 
//  AUTHORS:
//    Gabriel Taubin
//    Andre Gueziec
//    Claudio Silva
//    Ioana Martin
//  
//  IBM T. J. Watson Research Center
//  P.O.Box 704
//  Yorktown Heights, New York 10598
//  
//  BASED ON EARLIER IMPLEMENTATION OF
//  VRML COMPRESSED BINARY FORMAT PROPOSAL BY:
//    Gabriel Taubin
//    Bill Horn
//    Francis Lazarus
//
//  Functionality : Error Resilience
//
//  Sungjin Kim  (sunjin@sait.samsung.co.kr)
//  Mahn-Jin Han  (manjini@sait.samsung.co.kr)
//  Mun-Sup Song (mssong@sait.samsung.co.kr)  
//
//  Signal Processing Lab. Samsung A.I.T. 
//-------------------------------------------------------------------------

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "MPG_IfsQ_E_EXT.h"

FILE* logfp     = stderr;

#define not(b) ((b)==true)?false:true

char* baseFname = NULL;
char* inExt     = NULL;
char* inDir     = NULL;
char* outDir    = NULL;
char* inFname   = NULL;
char* outFname  = NULL;
char* logFname  = NULL;

//---------------------------------------------------------------------------
//
//   usage
//
//---------------------------------------------------------------------------

void usage()
{
  static char *mm[] = {
   	" -----------------------------------------------------------------------",
	"  Copyright (C) IBM Corporation & Samsung Electronics Co. LTD., 1996, 1997, 1998, 1999",
    "  Licensed Materials, Program Property of IBM & Samsung. ",
    " ",
    "  This software module is an implementation of a part of one or more ",
    "  MPEG-4 tools as specified by the MPEG-4 standard (ISO/IEC 14496). ",
    "  ISO/IEC gives users of the MPEG-4 standard (ISO/IEC 14496) free ",
    "  license to this software module or modifications thereof for use in ",
    "  hardware or software products claiming conformance to the MPEG-4 ",
    "  standard (ISO/IEC 14496).  Those intending to use this software module ",
    "  in hardware or software products are advised that its use may infringe ",
    "  existing patents.  The original developer of this software module and ",
    "  his/her company, the subsequent editors and their companies, and ",
    "  ISO/IEC have no liability for use of this software module or ",
    "  modifications thereof in an implementation.  Copyright is not released ",
    "  for non MPEG-4 standard (ISO/IEC 14496) conforming products.  IBM ",
    "  Corp. & Samsung Electronics Co. LTD retains full right to use the code  ",
    "  for his/her own purpose, assign or donate the code to a third party and ",
    "  to inhibit third parties from using the code for non MPEG-4 standard  ",
    "  (ISO/IEC 14496) conforming products. This copyright notice must be included ",
    "  in all copies or derivative works. ",
	" ",
	"  AUTHORS: ",
	" ",
	"  Gabriel Taubin ",
	"  Bill Horn ",
	"  Andre Gueziec ",
	"  Claudio Silva ",
	"  Francis Lazarus ",
	" ",
	"  IBM T. J. Watson Research Center ",
	"  P.O.Box 704 ",
	"  Yorktown Heights, New York 10598 ",
	"",
	"  Functionality : Error Resilience ",
	"",
	"  Sungjin Kim  (sunjin@sait.samsung.co.kr)",
    "  Mahn-Jin Han  (manjini@sait.samsung.co.kr)",
    "  Mun-Sup Song (mssong@sait.samsung.co.kr) ", 
    "",
    "  Signal Processing Lab. Samsung A.I.T. ",
	"",
	" -----------------------------------------------------------------------",
	" ",
    "USAGE: mpge [options] baseFname ",
    "    -h           for help",
    "    -id<inDir>   tryes to read input files from 'inDir' directory",
    "    -od<outDir>  places output files in 'outDir' directory",
    " ",
    "    -wrl         save wrl levels                         (default=off)",
    NULL
  };
  for(char **m=mm;*m;m++)
    fprintf(stderr,"%s\n",*m);
  exit(1);
}

//---------------------------------------------------------------------------
//
//   error
//
//---------------------------------------------------------------------------

void error(const char *msg)
{
  fprintf(logfp,"error : %s\n",(msg)?msg:"");
  exit(-1);
}

//---------------------------------------------------------------------------
//
//   main
//
//---------------------------------------------------------------------------

int main(int argc, char **argv)
{
  bool saveWrlLevels     = false;

  int i,j,success;
  if(argc==1)
    usage();
  else for(i=1;i<argc;i++)
    {
      if(!strcmp(argv[i],"-h") || !strcmp(argv[i],"-help"))
	usage();
      else if(!strncmp(argv[i],"-id",3))
        inDir = strdup(argv[i]+3);
      else if(!strncmp(argv[i],"-od",3))
        outDir = strdup(argv[i]+3);
      else if(!strcmp(argv[i],"-hm") ||
              !strcmp(argv[i],"-hierarchicalMode"))
        {
          ; // skip
        }
      else if(!strcmp(argv[i],"-wrl") ||
              !strcmp(argv[i],"-saveWrlLevels"))
        saveWrlLevels=not(saveWrlLevels);
      else if(argv[i][0]=='-')
	error("unknown option");
      else if(!baseFname)
	baseFname = strdup(argv[i]);
      else
	error("too many input files");
    }

  // check arguments
  if(!baseFname)
    error("no input file");

  // remove directory baseFname
  for(j=strlen(baseFname)-1;j>=0;j--)
    if(baseFname[j]=='/') { baseFname[j]='\0'; j++; break; }
  if(!inDir)
    inDir = (j>=0)?baseFname:strdup(".");
  else if(j>0 && strcmp(inDir,baseFname)!=0)
    error("inDir");
  if(j>0) baseFname += j;
  // remove extension
  for(j=strlen(baseFname)-1;j>0;j--)
    if(baseFname[j]=='.') { baseFname[j]='\0'; inExt = baseFname+j+1; break; }
  if(!baseFname)
    error("no input file");
  if(!outDir) outDir = strdup(inDir);
  if(!inExt)  inExt  = strdup("");

  MPG_IfsQ_E_EXT ifs;

  int start, end;
  // read from input file
  {
    int n = strlen(inDir)+strlen(baseFname)+8;
    inFname = new char[n];
    // construct input file name
    if(!strcmp(inDir,"."))
      sprintf(inFname,"%s.m3d",baseFname);
    else
      sprintf(inFname,"%s/%s.m3d",inDir,baseFname);

    MPG_InputStream_EXT istrm;
    success = istrm.open(inFname);

	start = clock();
	printf("%s\n", "start expanding..");

    if(success) success = ifs.expand(istrm,saveWrlLevels);
    if(success) success = istrm.close();
	
	delete [] inFname;
  }

  if(!success) error("reading and/or expanding input file");

  // write to output file
  {
    int n = strlen(outDir)+strlen(baseFname)+12;
    outFname = new char[n];
    if(!strcmp(outDir,"."))
      sprintf(outFname,"%s-E.wrl",baseFname);
    else
      sprintf(outFname,"%s/%s-E.wrl",outDir,baseFname);

    success = ifs.write(outFname);
//	success = ifs.writeOrders(outFname);

    delete [] outFname;
  }

  if(!success) error("writing output file");
	end = clock();
	double duration = ((double)(end-start))/CLK_TCK;
	printf("running time = %f\n", duration);

  printf("done!!!\n");

  return 0;
}

