//-------------------------------------------------------------------------
// 
// Copyright (C) IBM Corporation & Samsung Electronics Co. LTD., 1996, 1997, 1998, 1999
// Licensed Materials, Program Property of IBM & Samsung.
// 
// This software module is an implementation of a part of one or more
// MPEG-4 tools as specified by the MPEG-4 standard (ISO/IEC 14496).
// ISO/IEC gives users of the MPEG-4 standard (ISO/IEC 14496) free
// license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4
// standard (ISO/IEC 14496).  Those intending to use this software module
// in hardware or software products are advised that its use may infringe
// existing patents.  The original developer of this software module and
// his/her company, the subsequent editors and their companies, and
// ISO/IEC have no liability for use of this software module or
// modifications thereof in an implementation.  Copyright is not released
// for non MPEG-4 standard (ISO/IEC 14496) conforming products.  IBM
// Corp. & Samsung Electronics Co. LTD retains full right to use the code 
// for his/her own purpose, assign or donate the code to a third party and
// to inhibit third parties from using the code for non MPEG-4 standard 
// (ISO/IEC 14496) conforming products. This copyright notice must be included
// in all copies or derivative works.
// 
//  AUTHORS:
//    Gabriel Taubin
//    Andre Gueziec
//    Claudio Silva
//    Ioana Martin
//  
//  IBM T. J. Watson Research Center
//  P.O.Box 704
//  Yorktown Heights, New York 10598
//  
//  BASED ON EARLIER IMPLEMENTATION OF
//  VRML COMPRESSED BINARY FORMAT PROPOSAL BY:
//    Gabriel Taubin
//    Bill Horn
//    Francis Lazarus
//
//  Functionality : Error Resilience
//
//  Sungjin Kim  (sunjin@sait.samsung.co.kr)
//  Mahn-Jin Han  (manjini@sait.samsung.co.kr)
//  Mun-Sup Song (mssong@sait.samsung.co.kr)  
//
//  Signal Processing Lab. Samsung A.I.T. 
//-------------------------------------------------------------------------


#include "MPG_Lex.h"
#include "MPG_Heap.h"

//---------------------------------------------------------------------------
//
//   MPG_Heap_EXT::_switch
//
//---------------------------------------------------------------------------

void MPG_Heap_EXT::_switch(int i, int j)
{
  if(i!=j && i<_n() && j<_n())
    {
      Key key = _key[i];  _key[i]  = _key[j];  _key[j] = key;
      int   pos = _invp[i]; _invp[i] = _invp[j]; _invp[j] = pos;
      _perm[_invp[i]] = i;
      _perm[_invp[j]] = j;
    }
}

//---------------------------------------------------------------------------
//
//   MPG_Heap_EXT::_down
//
//---------------------------------------------------------------------------

void MPG_Heap_EXT::_down(int father)
{
  if(0<=father && father<_last-1)
    {
#define LEFT_SON(i)  2*(i)+1
#define RIGHT_SON(i) 2*(i)+2
      int left,right,child;
      while((left=LEFT_SON(father))<_last)
	{
	  child = father;
	  if(_less(left,child))
	    child = left;
	  if((right=RIGHT_SON(father))<_last && _less(right,child))
	    child = right;
	  if(child==father)
	    break;
	  _switch(father,child);
	  father = child;
	}
#undef LEFT_SON
#undef RIGHT_SON
    }
}

//---------------------------------------------------------------------------
//
//   MPG_Heap_EXT::_up
//
//---------------------------------------------------------------------------

void MPG_Heap_EXT::_up()
{
  int father;
  int child = _last-1;
  while(child>0 && !_less((father=(child-1)/2),child))
    { _switch(father,child); child = father; }
}

//---------------------------------------------------------------------------
//
//   MPG_Heap_EXT::MPG_Heap_EXT
//
//---------------------------------------------------------------------------

MPG_Heap_EXT::MPG_Heap_EXT(KeyOp LT, KeyOp EQ, int n):_LT(LT),_EQ(EQ),_last(0)
{
  if(n>0)
    {
      _key.insert(_key.end(),(size_t)n,(Key)0);
      _perm.insert(_perm.end(),(size_t)n,-1);
      _invp.insert(_invp.end(),(size_t)n,-1);
      int i;
      for(i=0;i<n;i++)
	_perm[i] = _invp[i] = i;
    }
}

//---------------------------------------------------------------------------
//
//   MPG_Heap_EXT::add
//
//---------------------------------------------------------------------------

int MPG_Heap_EXT::add(Key key)
{
  int last = _last++;
  if(last>=_n())
    {
      int n = _n();
      _key.push_back(key);
      _perm.push_back(n);
      _invp.push_back(n);
    }
  else
    _key[last] = key;
  int indx = _invp[last];
  _up();
  return indx;
}
