//-------------------------------------------------------------------------
// 
// Copyright (C) IBM Corporation & Samsung Electronics Co. LTD., 1996, 1997, 1998, 1999
// Licensed Materials, Program Property of IBM & Samsung.
// 
// This software module is an implementation of a part of one or more
// MPEG-4 tools as specified by the MPEG-4 standard (ISO/IEC 14496).
// ISO/IEC gives users of the MPEG-4 standard (ISO/IEC 14496) free
// license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4
// standard (ISO/IEC 14496).  Those intending to use this software module
// in hardware or software products are advised that its use may infringe
// existing patents.  The original developer of this software module and
// his/her company, the subsequent editors and their companies, and
// ISO/IEC have no liability for use of this software module or
// modifications thereof in an implementation.  Copyright is not released
// for non MPEG-4 standard (ISO/IEC 14496) conforming products.  IBM
// Corp. & Samsung Electronics Co. LTD retains full right to use the code 
// for his/her own purpose, assign or donate the code to a third party and
// to inhibit third parties from using the code for non MPEG-4 standard 
// (ISO/IEC 14496) conforming products. This copyright notice must be included
// in all copies or derivative works.
// 
//  AUTHORS:
//    Gabriel Taubin
//    Andre Gueziec
//    Claudio Silva
//    Ioana Martin
//  
//  IBM T. J. Watson Research Center
//  P.O.Box 704
//  Yorktown Heights, New York 10598
//  
//  BASED ON EARLIER IMPLEMENTATION OF
//  VRML COMPRESSED BINARY FORMAT PROPOSAL BY:
//    Gabriel Taubin
//    Bill Horn
//    Francis Lazarus
//
//  Functionality : Error Resilience
//
//  Sungjin Kim  (sunjin@sait.samsung.co.kr)
//  Mahn-Jin Han  (manjini@sait.samsung.co.kr)
//  Mun-Sup Song (mssong@sait.samsung.co.kr)  
//
//  Signal Processing Lab. Samsung A.I.T. 
//-------------------------------------------------------------------------

//#define ADD_VISUAL_HEADER	
#ifdef ADD_VISUAL_HEADER
const char visual_object_sequence_start_code	= 0xB0;
const char visual_object_sequence_end_code		= 0xB1;
const char visual_object_start_code				= 0xB5;
#endif 

#pragma warning(disable:4786)

#include <stdio.h>
#include <stdlib.h>
#include "MPG_IfsQ_EXT_C.h"
#include "MPG_ClustSurf.h"
#include "MPG_MeshTs.h"
#include "MPG_MeshFs.h"
#include "MPG_IfsEdit_EXT.h"
#include "MPG_QFac.h"
#include "MPG_Global.h"
#include "MPG_VGraph.h"
#include "MPG_SPolygon.h"
#include "MPG_Stitcher.h"
#include "MPG_Perm.h"
#include "MPG_Vector.h"
#include "MPG_ER.h"   

#ifndef M_PI
#define M_PI    3.14159265
#endif
#define boolStr(b) ((b)==true)?"true":"false"

extern char* baseFname;
extern char* inExt;
extern char* inDir;
extern char* outDir;
extern char* inFname;
extern char* outFname;

int representBit(int numVertices)
{
	int bits = 1;
	int exp = 1;
	while((exp*=2) < numVertices) bits++;
	
	return bits;
}
//DK
int __cdecl compare(const void *elem1, const void *elem2)
{
    if(*(int*)elem1 < *(int*)elem2) return -1;
    if(*(int*)elem1 > *(int*)elem2) return 1;
    return 0;
}
//~DK
//---------------------------------------------------------------------------
//
//   MPG_IfsQ_EXT_C::compress
//
//---------------------------------------------------------------------------
bool MPG_IfsQ_EXT_C::compress(MPG_OutputStream& ostrm,
						  MPG_OutputStream& t_ostrm,  
                          int  bitsPerCoord,
                          int  coordPrediction,
                          int  bitsPerNormal,
                          int  normalPrediction,
                          int  bitsPerColor,
                          int  colorPrediction,
                          int  bitsPerTexCoord,
                          int  texCoordPrediction,
						  int  erMode,    
						  int  packetSize,
						  int  boundaryPrediction,
                          bool hierarchicalMode,
                          bool saveClsFile,
                          bool saveWrlLevels,
						  //DMLAB (200501) - ADD
						  int img_flag,
						  int img_u,
						  int img_v,
						  //~DMLAB (200501)
						  bool vertex_order_flag,
						  bool face_order_flag,
						  bool face_order_per_CC_flag,
						  bool vertex_order_per_CC_flag)
{
	bool has_stitches = false;
	// determine if mesh has isolated vertices
	bool hasIsolatedVertices = false;
	{
		int nV = getNumberOfCoord();	// dmlab : number of vertex is the number of coordinate / 3?? why??
		vector<bool> used((size_t)nV,false);	// dmlab : maybe initialize "all coordinate are not used" 
		//         to check already used coordinate 
		vector<int>& coordIndex = getCoordIndex();  // dmlab : get the address of coordindex vector
		int i;
		for(i=0;i<coordIndex.size();i++)
			if(coordIndex[i]>=0)
			{
				if(coordIndex[i]>=nV) return false;
				used[coordIndex[i]] = true;
			}
			for(i=0;i<nV;i++)
				if(used[i]==false)			// dmlab : isolated vertices are not useless. so must remove them. 
					// if it is not in the coordindex, they are isolated vertices.
				{ hasIsolatedVertices = true; break; }
	}
	// remove isolated vertices
	if(hasIsolatedVertices)
    {
		MPG_IfsMesh_EXT mesh(*this);
		MPG_IfsEdit_EXT ifsEdit(mesh);
		if(ifsEdit.removeIsolatedVertices()==false) return false;
    }
	
	// deal with non-manifolds without isolated vertices
	vector<int>  v_cluster_enc;
	vector<int>  v_cluster_dec;
	vector<int>  v_stitch_enc;
	{
		MPG_IfsMesh_EXT mesh(*this);
		if(hierarchicalMode==true &&
			mesh.isTriangulated()==false)
			return false;

		if(mesh.isOrientedManifold()==false)
		{
			has_stitches = true;
			if(hierarchicalMode==true)
				return false;
			MPG_IfsEdit_EXT ifsEdit(mesh);
			if(ifsEdit.convertToOrientedManifold(v_stitch_enc)==false)
				return false;
			
			for(int i = 0; i < v_stitch_enc.size(); i++)
			{
				v_cluster_enc.push_back(v_stitch_enc[i]);				
			}
			
		}
		vector<bool> fmk;
		if(mesh.markFacesWithDuplicateVertices(fmk))
			return false;
	}
	// DMLAB temporary changed v_cluster_enc
	
	int duplicationCnt = 0;	// to figure out the number of duplicated vertices	
	{
		int max = 0;
		
		for(int dm = 0; dm < v_cluster_enc.size(); dm++)
			if (max < v_cluster_enc[dm])
				max = v_cluster_enc[dm];
			char* isused = new char[v_cluster_enc.size()];
			memset(isused, 0, v_cluster_enc.size());
			int count = 1;
			for(dm = 0; dm < v_cluster_enc.size(); dm++)
			{
				isused[v_cluster_enc[dm]]++;
				if (isused[v_cluster_enc[dm]] == 2)
				{
					duplicationCnt++;
					isused[v_cluster_enc[dm]]--;
					v_cluster_enc[dm] = max + count;
					count++;
				}
			}
	}
	
	//	printf("\nfact vertex count = %d\n", v_cluster_enc.size() - duplicationCnt);
	//	printf("\nincreased vertex = %d\n", duplicationCnt);
	
	// generate hierarchy of levels of detail
	int nLevelsOfDetail=1;
	MPG_ClustSurf csrf;
	if(hierarchicalMode==true)
	{
		// decompose into a base mesh and one or more forest splits
		csrf = *this; // uses MPG_ClustSurf::operator=(MPG_IfsQ_EXT& ifs)
		// simplify
		if(csrf.pfsGenerate(true)==false)
			return false;
		nLevelsOfDetail = csrf.getNumberOfLevels();
		if(saveClsFile==true)
		{
			int nL = nLevelsOfDetail; 
			int n = strlen(outDir)+strlen(baseFname)+12;
			outFname = new char[n];
			if(!strcmp(outDir,"."))
				sprintf(outFname,"%s-C%02d.cls",baseFname,nL);
			else
				sprintf(outFname,"%s/%s-C%02d.cls",outDir,baseFname,nL);
			bool success = csrf.write(outFname);
			delete [] outFname;
			outFname = NULL;
		}
		if(saveWrlLevels==true)
		{
			int n = strlen(outDir)+strlen(baseFname)+12;
			outFname = new char[n];
			MPG_Ifs_EXT ifs;
			int l,ll;
			for(l=0;l<nLevelsOfDetail;l++)
				if(csrf.getIfs(ifs,l)==true)
				{
					ll = nLevelsOfDetail-l-1;
					if(!strcmp(outDir,"."))
						sprintf(outFname,"%s-C%02d.wrl",baseFname,ll);
					else
						sprintf(outFname,"%s/%s-C%02d.wrl",outDir,baseFname,ll);
					bool success = ifs.write(outFname);
				}
				delete [] outFname;
				outFname = NULL;
		}
		// set *this to simplest model;
		if(nLevelsOfDetail>1)
		{
			// note that MPG_ClustSurf::getIfs(MPG_Ifs&,int)
			// does NOT reset the quantizer pointers of *this
			// which is exactly what we want
			if(csrf.getIfs(*this,nLevelsOfDetail-1)==false)
				return false;
		}
	}
	
	// compress base mesh
	MPG_MeshTs mesh(getCoordIndex());
	int nCC = mesh.getNumberOfConnectedComponents();
	printf("# of CC : %d\n",nCC);
	// create property headers and quantizers if not yet created
	MPG_Prediction prediction = PP_NONE;
	//DK
	int *array_cnt = new int[nCC];
	int *array_accumul_cnt=new int[nCC];
	//~DK
	
	// valid combinations of property/binding/prediction
	//
	//              | PER_VERTEX | PER_FACE   | PER_CORNER    |
	//   -----------+------------+------------+---------------+
	//   COORD      |     X      |            |               |
	//   NORMAL     |     X      |     X      |     X         |
	//   COLOR      |     X      |     X      |     X         |
	//   TEXCOORD   |     X      |            |     X         |
	//
	//              | NONE       | TREE       | PARALLELOGRAM |
	//   -----------+------------+------------+---------------+
	//   PER_VERTEX |     NS     |     NS     |     X         |
	//   PER_FACE   |     NS     |     X      |               |
	//   PER_CORNER |     NS     |     X      |               |
	
	MPG_Binding coordBinding = getCoordBinding();
	prediction = (MPG_Prediction)coordPrediction;
	MPG_Property pCoord(PT_COORD,coordBinding,prediction);
	MPG_BBoxQ*   coordQuant = getCoordQuantizer(bitsPerCoord);
	bitsPerCoord = coordQuant->getBitsPerCoordinate();
	pCoord.setBBoxQ(coordQuant);
	pCoord.setDefaultLambda();
	
	MPG_Binding normalBinding   = getNormalBinding();
	prediction = (MPG_Prediction)normalPrediction;
	MPG_Property pNormal(PT_NORMAL,normalBinding,prediction);
	MPG_NormalQ* normalQuant   = getNormalQuantizer(bitsPerNormal);
	bitsPerNormal = normalQuant->getBitsPerNormal();
	pNormal.setNormalQ(normalQuant);
	pNormal.setDefaultLambda();
	
	MPG_Binding colorBinding    = getColorBinding();
	prediction = (MPG_Prediction)colorPrediction;
	MPG_Property pColor(PT_COLOR,colorBinding,prediction);
	MPG_BBoxQ*   colorQuant    = getColorQuantizer(bitsPerColor);
	bitsPerColor = colorQuant->getBitsPerCoordinate();
	pColor.setBBoxQ(colorQuant);
	pColor.setDefaultLambda();
	
	MPG_Binding texCoordBinding = getTexCoordBinding();
	prediction = (MPG_Prediction)texCoordPrediction;
	MPG_Property pTexCoord(PT_TEXCOORD,texCoordBinding,prediction);
	MPG_BBoxQ*   texCoordQuant = getTexCoordQuantizer(bitsPerTexCoord);
	bitsPerTexCoord = texCoordQuant->getBitsPerCoordinate();
	pTexCoord.setBBoxQ(texCoordQuant);
	pTexCoord.setDefaultLambda();
	
	// decompose each connected component into a vertex graph
	// and a simple polygon, and collect quantized prediction
	// errors in the triData array in decoder order,
	// i.e., in the order of traversal of the simple polygon.
	// the collected quantized prediction errors are later
	// entropy encoded during a second simple polygon traversal
	// by sp.encodeTriangleData()
	
	vector<MPG_SPolygon*> vpsp;
	vector<MPG_VGraph*>   vpvg;
	vector<int>           v_visited;
	MPG_Stitches          v_stitches_dec;
	
	// these arrays are here because they are used in hiearchical mode
	int *face_array_cnt = new int[nCC];
    bool Order_mode=(vertex_order_flag || face_order_flag);
	
	vector<int> v_traversal;
	vector<int> v_traversal_inv;
	vector<int> f_traversal;
	vector<int> f_index;
	vector<int> c_index;
	MPG_BranchForest* root_btf = new MPG_BranchForest;
	vector <int> qCoord1;
	vector <int> qNormal1;
	vector <int> qColor1;
	
	vector <int> qTexCoord1;
	{
		if(coordBinding!=PB_NONE)
			coordQuant->quantize(getCoord(),qCoord1);
		if(normalBinding!=PB_NONE)
		{
			vector<int>& nIndex = getNormalIndex();
			if(nIndex.size()>0)
				normalQuant->quantize(getNormal(),nIndex,qNormal1);
			else
				normalQuant->quantize(getNormal(),qNormal1);
		}
		if(colorBinding!=PB_NONE)
		{
			vector<int>& cIndex = getColorIndex();
			if(cIndex.size()>0)
				colorQuant->quantize(getColor(),cIndex,qColor1);
			else
				colorQuant->quantize(getColor(),qColor1);
		}
		if(texCoordBinding!=PB_NONE)
		{
			vector<int>& tcIndex = getTexCoordIndex();
			//DMLAB (200501) - UPDATE
			if(tcIndex.size()>0)
				texCoordQuant->texquantize(getTexCoord(),tcIndex,qTexCoord1,img_flag,img_u,img_v);
			else
				texCoordQuant->texquantize(getTexCoord(),qTexCoord1,img_flag,img_u,img_v);
			
			pTexCoord.delta_flag = texCoordQuant->delta_flag;
			if (texCoordQuant->delta_flag) {
				pTexCoord.delta_u = texCoordQuant->delta_u;
				pTexCoord.delta_v = texCoordQuant->delta_v;
				
				bitsPerTexCoord = texCoordQuant->getBitsPerCoordinate();
				texCoordQuant = getTexCoordQuantizer(bitsPerTexCoord);
				pTexCoord.setBBoxQ(texCoordQuant);
			}
			//~DMLAB (200501)
		}
		else  //DMLAB ory 2006 07
			pTexCoord.delta_flag=false;
		//~DMLAB ory
		memset(face_array_cnt, 0, nCC * sizeof(int));
		
		vector<int> v_father_enc; // used to build v_father_dec
		if(mesh.representConnectivity(vpsp, vpvg, pCoord,	pNormal, 
			pColor, pTexCoord, qCoord1, qNormal1,
			qColor1, qTexCoord1, v_visited, v_cluster_enc,
			v_father_enc, v_traversal, f_traversal,
			f_index, c_index, root_btf, face_array_cnt)==false)
			return false;
		
		//	  printf("v_traversal.size():%d\n",v_traversal.size());
		if(MPG_Perm::makeInvPerm(v_traversal,v_traversal_inv)==false)
			return false;
		
		// construct v_cluster_dec 
		// dmlab stitching process _ lsw
		int i, max = v_cluster_enc.size();
		for(i=0;i<max;i++)
			v_cluster_dec.push_back(v_cluster_enc[v_traversal[i]]);

		

		/*
		if (!vertex_order_flag && has_stitches)
		{
			int cnt = 0;
			int* targetPos = new int[v_stitch_enc.size()];
			memset(targetPos, -1, v_stitch_enc.size() * 4);
			int uplus = 0;
			for(i = 0; i < duplicationCnt; i++)
			{
				int index;
				
				for(int q = v_cluster_dec.size(); q > 0; q--)
					if (v_cluster_dec[q-1] == (max - duplicationCnt + i))
					{
						index = q - 1;
						break;
					}

					if ((max - duplicationCnt + i) == 128) 
						printf("111");
					
					int temp = v_stitch_enc[max - duplicationCnt + i - uplus];
					if (index < (max - duplicationCnt + i - uplus))
					{
						targetPos[index] = 1;
						int startNum = v_stitch_enc[index];
						for(int j = index + 1; j <= max - duplicationCnt + i - uplus; j++)
						{						
							if (targetPos[j] == -1)
								v_stitch_enc[j] = ++startNum;
							else ++startNum;

								
						}
						for(int o = 0; o < v_cluster_dec.size(); o++)
							if (v_cluster_dec[o] == temp)
							{ 
								v_stitch_enc[index] = o;
								break;
								
							}

					}
					else if (index == (max - duplicationCnt + i - uplus))
						targetPos[index] = 1;
					else if (index > (max - duplicationCnt + i - uplus))
					{
						for(int j = (max - duplicationCnt + i - uplus); j < index ; j++)
						{
							v_stitch_enc[j] = v_stitch_enc[j+1];
							//targetPos[j-1] = 1;
						}
						targetPos[index] = 1;
						
						for(int o = 0; o < v_cluster_dec.size(); o++)
							if (v_cluster_dec[o] == temp)
							{ 
								v_stitch_enc[index] = o;
								break;
								
							}
						uplus++;

					}
				
			}

		delete targetPos;	
		}
		*/
			
		
		printf("# of vertex = %d\n", v_traversal.size());
		printf("# of face = %d\n", f_traversal.size());
		
		// AG 2/16/99: 
		// if v_cluster_dec[1]   = 256 and 
		//    v_cluster_dec[256] = 256 this is bad if 1 and 256 belong to
		// different connected components: the coordinates of 1 will be encoded
		// because 256 is not visited yet whereas they shouldn't (duplicate)
		// and this will crash the decoder which expects no duplication
		// so among all elements that cluster to the same location,
		// the minimum vertex index must be used in v_cluster_dec
		{
			int n = v_cluster_dec.size();
			vector <int> min_cluster_index((size_t) n);
			for (i=n-1;i>=0;i--) 
				min_cluster_index[v_cluster_dec[i]] = i;      
			for (i=0;i<n;i++) 
				v_cluster_dec[i] = min_cluster_index[v_cluster_dec[i]];
		}
		// construct v_father_dec
		vector<int> v_father_dec;
		for(i=0;i<v_father_enc.size();i++)
			v_father_dec.push_back(v_traversal_inv[v_father_enc[v_traversal[i]]]);
		vector <int> nVertCC;
		
		// we have to make sure that the components are in the decoder order
		for(i=0;i<nCC;i++) 
		{
			MPG_VGraph&   vg = *(vpvg[i]);
			nVertCC.push_back(vg.getNumberOfVertices());
		}
		for(i=1;i<nCC;i++) nVertCC[i] += nVertCC[i-1];
		
		// represent the v_cluster_enc as a sequence of stitch commands
		/* DELETE stitch methods
		MPG_Stitcher mc;
		if(mc.represent(nVertCC, v_father_dec, v_cluster_dec, v_stitches_dec)==false)
		return false;
		*/
  }
  // VisualObjectSequence() {
  //	visual_object_sequence_start_code
  //	profile_and_level_indication
  //    VisualObject() {
  //	    visual_object_start_code
  //	    is_visual_object_identifier
  //	    if (is_visual_object_identifier) {
  //	       visual_object_verid
  //	        visual_object_priority
  //	    }
  //	    visual_object_type
  //	    if (visual_object_type == "3D mesh ID") 
  //           3D_Mesh_Object () {
  //               3D_MO_start_code
  //	           3D_Mesh_Object_Header()
  //	           do {
  //		           3D_Mesh_Object_Layer()
  //	           } while (nextbits_bytealigned() == 3D_MOL_start_code)
  //           }
  //    }
  //    visual_object_sequence_end_code
  // }
  
  // open an output bit stream
  MPG_OutputBitStream obstrm(ostrm);
#ifdef ADD_VISUAL_HEADER
  {
	  int profile_and_level_indication = 0xFF; //MJH 062600: 0xF1;
	  int is_visual_object_identifier = 0x01;
	  int visual_object_verid = 0x0F; //MJH 062600: 0x02;
	  int visual_object_priority = 0x01;
	  int visual_object_type = 0x05;
	  if(ostrm.putVisualStartCode(visual_object_sequence_start_code)==false) return false;
	  if(obstrm.putMBitInt(8,profile_and_level_indication)==false) return false;
	  if(ostrm.putVisualStartCode(visual_object_start_code)==false) return false;
	  if(obstrm.putMBitInt(1,is_visual_object_identifier)==false) return false;
	  if(is_visual_object_identifier) { //MJH 062600: added this condition
		  if(obstrm.putMBitInt(4,visual_object_verid)==false) return false;
		  if(obstrm.putMBitInt(3,visual_object_priority)==false) return false;
	  }
	  if(obstrm.putMBitInt(4,visual_object_type)==false) return false;
  }
#endif //ADD_VISUAL_HEADER
  
  // write 3D_MO_start_code
  if(obstrm.putMOStartCodeBitInt(MPG_Global::MO_start_code)==false)
	  return false;
  
  // begin encode 3D_Mesh_Object_Header()  
  {      
	  // 3D_Mesh_Object_Header() {
	  //   ccw
	  //   convex
	  //   solid
	  //   creaseAngle
	  //   coord_header()
	  //   normal_header()
	  //   color_header()
	  //   texCoord_header()
	  //   ce_SNHC
	  //   if(ce_SNHC==1)
	  //      ce_SNHC_header()
	  // }
	  if(obstrm.putMOBit(_ccw               )==false) return false;
	  if(obstrm.putMOBit(_convex            )==false) return false;
	  if(obstrm.putMOBit(_solid             )==false) return false;
	  int cAng = (int)((_creaseAngle*32.0)/M_PI);
	  if(cAng<0 || cAng>=64) return false;
	  if(obstrm.putMOBitInt(6,cAng)==false) return false;
	  
	  // coord_header() {
	  //   coord_binding       <--- must be == 1 (PER_VERTEX)
	  //   coord_bbox
	  //   if(coord_bbox) {
	  //     coord_xmin
	  //     coord_ymin
	  //     coord_zmin
	  //     coord_size
	  //   }
	  //   coord_quant
	  //   coord_pred_type
	  //   if(coord_pred_type==2) {
	  //     coord_nlambda
	  //     for(i=1;i<coord_nlambda;i++)
	  //       coord_lambda[i]
	  //   }
	  // }
	  if(obstrm.putMBitInt(2,coordBinding)==false) return false;
	  if(coordBinding!=PB_NONE) {
		  if(coordBinding!=PB_PER_VERTEX) return false;
		  if(pCoord.encode(obstrm)==false) return false;
	  }
	  
	  // normal_header() {
	  //    normal_binding
	  //    if(normal_binding!=0) {
	  //       normal_bbox       <--- must be == 0
	  //       normal_quant
	  //       normal_pred_type
	  //       if(normal_pred_type==2) {
	  //          normal_nlambda
	  //          for(i=1;i<normal_nlambda;i++)
	  //              normal_lambda[i]
	  //       }
	  //    }
	  // }
	  if(obstrm.putMBitInt(2,normalBinding)==false) return false;
	  if(normalBinding!=PB_NONE)
	  {
		  if(pNormal.encode(obstrm)==false) return false;
	  }
	  
	  // color_header() {
	  //    color_binding
	  //    if(color_binding!=0) {
	  //        color_bbox
	  //        if(color_bbox) {
	  //           color_rmin
	  //           color_gmin
	  //           color_bmin
	  //           color_size
	  //        }
	  //        color_quant
	  //        color_pred_type
	  //        if(color_pred_type==2) {
	  //            color_nlambda
	  //            for(i=1;i<color_nlambda;i++)
	  //                color_lambda[i]
	  //        }
	  //    }
	  // }
	  if(obstrm.putMBitInt(2,colorBinding)==false) return false;
	  if(colorBinding!=PB_NONE)
	  {
		  if(pColor.encode(obstrm)==false) return false;
	  }
	  
	  // texCoord_header() {
	  //    texCoord_binding     <--- must be != 2 (PER_FACE)
	  //    if(texCoord_binding!=0) {
	  //       texCoord_bbox
	  //       if(texCoord_bbox) {
	  //          texCoord_umin
	  //          texCoord_vmin
	  //          texCoord_size
	  //        }
	  //        texCoord_quant
	  //        texCoord_pred_type
	  //        if(texCoord_pred_type==2) {
	  //           texCoord_nlambda
	  //           for(i=1;i<texCoord_nlambda;i++)
	  //               texCoord_lambda[i]
	  //        }
	  //    }
	  // }

	  int delta_u=0,delta_v=0;
	  if(obstrm.putMBitInt(2,texCoordBinding)==false) return false;
	  if(texCoordBinding!=PB_NONE)
	  {
		  if(texCoordBinding==PB_PER_FACE) return false;
		  //DMLAB (200501) - UPDATE
		  if(pTexCoord.texencode(obstrm)==false) return false;
		  delta_u=pTexCoord.delta_u;
		  delta_v=pTexCoord.delta_v;
		  //~DMLAB (200501)
	  }
	
	  //DMLAB ory 2006 07
	  //  3DMC_extension_header
	  bool _3DMC_extension_header=Order_mode || pTexCoord.delta_flag;
	  if(obstrm.putBit(_3DMC_extension_header)==false) return false;
	  if(_3DMC_extension_header)
	  {
		  if (Order_mode)
		  {
			  if(obstrm.putMBitInt(4, ORDER_MODE)==false)return false;//Order_mode = 0000

			  if(obstrm.putBit(vertex_order_flag)==false) return false;
			  if(vertex_order_flag) 
				  if(obstrm.putBit(vertex_order_per_CC_flag)==false) return false;
			  if(obstrm.putBit(face_order_flag)==false) return false;
			  if(face_order_flag) 
				  if(obstrm.putBit(face_order_per_CC_flag)==false) return false;
		  }
		  
		  // Adaptive_quant_texCoord_mode
		  if(pTexCoord.delta_flag)
		  {
			  obstrm.putMBitInt(4, ADAPTIVE_QUANT_TEXCOORD_MODE);//Adaptive_quant_texCoord_mode = 0011
			  if(obstrm.putMBitInt(16, delta_u)==false) return false;
			  if(obstrm.putMBitInt(16, delta_v)==false) return false;
		  }
		  
		  //Escape Code = 1111
		  obstrm.putMBitInt(4, ESCAPE_MODE);
	  }
	  //~DMLAB ory
	  
	  obstrm.flush();  // flush output buffer
  } // end encode 3D_Mesh_Object_Header()
  
  // begin encoding 3D_Mesh_Object_Layer ()
  int mol_id = 0;
  {
	  // 3D_Mesh_Object_Layer () {
	  //	3D_MOL_start_code
	  //	mol_id
	  //	if (mol_id == 0)
	  //		3DMeshObject_Base_Layer()
	  //	else
	  //		3DMeshObject_Extension_Layer()
	  // }
	  // write 3D_MOL_start_code
	  if(ostrm.putStartCode(MPG_Global::MOL_start_code)==false)
		  return false;
	  {
		  // open an output bit stream
		  MPG_OutputBitStream obstrm(ostrm);
		  // write mol_id
		  if(obstrm.putMBitInt(8,mol_id)==false) return false;
		  mol_id++;
		  // flush output buffer
		  obstrm.flush();
	  }
		  MPG_OutputBitStream t_obstrm(t_ostrm);  
		  MPG_QFac t_codec(t_ostrm, erMode);
		  MPG_OutputBitStream obstrm(ostrm);  
		  MPG_QFac codec(ostrm, erMode);  	
		  MPG_OutputStream vgostrm;  
		  MPG_OutputBitStream vgobstrm(vgostrm);  
		  MPG_QFac vgcodec(vgostrm, erMode);
		  MPG_OutputStream spostrm;  
		  MPG_OutputBitStream spobstrm(spostrm);  
		  MPG_QFac spcodec(spostrm, erMode);
		  
		  int header_bits=0, totalbits = 0, restbits = packetSize * 8;
		  int cpass=0, rootbits=0, tribits=0;
		  cpass=(erMode==1) ? 0: 1;
		  if(coordBinding==PB_PER_VERTEX) {
			  rootbits = 100;
			  tribits = bitsPerCoord;
		  }
		  if(normalBinding!=PB_NONE) {
			  rootbits += 50;
			  tribits += bitsPerNormal;
		  }
		  if(colorBinding!=PB_NONE) {
			  rootbits += 50;
			  tribits += bitsPerColor;
		  }
		  if(texCoordBinding==PB_PER_CORNER || texCoordBinding==PB_PER_VERTEX) {
			  rootbits += 50;
			  tribits += bitsPerTexCoord;
		  }
		  header_bits = ostrm.size()*8; 
		  MPG_ER2 ccbit;               
		  
		  do {  
			  // stitch encoding stack
			  vector<int>  stk;
			  int icc = 0;
			  vector<int> nV0;
			  vector<int> nV1;
			  nV0.push_back(0);
			  
			  typedef vector<int> vec_array;
			  vector<vec_array> v_loop_cc;
			  v_loop_cc.clear();
			  
			  int GiC=0, GiF=-1;		//MJH 001011 : GiF changed from 0 to -1 for per face
			  int prevCR=0, prevCL=0;	//MJH 001013 : added in case of partitioning at non-polygon edge
			  totalbits = header_bits;  
			  restbits = (packetSize * 8)-totalbits;
			  MPG_PacketQ_C codap(erMode,packetSize,totalbits,restbits);
			  MPG_BranchForest* cur_btf=root_btf;	
			  
			  // connected_component() {
			  //	  vertex_graph()
			  //	  qf_decode(has_stitches, has_stitches_context)
			  //	  if (has_stitches == '1')
			  //		  stitches()
			  //	  triangle_tree()
			  //	  triangle_data()
			  // } 
			  // decomposition is done at the beginning of the method,
			  // after creating the mesh
			  
			  // DMLAB (200503) -ADD
			  int TCbits=0, tempInt=0;
			  long sumTCbits=0;
			  // ~DMLAB (200503)
			  
			  for(icc=0;icc<nCC;icc++)
			  {
				  if(!cpass) {  
					  //encoding connected_component() with partition_type_0
					  codap.erMode=0;
					  MPG_VGraph&   vg = *(vpvg[icc]);
					  nV1.push_back(nV0[icc] + vg.getNumberOfVertices());
					  MPG_SPolygon& sp = *(vpsp[icc]); 
					  
					  vec_array tmp_v_loop_cc;
					  {
						  MPG_Loops loop;
						  vg.toLoop(loop);
						  loop.getLoop(0, tmp_v_loop_cc);
					  }
					  
					  // encode vertex_graph()
					  codap.vg_init(ccbit.partition_type);
					  codap.vg_sc_enable=1;
					  if(icc>0) codap.vg_sc_enable=0;
					  int vg_bits=0, vg_mobl_bits=0;
					  if(codap.vg_StartCode(vgostrm, t_ostrm, vgobstrm, t_obstrm,
						  vgcodec, t_codec, vg_mobl_bits)==false) return false;
					  MPG_Global::vg_restore();    
					  vgcodec.cbits=0;
					  if(vg.encode(vgcodec) == false) return false;
					  vg_bits+=vgcodec.cbits;
					  vgcodec.cbits=0;
					  
					  // encoding Stitches()
					  //v_stitches_dec.encode(nV0[icc],nV1[icc],stk,vgcodec);
					  //dmlab temporary alternative method
					  vgcodec.encode(has_stitches,MPG_Global::stitch_has);	
					  vg_bits+=vgcodec.cbits; 
					  vgcodec.cbits=0;
					  ccbit.putVGbits(vg_bits);  
					  
					  //write VG data to output buffer and 
					  //initialize variables for GF & buffer 
					  codap.vg_packetization(vgostrm,t_ostrm,vgobstrm,t_obstrm,
						  vgcodec,t_codec, icc, ccbit);
					  
					  sp.DetermineDirection();   //define traversal order with td_orientation 
					  int next=0;
					  vector<int> Grstk((size_t)1, -1);  //stack run value for subtree partitioning
					  vector<int> Gistk((size_t)1, -1);  //stack loop index for subtree partitioning
					  codap.sp_init(tmp_v_loop_cc.size(), rootbits, tribits);
					  cur_btf = cur_btf->GetNextBTree();
					  MPG_BranchTree* cur_bt = cur_btf->GetRootBTree();	
					  cur_bt->SetCurDir(LEFT_BRANCH);
					  int sp_mobl_bits=0, Gdepth=0;
					  if(codap.sp_StartCode(spostrm, t_ostrm,
						  spobstrm, t_obstrm, spcodec, t_codec, sp_mobl_bits)==false) return false;
					  MPG_Global::sp_restore();
					  spcodec.cbits=0;
					  
					  // encode triangle_tree()
					  if(sp.encodeTriangleTree(spcodec, Grstk, Gdepth, codap) == false) return false;
					  ccbit.putTTbits(spcodec.cbits);
					  spcodec.cbits=0;
					  
					  // encode triangle_data()
					  MPG_BranchTree* tmp_bt = cur_bt;	
					  // DMLAB (200503) -UPDATE
					  if(sp.encodeTriangleData(spcodec,
						  nV0[icc],  // first vertex index of cc
						  tmp_v_loop_cc,  // local  indices
						  v_visited,     // global decoder indices
						  v_cluster_dec, // global decoder indices
						  pCoord,
						  pNormal,
						  pColor,
						  pTexCoord,
						  qCoord1,
						  qNormal1,
						  qColor1,
						  qTexCoord1,
						  v_traversal, f_index, 
						  c_index,	cur_bt,	
						  Grstk, Gistk, Gdepth, GiC, GiF,
						  prevCR, prevCL,	//MJH 001013 : added in case of partitioning at non-polygon edge
						  restbits, codap, &tempInt
						  ) == false) return false;
					  // ~DMLAB (200503)
					  ccbit.putTDbits(spcodec.cbits);
					  spcodec.cbits=0;
					  //write SP data to output buffer and 
					  //initialize variables for GF & buffer 
					  codap.sp_packetization(
						  vgostrm, spostrm, t_ostrm, vgobstrm, spobstrm, t_obstrm,
						  vgcodec, spcodec, t_codec, icc, nCC, next, Gdepth, totalbits, restbits, 
						  cur_bt, tmp_bt, ccbit);
					  nV0.push_back(nV1[icc]);  
					  if(icc==nCC-1) {
						  codap.erMode=1;
						  v_visited.erase(v_visited.begin(),v_visited.end());
                          v_visited.insert(v_visited.end(),(size_t)nV1[icc],AVAIL_PRED);  
					  }
				  } else {
					  //encoding connected_component() with partition_type_1 or 2
					  int tcc=icc;
					  if(codap.erMode) {
						  if(ccbit.next==icc) {
							  if(ccbit.determine_PT(icc, nCC, codap, ostrm, obstrm, codec)==false)
								  return false;
							  codap.vg_sc_enable=1;
						  } else {
							  ccbit.partition_type=0;
							  codap.vg_sc_enable=0;
						  }
					  } else {
						  ccbit.curr=0;
						  ccbit.next=nCC;
						  ccbit.partition_type=0;
						  codap.vg_sc_enable=1;
						  if(icc>0) codap.vg_sc_enable=0;
					  }
					  codap.vg_init(ccbit.partition_type);
					  
					  // encode vertex_graph() & Stitches()
					  do {
						  MPG_VGraph&   vg = *(vpvg[icc]);
						  nV1.push_back(nV0[icc]+vg.getNumberOfVertices());
   	                      
						  vec_array tmp_v_loop;
						  v_loop_cc.push_back(tmp_v_loop);
						  {
							  MPG_Loops loop;
							  vg.toLoop(loop);
							  loop.getLoop(0 ,v_loop_cc[icc]);
						  }
						  
						  int vg_bits=0, vg_mobl_bits=0;
						  if(codap.vg_StartCode(vgostrm, ostrm, vgobstrm, obstrm,
							  vgcodec, codec, vg_mobl_bits)==false) return false;
						  MPG_Global::vg_restore();        
						  vgcodec.init_bits(codap.totalbits, codap.restbits); 
						  
						  // encode vertex_graph()
						  if(vg.encode(vgcodec) == false) return false;
						  codap.totalbits=vgcodec.tbits;
						  codap.restbits=vgcodec.rbits;
						  vg_bits+=vgcodec.cbits;
						  
						  // encoding Stitches()
						  vgcodec.init_bits(codap.totalbits, codap.restbits); 
						  //v_stitches_dec.encode(nV0[icc],nV1[icc],stk,vgcodec);
						  //dmlab temporary alternative method
						  vgcodec.encode(has_stitches,MPG_Global::stitch_has);	
						  //obstrm.putMOBitInt(4,10);
						  
						  codap.totalbits=vgcodec.tbits;
						  codap.restbits=vgcodec.rbits;
						  vg_bits+=vgcodec.cbits;
						  
						  if(!codap.vg_pt0_enable) {
							  vgcodec.init_bits(codap.totalbits, codap.restbits); 
							  if(icc+1==ccbit.next) {
								  vgcodec.encode(1, MPG_Global::codap_last_vg_context);
								  vgcodec.finish_encode();
							  } else {
								  vgcodec.encode(0, MPG_Global::codap_last_vg_context);
							  }
							  codap.totalbits=vgcodec.tbits;
							  codap.restbits=vgcodec.rbits;
							  vg_bits+=vgcodec.cbits;
						  }
						  
						  //write VG data to output buffer and 
						  //initialize variables for GF & buffer
						  codap.vg_packetization(vgostrm,ostrm,vgobstrm,obstrm,
							  vgcodec, codec, icc, ccbit);
						  codap.vg_sc_enable=0;
						  delete vpvg[icc]; vpvg[icc] = NULL;
						  nV0.push_back(nV1[icc]);
						  icc++;
					  } while(!codap.vg_pt0_enable && (icc<ccbit.next));
					  
					  if(codap.vg_pt0_enable) {
						  if((icc-1)==ccbit.curr)
							  codap.vg_sc_enable=1;
					  } else {
						  codap.vg_sc_enable=1;
					  }
					  icc=tcc;
					  codap.vg_id_len = (codap.nvg>1) ? slog2(codap.nvg): 0;  
					  codap.sp_vg_id=0;
					  
					  // encode triangle_tree() & triangle_data()
					  do {
						  MPG_SPolygon& sp = *(vpsp[icc]); 
						  sp.DetermineDirection();   //define traversal order with td_orientation 
						  
						  int next=0, Gdepth=0;	
						  vector<int> Grstk((size_t)1, -1);  //stack run value for subtree partitioning
						  vector<int> Gistk((size_t)1, -1);  //stack loop index for subtree partitioning
						  codap.sp_init(v_loop_cc[icc].size(), rootbits, tribits);
						  totalbits = codap.totalbits;  
						  restbits = codap.restbits;
						  cur_btf = cur_btf->GetNextBTree();
						  MPG_BranchTree* cur_bt = cur_btf->GetRootBTree();	
						  cur_bt->SetCurDir(LEFT_BRANCH);
						  do {
							  int sp_mobl_bits=0;
							  if(codap.sp_StartCode(spostrm, ostrm,
								  spobstrm, obstrm, spcodec, codec, sp_mobl_bits)==false) return false;
							  
							  MPG_Global::sp_restore();
							  spcodec.init_bits(codap.totalbits, codap.restbits);  
							  
							  // encode triangle_tree()
							  if(sp.encodeTriangleTree(spcodec, Grstk, Gdepth, codap) == false) return false;
							  if(codap.sp_pt2_enable && codap.sp_sub_pt_enable) {
								  spcodec.encodePT2(codap.bdry_loopbits,codap.sp_branch_len,
									  MPG_Global::codap_branch_len_context);
							  }
							  codap.totalbits=spcodec.tbits;
							  codap.restbits=spcodec.rbits;
							  if(codap.sp_pass0 && codap.sp_pt2_enable) {
								  sp.tt_avgbits = (float)spcodec.cbits/(v_loop_cc[icc].size()-2);
							  }
							  
							  // encode triangle_data()
							  MPG_BranchTree* tmp_bt = cur_bt;	
							  spcodec.init_bits(codap.totalbits, codap.restbits);  
							  // DMLAB (200503) - ADD
							  TCbits = 0;
							  // DMLAB (200503) - UPDATE
							  if(sp.encodeTriangleData(spcodec,
								  nV0[icc],   // first vertex index of cc
								  v_loop_cc[icc],     // local  indices
								  v_visited,     // global decoder indices
								  v_cluster_dec, // global decoder indices
								  pCoord,
								  pNormal,
								  pColor,
								  pTexCoord,
								  qCoord1,
								  qNormal1,
								  qColor1,
								  qTexCoord1,
								  v_traversal, f_index, 
								  c_index,	cur_bt,	
								  Grstk, Gistk, Gdepth, GiC, GiF,
								  prevCR, prevCL,	//MJH 001013 : added in case of partitioning at non-polygon edge
								  restbits, codap, &TCbits ) == false)
								  return false;
							  sumTCbits += TCbits;
							  //~DMLAB (200503)
							  //write SP data to output buffer and 
							  //initialize variables for GF & buffer
							  codap.sp_packetization(
								  vgostrm, spostrm, ostrm, vgobstrm, spobstrm, obstrm,
								  vgcodec, spcodec, codec, icc, nCC, next, Gdepth, totalbits, restbits, 
								  cur_bt, tmp_bt, ccbit);
							  
						  } while(!next); 
						  
						  delete vpsp[icc]; vpsp[icc] = NULL;
						  icc++;
					  } while(!codap.vg_pt0_enable && (icc<ccbit.next));
					  
					  if(erMode>0 && icc==nCC) {
						  //MJH 001218 : no stuffing
						  //int rbytes=codap.stuffing(spostrm,spobstrm,spcodec);
						  spobstrm.init();
						  
						  codap.writeStream(spostrm,ostrm);
						  codap.updateQFac(spobstrm,obstrm,spcodec,codec);
						  MPG_Global::sp_store();    
						  codap.totalbits=spcodec.tbits; //MJH 001218 +rbytes*8;
						  codap.restbits=spcodec.rbits; //MJH 001218 -(rbytes*8);
					  }
					  icc--; 
				  } //if(!cpass) 
              }  //for(icc)
			  for(int cc_cnt=0;cc_cnt<nCC;cc_cnt++)
			  {					
				  array_cnt[cc_cnt] = nV1[cc_cnt] - nV0[cc_cnt];
				  array_accumul_cnt[cc_cnt] = nV1[cc_cnt];					
			  }

          } while(++cpass < 2);  

		  
		  
			//Stitching
			if (has_stitches)
			{
				bool vertex_increase = true;
				bool face_increase = false;
				vector<int> dup_vertices_info;
				vector<int> dup_vertices_loc;
				int duplicatedCnt = 0;
				int* reArray = new int[v_stitch_enc.size()];;
				if (!vertex_order_flag)
				{			  
					for(int i = 0; i < v_cluster_enc.size(); i++)
					{
						reArray[v_traversal[v_cluster_dec[i]]] = i;
					}
				}
				for(int i = 0; i < v_stitch_enc.size() - duplicationCnt; i++)
				{
					int startInt = i;
					int exist = 0;
					for(int j = 0; j < duplicationCnt; j++)
					{
						int index = v_stitch_enc.size() - duplicationCnt + j;
						if (startInt == v_stitch_enc[index])
						{
							if (!exist)
							{
								exist = 1;
								dup_vertices_loc.push_back(startInt);
							}
							dup_vertices_info.push_back(index);
						}						
					}
					if (exist) 
					{
						dup_vertices_info.push_back(-1);
						duplicatedCnt++;
					}
					if (i > v_stitch_enc[i])
						break;
				}
				// write
				obstrm.putBit(vertex_increase);
				obstrm.putBit(face_increase);
				int allocatedBit = representBit(v_cluster_enc.size());
				obstrm.putMBitInt(allocatedBit, duplicatedCnt);
				int infoCount = 0;
				for(i = 0; i < duplicatedCnt; i++)
				{
					int loopCount = 0;
					for(int j = infoCount; dup_vertices_info[j] != -1; j++)
						loopCount++;
					obstrm.putMBitInt(allocatedBit, loopCount+1);
					if (!vertex_order_flag)
						obstrm.putMBitInt(allocatedBit, reArray[dup_vertices_loc[i]]);
					else 
						obstrm.putMBitInt(allocatedBit, dup_vertices_loc[i]);
					for(j = 0; j < loopCount; j++)
					{
						if (vertex_order_flag)
							obstrm.putMBitInt(allocatedBit, dup_vertices_info[j + infoCount]);
						else 
						{
							int temp = dup_vertices_info[j + infoCount];
							int temp2 = reArray[dup_vertices_info[j + infoCount]];
							obstrm.putMBitInt(allocatedBit, reArray[dup_vertices_info[j + infoCount]]);
						}
					}
					infoCount += j+1;
				}

				/*
				int* isdup = new int[v_stitch_enc.size()];
				vector<int> dup_vertices_info;
				
				
				for(int d = 0; d < v_stitch_enc.size(); d++)
					isdup[d] = -1;
				
				for(int i = 0; i < v_stitch_enc.size(); i++)
				{
					isdup[v_stitch_enc[i]]++;
					if (isdup[v_stitch_enc[i]] > 0 && !vertex_increase)
						vertex_increase = true;
				}
				if (vertex_increase == true)
				{
					for(int i = 0; i < v_stitch_enc.size(); i++)
					{
						if(isdup[i] == -1)
						{
							dup_vertices_loc.push_back(v_stitch_enc[i]);
							dup_vertices_info.push_back(i);						
						}
					}
				}
				
				int duplicatedCnt = 0;
				memset(isdup, 0, v_stitch_enc.size()*sizeof(int));
				for(i = 0; i < dup_vertices_loc.size(); i++)
				{
					if (isdup[dup_vertices_loc[i]] == 0)
					{
						isdup[dup_vertices_loc[i]] = 1;
						duplicatedCnt++;
					}				
				}
				
				// write
				obstrm.putBit(vertex_increase);
				obstrm.putBit(face_increase);
				int allocatedBit = representBit(v_cluster_enc.size());
				obstrm.putMBitInt(allocatedBit, duplicatedCnt);
				for(i = 0; i < v_stitch_enc.size(); i++)
				{
					int count = 0, isExist = 0;
					int *location = new int[dup_vertices_loc.size()];
					for(int j = 0; j < dup_vertices_loc.size(); j++)
					{
						if (i == dup_vertices_loc[j])
						{
							isExist = 1;
							location[count] = j;
							count++;
						}
					}
					
					if (isExist)
					{
						obstrm.putMBitInt(allocatedBit, count+1);
						if (!vertex_order_flag)
							obstrm.putMBitInt(allocatedBit, reArray[i]);
						else 
							obstrm.putMBitInt(allocatedBit, i);
						for(int j = 0; j < count; j++)
						{
							if (vertex_order_flag)
								obstrm.putMBitInt(allocatedBit, dup_vertices_info[location[j]]);
							else 
								obstrm.putMBitInt(allocatedBit, reArray[dup_vertices_info[location[j]]]);
						}
					}
					delete location;
				}
				*/
				delete reArray;
			}
			
			
      } // end encoding 3D_Mesh_Object_Base_Layer() 
	  
	  if(Order_mode)  
	  {
		  if(ostrm.putStartCode(MPG_Global::MOL_start_code)==false)
			  return false;

		  // write mol_id
		  if(obstrm.putMBitInt(8,mol_id)==false) return false;
		  
		  
		  //DK
		  //------------------  vertex order coding start -----------------------------//
  
		  if(vertex_order_flag)
		  {			
			  printf("vertex ordering starts\n");
			  if(vertex_order_per_CC_flag)
			  {				  
				  int i;int cnt = 0, cc_cnt; 
				  int min,max,offset_cnt;
				  
				  vector <int> offset;
				  vector <int> first_ID;
				  vector <int> num_CC_offset;
				  
				  vector <int> offset_all;
				  vector <int> first_ID_all;
				  vector <int> num_CC_offset_all;
				  
				  //	vector <int> offset_per_CC;
				  //	vector <int> first_ID_per_CC;
				  //	vector <int> num_CC_offset_per_CC;
				  
				  int l,m;
				  
				  //test
				  int *v_order = new int[v_traversal.size()];
				  int iterIFS;
				  //test
				  int *v_re_order;
				  int *v_order_CC_mapping;
				  int *v_re_order_difference;
				  int *threshold;
				  int origin_ID;
				  int transformed_ID;
				  
				  for(cc_cnt=0;cc_cnt<nCC;cc_cnt++)
				  {				
					  v_re_order = new int[array_cnt[cc_cnt]];	
					  v_order_CC_mapping = new int[array_cnt[cc_cnt]];
					  v_re_order_difference = new int[array_cnt[cc_cnt]-1];						
					  
					  if(cc_cnt == 0)
					  {	
						  min = 0, max = array_cnt[cc_cnt]; 
						  
						  i = 0, cnt = 0;
						  offset_cnt = 0;
						  //DMLAB ory Vertex Reordering 2006 05
						  int offset_sum=0;
						  for(i=min;i<max;i++)
						  {
							  v_re_order[cnt] = v_traversal[i];
							  cnt++;
						  }
						  
						  //for(i=0;i<(max-min);i++)  //  
						  {
							  qsort(v_re_order,max-min,sizeof(int), compare);
						  }
						  
						  // offset 
						  if(v_re_order[0] != 0)
						  {
							  offset_sum=v_re_order[0];
							  offset.push_back(v_re_order[0]);
							  first_ID.push_back(0);
							  offset_cnt++;
						  }
						  for(i=0;i<max-1;i++)
						  {
							  v_re_order_difference[i] = v_re_order[i+1] - v_re_order[i] -1;
							  if(v_re_order_difference[i] != 0)
							  {
								  offset_sum+=v_re_order_difference[i];
								  offset.push_back(offset_sum);
								  first_ID.push_back(i+1);		
								  offset_cnt++; // CC  ɼ  
							  }
						  }
						  num_CC_offset.push_back(offset_cnt);
						  
						  threshold = new int[offset_cnt+1];
						  for(m=0;m<offset.size();m++)
						  {
							  threshold[m] = offset[m] + first_ID[m];
						  }
						  threshold[m]=v_traversal.size();
						  
						  if(offset.size()>0)
						  {				
							  for(m=min;m<max;m++)
							  {
								  if(offset.size() == 1)
								  {
									  if(v_traversal[m] >= threshold[0] )
									  {
										  v_order_CC_mapping[m] = v_traversal[m] - offset[0];
									  }
									  else
									  {
										  v_order_CC_mapping[m] = v_traversal[m];
									  }
								  }
								  else
								  {
									  v_order_CC_mapping[m]=v_traversal[m];
									  int t = offset.size();
									  for(int k=0;k<=offset.size();k++)
									  {
										  int q = v_traversal[m];
										  if(v_traversal[m]>=threshold[k] && v_traversal[m]<threshold[k+1])
										  {										
											  v_order_CC_mapping[m]=v_traversal[m]-offset[k];
											  break;
										  }
										  
									  }
								  }
							  }
						  }
						  else
						  {
							  for(m=min;m<max;m++)
							  {					
								  v_order_CC_mapping[m] = v_traversal[m];
							  }
						  }
						  
						  for(l=0;l<offset.size();l++)
						  {
							  offset_all.push_back(offset[l]);
						  }
						  for(l=0;l<first_ID.size();l++)
						  {
							  first_ID_all.push_back(first_ID[l]);
						  }
						  for(l=0;l<num_CC_offset.size();l++)
						  {
							  num_CC_offset_all.push_back(num_CC_offset[l]);
						  }
						  offset.erase(offset.begin(),offset.end());
						  first_ID.erase(first_ID.begin(),first_ID.end());
						  num_CC_offset.erase(num_CC_offset.begin(),num_CC_offset.end());	
					}
					
					else
					{
						min = array_accumul_cnt[cc_cnt-1], max = array_accumul_cnt[cc_cnt]; 
						
						i = 0, cnt = 0;
						offset_cnt = 0;
						int offset_sum=0;
						
						for(i=min;i<max;i++)
						{
							v_re_order[cnt] = v_traversal[i];
							cnt++;
						}
						
						//for(i=0;i<(max-min);i++)  //  
						{
							qsort(v_re_order,max-min,sizeof(int), compare);
						}
						
						// offset 
						
						if(v_re_order[0] != 0)
						{
							offset.push_back(v_re_order[0]);
							first_ID.push_back(0);
							offset_cnt++;
							offset_sum=v_re_order[0];
						}
						for(i=0;i<max-min-1;i++)
						{
							v_re_order_difference[i] = v_re_order[i+1] - v_re_order[i] -1;
							if(v_re_order_difference[i] != 0)
							{
								offset_sum=offset_sum+v_re_order_difference[i];
								offset.push_back(offset_sum);
								first_ID.push_back(i+1);		
								offset_cnt++; // CC  ɼ  
							}
						}
						num_CC_offset.push_back(offset_cnt);
						
						threshold = new int[offset_cnt+1];
						for(m=0;m<offset.size();m++)
						{
							threshold[m] = offset[m] + first_ID[m];
						}
						threshold[m]=v_traversal.size();
						
						
						if(offset.size()>0)
						{				
							for(m=0;m<(max-min);m++)
							{
								if(offset.size() == 1)
								{
									if(v_traversal[m+min] >= threshold[0] )
									{
										v_order_CC_mapping[m] = v_traversal[m+min] - offset[0];
									}
									else
									{
										v_order_CC_mapping[m] = v_traversal[m+min];
									}
								}
								else
								{
									v_order_CC_mapping[m]=origin_ID=v_traversal[m+min];
									for(int k=0;k<offset.size();k++)
									{
										if(origin_ID>=threshold[k] && origin_ID<threshold[k+1])
										{
											transformed_ID=origin_ID-offset[k];
											v_order_CC_mapping[m]=transformed_ID;
											break;
										}
									}
								}
							}							
						}
						else
						{
							for(m=0;m<(max-min);m++)
							{					
								v_order_CC_mapping[m] = v_traversal[m];
							}
						}
						
						
						for(l=0;l<offset.size();l++)
						{
							offset_all.push_back(offset[l]);
						}
						for(l=0;l<first_ID.size();l++)
						{
							first_ID_all.push_back(first_ID[l]);
						}
						for(l=0;l<num_CC_offset.size();l++)
						{
							num_CC_offset_all.push_back(num_CC_offset[l]);
						}
						offset.erase(offset.begin(),offset.end());
						first_ID.erase(first_ID.begin(),first_ID.end());
						num_CC_offset.erase(num_CC_offset.begin(),num_CC_offset.end());	
					}
					
					if(cc_cnt==0)
					{
						for(iterIFS = 0; iterIFS<array_accumul_cnt[cc_cnt]; iterIFS++)
						{
							
							v_order[iterIFS] = v_order_CC_mapping[iterIFS];
						}
					}
					else
					{
						for(iterIFS = array_accumul_cnt[cc_cnt-1]; iterIFS<array_accumul_cnt[cc_cnt];iterIFS++)
						{
							v_order[iterIFS] = v_order_CC_mapping[iterIFS-array_accumul_cnt[cc_cnt-1]];
						}
					}					
					
					
					delete [] v_re_order;
					delete [] v_order_CC_mapping;
					delete [] v_re_order_difference;
					//~DMLAB ory
					//delete []threshold;				
					}
					
					
					l=0;
					for(int n=0;n<nCC;n++)
					{
						int nVOffset = num_CC_offset_all[n];
						obstrm.putMOBitInt(16, nVOffset);
						for(m=0;m<nVOffset;m++)
						{
							int vo_offset = offset_all[l];
							int firstVID = first_ID_all[l];
							l++;
							obstrm.putMOBitInt(24, vo_offset);
							obstrm.putMOBitInt(24, firstVID);
						}				
					}
					
					for(int cc=0;cc<nCC;cc++)	
					{		
						int inputbit = bitsToRepresent(array_cnt[cc]);//DMLAB ory 2006 05
						int firstbit = inputbit;
						int nV = array_cnt[cc];
						vector <int> vertexVector;
						for(int i = 0; i < nV; i++)
							vertexVector.push_back(i);
						
						int idx = (cc==0)?0 : array_accumul_cnt[cc-1];
						
						int* reArray = new int[nV];
						int acc_vertices = 0;
						
						for(;inputbit > 0;)
						{
							int Coding_vertices = nV - pow(2,inputbit-1); //pow(a,b) a b
							nV = nV - Coding_vertices;					
							
							int* delList = new int[Coding_vertices];
							int delCnt = 0;					
							
							if (inputbit == firstbit)
							{
								for(i=0;i<Coding_vertices;i++)					
								{
									obstrm.putMOBitInt(inputbit ,v_order[i+idx]);
									delList[delCnt++] = v_order[i+idx];
								}
							}
							else
							{
								for(i = acc_vertices; i < acc_vertices + Coding_vertices; i++)
								{
									obstrm.putMOBitInt(inputbit, reArray[v_order[i + idx]]);
									if (reArray[v_order[i + idx]] == -1) printf("invalid value error\n");
									delList[delCnt++] = reArray[v_order[i + idx]];							
								}
							}
							inputbit--;
							
							qsort(delList, delCnt, sizeof(int), compare);
							
							for(i = 0; i < delCnt ; i++)
								vertexVector.erase(vertexVector.begin() + delList[i] - i);
							
							memset(reArray, -1, nV*sizeof(int));
							for(int q = 0; q < vertexVector.size(); q++)
								reArray[vertexVector[q]] = q;
							
							acc_vertices += Coding_vertices;
							
						}
						obstrm.flush();
					}
					offset_all.erase(offset_all.begin(),offset_all.end());
					first_ID_all.erase(first_ID_all.begin(),first_ID_all.end());
					num_CC_offset_all.erase(num_CC_offset_all.begin(),num_CC_offset_all.end());
					offset.erase(offset.begin(),offset.end());
					first_ID.erase(first_ID.begin(),first_ID.end());
					num_CC_offset.erase(num_CC_offset.begin(),num_CC_offset.end());	
					delete [] v_order;	
			}
			else{				
				int i;
				int inputbit = bitsToRepresent(v_traversal.size());//DMLAB ory 2006 05
				// ifs.v_traversal.size() 8̸ inputbit 4 ȴ ׷Ƿ 8
				//ڵϴ  3Ʈ  ϴ. ׷Ƿ 1   ־ش. ory Լ ٲ
				int firstbit = inputbit;
				int nV = v_traversal.size();
				vector <int> vertexVector;
				for(i = 0; i < nV; i++)
					vertexVector.push_back(i);
				int* reArray = new int[nV];
				int acc_Vertices = 0;
				for(;inputbit > 0;)
				{
					int Coding_Vertices = nV - pow(2,inputbit-1); //pow(a,b) a b
					nV = nV - Coding_Vertices;					
					
					int* delList = new int[Coding_Vertices];
					int delCnt = 0;					
					
					if (inputbit == firstbit)
					{
						for(i=0;i<Coding_Vertices;i++)					
						{
							obstrm.putMOBitInt(inputbit ,v_traversal[i]);
							delList[delCnt++] = v_traversal[i];
						}
					}
					else
					{
						for(i = acc_Vertices; i < acc_Vertices + Coding_Vertices; i++)
						{
							obstrm.putMOBitInt(inputbit, reArray[v_traversal[i]]);
							delList[delCnt++] = reArray[v_traversal[i]];							
						}
					}
					inputbit--;
					
					qsort(delList, delCnt, sizeof(int), compare);
					
					for(i = 0; i < delCnt ; i++)
						vertexVector.erase(vertexVector.begin() + delList[i] - i);
					
					for(int q = 0; q < vertexVector.size(); q++)
						reArray[vertexVector[q]] = q;
					
					acc_Vertices += Coding_Vertices;
				}
				int temp = vertexVector[0];
			}
			obstrm.flush();
		} //--------------vertex order coding end -----------------------//
		

		
		// ----------------------face order coding start -----------------------//
		if(face_order_flag)
		{
			printf("face ordering starts\n");
			int i;
			int *face_accumul_cnt = new int[nCC];
			int *face_traversal = new int[f_traversal.size()];
			
			/* DMLAB figure the number of vertice of each face 
			accumulation face index array is needed 
			*/

			memset(face_accumul_cnt, 0, nCC * sizeof(int));
			
			for(i = 0; i < nCC; i++)
			{
				if (i != 0)
					face_accumul_cnt[i] += (face_array_cnt[i] + face_accumul_cnt[i-1]);
				else 
					face_accumul_cnt[0] = face_array_cnt[0];
			}
			if(face_order_per_CC_flag)
			{
				
				int i;int cnt = 0, cc_cnt; 
				int min,max,offset_cnt;
				
				vector <int> offset;
				vector <int> first_ID;
				vector <int> num_CC_offset;
				
				vector <int> offset_all;
				vector <int> first_ID_all;
				vector <int> num_CC_offset_all;
				
				//	vector <int> offset_per_CC;
				//	vector <int> first_ID_per_CC;
				//vector <int> num_CC_offset_per_CC;
				
				int l,m;
				
				//test
				int *f_order = new int[f_traversal.size()];
				int iterIFS;
				//test
				int *f_re_order;
				int *f_order_CC_mapping;
				int *f_re_order_difference;
				int *threshold;
				int origin_ID;
				int transformed_ID;
				for( cc_cnt=0;cc_cnt<nCC;cc_cnt++)
				{				
					f_re_order = new int[face_array_cnt[cc_cnt]];	
					f_order_CC_mapping = new int[face_array_cnt[cc_cnt]];
					f_re_order_difference = new int[face_array_cnt[cc_cnt]-1 >= 0 ? face_array_cnt[cc_cnt]-1 : face_array_cnt[cc_cnt]];						
					
					if(cc_cnt == 0)
					{	
						min = 0, max = face_array_cnt[cc_cnt]; 
						
						i = 0, cnt = 0;
						offset_cnt = 0;
						//DMLAB ory Face Reordering 2006 05
						int offset_sum=0;									
						for(i=min;i<max;i++)
							f_re_order[cnt++] = f_traversal[i];
						
						//	 printf("v_re_order[%d] = %d\n", max-min,v_re_order[max-min-1]);	
						//for(i=0;i<(max-min);i++)  //  
						{
							qsort(f_re_order,max-min,sizeof(int), compare);
						}
						// offset 
						if(f_re_order[0] != 0)
						{
							offset_sum=f_re_order[0];
							offset.push_back(f_re_order[0]);
							first_ID.push_back(0);
							offset_cnt++;
						}
						for(i=0;i<max-1;i++)
						{
							f_re_order_difference[i] = f_re_order[i+1] - f_re_order[i] -1;
							if(f_re_order_difference[i] != 0)
							{
								offset_sum += f_re_order_difference[i];
								offset.push_back(offset_sum);
								first_ID.push_back(i+1);		
								offset_cnt++; // CC  ɼ  
							}
						}
						num_CC_offset.push_back(offset_cnt);
						
						threshold = new int[offset_cnt+1];
						for(m=0;m<offset.size();m++)
						{
							threshold[m] = offset[m] + first_ID[m];
						}
						threshold[m]=f_traversal.size();
						
						if(offset.size()>0)
						{				
							for(m=min;m<max;m++)
							{
								if(offset.size() == 1)
								{
									if(f_traversal[m] >= threshold[0] )
									{
										f_order_CC_mapping[m] = f_traversal[m] - offset[0];
									}
									else
									{
										f_order_CC_mapping[m] = f_traversal[m];
									}
								}
								else
								{
									f_order_CC_mapping[m]=f_traversal[m];
									for(int k=0;k<offset.size();k++)
									{
										if(f_traversal[m]>=threshold[k] && f_traversal[m]<threshold[k+1])
										{
											f_order_CC_mapping[m]=f_traversal[m]-offset[k];
											break;
										}
									}
								}
							}
						}
						else
						{
							for(m=min;m<max;m++)
							{					
								f_order_CC_mapping[m] = f_traversal[m];
							}
						}
						
						for(l=0;l<offset.size();l++)
						{
							offset_all.push_back(offset[l]);
						}
						for(l=0;l<first_ID.size();l++)
						{
							first_ID_all.push_back(first_ID[l]);
						}
						for(l=0;l<num_CC_offset.size();l++)
						{
							num_CC_offset_all.push_back(num_CC_offset[l]);
						}
						offset.erase(offset.begin(),offset.end());
						first_ID.erase(first_ID.begin(),first_ID.end());
						num_CC_offset.erase(num_CC_offset.begin(),num_CC_offset.end());	
					}
					
					else
					{
						min = face_accumul_cnt[cc_cnt-1], max = face_accumul_cnt[cc_cnt]; 
						
						i = 0;
						cnt = 0;
						offset_cnt = 0;
						int offset_sum=0;
						
						for(i=min;i<max;i++)
						{
							f_re_order[cnt] = f_traversal[i];
							cnt++;
						}
						
						//for(i=0;i<(max-min);i++)  //  
						{
							qsort(f_re_order,max-min,sizeof(int), compare);
						}
						
						// offset 
						if(f_re_order[0] != 0)
						{
							offset.push_back(f_re_order[0]);
							first_ID.push_back(0);
							offset_cnt++;
							offset_sum=offset_sum+f_re_order[0];
						}
						for(i=0;i<max-min-1;i++)
						{
							f_re_order_difference[i] = f_re_order[i+1] - f_re_order[i] -1;
							if(f_re_order_difference[i] != 0)
							{
								offset_sum=offset_sum+f_re_order_difference[i];
								offset.push_back(offset_sum);
								first_ID.push_back(i+1);		
								offset_cnt++; // CC  ɼ  
							}
						}
						num_CC_offset.push_back(offset_cnt);
						
						threshold = new int[offset_cnt+1];
						for(m=0;m<offset.size();m++)
						{
							threshold[m] = offset[m] + first_ID[m];
						}
						
						threshold[m]=f_traversal.size();
						
						
						if(offset.size()>0)
						{				
							for(m=0;m<(max-min);m++)
							{
								if(offset.size() == 1)
								{
									if(f_traversal[m+min] >= threshold[0] )
									{
										f_order_CC_mapping[m] = f_traversal[m+min] - offset[0];
									}
									else
									{
										f_order_CC_mapping[m] = f_traversal[m+min];
									}
								}
								else
								{
									f_order_CC_mapping[m]=origin_ID=f_traversal[m+min];									
									for(int k=0;k<offset.size();k++)
									{
										if(origin_ID>=threshold[k] && origin_ID<threshold[k+1])
										{
											transformed_ID=origin_ID-offset[k];
											f_order_CC_mapping[m]=transformed_ID;
											break;
										}
									}
								}
								
							}
							
						}
						else
						{
							for(m=0;m<(max-min);m++)
							{					
								f_order_CC_mapping[m] = f_traversal[m];
							}
						}
						
						for(l=0;l<offset.size();l++)
						{
							offset_all.push_back(offset[l]);
						}
						for(l=0;l<first_ID.size();l++)
						{
							first_ID_all.push_back(first_ID[l]);
						}
						for(l=0;l<num_CC_offset.size();l++)
						{
							num_CC_offset_all.push_back(num_CC_offset[l]);
						}
						offset.erase(offset.begin(),offset.end());
						first_ID.erase(first_ID.begin(),first_ID.end());
						num_CC_offset.erase(num_CC_offset.begin(),num_CC_offset.end());	
					}
					
					if(cc_cnt==0)
					{
						for(iterIFS = 0; iterIFS<face_accumul_cnt[cc_cnt]; iterIFS++)
						{
							f_order[iterIFS] = f_order_CC_mapping[iterIFS];
						}
					}
					else
					{
						for(iterIFS = face_accumul_cnt[cc_cnt-1]; iterIFS<face_accumul_cnt[cc_cnt];iterIFS++)
						{
							f_order[iterIFS] = f_order_CC_mapping[iterIFS-face_accumul_cnt[cc_cnt-1]];
						}
					}		
					
					delete [] f_re_order;
					delete [] f_order_CC_mapping;
					delete [] f_re_order_difference;
					
					//~DMLAB ory
					
					//delete []threshold;				
				}
				
				l=0;
				for(int n=0;n<nCC;n++)
				{
					int nVOffset = num_CC_offset_all[n];
					obstrm.putMOBitInt(16, nVOffset);
					for(m=0;m<nVOffset;m++)
					{
						int vo_offset = offset_all[l];
						int firstVID = first_ID_all[l];
						l++;
						obstrm.putMOBitInt(24, vo_offset);
						obstrm.putMOBitInt(24, firstVID);
					}				
				}				
				
				for(int cc=0;cc<nCC;cc++)	
				{	
					int inputbit = bitsToRepresent(face_array_cnt[cc]);//DMLAB ory 2006 05
					int firstbit = inputbit;
					int nF = face_array_cnt[cc];
					vector <int> faceVector;
					for(int i = 0; i < nF; i++)
						faceVector.push_back(i);
					
					int idx = (cc==0)?0 : face_accumul_cnt[cc-1];
					
					int* reArray = new int[nF];
					int acc_Faces = 0;
					
					for(;inputbit > 0;)
					{
						int Coding_Faces = nF - pow(2,inputbit-1); //pow(a,b) a b
						nF = nF - Coding_Faces;					
						
						int* delList = new int[Coding_Faces];
						int delCnt = 0;					
						
						if (inputbit == firstbit)
						{
							for(i=0;i<Coding_Faces;i++)					
							{
								obstrm.putMOBitInt(inputbit ,f_order[i+idx]);
								delList[delCnt++] = f_order[i+idx];
							}
						}
						else
						{
							for(i = acc_Faces; i < acc_Faces + Coding_Faces; i++)
							{
								obstrm.putMOBitInt(inputbit, reArray[f_order[i + idx]]);
								if (reArray[f_order[i + idx]] == -1) printf("invalid value error\n");
								delList[delCnt++] = reArray[f_order[i + idx]];							
							}
						}
						inputbit--;
						
						qsort(delList, delCnt, sizeof(int), compare);
						
						for(i = 0; i < delCnt ; i++)
							faceVector.erase(faceVector.begin() + delList[i] - i);
						
						memset(reArray, -1, nF*sizeof(int));
						for(int q = 0; q < faceVector.size(); q++)
							reArray[faceVector[q]] = q;
						
						acc_Faces += Coding_Faces;
						
					}
					obstrm.flush();
				}
				
				offset_all.erase(offset_all.begin(),offset_all.end());
				first_ID_all.erase(first_ID_all.begin(),first_ID_all.end());
				num_CC_offset_all.erase(num_CC_offset_all.begin(),num_CC_offset_all.end());
				offset.erase(offset.begin(),offset.end());
				first_ID.erase(first_ID.begin(),first_ID.end());
				num_CC_offset.erase(num_CC_offset.begin(),num_CC_offset.end());	
				
				delete [] f_order;	
			}
			else
			{
				int i;
				int inputbit = bitsToRepresent(f_traversal.size());//DMLAB ory 2006 05
				// ifs.v_traversal.size() 8̸ inputbit 4 ȴ ׷Ƿ 8
				//ڵϴ  3Ʈ  ϴ. ׷Ƿ 1   ־ش. ory Լ ٲ
				int firstbit = inputbit;
				int nF = f_traversal.size();
				vector <int> faceVector;
				for(i = 0; i < nF; i++)
					faceVector.push_back(i);
				int* reArray = new int[nF];
				int acc_Faces = 0;
				for(;inputbit > 0;)
				{
					int Coding_Faces = nF - pow(2,inputbit-1); //pow(a,b) a b
					nF = nF - Coding_Faces;					
					
					int* delList = new int[Coding_Faces];
					int delCnt = 0;					
					
					if (inputbit == firstbit)
					{
						for(i=0;i<Coding_Faces;i++)					
						{
							obstrm.putMOBitInt(inputbit ,f_traversal[i]);
							delList[delCnt++] = f_traversal[i];
						}
					}
					else
					{
						for(i = acc_Faces; i < acc_Faces + Coding_Faces; i++)
						{
							obstrm.putMOBitInt(inputbit, reArray[f_traversal[i]]);
							delList[delCnt++] = reArray[f_traversal[i]];							
						}
					}
					inputbit--;
					
					qsort(delList, delCnt, sizeof(int), compare);
					
					for(i = 0; i < delCnt ; i++)
						faceVector.erase(faceVector.begin() + delList[i] - i);
					
					for(int q = 0; q < faceVector.size(); q++)
						reArray[faceVector[q]] = q;
					
					acc_Faces += Coding_Faces;
				}
				int temp = faceVector[0];
				//------------------------------------------------
				obstrm.flush();
				//------------------------------------------------
			}
		}
		// ----------------------face order coding end -----------------------//
		int bytes = obstrm.getOutputStream()->size();
		printf("bytes : %d\n", bytes);
		//~DK
  } //end encoding 3D_Mesh_Object_Layer() 
  
  //begin encoding 3DMeshObject_Refinement_Layer () 
  if(nLevelsOfDetail>1)
  {
      // encode forest split refinements
      // as a sequence of 3D_Mesh_Object_Layer()
      int morl_id = 0;
	  
      // connectivity_update
      // 0 : NOT_UPDATED
      // 1 : FS_UPDATE
      // 2 : RESERVED
      // 3 : RESERVED
      int connectivity_update = 0;
      
      // GT Wed Jun 09 23:40:17 1999
      // do not encode other_update yet
      int other_update = 0;
      
      // these variables should come as parameters
      bool  pre_smoothing         = false;
      float pre_smoothing_lambda  =   0.0;
      float pre_smoothing_mu      =   0.0;
      float pre_smoothing_n       =     0;
      bool  post_smoothing        = false;
      float post_smoothing_lambda =   0.0;
      float post_smoothing_mu     =   0.0;
      float post_smoothing_n      =     0;
      
      // to prevent MPG_SPolygon::decodeTriangleData()
      // to decode per vertex properties
      // reset the per_vertex property bindings, if any,
      // but save original setings;
      MPG_Binding bCoord    = pCoord.getBinding();
      if(bCoord == PB_PER_VERTEX)    pCoord.setBinding(PB_NONE);
      MPG_Binding bNormal   = pNormal.getBinding();
      if(bNormal == PB_PER_VERTEX)   pNormal.setBinding(PB_NONE);
      MPG_Binding bColor    = pColor.getBinding();
      if(bColor == PB_PER_VERTEX)    pColor.setBinding(PB_NONE);
      MPG_Binding bTexCoord = pTexCoord.getBinding();
      if(bTexCoord == PB_PER_VERTEX) pTexCoord.setBinding(PB_NONE);
	  
      vector<float> coord0;
      vector<float> normal0;
      vector<float> color0;
      vector<float> texCoord0;
      
      vector<int>   pqCoord0;
      vector<int>   pqNormal0;
      vector<int>   pqColor0;
      vector<int>   pqTexCoord0;
	  
      vector<int>   qCoord0;
      vector<int>   qNormal0;
      vector<int>   qColor0;
      vector<int>   qTexCoord0;
	  
      // smoothing variables
      vector<bool>  edge_is_sharp;
      vector<bool>  vertex_is_fixed;
	  
      int lod=nLevelsOfDetail-1;
	  
      // get quantized coordinates and properties of level lod
      {
		  if(bCoord != PB_NONE) {
			  if(csrf.getCoord(lod,coord0)==false) return false;
			  if(coordQuant->quantize(coord0,qCoord1)==false)
				  return false;
		  }
		  if(bNormal != PB_NONE) {
			  if(csrf.getNormal(lod,normal0)==false) return false;
			  if(normalQuant->quantize(normal0,qNormal1)==false)
				  return false;
		  }
		  if(bColor != PB_NONE) {
			  if(csrf.getColor(lod,color0)==false) return false;
			  if(colorQuant->quantize(color0,qColor1)==false)
				  return false;
		  }
		  if(bTexCoord != PB_NONE) {
			  if(csrf.getTexCoord(lod,texCoord0)==false) return false;
			  if(texCoordQuant->quantize(texCoord0,qTexCoord1)==false)
				  return false;
		  }
      }
      
      for(;lod>0;lod--)
	  {
          // encode 3D_Mesh_Object_Layer()
          
          // 3D_Mesh_Object_Layer() {
          //   3D_MOL_start_code
          //   mol_id
          //   if(cgd_data) {
          //     N_Vertices
          //     N_Triangles
          //     N_Edges
          //   }
          //   3D_Mesh_Object_Refinement_Layer()
          // }
          
          // write 3D_MOL_start_code
          if(ostrm.putStartCode(MPG_Global::MOL_start_code)==false)
			  return false;
          
          {
			  // open an output bit stream
			  MPG_OutputBitStream obstrm(ostrm);
			  // write mol_id
			  if(obstrm.putMBitInt(8,mol_id)==false) return false;
			  // flush output buffer
			  obstrm.flush();
          }
          
          // do not encode cgd data
          
          // 3D_Mesh_Object_Refinement_Layer() {
          //   do {
          //     3D_MORL_start_code
          //     morl_id
          //
          //     connectivity_update
          //
          //     pre_smoothing
          //     if(pre_smoothing)
          //       pre_smoothing_parameters()
          //     post_smoothing
          //     if(post_smoothing)
          //       post_smoothing_parameters()
          //
          //     while(!bytealigned())
          //       one_bit
          //
          //     start_qcoder()
          //
          //     if(connectivity_update==FS_UPDATE)
          //       fs_pre_update()
          //
          //     if(pre_smoothing || post_smoothing)
          //       smoothing_constraints()
          //
          //     if(pre_smoothing)
          //       apply pre_smoothing as predictor
          //       to vertex coordinates and re-quantize
          //
          //     if(connectivity_update==FS_UPDATE)
          //       fs_post_update()
          //
          //     if(connectivity_update!=NOT_UPDATED)
          //       other_update
          //
          //     if(connectivity_update==NOT_UPDATED || other_update)
          //       other_property_update()
          //
          //     if(post_smoothing)
          //       apply post_smoothing as predictor
          //       to vertex coordinates and re-quantize
          //
          //     stop_qcoder()
          //
          //   } while(nextbits_bytealigned()==3D_MORL_start_code)
          // }
          
          // write 3D_MORL_start_code
          if(ostrm.putStartCode(MPG_Global::MORL_start_code)==false)
			  return false;
          
          {
			  // open an output bit stream
			  MPG_OutputBitStream obstrm(ostrm);
			  
			  // write morl_id
			  if(obstrm.putMBitInt(8,morl_id)==false) return false;
			  morl_id++;
			  
			  // GT Wed Jun 09 22:27:23 1999
			  // don't we want 
			  // connectivity_update==NO_UPDATE
			  // if number of vertices does not change ?
			  
			  // connectivity_update==FS_UPDATE
			  connectivity_update = 1;
			  // write connectivity_update
			  if(obstrm.putMBitInt(2,connectivity_update)==false) return false;
			  morl_id++;
			  
			  bool  pre_smoothing =
				  (pre_smoothing_n>0 &&
				  (pre_smoothing_lambda!=0.0 ||
				  pre_smoothing_mu!=0.0))?true:false;
			  if(obstrm.putBit(pre_smoothing)==false) return false;
			  if(pre_smoothing==true)
              {
				  // GT Wed Jun 09 22:28:42 1999
				  // don't we want to reduce the number of bits
				  // used to encode these parameters ?
				  
				  if(obstrm.putMBitInt(8,pre_smoothing_n)==false)
					  return false;
				  if(obstrm.put32BitFloat(pre_smoothing_lambda)==false)
					  return false;
				  if(obstrm.put32BitFloat(pre_smoothing_mu)==false)
					  return false;
              }
			  
			  bool  post_smoothing =
				  (post_smoothing_n>0 &&
				  (post_smoothing_lambda!=0.0 ||
				  post_smoothing_mu!=0.0))?true:false;
			  if(obstrm.putBit(post_smoothing)==false) return false;
			  if(post_smoothing==true)
              {
				  // GT Wed Jun 09 22:28:42 1999
				  // don't we want to reduce the number of bits
				  // used to encode these parameters ?
				  
				  if(obstrm.putMBitInt(8,post_smoothing_n)==false)
					  return false;
				  if(obstrm.put32BitFloat(post_smoothing_lambda)==false)
					  return false;
				  if(obstrm.put32BitFloat(post_smoothing_mu)==false)
					  return false;
              }
			  
			  // while(!bytealigned())
			  //   one_bit
			  obstrm.flush();
          }
		  
          // get quantized coordinates and properties of level lod-1
          {
			  if(bCoord != PB_NONE) {
				  if(csrf.getCoord(lod-1,coord0)==false) return false;
				  if(coordQuant->quantize(coord0,qCoord0)==false)
					  return false;
			  }
			  if(bNormal != PB_NONE) {
				  if(csrf.getNormal(lod-1,normal0)==false) return false;
				  if(normalQuant->quantize(normal0,qNormal0)==false)
					  return false;
			  }
			  if(bColor != PB_NONE) {
				  if(csrf.getColor(lod-1,color0)==false) return false;
				  if(colorQuant->quantize(color0,qColor0)==false)
					  return false;
			  }
			  if(bTexCoord != PB_NONE) {
				  if(csrf.getTexCoord(lod-1,texCoord0)==false) return false;
				  if(texCoordQuant->quantize(texCoord0,qTexCoord0)==false)
					  return false;
			  }
          }
		  
          {
			  // represent and encode ...
			  
			  int  nV0   = csrf.getNumberOfCoord(lod-1);
			  int  nV1   = v_traversal.size();
			  
			  // get mesh of level lod-1 in encoder order
			  vector<int> coordIndex;
			  if(csrf.getCoordIndex(lod-1,coordIndex)==false) return false;
			  MPG_MeshFs mesh0(coordIndex);
			  
			  int nE0 = mesh0.getNumberOfEdges();
			  edge_is_sharp.insert(edge_is_sharp.end(),(size_t)nE0,false);
			  vertex_is_fixed.insert(vertex_is_fixed.end(),(size_t)nV0,false);
			  
			  // transfer quantized per vertex properties
			  // from level lod-1 to level lod
			  int* map01 = csrf.getMap(lod-1);
			  {
				  int i,j;
				  if(bCoord == PB_PER_VERTEX)
					  for(i=0;i<nV0;i++)
					  {
						  int* qi = &qCoord1[3*map01[i]];
						  for(j=0;j<3;j++) pqCoord0.push_back(qi[j]);
					  }
					  if(bNormal == PB_PER_VERTEX)
						  for(i=0;i<nV0;i++)
						  {
							  int* qi = &qNormal1[3*map01[i]];
							  for(j=0;j<3;j++) pqNormal0.push_back(qi[j]);
						  }
						  if(bColor == PB_PER_VERTEX)
							  for(i=0;i<nV0;i++)
							  {
								  int* qi = &qColor1[3*map01[i]];
								  for(j=0;j<3;j++) pqColor0.push_back(qi[j]);
							  }
							  if(bTexCoord == PB_PER_VERTEX)
								  for(i=0;i<nV0;i++)
								  {
									  int* qi = &qTexCoord1[2*map01[i]];
									  for(j=0;j<2;j++) pqTexCoord0.push_back(qi[j]);
								  }
			  }
			  
			  // v_traversal and f_traversal correspond to level lod (1) here
			  
			  // construct clustering map from
			  // level lod-1 encoder order to
			  // level lod decoder order
			  vector<int> map0enc1dec;
			  {
				  // put mesh vertices and faces of level lod in decoder order
				  vector<int> inv_v_traversal;
				  if(MPG_Perm::makeInvPerm(v_traversal,inv_v_traversal)==false)
				  {
					  return false;
				  }
				  
				  int i;
				  for(i=0;i<nV0;i++)
					  map0enc1dec.push_back(inv_v_traversal[map01[i]]);
				  
				  vector_erase(v_traversal);
				  
				  int nF0  = mesh0.getNumberOfFaces();
				  int nF1 = f_traversal.size();
				  
				  vector<int> t_nc;
				  vector<int> t_co;
				  mesh0.partitionCollapsedFaces(map0enc1dec,t_nc,t_co);
				  
				  for(i=0;i<nF1;i++)
					  f_traversal[i] = t_nc[f_traversal[i]];
				  for(i=0;i<t_co.size();i++)
					  f_traversal.push_back(t_co[i]);
				  
				  // GT Fri Jun 11 01:26:20 1999
				  // changing the order of the faces does not
				  // change the edges of the mesh graph
				  if(mesh0.permuteFaces(f_traversal)==false)
				  { 
					  return false;
				  }
				  
				  // f_traversal does not yet contain the level lod-1 decoder order
				  // only has the first nF1 faces in the right order
				  // the remaining nF0-nF1 faces, corresponding to the
				  // inserted faces, are reordered in the next step
			  }
			  
			  // CONNECTIVITY -------------------------------------------------
			  
			  vector<bool>          is_forest_edge;
			  vector<MPG_pSPolygon> vp_sp;
			  vector<MPG_pVecInt>   vp_v_loop;
			  vector<MPG_pVecInt>   vp_vcf_loop;
			  vector<int>           nTrees;
			  
			  // GT Wed Jun 09 23:26:17 1999 TODO
			  // remember that so far triData.size()==0 here :
			  // fill triData with per face and
			  // per corner properties of simple polygons
			  // in order of traversal inside
			  // MPG_MeshFs::representForestSplit()
			  // per vertex properties are taken care of later;
			  
			  // should we pass
			  // qCoord0, qNormal0, qColor0, qTexCoord0
			  // or
			  // pqCoord0, pqNormal0, pqColor0, pqTexCoord0
			  // ??
			  
			  vector<int> triData;	
			  {
				  vector<int> t_trav_relative;
				  if(mesh0.representForestSplit(map0enc1dec,
					  is_forest_edge,
					  vp_sp,
					  pCoord,
					  pNormal,
					  pColor,
					  pTexCoord,
					  qCoord0,
					  qNormal0,
					  qColor0,
					  qTexCoord0,
					  triData,
					  v_traversal,
					  t_trav_relative,
					  vp_v_loop,
					  vp_vcf_loop)==false)
				  {
					  return false;
				  }
				  int i;
				  for(i=0;i<t_trav_relative.size();i++)
					  t_trav_relative[i] = f_traversal[t_trav_relative[i]];
				  for(i=0;i<t_trav_relative.size();i++)
					  f_traversal[i] = t_trav_relative[i];
			  }
			  
			  // v_traversal and f_traversal correspond to level lod-1 (0) here
			  if(MPG_Perm::makeInvPerm(v_traversal,v_traversal_inv)==false)
				  return false;
			  
			  //     start_qcoder()
			  //            MPG_QFac codec(ostrm);
			  MPG_QFac codec(ostrm, 0);	
			  codec.start_encode();
			  MPG_Global::reset();
			  
			  int* tData = triData.begin();
			  
			  // fs_pre_update() {
			  //   for each connected component {
			  //     forest()
			  //     for each tree in forest {
			  //       triangle_tree()
			  //       /* for each tree loop vertex set visited=1 */
			  //       triangle_data()
			  //     }
			  //   }
			  // }
			  
			  // get next mesh with indices in decoder order
			  vector<int> coordIndex1;
			  MPG_MeshFs mesh1(mesh0.getCoordIndex(),map0enc1dec,coordIndex1);
			  int nCC = mesh1.getNumberOfConnectedComponents();
			  
			  // for each connected component {
			  int iVcc0    = 0; // first vertex index of current cc
			  int iTREEcc0 = 0; // first tree index of current cc
			  for(mesh1.ccBegin();mesh1.ccEnd()==false;mesh1.ccNext())
              {
				  int icc      = mesh1.getIndexCC();
				  int nVcc     = mesh1.getNumberOfVerticesCC();
				  int nFcc     = mesh1.getNumberOfFacesCC();
				  int nEcc     = mesh1.getNumberOfEdgesCC();
				  
				  int nTREEcc = 0;
				  
				  // forest()
				  {
					  // encode part of forest contained in
					  // current connected component
					  MPG_Partition p(nVcc);
					  MPG_Graph::pEdge e; 
					  for(e=mesh1.getFirstEdgeCC();
                      e!=(MPG_Graph::pEdge)0;
                      e=mesh1.getNextEdgeCC())
					  {
						  int i0cc = mesh1.getInvCoordIndexCC(e->getVertex(0));
						  int i1cc = mesh1.getInvCoordIndexCC(e->getVertex(1));
						  if(p.find(i0cc)!=p.find(i1cc))
						  {
							  bool ef = is_forest_edge[e->getIndex()];
							  codec.encode(ef,MPG_Global::pfs_forest_edge);
							  if(ef==true)
								  p.join(i0cc,i1cc);
						  }
						  // else skip encoding bit
					  }
					  
					  // count number of trees in forest
					  if(p.parts()<nVcc)
					  {
						  p.makeParts();
						  
						  int i;
						  for(i=0;i<nVcc;i++)
							  if(p.size(i)>1)
								  nTREEcc++;
					  }
				  }
				  nTrees.push_back(nTREEcc);
				  
				  //   for each tree in forest {
				  //     encode one simple polygon with no
				  //     associated property data per vertex;
				  //     but with per face and per corner
				  //     property data if present
				  //   }
				  
				  //   for each tree in forest {
				  //     triangle_tree()
				  //     for each vertex in tree loop
				  //       visited = 1
				  //     triangle_data()
				  //   }
				  // }
				  int i;
				  for(i=0;i<nTREEcc;i++)
                  {
					  MPG_SPolygon& sp = *(vp_sp[i+iTREEcc0]);
					  
					  // encode triangle_tree()
					  if(sp.encodeTriangleTree(codec) == false) return false;
					  
					  // encode triangle_data()
					  // since all the per vertex property binding
					  // have been reset, the vectors
					  // v_loop, v_visited and v_cluster are ignored
					  // but we need to pass them as arguments anyhow
					  // (dummy variables)
					  
					  vector<int> v_loop;
					  vector<int> v_visited;
					  vector<int> v_cluster;
					  
					  // quantized per face and per corner
					  // to be encoded for this simple polygon
					  // should have been collected
					  
					  if(sp.encodeTriangleData(codec,
						  0,
						  v_loop,
						  v_visited,
						  v_cluster,
						  pCoord,
						  pNormal,
						  pColor,
						  pTexCoord,
						  tData)==false)
						  return false;
					  
                  } // for(i=0;i<nTREEcc;i++)
				  
				  // advance to the first vertex index of next cc
				  iVcc0    += nVcc;
				  // advance to the first tree index of next cc
				  iTREEcc0 += nTREEcc;
              }
			  
			  //     if(pre_smoothing || post_smoothing)
			  //       smoothing_constraints()
			  if(pre_smoothing==true || post_smoothing==true)
              {
				  // smoothing_constraints() {
				  //   sharp_edges
				  //   if(sharp_edges)
				  //     edge_marks()
				  //   fixed_vertices
				  //   if(fixed_vertices)
				  //     vertex_marks()
				  // }
				  
				  // GT Wed Jun 09 23:35:58 1999
				  // do not encode smoothing constraints yet
				  codec.encode(0);
				  codec.encode(0);
              }
			  
			  // if(pre_smoothing)
			  //   apply pre_smoothing as predictor
			  //   to vertex coordinates transfered
			  //   from previous level and re-quantize
			  if(pre_smoothing==true)
              {
				  // unquantize qCoord0
				  if(coordQuant->unQuantize(pqCoord0,coord0)==false)
					  return false;
				  
				  // smooth with respect to mesh0 graph in place
				  if(mesh0.smooth(coord0,
					  coord0,
					  pre_smoothing_lambda,
					  pre_smoothing_mu,
					  pre_smoothing_n,
					  &edge_is_sharp,
					  &vertex_is_fixed)==false) return false;
				  
				  // unquantize qCoord0
				  if(coordQuant->quantize(coord0,pqCoord0)==false) return false;
              }
			  
			  // GT Fri Jun 11 01:28:34 1999
			  // so far only per_vertex properties are handled
			  
			  // pqCoord0    : predicted coordinates for level lod-1
			  // pqNormal0   : predicted normals for level lod-1
			  // pqColor0    : predicted color for level lod-1
			  // pqTexCoord0 : predicted tex coordinates for level lod-1
			  
			  // here is where the per vertex property
			  // displacements are encoded
			  
			  // if(connectivity_update==FS_UPDATE)
			  //   fs_post_update()
			  if(connectivity_update==1)
              {
				  // fs_post_update() {
				  //   for each connected component {
				  //     for each tree in forest {
				  //       tree_loop_property_update()
				  //     }
				  //   }
				  // }
				  
				  iVcc0    = 0;
				  iTREEcc0 = 0;
				  for(mesh1.ccBegin();mesh1.ccEnd()==false;mesh1.ccNext())
                  {
					  int icc      = mesh1.getIndexCC();
					  int nVcc     = mesh1.getNumberOfVerticesCC();
					  int nFcc     = mesh1.getNumberOfFacesCC();
					  int nEcc     = mesh1.getNumberOfEdgesCC();
					  int nTREEcc  = nTrees[icc];
					  
					  int i;
					  for(i=0;i<nTREEcc;i++)
                      {
						  MPG_SPolygon& sp     = *(vp_sp[i+iTREEcc0]);
						  vector<int>&  v_loop = *(vp_v_loop[i+iTREEcc0]);
						  
						  // tree_loop_update() {
						  //   loop_coord_update()
						  //   loop_normal_update()
						  //   loop_color_update()
						  //   loop_texCoord_update()
						  // }
						  
						  // loop_coord_update() {
						  //   for each tree loop vertex {
						  //     coord_sample()
						  //   }
						  // }
						  if(bCoord == PB_PER_VERTEX)
                          {
							  vector<int>& loop = v_loop;
							  int i,j,k,qj[3];
							  for(j=0;j<loop.size();j++)
                              {
								  k = loop[j];
								  for(i=0;i<3;i++)
									  qj[i] = qCoord0[3*k+i]-pqCoord0[3*k+i];
								  MPG_Sample::encodeCoordSample(codec,bitsPerCoord,qj);
                              }
                          }
						  
						  // loop_normal_update() {
						  //   if(normal_binding==PER_VERTEX) {
						  //     for each tree loop vertex {
						  //       normal_sample()
						  //     }
						  //   } else if(normal_binding==PER_FACE) {
						  //     for each tree loop face {
						  //       normal_sample()
						  //     }
						  //   } else if(normal_binding==PER_CORNER) {
						  //     for each tree loop corner {
						  //       normal_sample()
						  //     }
						  //   }
						  // }
						  if(bNormal == PB_PER_VERTEX)
                          {
							  int j,k,qj;
							  for(j=0;j<v_loop.size();j++)
                              {
								  k = v_loop[j];
								  qj = qNormal0[k]-pqNormal0[k];
								  MPG_Sample::
									  encodeNormalSample(codec,bitsPerNormal,&qj);
                              }
                          }
						  else if(bNormal == PB_PER_FACE)
                          {
							  
                          }
						  else if(bNormal == PB_PER_CORNER)
                          {
							  
                          }
						  
						  
						  // loop_color_update() {
						  //   if(color_binding==PER_VERTEX) {
						  //     for each tree loop vertex {
						  //       color_sample()
						  //     }
						  //   } else if(color_binding==PER_FACE) {
						  //     for each tree loop face {
						  //       color_sample()
						  //     }
						  //   } else if(color_binding==PER_CORNER) {
						  //     for each tree loop corner {
						  //       color_sample()
						  //     }
						  //   }
						  // }
						  if(bColor == PB_PER_VERTEX)
                          {
							  int i,j,k,qj[3];
							  for(j=0;j<v_loop.size();j++)
                              {
								  k = v_loop[j];
								  for(i=0;i<3;i++)
									  qj[i] = qColor0[3*k+i]-pqColor0[3*k+i];
								  MPG_Sample::encodeColorSample(codec,bitsPerColor,qj);
                              }
                          }
						  else if(bColor == PB_PER_FACE)
                          {
							  
                          }
						  else if(bColor == PB_PER_CORNER)
                          {
							  
                          }
						  
						  // loop_texCoord_update() {
						  //   if(texCoord_binding==PER_VERTEX) {
						  //     for each tree loop vertex {
						  //       texCoord_sample()
						  //     }
						  //   } else if(texCoord_binding==PER_CORNER) {
						  //     for each tree loop corner {
						  //       texCoord_sample()
						  //     }
						  //   }
						  // }
						  if(bTexCoord == PB_PER_VERTEX)
                          {
							  int i,j,k,qj[2];
							  for(j=0;j<v_loop.size();j++)
                              {
								  k = v_loop[j];
								  for(i=0;i<2;i++)
									  qj[i] = qTexCoord0[2*k+i]-pqTexCoord0[2*k+i];
								  MPG_Sample::encodeTexCoordSample(codec,bitsPerTexCoord,qj);
                              }
                          }
						  else if(bTexCoord == PB_PER_CORNER)
                          {
							  
                          }
						  
                      } // for(i=0;i<nTREEcc;i++)
					  
					  // advance to the first vertex index of next cc
					  iVcc0 += nVcc;
					  // advance to the first tree index of next cc
					  iTREEcc0 += nTREEcc;
                  }
              }
			  
			  // if(connectivity_update!=NOT_UPDATED)
			  //   other_update
			  if(connectivity_update!=0)
				  codec.encode(other_update);
			  
			  // if(connectivity_update==NOT_UPDATED || other_update==true)
			  //   other_property_update()
			  if(connectivity_update==0 || other_update==1)
              {
				  // other_property_update() {
				  //   other_coord_update()
				  //   other_normal_update()
				  //   other_color_update()
				  //   other_texCoord_update()
				  // }
				  
				  // other_coord_update() {
				  //   for each vertex in mesh {
				  //     if(not a tree loop vertex)
				  //       coord_sample()
				  //   }
				  // }
				  
				  // other_normal_update() {
				  //   if(normal_binding==PER_VERTEX) {
				  //     for each vertex in mesh {
				  //       if(not a tree loop vertex)
				  //         normal_sample()
				  //     }
				  //   } else if(normal_binding==PER_FACE) {
				  //     for each face in mesh {
				  //       if(not a tree loop face)
				  //         normal_sample()
				  //     }
				  //   } else if(normal_binding==PER_CORNER) {
				  //     for each corner in mesh {
				  //       if(not a tree loop corner)
				  //         normal_sample()
				  //     }
				  //   }
				  // }
				  
				  // other_color_update() {
				  //   if(color_binding==PER_VERTEX) {
				  //     for each vertex in mesh {
				  //       if(not a tree loop vertex)
				  //         color_sample()
				  //     }
				  //   } else if(color_binding==PER_FACE) {
				  //     for each face in mesh {
				  //       if(not a tree loop face)
				  //         color_sample()
				  //     }
				  //   } else if(color_binding==PER_CORNER) {
				  //     for each corner in mesh {
				  //       if(not a tree loop corner)
				  //         color_sample()
				  //     }
				  //   }
				  // }
				  
				  // other_texCoord_update() {
				  //   if(texCoord_binding==PER_VERTEX) {
				  //     for each vertex in mesh {
				  //       if(not a tree loop vertex)
				  //         texCoord_sample()
				  //     }
				  //   } else if(texCoord_binding==PER_CORNER) {
				  //     for each corner in mesh {
				  //       if(not a tree loop corner)
				  //         texCoord_sample()
				  //     }
				  //   }
				  // }
				  
              }
			  
			  // if(post_smoothing) {
			  //   apply post_smoothing as predictor
			  //   to vertex coordinates and re-quantize
			  // }
			  if(post_smoothing==true)
              {
				  // unquantize qCoord0
				  if(coordQuant->unQuantize(qCoord0,coord0)==false) return false;
				  
				  // smooth with respect to mesh0 graph in place
				  if(mesh0.smooth(coord0,
					  coord0,
					  post_smoothing_lambda,
					  post_smoothing_mu,
					  post_smoothing_n,
					  &edge_is_sharp,
					  &vertex_is_fixed)==false) return false;
				  
				  // unquantize qCoord0
				  if(coordQuant->quantize(coord0,qCoord0)==false) return false;
              }
			  
			  // stop_qcoder()
			  codec.finish_encode();
          }
		  
          qCoord1.swap(qCoord0);
          qNormal1.swap(qNormal0);
          qColor1.swap(qColor0);
          qTexCoord1.swap(qTexCoord0);
		  
          vector_erase(pqCoord0);
          vector_erase(pqNormal0);
          vector_erase(pqColor0);
          vector_erase(pqTexCoord0);
          
          vector_erase(edge_is_sharp);
          vector_erase(vertex_is_fixed);
        }
    } //end encoding 3DMeshObject_Refinement_Layer () 
	
#ifdef ADD_VISUAL_HEADER
	if(ostrm.putVisualStartCode(visual_object_sequence_end_code)==false)
		return false;
#endif //ADD_VISUAL_HEADER
	
	return true;
}
