//-------------------------------------------------------------------------
// 
// Copyright (C) IBM Corporation & Samsung Electronics Co. LTD., 1996, 1997, 1998, 1999
// Licensed Materials, Program Property of IBM & Samsung.
// 
// This software module is an implementation of a part of one or more
// MPEG-4 tools as specified by the MPEG-4 standard (ISO/IEC 14496).
// ISO/IEC gives users of the MPEG-4 standard (ISO/IEC 14496) free
// license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4
// standard (ISO/IEC 14496).  Those intending to use this software module
// in hardware or software products are advised that its use may infringe
// existing patents.  The original developer of this software module and
// his/her company, the subsequent editors and their companies, and
// ISO/IEC have no liability for use of this software module or
// modifications thereof in an implementation.  Copyright is not released
// for non MPEG-4 standard (ISO/IEC 14496) conforming products.  IBM
// Corp. & Samsung Electronics Co. LTD retains full right to use the code 
// for his/her own purpose, assign or donate the code to a third party and
// to inhibit third parties from using the code for non MPEG-4 standard 
// (ISO/IEC 14496) conforming products. This copyright notice must be included
// in all copies or derivative works.
// 
//  AUTHORS:
//    Gabriel Taubin
//    Andre Gueziec
//    Claudio Silva
//    Ioana Martin
//  
//  IBM T. J. Watson Research Center
//  P.O.Box 704
//  Yorktown Heights, New York 10598
//  
//  BASED ON EARLIER IMPLEMENTATION OF
//  VRML COMPRESSED BINARY FORMAT PROPOSAL BY:
//    Gabriel Taubin
//    Bill Horn
//    Francis Lazarus
//
//  Functionality : Error Resilience
//
//  Sungjin Kim  (sunjin@sait.samsung.co.kr)
//  Mahn-Jin Han  (manjini@sait.samsung.co.kr)
//  Mun-Sup Song (mssong@sait.samsung.co.kr)  
//
//  Signal Processing Lab. Samsung A.I.T. 
//-------------------------------------------------------------------------

//#define ADD_VISUAL_HEADER	
#ifdef ADD_VISUAL_HEADER
const char visual_object_sequence_start_code	= 0xB0;
const char visual_object_sequence_end_code		= 0xB1;
const char visual_object_start_code				= 0xB5;
#endif //ADD_VISUAL_HEADER

#pragma warning(disable:4786)

#include <stdio.h>
#include <stdlib.h>
#include "MPG_IfsQ_E.h"
#include "MPG_QFac.h"
#include "MPG_Global.h"
#include "MPG_VGraph.h"
#include "MPG_SPolygon.h"
//#include "MPG_Stitcher.h"
#include "MPG_IfsMesh.h"
#include "MPG_MeshFs.h"


#ifndef M_PI
#define M_PI    3.14159265
#endif

#ifndef MPGELIB
extern char* baseFname;
extern char* inExt;
extern char* inDir;
extern char* outDir;
extern char* inFname;
extern char* outFname;
extern char* logFname;
#endif // MPGELIB

int representBit(int numVertices)
{
	int bits = 1;
	int exp = 1;
	while((exp*=2) < numVertices) bits++;
	
	return bits;
}

int __cdecl compare(const void *elem1, const void *elem2)
{
    if(*(int*)elem1 < *(int*)elem2) return -1;
    if(*(int*)elem1 > *(int*)elem2) return 1;
    return 0;
}
//---------------------------------------------------------------------------
//
//   MPG_IfsQ_E_EXT::expand
//
//---------------------------------------------------------------------------

bool MPG_IfsQ_E_EXT::expand(MPG_InputStream_EXT& istrm, bool saveWrlLevels)
{

	MPG_QFpm_EXT::_shift = 16;
	
	bool success = false;
	
#ifdef MPGELIB
	bool newIFS = true;
#endif
	
	clear();
	
	// arrays to store quantized property values
	
	// AG: in 3D_Mesh_Object_Base_Layer
	// these arrays are global for per vertex properties
	// (containing all the values decoded so far),
	// and local otherwise
	// (containing only values corresponding to the current component)
	
	// GT: in 3D_Mesh_Object_Forest_Split
	// all these arrays are global
	
	vector <int> qCoord;
	vector <int> qNormal;
	vector <int> qColor;
	vector <int> qTexCoord;
	// create proper headers and quantizers
	// if not yet created by calling method
	MPG_Property_EXT pCoord(PT_COORD,PB_NONE);
	MPG_BBoxQ_EXT* coordQuantizer = getCoordQuantizer();
	if((MPG_BBoxQ_EXT*)0==coordQuantizer)
	{
		coordQuantizer    = new MPG_BBoxQ_EXT(3);
		setCoordQuantizer(coordQuantizer);
	}
	pCoord.setBBoxQ(coordQuantizer);
	
	MPG_Property_EXT pNormal(PT_NORMAL,PB_NONE);
	MPG_NormalQ_EXT* normalQuantizer = getNormalQuantizer();
	if((MPG_NormalQ_EXT*)0==normalQuantizer)
	{
		normalQuantizer   = new MPG_NormalQ_EXT();
		setNormalQuantizer(normalQuantizer);
	}
	pNormal.setNormalQ(normalQuantizer);
	
	MPG_Property_EXT pColor(PT_COLOR,PB_NONE);
	MPG_BBoxQ_EXT* colorQuantizer = getColorQuantizer();
	if((MPG_BBoxQ_EXT*)0==colorQuantizer)
	{
		colorQuantizer    = new MPG_BBoxQ_EXT(3);
		setColorQuantizer(colorQuantizer);
	}
	pColor.setBBoxQ(colorQuantizer);
	
	MPG_Property_EXT pTexCoord(PT_TEXCOORD,PB_NONE);
	MPG_BBoxQ_EXT* texCoordQuantizer = getTexCoordQuantizer();
	if((MPG_BBoxQ_EXT*)0==texCoordQuantizer)
	{
		texCoordQuantizer = new MPG_BBoxQ_EXT(2);
		setTexCoordQuantizer(texCoordQuantizer);
	}
	pTexCoord.setBBoxQ(texCoordQuantizer);
	
	// quantization parameters
	int bitsPerCoord    = 0;
	int bitsPerNormal   = 0;
	int bitsPerColor    = 0;
	int bitsPerTexCoord = 0;
	
	// input bitstream parsing --------------------------------------------------
	
	//MPG_EXT_Stitcher stitcher;
	vector<int>  v_father;
	//MPG_EXT_Stitches v_stitches;
	vector<int>  stk;  // stitch encoding stack
	
	vector<int>  v_visited;
	vector<int>  v_cluster;
	int          nV0    = 0; // number of vertices so far without current cc
	int          nV1    = 0; // number of vertices so far with current cc
	int          nVcls0 = 0; // AG: number of vertices after clustering
	int          nVcls1 = 0; // AG: number of vertices after clustering
	
	// open a bit stream
	MPG_InputBitStream_EXT ibstrm(istrm);

#ifdef ADD_VISUAL_HEADER
	{
		if(istrm.getVisualStartCode(visual_object_sequence_start_code)==false)
			return false;
		
		int profile_and_level_indication = 0;
		int is_visual_object_identifier = 0;
		int visual_object_verid = 0;
		int visual_object_priority = 0;
		int visual_object_type = 0;
		
		if(ibstrm.getMBitInt(8,profile_and_level_indication)==false) return false;
		//if(profile_and_level_indication != 0xFF) return false;	//MJH 062600 : 0xF1-->0xFF
		
		if(istrm.getVisualStartCode(visual_object_start_code)==false)
			return false;
		
		if(ibstrm.getMBitInt(1,is_visual_object_identifier)==false) return false;
		//if(is_visual_object_identifier != 0x01) return false;		
		
		if(is_visual_object_identifier == 0x01) //MJH 062600 :
		{
			if(ibstrm.getMBitInt(4,visual_object_verid)==false) return false;
			//if(visual_object_verid != 0x0F) return false;	
			
			if(ibstrm.getMBitInt(3,visual_object_priority)==false) return false;
			//if(visual_object_priority != 0x01) return false;		
		}
		
		if(ibstrm.getMBitInt(4,visual_object_type)==false) return false;
		//if(visual_object_type != 0x05) return false;		
	}
#endif //ADD_VISUAL_HEADER
	
	// decode 3D_Mesh_Object() 
	//
	// 3D_Mesh_Object() 
	//   3D_MO_start_code
	//   3D_Mesh_Object_Header()
	//   do 
	//     3D_Mesh_Object_Layer()
	//   while(nextbits_bytealigned()==3D_MOL_start_code)
	
	// read 3D_MO_start_code
	if(ibstrm.getMOStartCodeBitInt(MPG_Global_EXT::MO_start_code)==false)
		return false;

	// decode 3D_Mesh_Object_Header()
	{
		// 3D_Mesh_Object_Header() 
		//   ccw
		//   convex
		//   solid
		//   creaseAngle
		//   coord_header()
		//   normal_header()
		//   color_header()
		//   texCoord_header()
		//	3DMC_extension
		//	if (3DMC_extension == '1') 
		//		3DMC_extension_header()

		
		if(ibstrm.getMOBit(_ccw               )==false) return false;
		if(ibstrm.getMOBit(_convex            )==false) return false;
		if(ibstrm.getMOBit(_solid             )==false) return false;
		int cAng = 0;
		if(ibstrm.getMOBitInt(6,cAng)==false) return false;
		if(cAng<0 || cAng>=64) return false;
		_creaseAngle = M_PI*((float)cAng)/32;
		
		// decode coord_header()
		{
			// coord_header() 
			//   coord_binding       <--- must be == 1 (PER_VERTEX)
			
			int b = 0;
			if(ibstrm.getMBitInt(2,b)==false) return false;
			MPG_Binding binding =
				(b==0)?PB_NONE:(b==1)?PB_PER_VERTEX:(b==2)?PB_PER_FACE:PB_PER_CORNER;
			if(binding!=PB_NONE)
			{
				// coord can only be bound PER_VERTEX
				if(binding!=PB_PER_VERTEX) return false;
				pCoord.setBinding(binding);
				
				//   coord_bbox
				//   if(coord_bbox) 
				//     coord_xmin
				//     coord_ymin
				//     coord_zmin
				//     coord_size
				//   coord_quant
				//   coord_pred_type
				//   if (coord_pred_type=="tree_prediction" ||
				//       coord_pred_type=='"parallelogram_prediction") 
				//     coord_nlambda
				//     for(i=1;i<coord_nlambda;i++)
				//       coord_lambda[i]
				
				success = pCoord.decode(ibstrm);
				if(success==false) return false;
			}
		}
		
		// decode normal_header()
		{
			// normal_header() 
			//   normal_binding
			//   if(normal_binding!="not_bound") 
			
			int b = 0;
			if(ibstrm.getMBitInt(2,b)==false) return false;
			MPG_Binding binding =
				(b==0)?PB_NONE:(b==1)?PB_PER_VERTEX:(b==2)?PB_PER_FACE:PB_PER_CORNER;
			if(binding!=PB_NONE)
			{
				pNormal.setBinding(binding);
				
				//     normal_bbox       <--- must be == 0
				//     normal_quant
				//     normal_pred_type
				//     if (normal_pred_type=="tree_prediction"||
				//         normal_pred_type=="parallelogram_prediction") 
				//       normal_nlambda
				//       for(i=1;i<normal_nlambda;i++)
				//         normal_lambda[i]
				
				success = pNormal.decode(ibstrm);
				if(success==false) return false;
			}
			_normalPerVertex = (binding==PB_PER_FACE)?false:true;
		}
		
		// decode color_header()
		{
			// color_header()
			//   color_binding
			//   if(color_binding!="not_bound")
			
			int b = 0;
			if(ibstrm.getMBitInt(2,b)==false) return false;
			MPG_Binding binding =
				(b==0)?PB_NONE:(b==1)?PB_PER_VERTEX:(b==2)?PB_PER_FACE:PB_PER_CORNER;
			if(binding!=PB_NONE)
			{
				pColor.setBinding(binding);
				
				//     color_bbox
				//     if(color_bbox == '1') 
				//       color_rmin
				//       color_gmin
				//       color_bmin
				//       color_size
				//     color_quant
				//     color_pred_type
				//     if (color_pred_type=="tree_prediction" ||
				//         color_pred_type=='"parallelogram_prediction")
				//       color_nlambda
				//       for(i=1;i<color_nlambda;i++)
				//         color_lambda[i]
				
				success = pColor.decode(ibstrm);
				if(success==false) return false;
			}
			_colorPerVertex = (binding==PB_PER_FACE)?false:true;
		}
		
		// decode texture_header()
		{
			// texCoord_header() 
			//   texCoord_binding     <--- must be != 2 (PER_FACE)
			//   if(texCoord_binding!="not_bound")
			
			int b = 0;
			if(ibstrm.getMBitInt(2,b)==false) return false;
			MPG_Binding binding =
				(b==0)?PB_NONE:(b==1)?PB_PER_VERTEX:(b==2)?PB_PER_FACE:PB_PER_CORNER;
			if(binding!=PB_NONE)
			{
				// TexCoord cannot be bound per_face
				if(binding==PB_PER_FACE) return false;
				pTexCoord.setBinding(binding);
				
				//     texCoord_bbox
				//     if(texCoord_bbox=='1')
				//       texCoord_umin
				//       texCoord_vmin
				//       texCoord_size
				//     texCoord_quant
				//     texCoord_pred_type
				//     if (texCoord_pred_type=="tree_prediction" ||
				//         texCoord_pred_type=="parallelogram_prediction") 
				//       texCoord_nlambda
				//       for(i=1;i<texCoord_nlambda;i++)
				//         texCoord_lambda[i]
				
				//DMLAB (200501) - UPDATE
				success = pTexCoord.texdecode(ibstrm);
				pTexCoord.delta_flag=false;
				//~DMLAB (200501)
				if(success==false) return false;
			}
		}

		// decode extension header
		//3DMC_extension_header() {
		//	do {
		//		 function_type
		//		 if (function_type == "Order_mode")
		//			 Order_mode_header ()
		//		else if (function_type == "Adaptive_quant_texCoord_mode")
		//			Adaptive_quant_texCoord_mode_header()
		//	 } while(function_type  != "Escape_mode")
		//}

		{
			_3DMC_extension = false;
			if(ibstrm.getBit(_3DMC_extension)==false) return false;
			if(_3DMC_extension==true)
			{
				int function_type;	
				while(1)
				{
					ibstrm.getMBitInt(4, function_type);
					if (function_type == ESCAPE_MODE) break; // Escape_mode = 1111
					else if (function_type == ORDER_MODE) //Order_mode = 0000
					{
						ibstrm.getMOBitInt(1, vertex_order_flag);
						if(vertex_order_flag)
							ibstrm.getMOBitInt(1, vertex_order_per_CC_flag);
						ibstrm.getMOBitInt(1, face_order_flag);
						if(face_order_flag)
							ibstrm.getMOBitInt(1, face_order_per_CC_flag);
					}
					else if (function_type == ADAPTIVE_QUANT_TEXCOORD_MODE)//Adaptive_quant_texCoord_mode = 0001
					{
						int texCoord_quant_u,texCoord_quant_v;
						if(ibstrm.getMBitInt(16, texCoord_quant_u)==false) return false;
						if(ibstrm.getMBitInt(16, texCoord_quant_v)==false) return false;
						pTexCoord.delta_u=texCoord_quant_u;
						pTexCoord.delta_v=texCoord_quant_v;
						pTexCoord.delta_flag=true;
					}
				}
			}
		}		
		// flush input buffer
		ibstrm.flush();
  } // end decode 3D_Mesh_Object_Header()
  
  bitsPerCoord    =  pCoord.getBBoxQ()->getBitsPerCoordinate();
  bitsPerNormal   =  pNormal.getNormalQ()->getBitsPerNormal();
  bitsPerColor    =  pColor.getBBoxQ()->getBitsPerCoordinate();
  bitsPerTexCoord =  pTexCoord.getBBoxQ()->getBitsPerCoordinate();
  // decode one or more 3D_Mesh_Object_Layer()
  int mol_id = 0;
  do {
	  // decode 3D_Mesh_Object_Layer()
	  
	  // 3D_Mesh_Object_Layer()
	  //   3D_MOL_start_code
	  //   mol_id
	  //   if(mol_id==0)
	  //     3D_Mesh_Object_Base_Layer()
	  //   else
	  //     3D_Mesh_Object_Extension_Layer()
	  
	  // read 3D_MOL_start_code
	  if(istrm.getStartCode(MPG_Global_EXT::MOL_start_code)==false)
		  return false;
	  {
		  // open a bit stream
		  MPG_InputBitStream_EXT ibstrm(istrm);
		  
		  // read mol_id
		  if(ibstrm.getMBitInt(8,mol_id)==false) return false;
	  		  
		  ibstrm.flush();
	  }
	  bool has_stitches=false;
	  if(mol_id==0)
	  {
		  //
		  // decode 3D_Mesh_Object_Base_Layer()
		  //
		  
		  // 3D_Mesh_Object_Base_Layer() 
		  //	do 
		  //    3D_MOBL_start_code
		  //    mobl_id
		  //		while (!bytealigned())
		  //			one_bit
		  //		qf_start()
		  //		if (3D_MOBL_start_code == "partition_type_0") 
		  //			...
		  //		else if (3D_MOBL_start_code == "partition_type_1") 
		  //			...
		  //		else if (3D_MOBL_start_code == "partition_type_2") 
		  //			...
		  //	while (nextbits_bytealigned() == 3D_MOBL_start_code)
		  
		  int previous_mobl_id = -1; // Index of previous MOBL; used to check correct sequencing
		  int vg_number = 0;
		  int nVcc;
		  int totalVertices=0,ory_temp=0;
		  typedef vector<int> vec_array;
		  vector<vec_array> v_loop_cc;
		  vector<int> nVtotal;	// for pt1
		  MPG_SPolygon_EXT sp;	//MJH 001006 : moved from below
		  do {  // 3D_Mesh_Object_Base_Layer()
			  // read 3D_MOBL_start_code
			  // get mobl start code and determine the partition type
		 	  
synchronize:
		  int partition_type;
		  unsigned char mobl_sc = istrm.getMOBLStartCode(); 
		  if (mobl_sc == MPG_Global_EXT::MOBL0_start_code)
			  partition_type = 0;
		  else if (mobl_sc == MPG_Global_EXT::MOBL1_start_code)
			  partition_type = 1;
		  else if (mobl_sc == MPG_Global_EXT::MOBL2_start_code)
			  partition_type = 2;
		  else
			  break;
		  // open a bit stream
		  MPG_InputBitStream_EXT ibstrm(istrm);     
		  
		  // read mobl_id
		  int mobl_id = 0;
		  if(ibstrm.getMBitInt(8,mobl_id)==false) 
		  {
			  fprintf(stderr, "Error in mobl_id\n");
			  goto synchronize;	
		  }
		  //			else
		  //				printf("decoding mobl : %d\n", mobl_id);
		  
		  int correct_sequencing = (mobl_id == ((previous_mobl_id + 1) & 0xff));
		  //Check for partition loss error
		  if (!correct_sequencing) {
			  printf("Warning: partition number sequence incomplete. ");
			  printf("(%d, %d)\n", previous_mobl_id, mobl_id);
		  }
		  previous_mobl_id = mobl_id;
		  
		  ibstrm.flush();
		  
		  //  start the arithmetic codec
		  MPG_QFac_EXT codec(istrm);
		  codec.start_decode();
		  MPG_Global_EXT::reset();
		  
		  ///////////////////////////////////////////////////////////////
		  // partition type 0 ( one or more cc's)
		  //	if (3D_MOBL_start_code == "partition_type_0") 
		  //		do 
		  //			connected_component()
		  //			qf_decode(last_component, last_component_context)
		  //		while (last_component == '0')
		  //
		  if (partition_type == 0) {	
			  vg_number=0;	
			  do {
				  // decode connected_component()
				  
				  // connected_component() 
				  //   vertex_graph()
				  //   qf_decode(has_stitches, has_stitches_context)
				  //   if(has_stitches=='1')
				  //     stitches()
				  //   triangle_tree()
				  //   triangle_data()
				  // 
				  
				  // decode vertex_graph() and construct vertex loop
				  nV0 = nV1;	
				  v_loop_cc.clear(); 
				  vec_array v_loop_cc0;
				  v_loop_cc.push_back(v_loop_cc0);
				  {
					  // decode vertex_graph()
					  MPG_VGraph_EXT vg;
					  if(vg.decode(codec) == false) return false;
					  
					  nVcc = vg.getNumberOfVertices();
					  totalVertices+=nVcc;
					  nV1 = nV0 + nVcc;
					  
					  while(v_visited.size()<nV1)
						  v_visited.push_back(false);
					  
					  // convert to vertex loop look-up table
					  MPG_Loops_EXT loop;
					  vg.toLoop(loop);
					  loop.getLoop(0, v_loop_cc[0]);
					  
					  if(vg.getFather(v_father)==false || v_father.size()!=nV1)
						  return false;
				  }
				  
				  //v_stitches.decode(nV0,nV1,stk,codec);
				  has_stitches = (codec.decode(MPG_Global_EXT::stitch_has)==1)?true:false;
				   {
					  // use stitches nV0<=i<nV1 to partially recover v_cluster
					  int nV1 = v_father.size(); 
					  int nVcc = nV1-nV0;
					  
					  for (int i = nV0;i < nV1; i++) v_cluster.push_back(i);					  
					  //if(stitcher.use(nV0,v_father,v_stitches,v_cluster)==false)
					  //	return false;
				  }
				  					  

				  // we should have here :
				  // v_cluster.size()==nV1
				  if(v_cluster.size()!=nV1)
					  return false;

				  // for i=0,..,nV1-1
				  //   v_cluster[i]>=0 && v_cluster[i]<nV1
				  // v_father.size()==nV1 
				  // for i=0,..,nV1-1
				  //   v_father[i]<=i
				  // v_stitches.size()==nV1
				  
				  // decode triangle_tree()
				  //					MPG_SPolygon_EXT sp;	//MJH 001006 : moved to top
				  sp.clear();	//MJH 001006
				  
				  sp.codap_left_bloop_idx = 1;	
				  sp.codap_right_bloop_idx = 0; 
				  sp.codap_bdry_pred = 1;
				  
				  if(sp.decodeTriangleTree(codec,partition_type,v_loop_cc[0].size()) == false)	
					  goto synchronize;
				  
				  // decode triangle_data()
			      vector<int>   coordIndex;
				  if(sp.decodeTriangleData(codec,
					  nVcc,
					  nV0,
					  v_loop_cc[0],
					  v_visited,
					  v_cluster,
					  coordIndex,
					  // property headers and quantizers
					  pCoord,
					  pNormal,
					  pColor,
					  pTexCoord,
					  // quantized values
					  qCoord,
					  qNormal,
					  qColor,
					  qTexCoord,
#ifdef MPGELIB
					  newIFS,	//MJH 082100
#endif
					  partition_type	
					  )==false) 
					  goto synchronize;	
				
				  // transfer coordIndex to Ifs
				  {
					  
					  int j0,j1,j;	
					  for(j0=j1=0;j1<coordIndex.size();j1++)
						  if(coordIndex[j1]==-1)	
						  {
							  int nf = j1-j0;
							  //								bool skip_poly = 0;		//MJH 000111 : skip_poly related not needed for pt0
							  int* f = &(coordIndex[j0]);
							  for(j=0;j<nf;j++)
							  {
								  //									if (f[j] == -2)
								  //										skip_poly = 1;
								  //									if (skip_poly)
								  //										_coordIndex.push_back(-2);
								  //									else
								  _coordIndex.push_back(f[j]);
							  }
							  _coordIndex.push_back(-1);
							  //								skip_poly = 0;
							  j0 = j1+1;
						  }
				  
				  }
				  
			} while(codec.decode(MPG_Global_EXT::last_component) == false);	
			
			nVtotal.clear();
			nVtotal.push_back(nV0);
			nVtotal.push_back(nV1);	
		}
		
		///////////////////////////////////////////////////////////////
		// partition type 1 ( one or more vg's)
		//	else if (3D_MOBL_start_code == "partition_type_1") 
		//		vg_number=0
		//		do 
		//			vertex_graph()
		//			vg_number++
		//			qf_decode(has_stitches, has_stitches_context)
		//			if (has_stitches == '1')
		//				stitches()
		//			qf_decode(codap_last_vg, codap_last_vg_context)
		//		while (codap_last_vg == '0')
		//		
		else if (partition_type == 1) {	
			vg_number=0;	
			v_loop_cc.clear(); 
			nVtotal.clear();
			nVtotal.push_back(nV1);
			int codap_last_vg;
			
			do {
				nV0 = nV1;	
				
				vec_array v_loop_cc1;
				v_loop_cc.push_back(v_loop_cc1);
				
				// decode vertex_graph()
				MPG_VGraph_EXT vg;
				if(vg.decode(codec) == false) return false;
				
				nVcc = vg.getNumberOfVertices();
                totalVertices+=nVcc;

				nV1 = nV0 + nVcc;
				
				nVtotal.push_back(nV1);
				
				while(v_visited.size()<nV1)
					v_visited.push_back(false);

						  
				// convert to vertex loop look-up table
				MPG_Loops_EXT loop;
				vg.toLoop(loop);
				loop.getLoop(0, v_loop_cc[vg_number]);
				
				if(vg.getFather(v_father)==false || v_father.size()!=nV1)
				{
					return false;
				}
				
				//v_stitches.decode(nV0,nV1,stk,codec);
				has_stitches = (codec.decode(MPG_Global_EXT::stitch_has)==1)?true:false;
				{
					// use stitches nV0<=i<nV1 to partially recover v_cluster
					int nV1 = v_father.size(); 
					int nVcc = nV1-nV0;
					for (int i = nV0;i < nV1; i++) v_cluster.push_back(i);
					
					//if(stitcher.use(nV0,v_father,v_stitches,v_cluster)==false)
					//	return false;
				}
				
				// we should have here :
				// v_cluster.size()==nV1

				if(v_cluster.size()!=nV1)
					return false;
				
				// for i=0,..,nV1-1
				//   v_cluster[i]>=0 && v_cluster[i]<nV1
				// v_father.size()==nV1 
				// for i=0,..,nV1-1
				//   v_father[i]<=i
				// v_stitches.size()==nV1
				
				vg_number++;	
				codap_last_vg = codec.decode(MPG_Global_EXT::codap_last_vg_context);
				
			} while (!codap_last_vg);
			
		} // if (partition_type == 1)
		
		///////////////////////////////////////////////////////////////
		// partition type 2 (one tt/td pair)
		//	else if (3D_MOBL_start_code == "partition_type_2") 
		//		if(vg_number > 1)
		//			qf_decode(codap_vg_id)
		//		qf_decode(codap_left_bloop_idx)
		//		qf_decode(codap_right_bloop_idx)
		//		qf_decode(codap_bdry_pred)
		//		triangle_tree()
		//		triangle_data()
		//		
		else { // if (partition_type == 2)
			
			// decode triangle_tree()
			//			MPG_SPolygon_EXT sp;	//MJH 001006 : moved to top
			sp.clear();	//MJH 001006
			
			int l,nbits;
			int codap_vg_id = 0;
			if (vg_number>1) {
				nbits = slog2(vg_number);
				for (l=0; l<nbits; l++)
					codap_vg_id += codap_vg_id + codec.decode();
			}
			nbits = slog2(v_loop_cc[codap_vg_id].size());
			sp.codap_left_bloop_idx = 0;	
			for (l=0; l<nbits; l++)
				sp.codap_left_bloop_idx += sp.codap_left_bloop_idx + codec.decode();
			sp.codap_right_bloop_idx = 0; 
			for (l=0; l<nbits; l++)
				sp.codap_right_bloop_idx += sp.codap_right_bloop_idx + codec.decode();
			sp.codap_bdry_pred = codec.decode();
			
			// decode triangle_tree()
			if(sp.decodeTriangleTree(codec,partition_type,v_loop_cc[codap_vg_id].size()) == false) //return false;
				goto synchronize;
			
			nV0 = nVtotal[codap_vg_id];
			nVcc = nVtotal[codap_vg_id+1]-nVtotal[codap_vg_id];
			
			// decode triangle_data()
            vector<int>   coordIndex;
			if(sp.decodeTriangleData(codec,
				nVcc,
				nV0,
				v_loop_cc[codap_vg_id],
				v_visited,
				v_cluster,
				coordIndex,
				// property headers and quantizers
				pCoord,
				pNormal,
				pColor,
				pTexCoord,
				// quantized values
				qCoord,
				qNormal,
				qColor,
				qTexCoord,
#ifdef MPGELIB
				newIFS,	//MJH 082100
#endif
				partition_type	
				)==false) 
				goto synchronize;	
			// transfer coordIndex to Ifs
			{
				int j0,j1,j;
				for(j0=j1=0;j1<coordIndex.size();j1++)
					if(coordIndex[j1]==-1)	
					{
						int nf = j1-j0;
						bool skip_poly = 0;		
						int* f = &(coordIndex[j0]);
						for(j=0;j<nf;j++)
						{
							if (f[j] == -2)
								skip_poly = 1;
							_coordIndex.push_back(f[j]);
						}
						if (skip_poly)
						{
							int corner_cnt;
							//MJH 052500 : _coordIndex.size()>0 added in case the first triangle in _coordIndex has missing vertex
							for (corner_cnt=0; _coordIndex.back() != -1 && _coordIndex.size()>0 
								; corner_cnt++)
								_coordIndex.pop_back();
							for ( ; corner_cnt>0; corner_cnt--)
								_coordIndex.push_back(-2);
						}
						_coordIndex.push_back(-1);
						skip_poly = 0;
						j0 = j1+1;
					}
			}
		} // if (partition_type == 2)
		///////////////////////////////////////////////////////////////
		
		codec.finish_decode();


	} while (1);	// 3D_Mesh_Object_Base_Layer()


// dmlab stitch process _ lsw 
//	bool has_stitches = false;
//	if (ibstrm.getBit(has_stitches) == false) return false;
	numOfvertexStitching = 0;
	if(has_stitches)
	{
		printf("stitching starts\n");
 		int readBits = representBit(totalVertices);
		bool has_vertex_increase = false;
		bool has_face_increase = false;
		if (ibstrm.getBit(has_vertex_increase)== false) return false;
		if (ibstrm.getBit(has_face_increase)== false) return false;
		if (has_vertex_increase) 
		{
			int num, loop;
			ibstrm.getMBitInt(readBits, loop);
			for(int j = 0; j < loop; j++)
			{
				ibstrm.getMBitInt(readBits, num);
				for(int i = 0; i < num; i++)
				{
					int vertexLoc;
					ibstrm.getMBitInt(readBits, vertexLoc);
					vertexStitch.push_back(vertexLoc);
				}				
				vertexStitch.push_back(-1);
				numOfvertexStitching++;
			}
		}
	
	}			
	
	
	
	
	int i0 = 0; 
	nV0 = 0;	
	
	// determine the new number of vertices after clustering
	{
		int i;
		for (i=nV0; i < nV1; i++)
			if (v_cluster[i] > nVcls1) nVcls1 = v_cluster[i];
	}
	int nVcls11 = nVcls1+1;

	// unquantize vertices between nVcls0 and nVcls1
	// coord properties
	vector<float> coord((size_t) (3*(nVcls11-nVcls0)),0.0);
	MPG_BBoxQ_EXT*   coordQuantizer    = pCoord.getBBoxQ();
	coordQuantizer->unQuantize(qCoord.begin()+3*nVcls0,
		coord.begin(),
		nVcls11-nVcls0);
	
	//  normal properties
	vector<float> normal;
	MPG_NormalQ_EXT* normalQuantizer   = pNormal.getNormalQ();
	if(pNormal.getBinding() != PB_NONE)
	{
		// special case for per-vertex binding,
		// the qNormal coordinates are global to allow reuse
		// of some vertices by components decoded afterwards
		if(pNormal.getBinding() == PB_PER_VERTEX)
		{
			// take only the trailing part of qNormal
			vector <int> qNormal1;
			int i;
			for (i=nVcls0;i<nVcls11;i++)
				qNormal1.push_back(qNormal[i]);
			
			normalQuantizer->unQuantize(qNormal1, normal);
		}
		else normalQuantizer->unQuantize(qNormal, normal);
	}
	//  color properties
	vector<float> color;
	MPG_BBoxQ_EXT*   colorQuantizer    = pColor.getBBoxQ();
	if(pColor.getBinding() != PB_NONE)
	{
		if(pColor.getBinding() == PB_PER_VERTEX)
		{
			color.erase(color.begin(),color.end());
			color.insert(color.end(),(size_t)(3*(nVcls11-nVcls0)),0.0);
			colorQuantizer->unQuantize(qColor.begin()+3*nVcls0,
				color.begin(),
				nVcls11-nVcls0);
		}
		else colorQuantizer->unQuantize(qColor, color);
	}
	//  texture properties
	vector<float> texCoord;
	MPG_BBoxQ_EXT*   texCoordQuantizer = pTexCoord.getBBoxQ();
	if(pTexCoord.getBinding() != PB_NONE)
	{
		//DMLAB (200501) - ADD
		int delta_flag = pTexCoord.delta_flag;
		if(delta_flag) {		
			texCoordQuantizer->delta_u = pTexCoord.delta_u;
			texCoordQuantizer->delta_v = pTexCoord.delta_v;
		}
		//~DMLAB (200501)
		if(pTexCoord.getBinding() == PB_PER_VERTEX)
		{
			texCoord.erase(texCoord.begin(),texCoord.end());
			texCoord.insert(texCoord.end(),
				(size_t)(2*(nVcls11-nVcls0)),0.0);
			
			if(delta_flag)
				texCoordQuantizer->texunQuantize(qTexCoord.begin()+2*nVcls0,
				texCoord.begin(),
				nVcls11-nVcls0);
			else
				texCoordQuantizer->unQuantize(qTexCoord.begin()+2*nVcls0,
				texCoord.begin(),
				nVcls11-nVcls0);
		}
		else
			if(delta_flag)
				texCoordQuantizer->texunQuantize(qTexCoord, texCoord);
			else
				texCoordQuantizer->unQuantize(qTexCoord, texCoord);
	}
	
	// transfer coord to Ifs
	{
		int i;
		for(i=0;i<coord.size();i++) _coord.push_back(coord[i]);
	}
	
	if(pNormal.getBinding()==PB_PER_CORNER)
	{
		// adjust _normalIndex
		// normal.size() should be equal to
		// the number of new corners
		int i = i0;
		int j = _normal.size()/3;
		for(;i<_coordIndex.size();i++)
			if(_coordIndex[i]!=-1)
			{ _normalIndex.push_back(j); j++; }
			else	
			{ 
				if (_normalIndex.back()==-2)
				{
					while (_normalIndex.back() != -1)
						_normalIndex.pop_back();
				}
				else
					_normalIndex.push_back(-1); 
			}
	}
	
	// transfer normal to Ifs
	{
		int i;
		for(i=0;i<normal.size();i++)
			_normal.push_back(normal[i]);
	}
	
	if(pColor.getBinding()==PB_PER_CORNER)
	{
		// adjust _colorIndex
		// color.size() should be equal to
		// the number of new corners
		int i = i0;
		int j = _color.size()/3;
		for(;i<_coordIndex.size();i++)
			if(_coordIndex[i]!=-1)
			{ _colorIndex.push_back(j); j++; }
			else
			{ 
				if (_colorIndex.back()==-2)
				{
					while (_colorIndex.back() != -1)
						_colorIndex.pop_back();
				}
				else
					_colorIndex.push_back(-1); 
			}
	}
	
	// transfer color to Ifs
	{
		int i;
		for(i=0;i<color.size();i++)
			_color.push_back(color[i]);
	}
	
	if(pTexCoord.getBinding()==PB_PER_CORNER)
	{
		// adjust _texCoordIndex
		// texCoord.size() should be equal to
		// the number of new corners
		int i = i0;
		int j = _texCoord.size()/2;
		for(;i<_coordIndex.size();i++)
			if(_coordIndex[i]!=-1)
			{ _texCoordIndex.push_back(j); j++; }
			else
			{ 
				if (_texCoordIndex.back()==-2)
				{
					while (_texCoordIndex.back() != -1)
						_texCoordIndex.pop_back();
				}
				else
					_texCoordIndex.push_back(-1); 
			}
	}
	
	// transfer texCoord to Ifs
	{
		int i;
		for(i=0;i<texCoord.size();i++)
			_texCoord.push_back(texCoord[i]);
	}
	
	nVcls0 = nVcls11;
	nV0 = nV1;

	ibstrm.flush();
    }
    else // if(mol_id!=0)
    {
		//3D_Mesh_Object_Extension_Layer() {
		//	if(vertex_order_flag ) 
		//     {
		//		vertex_order()
		//		if(vertex_order_per_CC_flag)
		//			vertex_order_per_CC()
		//     }
		//	if(face_order_flag)
		//      {
		//		face_order()
		//		if(face_order_per_CC_flag)
		//			face_order_per_CC()
		//      }
		//}

		//DK
		//---------------------------------decoding-------------------------------// 
		int i, j;
		vector <int> offset_all;
		vector <int> first_ID_all;
		vector <int> num_CC_offset_all;

		//DMLAB ory Print Information 2006 05
	    MPG_Mesh_EXT mesh(getCoordIndex());
		int nnCC = mesh.getNumberOfConnectedComponents();
		int nV_total = mesh.getNumberOfVertices();
		int nF_total = mesh.getNumberOfFaces();
		printf("# of cc : %d\n", nnCC);
		printf("# of vertices : %d\n",nV_total);
		printf("# of faces : %d\n",nF_total);
		//~DMLAB ory

		if(vertex_order_flag)
		{
			vector <int> array_cnt;
			vector <int> array_accmul_cnt;

		    MPG_InputBitStream_EXT ibstrm(istrm);     
			printf("vo processing\n");
			if(vertex_order_per_CC_flag)
			{
				int *v_order_total = new int[nV_total];
				//DMLAB ory 2006 07
				int vtemp=0;
				//~DMLAB ory
				for(i=0;i<nnCC;i++)
				{
					int nVOffset;
					ibstrm.getMOBitInt(16, nVOffset);
					num_CC_offset_all.push_back(nVOffset);//dk
					
					int vo_offset, firstVID;
					for(j=0;j<nVOffset;j++)
					{
						ibstrm.getMOBitInt(24, vo_offset);
						offset_all.push_back(vo_offset);//dk
						ibstrm.getMOBitInt(24, firstVID);
						first_ID_all.push_back(firstVID);//dk
					}		 
					//DMLAB ory 2006 07
					vtemp+=mesh.getNumberOfVerticesCC(i);
					array_cnt.push_back(mesh.getNumberOfVerticesCC(i));
					array_accmul_cnt.push_back(vtemp);
					//~DMLAB ory
				}
				
				int iter_for_offset=0;
				for(i=0;i<nnCC;i++)
				{
					int *offset,*firstVID;
					int *v_order_IFS_mapping;
					
					v_order_IFS_mapping= new int[array_cnt[i]];
					offset = new int[num_CC_offset_all[i]];
					firstVID = new int[num_CC_offset_all[i]+1];
					for(int l=0;l<num_CC_offset_all[i];l++)
					{
						offset[l]=offset_all[iter_for_offset];
						firstVID[l]=first_ID_all[iter_for_offset];
						iter_for_offset++;
					}
					
					int totalVertices=array_cnt[i];
					vector <int> vertexVector;
					for(int q = 0; q < totalVertices; q++)
						vertexVector.push_back(q);
					
					int m;
					
					//DMLAB ory  bitsToRepresent 2006 05
					int vo_id, DecordingVertices, init_bpvi = bitsToRepresent(totalVertices), bpvi, itr;
					//~DMLAB ory 
					
					int orgTotalVx=totalVertices;
					
					for(bpvi=init_bpvi; bpvi>0; bpvi--)	// ݺ Ƚ Vertex ŭ
					{
						DecordingVertices = totalVertices - (1<<(bpvi-1));
						totalVertices = totalVertices - DecordingVertices;
						
						int* delList = new int[DecordingVertices];
						int delCnt = 0;
						
						for(itr=0; itr<DecordingVertices; itr++)
						{
							//vo_decode(vo_id, bpvi);
							ibstrm.getMOBitInt(bpvi,vo_id);
							if (bpvi == init_bpvi)
							{
								qVertexIndex.push_back(vo_id);//dk
								delList[delCnt++] = vo_id;
							}
							else
							{
								qVertexIndex.push_back(vertexVector[vo_id]);
								delList[delCnt++] = vo_id;
								
							}
						}
						qsort(delList, delCnt, sizeof(int), compare);
						
						for(q = 0; q < delCnt; q++)
							vertexVector.erase(vertexVector.begin() + delList[q] - q);
						
						delete delList;
					}
					qVertexIndex.push_back(vertexVector[0]);
					vertexVector.erase(vertexVector.begin());	
					
					
					//DMLAB ory Vertex Reordering 2006 05
					int offset_size = num_CC_offset_all[i];
					firstVID[offset_size] = array_accmul_cnt[nnCC-1];
					if(offset_size>0)
					{				
						for(m=0;m<array_cnt[i];m++)
						{
							if(offset_size == 1)
							{
								vo_id=qVertexIndex[m];
								if(qVertexIndex[m] >= firstVID[0] )
								{
									v_order_IFS_mapping[m] = qVertexIndex[m] + offset[0];
								}
								else
								{
									v_order_IFS_mapping[m] = qVertexIndex[m];
								}
							}
							else
							{
								v_order_IFS_mapping[m]=vo_id=qVertexIndex[m];							
								for(int k=0;k<offset_size;k++)
								{								
									if(vo_id>=firstVID[k] && vo_id<firstVID[k+1])
									{
										int final_vo_id=vo_id+offset[k];
										v_order_IFS_mapping[m]=final_vo_id;
										break;
									}
								}
								int t = v_order_IFS_mapping[m];
							}
							
						}
						
					}
					else
					{
						for(m=0;m<array_cnt[i];m++)
						{					
							v_order_IFS_mapping[m] = qVertexIndex[m];
						}
					}
					qVertexIndex.erase(qVertexIndex.begin(),qVertexIndex.end());
					ibstrm.flush();		
					
					if(i==0)
					{
						for(l = 0; l<array_cnt[i]; l++)
						{
							v_order_total[l] = v_order_IFS_mapping[l];
						}
					}
					else
					{
						for(l = array_accmul_cnt[i-1]; l<array_accmul_cnt[i];l++)
						{
							v_order_total[l] = v_order_IFS_mapping[l-array_accmul_cnt[i-1]];
						}
					}	
				delete offset;
				delete firstVID;
				delete v_order_IFS_mapping;
			} // end for
			for(int l=0;l<array_accmul_cnt[nnCC-1];l++)
			{
				qVertexIndex.push_back(v_order_total[l]);
			}
			delete v_order_total;
			//~DMLAB ory
		}
		else{
			int totalVertices=qCoord.size()/3;
			//DMLAB ory  bitsToRepresent 2006 05
			int vo_id, DecordingVertices, init_bpvi = bitsToRepresent(totalVertices);
			int bpvi, itr;
			vector <int> heapVector;
			//~DMLAB ory
			// dmlab heap vector _ lsw 060703
			
			
			for(int i = 0; i < totalVertices; i++)
				heapVector.push_back(i);
			
			for(bpvi=init_bpvi; bpvi>0; bpvi--)	// ݺ Ƚ Vertex ŭ
			{
				DecordingVertices = totalVertices - (1<<(bpvi-1));
				totalVertices = totalVertices - DecordingVertices;
				int* delList = new int[DecordingVertices];
				int delCnt = 0;
				
				for(itr = 0; itr < DecordingVertices; itr++)
				{
					//vo_decode(vo_id, bpvi);
					ibstrm.getMOBitInt(bpvi, vo_id);
					if (bpvi == init_bpvi)
					{
						qVertexIndex.push_back(vo_id);
						delList[delCnt++] = vo_id;
					}
					else
					{
						qVertexIndex.push_back(heapVector[vo_id]);
						delList[delCnt++] = vo_id;
					}
				}
				
				qsort((void *)delList, delCnt, sizeof(int), compare);
				
				for(int q = 0; q < delCnt; q++)
				{
					heapVector.erase(heapVector.begin() + delList[q] - q);
				}
				
				delete delList;
			}
			if (heapVector.size() != 1) printf("when reading vertex info, some problems occured\n");
			qVertexIndex.push_back(heapVector[0]);
			heapVector.erase(heapVector.begin(),heapVector.end());
		}
		ibstrm.flush();
	}
	num_CC_offset_all.erase(num_CC_offset_all.begin(),num_CC_offset_all.end());
	offset_all.erase(offset_all.begin(),offset_all.end());
	first_ID_all.erase(first_ID_all.begin(),first_ID_all.end());
	//need to get face_array_cnt[]
	
	if(face_order_flag)
	{
		vector <int> face_array_cnt;
		vector <int> face_array_accmul_cnt;
       
		MPG_InputBitStream_EXT ibstrm(istrm);     	
		printf("fo processing\n");
		if(face_order_per_CC_flag)
		{
			int *f_order_total = new int[nF_total];
			//DMLAB ory 2006 07
			int ftemp=0;
			//~DMLAB ory
			for(i=0;i<nnCC;i++)
			{
				int nVOffset;
				ibstrm.getMOBitInt(16, nVOffset);
				num_CC_offset_all.push_back(nVOffset);//dk
				
				int vo_offset, firstVID;
				for(j=0;j<nVOffset;j++)
				{
					ibstrm.getMOBitInt(24, vo_offset);
					offset_all.push_back(vo_offset);//dk
					ibstrm.getMOBitInt(24, firstVID);
					first_ID_all.push_back(firstVID);//dk
				}
				//DMLAB ory 2006 07
				ftemp+=mesh.getNumberOfFacesCC(i);
				face_array_cnt.push_back(mesh.getNumberOfFacesCC(i));
				face_array_accmul_cnt.push_back(ftemp);
				//~DMLAB ory
			}
			
			
			// DMLab-soho(Vertex Order Decording)
			//DMLAB ory Face Reordering 2006 05
			int iter_for_offset=0;
			for(i=0;i<nnCC;i++)
			{				
				int *offset,*firstVID;
				int *f_order_IFS_mapping;
				f_order_IFS_mapping= new int[face_array_cnt[i]];//array_cnt???
				offset = new int[num_CC_offset_all[i]];
				firstVID = new int[num_CC_offset_all[i]+1];

				for(int l=0;l<num_CC_offset_all[i];l++)
				{
					offset[l]=offset_all[iter_for_offset];
					firstVID[l]=first_ID_all[iter_for_offset];
					iter_for_offset++;
				}
				
				int totalFace=face_array_cnt[i];		
				int m;
				//DMLAB ory  bitsToRepresent 2006 05
				int fo_id, DecordingFace, init_bpfi = bitsToRepresent(totalFace), bpfi, itr;
				//~DMLAB ory
				vector <int> faceVector;
				for(int q = 0; q < totalFace; q++)
					faceVector.push_back(q);
				
				int orgTotalFx=totalFace;
				
				for(bpfi=init_bpfi; bpfi>0; bpfi--)	// ݺ Ƚ Vertex ŭ
				{
					DecordingFace = totalFace - (1<<(bpfi-1));
					totalFace = totalFace - DecordingFace;
					int* delList = new int[DecordingFace];
					int delCnt = 0;
					
					for(itr=DecordingFace; itr>0; itr--)
					{
						//fo_decode(fo_id, bpfi)
						ibstrm.getMOBitInt(bpfi, fo_id);
						if (bpfi == init_bpfi)
						{
							qFaceIndex.push_back(fo_id);
							delList[delCnt++] = fo_id;
						}
						else
						{
							qFaceIndex.push_back(faceVector[fo_id]);
							delList[delCnt++] = fo_id;
						}
					}
					
					qsort(delList, delCnt, sizeof(int), compare);
					
					for(int q = 0; q < delCnt; q++)
						faceVector.erase(faceVector.begin()+delList[q]-q);
					delete [] delList;			
				}
				
				qFaceIndex.push_back(faceVector[0]);
				faceVector.erase(faceVector.begin());
				
				int offset_size = num_CC_offset_all[i];
				firstVID[offset_size] = face_array_accmul_cnt[nnCC-1];
				if(offset_size>0)
				{				
					for(m=0;m<face_array_cnt[i];m++)
					{
						if(offset_size == 1)
						{
							if(qFaceIndex[m] >= firstVID[0] )
							{
								f_order_IFS_mapping[m] = qFaceIndex[m] + offset[0];
							}
							else
							{
								f_order_IFS_mapping[m] = qFaceIndex[m];
							}
						}
						else
						{
							f_order_IFS_mapping[m]=fo_id=qFaceIndex[m];
							for(int k=0;k<offset_size;k++)
							{
								if(fo_id>=firstVID[k] && fo_id<firstVID[k+1])
								{
									int final_fo_id=fo_id+offset[k];
									f_order_IFS_mapping[m]=final_fo_id;
									break;
								}
							}
						}						
					}					
				}
				else
				{
					for(m=0;m<face_array_cnt[i];m++)
					{					
						f_order_IFS_mapping[m] = qFaceIndex[m];
					}
				}
				
				qFaceIndex.erase(qFaceIndex.begin(),qFaceIndex.end());
				ibstrm.flush();		
				
				if(i==0)
				{
					for(l = 0; l<face_array_cnt[i]; l++)
					{
						f_order_total[l] = f_order_IFS_mapping[l];
					}
				}
				else
				{
					for(l = face_array_accmul_cnt[i-1]; l<face_array_accmul_cnt[i];l++)
					{
						f_order_total[l] = f_order_IFS_mapping[l-face_array_accmul_cnt[i-1]];
					}
				}	
				delete offset;
				delete firstVID;
				delete f_order_IFS_mapping;

			} // end for
			
			for(int l=0;l<face_array_accmul_cnt[nnCC-1];l++)
			{
				qFaceIndex.push_back(f_order_total[l]);
			}
			delete f_order_total;
			//~DMLAB ory
		}
		else
		{
			//dmlab : new vo fo algorithm  _ lsw 060703
			int TotalFaces=nF_total;
			//DMLAB ory  bitsToRepresent 2006 05
			int fo_id, DecordingFaces, init_bpfi = bitsToRepresent(TotalFaces),bpfi;
			//~DMLAB ory
			vector <int> faceVector;
			
			for(bpfi=0; bpfi<TotalFaces; bpfi++)
			{
				faceVector.push_back(bpfi);
			}
			
			int orgTotalFc=TotalFaces;
	
			for(bpfi=init_bpfi; bpfi>0; bpfi--)	// ݺ Ƚ Face ŭ
			{
				DecordingFaces = TotalFaces - (1<<(bpfi-1));
				TotalFaces = TotalFaces - DecordingFaces;
				
				int *delList = new int[DecordingFaces];
				int delCnt = 0;
				
				for(int itr=DecordingFaces; itr>0; itr--)
				{
					//fo_decode(fo_id, bpfi)
					ibstrm.getMOBitInt(bpfi,fo_id);
					if (bpfi == init_bpfi)
					{
						qFaceIndex.push_back(fo_id);
						delList[delCnt++] = fo_id;
					}
					else
					{
						qFaceIndex.push_back(faceVector[fo_id]);
						delList[delCnt++] = fo_id;
					}
				}
				//	printf("%d \n\n", delCnt);
				qsort(delList, delCnt, sizeof(int), compare);
							
				for(itr = 0; itr < delCnt; itr++)
				{
					faceVector.erase(faceVector.begin() + delList[itr] - itr);
				}
				delete delList;
			}
			qFaceIndex.push_back(faceVector[0]);
		}
		ibstrm.flush();     
	}
	num_CC_offset_all.erase(num_CC_offset_all.begin(),num_CC_offset_all.end());
	offset_all.erase(offset_all.begin(),offset_all.end());
	first_ID_all.erase(first_ID_all.begin(),first_ID_all.end());
	//~DK
	}// else if(mol_id!=0)
	
	// advance to next byte boundary and read ahead to determine
	// if anoter 3D_Mesh_Object_Layer() folllows
	
#ifndef MPGELIB
	if(saveWrlLevels==true)
	{
        int n = strlen(outDir)+strlen(baseFname)+16;
        outFname = new char[n];
        if(!strcmp(outDir,"."))
			sprintf(outFname,"%s-E%02d.wrl",baseFname,mol_id);
        else
			sprintf(outFname,"%s/%s-E%02d.wrl",outDir,baseFname,mol_id);
        bool success = write(outFname);
        delete [] outFname;
        outFname = NULL;
	}
#endif // MPGELIB   
  //ibstrm.flush();   
  } while(istrm.lookingAtStartCode(MPG_Global_EXT::MOL_start_code)==true);

#ifdef ADD_VISUAL_HEADER
  if(istrm.getVisualStartCode(visual_object_sequence_end_code)==false)
	  return false;
#endif //ADD_VISUAL_HEADER
  

		//----------------------------------------------------------------------//
  qCoord.clear();
  qNormal.clear();
  qColor.clear();
  qTexCoord.clear();
  
  success = true;
  return success;
}

#ifdef _SINGLE_RESOLUTION_PROGRESSIVE
// these are methods to be used for progressive decoding of a 
// single resolution .m3d stream
// the decoder resolution can be set to a number n of triangles at a time,
// one connected component at a time, or a two-step decoding process (header + rest)

//------------------------------------------------------
// expandHeader
//------------------------------------------------------
bool MPG_IfsQ_E_EXT::expandHeader(MPG_InputStream_EXT& istrm, MPG_State_EXT& state, 
                              MPG_Property_EXT& pCoord, MPG_Property_EXT& pNormal,
                              MPG_Property_EXT& pColor, MPG_Property_EXT& pTexCoord,
                              vector<int>& qCoord, vector<int>& qNormal,
                              vector<int>& qColor, vector<int>& qTexCoord,
                              vector<int>& v_visited, vector<int>& v_cluster,
                              MPG_EXT_Stitcher& stitcher, vector<int>& v_father,
                              MPG_EXT_Stitches& v_stitches, vector<int>& stk)
{
	MPG_QFpm_EXT::_shift = 16;
	
	bool success = false;
	
	clear();
	
	int nV0, nV1, nVcls0, nVcls1;
	state.GetNVertices(nV0, nV1, nVcls0, nVcls1);
	
	// arrays to store quantized property values
	
	// AG: in 3D_Mesh_Object_Base_Layer
	// these arrays are global for per vertex properties
	// (containing all the values decoded so far),
	// and local otherwise
	// (containing only values corresponding to the current component)
	
	// GT: in 3D_Mesh_Object_Forest_Split
	// all these arrays are global
	
	//vector <int> qCoord;
	//vector <int> qNormal;
	//vector <int> qColor;
	//vector <int> qTexCoord;
	
	// create proper headers and quantizers
	// if not yet created by calling method
	//MPG_Property_EXT pCoord(PT_COORD,PB_NONE);
	MPG_BBoxQ_EXT* coordQuantizer = getCoordQuantizer();
	if((MPG_BBoxQ_EXT*)0==coordQuantizer)
    {
		coordQuantizer    = new MPG_BBoxQ_EXT(3);
		setCoordQuantizer(coordQuantizer);
    }
	pCoord.setBBoxQ(coordQuantizer);
	
	//MPG_Property_EXT pNormal(PT_NORMAL,PB_NONE);
	MPG_NormalQ_EXT* normalQuantizer = getNormalQuantizer();
	if((MPG_NormalQ_EXT*)0==normalQuantizer)
    {
		normalQuantizer   = new MPG_NormalQ_EXT();
		setNormalQuantizer(normalQuantizer);
    }
	pNormal.setNormalQ(normalQuantizer);
	
	//MPG_Property_EXT pColor(PT_COLOR,PB_NONE);
	MPG_BBoxQ_EXT* colorQuantizer = getColorQuantizer();
	if((MPG_BBoxQ_EXT*)0==colorQuantizer)
    {
		colorQuantizer    = new MPG_BBoxQ_EXT(3);
		setColorQuantizer(colorQuantizer);
    }
	pColor.setBBoxQ(colorQuantizer);
	
	//MPG_Property_EXT pTexCoord(PT_TEXCOORD,PB_NONE);
	MPG_BBoxQ_EXT* texCoordQuantizer = getTexCoordQuantizer();
	if((MPG_BBoxQ_EXT*)0==texCoordQuantizer)
    {
		texCoordQuantizer = new MPG_BBoxQ_EXT(2);
		setTexCoordQuantizer(texCoordQuantizer);
    }
	pTexCoord.setBBoxQ(texCoordQuantizer);
	
	// quantization parameters
	int bitsPerCoord    = 0;
	int bitsPerNormal   = 0;
	int bitsPerColor    = 0;
	int bitsPerTexCoord = 0;
	
	// input bitstream parsing --------------------------------------------------
	
	//MPG_EXT_Stitcher stitcher;
	//vector<int>  v_father;
	//MPG_EXT_Stitches v_stitches;
	//vector<int>  stk;  // stitch encoding stack
	
	//vector<int>  v_visited;
	//vector<int>  v_cluster;
	//int          nV0    = 0; // number of vertices so far without current cc
	//int          nV1    = 0; // number of vertices so far with current cc
	//int          nVcls0 = 0; // AG: number of vertices after clustering
	//int          nVcls1 = 0; // AG: number of vertices after clustering
	
	// decode 3D_Mesh_Object() {
	//
	// 3D_Mesh_Object() {
	//   3D_MO_start_code
	//   3D_Mesh_Object_Header()
	//   do {
	//     3D_Mesh_Object_Layer()
	//   } while(nextbits_bytealigned()==3D_MOL_start_code)
	//
	
	// read 3D_MO_start_code
	if(istrm.getStartCode(MPG_Global_EXT::MO_start_code)==false)
		return false;
	
	// decode 3D_Mesh_Object_Header()
	{
		// open a bit stream
		MPG_InputBitStream_EXT ibstrm(istrm);
		
		// 3D_Mesh_Object_Header() {
		//   ccw
		//   convex
		//   solid
		//   creaseAngle
		//   coord_header()
		//   normal_header()
		//   color_header()
		//   texCoord_header()
		//   cgd_data
		//   if(cgd_data==1)
		//     cgd_header()
		// }
		
		if(ibstrm.getBit(_ccw               )==false) return false;
		if(ibstrm.getBit(_convex            )==false) return false;
		if(ibstrm.getBit(_solid             )==false) return false;
		
		int cAng = 0;
		if(ibstrm.getMBitInt(6,cAng)==false) return false;
		if(cAng<0 || cAng>=64) return false;
		_creaseAngle = M_PI*((float)cAng)/32;
		
		// decode coord_header()
		{
			// coord_header() {
			//   coord_binding       <--- must be == 1 (PER_VERTEX)
			
			int b = 0;
			if(ibstrm.getMBitInt(2,b)==false) return false;
			MPG_Binding binding =
				(b==0)?PB_NONE:(b==1)?PB_PER_VERTEX:(b==2)?PB_PER_FACE:PB_PER_CORNER;
			if(binding!=PB_NONE)
			{
				// coord can only be bound PER_VERTEX
				if(binding!=PB_PER_VERTEX) return false;
				pCoord.setBinding(binding);
				
				//   coord_bbox
				//   if(coord_bbox) {
				//     coord_xmin
				//     coord_ymin
				//     coord_zmin
				//     coord_size
				//   }
				//   coord_quant
				//   coord_pred_type
				//   if(coord_pred_type==2) {
				//     coord_nlambda
				//     for(i=1;i<coord_nlambda;i++)
				//       coord_lambda[i]
				//   }
				
				success = pCoord.decode(ibstrm);
				if(success==false) return false;
			}
			
			// }
		}
		
		// decode normal_header()
		{
			// normal_header() {
			//   normal_binding
			//   if(normal_binding!=0) {
			
			int b = 0;
			if(ibstrm.getMBitInt(2,b)==false) return false;
			MPG_Binding binding =
				(b==0)?PB_NONE:(b==1)?PB_PER_VERTEX:(b==2)?PB_PER_FACE:PB_PER_CORNER;
			if(binding!=PB_NONE)
			{
				pNormal.setBinding(binding);
				
				//     normal_bbox       <--- must be == 0
				//     normal_quant
				//     normal_pred_type
				//     if(normal_pred_type==2) {
				//       normal_nlambda
				//       for(i=1;i<normal_nlambda;i++)
				//         normal_lambda[i]
				//     }
				
				success = pNormal.decode(ibstrm);
				if(success==false) return false;
				
			}
			
			//   }
			// }
			
			_normalPerVertex = (binding==PB_PER_FACE)?false:true;
		}
		
		// decode color_header()
		{
			// color_header() {
			//   color_binding
			//   if(color_binding!=0) {
			
			int b = 0;
			if(ibstrm.getMBitInt(2,b)==false) return false;
			MPG_Binding binding =
				(b==0)?PB_NONE:(b==1)?PB_PER_VERTEX:(b==2)?PB_PER_FACE:PB_PER_CORNER;
			if(binding!=PB_NONE)
			{
				pColor.setBinding(binding);
				
				//     color_bbox
				//     if(color_bbox) {
				//       color_rmin
				//       color_gmin
				//       color_bmin
				//       color_size
				//     }
				//     color_quant
				//     color_pred_type
				//     if(color_pred_type==2) {
				//       color_nlambda
				//       for(i=1;i<color_nlambda;i++)
				//         color_lambda[i]
				//     }
				
				success = pColor.decode(ibstrm);
				if(success==false) return false;
			}
			
			//   }
			// }
			
			_colorPerVertex = (binding==PB_PER_FACE)?false:true;
		}
		
		// decode texture_header()
		{
			// texCoord_header() {
			//   texCoord_binding     <--- must be != 2 (PER_FACE)
			//   if(texCoord_binding!=0) {
			
			int b = 0;
			if(ibstrm.getMBitInt(2,b)==false) return false;
			MPG_Binding binding =
				(b==0)?PB_NONE:(b==1)?PB_PER_VERTEX:(b==2)?PB_PER_FACE:PB_PER_CORNER;
			if(binding!=PB_NONE)
			{
				// TexCoord cannot be bound per_face
				if(binding==PB_PER_FACE) return false;
				pTexCoord.setBinding(binding);
				
				//     texCoord_bbox
				//     if(texCoord_bbox) {
				//       texCoord_umin
				//       texCoord_vmin
				//       texCoord_size
				//     }
				//     texCoord_quant
				//     texCoord_pred_type
				//     if(texCoord_pred_type==2) {
				//       texCoord_nlambda
				//       for(i=1;i<texCoord_nlambda;i++)
				//         texCoord_lambda[i]
				//     }
				
				success = pTexCoord.decode(ibstrm);
				if(success==false) return false;
			}
			
			//   }
			// }
		}
		
		// decode cgd_data
		/*
		{
			_cgd_data = false;
			if(ibstrm.getBit(_cgd_data)==false) return false;
			
			if(_cgd_data==true)
			{
				// decode cgd_header()
				//
				// for now : just parse and ignore values
				// later   : create a new class to store values
				//
				int N_Proj_Surface_Spheres = 0;
				if(ibstrm.getMBitInt(4,N_Proj_Surface_Spheres)==false) return false;
				if(N_Proj_Surface_Spheres>0)
				{
					float x_coord_Center_Point = 0.0;
					if(ibstrm.get32BitFloat(x_coord_Center_Point)==false)
						return false;
					
					float y_coord_Center_Point = 0.0;
					if(ibstrm.get32BitFloat(y_coord_Center_Point)==false)
						return false;
					
					float z_coord_Center_Point = 0.0;
					if(ibstrm.get32BitFloat(z_coord_Center_Point)==false)
						return false;
					
					int Normalized_Screen_Distance_Factor = 0;
					if(ibstrm.getMBitInt(8,Normalized_Screen_Distance_Factor)==false)
						return false;
					
					int i;
					for(i=0;i<N_Proj_Surface_Spheres;i++) {
						
						float Radius = 0.0;
						if(ibstrm.get32BitFloat(Radius)==false)
							return false;
						
						float Min_Proj_Surface = 0.0;
						if(ibstrm.get32BitFloat(Min_Proj_Surface)==false)
							return false;
						
						int N_Proj_Points = 0;
						if(ibstrm.getMBitInt(8,N_Proj_Points)==false)
							return false;
						
						int j;
						for(j=0;j<N_Proj_Points;j++)
						{
							
							int Sphere_Point_Coord = 0;
							if(ibstrm.getMBitInt(11,Sphere_Point_Coord)==false)
								return false;
							
							float Proj_Surface = 0.0;
							if(ibstrm.get32BitFloat(Proj_Surface)==false)
								return false;
							
						}
					}
				}
			}
		}
		*/
		// flush input buffer
		ibstrm.flush();
		
  } // end decode 3D_Mesh_Object_Header()
  
  state.SetNVertices(nV0, nV1, nVcls0, nVcls1);
  success = true;
  return success;
}

//------------------------------------------------------
// expandRest
//------------------------------------------------------
bool MPG_IfsQ_E_EXT::expandRest(MPG_InputStream_EXT& istrm, 
                            MPG_Property_EXT& pCoord, MPG_Property_EXT& pNormal,
                            MPG_Property_EXT& pColor, MPG_Property_EXT& pTexCoord,
                            vector<int>& qCoord, vector<int>& qNormal,
                            vector<int>& qColor, vector<int>& qTexCoord,
                            vector<int>& v_visited, vector<int>& v_cluster, 
                            MPG_EXT_Stitcher& stitcher, vector<int>& v_father,
                            MPG_EXT_Stitches& v_stitches, vector<int>& stk,
                            int& nV0, int& nV1, int& nVcls0, int& nVcls1)
{
	int bitsPerCoord    =  pCoord.getBBoxQ()->getBitsPerCoordinate();
	int bitsPerNormal   =  pNormal.getNormalQ()->getBitsPerNormal();
	int bitsPerColor    =  pColor.getBBoxQ()->getBitsPerCoordinate();
	int bitsPerTexCoord =  pTexCoord.getBBoxQ()->getBitsPerCoordinate();
	
	bool success = false;
	bool has_stitches;
	// decode one or more 3D_Mesh_Object_Layer()
	int mol_id = 0;
	do {
		// decode 3D_Mesh_Object_Layer()
		
		// 3D_Mesh_Object_Layer() {
		//   3D_MOL_start_code
		//   mol_id
		//   if(cgd_data==1) {
		//     N_Vertices
		//     N_Triangles
		//     N_Edges
		//   }
		//   if(mol_id==0)
		//     3D_Mesh_Object_Base_Layer()
		//   else
		//     3D_Mesh_Object_Refinement_Layer()
		// }
		
		// read 3D_MOL_start_code
		if(istrm.getStartCode(MPG_Global_EXT::MOL_start_code)==false)
			return false;
		
		{
			// open a bit stream
			MPG_InputBitStream_EXT ibstrm(istrm);
			
			// read mol_id
			if(ibstrm.getMBitInt(8,mol_id)==false) return false;
			
			// read cgd data
			/*
			if(_cgd_data==true)
			{
				int N_Vertices = 0;
				if(ibstrm.getMBitInt(24,N_Vertices)==false) return false;
				
				int N_Triangles = 0;
				if(ibstrm.getMBitInt(24,N_Triangles)==false) return false;
				
				int N_Edges = 0;
				if(ibstrm.getMBitInt(24,N_Edges)==false) return false;
			}
			*/
			ibstrm.flush();
		}
		
		if(mol_id==0)
		{
			//
			// decode 3D_Mesh_Object_Base_Layer()
			//
			
			// GT Wed Jun 09 21:13:23 1999 TODO:
			// codap support ?
			
			// 3D_Mesh_Object_Base_Layer() {
			//   do {
			//     3D_MOBL_start_code
			//     mobl_id
			//     start_qcoder()
			//     do {
			//       connected_component()
			//       last_component
			//     } while(last_component==false)
			//     stop_qcoder()
			//   } while(nextbits_bytealigned()==3D_MOBL_start_code)
			// }
			
			do {
				
				// read 3D_MOBL_start_code
				if(istrm.getStartCode(MPG_Global_EXT::MOBL_start_code)==false)
					return false;
				
				{
					// open a bit stream
					MPG_InputBitStream_EXT ibstrm(istrm);     
					
					// read mobl_id
					int mobl_id = 0;
					if(ibstrm.getMBitInt(8,mobl_id)==false) return false;
					
					ibstrm.flush();
				}
				
				//  start the arithmetic codec
				MPG_QFac_EXT codec(istrm);
				codec.start_decode();
				MPG_Global_EXT::reset();
				
				do {
					// decode connected_component()
					
					// connected_component() {
					//   vertex_graph()
					//   has_stitches
					//   if(has_stitches==1)
					//     stitches()l
					//   triangle_tree()
					//   triangle_data()
					// }
					
					// decode vertex_graph() and construct vertex loop
					int nVcc = 0;
					vector<int> v_loop_cc;
					{
						// decode vertex_graph()
						MPG_VGraph_EXT vg;
						if(vg.decode(codec) == false) return false;
						
						nVcc = vg.getNumberOfVertices();
						nV1 = nV0 + nVcc;
						while(v_visited.size()<nV1)
							v_visited.push_back(false);
						
						// convert to vertex loop look-up table
						MPG_Loops_EXT loop;
						vg.toLoop(loop);
						loop.getLoop(0, v_loop_cc);
						
						if(vg.getFather(v_father)==false || v_father.size()!=nV1)
						{
							return false;
						}
					}
					
					//v_stitches.decode(nV0,nV1,stk,codec);
					has_stitches = (codec.decode(MPG_Global_EXT::stitch_has)==1)?true:false;
					{
						// use stitches nV0<=i<nV1 to partially recover v_cluster
						int nV1 = v_father.size(); 
						int nVcc = nV1-nV0;
						for (int i = nV0;i < nV1; i++) v_cluster.push_back(i);
						
						//if(stitcher.use(nV0,v_father,v_stitches,v_cluster)==false)
						//	return false;
					}
					
					// we should have here :
					// v_cluster.size()==nV1
					
					if(v_cluster.size()!=nV1)
						return false;
					
					// for i=0,..,nV1-1
					//   v_cluster[i]>=0 && v_cluster[i]<nV1
					// v_father.size()==nV1 
					// for i=0,..,nV1-1
					//   v_father[i]<=i
					// v_stitches.size()==nV1
					
					// decode triangle_tree()
					MPG_SPolygon_EXT sp;
					if(sp.decodeTriangleTree(codec) == false) return false;
					
					// decode triangle_data()
					vector<int>   coordIndex;
					if(sp.decodeTriangleData(codec,
						nVcc,
						nV0,
						v_loop_cc,
						v_visited,
						v_cluster,
						coordIndex,
						// property headers and quantizers
						pCoord,
						pNormal,
						pColor,
						pTexCoord,
						// quantized values
						qCoord,
						qNormal,
						qColor,
						qTexCoord)==false) return false;
					
					// save size of _coordIndex
					// used to update per corner properties
					int i0 = _coordIndex.size();
					
					// transfer coordIndex to Ifs
					{
						int j0,j1,j;
						for(j0=j1=0;j1<coordIndex.size();j1++)
							if(coordIndex[j1]<0)
							{
								int* f = &(coordIndex[j0]);
								int nf = j1-j0;
								for(j=0;j<nf;j++)
									_coordIndex.push_back(v_cluster[nV0+v_loop_cc[f[j]]]);
								_coordIndex.push_back(-1);
								j0 = j1+1;
							}
					}
					
					// 2/12/99 determine the new number of vertices after clustering
					{
						int i;
						for (i=nV0; i < nV1; i++)
							if (v_cluster[i] > nVcls1) nVcls1 = v_cluster[i];
					}
					int nVcls11 = nVcls1+1;
					
					// unquantize vertices between nVcls0 and nVcls1
					
					// coord properties
					vector<float> coord((size_t) (3*(nVcls11-nVcls0)),0.0);
					MPG_BBoxQ_EXT*   coordQuantizer    = pCoord.getBBoxQ();
					coordQuantizer->unQuantize(qCoord.begin()+3*nVcls0,
						coord.begin(),
						nVcls11-nVcls0);
					
					//  normal properties
					vector<float> normal;
					MPG_NormalQ_EXT* normalQuantizer   = pNormal.getNormalQ();
					if(pNormal.getBinding() != PB_NONE)
					{
						// special case for per-vertex binding,
						// the qNormal coordinates are global to allow reuse
						// of some vertices by components decoded afterwards
						if(pNormal.getBinding() == PB_PER_VERTEX)
						{
							// take only the trailing part of qNormal
							vector <int> qNormal1;
							int i;
							for (i=nVcls0;i<nVcls11;i++)
								qNormal1.push_back(qNormal[i]);
							
							normalQuantizer->unQuantize(qNormal1, normal);
						}
						else normalQuantizer->unQuantize(qNormal, normal);
					}
					//  color properties
					vector<float> color;
					MPG_BBoxQ_EXT*   colorQuantizer    = pColor.getBBoxQ();
					if(pColor.getBinding() != PB_NONE)
					{
						if(pColor.getBinding() == PB_PER_VERTEX)
						{
							color.erase(color.begin(),color.end());
							color.insert(color.end(),(size_t)(3*(nVcls11-nVcls0)),0.0);
							colorQuantizer->unQuantize(qColor.begin()+3*nVcls0,
								color.begin(),
								nVcls11-nVcls0);
						}
						else colorQuantizer->unQuantize(qColor, color);
					}
					//  texture properties
					vector<float> texCoord;
					MPG_BBoxQ_EXT*   texCoordQuantizer = pTexCoord.getBBoxQ();
					if(pTexCoord.getBinding() != PB_NONE)
					{
						if(pTexCoord.getBinding() == PB_PER_VERTEX)
						{
							texCoord.erase(texCoord.begin(),texCoord.end());
							texCoord.insert(texCoord.end(),
								(size_t)(2*(nVcls11-nVcls0)),0.0);
							texCoordQuantizer->unQuantize(qTexCoord.begin()+2*nVcls0,
								texCoord.begin(),
								nVcls11-nVcls0);
						}
						else
							texCoordQuantizer->unQuantize(qTexCoord, texCoord);
					}
					// transfer coord to Ifs
					{
						int i;
						for(i=0;i<coord.size();i++) _coord.push_back(coord[i]);
					}
					
					if(pNormal.getBinding()==PB_PER_CORNER)
					{
						// adjust _normalIndex
						// normal.size() should be equal to
						// the number of new corners
						int i = i0;
						int j = _normal.size()/3;
						for(;i<_coordIndex.size();i++)
							if(_coordIndex[i]>=0)
							{ _normalIndex.push_back(j); j++; }
							else
							{ _normalIndex.push_back(-1); }
					}
					
					// transfer normal to Ifs
					{
						int i;
						for(i=0;i<normal.size();i++)
							_normal.push_back(normal[i]);
					}
					
					if(pColor.getBinding()==PB_PER_CORNER)
					{
						// adjust _colorIndex
						// color.size() should be equal to
						// the number of new corners
						int i = i0;
						int j = _color.size()/3;
						for(;i<_coordIndex.size();i++)
							if(_coordIndex[i]>=0)
							{ _colorIndex.push_back(j); j++; }
							else
							{ _colorIndex.push_back(-1); }
					}
					
					// transfer color to Ifs
					{
						int i;
						for(i=0;i<color.size();i++)
							_color.push_back(color[i]);
					}
					
					if(pTexCoord.getBinding()==PB_PER_CORNER)
					{
						// adjust _texCoordIndex
						// texCoord.size() should be equal to
						// the number of new corners
						int i = i0;
						int j = _texCoord.size()/2;
						for(;i<_coordIndex.size();i++)
							if(_coordIndex[i]>=0)
							{ _texCoordIndex.push_back(j); j++; }
							else
							{ _texCoordIndex.push_back(-1); }
					}
					
					// transfer texCoord to Ifs
					{
						int i;
						for(i=0;i<texCoord.size();i++)
							_texCoord.push_back(texCoord[i]);
					}
					
					nVcls0 = nVcls11;
					nV0 = nV1;
					
          } while(codec.decode(MPG_Global_EXT::last_component) == false);
		  
          // stop_qcoder()
          codec.finish_decode();
          
          // advance to next byte boundary and read ahead to determine
          // if another 3D_Mesh_Object_Base_Layer() follows
		  
        } while(istrm.lookingAtStartCode(MPG_Global_EXT::MOBL_start_code)==true);
		
      }
	  
	  else // if(mol_id!=0)
      {
		  //
		  //
		  // progressive version not implemented
		  //
		  //
      } // else if(mol_id!=0)
	  
	  // advance to next byte boundary and read ahead to determine
	  // if anoter 3D_Mesh_Object_Layer() folllows
	  
	  
	  /*    if(saveWrlLevels==true)
	  {
	  int n = strlen(outDir)+strlen(baseFname)+16;
	  outFname = new char[n];
	  if(!strcmp(outDir,"."))
	  sprintf(outFname,"%s-E%02d.wrl",baseFname,mol_id);
	  else
	  sprintf(outFname,"%s/%s-E%02d.wrl",outDir,baseFname,mol_id);
	  bool success = write(outFname);
	  delete [] outFname;
	  outFname = NULL;
	  }
	  */    
  } while(istrm.lookingAtStartCode(MPG_Global_EXT::MOL_start_code)==true);
  
  success = true;
  return success;
}

//------------------------------------------------------
// expandCC
//------------------------------------------------------
bool MPG_IfsQ_E_EXT::expandCC(MPG_InputStream_EXT& istrm, 
                          MPG_Property_EXT& pCoord, MPG_Property_EXT& pNormal,
                          MPG_Property_EXT& pColor, MPG_Property_EXT& pTexCoord,
                          vector<int>& qCoord, vector<int>& qNormal,
                          vector<int>& qColor, vector<int>& qTexCoord,
                          vector<int>& v_visited, vector<int>& v_cluster,
                          MPG_EXT_Stitcher& stitcher, vector<int>& v_father,
                          MPG_EXT_Stitches& v_stitches, vector<int>& stk,
                          int& nV0, int& nV1, int& nVcls0, int& nVcls1,
                          bool& newMol, bool& newMobl, bool& newCC,
                          int& mol_id, MPG_QFac_EXT& codec, 
						  //vector<int>&   coordIndex,
						  //vector<float>& coord,
						  //vector<float>& normal,
						  //vector<float>& color,
						  //vector<float>& texCoord,
                          bool& expandDone)
{
	bool success = false;
	bool wasNewMobl = false;
	bool has_stitches = false;
	// decode one or more 3D_Mesh_Object_Layer()
	
	if(newMol)
    {
		// decode 3D_Mesh_Object_Layer()
		
		// 3D_Mesh_Object_Layer() {
		//   3D_MOL_start_code
		//   mol_id
		//   if(cgd_data==1) {
		//     N_Vertices
		//     N_Triangles
		//     N_Edges
		//   }
		//   if(mol_id==0)
		//     3D_Mesh_Object_Base_Layer()
		//   else
		//     3D_Mesh_Object_Refinement_Layer()
		// }
		
		// read 3D_MOL_start_code
		if(istrm.getStartCode(MPG_Global_EXT::MOL_start_code)==false)
			return false;
		
		mol_id = 0;
		{
			// open a bit stream
			MPG_InputBitStream_EXT ibstrm(istrm);
			
			// read mol_id
			if(ibstrm.getMBitInt(8,mol_id)==false) return false;
			
			// read cgd data
			/*
			if(_cgd_data==true)
			{
				int N_Vertices = 0;
				if(ibstrm.getMBitInt(24,N_Vertices)==false) return false;
				
				int N_Triangles = 0;
				if(ibstrm.getMBitInt(24,N_Triangles)==false) return false;
				
				int N_Edges = 0;
				if(ibstrm.getMBitInt(24,N_Edges)==false) return false;
			}
			*/
			ibstrm.flush();
		}
		
		newMol = false;
    }
	
	if(mol_id==0)
    {
		//
		// decode 3D_Mesh_Object_Base_Layer()
		//
		
		// GT Wed Jun 09 21:13:23 1999 TODO:
		// codap support ?
		
		// 3D_Mesh_Object_Base_Layer() {
		//   do {
		//     3D_MOBL_start_code
		//     mobl_id
		//     start_qcoder()
		//     do {
		//       connected_component()
		//       last_component
		//     } while(last_component==false)
		//     stop_qcoder()
		//   } while(nextbits_bytealigned()==3D_MOBL_start_code)
		// }
		
		if(newMobl) 
        {	
			// read 3D_MOBL_start_code
			if(istrm.getStartCode(MPG_Global_EXT::MOBL_start_code)==false)
				return false;
			
			{
				// open a bit stream
				MPG_InputBitStream_EXT ibstrm(istrm);     
				
				// read mobl_id
				int mobl_id = 0;
				if(ibstrm.getMBitInt(8,mobl_id)==false) return false;
				
				ibstrm.flush();
			}
			
			//  start the arithmetic codec
			// MPG_QFac_EXT codec(istrm);
			codec.start_decode();
			
			// reset probability model
			MPG_Global_EXT::reset();
			
			newMobl = false;
			wasNewMobl = true;
        }
		
		if(newCC)
        {
			// decode connected_component()
			
			// connected_component() {
			//   vertex_graph()
			//   has_stitches
			//   if(has_stitches==1)
			//     stitches()l
			//   triangle_tree()
			//   triangle_data()
			// }
			
			// decode vertex_graph() and construct vertex loop
			int nVcc = 0;
			vector<int> v_loop_cc;
			{
				// decode vertex_graph()
				MPG_VGraph_EXT vg;
				if(vg.decode(codec) == false) return false;
				
				nVcc = vg.getNumberOfVertices();
				nV1 = nV0 + nVcc;
				while(v_visited.size()<nV1)
					v_visited.push_back(false);
				
				// convert to vertex loop look-up table
				MPG_Loops_EXT loop;
				vg.toLoop(loop);
				loop.getLoop(0, v_loop_cc);
				
				if(vg.getFather(v_father)==false || v_father.size()!=nV1)
					return false;
			}
			
			//v_stitches.decode(nV0,nV1,stk,codec);
			has_stitches = (codec.decode(MPG_Global_EXT::stitch_has)==1)?true:false;
			
			{
				// use stitches nV0<=i<nV1 to partially recover v_cluster
				int nV1 = v_father.size(); 
				int nVcc = nV1-nV0;
				for (int i = nV0;i < nV1; i++) v_cluster.push_back(i);
				
				//if(stitcher.use(nV0,v_father,v_stitches,v_cluster)==false)
				//	return false;
			}		
			// we should have here :
			// v_cluster.size()==nV1
			
			if(v_cluster.size()!=nV1)
				return false;
			
			// for i=0,..,nV1-1
			//   v_cluster[i]>=0 && v_cluster[i]<nV1
			// v_father.size()==nV1 
			// for i=0,..,nV1-1
			//   v_father[i]<=i
			// v_stitches.size()==nV1
			
			// decode triangle_tree()
			MPG_SPolygon_EXT sp;
			if(sp.decodeTriangleTree(codec) == false) return false;
			
			// decode triangle_data()
			vector<int>   coordIndex;
			if(sp.decodeTriangleData(codec,
				nVcc,
				nV0,
				v_loop_cc,
				v_visited,
				v_cluster,
				coordIndex,
				// property headers and quantizers
				pCoord,
				pNormal,
				pColor,
				pTexCoord,
				// quantized values
				qCoord,
				qNormal,
				qColor,
				qTexCoord)==false) return false;
			
			// save size of _coordIndex
			// used to update per corner properties
			int i0 = _coordIndex.size();
			
			// transfer coordIndex to Ifs
			{
				int j0,j1,j;
				for(j0=j1=0;j1<coordIndex.size();j1++)
					if(coordIndex[j1]<0)
					{
						int* f = &(coordIndex[j0]);
						int nf = j1-j0;
						for(j=0;j<nf;j++)
							_coordIndex.push_back(v_cluster[nV0+v_loop_cc[f[j]]]);
						_coordIndex.push_back(-1);
						j0 = j1+1;
					}
			}
			
			// 2/12/99 determine the new number of vertices after clustering
			{
				int i;
				for (i=nV0; i < nV1; i++)
					if (v_cluster[i] > nVcls1) nVcls1 = v_cluster[i];
			}
			int nVcls11 = nVcls1+1;
			// unquantize vertices between nVcls0 and nVcls1
			
			// coord properties
			vector<float> coord((size_t) (3*(nVcls11-nVcls0)),0.0);
			MPG_BBoxQ_EXT*   coordQuantizer    = pCoord.getBBoxQ();
			coordQuantizer->unQuantize(qCoord.begin()+3*nVcls0,
				coord.begin(),
				nVcls11-nVcls0);
			
			//  normal properties
			vector<float> normal;
			MPG_NormalQ_EXT* normalQuantizer   = pNormal.getNormalQ();
			if(pNormal.getBinding() != PB_NONE)
            {
				// special case for per-vertex binding,
				// the qNormal coordinates are global to allow reuse
				// of some vertices by components decoded afterwards
				if(pNormal.getBinding() == PB_PER_VERTEX)
                {
					// take only the trailing part of qNormal
					vector <int> qNormal1;
					int i;
					for (i=nVcls0;i<nVcls11;i++)
						qNormal1.push_back(qNormal[i]);
					
					normalQuantizer->unQuantize(qNormal1, normal);
                }
				else normalQuantizer->unQuantize(qNormal, normal);
            }
			//  color properties
			vector<float> color;
			MPG_BBoxQ_EXT*   colorQuantizer    = pColor.getBBoxQ();
			if(pColor.getBinding() != PB_NONE)
            {
				if(pColor.getBinding() == PB_PER_VERTEX)
                {
					color.erase(color.begin(),color.end());
					color.insert(color.end(),(size_t)(3*(nVcls11-nVcls0)),0.0);
					colorQuantizer->unQuantize(qColor.begin()+3*nVcls0,
						color.begin(),
						nVcls11-nVcls0);
                }
				else colorQuantizer->unQuantize(qColor, color);
            }
			//  texture properties
			vector<float> texCoord;
			MPG_BBoxQ_EXT*   texCoordQuantizer = pTexCoord.getBBoxQ();
			if(pTexCoord.getBinding() != PB_NONE)
            {
				if(pTexCoord.getBinding() == PB_PER_VERTEX)
                {
					texCoord.erase(texCoord.begin(),texCoord.end());
					texCoord.insert(texCoord.end(),
						(size_t)(2*(nVcls11-nVcls0)),0.0);
					texCoordQuantizer->unQuantize(qTexCoord.begin()+2*nVcls0,
						texCoord.begin(),
						nVcls11-nVcls0);
                }
				else
					texCoordQuantizer->unQuantize(qTexCoord, texCoord);
            }
			// transfer coord to Ifs
			{
				int i;
				for(i=0;i<coord.size();i++) _coord.push_back(coord[i]);
			}
			
			if(pNormal.getBinding()==PB_PER_CORNER)
            {
				// adjust _normalIndex
				// normal.size() should be equal to
				// the number of new corners
				int i = i0;
				int j = _normal.size()/3;
				for(;i<_coordIndex.size();i++)
					if(_coordIndex[i]>=0)
					{ _normalIndex.push_back(j); j++; }
					else
					{ _normalIndex.push_back(-1); }
            }
			
			// transfer normal to Ifs
			{
				int i;
				for(i=0;i<normal.size();i++)
					_normal.push_back(normal[i]);
			}
			
			if(pColor.getBinding()==PB_PER_CORNER)
            {
				// adjust _colorIndex
				// color.size() should be equal to
				// the number of new corners
				int i = i0;
				int j = _color.size()/3;
				for(;i<_coordIndex.size();i++)
					if(_coordIndex[i]>=0)
					{ _colorIndex.push_back(j); j++; }
					else
					{ _colorIndex.push_back(-1); }
            }
			
			// transfer color to Ifs
			{
				int i;
				for(i=0;i<color.size();i++)
					_color.push_back(color[i]);
			}
			
			if(pTexCoord.getBinding()==PB_PER_CORNER)
            {
				// adjust _texCoordIndex
				// texCoord.size() should be equal to
				// the number of new corners
				int i = i0;
				int j = _texCoord.size()/2;
				for(;i<_coordIndex.size();i++)
					if(_coordIndex[i]>=0)
					{ _texCoordIndex.push_back(j); j++; }
					else
					{ _texCoordIndex.push_back(-1); }
            }
			
			// transfer texCoord to Ifs
			{
				int i;
				for(i=0;i<texCoord.size();i++)
					_texCoord.push_back(texCoord[i]);
			}
			
			nVcls0 = nVcls11;
			nV0 = nV1;
			
			newCC = false;
			
        }
		//} while(codec.decode(MPG_Global_EXT::last_component) == false);
		if(codec.decode(MPG_Global_EXT::last_component) == false)
			newCC = true;
		if(!newCC && wasNewMobl)
        {
			// stop_qcoder()
			codec.finish_decode();
        }
		
		// advance to next byte boundary and read ahead to determine
		// if another 3D_Mesh_Object_Base_Layer() follows
		//} while(istrm.lookingAtStartCode(MPG_Global_EXT::MOBL_start_code)==true);
		if(!newCC && (istrm.lookingAtStartCode(MPG_Global_EXT::MOBL_start_code)==true))
			newMobl = true;
		
    }
	else
    {
		// decode 3D_Mesh_Object_Forest_Split()
    }
	
	// advance to next byte boundary and read ahead to determine
	// if anoter 3D_Mesh_Object_Layer() folllows
	//} while(istrm.lookingAtStartCode(MPG_Global_EXT::MOL_start_code)==true);
	if(!newCC && !newMobl && (istrm.lookingAtStartCode(MPG_Global_EXT::MOL_start_code)==true))
		newMol = true;
	
	if(!newCC && !newMobl && !newMol)
		expandDone = true;
	
	success = true;
	return success;
}

//------------------------------------------------------
// expandTriangles
//------------------------------------------------------
bool MPG_IfsQ_E_EXT::expandTriangles(MPG_InputStream_EXT& istrm,
                                 MPG_State_EXT& state, 
                                 MPG_Property_EXT& pCoord, MPG_Property_EXT& pNormal,
                                 MPG_Property_EXT& pColor, MPG_Property_EXT& pTexCoord,
                                 vector<int>& qCoord, vector<int>& qNormal,
                                 vector<int>& qColor, vector<int>& qTexCoord,
                                 vector<int>& v_visited, vector<int>& v_cluster,
                                 MPG_EXT_Stitcher& stitcher, vector<int>& v_father,
                                 MPG_EXT_Stitches& v_stitches, 
                                 MPG_QFac_EXT& codec, 
                                 vector<int>&   coordIndex,
                                 vector<float>& coord,
                                 vector<float>& normal,
                                 vector<float>& color,
                                 vector<float>& texCoord,
                                 vector<int>& f_father,
                                 vector<int>& c_traversal,
                                 vector<int>& c_father,
                                 vector<int>& run_father,
                                 vector<int>& run_length,
                                 vector<bool>& run_ends_in_leaf,
                                 vector<int>& stk, 
                                 vector<int>& faceStack, 
                                 MPG_SPolygon_EXT& sp, MPG_VGraph_EXT& vg,
                                 vector<int>& v_loop_cc, 
                                 MPG_Loops_EXT& loop, MPG_Loops_EXT& c_loop,
                                 bool& expandDone)
{
	bool success = false;
	bool wasNewMobl = false;
	bool has_stitches = false;
	int nTriangles = 0;
	int nV0 = 0, nV1 = 0, nVcls0 = 0, nVcls1 = 0;
	bool newMol, newMobl, newCC, newTri;
	int mol_id, lastTriangle;
	int prev_marching_pattern, prev_polygon_edge, triangulated;
	int nPE, nF, nC, nCoord, nNormal, nColor, nTexCoord;
	int iP, iR, iL, iO, iF, iFp, iC;
	int cRp, cR, cLp, cL, cO, nVcc, iRun;
	
	state.GetState(nTriangles, nV0, nV1, nVcls0, nVcls1,
		newMol, newMobl, newCC, newTri, mol_id, lastTriangle, prev_marching_pattern, 
		prev_polygon_edge, triangulated, nPE, nF, nC, nCoord, nNormal, nColor, 
		nTexCoord, iP, iR, iL, iO, iF, iFp, iC, cRp, cR, cLp, cL, cO, nVcc, iRun);
	
	// decode one or more 3D_Mesh_Object_Layer()
	
	if(newMol)
    {
		// decode 3D_Mesh_Object_Layer()
		
		// 3D_Mesh_Object_Layer() {
		//   3D_MOL_start_code
		//   mol_id
		//   if(cgd_data==1) {
		//     N_Vertices
		//     N_Triangles
		//     N_Edges
		//   }
		//   if(mol_id==0)
		//     3D_Mesh_Object_Base_Layer()
		//   else
		//     3D_Mesh_Object_Refinement_Layer()
		// }
		
		// read 3D_MOL_start_code
		if(istrm.getStartCode(MPG_Global_EXT::MOL_start_code)==false)
			return false;
		
		mol_id = 0;
		{
			// open a bit stream
			MPG_InputBitStream_EXT ibstrm(istrm);
			
			// read mol_id
			if(ibstrm.getMBitInt(8,mol_id)==false) return false;
			
			// read cgd data
			/*
			if(_cgd_data==true)
			{
				int N_Vertices = 0;
				if(ibstrm.getMBitInt(24,N_Vertices)==false) return false;
				
				int N_Triangles = 0;
				if(ibstrm.getMBitInt(24,N_Triangles)==false) return false;
				
				int N_Edges = 0;
				if(ibstrm.getMBitInt(24,N_Edges)==false) return false;
			}
			*/
			ibstrm.flush();
		}
		
		newMol = false;
    }
	
	if(mol_id==0)
    {
		//
		// decode 3D_Mesh_Object_Base_Layer()
		//
		
		// GT Wed Jun 09 21:13:23 1999 TODO:
		// codap support ?
		
		// 3D_Mesh_Object_Base_Layer() {
		//   do {
		//     3D_MOBL_start_code
		//     mobl_id
		//     start_qcoder()
		//     do {
		//       connected_component()
		//       last_component
		//     } while(last_component==false)
		//     stop_qcoder()
		//   } while(nextbits_bytealigned()==3D_MOBL_start_code)
		// }
		
		if(newMobl) 
        {	
			// read 3D_MOBL_start_code
			if(istrm.getStartCode(MPG_Global_EXT::MOBL_start_code)==false)
				return false;
			
			{
				// open a bit stream
				MPG_InputBitStream_EXT ibstrm(istrm);     
				
				// read mobl_id
				int mobl_id = 0;
				if(ibstrm.getMBitInt(8,mobl_id)==false) return false;
				
				ibstrm.flush();
			}
			
			//  start the arithmetic codec
			// MPG_QFac_EXT codec(istrm);
			codec.start_decode();
			
			// reset probability model
			MPG_Global_EXT::reset();
			
			newMobl = false;
			wasNewMobl = true;
        }
		
		if(newCC)
        {
			// decode connected_component()
			
			// connected_component() {
			//   vertex_graph()
			//   has_stitches
			//   if(has_stitches==1)
			//     stitches()l
			//   triangle_tree()
			//   triangle_data()
			// }
			
			// decode vertex_graph() and construct vertex loop
			int nVcc = 0;
			//vector<int> v_loop_cc;
			v_loop_cc.erase(v_loop_cc.begin(), v_loop_cc.end());
			{
				// decode vertex_graph()
				//MPG_VGraph_EXT vg;
				vg.clear();
				if(vg.decode(codec) == false) return false;
				
				nVcc = vg.getNumberOfVertices();
				nV1 = nV0 + nVcc;
				while(v_visited.size()<nV1)
					v_visited.push_back(false);
				
				// convert to vertex loop look-up table
				MPG_Loops_EXT loop;
				vg.toLoop(loop);
				loop.getLoop(0, v_loop_cc);
				
				if(vg.getFather(v_father)==false || v_father.size()!=nV1)
					return false;
			}
			
			//v_stitches.decode(nV0,nV1,stk,codec);
			has_stitches = (codec.decode(MPG_Global_EXT::stitch_has)==1)?true:false;
			{
				// use stitches nV0<=i<nV1 to partially recover v_cluster
				int nV1 = v_father.size(); 
				int nVcc = nV1-nV0;
				for (int i = nV0;i < nV1; i++) v_cluster.push_back(i);
				
				//if(stitcher.use(nV0,v_father,v_stitches,v_cluster)==false)
				//	return false;
			}		
			// we should have here :
			// v_cluster.size()==nV1
			
			if(v_cluster.size()!=nV1)
            {
				return false;
            }
			
			// for i=0,..,nV1-1
			//   v_cluster[i]>=0 && v_cluster[i]<nV1
			// v_father.size()==nV1 
			// for i=0,..,nV1-1
			//   v_father[i]<=i
			// v_stitches.size()==nV1
			
			// decode triangle_tree()
			//MPG_SPolygon_EXT sp;
			sp.clear();
			if(sp.decodeTriangleTree(codec) == false) return false;
			
			// decode triangle_data()
			//vector<int>   coordIndex;
			coordIndex.erase(coordIndex.begin(), coordIndex.end());
			coord.erase(coord.begin(), coord.end());
			normal.erase(normal.begin(), normal.end());
			texCoord.erase(texCoord.begin(), texCoord.end());
			
			prev_marching_pattern = 0; 
			prev_polygon_edge = 1;
			triangulated = 0;
			
			// face tree array
			f_father.erase(f_father.begin(), f_father.end());
			
			// corner tree arrays
			c_traversal.erase(c_traversal.begin(), c_traversal.end());
			c_father.erase(c_father.begin(), c_father.end());
			
			run_father.erase(run_father.begin(), run_father.end());
			run_length.erase(run_length.begin(), run_length.end());
			run_ends_in_leaf.erase(run_ends_in_leaf.begin(), run_ends_in_leaf.end());
			
			// traversal stack
			stk.erase(stk.begin(), stk.end());
			stk.insert(stk.begin(), (size_t)1, -1);
			
			nPE    = 0;    //      number of {non_root polygon_edge==0}
			nF     = 0;   // - nPE
			nC     = 0; // - 2*nPE
			
			nCoord = 0;
			nNormal = 0;
			nColor = 0;
			nTexCoord = 0;
			
			iP = -1;
			iR =  0;
			iL =  1;
			iO = -1;
			
			faceStack.erase(faceStack.begin(), faceStack.end());
			
			iF  = f_father.size();	// == 0
			iFp = iF;
			
			iC   = 0;
			cRp  = 0;
			cR   = 0;
			cLp  = 1;
			cL   = 1;
			cO   = -1;
			
			loop.clear();
			c_loop.clear();
			
			sp.preprocessTriangleData(codec, nVcc,
				nV0, 
				v_loop_cc,
				v_visited,
				v_cluster,
				coordIndex,
				pCoord, // property headers and quantizers
				pNormal,
				pColor,
				pTexCoord,
				qCoord, // quantized values
				qNormal,
				qColor,
				qTexCoord,
				prev_marching_pattern, 
				prev_polygon_edge, 
				triangulated,
				f_father,
				c_traversal,
				c_father,
				run_father,
				run_length,
				run_ends_in_leaf,
				stk, nPE, nF, nC,
				nCoord, nNormal, nColor, nTexCoord,
				iP, iR, iL, iO,
				faceStack, iF, iFp, iC,
				cRp, cR, cLp, cL, cO);
			
			newCC = false;
			iRun = 0;
			lastTriangle = 0;
			//	nVcls0 = nV0; 
			nVcls1 = 0;
        }
		
		if(newTri)
        {
			int nT = sp.getNumberOfTriangles();
			int nVloop = v_loop_cc.size();
			for(int iT=lastTriangle;(iT<lastTriangle+nTriangles) && (iT<nT);iT++)
				//for(int iT=0;iT<nT;iT++)
            {
				int iR1 = 0, iL1 = 0, iO1 = 0;
				sp.decodeSingleTriangle(iT, iRun, codec, nVcc,
					nV0, nVloop,
					loop, c_loop,
					v_loop_cc,
					v_visited,
					v_cluster,
					coordIndex,
					pCoord, // property headers and quantizers
					pNormal,
					pColor,
					pTexCoord,
					qCoord, // quantized values
					qNormal,
					qColor,
					qTexCoord,
					prev_marching_pattern, 
					prev_polygon_edge, 
					triangulated,
					f_father,
					c_traversal,
					c_father,
					run_father,
					run_length,
					run_ends_in_leaf,
					stk, nPE, nF, nC,
					nCoord, nNormal, nColor, nTexCoord,
					iP, iR, iL, iO, iR1, iL1, iO1,
					faceStack, iF, iFp, iC,
					cRp, cR, cLp, cL, cO);
				
				_coordIndex.push_back(v_cluster[nV0+v_loop_cc[iR1]]);
				//if(v_cluster[nV0+v_loop_cc[iR1]] > nVcls1) 
				//nVcls1 = v_cluster[nV0+v_loop_cc[iR1]];
				_coordIndex.push_back(v_cluster[nV0+v_loop_cc[iL1]]);
				//if(v_cluster[nV0+v_loop_cc[iL1]] > nVcls1) 
				//nVcls1 = v_cluster[nV0+v_loop_cc[iL1]];
				_coordIndex.push_back(v_cluster[nV0+v_loop_cc[iO1]]);
				//if(v_cluster[nV0+v_loop_cc[iO1]] > nVcls1) 
				//nVcls1 = v_cluster[nV0+v_loop_cc[iO1]];
				_coordIndex.push_back(-1);
				
				//TRACE("\n\t triangle (%d, %d, %d)",
				//	v_cluster[nV0+v_loop_cc[iR1]], v_cluster[nV0+v_loop_cc[iL1]], v_cluster[nV0+v_loop_cc[iO1]]);
				//TRACE(" or \t triangle (%d, %d, %d)",
				//	nV0+v_loop_cc[iR1], nV0+v_loop_cc[iL1], nV0+v_loop_cc[iO1]);
				
				// write out statistics
				/*{
				int v[3];
				v[0] = nV0+v_loop_cc[iR1];
				v[1] = nV0+v_loop_cc[iL1];
				v[2] = nV0+v_loop_cc[iO1];
				
				  int k = 1;
				  while(k)
				  {
				  k = 0;
				  if(v[0] > v[1])
				  {
				  int a = v[0]; v[0] = v[1];
				  v[1] = a; k = 1;
				  }
				  if(v[1] > v[2])
				  {
				  int a = v[1]; v[1] = v[2];
				  v[2] = a; k = 1;
				  }
				  }
				  
					freq[v[1] - v[0]] ++;
					freq[v[2] - v[1]] ++;
			}*/
            }
			
			lastTriangle += nTriangles;
			
			// 2/12/99 determine the new number of vertices after clustering
			/*		{
			int i;
			for (i=nV0+lastTriangle*3; (i < nV0+(lastTriangle+nTriangles)*3) && (i<nV1); i++)
			if (v_cluster[i] > nVcls1) 
			nVcls1 = v_cluster[i];
		}*/
			int nVcls11 = nVcls1+1;
			
			// unquantize vertices between nVcls0 and nVcls1
			
			// coord properties
			MPG_BBoxQ_EXT*   coordQuantizer    = pCoord.getBBoxQ();
			coord.erase(coord.begin(), coord.end());
			/*if(nVcls11-nVcls0)
            {
            coord.insert(coord.end(),(size_t) (3 * (nVcls11-nVcls0)),(float)0.0);
            coordQuantizer->unQuantize(qCoord.begin()+3*nVcls0, coord.begin(), nVcls11-nVcls0);
            }*/
			// this could be optimized, as it is it unquantizes all the coordinates
			// decoded so far every time
			coordQuantizer->unQuantize(qCoord, coord);
			
			/*		
			// progresive decoding of properties, not yet implemented
			
			  //  normal properties
			  MPG_NormalQ_EXT* normalQuantizer   = pNormal.getNormalQ();
			  if(pNormal.getBinding() != PB_NONE)
			  {
			  // special case for per-vertex binding,
			  // the qNormal coordinates are global to allow reuse
			  // of some vertices by components decoded afterwards
			  if(pNormal.getBinding() == PB_PER_VERTEX)
			  {
			  // take only the trailing part of qNormal
			  vector <int> qNormal1;
			  int i;
			  for (i=nVcls0;i<nVcls11;i++)
			  qNormal1.push_back(qNormal[i]);
			  
				normalQuantizer->unQuantize(qNormal1, normal);
				}
				else normalQuantizer->unQuantize(qNormal, normal);
				}
				
				  //  color properties
				  MPG_BBoxQ_EXT*   colorQuantizer    = pColor.getBBoxQ();
				  if(pColor.getBinding() != PB_NONE)
				  {
				  if(pColor.getBinding() == PB_PER_VERTEX)
				  {
				  color.erase(color.begin(),color.end());
				  color.insert(color.end(),(size_t) (3 * (nVcls11-nVcls0)),(float)0.0);
				  colorQuantizer->unQuantize(qColor.begin()+3*nVcls0, color.begin(), nVcls11-nVcls0);
				  }
				  else colorQuantizer->unQuantize(qColor, color);
				  }
				  
					//  texture properties
					MPG_BBoxQ_EXT*   texCoordQuantizer = pTexCoord.getBBoxQ();
					if(pTexCoord.getBinding() != PB_NONE)
					{
					if(pTexCoord.getBinding() != PB_PER_VERTEX)
					{
					texCoord.erase(texCoord.begin(),texCoord.end());
					texCoord.insert(texCoord.end(),(size_t) (2 * (nVcls11-nVcls0)),(float)0.0);
					texCoordQuantizer->unQuantize(qTexCoord.begin()+2*nVcls0, texCoord.begin(), nVcls11-nVcls0);
					}
					else
					texCoordQuantizer->unQuantize(qTexCoord, texCoord);
					}
			*/
			//if(lastTriangle - nTriangles)
			//{
			//	while(_coord.size() > nV0*3)
			//		_coord.pop_back();
			//}
			
			// move decoded coordinates into _coord
			_coord.erase(_coord.begin(), _coord.end());
			for(int i=0;i<coord.size();i++)
				_coord.push_back(coord[i]);
			
				/*
				// transfer coord to Ifs
				{
				int i;
				for(i=0;i<coord.size();i++) _coord.push_back(coord[i]);
				}
				
				  if(pNormal.getBinding()==PB_PER_CORNER)
				  {
				  // adjust _normalIndex
				  // normal.size() should be equal to
				  // the number of new corners
				  int i = i0;
				  int j = _normal.size();
				  for(;i<_coordIndex.size();i++)
				  if(_coordIndex[i]>=0)
				  { _normalIndex.push_back(j); j++; }
				  else
				  { _normalIndex.push_back(-1); }
				  }
				  
					// transfer normal to Ifs
					{
					int i;
					for(i=0;i<normal.size();i++)
					_normal.push_back(normal[i]);
					}
					
					  if(pColor.getBinding()==PB_PER_CORNER)
					  {
					  // adjust _colorIndex
					  // color.size() should be equal to
					  // the number of new corners
					  int i = i0;
					  int j = _color.size();
					  for(;i<_coordIndex.size();i++)
					  if(_coordIndex[i]>=0)
					  { _colorIndex.push_back(j); j++; }
					  else
					  { _colorIndex.push_back(-1); }
					  }
					  
						// transfer color to Ifs
						{
						int i;
						for(i=0;i<color.size();i++)
						_color.push_back(color[i]);
						}
						
						  if(pTexCoord.getBinding()==PB_PER_CORNER)
						  {
						  // adjust _texCoordIndex
						  // texCoord.size() should be equal to
						  // the number of new corners
						  int i = i0;
						  int j = _texCoord.size();
						  for(;i<_coordIndex.size();i++)
						  if(_coordIndex[i]>=0)
						  { _texCoordIndex.push_back(j); j++; }
						  else
						  { _texCoordIndex.push_back(-1); }
						  }
						  
							// transfer texCoord to Ifs
							{
							int i;
							for(i=0;i<texCoord.size();i++)
							_texCoord.push_back(texCoord[i]);
							}
			*/
			//nVcls0 = nVcls11;
			
			// AG 1/27/99 bug: nV0 should be increased:
			//nV0 = nV1;
			
			if(lastTriangle < nT)
				newTri = true;
			else
				newTri = false;
        }
		//} while(codec.decode(MPG_Global_EXT::last_component) == false);
		if(!newTri)
			if(codec.decode(MPG_Global_EXT::last_component) == false)
			{
				newCC = newTri = true;
				nV0 = nV1;
			}
			
			if(!newTri && !newCC && wasNewMobl)
			{
				// stop_qcoder()
				codec.finish_decode();
			}
			
			// advance to next byte boundary and read ahead to determine
			// if another 3D_Mesh_Object_Base_Layer() follows
			//} while(istrm.lookingAtStartCode(MPG_Global_EXT::MOBL_start_code)==true);
			if(!newTri && !newCC)
				if((istrm.lookingAtStartCode(MPG_Global_EXT::MOBL_start_code)==true))
					newMobl = true;
				
    }
	else
    {
		// decode 3D_Mesh_Object_Forest_Split()
    }
	
	// advance to next byte boundary and read ahead to determine
	// if anoter 3D_Mesh_Object_Layer() folllows
	//} while(istrm.lookingAtStartCode(MPG_Global_EXT::MOL_start_code)==true);
	if(!newTri && !newCC && !newMobl)
		if((istrm.lookingAtStartCode(MPG_Global_EXT::MOL_start_code)==true))
			newMol = true;
		
		if(!newTri && !newCC && !newMobl && !newMol)
			expandDone = true;
		
		state.SetState(nTriangles, nV0, nV1, nVcls0, nVcls1,
			newMol, newMobl, newCC, newTri, mol_id, lastTriangle, prev_marching_pattern, 
			prev_polygon_edge, triangulated, nPE, nF, nC, nCoord, nNormal, nColor, 
			nTexCoord, iP, iR, iL, iO, iF, iFp, iC, cRp, cR, cLp, cL, cO, nVcc, iRun);
		
		success = true;
		return success;
}

#endif // _SINGLE_RESOLUTION_PROGRESSIVE
