//-------------------------------------------------------------------------
// 
// Copyright (C) IBM Corporation & Samsung Electronics Co. LTD., 1996, 1997, 1998, 1999
// Licensed Materials, Program Property of IBM & Samsung.
// 
// This software module is an implementation of a part of one or more
// MPEG-4 tools as specified by the MPEG-4 standard (ISO/IEC 14496).
// ISO/IEC gives users of the MPEG-4 standard (ISO/IEC 14496) free
// license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4
// standard (ISO/IEC 14496).  Those intending to use this software module
// in hardware or software products are advised that its use may infringe
// existing patents.  The original developer of this software module and
// his/her company, the subsequent editors and their companies, and
// ISO/IEC have no liability for use of this software module or
// modifcations thereof in an implementation.  Copyright is not released
// for non MPEG-4 standard (ISO/IEC 14496) conforming products.  IBM
// Corp. & Samsung Electronics Co. LTD retains full right to use the code 
// for his/her own purpose, assign or donate the code to a third party and
// to inhibit third parties from using the code for non MPEG-4 standard 
// (ISO/IEC 14496) conforming products. This copyright notice must be included
// in all copies or derivative works.
// 
//  AUTHORS:
//    Gabriel Taubin
//    Andre Gueziec
//    Claudio Silva
//    Ioana Martin
//  
//  IBM T. J. Watson Research Center
//  P.O.Box 704
//  Yorktown Heights, New York 10598
//  
//  BASED ON EARLIER IMPLEMENTATION OF
//  VRML COMPRESSED BINARY FORMAT PROPOSAL BY:
//    Gabriel Taubin
//    Bill Horn	
//    Francis Lazarus
//
//  Functionality : Error Resilience
//
//  Sungjin Kim  (sunjin@sait.samsung.co.kr)
//  Mahn-Jin Han  (manjini@sait.samsung.co.kr)
//  Mun-Sup Song (mssong@sait.samsung.co.kr)  
//
//  Signal Processing Lab. Samsung A.I.T. 
//-------------------------------------------------------------------------


#include <stdio.h>
#include <stdlib.h>
#include "MPG_Ifs.h"

#define VRML_HEADER "#VRML V2.0 utf8"

//---------------------------------------------------------------------------
//
//   MPG_Ifs::write
//
//---------------------------------------------------------------------------

//dmlab in the not-reordered face set, create structure to get index information

struct indexInfomation
{
	int *index;
	int indexoffset;
}*indexInfo;

//~dmlab (sinwook 060405)

bool MPG_Ifs::write(char* fname)
{
  FILE* fp = fopen(fname,"w");
  if(fp==(FILE*)0) return false;
  
  //dmlab index value j
  int j;
  //~dmlab
  
  vector<float>& coord           = getCoord();
  vector<int>&   coordIndex      = getCoordIndex();

  fprintf(fp,"#VRML V2.0 utf8\n");
  fprintf(fp,"Shape {\n");
  fprintf(fp," appearance Appearance {\n");
  fprintf(fp,"  material Material {  # grey\n");
  fprintf(fp,"   ambientIntensity  0.5\n");
  fprintf(fp,"   diffuseColor      0.7 0.7 0.7\n");
  fprintf(fp,"  }\n");
  fprintf(fp," }\n");

  fprintf(fp," geometry IndexedFaceSet {\n");

  if(getCcw()   ==false) fprintf(fp,"  ccw FALSE\n");
  if(getConvex()==false) fprintf(fp,"  convex FALSE\n");
  if(getSolid() ==false) fprintf(fp,"  solid FALSE\n");
  float creaseAngle = getCreaseAngle();
  if(creaseAngle>0.0)    fprintf(fp,"  creaseAngle %8.4f\n",creaseAngle);
  
/*ory
  if(qFaceIndex.size())
  {
	  qFaceIndexNum = new int[qFaceIndex.size()];
	  int faceCnt = 0, count = 0;
	  for(int j = 0; j < _coordIndex.size(); j++)
	  {
		  if (_coordIndex[j] == -1)
		  {
			  count++;
			  qFaceIndexNum[faceCnt++] = count;
			  count = 0;
		  }
		  else count++;
	  }
  }
*/

  // dmlab vertax order reconstruct array
  int *qVertexReArray = NULL;
  if (qVertexIndex.size())
  {  
	  qVertexReArray = new int[qVertexIndex.size()];
	  for(j = 0; j < qVertexIndex.size(); j++)
	  	  qVertexReArray[qVertexIndex[j]] = j;
  }
  // ~dmlab (sinwook 060405)
  // dmlab face order reconstruct array
  int *qFaceReArray = NULL;
  int *vmap=NULL;
  if (qFaceIndex.size())
  {
	  qFaceReArray = new int[qFaceIndex.size()];
	  for(j = 0; j < qFaceIndex.size(); j++)
	  {
		  qFaceReArray[qFaceIndex[j]] = j;
	  }

	  vmap = new int[qFaceIndex.size()];
	  int ory_cnt=0;
	  for(j=0;j<coordIndex.size();j++)
		if(j==0 || coordIndex[j-1]==-1)
				vmap[qFaceIndex[ory_cnt++]]=j;
  }
  // ~dmlab (sinwook 060405)
  // dmlab initialize indexinfo 
  //indexInfo = new indexInfomation[qFaceIndex.size()];
  //int *tempStoreIndex, tempCountIndex = 0, indexInfoCount = 0;
  // ~dmlab (sinwook 060405)

  //dmlab in the not-reordered face set, create structure to get index information
  /*
  tempStoreIndex = new int[10];
  for(j = 0; j < coordIndex.size(); j++)
  {
	  if (coordIndex[j] == -1)
	  {
		  indexInfo[indexInfoCount].index = new int[tempCountIndex];
		  for(int i = 0; i < tempCountIndex; i++)
			  indexInfo[indexInfoCount].index[i] = tempStoreIndex[i];
		  indexInfo[indexInfoCount].indexoffset = j;
		  tempCountIndex = 0;
		  indexInfoCount++;
	  }
	  else
		  tempStoreIndex[tempCountIndex++] = coordIndex[j];
	  
  }

  printf("%d\n", indexInfoCount);
  */

  {
    int i;
    fprintf(fp,"  coord Coordinate {\n");
    fprintf(fp,"   point [\n");

	if(qVertexIndex.size())
	{
		for(i=0;i<coord.size();i++)
		{
			int vertexOffset = qVertexReArray[i/3]*3+(i%3);
			fprintf(fp, "%8.4f ",coord[vertexOffset]);
			if(i%3==2) fprintf(fp, "\n");//fprintf(fp,"\n");
		}
	}
	else
	{
		for(i=0;i<coord.size();i++)
		{
			fprintf(fp, "%8.4f ",coord[i]);
			if(i%3==2) fprintf(fp, "\n");//fprintf(fp,"\n");
		}
	}

    fprintf(fp,"    ]\n");
    fprintf(fp,"  }\n");
    fprintf(fp,"  coordIndex [\n");
//~dmlab (sinwook 060405)

/*ory
	//dmlab index rearranging with face order
	for(i=0;i<coordIndex.size();i++)
      {
		//if(coordIndex[i] == -2)//
		//qVertexIndex[coordIndex[i]]);
		int faceOffset;
		int VertexIndex;
		if (qFaceReArray != NULL)
		{
			int accFaceIndex = 0;
			int res;
			for(int j = 0; j < qFaceIndex.size(); j++)
			{
				if (accFaceIndex + qFaceIndexNum[j] <= i)
					accFaceIndex += qFaceIndexNum[j];
				else 
				{
					res = i - accFaceIndex;
					accFaceIndex = 0;
					for(int k = 0; k < qFaceReArray[j]; k++)
						accFaceIndex += qFaceIndexNum[k];											
					break;
				}
			}
			faceOffset = accFaceIndex + res;
		}
		else faceOffset = i;
		
		if (qVertexReArray != NULL)
			VertexIndex = coordIndex[qVertexIndex[faceOffset]];
		else 
			VertexIndex = coordIndex[faceOffset];

		//if(qVertexIndex[coordIndex[temp]] == -2)
        if(VertexIndex == -2)
		fprintf(fp,"%d ",0);
		else {
			if (coordIndex[faceOffset] != -1)
				fprintf(fp,"%d ", VertexIndex);
			else fprintf(fp,"%d ",-1);
		}
        if(coordIndex[faceOffset]<0) fprintf(fp,"\n");
			//MJH 062600 : added bracket here
      }
		fprintf(fp,"    ]\n");
*/

//DMLAB ory FaceIndex Rearrange 2006 05
	if(qFaceIndex.size())
	{
		if(qVertexIndex.size())
		{
			for(i=0;i<qFaceIndex.size();i++)
			{
				for(int j=vmap[i];j<coordIndex.size();j++)
				{
					if(coordIndex[j]==-1)
					{
						fprintf(fp,"-1\n");
						break;
					}
					else
						fprintf(fp,"%d ", qVertexIndex[coordIndex[j]]);
				}
			}
		}
		else
		{
			for(i=0;i<qFaceIndex.size();i++)
			{
				for(int j=vmap[i];j<coordIndex.size();j++)
				{
					if(coordIndex[j]==-1)
					{
						fprintf(fp,"-1\n");
						break;
					}
					else
						fprintf(fp,"%d ", coordIndex[j]);
				}
			}
		}
	}
	else if(qVertexIndex.size())
	{
		for(i=0;i<coordIndex.size();i++)
		{
			if(coordIndex[i]==-1)
			{
				fprintf(fp,"-1\n");
			}
			else
			{
				fprintf(fp,"%d ",qVertexIndex[coordIndex[i]]);
			}
		}
	}
	else
	{
		for(i=0;i<coordIndex.size();i++)
		{
			fprintf(fp,"%d ",coordIndex[i]);
			if(coordIndex[i]==-1) fprintf(fp,"\n");
		}
	}

	fprintf(fp,"    ]\n");
//DMLAB ~ory
  }



  // if(color.size()==0)
  //   NO_COLORS
  // else if(colorPerVertex==FALSE)
  //   if(colorIndex.size()>0)
  //     COLOR_PER_FACE_INDEXED;
  //     colorIndex.size()==getNumberOfFaces()
  //   else
  //     COLOR_PER_FACE;
  //     color.size()/3==getNumberOfFaces()
  // else // if(colorPerVertex==TRUE)
  //   if(colorIndex.size()>0)
  //     COLOR_PER_CORNER_INDEXED;
  //     colorIndex.size()==coordIndex.size()
  //   else
  //     COLOR_PER_VERTEX;
  //     color.size()/3==coord.size()/3

  bool           colorPerVertex  = getColorPerVertex();
  vector<float>& color           = getColor();
  vector<int>&   colorIndex      = getColorIndex();
//DMLAB ory color VO FO 2006 06
  MPG_Binding	 colorbinding	 = getColorBinding();

  if(color.size()>0)
  {
    fprintf(fp,"  colorPerVertex %s\n",
		(colorPerVertex==true)?"TRUE":"FALSE\n");

    fprintf(fp,"  color Color {\n");
    fprintf(fp,"   color [\n");

	if(qFaceIndex.size() && colorbinding==PB_PER_FACE)
	{
		for(int i=0;i<qFaceIndex.size();i++)
		{
			int indx=qFaceReArray[i]*3;
			float ci1 = color[indx], ci2 = color[indx+1], ci3 = color[indx+2];
			if(ci1<0.0) ci1=0.0;if(ci1>1.0) ci1=1.0;
			if(ci2<0.0) ci2=0.0;if(ci2>1.0) ci2=1.0;
			if(ci3<0.0) ci3=0.0;if(ci3>1.0) ci3=1.0;
			fprintf(fp,"%8.4f %8.4f %8.4f\n",ci1,ci2,ci3);
		}
	}
	else if(qVertexIndex.size() && colorbinding==PB_PER_VERTEX)
	{
		for(int i=0;i<qVertexIndex.size();i++)
		{
			int indx=qVertexReArray[i]*3;
			float ci1 = color[indx], ci2 = color[indx+1], ci3 = color[indx+2];
			if(ci1<0.0) ci1=0.0;if(ci1>1.0) ci1=1.0;
			if(ci2<0.0) ci2=0.0;if(ci2>1.0) ci2=1.0;
			if(ci3<0.0) ci3=0.0;if(ci3>1.0) ci3=1.0;
			fprintf(fp,"%8.4f %8.4f %8.4f\n",ci1,ci2,ci3);
		}
	}
	else
	{
      for(int i=0;i<color.size();i++)
      {
          float ci = color[i]; //qVertexReArray[i/3]*3+(i%3)];
          if(ci<0.0) ci=0.0;
          if(ci>1.0) ci=1.0;
          fprintf(fp,"%8.4f ",ci);
          if(i%3==2) fprintf(fp,"\n");
      }
	}
    fprintf(fp,"    ]\n");
    fprintf(fp,"  }\n");

	if(colorbinding==PB_PER_CORNER)
	{
        fprintf(fp,"  colorIndex [\n");
		if(qFaceIndex.size())
		{
			for(int i=0;i<qFaceIndex.size();i++)
			{
				for(int j=vmap[i];j<colorIndex.size();j++)
				{
					if(colorIndex[j]==-1)
					{
						fprintf(fp,"-1\n");
						break;
					}
					else
						fprintf(fp,"%d ", colorIndex[j]);
				}
			}
		}
		else
		{
          for(int i=0;i<colorIndex.size();i++)
            {
              fprintf(fp,"%d ",colorIndex[i]); 
              if(colorIndex[i]<0) fprintf(fp,"\n");
            }
 		}
       fprintf(fp,"    ]\n");
	}
  }
//~DMLAB ory
  // if(normal.size()==0)
  //   NO_NORMALS
  // else if(normalPerVertex==FALSE)
  //   if(normalIndex.size()>0)
  //     NORMAL_PER_FACE_INDEXED;
  //     normalIndex.size()==getNumberOfFaces()
  //   else
  //     NORMAL_PER_FACE;
  //     normal.size()/3==getNumberOfFaces()
  // else // if(normalPerVertex==TRUE)
  //   if(normalIndex.size()>0)
  //     NORMAL_PER_CORNER_INDEXED;
  //     normalIndex.size()==coordIndex.size()
  //   else
  //     NORMAL_PER_VERTEX;
  //     normal.size()/3==coord.size()/3

  bool           normalPerVertex = getNormalPerVertex();
  vector<float>& normal          = getNormal();
  vector<int>&   normalIndex     = getNormalIndex();
//DMLAB ory normal VO FO 2006 06
  MPG_Binding	 normalbinding	 = getNormalBinding();
  if(normal.size()>0)
  {
    fprintf(fp,"  normalPerVertex %s\n",
		(normalPerVertex==true)?"TRUE":"FALSE\n");

    fprintf(fp,"  normal Normal {\n");
    fprintf(fp,"   vector [\n");

	if(qFaceIndex.size() && normalbinding==PB_PER_FACE)
	{
		for(int i=0;i<qFaceIndex.size();i++)
		{
			int indx=qFaceReArray[i]*3;
			float ci1 = normal[indx], ci2 = normal[indx+1], ci3 = normal[indx+2];
			if(ci1<0.0) ci1=0.0;if(ci1>1.0) ci1=1.0;
			if(ci2<0.0) ci2=0.0;if(ci2>1.0) ci2=1.0;
			if(ci3<0.0) ci3=0.0;if(ci3>1.0) ci3=1.0;
			fprintf(fp,"%8.4f %8.4f %8.4f\n",ci1,ci2,ci3);
		}
	}
	else if(qVertexIndex.size() && normalbinding==PB_PER_VERTEX)
	{
		for(int i=0;i<qVertexIndex.size();i++)
		{
			int indx=qVertexReArray[i]*3;
			float ci1 = normal[indx], ci2 = normal[indx+1], ci3 = normal[indx+2];
			if(ci1<0.0) ci1=0.0;if(ci1>1.0) ci1=1.0;
			if(ci2<0.0) ci2=0.0;if(ci2>1.0) ci2=1.0;
			if(ci3<0.0) ci3=0.0;if(ci3>1.0) ci3=1.0;
			fprintf(fp,"%8.4f %8.4f %8.4f\n",ci1,ci2,ci3);
		}
	}
	else
	{
      for(int i=0;i<normal.size();i++)
      {
          float ci = normal[i]; //qVertexReArray[i/3]*3+(i%3)];
          if(ci<0.0) ci=0.0;
          if(ci>1.0) ci=1.0;
          fprintf(fp,"%8.4f ",ci);
          if(i%3==2) fprintf(fp,"\n");
      }
	}
    fprintf(fp,"    ]\n");
    fprintf(fp,"  }\n");
	  
	if(normalbinding==PB_PER_CORNER)
	{
        fprintf(fp,"  normalIndex [\n");
		if(qFaceIndex.size())
		{
			for(int i=0;i<qFaceIndex.size();i++)
			{
				for(int j=vmap[i];j<normalIndex.size();j++)
				{
					if(normalIndex[j]==-1)
					{
						fprintf(fp,"-1\n");
						break;
					}
					else
						fprintf(fp,"%d ", normalIndex[j]);
				}
			}			
		}
		else
		{
          for(int i=0;i<normalIndex.size();i++)
            {
              fprintf(fp,"%d ",normalIndex[i]); 
              if(normalIndex[i]<0) fprintf(fp,"\n");
            }
		}
        fprintf(fp,"    ]\n");
	}
  }
//~DMLAB ory

  // if(texCoord.size()==0)
  //   NO_TEX_COORD
  // else if(texCoordIndex.size()>0)
  //   TEX_COORD_PER_CORNER_INDEXED;
  //   texCoordIndex.size()==coordIndex.size()
  // else
  //   TEX_COORD_PER_VERTEX;
  //   texCoord.size()/3==coord.size()/3

  vector<float>& texCoord        = getTexCoord();
  vector<int>&   texCoordIndex   = getTexCoordIndex();
//DMLAB ory tc VO FO 2006 06
  MPG_Binding	 texcoordbinding	 = getTexCoordBinding();

  if(texCoord.size()>0)
    {

      fprintf(fp,"  texCoord TextureCoordinate {\n");
      fprintf(fp,"   point [\n");
	  if(texcoordbinding==PB_PER_VERTEX && qVertexIndex.size())
	  {
		  for(int i=0;i<qVertexIndex.size();i++)
		  {
			  int indx=qVertexReArray[i]*2;
			  float ci1 = texCoord[indx], ci2 = texCoord[indx+1];
			  fprintf(fp,"%8.4f %8.4f\n",ci1,ci2);
		  }
	  }
	  else
	  {
		  for(int i=0;i<texCoord.size();i++)
			{
			  fprintf(fp,"%8.4f ",texCoord[i]);//qVertexReArray[i/2]*2+(i%2)]);
			  if(i%2==1) fprintf(fp,"\n");
			}
	  }
      fprintf(fp,"    ]\n");
      fprintf(fp,"  }\n");
	  
		
		if(texcoordbinding==PB_PER_CORNER)
        {
			fprintf(fp,"  texCoordIndex [\n");
			if(qFaceIndex.size())
			{
				for(int i=0;i<qFaceIndex.size();i++)
				{
					for(int j=vmap[i];j<texCoordIndex.size();j++)
					{
						if(texCoordIndex[j]==-1)
						{
							fprintf(fp,"-1\n");
							break;
						}
						else
							fprintf(fp,"%d ", texCoordIndex[j]);
					}
				}
			}
			else
			{
			  for(int i=0;i<texCoordIndex.size();i++)
				{
				  fprintf(fp,"%d ",texCoordIndex[i]);//qVertexIndex[texCoordIndex[temp]]);
				  if(texCoordIndex[i]<0) fprintf(fp,"\n");
				}
			}
	        fprintf(fp,"    ]\n");
		  }
    }

  fprintf(fp," }\n");		// IndexedFaceSet
  fprintf(fp,"}\n");		// Shape

 /*
  fprintf(fp,"#VertexOrder");
fprintf(fp, "%d\n ", coord.size());
  fprintf(fp, "%d\n ", qVertexIndex.size());
  fprintf(fp, "%d\n ", qFaceIndex.size());
  for(int i=0; i<qVertexIndex.size();i++)
  {
//	  fprintf(fp,"#");
	  fprintf(fp,"%d ",qVertexIndex[i]);
//	  fprintf(fp,"");
  }
  fprintf(fp,"\n");
*/
 // fprintf(fp,"#FaceOrder");
//  for(int i=0; i<qFaceIndex.size();i++)
 // {
//	  fprintf(fp,"#");
//	  fprintf(fp,"%d ",qFaceIndex[i]);
//	  fprintf(fp,"");
 // }

  free(qVertexReArray);
  free(qFaceReArray);
  free(vmap);  //DMLAB ory
  fclose(fp);

  return true;
}
