//-------------------------------------------------------------------------
// 
// Copyright (C) IBM Corporation & Samsung Electronics Co. LTD., 1996, 1997, 1998, 1999
// Licensed Materials, Program Property of IBM & Samsung.
// 
// This software module is an implementation of a part of one or more
// MPEG-4 tools as specified by the MPEG-4 standard (ISO/IEC 14496).
// ISO/IEC gives users of the MPEG-4 standard (ISO/IEC 14496) free
// license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4
// standard (ISO/IEC 14496).  Those intending to use this software module
// in hardware or software products are advised that its use may infringe
// existing patents.  The original developer of this software module and
// his/her company, the subsequent editors and their companies, and
// ISO/IEC have no liability for use of this software module or
// modifications thereof in an implementation.  Copyright is not released
// for non MPEG-4 standard (ISO/IEC 14496) conforming products.  IBM
// Corp. & Samsung Electronics Co. LTD retains full right to use the code 
// for his/her own purpose, assign or donate the code to a third party and
// to inhibit third parties from using the code for non MPEG-4 standard 
// (ISO/IEC 14496) conforming products. This copyright notice must be included
// in all copies or derivative works.
// 
//  AUTHORS:
//    Gabriel Taubin
//    Andre Gueziec
//    Claudio Silva
//    Ioana Martin
//  
//  IBM T. J. Watson Research Center
//  P.O.Box 704
//  Yorktown Heights, New York 10598
//  
//  BASED ON EARLIER IMPLEMENTATION OF
//  VRML COMPRESSED BINARY FORMAT PROPOSAL BY:
//    Gabriel Taubin
//    Bill Horn
//    Francis Lazarus
//
//  Functionality : Error Resilience
//
//  Sungjin Kim  (sunjin@sait.samsung.co.kr)
//  Mahn-Jin Han  (manjini@sait.samsung.co.kr)
//  Mun-Sup Song (mssong@sait.samsung.co.kr)  
//
//  Signal Processing Lab. Samsung A.I.T. 
//-------------------------------------------------------------------------


#include <stdio.h>
#include <stdlib.h>
#include "MPG_MeshEdit.h"

//---------------------------------------------------------------------------
//
//   MPG_MeshEdit_EXT::_cut
//
//---------------------------------------------------------------------------

bool MPG_MeshEdit_EXT::_cut(PartitionMode pm,
			// input : only used if pm==MARKED_EDGES
			vector<bool>* e_mark,
			// out : map [0,nVO) -> [0,nVI)
			vector<int>&  v_stitch)
{
  bool success = true;

  int nVI      = getNumberOfVertices(); // number of input vertices
  MPG_Partition_EXT *p = _partitionCorners(pm,e_mark);
  int nVS      = getNumberOfSingularVertices();
  int nVO      = p->parts();	     // number of output vertices
  if(nVO!=nVI)
    {
      bool had_edges = hasEdges();
      if(had_edges==true) deleteEdges();

      int nF = getNumberOfFaces();
      int nC = p->n(); // total number of face corners

      int    i,j,if0,nf0,if0v0;

      // initialize array of new vertex indices
      vector<int> new_iv((size_t)nC,-1);

      // initialize vertex remaping array
      v_stitch.erase(v_stitch.begin(),v_stitch.end());
      v_stitch.insert(v_stitch.end(),(size_t)nVO,-1);



      // assign vertex indices to the fathers vertices
      int nVU = (nVI>nVO)?nVI:nVO;
      vector<bool> used((size_t)nVU,false);
      // first assign indices in the range [0,nVI)
      for(i=if0=0;if0<nF;if0++)
	{
	  nf0 = getNumberOfFaceCorners(if0);
	  for(if0v0=0;if0v0<nf0;if0v0++,i++)
	    if(used[j=getFaceCorner(if0,if0v0)]==false && j<nVO)
	      { v_stitch[j] = new_iv[p->find(i)] = j; used[j] = true; }
	}
      // now assign remaining indices in [0,nVO)
      for(j=0;j<nVO&&used[j]==true;j++);
      for(i=if0=0;if0<nF;if0++)
	{
	  nf0 = getNumberOfFaceCorners(if0);
	  for(if0v0=0;if0v0<nf0;if0v0++,i++)
	    if(p->size(i)>0 && new_iv[i]==-1)
	      {
		v_stitch[j] = getFaceCorner(if0,if0v0);
		new_iv[i]  = j++;
		while(j<nVO&&used[j]==true) j++;
	      }
	}

      // assign vertex indices to non-fathers vertices
      for(i=0;i<nC;i++)
	if(p->size(i)==0)
	  new_iv[i] = new_iv[p->find(i)];

      // replace face indices ------------------------------------------------
      for(i=if0=0;if0<nF;if0++)
	for(if0v0=0;if0v0<getNumberOfFaceCorners(if0);if0v0++,i++)
	  setFaceCorner(if0,if0v0,new_iv[i]);
      _nV = nVO;

      if(had_edges==true) _createEdges();
    }

  delete p;
  return success;
}
