//-------------------------------------------------------------------------
// 
// Copyright (C) IBM Corporation & Samsung Electronics Co. LTD., 1996, 1997, 1998, 1999
// Licensed Materials, Program Property of IBM & Samsung.
// 
// This software module is an implementation of a part of one or more
// MPEG-4 tools as specified by the MPEG-4 standard (ISO/IEC 14496).
// ISO/IEC gives users of the MPEG-4 standard (ISO/IEC 14496) free
// license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4
// standard (ISO/IEC 14496).  Those intending to use this software module
// in hardware or software products are advised that its use may infringe
// existing patents.  The original developer of this software module and
// his/her company, the subsequent editors and their companies, and
// ISO/IEC have no liability for use of this software module or
// modifications thereof in an implementation.  Copyright is not released
// for non MPEG-4 standard (ISO/IEC 14496) conforming products.  IBM
// Corp. & Samsung Electronics Co. LTD retains full right to use the code 
// for his/her own purpose, assign or donate the code to a third party and
// to inhibit third parties from using the code for non MPEG-4 standard 
// (ISO/IEC 14496) conforming products. This copyright notice must be included
// in all copies or derivative works.
// 
//  AUTHORS:
//    Gabriel Taubin
//    Andre Gueziec
//    Claudio Silva
//    Ioana Martin
//  
//  IBM T. J. Watson Research Center
//  P.O.Box 704
//  Yorktown Heights, New York 10598
//  
//  BASED ON EARLIER IMPLEMENTATION OF
//  VRML COMPRESSED BINARY FORMAT PROPOSAL BY:
//    Gabriel Taubin
//    Bill Horn
//    Francis Lazarus
//
//  Functionality : Error Resilience
//
//  Sungjin Kim  (sunjin@sait.samsung.co.kr)
//  Mahn-Jin Han  (manjini@sait.samsung.co.kr)
//  Mun-Sup Song (mssong@sait.samsung.co.kr)  
//
//  Signal Processing Lab. Samsung A.I.T. 
//-------------------------------------------------------------------------

#include "MPG_QFpm.h"

int MPG_QFpm_EXT::_shift = 0;

struct MPG_QEntry_EXT MPG_QFpm_EXT::model[] = {
  { 0x5601,  1,  1, 1 },
  { 0x3401,  2,  6, 0 },
  { 0x1801,  3,  9, 0 },
  { 0x0ac1,  4, 12, 0 },
  { 0x0521,  5, 29, 0 },
  { 0x0221, 38, 33, 0 },

  { 0x5601,  7,  6, 1 },
  { 0x5401,  8, 14, 0 },
  { 0x4801,  9, 14, 0 },
  { 0x3801, 10, 14, 0 },
  { 0x3001, 11, 17, 0 },
  { 0x2401, 12, 18, 0 },
  { 0x1c01, 13, 20, 0 },
  { 0x1601, 29, 21, 0 },

  { 0x5601, 15, 14, 1 },
  { 0x5401, 16, 14, 0 },
  { 0x5101, 17, 15, 0 },
  { 0x4801, 18, 16, 0 },
  { 0x3801, 19, 17, 0 },
  { 0x3401, 20, 18, 0 },
  { 0x3001, 21, 19, 0 },
  { 0x2801, 22, 19, 0 },
  { 0x2401, 23, 20, 0 },
  { 0x2201, 24, 21, 0 },
  { 0x1c01, 25, 22, 0 },
  { 0x1801, 26, 23, 0 },
  { 0x1601, 27, 24, 0 },
  { 0x1401, 28, 25, 0 },
  { 0x1201, 29, 26, 0 },
  { 0x1101, 30, 27, 0 },
  { 0x0ac1, 31, 28, 0 },
  { 0x09c1, 32, 29, 0 },
  { 0x08a1, 33, 30, 0 },
  { 0x0521, 34, 31, 0 },
  { 0x0441, 35, 32, 0 },
  { 0x02a1, 36, 33, 0 },
  { 0x0221, 37, 34, 0 },
  { 0x0141, 38, 35, 0 },
  { 0x0111, 39, 36, 0 },
  { 0x0085, 40, 37, 0 },
  { 0x0049, 41, 38, 0 },
  { 0x0025, 42, 39, 0 },
  { 0x0015, 43, 40, 0 },
  { 0x0009, 44, 41, 0 },
  { 0x0005, 45, 42, 0 },
  { 0x0001, 45, 43, 0 }
};

void MPG_QFpm_EXT::update(int symbol) 
{
  if (symbol == MPS)
    index = model[index].nMPS;
  else {
    MPS ^= model[index].sMPS;
    index = model[index].nLPS;
  }
  Qe = model[index].Qe << _shift;
}
