// MPG_State_EXT.cpp

#include "MPG_State.h"

//--------------------------------------------------------------------------
// P_MPG_State_EXT
//--------------------------------------------------------------------------
MPG_State_EXT::MPG_State_EXT(int nTriangles)
{
	_nTriangles = nTriangles;
	_nV0 = 0;		// number of vertices so far without current cc
	_nV1 = 0;		// number of vertices so far with current cc
	_nVcls0 = 0;	// number of vertices before clustering
	_nVcls1 = 0;    // number of vertices after clustering

	_newMol = _newMobl = _newCC = _newTri = true;
	_mol_id = 0;
	_lastTriangle = 0;

	_prev_marching_pattern = 0; 
	_prev_polygon_edge = 1;
	_triangulated = 0;

	_nPE = _nF = _nC = 0;
	_nCoord = _nNormal = _nColor = _nTexCoord = 0;
	_iP = -1;
	_iR = 0;
	_iL = 1;
	_iO = -1;
	_iF = _iFp = _iC = 0; 
	_cRp = _cR = 0;
	_cLp = _cL = 1; 
	_cO = -1;
	_nVcc = 0;
	_iRun = 0;
};

//--------------------------------------------------------------------------
// SetState
//--------------------------------------------------------------------------
void MPG_State_EXT::SetState(int nTriangles, 
		int nV0, int nV1, int nVcls0, int nVcls1,
		bool newMol, bool newMobl, bool newCC, bool newTri,
		int mol_id, int lastTriangle, int prev_marching_pattern, 
		int prev_polygon_edge, int triangulated, int nPE, int nF,
		int nC, int nCoord, int nNormal, int nColor, int nTexCoord,
		int iP, int iR, int iL, int iO, int iF, int iFp, int iC,
		int cRp, int cR, int cLp, int cL, int cO, int nVcc, int iRun)
{
	_nTriangles = nTriangles;
	_nV0 = nV0;			// number of vertices so far without current cc
	_nV1 = nV1;		    // number of vertices so far with current cc
	_nVcls0 = nVcls0;	// number of vertices before clustering
	_nVcls1 = nVcls1;   // number of vertices after clustering

	_newMol  = newMol;
	_newMobl = newMobl;
	_newCC   = newCC;
	_newTri  = newTri;
	_mol_id  = mol_id;
	_lastTriangle = lastTriangle;

	_prev_marching_pattern = prev_marching_pattern; 
	_prev_polygon_edge = prev_polygon_edge;
	_triangulated = triangulated;

	_nPE = nPE;
	_nF  = nF;
	_nC  = nC;
	_nCoord    = nCoord;
	_nNormal   = nNormal;
	_nColor    = nColor;
	_nTexCoord = nTexCoord;
	_iP  = iP;
	_iR  = iR;
	_iL  = iL;
	_iO  = iO;
	_iF  = iF;
	_iFp = iFp;
	_iC  = iC; 
	_cRp = cRp;
	_cR  = cR;
	_cLp = cLp;
	_cL  = cL; 
	_cO  = cO;
	_nVcc = nVcc;
	_iRun = iRun;
};

//--------------------------------------------------------------------------
// GetState
//--------------------------------------------------------------------------
void MPG_State_EXT::GetState(int& nTriangles, 
		int& nV0, int& nV1, int& nVcls0, int& nVcls1,
		bool& newMol, bool& newMobl, bool& newCC, bool& newTri,
		int& mol_id, int& lastTriangle, int& prev_marching_pattern, 
		int& prev_polygon_edge, int& triangulated, int& nPE, int& nF,
		int& nC, int& nCoord, int& nNormal, int& nColor, int& nTexCoord,
		int& iP, int& iR, int& iL, int& iO, int& iF, int& iFp, int& iC,
		int& cRp, int& cR, int& cLp, int& cL, int& cO, int& nVcc, int& iRun)
{
	nTriangles = _nTriangles;
	nV0 = _nV0;			// number of vertices so far without current cc
	nV1 = _nV1; 		// number of vertices so far with current cc
	nVcls0 = _nVcls0;	// number of vertices before clustering
	nVcls1 = _nVcls1;   // number of vertices after clustering

	newMol  = _newMol;
	newMobl = _newMobl;
	newCC   = _newCC;
	newTri  = _newTri;
	mol_id  = _mol_id;
	lastTriangle = _lastTriangle;

	prev_marching_pattern = _prev_marching_pattern; 
	prev_polygon_edge = _prev_polygon_edge;
	triangulated = _triangulated;

	nPE = _nPE;
	nF  = _nF;
	nC  = _nC;
	nCoord    = _nCoord;
	nNormal   = _nNormal;
	nColor    = _nColor;
	nTexCoord = _nTexCoord;
	iP  = _iP;
	iR  = _iR;
	iL  = _iL;
	iO  = _iO;
	iF  = _iF;
	iFp = _iFp;
	iC  = _iC; 
	cRp = _cRp;
	cR  = _cR;
	cLp = _cLp;
	cL  = _cL; 
	cO  = _cO;
	nVcc = _nVcc;
	iRun = _iRun;
};

//--------------------------------------------------------------------------
// SetNVertices
//--------------------------------------------------------------------------
void MPG_State_EXT::SetNVertices(int nV0, int nV1, int nVcls0, int nVcls1)
{
	_nV0 = nV0;			// number of vertices so far without current cc
	_nV1 = nV1; 		// number of vertices so far with current cc
	_nVcls0 = nVcls0;	// number of vertices before clustering
	_nVcls1 = nVcls1;   // number of vertices after clustering
}

//--------------------------------------------------------------------------
// GetNVertices
//--------------------------------------------------------------------------
void MPG_State_EXT::GetNVertices(int& nV0, int& nV1, int& nVcls0, int& nVcls1)
{
	nV0 = _nV0;			// number of vertices so far without current cc
	nV1 = _nV1; 		// number of vertices so far with current cc
	nVcls0 = _nVcls0;	// number of vertices before clustering
	nVcls1 = _nVcls1;   // number of vertices after clustering
}