//-------------------------------------------------------------------------
// 
// Copyright (C) IBM Corporation & Samsung Electronics Co. LTD., 1996, 1997, 1998, 1999
// Licensed Materials, Program Property of IBM & Samsung.
// 
// This software module is an implementation of a part of one or more
// MPEG-4 tools as specified by the MPEG-4 standard (ISO/IEC 14496).
// ISO/IEC gives users of the MPEG-4 standard (ISO/IEC 14496) free
// license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4
// standard (ISO/IEC 14496).  Those intending to use this software module
// in hardware or software products are advised that its use may infringe
// existing patents.  The original developer of this software module and
// his/her company, the subsequent editors and their companies, and
// ISO/IEC have no liability for use of this software module or
// modifications thereof in an implementation.  Copyright is not released
// for non MPEG-4 standard (ISO/IEC 14496) conforming products.  IBM
// Corp. & Samsung Electronics Co. LTD retains full right to use the code 
// for his/her own purpose, assign or donate the code to a third party and
// to inhibit third parties from using the code for non MPEG-4 standard 
// (ISO/IEC 14496) conforming products. This copyright notice must be included
// in all copies or derivative works.
// 
//  AUTHORS:
//    Gabriel Taubin
//    Andre Gueziec
//    Claudio Silva
//    Ioana Martin
//  
//  IBM T. J. Watson Research Center
//  P.O.Box 704
//  Yorktown Heights, New York 10598
//  
//  BASED ON EARLIER IMPLEMENTATION OF
//  VRML COMPRESSED BINARY FORMAT PROPOSAL BY:
//    Gabriel Taubin
//    Bill Horn
//    Francis Lazarus
//
//  Functionality : Error Resilience
//
//  Sungjin Kim  (sunjin@sait.samsung.co.kr)
//  Mahn-Jin Han  (manjini@sait.samsung.co.kr)
//  Mun-Sup Song (mssong@sait.samsung.co.kr)  
//
//  Signal Processing Lab. Samsung A.I.T. 
//-------------------------------------------------------------------------


#include <stdio.h>
#include <stdlib.h>
#include "MPG_VGraph.h"

const char MPG_VGraph_EXT::LEAF        = 0x01;
const char MPG_VGraph_EXT::START       = 0x02;
const char MPG_VGraph_EXT::LAST        = 0x04;
const char MPG_VGraph_EXT::JUMP        = 0x08;
const char MPG_VGraph_EXT::J_INV_OR    = 0x10;

const char MPG_VGraph_EXT::NO_LEAF     = (char)0xfe;
const char MPG_VGraph_EXT::NO_START    = (char)0xfd;
const char MPG_VGraph_EXT::NO_LAST     = (char)0xfb;
const char MPG_VGraph_EXT::NO_JUMP     = (char)0xf7;
const char MPG_VGraph_EXT::NO_J_INV_OR = (char)0xef;

//---------------------------------------------------------------------------
//
//   MPG_VGraph_EXT::toLoop
//
//---------------------------------------------------------------------------

bool MPG_VGraph_EXT::toLoop(MPG_Loops_EXT& loop)
{
  loop.clear();
  int nRuns = getNumberOfRuns();
  if(nRuns>0)
    {
      int last,start,leaf,jump,jInvOr;
      int  length,jumpTo;

      vector<int> jumpStackL;
      vector<int> jumpStackR;

      int ivT = 0; // vtree traversal order index
      loop.start(ivT);
      int ivL = ivT;
      int ivR = ivT;
      ivT++;

      int iRun;
      for(iRun=0;iRun<nRuns;iRun++)
	{
	  getRun(iRun,last,start,length,leaf,jump,jInvOr,jumpTo);
	  if(!last)
	    {
	      loop.insert(ivL); ivR = ivL;
	      loop.pushNext();
	    }
	  if(!start)
	    { // end of jump
	      // ignore length (=0), leaf (=1), and jump (=1)
	      //
	      // WARNING
	      // this may invert the orientations
	      // of jumps in the jump stack;
	      // to fix this problem we keep
	      // two stacks jumpStackL and JumpStackR
	      // and if jumpStackR[jumpTo]!=next(jumpStackL[jumpTo])
	      // do the swap with !jInvOr instead
	      //
	      int jFromL = jumpStackL[jumpTo];
	      int jFromR = jumpStackR[jumpTo];
	      int notJInvOr = (jInvOr==1)?0:1;
	      if(loop.isNext(jFromL,jFromR)==true)
		loop.swap(loop.getCurr(),jFromL,jInvOr);
	      else // if(loop.isNext(jFromR,jFromL)==true)
		loop.swap(loop.getCurr(),jFromR,notJInvOr);

	      loop.pop(ivR,ivL);
	    }
	  else
	    {
	      // ignore jInvOr

	      // traverse run
	      int i;
	      for(i=0;i<length-1;i++)
		{
		  loop.insert(ivT); ivR = ivT;
		  loop.advance();
		  loop.insert(ivT); ivL = ivT;
		  ivT++;
		}
	      // last edge
	      if(leaf)
		{
		  // ignore jumpTo (=-1)
		  if(!jump)
		    {
		      loop.insert(ivT);
		      ivT++;
		    }
		  else // if(jump)
		    {
		      // loop.insert(-1);
		      // loop.advance();
		      // loop.insert(-1);
		      jumpStackL.push_back(loop.getCurr());
		      jumpStackR.push_back(loop.getNext());
		    }
		  loop.pop(ivR,ivL);
		}
	      else // if(!leaf)
		{
		  loop.insert(ivT); ivR = ivT;
		  loop.advance();
		  loop.insert(ivT); ivL = ivT;
		  ivT++;
		}
	    }
	}
    }

  return true;
}

