//-------------------------------------------------------------------------
// 
// Copyright (C) IBM Corporation & Samsung Electronics Co. LTD., 1996, 1997, 1998, 1999
// Licensed Materials, Program Property of IBM & Samsung.
// 
// This software module is an implementation of a part of one or more
// MPEG-4 tools as specified by the MPEG-4 standard (ISO/IEC 14496).
// ISO/IEC gives users of the MPEG-4 standard (ISO/IEC 14496) free
// license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4
// standard (ISO/IEC 14496).  Those intending to use this software module
// in hardware or software products are advised that its use may infringe
// existing patents.  The original developer of this software module and
// his/her company, the subsequent editors and their companies, and
// ISO/IEC have no liability for use of this software module or
// modifications thereof in an implementation.  Copyright is not released
// for non MPEG-4 standard (ISO/IEC 14496) conforming products.  IBM
// Corp. & Samsung Electronics Co. LTD retains full right to use the code 
// for his/her own purpose, assign or donate the code to a third party and
// to inhibit third parties from using the code for non MPEG-4 standard 
// (ISO/IEC 14496) conforming products. This copyright notice must be included
// in all copies or derivative works.
// 
//  AUTHORS:
//    Gabriel Taubin
//    Andre Gueziec
//    Claudio Silva
//    Ioana Martin
//  
//  IBM T. J. Watson Research Center
//  P.O.Box 704
//  Yorktown Heights, New York 10598
//  
//  BASED ON EARLIER IMPLEMENTATION OF
//  VRML COMPRESSED BINARY FORMAT PROPOSAL BY:
//    Gabriel Taubin
//    Bill Horn
//    Francis Lazarus
//
//  Functionality : Error Resilience
//
//  Sungjin Kim  (sunjin@sait.samsung.co.kr)
//  Mahn-Jin Han  (manjini@sait.samsung.co.kr)
//  Mun-Sup Song (mssong@sait.samsung.co.kr)  
//
//  Signal Processing Lab. Samsung A.I.T. 
//-------------------------------------------------------------------------

#include <stdio.h>
#include <stdlib.h>
#include "MPG_Global.h"
#include "MPG_VGraph.h"

//---------------------------------------------------------------------------
//
//   MPG_VGraph_EXT::getFather
//
//---------------------------------------------------------------------------

bool MPG_VGraph_EXT::getFather(vector<int>& v_father)
{
  int nRuns = getNumberOfRuns();
  if(nRuns>0)
    {
      int last,start,leaf,jump,jInvOr;
      int length,jumpTo;

      vector<int> fatherStack;
      int ivT = v_father.size(); // vtree traversal order index
      fatherStack.push_back(ivT);
      // father[root] = root

      v_father.push_back(ivT);
      // advance
      int ivF = ivT;
      ivT++;

      int iRun;
      for(iRun=0;iRun<nRuns;iRun++)
	{
	  getRun(iRun,last,start,length,leaf,jump,jInvOr,jumpTo);
	  if(!last)
	    fatherStack.push_back(ivF);
	  if(start)
	    {
	      int i;
	      // traverse edges in run but last one
	      for(i=0;i<length-1;i++)
		{
		  // v_father[ivT] = ivF;
		  v_father.push_back(ivF);
		  ivF = ivT++;
		}
	      // last edge
	      if(leaf)
		{
		  if(!jump)
		    {
		      // traverse last edge
		      // v_father[ivT] = ivF;
		      v_father.push_back(ivF);
		      ivT++;
		    }
		  ivF = fatherStack.back(); fatherStack.pop_back();
		}
	      else // branching node
		{
		  // traverse last edge
		  // v_father[ivT] = ivF;
		  v_father.push_back(ivF);
		  ivF = ivT++;
		}
	    }
	  else
	    { ivF = fatherStack.back(); fatherStack.pop_back(); }
	}
    }

  return true;
}

//---------------------------------------------------------------------------
//
//   MPG_VGraph_EXT::decode
//
//---------------------------------------------------------------------------

bool MPG_VGraph_EXT::decode(MPG_QFac_EXT &codec)
{
  bool success = false;

  clear();

  {
    // determine if all the graphs are simple or not
	  bool isSimple = (codec.decode(MPG_Global_EXT::vg_simple)==1)?true:false;

    vector<int> jStack;

    // vertex run record fields
    bool last   = false;
    bool start  = true;
    int  length = 0;
    bool leaf   = false;
    bool jump   = false;
    int  jumpTo = -1;

    // traversal variables
    int  iRun           = 0;	// within current VertexGraph
    int  vg_run_depth   = 0;	// depth in traversal stack
    bool after_branch   = false;// true if first run after branch ending run
    bool start_of_graph = false;// true if first run of a VertexGraph

    vg_run_depth = 1;
    after_branch = false;
    start_of_graph = true;

    //int i;
    //for(i=0;i<nRuns;i++)
    while(1)
      {
		last  = (after_branch) ? false : (codec.decode(MPG_Global_EXT::vg_last)==1)?true:false;
	if(!last) vg_run_depth++;

	//	start = (isSimple||start_of_graph==true||after_branch)?
	//true : codec.decode(bpmStart);

	int nS = 0;
	int i;
	for(i=0;i<jStack.size();i++)
	  if(jStack[i]>=0)
	    nS++;

	if(!after_branch && nS > 0) {
		start = (codec.decode(MPG_Global_EXT::vg_forward_run)==1)?true:false;
	}
	else
	  start = 1;

	//if(start_of_graph==true)
	//start_of_graph = false;

	if(start)
	  {
	    for(length=1;codec.decode(MPG_Global_EXT::vg_run_length)==false;length++);
		leaf=(codec.decode(MPG_Global_EXT::vg_leaf)==1)?true:false;
	    if(leaf)
	      {
		if(!isSimple)
		  {
			jump = (codec.decode(MPG_Global_EXT::vg_loop)==1)?true:false;
		    if(jump) jStack.push_back(iRun);
		  }
		vg_run_depth--;
	      }
	    else // set ignored fields
	      {
		jump = false;
	        //jInvOr = false;
	      }

	    // set ignored fields
	    jumpTo = -1;
	  }
	else // !start
	  {
	    //jInvOr = (isOriented)?false:codec.decode(bpmJInvOr);
	    {
	      int j = 0;
	      while (1) {
		if (j == nS-1)
		  break;
		if (codec.decode(MPG_Global_EXT::vg_loop_index))
		  break;
		j++;
	      }

	      jumpTo = 0;
	      while (jStack[jumpTo] < 0)
		jumpTo++;
	      for (i=0; i<j; i++) {
		jumpTo++;
		while (jStack[jumpTo] < 0)
		  jumpTo++;
	      }
	      // must have found jumpTo in jStack !!!
	      jStack[jumpTo] = -1;
	    }

	    vg_run_depth--;

	    // set ignored fields
	    length = 0;
	    leaf   = true;
	    jump   = true;
	  }

	pushBackRun(last,start,length,leaf,jump,false,jumpTo); iRun++;

	after_branch = (leaf)?false:true;

	if(vg_run_depth==0)
	  {
	    jStack.erase(jStack.begin(),jStack.end());
	    return true;
	  }
      }
  }

  return true;
}
