/***********************************************************************
This software module was originally developed by Marius PREDA
(INT/ARTEMIS) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. VUB retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2001
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Marius PREDA at marius.preda@int-evry.fr.
----------------------------------------------------------------------*/

// BBADataExchange.h: interface for the BBADecoderLib class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(_bba_data_exchange)
#define _bba_data_exchange

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

class BBADataExchange
{
public:
  enum { HEADER_SIZE = 2*sizeof(int) };
  enum {
    TAG_KEY_FRAME,
    TAG_NO_BONES,
    TAG_NO_MUSCLES,
    TAG_NO_MORPHS,
    TAG_BONES,
    TAG_MUSCLES,
    TAG_MORPHS
  };

  BBADataExchange(BYTE *pData, bool bInit, int size = 0)
  {
    m_pBuffer = pData;
    if (bInit)
    {
      m_size = ((int *) m_pBuffer)[0];
      if (m_size > 8)
        m_noTags = ((int *) m_pBuffer)[1];
      else
        m_noTags = 0;
    }
    else
    {
      m_size = size;
      m_noTags = 0;
    }

//    ASSERT(m_size >= 4);
    m_idxTag = 0;

    // skip the size field
    m_lastPos = HEADER_SIZE;
  };

  int ParseNoTags() { return m_noTags; }

  bool StoreNoTags(int noTags)
  {
    if (m_pBuffer == NULL || m_size < 8)
      return false;

    // skip the size field
    ((int *) m_pBuffer)[1] = noTags;
    return true;
  };

  void * FetchNext(BYTE &tag, DWORD &size)
  {
    if (m_pBuffer == NULL || ++m_idxTag > m_noTags)
      return NULL;

    int headerSize = sizeof(BYTE) + sizeof(DWORD);
//    ASSERT((m_lastPos + headerSize) < m_size);

    // read the tag and move the buffer pointer
    tag = m_pBuffer[m_lastPos];
    m_lastPos += sizeof(BYTE);

    // read the size of the part and move the buffer pointer
    size = *((DWORD *) &m_pBuffer[m_lastPos]);
    m_lastPos += sizeof(DWORD);

    void *buffer = &m_pBuffer[m_lastPos];
    m_lastPos += size;

    return buffer;
  };

  bool StoreNext(BYTE tag, DWORD size, void *pData)
  {
    int blockSize = sizeof(BYTE) + sizeof(DWORD) + size;

    if (m_pBuffer == NULL ||
        (m_lastPos + blockSize) >= m_size)
      return false;

    // write the tag and move the buffer pointer
    m_pBuffer[m_lastPos] = tag;
    m_lastPos += sizeof(BYTE);

    // write the size of the part and move the buffer pointer
    *((DWORD *) &m_pBuffer[m_lastPos]) = size;
    m_lastPos += sizeof(DWORD);

    // write the data
    memcpy(&m_pBuffer[m_lastPos], pData, size);
    m_lastPos += size;

    return true;
  };

  int Finish()
  {
    return *((int *) &m_pBuffer[0]) = m_lastPos;
  };

protected:
  DWORD   m_size;
  DWORD   m_lastPos;
  int     m_noTags;
  int     m_idxTag;
  BYTE  * m_pBuffer;
};

#endif // !defined(_bba_data_exchange)
