/********************************************************************************/
/* This software module was developped by										*/

/*	   Marius Preda (MP), marius.preda@int-evry.fr,								*/
/*                        Institut National des Telecommunications, France		*/

/* in the course of development of the MPEG-4 standard (Part 16 AFX).			*/
/* This software module is an implementation of a part of one or				*/
/* more MPEG-4 tools as specified by the										*/
/* MPEG-4 standard. ISO/IEC gives users of the MPEG-4 standard					*/
/* free license to this software module or modifications thereof				*/
/* for use in hardware or software products claiming conformance				*/
/* to the MPEG-4 standard. Those intending to use this software					*/
/* module in hardware or software products are advised that its					*/
/* use may infringe existing patents. The original developer of					*/
/* this software module and his company, the subsequent							*/
/* editors and their companies, and ISO/IEC have no liability					*/
/* for use of this software module or modifications thereof in					*/
/* an implementation. Copyright is not released for non							*/
/* MPEG-4 standard conforming products. INT ARTEMIS								*/
/* retain full right to use the code for their own purposes,					*/
/* assign or donate the code to a third party and to inhibit					*/
/* third parties from using the code for non MPEG-4 standard					*/
/* conforming products. This copyright notice must be included					*/
/* in all copies or derivative works.    										*/
/* Copyright (c)2002 - 2005														*/
/********************************************************************************/

// BBADecoder.h: interface for the BBADecoder class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_BBADECODER_H__93F1CF93_D37D_11D5_B9F1_000102A02E5B__INCLUDED_)
#define AFX_BBADECODER_H__93F1CF93_D37D_11D5_B9F1_000102A02E5B__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "BBATypeDefs.h"

#ifndef BYTE
typedef unsigned char BYTE;
#endif

#ifndef LPBYTE
typedef BYTE* LPBYTE;
#endif

class BBADecoderLib  
{
public:
	BBADecoderLib(void){
		initialised = 0;
		low=0;
		high=BBA_top;
		length=0;
		zerocount = 0;
		arithmeticAlreadyInit=false;
		dctAlreadyInit=false;
	}

	void initStream(FILE *fin);

	~BBADecoderLib(void){}

	int initialised;
	int coding_type;
	int zerorun;
	long low, high, code_value, bit, length, sacindex, cum, zerocount;

	
	int bba_pred_scale[32]; 
	void init(void);
	sb_object_plane_t bbaop;

	/* Adaptive Arithmetic Coding */
	void ArithmeticInit();
	int decode_bba_object_plane_header(unsigned int *PTS);
	int decode_bba_object_plane_data();
	bool decode_bba_object_plane_mask();
	int decode_bba_object_plane_values();
	bool decode_bone_mask(CBone *bn);
	bool decode_muscle_mask(CMuscle *ms);
	void decode_bba_new_minmax();
	int AllocThePlane();
	int FreeThePlane();
	int LoadThePlane();
	int UnLoadThePlane();
	void SetMinMax();
	bool arithmeticAlreadyInit;

	int decode_a_symbol(int*);
	void decoder_reset(void); 
	void bit_out_psc_layer(void);
	void update(int, int);
	void init_arith(void);

	int bba_decode_a_symbol(int*);
	void bba_decoder_reset(void); 
	void bba_bit_out_psc_layer(void);
	void bba_init_arith(void);
	void bba_update(int, int);

	void initbits(void);
	void resetbits(void);
	int getbits(int);
	int getbit(void);
	int shortgetbits(int n);

	void next_start_code(void);
	int nextbits_bytealigned(void);
	void adjust_bits(void);
	int cntbits(void);

	int  recon(int fap_diff, int fap_prev) ;
	int  dequant(int qfap, int qstep) ;



	/* DCT related functions */
	void DCTInit();
	void bba_init_htables();
	void decode_bba_i_segment(); 
	void decode_bba_p_segment(); 
	int  getIDC();
	void bba_getAC(int*);
	int  bba_getPDC(); 
	int  huffman_decode(int **table, int size);
	void bba_dctdequant(int*, float*, int);
	void bba_idctTrans(float*, int*);
	void AllocThePlaneSegment();
	void LoadThePlaneSegment();
	void UnLoadThePlaneSegment();
	void FreeThePlaneSegment();
	bool dctAlreadyInit;



	int  bba_object_plane_data() ;
	int  bba_object_plane_header();

	void bba_decode_skip_frames(void);
	void bba_decode_frame_rate(void);
	void bba_decode_time_code(void);
	void decode_skip_frames(void);
	void bba_decode_new_minmax(void);
	void decode_bba_i_frame();
	void decode_bba_p_frame();

	void print_fba_header(void);

	//inline void assert(int i){};
	
	int WriteDecodedData(FILE *bbafp);
	bool WriteDecodedData(LPBYTE pOutput, int &len);
	int WriteDecodedDataInStructure(sb_object_plane_t_decoded *bf);

	void WriteOneFrame(FILE *bbafp);
	bool WriteOneFrame(LPBYTE pOutput, int &len);
	void WriteOneFrameInStructure(sb_object_plane_t_decoded *bf);

	void WriteOneSegment(FILE *bbafp);
	bool WriteOneSegment(LPBYTE pOutput, int &len);
	void WriteOneSegmentInStructure(sb_object_plane_t_decoded *bf);

	void ReportError(char *buffer){
	}
	bool decode_morph_mask(CMorph *ms);

int
BBADecoderLib::bba_decoder(unsigned char *AUptr, int bitoffset, int AUbitcount,
        sb_object_plane_t_decoded *decodedPlane, int bba_bytecount, unsigned int *PTS); 
int
BBADecoderLib::bba_decoder(unsigned char *AUptr, int bitoffset, int AUbitcount,
						   LPBYTE decodedBuffer, int &outputLenght) ;

int 
BBADecoderLib::bba_parsing(unsigned char * &AUStream, int *AULenght, int n,int *cm);

//	int frame;
int BBADecoderLib::getPlaneStartOffset(void);
bool eof;


};

#endif // !defined(AFX_FBADECODER_H__93F1CF93_D37D_11D5_B9F1_000102A02E5B__INCLUDED_)
