/********************************************************************************/
/* This software module was developped by										*/

/*	   Marius Preda (MP), marius.preda@int-evry.fr,								*/
/*                        Institut National des Telecommunications, France		*/

/* in the course of development of the MPEG-4 standard (Part 16 AFX).			*/
/* This software module is an implementation of a part of one or				*/
/* more MPEG-4 tools as specified by the										*/
/* MPEG-4 standard. ISO/IEC gives users of the MPEG-4 standard					*/
/* free license to this software module or modifications thereof				*/
/* for use in hardware or software products claiming conformance				*/
/* to the MPEG-4 standard. Those intending to use this software					*/
/* module in hardware or software products are advised that its					*/
/* use may infringe existing patents. The original developer of					*/
/* this software module and his company, the subsequent							*/
/* editors and their companies, and ISO/IEC have no liability					*/
/* for use of this software module or modifications thereof in					*/
/* an implementation. Copyright is not released for non							*/
/* MPEG-4 standard conforming products. INT ARTEMIS								*/
/* retain full right to use the code for their own purposes,					*/
/* assign or donate the code to a third party and to inhibit					*/
/* third parties from using the code for non MPEG-4 standard					*/
/* conforming products. This copyright notice must be included					*/
/* in all copies or derivative works.    										*/
/* Copyright (c)2002 - 2005														*/
/********************************************************************************/

#ifndef __BBATYPEDEFS_H_
#define __BBATYPEDEFS_H_

#include <stdio.h>
#include <stdlib.h>
#include "CBone.h"
#include "CMuscle.h"
#include "CMorph.h"

#define MORPH 1

#ifndef PI
#define PI 3.1415926
#endif

#define MARKER_BIT 1

#ifndef min
#define min(a, b)       ((a) < (b) ? (a) : (b))
#define max(a, b)       ((a) < (b) ? (b) : (a))
#endif 

#define LOG_BUFSIZE  4096
#define MAX_BITS_PER_FRAME	4096

#define MY_DEBUG 0
#define MY_DEBUG_S 0

typedef int bool_t;

#define RANGE_DELTA	60


#define BBA_OBJECT_START_CODE			490 //257
#define BBA_OBJECT_PLANE_START_CODE     492 //259
#define BBA_OBJECT_END_CODE				491 //258

#define PREDICTIVE_CODING_TYPE          0 
#define DCT_CODING_TYPE                   1 


#define SEG_LENGTH        16

#define   q1    16384
#define   q2    32768
#define   q3    49152
#define   BBA_top   65535
#define SHORT_MIN	-32767
#define SHORT_MAX	32767


#define DEF_FRAME_RATE	30.0
#define DEBUG_BBA 0
#define ZEROMAX		22
#define	Max_frequency	16383
#define sign(x) 	((x) < 0 ? -1 : 1)
#define MAXINT 0x7FFFFFFF 

#ifndef DWORD
typedef unsigned long DWORD;
#endif

typedef struct
{
  unsigned char *buf;
  unsigned char inbfr;
  int p;
  int incnt;
  int bitcnt;
  int bufsize;
  unsigned char outbfr;// for encoder
  int outcnt; // for encoder 
  int outframebits;

} bba_in_buffer;

typedef struct 
{
	bool *sbc_mask;
	bool *sbc_mask_prev;
	int *sbc_values;
	int *sbc_values_prev;

  bool is_intra;
  int bba_quant;
  int pow2quant;
  int noOfControllerTypes;
  int bba_object_coding_type;
  int KeyFrameIndex;

  bool is_time_code;
  bool is_frame_rate;
  bool skip_frames;
  int frame_rate, seconds, frequency_offset;
  int time_code_hours, time_code_minutes, marker_bit, time_code_seconds;
  int number_of_frames_to_skip;
  
  bba_in_buffer stream;
  
  CBone *firstBone;
  CBone *lastBone;
  CMuscle *firstMuscle;
  CMuscle *lastMuscle;
  CMorph *firstMorph;
  CMorph *lastMorph;


  int NumberOfBones;
  int NumberOfMuscles;
  int NumberOfMorphs;
  int NumberOfSBComponentsInPlane;

  // arithmetic based encoding //
  int is_i_new_max, is_i_new_min, is_p_new_max, is_p_new_min;

  int *Irange_max, *Irange_min;
  int *range_max , *range_min;
  
  int **Ifreq , **Icumul_freq;
  int **freq, **cumul_freq;
  // end arithmetic based encoding //
  
  // DCT based encoding //

  int **sbc_DCTvalues;
  float **idctMatrix;
  int **dc_table;
  int **ac_table;
  int **runlength_table;
  int dc_table_size;
  int ac_table_size;
  int runlength_table_size;
  // end DCT based encoding //


} sb_object_plane_t;


typedef struct 
{
  CBone *firstBone;
  CMuscle *firstMuscle;
  CMorph *firstMorph;
  int KeyFrameIndex;

  int NumberOfBones;
  int NumberOfMuscles;
  int NumberOfMorphs;
  int NumberOfSBComponentsInPlane;
} sb_object_plane_t_decoded;



#endif
