/********************************************************************************/
/* This software module was developped by										*/

/*	   Marius Preda (MP), marius.preda@int-evry.fr,								*/
/*                        Institut National des Telecommunications, France		*/

/* in the course of development of the MPEG-4 standard (Part 16 AFX).			*/
/* This software module is an implementation of a part of one or				*/
/* more MPEG-4 tools as specified by the										*/
/* MPEG-4 standard. ISO/IEC gives users of the MPEG-4 standard					*/
/* free license to this software module or modifications thereof				*/
/* for use in hardware or software products claiming conformance				*/
/* to the MPEG-4 standard. Those intending to use this software					*/
/* module in hardware or software products are advised that its					*/
/* use may infringe existing patents. The original developer of					*/
/* this software module and his company, the subsequent							*/
/* editors and their companies, and ISO/IEC have no liability					*/
/* for use of this software module or modifications thereof in					*/
/* an implementation. Copyright is not released for non							*/
/* MPEG-4 standard conforming products. INT ARTEMIS								*/
/* retain full right to use the code for their own purposes,					*/
/* assign or donate the code to a third party and to inhibit					*/
/* third parties from using the code for non MPEG-4 standard					*/
/* conforming products. This copyright notice must be included					*/
/* in all copies or derivative works.    										*/
/* Copyright (c)2002 - 2005														*/
/********************************************************************************/

#include<stdio.h>
#include<stdlib.h>

#include "CMuscle.h"


CMuscle::CMuscle(){

	muscleDCTData = NULL;
	prec=NULL;
	suiv=NULL;
}

CMuscle::~CMuscle()
{

}

void CMuscle::AllocMuscle(){

	int i;
	muscleMask.isControlPointOnXChanged=(bool*)malloc(NumberControlPoints*sizeof(bool));
	muscleMask.isControlPointOnYChanged=(bool*)malloc(NumberControlPoints*sizeof(bool));
	muscleMask.isControlPointOnZChanged=(bool*)malloc(NumberControlPoints*sizeof(bool));
	muscleMask.isWheighOfControlPointChanged=(bool*)malloc(NumberControlPoints*sizeof(bool));
	muscleMask.isKnotChanged=(bool*)malloc(NumberKnots*sizeof(bool));

	muscleData.ControlPointOnX=(int*)malloc(NumberControlPoints*sizeof(int));
	muscleData.ControlPointOnY=(int*)malloc(NumberControlPoints*sizeof(int));
	muscleData.ControlPointOnZ=(int*)malloc(NumberControlPoints*sizeof(int));
	muscleData.WheighOfControlPoint=(int*)malloc(NumberControlPoints*sizeof(int));
	muscleData.Knot=(int*)malloc(NumberKnots*sizeof(int));
	for (i=0;i<NumberControlPoints;i++){
		muscleMask.isControlPointOnXChanged[i]=false;
		muscleMask.isControlPointOnYChanged[i]=false;
		muscleMask.isControlPointOnZChanged[i]=false;
		muscleMask.isWheighOfControlPointChanged[i]=false;
		muscleData.ControlPointOnX[i]=0;
		muscleData.ControlPointOnY[i]=0;
		muscleData.ControlPointOnZ[i]=0;
		muscleData.WheighOfControlPoint[i]=0;
	}
	for (i=0;i<NumberKnots;i++){
		muscleMask.isKnotChanged[i]=false;
		muscleData.Knot[i]=0;
	}


};

void CMuscle::FreeMuscle(){
	
	muscleMask.Free();
	muscleData.Free();

};

void CMuscle::AllocMuscle(int ncp, int nk){

	NumberControlPoints=ncp;
	NumberKnots=nk;
	AllocMuscle();
};

void CMuscle::AllocMinMaxMuscle(){

	int i;
	muscleDataIMax.ControlPointOnX=(int*)malloc(NumberControlPoints*sizeof(int));
	muscleDataIMax.ControlPointOnY=(int*)malloc(NumberControlPoints*sizeof(int));
	muscleDataIMax.ControlPointOnZ=(int*)malloc(NumberControlPoints*sizeof(int));
	muscleDataIMax.WheighOfControlPoint=(int*)malloc(NumberControlPoints*sizeof(int));
	muscleDataIMax.Knot=(int*)malloc(NumberKnots*sizeof(int));

	muscleDataIMin.ControlPointOnX=(int*)malloc(NumberControlPoints*sizeof(int));
	muscleDataIMin.ControlPointOnY=(int*)malloc(NumberControlPoints*sizeof(int));
	muscleDataIMin.ControlPointOnZ=(int*)malloc(NumberControlPoints*sizeof(int));
	muscleDataIMin.WheighOfControlPoint=(int*)malloc(NumberControlPoints*sizeof(int));
	muscleDataIMin.Knot=(int*)malloc(NumberKnots*sizeof(int));

	muscleDataPMax.ControlPointOnX=(int*)malloc(NumberControlPoints*sizeof(int));
	muscleDataPMax.ControlPointOnY=(int*)malloc(NumberControlPoints*sizeof(int));
	muscleDataPMax.ControlPointOnZ=(int*)malloc(NumberControlPoints*sizeof(int));
	muscleDataPMax.WheighOfControlPoint=(int*)malloc(NumberControlPoints*sizeof(int));
	muscleDataPMax.Knot=(int*)malloc(NumberKnots*sizeof(int));

	muscleDataPMin.ControlPointOnX=(int*)malloc(NumberControlPoints*sizeof(int));
	muscleDataPMin.ControlPointOnY=(int*)malloc(NumberControlPoints*sizeof(int));
	muscleDataPMin.ControlPointOnZ=(int*)malloc(NumberControlPoints*sizeof(int));
	muscleDataPMin.WheighOfControlPoint=(int*)malloc(NumberControlPoints*sizeof(int));
	muscleDataPMin.Knot=(int*)malloc(NumberKnots*sizeof(int));
	for (i=0;i<NumberControlPoints;i++){
		muscleDataIMax.ControlPointOnX[i]=1860;
		muscleDataIMax.ControlPointOnY[i]=1860;
		muscleDataIMax.ControlPointOnZ[i]=1860;
		muscleDataIMax.WheighOfControlPoint[i]=1860;

		muscleDataIMin.ControlPointOnX[i]=-1860;
		muscleDataIMin.ControlPointOnY[i]=-1860;
		muscleDataIMin.ControlPointOnZ[i]=-1860;
		muscleDataIMin.WheighOfControlPoint[i]=-1860;

		muscleDataPMax.ControlPointOnX[i]=600;
		muscleDataPMax.ControlPointOnY[i]=600;
		muscleDataPMax.ControlPointOnZ[i]=600;
		muscleDataPMax.WheighOfControlPoint[i]=600;

		muscleDataPMin.ControlPointOnX[i]=-600;
		muscleDataPMin.ControlPointOnY[i]=-600;
		muscleDataPMin.ControlPointOnZ[i]=-600;
		muscleDataPMin.WheighOfControlPoint[i]=-600;
	}
	for (i=0;i<NumberKnots;i++){
		muscleDataIMax.Knot[i]=1860;
		muscleDataIMin.Knot[i]=-1860;
		muscleDataPMax.Knot[i]=600;
		muscleDataPMin.Knot[i]=-600;
	}

};
void CMuscle::AllocDCTMuscle(){

	int i,seg;
	for (seg=0;seg<16;seg++){
		muscleDCTData[seg].ControlPointOnX=(int*)malloc(NumberControlPoints*sizeof(int));
		muscleDCTData[seg].ControlPointOnY=(int*)malloc(NumberControlPoints*sizeof(int));
		muscleDCTData[seg].ControlPointOnZ=(int*)malloc(NumberControlPoints*sizeof(int));
		muscleDCTData[seg].WheighOfControlPoint=(int*)malloc(NumberControlPoints*sizeof(int));
		muscleDCTData[seg].Knot=(int*)malloc(NumberKnots*sizeof(int));
		for (i=0;i<NumberControlPoints;i++){
			muscleDCTData[seg].ControlPointOnX[i]=0;
			muscleDCTData[seg].ControlPointOnY[i]=0;
			muscleDCTData[seg].ControlPointOnZ[i]=0;
			muscleDCTData[seg].WheighOfControlPoint[i]=0;
		}
		for (i=0;i<NumberKnots;i++){
			muscleDCTData[seg].Knot[i]=0;
		}
	}

};
void CMuscle::FreeDCTMuscle(){
	
	for (int seg=0;seg<16;seg++)
		muscleDCTData[seg].Free();

};

void CMuscle::FreeMinMaxMuscle(){
	muscleDataIMax.Free();
	muscleDataIMin.Free();
	muscleDataPMax.Free();
	muscleDataPMin.Free();
}

void CMuscle::AllocPrevMuscle(){

	muscleDataPreviousFrame.ControlPointOnX=(int*)malloc(NumberControlPoints*sizeof(int));
	muscleDataPreviousFrame.ControlPointOnY=(int*)malloc(NumberControlPoints*sizeof(int));
	muscleDataPreviousFrame.ControlPointOnZ=(int*)malloc(NumberControlPoints*sizeof(int));
	muscleDataPreviousFrame.WheighOfControlPoint=(int*)malloc(NumberControlPoints*sizeof(int));
	muscleDataPreviousFrame.Knot=(int*)malloc(NumberKnots*sizeof(int));
}
void CMuscle::FreePrevMuscle(){
	muscleDataPreviousFrame.Free();
}

void innod(CMuscle *p,CMuscle * &first,CMuscle * &last)
{
	// add or update a muscle content

  CMuscle *q;
  q=first;
  
  if(first==NULL){
      first=p;
      first->suiv=NULL;
      first->prec=NULL;
      last=p;
      last->prec=NULL;
      last->suiv=NULL;
      return;
    }
  if(p->muscleIdentifier < first->muscleIdentifier){
      p->suiv=first;
      p->prec=NULL;
      first->prec=p;
      first=p;
      return;
    }
  if(p->muscleIdentifier > last->muscleIdentifier){
      last->suiv=p;
      p->suiv=NULL;
      p->prec=last;
      last=p;
	  //p->muscleDataPreviousFrame.init();
      return;
    }
  while (q){
	  if (p->muscleIdentifier == q->muscleIdentifier){
	  // just update the fields
		//q->muscleDataPreviousFrame=p->muscleDataPreviousFrame;
		q->muscleMask=p->muscleMask;
		q->muscleDataIMax=p->muscleDataIMax;
		q->muscleDataIMin=p->muscleDataIMin;
		q->muscleDataPMax=p->muscleDataPMax;
		q->muscleDataPMin=p->muscleDataPMin;
		q->canBePredicted=p->canBePredicted;
		q->isInCurrentFrame=true;
		return;
	  }
	  else q=q->suiv;
  }
  q=first;
  while (q){
	  while (p->muscleIdentifier < q->muscleIdentifier)
		q=q->suiv;
	  p->suiv=q;
	  p->prec=q->prec;
	  q->prec->suiv=p;
	  q->prec=p;
	  return;
  }
}



CMuscle *getnodindexi(int index,CMuscle * &first,CMuscle * &last)
{
  CMuscle *q;
  q=first;
  
  if(first==NULL){
      return NULL;
    }
  
  for (int i=0;i<index;i++)
	q=q->suiv;
  return q;
}

CMuscle *getnodIDi(int ID,CMuscle * &first,CMuscle * &last)
{
  CMuscle *q;
  q=first;
  
  if(first==NULL){
      return NULL;
    }
  
  while (q) {
	  if (q->muscleIdentifier==ID)
		  return q;
	  q=q->suiv;
  }
  return q;
}


void outnod(CMuscle * p,CMuscle * &first,CMuscle * &last)
{
  
  if(first==NULL) return;

  if(p==first)
    {
      first=first->suiv;
      delete p;
      if(first==NULL) last=NULL;
      else first->prec=NULL;
      return;
     
    }
  if(p==last)
    {
      last=last->prec;
      delete p;
      if(last==NULL) first=NULL;
      else last->suiv=NULL;
      return;
    }
  p->prec->suiv=p->suiv;
  p->suiv->prec=p->prec;
  delete p;
}

void freeliste(CMuscle * &first,CMuscle* &last)
{
  CMuscle *p=NULL;

  if(first==NULL) return;
  
  do
    {
      p=first;
      outnod(p,first,last);
    }while(first != NULL);
}
      
 
void affiche_liste(CMuscle *first)
{
 
  CMuscle *p=NULL;
    
  if(first==NULL)
    {
      fprintf(stderr,"liste vide\n");
      return;
    }
  p=first;
  do
    {
     
      fprintf(stderr,"%d, ",p->muscleIdentifier);
      p=p->suiv;
    }while(p != NULL);
  fprintf(stderr,"\n");
}
