/********************************************************************************/
/* This software module was developped by										*/

/*	   Marius Preda (MP), marius.preda@int-evry.fr,								*/
/*                        Institut National des Telecommunications, France		*/

/* in the course of development of the MPEG-4 standard (Part 16 AFX).			*/
/* This software module is an implementation of a part of one or				*/
/* more MPEG-4 tools as specified by the										*/
/* MPEG-4 standard. ISO/IEC gives users of the MPEG-4 standard					*/
/* free license to this software module or modifications thereof				*/
/* for use in hardware or software products claiming conformance				*/
/* to the MPEG-4 standard. Those intending to use this software					*/
/* module in hardware or software products are advised that its					*/
/* use may infringe existing patents. The original developer of					*/
/* this software module and his company, the subsequent							*/
/* editors and their companies, and ISO/IEC have no liability					*/
/* for use of this software module or modifications thereof in					*/
/* an implementation. Copyright is not released for non							*/
/* MPEG-4 standard conforming products. INT ARTEMIS								*/
/* retain full right to use the code for their own purposes,					*/
/* assign or donate the code to a third party and to inhibit					*/
/* third parties from using the code for non MPEG-4 standard					*/
/* conforming products. This copyright notice must be included					*/
/* in all copies or derivative works.    										*/
/* Copyright (c)2002 - 2005														*/
/********************************************************************************/

#ifndef _C_MUSCLE_HEADER_
#define _C_MUSCLE_HEADER_

#include<stdio.h>

class CMuscleMask {
public:
	bool *isControlPointOnXChanged, *isControlPointOnYChanged, *isControlPointOnZChanged;
	bool *isWheighOfControlPointChanged;
	bool *isKnotChanged;
	void inline Free(){
			free(isControlPointOnXChanged);
			free(isControlPointOnYChanged);
			free(isControlPointOnZChanged);
			free(isWheighOfControlPointChanged);
			free(isKnotChanged);
	}

};

class CMuscleData{
public:
	int *ControlPointOnX, *ControlPointOnY, *ControlPointOnZ;
	int *WheighOfControlPoint;
	int *Knot;
	void inline Free(){
			free(ControlPointOnX);
			free(ControlPointOnY);
			free(ControlPointOnZ);
			free(WheighOfControlPoint);
			free(Knot);
	}
};

class CMuscle
{
public:

	int muscleIdentifier;
	int NumberControlPoints;
	int NumberKnots;
    bool canBePredicted;

	bool isInCurrentFrame;
	CMuscleMask muscleMask;
	CMuscleData muscleData;

	CMuscleData muscleDataPreviousFrame;
	CMuscleData muscleDataIMax, muscleDataIMin;
	CMuscleData muscleDataPMax ,muscleDataPMin;

	CMuscleData Irange_max, Irange_min;
  	CMuscleData range_max , range_min;
	CMuscleData *muscleDCTData;

	CMuscle *prec;
	CMuscle *suiv;

// operations
	inline bool isControlPointsChanged(){
		for (int i=0;i<NumberControlPoints;i++)
			if (muscleMask.isControlPointOnXChanged[i] ||
				muscleMask.isControlPointOnYChanged[i] ||
				muscleMask.isControlPointOnZChanged[i] )
				return true;
		return false;
	}

	inline bool isWheightOfControlPointsChanged(){
		for (int i=0;i<NumberControlPoints;i++)
			if (muscleMask.isWheighOfControlPointChanged[i])
				return true;
		return false;
	}

	inline bool isKnotChanged(){
		for (int i=0;i<NumberKnots;i++)
			if (muscleMask.isKnotChanged[i])
				return true;
		return false;
	}

	CMuscle(int bID);
	CMuscle();
	~CMuscle();

	void AllocMuscle();
	void AllocMuscle(int ncp, int nk);
	void AllocMinMaxMuscle();
	void AllocPrevMuscle();
	void AllocDCTMuscle();
	void FreeMuscle();
	void FreeMinMaxMuscle();
	void FreePrevMuscle();
	void FreeDCTMuscle();



};


CMuscle *getnodindexi(int index,CMuscle * &first,CMuscle * &last);
CMuscle *getnodIDi(int ID,CMuscle * &first,CMuscle * &last);

void innod(CMuscle *p,CMuscle * &first,CMuscle * &last);
void outnod(CMuscle * p,CMuscle * &first,CMuscle * &last);
void freeliste(CMuscle * &first,CMuscle* &last);
void affiche_liste(CMuscle *first);



#endif
