/********************************************************************************/
/* This software module was developped by										*/

/*	   Marius Preda (MP), marius.preda@int-evry.fr,								*/
/*                        Institut National des Telecommunications, France		*/

/* in the course of development of the MPEG-4 standard (Part 16 AFX).			*/
/* This software module is an implementation of a part of one or				*/
/* more MPEG-4 tools as specified by the										*/
/* MPEG-4 standard. ISO/IEC gives users of the MPEG-4 standard					*/
/* free license to this software module or modifications thereof				*/
/* for use in hardware or software products claiming conformance				*/
/* to the MPEG-4 standard. Those intending to use this software					*/
/* module in hardware or software products are advised that its					*/
/* use may infringe existing patents. The original developer of					*/
/* this software module and his company, the subsequent							*/
/* editors and their companies, and ISO/IEC have no liability					*/
/* for use of this software module or modifications thereof in					*/
/* an implementation. Copyright is not released for non							*/
/* MPEG-4 standard conforming products. INT ARTEMIS								*/
/* retain full right to use the code for their own purposes,					*/
/* assign or donate the code to a third party and to inhibit					*/
/* third parties from using the code for non MPEG-4 standard					*/
/* conforming products. This copyright notice must be included					*/
/* in all copies or derivative works.    										*/
/* Copyright (c)2002 - 2005														*/
/********************************************************************************/
#ifndef _C_BONE_HEADER_
#define _C_BONE_HEADER_

#include<stdio.h>

class CBoneMask {
public:
	bool isTonXChanged, isTonYChanged, isTonZChanged;
	bool isRonAx1Changed, isRonAx2Changed, isRonAx3Changed;
	bool isQuaternion, isRonQxChanged, isRonQyChanged, isRonQzChanged, isRonQwChanged;
	bool isSonXChanged, isSonYChanged, isSonZChanged;
	bool isSOonAxXChanged,isSOonAxYChanged,isSOonAxZChanged, isSOonValChanged; 
	bool isConXChanged, isConYChanged, isConZChanged;
	void inline init(){
		isTonXChanged=isTonYChanged=isTonZChanged=0;
		isRonAx1Changed=isRonAx2Changed=isRonAx3Changed=0;
		isQuaternion=isRonQxChanged=isRonQyChanged=isRonQzChanged=isRonQwChanged=0;
		isSonXChanged=isSonYChanged=isSonZChanged=0;
		isSOonAxXChanged=isSOonAxYChanged=isSOonAxZChanged=isSOonValChanged=0; 
		isConXChanged=isConYChanged=isConZChanged=0;
	}
	void inline operator = (CBoneMask bl){
		isTonXChanged=bl.isTonXChanged;isTonYChanged=bl.isTonYChanged;isTonZChanged=bl.isTonZChanged;
		isRonAx1Changed=bl.isRonAx1Changed;isRonAx2Changed=bl.isRonAx2Changed;isRonAx3Changed=bl.isRonAx3Changed;
		isQuaternion=bl.isQuaternion;isRonQxChanged=bl.isRonQxChanged;
		isRonQyChanged=bl.isRonQyChanged;isRonQzChanged=bl.isRonQzChanged;isRonQwChanged=bl.isRonQwChanged;
		isSonXChanged=bl.isSonXChanged;isSonYChanged=bl.isSonYChanged;isSonZChanged=bl.isSonZChanged;
		isSOonAxXChanged=bl.isSOonAxXChanged;isSOonAxYChanged=bl.isSOonAxYChanged;isSOonAxZChanged=bl.isSOonAxZChanged;isSOonValChanged=bl.isSOonValChanged; 
		isConXChanged=bl.isConXChanged;isConYChanged=bl.isConYChanged;isConZChanged=bl.isConZChanged;
	}

};

class CBoneData{
public:
	int TonX, TonY, TonZ;
	int RonAx1, RonAx2, RonAx3;
	int RonQx, RonQy, RonQz, RonQw;
	int SonX, SonY, SonZ;
	int SOonAxX,SOonAxY,SOonAxZ, SOonVal; 
	int ConX, ConY, ConZ;
	inline init(){
		TonX=TonY=TonZ=0;
		RonAx1=RonAx2=RonAx3=0;
		RonQx=RonQy=RonQz=RonQw=0;
		SonX=SonY=SonZ=0;
		SOonAxX=SOonAxY=SOonAxZ=SOonVal=0; 
		ConX=ConY=ConZ=0;
	}
	inline init(int tx, int ty, int tz, int rax1, int rax2, int rax3, int sx, 
		int sy, int sz, int soAxX, int soAxY, int soAxZ, int soVal,int cx, int cy, int cz){
		TonX=tx,TonY=ty;TonZ=tz;
		RonAx1=rax1;RonAx2=rax2;RonAx3=rax3;
		SonX=sx;SonY=sy;SonZ=sz;
		SOonAxX=soAxX;SOonAxY=soAxY;SOonAxZ=soAxZ;SOonVal=soVal; 
		ConX=cx;ConY=cy;ConZ=cz;
	}
	inline initQ(int tx, int ty, int tz, int qx, int qy, int qz, int qw, int sx, 
		int sy, int sz, int soAxX, int soAxY, int soAxZ, int soVal,int cx, int cy, int cz){
		TonX=tx,TonY=ty;TonZ=tz;
		RonQx=qx;RonQy=qy;RonQz=qz;RonQw=qw;
		SonX=sx;SonY=sy;SonZ=sz;
		SOonAxX=soAxX;SOonAxY=soAxY;SOonAxZ=soAxZ;SOonVal=soVal; 
		ConX=cx;ConY=cy;ConZ=cz;
	}
	inline operator = (CBoneData bd){
		TonX=bd.TonX,TonY=bd.TonY;TonZ=bd.TonZ;
		RonAx1=bd.RonAx1;RonAx2=bd.RonAx2;RonAx3=bd.RonAx3;
		RonQx=bd.RonQx;RonQy=bd.RonQy;RonQz=bd.RonQz;RonQw=bd.RonQw;
		SonX=bd.SonX;SonY=bd.SonY;SonZ=bd.SonZ;
		SOonAxX=bd.SOonAxX;SOonAxY=bd.SOonAxY;SOonAxZ=bd.SOonAxZ;SOonVal=bd.SOonVal; 
		ConX=bd.ConX;ConY=bd.ConY;ConZ=bd.ConZ;
	}
};

class CBone
{
public:

  int boneIdentifier;
  bool isInCurrentFrame;
  bool canBePredicted;
  CBoneMask boneMask;
  CBoneData boneData;
  CBoneData boneDataPreviousFrame;
  CBoneData boneDataIMax, boneDataIMin;
  CBoneData boneDataPMax ,boneDataPMin;
  CBoneData *boneDCTData;

  CBone *prec;
  CBone *suiv;

  inline bool isTranslation(){
		return 	(boneMask.isTonXChanged || boneMask.isTonYChanged || boneMask.isTonZChanged);
	}
	inline bool isRotation(){
		if (!boneMask.isQuaternion) return (boneMask.isRonAx1Changed || boneMask.isRonAx2Changed || boneMask.isRonAx3Changed);
		else return (boneMask.isRonQxChanged ||boneMask.isRonQyChanged ||boneMask.isRonQzChanged ||boneMask.isRonQwChanged); 
	}
	inline bool isScale(){
		return (boneMask.isSonXChanged || boneMask.isSonYChanged || boneMask.isSonZChanged);
	}
	inline bool isScaleOrientation(){
		return (boneMask.isSOonAxXChanged||boneMask.isSOonAxYChanged||boneMask.isSOonAxZChanged||boneMask.isSOonValChanged);
	}
	inline bool isCenter(){
		return (boneMask.isConXChanged || boneMask.isConYChanged || boneMask.isConZChanged);
	}


CBone(int bID);
CBone();
~CBone();
};


CBone *getnodindexi(int index,CBone * &first,CBone * &last);
CBone *getnodIDi(int ID,CBone * &first,CBone * &last);

void innod(CBone *p,CBone * &first,CBone * &last);
void outnod(CBone * p,CBone * &first,CBone * &last);
void freeliste(CBone * &first,CBone* &last);
void affiche_liste(CBone *first);



#endif
