/********************************************************************************/
/* This software module was developped by										*/

/*	   Marius Preda (MP), marius.preda@int-evry.fr,								*/
/*                        Institut National des Telecommunications, France		*/

/* in the course of development of the MPEG-4 standard (Part 16 AFX).			*/
/* This software module is an implementation of a part of one or				*/
/* more MPEG-4 tools as specified by the										*/
/* MPEG-4 standard. ISO/IEC gives users of the MPEG-4 standard					*/
/* free license to this software module or modifications thereof				*/
/* for use in hardware or software products claiming conformance				*/
/* to the MPEG-4 standard. Those intending to use this software					*/
/* module in hardware or software products are advised that its					*/
/* use may infringe existing patents. The original developer of					*/
/* this software module and his company, the subsequent							*/
/* editors and their companies, and ISO/IEC have no liability					*/
/* for use of this software module or modifications thereof in					*/
/* an implementation. Copyright is not released for non							*/
/* MPEG-4 standard conforming products. INT ARTEMIS								*/
/* retain full right to use the code for their own purposes,					*/
/* assign or donate the code to a third party and to inhibit					*/
/* third parties from using the code for non MPEG-4 standard					*/
/* conforming products. This copyright notice must be included					*/
/* in all copies or derivative works.    										*/
/* Copyright (c)2002 - 2005														*/
/********************************************************************************/

// BBAEncoder.h: interface for the BBAEncoder class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_BBAENCODER_H__3ADF6DA3_BA9B_49B6_8266_F62BF27466CD__INCLUDED_)
#define AFX_BBAENCODER_H__3ADF6DA3_BA9B_49B6_8266_F62BF27466CD__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "BBATypeDefs.h"
#include "BBAFileUtil.h"


class BBAEncoder  
{
public:
// data:
	BBAheader bbah;
	sb_object_plane_t bbaop;
	sb_object_plane_t bbaop_prev;
	int coding_type;
// arithmetic related data
	int bba_pred_scale[32]; // for frame based coding // 
	long low, high, opposite_bits, length;
	int  zerocount;
//DCT related data
	float DCTQuantStepsBAP[NUM_BAPS];// added by MP
	int bap_scale[32];		// for DCT coding         // added by MP

// operations

//read from uncoded file//
	bool ReadAFrame(unsigned char *buff);
	bool FreeTheFrame();

	bool ReadABoneMask(unsigned char *buff, CBone *bn, int *ct);
	bool ReadABoneData(unsigned char *buff, CBone *bn, int *ct);

	bool ReadAMuscleMask(unsigned char *buff, CMuscle *ms, int *ct);
	bool ReadAMuscleData(unsigned char *buff, CMuscle *ms, int *ct);
	//int ReadASegment(unsigned char *buff);
	int mygetint(unsigned char *buff, int *count);
// generic funtions
	void init(void);
	int AllocThePlane();
	int FreeThePlane();
	int LoadThePlane();
	int UnLoadThePlane();
	void ConfigureEncoder();
	bool  encode_bba_object_plane_data();
	bool  encode_bba_object_plane_mask();
	bool  encode_bba_object_plane_header();
	bool  encode_bba_object_plane_values();

	void bba_encode_frame_rate();
	void bba_encode_time_code();
	void bba_encode_skip_frames();

// arithmetic encoder related //
	void ArithmeticInit();
	void SetMinMax();
	bool encode_bone_mask(CBone *bone);
	bool encode_muscle_mask(CMuscle *muscle);
	void bba_init_arith(/*int *fap_mask*/void);
	void encode_bba_new_minmax();
	void encode_a_symbol(int index, int cumul_freq[ ]);
	void encode_bba_i_frame();
	void encode_bba_p_frame();
	void encoder_reset();
	int quantize(int bap, int quantStep);
	int dequant(int qfap, int qstep);
	void bit_in_psc_layer(int bit);
	void bit_opp_bits(int bit);
	void encoder_flush();
	bool CheckIsIntraFrame();
	void update_arith_encoder(int symbol, int index);

// DCT encoder related //
	void DCTInit();
	void ReadASegment(unsigned char *buff);
	void AllocThePlaneSegment();
	void LoadThePlaneSegment();
	void UnLoadThePlaneSegment();
	void FreeThePlaneSegment();

	void bba_init_htables();
	void encode_bba_i_segment();
	void encode_bba_p_segment();
	void bba_dctTrans(int *src, float *dest);
	void bba_dctquant(float *src, int *dest, int q);
	void bba_setDC(int DCCoef);
	void bba_setAC(int *ACCoef);
	void HuffmanCoding(int **table,int symbol);
	void bba_setPDC(int dc_Q);
	/*
	void bap_DCT_initialize();
	void bap_init_dctQP();
	void bap_init_dct();
	void bap_write1Segment(int curFrame, FILE *bapfile);
	bool CheckIsIntraSegment();
//	void GroupBapMask();
*/
// write in stream
	void PutBits (int nobits, int value);
	void ShortPutBits (short symbol);
	void PutBBAStartCode();
	void PutBBAPlaneStartCode();
	void PutBBAEndCode();
	void NextStartCode();
	void saveInFile(char *fname);

	//void bap_dctdequant(int *src, float *dest, int index);

	BBAEncoder();
	virtual ~BBAEncoder();

	bool ReadAMorphMask(unsigned char *buff, CMorph *ms, int *ct);
	bool ReadAMorphData(unsigned char *buff, CMorph *ms, int *ct);
	bool encode_morph_mask(CMorph *ms);

/*
	int vsptrind,vbdptrind,expvals;/*constructed array indices and expvals*/
/*	vbdptr **vsptr;/*8 bit vsptr array index is viseme_1(4 msbs)and viseme_2(4 lsbs)*/
/*	explink *curexplinkptr,*lastexplinkptr,*newexplinkptr;
*/
};

#endif // !defined(AFX_BBAENCODER_H__3ADF6DA3_BA9B_49B6_8266_F62BF27466CD__INCLUDED_)
