/***********************************************************************
This software module was originally developed by Alexander Zhirkov (
Moscow State University), Joo Yeon Han (Samsung Advance Institute of Technology)
in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)
standard. This software module is an implementation of a part of one or
more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4
Systems (ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 
Systems (ISO/IEC 14496-1) free license to this software module or 
modifications thereof for use in hardware or software products claiming
conformance to the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to 
use this software module in hardware or software products are advised that
its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Samsung Advance Institute of Technology
retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using
the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
This copyright notice must be included in all copies or derivative works.
Copyright ?2001
************************************************************************

For more information or to receive an up to date version of this module,
you may call Joo Yeon Han at jyhan@sait.samsung.co.kr
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Depth Image-based Representation
FILE:       Array.h
AUTHOR:     Joo Yeon Han (SAMSUNG AIT)
OVERVIEW:   Implementation of Depth Image-based Representation
-------------------------------------------------------------------------*/
//---------------------------------------------------------------------------
#ifndef ArrrayH
#define ArrrayH
//---------------------------------------------------------------------------

#include <assert.h>
#include <math.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#include "Common.h"

template <class T> class Array
{
protected:
	T *__D;
    int __A,__I;
    bool __S; // can automate increasing
public:
	Array(int a){__I=0;__D=0;__A=-1; __S=true; Alloc(a);};
	Array(){__I=0;__D=0; __A=-1; __S=true; Alloc(1); };
	void FromPointer(T *g,int a){
    	Assert(g,"<Array::FromPointer> null pointer");
    	Assert(a>=0,"<Array::FromPointer> bad pointer size="+a);
    	AllocSet(a);
        for(int i=0;i<a;i++) __D[i]=g[i];
    }
	T* Pointer(){
        return &__D[0];
    }
    void AutoSize(bool s){ __S=s;};
    ~Array(){delete []__D;};
    Array<T> This(){return *this;}
    void Copy(Array<T> &g){
		Alloc(g.Size());
        for(int i=0;i<__A;i++) Elem(i)=g[i];
        SetPoint(g.All());
	}
    int All(){return __I;}
    int Size(){return __A;}
    void Reset(){__I=0;};
	void Minimize(){__I=0; Alloc(1);}

    void SetPoint(int i){
    	Assert(i>=0 && i<=__A,"<Array::SetPoint> bad point i="+i);
        __I=i;
    }
    void AllocSet(int a){
    	Alloc(a);
        SetPoint(a);
    }
    void AllocSetIfNeed(int a){
    	AllocIfNeed(a);
        SetPoint(a);
    }
    void AllocIfNeed(int a){
     	Assert(a>0,"<Array::AllocIfNeed> bad allocation size= "+a);
        if(a<=__A) return;
        Alloc(a);
    }
    void Alloc(int a){
     	Assert(a>=0,"<Array::Alloc> bad allocation size= "+a);
        if(a==__A) return;
    	if(__A>0) delete []__D;
        __A=a;
        __D=new T[__A];
     	Exit(__D,"<Array::Alloc> allocation error");
    }
    int Add(T e){
    	if(__I<__A)	__D[__I]=e;
        else 	ReAlloc(__A*2+1),__D[__I]=e;
        __I++;
        return __I-1;
    }
    int Add(){
    	if(__I>=__A)  ReAlloc(__A*2+1);
        __I++;
        return __I-1;
    }
    int MultiAdd(int n){
		Assert(n>=0,"<MultiAdd> negative number of elemetns to add");
    	if(__I+n>__A)  ReAlloc(mx(__A*2,__I+n));
        __I+=n;
        return __I-1;
    }
	void Beg(){__I=-1;}
	int Next(){
        __I++;
        Assert(__I>=0,
        	"<Array::operator[]> bad array index "+__I);
    	if(__I>=__A) return 0;
	    return 1;}
    T& Cur(){
    	return Elem(__I);}
    T& CurNext(){
        int q=Next();
        Assert(q,"<Array::CurNext> overflow map");
		return Elem(__I-1);
    }
    T& Last(){
    	return Elem(__I-1);}
	void DeleteElem(int n){
    	AssertIndexText(n,__A,"<Array::DeleteElem>");
        int a=__A-1;
    	T *d=new T[a];
        Exit(d,"<Array::ReAlloc> can't allocate swap data");
        int i=0;
        for(;i<n;i++)
        	d[i]=__D[i];
        for(;i<a;i++)
        	d[i]=__D[i+1];
        Alloc(a);
        for(i=0;i<__A;i++)
        	__D[i]=d[i];
        delete []d;
    }
	void SetAll(int v){
        for(int i=0;i<__A;i++) __D[i]=v;
    }
    void ReAlloc(int a){
       	Exit(__S,"Array overflow !");
     	Assert(a>=0,"<Array::ReAlloc> bad allocation size= "+a);
        if(a<__A) return;
    	T *d=new T[a];
        Exit(d,"<Array::ReAlloc> can't allocate swap data");
        int i;
        for(i=0;i<__A;i++) d[i]=__D[i];
        Alloc(a);
        for(i=0;i<__A;i++) __D[i]=d[i];
        delete []d;
    }
    T& Elem(int i){
    	Assert(__D,"<Array::operator[]> Array not initialized");
        Assert(i>=0 && i<__A,"<Array::operator[]> Array index "+i+" not in range 0.."+(__A-1));
	    return __D[i];
    }
    T& ElemAdd(int i){
     	Assert(i>=0,"<Array::AddElem> i<0");
        if(i>=__A) ReAlloc(i+1);
        return __D[i];
    }
    T& operator[](int i){
    	return Elem(i);
	}
};
//---------------------- ONE DIMENSIONAL ARRAYS --------------------------------
struct ArrayInt1D:
	public Array<int>
{
	void ClearInt1D(int c){
		iforinc(i,All()) Elem(i)=c;}
};
struct ArrayFloat1D:
	public Array<flt>
{
	void ClearFloat1D(flt c){
		iforinc(i,All()) Elem(i)=c;}
};
struct ArrayDouble1D:
	public Array<dbl>
{
	void ClearDouble1D(dbl c){
		iforinc(i,All()) Elem(i)=c;}
};
//---------------------------------------------------------------------------
#endif