/*\   Programmed by Alexander Zhirkov <azh@graphics.cs.msu.su>
\*/

//---------------------------------------------------------------------------

#ifdef SAIT_CONFORMANCE_2003

#include "BinaryArithmetic.h"

#include "Entropy.h"

#include <math.h>
#include <assert.h>
#include <malloc.h>

//-------------------------------------------------------------------------
//--------  ADDITIONAL FUNCTIONS ------------------------------------------
//-------------------------------------------------------------------------

#define ALLOC(obj,typ,bts) obj=(((typ*)malloc((bts)*sizeof(typ))))
#define FREE(obj) {free(obj);}

typedef EntropyProbabilisticContext Prb;


#define Assert(st,txt) assert(st);

//-------------------------------------------------------------------------
//--------  PROBABILISTIC TREE CLASS FUNCTIONS ----------------------------
//-------------------------------------------------------------------------
/*
int lgd2(int x)
{
	if(!x) return -1;
	if(x==1) return 0;
	return (double)1.000000001+log(x-1)/log(2);
}

int ProbabilisticTreeRecurs(Prb* p,int x,int n)
{
    int x1;
	p->Tree[x]=0;
	if(n<0) return 0;
	if(!n)
        return x+1;
    if(n==2)
    {
        int *g=&(p->Tree[x]);

        g[1]=x+6;
        g[2]=g[4]=g[5]=g[6]=g[8]=g[9]=0;
        g[3]=x+5;
        g[7]=x+9;

        return x+10;
    }
    if(n==1)
    {
        p->Tree[x]=0;
        if(!n)
            return x+1;
        p->Tree[x+2]=0;
        p->Tree[x+1]=x+3;
        p->Tree[x+3]=0;
        return x+4;
    }
    x1=ProbabilisticTreeRecurs(p,x+2,n-1);
    p->Tree[x+1]=x1;
    return ProbabilisticTreeRecurs(p,x1,n-1);
}

void ProbabilisticInit(Prb* p)
{
    int n;
    assert(p && p->AllSym>=0);
	n=lgd2(p->AllSym);
    p->N=n;

	if(!(p->Prob)) ALLOC(p->Tree,int,(1<<(n+1))+5);

	p->Tree[0]=0;
    p->AllPast=0;

	if(!(p->Prob)) ALLOC(p->Prob,int,p->AllSym);

	ProbabilisticTreeRecurs(p,1,n-1);
    p->T=0;
	p->VS=p->AllSym;
	p->EDL=-1;
	p->TVS=0;
}


void Entropy_ProbabilisticRelease(Prb* p)
{
	FREE(p->Tree);
	FREE(p->Prob);
}

void ProbabilisticTreeAdd(Prb* p,int ind,int av)
{
    int *t=p->Tree;
    int i,j=1;
    assert(p && p->Tree);

    t[0]+=av;
		
	if(p->AllSym==256)
	{
        if(!(ind&(1<<7))) 
		{
			t[1]+=av,j+=2;

	        if(!(ind&(1<<6))) 
			{
				t[3]+=av,j+=2;
				if(!(ind&(1<<5)))
				{
					t[5]+=av,j+=2;
			        if(!(ind&(1<<4))) t[7]+=av,j=9;
					else              j=31;
				}
				else
				{              
					if(!(ind&(1<<4))) t[53]+=av,j=55;
					else              j=77;
				}
			}
			else
			{
				j=99;
				if(!(ind&(1<<5))) 
				{
					t[99]+=av;
			        if(!(ind&(1<<4))) t[101]+=av,j=103;
					else              j=125;
				}
				else
				{              
					j=147;
			        if(!(ind&(1<<4))) t[147]+=av,j=149;
					else              j=171;
				}
			}
		}
        else
		{              
			j=193;
			if(!(ind&(1<<6)))
			{
				t[193]+=av,j+=2;
				if(!(ind&(1<<5)))
				{ 
					t[195]+=av,j+=2;
			        if(!(ind&(1<<4))) t[j]+=av,j+=2;
					else              j=t[j+1];
				}
				else
				{
					j=t[j+1];
			        if(!(ind&(1<<4))) t[j]+=av,j+=2;
					else              j=t[j+1];
				}
			}
			else
			{              
				j=t[j+1];
				if(!(ind&(1<<5))) 
				{
					t[j]+=av,j+=2;
			        if(!(ind&(1<<4))) t[j]+=av,j+=2;
					else              j=t[j+1];
				}
				else
				{
					j=t[j+1];
			        if(!(ind&(1<<4))) t[j]+=av,j+=2;
					else              j=t[j+1];
				}
			}
		}

        if(!(ind&(1<<3))) t[j]+=av,j+=2;
        else              j=t[j+1];

        if(!(ind&(1<<2))) t[j]+=av,j+=2;
        else              j=t[j+1];

        if(!(ind&(1<<1))) t[j]+=av,j+=2;
        else              j=t[j+1];

        if(!(ind&(1<<0))) t[j]+=av,j+=2;
        else              j=t[j+1];

		return;
	}
    fordec(i,p->N)
        if(!(ind&(1<<i))) t[j]+=av,j+=2;
        else              j=t[j+1];
}

void ProbabilisticTreeBuild(Prb* p)
{
    int *prb=p->Prob,a=p->AllSym;
    int i,aa=1<<(p->N);
    assert(p && p->Tree);
	p->Tree[0]=0;
	p->AllPast=0;
	ProbabilisticTreeRecurs(p,1,p->N-1);
    p->T=0; 
	forinc(i,aa)
		ProbabilisticTreeAdd(p,i,i<a?prb[i]:0);
}

void ProbabilisticTreeFromValue(Prb *p,unt v,int *_ind,int *_rng)
{
	int *t=p->Tree;
    int i,j=1,n=p->N,ind=0,rng=0;
    Assert(p && p->Tree && v<p->Tree[0],"<ProbabilisticTreeFromValue> v="+v+"<?<"+p->Tree[0]);
	assert(p && p->Tree && v<p->Tree[0]);
    fordec(i,n)
	{
        if(v<t[j]) j+=2;
        else       v-=t[j],rng+=t[j],j=t[j+1],ind|=1<<i;
    }
    *_ind=ind,*_rng=rng;
}

void ProbabilisticTreeFromIndex(Prb* p,int ind,int *_rng)
{
    int *t=p->Tree;
    int i,j=1,n=p->N,rng=0;
    assert(p && p->Tree && ind<p->AllSym);
    fordec(i,n)
    {
        if(!(ind&(1<<i))) j+=2;
        else              rng+=t[j],j=t[j+1];
    }
    *_rng=rng;
}

void  ProbabilisticUpdate(Prb* p,int j)
{
    p->AllPast++;
	if((p->MaxCumul>0 && p->Tree[0]>p->MaxCumul) || (p->MaxCumul<=0 && !p->T && p->AN>0 && p->K>0))
	{
		int *prb=p->Prob,an=p->AN,a=p->AllSym;
		int i,sm=0;

		forinc(i,a)
		{
			prb[i]=(prb[i]+1)>>1;
			sm+=prb[i];
			Assert(prb[i]>0,"<ProbabilisticUpdate>");
		}

		Assert(sm<(1<<16),"<ProbabilisticUpdate> summ to large");

		ProbabilisticTreeBuild(p);
	}
    if(p->K)
    {
        p->Prob[j]+=p->K;
        ProbabilisticTreeAdd(p,j,p->K);
		p->T++;
        p->TVS++;
    }
    if(p->T==p->DL) p->T=0;
    if(p->TVS==p->EDL) p->TVS=0;
}
*/
//-------------------------------------------------------------------------
//--------  ARITHMETIC CODING ---------------------------------------------
//-------------------------------------------------------------------------

inline void BitStreamToBuffer(BinaryArithmetic *rc,uhr ch) {
	rc->strm->write_bits(8,ch); 
}

inline uhr BitStreamFromBuffer(BinaryArithmetic *rc){
	return rc->strm->read_bits(8);
}

#define Q1 16384
#define Q2 32768
#define Q3 49152
#define TOP 65535
#define ZEROMAX 22
#define CODEVALNBBITS 16

void write_bit(BinaryArithmetic *rc,int bit)
{
	rc->strm->write_bit(bit);
}

void flush_bits(BinaryArithmetic *rc)
{
	rc->strm->flush_bits();
}

int read_bit(BinaryArithmetic *rc)
{
	return rc->strm->read_bit();
}

void BinaryArithmeticStartEncode(BinaryArithmetic *rc)
{
	rc->low=0,
	rc->high=TOP,
	rc->length=0,
    rc->zerocount=0,
	rc->m_opposite_bits=0,

    rc->m_nbBits=0,
	rc->m_bitcount=0;
}

int BinaryArithmeticFinishEncode(BinaryArithmetic *rc)
{
	rc->flush();
	return rc->strm->pos;
}

void BinaryArithmeticStartDecode(BinaryArithmetic *rc)
{
    rc->d_flush();
    rc->d_reset();
}

void BinaryArithmeticFinishDecode()
{
}

void BinaryArithmeticEncode(BinaryArithmetic *rc,unt p0, unt pcur, unt pall)
{
	rc->length=rc->high-rc->low+1;
	rc->high = rc->low - 1 + ( rc->length * (pall-p0)) / pall;
	rc->low  += (rc->length * (pall-p0-pcur)) / pall;

	rc->m_bitcount=0;
	for (;;)
	{
		if (rc->high<Q2) rc->bit_opp_bits(false);
		else if (rc->low>=Q2)
			{
				rc->bit_opp_bits(true);
				rc->low -= Q2;
				rc->high -= Q2;
			}
			else
				if ( rc->low>= Q1 && rc->high < Q3 )
				{
					rc->m_opposite_bits++;
					rc->low -= Q1;
					rc->high -= Q1;
				}
				else break;

		rc->low *= 2;
		rc->high = 2 * rc->high +1;
	}
}

void BinaryArithmeticDecode(BinaryArithmetic *rc,unt p0, unt pcur, unt pall)
{
    rc->d_high= rc->d_low - 1 + (rc->d_length * (pall-p0)) / pall;
    rc->d_low+=            (rc->d_length * (pall-p0-pcur)) / pall;
    rc->d_bitcount=0;

    for ( ; ; )
    {
        if (rc->d_high < Q2)
            rc->d_opposite_bits=0;
        else if (rc->d_low >= Q2) {
            rc->d_code_value -= Q2;
            rc->d_low -= Q2;
            rc->d_high -= Q2;
            rc->d_opposite_bits=0;}
        else if (rc->d_low >= Q1 && rc->d_high < Q3) {
            rc->d_code_value -= Q1;
            rc->d_low -= Q1;
            rc->d_high -= Q1;
            rc->d_opposite_bits++; }
        else break;

        rc->d_low *= 2;
        rc->d_high = 2*rc->d_high + 1;
        if(!rc->bit_out_psc_layer())
            Assert(0,"Decompression Error!");
        rc->d_code_value = (2*rc->d_code_value + (rc->d_Bit ? 1 : 0)) ;//&TOP;
        rc->d_bits_used++;
    }
}

unt BinaryArithmeticValue(BinaryArithmetic *rc,unt pall)
{
    rc->d_length = rc->d_high - rc->d_low + 1;
    unt cum = ((unt)-1 + (rc->d_code_value - rc->d_low + 1) * pall) / rc->d_length;
    cum= pall-cum-1;
    return cum;
}

void BinaryArithmetic::addBit(int bit)
{
	m_lastvalue<<=1;
	m_lastnbbits++;
	if(bit) m_lastvalue|=1;
	else    m_lastvalue|=0;
    write_bit(this,bit);
}
//int g_zero1w=0;
void BinaryArithmetic::bit_in_psc_layer(int bit)
{
#ifdef ZEROCODES
	if(zerocount == ZEROMAX )
	{
		addBit(true);
//		g_zero1w++;
		m_bitcount++;
		zerocount=0;
	}
#endif
	addBit(bit);
	m_bitcount++;
	zerocount=bit?0:zerocount+1;
}

void BinaryArithmetic::bit_opp_bits(int bit)
{
	bit_in_psc_layer(bit);
	while(m_opposite_bits> 0)
	{
		bit_in_psc_layer(!bit);
		m_opposite_bits--;
	}
}

int BinaryArithmetic::reset()
{
	low=0;
	high=TOP;
	zerocount=0;
	m_bitcount=0;
	m_opposite_bits=0;
	return true;
}
int BinaryArithmetic::flush()
{
	m_bitcount=0;
	m_opposite_bits++;
	int retValue=1+m_opposite_bits;
	if (low<Q1) bit_opp_bits(false);
	else bit_opp_bits(true);
	reset();
    flush_bits(this);
	return retValue;
}

void BinaryArithmetic::d_flush()
{
    d_bits_used=0;
    d_bitcount=0;
    d_code_value = 0;
    d_low = 0;
    d_high = TOP;
    d_zerocount = 0;
    d_bitcount=0;
    d_opposite_bits=0;
}

int BinaryArithmetic::d_reset()
{
    int i;
    d_code_value = 0;
    d_low = 0;
    d_high = TOP;
    d_zerocount = 0;
    d_bitcount=0;
    d_opposite_bits=0;
    for (i = 1;   i <= CODEVALNBBITS;   i++)
    {
        if(!bit_out_psc_layer())
            return false;
        d_code_value = 2 *d_code_value + (d_Bit ? 1 : 0);
    }
    return true;
}

int BinaryArithmetic::bit_out_psc_layer()
{
	int pinBit;
	pinBit=strm->read_bit();
	
#ifdef ZEROCODES
	if(d_zerocount == ZEROMAX)
    {
		d_bitcount++;
		if(!pinBit)
		{
			Assert(0,"MARKER BIT error in arithmetic decoding");
			return false;
		}
		pinBit=strm->read_bit();
		d_zerocount = 0;
	}
#endif
	d_Bit = pinBit;
	d_bitcount++;
	
	if(!d_Bit) d_zerocount++;
	else d_zerocount = 0;
	return true;
}

//-------------------------------------------------------------------------
//--------  INTERFACE FUNCTIONS -------------------------------------------
//-------------------------------------------------------------------------

void Entropy_BinaryArithmeticEncodeInit(
 BinaryArithmetic* rc,
 BStream *strm_)
{
	rc->strm=strm_;
    rc->BufLen=strm_->BufLen;
    BinaryArithmeticStartEncode(rc);
}

int Entropy_BinaryArithmeticEncodeClose(BinaryArithmetic* rc)
{
	return BinaryArithmeticFinishEncode(rc);
}

void Entropy_BinaryArithmeticDecodeInit(BinaryArithmetic* rc,BStream *strm_)
{
	rc->strm=strm_;
	BinaryArithmeticStartDecode(rc);
}

void Entropy_BinaryArithmeticDecodeClose(BinaryArithmetic* rc)
{
}

#endif