/*\   Programmed by Alexander Zhirkov <azh@graphics.cs.msu.su>
\*/
#ifdef SAIT_CONFORMANCE_2003

#include "Common.h"

#include <assert.h>
#include <math.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <time.h>
//-------------------------------------------------------------------------

dbl accuracy(dbl d,int n)
{
	dbl sign=1;
	if(d<0) d=-d,sign=-1;
	if(!d) return 0;
	n-=log(d)/log(10);
	dbl q=pow(10,n);
	if(!q)
		return -1;
	return sign*int(pow(10,n)*d)/q;
}

int nearest(dbl d,int i1,int i2)
{
    int i;
    d+=0.5,i=d;
    if(i<i1) i=i1;
    if(i>i2) i=i2;
    return i;
}

int dbl2int(dbl z)
{
	return ((dbl)z<0.?-(-z+.5):(.5+z));
}

int int2log(dbl i,int n,int n1)
{
	if(!i) return 0;
    dbl a=(int)1<<n;
    dbl b=(int)1<<n1;
	return (dbl)b*log2(1.+fabs(i)*a);
}
int log2int(int l,int n,int n1)
{
	if(!l) return 0;
    dbl a=(int)1<<n;
    dbl b=(int)1<<n1;
	return (dbl)(pow(2,(dbl)l/b)-1.)/a;
}

//-------------------------------------------------------------------------
dbl fmx(dbl a,dbl b){
	return a>b?a:b;}
dbl fmn(dbl a,dbl b){
	return a<b?a:b;}
dbl fmx(dbl a,dbl b,dbl c){
	return fmx(a,fmx(b,c));}
dbl fmn(dbl a,dbl b,dbl c){
	return fmn(a,fmn(b,c));}
#ifdef __BORLANDC__
//dbl fabs(double d){
//	return d<0?-d:d;}
#endif
dbl sqrtsqr(dbl a,dbl b)
{
  return sqrt(a*a+b*b);
}
dbl fsqr(dbl a)
{
  return a*a;
}

int sqr(int i)
{
	return i*i;
}

dbl log2(dbl n)
{
    n=fabs(n)+1.;
    return log(n)*1.442695041;  //1/ln2
}

//-------------------------------------------------------------------------

int lg2(int i)
{
	if(!i) return -1;
    if(i<0) i=-i;
    int r=-1;
    if(i&0xFFFF0000)
      r+=16,i>>=16;
    if(i&0x0000FF00)
      r+=8,i>>=8;
    if(i&0x000000F0)
      r+=4,i>>=4;
    if(i&0x0000000C)
      r+=2,i>>=2;
    if(i&0x00000002)
      r+=1,i>>=1;
    return r+i;
}

int slg2(int i)
{
    if(!i)
		return 0;
    return 1<<lg2(i);
}

int round(dbl x)
{
    if(x>0.) return((int)(x+0.500));
	return((int) (x-0.500));
}

int quantdiv(dbl x,int q)
{
	return q?round((dbl)x/q):x;
}
int iquantdiv(int x,int q)
{
    return q?(x>0?((x<<1)+q)/(q<<1):((x<<1)-q)/(q<<1)):x;
}

int lg2roundup(int i)
{
	Assert(i>0,"<lg2roundup> bad number="+i);
    return lg2(i-1)+1;
}

int setb(int n)
{
	if(!n)
      return 0;
    return 1<<(n-1);
}

//-------------------------------------------------------------------------

int mx(int x,int y)
{
	return x>y?x:y;
}

int mn(int x,int y)
{
	return x<y?x:y;
}

//-------------------------------------------------------------------------

void swap(dbl &x,dbl &y)
{
	dbl q=x;x=y;y=q;
}

void swap(int &x,int &y)
{
	int q=x;x=y;y=q;
}

int sorti(int& x,int& y)
{
	if(x<y)
		return 0;
	int q=x;x=y;y=q;
	return 1;
}

//-------------------------------------------------------------------------

int range(int x,int x1,int x2)
{
    x=x<x1?x1:x;
    return x>x2?x2:x;
}
int rangebyte(int x)
{
    if(x<0) return 0;
    return x>255?255:x;
}

dbl frange(dbl x,dbl x1,dbl x2)
{
    x=x<x1?x1:x;
    return x>x2?x2:x;
}

int isrange(int x,int x1,int x2)
{
    int q=x;
    x=x<x1?x1:x;
    x=x>x2?x2:x;
    return x==q?1:0;
}

int ishiftl(int i,int s)
{
	if(!s) return i;
    if(s>0) return i<<s;
	s=-s;
    return i>>s;
}


//-------------------------------------------------------------------------

dbl hammingwindow(dbl x) // x=0..1
{
	x=x-.5;
	x=(0.54-0.46*cos(2.*3.1415*x));
    x=exp(-x*x);
	x=(x-1.000)*1.58+1.000;
	return x;
}

//-------------------------------------------------------------------------


dbl MatrixDeterminant(dbl** a,int n)
{
	int i,j,j1,j2;
	double det = 0;
	double **m = NULL;

	Assert(a,"<Determinant> Null pointer");

	if(n < 1)
		return 0;
	else
	if(n == 1)
	  det = a[0][0];
	else
	if(n == 2)
	  det = a[0][0] * a[1][1] - a[1][0] * a[0][1];
	else
	{
		det = 0;
		for (j1=0;j1<n;j1++)
		{
			m = (dbl**) malloc((n-1)*sizeof(double*));
			for (i=0;i<n-1;i++)
				m[i] = (dbl*)malloc((n-1)*sizeof(double));
			for (i=1;i<n;i++)
			{
				j2 = 0;
				for (j=0;j<n;j++)
				{
					if (j == j1)
						continue;
					m[i-1][j2] = a[i][j];
					j2++;
				}
			}
			det += pow(-1.0,j1+2.0) * a[0][j1] * MatrixDeterminant(m,n-1);
			for (i=0;i<n-1;i++)
				free(m[i]);
			free(m);
		}
	}
	return det;
}

//-------------------------------------------------------------------------
//-------------------------------------------------------------------------

//-------------------------------------------------------------------------

void memsetdword(int* p,int s,int l)
{
	if(!l) return;
    if(l<0) p+=l,l=-l;
	__asm
    {
        mov edi,p
		mov ecx,l
		mov eax,s
		cld
		rep stosd
    }
}
void memcpydword(int* dst,int *src,int l)
{
	if(!l) return;
	Assert(l>0,"<memcpydword> len="+l);
	__asm
    {
        push esi
		push edi
		mov esi,src
		mov edi,dst
		mov ecx,l
		cld
		rep movsd
		pop edi
		pop esi
    }
}

//-------------------------------------------------------------------------

void fputi(FILE* f,int l)
{
	fwrite(&l,4,1,f);
}
int fgeti(FILE* f)
{
 	int l;
	fread(&l,4,1,f);
    return l;
}
uhr f_byte(FILE *f){
    uhr c=getc(f);
	return c;}
void f_byte(FILE *f,uhr c){
    putc(c,f);}
urd f_word(FILE *f){
	urd q;
	fread(&q,2,1,f);
    return q;}
void f_word(FILE *f,urd q){
	fwrite(&q,2,1,f);}
unt f_dword(FILE *f){
	unt u;
    fread(&u,4,1,f);
    return u;}
void f_dword(FILE *f,unt u){
    fwrite(&u,4,1,f);}
flt f_float(FILE *f){
	float q;
    fread(&q,4,1,f);
    return q;}
void f_float(FILE *f,flt q){
    fwrite(&q,4,1,f);}
dbl f_dbl(FILE *f){
	dbl q;
    fread(&q,sizeof(dbl),1,f);
    return q;}
void f_dbl(FILE *f,dbl q){
    fwrite(&q,sizeof(dbl),1,f);}

void  f_str255r(FILE* f,chr* s){
	int l=f_byte(f);
	fread(s,1,l,f);
    s[l]=0;}
void  f_str255w(FILE* f,chr* s){
	int l=strlen(s);
    f_byte(f,l);
	fwrite(s,1,l,f);}

void f_line255r(FILE *f,chr *s)
{
    fgets(s,254,f);
    s[mx(strlen(s)-1,0)]=0;
}

void f_skip(FILE *f,int i){
	while(i--)
    	fgetc(f);}

int f_size(FILE *f)
{
	fseek(f,0,SEEK_END);
	int l = ftell(f);
	fseek(f,0,SEEK_SET);
    return l;
}

int f_exist(char* t)
{
	FILE *f=fopen(t,"rb");
	if(!f) return 0;
	fclose(f);
	return 1;
}

int f_isequal(chr* t1,chr* t2)
{
	int s=0,l1,l2;
	FILE *f1=fopen(t1,"rb");
	FILE *f2=fopen(t2,"rb");
	if(f1 && f2)
	{
		l1=f_size(f1);
		l2=f_size(f2);
		if(l1==l2)
		{
            s=1;
			iforinc(i,l1)
			if(f_byte(f1)!=f_byte(f2)){
				s=0;
				break;}
		}
	}
	fclose(f1);
	fclose(f2);
	return s;

}

#endif
//-------------------------------------------------------------------------
