/*\   Programmed by Alexander Zhirkov <azh@graphics.cs.msu.su>
\*/

//---------------------------------------------------------------------------
#ifdef SAIT_CONFORMANCE_2003

#include "Entropy.h"

#include <math.h>
#include <assert.h>
#include <malloc.h>

//-------------------------------------------------------------------------
//--------  ADDITIONAL FUNCTIONS ------------------------------------------
//-------------------------------------------------------------------------

#define ZEROCODES

#define ALLOC(obj,typ,bts) obj=(((typ*)malloc((bts)*sizeof(typ))))
#define FREE(obj) {free(obj);}

typedef EntropyProbabilisticContext Prb;

#define Assert(st,txt) assert(st);

#define forinc(i,n) for(i=0;i<(n);i++)
#define fordec(i,n) for(i=(n)-1;i>=0;i--)

//-------------------------------------------------------------------------
//--------  PROBABILISTIC TREE CLASS FUNCTIONS ----------------------------
//-------------------------------------------------------------------------

int lgd2(int x)
{
	if(!x) return -1;
	if(x==1) return 0;
	return (double)1.000000001+log(x-1)/log(2);
}

int ProbabilisticTreeRecurs(Prb* p,int x,int n)
{
    int x1;
	p->Tree[x]=0;
	if(n<0) return 0;
	if(!n)
        return x+1;
    if(n==2)
    {
        int *g=&(p->Tree[x]);

        g[1]=x+6;
        g[2]=g[4]=g[5]=g[6]=g[8]=g[9]=0;
        g[3]=x+5;
        g[7]=x+9;

        return x+10;
    }
    if(n==1)
    {
        p->Tree[x]=0;
        if(!n)
            return x+1;
        p->Tree[x+2]=0;
        p->Tree[x+1]=x+3;
        p->Tree[x+3]=0;
        return x+4;
    }
    x1=ProbabilisticTreeRecurs(p,x+2,n-1);
    p->Tree[x+1]=x1;
    return ProbabilisticTreeRecurs(p,x1,n-1);
}

void ProbabilisticInit(Prb* p)
{
    int n;
    assert(p && p->AllSym>=0);
	n=lgd2(p->AllSym);
    p->N=n;

	if(!(p->Prob)) ALLOC(p->Tree,int,(1<<(n+1))+5);

	p->Tree[0]=0;
    p->AllPast=0;

	if(!(p->Prob)) ALLOC(p->Prob,int,p->AllSym);

	ProbabilisticTreeRecurs(p,1,n-1);
    p->T=0;
	p->VS=p->AllSym;
	p->EDL=-1;
	p->TVS=0;
}


void Entropy_ProbabilisticRelease(Prb* p)
{
	FREE(p->Tree);
	FREE(p->Prob);
}

#include "conio.h"

void ProbabilisticTreeAdd(Prb* p,int ind,int av)
{
    int *t=p->Tree;
    int i,j=1;
    assert(p && p->Tree);

    t[0]+=av;
		
	if(p->AllSym==256)
	{
        if(!(ind&(1<<7))) 
		{
			t[1]+=av,j+=2;

	        if(!(ind&(1<<6))) 
			{
				t[3]+=av,j+=2;
				if(!(ind&(1<<5)))
				{
					t[5]+=av,j+=2;
			        if(!(ind&(1<<4))) t[7]+=av,j=9;
					else              j=31;
				}
				else
				{              
					if(!(ind&(1<<4))) t[53]+=av,j=55;
					else              j=77;
				}
			}
			else
			{
				j=99;
				if(!(ind&(1<<5))) 
				{
					t[99]+=av;
			        if(!(ind&(1<<4))) t[101]+=av,j=103;
					else              j=125;
				}
				else
				{              
					j=147;
			        if(!(ind&(1<<4))) t[147]+=av,j=149;
					else              j=171;
				}
			}
		}
        else
		{              
			j=193;
			if(!(ind&(1<<6)))
			{
				t[193]+=av,j+=2;
				if(!(ind&(1<<5)))
				{ 
					t[195]+=av,j+=2;
			        if(!(ind&(1<<4))) t[j]+=av,j+=2;
					else              j=t[j+1];
				}
				else
				{
					j=t[j+1];
			        if(!(ind&(1<<4))) t[j]+=av,j+=2;
					else              j=t[j+1];
				}
			}
			else
			{              
				j=t[j+1];
				if(!(ind&(1<<5))) 
				{
					t[j]+=av,j+=2;
			        if(!(ind&(1<<4))) t[j]+=av,j+=2;
					else              j=t[j+1];
				}
				else
				{
					j=t[j+1];
			        if(!(ind&(1<<4))) t[j]+=av,j+=2;
					else              j=t[j+1];
				}
			}
		}

        if(!(ind&(1<<3))) t[j]+=av,j+=2;
        else              j=t[j+1];

        if(!(ind&(1<<2))) t[j]+=av,j+=2;
        else              j=t[j+1];

        if(!(ind&(1<<1))) t[j]+=av,j+=2;
        else              j=t[j+1];

        if(!(ind&(1<<0))) t[j]+=av,j+=2;
        else              j=t[j+1];

		return;
	}
    fordec(i,p->N)
        if(!(ind&(1<<i))) t[j]+=av,j+=2;
        else              j=t[j+1];
}

void ProbabilisticTreeBuild(Prb* p)
{
    int *prb=p->Prob,a=p->AllSym;
    int i,aa=1<<(p->N);
    assert(p && p->Tree);
	p->Tree[0]=0;
	p->AllPast=0;
	ProbabilisticTreeRecurs(p,1,p->N-1);
    p->T=0; 
	forinc(i,aa)
		ProbabilisticTreeAdd(p,i,i<a?prb[i]:0);
}

void ProbabilisticTreeFromValue(Prb *p,unt v,int *_ind,int *_rng)
{
	int *t=p->Tree;
    int i,j=1,n=p->N,ind=0,rng=0;
    Assert(p && p->Tree && v<p->Tree[0],"<ProbabilisticTreeFromValue> v="+v+"<?<"+p->Tree[0]);
	assert(p && p->Tree && v<p->Tree[0]);
    fordec(i,n)
	{
        if(v<t[j]) j+=2;
        else       v-=t[j],rng+=t[j],j=t[j+1],ind|=1<<i;
    }
    *_ind=ind,*_rng=rng;
}

void ProbabilisticTreeFromIndex(Prb* p,int ind,int *_rng)
{
    int *t=p->Tree;
    int i,j=1,n=p->N,rng=0;
    assert(p && p->Tree && ind<p->AllSym);
    fordec(i,n)
    {
        if(!(ind&(1<<i))) j+=2;
        else              rng+=t[j],j=t[j+1];
    }
    *_rng=rng;
}

void  ProbabilisticUpdate(Prb* p,int j)
{
    p->AllPast++;
	if((p->MaxCumul>0 && p->Tree[0]>p->MaxCumul) || (p->MaxCumul<=0 && !p->T && p->AN>0 && p->K>0))
	{
		int *prb=p->Prob,an=p->AN,a=p->AllSym;
		int i,sm=0;

		forinc(i,a)
		{
			prb[i]=(prb[i]+1)>>1;
			sm+=prb[i];
			Assert(prb[i]>0,"<ProbabilisticUpdate>");
		}

		Assert(sm<(1<<16),"<ProbabilisticUpdate> summ to large");

		ProbabilisticTreeBuild(p);
	}
	/*
	if(!p->TVS && p->EDL>0)
	{
		dbl vsz=0,_l2=1./log(2.);
		int i,aa=1<<p->N,a=p->AllSym;
		forinc(i,aa) 
			if(i<a)
				vsz+=_l2*p->Prob[i]*log((dbl)a/p->Prob[i]);
		p->VS=vsz/p->Tree[0];
	}
	*/
    if(p->K)
    {
        p->Prob[j]+=p->K;
        ProbabilisticTreeAdd(p,j,p->K);
		p->T++;
        p->TVS++;
    }
    if(p->T==p->DL) p->T=0;
    if(p->TVS==p->EDL) p->TVS=0;
}

void Entropy_ProbabilisticInit
(
    Prb* p,
    int *prb,int a, // static data
	int k,    // k=0 no adaptive, k=1 simplest adaptive
    int an,   // rough probability range
	int dl,   // delay, if delay =-1 no normalization
	int edl
)
{
    int i;
    assert(p && k>=0 && an>=0 && a>=0);
    assert(k*dl+an<(1<<23));
    p->K=k;
    p->AN=an;
    p->DL=dl;
	p->EDL=edl;//-1;
	p->TVS=0;
    p->AllSym=a;
    ProbabilisticInit(p);
    if(prb) forinc(i,a) p->Prob[i]=prb[i];
    else    forinc(i,a) p->Prob[i]=1;
    ProbabilisticTreeBuild(p);
}

void Entropy_ProbabilisticInitFill
(
    Prb* p,
	Prb *p0,
	int k,    // k=0 no adaptive, k=1 simplest adaptive
    int an,   // rough probability range
	int dl,   // delay, if delay =-1 no normalization
	int edl
)
{
    int i;
    assert(p && k>=0);
    assert(k*dl+an<(1<<23));

    assert(p && k>=0);

    p->K=k;
    p->AN=an;
    p->DL=dl;
	p->EDL=edl;//-1;

	p->TVS=0;
	
    assert((!p0) || (p->AllSym==p0->AllSym && p->N==p0->N));

	int a=p0->AllSym;

    assert(p && p->AllSym>=0);

    assert(p->Tree);
    assert(p->Prob);
    assert(p0->Tree);
    assert(p0->Prob);

	if(p0)
	{
	    p->AllPast=p0->AllPast;
		int _a=(1<<(p0->N+2))+5;
		forinc(i,_a)
			p->Tree[i]=p0->Tree[i];
		forinc(i,a)
			p->Prob[i]=p0->Prob[i];

		p->T=p0->T;
		p->VS=p0->VS;
	}
	else
	{
		forinc(i,a) p->Prob[i]=1;
		ProbabilisticTreeBuild(p);
	}
}

void Entropy_ProbabilisticInitMaxCumul
(
	EntropyProbabilisticContext* p,
	int *prb,int a, // static data, if prb=0 => uniform 
	int k,    // k=0 no adaptive, k=1 simplest adaptive
	int maxcamul // delay, if delay =-1 no normalization
)
{
    int i;
    assert(maxcamul>0);
    p->AN=p->DL=p->EDL=p->TVS=0;
    p->K=k;
    p->AllSym=a;
	p->MaxCumul=maxcamul;
    ProbabilisticInit(p);
    if(prb) forinc(i,a) p->Prob[i]=prb[i];
    else    forinc(i,a) p->Prob[i]=1;
    ProbabilisticTreeBuild(p);
}


void Entropy_ProbabilisticInitFillMaxCamul
(
    Prb* p,
	Prb *p0,
	int k,    // k=0 no adaptive, k=1 simplest adaptive
	int maxcamul // delay, if delay =-1 no normalization
)
{
    int i;
    assert(p && k>=0 && maxcamul>0);

    p->AN=p->DL=p->EDL=p->TVS=0;
    p->K=k;
	p->MaxCumul=maxcamul;

    assert((!p0) || (p->AllSym==p0->AllSym && p->N==p0->N));

	int a=p0->AllSym;
    p->AllSym=a;
    assert(p && p->AllSym>=0);

    assert(p->Tree);
    assert(p->Prob);
    assert(p0->Tree);
    assert(p0->Prob);

	if(p0)
	{ 
	    p->AllPast=p0->AllPast;
		int _a=(1<<(p0->N+2))+5;
		forinc(i,_a)
			p->Tree[i]=p0->Tree[i];
		forinc(i,a)
			p->Prob[i]=p0->Prob[i];
		
		p->T=p0->T;
		p->VS=p0->VS;
	}
	else   
	{
		forinc(i,a) p->Prob[i]=1;		
		ProbabilisticTreeBuild(p);
	}

}

//----------------------------------------------------

typedef EntropyModelContext         Ppm;

//-------------------------------------------------------------------------
//--------  PROBABILISTIC TREE CLASS FUNCTIONS ----------------------------
//-------------------------------------------------------------------------

void Entropy_FastModelContextInit(Ppm* pm,int mxm,int minpast)
{
	int i;
	ALLOC(pm->PDis,Prb,mxm);
	ALLOC(pm->ID,int,mxm);
	pm->MaxCnt=mxm;
	pm->UseHash=0;
	pm->MinPast=minpast;
	forinc(i,mxm)
		pm->PDis[i].Prob=0; // no information available
}

void Entropy_ModelContextInit(Ppm* pm,int mxm,int asym,int k,int an,int dl,int edl)
{
	int i;
	//pm->PDis=ALLOC(Prb,mxm+1);
	//pm->ID=ALLOC(int,mxm);

	Entropy_FastModelContextInit(pm,mxm+1,0);
	
	pm->AllSym=asym;
	pm->AllCnt=0;
	pm->MaxCnt=mxm;
	//pm->UseHash=1;

	Entropy_ProbabilisticInit(&(pm->PDis[mxm]),0,asym,k,an,dl);
	forinc(i,mxm)
	{
		Entropy_ProbabilisticInit(&(pm->PDis[i]),0,asym,k,an,dl);
		pm->PDis[i].EDL=edl;
		pm->ID[i]=0; // no information available
	}
}

void Entropy_ModelCut(Ppm* pm)
{
	int i,j,q;
	for(q=1;(pm->AllCnt<<2)>pm->MaxCnt;q++)
	forinc(i,pm->MaxCnt)
	if(i && pm->ID[i] && pm->PDis[i].Tree[0]==q)
	{
		forinc(j,pm->AllSym)
			pm->PDis[i].Prob[j]=1;
		ProbabilisticTreeBuild(&(pm->PDis[i]));
		pm->ID[i]=0;
		pm->AllCnt--;
		if((pm->AllCnt<<2)<pm->MaxCnt) return;
	}
	q=1;
}

void Entropy_ModelContextClose(Ppm* pm)
{
	int i;
	forinc(i,pm->MaxCnt)
	if(pm->PDis[i].Prob)//ID[i])
		Entropy_ProbabilisticRelease(&(pm->PDis[i]));
	FREE(pm->PDis);
	FREE(pm->ID);
}

#endif