/*\   Programmed by Alexander Zhirkov <azh@graphics.cs.msu.su>
\*/
#ifdef SAIT_CONFORMANCE_2003

//---------------------------------------------------------------------------
#ifndef __Entropy__H
#define __Entropy__H
//---------------------------------------------------------------------------

#include <stdio.h>

typedef unsigned int unt;
typedef unsigned char uhr;
typedef unsigned short urd;
typedef double dbl;

typedef struct EntropyProbabilisticContext
{
	int AllSym; // all symbols
	int *Prob;  // integer probabilities
	int *Tree;
	int N;      // tree height
	int T;      // normalization time index
	int MaxCumul; // value for normalizing
	int K,AN,DL;
	dbl VS;     // virtual size
	int TVS,EDL;// cur and all time for VS estimation
	int AllPast;
}
EntropyProbabilisticContext;

void Entropy_ProbabilisticInitMaxCumul
(
	EntropyProbabilisticContext*,
	int *prb,int a, // static data, if prb=0 => uniform 
	int k,    // k=0 no adaptive, k=1 simplest adaptive
	int camul // delay, if delay =-1 no normalization
);
void Entropy_ProbabilisticInit
(
	EntropyProbabilisticContext*,
	int *prb,int a, // static data, if prb=0 => uniform 
	int k,    // k=0 no adaptive, k=1 simplest adaptive
	int an,   // rough probability range, if=0 => no normalization
	int dl,   // delay, if delay =-1 no normalization
	int edl=-1
);
void Entropy_ProbabilisticInitFillMaxCamul
(
    EntropyProbabilisticContext* p,
	EntropyProbabilisticContext *p0,
	int k,    // k=0 no adaptive, k=1 simplest adaptive
	int maxcamul // delay, if delay =-1 no normalization
);
void Entropy_ProbabilisticInitFill
(
	EntropyProbabilisticContext*,
	EntropyProbabilisticContext* p0,
	int k,    // k=0 no adaptive, k=1 simplest adaptive
	int an,   // rough probability range, if=0 => no normalization
	int dl,   // delay, if delay =-1 no normalization
	int edl=-1
);


void Entropy_ProbabilisticRelease(EntropyProbabilisticContext*);

//----------------------------------------------------

struct EntropyModelContext
{
	EntropyProbabilisticContext* PDis; // array of probability distributions
    int AllSym; // number of symbols, it is equal for all probabilistic models
    int AllCnt; // all contexts
	int MaxCnt;
	int UseHash;
	int MinPast; // minimum  
    int *ID; // quality of each context
};

void Entropy_FastModelContextInit
(
	EntropyModelContext* pm,
	int mxm,
	int minpast=256
);

void Entropy_ModelContextInit
(
	EntropyModelContext* pm,
	int mxm,int asym,
	int k,int an,int dl,
	int edl=-1
);

void Entropy_ModelContextClose(EntropyModelContext* pm);

void Entropy_ModelCut(EntropyModelContext*pm);


//----------------------------------------------------


//-- Some of internal functions
int  ProbabilisticTreeRecurs(EntropyProbabilisticContext* p,int x,int n);
void ProbabilisticInit(EntropyProbabilisticContext* p);
void Entropy_ProbabilisticRelease(EntropyProbabilisticContext* p);
void ProbabilisticTreeAdd(EntropyProbabilisticContext* p,int ind,int av);
void ProbabilisticTreeBuild(EntropyProbabilisticContext* p);
void ProbabilisticTreeFromValue(EntropyProbabilisticContext *p,unt v,int *_ind,int *_rng);
void ProbabilisticTreeFromIndex(EntropyProbabilisticContext* p,int ind,int *_rng);
void ProbabilisticUpdate(EntropyProbabilisticContext* p,int j);

//---------------------------------------------------------------------------

#endif

#endif