/***********************************************************************
This software module was originally developed by Alexander Zhirkov (
Moscow State University), Joo Yeon Han (Samsung Advance Institute of Technology)
in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)
standard. This software module is an implementation of a part of one or
more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4
Systems (ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 
Systems (ISO/IEC 14496-1) free license to this software module or 
modifications thereof for use in hardware or software products claiming
conformance to the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to 
use this software module in hardware or software products are advised that
its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Samsung Advance Institute of Technology
retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using
the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
This copyright notice must be included in all copies or derivative works.
Copyright ?2001
************************************************************************

For more information or to receive an up to date version of this module,
you may call Joo Yeon Han at jyhan@sait.samsung.co.kr
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Depth Image-based Representation
FILE:       Octree.cpp
AUTHOR:     Joo Yeon Han (SAMSUNG AIT)
OVERVIEW:   Implementation of Depth Image-based Representation
-------------------------------------------------------------------------*/

// Octree.cpp: implementation of the COctree class.
//
//////////////////////////////////////////////////////////////////////
#include "Octree.h"
#include <math.h>
#include "Common.h"
#include "Array.h"
#include "TBVOCompress.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

typedef unsigned char byte;

COctree::COctree()
:octree(NULL)
,voxelImageIndex(NULL)
,octreeBitstream(NULL)
{
}

COctree::~COctree()
{
	delete[] octree;
	delete[] voxelImageIndex;
	delete[] octreeBitstream;
}

// delete and new octree with size
int COctree::setLenOctreeBuf(int size){
	delete[] octree;
	octree=new int[size];
	return octree!=NULL;
}
// delete and new voxelImageIndex with size
int COctree::setLenVoxelImageIndexBuf(int size){
	delete[] voxelImageIndex;
	voxelImageIndex=new int[size];
	return voxelImageIndex!=NULL;
}
// delete and new octreeBitstream with size
int COctree::setLenOctreeBitstreamBuf(int size){
	delete[] octreeBitstream;
	octreeBitstream=new char[size];
	return octreeBitstream!=NULL;
}

int COctree::EncodeOctree(int iswithhead){
	len_octreeBitstream=0;
	int tmp;
	unsigned int ut;
	int octreeLevel;

	BStream strm(octreeBitstream,0);

	//Header
	octreeLevel=(int)((double)1.000000001+log(octreeResolution-1)/log(2));
	if(iswithhead)
	{
		for(tmp=32;!(octreeResolution&(1<<(tmp-1)));tmp--);
		strm.write_bits(5,tmp);
		strm.write_bits(tmp,octreeResolution);
		if(!numOfImages)
			strm.write_bits(3,0);
		else
		{
			for(tmp=32;!(numOfImages&(1<<(tmp-1)));tmp--);
			strm.write_bits(3,tmp);
			strm.write_bits(tmp,numOfImages);
		}
	}

	//frames
	int len=0;
	int t_ind=0,bvo_ind=0;
	EntropyModelContext pm;
#define ModelMAXCNTXTS (1<<14)
	Entropy_FastModelContextInit(&pm,ModelMAXCNTXTS);
	do{
		ut=0x000001C8; //start_code;
        strm.flush_bits();                
		strm.write_bits(32,ut);
		BinaryArithmetic rc_tbvo;
		Entropy_BinaryArithmeticEncodeInit(&rc_tbvo,&strm);
		
		len=Entropy_CodeFrameSeparatedTBVO
		(
			&rc_tbvo,
			&rc_tbvo,
			&pm,
			
			voxelImageIndex,
			octree,
			t_ind,bvo_ind,
			
			octreeLevel,
			numOfImages,
			1,
			len
		);
			
		Entropy_BinaryArithmeticEncodeClose(&rc_tbvo);

	}while(bvo_ind<len_octree || (t_ind<len_voxels && numOfImages));

	len_octreeBitstream=strm.pos;
	return 0;
}


int COctree::DecodeOctree(int iswithhead){
	unsigned int unt;
	int ind=0;
	int octreeLevel;


	BStream strm(octreeBitstream,len_octreeBitstream);

	if(iswithhead)
	{//Read header
		byte bt;
		// Read the bits
		bt=strm.read_bits(5);
		octreeResolution=strm.read_bits(bt);
		bt=strm.read_bits(3);
		if(bt) numOfImages=strm.read_bits(bt); else numOfImages=0;
	}
	
	octreeLevel=(int)((double)1.000000001+log(octreeResolution-1)/log(2));
	int len=0;
	len_voxels=0;
	len_octree=0;
	EntropyModelContext pm;
#define ModelMAXCNTXTS (1<<14)
	Entropy_FastModelContextInit(&pm,ModelMAXCNTXTS);
	do{
        strm.flush_bitsr();              
		unt=strm.read_bits(32);
		if(unt!=0x000001C8) {strm.back(32);break;}//start_code;

		BinaryArithmetic rc_tbvo;
		Entropy_BinaryArithmeticDecodeInit(&rc_tbvo,&strm);
		len=Entropy_CodeFrameSeparatedTBVO(
			&rc_tbvo,
			&rc_tbvo,
			&pm,

			voxelImageIndex,
			octree,
			len_voxels,
			len_octree,
			
			octreeLevel,
			numOfImages,
			0,
			len
			);
		Entropy_BinaryArithmeticDecodeClose(&rc_tbvo);
		{//find start code
			int n=0;
			strm.back(1); n++;
			while(strm.read_bit()==0) {strm.back(2);n++;}
			n--;
			for(;n>=0;n--){
				unt=strm.read_bits(32);
				strm.back(31);
				if(unt==0x000001C8) break;//start_code finded
			}
			strm.back(1);
		}
	}while(1);
	return 0;
}

// new memory
extern "C" HRESULT  __declspec(dllexport) GetOctree(COctree  ** pOctree){
	*pOctree=new COctree;
	return S_OK;
}
// delete memory
extern "C" HRESULT  __declspec(dllexport) FreeOctree(COctree ** pOctree){
	delete (*pOctree);
	*pOctree=NULL;
	return S_OK;
}
