/***********************************************************************
This software module was originally developed by Alexander Zhirkov (
Moscow State University), Joo Yeon Han (Samsung Advance Institute of Technology)
in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)
standard. This software module is an implementation of a part of one or
more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4
Systems (ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 
Systems (ISO/IEC 14496-1) free license to this software module or 
modifications thereof for use in hardware or software products claiming
conformance to the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to 
use this software module in hardware or software products are advised that
its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Samsung Advance Institute of Technology
retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using
the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
This copyright notice must be included in all copies or derivative works.
Copyright ?2001
************************************************************************

For more information or to receive an up to date version of this module,
you may call Joo Yeon Han at jyhan@sait.samsung.co.kr
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Depth Image-based Representation
FILE:       Octree.h
AUTHOR:     Joo Yeon Han (SAMSUNG AIT)
OVERVIEW:   Implementation of Depth Image-based Representation
-------------------------------------------------------------------------*/

// Octree.h: interface for the COctree class.
//
//////////////////////////////////////////////////////////////////////
#ifdef SAIT_CONFORMANCE_2003

#if !defined(AFX_OCTREE_H__7BF07521_962D_48EE_BC5F_F2EA27F76292__INCLUDED_)
#define AFX_OCTREE_H__7BF07521_962D_48EE_BC5F_F2EA27F76292__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <wtypes.h>

class COctree {
public:
	COctree();	
	virtual ~COctree();
	int octreeResolution;
	int* octree;          // use int* instead of unsigned char*
	int* voxelImageIndex; // if this is null, then it's BVO, right?
	int numOfImages;      // if BVO, then this value is 0, right?
	char* octreeBitstream;
	int len_octreeBitstream; // this is only for encoder use. 
	// the main encoder needs to copy the returned bitstream to the mpeg4 bitstream, so the encoder should return this value.
	// the decoder should not use this because we don't know how long the bitstream is.
	// however, the main decoder may need this information after the decoding is finished.
	// so the decoder must also give this value.

	//tmp
	int len_octree;
	int len_voxels;
		
	virtual int EncodeOctree(int iswithhead=1);   
	virtual int DecodeOctree(int iswithhead=1);

	virtual int setLenOctreeBuf(int);
	virtual int setLenVoxelImageIndexBuf(int);
	virtual int setLenOctreeBitstreamBuf(int);

protected:
};

typedef HRESULT (*GETOCTREE)(COctree  ** pOctree);
typedef HRESULT (*FREEOCTREE)(COctree ** pOctree);

#endif // !defined(AFX_OCTREE_H__7BF07521_962D_48EE_BC5F_F2EA27F76292__INCLUDED_)

#endif