/********************************************************************************/
/* This software module was developped by										*/

/*	   Royan Jerme , jerome.royan@francetelecom.com								*/
/*                   France Telecom R&D, France		*/

/* in the course of development of the MPEG-4 standard (Part 16 AFX).			*/
/* This software module is an implementation of a part of one or				*/
/* more MPEG-4 tools as specified by the										*/
/* MPEG-4 standard. ISO/IEC gives users of the MPEG-4 standard					*/
/* free license to this software module or modifications thereof				*/
/* for use in hardware or software products claiming conformance				*/
/* to the MPEG-4 standard. Those intending to use this software					*/
/* module in hardware or software products are advised that its					*/
/* use may infringe existing patents. The original developer of					*/
/* this software module and his company, the subsequent							*/
/* editors and their companies, and ISO/IEC have no liability					*/
/* for use of this software module or modifications thereof in					*/
/* an implementation. Copyright is not released for non							*/
/* MPEG-4 standard conforming products. INT ARTEMIS								*/
/* retain full right to use the code for their own purposes,					*/
/* assign or donate the code to a third party and to inhibit					*/
/* third parties from using the code for non MPEG-4 standard					*/
/* conforming products. This copyright notice must be included					*/
/* in all copies or derivative works.    										*/
/* Copyright (c)2002 - 2005														*/
/********************************************************************************/

// FPESet.cpp: implementation of the FPESet class.
// dependences :
//////////////////////////////////////////////////////////////////////

#include "FP.h"


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////


FP::FP(void)
{
	m_Type=FOOTPRINT;
	m_height=0;
	m_altitude=0;
	m_metricError=0;
	m_IsActive=false;
	m_IsCollapsable=false;
	m_pGravityCenter=NULL;
	m_footprint=NULL;
	m_pParent=NULL;
	m_NbRings=0;
	m_NbVerticesByRing=NULL;
	m_pNode=NULL;
	m_DefNameSwapNodes=new ZArray<ZString>;
	m_pChildren=new ZArray<FP*>;

}


FP::~FP(void)
{
	if (m_pGravityCenter!=NULL)
	{
		delete m_pGravityCenter;
		m_pGravityCenter=NULL;
	}
	

	if (m_pChildren!=NULL)
	{
		delete m_pChildren;
		m_pChildren=NULL;
	}
	delete m_DefNameSwapNodes;
	

	if (m_footprint!=NULL)
	{
		for (int i=0; i<m_NbRings; i++)
		{
			free ((m_footprint)[i]);
		}
		free(m_footprint);
	}
	if (m_NbVerticesByRing!=NULL) free(m_NbVerticesByRing);

}

FP & FP::operator =(FP &in)
{
	m_Type=in.Type();
	m_Index=in.Index();
	m_footprint=in.FootPrint();
	m_metricError=in.MetricError();
	m_IsActive=in.IsActive();
	m_IsCollapsable=in.IsCollapsable();
	m_pChildren=in.Children();
	m_pParent=in.Parent();
	m_pGravityCenter=new Vec2f(*(in.GravityCenter()));
	m_pNode=in.GetMediaObject();
	return *this;
}

Vec2f* FP::GravityCenter()
{
	if (m_pGravityCenter==NULL)
	{
		ComputeGravityCenter();
	}
	return m_pGravityCenter;
}

void FP::ComputeGravityCenter()
{
	float x=0.0,y=0.0;
	int i, nbvertices;
	if (m_pGravityCenter!=NULL)
	{
		delete m_pGravityCenter;
		m_pGravityCenter=NULL;
	}
	
	nbvertices=m_NbVerticesByRing[0];

	for (i=0; i<nbvertices; i++)
	{
		x+=m_footprint[0][i].x;
		y+=m_footprint[0][i].y;
	}
	
	m_pGravityCenter=new Vec2f(x/(float)nbvertices,y/(float)nbvertices);
}

float FP::ComputeDistanceFromPoint(Vec3f* viewpoint)
{
	Vec2f* gc=GravityCenter();
	float dist2D=sqrt( (gc->x-viewpoint->x)*(gc->x-viewpoint->x) + (gc->y-viewpoint->y)*(gc->y-viewpoint->y) );
	if (viewpoint->z<m_altitude)
		return sqrt(pow(dist2D,2)+pow(m_altitude-viewpoint->z,2));
	else if
		(viewpoint->z>m_altitude+m_height)
		return sqrt(pow(dist2D,2)+pow(viewpoint->z-m_altitude-m_height,2));
	else
		return dist2D;
}


bool FP::MustBeDevelopped(Vec3f* viewpoint)
{

	if (m_pChildren->GetLength()==0) return false;
	if (pow((ComputeDistanceFromPoint(viewpoint)/150),6)<m_metricError)
		return true;
	else
		return false;
}

bool FP::MustBeCollapsed(Vec3f* viewpoint)
{
	if (m_pParent==NULL) return false;
	if (pow((ComputeDistanceFromPoint(viewpoint)/150),6)>m_metricError)
		return true;
	else
		return false;
}

void FP::AddChild(FP* fpe)
{
	(*m_pChildren)+=fpe;
}

void FP::SuppressChild(FP* fpe)
{
	(*m_pChildren)-=fpe;
}

void FP::SuppressChildren()
{
	(*m_pChildren).Clear();
}
