/********************************************************************************/
/* This software module was developped by										*/

/*	   Jerome Royan (MP), jerome.royan@francetelecom.com,						*/
/*                        France Telecom R&D, France							*/

/* in the course of development of the MPEG-4 standard (Part 16 AFX).			*/
/* This software module is an implementation of a part of one or				*/
/* more MPEG-4 tools as specified by the										*/
/* MPEG-4 standard. ISO/IEC gives users of the MPEG-4 standard					*/
/* free license to this software module or modifications thereof				*/
/* for use in hardware or software products claiming conformance				*/
/* to the MPEG-4 standard. Those intending to use this software					*/
/* module in hardware or software products are advised that its					*/
/* use may infringe existing patents. The original developer of					*/
/* this software module and his company, the subsequent							*/
/* editors and their companies, and ISO/IEC have no liability					*/
/* for use of this software module or modifications thereof in					*/
/* an implementation. Copyright is not released for non							*/
/* MPEG-4 standard conforming products. INT ARTEMIS								*/
/* retain full right to use the code for their own purposes,					*/
/* assign or donate the code to a third party and to inhibit					*/
/* third parties from using the code for non MPEG-4 standard					*/
/* conforming products. This copyright notice must be included					*/
/* in all copies or derivative works.    										*/
/* Copyright (c)2002 - 2005														*/
/********************************************************************************/

// FP.h: interface for the FP class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_FP_H__93F1CF93_D37D_11D5_B9F1_000102A02E5B__INCLUDED_)
#define AFX_FP_H__93F1CF93_D37D_11D5_B9F1_000102A02E5B__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "../../../Core/MediaObj.h"
#include "../../../core/ztypes.h"


#ifndef BYTE
typedef unsigned char BYTE;
#endif

#ifndef LPBYTE
typedef BYTE* LPBYTE;
#endif

typedef enum {FOOTPRINT, BUILDING} FPEType;

class FP  
{
public:
	FP(void);
	virtual ~FP(void);

	FP & operator =(FP &in);


	/*! Return the node index
	 * @return the index
	 */
	inline long Index() const { return m_Index; };
	
	/*! Set the node index
	 * @param index the index
	 */
	inline void SetIndex(long index) { m_Index=index; };

	inline FPEType Type() { return m_Type; };
	inline void SetType(FPEType type) { m_Type=type; };
	
	/*! Return the node footprint
	 * @return the footprint
	 */
	//inline ZArray< ZArray< Vec2f >* >* Footprint() const {return m_footprint;};
	inline Vec2f** FootPrint() const {return m_footprint;};
	
	/*! Set the node footprint
	 * @param footprint the footprint
	 */
	//inline void SetFootprint(ZArray< ZArray< Vec2f >* >* footprint) {m_footprint=footprint;};
	inline void SetFootPrint(Vec2f** footprint) {m_footprint=footprint;};

	inline int NbRings() { return m_NbRings; };
	inline void SetNbRings(int nbRings) { m_NbRings=nbRings; };

	inline unsigned int* NbVerticesByRing() { return m_NbVerticesByRing; };
	inline void SetNbVerticesByRing (unsigned int* nbVerticesByRing) { m_NbVerticesByRing=nbVerticesByRing; };

	//inline SetSwapNode(int nbSwapNodes,char** defNames) { m_NbSwapNodes=nbSwapNodes; m_DefNameSwapNodes=defNames; };

	/*! Return the number of node children
	 * @return the number of node children
	 */
	inline int NbChildren() const {return (int)m_pChildren->GetLength();};
	
	/*! Return a pointer on the array of node children
	 * @return a pointer on the array of node children
	 */
	inline ZArray<FP*>* Children() {return m_pChildren;};

	inline void SetChildren(ZArray<FP*>* children) { m_pChildren=children;};
	
	void AddChild(FP* fpe);

	void SuppressChild(FP* fpe);

	void SuppressChildren();

	inline void AddSwapNode(ZString defname) { (*m_DefNameSwapNodes)+=defname; };

	inline void RemoveSwapNodes() { m_DefNameSwapNodes->Clear(); };

	inline ZArray<ZString>* GetSwapNodes() { return m_DefNameSwapNodes; };

	inline int GetNbSwapNodes() { return m_DefNameSwapNodes->GetLength(); };

	inline FP* Parent() const { return m_pParent; };

	inline void SetParent(FP* parent) { m_pParent=parent; };

	inline MediaObject* GetMediaObject() const { return m_pNode; };

	inline void SetMediaObject ( MediaObject* mo) { m_pNode=mo; };
	
	//! Compute the gravity center of the external ring of the footprint
	void ComputeGravityCenter();

	//! Compute the distance from the building to a 3D viewpoint
	float ComputeDistanceFromPoint(Vec3f* viewpoint); 

	
	/*! Return the gravity center of the external ring of the footprint (if not already compute, it automaticcaly calls CoomputeGravityCenter() )
	 * @return the gravity center of the external ring of the footprint
	 */
	Vec2f* GravityCenter();

	
	bool MustBeDevelopped(Vec3f* viewpoint);

	bool MustBeCollapsed(Vec3f* viewpoint);
	
	inline float Height() const { return m_height;};
	inline void SetHeight(float height) { m_height=height;};
	inline float Altitude() const { return m_altitude;};
	inline void SetAltitude(float altitude) { m_altitude=altitude;};
	inline float MetricError() const { return m_metricError;};
	inline void SetMetricError(float metricError) { m_metricError=metricError;};

	inline float IsActive() const { return m_IsActive;};
	inline void SetIsActive(long isActive) { m_IsActive=isActive;};
	inline float IsCollapsable() const { return m_IsCollapsable;};
	inline void SetIsCollapsable(long isCollapsable) { m_IsCollapsable=isCollapsable;};


protected:
	FPEType m_Type;
	long m_Index;
	float m_metricError;
	Vec2f** m_footprint;
	int m_NbRings;
	unsigned int* m_NbVerticesByRing;
	float m_height, m_altitude;
	ZArray<ZString>* m_DefNameSwapNodes;
	ZArray<FP*>* m_pChildren;
	FP* m_pParent;
	Vec2f* m_pGravityCenter;	
	MediaObject* m_pNode;
	bool m_IsActive;
	bool m_IsCollapsable;
	
};

#endif // !defined(AFX_FP_H__93F1CF93_D37D_11D5_B9F1_000102A02E5B__INCLUDED_)
