/********************************************************************************/
/* This software module was developped by										*/

/*	   Royan Jerme , jerome.royan@francetelecom.com								*/
/*                   France Telecom R&D, France		*/

/* in the course of development of the MPEG-4 standard (Part 16 AFX).			*/
/* This software module is an implementation of a part of one or				*/
/* more MPEG-4 tools as specified by the										*/
/* MPEG-4 standard. ISO/IEC gives users of the MPEG-4 standard					*/
/* free license to this software module or modifications thereof				*/
/* for use in hardware or software products claiming conformance				*/
/* to the MPEG-4 standard. Those intending to use this software					*/
/* module in hardware or software products are advised that its					*/
/* use may infringe existing patents. The original developer of					*/
/* this software module and his company, the subsequent							*/
/* editors and their companies, and ISO/IEC have no liability					*/
/* for use of this software module or modifications thereof in					*/
/* an implementation. Copyright is not released for non							*/
/* MPEG-4 standard conforming products. INT ARTEMIS								*/
/* retain full right to use the code for their own purposes,					*/
/* assign or donate the code to a third party and to inhibit					*/
/* third parties from using the code for non MPEG-4 standard					*/
/* conforming products. This copyright notice must be included					*/
/* in all copies or derivative works.    										*/
/* Copyright (c)2002 - 2005														*/
/********************************************************************************/

// FPBuilding.cpp: implementation of the FPBuilding class.
// dependences :
//////////////////////////////////////////////////////////////////////

#include "FPBuilding.h"
#include "FP.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////


FPBuilding::FPBuilding(void)
{
	m_Type=BUILDING;
	m_State3D=NOCOMPUTED;
	m_WallVertices=NULL;
	m_WallIndices=NULL;

	
}


FPBuilding::~FPBuilding(void)
{
	if (m_WallVertices!=NULL)
	{
		free(m_WallVertices);
		m_WallVertices=NULL;
	}

	if (m_WallIndices!=NULL)
	{
		for (int i=0;i<m_NbRings;i++)
			if (m_WallIndices[i]!=NULL)
			{
				free(m_WallIndices[i]);
				m_WallIndices[i]=NULL;
			}
		free (m_WallIndices);
		m_WallIndices=NULL;
	}
	if (m_Roofs!=NULL)
	{
		m_Roofs->Clear();
		m_Roofs=NULL;
	}
	if (m_Facades!=NULL)
	{
		m_Facades->Clear();
		m_Facades=NULL;
	}

}

FPBuilding & FPBuilding::operator =(FPBuilding &in)
{
	m_Type=BUILDING;
	SetIndex(in.Index());
	SetFootPrint(in.FootPrint());
	SetChildren(in.Children());
	SetParent((FP*)in.Parent());
	m_pGravityCenter=new Vec2f(*(in.GravityCenter()));
	SetMediaObject(in.GetMediaObject());
	SetHeight(in.Height());
	SetAltitude(in.Altitude());
	return *this;
}

void FPBuilding::Reconstruction3D()
{
	int i,j;

	if (m_WallVertices!=NULL) free(m_WallVertices);

	if (m_WallIndices!=NULL)
	{
		for (i=0;i<m_NbRings;i++)
			if (m_WallIndices[i]!=NULL)
			{
				free(m_WallIndices[i]);
				m_WallIndices[i]=NULL;
			}
		free (m_WallIndices);
	}

	int nbWallVertices=0;

	if (m_DefNameSwapNodes->GetLength()==0)
	{
		m_WallIndices=(unsigned int**)malloc(m_NbRings*sizeof(unsigned int*));
		
		for (i=0;i<m_NbRings;i++)
		{
			m_WallIndices[i]=(unsigned int*)malloc(m_NbVerticesByRing[i]*4*sizeof(unsigned int));
			nbWallVertices+=m_NbVerticesByRing[i];
		}


		
		unsigned int nbpoints=0;
		for (i=0;i<m_NbRings;i++)
		{
			for (j=0;j<(int)m_NbVerticesByRing[i];j++)
			{
				m_WallIndices[i][j*4]=nbpoints*2;
				m_WallIndices[i][j*4+1]=nbpoints*2+1;
				m_WallIndices[i][j*4+2]=(nbpoints+nbWallVertices)*2+1;
				m_WallIndices[i][j*4+3]=(nbpoints+nbWallVertices)*2;
				nbpoints++;
			}
		}
		m_WallVertices=(float*)malloc(nbWallVertices*8*4*sizeof(float));
			
		nbpoints=0;
		unsigned int nbpointsroof=0;
		float coordtex=0.0f;

		for (i=0;i<m_NbRings;i++)
		{
			m_WallVertices[nbpoints*16]=coordtex;
			m_WallVertices[nbpoints*16+1]=1.0f;
			m_WallVertices[nbpoints*16+5]=m_footprint[i][0].x;
			m_WallVertices[nbpoints*16+6]=m_footprint[i][0].y;
			m_WallVertices[nbpoints*16+7]=m_altitude+m_height;

			m_WallVertices[(nbpoints+nbWallVertices)*16]=coordtex;
			m_WallVertices[(nbpoints+nbWallVertices)*16+1]=0.0f;
			m_WallVertices[(nbpoints+nbWallVertices)*16+5]=m_footprint[i][0].x;
			m_WallVertices[(nbpoints+nbWallVertices)*16+6]=m_footprint[i][0].y;
			m_WallVertices[(nbpoints+nbWallVertices)*16+7]=m_altitude;
		
			nbpoints++;

			for (j=1;j<(int)m_NbVerticesByRing[i];j++)
			{
				coordtex+=1.0f;
				m_WallVertices[nbpoints*16-8]=m_WallVertices[nbpoints*16]=coordtex;
				m_WallVertices[nbpoints*16-7]=m_WallVertices[nbpoints*16+1]=1.0f;
				m_WallVertices[nbpoints*16-3]=m_WallVertices[nbpoints*16+5]=m_footprint[i][j].x;
				m_WallVertices[nbpoints*16-2]=m_WallVertices[nbpoints*16+6]=m_footprint[i][j].y;
				m_WallVertices[nbpoints*16-1]=m_WallVertices[nbpoints*16+7]=m_altitude+m_height;

				m_WallVertices[(nbpoints+nbWallVertices)*16-8]=m_WallVertices[(nbpoints+nbWallVertices)*16]=coordtex;
				m_WallVertices[(nbpoints+nbWallVertices)*16-7]=m_WallVertices[(nbpoints+nbWallVertices)*16+1]=0.0;
				m_WallVertices[(nbpoints+nbWallVertices)*16-3]=m_WallVertices[(nbpoints+nbWallVertices)*16+5]=m_footprint[i][j].x;
				m_WallVertices[(nbpoints+nbWallVertices)*16-2]=m_WallVertices[(nbpoints+nbWallVertices)*16+6]=m_footprint[i][j].y;
				m_WallVertices[(nbpoints+nbWallVertices)*16-1]=m_WallVertices[(nbpoints+nbWallVertices)*16+7]=m_altitude;
				
				nbpoints++;
				
			}

			coordtex+=1.0f;
			m_WallVertices[nbpoints*16-8]=coordtex;
			m_WallVertices[nbpoints*16-7]=1.0f;
			m_WallVertices[nbpoints*16-3]=m_footprint[i][0].x;
			m_WallVertices[nbpoints*16-2]=m_footprint[i][0].y;
			m_WallVertices[nbpoints*16-1]=m_altitude+m_height;
			
			
			m_WallVertices[(nbpoints+nbWallVertices)*16-7]=0.0;
			m_WallVertices[(nbpoints+nbWallVertices)*16-3]=m_footprint[i][0].x;
			m_WallVertices[(nbpoints+nbWallVertices)*16-2]=m_footprint[i][0].y;
			m_WallVertices[(nbpoints+nbWallVertices)*16-1]=m_altitude;

		}
		


		// Calcul des normales

		nbpoints=0;
		for (i=0; i<m_NbRings; i++)
		{
			for (j=0; j<(int)m_NbVerticesByRing[i]; j++)
			{
				float normx,normy,normz, x1,y1,z1,x2,y2,z2;
				
				x1=m_WallVertices[nbpoints*16+13]-m_WallVertices[nbpoints*16+5];
				y1=m_WallVertices[nbpoints*16+14]-m_WallVertices[nbpoints*16+6];
				z1=m_WallVertices[nbpoints*16+15]-m_WallVertices[nbpoints*16+7];
				x2=m_WallVertices[(nbpoints+nbWallVertices)*16+5]-m_WallVertices[nbpoints*16+5];
				y2=m_WallVertices[(nbpoints+nbWallVertices)*16+6]-m_WallVertices[nbpoints*16+6];
				z2=m_WallVertices[(nbpoints+nbWallVertices)*16+7]-m_WallVertices[nbpoints*16+7];
				normx=(y1*z2-z1*y2);
				normy=(z1*x2-x1*z2);
				normz=(x1*y2-y1*x2);

				float normlenght = sqrt(normx*normx+normy*normy+normz*normz);
				normx /= normlenght;
				normy /= normlenght;
				normz /= normlenght;

				m_WallVertices[nbpoints*16+2]=m_WallVertices[nbpoints*16+10]=m_WallVertices[(nbpoints+nbWallVertices)*16+2]=m_WallVertices[(nbpoints+nbWallVertices)*16+10]=normx;
				m_WallVertices[nbpoints*16+3]=m_WallVertices[nbpoints*16+11]=m_WallVertices[(nbpoints+nbWallVertices)*16+3]=m_WallVertices[(nbpoints+nbWallVertices)*16+11]=normy;
				m_WallVertices[nbpoints*16+4]=m_WallVertices[nbpoints*16+12]=m_WallVertices[(nbpoints+nbWallVertices)*16+4]=m_WallVertices[(nbpoints+nbWallVertices)*16+12]=normz;
			
				nbpoints++;
			}
		}
		
		
		ZArray<FP*>* footprints=new ZArray<FP*>();
		
		(*footprints)+=(FP*)this;
		float fHeight = m_altitude + m_height;
		int nFootPrint = 0;

		for (int num_roof=0; num_roof<m_Roofs->GetLength(); num_roof++)
		{
			FPRoof* roof = (*m_Roofs)[num_roof];
			
			try{
				roof->ComputeMultiRoof(footprints, fHeight);
			}
			catch(...)
			{
				roof->SetType(FLAT_ROOF);
				roof->ComputeMultiRoof(footprints, fHeight);
			}
			delete footprints;
			footprints = roof->GetNewFootPrint();
			fHeight += roof->GetHeight();
		}

		for (i=0; i<m_Facades->GetLength(); i++)
		{
			FPFacade* pFacade = (*m_Facades)[i];
			pFacade->InitFacade();

			if (pFacade->GetState() == STATE_READY)
				pFacade->ComputeFacade();
		}
	}

	m_State3D=COMPUTED;
}