/********************************************************************************/
/* This software module was developped by										*/

/*	   Jerome Royan (JR),														*/
/*																				*/

/* in the course of development of the MPEG-4 standard (Part 16 AFX).			*/
/* This software module is an implementation of a part of one or				*/
/* more MPEG-4 tools as specified by the										*/
/* MPEG-4 standard. ISO/IEC gives users of the MPEG-4 standard					*/
/* free license to this software module or modifications thereof				*/
/* for use in hardware or software products claiming conformance				*/
/* to the MPEG-4 standard. Those intending to use this software					*/
/* module in hardware or software products are advised that its					*/
/* use may infringe existing patents. The original developer of					*/
/* this software module and his company, the subsequent							*/
/* editors and their companies, and ISO/IEC have no liability					*/
/* for use of this software module or modifications thereof in					*/
/* an implementation. Copyright is not released for non							*/
/* MPEG-4 standard conforming products.  										*/
/* Copyright (c)2002 - 2005														*/
/********************************************************************************/

// FPDecoderLib.cpp: implementation of the FPDecoderLib class.
//////////////////////////////////////////////////////////////////////

#include "FPDecoderLib.h"

FPDecoderLib :: FPDecoderLib()
{
	m_pDecSpecificInfo=new FPSetDecoderSpecificInfo();
}

FPDecoderLib :: FPDecoderLib(FPSet* fpset)
{
	m_pDecSpecificInfo=new FPSetDecoderSpecificInfo();
	m_pFPSet=fpset;
}

FPDecoderLib :: ~FPDecoderLib()
{
	delete ((FPSetDecoderSpecificInfo*)m_pDecSpecificInfo);

}

BOOL FPDecoderLib :: Setup (AFXExtDescriptor *pSpecificInfo)
{
  m_pDecSpecificInfo->FPObjectType=((FPSetDecoderSpecificInfo*)pSpecificInfo)->FPObjectType;
  m_pDecSpecificInfo->MaxNbFootPrints=((FPSetDecoderSpecificInfo*)pSpecificInfo)->MaxNbFootPrints;
  m_pDecSpecificInfo->FootPrintNbBits=((FPSetDecoderSpecificInfo*)pSpecificInfo)->FootPrintNbBits;
  m_pDecSpecificInfo->Step=((FPSetDecoderSpecificInfo*)pSpecificInfo)->Step;
  m_pDecSpecificInfo->NbBitsMetricError=((FPSetDecoderSpecificInfo*)pSpecificInfo)->NbBitsMetricError;
  m_pDecSpecificInfo->MinX=((FPSetDecoderSpecificInfo*)pSpecificInfo)->MinX;
  m_pDecSpecificInfo->MaxX=((FPSetDecoderSpecificInfo*)pSpecificInfo)->MaxX;
  m_pDecSpecificInfo->MinY=((FPSetDecoderSpecificInfo*)pSpecificInfo)->MinY;
  m_pDecSpecificInfo->MaxY=((FPSetDecoderSpecificInfo*)pSpecificInfo)->MaxY;
  m_pDecSpecificInfo->DEFIdUsed=((FPSetDecoderSpecificInfo*)pSpecificInfo)->DEFIdUsed;
  m_pDecSpecificInfo->MinAltitude=((FPSetDecoderSpecificInfo*)pSpecificInfo)->MinAltitude;
  m_pDecSpecificInfo->MaxHeight=((FPSetDecoderSpecificInfo*)pSpecificInfo)->MaxHeight;
  m_pDecSpecificInfo->NbBitsZBuilding=((FPSetDecoderSpecificInfo*)pSpecificInfo)->NbBitsZBuilding;
  m_pDecSpecificInfo->NbBitsNbStories=((FPSetDecoderSpecificInfo*)pSpecificInfo)->NbBitsNbStories;
  m_pDecSpecificInfo->NbBitsStoryHeight=((FPSetDecoderSpecificInfo*)pSpecificInfo)->NbBitsStoryHeight;
  m_pDecSpecificInfo->NbBitsFacadeWidth=((FPSetDecoderSpecificInfo*)pSpecificInfo)->NbBitsFacadeWidth;
  m_pDecSpecificInfo->NbBitsNbFacadeCellsByStorey=((FPSetDecoderSpecificInfo*)pSpecificInfo)->NbBitsNbFacadeCellsByStorey;

  return TRUE;
}

BOOL FPDecoderLib :: DecodeFPSetStream (LPBYTE pInput, int nInputLength)
{
	mbitstream* bs=new mbitstream(pInput);

	DecodeFootPrintSetMessage(bs);

	m_pFPSet->AddActiveNode(m_pFPSet->Root());

	delete bs;
	return TRUE;
}

BOOL FPDecoderLib :: DecodeFromBuffer (LPBYTE pInput, int nInputLength)
{
	mbitstream* bs=new mbitstream(pInput);

	DecodeSpecificInfo(bs);
	DecodeFootPrintSetMessage(bs);
	m_pFPSet->AddActiveNode(m_pFPSet->Root());
	delete bs;
	return TRUE;
}

BOOL FPDecoderLib :: DecodeSpecificInfo(mbitstream* bs)
{
  unsigned int objectType, maxNbFootPrints, footPrintNbBits, nbBitsMetricError, nbBitsZBuilding, nbBitsNbStories, nbBitsStoreyHeight, nbBitsFacadeWidth, nbBitsNbFacadeCellsByStorey;
  float step, minX, maxX, minY, maxY, minAltitude, maxHeight;
  int defIdUsed;
  bs->readnbit(objectType,16);
  m_pDecSpecificInfo->FPObjectType=objectType;
  bs->readnbit(maxNbFootPrints,32);
  m_pDecSpecificInfo->MaxNbFootPrints=maxNbFootPrints;
  bs->readnbit(footPrintNbBits,6);
  m_pDecSpecificInfo->FootPrintNbBits=footPrintNbBits;
  bs->readnbit(step,32);
  m_pDecSpecificInfo->Step=step;
  bs->readnbit(nbBitsMetricError,6);
  m_pDecSpecificInfo->NbBitsMetricError=nbBitsMetricError;
  bs->readnbit(minX,32);
  m_pDecSpecificInfo->MinX=minX;
  bs->readnbit(maxX,32);
  m_pDecSpecificInfo->MaxX=maxX;
  bs->readnbit(minY,32);
  m_pDecSpecificInfo->MinY=minY;
  bs->readnbit(maxY,32);
  m_pDecSpecificInfo->MaxY=maxY;
  bs->readbit(defIdUsed);
  m_pDecSpecificInfo->DEFIdUsed=(bool)defIdUsed;
  if (objectType==1)
  {
	  bs->readnbit(minAltitude,32);
	  m_pDecSpecificInfo->MinAltitude=minAltitude;
	  bs->readnbit(maxHeight,32);
	  m_pDecSpecificInfo->MaxHeight=maxHeight;
	  bs->readnbit(nbBitsZBuilding,6);
	  m_pDecSpecificInfo->NbBitsZBuilding=nbBitsZBuilding;
	  bs->readnbit(nbBitsNbStories,6);
	  m_pDecSpecificInfo->NbBitsNbStories=nbBitsNbStories;
	  bs->readnbit(nbBitsStoreyHeight,6);
	  m_pDecSpecificInfo->NbBitsStoryHeight=nbBitsStoreyHeight;
	  bs->readnbit(nbBitsFacadeWidth,6);
	  m_pDecSpecificInfo->NbBitsFacadeWidth=nbBitsFacadeWidth;
	  bs->readnbit(nbBitsNbFacadeCellsByStorey,6);
	  m_pDecSpecificInfo->NbBitsNbFacadeCellsByStorey=nbBitsNbFacadeCellsByStorey;
  }
  return TRUE;
}

BOOL FPDecoderLib :: DecodeFootPrintSetMessage(mbitstream* bs)
{
	unsigned int NbFootPrints;
	bs->readnbit(NbFootPrints,32);

	for (unsigned int i=0; i<NbFootPrints; i++)
		DecodeFootPrintMessage(bs);
	return TRUE;	
}

BOOL FPDecoderLib :: DecodeFootPrintMessage(mbitstream* bs)
{
	unsigned int i, j;
	// FootPrint Parameters to decode
	unsigned int	index;
	bool			type;
	unsigned int	offspring;
	unsigned int	localIndex;
	float			MetricError;
	unsigned int	NbRings;
	unsigned int	FirstVertexIndex;

	// DSI Parameters useful for decoding
	int fpObjectType=m_pDecSpecificInfo->FPObjectType;
	float step=m_pDecSpecificInfo->Step;
	unsigned int footPrintNbBits=m_pDecSpecificInfo->FootPrintNbBits;

	// Creation of the proxy corresponding to the current decoded footprint
	bs->readnbit(index,footPrintNbBits);

	type=bs->readbit();

	unsigned int nbNewVertices;
	Vec2f* newVertices=NULL;
	
	DecodeFPNewVertices(bs, index, nbNewVertices, &newVertices);

	bs->readnbit(m_IndexNbBits,6);

	if (type)
	{
		bs->readnbit(offspring,10);

		// Get Parent Vertices
		FP* parent=m_pFPSet->GetFootPrint(index);
		Vec2f** parentFP=parent->FootPrint();
		unsigned int* parentNbVerticesByRing=parent->NbVerticesByRing();
		int parentNbRings=parent->NbRings();
		
		int nbvParent=0;
		for (i=0; i<parentNbRings; i++)
			nbvParent+=parentNbVerticesByRing[i];

		Vec2f* parentVertices=(Vec2f*)malloc(sizeof(Vec2f)*nbvParent);

		int numvParent=0;
		for (i=0; i<parentNbRings; i++)
			for (int j=0; j<parentNbVerticesByRing[i]; j++)
				parentVertices[numvParent++]=parentFP[i][j];

		// To store the decoded footprint
		ZArray<FP*>* decodedFP=new ZArray<FP*>();

		// For each refinement
		for (i=0; i<offspring; i++)
		{
			
			bs->readnbit(localIndex,m_pDecSpecificInfo->FootPrintNbBits);

			bs->readnbit(MetricError, m_pDecSpecificInfo->NbBitsMetricError);

			unsigned int* Indices;
			unsigned int nbIndices;
			unsigned int NbRings=0;
			DecodeIndexFootprintSet(bs, nbIndices, &Indices, NbRings);

			// Reconstruct footprint from : ParentVertices, NewVertices, Indices
			
			// Get Nb Vertices By Ring
			unsigned int* nbVerticesByRing=(unsigned int*)malloc(sizeof(unsigned int)*NbRings);
			unsigned int ringbegin=0;
			unsigned int numring=0;
			for (j=0; j<nbIndices; j++)
			{
				if (Indices[j]==0)
				{
					nbVerticesByRing[numring]=j-ringbegin;
					ringbegin=j+1;
					numring++;
				}
			}

			Vec2f** footprint=(Vec2f**)malloc(sizeof(Vec2f*)*NbRings);
			
			int temp=0;
			bool end=false;
			numring=0;
			while (!end)
			{
				Vec2f* ring=(Vec2f*)malloc(sizeof(Vec2f)*nbVerticesByRing[numring]);
				int numv=0;
				for (int ind=temp; ind<nbIndices; ind++)
				{
					if(Indices[ind]!=0)
					{
						if (Indices[ind]-1<nbvParent)
							ring[numv++]=parentVertices[Indices[ind]-1];
						else
							ring[numv++]=newVertices[Indices[ind]-1-nbvParent];
					}
					else
					{
						temp=ind+1;
						break;
					}
				}	
				if (ind+1>=nbIndices)
					end=true;
				footprint[numring++]=ring;
			}
			
			// Decode specific parameters corresponding to FPObjectType
			switch (fpObjectType)
			{
				case 0: // Footprint
					{
						FP* fp=new FP();
						fp->SetParent(parent);
						fp->SetNbRings(NbRings);
						fp->SetFootPrint(footprint);
						fp->SetNbVerticesByRing(nbVerticesByRing);
						fp->SetIndex(localIndex);
						fp->SetMetricError(MetricError);
						
						decodedFP->InsertAt(i,(FP*)fp);
						break;
					}
				case 1: // Building with Height, Altitude, Facades, and Roofs
					{
						FPBuilding* fpb=new FPBuilding();
						fpb->SetParent(parent);
						fpb->SetFootPrint(footprint);
						fpb->SetNbRings(NbRings);
						fpb->SetNbVerticesByRing(nbVerticesByRing);
						fpb->SetIndex(localIndex);
						fpb->SetMetricError(MetricError);
						fpb->SetState3D(NOCOMPUTED);
						
						DecodeFPBuildingParameters(bs, fpb);
						DecodeFPSwapNodes(bs,fpb);
						
						decodedFP->InsertAt(i,(FP*)fpb);
						break;
					}
				default:
						break;
			}
			free(Indices);
		}
		m_pFPSet->AddFootPrints(*decodedFP,index);
	}
	else
	{
		bs->readnbit(MetricError,m_pDecSpecificInfo->NbBitsMetricError);

		bs->readnbit(NbRings,8);

		// Decode first vertex (in NewVertices) for each ring
		int numv=0;
		unsigned int* nbVerticesByRing=(unsigned int*)malloc(NbRings*sizeof(unsigned int));
		for (i=0; i<NbRings-1; i++)
		{
			unsigned int FirstVertexIndices;
			bs->readnbit(FirstVertexIndices,m_IndexNbBits);
			nbVerticesByRing[i]=FirstVertexIndices-numv;
			numv=FirstVertexIndices;
		}
		nbVerticesByRing[NbRings-1]=nbNewVertices-numv;

		// Reconstruct the footprint from NewVertices and nbVerticesByRing
		Vec2f** footprint=(Vec2f**)malloc(sizeof(Vec2f*)*NbRings);
		numv=0;
		for (i=0; i<NbRings; i++)
		{
			footprint[i]=(Vec2f*)malloc(sizeof(Vec2f)*nbVerticesByRing[i]);
			for (j=0; j<nbVerticesByRing[i]; j++)
			{
				footprint[i][j]=newVertices[numv+j];
			}
			numv+=nbVerticesByRing[i];
		}

		// Decode specific parameters corresponding to FPObjectType
		switch (fpObjectType)
		{
			case 0: // Footprint
				{
					FP* fp=new FP();
					fp->SetParent(m_pFPSet->Root());
					fp->SetFootPrint(footprint);
					fp->SetNbRings(NbRings);
					fp->SetNbVerticesByRing(nbVerticesByRing);
					fp->SetIndex(index);
					fp->SetMetricError(MetricError);
					m_pFPSet->AddFootPrint(fp,-1);
					break;
				}
			case 1: // Building with Height, Altitude, Facades, and Roofs
				{
					FPBuilding* fpb=new FPBuilding();
					fpb->SetParent(m_pFPSet->Root());
					fpb->SetFootPrint(footprint);
					fpb->SetNbRings(NbRings);
					fpb->SetNbVerticesByRing(nbVerticesByRing);
					fpb->SetIndex(index);
					fpb->SetMetricError(MetricError);
					fpb->SetState3D(NOCOMPUTED);
					DecodeFPBuildingParameters(bs, fpb);
					DecodeFPSwapNodes(bs,fpb);
					m_pFPSet->AddFootPrint((FP*)fpb,-1);
					break;
				}
			default:
					break;
		}
	}


	if (newVertices!=NULL) 
		free(newVertices);

	return TRUE;	
}

BOOL FPDecoderLib :: DecodeFPNewVertices(mbitstream* bs, int index, unsigned int& nbNewVertices, Vec2f** newVertices)
{
	unsigned int i;
	unsigned int coordtype;

	FP* parent=m_pFPSet->GetFootPrint(index);
	
	float step=m_pDecSpecificInfo->Step;

	// Get Parent Barycentre
	Vec2f* gc;
	if (parent!=NULL)
	{	
		gc=parent->GravityCenter();
		gc->x=(float)(int)gc->x;
		gc->y=(float)(int)gc->y;
	}
	bs->readnbit(coordtype,6);
	bs->readnbit(nbNewVertices,16);

	if (nbNewVertices!=0)
		*newVertices=(Vec2f*)malloc(sizeof(Vec2f)*nbNewVertices);

	for (i=0; i<nbNewVertices; i++)
	{
		
		if (step==-1.0 || parent==NULL)
		{
			float x,y;
			bs->readnbit(x,(int)coordtype);
			bs->readnbit(y,(int)coordtype);
			(*newVertices)[i]=Vec2f(x,y);
		}
		else
		{
			if (coordtype!=0)
			{
				unsigned int x_temp,y_temp;
				int x,y;
				bool isPositive=bs->readbit();
				bs->readnbit(x_temp,coordtype-1);
				if (!isPositive) 
					x=(int)x_temp*(-1);
				else
					x=x_temp;
				isPositive=bs->readbit();
				bs->readnbit(y_temp,coordtype-1);
				if (!isPositive) 
					y=(int)y_temp*(-1);
				else
					y=y_temp;
				(*newVertices)[i]=Vec2f((((float)x*step)+gc->x), (((float)y*step)+gc->y));
			}
			else
			{
				(*newVertices)[i]=Vec2f(gc->x,gc->y);
			}
		}
	}

	return TRUE;	
}

BOOL FPDecoderLib :: DecodeIndexFootprintSet(mbitstream* bs, unsigned int& nbindices, unsigned int** indices, unsigned int& NbRings)
{
	bs->readnbit(nbindices,16);

	(*indices)=(unsigned int*)malloc(nbindices*sizeof(unsigned int));

	NbRings=0;
	for (unsigned int j=0; j<nbindices; j++)
	{
		unsigned int index;
		bs->readnbit(index,m_IndexNbBits);
		(*indices)[j]=(int)index;
		if (index==0)
			NbRings++;
	}
	return TRUE;	
}

BOOL FPDecoderLib :: DecodeFPBuildingParameters(mbitstream* bs, FPBuilding* pBuilding)
{
	unsigned short	buildingIndex;
	float			altitude;
	float			height;

	bs->readnbit(buildingIndex,m_pDecSpecificInfo->FootPrintNbBits);

	if (m_pDecSpecificInfo->Step==-1.0)
	{
		bs->readnbit(altitude,32);
		bs->readnbit(height,32);
	}
	else
	{
		unsigned short encodedalt, encodedheight;
		bs->readnbit(encodedalt,m_pDecSpecificInfo->NbBitsZBuilding);
		altitude=encodedalt*m_pDecSpecificInfo->Step;

		bs->readnbit(encodedheight,m_pDecSpecificInfo->NbBitsZBuilding);
		height=encodedheight*m_pDecSpecificInfo->Step;
	}

	pBuilding->SetAltitude(altitude);
	pBuilding->SetHeight(height);

	// Facades Decoding
	DecodeFacades(bs, pBuilding);

	// Roof Decoding
	DecodeRoofs(bs,pBuilding);

	return TRUE;	
}

BOOL FPDecoderLib :: DecodeFacades(mbitstream* bs, FPBuilding* pBuilding)
{
	unsigned int i, j;
	
	Vec2f**			footprint=pBuilding->FootPrint();
	unsigned int*	nbVerticesByRing=pBuilding->NbVerticesByRing();
	int				NbRings=pBuilding->NbRings();
	ZArray<FPFacade* >* facades=new ZArray<FPFacade* >();
	
	float altitude=pBuilding->Altitude();
	float height=pBuilding->Height();
	//unsigned int ivtx=0;
	
	// For each edge of the footprint
	for (i=0; i<NbRings; i++)
	{
		Vec2f* ring=footprint[i];
		unsigned int nbVerticesInRing=nbVerticesByRing[i];
		for (j=0; j<nbVerticesInRing; j++)
		{
			// Initialize facade by determining its bounding box in 2 dimension
			Vec2f p1=ring[j];
			Vec2f p2=ring[(j+1)%nbVerticesInRing];

			float x1, y1, z1;	// coord of first footprint point
			float x2, y2, z2;	// coord of second footprint point

			x1 = p1.x;
			y1 = p1.y;
			z1 = altitude;

			x2 = p2.x;
			y2 = p2.y;
			z2 = altitude;

			float xfacade = x1-x2;
			float yfacade = y1-y2;
			float zfacade = 0;

			float width = sqrt((x1-x2)*(x1-x2) + (y1-y2)*(y1-y2));
			xfacade /= width;
			yfacade /= width;
				
			FPFacade* facade = new FPFacade(x2, y2, z2, xfacade, yfacade, zfacade, width, height);
			DecodeFPFacade(bs, facade);
			(*facades) += facade;

			//ivtx++;
		}
	}

	pBuilding->SetFacades(facades);
	return TRUE;	
}

BOOL FPDecoderLib :: DecodeFPFacade(mbitstream* bs, FPFacade* pFacade)
{
	unsigned int j;

	unsigned int width;
	bs->readnbit(width,m_pDecSpecificInfo->NbBitsFacadeWidth);
	pFacade->SetWidthRatio(width/(pow(2,m_pDecSpecificInfo->NbBitsFacadeWidth)-1));

	unsigned int mappingmode;
	bs->readnbit(mappingmode, 2);

	float XScale=1.0;
	float YScale=1.0; 
	float XPosition=0.0; 
	float YPosition=0.0; 
	float RepeatIntervalX=0.0; 
	float RepeatIntervalY=0.0;
	
	switch(mappingmode)
	{
	case 0:
		
		break;
	case 1:
		bs->readnbit(XScale, 32); 
		bs->readnbit(YScale, 32);
		break;
	case 2:
		bs->readnbit(XScale, 32);
		bs->readnbit(YScale, 32);
		bs->readnbit(XPosition, 32);
		bs->readnbit(YPosition, 32);
		break;
	case 3:
		bs->readnbit(XScale, 32);
		bs->readnbit(YScale, 32);
		bs->readnbit(XPosition, 32);
		bs->readnbit(YPosition, 32);
		bs->readnbit(RepeatIntervalX, 32);
		bs->readnbit(RepeatIntervalY, 32);
		break;
	}
	pFacade->SetXScale(XScale);
	pFacade->SetYScale(YScale);
	pFacade->SetXPosition(XPosition);
	pFacade->SetYPosition(YPosition);
	pFacade->SetXRepeat(RepeatIntervalX);
	pFacade->SetYRepeat(RepeatIntervalY);

	bool bRepeat;
	bRepeat = bs->readbit();
	pFacade->SetRepeat(bRepeat);

	unsigned int FacadePrimitiveType;
	bs->readnbit(FacadePrimitiveType, 2);
	pFacade->SetPrimitiveType(FacadePrimitiveType);

	// URL
	unsigned int nbUrlLen;
	bs->readnbit(nbUrlLen, 8);

	char Url[50];
	memset(Url, 0, sizeof(Url));
	for (j=0; j<nbUrlLen; j++)
	{
		unsigned short UrlChar;
		bs->readnbit(UrlChar, 8);
		Url[j] = UrlChar;
	}
	pFacade->SetURL((char*)Url, nbUrlLen);
	
	unsigned int nbStories;
	bs->readnbit(nbStories, 8);

	if (nbStories > 0)
	{
		ZArray<float>* ArrayStoreyHeight = new ZArray<float>();
		
		for (j=0; j<(unsigned int)nbStories; j++)
		{
			unsigned int StoreyHeight;
			bs->readnbit(StoreyHeight, m_pDecSpecificInfo->NbBitsStoryHeight);
			float heightratio = (float)StoreyHeight/(pow(2, 8)-1);
			(*ArrayStoreyHeight)+=heightratio;
		}
		pFacade->SetStoriesHeight(ArrayStoreyHeight);

		// elements
		ZArray<ZArray<FPFacade*>* >* FacadeElements = new ZArray<ZArray<FPFacade*>* >;
		float posV=0;
		for (j=0; j<(unsigned int)nbStories; j++)
		{
			ZArray<FPFacade*>* StoreyElements = new ZArray<FPFacade*>();

			unsigned int nbFacadeCellsByStorey;
			bs->readnbit(nbFacadeCellsByStorey, m_pDecSpecificInfo->NbBitsNbFacadeCellsByStorey);

			float posH=0;	// horizontal position
			for (unsigned int k=0; k<nbFacadeCellsByStorey; k++)
			{
				//FPFacade* element = new FPFacade();
				FPFacade* element = new FPFacade();
				
				// initial position
				element->SetX(pFacade->GetX());
				element->SetY(pFacade->GetY());
				element->SetZ(pFacade->GetZ());
				// vector facade
				element->SetXFacade(pFacade->GetXFacade());
				element->SetYFacade(pFacade->GetYFacade());
				element->SetZFacade(pFacade->GetZFacade());
				
				// initial full width
				element->SetParentWidth(pFacade->GetWidth());
				// shift right
				element->ShiftRight(posH);
				// rescale width ??
				if (k == nbFacadeCellsByStorey-1)
				{	
					// for last element, force width
					element->SetWidthRatio(1-posH);
				}
				
				// initial full height
				element->SetParentHeight(pFacade->GetHeight());
				// shift up
				element->ShiftUp(posV);
				// rescale height
				if (j == nbStories-1)
				{	
					// for last element, force height
					element->SetHeightRatio(1-posV);
				}
				else
				{
					element->SetHeightRatio(ArrayStoreyHeight->ElementAt(j));
				}
				
				DecodeFPFacade(bs, element);
				posH+=element->GetWidthRatio();

				(*StoreyElements) += element;
			}
				
			posV += ArrayStoreyHeight->ElementAt(j);
			(*FacadeElements) += StoreyElements;
		}
		pFacade->SetElements(FacadeElements);
	}

	return true;
}


BOOL FPDecoderLib :: DecodeRoofs(mbitstream* bs, FPBuilding* pBuilding)
{
	unsigned int nbRoofs;
	bs->readnbit(nbRoofs,6);

	ZArray<FPRoof*>* roofs=new ZArray<FPRoof*>();

	for (int numroof=0; numroof<(int)nbRoofs; numroof++)
	{
		unsigned int typeRoof=0;
		bs->readnbit(typeRoof,3);
		
		float slopeAngle, roofHeight, eaveProjection;
		unsigned int support, textureindex;


		if (typeRoof!=0)
		{
			unsigned int support;
			bs->readnbit(slopeAngle,32);
			bs->readnbit(roofHeight,32);
			bs->readnbit(eaveProjection,32);
			if (typeRoof==3)
			{
				bs->readnbit(support,m_IndexNbBits);
			}
			else
				support=-1;

			//bs->readnbit(textureindex,32);
		}
		else
		{
			slopeAngle=-1.0;
			roofHeight=-1.0;
			eaveProjection=-1.0;
			support=-1;
			textureindex=-1;

		}

		FPRoof* roof;
		switch (typeRoof) {
		case 0 : roof=new FPRoof(FLAT_ROOF,slopeAngle, roofHeight, eaveProjection, support);
				 break;
		case 1 : roof=new FPRoof(HIP_ROOF,slopeAngle, roofHeight, eaveProjection, support);
				 break;
		case 2 : roof=new FPRoof(GABLE_ROOF,slopeAngle, roofHeight, eaveProjection, support);
				 break;
		case 3 : roof=new FPRoof(SALT_BOX_ROOF,slopeAngle, roofHeight, eaveProjection, support);
				 break;
		default : roof=new FPRoof(FLAT_ROOF,-1.0,-1.0,-1,-1);
		}

		unsigned int nbUrlLen;
		bs->readnbit(nbUrlLen, 8);

		char Url[50];
		memset(Url, 0, sizeof(Url));
		for (unsigned int j=0; j<nbUrlLen; j++)
		{
			unsigned short UrlChar;
			bs->readnbit(UrlChar, 8);
			Url[j] = UrlChar;
		}
		
		roof->SetTexURL((char*)Url, nbUrlLen);
		
		bool texIsGeneric=bs->readbit();
		roof->SetTexIsGeneric(texIsGeneric);

		unsigned short texMode;
		bs->readnbit(texMode,2);
		float texXScale=1.0;
		float texYScale=1.0;
		float texXPosition=0.0;
		float texYPosition=0.0;
		float texRotation=0.0;
		
		switch(texMode)
		{
		case 0:
			break;
		case 1:
			bs->readnbit(texXScale,32);
			bs->readnbit(texYScale,32);
			break;
		case 2:
			bs->readnbit(texXScale,32);
	
			bs->readnbit(texYScale,32);
			bs->readnbit(texXPosition,32);
			bs->readnbit(texYPosition,32);
			bs->readnbit(texRotation,32);
			break;
		}
		roof->SetTexXScale(texXScale);
		roof->SetTexYScale(texYScale);
		roof->SetTexXPosition(texXPosition);
		roof->SetTexYPosition(texYPosition);
		roof->SetTexRotation(texRotation);

		(*roofs)+=roof;
	}
	pBuilding->SetRoofs(roofs);

	return TRUE;	
}

BOOL FPDecoderLib :: DecodeFPSwapNodes(mbitstream* bs, FPBuilding* pBuilding)
{
	unsigned int nbSwapNodes;
	
	bs->readnbit(nbSwapNodes,8);

	if (nbSwapNodes!=0)
	{
		for (unsigned int numSwapNodes=0; numSwapNodes<nbSwapNodes; numSwapNodes++)
		{
			unsigned int nbcharSwapNode;
			bs->readnbit(nbcharSwapNode,8);
			ZString swapNode;
			for (unsigned int numchar=0; numchar<nbcharSwapNode; numchar++)
			{
				char charSwapNode;
				unsigned int temp;
				bs->readnbit(temp,8);
				memcpy(&charSwapNode,&temp,1);
				swapNode+=charSwapNode;
			}
			pBuilding->AddSwapNode(swapNode);
		}
	}
	return TRUE;
}

BOOL FPDecoderLib :: DecodeFPSetRefinement(mbitstream* bs)
{
	//FPE_LONG temp;
	unsigned int i,j;//, indexNbBits, index, offspring, coord_type, nbNewVertices;
	unsigned int index, offspring, coord_type, nbNewVertices;
	bool type;
	Vec2f* newVertices;
	

	float accuracy=m_pDecSpecificInfo->Step;
	
	//index=(int)bs->Readbits(m_pDecSpecificInfo->FootPrintNbBits);
 	bs->readnbit(index,m_pDecSpecificInfo->FootPrintNbBits);
	//type=(bs->Readbit()==1)?true:false;
	type=bs->readbit();

	if (type)
	{
		FP* parent=m_pFPSet->GetFootPrint(index);
		if (parent==NULL)
			return false;
		Vec2f* gc=parent->GravityCenter();
		gc->x=(float)(int)gc->x;
		gc->y=(float)(int)gc->y;
		
		//coord_type=(int)bs->Readbits(6);
		bs->readnbit(coord_type,6);
		//nbNewVertices=(int)bs->Readbits(16);
		bs->readnbit(nbNewVertices,16);

		if (nbNewVertices!=0)
			newVertices=(Vec2f*)malloc(sizeof(Vec2f)*nbNewVertices);

		for (i=0; i<nbNewVertices; i++)
		{
			
			if (accuracy==-1.0)
			{
				float x,y;
				bs->readnbit(x,(int)coord_type);
				//temp=bs->Readbits(coord_type);
				//memcpy(&x, &temp, 4);
				//temp=bs->Readbits(coord_type);
				//memcpy(&y, &temp, 4);
				bs->readnbit(y,(int)coord_type);
				newVertices[i]=Vec2f(x,y);
			}
			else
			{
				if (coord_type!=0)
				{
					unsigned int x_temp,y_temp;
					int x,y;
					bool isPositive=bs->readbit();
					bs->readnbit(x_temp,coord_type-1);
					if (!isPositive) 
						x=(int)x_temp*(-1);
					else
						x=x_temp;
					isPositive=bs->readbit();
					bs->readnbit(y_temp,coord_type-1);
					if (!isPositive) 
						y=(int)y_temp*(-1);
					else
						y=y_temp;
					newVertices[i]=Vec2f((((float)x*accuracy)+gc->x), (((float)y*accuracy)+gc->y));
				}
				else
				{
					newVertices[i]=Vec2f(gc->x,gc->y);
				}
			}
		}

		bs->readnbit(m_IndexNbBits,6);

		ZArray<FP*>* elevations=new ZArray<FP*>();


		Vec2f** parentFP=parent->FootPrint();
		unsigned int* parentNbVerticesByRing=parent->NbVerticesByRing();
		int parentNbRings=parent->NbRings();
		
		int nbvParent=0;
		for (i=0; i<parentNbRings; i++)
			nbvParent+=parentNbVerticesByRing[i];

		Vec2f* parentVertices=(Vec2f*)malloc(sizeof(Vec2f)*nbvParent);

		int numvParent=0;
		for (i=0; i<parentNbRings; i++)
			for (int j=0; j<parentNbVerticesByRing[i]; j++)
				parentVertices[numvParent++]=parentFP[i][j];

		bs->readnbit(offspring,10);

		for (i=0; i<offspring; i++)
		{
			unsigned int localIndex, nbVertexIndices,*indices, nbSwapNodes, *nodeId;
			float metricError, altitude, height;

			bs->readnbit(localIndex,m_pDecSpecificInfo->FootPrintNbBits);

			bs->readnbit(metricError, m_pDecSpecificInfo->NbBitsMetricError);	
			
			bs->readnbit(nbVertexIndices,16);

			indices=(unsigned int*)malloc(nbVertexIndices*sizeof(unsigned int));


			int NbRings=0;
			for (j=0; j<nbVertexIndices; j++)
			{
				unsigned int index;
				bs->readnbit(index,m_IndexNbBits);
				indices[j]=(int)index;
				if (index==0)
					NbRings++;
			}
		
			unsigned int* nbVerticesByRing=(unsigned int*)malloc(sizeof(unsigned int)*NbRings);
			unsigned int ringbegin=0;
			unsigned int numring=0;
			for (j=0; j<nbVertexIndices; j++)
			{
				if (indices[j]==0)
				{
					nbVerticesByRing[numring]=j-ringbegin;
					ringbegin=j+1;
					numring++;
				}
			}

			unsigned short buildingIndex;
			bs->readnbit(buildingIndex,m_pDecSpecificInfo->FootPrintNbBits);

			if (m_pDecSpecificInfo->Step==-1.0)
			{
				bs->readnbit(altitude,32);
				bs->readnbit(height,32);
			}
			else
			{
				unsigned short encodedalt, encodedheight;
				bs->readnbit(encodedalt,m_pDecSpecificInfo->NbBitsZBuilding);
				altitude=encodedalt*m_pDecSpecificInfo->Step;

				bs->readnbit(encodedheight,m_pDecSpecificInfo->NbBitsZBuilding);
				height=encodedheight*m_pDecSpecificInfo->Step;
			}
			
			FPBuilding* fpb=new FPBuilding();
			fpb->SetIndex(localIndex);
			fpb->SetParent(parent);
			fpb->SetState3D(NOCOMPUTED);
			fpb->SetMetricError(metricError);
			
			Vec2f** footprint=(Vec2f**)malloc(sizeof(Vec2f*)*NbRings);
			
			int temp=0;
			bool end=false;
			numring=0;
			while (!end)
			{
				Vec2f* ring=(Vec2f*)malloc(sizeof(Vec2f)*nbVerticesByRing[numring]);
				int numv=0;
				for (int ind=temp; ind<nbVertexIndices; ind++)
				{
					if(indices[ind]!=0)
					{
						if (indices[ind]-1<nbvParent)
							ring[numv++]=parentVertices[indices[ind]-1];
						else
							ring[numv++]=newVertices[indices[ind]-1-nbvParent];
					}
					else
					{
						temp=ind+1;
						break;
					}
				}	
				if (ind+1>=nbVertexIndices)
					end=true;
				footprint[numring++]=ring;
			}
			fpb->SetFootPrint(footprint);
			fpb->SetNbRings(NbRings);
			fpb->SetNbVerticesByRing(nbVerticesByRing);
			fpb->SetNbRings(NbRings);
			fpb->SetAltitude(altitude);
			fpb->SetHeight(height);
			elevations->InsertAt(i,(FP*)fpb);
			free(indices);

			// Decoder Facades Anne Jerome
			unsigned int ivtx=0;
			ZArray<FPFacade*>* facades=new ZArray<FPFacade*>();
			for (j=0; j<NbRings; j++)
			{
				Vec2f* ring=footprint[j];
				unsigned int nbVerticesInRing=nbVerticesByRing[j];
				for (unsigned int k=0; k<nbVerticesInRing; k++)
				{
					
					Vec2f p1=ring[k];
					Vec2f p2=ring[(k+1)%nbVerticesInRing];

					float x1, y1, z1;	// coord of first footprint point
					float x2, y2, z2;	// coord of second footprint point

					x1 = p1.x;
					y1 = p1.y;
					z1 = altitude;

					x2 = p2.x;
					y2 = p2.y;
					z2 = altitude;

					/*if (iVtx==nbVerticesByRing[numring])
					{
						x1 = newVertices[ivtx].x;
						y1 = newVertices[ivtx].y;
						z1 = altitude;

						x2 = newVertices[ivtx+1].x;
						y2 = newVertices[ivtx+1].y;
						z2 = altitude;
					}
					else*/
					/*
					{
						x1 = newVertices[ivtx].x;
						y1 = newVertices[ivtx].y;
						z1 = altitude;  

						x2 = newVertices[ivtx+1].x;
						y2 = newVertices[ivtx+1].y;
						z2 = altitude;  
					}
					*/
					// carefull: footprint is clowise, this means
					// x1 = right bottom point of facade
					// x2 = left bottom point of facade
					float xfacade = x1-x2;
					float yfacade = y1-y2;
					float zfacade = 0;

					float width = sqrt((x1-x2)*(x1-x2) + (y1-y2)*(y1-y2));
					xfacade /= width;
					yfacade /= width;
						
					FPFacade* facade = new FPFacade(x2, y2, z2, xfacade, yfacade, zfacade, width, height);
					DecodeFPFacade(bs, facade);
					(*facades) += facade;

					ivtx++;
				}
			}
			fpb->SetFacades(facades);

			// Fin Facades Anne Jerome
			

			unsigned int nbRoofs;
			bs->readnbit(nbRoofs,6);

			ZArray<FPRoof*>* roofs=new ZArray<FPRoof*>();

			for (int numroof=0; numroof<(int)nbRoofs; numroof++)
			{
				unsigned int typeRoof=0;
				bs->readnbit(typeRoof,3);
				
				float slopeAngle, roofHeight, eaveProjection;
				unsigned int support, textureindex;


				if (typeRoof!=0)
				{
					unsigned int support;
					bs->readnbit(slopeAngle,32);
					bs->readnbit(roofHeight,32);
					bs->readnbit(eaveProjection,32);
					if (typeRoof==3)
					{
						bs->readnbit(support,m_IndexNbBits);
					}
					else
						support=-1;

					//bs->readnbit(textureindex,32);
				}
				else
				{
					slopeAngle=-1.0;
					roofHeight=-1.0;
					eaveProjection=-1.0;
					support=-1;
					textureindex=-1;

				}

				FPRoof* roof;
				switch (typeRoof) {
				case 0 : roof=new FPRoof(FLAT_ROOF,slopeAngle, roofHeight, eaveProjection, support);
						 break;
				case 1 : roof=new FPRoof(HIP_ROOF,slopeAngle, roofHeight, eaveProjection, support);
						 break;
				case 2 : roof=new FPRoof(GABLE_ROOF,slopeAngle, roofHeight, eaveProjection, support);
					     break;
				case 3 : roof=new FPRoof(SALT_BOX_ROOF,slopeAngle, roofHeight, eaveProjection, support);
					     break;
				default : roof=new FPRoof(FLAT_ROOF,-1.0,-1.0,-1,-1);
				}

				unsigned int nbUrlLen;
				bs->readnbit(nbUrlLen, 8);

				char Url[50];
				memset(Url, 0, sizeof(Url));
				for (j=0; j<nbUrlLen; j++)
				{
					unsigned short UrlChar;
					bs->readnbit(UrlChar, 8);
					Url[j] = UrlChar;
				}
				
				roof->SetTexURL((char*)Url, nbUrlLen);
				
				bool texIsGeneric=bs->readbit();
				roof->SetTexIsGeneric(texIsGeneric);

				unsigned short texMode;
				bs->readnbit(texMode,2);
				float texXScale=1.0;
				float texYScale=1.0;
				float texXPosition=0.0;
				float texYPosition=0.0;
				float texRotation=0.0;
				
				switch(texMode)
				{
				case 0:
					break;
				case 1:
					bs->readnbit(texXScale,32);
					bs->readnbit(texYScale,32);
					break;
				case 2:
					bs->readnbit(texXScale,32);
					bs->readnbit(texYScale,32);
					bs->readnbit(texXPosition,32);
					bs->readnbit(texYPosition,32);
					bs->readnbit(texRotation,32);
					break;
				}
				roof->SetTexXScale(texXScale);
				roof->SetTexYScale(texYScale);
				roof->SetTexXPosition(texXPosition);
				roof->SetTexYPosition(texYPosition);
				roof->SetTexRotation(texRotation);

				(*roofs)+=roof;
			}
			fpb->SetRoofs(roofs);


			bs->readnbit(nbSwapNodes,8);

			if (nbSwapNodes!=0)
			{
				for (unsigned int numSwapNodes=0; numSwapNodes<nbSwapNodes; numSwapNodes++)
				{
					unsigned int nbcharSwapNode;
					bs->readnbit(nbcharSwapNode,8);
					ZString swapNode;
					for (unsigned int numchar=0; numchar<nbcharSwapNode; numchar++)
					{
						char charSwapNode;
						unsigned int temp;
						bs->readnbit(temp,8);
						memcpy(&charSwapNode,&temp,1);
						swapNode+=charSwapNode;
					}
					fpb->AddSwapNode(swapNode);
				}
			}
		}
		
		m_pFPSet->AddFootPrints(*elevations,index);
		if (nbNewVertices!=0)
			free(newVertices);
		free(parentVertices);
		
		
	}
	else
	{
		//int nbSwapNodes, *nodeId;
		unsigned int nbSwapNodes, *nodeId, NbRings;
		float metricError, altitude, height;
		
		//coord_type=(int)bs->Readbits(6);
		bs->readnbit(coord_type,6);

		//nbNewVertices=(int)bs->Readbits(16);
		bs->readnbit(nbNewVertices,16);

		if (nbNewVertices!=0)
			newVertices=(Vec2f*)malloc(sizeof(Vec2f)*2*nbNewVertices);
		else
			newVertices=NULL;

		for (i=0; i<nbNewVertices; i++)
		{
			float x,y;
			//temp=bs->Readbits(coord_type);
			//memcpy(&x, &temp, 4);
			//temp=bs->Readbits(coord_type);
			//memcpy(&y, &temp, 4);
			bs->readnbit(x,coord_type);
			bs->readnbit(y,coord_type);
			newVertices[i]=Vec2f(x,y);
		}
		
		//indexNbBits=(int)bs->Readbits(16);
		bs->readnbit(m_IndexNbBits,6);

		//temp=bs->Readbits(32);
		//memcpy(&metricError, &temp, 4);
		bs->readnbit(metricError,m_pDecSpecificInfo->NbBitsMetricError);
	
		//int NbRings=(int)bs->Readbits(8);
		bs->readnbit(NbRings,8);

		int numv=0;
		unsigned int* nbVerticesByRing=(unsigned int*)malloc(NbRings*sizeof(unsigned int));
		for (i=0; i<NbRings-1; i++)
		{
			unsigned int numring;
			bs->readnbit(numring,m_IndexNbBits);
			nbVerticesByRing[i]=numring-numv;
			numv=numring;
		}
		nbVerticesByRing[NbRings-1]=nbNewVertices-numv;
	
		unsigned short buildingIndex;
		bs->readnbit(buildingIndex,m_pDecSpecificInfo->FootPrintNbBits);

		if (m_pDecSpecificInfo->Step==-1.0)
		{
			bs->readnbit(altitude,32);
			bs->readnbit(height,32);
		}
		else
		{
			unsigned short encodedalt, encodedheight;
			bs->readnbit(encodedalt,m_pDecSpecificInfo->NbBitsZBuilding);
			altitude=encodedalt*m_pDecSpecificInfo->Step;

			bs->readnbit(encodedheight,m_pDecSpecificInfo->NbBitsZBuilding);
			height=encodedheight*m_pDecSpecificInfo->Step;
		}


		// Decoder Facades Anne Jerome
		unsigned int ivtx=0;
		ZArray<FPFacade* >* facades=new ZArray<FPFacade*>();
		for (j=0; j<NbRings; j++)
		{
			for (unsigned int k=0; k<nbVerticesByRing[j]; k++)
			{
				float x1, y1, z1;	// coord of first footprint point
				float x2, y2, z2;	// coord of second footprint point

				if (k==nbVerticesByRing[j]-1)
				{
					x1 = newVertices[ivtx].x;
					y1 = newVertices[ivtx].y;
					z1 = altitude;

					x2 = newVertices[ivtx+1-nbVerticesByRing[j]].x;
					y2 = newVertices[ivtx+1-nbVerticesByRing[j]].y;
					z2 = altitude;
				}
				else
				{
					x1 = newVertices[ivtx].x;

					y1 = newVertices[ivtx].y;
					z1 = altitude;  

					x2 = newVertices[ivtx+1].x;
					y2 = newVertices[ivtx+1].y;
					z2 = altitude;  
				}

				// carefull: footprint is clowise, this means
				// x1 = right bottom point of facade
				// x2 = left bottom point of facade
				float xfacade = x1-x2;
				float yfacade = y1-y2;
				float zfacade = 0;

				float width = sqrt((x1-x2)*(x1-x2) + (y1-y2)*(y1-y2));
				xfacade /= width;
				yfacade /= width;

					
				FPFacade* facade = new FPFacade(x2, y2, z2, xfacade, yfacade, zfacade, width, height);
				DecodeFPFacade(bs, facade);
				(*facades) += facade;

				ivtx++;
			}
		}
		// Fin Facades Anne Jerome

		unsigned int nbRoofs;
		bs->readnbit(nbRoofs,6);

		ZArray<FPRoof*>* roofs=new ZArray<FPRoof*>();
		
		for (i=0; i<nbRoofs; i++)
		{
			unsigned int typeRoof;
			bs->readnbit(typeRoof,3);
			
			float slopeAngle, roofHeight, eaveProjection;
			unsigned int support, textureindex;


			if (typeRoof!=0)
			{
				unsigned int support;
				bs->readnbit(slopeAngle,32);
				bs->readnbit(roofHeight,32);
				bs->readnbit(eaveProjection,32);
				if (typeRoof==3)
				{
					bs->readnbit(support,m_IndexNbBits);
				}
				else
					support=-1;

				//bs->readnbit(textureindex,32);
			}
			else
			{
				slopeAngle=-1.0;
				roofHeight=-1.0;
				eaveProjection=-1.0;
				support=-1;
				textureindex=-1;

			}
			FPRoof* roof;
			switch (typeRoof) {
			case 0 : roof=new FPRoof(FLAT_ROOF,slopeAngle, roofHeight, eaveProjection, support);
					 break;
			case 1 : roof=new FPRoof(HIP_ROOF,slopeAngle, roofHeight, eaveProjection, support);
					 break;
			case 2 : roof=new FPRoof(GABLE_ROOF,slopeAngle, roofHeight, eaveProjection, support);
					 break;
			case 3 : roof=new FPRoof(SALT_BOX_ROOF,slopeAngle, roofHeight, eaveProjection, support);
					 break;
			default : roof=new FPRoof(FLAT_ROOF,-1.0,-1.0,-1,-1);
			}
			
			unsigned int nbUrlLen;
			bs->readnbit(nbUrlLen, 8);

			char Url[50];
			memset(Url, 0, sizeof(Url));
			for (j=0; j<nbUrlLen; j++)
			{
				unsigned short UrlChar;
				bs->readnbit(UrlChar, 8);
				Url[j] = UrlChar;
			}
			
			roof->SetTexURL((char*)Url, nbUrlLen);

			bool texIsGeneric=bs->readbit();
			roof->SetTexIsGeneric(texIsGeneric);

			unsigned short texMode;
			bs->readnbit(texMode,2);
			float texXScale=1.0;
			float texYScale=1.0;
			float texXPosition=0.0;
			float texYPosition=0.0;
			float texRotation=0.0;
			
			switch(texMode)
			{
			case 0:
				break;
			case 1:
				bs->readnbit(texXScale,32);
				bs->readnbit(texYScale,32);
				break;
			case 2:
				bs->readnbit(texXScale,32);
				bs->readnbit(texYScale,32);
				bs->readnbit(texXPosition,32);
				bs->readnbit(texYPosition,32);
				bs->readnbit(texRotation,32);
				break;
			}
			roof->SetTexXScale(texXScale);
			roof->SetTexYScale(texYScale);
			roof->SetTexXPosition(texXPosition);
			roof->SetTexYPosition(texYPosition);
			roof->SetTexRotation(texRotation);

			(*roofs)+=roof;

		}
		
		bs->readnbit(nbSwapNodes,8);

		FPBuilding* fpb=new FPBuilding();
		
		if (nbSwapNodes!=0)
		{
			for (unsigned int numSwapNodes=0; numSwapNodes<nbSwapNodes; numSwapNodes++)
			{
				unsigned int nbcharSwapNode;
				bs->readnbit(nbcharSwapNode,8);
				ZString swapNode;
				for (unsigned int numchar=0; numchar<nbcharSwapNode; numchar++)
				{
					char charSwapNode;
					unsigned int temp;
					bs->readnbit(temp,8);
					memcpy(&charSwapNode,&temp,1);
					swapNode+=charSwapNode;
				}
				fpb->AddSwapNode(swapNode);
			}
		}
		
		fpb->SetAltitude(altitude);
		fpb->SetHeight(height);
		fpb->SetIndex(index);
		fpb->SetParent(m_pFPSet->Root());
		fpb->SetState3D(NOCOMPUTED);
		fpb->SetMetricError(metricError);
		fpb->SetFacades(facades);
		fpb->SetRoofs(roofs);

		Vec2f** footprint=(Vec2f**)malloc(sizeof(Vec2f*)*NbRings);
		int currentVertex=0;
		for (int numring=0; numring<NbRings; numring++)
		{
			int nbVertices=nbVerticesByRing[numring];
			Vec2f* ring=(Vec2f*)malloc(sizeof(Vec2f)*nbVertices);
			int k=0;
			for (int j=0; j<nbVertices; j++)
				ring[j]=newVertices[j+currentVertex];
			currentVertex+=nbVertices;
			footprint[numring]=ring;
		}

		//}
		
		fpb->SetFootPrint(footprint);
		fpb->SetNbRings(NbRings);
		fpb->SetNbVerticesByRing(nbVerticesByRing);
		fpb->SetNbRings(NbRings);
		ZArray<FP*>* elevations=new ZArray<FP*>;
		elevations->InsertAt(0,(FP*)fpb);
		m_pFPSet->AddFootPrints(*elevations,-1);
		if (nbNewVertices!=0)
			free(newVertices);
		
	}
	return TRUE;
}





