
#include "FPFacade.h" 
#include "IM1Decoders/AFX/FootPrint/Roof/polyline2D.h"

#include "facade3d.h"

#define NEW_POSXY
#define NEW_MODEL3D


/*--------- Constructor --------------*/
FacadeBuilder::FacadeBuilder(FPFacade* pFacade)
:m_DisplayType(EMPTY),
m_NbVertices(0),
m_NbQuads(0),
m_NbTriangles(0),
m_NbLines(0),
m_Vertices(NULL),
m_IndicesQuads(NULL),
m_IndicesTris(NULL),
m_IndicesLines(NULL),
m_NbModels(0),
m_Translations(NULL),
m_ScaleModelX(1),  
m_ScaleModelY(1),
m_ScaleModelZ(1),
m_Model(NULL),
m_pFacade(pFacade) 
{
	if (pFacade == NULL)
		return;

	if (pFacade->HasTexture())
	{
		if (pFacade->HasChild())
		{
			ZArray<FPFacade*>* pFilledElements = pFacade->GetFilledChildren();	
			if (pFilledElements == NULL)
			{
				// no elements to substract, then display as a quad
				m_DisplayType = QUAD;
			}
			else
			{
				// there are sub elements filled with another texture/model, 
				m_DisplayType = TRIANGLES;
				int nChild = pFilledElements->GetLength();
				for (int i=0; i<nChild; i++)
				{
					FPFacade* pChild = (*pFilledElements)[i];
					if (pChild->HasModel3D())
					{
						pFacade->SetState(STATE_WAITING_CHILD_MODEL);
						return;
					}
				}
			}
		}
		else
		{
			// this element has a texture and no sub elements to substrat
			// then display as a quad
			m_DisplayType = QUAD;		
		}
	}

	else if (pFacade->HasModel3D())
	{
		m_DisplayType = MODEL3D;
		pFacade->SetState(STATE_WAITING_MODEL);
		return;
	}

	pFacade->SetState(STATE_READY);
} 


/*--------- Destructor --------------*/
FacadeBuilder::~FacadeBuilder()
{
	if (m_Vertices != NULL) free(m_Vertices);
	if (m_IndicesQuads != NULL) free(m_IndicesQuads);
	if (m_IndicesTris != NULL) free(m_IndicesTris);
	if (m_IndicesLines != NULL)
	{
		for (int i=0; i<4; i++)
		{
			if (m_IndicesLines[i] != NULL) free(m_IndicesLines[i]);
		}
		free(m_IndicesLines);
	}
	if (m_Translations != NULL) free(m_Translations);
}


/*---------------------------------------------------------------------
	Function:	Build
	Purpose:	builds OPENGL arrays for textured elements 
				build position+scale, etc for 3D Model
/*--------------------------------------------------------------------*/
void FacadeBuilder::Build() 
{
	// indices
	switch (m_DisplayType)
	{
	case EMPTY:
#ifdef SHOW_LINES
		BuildLines();
#endif
		break;
	case QUAD:
		BuildQuads();
		break;
	case TRIANGLES:
		BuildTriangles();
		break;
	case MODEL3D:
		BuildModel();
		break;
	default:
		break;
	}
}


/*---------------------------------------------------------------------
	Function:	BuildFilledElements
	Purpose:	builds an array of filled facade elements
/*--------------------------------------------------------------------*/
void FacadeBuilder::BuildQuadVertexArray()
{
	// Fill 4 points of main quad
	m_Vertices = (float*)malloc(4*8*sizeof(float));
	m_NbVertices = 4;
	
	// clockwise

	// left bottom
	int n=0;
	m_Vertices[8*n] = m_pFacade->GetXPosition();
#ifdef INV_TEXTURE_METHOD2
	m_Vertices[8*n+1] = -m_pFacade->GetYPosition();
#else
	m_Vertices[8*n+1] = m_pFacade->GetYPosition();
#endif
	m_Vertices[8*n+2] = m_pFacade->GetXNormal();
	m_Vertices[8*n+3] = m_pFacade->GetYNormal();
	m_Vertices[8*n+4] = m_pFacade->GetZNormal();
	m_Vertices[8*n+5] = m_pFacade->GetX();
	m_Vertices[8*n+6] = m_pFacade->GetY();
	m_Vertices[8*n+7] = m_pFacade->GetZ();
	
	// left top
	n++;
	m_Vertices[8*n] = m_pFacade->GetXPosition();
#ifdef INV_TEXTURE_METHOD2
	m_Vertices[8*n+1] = - m_pFacade->GetYPosition() - m_pFacade->GetYScale();
#else
	m_Vertices[8*n+1] = m_pFacade->GetYPosition() + m_pFacade->GetYScale();
#endif
	m_Vertices[8*n+2] = m_pFacade->GetXNormal();
	m_Vertices[8*n+3] = m_pFacade->GetYNormal();
	m_Vertices[8*n+4] = m_pFacade->GetZNormal();
	m_Vertices[8*n+5] = m_pFacade->GetX();
	m_Vertices[8*n+6] = m_pFacade->GetY();
	m_Vertices[8*n+7] = m_pFacade->GetZ() + m_pFacade->GetHeight();

	// right top
	n++;
	m_Vertices[8*n] = m_pFacade->GetXPosition() + m_pFacade->GetXScale();
#ifdef INV_TEXTURE_METHOD2
	m_Vertices[8*n+1] = - m_pFacade->GetYPosition() - m_pFacade->GetYScale();
#else
	m_Vertices[8*n+1] = m_pFacade->GetYPosition() + m_pFacade->GetYScale();
#endif
	m_Vertices[8*n+2] = m_pFacade->GetXNormal();
	m_Vertices[8*n+3] = m_pFacade->GetYNormal();
	m_Vertices[8*n+4] = m_pFacade->GetZNormal();
	m_Vertices[8*n+5] = m_pFacade->GetX() + m_pFacade->GetXFacade()*m_pFacade->GetWidth();
	m_Vertices[8*n+6] = m_pFacade->GetY() + m_pFacade->GetYFacade()*m_pFacade->GetWidth();
	m_Vertices[8*n+7] = m_pFacade->GetZ() + m_pFacade->GetHeight();

	// right bottom
	n++;
	m_Vertices[8*n] = m_pFacade->GetXPosition() + m_pFacade->GetXScale();
#ifdef INV_TEXTURE_METHOD2
	m_Vertices[8*n+1] = -m_pFacade->GetYPosition();
#else
	m_Vertices[8*n+1] = m_pFacade->GetYPosition();
#endif
	m_Vertices[8*n+2] = m_pFacade->GetXNormal();
	m_Vertices[8*n+3] = m_pFacade->GetYNormal();
	m_Vertices[8*n+4] = m_pFacade->GetZNormal();
	m_Vertices[8*n+5] = m_pFacade->GetX() + m_pFacade->GetXFacade()*m_pFacade->GetWidth();
	m_Vertices[8*n+6] = m_pFacade->GetY() + m_pFacade->GetYFacade()*m_pFacade->GetWidth();
	m_Vertices[8*n+7] = m_pFacade->GetZ();
}


void FacadeBuilder::BuildLines()
{
	CFacade3D	Facade3D;
	
	// direction vectors
	Facade3D.SetNormal(m_pFacade->GetXNormal(), m_pFacade->GetYNormal(), m_pFacade->GetZNormal());
	Facade3D.SetFacade(m_pFacade->GetXFacade(), m_pFacade->GetYFacade(), m_pFacade->GetZFacade());

	CellParam cellparam;
	memset(&cellparam, 0, sizeof(CellParam));
	cellparam.originX = m_pFacade->GetX();
	cellparam.originY = m_pFacade->GetY();
	cellparam.originZ = m_pFacade->GetZ();
	cellparam.width = m_pFacade->GetWidth();
	cellparam.height = m_pFacade->GetHeight();
	cellparam.posX = m_pFacade->GetXPosition();
	cellparam.posY = m_pFacade->GetYPosition();
	cellparam.scaleX = m_pFacade->GetXScale();
	cellparam.scaleY = m_pFacade->GetYScale();
	cellparam.repeatX = m_pFacade->GetXRepeatInterval();
	cellparam.repeatY = m_pFacade->GetYRepeatInterval();
	cellparam.brepeat = m_pFacade->GetRepeat();

	Facade3D.AddLines(cellparam);

	// computation  
	Facade3D.BuildFacadeLines();  
	 
	// get nbs
	m_NbVertices = Facade3D.GetNbVertices();

#ifdef INV_TEXTURE_METHOD2
	Facade3D.FillVertexArray(FORMAT_GL_T2F_N3F_V3F, &m_Vertices, true);
#else
	Facade3D.FillVertexArray(FORMAT_GL_T2F_N3F_V3F, &m_Vertices);
#endif
	Facade3D.FillIndexArrayForGLLines(&m_NbLines, &m_IndicesLines); 
}



/*---------------------------------------------------------------------
	Function:	BuildQuads
	Purpose:	builds OPENGL arrays for quad type display
				uses facade.lib
/*--------------------------------------------------------------------*/
void FacadeBuilder::BuildQuads()
{ 
	CFacade3D*	pFacade3D = new CFacade3D;
	
	// direction vectors
	pFacade3D->SetNormal(m_pFacade->GetXNormal(), m_pFacade->GetYNormal(), m_pFacade->GetZNormal());
	pFacade3D->SetFacade(m_pFacade->GetXFacade(), m_pFacade->GetYFacade(), m_pFacade->GetZFacade());

	CellParam cellparam;
	memset(&cellparam, 0, sizeof(CellParam));
	cellparam.originX = m_pFacade->GetX();
	cellparam.originY = m_pFacade->GetY();
	cellparam.originZ = m_pFacade->GetZ();
	cellparam.width = m_pFacade->GetWidth();
	cellparam.height = m_pFacade->GetHeight();
	cellparam.posX = m_pFacade->GetXPosition();
	cellparam.posY = m_pFacade->GetYPosition();
	cellparam.scaleX = m_pFacade->GetXScale();
	cellparam.scaleY = m_pFacade->GetYScale();
	cellparam.repeatX = m_pFacade->GetXRepeatInterval();
	cellparam.repeatY = m_pFacade->GetYRepeatInterval();
	cellparam.brepeat = m_pFacade->GetRepeat();

	pFacade3D->AddQuads(cellparam); 

	// computation   
	pFacade3D->BuildFacadeQuads();  
	 
	// get nbs
	m_NbVertices = pFacade3D->GetNbVertices();
	m_NbQuads = pFacade3D->GetNbQuads();

#ifdef INV_TEXTURE_METHOD2
	pFacade3D->FillVertexArray(FORMAT_GL_T2F_N3F_V3F, &m_Vertices, true);
#else
	pFacade3D->FillVertexArray(FORMAT_GL_T2F_N3F_V3F, &m_Vertices);
#endif
	pFacade3D->FillIndexArrayForGLQuads(&m_NbQuads, &m_IndicesQuads); 
}


/*---------------------------------------------------------------------
	Function:	BuildTriangles
	Purpose:	builds OPENGL arrays for triangles type display
				uses facade.lib
/*--------------------------------------------------------------------*/
void FacadeBuilder::BuildTriangles()
{
	CFacade3D*	pFacade3D = new CFacade3D; 

	// direction vectors
	pFacade3D->SetNormal(m_pFacade->GetXNormal(), m_pFacade->GetYNormal(), m_pFacade->GetZNormal());
	pFacade3D->SetFacade(m_pFacade->GetXFacade(), m_pFacade->GetYFacade(), m_pFacade->GetZFacade());
	
	// Fill 4 points of main quad
	CellParam cellparam;
	memset(&cellparam, 0, sizeof(CellParam));
	cellparam.originX = m_pFacade->GetX();
	cellparam.originY = m_pFacade->GetY();
	cellparam.originZ = m_pFacade->GetZ();
	cellparam.width = m_pFacade->GetWidth();
	cellparam.height = m_pFacade->GetHeight();
	cellparam.posX = m_pFacade->GetXPosition();
	cellparam.posY = m_pFacade->GetYPosition();
	cellparam.scaleX = m_pFacade->GetXScale();
	cellparam.scaleY = m_pFacade->GetYScale();
	cellparam.repeatX = m_pFacade->GetXRepeatInterval();
	cellparam.repeatY = m_pFacade->GetYRepeatInterval();
	cellparam.brepeat = m_pFacade->GetRepeat();
	
	pFacade3D->AddOutlineQuad(cellparam);
							 
	// fill sub elements
	int	nSub = m_pFacade->GetNbFilledChildren(); 
	// Fill points of sub elements
	ZArray<FPFacade*>* pSubElements = m_pFacade->GetFilledChildren();
		
	for (int i=0; i<nSub; i++)
	{
		FPFacade* pSubElement = (*pSubElements)[i];

		CellParam childcellparam;
		memset(&childcellparam, 0, sizeof(CellParam));
		childcellparam.originX = pSubElement->GetX();
		childcellparam.originY = pSubElement->GetY();
		childcellparam.originZ = pSubElement->GetZ();
		childcellparam.width = pSubElement->GetWidth();
		childcellparam.height = pSubElement->GetHeight();
		childcellparam.posX = pSubElement->GetXPosition();
		childcellparam.posY = pSubElement->GetYPosition();
		childcellparam.scaleX = pSubElement->GetXScale();
		childcellparam.scaleY = pSubElement->GetYScale();
		childcellparam.repeatX = pSubElement->GetXRepeatInterval();
		childcellparam.repeatY = pSubElement->GetYRepeatInterval();
		childcellparam.brepeat = pSubElement->GetRepeat();

		if (pSubElement->HasTexture())    
		{
			/*-- substract quad corresponding to texture of child element --*/
			pFacade3D->AddHoleQuad(childcellparam);      
		}
		else if ((pSubElement->HasModel3D()) && (pSubElement->GetState() == STATE_DONE))
		{
			/*-- substract section of 3d model --*/
			pFacade3D->InitModel();
			
			ZArray<ModelGeometry*>* pChildModel = pSubElement->GetModel();
			int ngeom = pChildModel->GetLength();
			for (int i=0; i<ngeom; i++)
			{
				ModelGeometry* pMG = (*pChildModel)[i];
				pFacade3D->AddModelGeometry(pMG->m_nVtx, pMG->m_aVtx, pMG->m_nIdx, pMG->m_aIdx);
			}
			pFacade3D->EndModel();  

			pFacade3D->AddHoleModel(childcellparam); 
		}
	}

	// compute triangles 
	pFacade3D->BuildFacadeTriangles();      
 
	// get nbs
	m_NbVertices = pFacade3D->GetNbVertices(); 
	m_NbTriangles = pFacade3D->GetNbTriangles();  

	// fill arrays
#ifdef INV_TEXTURE_METHOD2
	pFacade3D->FillVertexArray(FORMAT_GL_T2F_N3F_V3F, &m_Vertices, true);
#else
	pFacade3D->FillVertexArray(FORMAT_GL_T2F_N3F_V3F, &m_Vertices);
#endif
	pFacade3D->FillIndexArrayForGLTriangles(&m_NbTriangles, &m_IndicesTris); 
}


/*---------------------------------------------------------------------
	Function:	BuildModel
	Purpose:	builds scale+position for 3d model type display
				uses facade.lib
/*--------------------------------------------------------------------*/
void FacadeBuilder::BuildModel()
{
	CFacade3D	Facade3D;

	if (m_Model == NULL)
		return;
	
	// direction vectors
	Facade3D.SetNormal(m_pFacade->GetXNormal(), m_pFacade->GetYNormal(), m_pFacade->GetZNormal());
	Facade3D.SetFacade(m_pFacade->GetXFacade(), m_pFacade->GetYFacade(), m_pFacade->GetZFacade());

	// give model to lib
	Facade3D.InitModel();
	int ngeom = m_Model->GetLength();  
	for (int i=0; i<ngeom; i++)
	{
		ModelGeometry* pMG = (*m_Model)[i]; 
		Facade3D.AddModelGeometry(pMG->m_nVtx, pMG->m_aVtx, pMG->m_nIdx, pMG->m_aIdx);
	}  
	Facade3D.EndModel();                                                                                 

	// compute section positions on facade
	CellParam cellparam; 
	memset(&cellparam, 0, sizeof(CellParam));
	cellparam.originX = m_pFacade->GetX();
	cellparam.originY = m_pFacade->GetY();
	cellparam.originZ = m_pFacade->GetZ();
	cellparam.width = m_pFacade->GetWidth();
	cellparam.height = m_pFacade->GetHeight();
	cellparam.posX = m_pFacade->GetXPosition();
	cellparam.posY = m_pFacade->GetYPosition();
	cellparam.scaleX = m_pFacade->GetXScale();
	cellparam.scaleY = m_pFacade->GetYScale();
	cellparam.repeatX = m_pFacade->GetXRepeatInterval();
	cellparam.repeatY = m_pFacade->GetYRepeatInterval();
	cellparam.brepeat = m_pFacade->GetRepeat();

	ModelParam modelparam; 
	memset(&modelparam, 0, sizeof(ModelParam)); 
 
	Facade3D.FillModelParam(cellparam, modelparam);    

	m_NbModels = modelparam.nbModels;
	m_Translations = modelparam.atranslations;
	m_ScaleModelX = modelparam.scaleX;
	m_ScaleModelY = modelparam.scaleY;
	m_ScaleModelZ = modelparam.scaleZ;
}


/*---------------------------------------------------------------------
	Function:	AddModelGeometry
	Purpose:	store model indexed face set loaded from scene
/*--------------------------------------------------------------------*/
void FacadeBuilder::AddModelGeometry(int nVtx, float* aVtx, int nIdx, int* aIdx)
{
	ModelGeometry* pMG = new ModelGeometry(nVtx, aVtx, nIdx, aIdx);
	
	if (m_Model == NULL)
	{
		m_Model = new ZArray<ModelGeometry*>; 
	}

	(*m_Model) += pMG; 
}


	






/*--------- Default Constructor --------------*/
FPFacade::FPFacade()
: m_XScale(1.0), 
m_YScale(1.0), 
m_XPosition(0.0), 
m_YPosition(0.0), 
m_XRepeatInterval(0.0), 
m_YRepeatInterval(0.0), 
m_bRepeat(false), 
m_PrimitiveType(NO_PRIMITIVE), 
m_URL(NULL), 
m_StoriesHeight(NULL),
m_Elements(NULL),
m_WidthRatio(-1),
m_HeightRatio(-1),
m_xBottomLeft(0),
m_yBottomLeft(0),
m_zBottomLeft(0),
m_xFacade(0),
m_yFacade(0),
m_zFacade(0),
m_xNormal(0),
m_yNormal(0),
m_zNormal(0),
m_elementwidth(0),
m_elementheight(0),
m_parentwidth(0),
m_parentheight(0),
//m_FilledElements(NULL),
m_FilledChildren(NULL),
m_FacadeBuilder(NULL),
m_State(STATE_WAITING)
{
}


/*--------- Constructor --------------*/
FPFacade::FPFacade(float x, float y, float z, float xfacade, float yfacade, float zfacade, float width, float height) 
: m_XScale(1.0), 
m_YScale(1.0), 
m_XPosition(0.0), 
m_YPosition(0.0), 
m_XRepeatInterval(0.0),  
m_YRepeatInterval(0.0), 
m_bRepeat(false), 
m_PrimitiveType(NO_PRIMITIVE), 
m_URL(NULL), 
m_StoriesHeight(NULL),
m_Elements(NULL),
m_WidthRatio(1),
m_HeightRatio(1),
//m_FilledElements(NULL),
m_FilledChildren(NULL),
m_FacadeBuilder(NULL),
m_State(STATE_WAITING)
{
	m_xBottomLeft = x;
	m_yBottomLeft = y;
	m_zBottomLeft = z;

	m_xFacade = xfacade;
	m_yFacade = yfacade;
	m_zFacade = zfacade;

	m_parentwidth = width; 
	m_parentheight = height;

	m_elementwidth = m_parentwidth;
	m_elementheight = m_parentheight;

	// compute normal to facade
	float xZ = 0;
	float yZ = 0;
	float zZ = 1;

	m_xNormal = (yfacade*zZ) - (yZ*zfacade);
	m_yNormal = -(xfacade*zZ) + (xZ*zfacade);
	m_zNormal = (xfacade*yZ) - (xZ*yfacade);

	float len = sqrt(m_xNormal*m_xNormal+m_yNormal*m_yNormal+m_zNormal*m_zNormal);
	m_xNormal /= len;
	m_yNormal /= len;
	m_zNormal /= len;
}


/*----------- Destructor ----------------*/
FPFacade::~FPFacade()
{
	if (m_URL != NULL) free(m_URL);

	// zarrays
	if (m_StoriesHeight != NULL) delete m_StoriesHeight;
//	if (m_FilledElements != NULL) delete m_FilledElements;
	if (m_FilledChildren != NULL) delete m_FilledChildren;
	if (m_FacadeBuilder != NULL) delete m_FacadeBuilder;
//	ZArray<ZArray<FPFacade*>* >* m_Elements;
}


void FPFacade::SetWidthRatio(float width)
{
	if ((m_WidthRatio != -1) && (m_WidthRatio != 1))
	{
		// do not modify width
		// it has already been set at decoding
		// in case of last horizontal element, ratio forced to 1-sum of previous ratios
		return;
	}

	m_WidthRatio=width; 
	m_elementwidth=m_parentwidth*m_WidthRatio;
}

void FPFacade::SetHeightRatio(float height)
{
	if ((m_HeightRatio != -1) && (m_HeightRatio != 1))
	{
		// do not modify height
		// it has already been set at decoding
		// in case of last vertical element, ratio forced to 1-sum of previous ratios
		return;
	}

	m_HeightRatio=height;
	m_elementheight=m_parentheight*m_HeightRatio;
}

int FPFacade::GetNbStories() const
{
	if (m_StoriesHeight == NULL)
		return 0;
	else
		return m_StoriesHeight->GetLength();
}

void FPFacade::ShiftRight(float posH)
{
	m_xBottomLeft += posH*m_parentwidth*m_xFacade;
	m_yBottomLeft += posH*m_parentwidth*m_yFacade;
}

void FPFacade::ShiftUp(float posV)
{
	m_zBottomLeft += posV*m_parentheight;
}

void FPFacade::SetURL(char* Url, int nUrlLen)
{
	m_URL = (char*)malloc(nUrlLen*sizeof(char) + sizeof(char));
	strcpy(m_URL, Url);
//	strcat(m_URL, '\0');
}

void FPFacade::InitFacade()
{
	if (HasChild())   
	{
		// list of filled children
		int nStories = m_Elements->GetLength();
		for (int i=0; i<nStories; i++)
		{
			ZArray<FPFacade*>* pArrayStorey = (*m_Elements)[i];

			int nCells = pArrayStorey->GetLength();
			for (int j=0; j<nCells; j++)
			{
				FPFacade* pElement = (*pArrayStorey)[j];
				pElement->BuildFilledElements(&m_FilledChildren);
				pElement->InitFacade();
			}
		}
	}

	m_FacadeBuilder = new FacadeBuilder(this); 
}

void FPFacade::ComputeFacade()
{
	m_FacadeBuilder->Build();

	if (HasChild()) 
	{
		// list of filled children
		int nStories = m_Elements->GetLength();
		for (int i=0; i<nStories; i++)
		{
			ZArray<FPFacade*>* pArrayStorey = (*m_Elements)[i];

			int nCells = pArrayStorey->GetLength();
			for (int j=0; j<nCells; j++)
			{
				FPFacade* pElement = (*pArrayStorey)[j];
				pElement->ComputeFacade();
			}
		}
	}
}

bool FPFacade::HasChild(void) const
{
	if (m_Elements == NULL)
		return false;
	else
	{
		if (m_Elements->GetLength() > 0)
			return true;
		else 
			return false;
	}
}

bool FPFacade::HasTexture(void) const
{
	if (m_PrimitiveType == PRIM_TEXTURE && m_URL!=NULL)
		return true;

	return false;
}

bool FPFacade::HasModel3D(void) const
{
	if (m_PrimitiveType == PRIM_3DMODEL && m_URL!=NULL)
		return true;

	return false;
}

/*bool FPFacade::HasFilledElements(void) const
{
	if (m_FilledElements != NULL)
		return true;

	return false;
}
*/


int FPFacade::GetNbFilledChildren(void)
{
	if (m_FilledChildren != NULL)
		return m_FilledChildren->GetLength();

	return 0;
}


/*---------------------------------------------------------------------
	Function:	BuildFilledElements
	Purpose:	builds an array of filled facade elements
/*--------------------------------------------------------------------*/
void FPFacade::BuildFilledElements(ZArray<FPFacade*>** pElements)
{
	if (HasTexture() || HasModel3D())
	{
		// append this element in array
		if (*pElements == NULL)
		{
			*pElements = new ZArray<FPFacade*>;
		}
		(**pElements) += this;

		return;
	}

	if (HasChild()) 
	{
		int nStories = m_Elements->GetLength();
		for (int i=0; i<nStories; i++)
		{
			ZArray<FPFacade*>* pArrayStorey = (*m_Elements)[i];

			int nCells = pArrayStorey->GetLength();
			for (int j=0; j<nCells; j++)
			{
				FPFacade* pElement = (*pArrayStorey)[j];

				// build sub elements elements
				pElement->BuildFilledElements(pElements);
			}
		}
	}
}



