// FPRoof.cpp: implementation of the FPRoof class.
//
//////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "FPRoof.h"
#include "RoofConstruction.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#define INV_TEXTURE_METHOD2
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

FPRoof::FPRoof(): 
m_Type(FLAT_ROOF), 
m_Height(0.0f), 
m_Angle(0.0f), 
m_EaveProjection(0.0f), 
m_WallSupport(-1),
m_TexURL(NULL),
m_TexIsGeneric(true),
m_TexXScale(1.0f),
m_TexYScale(1.0f),
m_TexXPosition(0.0f),
m_TexYPosition(0.0f),
m_TexRotation(0.0f), 
m_Vertices(NULL), 
m_nbVertices(0), 
m_nbSlopes(0), 
m_NbVerticesBySlope(NULL), 
m_PolygonIndices(NULL), 
m_LineIndices(NULL), 
m_nbLines(0), 
m_NbFootprints(0)
{
	m_Footprints=new ZArray<FP*>();
}

FPRoof::FPRoof(float angle, float height): 
m_Type(FLAT_ROOF), 
m_Height(height), 
m_Angle(angle), 
m_EaveProjection(0.0), 
m_WallSupport(-1),
m_TexURL(NULL), 
m_TexIsGeneric(true),
m_TexXScale(1.0f),
m_TexYScale(1.0f),
m_TexXPosition(0.0f),
m_TexYPosition(0.0f),
m_TexRotation(0.0f),
m_Vertices(NULL), 
m_nbVertices(0), 
m_nbSlopes(0), 
m_NbVerticesBySlope(NULL), 
m_PolygonIndices(NULL), 
m_LineIndices(NULL), 
m_nbLines(0), 
m_NbFootprints(0) 
{
	m_Footprints=new ZArray<FP*>();
}

FPRoof::FPRoof(typeRoof type, float angle, float height, float eaveProjection, int wallSupport): 
m_Type(type), 
m_Height(height), 
m_Angle(angle), 
m_EaveProjection(eaveProjection), 
m_WallSupport(wallSupport),
m_TexURL(NULL),
m_TexIsGeneric(true),
m_TexXScale(1.0f),
m_TexYScale(1.0f),
m_TexXPosition(0.0f),
m_TexYPosition(0.0f),
m_TexRotation(0.0f), 
m_Vertices(NULL), 
m_nbVertices(0), 
m_nbSlopes(0), 
m_NbVerticesBySlope(NULL), 
m_PolygonIndices(NULL), 
m_LineIndices(NULL), 
m_nbLines(0), 
m_NbFootprints(0)
{
	m_Footprints=new ZArray<FP*>();
}

FPRoof::FPRoof(const FPRoof &in)
{
	m_Height=in.GetHeight();
	m_Angle=in.GetAngle();
	m_Type=in.GetType();
	m_EaveProjection=in.GetEaveProjection();
	m_WallSupport=in.GetWallSupport();
	m_TexIsGeneric=in.GetTexIsGeneric();
	m_TexXScale=in.GetTexXScale();
	m_TexYScale=in.GetTexYScale();
	m_TexXPosition=in.GetTexXPosition();
	m_TexYPosition=in.GetTexYPosition();
	m_TexRotation=in.GetTexRotation();
	// added for FPRoof polygon computation
	m_Vertices = in.GetVertices();
	m_nbVertices=in.GetNbVertices();
	m_nbSlopes = in.GetNbSlopes();
	m_NbVerticesBySlope = in.GetNbVerticesBySlope();
	m_PolygonIndices = in.GetPolygonIndices();

	char* url=in.GetTexURL();
	if (m_TexURL!=NULL)
		free (m_TexURL);
	m_TexURL = (char*)malloc(strlen(url)*sizeof(char) + sizeof(char));
	strcpy(m_TexURL, url);

	// for FPRoof line computation
	m_LineIndices = in.GetLineIndices();
	m_nbLines = in.GetNbLines();

	// for FPRoof section
	m_Footprints = in.GetNewFootPrint();
}

FPRoof::~FPRoof()
{
	// free memory
	if (m_TexURL != NULL) free(m_TexURL);
	CleanArrays();
}

FPRoof & FPRoof::operator =(const FPRoof &in)
{
	m_Height=in.GetHeight();
	m_Angle=in.GetAngle();
	m_Type=in.GetType();
	m_EaveProjection=in.GetEaveProjection();
	m_WallSupport=in.GetWallSupport();
	m_TexIsGeneric=in.GetTexIsGeneric();
	m_TexXScale=in.GetTexXScale();
	m_TexYScale=in.GetTexYScale();
	m_TexXPosition=in.GetTexXPosition();
	m_TexYPosition=in.GetTexYPosition();
	m_TexRotation=in.GetTexRotation();

	char* url=in.GetTexURL();
	if (m_TexURL!=NULL)
		free (m_TexURL);
	m_TexURL = (char*)malloc(strlen(url)*sizeof(char) + sizeof(char));
	strcpy(m_TexURL, url);
	
	// added for polygon FPRoof computation
	m_Vertices = in.GetVertices();
	m_nbVertices=in.GetNbVertices();
	m_nbSlopes = in.GetNbSlopes();
	m_NbVerticesBySlope = in.GetNbVerticesBySlope();
	m_PolygonIndices = in.GetPolygonIndices();
	
	// for lines
	m_LineIndices = in.GetLineIndices();
	m_nbLines = in.GetNbLines();

	// for FPRoof section
	m_Footprints = in.GetNewFootPrint();

	return (*this);
}

int FPRoof::ComputeMultiRoof(ZArray<FP*>* footprints, float footprintheight)
{
	int i,j,k;
		// call to roof lib to construct 3D roof
	int	nSize = 0;		// init size of array in bytes
	int nCount = 0;		// init number of elements
	int	nRet = -1;		// return code
	int	nFootprint = 0;

	// clean 
	CleanArrays();

	// build roof
	POLYGON_VECTOR fps;
	
	int nFootprintCount = footprints->GetLength();
	for (i=0; i<nFootprintCount; i++)
	{
		FP* fp=(*footprints)[i];
		Polygon2D poly;
		int index=0;
		int begin_ring=0;
		for (j=0; j<fp->NbRings(); j++)
		{
			Polyline2D ring;
			int nbVertices=fp->NbVerticesByRing()[j];
			for (k=0; k<nbVertices; k++)
			{
				ring.AddPoint(Point2D(fp->FootPrint()[j][k].x,fp->FootPrint()[j][k].y));
			}
			poly.AddPolyline(ring);
		}

		fps.push_back(poly);
	}

	m_Vertices=(float**)malloc(nFootprintCount*sizeof(float*));
	m_nbVertices=(int*)malloc(nFootprintCount*sizeof(int));
	m_nbSlopes=(int*)malloc(nFootprintCount*sizeof(int));
	m_NbVerticesBySlope=(int**)malloc(nFootprintCount*sizeof(int*));
	m_PolygonIndices=(unsigned int***)malloc(nFootprintCount*sizeof(unsigned int**));
	m_nbLines=(int*)malloc(nFootprintCount*sizeof(int));
	m_LineIndices=(unsigned int**)malloc(nFootprintCount*sizeof(unsigned int*));


	POLYGON_VECTOR::iterator it;
	m_NbFootprints=0;
	for (it=fps.begin(); it!=fps.end(); it++, m_NbFootprints++)
	{
		CRoofConstruction	RC;
		Polygon2D footprint = (*it);

		Polygon2D easyfootprint = Polygon2D();
		int nRemoved = 0;
		nRemoved = footprint.RemoveColinearVertices(easyfootprint);
		if (nRemoved == 0) 
			easyfootprint = Polygon2D(footprint);
 
		RC.SetRoofTextureParam(m_TexXPosition, m_TexYPosition, m_TexXScale, m_TexYScale, m_TexRotation, m_TexIsGeneric);
 
		nRet = RC.BuildRoof3D(easyfootprint, footprintheight, m_Angle, m_Height, m_Type, m_EaveProjection);
		if (nRet != 0)
			return nRet;

#ifdef INV_TEXTURE_METHOD2		
		m_nbVertices[m_NbFootprints]=RC.FillVertexArray(FORMAT_GL_T2F_N3F_V3F,&m_Vertices[m_NbFootprints], true);
#else
		m_nbVertices[m_NbFootprints]=RC.FillVertexArray(FORMAT_GL_T2F_N3F_V3F,&m_Vertices[m_NbFootprints]);
#endif
		RC.FillIndexArrayForGLPolygon(m_nbSlopes[m_NbFootprints], &m_NbVerticesBySlope[m_NbFootprints], &m_PolygonIndices[m_NbFootprints]);
		RC.FillIndexArrayForGLLines(m_nbLines[m_NbFootprints],&m_LineIndices[m_NbFootprints]);
		vector<Polygon2D> newfps = RC.GetNewFootPrint();
		 
		for (vector<Polygon2D>::iterator pit=newfps.begin(); pit!=newfps.end(); pit++)
		{
			FP* newfp=new FP();
			unsigned int* nbVerticesByRing=(unsigned int*) malloc(pit->GetNbRings()*sizeof(unsigned int));
			Vec2f** vertices=(Vec2f**)malloc(pit->GetNbRings()*sizeof(Vec2f*));
			for (int num_ring=0; num_ring<pit->GetNbRings(); num_ring++)
			{
				vertices[num_ring]=(Vec2f*)malloc((*pit)[num_ring].GetNbPoints()*sizeof(Vec2f));
				nbVerticesByRing[num_ring]=(*pit)[num_ring].GetNbPoints();
				for (int num_point=0; num_point<(*pit)[num_ring].GetNbPoints(); num_point++)
				{
					Point2D pt;
					(*pit)[num_ring].GetPoint(pt,num_point);
					vertices[num_ring][num_point].x=pt.x();
					vertices[num_ring][num_point].y=pt.y();
				}
			}
			newfp->SetFootPrint(vertices);
			newfp->SetNbRings(pit->GetNbRings());
			newfp->SetNbVerticesByRing(nbVerticesByRing);
			(*m_Footprints)+=newfp;
		}
	}
	fps.clear();
	// for lines

	return nRet;

}


float FPRoof::GetHeight() const 
{ 
	 return m_Height; 
}

float FPRoof::GetAngle() const 
{ 
	return m_Angle; 
}

typeRoof FPRoof::GetType() const 
{ 
	return m_Type; 
}

float FPRoof::GetEaveProjection() const 
{ 
	return m_EaveProjection; 
}

int FPRoof::GetWallSupport() const 
{ 
	return m_WallSupport; 
}

float** FPRoof::GetVertices() const 
{ 
	return m_Vertices; 
}

int* FPRoof::GetNbVertices() const
{
	return m_nbVertices;
}

int* FPRoof::GetNbSlopes() const 
{ 
	return m_nbSlopes; 
}

int** FPRoof::GetNbVerticesBySlope() const 
{ 
	return m_NbVerticesBySlope; 
}

unsigned int*** FPRoof::GetPolygonIndices() const 
{ 
	return m_PolygonIndices; 
}

int* FPRoof::GetNbLines() const 
{ 
	return m_nbLines; 
}

unsigned int** FPRoof::GetLineIndices() const 
{ 
	return m_LineIndices; 
}

ZArray<FP*>* FPRoof::GetNewFootPrint() const
{
	return m_Footprints;
}

int FPRoof::GetNbFootprints() const
{
	return m_NbFootprints;
}

void FPRoof::SetHeight(float height) 
{ 
	m_Height=height; 
}

void FPRoof::SetAngle(float angle) 
{ 
	m_Angle=angle; 
}

void FPRoof::SetType(typeRoof type) 
{ 
	m_Type=type; 
}

void FPRoof::SetEaveProjection(float eaveProj) 
{ 
	m_EaveProjection=eaveProj; 
}

void FPRoof::SetWallSupport(int wallSupport)
{
	m_WallSupport=wallSupport;
}

void FPRoof::SetTexURL(char* Url, int nUrlLen)
{
	m_TexURL = (char*)malloc(nUrlLen*sizeof(char) + sizeof(char));
	strcpy(m_TexURL, Url);
}

void FPRoof::CleanArrays(void)
{
	int numfp;
	if (m_PolygonIndices!=NULL)
	{
		for (numfp=0; numfp<m_NbFootprints; numfp++)
		{
			if (m_PolygonIndices[numfp] != NULL)
			{
				for (int i=0; i<m_nbSlopes[numfp]; i++)
				{
					if (m_PolygonIndices[numfp][i]!=NULL)
					{
						free (m_PolygonIndices[numfp][i]);
						m_PolygonIndices[numfp][i]=NULL;
					}
				}
				free(m_PolygonIndices[numfp]);
				m_PolygonIndices[numfp]=NULL;
			}
		}
		free (m_PolygonIndices);
		m_PolygonIndices=NULL;
	}

	if (m_NbVerticesBySlope!=NULL)
	{
		for (numfp=0; numfp<m_NbFootprints; numfp++)
		{
		
			if (m_NbVerticesBySlope[numfp]!=NULL)
			{
				free(m_NbVerticesBySlope[numfp]);
				m_NbVerticesBySlope[numfp]=NULL;
			}
		}
		free (m_NbVerticesBySlope);
		m_NbVerticesBySlope=NULL;
	}
	if (m_Vertices!=NULL)
	{
		for (numfp=0; numfp<m_NbFootprints; numfp++)
		{
			if (m_Vertices[numfp]!=NULL)
			{
				free (m_Vertices[numfp]);
				m_Vertices[numfp]=NULL;
			}
		}
		free (m_Vertices);
		m_Vertices=NULL;
	}

	if (m_nbSlopes!=NULL)
	{
		free (m_nbSlopes);
		m_nbSlopes=NULL;
	}
	
	m_NbFootprints = 0;
	m_Footprints->Clear();
}
// #eof "$Id: FPRoof.cpp,v 1.2 2006/10/24 15:46:06 granger Exp $"
