// FPRoof.h: interface for the FPRoof class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_FPERoof_H__771C1C80_E474_4993_93D2_FFF2B7CE4421__INCLUDED_)
#define AFX_FPERoof_H__771C1C80_E474_4993_93D2_FFF2B7CE4421__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "FP.h"

typedef enum {FLAT_ROOF,HIP_ROOF,GABLE_ROOF,SALT_BOX_ROOF} typeRoof;



class FPRoof  
{
public:
public:
	FPRoof();
	~FPRoof();
	FPRoof(float angle, float height);
	FPRoof(typeRoof type, float angle, float height, float eaveProjection, int wallSupport);
	FPRoof(const FPRoof & in);
	FPRoof & operator =(const FPRoof &in);
	
	int ComputeMultiRoof(ZArray<FP*>* footprints, float footprintheight);
	float GetHeight() const;
	float GetAngle() const;
	typeRoof GetType() const;
	float GetEaveProjection() const;
	int GetWallSupport() const;
	inline char* GetTexURL() const { return m_TexURL; };
	inline bool  GetTexIsGeneric() const { return m_TexIsGeneric; };
	inline float GetTexXScale() const { return m_TexXScale; };
	inline float GetTexYScale() const { return m_TexYScale; };
	inline float GetTexXPosition() const { return m_TexXPosition; };
	inline float GetTexYPosition() const { return m_TexYPosition; };
	inline float GetTexRotation() const { return m_TexRotation; };

	float** GetVertices() const;
	int* GetNbVertices() const;
	int* GetNbSlopes() const;
	int** GetNbVerticesBySlope() const;
	unsigned int*** GetPolygonIndices() const;
	int* GetNbLines() const;
	unsigned int** GetLineIndices() const;
	ZArray<FP*>* GetNewFootPrint() const;
	int GetNbFootprints() const;

	void SetHeight(float height);
	void SetAngle(float angle);
	void SetType(typeRoof type);
	void SetEaveProjection(float eaveProj);
	void SetWallSupport(int wallSupport);
	void SetTexURL(char* Url, int nUrlLen); 
	inline void SetTexIsGeneric(bool texIsGeneric) { m_TexIsGeneric=texIsGeneric; };
	inline void SetTexXScale(float texXScale) { m_TexXScale=texXScale; };
	inline void SetTexYScale(float texYScale) { m_TexYScale=texYScale; };
	inline void SetTexXPosition(float texXPosition) { m_TexXPosition=texXPosition; };
	inline void SetTexYPosition(float texYPosition) { m_TexYPosition=texYPosition; };
	inline void SetTexRotation(float texRotation) { m_TexRotation=texRotation; };

	inline MediaObject* GetMediaObject() const { return m_pNode; };

	inline void SetMediaObject ( MediaObject* mo) { m_pNode=mo; };

	void CleanArrays(void);
protected:
	float m_Height;
	float m_Angle;
	typeRoof m_Type;
	float m_EaveProjection;
	int m_WallSupport;
	bool m_TexIsGeneric;
	float m_TexXScale;
	float m_TexYScale;
	float m_TexXPosition;
	float m_TexYPosition;
	float m_TexRotation;
	char*	m_TexURL;

	bool m_IsCompute;

	MediaObject* m_pNode;

	ZArray<FP*>* m_Footprints;

	int m_NbFootprints;
	float** m_Vertices;			// array of vertices coordinates (for GLInterleaved Array)
	int* m_nbVertices;
	int* m_nbSlopes;				// number of slopes forming the roof
	int** m_NbVerticesBySlope;	// array for number of vertices per slope
	unsigned int*** m_PolygonIndices;	// array of vertex indices forming the slopes (polygons)

	// for lines drawing
	int* m_nbLines;					// number of lines (aretes du toit)
	unsigned int** m_LineIndices;	// array of vertex indices forming the lines
};

#endif // !defined(AFX_FPERoof_H__771C1C80_E474_4993_93D2_FFF2B7CE4421__INCLUDED_)
