/********************************************************************************/
/* This software module was developped by										*/

/*	   Royan Jerme , jerome.royan@francetelecom.com								*/
/*                   France Telecom R&D, France		*/

/* in the course of development of the MPEG-4 standard (Part 16 AFX).			*/
/* This software module is an implementation of a part of one or				*/
/* more MPEG-4 tools as specified by the										*/
/* MPEG-4 standard. ISO/IEC gives users of the MPEG-4 standard					*/
/* free license to this software module or modifications thereof				*/
/* for use in hardware or software products claiming conformance				*/
/* to the MPEG-4 standard. Those intending to use this software					*/
/* module in hardware or software products are advised that its					*/
/* use may infringe existing patents. The original developer of					*/
/* this software module and his company, the subsequent							*/
/* editors and their companies, and ISO/IEC have no liability					*/
/* for use of this software module or modifications thereof in					*/
/* an implementation. Copyright is not released for non							*/
/* MPEG-4 standard conforming products. INT ARTEMIS								*/
/* retain full right to use the code for their own purposes,					*/
/* assign or donate the code to a third party and to inhibit					*/
/* third parties from using the code for non MPEG-4 standard					*/
/* conforming products. This copyright notice must be included					*/
/* in all copies or derivative works.    										*/
/* Copyright (c)2002 - 2005														*/
/********************************************************************************/

// FPSet.cpp: implementation of the FPSet class.
// dependences :
//////////////////////////////////////////////////////////////////////

#include "FPSet.h"


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////


FPSet::FPSet(void)
{
	m_FPObjectType=0;
	m_pRoot=new FP(); 
	m_pRoot->SetIndex(-1);
	m_CurrentActiveNode=0;
	m_CurrentCollapsableNode=0;
	m_pSwapNodesToSwitchOn=new ZArray<ZString>;
	m_pSwapNodesToSwitchOff=new ZArray<ZString>;
}


FPSet::~FPSet(void)
{
	if (m_pRoot!=NULL)
	{
		delete m_pRoot; // m_pRoot isn't added to m_pNodes;
		m_pRoot=NULL;
	}
	delete m_pSwapNodesToSwitchOn;
	delete m_pSwapNodesToSwitchOff;

/*	for (int i=0; i<m_pNodes.GetLength(); i++)
	{
		if (m_pNodes.Exists(i))
		{
			if (m_pNodes[i]!=NULL)
			{
				FP* footprint=m_pNodes[i];
				delete footprint;
				footprint=NULL;
			}
		}
	}
*/
/*
 	for (i=0; i<m_pActiveNodes.GetLength(); i++)
	{
		if (m_pActiveNodes[i]!=NULL)
		{
			delete m_pActiveNodes[i];
			m_pActiveNodes[i]=NULL;
		}
	}	

	for (i=0; i<m_pCollapsableNodes.GetLength(); i++)
	{
		if (m_pCollapsableNodes[i]!=NULL)
		{
			delete m_pCollapsableNodes[i];
			m_pCollapsableNodes[i]=NULL;
		}
	}	
*/
}

bool FPSet::AddFootPrint(FP* fp, int parentIndex)
{
	FP* parent;
	
	if (parentIndex!=-1)
		parent=m_pNodes[parentIndex];
	else
		parent=m_pRoot;

	parent->AddChild(fp);
	fp->SetParent(parent);
	m_pNodes.InsertAt(fp->Index(),fp);

	return true;

}

bool FPSet::AddFootPrints(ZArray<FP*>& fpes, int parentIndex)
{
	FP* parent;
	if (parentIndex!=-1)
		parent=m_pNodes[parentIndex];
	else
		parent=m_pRoot;

	for (int i=0; i<fpes.GetLength(); i++)
	{

		FP* newfp=fpes[i];
		
		parent->AddChild(newfp);

		newfp->SetParent(parent);
		
		m_pNodes.InsertAt(newfp->Index(),newfp);
	}

	return true;
}

FP* FPSet::GetFootPrint (int index)
{
	if (m_pNodes.Exists(index))
		return m_pNodes[index];
	else
		return NULL;
}

int FPSet::TestActiveNodes(Vec3f* viewpoint, int nbNodesToTest, ZArray<FP*>& NodesToModify, ZArray<int>& NodesState)
{
	int nbDeveloppedNodes=0;
	int nbTestedNodes=0;

	ZArray<FP*> nodesToDevelop;
	int beginActiveNode=m_CurrentActiveNode;
	bool firsttime=true;
 	while ((nbTestedNodes<nbNodesToTest))
	{
		if (m_CurrentActiveNode>=m_pActiveNodes.GetLength())
			m_CurrentActiveNode=0;
		FP* fp=m_pActiveNodes.ElementAt(m_CurrentActiveNode++);

		if (fp==m_pRoot)
		{
			Develop(fp, NodesToModify, NodesState);
			nbDeveloppedNodes++;
		}
		else if (fp->Children()->GetLength()!=0)
		{
			if (fp->MustBeDevelopped(viewpoint))
			{
				Develop(fp, NodesToModify, NodesState);
				nbDeveloppedNodes++;
			}
		}
		nbTestedNodes++;		
	}
		return nbDeveloppedNodes;
}

void FPSet::Develop(FP* fp, ZArray<FP*>& NodesToModify, ZArray<int>& NodesState)
{
	int i;

	FP* parent=fp->Parent();
	
	if (parent!=NULL)
	{
		m_pCollapsableNodes-=parent;
		parent->SetIsCollapsable(false);
	}
	
	m_pActiveNodes-=fp;
	
	bool find=false;
	for (i=0; i<NodesToModify.GetLength(); i++)
	{
		if (NodesToModify.ElementAt(i)==fp)
		{
			if (NodesState.ElementAt(i)==1)
			{
				NodesState[i]=0;
				find=true;
			}
		}
	}
	if (!find)
	{
		NodesToModify+=fp;
		NodesState+=2;
	}
	fp->SetIsActive(false);
	m_pCollapsableNodes+=fp;
	fp->SetIsCollapsable(true);
	
	ZArray<FP*>* children=fp->Children();
	for (i=0; i<children->GetLength(); i++)
	{
		if ((*children)[i]->Type()==BUILDING)
		{
			FPBuilding* b=(FPBuilding*)(*children)[i];
			if (b->State3D()==NOCOMPUTED)
			{
				b->Reconstruction3D();
			}
		}
		
		if ((*children)[i]->GetNbSwapNodes()>0)
		{
			ZArray<ZString>* swapNodes=(*children)[i]->GetSwapNodes();
			for (int numswap=0; numswap<swapNodes->GetLength(); numswap++)
				(*m_pSwapNodesToSwitchOn)+=(*swapNodes)[numswap];
		}
		m_pActiveNodes+=(*children)[i];
		NodesToModify+=(*children)[i];
		NodesState+=1;
		(*children)[i]->SetIsActive(true);
	}

}

int FPSet::TestCollapsableNodes(Vec3f* viewpoint, int nbNodesToTest, ZArray<FP*>& NodesToModify, ZArray<int>& NodesState)
{
	int nbCollapseNodes=0;
	int nbTestedNodes=0;
	
	ZArray<FP*> nodesToCollapse;
	bool firsttime=true;
	int beginCollapsableNode=m_CurrentCollapsableNode;
	
	while ((nbTestedNodes<=nbNodesToTest))
	{

		if (m_CurrentCollapsableNode>=m_pCollapsableNodes.GetLength())
			m_CurrentCollapsableNode=0;
	
		FP* fp=m_pCollapsableNodes.ElementAt(m_CurrentCollapsableNode++);
		if (fp->MustBeCollapsed(viewpoint))
		{
			Collapse(fp, NodesToModify, NodesState);
			nbCollapseNodes++;
		}
	
		nbTestedNodes++;
	}
	
	nodesToCollapse.Clear();

	return nbCollapseNodes;
}

void FPSet::Collapse(FP* fp, ZArray<FP*>& NodesToModify, ZArray<int>& NodesState)
{

	if (fp!=m_pRoot)
	{
		ZArray<FP*>* children=fp->Children();
		for (int i=0; i<children->GetLength(); i++)
		{
			m_pActiveNodes-=(*children)[i];
			
			bool find=false;
			for (int j=0; j<NodesToModify.GetLength(); j++)
			{
				if (NodesToModify.ElementAt(j)==(*children)[i])
				{
					if (NodesState.ElementAt(j)==1)
					{
						NodesState[j]=0;
						find=true;
					}
				}
			}
			if (!find)
			{
				NodesToModify+=(*children)[i];
				NodesState+=2;
			}
			(*children)[i]->SetIsActive(false);
			if ((*children)[i]->GetNbSwapNodes()>0)
			{
				ZArray<ZString>* swapNodes=(*children)[i]->GetSwapNodes();
				for (int numswap=0; numswap<swapNodes->GetLength(); numswap++)
					(*m_pSwapNodesToSwitchOff)+=(*swapNodes)[numswap];

			}
		}
		m_pCollapsableNodes-=fp;
		fp->SetIsCollapsable(false);
		m_pActiveNodes+=fp;
		NodesToModify+=fp;
		NodesState+=1;
		fp->SetIsActive(true);
	

		FP* parent=fp->Parent();
		if (parent!=NULL)
		{
			ZArray<FP*>* children=parent->Children();
			bool ParentCollapsable=true;
			for (int i=0; i<children->GetLength(); i++)
			{
				if (!((*children)[i]->IsActive()))
				{
					ParentCollapsable=false;
					break;
				}
			}
			if (ParentCollapsable)
			{
				m_pCollapsableNodes+=parent;
				parent->SetIsCollapsable(true);
			}
				
		}
	}

}

void FPSet::AddActiveNode(FP* fp)
{
	m_pActiveNodes+=fp;
	if (fp->Type()==BUILDING)
	{
		FPBuilding* b=(FPBuilding*)fp;
		if (b->State3D()==NOCOMPUTED)
			b->Reconstruction3D();
	}
}