/********************************************************************************/
/* This software module was developped by										*/

/*	   Jerome Royan (MP), jerome.royan@francetelecom.com,						*/
/*                        France Telecom R&D, France							*/

/* in the course of development of the MPEG-4 standard (Part 16 AFX).			*/
/* This software module is an implementation of a part of one or				*/
/* more MPEG-4 tools as specified by the										*/
/* MPEG-4 standard. ISO/IEC gives users of the MPEG-4 standard					*/
/* free license to this software module or modifications thereof				*/
/* for use in hardware or software products claiming conformance				*/
/* to the MPEG-4 standard. Those intending to use this software					*/
/* module in hardware or software products are advised that its					*/
/* use may infringe existing patents. The original developer of					*/
/* this software module and his company, the subsequent							*/
/* editors and their companies, and ISO/IEC have no liability					*/
/* for use of this software module or modifications thereof in					*/
/* an implementation. Copyright is not released for non							*/
/* MPEG-4 standard conforming products. INT ARTEMIS								*/
/* retain full right to use the code for their own purposes,					*/
/* assign or donate the code to a third party and to inhibit					*/
/* third parties from using the code for non MPEG-4 standard					*/
/* conforming products. This copyright notice must be included					*/
/* in all copies or derivative works.    										*/
/* Copyright (c)2002 - 2005														*/
/********************************************************************************/

// FPSet.h: interface for the FPSet class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_FPSet_H__93F1CF93_D37D_11D5_B9F1_000102A02E5B__INCLUDED_)
#define AFX_FPSet_H__93F1CF93_D37D_11D5_B9F1_000102A02E5B__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "StdAfx.h"
#include "FPBuilding.h"

#define FP_BUILDINGS 0

class FPSet  
{
public:
	FPSet(void);
	~FPSet(void);
	

	// Decoder Specific Info

	inline void SetFPObjectType(int type) {m_FPObjectType=type; };
	inline void SetMaxNbFPs(long nbFP) {m_MaxNbFPs=nbFP; };
	inline void SetFPNbBits(int nbBits) {m_FPNbBits=nbBits; };
	inline void SetStep(float step) {m_Step=step; };
	inline void SetMaxX(float maxX) {m_MaxX=maxX; };
	inline void SetMaxY(float maxY) {m_MaxY=maxY; };
	inline void SetMaxAltitude(float maxAltitude) {m_MaxAltitude=maxAltitude; };
	inline void SetMaxHeight(float maxHeight) {m_MaxHeight=maxHeight; };

	inline int FPObjectType(void) {return m_FPObjectType; };
	inline long MaxNbFPs(void) {return m_MaxNbFPs; };
	inline int FPNbBits(void) {return m_FPNbBits; };
	inline float MaxX(void) {return m_MaxX; };
	inline float MaxY(void) {return m_MaxY; };
	inline float MaxAltitude(void) {return m_MaxAltitude; };
	inline float MaxHeight(void) {return m_MaxHeight; };
	
	inline FP* Root() {return m_pRoot; };
	FP* GetFootPrint(int index);

	inline ZArray<FP*>& ActiveNodes() { return m_pActiveNodes;};
	inline ZArray<FP*>& CollapsableNodes() { return m_pCollapsableNodes;};

	bool AddFootPrint(FP* fp, int parentIndex);
	bool AddFootPrints(ZArray<FP*>& fps, int parentIndex);

	void AddActiveNode(FP* fpe);

	int TestActiveNodes(Vec3f* viewpoint, int nbNodesToTest, ZArray<FP*>& NodesToModify, ZArray<int>& NodesState);
	void Develop(FP* fpe, ZArray<FP*>& NodesToModify, ZArray<int>& NodesState);
	int TestCollapsableNodes(Vec3f* viewpoint, int nbNodesToTest, ZArray<FP*>& NodesToModify, ZArray<int>& NodesState);
	void Collapse(FP* fpe, ZArray<FP*>& NodesToModify, ZArray<int>& NodesState);

	inline MediaObject* GetMediaObject() const { return m_pNode; };

	inline void SetMediaObject ( MediaObject* mo) { m_pNode=mo; };

	inline ZArray<ZString>* SwapNodesToSwitchOn() { return m_pSwapNodesToSwitchOn; };

	inline ZArray<ZString>* SwapNodesToSwitchOff() { return m_pSwapNodesToSwitchOff; };

protected :
	
	
	FP* m_pRoot;
	ZMap<FP*> m_pNodes;

	ZArray<FP*> m_pActiveNodes;
	long m_CurrentActiveNode;
	
	ZArray<FP*> m_pCollapsableNodes;
	long m_CurrentCollapsableNode;

	MediaObject* m_pNode;

	ZArray<ZString>* m_pSwapNodesToSwitchOn;
	ZArray<ZString>* m_pSwapNodesToSwitchOff;

	// Decoder Specific Info
	int m_FPObjectType;
	long m_MaxNbFPs;
	int m_FPNbBits;
	float m_Step;
	float m_MaxX;
	float m_MaxY;
	float m_MaxAltitude;
	float m_MaxHeight;

};

#endif // !defined(AFX_FPSet_H__93F1CF93_D37D_11D5_B9F1_000102A02E5B__INCLUDED_)
