/********************************************************************************/
/* This software module was developped by										*/

/*	   Royan Jerme , jerome.royan@francetelecom.com								*/
/*                   France Telecom R&D, France		*/

/* in the course of development of the MPEG-4 standard (Part 16 AFX).			*/
/* This software module is an implementation of a part of one or				*/
/* more MPEG-4 tools as specified by the										*/
/* MPEG-4 standard. ISO/IEC gives users of the MPEG-4 standard					*/
/* free license to this software module or modifications thereof				*/
/* for use in hardware or software products claiming conformance				*/
/* to the MPEG-4 standard. Those intending to use this software					*/
/* module in hardware or software products are advised that its					*/
/* use may infringe existing patents. The original developer of					*/
/* this software module and his company, the subsequent							*/
/* editors and their companies, and ISO/IEC have no liability					*/
/* for use of this software module or modifications thereof in					*/
/* an implementation. Copyright is not released for non							*/
/* MPEG-4 standard conforming products. INT ARTEMIS								*/
/* retain full right to use the code for their own purposes,					*/
/* assign or donate the code to a third party and to inhibit					*/
/* third parties from using the code for non MPEG-4 standard					*/
/* conforming products. This copyright notice must be included					*/
/* in all copies or derivative works.    										*/
/* Copyright (c)2002 - 2005														*/
/********************************************************************************/

// FPETriangulation.cpp: implementation of the FPETriangulation class.
// dependences :
//////////////////////////////////////////////////////////////////////


#include "FPTriangulation.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////


void CALLBACK beginCB (GLenum type=GL_TRIANGLES, void* call_data=NULL)
{
  if (call_data!=NULL)
  {
	  triangulation_struct* triangulation=(triangulation_struct*)call_data;
	  triangulation->isFirstSet = false;
	  triangulation->isSecondSet = false;
	  triangulation->roofTriangleType = type;
	  triangulation->pair = true;
	  triangulation->current=0;
	  triangulation->nbtriangles = 0;
  }
}

void CALLBACK endCB ()
{

}

void CALLBACK putVertexCB (void* v, void* call_data)
{
  double *vec = (double *)v;
  Vec3f tmp;
  
  triangulation_struct* triangulation=(triangulation_struct*)call_data;	

  if(triangulation->isFirstSet) {
    if(triangulation->isSecondSet) {
      if(triangulation->pair){
        tmp = Vec3f(triangulation->firstSet[0], triangulation->firstSet[1], triangulation->firstSet[2]);
		triangulation->triangles[triangulation->current++]=tmp;

        tmp = Vec3f(triangulation->secondSet[0], triangulation->secondSet[1], triangulation->secondSet[2]);
		triangulation->triangles[triangulation->current++]=tmp;

		tmp = Vec3f(vec[0], vec[1], vec[2]); 
		triangulation->triangles[triangulation->current++]=tmp;

		triangulation->nbtriangles++;
      }
      else {
        tmp = Vec3f(triangulation->firstSet[0], triangulation->firstSet[1], triangulation->firstSet[2]);     
		triangulation->triangles[triangulation->current++]=tmp;

        tmp = Vec3f(vec[0], vec[1], vec[2]); 
		triangulation->triangles[triangulation->current++]=tmp;

        tmp = Vec3f(triangulation->secondSet[0], triangulation->secondSet[1], triangulation->secondSet[2]);    
		triangulation->triangles[triangulation->current++]=tmp;

		triangulation->nbtriangles++;
      }
      
      switch (triangulation->roofTriangleType){
      case GL_TRIANGLES:
        triangulation->isSecondSet = false;
        triangulation->isFirstSet  = false;
        break;
      case GL_TRIANGLE_STRIP:
        triangulation->firstSet[0]  = triangulation->secondSet[0];
        triangulation->firstSet[1]  = triangulation->secondSet[1];
        triangulation->firstSet[2]  = triangulation->secondSet[2];
        triangulation->secondSet[0] = vec[0];
        triangulation->secondSet[1] = vec[1];
        triangulation->secondSet[2] = vec[2];
        triangulation->pair         = !triangulation->pair;
        break;
      case GL_TRIANGLE_FAN:
        triangulation->secondSet[0] = vec[0];
        triangulation->secondSet[1] = vec[1];
        triangulation->secondSet[2] = vec[2];
        break;
      }
    }
    else{
      triangulation->isSecondSet  = true;
      triangulation->secondSet[0] = vec[0];
      triangulation->secondSet[1] = vec[1];
      triangulation->secondSet[2] = vec[2];
    }
  }
  else{
    triangulation->isFirstSet  = true;
    triangulation->firstSet[0] = vec[0];
    triangulation->firstSet[1] = vec[1];
    triangulation->firstSet[2] = vec[2];
  }
}

//typedef  void ( *__TessCallBackType)(void);

void triangule(ZArray<Vec3f> &contour, ZArray< ZArray< Vec3f > > &internContours, ZArray<Vec3f>& triangles)
{
  unsigned int i, j;
  GLUtesselator *tess = gluNewTess();

  //gluTessCallback(tess, GLU_TESS_BEGIN_DATA,  (__TessCallBackType)beginCB);
  //gluTessCallback(tess, GLU_TESS_VERTEX_DATA, (__TessCallBackType)putVertexCB);
  //gluTessCallback(tess, GLU_TESS_END_DATA, endCB);

  gluTessCallback(tess, GLU_TESS_BEGIN_DATA,  (GLvoid (CALLBACK*)())beginCB);
  gluTessCallback(tess, GLU_TESS_VERTEX_DATA, (GLvoid (CALLBACK*)())putVertexCB);
  gluTessCallback(tess, GLU_TESS_END, (GLvoid (CALLBACK*)())endCB);

  gluTessBeginPolygon(tess, &triangulation);
  gluTessBeginContour(tess);

  for (i=0; i<(unsigned int)contour.GetLength(); i++)
  {
	  GLdouble *v = (GLdouble *)calloc(3, sizeof(GLdouble));
	  v[0]= contour.ElementAt(i).x;
	  v[1]= contour.ElementAt(i).y;
	  v[2]= contour.ElementAt(i).z;
	  gluTessVertex(tess, v, &triangulation);
  }

  gluTessEndContour(tess);

  // Contour internes
  for (i = 0; i < (unsigned int)internContours.GetLength(); i++){
    gluTessBeginContour(tess);

    for(j = 0; j < (unsigned int)internContours.ElementAt(i).GetLength(); j++){
      GLdouble *v = (GLdouble *)calloc(3, sizeof(GLdouble));
      v[0] = internContours.ElementAt(i).ElementAt(j).x;
      v[1] = internContours.ElementAt(i).ElementAt(j).y;
      v[2] = internContours.ElementAt(i).ElementAt(j).z;
      gluTessVertex(tess, v, &triangulation);       
    }
     
    gluTessEndContour(tess);
  }

  gluTessEndPolygon(tess);
  gluDeleteTess(tess);
  glFlush();
  for (int k=0; k<triangulation.nbtriangles*3; k++)
	triangles+=triangulation.triangles[k];
  
}

