// Facade3D.h: interface for the CFacade3D class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_FACADE3D_H__E9AA6DAF_503F_4FC3_89EF_692E6AC135D0__INCLUDED_)
#define AFX_FACADE3D_H__E9AA6DAF_503F_4FC3_89EF_692E6AC135D0__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


#include "polyline2D.h"
#include "segment3d.h"

//#define TRACER

#ifdef TRACER
	#include "tracer.h"
#endif

typedef enum {FORMAT_GL_V3F, FORMAT_GL_T2F_V3F, FORMAT_GL_N3F_V3F , FORMAT_GL_T2F_N3F_V3F} GLformat;
typedef vector<Segment3D> FACETTE, RING3D;
typedef vector<FACETTE> OBJECT3D;

// precision used for geom operations
#define GEOM_PRECISION	0.001

#if 0
struct lt_Segment
{
  bool operator()(Segment2D s1, Segment2D s2) const
	{
/*	  return ((s1.source().x()<s2.source().x()) 
		   || ((s1.source().x()==s2.source().x()) && (s1.source().y()<s2.source().y())));
*/	
		return (s1.source() < s2.source()); // || ( s1.source().equal(s2.source(), GEOM_PRECISION) && (s1.target().y() < s2.target().y()));
  }
};
#endif

struct lt_Segment
{
	bool operator()(Segment2D s1, Segment2D s2) const
	{
		return ((s1.source() < s2.source()));// || (s1.source() == s2.source()));
	}
};


typedef multiset<Segment2D, lt_Segment> MultisetOfSegments;


struct CellParam
{
	float	originX;
	float	originY;
	float	originZ;
	float	width;
	float	height;
	float	posX;
	float	posY;
	float	scaleX;
	float	scaleY;
	float	repeatX;
	float	repeatY;
	bool	brepeat;
};

struct ModelParam
{
	float	nbModels;
	float	scaleX;
	float	scaleY;
	float	scaleZ;
	float*  atranslations;
};







/*****************************************************************************
		CLASS:	FacadeVertex
*****************************************************************************/
class FacadeVertex
{
public:

	FacadeVertex(void) : m_x2d(0), m_y2d(0), m_x3d(0), m_y3d(0), m_z3d(0), m_xTexture(0), m_yTexture(0) {};
	FacadeVertex(float x3d, float y3d, float z3d, float x2d, float y2d, float xTexture, float yTexture);
	virtual ~FacadeVertex() {};

	bool Match(FacadeVertex Vtx);

	FacadeVertex & operator =(const FacadeVertex &in)
	{
		m_x2d = in.m_x2d;
		m_y2d = in.m_y2d;
		m_x3d = in.m_x3d;
		m_y3d = in.m_y3d;
		m_z3d = in.m_z3d;
		m_xTexture = in.m_xTexture;
		m_yTexture = in.m_yTexture;

		return (*this);
	};

	float			m_x2d, m_y2d;				// 2D coordinates for polygon computation
	float			m_x3d, m_y3d, m_z3d;		// 3D coordinates for GL array filling
	float			m_xTexture;
	float			m_yTexture;
	unsigned int	m_nID;
};






/*****************************************************************************
		CLASS:	CFacade3D
*****************************************************************************/
class CFacade3D  
{
public:
	CFacade3D();
	virtual ~CFacade3D();

public:
	// init
	void SetNormal(float xNormal, float yNormal, float zNormal);
	void SetFacade(float xFacade, float yFacade, float zFacade);

public:
	// to add cell geometry
	void AddOutlineQuad(CellParam cellparam);
	void AddHoleQuad(CellParam cellparam);
	void AddHoleModel(CellParam cellparam);
	void AddQuads(CellParam cellparam);
	void AddLines(CellParam cellparam);

public:
	// for model construction + section computation
	void InitModel();
	void AddModelGeometry(int nVtx, float* aVtx, int nIdx, int* aIdx);
	void EndModel();

	// for OPENGL output
	int		GetNbVertices(void) { return m_NbVertices; };
	int		GetNbQuads(void) { return m_NbQuads; };
	int		GetNbTriangles(void) { return m_NbTriangles; };
	int		GetNbLines(void) { return m_NbLines; };
	int		FillVertexArray(GLformat format, float** vertices, bool bInvTex=false);
	void	FillIndexArrayForGLQuads(int* nbquads, unsigned int** QuadIndices);
	void	FillIndexArrayForGLTriangles(int* nbtriangles, unsigned int** TriangleIndices);
	void	FillIndexArrayForGLLines(int* nblines, unsigned int*** LinesIndices);
	
	// for Model position output
	void	FillModelParam(CellParam cellparam, ModelParam& modelparam); 

public:
	// geometry computation
	void	BuildFacadeQuads(void);
	void	BuildFacadeTriangles(void);
	void	BuildFacadeLines(void);

private:
	// model3d
	void	ComputeModelSection(void);
	void	ComputeNbModels(void);
	void	ComputeTranslations(void);
	void	ComputeSectionRange(vector <Polyline2D> vHoles);
	void	ComputeModelScale(void);
	void	FillModelTranslationArray(float** translations);

private:
	// 2d geometry
	void	AddVertex(float x3d, float y3d, float z3d, float x2d, float y2d, float xTexture, float yTexture);
	void	AddSegment(Segment2D segment);
	void	AddSection(vector<Polyline2D> vHoles);
	void	AddRepeatSection(vector<Polyline2D> vHoles, int iRepeatX=0, int iRepeatY=0);

private:
	// 2d geometry for triangles display
	vector<Segment2D> SimplifySegments(vector<Segment2D> vSegments);
	int RemoveOppositeSuperposed(Segment2D &pSeg1, Segment2D &pSeg2);
	vector<Polyline2D> BuildPolylines(vector<Segment2D> vInSegments);
	bool FindNextSegment(Segment2D pSegment, vector<Segment2D> VSegments, Segment2D* pNextSegment);
	vector<Polygon2D> BuildPolygons(vector<Polyline2D> vPolylines);
	vector<Polygon2D> BuildTriangles(vector<Polygon2D> vPolygons);

private:
	// opengl arrays construction
	void  BuildIndiceArray(vector<Polygon2D> vTriangles);
	void  BuildVertexArray(void);
	unsigned int AppendVertex(Point2D point);
	FacadeVertex GetVertex(Point2D point);

private: 
	// 3d model section computation
	vector<Segment2D> ComputeSectionSegments(OBJECT3D object, Plane p);
	vector<Segment2D> CleanSectionSegments(vector<Segment2D> vSeg);
	int IntersectionFacettePlane(FACETTE facette, Plane p, vector<Segment2D> & vinter);
	vector<Segment2D> SimplifySuperposedSegments(vector<Segment2D> vSegments);
	int RemoveSuperposed(Segment2D* pSeg1, Segment2D* pSeg2);
	vector<Segment2D> SplitInterSegments(vector<Segment2D> vSegments);
	vector<Polyline2D> BuildSectionContour(vector<Segment2D> vInSegments);
	bool FindNextHoleSegment(Segment2D Segment, MultisetOfSegments MSeg, Segment2D* pNextSegment);
	float ComputeAngle(Segment2D Segment1, Segment2D Segment2);
	int BuildSectionHole(vector<Segment2D> vInSegments, Polyline2D &hole);

private:
	// test purpose: 3d model construction
	OBJECT3D BuildModelObject();

private:
	// conversion
	FacadeVertex Point2DToFacadeVertex(Point2D point, int iRepeatX=0, int iRepeatY=0);
	Point2D ModelToFacadeCoordinates(Point2D ptModel, int iRepeatX=0, int iRepeatY=0);
	Point3D FacadeToSceneCoordinates(Point2D ptFacade);





private:
	vector<FacadeVertex>	m_VInVertex;	// initial vertices
	vector<FacadeVertex>	m_VOutVertex;	// left vertices after computation
	vector<Segment2D>		m_VSegments;	// segments of polygon 

	float			m_xNormal, m_yNormal, m_zNormal;	// normal vector to facade
	float			m_xFacade, m_yFacade, m_zFacade;	// vector along facade

private:
	// opengl info
	int				m_NbVertices;		// nb of vertices
	int				m_NbQuads;			// nb of quads
	int				m_NbTriangles;		// nb of triangles
	int				m_NbLines;			// nb of lines
	float*			m_Vertices;			// array of vertices coordinates (for GLInterleaved Array)
	unsigned int*	m_IndicesQuads;		// array of quad indices
	unsigned int*	m_IndicesTris;		// array of triangles indices
	unsigned int**	m_IndicesLines;		// array of line indices

private:
	// model info
	float					m_XminSection, m_XmaxSection, m_YminSection, m_YmaxSection;	// bounding box of section in model coordinates
	int						m_NbModels;			// number of models repeated
	vector<VectorGeom3D>	m_VTranslations;	// contains translations for models
	float					m_scaleModelX, m_scaleModelY, m_scaleModelZ;
	OBJECT3D				m_Model3D;			// 3d model
	vector<Polyline2D>		m_ModelSection;		// section of model

	// cell geometry info
	CellParam		m_maincell;
	CellParam		m_modelcell;

#ifdef TRACER
	CTracer			m_Tracer;
#endif
};

#endif // !defined(AFX_FACADE3D_H__E9AA6DAF_503F_4FC3_89EF_692E6AC135D0__INCLUDED_)
