#ifndef droite2d_h_
#define droite2d_h_
/*----------------------------------------------------------------
	Projet : Visualisation Villes / Librairie gomtrique de base.
	Fichier: Droite2D.h
	Date 	 :
	L.B 06/2002
	
	Commentaires:
	
-----------------------------------------------------------------*/


#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "GeomDefs.h"
#include "Point2D.h"
#include "Vector2D.h"

class Droite2D  
{
public:
	//! Constructeur par dfaut
	Droite2D();
	
	//! Constructeur
	/*!
	 * @param p : un sommet appartenant  la droite
	 * @param v : le vecteur directeur de la droite
	!*/
	Droite2D(Point2D p, Vector2D v);	
	
	//! Constructeur
	/*!
	 * @param p1 : un sommet appartenant  la droite
	 * @param p2 : un deuxime sommet appartenant  la droite diffrent de p1
	!*/
	Droite2D(Point2D p1, Point2D p2);

	//! Constructeur par recopie
	/*!
	 * @param s : une droite
	!*/
	Droite2D(Droite2D &s);

	//! Destructeur
	virtual ~Droite2D();
	
	//! Operateur de recopie
	/*!
	 *  @param in : une droite
	!*/
	Droite2D & operator =(const Droite2D &in);
	
	//! Retourne l'origine de la droite
	virtual Point2D origine() const;

	//! Retourne le vecteur directeur de la droite
	virtual Vector2D direction() const;

	//! Retourne les coefficients de la droite definit par ax+by+c=0
	virtual void coefficients(FT &a, FT &b, FT &c) const;

	//! Dtermine la position d'un sommet par rapport a la droite
	/*!
	 * @return LEFT, ON, ou RIGHT
	!*/
	virtual ORIENTATION side(const Point2D &p) const;

	//! Dtermine si un sommet appartient  la droite
	/*!
	 * @return true si appartenance, false sinon
	!*/
	virtual bool appartenance(const Point2D &p, FT null_value= LIBGEOM_NULL_VALUE) const;
	int dimension() const;

protected:
	Droite2D * m_impl;
};

#endif // _h_

// #eof "$Id: Droite2D.h,v 1.1 2006/10/20 14:14:55 granger Exp $"
