/*----------------------------------------------------------------
	Projet : Visualisation Villes / Librairie gomtrique de base.
	Fichier: Polygon2D.h
	Date 	 :
	A.C. 01/2001
	
	Commentaires:
	
-----------------------------------------------------------------*/









//


#if !defined(AFX_POLYGON2D_H__F0BA0F13_A9FA_496B_BD4F_9D2BAA0FE59A__INCLUDED_)
#define AFX_POLYGON2D_H__F0BA0F13_A9FA_496B_BD4F_9D2BAA0FE59A__INCLUDED_


#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "Polyline2D.h"
#include "BBox2D.h"
#include "GeomDefs.h"
#include "GeomOperators.h"
#include "GeomSTLDefs.h"
#include "Triangulation.h"



typedef enum {EMPTY_POLYGON,HOLED_POLYGON,SIMPLE_POLYGON, TRIANGLE} typePolygon;

//! Polygone  trou
/*!
 *	Cette classe permet de dfinir des polygones 2D  trous.
 *	Contraintes :
 *		- 2 segments conscutifs ne sont pas colinaires.
 *		- Un trou ne peut contenir de polygones.
 *		- Les trous sont strictement compris dans l'enveloppe externe, et leur intersection est nulle
 *		- L'enveloppe externe est oriente dans le sens des aiguilles d'une montre
 *		- Les trous sont orients dans le sens inverse
 *
 *	Structure :
 *		- Un polygone est constitu d'un tableau de polygone. Le premier polygone correspond au polygone lui-mme (enveloppe externe)
 *	et les suivants correspondent aux trous.
 *		- Tout polygone fait rfrence  son implmentation du type Polygon2DX, contenant les sommets du polygone.
 */

//class polygon2D;
typedef vector<Polygon2D> POLYGON_VECTOR;
typedef POLYGON_VECTOR::iterator POLYGON_ITERATOR;
class Polygon2D  
{
public:



	//! Constructeur par dfaut
	Polygon2D();
	//! Destructeur par dfaut
	virtual ~Polygon2D();
	
	//! Constructeur prenant en entre un tableau de Polygon2D. A utiliser en priorit plutt que AddPolygon(...).
	/*!
	 *	\param nbRings nombre de polygones.
	 *  \param rings tableau de polygones, le premier correspondant  l'enveloppe externe, les suivants aux trous.
	 */
	Polygon2D(int nbRings,	Polygon2D * rings);
	
	//! Constructeur prenant en entre un tableau de Point2D. A utiliser en priorit au lieu de AddPoint(...).
	/*!
	 *	\param nbPoints nombre de points du polygone externe.
	 *  \param pts tableau des points du polgones, orient CW pour une enveloppe externe et CCW pour un trou.
	 */
	Polygon2D(int nbPoints,	Point2D *	pts);
	
	//! constructeur
	/*
	 *	Le premier lment du tableau des polygones pointe sur l'implmentation de s.
	 */
	Polygon2D(const Polygon2D & s);
	
	Polygon2D(const Polyline2D & p);

	//! Recopie de la structure de s
	/*
	 *	Reconstruit une nouvelle implmentation
	 */
	void Copy(const Polygon2D & s);


	//! Oprateur de recopie
	Polygon2D & operator =(const Polygon2D &in);

	//! Retourne la ieme polyligne fermee du polygone
	Polygon2D operator [](int i) const;

	//! Operateur d'egalit
	bool operator ==(const Polygon2D & p) const;

	//! Operateur d'inegalit
	bool operator !=(const Polygon2D & p) const;	

	//! Retourne vrai si il n'existe aucune implementation du polygone (apres un simple constructeur par dfaut)
	bool empty() const;

	//! Permet de connatre le nombre de polygones (enveloppe externe + trous) 
	virtual int GetNbRings() const;

	//! Retourne un tableau de l'ensemble des sommets du polygone
	virtual POINT_VECTOR PointVector() ;
	
	//! Retourne un tableau de l'ensemble des segments du polygone
	virtual SEGMENT_VECTOR SegmentVector() ;
	
	//! Retourne la boite englobante ax selon x et y.
	virtual BBox2D bbox() const;
	
	//! Permet d'ajouter un point en fin de tableau des sommets du polygone.
	virtual bool AddPoint(const Point2D &in);
	
	//! Permet d'ajouter un ensemble de points en fin de tableau des sommets du polygone.
	virtual bool AddPoints(int nbPoints,	Point2D *	pts);
	
	//! Permet d'insrer un point en position pos dans le tableau des sommets de l'enveloppe externe du polygone.
	virtual bool InsertPoint(const Point2D &in,int pos);
	
	//! Permet d'insrer un ensemble de points en position pos dans le tableau des sommets de l'enveloppe externe du polygone.
	virtual bool InsertPoints(int nbPoints, Point2D * pts,int pos);

	//! Permet de supprimer nbPoints  partir de la position pos dans le tableau des sommets de l'enveloppe externe du polygone.
	virtual bool SuppressPoint(int pos, int nbPoints=1);
	
	//! Permet de supprimer le point2D p du polygone  trou
	virtual bool SuppressPoint(Point2D & p);

	//! Retourne le ime point du tableau des sommets de l'enveloppe externe du polygone.
	virtual bool GetPoint(Point2D &out, int i) const;

	//! Retourne le nombre de sommets constituant le polygone (enveloppe externe).
	virtual int GetNbPoints() const;

	//! Permet d'ajouter un trou au polygone
	virtual bool AddPolygon(const Polygon2D & p);

	virtual bool AddPolyline(const Polyline2D &p);

	//! Permet d'insrer un trou en position pos au polygone
	virtual bool InsertPolygon(const Polygon2D & p, int pos );
	virtual bool InsertPolyline(const Polyline2D & p, int pos );
	//! Permet de supprimer un trou en position pos
	virtual bool SuppressPolygon(int pos, int nbPoly=1);
	virtual bool SuppressPolyline(int pos, int nbPoly=1);

	//! Retourne le perimtre de l'enveloppe externe
	virtual FT SimplePerimeter();

	//! Retourne le perimetre du polygone (aire enveloppe externe - aire des trous)
	virtual FT HoledPerimeter();

	//! Retourne l'aire de l'enveloppe externe
	virtual FT SimpleArea() const;

	//! Retourne l'aire du polygone (aire enveloppe externe - aire des trous)
	virtual FT HoledArea() const;

	//! Verifie si un point appartient aux artes d'un polygone  trou.
	/*!
	 * @param pt le point  tester
	 * @param num_ring l'anneau du polygone dont une arte contient le point test (-1 si le point n'appartient pas aux artes du polygone).
	 * @param pos la position du point cible du segment contenant le point test (-1 si le point n'appartient pas aux artes du polygone).
	 */
	bool has_on(const Point2D &, int & num_ring, int & pos, FT dist) const;

	//! Verifie si un point appartient aux artes d'un polygone simple.
	/*!
	 * @param pt le point  tester
	 * @param pos la position du point cible du segment contenant le point test (-1 si le point n'appartient pas aux artes du polygone).
	 */
	virtual bool has_on(const Point2D &pt, int & pos, FT dist) const;

	virtual bool HasPoint(const Point2D &pt, int &ring, int &pos) const;

	//! Verifie si un polygone est simple ou  trous.
	bool isSimple() const;

	//! Inverse l'orientation des sommets du polygone
	virtual void InverseOrientation();

	//! Calcul la distance minimale d'un point au polygone  trou
	virtual FT DistanceFromPoint2D(const Point2D & p) const;

	//! Calcul la dsitance minimale d'un polygone p  trou au polygone (!! Ces polygones ont une intersection nulle !)
	virtual FT DistanceFromPolygon2D(const Polygon2D & p) const;

	//! Vrifie si un point est  l'interieur de l'enveloppe externe du polygone (!! Attention, problme si le point est sur la frontire du polygone).
	virtual int PointInsideExternEnveloppe(const Point2D & p, FT precision=LIBGEOM_NULL_VALUE) const;
	
	//! Vrifie si un point est  l'interieur du polygone (!! Attention, problme si le point est sur la frontire du polygone).
	virtual int PointInside(const Point2D & p, FT precision=LIBGEOM_NULL_VALUE) const;

	//! Vrifie si p est  l'interieur de l'enveloppe externe du polygone this 
	virtual int PolygonInsideExternEnveloppe(Polygon2D & p, FT precision=LIBGEOM_NULL_VALUE) const;

	//! Vrifie si p est  l'interieur de this 
	virtual int PolygonInside(Polygon2D & p, FT precision=LIBGEOM_NULL_VALUE) const;

	//! return -1 si seg est  l'extrieur de l'enveloppe externe ou en intersection avec l'enveloppe externe
	//!		    0 si seg est sur l'enveloppe externe
	//!		    1 si seg est  l'intrieur de l'enveloppe externe
	virtual int SegmentInside(const Segment2D &seg, FT precision= LIBGEOM_NULL_VALUE) const ;

	//! Retourne la position de p par rapport  l'enveloppe externe de this.
	virtual POLYLINE_POSITION PolygonPosition(const Polygon2D &p,
														  FT precision= LIBGEOM_NULL_VALUE) const ;

	//! Vrifie si p et this on une arrte en commun
	virtual bool Adjacent(const Polygon2D &p) const;
	
	//! Enleve les points en trop, qui sont colineaires
	virtual int RemoveColinearVertices(Polygon2D &newFootPrint, const FT colinear_threshold = 1e-1) const;

	//! Dtermine si un polygon est convex ou non
	virtual bool IsConvex() const;

	//! Decoupe le polygone en triangles
	/*!
	 *  @return un tableau des triangles issus du dcoupage
	!*/
	virtual vector<Polygon2D> clipping() const;

	//! Retourne le centre de gravit de l'enveloppe externe du polygone
	virtual Point2D GravityCenter() const;

	//! Dilatation d'un polygone dont les sommets sont CW
	virtual Polygon2D dilate(FT epsilon) const;

	//! Recherche d'un point  l'interieur du polygone
	virtual Point2D getInsidePoint() const;

	//! Retourne l'index du trou edjacent  l'enveloppe du polygone, -1 sinon
	virtual int HoleConnexeToEnveloppe() const;

	//! Retourne l'index du trou constitu de 2 trous connexes en 1 sommet
	virtual int ConnexeHoles() const;

	//! corrige un polygone qui  un trou adjacent  son enveloppe : n trous->n-1 trous
	virtual void CorrectHoleConnexeToEnveloppe(int num_hole);

	//! Gnre 2 trous  partir d'un trou contenant 2 fois le mme sommet
	virtual void CorrectConnexeHoles(int num_hole);

	virtual Polyline2D GetPolyline(int i) const;
	virtual POLYLINE_VECTOR GetPolylines() const;


protected:
	//! Tableau de pointeurs sur des polygones (le 1er correspond  l'enveloppe externe, les suivants correspondent aux trous).
	Polygon2D * m_impl;
	//! Type du polygone
	typePolygon m_type;
};

#endif // !defined(AFX_POLYGON2D_H__F0BA0F13_A9FA_496B_BD4F_9D2BAA0FE59A__INCLUDED_)
