// Polyline.h: interface for the Polyline2D class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_POLYLINE_H__6A723C72_5EF9_4E7D_9B13_27D6070EA38E__INCLUDED_)
#define AFX_POLYLINE_H__6A723C72_5EF9_4E7D_9B13_27D6070EA38E__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

typedef enum  POLYLINE_POSITION{
		POLYLINE_INTERSECTION, // les deux polygones sont en intersection
		POLYLINE_INCLUSION, // this inclus la polyline passe  la mthode polylineRelation
		POLYLINE_INCLUDED, // this est inclus par la polyline passe  la mthode polylineRelation
		POLYLINE_EQUALITY, // galit entre les deux polylines
		POLYLINE_EMPTY, // pas de relation entre les deux polylines
		POLYLINE_INTERNAL_EDGE_CONNECTED, // polyline interne  this et connexe au niveau d'une ou de plusieurs artes.
		POLYLINE_INTERNAL_VERTICE_CONNECTED, // polyline interne  this et connexe par un ou plusieurs sommets.
		POLYLINE_EXTERNAL_EDGE_CONNECTED, // polyline externe  this et connexe au niveau d'une ou de plusieurs artes.
		POLYLINE_EXTERNAL_VERTICE_CONNECTED // externe  this et connexe au niveau de un ou de plusieurs sommets.
	};


#include "BBox2D.h"
#include "GeomDefs.h"
#include "GeomOperators.h"
#include "GeomSTLDefs.h"
#include "Triangulation.h"


class Polyline2D  
{
public:
	//! Constructeur par dfaut
	Polyline2D();
	
	//! Destructeur par dfaut
	virtual ~Polyline2D();

	//! Constructeur  partir d'un tableau de sommets
	/*!
	 *  @param nbPoints : nombre de sommets
	 *  @param pts : pointeur sur le tableau de sommets
	!*/
	Polyline2D (int nbPoints,	Point2D *	pts);

	//! Constructeur par recopie
	Polyline2D(const Polyline2D & p);

	//! Oprateur de recopie
	Polyline2D & operator =(const Polyline2D &in);

	//! Permet d'accder au ime point (i=0 l'enveloppe externe, i>0 le ime trou).
	virtual Point2D operator [](int i) const;

	//! Operateur d'egalit
	bool operator ==(const Polyline2D & p) const;

	//! Operateur d'inegalit
	bool operator !=(const Polyline2D & p) const;	

	//! Detecte si l'implmentation existe (ce qui n'est pas le cas aprs l'appel du constructeur par defaut)
	bool empty() const;

	//! Retourne l'ensemble des sommets de la polyligne fermee
	virtual POINT_VECTOR PointVector() const;
	
	//! Retourne l'ensemble des segments de la polyligne fermee
	virtual SEGMENT_VECTOR SegmentVector() const ;
	
	//! Retourne la boite englobante ax selon x et y.
	virtual BBox2D bbox() const;
	
	//! Permet d'ajouter un point en fin de tableau des sommets du polygone.
	virtual bool AddPoint(const Point2D in);
	
	//! Permet d'ajouter un ensemble de points en fin de tableau des sommets du polygone.
	virtual bool AddPoints(int nbPoints,	Point2D *	pts);
	
	//! Permet d'insrer un point en position pos dans le tableau des sommets de l'enveloppe externe du polygone.
	virtual bool InsertPoint(const Point2D in,int pos);
	
	//! Permet d'insrer un ensemble de points en position pos dans le tableau des sommets de l'enveloppe externe du polygone.
	virtual bool InsertPoints(int nbPoints, Point2D * pts,int pos);

	//! Permet de supprimer nbPoints  partir de la position pos dans le tableau des sommets de l'enveloppe externe du polygone.
	virtual bool SuppressPoint(int pos, int nbPoints=1);
	
	//! Permet de supprimer le point2D p du polygone  trou
	virtual bool SuppressPoint(Point2D & p);

	//! Retourne le ime point du tableau des sommets de l'enveloppe externe du polygone.
	virtual bool GetPoint(Point2D &out, int i) const;

	//! Retourne le nombre de sommets constituant le polygone (enveloppe externe).
	virtual int GetNbPoints() const;

	//! Retourne le perimtre de l'enveloppe externe
	virtual FT Perimeter();

	//! Retourne l'aire de l'enveloppe externe
	virtual FT Area() const;

	//! Verifie si un point appartient aux artes d'un polygone  trou.
	/*!
	 * @param pt le point  tester
	 * @param num_ring l'anneau du polygone dont une arte contient le point test (-1 si le point n'appartient pas aux artes du polygone).
	 * @param pos la position du point cible du segment contenant le point test (-1 si le point n'appartient pas aux artes du polygone).
	 */
	bool has_on(const Point2D &, int & num_ring, int & pos, FT dist) const;

	//! Verifie si un point appartient aux artes d'un polygone simple.
	/*!
	 * @param pt le point  tester
	 * @param pos la position du point cible du segment contenant le point test (-1 si le point n'appartient pas aux artes du polygone).
	 */
	virtual bool has_on(const Point2D &pt, int & pos, FT dist) const;

	virtual bool HasPoint(const Point2D &pt, int &pos)const;

	//! Inverse l'orientation des sommets du polygone
	virtual void InverseOrientation();

	//! Calcul la distance minimale d'un point au polygone  trou
	virtual FT DistanceFromPoint2D(const Point2D & p) const;

	//! Calcul la dsitance minimale d'un polygone p  trou au polygone (!! Ces polygones ont une intersection nulle !)
	virtual FT DistanceFromPolyline2D(const Polyline2D & p) const;

	//! Vrifie si un point est  l'interieur de l'enveloppe externe du polygone (!! Attention, problme si le point est sur la frontire du polygone).
	//! Retourne : -1 si p hors de la polyline, 0 si p sur une arte de la polyline, et 1 si p  l'interieur de la polyline
	virtual int PointInside(const Point2D & p, FT precision=LIBGEOM_NULL_VALUE) const;

	//! Vrifie si p est  l'interieur de l'enveloppe externe du polygone this 
	virtual int PolylineInside(Polyline2D & p, FT precision=LIBGEOM_NULL_VALUE) const;

	//! Pr-condition : this et p ne doivent pas avoir de points colinaires non pertinents.
	virtual POLYLINE_POSITION PolylinePosition(const Polyline2D &p,
											   FT precision= LIBGEOM_NULL_VALUE) const ;

	//! return -1 si seg est  l'extrieur de this ou en intersection avec this.
	//!		    0 si seg est sur une arte de this
	//!		    1 si seg est  l'intrieur de this.
	virtual int SegmentInside(const Segment2D &seg, FT precision= LIBGEOM_NULL_VALUE) const ;

	//! Vrifie si p et this on une arrte en commun
	virtual bool Adjacent(Polyline2D &p);

	//! Decoupe le polygone en triangles
	/*!
	 *  @return un tableau des triangles issus du dcoupage
	!*/
	virtual vector<Polygon2D> clipping() const;

	//! Retourne le centre de gravit de la polyligne
	virtual Point2D GravityCenter() const;

	// Dilatation d'un polygone dont les sommets sont CW
	virtual Polyline2D dilate(FT epsilon) const;

	// Recherche d'un point  l'interieur du polygone
	virtual Point2D getInsidePoint();

protected:
	//! Tableau de pointeurs sur des polygones (le 1er correspond  l'enveloppe externe, les suivants correspondent aux trous).
	Polyline2D * m_impl;
};

#endif // !defined(AFX_POLYLINE_H__6A723C72_5EF9_4E7D_9B13_27D6070EA38E__INCLUDED_)
