// RoofConstruction.h: interface for the RoofConstruction class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_ROOFCONSTRUCTION_H__29C528BB_56E6_4D44_8051_3853423F151C__INCLUDED_)
#define AFX_ROOFCONSTRUCTION_H__29C528BB_56E6_4D44_8051_3853423F151C__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

typedef enum {FORMAT_GL_V3F, FORMAT_GL_T2F_V3F, FORMAT_GL_N3F_V3F , FORMAT_GL_T2F_N3F_V3F} GLformat;

#define EAVE_IN_SKELETON


#include "ray.h"
#include "intersection.h"
#include "vertexlist.h"
#include "geomSTLdefs.h"
#include "skeletonline.h"
#include "point3d.h"
#include "roofline.h"
#include "polygon2D.h"

/*
#ifdef _MSC_VER
#pragma warning(disable: 4786)
#endif
*/

#include <assert.h>
#include <deque>
#include <stack>
#include <queue>



//using namespace std;



typedef vector <CPoint2D>		SingleContour;		// outer boundary or a hole
typedef vector <SingleContour>	BuildingContour;	// contour of building



//#if (defined __WATCOMC__) || (defined __GNUC__) || (defined _MSC_VER)
//  typedef priority_queue <deque <Intersection>, greater <Intersection> > IntersectionQueue;
//#else
  typedef priority_queue <CIntersection, deque <CIntersection>, greater <CIntersection> > IntersectionQueue;
//#endif

 

class Skeleton : public list <CSkeletonLine>
{
  public:
    void push_back (const CSkeletonLine &x)
                   {
                     ((CSkeletonLine &)x).SetID( size () );     // automaticke cislovani
                     list <CSkeletonLine> :: push_back (x);
                   }
};


struct lt_RoofLine
{
  bool operator()(CRoofLine l1, CRoofLine l2) const
	{
	  return ((l1.m_LowerPoint.x()<l2.m_LowerPoint.x()) 
		   || ((l1.m_LowerPoint.x()==l2.m_LowerPoint.x()) && (l1.m_LowerPoint.y()<l2.m_LowerPoint.y())));
	}
};

typedef multiset<CRoofLine, lt_RoofLine> MultisetOfLines;
typedef list <Point3D> ListVertices;
typedef vector <Polyline2D> VectorOfLines;
typedef vector <CRoofLine> VectorOfRoofLines;
typedef vector <VectorOfRoofLines> VectorOfPolygons;
typedef vector <Polyline2D> VectorOfPolyline2D;
typedef vector <Polygon2D> VectorOfPolygon2D;
typedef vector <Point2D> VectorOfPoint2D;


class CRoofConstruction  
{
public:
	CRoofConstruction();
	virtual ~CRoofConstruction();

	FT			GetRoofAngle(void) const;
	FT			GetBuildingHeight(void) const;
	FT			GetEave(void) const;
	FT			GetRoofMaxHeight(void) const;
	typeRoof	GetRoofType(void) const;
	FT			GetRoofEave(void) const;
	bool		GetTruncatedFlag(void) const;
	void		SetFootprint(const Polygon2D in);
	void		SetRoofAngle(const FT in);
	void		SetBuildingHeight(const FT in);
	void		SetEave(const FT in);
	void		SetRoofMaxHeight(const FT in);
	void		SetRoofType(const typeRoof in);
	void		SetRoofEave(const FT in);
	void		SetTruncatedFlag(const bool in);

	void		SetRoofTextureParam(const float posX, const float posY, const float scaleX, const float scaleY, const float alpha, const bool bGeneric);

	void		ComputeTextureCoordinatesAerial(Point2D pt, Point2D ptOrigin, float* pTexCoordX, float* pTexCoordY);
	vector<Point2D> ComputeSlopeTextureCoordinatesGeneric(VectorOfRoofLines slope);

	// for output
	Skeleton			GetSkeleton(void) const;
	CVertexList			GetVertexList(void) const;
	MultisetOfLines		GetMultisetOfLines(void) const;
	MultisetOfLines		GetMultisetOfTruncatedLines(void) const;
	MultisetOfLines		GetMultisetOfTopLines(void) const;
	VectorOfLines		GetVectorOfLines(void) const;
	VectorOfPolygons	GetVectorOfPolygons(void) const;
	VectorOfPolygons	GetVectorOfSlopes(void) const;
	VectorOfPolygons	GetVectorOfTruncatedSlopes(void) const;
	vector<VectorOfPoint2D>	GetVectorOfTextureCoordinates(void) const;
	vector<VectorOfPoint2D>	GetVectorOfTruncatedTextureCoordinates(void) const;
	vector<Point3D>		GetVectorOfNormalCoordinates(void) const;
	vector<Point3D>		GetVectorOfTruncatedNormalCoordinates(void) const;
	VectorOfPolygon2D	GetNewFootPrint(void) const;
	int					GetNbOfSlopes(void) const;
	int					GetNbOfVertices(void) const;

	// geometrie
	FT			&NormalizeAngle (FT &angle);
	FT			NormalizedAngle (FT angle);
	int			Colinear(const CRay &a, const CRay &b);
	int			PointOnRay(const CPoint2D &p, const CRay &r);
	CPoint2D	Intersection2Ray(const CRay& a, const CRay &b);
	FT			DistPointToPoint(const CPoint2D &p, const CPoint2D &q);
	FT			DistPointToRay(const CPoint2D &p, const CRay &l);
	FT			DistRayToPoint(const CRay &l, const CPoint2D &p);
//	FT			DistPointToSegment(const CPoint2D &p, const Segment &l);
	CRay		AngleAxis (const CPoint2D &b, const CPoint2D &a, const CPoint2D &c); // vrati osu uhlu abc prochazejici bodem b
	int			FacingTowards(const CRay &a, const CRay &b);
	CPoint2D	IntersectionAnywhere (const CRay& a, const CRay &b);
	CPoint2D	CoordinatesOfAnyIntersectionOfTypeB (const CVertex &v, const CVertex &left, const CVertex &right);
	FT			NearestIntersection (const CVertex &v, CVertex **left, CVertex **right, CPoint2D &p); // vrati nejblizsi z pruseciku typu B
	CPoint2D	IntersectionOfTypeB (const CVertex &v, const CVertex &left, const CVertex &right);
	bool		InvalidIntersection (const CVertex &v, const CIntersection &is);
	int			ApplyConvexIntersection(const CIntersection &i);
	//int			ApplyTruncatedIntersection(const CIntersection &ifirst, const CIntersection &isecond);
	int			ApplyNonconvexIntersection (CIntersection &i);  // byl-li nejnizsi prusecik ve fronte nekonvexniho typu ...
	int			ApplyLast3(const CIntersection &i);

	// skeleton
	int			MakeSkeleton(BuildingContour &contours/*, CVertexList &vlist*/);
	int			MakeSkeleton(Polygon2D &poly);
	void		UpdateSkeleton(typeRoof type, FT eave);
	void		MakeGableSkeleton(void);
	void		MakeEaveSkeleton(void);

	// for computing roof slopes
	void		BuildPolylines();
	void		BuildSlopes(void);
	void		BuildConvexSlopes();
	void		BuildNewFootPrint();

	VectorOfPolygons		BuildPolygons(MultisetOfLines inMLines);
	VectorOfPolygons		BuildTruncatedPolygons(VectorOfPolygons inVPoly);
	void					BuildTextureCoordinates();
	void					BuildTextureCoordinatesAerial();
	void					BuildTextureCoordinatesGeneric();
	void					BuildNormalCoordinates();
	VectorOfRoofLines		TruncatePolygon(VectorOfRoofLines VLines);
	Polyline2D				VectorOfRoofLinesToPolyline2D(VectorOfRoofLines VLines);
	bool					FindNextLine(CRoofLine line, MultisetOfLines Mlines, MultisetOfLines::iterator* iNext, CRoofLine* pNextLine);
	CVertex					GetVertexFromID(int nID);

	// for output in correct format
	// format = GL_V3F, GL_T2F_V3F, GL_N3F_V3F or GL_T2F_N3F_V3F
	// return the number of verices
	int			FillVertexArray(GLformat format, float** vertices, bool bInvTex=false);
	void		FillIndexArrayForGLPolygon(int& nbslopes, int** nbVerticesBySlope, unsigned int*** SlopeIndices);	
	void		FillIndexArrayForGLLines(int& nblines, unsigned int** LineIndices);



	BuildingContour		Polygon2DToContour(const Polygon2D &in);
	FT			ComputeAngle(CRoofLine l1, CRoofLine l2);
	vector<Point2D> FillFootPrintPoints();
	void		FillInitialLines(Polygon2D poly);
	void		FillInitialLines(BuildingContour bcontour);
	void		FillAllSkeletonLines();


	int			AddNewVertexInList(Point3D point);
	int			BuildRoof3D(Polygon2D footprint, FT bHeight, FT angle, FT maxheight, typeRoof type, FT eave);
	int			BuildFlatRoof(Polygon2D footprint, FT bHeight);

protected:

	IntersectionQueue	m_iq;          // priority queue of intersection points
	CVertexList			m_vl;          // SLAV, jednotlive cykly (LAV) jsou udrzovany pomoci ukazatelu nextVertex, prevVertex
	Skeleton			m_skeleton;    // straigth skeleton

	// added for 3d roof
	Polygon2D			m_footprint;		// footprint
	FT					m_roofangle;		// input angle of roof, to conpute 3d points	
	FT					m_buildingheight;	// height of input building			
	FT					m_eave;				// roof eave in X (debord de toit)
	FT					m_maxheight;		// roof max height
	typeRoof			m_type;				// type of roof

//	MultiLine			m_alllines;			// contains all available lines for polygon construction
	bool				m_truncated;		// true if the skeleton goes above max roof height
	MultisetOfLines		m_lines;
	MultisetOfLines		m_truncatedlines;
	MultisetOfLines		m_toplines;
	MultisetOfLines		m_displaylines;
	VectorOfLines		m_polylines;
	VectorOfPolygons	m_slopes;
	VectorOfPolygons	m_truncatedslopes;
	vector<VectorOfPoint2D>	m_texturecoordinates;
	vector<VectorOfPoint2D>	m_truncatedtexturecoordinates;
	vector<Point3D>		m_normalcoordinates;
	vector<Point3D>		m_truncatednormalcoordinates;
	VectorOfPolyline2D	m_holepolygon;
	VectorOfPolyline2D	m_outerpolygon;
	VectorOfPolygon2D	m_polygons;
	VectorOfPolygon2D	m_triangles;
	ListVertices		m_listvertices;

	// added for roof texture positionning
	float				m_TexPosX, m_TexPosY;
	float				m_TexScaleX, m_TexScaleY;
	float				m_TexRotation;
	bool				m_bGeneric;
};


#endif // !defined(AFX_ROOFCONSTRUCTION_H__29C528BB_56E6_4D44_8051_3853423F151C__INCLUDED_)
