/*----------------------------------------------------------------
Projet : Visualisation Villes / Librairie gomtrique de base.
Fichier: Triangulation.h
Date 	 :
A.C. 01/2002

Commentaires:
	Triangulation de Delaunay Contrainte.
	La plupart des fonctions lancent une exception IllegalState.
-----------------------------------------------------------------*/


#ifndef __TRIANGULATION_H__
#define __TRIANGULATION_H__

#include "GeomDefs.h"
//#include "Segment2D.h"
//#include "Point2D.h"
#include "Polygon2D.h"
#include "GeomSTLDefs.h"


class Triangulation
{
public:
	//! Constructeur par dfaut
	Triangulation();

	//! Destructeur par dfaut
	virtual ~Triangulation();

	//! Constructeur par copie
	/*!
	 *	\param Triangulation const ref.
	*/
	Triangulation(const Triangulation& );

	//! Oprateur d'assignation
	/*!
	 *	\param Triangulation const ref.
	*/
	Triangulation& operator =(const Triangulation& );
	//! Initialise la triangulation. Permet de commencer l'entre de paramtres.
	/*!
	*/
	virtual void Begin();
	//! On ne peut plus inclure des lts ds la triangulation aprs. Lance le calcul.
	/*!
	*/
	virtual void End();
	//! Ajoute un segment.
	/*!
	 * \param Segment2D  inclure.
	*/
	virtual void Push(Segment2D s);
	/*!
	 * \param Point2D  inclure.
	 * \param isHole ? est-ce un trou (faux par dfaut)?
	*/
	virtual void Push(Point2D p, bool isHole=false);
	
	//! permet de specifie l'orientation des triangles issue de la triangulation
	virtual void SetOrientation(ORIENTATION o);

	//! Retourne les triangle 
	virtual vector<Polygon2D> Triangles();




protected:
	//! Implmentation.
	Triangulation * m_impl;
};
#endif //  __TRIANGULATION_H__

// #eof "$Id: Triangulation.h,v 1.1 2006/10/20 14:14:56 granger Exp $"
