// Vertex.h: interface for the CVertex class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_VERTEX_H__5D2338C7_AEB2_49C8_A42F_ECE43FC34D05__INCLUDED_)
#define AFX_VERTEX_H__5D2338C7_AEB2_49C8_A42F_ECE43FC34D05__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <assert.h>


#include "point.h"
#include "ray.h"
//#include "skeletonline.h"

class CSkeletonLine;

class CVertex  
{
public:
//	CVertex();
	virtual ~CVertex();

	CVertex (void) : m_ID (-1) { };       // Bezparametricky konstruktor kvuli STL
    CVertex (const CPoint2D &p, const CPoint2D &prev = CPoint2D (), const CPoint2D &next = CPoint2D ());
    CVertex (const CPoint2D &p, CVertex &left, CVertex &right);

    CVertex *highest (void) { return m_higher ? m_higher -> highest () : this; }
    bool atContour (void) const { return m_leftVertex == this && m_rightVertex == this; }

    operator == (const CVertex &v) const { return m_point == v.GetPoint(); }
    operator != (const CVertex &v) const { return m_point != v.GetPoint(); }
    operator < (const CVertex &) const { assert (false); return false; } // kvuli STL, jinak se nepouziva

	virtual CPoint2D GetPoint(void) const;
	virtual CRay GetAxis(void) const;
	virtual CRay GetLeftLine(void) const;
	virtual CRay GetRightLine(void) const;
	virtual CVertex* GetLeftVertex(void) const;
	virtual CVertex* GetRightVertex(void) const;
	virtual CVertex* GetPrevVertex(void) const;
	virtual CVertex* GetNextVertex(void) const;
//	virtual CVertex* GetHigher(void) const;
	virtual bool GetDoneFlag(void) const;
	virtual int GetID(void) const;
	virtual CSkeletonLine* GetLeftSkeletonLine(void) const;
	virtual CSkeletonLine* GetRightSkeletonLine(void) const;
	virtual FT GetAltitudeOffset(void) const;
	virtual bool GetConvexFlag(void) const;

	virtual void SetPoint(const CPoint2D in);
	virtual void SetLeftLine(const CRay in);
	virtual void SetRightLine(const CRay in);
	virtual void SetLeftVertex(/*const*/ CVertex* in);
	virtual void SetRightVertex(/*const*/ CVertex* in);
	virtual void SetPrevVertex(CVertex* in);
	virtual void SetNextVertex(CVertex* in);
	virtual void SetHigher(CVertex* in);
	virtual void SetDoneFlag(const bool in);
	virtual void SetID(const int in);
	virtual void SetLeftSkeletonLine(CSkeletonLine* in);
	virtual void SetRightSkeletonLine(CSkeletonLine* in);
	virtual void SetAltitudeOffset(const FT in);
	virtual void SetConvexFlag(const bool in);


protected:
	// data
	CPoint2D		m_point;			// coordonnes point
	CRay			m_axis;             // bissectrice
	CRay			m_leftLine, m_rightLine;           // leva a prava hranicni usecka, axis je jejich osou
	CVertex			*m_leftVertex, *m_rightVertex;  // 2 vrcholy, jejich zaniknutim vzikl tento
	CVertex			*m_nextVertex, *m_prevVertex;   // vrchol sousedici v LAV
	CVertex			*m_higher;                    // vrchol vznikly pri zaniknuti tohoto
	bool			m_done;                         // priznak aktivity
	int				m_ID;                            // cislo automaticky pridelovane pri vkladani do LAV
	CSkeletonLine	*m_leftSkeletonLine, *m_rightSkeletonLine;
	//, *advancingSkeletonLine; // Pouzivano pri konstrukci kostry z okridlenych hran
	FT				m_altitudeoffset;

	bool			m_ConvexFlag;			// flag added to gabble roof computation
};

#endif // !defined(AFX_VERTEX_H__5D2338C7_AEB2_49C8_A42F_ECE43FC34D05__INCLUDED_)
