// Segment3D.h: interface for the Segment3D class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_SEGMENT3D_H__781CD95D_EFB9_4C47_8F0B_FC7C355CBE1B__INCLUDED_)
#define AFX_SEGMENT3D_H__781CD95D_EFB9_4C47_8F0B_FC7C355CBE1B__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "GeomDefs.h"
#include "Point3D.h"
#include "VectorGeom3D.h"
#include "Plane.h"

typedef enum {INTER_NULL, INTER_SEGMENT, INTER_POINT} INTER_TYPE;

class Segment3D  
{
public:
	//! Constructeur par dfaut
	Segment3D();
	//! Constructeur
	/*!
	 *  @param s : le sommet d'origine du segment
	 *  @param t : le sommet cible du segment
	!*/
	Segment3D(Point3D s, Point3D t);

	//! Constructeur par recopie
	Segment3D(const Segment3D &);

	//! Destructeur par defaut
	virtual ~Segment3D();
	
	//! Operateur de recopie
	Segment3D & operator =(const Segment3D &s);

	//! Test d'galit
	virtual bool equal(const Segment3D& in, FT precision);

	//! Teste si un sommet appartient au segment
	/*!
	 *	@param p : le sommet test
	 *  @param dist : la prcision du calcul (par defaut 0.00001)
	!*/
	virtual bool has_on(const Point3D &p, FT dist=0.00001) const;
	
	
	bool is_degenerate() const;

	//! Retourne la longuer au carr du segment
	virtual FT squared_length() const;

	//! Retourne un sommet du segment : 0 pour le sommet d'origine, 1 pour le sommet cible
	Point3D vertex(int i) const;

	//! Retourne le sommet cible du segment
	virtual Point3D target() const;

	//! Retourne le sommet d'origine du segment
	virtual Point3D source() const;

	//! intersection avec un plan
	virtual INTER_TYPE plane_intersect(Plane p, Point3D & intersect); 

//	virtual bool appartenance(const Point3D &p, FT &t) const;
//	virtual bool inclus(Droite3D &) const;
	
	//! Retourne le sommet d'origine pour 0, et le sommet cible pour 1
	Point3D operator [](int);

	//! Retourne la distance d'un point au segment
//	virtual FT DistanceFromPoint(const Point2D &p) const;

	//! Calcul l'intersection de deux segments
//	virtual int intersect(Segment2D &s, Point2D & intersect) const;


protected :
	Segment3D* m_impl;
};


#endif // !defined(AFX_SEGMENT3D_H__781CD95D_EFB9_4C47_8F0B_FC7C355CBE1B__INCLUDED_)
