#include "bitstream.h"
#include <malloc.h>
#include <stdio.h>


//------------------------------------ Implmentation obitstream ------------------------------------

obitstream::obitstream() : ofstream() {
	m_newbyte = 0;
	m_bits = (unsigned char)128;
}

void obitstream::writebit(int i) {
	if (m_bits==0) {
		ofstream::write(&m_newbyte,1);
		m_newbyte = 0;
		m_bits = (unsigned char)128;
	}

	if (i) {	
		m_newbyte = m_newbyte|m_bits;
	}
	m_bits = m_bits>>1;
	//cout<<i<<"-";
	
	return;
}



void obitstream::writenbit(unsigned char bit,short int n) {
	unsigned char mask = 128>>(8-n);
	if (n>8) {
		return;
	}

	for (int i=0; i<n; i++) {
		if (m_bits==0) {
			ofstream::write(&m_newbyte,1);
			m_newbyte = 0;
			m_bits = (unsigned char)128;
		}

		if ((bit&mask)>0) {
			m_newbyte = m_newbyte|m_bits;
		}
		
		mask = mask>>1;
		m_bits = m_bits>>1;
	}
	return;
}

void obitstream::writenbit(unsigned short int bit,short int n) {
	unsigned short int mask = 32768>>(16-n);
	if (n>16) {
		return;
	}

	for (int i=0; i<n; i++) {
		if (m_bits==0) {
			ofstream::write(&m_newbyte,1);
			m_newbyte = 0;
			m_bits = (unsigned char)128;
		}

		if ((bit&mask)>0) {
			m_newbyte = m_newbyte|m_bits;
		}
		
		mask = mask>>1;
		m_bits = m_bits>>1;
	}
	return;
}

void obitstream::writenbit(unsigned int bit,short int n) {
	unsigned int mask = 2147483648>>(32-n);
	if (n>32) {
		return;
	}

	for (int i=0; i<n; i++) {
		if (m_bits==0) {
			ofstream::write(&m_newbyte,1);
			m_newbyte = 0;
			m_bits = (unsigned char)128;
		}

		if ((bit&mask)>0) {
			m_newbyte = m_newbyte|m_bits;
		}
		
		mask = mask>>1;
		m_bits = m_bits>>1;
	}
	return;
}

void obitstream::write(char i) {
	ofstream::write((char*)(&i),sizeof(i));
	return;
}

void obitstream::write(float f) {
	ofstream::write((char*)(&f),sizeof(f));
	return;
}


void obitstream::close() {
	ofstream::write(&m_newbyte,1);
	m_newbyte = 0;
	m_bits = (unsigned char)128;
	ofstream::close();
	return;
}

void obitstream::reset() {
	m_newbyte = 0;
	m_bits = (unsigned char)128;
	close();
}

void obitstream::append(obitstream) {
	
}




//------------------------------------ Implmentation ibitstream ------------------------------------

ibitstream::ibitstream() : ifstream() {
	m_newbyte = 0;
	m_bits = (unsigned char)0;
}

void ibitstream::read(char &i) {
	ifstream::read((char*)(&i),sizeof(char));
	return;
}

void ibitstream::read(float &f) {
	ifstream::read((char*)(&f),sizeof(float));
	return;
}

void ibitstream::close() {
	m_newbyte = 0;
	m_bits = (unsigned char)0;
	ifstream::close();
	return;
}

void ibitstream::reset() {
	m_newbyte = 0;
	m_bits = (unsigned char)0;
}


void ibitstream::readbit(int &bit){
	if (m_bits==0) {
		ifstream::read(&m_newbyte,1);
		m_bits = (unsigned char)128;
	}

	bit = ((m_newbyte&m_bits)>0);
	m_bits = m_bits>>1;
	//cout<<bit<<"-";
	
	return;
}

void ibitstream::readnbit(unsigned char& bit, short n) {
	unsigned char mask = 128>>(8-n);;
	bit = 0;
	if (n>8) {
		return;
	}

	for (int i=0; i<n; i++) {
		if (m_bits==0) {
			ifstream::read(&m_newbyte,1);
			m_bits = (unsigned char)128;
		}
		if ((m_newbyte&m_bits)>0) {
			bit = bit|mask;
		}

		mask = mask>>1;
		m_bits = m_bits>>1;
	}

	return;
}

void ibitstream::readnbit(unsigned short int& bit, short n) {
	unsigned short int mask = 32768>>(16-n);
	bit = 0;
	if (n>16) {
		return;
	}

	for (int i=0; i<n; i++) {
		if (m_bits==0) {
			ifstream::read(&m_newbyte,1);
			m_bits = (unsigned char)128;
		}
		if ((m_newbyte&m_bits)>0) {
			bit = bit|mask;
		}

		mask = mask>>1;
		m_bits = m_bits>>1;
	}

	return;
}

void ibitstream::readnbit(unsigned int& bit, short n) {
	unsigned int mask = 2147483648>>(32-n);
	bit = 0;
	if (n>32) {
		return;
	}

	for (int i=0; i<n; i++) {
		if (m_bits==0) {
			ifstream::read(&m_newbyte,1);
			m_bits = (unsigned char)128;
		}
		if ((m_newbyte&m_bits)>0) {
			bit = bit|mask;
		}

		mask = mask>>1;
		m_bits = m_bits>>1;
	}
	return;
}

//-----------------------------------------------------------
//mbitstream implementation

// WARNING ! Least significant bits first

/*
public:

	unsigned short *octets;
	int top;
	int offset;
	unsigned short tmp;
	
	mbitstream();
	void writebit(bool);
	void dump(obitstream);
	inline int size() {return top*8+offset;};

};
*/
mbitstream::mbitstream() {
	top = offset = 0;	
	octets=NULL;
	tmp=0;
}

mbitstream::mbitstream(unsigned char *buffer) {
	top = offset = 0;	
	octets=buffer;
	tmp=0;

}

mbitstream::mbitstream(char *filename) {
	
	unsigned char tmp;
	FILE *f=fopen(filename, "r");
	fseek(f,0,SEEK_END);
	long filelength = ftell(f);
	fseek(f,0,SEEK_SET);
	top = offset = 0;	
	octets=(unsigned char*)malloc(filelength*sizeof(unsigned char));
	tmp=0;

	for (int i=0; i<filelength; i++) {
		fscanf(f, "%c", &tmp);
		writenbit(tmp, 8);
	}

	fclose(f);

}

void mbitstream::writebit(bool bit) {
	
	unsigned char t;


	//Vidage
	if (offset == 8) {
		offset = 0;
		top ++;
		octets[top-1]=tmp;
		tmp=0;
	}

	//Remplissage
	tmp=tmp+(bit<<(7-offset));
	offset++;


}

void mbitstream::writenbit(unsigned char bits, int nbbits) {

	for (int i=0; i<nbbits; i++)
		writebit(bits&(1<<(nbbits-i-1)));


}

void mbitstream::writenbit(unsigned short bits, int nbbits) {


	for (int i=0; i<nbbits; i++)
		writebit(bits&(1<<(nbbits-i-1)));

	
}

void mbitstream::writenbit(unsigned int bits, int nbbits) {

	for (int i=0; i<nbbits; i++)
		writebit(bits&(1<<(nbbits-i-1)));


}

bool mbitstream::readbit() {
	
	bool tmp;

	//Passage au suivant
	if (offset == 8) {
		offset = 0;
		top ++;
	}

	//Lecture
	tmp=(((1<<(7-offset))&octets[top])!=0);
	offset++;

	return tmp;


}

void mbitstream::readbit(int &tmp) {

	//Passage au suivant
	if (offset == 8) {
		offset = 0;
		top ++;
	}

	//Lecture
	tmp=(((1<<(7-offset))&octets[top])!=0);
	offset++;

}

void mbitstream::readnbit(unsigned short &tmp, int length) {
	
	tmp=0;
	bool bit;

	int i;

	for (i=0; i<length; i++) {
		bit = readbit();
		tmp+=bit<<(length-i-1);
	}

}

void mbitstream::readnbit(unsigned int &tmp, int length) {
	
	tmp=0;
	bool bit;

	int i;

	for (i=0; i<length; i++) {
		bit = readbit();
		tmp+=bit<<(length-i-1);
	}

}

void mbitstream::dump(obitstream &obs) {

	int i;

	//Partie rgulire
	for (i=0; i<top; i++) {
		obs.writenbit(octets[i], 8);
	}

	//reste
	if (offset!=0){
		tmp=tmp<<(8-offset);
		for (i=0; i<offset; i++) {
			obs.writebit((tmp>=128));
			tmp=tmp<<1;
		}
	}
}

void mbitstream::dump(mbitstream &obs) {

	int i;

	//Partie rgulire
	for (i=0; i<top; i++) {
		obs.writenbit(octets[i], 8);
	}

	//reste
	if (offset!=0){
		//tmp=tmp<<(8-offset);
		for (i=0; i<offset; i++) {
			obs.writebit((tmp>=128));
			tmp=tmp<<1;
		}
	}
}

void mbitstream::dump(mbitstream &obs, int Offset, int length) {

	int i;
	bool bit;

	//Positionnement
	top=Offset/8;
	offset=Offset%8;

	//Lecture/Ecriture
	for(i=Offset; i<Offset+length; i++) {
		bit=readbit();
		obs.writebit(bit);
	}
}

void mbitstream::reset() {

	if (octets!=NULL)
		free(octets);
	octets=NULL;
	top=0;
	offset=0;
	tmp=0;
}

void mbitstream::close() {

	//Sert aussi d'alignement

	//Vidage

	offset = 0;
	top ++;
	octets[top-1]=tmp;
	tmp=0;

}

void mbitstream::writenbit(float float32,  int n) {

	unsigned int int32 = *(unsigned int*)&float32;
	writenbit(int32, n);

}

void mbitstream::readnbit(float& float32,  int n) {

	unsigned int int32;
	
	readnbit(int32, n);
	float32=*(float*)&int32;


}
