/***********************************************************************
This software module was originally developed by James D. K. Kim, 
Sang Oak Woo and Shin Jun Lee(Samsung Advance Institute of Technology)
in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)
standard. This software module is an implementation of a part of one or
more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4
Systems (ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 
Systems (ISO/IEC 14496-1) free license to this software module or 
modifications thereof for use in hardware or software products claiming
conformance to the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to 
use this software module in hardware or software products are advised that
its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Samsung Advance Institute of Technology
retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using
the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
This copyright notice must be included in all copies or derivative works.
Copyright  2001
************************************************************************

For more information or to receive an up to date version of this module,
you may contact James D. K. Kim at jacobkim@sait.samsung.co.kr
----------------------------------------------------------------------*/

// CCQ class
// Author: Shinjun Lee, Samsung AIT
// Date: 2001. 10. 17. Wed.
// 2nd revision: 2002. 1. 23. Wed.
// 4th revision: 2002-05-03

#ifndef _CCQ_H_
#define _CCQ_H_

#include <iostream.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>

#define DPCM_T	1
#define DPCM_S	2
#define DPCM_TS	3

class CCQ
	{
  public:
	// circular quantize
	int quantize(int nValue, int nMin, int nMax)
		{
		int nLength = nMax - nMin + 1;
		int nComplement = 0;
		int nReturn = nValue;

		if(nValue < 0)
			nComplement = nValue + nLength;
		else
			nComplement = nValue - nLength;

		if(abs(nValue) > abs(nComplement))
			nReturn = nComplement;
		else if(abs(nValue) == abs(nComplement) && nValue / nComplement == -1)
			nReturn = abs(nValue);

		return nReturn;
		}

	// inverse circular quantize
	int inverseQuantize(int nValue, int nMin, int nMax, int nPrevious = 0)
		{
		int nLength = nMax - nMin + 1;
		int nComplement = 0;
		int nReturn = nValue;

		if(nValue < 0)
			nComplement = nValue + nLength;
		else
			nComplement = nValue - nLength;

		if(nMin <= nPrevious + nValue && nPrevious + nValue <= nMax)
			nReturn = nPrevious + nValue;
		else if(nMin <= nPrevious + nComplement && nPrevious + nComplement <= nMax)
			nReturn = nPrevious + nComplement;

		return nReturn;
		}

	// circular quantize: first value
	int quantizeFirstValue(int nValue, int nMin, int nMax)
		{
		int nLow = nValue - nMin;
		int nHigh = nValue - nMax - 1;

		if(abs(nLow) <= abs(nHigh))
			nValue = nLow;
		else
			nValue = nHigh;

		return nValue;
		}

	// inverse circular quantize: first value
	int inverseQuantizeFirstValue(int nValue, int nMin, int nMax)
		{
		if(nValue < 0)
			nValue = nValue + nMax + 1;
		else
			nValue = nValue + nMin;

		return nValue;
		}
	};

#endif // _CCQ_H_