/***********************************************************************
This software module was originally developed by James D. K. Kim, 
Sang Oak Woo and Shin Jun Lee(Samsung Advance Institute of Technology)
in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)
standard. This software module is an implementation of a part of one or
more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4
Systems (ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 
Systems (ISO/IEC 14496-1) free license to this software module or 
modifications thereof for use in hardware or software products claiming
conformance to the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to 
use this software module in hardware or software products are advised that
its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Samsung Advance Institute of Technology
retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using
the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
This copyright notice must be included in all copies or derivative works.
Copyright  2001
************************************************************************

For more information or to receive an up to date version of this module,
you may contact James D. K. Kim at jacobkim@sait.samsung.co.kr
----------------------------------------------------------------------*/

// DPCM
// Author: Shinjun Lee, Samsung AIT
// Date: 2001-11-7-Wed
// 4th revision: 2002-05-03

#ifndef _CDPCM_H_
#define _CDPCM_H_

#include "CCQ.h"

#include <memory.h>
#include <math.h>

class CDPCM
	{
  private:
	int		m_nNumber;						// the number of data

	int*	m_a_nData;						// the array of input data
	int		m_nMin;							// the value of minimum
	int		m_nMax;							// the value of maximum

	int*	m_a_nData_dpcm;					// the data of dpcm
	int		m_nMin_dpcm;					// the value of dpcm minimum
	int		m_nMax_dpcm;					// the value of dpcm maximum
	
	int*	m_a_nDataXYZ;					// input data of X, Y, Z
	int		m_nComponent;					// the component among X, Y, Z

	int*	m_a_nBase;						// the base values of minimum
	int*	m_a_nBaseXYZ;					// the base values of minimum of X, Y, Z

  public:
	CDPCM(void)
		{
		m_nNumber = 0;
		m_a_nData = NULL;
		m_nMin = 0;
		m_nMax = 0;
		m_a_nData_dpcm = NULL;
		m_nMin_dpcm = 0;
		m_nMax_dpcm = 0;
		m_a_nDataXYZ = NULL;
		m_nComponent = 0;
		m_a_nBase = NULL;
		m_a_nBaseXYZ = NULL;
		}

	~CDPCM()
		{
		delete[] m_a_nData;
		delete[] m_a_nData_dpcm;
		delete[] m_a_nBase;
		}

	// clear: member variables
	void clear(void)
		{
		delete[] m_a_nData;
		delete[] m_a_nData_dpcm;
		delete[] m_a_nBase;

		m_nNumber = 0;
		m_a_nData = NULL;
		m_nMin = 0;
		m_nMax = 0;
		m_a_nData_dpcm = NULL;
		m_nMin_dpcm = 0;
		m_nMax_dpcm = 0;
		m_a_nDataXYZ = NULL;
		m_nComponent = 0;
		m_a_nBase = NULL;
		m_a_nBaseXYZ = NULL;
		}

	// set: the number of data
	void setNumber(int nNumber)
		{
		m_nNumber = nNumber;
		}

	// set: input data
	void setData(int* a_nData)
		{
		m_a_nDataXYZ = a_nData;

		delete[] m_a_nData;
		m_a_nData = new int[m_nNumber];
		memset((void*)m_a_nData, 0, sizeof(int)*m_nNumber);

		delete[] m_a_nData_dpcm;
		m_a_nData_dpcm = new int[m_nNumber];
		memset((void*)m_a_nData_dpcm, 0, sizeof(int)*m_nNumber);
		}

	// set: base data:
	void setBase(int* a_nBase)
		{
		m_a_nBaseXYZ = a_nBase;
		
		delete[] m_a_nBase;
		m_a_nBase = new int[m_nNumber];
		}

	// set: the component of X, Y, Z
	void setComponent(int nComponent)
		{
		int i = 0;

		m_nComponent = nComponent;
		for(i = 0; i < m_nNumber; i++)
			{
			m_a_nData[i] = m_a_nDataXYZ[i*3+nComponent];
			if(m_a_nBase != NULL)
				m_a_nBase[i] = m_a_nBaseXYZ[i*3+nComponent];
			if(i == 0)
				m_nMin = m_nMax = m_a_nData[i];
			else
				{
				if(m_a_nData[i] < m_nMin)
					m_nMin = m_a_nData[i];
				if(m_a_nData[i] > m_nMax)
					m_nMax = m_a_nData[i];
				}
			}
		}

	// DPCM: dpcm with CQ
	void dpcm(void)
		{
		int* a_nData = NULL;
		int i = 0;

		// CQ: circular quantization
		CCQ oCQ;

		// allocation: a_nData
		a_nData = new int[m_nNumber];

		// copy: a_nData <-- m_a_nData
		memcpy((void*)a_nData, m_a_nData, sizeof(int)*m_nNumber);

		// dpcm: 1st order dpcm
		i = 0;
		if(m_a_nBase != NULL)
			{
			if(m_nMin < m_a_nBase[i])
				{
				m_a_nData_dpcm[i] = a_nData[i] - m_a_nBase[i];
				m_a_nData_dpcm[i] = oCQ.quantize(m_a_nData_dpcm[i], m_a_nBase[i], m_nMax);
				}
			else
				{
				m_a_nData_dpcm[i] = a_nData[i] - m_nMin;
				m_a_nData_dpcm[i] = oCQ.quantize(m_a_nData_dpcm[i], m_nMin, m_nMax);
				}
			}
		else
			{
			m_a_nData_dpcm[i] = a_nData[i];											// dpcm
			m_a_nData_dpcm[i] = oCQ.quantize(m_a_nData_dpcm[i], m_nMin, m_nMax);	// cq
			}

		m_nMin_dpcm = m_nMax_dpcm = m_a_nData_dpcm[i];
		for(i = 1; i < m_nNumber; i++)
			{
			if(m_a_nBase != NULL && a_nData[i-1] < m_a_nBase[i])	// if the previous value is less than the base value
				{
				m_a_nData_dpcm[i] = a_nData[i] - m_a_nBase[i];		// dpcm is performed between the current value and the base value
				m_a_nData_dpcm[i] = oCQ.quantize(m_a_nData_dpcm[i], m_a_nBase[i], m_nMax);
				}
			else
				{
				m_a_nData_dpcm[i] = a_nData[i] - a_nData[i-1];		// dpcm is performed between the current value and the previous value
				m_a_nData_dpcm[i] = oCQ.quantize(m_a_nData_dpcm[i], m_nMin, m_nMax);
				}

			if(m_a_nData_dpcm[i] < m_nMin_dpcm)
				m_nMin_dpcm = m_a_nData_dpcm[i];
			if(m_a_nData_dpcm[i] > m_nMax_dpcm)
				m_nMax_dpcm = m_a_nData_dpcm[i];
			}

		delete[] a_nData;
		}

	// Inverse DPCM: inverse DPCM with CQ
	void inverseDPCM(bool* a_bQFlag_kv, int* a_nQStep_kv, int* a_nBase_kv, int c)
		{
		int* a_nData_dpcm = NULL;
		int i = 0;

		// CQ: circular quantization
		CCQ oCQ;

		// allocation: a_nData_dpcm
		a_nData_dpcm = new int[m_nNumber];

		// copy: a_nData_dpcm <-- m_a_nData_dpcm
		memcpy((void*)a_nData_dpcm, m_a_nData_dpcm, sizeof(int)*m_nNumber);

		// inverse dpcm: 1st order inverse dpcm
		i = 0;
		if(a_bQFlag_kv[i*3+c] == false && m_a_nBase != NULL)
			{
			int nQData = getData_base(i);
			m_a_nData[i] = nQData;
			}
		else if(a_bQFlag_kv[i*3+c] != false && a_nQStep_kv[i*3+c] == 0)
			{
			int nIndex_base = a_nBase_kv[i*3+c];
			int nQData = getData(nIndex_base);
			m_a_nData[i] = nQData;
			}
		else
			{
			if(m_a_nBase != NULL)
				{
				if(m_nMin < m_a_nBase[i])
					{
					m_a_nData[i] = a_nData_dpcm[i] + m_a_nBase[i];
					m_a_nData[i] = oCQ.inverseQuantize(m_a_nData[i], m_a_nBase[i], m_nMax, 0);
					}
				else
					{
					m_a_nData[i] = a_nData_dpcm[i] + m_nMin;
					m_a_nData[i] = oCQ.inverseQuantize(m_a_nData[i], m_nMin, m_nMax, 0);
					}
				}
			else
				{
				m_a_nData[i] = a_nData_dpcm[i];
				m_a_nData[i] = oCQ.inverseQuantize(m_a_nData[i], m_nMin, m_nMax, 0);
				}
			}

		m_nMin_dpcm = m_nMax_dpcm = m_a_nData_dpcm[i];
		for(i = 1; i < m_nNumber; i++)
			{
			if(a_bQFlag_kv[i*3+c] == false && m_a_nBase != NULL)
				{
				int nQData = getData_base(i);
				m_a_nData[i] = nQData;
				}
			else if(a_bQFlag_kv[i*3+c] != false && a_nQStep_kv[i*3+c] == 0)
				{
				int nIndex_base = a_nBase_kv[i*3+c];
				int nQData = getData(nIndex_base);
				m_a_nData[i] = nQData;
				}
			else
				{
				if(m_a_nBase != NULL && m_a_nData[i-1] < m_a_nBase[i])
					{
					m_a_nData[i] = a_nData_dpcm[i] + m_a_nBase[i];
					m_a_nData[i] = oCQ.inverseQuantize(m_a_nData[i], m_a_nBase[i], m_nMax, m_a_nData[i-1]);
					}
				else
					{
					m_a_nData[i] = a_nData_dpcm[i] + m_a_nData[i-1];		// dpcm
					m_a_nData[i] = oCQ.inverseQuantize(m_a_nData[i], m_nMin, m_nMax, m_a_nData[i-1]);
					}
				}
			
			if(m_a_nData_dpcm[i] < m_nMin_dpcm)
				m_nMin_dpcm = m_a_nData_dpcm[i];
			if(m_a_nData_dpcm[i] > m_nMax_dpcm)
				m_nMax_dpcm = m_a_nData_dpcm[i];
			}

		delete[] a_nData_dpcm;
		}

	// Inverse DPCM: inverse DPCM with CQ
	void inverseDPCM(bool* a_bQFlag_kv, int* a_nQStep_kv, int* a_nBase_kv, int nCoord, int nComponent)
		{
		int i = 0;

		// CQ: circular quantization
		CCQ oCQ;

		// inverse dpcm: 1st order inverse dpcm
		i = nCoord;
		if(nCoord == 0)
			{
			if(a_bQFlag_kv[i*3+nComponent] == false && m_a_nBase != NULL)
				{
				int nQData = getData_base(i);
				m_a_nData[i] = nQData;
				}
			else if(a_bQFlag_kv[i*3+nComponent] != false && a_nQStep_kv[i*3+nComponent] == 0)
				{
				int nIndex_base = a_nBase_kv[i*3+nComponent];
				int nQData = getData(nIndex_base);
				m_a_nData[i] = nQData;
				}
			else
				{
				if(m_a_nBase != NULL)
					{
					if(m_nMin < m_a_nBase[i])
						{
						m_a_nData[i] = m_a_nData_dpcm[i];														// i-dpcm
						m_a_nData[i] = oCQ.inverseQuantize(m_a_nData[i], m_a_nBase[i], m_nMax, m_a_nBase[i]);	// i-cq
						}
					else
						{
						m_a_nData[i] = m_a_nData_dpcm[i];														// i-dpcm
						m_a_nData[i] = oCQ.inverseQuantize(m_a_nData[i], m_nMin, m_nMax, m_nMin);				// i-cq
						}
					}
				else
					{
					m_a_nData[i] = m_a_nData_dpcm[i];															// i-dpcm
					m_a_nData[i] = oCQ.inverseQuantize(m_a_nData[i], m_nMin, m_nMax);							// i-cq
					}
				}
			}
		else
			{
			m_nMin_dpcm = m_nMax_dpcm = m_a_nData_dpcm[i];
			if(a_bQFlag_kv[i*3+nComponent] == false && m_a_nBase != NULL)
				{
				int nQData = getData_base(i);
				m_a_nData[i] = nQData;
				}
			else if(a_bQFlag_kv[i*3+nComponent] != false && a_nQStep_kv[i*3+nComponent] == 0)
				{
				int nIndex_base = a_nBase_kv[i*3+nComponent];
				int nQData = getData(nIndex_base);
				m_a_nData[i] = nQData;
				}
			else
				{
				if(m_a_nBase != NULL && m_a_nData[i-1] < m_a_nBase[i])
					{
					m_a_nData[i] = m_a_nData_dpcm[i];															// i-dpcm
					m_a_nData[i] = oCQ.inverseQuantize(m_a_nData[i], m_a_nBase[i], m_nMax, m_a_nBase[i]);		// i-cq
					}
				else
					{
					m_a_nData[i] = m_a_nData_dpcm[i];															// i-dpcm
					m_a_nData[i] = oCQ.inverseQuantize(m_a_nData[i], m_nMin, m_nMax, m_a_nData[i-1]);			// i-cq
					}
				}

			if(m_a_nData_dpcm[i] < m_nMin_dpcm)
				m_nMin_dpcm = m_a_nData_dpcm[i];
			if(m_a_nData_dpcm[i] > m_nMax_dpcm)
				m_nMax_dpcm = m_a_nData_dpcm[i];
			}
		}

	// get: Quantization Data
	int getData(int nIndex)
		{
		return m_a_nData[nIndex];
		}

	// set: Quantization Data
	void setData(int nIndex, int nValue)
		{
		m_a_nData[nIndex] = nValue;
		}

	// get: Data_dpcm
	int getData_dpcm(int nIndex)
		{
		return m_a_nData_dpcm[nIndex];
		}

	// set: Data_dpcm
	void setData_dpcm(int nIndex, int nValue)
		{
		m_a_nData_dpcm[nIndex] = nValue;
		}

	// get: Data_base
	int getData_base(int nIndex)
		{
		return m_a_nBase[nIndex];
		}

	// set: Data_base
	void setData_base(int nIndex, int nValue)
		{
		m_a_nBase[nIndex] = nValue;
		}

	// get: Min
	int getMin(void)
		{
		return m_nMin;
		}

	// set: Min
	void setMin(int nMin)
		{
		m_nMin = nMin;
		}

	// get: Max
	int getMax(void)
		{
		return m_nMax;
		}

	// set: Max
	void setMax(int nMax)
		{
		m_nMax = nMax;
		}

	// get: MinDPCM
	int getMinDPCM(void)
		{
		return m_nMin_dpcm;
		}

	// get: MaxDPCM
	int getMaxDPCM(void)
		{
		return m_nMax_dpcm;
		}

	// get: NBits
	int getNBits(int n)
		{
		return (int)(log10(n)/log10(2))+1;
		}

	// get: QStep
	int getQStep(void)
		{
		if(abs(m_nMin_dpcm) > abs(m_nMax_dpcm))
			return getNBits(abs(m_nMin_dpcm));
		return getNBits(abs(m_nMax_dpcm));
		}
	};

#endif // _CDPCM_H_