/***********************************************************************
This software module was originally developed by James D. K. Kim, 
Sang Oak Woo and Shin Jun Lee(Samsung Advance Institute of Technology)
in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)
standard. This software module is an implementation of a part of one or
more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4
Systems (ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 
Systems (ISO/IEC 14496-1) free license to this software module or 
modifications thereof for use in hardware or software products claiming
conformance to the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to 
use this software module in hardware or software products are advised that
its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Samsung Advance Institute of Technology
retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using
the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
This copyright notice must be included in all copies or derivative works.
Copyright  2001
************************************************************************

For more information or to receive an up to date version of this module,
you may contact James D. K. Kim at jacobkim@sait.samsung.co.kr
----------------------------------------------------------------------*/

// Key decoder class
// Date: 2001. 5. 29. Tue. - 6. 26. Tue.
// 2nd revision: 2001. 9. 4. Tue.
// 4th revision: 2002-05-03
// Author: Shinjun Lee

#include <iostream.h>
#include <fstream.h>
#include <math.h>

#define sign(n) ((n >= 0) ? 1 : -1))		// sign function

const double dKEY_MAX = 1.0;				// Key Max
const double dKEY_MIN = 0.0;				// Key Min

const int nDPCM_MAX = 3;					// DPCM Max
const int nDND_MAX = 7;						// DND MAX

// Key Decoder
class CKeyDecoder
	{
private:
	int*		m_a_nQKey;					// the quantized key data
	double*		m_a_dIQKey;					// the inversed quantized key data

	int			m_nNumberOfKey;				// the number of key

	int			m_nOrder_dpcm;				// DPCM order
	int			m_nOrder_dnd;				// DND order

	int			m_nKey_shift;				// Shift value
	int			m_nKey_shift_down;			// Shift down value

	int			m_nDigit_key;				// Key digit

	int			m_a_nKey_dnd_max[nDND_MAX];	// Max values in DND

	double		m_dKey_min;					// Key Min
	double		m_dKey_max;					// Key Max

public:
	CKeyDecoder(void)
		{
		m_a_nQKey = NULL;
		m_a_dIQKey = NULL;

		m_nNumberOfKey = 0;

		m_nOrder_dpcm = 0;
		m_nOrder_dnd = 0;

		m_nKey_shift = 0;
		m_nKey_shift_down = 0;

		m_nDigit_key = 0;

		m_dKey_min = dKEY_MIN;
		m_dKey_max = dKEY_MAX;
		}

	~CKeyDecoder()
		{
		if(m_a_nQKey != NULL)
			delete[] m_a_nQKey;
		if(m_a_dIQKey != NULL)
			delete[] m_a_dIQKey;
		}

	// Setting: key digit
	void setKeyDigit(int nDigit_key)
		{
		m_nDigit_key = nDigit_key;
		}

	// Setting: min, max
	void setMinMax(double dKey_min, double dKey_max)
		{
		m_dKey_min = dKey_min;
		m_dKey_max = dKey_max;
		}

	// decode
	void decode(int* a_nQKey, int nNumberOfKey, int nQStep_key)
		{
		int* a_nKey_dpcm = NULL;
		int* a_nKey_fold = NULL;
		int* a_nKey_dnd = NULL;
		int i;

		// Moving: the number of key, key digit
		m_nNumberOfKey = nNumberOfKey;
		// 2002-04-18-02, Shinjun
		if(m_nNumberOfKey == 0)
			return;

		// Allocation: the memory of key data
		m_a_nQKey = new int[m_nNumberOfKey];
		m_a_dIQKey = new double[m_nNumberOfKey];

		a_nKey_dpcm = new int[m_nNumberOfKey];
		a_nKey_fold = new int[m_nNumberOfKey];
		a_nKey_dnd = new int[m_nNumberOfKey];

		// Moving: key
		moveKey(a_nQKey, a_nKey_dnd);

		if(m_nOrder_dnd == -1)
			{
			shift(a_nKey_dnd, -m_nKey_shift);
			moveKey(a_nKey_dnd, a_nKey_dpcm);
			}
		else if(m_nOrder_dnd == 0)
			moveKey(a_nKey_dnd, a_nKey_fold);
		else
			{
			// Shift down
			if(m_nKey_shift_down != -1)
				shiftDown(a_nKey_dnd, m_nKey_shift_down);

			// Inverse DND
			for(i = m_nOrder_dnd; i > 0; i--)
				{
				int nKey_max = m_a_nKey_dnd_max[i-1];
				
				if(nKey_max >= 0)
					{
					if(i == 1)
						inverseDivide(a_nKey_dnd, nKey_max);
					else
						inverseDivide_down(a_nKey_dnd, nKey_max);
					}
				else
					{
					int nKey_min = nKey_max;
					inverseDivide_up(a_nKey_dnd, nKey_min);
					}
				}

			moveKey(a_nKey_dnd, a_nKey_fold);
			}

		if(m_nOrder_dnd != -1)
			{
			inverseFold(a_nKey_fold);

			// Shift
			shift(a_nKey_fold, -m_nKey_shift);

			//write(a_nKey_fold, "k_dec.txt");

			// Moving: key
			moveKey(a_nKey_fold, a_nKey_dpcm);
			}

		// Nth-order inverse DPCM
		for(i = 0; i < m_nOrder_dpcm; i++)
			{
			inverseDPCM(a_nKey_dpcm);
			}

		// Moving: key
		moveKey(a_nKey_dpcm, m_a_nQKey);

		// Inverse Quantization
		inverseQuantize(nQStep_key);

		delete[] a_nKey_dnd;
		delete[] a_nKey_fold;
		delete[] a_nKey_dpcm;
		}

	// Moving: inversed key data
	void getIQKey(double*& a_dIQKey)
		{
		a_dIQKey = new double[m_nNumberOfKey];
		for(int i = 0; i < m_nNumberOfKey; i++)
			a_dIQKey[i] = m_a_dIQKey[i];
		}

	// Moving: max values needed for dnd
	void setKey_dnd_max(int* a_nKey_dnd_max)
		{
		for(int i = 0; i < m_nOrder_dnd; i++)
			m_a_nKey_dnd_max[i] = a_nKey_dnd_max[i];
		}

	// Moving: dpcm order
	void setOrder_dpcm(int nOrder_dpcm)
		{
		m_nOrder_dpcm = nOrder_dpcm;
		}

	// Moving: dnd order
	void setOrder_dnd(int nOrder_dnd)
		{
		m_nOrder_dnd = nOrder_dnd;
		}

	// Moving: m_nKey_shift
	void setKey_shift(int nKey_shift)
		{
		m_nKey_shift = nKey_shift;
		}

	// Moving: m_nKey_shift_down
	void setKey_shift_down(int nKey_shift_down)
		{
		m_nKey_shift_down = nKey_shift_down;
		}

	// Dump
	void write(int* a_nKey, char* szFile, int nNum = -1)
		{
		if(nNum == -1)
			nNum = m_nNumberOfKey;

		ofstream fout(szFile);
		for(int i = 0; i < nNum; i++)
			fout << a_nKey[i] << endl;
		fout.close();
		}

	// Dump
	void write(double* a_dKey, char* szFile, int nNum = -1)
		{
		if(nNum == -1)
			nNum = m_nNumberOfKey;

		ofstream fout(szFile);
		for(int i = 0; i < nNum; i++)
			fout << a_dKey[i] << endl;
		fout.close();
		}

	// Dump
	void write(int* a_nKey, int nNum, char* szFile)
		{
		write(a_nKey, szFile, nNum);
		}

	// Dump
	void write(double* a_dKey, int nNum, char* szFile)
		{
		write(a_dKey, szFile, nNum);
		}

private:
	// Inverse Quantization
	void inverseQuantize(int nQStep_key)
		{
		int nQKey;
		double dIQKey;

		for(int i = 0; i < m_nNumberOfKey; i++)
			{
			nQKey = m_a_nQKey[i];
			dIQKey = inverseQuantizeKey(nQKey, m_dKey_max, m_dKey_min, nQStep_key);	// iQuantize
			dIQKey = roundOffWithDigit(dIQKey, m_nDigit_key);

			m_a_dIQKey[i] = dIQKey;
			}
		}

	// Inverse Quantization
	double inverseQuantizeKey(int nQKey, double dMax, double dMin, int nQStep)
		{
		double dIQKey;

		if(nQKey == 0)
			return dMin;
		else if(nQKey == ((1 << nQStep) - 1))
			return dMax;
		else
			dIQKey = (double)(dMin + ((double)nQKey / ((1 << nQStep) - 1)) * (dMax - dMin));

		return dIQKey;
		}

	// Moving: key
	void moveKey(int* a_nKey_source, int* a_nKey_target)
		{
		for(int i = 0; i < m_nNumberOfKey; i++)
			a_nKey_target[i] = a_nKey_source[i];
		}

	// Shift
	void shiftDown(int* a_nKey, int nKey_max)
		{
		for(int i = m_nOrder_dpcm; i < m_nNumberOfKey; i++)
			{ // d(n)=n-INT(n/(nKey_max/2+1))*(nKey_max+1)
			int nKey = a_nKey[i];
			if(nKey > nKey_max)
				a_nKey[i] = nKey_max - nKey;
			}
		}

	// Inverse divide
	void inverseDivide(int* a_nKey, int nKey_max)
		{
		for(int i = m_nOrder_dpcm; i < m_nNumberOfKey; i++)
			{ // d(n)=n-INT(n/(nKey_max/2+1))*(nKey_max+1)
			int nKey = a_nKey[i];
			nKey = nKey - (int)floor(nKey/((double)nKey_max/2+1)) * (nKey_max+1);
			a_nKey[i] = nKey; // i-dividing: // di(n)=n-floor(n/(nKey_max/2+1))*(nKey_max+1)
			}
		}

	// Inverse divide up
	void inverseDivide_up(int* a_nKey, int nKey_min)
		{
		for(int i = m_nOrder_dpcm; i < m_nNumberOfKey; i++)
			{
			int nKey = a_nKey[i];
			if(nKey < 0)
				{//=IF(H10>0,H10,IF(H10>=H$7/2,H10*2,(H$7-H10-1)*2+1))
				if(nKey % 2 != 0)
					nKey = (nKey_min-1)-(nKey-1)/2;
				else
					nKey /= 2;
				}
			a_nKey[i] = nKey;
			}
		}

	// Inverse divide down
	void inverseDivide_down(int* a_nKey, int nKey_max)
		{
		for(int i = m_nOrder_dpcm; i < m_nNumberOfKey; i++)
			{
			int nKey = a_nKey[i];
			if(nKey < 0)
				{
				if(nKey % 2 != 0)
					nKey = (nKey_max+1)+(nKey-1)/2;
				else
					nKey /= 2;
				}
			a_nKey[i] = nKey;
			}
		}

	// Inverse Fold
	void inverseFold(int* a_nKey)
		{
		for(int i = m_nOrder_dpcm; i < m_nNumberOfKey; i++)
			{
			int nKey = a_nKey[i];
			nKey = (nKey+nKey%2)/2*(1-2*(nKey%2)); // i-folding: fi(n)=(n+MOD(n,2))/2*(1-2*MOD(n,2))
			a_nKey[i] = nKey;
			}
		}

	// Shift
	void shift(int* a_nKey, int nKey_shift)
		{
		for(int i = m_nOrder_dpcm; i < m_nNumberOfKey; i++)
			a_nKey[i] -= nKey_shift;
		}

	// Inverse DPCM
	void inverseDPCM(int* a_nKey_dpcm)
		{
		for(int i = 1; i < m_nNumberOfKey; i++)
			a_nKey_dpcm[i] = a_nKey_dpcm[i] + a_nKey_dpcm[i-1];
		}
	
	// Round Off with Digit
	double roundOffWithDigit(double dSource, int nDigit)
		{
		int nDecimal = 0;
		int nSign = 0;
		char* szBuffer = NULL;
		double dValue = 0;
		
		szBuffer = _fcvt(dSource, 10, &nDecimal, &nSign);

		if(nSign == 0)
			dValue = (double)int(dSource*pow(10, nDigit-nDecimal)+0.5)/pow(10, nDigit-nDecimal);
		else
			dValue = (double)int(dSource*pow(10, nDigit-nDecimal)-0.5)/pow(10, nDigit-nDecimal);
		//cout << dSource << " -> " << dValue << endl;

		return dValue;
		}
	};