/***********************************************************************
This software module was originally developed by James D. K. Kim, 
Sang Oak Woo and Shin Jun Lee(Samsung Advance Institute of Technology)
in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)
standard. This software module is an implementation of a part of one or
more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4
Systems (ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 
Systems (ISO/IEC 14496-1) free license to this software module or 
modifications thereof for use in hardware or software products claiming
conformance to the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to 
use this software module in hardware or software products are advised that
its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Samsung Advance Institute of Technology
retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using
the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
This copyright notice must be included in all copies or derivative works.
Copyright  2001
************************************************************************

For more information or to receive an up to date version of this module,
you may contact James D. K. Kim at jacobkim@sait.samsung.co.kr
----------------------------------------------------------------------*/

// KeyValue decoder class
// Date: 2001. 9. 19. Wed.
// 4th revision: 2002-05-03
// Author: Shinjun Lee

#include <iostream.h>
#include <fstream.h>
#include <math.h>
#include "CCQ.h"

#define sign(n) ((n >= 0) ? 1 : -1))		// sign function

#define DPCM_T	1
#define DPCM_S	2
#define DPCM_TS	3

// KeyValue Decoder
class CKeyValueDecoder
	{
private:
	double*		m_a_dKeyValue;				// KeyValue data
	int*		m_a_nQKeyValue;				// quantized KeyValue data

	int			m_nNumberOfKey;				// the number of keys
	int			m_nNumberOfCoord;			// the number of vertices

	double		m_dX_max;					// x maximum
	double		m_dX_min;					// x minimum
	double		m_dY_max;					// y maximum
	double		m_dY_min;					// y minimum
	double		m_dZ_max;					// z maximum
	double		m_dZ_min;					// z minimum

	int			m_nDigit_keyValue;			// Key Value Digit

	bool*		m_a_bQFlag_kv;				// KeyValue quantization selection flag
	int*		m_a_nQStep_kv;				// KV QStep
	int*		m_a_nTSMode;				// TS Index of x, y, z
	int*		m_a_nQMin;					// quantized miminum values
	int*		m_a_nQMax;					// quantized maximum values
	int*		m_a_nBase;					// Base vertex index

public:
	CKeyValueDecoder(void)
		{
		m_a_dKeyValue = NULL;
		m_a_nQKeyValue = NULL;

		m_nNumberOfKey = 0;
		m_nNumberOfCoord = 0;

		m_dX_max = 0;
		m_dX_min = 0;
		m_dY_max = 0;
		m_dY_min = 0;
		m_dZ_max = 0;
		m_dZ_min = 0;

		m_nDigit_keyValue = 0;

		m_a_bQFlag_kv = NULL;
		m_a_nQStep_kv = NULL;
		m_a_nTSMode = NULL;
		m_a_nQMin = NULL;
		m_a_nQMax = NULL;
		m_a_nBase = NULL;
		}

	~CKeyValueDecoder()
		{
		delete[] m_a_dKeyValue;
		delete[] m_a_nQKeyValue;
		}

	// Setting: keyValue digit
	void setKeyValueDigit(int nDigit_keyValue)
		{
		m_nDigit_keyValue = nDigit_keyValue;
		}

	// set: Max, Min
	void setMaxMin(double dX_min, double dY_min, double dZ_min, double& dX_max, double& dY_max, double& dZ_max, double dXYZ_delta)
		{
		//double dDelta = 0;

		m_dX_min = dX_min;
		m_dY_min = dY_min;
		m_dZ_min = dZ_min;

		// round off
		m_dX_min = roundOffWithDigit(m_dX_min, m_nDigit_keyValue);
		m_dY_min = roundOffWithDigit(m_dY_min, m_nDigit_keyValue);
		m_dZ_min = roundOffWithDigit(m_dZ_min, m_nDigit_keyValue);

		m_dX_max = m_dX_min + dXYZ_delta;
		m_dY_max = m_dY_min + dXYZ_delta;
		m_dZ_max = m_dZ_min + dXYZ_delta;

		dX_max = m_dX_max;
		dY_max = m_dY_max;
		dZ_max = m_dZ_max;
		}

	// set: Max, Min
	void setMaxMin(double dX_min, double dY_min, double dZ_min, double& dX_max, double& dY_max, double& dZ_max, int nXYZ_max)
		{
		double dDelta = 0;

		m_dX_min = dX_min;
		m_dY_min = dY_min;
		m_dZ_min = dZ_min;

		// round off
		m_dX_min = roundOffWithDigit(m_dX_min, m_nDigit_keyValue);
		m_dY_min = roundOffWithDigit(m_dY_min, m_nDigit_keyValue);
		m_dZ_min = roundOffWithDigit(m_dZ_min, m_nDigit_keyValue);

		if(nXYZ_max == 0)
			{
			m_dX_max = dX_max;
			m_dX_max = roundOffWithDigit(m_dX_max, m_nDigit_keyValue);
			dDelta = m_dX_max - m_dX_min;
			}
		else if(nXYZ_max == 1)
			{
			m_dY_max = dY_max;
			m_dY_max = roundOffWithDigit(m_dY_max, m_nDigit_keyValue);
			dDelta = m_dY_max - m_dY_min;
			}
		else if(nXYZ_max == 2)
			{
			m_dZ_max = dZ_max;
			m_dZ_max = roundOffWithDigit(m_dZ_max, m_nDigit_keyValue);
			dDelta = m_dZ_max - m_dZ_min;
			}
		if(nXYZ_max != 3)
			{
			m_dX_max = m_dX_min + dDelta;
			m_dY_max = m_dY_min + dDelta;
			m_dZ_max = m_dZ_min + dDelta;
			
			dX_max = m_dX_max;
			dY_max = m_dY_max;
			dZ_max = m_dZ_max;
			}
		else
			{
			m_dX_max = dX_max;
			m_dY_max = dY_max;
			m_dZ_max = dZ_max;
			}
		}

	// QFlag_kv: selection flag
	void setQFlag_kv(bool* a_bQFlag_kv)
		{
		m_a_bQFlag_kv = a_bQFlag_kv;
		}

	// set: QStep_kv
	void setQStep_kv(int* a_nQStep_kv)
		{
		m_a_nQStep_kv = a_nQStep_kv;
		}

	// set: TSMode
	void setTSMode(int* a_nTSMode)
		{
		m_a_nTSMode = a_nTSMode;
		}

	// set: m_a_nQMin, m_a_nQMax
	void setQMinQMax(int* a_nQMin, int* a_nQMax)
		{
		m_a_nQMin = a_nQMin;
		m_a_nQMax = a_nQMax;
		}

	// set: base vertex
	void setBase(int* a_nBase)
		{
		m_a_nBase = a_nBase;
		}

	// decode
	void decode(int* a_nQKeyValue, int nNumberOfKey, int nNumberOfCoord, int nQStep_keyValue)
		{
		int j = 0;
		int c = 0;

		// Moving: the numbers
		m_nNumberOfKey = nNumberOfKey;
		m_nNumberOfCoord = nNumberOfCoord;

		// Allocation: m_a_nQKeyValue
		m_a_dKeyValue = new double[m_nNumberOfKey*m_nNumberOfCoord*3];
		m_a_nQKeyValue = new int[m_nNumberOfKey*m_nNumberOfCoord*3];

		memset((void*)m_a_dKeyValue, 0, sizeof(double)*(m_nNumberOfKey*m_nNumberOfCoord*3));
		memset((void*)m_a_nQKeyValue, 0, sizeof(int)*(m_nNumberOfKey*m_nNumberOfCoord*3));

		// Dump: DPCM File
		//dump_quantization("DPCM_dec", a_nQKeyValue);

		// inverse dpcm
		for(j = 0; j < m_nNumberOfCoord; j++)
			{
			for(c = 0; c < 3; c++)
				{
				switch(m_a_nTSMode[j*3+c])
					{
					case DPCM_T:
						inverseDPCM_temporal(m_a_nQKeyValue, a_nQKeyValue, j, c);
						break;

					case DPCM_S:
						inverseDPCM_spatial(m_a_nQKeyValue, a_nQKeyValue, j, c, m_a_nBase[j*3+c]);
						break;

					case DPCM_TS:
						inverseDPCM_tps(m_a_nQKeyValue, a_nQKeyValue, j, c, m_a_nBase[j*3+c]);
						break;
					}
				}
			}

		// Dump: Quantization File
		//dump_quantization("Q_dec", m_a_nQKeyValue);

		// Inverse Quantization: m_a_nQKeyValue -> m_a_dKeyValue
		inverseQuantize(m_a_dKeyValue, m_a_nQKeyValue, nQStep_keyValue);

		// Dump: Quantization File
		//dump_IQ("IQ_dec", m_a_dKeyValue);
		}

	// Moving: inversed quantized key value data
	void getIQKeyValue(double*& a_dIQKeyValue)
		{
		//ofstream fout("IQKeyValue.txt");
		a_dIQKeyValue = new double[m_nNumberOfKey*m_nNumberOfCoord*3];

		for(int i = 0; i < m_nNumberOfKey; i++)
			{
			for(int j = 0; j < m_nNumberOfCoord; j++)
				{
				a_dIQKeyValue[i*m_nNumberOfCoord*3+j*3] = m_a_dKeyValue[i*m_nNumberOfCoord*3+j*3];
				a_dIQKeyValue[i*m_nNumberOfCoord*3+j*3+1] = m_a_dKeyValue[i*m_nNumberOfCoord*3+j*3+1];
				a_dIQKeyValue[i*m_nNumberOfCoord*3+j*3+2] = m_a_dKeyValue[i*m_nNumberOfCoord*3+j*3+2];

				//fout << a_dIQKeyValue[i*m_nNumberOfCoord*3+j*3] << ",\t";
				//fout << a_dIQKeyValue[i*m_nNumberOfCoord*3+j*3+1] << ",\t";
				//fout << a_dIQKeyValue[i*m_nNumberOfCoord*3+j*3+2] << endl;
				}
			//fout << endl;
			}
		//fout.close();
		}

	// Dump: Quantization File
	void dump_quantization(char* szString, int* a_nQKeyValue)
		{
		char szFile[_MAX_FNAME];				// File name
		int i = 0;
		int j = 0;

		ofstream fout;
		strcpy(szFile, szString);
		strcat(szFile, ".txt");
		fout.open(szFile);

		fout << szString << "(x)\t";
		for(i = 0; i < m_nNumberOfKey; i++)
			fout << i << "\t";
		fout << endl;
		for(j = 0; j < m_nNumberOfCoord; j++)
			{
			fout << j;
			if(m_a_bQFlag_kv[j*3] == true)
				{
				for(i = 0; i < m_nNumberOfKey; i++)
 					fout << "\t" << a_nQKeyValue[i*m_nNumberOfCoord*3+j*3];
				}
			else
				{
				i = 0;
				fout << "\t" << a_nQKeyValue[i*m_nNumberOfCoord*3+j*3];
				}
			fout << endl;
			}
		fout << endl;

		fout << szString << "(y)\t";
		for(i = 0; i < m_nNumberOfKey; i++)
			fout << i << "\t";
		fout << endl;
		for(j = 0; j < m_nNumberOfCoord; j++)
			{
			fout << j;
			if(m_a_bQFlag_kv[j*3+1] == true)
				{
				for(i = 0; i < m_nNumberOfKey; i++)
					fout << "\t" << a_nQKeyValue[i*m_nNumberOfCoord*3+j*3+1];
				}
			else
				{
				i = 0;
				fout << "\t" << a_nQKeyValue[i*m_nNumberOfCoord*3+j*3+1];
				}
			fout << endl;
			}
		fout << endl;

		fout << szString << "(z)\t";
		for(i = 0; i < m_nNumberOfKey; i++)
			fout << i << "\t";
		fout << endl;
		for(j = 0; j < m_nNumberOfCoord; j++)
			{
			fout << j;
			if(m_a_bQFlag_kv[j*3+2] == true)
				{
				for(i = 0; i < m_nNumberOfKey; i++)
					fout << "\t" << a_nQKeyValue[i*m_nNumberOfCoord*3+j*3+2];
				}
			else
				{
				i = 0;
				fout << "\t" << a_nQKeyValue[i*m_nNumberOfCoord*3+j*3+2];
				}
			fout << endl;
			}
		fout.close();
		}

	// Dump: Quantization File
	void dump_IQ(char* szString, double* a_dKeyValue)
		{
		char szFile[_MAX_FNAME];				// File name
		int i = 0;
		int j = 0;

		ofstream fout;
		strcpy(szFile, szString);
		strcat(szFile, ".txt");
		fout.open(szFile);

		fout << szString << "(x)\t";
		for(i = 0; i < m_nNumberOfKey; i++)
			fout << i << "\t";
		fout << endl;
		for(j = 0; j < m_nNumberOfCoord; j++)
			{
			fout << j;
			if(m_a_bQFlag_kv[j*3] == true)
				{
				for(i = 0; i < m_nNumberOfKey; i++)
 					fout << "\t" << a_dKeyValue[i*m_nNumberOfCoord*3+j*3];
				}
			else
				{
				i = 0;
				fout << "\t" << a_dKeyValue[i*m_nNumberOfCoord*3+j*3];
				}
			fout << endl;
			}
		fout << endl;

		fout << szString << "(y)\t";
		for(i = 0; i < m_nNumberOfKey; i++)
			fout << i << "\t";
		fout << endl;
		for(j = 0; j < m_nNumberOfCoord; j++)
			{
			fout << j;
			if(m_a_bQFlag_kv[j*3+1] == true)
				{
				for(i = 0; i < m_nNumberOfKey; i++)
					fout << "\t" << a_dKeyValue[i*m_nNumberOfCoord*3+j*3+1];
				}
			else
				{
				i = 0;
				fout << "\t" << a_dKeyValue[i*m_nNumberOfCoord*3+j*3+1];
				}
			fout << endl;
			}
		fout << endl;

		fout << szString << "(z)\t";
		for(i = 0; i < m_nNumberOfKey; i++)
			fout << i << "\t";
		fout << endl;
		for(j = 0; j < m_nNumberOfCoord; j++)
			{
			fout << j;
			if(m_a_bQFlag_kv[j*3+2] == true)
				{
				for(i = 0; i < m_nNumberOfKey; i++)
					fout << "\t" << a_dKeyValue[i*m_nNumberOfCoord*3+j*3+2];
				}
			else
				{
				i = 0;
				fout << "\t" << a_dKeyValue[i*m_nNumberOfCoord*3+j*3+2];
				}
			fout << endl;
			}
		fout.close();
		}

	// Transpose: source data -> target data which is transposed
	void transpose(double* a_dKeyValue_target, double* a_dKeyValue_source, int nNumberOfKey, int nNumberOfCoord)
		{
		double* a_dKeyValue = new double[nNumberOfKey*nNumberOfCoord*3];
		memcpy((void*)a_dKeyValue, (void*)a_dKeyValue_source, sizeof(double)*(nNumberOfKey*nNumberOfCoord*3));

		for(int j = 0; j < nNumberOfCoord; j++)
			{
			for(int i = 0; i < nNumberOfKey; i++)
				{
				for(int c = 0; c < 3; c++)
					{
					double dValue = a_dKeyValue[i*nNumberOfCoord*3+j*3+c];

					a_dKeyValue_target[j*nNumberOfKey*3+i*3+c] = dValue;
					//cout << dValue;
					//cout << ", ";
					}
				//cout << endl;
				}
			//cout << endl;
			}

		delete[] a_dKeyValue;
		}

private:
	// Inverse Quantization
	void inverseQuantize(double* a_dKeyValue, int* a_nQKeyValue, int nQStep_keyValue)
		{
		int nQKeyValue = 0;
		double dIQKeyValue = 0;
		double dValue = 0;
		int nC = 0;
		bool bRoundOff = false;

		dValue = pow(2, nQStep_keyValue);
		while(dValue > 0)
			{
			dValue = int(dValue / 10);
			nC++;
			}
		if(nC >= m_nDigit_keyValue)
			bRoundOff = true;
		else
			bRoundOff = false;

		for(int i = 0; i < m_nNumberOfCoord; i++)
			{
			for(int j = 0; j < m_nNumberOfKey; j++)
				{
				nQKeyValue = a_nQKeyValue[j*m_nNumberOfCoord*3+i*3];		// X
				dIQKeyValue = inverseQuantizeKeyValue(nQKeyValue, m_dX_max, m_dX_min, nQStep_keyValue);
				if(bRoundOff == true)
					dIQKeyValue = roundOffWithDigit(dIQKeyValue, m_nDigit_keyValue);
				a_dKeyValue[j*m_nNumberOfCoord*3+i*3] = dIQKeyValue;

				nQKeyValue = a_nQKeyValue[j*m_nNumberOfCoord*3+i*3+1];		// Y
				dIQKeyValue = inverseQuantizeKeyValue(nQKeyValue, m_dY_max, m_dY_min, nQStep_keyValue);
				if(bRoundOff == true)
					dIQKeyValue = roundOffWithDigit(dIQKeyValue, m_nDigit_keyValue);
				a_dKeyValue[j*m_nNumberOfCoord*3+i*3+1] = dIQKeyValue;

				nQKeyValue = a_nQKeyValue[j*m_nNumberOfCoord*3+i*3+2];		// Z
				dIQKeyValue = inverseQuantizeKeyValue(nQKeyValue, m_dZ_max, m_dZ_min, nQStep_keyValue);
				if(bRoundOff == true)
					dIQKeyValue = roundOffWithDigit(dIQKeyValue, m_nDigit_keyValue);
				a_dKeyValue[j*m_nNumberOfCoord*3+i*3+2] = dIQKeyValue;
				}
			}
		}

	// InverseQuantization: only one keyValue
	double inverseQuantizeKeyValue(int nQKeyValue, double dMax, double dMin, int nQStep)
		{			
		double dIQKeyValue;		// inverse quantized key value

		if(nQKeyValue == 0)
			return dMin;
		else if(nQKeyValue == ((1 << nQStep) - 1))
			return dMax;
		else
			dIQKeyValue = (double)(dMin + ((double)nQKeyValue / ((1 << nQStep) - 1)) * (dMax - dMin));

		return dIQKeyValue;
		}

	// Inverse Temporal DPCM
	void inverseDPCM_temporal(int* a_nQKeyValue, int* a_nQKeyValue_dpcm, int nCoord, int nIndex)
		{
		int nValue = 0;
		int nQ = 0;
		int nCQ = 0;
		int i = 0;

		CCQ oCQ;

		if(m_a_bQFlag_kv[nCoord*3+nIndex] == true)
			{
			nCQ = a_nQKeyValue_dpcm[i*m_nNumberOfCoord*3+nCoord*3+nIndex];
			// inverse circular quantize: if(nCQ < 0) nQ = nCQ + Max + 1; else nQ = nCQ + Min;
			nQ = oCQ.inverseQuantizeFirstValue(nCQ, m_a_nQMin[nCoord*3+nIndex], m_a_nQMax[nCoord*3+nIndex]);
			a_nQKeyValue[i*m_nNumberOfCoord*3+nCoord*3+nIndex] = nQ;
			}
		else
			a_nQKeyValue[i*m_nNumberOfCoord*3+nCoord*3+nIndex] = a_nQKeyValue_dpcm[i*m_nNumberOfCoord*3+nCoord*3+nIndex];

		for(i = 1; i < m_nNumberOfKey; i++)
			{
			nValue = a_nQKeyValue[(i-1)*m_nNumberOfCoord*3+nCoord*3+nIndex];	// the previous value

			// inverse circular qunatization
			if(m_a_bQFlag_kv[nCoord*3+nIndex] == true)
				{
				nCQ = a_nQKeyValue_dpcm[i*m_nNumberOfCoord*3+nCoord*3+nIndex];
				nQ = oCQ.inverseQuantize(nCQ, m_a_nQMin[nCoord*3+nIndex], m_a_nQMax[nCoord*3+nIndex], nValue);
				a_nQKeyValue[i*m_nNumberOfCoord*3+nCoord*3+nIndex] = nQ;
				}
			else
				a_nQKeyValue[i*m_nNumberOfCoord*3+nCoord*3+nIndex] = a_nQKeyValue_dpcm[i*m_nNumberOfCoord*3+nCoord*3+nIndex] + nValue;
			}
		}

	// Inverse Spatial DPCM
	void inverseDPCM_spatial(int* a_nQKeyValue, int* a_nQKeyValue_dpcm, int nCoord, int nIndex, int nBase = -1)
		{
		int nValue = 0;
		int nQ = 0;
		int nCQ = 0;
		int i = 0;

		CCQ oCQ;

		for(i = 0; i < m_nNumberOfKey; i++)
			{
			if(nCoord == 0)
				a_nQKeyValue[i*m_nNumberOfCoord*3+nCoord*3+nIndex] = a_nQKeyValue_dpcm[i*m_nNumberOfCoord*3+nCoord*3+nIndex];
			else
				{
				if(nBase == -1)	// if there is no base vertex, the base vertex is the previous vertex
					nValue = a_nQKeyValue[i*m_nNumberOfCoord*3+(nCoord-1)*3+nIndex];
				else
					nValue = a_nQKeyValue[i*m_nNumberOfCoord*3+nBase*3+nIndex];

				// cutting-off
				if(nValue < m_a_nQMin[nCoord*3+nIndex])
					nValue = m_a_nQMin[nCoord*3+nIndex];
				else if(nValue > m_a_nQMax[nCoord*3+nIndex])
					nValue = m_a_nQMax[nCoord*3+nIndex];

				// inverse circular qunatization
				if(m_a_bQFlag_kv[nCoord*3+nIndex] == true)
					{
					nCQ = a_nQKeyValue_dpcm[i*m_nNumberOfCoord*3+nCoord*3+nIndex];
					nQ = oCQ.inverseQuantize(nCQ, m_a_nQMin[nCoord*3+nIndex], m_a_nQMax[nCoord*3+nIndex], nValue);
					a_nQKeyValue[i*m_nNumberOfCoord*3+nCoord*3+nIndex] = nQ;
					}
				else
					a_nQKeyValue[i*m_nNumberOfCoord*3+nCoord*3+nIndex] = nValue;
					//a_nQKeyValue[i*m_nNumberOfCoord*3+nCoord*3+nIndex] = a_nQKeyValue_dpcm[i*m_nNumberOfCoord*3+nCoord*3+nIndex] + nValue;
				}
			}
		}

	// Inverse Temporal Spatial DPCM
	void inverseDPCM_tps(int* a_nQKeyValue, int* a_nQKeyValue_dpcm, int nCoord, int nIndex, int nBase = -1)
		{
		int nValue = 0;
		int nQ = 0;
		int nCQ = 0;
		int i = 0;

		CCQ oCQ;

		if(m_a_bQFlag_kv[nCoord*3+nIndex] == true)
			{
			nCQ = a_nQKeyValue_dpcm[i*m_nNumberOfCoord*3+nCoord*3+nIndex];
			// inverse circular quantize: if(nCQ < 0) nQ = nCQ + Max + 1; else nQ = nCQ + Min;
			nQ = oCQ.inverseQuantizeFirstValue(nCQ, m_a_nQMin[nCoord*3+nIndex], m_a_nQMax[nCoord*3+nIndex]);
			a_nQKeyValue[i*m_nNumberOfCoord*3+nCoord*3+nIndex] = nQ;
			}
		else
			a_nQKeyValue[i*m_nNumberOfCoord*3+nCoord*3+nIndex] = a_nQKeyValue_dpcm[i*m_nNumberOfCoord*3+nCoord*3+nIndex];

		for(i = 1; i < m_nNumberOfKey; i++)
			{
			if(nCoord == 0)
				a_nQKeyValue[i*m_nNumberOfCoord*3+nCoord*3+nIndex] = (a_nQKeyValue_dpcm[i*m_nNumberOfCoord*3+nCoord*3+nIndex] + a_nQKeyValue[(i-1)*m_nNumberOfCoord*3+nCoord*3+nIndex]);
			else
				{
				if(nBase == -1) // if there is no base vertex, the prediction value is as follows.
					nValue = a_nQKeyValue[(i-1)*m_nNumberOfCoord*3+nCoord*3+nIndex]
							+ (a_nQKeyValue[i*m_nNumberOfCoord*3+(nCoord-1)*3+nIndex] 
							- a_nQKeyValue[(i-1)*m_nNumberOfCoord*3+(nCoord-1)*3+nIndex]);
				else
					nValue = a_nQKeyValue[(i-1)*m_nNumberOfCoord*3+nCoord*3+nIndex]
							+ (a_nQKeyValue[i*m_nNumberOfCoord*3+nBase*3+nIndex] 
							- a_nQKeyValue[(i-1)*m_nNumberOfCoord*3+nBase*3+nIndex]);

				// cutting-off
				if(nValue < m_a_nQMin[nCoord*3+nIndex])
					nValue = m_a_nQMin[nCoord*3+nIndex];
				else if(nValue > m_a_nQMax[nCoord*3+nIndex])
					nValue = m_a_nQMax[nCoord*3+nIndex];

				// inverse circular qunatization
				if(m_a_bQFlag_kv[nCoord*3+nIndex] == true)
					{
					nCQ = a_nQKeyValue_dpcm[i*m_nNumberOfCoord*3+nCoord*3+nIndex];
					nQ = oCQ.inverseQuantize(nCQ, m_a_nQMin[nCoord*3+nIndex], m_a_nQMax[nCoord*3+nIndex], nValue);
					a_nQKeyValue[i*m_nNumberOfCoord*3+nCoord*3+nIndex] = nQ;
					}
				else
					a_nQKeyValue[i*m_nNumberOfCoord*3+nCoord*3+nIndex] = a_nQKeyValue_dpcm[i*m_nNumberOfCoord*3+nCoord*3+nIndex] + nValue;
				}
			}
		}

	// Round Off with Digit
	double roundOffWithDigit(double dSource, int nDigit)
		{
		int nDecimal = 0;
		int nSign = 0;
		char* szBuffer = NULL;
		double dValue = 0;
		
		szBuffer = _fcvt(dSource, 10, &nDecimal, &nSign);

		if(nSign == 0)
			dValue = (double)int(dSource*pow(10, nDigit-nDecimal)+0.5)/pow(10, nDigit-nDecimal);
		else
			dValue = (double)int(dSource*pow(10, nDigit-nDecimal)-0.5)/pow(10, nDigit-nDecimal);
		//cout << dSource << " -> " << dValue << endl;

		return dValue;
		}

	// Moving: quantized KeyValue
	void moveQKeyValue(int* a_nQKeyValue_destination, int* a_nQKeyValue_source)
		{
		for(int i = 0; i < m_nNumberOfKey; i++)
			{
			for(int j = 0; j < m_nNumberOfCoord; j++)
				{
				a_nQKeyValue_destination[i*m_nNumberOfCoord*3+j*3] = a_nQKeyValue_source[i*m_nNumberOfCoord*3+j*3];
				a_nQKeyValue_destination[i*m_nNumberOfCoord*3+j*3+1] = a_nQKeyValue_source[i*m_nNumberOfCoord*3+j*3+1];
				a_nQKeyValue_destination[i*m_nNumberOfCoord*3+j*3+2] = a_nQKeyValue_source[i*m_nNumberOfCoord*3+j*3+2];
				}
			}
		}
	};
