/***********************************************************************
This software module was originally developed by James D. K. Kim, 
Sang Oak Woo and Shin Jun Lee(Samsung Advance Institute of Technology)
in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)
standard. This software module is an implementation of a part of one or
more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4
Systems (ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 
Systems (ISO/IEC 14496-1) free license to this software module or 
modifications thereof for use in hardware or software products claiming
conformance to the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to 
use this software module in hardware or software products are advised that
its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Samsung Advance Institute of Technology
retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using
the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
This copyright notice must be included in all copies or derivative works.
Copyright  2001
************************************************************************

For more information or to receive an up to date version of this module,
you may contact James D. K. Kim at jacobkim@sait.samsung.co.kr
----------------------------------------------------------------------*/

// CompressedCoordinateInterpolator.h: coordinate interpolator decoder
// Author: Shinjun Lee, Samsung AIT
// Date: 2001-12-27
// 4th revision: 2002-05-03

#ifndef _COMPRESSED_COORDINATE_INTERPOLATOR_H_
#define _COMPRESSED_COORDINATE_INTERPOLATOR_H_

#include <fstream.h>

#include "KeyHeader.h"
#include "CoordIKeyValueHeader.h"
#include "Key.h"
#include "CoordIKeyValue.h"

class CompressedCoordinateInterpolator
	{
  public:
	KeyHeader				kHeader;						// class of key header bitstream
	CoordIKeyValueHeader	coordIKVHeader;					// class of key value header bitstream
	Key						k;								// class of key bitstream
	CoordIKeyValue			coordIKeyValue;					// class of key value bitstream

// 2003-06-27, Shinjun {
	bool					m_bIsDEF;
	char					m_szDEF[256];
// 2003-06-27, Shinjun }

	char			m_szBitstreamFile[_MAX_PATH];			// bitstream file name
	char			m_szOutputTextFile[_MAX_PATH];			// output text file name

  public:
	CompressedCoordinateInterpolator(void)
		{
		memset((void*)m_szBitstreamFile, 0, _MAX_PATH);
		memset((void*)m_szOutputTextFile, 0, _MAX_PATH);
		}

	CompressedCoordinateInterpolator(int, char* [])
		{
		memset((void*)m_szBitstreamFile, 0, _MAX_PATH);
		memset((void*)m_szOutputTextFile, 0, _MAX_PATH);
		}

	~CompressedCoordinateInterpolator()
		{
		}

	// decode: data of coordinate interpolator
	void decode(char* szBitstreamFile)
		{
		int nNumberOfKeyTemp;

		strcpy(m_szBitstreamFile, szBitstreamFile);

		// read: bitstream of key header, keyValue header, key, keyValue
		readBitstream(m_szBitstreamFile);

		// dump: bitstream data
		//dump(m_szOutputTextFile);

		// key header decoding
		kHeader.decodingProcess();

		// key value header decoding
		coordIKVHeader.decodingProcess();

		// key decoding
		if(coordIKVHeader.bTranspose == true)
			{
			nNumberOfKeyTemp = kHeader.nNumberOfKey;
			kHeader.nNumberOfKey = coordIKVHeader.nNumberOfCoord;
			}
		k.decodingProcess(kHeader);

		// key value decoding
		if(coordIKVHeader.bTranspose == true)
			kHeader.nNumberOfKey = nNumberOfKeyTemp;
		coordIKeyValue.decodingProcess(coordIKVHeader, kHeader.nNumberOfKey);
		
		// write: decoded text file
		//writeTextFile(m_szOutputTextFile);
		//dump_real("Real_dec");
		}

	// read: data of bitstream
	void readBitstream(char* szBitstreamFile)
		{
		// aac codec
		CInStream istrm;
		SAIT_CAC::_shift = 16; 
		CAC_QF codec(istrm);
		bool bSuccess = false;	// to avoid compilation error
		bSuccess = istrm.opendec(szBitstreamFile);

		// read: bitstream of header
		kHeader.readBitstream(codec);
		// 2002-04-19, Shinjun
		coordIKVHeader.readBitstream(codec);

		// byte align
		int nRest = 8-codec._istrm->Inbuf.nbits;
		int nDummy = 0;	// to avoid compilation error
		nDummy = codec._istrm->getbits(nRest);

		// start: aac decoding
		codec.start_decode();
		//CQF_Status::reset();
		k.InitializeModel();
		coordIKeyValue.InitializeModel();

		// read: bitstream of aac data
		k.readBitstream(codec, kHeader);
		// 2002-04-19, Shinjun
		coordIKeyValue.readBitstream(codec, coordIKVHeader, kHeader.nNumberOfKey);

		// finish: aac decoding
		codec.finish_decode();
		istrm.bfree();
		}


	// 20020711 BitWrapper
	// decode: data of coordinate interpolator
	void decodeWithBuffer(char* szBitstreamFile, int nLength = -1)
		{
		int nNumberOfKeyTemp = 0;

		//strcpy(m_szBitstreamFile, szBitstreamFile);

		// read: bitstream of key header, keyValue header, key, keyValue
		readBitstreamWithBuffer(szBitstreamFile, nLength);

		// dump: bitstream data
		//dump(m_szOutputTextFile);

		// key header decoding
		kHeader.decodingProcess();

		// key value header decoding
		coordIKVHeader.decodingProcess();

		// key decoding
		if(coordIKVHeader.bTranspose == true)
			{
			nNumberOfKeyTemp = kHeader.nNumberOfKey;
			kHeader.nNumberOfKey = coordIKVHeader.nNumberOfCoord;
			}
		k.decodingProcess(kHeader);

		// key value decoding
		if(coordIKVHeader.bTranspose == true)
			kHeader.nNumberOfKey = nNumberOfKeyTemp;
		coordIKeyValue.decodingProcess(coordIKVHeader, kHeader.nNumberOfKey);
		
		// write: decoded text file
		//writeTextFile(m_szOutputTextFile);
		//dump_real("Real_dec");
		}

	// 20020711 BitWrapper
	// read: data of bitstream
	void readBitstreamWithBuffer(char* szBitstreamFile, int nLength = -1)
		{
		// aac codec
		CInStream istrm;
		SAIT_CAC::_shift = 16; 
		CAC_QF codec(istrm);
		bool bSuccess = false;	// to avoid compilation error
		bSuccess = istrm.opendecWithBuffer(szBitstreamFile, nLength);

		// read: bitstream of header
		kHeader.readBitstream(codec);
		// 2002-04-19, Shinjun
		coordIKVHeader.readBitstream(codec);

		// byte align
		int nRest = 8-codec._istrm->Inbuf.nbits;
		int nDummy = 0;	// to avoid compilation error
		nDummy = codec._istrm->getbits(nRest);

		// start: aac decoding
		codec.start_decode();
		//CQF_Status::reset();
		k.InitializeModel();
		coordIKeyValue.InitializeModel();

		// read: bitstream of aac data
		k.readBitstream(codec, kHeader);
		// 2002-04-19, Shinjun
		coordIKeyValue.readBitstream(codec, coordIKVHeader, kHeader.nNumberOfKey);

		// finish: aac decoding
		codec.finish_decode();
		istrm.bfree();
		}

#if defined SAIT_CONFORMANCE_2003
	void setValues(unsigned char* pData)
	{
		int* pData_NumberOfKeys = (int*) pData;
		int *pData_NumberOfCoords = (int*) (pData + sizeof(int));
		float* pData_Keys = (float*) (pData + 2*sizeof(int));
		int nNumberOfKeys = *pData_NumberOfKeys;
		float* pData_KeyValues = (float*) (pData + 2*sizeof(int) + nNumberOfKeys*sizeof(float));
		int nNumberOfCoords = *pData_NumberOfCoords;
		int i = 0;
		int j = 0;

		// allocation
		k.dKey = new double[nNumberOfKeys];
		coordIKeyValue.dKeyValue = new double[nNumberOfKeys*nNumberOfCoords*3];

		// initialization
		kHeader.nNumberOfKey = nNumberOfKeys;
		coordIKVHeader.nNumberOfCoord = nNumberOfCoords;

		// key data
		for(i = 0; i < nNumberOfKeys; i++)
		{
			k.dKey[i] = roundOffWithDigit(pData_Keys[i], 6);
		}

		// key value data
		for(i = 0; i < nNumberOfKeys; i++)
		{
			for(j = 0; j < nNumberOfCoords; j++)
			{
				coordIKeyValue.dKeyValue[i*3*nNumberOfCoords+j*3] = roundOffWithDigit(pData_KeyValues[i*3*nNumberOfCoords+j*3], 6);
				coordIKeyValue.dKeyValue[i*3*nNumberOfCoords+j*3+1] = roundOffWithDigit(pData_KeyValues[i*3*nNumberOfCoords+j*3+1], 6);
				coordIKeyValue.dKeyValue[i*3*nNumberOfCoords+j*3+2] = roundOffWithDigit(pData_KeyValues[i*3*nNumberOfCoords+j*3+2], 6);
			}
		}
	}
#endif SAIT_CONFORMANCE_2003

	// write: output decoded text file
	void write(char* szOutputTextFile, int nDefID = -1)
	{
		char		szNewOutputTextFile[_MAX_PATH] = "";	// new file name
		char		szBuffer[1024] = "";					// string buffer
		char*		pDest = NULL;							// pointer for string search
		int			i = 0;									// index
		int			j = 0;									// index

		ofstream fout;
		if(nDefID != -1)
		{
			// find ".txt" or ".dec" and remove the string
			strcpy(szNewOutputTextFile, szOutputTextFile);
			pDest = strstr(szNewOutputTextFile, ".txt");
			if(pDest)
				*pDest = NULL;
			pDest = strstr(szNewOutputTextFile, ".dec");
			if(pDest)
				*pDest = NULL;
			// add "_ID" + nDefID + ".txt"
			strcat(szNewOutputTextFile, "_ID");
			_itoa(nDefID, szBuffer, 10);
			strcat(szNewOutputTextFile, szBuffer);
			strcat(szNewOutputTextFile, ".txt");
			// setting when nDefID is existed
			m_bIsDEF = true;
			strcpy(m_szDEF, "ID");
			strcat(m_szDEF, szBuffer);
		}
		else
		{
			// setting when nDefID is not existed
			strcpy(szNewOutputTextFile, szOutputTextFile);
			m_bIsDEF = false;
			strcpy(m_szDEF, "myCOORDINATE");
		}

		fout.open(szNewOutputTextFile);
		if(!fout.is_open())
		{
			cout << "The file cannot be opened." << endl;
			cout << "File Name: " << szNewOutputTextFile << endl;
			return;
		}

		if(m_bIsDEF == true)
			fout << "DEF " << m_szDEF << " ";
		fout << "CoordinateInterpolator {" << endl;

		// write: the data of key
		fout << "\tkey [" << endl;
		for(i = 0; i < kHeader.nNumberOfKey; i++)
		{
			dtoa(k.dKey[i], szBuffer);
			fout << "\t\t" << szBuffer << endl;
		}
		fout << "\t]" << endl;

		// write: the data of keyValue
		fout << "\tkeyValue [" << endl;
		for(i = 0; i < kHeader.nNumberOfKey; i++)
		{
			for(j = 0; j < coordIKVHeader.nNumberOfCoord; j++)
			{
				fout << "\t\t";

				// Writing: X
				dtoa(coordIKeyValue.dKeyValue[i*3*coordIKVHeader.nNumberOfCoord+j*3], szBuffer);
				fout << szBuffer << " ";
				
				// Writing: Y
				dtoa(coordIKeyValue.dKeyValue[i*3*coordIKVHeader.nNumberOfCoord+j*3+1], szBuffer);
				fout << szBuffer << " ";
				
				// Writing: Z
				dtoa(coordIKeyValue.dKeyValue[i*3*coordIKVHeader.nNumberOfCoord+j*3+2], szBuffer);
				fout << szBuffer << "," << endl;
			}
		}
		fout << "\t]" << endl;
		fout << "}" << endl;
		fout.close();
	}

	// dump: each data of key, keyValue
	/*
	void dump(char* szOutputTextFile)
		{
		int			j = 0;									// index

		// write: DPCM mode
		ofstream fout_dm;
		fout_dm.open("DPCMMode_dec.txt");
		fout_dm << szOutputTextFile << endl;
		for(j = 0; j < coordIKVHeader.nNumberOfCoord; j++)
			{
			fout_dm << j << "\t";
			fout_dm << coordIKeyValue.nDPCMMode[j*3] << "\t";
			fout_dm << coordIKeyValue.nDPCMMode[j*3+1] << "\t";
			fout_dm << coordIKeyValue.nDPCMMode[j*3+2] << endl;
			}
		fout_dm.close();

		// write: quantization selection flag
		ofstream fout_flag("QSelFlag_kv_dec.txt");
		fout_flag << szOutputTextFile << endl;
		for(j = 0; j < coordIKVHeader.nNumberOfCoord; j++)
			{
			fout_flag << j << "\t";
			fout_flag << coordIKeyValue.bSelFlag[j*3] << "\t";
			fout_flag << coordIKeyValue.bSelFlag[j*3+1] << "\t";
			fout_flag << coordIKeyValue.bSelFlag[j*3+2] << endl;
			}
		fout_flag.close();

		// write: AQStep
		ofstream fout_aqstep("AQStep_kv_dec.txt");
		fout_aqstep << szOutputTextFile << endl;
		for(j = 0; j < coordIKVHeader.nNumberOfCoord; j++)
			{
			fout_aqstep << j << "\t";
			fout_aqstep << coordIKeyValue.nKVACodingBit[j*3] << "\t";
			fout_aqstep << coordIKeyValue.nKVACodingBit[j*3+1] << "\t";
			fout_aqstep << coordIKeyValue.nKVACodingBit[j*3+2] << endl;
			}
		fout_aqstep.close();

		// write: reference vertex
		ofstream fout_base("RefVertex_kv_dec.txt");
		fout_base << szOutputTextFile << endl;
		for(j = 0; j < coordIKVHeader.nNumberOfCoord; j++)
			{
			fout_base << j << "\t";
			if(coordIKeyValue.bSelFlag[j*3] == true)
				fout_base << coordIKeyValue.nRefVertex[j*3] << "\t";
			else
				fout_base << 0 << "\t";
			if(coordIKeyValue.bSelFlag[j*3+1] == true)
				fout_base << coordIKeyValue.nRefVertex[j*3+1] << "\t";
			else
				fout_base << 0 << "\t";
			if(coordIKeyValue.bSelFlag[j*3+2] == true)
				fout_base << coordIKeyValue.nRefVertex[j*3+2] << endl;
			else
				fout_base << 0 << endl;
			}
		fout_base << endl;
		fout_base.close();

		// write: qmin, qmax
		ofstream fout_qmm("QMinQMax_dec.txt");
		for(j = 0; j < coordIKVHeader.nNumberOfCoord; j++)
			{
			fout_qmm << j << "\t";
			fout_qmm << coordIKeyValue.nQMin[j*3] << "\t";
			fout_qmm << coordIKeyValue.nQMax[j*3] << "\t";
			fout_qmm << coordIKeyValue.nQMin[j*3+1] << "\t";
			fout_qmm << coordIKeyValue.nQMax[j*3+1] << "\t";
			fout_qmm << coordIKeyValue.nQMin[j*3+2] << "\t";
			fout_qmm << coordIKeyValue.nQMax[j*3+2] << endl;
			}
		fout_qmm.close();
		}
	//*/

	// Dump: real data File
	/*//
	void dump_real(char* szString)
		{
		char szFile[_MAX_FNAME];				// File name
		int i = 0;
		int j = 0;

		ofstream fout;
		strcpy(szFile, szString);
		strcat(szFile, ".txt");
		fout.open(szFile);

		fout << szString << "(x)\t";
		for(i = 0; i < kHeader.nNumberOfKey; i++)
			fout << i << "\t";
		fout << endl;
		for(j = 0; j < coordIKVHeader.nNumberOfCoord; j++)
			{
			fout << j;
			if(coordIKeyValue.bSelFlag[j*3] == true)
				{
				for(i = 0; i < kHeader.nNumberOfKey; i++)
 					fout << "\t" << coordIKeyValue.dKeyValue[i*coordIKVHeader.nNumberOfCoord*3+j*3];
				}
			else
				{
				i = 0;
				fout << "\t" << coordIKeyValue.dKeyValue[i*coordIKVHeader.nNumberOfCoord*3+j*3];
				}
			fout << endl;
			}
		fout << endl;

		fout << szString << "(y)\t";
		for(i = 0; i < kHeader.nNumberOfKey; i++)
			fout << i << "\t";
		fout << endl;
		for(j = 0; j < coordIKVHeader.nNumberOfCoord; j++)
			{
			fout << j;
			if(coordIKeyValue.bSelFlag[j*3+1] == true)
				{
				for(i = 0; i < kHeader.nNumberOfKey; i++)
					fout << "\t" << coordIKeyValue.dKeyValue[i*coordIKVHeader.nNumberOfCoord*3+j*3+1];
				}
			else
				{
				i = 0;
				fout << "\t" << coordIKeyValue.dKeyValue[i*coordIKVHeader.nNumberOfCoord*3+j*3+1];
				}
			fout << endl;
			}
		fout << endl;

		fout << szString << "(z)\t";
		for(i = 0; i < kHeader.nNumberOfKey; i++)
			fout << i << "\t";
		fout << endl;
		for(j = 0; j < coordIKVHeader.nNumberOfCoord; j++)
			{
			fout << j;
			if(coordIKeyValue.bSelFlag[j*3+2] == true)
				{
				for(i = 0; i < kHeader.nNumberOfKey; i++)
					fout << "\t" << coordIKeyValue.dKeyValue[i*coordIKVHeader.nNumberOfCoord*3+j*3+2];
				}
			else
				{
				i = 0;
				fout << "\t" << coordIKeyValue.dKeyValue[i*coordIKVHeader.nNumberOfCoord*3+j*3+2];
				}
			fout << endl;
			}
		fout.close();
		}
	//*/

  private:
	// convert: double to string
	void dtoa(double dValue, char* szString)
		{
		int			nStrLen = 0;							// the length of string
		char*		pchDot = NULL;							// the position of dot
		int			i = 0;									// index
		
		sprintf(szString, "%20.10g", dValue);				// format: "%20.10g"
		nStrLen = strlen(szString);
		pchDot = strrchr(szString, '.');
		for(i = nStrLen-1; i >= pchDot-szString && pchDot != NULL; i--)
			{
			if(szString[i] == '0')
				szString[i] = NULL;
			else if(szString[i] == '.')
				{
				szString[i] = NULL;
				break;
				}
			else
				break;
			}
		}

	// get: quantization bits needed for encoding a number
	int getNBits(int n)
		{
		return (int)(log10(n)/log10(2))+1;
		}

	// transpose: source data -> target data which is transposed
	void transposeKeyValue(double* a_dKeyValue_target, double* a_dKeyValue_source, int nNumberOfKey, int nNumberOfCoord)
		{
		double* a_dKeyValue = new double[nNumberOfKey*nNumberOfCoord*3];
		memcpy((void*)a_dKeyValue, (void*)a_dKeyValue_source, sizeof(double)*(nNumberOfKey*nNumberOfCoord*3));

		for(int j = 0; j < nNumberOfCoord; j++)
			{
			for(int i = 0; i < nNumberOfKey; i++)
				{
				for(int c = 0; c < 3; c++)
					{
					//double dValue = a_dKeyValue[i*nNumberOfCoord*3+j*3+c];
					double dValue = getValue(a_dKeyValue, i, j, c);

					//a_dKeyValue_target[j*nNumberOfKey*3+i*3+c] = dValue;
					getTransposedValue(a_dKeyValue_target, j, i, c) = dValue;
					//cout << dValue;
					//cout << ", ";
					}
				//cout << endl;
				}
			//cout << endl;
			}

		delete[] a_dKeyValue;
		}

	// get: the value with the indexes of nKey, nCoord, nComponent in keyValue
	double& getValue(double* a_dValue, int nKey, int nCoord, int nComponent)
		{
		double& dValue = a_dValue[nKey*coordIKVHeader.nNumberOfCoord*3 + nCoord*3 + nComponent];
		return dValue;
		}

	// get: the transposed value with the indexes of nKey, nCoord, nComponent in keyValue
	double& getTransposedValue(double* a_dValue, int nCoord, int nKey, int nComponent)
		{
		double& dValue = a_dValue[nCoord*kHeader.nNumberOfKey*3 + nKey*3 + nComponent];
		return dValue;
		}


		// Round Off with Digit
	double roundOffWithDigit(double dSource, int nDigit)
		{
		int nDecimal = 0;
		int nSign = 0;
		char* szBuffer = NULL;
		double dValue = 0;
		
		szBuffer = _fcvt(dSource, 10, &nDecimal, &nSign);

		if(nSign == 0)
			dValue = (double)int(dSource*pow(10, nDigit-nDecimal)+0.5)/pow(10, nDigit-nDecimal);
		else
			dValue = (double)int(dSource*pow(10, nDigit-nDecimal)-0.5)/pow(10, nDigit-nDecimal);
		//cout << dSource << " -> " << dValue << endl;

		return dValue;
		}
	};

#endif // _COMPRESSED_COORDINATE_INTERPOLATOR_H_
