/***********************************************************************
This software module was originally developed by James D. K. Kim, 
Sang Oak Woo and Shin Jun Lee(Samsung Advance Institute of Technology)
in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)
standard. This software module is an implementation of a part of one or
more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4
Systems (ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 
Systems (ISO/IEC 14496-1) free license to this software module or 
modifications thereof for use in hardware or software products claiming
conformance to the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to 
use this software module in hardware or software products are advised that
its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Samsung Advance Institute of Technology
retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using
the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
This copyright notice must be included in all copies or derivative works.
Copyright  2001
************************************************************************

For more information or to receive an up to date version of this module,
you may contact James D. K. Kim at jacobkim@sait.samsung.co.kr
----------------------------------------------------------------------*/

// CompressedOrientationInterpolator.cpp: implementation of the CompressedOrientationInterpolator class.
//
//////////////////////////////////////////////////////////////////////

//#include "stdafx.h"
#include "CompressedOrientationInterpolator.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CompressedOrientationInterpolator::CompressedOrientationInterpolator()
{

}

CompressedOrientationInterpolator::~CompressedOrientationInterpolator()
{

}


#define AACTotal() (codec._istrm->Inbuf.nbytes) * 8 + (codec._istrm->Inbuf.nbits)

void CompressedOrientationInterpolator::decode(char* fileName)
{
	// ---------------------------------------------------------
	//
	// AAC initialize 
	//
	// ---------------------------------------------------------
	CInStream istrm;
	SAIT_CAC::_shift = 16;
	CAC_QF codec(istrm);
	bool success = istrm.opendec(fileName);

	if(success == false)
	{
		// Jameskim 20020503
		// update for using in Windows application
		char temp[256];
		sprintf(temp, "Cannot open file : \n%s\n", fileName);
		//::AfxMessageBox(temp, MB_OK, 0);
		return;
	}



	// ---------------------------------------------------------
	//
	// Read Header information
	//
	// ---------------------------------------------------------
	kHeader.readBitstream(codec);
	oriIKVHeader.readBitstream(codec);



	// ---------------------------------------------------------
	//
	// AAC decoding start
	//
	// ---------------------------------------------------------
	// byte align
	if(codec._istrm->Inbuf.nbits != 0)
	{
		int rest = 8 - codec._istrm->Inbuf.nbits;
		codec._istrm->getbits(rest);
	}

	// qf_start();
	// decoder start();
	codec.start_decode();

	


	// ---------------------------------------------------------
	//
	// AAC decoding context initialize
	//
	// ---------------------------------------------------------
	// key context initialize
	k.InitializeModel();
	// keyflag context initialize
	ksFlag.InitializeModel();
	// keyvalue context initialize
	oriIDPCMKeyValue.InitializeModel();

	


	// ---------------------------------------------------------
	// 
	// Read AAC encoded bitstream
	// 
	// ---------------------------------------------------------
	// key selection flag
	ksFlag.readBitstream(codec, kHeader, oriIKVHeader.bPreserveKey);

	// key 
	k.readBitstream(codec, kHeader);

	// key value
	if(oriIKVHeader.nKVDPCMOrder == 0)
		oriIDPCMKeyValue.readBitstream(codec, oriIKVHeader.oriIDPCMKVHeader, ksFlag.nNumOfKeyValue-1);
	else
		oriIDPCMKeyValue.readBitstream(codec, oriIKVHeader.oriIDPCMKVHeader, ksFlag.nNumOfKeyValue-2);



	// ---------------------------------------------------------
	//
	// AAC decoding finish
	//
	// ---------------------------------------------------------
	// decoder finish();
	if(codec._istrm->Inbuf.nbits != 0)
	{
		int rest = 8 - codec._istrm->Inbuf.nbits;
		codec._istrm->getbits(rest);
	}
	codec.finish_decode();
	istrm.bfree();



	// ---------------------------------------------------------
	//
	// Decoding Process
	//
	// ---------------------------------------------------------
	// key header
	kHeader.decodingProcess();
	// key
	k.decodingProcess(kHeader);
	for(int i = 0;i < kHeader.nNumberOfKey;i++)
		dKey.Add(k.dKey[i]);

	// key value including interpolator synthesizer
	oriIDPCMKeyValue.decodingProcess(kHeader, ksFlag, oriIKVHeader, k.dKey, dKeyValue, dKeyValueQuat);

	//printf("Number of Decoded Bit : %d\n", AACTotal());

}

// 20020711 for BitWrapper
void CompressedOrientationInterpolator::decodeWithBuffer(char* fileName, int nLength)
{
	// ---------------------------------------------------------
	//
	// AAC initialize 
	//
	// ---------------------------------------------------------
	CInStream istrm;
	SAIT_CAC::_shift = 16;
	CAC_QF codec(istrm);
	bool success = istrm.opendecWithBuffer(fileName, nLength);

	if(success == false)
	{
		// Jameskim 20020503
		// update for using in Windows application
		char temp[256];
		sprintf(temp, "Cannot open file : \n%s\n", fileName);
		//::AfxMessageBox(temp, MB_OK, 0);
		return;
	}



	// ---------------------------------------------------------
	//
	// Read Header information
	//
	// ---------------------------------------------------------
	kHeader.readBitstream(codec);
	oriIKVHeader.readBitstream(codec);



	// ---------------------------------------------------------
	//
	// AAC decoding start
	//
	// ---------------------------------------------------------
	// byte align
	if(codec._istrm->Inbuf.nbits != 0)
	{
		int rest = 8 - codec._istrm->Inbuf.nbits;
		codec._istrm->getbits(rest);
	}

	// qf_start();
	// decoder start();
	codec.start_decode();

	


	// ---------------------------------------------------------
	//
	// AAC decoding context initialize
	//
	// ---------------------------------------------------------
	// key context initialize
	k.InitializeModel();
	// keyflag context initialize
	ksFlag.InitializeModel();
	// keyvalue context initialize
	oriIDPCMKeyValue.InitializeModel();

	


	// ---------------------------------------------------------
	// 
	// Read AAC encoded bitstream
	// 
	// ---------------------------------------------------------
	// key selection flag
	ksFlag.readBitstream(codec, kHeader, oriIKVHeader.bPreserveKey);

	// key 
	k.readBitstream(codec, kHeader);

	// key value
	if(oriIKVHeader.nKVDPCMOrder == 0)
		oriIDPCMKeyValue.readBitstream(codec, oriIKVHeader.oriIDPCMKVHeader, ksFlag.nNumOfKeyValue-1);
	else
		oriIDPCMKeyValue.readBitstream(codec, oriIKVHeader.oriIDPCMKVHeader, ksFlag.nNumOfKeyValue-2);



	// ---------------------------------------------------------
	//
	// AAC decoding finish
	//
	// ---------------------------------------------------------
	// decoder finish();
	if(codec._istrm->Inbuf.nbits != 0)
	{
		int rest = 8 - codec._istrm->Inbuf.nbits;
		codec._istrm->getbits(rest);
	}
	codec.finish_decode();
	istrm.bfree();



	// ---------------------------------------------------------
	//
	// Decoding Process
	//
	// ---------------------------------------------------------
	// key header
	kHeader.decodingProcess();
	// key
	k.decodingProcess(kHeader);
	for(int i = 0;i < kHeader.nNumberOfKey;i++)
		dKey.Add(k.dKey[i]);

	// key value including interpolator synthesizer
	oriIDPCMKeyValue.decodingProcess(kHeader, ksFlag, oriIKVHeader, k.dKey, dKeyValue, dKeyValueQuat);

	//printf("Number of Decoded Bit : %d\n", AACTotal());

}


#if defined SAIT_CONFORMANCE_2003
void CompressedOrientationInterpolator::setValues(unsigned char* pData)
{
	int* pData_NumberOfKeys = (int*) pData;
	float* pData_Keys = (float*) (pData + sizeof(int));
	int nNumberOfKeys = pData_NumberOfKeys[0];
	float* pData_KeyValues = (float*) (pData + sizeof(float)*nNumberOfKeys + sizeof(int));


	dKey.SetSize(nNumberOfKeys);
	dKeyValue.SetSize(nNumberOfKeys);

	for(int i=0,j=0; i<nNumberOfKeys; i++)
	{
		dKey[i] = pData_Keys[i];

		dKeyValue[i].x = pData_KeyValues[j++];
		dKeyValue[i].y = pData_KeyValues[j++];
		dKeyValue[i].z = pData_KeyValues[j++];
		dKeyValue[i].theta = pData_KeyValues[j++];
	}
}

void CompressedOrientationInterpolator::write(char* fileName, int defID)
#else
// SOWOO 20021210
void CompressedOrientationInterpolator::write(char* fileName)
#endif SAIT_CONFORMANCE_2003
{
	bool m_bIsDEF = false;
	char m_strDEF[100];

#if defined SAIT_CONFORMANCE_2003
	char fileName_new[100];
	ofstream file;

	if(defID != -1)
	{
		::sprintf(fileName_new, "%s_ID%d.dec", fileName, defID);
		file.open(fileName_new);
	}
	else
		file.open(fileName);

#else
	ofstream file;
	file.open(fileName);
#endif SAIT_CONFORMANCE_2003

	if(!file.is_open())
	{
		cout << "Can not open the file" << endl;
		cout << "File Name: " << fileName << endl;
		return;
	}

	// 3. Write key & keyValue
	// 3.1. Write DEF Name, if available

#if defined SAIT_CONFORMANCE_2003
	if(defID != -1)
	{
		m_bIsDEF = true;
		::sprintf(m_strDEF, "ID%d", defID);
	}
#else
	m_bIsDEF = true;
	::strcpy(m_strDEF, "myOrientation");
#endif SAIT_CONFORMANCE_2003

	if(m_bIsDEF)
		file << "DEF " << m_strDEF << ' ';

	// 3.2. PositionInterpolator {
	file << "OrientationInterpolator {" << endl;

	// 3.3. key
	file << "\t" << "key [" << endl;

	for(int i=0; i<dKey.GetSize(); i++)
	{
		if(i == dKey.GetSize() - 1)
			file << "\t\t" << dKey[i] << endl;
		else
			file << "\t\t" << dKey[i] << ", " << endl;
	}
	file << "\t" << "]" << endl;

	// 4. keyValue
	file << "\t" << "keyValue [" << endl;

	for(i=0; i<dKeyValue.GetSize(); i++)
	{
		file << "\t\t";

		if(i == dKeyValue.GetSize() - 1)
			file << dKeyValue[i] << endl;
		else
			file << dKeyValue[i] << ", " << endl;
	}
	file << "\t" << "]" << endl;

	file << '}' << endl;

	file.close();
}

