/***********************************************************************
This software module was originally developed by James D. K. Kim, 
Sang Oak Woo and Shin Jun Lee(Samsung Advance Institute of Technology)
in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)
standard. This software module is an implementation of a part of one or
more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4
Systems (ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 
Systems (ISO/IEC 14496-1) free license to this software module or 
modifications thereof for use in hardware or software products claiming
conformance to the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to 
use this software module in hardware or software products are advised that
its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Samsung Advance Institute of Technology
retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using
the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
This copyright notice must be included in all copies or derivative works.
Copyright ?2001
************************************************************************

For more information or to receive an up to date version of this module,
you may call Sang Oak Woo at sowoo@sait.samsung.co.kr
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Interpolator Compression
FILE:       CompressedPositionInterpolator.cpp
AUTHOR:     Sang Oak Woo (SAMSUNG AIT)
OVERVIEW:   Implementation of Interpolator Compression
-------------------------------------------------------------------------*/

// compressedPositionInterpolator.cpp: implementation of the CompressedPositionInterpolator class.
//
//////////////////////////////////////////////////////////////////////
#include "defs.h"
//#include "stdafx.h"


#include <stdio.h>
#include <string.h>
#include "compressedPositionInterpolator.h"

#define AACTotal() (codec._istrm->Inbuf.nbytes) * 8 + (codec._istrm->Inbuf.nbits)


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CompressedPositionInterpolator::CompressedPositionInterpolator():
	m_bIsDEF(false), m_nKeyDigit(-1), m_nKeyValueDigit(-1)
{

}

CompressedPositionInterpolator::~CompressedPositionInterpolator()
{

}


void CompressedPositionInterpolator::decode(char* fileName)
{
	// aac ʱȭ 
	CInStream istrm;
	SAIT_CAC::_shift = 16;
	CAC_QF codec(istrm);
	istrm.opendec(fileName);

	//////////////////////////////////////////////////////////////////
	//
	//		Header Information (KeyHeader, PosIKeyValueHeader)
	//
	//////////////////////////////////////////////////////////////////
	kHeader.readBitstream(codec);
	posIKVHeader.readBitstream(codec);

		//////////////////////////////////////////////////////////////
		m_nKeyDigit = kHeader.nKeyDigit;
		m_nKeyValueDigit = posIKVHeader.nKVDigit;
		//////////////////////////////////////////////////////////////

	// byte align
	if(codec._istrm->Inbuf.nbits != 0)
	{
		int rest = 8 - codec._istrm->Inbuf.nbits;
		codec._istrm->getbits(rest);
	}

	// qf_start();
	// decoder start();
	codec.start_decode();


	// key context initialize
	k.InitializeModel();
	// keyflag context initialize
	ksFlag.InitializeModel();
	// keyValue context initialize
	posIKV.InitializeModel();


	//////////////////////////////////////////////////////////////////
	//
	//		key & KeyValue
	//
	//////////////////////////////////////////////////////////////////
	ksFlag.readBitstream(codec, kHeader, posIKVHeader.bPreserveKey);
	k.readBitstream(codec, kHeader);
	posIKV.readBitstream(codec, posIKVHeader, ksFlag.nNumOfKeyValue);


	// decoder finish();
	if(codec._istrm->Inbuf.nbits != 0)
	{
		int rest = 8 - codec._istrm->Inbuf.nbits;
		codec._istrm->getbits(rest);
	}
	codec.finish_decode();
	istrm.bfree();



	//////////////////////////////////////////////////////////////////
	//
	//		Decoding Process
	//
	//////////////////////////////////////////////////////////////////
	// key header
	kHeader.decodingProcess();
	k.decodingProcess(kHeader);

	key.SetSize(kHeader.nNumberOfKey);
	for(int i = 0;i < kHeader.nNumberOfKey;i++)
		key[i] = k.dKey[i];
	keyValue.SetSize(kHeader.nNumberOfKey);


	posIKVHeader.decodingProcess();
	posIKV.decodingProcess(posIKVHeader, ksFlag, key, keyValue);


	//printf("Number of Decoded Bit : %d\n", AACTotal());
}

#if defined SAIT_CONFORMANCE_2003
void CompressedPositionInterpolator::setValues(unsigned char* pData)
{
	int* pData_NumberOfKeys = (int*) pData;
	float* pData_Keys = (float*) (pData + sizeof(int));
	int nNumberOfKeys = pData_NumberOfKeys[0];
	float* pData_KeyValues = (float*) (pData + sizeof(float)*nNumberOfKeys + sizeof(int));


	key.SetSize(nNumberOfKeys);
	keyValue.SetSize(nNumberOfKeys);

	for(int i=0,j=0; i<nNumberOfKeys; i++)
	{
		key[i] = pData_Keys[i];

		keyValue[i].x = pData_KeyValues[j++];
		keyValue[i].y = pData_KeyValues[j++];
		keyValue[i].z = pData_KeyValues[j++];
	}
}

void CompressedPositionInterpolator::write(char* fileName, int defID)
#else
void CompressedPositionInterpolator::write(char* fileName)
#endif SAIT_CONFORMANCE_2003
{
#if defined SAIT_CONFORMANCE_2003
	char fileName_new[100];
	if(defID != -1)
	{
		::sprintf(fileName_new, "%s_ID%d.dec", fileName, defID);
	}

	ofstream file;
	file.open(fileName_new);
#else
	ofstream file;
	file.open(fileName);
#endif SAIT_CONFORMANCE_2003

	if(!file.is_open())
	{
		cout << "Can not open the file" << endl;
		cout << "File Name: " << fileName << endl;
		return;
	}

	// 3. Write key & keyValue
	// 3.1. Write DEF Name, if available

#if defined SAIT_CONFORMANCE_2003
	if(defID != -1)
	{
		m_bIsDEF = true;
		::sprintf(m_strDEF, "ID%d", defID);
	}
#else
	m_bIsDEF = true;
	::strcpy(m_strDEF, "myPOSITION");
#endif SAIT_CONFORMANCE_2003

	if(m_bIsDEF)
		file << "DEF " << m_strDEF << ' ';

	// 3.2. PositionInterpolator {
	file << "PositionInterpolator {" << endl;

	// 3.3. key
	file << "\t" << "key [" << endl;


	for(int i=0; i<key.GetSize(); i++)
	{
		if(i == key.GetSize() - 1)
			file << "\t\t" << key[i] << endl;
		else
			file << "\t\t" << key[i] << ", " << endl;
	}
	file << "\t" << "]" << endl;

	// 4. keyValue
	file << "\t" << "keyValue [" << endl;

	for(i=0; i<keyValue.GetSize(); i++)
	{
		file << "\t\t";

		if(i == keyValue.GetSize() - 1)
			file << keyValue[i] << endl;
		else
			file << keyValue[i] << ", " << endl;
	}
	file << "\t" << "]" << endl;

	file << '}' << endl;

	file.close();
}



// 20020711 for BitWrapper
void CompressedPositionInterpolator::decodeWithBuffer(char* buffer, int nLength)
{
	// aac ʱȭ 
	CInStream istrm;
	SAIT_CAC::_shift = 16;
	CAC_QF codec(istrm);
	istrm.opendecWithBuffer(buffer, nLength);

	//////////////////////////////////////////////////////////////////
	//
	//		Header Information (KeyHeader, PosIKeyValueHeader)
	//
	//////////////////////////////////////////////////////////////////
	kHeader.readBitstream(codec);
	posIKVHeader.readBitstream(codec);

		//////////////////////////////////////////////////////////////
		m_nKeyDigit = kHeader.nKeyDigit;
		m_nKeyValueDigit = posIKVHeader.nKVDigit;
		//////////////////////////////////////////////////////////////

	// byte align
	if(codec._istrm->Inbuf.nbits != 0)
	{
		int rest = 8 - codec._istrm->Inbuf.nbits;
		codec._istrm->getbits(rest);
	}

	// qf_start();
	// decoder start();
	codec.start_decode();


	// key context initialize
	k.InitializeModel();
	// keyflag context initialize
	ksFlag.InitializeModel();
	// keyValue context initialize
	posIKV.InitializeModel();


	//////////////////////////////////////////////////////////////////
	//
	//		key & KeyValue
	//
	//////////////////////////////////////////////////////////////////
	ksFlag.readBitstream(codec, kHeader, posIKVHeader.bPreserveKey);
	k.readBitstream(codec, kHeader);
	posIKV.readBitstream(codec, posIKVHeader, ksFlag.nNumOfKeyValue);


	// decoder finish();
	if(codec._istrm->Inbuf.nbits != 0)
	{
		int rest = 8 - codec._istrm->Inbuf.nbits;
		codec._istrm->getbits(rest);
	}
	codec.finish_decode();
	istrm.bfree();



	//////////////////////////////////////////////////////////////////
	//
	//		Decoding Process
	//
	//////////////////////////////////////////////////////////////////
	// key header
	kHeader.decodingProcess();
	k.decodingProcess(kHeader);

	key.SetSize(kHeader.nNumberOfKey);
	for(int i = 0;i < kHeader.nNumberOfKey;i++)
		key[i] = k.dKey[i];
	keyValue.SetSize(kHeader.nNumberOfKey);


	posIKVHeader.decodingProcess();
	posIKV.decodingProcess(posIKVHeader, ksFlag, key, keyValue);


//	printf("Number of Decoded Bit : %d\n", AACTotal());
}
