// CCIEncoder.h: coordinate interpolator compression
// Author: Shinjun Lee, Samsung AIT
// Date: 2001-12-27

#include "../include/CKeyEncoder.h"
#include "../include/CKeyValueEncoder.h"
#include "../include/CAAC.h"
#include "../include/CMinMaxEncoder.h"
#include "../include/CDPCM.h"
#include "../include/CLinearKeyEncoder.h"

#define MACRO_FILE_OF_TOTAL_BITS "bits_total.txt"			// file name of total bits

#define AACTotal() (codec._ostrm->Outbuf.nbytes) * 8 + (codec._ostrm->Outbuf.nbits)

const int CMAX_DIGIT = 15;

class CCIEncoder
	{
  private:
	// variables: input data of key
	int				m_nQStep_key;							// QStep for key
	double*			m_a_dKey;								// data of key
	int				m_nNumberOfKey;							// the number of keys
	int				m_nDigit_key;							// key digit

	// variables: bitstream data of key
	int*			m_a_nQKey;								// data of quantized key
	int*			m_a_nKey_dnd_max;						// max value of quantized key
	int				m_nOrder_dpcm;							// order of DPCM
	int				m_nOrder_dnd;							// order of DND
	int				m_nKey_shift;							// value of shift
	int				m_nKey_shift_up;						// value of shift-up
	int				m_nQStep_key_shift;						// QStep for shift values
	int				m_nQStep_key_encoding;					// QStep for encoding key

	// variables: input data of keyValue
	int				m_nQStep_keyValue;						// QStep for keyValue
	double*			m_a_dKeyValue;							// data of keyValue
	int				m_nNumberOfCoord;						// the number of vertices
	int				m_nDigit_keyValue;						// keyValue digit

	// variables: data of transposition
	bool			m_bTranspose;							// flag of transposition
	int				m_nNumberOfKey_kv;						// the number of keys in the transposed keyValue data
	int				m_nNumberOfCoord_kv;					// the number of vertices in the transposed keyValue data

	// variables: bitstream data of keyValue
	CDPCM			m_oDPCM_X_Min;							// DPCM processor
	CDPCM			m_oDPCM_Y_Min;							// DPCM processor
	CDPCM			m_oDPCM_Z_Min;							// DPCM processor

	int				m_nX_QMin_Min;							// min of quantized min values
	int				m_nX_QMin_Max;							// max of quantized min values
	int				m_nY_QMin_Min;							// min of quantized min values
	int				m_nY_QMin_Max;							// max of quantized min values
	int				m_nZ_QMin_Min;							// min of quantized min values
	int				m_nZ_QMin_Max;							// max of quantized min values

	CDPCM			m_oDPCM_X_Max;							// DPCM processor
	CDPCM			m_oDPCM_Y_Max;							// DPCM processor
	CDPCM			m_oDPCM_Z_Max;							// DPCM processor

	int				m_nX_QMax_Min;							// min of quantized max values
	int				m_nX_QMax_Max;							// max of quantized max values
	int				m_nY_QMax_Min;							// min of quantized max values
	int				m_nY_QMax_Max;							// max of quantized max values
	int				m_nZ_QMax_Min;							// min of quantized max values
	int				m_nZ_QMax_Max;							// max of quantized max values

	// variables: bitstream data of keyValue
	int*			m_a_nQKeyValue;							// quantized value of keyValue
	double			m_dX_min;								// min of x
	double			m_dY_min;								// min of y
	double			m_dZ_min;								// min of z
	double			m_dX_max;								// max of x
	double			m_dY_max;								// max of y
	double			m_dZ_max;								// max of z
	double			m_dXYZ_max;								// max value which has max range among x, y, z
	int				m_nAxis_max;							// index of max value(0: x, 1: y, 2: z)

	int*			m_a_nDPCMMode_kv;						// DPCM mode for each vertex
	bool*			m_a_bQSelFlag_kv;						// quantization selection flag for each vertex of keyValue 
	int*			m_a_nAQStep_kv;							// acquired QStep for each vertex
	int*			m_a_nQMin_kv;							// quantized min values for each vertex to apply CQ
	int*			m_a_nQMax_kv;							// quantized max values for each vertex to apply CQ
	int*			m_a_nRefVertex_kv;						// reference vertex for each vertex

	// variables: bits of bitstream
	int				m_nTotalBits;							// total bits of coordinate interpolator

	// variables: data for linear key encoding
	bool			m_bIsLinearKeySubRegion;				// usage of the linear key encoding
	double			m_dLinearKeyMin;						// the minimum value of the linear key encoding
	double			m_dLinearKeyMax;						// the maximum value of the linear key encoding
	int				m_nNumberOfLinearKey;					// the number of key of the linear key encoding
	int				m_nNumberOfRemainingKey;				// the number of remaining key after linear key encoding

  public:
	CCIEncoder(void)
		{
		m_nQStep_key = 0;
		m_a_dKey = NULL;
		m_nNumberOfKey = 0;
		m_nDigit_key = 0;

		m_a_nQKey = NULL;
		m_a_nKey_dnd_max = NULL;
		m_nOrder_dpcm = 0;
		m_nOrder_dnd = 0;
		m_nKey_shift = 0;
		m_nKey_shift_up = 0;
		m_nQStep_key_shift = 0;
		m_nQStep_key_encoding = 0;

		m_nQStep_keyValue = 0;
		m_a_dKeyValue = NULL;
		m_nNumberOfCoord = 0;
		m_nDigit_keyValue = 0;

		m_bTranspose = 0;
		m_nNumberOfKey_kv = 0;
		m_nNumberOfCoord_kv = 0;

		m_nX_QMin_Min = 0;
		m_nX_QMin_Max = 0;
		m_nY_QMin_Min = 0;
		m_nY_QMin_Max = 0;
		m_nZ_QMin_Min = 0;
		m_nZ_QMin_Max = 0;

		m_nX_QMax_Min = 0;
		m_nX_QMax_Max = 0;
		m_nY_QMax_Min = 0;
		m_nY_QMax_Max = 0;
		m_nZ_QMax_Min = 0;
		m_nZ_QMax_Max = 0;

		m_a_nQKeyValue = NULL;
		m_dX_min = 0;
		m_dY_min = 0;
		m_dZ_min = 0;
		m_dX_max = 0;
		m_dY_max = 0;
		m_dZ_max = 0;
		m_dXYZ_max = 0;
		m_nAxis_max = 0;

		m_a_nDPCMMode_kv = NULL;
		m_a_bQSelFlag_kv = NULL;
		m_a_nAQStep_kv = NULL;
		m_a_nQMin_kv = NULL;
		m_a_nQMax_kv = NULL;
		m_a_nRefVertex_kv = NULL;

		m_nTotalBits = 0;

		m_bIsLinearKeySubRegion = false;
		m_dLinearKeyMin = 0;
		m_dLinearKeyMax = 0;
		m_nNumberOfLinearKey = 0;
		m_nNumberOfRemainingKey = 0;
		}

	~CCIEncoder()
		{
		// delete: bitstream data of keyValue
		delete[] m_a_nQKeyValue;
		delete[] m_a_nQMax_kv;
		delete[] m_a_nQMin_kv;
		delete[] m_a_nRefVertex_kv;
		delete[] m_a_nAQStep_kv;
		delete[] m_a_bQSelFlag_kv;
		delete[] m_a_nDPCMMode_kv;

		// delete: bitstream data of key
		delete[] m_a_nKey_dnd_max;
		delete[] m_a_nQKey;
		}

	// set: data of key
	void setKey(int nQStep_key, double* a_dKey, int nNumberOfKey, int nDigit_key)
		{
		m_nQStep_key = nQStep_key;
		m_a_dKey = a_dKey;
		m_nNumberOfKey = nNumberOfKey;
		m_nDigit_key = nDigit_key;
		}

	// set: data of keyValue
	void setKeyValue(int nQStep_keyValue, double* a_dKeyValue, int nNumberOfCoord, int nDigit_keyValue)
		{
		m_nQStep_keyValue = nQStep_keyValue;
		m_a_dKeyValue = a_dKeyValue;
		m_nNumberOfCoord = nNumberOfCoord;
		m_nDigit_keyValue = nDigit_keyValue;
		}

	// set: flag of transposition
	void setTransposition(bool bTranspose)
		{
		m_bTranspose = bTranspose;

		if(m_bTranspose == true)
			{
			m_nNumberOfKey_kv = m_nNumberOfCoord;
			m_nNumberOfCoord_kv = m_nNumberOfKey;
			transposeKeyValue(m_a_dKeyValue, m_a_dKeyValue, m_nNumberOfKey, m_nNumberOfCoord);
			}
		else
			{
			m_nNumberOfKey_kv = m_nNumberOfKey;
			m_nNumberOfCoord_kv = m_nNumberOfCoord;
			}
		}

	// encode: data of coordinate interpolator
	void encode(void)
		{
		encodeKey();
		encodeKeyValue();
		}

	// make: output bitstream file
	void makeBitstream(char* szBitstreamFile)
		{
		int			nBitsOfKeyHeader = 0;					// bits of key header
		int			nBitsOfAACKey = 0;						// bits of key in aac coding
		int			nBitsOfKeyValueHeader = 0;				// bits of keyValue header
		int			nBitsOfAACKeyValue = 0;					// bits of keyValue in aac coding
		int			i = 0;									// index

		// aac codec
		CAC::_shift = 0; 
		COutStream ostrm;
		CAC_QF codec(ostrm);
		CAACEncoder oAACEncoder;
		bool bSuccess = ostrm.openenc(szBitstreamFile);

		// make: bitstream of header
		makeBitstreamOfKeyHeader(codec, oAACEncoder, nBitsOfKeyHeader);
		makeBitstreamOfKeyValueHeader(codec, oAACEncoder, nBitsOfKeyValueHeader);

		// byte align
		int nRest = 8-codec._ostrm->Outbuf.nbits;
		for(i = 0; i < nRest; i++)
			codec._ostrm->putbit(1);

		// start: aac encoding
		codec.start_encode();
		CQF_Status::reset();

		// make: bitstream of aac data
		makeBitstreamOfAACKey(codec, oAACEncoder, nBitsOfAACKey);
		makeBitstreamOfAACKeyValue(codec, oAACEncoder, nBitsOfAACKeyValue);

		// finish: aac encoding
		codec.finish_encode();
		ostrm.close_enc();

		// print: bits of key coding
		cout << "Bits of key header = " << nBitsOfKeyHeader << endl;
		cout << "Bits of keys = " << nBitsOfAACKey << endl;
		cout << "Total key bits = " << nBitsOfKeyHeader + nBitsOfAACKey << endl;
		cout << endl;

		// print: bits of keyValue coding
		cout << "Bits of keyValue header = " << nBitsOfKeyValueHeader << endl;
		cout << "Bits of keyValues = " << nBitsOfAACKeyValue << endl;
		cout << "Total keyValue bits = " << nBitsOfKeyValueHeader + nBitsOfAACKeyValue << endl;
		cout << "-----------------------------------------------------------------------" << endl;
		cout << endl;

		// print: total bits of coordinate interpolator
		m_nTotalBits = (codec._ostrm->Outbuf.nbytes)*8 + (codec._ostrm->Outbuf.nbits);
		cout << "Total bits = " << m_nTotalBits << endl;
		cout << "-----------------------------------------------------------------------" << endl;

		// write: dump files
		//*//
		ofstream fout_bits;
		fout_bits.open("bits_k.txt", ios::app);
		fout_bits << nBitsOfKeyHeader + nBitsOfAACKey << endl;
		fout_bits.close();

		fout_bits.open("bits_kv.txt", ios::app);
		fout_bits << nBitsOfKeyValueHeader + nBitsOfAACKeyValue << endl;
		fout_bits.close();
		
		ofstream fout_q;
		fout_q.open("q_k.txt", ios::app);
		fout_q << m_nQStep_key << endl;
		fout_q.close();
		
		fout_q.open("q_kv.txt", ios::app);
		fout_q << m_nQStep_keyValue << endl;
		fout_q.close();
		
		fout_bits.open("bits_total.txt", ios::app);
		fout_bits << m_nTotalBits << endl;
		fout_bits.close();
		//*/

		ostrm.bfree();
		}

	// dump: each data of key, keyValue
	void dump(char* szInputCIFile)
		{
		int			j = 0;									// index

		// write: DPCM mode
		ofstream fout_dm;
		fout_dm.open("DPCMMode.txt");
		fout_dm << szInputCIFile << endl;
		for(j = 0; j < m_nNumberOfCoord_kv; j++)
			{
			fout_dm << j << "\t";
			fout_dm << m_a_nDPCMMode_kv[j*3] << "\t";
			fout_dm << m_a_nDPCMMode_kv[j*3+1] << "\t";
			fout_dm << m_a_nDPCMMode_kv[j*3+2] << endl;
			}
		fout_dm.close();

		// write: quantization selection flag
		ofstream fout_flag("QSelFlag_kv.txt");
		fout_flag << szInputCIFile << endl;
		for(j = 0; j < m_nNumberOfCoord_kv; j++)
			{
			fout_flag << j << "\t";
			fout_flag << m_a_bQSelFlag_kv[j*3] << "\t";
			fout_flag << m_a_bQSelFlag_kv[j*3+1] << "\t";
			fout_flag << m_a_bQSelFlag_kv[j*3+2] << endl;
			}
		fout_flag.close();

		// write: AQStep
		ofstream fout_aqstep("AQStep_kv.txt");
		fout_aqstep << szInputCIFile << endl;
		for(j = 0; j < m_nNumberOfCoord_kv; j++)
			{
			fout_aqstep << j << "\t";
			if(m_a_bQSelFlag_kv[j*3] == true)
				fout_aqstep << m_a_nAQStep_kv[j*3] << "\t";
			else
				fout_aqstep << 0 << "\t";
			if(m_a_bQSelFlag_kv[j*3+1] == true)
				fout_aqstep << m_a_nAQStep_kv[j*3+1] << "\t";
			else
				fout_aqstep << 0 << "\t";
			if(m_a_bQSelFlag_kv[j*3+2] == true)
				fout_aqstep << m_a_nAQStep_kv[j*3+2] << endl;
			else
				fout_aqstep << 0 << endl;
			}
		fout_aqstep.close();
		
		// write: reference vertex
		ofstream fout_base("RefVertex_kv.txt");
		fout_base << szInputCIFile << endl;
		for(j = 0; j < m_nNumberOfCoord_kv; j++)
			{
			fout_base << j << "\t";
			if(m_a_bQSelFlag_kv[j*3] == true)
				fout_base << m_a_nRefVertex_kv[j*3] << "\t";
			else
				fout_base << 0 << "\t";
			if(m_a_bQSelFlag_kv[j*3+1] == true)
				fout_base << m_a_nRefVertex_kv[j*3+1] << "\t";
			else
				fout_base << 0 << "\t";
			if(m_a_bQSelFlag_kv[j*3+2] == true)
				fout_base << m_a_nRefVertex_kv[j*3+2] << endl;
			else
				fout_base << 0 << endl;
			}
		fout_base << endl;
		fout_base.close();

		// write: qmin, qmax
		ofstream fout_qmm("QMinQMax.txt");
		fout_qmm << szInputCIFile << endl;
		for(j = 0; j < m_nNumberOfCoord_kv; j++)
			{
			fout_qmm << j << "\t";
			fout_qmm << m_a_nQMin_kv[j*3] << "\t";
			fout_qmm << m_a_nQMax_kv[j*3] << "\t";
			fout_qmm << m_a_nQMin_kv[j*3+1] << "\t";
			fout_qmm << m_a_nQMax_kv[j*3+1] << "\t";
			fout_qmm << m_a_nQMin_kv[j*3+2] << "\t";
			fout_qmm << m_a_nQMax_kv[j*3+2] << endl;
			}
		fout_qmm.close();

		// write: qmin, qmax after dpcm & cq
		fout_qmm.open("QMinQMax-aac.txt");
		fout_qmm << szInputCIFile << endl;
		for(j = 0; j < m_nNumberOfCoord_kv; j++)
			{
			fout_qmm << j << "\t";
			fout_qmm << m_oDPCM_X_Min.getData_dpcm(j) << "\t";
			fout_qmm << m_oDPCM_X_Max.getData_dpcm(j) << "\t";
			fout_qmm << m_oDPCM_Y_Min.getData_dpcm(j) << "\t";
			fout_qmm << m_oDPCM_Y_Max.getData_dpcm(j) << "\t";
			fout_qmm << m_oDPCM_Z_Min.getData_dpcm(j) << "\t";
			fout_qmm << m_oDPCM_Z_Max.getData_dpcm(j) << endl;
			}
		fout_qmm.close();
		}

	// Dump: real data File
	void dump_real(char* szString)
		{
		char szFile[_MAX_FNAME];				// File name
		int i = 0;
		int j = 0;

		ofstream fout;
		strcpy(szFile, szString);
		strcat(szFile, ".txt");
		fout.open(szFile);

		fout << szString << "(x)\t";
		for(i = 0; i < m_nNumberOfKey_kv; i++)
			fout << i << "\t";
		fout << endl;
		for(j = 0; j < m_nNumberOfCoord_kv; j++)
			{
			fout << j;
			if(m_a_bQSelFlag_kv[j*3] == true)
				{
				for(i = 0; i < m_nNumberOfKey_kv; i++)
 					fout << "\t" << m_a_dKeyValue[i*m_nNumberOfCoord_kv*3+j*3];
				}
			else
				{
				i = 0;
				fout << "\t" << m_a_dKeyValue[i*m_nNumberOfCoord_kv*3+j*3];
				}
			fout << endl;
			}
		fout << endl;

		fout << szString << "(y)\t";
		for(i = 0; i < m_nNumberOfKey_kv; i++)
			fout << i << "\t";
		fout << endl;
		for(j = 0; j < m_nNumberOfCoord_kv; j++)
			{
			fout << j;
			if(m_a_bQSelFlag_kv[j*3+1] == true)
				{
				for(i = 0; i < m_nNumberOfKey_kv; i++)
					fout << "\t" << m_a_dKeyValue[i*m_nNumberOfCoord_kv*3+j*3+1];
				}
			else
				{
				i = 0;
				fout << "\t" << m_a_dKeyValue[i*m_nNumberOfCoord_kv*3+j*3+1];
				}
			fout << endl;
			}
		fout << endl;

		fout << szString << "(z)\t";
		for(i = 0; i < m_nNumberOfKey_kv; i++)
			fout << i << "\t";
		fout << endl;
		for(j = 0; j < m_nNumberOfCoord_kv; j++)
			{
			fout << j;
			if(m_a_bQSelFlag_kv[j*3+2] == true)
				{
				for(i = 0; i < m_nNumberOfKey_kv; i++)
					fout << "\t" << m_a_dKeyValue[i*m_nNumberOfCoord_kv*3+j*3+2];
				}
			else
				{
				i = 0;
				fout << "\t" << m_a_dKeyValue[i*m_nNumberOfCoord_kv*3+j*3+2];
				}
			fout << endl;
			}
		fout.close();
		}

  private:
	// encode: data of key
	void encodeKey(void)
		{
		// 2002-04-17, Shinjun
		// variables: arrays for linear key encoding
		JWArray<double>		JWOriginalKeys;
		JWArray<double>		JWSelectedKeys;
		JWArray<double>		JWDecodedKey;

		// 2002-04-17, Shinjun
		// variables: data for linear key encoding
		int					preStartKeyIdx = -1;
		int					preEndKeyIdx = -1;
		bool				bIsKeyLinearCoder = true;
		bool				bPreserveKey = true;
		int					i = 0;

		// 2002-04-17, Shinjun
		// copy: the array of key to JWOriginalKeys
		JWDecodedKey.SetSize(m_nNumberOfKey);
		JWSelectedKeys.SetSize(m_nNumberOfKey);
		for(i = 0; i < m_nNumberOfKey; i++)
			JWOriginalKeys.Add(m_a_dKey[i]);

		// 2002-04-17, Shinjun
		// Linear key encoder
		CLinearKeyEncoder oLinearKeyEncoder;
		oLinearKeyEncoder.LinearKeySubRegionExtractor(m_nQStep_key, m_nDigit_key,
			bIsKeyLinearCoder, bPreserveKey,
			preStartKeyIdx, preEndKeyIdx,
			JWOriginalKeys, JWSelectedKeys);
		oLinearKeyEncoder.KeySplitter(bPreserveKey, preStartKeyIdx, preEndKeyIdx, m_nDigit_key,
			JWOriginalKeys, JWSelectedKeys, JWDecodedKey,
			m_dLinearKeyMin, m_dLinearKeyMax, m_nNumberOfLinearKey);

		// 2002-04-17, Shinjun
		if(preStartKeyIdx != -1 || preEndKeyIdx != -1)
			{
			m_bIsLinearKeySubRegion = true;
			m_nNumberOfRemainingKey = m_nNumberOfKey - m_nNumberOfLinearKey;
			}
		else
			{
			m_bIsLinearKeySubRegion = false;
			m_nNumberOfRemainingKey = m_nNumberOfKey;
			}

		// 2002-04-17, Shinjun
		// copy: JWOriginalKeys to the array of key
		for(i = 0; i < m_nNumberOfRemainingKey; i++)
			m_a_dKey[i] = JWOriginalKeys[i];

		// Key encoder
		CKeyEncoder oKeyEncoder;
		oKeyEncoder.setKeyDigit(m_nDigit_key);
		oKeyEncoder.encode(m_a_dKey, m_nNumberOfRemainingKey, m_nQStep_key);

		// get: result data from key encoding
		oKeyEncoder.getQKey(m_a_nQKey);
		oKeyEncoder.getKey_dnd_max(m_a_nKey_dnd_max);
		m_nOrder_dpcm = oKeyEncoder.getOrder_dpcm();
		m_nOrder_dnd = oKeyEncoder.getOrder_dnd();
		m_nKey_shift = oKeyEncoder.getKey_shift();
		m_nKey_shift_up = oKeyEncoder.getKey_shift_up();
		m_nQStep_key_shift = oKeyEncoder.getQStep_key_shift();
		m_nQStep_key_encoding = oKeyEncoder.getQStep_key_encoding();

		// print: result data from key encoding
		cout << "Number of keys = " << m_nNumberOfKey << endl;
		cout << "QStep_key = " << m_nQStep_key << endl;
		cout << "Key digit = " << m_nDigit_key << endl;
		cout << "Key Distortion = " << oKeyEncoder.getKeyDistortion() << endl;
		cout << "Order_dpcm = " << m_nOrder_dpcm << endl;
		cout << "Order_dnd = " << m_nOrder_dnd << endl;
		cout << "Key to shift = " << m_nKey_shift << endl;
		cout << "Key to shift-up = " << m_nKey_shift_up << endl;
		cout << "QStep(key_shift) = " << m_nQStep_key_shift << endl;
		cout << "QStep(key_encoding) = " << m_nQStep_key_encoding << endl;
		cout << endl;
		cout << "dKeyMin = " << m_dLinearKeyMin << " (index: " << preStartKeyIdx << ")" << endl;
		cout << "dKeyMax = " << m_dLinearKeyMax << " (index: " << preEndKeyIdx << ")" << endl;
		cout << "nNumberOfLinearKey = " << m_nNumberOfLinearKey << endl;
		cout << endl;
		}

	// make: bitstream of key header
	void makeBitstreamOfKeyHeader(CAC_QF& codec, CAACEncoder& oAACEncoder, int& nTotalBitsOfKey)
		{
		// variables: temporal data
		int			nQStep_qstep = 0;						// QStep for QStep
		int			nQStep = 0;								// temporal QStep
		int			i = 0;									// index

		ofstream fout("fout-encoding.txt", ios::app);
		fout << "[Start: KeyHeader]" << endl;

		// put: m_nQStep_key
		codec._ostrm->putbits(m_nQStep_key, 5);
		fout << "nQStep_Key: " << m_nQStep_key << endl;
		nTotalBitsOfKey += 5;

		// get: nQStep_qstep
		nQStep_qstep = getNBits(m_nQStep_key);

		// put: the number of keys
		nQStep = getNBits(m_nNumberOfKey);
		codec._ostrm->putbits(nQStep, 5);
		fout << "nQStep_Num: " << nQStep << endl;
		nTotalBitsOfKey += 5;
		codec._ostrm->putbits(m_nNumberOfKey, nQStep);
		fout << "nNumberOfKey: " << m_nNumberOfKey << endl;
		nTotalBitsOfKey += nQStep;

		// put: key digit
		codec._ostrm->putbits(m_nDigit_key, 4);
		fout << "nKey_Digit: " << m_nDigit_key << endl;
		nTotalBitsOfKey += 4;

		// 2002-04-17, Shinjun
		// put: linear key
		if(m_bIsLinearKeySubRegion == true)
			{
			// subset Ѵ.
			int	nIsColinear = 1;
			codec._ostrm->putbits(nIsColinear, 1);
			nTotalBitsOfKey += 1;

			makeBitstreamOfLinearKey(codec, m_dLinearKeyMin, m_dLinearKeyMax, m_nNumberOfLinearKey, m_nDigit_key, nTotalBitsOfKey);
			}
		else
			{
			// subset  ʴ´.
			int nIsColinear = 0;
			codec._ostrm->putbits(nIsColinear, 1);
			nTotalBitsOfKey += 1;
			}

		// put: m_a_dKey[0], m_a_dKey[m_nNumberOfKey-1]
		CMinMaxEncoder oMinMaxEncoder;
		if(m_a_dKey[0] < 0 || m_a_dKey[m_nNumberOfKey-1] > 1)
			{
			codec._ostrm->putbits(1, 1);
			fout << "bRangeFlag: " << 1 << endl;
			nTotalBitsOfKey += 1;
			oMinMaxEncoder.MakeBitstreamOfMinMax(codec, m_a_dKey[0], m_a_dKey[m_nNumberOfKey-1], m_nDigit_key, nTotalBitsOfKey);
			}
		else
			{
			codec._ostrm->putbits(0, 1);
			fout << "bRangeFlag: " << 0 << endl;
			nTotalBitsOfKey += 1;
			}

		// put: m_nQStep_key_shift
		codec._ostrm->putbits(m_nQStep_key_shift, 5);
		fout << "nQStep_Shift: " << m_nQStep_key_shift << endl;
		nTotalBitsOfKey += 5;

		// 2002-04-18-02, Shinjun
		// put: m_nOrder_dpcm
		if(m_nOrder_dpcm == 0)
			{
			m_a_nQKey = new int[1];
			m_a_nQKey[0] = 0;
			m_nOrder_dpcm++;
			}
		codec._ostrm->putbits(m_nOrder_dpcm-1, 2);
		fout << "nOrder_DPCM: " << m_nOrder_dpcm << endl;
		nTotalBitsOfKey += 2;

		// put: intra key data
		for(i = 0; i < m_nOrder_dpcm; i++)
			{
			if(i == 0 && m_a_nQKey[i] == 0)
				{
				codec._ostrm->putbits(1, 1);
				fout << "nSign_nQKey: " << 1 << endl;
				nTotalBitsOfKey += 1;
				continue;
				}
			if(m_a_nQKey[i] >= 0)
				{
				codec._ostrm->putbits(0, 1);
				fout << "nSign_nQKey: " << 0 << endl;
				}
			else
				{
				codec._ostrm->putbits(1, 1);
				fout << "nSign_nQKey: " << 1 << endl;
				}
			nTotalBitsOfKey += 1;
			codec._ostrm->putbits(abs(m_a_nQKey[i]), m_nQStep_key_shift);
			fout << "a_nQIntraKey: " << abs(m_a_nQKey[i]) << endl;
			nTotalBitsOfKey += m_nQStep_key_shift;
			}

		// put: shift key data
		if(m_nKey_shift != 0)
			{
			codec._ostrm->putbits(1, 1);
			fout << "nShift: " << 1 << endl;
			nTotalBitsOfKey += 1;
			if(m_nKey_shift >= 0)
				{
				codec._ostrm->putbits(0, 1);
				fout << "nSign_Shift: " << 0 << endl;
				}
			else
				{
				codec._ostrm->putbits(1, 1);
				fout << "nSign_Shift: " << 1 << endl;
				}
			nTotalBitsOfKey += 1;
			codec._ostrm->putbits(abs(m_nKey_shift), m_nQStep_key_shift);
			fout << "nKey_Shift: " << abs(m_nKey_shift) << endl;
			nTotalBitsOfKey += m_nQStep_key_shift;
			}
		else
			{
			codec._ostrm->putbits(0, 1);
			fout << "nShift: " << 0 << endl;
			nTotalBitsOfKey += 1;
			}

		// put: m_nOrder_dnd
		if(m_nOrder_dnd != -1)
			{
			codec._ostrm->putbits(m_nOrder_dnd, 3);
			fout << "unOrder_DND: " << m_nOrder_dnd << endl;
			nTotalBitsOfKey += 3;
			if(m_nOrder_dnd == 7)			// if 7, sign = 0(positive)
				{
				codec._ostrm->putbits(0, 1);
				fout << "nSign_Order: " << 0 << endl;
				nTotalBitsOfKey += 1;
				}
			}
		else
			{
			codec._ostrm->putbits(7, 3);	// if -1, put 7(binary: 111)
			fout << "unOrder_DND: " << 7 << endl;
			nTotalBitsOfKey += 3;
			codec._ostrm->putbits(1, 1);	// sign = 1(negative)
			fout << "nSign_Order: " << 1 << endl;
			nTotalBitsOfKey += 1;
			}

		// put: m_a_nKey_dnd_max
		nQStep = m_nQStep_key_shift;
		for(i = 0; i < m_nOrder_dnd; i++)
			{
			if(m_a_nKey_dnd_max[i] >= 0)
				{
				codec._ostrm->putbits(0, 1);
				fout << "nSign_Max: " << 0 << endl;
				}
			else
				{
				codec._ostrm->putbits(1, 1);
				fout << "nSign_Max: " << 1 << endl;
				}
			nTotalBitsOfKey += 1;
			codec._ostrm->putbits(abs(m_a_nKey_dnd_max[i]), nQStep);
			fout << "a_nKey_Max: " << abs(m_a_nKey_dnd_max[i]) << endl;
			nTotalBitsOfKey += nQStep;

			nQStep = getNBits(abs(m_a_nKey_dnd_max[i]));
			if(nQStep+1 < m_nQStep_key_shift)
				nQStep += 1;
			else
				nQStep = m_nQStep_key_shift;
			}

		// put: m_nQStep_key_encoding
		codec._ostrm->putbits(m_nQStep_key_encoding, nQStep_qstep);
		fout << "nQStep_Dec: " << m_nQStep_key_encoding << endl;
		nTotalBitsOfKey += nQStep_qstep;

		// put: nKey_shift_up
		if(m_nOrder_dnd != -1 && m_nOrder_dnd != 0)
			{
			if(m_nKey_shift_up != -1)
				{
				codec._ostrm->putbits(1, 1);
				fout << "nShift_Down: " << 1 << endl;
				nTotalBitsOfKey += 1;

				codec._ostrm->putbits(m_nKey_shift_up, m_nQStep_key_encoding);
				fout << "nKey_Shift_Down: " << m_nKey_shift_up << endl;
				nTotalBitsOfKey += m_nQStep_key_encoding;
				oAACEncoder.setSign(false);
				}
			else
				{
				codec._ostrm->putbits(0, 1);
				fout << "nShift_Down: " << 0 << endl;
				nTotalBitsOfKey += 1;
				oAACEncoder.setSign(true);
				}
			}
		else
			oAACEncoder.setSign(false);

		fout << "[End: KeyHeader]" << endl;
		fout << endl;
		fout.close();
		}

	// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
	//
	// 2002-04-17, Shinjun
	// make: Bitstream Of Linear Key
	// Linear Key Encoder ã dKeyMin, dKeyMax, nNumberOfLinearKey ȣȭѴ.
	// Author: James Kim
	// Editor: Shinjun Lee
	//
	// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
	int makeBitstreamOfLinearKey(CAC_QF& codec,
		double dKeyMin,
		double dKeyMax,
		int nNumberOfLinearKey,
		int nKeyDigit,
		int& KTotal)
		{
		int	prevBitTotal = AACTotal();
		int	KHeaderTotal = 0;

		// key   key  coding bit Ѵ.
		int numBitofOriginalKey = getNbBits(nNumberOfLinearKey);
		codec._ostrm->putbits(numBitofOriginalKey, 5);
		KTotal += 5;

		codec._ostrm->putbits(nNumberOfLinearKey, numBitofOriginalKey);
		KTotal += numBitofOriginalKey;
		
		// Start key
		// mantissa digit 4 
		int nDigit =  GetDigit(dKeyMin);
		if(nDigit == nKeyDigit)
			{
			codec._ostrm->putbits(1, 1);
			KTotal += 1;
			}
		else
			{
			codec._ostrm->putbits(0, 1);
			//codec._ostrm->putbits(nDigit, 3);
			// jameskim 20011222
			codec._ostrm->putbits(nDigit, 4);
			KTotal += 4;
			}
		
		int mantissa;
		int preexponent;
		int count;
		
		if(nDigit != 0)
			{
			if(nDigit < 8)
				{
				// mantissa 10 
				mantissa = GetMantissa(dKeyMin, nDigit);
				count = getNbBits((int)pow(10.0, nDigit)-1);
				
				if(dKeyMin >= 0)
					codec._ostrm->putbits(0, 1);
				else
					codec._ostrm->putbits(1, 1);
				KTotal += 1;
				
				codec._ostrm->putbits(mantissa, count);
				KTotal += count;
				
				// exponent digit 6 
				preexponent = GetLargestDigit(dKeyMin);
				
				if(preexponent >= 0)
					codec._ostrm->putbits(0, 1);
				else
					codec._ostrm->putbits(1, 1);
				KTotal += 1;
				
				codec._ostrm->putbits(abs(preexponent), 6);
				KTotal += 6;
				}
			else
				{
				float flot = (float)dKeyMin;
				codec._ostrm->putbits(*((int *)&flot), 32);
				KTotal += 32;
				}
			}
		
		// End key
		// mantissa digit 4
		nDigit =  GetDigit(dKeyMax);
		if(nDigit == nKeyDigit)
			{
			codec._ostrm->putbits(1, 1);
			KTotal += 1;
			}
		else
			{
			codec._ostrm->putbits(0, 1);
			KTotal += 1;
			//codec._ostrm->putbits(nDigit, 3);
			// jameskim 20011222
			codec._ostrm->putbits(nDigit, 4);
			KTotal += 4;
			}							
		
		if(nDigit != 0)
			{
			if(nDigit < 8)
				{
				// mantissa 10 
				mantissa = GetMantissa(dKeyMax, nDigit);
				count = getNbBits((int)pow(10.0, nDigit)-1);
				
				if(dKeyMax >= 0)
					codec._ostrm->putbits(0, 1);
				else
					codec._ostrm->putbits(1, 1);
				KTotal += 1;
				
				codec._ostrm->putbits(mantissa, count);
				KTotal += count;
				
				// exponent 6 
				int exponent = GetLargestDigit(dKeyMax);
				
				if(exponent == preexponent)
					{
					codec._ostrm->putbits(1, 1);
					KTotal += 1;
					}
				else
					{
					codec._ostrm->putbits(0, 1);
					KTotal += 1;
					
					if(exponent >= 0)
						codec._ostrm->putbits(0, 1);
					else
						codec._ostrm->putbits(1, 1);
					KTotal += 1;
					
					codec._ostrm->putbits(abs(exponent), 6);
					KTotal += 6;
					}
				}
			else
				{
				const float flot = (float)dKeyMax;
				codec._ostrm->putbits(*((int *)&flot), 32);
				KTotal += 32;
				}
			}							

		return KHeaderTotal = AACTotal() - prevBitTotal;
		}

	// make: bitstream of key data
	void makeBitstreamOfAACKey(CAC_QF& codec, CAACEncoder& oAACEncoder, int& nTotalBitsOfKey)
		{
		int nBitsOfStart = 0;
		int nBitsOfEnd = 0;
		int	i = 0; // index

		nBitsOfStart = (codec._ostrm->Outbuf.nbytes-1)*8 + (codec._ostrm->Outbuf.nbits-1);
		for(i = m_nOrder_dpcm; i < m_nNumberOfRemainingKey; i++)
			{
			int nQKey = m_a_nQKey[i];	// bitstream: AAC Key encoding
			oAACEncoder.encodeKey(codec, m_nQStep_key_encoding, nQKey);
			}
		nBitsOfEnd = (codec._ostrm->Outbuf.nbytes-1)*8 + (codec._ostrm->Outbuf.nbits-1);
		nTotalBitsOfKey += nBitsOfEnd - nBitsOfStart;
		}

	// encode: data of keyValue
	void encodeKeyValue(void)
		{
		CKeyValueEncoder oKeyValueEncoder;
		oKeyValueEncoder.setKeyValueDigit(m_nDigit_keyValue);
		oKeyValueEncoder.encode(m_a_dKeyValue, m_nNumberOfKey_kv, m_nNumberOfCoord_kv, m_nQStep_keyValue);
		oKeyValueEncoder.getQKeyValue(m_a_nQKeyValue);
		oKeyValueEncoder.getMaxMin(m_dX_min, m_dY_min, m_dZ_min, m_dX_max, m_dY_max, m_dZ_max, m_nAxis_max);
		oKeyValueEncoder.getQFlag_kv(m_a_bQSelFlag_kv);
		oKeyValueEncoder.getTSMode(m_a_nDPCMMode_kv);
		oKeyValueEncoder.getQMinQMax(m_a_nQMin_kv, m_a_nQMax_kv);
		oKeyValueEncoder.getBase(m_a_nRefVertex_kv);
		oKeyValueEncoder.getQStep_kv(m_a_nAQStep_kv, m_a_nRefVertex_kv);
		// ------------------------------------------------------------------------------------ //

		m_oDPCM_X_Min.setNumber(m_nNumberOfCoord_kv);
		m_oDPCM_Y_Min.setNumber(m_nNumberOfCoord_kv);
		m_oDPCM_Z_Min.setNumber(m_nNumberOfCoord_kv);

		m_oDPCM_X_Min.setData(m_a_nQMin_kv);
		m_oDPCM_Y_Min.setData(m_a_nQMin_kv);
		m_oDPCM_Z_Min.setData(m_a_nQMin_kv);

		m_oDPCM_X_Min.setComponent(0);
		m_oDPCM_Y_Min.setComponent(1);
		m_oDPCM_Z_Min.setComponent(2);

		m_oDPCM_X_Min.dpcm();
		m_oDPCM_Y_Min.dpcm();
		m_oDPCM_Z_Min.dpcm();

		m_nX_QMin_Min = m_oDPCM_X_Min.getMin();
		m_nX_QMin_Max = m_oDPCM_X_Min.getMax();

		m_nY_QMin_Min = m_oDPCM_Y_Min.getMin();
		m_nY_QMin_Max = m_oDPCM_Y_Min.getMax();

		m_nZ_QMin_Min = m_oDPCM_Z_Min.getMin();
		m_nZ_QMin_Max = m_oDPCM_Z_Min.getMax();

		m_oDPCM_X_Max.setNumber(m_nNumberOfCoord_kv);
		m_oDPCM_Y_Max.setNumber(m_nNumberOfCoord_kv);
		m_oDPCM_Z_Max.setNumber(m_nNumberOfCoord_kv);

		m_oDPCM_X_Max.setData(m_a_nQMax_kv);
		m_oDPCM_Y_Max.setData(m_a_nQMax_kv);
		m_oDPCM_Z_Max.setData(m_a_nQMax_kv);

		m_oDPCM_X_Max.setBase(m_a_nQMin_kv);
		m_oDPCM_Y_Max.setBase(m_a_nQMin_kv);
		m_oDPCM_Z_Max.setBase(m_a_nQMin_kv);

		m_oDPCM_X_Max.setComponent(0);
		m_oDPCM_Y_Max.setComponent(1);
		m_oDPCM_Z_Max.setComponent(2);

		m_oDPCM_X_Max.dpcm();
		m_oDPCM_Y_Max.dpcm();
		m_oDPCM_Z_Max.dpcm();

		m_nX_QMax_Min = m_oDPCM_X_Max.getMin();
		m_nX_QMax_Max = m_oDPCM_X_Max.getMax();

		m_nY_QMax_Min = m_oDPCM_Y_Max.getMin();
		m_nY_QMax_Max = m_oDPCM_Y_Max.getMax();

		m_nZ_QMax_Min = m_oDPCM_Z_Max.getMin();
		m_nZ_QMax_Max = m_oDPCM_Z_Max.getMax();

		// print: result data from keyValue encoding
		cout << "Number of coords = " << m_nNumberOfCoord << endl;
		cout << "QStep_keyValue = " << m_nQStep_keyValue << endl;
		cout << "KeyValue digit = " << m_nDigit_keyValue << endl;
		cout << "X_min ~ X_max = " << m_dX_min << " ~ " << m_dX_max << endl;
		cout << "Y_min ~ Y_max = " << m_dY_min << " ~ " << m_dY_max << endl;
		cout << "Z_min ~ Z_max = " << m_dZ_min << " ~ " << m_dZ_max << endl;
		cout << endl;
		}

	// make: bitstream of keyValue header
	void makeBitstreamOfKeyValueHeader(CAC_QF& codec, CAACEncoder& oAACEncoder, int& nBitsOfKeyValueHeader)
		{
		// variables: temporal data
		int			nQStep = 0;								// temporal QStep
		int			i = 0;									// index

		ofstream fout("fout-encoding.txt", ios::app);
		fout << "[Start: KeyValueHeader]" << endl;

		// put: flag of transposition
		if(m_bTranspose == 0)
			{
			codec._ostrm->putbits(0, 1);
			fout << "bTranspose: " << 0 << endl;
			}
		else
			{
			codec._ostrm->putbits(1, 1);
			fout << "bTranspose: " << 1 << endl;
			}
		nBitsOfKeyValueHeader += 1;

		// put: m_nQStep_keyValue
		codec._ostrm->putbits(m_nQStep_keyValue, 5);
		fout << "nQStep_KeyValue: " << m_nQStep_keyValue << endl;
		nBitsOfKeyValueHeader += 5;

		// put: the number of vertices
		nQStep = getNBits(m_nNumberOfCoord);
		codec._ostrm->putbits(nQStep, 5);
		fout << "nQStep_Coord: " << nQStep << endl;
		nBitsOfKeyValueHeader += 5;
		codec._ostrm->putbits(m_nNumberOfCoord, nQStep);
		fout << "nNumberOfCoord: " << m_nNumberOfCoord << endl;
		nBitsOfKeyValueHeader += nQStep;

		// put: keyValue digit
		codec._ostrm->putbits(m_nDigit_keyValue, 4);
		fout << "nKeyValueDigit: " << m_nDigit_keyValue << endl;
		nBitsOfKeyValueHeader += 4;

		// put: keyValue max, min
		CMinMaxEncoder oMinMaxEncoder;
		if(m_nAxis_max == 0)
			oMinMaxEncoder.EncodeReducedMinMaxData(codec, m_dX_min, m_dY_min, m_dZ_min, m_dX_max, m_nDigit_keyValue, m_nAxis_max, nBitsOfKeyValueHeader);
		else if(m_nAxis_max == 1)
			oMinMaxEncoder.EncodeReducedMinMaxData(codec, m_dX_min, m_dY_min, m_dZ_min, m_dY_max, m_nDigit_keyValue, m_nAxis_max, nBitsOfKeyValueHeader);
		else if(m_nAxis_max == 2)
			oMinMaxEncoder.EncodeReducedMinMaxData(codec, m_dX_min, m_dY_min, m_dZ_min, m_dZ_max, m_nDigit_keyValue, m_nAxis_max, nBitsOfKeyValueHeader);

		// put: min, max of quantized min values for each component
		codec._ostrm->putbits(m_nX_QMin_Min, m_nQStep_keyValue);
		fout << "nX_QMin_min: " << m_nX_QMin_Min << endl;
		codec._ostrm->putbits(m_nX_QMin_Max, m_nQStep_keyValue);
		fout << "nX_QMin_max: " << m_nX_QMin_Max << endl;

		codec._ostrm->putbits(m_nY_QMin_Min, m_nQStep_keyValue);
		fout << "nY_QMin_min: " << m_nY_QMin_Min << endl;
		codec._ostrm->putbits(m_nY_QMin_Max, m_nQStep_keyValue);
		fout << "nY_QMin_max: " << m_nY_QMin_Max << endl;

		codec._ostrm->putbits(m_nZ_QMin_Min, m_nQStep_keyValue);
		fout << "nZ_QMin_min: " << m_nZ_QMin_Min << endl;
		codec._ostrm->putbits(m_nZ_QMin_Max, m_nQStep_keyValue);
		fout << "nZ_QMin_max: " << m_nZ_QMin_Max << endl;

		// put: min, max of quantized max values for each component
		codec._ostrm->putbits(m_nX_QMax_Min, m_nQStep_keyValue);
		fout << "nX_QMax_min: " << m_nX_QMax_Min << endl;
		codec._ostrm->putbits(m_nX_QMax_Max, m_nQStep_keyValue);
		fout << "nX_QMax_max: " << m_nX_QMax_Max << endl;

		codec._ostrm->putbits(m_nY_QMax_Min, m_nQStep_keyValue);
		fout << "nY_QMax_min: " << m_nY_QMax_Min << endl;
		codec._ostrm->putbits(m_nY_QMax_Max, m_nQStep_keyValue);
		fout << "nY_QMax_max: " << m_nY_QMax_Max << endl;

		codec._ostrm->putbits(m_nZ_QMax_Min, m_nQStep_keyValue);
		fout << "nZ_QMax_min: " << m_nZ_QMax_Min << endl;
		codec._ostrm->putbits(m_nZ_QMax_Max, m_nQStep_keyValue);
		fout << "nZ_QMax_max: " << m_nZ_QMax_Max << endl;

		fout << "[End: KeyValueHeader]" << endl;
		fout << endl;
		fout.close();
		}

	// make: bitstream of keyValue
	void makeBitstreamOfAACKeyValue(CAC_QF& codec, CAACEncoder& oAACEncoder, int& nBitsOfAACKeyValue)
		{
		int			j = 0;									// index
		int			c = 0;									// index

		int nBitsOfStart = (codec._ostrm->Outbuf.nbytes-1)*8 + (codec._ostrm->Outbuf.nbits-1);
		makeBitstreamOfDPCMMode(codec, oAACEncoder);
		for(j = 0; j < m_nNumberOfCoord_kv; j++)
			{
			for(c = 0; c < 3; c++)
				{
				/*//
				makeBitstreamOfQSelFlag(codec, oAACEncoder, j, c);
				makeBitstreamOfAQStep(codec, oAACEncoder, j, c);
				makeBitstreamOfRefVertex(codec, oAACEncoder, j, c);
				makeBitstreamOfQMin(codec, oAACEncoder, j, c);
				makeBitstreamOfQMax(codec, oAACEncoder, j, c);
				makeBitstreamOfKeyValue(codec, oAACEncoder, j, c);
				//*/
				}
			}

		int nBitsOfEnd = (codec._ostrm->Outbuf.nbytes-1)*8 + (codec._ostrm->Outbuf.nbits-1);
		nBitsOfAACKeyValue = nBitsOfEnd - nBitsOfStart;
		}

	// make: bitstream of DPCM mode
	void makeBitstreamOfDPCMMode(CAC_QF& codec, CAACEncoder& oAACEncoder)
		{
		const int	nSizeOfAddress = 27;					// Size of address
		bool		a_bAddress_DPCMMode[nSizeOfAddress] = {false};		// Dictionary address table
		bool*		a_bIndex_DPCMMode = NULL;				// Dictionary index table
		int			nIndex_address = 0;						// Address indexs
		int			nNumberOfSymbol = 0;					// the number of symbols
		int			i = 0;									// index
		int			j = 0;									// index
		int			k = 0;									// index

		// initialize: a_bAddress_DPCMMode
		memset((void*)a_bAddress_DPCMMode, 0, sizeof(bool)*nSizeOfAddress);

		//ofstream fout_dm;
		//fout_dm.open("DPCMMode_kv.txt");

		// initialize: address
		for(j = 1; j < m_nNumberOfCoord_kv; j++)
			{
			int nIndex = 0;

			nIndex = 9*(m_a_nDPCMMode_kv[j*3+2]-1)+3*(m_a_nDPCMMode_kv[j*3+1]-1)+(m_a_nDPCMMode_kv[j*3]-1);
			a_bAddress_DPCMMode[nIndex] = true;
			}

		// send: address
		for(i = 0; i < nSizeOfAddress; i++)
			{
			if(a_bAddress_DPCMMode[i] == true)
				{
				codec.encode(1, CQF_Status::ci_kv_DPCMMode_dic_address_context);
				nNumberOfSymbol++;
				}
			else
				codec.encode(0, CQF_Status::ci_kv_DPCMMode_dic_address_context);
			}

		// initialize: nIndex_address
		nIndex_address = 0;

		// allocate: a_bIndex_DPCMMode
		a_bIndex_DPCMMode = new bool[m_nNumberOfCoord_kv];
		memset((void*)a_bIndex_DPCMMode, 0, sizeof(bool)*m_nNumberOfCoord_kv);
		for(int s = 0; s < nNumberOfSymbol; s++)			// repeat as the number of symbols
			{
			int nQ = 0;										// symbol
			int nValue = 0;									// comparing value

			while(a_bAddress_DPCMMode[nIndex_address] != true)
				nIndex_address++;

			nQ = nIndex_address;

			//fout_dm << nQ << ": ";

			// if last symbol, break out
			if(s == nNumberOfSymbol - 1)
				break;

			// increment: nIndex_address
			nIndex_address++;

			// set: one, zero
			int	nOne = 0;					// Flag: one value flag
			int	nZero = 1;					// Flag: zero value flag

			// a_bDic_index: index position coding
			for(k = 1; k < m_nNumberOfCoord_kv; k++)
				{
				if(a_bIndex_DPCMMode[k] == 0)
					{
					nValue = 9*(m_a_nDPCMMode_kv[k*3+2]-1)+3*(m_a_nDPCMMode_kv[k*3+1]-1)+(m_a_nDPCMMode_kv[k*3]-1);
					if(nQ == nValue)
						{
						a_bIndex_DPCMMode[k] = 1;
						codec.encode(nOne, CQF_Status::ci_kv_DPCMMode_dic_index_context);
						//fout_dm << nOne << flush;
						}
					else
						{
						codec.encode(nZero, CQF_Status::ci_kv_DPCMMode_dic_index_context);
						//fout_dm << nZero << flush;
						}
					}
				}
			//fout_dm << endl;
			}
		//fout_dm.close();
		delete[] a_bIndex_DPCMMode;
		a_bIndex_DPCMMode = NULL;
		}

	// make: bitstream of DPCM mode
	void makeBitstreamOfQSelFlag(CAC_QF& codec, CAACEncoder& oAACEncoder, int nCoord, int nComponent)
		{
		switch(nComponent)
			{
			case 0:
				if(m_nX_QMax_Min <= m_nX_QMin_Max)
					codec.encode(m_a_bQSelFlag_kv[nCoord*3+nComponent], CQF_Status::ci_kv_Selection_flag_context);
				break;

			case 1:
				if(m_nY_QMax_Min <= m_nY_QMin_Max)
					codec.encode(m_a_bQSelFlag_kv[nCoord*3+nComponent], CQF_Status::ci_kv_Selection_flag_context);
				break;

			case 2:
				if(m_nZ_QMax_Min <= m_nZ_QMin_Max)
					codec.encode(m_a_bQSelFlag_kv[nCoord*3+nComponent], CQF_Status::ci_kv_Selection_flag_context);
				break;
			}
		}

	// make: bitstream of acquired QStep
	void makeBitstreamOfAQStep(CAC_QF& codec, CAACEncoder& oAACEncoder, int nCoord, int nComponent)
		{
		int			nQStep_qstep_kv = 0;					// QStep for QStep

		// get: nQStep_qstep_kv
		nQStep_qstep_kv = getNBits(m_nQStep_keyValue);

		if(m_a_bQSelFlag_kv[nCoord*3+nComponent] == true)
			oAACEncoder.encodeQStep(codec, nQStep_qstep_kv, m_a_nAQStep_kv[nCoord*3+nComponent], nComponent);
		}

	// make: bitstream of reference vertex
	void makeBitstreamOfRefVertex(CAC_QF& codec, CAACEncoder& oAACEncoder, int nCoord, int nComponent)
		{
		if(nCoord > 0)
			{
			if(m_a_bQSelFlag_kv[nCoord*3+nComponent] == true && m_a_nDPCMMode_kv[nCoord*3+nComponent] != DPCM_T)
				{
				int nQStep = getNBits(nCoord-1);
				oAACEncoder.encodeBaseVertexIndex(codec, nQStep, m_a_nRefVertex_kv[nCoord*3+nComponent]);
				}
			}
		}

	// make: bitstream of quantized min values
	void makeBitstreamOfQMin(CAC_QF& codec, CAACEncoder& oAACEncoder, int nCoord, int nComponent)
		{
		// if (m_a_bQSelFlag_kv == false), the meaning is that min == max.
		// if (min == max), the all values of the vertex are not needed to be encoded.
		//      so, the only one value, min is to be encoded instead of the all equal values.
		if(m_a_bQSelFlag_kv[nCoord*3+nComponent] == false || 
			(m_a_bQSelFlag_kv[nCoord*3+nComponent] != false && m_a_nAQStep_kv[nCoord*3+nComponent] != 0))
			{
			switch(nComponent)
				{
				case 0:
					oAACEncoder.encodeQMin(codec, m_nQStep_keyValue, m_oDPCM_X_Min.getData_dpcm(nCoord));
					break;

				case 1:
					oAACEncoder.encodeQMin(codec, m_nQStep_keyValue, m_oDPCM_Y_Min.getData_dpcm(nCoord));
					break;

				case 2:
					oAACEncoder.encodeQMin(codec, m_nQStep_keyValue, m_oDPCM_Z_Min.getData_dpcm(nCoord));
					break;
				}
			}
		}

	// make: bitstream of quantized max values
	void makeBitstreamOfQMax(CAC_QF& codec, CAACEncoder& oAACEncoder, int nCoord, int nComponent)
		{
		if(m_a_bQSelFlag_kv[nCoord*3+nComponent] != false && m_a_nAQStep_kv[nCoord*3+nComponent] != 0)
			{
			switch(nComponent)
				{
				case 0:
					oAACEncoder.encodeQMax(codec, m_nQStep_keyValue, m_oDPCM_X_Max.getData_dpcm(nCoord));
					break;

				case 1:
					oAACEncoder.encodeQMax(codec, m_nQStep_keyValue, m_oDPCM_Y_Max.getData_dpcm(nCoord));
					break;

				case 2:
					oAACEncoder.encodeQMax(codec, m_nQStep_keyValue, m_oDPCM_Z_Max.getData_dpcm(nCoord));
					break;
				}
			}
		}

	// make: bitstream of keyValue
	void makeBitstreamOfKeyValue(CAC_QF& codec, CAACEncoder& oAACEncoder, int nCoord, int nComponent)
		{
		int			i = 0;									// index
		int			k = 0;									// index

		// after dpcm, if min(x) and Max(x) are not 0
		if(m_a_nAQStep_kv[nCoord*3+nComponent] != 0 && m_a_bQSelFlag_kv[nCoord*3+nComponent] != false)
			{
			bool*		a_bDic_index = NULL;				// dictionary index flag
			int			nNumberOfSymbol = 0;				// the number of symbols

			// allocate: a_bDic_index
			a_bDic_index = new bool[m_nNumberOfKey_kv];
			memset((void*)a_bDic_index, 0, sizeof(bool)*m_nNumberOfKey_kv);
			
			// calculate: the number of symbols
			for(i = 0; i < m_nNumberOfKey_kv; i++)
				{
				if(a_bDic_index[i] == 0)
					{
					a_bDic_index[i] = 1;
					nNumberOfSymbol++;
					
					int nQ = m_a_nQKeyValue[i*m_nNumberOfCoord_kv*3+nCoord*3+nComponent];
					for(k = i+1; k < m_nNumberOfKey_kv; k++)
						{
						if(a_bDic_index[k] == 0)
							{
							if(nQ == m_a_nQKeyValue[k*m_nNumberOfCoord_kv*3+nCoord*3+nComponent])
								a_bDic_index[k] = 1;
							}
						}
					}
				}
			
			// calculate: the size of incremental(address) mode and the size of occurence(dictionary) mode
			int nSizeOfAddress = (int)pow(2, m_a_nAQStep_kv[nCoord*3+nComponent]+1) - 1;
			int nSizeOfDictionary = nNumberOfSymbol*(m_a_nAQStep_kv[nCoord*3+nComponent]+1);

			// select: either incremental(address) mode or occurence(dictionary) mode
			if(nSizeOfAddress < nSizeOfDictionary)
				{
				makeBitstreamOfKeyValueWithIncrementalMode(codec, oAACEncoder, nCoord, nComponent, nNumberOfSymbol);
				}
			else
				{
				makeBitstreamOfKeyValueWithOccurenceMode(codec, oAACEncoder, nCoord, nComponent);
				}

			// delete: dictionary index
			delete[] a_bDic_index;
			}
		}

	// make: bitstream of keyValue with incremental mode
	void makeBitstreamOfKeyValueWithIncrementalMode(CAC_QF& codec, CAACEncoder& oAACEncoder, int nCoord, int nComponent, int nNumberOfSymbol)
		{
		bool*		a_bDic_index = NULL;					// dictionary index flag
		bool*		a_bDic_address = NULL;					// dictionary address index
		int			nIndex_address = 0;						// address index
		int			nSizeOfAddress = 0;						// the size of address
		int			i = 0;									// index
		int			k = 0;									// index

		// cic_kv_dic_flag_address: flag of address coding
		codec.encode(1, CQF_Status::ci_kv_Dic_method_flag_context);

		// allocate: a_bDic_index
		a_bDic_index = new bool[m_nNumberOfKey_kv];
		memset((void*)a_bDic_index, 0, sizeof(bool)*m_nNumberOfKey_kv);

		// calculate: the size of address
		nSizeOfAddress = (int)pow(2, m_a_nAQStep_kv[nCoord*3+nComponent]+1) - 1;

		// allocate: dictionary address index
		a_bDic_address = new bool[nSizeOfAddress];
		memset((void*)a_bDic_address, 0, sizeof(bool)*nSizeOfAddress);

		// initialize: address
		for(i = 0; i < m_nNumberOfKey_kv; i++)
			{
			int nIndex = 0;
			
			int nQ = m_a_nQKeyValue[i*m_nNumberOfCoord_kv*3+nCoord*3+nComponent];
			if(nQ == 0)
				nIndex = 0;
			else if(nQ > 0)
				nIndex = 2*nQ-1;
			else
				nIndex = -2*nQ;
			
			a_bDic_address[nIndex] = true;
			}

		// send: the values of address
		for(i = 0; i < nSizeOfAddress; i++)
			{
			if(a_bDic_address[i] == true)
				codec.encode(1, CQF_Status::ci_kv_Dic_address_context);
			else
				codec.encode(0, CQF_Status::ci_kv_Dic_address_context);
			}

		// initialize: nIndex_address
		nIndex_address = 0;

		// code: incremental mode
		for(int s = 0; s < nNumberOfSymbol; s++)	// repeat as the number of symbols
			{
			int nQ = 0;		// quantized value

			while(a_bDic_address[nIndex_address] != true)
				nIndex_address++;

			if(nIndex_address == 0)
				nQ = 0;
			else if(nIndex_address % 2 != 0)
				nQ = (nIndex_address+1)/2;
			else
				nQ = -nIndex_address/2;

			// if last symbol, break out
			if(s == nNumberOfSymbol - 1)
				break;

			nIndex_address++;

			// code: dictionary index
			int	nNumberOfOne = 0;			// the number of one flags
			int	nNumberOfZero = 0;			// the number of zero flags

			// count: the number of one flags and zero flags
			for(k = 0; k < m_nNumberOfKey_kv; k++)
				{
				if(a_bDic_index[k] == 0)
					{
					nNumberOfZero++;
					if(nQ == m_a_nQKeyValue[k*m_nNumberOfCoord_kv*3+nCoord*3+nComponent])
						nNumberOfOne++;
					}
				}

			int	nOne = 0;					// flag: one value flag
			int	nZero = 0;					// flag: zero value flag
			if(nNumberOfOne >= nNumberOfZero/2)
				{
				nOne = 0;
				nZero = 1;
				}
			else
				{
				nOne = 1;
				nZero = 0;
				}

			// nOne: the real value of one
			codec.encode(nOne, CQF_Status::ci_kv_Dic_one_context);

			// code: dictionary index
			for(k = 0; k < m_nNumberOfKey_kv; k++)
				{
				if(a_bDic_index[k] == 0)
					{
					if(nQ == m_a_nQKeyValue[k*m_nNumberOfCoord_kv*3+nCoord*3+nComponent])
						{
						a_bDic_index[k] = 1;
						codec.encode(nOne, CQF_Status::ci_kv_Dic_index_context);
						}
					else
						{
						codec.encode(nZero, CQF_Status::ci_kv_Dic_index_context);
						}
					}
				}
			}
		// delete: dictionary address index
		delete[] a_bDic_address;

		// delete: dictionary index
		delete[] a_bDic_index;
		}

	// make: bitstream of keyValue with occurence mode
	void makeBitstreamOfKeyValueWithOccurenceMode(CAC_QF& codec, CAACEncoder& oAACEncoder, int nCoord, int nComponent)
		{
		bool*		a_bDic_index = NULL;					// dictionary index flag
		int			i = 0;									// index
		int			k = 0;									// index

		// cic_kv_dic_flag_address: flag of dictionary coding
		codec.encode(0, CQF_Status::ci_kv_Dic_method_flag_context);

		// allocate: a_bDic_index
		a_bDic_index = new bool[m_nNumberOfKey_kv];
		memset((void*)a_bDic_index, 0, sizeof(bool)*m_nNumberOfKey_kv);

		// code: occurence mode
		for(i = 0; i < m_nNumberOfKey_kv; i++)
			{
			int nQ = 0;		// quantized value

			// if not scanned yet
			if(a_bDic_index[i] == 0)
				{
				a_bDic_index[i] = 1;	// scanned index

				// nQ: the value of dictionary
				nQ = m_a_nQKeyValue[i*m_nNumberOfCoord_kv*3+nCoord*3+nComponent];

				// send: the value of dictionary
				oAACEncoder.encodeKeyValue(codec, m_a_nAQStep_kv[nCoord*3+nComponent], nQ, nComponent);

				// code: dictionary index
				int nLast = 1;					// the flag of whether a symbol is only one
				int	nNumberOfOne = 0;			// the number of one flags
				int	nNumberOfZero = 0;			// the number of zero flags

				// count: the number of one flags and zero flags
				for(k = i+1; k < m_nNumberOfKey_kv; k++)
					{
					if(a_bDic_index[k] == 0)
						{
						nNumberOfZero++;
						if(nQ == m_a_nQKeyValue[k*m_nNumberOfCoord_kv*3+nCoord*3+nComponent])
							{
							nLast = 0;
							nNumberOfOne++;
							}
						}
					}

				// encode: whether the symbol is only one
				codec.encode(nLast, CQF_Status::ci_kv_Dic_last_flag_context);

				int	nOne = 0;					// flag: one value flag
				int	nZero = 0;					// flag: zero value flag							
				// nLast: if the number of symbols is more than one
				if(nLast == 0)
					{
					if(nNumberOfOne >= nNumberOfZero/2)
						{
						nOne = 0;
						nZero = 1;
						}
					else
						{
						nOne = 1;
						nZero = 0;
						}
					// nOne: the real value of one
					codec.encode(nOne, CQF_Status::ci_kv_Dic_one_context);

					// encode: dictionary index
					for(k = i+1; k < m_nNumberOfKey_kv; k++)
						{
						if(a_bDic_index[k] == 0)
							{
							if(nQ == m_a_nQKeyValue[k*m_nNumberOfCoord_kv*3+nCoord*3+nComponent])
								{
								a_bDic_index[k] = 1;
								
								codec.encode(nOne, CQF_Status::ci_kv_Dic_index_context);
								}
							else
								{
								codec.encode(nZero, CQF_Status::ci_kv_Dic_index_context);
								}
							}
						}
					}
				}
			}
		// delete: dictionary index
		delete[] a_bDic_index;
		}

	// get: quantization bits needed for encoding a number
	int getNBits(int n)
		{
		return (int)(log10(n)/log10(2))+1;
		}

	// transpose: source data -> target data which is transposed
	void transposeKeyValue(double* a_dKeyValue_target, double* a_dKeyValue_source, int nNumberOfKey, int nNumberOfCoord)
		{
		double* a_dKeyValue = new double[nNumberOfKey*nNumberOfCoord*3];
		memcpy((void*)a_dKeyValue, (void*)a_dKeyValue_source, sizeof(double)*(nNumberOfKey*nNumberOfCoord*3));

		for(int j = 0; j < nNumberOfCoord; j++)
			{
			for(int i = 0; i < nNumberOfKey; i++)
				{
				for(int c = 0; c < 3; c++)
					{
					//double dValue = a_dKeyValue[i*nNumberOfCoord*3+j*3+c];
					double dValue = getValue(a_dKeyValue, i, j, c);

					//a_dKeyValue_target[j*nNumberOfKey*3+i*3+c] = dValue;
					getTransposedValue(a_dKeyValue_target, j, i, c) = dValue;
					//cout << dValue;
					//cout << ", ";
					}
				//cout << endl;
				}
			//cout << endl;
			}

		delete[] a_dKeyValue;
		}

	// get: the value with the indexes of nKey, nCoord, nComponent in keyValue
	double& getValue(double* a_dValue, int nKey, int nCoord, int nComponent)
		{
		double& dValue = a_dValue[nKey*m_nNumberOfCoord*3 + nCoord*3 + nComponent];
		return dValue;
		}

	// get: the transposed value with the indexes of nKey, nCoord, nComponent in keyValue
	double& getTransposedValue(double* a_dValue, int nCoord, int nKey, int nComponent)
		{
		double& dValue = a_dValue[nCoord*m_nNumberOfKey*3 + nKey*3 + nComponent];
		return dValue;
		}

	// 2002-04-17, Shinjun
	// This function is for linear key encoding
	int getNbBits(int k)
		{
		int j=1;
		while (k >= (1<<j) ) 
			j++;
		
		return j;
		}

	// 2002-04-17, Shinjun
	// This function is for linear key encoding
	// sowoo 2001.10.11 (Min/Max Reduce)
	// Ǽ Digit 
	int GetDigit(double dValue)
		{
		//////////////////////////////////////////////////
		//
		//			Original Value 7ڸ !!!
		//			꿡 ؼ   
		//
		//////////////////////////////////////////////////
		
		char buffer[50];
		_gcvt(dValue, CMAX_DIGIT, buffer);
		dValue = atof(buffer);
		
		//
		//////////////////////////////////////////////////
		
		// "0.0"  ڸ̴.
		if(dValue == 0.0)
			return 0;
		
		int nFirstDigit, nLastDigit;
		
		nFirstDigit = GetLeadingZero(dValue);
		nLastDigit = GetLargestDigit(dValue);
		
		int nDigit = nLastDigit - nFirstDigit;
		
		// CMAX_DIGIT ū    .
		//  floating-point number ϴ ȿڸ 6~7!
		return (nDigit > CMAX_DIGIT)? CMAX_DIGIT: nDigit;
		}

	// 2002-04-17, Shinjun
	// This function is for linear key encoding
	// double Mantissa(10) ϴ α׷!!!
	// input:
	// -> dValue: Source floating point 
	// -> nDigit:  ڸ
	int GetMantissa(double dValue, int nDigit)
		{
		char* buffer;
		int nDummy1 = 0, nDummy2 = 0;
		
		buffer = _ecvt(dValue, nDigit, &nDummy1, &nDummy2);
		
		return ::atoi(buffer);
		}
	
	// 2002-04-17, Shinjun
	// This function is for linear key encoding
	int GetLargestDigit(double key)
		{
		//////////////////////////////////////////////////
		//
		//			Original Value 7ڸ !!!
		//			꿡 ؼ   
		//
		//////////////////////////////////////////////////
		
		char buffer[50];
		_gcvt(key, CMAX_DIGIT, buffer);
		key = atof(buffer);
		
		//
		//////////////////////////////////////////////////
		
		double a;
		__int64 mult;
		double dmult;
		__int64 Ia;
		double e = dEPSILONERROR;
		int nLargestDigit;
		
		mult=1; dmult=0.1;
		nLargestDigit = 0;
		
		a = fabs(key);
		Ia = (__int64) (a + mult*e);
		
		// sowoo 2001.10.11 (Digit   ϱ)
		// "0.1" binary ǥ ⶧ ǵ  ʴ  .
		// 0.1 ϴ  ſ 10  scheme !
		if(a == 0.0)
			return 1;
		
		double dTemp = a - (double)Ia;
		
		if((dTemp == 0.0) || (dTemp < e))
			{
			while(true)
				{
				a = fabs(key);
				a *= dmult;
				Ia = (__int64) (a + e);
				dTemp = a - (double)Ia;
				
				if(dTemp > e)
					return nLargestDigit;
				
				dmult *= 0.1;
				nLargestDigit--;
				}
			}
		
		//while(a - (double) Ia > e*mult)
		while(dTemp > e*mult)
			{
			mult*=10;
			nLargestDigit++;
			a = fabs(key);
			a=(double) mult *a;
			Ia = (__int64) (a + mult*e);
			dTemp = a - (double)Ia;
			}
		
		return nLargestDigit;
		}

	// 2002-04-17, Shinjun
	// This function is for linear key encoding
	int GetLeadingZero(double key)
		{
		//////////////////////////////////////////////////
		//
		//			Original Value 7ڸ !!!
		//			꿡 ؼ   
		//
		//////////////////////////////////////////////////
		
		char buffer[50];
		_gcvt(key, CMAX_DIGIT, buffer);
		key = atof(buffer);
		
		//
		//////////////////////////////////////////////////
		
		double a;
		double e = dEPSILONERROR;
		__int64 nMulti = 10;
		double dMulti = 0.1;
		int nCount = 0;
		
		// 1. 켱  ٲٰ,
		a = fabs(key);
		
		// 2. 0̰ų 0.1 ũ տ "0" .
		if(a >= 1.0)
			{
			nCount++;
			while(a*dMulti > 1.0)
				{
				nCount++;
				dMulti *= 0.1;
				}
			
			return -nCount;
			}
		else if(a == 0.0)
			return 0;
		else if(a >= 0.1)
			return 0;
		else
			{
			nCount++;
			while(a*nMulti < 0.1)
				{
				nCount++;
				nMulti *= 10;
				}
			
			return nCount;
			}
		}
	};
