// CCommandLineProcessor.h
// Author: Shinjun Lee, Samsung AIT
// Date: 2001-12-27

#ifndef _CLINEPROCESSOR_H_
#define _CLINEPROCESSOR_H_

#include <iostream.h>
#include <stdlib.h>
#include <string.h>

class CCommandLineProcessor
	{
  private:
	// variables: encoder part
	char			m_szInputCIFile[_MAX_FNAME];			// input file name
	char			m_szBitstreamFile[_MAX_FNAME];			// bitstream file name
	int				m_nQStep_key;							// quantization parameter for key
	int				m_nQStep_keyValue;						// quantization parameter for keyValue
	char			m_szMode1[_MAX_FNAME];					// first flag
	char			m_szMode2[_MAX_FNAME];					// second flag
	char			m_szMode3[_MAX_FNAME];					// third flag

	// Variable: decoder part
	char			m_szOutputTextFile[_MAX_FNAME];			// output text file name

	// m_nStatus: the status of encoding or decoding
	int				m_nStatus;								// status of encoding or decoding

  public:
	CCommandLineProcessor(void)
		{
		strcpy(m_szInputCIFile, "");
		strcpy(m_szBitstreamFile, "");
		m_nQStep_key = 0;
		m_nQStep_keyValue = 0;
		strcpy(m_szMode1, "");
		strcpy(m_szMode2, "");
		strcpy(m_szMode3, "");
		m_nStatus = 0;
		}

	// set: status of encoding
	void setEncoding(void)
		{
		m_nStatus = 1;					// 1: encoding
		}

	// set: status of decoding
	void setDecoding(void)
		{
		m_nStatus = 2;					// 2: decoding
		}

	// set: arguments
	void setArguments(int argc, char* argv[])
		{
		switch(m_nStatus)
			{
			case 1:
				readArgumentsOfEncoding(argc, argv);
				break;

			case 2:
				readArgumentsOfDecoding(argc, argv);
				break;

			default:
				cout << "Error: the status of encoding or decoding is not known." << endl;
				exit(1);
				break;
			}
		}

	// read: arguments
	void readArgumentsOfEncoding(int argc, char* argv[])
		{
		switch(argc)
			{
			case 8:
				strcpy(m_szMode3, argv[7]);

			case 7:
				strcpy(m_szMode2, argv[6]);

			case 6:
				strcpy(m_szMode1, argv[5]);

			case 5:
				strcpy(m_szInputCIFile, argv[1]);
				strcpy(m_szBitstreamFile, argv[2]);
				m_nQStep_key = atoi(argv[3]);
				m_nQStep_keyValue = atoi(argv[4]);
				break;

			default:
				cout << "Usage: CIEncoder <InputCIFile> <OutputBitstreamFile> <Key QStep> <KeyValue QStep> [-t/-v]" << endl;
				exit(1);
				break;
			}
		}

	// read: arguments
	void readArgumentsOfDecoding(int argc, char* argv[])
		{
		switch(argc)
			{
			case 3:
				strcpy(m_szBitstreamFile, argv[1]);
				strcpy(m_szOutputTextFile, argv[2]);
				break;

			default:
				cout << "Usage: CIDecoder <BitstreamFile> <OutputTextFile>" << endl;
				exit(1);
				break;
			}
		}

	// get: input ci file name
	char* getInputCIFile(void)
		{
		return m_szInputCIFile;
		}

	// get: bitstream file name
	char* getBitstreamFile(void)
		{
		return m_szBitstreamFile;
		}

	// get: output text file
	char* getOutputTextFile(void)
		{
		return m_szOutputTextFile;
		}

	// get: qp of key
	int getQStep_key(void)
		{
		return m_nQStep_key;
		}

	// get: qp of keyValue
	int getQStep_keyValue(void)
		{
		return m_nQStep_keyValue;
		}

	// check: mode of transposition
	bool IsTransposed(void)
		{
		if(!stricmp(m_szMode1, "-t") || !stricmp(m_szMode2, "-t") || !stricmp(m_szMode3, "-t"))	// option of transpose mode(default)
			return false;
		else if(!stricmp(m_szMode1, "-v") || !stricmp(m_szMode2, "-v") || !stricmp(m_szMode3, "-v")) // option of vertex mode
			return true;
		else
			return true;
		}

	// check: mode of linear key coding
	bool IsLinearKeyUsed(void)
		{
		if(!stricmp(m_szMode1, "-k") || !stricmp(m_szMode2, "-k") || !stricmp(m_szMode3, "-k"))	// option of key coding(default)
			return false;
		else if(!stricmp(m_szMode1, "-l") || !stricmp(m_szMode2, "-l") || !stricmp(m_szMode3, "-l"))	// option of linear key coding
			return true;
		else
			return false;
		}

	// check: mode of SAD measure
	bool IsSADUsed(void)
		{
		if(!stricmp(m_szMode1, "-r") || !stricmp(m_szMode2, "-r") || !stricmp(m_szMode3, "-r"))	// option of RQP measure(default)
			return false;
		else if(!stricmp(m_szMode1, "-s") || !stricmp(m_szMode2, "-s") || !stricmp(m_szMode3, "-s"))	// option of SAD measure
			return true;
		else
			return false;
		}
	};

#endif // _CLINEPROCESSOR_H_