// DPCM
// Author: Shinjun Lee, Samsung AIT
// Date: 2001-11-7-Wed

#ifndef _CDPCM_H_
#define _CDPCM_H_

#include "../include/CCQ.h"

#include <memory.h>
#include <math.h>

class CDPCM
	{
  private:
	int		m_nNumber;						// Ÿ 

	int*	m_a_nData;						// Է Ÿ
	int		m_nMin;							// ּҰ
	int		m_nMax;							// ִ밪

	int*	m_a_nData_dpcm;					// dpcm Ÿ
	int		m_nMin_dpcm;					// dpcm ּҰ
	int		m_nMax_dpcm;					// dpcm ִ밪
	
	int*	m_a_nDataXYZ;					// X, Y, Z Է Ÿ
	int		m_nComponent;					// X, Y, Z  

	int*	m_a_nBase;						// Max  Min  Ÿ
	int*	m_a_nBaseXYZ;					// Max  Min  Ÿ

  public:
	CDPCM(void)
		{
		m_nNumber = 0;
		m_a_nData = NULL;
		m_nMin = 0;
		m_nMax = 0;
		m_a_nData_dpcm = NULL;
		m_nMin_dpcm = 0;
		m_nMax_dpcm = 0;
		m_a_nDataXYZ = NULL;
		m_nComponent = 0;
		m_a_nBase = NULL;
		m_a_nBaseXYZ = NULL;
		}

	~CDPCM()
		{
		delete[] m_a_nData;
		delete[] m_a_nData_dpcm;
		delete[] m_a_nBase;
		}

	// clear: ʱȭ
	void clear(void)
		{
		delete[] m_a_nData;
		delete[] m_a_nData_dpcm;
		delete[] m_a_nBase;

		m_nNumber = 0;
		m_a_nData = NULL;
		m_nMin = 0;
		m_nMax = 0;
		m_a_nData_dpcm = NULL;
		m_nMin_dpcm = 0;
		m_nMax_dpcm = 0;
		m_a_nDataXYZ = NULL;
		m_nComponent = 0;
		m_a_nBase = NULL;
		m_a_nBaseXYZ = NULL;
		}

	// Number: Ÿ 
	void setNumber(int nNumber)
		{
		m_nNumber = nNumber;
		}

	// Data: Է Ÿ
	void setData(int* a_nData)
		{
		m_a_nDataXYZ = a_nData;

		delete[] m_a_nData;
		m_a_nData = new int[m_nNumber];
		memset((void*)m_a_nData, 0, sizeof(int)*m_nNumber);

		delete[] m_a_nData_dpcm;
		m_a_nData_dpcm = new int[m_nNumber];
		memset((void*)m_a_nData_dpcm, 0, sizeof(int)*m_nNumber);
		}

	// Base data: ̽ Ÿ
	void setBase(int* a_nBase)
		{
		m_a_nBaseXYZ = a_nBase;
		
		delete[] m_a_nBase;
		m_a_nBase = new int[m_nNumber];
		}

	// Component: X, Y, Z 
	void setComponent(int nComponent)
		{
		int i = 0;

		m_nComponent = nComponent;
		for(i = 0; i < m_nNumber; i++)
			{
			m_a_nData[i] = m_a_nDataXYZ[i*3+nComponent];
			if(m_a_nBase != NULL)
				m_a_nBase[i] = m_a_nBaseXYZ[i*3+nComponent];
			if(i == 0)
				m_nMin = m_nMax = m_a_nData[i];
			else
				{
				if(m_a_nData[i] < m_nMin)
					m_nMin = m_a_nData[i];
				if(m_a_nData[i] > m_nMax)
					m_nMax = m_a_nData[i];
				}
			}
		}

	// DPCM: dpcm CQ 
	void dpcm(void)
		{
		int* a_nData = NULL;
		int i = 0;

		// CQ: circular quantization
		CCQ oCQ;

		// allocation: a_nData
		a_nData = new int[m_nNumber];

		// copy: a_nData <-- m_a_nData
		memcpy((void*)a_nData, m_a_nData, sizeof(int)*m_nNumber);

		// dpcm: 1st order dpcm
		i = 0;
		if(m_a_nBase != NULL)
			{
			if(m_nMin < m_a_nBase[i])
				{
				m_a_nData_dpcm[i] = a_nData[i] - m_a_nBase[i];
				m_a_nData_dpcm[i] = oCQ.quantize(m_a_nData_dpcm[i], m_a_nBase[i], m_nMax);
				}
			else
				{
				m_a_nData_dpcm[i] = a_nData[i] - m_nMin;
				m_a_nData_dpcm[i] = oCQ.quantize(m_a_nData_dpcm[i], m_nMin, m_nMax);
				}
			}
		else
			{
			m_a_nData_dpcm[i] = a_nData[i];											// dpcm
			m_a_nData_dpcm[i] = oCQ.quantize(m_a_nData_dpcm[i], m_nMin, m_nMax);	// cq
			}

		m_nMin_dpcm = m_nMax_dpcm = m_a_nData_dpcm[i];
		for(i = 1; i < m_nNumber; i++)
			{
			if(m_a_nBase != NULL && a_nData[i-1] < m_a_nBase[i])	// if the previous value is less than the base value
				{
				m_a_nData_dpcm[i] = a_nData[i] - m_a_nBase[i];		// dpcm is performed between the current value and the base value
				m_a_nData_dpcm[i] = oCQ.quantize(m_a_nData_dpcm[i], m_a_nBase[i], m_nMax);
				}
			else
				{
				m_a_nData_dpcm[i] = a_nData[i] - a_nData[i-1];		// dpcm is performed between the current value and the previous value
				m_a_nData_dpcm[i] = oCQ.quantize(m_a_nData_dpcm[i], m_nMin, m_nMax);
				}

			if(m_a_nData_dpcm[i] < m_nMin_dpcm)
				m_nMin_dpcm = m_a_nData_dpcm[i];
			if(m_a_nData_dpcm[i] > m_nMax_dpcm)
				m_nMax_dpcm = m_a_nData_dpcm[i];
			}

		delete[] a_nData;
		}

	// Inverse DPCM: inverse DPCM CQ 
	void inverseDPCM(bool* a_bQFlag_kv, int* a_nQStep_kv, int* a_nBase_kv, int c)
		{
		int* a_nData_dpcm = NULL;
		int i = 0;

		// CQ: circular quantization
		CCQ oCQ;

		// allocation: a_nData_dpcm
		a_nData_dpcm = new int[m_nNumber];

		// copy: a_nData_dpcm <-- m_a_nData_dpcm
		memcpy((void*)a_nData_dpcm, m_a_nData_dpcm, sizeof(int)*m_nNumber);

		// inverse dpcm: 1st order inverse dpcm
		i = 0;
		if(a_bQFlag_kv[i*3+c] == false && m_a_nBase != NULL)
			{
			int nQData = getData_base(i);
			m_a_nData[i] = nQData;
			}
		else if(a_bQFlag_kv[i*3+c] != false && a_nQStep_kv[i*3+c] == 0)
			{
			int nIndex_base = a_nBase_kv[i*3+c];
			int nQData = getData(nIndex_base);
			m_a_nData[i] = nQData;
			}
		else
			{
			if(m_a_nBase != NULL)
				{
				if(m_nMin < m_a_nBase[i])
					{
					m_a_nData[i] = a_nData_dpcm[i] + m_a_nBase[i];
					m_a_nData[i] = oCQ.inverseQuantize(m_a_nData[i], m_a_nBase[i], m_nMax, 0);
					}
				else
					{
					m_a_nData[i] = a_nData_dpcm[i] + m_nMin;
					m_a_nData[i] = oCQ.inverseQuantize(m_a_nData[i], m_nMin, m_nMax, 0);
					}
				}
			else
				{
				m_a_nData[i] = a_nData_dpcm[i];
				m_a_nData[i] = oCQ.inverseQuantize(m_a_nData[i], m_nMin, m_nMax, 0);
				}
			}

		m_nMin_dpcm = m_nMax_dpcm = m_a_nData_dpcm[i];
		for(i = 1; i < m_nNumber; i++)
			{
			if(a_bQFlag_kv[i*3+c] == false && m_a_nBase != NULL)
				{
				int nQData = getData_base(i);
				m_a_nData[i] = nQData;
				}
			else if(a_bQFlag_kv[i*3+c] != false && a_nQStep_kv[i*3+c] == 0)
				{
				int nIndex_base = a_nBase_kv[i*3+c];
				int nQData = getData(nIndex_base);
				m_a_nData[i] = nQData;
				}
			else
				{
				if(m_a_nBase != NULL && m_a_nData[i-1] < m_a_nBase[i])
					{
					m_a_nData[i] = a_nData_dpcm[i] + m_a_nBase[i];
					m_a_nData[i] = oCQ.inverseQuantize(m_a_nData[i], m_a_nBase[i], m_nMax, m_a_nData[i-1]);
					}
				else
					{
					m_a_nData[i] = a_nData_dpcm[i] + m_a_nData[i-1];		// dpcm 
					m_a_nData[i] = oCQ.inverseQuantize(m_a_nData[i], m_nMin, m_nMax, m_a_nData[i-1]);
					}
				}
			
			if(m_a_nData_dpcm[i] < m_nMin_dpcm)
				m_nMin_dpcm = m_a_nData_dpcm[i];
			if(m_a_nData_dpcm[i] > m_nMax_dpcm)
				m_nMax_dpcm = m_a_nData_dpcm[i];
			}

		delete[] a_nData_dpcm;
		}

	// Inverse DPCM: inverse DPCM CQ 
	void inverseDPCM(bool* a_bQFlag_kv, int* a_nQStep_kv, int* a_nBase_kv, int nCoord, int nComponent)
		{
		int i = 0;

		// CQ: circular quantization
		CCQ oCQ;

		// inverse dpcm: 1st order inverse dpcm
		i = nCoord;
		if(nCoord == 0)
			{
			if(a_bQFlag_kv[i*3+nComponent] == false && m_a_nBase != NULL)
				{
				int nQData = getData_base(i);
				m_a_nData[i] = nQData;
				}
			else if(a_bQFlag_kv[i*3+nComponent] != false && a_nQStep_kv[i*3+nComponent] == 0)
				{
				int nIndex_base = a_nBase_kv[i*3+nComponent];
				int nQData = getData(nIndex_base);
				m_a_nData[i] = nQData;
				}
			else
				{
				if(m_a_nBase != NULL)
					{
					if(m_nMin < m_a_nBase[i])
						{
						m_a_nData[i] = m_a_nData_dpcm[i];														// i-dpcm
						m_a_nData[i] = oCQ.inverseQuantize(m_a_nData[i], m_a_nBase[i], m_nMax, m_a_nBase[i]);	// i-cq
						}
					else
						{
						m_a_nData[i] = m_a_nData_dpcm[i];														// i-dpcm
						m_a_nData[i] = oCQ.inverseQuantize(m_a_nData[i], m_nMin, m_nMax, m_nMin);				// i-cq
						}
					}
				else
					{
					m_a_nData[i] = m_a_nData_dpcm[i];															// i-dpcm
					m_a_nData[i] = oCQ.inverseQuantize(m_a_nData[i], m_nMin, m_nMax);							// i-cq
					}
				}
			}
		else
			{
			m_nMin_dpcm = m_nMax_dpcm = m_a_nData_dpcm[i];
			if(a_bQFlag_kv[i*3+nComponent] == false && m_a_nBase != NULL)
				{
				int nQData = getData_base(i);
				m_a_nData[i] = nQData;
				}
			else if(a_bQFlag_kv[i*3+nComponent] != false && a_nQStep_kv[i*3+nComponent] == 0)
				{
				int nIndex_base = a_nBase_kv[i*3+nComponent];
				int nQData = getData(nIndex_base);
				m_a_nData[i] = nQData;
				}
			else
				{
				if(m_a_nBase != NULL && m_a_nData[i-1] < m_a_nBase[i])
					{
					m_a_nData[i] = m_a_nData_dpcm[i];															// i-dpcm
					m_a_nData[i] = oCQ.inverseQuantize(m_a_nData[i], m_a_nBase[i], m_nMax, m_a_nBase[i]);		// i-cq
					}
				else
					{
					m_a_nData[i] = m_a_nData_dpcm[i];															// i-dpcm
					m_a_nData[i] = oCQ.inverseQuantize(m_a_nData[i], m_nMin, m_nMax, m_a_nData[i-1]);			// i-cq
					}
				}

			if(m_a_nData_dpcm[i] < m_nMin_dpcm)
				m_nMin_dpcm = m_a_nData_dpcm[i];
			if(m_a_nData_dpcm[i] > m_nMax_dpcm)
				m_nMax_dpcm = m_a_nData_dpcm[i];
			}
		}

	// Data: Quantization Ÿ
	int getData(int nIndex)
		{
		return m_a_nData[nIndex];
		}

	// Data: Quantization Ÿ
	void setData(int nIndex, int nValue)
		{
		m_a_nData[nIndex] = nValue;
		}

	// Data_dpcm: dpcm Ÿ
	int getData_dpcm(int nIndex)
		{
		return m_a_nData_dpcm[nIndex];
		}

	// Data_dpcm: dpcm Ÿ
	void setData_dpcm(int nIndex, int nValue)
		{
		m_a_nData_dpcm[nIndex] = nValue;
		}

	// Data_base: base Ÿ
	int getData_base(int nIndex)
		{
		return m_a_nBase[nIndex];
		}

	// Data_base: base Ÿ
	void setData_base(int nIndex, int nValue)
		{
		m_a_nBase[nIndex] = nValue;
		}

	// Min: Է ּҰ
	int getMin(void)
		{
		return m_nMin;
		}

	// Min: Է ּҰ
	void setMin(int nMin)
		{
		m_nMin = nMin;
		}

	// Max: Է ִ밪
	int getMax(void)
		{
		return m_nMax;
		}

	// Max: Է ִ밪
	void setMax(int nMax)
		{
		m_nMax = nMax;
		}

	// Min: dpcm ּҰ
	int getMinDPCM(void)
		{
		return m_nMin_dpcm;
		}

	// Max: dpcm ִ밪
	int getMaxDPCM(void)
		{
		return m_nMax_dpcm;
		}

	// NBits: ʿ Ʈ ϱ
	int getNBits(int n)
		{
		return (int)(log10(n)/log10(2))+1;
		}

	// QStep: Ű  ʿ Ʈ ϱ
	int getQStep(void)
		{
		if(abs(m_nMin_dpcm) > abs(m_nMax_dpcm))
			return getNBits(abs(m_nMin_dpcm));
		return getNBits(abs(m_nMax_dpcm));
		}
	};

#endif // _CDPCM_H_