// KeyValue encoder class
// Date: 2001. 9. 19. Wed.
// Author: Shinjun Lee

#include <stdio.h>
#include "../include/CCQ.h"

#define sign(n) ((n >= 0) ? 1 : -1)			// sign: ȣ Լ

#define DPCM_T	1
#define DPCM_S	2
#define DPCM_TS	3

#define FileName_Bitrate	"bitrate_kv.txt"

// Compare: qsort  Integer  Լ
int compareInt(const void* arg1, const void* arg2)
	{
	/* Compare all of both integers: */
	return int(*(int*)arg1 - *(int*)arg2);
	}

// KeyValue Encoder: KeyValue ȣȭ Ŭ
class CKeyValueEncoder
	{
private:
	double*		m_a_dKeyValue;				// KeyValue Ÿ
	int*		m_a_nQKeyValue;				// ȭ KeyValue Ÿ

	int			m_nNumberOfKey;				// Key 
	int			m_nNumberOfCoord;			// Coord 

	double		m_dX_max;					// x ִ밪
	double		m_dX_min;					// x ּҰ
	double		m_dY_max;					// y ִ밪
	double		m_dY_min;					// y ּҰ
	double		m_dZ_max;					// z ִ밪
	double		m_dZ_min;					// z ּҰ
	double		m_dXYZ_delta;				// x, y, z ִ Delta
	int			m_nXYZ_max;					// x, y, z ִ Range ִ밪 (0: x, 1: y, 2: z)

	int			m_nDigit_keyValue;			// Key Value Digit

	bool*		m_a_bQFlag_kv;				// KeyValue quantization selection flag
	int*		m_a_nQStep_kv;				// KV QStep
	int*		m_a_nTSMode;				// x, y, z TS Index
	int*		m_a_nQMin;					// ּ 谪 迭
	int*		m_a_nQMax;					// ִ 谪 迭
	int*		m_a_nBase;					// Base vertex index

	int*		m_a_nSymbol;				// x, y, z Symbol 
	int			m_nNumberOfSymbol;			// ü Symbol 

public:
	// Constructor
	CKeyValueEncoder(void)
		{
		m_a_dKeyValue = NULL;
		m_a_nQKeyValue = NULL;

		m_nNumberOfKey = 0;
		m_nNumberOfCoord = 0;

		m_dX_max = 0;
		m_dX_min = 0;
		m_dY_max = 0;
		m_dY_min = 0;
		m_dZ_max = 0;
		m_dZ_min = 0;
		m_dXYZ_delta = 0;
		m_nXYZ_max = 0;

		m_nDigit_keyValue = 0;

		m_a_bQFlag_kv = NULL;
		m_a_nQStep_kv = NULL;
		m_a_nTSMode = NULL;
		m_a_nQMin = NULL;
		m_a_nQMax = NULL;
		m_a_nBase = NULL;

		m_a_nSymbol = NULL;
		m_nNumberOfSymbol = 0;
		}

	// Destructor
	~CKeyValueEncoder()
		{
		delete[] m_a_nQKeyValue;

		delete[] m_a_bQFlag_kv;
		delete[] m_a_nQStep_kv;
		delete[] m_a_nTSMode;
		delete[] m_a_nQMin;
		delete[] m_a_nQMax;
		delete[] m_a_nBase;

		delete[] m_a_nSymbol;
		}

	// Setting: key value digit
	void setKeyValueDigit(int nDigit_keyValue)
		{
		m_nDigit_keyValue = nDigit_keyValue;
		}

	// KeyValue Encoding: KeyValue ڵϱ
	double encode(double* a_dKeyValue, int nNumberOfKey, int nNumberOfCoord, int nQStep_keyValue)
		{
		// Variables: ȭ KeyValue Ÿ, ε
		int* a_nQKeyValue = NULL;
		int i = 0;
		int j = 0;
		int k = 0;
		int c = 0;

		// Variables
		double dSAD_max = 0;				// SAD ִ밪  
		double dEntropy = 0;				//  Vertex Entropy
		double dEntropy_max = 0;			//  Vertex ִ Entropy
		double dEntropy_total = 0;			// ü Vertex Entropy
		int nNumberOfSymbol = 0;			// Symbol 
		int	nQStep_kv_c = 0;				// ȭ Ʈ

		// Moving:  Ѱܹޱ
		m_a_dKeyValue = a_dKeyValue;
		m_nNumberOfKey = nNumberOfKey;
		m_nNumberOfCoord = nNumberOfCoord;

		// Max, Min: ִ, ּ ϱ(m_dX_max, m_dX_min, m_dY_max, m_dY_min, m_dZ_max, m_dZ_min)
		getDMaxDMin(a_dKeyValue);

		// Allocation: m_a_nQKeyValue ޸ Ҵ
		m_a_nQKeyValue = new int[m_nNumberOfKey*m_nNumberOfCoord*3];
		a_nQKeyValue = new int[m_nNumberOfKey*m_nNumberOfCoord*3];

		// Allocation: m_a_bQFlag_kv, m_a_nQStep_kv, m_a_nTSMode, m_a_nQMin, m_a_nQMax, m_a_nBase
		m_a_bQFlag_kv = new bool[m_nNumberOfCoord*3];
		m_a_nQStep_kv = new int[m_nNumberOfCoord*3];
		m_a_nTSMode = new int[m_nNumberOfCoord*3];
		m_a_nQMin = new int[m_nNumberOfCoord*3];
		m_a_nQMax = new int[m_nNumberOfCoord*3];
		m_a_nBase = new int[m_nNumberOfCoord*3];

		// Allocation: m_a_nSymbol
		m_a_nSymbol = new int[m_nNumberOfCoord*3];

		// Initialization: m_a_nBase
		memset((void*)m_a_nBase, 0, sizeof(int)*(m_nNumberOfCoord*3));

		// Quantization: ȭϱ(a_dKeyValue -> a_nQKeyValue)
		quantize(m_a_nQKeyValue, m_a_dKeyValue, nQStep_keyValue);

		// QMax, QMin: ִ, ּ ϱ(m_a_nQMin, m_a_nQMax)
		getQMinQMax(m_a_nQMin, m_a_nQMax, m_a_nQKeyValue, m_nNumberOfKey, m_nNumberOfCoord);

		// Vertex selection: QMin, QMax  Vertex false 
		for(j = 0; j < m_nNumberOfCoord; j++)
			{
			if(m_a_nQMin[j*3] == m_a_nQMax[j*3])
				m_a_bQFlag_kv[j*3] = false;
			else
				m_a_bQFlag_kv[j*3] = true;
			if(m_a_nQMin[j*3+1] == m_a_nQMax[j*3+1])
				m_a_bQFlag_kv[j*3+1] = false;
			else
				m_a_bQFlag_kv[j*3+1] = true;
			if(m_a_nQMin[j*3+2] == m_a_nQMax[j*3+2])
				m_a_bQFlag_kv[j*3+2] = false;
			else
				m_a_bQFlag_kv[j*3+2] = true;
			}

		// Dump: Quantization File
		//dump_quantization("Q");

		// DPCM:  x, y, z component dpcm
		for(j = 0; j < m_nNumberOfCoord; j++)
			{
			for(c = 0; c < 3; c++)	// j: 0(=x), 1(=y), 2(=z)
				{
				// Temporal DPCM: ð dpcm
				dpcm_temporal(a_nQKeyValue, m_a_nQKeyValue, j, c);
				dEntropy_max = dEntropy = getEntropyOfVertex(a_nQKeyValue, j, c, nNumberOfSymbol);
				m_a_nTSMode[j*3+c] = DPCM_T;		// Temporal

				// Spatial DPCM:  dpcm
				if(j > 0)
					{
					for(k = j-1; k >= 0; k--)
						{
						dpcm_spatial(a_nQKeyValue, m_a_nQKeyValue, j, c, k);
						dEntropy = getEntropyOfVertex(a_nQKeyValue, j, c, nNumberOfSymbol);
						if(dEntropy < dEntropy_max)
							{
							dEntropy_max = dEntropy;
							m_a_nTSMode[j*3+c] = DPCM_S;	// Spatial
							m_a_nBase[j*3+c] = k;
							}
						}
					}

				// Temporal and spatial DPCM: ð dpcm
				if(j > 0)
					{
					for(k = j-1; k >= 0; k--)
						{
						dpcm_tps(a_nQKeyValue, m_a_nQKeyValue, j, c, k);
						dEntropy = getEntropyOfVertex(a_nQKeyValue, j, c, nNumberOfSymbol);
						if(dEntropy < dEntropy_max)
							{
							dEntropy_max = dEntropy;
							m_a_nTSMode[j*3+c] = DPCM_TS;	// T+S
							m_a_nBase[j*3+c] = k;
							}
						}
					}
				}
			}

		// DPCM:  x, y, z component dpcm
		m_nNumberOfSymbol = 0;
		//ofstream fout_symbol;
		//fout_symbol.open("Symbol.txt");
		//ofstream fout;
		//fout.open("Entropy.txt");
		dEntropy_total = 0;
		for(j = 0; j < nNumberOfCoord; j++)
			{
			//fout_symbol << j << ": ";
			//fout << j << ": ";
			for(c = 0; c < 3; c++)
				{
				switch(m_a_nTSMode[j*3+c])
					{
					case DPCM_T:
						dpcm_temporal(a_nQKeyValue, m_a_nQKeyValue, j, c);
						break;

					case DPCM_S:
						dpcm_spatial(a_nQKeyValue, m_a_nQKeyValue, j, c, m_a_nBase[j*3+c]);
						break;

					case DPCM_TS:
						dpcm_tps(a_nQKeyValue, m_a_nQKeyValue, j, c, m_a_nBase[j*3+c]);
						break;
					}
				dEntropy = getEntropyOfVertex(a_nQKeyValue, j, c, nNumberOfSymbol);
				dEntropy_total += dEntropy;
				m_a_nSymbol[j*3+c] = nNumberOfSymbol;
				m_nNumberOfSymbol += nNumberOfSymbol;
				//fout_symbol << "\t" << nNumberOfSymbol;
				//fout << "\t" << dEntropy;
				}
			//fout_symbol << endl;
			//fout << endl;
			}
		//fout_symbol.close();
		//fout.close();

		// Moving: keyValue ϱ
		moveQKeyValue(m_a_nQKeyValue, a_nQKeyValue);

		// Dump: DPCM File
		//dump_quantization("DPCM");

		delete[] a_nQKeyValue;

		return dEntropy_total;
		}

	// Moving: ȭ Ű Ѱֱ
	void getQKeyValue(int*& a_nQKeyValue)
		{
		a_nQKeyValue = new int[m_nNumberOfKey*m_nNumberOfCoord*3];

		for(int i = 0; i < m_nNumberOfKey; i++)
			{
			for(int j = 0; j < m_nNumberOfCoord; j++)
				{
				a_nQKeyValue[i*m_nNumberOfCoord*3+j*3] = m_a_nQKeyValue[i*m_nNumberOfCoord*3+j*3];
				a_nQKeyValue[i*m_nNumberOfCoord*3+j*3+1] = m_a_nQKeyValue[i*m_nNumberOfCoord*3+j*3+1];
				a_nQKeyValue[i*m_nNumberOfCoord*3+j*3+2] = m_a_nQKeyValue[i*m_nNumberOfCoord*3+j*3+2];
				}
			}
		}

	// Max, Min: ִ밪, ּҰ Ѱֱ
	void getMaxMin(double& dX_min, double& dY_min, double& dZ_min, double& dX_max, double& dY_max, double& dZ_max, double& dXYZ_delta)
		{
		dX_min = m_dX_min;
		dY_min = m_dY_min;
		dZ_min = m_dZ_min;

		dX_max = m_dX_max;
		dY_max = m_dY_max;
		dZ_max = m_dZ_max;

		dXYZ_delta = m_dXYZ_delta;
		}

	// Max, Min: ִ밪, ּҰ Ѱֱ
	void getMaxMin(double& dX_min, double& dY_min, double& dZ_min, double& dX_max, double& dY_max, double& dZ_max, unsigned int& nXYZ_max)
		{
		dX_min = m_dX_min;
		dY_min = m_dY_min;
		dZ_min = m_dZ_min;

		dX_max = m_dX_max;
		dY_max = m_dY_max;
		dZ_max = m_dZ_max;

		nXYZ_max = m_nXYZ_max;
		}

	// NBits: ʿ Ʈ ϱ
	int getNBits(int n)
		{
		if(n == 0)
			return 1;
		return (int)(log10(n)/log10(2))+1;
		}

	// QStep: Ű  ʿ Ʈ ϱ
	int getQStep(int nMax, int nMin)
		{
		if(nMin == 0 && nMax == 0)
			return 0;
		if(abs(nMin) > abs(nMax))
			return getNBits(abs(nMin));
		return getNBits(abs(nMax));
		}

	// TSMode: TS Index ϱ
	void getTSMode(int*& a_nTSMode)
		{
		a_nTSMode = new int[m_nNumberOfCoord*3];

		for(int j = 0; j < m_nNumberOfCoord; j++)
			{
			a_nTSMode[j*3] = m_a_nTSMode[j*3];
			a_nTSMode[j*3+1] = m_a_nTSMode[j*3+1];
			a_nTSMode[j*3+2] = m_a_nTSMode[j*3+2];
			}
		}

	// QStep_kv: KV ȭ Ʈ
	void getQStep_kv(int*& a_nQStep_kv, int* a_nBase_kv)
		{
		a_nQStep_kv = new int[m_nNumberOfCoord*3];

		int nQX = 0;				// ȭ X
		int nQY = 0;				// ȭ Y
		int nQZ = 0;				// ȭ Z
		int nQX_max = INT_MIN;		// ȭ X ִ밪
		int nQX_min = INT_MAX;		// ȭ X ּҰ
		int nQY_max = INT_MIN;		// ȭ Y ִ밪
		int nQY_min = INT_MAX;		// ȭ Y ּҰ
		int nQZ_max = INT_MIN;		// ȭ Z ִ밪
		int nQZ_min = INT_MAX;		// ȭ Z ּҰ
		int nQX_max_max = INT_MIN;	// ȭ X ִ밪
		int nQX_min_min = INT_MAX;	// ȭ X ּҰ
		int nQY_max_max = INT_MIN;	// ȭ Y ִ밪
		int nQY_min_min = INT_MAX;	// ȭ Y ּҰ
		int nQZ_max_max = INT_MIN;	// ȭ Z ִ밪
		int nQZ_min_min = INT_MAX;	// ȭ Z ּҰ
		int nIndex = 0;				// ε

		for(int i = 0; i < m_nNumberOfCoord; i++)
			{
			nQX_max = INT_MIN;
			nQX_min = INT_MAX;
			nQY_max = INT_MIN;
			nQY_min = INT_MAX;
			nQZ_max = INT_MIN;
			nQZ_min = INT_MAX;
			for(int j = 0; j < m_nNumberOfKey; j++)
				{
				nIndex = i;

				nQX = m_a_nQKeyValue[j*m_nNumberOfCoord*3+nIndex*3];
				nQY = m_a_nQKeyValue[j*m_nNumberOfCoord*3+nIndex*3+1];
				nQZ = m_a_nQKeyValue[j*m_nNumberOfCoord*3+nIndex*3+2];

				if(nQX > nQX_max)
					nQX_max = nQX;
				if(nQX < nQX_min)
					nQX_min = nQX;

				if(nQY > nQY_max)
					nQY_max = nQY;
				if(nQY < nQY_min)
					nQY_min = nQY;

				if(nQZ > nQZ_max)
					nQZ_max = nQZ;
				if(nQZ < nQZ_min)
					nQZ_min = nQZ;

				}
			a_nQStep_kv[i*3] = m_a_nQStep_kv[i*3] = getQStep(nQX_max, nQX_min);
			a_nQStep_kv[i*3+1] = m_a_nQStep_kv[i*3+1] = getQStep(nQY_max, nQY_min);
			a_nQStep_kv[i*3+2] = m_a_nQStep_kv[i*3+2] = getQStep(nQZ_max, nQZ_min);

			if(i > 0)
				{
				int nBase = 0;
				nBase = a_nBase_kv[i*3];
				if(m_a_nQStep_kv[i*3] == 0)
					if(m_a_nQMin[nBase*3] != m_a_nQMin[i*3] || m_a_nQMax[nBase*3] != m_a_nQMax[i*3])
						a_nQStep_kv[i*3] = m_a_nQStep_kv[i*3] = 1;
				nBase = a_nBase_kv[i*3+1];
				if(m_a_nQStep_kv[i*3+1] == 0)
					if(m_a_nQMin[nBase*3+1] != m_a_nQMin[i*3+1] || m_a_nQMax[nBase*3+1] != m_a_nQMax[i*3+1])
						a_nQStep_kv[i*3+1] = m_a_nQStep_kv[i*3+1] = 1;
				nBase = a_nBase_kv[i*3+2];
				if(m_a_nQStep_kv[i*3+2] == 0)
					if(m_a_nQMin[nBase*3+2] != m_a_nQMin[i*3+2] || m_a_nQMax[nBase*3+2] != m_a_nQMax[i*3+2])
						a_nQStep_kv[i*3+2] = m_a_nQStep_kv[i*3+2] = 1;
				}
			}
		}

	// QStep_kv: KV ȭ Ʈ
	void getQStep_kv(int*& a_nQStep_kv, bool* a_bFlag_plus)
		{
		a_nQStep_kv = new int[m_nNumberOfCoord*3];

		int nQX = 0;				// ȭ X
		int nQY = 0;				// ȭ Y
		int nQZ = 0;				// ȭ Z
		int nQX_max = INT_MIN;		// ȭ X ִ밪
		int nQX_min = INT_MAX;		// ȭ X ּҰ
		int nQY_max = INT_MIN;		// ȭ Y ִ밪
		int nQY_min = INT_MAX;		// ȭ Y ּҰ
		int nQZ_max = INT_MIN;		// ȭ Z ִ밪
		int nQZ_min = INT_MAX;		// ȭ Z ּҰ
		int nQX_max_max = INT_MIN;	// ȭ X ִ밪
		int nQX_min_min = INT_MAX;	// ȭ X ּҰ
		int nQY_max_max = INT_MIN;	// ȭ Y ִ밪
		int nQY_min_min = INT_MAX;	// ȭ Y ּҰ
		int nQZ_max_max = INT_MIN;	// ȭ Z ִ밪
		int nQZ_min_min = INT_MAX;	// ȭ Z ּҰ
		int nIndex = 0;				// ε

		for(int i = 0; i < m_nNumberOfCoord; i++)
			{
			a_bFlag_plus[i*3] = true;
			a_bFlag_plus[i*3+1] = true;
			a_bFlag_plus[i*3+2] = true;

			nQX_max = INT_MIN;
			nQX_min = INT_MAX;
			nQY_max = INT_MIN;
			nQY_min = INT_MAX;
			nQZ_max = INT_MIN;
			nQZ_min = INT_MAX;
			for(int j = 0; j < m_nNumberOfKey; j++)
				{
				nIndex = i;

				nQX = m_a_nQKeyValue[j*m_nNumberOfCoord*3+nIndex*3];
				nQY = m_a_nQKeyValue[j*m_nNumberOfCoord*3+nIndex*3+1];
				nQZ = m_a_nQKeyValue[j*m_nNumberOfCoord*3+nIndex*3+2];

				if(nQX > nQX_max)
					nQX_max = nQX;
				if(nQX < nQX_min)
					nQX_min = nQX;

				if(nQY > nQY_max)
					nQY_max = nQY;
				if(nQY < nQY_min)
					nQY_min = nQY;

				if(nQZ > nQZ_max)
					nQZ_max = nQZ;
				if(nQZ < nQZ_min)
					nQZ_min = nQZ;

				if(nQX < 0)
					a_bFlag_plus[i*3] = false;
				if(nQY < 0)
					a_bFlag_plus[i*3+1] = false;
				if(nQZ < 0)
					a_bFlag_plus[i*3+2] = false;
				}
			a_nQStep_kv[i*3] = m_a_nQStep_kv[i*3] = getQStep(nQX_max, nQX_min);
			a_nQStep_kv[i*3+1] = m_a_nQStep_kv[i*3+1] = getQStep(nQY_max, nQY_min);
			a_nQStep_kv[i*3+2] = m_a_nQStep_kv[i*3+2] = getQStep(nQZ_max, nQZ_min);
			}
		}

	// a_bQFlag_kv: KeyValue selection flag ϱ
	void getQFlag_kv(bool*& a_bQFlag_kv)
		{
		a_bQFlag_kv = new bool[m_nNumberOfCoord*3];

		for(int j = 0; j < m_nNumberOfCoord; j++)
			{
			a_bQFlag_kv[j*3] = m_a_bQFlag_kv[j*3];
			a_bQFlag_kv[j*3+1] = m_a_bQFlag_kv[j*3+1];
			a_bQFlag_kv[j*3+2] = m_a_bQFlag_kv[j*3+2];
			}
		}

	// m_a_nSymbol:  Vertex  Symbol  ϱ
	void getSymbol(int*& a_nSymbol)
		{
		a_nSymbol = m_a_nSymbol;
		}

	// nNumberOfSymbol: ü Symbol 
	void getNumberOfSymbol(int& nNumberOfSymbol)
		{
		nNumberOfSymbol = m_nNumberOfSymbol;
		}

	// a_nQMin, a_nQMax: ִ, ּ   ϱ
	void getQMinQMax(int*& a_nQMin, int*& a_nQMax)
		{
		int j = 0;

		a_nQMin = new int[m_nNumberOfCoord*3];

		for(j = 0; j < m_nNumberOfCoord; j++)
			{
			a_nQMin[j*3] = m_a_nQMin[j*3];
			a_nQMin[j*3+1] = m_a_nQMin[j*3+1];
			a_nQMin[j*3+2] = m_a_nQMin[j*3+2];
			}

		a_nQMax = new int[m_nNumberOfCoord*3];

		for(j = 0; j < m_nNumberOfCoord; j++)
			{
			a_nQMax[j*3] = m_a_nQMax[j*3];
			a_nQMax[j*3+1] = m_a_nQMax[j*3+1];
			a_nQMax[j*3+2] = m_a_nQMax[j*3+2];
			}
		}

	// a_bBase: Base vertex ã
	void getBase(int*& a_nBase)
		{
		a_nBase = new int[m_nNumberOfCoord*3];

		for(int j = 0; j < m_nNumberOfCoord; j++)
			{
			a_nBase[j*3] = m_a_nBase[j*3];
			a_nBase[j*3+1] = m_a_nBase[j*3+1];
			a_nBase[j*3+2] = m_a_nBase[j*3+2];
			}
		}

	// Dump: Quantization File
	void dump_quantization(char* szString)
		{
		char szFile[_MAX_FNAME];				// File name
		int i = 0;
		int j = 0;

		ofstream fout;
		strcpy(szFile, szString);
		strcat(szFile, ".txt");
		fout.open(szFile);

		fout << szString << "(x)\t";
		for(i = 0; i < m_nNumberOfKey; i++)
			fout << i << "\t";
		fout << endl;
		for(j = 0; j < m_nNumberOfCoord; j++)
			{
			fout << j;
			if(m_a_bQFlag_kv[j*3] == true)
				{
				for(i = 0; i < m_nNumberOfKey; i++)
 					fout << "\t" << m_a_nQKeyValue[i*m_nNumberOfCoord*3+j*3];
				}
			else
				{
				i = 0;
				fout << "\t" << m_a_nQKeyValue[i*m_nNumberOfCoord*3+j*3];
				}
			fout << endl;
			}
		fout << endl;

		fout << szString << "(y)\t";
		for(i = 0; i < m_nNumberOfKey; i++)
			fout << i << "\t";
		fout << endl;
		for(j = 0; j < m_nNumberOfCoord; j++)
			{
			fout << j;
			if(m_a_bQFlag_kv[j*3+1] == true)
				{
				for(i = 0; i < m_nNumberOfKey; i++)
					fout << "\t" << m_a_nQKeyValue[i*m_nNumberOfCoord*3+j*3+1];
				}
			else
				{
				i = 0;
				fout << "\t" << m_a_nQKeyValue[i*m_nNumberOfCoord*3+j*3+1];
				}
			fout << endl;
			}
		fout << endl;

		fout << szString << "(z)\t";
		for(i = 0; i < m_nNumberOfKey; i++)
			fout << i << "\t";
		fout << endl;
		for(j = 0; j < m_nNumberOfCoord; j++)
			{
			fout << j;
			if(m_a_bQFlag_kv[j*3+2] == true)
				{
				for(i = 0; i < m_nNumberOfKey; i++)
					fout << "\t" << m_a_nQKeyValue[i*m_nNumberOfCoord*3+j*3+2];
				}
			else
				{
				i = 0;
				fout << "\t" << m_a_nQKeyValue[i*m_nNumberOfCoord*3+j*3+2];
				}
			fout << endl;
			}
		fout.close();
		}

private:
	// Max, Min: ִ밪, ּҰ ϱ
	void getDMaxDMin(double* a_dKeyValue)
		{
		double fX = 0;						// X 
		double fY = 0;						// Y 
		double fZ = 0;						// Z 
		bool bFirst = true;					// ó ÷ 

		m_dX_max = 0;
		m_dX_min = 0;
		m_dY_max = 0;
		m_dY_min = 0;
		m_dZ_max = 0;
		m_dZ_min = 0;

		for(int i = 0; i < m_nNumberOfKey; i++)
			{
			for(int j = 0; j < m_nNumberOfCoord; j++)
				{
				fX = a_dKeyValue[i*m_nNumberOfCoord*3+j*3];
				fY = a_dKeyValue[i*m_nNumberOfCoord*3+j*3+1];
				fZ = a_dKeyValue[i*m_nNumberOfCoord*3+j*3+2];

				if(bFirst == true)
					{
					m_dX_max = fX;
					m_dX_min = fX;
					m_dY_max = fY;
					m_dY_min = fY;
					m_dZ_max = fZ;
					m_dZ_min = fZ;
					bFirst = false;
					}
				else
					{
					if(fX > m_dX_max)
						m_dX_max = fX;
					if(fX < m_dX_min)
						m_dX_min = fX;

					if(fY > m_dY_max)
						m_dY_max = fY;
					if(fY < m_dY_min)
						m_dY_min = fY;

					if(fZ > m_dZ_max)
						m_dZ_max = fZ;
					if(fZ < m_dZ_min)
						m_dZ_min = fZ;
					}
				}
			}

		//*// 2001-12-19, New normalization
		double dXYZ_delta = fabs(m_dX_max - m_dX_min);
		m_nXYZ_max = 0;
		if(dXYZ_delta < fabs(m_dY_max - m_dY_min))
			{
			dXYZ_delta = fabs(m_dY_max - m_dY_min);
			m_nXYZ_max = 1;
			}
		if(dXYZ_delta < fabs(m_dZ_max - m_dZ_min))
			{
			dXYZ_delta = fabs(m_dZ_max - m_dZ_min);
			m_nXYZ_max = 2;
			}
		m_dX_max = m_dX_min + dXYZ_delta;
		m_dY_max = m_dY_min + dXYZ_delta;
		m_dZ_max = m_dZ_min + dXYZ_delta;
		m_dXYZ_delta = dXYZ_delta;
		//*/
		}

	// QMin, QMax: QMin, QMax 迭 ϱ
	void getQMinQMax(int* a_nQMin, int* a_nQMax, int* a_nQKeyValue, int nNumberOfKey, int nNumberOfCoord)
		{
		int nQX = 0;
		int nQY = 0;
		int nQZ = 0;
		int nQMin_X = 0;
		int nQMax_X = 0;
		int nQMin_Y = 0;
		int nQMax_Y = 0;
		int nQMin_Z = 0;
		int nQMax_Z = 0;
		bool bFirst = false;
		int i = 0;
		int j = 0;

		for(i = 0; i < nNumberOfCoord; i++)
			{
			bFirst = true;
			for(j = 0; j < nNumberOfKey; j++)
				{
				nQX = a_nQKeyValue[j*nNumberOfCoord*3+i*3];
				nQY = a_nQKeyValue[j*nNumberOfCoord*3+i*3+1];
				nQZ = a_nQKeyValue[j*nNumberOfCoord*3+i*3+2];

				if(bFirst == true)
					{
					nQMin_X = nQX;
					nQMax_X = nQX;
					nQMin_Y = nQY;
					nQMax_Y = nQY;
					nQMin_Z = nQZ;
					nQMax_Z = nQZ;
					bFirst = false;
					}
				else
					{
					if(nQX < nQMin_X)
						nQMin_X = nQX;
					else if(nQX > nQMax_X)
						nQMax_X = nQX;
					if(nQY < nQMin_Y)
						nQMin_Y = nQY;
					else if(nQY > nQMax_Y)
						nQMax_Y = nQY;
					if(nQZ < nQMin_Z)
						nQMin_Z = nQZ;
					else if(nQZ > nQMax_Z)
						nQMax_Z = nQZ;
					}
				}
			a_nQMin[i*3] = nQMin_X;
			a_nQMax[i*3] = nQMax_X;
			a_nQMin[i*3+1] = nQMin_Y;
			a_nQMax[i*3+1] = nQMax_Y;
			a_nQMin[i*3+2] = nQMin_Z;
			a_nQMax[i*3+2] = nQMax_Z;
			}
		}

	// Quantization: ü KeyValue ȭϱ
	void quantize(int* a_nQKeyValue, double* a_dKeyValue, int nQStep_keyValue)
		{
		double fKeyValue = 0;
		int nQKeyValue = 0;
		double fIQKeyValue = 0;

		for(int i = 0; i < m_nNumberOfCoord; i++)
			{
			for(int j = 0; j < m_nNumberOfKey; j++)
				{
				fKeyValue = a_dKeyValue[j*m_nNumberOfCoord*3+i*3];		// X
				nQKeyValue = quantizeKeyValue(fKeyValue, m_dX_max, m_dX_min, nQStep_keyValue);
				a_nQKeyValue[j*m_nNumberOfCoord*3+i*3] = nQKeyValue;

				fKeyValue = a_dKeyValue[j*m_nNumberOfCoord*3+i*3+1];	// Y
				nQKeyValue = quantizeKeyValue(fKeyValue, m_dY_max, m_dY_min, nQStep_keyValue);
				a_nQKeyValue[j*m_nNumberOfCoord*3+i*3+1] = nQKeyValue;

				fKeyValue = a_dKeyValue[j*m_nNumberOfCoord*3+i*3+2];	// Z
				nQKeyValue = quantizeKeyValue(fKeyValue, m_dZ_max, m_dZ_min, nQStep_keyValue);
				a_nQKeyValue[j*m_nNumberOfCoord*3+i*3+2] = nQKeyValue;
				}
			}
		}

	// Quantization: ϳ keyValue ȭϱ
	int quantizeKeyValue(double fKeyValue, double fMax, double fMin, int nQStep)
		{
		int nQKeyValue;			// ȭ 

		if(fKeyValue <= fMin)
			return 0;
		else if(fKeyValue >= fMax)
			return ((1 << nQStep) - 1);

		nQKeyValue = (int)(((fKeyValue-fMin) / (fMax-fMin)) * ((1 << nQStep) - 1) + 0.5);

		return nQKeyValue;
		}

	// InverseQuantization: ϳ keyValue  ȭϱ
	double inverseQuantizeKeyValue(int nQKeyValue, double fMax, double fMin, int nQStep)
		{			
		double fIQKeyValue;		// ȭ 
		
		if(nQKeyValue == 0)
			return fMin;
		else if(nQKeyValue == ((1 << nQStep) - 1))
			return fMax;
		else
			fIQKeyValue = (double)(fMin + ((double)nQKeyValue / ((1 << nQStep) - 1)) * (fMax - fMin));
		
		return fIQKeyValue;
		}

	// Temporal DPCM: KeyValue  ϱ.
	void dpcm_temporal(int* a_nQKeyValue_dpcm, int* a_nQKeyValue, int nCoord, int nIndex)
		{
		int nValue = 0;	// prediction value
		int nQ = 0;
		int nCQ = 0;
		int i = 0;

		CCQ oCQ;

		if(m_a_bQFlag_kv[nCoord*3+nIndex] == true)
			{
			nQ = a_nQKeyValue[i*m_nNumberOfCoord*3+nCoord*3+nIndex];
			// circular quantization
			nCQ = oCQ.quantizeFirstValue(nQ, m_a_nQMin[nCoord*3+nIndex], m_a_nQMax[nCoord*3+nIndex]);
			a_nQKeyValue_dpcm[i*m_nNumberOfCoord*3+nCoord*3+nIndex]	= nCQ;
			}
		else
			a_nQKeyValue_dpcm[i*m_nNumberOfCoord*3+nCoord*3+nIndex]
				= a_nQKeyValue[i*m_nNumberOfCoord*3+nCoord*3+nIndex];

		for(i = 1; i < m_nNumberOfKey; i++)
			{
			if(m_a_bQFlag_kv[nCoord*3+nIndex] == true)
				{
				nValue = a_nQKeyValue[(i-1)*m_nNumberOfCoord*3+nCoord*3+nIndex];

				// circular quantization
				nQ = a_nQKeyValue[i*m_nNumberOfCoord*3+nCoord*3+nIndex] - nValue;
				nCQ = oCQ.quantize(nQ, m_a_nQMin[nCoord*3+nIndex], m_a_nQMax[nCoord*3+nIndex]);
				a_nQKeyValue_dpcm[i*m_nNumberOfCoord*3+nCoord*3+nIndex] = nCQ;
				}
			else
				a_nQKeyValue_dpcm[i*m_nNumberOfCoord*3+nCoord*3+nIndex]
					= a_nQKeyValue[i*m_nNumberOfCoord*3+nCoord*3+nIndex];
			}
		}

	// Spatial DPCM: KeyValue  ϱ.
	void dpcm_spatial(int* a_nQKeyValue_dpcm, int* a_nQKeyValue, int nCoord, int nIndex, int nBase = -1)
		{
		int nValue = 0;	// prediction value
		int nQ = 0;
		int nCQ = 0;
		int i = 0;

		CCQ oCQ;

		for(i = 0; i < m_nNumberOfKey; i++)
			{
			if(m_a_bQFlag_kv[nCoord*3+nIndex] == true)
				{
				if(nCoord == 0)
					a_nQKeyValue_dpcm[i*m_nNumberOfCoord*3+nCoord*3+nIndex]
						= a_nQKeyValue[i*m_nNumberOfCoord*3+nCoord*3+nIndex];
				else
					{
					if(nBase == -1)		// if the parameter of nBase is empty, the base vertex is the previous vertex.
						nValue = a_nQKeyValue[i*m_nNumberOfCoord*3+(nCoord-1)*3+nIndex];
					else
						nValue = a_nQKeyValue[i*m_nNumberOfCoord*3+nBase*3+nIndex];

					// cutting-off
					if(nValue < m_a_nQMin[nCoord*3+nIndex])
						nValue = m_a_nQMin[nCoord*3+nIndex];
					else if(nValue > m_a_nQMax[nCoord*3+nIndex])
						nValue = m_a_nQMax[nCoord*3+nIndex];

					// circular quantization
					nQ = a_nQKeyValue[i*m_nNumberOfCoord*3+nCoord*3+nIndex] - nValue;
					nCQ = oCQ.quantize(nQ, m_a_nQMin[nCoord*3+nIndex], m_a_nQMax[nCoord*3+nIndex]);
					a_nQKeyValue_dpcm[i*m_nNumberOfCoord*3+nCoord*3+nIndex] = nCQ;
					}
				}
			else
				a_nQKeyValue_dpcm[i*m_nNumberOfCoord*3+nCoord*3+nIndex]
					= a_nQKeyValue[i*m_nNumberOfCoord*3+nCoord*3+nIndex];
			}
		}

	// Temporal and spatial DPCM: KeyValue  ϱ.
	void dpcm_tps(int* a_nQKeyValue_dpcm, int* a_nQKeyValue, int nCoord, int nIndex, int nBase = -1)
		{
		int nValue = 0;	// prediction value
		int nQ = 0;
		int nCQ = 0;
		int i = 0;

		CCQ oCQ;

		if(m_a_bQFlag_kv[nCoord*3+nIndex] == true)
			{
			nQ = a_nQKeyValue[i*m_nNumberOfCoord*3+nCoord*3+nIndex];
			// circular quantization
			nCQ = oCQ.quantizeFirstValue(nQ, m_a_nQMin[nCoord*3+nIndex], m_a_nQMax[nCoord*3+nIndex]);
			a_nQKeyValue_dpcm[i*m_nNumberOfCoord*3+nCoord*3+nIndex] = nCQ;
			}
		else
			a_nQKeyValue_dpcm[i*m_nNumberOfCoord*3+nCoord*3+nIndex]
				= a_nQKeyValue[i*m_nNumberOfCoord*3+nCoord*3+nIndex];

		for(i = 1; i < m_nNumberOfKey; i++)
			{
			if(m_a_bQFlag_kv[nCoord*3+nIndex] == true)
				{
				if(nCoord == 0)
					a_nQKeyValue_dpcm[i*m_nNumberOfCoord*3+nCoord*3+nIndex]
						= a_nQKeyValue[i*m_nNumberOfCoord*3+nCoord*3+nIndex];
				else
					{
					if(nBase == -1)		// if the parameter of nBase is empty, the base vertex is the previous vertex.
						nValue = a_nQKeyValue[(i-1)*m_nNumberOfCoord*3+nCoord*3+nIndex]
								+ (a_nQKeyValue[i*m_nNumberOfCoord*3+(nCoord-1)*3+nIndex] 
								- a_nQKeyValue[(i-1)*m_nNumberOfCoord*3+(nCoord-1)*3+nIndex]);
					else
						nValue = a_nQKeyValue[(i-1)*m_nNumberOfCoord*3+nCoord*3+nIndex]
								+ (a_nQKeyValue[i*m_nNumberOfCoord*3+nBase*3+nIndex] 
								- a_nQKeyValue[(i-1)*m_nNumberOfCoord*3+nBase*3+nIndex]);

					// cutting-off
					if(nValue < m_a_nQMin[nCoord*3+nIndex])
						nValue = m_a_nQMin[nCoord*3+nIndex];
					else if(nValue > m_a_nQMax[nCoord*3+nIndex])
						nValue = m_a_nQMax[nCoord*3+nIndex];

					// circular quantization
					nQ = a_nQKeyValue[i*m_nNumberOfCoord*3+nCoord*3+nIndex] - nValue;
					nCQ = oCQ.quantize(nQ, m_a_nQMin[nCoord*3+nIndex], m_a_nQMax[nCoord*3+nIndex]);
					a_nQKeyValue_dpcm[i*m_nNumberOfCoord*3+nCoord*3+nIndex] = nCQ;
					}
				}
			else
				a_nQKeyValue_dpcm[i*m_nNumberOfCoord*3+nCoord*3+nIndex]
					= a_nQKeyValue[i*m_nNumberOfCoord*3+nCoord*3+nIndex];
			}
		}

	// Round Off: Digit Ͽ ݿøϱ
	double roundOffWithDigit(double dSource, int nDigit)
		{
		int nDecimal = 0;
		int nSign = 0;
		char* szBuffer = NULL;
		double dValue = 0;

		szBuffer = _fcvt(dSource, 10, &nDecimal, &nSign);
		if(nSign == 0)
			dValue = (double)int(dSource*pow(10, nDigit-nDecimal)+0.5)/pow(10, nDigit-nDecimal);
		else
			dValue = (double)int(dSource*pow(10, nDigit-nDecimal)-0.5)/pow(10, nDigit-nDecimal);
		//cout << dSource << " -> " << dValue << endl;

		return dValue;
		}

	// Moving: KeyValue ϱ
	void moveQKeyValue(int* a_nQKeyValue_destination, int* a_nQKeyValue_source)
		{
		for(int i = 0; i < m_nNumberOfKey; i++)
			{
			for(int j = 0; j < m_nNumberOfCoord; j++)
				{
				a_nQKeyValue_destination[i*m_nNumberOfCoord*3+j*3] = a_nQKeyValue_source[i*m_nNumberOfCoord*3+j*3];
				a_nQKeyValue_destination[i*m_nNumberOfCoord*3+j*3+1] = a_nQKeyValue_source[i*m_nNumberOfCoord*3+j*3+1];
				a_nQKeyValue_destination[i*m_nNumberOfCoord*3+j*3+2] = a_nQKeyValue_source[i*m_nNumberOfCoord*3+j*3+2];
				}
			}
		}

	// Moving: Component KeyValue ϱ
	void moveQKeyValue(int* a_nQKeyValue_destination, int* a_nQKeyValue_source, int nCoord, int nIndex)
		{
		for(int i = 0; i < m_nNumberOfKey; i++)
			{
			switch(nIndex)
				{
				case 0:
					a_nQKeyValue_destination[i*m_nNumberOfCoord*3+nCoord*3] = a_nQKeyValue_source[i*m_nNumberOfCoord*3+nCoord*3];
					break;

				case 1:
					a_nQKeyValue_destination[i*m_nNumberOfCoord*3+nCoord*3+1] = a_nQKeyValue_source[i*m_nNumberOfCoord*3+nCoord*3+1];
					break;

				case 2:
					a_nQKeyValue_destination[i*m_nNumberOfCoord*3+nCoord*3+2] = a_nQKeyValue_source[i*m_nNumberOfCoord*3+nCoord*3+2];
					break;
				}
			}
		}

	// Entropy:  Vertex Entropy ϱ
	double getEntropyOfVertex(int* a_nQKeyValue, int nCoord, int nIndex, int& nNumberOfSymbol)
		{
		double dEntropy = 0;			// ü Ʈ
		int* a_nQData = NULL;			// Է Ǵ 迭
		int i = 0;

		// Allocation: ޸ Ҵ
		a_nQData = new int[m_nNumberOfKey];
		memset((void*)a_nQData, 0, sizeof(int)*m_nNumberOfKey);

		// Entropy: Ʈ ϱ
		for(i = 0; i < m_nNumberOfKey; i++)
			{
			a_nQData[i] = a_nQKeyValue[i*m_nNumberOfCoord*3+nCoord*3+nIndex];
			}
		dEntropy = getEntropy(a_nQData, m_nNumberOfKey, nNumberOfSymbol);

		delete[] a_nQData;

		return dEntropy;
		}

	// Entropy: Ʈ ϴ Լ, 2001/9/27, (c) Shinjun Lee
	double getEntropy(int* a_nData, int nNumberOfData, int& nNumberOfSymbol)
		{
		// Variables: Max, Min
		double dEntropy = 0;		// Ʈ
		double dSum = 0;			// 
		int* a_nSymbol = NULL;		// Symbol
		int nMax = 0;				// Symbol  ִ밪
		int nMin = 0;				// Symbol  ּҰ
		//int nNumberOfSymbol = 0;	// Symbol 
		int nSymbol_previous = 0;	//  Symbol
		int nFrequencyOfSymbol = 0;	//  Symbol 󵵼
		int i = 0;

		// Allocation: symbol 迭
		a_nSymbol = new int[nNumberOfData];

		// Copying: ϱ
		// Max, Min: ִ밪, ּҰ ϱ
		for(i = 0; i < nNumberOfData; i++)
			{
			a_nSymbol[i] = a_nData[i];
			if(i == 0)
				nMax = nMin = a_nSymbol[i];
			if(a_nSymbol[i] > nMax)
				nMax = a_nSymbol[i];
			if(a_nSymbol[i] < nMin)
				nMin = a_nSymbol[i];
			}
			/*// Print: Max, Min
			cout << "Max = " << nMax << endl;
			cout << "Min = " << nMin << endl;
		//*/

		// Sort: ϱ
		qsort((void*)a_nSymbol, (size_t)nNumberOfData, sizeof(int), compareInt);
		/*// Print: ĵ symbol data
		for(i = 0; i < nNumberOfData; i++)
		cout << a_nSymbol[i] << endl;
		//*/

		// Counting: ü symbol  
		nNumberOfSymbol = 0;
		dSum = 0;
		for(i = 0; i < nNumberOfData; i++)
			{
			if(i == 0)
				{
				nNumberOfSymbol = 1;
				nSymbol_previous = a_nSymbol[i];
				nFrequencyOfSymbol = 1;
				}
			else
				{
				if(a_nSymbol[i] != nSymbol_previous)
					{
					dSum += ((double)nFrequencyOfSymbol/nNumberOfData)*(log10((double)nFrequencyOfSymbol/nNumberOfData)/log10(2));
					nNumberOfSymbol++;
					nFrequencyOfSymbol = 1;
					nSymbol_previous = a_nSymbol[i];
					}
				else
					{
					nFrequencyOfSymbol++;
					}
				}
			}
		// Entropy: Ʈ ϱ
		dSum += ((double)nFrequencyOfSymbol/nNumberOfData)*(log10((double)nFrequencyOfSymbol/nNumberOfData)/log10(2));
		dEntropy = -dSum * nNumberOfData;
		/*// Print: nNumberOfSymbol, dEntropy
		cout << "Number of symbol = " << nNumberOfSymbol << endl;
		cout << "Entropy = " << dEntropy << endl;
		//*/

		delete[] a_nSymbol;

		return dEntropy;
		}
	};
