// CMinMaxEncoder
// Author: Sang-oak woo
// Editor: Shinjun Lee (modified code into class)
// Date: 2001. 12. 27.

const int cnMAX_DIGIT = 15;
const double dEPSILONERROR = 0.0000000001;		// 10

class CMinMaxEncoder
	{
  public:
	void MakeBitstreamOfMinMax(CAC_QF& codec, double dMin, double dMax, int maxDigit, int& KTotal)
		{
		// Start key
		// mantissa digit 4 
		int nDigit = GetDigit(dMin); //  nDigit: ù°   digit
		// 0  digit = 0.
		if(nDigit == maxDigit) // maxDigit: ü digit
			{
			codec._ostrm->putbits(1, 1);
			KTotal += 1;
			}
		else
			{
			codec._ostrm->putbits(0, 1);
			KTotal += 1;
			codec._ostrm->putbits(nDigit, 4);
			KTotal += 4;
			}

		int mantissa;
		int preexponent;
		int count;

		if(nDigit != 0)
			{
			if(nDigit < 8)
				{
				// mantissa 10 
				int nSign = 0;
				if(dMin >= 0)
					codec._ostrm->putbit(0);
				else
					codec._ostrm->putbit(1);
				KTotal += 1;
				mantissa = GetMantissa(dMin, nDigit);
				count = getNbBits((int)pow(10.0, nDigit)-1);
				codec._ostrm->putbits(mantissa, count);
				KTotal += count;

				// exponent digit 6
				preexponent = GetLargestDigit(dMin);
				if(preexponent >= 0)
					codec._ostrm->putbit(0);
				else
					codec._ostrm->putbit(1);
				KTotal += 1;
				codec._ostrm->putbits(abs(preexponent), 6);
				KTotal += 6;
				}
			else
				{
				float flot = (float)dMin;
				codec._ostrm->putbits(*((int *)&flot), 32);
				KTotal += 32;
				}
			}

		// End key
		// mantissa digit 4
		nDigit =  GetDigit(dMax);
		if(nDigit == maxDigit)
			{
			codec._ostrm->putbits(1, 1);
			KTotal += 1;
			}
		else
			{
			codec._ostrm->putbits(0, 1);
			KTotal += 1;
			codec._ostrm->putbits(nDigit, 4);
			KTotal += 4;
			}							

		if(nDigit != 0)
			{
			if(nDigit < 8)
				{
				// mantissa 10 
				int nSign = 0;
				if(dMax >= 0)
					codec._ostrm->putbit(0);
				else
					codec._ostrm->putbit(1);
				KTotal += 1;
				mantissa = GetMantissa(dMax, nDigit);
				count = getNbBits((int)pow(10.0, nDigit)-1);
				codec._ostrm->putbits(mantissa, count);
				KTotal += count;

				// exponent 6 
				int exponent = GetLargestDigit(dMax);
				if(exponent == preexponent)
					{
					codec._ostrm->putbits(1, 1);
					KTotal += 1;
					}
				else
					{
					codec._ostrm->putbits(0, 1);
					KTotal += 1;

					if(exponent >= 0)
						codec._ostrm->putbit(0);
					else
						codec._ostrm->putbit(1);
					KTotal += 1;
					codec._ostrm->putbits(abs(exponent), 6);
					KTotal += 6;
					}
				}
			else
				{
				const float flot = (float)dMax;
				codec._ostrm->putbits(*((int *)&flot), 32);
				KTotal += 32;
				}
			}
		}

	// sowoo 2001.10.12 (Min/Max Reduce)
	void EncodeReducedMinMaxData(
		CAC_QF& codec,
		double dXMin, double dYMin, double dZMin,
		double dXYZ_max,

		// sowoo 2001.11.24
		// ̹  KeyValueDigit  digit   ʴ´.
		int nKeyValueDigit,
		int nWhichAxis,
		int& KVTotal
		)
		{
		//int nUsedBits = 0;
		//int nHeaderBits, nXMinBits, nYMinBits, nZMinBits, nDiffBits;
		int nBits;

		/////////////////////////////////////////////////
		//
		// Part 1. ʿ  Ѵ.
		//
		/////////////////////////////////////////////////

		// 1. Mantissa Digit(x, y, z) Ѵ.
		int nX_MantissaDigit, nY_MantissaDigit, nZ_MantissaDigit, nMax_MantissaDigit;

		nX_MantissaDigit = GetDigit(dXMin);
		nY_MantissaDigit = GetDigit(dYMin);
		nZ_MantissaDigit = GetDigit(dZMin);
		nMax_MantissaDigit = GetDigit(dXYZ_max);

		///////////////////////////////////////////////////
		//
		//	ҸǴ bit Ͽ 128̸̻,
		//	  .
		//
		///////////////////////////////////////////////////
		if(GetTotalMinMaxBits(
			nX_MantissaDigit, nY_MantissaDigit, nZ_MantissaDigit,
			nMax_MantissaDigit) > 128)
			{
			codec._ostrm->putbit(0);
			KVTotal += 1;

			float fTemp;

			fTemp = (float) dXMin;
			codec._ostrm->putbits(*((int*)&fTemp), 32);
			KVTotal += 32;

			fTemp = (float) dYMin;
			codec._ostrm->putbits(*((int*)&fTemp), 32);
			KVTotal += 32;

			fTemp = (float) dZMin;
			codec._ostrm->putbits(*((int*)&fTemp), 32);
			KVTotal += 32;

			fTemp = (float) dXYZ_max;
			codec._ostrm->putbits(*((int*)&fTemp), 32);
			KVTotal += 32;

			return;
			}
		else
			{
			codec._ostrm->putbit(1);
			}

		// 3.1.0   max ?
		codec._ostrm->putbits(nWhichAxis, 2);

		bool bNeedDiffDigit = true;

		switch(nWhichAxis) {
			case 0: // X
				if(nMax_MantissaDigit == nX_MantissaDigit)
					bNeedDiffDigit = false;
				break;

			case 1: // Y
				if(nMax_MantissaDigit == nY_MantissaDigit)
					bNeedDiffDigit = false;
				break;

			case 2: // Z
				if(nMax_MantissaDigit == nZ_MantissaDigit)
					bNeedDiffDigit = false;
				break;
			}

		// 2. Exponent Digit Sign  Ѵ.
		int nX_ExponentDigit, nY_ExponentDigit, nZ_ExponentDigit, nMax_ExponentDigit;
		int nX_ExponentSign, nY_ExponentSign, nZ_ExponentSign, nMax_ExponentSign;

		nX_ExponentDigit = GetLargestDigit(dXMin);
		nY_ExponentDigit = GetLargestDigit(dYMin);
		nZ_ExponentDigit = GetLargestDigit(dZMin);
		nMax_ExponentDigit = GetLargestDigit(dXYZ_max);

		nX_ExponentSign = (nX_ExponentDigit < 0)? 1: 0;
		nY_ExponentSign = (nY_ExponentDigit < 0)? 1: 0;
		nZ_ExponentSign = (nZ_ExponentDigit < 0)? 1: 0;
		nMax_ExponentSign = (nMax_ExponentDigit < 0)? 1: 0;

		/////////////////////////////////////////////////
		//
		// Part 2.   EncodingѴ.
		//
		/////////////////////////////////////////////////

		// 3. Float-Point Number ڵѴ.

		//nHeaderBits = AACToTalBits;
		// 3.1 [Header Coding *******************************************]

		// 3.1.1   Digit ϴ°?
		bool bAllSameDigit = false;

		GetDigitBits(nX_MantissaDigit, nY_MantissaDigit, nZ_MantissaDigit, bAllSameDigit);

		codec._ostrm->putbit((bAllSameDigit)? 1: 0);

		// 3.1.2  Digit ƴϸ, Digit ش.
		if(bAllSameDigit)
			{
			if(nKeyValueDigit == nX_MantissaDigit)
				{
				codec._ostrm->putbit(1);
				KVTotal += 1;
				}
			else
				{
				codec._ostrm->putbit(0);
				KVTotal += 1;
				//codec._ostrm->putbits(nX_MantissaDigit, 3);
				codec._ostrm->putbits(nX_MantissaDigit, 4);
				KVTotal += 4;
				}
			nY_MantissaDigit = nX_MantissaDigit;
			nZ_MantissaDigit = nX_MantissaDigit;
			}
		else
			{
			//codec._ostrm->putbits(nX_MantissaDigit, 3);
			//codec._ostrm->putbits(nY_MantissaDigit, 3);
			//codec._ostrm->putbits(nZ_MantissaDigit, 3);
			codec._ostrm->putbits(nX_MantissaDigit, 4);
			codec._ostrm->putbits(nY_MantissaDigit, 4);
			codec._ostrm->putbits(nZ_MantissaDigit, 4);
			KVTotal += 12;
			}

		// 3.1.2.1
		// Digit ̰ 2̸̻ Diff Digit  .
		// 2̸̻ 2bit̻ ̵   ִ.
		if(bNeedDiffDigit)
			{
			codec._ostrm->putbit(1);
			KVTotal += 1;
			//codec._ostrm->putbits(nMax_MantissaDigit, 3);
			codec._ostrm->putbits(nMax_MantissaDigit, 4);
			KVTotal += 4;
			}
		else
			{
			codec._ostrm->putbit(0);
			KVTotal += 1;
			//nMax_MantissaDigit = nX_MantissaDigit;
			// 2001-1-8, shinjun: {
			switch(nWhichAxis)
				{
				case 0:
					nMax_MantissaDigit = nX_MantissaDigit;
					break;
				case 1:
					nMax_MantissaDigit = nY_MantissaDigit;
					break;
				case 2:
					nMax_MantissaDigit = nZ_MantissaDigit;
					break;
				}
			// 2001-1-8, shinjun: }
			}

		// 3.1.3 Exponent Digit
		int nLargestExponentDigit = abs(nX_ExponentDigit);

		if(nLargestExponentDigit < abs(nY_ExponentDigit))
			nLargestExponentDigit = abs(nY_ExponentDigit);
		if(nLargestExponentDigit < abs(nZ_ExponentDigit))
			nLargestExponentDigit = abs(nZ_ExponentDigit);
		if(nLargestExponentDigit < abs(nMax_ExponentDigit))
			nLargestExponentDigit = abs(nMax_ExponentDigit);

		// [0 ~ 38]   CodingϹǷ 6 bit
		codec._ostrm->putbits(nLargestExponentDigit, 6);
		KVTotal += 6;

		// 3.1.4 Exponent Sign  ?
		bool bAllSameSign = false;

		if(nX_ExponentSign == nY_ExponentSign)
			{
			if(nX_ExponentSign == nZ_ExponentSign)
				{
				if(nX_ExponentSign == nMax_ExponentSign)
					{
					bAllSameSign = true;
					}
				}
			}

		if(bAllSameSign)
			{
			//   Sign̴.
			codec._ostrm->putbit(1);
			KVTotal += 1;
			
			//   sign Ѵ.
			codec._ostrm->putbit(nX_ExponentSign);
			KVTotal += 1;
			}
		else
			{
			//   Sign ƴϴ.
			codec._ostrm->putbit(0);
			KVTotal += 1;
			}
		//nHeaderBits = AACToTalBits - nHeaderBits;

		//nXMinBits = AACToTalBits;
		// 3.2 [X MIN *******************************************]

		int nMantissa;
		int nExponent;

		// digit '0'̶  0.0 en/decoding϶ !
		if(nX_MantissaDigit != 0)
			{
			// 3.2.1 Sign Coding
			codec._ostrm->putbit((dXMin < 0)? 1: 0);
			KVTotal += 1;
			
			// 3.2.2 Mantissa Coding
			nExponent = GetLargestDigit(dXMin);
			
			nMantissa = GetMantissa(dXMin, nX_MantissaDigit);
			nBits = (int) (log(pow(10, nX_MantissaDigit) - 1)/log(2) + 1);
			
			if(nMantissa < 0)
				nMantissa = -nMantissa;
			codec._ostrm->putbits(nMantissa, nBits);
			KVTotal += nBits;
			
			// 3.2.3 Exponent Coding
			nBits = getNbBits(nLargestExponentDigit);
			codec._ostrm->putbits(abs(nExponent), nBits);
			KVTotal += nBits;
			
			if(!bAllSameSign)
				{
				codec._ostrm->putbit(nX_ExponentSign);
				KVTotal += 1;
				}
			}

		//nXMinBits = AACToTalBits - nXMinBits;

		//nYMinBits = AACToTalBits;
		// 3.3 [Y MIN *******************************************]

		// digit '0'̶  0.0 en/decoding϶ !
		if(nY_MantissaDigit != 0)
			{
			// 3.3.1 Sign Coding
			codec._ostrm->putbit((dYMin < 0)? 1: 0);
			KVTotal += 1;

			nExponent = GetLargestDigit(dYMin);

			// 3.3.2 Mantissa Coding
			nMantissa = GetMantissa(dYMin, nY_MantissaDigit);
			nBits = (int) (log(pow(10, nY_MantissaDigit) - 1)/log(2) + 1);

			if(nMantissa < 0)
				nMantissa = -nMantissa;
			codec._ostrm->putbits(nMantissa, nBits);
			KVTotal += nBits;

			// 3.3.3 Exponent Coding

			nBits = getNbBits(nLargestExponentDigit);
			codec._ostrm->putbits(abs(nExponent), nBits);
			KVTotal += nBits;

			if(!bAllSameSign)
				{
				codec._ostrm->putbit(nY_ExponentSign);
				KVTotal += 1;
				}
			}

		//nYMinBits = AACToTalBits - nYMinBits;

		//nZMinBits = AACToTalBits;
		// 3.4 [Z MIN *******************************************]

		// digit '0'̶  0.0 en/decoding϶ !
		if(nZ_MantissaDigit != 0)
			{
			// 3.4.1 Sign Coding
			codec._ostrm->putbit((dZMin < 0)? 1: 0);
			KVTotal += 1;

			nExponent = GetLargestDigit(dZMin);

			// 3.4.2 Mantissa Coding
			nMantissa = GetMantissa(dZMin, nZ_MantissaDigit);
			nBits = (int) (log(pow(10, nZ_MantissaDigit) - 1)/log(2) + 1);

			if(nMantissa < 0)
				nMantissa = -nMantissa;
			codec._ostrm->putbits(nMantissa, nBits);
			KVTotal += nBits;

			// 3.4.3 Exponent Coding

			nBits = getNbBits(nLargestExponentDigit);
			codec._ostrm->putbits(abs(nExponent), nBits);
			KVTotal += nBits;

			if(!bAllSameSign)
				{
				codec._ostrm->putbit(nZ_ExponentSign);
				KVTotal += 1;
				}
			}

		//nZMinBits = AACToTalBits - nZMinBits;

		//nDiffBits = AACToTalBits;
		// 3.4 [Max Diff *******************************************]

		// digit '0'̶  0.0 en/decoding϶ !
		if(nMax_MantissaDigit != 0)
			{
			// 3.4.1 Sign Coding
			codec._ostrm->putbit((dXYZ_max < 0)? 1: 0);
			KVTotal += 1;

			nExponent = GetLargestDigit((double)dXYZ_max);

			// 3.4.2 Mantissa Coding
			nMantissa = GetMantissa(dXYZ_max, nMax_MantissaDigit);
			nBits = (int) (log(pow(10, nMax_MantissaDigit) - 1)/log(2) + 1);

			if(nMantissa < 0)
				nMantissa = -nMantissa;
			codec._ostrm->putbits(nMantissa, nBits);
			KVTotal += nBits;

			// 3.4.3 Exponent Coding

			nBits = getNbBits(nLargestExponentDigit);
			codec._ostrm->putbits(abs(nExponent), nBits);
			KVTotal += nBits;
			//codec._ostrm->putbit((nExponent < 0)? 1: 0); // 2001-12-26, shinjun

			if(!bAllSameSign)
				{
				codec._ostrm->putbit(nMax_ExponentSign);
				KVTotal += 1;
				}
			}

		//nDiffBits = AACToTalBits - nDiffBits;

		//nUsedBits = AACToTalBits - nUsedBits;

		//////////////////////////////////////////////////////////////
		//
		//				ϴ κ
		//
		//////////////////////////////////////////////////////////////

		/*ofstream file;
		file.open("MINMAX.TXT", ios::app);
		
		  //file << argv[2] << '\t';
		  file << dXMin << '\t';
		  file << dYMin << '\t';
		  file << dZMin << '\t';
		  file << dXYZ_max << '\t';
		  file << GetDigit(dXMin) << '\t';
		  file << nY_MantissaDigit << '\t';
		  file << nZ_MantissaDigit << '\t';
		  file << nMax_MantissaDigit << '\t';
		  file << nHeaderBits << '\t';
		  file << nXMinBits << '\t';
		  file << nYMinBits << '\t';
		  file << nZMinBits << '\t';
		  file << nDiffBits << '\t';
		  file << 128 << '\t';
		  file << nUsedBits << endl;
		  
			file.close();
			
			  ////////////////////////////////////////////
			  // 
			  // Verity Routine
			  //
			  ////////////////////////////////////////////
			  /*
			  ofstream fileV;
			  fileV.open("VerifyMinMax.Encoder", ios::app);
			  
				fileV << dXMin << '\t';
				fileV << dYMin << '\t';
				fileV << dZMin << '\t';
				fileV << dXYZ_max << endl;
		*/
		//
		////////////////////////////////////////////
		
		//exit(0);
		}

  private:
	int GetLeadingZero(double key)
		{
		//////////////////////////////////////////////////
		//
		//			Original Value 7ڸ !!!
		//			꿡 ؼ   
		//
		//////////////////////////////////////////////////
		
		char buffer[50];
		_gcvt(key, cnMAX_DIGIT, buffer);
		key = atof(buffer);
		
		//
		//////////////////////////////////////////////////
		
		double a;
		double e = dEPSILONERROR;
		__int64 nMulti = 10;
		double dMulti = 0.1;
		int nCount = 0;
		
		double dTemp;
		
		// 1. 켱  ٲٰ,
		a = fabs(key);
		
		// 2. 0̰ų 0.1 ũ տ "0" .
		if(a >= 1.0)
			{
			nCount++;
			while(a*dMulti > 1.0)
				{
				nCount++;
				dMulti *= 0.1;
				}
			
			return -nCount;
			}
		else if(a == 0.0)
			return 0;
		else if(a >= 0.1)
			return 0;
		else
			{
			nCount++;
			dTemp = a * nMulti;
			_gcvt(dTemp, 7, buffer);
			dTemp = ::atof(buffer);
			
			//while(a*nMulti < 0.1)
			while( dTemp < 0.1)
				{
				nCount++;
				nMulti *= 10;
				dTemp = a * nMulti;
				_gcvt(dTemp, 7, buffer);
				dTemp = ::atof(buffer);
				}
			
			return nCount;
			}
		}
	
	int GetLargestDigit(double key)
		{
		//////////////////////////////////////////////////
		//
		//			Original Value 7ڸ !!!
		//			꿡 ؼ   
		//
		//////////////////////////////////////////////////
		
		char buffer[50];
		_gcvt(key, cnMAX_DIGIT, buffer);
		key = atof(buffer);
		
		//
		//////////////////////////////////////////////////
		
		double a;
		__int64 mult;
		double dmult;
		__int64 Ia;
		double e = dEPSILONERROR;
		int nLargestDigit;
		
		mult=1; dmult=0.1;
		nLargestDigit = 0;
		
		a = fabs(key);
		Ia = (__int64) (a + mult*e);
		
		// sowoo 2001.10.11 (Digit   ϱ)
		// "0.1" binary ǥ ⶧ ǵ  ʴ  .
		// 0.1 ϴ  ſ 10  scheme !
		if(a == 0.0)
			return 1;
		
		double dTemp = a - (double)Ia;
		
		if((dTemp == 0.0) || (dTemp < e))
			{
			while(true)
				{
				a = fabs(key);
				a *= dmult;
				Ia = (__int64) (a + e);
				dTemp = a - (double)Ia;
				
				if(dTemp > e)
					return nLargestDigit;
				
				dmult *= 0.1;
				nLargestDigit--;
				}
			}
		
		//while(a - (double) Ia > e*mult)
		while(dTemp > e*mult)
			{
			mult*=10;
			nLargestDigit++;
			a = fabs(key);
			a=(double) mult *a;
			Ia = (__int64) (a + mult*e);
			dTemp = a - (double)Ia;
			}
		
		return nLargestDigit;
		}
	
	// sowoo 2001.10.11 (Min/Max Reduce)
	// Ǽ Digit 
	int GetDigit(double dValue)
		{
		//////////////////////////////////////////////////
		//
		//			Original Value 7ڸ !!!
		//			꿡 ؼ   
		//
		//////////////////////////////////////////////////
		
		char buffer[50];
		_gcvt(dValue, cnMAX_DIGIT, buffer);
		dValue = atof(buffer);
		
		//
		//////////////////////////////////////////////////
		
		// "0.0"  ڸ̴.
		if(dValue == 0.0)
			return 1;
		
		int nFirstDigit, nLastDigit;
		
		nFirstDigit = GetLeadingZero(dValue);
		nLastDigit = GetLargestDigit(dValue);
		
		int nDigit = nLastDigit - nFirstDigit;
		
		// 7 ū    .
		//  floating-point number ϴ ȿڸ 6~7!
		// ƴ,  ̻   .
		return (nDigit > cnMAX_DIGIT)? cnMAX_DIGIT: nDigit;
		}
	
	int GetTotalMinMaxBits(
		int nDigit_X, int nDigit_Y, int nDigit_Z,
		int nDigit_diff
		)
		{
		int nTemp;
		
		nTemp = 4 + (int) (log(pow(10, nDigit_X) - 1)/log(2) + 1) + 7;
		nTemp += 4 + (int) (log(pow(10, nDigit_Y) - 1)/log(2) + 1) + 7;
		nTemp += 4 + (int) (log(pow(10, nDigit_Z) - 1)/log(2) + 1) + 7;
		nTemp += 4 + (int) (log(pow(10, nDigit_diff) - 1)/log(2) + 1) + 7;
		
		return nTemp;
		}
	
	// sowoo 2001.10.11 (Min/Max Reduce)
	//  Ȯ  ƴϴ.
	int GetDigitBits(int nXDigit, int nYDigit, int nZDigit, bool& bIsOneDigit)
		{
		int nBits;
		int nMaxDigit = nXDigit, nMinDigit = nXDigit;
		
		if(nMaxDigit < nYDigit)
			nMaxDigit = nYDigit;
		if(nMaxDigit < nZDigit)
			nMaxDigit = nZDigit;
		
		if(nMinDigit > nYDigit)
			nMinDigit = nYDigit;
		if(nMinDigit > nZDigit)
			nMinDigit = nZDigit;
		
		if(nMinDigit == nMaxDigit)
			{
			// 1.  digit !!!
			nBits = 1;
			bIsOneDigit = true;
			
			return nBits;
			}
		else
			{
			bIsOneDigit = false;
			
			return 1 + 3 + 3;
			/*		int nDiffDigit = nMaxDigit - nMinDigit;
			
			  // 2. digit   ̵ .   ʴ´.
			  if(nDiffDigit == 1)
			  {
			  nBits = 1;
			  bIsOneDigit = true;
			  
				return nBits;
				}
				
				  nBits = 1 + 3 + 3;
			*/	}
			
			return nBits;
		}
	
	// double Mantissa(10) ϴ α׷!!!
	// input:
	// -> dValue: Source floating point 
	// -> nDigit:  ڸ
	int GetMantissa(double dValue, int nDigit)
		{
		char* buffer;
		int nDummy1 = 0, nDummy2 = 0;
		
		buffer = _ecvt(dValue, nDigit, &nDummy1, &nDummy2);
		
		return ::atoi(buffer);
		}

	int getNbBits(int k)
		{
		int j=1;
		while (k >= (1<<j) ) 
			{
			j++;
			}
		return j;
		}
	};
