// CoordIDPCMMode.h: coordinate key value encoder
// Author: Shinjun Lee, Samsung AIT
// 4th revision: 2002-04-25

#ifndef _COORDIDPCMMODE_H_
#define _COORDIDPCMMODE_H_

#include "../include/Entropy.h"

class CoordIDPCMMode
	{
  public:
	// variables: bitstream data of key value
	bool		bAddressOfDPCMMode[27];						// Dictionary address table
	bool*		bIndexDPCMMode;								// Dictionary index table

	// variables: nonparsable data
	CAC dpcmModeDicAddressContext;			// TS mode: mode for temporal or spatial or temporal-spatial dpcm
	CAC dpcmModeDicIndexContext;

  public:
	CoordIDPCMMode(void)
		{
		memset((void*)bAddressOfDPCMMode, 0, sizeof(bool)*27);
		bIndexDPCMMode = NULL;
		}

	~CoordIDPCMMode()
		{
		delete[] bIndexDPCMMode;
		}

	void InitializeModel(void)
		{
		// TS mode: mode for temporal or spatial or temporal-spatial dpcm
		dpcmModeDicAddressContext.reset();
		dpcmModeDicIndexContext.reset();
		}

	void makeBitstream(CAC_QF& codec, CoordIKeyValueHeader& coordIKVHeader, int* nDPCMMode)
		{
		int			nNumberOfSymbol = 0;					// the number of symbols
		int			i = 0;									// index
		int			j = 0;									// index
		int			k = 0;									// index

		//ofstream fout_dm;
		//fout_dm.open("DPCMMode_kv.txt");

		// allocate: bIndexDPCMMode
		bIndexDPCMMode = new bool[coordIKVHeader.nNumberOfCoord];
		memset((void*)bIndexDPCMMode, 0, sizeof(bool)*coordIKVHeader.nNumberOfCoord);

		// initialize: address
		for(j = 1; j < coordIKVHeader.nNumberOfCoord; j++)
			{
			int nIndex = 9*(nDPCMMode[j*3+2]-1)+3*(nDPCMMode[j*3+1]-1)+(nDPCMMode[j*3]-1);
			bAddressOfDPCMMode[nIndex] = true;
			}

		// send: address
		for(i = 0; i < 27; i++)
			{
			if(bAddressOfDPCMMode[i] == true)
				{
				codec.encode(1, dpcmModeDicAddressContext);
				nNumberOfSymbol++;
				}
			else
				codec.encode(0, dpcmModeDicAddressContext);
			}

		// encode: position index
		int	nIndex_address = 0;						// Address index
		bIndexDPCMMode = new bool[coordIKVHeader.nNumberOfCoord];
		memset((void*)bIndexDPCMMode, 0, sizeof(bool)*coordIKVHeader.nNumberOfCoord);
		for(int s = 0; s < nNumberOfSymbol; s++)			// repeat as the number of symbols
			{
			int nQKV = 0;									// symbol
			int nValue = 0;									// comparing value

			while(bAddressOfDPCMMode[nIndex_address] != true)
				nIndex_address++;
			nQKV = nIndex_address;
			//fout_dm << nQKV << ": ";

			if(s < nNumberOfSymbol - 1)
				nIndex_address++;
			else // if last symbol, break out
				break;

			// a_bDic_index: index position coding
			for(k = 1; k < coordIKVHeader.nNumberOfCoord; k++)
				{
				if(bIndexDPCMMode[k] == 0)
					{
					nValue = 9*(nDPCMMode[k*3+2]-1)+3*(nDPCMMode[k*3+1]-1)+(nDPCMMode[k*3]-1);
					if(nQKV == nValue)
						{
						bIndexDPCMMode[k] = 1;
						codec.encode(1, dpcmModeDicIndexContext);
						//fout_dm << 1 << flush;
						}
					else
						{
						codec.encode(0, dpcmModeDicIndexContext);
						//fout_dm << 0 << flush;
						}
					}
				}
			//fout_dm << endl;
			}
		//fout_dm.close();
		delete[] bIndexDPCMMode;
		bIndexDPCMMode = NULL;
		}

	void encodingProcess(CoordIKeyValueHeader& coordIKVHeader)
		{
		// No codes.
		}
	};

#endif // _COORDIDPCMMODE_H_
