// CoordIIncrementalMode.h: coordinate key value encoder
// Author: Shinjun Lee, Samsung AIT
// 4th revision: 2002-04-25

#ifndef _COORDIINCREMENTALMODE_H_
#define _COORDIINCREMENTALMODE_H_

#include <limits.h>

#include "../include/Entropy.h"

class CoordIIncrementalMode
	{
  public:
	// variables: bitstream data of key value
	bool*			bAddress;								// dictionary address index
	bool*			bAddrIndex;								// dictionary index flag
	int				nTrueOne;								// the value of one

	// variables: nonparsable data
	CAC dicAddressContext;							// Dictionary address

  public:
	CoordIIncrementalMode(void)
		{
		bAddress = NULL;
		bAddrIndex = NULL;
		nTrueOne = 1;
		}

	~CoordIIncrementalMode()
		{
		delete[] bAddress;
		delete[] bAddrIndex;
		}

	void InitializeModel(void)
		{
		// Dictionary address
		dicAddressContext.reset();
		}

	void makeBitstream(CAC_QF& codec, int* nKVACodingBit, int nNumberOfKey, int c, int j, CoordIKeyValueHeader& coordIKVHeader, int* nQKeyValue, int nNumberOfSymbol)
		{
		int			nSizeOfAddress = 0;						// the size of address
		int			nZero = 0;								// the value of zero
		int			i = 0;									// index
		int			k = 0;									// index

		// calculate: the size of address
		nSizeOfAddress = (int)pow(2, nKVACodingBit[j*3+c]+1) - 1;

		// allocate: dictionary address index
		bAddress = new bool[nSizeOfAddress];
		memset((void*)bAddress, 0, sizeof(bool)*nSizeOfAddress);

		// allocate: bAddrIndex
		bAddrIndex = new bool[nNumberOfKey];
		memset((void*)bAddrIndex, 0, sizeof(bool)*nNumberOfKey);

		// initialize: address
		for(i = 0; i < nNumberOfKey; i++)
			{
			int nIndex = 0;
			
			int nQKV = nQKeyValue[i*coordIKVHeader.nNumberOfCoord*3+j*3+c];
			if(nQKV == 0)
				nIndex = 0;
			else if(nQKV > 0)
				nIndex = 2*nQKV-1;
			else
				nIndex = -2*nQKV;
			
			bAddress[nIndex] = true;
			}

		// send: the values of address
		for(i = 0; i < nSizeOfAddress; i++)
			{
			if(bAddress[i] == true)
				codec.encode(1, dicAddressContext);
			else
				codec.encode(0, dicAddressContext);
			}

		// encode: incremental mode
		int nIndex_address = 0;
		for(int s = 0; s < nNumberOfSymbol; s++)	// repeat as the number of symbols
			{
			int nQKV = 0;		// quantized value

			while(bAddress[nIndex_address] != true)
				nIndex_address++;

			if(nIndex_address == 0)
				nQKV = 0;
			else if(nIndex_address % 2 != 0)
				nQKV = (nIndex_address+1)/2;
			else
				nQKV = -nIndex_address/2;

			if(s < nNumberOfSymbol - 1)
				nIndex_address++;
			else // if last symbol, break out
				break;

			// code: dictionary index
			int	nNumberOfOne = 0;			// the number of one flags
			int	nNumberOfZero = 0;			// the number of zero flags

			// count: the number of one flags and zero flags
			for(k = 0; k < nNumberOfKey; k++)
				{
				if(bAddrIndex[k] == 0)
					{
					nNumberOfZero++;
					if(nQKV == nQKeyValue[k*coordIKVHeader.nNumberOfCoord*3+j*3+c])
						nNumberOfOne++;
					}
				}

			nTrueOne = nNumberOfOne >= nNumberOfZero/2 ? 0 : 1;
			nZero = nTrueOne == 0 ? 1 : 0;

			// nOne: the real value of one
			codec.encode(nTrueOne, CQF_Status::dicOneContext);

			// code: dictionary index
			for(k = 0; k < nNumberOfKey; k++)
				{
				if(bAddrIndex[k] == 0)
					{
					if(nQKV == nQKeyValue[k*coordIKVHeader.nNumberOfCoord*3+j*3+c])
						{
						bAddrIndex[k] = 1;
						codec.encode(nTrueOne, CQF_Status::dicIndexContext);
						}
					else
						{
						codec.encode(nZero, CQF_Status::dicIndexContext);
						}
					}
				}
			}
		// delete: dictionary address index
		delete[] bAddress;
		bAddress = NULL;

		// delete: dictionary index
		delete[] bAddrIndex;
		bAddrIndex = NULL;
		}

	void encodingProcess(CoordIKeyValueHeader& coordIKVHeader)
		{
		// No codes.
		}
	};

#endif // _COORDIINCREMENTALMODE_H_
