// CoordIKeyValue.h: coordinate key value encoder
// Author: Shinjun Lee, Samsung AIT
// 4th revision: 2002-04-25

#ifndef _COORDIKEYVALUE_H_
#define _COORDIKEYVALUE_H_

#include "../include/CDPCM.h"
#include "../include/CoordIDPCMMode.h"
#include "../include/CoordIKeyValueDic.h"
#include "../include/CKeyValueEncoder.h"
#include "../include/ArithmeticEncoder.h"

class CoordIKeyValue
	{
  public:
	// variables: bitstream data of key value
	int*			nDPCMMode;
	bool*			bSelFlag;
	int*			nKVACodingBit;
	int*			nRefVertex;
	int*			nQMin;
	int*			nQMax;

	CoordIDPCMMode		coordIDPCMMode;
	CoordIKeyValueDic	coordIKeyValueDic;

	// variables: nonparsable data
	int				nNumberOfKey_org;
	int				nNumberOfCoord_org;

	int				nNumberOfKey_tr;
	int				nNumberOfCoord_tr;

	int*			nQKeyValue;								// quantized key value

	CDPCM			oDPCM_X_Min;							// DPCM processor
	CDPCM			oDPCM_Y_Min;							// DPCM processor
	CDPCM			oDPCM_Z_Min;							// DPCM processor

	CDPCM			oDPCM_X_Max;							// DPCM processor
	CDPCM			oDPCM_Y_Max;							// DPCM processor
	CDPCM			oDPCM_Z_Max;							// DPCM processor

	double*			dKeyValue;								// inversed quantized key value

	// contexts
	CAC selectionFlagContext;						// Selection flag: selection flag context
	CAC aqpXContext[32];							// Required QStep: RQP context
	CAC aqpYContext[32];
	CAC aqpZContext[32];
	CAC qMinContext[32];								// QMin: qmin for CQ
	CAC qMinSignContext;
	CAC qMaxContext[32];								// QMax: qmax for CQ
	CAC qMaxSignContext;
	CAC refContext[32];				// Base vertex index: Base vertex index

  public:
	CoordIKeyValue(void)
		{
		nDPCMMode = NULL;
		bSelFlag = NULL;
		nKVACodingBit = NULL;
		nRefVertex = NULL;
		nQMin = NULL;
		nQMax = NULL;

		nNumberOfKey_org = 0;
		nNumberOfCoord_org = 0;

		nNumberOfKey_tr = 0;
		nNumberOfCoord_tr = 0;

		nQKeyValue = NULL;
		dKeyValue = NULL;
		}

	~CoordIKeyValue()
		{
		delete[] nDPCMMode;
		delete[] bSelFlag;
		delete[] nKVACodingBit;
		delete[] nRefVertex;
		delete[] nQMin;
		delete[] nQMax;

		delete[] nQKeyValue;

		delete[] dKeyValue;
		}

	void InitializeModel(void)
		{
		int i=0;

		// Selection flag: selection flag context
		selectionFlagContext.reset();
		
		// Required QStep: RQP context
		for(i = 0; i < 32; i++)
			{
			aqpXContext[i].reset();
			aqpYContext[i].reset();
			aqpZContext[i].reset();
			}

		// QMin: qmin for CQ
		qMinSignContext.reset();
		for(i = 0; i < 32; i++)
			qMinContext[i].reset();
		
		// QMax: qmax for CQ
		qMaxSignContext.reset();
		for(i = 0; i < 32; i++)
			qMaxContext[i].reset();
		
		// Base vertex index
		for(i = 0; i < 32; i++)
			refContext[i].reset();

		coordIDPCMMode.InitializeModel();
		coordIKeyValueDic.InitializeModel();
		}

	void makeBitstream(CAC_QF& codec, CoordIKeyValueHeader& coordIKVHeader, int nNumberOfKey, int& nBitsOfAACKeyValue)
		{
		int			j = 0;									// index
		int			c = 0;									// index
		int			nKVACodingBitQBit = 0;					// QStep for QStep

		// get: nKVACodingBitQBit
		nKVACodingBitQBit = (int)(log10(abs(coordIKVHeader.nKVQBit))/log10(2))+1;

		int nBitsOfStart = (codec._ostrm->Outbuf.nbytes-1)*8 + (codec._ostrm->Outbuf.nbits-1);
		coordIDPCMMode.makeBitstream(codec, coordIKVHeader, nDPCMMode);
		for(j = 0; j < coordIKVHeader.nNumberOfCoord; j++)
			{
			for(c = 0; c < 3; c++)
				{
				if(c == 0)
					{
					if(coordIKVHeader.nXQMax_Min <= coordIKVHeader.nXQMin_Max)
						{
						codec.encode(bSelFlag[j*3+c], selectionFlagContext);
						}
					}
				else if(c == 1)
					{
					if(coordIKVHeader.nYQMax_Min <= coordIKVHeader.nYQMin_Max)
						{
						codec.encode(bSelFlag[j*3+c], selectionFlagContext);
						}
					}
				else if(c == 2)
					{
					if(coordIKVHeader.nZQMax_Min <= coordIKVHeader.nZQMin_Max)
						{
						codec.encode(bSelFlag[j*3+c], selectionFlagContext);
						}
					}

				if(bSelFlag[j*3+c] == true)
					{
					if(c == 0)
						encodeUnsignedAAC(codec, nKVACodingBit[j*3+c], nKVACodingBitQBit, aqpXContext);
					else if(c == 1)
						encodeUnsignedAAC(codec, nKVACodingBit[j*3+c], nKVACodingBitQBit, aqpYContext);
					else if(c == 2)
						encodeUnsignedAAC(codec, nKVACodingBit[j*3+c], nKVACodingBitQBit, aqpZContext);

					if(j > 0)
						{
						if(nDPCMMode[j*3+c] == DPCM_S || nDPCMMode[j*3+c] == DPCM_TS)
							{
							int nQBitOfRef = (int)(log10(abs(j-1))/log10(2))+1;
							encodeUnsignedAAC(codec, nRefVertex[j*3+c], nQBitOfRef, refContext);
							}
						}

					if(nKVACodingBit[j*3+c] != 0)
						{
						int nQMin = 0;
						if(c == 0)
							nQMin = oDPCM_X_Min.getData_dpcm(j);
						else if(c == 1)
							nQMin = oDPCM_Y_Min.getData_dpcm(j);
						else if(c == 2)
							nQMin = oDPCM_Z_Min.getData_dpcm(j);
						encodeSignedAAC(codec, nQMin, coordIKVHeader.nKVQBit+1,
							qMinSignContext, qMinContext);

						int nQMax = 0;
						if(c == 0)
							nQMax = oDPCM_X_Max.getData_dpcm(j);
						else if(c == 1)
							nQMax = oDPCM_Y_Max.getData_dpcm(j);
						else if(c == 2)
							nQMax = oDPCM_Z_Max.getData_dpcm(j);
						encodeSignedAAC(codec, nQMax, coordIKVHeader.nKVQBit+1,
							qMaxSignContext, qMaxContext);
						}
					}
				else
					{
					int nQMin = 0;
					if(c == 0)
						nQMin = oDPCM_X_Min.getData_dpcm(j);
					else if(c == 1)
						nQMin = oDPCM_Y_Min.getData_dpcm(j);
					else if(c == 2)
						nQMin = oDPCM_Z_Min.getData_dpcm(j);
					encodeSignedAAC(codec, nQMin, coordIKVHeader.nKVQBit+1,
						qMinSignContext, qMinContext);
					}

				coordIKeyValueDic.makeBitstream(codec, bSelFlag, nKVACodingBit, nNumberOfKey, c, j, coordIKVHeader, nQKeyValue);
				}
			}

		int nBitsOfEnd = (codec._ostrm->Outbuf.nbytes-1)*8 + (codec._ostrm->Outbuf.nbits-1);
		nBitsOfAACKeyValue = nBitsOfEnd - nBitsOfStart;
		}

	void encodingProcess(CoordIKeyValueHeader& coordIKVHeader, int& nNumberOfKey)
		{
		// transpose: information of keyValue whether the data is transposed
		nNumberOfKey_tr = nNumberOfKey_org = nNumberOfKey;
		nNumberOfCoord_tr = nNumberOfCoord_org = coordIKVHeader.nNumberOfCoord;
		if(coordIKVHeader.bTranspose == true)
			{
			nNumberOfKey_tr = nNumberOfKey = nNumberOfCoord_org;
			nNumberOfCoord_tr = coordIKVHeader.nNumberOfCoord = nNumberOfKey_org;
			transposeKeyValue(dKeyValue, dKeyValue, nNumberOfKey_org, nNumberOfCoord_org);
			}

		CKeyValueEncoder oKeyValueEncoder;
		oKeyValueEncoder.setKeyValueDigit(coordIKVHeader.nKVDigit);
		oKeyValueEncoder.encode(dKeyValue, nNumberOfKey, coordIKVHeader.nNumberOfCoord, coordIKVHeader.nKVQBit);
		oKeyValueEncoder.getQKeyValue(nQKeyValue);
		oKeyValueEncoder.getMaxMin(coordIKVHeader.kVMinMax.fMin_X, coordIKVHeader.kVMinMax.fMin_Y, coordIKVHeader.kVMinMax.fMin_Z,
			coordIKVHeader.kVMinMax.fMax_X, coordIKVHeader.kVMinMax.fMax_Y, coordIKVHeader.kVMinMax.fMax_Z, coordIKVHeader.kVMinMax.nWhichAxis);
		oKeyValueEncoder.getQFlag_kv(bSelFlag);
		oKeyValueEncoder.getTSMode(nDPCMMode);
		oKeyValueEncoder.getQMinQMax(nQMin, nQMax);
		oKeyValueEncoder.getBase(nRefVertex);
		oKeyValueEncoder.getQStep_kv(nKVACodingBit, nRefVertex);
		// ------------------------------------------------------------------------------------ //

		oDPCM_X_Min.setNumber(coordIKVHeader.nNumberOfCoord);
		oDPCM_Y_Min.setNumber(coordIKVHeader.nNumberOfCoord);
		oDPCM_Z_Min.setNumber(coordIKVHeader.nNumberOfCoord);

		oDPCM_X_Min.setData(nQMin);
		oDPCM_Y_Min.setData(nQMin);
		oDPCM_Z_Min.setData(nQMin);

		oDPCM_X_Min.setComponent(0);
		oDPCM_Y_Min.setComponent(1);
		oDPCM_Z_Min.setComponent(2);

		oDPCM_X_Min.dpcm();
		oDPCM_Y_Min.dpcm();
		oDPCM_Z_Min.dpcm();

		coordIKVHeader.nXQMin_Min = oDPCM_X_Min.getMin();
		coordIKVHeader.nXQMin_Max = oDPCM_X_Min.getMax();

		coordIKVHeader.nYQMin_Min = oDPCM_Y_Min.getMin();
		coordIKVHeader.nYQMin_Max = oDPCM_Y_Min.getMax();

		coordIKVHeader.nZQMin_Min = oDPCM_Z_Min.getMin();
		coordIKVHeader.nZQMin_Max = oDPCM_Z_Min.getMax();

		oDPCM_X_Max.setNumber(coordIKVHeader.nNumberOfCoord);
		oDPCM_Y_Max.setNumber(coordIKVHeader.nNumberOfCoord);
		oDPCM_Z_Max.setNumber(coordIKVHeader.nNumberOfCoord);

		oDPCM_X_Max.setData(nQMax);
		oDPCM_Y_Max.setData(nQMax);
		oDPCM_Z_Max.setData(nQMax);

		oDPCM_X_Max.setBase(nQMin);
		oDPCM_Y_Max.setBase(nQMin);
		oDPCM_Z_Max.setBase(nQMin);

		oDPCM_X_Max.setComponent(0);
		oDPCM_Y_Max.setComponent(1);
		oDPCM_Z_Max.setComponent(2);

		oDPCM_X_Max.dpcm();
		oDPCM_Y_Max.dpcm();
		oDPCM_Z_Max.dpcm();

		coordIKVHeader.nXQMax_Min = oDPCM_X_Max.getMin();
		coordIKVHeader.nXQMax_Max = oDPCM_X_Max.getMax();

		coordIKVHeader.nYQMax_Min = oDPCM_Y_Max.getMin();
		coordIKVHeader.nYQMax_Max = oDPCM_Y_Max.getMax();

		coordIKVHeader.nZQMax_Min = oDPCM_Z_Max.getMin();
		coordIKVHeader.nZQMax_Max = oDPCM_Z_Max.getMax();

		// print: result data from keyValue encoding
		cout << "Number of coords = " << coordIKVHeader.nNumberOfCoord_org << endl;
		cout << "QStep_keyValue = " << coordIKVHeader.nKVQBit << endl;
		cout << "KeyValue digit = " << coordIKVHeader.nKVDigit << endl;
		cout << "X_min ~ X_max = " << coordIKVHeader.kVMinMax.fMin_X << " ~ " << coordIKVHeader.kVMinMax.fMax_X << endl;
		cout << "Y_min ~ Y_max = " << coordIKVHeader.kVMinMax.fMin_Y << " ~ " << coordIKVHeader.kVMinMax.fMax_Y << endl;
		cout << "Z_min ~ Z_max = " << coordIKVHeader.kVMinMax.fMin_Z << " ~ " << coordIKVHeader.kVMinMax.fMax_Z << endl;
		cout << endl;
		}

  private:
	// transpose: source data -> target data which is transposed
	void transposeKeyValue(double* a_dKeyValue_target, double* a_dKeyValue_source, int nNumberOfKey, int nNumberOfCoord)
		{
		double* a_dKeyValue = new double[nNumberOfKey*nNumberOfCoord*3];
		memcpy((void*)a_dKeyValue, (void*)a_dKeyValue_source, sizeof(double)*(nNumberOfKey*nNumberOfCoord*3));

		for(int j = 0; j < nNumberOfCoord; j++)
			{
			for(int i = 0; i < nNumberOfKey; i++)
				{
				for(int c = 0; c < 3; c++)
					{
					//double dValue = a_dKeyValue[i*nNumberOfCoord*3+j*3+c];
					double dValue = getValue(a_dKeyValue, i, j, c);

					//a_dKeyValue_target[j*nNumberOfKey*3+i*3+c] = dValue;
					getTransposedValue(a_dKeyValue_target, j, i, c) = dValue;
					//cout << dValue;
					//cout << ", ";
					}
				//cout << endl;
				}
			//cout << endl;
			}

		delete[] a_dKeyValue;
		}

	// get: the value with the indexes of nKey, nCoord, nComponent in keyValue
	double& getValue(double* a_dValue, int nKey, int nCoord, int nComponent)
		{
		double& dValue = a_dValue[nKey*nNumberOfCoord_org*3 + nCoord*3 + nComponent];
		return dValue;
		}

	// get: the transposed value with the indexes of nKey, nCoord, nComponent in keyValue
	double& getTransposedValue(double* a_dValue, int nCoord, int nKey, int nComponent)
		{
		double& dValue = a_dValue[nCoord*nNumberOfKey_org*3 + nKey*3 + nComponent];
		return dValue;
		}
	};

#endif // _COORDIKEYVALUE_H_
