// CoordIKeyValueDic.h: coordinate key value encoder
// Author: Shinjun Lee, Samsung AIT
// 4th revision: 2002-04-25

#ifndef _COORDIKEYVALUEDIC_H_
#define _COORDIKEYVALUEDIC_H_

#include "../include/CoordIIncrementalMode.h"
#include "../include/CoordIOccurrenceMode.h"
#include "../include/Entropy.h"

class CoordIKeyValueDic
	{
  public:
	// variables: bitstream data of key value
	int nDicModeSelect;

	CoordIIncrementalMode coordIIncrementalMode;
	CoordIOccurrenceMode coordIOccurrenceMode;

	// variables: nonparsable data
	CAC dicModeSelectionContext;				// Dictionary selection flag: dictionary value or dictionary address

  public:
	CoordIKeyValueDic(void)
		{
		nDicModeSelect = 0;
		}

	~CoordIKeyValueDic()
		{
		}

	void InitializeModel(void)
		{
		// Dictionary selection flag: dictionary value or dictionary address
		dicModeSelectionContext.reset();

		coordIIncrementalMode.InitializeModel();
		coordIOccurrenceMode.InitializeModel();
		}

	void makeBitstream(CAC_QF& codec, bool* bSelFlag, int* nKVCodingBit, int nNumberOfKey, int c, int j, CoordIKeyValueHeader& coordIKVHeader, int* nQKeyValue)
		{
		int			i = 0;									// index
		int			k = 0;									// index

		// after dpcm, if min(x) and Max(x) are not 0
		if(bSelFlag[j*3+c] == true && nKVCodingBit[j*3+c] != 0)
			{
			bool*		a_bDic_index = NULL;				// dictionary index flag
			int			nNumberOfSymbol = 0;				// the number of symbols

			// allocate: a_bDic_index
			a_bDic_index = new bool[nNumberOfKey];
			memset((void*)a_bDic_index, 0, sizeof(bool)*nNumberOfKey);

			// calculate: the number of symbols
			for(i = 0; i < nNumberOfKey; i++)
				{
				if(a_bDic_index[i] == 0)
					{
					a_bDic_index[i] = 1;
					nNumberOfSymbol++;

					int nQKV = nQKeyValue[i*coordIKVHeader.nNumberOfCoord*3+j*3+c];
					for(k = i+1; k < nNumberOfKey; k++)
						{
						if(a_bDic_index[k] == 0)
							{
							if(nQKV == nQKeyValue[k*coordIKVHeader.nNumberOfCoord*3+j*3+c])
								a_bDic_index[k] = 1;
							}
						}
					}
				}

			// calculate: the size of incremental(address) mode and the size of occurence(dictionary) mode
			int nSizeOfAddress = (int)pow(2, nKVCodingBit[j*3+c]+1) - 1;
			int nSizeOfDictionary = nNumberOfSymbol*(nKVCodingBit[j*3+c]+1);

			// select: either incremental(address) mode or occurence(dictionary) mode
			if(nSizeOfAddress < nSizeOfDictionary)
				{
				// dicModeSelectionContext: flag of address coding
				codec.encode(1, dicModeSelectionContext);
				coordIIncrementalMode.makeBitstream(codec, nKVCodingBit, nNumberOfKey,
													c, j, coordIKVHeader, nQKeyValue, nNumberOfSymbol);
				}
			else
				{
				// dicModeSelectionContext: flag of dictionary coding
				codec.encode(0, dicModeSelectionContext);
				coordIOccurrenceMode.makeBitstream(codec, nKVCodingBit, nNumberOfKey, c,
													j, coordIKVHeader, nQKeyValue);
				}

			// delete: dictionary index
			delete[] a_bDic_index;
			}
		}

	void encodingProcess(CoordIKeyValueHeader& coordIKVHeader)
		{
		// No codes.
		}
	};

#endif // _COORDIKEYVALUEDIC_H_
