// CoordIOccurrenceMode.h: coordinate key value encoder
// Author: Shinjun Lee, Samsung AIT
// 4th revision: 2002-04-25

#ifndef _COORDIOCCURRENCEMODE_H_
#define _COORDIOCCURRENCEMODE_H_

#include "../include/Entropy.h"

class CoordIOccurrenceMode
	{
  public:
	// variables: bitstream data of key value
	int				nQKV;									// the dictionary symbol
	bool			bSoleKV;								// the flag of whether a symbol is only one
	int				nTrueOne;								// the value of one

	// variables: nonparsable data
	CAC kvSignContext;										// Key value: keyValue context (dictionary value)
	CAC kvXContext[32];
	CAC kvYContext[32];
	CAC kvZContext[32];
	CAC dicSoleKVContext;									// Dictionary last

  public:
	CoordIOccurrenceMode(void)
		{
		nQKV = 0;
		bSoleKV = false;
		nTrueOne = 1;
		}

	~CoordIOccurrenceMode()
		{
		}

	void InitializeModel(void)
		{
		int i=0;

		// Key value: keyValue context (dictionary value)
		kvSignContext.reset();
		for(i = 0; i < 32; i++)
			{
			kvXContext[i].reset();
			kvYContext[i].reset();
			kvZContext[i].reset();
			}

		// Dictionary last
		dicSoleKVContext.reset();
		}

	void makeBitstream(CAC_QF& codec, int* nKVCodingBit, int nNumberOfKey, int c, int j, CoordIKeyValueHeader& coordIKVHeader, int* nQKeyValue)
		{
		bool*		bIndexOfDic = NULL;						// dictionary index flag
		int			nZero = 0;								// the value of zero
		int			i = 0;									// index
		int			k = 0;									// index

		// allocate: bIndexOfDic
		bIndexOfDic = new bool[nNumberOfKey];
		memset((void*)bIndexOfDic, 0, sizeof(bool)*nNumberOfKey);

		// code: occurence mode
		for(i = 0; i < nNumberOfKey; i++)
			{
			// if not scanned yet
			if(bIndexOfDic[i] == 0)
				{
				bIndexOfDic[i] = 1;	// scanned index

				// nQKV: the value of dictionary
				int nQKV = nQKeyValue[i*coordIKVHeader.nNumberOfCoord*3+j*3+c];

				// send: the value of dictionary
				if(c == 0)
					encodeSignedQuasiAAC(codec, nQKV, nKVCodingBit[j*3+c]+1, kvSignContext, kvXContext);
				else if(c == 1)
					encodeSignedQuasiAAC(codec, nQKV, nKVCodingBit[j*3+c]+1, kvSignContext, kvYContext);
				else if(c == 2)
					encodeSignedQuasiAAC(codec, nQKV, nKVCodingBit[j*3+c]+1, kvSignContext, kvZContext);

				// code: dictionary index
				bSoleKV = true;					// the flag of whether a symbol is only one
				int	nNumberOfOne = 0;			// the number of one flags
				int	nNumberOfZero = 0;			// the number of zero flags

				// count: the number of one flags and zero flags
				for(k = i+1; k < nNumberOfKey; k++)
					{
					if(bIndexOfDic[k] == 0)
						{
						nNumberOfZero++;
						if(nQKV == nQKeyValue[k*coordIKVHeader.nNumberOfCoord*3+j*3+c])
							{
							bSoleKV = false;
							nNumberOfOne++;
							}
						}
					}

				// encode: whether the symbol is only one
				codec.encode(bSoleKV, dicSoleKVContext);

				// nSoleKV: if the number of symbols is more than one
				if(bSoleKV == false)
					{
					nTrueOne = nNumberOfOne >= nNumberOfZero/2 ? 0 : 1;
					nZero = nTrueOne == 0 ? 1 : 0;
					// nOne: the real value of one
					codec.encode(nTrueOne, CQF_Status::dicOneContext);

					// encode: dictionary index
					for(k = i+1; k < nNumberOfKey; k++)
						{
						if(bIndexOfDic[k] == 0)
							{
							if(nQKV == nQKeyValue[k*coordIKVHeader.nNumberOfCoord*3+j*3+c])
								{
								bIndexOfDic[k] = 1;
								
								codec.encode(nTrueOne, CQF_Status::dicIndexContext);
								}
							else
								{
								codec.encode(nZero, CQF_Status::dicIndexContext);
								}
							}
						}
					}
				}
			}
		// delete: dictionary index
		delete[] bIndexOfDic;
		}

	void encodingProcess(CoordIKeyValueHeader& coordIKVHeader)
		{
		// No codes.
		}
	};

#endif // _COORDIOCCURRENCEMODE_H_
