// Key.h: key encoder
// Author: Shinjun Lee, Samsung AIT
// 4th revision: 2002-04-18

#ifndef _KEY_H_
#define _KEY_H_

#include "../include/ArithmeticEncoder.h"
#include "../include/KeyHeader.h"
#include "../include/CKeyEncoder.h"
#include "../include/LinearKey.h"

class Key
	{
  public:
	// variables: bitstream data of key
	int*			nQKey;									// key array

	// variables: nonparsable data for key encoding
	int				nNumberOfRemainingKey;					// the number of remaining key after linear key encoding

	double*			dKey;									// data of key

	// Key: key context
	CAC				keySignContext;
	CAC				keyContext[32];

  public:
	Key(void)
		{
		int i = 0;

		nQKey = NULL;

		nNumberOfRemainingKey = 0;

		dKey = NULL;

		// Key: key context
		keySignContext.reset();
		for(i = 0; i < 32; i++)
			keyContext[i].reset();
		}

	~Key()
		{
		delete[] nQKey;
		}

	void InitializeModel(void)
		{
		int i = 0;

		// Key: key context
		keySignContext.reset();
		for(i = 0; i < 32; i++)
			keyContext[i].reset();
		}

	void makeBitstream(CAC_QF& codec, KeyHeader& kHeader, int& nTotalBitsOfKey)
		{
		int nBitsOfStart = 0;
		int nBitsOfEnd = 0;
		int	i = 0; // index

		nBitsOfStart = (codec._ostrm->Outbuf.nbytes-1)*8 + (codec._ostrm->Outbuf.nbits-1);
		for(i = kHeader.nKDPCMOrder; i < nNumberOfRemainingKey; i++)
			{
			if(kHeader.bSignedAACFlag == false)
				encodeUnsignedAAC(codec, nQKey[i], kHeader.nKeyCodingBit, keyContext);
			else
				encodeSignedAAC(codec, nQKey[i], kHeader.nKeyCodingBit+1, keySignContext, keyContext);
			}
		nBitsOfEnd = (codec._ostrm->Outbuf.nbytes-1)*8 + (codec._ostrm->Outbuf.nbits-1);
		nTotalBitsOfKey += nBitsOfEnd - nBitsOfStart;
		}

	void encodingProcess(KeyHeader& kHeader)
		{
		// 2002-04-17, Shinjun
		// variables: arrays for linear key encoding
		JWArray<double>		JWOriginalKeys;
		JWArray<double>		JWSelectedKeys;
		JWArray<double>		JWDecodedKey;

		// 2002-04-17, Shinjun
		// variables: data for linear key encoding
		int					preStartKeyIdx = -1;
		int					preEndKeyIdx = -1;
		bool				bIsKeyLinearCoder = kHeader.bIsLinearKeyUsed;
		bool				bPreserveKey = true;
		int					i = 0;

		// 2002-04-17, Shinjun
		// copy: the array of key to JWOriginalKeys
		JWDecodedKey.SetSize(kHeader.nNumberOfKey);
		JWSelectedKeys.SetSize(kHeader.nNumberOfKey);
		for(i = 0; i < kHeader.nNumberOfKey; i++)
			JWOriginalKeys.Add(dKey[i]);

		// 2002-04-17, Shinjun
		// Linear key encoder
		kHeader.lKey.LinearKeySubRegionExtractor(kHeader.nKeyQBit, kHeader.nKeyDigit,
			bIsKeyLinearCoder, bPreserveKey,
			preStartKeyIdx, preEndKeyIdx,
			JWOriginalKeys, JWSelectedKeys);
		kHeader.lKey.KeySplitter(bPreserveKey, preStartKeyIdx, preEndKeyIdx, kHeader.nKeyDigit,
			JWOriginalKeys, JWSelectedKeys, JWDecodedKey,
			kHeader.lKey.fKeyMin, kHeader.lKey.fKeyMax, kHeader.lKey.nNumberOfLinearKey);

		// 2002-04-17, Shinjun
		if(preStartKeyIdx != -1 || preEndKeyIdx != -1)
			{
			kHeader.bIsLinearKeySubRegion = true;
			nNumberOfRemainingKey = kHeader.nNumberOfKey - kHeader.lKey.nNumberOfLinearKey;
			}
		else
			{
			kHeader.bIsLinearKeySubRegion = false;
			nNumberOfRemainingKey = kHeader.nNumberOfKey;
			}

		// 2002-04-17, Shinjun
		// copy: JWOriginalKeys to the array of key
		for(i = 0; i < nNumberOfRemainingKey; i++)
			dKey[i] = JWOriginalKeys[i];

		// Key encoder
		CKeyEncoder oKeyEncoder;
		oKeyEncoder.setKeyDigit(kHeader.nKeyDigit);
		oKeyEncoder.setMeasureOfDNDOrder(kHeader.bIsSADUsed);
		oKeyEncoder.encode(dKey, nNumberOfRemainingKey, kHeader.nKeyQBit);

		// get: result data from key encoding
		oKeyEncoder.getQKey(nQKey);
		oKeyEncoder.getKey_dnd_max(kHeader.nKeyMax);
		kHeader.nKDPCMOrder = oKeyEncoder.getOrder_dpcm();
		kHeader.nDNDOrder = oKeyEncoder.getOrder_dnd();
		kHeader.nKeyShift = oKeyEncoder.getKey_shift();
		kHeader.nKeyInvertUp = oKeyEncoder.getKey_shift_up();
		kHeader.nBitSize = oKeyEncoder.getQStep_key_shift();
		kHeader.nKeyCodingBit = oKeyEncoder.getQStep_key_encoding();

		if(kHeader.nKDPCMOrder > 0)
			{
			kHeader.nQIntraKey = new int[kHeader.nKDPCMOrder];
			kHeader.nQIntraKeySign = new int[kHeader.nKDPCMOrder];
			}
		for(i = 0; i < kHeader.nKDPCMOrder; i++)
			kHeader.nQIntraKey[i] = nQKey[i];

		kHeader.keyMinMax.fKeyMin = kHeader.fKeyMin = dKey[0];
		kHeader.keyMinMax.fKeyMax = kHeader.fKeyMax = dKey[nNumberOfRemainingKey-1];

		// print: result data from key encoding
		cout << "Number of keys = " << kHeader.nNumberOfKey << endl;
		cout << "QStep_key = " << kHeader.nKeyQBit << endl;
		cout << "Key digit = " << kHeader.nKeyDigit << endl;
		cout << "Key Distortion = " << oKeyEncoder.getKeyDistortion() << endl;
		cout << "Order_dpcm = " << kHeader.nKDPCMOrder << endl;
		cout << "Order_dnd = " << kHeader.nDNDOrder << endl;
		cout << "Key to shift = " << kHeader.nKeyShift << endl;
		cout << "Key to shift-up = " << kHeader.nKeyInvertUp << endl;
		cout << "QStep(key_shift) = " << kHeader.nBitSize << endl;
		cout << "QStep(key_encoding) = " << kHeader.nKeyCodingBit << endl;
		cout << endl;
		cout << "dKeyMin = " << kHeader.lKey.fKeyMin << " (index: " << preStartKeyIdx << ")" << endl;
		cout << "dKeyMax = " << kHeader.lKey.fKeyMax << " (index: " << preEndKeyIdx << ")" << endl;
		cout << "nNumberOfLinearKey = " << kHeader.lKey.nNumberOfLinearKey << endl;
		cout << endl;
		}
	};

#endif // _KEY_H_