// KeyHeader.h: key header encoder
// Author: Shinjun Lee, Samsung AIT
// 4th revision: 2002-04-23

#ifndef _KEYHEADER_H_
#define _KEYHEADER_H_

#include "../include/LinearKey.h"
#include "../include/KeyMinMax.h"

class KeyHeader
	{
  public:
	// variables: bitstream data of key
	int				nKeyQBit;								// quantization bit for key
	int				nNumKeyCodingBit;						// the bit for nNumberOfKey
	int				nNumberOfKey;							// the number of keys
	int				nKeyDigit;								// key digit

	bool			bIsLinearKeySubRegion;					// flag of linear key encoding
	LinearKey		lKey;									// class of linear key encoder

	bool			bRangeFlag;								// flag of key range
	KeyMinMax		keyMinMax;								// class of key min and key max encoder

	int				nBitSize;								// bit size
	int				nKDPCMOrder;							// order of key DPCM
	int*			nQIntraKeySign;							// sign array for intra keys
	int*			nQIntraKey;								// intra key array

	bool			bShiftFlag;								// flag of shift
	int				nKeyShiftSign;							// sign of key shift
	int				nKeyShift;								// the value of key shift

	int				nDNDOrder;								// order of DND

	int*			nKeyMaxSign;							// sign array for intra DND values
	int*			nKeyMax;								// intra DND values

	int				nKeyCodingBit;							// coding bit size for key
	bool			bKeyInvertUpFlag;						// flag of key invert up
	int				nKeyInvertUp;							// the value of key invert up

	// variables: nonparsable data for key encoding
	bool			bSignedAACFlag;							// flag of signed aac encoding

	double			fKeyMin;								// key min
	double			fKeyMax;								// key max

	int				nNumberOfKey_org;						// the original number of keys

	bool			bIsLinearKeyUsed;						// the flag of using the linear key coding
	bool			bIsSADUsed;								// the flag of using the measure of SAD

  public:
	KeyHeader(void)
		{
		nKeyQBit = 0;
		nNumKeyCodingBit = 0;
		nNumberOfKey = 0;
		nKeyDigit = 0;

		bIsLinearKeySubRegion = false;

		bRangeFlag = false;

		nBitSize = 0;
		nKDPCMOrder = 0;
		nQIntraKeySign = NULL;
		nQIntraKey = NULL;

		bShiftFlag = false;
		nKeyShiftSign = 0;
		nKeyShift = 0;

		nDNDOrder = 0;

		nKeyMaxSign = NULL;
		nKeyMax = NULL;

		nKeyCodingBit = 0;
		bKeyInvertUpFlag = false;
		nKeyInvertUp = 0;

		bSignedAACFlag = false;

		nNumberOfKey_org = 0;

		bIsLinearKeyUsed = false;
		bIsSADUsed = false;
		}

	~KeyHeader()
		{
		delete[] nQIntraKeySign;
		delete[] nQIntraKey;

		delete[] nKeyMaxSign;
		delete[] nKeyMax;
		}

	void makeBitstream(CAC_QF& codec, int& nTotalBitsOfKey)
		{
		int			i = 0;									// index

		ofstream fout("fout-encoding.txt", ios::app);
		fout << "[Start: KeyHeader]" << endl;

		// put: nKeyQBit
		codec._ostrm->putbits(nKeyQBit, 5);
		fout << "nQStep_Key: " << nKeyQBit << endl;
		nTotalBitsOfKey += 5;

		// put: the number of keys
		nNumKeyCodingBit = getNbBits(nNumberOfKey_org);
		codec._ostrm->putbits(nNumKeyCodingBit, 5);
		fout << "nQStep_Num: " << nNumKeyCodingBit << endl;
		nTotalBitsOfKey += 5;
		codec._ostrm->putbits(nNumberOfKey_org, nNumKeyCodingBit);
		fout << "nNumberOfKey: " << nNumberOfKey_org << endl;
		nTotalBitsOfKey += nNumKeyCodingBit;

		// put: key digit
		codec._ostrm->putbits(nKeyDigit, 4);
		fout << "nKey_Digit: " << nKeyDigit << endl;
		nTotalBitsOfKey += 4;

		// 2002-04-17, Shinjun
		// put: linear key
		if(bIsLinearKeySubRegion == true)
			{
			// subset Ѵ.
			int	nIsColinear = 1;
			codec._ostrm->putbits(nIsColinear, 1);
			nTotalBitsOfKey += 1;

			lKey.makeBitstream(codec, nKeyDigit, nTotalBitsOfKey);
			}
		else
			{
			// subset  ʴ´.
			int nIsColinear = 0;
			codec._ostrm->putbits(nIsColinear, 1);
			nTotalBitsOfKey += 1;
			}

		// put: fKeyMin, fKeyMax
		if(fKeyMin < 0 || fKeyMax > 1)
			{
			codec._ostrm->putbits(1, 1);
			fout << "bRangeFlag: " << 1 << endl;
			nTotalBitsOfKey += 1;

			keyMinMax.makeBitstream(codec, nKeyDigit, nTotalBitsOfKey);
			}
		else
			{
			codec._ostrm->putbits(0, 1);
			fout << "bRangeFlag: " << 0 << endl;
			nTotalBitsOfKey += 1;
			}

		// put: m_nQStep_key_shift
		codec._ostrm->putbits(nBitSize, 5);
		fout << "nQStep_Shift: " << nBitSize << endl;
		nTotalBitsOfKey += 5;

		// 2002-04-18-02, Shinjun
		// put: nKDPCMOrder
		if(nKDPCMOrder == 0)
			{
			nQIntraKeySign = new int[1];
			nQIntraKeySign[0] = 1;
			nQIntraKey = new int[1];
			nQIntraKey[0] = 0;
			nKDPCMOrder++;
			}
		codec._ostrm->putbits(nKDPCMOrder-1, 2);
		fout << "nOrder_DPCM: " << nKDPCMOrder << endl;
		nTotalBitsOfKey += 2;

		// put: intra key data
		for(i = 0; i < nKDPCMOrder; i++)
			{
			if(i == 0 && nQIntraKey[i] == 0)
				{
				nQIntraKeySign[i] = 1;
				codec._ostrm->putbits(nQIntraKeySign[i], 1);
				fout << "nSign_nQKey: " << 1 << endl;
				nTotalBitsOfKey += 1;
				continue;
				}
			if(nQIntraKey[i] >= 0)
				{
				nQIntraKeySign[i] = 0;
				codec._ostrm->putbits(nQIntraKeySign[i], 1);
				fout << "nSign_nQKey: " << 0 << endl;
				}
			else
				{
				nQIntraKeySign[i] = 1;
				codec._ostrm->putbits(nQIntraKeySign[i], 1);
				fout << "nSign_nQKey: " << 1 << endl;
				}
			nTotalBitsOfKey += 1;
			codec._ostrm->putbits(abs(nQIntraKey[i]), nBitSize);
			fout << "a_nQIntraKey: " << abs(nQIntraKey[i]) << endl;
			nTotalBitsOfKey += nBitSize;
			}

		// put: shift key data
		if(nKeyShift != 0)
			{
			codec._ostrm->putbits(1, 1);
			fout << "nShift: " << 1 << endl;
			nTotalBitsOfKey += 1;
			if(nKeyShift >= 0)
				{
				codec._ostrm->putbits(0, 1);
				fout << "nSign_Shift: " << 0 << endl;
				}
			else
				{
				codec._ostrm->putbits(1, 1);
				fout << "nSign_Shift: " << 1 << endl;
				}
			nTotalBitsOfKey += 1;
			codec._ostrm->putbits(abs(nKeyShift), nBitSize);
			fout << "nKey_Shift: " << abs(nKeyShift) << endl;
			nTotalBitsOfKey += nBitSize;
			}
		else
			{
			codec._ostrm->putbits(0, 1);
			fout << "nShift: " << 0 << endl;
			nTotalBitsOfKey += 1;
			}

		// put: nDNDOrder
		if(nDNDOrder != -1)
			{
			codec._ostrm->putbits(nDNDOrder, 3);
			fout << "unOrder_DND: " << nDNDOrder << endl;
			nTotalBitsOfKey += 3;
			if(nDNDOrder == 7)			// if 7, sign = 0(positive)
				{
				codec._ostrm->putbits(0, 1);
				fout << "nSign_Order: " << 0 << endl;
				nTotalBitsOfKey += 1;
				}
			}
		else
			{
			codec._ostrm->putbits(7, 3);	// if -1, put 7(binary: 111)
			fout << "unOrder_DND: " << 7 << endl;
			nTotalBitsOfKey += 3;
			codec._ostrm->putbits(1, 1);	// sign = 1(negative)
			fout << "nSign_Order: " << 1 << endl;
			nTotalBitsOfKey += 1;
			}

		// put: nKeyMax
		int nMaxQBit = nBitSize;
		for(i = 0; i < nDNDOrder; i++)
			{
			if(nKeyMax[i] >= 0)
				{
				codec._ostrm->putbits(0, 1);
				fout << "nSign_Max: " << 0 << endl;
				}
			else
				{
				codec._ostrm->putbits(1, 1);
				fout << "nSign_Max: " << 1 << endl;
				}
			nTotalBitsOfKey += 1;
			codec._ostrm->putbits(abs(nKeyMax[i]), nMaxQBit);
			fout << "a_nKey_Max: " << abs(nKeyMax[i]) << endl;
			nTotalBitsOfKey += nMaxQBit;

			nMaxQBit = getNbBits(abs(nKeyMax[i]));
			if(nMaxQBit+1 < nBitSize)
				nMaxQBit += 1;
			else
				nMaxQBit = nBitSize;
			}

		// put: nKeyCodingBit
		int nKeyCodingBitQBit = (int)(log10(abs(nKeyQBit))/log10(2))+1;
		codec._ostrm->putbits(nKeyCodingBit, nKeyCodingBitQBit);
		fout << "nQStep_Enc: " << nKeyCodingBit << endl;
		nTotalBitsOfKey += nKeyCodingBitQBit;

		// put: nKeyInvertUp
		if(nDNDOrder != -1 && nDNDOrder != 0)
			{
			if(nKeyInvertUp != -1)
				{
				codec._ostrm->putbits(1, 1);
				fout << "nShift_Up: " << 1 << endl;
				nTotalBitsOfKey += 1;

				codec._ostrm->putbits(nKeyInvertUp, nKeyCodingBit);
				fout << "nKey_Shift_Up: " << nKeyInvertUp << endl;
				nTotalBitsOfKey += nKeyCodingBit;
				bSignedAACFlag = false;
				}
			else
				{
				codec._ostrm->putbits(0, 1);
				fout << "nShift_Up: " << 0 << endl;
				nTotalBitsOfKey += 1;
				bSignedAACFlag = true;
				}
			}
		else
			bSignedAACFlag = false;

		fout << "[End: KeyHeader]" << endl;
		fout << endl;
		fout.close();
		}

	void encodingProcess(void)
		{
		nNumberOfKey_org = nNumberOfKey;
		}

  private:
	int getNbBits(int k)
		{
		int j=1;
		while (k >= (1<<j) ) 
			{
			j++;
			}
		return j;
		}
	};

#endif // _KEYHEADER_H_
