// KeyMinMaxEncoder
// Author: Sang-Oak Woo
// Editor: Shinjun Lee (codes are modified into classes)
// Date: 2001. 12. 27.
// 4th revision: 2002-04-23

#ifndef _KEYMINMAX_H_
#define _KEYMINMAX_H_

#include <math.h>
#include <stdlib.h>

#include "Entropy.h"

const int cnMAX_DIGIT = 15;
const double dEPSILONERROR = 0.0000000001;		// 10

class KeyMinMax
	{
  public:
	int			bMinKeyDigitSame;
	int			nMinKeyDigit;

	int			nMinKeyMantissaSign;
	int			nMinKeyMantissa;

	int			nMinKeyExponentSign;
	int			nMinKeyExponent;

	int			nMaxKeyDigit;

	int			nMaxKeyMantissaSign;
	int			nMaxKeyMantissa;

	int			nMaxKeyExponentSign;
	int			nMaxKeyExponent;

	int			bSameExponent;

	double		fKeyMin;
	double		fKeyMax;

  public:
	KeyMinMax(void)
		{
		bMinKeyDigitSame = 0;
		nMinKeyDigit = 0;

		nMinKeyMantissaSign = 0;
		nMinKeyMantissa = 0;

		nMinKeyExponentSign = 0;
		nMinKeyExponent = 0;

		nMaxKeyDigit = 0;

		nMaxKeyMantissaSign = 0;
		nMaxKeyMantissa = 0;

		nMaxKeyExponentSign = 0;
		nMaxKeyExponent = 0;

		bSameExponent = 0;

		fKeyMin = 0;
		fKeyMax = 0;
		}

	void makeBitstream(CAC_QF& codec, int nKeyDigit, int& KTotal)
		{
		// Start key
		// mantissa digit 4 
		int nDigit = GetDigit(fKeyMin); //  nDigit: ù°   digit
		// 0  digit = 0.
		if(nDigit == nKeyDigit) // nKeyDigit: ü digit
			{
			bMinKeyDigitSame = 1;
			codec._ostrm->putbits(bMinKeyDigitSame, 1);
			KTotal += 1;
			}
		else
			{
			bMinKeyDigitSame = 0;
			codec._ostrm->putbits(bMinKeyDigitSame, 1);
			KTotal += 1;
			codec._ostrm->putbits(nDigit, 4);
			KTotal += 4;
			}

		int mantissa;
		int preexponent;
		int count;

		if(nDigit != 0)
			{
			if(nDigit < 8)
				{
				// mantissa 10 
				int nSign = 0;
				if(fKeyMin >= 0)
					codec._ostrm->putbit(0);
				else
					codec._ostrm->putbit(1);
				KTotal += 1;
				mantissa = GetMantissa(fKeyMin, nDigit);
				count = getNbBits((int)pow(10.0, nDigit)-1);
				codec._ostrm->putbits(mantissa, count);
				KTotal += count;

				// exponent digit 6
				preexponent = GetLargestDigit(fKeyMin);
				if(preexponent >= 0)
					codec._ostrm->putbit(0);
				else
					codec._ostrm->putbit(1);
				KTotal += 1;
				codec._ostrm->putbits(abs(preexponent), 6);
				KTotal += 6;
				}
			else
				{
				float flot = (float)fKeyMin;
				codec._ostrm->putbits(*((int *)&flot), 32);
				KTotal += 32;
				}
			}

		// End key
		// mantissa digit 4
		nDigit =  GetDigit(fKeyMax);
		if(nDigit == nKeyDigit)
			{
			codec._ostrm->putbits(1, 1);
			KTotal += 1;
			}
		else
			{
			codec._ostrm->putbits(0, 1);
			KTotal += 1;
			codec._ostrm->putbits(nDigit, 4);
			KTotal += 4;
			}							

		if(nDigit != 0)
			{
			if(nDigit < 8)
				{
				// mantissa 10 
				int nSign = 0;
				if(fKeyMax >= 0)
					codec._ostrm->putbit(0);
				else
					codec._ostrm->putbit(1);
				KTotal += 1;
				mantissa = GetMantissa(fKeyMax, nDigit);
				count = getNbBits((int)pow(10.0, nDigit)-1);
				codec._ostrm->putbits(mantissa, count);
				KTotal += count;

				// exponent 6 
				int exponent = GetLargestDigit(fKeyMax);
				if(exponent == preexponent)
					{
					codec._ostrm->putbits(1, 1);
					KTotal += 1;
					}
				else
					{
					codec._ostrm->putbits(0, 1);
					KTotal += 1;

					if(exponent >= 0)
						codec._ostrm->putbit(0);
					else
						codec._ostrm->putbit(1);
					KTotal += 1;
					codec._ostrm->putbits(abs(exponent), 6);
					KTotal += 6;
					}
				}
			else
				{
				const float flot = (float)fKeyMax;
				codec._ostrm->putbits(*((int *)&flot), 32);
				KTotal += 32;
				}
			}
		}

	void encodingProcess(void)
		{
		// no code
		}

  private:
	int GetLeadingZero(double key)
		{
		//////////////////////////////////////////////////
		//
		//			Original Value 7ڸ !!!
		//			꿡 ؼ   
		//
		//////////////////////////////////////////////////
		
		char buffer[50];
		_gcvt(key, cnMAX_DIGIT, buffer);
		key = atof(buffer);
		
		//
		//////////////////////////////////////////////////
		
		double a;
		double e = dEPSILONERROR;
		__int64 nMulti = 10;
		double dMulti = 0.1;
		int nCount = 0;
		
		double dTemp;
		
		// 1. 켱  ٲٰ,
		a = fabs(key);
		
		// 2. 0̰ų 0.1 ũ տ "0" .
		if(a >= 1.0)
			{
			nCount++;
			while(a*dMulti > 1.0)
				{
				nCount++;
				dMulti *= 0.1;
				}
			
			return -nCount;
			}
		else if(a == 0.0)
			return 0;
		else if(a >= 0.1)
			return 0;
		else
			{
			nCount++;
			dTemp = a * nMulti;
			_gcvt(dTemp, 7, buffer);
			dTemp = ::atof(buffer);
			
			//while(a*nMulti < 0.1)
			while( dTemp < 0.1)
				{
				nCount++;
				nMulti *= 10;
				dTemp = a * nMulti;
				_gcvt(dTemp, 7, buffer);
				dTemp = ::atof(buffer);
				}
			
			return nCount;
			}
		}

	// sowoo 2001.10.11 (Min/Max Reduce)
	// Ǽ Digit 
	int GetDigit(double dValue)
		{
		//////////////////////////////////////////////////
		//
		//			Original Value 7ڸ !!!
		//			꿡 ؼ   
		//
		//////////////////////////////////////////////////
		
		char buffer[50];
		_gcvt(dValue, cnMAX_DIGIT, buffer);
		dValue = atof(buffer);
		
		//
		//////////////////////////////////////////////////
		
		// "0.0"  ڸ̴.
		if(dValue == 0.0)
			return 1;
		
		int nFirstDigit, nLastDigit;
		
		nFirstDigit = GetLeadingZero(dValue);
		nLastDigit = GetLargestDigit(dValue);
		
		int nDigit = nLastDigit - nFirstDigit;
		
		// 7 ū    .
		//  floating-point number ϴ ȿڸ 6~7!
		// ƴ,  ̻   .
		return (nDigit > cnMAX_DIGIT)? cnMAX_DIGIT: nDigit;
		}

	int GetLargestDigit(double key)
		{
		//////////////////////////////////////////////////
		//
		//			Original Value 7ڸ !!!
		//			꿡 ؼ   
		//
		//////////////////////////////////////////////////
		
		char buffer[50];
		_gcvt(key, cnMAX_DIGIT, buffer);
		key = atof(buffer);
		
		//
		//////////////////////////////////////////////////
		
		double a;
		__int64 mult;
		double dmult;
		__int64 Ia;
		double e = dEPSILONERROR;
		int nLargestDigit;
		
		mult=1; dmult=0.1;
		nLargestDigit = 0;
		
		a = fabs(key);
		Ia = (__int64) (a + mult*e);
		
		// sowoo 2001.10.11 (Digit   ϱ)
		// "0.1" binary ǥ ⶧ ǵ  ʴ  .
		// 0.1 ϴ  ſ 10  scheme !
		if(a == 0.0)
			return 1;
		
		double dTemp = a - (double)Ia;
		
		if((dTemp == 0.0) || (dTemp < e))
			{
			while(true)
				{
				a = fabs(key);
				a *= dmult;
				Ia = (__int64) (a + e);
				dTemp = a - (double)Ia;
				
				if(dTemp > e)
					return nLargestDigit;
				
				dmult *= 0.1;
				nLargestDigit--;
				}
			}
		
		//while(a - (double) Ia > e*mult)
		while(dTemp > e*mult)
			{
			mult*=10;
			nLargestDigit++;
			a = fabs(key);
			a=(double) mult *a;
			Ia = (__int64) (a + mult*e);
			dTemp = a - (double)Ia;
			}
		
		return nLargestDigit;
		}

	// double Mantissa(10) ϴ α׷!!!
	// input:
	// -> dValue: Source floating point 
	// -> nDigit:  ڸ
	int GetMantissa(double dValue, int nDigit)
		{
		char* buffer;
		int nDummy1 = 0, nDummy2 = 0;
		
		buffer = _ecvt(dValue, nDigit, &nDummy1, &nDummy2);
		
		return ::atoi(buffer);
		}

	int getNbBits(int k)
		{
		int j=1;
		while (k >= (1<<j) ) 
			{
			j++;
			}
		return j;
		}
	};

#endif // _KEYMINMAX_H_