// KeyValueMinMax.h: KeyValueMinMax encoder
// Author: Shinjun Lee, Samsung AIT
// 4th revision: 2002-04-25

#ifndef _KEYVALUEMINMAX_H_
#define _KEYVALUEMINMAX_H_

#include "Entropy.h"

#define AACToTalBits() (codec._ostrm->Outbuf.nbytes) * 8 + (codec._ostrm->Outbuf.nbits)

class KeyValueMinMax  
	{
  public:
	// if bUsed32Float == 0
	unsigned int nWhichAxis;

	// if bUsed32Float == 1
	double fMin_X;
	double fMin_Y;
	double fMin_Z;
	double fMax;

	double fMax_X;
	double fMax_Y;
	double fMax_Z;

  public:
	KeyValueMinMax(void)
		{
		fMin_X = 0;
		fMin_Y = 0;
		fMin_Z = 0;
		fMax = 0;

		fMax_X = 0;
		fMax_Y = 0;
		fMax_Z = 0;
		}

	void makeBitstream(CAC_QF& codec, int nKeyValueDigit)
		{
		int nHeaderBits, nXMinBits, nYMinBits, nZMinBits, nDiffBits;
		int nBits;
		
		/////////////////////////////////////////////////
		//
		// Part 1. ʿ  Ѵ.
		//
		/////////////////////////////////////////////////
		
		// 1. Mantissa Digit(x, y, z) Ѵ.
		int nX_MantissaDigit, nY_MantissaDigit, nZ_MantissaDigit, nDiff_MantissaDigit;
		
		nX_MantissaDigit = GetDigit(fMin_X);
		nY_MantissaDigit = GetDigit(fMin_Y);
		nZ_MantissaDigit = GetDigit(fMin_Z);
		nDiff_MantissaDigit = GetDigit(fMax);

		///////////////////////////////////////////////////
		//
		//	ҸǴ bit Ͽ 128̸̻,
		//	  .
		//
		///////////////////////////////////////////////////
		if(GetTotalMinMaxBits(
			nX_MantissaDigit, nY_MantissaDigit, nZ_MantissaDigit,
			nDiff_MantissaDigit) > 128)
			{
			// PDAM4
			// 0 1 ȯ
			codec._ostrm->putbit(1);
			
			float fTemp;
			
			fTemp = (float) fMin_X;
			codec._ostrm->putbits(*((int*)&fTemp), 32);
			
			fTemp = (float) fMin_Y;
			codec._ostrm->putbits(*((int*)&fTemp), 32);
			
			fTemp = (float) fMin_Z;
			codec._ostrm->putbits(*((int*)&fTemp), 32);
			
			
			switch(nWhichAxis){
				case 0: // X
					fMax = fMax - fMin_X;
					break;
				case 1: // Y
					fMax = fMax - fMin_Y;
					break;
				case 2: // Z
					fMax = fMax - fMin_Z;
					break;
				}
			
			fTemp = (float) fMax;
			codec._ostrm->putbits(*((int*)&fTemp), 32);
			
			
			return ;
			}
		else
			{
			// PDAM4
			// 0 1 ȯ...
			codec._ostrm->putbit(0);
			}
		
		
		
		// 3.1.0   max ?
		codec._ostrm->putbits(nWhichAxis, 2);
		
		
		
		
		bool bNeedDiffDigit = true;
		
		switch(nWhichAxis) {
			case 0: // X
				if(nDiff_MantissaDigit == nX_MantissaDigit)
					{
					bNeedDiffDigit = false;
					nDiff_MantissaDigit = nX_MantissaDigit;
					}
				break;
			case 1: // Y
				if(nDiff_MantissaDigit == nY_MantissaDigit)
					{
					bNeedDiffDigit = false;
					nDiff_MantissaDigit = nY_MantissaDigit;
					}
				break;
			case 2: // Z
				if(nDiff_MantissaDigit == nZ_MantissaDigit)
					{
					bNeedDiffDigit = false;
					nDiff_MantissaDigit = nZ_MantissaDigit;
					}
				break;
			}
		
		// 2. Exponent Digit Sign  Ѵ.
		int nX_ExponentDigit, nY_ExponentDigit, nZ_ExponentDigit, nDiff_ExponentDigit;
		int nX_ExponentSign, nY_ExponentSign, nZ_ExponentSign, nDiff_ExponentSign;
		
		nX_ExponentDigit = GetLargestDigit(fMin_X);
		nY_ExponentDigit = GetLargestDigit(fMin_Y);
		nZ_ExponentDigit = GetLargestDigit(fMin_Z);
		nDiff_ExponentDigit = GetLargestDigit(fMax);
		
		nX_ExponentSign = (nX_ExponentDigit < 0)? 1: 0;
		nY_ExponentSign = (nY_ExponentDigit < 0)? 1: 0;
		nZ_ExponentSign = (nZ_ExponentDigit < 0)? 1: 0;
		nDiff_ExponentSign = (nDiff_ExponentDigit < 0)? 1: 0;
		
		
		/////////////////////////////////////////////////
		//
		// Part 2.   EncodingѴ.
		//
		/////////////////////////////////////////////////
		
		// 3. Float-Point Number ڵѴ.
		
		nHeaderBits = AACToTalBits();
		// 3.1 [Header Coding *******************************************]
		
		// 3.1.1   Digit ϴ°?
		bool bAllSameDigit = false;
		
		GetDigitBits(nX_MantissaDigit, nY_MantissaDigit, nZ_MantissaDigit, bAllSameDigit);
		
		codec._ostrm->putbit((bAllSameDigit)? 1: 0);
		
		// 3.1.2  Digit ƴϸ, Digit ش.
		if(bAllSameDigit)
			{
			if(nKeyValueDigit == nX_MantissaDigit)
				{
				codec._ostrm->putbit(1);
				}
			else
				{
				codec._ostrm->putbit(0);
				//codec._ostrm->putbits(nX_MantissaDigit, 3);
				codec._ostrm->putbits(nX_MantissaDigit, 4);
				}
			
			nY_MantissaDigit = nX_MantissaDigit;
			nZ_MantissaDigit = nX_MantissaDigit;
			}
		else
			{
			//codec._ostrm->putbits(nX_MantissaDigit, 3);
			//codec._ostrm->putbits(nY_MantissaDigit, 3);
			//codec._ostrm->putbits(nZ_MantissaDigit, 3);
			codec._ostrm->putbits(nX_MantissaDigit, 4);
			codec._ostrm->putbits(nY_MantissaDigit, 4);
			codec._ostrm->putbits(nZ_MantissaDigit, 4);
			}
		
		// 3.1.2.1
		// Digit ̰ 2̸̻ Diff Digit  .
		// 2̸̻ 2bit̻ ̵   ִ.
		if(bNeedDiffDigit)
			{
			codec._ostrm->putbit(1);
			//codec._ostrm->putbits(nDiff_MantissaDigit, 3);
			codec._ostrm->putbits(nDiff_MantissaDigit, 4);
			}
		else
			{
			codec._ostrm->putbit(0);
			//nDiff_MantissaDigit = nX_MantissaDigit;
			// 2001-1-8, shinjun: {
			switch(nWhichAxis)
				{
				case 0:
					nDiff_MantissaDigit = nX_MantissaDigit;
					break;
				case 1:
					nDiff_MantissaDigit = nY_MantissaDigit;
					break;
				case 2:
					nDiff_MantissaDigit = nZ_MantissaDigit;
					break;
				}
			// 2001-1-8, shinjun: }
			}
		
		// 3.1.3 Exponent Digit
		int nLargestExponentDigit = abs(nX_ExponentDigit);
		
		if(nLargestExponentDigit < abs(nY_ExponentDigit))
			nLargestExponentDigit = abs(nY_ExponentDigit);
		if(nLargestExponentDigit < abs(nZ_ExponentDigit))
			nLargestExponentDigit = abs(nZ_ExponentDigit);
		if(nLargestExponentDigit < abs(nDiff_ExponentDigit))
			nLargestExponentDigit = abs(nDiff_ExponentDigit);
		
		// [0 ~ 38]   CodingϹǷ 6 bit
		codec._ostrm->putbits(nLargestExponentDigit, 6);
		
		// 3.1.4 Exponent Sign  ?
		bool bAllSameSign = false;
		
		if(nX_ExponentSign == nY_ExponentSign)
			{
			if(nX_ExponentSign == nZ_ExponentSign)
				{
				if(nX_ExponentSign == nDiff_ExponentSign)
					{
					bAllSameSign = true;
					}
				}
			}
		
		if(bAllSameSign)
			{
			//   Sign̴.
			codec._ostrm->putbit(1);
			
			//   sign Ѵ.
			codec._ostrm->putbit(nX_ExponentSign);
			}
		else
			{
			//   Sign ƴϴ.
			codec._ostrm->putbit(0);
			}
		nHeaderBits = AACToTalBits() - nHeaderBits;
		
		
		nXMinBits = AACToTalBits();
		// 3.2 [X MIN *******************************************]
		
		int nMantissa;
		int nExponent;
		
		// digit '0'̶  0.0 en/decoding϶ !
		if(nX_MantissaDigit != 0)
			{
			// 3.2.1 Sign Coding
			codec._ostrm->putbit((fMin_X < 0)? 1: 0);
			
			// 3.2.2 Mantissa Coding
			nExponent = GetLargestDigit(fMin_X);
			
			nMantissa = GetMantissa(fMin_X, nX_MantissaDigit);
			nBits = (int) (log(pow(10, nX_MantissaDigit) - 1)/log(2) + 1);
			
			if(nMantissa < 0)
				nMantissa = -nMantissa;
			codec._ostrm->putbits(nMantissa, nBits);
			
			// 3.2.3 Exponent Coding
			nBits = getNbBits(nLargestExponentDigit);
			codec._ostrm->putbits(abs(nExponent), nBits);
			
			if(!bAllSameSign)
				codec._ostrm->putbit(nX_ExponentSign);
			}
		
		nXMinBits = AACToTalBits() - nXMinBits;
		
		nYMinBits = AACToTalBits();
		// 3.3 [Y MIN *******************************************]
		
		// digit '0'̶  0.0 en/decoding϶ !
		if(nY_MantissaDigit != 0)
			{
			// 3.3.1 Sign Coding
			codec._ostrm->putbit((fMin_Y < 0)? 1: 0);
			
			nExponent = GetLargestDigit(fMin_Y);
			
			// 3.3.2 Mantissa Coding
			nMantissa = GetMantissa(fMin_Y, nY_MantissaDigit);
			nBits = (int) (log(pow(10, nY_MantissaDigit) - 1)/log(2) + 1);
			
			if(nMantissa < 0)
				nMantissa = -nMantissa;
			codec._ostrm->putbits(nMantissa, nBits);
			
			// 3.3.3 Exponent Coding
			
			nBits = getNbBits(nLargestExponentDigit);
			codec._ostrm->putbits(abs(nExponent), nBits);
			
			if(!bAllSameSign)
				codec._ostrm->putbit(nY_ExponentSign);
			}
		
		nYMinBits = AACToTalBits() - nYMinBits;
		
		nZMinBits = AACToTalBits();
		// 3.4 [Z MIN *******************************************]
		
		// digit '0'̶  0.0 en/decoding϶ !
		if(nZ_MantissaDigit != 0)
			{
			// 3.4.1 Sign Coding
			codec._ostrm->putbit((fMin_Z < 0)? 1: 0);
			
			nExponent = GetLargestDigit(fMin_Z);
			
			// 3.4.2 Mantissa Coding
			nMantissa = GetMantissa(fMin_Z, nZ_MantissaDigit);
			nBits = (int) (log(pow(10, nZ_MantissaDigit) - 1)/log(2) + 1);
			
			if(nMantissa < 0)
				nMantissa = -nMantissa;
			codec._ostrm->putbits(nMantissa, nBits);
			
			// 3.4.3 Exponent Coding
			
			nBits = getNbBits(nLargestExponentDigit);
			codec._ostrm->putbits(abs(nExponent), nBits);
			
			if(!bAllSameSign)
				codec._ostrm->putbit(nZ_ExponentSign);
			}
		
		nZMinBits = AACToTalBits() - nZMinBits;
		
		nDiffBits = AACToTalBits();
		// 3.4 [Max Diff *******************************************]
		
		// digit '0'̶  0.0 en/decoding϶ !
		if(nDiff_MantissaDigit != 0)
			{
			// 3.4.1 Sign Coding
			codec._ostrm->putbit((fMax < 0)? 1: 0);
			
			nExponent = GetLargestDigit((double)fMax);
			
			// 3.4.2 Mantissa Coding
			nMantissa = GetMantissa(fMax, nDiff_MantissaDigit);
			nBits = (int) (log(pow(10, nDiff_MantissaDigit) - 1)/log(2) + 1);
			
			if(nMantissa < 0)
				nMantissa = -nMantissa;
			codec._ostrm->putbits(nMantissa, nBits);
			
			// 3.4.3 Exponent Coding
			
			nBits = getNbBits(nLargestExponentDigit);
			codec._ostrm->putbits(abs(nExponent), nBits);
			
			if(!bAllSameSign)
				codec._ostrm->putbit(nDiff_ExponentSign);
			}
		}
		
	void encodingProcess(void)
		{
		}
		
  private:
	  // sowoo 2001.10.11 (Min/Max Reduce)
	  // Ǽ Digit 
	  int GetDigit(double dValue)
		  {
		  //////////////////////////////////////////////////
		  //
		  //			Original Value 7ڸ !!!
		  //			꿡 ؼ   
		  //
		  //////////////////////////////////////////////////
		  
		  char buffer[50];
		  _gcvt(dValue, cnMAX_DIGIT, buffer);
		  dValue = atof(buffer);
		  
		  //
		  //////////////////////////////////////////////////
		  
		  // "0.0"  ڸ̴.
		  if(dValue == 0.0)
			  return 0;
		  
		  int nFirstDigit, nLastDigit;
		  
		  nFirstDigit = GetLeadingZero(dValue);
		  nLastDigit = GetLargestDigit(dValue);
		  
		  int nDigit = nLastDigit - nFirstDigit;
		  
		  // 7 ū    .
		  //  floating-point number ϴ ȿڸ 6~7!
		  // ƴ,  ̻   .
		  return (nDigit > cnMAX_DIGIT)? cnMAX_DIGIT: nDigit;
		  }

	  int GetTotalMinMaxBits(
		  int nDigit_X, int nDigit_Y, int nDigit_Z,
		  int nDigit_diff
		  )
		  {
		  int nTemp;
		  
		  nTemp = 4 + (int) (log(pow(10, nDigit_X) - 1)/log(2) + 1) + 7;
		  nTemp += 4 + (int) (log(pow(10, nDigit_Y) - 1)/log(2) + 1) + 7;
		  nTemp += 4 + (int) (log(pow(10, nDigit_Z) - 1)/log(2) + 1) + 7;
		  nTemp += 4 + (int) (log(pow(10, nDigit_diff) - 1)/log(2) + 1) + 7;
		  
		  return nTemp;
		  }

	  int GetLargestDigit(double key)
		  {
		  //////////////////////////////////////////////////
		  //
		  //			Original Value 7ڸ !!!
		  //			꿡 ؼ   
		  //
		  //////////////////////////////////////////////////
		  
		  char buffer[50];
		  _gcvt(key, cnMAX_DIGIT, buffer);
		  key = atof(buffer);
		  
		  //
		  //////////////////////////////////////////////////
		  
		  double a;
		  __int64 mult;
		  double dmult;
		  __int64 Ia;
		  double e = dEPSILONERROR;
		  int nLargestDigit;
		  
		  mult=1; dmult=0.1;
		  nLargestDigit = 0;
		  
		  a = fabs(key);
		  Ia = (__int64) (a + mult*e);
		  
		  // sowoo 2001.10.11 (Digit   ϱ)
		  // "0.1" binary ǥ ⶧ ǵ  ʴ  .
		  // 0.1 ϴ  ſ 10  scheme !
		  if(a == 0.0)
			  return 1;
		  
		  double dTemp = a - (double)Ia;
		  
		  if((dTemp == 0.0) || (dTemp < e))
			  {
			  while(true)
				  {
				  a = fabs(key);
				  a *= dmult;
				  Ia = (__int64) (a + e);
				  dTemp = a - (double)Ia;
				  
				  if(dTemp > e)
					  return nLargestDigit;
				  
				  dmult *= 0.1;
				  nLargestDigit--;
				  }
			  }
		  
		  //while(a - (double) Ia > e*mult)
		  while(dTemp > e*mult)
			  {
			  mult*=10;
			  nLargestDigit++;
			  a = fabs(key);
			  a=(double) mult *a;
			  Ia = (__int64) (a + mult*e);
			  dTemp = a - (double)Ia;
			  }
		  
		  return nLargestDigit;
		  }

	  // sowoo 2001.10.11 (Min/Max Reduce)
	  //  Ȯ  ƴϴ.
	  int GetDigitBits(int nXDigit, int nYDigit, int nZDigit, bool& bIsOneDigit)
		  {
		  int nBits;
		  int nMaxDigit = nXDigit, nMinDigit = nXDigit;
		  
		  if(nMaxDigit < nYDigit)
			  nMaxDigit = nYDigit;
		  if(nMaxDigit < nZDigit)
			  nMaxDigit = nZDigit;
		  
		  if(nMinDigit > nYDigit)
			  nMinDigit = nYDigit;
		  if(nMinDigit > nZDigit)
			  nMinDigit = nZDigit;
		  
		  if(nMinDigit == nMaxDigit)
			  {
			  // 1.  digit !!!
			  nBits = 1;
			  bIsOneDigit = true;
			  
			  return nBits;
			  }
		  else
			  {
			  bIsOneDigit = false;
			  
			  return 1 + 3 + 3;
			  /*		int nDiffDigit = nMaxDigit - nMinDigit;
			  
				// 2. digit   ̵ .   ʴ´.
				if(nDiffDigit == 1)
				{
				nBits = 1;
				bIsOneDigit = true;
				
				  return nBits;
				  }
				  
					nBits = 1 + 3 + 3;
			  */	}
			  
			  return nBits;
		  }

	  // double Mantissa(10) ϴ α׷!!!
	  // input:
	  // -> dValue: Source floating point 
	  // -> nDigit:  ڸ
	  int GetMantissa(double dValue, int nDigit)
		  {
		  char* buffer;
		  int nDummy1 = 0, nDummy2 = 0;
		  
		  buffer = _ecvt(dValue, nDigit, &nDummy1, &nDummy2);
		  
		  return ::atoi(buffer);
		  }

	  int GetLeadingZero(double key)
		  {
		  //////////////////////////////////////////////////
		  //
		  //			Original Value 7ڸ !!!
		  //			꿡 ؼ   
		  //
		  //////////////////////////////////////////////////
		  
		  char buffer[50];
		  _gcvt(key, cnMAX_DIGIT, buffer);
		  key = atof(buffer);
		  
		  //
		  //////////////////////////////////////////////////
		  
		  double a;
		  double e = dEPSILONERROR;
		  __int64 nMulti = 10;
		  double dMulti = 0.1;
		  int nCount = 0;
		  
		  double dTemp;
		  
		  // 1. 켱  ٲٰ,
		  a = fabs(key);
		  
		  // 2. 0̰ų 0.1 ũ տ "0" .
		  if(a >= 1.0)
			  {
			  nCount++;
			  while(a*dMulti >= 1.0)
				  {
				  nCount++;
				  dMulti *= 0.1;
				  }
			  
			  return -nCount;
			  }
		  else if(a == 0.0)
			  return 0;
		  else if(a >= 0.1)
			  return 0;
		  else
			  {
			  nCount++;
			  dTemp = a * nMulti;
			  _gcvt(dTemp, 7, buffer);
			  dTemp = ::atof(buffer);
			  
			  //while(a*nMulti < 0.1)
			  while( dTemp < 0.1)
				  {
				  nCount++;
				  nMulti *= 10;
				  dTemp = a * nMulti;
				  _gcvt(dTemp, 7, buffer);
				  dTemp = ::atof(buffer);
				  }
			  
			  return nCount;
			  }
		  }

	int getNbBits(int k)
		{
		int j=1;
		while (k >= (1<<j) ) 
			{
			j++;
			}
		return j;
		}
	};
	
#endif // _KEYVALUEMINMAX_H_
