// Linear key encoder class
// Date: 2002. 4. 23. Wed.
// Author: James Kim
// Editor: Shinjun Lee

#ifndef _LINEARKEY_H_
#define _LINEARKEY_H_

#include <float.h>

#include "../include/KeyMinMax.h"
#include "../include/JWArray.h"

// Linear key encoder
class LinearKey
	{
  public:
	int nNumLinearKeyCodingBit;
	int nNumberOfLinearKey;
	KeyMinMax kMinMax;

	double fKeyMin;
	double fKeyMax;

  public:
	LinearKey(void)
		{
		nNumLinearKeyCodingBit = 0;
		nNumberOfLinearKey = 0;

		fKeyMin = 0;
		fKeyMax = 0;
		}

	void makeBitstream(CAC_QF& codec, int nKeyDigit, int& KTotal)
		{
		nNumLinearKeyCodingBit = getNbBits(nNumberOfLinearKey);
		codec._ostrm->putbits(nNumLinearKeyCodingBit, 5);
		KTotal += 5;

		codec._ostrm->putbits(nNumberOfLinearKey, nNumLinearKeyCodingBit);
		KTotal += nNumLinearKeyCodingBit;

		kMinMax.fKeyMin = fKeyMin;
		kMinMax.fKeyMax = fKeyMax;

		kMinMax.makeBitstream(codec, nKeyDigit, KTotal);
		}

	// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
	//
	//
	//		Key Analyzer : 
	//		ü  1/2 ̻Ǵ key   key sub set߿,,
	//		ϴ error ȭ   ũ  ϰ Ǵ subset ã
	//		 ߿  error  subset ã´.
	//		 ϳ sublinear  ã Ǿ ִ.
	//
	//
	// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
	void LinearKeySubRegionExtractor(
		int nKeyQBits,
		int maxDigit,
		bool m_bIsKeyLinearCoder,
		bool m_bPreserveKey,
		int& preStartKeyIdx,
		int& preEndKeyIdx, 
		JWArray<double>& JWOriginalKeys,
		JWArray<double>& JWSelectedKeys)
		{
		double				max_error = -DBL_MAX;
		int					startIdx, endIdx;
		int					limit;

		double				preSubSetError = DBL_MAX;

		if(m_bIsKeyLinearCoder)
			{
			double QuantLimit = pow(2.0, -(double)(nKeyQBits+1));
			if(m_bPreserveKey)
				{
				startIdx = 0;
				limit = (int)((double)(JWOriginalKeys.GetSize()-1)/2.0);

				for(int idx = JWOriginalKeys.GetSize()-1;idx > limit;idx--)
					{
					startIdx = 0;
					endIdx = idx;
					max_error = -DBL_MAX;
					do{
						double pre_calc = (JWOriginalKeys[endIdx]-JWOriginalKeys[startIdx])
							/(double)(endIdx - startIdx);
						// ü  straight lineΰ ?
						for(int keyidx=startIdx;keyidx <= endIdx;keyidx++)
							{
							double GeneratedKey = pre_calc*(double)(keyidx-startIdx) + JWOriginalKeys[startIdx];

							char num[50];
							_gcvt(GeneratedKey, maxDigit, num);
							GeneratedKey = atof(num);		

							double key_error = fabs( GeneratedKey - JWOriginalKeys[keyidx] );

							if(key_error > max_error)
								{
								max_error = key_error;
								}
							if(max_error >= QuantLimit)
								break;
							}

						//   key subset peak error 
						// ȭ   ũ error   ˻Ѵ.
						if(max_error < QuantLimit)
							{
							// ׷ ߿   error  subset ã´.
							if(preSubSetError > max_error)
								{
								preSubSetError = max_error;
								preStartKeyIdx = startIdx;
								preEndKeyIdx = endIdx;
								//goto TT; // ӽ
								return;
								}
							}
						startIdx++;
						endIdx++;
						max_error = -DBL_MAX;
						
						}while(endIdx <= (JWOriginalKeys.GetSize()-1));
					}
				}
			// KEY preserving ʿ䰡   ....
			else
				{
				startIdx = 0;
				limit = (int)((double)(JWSelectedKeys.GetSize()-1)/2.0);

				for(int idx = JWSelectedKeys.GetSize()-1;idx > limit;idx--)
					{
					startIdx = 0;
					endIdx = idx;
					max_error = -DBL_MAX;
					do{
						double pre_calc = (JWSelectedKeys[endIdx]-JWSelectedKeys[startIdx])
							/(double)(endIdx - startIdx);
						// ü  straight lineΰ ?
						for(int keyidx=startIdx;keyidx <= endIdx;keyidx++)
							{
							double GeneratedKey = pre_calc*(double)(keyidx-startIdx) + JWSelectedKeys[startIdx];
							char num[50];
							_gcvt(GeneratedKey, maxDigit, num);
							GeneratedKey = atof(num);		
							
							double key_error = fabs( GeneratedKey - JWSelectedKeys[keyidx] );
							
							if(key_error > max_error)
								{
								max_error = key_error;
								}
							if(max_error >= QuantLimit)
								break;
							}

						//   key subset peak error 
						// ȭ   ũ error   ˻Ѵ.
						if(max_error < QuantLimit)
							{
							// ׷ ߿   error  subset ã´.
							if(preSubSetError > max_error)
								{
								preSubSetError = max_error;
								preStartKeyIdx = startIdx;
								preEndKeyIdx = endIdx;
								//goto TT; // ӽ
								return;
								}
							}
						startIdx++;
						endIdx++;
						max_error = -DBL_MAX;

						}while(endIdx <= (JWSelectedKeys.GetSize()-1));
					} // end for
				} // end else
			} // end if m_bIsKeyLinearCoder

		return;
		}

	// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
	//
	//
	// Key Splitter :
	// Linear Key Analyzer ã key susbet ȣȭ ϰ  key 
	//  keycoder ش.
	//
	//
	// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
	void KeySplitter(bool m_bPreserveKey,
					int preStartKeyIdx,
					int preEndKeyIdx,
					int maxDigit,
					JWArray<double>& JWOriginalKeys,
					JWArray<double>& JWSelectedKeys,
					JWArray<double>& JWDecodedKey,
					double& dKeyMin,
					double& dKeyMax,
					int& nNumberOfLinearKey)
		{
		int					is_colinear = -1;
		int					startIdx, endIdx;
		JWArray<double>		JWTempOriginalKeys;
		JWArray<double>		JWTempDecodedKey;

		if(m_bPreserveKey)
			{
			if((preStartKeyIdx != -1) && ( preEndKeyIdx != -1))
				{
				startIdx = preStartKeyIdx;
				endIdx = preEndKeyIdx;
				int num_key = endIdx - startIdx + 1;

				// extract: data of linear key encoding
				nNumberOfLinearKey = num_key;
				dKeyMin = JWOriginalKeys[startIdx];
				dKeyMax = JWOriginalKeys[endIdx];

				// decodekey for error measure
				JWTempDecodedKey.RemoveAll();
				JWTempDecodedKey.SetSize(num_key);
				for(int decIdx = 0;decIdx < num_key;decIdx++)
					{
					double GeneratedKey = ((JWOriginalKeys[endIdx]-JWOriginalKeys[startIdx])/(double)(num_key-1))*decIdx;

					JWTempDecodedKey[decIdx] = JWOriginalKeys[startIdx] + GeneratedKey;

					char num[50];
					_gcvt(JWTempDecodedKey[decIdx] , maxDigit, num);
					JWTempDecodedKey[decIdx]  = atof(num);		

					//JWTempDecodedKey[decIdx] = JWOriginalKeys[startIdx] +
					//	((JWOriginalKeys[endIdx]-JWOriginalKeys[startIdx])/(double)(num_key-1))*decIdx;
					}
				
				if(JWDecodedKey.GetSize() == 0)
					{
					for(int decIdx = 0;decIdx < num_key;decIdx++)
						{
						JWDecodedKey.Add(JWTempDecodedKey[decIdx]);
						}
					}
				else
					{
					for(int decIdx = 0;decIdx < num_key;decIdx++)
						for(int dectmpIdx = 0;dectmpIdx < JWDecodedKey.GetSize();dectmpIdx++)
							{
							// Ǿ̸
							if((dectmpIdx == 0) && (JWDecodedKey[dectmpIdx] >= JWTempDecodedKey[decIdx]))
								{
								JWDecodedKey.InsertAt(0, JWTempDecodedKey[decIdx]);
								}
							else if(JWDecodedKey[dectmpIdx] < JWTempDecodedKey[decIdx]) 
								{
								if((dectmpIdx+1) < JWDecodedKey.GetSize())
									{
									if(JWDecodedKey[dectmpIdx+1] >= JWTempDecodedKey[decIdx])
										JWDecodedKey.InsertAt(dectmpIdx+1, JWTempDecodedKey[decIdx]);
									}
								else
									JWDecodedKey.Add(JWTempDecodedKey[decIdx]);
								}
							}
					}

				//  key 
				JWTempOriginalKeys.RemoveAll();
				for(int tempidx = 0;tempidx < startIdx;tempidx++)
					{
					JWTempOriginalKeys.Add(JWOriginalKeys[tempidx]);
					}
				for(tempidx = endIdx+1;tempidx < JWOriginalKeys.GetSize();tempidx++)
					{
					JWTempOriginalKeys.Add(JWOriginalKeys[tempidx]);	
					}

				JWOriginalKeys.RemoveAll();
				for(tempidx = 0;tempidx < JWTempOriginalKeys.GetSize();tempidx++)
					{
					JWOriginalKeys.Add(JWTempOriginalKeys[tempidx]);		
					}
				}
			}
		else
			{
			if((preStartKeyIdx != -1) && ( preEndKeyIdx != -1))
				{
				startIdx = preStartKeyIdx;
				endIdx = preEndKeyIdx;
				int num_key = endIdx - startIdx + 1;

				// extract: data of linear key encoding
				nNumberOfLinearKey = num_key;
				dKeyMin = JWSelectedKeys[startIdx];
				dKeyMax = JWSelectedKeys[endIdx];

				// decodekey for error measure
				JWTempDecodedKey.RemoveAll();
				JWTempDecodedKey.SetSize(num_key);
				for(int decIdx = 0;decIdx < num_key;decIdx++)
					{
					double GeneratedKey = ((JWSelectedKeys[endIdx]-JWSelectedKeys[startIdx])/
						(double)(num_key-1))*decIdx;

					JWTempDecodedKey[decIdx] = JWSelectedKeys[startIdx] + GeneratedKey;

					char num[50];
					_gcvt(JWTempDecodedKey[decIdx] , maxDigit, num);
					JWTempDecodedKey[decIdx]  = atof(num);		
					
					//JWTempDecodedKey[decIdx] = JWOriginalKeys[startIdx] +
					//	((JWOriginalKeys[endIdx]-JWOriginalKeys[startIdx])/(double)(num_key-1))*decIdx;
					}

				if(JWDecodedKey.GetSize() == 0)
					{
					for(int decIdx = 0;decIdx < num_key;decIdx++)
						{
						JWDecodedKey.Add(JWTempDecodedKey[decIdx]);
						}
					}
				else
					{
					for(int decIdx = 0;decIdx < num_key;decIdx++)
						for(int dectmpIdx = 0;dectmpIdx < JWDecodedKey.GetSize();dectmpIdx++)
							{
							// Ǿ̸
							if((dectmpIdx == 0) && (JWDecodedKey[dectmpIdx] >= JWTempDecodedKey[decIdx]))
								{
								JWDecodedKey.InsertAt(0, JWTempDecodedKey[decIdx]);
								}
							else if(JWDecodedKey[dectmpIdx] < JWTempDecodedKey[decIdx]) 
								{
								if((dectmpIdx+1) < JWDecodedKey.GetSize())
									{
									if(JWDecodedKey[dectmpIdx+1] >= JWTempDecodedKey[decIdx])
										JWDecodedKey.InsertAt(dectmpIdx+1, JWTempDecodedKey[decIdx]);
									}
								else
									JWDecodedKey.Add(JWTempDecodedKey[decIdx]);
								}
							}
					}

				//  key 
				JWTempOriginalKeys.RemoveAll();
				for(int tempidx = 0;tempidx < startIdx;tempidx++)
					{
					JWTempOriginalKeys.Add(JWSelectedKeys[tempidx]);
					}
				for(tempidx = endIdx+1;tempidx < JWSelectedKeys.GetSize();tempidx++)
					{
					JWTempOriginalKeys.Add(JWSelectedKeys[tempidx]);	
					}

				JWSelectedKeys.RemoveAll();
				for(tempidx = 0;tempidx < JWTempOriginalKeys.GetSize();tempidx++)
					{
					JWSelectedKeys.Add(JWTempOriginalKeys[tempidx]);		
					}
				}
			}
		}

  private:
	// 2002-04-17, Shinjun
	// This function is for linear key encoding
	int getNbBits(int k)
		{
		int j=1;
		while (k >= (1<<j) ) 
			j++;
		
		return j;
		}

	};

#endif // _LINEARKEY_H_