// cStream.h
// Editor: Shinjun Lee, Samsung AIT
// Date: 2001-12-27

#ifndef _CSTREAM_H_
#define _CSTREAM_H_

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>

#define M_SIZE 1024

class CBuffer
	{
  public:
	int				size;
	unsigned char*	buf;
	int				nbits;
	int				nbytes;
	};

class CInStream
	{
  public:
	CBuffer			Inbuf;
	char*			_Fname;
	int				num_node;

	//AAC sungjin
	unsigned char   _buf;
    int             _ibits;
	bool            _align;

  public:
	CInStream(void)
		{
		Inbuf.size=0;
		Inbuf.buf=NULL;
		Inbuf.nbits=0;
		Inbuf.nbytes=0;
		_Fname=NULL;
		num_node = 0;
		
		//AAC sungjin
		_ibits=8;
		_buf = 0x00;
		_align = false;
		}

	bool opendec(char* fname)  //AAC sungjin 
		{
		bool success = false;
		int i=1, t_size, ch;
		
		//read the binary key & keyValue file
		if((Inbuf.buf=(unsigned char *)malloc(M_SIZE*sizeof(char)))==NULL)
			return success;
		if(_Fname) 
			_Fname = NULL; 
		if(fname!=NULL)
			{
			FILE *fp = fopen(fname,"rb");
			if(fp!=(FILE*)0) 
				{
				_Fname = strdup(fname);
				while(!feof(fp)) 
					{
					ch=fgetc(fp);
					Inbuf.buf[Inbuf.size++] = (unsigned char)ch;
					if((t_size=M_SIZE*i)==(getsize()+1)) 
						{
						i++;
						if((Inbuf.buf=(unsigned char *)realloc(Inbuf.buf,(getsize()+1)+(M_SIZE*sizeof(char))))==NULL)
							return success; 
						}
					}
				success = true;
				} 
			fclose(fp);
			}
		return success;
		}

	int	getbit(void)
		{
		int res = (Inbuf.buf[getnbytes()] >> (7-getnbits())) & 1;
		Inbuf.nbits++;
		if (getnbits() == 8) 
			{
			putnbits(0);
			Inbuf.nbytes++;
			}
		return res;
		}

	int getbits(int n)
		{
		int res = 0;
		for (int i=0; i<n; i++)
			{
			res <<= 1;
			res += getbit();
			}
		return res;
		}

	int	getsize(void)
		{
		return Inbuf.size;
		}

	int getnbits(void)
		{
		return Inbuf.nbits;
		}

	int	getnbytes(void)
		{
		return Inbuf.nbytes;
		}

	char getbuf(int n)
		{
		return Inbuf.buf[n];
		}

	bool getUChar(unsigned char& c)
		{
		c = (getnbytes() < getsize()) ? Inbuf.buf[getnbytes()] : 0;
		Inbuf.nbytes++;
		return true;
		}

	void putsize(int n)
		{
		Inbuf.size=n;
		}

	void putnbits(int n)
		{
		Inbuf.nbits=n;
		}

	void putnbytes(int n)
		{
		Inbuf.nbytes=n;
		}

	void bfree(void)
		{
		free(Inbuf.buf);
		}

	//AAC sungjin
	bool get32BitFloat(float& x)
		{
		int& ix = *(int*)(&x);
		return getMBitInt(32,ix);
		}

	bool getMBitInt(int nbits, int& mbi)
		{
		mbi = 0;  // Most significant bit first
		bool bit = false;
		
		_align=false;
		_buf = 0x00;
		_buf |= Inbuf.buf[getnbytes()];
		_ibits = 8-Inbuf.nbits;
		for(int i=0;i<nbits;i++) 
			{
			if(getBitf(bit, i)==false) return false;
			mbi <<= 1;
			if(bit!=false) mbi |= 1;
			}
		if(_align==false) 
			{
			if(_ibits==8) 
				{
				Inbuf.nbits = 8-_ibits;
				Inbuf.nbytes++;    
				} 
			else 
				{
				Inbuf.nbits = 8-_ibits;				
				}
			} 
		else
			{
			Inbuf.nbits = 8-_ibits;
			}
		_ibits=8;
		_buf = 0x00;
		_align = false;
		
		return true;
		}

	bool getBitf(bool& bit, int i)
		{
		bool success = true;
		_align=false;
		if(_ibits==8) 
			{
			if(i != 0)
				Inbuf.nbytes++;
			success = getUChar(_buf);
			Inbuf.nbytes--;   
			}
		if(!(success==false))
			{
			bit = (_buf & (1<<--_ibits)) ? true : false; // MSB first, FJB 7/5/98
			if(_ibits==0) 
				{
				_ibits=8;
				// CS 021799
				if(_buf == 0x00)
					{
					//if(i != 0)
					Inbuf.nbytes++;
					getUChar(_buf);
					Inbuf.nbytes--;   
					_ibits=7;         
					_align=true;
					}
				} 
			}
		return success;
		}
	
	bool flush()
		{
		if(_ibits!=8)
			_ibits = 8;

		return true;
		}
	};

class COutStream
	{
  public:
	CBuffer			Outbuf;
	char*			_Fname;
	unsigned char	_buf;
	int				_ibits;
	bool			_align;

	COutStream(void)
		{
		Outbuf.size=0;
		Outbuf.buf=NULL;
		Outbuf.nbits=0;
		Outbuf.nbytes=0;
		_Fname=NULL;
		
		//AAC sungjin
		_ibits=8;
		_buf = 0x00;
		_align = false;
		}

	bool openenc(char* fname)
		{
		if(fname!=NULL)
			{
			_Fname = strdup(fname);
			if( (Outbuf.buf=(unsigned char *)malloc(M_SIZE*sizeof(char)))==NULL)
				return false;
			memset(Outbuf.buf,0x0,M_SIZE);
			Outbuf.size = _msize(Outbuf.buf);
			return true;
			}
		return false;
		}

	bool close_enc(void)
		{
		bool success = false;
		if(_Fname!=NULL)
			{
			FILE *fp = fopen(_Fname,"wb");
			if(fp!=(FILE*)0)
				{
				int  nBytes = getnbytes()-1;
				int  nBytesWritten = fwrite(Outbuf.buf,1,nBytes,fp);
				success = (nBytesWritten<nBytes)?false:true;
				fclose(fp);
				}
			}
		return success;
		}

	void putbit(int b)
		{
		if (getnbytes() == getsize()-1) 
			{
			Outbuf.buf=(unsigned char *)realloc(Outbuf.buf,getsize()+(M_SIZE*sizeof(char)));
			Outbuf.size = _msize(Outbuf.buf);
			}
		if(getnbits()==0)
			Outbuf.buf[getnbytes()] &= 0x00;
		Outbuf.buf[getnbytes()] |= (b<<(7-getnbits()));   //MSB First
		Outbuf.nbits++;
		if (getnbits() == 8) 
			{
			putnbits(0);
			Outbuf.nbytes++;
			if (getnbytes() == getsize()-1) 
				{
				Outbuf.buf=(unsigned char *)realloc(Outbuf.buf,getsize()+(M_SIZE*sizeof(char)));
				Outbuf.size = _msize(Outbuf.buf);
				}
			}
		}

	void putbits(int v, int n)
		{
		for (int i=0; i<n; i++)
			putbit((v >> (n-i-1)) & 1);
		}

	int getsize(void)
		{
		return Outbuf.size;
		}

	int getnbits(void)
		{
		return Outbuf.nbits;
		}

	int getnbytes(void)
		{
		return Outbuf.nbytes;
		}

	void putsize(int n)
		{
		Outbuf.size=n;
		}

	void putnbits(int n)
		{
		Outbuf.nbits=n;
		}

	void putnbytes(int n)
		{
		Outbuf.nbytes=n;
		}

	void putbuf(int n, unsigned char& c)
		{
		Outbuf.buf[n]=c;
		}

	bool putUChar(unsigned char c)
		{
		if (getnbytes() == getsize()-1) 
			{
			Outbuf.buf=(unsigned char *)realloc(Outbuf.buf,getsize()+(M_SIZE*sizeof(char)));
			Outbuf.size = _msize(Outbuf.buf);
			}
		Outbuf.buf[getnbytes()]=c;
		Outbuf.nbytes++;
		return true;
		}

	void bfree(void) 
		{
		free(Outbuf.buf);
		}

	bool put32BitFloat(float x)
		{
		int ix = *(int*)(&x);
		return putMBitInt(32,ix);
		}

	bool putMBitInt(int nbits, int mbi)
		{
		// Most significant bit first
		bool bit = false;
		_align=false;
		_buf = 0x00;
		unsigned char mask = 0xff << (8-getnbits());
		Outbuf.buf[getnbytes()] &= mask;
		_buf |= Outbuf.buf[getnbytes()];
		_ibits = 8-Outbuf.nbits;  
		for(int i=0;i<nbits;i++)
			{
			bit = (mbi & (1<<(nbits-i-1)))?true:false;
			if(putBitf(bit)==false) return false;
			}
		if(_align==false) 
			{
			putUChar(_buf);
			Outbuf.nbits = 8-_ibits;
			Outbuf.nbytes--;
			} 
		else 
			{
			if(_buf==0x80 && _ibits==7) 
				{
				putUChar(_buf);
				Outbuf.nbits = 8-_ibits;
				Outbuf.nbytes--;
				} 
			else 
				{
				Outbuf.nbits=0;
				}
			}
		_align=false;
		_buf = 0x00;
		_ibits = 8;
		
		return true;
		}

	bool putBitf(bool bit)
		{
		--_ibits;
		if(bit!=false) 
			_buf |= (1<<_ibits); // MSB first FJB 7/5/98
		if(_ibits==0) 
			{
			if(putUChar(_buf)==false) return false;
			// CS 021799
			if(_buf != 0x00)
				{
				_buf = 0x00;
				_ibits = 8;
				}
			else 
				{
				_buf = 0x80;
				_ibits = 7;	  	  
				}
			_align=true;
			} 
		else 
			{
			_align=false;
			}

		return true;
		}

	//AAC sungjin
	bool    flush()
		{
		if(_ibits!=8)
			{
			if(putUChar(_buf)==false) return false;
			_buf = 0x00;
			_ibits = 8;
			}
		return true;
		}
	};

#endif // _STREAM_H_