// CAAC.h
// Editor: Shinjun Lee, Samsung AIT
// Date: 2001-12-27

#include "Entropy.h"

// AAC encoder
class CAACEncoder
	{
private:
	bool m_bSign;											// Sign in AAC

public:
	// Constructor
	CAACEncoder(void)
		{
		m_bSign = false;
		}

	// Sign: setting sign (default: true)
	void setSign(bool bSign = true)
		{
		m_bSign = bSign;
		}

	// Sign: getting sign
	bool getSign(void)
		{
		return m_bSign;
		}

	// Key: encoding key
	void encodeKey(CAC_QF& codec, int qstep, int dQ)
		{
		if(m_bSign == false)
			encodeKeyWithoutSign(codec, qstep, dQ);
		else
			encodeKeyWithSign(codec, qstep, dQ);
		}

	// Key: encoding key without sign
	void encodeKeyWithoutSign(CAC_QF& codec, int qstep, int dQ)
		{
		int i;
		for(i=qstep-1;i>=0;i--)
			{
			int val = (dQ >= 0) ? dQ : -dQ;
			val >>= i;
			int bit = val & 0x1;
			
			if(val<=1)
				codec.encode(bit, CQF_Status::ci_k_key_context[i]);
			else
				codec.encode(bit, CQF_Status::ci_k_key_context[i]);
			}
		}

	// Key: encoding key with sign
	void encodeKeyWithSign(CAC_QF& codec, int qstep, int dQ)
		{
		int i;
		for(i=qstep-1;i>=0;i--)
			{
			int val = (dQ >= 0) ? dQ : -dQ;
			int sgn = (dQ >= 0) ? 0 : 1;
			val >>= i;
			int bit = val & 0x1;

			if(val<=1)
				codec.encode(bit, CQF_Status::ci_k_key_context[i]);
			else
				codec.encode(bit, CQF_Status::ci_k_key_context[i]);
			if(val==1)            //if first sign bit
				codec.encode(sgn, CQF_Status::ci_k_sign_context);
			}
		}

	// KeyValue: encoding keyValue with AAC
	void encodeKeyValue(CAC_QF& codec, int qstep, int e, int nD)
		{
		int i;
		int x = e;
		for(i=qstep-1;i>=0;i--)
			{
			int val = (x >= 0) ? x : -x;
			int sgn = (x >= 0) ? 0 : 1;
			val >>= i;
			int bit = val & 0x1;
			if(val<=1)
				{
				if(nD==0)
					codec.encode(bit, CQF_Status::ci_kv_keyValue_x_context[i]);
				else if(nD==1)
					codec.encode(bit, CQF_Status::ci_kv_keyValue_y_context[i]);
				else if(nD==2)
					codec.encode(bit, CQF_Status::ci_kv_keyValue_z_context[i]);
				}
			else
				{
				codec.encode(bit);
				}
			if(val==1)			// if first sign bit
				codec.encode(sgn, CQF_Status::ci_kv_sign_context);
			}
		}

	// KeyValue: encoding keyValue wightout sign
	void encodeKeyValueWithoutSign(CAC_QF& codec, int qstep, int e, int nD)
		{
		int i;
		int x = e;
		for(i=qstep-1;i>=0;i--)
			{
			int val = (x >= 0) ? x : -x;
			val >>= i;
			int bit = val & 0x1;
			
			if(val<=1)
				{
				if(nD == 0)
					codec.encode(bit, CQF_Status::ci_kv_keyValue_x_context[i]);
				else if(nD == 1)
					codec.encode(bit, CQF_Status::ci_kv_keyValue_y_context[i]);
				else if(nD == 2)
					codec.encode(bit, CQF_Status::ci_kv_keyValue_z_context[i]);
				}
			else
				{
				codec.encode(bit);
				}
			}
		}

	// QStep: encoding required QStep
	void encodeQStep(CAC_QF& codec, int qstep, int e, int nD)
		{
		int i;
		int x = e;
		for(i=qstep-1;i>=0;i--)
			{
			int val = (x >= 0) ? x : -x;
			val >>= i;
			int bit = val & 0x1;
			if(val<=1)
				{
				if(nD==0)
					codec.encode(bit, CQF_Status::ci_kv_QStep_x_context[i]);
				else if(nD==1)
					codec.encode(bit, CQF_Status::ci_kv_QStep_y_context[i]);
				else if(nD==2)
					codec.encode(bit, CQF_Status::ci_kv_QStep_z_context[i]);
				}
			else
				{
				if(nD==0)
					codec.encode(bit, CQF_Status::ci_kv_QStep_x_context[i]);
				else if(nD==1)
					codec.encode(bit, CQF_Status::ci_kv_QStep_y_context[i]);
				else if(nD==2)
					codec.encode(bit, CQF_Status::ci_kv_QStep_z_context[i]);
				}
			}
		}

	// QMin: encoding QMin needed in CQ
	void encodeQMin(CAC_QF& codec, int qstep, int dQ)
		{
		int i;
		for(i=qstep-1;i>=0;i--)
			{
			int val = (dQ >= 0) ? dQ : -dQ;
			int sgn = (dQ >= 0) ? 0 : 1;
			val >>= i;
			int bit = val & 0x1;
			
			if(val<=1)
				codec.encode(bit, CQF_Status::ci_kv_QMin_context[i]);
			else
				codec.encode(bit, CQF_Status::ci_kv_QMin_context[i]);
			if(val==1)            //if first sign bit
				codec.encode(sgn, CQF_Status::ci_kv_QMin_sign_context);
			}
		}

	// QMax: encoding QMax needed in CQ
	void encodeQMax(CAC_QF& codec, int qstep, int dQ)
		{
		int i;
		for(i=qstep-1;i>=0;i--)
			{
			int val = (dQ >= 0) ? dQ : -dQ;
			int sgn = (dQ >= 0) ? 0 : 1;
			val >>= i;
			int bit = val & 0x1;
			
			if(val<=1)
				codec.encode(bit, CQF_Status::ci_kv_QMax_context[i]);
			else
				codec.encode(bit, CQF_Status::ci_kv_QMax_context[i]);
			if(val==1)            //if first sign bit
				codec.encode(sgn, CQF_Status::ci_kv_QMax_sign_context);
			}
		}

	// Base vertex index: encoding index for base vertex
	void encodeBaseVertexIndex(CAC_QF& codec, int qstep, int dQ)
		{
		int i;
		for(i=qstep-1;i>=0;i--)
			{
			int val = (dQ >= 0) ? dQ : -dQ;
			val >>= i;
			int bit = val & 0x1;
			
			if(val<=1)
				codec.encode(bit, CQF_Status::ci_kv_Base_vertex_index_context[i]);
			else
				codec.encode(bit, CQF_Status::ci_kv_Base_vertex_index_context[i]);
			}
		}
	};


// AAC decoder
class CAACDecoder
	{
private:
	bool m_bSign;											// Sign in AAC

public:
	// Destructor
	CAACDecoder(void)
		{
		m_bSign = false;
		}

	// Sign: setting sign (default: true)
	void setSign(bool bSign = true)
		{
		m_bSign = bSign;
		}

	// Sign: getting sign
	bool getSign(void)
		{
		return m_bSign;
		}

	// Key: decoding key
	void decodeKey(CAC_QF& codec, int qstep, int& dQ)
		{
		if(m_bSign == false)
			decodeKeyWithoutSign(codec, qstep, dQ);
		else
			decodeKeyWithSign(codec, qstep, dQ);
		}

	// Key: decoding key without sign
	void decodeKeyWithoutSign(CAC_QF& codec, int qstep, int& dQ)
		{
		int b;
		dQ=0;
		b = qstep-1;
		int msb = 0;
		do {
			msb = codec.decode(CQF_Status::ci_k_key_context[b]);
			msb = msb << b;
			b--;
			}
		while (msb==0 && b >=0);
		
		int rest = 0;
		if(msb != 0)
			{
			rest = 0;
			while (b>=0)
				{
				rest |= (codec.decode(CQF_Status::ci_k_key_context[b]) << b);
				b--;
				}
			}
		dQ = (msb+rest);
		}
	
	// Key: decoding key with sign
	void decodeKeyWithSign(CAC_QF& codec, int qstep, int &dQ)
		{
		int b;
		dQ=0;
		b = qstep-1;
		int msb = 0;
		do {
			msb = codec.decode(CQF_Status::ci_k_key_context[b]);
			msb = msb << b;
			b--;
			}
		while (msb==0 && b >=0);
		
		int sgn = 0;
		int rest = 0;
		if(msb != 0)
			{
			sgn = codec.decode(CQF_Status::ci_k_sign_context);
			rest = 0;
			while (b>=0)
				{
				rest |= (codec.decode(CQF_Status::ci_k_key_context[b]) << b);
				b--;
				}
			}
		if(sgn)
			dQ = -(msb+rest);
		else
			dQ = (msb+rest);
		}

	// KeyValue: decoding keyValue
	void decodeKeyValue(CAC_QF& codec, int qstep, int &dQ, int nD)
		{
		int b;
		int *iv = (int*)&dQ;
		*iv = 0;

		b = qstep-1;
		int msb = 0;
		do {
			msb =
				(nD==0)? codec.decode(CQF_Status::ci_kv_keyValue_x_context[b]) :
				(nD==1)? codec.decode(CQF_Status::ci_kv_keyValue_y_context[b]) :
				(nD==2)? codec.decode(CQF_Status::ci_kv_keyValue_z_context[b]) : 0;
			msb = msb << b;
			b--;
			}
		while (msb==0 && b >=0);

		int sgn = 0;
		int rest = 0;
		if(msb != 0) {
			sgn = codec.decode(CQF_Status::ci_kv_sign_context);
			rest = 0;
			while (b>=0) {
				rest |= (codec.decode() << b);
				b--;
				}
			}
		if(sgn)
			*iv = -(msb+rest);
		else
			*iv = (msb+rest);
		}

	// KeyValue: decoding keyValue without sign
	void decodeKeyValueWithoutSign(CAC_QF& codec, int qstep, int &dQ, int nD)
		{
		int b;
		int *iv = (int*)&dQ;
		*iv = 0;

		b = qstep-1;
		int msb = 0;
		do {
			msb =
				(nD==0)? codec.decode(CQF_Status::ci_kv_keyValue_x_context[b]) :
				(nD==1)? codec.decode(CQF_Status::ci_kv_keyValue_y_context[b]) :
				(nD==2)? codec.decode(CQF_Status::ci_kv_keyValue_z_context[b]) : 0;
			msb = msb << b;
			b--;
			}
		while (msb==0 && b >=0);

		int rest = 0;
		if(msb != 0) {
			rest = 0;
			while (b>=0) {
				rest |= (codec.decode() << b);
				b--;
				}
			}
		*iv = (msb+rest);
		}

	// QStep: decoding required QStep
	void decodeQStep(CAC_QF& codec, int qstep, int &dQ, int nD)
		{
		int b;
		int *iv = (int*)&dQ;
		*iv = 0;

		b = qstep-1;
		int msb = 0;
		do {
			msb =
				(nD==0)? codec.decode(CQF_Status::ci_kv_QStep_x_context[b]) :
				(nD==1)? codec.decode(CQF_Status::ci_kv_QStep_y_context[b]) :
				(nD==2)? codec.decode(CQF_Status::ci_kv_QStep_z_context[b]) : 0;
			msb = msb << b;
			b--;
			}
		while (msb==0 && b >=0);

		int rest = 0;
		if(msb != 0) {
			rest = 0;
			while (b>=0) {
				rest |= (nD==0)? (codec.decode(CQF_Status::ci_kv_QStep_x_context[b]) << b) :
						(nD==1)? (codec.decode(CQF_Status::ci_kv_QStep_y_context[b]) << b) :
						(nD==2)? (codec.decode(CQF_Status::ci_kv_QStep_z_context[b]) << b) : 0;
				b--;
				}
			}
		*iv = (msb+rest);
		}

	// QMin: decoding QMin needed in CQ
	void decodeQMin(CAC_QF& codec, int qstep, int& dQ)
		{
		int b;
		dQ=0;
		b = qstep-1;
		int msb = 0;
		do {
			msb = codec.decode(CQF_Status::ci_kv_QMin_context[b]);
			msb = msb << b;
			b--;
			}
		while (msb==0 && b >=0);
		
		int sgn = 0;
		int rest = 0;
		if(msb != 0)
			{
			sgn = codec.decode(CQF_Status::ci_kv_QMin_sign_context);
			rest = 0;
			while (b>=0)
				{
				rest |= (codec.decode(CQF_Status::ci_kv_QMin_context[b]) << b);
				b--;
				}
			}
		if(sgn)
			dQ = -(msb+rest);
		else
			dQ = (msb+rest);
		}

	// QMax: decoding QMax needed in CQ
	void decodeQMax(CAC_QF& codec, int qstep, int& dQ)
		{
		int b;
		dQ=0;
		b = qstep-1;
		int msb = 0;
		do {
			msb = codec.decode(CQF_Status::ci_kv_QMax_context[b]);
			msb = msb << b;
			b--;
			}
		while (msb==0 && b >=0);
		
		int sgn = 0;
		int rest = 0;
		if(msb != 0)
			{
			sgn = codec.decode(CQF_Status::ci_kv_QMax_sign_context);
			rest = 0;
			while (b>=0)
				{
				rest |= (codec.decode(CQF_Status::ci_kv_QMax_context[b]) << b);
				b--;
				}
			}
		if(sgn)
			dQ = -(msb+rest);
		else
			dQ = (msb+rest);
		}

	// Base vertex index: decoding index for base vertex
	void decodeBaseVertexIndex(CAC_QF& codec, int qstep, int& dQ)
		{
		int b;
		dQ=0;
		b = qstep-1;
		int msb = 0;
		do {
			msb = codec.decode(CQF_Status::ci_kv_Base_vertex_index_context[b]);
			msb = msb << b;
			b--;
			}
		while (msb==0 && b >=0);
		
		int rest = 0;
		if(msb != 0)
			{
			rest = 0;
			while (b>=0)
				{
				rest |= (codec.decode(CQF_Status::ci_kv_Base_vertex_index_context[b]) << b);
				b--;
				}
			}
		dQ = (msb+rest);
		}
	};
