/***********************************************************************
This software module was originally developed by James D. K. Kim, 
Sang Oak Woo and Shin Jun Lee(Samsung Advance Institute of Technology)
in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)
standard. This software module is an implementation of a part of one or
more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4
Systems (ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 
Systems (ISO/IEC 14496-1) free license to this software module or 
modifications thereof for use in hardware or software products claiming
conformance to the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to 
use this software module in hardware or software products are advised that
its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Samsung Advance Institute of Technology
retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using
the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
This copyright notice must be included in all copies or derivative works.
Copyright  2001
************************************************************************

For more information or to receive an up to date version of this module,
you may contact James D. K. Kim at jacobkim@sait.samsung.co.kr
----------------------------------------------------------------------*/

// CoordIDPCMMode.h: coordinate key value decoder
// Author: Shinjun Lee, Samsung AIT
// 4th revision: 2002-05-03

#ifndef _COORDIDPCMMODE_H_
#define _COORDIDPCMMODE_H_

#include "Entropy.h"
#include "CoordIKeyValueHeader.h"

class CoordIDPCMMode
	{
  public:
	// variables: bitstream data of key value
	bool		bAddressOfDPCMMode[27];						// Dictionary address table
	bool*		bIndexDPCMMode;								// Dictionary index table

	// variables: nonparsable data
	SAIT_CAC dpcmModeDicAddressContext;			// TS mode: mode for temporal or spatial or temporal-spatial dpcm
	SAIT_CAC dpcmModeDicIndexContext;

  public:
	CoordIDPCMMode(void)
		{
		memset((void*)bAddressOfDPCMMode, 0, sizeof(bool)*27);
		bIndexDPCMMode = NULL;
		}

	~CoordIDPCMMode()
		{
		delete[] bIndexDPCMMode;
		}

	void InitializeModel(void)
		{
		// TS mode: mode for temporal or spatial or temporal-spatial dpcm
		dpcmModeDicAddressContext.reset();
		dpcmModeDicIndexContext.reset();
		}

	void readBitstream(CAC_QF& codec, CoordIKeyValueHeader& coordIKVHeader, int* nDPCMMode)
		{
		int			nNumberOfSymbol = 0;					// the number of symbols
		int			i = 0;									// index
		int			k = 0;									// index
		int			s = 0;									// index

		//ofstream fout_dm;
		//fout_dm.open("DPCMMode_kv_dec.txt");

		// allocate: bIndexDPCMMode
		bIndexDPCMMode = new bool[coordIKVHeader.nNumberOfCoord];
		memset((void*)bIndexDPCMMode, 0, sizeof(bool)*coordIKVHeader.nNumberOfCoord);

		// set: the dpcm mode of the first(0-th) vertex as T. DPCM
		nDPCMMode[0*3+2] = 1;
		nDPCMMode[0*3+1] = 1;
		nDPCMMode[0*3] = 1;

		// receive: bAddressOfDPCMMode
		for(i = 0; i < 27; i++)
			{
			bAddressOfDPCMMode[i] = codec.decode(dpcmModeDicAddressContext) == 1 ? true : false;
			if(bAddressOfDPCMMode[i] == true)
				nNumberOfSymbol++;
			}

		// receive: position index
		int nIndex_address = 0;					// address index
		for(s = 0; s < nNumberOfSymbol; s++)	// repeat as the number of symbols
			{
			int nQKV = 0;						// dictionary symbol
			int nValue = 0;						// comparing value

			while(bAddressOfDPCMMode[nIndex_address] != true)
				nIndex_address++;
			nQKV = nIndex_address;
			//fout_dm << nQKV << ": ";

			if(s < nNumberOfSymbol-1)
				nIndex_address++;
			else
				{
				for(k = 1; k < coordIKVHeader.nNumberOfCoord; k++)
					{
					if(bIndexDPCMMode[k] == 0)
						{
						int nMode = nQKV;
						nValue = nMode / 9;
						nDPCMMode[k*3+2] = nValue + 1;
						nMode -= nValue * 9;

						nValue = nMode / 3;
						nDPCMMode[k*3+1] = nValue + 1;
						nMode -= nValue * 3;

						nValue = nMode;
						nDPCMMode[k*3] = nValue + 1;
						bIndexDPCMMode[k] = 1;
						}
					}
				break;
				}

			// bIndexDPCMMode: position index decoding
			for(k = 1; k < coordIKVHeader.nNumberOfCoord; k++)
				{
				if(bIndexDPCMMode[k] == 0)
					{
					if(codec.decode(dpcmModeDicIndexContext) == 1)
						{
						int nMode = nQKV;
						nValue = nMode / 9;
						nDPCMMode[k*3+2] = nValue + 1;
						nMode -= nValue * 9;

						nValue = nMode / 3;
						nDPCMMode[k*3+1] = nValue + 1;
						nMode -= nValue * 3;

						nValue = nMode;
						nDPCMMode[k*3] = nValue + 1;
						bIndexDPCMMode[k] = 1;
						//fout_dm << 1 << flush;
						}
					//else
						//fout_dm << 0 << flush;
					}
				}
			//fout_dm << endl;
			}
		//fout_dm.close();
		delete[] bIndexDPCMMode;
		bIndexDPCMMode = NULL;
		}

	void decodingProcess(CoordIKeyValueHeader&)
		{
		// No codes.
		}
	};

#endif // _COORDIDPCMMODE_H_
