/***********************************************************************
This software module was originally developed by James D. K. Kim, 
Sang Oak Woo and Shin Jun Lee(Samsung Advance Institute of Technology)
in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)
standard. This software module is an implementation of a part of one or
more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4
Systems (ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 
Systems (ISO/IEC 14496-1) free license to this software module or 
modifications thereof for use in hardware or software products claiming
conformance to the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to 
use this software module in hardware or software products are advised that
its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Samsung Advance Institute of Technology
retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using
the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
This copyright notice must be included in all copies or derivative works.
Copyright  2001
************************************************************************

For more information or to receive an up to date version of this module,
you may contact James D. K. Kim at jacobkim@sait.samsung.co.kr
----------------------------------------------------------------------*/

// CoordIIncrementalMode.h: coordinate key value decoder
// Author: Shinjun Lee, Samsung AIT
// 4th revision: 2002-05-03

#ifndef _COORDIINCREMENTALMODE_H_
#define _COORDIINCREMENTALMODE_H_

#include <limits.h>

#include "Entropy.h"

class CoordIIncrementalMode
	{
  public:
	// variables: bitstream data
	bool*			bAddress;								// dictionary address index
	bool*			bAddrIndex;								// dictionary index flag
	int				nTrueOne;								// the value of one

	// variables: nonparsable data
	SAIT_CAC dicAddressContext;							// Dictionary address

  public:
	CoordIIncrementalMode(void)
		{
		bAddress = NULL;
		bAddrIndex = NULL;
		nTrueOne = 1;
		}

	~CoordIIncrementalMode()
		{
		delete[] bAddress;
		delete[] bAddrIndex;
		}

	void InitializeModel(void)
		{
		// Dictionary address
		dicAddressContext.reset();
		}

	void readBitstream(CAC_QF& codec, int* nKVCodingBit, int nNumberOfKey, int c, int j, CoordIKeyValueHeader& coordIKVHeader, int* nQKeyValue, SAIT_CAC& dicOneContext, SAIT_CAC& dicIndexContext)
		{
		int			nSizeOfAddress = 0;						// the size of address
		int			nNumberOfSymbol = 0;					// the number of symbols
		int			nZero = 0;								// the value of zero
		int			i = 0;									// index
		int			k = 0;									// index
		int			s = 0;									// index

		// calculate: the size of address
		nSizeOfAddress = (int)pow(2, nKVCodingBit[j*3+c]+1) - 1;

		// allocate: dictionary address index
		bAddress = new bool[nSizeOfAddress];
		memset((void*)bAddress, 0, sizeof(bool)*nSizeOfAddress);

		// allocate: bAddrIndex
		bAddrIndex = new bool[nNumberOfKey];
		memset((void*)bAddrIndex, 0, sizeof(bool)*nNumberOfKey);

		// receive: the values of address
		nNumberOfSymbol = 0;
		for(i = 0; i < nSizeOfAddress; i++)
			{
			bAddress[i] = codec.decode(dicAddressContext) == 1 ? true : false;
			if(bAddress[i] == true)
				nNumberOfSymbol++;
			}

		// decode: incremental mode
		int nIndex_address = 0;					// address index
		for(s = 0; s < nNumberOfSymbol; s++)	// repeat as the number of symbols
			{
			int nQKV = 0;						// quantized value
			while(bAddress[nIndex_address] != true)
				nIndex_address++;

			if(nIndex_address == 0)
				nQKV = 0;
			else if(nIndex_address % 2 != 0)
				nQKV = (nIndex_address+1)/2;
			else
				nQKV = -nIndex_address/2;

			if(s < nNumberOfSymbol-1)
				nIndex_address++;
			else // if last symbol, break out
				{
				for(k = 0; k < nNumberOfKey; k++)
					if(bAddrIndex[k] == 0)
						nQKeyValue[k*coordIKVHeader.nNumberOfCoord*3+j*3+c] = nQKV;
				break;
				}

			// nTrueOne: the real value of one
			nTrueOne = codec.decode(dicOneContext) == 1 ? 1 : 0;
			nZero = nTrueOne == 0 ? 1 : 0;

			// decode: position index
			for(k = 0; k < nNumberOfKey; k++)
				{
				if(bAddrIndex[k] == 0)
					{
					if(codec.decode(dicIndexContext) == nTrueOne)
						{
						bAddrIndex[k] = 1;
						nQKeyValue[k*coordIKVHeader.nNumberOfCoord*3+j*3+c] = nQKV;
						}
					}
				}
			}
		// delete: dictionary address index
		delete[] bAddress;
		bAddress = NULL;
		// delete: position index
		delete[] bAddrIndex;
		bAddrIndex = NULL;
		}

	void decodingProcess(CoordIKeyValueHeader&)
		{

		}
	};

#endif // _COORDIINCREMENTALMODE_H_
