/***********************************************************************
This software module was originally developed by James D. K. Kim, 
Sang Oak Woo and Shin Jun Lee(Samsung Advance Institute of Technology)
in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)
standard. This software module is an implementation of a part of one or
more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4
Systems (ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 
Systems (ISO/IEC 14496-1) free license to this software module or 
modifications thereof for use in hardware or software products claiming
conformance to the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to 
use this software module in hardware or software products are advised that
its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Samsung Advance Institute of Technology
retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using
the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
This copyright notice must be included in all copies or derivative works.
Copyright  2001
************************************************************************

For more information or to receive an up to date version of this module,
you may contact James D. K. Kim at jacobkim@sait.samsung.co.kr
----------------------------------------------------------------------*/

// CoordIKeyValue.h: coordinate key value decoder
// Author: Shinjun Lee, Samsung AIT
// 4th revision: 2002-05-03

#ifndef _COORDIKEYVALUE_H_
#define _COORDIKEYVALUE_H_

#include "ArithmeticDecoder.h"
#include "CDPCM.h"
#include "CoordIDPCMMode.h"
#include "CoordIKeyValueDic.h"
#include "CKeyValueDecoder.h"

class CoordIKeyValue
	{
  public:
	// variables: bitstream data of key value
	int*			nDPCMMode;
	bool*			bSelFlag;
	int*			nKVACodingBit;
	int*			nRefVertex;
	int*			nQMin;
	int*			nQMax;

	CoordIDPCMMode		coordIDPCMMode;
	CoordIKeyValueDic	coordIKeyValueDic;

	// variables: nonparsable data
	int				nNumberOfKey_org;
	int				nNumberOfCoord_org;

	int				nNumberOfKey_tr;
	int				nNumberOfCoord_tr;

	int*			nQKeyValue;								// quantized key value

	CDPCM			oDPCM_X_Min;							// DPCM processor
	CDPCM			oDPCM_Y_Min;							// DPCM processor
	CDPCM			oDPCM_Z_Min;							// DPCM processor

	CDPCM			oDPCM_X_Max;							// DPCM processor
	CDPCM			oDPCM_Y_Max;							// DPCM processor
	CDPCM			oDPCM_Z_Max;							// DPCM processor

	double*			dKeyValue;								// inversed quantized key value

	// contexts
	SAIT_CAC selectionFlagContext;						// Selection flag: selection flag context
	SAIT_CAC aqpXContext[32];							// Required QStep: RQP context
	SAIT_CAC aqpYContext[32];
	SAIT_CAC aqpZContext[32];
	SAIT_CAC qMinContext[32];								// QMin: qmin for CQ
	SAIT_CAC qMinSignContext;
	SAIT_CAC qMaxContext[32];								// QMax: qmax for CQ
	SAIT_CAC qMaxSignContext;
	SAIT_CAC refContext[32];				// Base vertex index: Base vertex index

  public:
	CoordIKeyValue(void)
		{
		nDPCMMode = NULL;
		bSelFlag = NULL;
		nKVACodingBit = NULL;
		nRefVertex = NULL;
		nQMin = NULL;
		nQMax = NULL;

		nNumberOfKey_org = 0;
		nNumberOfCoord_org = 0;

		nNumberOfKey_tr = 0;
		nNumberOfCoord_tr = 0;

		nQKeyValue = NULL;
		dKeyValue = NULL;
		}

	~CoordIKeyValue()
		{
		delete[] nDPCMMode;
		delete[] bSelFlag;
		delete[] nKVACodingBit;
		delete[] nRefVertex;
		delete[] nQMin;
		delete[] nQMax;

		delete[] nQKeyValue;

		delete[] dKeyValue;
		}

	void InitializeModel(void)
		{
		int i=0;

		// Selection flag: selection flag context
		selectionFlagContext.reset();
		
		// Required QStep: RQP context
		for(i = 0; i < 32; i++)
			{
			aqpXContext[i].reset();
			aqpYContext[i].reset();
			aqpZContext[i].reset();
			}

		// QMin: qmin for CQ
		qMinSignContext.reset();
		for(i = 0; i < 32; i++)
			qMinContext[i].reset();

		// QMax: qmax for CQ
		qMaxSignContext.reset();
		for(i = 0; i < 32; i++)
			qMaxContext[i].reset();
		
		// Base vertex index
		for(i = 0; i < 32; i++)
			refContext[i].reset();

		coordIDPCMMode.InitializeModel();
		coordIKeyValueDic.InitializeModel();
		}

	void readBitstream(CAC_QF& codec, CoordIKeyValueHeader& coordIKVHeader, int& nNumberOfKey)
		{
		int			j = 0;									// index
		int			c = 0;									// index
		int			nKVACodingBitQBit = 0;					// QStep for QStep
		
		// transpose: information of keyValue whether the data is transposed
		nNumberOfKey_tr = nNumberOfKey_org = nNumberOfKey;
		nNumberOfCoord_tr = nNumberOfCoord_org = coordIKVHeader.nNumberOfCoord;
		if(coordIKVHeader.bTranspose == true)
			{
			nNumberOfKey_tr = nNumberOfKey = nNumberOfCoord_org;
			nNumberOfCoord_tr = coordIKVHeader.nNumberOfCoord = nNumberOfKey_org;
			}
		
		// get: nKVACodingBitQBit
		nKVACodingBitQBit = (int)(log10(abs(coordIKVHeader.nKVQBit))/log10(2))+1;
		
		// allocate: nDPCMMode
		nDPCMMode = new int[coordIKVHeader.nNumberOfCoord*3];
		memset((void*)nDPCMMode, 0, sizeof(int)*(coordIKVHeader.nNumberOfCoord*3));
		
		// allocate: bSelFlag
		bSelFlag = new bool[coordIKVHeader.nNumberOfCoord*3];
		memset((void*)bSelFlag, 1, sizeof(bool)*(coordIKVHeader.nNumberOfCoord*3));
		
		// allocate: nKVACodingBit
		nKVACodingBit = new int[coordIKVHeader.nNumberOfCoord*3];
		memset((void*)nKVACodingBit, 0, sizeof(int)*(coordIKVHeader.nNumberOfCoord*3));
		
		// allocate: nRefVertex
		nRefVertex = new int[coordIKVHeader.nNumberOfCoord*3];
		memset((void*)nRefVertex, 0, sizeof(int)*(coordIKVHeader.nNumberOfCoord*3));
		
		// allocate: nQMin
		nQMin = new int[coordIKVHeader.nNumberOfCoord*3];
		memset((void*)nQMin, 0, sizeof(int)*(coordIKVHeader.nNumberOfCoord*3));
		
		// allocate: nQMax
		nQMax = new int[coordIKVHeader.nNumberOfCoord*3];
		memset((void*)nQMax, 0, sizeof(int)*(coordIKVHeader.nNumberOfCoord*3));
		
		// allocate: m_a_nQKeyValue
		nQKeyValue = new int[nNumberOfKey*coordIKVHeader.nNumberOfCoord*3];
		memset((void*)nQKeyValue, 0, sizeof(int)*(nNumberOfKey*coordIKVHeader.nNumberOfCoord*3));
		
		// initialize: oDPCM_X_Min, oDPCM_Y_Min, oDPCM_Z_Min
		oDPCM_X_Min.setNumber(coordIKVHeader.nNumberOfCoord);
		oDPCM_Y_Min.setNumber(coordIKVHeader.nNumberOfCoord);
		oDPCM_Z_Min.setNumber(coordIKVHeader.nNumberOfCoord);
		
		oDPCM_X_Min.setData(NULL);
		oDPCM_Y_Min.setData(NULL);
		oDPCM_Z_Min.setData(NULL);
		
		oDPCM_X_Min.setMin(coordIKVHeader.nXQMin_Min);
		oDPCM_X_Min.setMax(coordIKVHeader.nXQMin_Max);
		oDPCM_Y_Min.setMin(coordIKVHeader.nYQMin_Min);
		oDPCM_Y_Min.setMax(coordIKVHeader.nYQMin_Max);
		oDPCM_Z_Min.setMin(coordIKVHeader.nZQMin_Min);
		oDPCM_Z_Min.setMax(coordIKVHeader.nZQMin_Max);
		
		// initialize: oDPCM_X_Max, oDPCM_Y_Max, oDPCM_Y_Max
		oDPCM_X_Max.setNumber(coordIKVHeader.nNumberOfCoord);
		oDPCM_Y_Max.setNumber(coordIKVHeader.nNumberOfCoord);
		oDPCM_Z_Max.setNumber(coordIKVHeader.nNumberOfCoord);
		
		oDPCM_X_Max.setData(NULL);
		oDPCM_Y_Max.setData(NULL);
		oDPCM_Z_Max.setData(NULL);
		
		oDPCM_X_Max.setBase(NULL);
		oDPCM_Y_Max.setBase(NULL);
		oDPCM_Z_Max.setBase(NULL);
		
		oDPCM_X_Max.setMin(coordIKVHeader.nXQMax_Min);
		oDPCM_X_Max.setMax(coordIKVHeader.nXQMax_Max);
		oDPCM_Y_Max.setMin(coordIKVHeader.nYQMax_Min);
		oDPCM_Y_Max.setMax(coordIKVHeader.nYQMax_Max);
		oDPCM_Z_Max.setMin(coordIKVHeader.nZQMax_Min);
		oDPCM_Z_Max.setMax(coordIKVHeader.nZQMax_Max);
		
		// read: bitstream of keyValue
		coordIDPCMMode.readBitstream(codec, coordIKVHeader, nDPCMMode);
		for(j = 0; j < coordIKVHeader.nNumberOfCoord; j++)
			{
			for(c = 0; c < 3; c++)
				{
				if(c == 0)
					{
					if(coordIKVHeader.nXQMax_Min <= coordIKVHeader.nXQMin_Max)
						{
						bSelFlag[j*3+c] = codec.decode(selectionFlagContext) == 0 ? false : true;
						}
					}
				else if(c == 1)
					{
					if(coordIKVHeader.nYQMax_Min <= coordIKVHeader.nYQMin_Max)
						{
						bSelFlag[j*3+c] = codec.decode(selectionFlagContext) == 0 ? false : true;
						}
					}
				else if(c == 2)
					{
					if(coordIKVHeader.nZQMax_Min <= coordIKVHeader.nZQMin_Max)
						{
						bSelFlag[j*3+c] = codec.decode(selectionFlagContext) == 0 ? false : true;
						}
					}
				
				if(bSelFlag[j*3+c] == true)
					{
					if(c == 0)
						decodeUnsignedAAC(codec, &nKVACodingBit[j*3+c], nKVACodingBitQBit, aqpXContext);
					else if(c == 1)
						decodeUnsignedAAC(codec, &nKVACodingBit[j*3+c], nKVACodingBitQBit, aqpYContext);
					else if(c == 2)
						decodeUnsignedAAC(codec, &nKVACodingBit[j*3+c], nKVACodingBitQBit, aqpZContext);
					
					if(j > 0)
						{
						if(nDPCMMode[j*3+c] == DPCM_S || nDPCMMode[j*3+c] == DPCM_TS)
							{
							int nQBitOfRef = (int)(log10(abs(j-1))/log10(2))+1;
							decodeUnsignedAAC(codec, &nRefVertex[j*3+c], nQBitOfRef, refContext);
							}
						}
					
					if(nKVACodingBit[j*3+c] != 0)
						{
						int nQMin_dpcm = 0;
						decodeSignedAAC(codec, &nQMin_dpcm, coordIKVHeader.nKVQBit+1,
							qMinSignContext, qMinContext);
						if(c == 0)
							oDPCM_X_Min.setData_dpcm(j, nQMin_dpcm);
						else if(c == 1)
							oDPCM_Y_Min.setData_dpcm(j, nQMin_dpcm);
						else if(c == 2)
							oDPCM_Z_Min.setData_dpcm(j, nQMin_dpcm);
						}
					if(c == 0)
						{
						oDPCM_X_Min.inverseDPCM(bSelFlag, nKVACodingBit, nRefVertex, j, c);
						int nQMax_base = nQMin[j*3+c] = oDPCM_X_Min.getData(j);
						oDPCM_X_Max.setData_base(j, nQMax_base);
						}
					else if(c == 1)
						{
						oDPCM_Y_Min.inverseDPCM(bSelFlag, nKVACodingBit, nRefVertex, j, c);
						int nQMax_base = nQMin[j*3+c] = oDPCM_Y_Min.getData(j);
						oDPCM_Y_Max.setData_base(j, nQMax_base);
						}
					else if(c == 2)
						{
						oDPCM_Z_Min.inverseDPCM(bSelFlag, nKVACodingBit, nRefVertex, j, c);
						int nQMax_base = nQMin[j*3+c] = oDPCM_Z_Min.getData(j);
						oDPCM_Z_Max.setData_base(j, nQMax_base);
						}
					
					if(nKVACodingBit[j*3+c] != 0)
						{
						int nQMax_dpcm = 0;
						decodeSignedAAC(codec, &nQMax_dpcm, coordIKVHeader.nKVQBit+1,
							qMaxSignContext, qMaxContext);
						if(c == 0)
							oDPCM_X_Max.setData_dpcm(j, nQMax_dpcm);
						else if(c == 1)
							oDPCM_Y_Max.setData_dpcm(j, nQMax_dpcm);
						else if(c == 2)
							oDPCM_Z_Max.setData_dpcm(j, nQMax_dpcm);
						}
					if(c == 0)
						{
						oDPCM_X_Max.inverseDPCM(bSelFlag, nKVACodingBit, nRefVertex, j, c);
						nQMax[j*3+c] = oDPCM_X_Max.getData(j);
						}
					else if(c == 1)
						{
						oDPCM_Y_Max.inverseDPCM(bSelFlag, nKVACodingBit, nRefVertex, j, c);
						nQMax[j*3+c] = oDPCM_Y_Max.getData(j);
						}
					else if(c == 2)
						{
						oDPCM_Z_Max.inverseDPCM(bSelFlag, nKVACodingBit, nRefVertex, j, c);
						nQMax[j*3+c] = oDPCM_Z_Max.getData(j);
						}
					}
				else
					{
					int nQMin_dpcm = 0;
					decodeSignedAAC(codec, &nQMin_dpcm, coordIKVHeader.nKVQBit+1,
						qMinSignContext, qMinContext);
					if(c == 0)
						{
						oDPCM_X_Min.setData_dpcm(j, nQMin_dpcm);
						oDPCM_X_Min.inverseDPCM(bSelFlag, nKVACodingBit, nRefVertex, j, c);
						int nQMax_base = nQMin[j*3+c] = oDPCM_X_Min.getData(j);
						oDPCM_X_Max.setData_base(j, nQMax_base);
						oDPCM_X_Max.inverseDPCM(bSelFlag, nKVACodingBit, nRefVertex, j, c);
						nQMax[j*3+c] = oDPCM_X_Max.getData(j);
						}
					else if(c == 1)
						{
						oDPCM_Y_Min.setData_dpcm(j, nQMin_dpcm);
						oDPCM_Y_Min.inverseDPCM(bSelFlag, nKVACodingBit, nRefVertex, j, c);
						int nQMax_base = nQMin[j*3+c] = oDPCM_Y_Min.getData(j);
						oDPCM_Y_Max.setData_base(j, nQMax_base);
						oDPCM_Y_Max.inverseDPCM(bSelFlag, nKVACodingBit, nRefVertex, j, c);
						nQMax[j*3+c] = oDPCM_Y_Max.getData(j);
						}
					else if(c == 2)
						{
						oDPCM_Z_Min.setData_dpcm(j, nQMin_dpcm);
						oDPCM_Z_Min.inverseDPCM(bSelFlag, nKVACodingBit, nRefVertex, j, c);
						int nQMax_base = nQMin[j*3+c] = oDPCM_Z_Min.getData(j);
						oDPCM_Z_Max.setData_base(j, nQMax_base);
						oDPCM_Z_Max.inverseDPCM(bSelFlag, nKVACodingBit, nRefVertex, j, c);
						nQMax[j*3+c] = oDPCM_Z_Max.getData(j);
						}
					}
				
				coordIKeyValueDic.readBitstream(codec, bSelFlag, nKVACodingBit, nNumberOfKey, c, j, coordIKVHeader, nQKeyValue, nQMin);
			}
		}
	}

	void decodingProcess(CoordIKeyValueHeader& coordIKVHeader, int& nNumberOfKey)
		{
		CKeyValueDecoder oKeyValueDecoder;
		oKeyValueDecoder.setKeyValueDigit(coordIKVHeader.nKVDigit);
		oKeyValueDecoder.setMaxMin(coordIKVHeader.kVMinMax.fMin_X, coordIKVHeader.kVMinMax.fMin_Y, coordIKVHeader.kVMinMax.fMin_Z,
			coordIKVHeader.kVMinMax.fMax_X, coordIKVHeader.kVMinMax.fMax_Y, coordIKVHeader.kVMinMax.fMax_Z, (int)coordIKVHeader.kVMinMax.nWhichAxis);
		oKeyValueDecoder.setBase(nRefVertex);
		oKeyValueDecoder.setTSMode(nDPCMMode);
		oKeyValueDecoder.setQMinQMax(nQMin, nQMax);
		oKeyValueDecoder.setQStep_kv(nKVACodingBit);
		oKeyValueDecoder.setQFlag_kv(bSelFlag);
		oKeyValueDecoder.decode(nQKeyValue, nNumberOfKey, coordIKVHeader.nNumberOfCoord, coordIKVHeader.nKVQBit);
		oKeyValueDecoder.getIQKeyValue(dKeyValue);

		if(coordIKVHeader.bTranspose == true)
			{
			transposeKeyValue(dKeyValue, dKeyValue, nNumberOfKey, coordIKVHeader.nNumberOfCoord);
			coordIKVHeader.nNumberOfCoord = nNumberOfCoord_org;
			nNumberOfKey = nNumberOfKey_org;
			}

		// print: result data from key decoding
		/*
		cout << "Number of coords = " << coordIKVHeader.nNumberOfCoord << endl;
		cout << "QStep_keyValue = " << coordIKVHeader.nKVQBit << endl;
		cout << "KeyValue digit = " << coordIKVHeader.nKVDigit << endl;
		cout << "X_min ~ X_max = " << coordIKVHeader.kVMinMax.fMin_X
			 << " ~ " << coordIKVHeader.kVMinMax.fMax_X << endl;
		cout << "Y_min ~ Y_max = " << coordIKVHeader.kVMinMax.fMin_Y 
			 << " ~ " << coordIKVHeader.kVMinMax.fMax_Y << endl;
		cout << "Z_min ~ Z_max = " << coordIKVHeader.kVMinMax.fMin_Z
			 << " ~ " << coordIKVHeader.kVMinMax.fMax_Z << endl;
		cout << endl;
		*/
		}

  	// transpose: source data -> target data which is transposed
	void transposeKeyValue(double* a_dKeyValue_target, double* a_dKeyValue_source, int nNumberOfKey, int nNumberOfCoord)
		{
		double* a_dKeyValue = new double[nNumberOfKey*nNumberOfCoord*3];
		memcpy((void*)a_dKeyValue, (void*)a_dKeyValue_source, sizeof(double)*(nNumberOfKey*nNumberOfCoord*3));

		for(int j = 0; j < nNumberOfCoord; j++)
			{
			for(int i = 0; i < nNumberOfKey; i++)
				{
				for(int c = 0; c < 3; c++)
					{
					//double dValue = a_dKeyValue[i*nNumberOfCoord*3+j*3+c];
					double dValue = getValue(a_dKeyValue, i, j, c);

					//a_dKeyValue_target[j*nNumberOfKey*3+i*3+c] = dValue;
					getTransposedValue(a_dKeyValue_target, j, i, c) = dValue;
					//cout << dValue;
					//cout << ", ";
					}
				//cout << endl;
				}
			//cout << endl;
			}

		delete[] a_dKeyValue;
		}

	// get: the value with the indexes of nKey, nCoord, nComponent in keyValue
	double& getValue(double* a_dValue, int nKey, int nCoord, int nComponent)
		{
		double& dValue = a_dValue[nKey*nNumberOfCoord_tr*3 + nCoord*3 + nComponent];
		return dValue;
		}

	// get: the transposed value with the indexes of nKey, nCoord, nComponent in keyValue
	double& getTransposedValue(double* a_dValue, int nCoord, int nKey, int nComponent)
		{
		double& dValue = a_dValue[nCoord*nNumberOfKey_tr*3 + nKey*3 + nComponent];
		return dValue;
		}
	};

#endif // _COORDIKEYVALUE_H_
