/***********************************************************************
This software module was originally developed by James D. K. Kim, 
Sang Oak Woo and Shin Jun Lee(Samsung Advance Institute of Technology)
in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)
standard. This software module is an implementation of a part of one or
more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4
Systems (ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 
Systems (ISO/IEC 14496-1) free license to this software module or 
modifications thereof for use in hardware or software products claiming
conformance to the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to 
use this software module in hardware or software products are advised that
its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Samsung Advance Institute of Technology
retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using
the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
This copyright notice must be included in all copies or derivative works.
Copyright  2001
************************************************************************

For more information or to receive an up to date version of this module,
you may contact James D. K. Kim at jacobkim@sait.samsung.co.kr
----------------------------------------------------------------------*/

// CoordIKeyValueDic.h: coordinate key value decoder
// Author: Shinjun Lee, Samsung AIT
// 4th revision: 2002-05-03

#ifndef _COORDIKEYVALUEDIC_H_
#define _COORDIKEYVALUEDIC_H_

#include "CoordIIncrementalMode.h"
#include "CoordIOccurrenceMode.h"
#include "Entropy.h"

class CoordIKeyValueDic
	{
  public:
	// variables: bitstream data of key value
	int nDicModeSelect;

	CoordIIncrementalMode coordIIncrementalMode;
	CoordIOccurrenceMode coordIOccurrenceMode;

	// variables: nonparsable data
	SAIT_CAC dicModeSelectionContext;				// Dictionary selection flag: dictionary value or dictionary address
	SAIT_CAC dicOneContext;							// Dictionary one
	SAIT_CAC dicIndexContext;						// Dictionary index

  public:
	CoordIKeyValueDic(void)
		{
		nDicModeSelect = 0;
		}

	~CoordIKeyValueDic()
		{
		}

	void InitializeModel(void)
		{
		// Dictionary selection flag: dictionary value or dictionary address
		dicModeSelectionContext.reset();

		coordIIncrementalMode.InitializeModel();
		coordIOccurrenceMode.InitializeModel();

		// Dictionary one
		dicOneContext.reset();
		
		// Dictionary index
		dicIndexContext.reset();
		}

	void readBitstream(CAC_QF& codec, bool* bSelFlag, int* nKVCodingBit, int nNumberOfKey, int c, int j, CoordIKeyValueHeader& coordIKVHeader, int* nQKeyValue, int* nQMin)
		{
		int i = 0;
		// before dpcm, if min and max are equal (all values of the vertex are equal)
		if(bSelFlag[j*3+c] == false)
			{
			i = 0;
			nQKeyValue[i*coordIKVHeader.nNumberOfCoord*3+j*3+c] = nQMin[j*3+c];
			}
		// after dpcm, if min and max are 0
		else if(bSelFlag[j*3+c] != false && nKVCodingBit[j*3+c] == 0)
			{
			for(i = 0; i < nNumberOfKey; i++)
				nQKeyValue[i*coordIKVHeader.nNumberOfCoord*3+j*3+c] = 0;
			}
		// after dpcm, if min and Max are not 0
		else //	if(bSelFlag[j*3+c] == true && nKVCodingBit[j*3+c] != 0)
			{
			// select: either incremental(address) mode or occurence(dictionary) mode
			nDicModeSelect = codec.decode(dicModeSelectionContext);
			if(nDicModeSelect == 1)
				coordIIncrementalMode.readBitstream(codec, nKVCodingBit, nNumberOfKey,
													c, j, coordIKVHeader, nQKeyValue, dicOneContext, dicIndexContext);
			else
				coordIOccurrenceMode.readBitstream(codec, nKVCodingBit, nNumberOfKey, c,
													j, coordIKVHeader, nQKeyValue, dicOneContext, dicIndexContext);
			}
		}

	void decodingProcess(CoordIKeyValueHeader&)
		{
		// No codes.
		}
	};

#endif // _COORDIKEYVALUEDIC_H_
