/***********************************************************************
This software module was originally developed by James D. K. Kim, 
Sang Oak Woo and Shin Jun Lee(Samsung Advance Institute of Technology)
in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)
standard. This software module is an implementation of a part of one or
more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4
Systems (ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 
Systems (ISO/IEC 14496-1) free license to this software module or 
modifications thereof for use in hardware or software products claiming
conformance to the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to 
use this software module in hardware or software products are advised that
its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Samsung Advance Institute of Technology
retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using
the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
This copyright notice must be included in all copies or derivative works.
Copyright  2001
************************************************************************

For more information or to receive an up to date version of this module,
you may contact James D. K. Kim at jacobkim@sait.samsung.co.kr
----------------------------------------------------------------------*/

// CoordIKeyValueHeader.h: key value decoder
// Author: Shinjun Lee, Samsung AIT
// 4th revision: 2002-05-03

#ifndef _COORDIKEYVALUEHEADER_H_
#define _COORDIKEYVALUEHEADER_H_

#include "KeyValueMinMax.h"
#include "Entropy.h"

class CoordIKeyValueHeader
	{
  public:
	// variables: bitstream data of key value
	bool			bTranspose;								// flag of transposition
	int				nKVQBit;								// quantiztion bit size for key value decoding
	int				nCoordQBit;								// coding bit size
	int				nNumberOfCoord;							// the number of vertices
	int				nKVDigit;								// key value digit
	KeyValueMinMax	kVMinMax;								// class of reading minimum and maximum for key value
	int				nXQMin_Min;								// minimum value among the minimum values to the quantized x values
	int				nXQMin_Max;								// minimum value among the maximum values to the quantized x values
	int				nYQMin_Min;								// minimum value among the minimum values to the quantized y values
	int				nYQMin_Max;								// minimum value among the maximum values to the quantized y values
	int				nZQMin_Min;								// minimum value among the minimum values to the quantized z values
	int				nZQMin_Max;								// minimum value among the maximum values to the quantized z values
	int				nXQMax_Min;								// maximum value among the minimum values to the quantized x values
	int				nXQMax_Max;								// maximum value among the maximum values to the quantized x values
	int				nYQMax_Min;								// maximum value among the minimum values to the quantized y values
	int				nYQMax_Max;								// maximum value among the maximum values to the quantized y values
	int				nZQMax_Min;								// maximum value among the minimum values to the quantized z values
	int				nZQMax_Max;								// maximum value among the maximum values to the quantized z values

	// variables: nonparsable data
	int				nNumberOfCoord_org;

  public:
	CoordIKeyValueHeader(void)
		{
		bTranspose = false;
		nKVQBit = 0;
		nCoordQBit = 0;
		nNumberOfCoord = 0;
		nKVDigit = 0;
		nXQMin_Min = 0;
		nXQMin_Max = 0;
		nYQMin_Min = 0;
		nYQMin_Max = 0;
		nZQMin_Min = 0;
		nZQMin_Max = 0;
		nXQMax_Min = 0;
		nXQMax_Max = 0;
		nYQMax_Min = 0;
		nYQMax_Max = 0;
		nZQMax_Min = 0;
		nZQMax_Max = 0;

		nNumberOfCoord_org = 0;
		}

	void readBitstream(CAC_QF& codec)
		{
		// get: flag of transposition
		if(codec._istrm->getbits(1) == 0)
			bTranspose = false;
		else
			bTranspose = true;

		// get: nKVQBit
		nKVQBit = codec._istrm->getbits(5);

		// get: the number of vertices
		nCoordQBit = codec._istrm->getbits(5);
		nNumberOfCoord_org = nNumberOfCoord = codec._istrm->getbits(nCoordQBit);

		// get: keyValue digit
		nKVDigit = codec._istrm->getbits(4);

		// get: keyValue max, min
		kVMinMax.readBitstream(codec, nKVDigit);
		kVMinMax.decodingProcess();

		// get: min, max of quantized min values for each component
		nXQMin_Min = codec._istrm->getbits(nKVQBit);
		nXQMin_Max = codec._istrm->getbits(nKVQBit);

		nYQMin_Min = codec._istrm->getbits(nKVQBit);
		nYQMin_Max = codec._istrm->getbits(nKVQBit);

		nZQMin_Min = codec._istrm->getbits(nKVQBit);
		nZQMin_Max = codec._istrm->getbits(nKVQBit);

		// get: min, max of quantized max values for each component
		nXQMax_Min = codec._istrm->getbits(nKVQBit);
		nXQMax_Max = codec._istrm->getbits(nKVQBit);

		nYQMax_Min = codec._istrm->getbits(nKVQBit);
		nYQMax_Max = codec._istrm->getbits(nKVQBit);

		nZQMax_Min = codec._istrm->getbits(nKVQBit);
		nZQMax_Max = codec._istrm->getbits(nKVQBit);
		}

	void decodingProcess(void)
		{
		}
	};

#endif // _COORDIKEYVALUEHEADER_H_