/***********************************************************************
This software module was originally developed by James D. K. Kim, 
Sang Oak Woo and Shin Jun Lee(Samsung Advance Institute of Technology)
in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)
standard. This software module is an implementation of a part of one or
more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4
Systems (ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 
Systems (ISO/IEC 14496-1) free license to this software module or 
modifications thereof for use in hardware or software products claiming
conformance to the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to 
use this software module in hardware or software products are advised that
its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Samsung Advance Institute of Technology
retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using
the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
This copyright notice must be included in all copies or derivative works.
Copyright  2001
************************************************************************

For more information or to receive an up to date version of this module,
you may contact James D. K. Kim at jacobkim@sait.samsung.co.kr
----------------------------------------------------------------------*/

// CoordIOccurrenceMode.h: coordinate key value decoder
// Author: Shinjun Lee, Samsung AIT
// 4th revision: 2002-05-03

#ifndef _COORDIOCCURRENCEMODE_H_
#define _COORDIOCCURRENCEMODE_H_

#include "Entropy.h"

class CoordIOccurrenceMode
	{
  public:
	// variables: bitstream data of key value
	int				nQKV;									// the dictionary symbol
	bool			bSoleKV;								// the flag of whether a symbol is only one
	int				nTrueOne;								// the value of one

	// variables: nonparsable data
	SAIT_CAC kvSignContext;									// Key value: keyValue context (dictionary value)
	SAIT_CAC kvXContext[32];
	SAIT_CAC kvYContext[32];
	SAIT_CAC kvZContext[32];
	SAIT_CAC dicSoleKVContext;						// Dictionary last

  public:
	CoordIOccurrenceMode(void)
		{
		nQKV = 0;
		bSoleKV = false;
		nTrueOne = 1;
		}

	~CoordIOccurrenceMode()
		{
		}

	void InitializeModel(void)
		{
		int i=0;

		// Key value: keyValue context (dictionary value)
		kvSignContext.reset();
		for(i = 0; i < 32; i++)
			{
			kvXContext[i].reset();
			kvYContext[i].reset();
			kvZContext[i].reset();
			}

		// Dictionary last
		dicSoleKVContext.reset();
		}

	void readBitstream(CAC_QF& codec, int* nKVCodingBit, int nNumberOfKey, int c, int j, CoordIKeyValueHeader& coordIKVHeader, int* nQKeyValue, SAIT_CAC& dicOneContext, SAIT_CAC& dicIndexContext)
		{
		bool*		bIndexOfDic = NULL;						// dictionary index flag
		int			nZero = 0;								// the value of zero
		int			i = 0;									// index
		int			k = 0;									// index

		// allocate: bIndexOfDic
		bIndexOfDic = new bool[nNumberOfKey];
		memset((void*)bIndexOfDic, 0, sizeof(bool)*nNumberOfKey);

		// decode: occurence mode
		for(i = 0; i < nNumberOfKey; i++)
			{
			// if not scanned yet
			if(bIndexOfDic[i] == 0)
				{
				bIndexOfDic[i] = 1;	// scanned index

				// receive: dictionary symbol
				if(c == 0)
					decodeSignedQuasiAAC(codec, &nQKV, nKVCodingBit[j*3+c]+1, kvSignContext, kvXContext);
				else if(c == 1)
					decodeSignedQuasiAAC(codec, &nQKV, nKVCodingBit[j*3+c]+1, kvSignContext, kvYContext);
				else if(c == 2)
					decodeSignedQuasiAAC(codec, &nQKV, nKVCodingBit[j*3+c]+1, kvSignContext, kvZContext);

				// nQKV: dictionary symbol
				nQKeyValue[i*coordIKVHeader.nNumberOfCoord*3+j*3+c] = nQKV;

				// decode: position index
				bSoleKV = codec.decode(dicSoleKVContext) == 0 ? false : true;
				// bSoleKV: if the number of symbols is more than one
				if(bSoleKV == false)
					{
					// nTrueOne: the real value of one
					nTrueOne = codec.decode(dicOneContext) == 1 ? 1 : 0;
					nZero = nTrueOne == 0 ? 1 : 0;

					// decode: position index
					for(k = i+1; k < nNumberOfKey; k++)
						{
						if(bIndexOfDic[k] == 0)
							{
							if(codec.decode(dicIndexContext) == nTrueOne)
								{
								bIndexOfDic[k] = 1;
								nQKeyValue[k*coordIKVHeader.nNumberOfCoord*3+j*3+c] = nQKV;
								}
							}
						}
					}
				}
			}
		// delete: dictionary index
		delete[] bIndexOfDic;
		}

	void decodingProcess(CoordIKeyValueHeader&)
		{
		// No codes.
		}
	};

#endif // _COORDIOCCURRENCEMODE_H_
