/***********************************************************************
This software module was originally developed by James D. K. Kim, 
Sang Oak Woo and Shin Jun Lee(Samsung Advance Institute of Technology)
in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)
standard. This software module is an implementation of a part of one or
more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4
Systems (ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 
Systems (ISO/IEC 14496-1) free license to this software module or 
modifications thereof for use in hardware or software products claiming
conformance to the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to 
use this software module in hardware or software products are advised that
its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Samsung Advance Institute of Technology
retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using
the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
This copyright notice must be included in all copies or derivative works.
Copyright  2001
************************************************************************

For more information or to receive an up to date version of this module,
you may contact James D. K. Kim at jacobkim@sait.samsung.co.kr
----------------------------------------------------------------------*/

// FloatingPointNumber.h: FloatingPointNumber decoder
// Author: Sang-Oak Woo
// Editor: Shinjun Lee, Samsung AIT
// 4th revision: 2002-05-03

#ifndef _FLOATINGPOINTNUMBER_H_
#define _FLOATINGPOINTNUMBER_H_

#include "Entropy.h"

#include <math.h>

class FloatingPointNumber  
	{
  private:
	int getNbBits(int k)
		{
		int j=1;
		while (k >= (1<<j) ) 
			{
			j++;
			}
		return j;
		}
  public:
	// if nDigit != 0
	unsigned int nSign;
	int nMantissa;
	int nExponent;
	unsigned int nExponentSign;

	// for decoding process
	int nDigit;

public:
	void readBitstream(CAC_QF& codec, unsigned int nDigit, unsigned int nExponentBits,
		unsigned int bAllSameExponentSign, unsigned int nSameExponentSign)
		{
		this->nDigit = nDigit;
		
		// if nDigit != 0
		if(nDigit != 0)
			{
			nSign = codec._istrm->getbit();
			
			int nBits = getNbBits((int) ::pow(10, nDigit) - 1);
			nMantissa = codec._istrm->getbits(nBits);
			
			nExponent = codec._istrm->getbits(nExponentBits);
			
			if(bAllSameExponentSign == 0)
				nExponentSign = codec._istrm->getbit();
			else
				nExponentSign = nSameExponentSign;
			}
		}

	void decodingProcess(double& fValue)
		{
		if(nDigit != 0)
			{
			if(nExponentSign == 1)
				nExponent = -nExponent;
			
			fValue = (double)nMantissa / (double)::pow(10, nExponent);
			
			if(nSign == 1)
				fValue = -fValue;
			}
		else
			fValue = 0.0f;
		}

	};

#endif // _FLOATINGPOINTNUMBER_H_
