/***********************************************************************
This software module was originally developed by James D. K. Kim, 
Sang Oak Woo and Shin Jun Lee(Samsung Advance Institute of Technology)
in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)
standard. This software module is an implementation of a part of one or
more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4
Systems (ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 
Systems (ISO/IEC 14496-1) free license to this software module or 
modifications thereof for use in hardware or software products claiming
conformance to the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to 
use this software module in hardware or software products are advised that
its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Samsung Advance Institute of Technology
retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using
the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
This copyright notice must be included in all copies or derivative works.
Copyright ?2001
************************************************************************

For more information or to receive an up to date version of this module,
you may call Sang Oak Woo at sowoo@sait.samsung.co.kr
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Interpolator Compression
FILE:       JWTemplArray.h
AUTHOR:     Sang Oak Woo (SAMSUNG AIT)
OVERVIEW:   Implementation of Array
-------------------------------------------------------------------------*/


#ifndef __JWTemplArray__H__
#define __JWTemplArray__H__

#include <memory.h>
#include <malloc.h>
#include <assert.h>

#include "iostream.h"


/*
	Please check JWArray has
		(1) Default constructor (o)
		(2) Copy consturctor (o)
		(3) Equal operator (x) 
		(4) Destructor (o)

	2002.11.22. by sowoo
*/


/*
	 Ŭ  ϱ ؼ
	  Լ Ǿ ־ Ѵ.
*/

template<class type> class JWArray
{
public:
	JWArray();
	JWArray(const JWArray<type>& a);

	virtual ~JWArray();
	

public:
	// member functions

	// sowoo 2001.07.07 (FASTFAST)
	// ʱȭ  ʰų ʿ   Լ
//	void	Initialize();

	bool	SetSize(int size);
	 int	GetSize()			{ return m_dSize; } ;
	 int	GetLength()			{ return m_dLength; } ;
	bool	Add(type instance);
	bool	InsertAt(int index, type instance);

	// sowoo 2001.09.06 (Add RemoveAt)
	// index ִ element  Լ
	void	RemoveAt(int index);

	bool	IsIn(type instance);


	// sowoo 2001.12.14 (  ä!)
	void Fill(type instance);


	bool	SetAt(int index, type instance);
	type	GetAt(int index) { return m_pData[index]; } ;
	void	RemoveAll()
	{
		if(m_pData != 0)
			delete [] m_pData;

		m_dSize = 0;
		m_pData = 0;
		m_dNumOfIncrease = 1;
	} ;

	bool Append(const JWArray<type>& arr);
	bool Copy(const JWArray<type>& arr);
	bool Copy(type* arr, int num);

public:
	// operator overloading function
	type& operator[](int index) 
	{
		assert(m_dSize > index);
		return m_pData[index];
	};

	type operator[](int index) const
	{
		assert(m_dSize > index);
		return m_pData[index];
	};

	// 2002.11.22 by sowoo
	JWArray<type>& operator=(const JWArray<type>& arr)
	{
		// 1.  ִ ͵  ش.
		RemoveAll();

		// 2. copy ͵鿡  غ  ,
		SetSize(arr.m_dSize);
		m_dNumOfIncrease = arr.m_dNumOfIncrease;

		// 3.  copy Ѵ.
		for(int i=0; i<m_dSize; i++)
		{
			m_pData[i] = arr.m_pData[i];
		}

		return *this;
	}

private:
	bool IncreaseSize(int increaseby = 1);

protected:
	// member variables
	// Ҵ ޸ ũ
	int m_dSize;
	//  Ͱ Ǵ  
	type* m_pData;

private:
	int m_dNumOfIncrease;
};

template <class type>
JWArray<type>::JWArray():
	m_dSize(0),
	m_pData(0),
	m_dNumOfIncrease(1)
{}

// sowoo 2001.12.14
template <class type>
void JWArray<type>::Fill(type instance)
{
	for(int i=0; i<m_dSize; i++)
		m_pData[i] = instance;
}


// sowoo 2001.09.06
template <class type>
bool JWArray<type>::IsIn(type instance)
{
	for(int i=0; i<m_dSize; i++)
	{
		if(m_pData[i] == instance)
			return true;
	}

	return false;
}

/*
// 20021123 sowoo
// new/delete version ٲ鼭 ʿ.
// ʿ  翬.
template <class type>
void JWArray<type>::Initialize()
{
	m_dSize = 0;
	m_pData = 0;
	m_dNumOfIncrease = 1;
}
*/

template <class type>
JWArray<type>::JWArray(const JWArray<type>& a):
	m_pData(0)
{
	SetSize(a.m_dSize);
	m_dNumOfIncrease = a.m_dNumOfIncrease;

	for(int i=0; i<m_dSize; i++)
	{
		m_pData[i] = a.m_pData[i];
	}
}


template <class type>
JWArray<type>::~JWArray()
{
	if(m_pData != 0)
	{
		delete [] m_pData;
		m_pData = 0;
	}
}

template <class type>
bool JWArray<type>::SetSize(int size)
{
	if(m_pData != 0)
	{
		delete [] m_pData;
	}
	m_pData = new type[size];
	m_dSize = size;
	m_dNumOfIncrease = size;
	return (m_pData != 0);
}

template <class type>
bool JWArray<type>::Add(type instance)
{
	int tempSize = m_dSize;
	if(!IncreaseSize())
		return false;
	m_pData[tempSize] = instance;
	return true;
}

template <class type>
bool JWArray<type>::InsertAt(int index, type instance)
{
	assert(m_dSize > index);
	type* tempPointer;
	int prevSize;

	prevSize = m_dSize;

	if(!IncreaseSize())
		return false;

	tempPointer = new type[prevSize - index];

	for(int i=0; i<prevSize - index; i++)
		tempPointer[i] = m_pData[index + i];

	m_pData[index] = instance;

	for(i=0; i<prevSize - index; i++)
		m_pData[index + 1 + i] = tempPointer[i];

	delete [] tempPointer;
	tempPointer = 0;

	return true;
}

// sowoo 2001.09.06 (Add RemoveAt)
template <class type>
void JWArray<type>::RemoveAt(int index)
{
	// 1.   ִ ۾ΰ?
	assert(m_dSize > 0);
	type* tempPointer;

	// 2. size-1ŭ ޸ Ҵ
	tempPointer = new type[m_dSize - 1];

	// 3. [start, index-1] !
	for(int i=0; i<index; i++)
		tempPointer[i] = m_pData[i];

	// 4. [index+1, end] !
	for(i=index; i<m_dSize-1; i++)
		tempPointer[i] = m_pData[i + 1];

	delete [] m_pData;


	// 5. tempPointer ° --> m_pData...
	m_pData = tempPointer;

	// 6.  ũ  !
	m_dSize--;
}


template <class type>
bool JWArray<type>::SetAt(int index, type instance)
{
	assert(m_dSize > index);
	m_pData[index] = instance;
	return true;
}

// private function zone

template <class type>
bool JWArray<type>::IncreaseSize(int increaseby)
{
	if(m_dSize == 0)
	{
		m_pData = new type[increaseby];
		m_dSize += increaseby;
		return true;
	}

	type* tempPointer;
	tempPointer = new type[m_dSize];
	assert(tempPointer != 0);

	for(int i=0; i<m_dSize; i++)
		tempPointer[i] = m_pData[i];

	delete [] m_pData;
	m_pData = 0;

	int prevSize = m_dSize;
	m_dSize += increaseby;

	m_pData = new type[m_dSize];
	assert(m_pData != 0);

	for(i=0; i<prevSize; i++)
		m_pData[i] = tempPointer[i];


	delete [] tempPointer;
	tempPointer = 0;

	return true;
}


template <class type>
bool JWArray<type>::Append(const JWArray<type>& arr)
{
	int size = arr.m_dSize;

	for(int i=0; i<size; i++)
	{
		if(!Add(arr.m_pData[i]))
			return false;
	}

	return true;
}

template <class type>
bool JWArray<type>::Copy(const JWArray<type>& arr)
{
	RemoveAll();

	m_dSize = arr.m_dSize;
	m_dNumOfIncrease = arr.m_dNumOfIncrease;

	m_pData = new type[m_dSize];
	assert(m_pData != 0);

	for(int i=0; i<m_dSize; i++)
		m_pData[i] = arr.m_pData[i];

	return true;
}




template <class type>
bool JWArray<type>::Copy(type* arr, int num)
{
	RemoveAll();

	m_dSize = num;
	m_dNumOfIncrease = 1;

	m_pData = new type[m_dSize];
	assert(m_pData != 0);

	for(int i=0; i<m_dSize; i++)
		m_pData[i] = arr[i];

	return true;
}


#endif