/***********************************************************************
This software module was originally developed by James D. K. Kim, 
Sang Oak Woo and Shin Jun Lee(Samsung Advance Institute of Technology)
in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)
standard. This software module is an implementation of a part of one or
more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4
Systems (ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 
Systems (ISO/IEC 14496-1) free license to this software module or 
modifications thereof for use in hardware or software products claiming
conformance to the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to 
use this software module in hardware or software products are advised that
its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Samsung Advance Institute of Technology
retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using
the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
This copyright notice must be included in all copies or derivative works.
Copyright  2001
************************************************************************

For more information or to receive an up to date version of this module,
you may contact James D. K. Kim at jacobkim@sait.samsung.co.kr
----------------------------------------------------------------------*/

// Key.h: key decoder
// Author: Shinjun Lee, Samsung AIT
// 4th revision: 2002-05-03

#ifndef _KEY_H_
#define _KEY_H_

#include "ArithmeticDecoder.h"
#include "KeyHeader.h"
#include "CKeyDecoder.h"

class Key
	{
  public:
	// variables: bitstream data of key
	int*			nQKey;									// key array

	// variables: nonparsable data for key decoding
	int				nNumberOfRemainingKey;					// the number of remaining key after linear key decoding

	double*			dKey;									// array of decoded key

	// Key: key context
	SAIT_CAC				keySignContext;
	SAIT_CAC				keyContext[32];

  public:
	Key(void)
		{
		//int i = 0;

		nQKey = NULL;

		nNumberOfRemainingKey = 0;

		dKey = NULL;
		}

	~Key()
		{
		delete[] nQKey;
		}

	void InitializeModel(void)
		{
		int i = 0;

		// Key: key context
		keySignContext.reset();
		for(i = 0; i < 32; i++)
			keyContext[i].reset();
		}

	void readBitstream(CAC_QF& codec, KeyHeader& kHeader)
		{
		int			i = 0;									// index

		nQKey = new int[kHeader.nNumberOfKey];
		if(kHeader.bIsLinearKeySubRegion == true)
			nNumberOfRemainingKey = kHeader.nNumberOfKey - kHeader.lKey.nNumberOfLinearKey;
		else
			nNumberOfRemainingKey = kHeader.nNumberOfKey;

		for(i = kHeader.nKDPCMOrder+1; i < nNumberOfRemainingKey; i++)
			{
			if(kHeader.bSignedAACFlag == false)
				decodeUnsignedAAC(codec, &nQKey[i], kHeader.nKeyCodingBit, keyContext);
			else
				decodeSignedAAC(codec, &nQKey[i], kHeader.nKeyCodingBit+1, keySignContext, keyContext);
			}
		}

	void decodingProcess(KeyHeader& kHeader)
		{
		int i = 0;

		// 2002-04-18, Shinjun
		JWArray<double> JWDecodedKey;			//  decoded key;
		// 2002-04-18-02, Shinjun
		if(kHeader.bIsLinearKeySubRegion == true)
			kHeader.lKey.KeyReconstructor(kHeader.nKeyDigit, kHeader.lKey.fKeyMin, kHeader.lKey.fKeyMax,
				kHeader.lKey.nNumberOfLinearKey, JWDecodedKey);

		for(i = 0; i < kHeader.nKDPCMOrder+1; i++)
			nQKey[i] = kHeader.nQIntraKey[i];

		// key decoder
		CKeyDecoder oKeyDecoder;
		oKeyDecoder.setOrder_dpcm(kHeader.nKDPCMOrder+1);
		oKeyDecoder.setOrder_dnd(kHeader.nDNDOrder);
		oKeyDecoder.setKey_shift(kHeader.nKeyShift);
		oKeyDecoder.setKey_shift_down(kHeader.nKeyInvertDown);
		oKeyDecoder.setKey_dnd_max(kHeader.nKeyMax);
		oKeyDecoder.setKeyDigit(kHeader.nKeyDigit);
		oKeyDecoder.setMinMax(kHeader.fKeyMin, kHeader.fKeyMax);
		oKeyDecoder.decode(nQKey, nNumberOfRemainingKey, kHeader.nKeyQBit);

		// get: result data from key decoding
		oKeyDecoder.getIQKey(dKey);

		// print: result data from key decoding
		/*
		cout << "Number of keys = " << kHeader.nNumberOfKey << endl;
		cout << "QStep_key = " << kHeader.nKeyQBit << endl;
		cout << "Key digit = " << kHeader.nKeyDigit << endl;
		cout << "Order_dpcm = " << kHeader.nKDPCMOrder+1 << endl;
		cout << "Order_dnd = " << kHeader.nDNDOrder << endl;
		cout << "Key to shift = " << kHeader.nKeyShift << endl;
		cout << "Key to shift-down = " << kHeader.nKeyInvertDown << endl;
		cout << "QStep(key_shift) = " << kHeader.nBitSize << endl;
		cout << "QStep(key_decoding) = " << kHeader.nKeyCodingBit << endl;
		cout << endl;
		cout << "dKeyMin = " << kHeader.lKey.fKeyMin << endl;
		cout << "dKeyMax = " << kHeader.lKey.fKeyMax << endl;
		cout << "nNumberOfLinearKey = " << kHeader.lKey.nNumberOfLinearKey << endl;
		cout << endl;
		*/
		// 2002-04-18-02, Shinjun
		if(kHeader.bIsLinearKeySubRegion == true)
			kHeader.lKey.KeyMerger(kHeader.nNumberOfKey, dKey, nNumberOfRemainingKey, JWDecodedKey, kHeader.lKey.nNumberOfLinearKey);
		}
	};

#endif // _KEY_H_