/***********************************************************************
This software module was originally developed by James D. K. Kim, 
Sang Oak Woo and Shin Jun Lee(Samsung Advance Institute of Technology)
in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)
standard. This software module is an implementation of a part of one or
more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4
Systems (ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 
Systems (ISO/IEC 14496-1) free license to this software module or 
modifications thereof for use in hardware or software products claiming
conformance to the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to 
use this software module in hardware or software products are advised that
its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Samsung Advance Institute of Technology
retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using
the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
This copyright notice must be included in all copies or derivative works.
Copyright  2001
************************************************************************

For more information or to receive an up to date version of this module,
you may contact James D. K. Kim at jacobkim@sait.samsung.co.kr
----------------------------------------------------------------------*/

// KeyHeader.h: key header decoder
// Author: Shinjun Lee, Samsung AIT
// 4th revision: 2002-05-03

#ifndef _KEYHEADER_H_
#define _KEYHEADER_H_

#include "LinearKey.h"
#include "KeyMinMax.h"

class KeyHeader
	{
  public:
	// variables: bitstream data of key
	int				nKeyQBit;								// quantization bit for key
	int				nNumKeyCodingBit;						// the bit for nNumberOfKey
	int				nNumberOfKey;							// the number of keys
	int				nKeyDigit;								// key digit

	bool			bIsLinearKeySubRegion;					// flag of linear key decoding
	LinearKey		lKey;									// class of linear key decoder

	bool			bRangeFlag;								// flag of key range
	KeyMinMax		keyMinMax;								// class of key min and key max decoder

	int				nBitSize;								// bit size
	int				nKDPCMOrder;							// order of key DPCM
	int*			nQIntraKeySign;							// sign array for intra keys
	int*			nQIntraKey;								// intra key array

	bool			bShiftFlag;								// flag of shift
	int				nKeyShiftSign;							// sign of key shift
	int				nKeyShift;								// the value of key shift

	int				nDNDOrder;								// order of DND

	int*			nKeyMaxSign;							// sign array for intra DND values
	int*			nKeyMax;								// intra DND values

	int				nKeyCodingBit;							// coding bit size for key
	int				nKeyInvertDown;							// the value of key invert down

	// variables: nonparsable data for key decoding
	bool			bSignedAACFlag;							// flag of signed aac decoding

	double			fKeyMin;								// key min
	double			fKeyMax;								// key max

  public:
	KeyHeader(void)
		{
		nKeyQBit = 0;
		nNumKeyCodingBit = 0;
		nNumberOfKey = 0;
		nKeyDigit = 0;

		bIsLinearKeySubRegion = false;

		bRangeFlag = false;

		nBitSize = 0;
		nKDPCMOrder = 0;
		nQIntraKeySign = NULL;
		nQIntraKey = NULL;

		bShiftFlag = false;
		nKeyShiftSign = 0;
		nKeyShift = 0;

		nDNDOrder = 0;

		nKeyMaxSign = NULL;
		nKeyMax = NULL;

		nKeyCodingBit = 0;
		nKeyInvertDown = 0;

		bSignedAACFlag = false;
		}

	~KeyHeader()
		{
		delete[] nQIntraKeySign;
		delete[] nQIntraKey;

		delete[] nKeyMaxSign;
		delete[] nKeyMax;
		}

	void readBitstream(CAC_QF& codec)
		{
		int			i = 0;									// index

		// get: nKeyQBit
		nKeyQBit = codec._istrm->getbits(5);
		
		// get: the number of keys
		nNumKeyCodingBit = codec._istrm->getbits(5);
		nNumberOfKey = codec._istrm->getbits(nNumKeyCodingBit);

		// get: key digit
		nKeyDigit = codec._istrm->getbits(4);

		// get: linear key
		if(codec._istrm->getbits(1) == 1)
			{
			bIsLinearKeySubRegion = true;
			lKey.readBitstream(codec, nKeyDigit);
			}
		else
			bIsLinearKeySubRegion = false;

		// get: fKeyMin, fKeyMax
		if(codec._istrm->getbits(1) == 1)
			{
			bRangeFlag = true;
			keyMinMax.readBitstream(codec, nKeyDigit);
			}
		else
			{
			bRangeFlag = false;
			}

		// get: nBitSize
		nBitSize = codec._istrm->getbits(5);

		// get: nKDPCMOrder
		nKDPCMOrder = codec._istrm->getbits(2);
		nQIntraKeySign = new int[nKDPCMOrder+1];
		nQIntraKey = new int[nKDPCMOrder+1];

		// get: intra key data
		for(i = 0; i < nKDPCMOrder+1; i++)
			{
			nQIntraKeySign[i] = codec._istrm->getbits(1);
			if(i == 0 && nQIntraKeySign[i] == 1)
				continue;
			nQIntraKey[i] = codec._istrm->getbits(nBitSize);
			}

		// get: shift key data
		bShiftFlag = codec._istrm->getbits(1) == 1 ? true : false;
		if(bShiftFlag == true)
			{
			nKeyShiftSign = codec._istrm->getbits(1);
			nKeyShift = codec._istrm->getbits(nBitSize);
			}

		// get: nDNDOrder
		nDNDOrder = codec._istrm->getbits(3);
		if(nDNDOrder == 7)
			{
			int bNoDND = codec._istrm->getbits(1);
			if(bNoDND == 1)
				nDNDOrder = -1;
			}

		// get: nKeyMax
		int nMaxQBit = nBitSize;
		if(nDNDOrder > 0)
			{
			nKeyMaxSign = new int[nDNDOrder];
			nKeyMax = new int[nDNDOrder];
			}
		for(i = 0; i < nDNDOrder; i++)
			{
			nKeyMaxSign[i] = codec._istrm->getbits(1);
			nKeyMax[i] = codec._istrm->getbits(nMaxQBit);

			nMaxQBit = (int)(log10(abs(nKeyMax[i]))/log10(2))+1;
			if(nMaxQBit+1 < nBitSize)
				nMaxQBit += 1;
			else
				nMaxQBit = nBitSize;
			}

		// get: nKeyCodingBit
		int nKeyCodingBitQBit = (int)(log10(abs(nKeyQBit))/log10(2))+1;
		nKeyCodingBit = codec._istrm->getbits(nKeyCodingBitQBit); // 2001-11-20

		// get: nKeyInvertDown
		if(nDNDOrder != -1 && nDNDOrder != 0)
			{
			if(codec._istrm->getbits(1) == 1)
				{
				nKeyInvertDown = codec._istrm->getbits(nKeyCodingBit);
				bSignedAACFlag = false;
				}
			else
				{
				nKeyInvertDown = -1;
				bSignedAACFlag = true;
				}
			}
		else
			{
			nKeyInvertDown = -1;
			bSignedAACFlag = false;
			}
		}

	void decodingProcess(void)
		{
		int			i = 0;									// index

		// get: linear key
		if(bIsLinearKeySubRegion == true)
			lKey.decodingProcess();

		// get: fKeyMin, fKeyMax
		if(bRangeFlag == true)
		{
			keyMinMax.decodingProcess();

			fKeyMin = keyMinMax.fKeyMin;
			fKeyMax = keyMinMax.fKeyMax;
		}
		else
			{
			fKeyMin = 0.0;
			fKeyMax = 1.0;
			}

		// get: intra key data
		for(i = 0; i < nKDPCMOrder+1; i++)
			{
			if(i == 0 && nQIntraKeySign[i] == 1)
				{
				nQIntraKey[i] = 0;
				continue;
				}

			if(nQIntraKeySign[i] == 1)
				nQIntraKey[i] *= -1;
			}

		// get: shift key data
		if(bShiftFlag == 1)
			{
			if(nKeyShiftSign == 1)
				nKeyShift *= -1;
			}

		// get: nKeyMax
		for(i = 0; i < nDNDOrder; i++)
			{
			if(nKeyMaxSign[i] == 1)
				nKeyMax[i] *= -1;
			}
		}
	};

#endif // _KEYHEADER_H_
