/***********************************************************************
This software module was originally developed by James D. K. Kim, 
Sang Oak Woo and Shin Jun Lee(Samsung Advance Institute of Technology)
in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)
standard. This software module is an implementation of a part of one or
more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4
Systems (ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 
Systems (ISO/IEC 14496-1) free license to this software module or 
modifications thereof for use in hardware or software products claiming
conformance to the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to 
use this software module in hardware or software products are advised that
its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Samsung Advance Institute of Technology
retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using
the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
This copyright notice must be included in all copies or derivative works.
Copyright  2001
************************************************************************

For more information or to receive an up to date version of this module,
you may contact James D. K. Kim at jacobkim@sait.samsung.co.kr
----------------------------------------------------------------------*/

// KeyMinMaxDecoder
// Author: Sang-Oak Woo
// Editor: Shinjun Lee (codes are modified into classes)
// Date: 2001. 12. 27.
// 4th revision: 2002-05-03

#ifndef _KEYMINMAX_H_
#define _KEYMINMAX_H_

#include "Entropy.h"

#include <math.h>

class KeyMinMax
	{
  public:
	int			nMinKeyDigit;

	int			nMinKeyMantissaSign;
	int			nMinKeyMantissa;

	int			nMinKeyExponentSign;
	int			nMinKeyExponent;

	int			nMaxKeyDigit;

	int			nMaxKeyMantissaSign;
	int			nMaxKeyMantissa;

	int			nMaxKeyExponentSign;
	int			nMaxKeyExponent;

	int			bSameExponent;

	double		fKeyMin;
	double		fKeyMax;

  public:
	KeyMinMax(void)
		{
		nMinKeyDigit = 0;

		nMinKeyMantissaSign = 0;
		nMinKeyMantissa = 0;

		nMinKeyExponentSign = 0;
		nMinKeyExponent = 0;

		nMaxKeyDigit = 0;

		nMaxKeyMantissaSign = 0;
		nMaxKeyMantissa = 0;

		nMaxKeyExponentSign = 0;
		nMaxKeyExponent = 0;

		bSameExponent = 0;

		fKeyMin = 0;
		fKeyMax = 0;
		}

	void readBitstream(CAC_QF& codec, int nKeyDigit)
		{
		int bMinKeyDigitSame = codec._istrm->getbits(1);
		if(bMinKeyDigitSame == 0)
			nMinKeyDigit = codec._istrm->getbits(4);
		else
			nMinKeyDigit = nKeyDigit;

		if(nMinKeyDigit != 0)
			{
			if(nMinKeyDigit < 8)
				{
				// start key
				int count = getNbBits((int)pow(10.0, nMinKeyDigit));
				nMinKeyMantissaSign = codec._istrm->getbit();
				nMinKeyMantissa = codec._istrm->getbits(count);
				
				nMinKeyExponentSign = codec._istrm->getbit();
				nMinKeyExponent = codec._istrm->getbits(6);
				}
			else
				{
				int integer = codec._istrm->getbits(32);
				fKeyMin = *((float *)&integer);
				}
			}

		int bMaxKeyDigitSame = codec._istrm->getbits(1);
		if(bMaxKeyDigitSame == 0)
			nMaxKeyDigit = codec._istrm->getbits(4);
		else
			nMaxKeyDigit = nKeyDigit;

		if(nMaxKeyDigit != 0)
			{
			if(nMaxKeyDigit < 8)
				{
				// end key
				int count = getNbBits((int)pow(10.0, nMaxKeyDigit));
				nMaxKeyMantissaSign = codec._istrm->getbit();
				nMaxKeyMantissa = codec._istrm->getbits(count);
				
				bSameExponent = codec._istrm->getbits(1);
				if(bSameExponent == 0)
					{
					nMaxKeyExponentSign = codec._istrm->getbit();
					nMaxKeyExponent = codec._istrm->getbits(6);
					}
				else
					{
					nMaxKeyExponent = nMinKeyExponent;
					}
				}
			else
				{
				int integer = codec._istrm->getbits(32);
				fKeyMax = *((float *)&integer);
				}
			}
		}

	void decodingProcess(void)
		{
		if(nMinKeyDigit != 0)
			{
			if(nMinKeyDigit < 8)
				{
				// start key
				if(nMinKeyMantissaSign == 1)
					nMinKeyMantissa *= -1;
				
				if(nMinKeyExponentSign == 1)
					nMinKeyExponent *= -1;

				fKeyMin = nMinKeyMantissa / pow(10.0, nMinKeyExponent);
				}
			}
		else
			{
			fKeyMin = 0.0;
			}

		if(nMaxKeyDigit != 0)
			{
			if(nMaxKeyDigit < 8)
				{
				// end key
				if(nMaxKeyMantissaSign == 1)
					nMaxKeyMantissa *= -1;
				
				if(bSameExponent == 0)
					{
					if(nMaxKeyExponentSign == 1)
						nMaxKeyExponent *= -1;
					}
				else
					{
					nMaxKeyExponent = nMinKeyExponent;
					}

				fKeyMax = nMaxKeyMantissa / pow(10.0, nMaxKeyExponent);
				}
			}
		else
			{
			fKeyMax = 0.0;
			}
		}

  private:
	int getNbBits(int k)
		{
		int j=1;
		while (k >= (1<<j) ) 
			{
			j++;
			}
		return j;
		}
	};

#endif // _KEYMINMAX_H_