/***********************************************************************
This software module was originally developed by James D. K. Kim, 
Sang Oak Woo and Shin Jun Lee(Samsung Advance Institute of Technology)
in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)
standard. This software module is an implementation of a part of one or
more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4
Systems (ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 
Systems (ISO/IEC 14496-1) free license to this software module or 
modifications thereof for use in hardware or software products claiming
conformance to the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to 
use this software module in hardware or software products are advised that
its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Samsung Advance Institute of Technology
retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using
the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
This copyright notice must be included in all copies or derivative works.
Copyright  2001
************************************************************************

For more information or to receive an up to date version of this module,
you may contact James D. K. Kim at jacobkim@sait.samsung.co.kr
----------------------------------------------------------------------*/

// KeyValueMinMax.h: KeyValueMinMax decoder
// Author: Sang-Oak Woo
// Editor: Shinjun Lee, Samsung AIT
// 4th revision: 2002-05-03

#ifndef _KEYVALUEMINMAX_H_
#define _KEYVALUEMINMAX_H_

#include "Entropy.h"
#include "FloatingPointNumber.h"

class KeyValueMinMax  
	{
  public:
	unsigned int bUse32Float;

	// if bUsed32Float == 0
	unsigned int nWhichAxis;
	unsigned int bAllSameMantissaDigitFlag;
	unsigned int nMantissaDigit_X;
	unsigned int nMantissaDigit_Y;
	unsigned int nMantissaDigit_Z;
	unsigned int bSameKVDigitFlag;

	unsigned int bMaxDigitFlag;
	unsigned int nMantissaDigit_M;
	unsigned int nExponentBits;
	unsigned int bAllSameExponentSign;

	// if bAllSameExponentSign == 1
	unsigned int nExponentSign;

	FloatingPointNumber fpnMin_X;
	FloatingPointNumber fpnMin_Y;
	FloatingPointNumber fpnMin_Z;
	FloatingPointNumber fpnMax;

	// if bUsed32Float == 1
	double fMin_X;
	double fMin_Y;
	double fMin_Z;
	double fMax;

	// for decoding process
	double fMax_X;
	double fMax_Y;
	double fMax_Z;

  public:
	  KeyValueMinMax(): nWhichAxis(3) {}	// SAIT_CONFORMANCE_2003

	void readBitstream(CAC_QF& codec, int nDigit)
		{
		bUse32Float = codec._istrm->getbit();
		
		// if bUsed32Float == 0
		if(bUse32Float == 0)
			{
			nWhichAxis = codec._istrm->getbits(2);
			bAllSameMantissaDigitFlag = codec._istrm->getbit();
			
			if(bAllSameMantissaDigitFlag == 0)
				{
				nMantissaDigit_X = codec._istrm->getbits(4);
				nMantissaDigit_Y = codec._istrm->getbits(4);
				nMantissaDigit_Z = codec._istrm->getbits(4);
				}
			else
				{
				bSameKVDigitFlag = codec._istrm->getbit();
				
				if(bSameKVDigitFlag == 0)
					nMantissaDigit_X = codec._istrm->getbits(4);
				else
					nMantissaDigit_X = nDigit;
				
				nMantissaDigit_Y = nMantissaDigit_X;
				nMantissaDigit_Z = nMantissaDigit_X;
				}

			bMaxDigitFlag = codec._istrm->getbit();
			
			if(bMaxDigitFlag == 1)
				nMantissaDigit_M = codec._istrm->getbits(4);
			else
				{
				switch(nWhichAxis){
					case 0:
						nMantissaDigit_M = nMantissaDigit_X;
						break;
					case 1:
						nMantissaDigit_M = nMantissaDigit_Y;
						break;
					case 2:
						nMantissaDigit_M = nMantissaDigit_Z;
						break;
					default:
						// error!!!
						break;
					}
				}
			
			nExponentBits = codec._istrm->getbits(6);
			nExponentBits = getNbBits(nExponentBits);
			
			bAllSameExponentSign = codec._istrm->getbit();
			
			// if bAllSameExponentSign == 1
			if(bAllSameExponentSign == 1)
				nExponentSign = codec._istrm->getbit();
			
			fpnMin_X.readBitstream(codec, nMantissaDigit_X, 
				nExponentBits, bAllSameExponentSign, nExponentSign);
			fpnMin_Y.readBitstream(codec, nMantissaDigit_Y, 
				nExponentBits, bAllSameExponentSign, nExponentSign);
			fpnMin_Z.readBitstream(codec, nMantissaDigit_Z, 
				nExponentBits, bAllSameExponentSign, nExponentSign);
			fpnMax.readBitstream(codec, nMantissaDigit_M, 
				nExponentBits, bAllSameExponentSign, nExponentSign);
			}
		else
			{
			// if bUsed32Float == 1
			int nTemp;
			
			nTemp = codec._istrm->getbits(32);
			fMin_X = (double)*((float*)&nTemp);
			
			nTemp = codec._istrm->getbits(32);
			fMin_Y = (double)*((float*)&nTemp);
			
			nTemp = codec._istrm->getbits(32);
			fMin_Z = (double)*((float*)&nTemp);
			
			nTemp = codec._istrm->getbits(32);
			fMax = (double)*((float*)&nTemp);
			}
		}
	
	void decodingProcess(void)
		{
		if(bUse32Float == 0)
			{
			fpnMin_X.decodingProcess(fMin_X);
			fpnMin_Y.decodingProcess(fMin_Y);
			fpnMin_Z.decodingProcess(fMin_Z);
			fpnMax.decodingProcess(fMax);
			}
		
		double RangeMax;
		switch(nWhichAxis){
			case 0:
				RangeMax = fMax - fMin_X;
				fMax_X = fMax;
				fMax_Y = RangeMax + fMin_Y;
				fMax_Z = RangeMax + fMin_Z;
				break;
			case 1:
				RangeMax = fMax - fMin_Y;
				fMax_X = RangeMax + fMin_X;
				fMax_Y = fMax;
				fMax_Z = RangeMax + fMin_Z;
				break;
			case 2:
				RangeMax = fMax - fMin_Z;
				fMax_X = RangeMax + fMin_X;
				fMax_Y = RangeMax + fMin_Y;
				fMax_Z = fMax;
				break;
			case 3: // default case
				fMax_X = fMin_X + fMax;
				fMax_Y = fMin_Y + fMax;
				fMax_Z = fMin_Z + fMax;
				break;
			}
		}

  private:
	int getNbBits(int k)
		{
		int j=1;
		while (k >= (1<<j) ) 
			{
			j++;
			}
		return j;
		}
	};
	
#endif // _KEYVALUEMINMAX_H_
