/***********************************************************************
This software module was originally developed by James D. K. Kim, 
Sang Oak Woo and Shin Jun Lee(Samsung Advance Institute of Technology)
in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)
standard. This software module is an implementation of a part of one or
more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4
Systems (ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 
Systems (ISO/IEC 14496-1) free license to this software module or 
modifications thereof for use in hardware or software products claiming
conformance to the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to 
use this software module in hardware or software products are advised that
its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Samsung Advance Institute of Technology
retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using
the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
This copyright notice must be included in all copies or derivative works.
Copyright  2001
************************************************************************

For more information or to receive an up to date version of this module,
you may contact James D. K. Kim at jacobkim@sait.samsung.co.kr
----------------------------------------------------------------------*/

// Linear key decoder class
// Author: James Kim
// Editor: Shinjun Lee
// Date: 2002. 4. 18. Thu.
// 4th revision: 2002-05-03

#ifndef _LINEARKEY_H_
#define _LINEARKEY_H_

#include "KeyMinMax.h"
#include "JWTemplArray.h"
#include <stdlib.h>

// Linear key decoder
class LinearKey
	{
  public:
	double fKeyMin;
	double fKeyMax;
	int nNumberOfLinearKey;
	KeyMinMax kMinMax;

  public:
	LinearKey(void)
		{
		fKeyMin = 0;
		fKeyMax = 0;
		nNumberOfLinearKey = 0;
		}

	void readBitstream(CAC_QF& codec, int& nKeyDigit)
		{
		int nNumLinearKeyCodingBit = 0;

		nNumLinearKeyCodingBit = codec._istrm->getbits(5);
		nNumberOfLinearKey = codec._istrm->getbits(nNumLinearKeyCodingBit);

		kMinMax.readBitstream(codec, nKeyDigit);
		}

	void decodingProcess(void)
		{
		kMinMax.decodingProcess();

		fKeyMin = kMinMax.fKeyMin;
		fKeyMax = kMinMax.fKeyMax;
		}

	// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
	//
	//
	// Key Reconstructor :
	// reconstruct: the subsed of key data
	//
	//
	// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
	void KeyReconstructor(int nKeyDigit,
							double dKeyMin,
							double dKeyMax,
							int nNumberOfLinearKey,
							JWArray<double>& JWDecodedKey)
		{
		JWArray<double>		JWTempDecodedKey;

		double startKey = dKeyMin;
		double endKey = dKeyMax;

		// decodekey for error measure
		JWTempDecodedKey.RemoveAll();
		JWTempDecodedKey.SetSize(nNumberOfLinearKey);
		for(int decIdx = 0;decIdx < nNumberOfLinearKey;decIdx++)
			{
			JWTempDecodedKey[decIdx] = startKey +
				((endKey-startKey)/(double)(nNumberOfLinearKey-1))*decIdx;
			
			char num[50];
			_gcvt(JWTempDecodedKey[decIdx] , nKeyDigit, num);
			JWTempDecodedKey[decIdx]  = atof(num);		
			}
		
		if(JWDecodedKey.GetSize() == 0)
			{
			for(int decIdx = 0;decIdx < nNumberOfLinearKey;decIdx++)
				{
				JWDecodedKey.Add(JWTempDecodedKey[decIdx]);
				}
			}
		else
			{
			for(int decIdx = 0;decIdx < nNumberOfLinearKey;decIdx++)
				for(int dectmpIdx = 0;dectmpIdx < JWDecodedKey.GetSize();dectmpIdx++)
					{
					// if first
					if((dectmpIdx == 0) && (JWDecodedKey[dectmpIdx] >= JWTempDecodedKey[decIdx]))
						{
						JWDecodedKey.InsertAt(0, JWTempDecodedKey[decIdx]);
						}
					else if(JWDecodedKey[dectmpIdx] < JWTempDecodedKey[decIdx]) 
						{
						if((dectmpIdx+1) < JWDecodedKey.GetSize())
							{
							if(JWDecodedKey[dectmpIdx+1] >= JWTempDecodedKey[decIdx])
								JWDecodedKey.InsertAt(dectmpIdx+1, JWTempDecodedKey[decIdx]);
							}
						else
							JWDecodedKey.Add(JWTempDecodedKey[decIdx]);
						}
					}
			}
		}

	// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
	//
	//
	// Key Merger :
	// merge the subset data and the decoded key data from key decoder
	//
	//
	// @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
	void KeyMerger(int nNumberOfKey, double*& a_dKey, int nNumberOfRemainingKey, JWArray<double>& JWDecodedKey, int nNumberOfLinearKey)
		{
		double* a_dKeyTotal = new double[nNumberOfKey];
		int i = 0;
		int j = 0;
		int k = 0;

		for(i = 0; i < nNumberOfKey; i++)
			{
			if(j >= nNumberOfRemainingKey)
				{
				a_dKeyTotal[i] = JWDecodedKey[k];
				k++;
				continue;
				}
			else if(k >= nNumberOfLinearKey)
				{
				a_dKeyTotal[i] = a_dKey[j];
				j++;
				continue;
				}

			if(a_dKey[j] < JWDecodedKey[k])
				{
				a_dKeyTotal[i] = a_dKey[j];
				j++;
				}
			else
				{
				a_dKeyTotal[i] = JWDecodedKey[k];
				k++;
				}
			}

		delete[] a_dKey;
		a_dKey = a_dKeyTotal;
		}
	};

#endif // _LINEARKEY_H_