// Key encoder class
// Date: 2001. 5. 29. Tue. - 6. 26. Tue.
// 2nd revision: 2001. 9. 4. Tue.
// Author: Shinjun Lee

#include <iostream.h>
#include <math.h>
#include <float.h>
#include <limits.h>
#include <stdlib.h>

#define sign(n) ((n >= 0) ? 1 : -1)			// sign: ȣ Լ

const double dKEY_MIN = 0.0;				// Key Min: Ű ּ 
const double dKEY_MAX = 1.0;				// Key Max: Ű ִ 

const int nDPCM_MAX = 3;					// DPCM Max: dpcm ִ 
const int nDND_MAX = 7;						// DND MAX: dnd ִ 

const double dVAR_MAX = 0.90;				// Var. MAX: Variance   Ѱ

// Compare: qsort   Լ
int compare(const void* arg1, const void* arg2);

// Key Encoder: Ű ȣȭ Ŭ
class CKeyEncoder
	{
private:
	double*		m_a_dKey;					// Ű Ÿ
	int*		m_a_nQKey;					// ȭ Ű Ÿ

	int			m_nNumberOfKey;				// Ű 
	double		m_dKeyDistortion;			// ְ 

	int			m_nOrder_dpcm;				// DPCM 
	int			m_nOrder_dnd;				// DND 

	int			m_nKey_shift;				// Shift ų 
	int			m_nKey_shift_up;			// Shift up ų 

	int			m_nDigit_key;				// Key digit 

	int			m_a_nKey_dnd_max[nDND_MAX];	// dnd  Max 

	double		m_dKey_min;					// Key Min 
	double		m_dKey_max;					// Key Max 

	// sowoo 2002.02.04 (Add Min/Max)
	bool m_bUseMinMax_bySowoo;

public:


	// sowoo 2002.02.04 (Add Min/Max)
	void SetUseMinMax_bySowoo(double dMin, double dMax)
	{
		m_bUseMinMax_bySowoo = true;
		
		m_dKey_min = dMin;
		m_dKey_max = dMax;
	}

	CKeyEncoder(void)
		{
		m_a_dKey = NULL;
		m_a_nQKey = NULL;

		m_nNumberOfKey = 0;
		m_dKeyDistortion = 0;

		m_nOrder_dpcm = 0;
		m_nOrder_dnd = 0;

		m_nKey_shift = 0;
		m_nKey_shift_up = 0;

		m_nDigit_key = 0;

		m_dKey_min = dKEY_MIN;
		m_dKey_max = dKEY_MAX;
		}

	~CKeyEncoder()
		{
		if(m_a_dKey != NULL)
			delete[] m_a_dKey;
		if(m_a_nQKey != NULL)
			delete[] m_a_nQKey;
		}

	// Setting: key digit
	void setKeyDigit(int nDigit_key)
		{
		m_nDigit_key = nDigit_key;
		}

	// KeyEncoder: ڵ Լ
	void encode(double* a_dKey, int nNumberOfKey, int nQStep_key)
		{
		int* a_nKey_dpcm = NULL;
		int* a_nKey_dpcm_min = NULL;
		int* a_nKey_fold = NULL;
		int* a_nKey_dnd = NULL;
		int* a_nKey_dnd_min = NULL;
		int* a_nKey_shift_up = NULL;
		double dQuartileDeviation = 0;
		double dQuartileDeviation_min = 0;
		int nKey_max = 0;
		int nKey_min = 0;
		int nIndex_dpcm = 0;
		int nIndex_dnd = 0;
		int nQStep_dpcm = 0;
		int nQStep_fold = 0;
		int nQStep_dnd = 0;
		bool bFirst = true;
		int i;

		// Moving: Ű , Key digit Ѱܹޱ
		m_nNumberOfKey = nNumberOfKey;
		// 2002-04-18-02, Shinjun
		if(m_nNumberOfKey == 0)
			return;

		// Allocation: ޸ Ҵ
		m_a_dKey = new double[m_nNumberOfKey];
		m_a_nQKey = new int[m_nNumberOfKey];

		a_nKey_dpcm = new int[m_nNumberOfKey];
		a_nKey_dpcm_min = new int[m_nNumberOfKey];
		a_nKey_fold = new int[m_nNumberOfKey];
		a_nKey_dnd = new int[m_nNumberOfKey];
		a_nKey_dnd_min = new int[m_nNumberOfKey];
		a_nKey_shift_up = new int[m_nNumberOfKey];

		// Moving: Ű Ѱܹޱ
		for(i = 0; i < m_nNumberOfKey; i++)
			{
			m_a_dKey[i] = a_dKey[i];
			}

		// Min, Max: min, max ϱ
		m_dKey_min = dKEY_MIN;
		m_dKey_max = dKEY_MAX;
		bFirst = true;
		if(m_a_dKey[0] < 0 || m_a_dKey[nNumberOfKey-1] > 1)
			{
			for(i = 0; i < m_nNumberOfKey; i++)
				{
				if(bFirst == true)
					{
					m_dKey_min = m_a_dKey[i];
					m_dKey_max = m_a_dKey[i];
					bFirst = false;
					}
				else
					{
					if(m_dKey_min > a_dKey[i])
						m_dKey_min = a_dKey[i];
					if(m_dKey_max < a_dKey[i])
						m_dKey_max = a_dKey[i];
					}
				}
			}

		// Quantizer: ȭϱ
		quantize(nQStep_key);

		// DPCM: n DPCM ϱ
		dQuartileDeviation_min = FLT_MAX;
		moveKey(m_a_nQKey, a_nKey_dpcm);
		for(i = 0; i < nDPCM_MAX; i++)
			{
			dpcm(a_nKey_dpcm);
			dQuartileDeviation = getQuartileDeviation(a_nKey_dpcm, i+1);
			if(dQuartileDeviation < dQuartileDeviation_min)
				{
				moveKey(a_nKey_dpcm, a_nKey_dpcm_min);
				dQuartileDeviation_min = dQuartileDeviation;
				nIndex_dpcm = i+1;
				}
			if(m_nNumberOfKey < 5)	// Key  5  , 1 DPCM 
				break;
			}
		m_nOrder_dpcm = nIndex_dpcm;
		nQStep_dpcm = getQStep(a_nKey_dpcm_min);

		// Selection:   󵵼   
		moveKey(a_nKey_dpcm_min, a_nKey_fold);
		m_nKey_shift = getFrequencyMax(a_nKey_fold);

		//write(a_nKey_fold, "k_enc.txt");

		// shift: 󵵼  ū  
		shift(a_nKey_fold, m_nKey_shift);

		// Fold: folding 
		fold(a_nKey_fold);
		nQStep_fold = getQStep(a_nKey_fold);

		// DND: divide-and-divide 
		moveKey(a_nKey_fold, a_nKey_dnd);
		nQStep_dnd = INT_MAX;
		for(i = 1; i <= nDND_MAX; i++)
			{
			nKey_max = getMax(a_nKey_dnd);
			nKey_min = getMin(a_nKey_dnd);
			if(nKey_max >= abs(nKey_min))
				{
				m_a_nKey_dnd_max[i-1] = nKey_max;
				if(i == 1)
					divide(a_nKey_dnd, nKey_max);
				else
					divide_down(a_nKey_dnd, nKey_max);
				}
			else
				{
				m_a_nKey_dnd_max[i-1] = nKey_min;
				divide_up(a_nKey_dnd, nKey_min);
				}
			int nQStep = getQStep(a_nKey_dnd);
			if(nQStep < nQStep_dnd)
				{
				moveKey(a_nKey_dnd, a_nKey_dnd_min);
				nQStep_dnd = nQStep;
				nIndex_dnd = i;
				}
			}
		m_nOrder_dnd = nIndex_dnd;
		nKey_min = getMin(a_nKey_dpcm_min);
		if((nQStep_dpcm < nQStep_dnd && nKey_min >= 0) ||
			(nQStep_dpcm <= nQStep_dnd && nKey_min == m_nKey_shift))
			{
			moveKey(a_nKey_dpcm_min, a_nKey_dnd);
			
			m_nKey_shift = nKey_min;
			shift(a_nKey_dnd, m_nKey_shift);

			m_nOrder_dnd = -1;
			m_nKey_shift_up = -1;
			}
		else if(nQStep_fold <= nQStep_dnd)
			{
			moveKey(a_nKey_fold, a_nKey_dnd);

			m_nOrder_dnd = 0;
			m_nKey_shift_up = -1;
			}
		else
			{
			double dVariance = 0;
			double dVariance_dnd_min = 0;
			moveKey(a_nKey_dnd_min, a_nKey_dnd);
			dVariance_dnd_min = getVariance(a_nKey_dnd, m_nOrder_dpcm);

			// Shift up:  κ  
			moveKey(a_nKey_dnd_min, a_nKey_shift_up);
			m_nKey_shift_up = getMax(a_nKey_shift_up);
			shiftUp(a_nKey_shift_up, m_nKey_shift_up);
			dVariance = getVariance(a_nKey_shift_up, m_nOrder_dpcm);
			if(sqrt(dVariance) < sqrt(dVariance_dnd_min)*2)
				moveKey(a_nKey_shift_up, a_nKey_dnd);
			else
				m_nKey_shift_up = -1;
			}

		// Moving: Ű Ѱֱ
		moveKey(a_nKey_dnd, m_a_nQKey);

		delete[] a_nKey_shift_up;
		delete[] a_nKey_dnd_min;
		delete[] a_nKey_dnd;
		delete[] a_nKey_fold;
		delete[] a_nKey_dpcm_min;
		delete[] a_nKey_dpcm;
		}

	// Moving: ȭ Ű Ѱֱ
	void getQKey(int*& a_nQKey)
		{
		a_nQKey = new int[m_nNumberOfKey];
		for(int i = 0; i < m_nNumberOfKey; i++)
			a_nQKey[i] = m_a_nQKey[i];
		}

	// Moving: Ű ְ  Ѱֱ
	double getKeyDistortion(void)
		{
		return m_dKeyDistortion;
		}

	// NBits: ʿ Ʈ ϱ
	int getNBits(int n)
		{
		return (int)(log10(n)/log10(2))+1;
		}

	// Range: Ű鿡  ִ, ּ   ϱ
	int getRange(int* a_nKey)
		{
		int	nMax;				// ִ 
		int	nMin;				// ּ 
		int nOrder_dpcm = m_nOrder_dpcm;

		nMax = getMax(a_nKey, nOrder_dpcm);
		nMin = getMin(a_nKey, nOrder_dpcm);

		return nMax-nMin;
		}

	// QStep: Ű  ʿ Ʈ ϱ
	int getQStep(int* a_nKey)
		{
		int nRange = getRange(a_nKey);

		return getNBits(nRange);
		}

	// QStep: Ű ڵ ʿ Ʈ ϱ
	int getQStep_key_encoding(void)
		{
		int nMax = 0;
		int nMin = 0;

		if(m_nKey_shift_up != -1 || m_nOrder_dnd < 1)
			{
			return getQStep(m_a_nQKey);
			}
		else
			{
			nMax = getMax(m_a_nQKey, m_nOrder_dpcm);
			nMin = getMin(m_a_nQKey, m_nOrder_dpcm);

			if(nMax > abs(nMin))
				return getNBits(nMax);
			else
				return getNBits(abs(nMin));
			}
		}

	// Moving: dpcm  Ѱֱ
	int getOrder_dpcm(void)
		{
		return m_nOrder_dpcm;
		}

	// Moving: dnd  Ѱֱ
	int getOrder_dnd(void)
		{
		return m_nOrder_dnd;
		}

	// Moving: shift  Ѱֱ
	int getKey_shift(void)
		{
		return m_nKey_shift;
		}

	// Moving: shift up  Ѱֱ
	int getKey_shift_up(void)
		{
		return m_nKey_shift_up;
		}

	// Moving: dnd ʿ Max  Ѱֱ
	void getKey_dnd_max(int*& a_nKey_dnd_max)
		{
		if(m_nOrder_dnd == -1)
		{
			// Jameskim 20020426
			a_nKey_dnd_max = NULL;
			//
			return;
		}
		a_nKey_dnd_max = new int[m_nOrder_dnd];
		for(int i = 0; i < m_nOrder_dnd; i++)
			a_nKey_dnd_max[i] = m_a_nKey_dnd_max[i];
		}

	// Moving: key shift ʿ Ʈ Ѱֱ
	int	getQStep_key_shift(void)
		{
		int nMax = 0;
		int i = 0;

		if(abs(m_nKey_shift) > nMax)
			nMax = abs(m_nKey_shift);
		for(i = 0; i < m_nOrder_dpcm; i++)
			{
			if(abs(m_a_nQKey[i]) > nMax)
				nMax = abs(m_a_nQKey[i]);
			}
		for(i = 0; i < m_nOrder_dnd; i++)
			{
			if(abs(m_a_nKey_dnd_max[i]) > nMax)
				nMax = abs(m_a_nKey_dnd_max[i]);
			}

		return getNBits(nMax);
		}

	// Dump: dump ȭ 
	void write(int* a_nKey, char* szFile, int nNum = -1)
		{
		if(nNum == -1)
			nNum = m_nNumberOfKey;

		ofstream fout(szFile);
		for(int i = 0; i < nNum; i++)
			fout << a_nKey[i] << endl;
		fout.close();
		}

	// Dump: dump ȭ 
	void write(double* a_dKey, char* szFile, int nNum = -1)
		{
		if(nNum == -1)
			nNum = m_nNumberOfKey;

		ofstream fout(szFile);
		for(int i = 0; i < nNum; i++)
			fout << a_dKey[i] << endl;
		fout.close();
		}

	// Dump: dump ȭ 
	void write(float* a_dKey, char* szFile, int nNum = -1)
		{
		if(nNum == -1)
			nNum = m_nNumberOfKey;

		ofstream fout(szFile);
		for(int i = 0; i < nNum; i++)
			fout << a_dKey[i] << endl;
		fout.close();
		}

	// Dump: dump ȭ 
	void write(int* a_nKey, int nNum, char* szFile)
		{
		write(a_nKey, szFile, nNum);
		}

	// Dump: dump ȭ 
	void write(double* a_dKey, int nNum, char* szFile)
		{
		write(a_dKey, szFile, nNum);
		}

	// Dump: dump ȭ 
	void write(float* a_dKey, int nNum, char* szFile)
		{
		write(a_dKey, szFile, nNum);
		}

private:
	// Quantization: ü Ű ȭϱ
	void quantize(int nQStep_key)
		{
		double dKey;
		int nQKey;
		double dIQKey;
		double dKeyDistortion = 0;

		for(int i = 0; i < m_nNumberOfKey; i++)
			{
			dKey = m_a_dKey[i];														// key
			nQKey = quantizeKey(dKey, m_dKey_max, m_dKey_min, nQStep_key);			// Quantize
			m_a_nQKey[i] = nQKey;

			dIQKey = inverseQuantizeKey(nQKey, m_dKey_max, m_dKey_min, nQStep_key);	// iQuantize
			dIQKey = roundOffWithDigit(dIQKey, m_nDigit_key);

			dKeyDistortion += fabs(dKey-dIQKey);
			}
		m_dKeyDistortion = (double)(dKeyDistortion / m_nNumberOfKey);
		}

	// Quantization: ϳ Ű ȭϱ
	int quantizeKey(double dKey, double dMax, double dMin, int nQStep)
		{
		int nQKey;

		if(dKey <= dMin)
			return 0;
		else if(dKey >= dMax)
			return ((1 << nQStep) - 1);

		nQKey = (int)(((dKey-dMin) / (dMax-dMin)) * ((1 << nQStep) - 1) + 0.5);

		return nQKey;
		}

	// InverseQuantization: ϳ Ű  ȭϱ
	double inverseQuantizeKey(int nQKey, double dMax, double dMin, int nQStep)
		{			
		double dIQKey;
		
		if(nQKey == 0)
			return dMin;
		else if(nQKey == ((1 << nQStep) - 1))
			return dMax;
		else
			dIQKey = (double)(dMin + ((double)nQKey / ((1 << nQStep) - 1)) * (dMax - dMin));
		
		return dIQKey;
		}

	// Moving: Ű Ѱֱ
	void moveKey(int* a_nKey_source, int* a_nKey_target)
		{
		for(int i = 0; i < m_nNumberOfKey; i++)
			a_nKey_target[i] = a_nKey_source[i];
		}

	// DPCM: Ű  ϱ.  ϸ n dpcm
	void dpcm(int* a_nKey_dpcm)
		{
		for(int i = m_nNumberOfKey-1; i > 0; i--)
			a_nKey_dpcm[i] = a_nKey_dpcm[i] - a_nKey_dpcm[i-1];
		}

	// Quartile Deviation:   ϱ
	double getQuartileDeviation(int* a_nKey_dpcm, int nOrder_dpcm)
		{
		double dMode = 0;
		double dLowerQuartile = 0;
		double dUpperQuartile = 0;
		double dQuartileDeviation = 0;
		int nNumberOfDeviation = m_nNumberOfKey-nOrder_dpcm;
		int nIndex_mode = 0;
		int nIndex_lower_quartile = 0;
		int nIndex_upper_quartile = 0;
		int* a_nKey = new int[nNumberOfDeviation];
		double* a_dDeviation = new double[nNumberOfDeviation];
		int i = 0;

		// Copying: ϱ
		for(i = nOrder_dpcm; i < m_nNumberOfKey; i++)
			a_nKey[i-nOrder_dpcm] = a_nKey_dpcm[i];

		// Mode: ֺ ϱ
		dMode = getFrequencyMax(a_nKey_dpcm);

		// Deviation:  ϱ
		for(i = 0; i < nNumberOfDeviation; i++)
			a_dDeviation[i] = a_nKey[i] - dMode;

		// Sorting: ϱ
		qsort((void*)a_dDeviation, (size_t)nNumberOfDeviation, sizeof(double), compare);

		// Quartile Deviation:   ϱ
		if(nNumberOfDeviation%2 == 0)
			{
			if((nNumberOfDeviation/2)%2 == 0)
				{
				dLowerQuartile = (a_dDeviation[nNumberOfDeviation/2/2-1] + a_dDeviation[nNumberOfDeviation/2/2])/2;
				dUpperQuartile = (a_dDeviation[nNumberOfDeviation/2+nNumberOfDeviation/2/2-1] + a_dDeviation[nNumberOfDeviation/2+nNumberOfDeviation/2/2])/2;
				}
			else
				{
				dLowerQuartile = a_dDeviation[nNumberOfDeviation/2/2];
				dUpperQuartile = a_dDeviation[nNumberOfDeviation/2+nNumberOfDeviation/2/2];
				}
			}
		else
			{
			if(((nNumberOfDeviation+1)/2)%2 == 0)
				{
				dLowerQuartile = (a_dDeviation[nNumberOfDeviation/2/2] + a_dDeviation[nNumberOfDeviation/2/2+1])/2;
				dUpperQuartile = (a_dDeviation[nNumberOfDeviation/2+nNumberOfDeviation/2/2] + a_dDeviation[nNumberOfDeviation/2+nNumberOfDeviation/2/2+1])/2;
				}
			else
				{
				dLowerQuartile = a_dDeviation[nNumberOfDeviation/2/2];
				dUpperQuartile = a_dDeviation[nNumberOfDeviation/2+nNumberOfDeviation/2/2];
				}
			}

		dQuartileDeviation = (dUpperQuartile - dLowerQuartile) / 2;

		delete[] a_dDeviation;
		delete[] a_nKey;

		return (double)dQuartileDeviation;
		}

	// Mean Deviation:   ϱ
	double getMeanDeviation(int* a_nKey_dpcm, int nOrder_dpcm)
		{
		double dSum = 0;
		double dMode = 0;
		double dMeanDeviation = 0;
		int* a_nKey = new int[m_nNumberOfKey];
		double* a_dDeviation = new double[m_nNumberOfKey];
		int nNumberOfDeviation = 0;
		int i = 0;

		// Copying: ϱ
		for(i = 0; i < m_nNumberOfKey; i++)
			a_nKey[i] = a_nKey_dpcm[i];

		// Mode: ֺ ϱ
		dMode = getFrequencyMax(a_nKey_dpcm);

		// Deviation:  ϱ
		for(i = nOrder_dpcm; i < m_nNumberOfKey; i++)
			a_dDeviation[i] = fabs(a_nKey[i] - dMode);

		// Mean Deviation:   ϱ
		dSum = 0;
		nNumberOfDeviation = 0;
		for(i = nOrder_dpcm; i < m_nNumberOfKey; i++, nNumberOfDeviation++)
			dSum += a_dDeviation[i];
		dMeanDeviation = dSum / nNumberOfDeviation;

		delete[] a_dDeviation;
		delete[] a_nKey;

		return (double)dMeanDeviation;
		}

	// Variance: л  ϱ
	double getVariance(int* a_nKey_dpcm, int nOrder_dpcm, double dVar_Max = 1.0)
		{
		double dSum = 0;
		double dCenter = 0;
		double dVariance = 0;
		int* a_nKey = new int[m_nNumberOfKey];
		double* a_dDeviation = new double[m_nNumberOfKey];
		int nNumberOfDeviation = 0;
		int i = 0;

		// Copying: ϱ
		for(i = 0; i < nOrder_dpcm; i++)
			a_nKey[i] = 0;
		for(i = nOrder_dpcm; i < m_nNumberOfKey; i++)
			a_nKey[i] = a_nKey_dpcm[i];

		// Center: ߽ɰ ϱ
		dCenter = getFrequencyMax(a_nKey_dpcm);

		// Deviation:  ϱ
		for(i = 0; i < nOrder_dpcm; i++)
			a_dDeviation[i] = 0;
		for(i = nOrder_dpcm; i < m_nNumberOfKey; i++)
			a_dDeviation[i] = fabs(a_nKey[i] - dCenter);

		// Sorting: ϱ
		qsort((void*)a_dDeviation, (size_t)m_nNumberOfKey, sizeof(double), compare);

		// Variance: л ϱ
		dSum = 0;
		nNumberOfDeviation = 0;
		for(i = nOrder_dpcm; i < m_nNumberOfKey*dVar_Max; i++, nNumberOfDeviation++)
			dSum += a_dDeviation[i] * a_dDeviation[i];
		dVariance = dSum / nNumberOfDeviation;

		delete[] a_dDeviation;
		delete[] a_nKey;

		return (double)dVariance;
		}

	// Max: ִ  ϱ
	int getMax(int* a_nKey, int nOrder_dpcm = -1)
		{
		int nMax = INT_MIN;

		if(nOrder_dpcm == -1)
			nOrder_dpcm = m_nOrder_dpcm;
		for(int i = nOrder_dpcm; i < m_nNumberOfKey; i++)
			{
			if(a_nKey[i] > nMax)
				nMax = a_nKey[i];
			}
		return nMax;
		}

	// Min: ּ  ϱ
	int getMin(int* a_nKey, int nOrder_dpcm = -1)
		{
		int nMin = INT_MAX;

		if(nOrder_dpcm == -1)
			nOrder_dpcm = m_nOrder_dpcm;
		for(int i = nOrder_dpcm; i < m_nNumberOfKey; i++)
			{
			if(a_nKey[i] < nMin)
				nMin = a_nKey[i];
			}
		return nMin;
		}

	// Frequency Max: 󵵼 ִ  ϱ
	int getFrequencyMax(int* a_nKey, int nOrder_dpcm = -1)
		{
		int nMax = 0;
		int nMin = 0;
		int i = 0;

		if(nOrder_dpcm == -1)
			nOrder_dpcm = m_nOrder_dpcm;

		nMax = getMax(a_nKey, nOrder_dpcm);
		nMin = getMin(a_nKey, nOrder_dpcm);

		// Histogram: ׷ ʱȭ
		int(* a_nKey_histogram)[2] = new int[nMax - nMin + 1][2];
		for(i = 0; i < nMax - nMin + 1; i++)
			{
			a_nKey_histogram[i][0] = 0;
			a_nKey_histogram[i][1] = 0;
			}
		
		// Histogram: ׷ ϱ
		for(i = nOrder_dpcm; i < m_nNumberOfKey; i++)
			{
			int nIndex = a_nKey[i]-nMin;
			a_nKey_histogram[nIndex][0] = a_nKey[i];
			a_nKey_histogram[nIndex][1]++;
			}
		
		// Histogram: ִ 󵵼  ϱ
		int nKey_frequency_max = 0;
		int nIndex_frequency_max = 0;
		for(i = 0; i < nMax - nMin + 1; i++)
			{
			if(nKey_frequency_max < a_nKey_histogram[i][1])
				{
				nKey_frequency_max = a_nKey_histogram[i][1];
				nIndex_frequency_max = i;
				}
			}
		// Moving: ִ 󵵼  Ѱֱ
		nKey_frequency_max = a_nKey_histogram[nIndex_frequency_max][0];

		delete[] a_nKey_histogram;
		return nKey_frequency_max;
		}

	// Shift: Ű shiftϱ
	void shift(int* a_nKey, int nKey_shift, int nOrder_dpcm = -1)
		{
		if(nOrder_dpcm == -1)
			nOrder_dpcm = m_nOrder_dpcm;

		for(int i = nOrder_dpcm; i < m_nNumberOfKey; i++)
			a_nKey[i] -= nKey_shift;
		}

	// Fold: Ű foldingϱ
	void fold(int* a_nKey, int nOrder_dpcm = -1)
		{
		if(nOrder_dpcm == -1)
			nOrder_dpcm = m_nOrder_dpcm;

		for(int i = nOrder_dpcm; i < m_nNumberOfKey; i++)
			{ // f(n)=2*ABS(n)-INT((1-SIGN(n))/2)
			int nKey = 2*abs(a_nKey[i]) - int((1-sign(a_nKey[i]))/2);
			a_nKey[i] = nKey;
			}
		}

	// Divide: Ű dividingϱ
	void divide(int* a_nKey, int nKey_max, int nOrder_dpcm = -1)
		{
		if(nOrder_dpcm == -1)
			nOrder_dpcm = m_nOrder_dpcm;

		for(int i = nOrder_dpcm; i < m_nNumberOfKey; i++)
			{ // d(n)=n-INT(n/(nKey_max/2+1))*(nKey_max+1)
			int nKey = a_nKey[i] - int(a_nKey[i] / (nKey_max/2+1)) * (nKey_max+1);
			a_nKey[i] = nKey;
			}
		}

	// Divide up: Ű dividing upϱ
	void divide_up(int* a_nKey, int nKey_min, int nOrder_dpcm = -1)
		{
		if(nOrder_dpcm == -1)
			nOrder_dpcm = m_nOrder_dpcm;

		for(int i = nOrder_dpcm; i < m_nNumberOfKey; i++)
			{ // d_up(n)=IF(n>0,n,IF(n>=nKey_min/2,n*2,(nKey_min-n-1)*2+1)
			int nKey = a_nKey[i];
			if(nKey < 0)
				{
				if(nKey < nKey_min/2)
					nKey = (nKey_min-nKey-1)*2+1;
				else
					nKey *= 2;
				}
			a_nKey[i] = nKey;
			}
		}

	// Divide down: Ű dividing downϱ
	void divide_down(int* a_nKey, int nKey_max, int nOrder_dpcm = -1)
		{
		if(nOrder_dpcm == -1)
			nOrder_dpcm = m_nOrder_dpcm;

		for(int i = nOrder_dpcm; i < m_nNumberOfKey; i++)
			{ // d_down(n)=IF(n<0,n*2,IF(n<=nKey_max/2,n,(nKey_max-n+1)*-2+1))
			int nKey = a_nKey[i];
			if(nKey < 0)
				nKey *= 2;
			else if(nKey > nKey_max/2)
				nKey = (nKey_max-nKey+1)*(-2)+1;
			a_nKey[i] = nKey;
			}
		}

	// Shift up: Ű shift upϱ.  κ  .
	void shiftUp(int* a_nKey, int nKey_max, int nOrder_dpcm = -1)
		{
		if(nOrder_dpcm == -1)
			nOrder_dpcm = m_nOrder_dpcm;

		for(int i = nOrder_dpcm; i < m_nNumberOfKey; i++)
			{ // d(n)=n-INT(n/(nKey_max/2+1))*(nKey_max+1)
			int nKey = a_nKey[i];
			if(nKey < 0)
				a_nKey[i] = nKey_max - nKey;
			}
		}

	// Round Off: Digit Ͽ ݿøϱ
	double roundOffWithDigit(double dSource, int nDigit)
		{
		int nDecimal = 0;
		int nSign = 0;
		char* szBuffer = NULL;
		double dValue = 0;
		
		szBuffer = _fcvt(dSource, 10, &nDecimal, &nSign);

		if(nSign == 0)
			dValue = (double)int(dSource*pow(10, nDigit-nDecimal)+0.5)/pow(10, nDigit-nDecimal);
		else
			dValue = (double)int(dSource*pow(10, nDigit-nDecimal)-0.5)/pow(10, nDigit-nDecimal);
		//cout << dSource << " -> " << dValue << endl;

		return dValue;
		}
	};
