// CompressedOrientationInterpolator.cpp: implementation of the CompressedOrientationInterpolator class.
//
//////////////////////////////////////////////////////////////////////

#include "CompressedOrientationInterpolator.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CompressedOrientationInterpolator::CompressedOrientationInterpolator()
{

}

CompressedOrientationInterpolator::~CompressedOrientationInterpolator()
{

}

// Global Fucntions
void ParseIM1Text(JWToken& token, JWInterpolator*& pInterpolator);




extern char *gAAC, *gLinearKeyCoder, *gPreservingMode, *gKeyQuant, *gDPCM;


#define AACTotal() (codec._ostrm->Outbuf.nbytes) * 8 + (codec._ostrm->Outbuf.nbits)

int test_bit;








void CompressedOrientationInterpolator::encode(
				char* orgFileName, char* quantFileName, char* aacFileName, char* errorFileName,
				bool bPreserveKey, bool IsLinearKeyCoder, 
				bool bIsUnaryAAC_X, bool bIsUnaryAAC_Y, bool bIsUnaryAAC_Z,
				bool bKeyQuant, char chDPCM,
				JWArray<double> adThresholds, JWArray<int>anKQBits, JWArray<int> anKVQBits)
{
	int								i = 0;
	int								nT = 0;
	int								nTotalKeyBit = 0;
	int								nOrgNumberOfKey = 0;
	JWArray<CSelectedKeyFrame>		aKeyFrames;
	char							fName[256], fQName[256], fDEC[256], fTMP[256];


	//Encoding Test 
	FILE *fReport;
	char buffer[256];

	if(gLinearKeyCoder[0] == 'l')
	{
		if(gPreservingMode[0] == 'k')
		{
			if(gAAC[0] == 'u')
				sprintf(buffer, "LinearKeyUnaryAAC.txt");
			else
				sprintf(buffer, "LinearKeyBinaryAAC.txt");
		}
		else
		{
			if(gAAC[0] == 'u')
				sprintf(buffer, "LinearPathUnaryAAC.txt");
			else
				sprintf(buffer, "LinearPathBinaryAAC.txt");
		}
	}
	else
	{
		if(gPreservingMode[0] == 'k')
		{
			if(gAAC[0] == 'u')
				sprintf(buffer, "KeyUnaryAAC.txt");
			else
				sprintf(buffer, "KeyBinaryAAC.txt");
		}
		else
		{
			if(gAAC[0] == 'u')
				sprintf(buffer, "PathUnaryAAC.txt");
			else
				sprintf(buffer, "PathBinaryAAC.txt");
		}
	}

	fReport = fopen(buffer, "w");


	
	// Initialize aKeyFrames
	aKeyFrames.SetSize(adThresholds.GetSize());
	for(i=0; i<adThresholds.GetSize(); i++)
		aKeyFrames[i].Initialize();
	


	// ---------------------------------------------------------
	//
	// Read original source file
	//
	// ---------------------------------------------------------
	JWToken token;
	if(!token.SetFile(orgFileName))
	{
		cout << "Cannot open file " << endl;
		cout << "File Name: " << orgFileName << endl;

		return;
	}
	JWInterpolator* pInterpolator = NULL;
	ParseIM1Text(token, pInterpolator);

	

	// ---------------------------------------------------------
	//
	// Set preliminary information
	//
	// ---------------------------------------------------------
	
	// set # of keys
	nOrgNumberOfKey = pInterpolator->GetKey().GetSize();
	// set key
	dKey = pInterpolator->GetKey();
	// set key value
	dKeyValue = *((JWArray<SFRotation>*)pInterpolator->GetKeyValues());
	// set key value quat
	dKeyValueQuat.SetSize(dKeyValue.GetSize());
	for(i = 0;i < dKeyValue.GetSize();i++)
		dKeyValueQuat[i].RotationToQuaternion(&dKeyValue[i]);



	// ---------------------------------------------------------
	//
	// key value selection
	//
	// ---------------------------------------------------------
	downsampling_with_QC(dKeyValueQuat, dKey, dKey.GetSize(), adThresholds, aKeyFrames);

	for(nT=0; nT<adThresholds.GetSize(); nT++)
	{
		aKeyFrames[nT].SetThreshold(adThresholds[nT]);

		JWArray<double> m_adSelectedKeys;
		JWArray<Quaternion> m_aQuatSelectedQuat;
		JWArray<SFRotation> m_aRotSelectedRot;
		JWArray<JWArray<int> > m_anKeyFlags;
		JWArray<int> m_anNotInterpolatedIndex;
		int nSelectedKey;

		m_adSelectedKeys.SetSize(nOrgNumberOfKey);
		m_aQuatSelectedQuat.SetSize(nOrgNumberOfKey);
		m_aRotSelectedRot.SetSize(nOrgNumberOfKey);
		m_anNotInterpolatedIndex.SetSize(nOrgNumberOfKey);

		select_key_and_kv(
			dKey,			m_adSelectedKeys,			// input, output
			dKeyValueQuat,	m_aQuatSelectedQuat,		// input, output
			dKeyValue,		m_aRotSelectedRot,			// input, output
			nSelectedKey,	aKeyFrames[nT].GetKeyFlag(), nOrgNumberOfKey
		);

		// jameskim 20010106
		printf(
			"[Threshold : %g] key selection complete : %d -> %d\n", 
			adThresholds[nT], nOrgNumberOfKey, nSelectedKey
		);

		for(i=0; i<nSelectedKey; i++)
			m_anNotInterpolatedIndex[i] = 1;

		aKeyFrames[nT].SetOriginalKeys(dKey);
		aKeyFrames[nT].SetSelectedKeys(m_adSelectedKeys, nSelectedKey);
		aKeyFrames[nT].SetOriginalQuaternions(dKeyValueQuat);
		aKeyFrames[nT].SetSelectedQuaternions(m_aQuatSelectedQuat);
		aKeyFrames[nT].SetOriginalRotation(dKeyValue);
		aKeyFrames[nT].SetSelectedRotation(m_aRotSelectedRot);
		aKeyFrames[nT].SetNotInterpolatoredIndex(m_anNotInterpolatedIndex);

	}


	// ---------------------------------------------------------
	//
	// get file extension
	//
	// ---------------------------------------------------------
	int		ch = '.';
	char	*pdest, *pdest1;
	char	sTmp[256], sTmp1[256];

	// µǴ AAC file Ȯڸ ´.
	pdest = strrchr( aacFileName, ch );
	if(pdest != NULL)
	{
		strcpy(sTmp, pdest);
		pdest[0] = NULL;
	}
	// µǴ Quantized file Ȯڸ ´.
	pdest1 = strrchr( quantFileName, ch );
	if(pdest1 != NULL)
	{
		strcpy(sTmp1, pdest1);
		pdest1[0] = NULL;
	}




	// ---------------------------------------------------------
	//
	// Encoding Loop
	//
	// ---------------------------------------------------------
	// key loop
	for(int nK=0; nK < anKQBits.GetSize(); nK++)
	{

		// kHeader.nKeyQBit = anKQBits[nK];
		int nKeyQBit = anKQBits[nK];
		
		double preNumKey = -1.0;
		// ---------------------------------------------------------
		// threshold loop
		for(int nT=0; nT < adThresholds.GetSize(); nT++)
		{
			double fThreshold = adThresholds[nT];
			double currentNumKey = aKeyFrames[nT].GetNumberOfSelectedKeys();
			
			// ̿ϴ threshold key    ٽ encoding ʴ´.
			if(currentNumKey == preNumKey)
			{
				preNumKey = currentNumKey;
			}
			else
			{

				preNumKey = currentNumKey;

				// ---------------------------------------------------------
				// key value loop
				for(int nKV=0; nKV < anKVQBits.GetSize(); nKV++)
				{
					// test
					printf("Threshold %f\n", fThreshold);
					//
					int nKVQBit = anKVQBits[nKV];
					//@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
					// jameskim 20010925
					//  threshold  DPCM Order 
					// δ õǴ key  ü  %ķ   ʿ䰡 ִ.
					int nDPCMOrder;
					switch(chDPCM) {
						case '1':
							nDPCMOrder = 0;
							break;
						case '2':
							nDPCMOrder = 1;
							break;
						case 's':
						default:
							if(fThreshold <= 0.005)
								nDPCMOrder = 1;
							else
								nDPCMOrder = 0;
							break;
					}
					//@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@		


					sprintf(fName, "%s_%02d_%02d_%g_%s_%s_%s_%s_%s.aac",
						aacFileName,
						nKeyQBit, nKVQBit, fThreshold,
						gAAC, gLinearKeyCoder, gPreservingMode,	gKeyQuant, gDPCM);

					sprintf(fQName, "%s_%g-%d-%d%s", 
						aacFileName, 
						fThreshold, nKeyQBit, nKVQBit, sTmp1);
					
					sprintf(fDEC, "%s_%g-%d-%d%s", 
						aacFileName, 
						fThreshold, nKeyQBit, nKVQBit, ".DEC");

					sprintf(fTMP, "%s_%02d_%02d_%g_%s_%s_%s_%s_%s.TMP",
						aacFileName,
						nKeyQBit, nKVQBit, fThreshold,
						gAAC, gLinearKeyCoder, gPreservingMode,	gKeyQuant, gDPCM);


					//printf("point 1\n");


					// ---------------------------------------------------------
					//
					// AAC initialize 
					//
					// ---------------------------------------------------------
					CAC::_shift = 0; 
					COutStream ostrm;
					CAC_QF codec(ostrm);
					bool success = ostrm.openenc(fName);

					if(ksFlag.keyFlag.GetSize() != 0)
						ksFlag.keyFlag.RemoveAll();

					if(oriIDPCMKeyValue.decodedKeyValue.GetSize() != 0)
						oriIDPCMKeyValue.decodedKeyValue.RemoveAll();
					if(oriIDPCMKeyValue.DPCMKeyValue.GetSize() != 0)
						oriIDPCMKeyValue.DPCMKeyValue.RemoveAll();
					if(oriIDPCMKeyValue.ReDPCMQKeyValue.GetSize() != 0)
						oriIDPCMKeyValue.ReDPCMQKeyValue.RemoveAll();
					if(oriIDPCMKeyValue.DeltaKeyValue.GetSize() != 0)
						oriIDPCMKeyValue.DeltaKeyValue.RemoveAll();

					if(k.dKey != NULL)
						delete [] k.dKey;
					if(k.nQKey != NULL)
						delete [] k.nQKey;
					if(kHeader.nQIntraKeySign != NULL)
						delete [] kHeader.nQIntraKeySign;
					if(kHeader.nQIntraKey != NULL)
						delete [] kHeader.nQIntraKey;
					
					if(kHeader.nKeyMaxSign != NULL)
						delete [] kHeader.nKeyMaxSign;
					if(kHeader.nKeyMax != NULL)
						delete [] kHeader.nKeyMax;

					// ---------------------------------------------------------
					//
					// Encoding Process
					//
					// ---------------------------------------------------------
					ksFlag.encodingProcess(aKeyFrames[nT]);
					kHeader.encodingProcess(bPreserveKey, nKeyQBit, aKeyFrames[nT]);
					k.encodingProcess(bPreserveKey, IsLinearKeyCoder, kHeader, aKeyFrames[nT], gKeyQuant);
					oriIKVHeader.encodingProcess(bPreserveKey, nDPCMOrder, nKVQBit);
					oriIDPCMKeyValue.encodingProcess(nDPCMOrder, nKVQBit, aKeyFrames[nT]);

					// test
					FILE *ftest;
					ftest = fopen("key.txt", "w");
					for(int xxx = 0;xxx < k.nNumberOfRemainingKey;xxx++)
						fprintf(ftest, "%g\n", k.dKey[xxx]);
					fclose(ftest);
					
					
					ftest = fopen("dpcm.txt", "w");
					for(xxx = 0;xxx < oriIDPCMKeyValue.DPCMKeyValue.GetSize();xxx++)
						fprintf(ftest, "%g\t%g\t%g\t%g\n", 
							oriIDPCMKeyValue.DPCMKeyValue[xxx].s,
							oriIDPCMKeyValue.DPCMKeyValue[xxx].x,
							oriIDPCMKeyValue.DPCMKeyValue[xxx].y,
							oriIDPCMKeyValue.DPCMKeyValue[xxx].z);
					fclose(ftest);

					ftest = fopen("redpcm.txt", "w");
					for(xxx = 0;xxx < oriIDPCMKeyValue.ReDPCMQKeyValue.GetSize();xxx++)
						fprintf(ftest, "%g\t%g\t%g\t%g\n", 
							oriIDPCMKeyValue.ReDPCMQKeyValue[xxx].s,
							oriIDPCMKeyValue.ReDPCMQKeyValue[xxx].x,
							oriIDPCMKeyValue.ReDPCMQKeyValue[xxx].y,
							oriIDPCMKeyValue.ReDPCMQKeyValue[xxx].z);
					fclose(ftest);


					// ---------------------------------------------------------
					//
					// Write Header information
					//
					// ---------------------------------------------------------
					//
					test_bit = AACTotal();
					kHeader.makeBitstream(codec, nTotalKeyBit);
					
					//
					test_bit = AACTotal() - test_bit;
					printf("KEY HEADER BIT %d\n", test_bit);
					test_bit = AACTotal();
					//

					oriIKVHeader.makeBitstream(codec, 
												bIsUnaryAAC_X, bIsUnaryAAC_Y, bIsUnaryAAC_Z,
												oriIDPCMKeyValue);
					//
					test_bit = AACTotal() - test_bit;
					printf("KV HEADER BIT %d\n", test_bit);
					test_bit = AACTotal();
					//



					// ---------------------------------------------------------
					//
					// AAC encoding start
					//
					// ---------------------------------------------------------
					// byte align
					if(codec._ostrm->Outbuf.nbits != 0)
					{
						codec._ostrm->putbits(1, 8 - codec._ostrm->Outbuf.nbits);
					}

					// qf_start();
					// encoder start();
					codec.start_encode();

					
					// ---------------------------------------------------------
					//
					// AAC encoding context initialize
					//
					// ---------------------------------------------------------
					// key context initialize
					k.InitializeModel();
					// keyflag context initialize
					ksFlag.InitializeModel();
					// keyvalue context initialize
					oriIDPCMKeyValue.InitializeModel();



					// ---------------------------------------------------------
					// 
					// Write AAC encoded bitstream
					// 
					// ---------------------------------------------------------

					ftest = fopen("encode key flag.txt", "w");
					for(xxx = 0;xxx < ksFlag.keyFlag.GetSize();xxx++)
						fprintf(ftest, "%d\n", ksFlag.keyFlag[xxx] );
					fclose(ftest);






					ksFlag.makeBitstream(codec, bPreserveKey);

					//
					test_bit = AACTotal() - test_bit;
					printf("KEY FLAG BIT %d\n", test_bit);
					test_bit = AACTotal();
					//


					ftest = fopen("encode key .txt", "w");
					for(xxx = kHeader.nKDPCMOrder;xxx < k.nNumberOfRemainingKey;xxx++)
						fprintf(ftest, "%d\n", k.nQKey[xxx] );
					fclose(ftest);




					k.makeBitstream(codec, kHeader, nTotalKeyBit);

					//
					test_bit = AACTotal() - test_bit;
					printf("KEY BIT %d\n", test_bit);
					test_bit = AACTotal();
					//



					ftest = fopen("AAC.txt", "w");
					for(xxx = 0;xxx < oriIDPCMKeyValue.DeltaKeyValue.GetSize();xxx++)
						fprintf(ftest, "%g\t%g\t%g\t%g\n", 
							oriIDPCMKeyValue.DeltaKeyValue[xxx].s,
							oriIDPCMKeyValue.DeltaKeyValue[xxx].x,
							oriIDPCMKeyValue.DeltaKeyValue[xxx].y,
							oriIDPCMKeyValue.DeltaKeyValue[xxx].z);
					fclose(ftest);



					oriIDPCMKeyValue.makeBitstream(codec, 
											oriIKVHeader.oriIDPCMKVHeader.x_keyvalue_flag,
											oriIKVHeader.oriIDPCMKVHeader.y_keyvalue_flag,
											oriIKVHeader.oriIDPCMKVHeader.z_keyvalue_flag,
											oriIKVHeader.oriIDPCMKVHeader.numBitsKV,
											bIsUnaryAAC_X, bIsUnaryAAC_Y, bIsUnaryAAC_Z,
											nDPCMOrder);


					
					//
					test_bit = AACTotal() - test_bit;
					printf("KEYVLAUE BIT %d\n", test_bit);
					test_bit = AACTotal();
					//

					// ---------------------------------------------------------
					//
					// AAC encoding finish
					//
					// ---------------------------------------------------------
					// encoder finish();
					codec.finish_encode();
					if(codec._ostrm->Outbuf.nbits != 0)
					{
						codec._ostrm->putbits(1, 8 - codec._ostrm->Outbuf.nbits);
					}

					nTotalKeyBit = 0;
					// ---------------------------------------------------------
					//
					//
					// Statistics
					//
					//
					// ---------------------------------------------------------
					
					
					// 带 ϴ Original key, keyvalue  bit.
					// key SFFloat, key value MFRotation(4 components)
					int nAsciiTotalBits = (int)((double)aKeyFrames[nT].GetNumberOfOriginalKeys()*32.0*5.0); 

					// 带 ϴ Original key, keyvalue ȭ  bit.
					// MPEG-4(14496-1 Ver 1.0) OrientationInterpolator ȭ  
					int qTotalBits = nKVQBit*aKeyFrames[nT].GetNumberOfOriginalKeys()*3 
									+ 2*aKeyFrames[nT].GetNumberOfOriginalKeys() 
									+ nKeyQBit*aKeyFrames[nT].GetNumberOfOriginalKeys();

					// 带 ϴ Original key, keyvalue ȣȭ  bit.
					int aacTotalBits = (codec._ostrm->Outbuf.nbytes) * 8 + (codec._ostrm->Outbuf.nbits);

					// Original key, keyvalue  bit  ȣȭ  bit 
					double nAsciiCompRatio = 100.0*(double)aacTotalBits/(double)nAsciiTotalBits;				
					
					// Original key, keyvalue ȭ  bit  ȣȭ  bit 
					double compressionRatio = 100.0*(double)aacTotalBits/(double)qTotalBits;

					printf("Original Total Number of Keys: %d, Selected Total Number Of Keys %d\nAcsiiTotalBits : %d, QuantizationTotalBits %d\nAACTotalBits : %d \n",
						aKeyFrames[nT].GetNumberOfOriginalKeys(),
						aKeyFrames[nT].GetNumberOfSelectedKeys(), 
						nAsciiTotalBits, 
						qTotalBits, 
						aacTotalBits);

					ostrm.close_enc();
					ostrm.bfree();

					// sowoo 2001.10.10 (Batch Job)
					// .TMP  ϱ
					ofstream fileTemp;
					char bufferTemp[1000];
					fileTemp.open(fTMP);

					//sprintf(bufferTemp, "%g\t%d\t%d\t%d\t%d\t%d\t%d\t%g\t%g\t%g\t%g\n",
					//			fThreshold, nKeyQBits, m, dNumEncodedKey, nAsciiTotalBits,
					//			qTotalBits, aacTotalBits,
					//			rmsError, gPeakError,
					//			nAsciiCompRatio, compressionRatio);

					fprintf(fReport, "%s\t%02d\t%02d\t%d\t%g\t%s\t%s\t%s\t%s\t%s\n",
						aacFileName,
						nKeyQBit, nKVQBit,
						aacTotalBits,
						fThreshold,
						gAAC, gLinearKeyCoder, gPreservingMode,
						gKeyQuant, gDPCM
					);

					sprintf(bufferTemp, "%s\t%02d\t%02d\t%d\t%g\t%s\t%s\t%s\t%s\t%s",
						aacFileName,
						nKeyQBit, nKVQBit,
						aacTotalBits,
						fThreshold,
						gAAC, gLinearKeyCoder, gPreservingMode,
						gKeyQuant, gDPCM
					);


					fileTemp << bufferTemp << endl;
					fileTemp.close();
					// end (Batch Job)

				} // end of key value loop
			} // end of else
		} // end of threshold loop
	} // end of key loop


	// test
	fclose(fReport);

	// Memory Free

	//
	printf("Encoding Complete\n");

}






double CompressedOrientationInterpolator::downsampling_with_QC(
	JWArray<Quaternion>& kvn,
	JWArray<double>& key,
	int key_count, 
	JWArray<double>& adThresholds,
	JWArray<CSelectedKeyFrame>& aKeyFrames
)
{
	int i;
	JWArray<Quaternion> kvi;
	QuatError Error;

	kvi.SetSize(key_count);

	//double abs_error = max-min;
	double distortion, rms, peak, duration;
	//uble distortion_threshold;

	//stortion_threshold = ::ori_distortion_threshold;

	//select two points
	// Quaternion error measure  Ʈ õǸ ̿   Ʈ  õ ǹѴ.
	// sowoo 2001.07.10 (FASTFASTFAST)
	JWArray<int> keyFlag;
	keyFlag.SetSize(key_count);
	for(i=0;i<key_count;i++)
		keyFlag[i] = 0;

	keyFlag[0] = keyFlag[key_count-1] = 1;

	//interpolate the intermediate points
	//interpolate(kvn, kvi, flag);
	//   interpolationѴ.
	interpolate_with_QC(kvn, kvi, keyFlag, key, key_count);

	//calculate difference
	//ù° distortion Ѵ.
	//distortion=compute_distortion(kvn, kvi)*abs_error;
	//distortion = compute_distortion_with_QC(kvn, kvi, key_count);//*abs_error;
	//Error.ComputeError(rms, peak, duration, key_count, key, kvn, key_count, key, kvi);
	Error.ComputeError(rms, peak, duration, key_count, key, kvn, key_count, key, kvi, 1);  // HELLO! //

	rms /= duration;
	distortion = sqrt(rms);
	
	//while loop: if difference > threshold
	int one_flag_count=2;
	int kim_one_flag_count=2;
	
	// total distorion threshold    .


	// sowoo 2001.07.10 (FASTFASTFAST)
	int nT = 0;
	//while (distortion > distortion_threshold)
	while (distortion > 0.0)
	{
		//double error=abs_error;
		double error=DBL_MAX;
		int min_point=-1;
		for(i=0;i<key_count;i++)
		{
			if (keyFlag[i] == 0) 
			{
				keyFlag[i] = 1;
				
				
				interpolate_with_QC(kvn, kvi, keyFlag, key, key_count);

				/*
				if(one_flag_count == 128)
				{
					for(int xxx = 0;xxx < key_count;xxx++)
						if((kvn[xxx].s != kvi[xxx].s) ||
							(kvn[xxx].x != kvi[xxx].x) ||
							(kvn[xxx].y != kvi[xxx].y) ||
							(kvn[xxx].z != kvi[xxx].z))
						{
							printf("ٸ!!\n");
						}
				}
				*/

				//distortion = ::compute_distortion_with_QC(kvn, kvi, key_count);//*abs_error;
				//Error.ComputeError(rms, peak, duration, key_count, key, kvn, key_count, key, kvi);
				Error.ComputeError(rms, peak, duration, key_count, key, kvn, key_count, key, kvi, 1);  // HELLO! //
				rms /= duration;
				distortion = sqrt(rms);

				if(distortion<error)
				{
					error=distortion;
					min_point=i;
				}

				keyFlag[i] = 0;
			}
		}
		
		//  õ   flag setting....
		keyFlag[min_point] = 1;
		one_flag_count++;
		kim_one_flag_count++;
		//test++;
		distortion=error;
		//printf("distortion : %g \n", error);
		//if(one_flag_count == 129) getchar();

		if(adThresholds.GetSize() <= nT)
			break;
		/////////////////////////////////////////////////////////////////////////
		/////////////////////////////////////////////////////////////////////////
		//  threshold    Key  Ѵ.
		while(distortion <= adThresholds[nT])
		{
			// 1.  Key_Flag !!!
			// keyFlag ִٸ, key ã  ִ?
			// ------------------> Y/E/S/
			aKeyFrames[nT].SetKeyFlag(keyFlag);
			nT++;
			if(adThresholds.GetSize() <= nT)
				break;
		}
		/////////////////////////////////////////////////////////////////////////
		/////////////////////////////////////////////////////////////////////////
	}

	if(one_flag_count == 2)
	{
		for(int i=0; i<adThresholds.GetSize(); i++)
		{
			aKeyFrames[nT].SetKeyFlag(keyFlag);
			nT++;
		}
	}
	else
	{
		if(adThresholds.GetSize() > nT)
			aKeyFrames[nT].SetKeyFlag(keyFlag);
	}
	//test = 0;


	return distortion;
}


//jameskim 20010106
void CompressedOrientationInterpolator::interpolate_with_QC(
	JWArray<Quaternion>& kvn,
	JWArray<Quaternion>& kvi,
	JWArray<int>& keyFlag,
	JWArray<double>& key,
	int key_count
)

{
	int i;
	//double left_end, right_end, left_end_key, right_end_key;
	Quaternion left_end, right_end;
	double left_end_key, right_end_key;
	double * keyi;

	// Jameskim 20020426
	double TargetQuaternion[4], LowerQuaternion[4], UpperQuaternion[4];
	double TargetTime; 
	double LowerTime; 
	double UpperTime;
	QuatError SLERP;
	//
	
	keyi= new double[key_count];

	//reverse fill of the right-end point
	left_end =kvn[0];
	left_end_key= key[0];
	right_end=kvn[key_count-1];
	right_end_key=key[key_count-1];

	for (i=key_count-1;i>=0;i--)
	{
		if (keyFlag[i] == 1) 
		{
			right_end=kvn[i];
			right_end_key=key[i];
		}
		kvi[i]=right_end;
		keyi[i]=right_end_key;
	}

	//For loop
	for(i=0;i<key_count;i++)
	{
		if (keyFlag[i] ==1)
		{
			left_end=kvn[i];
			left_end_key=key[i];
		}
		else // to be interpolated.
		{
			right_end=kvi[i];
			right_end_key=keyi[i];
			
			//left_end.Slerp(&right_end, ((key[i]-left_end_key)/(right_end_key-left_end_key)), &kvi[i]);
			// Jameskim 20020426
			// ѹ QuatError ִ SLERP ٲپ.
			LowerQuaternion[0] = left_end.s;LowerQuaternion[1] = left_end.x;
			LowerQuaternion[2] = left_end.y;LowerQuaternion[3] = left_end.z;
			UpperQuaternion[0] = right_end.s;UpperQuaternion[1] = right_end.x;
			UpperQuaternion[2] = right_end.y;UpperQuaternion[3] = right_end.z;
			LowerTime = left_end_key;
			UpperTime = right_end_key;
			TargetTime = key[i];
			SLERP.SphericalLinearInterpolator(TargetQuaternion, 
												LowerQuaternion, 
												UpperQuaternion, 
												TargetTime, 
												LowerTime, 
												UpperTime);
			kvi[i].s = TargetQuaternion[0];kvi[i].x = TargetQuaternion[1];
			kvi[i].y = TargetQuaternion[2];kvi[i].z = TargetQuaternion[3]; 
		}
	}

	delete [] keyi;
}


void CompressedOrientationInterpolator::select_key_and_kv (
	JWArray<double>& key,
	JWArray<double>& q_key,
	JWArray<Quaternion>& keyValue,
	JWArray<Quaternion>& q_keyValue,
	JWArray<SFRotation>& InRot,
	JWArray<SFRotation>& q_InRot,
	int &n_q_key,
	JWArray<int>& keyFlag,
	int key_count
)
{
	int i,j;

	j=0;
	for(i=0;i<key_count;i++)
	{
		if(keyFlag[i]) 
		{			
			q_key[j] = key[i];
			q_keyValue[j] = keyValue[i];
			q_InRot[j] = InRot[i];

			j++;
		}
	}
	n_q_key=j;
}