// JWOrientationInterpolator.cpp: implementation of the JWOrientationInterpolator class.
//
//////////////////////////////////////////////////////////////////////
#include <math.h>
#include <float.h>
#include <stdlib.h>

#include "JWOrientationInterpolator.h"
#include "JWToken.h"
//#include "SFRotation.h"
#include "Quaternion.h"
#include "QuatError.h"



//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

JWOrientationInterpolator::JWOrientationInterpolator()
{

}

JWOrientationInterpolator::~JWOrientationInterpolator()
{

}


void JWOrientationInterpolator::SetSize(int size)
{
	aKeys.SetSize(size);
	m_aKeyValues.SetSize(size);
}

void JWOrientationInterpolator::Add(double key, void* keyValue)
{
	aKeys.Add(key);
	m_aKeyValues.Add(*((SFRotation*)keyValue));
}


void JWOrientationInterpolator::Parse(JWToken& token)
{
	double tempDouble;
	SFRotation tempRotation;

	token.GetNextToken();
	if(token == BRACE_OPEN)
	{
		// key д´.
		token.GetNextToken();
		if(token == key)
		{
			token.GetNextToken();
			if(token == SQUAREBRACKET_OPEN)
			{
				while(true)
				{
					tempDouble = token.GetRealNumber();
					if(token == SQUAREBRACKET_CLOSE)
						break;
					aKeys.Add(tempDouble);
				}
			}
		}

		// KeyValue д´.
		token.GetNextToken();
		if(token == keyValue)
		{
			token.GetNextToken();
			if(token == SQUAREBRACKET_OPEN)
			{
				while(true)
				{
					tempRotation.x = token.GetRealNumber();
					if(token == SQUAREBRACKET_CLOSE)
						break;
					
					 tempRotation.y = token.GetRealNumber();
					 tempRotation.z = token.GetRealNumber();
					 tempRotation.theta = token.GetRealNumber();

					m_aKeyValues.Add(tempRotation);
				}
			}
		}
	}
}

void JWOrientationInterpolator::Write(ofstream& file)
{
	//   ִ Ȯϱ!
	assert(file.is_open());

	// Ŀ indentation . ?

	// 1. DEF Name ִٸ װ 
	if(m_bIsDEF)
		file << "\t\t" << "DEF " << m_strDEF << ' ';

	// 2. PositionInterpolator {
	file << "OrientationInterpolator {" << endl;

	// 3. key
	file << "\t\t\t" << "key [" << endl;

	for(int i=0; i<aKeys.GetSize(); i++)
	{
		if(i%5 == 0)
			file << "\t\t\t\t";

		file << aKeys[i];

		if(i != aKeys.GetSize() - 1)
			file << ", ";
		else
		{
			file << endl << "\t\t\t" << " ]" << endl;
			break;
		}

		if(i%5 == 4)
			file << endl;
	}

	// 4. keyValue
	file << "\t\t\t" << "keyValue [" << endl;

	for(i=0; i<m_aKeyValues.GetSize(); i++)
	{
		if(i%2 == 0)
			file << "\t\t\t\t";

		file << m_aKeyValues[i];

		if(i != m_aKeyValues.GetSize() - 1)
			file << ", ";
		else
		{
			file << endl << "\t\t\t" << " ]" << endl;
			break;
		}

		if(i%2 == 1)
			file << endl;
	}

	file << "\t\t" << '}' << endl;
}

ofstream& operator << (ofstream& file, JWOrientationInterpolator& pI)
{
	// 1. DEF Name ִٸ װ 
	if(pI.m_bIsDEF)
		file << "DEF " << pI.m_strDEF << ' ';

	// 2. PositionInterpolator {
	file << "OrientationInterpolator {" << endl;

	// 3. key
	file << '\t' << "key [" << endl;

	for(int i=0; i<pI.aKeys.GetSize(); i++)
	{
		if(i%5 == 0)
			file << "\t\t";

		file << pI.aKeys[i];

		if(i != pI.aKeys.GetSize() - 1)
			file << ", ";
		else
		{
			file << " ]" << endl;
			break;
		}

		if(i%5 == 4)
			file << endl;
	}

	// 4. keyValue
	file << '\t' << "keyValue [" << endl;

	for(i=0; i<pI.m_aKeyValues.GetSize(); i++)
	{
		if(i%5 == 0)
			file << "\t\t";

		file << pI.m_aKeyValues[i];

		if(i != pI.m_aKeyValues.GetSize() - 1)
			file << ", ";
		else
		{
			file << " ]" << endl;
			break;
		}

		if(i%5 == 4)
			file << endl;
	}

	return file;
}


double JWOrientationInterpolator::Purify(double threshold, JWInterpolator*& pInterpolator/*Output*/)
{
	// jameskim 20010803
	// Variables
	int		i = 0;
	int		dNumSelectedsKeys = 0;
	double	fDistortion = 0.0;
	JWArray <Quaternion>	qKeyValues, qSelectedKeyValues;
	JWArray <SFRotation>	rSelectedKeyValues;
	JWArray <int>			dKeyFlags;
	JWArray <double>		fSelectedKeys;				


	printf("Node Name : %s , # of Keys : %d\n", m_strDEF, aKeys.GetSize());

	//  OrientationInterpolator  key ŭ Quaternion array ҴѴ.
	qKeyValues.SetSize(aKeys.GetSize());
	//  OrientationInterpolator  key  Ÿ key flag array ҴѴ.
	dKeyFlags.SetSize(aKeys.GetSize());
	
	//  OrientationInterpolator  SFRotation type key value Quaternion ȯѴ.
	for(i = 0;i < m_aKeyValues.GetSize();i++)
		qKeyValues[i].RotationToQuaternion(&m_aKeyValues[i]);

	fDistortion = DownSampling_With_QC(qKeyValues, dKeyFlags, aKeys, aKeys.GetSize(), threshold);

	// key keyValue δ.
	// پ     .
	//    selectedKeys, selectedQuat, selectedRot, nSelectedKey, keyFlag
	qSelectedKeyValues.SetSize(aKeys.GetSize());
	rSelectedKeyValues.SetSize(aKeys.GetSize());
	fSelectedKeys.SetSize(aKeys.GetSize());


	Select_Key_and_KV(
			aKeys, fSelectedKeys,
			qKeyValues, qSelectedKeyValues,
			m_aKeyValues, rSelectedKeyValues,	
			dNumSelectedsKeys, dKeyFlags, aKeys.GetSize());
			
	// jameskim 20010106 : test
	printf("key selection complete : # of Original %d # of Selected %d\n", aKeys.GetSize(),
			dNumSelectedsKeys);


	// jameskim 20010803 
	//  Output .
	pInterpolator = new JWOrientationInterpolator;
	pInterpolator->SetSize(dNumSelectedsKeys);
	if(m_bIsDEF == true)
		pInterpolator->SetDEF(m_strDEF);
	for(i = 0;i < dNumSelectedsKeys;i++)
	{
		pInterpolator->SetKey(i, fSelectedKeys[i]);
		pInterpolator->SetKeyValue(i, (void*)&rSelectedKeyValues[i]);
	}

	return fDistortion;
}

//jameskim 20010803
double JWOrientationInterpolator::DownSampling_With_QC(
	JWArray<Quaternion>& kvn,
	JWArray<int>& keyFlag,
	JWArray<double>& key,
	int key_count, 
	double distortion_threshold
)
{
	// variables
	int i;
	double distortion, rms, peak, duration;
	JWArray<Quaternion> kvi;
	QuatError Error;
	// jameskim 20010803 --> ٷ ϳ  distortion ϴ  
	//  ʿϳ ϸ target distortion   purified version ϱ 
	double	prev_distortion;
	

	kvi.SetSize(key_count);

	//select two points
	// Quaternion error measure  Ʈ õǸ ̿   Ʈ  õ ǹѴ.
	for(i=0;i<key_count;i++)
		keyFlag[i] = 0;

	keyFlag[0] = keyFlag[key_count-1] = 1;

	//   interpolationѴ.
	Interpolate_With_QC(kvn, kvi, keyFlag, key, key_count);

	//calculate difference
	//ù° distortion Ѵ.
	Error.ComputeError(rms, peak, duration, key_count, key, kvn, key_count, key, kvi, 1);  // HELLO! //

	rms /= duration;
	distortion = sqrt(rms);
	

	
	//while loop: if difference > threshold
	int one_flag_count=2;
		
	// total distorion threshold    .
	int min_point=-1;
	while (distortion > distortion_threshold)
	{
		double error=DBL_MAX;
		min_point=-1;

		// jameskim 20010803
		prev_distortion = distortion;

		for(i=0;i<key_count;i++)
		{
			if (keyFlag[i] == 0) 
			{
				keyFlag[i] = 1;
				
				Interpolate_With_QC(kvn, kvi, keyFlag, key, key_count);

				Error.ComputeError(rms, peak, duration, key_count, key, kvn, key_count, key, kvi, 1);  // HELLO! //

				rms /= duration;
				distortion = sqrt(rms);

				if(distortion<error)
				{
					error=distortion;
					min_point=i;
				}

				keyFlag[i] = 0;
			}
		}
		
		distortion=error;
		printf("prev_distortion %g, distortion %g\n", prev_distortion, distortion);
		
		//  õ   flag setting....
		keyFlag[min_point] = 1;
		one_flag_count++;
	}

	// jameskim 20010803 prev_distortion distortion߿ target distortion   Purified version
	//  Ѵ.

	//  distortion distortion_threshold ̰ 
	//  distortion distortion_threshold ̺ ũ
	if((one_flag_count != 2) && (fabs(distortion_threshold - distortion) > fabs(distortion_threshold - prev_distortion)))
	{
		keyFlag[min_point] = 0;
		distortion = prev_distortion;
	}


	return distortion;
}

//jameskim 20010803
void JWOrientationInterpolator::Interpolate_With_QC(
	JWArray<Quaternion>& kvn,
	JWArray<Quaternion>& kvi,
	JWArray<int>& keyFlag,
	JWArray<double>& key,
	int key_count
)

{
	// Variables
	int			i;
	Quaternion	left_end, right_end;
	double		left_end_key, right_end_key;
	double		*keyi;

	keyi= new double[key_count];

	//reverse fill of the right-end point
	left_end =kvn[0];
	left_end_key= key[0];
	right_end=kvn[key_count-1];
	right_end_key=key[key_count-1];

	for (i=key_count-1;i>=0;i--)
	{
		if (keyFlag[i] == 1) 
		{
			right_end=kvn[i];
			right_end_key=key[i];
		}
		kvi[i]=right_end;
		keyi[i]=right_end_key;
	}

	//For loop
	for(i=0;i<key_count;i++)
	{
		if (keyFlag[i] ==1)
		{
			left_end=kvn[i];
			left_end_key=key[i];
		}
		else // to be interpolated.
		{
			right_end=kvi[i];
			right_end_key=keyi[i];
			
			left_end.Slerp(&right_end, ((key[i]-left_end_key)/(right_end_key-left_end_key)), &kvi[i]);
		}
	}

	delete [] keyi;
}


//jameskim 20010803
void JWOrientationInterpolator::Select_Key_and_KV (
	JWArray<double>& key,
	JWArray<double>& q_key,
	JWArray<Quaternion>& keyValue,
	JWArray<Quaternion>& q_keyValue,
	JWArray<SFRotation>& InRot,
	JWArray<SFRotation>& q_InRot,
	int &n_q_key,
	JWArray<int>& keyFlag,
	int key_count
)
{
	int i,j;

	j=0;
	for(i=0;i<key_count;i++)
	{
		if(keyFlag[i]) 
		{			
			q_key[j] = key[i];
			q_keyValue[j] = keyValue[i];
			q_InRot[j] = InRot[i];

			j++;
		}
	}
	n_q_key=j;
}




// sowoo 2002.01.24 (Resampling)
// յ  ־ ŭ ø Ѵ.
// sowoo 2002.01.25 - digit ϶!, Ȯ ©!
void JWOrientationInterpolator::Resampling(int nNumKey, int nDigit)
{
	int nNumberOfKey = aKeys.GetSize();

	//  key    볳  .
//	if(nNumberOfKey < nNumKey)
//		nNumKey = nNumberOfKey;

	// 1. ־  key .
	double* aSampledKeys = new double[nNumKey];
	SFRotation* aSampledKeyValues = new SFRotation[nNumKey];


	double dStep = (aKeys[nNumberOfKey-1] - aKeys[0])/(nNumKey-1);

	char buffer[50];

	for(int i=0; i<nNumKey; i++)
	{
		aSampledKeys[i] = dStep*i + aKeys[0];

		// sowoo 2002.01.25 ©!!! Ѥ'
		if(nDigit > 0)
		{
			_gcvt(aSampledKeys[i], nDigit, buffer);
			aSampledKeys[i] = atof(buffer);
		}
	}


	// 2. ־  keyValue .
	int j;


	aSampledKeyValues[0] = m_aKeyValues[0];
	aSampledKeyValues[nNumKey - 1] = m_aKeyValues[nNumberOfKey - 1];

	//  Key    Samplingϴ 
//	if(nNumberOfKey > nNumKey)
//	{
		for(i=1, j=1; i<nNumberOfKey;)
		{
			//if(aKeys[i] < aSampledKeys[j])
			// i-1 < j < i ̳?
			// ƴϸ,  ...
			if(!((aKeys[i-1] <= aSampledKeys[j]) && (aSampledKeys[j] < aKeys[i])))
			{
				i++;
				continue;
			}

			// ´  ´ٸ...
			//InterpolateKeyValue(
			//	aKeys[i-1], aKeys[i], aSampledKeys[j],
			//	(void*) &m_aKeyValues[i-1], (void*) &m_aKeyValues[i],
			//	(void*) &aSampledKeyValues[j]
			//);
			m_aKeyValues[i-1].Slerp(
				&m_aKeyValues[i],
				(aSampledKeys[j] - aKeys[i-1])/(aKeys[i] - aKeys[i-1]),
				&aSampledKeyValues[j]
			);


			if(nNumKey <= j)
				break;

			j++;
		}

		/*
		for(i=1, j=1; i<nNumberOfKey; i++)
		{
			if(aKeys[i] < aSampledKeys[j])
				continue;

			// ´  ´ٸ...
			//InterpolateKeyValue(
			//	aKeys[i-1], aKeys[i], aSampledKeys[j],
			//	(void*) &m_aKeyValues[i-1], (void*) &m_aKeyValues[i],
			//	(void*) &aSampledKeyValues[j]
			//);
			m_aKeyValues[i-1].Slerp(
				&m_aKeyValues[i],
				(aSampledKeys[j] - aKeys[i-1])/(aKeys[i] - aKeys[i-1]),
				&aSampledKeyValues[j]
			);



			if(nNumKey <= j)
				break;

			j++;
		}*/
//	}
/*
	//  key   samplingϴ 
	else
	{
		for(i=1, j=1; i<nNumberOfKey;)
		{
			if(aKeys[i] < aSampledKeys[j])
			{
				i++;
				continue;
			}

			// ´  ´ٸ...
			//InterpolateKeyValue(
			//	aKeys[i-1], aKeys[i], aSampledKeys[j],
			//	(void*) &m_aKeyValues[i-1], (void*) &m_aKeyValues[i],
			//	(void*) &aSampledKeyValues[j]
			//);
			m_aKeyValues[i-1].Slerp(
				&m_aKeyValues[i],
				(aSampledKeys[j] - aKeys[i-1])/(aKeys[i] - aKeys[i-1]),
				&aSampledKeyValues[j]
			);


			j++;

			if(nNumKey <= j)
				break;
		}
	}
*/

	//  ִ  Key & KeyValue ϰ
	aKeys.Copy(aSampledKeys, nNumKey);
	m_aKeyValues.Copy(aSampledKeyValues, nNumKey);
}
