// Quantizer.cpp: implementation of the Quantizer class.
//
//////////////////////////////////////////////////////////////////////

#include "Quantizer.h"

#include <math.h>


#define MAX(x, y)       ( (x) < (y) ? (y) : (x) )
#define MIN(x, y)       ( (x) < (y) ? (x) : (y) )
#define Nonlinearity    2.0

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Quantizer::Quantizer()
{

}

Quantizer::~Quantizer()
{

}

//unsigned long int Quantizer::Quantize(double mini, double maxi, int nbits, double v)
long int Quantizer::Quantize(double mini, double maxi, int nbits, double v)
{
	if(v<=mini)
		return 0;
	if(v>=maxi) 
		return (1<<nbits)-1;
	  
	//unsigned long int tmp = (unsigned long int) floor( ((v-mini)/(maxi-mini)*((1<<nbits)-1)) + 0.5);
	long int tmp = (long int) floor( ((v-mini)/(maxi-mini)*((1<<nbits)-1)) + 0.5);
  
	return tmp;
}


void Quantizer::QuantizerInitialize(int bits)
{
	int i;
	int nBitsQuantization = MAX((bits-1), 1);
	QuantizerScale = 1.0;
	for ( i = 0 ; i < nBitsQuantization ; ++i ) 
		QuantizerScale *= 2.0;
	QuantizerOffset = QuantizerScale;
}


void Quantizer::UniformQuantizer(double *v, unsigned long int *QuantizedIntegerData)
{
	int               sign_bit;
	double            TmpData;
	unsigned long int QuantData;
	TmpData = *v;
	TmpData = MAX(-1.0, TmpData);
	TmpData = MIN( 1.0, TmpData);

	if ( TmpData < 0.0 ) { sign_bit = 1; TmpData *= -1.0; }
	else                   sign_bit = 0;

	QuantData = (unsigned long int)floor( TmpData * ( QuantizerScale - 1.0 ) + 0.5 );

	if ( sign_bit ) QuantData = (unsigned long int)floor( QuantizerOffset + 0.5 ) - QuantData;
	else            QuantData = (unsigned long int)floor( QuantizerOffset + 0.5 ) + QuantData;

	*QuantizedIntegerData = QuantData;

	//TmpData = ( ( (double)QuantData - QuantizerOffset ) / ( QuantizerScale - 1.0 ) );
	//*v = TmpData;
}


//double Quantizer::InvUniformQuantizer(unsigned long int QuantizedIntegerData)
double Quantizer::InvUniformQuantizer(long int QuantizedIntegerData)
{
	double TmpData;

	TmpData = ( ( (double)QuantizedIntegerData - QuantizerOffset ) / ( QuantizerScale - 1.0 ) );

	return TmpData;
}

//double Quantizer::InvQuantize(double mini, double maxi, int nbits, unsigned long int v) 
double Quantizer::InvQuantize(double mini, double maxi, int nbits, int v) 
{

	if(v==0) 
		return mini;
	if(v==(int) ((1<<nbits)-1)) 
		return maxi;
  
	return (double) (mini+(maxi-mini)/((double) ((1<<nbits)-1))*(double) v);
}


void Quantizer::AdaptiveTransform(double *SourceKeyValues, int Direction)
{
	int     i, sign_flag;

	if ( Direction ) {
		for ( i = 0 ; i < 3 ; ++i ) {
			// ڿ root     ֱ ؼ
			if ( SourceKeyValues[i] < 0.0 ) { 
				sign_flag = 1;
				SourceKeyValues[i] *= -1.0;
			}
			else {
				sign_flag = 0;
			}
			
			SourceKeyValues[i] = pow(SourceKeyValues[i], (1.0/Nonlinearity));
			
			if ( sign_flag ) SourceKeyValues[i] *= -1.0;
		}
	}
	else {
		for ( i = 0 ; i < 3 ; ++i ) {
			// ڿ root     ֱ ؼ
			if ( SourceKeyValues[i] < 0.0 ) {
			  sign_flag = 1;
			  SourceKeyValues[i] *= -1.0; 
			}
			else {
				sign_flag = 0;
			}
			
			SourceKeyValues[i] = pow(SourceKeyValues[i], (Nonlinearity));
			
			if ( sign_flag ) SourceKeyValues[i] *= -1.0;
		}
	}
}
