// Rotation.cpp: implementation of the Rotation class.
//
//////////////////////////////////////////////////////////////////////

#include "SFRotation.h"
#include "Quaternion.h"

#include <math.h>



#define DELTA			1.0E-50

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

SFRotation::SFRotation()
{
	x = 0.0;
	y = 0.0;
	z = 0.0;
	theta = 0.0;
}

SFRotation::SFRotation(double in_x, double in_y, double in_z, double in_theta)
{
	x = in_x;
	y = in_y;
	z = in_z;
	theta = in_theta;
}


SFRotation::~SFRotation()
{

}


void SFRotation::QuaternionToRotation(Quaternion* m_pSrcQuat)
{
	theta = acos(m_pSrcQuat->s) * 2.0;
	
	double norm = sin(theta);
	if(fabs(norm) < DELTA)
	{
		x = 0.0;
		y = 1.0;
		z = 0.0;
		theta = 0.0;
	}
	else
	{
		x = m_pSrcQuat->x/sin(theta/2.0);
		y = m_pSrcQuat->y/sin(theta/2.0);
		z = m_pSrcQuat->z/sin(theta/2.0);
	}
}


//Spherical Linear interpolation
// Jameskim 2002.04.23 (Add Resampling)
// sowoo 2002.01.29 (Add Resampling)
// , ̳ Rotation Է Ѵ.
void SFRotation::Slerp(SFRotation* pDestRot, double u, SFRotation* pResRot)
{
	Quaternion sourceQuat, destQuat, resQuat;
	sourceQuat.RotationToQuaternion(this);
	destQuat.RotationToQuaternion(pDestRot);

	sourceQuat.Slerp(&destQuat, u, &resQuat);

	pResRot->QuaternionToRotation(&resQuat);
}