/***********************************************************************
This software module was originally developed by James D. K. Kim, 
Sang Oak Woo and Shin Jun Lee(Samsung Advance Institute of Technology)
in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)
standard. This software module is an implementation of a part of one or
more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4
Systems (ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 
Systems (ISO/IEC 14496-1) free license to this software module or 
modifications thereof for use in hardware or software products claiming
conformance to the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to 
use this software module in hardware or software products are advised that
its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Samsung Advance Institute of Technology
retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using
the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
This copyright notice must be included in all copies or derivative works.
Copyright  2001
************************************************************************

For more information or to receive an up to date version of this module,
you may contact James D. K. Kim at jacobkim@sait.samsung.co.kr
----------------------------------------------------------------------*/

// OriIDPCMKeyValue.cpp: implementation of the OriIDPCMKeyValue class.
//
//////////////////////////////////////////////////////////////////////

//#include "stdafx.h"
#include "OriIDPCMKeyValue.h"

#define M_PI				3.1415926535897932384338

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

OriIDPCMKeyValue::OriIDPCMKeyValue()
{

}

OriIDPCMKeyValue::~OriIDPCMKeyValue()
{

}

void OriIDPCMKeyValue::readBitstream(CAC_QF& codec, OriIDPCMKeyValueHeader& kvHeader, int nNumKV)
{
	int i;
	int tmpVal;

	// Jameskim 20020426
	// memory allocation
	DeltaKeyValue.SetSize(nNumKV);

	// read x component of inter key value quaternion
	if(kvHeader.x_keyvalue_flag != 0)
	{
		if(kvHeader.oriIKVCodingBit_X.bIsUnaryAAC == 1)
			for(i = 0;i < nNumKV;i++)
			{
				decodeUnaryAAC(codec, &tmpVal, kvXUSignContext, kvXUContext);
				DeltaKeyValue[i].x = (double)tmpVal;
			}
		else
			for(i = 0;i < nNumKV;i++)
			{
				decodeSignedAAC(codec, &tmpVal, kvHeader.oriIKVCodingBit_X.nKVCodingBit,
								kvXSignContext, kvXContext);
				DeltaKeyValue[i].x = (double)tmpVal;
			}
	}
	else
	{
		for(i = 0;i < nNumKV;i++)
			DeltaKeyValue[i].x = kvHeader.oriIKVCodingBit_X.nAllKeyValue;
	}


	// read y component of inter key value quaternion
	if(kvHeader.y_keyvalue_flag != 0)
	{
		if(kvHeader.oriIKVCodingBit_Y.bIsUnaryAAC == 1)
			for(i = 0;i < nNumKV;i++)
			{
				decodeUnaryAAC(codec, &tmpVal, kvYUSignContext, kvYUContext);
				DeltaKeyValue[i].y = (double)tmpVal;
			}
		else
			for(i = 0;i < nNumKV;i++)
			{
				//decodeSignedAAC(codec, (int *)(&DeltaKeyValue[i].y), kvHeader.oriIKVCodingBit_Y.nKVCodingBit,
				//				kvYSignContext, kvYContext);
				decodeSignedAAC(codec, &tmpVal, kvHeader.oriIKVCodingBit_Y.nKVCodingBit,
								kvYSignContext, kvYContext);
				DeltaKeyValue[i].y = (double)tmpVal;
			}
	}
	else
	{
		for(i = 0;i < nNumKV;i++)
			DeltaKeyValue[i].y = kvHeader.oriIKVCodingBit_Y.nAllKeyValue;
	}


	// read z component of inter key value quaternion
	if(kvHeader.z_keyvalue_flag != 0)
	{
		if(kvHeader.oriIKVCodingBit_Z.bIsUnaryAAC == 1)
			for(i = 0;i < nNumKV;i++)
			{
				decodeUnaryAAC(codec, &tmpVal, kvZUSignContext, kvZUContext);
				DeltaKeyValue[i].z = (double)tmpVal;
			}
		else
			for(i = 0;i < nNumKV;i++)
			{
				decodeSignedAAC(codec, &tmpVal, kvHeader.oriIKVCodingBit_Z.nKVCodingBit,
								kvZSignContext, kvZContext);
				DeltaKeyValue[i].z = (double)tmpVal;
			}
	}
	else
	{
		for(i = 0;i < nNumKV;i++)
			DeltaKeyValue[i].z = kvHeader.oriIKVCodingBit_Z.nAllKeyValue;
	}

}

void OriIDPCMKeyValue::InitializeModel()
{
	int i;

	kvXUSignContext.reset();  
	kvXUContext.reset();
	kvYUSignContext.reset();  
	kvYUContext.reset();
	kvZUSignContext.reset();  
	kvZUContext.reset();
	kvXSignContext.reset();
	for(i = 0;i < 32;i++)
		kvXContext[i].reset();
	kvYSignContext.reset();
	for(i = 0;i < 32;i++)
		kvYContext[i].reset();
	kvZSignContext.reset();
	for(i = 0;i < 32;i++)
		kvZContext[i].reset();
}

void OriIDPCMKeyValue::decodingProcess(KeyHeader& kHeader,
									   KeySelectionFlag& ksFlag,
									   OriIKeyValueHeader& oriIKVHeader,
									   double* key,
									   JWArray <SFRotation>& keyValue,
									   JWArray <Quaternion>& keyValueQuat)
{
	int						i, j;
	int						value = 0;
	int						sgn = 0;
	Quaternion				DeQuant;
	double					deQuantValue[3];
	JWArray <Quaternion>	DecodedKeyValueQuat;
	JWArray <Quaternion>	ReDPCMDecodedKeyValueQuat;
	JWArray<Quaternion>		nodeReconstructedQuat;
	JWQuantizer				DeQuantizer;


	// set size keyvalue
	keyValue.SetSize(kHeader.nNumberOfKey);
	DecodedKeyValueQuat.SetSize(ksFlag.nNumOfKeyValue);

	// --------------------------------------------------------------------------
	//
	// 1st Inverse DPCM
	//
	// --------------------------------------------------------------------------
	if(oriIKVHeader.nKVDPCMOrder == 0)
	{
		// Reconstruct first keyvalue
		value = (int)oriIKVHeader.oriIDPCMKVHeader.firstQKV_S;
		sgn = ((value>=0)?1:-1);
		DeQuant.s = sgn * DeQuantizer.InvQuantize(0, 1, oriIKVHeader.nKVQBit-1,sgn*value);
		
		//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		// if nFirstXSign is 1, firstQKV_X has positive value(+)
		//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		if(oriIKVHeader.oriIDPCMKVHeader.nFirstXSign == 1)
			value = (int)oriIKVHeader.oriIDPCMKVHeader.firstQKV_X;
		else
			value = (int)(-1.0 * (int)oriIKVHeader.oriIDPCMKVHeader.firstQKV_X);
		sgn = ((value>=0)?1:-1);
		DeQuant.x = sgn * DeQuantizer.InvQuantize(0, 1, oriIKVHeader.nKVQBit-1,sgn*value);

		if(oriIKVHeader.oriIDPCMKVHeader.nFirstYSign == 1)
			value = (int)oriIKVHeader.oriIDPCMKVHeader.firstQKV_Y;
		else
			value = (int)(-1.0 * (int)oriIKVHeader.oriIDPCMKVHeader.firstQKV_Y);
		sgn = ((value>=0)?1:-1);
		DeQuant.y = sgn * DeQuantizer.InvQuantize(0, 1, oriIKVHeader.nKVQBit-1,sgn*value);

		if(oriIKVHeader.oriIDPCMKVHeader.nFirstZSign == 1)
			value = (int)oriIKVHeader.oriIDPCMKVHeader.firstQKV_Z;
		else
			value = (int)(-1.0 * (int)oriIKVHeader.oriIDPCMKVHeader.firstQKV_Z);
		sgn = ((value>=0)?1:-1);
		DeQuant.z = sgn * DeQuantizer.InvQuantize(0, 1, oriIKVHeader.nKVQBit-1,sgn*value);

		DeQuant.s = tan(M_PI * DeQuant.s / 4.0);
		DeQuant.x = tan(M_PI * DeQuant.x / 4.0);
		DeQuant.y = tan(M_PI * DeQuant.y / 4.0);
		DeQuant.z = tan(M_PI * DeQuant.z / 4.0);

		DeQuant.Normalize();

		DecodedKeyValueQuat[0].s = DeQuant.s;
		DecodedKeyValueQuat[0].x = DeQuant.x;
		DecodedKeyValueQuat[0].y = DeQuant.y;
		DecodedKeyValueQuat[0].z = DeQuant.z;


		// Reconstruct remaining key values
		for(i = 1;i <= ksFlag.nNumOfKeyValue-1;i++)
		{

			value = (int)DeltaKeyValue[i-1].x;
			sgn = ((value>=0)?1:-1);
			deQuantValue[0] = sgn * DeQuantizer.InvQuantize(0, 1, oriIKVHeader.nKVQBit-1,sgn*value);

			value = (int)DeltaKeyValue[i-1].y;
			sgn = ((value>=0)?1:-1);
			deQuantValue[1] = sgn * DeQuantizer.InvQuantize(0, 1, oriIKVHeader.nKVQBit-1,sgn*value);

			value = (int)DeltaKeyValue[i-1].z;
			sgn = ((value>=0)?1:-1);
			deQuantValue[2] = sgn * DeQuantizer.InvQuantize(0, 1, oriIKVHeader.nKVQBit-1,sgn*value);
			
			// Non-linear quatization
			for(j = 0;j < 3;j++)
			{
				deQuantValue[j] = tan(M_PI * deQuantValue[j] / 4.0);
			}

			// caculate q0
			double temp = (pow(deQuantValue[0], 2.0)+pow(deQuantValue[1], 2.0)
					+pow(deQuantValue[2], 2.0));
			
			// Exceptional Handling
			if(temp > 1.0)
			{
				printf("q0 is less than 0\n");
				getchar();
				exit(0);
			}
			else
				temp = sqrt(1.0 - (pow(deQuantValue[0], 2.0)+pow(deQuantValue[1], 2.0)
					+pow(deQuantValue[2], 2.0)));

			DeQuant.s = temp;
			DeQuant.x = deQuantValue[0];
			DeQuant.y = deQuantValue[1];
			DeQuant.z = deQuantValue[2];
			DeQuant.Normalize();

			DecodedKeyValueQuat[i] = DeQuant * DecodedKeyValueQuat[i-1];
		}
	} // end of if(oriIKVHeader.nKVDPCMOrder == 0)
	


	// --------------------------------------------------------------------------
	//
	// 2nd Inverse DPCM
	//
	// --------------------------------------------------------------------------
	else if(oriIKVHeader.nKVDPCMOrder == 1)
	{
		int tmpVal;
		// set size reDPCM keyvalue
		ReDPCMDecodedKeyValueQuat.SetSize(ksFlag.nNumOfKeyValue);

		// Reconstruct first quantized reDPCM keyvalue
		ReDPCMDecodedKeyValueQuat[0].s = (int)oriIKVHeader.oriIDPCMKVHeader.firstQKV_S;

		if(oriIKVHeader.oriIDPCMKVHeader.nFirstXSign == 1)
			ReDPCMDecodedKeyValueQuat[0].x = (int)oriIKVHeader.oriIDPCMKVHeader.firstQKV_X;
		else
			ReDPCMDecodedKeyValueQuat[0].x = (int)(-1.0 * (int)oriIKVHeader.oriIDPCMKVHeader.firstQKV_X);

		if(oriIKVHeader.oriIDPCMKVHeader.nFirstYSign == 1)
			ReDPCMDecodedKeyValueQuat[0].y = (int)oriIKVHeader.oriIDPCMKVHeader.firstQKV_Y;
		else
			ReDPCMDecodedKeyValueQuat[0].y = (int)(-1.0 * (int)oriIKVHeader.oriIDPCMKVHeader.firstQKV_Y);

		if(oriIKVHeader.oriIDPCMKVHeader.nFirstZSign == 1)
			ReDPCMDecodedKeyValueQuat[0].z = (int)oriIKVHeader.oriIDPCMKVHeader.firstQKV_Z;
		else
			ReDPCMDecodedKeyValueQuat[0].z = (int)(-1.0 * (int)oriIKVHeader.oriIDPCMKVHeader.firstQKV_Z);


		// Reconstruct second quantized reDPCM keyvalue
		if(oriIKVHeader.oriIDPCMKVHeader.nSecondXSign == 1)
			ReDPCMDecodedKeyValueQuat[1].x = (int)oriIKVHeader.oriIDPCMKVHeader.secondQKV_X;
		else
			ReDPCMDecodedKeyValueQuat[1].x = (int)(-1.0 * (int)oriIKVHeader.oriIDPCMKVHeader.secondQKV_X);

		if(oriIKVHeader.oriIDPCMKVHeader.nSecondYSign == 1)
			ReDPCMDecodedKeyValueQuat[1].y = (int)oriIKVHeader.oriIDPCMKVHeader.secondQKV_Y;
		else
			ReDPCMDecodedKeyValueQuat[1].y = (int)(-1.0 * (int)oriIKVHeader.oriIDPCMKVHeader.secondQKV_Y);

		if(oriIKVHeader.oriIDPCMKVHeader.nSecondZSign == 1)
			ReDPCMDecodedKeyValueQuat[1].z = (int)oriIKVHeader.oriIDPCMKVHeader.secondQKV_Z;
		else
			ReDPCMDecodedKeyValueQuat[1].z = (int)(-1.0 * (int)oriIKVHeader.oriIDPCMKVHeader.secondQKV_Z);
		
		
		// Reconstruct remaining quantized reDPCM keyvalue
		for(i = 2;i <= ksFlag.nNumOfKeyValue-1;i++)
		{
			//ICDPCM((int *)&ReDPCMDecodedKeyValueQuat[i].x,
			//		(int)DeltaKeyValue[i-2].x, (int)ReDPCMDecodedKeyValueQuat[i-1].x, oriIKVHeader.nKVQBit);
			ICDPCM(&tmpVal,
					(int)DeltaKeyValue[i-2].x, (int)ReDPCMDecodedKeyValueQuat[i-1].x, oriIKVHeader.nKVQBit);
			ReDPCMDecodedKeyValueQuat[i].x = (double)tmpVal;
			
			ICDPCM(&tmpVal,
					(int)DeltaKeyValue[i-2].y, (int)ReDPCMDecodedKeyValueQuat[i-1].y, oriIKVHeader.nKVQBit);
			ReDPCMDecodedKeyValueQuat[i].y = (double)tmpVal;
			
			ICDPCM(&tmpVal,
					(int)DeltaKeyValue[i-2].z, (int)ReDPCMDecodedKeyValueQuat[i-1].z, oriIKVHeader.nKVQBit);
			ReDPCMDecodedKeyValueQuat[i].z = (double)tmpVal;
		}


		// Reconstruct first keyvalue
		value = (int)ReDPCMDecodedKeyValueQuat[0].s;
		sgn = ((value>=0)?1:-1);
		DeQuant.s = sgn * DeQuantizer.InvQuantize(0, 1, oriIKVHeader.nKVQBit-1,sgn*value);

		value = (int)ReDPCMDecodedKeyValueQuat[0].x;
		sgn = ((value>=0)?1:-1);
		DeQuant.x = sgn * DeQuantizer.InvQuantize(0, 1, oriIKVHeader.nKVQBit-1,sgn*value);

		value = (int)ReDPCMDecodedKeyValueQuat[0].y;
		sgn = ((value>=0)?1:-1);
		DeQuant.y = sgn * DeQuantizer.InvQuantize(0, 1, oriIKVHeader.nKVQBit-1,sgn*value);

		value = (int)ReDPCMDecodedKeyValueQuat[0].z;
		sgn = ((value>=0)?1:-1);
		DeQuant.z = sgn * DeQuantizer.InvQuantize(0, 1, oriIKVHeader.nKVQBit-1,sgn*value);

		DeQuant.s = tan(M_PI * DeQuant.s / 4.0);
		DeQuant.x = tan(M_PI * DeQuant.x / 4.0);
		DeQuant.y = tan(M_PI * DeQuant.y / 4.0);
		DeQuant.z = tan(M_PI * DeQuant.z / 4.0);

		DeQuant.Normalize();

		DecodedKeyValueQuat[0].s = DeQuant.s;
		DecodedKeyValueQuat[0].x = DeQuant.x;
		DecodedKeyValueQuat[0].y = DeQuant.y;
		DecodedKeyValueQuat[0].z = DeQuant.z;

		// Reconstruct remaining keyvalues
		for(i = 1;i <= ksFlag.nNumOfKeyValue-1;i++)
		{

			value = (int)ReDPCMDecodedKeyValueQuat[i].x;
			sgn = ((value>=0)?1:-1);
			deQuantValue[0] = sgn * DeQuantizer.InvQuantize(0, 1, oriIKVHeader.nKVQBit-1,sgn*value);

			value = (int)ReDPCMDecodedKeyValueQuat[i].y;
			sgn = ((value>=0)?1:-1);
			deQuantValue[1] = sgn * DeQuantizer.InvQuantize(0, 1, oriIKVHeader.nKVQBit-1,sgn*value);

			value = (int)ReDPCMDecodedKeyValueQuat[i].z;
			sgn = ((value>=0)?1:-1);
			deQuantValue[2] = sgn * DeQuantizer.InvQuantize(0, 1, oriIKVHeader.nKVQBit-1,sgn*value);
			
			// Non-linear dequanize
			for(j = 0;j < 3;j++)
			{
				deQuantValue[j] = tan(M_PI * deQuantValue[j] / 4.0);
			}

			// caculate q0
			double temp = (pow(deQuantValue[0], 2.0)+pow(deQuantValue[1], 2.0)
					+pow(deQuantValue[2], 2.0));
			
			// Exceptional handling
			if(temp > 1.0)
			{
				printf("q0 is less than 0\n");
				getchar();
				exit(0); 
			}					
			else
				temp = sqrt(1.0 - (pow(deQuantValue[0], 2.0)+pow(deQuantValue[1], 2.0)
					+pow(deQuantValue[2], 2.0)));

			DeQuant.s = temp;
			DeQuant.x = deQuantValue[0];
			DeQuant.y = deQuantValue[1];
			DeQuant.z = deQuantValue[2];
			DeQuant.Normalize();

			DecodedKeyValueQuat[i] = DeQuant * DecodedKeyValueQuat[i-1];
		}
	} // end if else if(oriIKVHeader.nKVDPCMOrder == 1)
	
	// Exceptional Handling of DPCM order
	else 
	{
		printf("Decoding failed in nKVDPCMOrder\n");
		getchar();
		exit(0);
	} // end of Exceptional Handling



	



	// --------------------------------------------------------------------------
	//
	// Intepolating missing key values in key preserving mode : Interpolator Synthesizer
	//
	// --------------------------------------------------------------------------
	if(oriIKVHeader.bPreserveKey)
	{
		Quaternion initQuat;
		Quaternion qSource, qDest, qInterpolatedOne;

		int	l = 0;
		double u, fSource, fDest, fTarget;
		
		nodeReconstructedQuat.SetSize(kHeader.nNumberOfKey);

		for(int k = 0; k < kHeader.nNumberOfKey;k++)
		{
			if(ksFlag.keyFlag[k] == 1)
				nodeReconstructedQuat[k] = DecodedKeyValueQuat[l++];
			else
				nodeReconstructedQuat[k] = initQuat;
		}
		l = 0;

		for(k = 0; k < kHeader.nNumberOfKey;k++)
		{
			// interpolatr key values that their corresponding keys have key flag '0'
			if(ksFlag.keyFlag[k] == 0)
			{
				l = k;			
				// find the first lower key flag index of which value is not '0'
				while(ksFlag.keyFlag[l] == 0)
				{
					l--;
				}
				fSource = key[l];
				qSource = nodeReconstructedQuat[l];

				l = k;			
				// find the first upper key flag index of which value is not '0'
				while(ksFlag.keyFlag[l] == 0)
				{
					l++;
				}
				fDest = key[l];
				qDest = nodeReconstructedQuat[l];
			
				fTarget = key[k];

				u = (fTarget - fSource)/(fDest - fSource);
			
				
				double	TargetQuaternion[4], LowerQuaternion[4], UpperQuaternion[4];

				LowerQuaternion[0]=qSource.s;LowerQuaternion[1]=qSource.x;
				LowerQuaternion[2]=qSource.y;LowerQuaternion[3]=qSource.z;
				UpperQuaternion[0]=qDest.s;UpperQuaternion[1]=qDest.x;
				UpperQuaternion[2]=qDest.y;UpperQuaternion[3]=qDest.z;
			
				QuatError IntError;
				IntError.SphericalLinearInterpolator(
														&TargetQuaternion[0],
														&LowerQuaternion[0],
														&UpperQuaternion[0],
														fTarget,
														fSource,
														fDest
													);
			
				qInterpolatedOne.s = TargetQuaternion[0];qInterpolatedOne.x = TargetQuaternion[1];
				qInterpolatedOne.y = TargetQuaternion[2];qInterpolatedOne.z = TargetQuaternion[3];

				nodeReconstructedQuat[k] = qInterpolatedOne;
			}
		}
			
/*	
		Error.ComputeError(dRmsError, dPeakError, dNodeDuration,
							nodeOriginalNumKey, nodeOriginalKey, nodeOriginalQuat,
							nodeDecodedNumKey, nodeDecodedKey, nodeReconstructedQuat, 0);  // HELLO! //
*/
		
		
	}

	else
	{
/*
		Error.ComputeError(dRmsError, dPeakError, dNodeDuration,
							nodeOriginalNumKey, nodeOriginalKey, nodeOriginalQuat,
							nodeDecodedNumKey, nodeDecodedKey, DecodedKeyValueQuat, 0);  // HELLO! //
*/
		nodeReconstructedQuat.Copy(DecodedKeyValueQuat);
	}

	// --------------------------------------------------------------------------
	//
	// Convert key values from Quaternion to SFRotation
	//
	// --------------------------------------------------------------------------
	for(i = 0;i < kHeader.nNumberOfKey;i++)
		keyValue[i].QuaternionToRotation(&nodeReconstructedQuat[i]);

	// for temporary use
	keyValueQuat.Copy(nodeReconstructedQuat);
}


void OriIDPCMKeyValue::ICDPCM(int* curIDPCMKeyValue, int deltaKeyValue, int prevIDPCMKeyValue, int nKVQBit)
{
	int circularDelta;
	int tempIDPCMKeyValue;

	prevIDPCMKeyValue += ((1 << (nKVQBit-1))-1);

	if(deltaKeyValue >= 0.0)
		circularDelta = deltaKeyValue - ((1 << nKVQBit)-1);
	else
		circularDelta = deltaKeyValue + ((1 << nKVQBit)-1);

	tempIDPCMKeyValue = circularDelta + prevIDPCMKeyValue;

	if((tempIDPCMKeyValue >= 0.0) && (tempIDPCMKeyValue < ((1 << nKVQBit)-1)))
		*curIDPCMKeyValue = tempIDPCMKeyValue;
	else
		*curIDPCMKeyValue = deltaKeyValue + prevIDPCMKeyValue;

	*curIDPCMKeyValue -= ((1 << (nKVQBit-1))-1);
}