#include "ArithmeticDecoder.h"

void decodeUnaryAAC(
	CAC_QF& codec,
	int* nDecodedValue,
	CAC& signContext, CAC& valueContext
)
{
	int nBits = -1;
	int bBit;
	do {
		//qf_decode(&bBit, valueContext);
		bBit = codec.decode(valueContext);
		nBits++;
	} while(bBit == 0);

	if(nBits != 0) {
		//qf_decode(&bBit, signContext);
		bBit = codec.decode(signContext);
		if(bBit == 0)
			*nDecodedValue = nBits;
		else
			*nDecodedValue = -nBits;
	}
	else
		*nDecodedValue = 0;
}


void decodeSQAAC(
	CAC_QF& codec,
	int* anDecodedValues, int qstep, 
	CAC& signContext, CAC& maxValueContext,
	CAC& foundContext, CAC& notFoundContext,
	int keynum, int start
)
{
	int* range_mid = new int[keynum];
	int* range_min = new int[keynum];
	int* range_max = new int[keynum];
	int* sign = new int[keynum];
	bool* found = new bool[keynum];

	int max_val = 0;
	int max_count = 1;
	bool all_level_done = false;
	int max_index = -1;
	int bBit; 

	for(int j=qstep-2; j>=0; j--)
	{
		//qf_decode(&bBit, &maxValueContext);
		bBit = codec.decode(maxValueContext);
		max_val = (max_val << 1) + bBit;
	}

	for(int i=start; i<keynum; i++)
	{
		range_min[i] = 0;
		range_max[i] = max_val;
		found[i] = false;
		//qf_decode(&bBit, &signContext);
		bBit = codec.decode(signContext);
		if (bBit)
			sign[i] = -1;
		else
			sign[i] = 1;
	}

	while (!all_level_done)
	{
		all_level_done = true;

		max_index = -1;
		for(i=start; i<keynum; i++)
		{
			if (range_max[i] != range_min[i])
			{	
				all_level_done = false;
				range_mid[i] = (int)((range_max[i] - range_min[i])/2) + 1 + range_min[i];

				if(found[i])
					//qf_decode(&bBit, &foundContext);
					bBit = codec.decode(foundContext);
				else
					//qf_decode(&bBit, &notFoundContext);
					bBit = codec.decode(notFoundContext);

				if(bBit)
				{
					if (max_count == 1 && range_max[i] == max_val)
					{
						if (max_index == -1)
							max_index = i;
						else
							max_index = keynum;
					}
					range_min[i] = range_mid[i];
					found[i] = true;
				}
				else
					range_max[i] = range_mid[i]-1;
			}
		}
		if (max_index >= 0 && max_index < keynum)
		{
			range_min[max_index] = max_val;
			max_count = 0;
		}
	}
	for(i=start; i<keynum; i++)
		anDecodedValues[i] = range_max[i] * sign[i];

	delete[] range_mid;
	delete[] range_min;
	delete[] range_max;
	delete[] sign;
	delete[] found;
}

// decode: aac values with sign
void decodeSignedAAC(CAC_QF& codec, int* nDecodedValue, int qstep, CAC& signContext, CAC* valueContext)
{
	int b;
	b = qstep-2;
	int msb = 0;
	do {
		msb = codec.decode(valueContext[b]);
		msb = msb << b;
		b--;
		}
	while (msb == 0 && b >= 0);

	int sgn = 0;
	int rest = 0;
	if(msb != 0)
		{
		sgn = codec.decode(signContext);
		rest = 0;
		while (b >= 0)
			{
			rest |= (codec.decode(valueContext[b]) << b);
			b--;
			}
		}
	if(sgn)
		*nDecodedValue = -(msb+rest);
	else
		*nDecodedValue = (msb+rest);
}

// decode: aac values without sign (using zero context after decoding the msb)
void decodeUnsignedAAC(CAC_QF& codec, int* nDecodedValue, int qstep, CAC* valueContext)
{
	int b;
	b = qstep-1;
	int msb = 0;
	do {
		msb = codec.decode(valueContext[b]);
		msb = msb << b;
		b--;
		}
	while (msb == 0 && b >= 0);
	
	int rest = 0;
	if(msb != 0)
		{
		rest = 0;
		while (b >= 0)
			{
			rest |= (codec.decode(valueContext[b]) << b);
			b--;
			}
		}
	*nDecodedValue = (msb+rest);
}

// decode: aac values with sign
void decodeSignedQuasiAAC(CAC_QF& codec, int* nDecodedValue, int qstep, CAC& signContext, CAC* valueContext)
{
	int b;
	b = qstep-2;
	int msb = 0;
	do {
		msb = codec.decode(valueContext[b]);
		msb = msb << b;
		b--;
		}
	while (msb == 0 && b >= 0);
	
	int sgn = 0;
	int rest = 0;
	if(msb != 0)
		{
		sgn = codec.decode(signContext);
		rest = 0;
		while (b >= 0)
			{
			rest |= (codec.decode() << b);
			b--;
			}
		}
	if(sgn)
		*nDecodedValue = -(msb+rest);
	else
		*nDecodedValue = (msb+rest);
}
