// JWPositionInterpolator.cpp: implementation of the JWPositionInterpolator class.
//
//////////////////////////////////////////////////////////////////////

#include "JWPositionInterpolator.h"
#include "JWTemplArray.h"
#include "JWToken.h"
#include "SFVec3f.h"
#include <float.h>
#include <stdlib.h>

typedef JWArray<double> MFFloat;
typedef JWArray<SFVec3f> MFVec3f;
const double dEPSILON = 1.0E-50;


void MakePointsDouble(JWInterpolator* pInterp1, JWInterpolator* pInterp2);
void EstimateError(
	JWInterpolator* pInterp1, JWInterpolator* pInterp2,
	double& rms, double& peak,
	int nNorm_type = 3);


// sowoo 2001.09.07 (Add FindKeyDigit)
// 1.E-13
//const double dEPSILONERROR = 0.0000000000001;
// 1.e-11
//const double dEPSILONERROR = 0.00000000001;
// 1.e-10
const double dEPSILONERROR = 0.0000000001;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

JWPositionInterpolator::JWPositionInterpolator()
{
	m_min.x = DBL_MAX;
	m_min.y = DBL_MAX;
	m_min.z = DBL_MAX;

	m_max.x = -DBL_MAX;
	m_max.y = -DBL_MAX;
	m_max.z = -DBL_MAX;
}

JWPositionInterpolator::~JWPositionInterpolator()
{

}

void JWPositionInterpolator::Parse(JWToken& token)
{
	double tempDouble;
	SFVec3f tempSFVec3f;

	token.GetNextToken();
	if(token == BRACE_OPEN)
	{
		// key д´.
		token.GetNextToken();
		if(token == key)
		{
			token.GetNextToken();
			if(token == SQUAREBRACKET_OPEN)
			{
				while(true)
				{
					tempDouble = token.GetRealNumber();
					if(token == SQUAREBRACKET_CLOSE)
						break;
					aKeys.Add(tempDouble);
				}
			}
		}

		// KeyValue д´.
		token.GetNextToken();
		if(token == keyValue)
		{
			token.GetNextToken();
			if(token == SQUAREBRACKET_OPEN)
			{
				while(true)
				{
					tempSFVec3f.x = token.GetRealNumber();
					if(token == SQUAREBRACKET_CLOSE)
						break;

					tempSFVec3f.y = token.GetRealNumber();
					tempSFVec3f.z = token.GetRealNumber();

					m_aKeyValues.Add(tempSFVec3f);
				}
			}
		}
	}
}

void JWPositionInterpolator::Write(ofstream& file)
{
	//   ִ Ȯϱ!
	assert(file.is_open());

	// Ŀ indentation . ?

	// 1. DEF Name ִٸ װ 
	if(m_bIsDEF)
		file << "\t\t" << "DEF " << m_strDEF << ' ';

	// 2. PositionInterpolator {
	file << "PositionInterpolator {" << endl;

	// 3. key
	file << "\t\t\t" << "key [" << endl;

	for(int i=0; i<aKeys.GetSize(); i++)
	{
		if(i%5 == 0)
			file << "\t\t\t\t";

		file << aKeys[i];

		if(i != aKeys.GetSize() - 1)
			file << ", ";
		else
		{
			file << endl << "\t\t\t" << " ]" << endl;
			break;
		}

		if(i%5 == 4)
			file << endl;
	}

	// 4. keyValue
	file << "\t\t\t" << "keyValue [" << endl;

	for(i=0; i<m_aKeyValues.GetSize(); i++)
	{
		if(i%2 == 0)
			file << "\t\t\t\t";

		file << m_aKeyValues[i];

		if(i != m_aKeyValues.GetSize() - 1)
			file << ", ";
		else
		{
			file << endl << "\t\t\t" << " ]" << endl;
			break;
		}

		if(i%2 == 1)
			file << endl;
	}

	file << "\t\t" << '}' << endl;
}

ofstream& operator << (ofstream& file, JWPositionInterpolator& pI)
{
	// 1. DEF Name ִٸ װ 
	if(pI.m_bIsDEF)
		file << "DEF " << pI.m_strDEF << ' ';

	// 2. PositionInterpolator {
	file << "OrientationInterpolator {" << endl;

	// 3. key
	file << '\t' << "key [" << endl;

	for(int i=0; i<pI.aKeys.GetSize(); i++)
	{
		if(i%5 == 0)
			file << "\t\t";

		file << pI.aKeys[i];

		if(i != pI.aKeys.GetSize() - 1)
			file << ", ";
		else
		{
			file << " ]" << endl;
			break;
		}

		if(i%5 == 4)
			file << endl;
	}

	// 4. keyValue
	file << '\t' << "keyValue [" << endl;

	for(i=0; i<pI.m_aKeyValues.GetSize(); i++)
	{
		if(i%5 == 0)
			file << "\t\t";

		file << pI.m_aKeyValues[i];

		if(i != pI.m_aKeyValues.GetSize() - 1)
			file << ", ";
		else
		{
			file << " ]" << endl;
			break;
		}

		if(i%5 == 4)
			file << endl;
	}

	return file;
}


double JWPositionInterpolator::Purify(double dThreshold, JWInterpolator*& pInterpolator)
{
	double dDistortion = DBL_MAX;
	double dCurrDistortion, dPrevDistortion;
	double dTempDistortion;

	int size = aKeys.GetSize();

	JWArray<bool> aFlags;
	aFlags.SetSize(size);

	for(int i=1; i<size-1; i++)
		aFlags[i] = false;
	aFlags[0] = aFlags[size-1] = true;

	int nMinPointIndex = -1;
	int nNumSelected = 2;

	//  2 Ͽ distortion Ѵ.
	dDistortion = CalculateDistortion(this, aFlags);
	dPrevDistortion = dDistortion;

	while(dDistortion > dThreshold)
	{
		dPrevDistortion = dDistortion;
		nMinPointIndex = -1;
		dTempDistortion = DBL_MAX;

		for(i=1; i<size-1; i++)
		{
			// 0. ̹ õ ̶, ~~~
			if(aFlags[i] == true)
				continue;

			// 1. error     ִ   Ѵ.
			aFlags[i] = true;
			dCurrDistortion = CalculateDistortion(this, aFlags);
			aFlags[i] = false;

			// 2. Error   ߿   ΰ?
			if(dTempDistortion > dCurrDistortion)
			{
				dTempDistortion = dCurrDistortion;
				nMinPointIndex = i;
			}
		}

		// 2. 1 thresholdŭ error     .
		dDistortion = dTempDistortion;
		aFlags[nMinPointIndex] = true;
		nNumSelected++;
	}

	if((nNumSelected != 2) && ::fabs(dDistortion - dThreshold) > ::fabs(dPrevDistortion - dThreshold))
	{
		aFlags[nMinPointIndex] = false;
		dDistortion = dPrevDistortion;
		nNumSelected--;
	}


	// 3. purify interpolator Ѵ.
	pInterpolator = new JWPositionInterpolator;
	if(m_bIsDEF)
		pInterpolator->SetDEF(m_strDEF);

	int j;
	pInterpolator->SetSize(nNumSelected);
	for(i=0, j=0; i<size; i++)
	{
		if(aFlags[i])
		{
			pInterpolator->SetKey(j, aKeys[i]);
			pInterpolator->SetKeyValue(j, (void*)&m_aKeyValues[i]);
			j++;
		}
	}

	return dDistortion;
}

void JWPositionInterpolator::Add(double key, void* keyValue)
{
	aKeys.Add(key);
	m_aKeyValues.Add(*((SFVec3f*)keyValue));
}

void JWPositionInterpolator::SetKeyFrame(int i, double key, void* keyValue)
{
	aKeys[i] = key;
	m_aKeyValues[i] = *((SFVec3f*)keyValue);
}

double JWPositionInterpolator::CalculateDistortion(
	JWInterpolator* pInterp, JWArray<bool>& aFlags)
{
	// flag Ͽ   Interpolator Ѵ.
	JWPositionInterpolator interpolator;
	JWPositionInterpolator* pPInterp = (JWPositionInterpolator*) pInterp;

	for(int i=0; i<aKeys.GetSize(); i++)
	{
		if(aFlags[i])
			interpolator.Add(pPInterp->aKeys[i], (void*) &(pPInterp->m_aKeyValues[i]));
	}

	//  Interpolator Ͽ  Ѵ.
	return CalculateDistortion(&interpolator);
}

// sowoo 2001.09.11 (Component Selection)
//  ä   ƴ϶, x, y, z  ϳ 
double JWPositionInterpolator::CalculateDistortion(
	JWInterpolator* pInterp,
	JWArray<bool>& aXs,
	JWArray<bool>& aYs,
	JWArray<bool>& aZs
)
{
	// flag Ͽ   Interpolator Ѵ.
	JWPositionInterpolator interpolator;
	JWPositionInterpolator* pPInterp = (JWPositionInterpolator*) pInterp;

	int size = aKeys.GetSize();
	SFVec3f tempSFVec3f;

	// ù ׳ ֱ!!!
	interpolator.Add(pPInterp->aKeys[0], (void*) &(pPInterp->m_aKeyValues[0]));

	for(int i=1; i<size-1; i++)
	{
		//    key frame .
		if(!aXs[i] && !aYs[i] && !aZs[i])
			continue;

		//   ޲پ ־!!!
		if(aXs[i])
			tempSFVec3f.x = pPInterp->m_aKeyValues[i].x;
		else
			tempSFVec3f.x = pPInterp->LinearInterpolation(i, aXs, 0).x;

		if(aYs[i])
			tempSFVec3f.y = pPInterp->m_aKeyValues[i].y;
		else
			tempSFVec3f.y = pPInterp->LinearInterpolation(i, aYs, 1).y;

		if(aZs[i])
			tempSFVec3f.z = pPInterp->m_aKeyValues[i].z;
		else
			tempSFVec3f.z = pPInterp->LinearInterpolation(i, aZs, 2).z;

		//  key, keyValue ִ´.
		interpolator.Add(pPInterp->aKeys[i], (void*) &tempSFVec3f);
	}

	//   ׳ ֱ!!!
	interpolator.Add(pPInterp->aKeys[size-1], (void*) &(pPInterp->m_aKeyValues[size-1]));


	//  Interpolator Ͽ  Ѵ.
	return CalculateDistortion(&interpolator);
}


SFVec3f JWPositionInterpolator::LinearInterpolation(int index, JWArray<bool>& aFlags, int nType)
{
	int nLeft, nRight;
	SFVec3f tempSFVec3f;

	// 1. ʿ  ù° 
	for(int i=index+1; i<aFlags.GetSize(); i++)
	{
		if(aFlags[i])
		{
			nRight = i;
			break;
		}
	}

	// 2. ʿ  ù° 
	for(i=index-1; i>=0; i--)
	{
		if(aFlags[i])
		{
			nLeft = i;
			break;
		}
	}

	switch(nType){
	case 0: // x
		tempSFVec3f.x = 
			(m_aKeyValues[nRight].x - m_aKeyValues[nLeft].x)*(aKeys[index] - aKeys[nLeft])
			/(aKeys[nRight] - aKeys[nLeft]) + m_aKeyValues[nLeft].x;
		tempSFVec3f.y = m_aKeyValues[index].y;
		tempSFVec3f.z = m_aKeyValues[index].z;
		break;

	case 1: // y
		tempSFVec3f.x = m_aKeyValues[index].x;
		tempSFVec3f.y = 
			(m_aKeyValues[nRight].y - m_aKeyValues[nLeft].y)*(aKeys[index] - aKeys[nLeft])
			/(aKeys[nRight] - aKeys[nLeft]) + m_aKeyValues[nLeft].y;
		tempSFVec3f.z = m_aKeyValues[index].z;
		break;

	case 2: // z
		tempSFVec3f.x = m_aKeyValues[index].x;
		tempSFVec3f.y = m_aKeyValues[index].y;
		tempSFVec3f.z = 
			(m_aKeyValues[nRight].z - m_aKeyValues[nLeft].z)*(aKeys[index] - aKeys[nLeft])
			/(aKeys[nRight] - aKeys[nLeft]) + m_aKeyValues[nLeft].z;
		break;

	case 3: // xy
		tempSFVec3f.x = 
			(m_aKeyValues[nRight].x - m_aKeyValues[nLeft].x)*(aKeys[index] - aKeys[nLeft])
			/(aKeys[nRight] - aKeys[nLeft]) + m_aKeyValues[nLeft].x;
		tempSFVec3f.y = 
			(m_aKeyValues[nRight].y - m_aKeyValues[nLeft].y)*(aKeys[index] - aKeys[nLeft])
			/(aKeys[nRight] - aKeys[nLeft]) + m_aKeyValues[nLeft].y;
		tempSFVec3f.z = m_aKeyValues[index].z;
		break;

	case 4: // yz
		tempSFVec3f.x = m_aKeyValues[index].x;
		tempSFVec3f.y = 
			(m_aKeyValues[nRight].y - m_aKeyValues[nLeft].y)*(aKeys[index] - aKeys[nLeft])
			/(aKeys[nRight] - aKeys[nLeft]) + m_aKeyValues[nLeft].y;
		tempSFVec3f.z = 
			(m_aKeyValues[nRight].z - m_aKeyValues[nLeft].z)*(aKeys[index] - aKeys[nLeft])
			/(aKeys[nRight] - aKeys[nLeft]) + m_aKeyValues[nLeft].z;
		break;

	case 5: // zx
		tempSFVec3f.x = 
			(m_aKeyValues[nRight].x - m_aKeyValues[nLeft].x)*(aKeys[index] - aKeys[nLeft])
			/(aKeys[nRight] - aKeys[nLeft]) + m_aKeyValues[nLeft].x;
		tempSFVec3f.y = m_aKeyValues[index].y;
		tempSFVec3f.z = 
			(m_aKeyValues[nRight].z - m_aKeyValues[nLeft].z)*(aKeys[index] - aKeys[nLeft])
			/(aKeys[nRight] - aKeys[nLeft]) + m_aKeyValues[nLeft].z;
		break;

	case 6: // xyz
		tempSFVec3f.x = 
			(m_aKeyValues[nRight].x - m_aKeyValues[nLeft].x)*(aKeys[index] - aKeys[nLeft])
			/(aKeys[nRight] - aKeys[nLeft]) + m_aKeyValues[nLeft].x;
		tempSFVec3f.y = 
			(m_aKeyValues[nRight].y - m_aKeyValues[nLeft].y)*(aKeys[index] - aKeys[nLeft])
			/(aKeys[nRight] - aKeys[nLeft]) + m_aKeyValues[nLeft].y;
		tempSFVec3f.z = 
			(m_aKeyValues[nRight].z - m_aKeyValues[nLeft].z)*(aKeys[index] - aKeys[nLeft])
			/(aKeys[nRight] - aKeys[nLeft]) + m_aKeyValues[nLeft].z;
		break;
	}

	return tempSFVec3f;
}


double JWPositionInterpolator::CalculateDistortion(JWInterpolator* pInterp)
{
	JWPositionInterpolator tempInterpolator1;
	tempInterpolator1.Copy(this);
	JWPositionInterpolator tempInterpolator2;
	tempInterpolator2.Copy(pInterp);


//	tempInterpolator1.CalculateMaxDistanceOfInterval();
//	tempInterpolator1.CalculateTotalLengthOfPath();
//	tempInterpolator1.CalculateTotalLengthOfPathOverBox();

	// 1.  ġ 
	tempInterpolator1.InsertPointWith(&tempInterpolator2);
	tempInterpolator2.InsertPointWith(&tempInterpolator1);


	// 2. ߰  Ѵ.
//	MakePointsDouble(&tempInterpolator1, &tempInterpolator2);


	// 3.   distortionϱ!
	double rms = 0.0, peak = 0.0;
	EstimateError(&tempInterpolator1, &tempInterpolator2, rms, peak, m_nTypeOfError);
	return rms;
}

void JWPositionInterpolator::InsertPointWith(JWInterpolator* pInterpolator)
{
	JWPositionInterpolator& rPositionInterpolator = 
		*((JWPositionInterpolator*) pInterpolator);
	MFFloat& rKeyMe = aKeys;
	MFFloat& rKeyYou = rPositionInterpolator.aKeys;
	SFVec3f interpolatoredKeyValue;
	MFVec3f& rKeyValue = m_aKeyValues;

	int nNumKeyYou = rKeyYou.GetSize();
	bool bIsMoreKey = true;
	int nIndexMe = 0, nIndexYou = 0;

	while(bIsMoreKey)
	{
		//  
		//  key  ̻  ʴ 
		if(nIndexYou >= nNumKeyYou)
		{
			return;
		}

		// ̰   쿡 key ߰ ʰ,
		//  key Ѿ.
		if(::fabs(rKeyMe[nIndexMe] - rKeyYou[nIndexYou]) < dEPSILON )
		{
			nIndexMe++;
			nIndexYou++;
			continue;
		}

		// 
		else if(rKeyMe[nIndexMe] > rKeyYou[nIndexYou])
		{
			if(nIndexMe == 0)
			{
				// ó ϴ ̹Ƿ
				InsertKey(0, rKeyYou[nIndexYou], (void*)&rKeyValue[0]);
				nIndexMe++;
			}
			else
			{
				// interpolator Ͽ ߰  ã .
				InterpolateKeyValue(
					rKeyMe[nIndexMe-1], rKeyMe[nIndexMe], rKeyYou[nIndexYou],
					(void*) &rKeyValue[nIndexMe-1], (void*) &rKeyValue[nIndexMe], (void*)&interpolatoredKeyValue);

				//  KEY Ѵ.
				InsertKey(nIndexMe, rKeyYou[nIndexYou], (void*)&interpolatoredKeyValue);
				nIndexMe++;
			}

			// ε ϳ Ͽ,  key Ѵ.
			nIndexYou++;
		}

		// 
		else if(rKeyMe[nIndexMe] < rKeyYou[nIndexYou])
		{
			//  key Ѿ.
			nIndexMe++;

			// Me  ̻ Key   ʴٸ,
			if(nIndexMe >= rKeyMe.GetSize())
			{
				//  ϴ ̹Ƿ
				for(; nIndexYou<nNumKeyYou; nIndexYou++)
					InsertKey(nIndexMe, rKeyYou[nIndexYou], (void*)&rKeyValue[nIndexMe-1]);
			}
		}
	} // end of while

}


void JWPositionInterpolator::InterpolateKeyValue(
	double key1, double key2, double key, 
	void* kv1, void* kv2, void* keyValue)
{
	SFVec3f& keyValue1 = *(SFVec3f*)kv1;
	SFVec3f& keyValue2 = *(SFVec3f*)kv2;
	SFVec3f& interpolatedKeyValue = *(SFVec3f*)keyValue;
	double weight = (key - key1)/(key2 - key1);

	interpolatedKeyValue.x = (keyValue2.x - keyValue1.x)*weight + keyValue1.x;
	interpolatedKeyValue.y = (keyValue2.y - keyValue1.y)*weight + keyValue1.y;
	interpolatedKeyValue.z = (keyValue2.z - keyValue1.z)*weight + keyValue1.z;
}


void JWPositionInterpolator::InsertKey(int index, double key, void* kv)
{
	SFVec3f& keyValue = *(SFVec3f*)kv;
	MFVec3f& rKeyValues = m_aKeyValues;

	if(index >= aKeys.GetSize())
	{
		aKeys.Add(key);
		rKeyValues.Add(keyValue);
	}
	else
	{
		aKeys.InsertAt(index, key);
		rKeyValues.InsertAt(index, keyValue);
	}
}



void JWPositionInterpolator::Copy(JWInterpolator* pInterp)
{
	aKeys.Copy(pInterp->GetKeys());
	m_aKeyValues.Copy(*((JWArray<SFVec3f>*) pInterp->GetKeyValues()));

	m_min = ((JWPositionInterpolator*)pInterp)->m_min;
	m_max = ((JWPositionInterpolator*)pInterp)->m_max;
}


void EstimateError(
	JWInterpolator* pInterp1, JWInterpolator* pInterp2,
	double& rms, double& peak,
	int nNorm_type)
{
	double a, b, h;
	double abs_a, abs_b;
	int size = pInterp1->GetKeys().GetSize();
	double dArea, dTotalArea_X = 0.0, dTotalArea_Y = 0.0, dTotalArea_Z = 0.0;
	double dPeakArea = -DBL_MAX;
	double
		dMin_X = DBL_MAX, dMax_X = -DBL_MAX,
		dMin_Y = DBL_MAX, dMax_Y = -DBL_MAX,
		dMin_Z = DBL_MAX, dMax_Z = -DBL_MAX;

	MFFloat& rKey1 = pInterp1->GetKeys();
	MFFloat& rKey2 = pInterp2->GetKeys();
	MFVec3f& rKeyValues1 = *((MFVec3f*) pInterp1->GetKeyValues());
	MFVec3f& rKeyValues2 = *((MFVec3f*) pInterp2->GetKeyValues());


	for(int i=0; i<size; i++)
	{
		/////////////////////////////////////////////
		// X, Y, Z Min/Max ã´.
		/////////////////////////////////////////////

		if(dMin_X > rKeyValues1[i].x)
			dMin_X = rKeyValues1[i].x;
		if(dMax_X < rKeyValues1[i].x)
			dMax_X = rKeyValues1[i].x;

		if(dMin_Y > rKeyValues1[i].y)
			dMin_Y = rKeyValues1[i].y;
		if(dMax_Y < rKeyValues1[i].y)
			dMax_Y = rKeyValues1[i].y;

		if(dMin_Z > rKeyValues1[i].z)
			dMin_Z = rKeyValues1[i].z;
		if(dMax_Z < rKeyValues1[i].z)
			dMax_Z = rKeyValues1[i].z;

		// Min/Max ϴ κп .
		if(i == size-1)
			break;


		/////////////////////////////////////////////
		// ̸ Ѵ.
		/////////////////////////////////////////////

		h = rKey1[i+1] - rKey1[i];

		// ̰ ٸ,  !
		if(h == 0.0)
			continue;


		/////////////////////////////////////////////
		// X
		/////////////////////////////////////////////

		a = rKeyValues1[i].x - rKeyValues2[i].x;
		b = rKeyValues1[i+1].x - rKeyValues2[i+1].x;

		abs_a = fabs(a);
		abs_b = fabs(b);

		if(a == 0.0 && b == 0.0)
			dArea = 0.0;
		else if(a*b > 0)
			// ٸ
			dArea = 0.5*(abs_a + abs_b)*h;
		else
			// ġ
			dArea = 0.5*(abs_a*abs_a + abs_b*abs_b)*h / (abs_a + abs_b);

		if(dPeakArea < abs_a)
			dPeakArea = abs_a;
		if(dPeakArea < abs_b)
			dPeakArea = abs_b;

		dTotalArea_X += dArea;


		/////////////////////////////////////////////
		// Y
		/////////////////////////////////////////////

		a = rKeyValues1[i].y - rKeyValues2[i].y;
		b = rKeyValues1[i+1].y - rKeyValues2[i+1].y;

		abs_a = fabs(a);
		abs_b = fabs(b);

		if(a == 0.0 && b == 0.0)
			dArea = 0.0;
		else if(a*b > 0)
			// ٸ
			dArea = 0.5*(abs_a + abs_b)*h;
		else
			// ġ
			dArea = 0.5*(abs_a*abs_a + abs_b*abs_b)*h/(abs_a+abs_b);

		if(dPeakArea < abs_a)
			dPeakArea = abs_a;
		if(dPeakArea < abs_b)
			dPeakArea = abs_b;

		dTotalArea_Y += dArea;


		/////////////////////////////////////////////
		// Z
		/////////////////////////////////////////////

		a = rKeyValues1[i].z - rKeyValues2[i].z;
		b = rKeyValues1[i+1].z - rKeyValues2[i+1].z;

		abs_a = fabs(a);
		abs_b = fabs(b);

		if(dPeakArea < abs_a)
			dPeakArea = abs_a;
		if(dPeakArea < abs_b)
			dPeakArea = abs_b;

		if(a == 0.0 && b == 0.0)
			dArea = 0.0;
		else if(a*b > 0)
			// ٸ
			dArea = 0.5*(abs_a+abs_b)*h;
		else
			// ġ
			dArea = 0.5*(abs_a*abs_a + abs_b*abs_b)*h/(abs_a+abs_b);

		if(dPeakArea < dArea)
			dPeakArea = dArea;

		dTotalArea_Z += dArea;
	}



	/////////////////////////////////////////////
	// Normalize
	/////////////////////////////////////////////
	double
		dDiff_Max = fabs(dMax_X - dMin_X),
		dDiff_Y = fabs(dMax_Y - dMin_Y),
		dDiff_Z = fabs(dMax_Z - dMin_Z);

	if(dDiff_Max < dDiff_Y)
		dDiff_Max = dDiff_Y;
	if(dDiff_Max < dDiff_Z)
		dDiff_Max = dDiff_Z;

	double dNormFactor = 3 * (rKey1[rKey1.GetSize()-1] - rKey1[0]) * dDiff_Max;

	rms = (dTotalArea_X + dTotalArea_Y + dTotalArea_Z) / dNormFactor;
	peak = dPeakArea / dDiff_Max;
}

void MakePointsDouble(JWInterpolator* pInterp1, JWInterpolator* pInterp2)
{
	JWPositionInterpolator& rInterp1 = *(JWPositionInterpolator*)pInterp1;
	JWPositionInterpolator& rInterp2 = *(JWPositionInterpolator*)pInterp2;

	MFFloat& rKey1 = rInterp1.GetKeys();
	MFFloat& rKey2 = rInterp2.GetKeys();
	MFVec3f& rKeyValue1 = *((MFVec3f*)rInterp1.GetKeyValues());
	MFVec3f& rKeyValue2 = *((MFVec3f*)rInterp2.GetKeyValues());

	if(rKey1.GetSize() != rKey2.GetSize())
	{
		cout << "Key  ٸ." << endl;
		cout << "Error   ." << endl;
		return;
	}

	bool bIsMore = true;
	int i=0;
	double a, b;
	double key;
	SFVec3f interpolatedKeyValue;

	while(bIsMore)
	{
		//  !
		if(i >= rKey1.GetSize()-1)
			break;

		// 1. a, b Ѵ.
		a = rKeyValue1[i].GetDistance(rKeyValue2[i]);
		b = rKeyValue1[i+1].GetDistance(rKeyValue2[i+1]);

		// 2.   ߰ Ѵ.
		key = a*(rKey1[i+1] - rKey1[i])/(a+b) + rKey1[i];


		// ࿡ ߰  i, i+1 °  ٸ  !!!
		// !!!!
			//  2 key    ־ٸ??????? ->  õ!
		if(::fabs(key - rKey1[i]) < dEPSILON)
		{
			i++;
			continue;
		}
		else if(::fabs(key - rKey1[i+1]) < dEPSILON)
		{
			i++;
			continue;
		}

		// 2.1 Encoded Path
		rInterp1.InterpolateKeyValue(
			rKey1[i], rKey1[i+1], key,
			(void*) &rKeyValue1[i], (void*) &rKeyValue1[i+1], (void*) &interpolatedKeyValue
		);
		rInterp1.InsertKey(i+1, key, (void*) &interpolatedKeyValue);

		// 2.2 Decoded Path
		rInterp2.InterpolateKeyValue(
			rKey2[i], rKey2[i+1], key,
			(void*) &rKeyValue2[i], (void*) &rKeyValue2[i+1], (void*) &interpolatedKeyValue
		);
		rInterp2.InsertKey(i+1, key, (void*) &interpolatedKeyValue);

		// 3.   
		i += 2;
	}

}

SFVec3f& JWPositionInterpolator::GetMin()
{
	for(int i=0; i<aKeys.GetSize(); i++)
	{
		// ִ/ּҸ ϴ κ
		if(m_min.x > m_aKeyValues[i].x)
			m_min.x = m_aKeyValues[i].x;
		if(m_min.y > m_aKeyValues[i].y)
			m_min.y = m_aKeyValues[i].y;
		if(m_min.z > m_aKeyValues[i].z)
			m_min.z = m_aKeyValues[i].z;
	}

	return m_min;
}

SFVec3f& JWPositionInterpolator::GetMax()
{
	for(int i=0; i<aKeys.GetSize(); i++)
	{
		// ִ/ּҸ ϴ κ
		if(m_max.x < m_aKeyValues[i].x)
			m_max.x = m_aKeyValues[i].x;
		if(m_max.y < m_aKeyValues[i].y)
			m_max.y = m_aKeyValues[i].y;
		if(m_max.z < m_aKeyValues[i].z)
			m_max.z = m_aKeyValues[i].z;
	}

	return m_max;
}


double JWPositionInterpolator::CalculateMaxDistanceOfInterval()
{
	double dMaxDistance = -DBL_MAX;
	double dCurDistance, tempDBL;
	MFVec3f& raKeyValues = m_aKeyValues;
	MFFloat& raKeys = aKeys;

	for(int i=1; i<raKeys.GetSize(); i++)
	{
		dCurDistance = raKeyValues[i-1].GetDistance(raKeyValues[i]);
		tempDBL = raKeys[i] - raKeys[i-1];
		if(tempDBL == 0.0)
			dCurDistance = 0.0;
		else
			dCurDistance /= tempDBL;
		tempDBL = (raKeys[raKeys.GetSize()-1] - raKeys[0])/((double)raKeys.GetSize()-1);
		dCurDistance *= tempDBL;

		if(dMaxDistance < dCurDistance)
		{
			dMaxDistance = dCurDistance;
		}
	}


	m_dMaxDistanceOfInterval =  dMaxDistance;

	return dMaxDistance;
}


// sowoo 2001.09.04 (Modify Key Selection)
void JWPositionInterpolator::FindKeyFlag(
	double dThreshold[], int nNumThreshold, 
	JWArray<JWArray<int> >& aXs, 
	JWArray<JWArray<int> >& aYs, 
	JWArray<JWArray<int> >& aZs
)
{
	double dDistortion = DBL_MAX;
	double dCurrDistortion, dPrevDistortion;
	double dTempDistortion;

	int size = aKeys.GetSize();

	JWArray<bool> aFlags;
	aFlags.SetSize(size);

	for(int i=1; i<size-1; i++)
		aFlags[i] = false;
	aFlags[0] = aFlags[size-1] = true;

	int nMinPointIndex = -1;
	int nNumSelected = 2;

	//  2 Ͽ distortion Ѵ.
	dDistortion = CalculateDistortion(this, aFlags);
	dPrevDistortion = dDistortion;

	int nT = 0;

	while(dDistortion < dThreshold[nT])
	{
		// 3.1 Flag Ѵ.
		for(i=0; i<size; i++)
		{
			aXs[nT][i] = aYs[nT][i] = aZs[nT][i] = (aFlags[i] == true)? 1: 0;
		}

		nT++;

		if(nT >= nNumThreshold)
			break;
	}

	while(dDistortion > dThreshold[nNumThreshold-1])
	{
		dPrevDistortion = dDistortion;
		nMinPointIndex = -1;
		dTempDistortion = DBL_MAX;

		for(i=1; i<size-1; i++)
		{
			// 0. ̹ õ ̶, ~~~
			if(aFlags[i] == true)
				continue;

			// 1. error     ִ   Ѵ.
			aFlags[i] = true;
			dCurrDistortion = CalculateDistortion(this, aFlags);
			aFlags[i] = false;

			// 2. Error   ߿   ΰ?
			if(dTempDistortion > dCurrDistortion)
			{
				dTempDistortion = dCurrDistortion;
				nMinPointIndex = i;
			}
		}

		// 2. 1 thresholdŭ error     .
		dDistortion = dTempDistortion;
		aFlags[nMinPointIndex] = true;
		nNumSelected++;

		// 3.  threshold ؾ ϴ°?
		if(nT >= nNumThreshold)
			break;

		while(dDistortion <= dThreshold[nT])
		{
			// 3.1 Flag Ѵ.
			for(i=0; i<size; i++)
			{
				aXs[nT][i] = aYs[nT][i] = aZs[nT][i] = (aFlags[i] == true)? 1: 0;
			}

			nT++;

			if(nT >= nNumThreshold)
				break;
		}
	}


	if(nNumSelected == 2)
	{
		for(nT=0; nT<nNumThreshold; nT++)
		{
			for(i=0; i<size; i++)
			{
				aXs[nT][i] = aYs[nT][i] = aZs[nT][i] = (aFlags[i] == true)? 1: 0;
			}
		}
	}


	// sowoo 2001.11.08 (Threshold "0" ϶~!!)
	// Threshold "0" 쿡  KeyValue õǵ Ѵ.
	if((nNumSelected != size) && (dThreshold[nNumThreshold-1] == 0.0))
	{
		for(i=0; i<size; i++)
		{
			aXs[nNumThreshold-1][i] = 1;
			aYs[nNumThreshold-1][i] = 1;
			aZs[nNumThreshold-1][i] = 1;
		}
	}


	// sowoo 2001.09.07 (Component Selection)
	// 4.  component selectionѴ.
/*	for(nT=0; nT<nNumThreshold; nT++)
	{
		// 4.1 X
		FindKeyFlag_OneCompnent(0, aXs[nT]);
		// 4.2 X
		FindKeyFlag_OneCompnent(1, aYs[nT]);
		// 4.3 X
		FindKeyFlag_OneCompnent(2, aZs[nT]);

		cout << endl;
	}
*/
}


// sowoo 2001.09.04 (Modify Key Selection)
void JWPositionInterpolator::FindKeyFlag_ValidDigit(
	double dThreshold[], int nNumThreshold, 
	JWArray<JWArray<int> >& aXs, 
	JWArray<JWArray<int> >& aYs, 
	JWArray<JWArray<int> >& aZs
)
{
	double dDistortion = DBL_MAX;
	double dCurrDistortion, dPrevDistortion;
	double dTempDistortion;

	int size = aKeys.GetSize();

	JWArray<bool> aFlags;
	aFlags.SetSize(size);

	for(int i=1; i<size-1; i++)
		aFlags[i] = false;
	aFlags[0] = aFlags[size-1] = true;

	int nMinPointIndex = -1;
	int nNumSelected = 2;

	//  2 Ͽ distortion Ѵ.
	dDistortion = CalculateDistortion(this, aFlags);
	dPrevDistortion = dDistortion;

	int nT = 0;

	while(dDistortion < dThreshold[nT])
	{
		// 3.1 Flag Ѵ.
		for(i=0; i<size; i++)
		{
			aXs[nT][i] = aYs[nT][i] = aZs[nT][i] = (aFlags[i] == true)? 1: 0;
		}

		nT++;

		if(nT >= nNumThreshold)
			break;
	}

	while(dDistortion > dThreshold[nNumThreshold-1])
	{
		dPrevDistortion = dDistortion;
		nMinPointIndex = -1;
		dTempDistortion = DBL_MAX;

		for(i=1; i<size-1; i++)
		{
			// 0. ̹ õ ̶, ~~~
			if(aFlags[i] == true)
				continue;

			// 1. error     ִ   Ѵ.
			aFlags[i] = true;
			dCurrDistortion = CalculateDistortion(this, aFlags);
			aFlags[i] = false;

			// 2. Error   ߿   ΰ?
			if(dTempDistortion > dCurrDistortion)
			{
				dTempDistortion = dCurrDistortion;
				nMinPointIndex = i;
			}
		}

		// 2. 1 thresholdŭ error     .
		dDistortion = dTempDistortion;
		aFlags[nMinPointIndex] = true;
		nNumSelected++;

		// 3.  threshold ؾ ϴ°?
		if(nT >= nNumThreshold)
			break;

		while(dDistortion <= dThreshold[nT])
		{
			// 3.1 Flag Ѵ.
			for(i=0; i<size; i++)
			{
				aXs[nT][i] = aYs[nT][i] = aZs[nT][i] = (aFlags[i] == true)? 1: 0;
			}

			nT++;

			if(nT >= nNumThreshold)
				break;
		}
	}


	if(nNumSelected == 2)
	{
		for(nT=0; nT<nNumThreshold; nT++)
		{
			for(i=0; i<size; i++)
			{
				aXs[nT][i] = aYs[nT][i] = aZs[nT][i] = (aFlags[i] == true)? 1: 0;
			}
		}
	}

	// sowoo 2001.09.07 (Component Selection)
	// 4.  component selectionѴ.
	for(nT=0; nT<nNumThreshold; nT++)
	{
		// 4.1 X
		FindKeyFlag_OneCompnent(0, aXs[nT]);
		// 4.2 X
		FindKeyFlag_OneCompnent(1, aYs[nT]);
		// 4.3 X
		FindKeyFlag_OneCompnent(2, aZs[nT]);

		cout << endl;
	}
}



// sowoo 2001.09.11 (Component Selection)
void JWPositionInterpolator::FindKeyFlag_EachComponent(
	double dThreshold[], int nNumThreshold, 
	JWArray<JWArray<int> >& aXs,
	JWArray<JWArray<int> >& aYs,
	JWArray<JWArray<int> >& aZs
)
{
	double dDistortion = DBL_MAX;
	double dCurrDistortion, dCurrDistortionTemp, dPrevDistortion;
	double dTempDistortion;


	int size = aKeys.GetSize();

	// 1. Flag Ͽ ´.
	JWArray<bool> aFlagXs, aFlagYs, aFlagZs;
	aFlagXs.SetSize(size);
	aFlagYs.SetSize(size);
	aFlagZs.SetSize(size);

	for(int i=1; i<size-1; i++)
		aFlagXs[i] = aFlagYs[i] = aFlagZs[i] = false;

	aFlagXs[0] = aFlagXs[size-1] = true;
	aFlagYs[0] = aFlagYs[size-1] = true;
	aFlagZs[0] = aFlagZs[size-1] = true;

	int nMinPointIndex = -1;
	int nMinType, nType;
	int nNumSelected = 2;

	bool bX, bY, bZ;

	// 2.  ó  2 Ͽ distortion Ѵ.
	dDistortion = CalculateDistortion(this, aFlagXs, aFlagYs, aFlagZs);
	dPrevDistortion = dDistortion;

	int nT = 0;


	while(dDistortion < dThreshold[nT])
	{
		// 3.1 Flag Ѵ.
		for(i=0; i<size; i++)
		{
			aXs[nT][i] = (aFlagXs[i] == true)? 1: 0;
			aYs[nT][i] = (aFlagYs[i] == true)? 1: 0;
			aZs[nT][i] = (aFlagZs[i] == true)? 1: 0;
		}

		nT++;

		if(nT >= nNumThreshold)
			break;
	}

	while(dDistortion > dThreshold[nNumThreshold-1])
	{
		dPrevDistortion = dDistortion;
		nMinPointIndex = -1;
		nMinType = -1;
		dTempDistortion = DBL_MAX;

		for(i=1; i<size-1; i++)
		{
			// 0. ̹  õ ̶, ~~~
			if((aFlagXs[i] == true) && (aFlagYs[i] == true) && (aFlagZs[i] == true))
				continue;

			// 0.1   õǾ Ͽ ϱ  Ƿ 
			//   .
			// ??  Ҵ, ٽ 󺹱ϴ !!!
			bX = aFlagXs[i];
			bY = aFlagYs[i];
			bZ = aFlagZs[i];

			dCurrDistortion = DBL_MAX;


			// 1. error     ִ   Ѵ.
			// ̹    ؼ  ؾ ϳ?
			// 1.0 'x'
			aFlagXs[i] = true;
			dCurrDistortionTemp = CalculateDistortion(this, aFlagXs, aFlagYs, aFlagZs);
			aFlagXs[i] = false;

			if(dCurrDistortion > dCurrDistortionTemp)
			{
				dCurrDistortion = dCurrDistortionTemp;
				nType = 0;
			}

			// 1.1 'y'
			aFlagYs[i] = true;
			dCurrDistortionTemp = CalculateDistortion(this, aFlagXs, aFlagYs, aFlagZs);
			aFlagYs[i] = false;

			if(dCurrDistortion > dCurrDistortionTemp)
			{
				dCurrDistortion = dCurrDistortionTemp;
				nType = 1;
			}

			// 1.2 'z'
			aFlagZs[i] = true;
			dCurrDistortionTemp = CalculateDistortion(this, aFlagXs, aFlagYs, aFlagZs);
			aFlagZs[i] = false;

			if(dCurrDistortion > dCurrDistortionTemp)
			{
				dCurrDistortion = dCurrDistortionTemp;
				nType = 2;
			}

			// 1.3 'xy'
			aFlagXs[i] = true; aFlagYs[i] = true;
			dCurrDistortionTemp = CalculateDistortion(this, aFlagXs, aFlagYs, aFlagZs);
			aFlagXs[i] = false; aFlagYs[i] = false;

			if(dCurrDistortion > dCurrDistortionTemp)
			{
				dCurrDistortion = dCurrDistortionTemp;
				nType = 3;
			}

			// 1.4 'yz'
			aFlagYs[i] = true; aFlagZs[i] = true;
			dCurrDistortionTemp = CalculateDistortion(this, aFlagXs, aFlagYs, aFlagZs);
			aFlagYs[i] = false; aFlagZs[i] = false;

			if(dCurrDistortion > dCurrDistortionTemp)
			{
				dCurrDistortion = dCurrDistortionTemp;
				nType = 4;
			}

			// 1.5 'zx'
			aFlagXs[i] = true; aFlagZs[i] = true;
			dCurrDistortionTemp = CalculateDistortion(this, aFlagXs, aFlagYs, aFlagZs);
			aFlagXs[i] = false; aFlagZs[i] = false;

			if(dCurrDistortion > dCurrDistortionTemp)
			{
				dCurrDistortion = dCurrDistortionTemp;
				nType = 5;
			}

			// 1.6 'xyz'
			aFlagXs[i] = true; aFlagYs[i] = true; aFlagZs[i] = true;
			dCurrDistortionTemp = CalculateDistortion(this, aFlagXs, aFlagYs, aFlagZs);
			aFlagXs[i] = false; aFlagYs[i] = false; aFlagZs[i] = false;

			if(dCurrDistortion > dCurrDistortionTemp)
			{
				dCurrDistortion = dCurrDistortionTemp;
				nType = 6;
			}


			// 󺹱Ѵ.
			aFlagXs[i] = bX;
			aFlagYs[i] = bY;
			aFlagZs[i] = bZ;

			// 2. Error   ߿   ΰ?
			if(dTempDistortion > dCurrDistortion)
			{
				// ̹ õ    error    Ѵٸ?
				// װ  ϰ ؾ Ѵ.
				switch(nType){
				case 0: //x
					if(aFlagXs[i])
						continue;
					break;
				case 1: //y
					if(aFlagYs[i])
						continue;
					break;
				case 2: //z
					if(aFlagXs[i])
						continue;
					break;
				case 3: //xy
					if(aFlagXs[i] && aFlagYs[i])
						continue;
					break;
				case 4: //yz
					if(aFlagYs[i] && aFlagZs[i])
						continue;
					break;
				case 5: //zx
					if(aFlagXs[i] && aFlagZs[i])
						continue;
					break;
				case 6: // xyz
					// ̰    ƴϴ.
					break;
				}

				dTempDistortion = dCurrDistortion;
				nMinPointIndex = i;
				nMinType = nType;
			}
		}

		// 2. 1 thresholdŭ error     .
		dDistortion = dTempDistortion;

		switch(nMinType){
		case 0: //x
			aFlagXs[nMinPointIndex] = true;
			break;
		case 1: //y
			aFlagYs[nMinPointIndex] = true;
			break;
		case 2: //z
			aFlagZs[nMinPointIndex] = true;
			break;
		case 3: //xy
			aFlagXs[nMinPointIndex] = true;
			aFlagYs[nMinPointIndex] = true;
			break;
		case 4: // yz
			aFlagYs[nMinPointIndex] = true;
			aFlagZs[nMinPointIndex] = true;
			break;
		case 5: // zx
			aFlagXs[nMinPointIndex] = true;
			aFlagZs[nMinPointIndex] = true;
			break;
		case 6: // xyz
			aFlagXs[nMinPointIndex] = true;
			aFlagYs[nMinPointIndex] = true;
			aFlagZs[nMinPointIndex] = true;
			break;
		}

		nNumSelected++;

		// 3.  threshold ؾ ϴ°?
		if(nT >= nNumThreshold)
			break;

		while(dDistortion <= dThreshold[nT])
		{
			// 3.1 Flag Ѵ.
			for(i=0; i<size; i++)
			{
				aXs[nT][i] = (aFlagXs[i] == true)? 1: 0;
				aYs[nT][i] = (aFlagYs[i] == true)? 1: 0;
				aZs[nT][i] = (aFlagZs[i] == true)? 1: 0;
			}

			nT++;

			if(nT >= nNumThreshold)
				break;
		}
	}


	if(nNumSelected == 2)
	{
		for(nT=0; nT<nNumThreshold; nT++)
		{
			for(i=0; i<size; i++)
			{
				aXs[nT][i] = (aFlagXs[i] == true)? 1: 0;
				aYs[nT][i] = (aFlagYs[i] == true)? 1: 0;
				aZs[nT][i] = (aFlagZs[i] == true)? 1: 0;
			}
		}
	}
}

// sowoo 2001.09.11 (Component Selection)
void JWPositionInterpolator::FindKeyFlag_EachComponent_Updated(
	double dThreshold[], int nNumThreshold, 
	JWArray<JWArray<int> >& aXs,
	JWArray<JWArray<int> >& aYs,
	JWArray<JWArray<int> >& aZs
)
{
	double dDistortion = DBL_MAX;
	double dCurrDistortion, dCurrDistortionTemp, dPrevDistortion;
	double dTempDistortion;


	int size = aKeys.GetSize();

	// 1. Flag Ͽ ´.
	JWArray<bool> aFlagXs, aFlagYs, aFlagZs;
	aFlagXs.SetSize(size);
	aFlagYs.SetSize(size);
	aFlagZs.SetSize(size);

	for(int i=1; i<size-1; i++)
		aFlagXs[i] = aFlagYs[i] = aFlagZs[i] = false;

	aFlagXs[0] = aFlagXs[size-1] = true;
	aFlagYs[0] = aFlagYs[size-1] = true;
	aFlagZs[0] = aFlagZs[size-1] = true;

	int nMinPointIndex = -1;
	int nMinType, nType;
	int nNumSelected_X = 2, nNumSelected_Y = 2, nNumSelected_Z = 2;

	bool bX, bY, bZ;

	// 2.  ó  2 Ͽ distortion Ѵ.
	dDistortion = CalculateDistortion(this, aFlagXs, aFlagYs, aFlagZs);
	dPrevDistortion = dDistortion;

	int nT = 0;


	while(dDistortion < dThreshold[nT])
	{
		// 3.1 Flag Ѵ.
		for(i=0; i<size; i++)
		{
			aXs[nT][i] = (aFlagXs[i] == true)? 1: 0;
			aYs[nT][i] = (aFlagYs[i] == true)? 1: 0;
			aZs[nT][i] = (aFlagZs[i] == true)? 1: 0;
		}

		nT++;

		if(nT >= nNumThreshold)
			break;
	}

	while(dDistortion > dThreshold[nNumThreshold-1])
	{
		dPrevDistortion = dDistortion;
		nMinPointIndex = -1;
		nMinType = -1;
		dTempDistortion = DBL_MAX;

		for(i=1; i<size-1; i++)
		{
			// 0. ̹  õ ̶, ~~~
			if((aFlagXs[i] == true) && (aFlagYs[i] == true) && (aFlagZs[i] == true))
				continue;

			// 0.1   õǾ Ͽ ϱ  Ƿ 
			//   .
			// ??  Ҵ, ٽ 󺹱ϴ !!!
			bX = aFlagXs[i];
			bY = aFlagYs[i];
			bZ = aFlagZs[i];

			dCurrDistortion = DBL_MAX;


			// 1. error     ִ   Ѵ.
			// ̹    ؼ  ؾ ϳ?
			// 1.0 'x'
			if(!bX)
			{
				aFlagXs[i] = true;
				dCurrDistortionTemp = CalculateDistortion(this, aFlagXs, aFlagYs, aFlagZs);
				aFlagXs[i] = false;

				if(dCurrDistortion > dCurrDistortionTemp)
				{
					dCurrDistortion = dCurrDistortionTemp;
					nType = 0;
				}
			}

			// 1.1 'y'
			if(!bY)
			{
				aFlagYs[i] = true;
				dCurrDistortionTemp = CalculateDistortion(this, aFlagXs, aFlagYs, aFlagZs);
				aFlagYs[i] = false;

				if(dCurrDistortion > dCurrDistortionTemp)
				{
					dCurrDistortion = dCurrDistortionTemp;
					nType = 1;
				}
			}

			// 1.2 'z'
			if(!bZ)
			{
				aFlagZs[i] = true;
				dCurrDistortionTemp = CalculateDistortion(this, aFlagXs, aFlagYs, aFlagZs);
				aFlagZs[i] = false;

				if(dCurrDistortion > dCurrDistortionTemp)
				{
					dCurrDistortion = dCurrDistortionTemp;
					nType = 2;
				}
			}


			// 󺹱Ѵ.
			aFlagXs[i] = bX;
			aFlagYs[i] = bY;
			aFlagZs[i] = bZ;

			// 2. Error   ߿   ΰ?
			if(dTempDistortion > dCurrDistortion)
			{
				// ̹ õ    error    Ѵٸ?
				// װ  ϰ ؾ Ѵ.
				switch(nType){
				case 0: //x
					if(aFlagXs[i])
						continue;
					break;
				case 1: //y
					if(aFlagYs[i])
						continue;
					break;
				case 2: //z
					if(aFlagZs[i])
						continue;
					break;
				}

				dTempDistortion = dCurrDistortion;
				nMinPointIndex = i;
				nMinType = nType;
			}
		}

		// 2. 1 thresholdŭ error     .
		dDistortion = dTempDistortion;

		switch(nMinType){
		case 0: //x
			nNumSelected_X++;
			aFlagXs[nMinPointIndex] = true;
			break;
		case 1: //y
			nNumSelected_Y++;
			aFlagYs[nMinPointIndex] = true;
			break;
		case 2: //z
			nNumSelected_Z++;
			aFlagZs[nMinPointIndex] = true;
			break;
		}


		// 3.  threshold ؾ ϴ°?
		if(nT >= nNumThreshold)
			break;

		while(dDistortion <= dThreshold[nT])
		{
			// 3.1 Flag Ѵ.
			for(i=0; i<size; i++)
			{
				aXs[nT][i] = (aFlagXs[i] == true)? 1: 0;
				aYs[nT][i] = (aFlagYs[i] == true)? 1: 0;
				aZs[nT][i] = (aFlagZs[i] == true)? 1: 0;
			}

			nT++;

			if(nT >= nNumThreshold)
				break;
		}
	}


/*
//  ̰   ܴ!!!
	if(nNumSelected == 2)
	{
		for(nT=0; nT<nNumThreshold; nT++)
		{
			for(i=0; i<size; i++)
			{
				aXs[nT][i] = (aFlagXs[i] == true)? 1: 0;
				aYs[nT][i] = (aFlagYs[i] == true)? 1: 0;
				aZs[nT][i] = (aFlagZs[i] == true)? 1: 0;
			}
		}
	}
*/
}

void JWPositionInterpolator::FindKeyFlag_OneCompnent(int nWhich, JWArray<int>& aFlags)
{
	int i;
	int i1, i2, im;
	int size = aFlags.GetSize();

	double x, y, x1, y1, x2, y2, dY;


	switch(nWhich) {
	case 0: // X
		i = 0;
		while(i<size-1)
		{
			// P1 ã´.
			while(aFlags[i] != 1)
			{
				i++;
				if(i >= size)
					return;
			}
			i1 = i;

			// Pm ã´.
			do {
				i++;
				if(i >= size)
					return;
			} while(aFlags[i] != 1);
			im = i;

			// P2 ã´.
			do {
				i++;
				if(i >= size)
					return;
			} while(aFlags[i] != 1);
			i2 = i;


			x1 = aKeys[i1];
			y1 = m_aKeyValues[i1].x;
			x2 = aKeys[i2];
			y2 = m_aKeyValues[i2].x;

			x = aKeys[im];
			y = m_aKeyValues[im].x;

			dY = (y1-y2)*(x-x1)/(x1-x2) + y1;

			if(dY - y == 0.0)
			{
				//    !
				aFlags[im] = 0;
				i = i1;

				cout << "x";
			}
			else
			{
				i = im;
			}
		}

		break;

	case 1: // Y
		i = 0;
		while(i<size-1)
		{
			// P1 ã´.
			while(aFlags[i] != 1)
			{
				i++;
				if(i >= size)
					return;
			}
			i1 = i;

			// Pm ã´.
			do {
				i++;
				if(i >= size)
					return;
			} while(aFlags[i] != 1);
			im = i;

			// P2 ã´.
			do {
				i++;
				if(i >= size)
					return;
			} while(aFlags[i] != 1);
			i2 = i;


			x1 = aKeys[i1];
			y1 = m_aKeyValues[i1].y;
			x2 = aKeys[i2];
			y2 = m_aKeyValues[i2].y;

			x = aKeys[im];
			y = m_aKeyValues[im].y;

			dY = (y1-y2)*(x-x1)/(x1-x2) + y1;

			if(dY - y == 0.0)
			{
				//    !
				aFlags[im] = 0;
				i = i1;

				cout << "y";
			}
			else
			{
				i = im;
			}
		}
		break;

	case 2: // Z
		i = 0;
		while(i<size-1)
		{
			// P1 ã´.
			while(aFlags[i] != 1)
			{
				i++;
				if(i >= size)
					return;
			}
			i1 = i;

			// Pm ã´.
			do {
				i++;
				if(i >= size)
					return;
			} while(aFlags[i] != 1);
			im = i;

			// P2 ã´.
			do {
				i++;
				if(i >= size)
					return;
			} while(aFlags[i] != 1);
			i2 = i;


			x1 = aKeys[i1];
			y1 = m_aKeyValues[i1].z;
			x2 = aKeys[i2];
			y2 = m_aKeyValues[i2].z;

			x = aKeys[im];
			y = m_aKeyValues[im].z;

			dY = (y1-y2)*(x-x1)/(x1-x2) + y1;

			if(dY - y == 0.0)
			{
				//    !
				aFlags[im] = 0;
				i = i1;

				cout << "z";
			}
			else
			{
				i = im;
			}
		}
		break;

	default:
		cout << "⿡  ȵȴ!!!!" << endl;
		break;
	}
}



int JWPositionInterpolator::FindKeyDigit()
{
	int i;
	int nLargestDigit, nLeadingZero, nKeyDigit;

	//I should find the minimum resolution first.
	nKeyDigit = 1;

	for(i=0; i<aKeys.GetSize(); i++)
	{
		// ó 0 ƴ ڰ  ?
		// nLeadingZero
		nLeadingZero = GetLeadingZero(aKeys[i]);
		nLargestDigit = GetLargestDigit(aKeys[i]);

		if(nKeyDigit < (nLargestDigit - nLeadingZero))
			nKeyDigit = nLargestDigit - nLeadingZero;
	}

	m_nKeyDigit = nKeyDigit;

	return (nKeyDigit < 15)? nKeyDigit: 15;
}



int JWPositionInterpolator::GetLargestDigit(double key)
{
	//////////////////////////////////////////////////
	//
	//			Original Value 7ڸ !!!
	//			꿡 ؼ   
	//
	//////////////////////////////////////////////////

	char buffer[50];
	_gcvt(key, 15, buffer);
	key = atof(buffer);

	//
	//////////////////////////////////////////////////


	double a;
	__int64 mult;
	double dmult;
	__int64 Ia;
	double e = dEPSILONERROR;
	int nLargestDigit;

	mult=1; dmult=0.1;
	nLargestDigit = 0;

	a = fabs(key);
	Ia = (__int64) (a + mult*e);

	// sowoo 2001.10.11 (Digit   ϱ)
	if(a == 0.0)
		return 1;

	double dTemp = a - (double)Ia;

	if((dTemp == 0.0) || (dTemp < e))
	{
		while(true)
		{
			a = fabs(key);
			a *= dmult;
			Ia = (__int64) (a + e);
			dTemp = a - (double)Ia;

			if(dTemp > e)
				return nLargestDigit;

			dmult *= 0.1;
			nLargestDigit--;
		}
	}


	dTemp = a - (double) Ia;
	while(a - (double) Ia > e*mult)
	{
		mult*=10;
		nLargestDigit++;
		a = fabs(key);
		a=(double) mult *a;
		Ia = (__int64) (a + mult*e);

		dTemp = a - (double) Ia;
	}

	return nLargestDigit;
}

int JWPositionInterpolator::GetLeadingZero(double a)
{
	//////////////////////////////////////////////////
	//
	//			Original Value 7ڸ !!!
	//			꿡 ؼ   
	//
	//////////////////////////////////////////////////

	char buffer[50];
	_gcvt(a, 15, buffer);
	a = atof(buffer);

	//
	//////////////////////////////////////////////////


	double e = dEPSILONERROR;
	__int64 nMulti = 10;
	double dMulti = 0.1;
	int nCount = 0;

	// 1. 켱  ٲٰ,
	a = fabs(a);

	// 2. 0̰ų 0.1 ũ տ "0" .
	if(a > 1.0)
	{
		nCount++;
		while(a*dMulti > 1.0)
		{
			nCount++;
			dMulti *= 0.1;
		}

		return -nCount;
	}
	else if(a == 0.0)
		return 0;
	else if(a >= 0.1)
		return 0;
	else
	{
		nCount++;
		while(a*nMulti < 0.1)
		{
			nCount++;
			nMulti *= 10;
		}

		return nCount;
	}
}

int JWPositionInterpolator::FindKeyValueDigit()
{
	int i;
	int nLargestDigit, nLeadingZero, nKeyValueDigit;

	//I should find the minimum resolution first.
	nKeyValueDigit = 1;

	for(i=0; i<m_aKeyValues.GetSize(); i++)
	{
		// ó 0 ƴ ڰ  ?
		// nLeadingZero
		nLeadingZero = GetLeadingZero(m_aKeyValues[i].x);
		nLargestDigit = GetLargestDigit(m_aKeyValues[i].x);

		if(nKeyValueDigit < (nLargestDigit - nLeadingZero))
			nKeyValueDigit = nLargestDigit - nLeadingZero;
	}

	// sowoo 2001.10.11 (Min/Max Reduce)
	m_nKeyValueDigit_X = nKeyValueDigit;
	m_nKeyValueDigit = nKeyValueDigit;

	nKeyValueDigit = 1;
	for(i=0; i<m_aKeyValues.GetSize(); i++)
	{
		// ó 0 ƴ ڰ  ?
		// nLeadingZero
		nLeadingZero = GetLeadingZero(m_aKeyValues[i].y);
		nLargestDigit = GetLargestDigit(m_aKeyValues[i].y);

		if(nKeyValueDigit < (nLargestDigit - nLeadingZero))
			nKeyValueDigit = nLargestDigit - nLeadingZero;
	}

	// sowoo 2001.10.11 (Min/Max Reduce)
	m_nKeyValueDigit_Y = nKeyValueDigit;
	if(m_nKeyValueDigit < m_nKeyValueDigit_Y)
		m_nKeyValueDigit = m_nKeyValueDigit_Y;

	nKeyValueDigit = 1;
	for(i=0; i<m_aKeyValues.GetSize(); i++)
	{
		// ó 0 ƴ ڰ  ?
		// nLeadingZero
		nLeadingZero = GetLeadingZero(m_aKeyValues[i].z);
		nLargestDigit = GetLargestDigit(m_aKeyValues[i].z);

		if(nKeyValueDigit < (nLargestDigit - nLeadingZero))
			nKeyValueDigit = nLargestDigit - nLeadingZero;

		if(i == 103)
			i = 103;
	}

	// sowoo 2001.10.11 (Min/Max Reduce)
	m_nKeyValueDigit_Z = nKeyValueDigit;
	if(m_nKeyValueDigit < m_nKeyValueDigit_Z)
		m_nKeyValueDigit = m_nKeyValueDigit_Z;


	//  ū  ǥ ´.
	return (m_nKeyValueDigit < 15)? m_nKeyValueDigit: 15;
}



double JWPositionInterpolator::CalculateTotalLengthOfPath()
{
	double dTotalPath = 0.0;
	double dCurDistance;

	MFVec3f& raKeyValues = m_aKeyValues;
	MFFloat& raKeys = aKeys;
	
	for(int i=1; i<raKeys.GetSize(); i++)
	{
		dCurDistance = raKeyValues[i-1].GetDistance(raKeyValues[i]);
		dTotalPath += dCurDistance;
	}


	m_dTotalLengthOfPath = dTotalPath;

	return dTotalPath;
}


double JWPositionInterpolator::CalculateTotalLengthOfPathOverBox()
{
	double dTotalPath = 0.0;
	double dCurDistance;
	SFVec3f max, min;

	MFVec3f& raKeyValues = m_aKeyValues;
	MFFloat& raKeys = aKeys;

	max.x = raKeyValues[0].x;
	max.y = raKeyValues[0].y;
	max.z = raKeyValues[0].z;

	min.x = raKeyValues[0].x;
	min.y = raKeyValues[0].y;
	min.z = raKeyValues[0].z;

	for(int i=1; i<raKeys.GetSize(); i++)
	{
		dCurDistance = raKeyValues[i-1].GetDistance(raKeyValues[i]);
		dTotalPath += dCurDistance;

		if(max.x < raKeyValues[i].x)
			max.x = raKeyValues[i].x;
		if(max.y < raKeyValues[i].y)
			max.y = raKeyValues[i].y;
		if(max.z < raKeyValues[i].z)
			max.z = raKeyValues[i].z;

		if(min.x > raKeyValues[i].x)
			min.x = raKeyValues[i].x;
		if(min.y > raKeyValues[i].y)
			min.y = raKeyValues[i].y;
		if(min.z > raKeyValues[i].z)
			min.z = raKeyValues[i].z;
	}

	m_dTotalLengthOfPathOverBox = dTotalPath / max.GetDistance(min);

	return m_dTotalLengthOfPathOverBox;
}


void JWPositionInterpolator::Write_For_Debugging(char* fileName)
{
	ofstream file(fileName);

	for(int i=0; i<aKeys.GetSize(); i++)
	{
		file << aKeys[i] << '\t';
		file << m_aKeyValues[i].x << '\t';
		file << m_aKeyValues[i].y << '\t';
		file << m_aKeyValues[i].z << endl;
	}

	file.close();
}



// sowoo 2001.10.09 (List Norm. Factor)
void JWPositionInterpolator::ListNormFactor()
{
	JWPositionInterpolator tempInterpolator1;
	tempInterpolator1.Copy(this);


	tempInterpolator1.CalculateMaxDistanceOfInterval();
	tempInterpolator1.CalculateTotalLengthOfPath();
	tempInterpolator1.CalculateTotalLengthOfPathOverBox();
	tempInterpolator1.CalculateTwoPointsError();

	ofstream file;
	file.open("NormFactor_Values.txt");

	file << "3-Norm\t" << tempInterpolator1.GetMaxDistanceOfInterval() << endl;
	file << "6-Norm\t" << tempInterpolator1.GetTwoPointsError() << endl;

	file.close();
}


// sowoo 2001.10.08 (Add Error Measure 6)
// Error Measure 6  ̴.
// ù/ error  Ŀ   '' Ѵ.
double JWPositionInterpolator::CalculateTwoPointsError()
{
	// flag Ͽ   Interpolator Ѵ.
	JWPositionInterpolator thisInterp;
	JWPositionInterpolator twoPointsInterp;

	thisInterp.Copy(this);

	int nEnd = thisInterp.GetKeys().GetSize() - 1;

	twoPointsInterp.Add(aKeys[0], (void*) &(m_aKeyValues[0]));
	twoPointsInterp.Add(aKeys[nEnd], (void*) &(m_aKeyValues[nEnd]));


	// 1.  ġ 
	thisInterp.InsertPointWith(&twoPointsInterp);
	twoPointsInterp.InsertPointWith(&thisInterp);


	// 2. ߰  Ѵ.
	MakePointsDouble(&thisInterp, &twoPointsInterp);


	// 3.   distortionϱ!
	double rms = 0.0, peak = 0.0;
	EstimateError(&thisInterp, &twoPointsInterp, rms, peak, 0);

	//  Interpolator Ͽ  Ѵ.
	m_dTwoPointsError = rms;

	return m_dTwoPointsError;
}





// sowoo 2002.01.24 (Resampling)
// յ  ־ ŭ ø Ѵ.
// sowoo 2002.01.25 - digit ϶!, Ȯ ©!
void JWPositionInterpolator::Resampling(int nNumKey, int nDigit)
{
	int nNumberOfKey = aKeys.GetSize();

	//  key    볳  .
//	if(nNumberOfKey < nNumKey)
//		nNumKey = nNumberOfKey;

	// 1. ־  key .
	double* aSampledKeys = new double[nNumKey];
	SFVec3f* aSampledKeyValues = new SFVec3f[nNumKey];


	double dStep = (aKeys[nNumberOfKey-1] - aKeys[0])/(nNumKey-1);

	char buffer[50];

	for(int i=0; i<nNumKey; i++)
	{
		aSampledKeys[i] = dStep*i + aKeys[0];

		// sowoo 2002.01.25 ©!!! Ѥ'
		if(nDigit > 0)
		{
			_gcvt(aSampledKeys[i], nDigit, buffer);
			aSampledKeys[i] = atof(buffer);
		}
	}


	// 2. ־  keyValue .
	int j;


	aSampledKeyValues[0] = m_aKeyValues[0];
	aSampledKeyValues[nNumKey-1] = m_aKeyValues[nNumberOfKey-1];

	//  Key    Samplingϴ 
//	if(nNumberOfKey > nNumKey)
//	{
		for(i=1, j=1; i<nNumberOfKey;)
		{
			//if(aKeys[i] < aSampledKeys[j])
			// i-1 < j < i ̳?
			// ƴϸ,  ...
			if(!((aKeys[i-1] <= aSampledKeys[j]) && (aSampledKeys[j] < aKeys[i])))
			{
				i++;
				continue;
			}

			// ´  ´ٸ...
			InterpolateKeyValue(
				aKeys[i-1], aKeys[i], aSampledKeys[j],
				(void*) &m_aKeyValues[i-1], (void*) &m_aKeyValues[i],
				(void*) &aSampledKeyValues[j]
			);


			if(nNumKey <= j)
				break;

			j++;
		}
//	}
/*
	//  key   samplingϴ 
	else
	{
		for(i=1, j=1; i<nNumberOfKey;)
		{
			if(aKeys[i] < aSampledKeys[j])
			{
				i++;
				continue;
			}

			// ´  ´ٸ...
			InterpolateKeyValue(
				aKeys[i-1], aKeys[i], aSampledKeys[j],
				(void*) &m_aKeyValues[i-1], (void*) &m_aKeyValues[i],
				(void*) &aSampledKeyValues[j]
			);


			j++;

			if(nNumKey <= j)
				break;
		}
	}

*/
	//  ִ  Key & KeyValue ϰ
	aKeys.Copy(aSampledKeys, nNumKey);
	m_aKeyValues.Copy(aSampledKeyValues, nNumKey);
}
