// JWPositionInterpolator.h: interface for the JWPositionInterpolator class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_JWPOSITIONINTERPOLATOR_H__3D3BA1B6_166F_4ED3_9263_47C954E0A1C7__INCLUDED_)
#define AFX_JWPOSITIONINTERPOLATOR_H__3D3BA1B6_166F_4ED3_9263_47C954E0A1C7__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "JWInterpolator.h"
#include "SFVec3f.h"

class JWToken;
class ofstream;

typedef enum _tagKeySelectionType {
	KS_KeySelection, KS_KeySelectionPurify,
	KS_ComponentSelection, KS_UpdatedComponentSelection,
	KS_LinearKeyCoder, 
	NoneOfThem
} KeySelectionType;

class JWPositionInterpolator : public JWInterpolator  
{
public:
	JWPositionInterpolator();
	virtual ~JWPositionInterpolator();

public:
	void Parse(JWToken& token);
	void Write(ofstream& file);
	double Purify(double threshold, JWInterpolator*& pInterpolator);
	void Copy(JWInterpolator* pInterp);

	friend ofstream& operator << (ofstream& os, JWPositionInterpolator& pI);

public:
	void SetSize(int size)
	{
		aKeys.SetSize(size);
		m_aKeyValues.SetSize(size);
	}

	void Add(double key, void* keyValue);
	void SetKeyValue(int i, void* keyValue) { m_aKeyValues[i] = *((SFVec3f*)keyValue); }
	void SetKeyValues(void* keyValues) { m_aKeyValues.Copy(*((JWArray<SFVec3f>*)keyValues)); }
	void* GetKeyValues() { return (void*) &m_aKeyValues; }
	void SetKeyFrame(int i, double key, void* keyValue);

public:
	void InsertKey(int index, double key, void* kv);
	void InterpolateKeyValue(
		double key1, double key2, double key, 
		void* kv1, void* kv2, void* keyValue
	);
	void InsertPointWith(JWInterpolator* pInterpolator);
	double CalculateDistortion(JWInterpolator* pInterp, JWArray<bool>& aFlags);
	double CalculateDistortion(JWInterpolator* pInterp);

	double CalculateMaxDistanceOfInterval();
	double GetMaxDistanceOfInterval() { return m_dMaxDistanceOfInterval; }


	// sowoo 2001.09.04 (Key Selection)
	void JWPositionInterpolator::FindKeyFlag(
		double dThreshold[], int nNumThreshold, 
		JWArray<JWArray<int> >& aXs, 
		JWArray<JWArray<int> >& aYs, 
		JWArray<JWArray<int> >& aZs
	);

	void FindKeyFlag_ValidDigit(
		double dThreshold[], int nNumThreshold, 
		JWArray<JWArray<int> >& aXs, 
		JWArray<JWArray<int> >& aYs, 
		JWArray<JWArray<int> >& aZs
	);


	// sowoo 2002.01.24 (Resampling)
	void Resampling(int nNumKey, int nDigit = -1);


// MIN/MAXϴµ ʿ κ
	SFVec3f& GetMin();
	SFVec3f& GetMax();


	// sowoo 2001.09.07 (Component Selection)
	// linear interpolatable  component !
	void FindKeyFlag_OneCompnent(int nWhich, JWArray<int>& aFlags);

	void SetKeyDigit(int nKD) { m_nKeyDigit = nKD; }
	void SetKeyValueDigit(int nKVD) { m_nKeyValueDigit = nKVD; }

	int GetKeyDigit()
	{
		if(m_nKeyDigit == -1)
			m_nKeyDigit = FindKeyDigit();

		return m_nKeyDigit;
	}
	int GetKeyValueDigit()
	{
		if(m_nKeyValueDigit == -1)
			m_nKeyValueDigit = FindKeyValueDigit();

		return m_nKeyValueDigit;
	}

	int FindKeyDigit();
	int FindKeyValueDigit();


	// sowoo 2001.10.11 (Min/Max Reduce)
	int GetKeyValueXDigit()
	{
		if(m_nKeyValueDigit == -1)
			FindKeyValueDigit();

		return m_nKeyValueDigit_X;
	}

	int GetKeyValueYDigit()
	{
		if(m_nKeyValueDigit == -1)
			FindKeyValueDigit();

		return m_nKeyValueDigit_Y;
	}

	int GetKeyValueZDigit()
	{
		if(m_nKeyValueDigit == -1)
			FindKeyValueDigit();

		return m_nKeyValueDigit_Z;
	}
	// end (Min/Max Reduce)


	// KeyDigit  !!!
	int GetLeadingZero(double a);
	int GetLargestDigit(double a);

	int GetTypeOfError() { return m_nTypeOfError; }
	void SetTypeOfError(int type) { m_nTypeOfError = type; }

	double GetTotalLengthOfPath() { return m_dTotalLengthOfPath; }
	double CalculateTotalLengthOfPath();

	double GetTotalLengthOfPathOverBox() { return m_dTotalLengthOfPathOverBox; }
	double CalculateTotalLengthOfPathOverBox();


	// sowoo 2001.09.11 (Component Selection)
	void FindKeyFlag_EachComponent(
		double dThreshold[], int nNumThreshold, 
		JWArray<JWArray<int> >& aXs,
		JWArray<JWArray<int> >& aYs,
		JWArray<JWArray<int> >& aZs
	);

	void FindKeyFlag_EachComponent_Updated(
		double dThreshold[], int nNumThreshold, 
		JWArray<JWArray<int> >& aXs,
		JWArray<JWArray<int> >& aYs,
		JWArray<JWArray<int> >& aZs
	);

	double JWPositionInterpolator::CalculateDistortion(
		JWInterpolator* pInterp,
		JWArray<bool>& aXs,
		JWArray<bool>& aYs,
		JWArray<bool>& aZs
	);
	SFVec3f LinearInterpolation(int i, JWArray<bool>& aFlags, int nType);


	// sowoo 2001.09.13 (Debugging)
	void Write_For_Debugging(char* fileName);

	// sowoo 2001.09.15 (Add KeySelectionType)
	KeySelectionType GetKeySelectionType() { return eKeySelectionType; }
	void SetKeySelectionType(KeySelectionType eType) { eKeySelectionType = eType; }


	// sowoo 2001.10.08 (Add Error Measure 6)
	double CalculateTwoPointsError();
	double GetTwoPointsError() { return m_dTwoPointsError; }

	// sowoo 2001.10.09 (List Norm. Factor)
	void JWPositionInterpolator::ListNormFactor();

protected:
	SFVec3f m_max;
	SFVec3f m_min;

	int m_nKeyDigit;
	int m_nKeyValueDigit;

	// sowoo 2001.10.11 (Min/Max Reduce)
	int m_nKeyValueDigit_X;
	int m_nKeyValueDigit_Y;
	int m_nKeyValueDigit_Z;

protected:
	JWArray<SFVec3f> m_aKeyValues;
	double m_dMaxDistanceOfInterval;
	double m_dTotalLengthOfPathOverBox;
	double m_dTotalLengthOfPath;
	int m_nTypeOfError;

	// sowoo 2001.09.15 (Add KeySelectionType)
	KeySelectionType eKeySelectionType;

	// sowoo 2001.10.08 (Add Error Measure 6)
	double m_dTwoPointsError;
};

#endif // !defined(AFX_JWPOSITIONINTERPOLATOR_H__3D3BA1B6_166F_4ED3_9263_47C954E0A1C7__INCLUDED_)
