// Key.h: key decoder
// Author: Shinjun Lee, Samsung AIT
// 4th revision: 2002-04-18

#ifndef _KEY_H_
#define _KEY_H_

#include "ArithmeticDecoder.h"
#include "ArithmeticEncoder.h"
#include "KeyHeader.h"
#include "CKeyDecoder.h"
#include "Entropy.h"
#include "CKeyEncoder.h"

#if!defined ABS
#define  ABS(x)     ( (x) < 0.0 ? -(x) : (x) )
#endif

#if!defined MAX
#define  MAX(x, y)  ( (x) < (y) ?  (y) : (x) )
#endif

#if!defined MIN
#define  MIN(x, y)  ( (y) < (x) ?  (y) : (x) )
#endif



class Key
	{
  public:
	// variables: bitstream data of key
	int*			nQKey;									// key array

	// variables: nonparsable data for key decoding
	int				nNumberOfRemainingKey;					// the number of remaining key after linear key decoding

	double*			dKey;									// array of decoded key

	// Key: key context
	CAC				keySignContext;
	CAC				keyContext[32];

  public:
	Key(void)
		{
		nQKey = NULL;

		nNumberOfRemainingKey = 0;

		dKey = NULL;
		}

	~Key()
		{
		delete[] nQKey;
		}

	void InitializeModel(void)
		{
		int i = 0;

		keySignContext.reset();
		for(i = 0; i < 32 ; i++)
			keyContext[i].reset();
		}

	void readBitstream(CAC_QF& codec, KeyHeader& kHeader)
		{
		int			i = 0;									// index

		nQKey = new int[kHeader.nNumberOfKey];
		if(kHeader.bIsLinearKeySubRegion == true)
			nNumberOfRemainingKey = kHeader.nNumberOfKey - kHeader.lKey.nNumberOfLinearKey;
		else
			nNumberOfRemainingKey = kHeader.nNumberOfKey;

		for(i = kHeader.nKDPCMOrder+1; i < nNumberOfRemainingKey; i++)
			{
			if(kHeader.bSignedAACFlag == false)
				decodeUnsignedAAC(codec, &nQKey[i], kHeader.nKeyCodingBit, keyContext);
			else
				decodeSignedAAC(codec, &nQKey[i], kHeader.nKeyCodingBit+1, keySignContext, keyContext);
			}
		}

	void decodingProcess(KeyHeader& kHeader)
		{
		int i = 0;

		// 2002-04-18, Shinjun
		JWArray<double> JWDecodedKey;			//  decoded key;
		// 2002-04-18-02, Shinjun
		if(kHeader.bIsLinearKeySubRegion == true)
			kHeader.lKey.KeyReconstructor(kHeader.nKeyDigit, kHeader.lKey.fKeyMin, kHeader.lKey.fKeyMax,
				kHeader.lKey.nNumberOfLinearKey, JWDecodedKey);

		for(i = 0; i < kHeader.nKDPCMOrder+1; i++)
			nQKey[i] = kHeader.nQIntraKey[i];

		// key decoder
		CKeyDecoder oKeyDecoder;
		oKeyDecoder.setOrder_dpcm(kHeader.nKDPCMOrder+1);
		oKeyDecoder.setOrder_dnd(kHeader.nDNDOrder);
		oKeyDecoder.setKey_shift(kHeader.nKeyShift);
		oKeyDecoder.setKey_shift_down(kHeader.nKeyInvertDown);
		oKeyDecoder.setKey_dnd_max(kHeader.nKeyMax);
		oKeyDecoder.setKeyDigit(kHeader.nKeyDigit);
		oKeyDecoder.setMinMax(kHeader.fKeyMin, kHeader.fKeyMax);
		oKeyDecoder.decode(nQKey, nNumberOfRemainingKey, kHeader.nKeyQBit);

		// get: result data from key decoding
		oKeyDecoder.getIQKey(dKey);

		// print: result data from key decoding
		cout << "Number of keys = " << kHeader.nNumberOfKey << endl;
		cout << "QStep_key = " << kHeader.nKeyQBit << endl;
		cout << "Key digit = " << kHeader.nKeyDigit << endl;
		cout << "Order_dpcm = " << kHeader.nKDPCMOrder << endl;
		cout << "Order_dnd = " << kHeader.nDNDOrder << endl;
		cout << "Key to shift = " << kHeader.nKeyShift << endl;
		cout << "Key to shift-down = " << kHeader.nKeyInvertDown << endl;
		cout << "QStep(key_shift) = " << kHeader.nBitSize << endl;
		cout << "QStep(key_decoding) = " << kHeader.nKeyCodingBit << endl;
		cout << endl;
		cout << "dKeyMin = " << kHeader.lKey.fKeyMin << endl;
		cout << "dKeyMax = " << kHeader.lKey.fKeyMax << endl;
		cout << "nNumberOfLinearKey = " << kHeader.lKey.nNumberOfLinearKey << endl;
		cout << endl;

		// 2002-04-18-02, Shinjun
		if(kHeader.bIsLinearKeySubRegion == true)
			kHeader.lKey.KeyMerger(kHeader.nNumberOfKey, dKey, nNumberOfRemainingKey, JWDecodedKey, kHeader.lKey.nNumberOfLinearKey);
		}


	// sowoo 20020430
	// Jameskim 20020423
	//void encodingProcess(KeyHeader& kHeader)
	void encodingProcess(
		//bool bPreserveKey, 
		bool IsLinearKeyCoder, 
		KeyHeader& kHeader, 
		//CSelectedKeyFrame selectedKeyFrame,
		JWArray<double>& key,
		//char* gKeyQuant
		bool bMinimizeKeyQuantError
	)
		{
		// 2002-04-17, Shinjun
		// variables: arrays for linear key encoding
		JWArray<double>		JWOriginalKeys;
		JWArray<double>		JWSelectedKeys;
		JWArray<double>		JWDecodedKey;

		// 2002-04-17, Shinjun
		// variables: data for linear key encoding
		int					preStartKeyIdx = -1;
		int					preEndKeyIdx = -1;
		//bool				bIsKeyLinearCoder = true;
		//bool				bPreserveKey = true;
		int					i = 0;

		// Jameskim 20020426
		JWOriginalKeys.SetSize(key.GetSize());
		for(i = 0;i < key.GetSize();i++)
			JWOriginalKeys[i] = key[i];

		//for(i = 0;i < selectedKeyFrame.GetNumberOfSelectedKeys();i++)
		//	JWSelectedKeys.Add(selectedKeyFrame.GetSelectedKeys()[i]);

		// 2002-04-17, Shinjun
		// copy: the array of key to JWOriginalKeys
		//JWDecodedKey.SetSize(kHeader.nNumberOfKey);

		// Jameskim 20020423
		//JWSelectedKeys.SetSize(kHeader.nNumberOfKey);
		//for(i = 0; i < kHeader.nNumberOfKey; i++)
		//	JWOriginalKeys.Add(dKey[i]);

		// Jameskim 20020423
		//JWArray<double>&		JWOriginalKeys = selectedKeyFrame.GetOriginalKeys();
		//JWSelectedKeys = selectedKeyFrame.GetSelectedKeys();
		//JWDecodedKey.Add(1.0);

		// 2002-04-17, Shinjun
		// Linear key encoder
		// Jameskim 20020426
		kHeader.lKey.LinearKeySubRegionExtractor(
			kHeader.nKeyQBit, kHeader.nKeyDigit,
			IsLinearKeyCoder, //bPreserveKey,
			preStartKeyIdx, preEndKeyIdx,
			JWOriginalKeys//, JWSelectedKeys
		);

		kHeader.lKey.KeySplitter(
			//bPreserveKey,
			preStartKeyIdx, preEndKeyIdx, kHeader.nKeyDigit,
			JWOriginalKeys, //JWSelectedKeys,
			JWDecodedKey,
			kHeader.lKey.fKeyMin, kHeader.lKey.fKeyMax, kHeader.lKey.nNumberOfLinearKey);
		/*
		kHeader.lKey.LinearKeySubRegionExtractor(kHeader.nKeyQBit, kHeader.nKeyDigit,
			IsLinearKeyCoder, bPreserveKey,
			preStartKeyIdx, preEndKeyIdx,
			selectedKeyFrame.GetOriginalKeys(), selectedKeyFrame.GetSelectedKeys());

		kHeader.lKey.KeySplitter(bPreserveKey, preStartKeyIdx, preEndKeyIdx, kHeader.nKeyDigit,
			selectedKeyFrame.GetOriginalKeys(), selectedKeyFrame.GetSelectedKeys(), JWDecodedKey,
			kHeader.lKey.fKeyMin, kHeader.lKey.fKeyMax, kHeader.lKey.nNumberOfLinearKey);
		*/
		// 2002-04-17, Shinjun
		if(preStartKeyIdx != -1 || preEndKeyIdx != -1)
			{
			kHeader.bIsLinearKeySubRegion = true;
			nNumberOfRemainingKey = kHeader.nNumberOfKey - kHeader.lKey.nNumberOfLinearKey;
			}
		else
			{
			kHeader.bIsLinearKeySubRegion = false;
			nNumberOfRemainingKey = kHeader.nNumberOfKey;
			}

		
		//if(bPreserveKey)
		//{
			// Jameskim 20020424
			// ??
			//dKey = new double[selectedKeyFrame.GetNumberOfOriginalKeys()];
			dKey = new double[JWOriginalKeys.GetSize()];
			// 2002-04-17, Shinjun
			// copy: JWOriginalKeys to the array of key
			for(i = 0; i < nNumberOfRemainingKey; i++)
				dKey[i] = JWOriginalKeys[i];
		//}
		/*
		else
		{
			// Jameskim 20020424
			//dKey = new double[selectedKeyFrame.GetNumberOfSelectedKeys()];
			dKey = new double[JWSelectedKeys.GetSize()];
			// 2002-04-17, Shinjun
			// copy: JWOriginalKeys to the array of key
			for(i = 0; i < nNumberOfRemainingKey; i++)
				dKey[i] = JWSelectedKeys[i];
		}*/

		//JWOriginalKeys.RemoveAll();
		//JWSelectedKeys.RemoveAll();
		//JWDecodedKey.RemoveAll();
		
		// Jameskim 20020424
		// -------------------------------------------------------------------------------
		//
		// A hook for ResizeKeyInterval
		//
		// -------------------------------------------------------------------------------

		// Key encoder
		// Jameskim 20020424
		CKeyEncoder oKeyEncoder;
		oKeyEncoder.setKeyDigit(kHeader.nKeyDigit);
		// Jameskim 20020424

		//if(gKeyQuant[0] == 'k')
		if(bMinimizeKeyQuantError)
		{
			/////////////////////////////////////////////////////////////
			/////////////////////////////////////////////////////////////
			/////////////////////////////////////////////////////////////
			// sowoo 2002.02.04 
			// Key ȭ  ̴ Լ
			int nReturnValue = 0;

			double dNewMin = 0, dNewMax = 0;
			//int nNewQBit = nKeyQBits;
			int nNewQBit = kHeader.nKeyQBit;

			nReturnValue = ResizeKeyInterval(
				&dNewMax, &dNewMin, &nNewQBit, // output 
												// ο min/max, QBit
				nNumberOfRemainingKey, dKey // input
			);


			if(nReturnValue != 0)
				// sowoo 2002.02.04 (Add 
				oKeyEncoder.SetUseMinMax_bySowoo(dNewMin, dNewMax);

			/////////////////////////////////////////////////////////////
			/////////////////////////////////////////////////////////////
			/////////////////////////////////////////////////////////////
		}


		if(nNumberOfRemainingKey > 0)
		{
			///////////////////////////////////////////////
			///////////////////////////////////////////////
			// Jeju meeting    .
			oKeyEncoder.setMeasureOfSAD(true);
			///////////////////////////////////////////////
			///////////////////////////////////////////////


			oKeyEncoder.encode(dKey, nNumberOfRemainingKey, kHeader.nKeyQBit);

			// get: result data from key encoding
			oKeyEncoder.getQKey(nQKey);
			oKeyEncoder.getKey_dnd_max(kHeader.nKeyMax);
			kHeader.nKDPCMOrder = oKeyEncoder.getOrder_dpcm();
			kHeader.nDNDOrder = oKeyEncoder.getOrder_dnd();
			kHeader.nKeyShift = oKeyEncoder.getKey_shift();
			kHeader.nKeyInvertUp = oKeyEncoder.getKey_shift_up();
			kHeader.nBitSize = oKeyEncoder.getQStep_key_shift();
			kHeader.nKeyCodingBit = oKeyEncoder.getQStep_key_encoding();

			if(kHeader.nKDPCMOrder > 0)
				kHeader.nQIntraKey = new int[kHeader.nKDPCMOrder+1];
			for(i = 0; i < kHeader.nKDPCMOrder+1; i++)
				kHeader.nQIntraKey[i] = nQKey[i];

			kHeader.keyMinMax.fKeyMin = kHeader.fKeyMin = dKey[0];
			// Jameskim 20020426
			kHeader.keyMinMax.fKeyMax = kHeader.fKeyMax = dKey[nNumberOfRemainingKey-1];

			// print: result data from key encoding
			/*
			cout << "Number of keys = " << kHeader.nNumberOfKey << endl;
			cout << "QStep_key = " << kHeader.nKeyQBit << endl;
			cout << "Key digit = " << kHeader.nKeyDigit << endl;
			cout << "Key Distortion = " << oKeyEncoder.getKeyDistortion() << endl;
			cout << "Order_dpcm = " << kHeader.nKDPCMOrder << endl;
			cout << "Order_dnd = " << kHeader.nDNDOrder << endl;
			cout << "Key to shift = " << kHeader.nKeyShift << endl;
			cout << "Key to shift-up = " << kHeader.nKeyInvertUp << endl;
			cout << "QStep(key_shift) = " << kHeader.nBitSize << endl;
			cout << "QStep(key_encoding) = " << kHeader.nKeyCodingBit << endl;
			cout << endl;
			cout << "dKeyMin = " << kHeader.lKey.fKeyMin << " (index: " << preStartKeyIdx << ")" << endl;
			cout << "dKeyMax = " << kHeader.lKey.fKeyMax << " (index: " << preEndKeyIdx << ")" << endl;
			cout << "nNumberOfLinearKey = " << kHeader.lKey.nNumberOfLinearKey << endl;
			cout << endl;
			*/
		}
		else
		{
			//KHeaderTotal = AACTotal() - prevBitTotal;
			//return KHeaderTotal;
		}

		// Key encoder
		// CKeyEncoder oKeyEncoder;
		// oKeyEncoder.setKeyDigit(kHeader.nKeyDigit);
		// oKeyEncoder.encode(dKey, nNumberOfRemainingKey, kHeader.nKeyQBit);

		}




		// sowoo 2002.02.04 
		// Key ȭ  ̴ Լ
		//int ResizeKeyInterval(double* max, double* min, int* qBits, int nKey, JWArray<double>& Key)
		int ResizeKeyInterval(double* max, double* min, int* qBits, int nKey, double* Key)
		{
			double   *dKey;
			int      aKey;
			double   sumKey, sumError, qStep, qStepPrev, iqKey;
			double   peakError;
			double   max_org;
			double   TmpKey;
			double   DifferentialThreshold;
			double   IterationThreshold;
			int      *ranking;
			int      *frequency;
			int      MostProbable;
			int      i, j;
			int      nqBits;
			long int qValue, qLevel, nLevel;

			*max = Key[0]; *min = Key[0];
			for ( i = 0 ; i < nKey ; ++i ) {
				*max = MAX(*max, Key[i]);
				*min = MIN(*min, Key[i]);
			}
			max_org = *max;

			dKey = (double *)calloc(nKey, sizeof(double));
			aKey = 0;
			for ( i = 0 ; i < (nKey-1) ; ++i ) {
				if ( 0.0 < ( Key[i+1] - Key[i] ) ) {
					dKey[aKey] = Key[i+1] - Key[i];
					++aKey;
				}
			}

			qStep = *max - *min;
			DifferentialThreshold = 1.0E-2;
			IterationThreshold = 1.0E-10;
			do {
				qStepPrev = qStep;
				for ( i = 0 ; i < (aKey-1) ; ++i ) {
					TmpKey = dKey[i];
					for ( j = i ; j < aKey ; ++j ) {
						if ( dKey[j] < TmpKey ) {
							dKey[i] = dKey[j];
							dKey[j] = TmpKey;
							TmpKey = dKey[i];
						}
					}
				}
				ranking = (int *)calloc(aKey, sizeof(int));
				TmpKey = dKey[0];
				j = 0;
				for ( i = 0 ; i < aKey ; ++i ) {
					if ( DifferentialThreshold < ((dKey[i]-TmpKey)/TmpKey) ) {
						TmpKey = dKey[i];
						j = i;
					}
					TmpKey = dKey[i];
					ranking[i] = j;
				}
				frequency = (int *)calloc(aKey, sizeof(int));
				for ( i = 0 ; i < aKey ; ++i ) frequency[i] = 0;
				for ( i = 0 ; i < aKey ; ++i ) ++frequency[ranking[i]];
				MostProbable = 0;
				for ( i = 0 ; i < aKey ; ++i ) if ( frequency[MostProbable] < frequency[i] ) MostProbable = i;
				sumKey = 0.0;
				for ( i = 0 ; i < aKey ; ++i ) if ( ranking[i] == MostProbable ) sumKey += dKey[i];
				qStep = sumKey / (double)frequency[MostProbable];
				free(ranking);
				free(frequency);

				for ( i = 0 ; i < aKey ; ++i ) {
					qValue = (long int)( dKey[i] / qStep + 0.4999999 );
					dKey[i] = qStep + ( dKey[i] - (double)qValue * qStep );
				}
			} while ( IterationThreshold < ABS(qStepPrev-qStep)/qStepPrev );
			
			qLevel = 1 + (long int)( ( *max - *min ) / qStep + 0.4999999 );
			nLevel = 1; nqBits = 0;
			do {
				nLevel *= 2; nqBits += 1;
				if ( (2*qLevel) <= nLevel ) qStepPrev /= 2.0;
			} while ( nqBits < *qBits );
			if ( qLevel <= nLevel ) *max = (double)( nLevel - 1 ) * qStepPrev + *min;

			sumError = 0.0; peakError = 0.0;
			for ( i = 0 ; i < nKey ; ++i ) {
				qValue = (long int)( ( Key[i] - *min ) / ( *max - *min ) * ( (double)( nLevel - 1 ) ) + 0.4999999 );
				iqKey  = ( (double)qValue ) * ( *max - *min ) / ( (double)( nLevel - 1 ) ) + *min;
				sumError += ABS( iqKey - Key[i] );
				peakError = MAX(peakError, ABS(iqKey-Key[i]));
			}
			sumError /= (double)nKey;
			if ( qStepPrev * 0.125 < sumError ) {
				*max = max_org;
				printf("%d\t%.10lf\t%.10lf\n", *qBits, *min, *max);
				return(0);
			}
			else {
				printf("%d\t%.10lf\t%.10lf\t%.10lf\t%.10lf\t%.10lf\n", *qBits, *min, *max, sumError, peakError, sumError/qStepPrev);
				return(1);
			}
		}

	void makeBitstream(CAC_QF& codec, KeyHeader& kHeader, int& nTotalBitsOfKey)
		{
		int nBitsOfStart = 0;
		int nBitsOfEnd = 0;
		int	i = 0; // index

		nBitsOfStart = (codec._ostrm->Outbuf.nbytes-1)*8 + (codec._ostrm->Outbuf.nbits-1);
		for(i = kHeader.nKDPCMOrder; i < nNumberOfRemainingKey; i++)
			{
			if(kHeader.bSignedAACFlag == false)
				encodeUnsignedAAC(codec, nQKey[i], kHeader.nKeyCodingBit, keyContext);
			else
				encodeSignedAAC(codec, nQKey[i], kHeader.nKeyCodingBit+1, keySignContext, keyContext);
			}
		nBitsOfEnd = (codec._ostrm->Outbuf.nbytes-1)*8 + (codec._ostrm->Outbuf.nbits-1);
		nTotalBitsOfKey += nBitsOfEnd - nBitsOfStart;

		///////////////////////////////////////////////////////////////////
		// for debugging
		ofstream file;
		file.open("PDAM_Key_Encoder.txt");

		for(i = kHeader.nKDPCMOrder; i < nNumberOfRemainingKey; i++)
		{
			file << nQKey[i] << endl;
		}
		file.close();
		///////////////////////////////////////////////////////////////

		}

	};

#endif // _KEY_H_