// KeyMinMaxDecoder
// Author: Sang-Oak Woo
// Editor: Shinjun Lee (codes are modified into classes)
// Date: 2001. 12. 27.
// 4th revision: 2002-04-18

#ifndef _KEYMINMAX_H_
#define _KEYMINMAX_H_

#include "Entropy.h"
#include "functionsforEncoding.h"

#include <math.h>

class KeyMinMax
	{
  public:
	int			nMinKeyDigit;

	int			nMinKeyMantissaSign;
	int			nMinKeyMantissa;

	int			nMinKeyExponentSign;
	int			nMinKeyExponent;

	int			nMaxKeyDigit;

	int			nMaxKeyMantissaSign;
	int			nMaxKeyMantissa;

	int			nMaxKeyExponentSign;
	int			nMaxKeyExponent;

	int			bSameExponent;

	double		fKeyMin;
	double		fKeyMax;

  public:
	KeyMinMax(void)
		{
		nMinKeyDigit = 0;

		nMinKeyMantissaSign = 0;
		nMinKeyMantissa = 0;

		nMinKeyExponentSign = 0;
		nMinKeyExponent = 0;

		nMaxKeyDigit = 0;

		nMaxKeyMantissaSign = 0;
		nMaxKeyMantissa = 0;

		nMaxKeyExponentSign = 0;
		nMaxKeyExponent = 0;

		bSameExponent = 0;

		fKeyMin = 0;
		fKeyMax = 0;
		}

	void readBitstream(CAC_QF& codec, int nKeyDigit)
		{
		int bMinKeyDigitSame = codec._istrm->getbits(1);
		if(bMinKeyDigitSame == 0)
			nMinKeyDigit = codec._istrm->getbits(4);
		else
			nMinKeyDigit = nKeyDigit;

		if(nMinKeyDigit != 0)
			{
			if(nMinKeyDigit < 8)
				{
				// start key
				int count = getNbBits((int)pow(10.0, nMinKeyDigit));
				nMinKeyMantissaSign = codec._istrm->getbit();
				nMinKeyMantissa = codec._istrm->getbits(count);
				
				nMinKeyExponentSign = codec._istrm->getbit();
				nMinKeyExponent = codec._istrm->getbits(6);
				}
			else
				{
				int integer = codec._istrm->getbits(32);
				fKeyMin = *((float *)&integer);
				}
			}

		int bMaxKeyDigitSame = codec._istrm->getbits(1);
		if(bMaxKeyDigitSame == 0)
			nMaxKeyDigit = codec._istrm->getbits(4);
		else
			nMaxKeyDigit = nKeyDigit;

		if(nMaxKeyDigit != 0)
			{
			if(nMaxKeyDigit < 8)
				{
				// end key
				int count = getNbBits((int)pow(10.0, nMaxKeyDigit));
				nMaxKeyMantissaSign = codec._istrm->getbit();
				nMaxKeyMantissa = codec._istrm->getbits(count);
				
				bSameExponent = codec._istrm->getbits(1);
				if(bSameExponent == 0)
					{
					nMaxKeyExponentSign = codec._istrm->getbit();
					nMaxKeyExponent = codec._istrm->getbits(6);
					}
				else
					{
					nMaxKeyExponent = nMinKeyExponent;
					}
				}
			else
				{
				int integer = codec._istrm->getbits(32);
				fKeyMax = *((float *)&integer);
				}
			}
		}

	void decodingProcess(void)
		{
		if(nMinKeyDigit != 0)
			{
			if(nMinKeyDigit < 8)
				{
				// start key
				if(nMinKeyMantissaSign == 1)
					nMinKeyMantissa *= -1;
				
				if(nMinKeyExponentSign == 1)
					nMinKeyExponent *= -1;

				fKeyMin = nMinKeyMantissa / pow(10.0, nMinKeyExponent);
				}
			}
		else
			{
			fKeyMin = 0.0;
			}

		if(nMaxKeyDigit != 0)
			{
			if(nMaxKeyDigit < 8)
				{
				// end key
				if(nMaxKeyMantissaSign == 1)
					nMaxKeyMantissa *= -1;
				
				if(bSameExponent == 0)
					{
					if(nMaxKeyExponentSign == 1)
						nMaxKeyExponent *= -1;
					}

				fKeyMax = nMaxKeyMantissa / pow(10.0, nMaxKeyExponent);
				}
			}
		else
			{
			fKeyMax = 0.0;
			}
		}


	// for encoding
	void makeBitstream(CAC_QF& codec, int nKeyDigit, int& KTotal)
		{
		// Start key
		// mantissa digit 4 
		int nDigit = GetDigit(fKeyMin); //  nDigit: ù°   digit
		// 0  digit = 0.
		if(nDigit == nKeyDigit) // nKeyDigit: ü digit
			{
			codec._ostrm->putbits(1, 1);
			KTotal += 1;
			}
		else
			{
			codec._ostrm->putbits(0, 1);
			KTotal += 1;
			codec._ostrm->putbits(nDigit, 4);
			KTotal += 4;
			}

		int mantissa;
		int preexponent = 70; // exponent      ʱȭ!
		int count;

		if(nDigit != 0)
			{
			if(nDigit < 8)
				{
				// mantissa 10 
				int nSign = 0;
				if(fKeyMin >= 0)
					codec._ostrm->putbit(0);
				else
					codec._ostrm->putbit(1);
				KTotal += 1;
				mantissa = GetMantissa(fKeyMin, nDigit);
				count = getNbBits((int)pow(10.0, nDigit)-1);
				codec._ostrm->putbits(mantissa, count);
				KTotal += count;

				// exponent digit 6
				preexponent = GetLargestDigit(fKeyMin);
				if(preexponent >= 0)
					codec._ostrm->putbit(0);
				else
					codec._ostrm->putbit(1);
				KTotal += 1;
				codec._ostrm->putbits(abs(preexponent), 6);
				KTotal += 6;
				}
			else
				{
				float flot = (float)fKeyMin;
				codec._ostrm->putbits(*((int *)&flot), 32);
				KTotal += 32;
				}
			}

		// End key
		// mantissa digit 4
		nDigit =  GetDigit(fKeyMax);
		if(nDigit == nKeyDigit)
			{
			codec._ostrm->putbits(1, 1);
			KTotal += 1;
			}
		else
			{
			codec._ostrm->putbits(0, 1);
			KTotal += 1;
			codec._ostrm->putbits(nDigit, 4);
			KTotal += 4;
			}							

		if(nDigit != 0)
			{
			if(nDigit < 8)
				{
				// mantissa 10 
				int nSign = 0;
				if(fKeyMax >= 0)
					codec._ostrm->putbit(0);
				else
					codec._ostrm->putbit(1);
				KTotal += 1;
				mantissa = GetMantissa(fKeyMax, nDigit);
				count = getNbBits((int)pow(10.0, nDigit)-1);
				codec._ostrm->putbits(mantissa, count);
				KTotal += count;

				// exponent 6 
				int exponent = GetLargestDigit(fKeyMax);
				if(exponent == preexponent)
					{
					codec._ostrm->putbits(1, 1);
					KTotal += 1;
					}
				else
					{
					codec._ostrm->putbits(0, 1);
					KTotal += 1;

					if(exponent >= 0)
						codec._ostrm->putbit(0);
					else
						codec._ostrm->putbit(1);
					KTotal += 1;
					codec._ostrm->putbits(abs(exponent), 6);
					KTotal += 6;
					}
				}
			else
				{
				const float flot = (float)fKeyMax;
				codec._ostrm->putbits(*((int *)&flot), 32);
				KTotal += 32;
				}
			}
		}


  private:
	int getNbBits(int k)
		{
		int j=1;
		while (k >= (1<<j) ) 
			{
			j++;
			}
		return j;
		}
	};

#endif // _KEYMINMAX_H_