// PosIKeyValue.cpp: implementation of the PosIKeyValue class.
//
//////////////////////////////////////////////////////////////////////

#include "PosIKeyValue.h"
#include "FunctionsForDecoding.h"
#include "ArithmeticDecoder.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

PosIKeyValue::PosIKeyValue()
{
	keyValue_X = NULL;
	keyValue_Y = NULL;
	keyValue_Z = NULL;
}

PosIKeyValue::~PosIKeyValue()
{
	if(keyValue_X != NULL)
		delete[] keyValue_X;
	if(keyValue_Y != NULL)
		delete[] keyValue_Y;
	if(keyValue_Z != NULL)
		delete[] keyValue_Z;
}

void PosIKeyValue::readBitstream(
	CAC_QF& codec, PosIKeyValueHeader kvHeader,
	int nNumberOfKeyValue
)
{
	keyValue_X = new int[nNumberOfKeyValue];
	keyValue_Y = new int[nNumberOfKeyValue];
	keyValue_Z = new int[nNumberOfKeyValue];

	int i;
	if(kvHeader.x_keyvalue_flag == 1)
	{
		if(kvHeader.bIsUnaryAAC_X == 1)
			for(i=kvHeader.nStartIndex_X; i< nNumberOfKeyValue; i++)
				//
				decodeUnaryAAC(codec, &keyValue_X[i], kvXSignContext, kvXUContext);
		else
			decodeSQAAC(codec, keyValue_X, kvHeader.nKVCodingBit_X, 
				kvXSignContext, kvXMaxValueContext, kvXFoundContext,
				kvXNotFoundContext, nNumberOfKeyValue, kvHeader.nStartIndex_X);
	}

	if(kvHeader.y_keyvalue_flag == 1)
	{
		if(kvHeader.bIsUnaryAAC_Y == 1)
			for(i=kvHeader.nStartIndex_Y; i< nNumberOfKeyValue; i++)
				//
				decodeUnaryAAC(codec, &keyValue_Y[i], kvYSignContext, kvYUContext);
		else
			decodeSQAAC(codec, keyValue_Y, kvHeader.nKVCodingBit_Y, 
				kvYSignContext, kvYMaxValueContext, kvYFoundContext,
				kvYNotFoundContext, nNumberOfKeyValue, kvHeader.nStartIndex_Y);
	}

	if(kvHeader.z_keyvalue_flag == 1)
	{
		if(kvHeader.bIsUnaryAAC_X == 1)
			for(i=kvHeader.nStartIndex_Z; i< nNumberOfKeyValue; i++)
				//
				decodeUnaryAAC(codec, &keyValue_Z[i], kvZSignContext, kvZUContext);
		else
			decodeSQAAC(codec, keyValue_Z, kvHeader.nKVCodingBit_Z, 
				kvZSignContext, kvZMaxValueContext, kvZFoundContext,
				kvZNotFoundContext, nNumberOfKeyValue, kvHeader.nStartIndex_Z);
	}
}


void PosIKeyValue::decodingProcess(
	PosIKeyValueHeader kvHeader,
	KeySelectionFlag ksFlag,
	JWArray<double>& key,
	JWArray<SFVec3f>& keyValue
)
{
	int nNumberOfKeyValue = keyValue.GetSize();
	int nNumberOfKey = key.GetSize();

	double* adTemp = new double[nNumberOfKeyValue];
	double* adTemp2 = new double[nNumberOfKey];

	int i;
	if(kvHeader.x_keyvalue_flag == 1)
	{
		// Intra Coding
		if((kvHeader.nStartIndex_X == 1) && (kvHeader.x_keyvalue_flag == 1))
			keyValue_X[0] = kvHeader.firstKV_X;

		// Inverse DPCM
		if(kvHeader.nKVDPCMOrder_X ==  0)
			// 1st-order DPCM
			Inverse_DPCM_CircularQuantize(
				keyValue_X, nNumberOfKeyValue, (1<<kvHeader.nKVQBit) -1,
				kvHeader.nStartIndex_X
			);
		else
			// 2nd-order DPCM
			Inverse_Second_DPCM_CircularQuantize(
				keyValue_X, nNumberOfKeyValue, (1<<kvHeader.nKVQBit) -1,
				kvHeader.nStartIndex_X
			);

		// Inverse Quantization & Normalization
		Inverse_Quantize(
			keyValue_X, adTemp, nNumberOfKeyValue,
			kvHeader.kvMinMax.fMin_X, kvHeader.kvMinMax.fMax_X,
			kvHeader.nKVQBit, kvHeader.nKVDigit
		);

		// Interpolator Synthesizer
		InterpolatorSythesizer(
			adTemp, adTemp2, ksFlag.keyFlag,
			nNumberOfKey, nNumberOfKeyValue,
			key
		);

		for(i=0; i<nNumberOfKey; i++)
			keyValue[i].x = adTemp2[i];
	}
	else
		for(i=0; i<nNumberOfKeyValue; i++)
			keyValue[i].x = kvHeader.kvMinMax.fMin_X;


	if(kvHeader.y_keyvalue_flag == 1)
	{
		// Intra Coding
		if((kvHeader.nStartIndex_Y == 1) && (kvHeader.y_keyvalue_flag == 1))
			keyValue_Y[0] = kvHeader.firstKV_Y;

		// Inverse DPCM
		if(kvHeader.nKVDPCMOrder_Y ==  0)
			// 1st-order DPCM
			Inverse_DPCM_CircularQuantize(
				keyValue_Y, nNumberOfKeyValue, (1<<kvHeader.nKVQBit) -1,
				kvHeader.nStartIndex_Y
			);
		else
			// 2nd-order DPCM
			Inverse_Second_DPCM_CircularQuantize(
				keyValue_Y, nNumberOfKeyValue, (1<<kvHeader.nKVQBit) -1,
				kvHeader.nStartIndex_Y
			);

		// Inverse Quantization & Normalization
		Inverse_Quantize(
			keyValue_Y, adTemp, nNumberOfKeyValue,
			kvHeader.kvMinMax.fMin_Y, kvHeader.kvMinMax.fMax_Y,
			kvHeader.nKVQBit, kvHeader.nKVDigit
		);

		// Interpolator Synthesizer
		InterpolatorSythesizer(
			adTemp, adTemp2, ksFlag.keyFlag,
			nNumberOfKey, nNumberOfKeyValue,
			key
		);

		for(i=0; i<nNumberOfKey; i++)
			keyValue[i].y = adTemp2[i];
	}
	else
		for(i=0; i<nNumberOfKeyValue; i++)
			keyValue[i].y = kvHeader.kvMinMax.fMin_Y;

	if(kvHeader.z_keyvalue_flag == 1)
	{
		// Intra Coding
		if((kvHeader.nStartIndex_Z == 1) && (kvHeader.z_keyvalue_flag == 1))
			keyValue_Z[0] = kvHeader.firstKV_Z;

		// Inverse DPCM
		if(kvHeader.nKVDPCMOrder_Z ==  0)
			// 1st-order DPCM
			Inverse_DPCM_CircularQuantize(
				keyValue_Z, nNumberOfKeyValue, (1<<kvHeader.nKVQBit) -1,
				kvHeader.nStartIndex_Z
			);
		else
			// 2nd-order DPCM
			Inverse_Second_DPCM_CircularQuantize(
				keyValue_Z, nNumberOfKeyValue, (1<<kvHeader.nKVQBit) -1,
				kvHeader.nStartIndex_Z
			);

		// Inverse Quantization & Normalization
		Inverse_Quantize(
			keyValue_Z, adTemp, nNumberOfKeyValue,
			kvHeader.kvMinMax.fMin_Z, kvHeader.kvMinMax.fMax_Z,
			kvHeader.nKVQBit, kvHeader.nKVDigit
		);

		// Interpolator Synthesizer
		InterpolatorSythesizer(
			adTemp, adTemp2, ksFlag.keyFlag,
			nNumberOfKey, nNumberOfKeyValue,
			key
		);

		for(i=0; i<nNumberOfKey; i++)
			keyValue[i].z = adTemp2[i];
	}
	else
		for(i=0; i<nNumberOfKeyValue; i++)
			keyValue[i].z = kvHeader.kvMinMax.fMin_Z;
}


void PosIKeyValue::InitializeModel()
{
	kvXSignContext.reset();
	kvXUContext.reset();
	kvXMaxValueContext.reset();
	kvXFoundContext.reset();
	kvXNotFoundContext.reset();

	kvYSignContext.reset();
	kvYUContext.reset();
	kvYMaxValueContext.reset();
	kvYFoundContext.reset();
	kvYNotFoundContext.reset();

	kvZSignContext.reset();
	kvZUContext.reset();
	kvZMaxValueContext.reset();
	kvZFoundContext.reset();
	kvZNotFoundContext.reset();
}

