// SPositionInterpolator.cpp: implementation of the CSPositionInterpolator class.
//
//////////////////////////////////////////////////////////////////////
#include <stdlib.h>
#include <math.h>
#include <float.h>

#include "SPositionInterpolator.h"
#include "SError.h"
//#include "misc.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CSPositionInterpolator::CSPositionInterpolator()
{
	m_min.x = DBL_MAX;
	m_min.y = DBL_MAX;
	m_min.z = DBL_MAX;

	m_max.x = -DBL_MAX;
	m_max.y = -DBL_MAX;
	m_max.z = -DBL_MAX;
}

CSPositionInterpolator::~CSPositionInterpolator()
{

}

void CSPositionInterpolator::ParseKeyValues(ifstream& file)
{
	if(m_paKeyValues == NULL)
	{
		m_paKeyValues = new MFVec3f;
		((MFVec3f*) m_paKeyValues)->SetSize(m_nNumberOfKeys);
	}

	MFVec3f& rKeyValues = *((MFVec3f*) m_paKeyValues);
	char buffer[256];

	for(int i=0; i<m_nNumberOfKeys; i++)
	{
		file.getline(buffer, 256);
		if(::strcmp(buffer, "") == 0)
		{
			rKeyValues.SetSize(i);
			return ;
		}
		rKeyValues[i].x = ::atof(buffer);

		file.getline(buffer, 256);
		if(::strcmp(buffer, "") == 0)
		{
			rKeyValues.SetSize(i);
			return ;
		}
		rKeyValues[i].y = ::atof(buffer);

		file.getline(buffer, 256);
		if(::strcmp(buffer, "") == 0)
		{
			rKeyValues.SetSize(i);
			return ;
		}
		rKeyValues[i].z = ::atof(buffer);


		// ִ/ּҸ ϴ κ
		if(m_min.x > rKeyValues[i].x)
			m_min.x = rKeyValues[i].x;
		if(m_min.y > rKeyValues[i].y)
			m_min.y = rKeyValues[i].y;
		if(m_min.z > rKeyValues[i].z)
			m_min.z = rKeyValues[i].z;

		if(m_max.x < rKeyValues[i].x)
			m_max.x = rKeyValues[i].x;
		if(m_max.y < rKeyValues[i].y)
			m_max.y = rKeyValues[i].y;
		if(m_max.z < rKeyValues[i].z)
			m_max.z = rKeyValues[i].z;
	}
}


void CSPositionInterpolator::InsertPointWith(CSInterpolator* pInterpolator)
{
	CSPositionInterpolator& rPositionInterpolator = 
		*((CSPositionInterpolator*) pInterpolator);
	MFFloat& rKeyMe = *m_paKeys;
	MFFloat& rKeyYou = rPositionInterpolator.GetKeys();
	SFVec3f interpolatoredKeyValue;
	MFVec3f& rKeyValue = *((MFVec3f*) m_paKeyValues);

	int nNumKeyYou = rPositionInterpolator.GetNumberOfKeys();
	bool bIsMoreKey = true;
	int nIndexMe = 0, nIndexYou = 0;

	while(bIsMoreKey)
	{
		//  
		//  key  ̻  ʴ 
		if(nIndexYou >= nNumKeyYou)
		{
			return;
		}

		// ̰   쿡 key ߰ ʰ,
		//  key Ѿ.
		if(::fabs(rKeyMe[nIndexMe] - rKeyYou[nIndexYou]) < dEPSILON )
		{
			nIndexMe++;
			nIndexYou++;

			// Me  ̻ Key   ʴٸ,
			if(nIndexMe >= rKeyMe.GetSize())
			{
				//  ϴ ̹Ƿ
				for(; nIndexYou<nNumKeyYou; nIndexYou++)
					InsertKey(nIndexMe, rKeyYou[nIndexYou], (void*)&rKeyValue[nIndexMe-1]);
			}

			continue;
		}

		// 
		else if(rKeyMe[nIndexMe] > rKeyYou[nIndexYou])
		{
			if(nIndexMe == 0)
			{
				// ó ϴ ̹Ƿ
				InsertKey(0, rKeyYou[nIndexYou], (void*)&rKeyValue[0]);
				nIndexMe++;
			}
			else
			{
				// interpolator Ͽ ߰  ã .
				InterpolateKeyValue(
					rKeyMe[nIndexMe-1], rKeyMe[nIndexMe], rKeyYou[nIndexYou],
					(void*) &rKeyValue[nIndexMe-1], (void*) &rKeyValue[nIndexMe], (void*)&interpolatoredKeyValue);

				//  KEY Ѵ.
				InsertKey(nIndexMe, rKeyYou[nIndexYou], (void*)&interpolatoredKeyValue);
				nIndexMe++;
			}

			// ε ϳ Ͽ,  key Ѵ.
			nIndexYou++;
		}

		// 
		else if(rKeyMe[nIndexMe] < rKeyYou[nIndexYou])
		{
			//  key Ѿ.
			nIndexMe++;

			// Me  ̻ Key   ʴٸ,
			if(nIndexMe >= rKeyMe.GetSize())
			{
				//  ϴ ̹Ƿ
				for(; nIndexYou<nNumKeyYou; nIndexYou++)
					InsertKey(nIndexMe, rKeyYou[nIndexYou], (void*)&rKeyValue[nIndexMe-1]);
			}
		}
	} // end of while

}


void CSPositionInterpolator::InterpolateKeyValue(
	double key1, double key2, double key, 
	void* kv1, void* kv2, void* keyValue)
{
	SFVec3f& keyValue1 = *(SFVec3f*)kv1;
	SFVec3f& keyValue2 = *(SFVec3f*)kv2;
	SFVec3f& interpolatedKeyValue = *(SFVec3f*)keyValue;
	double weight = (key - key1)/(key2 - key1);

	interpolatedKeyValue.x = (keyValue2.x - keyValue1.x)*weight + keyValue1.x;
	interpolatedKeyValue.y = (keyValue2.y - keyValue1.y)*weight + keyValue1.y;
	interpolatedKeyValue.z = (keyValue2.z - keyValue1.z)*weight + keyValue1.z;
}


void CSPositionInterpolator::InsertKey(int index, double key, void* kv)
{
	SFVec3f& keyValue = *(SFVec3f*)kv;
	MFVec3f& rKeyValues = *(MFVec3f*)m_paKeyValues;

	if(index >= m_paKeys->GetSize())
	{
		m_paKeys->Add(key);
		rKeyValues.Add(keyValue);
	}
	else
	{
		m_paKeys->InsertAt(index, key);
		rKeyValues.InsertAt(index, keyValue);
	}

	m_nNumberOfKeys++;
}


void MakePointsDouble(CSInterpolator* pInterp1, CSInterpolator* pInterp2)
{
	CSPositionInterpolator& rInterp1 = *(CSPositionInterpolator*)pInterp1;
	CSPositionInterpolator& rInterp2 = *(CSPositionInterpolator*)pInterp2;

	MFFloat& rKey1 = rInterp1.GetKeys();
	MFFloat& rKey2 = rInterp2.GetKeys();
	MFVec3f& rKeyValue1 = *((MFVec3f*)rInterp1.GetKeyValues());
	MFVec3f& rKeyValue2 = *((MFVec3f*)rInterp2.GetKeyValues());

	if(rInterp1.GetNumberOfKeys() != rInterp2.GetNumberOfKeys())
	{
		cout << "Key  ٸ." << endl;
		cout << "Error   ." << endl;
		return;
	}

	bool bIsMore = true;
	int i=0;
	double a, b;
	double key;
	SFVec3f interpolatedKeyValue;

	while(bIsMore)
	{
		//  !
		if(i >= rKey1.GetSize()-1)
			break;

		// 1. a, b Ѵ.
		a = rKeyValue1[i].GetDistance(rKeyValue2[i]);
		b = rKeyValue1[i+1].GetDistance(rKeyValue2[i+1]);

		// 2.   ߰ Ѵ.
		key = a*(rKey1[i+1] - rKey1[i])/(a+b) + rKey1[i];


		// ࿡ ߰  i, i+1 °  ٸ  !!!
		// !!!!
			//  2 key    ־ٸ??????? ->  õ!
		if(::fabs(key - rKey1[i]) < dEPSILON)
		{
			i++;
			continue;
		}
		else if(::fabs(key - rKey1[i+1]) < dEPSILON)
		{
			i++;
			continue;
		}

		// 2.1 Encoded Path
		rInterp1.InterpolateKeyValue(
			rKey1[i], rKey1[i+1], key,
			(void*) &rKeyValue1[i], (void*) &rKeyValue1[i+1], (void*) &interpolatedKeyValue
		);
		rInterp1.InsertKey(i+1, key, (void*) &interpolatedKeyValue);

		// 2.2 Decoded Path
		rInterp2.InterpolateKeyValue(
			rKey2[i], rKey2[i+1], key,
			(void*) &rKeyValue2[i], (void*) &rKeyValue2[i+1], (void*) &interpolatedKeyValue
		);
		rInterp2.InsertKey(i+1, key, (void*) &interpolatedKeyValue);

		// 3.   
		i += 2;
	}

}

double CSPositionInterpolator::CalculateMaxDistanceOfInterval()
{
	double dMaxDistance = -DBL_MAX;
	double dCurDistance, tempDBL;
	MFVec3f& raKeyValues = *((MFVec3f*)m_paKeyValues);
	MFFloat& raKeys = *((MFFloat*)m_paKeys);

	for(int i=1; i<raKeys.GetSize(); i++)
	{
		dCurDistance = raKeyValues[i-1].GetDistance(raKeyValues[i]);
		tempDBL = raKeys[i] - raKeys[i-1];
		if(tempDBL == 0.0)
			dCurDistance = 0.0;
		else
			dCurDistance /= tempDBL;
		tempDBL = (raKeys[raKeys.GetSize()-1] - raKeys[0])/((double)raKeys.GetSize()-1);
		dCurDistance *= tempDBL;

		if(dMaxDistance < dCurDistance)
		{
			dMaxDistance = dCurDistance;
		}
	}


	m_dMaxDistanceOfInterval =  dMaxDistance;

	return dMaxDistance;
}

double CSPositionInterpolator::CalculateTotalLengthOfPath()
{
	double dTotalPath = 0.0;
	double dCurDistance;

	MFVec3f& raKeyValues = *((MFVec3f*)m_paKeyValues);
	MFFloat& raKeys = *((MFFloat*)m_paKeys);
	
	for(int i=1; i<raKeys.GetSize(); i++)
	{
		dCurDistance = raKeyValues[i-1].GetDistance(raKeyValues[i]);
		dTotalPath += dCurDistance;
	}


	m_dTotalLengthOfPath = dTotalPath;

	return dTotalPath;
}


double CSPositionInterpolator::CalculateTotalLengthOfPathOverBox()
{
	double dTotalPath = 0.0;
	double dCurDistance;
	SFVec3f max, min;

	MFVec3f& raKeyValues = *((MFVec3f*)m_paKeyValues);
	MFFloat& raKeys = *((MFFloat*)m_paKeys);

	max.x = raKeyValues[0].x;
	max.y = raKeyValues[0].y;
	max.z = raKeyValues[0].z;

	min.x = raKeyValues[0].x;
	min.y = raKeyValues[0].y;
	min.z = raKeyValues[0].z;

	for(int i=1; i<raKeys.GetSize(); i++)
	{
		dCurDistance = raKeyValues[i-1].GetDistance(raKeyValues[i]);
		dTotalPath += dCurDistance;

		if(max.x < raKeyValues[i].x)
			max.x = raKeyValues[i].x;
		if(max.y < raKeyValues[i].y)
			max.y = raKeyValues[i].y;
		if(max.z < raKeyValues[i].z)
			max.z = raKeyValues[i].z;

		if(min.x > raKeyValues[i].x)
			min.x = raKeyValues[i].x;
		if(min.y > raKeyValues[i].y)
			min.y = raKeyValues[i].y;
		if(min.z > raKeyValues[i].z)
			min.z = raKeyValues[i].z;
	}

	m_dTotalLengthOfPathOverBox = dTotalPath / max.GetDistance(min);

	return m_dTotalLengthOfPathOverBox;
}

void CSPositionInterpolator::Write_For_Debugging(char* fileName)
{
	JWArray<double>& aKeys = *m_paKeys;
	JWArray<SFVec3f>& aKeyValues = *((JWArray<SFVec3f>*) m_paKeyValues);

	ofstream file(fileName);
	for(int i=0; i<aKeys.GetSize(); i++)
	{
		file << aKeys[i] << '\t';
		file << aKeyValues[i].x << '\t';
		file << aKeyValues[i].y << '\t';
		file << aKeyValues[i].z << endl;
	}
	file.close();
}

// sowoo 2001.10.08 (Add Error Measure 6)
// Error Measure 6  ̴.
// ù/ error  Ŀ   '' Ѵ.
/*
double CSPositionInterpolator::CalculateTwoPointsError()
{
	CSError error;

	CSInterpolator* pThisInterp = new CSPositionInterpolator;
	CSInterpolator* pInterpolator = new CSPositionInterpolator;

	int nSize = m_paKeys->GetSize();

	pThisInterp->SetSize(nSize);

	MFFloat& aThisKeys = pThisInterp->GetKeys();
	MFVec3f& aThisKeyValues = *((MFVec3f*)pThisInterp->GetKeyValues());

	for(int i=0; i<nSize; i++)
	{
		aThisKeys[i] = (*((JWArray<double>*)m_paKeys))[i];
		aThisKeyValues[i] = (*((MFVec3f*)m_paKeyValues))[i];
	}


	pInterpolator->SetSize(2);

	MFFloat& aKeys = pInterpolator->GetKeys();
	MFVec3f& aKeyValues = *((MFVec3f*)pInterpolator->GetKeyValues());

	int nEnd = m_paKeys->GetSize() - 1;

	aKeys[0] = (*((JWArray<double>*)m_paKeys))[0];
	aKeys[1] = (*((JWArray<double>*)m_paKeys))[nEnd];
	aKeyValues[0] = (*((MFVec3f*)m_paKeyValues))[0];
	aKeyValues[1] = (*((MFVec3f*)m_paKeyValues))[nEnd];


	// 2.  ģ.
	pThisInterp->InsertPointWith(pInterpolator);
	pInterpolator->InsertPointWith(pThisInterp);


	// 2.1 ߰  Ѵ.
	MakePointsDouble(pThisInterp, pInterpolator);


	// 3.   Error Ѵ.
	EstimateError(pThisInterp, pInterpolator, error, 0);


	m_dTwoPointsError = error.rms;



	delete pThisInterp;
	delete pInterpolator;

	return m_dTwoPointsError;
}
*/