// compressedPositionInterpolator.cpp: implementation of the CompressedPositionInterpolator class.
//
//////////////////////////////////////////////////////////////////////


#include <stdio.h>
#include <string.h>
#include <fstream.h>

#include "compressedPositionInterpolator.h"
#include "functionsforencoding.h"
#include "JWDictionary.h"

#define AACTotal() (codec._istrm->Inbuf.nbytes) * 8 + (codec._istrm->Inbuf.nbits)

int getNbBits(int);

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CompressedPositionInterpolator::CompressedPositionInterpolator():
	m_bIsDEF(false),
	m_nKeyDigit(-1), m_nKeyValueDigit(-1),
	m_nKeyQBit(-1), m_nKeyValueQBit(-1),
	m_nDPCMOrder_X(-1), m_nDPCMOrder_Y(-1), m_nDPCMOrder_Z(-1),
	m_nCodingBit_X(-1), m_nCodingBit_Y(-1), m_nCodingBit_Z(-1),
	m_bKeyValueFlag_X(false), m_bKeyValueFlag_Y(false), m_bKeyValueFlag_Z(false),
	m_bUnaryAAC_X(false), m_bUnaryAAC_Y(false), m_bUnaryAAC_Z(false),
	m_bIsKeyPreserve(false),
	m_bLinearKeyCoder(false),
	m_bKeyQuant(false),
	m_nKeyHeaderBits(-1), m_nKeyValueHeaderBits(-1),
	m_nKeySelectionFlagBits(-1), m_nKeyBits(-1),
	m_nKeyValueBits(-1), m_nTotalBits(-1)
{

}

CompressedPositionInterpolator::~CompressedPositionInterpolator()
{

}


void CompressedPositionInterpolator::decode(char* fileName)
{
	// aac ʱȭ 
	CInStream istrm;
	CAC::_shift = 16;
	CAC_QF codec(istrm);
	istrm.opendec(fileName);

	//////////////////////////////////////////////////////////////////
	//
	//		Header Information (KeyHeader, PosIKeyValueHeader)
	//
	//////////////////////////////////////////////////////////////////
	kHeader.readBitstream(codec);
	posIKVHeader.readBitstream(codec);


		//////////////////////////////////////////////////////////////
		m_nKeyDigit = kHeader.nKeyDigit;
		m_nKeyValueDigit = posIKVHeader.nKVDigit;
		//////////////////////////////////////////////////////////////

	// byte align
	if(codec._istrm->Inbuf.nbits != 0)
	{
		int rest = 8 - codec._istrm->Inbuf.nbits;
		codec._istrm->getbits(rest);
	}

	// qf_start();
	// decoder start();
	codec.start_decode();


	// key context initialize
	k.InitializeModel();
	// keyflag context initialize
	ksFlag.InitializeModel();
	// keyValue context initialize
	posIKV.InitializeModel();


	//////////////////////////////////////////////////////////////////
	//
	//		key & KeyValue
	//
	//////////////////////////////////////////////////////////////////
	ksFlag.readBitstream(codec, kHeader, posIKVHeader.bPreserveKey);
	k.readBitstream(codec, kHeader);
	posIKV.readBitstream(codec, posIKVHeader, ksFlag.nNumOfKeyValue);


	// decoder finish();
	if(codec._istrm->Inbuf.nbits != 0)
	{
		int rest = 8 - codec._istrm->Inbuf.nbits;
		codec._istrm->getbits(rest);
	}
	codec.finish_decode();
	istrm.bfree();



	//////////////////////////////////////////////////////////////////
	//
	//		Decoding Process
	//
	//////////////////////////////////////////////////////////////////
	// key header
	kHeader.decodingProcess();
	k.decodingProcess(kHeader);

	key.SetSize(kHeader.nNumberOfKey);
	for(int i = 0;i < kHeader.nNumberOfKey;i++)
		key[i] = k.dKey[i];


	posIKVHeader.decodingProcess();

	posIKV.decodingProcess(posIKVHeader, ksFlag, key, keyValue);


	printf("Number of Decoded Bit : %d\n", AACTotal());
}

void CompressedPositionInterpolator::write(char* fileName)
{
	ofstream file;
	file.open(fileName);

	if(!file.is_open())
	{
		cout << "    . ̾~" << endl;
		cout << "File Name: " << fileName << endl;
		return;
	}

	// 1. Comment ۼѴ.
	//file << buffer << endl << endl;

	// 2. Group .
	file << "Group {" << endl;
	file << '\t' << "children [" << endl;

	// 3. key & keyValue .
	// 3.1. DEF Name ִٸ װ 

			////////////////////////////////////////////////////////////
			// ӽ÷....
			////////////////////////////////////////////////////////////
			m_bIsDEF = true;
			::strcpy(m_strDEF, "myPOSITION");
			////////////////////////////////////////////////////////////

	if(m_bIsDEF)
		file << "\t\t" << "DEF " << m_strDEF << ' ';

	// 3.2. PositionInterpolator {
	file << "PositionInterpolator {" << endl;

	// 3.3. key
	file << "\t\t\t" << "key [" << endl;



	// sowoo 2002.01.16
	// set pricision
	file.precision(m_nKeyDigit);


	for(int i=0; i<key.GetSize(); i++)
	{
		if(i%5 == 0)
			file << "\t\t\t\t";

		file << key[i];

		if(i != key.GetSize() - 1)
			file << ", ";
		else
		{
			file << endl << "\t\t\t" << " ]" << endl;
			break;
		}

		if(i%5 == 4)
			file << endl;
	}

	// 4. keyValue
	file << "\t\t\t" << "keyValue [" << endl;



	// sowoo 2002.01.16
	// set pricision
	file.precision(m_nKeyValueDigit);




	for(i=0; i<keyValue.GetSize(); i++)
	{
		if(i%2 == 0)
			file << "\t\t\t\t";

		file << keyValue[i];

		if(i != keyValue.GetSize() - 1)
			file << ", ";
		else
		{
			file << endl << "\t\t\t" << " ]" << endl;
			break;
		}

		if(i%2 == 1)
			file << endl;
	}

	file << "\t\t" << '}' << endl;
	file << '\t' << ']' << endl;
	file << '}' << endl;

	file.close();
}

void CompressedPositionInterpolator::encode()
{
	int i;

	/////////////////////////////////////////////////////////////////
	//
	//		Encoding Process
	//
	/////////////////////////////////////////////////////////////////
	ksFlag.encodingProcess(m_KeySelectionFlag);
	kHeader.encodingProcess(m_nKeyQBit, key);
	k.encodingProcess(m_bLinearKeyCoder, kHeader, key, m_bKeyQuant);
	//posIKV.encodingProcess(...);


	/////////////////////////////////////////////////////////////////
	/////////////////////////////////////////////////////////////////
	//	KeyValue Encoding Process Լ  ϰ ⿡ .
	/////////////////////////////////////////////////////////////////
	/////////////////////////////////////////////////////////////////

	////////////////////////////////////////////////////////////////
	//
	//  1. Quantization
	//
	////////////////////////////////////////////////////////////////
	// sowoo 2001.11.17 ȭ-Revisit
	// 2002.01.31 -  min/max ȭ Ѵ.
	JWArray<int> aQuantValue_X;
	JWArray<int> aQuantValue_Y;
	JWArray<int> aQuantValue_Z;

	aQuantValue_X.SetSize(m_keyValue_X.GetSize());
	aQuantValue_Y.SetSize(m_keyValue_Y.GetSize());
	aQuantValue_Z.SetSize(m_keyValue_Z.GetSize());


	double dTempMax;

	dTempMax = m_dMaxDiff + m_dMin_X;
	Quantize(m_keyValue_X, m_nKeyValueQBit, m_dMin_X, dTempMax, aQuantValue_X);

	dTempMax = m_dMaxDiff + m_dMin_Y;
	Quantize(m_keyValue_Y, m_nKeyValueQBit, m_dMin_Y, dTempMax, aQuantValue_Y);

	dTempMax = m_dMaxDiff + m_dMin_Z;
	Quantize(m_keyValue_Z, m_nKeyValueQBit, m_dMin_Z, dTempMax, aQuantValue_Z);
	////////////////////////////////////////////////////////////////



	ofstream file;
	file.open("PosIKV_Quantization_Encoder.txt");

	for(i=0; i<aQuantValue_X.GetSize(); i++)
	{
		file << aQuantValue_X[i] << '\t';
		file << aQuantValue_Y[i] << '\t';
		file << aQuantValue_Z[i] << endl;
	}

	file.close();

	////////////////////////////////////////////////////////////////
	//
	//  2. Investigate all the quantization value is the same
	//
	////////////////////////////////////////////////////////////////
	for(i=1; i<aQuantValue_X.GetSize(); i++)
	{
		// ϳ ٸ   !!!
		if(aQuantValue_X[0] != aQuantValue_X[i])
		{
			m_bKeyValueFlag_X = true; // 1
			break;
		}
	}

	for(i=1; i<aQuantValue_Y.GetSize(); i++)
	{
		// ϳ ٸ   !!!
		if(aQuantValue_Y[0] != aQuantValue_Y[i])
		{
			m_bKeyValueFlag_Y = true; // 1
			break;
		}
	}

	for(i=1; i<aQuantValue_Z.GetSize(); i++)
	{
		// ϳ ٸ   !!!
		if(aQuantValue_Z[0] != aQuantValue_Z[i])
		{
			m_bKeyValueFlag_Z = true; // 1
			break;
		}
	}




	// sowoo 2001.11.05 (Choose Best)
	////////////////////////////////////////////////////////////////
	//
	//  3. Choose the Order of DPCM for best encoding
	//
	////////////////////////////////////////////////////////////////
	//    ȿ ?
	bool bTemp;
	if(m_bKeyValueFlag_X)
	{
		m_nDPCMOrder_X = ChooseBest(
			aQuantValue_X, m_chDPCMOrder, m_nStartIndex_X, m_nKeyValueQBit, bTemp);
	}

	if(m_bKeyValueFlag_Y)
	{
		m_nDPCMOrder_Y = ChooseBest(
			aQuantValue_Y, m_chDPCMOrder, m_nStartIndex_Y, m_nKeyValueQBit, bTemp);
	}

	if(m_bKeyValueFlag_Z)
	{
		m_nDPCMOrder_Z = ChooseBest(
			aQuantValue_Z, m_chDPCMOrder, m_nStartIndex_Z, m_nKeyValueQBit, bTemp);
	}
	////////////////////////////////////////////////////////////////


	////////////////////////////////////////////////////////////////
	//
	//  3. DPCM
	//
	////////////////////////////////////////////////////////////////

	// for X
	if(m_bKeyValueFlag_X)
	{
		if(m_nDPCMOrder_X == 0)
		{
			DPCM_CircularQuantize(
				aQuantValue_X, (1 << m_nKeyValueQBit) - 1, m_nStartIndex_X);
			m_nCodingBit_X = GetRealQBit(aQuantValue_X, m_nStartIndex_X);
		}
		else
		{
			Second_DPCM_CircularQuantize(
				aQuantValue_X, (1 << m_nKeyValueQBit) - 1, m_nStartIndex_X);
			m_nCodingBit_X = GetRealQBit(aQuantValue_X, m_nStartIndex_X);
		}
	}

	// for Y
	if(m_bKeyValueFlag_Y)
	{
		if(m_nDPCMOrder_Y == 0)
		{
			DPCM_CircularQuantize(
				aQuantValue_Y, (1 << m_nKeyValueQBit) - 1, m_nStartIndex_Y);
			m_nCodingBit_Y = GetRealQBit(aQuantValue_Y, m_nStartIndex_Y);
		}
		else
		{
			Second_DPCM_CircularQuantize(
				aQuantValue_Y, (1 << m_nKeyValueQBit) - 1, m_nStartIndex_Y);
			m_nCodingBit_Y = GetRealQBit(aQuantValue_Y, m_nStartIndex_Y);
		}
	}

	// for Z
	if(m_bKeyValueFlag_Z)
	{
		if(m_nDPCMOrder_Z == 0)
		{
			DPCM_CircularQuantize(
				aQuantValue_Z, (1 << m_nKeyValueQBit) - 1, m_nStartIndex_Z);
			m_nCodingBit_Z = GetRealQBit(aQuantValue_Z, m_nStartIndex_Z);
		}
		else
		{
			Second_DPCM_CircularQuantize(
				aQuantValue_Z, (1 << m_nKeyValueQBit) - 1, m_nStartIndex_Z);
			m_nCodingBit_Z = GetRealQBit(aQuantValue_Z, m_nStartIndex_Z);
		}
	}


	m_aQuantValue_X.SetSize(aQuantValue_X.GetSize());
	m_aQuantValue_Y.SetSize(aQuantValue_Y.GetSize());
	m_aQuantValue_Z.SetSize(aQuantValue_Z.GetSize());

	for(i=0; i<aQuantValue_X.GetSize(); i++)
	{
		m_aQuantValue_X[i] = aQuantValue_X[i];
		m_aQuantValue_Y[i] = aQuantValue_Y[i];
		m_aQuantValue_Z[i] = aQuantValue_Z[i];
	}


	file.open("PosIKV_DPCM_Encoder.txt");

	for(i=0; i<aQuantValue_X.GetSize(); i++)
	{
		file << aQuantValue_X[i] << '\t';
		file << aQuantValue_Y[i] << '\t';
		file << aQuantValue_Z[i] << endl;
	}

	file.close();

}


void CompressedPositionInterpolator::SetDPDMChoice(char choice)
{
	m_chDPCMOrder = choice;
}


void CompressedPositionInterpolator::SetKeyAndKeyValue(
	JWArray<double>& keys,
	JWArray<double>& keyValue_X, JWArray<double>& keyValue_Y, JWArray<double>& keyValue_Z,
	JWArray<int>& flag_X, JWArray<int>& flag_Y, JWArray<int>& flag_Z,
	bool bIsKeyPreserve
)
{
	m_bIsKeyPreserve = bIsKeyPreserve;

	if(bIsKeyPreserve)
	{
		// X Flag Ѵ.
		int nSize = flag_X.GetSize();

		key.SetSize(nSize);
		m_KeySelectionFlag.SetSize(nSize);

		nSize = GetNumberOfThisFromArray(flag_X, 1);

		m_keyValue_X.SetSize(nSize);
		m_keyValue_Y.SetSize(nSize);
		m_keyValue_Z.SetSize(nSize);

		for(int i=0, j=0; i<flag_X.GetSize(); i++)
		{
			key[i] = keys[i];

			m_KeySelectionFlag[i] = flag_X[i];

			if(flag_X[i])
			{
				m_keyValue_X[j] = keyValue_X[i];
				m_keyValue_Y[j] = keyValue_Y[i];
				m_keyValue_Z[j] = keyValue_Z[i];
				j++;
			}
		}
	}
	else
	{
		int nSize = GetNumberOfThisFromArray(flag_X, 1);

		key.SetSize(nSize);
		m_keyValue_X.SetSize(nSize);
		m_keyValue_Y.SetSize(nSize);
		m_keyValue_Z.SetSize(nSize);

		for(int i=0, j=0; i<flag_X.GetSize(); i++)
		{
			if(flag_X[i])
			{
				key[j] = keys[i];

				m_keyValue_X[j] = keyValue_X[i];
				m_keyValue_Y[j] = keyValue_Y[i];
				m_keyValue_Z[j] = keyValue_Z[i];
				j++;
			}
		}
	}
}

void CompressedPositionInterpolator::SetMinMax(
	double dMin_X, double dMin_Y, double dMin_Z,
	double dMax_X, double dMax_Y, double dMax_Z
)
{
	m_dMin_X = dMin_X;
	m_dMin_Y = dMin_Y;
	m_dMin_Z = dMin_Z;
	m_dMax_X = dMax_X;
	m_dMax_Y = dMax_Y;
	m_dMax_Z = dMax_Z;

	m_dMaxDiff = m_dMax_X - m_dMin_X;

	if(m_dMaxDiff < (m_dMax_Y - m_dMin_Y))
		m_dMaxDiff = m_dMax_Y - m_dMin_Y;
	if(m_dMaxDiff < (m_dMax_Z - m_dMin_Z))
		m_dMaxDiff = m_dMax_Z - m_dMin_Z;
}

void CompressedPositionInterpolator::SetDigit(int nKeyDigit, int nKeyValueDigit)
{
	m_nKeyDigit = nKeyDigit;
	m_nKeyValueDigit = nKeyValueDigit;
}


void CompressedPositionInterpolator::SetQBit(int nKeyQBit, int nKeyValueQBit)
{
	m_nKeyQBit = nKeyQBit;
	m_nKeyValueQBit = nKeyValueQBit;
}

void CompressedPositionInterpolator::SetStartIndex(
	int nStartIndex_X, int nStartIndex_Y, int nStartIndex_Z
)
{
	m_nStartIndex_X = nStartIndex_X;
	m_nStartIndex_Y = nStartIndex_Y;
	m_nStartIndex_Z = nStartIndex_Z;
}

void CompressedPositionInterpolator::SetUnaryAAC(bool bUnaryX, bool bUnaryY, bool bUnaryZ)
{
	m_bUnaryAAC_X = bUnaryX;
	m_bUnaryAAC_Y = bUnaryY;
	m_bUnaryAAC_Z = bUnaryZ;
}


void CompressedPositionInterpolator::makeBitstream(char* fileName)
{
	CAC::_shift = 0;
	COutStream ostrm;
	CAC_QF codec(ostrm);
	bool success = ostrm.openenc(fileName);

	int nTotalKeyBitTemp = 0;

	int nTempAACBits = AACTotal_ENC();
	kHeader.makeBitstream(codec, nTotalKeyBitTemp);
	m_nKeyHeaderBits = AACTotal_ENC() - nTempAACBits;


	nTempAACBits = AACTotal_ENC();
	/////////////////////////////////////////////////////////////////
	//
	//	Header Information
	//
	/////////////////////////////////////////////////////////////////
	codec._ostrm->putbit((m_bIsKeyPreserve)? 1:0);
	codec._ostrm->putbits(m_nKeyValueQBit, 5);
	codec._ostrm->putbit((m_bKeyValueFlag_X)? 1: 0);
	codec._ostrm->putbit((m_bKeyValueFlag_Y)? 1: 0);
	codec._ostrm->putbit((m_bKeyValueFlag_Z)? 1: 0);
	codec._ostrm->putbits(m_nKeyValueDigit, 4);


	int nBits = getNbBits(m_nKeyValueQBit);

	// sowoo 2001.11.24
	// ȭ    RealQBit ʿ!
	if(m_bKeyValueFlag_X)
	{
		// sowoo 2001.10.09 (1st DPCM)
		codec._ostrm->putbit((m_nDPCMOrder_X)? 1:0);
		// UnaryAAC  ΰ?
		codec._ostrm->putbit((m_bUnaryAAC_X)? 1:0);

		if(!m_bUnaryAAC_X)
		{
			codec._ostrm->putbits(m_nCodingBit_X, nBits);

			// sowoo 2001.11.09 (Intra Coding)
			codec._ostrm->putbit((m_nStartIndex_X)? 1:0);
		}
	}

	if(m_bKeyValueFlag_Y)
	{
		// sowoo 2001.10.09 (1st DPCM)
		codec._ostrm->putbit((m_nDPCMOrder_Y)? 1:0);
		// UnaryAAC  ΰ?
		codec._ostrm->putbit((m_bUnaryAAC_Y)? 1:0);

		if(!m_bUnaryAAC_Y)
		{
			codec._ostrm->putbits(m_nCodingBit_Y, nBits);

			// sowoo 2001.11.09 (Intra Coding)
			codec._ostrm->putbit((m_nStartIndex_Y)? 1:0);
		}
	}

	if(m_bKeyValueFlag_Z)
	{
		// sowoo 2001.10.09 (1st DPCM)
		codec._ostrm->putbit((m_nDPCMOrder_Z)? 1:0);
		// UnaryAAC  ΰ?
		codec._ostrm->putbit((m_bUnaryAAC_Z)? 1:0);

		if(!m_bUnaryAAC_Z)
		{
			codec._ostrm->putbits(m_nCodingBit_Z, nBits);

			// sowoo 2001.11.09 (Intra Coding)
			codec._ostrm->putbit((m_nStartIndex_Z)? 1:0);
		}
	}



	// sowoo 2001.10.09 (1st DPCM)
	// Intra Value Coding
	if((m_nStartIndex_X == 1) && (m_bKeyValueFlag_X == 1) && (m_bUnaryAAC_X != 1))
		codec._ostrm->putbits(m_aQuantValue_X[0], m_nKeyValueQBit);

	if((m_nStartIndex_Y == 1) && (m_bKeyValueFlag_Y == 1) && (m_bUnaryAAC_Y != 1))
		codec._ostrm->putbits(m_aQuantValue_Y[0], m_nKeyValueQBit);

	if((m_nStartIndex_Z == 1) && (m_bKeyValueFlag_Z == 1) && (m_bUnaryAAC_Z != 1))
		codec._ostrm->putbits(m_aQuantValue_Z[0], m_nKeyValueQBit);


	//  Ǿ ִ MaxDiff Ѵ.
	double dMaxDiff;
	double dMax = m_dMax_X;
	int nWhichAxis = 0;

	dMaxDiff = m_dMax_X - m_dMin_X;

	if(dMaxDiff < (m_dMax_Y - m_dMin_Y))
	{
		dMaxDiff = m_dMax_Y - m_dMin_Y;
		dMax = m_dMax_Y;
		nWhichAxis = 1;
	}

	if(dMaxDiff < (m_dMax_Z - m_dMin_Z))
	{
		dMaxDiff = m_dMax_Z - m_dMin_Z;
		dMax = m_dMax_Z;
		nWhichAxis = 2;
	}



	///////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////
	// __> to file
	ofstream file;
	file.open("PosIKVHeader_Encoder.txt");

	//codec._ostrm->putbit((m_bIsKeyPreserve)? 1:0);
	file << "m_bIsKeyPreserve: " << m_bIsKeyPreserve << endl;

	//codec._ostrm->putbits(m_nKeyValueQBit, 5);
	file << "m_nKeyValueQBit: " << m_nKeyValueQBit << endl;

	//codec._ostrm->putbit((m_bKeyValueFlag_X)? 1: 0);
	file << "m_bKeyValueFlag_X: " << m_bKeyValueFlag_X << endl;

	//codec._ostrm->putbit((m_bKeyValueFlag_Y)? 1: 0);
	file << "m_bKeyValueFlag_Y: " << m_bKeyValueFlag_Y << endl;

	//codec._ostrm->putbit((m_bKeyValueFlag_Z)? 1: 0);
	file << "m_bKeyValueFlag_Z: " << m_bKeyValueFlag_Z << endl;

	//codec._ostrm->putbits(m_nKeyValueDigit, 4);
	file << "m_nKeyValueDigit: " << m_nKeyValueDigit << endl;


	//int nBits = getNbBits(m_nKeyValueQBit);

	// sowoo 2001.11.24
	// ȭ    RealQBit ʿ!
	if(m_bKeyValueFlag_X)
	{
		// sowoo 2001.10.09 (1st DPCM)
		//codec._ostrm->putbit((m_nDPCMOrder_X)? 1:0);
		file << "m_nDPCMOrder_X: " << m_nDPCMOrder_X << endl;

		// UnaryAAC  ΰ?
		//codec._ostrm->putbit((m_bUnaryAAC_X)? 1:0);
		file << "m_bUnaryAAC_X: " << m_bUnaryAAC_X << endl;

		if(!m_bUnaryAAC_X)
		{
			//codec._ostrm->putbits(m_nCodingBit_X, nBits);
			file << "m_nCodingBit_X: " << m_nCodingBit_X << endl;

			// sowoo 2001.11.09 (Intra Coding)
			//codec._ostrm->putbit((m_nStartIndex_X)? 1:0);
			file << "m_nStartIndex_X: " << m_nStartIndex_X << endl;
		}
	}

	if(m_bKeyValueFlag_Y)
	{
		// sowoo 2001.10.09 (1st DPCM)
		//codec._ostrm->putbit((m_nDPCMOrder_Y)? 1:0);
		file << "m_nDPCMOrder_Y: " << m_nDPCMOrder_Y << endl;

		// UnaryAAC  ΰ?
		//codec._ostrm->putbit((m_bUnaryAAC_Y)? 1:0);
		file << "m_bUnaryAAC_Y: " << m_bUnaryAAC_Y << endl;

		if(!m_bUnaryAAC_Y)
		{
			//codec._ostrm->putbits(m_nCodingBit_Y, nBits);
			file << "m_nCodingBit_Y: " << m_nCodingBit_Y << endl;

			// sowoo 2001.11.09 (Intra Coding)
			//codec._ostrm->putbit((m_nStartIndex_Y)? 1:0);
			file << "m_nStartIndex_Y: " << m_nStartIndex_Y << endl;
		}
	}

	if(m_bKeyValueFlag_Z)
	{
		// sowoo 2001.10.09 (1st DPCM)
		//codec._ostrm->putbit((m_nDPCMOrder_Z)? 1:0);
		file << "m_nDPCMOrder_Z: " << m_nDPCMOrder_Z << endl;

		// UnaryAAC  ΰ?
		//codec._ostrm->putbit((m_bUnaryAAC_Z)? 1:0);
		file << "m_bUnaryAAC_Z: " << m_bUnaryAAC_Z << endl;

		if(!m_bUnaryAAC_Z)
		{
			//codec._ostrm->putbits(m_nCodingBit_Z, nBits);
			file << "m_nCodingBit_Z: " << m_nCodingBit_Z << endl;

			// sowoo 2001.11.09 (Intra Coding)
			//codec._ostrm->putbit((m_nStartIndex_Z)? 1:0);
			file << "m_nStartIndex_Z: " << m_nStartIndex_Z << endl;
		}
	}



	// sowoo 2001.10.09 (1st DPCM)
	// Intra Value Coding
	if((m_nStartIndex_X == 1) && (m_bKeyValueFlag_X == 1) && (m_bUnaryAAC_X != 1))
		//codec._ostrm->putbits(m_aQuantValue_X[0], m_nKeyValueQBit);
		file << "m_aQuantValue_X[0]: " << m_aQuantValue_X[0] << endl;

	if((m_nStartIndex_Y == 1) && (m_bKeyValueFlag_Y == 1) && (m_bUnaryAAC_Y != 1))
		//codec._ostrm->putbits(m_aQuantValue_Y[0], m_nKeyValueQBit);
		file << "m_aQuantValue_Y[0]: " << m_aQuantValue_Y[0] << endl;

	if((m_nStartIndex_Z == 1) && (m_bKeyValueFlag_Z == 1) && (m_bUnaryAAC_Z != 1))
		//codec._ostrm->putbits(m_aQuantValue_Z[0], m_nKeyValueQBit);
		file << "m_aQuantValue_Z[0]: " << m_aQuantValue_Z[0] << endl;

	file.close();
	///////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////




	EncodeReducedMinMaxData(
		codec, 
		m_dMin_X, m_dMin_Y, m_dMin_Z,
		dMax,

		// sowoo 2001.11.24
		// ̹  KeyValueDigit  digit   ʴ´.
		m_nKeyValueDigit,
		nWhichAxis
	);





	//////////////////////////////////////////////////////////////////////
	//
	//		byte align
	//									jameskim 20010326	
	//////////////////////////////////////////////////////////////////////
	if(codec._ostrm->Outbuf.nbits != 0)
	{
		codec._ostrm->putbits(1, 8 - codec._ostrm->Outbuf.nbits);
	}


	m_nKeyValueHeaderBits = AACTotal_ENC() - nTempAACBits;





	codec.start_encode();

	// key context initialize
	k.InitializeModel();
	// keyflag context initialize
	ksFlag.InitializeModel();
	// keyValue context initialize
	posIKV.InitializeModel();


	nTempAACBits = AACTotal_ENC();
	//////////////////////////////////////////////////////////////////////
	//
	//		KeySelection Flag Encoding
	//
	//////////////////////////////////////////////////////////////////////
	ksFlag.makeBitstream(codec, m_bIsKeyPreserve);

	m_nKeySelectionFlagBits = AACTotal_ENC() - nTempAACBits;


	nTempAACBits = AACTotal_ENC();
	//////////////////////////////////////////////////////////////////////
	//
	//		Key Encoding
	//
	//////////////////////////////////////////////////////////////////////
	k.makeBitstream(codec, kHeader, nTotalKeyBitTemp);
	m_nKeyBits = AACTotal_ENC() - nTempAACBits;


	nTempAACBits = AACTotal_ENC();
	//////////////////////////////////////////////////////////////////////
	//
	//		KeyValue Encoding
	//
	//////////////////////////////////////////////////////////////////////
	bool bIsKeyValueAAC = 
		(m_bKeyValueFlag_X || m_bKeyValueFlag_Y || m_bKeyValueFlag_Z);


	// for X
	if(m_bKeyValueFlag_X)
	{
		if(m_bUnaryAAC_X)
		{
			JWDictionary knockDownEncoder_X;
			knockDownEncoder_X.Encode(m_aQuantValue_X);
			knockDownEncoder_X.MakeBitstream_AAC(codec);
		}
		else
		{
			// 6.1 MJH : 6. keyValue ڵ  successive quantization ϱ
			// - function : for each component (x,y,z), do successive quantization and aac
			// - parameters : codec, key, interpolator.q_x_kvb array) 
			// - AAC context : found_context, not_found_context --> entropy.h & cpp 

			// encodeKeyValue_SQ_? Լ...
			//  kV  mid, min, max  found(bool)   (array)
			// 6.1.1 : Max Ѵ. (from interpolator.q_x(y,z)_kvb array)
			// 6.1.2 : for each level, until all range is 0 (min==max), do...
			// 6.1.2.1 : for each kV
			// 6.1.2.1.1 : mid point Ѵ. -->  mid = (int)((max-min)/2) + 1 + min 
			// 6.1.2.1.2 : get 0 or 1
			// 6.1.2.1.3 : aac it using context according to found

			encodeData_SQ(codec, m_nCodingBit_X+1, m_aQuantValue_X, m_nStartIndex_X);
		}
	}

	// for Y
	if(m_bKeyValueFlag_Y)
	{
		if(m_bUnaryAAC_Y)
		{
			JWDictionary knockDownEncoder_Y;
			knockDownEncoder_Y.Encode(m_aQuantValue_Y);
			knockDownEncoder_Y.MakeBitstream_AAC(codec);
		}
		else
		{
			// 6.1 MJH : 6. keyValue ڵ  successive quantization ϱ
			// - function : for each component (x,y,z), do successive quantization and aac
			// - parameters : codec, key, interpolator.q_x_kvb array) 
			// - AAC context : found_context, not_found_context --> entropy.h & cpp 

			// encodeKeyValue_SQ_? Լ...
			//  kV  mid, min, max  found(bool)   (array)
			// 6.1.1 : Max Ѵ. (from interpolator.q_x(y,z)_kvb array)
			// 6.1.2 : for each level, until all range is 0 (min==max), do...
			// 6.1.2.1 : for each kV
			// 6.1.2.1.1 : mid point Ѵ. -->  mid = (int)((max-min)/2) + 1 + min 
			// 6.1.2.1.2 : get 0 or 1
			// 6.1.2.1.3 : aac it using context according to found

			encodeData_SQ(codec, m_nCodingBit_Y+1, m_aQuantValue_Y, m_nStartIndex_Y);
		}
	}


	// for Z
	if(m_bKeyValueFlag_Z)
	{
		if(m_bUnaryAAC_Z)
		{
			JWDictionary knockDownEncoder_Z;
			knockDownEncoder_Z.Encode(m_aQuantValue_Z);
			knockDownEncoder_Z.MakeBitstream_AAC(codec);
		}
		else
		{
			// 6.1 MJH : 6. keyValue ڵ  successive quantization ϱ
			// - function : for each component (x,y,z), do successive quantization and aac
			// - parameters : codec, key, interpolator.q_x_kvb array) 
			// - AAC context : found_context, not_found_context --> entropy.h & cpp 

			// encodeKeyValue_SQ_? Լ...
			//  kV  mid, min, max  found(bool)   (array)
			// 6.1.1 : Max Ѵ. (from interpolator.q_x(y,z)_kvb array)
			// 6.1.2 : for each level, until all range is 0 (min==max), do...
			// 6.1.2.1 : for each kV
			// 6.1.2.1.1 : mid point Ѵ. -->  mid = (int)((max-min)/2) + 1 + min 
			// 6.1.2.1.2 : get 0 or 1
			// 6.1.2.1.3 : aac it using context according to found

			encodeData_SQ(codec, m_nCodingBit_Z+1, m_aQuantValue_Z, m_nStartIndex_Z);
		}
	}
	m_nKeyValueBits = AACTotal_ENC() - nTempAACBits;

	// encoder finish();
	codec.finish_encode();
	if(codec._ostrm->Outbuf.nbits != 0)
	{
		codec._ostrm->putbits(1, 8 - codec._ostrm->Outbuf.nbits);
	}


	// Total encoding bits
	m_nTotalBits = (codec._ostrm->Outbuf.nbytes)*8 + codec._ostrm->Outbuf.nbits;

	ostrm.close_enc();
	ostrm.bfree();
}



void CompressedPositionInterpolator::SetLinearKeyFlag(bool bLinearKeyCoder)
{
	m_bLinearKeyCoder = bLinearKeyCoder;
}

void CompressedPositionInterpolator::SetMinimieKeyQuantErrorFlag(bool bKeyQuant)
{
	m_bKeyQuant = bKeyQuant;
}