#if !defined __FUNCTIONS_FOR_ENCODING___
#define __FUNCTIONS_FOR_ENCODING___

#include "JWTemplArray.h"
#include "JWToken.h"
#include "JWInterpolator.h"
#include "SInterpolator.h"
#include "SError.h"
#include "stream.h"
#include "entropy.h"

void WriteIM1Text(char* fileName, JWInterpolator* pInterpolator, char* buffer);

void ParseIM1Text(JWToken& token, JWInterpolator*& pInterpolator);

void Quantize(
	double* aValues,
	int nSize,
	int nQBit,
	double dMin, double dMax, 
	int* aQuantValues
);

void Quantize(
	JWArray<double>& aValues,
	int nQBit,
	double dMin, double dMax, 
	JWArray<int>& aQuantValues
);

void Quantize(
	JWArray<double>& aValues,
	int nQBit,
	double dMin, double dMax, 
	int* aQuantValues
);

void QuantizeMM(
	JWArray<double>& aValues,
	int nQBit,
	double& dMin, double& dMax, 
	int* aQuantValues
);


int ChooseBest(
	JWArray<int>& aSelectedValues,
	char chType,
	int nStart, int nQBit,

	// sowoo 2001.11.08 (Intra Coding)
	//   ( )
	bool& bIsIntra
);



void DPCM_CircularQuantize(JWArray<int>& aValues, int nMax, int nStart);
void DPCM_CircularQuantize(
	JWArray<int>& aValues, JWArray<int>& aFinalValues, int nMax, int nStart);

void Second_DPCM_CircularQuantize(JWArray<int>& aValues, int nMax, int nStart);
void Second_DPCM_CircularQuantize(JWArray<int>& aValues, JWArray<int>& aFinalValues, int nMax, int nStart);


int GetDigit(double dValue);
int GetLargestDigit(double key);
int GetLeadingZero(double key);

int GetMantissa(double dValue, int nDigit);

int GetRealQBit(JWArray<int>& paValues, int nStart);


void DeQuantize(
	int* aQuantValues,
	int nSize,
	int nQBit,
	double dMin, double dMax, 
	double* aDeQuantValues
);

void DeQuantize(
	JWArray<int>& aQuantValues,
	int nQBit,
	double dMin, double dMax, 
	JWArray<double>& aDeQuantValues
);


double QuantizationError(double* aDeQuantValues, JWArray<double>& aValues);

double QuantizationError(
	double* aDeQuantValues, JWArray<double>& aValues,

	// Key Ͽ  Ѵ.
	double* aDeQuantKeys, JWArray<double>& aKeys
);

double QuantizationError(double* aValues, int nSize, double* aDeQuantValues);


int GetMin(int* aValues, int nSize, int nStart);

int GetMax(int* aValues, int nSize, int nStart);

int GetMin(JWArray<int>& aValues, int nStart);

int GetMax(JWArray<int>& aValues, int nStart);


void Convert(CSInterpolator* pInterp1, JWInterpolator* pInterp2);

void EstimateError(
	CSInterpolator* pInterp1, CSInterpolator* pInterp2,
	CSError& error,
	int nNorm_type
);



// sowoo 2001.10.11 (Min/Max Reduce)
//  Ȯ  ƴϴ.
int GetDigitBits(int nXDigit, int nYDigit, int nZDigit, bool& bIsOneDigit);

int GetTotalMinMaxBits(
	int nDigit_X, int nDigit_Y, int nDigit_Z,
	int nDigit_diff
);

// sowoo 2001.10.12 (Min/Max Reduce)
void EncodeReducedMinMaxData(
	CAC_QF& codec, 
	double dXMin, double dYMin, double dZMin,
	double fMaxDiff,

	// sowoo 2001.11.24
	// ̹  KeyValueDigit  digit   ʴ´.
	int nKeyValueDigit,
	int nWhichAxis
);


void encodeData_SQ(CAC_QF& codec, int qstep, JWArray<int>& pdQ, int start);


#endif