/***********************************************************************
This software module was originally developed by James D. K. Kim, 
Sang Oak Woo and Shin Jun Lee(Samsung Advance Institute of Technology)
in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)
standard. This software module is an implementation of a part of one or
more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4
Systems (ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 
Systems (ISO/IEC 14496-1) free license to this software module or 
modifications thereof for use in hardware or software products claiming
conformance to the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to 
use this software module in hardware or software products are advised that
its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Samsung Advance Institute of Technology
retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using
the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
This copyright notice must be included in all copies or derivative works.
Copyright ?2001
************************************************************************

For more information or to receive an up to date version of this module,
you may call Sang Oak Woo at sowoo@sait.samsung.co.kr
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Interpolator Compression
FILE:       PosIKeyValue.cpp
AUTHOR:     Sang Oak Woo (SAMSUNG AIT)
OVERVIEW:   Implementation of Interpolator Compression
-------------------------------------------------------------------------*/


// PosIKeyValue.cpp: implementation of the PosIKeyValue class.
//
//////////////////////////////////////////////////////////////////////
#include "defs.h"
//#include "stdafx.h"

#include <fstream.h>

#include "PosIKeyValue.h"
#include "FunctionsForDecoding.h"
#include "ArithmeticDecoder.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

PosIKeyValue::PosIKeyValue()
{
	keyValue_X = NULL;
	keyValue_Y = NULL;
	keyValue_Z = NULL;
}

PosIKeyValue::~PosIKeyValue()
{
	if(keyValue_X != NULL)
		delete[] keyValue_X;
	if(keyValue_Y != NULL)
		delete[] keyValue_Y;
	if(keyValue_Z != NULL)
		delete[] keyValue_Z;
}

void PosIKeyValue::readBitstream(
	CAC_QF& codec, PosIKeyValueHeader kvHeader,
	int nNumberOfKeyValue
)
{
	keyValue_X = new int[nNumberOfKeyValue];
	keyValue_Y = new int[nNumberOfKeyValue];
	keyValue_Z = new int[nNumberOfKeyValue];

	int i;
	if(kvHeader.x_keyvalue_flag == 1)
	{
		if(kvHeader.bIsUnaryAAC_X == 1)
			for(i=0; i< nNumberOfKeyValue; i++)
				//
				decodeUnaryAAC(codec, &keyValue_X[i], kvXSignContext, kvXUContext);
		else
			decodeSQAAC(codec, keyValue_X, kvHeader.nKVCodingBit_X, 
				kvXSignContext, kvXMaxValueContext, kvXFoundContext,
				kvXNotFoundContext, nNumberOfKeyValue, kvHeader.nStartIndex_X);
	}

	if(kvHeader.y_keyvalue_flag == 1)
	{
		if(kvHeader.bIsUnaryAAC_Y == 1)
			for(i=0; i< nNumberOfKeyValue; i++)
				//
				decodeUnaryAAC(codec, &keyValue_Y[i], kvYSignContext, kvYUContext);
		else
			decodeSQAAC(codec, keyValue_Y, kvHeader.nKVCodingBit_Y, 
				kvYSignContext, kvYMaxValueContext, kvYFoundContext,
				kvYNotFoundContext, nNumberOfKeyValue, kvHeader.nStartIndex_Y);
	}

	if(kvHeader.z_keyvalue_flag == 1)
	{
		if(kvHeader.bIsUnaryAAC_Z == 1)
			for(i=0; i< nNumberOfKeyValue; i++)
				//
				decodeUnaryAAC(codec, &keyValue_Z[i], kvZSignContext, kvZUContext);
		else
			decodeSQAAC(codec, keyValue_Z, kvHeader.nKVCodingBit_Z, 
				kvZSignContext, kvZMaxValueContext, kvZFoundContext,
				kvZNotFoundContext, nNumberOfKeyValue, kvHeader.nStartIndex_Z);
	}

/*
	ofstream file;
	file.open("PosIKV_DPCM_Decoder.txt");

	for(i=0; i<nNumberOfKeyValue ; i++)
	{
		file << keyValue_X[i] << '\t';
		file << keyValue_Y[i] << '\t';
		file << keyValue_Z[i] << endl;
	}

	file.close();
*/
}


void PosIKeyValue::decodingProcess(
	PosIKeyValueHeader& kvHeader,
	KeySelectionFlag& ksFlag,
	JWArray<double>& key,
	JWArray<SFVec3f>& keyValue
)
{
	int nNumberOfKeyValue = ksFlag.nNumOfKeyValue; //keyValue.GetSize();
	int nNumberOfKey = key.GetSize();

	double* adTemp_X = new double[nNumberOfKeyValue];
	double* adTemp_Y = new double[nNumberOfKeyValue];
	double* adTemp_Z = new double[nNumberOfKeyValue];

	double* adTemp2 = new double[nNumberOfKey];

	int i;
	if(kvHeader.x_keyvalue_flag == 1)
	{
		// Intra Coding
		if((kvHeader.nStartIndex_X == 1) && (kvHeader.x_keyvalue_flag == 1))
			keyValue_X[0] = kvHeader.firstKV_X;

		// Inverse DPCM
		if(kvHeader.nKVDPCMOrder_X ==  0)
			// 1st-order DPCM
			Inverse_DPCM_CircularQuantize(
				keyValue_X, nNumberOfKeyValue, (1<<kvHeader.nKVQBit) -1,
				kvHeader.nStartIndex_X
			);
		else
			// 2nd-order DPCM
			Inverse_Second_DPCM_CircularQuantize(
				keyValue_X, nNumberOfKeyValue, (1<<kvHeader.nKVQBit) -1,
				kvHeader.nStartIndex_X
			);

		// Inverse Quantization & Normalization
		Inverse_Quantize(
			keyValue_X, adTemp_X, nNumberOfKeyValue,
			kvHeader.kvMinMax.fMin_X, kvHeader.kvMinMax.fMax_X,
			kvHeader.nKVQBit, kvHeader.nKVDigit
		);

		if(kvHeader.bPreserveKey)
		{
			// Interpolator Synthesizer
			InterpolatorSythesizer(
				adTemp_X, adTemp2, ksFlag.keyFlag,
				nNumberOfKeyValue, nNumberOfKey,
				key
			);

			for(i=0; i<nNumberOfKey; i++)
				keyValue[i].x = adTemp2[i];
		}
		else
		{
			for(i=0; i<nNumberOfKey; i++)
				keyValue[i].x = adTemp_X[i];
		}
	}
	else
		for(i=0; i<nNumberOfKey; i++)
			keyValue[i].x = kvHeader.kvMinMax.fMin_X;


	if(kvHeader.y_keyvalue_flag == 1)
	{
		// Intra Coding
		if((kvHeader.nStartIndex_Y == 1) && (kvHeader.y_keyvalue_flag == 1))
			keyValue_Y[0] = kvHeader.firstKV_Y;

		// Inverse DPCM
		if(kvHeader.nKVDPCMOrder_Y ==  0)
			// 1st-order DPCM
			Inverse_DPCM_CircularQuantize(
				keyValue_Y, nNumberOfKeyValue, (1<<kvHeader.nKVQBit) -1,
				kvHeader.nStartIndex_Y
			);
		else
			// 2nd-order DPCM
			Inverse_Second_DPCM_CircularQuantize(
				keyValue_Y, nNumberOfKeyValue, (1<<kvHeader.nKVQBit) -1,
				kvHeader.nStartIndex_Y
			);

		// Inverse Quantization & Normalization
		Inverse_Quantize(
			keyValue_Y, adTemp_Y, nNumberOfKeyValue,
			kvHeader.kvMinMax.fMin_Y, kvHeader.kvMinMax.fMax_Y,
			kvHeader.nKVQBit, kvHeader.nKVDigit
		);

		if(kvHeader.bPreserveKey)
		{
			// Interpolator Synthesizer
			InterpolatorSythesizer(
				adTemp_Y, adTemp2, ksFlag.keyFlag,
				nNumberOfKeyValue, nNumberOfKey,
				key
			);

			for(i=0; i<nNumberOfKey; i++)
				keyValue[i].y = adTemp2[i];
		}
		else
		{
			for(i=0; i<nNumberOfKey; i++)
				keyValue[i].y = adTemp_Y[i];
		}
	}
	else
		for(i=0; i<nNumberOfKey; i++)
			keyValue[i].y = kvHeader.kvMinMax.fMin_Y;

	if(kvHeader.z_keyvalue_flag == 1)
	{
		// Intra Coding
		if((kvHeader.nStartIndex_Z == 1) && (kvHeader.z_keyvalue_flag == 1))
			keyValue_Z[0] = kvHeader.firstKV_Z;

		// Inverse DPCM
		if(kvHeader.nKVDPCMOrder_Z ==  0)
			// 1st-order DPCM
			Inverse_DPCM_CircularQuantize(
				keyValue_Z, nNumberOfKeyValue, (1<<kvHeader.nKVQBit) -1,
				kvHeader.nStartIndex_Z
			);
		else
			// 2nd-order DPCM
			Inverse_Second_DPCM_CircularQuantize(
				keyValue_Z, nNumberOfKeyValue, (1<<kvHeader.nKVQBit) -1,
				kvHeader.nStartIndex_Z
			);

		// Inverse Quantization & Normalization
		Inverse_Quantize(
			keyValue_Z, adTemp_Z, nNumberOfKeyValue,
			kvHeader.kvMinMax.fMin_Z, kvHeader.kvMinMax.fMax_Z,
			kvHeader.nKVQBit, kvHeader.nKVDigit
		);

		if(kvHeader.bPreserveKey)
		{
			// Interpolator Synthesizer
			InterpolatorSythesizer(
				adTemp_Z, adTemp2, ksFlag.keyFlag,
				nNumberOfKeyValue, nNumberOfKey,
				key
			);

			for(i=0; i<nNumberOfKey; i++)
				keyValue[i].z = adTemp2[i];
		}
		else
		{
			for(i=0; i<nNumberOfKey; i++)
				keyValue[i].z = adTemp_Z[i];
		}
	}
	else
		for(i=0; i<nNumberOfKey; i++)
			keyValue[i].z = kvHeader.kvMinMax.fMin_Z;

/*
	ofstream file;
	file.open("PosIKV_Quantization_Decoder.txt");

	for(i=0; i<nNumberOfKeyValue; i++)
	{
		file << keyValue_X[i] << '\t';
		file << keyValue_Y[i] << '\t';
		file << keyValue_Z[i] << endl;
	}

	file.close();
*/

	delete[] adTemp_X;
	delete[] adTemp_Y;
	delete[] adTemp_Z;

	delete[] adTemp2;
}


void PosIKeyValue::InitializeModel()
{
	kvXSignContext.reset();
	kvXUContext.reset();
	kvXMaxValueContext.reset();
	kvXFoundContext.reset();
	kvXNotFoundContext.reset();

	kvYSignContext.reset();
	kvYUContext.reset();
	kvYMaxValueContext.reset();
	kvYFoundContext.reset();
	kvYNotFoundContext.reset();

	kvZSignContext.reset();
	kvZUContext.reset();
	kvZMaxValueContext.reset();
	kvZFoundContext.reset();
	kvZNotFoundContext.reset();
}

