/***********************************************************************
This software module was originally developed by James D. K. Kim, 
Sang Oak Woo and Shin Jun Lee(Samsung Advance Institute of Technology)
in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)
standard. This software module is an implementation of a part of one or
more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4
Systems (ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 
Systems (ISO/IEC 14496-1) free license to this software module or 
modifications thereof for use in hardware or software products claiming
conformance to the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to 
use this software module in hardware or software products are advised that
its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Samsung Advance Institute of Technology
retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using
the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
This copyright notice must be included in all copies or derivative works.
Copyright ?2001
************************************************************************

For more information or to receive an up to date version of this module,
you may call Sang Oak Woo at sowoo@sait.samsung.co.kr
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Interpolator Compression
FILE:       PosIKeyValueHeader.cpp
AUTHOR:     Sang Oak Woo (SAMSUNG AIT)
OVERVIEW:   Implementation of Interpolator Compression
-------------------------------------------------------------------------*/

// PosIKeyValueHeader.cpp: implementation of the PosIKeyValueHeader class.
//
//////////////////////////////////////////////////////////////////////
#include "defs.h"
//#include "stdafx.h"

#include "PosIKeyValueHeader.h"
//#include "FunctionsForDecoding.h"

int getNbBits(int);

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

PosIKeyValueHeader::PosIKeyValueHeader():
	nStartIndex_X(0), nStartIndex_Y(0), nStartIndex_Z(0)
{

}

PosIKeyValueHeader::~PosIKeyValueHeader()
{

}

void PosIKeyValueHeader::readBitstream(CAC_QF& codec)
{
	bPreserveKey = codec._istrm->getbit();
	nKVQBit = codec._istrm->getbits(5);
	x_keyvalue_flag = codec._istrm->getbit();
	y_keyvalue_flag = codec._istrm->getbit();
	z_keyvalue_flag = codec._istrm->getbit();
	nKVDigit = codec._istrm->getbits(4);

	int nBits = getNbBits(nKVQBit);

	// if x_keyvalue_flag == 1
	if(x_keyvalue_flag == 1)
	{
		nKVDPCMOrder_X = codec._istrm->getbit();
		bIsUnaryAAC_X = codec._istrm->getbit();

		if(bIsUnaryAAC_X != 1)
		{
			nKVCodingBit_X = codec._istrm->getbits(nBits);
			nStartIndex_X = codec._istrm->getbit();
		}
	}

	// if y_keyvalue_flag == 1
	if(y_keyvalue_flag == 1)
	{
		nKVDPCMOrder_Y = codec._istrm->getbit();
		bIsUnaryAAC_Y = codec._istrm->getbit();

		if(bIsUnaryAAC_Y != 1)
		{
			nKVCodingBit_Y = codec._istrm->getbits(nBits);
			nStartIndex_Y = codec._istrm->getbit();
		}
	}

	// if x_keyvalue_flag == 1
	if(z_keyvalue_flag == 1)
	{
		nKVDPCMOrder_Z = codec._istrm->getbit();
		bIsUnaryAAC_Z = codec._istrm->getbit();

		if(bIsUnaryAAC_Z != 1)
		{
			nKVCodingBit_Z = codec._istrm->getbits(nBits);
			nStartIndex_Z = codec._istrm->getbit();
		}
	}

	// if Intra coding is used
	if((nStartIndex_X == 1) && (x_keyvalue_flag == 1))
		firstKV_X = codec._istrm->getbits(nKVQBit);
	if((nStartIndex_Y == 1) && (y_keyvalue_flag == 1))
		firstKV_Y = codec._istrm->getbits(nKVQBit);
	if((nStartIndex_Z == 1) && (z_keyvalue_flag == 1))
		firstKV_Z = codec._istrm->getbits(nKVQBit);



/*	/////////////////////////////////////////////////////////
	// to file
	ofstream file;
	file.open("PosIKVHeader_Decoder.txt");

	//bPreserveKey = codec._istrm->getbit();
	file << "bPreserveKey: " << bPreserveKey << endl;

	//nKVQBit = codec._istrm->getbits(5);
	file << "nKVQBit: " << nKVQBit << endl;

	//x_keyvalue_flag = codec._istrm->getbit();
	file << "x_keyvalue_flag: " << x_keyvalue_flag << endl;

	//y_keyvalue_flag = codec._istrm->getbit();
	file << "y_keyvalue_flag: " << y_keyvalue_flag << endl;

	//z_keyvalue_flag = codec._istrm->getbit();
	file << "z_keyvalue_flag: " << z_keyvalue_flag << endl;

	//nKVDigit = codec._istrm->getbits(4);
	file << "nKVDigit: " << nKVDigit << endl;

//	int nBits = getNbBits(nKVQBit);

	// if x_keyvalue_flag == 1
	if(x_keyvalue_flag == 1)
	{
		//nKVDPCMOrder_X = codec._istrm->getbit();
		file << "nKVDPCMOrder_X: " << nKVDPCMOrder_X << endl;

		//bIsUnaryAAC_X = codec._istrm->getbit();
		file << "bIsUnaryAAC_X: " << bIsUnaryAAC_X << endl;

		if(bIsUnaryAAC_X != 1)
		{
			//nKVCodingBit_X = codec._istrm->getbits(nBits);
			file << "nKVCodingBit_X: " << nKVCodingBit_X << endl;

			//nStartIndex_X = codec._istrm->getbit();
			file << "nStartIndex_X: " << nStartIndex_X << endl;
		}
	}

	// if y_keyvalue_flag == 1
	if(y_keyvalue_flag == 1)
	{
		//nKVDPCMOrder_Y = codec._istrm->getbit();
		file << "nKVDPCMOrder_Y: " << nKVDPCMOrder_Y << endl;

		//bIsUnaryAAC_Y = codec._istrm->getbit();
		file << "bIsUnaryAAC_Y: " << bIsUnaryAAC_Y << endl;

		if(bIsUnaryAAC_Y != 1)
		{
			//nKVCodingBit_Y = codec._istrm->getbits(nBits);
			file << "nKVCodingBit_Y: " << nKVCodingBit_Y << endl;

			//nStartIndex_Y = codec._istrm->getbit();
			file << "nStartIndex_Y: " << nStartIndex_Y << endl;
		}
	}

	// if z_keyvalue_flag == 1
	if(z_keyvalue_flag == 1)
	{
		//nKVDPCMOrder_Z = codec._istrm->getbit();
		file << "nKVDPCMOrder_Z: " << nKVDPCMOrder_Z << endl;

		//bIsUnaryAAC_Z = codec._istrm->getbit();
		file << "bIsUnaryAAC_Z: " << bIsUnaryAAC_Z << endl;

		if(bIsUnaryAAC_Z != 1)
		{
			//nKVCodingBit_Z = codec._istrm->getbits(nBits);
			file << "nKVCodingBit_Z: " << nKVCodingBit_Z << endl;

			//nStartIndex_Z = codec._istrm->getbit();
			file << "nStartIndex_Z: " << nStartIndex_Z << endl;
		}
	}

	// if Intra coding is used
	if((nStartIndex_X == 1) && (x_keyvalue_flag == 1))
		//firstKV_X = codec._istrm->getbits(nKVQBit);
		file << "firstKV_X: " << firstKV_X << endl;
	if((nStartIndex_Y == 1) && (y_keyvalue_flag == 1))
		//firstKV_Y = codec._istrm->getbits(nKVQBit);
		file << "firstKV_Y: " << firstKV_Y << endl;
	if((nStartIndex_Z == 1) && (z_keyvalue_flag == 1))
		//firstKV_Z = codec._istrm->getbits(nKVQBit);
		file << "firstKV_Z: " << firstKV_Z << endl;

	file.close();
	///////////////////////////////////////////////////////////////
*/


	kvMinMax.readBitstream(codec, nKVDigit);
}


void PosIKeyValueHeader::decodingProcess()
{
	kvMinMax.decodingProcess();
}
