/***********************************************************************
This software module was originally developed by James D. K. Kim, 
Sang Oak Woo and Shin Jun Lee(Samsung Advance Institute of Technology)
in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)
standard. This software module is an implementation of a part of one or
more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4
Systems (ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 
Systems (ISO/IEC 14496-1) free license to this software module or 
modifications thereof for use in hardware or software products claiming
conformance to the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to 
use this software module in hardware or software products are advised that
its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Samsung Advance Institute of Technology
retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using
the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
This copyright notice must be included in all copies or derivative works.
Copyright  2001
************************************************************************

For more information or to receive an up to date version of this module,
you may contact James D. K. Kim at jacobkim@sait.samsung.co.kr
----------------------------------------------------------------------*/

// Quantizer.cpp: implementation of the Quantizer class.
//
//////////////////////////////////////////////////////////////////////

//#include "stdafx.h"
#include "Quantizer.h"

#include <math.h>


#define MAX(x, y)       ( (x) < (y) ? (y) : (x) )
#define MIN(x, y)       ( (x) < (y) ? (x) : (y) )
#define Nonlinearity    2.0

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

JWQuantizer::JWQuantizer()
{

}

JWQuantizer::~JWQuantizer()
{

}

//unsigned long int Quantizer::Quantize(double mini, double maxi, int nbits, double v)
long int JWQuantizer::Quantize(double mini, double maxi, int nbits, double v)
{
	if(v<=mini)
		return 0;
	if(v>=maxi) 
		return (1<<nbits)-1;
	  
	//unsigned long int tmp = (unsigned long int) floor( ((v-mini)/(maxi-mini)*((1<<nbits)-1)) + 0.5);
	long int tmp = (long int) floor( ((v-mini)/(maxi-mini)*((1<<nbits)-1)) + 0.5);
  
	return tmp;
}


void JWQuantizer::QuantizerInitialize(int bits)
{
	int i;
	int nBitsQuantization = MAX((bits-1), 1);
	QuantizerScale = 1.0;
	for ( i = 0 ; i < nBitsQuantization ; ++i ) 
		QuantizerScale *= 2.0;
	QuantizerOffset = QuantizerScale;
}


void JWQuantizer::UniformQuantizer(double *v, unsigned long int *QuantizedIntegerData)
{
	int               sign_bit;
	double            TmpData;
	unsigned long int QuantData;
	TmpData = *v;
	TmpData = MAX(-1.0, TmpData);
	TmpData = MIN( 1.0, TmpData);

	if ( TmpData < 0.0 ) { sign_bit = 1; TmpData *= -1.0; }
	else                   sign_bit = 0;

	QuantData = (unsigned long int)floor( TmpData * ( QuantizerScale - 1.0 ) + 0.5 );

	if ( sign_bit ) QuantData = (unsigned long int)floor( QuantizerOffset + 0.5 ) - QuantData;
	else            QuantData = (unsigned long int)floor( QuantizerOffset + 0.5 ) + QuantData;

	*QuantizedIntegerData = QuantData;

	//TmpData = ( ( (double)QuantData - QuantizerOffset ) / ( QuantizerScale - 1.0 ) );
	//*v = TmpData;
}


//double Quantizer::InvUniformQuantizer(unsigned long int QuantizedIntegerData)
double JWQuantizer::InvUniformQuantizer(long int QuantizedIntegerData)
{
	double TmpData;

	TmpData = ( ( (double)QuantizedIntegerData - QuantizerOffset ) / ( QuantizerScale - 1.0 ) );

	return TmpData;
}

//double Quantizer::InvQuantize(double mini, double maxi, int nbits, unsigned long int v) 
double JWQuantizer::InvQuantize(double mini, double maxi, int nbits, int v) 
{

	if(v==0) 
		return mini;
	if(v==(int) ((1<<nbits)-1)) 
		return maxi;
  
	return (double) (mini+(maxi-mini)/((double) (1<<nbits)-1)*(double) v);
}


void JWQuantizer::AdaptiveTransform(double *SourceKeyValues, int Direction)
{
	int     i, sign_flag;

	if ( Direction ) {
		for ( i = 0 ; i < 3 ; ++i ) {
			if ( SourceKeyValues[i] < 0.0 ) { 
				sign_flag = 1;
				SourceKeyValues[i] *= -1.0;
			}
			else {
				sign_flag = 0;
			}
			
			SourceKeyValues[i] = pow(SourceKeyValues[i], (1.0/Nonlinearity));
			
			if ( sign_flag ) SourceKeyValues[i] *= -1.0;
		}
	}
	else {
		for ( i = 0 ; i < 3 ; ++i ) {
			if ( SourceKeyValues[i] < 0.0 ) {
			  sign_flag = 1;
			  SourceKeyValues[i] *= -1.0; 
			}
			else {
				sign_flag = 0;
			}
			
			SourceKeyValues[i] = pow(SourceKeyValues[i], (Nonlinearity));
			
			if ( sign_flag ) SourceKeyValues[i] *= -1.0;
		}
	}
}
