/***********************************************************************
This software module was originally developed by James D. K. Kim, 
Sang Oak Woo and Shin Jun Lee(Samsung Advance Institute of Technology)
in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)
standard. This software module is an implementation of a part of one or
more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4
Systems (ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 
Systems (ISO/IEC 14496-1) free license to this software module or 
modifications thereof for use in hardware or software products claiming
conformance to the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to 
use this software module in hardware or software products are advised that
its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Samsung Advance Institute of Technology
retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using
the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
This copyright notice must be included in all copies or derivative works.
Copyright  2001
************************************************************************

For more information or to receive an up to date version of this module,
you may contact James D. K. Kim at jacobkim@sait.samsung.co.kr
----------------------------------------------------------------------*/

// QuatError.cpp: implementation of the QuatError class.
//
//////////////////////////////////////////////////////////////////////



//#include "stdafx.h"
#include "QuatError.h"
#include "SFRotation.h"
#include "Quaternion.h"

//typedef	 SFRotation Rotation;

#include <math.h>
#include <string.h>
#include <stdlib.h>


#define MAX(x, y)       ( (x) < (y) ? (y) : (x) )
#define MIN(x, y)       ( (x) < (y) ? (x) : (y) )
#define ABS(x)          ( (x) < 0.0 ? -1.0*(x) : (x) ) // HELLO! //
#define DELTA			1.0E-50

// jameskim 20011126
#define SmallestNumber 1.0E-16                    // Modefied: 11.26


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
//extern int test;

QuatError::QuatError()
{
	srcfp = NULL;
	decfp = NULL;
	outfp = NULL;
	nBits = 0;
}

QuatError::QuatError(FILE* in_srcfp, FILE* in_decfp, FILE* in_outfp, int in_nBits)
{
	srcfp = in_srcfp;
	decfp = in_decfp;
	outfp = in_outfp;
	nBits = in_nBits;
}

QuatError::~QuatError()
{

}


double QuatError::CalculateErrorAngle(Quaternion srcQuat, Quaternion decQuat)
{
	double errorAngle = 0.0;
	Quaternion	errorQuat(1.0, 0.0, 0.0, 0.0);

	srcQuat.Conjugate();

	errorQuat = decQuat * srcQuat;

	errorQuat.s = MAX(-1.0, errorQuat.s);
	errorQuat.s = MIN(1.0, errorQuat.s);

	// start jacobkim 20010107 
	srcQuat.Conjugate();
	if((srcQuat.s == decQuat.s) && (srcQuat.x == decQuat.x) && (srcQuat.y == decQuat.y)
		&& (srcQuat.z == decQuat.z))
		return 0.0;
	else
		return(fabs(2.0 * acos(errorQuat.s)));

	//return(fabs(2.0 * acos(errorQuat.s)));
}


void QuatError::ComputeError(double & rms, double & peak, double & dur, int orgNKey,
							 JWArray<double>& orgKey, JWArray<Quaternion>& orgQuat, 
							 int decNKey, JWArray<double>& decKey, JWArray<Quaternion>& decQuat, int IdenticalArray)  // HELLO! //
{
	//FILE *file;
	double MaxKey, MinKey;
	int input_key_counter = 0;
	int output_key_counter = 0;
	int tmp_key_counter;
	int sli_upper, sli_lower;
	int key_counter, NumberOfKeys;
	// jameskim 20010803
	double KeyNeck, ErrorAngleNeck;     // Neck !!
	double InputQuaternionNeck[4];      // Neck !!
	double OutputQuaternionNeck[4];     // Neck !!
	// end - jameskim 20010803


	MaxKey = 0.0;
	MinKey = 1.0;

	if ( !IdenticalArray ) NumberOfKeys = orgNKey + decNKey;
	else                   NumberOfKeys = orgNKey;
	double* MixedKeys = (double *)calloc(NumberOfKeys, sizeof(double));
	double** InputQuaternions = (double **)calloc(NumberOfKeys, sizeof(double *));
	double** OutputQuaternions = (double **)calloc(NumberOfKeys, sizeof(double *));
	for ( key_counter = 0 ; key_counter < NumberOfKeys ; ++key_counter ) {
		InputQuaternions[key_counter] = (double *)calloc(4, sizeof(double));
		OutputQuaternions[key_counter] = (double *)calloc(4, sizeof(double));
	}


	if ( !IdenticalArray ) {
		input_key_counter = 0; output_key_counter = 0;                              
		for ( key_counter = 0 ; key_counter < NumberOfKeys ; ++key_counter ) {      
			if(input_key_counter == orgNKey)
			{
					MixedKeys[key_counter] = decKey[output_key_counter];                
					OutputQuaternions[key_counter][0] = decQuat[output_key_counter].s;  
					OutputQuaternions[key_counter][1] = decQuat[output_key_counter].x;  
					OutputQuaternions[key_counter][2] = decQuat[output_key_counter].y;  
					OutputQuaternions[key_counter][3] = decQuat[output_key_counter].z;  
					InputQuaternions[key_counter][0] = -2.0;                            
					InputQuaternions[key_counter][1] =  0.0;                            
					InputQuaternions[key_counter][2] =  0.0;                            
					InputQuaternions[key_counter][3] =  0.0;                            
					++output_key_counter;
			}
			else if(output_key_counter == decNKey)
			{
					MixedKeys[key_counter] = orgKey[input_key_counter];                 
					InputQuaternions[key_counter][0] = orgQuat[input_key_counter].s;    
					InputQuaternions[key_counter][1] = orgQuat[input_key_counter].x;    
					InputQuaternions[key_counter][2] = orgQuat[input_key_counter].y;    
					InputQuaternions[key_counter][3] = orgQuat[input_key_counter].z;    
					OutputQuaternions[key_counter][0] = -2.0;                           
					OutputQuaternions[key_counter][1] =  0.0;                           
					OutputQuaternions[key_counter][2] =  0.0;                           
					OutputQuaternions[key_counter][3] =  0.0;                           
					++input_key_counter;  
			}
			else
			{
				if ( orgKey[input_key_counter] < decKey[output_key_counter] ) {         
					MixedKeys[key_counter] = orgKey[input_key_counter];                 
					InputQuaternions[key_counter][0] = orgQuat[input_key_counter].s;    
					InputQuaternions[key_counter][1] = orgQuat[input_key_counter].x;    
					InputQuaternions[key_counter][2] = orgQuat[input_key_counter].y;    
					InputQuaternions[key_counter][3] = orgQuat[input_key_counter].z;    
					OutputQuaternions[key_counter][0] = -2.0;                           
					OutputQuaternions[key_counter][1] =  0.0;                           
					OutputQuaternions[key_counter][2] =  0.0;                           
					OutputQuaternions[key_counter][3] =  0.0;                           
					++input_key_counter;                                                
				}                                                                       
				else {                                                                  
					MixedKeys[key_counter] = decKey[output_key_counter];                
					OutputQuaternions[key_counter][0] = decQuat[output_key_counter].s;  
					OutputQuaternions[key_counter][1] = decQuat[output_key_counter].x;  
					OutputQuaternions[key_counter][2] = decQuat[output_key_counter].y;  
					OutputQuaternions[key_counter][3] = decQuat[output_key_counter].z;  
					InputQuaternions[key_counter][0] = -2.0;                            
					InputQuaternions[key_counter][1] =  0.0;                            
					InputQuaternions[key_counter][2] =  0.0;                            
					InputQuaternions[key_counter][3] =  0.0;                            
					++output_key_counter;                                               
				}                                                                       
			}
		}                                                                           
		MinKey = MixedKeys[0];                                                      
		MaxKey = MixedKeys[NumberOfKeys-1];                                         
	}
	else {
		for ( key_counter = 0 ; key_counter < NumberOfKeys ; ++key_counter ) {
			MixedKeys[key_counter] = orgKey[key_counter];
			InputQuaternions[key_counter][0] = orgQuat[key_counter].s;
			InputQuaternions[key_counter][1] = orgQuat[key_counter].x;
			InputQuaternions[key_counter][2] = orgQuat[key_counter].y;
			InputQuaternions[key_counter][3] = orgQuat[key_counter].z;
			OutputQuaternions[key_counter][0] = decQuat[key_counter].s;
			OutputQuaternions[key_counter][1] = decQuat[key_counter].x;
			OutputQuaternions[key_counter][2] = decQuat[key_counter].y;
			OutputQuaternions[key_counter][3] = decQuat[key_counter].z;
			MaxKey = MAX(MixedKeys[key_counter], MaxKey);
			MinKey = MIN(MixedKeys[key_counter], MinKey);
		}
	}


	if ( !IdenticalArray ) {
	
		if ( InputQuaternions[0][0] <= -2.0 ) {
			tmp_key_counter = 0;
			do {
				++tmp_key_counter;
			} while ( InputQuaternions[tmp_key_counter][0] <= -2.0 );
			InputQuaternions[0][0] = InputQuaternions[tmp_key_counter][0];
			InputQuaternions[0][1] = InputQuaternions[tmp_key_counter][1];
			InputQuaternions[0][2] = InputQuaternions[tmp_key_counter][2];
			InputQuaternions[0][3] = InputQuaternions[tmp_key_counter][3];
		}
		if ( InputQuaternions[NumberOfKeys-1][0] <= -2.0 ) {
			tmp_key_counter = NumberOfKeys - 1;
			do {
				--tmp_key_counter;
			} while ( InputQuaternions[tmp_key_counter][0] <= -2.0 );
			InputQuaternions[NumberOfKeys-1][0] = InputQuaternions[tmp_key_counter][0];
			InputQuaternions[NumberOfKeys-1][1] = InputQuaternions[tmp_key_counter][1];
			InputQuaternions[NumberOfKeys-1][2] = InputQuaternions[tmp_key_counter][2];
			InputQuaternions[NumberOfKeys-1][3] = InputQuaternions[tmp_key_counter][3];
		}
		if ( OutputQuaternions[0][0] <= -2.0 ) {
			tmp_key_counter = 0;
			do {
				++tmp_key_counter;
			} while ( OutputQuaternions[tmp_key_counter][0] <= -2.0 );
			OutputQuaternions[0][0] = OutputQuaternions[tmp_key_counter][0];
			OutputQuaternions[0][1] = OutputQuaternions[tmp_key_counter][1];
			OutputQuaternions[0][2] = OutputQuaternions[tmp_key_counter][2];
			OutputQuaternions[0][3] = OutputQuaternions[tmp_key_counter][3];
		}
		if ( OutputQuaternions[NumberOfKeys-1][0] <= -2.0 ) {
			tmp_key_counter = NumberOfKeys - 1;
			do {
				--tmp_key_counter;
			} while ( OutputQuaternions[tmp_key_counter][0] <= -2.0 );
			OutputQuaternions[NumberOfKeys-1][0] = OutputQuaternions[tmp_key_counter][0];
			OutputQuaternions[NumberOfKeys-1][1] = OutputQuaternions[tmp_key_counter][1];
			OutputQuaternions[NumberOfKeys-1][2] = OutputQuaternions[tmp_key_counter][2];
			OutputQuaternions[NumberOfKeys-1][3] = OutputQuaternions[tmp_key_counter][3];
		}

		for ( key_counter = 0 ; key_counter < NumberOfKeys ; ++key_counter ) {
			if ( InputQuaternions[key_counter][0] <= -2.0 ) {
				sli_upper = key_counter;
				sli_lower = key_counter;
				do {
					++sli_upper;
				} while ( InputQuaternions[sli_upper][0] <= -2.0 );
				do {
					--sli_lower;
				} while ( InputQuaternions[sli_lower][0] <= -2.0 );

				SphericalLinearInterpolator(InputQuaternions[key_counter],
											InputQuaternions[sli_lower],
											InputQuaternions[sli_upper],
											MixedKeys[key_counter],
											MixedKeys[sli_lower],
											MixedKeys[sli_upper]);
			}
			if ( OutputQuaternions[key_counter][0] <= -2.0 ) {
				sli_upper = key_counter;
				sli_lower = key_counter;
				do {
					++sli_upper;
				} while ( OutputQuaternions[sli_upper][0] <= -2.0 );
				do {
					--sli_lower;
				} while ( OutputQuaternions[sli_lower][0] <= -2.0 );

				SphericalLinearInterpolator(OutputQuaternions[key_counter],
											OutputQuaternions[sli_lower],
											OutputQuaternions[sli_upper],
											MixedKeys[key_counter],
											MixedKeys[sli_lower],
											MixedKeys[sli_upper]);
			}
		}
	}

	double PeakError = 0.0;
	double NodeMeanError = 0.0;
	double TmpData, ErrorAngle1, ErrorAngle2;
	double TimeDuration;

	for ( key_counter = 0 ; key_counter < (NumberOfKeys-1) ; ++key_counter ) { 
		// ignore the error if the difference between successive keys is below 1.0E-50
		if ( DELTA < (MixedKeys[key_counter+1]-MixedKeys[key_counter]) ) {
			// jameskim 20010803	
			ErrorAngle1 = InnerProduct(InputQuaternions[key_counter  ], OutputQuaternions[key_counter  ]);
			ErrorAngle2 = InnerProduct(InputQuaternions[key_counter+1], OutputQuaternions[key_counter+1]);
			if ( ErrorAngle1 < 0.0 ) ErrorAngle1 = -ErrorAngle1; ErrorAngle1 = acos(ErrorAngle1);
			if ( ErrorAngle2 < 0.0 ) ErrorAngle2 = -ErrorAngle2; ErrorAngle2 = acos(ErrorAngle2);
			KeyNeck = MixedKeys[key_counter+1] - MixedKeys[key_counter];
			if ( KeyNeck < DELTA ) {
				KeyNeck = MixedKeys[key_counter+1];
				ErrorAngleNeck = ( ErrorAngle1 + ErrorAngle2 ) / 2.0;
			}
			else {
				if ( (ErrorAngle1+ErrorAngle2) < DELTA ) {
					KeyNeck *= 0.5;
				}
				else {
					KeyNeck *= ErrorAngle1 / ( ErrorAngle1 + ErrorAngle2 );
				}
				KeyNeck += MixedKeys[key_counter];
				SphericalLinearInterpolator(&InputQuaternionNeck[0],
					InputQuaternions[key_counter],
					InputQuaternions[key_counter+1],
					KeyNeck,
					MixedKeys[key_counter],
					MixedKeys[key_counter+1]);
				
				if(key_counter == 297)
				{ 
					printf("hi");
				}
				
				SphericalLinearInterpolator(&OutputQuaternionNeck[0],
					OutputQuaternions[key_counter],
					OutputQuaternions[key_counter+1],
					KeyNeck,
					MixedKeys[key_counter],
					MixedKeys[key_counter+1]);
				ErrorAngleNeck = InnerProduct(&InputQuaternionNeck[0], &OutputQuaternionNeck[0]);
				if ( ErrorAngleNeck < 0.0 ) ErrorAngleNeck = -ErrorAngleNeck; ErrorAngleNeck = acos(ErrorAngleNeck);
			}
			TmpData = ErrorAngle1 * ErrorAngle1 + ErrorAngleNeck * ErrorAngleNeck + ErrorAngle1 * ErrorAngleNeck;
			TmpData = 4.0 * ( KeyNeck - MixedKeys[key_counter] ) * TmpData / 3.0;
			NodeMeanError += TmpData;

			TmpData = ErrorAngleNeck * ErrorAngleNeck + ErrorAngle2 * ErrorAngle2 + ErrorAngleNeck * ErrorAngle2;
			TmpData = 4.0 * ( MixedKeys[key_counter+1] - KeyNeck ) * TmpData / 3.0;
			NodeMeanError += TmpData;

			PeakError = MAX((2.0*ErrorAngle1), PeakError); 
			PeakError = MAX((2.0*ErrorAngle2), PeakError); 
			PeakError = MAX((2.0*ErrorAngleNeck), PeakError); 
		} // jameskim 20010828 end if
	}

	TimeDuration = MaxKey -	MinKey;
	
	rms = NodeMeanError;
	peak = PeakError;
	dur = TimeDuration;
	
	for ( key_counter = 0 ; key_counter < NumberOfKeys ; ++key_counter ) { // HELLO! //
		free(InputQuaternions[key_counter]);
		free(OutputQuaternions[key_counter]);
	}

	free(MixedKeys);
	free(InputQuaternions);
	free(OutputQuaternions);

}


void QuatError::DataSwapping(double *Data1, double *Data2)
{
	double TmpData;
	TmpData = *Data1;
	*Data1 = *Data2;
	*Data2 = TmpData;
	return;
}

double QuatError::Norm(double *SrcQuaternion)
{
	double TmpData;
	TmpData = SrcQuaternion[0] * SrcQuaternion[0]
	        + SrcQuaternion[1] * SrcQuaternion[1]
			+ SrcQuaternion[2] * SrcQuaternion[2]
			+ SrcQuaternion[3] * SrcQuaternion[3];
	if ( 1.0 <= TmpData ) return(1.0);
	return(sqrt(TmpData));
}


double QuatError::InnerProduct(double *SrcQuaternion1, double *SrcQuaternion2)
{
	double TmpData, norm1, norm2;

	if ( ( SrcQuaternion1[0] == SrcQuaternion2[0] ) &&
		 ( SrcQuaternion1[1] == SrcQuaternion2[1] ) &&
		 ( SrcQuaternion1[2] == SrcQuaternion2[2] ) && 
		 ( SrcQuaternion1[3] == SrcQuaternion2[3] ) )
	{
		return 1.0;
	}
	if ( ( SrcQuaternion1[0] == -SrcQuaternion2[0] ) &&  
		 ( SrcQuaternion1[1] == -SrcQuaternion2[1] ) &&  
		 ( SrcQuaternion1[2] == -SrcQuaternion2[2] ) &&  
		 ( SrcQuaternion1[3] == -SrcQuaternion2[3] ) )   
	{                                                    
		return -1.0;                                     
	}                                                    
		
	norm1 = Norm(SrcQuaternion1);
	norm2 = Norm(SrcQuaternion2);
	TmpData = SrcQuaternion1[0] * SrcQuaternion2[0]
	        + SrcQuaternion1[1] * SrcQuaternion2[1]
			+ SrcQuaternion1[2] * SrcQuaternion2[2]
			+ SrcQuaternion1[3] * SrcQuaternion2[3];
	TmpData /= norm1 * norm2;
	if ( ( ABS(TmpData) <= ( norm1 * SmallestNumber ) ) || ( ABS(TmpData) <= ( norm2 * SmallestNumber ) ) ) TmpData = 0.0; // Modefied: 11.26
	if ( TmpData <= -1.0 ) return (-1.0);
	if ( 1.0 <= TmpData  ) return ( 1.0);
	return (TmpData);
}

void QuatError::SphericalLinearInterpolator(double *TargetQuaternion, double *LowerQuaternion, double *UpperQuaternion, double TargetTime, double LowerTime, double UpperTime)
{
	double phi, eta, norm, TimeDuration, weightL, weightU; 
	TimeDuration = UpperTime - LowerTime;
	if ( TimeDuration < DELTA ) {                          
		TargetQuaternion[0] = LowerQuaternion[0];
		TargetQuaternion[1] = LowerQuaternion[1];
		TargetQuaternion[2] = LowerQuaternion[2];
		TargetQuaternion[3] = LowerQuaternion[3];
	}
	else {
		norm = InnerProduct(LowerQuaternion, UpperQuaternion);
		eta = ( TargetTime - LowerTime ) / TimeDuration;
		if ( eta < DELTA )
		{
			TargetQuaternion[0] = LowerQuaternion[0];
			TargetQuaternion[1] = LowerQuaternion[1];
			TargetQuaternion[2] = LowerQuaternion[2];
			TargetQuaternion[3] = LowerQuaternion[3];
		}
		else if ( (1-eta) < DELTA )                        
		{
			TargetQuaternion[0] = UpperQuaternion[0];
			TargetQuaternion[1] = UpperQuaternion[1];
			TargetQuaternion[2] = UpperQuaternion[2];
			TargetQuaternion[3] = UpperQuaternion[3];
		}
		else
		{
			phi = acos(fabs(norm));
			if ( phi < DELTA ) {                       
				TargetQuaternion[0] = UpperQuaternion[0];
				TargetQuaternion[1] = UpperQuaternion[1];
				TargetQuaternion[2] = UpperQuaternion[2];
				TargetQuaternion[3] = UpperQuaternion[3];
			}
			else {
				if ( /*((double)3.1415926535897932384338/2.0) < phi*/ norm < 0.0 ) {
					Quaternion temp;
					temp.s = - UpperQuaternion[0];
					temp.x = - UpperQuaternion[1];
					temp.y = - UpperQuaternion[2];
					temp.z = - UpperQuaternion[3];

					//phi = (double)3.1415926535897932384338 - phi;
					//phi = acos(0.0) - phi;
					weightL = sin((1.0-eta)*phi)/sin(phi);              
					weightU = sin(     eta *phi)/sin(phi);              
					TargetQuaternion[0] = LowerQuaternion[0] * weightL + temp.s * weightU; 
					TargetQuaternion[1] = LowerQuaternion[1] * weightL + temp.x * weightU; 
					TargetQuaternion[2] = LowerQuaternion[2] * weightL + temp.y * weightU; 
					TargetQuaternion[3] = LowerQuaternion[3] * weightL + temp.z * weightU; 
					//TargetQuaternion[0] = LowerQuaternion[0] * weightL - UpperQuaternion[0] * weightU; 
					//TargetQuaternion[1] = LowerQuaternion[1] * weightL - UpperQuaternion[1] * weightU; 
					//TargetQuaternion[2] = LowerQuaternion[2] * weightL - UpperQuaternion[2] * weightU; 
					//TargetQuaternion[3] = LowerQuaternion[3] * weightL - UpperQuaternion[3] * weightU; 
				}
				else {
					weightL = sin((1.0-eta)*phi)/sin(phi);              
					weightU = sin(     eta *phi)/sin(phi);              
					TargetQuaternion[0] = LowerQuaternion[0] * weightL + UpperQuaternion[0] * weightU; 
					TargetQuaternion[1] = LowerQuaternion[1] * weightL + UpperQuaternion[1] * weightU; 
					TargetQuaternion[2] = LowerQuaternion[2] * weightL + UpperQuaternion[2] * weightU; 
					TargetQuaternion[3] = LowerQuaternion[3] * weightL + UpperQuaternion[3] * weightU; 
				}
				norm = Norm(TargetQuaternion);
				TargetQuaternion[0] /= norm;
				TargetQuaternion[1] /= norm;
				TargetQuaternion[2] /= norm;
				TargetQuaternion[3] /= norm;
			}
		}
	}
	return;
}