/***********************************************************************
This software module was originally developed by James D. K. Kim, 
Sang Oak Woo and Shin Jun Lee(Samsung Advance Institute of Technology)
in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)
standard. This software module is an implementation of a part of one or
more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4
Systems (ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 
Systems (ISO/IEC 14496-1) free license to this software module or 
modifications thereof for use in hardware or software products claiming
conformance to the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to 
use this software module in hardware or software products are advised that
its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Samsung Advance Institute of Technology
retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using
the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
This copyright notice must be included in all copies or derivative works.
Copyright  2001
************************************************************************

For more information or to receive an up to date version of this module,
you may contact James D. K. Kim at jacobkim@sait.samsung.co.kr
----------------------------------------------------------------------*/

// Quaternion.cpp: implementation of the Quaternion class.
//
//////////////////////////////////////////////////////////////////////

//#include "stdafx.h"
#include "Quaternion.h"
#include "SFRotation.h"

#include <math.h>

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
#define M_PI            3.1415926535897932384338
#define DELTA			1.0E-50


Quaternion::Quaternion()
{
	s = 0.0;
	x = 0.0;
	y = 0.0;
	z = 0.0;

	Normalize();
}

Quaternion::~Quaternion()
{

}

	
Quaternion::Quaternion(double in_s, double in_x, double in_y, double in_z)
{
	s = in_s;
	x = in_x;
	y = in_y;
	z = in_z;
}

Quaternion::Quaternion(const Quaternion& src)
{
	s = src.s;
	x = src.x;
	y = src.y;
	z = src.z;
}

// Quaternion Multiplication
Quaternion Quaternion::operator* (const Quaternion &r_operand) const   
{
	
	Quaternion temp;

	temp.s = s * r_operand.s - x * r_operand.x - y * r_operand.y - z * r_operand.z;
	temp.x = s * r_operand.x + x * r_operand.s + y * r_operand.z - z * r_operand.y;
	temp.y = s * r_operand.y - x * r_operand.z + y * r_operand.s + z * r_operand.x;
	temp.z = s * r_operand.z + x * r_operand.y - y * r_operand.x + z * r_operand.s;
	//s = Q2[0]; x = Q2[1]; y = Q2[2]; z = Q2[3];

	
	/*
	double A, B, C, D, E, F, G, H;

	A = (s + x)*(r_operand.s + r_operand.x);
	B = (z - y)*(r_operand.y - r_operand.z);
	C = (x - s)*(r_operand.y + r_operand.z);
	D = (y + z)*(r_operand.x - r_operand.s);
	E = (x + z)*(r_operand.x + r_operand.y);
	F = (x - z)*(r_operand.x - r_operand.y);
	G = (s + y)*(r_operand.s - r_operand.z);
	H = (s - y)*(r_operand.s + r_operand.z);

	temp.s = B + (-E - F + G + H) /2;
	temp.x = A - (E + F + G + H)/2; 
	temp.y = -C + (E - F + G - H)/2;
	temp.z = - D + (E - F - G + H)/2;
	*/
	temp.Normalize();
	return temp;
}

// Quaternion Addition
Quaternion Quaternion::operator+ (const Quaternion &r_operand) const   
{
	Quaternion temp;

	temp.s = s + r_operand.s;
	temp.x = x + r_operand.x;
	temp.y = y + r_operand.y;
	temp.z = z + r_operand.z;

	return temp;
}

// Quaternion Inner Product
double Quaternion::operator^ (const Quaternion &r_operand) const  
{
	double temp;

	if((s == r_operand.s) && (x == r_operand.x) 
		&& (y == r_operand.y) && (z == r_operand.z))
	{
		return 1.0;
	}
	if((s == -r_operand.s) && (x == -r_operand.x) 
		&& (y == -r_operand.y) && (z == -r_operand.z))
	{
		return -1.0;
	}

	double norm1 = sqrt(pow(s, 2.0) + pow(x, 2.0) + pow(y, 2.0) + pow(z, 2.0)); 
	if ( 1.0 <= norm1 ) norm1 = 1.0;
	double norm2 = sqrt(pow(r_operand.s, 2.0) + pow(r_operand.x, 2.0) + pow(r_operand.y, 2.0) + pow(r_operand.z, 2.0)); 
	if ( 1.0 <= norm2 ) norm2 = 1.0;
	
	temp = s*r_operand.s + x*r_operand.x + y*r_operand.y + z*r_operand.z;
	temp /= norm1*norm2;
	if ( temp <= -1.0 ) return (-1.0);
	if ( 1.0 <= temp  ) return ( 1.0);

	return temp;




}

// Quaternion Equal
const Quaternion& Quaternion::operator= (const Quaternion &r_operand)  
{ 
		
	s = r_operand.s;
	x = r_operand.x;
	y = r_operand.y;
	z = r_operand.z;

	return(*this);
}


//Spherical Linear interpolation
void Quaternion::Slerp(Quaternion* m_pDestQuat, double u, Quaternion* m_pResQuat)
{

	Quaternion DestQuat;
    double        omega, cos_ohm, sin_ohm, scale0, scale1;

	//calc cosine = innner product between q1, q2
	//cos_ohm = x * m_pDestQuat->x + y * m_pDestQuat->y + z * m_pDestQuat->z
	//		       + s * m_pDestQuat->w;
	cos_ohm = (*this)^(*m_pDestQuat);

    // adjust signs (if necessary)
	// find the shortest arclength
    if ( cos_ohm <0.0 )
	{ 
		cos_ohm = -cos_ohm; 
		DestQuat.s = - m_pDestQuat->s;
		DestQuat.x = - m_pDestQuat->x;
		DestQuat.y = - m_pDestQuat->y;
		DestQuat.z = - m_pDestQuat->z;
    } 
	else  
	{
		//DestQuat.s = m_pDestQuat->s;
		//DestQuat.x = m_pDestQuat->x;
		//DestQuat.y = m_pDestQuat->y;
		//DestQuat.z = m_pDestQuat->z;
		DestQuat = (*m_pDestQuat);
	}

    // calculate coefficients
    if ( (1.0 - cos_ohm) > DELTA )
	{
		// standard case (slerp)
        omega = acos(cos_ohm);
        sin_ohm = sin(omega);
        scale0 = sin((1.0 - u) * omega) / sin_ohm;
        scale1 = sin(u * omega) / sin_ohm;
    }
	else 
	{        
		// "from" and "to" quaternions are very close 
	    //  ... so we can do a linear interpolation
        scale0 = 1.0 - u;
        scale1 = u;
    }

	// calculate final values
	m_pResQuat->s = scale0 * s + scale1 * DestQuat.s;
	m_pResQuat->x = scale0 * x + scale1 * DestQuat.x;
	m_pResQuat->y = scale0 * y + scale1 * DestQuat.y;
	m_pResQuat->z = scale0 * z + scale1 * DestQuat.z;

	m_pResQuat->Normalize();
}





//Convert AngularDisplacement to UnitQuaternion 
void Quaternion::RotationToQuaternion(SFRotation* m_pSrcRot)
{
	double norm;
	double angle, scale;

	angle = RotationAngleTruncation(m_pSrcRot->theta);
	norm = sqrt(pow(m_pSrcRot->x, 2.0) + pow(m_pSrcRot->y, 2.0) + pow(m_pSrcRot->z, 2.0));	

	if(norm < DELTA) 
	{
		s = 1.0;
		x = 0.0;
		y = 0.0;
		z = 0.0;
	}
	else 
	{
		scale = sin(angle/2.0)/norm;
		s = cos(angle/2.0);
		x = m_pSrcRot->x * scale;
		y = m_pSrcRot->y * scale;
		z = m_pSrcRot->z * scale;
		this->Normalize();
	}
}

// keep the value of theta from -PI ~ +PI
double Quaternion::RotationAngleTruncation(double theta)
{
	double adjusted_theta;

	if(theta < -M_PI)
	{
		while(theta < -M_PI)
		{
			theta += 2.0 * M_PI;
		}
		adjusted_theta = theta;

		return adjusted_theta;
	}
	else if(theta >= M_PI)
	{
		while(theta >= M_PI)
		{
			theta -= 2.0 * M_PI;
		}
		adjusted_theta = theta;

		return adjusted_theta;
	}
	
	return theta;
}


void Quaternion::Conjugate(void)
{
	s = s;
	x = -x;
	y = -y;
	z = -z;
}

void Quaternion::Normalize()
{
	double norm;

	norm = pow(s, 2.0) + pow(x, 2.0) + pow(y, 2.0) + pow(z, 2.0);
	norm = sqrt(norm);

	if(norm == 0.0)
	{
		s = 1.0;
		x = 0.0;
		y = 0.0;
		z = 0.0;
		return;
	}

	s /= norm;
	x /= norm;
	y /= norm;
	z /= norm;
}