/***********************************************************************
This software module was originally developed by James D. K. Kim, 
Sang Oak Woo and Shin Jun Lee(Samsung Advance Institute of Technology)
in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)
standard. This software module is an implementation of a part of one or
more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4
Systems (ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 
Systems (ISO/IEC 14496-1) free license to this software module or 
modifications thereof for use in hardware or software products claiming
conformance to the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to 
use this software module in hardware or software products are advised that
its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Samsung Advance Institute of Technology
retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using
the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
This copyright notice must be included in all copies or derivative works.
Copyright  2001
************************************************************************

For more information or to receive an up to date version of this module,
you may contact James D. K. Kim at jacobkim@sait.samsung.co.kr
----------------------------------------------------------------------*/

// Quaternion.h: interface for the Quaternion class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_QUATERNION_H__4FBBD18F_0CE8_4134_AF89_FCCFF71CB5CF__INCLUDED_)
#define AFX_QUATERNION_H__4FBBD18F_0CE8_4134_AF89_FCCFF71CB5CF__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

class SFRotation;

class Quaternion  
{
public:
	Quaternion(); // default ctor
	Quaternion(double in_s, double in_x, double in_y, double in_z); // customize initialization ctor
	Quaternion(const Quaternion& src);// copy ctor
	virtual ~Quaternion();

//member variables
public:
	double s;
	double x;
	double y;
	double z;

//member function
public:
	//Spherical Linear interpolation
	void Slerp(Quaternion* m_pDestQuat, double u, Quaternion* m_pResQuat);

	//Convert AngularDisplacement to UnitQuaternion 
	void RotationToQuaternion(SFRotation* m_pSrcRot);
	double RotationAngleTruncation(double theta);
	//complex conjugate
	void Conjugate(void);
	// normalize
	void Normalize(void);


//member operator
public:
	//Quaternion Multiplication Operator
	Quaternion operator* (const Quaternion &r_operand) const;   
	//Quaternion Addition Operator
	Quaternion operator+ (const Quaternion &r_operand) const;   
	//Quaternion Inner Product Operator
	double operator^ (const Quaternion &r_operand) const;   
	//Quaternion Equal Operator
	const Quaternion& operator= (const Quaternion &r_operand);   

};

#endif // !defined(AFX_QUATERNION_H__4FBBD18F_0CE8_4134_AF89_FCCFF71CB5CF__INCLUDED_)
