/***********************************************************************
This software module was originally developed by IBM Corporation & Samsung
Electronics Co. LTD., and modified by James D. K. Kim, Sang Oak Woo 
and Shin Jun Lee(Samsung Advance Institute of Technology)
in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)
standard. This software module is an implementation of a part of one or
more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4
Systems (ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 
Systems (ISO/IEC 14496-1) free license to this software module or 
modifications thereof for use in hardware or software products claiming
conformance to the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to 
use this software module in hardware or software products are advised that
its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Samsung Advance Institute of Technology
retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using
the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
This copyright notice must be included in all copies or derivative works.
Copyright  2001
************************************************************************

For more information or to receive an up to date version of this module,
you may contact James D. K. Kim at jacobkim@sait.samsung.co.kr
----------------------------------------------------------------------*/

#include "defs.h"
//#include "stdafx.h"
#include "entropy.h"

//////////////////////////////////////////////////////////////////////////////////
///   Basic Table for Arithmetic Coding  /////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////
int SAIT_CAC::_shift = 0;

struct AC_QEntry SAIT_CAC::model[] = {
  { 0x5601,  1,  1, 1 },
  { 0x3401,  2,  6, 0 },
  { 0x1801,  3,  9, 0 },
  { 0x0ac1,  4, 12, 0 },
  { 0x0521,  5, 29, 0 },
  { 0x0221, 38, 33, 0 },

  { 0x5601,  7,  6, 1 },
  { 0x5401,  8, 14, 0 },
  { 0x4801,  9, 14, 0 },
  { 0x3801, 10, 14, 0 },
  { 0x3001, 11, 17, 0 },
  { 0x2401, 12, 18, 0 },
  { 0x1c01, 13, 20, 0 },
  { 0x1601, 29, 21, 0 },

  { 0x5601, 15, 14, 1 },
  { 0x5401, 16, 14, 0 },
  { 0x5101, 17, 15, 0 },
  { 0x4801, 18, 16, 0 },
  { 0x3801, 19, 17, 0 },
  { 0x3401, 20, 18, 0 },
  { 0x3001, 21, 19, 0 },
  { 0x2801, 22, 19, 0 },
  { 0x2401, 23, 20, 0 },
  { 0x2201, 24, 21, 0 },
  { 0x1c01, 25, 22, 0 },
  { 0x1801, 26, 23, 0 },
  { 0x1601, 27, 24, 0 },
  { 0x1401, 28, 25, 0 },
  { 0x1201, 29, 26, 0 },
  { 0x1101, 30, 27, 0 },
  { 0x0ac1, 31, 28, 0 },
  { 0x09c1, 32, 29, 0 },
  { 0x08a1, 33, 30, 0 },
  { 0x0521, 34, 31, 0 },
  { 0x0441, 35, 32, 0 },
  { 0x02a1, 36, 33, 0 },
  { 0x0221, 37, 34, 0 },
  { 0x0141, 38, 35, 0 },
  { 0x0111, 39, 36, 0 },
  { 0x0085, 40, 37, 0 },
  { 0x0049, 41, 38, 0 },
  { 0x0025, 42, 39, 0 },
  { 0x0015, 43, 40, 0 },
  { 0x0009, 44, 41, 0 },
  { 0x0005, 45, 42, 0 },
  { 0x0001, 45, 43, 0 },
  { 0x5601, 46, 46, 0 }
};

SAIT_CAC::~SAIT_CAC() 
{}

SAIT_CAC::SAIT_CAC() 
{ 
	reset(); 
}

void SAIT_CAC::reset() { 
	index = 0; 
	MPS = 0; 
	Qe = model[index].Qe << _shift; 
}

void SAIT_CAC::update(int symbol) 
{
	if (symbol == MPS)
		index = model[index].nMPS;
	else {
		MPS ^= model[index].sMPS;
		index = model[index].nLPS;
	}
	Qe = model[index].Qe << _shift;
}

//////////////////////////////////////////////////////////////////////////////////
///   Basic Modules for QF Arithmetic Coding  ////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////
CAC_QF::CAC_QF(COutStream &o) 
{
	//_istrm = (CInStream*)0;  //NULL; 
	_ostrm = &o; 
	_ostrm_bits = 0;
	_istrm_bits = 0;
}  

CAC_QF::CAC_QF(CInStream &i) 
{
	_istrm = &i;
	//_ostrm = (COutStream*)0;  //NULL;
	_ostrm_bits = 0;
	_istrm_bits = 0;
}

CAC_QF::~CAC_QF()
{}

int CAC_QF::getOBits()
{ 
	return _ostrm_bits; 
}

int CAC_QF::getIBits()
{ 
	return _istrm_bits-32; 
}

void CAC_QF::encode(int bit)
{ 
	bits = 0; 
	hw_encode(bit); 
}

void CAC_QF::encode(int bit, SAIT_CAC &qfPM)
{ 
	bits = 0; 
	hw_encode(bit, qfPM); 
}

int CAC_QF::decode()  
{ 
	return sw_decode(); 
}

int CAC_QF::decode(SAIT_CAC &qfPM)
{ 
	return sw_decode(qfPM); 
}

void CAC_QF::start_encode(void)
{ 
	hw_start_encode(); 
}

void CAC_QF::finish_encode(void)
{ 
	hw_finish_encode(); 
}

void CAC_QF::start_decode(void)
{ 
	sw_start_decode(); 
}

void CAC_QF::finish_decode(void)
{ 
	sw_finish_decode(); 
}

///////////////////////////////////////////////////////////////////////////////////////
void CAC_QF::hw_start_encode() 
{
	C = 0x0;
	C_high = 0x0;
	A = 0x8000;
	count = 12;      
	first = 1;
}

void CAC_QF::hw_renorm_encode() 
{
	do {
		C <<= 1;
		A <<= 1;
		count--;  
		if (count == 0) {
			if (C_high == 0xff) {          
l1:
				put_byte(0xff);  
				C_high = C >> 20;          
				C &= 0x000fffff;           
				count = 7;
			} else {
				if (C < 0x08000000) {      
l2:
					put_byte(C_high);  
					C_high = C >> 19;      
					C &= 0x0007ffff;       
					count = 8;
				} else {
					C_high++;              
					if (C_high == 0xff) {
						C &= 0x07ffffff;
						goto l1;
					} else
						goto l2;
				}
			}
		}
		bits++;
	} while (!(A & 0x8000));
}

void CAC_QF::hw_encode(int symbol, SAIT_CAC &state) 
{
	int cond_exchg = A < 2*state.Qe;          

	if (symbol == (state.MPS ^ cond_exchg)) {  
		C += state.Qe;
		A -= state.Qe;
	} else                                     
		A = state.Qe;              
	if (!(A & 0x8000)) {                       
		state.update(symbol);
		hw_renorm_encode();
	}
}

void CAC_QF::hw_encode(int symbol) 
{
	int cond_exchg = A < 0xac02;
	if (symbol == cond_exchg) {
		C += 0x5601;
		A -= 0x5601;
	} else
		A = 0x5601;
	
	if (!(A & 0x8000)) 
		hw_renorm_encode();
}

void CAC_QF::hw_finish_encode() 
{
	C += 0x4000;
	C &= 0xffffc000;
	
	A >>= 15;                
	hw_renorm_encode();      
	
	A >>= 3;                 
	hw_renorm_encode();      

	put_byte(C_high);
	put_byte(0x00);
}

void CAC_QF::hw_start_decode() 
{
	int b = get_byte() ^ 0xff;
	C = b << 23;
	b = get_byte() ^ 0xff;
	C += b << 15;
	if ((b + 1) >> 8) {
		C += (get_byte() ^ 0xff) << 8;
		count = 8;
	} else
		count = 1;
	A = 0x80000000;
}

void CAC_QF::hw_renorm_decode() 
{
	do {
		C <<= 1;
		if (--count == 0) {
			int b = get_byte() ^ 0xff;
			C += b << 8;
			count = 8 - ((b + 1) >> 8);
		}
		A <<= 1;
	} while (!(A & 0x80000000));
}

int CAC_QF::hw_decode(SAIT_CAC &state) 
{
	int cond_exchg = A < (state.Qe << 17);
	int symbol = state.MPS ^ cond_exchg;
	if (C >= (state.Qe << 16)) {
		C -= state.Qe << 16;
		A -= state.Qe << 16;
	} else {
		symbol ^= 1;
		A = state.Qe << 16;
	}
	
	if (!(A & 0x80000000)) {
		state.update(symbol);
		hw_renorm_decode();
	}
	return symbol;
}

void CAC_QF::hw_finish_decode() 
{ }

void CAC_QF::sw_start_decode() 
{
	A = 0x00010000;
	C = 0x00010000;
	count = 0;
	sw_renorm_decode();
}

int CAC_QF::sw_renorm_decode() 
{
	do {
		if (count == 0) {
			int b = get_byte();
			if (b < 0)
				return -1;

			C -= 0xff << 8;                
			C += b << 8;                   
			count = 8 - ((0x100-b) >> 8);  
		}
		count--;
		C = C << 1;
		A = A << 1;
	} while (!(A & 0x80000000));
	return 0;
}

int CAC_QF::sw_decode(SAIT_CAC &state) 
{
	int symbol = state.MPS;
	A -= state.Qe;
	if (C <= A) {                   
		if (A & 0x80000000)
			return symbol;
		symbol ^= A < state.Qe;     
	} else {                        
		symbol ^= A >= state.Qe;    
		C = C - A;
		A = state.Qe;
	}
	state.update(symbol);
	symbol = (sw_renorm_decode()) ? -1 : symbol;
	return symbol;
}

int CAC_QF::sw_decode() 
{
	int symbol = 0;
	A -= 0x56010000;
	if (C <= A) {                    
		if (A & 0x80000000)
			return symbol;
		symbol = A < 0x56010000;
	} else {
		symbol = A >= 0x56010000;
		C = C - A;
		A = 0x56010000;
	}
	symbol = (sw_renorm_decode()) ? -1 : symbol;
	return symbol;
}

void CAC_QF::sw_finish_decode() 
{}

void CAC_QF::put_byte(int b) 
{
	if (first)
		first = 0;
	else {
		_ostrm->putUChar(b ^ 0xff);
	}
}

int CAC_QF::get_byte() 
{
	unsigned char c;
	_istrm->getUChar(c);
	return c;
}

