/***********************************************************************
This software module was originally developed by James D. K. Kim, 
Sang Oak Woo and Shin Jun Lee(Samsung Advance Institute of Technology)
in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)
standard. This software module is an implementation of a part of one or
more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4
Systems (ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 
Systems (ISO/IEC 14496-1) free license to this software module or 
modifications thereof for use in hardware or software products claiming
conformance to the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to 
use this software module in hardware or software products are advised that
its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Samsung Advance Institute of Technology
retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using
the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
This copyright notice must be included in all copies or derivative works.
Copyright ?2001
************************************************************************

For more information or to receive an up to date version of this module,
you may call Sang Oak Woo at sowoo@sait.samsung.co.kr
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Interpolator Compression
FILE:       functionsForDecoding.cpp
AUTHOR:     Sang Oak Woo (SAMSUNG AIT)
OVERVIEW:   Implementation of Interpolator Compression
-------------------------------------------------------------------------*/

#include "defs.h"
//#include "stdafx.h"

#include "functionsForDecoding.h"
#include <math.h>
#include <stdlib.h>

void Inverse_DPCM_CircularQuantize(
	int* aValues, int nSize, int nMax, int nStart
)
{
	int nValue, nPrevValue, nCircularValue;

	int* aFinalValues = new int[nSize];

	if(nStart == 0)
	{
		nPrevValue = 0;
	}
	else
	{
		aFinalValues[0] = aValues[0];
		nPrevValue = aValues[0];

		nStart = 1;
	}

	for(int i=nStart; i<nSize; i++)
	{
		nValue = aValues[i];

		if(nValue >= 0)
			nCircularValue = nValue - (nMax + 1);
		else
			nCircularValue = nValue + (nMax + 1);

		nValue = aValues[i] + nPrevValue;

		//  ۿ  쿡 ...
		//  .
		if(nValue < 0)
			nValue = nPrevValue + nCircularValue;
		if(nValue > nMax)
			nValue = nPrevValue + nCircularValue;

		//
		nPrevValue = nValue;
		aFinalValues[i] = nValue;
	}

	for(i=0; i<nSize; i++)
		aValues[i] = aFinalValues[i];

	delete[] aFinalValues;
}



void Inverse_Second_DPCM_CircularQuantize(
	int* aValues, int nSize, int nMax, int nStart
)
{
	int nValue, nPrevValue, nCircularValue;
	int nDiffValue, nPredValue;

	int* aFinalValues = new int[nSize];

	if(nStart == 0)
	{
		nPrevValue = 0;
		nDiffValue = 0;
	}
	else
	{
		aFinalValues[0] = aValues[0];

		nPrevValue = aValues[0];
		nDiffValue = aValues[0];

		nStart = 1;
	}

	for(int i=nStart; i<nSize; i++)
	{
		nPredValue = nDiffValue + nPrevValue;

		if(nPredValue > nMax)
		{
			nValue = nMax - aValues[i];

			//  ũ?
			if(nValue > nMax)
				nValue = nValue - (nMax + 1);
		}
		else if(nPredValue < 0)
			nValue = aValues[i];
		else
			nValue = aValues[i] + nPredValue;


		if(nValue >= 0)
			nCircularValue = nValue - (nMax + 1);
		else
			nCircularValue = nValue + (nMax + 1);


		//  ۿ  쿡 ...
		//  .
		if(nValue < 0)
			nValue = nCircularValue;
		if(nValue > nMax)
			nValue = nCircularValue;

		//
		nDiffValue = nValue - nPrevValue;
		nPrevValue = nValue;

		aFinalValues[i] = nValue;
	}


	for(i=0; i<nSize; i++)
		aValues[i] = aFinalValues[i];

	delete [] aFinalValues;
}


void Inverse_Quantize(
	int* aQuant, double* aOriginal, 
	int nNum, double dMin, double dMax, int nQBit, int nDigit
)
{
	double dValue;
	int nMaximum = (int) pow(2, nQBit) - 1;

	for(int i=0; i<nNum; i++)
	{
		dValue = (double) aQuant[i] / (double) nMaximum;
		dValue = dValue*(dMax - dMin) + dMin;

		aOriginal[i] = floor_keyValue(dValue, nDigit);
	}
}


double floor_keyValue(double value, int kv_digit)
{
	//////////////////////////////////////////////////
	//
	//			Original Value 7ڸ !!!
	//			꿡 ؼ   
	//
	//////////////////////////////////////////////////

	char buffer[50];
	_gcvt(value, kv_digit, buffer);
	value = atof(buffer);

	return value;
}


void InterpolatorSythesizer(
	double *kv_org, double *kv_ext, JWArray<unsigned int>& flag,
	int n_org, int n_q_key, JWArray<double>& key_final
)
{
	int i,j;
	double st, en, stv, env;
	double *kee;

	kee= new double[n_q_key];
	st = stv = 0.0;

	en=key_final[n_q_key-1]; env=kv_org[n_org-1];
	j=n_org-1;
	for(i=n_q_key-1;i>=0;i--)
	{
		if(flag[i]==1){
			env=kv_org[j];
			en=key_final[i];
			j--;
		}
		kv_ext[i]=env;
		kee[i]=en;
	}

   
    for(i=0;i<n_q_key;i++)
	{
		if (flag[i] == 1) // it is NOT to be interpolated.
		{
			st=kee[i];
			stv=kv_ext[i];
		}
		else // to be interpolated.
		{
			en=kee[i];
			env=kv_ext[i];

			if((en - st) == 0.0)
				kv_ext[i] = stv;
			else
				kv_ext[i]=(env - stv)*(key_final[i]-st)/(en-st) +stv;
		}
	}

	delete [] kee;
}