/***********************************************************************
This software module was originally developed by Sang Oak Woo
(SAMSUNG AIT) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. SAMSUNG AIT retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright ?2002.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Sang Oak Woo at sowoo@sait.samsung.co.kr.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Decoder
FILE:       JWAFXStreamDecoder.cpp
AUTHOR:     Sang Oak Woo (SAMSUNG AIT)
OVERVIEW:   Decoding the buffer of BitWrapper
-------------------------------------------------------------------------*/

#include "defs.h"
#include "nodes.h"
#include "bifsdec.h"
#include "jwafxstreamdecoder.h"
#include "3DMC/Expand/source/MPG_IfsQ_E.h"
//ory DMLAB 2007 03
#include "3DMCExtension/Inc/MPG_IfsQ_E.h"
//~ory DMLAB 

JWAFXObjectStreamDecoder::JWAFXObjectStreamDecoder():
	m_pBuffer(0), m_nLength(-1)
{}

#ifdef TRACE_ENCODING
void JWAFXObjectStreamDecoder::WriteIntoTracetxt(BIFSDecoder *pCoder)
{
	pCoder->TraceNewLine();
	pCoder->TraceNewLine();
	pCoder->Tracef("# If this BitWrapper node is decoded, you can get the following values");
}
#endif


///////////////////////////////////////////////////////////////////////////////////////
//	PositionInterpolator
///////////////////////////////////////////////////////////////////////////////////////


void JWAFXPositionInterpolatorDecoder::Decode()
{
	// fetch the string to buffer of decoder &	decode the buffer
	m_PositionInterpolator.decodeWithBuffer(m_pBuffer, m_nLength);

	// test the value of key & keyValue pairs
	m_PositionInterpolator.write("POSI_K_KV.txt");
}


#if defined SAIT_CONFORMANCE_2003

void JWAFXPositionInterpolatorDecoder::Decode(
	LPBYTE pInput, int nInputLength, LPBYTE& pOutput, int &rnOutputLength)
{
	m_PositionInterpolator.decodeWithBuffer((char*) pInput, nInputLength);
	//m_PositionInterpolator.write("AFXObject-Position.txt");

	// initial value of 
		// pOutput: the address of memory allocated by DecoderImp
		// rnOutputLength: the size of pOutput
	// final value of
		// rnOutputLength: the real size (actual data size) of pOutput

	int nNumberOfKeys = m_PositionInterpolator.key.GetSize();

	if(pOutput == NULL)
	{	// in case of BitWrapper.buffer case, pOutput is NULL
		// therefore, pOutput need to be allocated
		pOutput = (LPBYTE) malloc(nNumberOfKeys*sizeof(float)*4 + sizeof(int));
	}
	else
	{
		// if the size of pOutput is less then the size needed, return false
		if((int)(nNumberOfKeys*sizeof(float)*4 + sizeof(int))> rnOutputLength)
			return;
	}

	// the Format of pOutput
		// # of frames
		// all of the keys
		// all of the keyValues
	int *pOutput_NumberOfKey = (int*) pOutput;
	float *pOutput_Keys = (float*) (pOutput + sizeof(int));
	float *pOutput_KeyValues = (float*) (pOutput + sizeof(float)*nNumberOfKeys + sizeof(int));

	// Fill pOutput
	// # of frames
	pOutput_NumberOfKey[0] = nNumberOfKeys;

	for(int i=0, j=0; i<nNumberOfKeys; i++)
	{
		// all of the keys
		pOutput_Keys[i] = m_PositionInterpolator.key[i];

		// all of the keyValues
		pOutput_KeyValues[j++] = m_PositionInterpolator.keyValue[i].x;
		pOutput_KeyValues[j++] = m_PositionInterpolator.keyValue[i].y;
		pOutput_KeyValues[j++] = m_PositionInterpolator.keyValue[i].z;
	}

	rnOutputLength = nNumberOfKeys*sizeof(float)*4 + sizeof(int);
}


void JWAFXPositionInterpolatorDecoder::Write(char* fileName, int defID)
{
	m_PositionInterpolator.write(fileName, defID);
}

void JWAFXPositionInterpolatorDecoder::SetValues(unsigned char* pData)
{
	m_PositionInterpolator.setValues(pData);
}

#endif SAIT_CONFORMANCE_2003


#ifdef TRACE_ENCODING
void JWAFXPositionInterpolatorDecoder::WriteIntoTracetxt(BIFSDecoder *pCoder)
{
	JWAFXObjectStreamDecoder::WriteIntoTracetxt(pCoder);

	pCoder->TraceNewLine();

	// write the name of node
	pCoder->Tracef("# PositionInterpolator {");
	pCoder->TraceNewLine();
	pCoder->TraceIndent(1);

	// write the information on key
	pCoder->Tracef("# key [");
	pCoder->TraceNewLine();
	pCoder->TraceIndent(1);

	for(int i=0; i<m_PositionInterpolator.key.GetSize(); i++)
	{
		pCoder->Tracef("# %g", m_PositionInterpolator.key[i]);
		pCoder->TraceNewLine();
	}

	pCoder->TraceIndent(-1);
	pCoder->Tracef("# ]");
	pCoder->TraceNewLine();

	// write the information on keyValue
	pCoder->Tracef("# keyValue [");
	pCoder->TraceNewLine();
	pCoder->TraceIndent(1);

	for(i=0; i<m_PositionInterpolator.keyValue.GetSize(); i++)
	{
		pCoder->Tracef("# %g %g %g",
			m_PositionInterpolator.keyValue[i].x,
			m_PositionInterpolator.keyValue[i].y,
			m_PositionInterpolator.keyValue[i].z
			);
		pCoder->TraceNewLine();
	}

	pCoder->TraceIndent(-1);
	pCoder->Tracef("# ]");
	pCoder->TraceNewLine();

	// write the ending brace
	pCoder->TraceIndent(-1);
	pCoder->Tracef("# }");
	pCoder->TraceNewLine();
}
#endif

///////////////////////////////////////////////////////////////////////////////////////
//	OrientationInterpolator
///////////////////////////////////////////////////////////////////////////////////////

void JWAFXOrientationInterpolatorDecoder::Decode()
{
	// fetch the string to buffer of decoder &	decode the buffer
	m_OrientationInterpolator.decodeWithBuffer(m_pBuffer, m_nLength);

	// test the value of key & keyValue pairs
	m_OrientationInterpolator.write("ORII_K_KV.txt");
}

#if defined SAIT_CONFORMANCE_2003

void JWAFXOrientationInterpolatorDecoder::Decode(
	LPBYTE pInput, int nInputLength, LPBYTE& pOutput, int &rnOutputLength)
{
	m_OrientationInterpolator.decodeWithBuffer((char*) pInput, nInputLength);
	//m_PositionInterpolator.write("AFXObject-Position.txt");

	// initial value of 
		// pOutput: the address of memory allocated by DecoderImp
		// rnOutputLength: the size of pOutput
	// final value of
		// rnOutputLength: the real size (actual data size) of pOutput

	int nNumberOfKeys = m_OrientationInterpolator.dKey.GetSize();

	if(pOutput == NULL)
	{	// in case of BitWrapper.buffer case, pOutput is NULL
		// therefore, pOutput need to be allocated
		pOutput = (LPBYTE) malloc(nNumberOfKeys*sizeof(float)*5 + sizeof(int));
	}
	else
	{
		// if the size of pOutput is less then the size needed, return false
		if((int)(nNumberOfKeys*sizeof(float)*5 + sizeof(int))> rnOutputLength)
			return;
	}

	// the Format of pOutput
		// # of frames
		// all of the keys
		// all of the keyValues
	int *pOutput_NumberOfKey = (int*) pOutput;
	float *pOutput_Keys = (float*) (pOutput + sizeof(int));
	float *pOutput_KeyValues = (float*) (pOutput + sizeof(float)*nNumberOfKeys + sizeof(int));

	// Fill pOutput
	// # of frames
	pOutput_NumberOfKey[0] = nNumberOfKeys;

	for(int i=0, j=0; i<nNumberOfKeys; i++)
	{
		// all of the keys
		pOutput_Keys[i] = m_OrientationInterpolator.dKey[i];

		// all of the keyValues
		pOutput_KeyValues[j++] = m_OrientationInterpolator.dKeyValue[i].x;
		pOutput_KeyValues[j++] = m_OrientationInterpolator.dKeyValue[i].y;
		pOutput_KeyValues[j++] = m_OrientationInterpolator.dKeyValue[i].z;
		pOutput_KeyValues[j++] = m_OrientationInterpolator.dKeyValue[i].theta;
	}

	rnOutputLength = nNumberOfKeys*sizeof(float)*5 + sizeof(int);
}


void JWAFXOrientationInterpolatorDecoder::Write(char* fileName, int defID)
{
	m_OrientationInterpolator.write(fileName, defID);
}

void JWAFXOrientationInterpolatorDecoder::SetValues(unsigned char* pData)
{
	m_OrientationInterpolator.setValues(pData);
}

#endif SAIT_CONFORMANCE_2003

#ifdef TRACE_ENCODING
void JWAFXOrientationInterpolatorDecoder::WriteIntoTracetxt(BIFSDecoder *pCoder)
{
	JWAFXObjectStreamDecoder::WriteIntoTracetxt(pCoder);

	pCoder->TraceNewLine();

	// write the name of node
	pCoder->Tracef("# OrientationInterpolator {");
	pCoder->TraceNewLine();
	pCoder->TraceIndent(1);

	// write the information on key
	pCoder->Tracef("# key [");
	pCoder->TraceNewLine();
	pCoder->TraceIndent(1);

	for(int i=0; i<m_OrientationInterpolator.dKey.GetSize(); i++)
	{
		pCoder->Tracef("# %g", m_OrientationInterpolator.dKey[i]);
		pCoder->TraceNewLine();
	}

	pCoder->TraceIndent(-1);
	pCoder->Tracef("# ]");
	pCoder->TraceNewLine();

	// write the information on keyValue
	pCoder->Tracef("# keyValue [");
	pCoder->TraceNewLine();
	pCoder->TraceIndent(1);

	for(i=0; i<m_OrientationInterpolator.dKeyValue.GetSize(); i++)
	{
		pCoder->Tracef("# %g %g %g %g",
			m_OrientationInterpolator.dKeyValue[i].x,
			m_OrientationInterpolator.dKeyValue[i].y,
			m_OrientationInterpolator.dKeyValue[i].z,
			m_OrientationInterpolator.dKeyValue[i].theta
			);
		pCoder->TraceNewLine();
	}

	pCoder->TraceIndent(-1);
	pCoder->Tracef("# ]");
	pCoder->TraceNewLine();

	// write the ending brace
	pCoder->TraceIndent(-1);
	pCoder->Tracef("# }");
	pCoder->TraceNewLine();
}
#endif


///////////////////////////////////////////////////////////////////////////////////////
//	CoordinateInterpolator
///////////////////////////////////////////////////////////////////////////////////////
void JWAFXCoordinateInterpolatorDecoder::Decode()
{
	// fetch the string to buffer of decoder &	decode the buffer
	m_CoordinateInterpolator.decodeWithBuffer(m_pBuffer, m_nLength);

	// test the value of key & keyValue pairs
	m_CoordinateInterpolator.write("COOI_K_KV.txt");
}

#if defined SAIT_CONFORMANCE_2003

void JWAFXCoordinateInterpolatorDecoder::Decode(
	LPBYTE pInput, int nInputLength, LPBYTE& pOutput, int &rnOutputLength)
{
	// variables: indices
	int i = 0;
	int j = 0;

	m_CoordinateInterpolator.decodeWithBuffer((char*) pInput, nInputLength);
	//m_CoordinateInterpolator.write("AFXObject-Coordinate.txt");

	// initial value of 
		// pOutput: the address of memory allocated by DecoderImp
		// rnOutputLength: the size of pOutput
	// final value of
		// rnOutputLength: the real size (actual data size) of pOutput

	int nNumberOfKeys = m_CoordinateInterpolator.kHeader.nNumberOfKey;
	int nNumberOfCoords = m_CoordinateInterpolator.coordIKVHeader.nNumberOfCoord;

	if(pOutput == NULL)
	{	// in case of BitWrapper.buffer case, pOutput is NULL
		// therefore, pOutput need to be allocated
		pOutput = (LPBYTE) malloc(2*sizeof(int) + nNumberOfKeys*sizeof(float) + 3*nNumberOfKeys*nNumberOfCoords*sizeof(float));
	}
	else
	{
		// if the size of pOutput is less then the size needed, return false
		if((int)(2*sizeof(int) + nNumberOfKeys*sizeof(float) + 3*nNumberOfKeys*nNumberOfCoords*sizeof(float))> rnOutputLength)
			return;
	}

	// the Format of pOutput
		// # of frames
		// # of vertices
		// all of the keys
		// all of the keyValues
	int *pOutput_NumberOfKeys = (int*) pOutput;
	int *pOutput_NumberOfCoords = (int*) (pOutput + sizeof(int));
	float *pOutput_Keys = (float*) (pOutput + 2*sizeof(int));
	float *pOutput_KeyValues = (float*) (pOutput + 2*sizeof(int) + nNumberOfKeys*sizeof(float));

	// Fill pOutput
	// # of frames
	// # of vertices
	*pOutput_NumberOfKeys = nNumberOfKeys;
	*pOutput_NumberOfCoords = nNumberOfCoords;

	// Fill key data
	for(i = 0; i < nNumberOfKeys; i++)
		pOutput_Keys[i] = (float)m_CoordinateInterpolator.k.dKey[i];

	// Fill key value data
	for(i = 0; i < nNumberOfKeys; i++)
	{
		for(j = 0; j < nNumberOfCoords; j++)
		{
			pOutput_KeyValues[i*3*nNumberOfCoords+j*3] = (float)m_CoordinateInterpolator.coordIKeyValue.dKeyValue[i*3*nNumberOfCoords+j*3];
			pOutput_KeyValues[i*3*nNumberOfCoords+j*3+1] = (float)m_CoordinateInterpolator.coordIKeyValue.dKeyValue[i*3*nNumberOfCoords+j*3+1];
			pOutput_KeyValues[i*3*nNumberOfCoords+j*3+2] = (float)m_CoordinateInterpolator.coordIKeyValue.dKeyValue[i*3*nNumberOfCoords+j*3+2];
		}
	}

	rnOutputLength = 2*sizeof(int) + nNumberOfKeys*sizeof(float) + 3*nNumberOfKeys*nNumberOfCoords*sizeof(float);
}


void JWAFXCoordinateInterpolatorDecoder::Write(char* fileName, int defID)
{
	m_CoordinateInterpolator.write(fileName, defID);
}

void JWAFXCoordinateInterpolatorDecoder::SetValues(unsigned char* pData)
{
	m_CoordinateInterpolator.setValues(pData);
}

#endif SAIT_CONFORMANCE_2003

#ifdef TRACE_ENCODING
void JWAFXCoordinateInterpolatorDecoder::WriteIntoTracetxt(BIFSDecoder *pCoder)
{
	JWAFXObjectStreamDecoder::WriteIntoTracetxt(pCoder);

	pCoder->TraceNewLine();

	// write the name of node
	pCoder->Tracef("# OrientationInterpolator {");
	pCoder->TraceNewLine();
	pCoder->TraceIndent(1);

	// write the information on key
	pCoder->Tracef("# key [");
	pCoder->TraceNewLine();
	pCoder->TraceIndent(1);

	for(int i=0; i<m_CoordinateInterpolator.kHeader.nNumberOfKey; i++)
	{
		pCoder->Tracef("# %g", m_CoordinateInterpolator.k.dKey[i]);
		pCoder->TraceNewLine();
	}

	pCoder->TraceIndent(-1);
	pCoder->Tracef("# ]");
	pCoder->TraceNewLine();

	// write the information on keyValue
	pCoder->Tracef("# keyValue [");
	pCoder->TraceNewLine();
	pCoder->TraceIndent(1);

	for(i=0; i<m_CoordinateInterpolator.kHeader.nNumberOfKey; i++)
	{
		for(int j=0; j<m_CoordinateInterpolator.coordIKVHeader.nNumberOfCoord; j++)
		{
			pCoder->Tracef("# %g %g %g",
				m_CoordinateInterpolator.coordIKeyValue.dKeyValue[
				i*3*m_CoordinateInterpolator.coordIKVHeader.nNumberOfCoord+j*3],
				m_CoordinateInterpolator.coordIKeyValue.dKeyValue[
				i*3*m_CoordinateInterpolator.coordIKVHeader.nNumberOfCoord+j*3+1],
				m_CoordinateInterpolator.coordIKeyValue.dKeyValue[
				i*3*m_CoordinateInterpolator.coordIKVHeader.nNumberOfCoord+j*3+2]
				);
			pCoder->TraceNewLine();
		}
	}

	pCoder->TraceIndent(-1);
	pCoder->Tracef("# ]");
	pCoder->TraceNewLine();

	// write the ending brace
	pCoder->TraceIndent(-1);
	pCoder->Tracef("# }");
	pCoder->TraceNewLine();
}
#endif


///////////////////////////////////////////////////////////////////////////////////////
//	3DMC
///////////////////////////////////////////////////////////////////////////////////////
void JWAFX3DMCDecoder::Decode()
{
	// fetch the string to buffer of decoder &	decode the buffer
	m_3DMC.DecodeWithBuffer(m_pBuffer, m_nLength);

	// test the value of key & keyValue pairs
	//m_3DMC.write("ORII_K_KV.txt");
}


#if defined SAIT_CONFORMANCE_2003

void JWAFX3DMCDecoder::Decode(
	LPBYTE pInput, int nInputLength, LPBYTE& pOutput, int &rnOutputLength)
{

	m_3DMC.DecodeWithBuffer((char*) pInput, nInputLength);
	//m_3DMC.Decode(fileName);
//	m_3DMC.write("AFXObject-3DMC.txt",-1);

#if defined SAIT_CONFORMANCE_2003

	// 3DMC  ũ⸦ .
	//  ũ⸦ rnOutputLength   rnOutputLength ,
	// rnOutputLength  ũ⸦  !
	// , return FALSE
	
	//  д´.
	bool ccw = m_3DMC.GetCcw(); // Read ccw value
	bool convex = m_3DMC.GetConvex();
	float creaseAngle = m_3DMC.GetCreaseAngle();
	bool solid = m_3DMC.GetSolid(); 
	
	int nCoord = m_3DMC.GetCoord(); 
	int nCoordIndex = m_3DMC.GetCoordIdx();
	
	int nColor = m_3DMC.GetColor();
	bool colorPerVertex = m_3DMC.GetColorPerVertex();
	int nColorIndex = m_3DMC.GetColorIndex();
	
	int nNormal = m_3DMC.GetNormal();
	bool normalPerVertex = m_3DMC.GetNormalPerVertex();
	int nNormalIndex = m_3DMC.GetNormalIndex();
	
	int nTexCoord  = m_3DMC.GetTexCoord();
	int nTexCoordIndex = m_3DMC.GetTexCoordIndex();

	// Field Values
	int    *pCoordIndex;
	float  *pCoord;
	int    *pColorIndex;
	float  *pColor;
	int    *pNormalIndex;
	float  *pNormal;
	int    *pTexCoordIndex;
	float  *pTexCoord;

	int nMemorySizeNeeded = 
		sizeof(bool)*5 +
		sizeof(int)*(8+nCoordIndex+nColorIndex+nNormalIndex+nTexCoordIndex) + 
		sizeof(float)*(1+nCoord+nColor+nNormal+nTexCoord);
	
	if(pOutput == NULL)
	{	// in case of BitWrapper.buffer case, pOutput is NULL
		// therefore, pOutput need to be allocated
		pOutput = (LPBYTE) malloc(nMemorySizeNeeded+10000000);
		rnOutputLength = nMemorySizeNeeded;
	}

	int i = 0;
	int   nLengthOfTotal = 0;

	// Headers: ccw
	bool* pCCW = (bool*) pOutput;      
    nLengthOfTotal += sizeof(bool);    // Increase Total Output size
	pCCW[0] = ccw;                     //   Ѵ.

	// Headers: convex
	bool* pConvex = (bool*)( pOutput + nLengthOfTotal );
	nLengthOfTotal += sizeof(bool);
	pConvex[0] = convex;

	// Headers: creaseAngle
	float* pCreaseAngle = (float *)(pOutput + nLengthOfTotal);
	nLengthOfTotal += sizeof(float);
	pCreaseAngle[0] = creaseAngle;

	// Headers: Solid	
    bool* pSolid = (bool *)( pOutput + nLengthOfTotal);
	nLengthOfTotal += sizeof(bool);
	pSolid[0] = solid;

	// Values: Coord (x,y,z)
    int* pNumCoord = (int*)(pOutput + nLengthOfTotal);
	nLengthOfTotal += sizeof(int);
	pNumCoord[0] = nCoord;                   // Store the number of Coord
	
	pCoord = (float *)( pOutput + nLengthOfTotal); // ŭ ũ ´.

	for(i = 0 ; i < nCoord; i++) {
		pCoord[i] = m_3DMC.pCoord[i]; // Store Coord
    }	
	nLengthOfTotal += sizeof(float)*nCoord;

	// Values: coordIndex
	int* pNumCoordIndex = (int*)(pOutput + nLengthOfTotal);
    pNumCoordIndex[0] = nCoordIndex;  // Store the number of CoordIndex
	nLengthOfTotal += sizeof(int);

	pCoordIndex = (int *)( pOutput + nLengthOfTotal);
	for(i = 0 ; i < nCoordIndex; i++) {
		pCoordIndex[i] = m_3DMC.pCoordIdx[i]; //  Ѵ.
    }
	nLengthOfTotal += sizeof(float)*nCoordIndex;
		

	// Values: Color
	
	int* pNumColor = (int*)(pOutput + nLengthOfTotal);
    pNumColor[0] = nColor;  // Store the number of Color
	nLengthOfTotal += sizeof(int);

	//  Color  Ѵٸ
	if(nColor > 0) 
	{
		// ColorPerVertex:  True or False
        bool* pColorPerVertex = (bool *)( pOutput + nLengthOfTotal);
		nLengthOfTotal += sizeof(bool);
	    pColorPerVertex[0] = colorPerVertex;

		// Color Values
		pColor = (float *)( pOutput + nLengthOfTotal);
		for(i = 0 ; i < nColor; i++) {
			pColor[i] = m_3DMC.pColor[i]; //  Ѵ.
		}
		nLengthOfTotal += sizeof(float)*nColor;

		// Color Index
		pColorIndex = (int *)(pOutput + nLengthOfTotal);
		nLengthOfTotal += sizeof(int);
		pColorIndex[0] = nColorIndex;

		if(nColorIndex > 0)
		{
			int *pColorIndex = (int *)( pOutput + nLengthOfTotal);
			nLengthOfTotal += sizeof(int)*nColorIndex;
			for(i = 0 ; i < nColorIndex; i++)
			{
				pColorIndex[i] = m_3DMC.pColorIndex[i];
			}
		}
    } 	// if
	
	
	int* pNumNormal = (int*)(pOutput + nLengthOfTotal);
    pNumNormal[0] = nNormal;  // Store the number of Color
	nLengthOfTotal += sizeof(int);

	//  Normal  Ѵٸ
	if(nNormal > 0) 
	{
		// NormalPerVertex:  True or False
		bool* pNormalPerVertex = (bool *)( pOutput + nLengthOfTotal);
		pNormalPerVertex[0] = normalPerVertex;
		nLengthOfTotal += sizeof(bool);

		// Normal Values
		pNormal = (float *)( pOutput + nLengthOfTotal);
		for(i = 0 ; i < nNormal; i++) {
			pNormal[i] = m_3DMC.pNormal[i]; //  Ѵ.
		}
		nLengthOfTotal += sizeof(float)*nNormal;

		pNormalIndex = (int *)( pOutput + nLengthOfTotal);
		pNormalIndex[0] = nNormalIndex;
		nLengthOfTotal += sizeof(int);

		// Normal Index
		if(nNormalIndex > 0)
		{
			int *pNormalIndex = (int *)( pOutput + nLengthOfTotal);
			for(i = 0 ; i < nNormalIndex; i++)
			{
				pNormalIndex[i] = m_3DMC.pNormalIndex[i];
			}
			nLengthOfTotal += sizeof(int)*nNormalIndex;
		}
    } 	// if


	int* pNumTexCoord = (int*)(pOutput + nLengthOfTotal);
    pNumTexCoord[0] = nTexCoord;  // Store the number of TexCoord
	nLengthOfTotal += sizeof(int);

	// TexCoord
	if(nTexCoord > 0)
	{
		// Texcoord Values
		pTexCoord = (float *)( pOutput + nLengthOfTotal);
		nLengthOfTotal += sizeof(float)*nTexCoord;

		for(i = 0 ; i < nTexCoord; i++) {
			pTexCoord[i] = m_3DMC.pTexCoord[i]; //  Ѵ.
		}

		pTexCoordIndex = (int *)( pOutput + nLengthOfTotal);
		nLengthOfTotal += sizeof(int);
		pTexCoordIndex[0] = nTexCoordIndex;

		// TexCoord Index
		if(nTexCoordIndex > 0)
		{
			int *pTexCoordIndex = (int *)( pOutput + nLengthOfTotal);
			nLengthOfTotal += sizeof(int)*nTexCoordIndex;

			for(i = 0 ; i < nTexCoordIndex; i++)
			{
				pTexCoordIndex[i] = m_3DMC.pTexCoordIndex[i];
			}
		}
    } 	// if


	rnOutputLength = nLengthOfTotal;

#endif SAIT_CONFORMANCE_2003

}


void JWAFX3DMCDecoder::Write(char* fileName, int defID)
{
	m_3DMC.write(fileName, defID);
}

void JWAFX3DMCDecoder::SetValues(unsigned char* pData)
{
	m_3DMC.setValues(pData);
}
#endif SAIT_CONFORMANCE_2003


#ifdef TRACE_ENCODING
void JWAFX3DMCDecoder::WriteIntoTracetxt(BIFSDecoder *pCoder)
{
	JWAFXObjectStreamDecoder::WriteIntoTracetxt(pCoder);

	MPG_IfsQ_E& a3dmc = * (m_3DMC.GetCIFS());

	pCoder->TraceNewLine();

	// write the name of node
	pCoder->Tracef("# IndexedFaceSet {");
	pCoder->TraceNewLine();
	pCoder->TraceIndent(1);


	if(a3dmc.getCcw() == false)
	{
		//fprintf(fp,"  ccw FALSE\n");
		pCoder->Tracef("# ccw FALSE");
		pCoder->TraceNewLine();
	}

	if(a3dmc.getConvex() == false)
	{
		//fprintf(fp,"  convex FALSE\n");
		pCoder->Tracef("# convex FALSE");
		pCoder->TraceNewLine();
	}

	if(a3dmc.getSolid() == false)
	{
		//fprintf(fp,"  solid FALSE\n");
		pCoder->Tracef("# solid FALSE");
		pCoder->TraceNewLine();
	}

	float creaseAngle = a3dmc.getCreaseAngle();
	if(creaseAngle>0.0)
	{
		//fprintf(fp,"  creaseAngle %8.4f\n",creaseAngle);
		pCoder->Tracef("# creaseAngle %8.4", creaseAngle);
		pCoder->TraceNewLine();
	}

	vector<float>& coord = a3dmc.getCoord();
	vector<int>& coordIndex = a3dmc.getCoordIndex();
	{
		int i;
		//fprintf(fp,"  coord Coordinate {\n");
		pCoder->Tracef("# coord Coordinate {");
		pCoder->TraceNewLine();
		pCoder->TraceIndent(1);

		//fprintf(fp,"   point [\n");
		pCoder->Tracef("# point [");
		pCoder->TraceNewLine();
		pCoder->TraceIndent(1);

		for(i=0; i<coord.size();i+=3)
		{
			//fprintf(fp,"%8.4f ",coord[i]);
			pCoder->Tracef("# %8.4f %8.4f %8.4f", coord[i], coord[i+1], coord[i+2]);
			pCoder->TraceNewLine();
		}

		//fprintf(fp,"    ]\n");
		pCoder->TraceIndent(-1);
		pCoder->Tracef("# ]");
		pCoder->TraceNewLine();

		//fprintf(fp,"  }\n");
		pCoder->TraceIndent(-1);
		pCoder->Tracef("# }");
		pCoder->TraceNewLine();

		//fprintf(fp,"  coordIndex [\n");
		pCoder->Tracef("# coordIndex [");
		pCoder->TraceNewLine();
		pCoder->TraceIndent(1);

		for(i=0;i<coordIndex.size(); i++)
		{
			pCoder->Tracef("# ");

			while(coordIndex[i] != -1)
			{
				if(coordIndex[i] == 2)
					pCoder->Tracef("0 ");
				else
					pCoder->Tracef("%d ", coordIndex[i]);

				i++;
			}

			pCoder->Tracef("%d ", coordIndex[i]);
			pCoder->TraceNewLine();
		}

		//fprintf(fp,"    ]\n");
		pCoder->TraceIndent(-1);
		pCoder->Tracef("# ]");
		pCoder->TraceNewLine();
	}

	bool colorPerVertex = a3dmc.getColorPerVertex();
	vector<float>& color = a3dmc.getColor();
	vector<int>& colorIndex = a3dmc.getColorIndex();

	if(color.size() > 0)
	{
		int i;
		//fprintf(fp,"  colorPerVertex %s\n", (colorPerVertex==true)?"TRUE":"FALSE\n");
		pCoder->Tracef("# colorPerVertex %s", (colorPerVertex == true)? "TRUE":"FALSE");
		pCoder->TraceNewLine();

		//fprintf(fp,"  color Color {\n");
		pCoder->Tracef("# color Color {");
		pCoder->TraceNewLine();
		pCoder->TraceIndent(1);

		//fprintf(fp,"   color [\n");
		pCoder->Tracef("# color [");
		pCoder->TraceNewLine();
		pCoder->TraceIndent(1);

		for(i=0; i<color.size(); i+=3)
		{
			pCoder->Tracef("# ");

			float ci = color[i];
			if(ci<0.0) ci=0.0;
			if(ci>1.0) ci=1.0;
			//fprintf(fp,"%8.4f ",ci);
			pCoder->Tracef("%8.4f ", ci);

			ci = color[i+1];
			if(ci<0.0) ci=0.0;
			if(ci>1.0) ci=1.0;
			//fprintf(fp,"%8.4f ",ci);
			pCoder->Tracef("%8.4f ", ci);

			ci = color[i+2];
			if(ci<0.0) ci=0.0;
			if(ci>1.0) ci=1.0;
			//fprintf(fp,"%8.4f ",ci);
			pCoder->Tracef("%8.4f", ci);

			pCoder->TraceNewLine();

		}

		//fprintf(fp,"    ]\n");
		pCoder->TraceIndent(-1);
		pCoder->Tracef("# ]");
		pCoder->TraceNewLine();

		//fprintf(fp,"  }\n");
		pCoder->TraceIndent(-1);
		pCoder->Tracef("# }");
		pCoder->TraceNewLine();

		if(colorIndex.size() > 0)
		{
			//fprintf(fp,"  colorIndex [\n");
			pCoder->Tracef("# colorIndex [");
			pCoder->TraceNewLine();
			pCoder->TraceNewLine();

			for(i=0; i<colorIndex.size(); i++)
			{
				//fprintf(fp,"%d ",colorIndex[i]);
				//if(colorIndex[i]<0) fprintf(fp,"\n");
				pCoder->Tracef("# ");
				while(colorIndex[i] >= 0)
				{
					pCoder->Tracef("%d ", colorIndex[i]);
					i++;
				}
				pCoder->Tracef("%d", colorIndex[i]);
				pCoder->TraceNewLine();
			}

			//fprintf(fp,"    ]\n");
			pCoder->TraceIndent(-1);
			pCoder->Tracef("# ]");
			pCoder->TraceNewLine();
		}
	}

	bool normalPerVertex = a3dmc.getNormalPerVertex();
	vector<float>& normal = a3dmc.getNormal();
	vector<int>& normalIndex = a3dmc.getNormalIndex();

	if(normal.size() > 0)
	{
		int i;
		//fprintf(fp,"  normalPerVertex %s\n", (normalPerVertex==true)?"TRUE":"FALSE\n");
		pCoder->Tracef("# normalPerVertex %s", (normalPerVertex == true)? "TRUE":"FALSE");
		pCoder->TraceNewLine();

		//fprintf(fp,"  normal Normal {\n");
		pCoder->Tracef("# normal Normal {");
		pCoder->TraceNewLine();
		pCoder->TraceIndent(1);

		//fprintf(fp,"   vector [\n");
		pCoder->Tracef("# vector [");
		pCoder->TraceNewLine();

		for(i=0; i<normal.size(); i+=3)
		{
			//fprintf(fp,"%8.4f ",normal[i]);
			//if(i%3==2) fprintf(fp,"\n");
			pCoder->Tracef("# %8.4f %8.4f %8.4f", normal[i], normal[i+1], normal[i+2]);
			pCoder->TraceNewLine();
		}

		//fprintf(fp,"    ]\n");
		pCoder->TraceIndent(-1);
		pCoder->Tracef("# ]");
		pCoder->TraceNewLine();

		//fprintf(fp,"  }\n");
		pCoder->TraceIndent(-1);
		pCoder->Tracef("# }");
		pCoder->TraceNewLine();

		if(normalIndex.size() > 0)
		{
			//fprintf(fp,"  normalIndex [\n");
			pCoder->Tracef("# normalIndex [");
			pCoder->TraceNewLine();

			for(i=0; i<normalIndex.size(); i++)
			{
				//fprintf(fp,"%d ",normalIndex[i]);
				//if(normalIndex[i]<0) fprintf(fp,"\n");
				pCoder->Tracef("# ");
				while(normalIndex[i] >= 0)
				{
					pCoder->Tracef("%d ", normalIndex[i]);
					i++;
				}
				pCoder->Tracef("%d", normalIndex[i]);
				pCoder->TraceNewLine();
			}

			//fprintf(fp,"    ]\n");
			pCoder->TraceIndent(-1);
			pCoder->Tracef("# ]");
			pCoder->TraceNewLine();
		}
	}

	vector<float>& texCoord = a3dmc.getTexCoord();
	vector<int>& texCoordIndex = a3dmc.getTexCoordIndex();

	if(texCoord.size()>0)
	{
		int i;

		//fprintf(fp,"  texCoord TextureCoordinate {\n");
		pCoder->Tracef("# texCoord TextureCoordinate {");
		pCoder->TraceNewLine();
		pCoder->TraceIndent(1);

		//fprintf(fp,"   point [\n");
		pCoder->Tracef("# point [");
		pCoder->TraceNewLine();
		pCoder->TraceIndent(1);

		for(i=0; i<texCoord.size();i+=2)
		{
			//fprintf(fp,"%8.4f ",texCoord[i]);
			//if(i%2==1) fprintf(fp,"\n");
			pCoder->Tracef("# %8.4f %8.4f", texCoord[i], texCoord[i+1]);
			pCoder->TraceNewLine();
		}

		//fprintf(fp,"    ]\n");
		pCoder->TraceIndent(-1);
		pCoder->Tracef("# ]");
		pCoder->TraceNewLine();

		//fprintf(fp,"  }\n");
		pCoder->TraceIndent(-1);
		pCoder->Tracef("# }");
		pCoder->TraceNewLine();

		if(texCoordIndex.size() > 0)
		{
			//fprintf(fp,"  texCoordIndex [\n");
			pCoder->Tracef("# texCoordIndex [");
			pCoder->TraceNewLine();
			pCoder->TraceIndent(1);

			for(i=0;i<texCoordIndex.size(); i++)
			{
				//fprintf(fp,"%d ",texCoordIndex[i]);
				//if(texCoordIndex[i]<0) fprintf(fp,"\n");
				pCoder->Tracef("# ");
				while(texCoordIndex[i] >= 0)
				{
					pCoder->Tracef("%d ", texCoordIndex[i]);
					i++;
				}
				pCoder->Tracef("%d", texCoordIndex[i]);
				pCoder->TraceNewLine();
			}

			//fprintf(fp,"    ]\n");
			pCoder->TraceIndent(-1);
			pCoder->Tracef("# ]");
			pCoder->TraceNewLine();
		}
	}

	//fprintf(fp," }\n");		// IndexedFaceSet
	pCoder->TraceIndent(-1);
	pCoder->Tracef("# }");
	pCoder->TraceNewLine();
}
#endif

///////////////////////////////////////////////////////////////////////////////////////
//	OctreeImage
///////////////////////////////////////////////////////////////////////////////////////


void YAFXOctreeDecoder::Decode(LPBYTE pInput, int nInputLength, LPBYTE& pOutput, int &rnOutputLength)
{
	if(GetOctree(&octdata) != S_OK)
		err("Can't create octree-object");

	CInStream istrm;
	SAIT_CAC::_shift = 16;
	CAC_QF codec(istrm);
	istrm.opendecWithBuffer((char*) pInput, nInputLength);

	octdata->len_octreeBitstream = istrm.Inbuf.size;
	octdata->setLenOctreeBitstreamBuf(octdata->len_octreeBitstream);

	memcpy(octdata->octreeBitstream,istrm.Inbuf.buf,octdata->len_octreeBitstream);	

	octdata->setLenOctreeBuf(octdata->len_octreeBitstream*10);
	octdata->setLenVoxelImageIndexBuf(octdata->len_octreeBitstream*10);

	if(octdata->DecodeOctree())
		err("Can't decode");

    int count = 0;

    rnOutputLength = octdata->len_octree + octdata->len_voxels + 8;
    pOutput = new byte[rnOutputLength];

	unsigned char buff[4];
	buff[0]=(octdata->len_octree & 0xFF000000)>>24;
	buff[1]=(octdata->len_octree & 0x00FF0000)>>16;
	buff[2]=(octdata->len_octree & 0x0000FF00)>>8;
	buff[3]=(octdata->len_octree & 0x000000FF);

	memcpy(pOutput,buff,4);
    count += 4;

    for ( int i=0 ; i<octdata->len_octree ; i++ ) {
        pOutput[count++] = (unsigned char) octdata->octree[i];
    }

	buff[0]=(octdata->len_voxels & 0xFF000000)>>24;
	buff[1]=(octdata->len_voxels & 0x00FF0000)>>16;
	buff[2]=(octdata->len_voxels & 0x0000FF00)>>8;
	buff[3]=(octdata->len_voxels & 0x000000FF);

	memcpy(pOutput+4+octdata->len_octree,buff,sizeof(int));
    
    count += 4;
    for ( i=0 ; i<octdata->len_voxels ; i++ ) {
        pOutput[count++] = (unsigned char) octdata->voxelImageIndex[i];
    }
}

void YAFXOctreeDecoder::Write(char* fileName, int defID)
{
	FILE* fp;
	fp = fopen(fileName, "w");

	fprintf(fp, "OctreeImage {\n\toctree [");

	for(int i=0; i<octdata->len_octree; i++)
	{
		if(i%50 == 0)
			fprintf(fp, "\n\t\t");

		fprintf(fp, " %d, ", ((unsigned char*) octdata->octree)[i]);
	}

	fprintf(fp, "\n\t]\n\tvoxelImage [");

	for(i=0; i<octdata->len_voxels; i++)
	{	
		if(i%50==0)
			fprintf(fp, "\n\t\t");

		fprintf(fp, " %d,", ((unsigned char*) octdata->voxelImageIndex)[i]);
	}
	fprintf(fp, "\n\t]\n}\n");

	fclose(fp);

	FreeOctree(&octdata);
}

void YAFXOctreeDecoder::SetValues(unsigned char* pData)
{
	int len_octree, len_voxels;
	unsigned char* octree;
	unsigned char* voxels;
	unsigned char buff[4];

	memcpy(buff,pData,4);
	len_octree=0;
	len_octree=buff[0]<<24;
	len_octree|=buff[1]<<16;
	len_octree|=buff[2]<<8;
	len_octree|=buff[3];

	memcpy(buff,pData+4+len_octree,4);
	len_voxels=0;
	len_voxels=buff[0]<<24;
	len_voxels|=buff[1]<<16;
	len_voxels|=buff[2]<<8;
	len_voxels|=buff[3];

	octree=new unsigned char[len_octree];
	memcpy(octree, pData+4,len_octree);

	voxels=new unsigned char[len_voxels];
	memcpy(voxels,pData+8+len_octree,len_voxels);

    octdata = new COctree;

	octdata->len_octree = len_octree;
    octdata->len_voxels = len_voxels;

    octdata->octree = (int*) octree;
    octdata->voxelImageIndex = (int*) voxels;

}

#ifdef TRACE_ENCODING
void YAFXOctreeDecoder::WriteIntoTracetxt(BIFSDecoder *pCoder)
{
	JWAFXObjectStreamDecoder::WriteIntoTracetxt(pCoder);

	pCoder->TraceNewLine();

	// write the name of node
	pCoder->Tracef("# OctreeImage {");
	pCoder->TraceNewLine();
	pCoder->TraceIndent(1);

	// write the information on key
	pCoder->Tracef("# octree [");
	pCoder->TraceNewLine();
	pCoder->TraceIndent(1);

	for(int i=0; i<octdata->len_octree; i++)
	{
		if(i%50==0){
			pCoder->TraceNewLine();
			pCoder->Tracef("#");			
		}
		pCoder->Tracef(" %d,", octdata->octree[i]);		
	}
	pCoder->TraceNewLine();

	pCoder->TraceIndent(-1);
	pCoder->Tracef("# ]");
	pCoder->TraceNewLine();

	// write the information on keyValue
	pCoder->Tracef("# voxelImageIndex [");
	pCoder->TraceNewLine();
	pCoder->TraceIndent(1);

	for(i=0; i<octdata->len_voxels; i++)
	{	
		if(i%50==0){
			pCoder->TraceNewLine();
			pCoder->Tracef("#");			
		}
		pCoder->Tracef(" %d,",octdata->voxelImageIndex[i]);
	}
	pCoder->TraceNewLine();

	pCoder->TraceIndent(-1);
	pCoder->Tracef("# ]");
	pCoder->TraceNewLine();

	// write the ending brace
	pCoder->TraceIndent(-1);
	pCoder->Tracef("# }");
	pCoder->TraceNewLine();

	FreeOctree(&octdata);
}
#endif

// 2004-07-15, Shinjun {
///////////////////////////////////////////////////////////////////////////////////////
//	PointTexture Compression
///////////////////////////////////////////////////////////////////////////////////////
void AFXPointTextureDecoder::Decode()
{
	// fetch the string to buffer of decoder &	decode the buffer
	m_oPTDecoder.decodeWithBuffer(m_pBuffer, m_nLength);

	// test the value of depth & color pairs
	m_oPTDecoder.write("PointTexture_dec.txt");
}

#if defined SAIT_CONFORMANCE_2003

void AFXPointTextureDecoder::Decode(
	LPBYTE pInput, int nInputLength, LPBYTE& pOutput, int &rnOutputLength)
{
	// variables: pointers, resolution values, indices
	int nX_max = 0;			// maximum in X-axis
	int nY_max = 0;			// maximum in Y-axis
	int nZ_max = 0;			// maximum in Z-axis
	int nIndexOfDepth = 0;
	int nIndexOfColor = 0;
	int i = 0;
	int j = 0;
	int k = 0;

	m_oPTDecoder.decodeWithBuffer((char*)pInput, nInputLength);
	//m_oPTDecoder.write("AFXObject-PT.txt");

	// initial value of 
		// pOutput: the address of memory allocated by DecoderImp
		// rnOutputLength: the size of pOutput
	// final value of
		// rnOutputLength: the real size (actual data size) of pOutput

	// set: resolution values
	nX_max = m_oPTDecoder.m_nWidth-1;						// maximum in X-axis
	nY_max = m_oPTDecoder.m_nHeight-1;						// maximum in Y-axis
	nZ_max = (int)pow(2, m_oPTDecoder.m_nDepthNbBits+1)-1;	// maximum in Z-axis

	// allocate: the output memory
	if(pOutput != NULL)
		{
		free(pOutput);
		pOutput = NULL;
		}
	rnOutputLength = sizeof(int)*3; // width, height, depth
	pOutput = (LPBYTE)realloc((void*)pOutput, rnOutputLength);
	
	// set: the resoution values into the output memory
	((int*)pOutput)[0] = m_oPTDecoder.m_nWidth;
	((int*)pOutput)[1] = m_oPTDecoder.m_nHeight;
	((int*)pOutput)[2] = m_oPTDecoder.m_nDepthNbBits;

	// transfer: the depth data into the output memory
	for(j = 0; j <= nY_max; j++)
		{
		for(i = 0; i <= nX_max; i++)
			{
			int nSize = m_oPTDecoder.m_oColor.getSizeOfDepth(i, j);
			rnOutputLength += sizeof(int);
			pOutput = (LPBYTE)realloc((void*)pOutput, rnOutputLength);
			((int*)pOutput)[3+nIndexOfDepth++] = nSize;

			int* pDepth = m_oPTDecoder.m_oColor.getDepth(i, j);
			for(k = 0; k < nSize; k++)
				{
				rnOutputLength += sizeof(int);
				pOutput = (LPBYTE)realloc((void*)pOutput, rnOutputLength);
				((int*)pOutput)[3+nIndexOfDepth++] = pDepth[k];
				}
			}
		}

	// transfer: the color data into the output memory
	for(j = 0; j <= nY_max; j++)
		{
		for(i = 0; i <= nX_max; i++)
			{
			int nSize = m_oPTDecoder.m_oColor.getSizeOfDepth(i, j);
			
			unsigned char* pColor = m_oPTDecoder.m_oColor.getColor(i, j);
			for(k = 0; k < nSize; k++)
				{
				rnOutputLength += sizeof(unsigned char)*3;
				pOutput = (LPBYTE)realloc((void*)pOutput, rnOutputLength);
				((unsigned char*)(pOutput+sizeof(int)*(3+nIndexOfDepth)))[nIndexOfColor++] = pColor[k*3];
				((unsigned char*)(pOutput+sizeof(int)*(3+nIndexOfDepth)))[nIndexOfColor++] = pColor[k*3+1];
				((unsigned char*)(pOutput+sizeof(int)*(3+nIndexOfDepth)))[nIndexOfColor++] = pColor[k*3+2];
				}
			}
		}
}


void AFXPointTextureDecoder::Write(char* fileName, int defID)
{
	m_oPTDecoder.write(fileName, defID);
}

void AFXPointTextureDecoder::SetValues(unsigned char* pData)
{
	m_oPTDecoder.setValues(pData);
}

#endif SAIT_CONFORMANCE_2003

#ifdef TRACE_ENCODING
void AFXPointTextureDecoder::WriteIntoTracetxt(BIFSDecoder *pCoder)
{
	// variables: resolution values, indices
	int nX_max = 0;			// maximum in X-axis
	int nY_max = 0;			// maximum in Y-axis
	int nZ_max = 0;			// maximum in Z-axis
	int nIndexOfDepth = 0;
	int nIndexOfColor = 0;
	int i = 0;
	int j = 0;
	int k = 0;

	JWAFXObjectStreamDecoder::WriteIntoTracetxt(pCoder);

	// set: resolution values
	nX_max = m_oPTDecoder.m_nWidth-1;						// maximum in X-axis
	nY_max = m_oPTDecoder.m_nHeight-1;						// maximum in Y-axis
	nZ_max = (int)pow(2, m_oPTDecoder.m_nDepthNbBits+1)-1;	// maximum in Z-axis

	pCoder->TraceNewLine();

	// write the name of node
	pCoder->Tracef("# PointTexture {");
	pCoder->TraceNewLine();
	pCoder->TraceIndent(1);

	// write the information of detph
	pCoder->Tracef("# depth [");
	pCoder->TraceNewLine();
	pCoder->TraceIndent(1);

	// trace: the depth data
	for(j = 0; j <= nY_max; j++)
		{
		for(i = 0; i <= nX_max; i++)
			{
			int nSize = m_oPTDecoder.m_oColor.getSizeOfDepth(i, j);
			pCoder->Tracef("# %d, ", nSize);

			int* pDepth = m_oPTDecoder.m_oColor.getDepth(i, j);
			for(k = 0; k < nSize; k++)
				{
				pCoder->Tracef("%d, ", pDepth[k]);
				}
			pCoder->TraceNewLine();
			}
		}

	pCoder->TraceIndent(-1);
	pCoder->Tracef("# ]");
	pCoder->TraceNewLine();

	// write the information of color
	pCoder->Tracef("# color [");
	pCoder->TraceNewLine();
	pCoder->TraceIndent(1);

	// trace: the color data
	nIndexOfColor = 0;
	for(j = 0; j <= nY_max; j++)
		{
		for(i = 0; i <= nX_max; i++)
			{
			int nSize = m_oPTDecoder.m_oColor.getSizeOfDepth(i, j);
			if(nSize > 0)
				pCoder->Tracef("# ");
			
			unsigned char* pColor = m_oPTDecoder.m_oColor.getColor(i, j);
			for(k = 0; k < nSize; k++)
				{
				pCoder->Tracef("%lf, %lf, %lf, ", (double)pColor[k*3]/255, (double)pColor[k*3+1]/255, (double)pColor[k*3+2]/255);
				}
			pCoder->TraceNewLine();
			}
		}

	pCoder->TraceIndent(-1);
	pCoder->Tracef("# ]");
	pCoder->TraceNewLine();

	// write the ending brace
	pCoder->TraceIndent(-1);
	pCoder->Tracef("# }");
	pCoder->TraceNewLine();

}
#endif
// 2004-07-15, Shinjun }


//ory DMLAB 2006 12
///////////////////////////////////////////////////////////////////////////////////////
//	3DMCExtension
///////////////////////////////////////////////////////////////////////////////////////
void JWAFX3DMCExtensionDecoder::Decode()
{
	// fetch the string to buffer of decoder &	decode the buffer
	m_3DMCExtension.DecodeWithBuffer(m_pBuffer, m_nLength);
}


#if defined _3DMCEXT_CONFORMANCE_

void JWAFX3DMCExtensionDecoder::Decode(
	LPBYTE pInput, int nInputLength, LPBYTE& pOutput, int &rnOutputLength)
{
	m_3DMCExtension.DecodeWithBuffer((char*) pInput, nInputLength);

	bool ccw = m_3DMCExtension.GetCcw(); // Read ccw value
	bool convex = m_3DMCExtension.GetConvex();
	float creaseAngle = m_3DMCExtension.GetCreaseAngle();
	bool solid = m_3DMCExtension.GetSolid(); 
	
	int nCoord = m_3DMCExtension.GetCoord(); 
	int nCoordIndex = m_3DMCExtension.GetCoordIdx();
	
	int nColor = m_3DMCExtension.GetColor();
	bool colorPerVertex = m_3DMCExtension.GetColorPerVertex();
	int nColorIndex = m_3DMCExtension.GetColorIndex();
	
	int nNormal = m_3DMCExtension.GetNormal();
	bool normalPerVertex = m_3DMCExtension.GetNormalPerVertex();
	int nNormalIndex = m_3DMCExtension.GetNormalIndex();
	
	int nTexCoord  = m_3DMCExtension.GetTexCoord();
	int nTexCoordIndex = m_3DMCExtension.GetTexCoordIndex();

	// Field Values
	int    *pCoordIndex;
	float  *pCoord;
	int    *pColorIndex;
	float  *pColor;
	int    *pNormalIndex;
	float  *pNormal;
	int    *pTexCoordIndex;
	float  *pTexCoord;

	int nMemorySizeNeeded = 
		sizeof(bool)*5 +
		sizeof(int)*(8+nCoordIndex+nColorIndex+nNormalIndex+nTexCoordIndex) + 
		sizeof(float)*(1+nCoord+nColor+nNormal+nTexCoord);
	
	if(pOutput == NULL)
	{	// in case of BitWrapper.buffer case, pOutput is NULL
		// therefore, pOutput need to be allocated
		pOutput = (LPBYTE) malloc(nMemorySizeNeeded+10000000);
		rnOutputLength = nMemorySizeNeeded;
	}

	int i = 0;
	int   nLengthOfTotal = 0;

	// Headers: ccw
	bool* pCCW = (bool*) pOutput;      
    nLengthOfTotal += sizeof(bool);    
	pCCW[0] = ccw;                    

	// Headers: convex
	bool* pConvex = (bool*)( pOutput + nLengthOfTotal );
	nLengthOfTotal += sizeof(bool);
	pConvex[0] = convex;

	// Headers: creaseAngle
	float* pCreaseAngle = (float *)(pOutput + nLengthOfTotal);
	nLengthOfTotal += sizeof(float);
	pCreaseAngle[0] = creaseAngle;

	// Headers: Solid	
    bool* pSolid = (bool *)( pOutput + nLengthOfTotal);
	nLengthOfTotal += sizeof(bool);
	pSolid[0] = solid;

	// Values: Coord (x,y,z)
    int* pNumCoord = (int*)(pOutput + nLengthOfTotal);
	nLengthOfTotal += sizeof(int);
	pNumCoord[0] = nCoord;                   // Store the number of Coord
	
	pCoord = (float *)( pOutput + nLengthOfTotal); 

	for(i = 0 ; i < nCoord; i++) {
		pCoord[i] = m_3DMCExtension.pCoord[i]; // Store Coord
    }	
	nLengthOfTotal += sizeof(float)*nCoord;

	// Values: coordIndex
	int* pNumCoordIndex = (int*)(pOutput + nLengthOfTotal);
    pNumCoordIndex[0] = nCoordIndex;  // Store the number of CoordIndex
	nLengthOfTotal += sizeof(int);

	pCoordIndex = (int *)( pOutput + nLengthOfTotal);
	for(i = 0 ; i < nCoordIndex; i++) {
		pCoordIndex[i] = m_3DMCExtension.pCoordIdx[i]; 
    }
	nLengthOfTotal += sizeof(float)*nCoordIndex;
		

	// Values: Color
	
	int* pNumColor = (int*)(pOutput + nLengthOfTotal);
    pNumColor[0] = nColor;  // Store the number of Color
	nLengthOfTotal += sizeof(int);


	if(nColor > 0) 
	{
		// ColorPerVertex:  True or False
        bool* pColorPerVertex = (bool *)( pOutput + nLengthOfTotal);
		nLengthOfTotal += sizeof(bool);
	    pColorPerVertex[0] = colorPerVertex;

		// Color Values
		pColor = (float *)( pOutput + nLengthOfTotal);
		for(i = 0 ; i < nColor; i++) {
			pColor[i] = m_3DMCExtension.pColor[i]; 
		}
		nLengthOfTotal += sizeof(float)*nColor;

		// Color Index
		pColorIndex = (int *)(pOutput + nLengthOfTotal);
		nLengthOfTotal += sizeof(int);
		pColorIndex[0] = nColorIndex;

		if(nColorIndex > 0)
		{
			int *pColorIndex = (int *)( pOutput + nLengthOfTotal);
			nLengthOfTotal += sizeof(int)*nColorIndex;
			for(i = 0 ; i < nColorIndex; i++)
			{
				pColorIndex[i] = m_3DMCExtension.pColorIndex[i];
			}
		}
    } 	// if
	
	
	int* pNumNormal = (int*)(pOutput + nLengthOfTotal);
    pNumNormal[0] = nNormal;  // Store the number of Color
	nLengthOfTotal += sizeof(int);


	if(nNormal > 0) 
	{
		// NormalPerVertex:  True or False
		bool* pNormalPerVertex = (bool *)( pOutput + nLengthOfTotal);
		pNormalPerVertex[0] = normalPerVertex;
		nLengthOfTotal += sizeof(bool);

		// Normal Values
		pNormal = (float *)( pOutput + nLengthOfTotal);
		for(i = 0 ; i < nNormal; i++) {
			pNormal[i] = m_3DMCExtension.pNormal[i]; 
		}
		nLengthOfTotal += sizeof(float)*nNormal;

		pNormalIndex = (int *)( pOutput + nLengthOfTotal);
		pNormalIndex[0] = nNormalIndex;
		nLengthOfTotal += sizeof(int);

		// Normal Index
		if(nNormalIndex > 0)
		{
			int *pNormalIndex = (int *)( pOutput + nLengthOfTotal);
			for(i = 0 ; i < nNormalIndex; i++)
			{
				pNormalIndex[i] = m_3DMCExtension.pNormalIndex[i];
			}
			nLengthOfTotal += sizeof(int)*nNormalIndex;
		}
    } 	// if


	int* pNumTexCoord = (int*)(pOutput + nLengthOfTotal);
    pNumTexCoord[0] = nTexCoord;  // Store the number of TexCoord
	nLengthOfTotal += sizeof(int);

	// TexCoord
	if(nTexCoord > 0)
	{
		// Texcoord Values
		pTexCoord = (float *)( pOutput + nLengthOfTotal);
		nLengthOfTotal += sizeof(float)*nTexCoord;

		for(i = 0 ; i < nTexCoord; i++) {
			pTexCoord[i] = m_3DMCExtension.pTexCoord[i]; 
		}

		pTexCoordIndex = (int *)( pOutput + nLengthOfTotal);
		nLengthOfTotal += sizeof(int);
		pTexCoordIndex[0] = nTexCoordIndex;

		// TexCoord Index
		if(nTexCoordIndex > 0)
		{
			int *pTexCoordIndex = (int *)( pOutput + nLengthOfTotal);
			nLengthOfTotal += sizeof(int)*nTexCoordIndex;

			for(i = 0 ; i < nTexCoordIndex; i++)
			{
				pTexCoordIndex[i] = m_3DMCExtension.pTexCoordIndex[i];
			}
		}
    } 	// if

	rnOutputLength = nLengthOfTotal;
}

void JWAFX3DMCExtensionDecoder::Write(char* fileName, int defID)
{
	m_3DMCExtension.write(fileName, defID);
}

void JWAFX3DMCExtensionDecoder::SetValues(unsigned char* pData)
{
	m_3DMCExtension.setValues(pData);
}
#endif _3DMCEXT_CONFORMANCE_


#ifdef TRACE_ENCODING
void JWAFX3DMCExtensionDecoder::WriteIntoTracetxt(BIFSDecoder *pCoder)
{
	JWAFXObjectStreamDecoder::WriteIntoTracetxt(pCoder);

	MPG_IfsQ_E_EXT& a3dmc = * (m_3DMCExtension.GetCIFS());

	pCoder->TraceNewLine();

	// write the name of node
	pCoder->Tracef("# IndexedFaceSet {");
	pCoder->TraceNewLine();
	pCoder->TraceIndent(1);


	if(a3dmc.getCcw() == false)
	{
		//fprintf(fp,"  ccw FALSE\n");
		pCoder->Tracef("# ccw FALSE");
		pCoder->TraceNewLine();
	}

	if(a3dmc.getConvex() == false)
	{
		//fprintf(fp,"  convex FALSE\n");
		pCoder->Tracef("# convex FALSE");
		pCoder->TraceNewLine();
	}

	if(a3dmc.getSolid() == false)
	{
		//fprintf(fp,"  solid FALSE\n");
		pCoder->Tracef("# solid FALSE");
		pCoder->TraceNewLine();
	}

	float creaseAngle = a3dmc.getCreaseAngle();
	if(creaseAngle>0.0)
	{
		//fprintf(fp,"  creaseAngle %8.4f\n",creaseAngle);
		pCoder->Tracef("# creaseAngle %8.4", creaseAngle);
		pCoder->TraceNewLine();
	}

	vector<float>& coord = a3dmc.getCoord();
	vector<int>& coordIndex = a3dmc.getCoordIndex();
	{
		int i;
		//fprintf(fp,"  coord Coordinate {\n");
		pCoder->Tracef("# coord Coordinate {");
		pCoder->TraceNewLine();
		pCoder->TraceIndent(1);

		//fprintf(fp,"   point [\n");
		pCoder->Tracef("# point [");
		pCoder->TraceNewLine();
		pCoder->TraceIndent(1);

		for(i=0; i<coord.size();i+=3)
		{
			//fprintf(fp,"%8.4f ",coord[i]);
			pCoder->Tracef("# %8.4f %8.4f %8.4f", coord[i], coord[i+1], coord[i+2]);
			pCoder->TraceNewLine();
		}

		//fprintf(fp,"    ]\n");
		pCoder->TraceIndent(-1);
		pCoder->Tracef("# ]");
		pCoder->TraceNewLine();

		//fprintf(fp,"  }\n");
		pCoder->TraceIndent(-1);
		pCoder->Tracef("# }");
		pCoder->TraceNewLine();

		//fprintf(fp,"  coordIndex [\n");
		pCoder->Tracef("# coordIndex [");
		pCoder->TraceNewLine();
		pCoder->TraceIndent(1);

		for(i=0;i<coordIndex.size(); i++)
		{
			pCoder->Tracef("# ");

			while(coordIndex[i] != -1)
			{
				if(coordIndex[i] == 2)
					pCoder->Tracef("0 ");
				else
					pCoder->Tracef("%d ", coordIndex[i]);

				i++;
			}

			pCoder->Tracef("%d ", coordIndex[i]);
			pCoder->TraceNewLine();
		}

		//fprintf(fp,"    ]\n");
		pCoder->TraceIndent(-1);
		pCoder->Tracef("# ]");
		pCoder->TraceNewLine();
	}

	bool colorPerVertex = a3dmc.getColorPerVertex();
	vector<float>& color = a3dmc.getColor();
	vector<int>& colorIndex = a3dmc.getColorIndex();

	if(color.size() > 0)
	{
		int i;
		//fprintf(fp,"  colorPerVertex %s\n", (colorPerVertex==true)?"TRUE":"FALSE\n");
		pCoder->Tracef("# colorPerVertex %s", (colorPerVertex == true)? "TRUE":"FALSE");
		pCoder->TraceNewLine();

		//fprintf(fp,"  color Color {\n");
		pCoder->Tracef("# color Color {");
		pCoder->TraceNewLine();
		pCoder->TraceIndent(1);

		//fprintf(fp,"   color [\n");
		pCoder->Tracef("# color [");
		pCoder->TraceNewLine();
		pCoder->TraceIndent(1);

		for(i=0; i<color.size(); i+=3)
		{
			pCoder->Tracef("# ");

			float ci = color[i];
			if(ci<0.0) ci=0.0;
			if(ci>1.0) ci=1.0;
			//fprintf(fp,"%8.4f ",ci);
			pCoder->Tracef("%8.4f ", ci);

			ci = color[i+1];
			if(ci<0.0) ci=0.0;
			if(ci>1.0) ci=1.0;
			//fprintf(fp,"%8.4f ",ci);
			pCoder->Tracef("%8.4f ", ci);

			ci = color[i+2];
			if(ci<0.0) ci=0.0;
			if(ci>1.0) ci=1.0;
			//fprintf(fp,"%8.4f ",ci);
			pCoder->Tracef("%8.4f", ci);

			pCoder->TraceNewLine();

		}

		//fprintf(fp,"    ]\n");
		pCoder->TraceIndent(-1);
		pCoder->Tracef("# ]");
		pCoder->TraceNewLine();

		//fprintf(fp,"  }\n");
		pCoder->TraceIndent(-1);
		pCoder->Tracef("# }");
		pCoder->TraceNewLine();

		if(colorIndex.size() > 0)
		{
			//fprintf(fp,"  colorIndex [\n");
			pCoder->Tracef("# colorIndex [");
			pCoder->TraceNewLine();
			pCoder->TraceNewLine();

			for(i=0; i<colorIndex.size(); i++)
			{
				//fprintf(fp,"%d ",colorIndex[i]);
				//if(colorIndex[i]<0) fprintf(fp,"\n");
				pCoder->Tracef("# ");
				while(colorIndex[i] >= 0)
				{
					pCoder->Tracef("%d ", colorIndex[i]);
					i++;
				}
				pCoder->Tracef("%d", colorIndex[i]);
				pCoder->TraceNewLine();
			}

			//fprintf(fp,"    ]\n");
			pCoder->TraceIndent(-1);
			pCoder->Tracef("# ]");
			pCoder->TraceNewLine();
		}
	}

	bool normalPerVertex = a3dmc.getNormalPerVertex();
	vector<float>& normal = a3dmc.getNormal();
	vector<int>& normalIndex = a3dmc.getNormalIndex();

	if(normal.size() > 0)
	{
		int i;
		//fprintf(fp,"  normalPerVertex %s\n", (normalPerVertex==true)?"TRUE":"FALSE\n");
		pCoder->Tracef("# normalPerVertex %s", (normalPerVertex == true)? "TRUE":"FALSE");
		pCoder->TraceNewLine();

		//fprintf(fp,"  normal Normal {\n");
		pCoder->Tracef("# normal Normal {");
		pCoder->TraceNewLine();
		pCoder->TraceIndent(1);

		//fprintf(fp,"   vector [\n");
		pCoder->Tracef("# vector [");
		pCoder->TraceNewLine();

		for(i=0; i<normal.size(); i+=3)
		{
			//fprintf(fp,"%8.4f ",normal[i]);
			//if(i%3==2) fprintf(fp,"\n");
			pCoder->Tracef("# %8.4f %8.4f %8.4f", normal[i], normal[i+1], normal[i+2]);
			pCoder->TraceNewLine();
		}

		//fprintf(fp,"    ]\n");
		pCoder->TraceIndent(-1);
		pCoder->Tracef("# ]");
		pCoder->TraceNewLine();

		//fprintf(fp,"  }\n");
		pCoder->TraceIndent(-1);
		pCoder->Tracef("# }");
		pCoder->TraceNewLine();

		if(normalIndex.size() > 0)
		{
			//fprintf(fp,"  normalIndex [\n");
			pCoder->Tracef("# normalIndex [");
			pCoder->TraceNewLine();

			for(i=0; i<normalIndex.size(); i++)
			{
				//fprintf(fp,"%d ",normalIndex[i]);
				//if(normalIndex[i]<0) fprintf(fp,"\n");
				pCoder->Tracef("# ");
				while(normalIndex[i] >= 0)
				{
					pCoder->Tracef("%d ", normalIndex[i]);
					i++;
				}
				pCoder->Tracef("%d", normalIndex[i]);
				pCoder->TraceNewLine();
			}

			//fprintf(fp,"    ]\n");
			pCoder->TraceIndent(-1);
			pCoder->Tracef("# ]");
			pCoder->TraceNewLine();
		}
	}

	vector<float>& texCoord = a3dmc.getTexCoord();
	vector<int>& texCoordIndex = a3dmc.getTexCoordIndex();

	if(texCoord.size()>0)
	{
		int i;

		//fprintf(fp,"  texCoord TextureCoordinate {\n");
		pCoder->Tracef("# texCoord TextureCoordinate {");
		pCoder->TraceNewLine();
		pCoder->TraceIndent(1);

		//fprintf(fp,"   point [\n");
		pCoder->Tracef("# point [");
		pCoder->TraceNewLine();
		pCoder->TraceIndent(1);

		for(i=0; i<texCoord.size();i+=2)
		{
			//fprintf(fp,"%8.4f ",texCoord[i]);
			//if(i%2==1) fprintf(fp,"\n");
			pCoder->Tracef("# %8.4f %8.4f", texCoord[i], texCoord[i+1]);
			pCoder->TraceNewLine();
		}

		//fprintf(fp,"    ]\n");
		pCoder->TraceIndent(-1);
		pCoder->Tracef("# ]");
		pCoder->TraceNewLine();

		//fprintf(fp,"  }\n");
		pCoder->TraceIndent(-1);
		pCoder->Tracef("# }");
		pCoder->TraceNewLine();

		if(texCoordIndex.size() > 0)
		{
			//fprintf(fp,"  texCoordIndex [\n");
			pCoder->Tracef("# texCoordIndex [");
			pCoder->TraceNewLine();
			pCoder->TraceIndent(1);

			for(i=0;i<texCoordIndex.size(); i++)
			{
				//fprintf(fp,"%d ",texCoordIndex[i]);
				//if(texCoordIndex[i]<0) fprintf(fp,"\n");
				pCoder->Tracef("# ");
				while(texCoordIndex[i] >= 0)
				{
					pCoder->Tracef("%d ", texCoordIndex[i]);
					i++;
				}
				pCoder->Tracef("%d", texCoordIndex[i]);
				pCoder->TraceNewLine();
			}

			//fprintf(fp,"    ]\n");
			pCoder->TraceIndent(-1);
			pCoder->Tracef("# ]");
			pCoder->TraceNewLine();
		}
	}

	//fprintf(fp," }\n");		// IndexedFaceSet
	pCoder->TraceIndent(-1);
	pCoder->Tracef("# }");
	pCoder->TraceNewLine();
}
#endif
//~ory DMLAB