/***********************************************************************
This software module was originally developed by Sang Oak Woo
(SAMSUNG AIT) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. SAMSUNG AIT retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright ?2002.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Sang Oak Woo at sowoo@sait.samsung.co.kr.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Decoder
FILE:       JWAFXStreamDecoder.h
AUTHOR:     Sang Oak Woo (SAMSUNG AIT)
OVERVIEW:   Decoding the buffer of BitWrapper
-------------------------------------------------------------------------*/


#if !defined SAIT_AFX_STREAM_DECODER
#define SAIT_AFX_STREAM_DECODER

#include "InterpolatorCompression/CompressedCoordinateInterpolator.h"
#include "InterpolatorCompression/CompressedPositionInterpolator.h"
#include "InterpolatorCompression/CompressedOrientationInterpolator.h"

#include "3DMC/Expand/mpgelib/cifsdec.h"
#include "DIBR/Octree.h"
// 2004-07-15, Shinjun {
#include "PTCompression/CPTDecoder.h"
// 2004-07-15, Shinjun }
//ory DMLAB 2007 12 26
#include "3DMCExtension/Expand/MpgeExtLib/cifsdec.h"
//~ory DMLAB


class BitWrapper;
class BIFSDecoder;

class JWAFXObjectStreamDecoder
{
public:
	JWAFXObjectStreamDecoder();
	~JWAFXObjectStreamDecoder()
	{
		if(m_pBuffer != 0)
			::free(m_pBuffer);
	}

	// methods
	virtual void Decode() = 0;

#if defined SAIT_CONFORMANCE_2003 
	virtual void Decode(LPBYTE pInput, int nInputLength, LPBYTE& pOutput, int &rnOutputLength) = 0;
	virtual void Write(char* fileName, int defID) = 0;
	virtual void SetValues(unsigned char* pData) = 0;
#endif SAIT_CONFORMANCE_2003

#if defined _3DMCEXT_CONFORMANCE_
#ifndef SAIT_CONFORMANCE_2003
	virtual void Decode(LPBYTE pInput, int nInputLength, LPBYTE& pOutput, int &rnOutputLength) = 0;
	virtual void Write(char* fileName, int defID) = 0;
	virtual void SetValues(unsigned char* pData) = 0;
#endif
#endif



#ifdef TRACE_ENCODING
	virtual void WriteIntoTracetxt(BIFSDecoder *pCoder); //changed by Helge Drumm
#endif
	void SetBuffer(char* pBuffer, int nLength)
	{
		m_pBuffer = (char*) ::malloc(nLength*sizeof(char));
		memcpy(m_pBuffer, pBuffer, nLength);

		m_nLength = nLength;
	}

protected:
	char* m_pBuffer;
	int m_nLength;
};

class JWAFX3DMCDecoder: public JWAFXObjectStreamDecoder
{
public:
	JWAFX3DMCDecoder() {}
	~JWAFX3DMCDecoder() {}
	
	// methods
	void Decode();
#ifdef TRACE_ENCODING
	virtual void WriteIntoTracetxt(BIFSDecoder *pCoder); //changed by Helge Drumm
#endif

#if defined SAIT_CONFORMANCE_2003
	void Decode(LPBYTE pInput, int nInputLength, LPBYTE& pOutput, int &rnOutputLength);
	void Write(char* fileName, int defID);
	void SetValues(unsigned char* pData);
#endif SAIT_CONFORMANCE_2003

protected:
	CIFSDecoder m_3DMC;
};

//ory DMLAB 2006 12 27
class JWAFX3DMCExtensionDecoder: public JWAFXObjectStreamDecoder
{
public:
	JWAFX3DMCExtensionDecoder() {}
	~JWAFX3DMCExtensionDecoder() {}
	
	// methods
	void Decode();
#ifdef TRACE_ENCODING
	virtual void WriteIntoTracetxt(BIFSDecoder *pCoder); //changed by Helge Drumm
#endif

#if defined _3DMCEXT_CONFORMANCE_
	void Decode(LPBYTE pInput, int nInputLength, LPBYTE& pOutput, int &rnOutputLength);
	void Write(char* fileName, int defID);
	void SetValues(unsigned char* pData);
#endif _3DMCEXT_CONFORMANCE_

protected:	
	CIFSDecoderExt m_3DMCExtension;
	
};
//~ory DMLAB

class JWAFXCoordinateInterpolatorDecoder: public JWAFXObjectStreamDecoder
{
public:
	JWAFXCoordinateInterpolatorDecoder() {}
	~JWAFXCoordinateInterpolatorDecoder() {}
	
	// methods
	void Decode();
#ifdef TRACE_ENCODING
	virtual void WriteIntoTracetxt(BIFSDecoder *pCoder);
#endif


#if defined SAIT_CONFORMANCE_2003
	void Decode(LPBYTE pInput, int nInputLength, LPBYTE& pOutput, int &rnOutputLength);
	void Write(char* fileName, int defID);
	void SetValues(unsigned char* pData);
#endif SAIT_CONFORMANCE_2003

protected:
	CompressedCoordinateInterpolator m_CoordinateInterpolator;
};


class JWAFXPositionInterpolatorDecoder: public JWAFXObjectStreamDecoder
{
public:
	JWAFXPositionInterpolatorDecoder() {}
	~JWAFXPositionInterpolatorDecoder() {}
	
	// methods
	void Decode();
#ifdef TRACE_ENCODING
	virtual void WriteIntoTracetxt(BIFSDecoder *pCoder); 
#endif

#if defined SAIT_CONFORMANCE_2003
	void Decode(LPBYTE pInput, int nInputLength, LPBYTE& pOutput, int &rnOutputLength);
	void Write(char* fileName, int defID);
	void SetValues(unsigned char* pData);
#endif SAIT_CONFORMANCE_2003

protected:
	CompressedPositionInterpolator m_PositionInterpolator;
};


class JWAFXOrientationInterpolatorDecoder: public JWAFXObjectStreamDecoder
{
public:
	JWAFXOrientationInterpolatorDecoder() {}
	~JWAFXOrientationInterpolatorDecoder() {}

	// method
	void Decode();
#ifdef TRACE_ENCODING
	virtual void WriteIntoTracetxt(BIFSDecoder *pCoder);
#endif

#if defined SAIT_CONFORMANCE_2003
	void Decode(LPBYTE pInput, int nInputLength, LPBYTE& pOutput, int &rnOutputLength);
	void Write(char* fileName, int defID);
	void SetValues(unsigned char* pData);
#endif SAIT_CONFORMANCE_2003

protected:
	CompressedOrientationInterpolator m_OrientationInterpolator;
};


//jyhan 2002.12.2
class OctreeDecoder;

class YAFXOctreeDecoder: public JWAFXObjectStreamDecoder
{
public:
	YAFXOctreeDecoder() {}
	~YAFXOctreeDecoder() {}

	// methods
	void Decode() {};


#if defined SAIT_CONFORMANCE_2003
	virtual void Decode(LPBYTE pInput, int nInputLength, LPBYTE& pOutput, int &rnOutputLength);
    virtual void Write(char* fileName, int defID);
    virtual void SetValues(unsigned char* pData);
#endif SAIT_CONFORMANCE_2003

#ifdef TRACE_ENCODING
	virtual void WriteIntoTracetxt(BIFSDecoder *pCoder);
#endif

private: // function
	COctree *octdata;

	void err(const char *str){
		printf("\nError: %s\n",str);
		exit(1);
	}

	int GetOctree(COctree  ** pOctree)
	{
		*pOctree=new COctree;
		return S_OK;
	}

	int FreeOctree(COctree ** pOctree)
	{
		if(*pOctree != NULL)
			delete (*pOctree);
		*pOctree=NULL;
		return S_OK;
	}

protected:
	OctreeDecoder *m_decoder;
};

// 2004-07-15, Shinjun {
class AFXPointTextureDecoder: public JWAFXObjectStreamDecoder
{
public:
	AFXPointTextureDecoder() {}
	~AFXPointTextureDecoder() {}

	// methods
	void Decode();
#ifdef TRACE_ENCODING
	virtual void WriteIntoTracetxt(BIFSDecoder *pCoder);
#endif

#if defined SAIT_CONFORMANCE_2003
	void Decode(LPBYTE pInput, int nInputLength, LPBYTE& pOutput, int &rnOutputLength);
	void Write(char* fileName, int defID);
	void SetValues(unsigned char* pData);
#endif SAIT_CONFORMANCE_2003

protected:
	CPTDecoder m_oPTDecoder;
};
// 2004-07-15, Shinjun }

#endif