/***********************************************************************
This software module was originally developed by Ioan Alexandru Salomie
(Vrije Universiteit Brussel VUB) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. VUB retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2001
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Ioan Alexandru Salomie at iasalomi@etro.vub.ac.be.
----------------------------------------------------------------------*/

#if !defined(_MshGrd_h)	// Sentry, use file only if it's not already included.
#define _MshGrd_h

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Decoder
FILE:       MshGrd.h
AUTHOR:     Ioan Alexandru Salomie (VUB)
OVERVIEW:   Definition of MGDecoder 
-------------------------------------------------------------------------*/
#include "AFX.h"
#include "MshGrd\BasicTypes.h"
#include "..\..\include\ztl.h"

class CMGMesh;
class CMGGrid;
class CMGStreamParser;
class MeshGridDecoderSpecificInfo;

class MGDecoder : public AFXTemplate
{
public:
  MGDecoder();
  ~MGDecoder();

	virtual int GetOptimalOutputSize ();
	virtual BOOL Setup (AFXExtDescriptor *pSpecificInfo);
	virtual BOOL Decode (LPBYTE pInput, int nInputLength, LPBYTE pOutput, int &rnOutputLength);
  bool Decode (LPBYTE pInput, int nInputLength, ZArray<Vec3f> &gridPoints, ZArray<Vec3f> &points);
  bool Decode (ZArray<Vec3f> &gridPoints, ZArray<Vec3f> &points, ZArray<int> &vertexLink,
               ZArray<float> &vertexOffset, const CIPointUVW &nrSlices, const CIPointUVW &nrLevels);
  bool AttachToData(CMemFile *pFile, LPBYTE pInput, int nInputLength, int &dataSize);

  // initialise and destroy mesh and grid objects
  bool CreateMGObjects();
  void DeleteMGObjects();
  void UnrefMGObjects (CMGMesh *&pMesh, CMGGrid *&pGrid);

  bool CheckGridCoord(ZArray<Vec3f> &gridPoints, const CIPointUVW &nrSlices, int cyclicMode, bool &defaultInit);
  void FillInCorners(ZArray<Vec3f> &gridPoints, int uMax, int vMax, int wMax);

private:
	bool			        initMesh;
  CMGGrid         * m_pGrid;
  CMGMesh         * m_pMesh;
  CMGStreamParser * m_pParser;
};

#endif                                      // _MshGrd_h sentry.
