/***********************************************************************
This software module was originally developed by Ioan Alexandru Salomie
(Vrije Universiteit Brussel VUB) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. VUB retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Ioan Alexandru Salomie at iasalomi@etro.vub.ac.be.
----------------------------------------------------------------------*/

// BitStream.h: interface for the CBitStream class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_BITSTREAM_H__315E0CA0_5F02_47F3_AF4A_2C9EEB1BEFBA__INCLUDED_)
#define AFX_BITSTREAM_H__315E0CA0_5F02_47F3_AF4A_2C9EEB1BEFBA__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

class EEC_WritingLimitReached
{
public:
	EEC_WritingLimitReached() {};
};

class EEC_ReadingLimitReached
{
public:
	EEC_ReadingLimitReached() {};
};

class CFile;

class CBitStream  
{
public:
	enum BITSTREAM_OPERATIONS { OP_NA = 0, OP_READ = 1, OP_WRITE = 2 };

	CBitStream();
	CBitStream(BYTE*, UINT, BITSTREAM_OPERATIONS, UINT = -1);
	CBitStream(CFile*, BITSTREAM_OPERATIONS, UINT = -1);
	~CBitStream();

	void Init(CFile*, BITSTREAM_OPERATIONS, UINT = -1);
	void SetOperationMode(BITSTREAM_OPERATIONS);

	// warning: Flush will alter the bitcount
	// if you would like to know the exact bitcount
	// you should call GetBitCount() before Flush()
	// the reason for that is that if other write operations follow
	// the GetBitCount should return the full size of the written stream
	void Flush();
	void Reset();
	void Close(); // Closes the bit stream and reset the content

	void SetBitLimit(UINT l)		{ m_limit = l; }
	void SetByteLimit(UINT l)		{ m_limit = l*8; }
	int  GetBitLimit() const		{ return m_limit; }
	int  GetBitCount() const		{ return m_bitCount; }
	BITSTREAM_OPERATIONS GetOp() const	{ return m_operation; };
	BYTE GetRemainingValue() const;
	BYTE GetBitPosition() const		{ return m_bit; }
	bool LimitReached() const		{ return (m_bitCount >= m_limit); }

	void ReadBit(BYTE* val)			{ *val = !!(ReadBit()); } // count = 1
	BYTE ReadBit();					// count = 1
	void WriteBit(BYTE);			// count = 1

	void ReadBits(BYTE*, BYTE);		// count <= 8
	void WriteBits(BYTE, BYTE);		// count <= 8

	void ReadBits(WORD*, BYTE);		// count <= 16
	void WriteBits(WORD, BYTE);		// count <= 16

	void ReadBits(DWORD*, BYTE);	// count <= 32
	void WriteBits(DWORD, BYTE);	// count <= 32

	void ReadData(BYTE* val)		{ ReadBits(val, 8) ; }
	void WriteData(BYTE val)		{ WriteBits(val, 8); }

	void ReadData(WORD* val)		{ ReadBits(val, 16) ; }
	void WriteData(WORD val)		{ WriteBits(val, 16); }

	void ReadData(DWORD* val)		{ ReadBits(val, 32) ; }
	void WriteData(DWORD val)		{ WriteBits(val, 32); }

	int  ReadDynData(DWORD*);
	void WriteDynData(DWORD);

	int  ReadBytes(BYTE *buffer, int count);

	int  GetFilePosition() const;
	void SetFilePosition(int pos);

protected:
	bool					m_bAllocInternally;
	CFile*					m_stream;
	BITSTREAM_OPERATIONS	m_operation;
	BYTE					m_buffer;
	BYTE					m_bit;
	UINT					m_bitCount;
	UINT					m_limit;
};

#endif // !defined(AFX_BITSTREAM_H__315E0CA0_5F02_47F3_AF4A_2C9EEB1BEFBA__INCLUDED_)
