/***********************************************************************
This software module was originally developed by Ioan Alexandru Salomie
(Vrije Universiteit Brussel VUB) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. VUB retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Ioan Alexandru Salomie at iasalomi@etro.vub.ac.be.
----------------------------------------------------------------------*/

// MGStreamDescription.h: interface for the CMGStreamDescription class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(_GLOBALS)
#define _GLOBALS

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "BasicTypes.h"

#define DevideBy2Even(val, times)		((val) >> (times))
#define MultiplyBy2Even(val, times)		((val) << (times))
#define DevideBy2Odd(val, times)		(((val) + (1 << (times)) - 1) >> (times))
#define DevideBy2EvenMin1(val, times)	(max(((val) >> (times)), 1))
#define MapPositionUp(val, times)		((val) << (times))
#define MapSizeUp(val, times)			((((val) - 1) << (times)) + 1)

inline int EstimateNrROIsSublevel(int nrROIs, int levelDiff)
{
	// estimate nrROIs, at least one ROI
	return DevideBy2EvenMin1(nrROIs, levelDiff);
}

inline void EstimateNrROIsSublevel(CIPointUVW & nrROIs, const CIPointUVW & levelDiff)
{
	// estimate nrROIs, at least one ROI
	nrROIs.u = DevideBy2EvenMin1(nrROIs.u, levelDiff.u);
	nrROIs.v = DevideBy2EvenMin1(nrROIs.v, levelDiff.v);
	nrROIs.w = DevideBy2EvenMin1(nrROIs.w, levelDiff.w);
}

// mapping an index to a position
inline void Index2Position(int index, const CIPointUVW &nrROIs, CIPointUVW &pos)
{
	int area = nrROIs.u * nrROIs.v;
	pos.w = index / area;
	int remaining = index - pos.w * area;
	pos.v = remaining / nrROIs.u;
	pos.u = remaining - pos.v * nrROIs.u;
}

// mapping a position to an index
inline int Position2Index(const CIPointUVW &pos, const CIPointUVW &nrROIs)
{
	int area = nrROIs.u * nrROIs.v;
	return pos.w * area + pos.v * nrROIs.u + pos.u;
}

#endif