#ifndef __LIST_H__
#define __LIST_H__

/**************************************************************************************
// List template
**************************************************************************************/
template <class TYPE, class ARG_TYPE = const TYPE&> class cList
{
public:
	
	cList(int xSize = 16, int xGrow = 16)
	{
		vector = NULL;
		vectorSize = 0;
		size = -1;
		grow = xGrow;
		if (xSize > 0)
			Grow(xSize);
	}

	~cList()
	{
		if (vector != NULL)
		{
			for (int i=0; i<=size; i++)
				(vector+i)->~TYPE();
			delete[] (BYTE*)vector;
		}
	}

	void	Grow(int gr)
	{
		TYPE *tmp;
		tmp = (TYPE*)(new BYTE[(vectorSize + gr)*sizeof(TYPE)]);
		if (gr > 0)
		{
			memcpy(tmp, vector, vectorSize * sizeof(TYPE));
			vectorSize += gr;
		}
		else
		{
			ASSERT(gr < 0);
			vectorSize += gr;
			for (int i=size; i>=vectorSize; i--)
				(vector+i)->~TYPE();
			memcpy(tmp, vector, vectorSize * sizeof(TYPE));
		}
		delete[] (BYTE*)vector;
		vector = tmp;
	}

	void	Reset(int siz)
	{
		ASSERT(siz > 0);
		for (int i=0; i<=size; i++)
			(vector+i)->~TYPE();
		delete[] (BYTE*)vector;
		vector = (TYPE*)(new BYTE[siz*sizeof(TYPE)]);
		for (int i=0; i<siz; i++)
			new((void*)(vector + i)) TYPE;
		vectorSize = siz;
		size = siz - 1;
	}

	inline	TYPE*	GetData()
	{
		return vector;
	}

	inline	TYPE&	operator[](const int index) const
	{
		ASSERT(index<=size);
		return vector[index];
	}

	inline	int		Find(ARG_TYPE elem, int (__cdecl *xCompare) (const void *, const void *)) const
	{
		for (int i = 0; i < size + 1; i++)
		{
			if (xCompare ((void *)&vector[i], (void *)&elem) == 0)
			{
				return i;
			}
		}
		return -1;
	}

	inline void		Insert(ARG_TYPE elem)
	{
		if (vectorSize <= ++size)
			Grow(grow);
		vector[size] = elem;
	}

	inline void		InsertAt(int nIndex, ARG_TYPE elem)
	{
		if (nIndex <= size)
		{
			if (vectorSize <= ++size)
				Grow(grow);
			memmove(vector + nIndex + 1, vector + nIndex, (size - nIndex) * sizeof(TYPE));
			vector[nIndex] = elem;
		}
		else
		{
			if (vectorSize <= nIndex)
				Grow(nIndex - vectorSize + 1);
			for (int i=size+1; i<nIndex; i++)
				new((void*)(vector + i)) TYPE;
			size = nIndex;
			vector[nIndex] = elem;
		}
	}

	inline int		InsertSort(ARG_TYPE elem)
	{
		register int l1 = 0, l2 = size+1;
		while (l1 < l2)
		{
			register int i = (l1 + l2) / 2;
			register TYPE &compElem = vector[i];
			if (compElem == elem)
			{
				l1 = i;
				break;
			}
			if (compElem < elem)
			{
				l1 = i + 1;
			}
			else
			{
				l2 = i;
			}
		}
		InsertAt(l1, elem);
		return l1;
	}

	inline int		FindFirstBelow(ARG_TYPE nSearchedKey) const
	{
		register int l1 = 0, l2 = size+1;
		while (l1 < l2)
		{
			register int i = (l1 + l2) / 2;
			if (vector[i] < nSearchedKey)
			{
				l1 = i + 1;
			}
			else
			{
				l2 = i;
			}
		}
		return l1;
	}

	inline int		FindFirst(ARG_TYPE nSearchedKey) const
	{
		register int l1 = 0, l2 = size+1;
		while (l1 < l2)
		{
			register int i = (l1 + l2) / 2;
			register TYPE &nKey = vector[i];
			if (nKey == nSearchedKey)
			{
				return i;
			}
			if (nKey < nSearchedKey)
			{
				l1 = i + 1;
			}
			else
			{
				l2 = i;
			}
		}
		return -1;
	}

	inline void	RemoveAt(int index)
	{
		if (size > index)
		{
			(vector+index)->~TYPE();
			vector[index] = vector[size];
		}
		size--;
	}

	inline void	DeleteAndMove(int index)
	{
		if (size > index)
		{
			(vector+index)->~TYPE();
			memmove((void *)&vector[index], (void *)&vector[index + 1], (size - index) * sizeof(TYPE));
		}
		size--;
	}

	inline	void	Empty()
	{
		for (int i=0; i<=size; i++)
			(vector+i)->~TYPE();
		size = -1;
	}

	inline	int		GetUpperBound() const
	{
		return size;
	}

	inline	int		GetSize() const
	{
		return (size + 1);
	}

	inline	void	SetSize(int siz)
	{
		if (vectorSize != siz)
			Grow(siz - vectorSize);
		for (int i=siz-1; i>size; i--)
			new((void*)(vector + i)) TYPE;
		size = siz - 1;
	}

	inline	bool	IsEmpty() const
	{
		return (size < 0);
	}

	inline	void	SetGrow(int nNewGrow)
	{
		grow = nNewGrow;
	}

	inline void Update(ARG_TYPE elem, int pos)
	{
		if (pos <= size)
		{
			vector[pos] = elem;
		}
	}

protected:
	int		size;
	int		grow;
	int		vectorSize;
	TYPE*	vector;
};

#endif 