/***********************************************************************
This software module was originally developed by Ioan Alexandru Salomie
(Vrije Universiteit Brussel VUB) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. VUB retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Ioan Alexandru Salomie at iasalomi@etro.vub.ac.be.
----------------------------------------------------------------------*/

// MGDump.cpp: implementation of the CMGMesh class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "MGMesh.h"
#include "MGGrid.h"

void CMGMesh::Dump(CStdioFile &file)
{
	ASSERT(m_pStreamDescr != NULL);

	CString str;
	str.Format("<?xml version=\"1.0\"?>\n\n");  file.WriteString(str);
	str.Format("<MeshGrid>\n");  file.WriteString(str);

	// dump the decoder specific info
	DumpDecoderSpecificInfo(file);

	// dump the reference grid
	DumpGrid(file);

	// dump the connectivity wireframe
	DumpConnectivityWireframe(file);

	// dump the triangles
	DumpTriangles(file);

	str.Format("</MeshGrid>\n");  file.WriteString(str);
}

void CMGMesh::DumpDecoderSpecificInfo(CStdioFile &file)
{
	CString str;

	str.Format("\t<DecoderInfo \n");  file.WriteString(str);

	// number levels
	const CIPointUVW & nrLevels = m_pStreamDescr->GetNrLevelsMesh();
	str.Format("\t\tlevelsU=\"%d\" levelsV=\"%d\" levelsW=\"%d\"\n",
		nrLevels.u, nrLevels.v, nrLevels.w);
	file.WriteString(str);

	// number slices
	const CIPointUVW & nrSlices = m_pStreamDescr->GetMaxNrSlices();
	str.Format("\t\tslicesU=\"%d\" slicesV=\"%d\" slicesW=\"%d\"\n",
		nrSlices.u, nrSlices.v, nrSlices.w);
	file.WriteString(str);

	// number ROIs
	const CIPointUVW & nrROIs = m_pStreamDescr->GetNrROIs();
	str.Format("\t\tnROIsU=\"%d\" nROIsV=\"%d\" nROIsW=\"%d\"\n",
		nrROIs.u, nrROIs.v, nrROIs.w);
	file.WriteString(str);

	// connectivity, refine, reposition and grid flags
	str.Format("\t\tHasConnectivityInfo=\"%s\" ", m_pStreamDescr->HasConnectivityInfo() ? "true": "false");
	file.WriteString(str);

	str.Format("HasRefinementInfo=\"%s\"\n", m_pStreamDescr->HasRefineInfo() ? "true": "false");
	file.WriteString(str);

	str.Format("\t\tHasRepositionInfo=\"%s\" ", m_pStreamDescr->HasRepositionInfo() ? "true": "false");
	file.WriteString(str);

	str.Format("HasGridInfo=\"%s\"\n", m_pStreamDescr->HasGridInfo() ? "true": "false");
	file.WriteString(str);

	// layers
	str.Format("\t\tnLayers=\"%d\" ", m_pStreamDescr->GetNrLayers());
	file.WriteString(str);

	// mesh type
	CString meshType;
	switch (m_pStreamDescr->GetMeshType())
	{
	case GENERIC_MESH: meshType = "Generic";        break;
	case TRI_MESH:     meshType = "Triangular";     break;
	case QUADRI_MESH:  meshType = "Quadrilateral";  break;
	case HEXA_MESH:    meshType = "Hexagonal";      break;
	}
	str.Format("\t\tMeshType=\"%s\" ", meshType);
	file.WriteString(str);

	str.Format("SameBorderDirection=\"%s\" ", m_pStreamDescr->IsSameConnectivity() ? "true": "false");
	file.WriteString(str);

	str.Format("UniformSplit=\"%s\"\n", m_pStreamDescr->IsAutoResolution() ? "true": "false");
	file.WriteString(str);

	// cyclic mesh
	CString cyclicU, cyclicV, folded;
	switch (m_pStreamDescr->GetCyclicMode())
	{
	case CYCLIC_NONE:	cyclicU = "false"; cyclicV = "false";  folded = "false";	break;
	case CYCLIC_U:		cyclicU = "true";  cyclicV = "false";  folded = "false";	break;
	case CYCLIC_UV:		cyclicU = "true";  cyclicV = "true";   folded = "false";	break;
	case FOLDED_SINGLE: cyclicU = "true"; cyclicV = "false";  folded = "single";	break;
	case FOLDED_BOTH:	cyclicU = "true"; cyclicV = "false";  folded = "both";		break;
	}
	str.Format("\t\tcyclicU=\"%s\" cyclicV=\"%s\" folded=\"%s\"\n", cyclicU, cyclicV, folded);
	file.WriteString(str);

	// refine information
	str.Format("\t\trefineBits=\"%d\" ", m_pStreamDescr->GetNrRefineBitsVertex());
	file.WriteString(str);

	str.Format("fullRefine=\"%s\"\n", m_pStreamDescr->InterleaveRefine() ? "true": "false");
	file.WriteString(str);

	// grid scaling
	RGCoding *gridCoding = m_pStreamDescr->GetGridCodingParam();
	str.Format("\t\tgridScaleX=\"%d\" gridScaleY=\"%d\" gridScaleZ=\"%d\">\n",
		gridCoding[0].scale, gridCoding[1].scale, gridCoding[2].scale);
	file.WriteString(str);

	str.Format("\t</DecoderInfo>\n");  file.WriteString(str);
}

void CMGMesh::DumpConnectivityWireframe(CStdioFile &file)
{
	CString str;
	str.Format("\t<ConnectivityWireframe>\n");  file.WriteString(str);

	for (int level = 0; level < GetMaxNrLevels(); level++)
	{
		// resolution
		str.Format("\t\t<resolution level=\"%d\">\n", level);
		file.WriteString(str);

		for (int layer = 0; layer < GetNrLayers(); layer++)
		{
			// layer
			str.Format("\t\t<layer index=\"%d\">\n", layer);
			file.WriteString(str);

			for (int u = 0; u < m_nrSlices.u; u++)
			{
				CSurfNodeList *pList = m_nodeStack[u];
				if (pList == NULL) continue;

				for (int v = 0; v < m_nrSlices.v; v++)
				{
					register CSurfNodeT *pSN = pList[v].GetFirstNode();

					while (pSN != NULL)
					{
						if (pSN->GetLevel() <= level)
						{
							// vertex
							str.Format("\t\t\t<vertex index=\"%d\"\n", pSN->GetIndex());
							file.WriteString(str);

							// position
							const CIPointUVW &pos = pSN->GetPos();
							str.Format("\t\t\t\tu=\"%d\" v=\"%d\" w=\"%d\"\n", pos.u, pos.v, pos.w);
							file.WriteString(str);

							// offset, dir
							CString dir;
							switch (pSN->GetDir())
							{
							case FRONT_FACE:	dir = "FRONT";		break;
							case BACK_FACE:		dir = "BACK";		  break;
							case RIGHT_FACE:	dir = "RIGHT";		break;
							case LEFT_FACE:		dir = "LEFT";		  break;
							case TOP_FACE:		dir = "TOP";			break;
							case BOTTOM_FACE:	dir = "BOTTOM";	  break;
							}
							str.Format("\t\t\t\toffset=\"%.3f\" borderDir=\"%s\"\n", pSN->GetOffset(), dir);
							file.WriteString(str);

							// coordinates
							const CFPoint3DT &p = m_vertices[pSN->GetIndex()];
							str.Format("\t\t\t\tx=\"%.3f\" y=\"%.3f\" z=\"%.3f\"\n", p.x, p.y, p.z);
							file.WriteString(str);

							// connectivity
							CSurfNodeT *pSN1 = pSN->GetNextNode1(level);
							CSurfNodeT *pSN2 = pSN->GetNextNode2(level);
							CSurfNodeT *pSP1 = pSN->GetPrevNode1(level);
							CSurfNodeT *pSP2 = pSN->GetPrevNode2(level);

							if (pSN1 != NULL)
							{
								str.Format("\t\t\t\tlinkNext1=\"%d\" ", pSN1->GetIndex());
								file.WriteString(str);
							}
							else
							{
								str.Format("\t\t\t\tlinkNext1=\"open\" ");
								file.WriteString(str);
							}

							if (pSN2 != NULL)
							{
								str.Format("linkNext2=\"%d\" ", pSN2->GetIndex());
								file.WriteString(str);
							}
							else
							{
								str.Format("linkNext2=\"open\" ");
								file.WriteString(str);
							}

							if (pSP1 != NULL)
							{
								str.Format("linkPrev1=\"%d\" ", pSP1->GetIndex());
								file.WriteString(str);
							}
							else
							{
								str.Format("linkPrev1=\"open\" ");
								file.WriteString(str);
							}

							if (pSP2 != NULL)
							{
								str.Format("linkPrev2=\"%d\">\n", pSP2->GetIndex());
								file.WriteString(str);
							}
							else
							{
								str.Format("linkPrev2=\"open\">\n");
								file.WriteString(str);
							}

							str.Format("\t\t\t</vertex>\n");
							file.WriteString(str);
						}
						pSN = pSN->GetDownList();
					}
				}
			}
			str.Format("\t\t</layer>\n");  file.WriteString(str);
		}
		str.Format("\t\t</resolution>\n");  file.WriteString(str);
	}

	str.Format("\t</ConnectivityWireframe>\n");  file.WriteString(str);
}

void CMGMesh::DumpTriangles(CStdioFile &file)
{
	CString str;
	str.Format("\t<Triangles>\n");  file.WriteString(str);

	for (int level = 0; level < GetMaxNrLevels(); level++)
	{
		// resolution
		str.Format("\t\t<resolution level=\"%d\">\n", level);
		file.WriteString(str);

		DynaSurfPolyT &poly = m_polyLevels[level];
		register int levelSz = poly.GetLevelSize();
		register int count = poly.GetSize();
		register int dynaLevel = -1;
		register int pos = levelSz;
		int counter = GetTotalNrVertices();

		register CSurfPolyT *polyBuf = NULL;
		while (count--)
		{
			if (pos == levelSz)
			{
				pos = 0; ++dynaLevel;
				polyBuf = poly[dynaLevel].GetData();
			}

			// we should find at least a triangle
			CSurfNodeT *pSN1 = polyBuf->GetRef1();
			ASSERT (pSN1 != NULL);

			CSurfNodeT *pSN2 = polyBuf->GetRef2();
			ASSERT (pSN2 != NULL);

			CSurfNodeT *pSN3 = polyBuf->GetRef3();
			ASSERT (pSN3 != NULL);

			ASSERT(pSN1->GetIndex() >= 0 && pSN1->GetIndex() < counter);
			ASSERT(pSN2->GetIndex() >= 0 && pSN2->GetIndex() < counter);
			ASSERT(pSN3->GetIndex() >= 0 && pSN3->GetIndex() < counter);

			str.Format("\t\t\t<triangle index1=\"%d\" index2=\"%d\" index3=\"%d\"></triangle>\n",
				pSN1->GetIndex(), pSN2->GetIndex(), pSN3->GetIndex());
			file.WriteString(str);

			++polyBuf; ++pos;	
		}

		str.Format("\t\t</resolution>\n");  file.WriteString(str);
	}

	str.Format("\t</Triangles>\n");  file.WriteString(str);
}

void CMGGrid::Dump(CStdioFile &file)
{
	CString str;
	str.Format("\t<ReferenceGrid>\n");  file.WriteString(str);

	for (int w = 0; w < m_maxNrSlices.w; w++)
	{
		const GridUV &gridUV = m_gridUVW[w];
		str.Format("\t\t<depth w=\"%d\">\n", w);
		file.WriteString(str);

		for (int v = 0; v < m_maxNrSlices.v; v++)
		{
			const GridU &gridU = gridUV[v];
			str.Format("\t\t\t<width v=\"%d\">\n", v);
			file.WriteString(str);

			for (int u = 0; u < m_maxNrSlices.u; u++)
			{
				const CFPoint3DT &pos = gridU[u];
				str.Format("\t\t\t\t<height u=\"%d\" x=\"%.3f\" y=\"%.3f\" z=\"%.3f\"></height>\n",
					u, pos.x, pos.y, pos.z);
				file.WriteString(str);
			}

			str.Format("\t\t\t</width>\n");
			file.WriteString(str);
		}

		str.Format("\t\t</depth>\n");
		file.WriteString(str);
	}

	str.Format("\t</ReferenceGrid>\n");  file.WriteString(str);
}

