/***********************************************************************
This software module was originally developed by Ioan Alexandru Salomie
(Vrije Universiteit Brussel VUB) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. VUB retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Ioan Alexandru Salomie at iasalomi@etro.vub.ac.be.
----------------------------------------------------------------------*/

// MGGridInterpolLinear.cpp: implementation of the CMGGrid class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "MGGrid.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

void CMGGrid::ComputeDetailsLinear(int baseLevel)
{
	// resolution level of the base grid points
	int maxLevel = GetMaxNrLevels() - 1;
	if (baseLevel < 0 || baseLevel >= maxLevel)  return;

	// compute the steps between the grid lines of same resolution level
	CIPointUVW step, step_2, levelDiff;
	ComputeHomogeneousSteps(step, step_2, levelDiff, baseLevel);

	// handle the w direction
	if (levelDiff.w >= 0)
	{
		for (int w1 = 0, w2 = step.w; w2 < m_maxNrSlices.w; w2 += step.w)
		{
			GridUV &gridUV1  = m_gridUVW[w1];
			GridUV &gridUV2  = m_gridUVW[w1 + step_2.w];
			GridUV &detailUV = m_detailUVW[w1 + step_2.w];
			GridUV &gridUV3  = m_gridUVW[w2];

			for (int v = 0; v < m_maxNrSlices.v; v += step.v)
			{
				GridU &gridU1  = gridUV1[v];
				GridU &gridU2  = gridUV2[v];
				GridU &detailU = detailUV[v];
				GridU &gridU3  = gridUV3[v];

				for (int u = 0; u < m_maxNrSlices.u; u += step.u)
				{
					detailU[u] = gridU2[u] - (gridU1[u] + gridU3[u]) / 2.0f;
				}
			}
			w1 = w2;
		}	
	}

	// finished with z, handle v and u directions
	for (int w = 0; w < m_maxNrSlices.w; w += step_2.w)
	{
		GridUV &gridUV   = m_gridUVW[w];
		GridUV &detailUV = m_detailUVW[w];

		if (levelDiff.v >= 0)
		{
			for (int v1 = 0, v2 = step.v; v2 < m_maxNrSlices.v; v2 += step.v)
			{
				GridU &gridU1  = gridUV[v1];
				GridU &gridU2  = gridUV[v1 + step_2.v];
				GridU &detailU = detailUV[v1 + step_2.v];
				GridU &gridU3  = gridUV[v2];

				for (int u = 0; u < m_maxNrSlices.u; u += step.u)
				{
					detailU[u] = gridU2[u] - (gridU1[u] + gridU3[u]) / 2.0f;
				}
				v1 = v2;
			}
		}

		if (levelDiff.u >= 0)
		{
			for (int v = 0; v < m_maxNrSlices.v; v += step_2.v)
			{
				GridU &gridU   = gridUV[v];
				GridU &detailU = detailUV[v];

				for (int u1 = 0, u2 = step.u; u2 < m_maxNrSlices.u; u2 += step.u)
				{
					detailU[u1 + step_2.u] = gridU[u1 + step_2.u] - (gridU[u1] + gridU[u2]) / 2.0f;
					u1 = u2;
				}
			}
		}
	}
}

void CMGGrid::InterpolateLinear(int baseLevel)
{
	// resolution level of the base grid points
	int maxLevel = GetMaxNrLevels() - 1;
	if (baseLevel < 0 || baseLevel >= maxLevel)  return;

	// compute the steps between the grid lines of same resolution level
	CIPointUVW step, step_2, levelDiff;
	ComputeHomogeneousSteps(step, step_2, levelDiff, baseLevel);

	// handle the w direction
	if (levelDiff.w >= 0)
	{
		for (int w1 = 0, w2 = step.w; w2 < m_maxNrSlices.w; w2 += step.w)
		{
			GridUV &gridUV1 = m_gridUVW[w1];
			GridUV &gridUV2 = m_gridUVW[w1 + step_2.w];
			GridUV &gridUV3 = m_gridUVW[w2];

			for (int v = 0; v < m_maxNrSlices.v; v += step.v)
			{
				GridU &gridU1 = gridUV1[v];
				GridU &gridU2 = gridUV2[v];
				GridU &gridU3 = gridUV3[v];

				for (int u = 0; u < m_maxNrSlices.u; u += step.u)
				{
					gridU2[u] = (gridU1[u] + gridU3[u]) / 2.0f;
				}
			}
			w1 = w2;
		}
	}

	// finished with z, handle v and u directions
	for (int w = 0; w < m_maxNrSlices.w; w += step_2.w)
	{
		GridUV &gridUV = m_gridUVW[w];

		if (levelDiff.v >= 0)
		{
			for (int v1 = 0, v2 = step.v; v2 < m_maxNrSlices.v; v2 += step.v)
			{
				GridU &gridU1 = gridUV[v1];
				GridU &gridU2 = gridUV[v1 + step_2.v];
				GridU &gridU3 = gridUV[v2];

				for (int u = 0; u < m_maxNrSlices.u; u += step.u)
				{
					gridU2[u] = (gridU1[u] + gridU3[u]) / 2.0f;
				}
				v1 = v2;
			}
		}

		if (levelDiff.u >= 0)
		{
			for (int v = 0; v < m_maxNrSlices.v; v += step_2.v)
			{
				GridU &gridU = gridUV[v];
				for (int u1 = 0, u2 = step.u; u2 < m_maxNrSlices.u; u2 += step.u)
				{
					gridU[u1 + step_2.u] = (gridU[u1] + gridU[u2]) / 2.0f;
					u1 = u2;
				}
			}
		}
	}
}

void CMGGrid::InterpolateLinearWithDetail(int baseLevel)
{
	// resolution level of the base grid points
	int maxLevel = GetMaxNrLevels() - 1;
	if (baseLevel < 0 || baseLevel >= maxLevel)  return;

	// compute the steps between the grid lines of same resolution level
	CIPointUVW step, step_2, levelDiff;
	ComputeHomogeneousSteps(step, step_2, levelDiff, baseLevel);

	// handle the w direction
	if (levelDiff.w >= 0)
	{
		for (int w1 = 0, w2 = step.w; w2 < m_maxNrSlices.w; w2 += step.w)
		{
			GridUV &gridUV1  = m_gridUVW[w1];
			GridUV &gridUV2  = m_gridUVW[w1 + step_2.w];
			GridUV &detailUV = m_detailUVW[w1 + step_2.w];
			GridUV &gridUV3  = m_gridUVW[w2];

			for (int v = 0; v < m_maxNrSlices.v; v += step.v)
			{
				GridU &gridU1  = gridUV1[v];
				GridU &gridU2  = gridUV2[v];
				GridU &detailU = detailUV[v];
				GridU &gridU3  = gridUV3[v];

				for (int u = 0; u < m_maxNrSlices.u; u += step.u)
				{
					gridU2[u] = detailU[u] + (gridU1[u] + gridU3[u]) / 2.0f;
				}
			}
			w1 = w2;
		}
	}

	// finished with z, handle v and u directions
	for (int w = 0; w < m_maxNrSlices.w; w += step_2.w)
	{
		GridUV &gridUV   = m_gridUVW[w];
		GridUV &detailUV = m_detailUVW[w];

		if (levelDiff.v >= 0)
		{
			for (int v1 = 0, v2 = step.v; v2 < m_maxNrSlices.v; v2 += step.v)
			{
				GridU &gridU1  = gridUV[v1];
				GridU &gridU2  = gridUV[v1 + step_2.v];
				GridU &detailU = detailUV[v1 + step_2.v];
				GridU &gridU3  = gridUV[v2];

				for (int u = 0; u < m_maxNrSlices.u; u += step.u)
				{
					gridU2[u] = detailU[u] + (gridU1[u] + gridU3[u]) / 2.0f;
				}
				v1 = v2;
			}
		}

		if (levelDiff.u >= 0)
		{
			for (int v = 0; v < m_maxNrSlices.v; v += step_2.v)
			{
				GridU &gridU   = gridUV[v];
				GridU &detailU = detailUV[v];

				for (int u1 = 0, u2 = step.u; u2 < m_maxNrSlices.u; u2 += step.u)
				{
					gridU[u1 + step_2.u] = detailU[u1 + step_2.u] + (gridU[u1] + gridU[u2]) / 2.0f;
					u1 = u2;
				}
			}
		}
	}
}

void CMGGrid::InterpolateLinearWithDetailBackup(int baseLevel)
{
	// resolution level of the base grid points
	int maxLevel = GetMaxNrLevels() - 1;
	if (baseLevel < 0 || baseLevel >= maxLevel)  return;

	// compute the steps between the grid lines of same resolution level
	CIPointUVW step, step_2, levelDiff;
	ComputeHomogeneousSteps(step, step_2, levelDiff, baseLevel);

	// handle the w direction
	if (levelDiff.w >= 0)
	{
		for (int w1 = 0, w2 = step.w; w2 < m_maxNrSlices.w; w2 += step.w)
		{
			GridUV &gridUV1  = m_gridUVW[w1];
			GridUV &gridUV   = m_gridUVW[w1 + step_2.w];
			GridUV &detailUV = m_detailUVW[w1 + step_2.w];
			GridUV &backupUV = m_backupUVW[w1 + step_2.w];
			GridUV &gridUV2  = m_gridUVW[w2];

			for (int v = 0; v < m_maxNrSlices.v; v += step.v)
			{
				GridU &gridU1  = gridUV1[v];
				GridU &gridU   = gridUV[v];
				GridU &detailU = detailUV[v];
				GridU &backupU = backupUV[v];
				GridU &gridU2  = gridUV2[v];

				for (int u = 0; u < m_maxNrSlices.u; u += step.u)
				{
					if (gridU[u] == backupU[u])
						gridU[u] = detailU[u] + (gridU1[u] + gridU2[u]) / 2.0f;

					// make a backup of the new value
					backupU[u] = gridU[u];
				}
			}
			w1 = w2;
		}
	}

	// finished with z, handle v and u directions
	for (int w = 0; w < m_maxNrSlices.w; w += step_2.w)
	{
		GridUV &gridUV   = m_gridUVW[w];
		GridUV &detailUV = m_detailUVW[w];
		GridUV &backupUV = m_backupUVW[w];

		if (levelDiff.v >= 0)
		{
			for (int v1 = 0, v2 = step.v; v2 < m_maxNrSlices.v; v2 += step.v)
			{
				GridU &gridU1  = gridUV[v1];
				GridU &gridU   = gridUV[v1 + step_2.v];
				GridU &detailU = detailUV[v1 + step_2.v];
				GridU &backupU = backupUV[v1 + step_2.v];
				GridU &gridU2  = gridUV[v2];

				for (int u = 0; u < m_maxNrSlices.u; u += step.u)
				{
					if (gridU[u] == backupU[u])
						gridU[u] = detailU[u] + (gridU1[u] + gridU2[u]) / 2.0f;

					// make a backup of the new value
					backupU[u] = gridU[u];
				}
				v1 = v2;
			}	
		}

		if (levelDiff.u >= 0)
		{
			for (int v = 0; v < m_maxNrSlices.v; v += step_2.v)
			{
				GridU &gridU   = gridUV[v];
				GridU &detailU = detailUV[v];
				GridU &backupU = backupUV[v];

				for (int u1 = 0, u2 = step.u; u2 < m_maxNrSlices.u; u2 += step.u)
				{
					int u = u1 + step_2.u;
					if (gridU[u] == backupU[u])
						gridU[u] = detailU[u] + (gridU[u1] + gridU[u2]) / 2.0f;

					// make a backup of the new value
					backupU[u] = gridU[u];

					u1 = u2;
				}
			}
		}
	}
}
