/***********************************************************************
This software module was originally developed by Ioan Alexandru Salomie
(Vrije Universiteit Brussel VUB) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. VUB retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Ioan Alexandru Salomie at iasalomi@etro.vub.ac.be.
----------------------------------------------------------------------*/

// MGGridInterpolSmooth.cpp: implementation of the CMGGrid class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "MGGrid.h"
#include "MGStreamParse.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

void ComputeMirrorPoint(const CFPoint3DT &p1, const CFPoint3DT &p2, CFPoint3DT &p3, CFPoint3DT &p4)
{
	CFPoint3DT a(p2.x - p1.x, p2.y - p1.y, p2.z - p1.z);
	CFPoint3DT b(p3.x - p2.x, p3.y - p2.y, p3.z - p2.z);

	float dist = b.x*b.x + b.y*b.y + b.z*b.z;
	if (dist > 0.0f)
	{
		float k = (a.x*b.x + a.y*b.y + a.z*b.z) / dist;

		p4.x = p1.x + (2*k + 1) * b.x;
		p4.y = p1.y + (2*k + 1) * b.y;
		p4.z = p1.z + (2*k + 1) * b.z;
	}
	else
	{
		p4.x = p1.x + 2*a.x;
		p4.y = p1.y + 2*a.y;
		p4.z = p1.z + 2*a.z;
	}
}

void CMGGrid::ComputeDetailsSmooth(int baseLevel)
{
	// resolution level of the base grid points
	int maxLevel = GetMaxNrLevels() - 1;
	if (baseLevel < 0 || baseLevel >= maxLevel)  return;

	const float c14 = -1.0f/16.0f;
	const float c23 =  9.0f/16.0f;

	// compute the steps between the grid lines of same resolution level
	CIPointUVW step, step_2, levelDiff;
	ComputeHomogeneousSteps(step, step_2, levelDiff, baseLevel);

	const CIPointUVW &nrSlices = GetNrSlices(baseLevel);

	// handle the w direction
	if (levelDiff.w >= 0)
	{
		if (nrSlices.w == 2)
		{
			// only two points, use linear interpolation
			GridUV &gridUV1  = m_gridUVW[0];
			GridUV &gridUV   = m_gridUVW[step_2.w];
			GridUV &detailUV = m_detailUVW[step_2.w];
			GridUV &gridUV2  = m_gridUVW[step.w];

			for (int v = 0; v < m_maxNrSlices.v; v += step.v)
			{
				GridU &gridU1  = gridUV1[v];
				GridU &gridU   = gridUV1[v];
				GridU &detailU = detailUV[v];
				GridU &gridU2  = gridUV1[v];

				for (int u = 0; u < m_maxNrSlices.u; u += step.u)
				{
					detailU[u].x = gridU[u].x - (gridU1[u].x + gridU2[u].x) / 2.0f;
					detailU[u].y = gridU[u].y - (gridU1[u].y + gridU2[u].y) / 2.0f;
					detailU[u].z = gridU[u].z - (gridU1[u].z + gridU2[u].z) / 2.0f;
				}
			}
		}
		else
		{
			// handle the borders in w direction
			{
				{
					GridUV &gridUV2  = m_gridUVW[0];
					GridUV &gridUV   = m_gridUVW[step_2.w];
					GridUV &detailUV = m_detailUVW[step_2.w];
					GridUV &gridUV3  = m_gridUVW[step.w];
					GridUV &gridUV4  = m_gridUVW[2*step.w];

					for (int v = 0; v < m_maxNrSlices.v; v += step.v)
					{
						GridU &gridU2  = gridUV2[v];
						GridU &gridU   = gridUV[v];
						GridU &detailU = detailUV[v];
						GridU &gridU3  = gridUV3[v];
						GridU &gridU4  = gridUV4[v];

						CFPoint3DT p;
						for (int u = 0; u < m_maxNrSlices.u; u += step.u)
						{
							ComputeMirrorPoint(gridU4[u], gridU3[u], gridU2[u], p);
							detailU[u].x = gridU[u].x - c14*(p.x + gridU4[u].x) - c23*(gridU2[u].x + gridU3[u].x);
							detailU[u].y = gridU[u].y - c14*(p.y + gridU4[u].y) - c23*(gridU2[u].y + gridU3[u].y);
							detailU[u].z = gridU[u].z - c14*(p.z + gridU4[u].z) - c23*(gridU2[u].z + gridU3[u].z);
						}
					}
				}

				int z_end = m_maxNrSlices.w - 1;
				{
					GridUV &gridUV1  = m_gridUVW[z_end - 2*step.w];
					GridUV &gridUV2  = m_gridUVW[z_end - step.w];
					GridUV &gridUV   = m_gridUVW[z_end - step_2.w];
					GridUV &detailUV = m_detailUVW[z_end - step_2.w];
					GridUV &gridUV3  = m_gridUVW[z_end];

					for (int v = 0; v < m_maxNrSlices.v; v += step.v)
					{
						GridU &gridU1  = gridUV1[v];
						GridU &gridU2  = gridUV2[v];
						GridU &gridU   = gridUV[v];
						GridU &detailU = detailUV[v];
						GridU &gridU3  = gridUV3[v];

						CFPoint3DT p;
						for (int u = 0; u < m_maxNrSlices.u; u += step.u)
						{
							ComputeMirrorPoint(gridU1[u], gridU2[u], gridU3[u], p);
							detailU[u].x = gridU[u].x - c14*(gridU1[u].x + p.x) - c23*(gridU2[u].x + gridU3[u].x);
							detailU[u].y = gridU[u].y - c14*(gridU1[u].y + p.y) - c23*(gridU2[u].y + gridU3[u].y);
							detailU[u].z = gridU[u].z - c14*(gridU1[u].z + p.z) - c23*(gridU2[u].z + gridU3[u].z);
						}
					}
				}
			}

			// handle the rest in w direction
			for (int w1 = 0, w2 = step.w, w3 = 2*step.w, w4 = 3*step.w; w4 < m_maxNrSlices.w; w4 += step.w)
			{
				GridUV &gridUV1  = m_gridUVW[w1];
				GridUV &gridUV2  = m_gridUVW[w2];
				GridUV &gridUV   = m_gridUVW[w2 + step_2.w];
				GridUV &detailUV = m_detailUVW[w2 + step_2.w];
				GridUV &gridUV3  = m_gridUVW[w3];
				GridUV &gridUV4  = m_gridUVW[w4];

				for (int v = 0; v < m_maxNrSlices.v; v += step.v)
				{
					GridU &gridU1  = gridUV1[v];
					GridU &gridU2  = gridUV2[v];
					GridU &gridU   = gridUV[v];
					GridU &detailU = detailUV[v];
					GridU &gridU3  = gridUV3[v];
					GridU &gridU4  = gridUV4[v];

					for (int u = 0; u < m_maxNrSlices.u; u += step.u)
					{
						detailU[u].x = gridU[u].x - c14*(gridU1[u].x + gridU4[u].x) - c23*(gridU2[u].x + gridU3[u].x);
						detailU[u].y = gridU[u].y - c14*(gridU1[u].y + gridU4[u].y) - c23*(gridU2[u].y + gridU3[u].y);
						detailU[u].z = gridU[u].z - c14*(gridU1[u].z + gridU4[u].z) - c23*(gridU2[u].z + gridU3[u].z);
					}
				}
				w1 = w2;  w2 = w3;  w3 = w4;
			}
		}
	}

	// finished with w, handle v and u directions
	for (int w = 0; w < m_maxNrSlices.w; w += step_2.w)
	{
		GridUV &gridUV   = m_gridUVW[w];
		GridUV &detailUV = m_detailUVW[w];

		if (levelDiff.v >= 0)
		{
			if (nrSlices.v == 2)
			{
				// only two points, use linear interpolation
				GridU &gridU1  = gridUV[0];
				GridU &gridU   = gridUV[step_2.v];
				GridU &detailU = detailUV[step_2.v];
				GridU &gridU2  = gridUV[step.v];

				for (int u = 0; u < m_maxNrSlices.u; u += step.u)
				{
					detailU[u].x = gridU[u].x - (gridU1[u].x + gridU2[u].x) / 2.0f;
					detailU[u].y = gridU[u].y - (gridU1[u].y + gridU2[u].y) / 2.0f;
					detailU[u].z = gridU[u].z - (gridU1[u].z + gridU2[u].z) / 2.0f;
				}
			}
			else
			{
				// handle the borders in v direction
				{
					{
						GridU &gridU1  = gridUV[m_maxNrSlices.v - 1 - step.v]; // in case of cyclic mode
						GridU &gridU2  = gridUV[0];
						GridU &gridU   = gridUV[step_2.v];
						GridU &detailU = detailUV[step_2.v];
						GridU &gridU3  = gridUV[step.v];
						GridU &gridU4  = gridUV[2*step.v];

						if (IsCyclicV())
						{
							for (int u = 0; u < m_maxNrSlices.u; u += step.u)
							{
								detailU[u].x = gridU[u].x - c14*(gridU1[u].x + gridU4[u].x) - c23*(gridU2[u].x + gridU3[u].x);
								detailU[u].y = gridU[u].y - c14*(gridU1[u].y + gridU4[u].y) - c23*(gridU2[u].y + gridU3[u].y);
								detailU[u].z = gridU[u].z - c14*(gridU1[u].z + gridU4[u].z) - c23*(gridU2[u].z + gridU3[u].z);
							}
						}
						else if (IsFoldedSingle())
						{
							for (int u = 0; u < m_maxNrSlices.u; u += step.u)
							{
								detailU[u].x = gridU[u].x - c14*(gridU1[u].x + gridU4[u].x) - c23*(gridU2[u].x + gridU3[u].x);
								detailU[u].y = gridU[u].y - c14*(gridU1[u].y + gridU4[u].y) - c23*(gridU2[u].y + gridU3[u].y);
								detailU[u].z = gridU[u].z - c14*(gridU1[u].z + gridU4[u].z) - c23*(gridU2[u].z + gridU3[u].z);
							}
						}
						else
						{
							CFPoint3DT p;
							for (int u = 0; u < m_maxNrSlices.u; u += step.u)
							{
								ComputeMirrorPoint(gridU4[u], gridU3[u], gridU2[u], p);
								detailU[u].x = gridU[u].x - c14*(p.x + gridU4[u].x) - c23*(gridU2[u].x + gridU3[u].x);
								detailU[u].y = gridU[u].y - c14*(p.y + gridU4[u].y) - c23*(gridU2[u].y + gridU3[u].y);
								detailU[u].z = gridU[u].z - c14*(p.z + gridU4[u].z) - c23*(gridU2[u].z + gridU3[u].z);
							}
						}
					}

					int y_end = m_maxNrSlices.v - 1;
					{
						GridU &gridU1  = gridUV[y_end - 2*step.v];
						GridU &gridU2  = gridUV[y_end - step.v];
						GridU &gridU   = gridUV[y_end - step_2.v];
						GridU &detailU = detailUV[y_end - step_2.v];
						GridU &gridU3  = gridUV[y_end];
						GridU &gridU4  = gridUV[step.v]; // in case of cyclic mode

						if (IsCyclicV())
						{
							for (int u = 0; u < m_maxNrSlices.u; u += step.u)
							{
								detailU[u].x = gridU[u].x - c14*(gridU1[u].x + gridU4[u].x) - c23*(gridU2[u].x + gridU3[u].x);
								detailU[u].y = gridU[u].y - c14*(gridU1[u].y + gridU4[u].y) - c23*(gridU2[u].y + gridU3[u].y);
								detailU[u].z = gridU[u].z - c14*(gridU1[u].z + gridU4[u].z) - c23*(gridU2[u].z + gridU3[u].z);
							}
						}
						else
						{
							CFPoint3DT p;
							for (int u = 0; u < m_maxNrSlices.u; u += step.u)
							{
								ComputeMirrorPoint(gridU1[u], gridU2[u], gridU3[u], p);
								detailU[u].x = gridU[u].x - c14*(gridU1[u].x + p.x) - c23*(gridU2[u].x + gridU3[u].x);
								detailU[u].y = gridU[u].y - c14*(gridU1[u].y + p.y) - c23*(gridU2[u].y + gridU3[u].y);
								detailU[u].z = gridU[u].z - c14*(gridU1[u].z + p.z) - c23*(gridU2[u].z + gridU3[u].z);
							}
						}
					}
				}

				// handle the rest in v direction
				for (int v1 = 0, v2 = step.v, v3 = 2*step.v, v4 = 3*step.v; v4 < m_maxNrSlices.v; v4 += step.v)
				{
					GridU &gridU1  = gridUV[v1];
					GridU &gridU2  = gridUV[v2];
					GridU &gridU   = gridUV[v2 + step_2.v];
					GridU &detailU = detailUV[v2 + step_2.v];
					GridU &gridU3  = gridUV[v3];
					GridU &gridU4  = gridUV[v4];

					for (int u = 0; u < m_maxNrSlices.u; u += step.u)
					{
						detailU[u].x = gridU[u].x - c14*(gridU1[u].x + gridU4[u].x) - c23*(gridU2[u].x + gridU3[u].x);
						detailU[u].y = gridU[u].y - c14*(gridU1[u].y + gridU4[u].y) - c23*(gridU2[u].y + gridU3[u].y);
						detailU[u].z = gridU[u].z - c14*(gridU1[u].z + gridU4[u].z) - c23*(gridU2[u].z + gridU3[u].z);
					}
					v1 = v2;  v2 = v3;  v3 = v4;
				}
			}
		}

		if (levelDiff.u >= 0)
		{
			for (int v = 0; v < m_maxNrSlices.v; v += step_2.v)
			{
				GridU &gridU = gridUV[v];
				GridU &detailU = detailUV[v];

				if (nrSlices.u == 2)
				{
					// only two points, use linear interpolation
					detailU[step_2.u].x = gridU[step_2.u].x - (gridU[0].x + gridU[step.u].x) / 2.0f;
					detailU[step_2.u].y = gridU[step_2.u].y - (gridU[0].y + gridU[step.u].y) / 2.0f;
					detailU[step_2.u].z = gridU[step_2.u].z - (gridU[0].z + gridU[step.u].z) / 2.0f;
				}
				else
				{
					// handle the borders in u direction
					{
						int step2U = step.u << 1;

						int u_end = m_maxNrSlices.u - 1;
						int u  = u_end - step_2.u;
						int u1 = u_end - step2U;
						int u2 = u_end - step.u;

						if (IsCyclicU())
						{
							detailU[step_2.u].x = gridU[step_2.u].x - c14*(gridU[u2].x + gridU[step2U].x) - c23*(gridU[0].x + gridU[step.u].x);
							detailU[step_2.u].y = gridU[step_2.u].y - c14*(gridU[u2].y + gridU[step2U].y) - c23*(gridU[0].y + gridU[step.u].y);
							detailU[step_2.u].z = gridU[step_2.u].z - c14*(gridU[u2].z + gridU[step2U].z) - c23*(gridU[0].z + gridU[step.u].z);

							detailU[u].x = gridU[u].x - c14*(gridU[u1].x + gridU[step.u].x) - c23*(gridU[u2].x + gridU[u_end].x);
							detailU[u].y = gridU[u].y - c14*(gridU[u1].y + gridU[step.u].y) - c23*(gridU[u2].y + gridU[u_end].y);
							detailU[u].z = gridU[u].z - c14*(gridU[u1].z + gridU[step.u].z) - c23*(gridU[u2].z + gridU[u_end].z);
						}
						else
						{
							CFPoint3DT p;
							ComputeMirrorPoint(gridU[step2U], gridU[step.u], gridU[0], p);
							detailU[step_2.u].x = gridU[step_2.u].x - c14*(p.x + gridU[step2U].x) - c23*(gridU[0].x + gridU[step.u].x);
							detailU[step_2.u].y = gridU[step_2.u].y - c14*(p.y + gridU[step2U].y) - c23*(gridU[0].y + gridU[step.u].y);
							detailU[step_2.u].z = gridU[step_2.u].z - c14*(p.z + gridU[step2U].z) - c23*(gridU[0].z + gridU[step.u].z);

							ComputeMirrorPoint(gridU[u1], gridU[u2], gridU[u_end], p);
							detailU[u].x = gridU[u].x - c14*(gridU[u1].x + p.x) - c23*(gridU[u2].x + gridU[u_end].x);
							detailU[u].y = gridU[u].y - c14*(gridU[u1].y + p.y) - c23*(gridU[u2].y + gridU[u_end].y);
							detailU[u].z = gridU[u].z - c14*(gridU[u1].z + p.z) - c23*(gridU[u2].z + gridU[u_end].z);
						}
					}

					// handle the rest in u direction
					for (int u1 = 0, u2 = step.u, u3 = 2*step.u, u4 = 3*step.u; u4 < m_maxNrSlices.u; u4 += step.u)
					{
						int u = u2 + step_2.u;
						detailU[u].x = gridU[u].x - c14*(gridU[u1].x + gridU[u4].x) - c23*(gridU[u2].x + gridU[u3].x);
						detailU[u].y = gridU[u].y - c14*(gridU[u1].y + gridU[u4].y) - c23*(gridU[u2].y + gridU[u3].y);
						detailU[u].z = gridU[u].z - c14*(gridU[u1].z + gridU[u4].z) - c23*(gridU[u2].z + gridU[u3].z);

						u1 = u2;  u2 = u3;  u3 = u4;
					}
				}
			}
		}
	}
}

void CMGGrid::InterpolateSmooth(int baseLevel)
{
	// resolution level of the base grid points
	int maxLevel = GetMaxNrLevels() - 1;
	if (baseLevel < 0 || baseLevel >= maxLevel)  return;

	const float c14 = -1.0f/16.0f;
	const float c23 =  9.0f/16.0f;

	// compute the steps between the grid lines of same resolution level
	CIPointUVW step, step_2, levelDiff;
	ComputeHomogeneousSteps(step, step_2, levelDiff, baseLevel);

	const CIPointUVW &nrSlices = GetNrSlices(baseLevel);

	// handle the w direction
	if (levelDiff.w >= 0)
	{
		if (nrSlices.w == 2)
		{
			// only two points, use linear interpolation
			GridUV &gridUV1 = m_gridUVW[0];
			GridUV &gridUV  = m_gridUVW[step_2.w];
			GridUV &gridUV2 = m_gridUVW[step.w];

			for (int v = 0; v < m_maxNrSlices.v; v += step.v)
			{
				GridU &gridU1 = gridUV1[v];
				GridU &gridU  = gridUV1[v];
				GridU &gridU2 = gridUV1[v];

				for (int u = 0; u < m_maxNrSlices.u; u += step.u)
				{
					gridU[u].x = (gridU1[u].x + gridU2[u].x) / 2.0f;
					gridU[u].y = (gridU1[u].y + gridU2[u].y) / 2.0f;
					gridU[u].z = (gridU1[u].z + gridU2[u].z) / 2.0f;
				}
			}
		}
		else
		{
			// handle the borders in w direction
			{
				{
					GridUV &gridUV2 = m_gridUVW[0];
					GridUV &gridUV  = m_gridUVW[step_2.w];
					GridUV &gridUV3 = m_gridUVW[step.w];
					GridUV &gridUV4 = m_gridUVW[2*step.w];

					for (int v = 0; v < m_maxNrSlices.v; v += step.v)
					{
						GridU &gridU2 = gridUV2[v];
						GridU &gridU  = gridUV[v];
						GridU &gridU3 = gridUV3[v];
						GridU &gridU4 = gridUV4[v];

						CFPoint3DT p;
						for (int u = 0; u < m_maxNrSlices.u; u += step.u)
						{
							ComputeMirrorPoint(gridU4[u], gridU3[u], gridU2[u], p);
							gridU[u].x = c14*(p.x + gridU4[u].x) + c23*(gridU2[u].x + gridU3[u].x);
							gridU[u].y = c14*(p.y + gridU4[u].y) + c23*(gridU2[u].y + gridU3[u].y);
							gridU[u].z = c14*(p.z + gridU4[u].z) + c23*(gridU2[u].z + gridU3[u].z);
						}
					}
				}

				int z_end = m_maxNrSlices.w - 1;
				{
					GridUV &gridUV1 = m_gridUVW[z_end - 2*step.w];
					GridUV &gridUV2 = m_gridUVW[z_end - step.w];
					GridUV &gridUV  = m_gridUVW[z_end - step_2.w];
					GridUV &gridUV3 = m_gridUVW[z_end];

					for (int v = 0; v < m_maxNrSlices.v; v += step.v)
					{
						GridU &gridU1 = gridUV1[v];
						GridU &gridU2 = gridUV2[v];
						GridU &gridU  = gridUV[v];
						GridU &gridU3 = gridUV3[v];

						CFPoint3DT p;
						for (int u = 0; u < m_maxNrSlices.u; u += step.u)
						{
							ComputeMirrorPoint(gridU1[u], gridU2[u], gridU3[u], p);
							gridU[u].x = c14*(gridU1[u].x + p.x) + c23*(gridU2[u].x + gridU3[u].x);
							gridU[u].y = c14*(gridU1[u].y + p.y) + c23*(gridU2[u].y + gridU3[u].y);
							gridU[u].z = c14*(gridU1[u].z + p.z) + c23*(gridU2[u].z + gridU3[u].z);
						}
					}
				}
			}

			// handle the rest in w direction
			for (int w1 = 0, w2 = step.w, w3 = 2*step.w, w4 = 3*step.w; w4 < m_maxNrSlices.w; w4 += step.w)
			{
				GridUV &gridUV1 = m_gridUVW[w1];
				GridUV &gridUV2 = m_gridUVW[w2];
				GridUV &gridUV  = m_gridUVW[w2 + step_2.w];
				GridUV &gridUV3 = m_gridUVW[w3];
				GridUV &gridUV4 = m_gridUVW[w4];

				for (int v = 0; v < m_maxNrSlices.v; v += step.v)
				{
					GridU &gridU1 = gridUV1[v];
					GridU &gridU2 = gridUV2[v];
					GridU &gridU  = gridUV[v];
					GridU &gridU3 = gridUV3[v];
					GridU &gridU4 = gridUV4[v];

					for (int u = 0; u < m_maxNrSlices.u; u += step.u)
					{
						gridU[u].x = c14*(gridU1[u].x + gridU4[u].x) + c23*(gridU2[u].x + gridU3[u].x);
						gridU[u].y = c14*(gridU1[u].y + gridU4[u].y) + c23*(gridU2[u].y + gridU3[u].y);
						gridU[u].z = c14*(gridU1[u].z + gridU4[u].z) + c23*(gridU2[u].z + gridU3[u].z);
					}
				}
				w1 = w2;  w2 = w3;  w3 = w4;
			}
		}
	}

	// finished with w, handle v and u directions
	for (int w = 0; w < m_maxNrSlices.w; w += step_2.w)
	{
		GridUV &gridUV = m_gridUVW[w];

		if (levelDiff.v >= 0)
		{
			if (nrSlices.v == 2)
			{
				// only two points, use linear interpolation
				GridU &gridU1 = gridUV[0];
				GridU &gridU  = gridUV[step_2.v];
				GridU &gridU2 = gridUV[step.v];

				for (int u = 0; u < m_maxNrSlices.u; u += step.u)
				{
					gridU[u].x = (gridU1[u].x + gridU2[u].x) / 2.0f;
					gridU[u].y = (gridU1[u].y + gridU2[u].y) / 2.0f;
					gridU[u].z = (gridU1[u].z + gridU2[u].z) / 2.0f;
				}
			}
			else
			{
				// handle the borders in v direction
				{
					{
						GridU &gridU1 = gridUV[m_maxNrSlices.v - 1 - step.v]; // in case of cyclic mode
						GridU &gridU2 = gridUV[0];
						GridU &gridU  = gridUV[step_2.v];
						GridU &gridU3 = gridUV[step.v];
						GridU &gridU4 = gridUV[2*step.v];

						if (IsCyclicV())
						{
							for (int u = 0; u < m_maxNrSlices.u; u += step.u)
							{
								gridU[u].x = c14*(gridU1[u].x + gridU4[u].x) + c23*(gridU2[u].x + gridU3[u].x);
								gridU[u].y = c14*(gridU1[u].y + gridU4[u].y) + c23*(gridU2[u].y + gridU3[u].y);
								gridU[u].z = c14*(gridU1[u].z + gridU4[u].z) + c23*(gridU2[u].z + gridU3[u].z);
							}
						}
						else
						{
							CFPoint3DT p;
							for (int u = 0; u < m_maxNrSlices.u; u += step.u)
							{
								ComputeMirrorPoint(gridU4[u], gridU3[u], gridU2[u], p);
								gridU[u].x = c14*(p.x + gridU4[u].x) + c23*(gridU2[u].x + gridU3[u].x);
								gridU[u].y = c14*(p.y + gridU4[u].y) + c23*(gridU2[u].y + gridU3[u].y);
								gridU[u].z = c14*(p.z + gridU4[u].z) + c23*(gridU2[u].z + gridU3[u].z);
							}
						}
					}

					int y_end = m_maxNrSlices.v - 1;
					{
						GridU &gridU1 = gridUV[y_end - 2*step.v];
						GridU &gridU2 = gridUV[y_end - step.v];
						GridU &gridU  = gridUV[y_end - step_2.v];
						GridU &gridU3 = gridUV[y_end];
						GridU &gridU4 = gridUV[step.v]; // in case of cyclic mode

						if (IsCyclicV())
						{
							for (int u = 0; u < m_maxNrSlices.u; u += step.u)
							{
								gridU[u].x = c14*(gridU1[u].x + gridU4[u].x) + c23*(gridU2[u].x + gridU3[u].x);
								gridU[u].y = c14*(gridU1[u].y + gridU4[u].y) + c23*(gridU2[u].y + gridU3[u].y);
								gridU[u].z = c14*(gridU1[u].z + gridU4[u].z) + c23*(gridU2[u].z + gridU3[u].z);
							}
						}
						else
						{
							CFPoint3DT p;
							for (int u = 0; u < m_maxNrSlices.u; u += step.u)
							{
								ComputeMirrorPoint(gridU1[u], gridU2[u], gridU3[u], p);
								gridU[u].x = c14*(gridU1[u].x + p.x) + c23*(gridU2[u].x + gridU3[u].x);
								gridU[u].y = c14*(gridU1[u].y + p.y) + c23*(gridU2[u].y + gridU3[u].y);
								gridU[u].z = c14*(gridU1[u].z + p.z) + c23*(gridU2[u].z + gridU3[u].z);
							}
						}
					}
				}

				// handle the rest in v direction
				for (int v1 = 0, v2 = step.v, v3 = 2*step.v, v4 = 3*step.v; v4 < m_maxNrSlices.v; v4 += step.v)
				{
					GridU &gridU1 = gridUV[v1];
					GridU &gridU2 = gridUV[v2];
					GridU &gridU  = gridUV[v2 + step_2.v];
					GridU &gridU3 = gridUV[v3];
					GridU &gridU4 = gridUV[v4];

					for (int u = 0; u < m_maxNrSlices.u; u += step.u)
					{
						gridU[u].x = c14*(gridU1[u].x + gridU4[u].x) + c23*(gridU2[u].x + gridU3[u].x);
						gridU[u].y = c14*(gridU1[u].y + gridU4[u].y) + c23*(gridU2[u].y + gridU3[u].y);
						gridU[u].z = c14*(gridU1[u].z + gridU4[u].z) + c23*(gridU2[u].z + gridU3[u].z);
					}
					v1 = v2;  v2 = v3;  v3 = v4;
				}
			}
		}

		if (levelDiff.u >= 0)
		{
			for (int v = 0; v < m_maxNrSlices.v; v += step_2.v)
			{
				GridU &gridU = gridUV[v];

				if (nrSlices.u == 2)
				{
					// only two points, use linear interpolation
					gridU[step_2.u].x = (gridU[0].x + gridU[step.u].x) / 2.0f;
					gridU[step_2.u].y = (gridU[0].y + gridU[step.u].y) / 2.0f;
					gridU[step_2.u].z = (gridU[0].z + gridU[step.u].z) / 2.0f;
				}
				else
				{
					// handle the borders in u direction
					{
						int step2U = step.u << 1;

						int u_end = m_maxNrSlices.u - 1;
						int u  = u_end - step_2.u;
						int u1 = u_end - step2U;
						int u2 = u_end - step.u;

						if (IsCyclicU())
						{
							gridU[step_2.u].x = c14*(gridU[u2].x + gridU[step2U].x) + c23*(gridU[0].x + gridU[step.u].x);
							gridU[step_2.u].y = c14*(gridU[u2].y + gridU[step2U].y) + c23*(gridU[0].y + gridU[step.u].y);
							gridU[step_2.u].z = c14*(gridU[u2].z + gridU[step2U].z) + c23*(gridU[0].z + gridU[step.u].z);

							gridU[u].x = c14*(gridU[u1].x + gridU[step.u].x) + c23*(gridU[u2].x + gridU[u_end].x);
							gridU[u].y = c14*(gridU[u1].y + gridU[step.u].y) + c23*(gridU[u2].y + gridU[u_end].y);
							gridU[u].z = c14*(gridU[u1].z + gridU[step.u].z) + c23*(gridU[u2].z + gridU[u_end].z);
						}
						else
						{
							CFPoint3DT p;
							ComputeMirrorPoint(gridU[step2U], gridU[step.u], gridU[0], p);
							gridU[step_2.u].x = c14*(p.x + gridU[step2U].x) + c23*(gridU[0].x + gridU[step.u].x);
							gridU[step_2.u].y = c14*(p.y + gridU[step2U].y) + c23*(gridU[0].y + gridU[step.u].y);
							gridU[step_2.u].z = c14*(p.z + gridU[step2U].z) + c23*(gridU[0].z + gridU[step.u].z);

							ComputeMirrorPoint(gridU[u1], gridU[u2], gridU[u_end], p);
							gridU[u].x = c14*(gridU[u1].x + p.x) + c23*(gridU[u2].x + gridU[u_end].x);
							gridU[u].y = c14*(gridU[u1].y + p.y) + c23*(gridU[u2].y + gridU[u_end].y);
							gridU[u].z = c14*(gridU[u1].z + p.z) + c23*(gridU[u2].z + gridU[u_end].z);
						}
					}

					// handle the rest in u direction
					for (int u1 = 0, u2 = step.u, u3 = 2*step.u, u4 = 3*step.u; u4 < m_maxNrSlices.u; u4 += step.u)
					{
						int u = u2 + step_2.u;
						gridU[u].x = c14*(gridU[u1].x + gridU[u4].x) + c23*(gridU[u2].x + gridU[u3].x);
						gridU[u].y = c14*(gridU[u1].y + gridU[u4].y) + c23*(gridU[u2].y + gridU[u3].y);
						gridU[u].z = c14*(gridU[u1].z + gridU[u4].z) + c23*(gridU[u2].z + gridU[u3].z);

						u1 = u2;  u2 = u3;  u3 = u4;
					}
				}
			}
		}
	}
}

void CMGGrid::InterpolateSmoothWithDetail(int baseLevel)
{
	// resolution level of the base grid points
	int maxLevel = GetMaxNrLevels() - 1;
	if (baseLevel < 0 || baseLevel >= maxLevel)  return;

	const float c14 = -1.0f/16.0f;
	const float c23 =  9.0f/16.0f;

	// compute the steps between the grid lines of same resolution level
	CIPointUVW step, step_2, levelDiff;
	ComputeHomogeneousSteps(step, step_2, levelDiff, baseLevel);

	const CIPointUVW &nrSlices = GetNrSlices(baseLevel);

	// handle the w direction
	if (levelDiff.w >= 0)
	{
		if (nrSlices.w == 2)
		{
			// only two points, use linear interpolation
			GridUV &gridUV1  = m_gridUVW[0];
			GridUV &gridUV   = m_gridUVW[step_2.w];
			GridUV &detailUV = m_detailUVW[step_2.w];
			GridUV &gridUV2  = m_gridUVW[step.w];

			for (int v = 0; v < m_maxNrSlices.v; v += step.v)
			{
				GridU &gridU1  = gridUV1[v];
				GridU &gridU   = gridUV1[v];
				GridU &detailU = detailUV[v];
				GridU &gridU2  = gridUV1[v];

				for (int u = 0; u < m_maxNrSlices.u; u += step.u)
				{
					gridU[u].x = detailU[u].x + (gridU1[u].x + gridU2[u].x) / 2.0f;
					gridU[u].y = detailU[u].y + (gridU1[u].y + gridU2[u].y) / 2.0f;
					gridU[u].z = detailU[u].z + (gridU1[u].z + gridU2[u].z) / 2.0f;
				}
			}
		}
		else
		{
			// handle the borders in w direction
			{
				{
					GridUV &gridUV2  = m_gridUVW[0];
					GridUV &gridUV   = m_gridUVW[step_2.w];
					GridUV &detailUV = m_detailUVW[step_2.w];
					GridUV &gridUV3  = m_gridUVW[step.w];
					GridUV &gridUV4  = m_gridUVW[2*step.w];

					for (int v = 0; v < m_maxNrSlices.v; v += step.v)
					{
						GridU &gridU2  = gridUV2[v];
						GridU &gridU   = gridUV[v];
						GridU &detailU = detailUV[v];
						GridU &gridU3  = gridUV3[v];
						GridU &gridU4  = gridUV4[v];

						CFPoint3DT p;
						for (int u = 0; u < m_maxNrSlices.u; u += step.u)
						{
							ComputeMirrorPoint(gridU4[u], gridU3[u], gridU2[u], p);
							gridU[u].x = detailU[u].x + c14*(p.x + gridU4[u].x) + c23*(gridU2[u].x + gridU3[u].x);
							gridU[u].y = detailU[u].y + c14*(p.y + gridU4[u].y) + c23*(gridU2[u].y + gridU3[u].y);
							gridU[u].z = detailU[u].z + c14*(p.z + gridU4[u].z) + c23*(gridU2[u].z + gridU3[u].z);
						}
					}
				}

				int z_end = m_maxNrSlices.w - 1;
				{
					GridUV &gridUV1  = m_gridUVW[z_end - 2*step.w];
					GridUV &gridUV2  = m_gridUVW[z_end - step.w];
					GridUV &gridUV   = m_gridUVW[z_end - step_2.w];
					GridUV &detailUV = m_detailUVW[z_end - step_2.w];
					GridUV &gridUV3  = m_gridUVW[z_end];

					for (int v = 0; v < m_maxNrSlices.v; v += step.v)
					{
						GridU &gridU1  = gridUV1[v];
						GridU &gridU2  = gridUV2[v];
						GridU &gridU   = gridUV[v];
						GridU &detailU = detailUV[v];
						GridU &gridU3  = gridUV3[v];

						CFPoint3DT p;
						for (int u = 0; u < m_maxNrSlices.u; u += step.u)
						{
							ComputeMirrorPoint(gridU1[u], gridU2[u], gridU3[u], p);
							gridU[u].x = detailU[u].x + c14*(gridU1[u].x + p.x) + c23*(gridU2[u].x + gridU3[u].x);
							gridU[u].y = detailU[u].y + c14*(gridU1[u].y + p.y) + c23*(gridU2[u].y + gridU3[u].y);
							gridU[u].z = detailU[u].z + c14*(gridU1[u].z + p.z) + c23*(gridU2[u].z + gridU3[u].z);
						}
					}
				}
			}

			// handle the rest in w direction
			for (int w1 = 0, w2 = step.w, w3 = 2*step.w, w4 = 3*step.w; w4 < m_maxNrSlices.w; w4 += step.w)
			{
				GridUV &gridUV1  = m_gridUVW[w1];
				GridUV &gridUV2  = m_gridUVW[w2];
				GridUV &gridUV   = m_gridUVW[w2 + step_2.w];
				GridUV &detailUV = m_detailUVW[w2 + step_2.w];
				GridUV &gridUV3  = m_gridUVW[w3];
				GridUV &gridUV4  = m_gridUVW[w4];

				for (int v = 0; v < m_maxNrSlices.v; v += step.v)
				{
					GridU &gridU1  = gridUV1[v];
					GridU &gridU2  = gridUV2[v];
					GridU &gridU   = gridUV[v];
					GridU &detailU = detailUV[v];
					GridU &gridU3  = gridUV3[v];
					GridU &gridU4  = gridUV4[v];

					for (int u = 0; u < m_maxNrSlices.u; u += step.u)
					{
						gridU[u].x = detailU[u].x + c14*(gridU1[u].x + gridU4[u].x) + c23*(gridU2[u].x + gridU3[u].x);
						gridU[u].y = detailU[u].y + c14*(gridU1[u].y + gridU4[u].y) + c23*(gridU2[u].y + gridU3[u].y);
						gridU[u].z = detailU[u].z + c14*(gridU1[u].z + gridU4[u].z) + c23*(gridU2[u].z + gridU3[u].z);
					}
				}
				w1 = w2;  w2 = w3;  w3 = w4;
			}
		}
	}

	// finished with w, handle v and u directions
	for (int w = 0; w < m_maxNrSlices.w; w += step_2.w)
	{
		GridUV &gridUV   = m_gridUVW[w];
		GridUV &detailUV = m_detailUVW[w];

		if (levelDiff.v >= 0)
		{
			if (nrSlices.v == 2)
			{
				// only two points, use linear interpolation
				GridU &gridU1  = gridUV[0];
				GridU &gridU   = gridUV[step_2.v];
				GridU &detailU = detailUV[step_2.v];
				GridU &gridU2  = gridUV[step.v];

				for (int u = 0; u < m_maxNrSlices.u; u += step.u)
				{
					gridU[u].x = detailU[u].x + (gridU1[u].x + gridU2[u].x) / 2.0f;
					gridU[u].y = detailU[u].y + (gridU1[u].y + gridU2[u].y) / 2.0f;
					gridU[u].z = detailU[u].z + (gridU1[u].z + gridU2[u].z) / 2.0f;
				}
			}
			else
			{
				// handle the borders in v direction
				{
					{
						GridU &gridU1  = gridUV[m_maxNrSlices.v - 1 - step.v]; // in case of cyclic mode
						GridU &gridU2  = gridUV[0];
						GridU &gridU   = gridUV[step_2.v];
						GridU &detailU = detailUV[step_2.v];
						GridU &gridU3  = gridUV[step.v];
						GridU &gridU4  = gridUV[2*step.v];

						if (IsCyclicV())
						{
							for (int u = 0; u < m_maxNrSlices.u; u += step.u)
							{
								gridU[u].x = detailU[u].x + c14*(gridU1[u].x + gridU4[u].x) + c23*(gridU2[u].x + gridU3[u].x);
								gridU[u].y = detailU[u].y + c14*(gridU1[u].y + gridU4[u].y) + c23*(gridU2[u].y + gridU3[u].y);
								gridU[u].z = detailU[u].z + c14*(gridU1[u].z + gridU4[u].z) + c23*(gridU2[u].z + gridU3[u].z);
							}
						}
						else
						{
							CFPoint3DT p;
							for (int u = 0; u < m_maxNrSlices.u; u += step.u)
							{
								ComputeMirrorPoint(gridU4[u], gridU3[u], gridU2[u], p);
								gridU[u].x = detailU[u].x + c14*(p.x + gridU4[u].x) + c23*(gridU2[u].x + gridU3[u].x);
								gridU[u].y = detailU[u].y + c14*(p.y + gridU4[u].y) + c23*(gridU2[u].y + gridU3[u].y);
								gridU[u].z = detailU[u].z + c14*(p.z + gridU4[u].z) + c23*(gridU2[u].z + gridU3[u].z);
							}
						}
					}

					int y_end = m_maxNrSlices.v - 1;
					{
						GridU &gridU1  = gridUV[y_end - 2*step.v];
						GridU &gridU2  = gridUV[y_end - step.v];
						GridU &gridU   = gridUV[y_end - step_2.v];
						GridU &detailU = detailUV[y_end - step_2.v];
						GridU &gridU3  = gridUV[y_end];
						GridU &gridU4  = gridUV[step.v]; // in case of cyclic mode

						if (IsCyclicV())
						{
							for (int u = 0; u < m_maxNrSlices.u; u += step.u)
							{
								gridU[u].x = detailU[u].x + c14*(gridU1[u].x + gridU4[u].x) + c23*(gridU2[u].x + gridU3[u].x);
								gridU[u].y = detailU[u].y + c14*(gridU1[u].y + gridU4[u].y) + c23*(gridU2[u].y + gridU3[u].y);
								gridU[u].z = detailU[u].z + c14*(gridU1[u].z + gridU4[u].z) + c23*(gridU2[u].z + gridU3[u].z);
							}
						}
						else
						{
							CFPoint3DT p;
							for (int u = 0; u < m_maxNrSlices.u; u += step.u)
							{
								ComputeMirrorPoint(gridU1[u], gridU2[u], gridU3[u], p);
								gridU[u].x = detailU[u].x + c14*(gridU1[u].x + p.x) + c23*(gridU2[u].x + gridU3[u].x);
								gridU[u].y = detailU[u].y + c14*(gridU1[u].y + p.y) + c23*(gridU2[u].y + gridU3[u].y);
								gridU[u].z = detailU[u].z + c14*(gridU1[u].z + p.z) + c23*(gridU2[u].z + gridU3[u].z);
							}
						}
					}
				}

				// handle the rest in v direction
				for (int v1 = 0, v2 = step.v, v3 = 2*step.v, v4 = 3*step.v; v4 < m_maxNrSlices.v; v4 += step.v)
				{
					GridU &gridU1  = gridUV[v1];
					GridU &gridU2  = gridUV[v2];
					GridU &gridU   = gridUV[v2 + step_2.v];
					GridU &detailU = detailUV[v2 + step_2.v];
					GridU &gridU3  = gridUV[v3];
					GridU &gridU4  = gridUV[v4];

					for (int u = 0; u < m_maxNrSlices.u; u += step.u)
					{
						gridU[u].x = detailU[u].x + c14*(gridU1[u].x + gridU4[u].x) + c23*(gridU2[u].x + gridU3[u].x);
						gridU[u].y = detailU[u].y + c14*(gridU1[u].y + gridU4[u].y) + c23*(gridU2[u].y + gridU3[u].y);
						gridU[u].z = detailU[u].z + c14*(gridU1[u].z + gridU4[u].z) + c23*(gridU2[u].z + gridU3[u].z);
					}
					v1 = v2;  v2 = v3;  v3 = v4;
				}
			}
		}

		if (levelDiff.u >= 0)
		{
			for (int v = 0; v < m_maxNrSlices.v; v += step_2.v)
			{
				GridU &gridU = gridUV[v];
				GridU &detailU = detailUV[v];

				if (nrSlices.u == 2)
				{
					// only two points, use linear interpolation
					gridU[step_2.u].x = detailU[step_2.u].x + (gridU[0].x + gridU[step.u].x) / 2.0f;
					gridU[step_2.u].y = detailU[step_2.u].y + (gridU[0].y + gridU[step.u].y) / 2.0f;
					gridU[step_2.u].z = detailU[step_2.u].z + (gridU[0].z + gridU[step.u].z) / 2.0f;
				}
				else
				{
					// handle the borders in u direction
					{
						int step2U = step.u << 1;

						int u_end = m_maxNrSlices.u - 1;
						int u  = u_end - step_2.u;
						int u1 = u_end - step2U;
						int u2 = u_end - step.u;

						if (IsCyclicU())
						{
							gridU[step_2.u].x = detailU[step_2.u].x + c14*(gridU[u2].x + gridU[step2U].x) + c23*(gridU[0].x + gridU[step.u].x);
							gridU[step_2.u].y = detailU[step_2.u].y + c14*(gridU[u2].y + gridU[step2U].y) + c23*(gridU[0].y + gridU[step.u].y);
							gridU[step_2.u].z = detailU[step_2.u].z + c14*(gridU[u2].z + gridU[step2U].z) + c23*(gridU[0].z + gridU[step.u].z);

							gridU[u].x = detailU[u].x + c14*(gridU[u1].x + gridU[step.u].x) + c23*(gridU[u2].x + gridU[u_end].x);
							gridU[u].y = detailU[u].y + c14*(gridU[u1].y + gridU[step.u].y) + c23*(gridU[u2].y + gridU[u_end].y);
							gridU[u].z = detailU[u].z + c14*(gridU[u1].z + gridU[step.u].z) + c23*(gridU[u2].z + gridU[u_end].z);
						}
						else
						{
							CFPoint3DT p;
							ComputeMirrorPoint(gridU[step2U], gridU[step.u], gridU[0], p);
							gridU[step_2.u].x = detailU[step_2.u].x + c14*(p.x + gridU[step2U].x) + c23*(gridU[0].x + gridU[step.u].x);
							gridU[step_2.u].y = detailU[step_2.u].y + c14*(p.y + gridU[step2U].y) + c23*(gridU[0].y + gridU[step.u].y);
							gridU[step_2.u].z = detailU[step_2.u].z + c14*(p.z + gridU[step2U].z) + c23*(gridU[0].z + gridU[step.u].z);

							ComputeMirrorPoint(gridU[u1], gridU[u2], gridU[u_end], p);
							gridU[u].x = detailU[u].x + c14*(gridU[u1].x + p.x) + c23*(gridU[u2].x + gridU[u_end].x);
							gridU[u].y = detailU[u].y + c14*(gridU[u1].y + p.y) + c23*(gridU[u2].y + gridU[u_end].y);
							gridU[u].z = detailU[u].z + c14*(gridU[u1].z + p.z) + c23*(gridU[u2].z + gridU[u_end].z);
						}
					}

					// handle the rest in u direction
					for (int u1 = 0, u2 = step.u, u3 = 2*step.u, u4 = 3*step.u; u4 < m_maxNrSlices.u; u4 += step.u)
					{
						int u = u2 + step_2.u;
						gridU[u].x = detailU[u].x + c14*(gridU[u1].x + gridU[u4].x) + c23*(gridU[u2].x + gridU[u3].x);
						gridU[u].y = detailU[u].y + c14*(gridU[u1].y + gridU[u4].y) + c23*(gridU[u2].y + gridU[u3].y);
						gridU[u].z = detailU[u].z + c14*(gridU[u1].z + gridU[u4].z) + c23*(gridU[u2].z + gridU[u3].z);

						u1 = u2;  u2 = u3;  u3 = u4;
					}
				}
			}
		}
	}
}

void CMGGrid::InterpolateSmoothWithDetailBackup(int baseLevel)
{
	// resolution level of the base grid points
	int maxLevel = GetMaxNrLevels() - 1;
	if (baseLevel < 0 || baseLevel >= maxLevel)  return;

	const float c14 = -1.0f/16.0f;
	const float c23 =  9.0f/16.0f;

	// compute the steps between the grid lines of same resolution level
	CIPointUVW step, step_2, levelDiff;
	ComputeHomogeneousSteps(step, step_2, levelDiff, baseLevel);

	const CIPointUVW &nrSlices = GetNrSlices(baseLevel);

	// handle the w direction
	if (levelDiff.w >= 0)
	{
		if (nrSlices.w == 2)
		{
			// only two points, use linear interpolation
			GridUV &gridUV1  = m_gridUVW[0];
			GridUV &gridUV   = m_gridUVW[step_2.w];
			GridUV &detailUV = m_detailUVW[step_2.w];
			GridUV &backupUV = m_backupUVW[step_2.w];
			GridUV &gridUV2  = m_gridUVW[step.w];

			for (int v = 0; v < m_maxNrSlices.v; v += step.v)
			{
				GridU &gridU1  = gridUV1[v];
				GridU &gridU   = gridUV1[v];
				GridU &detailU = detailUV[v];
				GridU &backupU = backupUV[v];
				GridU &gridU2  = gridUV1[v];

				for (int u = 0; u < m_maxNrSlices.u; u += step.u)
				{
					if (gridU[u] == backupU[u])
					{
						// this value has not been changed externally, interpolate it
						gridU[u].x = detailU[u].x + (gridU1[u].x + gridU2[u].x) / 2.0f;
						gridU[u].y = detailU[u].y + (gridU1[u].y + gridU2[u].y) / 2.0f;
						gridU[u].z = detailU[u].z + (gridU1[u].z + gridU2[u].z) / 2.0f;
					}

					// make a backup of the new value
					backupU[u] = gridU[u];
				}
			}
		}
		else
		{
			// handle the borders in w direction
			{
				{
					GridUV &gridUV2  = m_gridUVW[0];
					GridUV &gridUV   = m_gridUVW[step_2.w];
					GridUV &detailUV = m_detailUVW[step_2.w];
					GridUV &backupUV = m_backupUVW[step_2.w];
					GridUV &gridUV3  = m_gridUVW[step.w];
					GridUV &gridUV4  = m_gridUVW[2*step.w];

					for (int v = 0; v < m_maxNrSlices.v; v += step.v)
					{
						GridU &gridU2  = gridUV2[v];
						GridU &gridU   = gridUV[v];
						GridU &detailU = detailUV[v];
						GridU &backupU = backupUV[v];
						GridU &gridU3  = gridUV3[v];
						GridU &gridU4  = gridUV4[v];

						CFPoint3DT p;
						for (int u = 0; u < m_maxNrSlices.u; u += step.u)
						{
							if (gridU[u] == backupU[u])
							{
								// this value has not been changed externally, interpolate it
								ComputeMirrorPoint(gridU4[u], gridU3[u], gridU2[u], p);
								gridU[u].x = detailU[u].x + c14*(p.x + gridU4[u].x) + c23*(gridU2[u].x + gridU3[u].x);
								gridU[u].y = detailU[u].y + c14*(p.y + gridU4[u].y) + c23*(gridU2[u].y + gridU3[u].y);
								gridU[u].z = detailU[u].z + c14*(p.z + gridU4[u].z) + c23*(gridU2[u].z + gridU3[u].z);
							}

							// make a backup of the new value
							backupU[u] = gridU[u];
						}
					}
				}

				int z_end = m_maxNrSlices.w - 1;
				{
					GridUV &gridUV1  = m_gridUVW[z_end - 2*step.w];
					GridUV &gridUV2  = m_gridUVW[z_end - step.w];
					GridUV &gridUV   = m_gridUVW[z_end - step_2.w];
					GridUV &detailUV = m_detailUVW[z_end - step_2.w];
					GridUV &backupUV = m_backupUVW[z_end - step_2.w];
					GridUV &gridUV3  = m_gridUVW[z_end];

					for (int v = 0; v < m_maxNrSlices.v; v += step.v)
					{
						GridU &gridU1  = gridUV1[v];
						GridU &gridU2  = gridUV2[v];
						GridU &gridU   = gridUV[v];
						GridU &detailU = detailUV[v];
						GridU &backupU = backupUV[v];
						GridU &gridU3  = gridUV3[v];

						CFPoint3DT p;
						for (int u = 0; u < m_maxNrSlices.u; u += step.u)
						{
							if (gridU[u] == backupU[u])
							{
								// this value has not been changed externally, interpolate it
								ComputeMirrorPoint(gridU1[u], gridU2[u], gridU3[u], p);
								gridU[u].x = detailU[u].x + c14*(gridU1[u].x + p.x) + c23*(gridU2[u].x + gridU3[u].x);
								gridU[u].y = detailU[u].y + c14*(gridU1[u].y + p.y) + c23*(gridU2[u].y + gridU3[u].y);
								gridU[u].z = detailU[u].z + c14*(gridU1[u].z + p.z) + c23*(gridU2[u].z + gridU3[u].z);
							}

							// make a backup of the new value
							backupU[u] = gridU[u];
						}
					}
				}
			}

			// handle the rest in w direction
			for (int w1 = 0, w2 = step.w, w3 = 2*step.w, w4 = 3*step.w; w4 < m_maxNrSlices.w; w4 += step.w)
			{
				GridUV &gridUV1  = m_gridUVW[w1];
				GridUV &gridUV2  = m_gridUVW[w2];
				GridUV &gridUV   = m_gridUVW[w2 + step_2.w];
				GridUV &detailUV = m_detailUVW[w2 + step_2.w];
				GridUV &backupUV = m_backupUVW[w2 + step_2.w];
				GridUV &gridUV3  = m_gridUVW[w3];
				GridUV &gridUV4  = m_gridUVW[w4];

				for (int v = 0; v < m_maxNrSlices.v; v += step.v)
				{
					GridU &gridU1  = gridUV1[v];
					GridU &gridU2  = gridUV2[v];
					GridU &gridU   = gridUV[v];
					GridU &detailU = detailUV[v];
					GridU &backupU = backupUV[v];
					GridU &gridU3  = gridUV3[v];
					GridU &gridU4  = gridUV4[v];

					for (int u = 0; u < m_maxNrSlices.u; u += step.u)
					{
						if (gridU[u] == backupU[u])
						{
							// this value has not been changed externally, interpolate it
							gridU[u].x = detailU[u].x + c14*(gridU1[u].x + gridU4[u].x) + c23*(gridU2[u].x + gridU3[u].x);
							gridU[u].y = detailU[u].y + c14*(gridU1[u].y + gridU4[u].y) + c23*(gridU2[u].y + gridU3[u].y);
							gridU[u].z = detailU[u].z + c14*(gridU1[u].z + gridU4[u].z) + c23*(gridU2[u].z + gridU3[u].z);
						}

						// make a backup of the new value
						backupU[u] = gridU[u];
					}
				}
				w1 = w2;  w2 = w3;  w3 = w4;
			}
		}
	}

	// finished with w, handle v and u directions
	for (int w = 0; w < m_maxNrSlices.w; w += step_2.w)
	{
		GridUV &gridUV   = m_gridUVW[w];
		GridUV &detailUV = m_detailUVW[w];
		GridUV &backupUV = m_backupUVW[w];

		if (levelDiff.v >= 0)
		{
			if (nrSlices.v == 2)
			{
				// only two points, use linear interpolation
				GridU &gridU1  = gridUV[0];
				GridU &gridU   = gridUV[step_2.v];
				GridU &detailU = detailUV[step_2.v];
				GridU &backupU = backupUV[step_2.v];
				GridU &gridU2  = gridUV[step.v];

				for (int u = 0; u < m_maxNrSlices.u; u += step.u)
				{
					if (gridU[u] == backupU[u])
					{
						// this value has not been changed externally, interpolate it
						gridU[u].x = detailU[u].x + (gridU1[u].x + gridU2[u].x) / 2.0f;
						gridU[u].y = detailU[u].y + (gridU1[u].y + gridU2[u].y) / 2.0f;
						gridU[u].z = detailU[u].z + (gridU1[u].z + gridU2[u].z) / 2.0f;
					}

					// make a backup of the new value
					backupU[u] = gridU[u];
				}
			}
			else
			{
				// handle the borders in v direction
				{
					{
						GridU &gridU1  = gridUV[m_maxNrSlices.v - 1 - step.v]; // in case of cyclic mode
						GridU &gridU2  = gridUV[0];
						GridU &gridU   = gridUV[step_2.v];
						GridU &detailU = detailUV[step_2.v];
						GridU &backupU = backupUV[step_2.v];
						GridU &gridU3  = gridUV[step.v];
						GridU &gridU4  = gridUV[2*step.v];

						if (IsCyclicV())
						{
							for (int u = 0; u < m_maxNrSlices.u; u += step.u)
							{
								if (gridU[u] == backupU[u])
								{
									// this value has not been changed externally, interpolate it
									gridU[u].x = detailU[u].x + c14*(gridU1[u].x + gridU4[u].x) + c23*(gridU2[u].x + gridU3[u].x);
									gridU[u].y = detailU[u].y + c14*(gridU1[u].y + gridU4[u].y) + c23*(gridU2[u].y + gridU3[u].y);
									gridU[u].z = detailU[u].z + c14*(gridU1[u].z + gridU4[u].z) + c23*(gridU2[u].z + gridU3[u].z);
								}

								// make a backup of the new value
								backupU[u] = gridU[u];
							}
						}
						else
						{
							CFPoint3DT p;
							for (int u = 0; u < m_maxNrSlices.u; u += step.u)
							{
								if (gridU[u] == backupU[u])
								{
									// this value has not been changed externally, interpolate it
									ComputeMirrorPoint(gridU4[u], gridU3[u], gridU2[u], p);
									gridU[u].x = detailU[u].x + c14*(p.x + gridU4[u].x) + c23*(gridU2[u].x + gridU3[u].x);
									gridU[u].y = detailU[u].y + c14*(p.y + gridU4[u].y) + c23*(gridU2[u].y + gridU3[u].y);
									gridU[u].z = detailU[u].z + c14*(p.z + gridU4[u].z) + c23*(gridU2[u].z + gridU3[u].z);
								}

								// make a backup of the new value
								backupU[u] = gridU[u];
							}
						}
					}

					int y_end = m_maxNrSlices.v - 1;
					{
						GridU &gridU1  = gridUV[y_end - 2*step.v];
						GridU &gridU2  = gridUV[y_end - step.v];
						GridU &gridU   = gridUV[y_end - step_2.v];
						GridU &detailU = detailUV[y_end - step_2.v];
						GridU &backupU = backupUV[y_end - step_2.v];
						GridU &gridU3  = gridUV[y_end];
						GridU &gridU4  = gridUV[step.v]; // in case of cyclic mode

						if (IsCyclicV())
						{
							for (int u = 0; u < m_maxNrSlices.u; u += step.u)
							{
								if (gridU[u] == backupU[u])
								{
									// this value has not been changed externally, interpolate it
									gridU[u].x = detailU[u].x + c14*(gridU1[u].x + gridU4[u].x) + c23*(gridU2[u].x + gridU3[u].x);
									gridU[u].y = detailU[u].y + c14*(gridU1[u].y + gridU4[u].y) + c23*(gridU2[u].y + gridU3[u].y);
									gridU[u].z = detailU[u].z + c14*(gridU1[u].z + gridU4[u].z) + c23*(gridU2[u].z + gridU3[u].z);
								}

								// make a backup of the new value
								backupU[u] = gridU[u];
							}
						}
						else
						{
							CFPoint3DT p;
							for (int u = 0; u < m_maxNrSlices.u; u += step.u)
							{
								if (gridU[u] == backupU[u])
								{
									// this value has not been changed externally, interpolate it
									ComputeMirrorPoint(gridU1[u], gridU2[u], gridU3[u], p);
									gridU[u].x = detailU[u].x + c14*(gridU1[u].x + p.x) + c23*(gridU2[u].x + gridU3[u].x);
									gridU[u].y = detailU[u].y + c14*(gridU1[u].y + p.y) + c23*(gridU2[u].y + gridU3[u].y);
									gridU[u].z = detailU[u].z + c14*(gridU1[u].z + p.z) + c23*(gridU2[u].z + gridU3[u].z);
								}

								// make a backup of the new value
								backupU[u] = gridU[u];
							}
						}
					}
				}

				// handle the rest in v direction
				for (int v1 = 0, v2 = step.v, v3 = 2*step.v, v4 = 3*step.v; v4 < m_maxNrSlices.v; v4 += step.v)
				{
					GridU &gridU1  = gridUV[v1];
					GridU &gridU2  = gridUV[v2];
					GridU &gridU   = gridUV[v2 + step_2.v];
					GridU &detailU = detailUV[v2 + step_2.v];
					GridU &backupU = backupUV[v2 + step_2.v];
					GridU &gridU3  = gridUV[v3];
					GridU &gridU4  = gridUV[v4];

					for (int u = 0; u < m_maxNrSlices.u; u += step.u)
					{
						if (gridU[u] == backupU[u])
						{
							// this value has not been changed externally, interpolate it
							gridU[u].x = detailU[u].x + c14*(gridU1[u].x + gridU4[u].x) + c23*(gridU2[u].x + gridU3[u].x);
							gridU[u].y = detailU[u].y + c14*(gridU1[u].y + gridU4[u].y) + c23*(gridU2[u].y + gridU3[u].y);
							gridU[u].z = detailU[u].z + c14*(gridU1[u].z + gridU4[u].z) + c23*(gridU2[u].z + gridU3[u].z);
						}

						// make a backup of the new value
						backupU[u] = gridU[u];
					}
					v1 = v2;  v2 = v3;  v3 = v4;
				}
			}
		}

		if (levelDiff.u >= 0)
		{
			for (int v = 0; v < m_maxNrSlices.v; v += step_2.v)
			{
				GridU &gridU = gridUV[v];
				GridU &detailU = detailUV[v];
				GridU &backupU = backupUV[v];

				if (nrSlices.u == 2)
				{
					// only two points, use linear interpolation
					if (gridU[step_2.u] == backupU[step_2.u])
					{
						gridU[step_2.u].x = detailU[step_2.u].x + (gridU[0].x + gridU[step.u].x) / 2.0f;
						gridU[step_2.u].y = detailU[step_2.u].y + (gridU[0].y + gridU[step.u].y) / 2.0f;
						gridU[step_2.u].z = detailU[step_2.u].z + (gridU[0].z + gridU[step.u].z) / 2.0f;
					}

					// make a backup of the new value
					backupU[step_2.u] = gridU[step_2.u];
				}
				else
				{
					// handle the borders in u direction
					{
						int step2U = step.u << 1;

						int u_end = m_maxNrSlices.u - 1;
						int u  = u_end - step_2.u;
						int u1 = u_end - step2U;
						int u2 = u_end - step.u;

						if (IsCyclicU())
						{
							if (gridU[step_2.u] == backupU[step_2.u])
							{
								// this value has not been changed externally, interpolate it
								gridU[step_2.u].x = detailU[step_2.u].x + c14*(gridU[u2].x + gridU[step2U].x) + c23*(gridU[0].x + gridU[step.u].x);
								gridU[step_2.u].y = detailU[step_2.u].y + c14*(gridU[u2].y + gridU[step2U].y) + c23*(gridU[0].y + gridU[step.u].y);
								gridU[step_2.u].z = detailU[step_2.u].z + c14*(gridU[u2].z + gridU[step2U].z) + c23*(gridU[0].z + gridU[step.u].z);
							}

							if (gridU[u] == backupU[u])
							{
								// this value has not been changed externally, interpolate it
								gridU[u].x = detailU[u].x + c14*(gridU[u1].x + gridU[step.u].x) + c23*(gridU[u2].x + gridU[u_end].x);
								gridU[u].y = detailU[u].y + c14*(gridU[u1].y + gridU[step.u].y) + c23*(gridU[u2].y + gridU[u_end].y);
								gridU[u].z = detailU[u].z + c14*(gridU[u1].z + gridU[step.u].z) + c23*(gridU[u2].z + gridU[u_end].z);
							}
						}
						else
						{
							CFPoint3DT p;
							if (gridU[step_2.u] == backupU[step_2.u])
							{
								// this value has not been changed externally, interpolate it
								ComputeMirrorPoint(gridU[step2U], gridU[step.u], gridU[0], p);
								gridU[step_2.u].x = detailU[step_2.u].x + c14*(p.x + gridU[step2U].x) + c23*(gridU[0].x + gridU[step.u].x);
								gridU[step_2.u].y = detailU[step_2.u].y + c14*(p.y + gridU[step2U].y) + c23*(gridU[0].y + gridU[step.u].y);
								gridU[step_2.u].z = detailU[step_2.u].z + c14*(p.z + gridU[step2U].z) + c23*(gridU[0].z + gridU[step.u].z);
							}

							if (gridU[u] == backupU[u])
							{
								// this value has not been changed externally, interpolate it
								ComputeMirrorPoint(gridU[u1], gridU[u2], gridU[u_end], p);
								gridU[u].x = detailU[u].x + c14*(gridU[u1].x + p.x) + c23*(gridU[u2].x + gridU[u_end].x);
								gridU[u].y = detailU[u].y + c14*(gridU[u1].y + p.y) + c23*(gridU[u2].y + gridU[u_end].y);
								gridU[u].z = detailU[u].z + c14*(gridU[u1].z + p.z) + c23*(gridU[u2].z + gridU[u_end].z);
							}
						}

						// make a backup of the new value
						backupU[step_2.u] = gridU[step_2.u];
						backupU[u] = gridU[u];
					}

					// handle the rest in u direction
					for (int u1 = 0, u2 = step.u, u3 = 2*step.u, u4 = 3*step.u; u4 < m_maxNrSlices.u; u4 += step.u)
					{
						int u = u2 + step_2.u;
						if (gridU[u] == backupU[u])
						{
							// this value has not been changed externally, interpolate it
							gridU[u].x = detailU[u].x + c14*(gridU[u1].x + gridU[u4].x) + c23*(gridU[u2].x + gridU[u3].x);
							gridU[u].y = detailU[u].y + c14*(gridU[u1].y + gridU[u4].y) + c23*(gridU[u2].y + gridU[u3].y);
							gridU[u].z = detailU[u].z + c14*(gridU[u1].z + gridU[u4].z) + c23*(gridU[u2].z + gridU[u3].z);
						}

						// make a backup of the new value
						backupU[u] = gridU[u];

						u1 = u2;  u2 = u3;  u3 = u4;
					}
				}
			}
		}
	}
}
