/***********************************************************************
This software module was originally developed by Ioan Alexandru Salomie
(Vrije Universiteit Brussel VUB) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. VUB retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Ioan Alexandru Salomie at iasalomi@etro.vub.ac.be.
----------------------------------------------------------------------*/

// MGMemoryDecoder.cpp: implementation of the CMGStreamParser class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "MGStreamParse.h"
#include "BitStream.h"
#include "MGErrMsg.h"
#include "MGMesh.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


void CMGStreamParser::Attach(BYTE *data, int nrBytes)
{
  m_memFile.Detach();
  m_memFile.Attach(data, nrBytes, 0);

  // create the stream object
  CreateStream(&m_memFile, CBitStream::OP_READ);
}

void CMGStreamParser::ParseMeshGridHeader(BYTE *data, int nrBytes)
{
  // attach the data to a stream
  Attach(data, nrBytes);

  // parse the mesh description
  ParseMeshGridHeader();
}

bool CMGStreamParser::ReadMeshGridHeader(BYTE *&buf)
{
  return (buf != NULL && 
          m_streamMG.RawRead(buf, 0, m_streamDescr.GetHeaderSize()));
}

void CMGStreamParser::ParseMeshLevelHeader(BYTE *data, int nrBytes, int level)
{
  // attach the data to a stream
  Attach(data, nrBytes);

  // parse the mesh description & byte allign data
//  MeshDescriptor(level);
}

bool CMGStreamParser::ReadMeshLevelHeader(BYTE *&buf, int level)
{
  MStreamLevel &meshlevel = m_streamDescr.GetMeshLevel(level);

  return (buf != NULL && 
          m_streamMG.RawRead(buf, meshlevel.GetHeaderPosition(),
                                  meshlevel.GetHeaderSize()));
}

void CMGStreamParser::ParseGridLevelHeader(BYTE *data, int nrBytes, int level)
{
  // attach the data to a stream
  Attach(data, nrBytes);

  // parse the grid description
  GridDescriptor(level);
}

bool CMGStreamParser::ReadGridLevelHeader(BYTE *&buf, int level)
{
  GStreamLevel &gridlevel = m_streamDescr.GetGridLevel(level);

  return (buf != NULL && 
          m_streamMG.RawRead(buf, gridlevel.GetHeaderPosition(),
                                  gridlevel.GetHeaderSize()));
}
