/***********************************************************************
This software module was originally developed by Ioan Alexandru Salomie
(Vrije Universiteit Brussel VUB) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. VUB retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Ioan Alexandru Salomie at iasalomi@etro.vub.ac.be.
----------------------------------------------------------------------*/

// MGMesh.h: interface for the CMGMesh class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_MGMESH_H__98F34843_D015_481A_888D_DB966BC3461B__INCLUDED_)
#define AFX_MGMESH_H__98F34843_D015_481A_888D_DB966BC3461B__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "BasicTypes.h"
#include "MGGrid.h"
#include "MGErrMsg.h"
#include "MGGridCodec.h"
#include "MGStreamdescription.h"

#define BACK_FACE    32   // 00100000
#define FRONT_FACE   64   // 01000000
#define LEFT_FACE    96   // 01100000
#define RIGHT_FACE  128   // 10000000
#define BOTTOM_FACE 160   // 10100000
#define TOP_FACE    192   // 11000000
#define BITS_FACE   224   // 11100000
#define RESET_FACE   31   // 00011111
#define OFFSET_DIR		5		// skip first five bits

#define BITS_COUNT    7   // 00000111
#define RESET_COUNT 248   // 11111000

#define KEEP_MASK             128 // 10000000
#define RESET_KEEP_MASK       127 // 01111111
#define LINK1_MASK             64 // 01000000
#define RESET_LINK1_MASK      191 // 10111111
#define LINK2_MASK             32 // 00100000
#define RESET_LINK2_MASK      223 // 11011111
#define LINK_MASK              96 // 01100000
#define RESET_LINK_MASK       159 // 10011111
#define HEXAGONE_MASK1         16 // 00010000
#define RESET_HEXAGONE_MASK1  239 // 11101111
#define TRIANGLE_MASK1          8 // 00001000
#define RESET_TRIANGLE_MASK1  247 // 11110111
#define TRIANGLE_MASK2          4 // 00000100
#define RESET_TRIANGLE_MASK2  251 // 11111011
#define	STREAM_MASK							1 // 00000001
#define RESET_STREAM_MASK			254 // 11111110

#define BITS_LEVEL						 15 // 00001111
#define RESET_LEVEL						240 // 11110000
#define MAX_LEVEL_IDX					BITS_LEVEL

#define NEXT1_MASK						128 // 10000000
#define RESET_NEXT1_MASK			127 // 01111111
#define NEXT2_MASK						 64 // 01000000
#define RESET_NEXT2_MASK			191 // 10111111
#define PREV1_MASK						 32 // 00100000
#define RESET_PREV1_MASK			223 // 11011111
#define PREV2_MASK						 16 // 00010000
#define RESET_PREV2_MASK			239 // 11101111


////////////////////////////////////////////////////////////////////////
//MAXAD
////////////////////////////////////////////////////////////////////////

struct STruncationPoint
{
	int		threshold;
	int		length;		//Delta
	double	distortion;	//Delta
	double	lambda;
};

class CTruncationPoints: public CDataArray<STruncationPoint, STruncationPoint&>
{
public:
	STruncationPoint&	GetPoint(int threshold)
	{
		for (int i=m_nSize-1; i>=0; i--)
			if (m_pData[i].threshold == threshold)
				return m_pData[i];
		STruncationPoint &trPoint = AddEmpty();
		trPoint.threshold = threshold;
		trPoint.length = 0;
		trPoint.distortion = -1.0;
		return trPoint;
	}
};
struct CTruncations
{
	CTruncationPoints	points;
	int					truncation;	//Index of the point till which we have to code (inclusiv)
};

////////////////////////////////////////////////////////////////////////
//class CSurfNodeT
////////////////////////////////////////////////////////////////////////

class CSurfNodeT
{
	// construct, destruct
public:
	CSurfNodeT(const CIPointUVW &p, int dir, int nrLevels, int index, int level, BYTE layer, float offset);

	~CSurfNodeT();

	// Member functions
public:
	CIPointUVW &GetPos() { return m_pos; };
	void	SetPos(const CIPointUVW &pos);
	int		GetUPos() const { return m_pos.u; };
	int		GetVPos() const { return m_pos.v; };
	int		GetWPos() const { return m_pos.w; };

	BYTE	GetLayer() const { return m_layer; };
	void	SetOffset(float val) { m_offset = val; };
	float	GetOffset() const { return m_offset; };


	// used for streaming I/O, hierarchy display
	void SetVisited() { m_flag |= STREAM_MASK; };
	int  IsVisited() const { return m_flag & STREAM_MASK; };
	void ResetVisited() { m_flag = m_flag & RESET_STREAM_MASK; };

	// avoid polygon duplicates
	void SetHexagonType1() { m_flag |= HEXAGONE_MASK1; };
	void ResetHexagonType1() { m_flag = m_flag & RESET_HEXAGONE_MASK1; };
	int  IsHexagonType1() const { return m_flag & HEXAGONE_MASK1; };
	void SetTriangleType1() { m_flag |= TRIANGLE_MASK1; };
	void ResetTriangleType1() { m_flag = m_flag & RESET_TRIANGLE_MASK1; };
	int  IsTriangleType1() const { return m_flag & TRIANGLE_MASK1; };
	void SetTriangleType2() { m_flag |= TRIANGLE_MASK2; };
	void ResetTriangleType2() { m_flag = m_flag & RESET_TRIANGLE_MASK2; };
	int  IsTriangleType2() const { return m_flag & TRIANGLE_MASK2; };

	int  GetCount() const { return (int) m_count & BITS_COUNT; };
	void IncCount() { ++m_count; };
	void DecCount() { --m_count; };
	void ResetCount() { m_count = m_count & RESET_COUNT; };
	void SetCount(int count) { m_count = (m_count & RESET_COUNT) + (count & BITS_COUNT); };
	void SetDir(int dir) { m_count |= dir; };
	int  GetDir() const { return m_count & BITS_FACE; };
	void ResetDir() { m_count = m_count & RESET_FACE; };

	CSurfNodeT *GetPrevNode1(int level) const;
	CSurfNodeT *GetPrevNode2(int level) const;
	CSurfNodeT *GetNextNode1(int level) const;
	CSurfNodeT *GetNextNode2(int level) const;

	void SetPrevNode1(CSurfNodeT *n, int level);
	void SetPrevNode2(CSurfNodeT *n, int level);
	void SetNextNode1(CSurfNodeT *n, int level);
	void SetNextNode2(CSurfNodeT *n, int level);

	void SetPrevNodeVW(CSurfNodeT *pSN, int dir);
	void SetNextNodeVW(CSurfNodeT *pSN, int dir);
	void SetPrevNodeWU(CSurfNodeT *pSN, int dir);
	void SetNextNodeWU(CSurfNodeT *pSN, int dir);
	void SetPrevNodeUV(CSurfNodeT *pSN, int dir);
	void SetNextNodeUV(CSurfNodeT *pSN, int dir);

	void Remove();
	void Remove(CSurfNodeT * pSN);

	CSurfNodeT *GetUpList() const { return m_upList; };
	CSurfNodeT *GetDownList() const { return m_downList; };

	void SetUpList(CSurfNodeT *n) { m_upList = n; };
	void SetDownList(CSurfNodeT *n) { m_downList = n; };

	void SetLevel(int level) { m_level = level; };
	int  GetLevel() const { return m_level; };

	void SetIndex(int index) { m_index = index; }
	int  GetIndex() const { return m_index; };

	// Member variables
protected:
	int				m_index;  					// for storing the index of the vertex (IndexedFaceSet)
	BYTE			m_flag; 					// 00010000 set for type 1 hexagones
	// 00001000 set for type 1 triangles
	// 00000100 set for type 2 triangles
	// 00000001 set when streamed
	BYTE			m_count;					// incremented for each new polygon it generates = 4
	CIPointUVW		m_pos;						// position of the slicing planes
	float			m_offset;					// difference between the pos of the vertex and the position of the grid
	BYTE			m_level;
	BYTE			m_layer;

	CSurfNodeT	* m_hierarchyNode;	// pointer to an equivalent node in the
	// hierarchical representation

	CSurfNodeT	* m_upList;				// link to the previous node in the list
	CSurfNodeT	* m_downList;			// link to the next node in the list
	CSurfNodeT	**m_prevNode1;			// previous node in scan direction 1
	CSurfNodeT	**m_prevNode2;			// previous node in scan direction 2
	CSurfNodeT	**m_nextNode1;			// next node in scan direction 1
	CSurfNodeT	**m_nextNode2;			// next node in scan direction 2
};

inline void CSurfNodeT::SetPos(const CIPointUVW &pos)
{
	m_pos.u = pos.u; m_pos.v = pos.v; m_pos.w = pos.w;
}

inline CSurfNodeT * CSurfNodeT::GetPrevNode1(int level) const
{
	ASSERT(level >= 0);  return m_prevNode1[level];
}

inline CSurfNodeT * CSurfNodeT::GetPrevNode2(int level) const
{
	ASSERT(level >= 0);  return m_prevNode2[level];
}

inline CSurfNodeT * CSurfNodeT::GetNextNode1(int level) const
{
	ASSERT(level >= 0);  return m_nextNode1[level];
}

inline CSurfNodeT * CSurfNodeT::GetNextNode2(int level) const
{
	ASSERT(level >= 0);  return m_nextNode2[level];
}

inline void CSurfNodeT::SetPrevNode1(CSurfNodeT *n, int level)
{
	ASSERT(level >= 0 && n != NULL);
	if (m_prevNode1[level] != NULL && m_prevNode1[level] != n)
		throw CMGDataException(CMGErrMsg::WRONG_CONNECTIVITY);
	m_prevNode1[level] = n;
}

inline void CSurfNodeT::SetPrevNode2(CSurfNodeT *n, int level)
{
	ASSERT(level >= 0 && n != NULL);
	if (m_prevNode2[level] != NULL && m_prevNode2[level] != n)
		throw CMGDataException(CMGErrMsg::WRONG_CONNECTIVITY);
	m_prevNode2[level] = n;
}

inline void CSurfNodeT::SetNextNode1(CSurfNodeT *n, int level)
{
	ASSERT(level >= 0 && n != NULL);
	if (m_nextNode1[level] != NULL && m_nextNode1[level] != n)
		throw CMGDataException(CMGErrMsg::WRONG_CONNECTIVITY);
	m_nextNode1[level] = n;
}

inline void CSurfNodeT::SetNextNode2(CSurfNodeT *n, int level)
{
	ASSERT(level >= 0 && n != NULL);
	if (m_nextNode2[level] != NULL && m_nextNode2[level] != n)
		throw CMGDataException(CMGErrMsg::WRONG_CONNECTIVITY);
	m_nextNode2[level] = n;
}

inline void CSurfNodeT::SetPrevNodeVW(CSurfNodeT *pSN, int dir)
{
	if (dir == LEFT_FACE || dir == TOP_FACE)
		SetPrevNode1(pSN, pSN->GetLevel());
	else
		SetPrevNode2(pSN, pSN->GetLevel());
}

inline void CSurfNodeT::SetNextNodeVW(CSurfNodeT *pSN, int dir)
{
	if (dir == LEFT_FACE || dir == TOP_FACE)
		SetNextNode1(pSN, pSN->GetLevel());
	else
		SetNextNode2(pSN, pSN->GetLevel());
}

inline void CSurfNodeT::SetPrevNodeWU(CSurfNodeT *pSN, int dir)
{
	if (dir == BOTTOM_FACE || dir == FRONT_FACE)
		SetPrevNode1(pSN, pSN->GetLevel());
	else
		SetPrevNode2(pSN, pSN->GetLevel());
}

inline void CSurfNodeT::SetNextNodeWU(CSurfNodeT *pSN, int dir)
{
	if (dir == BOTTOM_FACE || dir == FRONT_FACE)
		SetNextNode1(pSN, pSN->GetLevel());
	else
		SetNextNode2(pSN, pSN->GetLevel());
}

inline void CSurfNodeT::SetPrevNodeUV(CSurfNodeT *pSN, int dir)
{
	if (dir == BACK_FACE || dir == RIGHT_FACE)
		SetPrevNode1(pSN, pSN->GetLevel());
	else
		SetPrevNode2(pSN, pSN->GetLevel());
}

inline void CSurfNodeT::SetNextNodeUV(CSurfNodeT *pSN, int dir)
{
	if (dir == BACK_FACE || dir == RIGHT_FACE)
		SetNextNode1(pSN, pSN->GetLevel());
	else
		SetNextNode2(pSN, pSN->GetLevel());
}

////////////////////////////////////////////////////////////////////////
//class CSurfNodeList
////////////////////////////////////////////////////////////////////////

class CSurfNodeList
{
	// construct, destruct
public:
	CSurfNodeList() {	m_pList = NULL; };
	~CSurfNodeList() { RemoveAll(); };

	// Member functions
public:
	void RemoveAll();

	CSurfNodeT *AddNode(const CIPointUVW &pos, int dir, int nrLevels, int index, int level = 0, BYTE layer = 0, float offset = 0.0f);
	CSurfNodeT *GetNode(const CIPointUVW &pos, int dir);
	CSurfNodeT *GetFirstNode() { return m_pList; };
	void		SetFirstNode(CSurfNodeT *pSN) { m_pList = pSN; };

	// Member variables
protected:
	CSurfNodeT *m_pList;  // list of elements of type CSurfNodeT *
};

////////////////////////////////////////////////////////////////////////
//class CSurfPolyT
////////////////////////////////////////////////////////////////////////

class CSurfPolyT
{
	// construct, destruct
public:
	CSurfPolyT() { m_n1 = m_n2 = m_n3 = NULL; };

	// Member functions
public:
	CSurfNodeT *GetRef1() { return m_n1; };
	CSurfNodeT *GetRef2() { return m_n2; };
	CSurfNodeT *GetRef3() { return m_n3; };

	void SetPoint1(CSurfNodeT *n) { m_n1 = n; };
	void SetPoint2(CSurfNodeT *n) { m_n2 = n; };
	void SetPoint3(CSurfNodeT *n) { m_n3 = n; };

	const CSurfPolyT &operator=(const CSurfPolyT &poly);

	// Member variables
protected:
	CSurfNodeT *m_n1;
	CSurfNodeT *m_n2;
	CSurfNodeT *m_n3;
};

inline const CSurfPolyT &CSurfPolyT::operator=(const CSurfPolyT &poly)
{
	m_n1 = poly.m_n1; m_n2 = poly.m_n2; m_n3 = poly.m_n3;

	return poly;
}

////////////////////////////////////////////////////////////////////////
//class CMGMesh
////////////////////////////////////////////////////////////////////////

typedef CDynaArray<CSurfPolyT, CSurfPolyT&>	DynaSurfPolyT;

class CMGMesh  
{
public:
	CMGMesh();
	virtual ~CMGMesh();

	enum {
		NO_SPLIT = 1,
		UNIFORM = 2,
		NON_UNIFORM = 3
	};

	enum {
		CHANGED_CONNECTIVITY = 1,
		CHANGED_VERTICES = 2,
		CHANGED_GRID = 4,
		CHANGED_GRID_CORNERS = 8
	};

protected:
	CMGGrid		  * m_grid;									// the reference grid

	bool			m_hasGridInfo;							// is the ref grid encoded in the stream?
	bool			m_hasConnectivityInfo;					// is the connectivity encoded in the stream?
	bool			m_hasRefineInfo;						// are the offsets encoded in the stream?
	bool			m_hasRepositionInfo;					// are the reposition bits encoded in the stream?
	bool			m_bSingleLayered;                       // if true, at least one grid dimension is one
	int				m_meshType;                             // if the mesh consists only of quadrilaterals, triangles, hexagons or it is generic.
	bool			m_bSameConnectivity;                    // it is a quadric mesh with all the connectivities in the same reference plane
	bool			m_bAutoResolution;                      // it is a quadric mesh with homogeneous subdivision-based multi-resolution 
	int				m_offsetAmp;                            // the maximum offset in case of quadrilateral meshes with one layer of grid points
	int				m_level;								// used when adding the polygons to the right level
	int				m_maxNrLevels;							// max number of resolution levels
	CIPointUVW	    m_nrSlices;								// number of slices of the reference grid in u, v, w directions
	CIPointUVW	    m_maxNrROIs;							// number of ROIs 
	BYTE			m_nrLayers;								// number of layers
	BYTE			m_pendingLevel;							// what resolution level?
	BYTE			m_updatesMode;							// how to update data?
	BYTE			m_pendingUpdates;						// what updates are pending?


	int			  * m_nrVertices;                           // nr of vertices corresponding to each resolution level
	int				m_nrVerticesROI;                        // track the number of vertices per ROI for coding/decoding only
	int				m_nrTriangles;                          // nr of triangles generated

	// VRML access
	int				m_countLink;
	int				m_countOffset;
	int				m_nrVertexLinks;
	int				m_nrVertexOffsets;
	const int  	  * m_vertexLink;
	float		  * m_vertexOffset;
	int				m_totalNrVertices;              // total number of vertices from the buffer
	CFPoint3DT	  * m_vertices;                     // buffer contining the mesh vertices

	int				m_nrRefBitsVert;  		        // nr of bits to store the vertex position offset
	float			m_refGridCR;					// compression ratio for storing the ref grid
	float			m_refGridMAXAD;					// the maximum absolute error when storing the ref grid
	int				m_refGridQuant;				    // quantisation bits for storing the ref grid
	bool			m_bInterleaveRefine;            // specify the vertices offsets for each level: true or false

	BYTE			m_selDir;						// direction of the selected vertex
	CIPointUVW		m_selPos;						// position of the selected grid point

	// coding / decoding related
	CPtrList    *** m_startVertsROI;                        // array of lists of vertex start positions of ROI patches
	CPtrList		m_nodeList;								// a FIFO buffer of vertices to be stored
	CIPointUVW		m_posVertMin;							// min values of the vertices mapped inside the box
	CIPointUVW		m_posVertMax;							// max values of the vertices mapped inside the box
	CIPointUVW		m_defaultSizeROI;						// the size of the ROI with respect to a given resolution level
	CIPointUVW		m_defaultNoROIs;						// the number of ROIs at the highest resolution level
	Cube			m_localROI;                             // position and size of the ROI with respect to a given resolution level
	Cube			m_globalROI;                            // position and size of the ROI with respect to the last resolution level

	CMGGridDecoder<float>	* m_pMGIN;				        // only for the reference grid
	CMGGridCoder<float>		* m_pMGOUT;				        // only for the reference grid
	CMGStreamParser         * m_pParser;                    // I/O parser of the stream
	MGStreamDescr           * m_pStreamDescr;               // keeps MeshGrid stream information

	// vertex and polygon lists
	CSurfNodeList           **m_nodeStack;					// buffer for storing the Node connectivity
	CArray<DynaSurfPolyT, DynaSurfPolyT&>	m_polyLevels;	// array of 4 pointers of type CSurfNodeT

public:
	void	SetGrid(CMGGrid *pGrid) { m_grid = pGrid; };
	int     GetMeshType() const { return m_meshType; };
	void    SetMeshType(int type) { m_meshType = type; };
	int     GetCyclicMode() const { return m_grid->GetCyclicMode(); };
	void    SetCyclicMode(int mode) { m_grid->SetCyclicMode(mode); };
	bool	IsCyclicU() const { return m_grid->IsCyclicU(); };
	bool	IsCyclicV() const { return m_grid->IsCyclicV(); };
	bool	IsCyclicUV() const { return m_grid->IsCyclicUV(); };
	bool	IsFoldedSingle() const { return m_grid->IsFoldedSingle(); };
	bool	IsFoldedBoth() const { return m_grid->IsFoldedBoth(); };
	bool    IsSameConnectivity() const { return m_bSameConnectivity; };
	void    SetSameConnectivity(bool flag) { m_bSameConnectivity = flag; };
	bool    IsAutoResolution() const { return m_bAutoResolution; };
	void    SetAutoResolution(bool flag) { m_bAutoResolution = flag; };
	int     GetOffsetAmplitude() const { return m_offsetAmp; };
	void    SetOffsetAmplitude(int offsetAmp) { m_offsetAmp = offsetAmp; }
	bool    IsSingleLayerGrid() const { return m_bSingleLayered; };
	void    SetSingleLayerGrid(bool flag) { m_bSingleLayered = flag; };

	bool	Init(CMGGrid *grid, CFPoint3DT *vertices, CString & error);
	bool	Init(CMGGrid *grid, CMGStreamParser *parser, CFPoint3DT *vertices, CString & error);
	bool	Load(bool hasConnectivityInfo, bool hasRefineInfo, bool hasRepositionInfo, bool dump, CString & error);
	bool	LoadCompressed(CString & error, int streamSize, bool dump);
	bool    Save(CFile *file, const CIPointUVW &nrROIs, bool hasHeader, bool hasConnectivityInfo, bool hasRefineInfo, bool hasRepositionInfo,
				   bool hasGridInfo, bool encodeMultiLayer, int nrRefinementBits, bool interleaveRefine, int nrQuabtizationBits, int filterType,
				   float fMAXAD, CString &error);
	void	ResetConnectivity();
	void    RecoverEncodingParameters(const int *vertexLink, float *vertexOffset, int nrVertexLinks, int nrVertexOffsets,
									  bool &hasConnectivityInfo, bool &hasRefineInfo, bool &hasRepositionInfo);

	MGStreamDescr *GetStreamDescription() { return m_pStreamDescr; };
	CSurfNodeList	**GetNodeStack() { return m_nodeStack; };

	void    RetrieveMeshLevelHeaders(int level);
	bool    DecodeMeshConnectivity(int roiIdx, int level);
	bool    DecodeMeshRepositionBit(int roiIdx, int level);
	bool    DecodeMeshRefineBits(int roiIdx, int level);

	void    ComputeSmoothNormals(CFPoint3DT *normals, int level);

	void    UpdateVertexCoordSingleLayer(int level, CSurfNodeT *pSN);
	void	UpdateVertexCoord(CSurfNodeT *pSN);
	void	UpdateVertexCoord(int level, CSurfNodeT *pSN);
	void	UpdateMGfromRG(int level);
	void	UpdateMGfromRG();
	void	BuildSmoothSurface(int level);
	void	BuildQuadrilateralSurface(int level, BYTE layer);
	void    ClearSurfaceBuffer(int level) { m_polyLevels[level].RemoveAll(); };

	bool	IsEmpty() const { return (m_nodeStack == NULL); };
	const   CIPointUVW & GetNrSlices() const { return m_nrSlices; };
	const   CIPointUVW & GetNrSlices(int level) const { return m_grid->GetNrSlices(level); };

	int		GetNrSlicesU() const { return m_grid->GetNrSlicesU(); };
	int		GetNrSlicesV() const { return m_grid->GetNrSlicesV(); };
	int		GetNrSlicesW() const { return m_grid->GetNrSlicesW(); };

	int		GetNrSlicesU(int level) const { return m_grid->GetNrSlicesU(level); };
	int		GetNrSlicesV(int level) const { return m_grid->GetNrSlicesV(level); };
	int		GetNrSlicesW(int level) const { return m_grid->GetNrSlicesW(level); };

	BYTE	GetNrLayers() const { return m_nrLayers; };

	int		GetPrevSliceU(int u, BYTE level) const
	{ return m_grid->GetPrevSliceU(u, level, IsCyclicU()); };
	int		GetNextSliceU(int u, BYTE level) const
	{ return m_grid->GetNextSliceU(u, level, IsCyclicU()); };
	int		GetPrevSliceV(int v, BYTE level) const
	{ return m_grid->GetPrevSliceV(v, level, IsCyclicV()); };
	int		GetNextSliceV(int v, BYTE level) const
	{ return m_grid->GetNextSliceV(v, level, IsCyclicV()); };
	int		GetPrevSliceW(int w, BYTE level) const
	{ return m_grid->GetPrevSliceW(w, level, false); };
	int		GetNextSliceW(int w, BYTE level) const
	{ return m_grid->GetNextSliceW(w, level, false); };

	int		GetMaxNrLevels() const { return m_maxNrLevels; };
	int     GetTotalNrVertices() { return m_totalNrVertices; };
	int		GetNrVertices(int level);
	int		ComputeNrVertices(int level, BYTE layer);
	void	Update(float *v, int nrPts, CString &error);
	void	GetVertexOffset(float *offset);
	void    GetVertexNormals(CFPoint3DT *normals, int level);
	int		GetNrPolygons(int level) { return m_polyLevels[level].GetSize(); };
	void    GetTriangles(int *f, int level);
	DynaSurfPolyT &GetTrianglesBuffer(int level);
	void    GetTextureIndices(int *ti, int level);
	void    GetTextureCoordinates(float *tc);

	int		GetNrRefBitsVert() const { return m_nrRefBitsVert; };
	void	SetNrRefBitsVert(int nrBits) { ASSERT(nrBits >= 0); m_nrRefBitsVert = nrBits; };
	float	GetRefGridCR() const { return m_refGridCR; };
	void	SetRefGridCR(float cr) { ASSERT(cr >= 0.0f); m_refGridCR = cr; };
	int		GetRefGridQuant() const { return m_refGridQuant; };
	void	SetRefGridQuant(int quant) { ASSERT(quant >= 1); m_refGridQuant = quant; };
	bool    InterleaveRefine() const { return m_bInterleaveRefine; };
	void    SetInterleaveRefine(bool flag) { m_bInterleaveRefine = flag; };

	// MAXAD related
	void ComputeOptimalTruncation(double MAXAD, int nFilterType);
	void ComputeTruncationPointsList(int nFilterType, CTruncations* arrTruncationPointsList[], CDoubleList &arrLambdas);
	void SearchTruncationThresholds(CMatrix<CODEC_TYPE> orgCoeff[], CMatrix<int> orgCoeffQ[], CMatrix<CODEC_TYPE> modCoeff[], CMatrix<int> modCoeffQ[], CMatrix<CODEC_TYPE> modCoeffW[], double MAXAD, double scaleMAXAD, int nFilterType, CTruncations* arrTruncationPointsList[], CDoubleList &arrLambdas);

	double	GetDDistortion(int level, int maxLevels, int threshold, int scale, int filterType) const;
	void	ComputeFeasibleTruncationPoints(CTruncationPoints &arrTruncationPoints);
	void	FindTruncationPoints(CMatrix<CODEC_TYPE> orgCoeff[], CMatrix<int> orgCoeffQ[], CMatrix<CODEC_TYPE> modCoeff[], CMatrix<int> modCoeffQ[], CMatrix<CODEC_TYPE> modCoeffW[], double MAXAD, double scaleMAXAD, CTruncations* arrTruncationPointsList[], CDoubleList &arrLambdas);
	void	ExtraSearch(CMatrix<CODEC_TYPE> orgCoeff[], CMatrix<int> orgCoeffQ[], CMatrix<CODEC_TYPE> modCoeff[], CMatrix<int> modCoeffQ[], CMatrix<CODEC_TYPE> modCoeffW[], double MAXAD, double scaleMAXAD, CTruncations* arrTruncationPointsList[]);
	int		GetStopThreshold(CTruncations &arrTruncations, int gridLevelThreshold, int deltaDomain = 0);
	void	InversePreProcess(CMatrix<int> matIn[], CMatrix<CODEC_TYPE> matOut[]);
	void	StandardDeviation(CMatrix<CODEC_TYPE> mats1[], CMatrix<CODEC_TYPE> mats2[], double std[], double MSE[], double mmm[]);
	void	GetRealMAXAD(CMatrix<CODEC_TYPE> orgCoeff[], CMatrix<int> orgCoeffQ[], CMatrix<CODEC_TYPE> modCoeff[], CMatrix<int> modCoeffQ[], CMatrix<CODEC_TYPE> modCoeffW[], CTruncations* arrTruncationPointsList[], double std[], double MSE[], double realMAXAD[], int compLevel = -1);
	void	GetLength(CTruncations* arrTruncationPointsList[], int length[]);
	void	EstimateError(CMatrix<int> &matData, const Cube &cube, int stopThreshold);

	static void ComputeROISize(const CIPointUVW &nrSlices, CIPointUVW &szROI, CIPointUVW &maxSzROI, CIPointUVW &nrROIs);
	static void MapIndexToPosition(int index, const CIPointUVW &count, Cube &roi);
	static int  GetNrROIs(const CIPointUVW &nrSlices, const CIPointUVW &szROI);
	static int  GetNrROIs(const CIPointUVW &nrSlices, const CIPointUVW &szROI, CIPointUVW &nrROIs);
	static bool IsLastROI(int pos, int &sz, int nrSlices);
	static void InitROI(int localPos, int &localSz, int &globalPos, int &globalSz,
		int sz0, int nrSlices0, int nrSlices, int level);
	static void InitFirstROI(Cube &localROI, Cube &globalROI,
		const CIPointUVW &nrSlices0, const CIPointUVW &nrSlices,
		const CIPointUVW &szROI, const CIPointUVW &levelDiff);
	static bool GetNextROI(Cube &localROI, Cube &globalROI,
		const CIPointUVW &nrSlices0, const CIPointUVW &nrSlices,
		const CIPointUVW &szROI, const CIPointUVW &levelDiff);
	static void InitFirstROI(Cube &roi, const CIPointUVW &nrSlices, const CIPointUVW &szROI);
	static bool GetNextROI(Cube &roi, const CIPointUVW &nrSlices, const CIPointUVW &szROI);

protected:
	// binary stream coding / decoding 
	void CheckROISize(CIPointUVW *szROI);

	void    CreateParser(CFile *file, CBitStream::BITSTREAM_OPERATIONS op);
	void    CreateAndStartEncoder(int quantizationBits, int filterType);
	void    StopEncoder();
	void    StartDecoder();
	void    StopDecoder();

	void    InitExchangeObjectMeshBinary(const CIPointUVW &nrROIs, bool hasConnectivityInfo,
										   bool hasRefineInfo, bool hasRepositionInfo, bool hasGridInfo,
										   int refinmentBits, bool interleaveRefine, bool encodeMultiLayer);
	void    InitExchangeObjectMesh(bool hasConnectivityInfo, bool hasRefineInfo, bool hasRepositionInfo, bool interleaveRefine);

	void    UpdateGridCorners();

	// VRML stream coding
	bool    Export(CStdioFile &file, bool bCompact,
					 bool hasConnectivityInfo, bool hasRefineInfo, bool hasRepositionInfo, bool interleaveRefine);
	bool    Export(CStdioFile &file, int level, bool bCompact,
					 bool hasConnectivityInfo, bool hasRefineInfo, bool hasRepositionInfo, bool interleaveRefine);
	void    ExportMeshConnectivity(int level, CStdioFile &file);
	void    ExportMeshRepositionBits(int level, CStdioFile &file);
	void    ExportMeshRefineBits(int level, CStdioFile &file);
	void    ExportMeshGridFields(CStdioFile &file);
	void    ExportMeshParameters(CStdioFile &file);
	void    ExportAbsoluteOffset(CStdioFile &file);
	void    ExportRefineParameters(CStdioFile &file);
	void    ExportField(unsigned int number, CStdioFile &file);
	void    ExportByteField(BYTE number, CStdioFile &file);
	int     ExportRefineMode(int level, CStdioFile &file);
	void    SaveMeshConnectivityROI(bool &bFinish, CStdioFile &file, int level, int roiIdx);
	void    SaveMeshRefine(int level, int roiIdx, CStdioFile &file);
	void    SaveMeshReposition(int level, int roiIdx, CStdioFile &file);
	void    StreamNodeBox(int level, CSurfNodeT *pSNStart, CStdioFile &file);
	void    StreamStartNode(int level, CSurfNodeT *pSNStart, CStdioFile &file);
	void    StreamAbsoluteNode(const CIPointUVW &pos3D, BYTE dir, CStdioFile &file);
	void    StreamVerticesRefineBits(int level, CSurfNodeT *pSNStart, CStdioFile &file);
	void    StreamVerticesRepositionBit(int level, CSurfNodeT *pSNStart, CStdioFile &file);
	void    WriteConnectivitySymbol(int link1, int link2, CStdioFile &file);
	void    WriteOffset(float offset, CStdioFile &file);

	// dump the MeshGrid information
	void    Dump(CStdioFile &file);
	void    DumpDecoderSpecificInfo(CStdioFile &file);
	void    DumpConnectivityWireframe(CStdioFile &file);
	void    DumpTriangles(CStdioFile &file);
	void    DumpGrid(CStdioFile &file) { m_grid->Dump(file); };

	// VRML stream decoding
	void    InitialiseDecoderData(bool hasConnectivityInfo, bool hasRefineInfo, bool hasRepositionInfo);
	void    RecoverMeshParameters(bool &hasConnectivityInfo);
	int     RecoverAbsoluteOffset();
	void    RecoverRefineParameters(bool &hasRefineInfo, bool &hasRepositionInfo);
	void    InitialiseLevelData(int level);
	int     RecoverField();
	int     RecoverByteField();
	int     RecoverRepositionMode();
	int     RecoverNumberVertices();
	CSurfNodeT *RecoverNextStartingPoint(int level);
	void    RecoverMeshConnectivity(int level, bool fullRefine);
	void	RecoverMeshConnectivity(int level, CSurfNodeT *pSNStart, bool bUpdate);
	void	RecoverVerticesRepositionBit(int level);
	void	RecoverRepositionBits(int level, CSurfNodeT *pSNStart);
	int		GetNextMeshFlag();
	int		GetNextRefineFlag();
	int		GetNextLink();
	float	GetNextOffset();
	float	GetNextReposition();

	// Binary stream coding
	void    DoConnectivityCoding(int level, BYTE layer, int roiIdx);
	void    DoVertsReposCoding(int level, BYTE layer, int roiIdx);
	void    DoVertsRefineCoding(int level, BYTE layer, int roiIdx);

	void	StoreGridCorners(bool differential);
	void    StoreMeshGridHeader(bool keepHeader);
	void    StoreBeginFrame();
	void    WriteOutput(CFile *file);
	void    StoreMGMesh(int level, BYTE update_replace);
	void    StoreMGGrid(int level, BYTE update_replace);
	void    SaveMeshConnectivityROI(bool &bFinish, int level, BYTE layer, int roiIdx);
	void    SaveMeshRefineROI(int level, BYTE layer, int roiIdx);
	void    SaveMeshReposROI(int level, BYTE layer, int roiIdx);
	void    CleanUpVerticesList();
	void    CleanUpVerticesList(int level);
	void    CleanUpVerticesList(int level, BYTE layer);
	void    CleanUpVerticesList(int level, BYTE layer, int boxIndex);

	void    OnBeginMeshCodingLevel(int level);
	void    OnBeginMeshCodingLayer(int level, BYTE layer);
	void    OnBeginMeshExportLevel(int level);
	void    OnBeginMeshCodingROI(int level, BYTE layer, int roiIdx);
	void    OnBeginMeshCodingConnectivityROI(BYTE layer);
	void    OnBeginMeshCodingVertsReposROI(BYTE layer);
	void    OnBeginMeshCodingVertsRefineROI(BYTE layer);
	void    OnEndMeshCodingLevel(int level, BYTE update_replace);
	void    OnEndMeshCodingLayer(int level, BYTE layer);
	void    OnEndMeshCodingROI(bool bEmpty, int level, BYTE layer, int roiIdx);
	void    OnEndMeshCodingConnectivityROI(int level, BYTE layer, int roiIdx);
	void    OnEndMeshCodingVertsReposROI(int level, BYTE layer, int roiIdx);
	void    OnEndMeshCodingVertsRefineROI(int level, BYTE layer, int roiIdx);

	void    GetNextStartingPoint(int level, BYTE layer);
	bool    TrackAppropriateEdge(CSurfNodeT *pSN, bool bLink1, LPCTSTR name, int level, BYTE layer);
	bool    IsROIStreamed(int level, BYTE layer);
	bool    IsROIEmpty(int level, BYTE layer);
	bool    IsVertexInROI(CSurfNodeT *pSN, int level, BYTE layer);
	int     GetNrVerticesROI(int level, BYTE layer);
	void	TraceNonVisitedVertices(int level, BYTE layer);

	CSurfNodeT *GetFirstPrevious(CSurfNodeT *pSN, bool &bLink1, int level, BYTE layer);
	CSurfNodeT *CheckStartingPoint(int level, BYTE layer);

	void    StreamAbsoluteNode(int level, BYTE layer, const CIPointUVW &pos3D, BYTE dir);
	void    StreamNodeBox(int level, BYTE layer, CSurfNodeT *pSNStart);
	void    StreamStartNode(int level, BYTE layer, CSurfNodeT *pSNStart);
	void    StreamVerticesRepositionBit(int level, BYTE layer, CSurfNodeT *pSNStart);
	void    StreamVerticesRefineBits(int level, int mask, int maxVal, BYTE layer, CSurfNodeT *pSNStart);

	// read / write binary symbols
	void    WriteConnectivitySymbol(BYTE symbol);
	void    ReadConnectivitySymbol(BYTE &symbol);
	void    WriteRefineSymbol(BYTE symbol);
	void    ReadRefineSymbol(BYTE &symbol);
	void    WriteRepositionSymbol(BYTE symbol, bool bSpecialCase);
	void    ReadRepositionSymbol(BYTE &symbol, bool bSpecialCase);

	// Binary stream decoding
	void    ParseStream(int streamSize);
	int     ParseStreamSize(int &counterLength);
	int     ParseMeshGridPart(int levelDisplay, BYTE &tag);
	void	DoPendingUpdates();
	int     ParseTagAndSize(BYTE &tag, int &size);
	void    ParseLevelDescription(int &level, BYTE &update_replace);
	void    ParseLayerDescription(int &level, BYTE &update_replace, CIntArray &layers);
	int   * MakeIndices(int number);
	void    DefaultMeshDescriptor();
	void	CopyMeshDescriptor(int level, int update_replace, BYTE layer0, BYTE layer1);
	void    MeshDescriptor(int level, int update_replace, BYTE layer);
	void    GridDescriptor(int level);
	void    MeshConnectivityROI(int, int, BYTE) {};
	void    MeshConnectivity(int level, int update_replace, BYTE layer);
	void    VertexRepositionROI(int, int, BYTE) {};
	void    VertexReposition(int level, int update_replace, BYTE layer);
	void    VertexRefineROI(int, int, BYTE) {};
	void    VertexRefine(int level, int update_replace, BYTE layer);
	void    GridCoefficientsROI(int, BYTE) {};
	void    GridCoefficients(int levelG, BYTE updatesMode);
	void	ProcessGridCorners();

	void    BuildDefaultCW();
	void    BuildDefaultCWLevels(BYTE layer);
	void    DefaultMeshConnectivity(int level, BYTE layer);                                   // build the connectivity for auto resolution
	void    BuildMeshUV(int u1, int u2, int v1, int v2, BYTE layer);
	void    BuildMeshUV(int u1, int u2, int v1, int v2, bool bUpdate);
	bool    DefineConectivityUV(BYTE layer);
	bool    DefineConectivityUV(bool bUpdate);
	void    SubdividePatch(CSurfNodeT *pSN, CSurfNodeT *pN1, CSurfNodeT *pN2, CSurfNodeT *pN1N2, int level, BYTE layer);
	void    GetSplitPos(const CIPointUVW &pos1, const CIPointUVW &pos2, CIPointUVW &pos, int dirSplit, int level);
	void    GetSplitPos(const CIPointUVW &pos1, const CIPointUVW &pos2, CIPointUVW &pos11, CIPointUVW &pos22, int dirSplit, int level);
	void    UpdateLink1(CSurfNodeT *pSN1, CSurfNodeT *pSN2, int level);
	void    UpdateLink2(CSurfNodeT *pSN1, CSurfNodeT *pSN2, int level);
	void    SplitLink1(CSurfNodeT *pSN1, CSurfNodeT *pSN2, CSurfNodeT *&pNS, int dirSplit, int level, BYTE layer);
	void    SplitLink2(CSurfNodeT *pSN1, CSurfNodeT *pSN2, CSurfNodeT *&pNS, int dirSplit, int level, BYTE layer);
	void    SplitLink1(CSurfNodeT *pSN1, CSurfNodeT *pSN2, CSurfNodeT *&pNS1, CSurfNodeT *&pNS2, int dirSplit, int level, BYTE layer);
	void    SplitLink2(CSurfNodeT *pSN1, CSurfNodeT *pSN2, CSurfNodeT *&pNS1, CSurfNodeT *&pNS2, int dirSplit, int level, BYTE layer);
	int     IsSpecialSplit(int nextLevel, int dir, const CIPointUVW &pos1, const CIPointUVW &pos2, int &dirSplit);

	void    InitMeshLevelHeaders(int level, BYTE layer);
	void    RetrieveMeshConnectivityPart(int level, BYTE layer);
	void    RetrieveVertexRepositionPart(int level, BYTE layer);
	void    RetrieveVertexRefinePart(int level, BYTE layer);
	bool    RetrieveMeshConnectivityROI(int level, BYTE layer, int roiIdx);
	bool    RetrieveMeshRepositionBitROI(int level, BYTE layer, int roiIdx);
	bool    RetrieveMeshRefineBitsROI(int level, BYTE layer, int roiIdx);

	void    RecoverAbsoluteNode(int level, BYTE layer, CIPointUVW &pos3D, BYTE &dir);
	void    RecoverNodeBox(int level, BYTE layer, CSurfNodeT *pSNStart);
	void    RecoverStartNode(int level, BYTE layer, CSurfNodeT *&pSNStart);
	void    RecoverVerticesRepositionBit(int level, BYTE layer, CSurfNodeT *pSNStart, int reposMode);
	void    RecoverVerticesRefineBits(int level, int t, float maxVal, BYTE layer, CSurfNodeT *pSNStart);
	void    AddPosOffsetBit(int level, CSurfNodeT *pSN, float val);

	// other functions
	void    ResetVisitedVertices();
	CSurfNodeT * ResetVisitedVertices(int level);
	void    ResetVisitedVerticesGrid(int level, BYTE layer);
	void    ResetVisitedVerticesBox(int level, BYTE layer);
	void    AddStartNodeBox(CSurfNodeT *pSNStart, bool bCount = false);
	void    AddNodesToList(CSurfNodeT *pSNNext1, CSurfNodeT *pSNNext2, int level, bool bCount);
	void    AddNodesToListBox(CSurfNodeT *pSNNext1, CSurfNodeT *pSNNext2, int level, BYTE layer, bool bCount);
	float   GetLevelOffset(int dir, const CIPointUVW &pos, float offset, int level);
	BYTE    VertexRepositionBit(int nextLevel, CSurfNodeT *pSN, bool &bSpecialCase);
	bool    IsSpecialCase(int nextLevel, CSurfNodeT *pSN);
	void	FindPosNodeLocal(CIPointUVW &pos, BYTE dir, int level, BYTE layer);
	void	FindPosNodeGlobal(CIPointUVW &pos, BYTE dir, int level);

	void    DeleteContent();

	// commun functions
	bool    InROI(CSurfNodeT *pSN, int level, BYTE layer);
	bool    InROI(CSurfNodeT *pSN, int level);
	void    CacheStartPoint(CSurfNodeT *pSN, int level, BYTE layer, int roiIdx);

	void    InitROI(int level, BYTE layer, int roiIdx);
	void    ComputeROIVertices(int level);
	void    CheckValidVerticesNumber(int nrROIs, int level, BYTE layer);

	void	RestorePosNode(CIPointUVW &pos, int level, BYTE layer);
	void	RepositionVertex(int nextLevel, CSurfNodeT *pSN);
	void	InsertNodeInList(CSurfNodeT *pSN, CSurfNodeList &nodeList);
	CSurfNodeT * RemoveNodeInList(CSurfNodeT *pSN, CSurfNodeList &nodeList);

	// vertex coordinate update from conectivity
	void    GetDirectionDifference(int dir, int dirNext, BYTE &val);
	void    UpdateBackFaceCoordinates(BYTE &dirNext, int flag, bool &dirSwitch, CIPointUVW &pos, int level);
	void    UpdateFrontFaceCoordinates(BYTE &dirNext, int flag, bool &dirSwitch, CIPointUVW &pos, int level);
	void    UpdateLeftFaceCoordinates(BYTE &dirNext, int flag, bool &dirSwitch, CIPointUVW &pos, int level);
	void    UpdateRightFaceCoordinates(BYTE &dirNext, int flag, bool &dirSwitch, CIPointUVW &pos, int level);
	void    UpdateBottomFaceCoordinates(BYTE &dirNext, int flag, bool &dirSwitch, CIPointUVW &pos, int level);
	void    UpdateTopFaceCoordinates(BYTE &dirNext, int flag, bool &dirSwitch, CIPointUVW &pos, int level);
	void    UpdateCoordinateFromConnectivity(int dir, BYTE &dirNext, int flag, bool &dirSwitch, CIPointUVW &pos, int level);

	// Triangulation procedures
	void	StoreTriangle(CSurfNodeT *pSN1, CSurfNodeT *pSN2, CSurfNodeT *pSN3);
	void	StoreRectangle(CSurfNodeT *pSN1, CSurfNodeT *pSN2, CSurfNodeT *pSN3, CSurfNodeT *pSN4);
	void	StoreSubPentagone(CSurfNodeT *pSN1, CSurfNodeT *pSN2, CSurfNodeT *pSN3, CSurfNodeT *pSN4, CSurfNodeT *pSN5);
	void	StorePentagone(CSurfNodeT *pSN1, CSurfNodeT *pSN2, CSurfNodeT *pSN3, CSurfNodeT *pSN4, CSurfNodeT *pSN5);
	void	StoreSubHexagone2(CSurfNodeT *pSN1, CSurfNodeT *pSN2, CSurfNodeT *pSN3, CSurfNodeT *pSN4, CSurfNodeT *pSN5, CSurfNodeT *pSN6);
	void	StoreHexagone1(CSurfNodeT *pSN1, CSurfNodeT *pSN2, CSurfNodeT *pSN3, CSurfNodeT *pSN4, CSurfNodeT *pSN5, CSurfNodeT *pSN6);
	void	StoreHexagone2(CSurfNodeT *pSN1, CSurfNodeT *pSN2, CSurfNodeT *pSN3, CSurfNodeT *pSN4, CSurfNodeT *pSN5, CSurfNodeT *pSN6);
	void	StoreHexagone3(CSurfNodeT *pSN1, CSurfNodeT *pSN2, CSurfNodeT *pSN3, CSurfNodeT *pSN4, CSurfNodeT *pSN5, CSurfNodeT *pSN6);
	void	StoreSeptagone(CSurfNodeT *pSN1, CSurfNodeT *pSN2, CSurfNodeT *pSN3, CSurfNodeT *pSN4, CSurfNodeT *pSN5, CSurfNodeT *pSN6, CSurfNodeT *pSN7);
	bool	BuildSeptagones(CSurfNodeT *pSN, int level);
	bool	BuildHexagones(CSurfNodeT *pSN, int level);
	bool	BuildPentagones(CSurfNodeT *pSN, int level);
	bool	BuildRectangles(CSurfNodeT *pSN, int level);
	bool	BuildTriangles(CSurfNodeT *pSN, int level);

	void	AddPolygon(CSurfPolyT &sp) { m_polyLevels[m_level].Add(sp); };

	// locate the next/previous vertex position in the grid
	void	GetBackPos	(const CIPointUVW &pos, CFPoint3DT &p, const float &offset);
	void	GetFrontPos	(const CIPointUVW &pos, CFPoint3DT &p, const float &offset);
	void	GetLeftPos	(const CIPointUVW &pos, CFPoint3DT &p, const float &offset);
	void	GetRightPos	(const CIPointUVW &pos, CFPoint3DT &p, const float &offset);
	void	GetBottomPos(const CIPointUVW &pos, CFPoint3DT &p, const float &offset);
	void	GetTopPos		(const CIPointUVW &pos, CFPoint3DT &p, const float &offset);

	void	GetBackPos	(const CIPointUVW &pos, CFPoint3DT &p, const float &offset, int level);
	void	GetFrontPos	(const CIPointUVW &pos, CFPoint3DT &p, const float &offset, int level);
	void	GetLeftPos	(const CIPointUVW &pos, CFPoint3DT &p, const float &offset, int level);
	void	GetRightPos	(const CIPointUVW &pos, CFPoint3DT &p, const float &offset, int level);
	void	GetBottomPos(const CIPointUVW &pos, CFPoint3DT &p, const float &offset, int level);
	void	GetTopPos		(const CIPointUVW &pos, CFPoint3DT &p, const float &offset, int level);

	// normal direction computation
	void	ComputeNormal(const CFPoint3DT &l, const CFPoint3DT &r, const CFPoint3DT &d, const CFPoint3DT &u, CFPoint3DT &res, float sgn);
	void	GetHalfOrthoNormal(CFPoint3DT &n, int dir);

	// vertex store in the local memory
	CSurfNodeT *GetNode(const CIPointUVW &pos, int dir);
	CSurfNodeT *StoreVertex(CIPointUVW &pos3D, int level, BYTE layer, int dir);
	CSurfNodeT *StoreVertex(CIPointUVW &pos3D, int level, BYTE layer, int dir, float offset);
	CSurfNodeT *GetAddNode(CSurfNodeList *pNodeList, const CIPointUVW &pos, int dir, int level, BYTE layer);
	CSurfNodeT *GetAddNode(CSurfNodeList *pNodeList, const CIPointUVW &pos, int dir, int level, BYTE layer, float offset);
	CSurfNodeList *GetColumn(int x);
};

inline void CMGMesh::GetBackPos(const CIPointUVW &pos, CFPoint3DT &p, const float &offset)
{
	GridU &gridU = (*m_grid)[pos.w][pos.v];
	CFPoint3DT &p1 = gridU[pos.u];
	CFPoint3DT &p2 = gridU[pos.u - 1];

	p.x = p1.x + (p2.x - p1.x) * offset;
	p.y = p1.y + (p2.y - p1.y) * offset;
	p.z = p1.z + (p2.z - p1.z) * offset;
}

inline void CMGMesh::GetFrontPos(const CIPointUVW &pos, CFPoint3DT &p, const float &offset)
{
	GridU &gridU = (*m_grid)[pos.w][pos.v];
	CFPoint3DT &p1 = gridU[pos.u];
	CFPoint3DT &p2 = gridU[pos.u + 1];

	p.x = p1.x + (p2.x - p1.x) * offset;
	p.y = p1.y + (p2.y - p1.y) * offset;
	p.z = p1.z + (p2.z - p1.z) * offset;
}

inline void CMGMesh::GetLeftPos(const CIPointUVW &pos, CFPoint3DT &p, const float &offset)
{
	GridUV &gridUV = (*m_grid)[pos.w];
	CFPoint3DT  &p1 = gridUV[pos.v][pos.u];
	CFPoint3DT  &p2 = gridUV[pos.v - 1][pos.u];

	p.x = p1.x + (p2.x - p1.x) * offset;
	p.y = p1.y + (p2.y - p1.y) * offset;
	p.z = p1.z + (p2.z - p1.z) * offset;
}

inline void CMGMesh::GetRightPos(const CIPointUVW &pos, CFPoint3DT &p, const float &offset)
{
	GridUV &gridUV = (*m_grid)[pos.w];
	CFPoint3DT &p1 = gridUV[pos.v][pos.u];
	CFPoint3DT &p2 = gridUV[pos.v + 1][pos.u];

	p.x = p1.x + (p2.x - p1.x) * offset;
	p.y = p1.y + (p2.y - p1.y) * offset;
	p.z = p1.z + (p2.z - p1.z) * offset;
}

inline void CMGMesh::GetBottomPos(const CIPointUVW &pos, CFPoint3DT &p, const float &offset)
{
	CFPoint3DT &p1 = (*m_grid)[pos.w][pos.v][pos.u];
	CFPoint3DT &p2 = (*m_grid)[pos.w - 1][pos.v][pos.u];

	p.x = p1.x + (p2.x - p1.x) * offset;
	p.y = p1.y + (p2.y - p1.y) * offset;
	p.z = p1.z + (p2.z - p1.z) * offset;
}

inline void CMGMesh::GetTopPos(const CIPointUVW &pos, CFPoint3DT &p, const float &offset)
{
	CFPoint3DT &p1 = (*m_grid)[pos.w][pos.v][pos.u];
	CFPoint3DT &p2 = (*m_grid)[pos.w + 1][pos.v][pos.u];

	p.x = p1.x + (p2.x - p1.x) * offset;
	p.y = p1.y + (p2.y - p1.y) * offset;
	p.z = p1.z + (p2.z - p1.z) * offset;
}

inline void CMGMesh::GetBackPos(const CIPointUVW &pos, CFPoint3DT &p, const float &offset, int level)
{
	p = (*m_grid)[pos.w][pos.v][pos.u];
	int u = m_grid->GetPrevSliceU(pos.u, level);
	const CFPoint3DT &p2 = (*m_grid)[pos.w][pos.v][u];

	p.x += (p2.x - p.x) * offset;
	p.y += (p2.y - p.y) * offset;
	p.z += (p2.z - p.z) * offset;
}

inline void CMGMesh::GetFrontPos(const CIPointUVW &pos, CFPoint3DT &p, const float &offset, int level)
{
	p = (*m_grid)[pos.w][pos.v][pos.u];
	int u = m_grid->GetNextSliceU(pos.u, level);
	const CFPoint3DT &p2 = (*m_grid)[pos.w][pos.v][u];

	p.x += (p2.x - p.x) * offset;
	p.y += (p2.y - p.y) * offset;
	p.z += (p2.z - p.z) * offset;
}

inline void CMGMesh::GetLeftPos(const CIPointUVW &pos, CFPoint3DT &p, const float &offset, int level)
{
	p = (*m_grid)[pos.w][pos.v][pos.u];
	int v = m_grid->GetPrevSliceV(pos.v, level);
	const CFPoint3DT &p2 = (*m_grid)[pos.w][v][pos.u];

	p.x += (p2.x - p.x) * offset;
	p.y += (p2.y - p.y) * offset;
	p.z += (p2.z - p.z) * offset;
}

inline void CMGMesh::GetRightPos(const CIPointUVW &pos, CFPoint3DT &p, const float &offset, int level)
{
	p = (*m_grid)[pos.w][pos.v][pos.u];
	int v = m_grid->GetNextSliceV(pos.v, level);
	const CFPoint3DT &p2 = (*m_grid)[pos.w][v][pos.u];

	p.x += (p2.x - p.x) * offset;
	p.y += (p2.y - p.y) * offset;
	p.z += (p2.z - p.z) * offset;
}

inline void CMGMesh::GetBottomPos(const CIPointUVW &pos, CFPoint3DT &p, const float &offset, int level)
{
	p = (*m_grid)[pos.w][pos.v][pos.u];
	int w = m_grid->GetPrevSliceW(pos.w, level);
	const CFPoint3DT &p2 = (*m_grid)[w][pos.v][pos.u];

	p.x += (p2.x - p.x) * offset;
	p.y += (p2.y - p.y) * offset;
	p.z += (p2.z - p.z) * offset;
}

inline void CMGMesh::GetTopPos(const CIPointUVW &pos, CFPoint3DT &p, const float &offset, int level)
{
	p = (*m_grid)[pos.w][pos.v][pos.u];
	int w = m_grid->GetNextSliceW(pos.w, level);
	const CFPoint3DT &p2 = (*m_grid)[w][pos.v][pos.u];

	p.x += (p2.x - p.x) * offset;
	p.y += (p2.y - p.y) * offset;
	p.z += (p2.z - p.z) * offset;
}

inline void CMGMesh::GetHalfOrthoNormal(CFPoint3DT &n, int dir)
{
	switch (dir)
	{
	case BACK_FACE:   n.x = -0.5f;  n.y =  0.0f;  n.z =  0.0f;  break;
	case FRONT_FACE:  n.x =  0.5f;  n.y =  0.0f;  n.z =  0.0f;  break;
	case LEFT_FACE:   n.x =  0.0f;  n.y = -0.5f;  n.z =  0.0f;  break;
	case RIGHT_FACE:  n.x =  0.0f;  n.y =  0.5f;  n.z =  0.0f;  break;
	case BOTTOM_FACE: n.x =  0.0f;  n.y =  0.0f;  n.z = -0.5f;  break;
	case TOP_FACE:    n.x =  0.0f;  n.y =  0.0f;  n.z =  0.5f;  break;
	}
}

#endif // !defined(AFX_MGMESH_H__98F34843_D015_481A_888D_DB966BC3461B__INCLUDED_)
