/***********************************************************************
This software module was originally developed by Ioan Alexandru Salomie
(Vrije Universiteit Brussel VUB) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. VUB retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Ioan Alexandru Salomie at iasalomi@etro.vub.ac.be.
----------------------------------------------------------------------*/

// MGStreamDescription.h: interface for the CMGStreamDescription class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_MGSTREAMDESCRIPTION_H__8286638B_8ED6_4946_85B9_AAAC07849D05__INCLUDED_)
#define AFX_MGSTREAMDESCRIPTION_H__8286638B_8ED6_4946_85B9_AAAC07849D05__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "BitStream.h"
#include "ROISizePos.h"

enum {
	CYCLIC_NONE = 0,                                          // The encoded mesh is not cyclic
	CYCLIC_U = 1,                                             // The encoded mesh is cyclic in direction "U"
	CYCLIC_BITVAL_U = 1,
	CYCLIC_UV = 3,                                            // The encoded mesh is cyclic in direction "UV"
	CYCLIC_BITVAL_V = 2,
	FOLDED_SINGLE = 5,                                        // The encoded mesh is cyclic in direction "U" and folded in the "V" direction at the end corresponding to the first index
	FOLDED_BOTH = 7,                                          // The encoded mesh is cyclic in direction "U" and folded in the "V" direction at both ends
	CYCLIC_BITVAL_FOLDED = 4
};

class RGCoding
{
public:
	RGCoding();

public:
	int          gridCorners[8];     // the 8 corners of the grid
	int          scale;              // the scaling for the coded coefficients
};

class MStreamLayer
{
public:
	MStreamLayer();

	int   GetHeaderSize() const { return m_headerSz; };                                               // retrieve the size of the stream header
	void  SetHeaderSize(int size) { m_headerSz = size; };                                             // set the size of the stream header

	int   GetHeaderPosition() const { return m_headerPos; };                                          // retrieve the position of the header in the stream
	void  SetHeaderPosition(int pos) { m_headerPos = pos; };                                          // set the position of the header in the stream

	int   Init(const CIPointUVW &nrSlices, const CIPointUVW &nrROIs, const CIPointUVW &maxROISz,
			   const CIPointUVW &levelDiff);														  // set the number of ROIs and allocate the arrays elements for each level

	int   GetTotalNrROIs() const { return m_nrROIs.u * m_nrROIs.v * m_nrROIs.w; };					  // return the total number of ROIs
	const CIPointUVW & GetNrROIs() const { return m_nrROIs; };										  // return the number of ROIs
	const CIPointUVW & GetNrSlices() const { return m_nrSlices; };									  // return the number of Slices
	const CIPointUVW & GetLevelDiff() const { return m_levelDiff; }									  // return the difference in levels from top level

	int   GetRepositionBitsMode() const { return m_reposMode; };                                      // retrieve the vertices' reposition mode
	void  SetRepositionBitsMode(int mode) { m_reposMode = mode; };                                    // set the vertices' reposition mode

	int   GetNrBitsROIIndex() const { return m_nrBitsROIIndex; };                                     // retrieve the number of bits allocated for the ROI index
	void  SetNrBitsROIIndex(BYTE nrBits) { m_nrBitsROIIndex = nrBits; };                              // set the number of bits allocated for the ROI index

	int   GetCodingLookup(int roiIdx) const { return m_codingLookup[roiIdx]; };                       // retrieve the coding lookup for each ROI: false means no coding info
	void  SetCodingLookup(int roiIdx, bool flag) { m_codingLookup[roiIdx] = flag; };                  // set the coding lookup for each ROI: false means no coding info
	CBoolArray & GetCodingLookup()  { return m_codingLookup; };                                       // retrieve the coding lookup for each ROI: false means no coding info

	const CIPointUVW & GetNrBitsVertPosField() const { return m_nrBitsVertPos;  };                    // retrieve the nr of bits per counter holding the size of the ROI (x, y, z)
	void  SetNrBitsVertPosField(const CIPointUVW &nrBits) { m_nrBitsVertPos = nrBits; };              // set the nr of bits per counter holding the size of the ROI (x, y, z)

	int   GetNrBitsConnectivityCounterField() const { return m_nrBitsConnetivityCounter; };           // retrieve the number of bits allocated for the connectivity counter field
	void  SetNrBitsConnectivityCounterField(BYTE nrBits) { m_nrBitsConnetivityCounter = nrBits; };    // set the number of bits allocated for the connectivity counter field

	int   GetNrBitsVertsReposCounterField() const { return m_nrBitsVertsReposCounter; };              // retrieve the number of bits allocated for the vertex reposition counter field
	void  SetNrBitsVertsReposCounterField(BYTE nrBits) { m_nrBitsVertsReposCounter = nrBits; };       // set the number of bits allocated for the vertex reposition counter field

	int   GetNrBitsVertsRefineCounterField() const { return m_nrBitsVertsRefineCounter; };            // retrieve the number of bits allocated for the vertex refinement counter field
	void  SetNrBitsVertsRefineCounterField(BYTE nrBits) { m_nrBitsVertsRefineCounter = nrBits; };     // set the number of bits allocated for the vertex refinement counter field

	int   GetNrBytesConnectivityCoding(int roiIdx) const { return m_connectivityCount[roiIdx]; };         // retrieve the number of vertices from the ROI
	void  SetNrBytesConnectivityCoding(int roiIdx, int count) { m_connectivityCount[roiIdx] = count; };   // set the number of vertices from the ROI
	CIntArray & GetNrBytesConnectivityCoding() { return m_connectivityCount; };                           // retrieve the number of vertices from the ROI

	int   GetNrBytesVertsReposCoding(int roiIdx) const { return m_vertsReposCount[roiIdx]; };             // retrieve the number of vertices from the ROI
	void  SetNrBytesVertsReposCoding(int roiIdx, int count) { m_vertsReposCount[roiIdx] = count; };       // set the number of vertices from the ROI
	CIntArray & GetNrBytesVertsReposCoding() { return m_vertsReposCount; };                               // retrieve the number of vertices from the ROI

	int   GetNrBytesVertsRefineCoding(int roiIdx) const { return m_vertsRefineCount[roiIdx]; };           // retrieve the number of vertices from the ROI
	void  SetNrBytesVertsRefineCoding(int roiIdx, int count) { m_vertsRefineCount[roiIdx] = count; };     // set the number of vertices from the ROI
	CIntArray & GetNrBytesVertsRefineCoding() { return m_vertsRefineCount; };                             // retrieve the number of vertices from the ROI

	int   GetPosStreamConnectivity(int roiIdx) const { return m_posStreamConnectivity[roiIdx]; };     // retrieve the stream position of the mesh connectivity bits
	void  SetPosStreamConnectivity(int roiIdx, int pos) { m_posStreamConnectivity[roiIdx] = pos; };   // set the stream position of the mesh connectivity bits
	CIntArray & GetPosStreamConnectivity() { return m_posStreamConnectivity; };                       // retrieve the stream position of the mesh connectivity bits

	int   GetPosStreamRefine(int roiIdx) const { return m_posStreamRefine[roiIdx]; };                 // retrieve the stream position of the vertices' refine bits
	void  SetPosStreamRefine(int roiIdx, int pos) { m_posStreamRefine[roiIdx] = pos; };               // set the stream position of the vertices' refine bits
	CIntArray & GetPosStreamRefine() { return m_posStreamRefine; };                                   // retrieve the stream position of the vertices' refine bits 

	int   GetPosStreamReposition(int roiIdx) const { return m_posStreamReposition[roiIdx]; };         // retrieve the stream position of the vertices' reposition bits
	void  SetPosStreamReposition(int roiIdx, int pos) { m_posStreamReposition[roiIdx] = pos; };       // set the stream position of the vertices' reposition bits
	CIntArray & GetPosStreamReposition() { return m_posStreamReposition; };                           // retrieve the stream position of the vertices' reposition bits 

	void  DumpStream(CStringArray &msgs, const ROIDescription &roiDescr);                             // dump the description about the MeshGrid stream

	bool  IsDoneROIConnectivity(int roiIdx) const { return m_cacheROI[roiIdx]; };                     // check whether the ROI connectivity has been recovered
	bool  IsDoneROIRefine(int roiIdx) const { return m_cacheRefine[roiIdx]; };                        // check whether the ROI Refine bits have been recovered
	bool  IsDoneROIReposition(int roiIdx) const { return m_cacheReposition[roiIdx]; };                // check whether the ROI Reposition bits have been recovered
	void  DoneROIConnectivity(int roiIdx, bool flag) { m_cacheROI[roiIdx] = flag; };                  // set the recovered flag for the ROI connectivity
	void  DoneROIRefine(int roiIdx, bool flag) { m_cacheRefine[roiIdx] = flag; };                     // set the recovered flag for the ROI Refine bits
	void  DoneROIReposition(int roiIdx, bool flag) { m_cacheReposition[roiIdx] = flag; };             // set the recovered flag for the ROI Reposition bits

protected:
	int   AddMsg(LPCTSTR lpszFormat, ...);                                                            // add a message to the message list
	int   AddMsg(const CString &msg) { return m_msgs->Add(msg); };                                    // add a message to the message list
	void  DeleteMsg(int idx) { m_msgs->RemoveAt(idx); };                                              // delete a message from the list

public:
	int          m_headerSz;         // size of the stream header
	int          m_headerPos;        // position of the header in the stream
	int          m_reposMode;        // specifies the default resposition mode for vertices

protected:
	CIPointUVW	   m_nrROIs;
	CIPointUVW	   m_nrSlices;
	CIPointUVW	   m_levelDiff;
	int            m_nrBitsROIIndex;            // nr of bits per counter holding the index of the ROI
	CIPointUVW     m_nrBitsVertPos;             // nr of bits per variable holding the position of a starting vertex
	int            m_nrBitsConnetivityCounter;  // nr of bits per counter holding the size of the coded connectivity
	int            m_nrBitsVertsReposCounter;   // nr of bits per counter holding the size of the coded vertices reposition bits
	int            m_nrBitsVertsRefineCounter;  // nr of bits per counter holding the size of the coded vertices refinement bits
	CBoolArray     m_codingLookup;              // lookup holding the coding status of ROIs: false means no coding bits
	CIntArray      m_connectivityCount;         // counter holding the size of the coded mesh connectivity info, per roi
	CIntArray      m_vertsReposCount;           // counter holding the size of the coded vertices reposition info, per roi
	CIntArray      m_vertsRefineCount;          // counter holding the size of the coded vertices refine info, per roi
	CIntArray      m_posStreamConnectivity;     // position in the stream of the mesh connectivity bits for each ROI
	CIntArray      m_posStreamRefine;           // position in the stream of the vertices' refinement bits for each ROI
	CIntArray      m_posStreamReposition;       // position in the stream of the vertices' reposition bits for each ROI
	CStringArray * m_msgs;                      // used for dumping the stream
	CBoolArray     m_cacheROI;                  // mark the ROIs (vertices) that have been decoded (vertices)
	CBoolArray     m_cacheRefine;               // mark the ROIs (vertices) that have been refined
	CBoolArray     m_cacheReposition;           // mark the ROIs (vertices) that have been repositioned
};

class GStreamLevel
{
public:
	GStreamLevel();

	int   GetHeaderSize() const { return m_headerSz; };                                                       // retrieve the size of the stream header
	void  SetHeaderSize(int size) { m_headerSz = size; };                                                     // set the size of the stream header

	int   GetHeaderPosition() const { return m_headerPos; };                                                  // retrieve the position of the header in the stream
	void  SetHeaderPosition(int pos) { m_headerPos = pos; };                                                  // set the position of the header in the stream

	int   Init(const CIPointUVW &nrSlices, const CIPointUVW &nrTiles, const CIPointUVW &levelDiff, int initValue);                                                // allocate the arrays elements for each level
	const CIPointUVW &GetLevelDiff() const { return m_levelDiff; };
	const CIPointUVW &GetNrSlices() const { return m_nrSlices; };

	const CIPointUVW &GetNrTiles() const { return m_nrTiles; };                                               // retrieve the number of Tiles for a level
	int   GetTotalNrTiles() const { return m_nrTiles.u * m_nrTiles.v * m_nrTiles.w; };                        // retrieve the total number of Tiles for a level

	bool  CanSplit() const { return m_bSplit; };                                                              // retrieve the split-into-subbands flag
	void  SetSplit(bool flag) { m_bSplit = flag; };                                                           // set the split-into-subbands flag

	int   GetNrBitsTileIndex() const { return m_nrBitsTileIndex; };                                           // retrieve the number of bits allocated for the Tile field
	void  SetNrBitsTileIndex(BYTE nrBits) { m_nrBitsTileIndex = nrBits; };                                    // set the number of bits allocated for the Tile field

	int   GetNrBitsGridCounterField() const { return m_nrBitsGridCounter; };                                  // retrieve the number of bits allocated for the grid counter field
	void  SetNrBitsGridCounterField(BYTE nrBits) { m_nrBitsGridCounter = nrBits; };                           // set the number of bits allocated for the grid counter field

	int   GetNrBytesTileCoding(int dir, int tileIdx) const { return m_tilesCount[dir][tileIdx]; };            // retrieve the number of bits allocated for the grid counter field
	void  SetNrBytesTileCoding(int dir, int tileIdx, int nrBytes) { m_tilesCount[dir][tileIdx] = nrBytes; };  // set the number of bits allocated for the grid counter field
	CIntArray & GetNrBytesTileCoding(int dir) { return m_tilesCount[dir]; };                                  // retrieve the number of bits allocated for the grid counter field

	int   GetPosStreamTile(int dir, int tileIdx) const { return m_posStreamTile[dir][tileIdx]; };             // retrieve the stream position of the mesh coding
	void  SetPosStreamTile(int dir, int tileIdx, int pos) { m_posStreamTile[dir][tileIdx] = pos; };           // set the stream position of the mesh coding
	CIntArray & GetPosStreamTile(int dir) { return m_posStreamTile[dir]; };                                   // retrieve the stream position of the mesh coding

	int   GetThreshold(int dir) const { return m_threshold[dir]; };                                           // retrieve the threshold for the specified dir
	int  *GetThreshold() { return m_threshold; };                                                             // retrieve the thresholds
	void  SetThreshold(int dir, int threshold) { m_threshold[dir] = threshold; };                             // set the threshold for the specified dir

	int   GetStopThreshold(int dir) const { return m_stopThreshold[dir]; };                                           // retrieve the threshold for the specified dir
	int  *GetStopThreshold() { return m_stopThreshold; };                                                             // retrieve the thresholds
	void  SetStopThreshold(int dir, int threshold) { m_stopThreshold[dir] = threshold; };                             // set the threshold for the specified dir

	void  DumpStream(const CIPointUVW &nrSlices, CStringArray &msgs, ROIDescription &roiDescr);               // dump the description about the MeshGrid stream

	bool  IsTileStreamed(int dir, int tileIdx) const { return m_cacheTile[dir][tileIdx]; };                   // check whether the Tile has been streamed/recovered to/from the stream
	void  SetStreamedTile(int dir, int tileIdx, bool flag) { m_cacheTile[dir][tileIdx] = flag; };             // set the streamed/recovered flag for the Tile

	bool  IsRequested() const { return m_bRequested; };                                                 
	bool  IsDecoded() const { return m_bDecoded; };
	void  SetRequested() { m_bRequested = true; };                                                 
	void  SetDecoded() { m_bDecoded = true; };

protected:
	int   AddMsg(LPCTSTR lpszFormat, ...);                                                                    // add a message to the message list
	int   AddMsg(const CString &msg) { return m_msgs->Add(msg); };                                            // add a message to the message list
	void  DeleteMsg(int idx) { m_msgs->RemoveAt(idx); };                                                      // delete a message from the list

public:
	CIPointUVW	 m_nrSlices;
	CIPointUVW	 m_levelDiff;
	CIPointUVW   m_nrTiles;               // nr of tiles per sublevel
	int          m_headerSz;              // size of the stream header
	int          m_headerPos;             // position of the header in the stream
	bool         m_bSplit;                // flag = true if grid size is large enough to be split into subbands
	int          m_threshold[3];          // the threshold for each direction
	int          m_stopThreshold[3];      // the threshold for each direction where the coding should stop

protected:
	bool           m_bRequested;            // flag specifying that the corresponding information has been requested to the server
	bool           m_bDecoded;              // flag specifying that the corresponding information has been decoded
	int            m_nrBitsTileIndex;       // nr of bits per counter holding the index of the Tile
	int            m_nrBitsGridCounter;     // nr of bits per counter holding the size of the coded grid
	CIntArray      m_tilesCount[3];         // size in bits of each of the coded tiles
	CIntArray      m_posStreamTile[3];      // position in the stream of the coded tiles
	CStringArray * m_msgs;                  // used for dumping the stream
	CBoolArray     m_cacheTile[3];          // cache the coded/decoded tiles to/from the stream
};

typedef CDataArray<MStreamLayer, MStreamLayer&>		MStreamLayers;
typedef CDataArray<MStreamLayers, MStreamLayers&>	MStreamLevels;
typedef CDataArray<GStreamLevel, GStreamLevel&>		GStreamLevels;

class MGStreamDescr
{
public:
	MGStreamDescr();

	int   GetHeaderSize() const { return m_headerSz; };                         // retrieve the size of the stream header
	void  SetHeaderSize(int size) { m_headerSz = size; };                       // set the size of the stream header

	char  GetMaxNrLevelsMesh() const { return m_maxNrLevelsM; };                // retrieve the max number of levels mesh;
	char  GetMaxNrLevelsGrid() const { return m_maxNrLevelsG; };                // retrieve the max number of levels grid;
	const CIPointUVW &GetNrLevelsMesh() const { return m_nrLevelsM; };          // retrieve the number of levels of the Mesh;
	const CIPointUVW &GetNrLevelsGrid() const { return m_nrLevelsG; };          // retrieve the number of levels of the Grid;
	const CIPointUVW &GetNrROIs() const { return m_roiDescr.GetNrROIs(); };     // retrieve the number of ROIS;
	int	  GetTotalNrROIs() const { return m_roiDescr.GetTotalNrROIs(); };     // retrieve the number of ROIS;
	const CIPointUVW &GetROISize() const { return m_roiDescr.GetROISize(); };   // retrieve the size of the ROIS;
	const CIPointUVW &GetMaxROISize() const { return m_roiDescr.GetMaxROISize(); };   // retrieve the max size of the ROIS;
	void  Init(const CIPointUVW &nrLevels, const CIPointUVW &nrSlices, const CIPointUVW &nrROIs);    // set the nr of levels of the mesh and initilaise the levels objects
	void  InitMeshLayers(BYTE nrLayers);										// set the nr of layers of the mesh and initilaise the layers objects
	BYTE  GetNrLayers() const { return m_nrLayers; };							// retrieve the number of layers

	void  InitGridLevels(BYTE nrLevels);                                        // set the max nr of levels of the grid and initilaise the levels objects

	int   GetStreamSize() const { return m_streamSize; };                       // retrieve the size of the stream
	void  SetStreamSize(int size) { m_streamSize = size; };                     // set the size of the stream

	const CIPointUVW & GetMaxNrSlices() const { return m_maxNrSlices; };        // retrieve the max number of slices
	bool  IsSingleLayered() const { return m_bSingleLayered; };                 // retrieve whether the number of slices is 1
	void  SetSingleLayered(bool flag) { m_bSingleLayered = flag; };             // set whether the number of slices is 1

	bool  HasConnectivityInfo() const { return m_bHasConnectivityInfo; };       // retrieve the connectivity availability flag
	void  HasConnectivityInfo(bool flag) { m_bHasConnectivityInfo = flag; };    // set the connectivity availability flag

	bool  HasRefineInfo() const { return m_bHasRefineInfo; };                   // retrieve the refine availability flag
	void  HasRefineInfo(bool flag) { m_bHasRefineInfo = flag; };                // set the refine availability flag

	bool  HasRepositionInfo() const { return m_bHasRepositionInfo; };           // retrieve the reposition availability flag
	void  HasRepositionInfo(bool flag) { m_bHasRepositionInfo = flag; };        // set the reposition availability flag

	bool  IsRepositionInfoAvailable(int level) const
	{ return HasRepositionInfo() && !IsAutoResolution() && level < GetMaxNrLevelsMesh() - 1; }

	bool  HasGridInfo() const { return m_bHasGridInfo; };                       // retrieve the grid availability flag
	void  HasGridInfo(bool flag) { m_bHasGridInfo = flag; };                    // set the grid availability flag

	bool  HasNormalsInfo() const { return m_bHasNormalsInfo; };                 // retrieve the normals availability flag
	void  HasNormalsInfo(bool flag) { m_bHasNormalsInfo = flag; };              // set the normals availability flag

	bool  IsConnectivityInfoAvailable(int level) const;                         // retrieve whether the connectivity links are / have to be stored in the stream
	bool  IsRefinementInfoAvailable(int level) const;                           // retrieve whether the refine bits are / have to be stored in the stream
	bool  IsRepositionInfoAvailable(int level, BYTE layer) const;               // retrieve whether the reposition bits are / have to be stored in the stream
	bool  IsGridInfoAvailable(int lavel) const;                                 // retrieve whether the grid coefficients are / have to be stored in the stream

	BYTE  GetNrBitsLayer() const { return m_nrBitsLayer; };						// retrieve the number of bits for storing the layer
	void  SetNrBitsLayer(BYTE nrBits) { m_nrBitsLayer = nrBits; };				// set the number of bits for storing the layer

	BYTE  GetNrRefineBitsVertex() const { return m_nrRefBitsVert; };            // retrieve the number of refine bits for the vertices
	void  SetNrRefineBitsVertex(BYTE nrBits) { m_nrRefBitsVert = nrBits; };     // set the number of refine bits for the vertices

	bool  InterleaveRefine() const { return m_bInterleaveRefine; };             // retrieve the interleave vertices refine bits flag
	void  SetInterleaveRefine(bool flag) { m_bInterleaveRefine = flag; };       // set the interleave vertices refine bits flag

	int   GetMeshType() const { return m_meshType; };                           // retrieve the type of mesh
	void  SetMeshType(int type) { m_meshType = type; };                         // set the type of mesh

	bool  IsSameConnectivity() const { return m_bSameConnect; };                // retrieve the same connectivity flag
	void  SetSameConnectivity(bool bSameConnect) { m_bSameConnect = bSameConnect; }; // set the same connectivity flag

	bool  IsAutoResolution() const { return m_bAutoResolution; }                // retrieve the auto resolution mode
	void  SetAutoResolution(bool flag) { m_bAutoResolution = flag; };           // set the auto resolution mode

	int   GetCyclicMode() const { return m_cyclicMode; }                        // retrieve the cyclic mode
	void  SetCyclicMode(int mode) { m_cyclicMode = mode; };                     // set the auto cyclic mode
	bool  IsCyclicU() const { return m_cyclicMode & CYCLIC_BITVAL_U; };
	bool  IsCyclicV() const { return m_cyclicMode == CYCLIC_UV; };
	bool  IsCyclicUV() const { return m_cyclicMode == CYCLIC_UV; };
	bool  IsFoldedSingle() const { return m_cyclicMode == FOLDED_SINGLE; };
	bool  IsFoldedBoth() const { return m_cyclicMode == FOLDED_BOTH; };

	int   GetOffsetAmplitude() const { return m_offsetAmp; };                   // retrieve the amplitude of the offset
	int   SetOffsetAmplitude(int offsetAmp) { return m_offsetAmp = offsetAmp; };// retrieve the amplitude of the offset

	int   GetFilterType() const { return m_filterType; };                       // retrieve the filter type
	void  SetFilterType(int filterType) { m_filterType = filterType; };         // set the filter type

	int   GetNrQuantBitsGrid() const { return m_nrQuantBits; };                 // retrieve the number of quantization bits
	void  SetNrQuantBitsGrid(int nrBits) { m_nrQuantBits = nrBits; };           // retrieve the number of quantization bits

	RGCoding * GetGridCodingParam() { return m_gridCoding; };                   // retrieve the coding parameters for the grid
	RGCoding & GetGridCodingParam(int dir) { return m_gridCoding[dir]; };       // retrieve the coding parameters for the grid

	MStreamLevels &GetMeshLevel() { return m_streamLevelM; };                   // retrieve the resolution levels from the mesh coding
	MStreamLayers &GetMeshLevel(int level) { return m_streamLevelM[level]; };   // retrieve a resolution level from the mesh coding
	MStreamLayer &GetMeshLayer(int level, BYTE layer) { return m_streamLevelM[level][layer]; };		// retrieve a resolution level from the mesh coding

	GStreamLevel &GetGridLevel(int level) { return m_streamLevelG[level]; };    // retrieve a resolution level from the grid coding
	GStreamLevels &GetGridLevel() { return m_streamLevelG; };                   // retrieve the resolution levels from the grid coding

	int   GetNrBitsGridParam() const { return m_nrBitsFieldGrid; };             // retrieve the number of bits allocated for the grid param field
	void  SetNrBitsGridParam(BYTE nrBits) { m_nrBitsFieldGrid = nrBits; };      // set the number of bits allocated for the grid param field

	void  SetThresholds(int dir, const CIntArray &thresholds);                  // set the thresholds for the grid resolution levels

	void  DumpStream(CStringArray &msgs);                                       // dump the description about the MeshGrid stream
	ROIDescription & GetROIDescr() { return m_roiDescr; };						// retrieve the roi size and position description

protected:
	int   AddMsg(LPCTSTR lpszFormat, ...);                                      // add a message to the message list
	int   AddMsg(const CString &msg) { return m_msgs->Add(msg); };               // add a message to the message list
	void  DeleteMsg(int idx) { m_msgs->RemoveAt(idx); };                         // delete a message from the list

public:
	int            m_streamSize;          // the number of parts that make the stream
	int            m_headerSz;            // size of the stream header
	char           m_maxNrLevelsM;        // max number of resolution levels of the mesh
	char           m_maxNrLevelsG;        // max number of resolution levels of the grid
	CIPointUVW	   m_nrLevelsM;			  // number of resolution levels of the mesh
	CIPointUVW	   m_nrLevelsG;			  // number of resolution levels of the grid
	CIPointUVW	   m_maxNrSlices;		  // max nr of slices of the mesh grid
	bool           m_bSingleLayered;      // whether the number of slices is 1

	bool           m_bHasConnectivityInfo;// whether the connectivity info is available in the stream
	bool           m_bHasRefineInfo;      // whether the refine info is available in the stream
	bool           m_bHasRepositionInfo;  // whether the reposition info is available in the stream
	bool           m_bHasGridInfo;        // whether the grid info is available in the stream
	bool           m_bHasNormalsInfo;     // whether the normals info is available in the stream
	bool           m_bInterleaveRefine;   // 0 - no interleave, 1 - interleave vertices refine bits
	int            m_meshType;            // store the type of mesh
	bool           m_bSameConnect;        // if true the connectivity type between the vertices of the quadrilateral mesh is the same
	bool           m_bAutoResolution;     // whether the uniform subdivision can be applied to obtain higher resolution levels
	int            m_offsetAmp;           // amplitude of the offset
	int            m_cyclicMode;          // specifies the cyclic behaviour of the mesh 
	BYTE           m_nrRefBitsVert;  	  // nr of refine bits for the vertices offsets 
	BYTE		   m_nrBitsLayer;		  // nr of bits for storing the layer
	BYTE		   m_nrLayers;			  // the number of layers
	ROIDescription m_roiDescr;			  // keep track of the ROI size and position at the different levels

	int            m_filterType;          // type of the filter used for the interpolation
	int            m_nrQuantBits;         // nr of bits to quantize the grid values
	RGCoding       m_gridCoding[3];       // the coding parametres of the grid

protected:
	int            m_nrBitsFieldGrid;     // nr of bits per counter holding the coded grid parameters
	CStringArray * m_msgs;                // used for dumping the stream

public:
	MStreamLevels  m_streamLevelM;        // the level description for the mesh coding
	GStreamLevels  m_streamLevelG;        // the level description for the grid coding
};

class MGStream
{
public:
	MGStream();
	~MGStream();

	void  Initialise(CFile *file, CBitStream::BITSTREAM_OPERATIONS op);               // initialise the file and attach the stream
	void  DeleteStream();															  // detach and delete the stream

	CBitStream *InitStream(int limit);                                                // initilaise and attach the stream
	CBitStream *GetStream() { return m_pStream; };                                    // retrieve the bitream

	void  Flush() { m_pStream->Flush(); };                                            // empty the stream buffers
	void  SetPosition(int pos);                                                       // position the file and reset the stream
	void  SetPositionAndLimit(int pos, int limit, CBitStream::BITSTREAM_OPERATIONS op); // position the file and create new stream
	void  MovePosition(int pos);                                                      // position the file and reset the stream
	int   GetPosition() const { return (int) m_pFile->GetPosition(); };                     // get the current file position
	int   GetBitCount() const { return m_pStream->GetBitCount(); };                   // get the bit count from the stream

	BYTE  GetRemainingValue() const { return m_pStream->GetRemainingValue(); };
	BYTE  GetBitPosition() const { return m_pStream->GetBitPosition(); };

	void  Write(BYTE *buffer, int sz) { m_pFile->Write(buffer, sz); };                // write raw data to the file

	void  ReadData(BYTE *res) { m_pStream->ReadData(res); };                          // read a BYTE from the stream
	void  WriteData(BYTE value) { m_pStream->WriteData(value); };                     // write a BYTE to the stream

	void  ReadBit(BYTE *res) { m_pStream->ReadBit(res); };                            // read a BIT to the stream
	void  ReadBits(BYTE *res, BYTE count) { m_pStream->ReadBits(res, count); };       // read a BYTE to the stream
	void  ReadBits(WORD *res, BYTE count) { m_pStream->ReadBits(res, count); };       // read a WORD to the stream
	void  ReadBits(DWORD *res, BYTE count) { m_pStream->ReadBits(res, count); };      // read a DWORD to the stream
	void  WriteBit(BYTE value) { m_pStream->WriteBit(value); };                       // write a BIT to the stream
	void  WriteBits(BYTE value, BYTE count) { m_pStream->WriteBits(value, count); };  // write a BYTE to the stream
	void  WriteBits(WORD value, BYTE count) { m_pStream->WriteBits(value, count); };  // write a WORD to the stream
	void  WriteBits(DWORD value, BYTE count) { m_pStream->WriteBits(value, count); }; // write a DWORD to the stream

	bool  RawRead(BYTE *buf, int pos, int count);                                     // read raw data from the stream
	int   Read(BYTE *buffer, int sz) { return m_pFile->Read(buffer, sz); };           // read raw data from the file

	void  SeekToBegin() { m_pFile->SeekToBegin(); };                                  // position the stream file at the begining
	void  ResetStream() { m_pStream->Reset(); };                                      // reset the stream without affecting the file  

public:
	CBitStream		* m_pStream; 		// stream for coding
	CFile			* m_pFile;          // buffer for coding

protected:
	bool			m_bAllocInternal;
};

#endif // !defined(AFX_MGSTREAMDESCRIPTION_H__8286638B_8ED6_4946_85B9_AAAC07849D05__INCLUDED_)
