/***********************************************************************
This software module was originally developed by Ioan Alexandru Salomie
(Vrije Universiteit Brussel VUB) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. VUB retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Ioan Alexandru Salomie at iasalomi@etro.vub.ac.be.
----------------------------------------------------------------------*/

#include "StdAfx.h"
#include "MGMesh.h"


void CMGMesh::StoreTriangle(CSurfNodeT *pSN1, CSurfNodeT *pSN2, CSurfNodeT *pSN3)
{
	CSurfPolyT sp;

	// check for zero sized triangles when the vertices are too close
	//	const CFPoint3DT &p1 = m_vertices[pSN1->GetIndex()];
	//	const CFPoint3DT &p2 = m_vertices[pSN2->GetIndex()];
	//	const CFPoint3DT &p3 = m_vertices[pSN3->GetIndex()];
	//  if ((FloatEq(p1.x, p2.x) && FloatEq(p1.y, p2.y) && FloatEq(p1.z, p2.z)) ||
	//      (FloatEq(p2.x, p3.x) && FloatEq(p2.y, p3.y) && FloatEq(p2.z, p3.z)) ||
	//      (FloatEq(p1.x, p3.x) && FloatEq(p1.y, p3.y) && FloatEq(p1.z, p3.z)))
	//    return;

	sp.SetPoint1(pSN1);	sp.SetPoint2(pSN2);	sp.SetPoint3(pSN3);

	AddPolygon(sp);		++m_nrTriangles;
}

void CMGMesh::StoreRectangle(CSurfNodeT *pSN1, CSurfNodeT *pSN2, CSurfNodeT *pSN3, CSurfNodeT *pSN4)
{
	StoreTriangle(pSN1, pSN2, pSN3);
	StoreTriangle(pSN1, pSN3, pSN4);
}

void CMGMesh::StoreSubPentagone(CSurfNodeT *pSN1, CSurfNodeT *pSN2, CSurfNodeT *pSN3, CSurfNodeT *pSN4, CSurfNodeT *pSN5)
{
	pSN1->IncCount();	pSN2->IncCount();
	pSN3->IncCount();	pSN4->IncCount();	pSN5->IncCount();

	StoreTriangle(pSN1, pSN2, pSN5);
	StoreRectangle(pSN2, pSN3, pSN4, pSN5);
}

void CMGMesh::StorePentagone(CSurfNodeT *pSN1, CSurfNodeT *pSN2, CSurfNodeT *pSN3, CSurfNodeT *pSN4, CSurfNodeT *pSN5)
{
	if (pSN5->GetDir() == pSN1->GetDir() &&
		pSN2->GetDir() == pSN1->GetDir()) 
	{
		StoreSubPentagone(pSN1, pSN2, pSN3, pSN4, pSN5);
	}
	else if (pSN1->GetDir() == pSN2->GetDir() &&
		pSN3->GetDir() == pSN2->GetDir())
	{
		StoreSubPentagone(pSN2, pSN3, pSN4, pSN5, pSN1);
	}
	else if (pSN2->GetDir() == pSN3->GetDir() &&
		pSN4->GetDir() == pSN3->GetDir())
	{
		StoreSubPentagone(pSN3, pSN4, pSN5, pSN1, pSN2);
	}
	else if (pSN3->GetDir() == pSN4->GetDir() &&
		pSN5->GetDir() == pSN4->GetDir())
	{
		StoreSubPentagone(pSN4, pSN5, pSN1, pSN2, pSN3);
	}
	else if (pSN4->GetDir() == pSN5->GetDir() &&
		pSN1->GetDir() == pSN5->GetDir())
	{
		StoreSubPentagone(pSN5, pSN1, pSN2, pSN3, pSN4);
	}
	else
	{
		ASSERT(0);
	}
}

void CMGMesh::StoreHexagone1(CSurfNodeT *pSN1, CSurfNodeT *pSN2, CSurfNodeT *pSN3, CSurfNodeT *pSN4, CSurfNodeT *pSN5, CSurfNodeT *pSN6)
{
	pSN1->SetHexagonType1();	pSN1->IncCount();
	pSN2->SetHexagonType1();	pSN2->IncCount();
	pSN3->SetHexagonType1();	pSN3->IncCount();
	pSN4->SetHexagonType1();	pSN4->IncCount();
	pSN5->SetHexagonType1();	pSN5->IncCount();
	pSN6->SetHexagonType1();	pSN6->IncCount();

	StoreTriangle(pSN1, pSN2, pSN3);
	StoreTriangle(pSN2, pSN4, pSN6);
	StoreTriangle(pSN6, pSN5, pSN3);
	StoreTriangle(pSN2, pSN6, pSN3);
}

void CMGMesh::StoreSubHexagone2(CSurfNodeT *pSN1, CSurfNodeT *pSN2, CSurfNodeT *pSN3, CSurfNodeT *pSN4, CSurfNodeT *pSN5, CSurfNodeT *pSN6)
{
	CFPoint3DT n1, n2;
	GetHalfOrthoNormal(n1, pSN1->GetDir());
	GetHalfOrthoNormal(n2, pSN2->GetDir());

	int index1 = pSN1->GetIndex();    ASSERT(index1 >=0 && index1 < m_totalNrVertices);
	int index2 = pSN2->GetIndex();    ASSERT(index2 >=0 && index2 < m_totalNrVertices);

	const CFPoint3DT &p1 = m_vertices[index1];
	const CFPoint3DT &p2 = m_vertices[index2];

	if ((p1.x + n1.x == p2.x + n2.x) &&
		(p1.y + n1.y == p2.y + n2.y) &&
		(p1.z + n1.z == p2.z + n2.z))
	{
		// case 4
		StoreTriangle(pSN1, pSN2, pSN6);
		StoreTriangle(pSN4, pSN5, pSN3);
		StoreTriangle(pSN3, pSN6, pSN2);
		StoreTriangle(pSN6, pSN3, pSN5);
	}
	else
	{
		// case 4
		StoreTriangle(pSN1, pSN5, pSN6);
		StoreTriangle(pSN4, pSN2, pSN3);
		StoreTriangle(pSN1, pSN4, pSN5);
		StoreTriangle(pSN4, pSN1, pSN2);
	}
}

void CMGMesh::StoreHexagone2(CSurfNodeT *pSN1, CSurfNodeT *pSN2, CSurfNodeT *pSN3, CSurfNodeT *pSN4, CSurfNodeT *pSN5, CSurfNodeT *pSN6)
{
	pSN1->IncCount();	pSN2->IncCount(); pSN3->IncCount();
	pSN4->IncCount();	pSN5->IncCount();	pSN6->IncCount();

	// identify two consecutive nodes located on the same plane
	if (pSN1->GetDir() == pSN2->GetDir())
	{
		ASSERT(pSN4->GetDir() == pSN5->GetDir());
		StoreSubHexagone2(pSN2, pSN3, pSN4, pSN5, pSN6, pSN1);
	}
	else if (pSN2->GetDir() == pSN3->GetDir())
	{
		ASSERT(pSN5->GetDir() == pSN6->GetDir());
		StoreSubHexagone2(pSN3, pSN4, pSN5, pSN6, pSN1, pSN2);
	}
	else if (pSN3->GetDir() == pSN4->GetDir())
	{
		ASSERT(pSN6->GetDir() == pSN1->GetDir());
		StoreSubHexagone2(pSN4, pSN5, pSN6, pSN1, pSN2, pSN3);
	}
	else
	{
		// flat hexagone
		StoreRectangle(pSN1, pSN2, pSN3, pSN4);
		StoreRectangle(pSN4, pSN5, pSN6, pSN1);
	}
}

void CMGMesh::StoreHexagone3(CSurfNodeT *pSN1, CSurfNodeT *pSN2, CSurfNodeT *pSN3, CSurfNodeT *pSN4, CSurfNodeT *pSN5, CSurfNodeT *pSN6)
{
	pSN1->IncCount();	pSN2->IncCount(); pSN3->IncCount();
	pSN4->IncCount();	pSN5->IncCount();	pSN6->IncCount();

	StoreRectangle(pSN1, pSN2, pSN4, pSN6);
	StoreRectangle(pSN6, pSN5, pSN3, pSN1);
}

void CMGMesh::StoreSeptagone(CSurfNodeT *pSN1, CSurfNodeT *pSN2, CSurfNodeT *pSN3, CSurfNodeT *pSN4, CSurfNodeT *pSN5, CSurfNodeT *pSN6, CSurfNodeT *pSN7)
{
	pSN1->IncCount();	pSN2->IncCount(); pSN3->IncCount();
	pSN4->IncCount();	pSN5->IncCount();	pSN6->IncCount();	pSN7->IncCount();

	StoreTriangle(pSN1, pSN2, pSN4);
	StoreTriangle(pSN1, pSN4, pSN6);
	StoreTriangle(pSN1, pSN6, pSN7);
	StoreTriangle(pSN1, pSN7, pSN5);
	StoreTriangle(pSN1, pSN5, pSN3);
}

bool CMGMesh::BuildSeptagones(CSurfNodeT *pSN, int level)
{
	CSurfNodeT *pN1 = pSN->GetNextNode1(level);
	CSurfNodeT *pN2 = pSN->GetNextNode2(level);
	CSurfNodeT *pP1 = pSN->GetPrevNode1(level);
	CSurfNodeT *pP2 = pSN->GetPrevNode2(level);

	if(pN1 != NULL && pN2 != NULL)
	{
		CSurfNodeT *pN1N1 = pN1->GetNextNode1(level);
		CSurfNodeT *pN1N2 = pN1->GetNextNode2(level);
		CSurfNodeT *pN2N1 = pN2->GetNextNode1(level);
		CSurfNodeT *pN2N2 = pN2->GetNextNode2(level);
		CSurfNodeT *pN1P1 = pN1->GetPrevNode1(level);
		CSurfNodeT *pN2P2 = pN2->GetPrevNode2(level);

		if(pN1N2 != NULL && pN2N2 != NULL && pN2N1 != NULL)
		{
			CSurfNodeT *pN1N2N2 = pN1N2->GetNextNode2(level);
			CSurfNodeT *pN2N1P2 = pN2N1->GetPrevNode2(level);
			if(pN1N2N2 != NULL && pN2N1P2 != NULL)
			{
				CSurfNodeT *pN2N1P2N2 = pN2N1P2->GetNextNode2(level);

				if (pN2N1P2N2 != NULL && pN1N2N2 == pN2N1P2N2 && pN2N1P2N2 != pN2N1 && pN2N1P2N2 != pN2N2)
				{
					StoreSeptagone(pN1N2N2, pN2N1P2, pN1N2, pN2N1, pN1, pN2, pSN);		return true;
				}
			}
		}

		// next 2 are symetric (matches pentagones as well)
		if(pN1N2 != NULL && pN2N2 != NULL && pN2N1 != NULL)
		{
			CSurfNodeT *pN1N2P1 = pN1N2->GetPrevNode1(level);
			CSurfNodeT *pN2N1N1 = pN2N1->GetNextNode1(level);
			if(pN1N2P1 != NULL && pN2N1N1 != NULL)
			{
				CSurfNodeT *pN1N2P1N1 = pN1N2P1->GetNextNode1(level);

				if (pN1N2P1N1 != NULL && 
					pN2N1N1 == pN1N2P1N1 &&	pN1N2P1N1 != pN1N2 && pN1N2P1N1 != pN1N1)
				{
					StoreSeptagone(pN2N1N1, pN2N1, pN1N2P1, pN2, pN1N2, pSN, pN1);		return true;
				}
			}
		}

		// next 2 are symetric
		if (pN1N1 != NULL && pN2N1 != NULL && pN1N2 != NULL)
		{
			CSurfNodeT *pN1N2P1 = pN1N2->GetPrevNode1(level);
			CSurfNodeT *pN1N1N2 = pN1N1->GetNextNode2(level);
			CSurfNodeT *pN1N2N2 = pN1N2->GetNextNode2(level);
			if (pN1N2N2 != NULL && pN1N2P1 != NULL && pN1N1N2 != NULL)
			{
				CSurfNodeT *pN1N2P1N1 = pN1N2P1->GetNextNode1(level);
				CSurfNodeT *pN1N2P1N1N2 = pN1N2P1N1->GetNextNode2(level);
				if (pN1N2P1N1 != NULL && pN1N2P1N1N2 != NULL && 
					pN2N1 == pN1N2P1N1N2 && pN1N2P1N1N2 != pN1N1N2) // random image fails
				{
					ASSERT(pN1N2P1N1N2 != pN1N2N2);
					StoreSeptagone(pN2, pSN, pN2N1, pN1, pN1N2P1N1, pN1N2, pN1N2P1);		return true;
				}
			}
		}

		if (pN1N2 != NULL && pN2N1 != NULL && pN2N2 != NULL)
		{
			CSurfNodeT *pN2N2N1 = pN2N2->GetNextNode1(level);
			CSurfNodeT *pN2N1P2 = pN2N1->GetPrevNode2(level);
			CSurfNodeT *pN2N1N1 = pN2N1->GetNextNode1(level);
			if (pN2N1N1 != NULL && pN2N2N1 != NULL && pN2N1P2 != NULL)
			{
				CSurfNodeT *pN2N1P2N2 = pN2N1P2->GetNextNode2(level);
				CSurfNodeT *pN2N1P2N2N1 = pN2N1P2N2->GetNextNode1(level);
				if (pN2N1P2N2 != NULL && pN2N1P2N2N1 != NULL && 
					pN1N2 == pN2N1P2N2N1 && pN2N1P2N2N1 != pN2N2N1) // random image fails
				{
					ASSERT(pN2N1P2N2N1 != pN2N1N1);
					StoreSeptagone(pN1, pN1N2, pSN, pN2N1P2N2, pN2, pN2N1P2, pN2N1);		return true;
				}
			}
		}

		if (pN1N2 != NULL && pN2P2 != NULL && pN2N1 != NULL)
		{
			// next 2 are symetric
			CSurfNodeT *pN2P2N2 = pN2P2->GetNextNode2(level);
			CSurfNodeT *pN1N2N2 = pN1N2->GetNextNode2(level);
			if (pN1N2N2 != NULL && pN2P2N2 != NULL)
			{
				CSurfNodeT *pN2P2N2N1 = pN2P2N2->GetNextNode1(level);
				if (pN2P2N2N1 != NULL && pN1N2N2 == pN2P2N2N1 && pN2P2N2N1 != pN2N1)
				{
					// generates pentagons without this condition
					StoreSeptagone(pSN, pN1, pN2, pN1N2, pN2P2, pN1N2N2, pN2P2N2);		return true;
				}
			}
		}

		if (pN1P1 != NULL && pN2N1 != NULL)
		{
			CSurfNodeT *pN1P1N1 = pN1P1->GetNextNode1(level);
			CSurfNodeT *pN2N1N1 = pN2N1->GetNextNode1(level);
			if (pN1P1N1 != NULL && pN2N1N1 != NULL)
			{
				CSurfNodeT *pN1P1N1N2 = pN1P1N1->GetNextNode2(level);
				if (pN1P1N1N2 != NULL && pN2N1N1 == pN1P1N1N2 && pN1P1N1N2 != pN1N2)
				{
					// generates pentagons without this condition
					StoreSeptagone(pSN, pN1, pN2, pN1P1, pN2N1, pN1P1N1, pN2N1N1);		return true;
				}
			}
		}

		if (pN2N1 != NULL && pN1N2 != NULL)
		{
			CSurfNodeT *pN1N2N2 = pN1N2->GetNextNode2(level);
			CSurfNodeT *pN2N1N1 = pN2N1->GetNextNode1(level);
			if (pN1N2N2 != NULL && pN2N1N1 != NULL)
			{
				CSurfNodeT *pN2N1N1N1 = pN2N1N1->GetNextNode1(level);
				if (pN2N1N1N1 != NULL && pN1N2N2 == pN2N1N1N1 && pN1N2N2 != pN1P1)
				{
					StoreSeptagone(pN2N1, pN2, pN2N1N1, pSN, pN1N2N2, pN1, pN1N2);		return true;
				}
			}
		}

		if (pN1N2 != NULL && pN2N1 != NULL)
		{
			CSurfNodeT *pN1N2N2 = pN1N2->GetNextNode2(level);
			CSurfNodeT *pN2N1N1 = pN2N1->GetNextNode1(level);
			if (pN1N2N2 != NULL && pN2N1N1 != NULL)
			{
				CSurfNodeT *pN1N2N2N2 = pN1N2N2->GetNextNode2(level);
				if (pN1N2N2N2 != NULL && pN2N1N1 == pN1N2N2N2 && pN2N1N1 != pN2P2)
				{
					StoreSeptagone(pN1N2, pN1N2N2, pN1, pN2N1N1, pSN, pN2N1, pN2);		return true;
				}
			}
		}
	}
	return false;
}

bool CMGMesh::BuildHexagones(CSurfNodeT *pSN, int level)
{ 
	CSurfNodeT *pN1 = pSN->GetNextNode1(level);
	CSurfNodeT *pN2 = pSN->GetNextNode2(level);
	CSurfNodeT *pP1 = pSN->GetPrevNode1(level);
	CSurfNodeT *pP2 = pSN->GetPrevNode2(level);

	if (pN1 != NULL && pN2 != NULL)
	{

		CSurfNodeT *pN1P1 = pN1->GetPrevNode1(level);
		CSurfNodeT *pN2P2 = pN2->GetPrevNode2(level);

		if (pN1P1 != NULL && pN2P2 != NULL)
		{
			CSurfNodeT *pN1P1N1 = pN1P1->GetNextNode1(level);
			CSurfNodeT *pN2P2N2 = pN2P2->GetNextNode2(level);

			// generates flat hexagones
			if (pN1P1N1 != NULL && pN2P2N2 != NULL && pN1P1N1 == pN2P2N2 &&
				(!pSN->IsHexagonType1() || !pN1->IsHexagonType1() ||
				!pN2->IsHexagonType1() || !pN1P1->IsHexagonType1() ||
				!pN1P1N1->IsHexagonType1() || !pN2P2->IsHexagonType1()))
			{
				StoreHexagone1(pSN, pN1, pN2, pN1P1, pN2P2, pN1P1N1);//		do not return;
			}
		}
	}

	if (pN1 != NULL && pN2 != NULL)
	{
		CSurfNodeT *pN1N2 = pN1->GetNextNode2(level);
		CSurfNodeT *pN2N1 = pN2->GetNextNode1(level);

		if (pN1N2 != NULL && pN2N1 != NULL)
		{

			CSurfNodeT *pN1N2N2 = pN1N2->GetNextNode2(level);
			CSurfNodeT *pN2N1N1 = pN2N1->GetNextNode1(level);

			// Case 1 & 2: generates both planar and non-planar hexagones 
			if (pN1N2N2 != NULL && pN2N1N1 != NULL && pN1N2N2 == pN2N1N1)
			{
				StoreHexagone2(pSN, pN1, pN1N2, pN1N2N2, pN2N1, pN2);//		do not return;
			}
		}
	}

	if (pN1 != NULL && pN2 != NULL)
	{
		CSurfNodeT *pN1N2 = pN1->GetNextNode2(level);
		CSurfNodeT *pN2N1 = pN2->GetNextNode1(level);

		if (pN1N2 != NULL && pN2N1 != NULL)
		{
			CSurfNodeT *pN1N2P1 = pN1N2->GetPrevNode1(level);
			CSurfNodeT *pN2N1P2 = pN2N1->GetPrevNode2(level);
			// Case 2: split into triangles
			if (pN1N2P1 != NULL && pN2N1P2 != NULL && pN1N2P1 == pN2N1P2 && pN1N2P1 != pN2 && pN2N1P2 != pN1)
			{
				StoreHexagone2(pSN, pN1, pN1N2, pN1N2P1, pN2N1, pN2);	return true;
			}
		}
	}


	if (pP1 != NULL && pP2 != NULL)
	{
		CSurfNodeT *pP1N1 = pP1->GetNextNode1(level);
		CSurfNodeT *pP2N2 = pP2->GetNextNode2(level);

		if (pP1N1 != NULL && pP2N2 != NULL)
		{
			CSurfNodeT *pP1N1N2 = pP1N1->GetNextNode2(level);
			CSurfNodeT *pP2N2N1 = pP2N2->GetNextNode1(level);

			// Case 2: split into triangles
			if (pP2N2N1 != NULL && pP1N1N2 != NULL && pP2N2N1 == pP1N1N2)
			{
				StoreHexagone2(pSN, pP1, pP1N1, pP1N1N2, pP2N2, pP2);//		do not return;
			}
		}
	}


	if (pN1 != NULL && pN2 != NULL)
	{
		CSurfNodeT *pN1N2 = pN1->GetNextNode2(level);
		CSurfNodeT *pN2P2 = pN2->GetPrevNode2(level);

		if (pN1N2 != NULL && pN2P2 != NULL)
		{
			CSurfNodeT *pN1N2N2 = pN1N2->GetNextNode2(level);
			CSurfNodeT *pN2P2N2 = pN2P2->GetNextNode2(level);

			// Case 3: generates 2 trapez
			if (pN1N2N2 != NULL && pN2P2N2 != NULL && pN1N2N2 == pN2P2N2 && pN2 != pN2P2N2)
			{
				// generates rectangles without the condition
				StoreHexagone3(pSN, pN1, pN2, pN1N2, pN2P2, pN1N2N2);		return true;
			}
		}
	}

	if (pN1 != NULL && pN2 != NULL)
	{
		CSurfNodeT *pN2N1 = pN2->GetNextNode1(level);
		CSurfNodeT *pN1P1 = pN1->GetPrevNode1(level);

		if (pN1P1 != NULL && pN2N1 != NULL)
		{
			CSurfNodeT *pN2N1N1 = pN2N1->GetNextNode1(level);
			CSurfNodeT *pN1P1N1 = pN1P1->GetNextNode1(level);

			// Case 3: generates 2 trapez
			if (pN2N1N1 != NULL && pN1P1N1 != NULL && pN2N1N1 == pN1P1N1 && pN1 != pN1P1N1)
			{
				// generates rectangles without the condition
				StoreHexagone3(pSN, pN1, pN2, pN1P1, pN2N1, pN1P1N1);		return true;
			}
		}
	}

	if (pN1 != NULL && pN2 != NULL)
	{
		CSurfNodeT *pN1N2 = pN1->GetNextNode2(level);
		CSurfNodeT *pN2N1 = pN2->GetNextNode1(level);

		if (pN1N2 != NULL && pN2N1 != NULL)
		{
			CSurfNodeT *pN1N2N2 = pN1N2->GetNextNode2(level);

			if (pN1 != NULL && pN2 != NULL)
			{
				CSurfNodeT *pN1N2N2 = pN1N2->GetNextNode2(level);

				if (pN1N2N2 != NULL)
				{
					// Case 3: generates 2 trapez
					CSurfNodeT *pN1N2N2N2 = pN1N2N2->GetNextNode2(level);

					if (pN1N2N2N2 != NULL && pN1N2N2N2 == pN2N1)
					{
						StoreHexagone3(pN2, pSN, pN2N1, pN1, pN1N2N2, pN1N2);		return true;
					}
				}
			}
		}
	}

	if (pN1 != NULL && pN2 != NULL)
	{
		CSurfNodeT *pN1N2 = pN1->GetNextNode2(level);
		CSurfNodeT *pN2N1 = pN2->GetNextNode1(level);

		if (pN1N2 != NULL && pN2N1 != NULL)
		{
			CSurfNodeT *pN2N1N1 = pN2N1->GetNextNode1(level);
			if (pN2N1N1 != NULL)
			{
				// Case 3: generates 2 trapez
				CSurfNodeT *pN2N1N1N1 = pN2N1N1->GetNextNode1(level);

				if (pN2N1N1N1 != NULL && pN2N1N1N1 == pN1N2)
				{
					StoreHexagone3(pN1, pN1N2, pSN, pN2N1N1, pN2, pN2N1);		return true;
				}
			}
		}
	}

	return false;
}

bool CMGMesh::BuildPentagones(CSurfNodeT *pSN, int level)
{ 
	CSurfNodeT *pN1 = pSN->GetNextNode1(level);
	CSurfNodeT *pN2 = pSN->GetNextNode2(level);

	if (pN1 != NULL && pN2 != NULL)
	{
		CSurfNodeT *pN2N1 = pN2->GetNextNode1(level);
		CSurfNodeT *pN1P1 = pN1->GetPrevNode1(level);

		if (pN1P1 !=NULL)
		{
			CSurfNodeT *pN1P1N1 = pN1P1->GetNextNode1(level);

			if (pN1P1N1 != pN1 && pN2N1 != NULL && pN1P1N1 == pN2N1)
			{
				// generates triangles without condition
				StorePentagone(pSN, pN1, pN1P1, pN2N1, pN2);		return true;
			}
		}

		CSurfNodeT *pN1N2 = pN1->GetNextNode2(level);
		CSurfNodeT *pN2P2 = pN2->GetPrevNode2(level);

		if (pN2P2 != NULL)
		{
			CSurfNodeT *pN2P2N2 = pN2P2->GetNextNode2(level);

			if (pN2P2N2 != pN2 && pN1N2 != NULL && pN2P2N2 == pN1N2)
			{
				// generates triangles without condition
				StorePentagone(pSN, pN1, pN1N2, pN2P2, pN2);		return true;
			}
		}

		if (pN1N2 != NULL)
		{
			CSurfNodeT *pN1N2N2 = pN1N2->GetNextNode2(level);

			if (pN1N2N2 != NULL && pN2N1 != NULL && pN1N2N2 == pN2N1)
			{
				StorePentagone(pSN, pN1, pN1N2, pN2N1, pN2);		return true;
			}
		}

		if (pN2N1 != NULL)
		{
			CSurfNodeT *pN2N1N1 = pN2N1->GetNextNode1(level);

			if (pN2N1N1 != NULL && pN1N2 != NULL && pN2N1N1 == pN1N2)
			{
				StorePentagone(pSN, pN1, pN1N2, pN2N1, pN2);		return true;
			}
		}
	}

	return false;
}

bool CMGMesh::BuildRectangles(CSurfNodeT *pSN, int level)
{
	CSurfNodeT *pN1 = pSN->GetNextNode1(level);
	CSurfNodeT *pN2 = pSN->GetNextNode2(level);

	if (pN1 != NULL && pN2 != NULL)
	{
		CSurfNodeT *pN1N2 = pN1->GetNextNode2(level);
		CSurfNodeT *pN2N1 = pN2->GetNextNode1(level);

		if (pN1N2 != NULL && pN2N1 != NULL && pN1N2 == pN2N1)
		{
			pSN->IncCount();	pN1->IncCount();	pN1N2->IncCount();	pN2->IncCount();
			StoreRectangle(pSN, pN1, pN1N2, pN2);		return true;
		}

		CSurfNodeT *pN2P2 = pN2->GetPrevNode2(level);

		if (pN1N2 != NULL && pN2P2 != NULL && pN1N2 == pN2P2)
		{
			// detected by the hexagones
			pSN->IncCount();	pN1->IncCount();	pN1N2->IncCount();	pN2->IncCount();
			StoreRectangle(pSN, pN1, pN1N2, pN2);		return true;
		}

		CSurfNodeT *pN1P1 = pN1->GetPrevNode1(level);

		if (pN2N1 != NULL && pN1P1 != NULL && pN2N1 == pN1P1)
		{
			// detected by the hexagones
			pSN->IncCount();	pN1->IncCount();	pN1P1->IncCount();	pN2->IncCount();
			StoreRectangle(pSN, pN1, pN1P1, pN2);		return true;
		}
	}

	return false;
}

bool CMGMesh::BuildTriangles(CSurfNodeT *pSN, int level)
{
	CSurfNodeT *pN1 = pSN->GetNextNode1(level);
	CSurfNodeT *pN2 = pSN->GetNextNode2(level);

	if(pN1 != NULL)
	{
		CSurfNodeT *pN1N1 = pN1->GetNextNode1(level);
		if (pN1N1 != NULL)
		{
			CSurfNodeT *pN1N1N1 = pN1N1->GetNextNode1(level);
			if (pN1N1N1 != NULL)
			{

				if (pSN == pN1N1N1 && !(pSN->IsTriangleType1() &&
					pN1N1->IsTriangleType1() && pN1->IsTriangleType1()))
				{
					// 3 times the same triangles
					pSN->IncCount();					pN1N1->IncCount();					pN1->IncCount();
					pSN->SetTriangleType1();	pN1N1->SetTriangleType1();	pN1->SetTriangleType1();

					// the same node will generate other triangles
					StoreTriangle(pSN, pN1N1, pN1);		// do not return
				}
			}
		}
	}

	if(pN2 != NULL)
	{
		CSurfNodeT *pN2N2 = pN2->GetNextNode2(level);
		if (pN2N2 != NULL)
		{
			CSurfNodeT *pN2N2N2 = pN2N2->GetNextNode2(level);
			if (pN2N2N2 != NULL)
			{
				if (pSN == pN2N2N2 && !(pSN->IsTriangleType2() && 
					pN2N2->IsTriangleType2() && pN2->IsTriangleType2()))
				{
					// 3 times the same triangles
					pSN->IncCount();					pN2N2->IncCount();					pN2->IncCount();
					pSN->SetTriangleType2();	pN2N2->SetTriangleType2();	pN2->SetTriangleType2();

					// the same node will generate other triangles
					StoreTriangle(pSN, pN2, pN2N2);		// do not return
				}
			}
		}
	}

	if(pN1 != NULL)
	{
		CSurfNodeT *pN1N2 = pN1->GetNextNode2(level);

		if (pN1N2 != NULL && pN1N2 == pN2)
		{
			// detected by the pentagons
			pSN->IncCount();	pN1->IncCount();	pN2->IncCount();
			StoreTriangle(pSN, pN1, pN2);		return true;
		}
	}

	if(pN2 != NULL)
	{
		CSurfNodeT *pN2N1 = pN2->GetNextNode1(level);

		if (pN2N1 != NULL && pN1 == pN2N1)
		{
			// detected by the pentagons
			pSN->IncCount();	pN1->IncCount();	pN2->IncCount();
			StoreTriangle(pSN, pN1, pN2);		return true;
		}
	}

	return false;
}

void CMGMesh::BuildSmoothSurface(int level)
{
	if (level >= GetMaxNrLevels()) return;

	m_level = level;

	int u;
	for (u = 0; u < m_nrSlices.u; u++)
	{
		CSurfNodeList *pList = m_nodeStack[u];
		if (pList == NULL) continue;

		for (int v = 0; v < m_nrSlices.v; v++)
		{
			register CSurfNodeT *pSN = pList[v].GetFirstNode();
			for (; pSN != NULL; pSN = pSN->GetDownList())
			{
				if (pSN->GetLevel() > level)	continue;

				pSN->ResetCount();
				pSN->ResetTriangleType1();  pSN->ResetTriangleType2();
				pSN->ResetHexagonType1();
			}
		}
	}

	for (u = 0; u < m_nrSlices.u; u++)
	{
		CSurfNodeList *pList = m_nodeStack[u];
		if (pList == NULL) continue;

		for (int v = 0; v < m_nrSlices.v; v++)
		{
			register CSurfNodeT *pSN = pList[v].GetFirstNode();
			for (; pSN != NULL; pSN = pSN->GetDownList())
			{
				if (pSN->GetLevel() > level)	continue;

				// all polygons have been defined
				if (pSN->GetCount() == 4) continue;
				BuildTriangles(pSN, level);

				if (pSN->GetCount() == 4) continue;
				BuildRectangles(pSN, level);

				if (pSN->GetCount() == 4) continue;
				BuildPentagones(pSN, level);

				if (pSN->GetCount() == 4) continue;
				BuildHexagones(pSN, level);

				if (pSN->GetCount() == 4) continue;
				BuildSeptagones(pSN, level);
			}
		}
	}
}

void CMGMesh::BuildQuadrilateralSurface(int level, BYTE layer)
{
	if (level >= GetMaxNrLevels()) return;

	m_level = level;

	int u;
	for (u = 0; u < m_nrSlices.u; u++)
	{
		CSurfNodeList *pList = m_nodeStack[u];
		if (pList == NULL) continue;

		for (int v = 0; v < m_nrSlices.v; v++)
		{
			register CSurfNodeT *pSN = pList[v].GetFirstNode();
			for (; pSN != NULL; pSN = pSN->GetDownList())
			{
				if (pSN->GetLevel() > level)	continue;

				pSN->ResetCount();
				pSN->ResetTriangleType1();  pSN->ResetTriangleType2();
				pSN->ResetHexagonType1();
			}
		}
	}

	for (u = 0; u < m_nrSlices.u; u++)
	{
		CSurfNodeList *pList = m_nodeStack[u];
		if (pList == NULL) continue;

		for (int v = 0; v < m_nrSlices.v; v++)
		{
			register CSurfNodeT *pSN = pList[v].GetFirstNode();
			for (; pSN != NULL; pSN = pSN->GetDownList())
			{
				if (pSN->GetLevel() > level)	continue;

				// all polygons have been defined
				if (pSN->GetCount() == 4) continue;
				BuildRectangles(pSN, level);
			}
		}
	}
}
