#ifndef __QUEUE_H__
#define __QUEUE_H__

/**************************************************************************************
// Queue template
**************************************************************************************/
template <class TYPE, class ARG_TYPE = const TYPE&, int xGrow = 8> class cQueue
{
public:
	
	cQueue(int xSize = 8)
	{
		vector = NULL;
		vectorSize = 0;
		first = last = -1;
		if (xSize > 0)
			Grow(xSize);
	}

	~cQueue()
	{
		if (vector != NULL)
		{
			if (first > last)
			{
                for (; first < vectorSize; first++)
					(vector+first)->~TYPE();
                for (; last >= 0; last--)
					(vector+last)->~TYPE();
			}
			else if (first >= 0)
			{
                for (; first <= last; first++)
					(vector+first)->~TYPE();
			}
			delete[] (BYTE*)vector;
		}
	}

	void	Grow(int gr)
	{
		ASSERT(gr > 0);
		if (IsEmpty())
		{
			ASSERT(first == -1 && last == -1);
			delete[] (BYTE*)vector;
			vectorSize += gr;
			vector = (TYPE*)(new BYTE[vectorSize*sizeof(TYPE)]);
			return;
		}
		TYPE *tmp;
		tmp = (TYPE*)(new BYTE[(vectorSize + gr)*sizeof(TYPE)]);
		if (first > last)
		{
			memcpy(tmp, vector + first, (vectorSize - first) * sizeof(TYPE));
			memcpy(tmp + (vectorSize - first), vector, (last + 1) * sizeof(TYPE));
			last += vectorSize - first;
		}
		else
		{
			memcpy(tmp, vector + first, (last - first + 1) * sizeof(TYPE));
			last = last - first;
		}
		first = 0;
		vectorSize += gr;
		delete[] (BYTE*)vector;
		vector = tmp;
	}

	inline	TYPE*	GetData()
	{
		return vector;
	}

	inline void		AddHead(ARG_TYPE elem)
	{
		if (first > last)
		{
			if (last + 1 == first)
			{
				Grow(xGrow);
				first = vectorSize;
			}
		}
		else
		{
			if (first <= 0)
			{
				if (last + 1 >= vectorSize)
					Grow(xGrow);
				first = vectorSize;
			}
			if (last < 0)
				last = first - 1;
		}
		vector[--first] = elem;
	}

	inline void		AddTail(ARG_TYPE elem)
	{
		if (last >= first)
		{
			if (last + 1 >= vectorSize)
			{
				if (first <= 0)
					Grow(xGrow);
				else
					last = -1;
			}
			if (first < 0)
				first = 0;
		}
		else
		{
			if (last + 1 == first)
				Grow(xGrow);
		}
		vector[++last] = elem;
	}

	inline	TYPE&	GetHead() const
	{
		ASSERT(first >= 0 && last >= 0);
        return vector[first];
	}

	inline	TYPE&	GetTail() const
	{
		ASSERT(first >= 0 && last >= 0);
        return vector[last];
	}

	inline TYPE&	RemoveHead()
	{
		ASSERT(first >= 0 && last >= 0);
        TYPE& elem = vector[first];
		if (first > last)
		{
			if (++first >= vectorSize)
			{
				first = 0;
			}
		}
		else
		{
			if (++first > last)
			{
				first = last = -1;
			}
		}
		return elem;
	}

	inline TYPE&	RemoveTail()
	{
		ASSERT(first >= 0 && last >= 0);
        TYPE& elem = vector[last];
		if (last >= first)
		{
			if (--last < first)
			{
				first = last = -1;
			}
		}
		else
		{
			if (--last < 0)
			{
				last = vectorSize-1;
			}
		}
		return elem;
	}

	inline	int		GetSize() const
	{
		if (first > last) // circular
			return (vectorSize - (first - last) + 1);
		if (first < 0)
			return 0;
		return (last - first + 1);
	}

	inline	bool	IsEmpty() const
	{
		return (first < 0);
	}

	inline void Update(ARG_TYPE elem, int pos)
	{
		vector[pos] = elem;
	}

protected:
	int		first;
	int		last;
	int		vectorSize;
	TYPE*	vector;
};

#endif 