/***********************************************************************
This software module was originally developed by Ioan Alexandru Salomie
(Vrije Universiteit Brussel VUB) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. VUB retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Ioan Alexandru Salomie at iasalomi@etro.vub.ac.be.
----------------------------------------------------------------------*/

// Wavelet3D.h: interface for the CWaveletCodec class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_WAVELET3D_H__26A17B90_BBEF_4A66_8833_93501906C6AB__INCLUDED_)
#define AFX_WAVELET3D_H__26A17B90_BBEF_4A66_8833_93501906C6AB__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

////////////////////////////////////////////////////////////////////////////////////////////////
// Wavelet procedures: decomposition filter (-0.5, 1, -0.5), reconstruction filter (0.5, 1. 0.5)
////////////////////////////////////////////////////////////////////////////////////////////////

template<class TYPE>
class CWaveletCodec
{
public:
	CWaveletCodec(const CIPointUVW &dim, int cyclicMode, bool bAll);
	virtual ~CWaveletCodec();

	enum { CYCLIC_U = 1, CYCLIC_UV = 5 };

	bool IsCyclicU() const { return (m_cyclicMode == CYCLIC_U); }
	bool IsCyclicV() const { return (m_cyclicMode == CYCLIC_UV); }

	virtual void WaveletDec3D(CMatrix<TYPE>& data) = 0;
	virtual void WaveletRec3D(CMatrix<TYPE>& in, CMatrix<TYPE>& out) = 0;
	virtual void WaveletDec3D(CMatrix<TYPE>& dataX, CMatrix<TYPE>& dataY, CMatrix<TYPE>& dataZ) = 0;
	virtual void WaveletRec3D(CMatrix<TYPE>& inX,  CMatrix<TYPE>& inY,  CMatrix<TYPE>& inZ, CMatrix<TYPE>& outX, CMatrix<TYPE>& outY, CMatrix<TYPE>& outZ) = 0;

	const CIPointUVW &GetDecLevels() const { return m_decLevels; };

	static int	GetDecLevel(int sz);
	static int	GetResLevel(int sz);
	static void GetNrResolutionLevels(const CIPointUVW &sz, CIPointUVW &nrLevels, char &maxNrLevels);
	static void GetNrDecLevels(const CIPointUVW &sz, CIPointUVW &decLevels);
	static void GetNrResLevels(const CIPointUVW &sz, CIPointUVW &decLevels);
	static int  GetMaxDecLevel(const CIPointUVW &decLevels);
	static void	GetEfficientDim(const CIPointUVW &sz, CIPointUVW &newSz);

protected:
	void	GetDecLevel(int sz, int *&len, int &decLevel);

protected:
	int						m_cyclicMode;
	int						m_maxSz;
	int						m_maxDecLevel;
	CIPointUVW				m_sz;
	CIPointUVW				m_decLevels;


	TYPE					* m_src;
	TYPE					* m_srcX;
	TYPE					* m_srcY;
	TYPE					* m_srcZ;
	TYPE					* m_dst;
	CODEC_INTERNAL_TYPE		* m_dstX;
	CODEC_INTERNAL_TYPE		* m_dstY;
	CODEC_INTERNAL_TYPE		* m_dstZ;

	int						* m_lenX,		* m_lenY,		* m_lenZ;

protected:
	void	ComputeNonUniformFilters();
	void	InitFilterCoef(int decLevel, int *&coef1, int *&coef2, bool *&bSpecialFilter);
	void	ComputeFilterCoef(int decLevel, int *len, int *&coef1, int *&coef2, bool *&bSpecialFilter);

protected:
	int		* m_coefX1,	* m_coefY1,	* m_coefZ1;
	int		* m_coefX2, * m_coefY2, * m_coefZ2;
	bool	*m_bSpecialFilterX;
	bool	*m_bSpecialFilterY;
	bool	*m_bSpecialFilterZ;
};

template <class TYPE>
CWaveletCodec<TYPE>::CWaveletCodec(const CIPointUVW &sz, int cyclicMode, bool bAll)
{
	ASSERT(sz.u >= 1 && sz.v >= 1 && sz.w >= 1);

	m_sz = sz;

	m_src = m_srcX = m_srcY = m_srcZ = NULL;
	m_dst = NULL; m_dstX = m_dstY = m_dstZ = NULL;

	m_lenX = m_lenY = m_lenZ = NULL;

	m_cyclicMode = cyclicMode;

	// allocate intermediate buffers to compute the transform
	m_maxSz = max(m_sz.u, max(m_sz.v, m_sz.w));
	if (bAll)
	{
		m_srcX = new TYPE[m_maxSz];
		m_srcY = new TYPE[m_maxSz];
		m_srcZ = new TYPE[m_maxSz];
		m_dstX = new CODEC_INTERNAL_TYPE[m_maxSz];
		m_dstY = new CODEC_INTERNAL_TYPE[m_maxSz];
		m_dstZ = new CODEC_INTERNAL_TYPE[m_maxSz];
	}
	else
	{
		m_src = new TYPE[m_maxSz];
		m_dst = new TYPE[m_maxSz];
	}

	// compute max decomposition level to allocate internal buffers
	m_maxDecLevel = GetDecLevel(m_maxSz);

	// compute the nr of decomposition levels for each direction
	GetDecLevel(m_sz.u, m_lenX, m_decLevels.u);
	GetDecLevel(m_sz.v, m_lenY, m_decLevels.v);
	GetDecLevel(m_sz.w, m_lenZ, m_decLevels.w);

	m_coefX1 = m_coefY1 = m_coefZ1 = NULL;
	m_coefX2 = m_coefY2 = m_coefZ2 = NULL;
	m_bSpecialFilterX = m_bSpecialFilterY = m_bSpecialFilterZ = NULL;
}

template <class TYPE>
CWaveletCodec<TYPE>::~CWaveletCodec()
{
	delete []m_src;		m_src = NULL;
	delete []m_dst;		m_dst = NULL;

	delete []m_lenX;		delete []m_lenY;		delete []m_lenZ;

	m_lenX = m_lenY = m_lenZ = NULL;
}

template <class TYPE>
int CWaveletCodec<TYPE>::GetDecLevel(int sz)
{
	int decLevel = 0;
	while (sz >= MIN_DEC_SIZE)
	{
		sz = (sz + 1) >> 1;			++decLevel;
	}
	return decLevel;
}

template <class TYPE>
int CWaveletCodec<TYPE>::GetResLevel(int sz)
{
	int resLevel = 1;
	while (sz >= MIN_DEC_SIZE)
	{
		sz = (sz + 1) >> 1;			++resLevel;
	}
	return resLevel;
}

template <class TYPE>
void CWaveletCodec<TYPE>::GetDecLevel(int sz, int *&len, int &decLevel)
{
	len = new int[m_maxDecLevel + 1];

	decLevel = 0;
	len[0] = sz;
	while (sz >= MIN_DEC_SIZE)
	{
		sz = (sz + 1) >> 1;   len[++decLevel] = sz;
	}
}

template <class TYPE>
void CWaveletCodec<TYPE>::GetNrDecLevels(const CIPointUVW &sz, CIPointUVW &decLevels)
{
	decLevels.u = GetDecLevel(sz.u);
	decLevels.v = GetDecLevel(sz.v);
	decLevels.w = GetDecLevel(sz.w);
}

template <class TYPE>
void CWaveletCodec<TYPE>::GetNrResLevels(const CIPointUVW &sz, CIPointUVW &resLevels)
{
	resLevels.u = GetResLevel(sz.u);
	resLevels.v = GetResLevel(sz.v);
	resLevels.w = GetResLevel(sz.w);
}

template <class TYPE>
void CWaveletCodec<TYPE>::GetNrResolutionLevels(const CIPointUVW &sz, CIPointUVW &nrLevels, char &maxNrLevels)
{
	GetNrResLevels(sz, nrLevels);
	
	maxNrLevels = max(GetMaxDecLevel(nrLevels) - 2, 1);
	nrLevels.u = min(maxNrLevels, nrLevels.u);
	nrLevels.v = min(maxNrLevels, nrLevels.v);
	nrLevels.w = min(maxNrLevels, nrLevels.w);
}

template <class TYPE>
int CWaveletCodec<TYPE>::GetMaxDecLevel(const CIPointUVW &decLevels)
{
	return max(decLevels.u, max(decLevels.v, decLevels.w));
}

template <class TYPE>
void CWaveletCodec<TYPE>::GetEfficientDim(const CIPointUVW &sz, CIPointUVW &newSz)
{
	// the x size should not be even
	newSz = sz;
	if (sz.u % 2 == 0)	++newSz.u;
}

template <class TYPE>
void CWaveletCodec<TYPE>::InitFilterCoef(int decLevel, int *&coef1, int *&coef2, bool *&bSpecialFilter)
{
	coef1 = new int[decLevel];
	coef2 = new int[decLevel];
	bSpecialFilter = new bool[decLevel];

	for (int level = 0; level < decLevel; level++)
	{
		bSpecialFilter[level] = false;
		coef1[level] = 1;		coef2[level] = 2;
	}
}

template <class TYPE>
void CWaveletCodec<TYPE>::ComputeFilterCoef(int decLevel, int *len, int *&coef1, int *&coef2, bool *&bSpecialFilter)
{
	if (decLevel == 0) return;

	coef1 = new int[decLevel];
	coef2 = new int[decLevel];
	bSpecialFilter = new bool[decLevel];

	int k = 0, count = 0;
	bool bEven = false;

	for (int level = 0; level < decLevel; level++)
	{
		if (len[level] % 2 == 0)
		{
			if (!bEven)
			{
				bEven = true;
				bSpecialFilter[level] = true;
				coef1[level] = 1;			k = 1 << count;
				coef2[level] = 3;			++count;
			}
			else
			{
				bSpecialFilter[level] = true;
				coef1[level] = coef1[level - 1] + k;			k = 1 << count;
				coef2[level] = coef2[level - 1] + k * 2;	++count;
			}
		}
		else if (bEven)
		{
			bSpecialFilter[level] = true;
			coef1[level] = coef1[level - 1] + k;			k = 1 << count;
			coef2[level] = coef2[level - 1] + k;			++count;
		}
		else
		{
			bSpecialFilter[level] = false;
			coef1[level] = 1;		coef2[level] = 2;
		}
	}
}

template <class TYPE>
void CWaveletCodec<TYPE>::ComputeNonUniformFilters()
{
	if (((1 << m_decLevels.u) + 1) != m_sz.u ||
		((1 << m_decLevels.v) + 1) != m_sz.v ||
		((1 << m_decLevels.w) + 1) != m_sz.w)
	{
		ComputeFilterCoef(m_decLevels.u, m_lenX, m_coefX1, m_coefX2, m_bSpecialFilterX);
		ComputeFilterCoef(m_decLevels.v, m_lenY, m_coefY1, m_coefY2, m_bSpecialFilterY);
		ComputeFilterCoef(m_decLevels.w, m_lenZ, m_coefZ1, m_coefZ2, m_bSpecialFilterZ);
	}
	else
	{
		InitFilterCoef(m_decLevels.u, m_coefX1, m_coefX2, m_bSpecialFilterX);
		InitFilterCoef(m_decLevels.v, m_coefY1, m_coefY2, m_bSpecialFilterY);
		InitFilterCoef(m_decLevels.w, m_coefZ1, m_coefZ2, m_bSpecialFilterZ);
	}
}

#endif // !defined(AFX_WAVELET3D_H__26A17B90_BBEF_4A66_8833_93501906C6AB__INCLUDED_)
