/***********************************************************************
This software module was originally developed by Ioan Alexandru Salomie
(Vrije Universiteit Brussel VUB) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. VUB retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Ioan Alexandru Salomie at iasalomi@etro.vub.ac.be.
----------------------------------------------------------------------*/

// WaveletShort3D.h: interface for the CWaveletCodecShort class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_WAVELETSHORT3D_H__26A17B90_BBEF_4A66_8833_93501906C6AB__INCLUDED_)
#define AFX_WAVELETSHORT3D_H__26A17B90_BBEF_4A66_8833_93501906C6AB__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

////////////////////////////////////////////////////////////////////////////////////////////////
// Wavelet procedures: decomposition filter (-0.5, 1, -0.5), reconstruction filter (0.5, 1. 0.5)
////////////////////////////////////////////////////////////////////////////////////////////////

template<class TYPE>
class CWaveletCodecShort : public CWaveletCodec<TYPE>
{
public:
	CWaveletCodecShort(const CIPointUVW &dim, int cyclicMode, bool bAll);
	virtual ~CWaveletCodecShort();

	virtual void WaveletDec3D(CMatrix<TYPE>& data);
	virtual void WaveletRec3D(CMatrix<TYPE>& in, CMatrix<TYPE>& out);
	virtual void WaveletDec3D(CMatrix<TYPE>& dataX, CMatrix<TYPE>& dataY, CMatrix<TYPE>& dataZ);
	virtual void WaveletRec3D(CMatrix<TYPE>& inX,  CMatrix<TYPE>& inY,  CMatrix<TYPE>& inZ,
		CMatrix<TYPE>& outX, CMatrix<TYPE>& outY, CMatrix<TYPE>& outZ);
};

template <class TYPE>
CWaveletCodecShort<TYPE>::CWaveletCodecShort(const CIPointUVW &sz, int cyclicMode, bool bAll)
: CWaveletCodec<TYPE>(sz, cyclicMode, bAll)
{
	// compute the filters for filtering the last elements of "non-regular" sizes
	ComputeNonUniformFilters();
}

template <class TYPE>
CWaveletCodecShort<TYPE>::~CWaveletCodecShort()
{
	delete []m_coefX1;	delete []m_coefY1;	delete []m_coefZ1;
	delete []m_coefX2;	delete []m_coefY2;	delete []m_coefZ2;
	delete []m_bSpecialFilterX;	delete []m_bSpecialFilterY;	delete []m_bSpecialFilterZ;

	m_coefX1 = m_coefY1 = m_coefZ1 = NULL;
	m_coefX2 = m_coefY2 = m_coefZ2 = NULL;
	m_bSpecialFilterX = m_bSpecialFilterY = m_bSpecialFilterZ = NULL;
}

template<class TYPE>
void CWaveletCodecShort<TYPE>::WaveletDec3D(CMatrix<TYPE>& dataX, CMatrix<TYPE>& dataY, CMatrix<TYPE>& dataZ)
{
	ASSERT(m_sz.u <= dataX.dim1 && m_sz.v <= dataX.dim2 && m_sz.w <= dataX.dim3);

	int x,y,z;

	int Lprev_x = MIN_DEC_SIZE, Lapp_x = m_lenX[0];
	int Lprev_y = MIN_DEC_SIZE, Lapp_y = m_lenY[0];
	int Lprev_z = MIN_DEC_SIZE, Lapp_z = m_lenZ[0];

	TYPE c1, c2, c3, c4;

	TYPE ***volX = dataX.GetMatrix3D();
	TYPE ***volY = dataY.GetMatrix3D();
	TYPE ***volZ = dataZ.GetMatrix3D();
	for(int level = 0; level < m_maxDecLevel; level++)
	{
		{
			int nextLevel = level + 1;
			if (Lprev_z >= MIN_DEC_SIZE)
			{
				Lprev_z = Lapp_z;		Lapp_z = m_lenZ[nextLevel];
			}

			if (Lprev_y >= MIN_DEC_SIZE)
			{
				Lprev_y = Lapp_y;		Lapp_y = m_lenY[nextLevel];
			}

			if (Lprev_x >= MIN_DEC_SIZE)
			{
				Lprev_x = Lapp_x;		Lapp_x = m_lenX[nextLevel];
			}
		}

		// filtering along Z
		if (Lprev_z >= MIN_DEC_SIZE)
		{
			// if a previous size was even we need to update the filter coeficients
			if (Lprev_z % 2 == 0)
			{
				c2 = (float)m_coefZ1[level] / (float)m_coefZ2[level];		c1 = (TYPE)1.0 - c2;
				c4 = (TYPE)2.0 * c2;																		c3 = (TYPE)1.0 - c4;
			}
			else
			{
				c2 = (float)m_coefZ1[level] / (float)m_coefZ2[level];		c1 = (TYPE)1.0 - c2;
			}

			for (y = 0; y < Lprev_y ; y++)
			{
				for (x = 0; x < Lprev_x ; x++)
				{
					for (z = 0; z < Lprev_z; z++)
					{
						m_srcX[z] = volX[z][y][x];  
						m_srcY[z] = volY[z][y][x];
						m_srcZ[z] = volZ[z][y][x];
					}

					for (z = 0; z < Lprev_z-3; z++)
					{
						if (z % 2 == 0) //low(even pos)
						{
							m_dstX[z>>1] = m_srcX[z];
							m_dstY[z>>1] = m_srcY[z];
							m_dstZ[z>>1] = m_srcZ[z];
						}
						else //high(odd pos)
						{
							m_dstX[Lapp_z+(z>>1)] = ((TYPE)-0.5)*m_srcX[z-1] + m_srcX[z] + ((TYPE)-0.5)*m_srcX[z+1];
							m_dstY[Lapp_z+(z>>1)] = ((TYPE)-0.5)*m_srcY[z-1] + m_srcY[z] + ((TYPE)-0.5)*m_srcY[z+1];
							m_dstZ[Lapp_z+(z>>1)] = ((TYPE)-0.5)*m_srcZ[z-1] + m_srcZ[z] + ((TYPE)-0.5)*m_srcZ[z+1];
						}
					}

					// assymetric decomposition might be needed for elements (Lprev_z-3), and (Lprev_z-2)
					if (Lprev_z % 2)
					{
						// odd image size
						m_dstX[z>>1] = m_srcX[z];
						m_dstY[z>>1] = m_srcY[z];
						m_dstZ[z>>1] = m_srcZ[z];
						++z;
						m_dstX[Lapp_z+(z>>1)] = -c1*m_srcX[z-1] + m_srcX[z] + -c2*m_srcX[z+1];
						m_dstY[Lapp_z+(z>>1)] = -c1*m_srcY[z-1] + m_srcY[z] + -c2*m_srcY[z+1];
						m_dstZ[Lapp_z+(z>>1)] = -c1*m_srcZ[z-1] + m_srcZ[z] + -c2*m_srcZ[z+1];
						++z;
						m_dstX[z>>1] = m_srcX[z];
						m_dstY[z>>1] = m_srcY[z];
						m_dstZ[z>>1] = m_srcZ[z];
					}
					else
					{
						// even image size
						m_dstX[Lapp_z+(z>>1)] = -c1*m_srcX[z-1] + m_srcX[z] + -c2*m_srcX[z+2];
						m_dstY[Lapp_z+(z>>1)] = -c1*m_srcY[z-1] + m_srcY[z] + -c2*m_srcY[z+2];
						m_dstZ[Lapp_z+(z>>1)] = -c1*m_srcZ[z-1] + m_srcZ[z] + -c2*m_srcZ[z+2];
						++z;
						m_dstX[Lapp_z+(z>>1)] = -c3*m_srcX[z-2] + m_srcX[z] + -c4*m_srcX[z+1];
						m_dstY[Lapp_z+(z>>1)] = -c3*m_srcY[z-2] + m_srcY[z] + -c4*m_srcY[z+1];
						m_dstZ[Lapp_z+(z>>1)] = -c3*m_srcZ[z-2] + m_srcZ[z] + -c4*m_srcZ[z+1];
						++z;
						m_dstX[z>>1] = m_srcX[z];
						m_dstY[z>>1] = m_srcY[z];
						m_dstZ[z>>1] = m_srcZ[z];
					}

					for (z = 0; z < Lprev_z; z++)
					{
						volX[z][y][x] = m_dstX[z];
						volY[z][y][x] = m_dstY[z];
						volZ[z][y][x] = m_dstZ[z];
					}
				}
			}
		}

		// filtering along Y
		if (Lprev_y >= MIN_DEC_SIZE)
		{
			// if a previous size was even we need to update the filter coeficients
			if (Lprev_y % 2 == 0)
			{
				c2 = (float)m_coefY1[level] / (float)m_coefY2[level];		c1 = (TYPE)1.0 - c2;
				c4 = (TYPE)2.0 * c2;																		c3 = (TYPE)1.0 - c4;
			}
			else
			{
				c2 = (float)m_coefY1[level] / (float)m_coefY2[level];		c1 = (TYPE)1.0 - c2;
			}

			for (z = 0; z < Lprev_z; z++)
			{
				TYPE **sliceX = volX[z];
				TYPE **sliceY = volY[z];
				TYPE **sliceZ = volZ[z];
				for (x = 0; x < Lprev_x; x++)
				{
					for (y = 0; y < Lprev_y; y++)
					{
						m_srcX[y] = sliceX[y][x];
						m_srcY[y] = sliceY[y][x];
						m_srcZ[y] = sliceZ[y][x];
					}

					for (y = 0; y < Lprev_y - 3; y++)
					{
						if (y % 2 == 0) //low(even pos)
						{
							m_dstX[y>>1] = m_srcX[y];
							m_dstY[y>>1] = m_srcY[y];
							m_dstZ[y>>1] = m_srcZ[y];
						}
						else //high(odd pos)
						{
							m_dstX[Lapp_y+(y>>1)] = ((TYPE)-0.5)*m_srcX[y-1] + m_srcX[y] + ((TYPE)-0.5)*m_srcX[y+1];
							m_dstY[Lapp_y+(y>>1)] = ((TYPE)-0.5)*m_srcY[y-1] + m_srcY[y] + ((TYPE)-0.5)*m_srcY[y+1];
							m_dstZ[Lapp_y+(y>>1)] = ((TYPE)-0.5)*m_srcZ[y-1] + m_srcZ[y] + ((TYPE)-0.5)*m_srcZ[y+1];
						}
					}

					// assymetric decomposition might be needed for elements (Lprev_y-3), and (Lprev_y-2)
					if (Lprev_y % 2)
					{
						// odd image size
						m_dstX[y>>1] = m_srcX[y];
						m_dstY[y>>1] = m_srcY[y];
						m_dstZ[y>>1] = m_srcZ[y];
						++y;
						m_dstX[Lapp_y+(y>>1)] = -c1*m_srcX[y-1] + m_srcX[y] + -c2*m_srcX[y+1];
						m_dstY[Lapp_y+(y>>1)] = -c1*m_srcY[y-1] + m_srcY[y] + -c2*m_srcY[y+1];
						m_dstZ[Lapp_y+(y>>1)] = -c1*m_srcZ[y-1] + m_srcZ[y] + -c2*m_srcZ[y+1];
						++y;
						m_dstX[y>>1] = m_srcX[y];
						m_dstY[y>>1] = m_srcY[y];
						m_dstZ[y>>1] = m_srcZ[y];
					}
					else
					{
						// even image size
						m_dstX[Lapp_y+(y>>1)] = -c1*m_srcX[y-1] + m_srcX[y] + -c2*m_srcX[y+2];
						m_dstY[Lapp_y+(y>>1)] = -c1*m_srcY[y-1] + m_srcY[y] + -c2*m_srcY[y+2];
						m_dstZ[Lapp_y+(y>>1)] = -c1*m_srcZ[y-1] + m_srcZ[y] + -c2*m_srcZ[y+2];
						++y;
						m_dstX[Lapp_y+(y>>1)] = -c3*m_srcX[y-2] + m_srcX[y] + -c4*m_srcX[y+1];
						m_dstY[Lapp_y+(y>>1)] = -c3*m_srcY[y-2] + m_srcY[y] + -c4*m_srcY[y+1];
						m_dstZ[Lapp_y+(y>>1)] = -c3*m_srcZ[y-2] + m_srcZ[y] + -c4*m_srcZ[y+1];
						++y;
						m_dstX[y>>1] = m_srcX[y];
						m_dstY[y>>1] = m_srcY[y];
						m_dstZ[y>>1] = m_srcZ[y];
					}

					for (y = 0; y < Lprev_y; y++)
					{
						sliceX[y][x] = m_dstX[y];
						sliceY[y][x] = m_dstY[y];
						sliceZ[y][x] = m_dstZ[y];
					}
				}
			}
		}

		// filtering along X
		if (Lprev_x >= MIN_DEC_SIZE)
		{
			// if a previous size was even we need to update the filter coeficients
			if (Lprev_x % 2 == 0)
			{
				c2 = (float)m_coefX1[level] / (float)m_coefX2[level];		c1 = (TYPE)1.0 - c2;
				c4 = (TYPE)2.0 * c2;																		c3 = (TYPE)1.0 - c4;
			}
			else
			{
				c2 = (float)m_coefX1[level] / (float)m_coefX2[level];		c1 = (TYPE)1.0 - c2;
			}

			for (z = 0; z < Lprev_z; z++)
			{
				TYPE **sliceX = volX[z];
				TYPE **sliceY = volY[z];
				TYPE **sliceZ = volZ[z];
				for (y = 0; y < Lprev_y; y++)
				{
					TYPE *colX = sliceX[y];
					TYPE *colY = sliceY[y];
					TYPE *colZ = sliceZ[y];
					for (x = 0; x < Lprev_x - 3; x++)
					{
						if (x % 2 == 0) // low(even pos)
						{
							m_dstX[x>>1] = colX[x];
							m_dstY[x>>1] = colY[x];
							m_dstZ[x>>1] = colZ[x];
						}
						else // high(odd pos)
						{
							m_dstX[Lapp_x+(x>>1)] = ((TYPE)-0.5)*colX[x-1] + colX[x] + ((TYPE)-0.5)*colX[x+1];
							m_dstY[Lapp_x+(x>>1)] = ((TYPE)-0.5)*colY[x-1] + colY[x] + ((TYPE)-0.5)*colY[x+1];
							m_dstZ[Lapp_x+(x>>1)] = ((TYPE)-0.5)*colZ[x-1] + colZ[x] + ((TYPE)-0.5)*colZ[x+1];
						}
					}

					// assymetric decomposition might be needed for elements (Lprev_x-3), and (Lprev_x-2)
					if (Lprev_x % 2)
					{
						// odd image size
						m_dstX[x>>1] = colX[x];
						m_dstY[x>>1] = colY[x];
						m_dstZ[x>>1] = colZ[x];
						++x;
						m_dstX[Lapp_x+(x>>1)] = -c1*colX[x-1] + colX[x] + -c2*colX[x+1];
						m_dstY[Lapp_x+(x>>1)] = -c1*colY[x-1] + colY[x] + -c2*colY[x+1];
						m_dstZ[Lapp_x+(x>>1)] = -c1*colZ[x-1] + colZ[x] + -c2*colZ[x+1];
						++x;
						m_dstX[x>>1] = colX[x];
						m_dstY[x>>1] = colY[x];
						m_dstZ[x>>1] = colZ[x];
					}
					else
					{
						// even image size
						m_dstX[Lapp_x+(x>>1)] = -c1*colX[x-1] + colX[x] + -c2*colX[x+2];
						m_dstY[Lapp_x+(x>>1)] = -c1*colY[x-1] + colY[x] + -c2*colY[x+2];
						m_dstZ[Lapp_x+(x>>1)] = -c1*colZ[x-1] + colZ[x] + -c2*colZ[x+2];
						++x;
						m_dstX[Lapp_x+(x>>1)] = -c3*colX[x-2] + colX[x] + -c4*colX[x+1];
						m_dstY[Lapp_x+(x>>1)] = -c3*colY[x-2] + colY[x] + -c4*colY[x+1];
						m_dstZ[Lapp_x+(x>>1)] = -c3*colZ[x-2] + colZ[x] + -c4*colZ[x+1];
						++x;
						m_dstX[x>>1] = colX[x];
						m_dstY[x>>1] = colY[x];
						m_dstZ[x>>1] = colZ[x];
					}

					for (x = 0; x < Lprev_x; x++)
					{
						colX[x] = m_dstX[x];
						colY[x] = m_dstY[x];
						colZ[x] = m_dstZ[x];
					}
				}
			}
		}
	}
}

template<class TYPE>
void CWaveletCodecShort<TYPE>::WaveletRec3D(CMatrix<TYPE>& inX,  CMatrix<TYPE>& inY,  CMatrix<TYPE>& inZ,
											CMatrix<TYPE>& outX, CMatrix<TYPE>& outY, CMatrix<TYPE>& outZ)
{
	ASSERT(m_sz.u <= inX.dim1  && m_sz.v <= inX.dim2  && m_sz.w <= inX.dim3);
	ASSERT(m_sz.u <= outX.dim1 && m_sz.v <= outX.dim2 && m_sz.w <= outX.dim3);

	// copy into the output and keep the in intact
	outX = inX;   outY = inY;   outZ = inZ;

	int x,y,z,index;

	int decLevelU = m_maxDecLevel;
	int decLevelV = m_maxDecLevel;
	int decLevelW = m_maxDecLevel;

	int L_x, L_y, L_z;
	int Lapp_x, Lapp_y, Lapp_z;

	L_x = Lapp_x = m_lenX[m_decLevels.u];
	L_y = Lapp_y = m_lenY[m_decLevels.v];
	L_z = Lapp_z = m_lenZ[m_decLevels.w];

	TYPE c1, c2, c3, c4;

	TYPE ***volX = outX.GetMatrix3D();
	TYPE ***volY = outY.GetMatrix3D();
	TYPE ***volZ = outZ.GetMatrix3D();
	for (int level = m_maxDecLevel-1; level >= 0; level--)
	{
		if (--decLevelU < m_decLevels.u)
		{
			Lapp_x = L_x;			L_x = m_lenX[decLevelU];
		}

		if (--decLevelV < m_decLevels.v)
		{
			Lapp_y = L_y;			L_y = m_lenY[decLevelV];
		}

		if (--decLevelW < m_decLevels.w)
		{
			Lapp_z = L_z;			L_z = m_lenZ[decLevelW];
		}

		// back filtering along X
		if (decLevelU < m_decLevels.u)
		{
			// if a previous size was even we need to update the filter coeficients
			if (L_x % 2 == 0)
			{
				c2 = (float)m_coefX1[level] / (float)m_coefX2[level];		c1 = (TYPE)1.0 - c2;
				c4 = (TYPE)2.0 * c2;																		c3 = (TYPE)1.0 - c4;
			}
			else
			{
				c2 = (float)m_coefX1[level] / (float)m_coefX2[level];		c1 = (TYPE)1.0 - c2;
			}

			for (z = 0; z < L_z; z++)
			{
				TYPE  **sliceX = volX[z];
				TYPE  **sliceY = volY[z];
				TYPE  **sliceZ = volZ[z];
				for (y = 0; y < L_y; y++)
				{
					TYPE  *colX = sliceX[y];
					TYPE  *colY = sliceY[y];
					TYPE  *colZ = sliceZ[y];
					for (x = 0, index = 0; x < L_x-3; x++)
					{
						if (x % 2==0)
						{
							m_dstX[x] = colX[index];
							m_dstY[x] = colY[index];
							m_dstZ[x] = colZ[index];
						}
						else
						{
							m_dstX[x] = ((TYPE) 0.5)*colX[index] + colX[index+Lapp_x] + ((TYPE) 0.5)*colX[index+1];
							m_dstY[x] = ((TYPE) 0.5)*colY[index] + colY[index+Lapp_x] + ((TYPE) 0.5)*colY[index+1];
							m_dstZ[x] = ((TYPE) 0.5)*colZ[index] + colZ[index+Lapp_x] + ((TYPE) 0.5)*colZ[index+1];
							index++;
						}
					}

					// assymetric reconstruction might be needed for elements (Lprev_x-3), and (Lprev_x-2)
					if (L_x % 2)
					{
						// odd image size
						m_dstX[x] = colX[index];
						m_dstY[x] = colY[index];
						m_dstZ[x] = colZ[index];
						x++;
						m_dstX[x] = c1*colX[index] + colX[index+Lapp_x] + c2*colX[index+1];
						m_dstY[x] = c1*colY[index] + colY[index+Lapp_x] + c2*colY[index+1];
						m_dstZ[x] = c1*colZ[index] + colZ[index+Lapp_x] + c2*colZ[index+1];
						index++;		x++;
						m_dstX[x] = colX[index];
						m_dstY[x] = colY[index];
						m_dstZ[x] = colZ[index];
					}
					else
					{
						// even image size
						m_dstX[x] = c1*colX[index] + colX[index+Lapp_x] + c2*colX[index+1];
						m_dstY[x] = c1*colY[index] + colY[index+Lapp_x] + c2*colY[index+1];
						m_dstZ[x] = c1*colZ[index] + colZ[index+Lapp_x] + c2*colZ[index+1];
						index++;		x++;
						m_dstX[x] = c3*colX[index-1] + colX[index+Lapp_x] + c4*colX[index];
						m_dstY[x] = c3*colY[index-1] + colY[index+Lapp_x] + c4*colY[index];
						m_dstZ[x] = c3*colZ[index-1] + colZ[index+Lapp_x] + c4*colZ[index];
						x++;
						m_dstX[x] = colX[index];
						m_dstY[x] = colY[index];
						m_dstZ[x] = colZ[index];
					}

					for (x = 0; x < L_x; x++)
					{
						colX[x] = m_dstX[x];
						colY[x] = m_dstY[x];
						colZ[x] = m_dstZ[x];
					}
				}
			}
		}

		// back filtering along Y
		if (decLevelV < m_decLevels.v)
		{
			// if a previous size was even we need to update the filter coeficients
			if (L_y % 2 == 0)
			{
				c2 = (float)m_coefY1[level] / (float)m_coefY2[level];		c1 = (TYPE)1.0 - c2;
				c4 = (TYPE)2.0 * c2;																		c3 = (TYPE)1.0 - c4;
			}
			else
			{
				c2 = (float)m_coefY1[level] / (float)m_coefY2[level];		c1 = (TYPE)1.0 - c2;
			}

			for (z = 0; z < L_z; z++)
			{
				TYPE  **sliceX = volX[z];
				TYPE  **sliceY = volY[z];
				TYPE  **sliceZ = volZ[z];
				for (x = 0; x < L_x; x++)
				{
					for (y = 0, index = 0; y < L_y; y++)
					{
						m_srcX[y] = sliceX[y][x];
						m_srcY[y] = sliceY[y][x];
						m_srcZ[y] = sliceZ[y][x];
					}

					for (y = 0, index = 0; y < L_y-3; y++)
					{
						if (y % 2==0)
						{
							m_dstX[y] = m_srcX[index];
							m_dstY[y] = m_srcY[index];
							m_dstZ[y] = m_srcZ[index];
						}
						else
						{
							m_dstX[y] = ((TYPE) 0.5)*m_srcX[index] + m_srcX[index+Lapp_y] + ((TYPE) 0.5)*m_srcX[index+1];
							m_dstY[y] = ((TYPE) 0.5)*m_srcY[index] + m_srcY[index+Lapp_y] + ((TYPE) 0.5)*m_srcY[index+1];
							m_dstZ[y] = ((TYPE) 0.5)*m_srcZ[index] + m_srcZ[index+Lapp_y] + ((TYPE) 0.5)*m_srcZ[index+1];
							index++;
						}
					}

					// assymetric reconstruction might be needed for elements (Lprev_y-3), and (Lprev_y-2)
					if (L_y % 2)
					{
						// odd image size
						m_dstX[y] = m_srcX[index];
						m_dstY[y] = m_srcY[index];
						m_dstZ[y] = m_srcZ[index];
						y++;
						m_dstX[y] = c1*m_srcX[index] + m_srcX[index+Lapp_y] + c2*m_srcX[index+1];
						m_dstY[y] = c1*m_srcY[index] + m_srcY[index+Lapp_y] + c2*m_srcY[index+1];
						m_dstZ[y] = c1*m_srcZ[index] + m_srcZ[index+Lapp_y] + c2*m_srcZ[index+1];
						index++;		y++;
						m_dstX[y] = m_srcX[index];
						m_dstY[y] = m_srcY[index];
						m_dstZ[y] = m_srcZ[index];
					}
					else
					{
						// even image size
						m_dstX[y] = c1*m_srcX[index] + m_srcX[index+Lapp_y] + c2*m_srcX[index+1];
						m_dstY[y] = c1*m_srcY[index] + m_srcY[index+Lapp_y] + c2*m_srcY[index+1];
						m_dstZ[y] = c1*m_srcZ[index] + m_srcZ[index+Lapp_y] + c2*m_srcZ[index+1];
						index++;		y++;
						m_dstX[y] = c3*m_srcX[index-1] + m_srcX[index+Lapp_y] + c4*m_srcX[index];
						m_dstY[y] = c3*m_srcY[index-1] + m_srcY[index+Lapp_y] + c4*m_srcY[index];
						m_dstZ[y] = c3*m_srcZ[index-1] + m_srcZ[index+Lapp_y] + c4*m_srcZ[index];
						y++;
						m_dstX[y] = m_srcX[index];
						m_dstY[y] = m_srcY[index];
						m_dstZ[y] = m_srcZ[index];
					}

					for (y = 0, index = 0; y < L_y; y++)
					{
						sliceX[y][x] = m_dstX[y];
						sliceY[y][x] = m_dstY[y];
						sliceZ[y][x] = m_dstZ[y];
					}
				}
			}
		}

		// back filtering along Z
		if (decLevelW < m_decLevels.w)
		{
			// if a previous size was even we need to update the filter coeficients
			if (L_z % 2 == 0)
			{
				c2 = (float)m_coefZ1[level] / (float)m_coefZ2[level];		c1 = (TYPE)1.0 - c2;
				c4 = (TYPE)2.0 * c2;																		c3 = (TYPE)1.0 - c4;
			}
			else
			{
				c2 = (float)m_coefZ1[level] / (float)m_coefZ2[level];		c1 = (TYPE)1.0 - c2;
			}

			for (y = 0; y < L_y; y++)
			{
				for (x = 0; x < L_x; x++)
				{
					for (z = 0, index = 0; z < L_z; z++)
					{
						m_srcX[z] = volX[z][y][x];
						m_srcY[z] = volY[z][y][x];
						m_srcZ[z] = volZ[z][y][x];
					}

					for (z = 0, index = 0; z < L_z-3; z++)
					{
						if (z % 2==0)
						{
							m_dstX[z] = m_srcX[index];
							m_dstY[z] = m_srcY[index];
							m_dstZ[z] = m_srcZ[index];
						}
						else
						{
							m_dstX[z] = ((TYPE) 0.5)*m_srcX[index] + m_srcX[index+Lapp_z] + ((TYPE) 0.5)*m_srcX[index+1];
							m_dstY[z] = ((TYPE) 0.5)*m_srcY[index] + m_srcY[index+Lapp_z] + ((TYPE) 0.5)*m_srcY[index+1];
							m_dstZ[z] = ((TYPE) 0.5)*m_srcZ[index] + m_srcZ[index+Lapp_z] + ((TYPE) 0.5)*m_srcZ[index+1];
							index++;
						}
					}

					// assymetric reconstruction might be needed for elements (Lprev_z-3), and (Lprev_z-2)
					if (L_z % 2)
					{
						// odd image size
						m_dstX[z] = m_srcX[index];
						m_dstY[z] = m_srcY[index];
						m_dstZ[z] = m_srcZ[index];
						z++;
						m_dstX[z] = c1*m_srcX[index] + m_srcX[index+Lapp_z] + c2*m_srcX[index+1];
						m_dstY[z] = c1*m_srcY[index] + m_srcY[index+Lapp_z] + c2*m_srcY[index+1];
						m_dstZ[z] = c1*m_srcZ[index] + m_srcZ[index+Lapp_z] + c2*m_srcZ[index+1];
						index++;		z++;
						m_dstX[z] = m_srcX[index];
						m_dstY[z] = m_srcY[index];
						m_dstZ[z] = m_srcZ[index];
					}
					else
					{
						// even image size
						m_dstX[z] = c1*m_srcX[index] + m_srcX[index+Lapp_z] + c2*m_srcX[index+1];
						m_dstY[z] = c1*m_srcY[index] + m_srcY[index+Lapp_z] + c2*m_srcY[index+1];
						m_dstZ[z] = c1*m_srcZ[index] + m_srcZ[index+Lapp_z] + c2*m_srcZ[index+1];
						index++;		z++;
						m_dstX[z] = c3*m_srcX[index-1] + m_srcX[index+Lapp_z] + c4*m_srcX[index];
						m_dstY[z] = c3*m_srcY[index-1] + m_srcY[index+Lapp_z] + c4*m_srcY[index];
						m_dstZ[z] = c3*m_srcZ[index-1] + m_srcZ[index+Lapp_z] + c4*m_srcZ[index];
						z++;
						m_dstX[z] = m_srcX[index];
						m_dstY[z] = m_srcY[index];
						m_dstZ[z] = m_srcZ[index];
					}

					for (z = 0, index = 0; z < L_z; z++)
					{
						volX[z][y][x] = m_dstX[z];
						volY[z][y][x] = m_dstY[z];
						volZ[z][y][x] = m_dstZ[z];
					}
				}
			}
		}
	}
}

template<class TYPE>
void CWaveletCodecShort<TYPE>::WaveletDec3D(CMatrix<TYPE>& data)
{
	ASSERT(m_sz.u <= data.dim1 && m_sz.v <= data.dim2 && m_sz.w <= data.dim3);

	int x,y,z;

	int Lprev_x = MIN_DEC_SIZE, Lapp_x = m_lenX[0];
	int Lprev_y = MIN_DEC_SIZE, Lapp_y = m_lenY[0];
	int Lprev_z = MIN_DEC_SIZE, Lapp_z = m_lenZ[0];

	TYPE c1, c2, c3, c4;

	TYPE ***vol = data.GetMatrix3D();
	for(int level = 0; level < m_maxDecLevel; level++)
	{
		{
			int nextLevel = level + 1;
			if (Lprev_z >= MIN_DEC_SIZE)
			{
				Lprev_z = Lapp_z;		Lapp_z = m_lenZ[nextLevel];
			}

			if (Lprev_y >= MIN_DEC_SIZE)
			{
				Lprev_y = Lapp_y;		Lapp_y = m_lenY[nextLevel];
			}

			if (Lprev_x >= MIN_DEC_SIZE)
			{
				Lprev_x = Lapp_x;		Lapp_x = m_lenX[nextLevel];
			}
		}

		// filtering along Z
		if (Lprev_z >= MIN_DEC_SIZE)
		{
			// if a previous size was even we need to update the filter coeficients
			if (Lprev_z % 2 == 0)
			{
				c2 = (float)m_coefZ1[level] / (float)m_coefZ2[level];		c1 = (TYPE)1.0 - c2;
				c4 = (TYPE)2.0 * c2;																		c3 = (TYPE)1.0 - c4;
			}
			else
			{
				c2 = (float)m_coefZ1[level] / (float)m_coefZ2[level];		c1 = (TYPE)1.0 - c2;
			}

			for(y=0;y<Lprev_y;y++)
			{
				for(x=0;x<Lprev_x;x++)
				{
					for(z=0;z<Lprev_z;z++)
						m_src[z] = vol[z][y][x];

					for(z=0;z<Lprev_z-3;z++)
					{
						if(z % 2==0) //low(even pos)
							m_dst[z>>1] = m_src[z];
						else //high(odd pos)
							m_dst[Lapp_z+(z>>1)] = ((TYPE)-0.5)*m_src[z-1] + m_src[z] + ((TYPE)-0.5)*m_src[z+1];
					}

					// assymetric decomposition might be needed for elements (Lprev_z-3), and (Lprev_z-2)
					if (Lprev_z % 2)
					{
						// odd image size
						m_dst[z>>1] = m_src[z];
						++z;
						m_dst[Lapp_z+(z>>1)] = -c1*m_src[z-1] + m_src[z] + -c2*m_src[z+1];
						++z;
						m_dst[z>>1] = m_src[z];
					}
					else
					{
						// even image size
						m_dst[Lapp_z+(z>>1)] = -c1*m_src[z-1] + m_src[z] + -c2*m_src[z+2];
						++z;
						m_dst[Lapp_z+(z>>1)] = -c3*m_src[z-2] + m_src[z] + -c4*m_src[z+1];
						++z;
						m_dst[z>>1] = m_src[z];
					}

					for(z=0;z<Lprev_z;z++)
						vol[z][y][x] = m_dst[z];
				}
			}
		}

		// filtering along Y
		if (Lprev_y >= MIN_DEC_SIZE)
		{
			// if a previous size was even we need to update the filter coeficients
			if (Lprev_y % 2 == 0)
			{
				c2 = (float)m_coefY1[level] / (float)m_coefY2[level];		c1 = (TYPE)1.0 - c2;
				c4 = (TYPE)2.0 * c2;																		c3 = (TYPE)1.0 - c4;
			}
			else
			{
				c2 = (float)m_coefY1[level] / (float)m_coefY2[level];		c1 = (TYPE)1.0 - c2;
			}

			for(z=0;z<Lprev_z;z++)
			{
				TYPE **slice = vol[z];
				for(x=0;x<Lprev_x;x++)
				{
					for(y=0;y<Lprev_y;y++)
						m_src[y] = slice[y][x];

					for(y=0;y<Lprev_y - 3;y++)
					{
						if(y % 2==0) //low(even pos)
							m_dst[y>>1] = m_src[y];
						else //high(odd pos)
							m_dst[Lapp_y+(y>>1)] = ((TYPE)-0.5)*m_src[y-1] + m_src[y] + ((TYPE)-0.5)*m_src[y+1];
					}

					// assymetric decomposition might be needed for elements (Lprev_y-3), and (Lprev_y-2)
					if (Lprev_y % 2)
					{
						// odd image size
						m_dst[y>>1] = m_src[y];
						++y;
						m_dst[Lapp_y+(y>>1)] = -c1*m_src[y-1] + m_src[y] + -c2*m_src[y+1];
						++y;
						m_dst[y>>1] = m_src[y];
					}
					else
					{
						// even image size
						m_dst[Lapp_y+(y>>1)] = -c1*m_src[y-1] + m_src[y] + -c2*m_src[y+2];
						++y;
						m_dst[Lapp_y+(y>>1)] = -c3*m_src[y-2] + m_src[y] + -c4*m_src[y+1];
						++y;
						m_dst[y>>1] = m_src[y];
					}

					for(y=0;y<Lprev_y;y++)
						slice[y][x] = m_dst[y];
				}
			}
		}

		// filtering along X
		if (Lprev_x >= MIN_DEC_SIZE)
		{
			// if a previous size was even we need to update the filter coeficients
			if (Lprev_x % 2 == 0)
			{
				c2 = (float)m_coefX1[level] / (float)m_coefX2[level];		c1 = (TYPE)1.0 - c2;
				c4 = (TYPE)2.0 * c2;																		c3 = (TYPE)1.0 - c4;
			}
			else
			{
				c2 = (float)m_coefX1[level] / (float)m_coefX2[level];		c1 = (TYPE)1.0 - c2;
			}

			for(z=0;z<Lprev_z;z++)
			{
				TYPE **slice = vol[z];
				for(y=0;y<Lprev_y;y++)
				{
					TYPE *col = slice[y];

					for(x=0;x<Lprev_x - 3;x++)
					{
						if(x % 2==0) // low(even pos)
							m_dst[x>>1] = col[x];
						else // high(odd pos)
							m_dst[Lapp_x+(x>>1)] = ((TYPE)-0.5)*col[x-1] + col[x] + ((TYPE)-0.5)*col[x+1];
					}

					// assymetric decomposition might be needed for elements (Lprev_x-3), and (Lprev_x-2)
					if (Lprev_x % 2)
					{
						// odd image size
						m_dst[x>>1] = col[x];
						++x;
						m_dst[Lapp_x+(x>>1)] = -c1*col[x-1] + col[x] + -c2*col[x+1];
						++x;
						m_dst[x>>1] = col[x];
					}
					else
					{
						// even image size
						m_dst[Lapp_x+(x>>1)] = -c1*col[x-1] + col[x] + -c2*col[x+2];
						++x;
						m_dst[Lapp_x+(x>>1)] = -c3*col[x-2] + col[x] + -c4*col[x+1];
						++x;
						m_dst[x>>1] = col[x];
					}

					for(x=0;x<Lprev_x;x++)
						col[x] = m_dst[x];
				}
			}
		}
	}
}

template<class TYPE>
void CWaveletCodecShort<TYPE>::WaveletRec3D(CMatrix<TYPE>& in, CMatrix<TYPE>& out)
{
	ASSERT(m_sz.u <= in.dim1  && m_sz.v <= in.dim2  && m_sz.w <= in.dim3);
	ASSERT(m_sz.u <= out.dim1 && m_sz.v <= out.dim2 && m_sz.w <= out.dim3);

	// copy into the output and keep the in intact
	out = in;

	int x,y,z,index;

	int decLevelU = m_maxDecLevel;
	int decLevelV = m_maxDecLevel;
	int decLevelW = m_maxDecLevel;

	int L_x, L_y, L_z;
	int Lapp_x, Lapp_y, Lapp_z;

	L_x = Lapp_x = m_lenX[m_decLevels.u];
	L_y = Lapp_y = m_lenY[m_decLevels.v];
	L_z = Lapp_z = m_lenZ[m_decLevels.w];

	TYPE c1, c2, c3, c4;

	TYPE ***vol = out.GetMatrix3D();
	for (int level = m_maxDecLevel-1; level >= 0; level--)
	{
		if (--decLevelU < m_decLevels.u)
		{
			Lapp_x = L_x;			L_x = m_lenX[decLevelU];
		}

		if (--decLevelV < m_decLevels.v)
		{
			Lapp_y = L_y;			L_y = m_lenY[decLevelV];
		}

		if (--decLevelW < m_decLevels.w)
		{
			Lapp_z = L_z;			L_z = m_lenZ[decLevelW];
		}

		// back filtering along X
		if (decLevelU < m_decLevels.u)
		{
			// if a previous size was even we need to update the filter coeficients
			if (L_x % 2 == 0)
			{
				c2 = (float)m_coefX1[level] / (float)m_coefX2[level];		c1 = (TYPE)1.0 - c2;
				c4 = (TYPE)2.0 * c2;																		c3 = (TYPE)1.0 - c4;
			}
			else
			{
				c2 = (float)m_coefX1[level] / (float)m_coefX2[level];		c1 = (TYPE)1.0 - c2;
			}

			for(z=0;z<L_z;z++)
			{
				TYPE  **slice = vol[z];
				for(y=0;y<L_y;y++)
				{
					TYPE  *col  = slice[y];
					for(x=0,index=0;x<L_x-3;x++)
					{
						if(x % 2==0)
						{
							m_dst[x] = col[index];
						}
						else
						{
							m_dst[x] = ((TYPE) 0.5)*col[index] + col[index+Lapp_x] + ((TYPE) 0.5)*col[index+1];
							index++;
						}
					}

					// assymetric reconstruction might be needed for elements (Lprev_x-3), and (Lprev_x-2)
					if (L_x % 2)
					{
						// odd image size
						m_dst[x] = col[index];
						x++;
						m_dst[x] = c1*col[index] + col[index+Lapp_x] + c2*col[index+1];
						index++;		x++;
						m_dst[x] = col[index];
					}
					else
					{
						// even image size
						m_dst[x] = c1*col[index] + col[index+Lapp_x] + c2*col[index+1];
						index++;		x++;
						m_dst[x] = c3*col[index-1] + col[index+Lapp_x] + c4*col[index];
						x++;
						m_dst[x] = col[index];
					}

					for(x=0;x<L_x;x++)
						col[x] = m_dst[x];
				}
			}
		}

		// back filtering along Y
		if (decLevelV < m_decLevels.v)
		{
			// if a previous size was even we need to update the filter coeficients
			if (L_y % 2 == 0)
			{
				c2 = (float)m_coefY1[level] / (float)m_coefY2[level];		c1 = (TYPE)1.0 - c2;
				c4 = (TYPE)2.0 * c2;																		c3 = (TYPE)1.0 - c4;
			}
			else
			{
				c2 = (float)m_coefY1[level] / (float)m_coefY2[level];		c1 = (TYPE)1.0 - c2;
			}

			for(z=0;z<L_z;z++)
			{
				TYPE  **slice = vol[z];
				for(x=0;x<L_x;x++)
				{
					for(y=0,index=0;y<L_y;y++)
						m_src[y] = slice[y][x];

					for(y=0,index=0;y<L_y-3;y++)
					{
						if(y % 2==0)
						{
							m_dst[y] = m_src[index];
						}
						else
						{
							m_dst[y] = ((TYPE) 0.5)*m_src[index] + m_src[index+Lapp_y] + ((TYPE) 0.5)*m_src[index+1];
							index++;
						}
					}

					// assymetric reconstruction might be needed for elements (Lprev_y-3), and (Lprev_y-2)
					if (L_y % 2)
					{
						// odd image size
						m_dst[y] = m_src[index];
						y++;
						m_dst[y] = c1*m_src[index] + m_src[index+Lapp_y] + c2*m_src[index+1];
						index++;		y++;
						m_dst[y] = m_src[index];
					}
					else
					{
						// even image size
						m_dst[y] = c1*m_src[index] + m_src[index+Lapp_y] + c2*m_src[index+1];
						index++;		y++;
						m_dst[y] = c3*m_src[index-1] + m_src[index+Lapp_y] + c4*m_src[index];
						y++;
						m_dst[y] = m_src[index];
					}

					for(y=0,index=0;y<L_y;y++)
						slice[y][x] = m_dst[y];
				}
			}
		}

		// back filtering along Z
		if (decLevelW < m_decLevels.w)
		{
			// if a previous size was even we need to update the filter coeficients
			if (L_z % 2 == 0)
			{
				c2 = (float)m_coefZ1[level] / (float)m_coefZ2[level];		c1 = (TYPE)1.0 - c2;
				c4 = (TYPE)2.0 * c2;																		c3 = (TYPE)1.0 - c4;
			}
			else
			{
				c2 = (float)m_coefZ1[level] / (float)m_coefZ2[level];		c1 = (TYPE)1.0 - c2;
			}

			for(y=0;y<L_y;y++)
			{
				for(x=0;x<L_x;x++)
				{
					for(z=0,index=0;z<L_z;z++)
						m_src[z] = vol[z][y][x];

					for(z=0,index=0;z<L_z-3;z++)
					{
						if(z % 2==0)
						{
							m_dst[z] = m_src[index];
						}
						else
						{
							m_dst[z] = ((TYPE) 0.5)*m_src[index] + m_src[index+Lapp_z] + ((TYPE) 0.5)*m_src[index+1];
							index++;
						}
					}

					// assymetric reconstruction might be needed for elements (Lprev_z-3), and (Lprev_z-2)
					if (L_z % 2)
					{
						// odd image size
						m_dst[z] = m_src[index];
						z++;
						m_dst[z] = c1*m_src[index] + m_src[index+Lapp_z] + c2*m_src[index+1];
						index++;		z++;
						m_dst[z] = m_src[index];
					}
					else
					{
						// even image size
						m_dst[z] = c1*m_src[index] + m_src[index+Lapp_z] + c2*m_src[index+1];
						index++;		z++;
						m_dst[z] = c3*m_src[index-1] + m_src[index+Lapp_z] + c4*m_src[index];
						z++;
						m_dst[z] = m_src[index];
					}

					for(z=0,index=0;z<L_z;z++)
						vol[z][y][x] = m_dst[z];
				}
			}
		}
	}
}

#endif // !defined(AFX_WAVELETSHORT3D_H__26A17B90_BBEF_4A66_8833_93501906C6AB__INCLUDED_)
