/***********************************************************************
This software module was originally developed by Ioan Alexandru Salomie
(Vrije Universiteit Brussel VUB) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. VUB retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Ioan Alexandru Salomie at iasalomi@etro.vub.ac.be.
----------------------------------------------------------------------*/

// WaveletSmooth3D.h: interface for the CWaveletCodecSmooth class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_WAVELETSMOOTH3D_H__26A17B90_BBEF_4A66_8833_93501906C6AB__INCLUDED_)
#define AFX_WAVELETSMOOTH3D_H__26A17B90_BBEF_4A66_8833_93501906C6AB__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#define MIRRORING // Enable/Disable Wavelet Mirroring

////////////////////////////////////////////////////////////////////////////////////////////////
// Wavelet procedures: decomposition filter  (1/16, 0, -9/16, 1, -9/16, 0, 1/16),
//                     reconstruction filter (-1/16, 0, 9/16, 1, 9/16, 0, -1/16)
////////////////////////////////////////////////////////////////////////////////////////////////

template<class TYPE>
class CWaveletCodecSmooth : public CWaveletCodec<TYPE>
{
public:
	CWaveletCodecSmooth(const CIPointUVW &dim, int m_cyclicMode, bool bAll);
	virtual ~CWaveletCodecSmooth();

	virtual void WaveletDec3D(CMatrix<TYPE>& data) {};
	virtual void WaveletRec3D(CMatrix<TYPE>& in, CMatrix<TYPE>& out) {};
	virtual void WaveletDec3D(CMatrix<TYPE>& dataX, CMatrix<TYPE>& dataY, CMatrix<TYPE>& dataZ);
	virtual void WaveletRec3D(CMatrix<TYPE>& inX,  CMatrix<TYPE>& inY,  CMatrix<TYPE>& inZ,	CMatrix<TYPE>& outX, CMatrix<TYPE>& outY, CMatrix<TYPE>& outZ);

protected:
	void ComputeMirrorPointLowDec(TYPE *valX, TYPE *valY, TYPE *valZ, CODEC_INTERNAL_TYPE &pX, CODEC_INTERNAL_TYPE &pY, CODEC_INTERNAL_TYPE &pZ);
	void ComputeMirrorPointHighDec(TYPE *valX, TYPE *valY, TYPE *valZ, CODEC_INTERNAL_TYPE &pX, CODEC_INTERNAL_TYPE &pY, CODEC_INTERNAL_TYPE &pZ);
	void ComputeMirrorPointLowRec(TYPE *valX, TYPE *valY, TYPE *valZ, CODEC_INTERNAL_TYPE &pX, CODEC_INTERNAL_TYPE &pY, CODEC_INTERNAL_TYPE &pZ);
	void ComputeMirrorPointHighRec(TYPE *valX, TYPE *valY, TYPE *valZ, CODEC_INTERNAL_TYPE &pX, CODEC_INTERNAL_TYPE &pY, CODEC_INTERNAL_TYPE &pZ);
};

template <class TYPE>
CWaveletCodecSmooth<TYPE>::CWaveletCodecSmooth(const CIPointUVW &sz, int m_cyclicMode, bool bAll)
: CWaveletCodec<TYPE>(sz, m_cyclicMode, bAll)
{
	// compute the filters for filtering the last elements of "non-regular" sizes
	ComputeNonUniformFilters();
}

template <class TYPE>
CWaveletCodecSmooth<TYPE>::~CWaveletCodecSmooth()
{
	delete []m_coefX1;	delete []m_coefY1;	delete []m_coefZ1;
	delete []m_coefX2;	delete []m_coefY2;	delete []m_coefZ2;
	delete []m_bSpecialFilterX;	delete []m_bSpecialFilterY;	delete []m_bSpecialFilterZ;

	m_coefX1 = m_coefY1 = m_coefZ1 = NULL;
	m_coefX2 = m_coefY2 = m_coefZ2 = NULL;
	m_bSpecialFilterX = m_bSpecialFilterY = m_bSpecialFilterZ = NULL;
}

template<class TYPE>
void CWaveletCodecSmooth<TYPE>::ComputeMirrorPointLowDec(TYPE *valX, TYPE *valY, TYPE *valZ, CODEC_INTERNAL_TYPE &pX, CODEC_INTERNAL_TYPE &pY, CODEC_INTERNAL_TYPE &pZ)
{
#ifdef MIRRORING
	CODEC_INTERNAL_TYPE aX = valX[2] - valX[4];
	CODEC_INTERNAL_TYPE aY = valY[2] - valY[4];
	CODEC_INTERNAL_TYPE aZ = valZ[2] - valZ[4];

	CODEC_INTERNAL_TYPE bX = valX[0] - valX[2];
	CODEC_INTERNAL_TYPE bY = valY[0] - valY[2];
	CODEC_INTERNAL_TYPE bZ = valZ[0] - valZ[2];

	CODEC_INTERNAL_TYPE dist = bX*bX + bY*bY + bZ*bZ;
	if (dist > 0)
	{
		CODEC_INTERNAL_TYPE k = (aX*bX + aY*bY + aZ*bZ) / dist;

		pX = valX[4] + (2*k + 1) * bX;
		pY = valY[4] + (2*k + 1) * bY;
		pZ = valZ[4] + (2*k + 1) * bZ;
	}
	else
	{
		pX = valX[4] + 2*aX;
		pY = valY[4] + 2*aY;
		pZ = valZ[4] + 2*aZ;
	}
#else
	pX = valX[0];
	pY = valY[0];
	pZ = valZ[0];
#endif
}

template<class TYPE>
void CWaveletCodecSmooth<TYPE>::ComputeMirrorPointHighDec(TYPE *valX, TYPE *valY, TYPE *valZ, CODEC_INTERNAL_TYPE &pX, CODEC_INTERNAL_TYPE &pY, CODEC_INTERNAL_TYPE &pZ)
{
#ifdef MIRRORING
	CODEC_INTERNAL_TYPE aX = valX[2] - valX[0];
	CODEC_INTERNAL_TYPE aY = valY[2] - valY[0];
	CODEC_INTERNAL_TYPE aZ = valZ[2] - valZ[0];

	CODEC_INTERNAL_TYPE bX = valX[4] - valX[2];
	CODEC_INTERNAL_TYPE bY = valY[4] - valY[2];
	CODEC_INTERNAL_TYPE bZ = valZ[4] - valZ[2];

	CODEC_INTERNAL_TYPE dist = bX*bX + bY*bY + bZ*bZ;
	if (dist > 0)
	{
		CODEC_INTERNAL_TYPE k = (aX*bX + aY*bY + aZ*bZ) / dist;

		pX = valX[0] + (2*k + 1) * bX;
		pY = valY[0] + (2*k + 1) * bY;
		pZ = valZ[0] + (2*k + 1) * bZ;
	}
	else
	{
		pX = valX[0] + 2*aX;
		pY = valY[0] + 2*aY;
		pZ = valZ[0] + 2*aZ;
	}
#else
	pX = valX[4];
	pY = valY[4];
	pZ = valZ[4];
#endif
}

template<class TYPE>
void CWaveletCodecSmooth<TYPE>::ComputeMirrorPointLowRec(TYPE *valX, TYPE *valY, TYPE *valZ, CODEC_INTERNAL_TYPE &pX, CODEC_INTERNAL_TYPE &pY, CODEC_INTERNAL_TYPE &pZ)
{
#ifdef MIRRORING
	CODEC_INTERNAL_TYPE aX = valX[1] - valX[2];
	CODEC_INTERNAL_TYPE aY = valY[1] - valY[2];
	CODEC_INTERNAL_TYPE aZ = valZ[1] - valZ[2];

	CODEC_INTERNAL_TYPE bX = valX[0] - valX[1];
	CODEC_INTERNAL_TYPE bY = valY[0] - valY[1];
	CODEC_INTERNAL_TYPE bZ = valZ[0] - valZ[1];

	CODEC_INTERNAL_TYPE dist = bX*bX + bY*bY + bZ*bZ;
	if (dist > 0)
	{
		CODEC_INTERNAL_TYPE k = (aX*bX + aY*bY + aZ*bZ) / dist;

		pX = valX[2] + (2*k + 1) * bX;
		pY = valY[2] + (2*k + 1) * bY;
		pZ = valZ[2] + (2*k + 1) * bZ;
	}
	else
	{
		pX = valX[2] + 2*aX;
		pY = valY[2] + 2*aY;
		pZ = valZ[2] + 2*aZ;
	}
#else
	pX = valX[0];
	pY = valY[0];
	pZ = valZ[0];
#endif
}

template<class TYPE>
void CWaveletCodecSmooth<TYPE>::ComputeMirrorPointHighRec(TYPE *valX, TYPE *valY, TYPE *valZ, CODEC_INTERNAL_TYPE &pX, CODEC_INTERNAL_TYPE &pY, CODEC_INTERNAL_TYPE &pZ)
{
#ifdef MIRRORING
	CODEC_INTERNAL_TYPE aX = valX[1] - valX[0];
	CODEC_INTERNAL_TYPE aY = valY[1] - valY[0];
	CODEC_INTERNAL_TYPE aZ = valZ[1] - valZ[0];

	CODEC_INTERNAL_TYPE bX = valX[2] - valX[1];
	CODEC_INTERNAL_TYPE bY = valY[2] - valY[1];
	CODEC_INTERNAL_TYPE bZ = valZ[2] - valZ[1];

	CODEC_INTERNAL_TYPE dist = bX*bX + bY*bY + bZ*bZ;
	if (dist > 0)
	{
		CODEC_INTERNAL_TYPE k = (aX*bX + aY*bY + aZ*bZ) / dist;

		pX = valX[0] + (2*k + 1) * bX;
		pY = valY[0] + (2*k + 1) * bY;
		pZ = valZ[0] + (2*k + 1) * bZ;
	}
	else
	{
		pX = valX[0] + 2*aX;
		pY = valY[0] + 2*aY;
		pZ = valZ[0] + 2*aZ;
	}
#else
	pX = valX[2];
	pY = valY[2];
	pZ = valZ[2];
#endif
}

template<class TYPE>
void CWaveletCodecSmooth<TYPE>::WaveletDec3D(CMatrix<TYPE>& dataX, CMatrix<TYPE>& dataY, CMatrix<TYPE>& dataZ)
{
	ASSERT(m_sz.u <= dataX.dim1 && m_sz.v <= dataX.dim2 && m_sz.w <= dataX.dim3);

	int x,y,z;
	CODEC_INTERNAL_TYPE pX,pY,pZ;

	int Lprev_x = MIN_DEC_SIZE, Lapp_x = m_lenX[0];
	int Lprev_y = MIN_DEC_SIZE, Lapp_y = m_lenY[0];
	int Lprev_z = MIN_DEC_SIZE, Lapp_z = m_lenZ[0];

	TYPE c1, c2, c3, c4;
	const TYPE c14 = (TYPE) 1.0/16.0;
	const TYPE c23 = (TYPE)-9.0/16.0;

	TYPE ***volX = dataX.GetMatrix3D();
	TYPE ***volY = dataY.GetMatrix3D();
	TYPE ***volZ = dataZ.GetMatrix3D();
	for(int level = 0; level < m_maxDecLevel; level++)
	{
		{
			int nextLevel = level + 1;
			if (Lprev_z >= MIN_DEC_SIZE)
			{
				Lprev_z = Lapp_z;		Lapp_z = m_lenZ[nextLevel];
			}

			if (Lprev_y >= MIN_DEC_SIZE)
			{
				Lprev_y = Lapp_y;		Lapp_y = m_lenY[nextLevel];
			}

			if (Lprev_x >= MIN_DEC_SIZE)
			{
				Lprev_x = Lapp_x;		Lapp_x = m_lenX[nextLevel];
			}
		}

		// filtering along Z
		if (Lprev_z >= MIN_DEC_SIZE)
		{
			// if a previous size was even we need to update the filter coeficients
			if  (m_bSpecialFilterZ[level])
			{
				if (Lprev_z % 2 == 0)
				{
					c2 = (float)m_coefZ1[level] / (float)m_coefZ2[level];		c1 = (TYPE)1.0 - c2;
					c4 = (TYPE)2.0 * c2;										c3 = (TYPE)1.0 - c4;
				}
				else
				{
					c2 = (float)m_coefZ1[level] / (float)m_coefZ2[level];		c1 = (TYPE)1.0 - c2;
				}

				for (y = 0; y < Lprev_y ; y++)
				{
					for (x = 0; x < Lprev_x ; x++)
					{
						for (z = 0; z < Lprev_z; z++)
						{
							m_srcX[z] = volX[z][y][x];  
							m_srcY[z] = volY[z][y][x];
							m_srcZ[z] = volZ[z][y][x];
						}

						for (z = 0; z < Lprev_z-3; z++)
						{
							if (z % 2 == 0) //low(even pos)
							{
								m_dstX[z>>1] = m_srcX[z];
								m_dstY[z>>1] = m_srcY[z];
								m_dstZ[z>>1] = m_srcZ[z];
							}
							else //high(odd pos)
							{
								m_dstX[Lapp_z+(z>>1)] = ((TYPE)-0.5)*m_srcX[z-1] + m_srcX[z] + ((TYPE)-0.5)*m_srcX[z+1];
								m_dstY[Lapp_z+(z>>1)] = ((TYPE)-0.5)*m_srcY[z-1] + m_srcY[z] + ((TYPE)-0.5)*m_srcY[z+1];
								m_dstZ[Lapp_z+(z>>1)] = ((TYPE)-0.5)*m_srcZ[z-1] + m_srcZ[z] + ((TYPE)-0.5)*m_srcZ[z+1];
							}
						}

						// assymetric decomposition might be needed for elements (Lprev_z-3), and (Lprev_z-2)
						if (Lprev_z % 2)
						{
							// odd image size
							m_dstX[z>>1] = m_srcX[z];
							m_dstY[z>>1] = m_srcY[z];
							m_dstZ[z>>1] = m_srcZ[z];
							++z;
							m_dstX[Lapp_z+(z>>1)] = -c1*m_srcX[z-1] + m_srcX[z] + -c2*m_srcX[z+1];
							m_dstY[Lapp_z+(z>>1)] = -c1*m_srcY[z-1] + m_srcY[z] + -c2*m_srcY[z+1];
							m_dstZ[Lapp_z+(z>>1)] = -c1*m_srcZ[z-1] + m_srcZ[z] + -c2*m_srcZ[z+1];
							++z;
							m_dstX[z>>1] = m_srcX[z];
							m_dstY[z>>1] = m_srcY[z];
							m_dstZ[z>>1] = m_srcZ[z];
						}
						else
						{
							// even image size
							m_dstX[Lapp_z+(z>>1)] = -c1*m_srcX[z-1] + m_srcX[z] + -c2*m_srcX[z+2];
							m_dstY[Lapp_z+(z>>1)] = -c1*m_srcY[z-1] + m_srcY[z] + -c2*m_srcY[z+2];
							m_dstZ[Lapp_z+(z>>1)] = -c1*m_srcZ[z-1] + m_srcZ[z] + -c2*m_srcZ[z+2];
							++z;
							m_dstX[Lapp_z+(z>>1)] = -c3*m_srcX[z-2] + m_srcX[z] + -c4*m_srcX[z+1];
							m_dstY[Lapp_z+(z>>1)] = -c3*m_srcY[z-2] + m_srcY[z] + -c4*m_srcY[z+1];
							m_dstZ[Lapp_z+(z>>1)] = -c3*m_srcZ[z-2] + m_srcZ[z] + -c4*m_srcZ[z+1];
							++z;
							m_dstX[z>>1] = m_srcX[z];
							m_dstY[z>>1] = m_srcY[z];
							m_dstZ[z>>1] = m_srcZ[z];
						}

						for (z = 0; z < Lprev_z; z++)
						{
							volX[z][y][x] = m_dstX[z];
							volY[z][y][x] = m_dstY[z];
							volZ[z][y][x] = m_dstZ[z];
						}
					}
				}
			}
			else
			{
				for (y = 0; y < Lprev_y ; y++)
				{
					for (x = 0; x < Lprev_x ; x++)
					{
						for (z = 0; z < Lprev_z; z++)
						{
							m_srcX[z] = volX[z][y][x];  
							m_srcY[z] = volY[z][y][x];
							m_srcZ[z] = volZ[z][y][x];
						}

						// handle the borders
						{
							// copy first and last values (even pos)
							m_dstX[0] = m_srcX[0];
							m_dstY[0] = m_srcY[0];
							m_dstZ[0] = m_srcZ[0];

							z = Lprev_z-1;
							m_dstX[z>>1] = m_srcX[z];
							m_dstY[z>>1] = m_srcY[z];
							m_dstZ[z>>1] = m_srcZ[z];
						}

						if (Lprev_z == 3)
						{
							// use short filter -0.5, 1, -0.5
							m_dstX[2] = ((TYPE)-0.5)*m_srcX[0] + m_srcX[1] + ((TYPE)-0.5)*m_srcX[2];
							m_dstY[2] = ((TYPE)-0.5)*m_srcY[0] + m_srcY[1] + ((TYPE)-0.5)*m_srcY[2];
							m_dstZ[2] = ((TYPE)-0.5)*m_srcZ[0] + m_srcZ[1] + ((TYPE)-0.5)*m_srcZ[2];
						}
						else
						{
							// compute border mirror points and filter
							ASSERT(Lprev_z >= 5);

							ComputeMirrorPointLowDec(m_srcX, m_srcY, m_srcZ, pX, pY, pZ);
							m_dstX[Lapp_z] = c14*(pX + m_srcX[4]) + m_srcX[1] + c23*(m_srcX[0] + m_srcX[2]);
							m_dstY[Lapp_z] = c14*(pY + m_srcY[4]) + m_srcY[1] + c23*(m_srcY[0] + m_srcY[2]);
							m_dstZ[Lapp_z] = c14*(pZ + m_srcZ[4]) + m_srcZ[1] + c23*(m_srcZ[0] + m_srcZ[2]);

							z = Lprev_z-5;
							ComputeMirrorPointHighDec(&m_srcX[z], &m_srcY[z], &m_srcZ[z], pX, pY, pZ);
							m_dstX[Lprev_z-1] = c14*(m_srcX[z] + pX) + m_srcX[z+3] + c23*(m_srcX[z+2] + m_srcX[z+4]);
							m_dstY[Lprev_z-1] = c14*(m_srcY[z] + pY) + m_srcY[z+3] + c23*(m_srcY[z+2] + m_srcY[z+4]);
							m_dstZ[Lprev_z-1] = c14*(m_srcZ[z] + pZ) + m_srcZ[z+3] + c23*(m_srcZ[z+2] + m_srcZ[z+4]);
						}

						// handle the rest
						for (z = 2; z < Lprev_z - 2; z++)
						{
							if (z % 2 == 0) //low(even pos)
							{
								m_dstX[z>>1] = m_srcX[z];
								m_dstY[z>>1] = m_srcY[z];
								m_dstZ[z>>1] = m_srcZ[z];
							}
							else //high(odd pos)
							{
								m_dstX[Lapp_z+(z>>1)] = c14*(m_srcX[z-3] + m_srcX[z+3]) + m_srcX[z] + c23*(m_srcX[z-1] + m_srcX[z+1]);
								m_dstY[Lapp_z+(z>>1)] = c14*(m_srcY[z-3] + m_srcY[z+3]) + m_srcY[z] + c23*(m_srcY[z-1] + m_srcY[z+1]);
								m_dstZ[Lapp_z+(z>>1)] = c14*(m_srcZ[z-3] + m_srcZ[z+3]) + m_srcZ[z] + c23*(m_srcZ[z-1] + m_srcZ[z+1]);
							}
						}

						for (z = 0; z < Lprev_z; z++)
						{
							volX[z][y][x] = m_dstX[z];
							volY[z][y][x] = m_dstY[z];
							volZ[z][y][x] = m_dstZ[z];
						}
					}
				}
			}
		}

		// filtering along Y
		if (Lprev_y >= MIN_DEC_SIZE)
		{
			// if a previous size was even we need to update the filter coeficients
			if  (m_bSpecialFilterY[level])
			{
				if (Lprev_y % 2 == 0)
				{
					c2 = (float)m_coefY1[level] / (float)m_coefY2[level];		c1 = (TYPE)1.0 - c2;
					c4 = (TYPE)2.0 * c2;										c3 = (TYPE)1.0 - c4;
				}
				else
				{
					c2 = (float)m_coefY1[level] / (float)m_coefY2[level];		c1 = (TYPE)1.0 - c2;
				}

				for (z = 0; z < Lprev_z; z++)
				{
					TYPE **sliceX = volX[z];
					TYPE **sliceY = volY[z];
					TYPE **sliceZ = volZ[z];
					for (x = 0; x < Lprev_x; x++)
					{
						for (y = 0; y < Lprev_y; y++)
						{
							m_srcX[y] = sliceX[y][x];
							m_srcY[y] = sliceY[y][x];
							m_srcZ[y] = sliceZ[y][x];
						}

						for (y = 0; y < Lprev_y - 3; y++)
						{
							if (y % 2 == 0) //low(even pos)
							{
								m_dstX[y>>1] = m_srcX[y];
								m_dstY[y>>1] = m_srcY[y];
								m_dstZ[y>>1] = m_srcZ[y];
							}
							else //high(odd pos)
							{
								m_dstX[Lapp_y+(y>>1)] = ((TYPE)-0.5)*m_srcX[y-1] + m_srcX[y] + ((TYPE)-0.5)*m_srcX[y+1];
								m_dstY[Lapp_y+(y>>1)] = ((TYPE)-0.5)*m_srcY[y-1] + m_srcY[y] + ((TYPE)-0.5)*m_srcY[y+1];
								m_dstZ[Lapp_y+(y>>1)] = ((TYPE)-0.5)*m_srcZ[y-1] + m_srcZ[y] + ((TYPE)-0.5)*m_srcZ[y+1];
							}
						}

						// assymetric decomposition might be needed for elements (Lprev_y-3), and (Lprev_y-2)
						if (Lprev_y % 2)
						{
							// odd image size
							m_dstX[y>>1] = m_srcX[y];
							m_dstY[y>>1] = m_srcY[y];
							m_dstZ[y>>1] = m_srcZ[y];
							++y;
							m_dstX[Lapp_y+(y>>1)] = -c1*m_srcX[y-1] + m_srcX[y] + -c2*m_srcX[y+1];
							m_dstY[Lapp_y+(y>>1)] = -c1*m_srcY[y-1] + m_srcY[y] + -c2*m_srcY[y+1];
							m_dstZ[Lapp_y+(y>>1)] = -c1*m_srcZ[y-1] + m_srcZ[y] + -c2*m_srcZ[y+1];
							++y;
							m_dstX[y>>1] = m_srcX[y];
							m_dstY[y>>1] = m_srcY[y];
							m_dstZ[y>>1] = m_srcZ[y];
						}
						else
						{
							// even image size
							m_dstX[Lapp_y+(y>>1)] = -c1*m_srcX[y-1] + m_srcX[y] + -c2*m_srcX[y+2];
							m_dstY[Lapp_y+(y>>1)] = -c1*m_srcY[y-1] + m_srcY[y] + -c2*m_srcY[y+2];
							m_dstZ[Lapp_y+(y>>1)] = -c1*m_srcZ[y-1] + m_srcZ[y] + -c2*m_srcZ[y+2];
							++y;
							m_dstX[Lapp_y+(y>>1)] = -c3*m_srcX[y-2] + m_srcX[y] + -c4*m_srcX[y+1];
							m_dstY[Lapp_y+(y>>1)] = -c3*m_srcY[y-2] + m_srcY[y] + -c4*m_srcY[y+1];
							m_dstZ[Lapp_y+(y>>1)] = -c3*m_srcZ[y-2] + m_srcZ[y] + -c4*m_srcZ[y+1];
							++y;
							m_dstX[y>>1] = m_srcX[y];
							m_dstY[y>>1] = m_srcY[y];
							m_dstZ[y>>1] = m_srcZ[y];
						}

						for (y = 0; y < Lprev_y; y++)
						{
							sliceX[y][x] = m_dstX[y];
							sliceY[y][x] = m_dstY[y];
							sliceZ[y][x] = m_dstZ[y];
						}
					}
				}
			}
			else
			{
				for (z = 0; z < Lprev_z; z++)
				{
					TYPE **sliceX = volX[z];
					TYPE **sliceY = volY[z];
					TYPE **sliceZ = volZ[z];
					for (x = 0; x < Lprev_x; x++)
					{
						for (y = 0; y < Lprev_y; y++)
						{
							m_srcX[y] = sliceX[y][x];
							m_srcY[y] = sliceY[y][x];
							m_srcZ[y] = sliceZ[y][x];
						}

						// handle the borders
						{
							// copy first and last values (even pos)
							m_dstX[0] = m_srcX[0];
							m_dstY[0] = m_srcY[0];
							m_dstZ[0] = m_srcZ[0];

							y = Lprev_y-1;
							m_dstX[y>>1] = m_srcX[y];
							m_dstY[y>>1] = m_srcY[y];
							m_dstZ[y>>1] = m_srcZ[y];
						}

						if (Lprev_y == 3)
						{
							// use short filter -0.5, 1, -0.5
							m_dstX[2] = ((TYPE)-0.5)*m_srcX[0] + m_srcX[1] + ((TYPE)-0.5)*m_srcX[2];
							m_dstY[2] = ((TYPE)-0.5)*m_srcY[0] + m_srcY[1] + ((TYPE)-0.5)*m_srcY[2];
							m_dstZ[2] = ((TYPE)-0.5)*m_srcZ[0] + m_srcZ[1] + ((TYPE)-0.5)*m_srcZ[2];
						}
						else
						{
							// compute border mirror points and filter
							ASSERT(Lprev_y >= 5);

							if (IsCyclicV())
							{
								// last grid point m_src[Lprev_y-1] is the same as first m_src[0] => take m_src[Lprev_y-3]
								m_dstX[Lapp_y] = c14*(m_srcX[Lprev_y-3] + m_srcX[4]) + m_srcX[1] + c23*(m_srcX[0] + m_srcX[2]);
								m_dstY[Lapp_y] = c14*(m_srcY[Lprev_y-3] + m_srcY[4]) + m_srcY[1] + c23*(m_srcY[0] + m_srcY[2]);
								m_dstZ[Lapp_y] = c14*(m_srcZ[Lprev_y-3] + m_srcZ[4]) + m_srcZ[1] + c23*(m_srcZ[0] + m_srcZ[2]);

								// last grid point m_src[Lprev_y-1] is the same as first m_src[0] => take m_src[2]
								y = Lprev_y-5;
								m_dstX[Lprev_y-1] = c14*(m_srcX[y] + m_srcX[2]) + m_srcX[y+3] + c23*(m_srcX[y+2] + m_srcX[y+4]);
								m_dstY[Lprev_y-1] = c14*(m_srcY[y] + m_srcY[2]) + m_srcY[y+3] + c23*(m_srcY[y+2] + m_srcY[y+4]);
								m_dstZ[Lprev_y-1] = c14*(m_srcZ[y] + m_srcZ[2]) + m_srcZ[y+3] + c23*(m_srcZ[y+2] + m_srcZ[y+4]);
							}
							else
							{
								ComputeMirrorPointLowDec(m_srcX, m_srcY, m_srcZ, pX, pY, pZ);
								m_dstX[Lapp_y] = c14*(pX + m_srcX[4]) + m_srcX[1] + c23*(m_srcX[0] + m_srcX[2]);
								m_dstY[Lapp_y] = c14*(pY + m_srcY[4]) + m_srcY[1] + c23*(m_srcY[0] + m_srcY[2]);
								m_dstZ[Lapp_y] = c14*(pZ + m_srcZ[4]) + m_srcZ[1] + c23*(m_srcZ[0] + m_srcZ[2]);

								y = Lprev_y-5;
								ComputeMirrorPointHighDec(&m_srcX[y], &m_srcY[y], &m_srcZ[y], pX, pY, pZ);
								m_dstX[Lprev_y-1] = c14*(m_srcX[y] + pX) + m_srcX[y+3] + c23*(m_srcX[y+2] + m_srcX[y+4]);
								m_dstY[Lprev_y-1] = c14*(m_srcY[y] + pY) + m_srcY[y+3] + c23*(m_srcY[y+2] + m_srcY[y+4]);
								m_dstZ[Lprev_y-1] = c14*(m_srcZ[y] + pZ) + m_srcZ[y+3] + c23*(m_srcZ[y+2] + m_srcZ[y+4]);
							}
						}

						for (y = 2; y < Lprev_y - 2; y++)
						{
							if (y % 2 == 0) //low(even pos)
							{
								m_dstX[y>>1] = m_srcX[y];
								m_dstY[y>>1] = m_srcY[y];
								m_dstZ[y>>1] = m_srcZ[y];
							}
							else //high(odd pos)
							{
								m_dstX[Lapp_y+(y>>1)] = c14*(m_srcX[y-3] + m_srcX[y+3]) + m_srcX[y] + c23*(m_srcX[y-1] + m_srcX[y+1]);
								m_dstY[Lapp_y+(y>>1)] = c14*(m_srcY[y-3] + m_srcY[y+3]) + m_srcY[y] + c23*(m_srcY[y-1] + m_srcY[y+1]);
								m_dstZ[Lapp_y+(y>>1)] = c14*(m_srcZ[y-3] + m_srcZ[y+3]) + m_srcZ[y] + c23*(m_srcZ[y-1] + m_srcZ[y+1]);
							}
						}

						for (y = 0; y < Lprev_y; y++)
						{
							sliceX[y][x] = m_dstX[y];
							sliceY[y][x] = m_dstY[y];
							sliceZ[y][x] = m_dstZ[y];
						}
					}
				}
			}
		}

		// filtering along X
		if (Lprev_x >= MIN_DEC_SIZE)
		{
			// if a previous size was even we need to update the filter coeficients
			if  (m_bSpecialFilterX[level])
			{
				if (Lprev_x % 2 == 0)
				{
					c2 = (float)m_coefX1[level] / (float)m_coefX2[level];		c1 = (TYPE)1.0 - c2;
					c4 = (TYPE)2.0 * c2;										c3 = (TYPE)1.0 - c4;
				}
				else
				{
					c2 = (float)m_coefX1[level] / (float)m_coefX2[level];		c1 = (TYPE)1.0 - c2;
				}

				for (z = 0; z < Lprev_z; z++)
				{
					TYPE **sliceX = volX[z];
					TYPE **sliceY = volY[z];
					TYPE **sliceZ = volZ[z];
					for (y = 0; y < Lprev_y; y++)
					{
						TYPE *colX = sliceX[y];
						TYPE *colY = sliceY[y];
						TYPE *colZ = sliceZ[y];
						for (x = 0; x < Lprev_x - 3; x++)
						{
							if (x % 2 == 0) // low(even pos)
							{
								m_dstX[x>>1] = colX[x];
								m_dstY[x>>1] = colY[x];
								m_dstZ[x>>1] = colZ[x];
							}
							else // high(odd pos)
							{
								m_dstX[Lapp_x+(x>>1)] = ((TYPE)-0.5)*colX[x-1] + colX[x] + ((TYPE)-0.5)*colX[x+1];
								m_dstY[Lapp_x+(x>>1)] = ((TYPE)-0.5)*colY[x-1] + colY[x] + ((TYPE)-0.5)*colY[x+1];
								m_dstZ[Lapp_x+(x>>1)] = ((TYPE)-0.5)*colZ[x-1] + colZ[x] + ((TYPE)-0.5)*colZ[x+1];
							}
						}

						// assymetric decomposition might be needed for elements (Lprev_x-3), and (Lprev_x-2)
						if (Lprev_x % 2)
						{
							// odd image size
							m_dstX[x>>1] = colX[x];
							m_dstY[x>>1] = colY[x];
							m_dstZ[x>>1] = colZ[x];
							++x;
							m_dstX[Lapp_x+(x>>1)] = -c1*colX[x-1] + colX[x] + -c2*colX[x+1];
							m_dstY[Lapp_x+(x>>1)] = -c1*colY[x-1] + colY[x] + -c2*colY[x+1];
							m_dstZ[Lapp_x+(x>>1)] = -c1*colZ[x-1] + colZ[x] + -c2*colZ[x+1];
							++x;
							m_dstX[x>>1] = colX[x];
							m_dstY[x>>1] = colY[x];
							m_dstZ[x>>1] = colZ[x];
						}
						else
						{
							// even image size
							m_dstX[Lapp_x+(x>>1)] = -c1*colX[x-1] + colX[x] + -c2*colX[x+2];
							m_dstY[Lapp_x+(x>>1)] = -c1*colY[x-1] + colY[x] + -c2*colY[x+2];
							m_dstZ[Lapp_x+(x>>1)] = -c1*colZ[x-1] + colZ[x] + -c2*colZ[x+2];
							++x;
							m_dstX[Lapp_x+(x>>1)] = -c3*colX[x-2] + colX[x] + -c4*colX[x+1];
							m_dstY[Lapp_x+(x>>1)] = -c3*colY[x-2] + colY[x] + -c4*colY[x+1];
							m_dstZ[Lapp_x+(x>>1)] = -c3*colZ[x-2] + colZ[x] + -c4*colZ[x+1];
							++x;
							m_dstX[x>>1] = colX[x];
							m_dstY[x>>1] = colY[x];
							m_dstZ[x>>1] = colZ[x];
						}

						for (x = 0; x < Lprev_x; x++)
						{
							colX[x] = m_dstX[x];
							colY[x] = m_dstY[x];
							colZ[x] = m_dstZ[x];
						}
					}
				}
			}
			else
			{
				for (z = 0; z < Lprev_z; z++)
				{
					TYPE **sliceX = volX[z];
					TYPE **sliceY = volY[z];
					TYPE **sliceZ = volZ[z];
					for (y = 0; y < Lprev_y; y++)
					{
						TYPE *colX = sliceX[y];
						TYPE *colY = sliceY[y];
						TYPE *colZ = sliceZ[y];

						// handle the borders
						{
							// copy first and last values (even pos)
							m_dstX[0] = colX[0];
							m_dstY[0] = colY[0];
							m_dstZ[0] = colZ[0];

							x = Lprev_x-1;
							m_dstX[x>>1] = colX[x];
							m_dstY[x>>1] = colY[x];
							m_dstZ[x>>1] = colZ[x];
						}

						if (Lprev_x == 3)
						{
							// use short filter -0.5, 1, -0.5
							m_dstX[2] = ((TYPE)-0.5)*colX[0] + colX[1] + ((TYPE)-0.5)*colX[2];
							m_dstY[2] = ((TYPE)-0.5)*colY[0] + colY[1] + ((TYPE)-0.5)*colY[2];
							m_dstZ[2] = ((TYPE)-0.5)*colZ[0] + colZ[1] + ((TYPE)-0.5)*colZ[2];
						}
						else
						{
							// compute border mirror points and filter
							ASSERT(Lprev_x >= 5);

							if (IsCyclicU())
							{
								// last grid point m_src[Lprev_x-1] is the same as first m_src[0] => take m_src[Lprev_x-3]
								m_dstX[Lapp_x] = c14*(colX[Lprev_x-3] + colX[4]) + colX[1] + c23*(colX[0] + colX[2]);
								m_dstY[Lapp_x] = c14*(colY[Lprev_x-3] + colY[4]) + colY[1] + c23*(colY[0] + colY[2]);
								m_dstZ[Lapp_x] = c14*(colZ[Lprev_x-3] + colZ[4]) + colZ[1] + c23*(colZ[0] + colZ[2]);

								// last grid point m_src[Lprev_x-1] is the same as first m_src[0] => take m_src[2]
								x = Lprev_x-5;
								m_dstX[Lprev_x-1] = c14*(colX[x] + colX[2]) + colX[x+3] + c23*(colX[x+2] + colX[x+4]);
								m_dstY[Lprev_x-1] = c14*(colY[x] + colY[2]) + colY[x+3] + c23*(colY[x+2] + colY[x+4]);
								m_dstZ[Lprev_x-1] = c14*(colZ[x] + colZ[2]) + colZ[x+3] + c23*(colZ[x+2] + colZ[x+4]);
							}
							else
							{
								ComputeMirrorPointLowDec(colX, colY, colZ, pX, pY, pZ);
								m_dstX[Lapp_x] = c14*(pX + colX[4]) + colX[1] + c23*(colX[0] + colX[2]);
								m_dstY[Lapp_x] = c14*(pY + colY[4]) + colY[1] + c23*(colY[0] + colY[2]);
								m_dstZ[Lapp_x] = c14*(pZ + colZ[4]) + colZ[1] + c23*(colZ[0] + colZ[2]);

								x = Lprev_x-5;
								ComputeMirrorPointHighDec(&colX[x], &colY[x], &colZ[x], pX, pY, pZ);
								m_dstX[Lprev_x-1] = c14*(colX[x] + pX) + colX[x+3] + c23*(colX[x+2] + colX[x+4]);
								m_dstY[Lprev_x-1] = c14*(colY[x] + pY) + colY[x+3] + c23*(colY[x+2] + colY[x+4]);
								m_dstZ[Lprev_x-1] = c14*(colZ[x] + pZ) + colZ[x+3] + c23*(colZ[x+2] + colZ[x+4]);
							}
						}

						for (x = 2; x < Lprev_x - 2; x++)
						{
							if (x % 2 == 0) // low(even pos)
							{
								m_dstX[x>>1] = colX[x];
								m_dstY[x>>1] = colY[x];
								m_dstZ[x>>1] = colZ[x];
							}
							else // high(odd pos)
							{
								m_dstX[Lapp_x+(x>>1)] = c14*(colX[x-3] + colX[x+3]) + colX[x] + c23*(colX[x-1] + colX[x+1]);
								m_dstY[Lapp_x+(x>>1)] = c14*(colY[x-3] + colY[x+3]) + colY[x] + c23*(colY[x-1] + colY[x+1]);
								m_dstZ[Lapp_x+(x>>1)] = c14*(colZ[x-3] + colZ[x+3]) + colZ[x] + c23*(colZ[x-1] + colZ[x+1]);
							}
						}

						for (x = 0; x < Lprev_x; x++)
						{
							colX[x] = m_dstX[x];
							colY[x] = m_dstY[x];
							colZ[x] = m_dstZ[x];
						}
					}
				}
			}
		}
	}
}

template<class TYPE>
void CWaveletCodecSmooth<TYPE>::WaveletRec3D(CMatrix<TYPE>& inX,  CMatrix<TYPE>& inY,  CMatrix<TYPE>& inZ,
											 CMatrix<TYPE>& outX, CMatrix<TYPE>& outY, CMatrix<TYPE>& outZ)
{
	ASSERT(m_sz.u <= inX.dim1  && m_sz.v <= inX.dim2  && m_sz.w <= inX.dim3);
	ASSERT(m_sz.u <= outX.dim1 && m_sz.v <= outX.dim2 && m_sz.w <= outX.dim3);

	// copy into the output and keep the in intact
	outX = inX;   outY = inY;   outZ = inZ;

	int x,y,z,index;
	CODEC_INTERNAL_TYPE pX,pY,pZ;

	int decLevelU = m_maxDecLevel;
	int decLevelV = m_maxDecLevel;
	int decLevelW = m_maxDecLevel;

	int L_x, L_y, L_z;
	int Lapp_x, Lapp_y, Lapp_z;

	L_x = Lapp_x = m_lenX[m_decLevels.u];
	L_y = Lapp_y = m_lenY[m_decLevels.v];
	L_z = Lapp_z = m_lenZ[m_decLevels.w];

	TYPE c1, c2, c3, c4;
	const CODEC_INTERNAL_TYPE c14 = (TYPE)-1.0/16.0;
	const CODEC_INTERNAL_TYPE c23 = (TYPE) 9.0/16.0;

	TYPE ***volX = outX.GetMatrix3D();
	TYPE ***volY = outY.GetMatrix3D();
	TYPE ***volZ = outZ.GetMatrix3D();
	for (int level = m_maxDecLevel-1; level >= 0; level--)
	{
		if (--decLevelU < m_decLevels.u)
		{
			Lapp_x = L_x;			L_x = m_lenX[decLevelU];
		}

		if (--decLevelV < m_decLevels.v)
		{
			Lapp_y = L_y;			L_y = m_lenY[decLevelV];
		}

		if (--decLevelW < m_decLevels.w)
		{
			Lapp_z = L_z;			L_z = m_lenZ[decLevelW];
		}

		// back filtering along X
		if (decLevelU < m_decLevels.u)
		{
			// if a previous size was even we need to update the filter coeficients
			if  (m_bSpecialFilterX[level])
			{
				if (L_x % 2 == 0)
				{
					c2 = (float)m_coefX1[level] / (float)m_coefX2[level];		c1 = (TYPE)1.0 - c2;
					c4 = (TYPE)2.0 * c2;										c3 = (TYPE)1.0 - c4;
				}
				else
				{
					c2 = (float)m_coefX1[level] / (float)m_coefX2[level];		c1 = (TYPE)1.0 - c2;
				}

				for (z = 0; z < L_z; z++)
				{
					TYPE  **sliceX = volX[z];
					TYPE  **sliceY = volY[z];
					TYPE  **sliceZ = volZ[z];
					for (y = 0; y < L_y; y++)
					{
						TYPE  *colX = sliceX[y];
						TYPE  *colY = sliceY[y];
						TYPE  *colZ = sliceZ[y];
						for (x = 0, index = 0; x < L_x-3; x++)
						{
							if (x % 2==0)
							{
								m_dstX[x] = colX[index];
								m_dstY[x] = colY[index];
								m_dstZ[x] = colZ[index];
							}
							else
							{
								m_dstX[x] = ((TYPE) 0.5)*colX[index] + colX[index+Lapp_x] + ((TYPE) 0.5)*colX[index+1];
								m_dstY[x] = ((TYPE) 0.5)*colY[index] + colY[index+Lapp_x] + ((TYPE) 0.5)*colY[index+1];
								m_dstZ[x] = ((TYPE) 0.5)*colZ[index] + colZ[index+Lapp_x] + ((TYPE) 0.5)*colZ[index+1];
								index++;
							}
						}

						// assymetric reconstruction might be needed for elements (Lprev_x-3), and (Lprev_x-2)
						if (L_x % 2)
						{
							// odd image size
							m_dstX[x] = colX[index];
							m_dstY[x] = colY[index];
							m_dstZ[x] = colZ[index];
							x++;
							m_dstX[x] = c1*colX[index] + colX[index+Lapp_x] + c2*colX[index+1];
							m_dstY[x] = c1*colY[index] + colY[index+Lapp_x] + c2*colY[index+1];
							m_dstZ[x] = c1*colZ[index] + colZ[index+Lapp_x] + c2*colZ[index+1];
							index++;		x++;
							m_dstX[x] = colX[index];
							m_dstY[x] = colY[index];
							m_dstZ[x] = colZ[index];
						}
						else
						{
							// even image size
							m_dstX[x] = c1*colX[index] + colX[index+Lapp_x] + c2*colX[index+1];
							m_dstY[x] = c1*colY[index] + colY[index+Lapp_x] + c2*colY[index+1];
							m_dstZ[x] = c1*colZ[index] + colZ[index+Lapp_x] + c2*colZ[index+1];
							index++;		x++;
							m_dstX[x] = c3*colX[index-1] + colX[index+Lapp_x] + c4*colX[index];
							m_dstY[x] = c3*colY[index-1] + colY[index+Lapp_x] + c4*colY[index];
							m_dstZ[x] = c3*colZ[index-1] + colZ[index+Lapp_x] + c4*colZ[index];
							x++;
							m_dstX[x] = colX[index];
							m_dstY[x] = colY[index];
							m_dstZ[x] = colZ[index];
						}

						for (x = 0; x < L_x; x++)
						{
							colX[x] = m_dstX[x];
							colY[x] = m_dstY[x];
							colZ[x] = m_dstZ[x];
						}
					}
				}
			}
			else
			{
				for (z = 0; z < L_z; z++)
				{
					TYPE  **sliceX = volX[z];
					TYPE  **sliceY = volY[z];
					TYPE  **sliceZ = volZ[z];
					for (y = 0; y < L_y; y++)
					{
						TYPE  *colX = sliceX[y];
						TYPE  *colY = sliceY[y];
						TYPE  *colZ = sliceZ[y];

						// handle the borders
						{
							// copy first and last values (even pos)
							m_dstX[0] = colX[0];
							m_dstY[0] = colY[0];
							m_dstZ[0] = colZ[0];

							x = L_x-1;
							m_dstX[x] = colX[x>>1];
							m_dstY[x] = colY[x>>1];
							m_dstZ[x] = colZ[x>>1];
						}

						if (L_x == 3)
						{
							// use short filter -0.5, 1, -0.5
							m_dstX[1] = ((TYPE) 0.5)*colX[0] + colX[2] + ((TYPE) 0.5)*colX[1];
							m_dstY[1] = ((TYPE) 0.5)*colY[0] + colY[2] + ((TYPE) 0.5)*colY[1];
							m_dstZ[1] = ((TYPE) 0.5)*colZ[0] + colZ[2] + ((TYPE) 0.5)*colZ[1];
						}
						else
						{
							// compute border mirror points and filter
							ASSERT(L_x >= 5);

							if (IsCyclicU())
							{
								// last grid point m_src[Lapp_x-1] is the same as first m_src[0] => take m_src[Lapp_x-2]
								m_dstX[1] = c14*(colX[Lapp_x-2] + colX[2]) + colX[Lapp_x] + c23*(colX[0] + colX[1]);
								m_dstY[1] = c14*(colY[Lapp_x-2] + colY[2]) + colY[Lapp_x] + c23*(colY[0] + colY[1]);
								m_dstZ[1] = c14*(colZ[Lapp_x-2] + colZ[2]) + colZ[Lapp_x] + c23*(colZ[0] + colZ[1]);

								// last grid point m_src[Lapp_x-1] is the same as first m_src[0] => take m_src[1]
								x = Lapp_x-3;
								m_dstX[L_x-2] = c14*(colX[x] + colX[1]) + colX[L_x-1] + c23*(colX[x+1] + colX[x+2]);
								m_dstY[L_x-2] = c14*(colY[x] + colY[1]) + colY[L_x-1] + c23*(colY[x+1] + colY[x+2]);
								m_dstZ[L_x-2] = c14*(colZ[x] + colZ[1]) + colZ[L_x-1] + c23*(colZ[x+1] + colZ[x+2]);
							}
							else
							{
								ComputeMirrorPointLowRec(colX, colY, colZ, pX, pY, pZ);
								m_dstX[1] = c14*(pX + colX[2]) + colX[Lapp_x] + c23*(colX[0] + colX[1]);
								m_dstY[1] = c14*(pY + colY[2]) + colY[Lapp_x] + c23*(colY[0] + colY[1]);
								m_dstZ[1] = c14*(pZ + colZ[2]) + colZ[Lapp_x] + c23*(colZ[0] + colZ[1]);

								x = Lapp_x-3;
								ComputeMirrorPointHighRec(&colX[x], &colY[x], &colZ[x], pX, pY, pZ);
								m_dstX[L_x-2] = c14*(colX[x] + pX) + colX[L_x-1] + c23*(colX[x+1] + colX[x+2]);
								m_dstY[L_x-2] = c14*(colY[x] + pY) + colY[L_x-1] + c23*(colY[x+1] + colY[x+2]);
								m_dstZ[L_x-2] = c14*(colZ[x] + pZ) + colZ[L_x-1] + c23*(colZ[x+1] + colZ[x+2]);
							}
						}

						for (x = 2, index = 1; x < L_x - 2; x++)
						{
							if (x % 2==0)
							{
								m_dstX[x] = colX[index];
								m_dstY[x] = colY[index];
								m_dstZ[x] = colZ[index];
							}
							else
							{
								m_dstX[x] = c14*(colX[index-1] + colX[index+2]) + colX[Lapp_x+index] + c23*(colX[index] + colX[index+1]);
								m_dstY[x] = c14*(colY[index-1] + colY[index+2]) + colY[Lapp_x+index] + c23*(colY[index] + colY[index+1]);
								m_dstZ[x] = c14*(colZ[index-1] + colZ[index+2]) + colZ[Lapp_x+index] + c23*(colZ[index] + colZ[index+1]);
								index++;
							}
						}

						for (x = 0; x < L_x; x++)
						{
							colX[x] = m_dstX[x];
							colY[x] = m_dstY[x];
							colZ[x] = m_dstZ[x];
						}
					}
				}
			}
		}

		// back filtering along Y
		if (decLevelV < m_decLevels.v)
		{
			// if a previous size was even we need to update the filter coeficients
			if  (m_bSpecialFilterY[level])
			{
				if (L_y % 2 == 0)
				{
					c2 = (float)m_coefY1[level] / (float)m_coefY2[level];		c1 = (TYPE)1.0 - c2;
					c4 = (TYPE)2.0 * c2;										c3 = (TYPE)1.0 - c4;
				}
				else
				{
					c2 = (float)m_coefY1[level] / (float)m_coefY2[level];		c1 = (TYPE)1.0 - c2;
				}

				for (z = 0; z < L_z; z++)
				{
					TYPE  **sliceX = volX[z];
					TYPE  **sliceY = volY[z];
					TYPE  **sliceZ = volZ[z];
					for (x = 0; x < L_x; x++)
					{
						for (y = 0, index = 0; y < L_y; y++)
						{
							m_srcX[y] = sliceX[y][x];
							m_srcY[y] = sliceY[y][x];
							m_srcZ[y] = sliceZ[y][x];
						}

						for (y = 0, index = 0; y < L_y-3; y++)
						{
							if (y % 2==0)
							{
								m_dstX[y] = m_srcX[index];
								m_dstY[y] = m_srcY[index];
								m_dstZ[y] = m_srcZ[index];
							}
							else
							{
								m_dstX[y] = ((TYPE) 0.5)*m_srcX[index] + m_srcX[index+Lapp_y] + ((TYPE) 0.5)*m_srcX[index+1];
								m_dstY[y] = ((TYPE) 0.5)*m_srcY[index] + m_srcY[index+Lapp_y] + ((TYPE) 0.5)*m_srcY[index+1];
								m_dstZ[y] = ((TYPE) 0.5)*m_srcZ[index] + m_srcZ[index+Lapp_y] + ((TYPE) 0.5)*m_srcZ[index+1];
								index++;
							}
						}

						// assymetric reconstruction might be needed for elements (Lprev_y-3), and (Lprev_y-2)
						if (L_y % 2)
						{
							// odd image size
							m_dstX[y] = m_srcX[index];
							m_dstY[y] = m_srcY[index];
							m_dstZ[y] = m_srcZ[index];
							y++;
							m_dstX[y] = c1*m_srcX[index] + m_srcX[index+Lapp_y] + c2*m_srcX[index+1];
							m_dstY[y] = c1*m_srcY[index] + m_srcY[index+Lapp_y] + c2*m_srcY[index+1];
							m_dstZ[y] = c1*m_srcZ[index] + m_srcZ[index+Lapp_y] + c2*m_srcZ[index+1];
							index++;		y++;
							m_dstX[y] = m_srcX[index];
							m_dstY[y] = m_srcY[index];
							m_dstZ[y] = m_srcZ[index];
						}
						else
						{
							// even image size
							m_dstX[y] = c1*m_srcX[index] + m_srcX[index+Lapp_y] + c2*m_srcX[index+1];
							m_dstY[y] = c1*m_srcY[index] + m_srcY[index+Lapp_y] + c2*m_srcY[index+1];
							m_dstZ[y] = c1*m_srcZ[index] + m_srcZ[index+Lapp_y] + c2*m_srcZ[index+1];
							index++;		y++;
							m_dstX[y] = c3*m_srcX[index-1] + m_srcX[index+Lapp_y] + c4*m_srcX[index];
							m_dstY[y] = c3*m_srcY[index-1] + m_srcY[index+Lapp_y] + c4*m_srcY[index];
							m_dstZ[y] = c3*m_srcZ[index-1] + m_srcZ[index+Lapp_y] + c4*m_srcZ[index];
							y++;
							m_dstX[y] = m_srcX[index];
							m_dstY[y] = m_srcY[index];
							m_dstZ[y] = m_srcZ[index];
						}

						for (y = 0, index = 0; y < L_y; y++)
						{
							sliceX[y][x] = m_dstX[y];
							sliceY[y][x] = m_dstY[y];
							sliceZ[y][x] = m_dstZ[y];
						}
					}
				}
			}
			else
			{
				for (z = 0; z < L_z; z++)
				{
					TYPE  **sliceX = volX[z];
					TYPE  **sliceY = volY[z];
					TYPE  **sliceZ = volZ[z];
					for (x = 0; x < L_x; x++)
					{
						for (y = 0; y < L_y; y++)
						{
							m_srcX[y] = sliceX[y][x];
							m_srcY[y] = sliceY[y][x];
							m_srcZ[y] = sliceZ[y][x];
						}

						// handle the borders
						{
							// copy first and last values (even pos)
							m_dstX[0] = m_srcX[0];
							m_dstY[0] = m_srcY[0];
							m_dstZ[0] = m_srcZ[0];

							y = L_y-1;
							m_dstX[y] = m_srcX[y>>1];
							m_dstY[y] = m_srcY[y>>1];
							m_dstZ[y] = m_srcZ[y>>1];
						}

						if (L_y == 3)
						{
							// use short filter -0.5, 1, -0.5
							m_dstX[1] = ((TYPE) 0.5)*m_srcX[0] + m_srcX[2] + ((TYPE) 0.5)*m_srcX[1];
							m_dstY[1] = ((TYPE) 0.5)*m_srcY[0] + m_srcY[2] + ((TYPE) 0.5)*m_srcY[1];
							m_dstZ[1] = ((TYPE) 0.5)*m_srcZ[0] + m_srcZ[2] + ((TYPE) 0.5)*m_srcZ[1];
						}
						else
						{
							// compute border mirror points and filter
							ASSERT(L_y >= 5);

							if (IsCyclicV())
							{
								// last grid point m_src[Lapp_y-1] is the same as first m_src[0] => take m_src[Lapp_y-2]
								m_dstX[1] = c14*(m_srcX[Lapp_y-2] + m_srcX[2]) + m_srcX[Lapp_y] + c23*(m_srcX[0] + m_srcX[1]);
								m_dstY[1] = c14*(m_srcY[Lapp_y-2] + m_srcY[2]) + m_srcY[Lapp_y] + c23*(m_srcY[0] + m_srcY[1]);
								m_dstZ[1] = c14*(m_srcZ[Lapp_y-2] + m_srcZ[2]) + m_srcZ[Lapp_y] + c23*(m_srcZ[0] + m_srcZ[1]);

								// last grid point m_src[Lapp_y-1] is the same as first m_src[0] => take m_src[1]
								y = Lapp_y-3;
								m_dstX[L_y-2] = c14*(m_srcX[y] + m_srcX[1]) + m_srcX[L_y-1] + c23*(m_srcX[y+1] + m_srcX[y+2]);
								m_dstY[L_y-2] = c14*(m_srcY[y] + m_srcY[1]) + m_srcY[L_y-1] + c23*(m_srcY[y+1] + m_srcY[y+2]);
								m_dstZ[L_y-2] = c14*(m_srcZ[y] + m_srcZ[1]) + m_srcZ[L_y-1] + c23*(m_srcZ[y+1] + m_srcZ[y+2]);
							}
							else
							{
								ComputeMirrorPointLowRec(m_srcX, m_srcY, m_srcZ, pX, pY, pZ);
								m_dstX[1] = c14*(pX + m_srcX[2]) + m_srcX[Lapp_y] + c23*(m_srcX[0] + m_srcX[1]);
								m_dstY[1] = c14*(pY + m_srcY[2]) + m_srcY[Lapp_y] + c23*(m_srcY[0] + m_srcY[1]);
								m_dstZ[1] = c14*(pZ + m_srcZ[2]) + m_srcZ[Lapp_y] + c23*(m_srcZ[0] + m_srcZ[1]);

								y = Lapp_y-3;
								ComputeMirrorPointHighRec(&m_srcX[y], &m_srcY[y], &m_srcZ[y], pX, pY, pZ);
								m_dstX[L_y-2] = c14*(m_srcX[y] + pX) + m_srcX[L_y-1] + c23*(m_srcX[y+1] + m_srcX[y+2]);
								m_dstY[L_y-2] = c14*(m_srcY[y] + pY) + m_srcY[L_y-1] + c23*(m_srcY[y+1] + m_srcY[y+2]);
								m_dstZ[L_y-2] = c14*(m_srcZ[y] + pZ) + m_srcZ[L_y-1] + c23*(m_srcZ[y+1] + m_srcZ[y+2]);
							}
						}

						for (y = 2, index = 1; y < L_y - 2; y++)
						{
							if (y % 2==0)
							{
								m_dstX[y] = m_srcX[index];
								m_dstY[y] = m_srcY[index];
								m_dstZ[y] = m_srcZ[index];
							}
							else
							{
								m_dstX[y] = c14*(m_srcX[index-1] + m_srcX[index+2]) + m_srcX[Lapp_y+index] + c23*(m_srcX[index] + m_srcX[index+1]);
								m_dstY[y] = c14*(m_srcY[index-1] + m_srcY[index+2]) + m_srcY[Lapp_y+index] + c23*(m_srcY[index] + m_srcY[index+1]);
								m_dstZ[y] = c14*(m_srcZ[index-1] + m_srcZ[index+2]) + m_srcZ[Lapp_y+index] + c23*(m_srcZ[index] + m_srcZ[index+1]);
								index++;
							}
						}

						for (y = 0; y < L_y; y++)
						{
							sliceX[y][x] = m_dstX[y];
							sliceY[y][x] = m_dstY[y];
							sliceZ[y][x] = m_dstZ[y];
						}
					}
				}
			}
		}

		// back filtering along Z
		if (decLevelW < m_decLevels.w)
		{
			// if a previous size was even we need to update the filter coeficients
			if  (m_bSpecialFilterZ[level])
			{
				if (L_z % 2 == 0)
				{
					c2 = (float)m_coefZ1[level] / (float)m_coefZ2[level];		c1 = (TYPE)1.0 - c2;
					c4 = (TYPE)2.0 * c2;										c3 = (TYPE)1.0 - c4;
				}
				else
				{
					c2 = (float)m_coefZ1[level] / (float)m_coefZ2[level];		c1 = (TYPE)1.0 - c2;
				}

				for (y = 0; y < L_y; y++)
				{
					for (x = 0; x < L_x; x++)
					{
						for (z = 0, index = 0; z < L_z; z++)
						{
							m_srcX[z] = volX[z][y][x];
							m_srcY[z] = volY[z][y][x];
							m_srcZ[z] = volZ[z][y][x];
						}

						for (z = 0, index = 0; z < L_z-3; z++)
						{
							if (z % 2==0)
							{
								m_dstX[z] = m_srcX[index];
								m_dstY[z] = m_srcY[index];
								m_dstZ[z] = m_srcZ[index];
							}
							else
							{
								m_dstX[z] = ((TYPE) 0.5)*m_srcX[index] + m_srcX[index+Lapp_z] + ((TYPE) 0.5)*m_srcX[index+1];
								m_dstY[z] = ((TYPE) 0.5)*m_srcY[index] + m_srcY[index+Lapp_z] + ((TYPE) 0.5)*m_srcY[index+1];
								m_dstZ[z] = ((TYPE) 0.5)*m_srcZ[index] + m_srcZ[index+Lapp_z] + ((TYPE) 0.5)*m_srcZ[index+1];
								index++;
							}
						}

						// assymetric reconstruction might be needed for elements (Lprev_z-3), and (Lprev_z-2)
						if (L_z % 2)
						{
							// odd image size
							m_dstX[z] = m_srcX[index];
							m_dstY[z] = m_srcY[index];
							m_dstZ[z] = m_srcZ[index];
							z++;
							m_dstX[z] = c1*m_srcX[index] + m_srcX[index+Lapp_z] + c2*m_srcX[index+1];
							m_dstY[z] = c1*m_srcY[index] + m_srcY[index+Lapp_z] + c2*m_srcY[index+1];
							m_dstZ[z] = c1*m_srcZ[index] + m_srcZ[index+Lapp_z] + c2*m_srcZ[index+1];
							index++;		z++;
							m_dstX[z] = m_srcX[index];
							m_dstY[z] = m_srcY[index];
							m_dstZ[z] = m_srcZ[index];
						}
						else
						{
							// even image size
							m_dstX[z] = c1*m_srcX[index] + m_srcX[index+Lapp_z] + c2*m_srcX[index+1];
							m_dstY[z] = c1*m_srcY[index] + m_srcY[index+Lapp_z] + c2*m_srcY[index+1];
							m_dstZ[z] = c1*m_srcZ[index] + m_srcZ[index+Lapp_z] + c2*m_srcZ[index+1];
							index++;		z++;
							m_dstX[z] = c3*m_srcX[index-1] + m_srcX[index+Lapp_z] + c4*m_srcX[index];
							m_dstY[z] = c3*m_srcY[index-1] + m_srcY[index+Lapp_z] + c4*m_srcY[index];
							m_dstZ[z] = c3*m_srcZ[index-1] + m_srcZ[index+Lapp_z] + c4*m_srcZ[index];
							z++;
							m_dstX[z] = m_srcX[index];
							m_dstY[z] = m_srcY[index];
							m_dstZ[z] = m_srcZ[index];
						}

						for (z = 0, index = 0; z < L_z; z++)
						{
							volX[z][y][x] = m_dstX[z];
							volY[z][y][x] = m_dstY[z];
							volZ[z][y][x] = m_dstZ[z];
						}
					}
				}
			}
			else
			{
				for (y = 0; y < L_y; y++)
				{
					for (x = 0; x < L_x; x++)
					{
						for (z = 0; z < L_z; z++)
						{
							m_srcX[z] = volX[z][y][x];
							m_srcY[z] = volY[z][y][x];
							m_srcZ[z] = volZ[z][y][x];
						}

						// handle the borders
						{
							// copy first and last values (even pos)
							m_dstX[0] = m_srcX[0];
							m_dstY[0] = m_srcY[0];
							m_dstZ[0] = m_srcZ[0];

							z = L_z-1;
							m_dstX[z] = m_srcX[z>>1];
							m_dstY[z] = m_srcY[z>>1];
							m_dstZ[z] = m_srcZ[z>>1];
						}

						if (L_z == 3)
						{
							// use short filter -0.5, 1, -0.5
							m_dstX[1] = ((TYPE) 0.5)*m_srcX[0] + m_srcX[2] + ((TYPE) 0.5)*m_srcX[1];
							m_dstY[1] = ((TYPE) 0.5)*m_srcY[0] + m_srcY[2] + ((TYPE) 0.5)*m_srcY[1];
							m_dstZ[1] = ((TYPE) 0.5)*m_srcZ[0] + m_srcZ[2] + ((TYPE) 0.5)*m_srcZ[1];
						}
						else
						{
							// compute border mirror points and filter
							ASSERT(L_z >= 5);

							ComputeMirrorPointLowRec(m_srcX, m_srcY, m_srcZ, pX, pY, pZ);
							m_dstX[1] = c14*(pX + m_srcX[2]) + m_srcX[Lapp_z] + c23*(m_srcX[0] + m_srcX[1]);
							m_dstY[1] = c14*(pY + m_srcY[2]) + m_srcY[Lapp_z] + c23*(m_srcY[0] + m_srcY[1]);
							m_dstZ[1] = c14*(pZ + m_srcZ[2]) + m_srcZ[Lapp_z] + c23*(m_srcZ[0] + m_srcZ[1]);

							z = Lapp_z-3;
							ComputeMirrorPointHighRec(&m_srcX[z], &m_srcY[z], &m_srcZ[z], pX, pY, pZ);
							m_dstX[L_z-2] = c14*(m_srcX[z] + pX) + m_srcX[L_z-1] + c23*(m_srcX[z+1] + m_srcX[z+2]);
							m_dstY[L_z-2] = c14*(m_srcY[z] + pY) + m_srcY[L_z-1] + c23*(m_srcY[z+1] + m_srcY[z+2]);
							m_dstZ[L_z-2] = c14*(m_srcZ[z] + pZ) + m_srcZ[L_z-1] + c23*(m_srcZ[z+1] + m_srcZ[z+2]);
						}

						for (z = 2, index = 1; z < L_z - 2; z++)
						{
							if (z % 2==0)
							{
								m_dstX[z] = m_srcX[index];
								m_dstY[z] = m_srcY[index];
								m_dstZ[z] = m_srcZ[index];
							}
							else
							{
								m_dstX[z] = c14*(m_srcX[index-1] + m_srcX[index+2]) + m_srcX[Lapp_z+index] + c23*(m_srcX[index] + m_srcX[index+1]);
								m_dstY[z] = c14*(m_srcY[index-1] + m_srcY[index+2]) + m_srcY[Lapp_z+index] + c23*(m_srcY[index] + m_srcY[index+1]);
								m_dstZ[z] = c14*(m_srcZ[index-1] + m_srcZ[index+2]) + m_srcZ[Lapp_z+index] + c23*(m_srcZ[index] + m_srcZ[index+1]);
								index++;
							}
						}

						for (z = 0; z < L_z; z++)
						{
							volX[z][y][x] = m_dstX[z];
							volY[z][y][x] = m_dstY[z];
							volZ[z][y][x] = m_dstZ[z];
						}
					}
				}
			}
		}
	}
}

#endif // !defined(AFX_WAVELETSMOOTH3D_H__26A17B90_BBEF_4A66_8833_93501906C6AB__INCLUDED_)
