/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player/ Mux
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  OD Decoder/ Encoder
FILE:       odp.cpp
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Implementation of parsing/encoding of descriptors of OD stream
-------------------------------------------------------------------------*/

#include "defs.h"
#include "ODP.h" // SOLARIS PORTING - typo
#include "odtags.h"
//#include "ODIPMPX.h"
#include "IPMP_Messages.h"
#include "CrSdl.h"

OD_IMPLEMENT_DESCRIPTOR_START (AFXConfig)
	OD_IMPLEMENT_FIELD (AFXConfig, afxext)
OD_IMPLEMENT_DESCRIPTOR_END (AFXConfig)

OD_IMPLEMENT_ABSTRACT_DESCRIPTOR_START (AFXExtDescriptor)
	OD_IMPLEMENT_EXTENSION (A3DMCDecoderSpecific)
	OD_IMPLEMENT_EXTENSION (BBADecoderSpecific)
	OD_IMPLEMENT_EXTENSION (CoordInterpCompDecoderSpecific)
	OD_IMPLEMENT_EXTENSION (MeshGridDecoderSpecificInfo)
	OD_IMPLEMENT_EXTENSION (WMDecoderSpecificInfo)
	OD_IMPLEMENT_EXTENSION (OriInterpCompDecoderSpecific)
	OD_IMPLEMENT_EXTENSION (PosInterpCompDecoderSpecific)
// 2004-12-27, GyeongJa Jang <-
	//OD_IMPLEMENT_EXTENSION (DIBRDecoderSpecific)
// 2004-07-15, Shinjun {
	//OD_IMPLEMENT_EXTENSION (PTCompDecoderSpecific)
// 2004-07-15, Shinjun }
	OD_IMPLEMENT_EXTENSION (OctreeImageDecoderSpecific)
	OD_IMPLEMENT_EXTENSION (PointTextureCompDecoderSpecific)
// 2004-12-27, GyeongJa Jang ->
//ory DMLAB 2007 01 07 
	OD_IMPLEMENT_EXTENSION (A3DMCExtensionDecoderSpecific)
//~ory DMLAB
    OD_IMPLEMENT_EXTENSION (FPSetDecoderSpecificInfo)
//INT KM 2007 12 03
	OD_IMPLEMENT_EXTENSION (FAMCDecoderSpecific)
//INT KM


OD_IMPLEMENT_ABSTRACT_DESCRIPTOR_END (AFXExtDescriptor)

OD_IMPLEMENT_DESCRIPTOR_START (MeshGridDecoderSpecificInfo)
	OD_IMPLEMENT_FIELD (MeshGridDecoderSpecificInfo, nLevels)
	OD_IMPLEMENT_FIELD (MeshGridDecoderSpecificInfo, nSlices)
	OD_IMPLEMENT_FIELD (MeshGridDecoderSpecificInfo, nROIs)
	OD_IMPLEMENT_FIELD (MeshGridDecoderSpecificInfo, hasConnectivityInfo)
	OD_IMPLEMENT_FIELD (MeshGridDecoderSpecificInfo, hasRefineInfo)
	OD_IMPLEMENT_FIELD (MeshGridDecoderSpecificInfo, hasRepositionInfo)
	OD_IMPLEMENT_FIELD (MeshGridDecoderSpecificInfo, hasGridInfo)
	OD_IMPLEMENT_FIELD (MeshGridDecoderSpecificInfo, attributes)
	OD_IMPLEMENT_FIELD (MeshGridDecoderSpecificInfo, hasMultiLayer)
	OD_IMPLEMENT_FIELD (MeshGridDecoderSpecificInfo, nLayers)
	OD_IMPLEMENT_FIELD (MeshGridDecoderSpecificInfo, meshType)
	OD_IMPLEMENT_FIELD (MeshGridDecoderSpecificInfo, sameBorderOrientation)
	OD_IMPLEMENT_FIELD (MeshGridDecoderSpecificInfo, uniformSplit)
	OD_IMPLEMENT_FIELD (MeshGridDecoderSpecificInfo, offsetAmplitude)
	OD_IMPLEMENT_FIELD (MeshGridDecoderSpecificInfo, cyclicMode)
	OD_IMPLEMENT_FIELD (MeshGridDecoderSpecificInfo, fullRefine)
	OD_IMPLEMENT_FIELD (MeshGridDecoderSpecificInfo, nRefineBits)
	OD_IMPLEMENT_FIELD (MeshGridDecoderSpecificInfo, filterType)
	OD_IMPLEMENT_FIELD (MeshGridDecoderSpecificInfo, gridScale)
	OD_IMPLEMENT_FIELD (MeshGridDecoderSpecificInfo, nQuantBits)
	OD_IMPLEMENT_FIELD (MeshGridDecoderSpecificInfo, sign)
	OD_IMPLEMENT_FIELD (MeshGridDecoderSpecificInfo, corner)
	OD_IMPLEMENT_FIELD (MeshGridDecoderSpecificInfo, gridCorners)
	OD_IMPLEMENT_FIELD (MeshGridDecoderSpecificInfo, nBits)
OD_IMPLEMENT_DESCRIPTOR_END (MeshGridDecoderSpecificInfo)

OD_IMPLEMENT_DESCRIPTOR_START (WMDecoderSpecificInfo)
	OD_IMPLEMENT_FIELD (WMDecoderSpecificInfo, hasScaleCoeff)
	OD_IMPLEMENT_FIELD (WMDecoderSpecificInfo, NbBpSC)
	OD_IMPLEMENT_FIELD (WMDecoderSpecificInfo, NbBPX)
	OD_IMPLEMENT_FIELD (WMDecoderSpecificInfo, NbBPY)
	OD_IMPLEMENT_FIELD (WMDecoderSpecificInfo, NbBPZ)
	OD_IMPLEMENT_FIELD (WMDecoderSpecificInfo, Wtype)
	OD_IMPLEMENT_FIELD (WMDecoderSpecificInfo, lift)
	OD_IMPLEMENT_FIELD (WMDecoderSpecificInfo, NumberOfLevels)
	OD_IMPLEMENT_FIELD (WMDecoderSpecificInfo, Xmax)
	OD_IMPLEMENT_FIELD (WMDecoderSpecificInfo, Ymax)
	OD_IMPLEMENT_FIELD (WMDecoderSpecificInfo, Zmax)
OD_IMPLEMENT_DESCRIPTOR_END (WMDecoderSpecificInfo)



OD_IMPLEMENT_DESCRIPTOR_START (A3DMCDecoderSpecific)
	OD_IMPLEMENT_FIELD (A3DMCDecoderSpecific, null)
OD_IMPLEMENT_DESCRIPTOR_END (A3DMCDecoderSpecific)

//ory DMLAB 2007 01 07
OD_IMPLEMENT_DESCRIPTOR_START (A3DMCExtensionDecoderSpecific)
	OD_IMPLEMENT_FIELD (A3DMCExtensionDecoderSpecific, null)
OD_IMPLEMENT_DESCRIPTOR_END (A3DMCExtensionDecoderSpecific)
//~ory DMLAB

//INT KM 2007 12 03
OD_IMPLEMENT_DESCRIPTOR_START (FAMCDecoderSpecific)
	OD_IMPLEMENT_FIELD (FAMCDecoderSpecific, null)
OD_IMPLEMENT_DESCRIPTOR_END (FAMCDecoderSpecific)
//INT KM

OD_IMPLEMENT_DESCRIPTOR_START (CoordInterpCompDecoderSpecific)
	OD_IMPLEMENT_FIELD (CoordInterpCompDecoderSpecific, null)
OD_IMPLEMENT_DESCRIPTOR_END (CoordInterpCompDecoderSpecific)

OD_IMPLEMENT_DESCRIPTOR_START (OriInterpCompDecoderSpecific)
	OD_IMPLEMENT_FIELD (OriInterpCompDecoderSpecific, null)
OD_IMPLEMENT_DESCRIPTOR_END (OriInterpCompDecoderSpecific)

OD_IMPLEMENT_DESCRIPTOR_START (PosInterpCompDecoderSpecific)
	OD_IMPLEMENT_FIELD (PosInterpCompDecoderSpecific, null)
OD_IMPLEMENT_DESCRIPTOR_END (PosInterpCompDecoderSpecific)

// 2004-12-27, GyeongJa Jang <-
/*
OD_IMPLEMENT_DESCRIPTOR_START (DIBRDecoderSpecific)
	OD_IMPLEMENT_FIELD (DIBRDecoderSpecific, null)
OD_IMPLEMENT_DESCRIPTOR_END (DIBRDecoderSpecific)
*/
OD_IMPLEMENT_DESCRIPTOR_START (OctreeImageDecoderSpecific)
	OD_IMPLEMENT_FIELD (OctreeImageDecoderSpecific, null)
OD_IMPLEMENT_DESCRIPTOR_END (OctreeImageDecoderSpecific)
// 2004-12-27, GyeongJa Jang ->
OD_IMPLEMENT_DESCRIPTOR_START (BBADecoderSpecific)
	OD_IMPLEMENT_FIELD (BBADecoderSpecific, null)
OD_IMPLEMENT_DESCRIPTOR_END (BBADecoderSpecific)

// 2004-12-27, GyeongJa Jang <-
// 2004-07-15, Shinjun {
/*
OD_IMPLEMENT_DESCRIPTOR_START (PTCompDecoderSpecific)
	OD_IMPLEMENT_FIELD (PTCompDecoderSpecific, null)
OD_IMPLEMENT_DESCRIPTOR_END (PTCompDecoderSpecific)
*/
OD_IMPLEMENT_DESCRIPTOR_START (PointTextureCompDecoderSpecific)
	OD_IMPLEMENT_FIELD (PointTextureCompDecoderSpecific, null)
OD_IMPLEMENT_DESCRIPTOR_END (PointTextureCompDecoderSpecific)
// 2004-07-15, Shinjun }
// 2004-12-27, GyeongJa Jang ->

OD_IMPLEMENT_DESCRIPTOR_START (FPSetDecoderSpecificInfo)
	OD_IMPLEMENT_FIELD (FPSetDecoderSpecificInfo, FPObjectType)
	OD_IMPLEMENT_FIELD (FPSetDecoderSpecificInfo, MaxNbFootPrints)
	OD_IMPLEMENT_FIELD (FPSetDecoderSpecificInfo, FootPrintNbBits)
	OD_IMPLEMENT_FIELD (FPSetDecoderSpecificInfo, Step)
	OD_IMPLEMENT_FIELD (FPSetDecoderSpecificInfo, NbBitsMetricError)
	OD_IMPLEMENT_FIELD (FPSetDecoderSpecificInfo, MinX)
	OD_IMPLEMENT_FIELD (FPSetDecoderSpecificInfo, MaxX)
	OD_IMPLEMENT_FIELD (FPSetDecoderSpecificInfo, MinY)
	OD_IMPLEMENT_FIELD (FPSetDecoderSpecificInfo, MaxY)
	OD_IMPLEMENT_FIELD (FPSetDecoderSpecificInfo, DEFIdUsed)
	OD_IMPLEMENT_FIELD (FPSetDecoderSpecificInfo, MinAltitude)
	OD_IMPLEMENT_FIELD (FPSetDecoderSpecificInfo, MaxHeight)
	OD_IMPLEMENT_FIELD (FPSetDecoderSpecificInfo, NbBitsZBuilding)
	OD_IMPLEMENT_FIELD (FPSetDecoderSpecificInfo, NbBitsNbStories)
	OD_IMPLEMENT_FIELD (FPSetDecoderSpecificInfo, NbBitsStoryHeight)
	OD_IMPLEMENT_FIELD (FPSetDecoderSpecificInfo, NbBitsFacadeWidth)
	OD_IMPLEMENT_FIELD (FPSetDecoderSpecificInfo, NbBitsNbFacadeCellsByStorey)
OD_IMPLEMENT_DESCRIPTOR_END (FPSetDecoderSpecificInfo)

