/***********************************************************************
This software module was originally developed by Shinjun Lee
(SAMSUNG AIT) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. SAMSUNG AIT retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright ?2002.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Shinjun Lee at sjlee@sait.samsung.co.kr.
----------------------------------------------------------------------*/

// CAdjustableOctree.h
// Author: Shinjun Lee, Samsung AIT
// Date: 2003-11-24 ~ 2003-12-16

#ifndef _CADJUSTABLEOCTREE_H_
#define _CADJUSTABLEOCTREE_H_

#include "CVoxel.h"
#include "CColor.h"
#include "CQueue.h"
#include "bitio.h"
#include "arith.h"
#include "stats.h"

class CAdjustableOctree
	{
  private:
	unsigned char*	m_a_ucOctree;				// the adjustable octree data
	int				m_nSizeOfOctree;			// the size of the octree

	unsigned char*	m_a_ucMeanColor;			// the mean color of octree region
	int				m_nSizeOfMeanColor;			// the size of the mean color

	unsigned char*	m_a_ucSOP;					// the SOP(split or ppm) data
	int				m_nSizeOfSOP;				// the size of the SOP
	int				m_nThresholdLevelOfPPM;		// the threshold level of PPM coding in the octree

	int*			m_a_nCountOfTreeSize;		// the count data of tree size
	int				m_nSizeOfCount;				// the size of the count

	int				m_nWidth;					// width
	int				m_nHeight;					// height
	int				m_nDepth;					// depth
	int				m_nNumberOfTreeNodes;		// the number of tree nodes

	CVoxel*			m_pVoxel;					// voxel object pointer
	CColor*			m_pColor;					// color object pointer

	double*			m_a_dProbabilityTable;		// a probability table
	int				m_nSizeOfTable;				// the size of the probality table

  public:
	 CAdjustableOctree(void)
		{
		// initalize: the size of the probability table and the other member variables
		const int nSizeOfTable = 1024;

		m_a_ucOctree = NULL;
		m_nSizeOfOctree = 0;

		m_a_ucMeanColor = NULL;
		m_nSizeOfMeanColor = 0;

		m_a_ucSOP = NULL;
		m_nSizeOfSOP = 0;
		m_nThresholdLevelOfPPM = 4;				// 4(16*16*16): PPM coding, 5(32*32*32): Split coding

		m_a_nCountOfTreeSize = NULL;
		m_nSizeOfCount = 0;

		m_nWidth = 0;
		m_nHeight = 0;
		m_nDepth = 0;
		m_nNumberOfTreeNodes = 0;

		m_pVoxel = NULL;
		m_pColor = NULL;

		// create: a probability table
		m_nSizeOfTable = nSizeOfTable;
		m_a_dProbabilityTable = new double[m_nSizeOfTable]; // 0 ~ 1023 context (10's model data --> 10's context)

		// create: the probability table for one symbol
		double a_dTableOfOne[nSizeOfTable] =
			{
			0.000009, 0.043776, 0.001703, 0.008942, 0.019989, 0.146560, 0.186414, 0.634648, 0.343058, 0.108442, 0.159287, 0.057908, 0.169686, 0.094540, 0.311746, 0.077018,
			0.077586, 0.154034, 0.009829, 0.044751, 0.013518, 0.121343, 0.149113, 0.570621, 0.027847, 0.040279, 0.008826, 0.010724, 0.010219, 0.008439, 0.009083, 0.013650,
			0.039365, 0.143885, 0.002871, 0.005878, 0.133012, 0.623898, 0.113758, 0.567495, 0.206926, 0.098036, 0.013645, 0.007213, 0.050663, 0.041990, 0.037917, 0.012369,
			0.059299, 0.141037, 0.004711, 0.022282, 0.025628, 0.201706, 0.106676, 0.408939, 0.009691, 0.020869, 0.001298, 0.000359, 0.004802, 0.001068, 0.011249, 0.002089,
			0.056267, 0.549316, 0.130187, 0.603306, 0.416606, 0.013523, 0.225597, 0.113384, 0.903853, 0.881549, 0.725516, 0.398368, 0.824299, 0.103538, 0.696358, 0.008434,
			0.262445, 0.861180, 0.198925, 0.857370, 0.893193, 0.142655, 0.745833, 0.251377, 0.979445, 0.985775, 0.527381, 0.653726, 0.266675, 0.056059, 0.333333, 0.016835, 
			0.191049, 0.953782, 0.301480, 0.825746, 0.791691, 0.402501, 0.285714, 0.188891, 0.957345, 0.975631, 0.728571, 0.784946, 0.897687, 0.095513, 0.288889, 0.011976, 
			0.213329, 0.612137, 0.500000, 0.781795, 0.954411, 0.169941, 0.750000, 0.276190, 0.887481, 0.804074, 0.212500, 0.177778, 0.243655, 0.068056, 0.111111, 0.005743, 
			0.160144, 0.095378, 0.223135, 0.150158, 0.957704, 0.980433, 0.783765, 0.723037, 0.280996, 0.966136, 0.831529, 0.629451, 0.213983, 0.505039, 0.201875, 0.262987, 
			0.701672, 0.681540, 0.899466, 0.627778, 0.987894, 0.954789, 0.846753, 0.680940, 0.805260, 0.925950, 0.962210, 0.734003, 0.249705, 0.449708, 0.152947, 0.375000, 
			0.467919, 0.400348, 0.695065, 0.525870, 0.848763, 0.808568, 0.822548, 0.701860, 0.334336, 0.625409, 0.804895, 0.488678, 0.146004, 0.424028, 0.072839, 0.215909, 
			0.490341, 0.587741, 0.901519, 0.590476, 0.952741, 0.868254, 0.815507, 0.700000, 0.134764, 0.404189, 0.437350, 0.588695, 0.144429, 0.164211, 0.037691, 0.000387, 
			0.009426, 0.037007, 0.406352, 0.106066, 0.833948, 0.021348, 0.608697, 0.004246, 0.104474, 0.290043, 0.139669, 0.244048, 0.043383, 0.137406, 0.015945, 0.015046, 
			0.373529, 0.256250, 0.766574, 0.254762, 0.989934, 0.385714, 0.917088, 0.048718, 0.299806, 0.556003, 0.734512, 0.542857, 0.210040, 0.187065, 0.048951, 0.166667, 
			0.356814, 0.132067, 0.780107, 0.159560, 0.753125, 0.150000, 0.900826, 0.500000, 0.050048, 0.156439, 0.725000, 0.263636, 0.070662, 0.285714, 0.500000, 0.500000, 
			0.240876, 0.500000, 0.856013, 0.154762, 0.978000, 0.642857, 0.909091, 0.041667, 0.158261, 0.600000, 0.250000, 0.253968, 0.171132, 0.288889, 0.004456, 0.011920, 
			0.059757, 0.073829, 0.007047, 0.008695, 0.542154, 0.991976, 0.296512, 0.971688, 0.827774, 0.544973, 0.828927, 0.539795, 0.632291, 0.255943, 0.885353, 0.237047, 
			0.651452, 0.119938, 0.292908, 0.139881, 0.277852, 0.948754, 0.054762, 0.587137, 0.271687, 0.262967, 0.475352, 0.312983, 0.134663, 0.066285, 0.147083, 0.056896, 
			0.288227, 0.013366, 0.038965, 0.006540, 0.378571, 0.742382, 0.115909, 0.631319, 0.364797, 0.036119, 0.264355, 0.281449, 0.417584, 0.033849, 0.339583, 0.147495, 
			0.820301, 0.114618, 0.287460, 0.153866, 0.391392, 0.951910, 0.160000, 0.784799, 0.230749, 0.009333, 0.201848, 0.009949, 0.176398, 0.011421, 0.091837, 0.010118, 
			0.089369, 0.684100, 0.109596, 0.605043, 0.725294, 0.336579, 0.512500, 0.227331, 0.804618, 0.473684, 0.947210, 0.575181, 0.811073, 0.211121, 0.913462, 0.234509, 
			0.837963, 0.681548, 0.277211, 0.555556, 0.893386, 0.633333, 0.803571, 0.888889, 0.628571, 0.292560, 0.594444, 0.733333, 0.537143, 0.026770, 0.791667, 0.500000, 
			0.129286, 0.348167, 0.056437, 0.569445, 0.666667, 0.285714, 0.563889, 0.500000, 0.882288, 0.370670, 0.992063, 0.642857, 0.722756, 0.016783, 0.500000, 0.500000, 
			0.508081, 0.666667, 0.500000, 0.833334, 0.980159, 0.722222, 0.833333, 0.555556, 0.548077, 0.133333, 0.625000, 0.375000, 0.687500, 0.005051, 0.500000, 0.026337, 
			0.273069, 0.017311, 0.024118, 0.089808, 0.987149, 0.416282, 0.800247, 0.324383, 0.037159, 0.403197, 0.331538, 0.226096, 0.049293, 0.333361, 0.190727, 0.293916, 
			0.864182, 0.309942, 0.541774, 0.250000, 0.989265, 0.875722, 0.600000, 0.357143, 0.135729, 0.483865, 0.816906, 0.414184, 0.122957, 0.532903, 0.321429, 0.384398, 
			0.822224, 0.064745, 0.471350, 0.126391, 0.978857, 0.888235, 0.875000, 0.255102, 0.057496, 0.120469, 0.614286, 0.262755, 0.024608, 0.047669, 0.333333, 0.285714, 
			0.872490, 0.250000, 0.436610, 0.200000, 0.956069, 0.768421, 0.805556, 0.520000, 0.076475, 0.170446, 0.308333, 0.251429, 0.025433, 0.058391, 0.054945, 0.082703, 
			0.122462, 0.010879, 0.019194, 0.087543, 0.868885, 0.073987, 0.346528, 0.006029, 0.012110, 0.054160, 0.048522, 0.034632, 0.010468, 0.176354, 0.016731, 0.257423, 
			0.537050, 0.500000, 0.316667, 0.166667, 0.975890, 0.500000, 0.542857, 0.222222, 0.163127, 0.011765, 0.700000, 0.135417, 0.029931, 0.027968, 0.158415, 0.272527, 
			0.604921, 0.020856, 0.196693, 0.025141, 0.814546, 0.125000, 0.500000, 0.091837, 0.063357, 0.320000, 0.633333, 0.250000, 0.014817, 0.010870, 0.500000, 0.200000, 
			0.580445, 0.187500, 0.150000, 0.166667, 0.970156, 0.320000, 0.883333, 0.200000, 0.104961, 0.500000, 0.500000, 0.500000, 0.037130, 0.500000, 0.019048, 0.018825, 
			0.302862, 0.928048, 0.542997, 0.894260, 0.102528, 0.823933, 0.204336, 0.988545, 0.699551, 0.251633, 0.738220, 0.244194, 0.922036, 0.437114, 0.953067, 0.401874, 
			0.596170, 0.828145, 0.629573, 0.787641, 0.117363, 0.299987, 0.159263, 0.358239, 0.052293, 0.095777, 0.195334, 0.065238, 0.233154, 0.110417, 0.183333, 0.308935, 
			0.767537, 0.928348, 0.335281, 0.959071, 0.275952, 0.954365, 0.190476, 0.810224, 0.257682, 0.148701, 0.118150, 0.105087, 0.652532, 0.509341, 0.502222, 0.223493, 
			0.778728, 0.845532, 0.397736, 0.931399, 0.415794, 0.608333, 0.179167, 0.634354, 0.134321, 0.035013, 0.068571, 0.018441, 0.382106, 0.049603, 0.322222, 0.105589, 
			0.186911, 0.837441, 0.464338, 0.873198, 0.015644, 0.004820, 0.500000, 0.091620, 0.100905, 0.050303, 0.431373, 0.100736, 0.337945, 0.028906, 0.513075, 0.224863, 
			0.859554, 0.904583, 0.545627, 0.858440, 0.077442, 0.023649, 0.038462, 0.031667, 0.022820, 0.017402, 0.071429, 0.020450, 0.133889, 0.115209, 0.500000, 0.144599, 
			0.588690, 0.908528, 0.337730, 0.962310, 0.056546, 0.100000, 0.500000, 0.333333, 0.079746, 0.108560, 0.075000, 0.092852, 0.380852, 0.125000, 0.500000, 0.388344, 
			0.874777, 0.948889, 0.586667, 0.885266, 0.147326, 0.066667, 0.500000, 0.083333, 0.081368, 0.020869, 0.129167, 0.011039, 0.478788, 0.119192, 0.125000, 0.017985, 
			0.174934, 0.147299, 0.716617, 0.097153, 0.074583, 0.044756, 0.207143, 0.034293, 0.020732, 0.022854, 0.016998, 0.034039, 0.058649, 0.121401, 0.040739, 0.283803, 
			0.722344, 0.711974, 0.914050, 0.436905, 0.150714, 0.034167, 0.671875, 0.068027, 0.079479, 0.087902, 0.084668, 0.129963, 0.216667, 0.694375, 0.049084, 0.377778, 
			0.620671, 0.548312, 0.917001, 0.708334, 0.279557, 0.017143, 0.579365, 0.076923, 0.139286, 0.052487, 0.052778, 0.233333, 0.110128, 0.200000, 0.083333, 0.250000, 
			0.842261, 0.808754, 0.952903, 0.725000, 0.425148, 0.180785, 0.615714, 0.100000, 0.085423, 0.049242, 0.069913, 0.250000, 0.188435, 0.444445, 0.066667, 0.313393, 
			0.556379, 0.545426, 0.920995, 0.458810, 0.312837, 0.085264, 0.332292, 0.065150, 0.121903, 0.141393, 0.044823, 0.255018, 0.397210, 0.364583, 0.121689, 0.419002, 
			0.939729, 0.785714, 0.974872, 0.600000, 0.420385, 0.080000, 0.542857, 0.375000, 0.169266, 0.202957, 0.063069, 0.214899, 0.513333, 0.653846, 0.179654, 0.379845, 
			0.846204, 0.660526, 0.940354, 0.568831, 0.718750, 0.133333, 0.857143, 0.500000, 0.161905, 0.258333, 0.070248, 0.561765, 0.292208, 0.533333, 0.500000, 0.500000, 
			0.932605, 0.920000, 0.976675, 0.500000, 0.475420, 0.500000, 0.500000, 0.500000, 0.211279, 0.230159, 0.086086, 0.486667, 0.819826, 0.500000, 0.888889, 0.554660, 
			0.849267, 0.972894, 0.536032, 0.995159, 0.621831, 0.985835, 0.310635, 0.999166, 0.923365, 0.413662, 0.945679, 0.809380, 0.987505, 0.621570, 0.988592, 0.933204, 
			0.968130, 0.819687, 0.836236, 0.808333, 0.716569, 0.733333, 0.451513, 0.670000, 0.463536, 0.248600, 0.748110, 0.186869, 0.820000, 0.138095, 0.638528, 0.625000, 
			0.930815, 0.759046, 0.619553, 0.822222, 0.766667, 0.500000, 0.535715, 0.875000, 0.806327, 0.141846, 0.394841, 0.100000, 0.866839, 0.280000, 0.750000, 0.500000, 
			0.970485, 0.893982, 0.747798, 0.500000, 0.766541, 0.687500, 0.408333, 0.500000, 0.574765, 0.067168, 0.243056, 0.185185, 0.799794, 0.161111, 0.500000, 0.420635, 
			0.539550, 0.952331, 0.240476, 0.850321, 0.222985, 0.273146, 0.064286, 0.373469, 0.743805, 0.196930, 0.806878, 0.560140, 0.921429, 0.426191, 0.729437, 0.618750, 
			0.963455, 0.898611, 0.567708, 0.900000, 0.350000, 0.500000, 0.163636, 0.500000, 0.122222, 0.150414, 0.479167, 0.300000, 0.500000, 0.333333, 0.500000, 0.500000, 
			0.793208, 0.762037, 0.402983, 0.500000, 0.466667, 0.250000, 0.120000, 0.083333, 0.187500, 0.126608, 0.333333, 0.148148, 0.500000, 0.142857, 0.500000, 0.500000, 
			0.990990, 0.802469, 0.500000, 0.500000, 0.666667, 0.500000, 0.500000, 0.500000, 0.354978, 0.013816, 0.500000, 0.025000, 0.500000, 0.500000, 0.500000, 0.306822, 
			0.636703, 0.147010, 0.240847, 0.089415, 0.243992, 0.149268, 0.124258, 0.037742, 0.042100, 0.100000, 0.039274, 0.165152, 0.044748, 0.197272, 0.013776, 0.501010, 
			0.945303, 0.658730, 0.849359, 0.500000, 0.504075, 0.217424, 0.373016, 0.031250, 0.147964, 0.075556, 0.204933, 0.125000, 0.152191, 0.133333, 0.303030, 0.500000, 
			0.930802, 0.337500, 0.700758, 0.275000, 0.550000, 0.158730, 0.500000, 0.250000, 0.054466, 0.500000, 0.050000, 0.333333, 0.030970, 0.250000, 0.188000, 0.200000, 
			0.948133, 0.688552, 0.872175, 0.625641, 0.856055, 0.666667, 0.666667, 0.513158, 0.159087, 0.125000, 0.061303, 0.500000, 0.117048, 0.500000, 0.025000, 0.517771, 
			0.869002, 0.519140, 0.375161, 0.435642, 0.683283, 0.230025, 0.900000, 0.079631, 0.136143, 0.237143, 0.192905, 0.350000, 0.215328, 0.575397, 0.141692, 0.535353, 
			0.971217, 0.625000, 0.825926, 0.555556, 0.732395, 0.250000, 0.357143, 0.500000, 0.084091, 0.027778, 0.255102, 0.166667, 0.418704, 0.500000, 0.076389, 0.664368, 
			0.972403, 0.292857, 0.776190, 0.666667, 0.750000, 0.500000, 0.500000, 0.250000, 0.041396, 0.388889, 0.222222, 0.250000, 0.225371, 0.166667, 0.483333, 0.500000, 
			0.993186, 0.928572, 0.910000, 0.652174, 0.498576, 0.500000, 0.500000, 0.222222, 0.130658, 0.500000, 0.190657, 0.083333, 0.840562, 0.125000, 0.212121, 0.500000
			};

		// create: the probability table for zero symbol
		double a_dTableOfZero[nSizeOfTable] = 
			{
			0.999991, 0.956224, 0.998297, 0.991058, 0.980011, 0.853440, 0.813586, 0.365352, 0.656942, 0.891559, 0.840714, 0.942092, 0.830314, 0.905460, 0.688254, 0.922982, 
			0.922414, 0.845966, 0.990171, 0.955249, 0.986482, 0.878657, 0.850887, 0.429379, 0.972154, 0.959721, 0.991174, 0.989276, 0.989781, 0.991561, 0.990917, 0.986350, 
			0.960635, 0.856115, 0.997129, 0.994122, 0.866988, 0.376102, 0.886242, 0.432505, 0.793074, 0.901964, 0.986356, 0.992788, 0.949338, 0.958010, 0.962083, 0.987631, 
			0.940701, 0.858963, 0.995289, 0.977718, 0.974372, 0.798294, 0.893324, 0.591061, 0.990309, 0.979131, 0.998702, 0.999641, 0.995198, 0.998932, 0.988751, 0.997911, 
			0.943733, 0.450684, 0.869813, 0.396695, 0.583394, 0.986477, 0.774403, 0.886616, 0.096148, 0.118451, 0.274484, 0.601632, 0.175701, 0.896462, 0.303642, 0.991566, 
			0.737555, 0.138820, 0.801075, 0.142630, 0.106807, 0.857345, 0.254167, 0.748623, 0.020555, 0.014225, 0.472619, 0.346274, 0.733325, 0.943941, 0.666667, 0.983165, 
			0.808951, 0.046218, 0.698520, 0.174254, 0.208309, 0.597499, 0.714286, 0.811109, 0.042655, 0.024369, 0.271429, 0.215054, 0.102313, 0.904487, 0.711111, 0.988024, 
			0.786671, 0.387863, 0.500000, 0.218205, 0.045589, 0.830060, 0.250000, 0.723810, 0.112519, 0.195926, 0.787500, 0.822222, 0.756345, 0.931944, 0.888889, 0.994257, 
			0.839856, 0.904622, 0.776865, 0.849842, 0.042296, 0.019567, 0.216235, 0.276963, 0.719004, 0.033864, 0.168471, 0.370549, 0.786017, 0.494961, 0.798125, 0.737013, 
			0.298328, 0.318461, 0.100534, 0.372222, 0.012106, 0.045211, 0.153247, 0.319061, 0.194740, 0.074050, 0.037790, 0.265997, 0.750295, 0.550292, 0.847053, 0.625000, 
			0.532081, 0.599652, 0.304935, 0.474130, 0.151237, 0.191432, 0.177453, 0.298140, 0.665664, 0.374591, 0.195105, 0.511323, 0.853996, 0.575972, 0.927161, 0.784091, 
			0.509659, 0.412259, 0.098481, 0.409524, 0.047259, 0.131746, 0.184494, 0.300000, 0.865237, 0.595811, 0.562650, 0.411305, 0.855571, 0.835789, 0.962309, 0.999614, 
			0.990574, 0.962993, 0.593648, 0.893934, 0.166052, 0.978652, 0.391303, 0.995754, 0.895526, 0.709957, 0.860331, 0.755952, 0.956617, 0.862594, 0.984055, 0.984955, 
			0.626471, 0.743750, 0.233427, 0.745238, 0.010066, 0.614286, 0.082912, 0.951282, 0.700194, 0.443997, 0.265488, 0.457143, 0.789960, 0.812935, 0.951049, 0.833333, 
			0.643186, 0.867933, 0.219893, 0.840440, 0.246875, 0.850000, 0.099174, 0.500000, 0.949952, 0.843561, 0.275000, 0.736364, 0.929338, 0.714286, 0.500000, 0.500000, 
			0.759124, 0.500000, 0.143987, 0.845238, 0.022000, 0.357143, 0.090909, 0.958333, 0.841739, 0.400000, 0.750000, 0.746032, 0.828868, 0.711111, 0.995544, 0.988080, 
			0.940243, 0.926171, 0.992953, 0.991305, 0.457846, 0.008024, 0.703488, 0.028312, 0.172226, 0.455027, 0.171073, 0.460205, 0.367709, 0.744057, 0.114647, 0.762953, 
			0.348548, 0.880062, 0.707092, 0.860119, 0.722148, 0.051246, 0.945238, 0.412863, 0.728313, 0.737033, 0.524648, 0.687017, 0.865338, 0.933715, 0.852917, 0.943104, 
			0.711773, 0.986634, 0.961035, 0.993460, 0.621429, 0.257618, 0.884091, 0.368681, 0.635203, 0.963881, 0.735645, 0.718551, 0.582416, 0.966151, 0.660417, 0.852505, 
			0.179699, 0.885382, 0.712540, 0.846134, 0.608608, 0.048090, 0.840000, 0.215202, 0.769251, 0.990667, 0.798152, 0.990051, 0.823602, 0.988579, 0.908163, 0.989882, 
			0.910631, 0.315900, 0.890404, 0.394957, 0.274706, 0.663421, 0.487500, 0.772669, 0.195382, 0.526316, 0.052790, 0.424819, 0.188927, 0.788879, 0.086539, 0.765491, 
			0.162037, 0.318452, 0.722789, 0.444444, 0.106614, 0.366667, 0.196429, 0.111111, 0.371429, 0.707441, 0.405556, 0.266667, 0.462857, 0.973230, 0.208333, 0.500000, 
			0.870714, 0.651833, 0.943563, 0.430556, 0.333333, 0.714286, 0.436111, 0.500000, 0.117712, 0.629330, 0.007937, 0.357143, 0.277244, 0.983217, 0.500000, 0.500000, 
			0.491919, 0.333333, 0.500000, 0.166667, 0.019841, 0.277778, 0.166667, 0.444444, 0.451923, 0.866667, 0.375000, 0.625000, 0.312500, 0.994949, 0.500000, 0.973663, 
			0.726931, 0.982689, 0.975883, 0.910192, 0.012851, 0.583718, 0.199753, 0.675617, 0.962841, 0.596803, 0.668462, 0.773904, 0.950707, 0.666639, 0.809273, 0.706084, 
			0.135818, 0.690058, 0.458226, 0.750000, 0.010735, 0.124278, 0.400000, 0.642857, 0.864271, 0.516135, 0.183094, 0.585816, 0.877043, 0.467097, 0.678571, 0.615602, 
			0.177776, 0.935255, 0.528650, 0.873609, 0.021143, 0.111765, 0.125000, 0.744898, 0.942504, 0.879531, 0.385714, 0.737245, 0.975392, 0.952331, 0.666667, 0.714286, 
			0.127510, 0.750000, 0.563390, 0.800000, 0.043931, 0.231579, 0.194445, 0.480000, 0.923525, 0.829554, 0.691667, 0.748571, 0.974567, 0.941609, 0.945055, 0.917297, 
			0.877538, 0.989121, 0.980806, 0.912457, 0.131115, 0.926013, 0.653472, 0.993971, 0.987890, 0.945840, 0.951478, 0.965368, 0.989532, 0.823646, 0.983269, 0.742577, 
			0.462950, 0.500000, 0.683333, 0.833333, 0.024110, 0.500000, 0.457143, 0.777778, 0.836873, 0.988235, 0.300000, 0.864583, 0.970069, 0.972032, 0.841585, 0.727473, 
			0.395079, 0.979144, 0.803307, 0.974859, 0.185455, 0.875000, 0.500000, 0.908163, 0.936643, 0.680000, 0.366667, 0.750000, 0.985183, 0.989131, 0.500000, 0.800000, 
			0.419555, 0.812500, 0.850000, 0.833333, 0.029844, 0.680000, 0.116667, 0.800000, 0.895039, 0.500000, 0.500000, 0.500000, 0.962870, 0.500000, 0.980952, 0.981176, 
			0.697138, 0.071952, 0.457003, 0.105740, 0.897473, 0.176067, 0.795664, 0.011455, 0.300449, 0.748367, 0.261780, 0.755806, 0.077964, 0.562886, 0.046933, 0.598126, 
			0.403830, 0.171855, 0.370427, 0.212359, 0.882637, 0.700013, 0.840737, 0.641761, 0.947707, 0.904223, 0.804666, 0.934762, 0.766846, 0.889583, 0.816667, 0.691065, 
			0.232464, 0.071652, 0.664719, 0.040929, 0.724049, 0.045635, 0.809524, 0.189776, 0.742318, 0.851299, 0.881850, 0.894913, 0.347468, 0.490659, 0.497778, 0.776507, 
			0.221272, 0.154468, 0.602264, 0.068601, 0.584206, 0.391667, 0.820833, 0.365646, 0.865679, 0.964987, 0.931429, 0.981559, 0.617894, 0.950397, 0.677778, 0.894411, 
			0.813089, 0.162559, 0.535662, 0.126802, 0.984357, 0.995180, 0.500000, 0.908380, 0.899095, 0.949697, 0.568627, 0.899264, 0.662055, 0.971095, 0.486926, 0.775137, 
			0.140446, 0.095417, 0.454373, 0.141560, 0.922558, 0.976351, 0.961539, 0.968333, 0.977180, 0.982598, 0.928571, 0.979550, 0.866111, 0.884791, 0.500000, 0.855401, 
			0.411310, 0.091472, 0.662270, 0.037690, 0.943454, 0.900000, 0.500000, 0.666667, 0.920254, 0.891440, 0.925000, 0.907148, 0.619148, 0.875000, 0.500000, 0.611656, 
			0.125224, 0.051111, 0.413333, 0.114734, 0.852674, 0.933333, 0.500000, 0.916667, 0.918632, 0.979131, 0.870833, 0.988961, 0.521212, 0.880808, 0.875000, 0.982015, 
			0.825066, 0.852701, 0.283383, 0.902847, 0.925417, 0.955244, 0.792857, 0.965707, 0.979268, 0.977146, 0.983002, 0.965961, 0.941351, 0.878599, 0.959261, 0.716197, 
			0.277656, 0.288026, 0.085950, 0.563095, 0.849286, 0.965833, 0.328125, 0.931973, 0.920521, 0.912098, 0.915332, 0.870037, 0.783333, 0.305625, 0.950916, 0.622222, 
			0.379329, 0.451688, 0.082999, 0.291667, 0.720443, 0.982857, 0.420635, 0.923077, 0.860714, 0.947513, 0.947222, 0.766667, 0.889872, 0.800000, 0.916667, 0.750000, 
			0.157739, 0.191246, 0.047097, 0.275000, 0.574852, 0.819215, 0.384286, 0.900000, 0.914577, 0.950758, 0.930087, 0.750000, 0.811565, 0.555556, 0.933333, 0.686607, 
			0.443621, 0.454574, 0.079005, 0.541190, 0.687163, 0.914736, 0.667708, 0.934850, 0.878097, 0.858607, 0.955177, 0.744982, 0.602790, 0.635417, 0.878311, 0.580998, 
			0.060271, 0.214286, 0.025128, 0.400000, 0.579615, 0.920000, 0.457143, 0.625000, 0.830734, 0.797043, 0.936931, 0.785101, 0.486667, 0.346154, 0.820346, 0.620155, 
			0.153796, 0.339474, 0.059646, 0.431169, 0.281250, 0.866667, 0.142857, 0.500000, 0.838095, 0.741667, 0.929752, 0.438235, 0.707792, 0.466667, 0.500000, 0.500000, 
			0.067395, 0.080000, 0.023325, 0.500000, 0.524580, 0.500000, 0.500000, 0.500000, 0.788721, 0.769841, 0.913914, 0.513333, 0.180174, 0.500000, 0.111111, 0.445341, 
			0.150733, 0.027106, 0.463968, 0.004841, 0.378169, 0.014165, 0.689365, 0.000834, 0.076636, 0.586338, 0.054321, 0.190620, 0.012495, 0.378430, 0.011408, 0.066796, 
			0.031870, 0.180313, 0.163764, 0.191667, 0.283431, 0.266667, 0.548487, 0.330000, 0.536464, 0.751400, 0.251891, 0.813132, 0.180000, 0.861905, 0.361472, 0.375000, 
			0.069185, 0.240954, 0.380447, 0.177778, 0.233333, 0.500000, 0.464286, 0.125000, 0.193673, 0.858154, 0.605159, 0.900000, 0.133161, 0.720000, 0.250000, 0.500000, 
			0.029515, 0.106019, 0.252202, 0.500000, 0.233459, 0.312500, 0.591667, 0.500000, 0.425235, 0.932832, 0.756944, 0.814815, 0.200206, 0.838889, 0.500000, 0.579365, 
			0.460450, 0.047669, 0.759524, 0.149680, 0.777015, 0.726854, 0.935714, 0.626531, 0.256195, 0.803071, 0.193122, 0.439860, 0.078571, 0.573809, 0.270563, 0.381250, 
			0.036546, 0.101389, 0.432292, 0.100000, 0.650000, 0.500000, 0.836364, 0.500000, 0.877778, 0.849586, 0.520833, 0.700000, 0.500000, 0.666667, 0.500000, 0.500000, 
			0.206792, 0.237963, 0.597017, 0.500000, 0.533333, 0.750000, 0.880000, 0.916667, 0.812500, 0.873392, 0.666667, 0.851852, 0.500000, 0.857143, 0.500000, 0.500000, 
			0.009010, 0.197531, 0.500000, 0.500000, 0.333333, 0.500000, 0.500000, 0.500000, 0.645022, 0.986184, 0.500000, 0.975000, 0.500000, 0.500000, 0.500000, 0.693178, 
			0.363297, 0.852990, 0.759153, 0.910585, 0.756008, 0.850732, 0.875742, 0.962258, 0.957901, 0.900000, 0.960726, 0.834849, 0.955252, 0.802729, 0.986224, 0.498990, 
			0.054697, 0.341270, 0.150641, 0.500000, 0.495925, 0.782576, 0.626984, 0.968750, 0.852036, 0.924444, 0.795067, 0.875000, 0.847809, 0.866667, 0.696970, 0.500000, 
			0.069198, 0.662500, 0.299243, 0.725000, 0.450000, 0.841270, 0.500000, 0.750000, 0.945534, 0.500000, 0.950000, 0.666667, 0.969030, 0.750000, 0.812000, 0.800000, 
			0.051867, 0.311448, 0.127825, 0.374359, 0.143945, 0.333333, 0.333333, 0.486842, 0.840913, 0.875000, 0.938697, 0.500000, 0.882952, 0.500000, 0.975000, 0.482229, 
			0.130998, 0.480860, 0.624839, 0.564358, 0.316717, 0.769975, 0.100000, 0.920369, 0.863857, 0.762857, 0.807095, 0.650000, 0.784672, 0.424603, 0.858308, 0.464647, 
			0.028783, 0.375000, 0.174074, 0.444444, 0.267605, 0.750000, 0.642857, 0.500000, 0.915909, 0.972222, 0.744898, 0.833333, 0.581296, 0.500000, 0.923611, 0.335632, 
			0.027597, 0.707143, 0.223810, 0.333333, 0.250000, 0.500000, 0.500000, 0.750000, 0.958604, 0.611111, 0.777778, 0.750000, 0.774629, 0.833333, 0.516667, 0.500000, 
			0.006814, 0.071429, 0.090000, 0.347826, 0.501424, 0.500000, 0.500000, 0.777778, 0.869342, 0.500000, 0.809344, 0.916667, 0.159438, 0.875000, 0.787879, 0.500000, 
			};

		// initialize: the probability table
		for(int i = 0; i < m_nSizeOfTable; i++)
			{
			if(a_dTableOfOne[i] == 0.0 || a_dTableOfZero[i] == 0.0)
				m_a_dProbabilityTable[i] = 0.0;
			else
				m_a_dProbabilityTable[i] = -1.0 * ( (a_dTableOfOne[i])*(log(a_dTableOfOne[i])/log(2.0))
													+ (a_dTableOfZero[i])*(log(a_dTableOfZero[i])/log(2.0)) );
			}
		}

	~CAdjustableOctree()
		{
		if(m_a_ucOctree != NULL)
			{
			delete[] m_a_ucOctree;
			m_a_ucOctree = NULL;
			}

		if(m_a_ucMeanColor != NULL)
			{
			delete[] m_a_ucMeanColor;
			m_a_ucMeanColor = NULL;
			}

		if(m_a_ucSOP != NULL)
			{
			delete[] m_a_ucSOP;
			m_a_ucSOP = NULL;
			}

		if(m_a_nCountOfTreeSize != NULL)
			{
			delete[] m_a_nCountOfTreeSize;
			m_a_nCountOfTreeSize = NULL;
			}

		if(m_a_dProbabilityTable != NULL)
			{
			delete[] m_a_dProbabilityTable;
			m_a_dProbabilityTable = NULL;
			}
		}

	// initialize: the octree data with width, height, depth
	int initialize(int nWidth, int nHeight, int nDepthNbBits, CVoxel& m_oVoxel, CColor& m_oColor)
		{
		// variable: the level of tree
		int nLevelOfTree = 0;

		// initialize: information of the resolution
		m_nWidth = nWidth;
		m_nHeight = nHeight;
		m_nDepth = (int)pow(2, nDepthNbBits+1);

		// set: the voxel data and the color data
		m_pVoxel = &m_oVoxel;
		m_pColor = &m_oColor;

		// get: the sizes of the octree, the mean color, the SOP and the count
		nLevelOfTree = getLevelOfTree();
		if(m_nThresholdLevelOfPPM > nLevelOfTree)
			m_nThresholdLevelOfPPM = nLevelOfTree-1;
		m_nSizeOfOctree = getSizeOfTree(nLevelOfTree);
		m_nSizeOfMeanColor = getSizeOfTree(nLevelOfTree - m_nThresholdLevelOfPPM - 1);	// nLevelOfTree - 4 - 1
		m_nSizeOfSOP = getSizeOfTree(nLevelOfTree - 1);									// nLevelOfTree - 1
		m_nSizeOfCount = getSizeOfTree(nLevelOfTree - m_nThresholdLevelOfPPM);			// nLevelOfTree - 4

		// get: the sizes by bytes
		int nSizeOfOctreeByBytes = (int)ceil((double)m_nSizeOfOctree/8);
		int nSizeOfSOPByBytes = (int)ceil((double)m_nSizeOfSOP/8);

		// allocate: the octree data, the empty data, the mean color and the sop data
		m_a_ucOctree = new unsigned char[nSizeOfOctreeByBytes];
		m_a_ucMeanColor = new unsigned char[m_nSizeOfMeanColor*3];
		m_a_ucSOP = new unsigned char[nSizeOfSOPByBytes];
		m_a_nCountOfTreeSize = new int[m_nSizeOfCount];

		// initialize: the data with 0
		memset((void*)m_a_ucOctree, 0, sizeof(unsigned char)*nSizeOfOctreeByBytes);
		memset((void*)m_a_ucMeanColor, 0, sizeof(unsigned char)*m_nSizeOfMeanColor*3);
		memset((void*)m_a_ucSOP, 0, sizeof(unsigned char)*nSizeOfSOPByBytes);
		memset((void*)m_a_nCountOfTreeSize, 0, sizeof(int)*m_nSizeOfCount);

		// variable: a tree level
		int nTreeLevel = 0;

		// variables: the coordinates for the inital octree
		int nX0 = 0;
		int nY0 = 0;
		int nZ0 = 0;
		int nX1 = m_nWidth-1;
		int nY1 = m_nHeight-1;
		int nZ1 = m_nDepth-1;

		// initialize: the empty label data - (0, 0, 0) ~ (m_nWidth-1, m_nHeight-1, m_nDepth-1)
		m_nNumberOfTreeNodes = initializeEmptyLabels(nTreeLevel, nX0, nY0, nZ0, nX1, nY1, nZ1, 0); // 0 == root

		return m_nNumberOfTreeNodes;
		}

	// read: the bitstream of split node
	void readBitstreamOfSplitNode(cInStream& fin, int nIndexOfBits, int& nCount, CQueue& oQueue, context* contextBOW, context* contextRedOfRegion, context* contextGreenOfRegion, context* contextBlueOfRegion)
		{
		// decode: the mean color
		int nRed = decode(contextRedOfRegion, fin);
		int nGreen = decode(contextGreenOfRegion, fin);
		int nBlue = decode(contextBlueOfRegion, fin);

		// set: the mean color
		setMeanColor(nRed, nGreen, nBlue, nIndexOfBits);

		// encode: the voxel and the color in the region
		for(int nChild = 1; nChild <= 8; nChild++)
			{
			// decode: the BOW
			if(decode(contextBOW, fin) == 0)	// 0 == White
				{
				// reset: the octree voxel of an index
				// resetOctreeVoxel(nIndexOfBits*8+nChild);	// set 0 (= White)

				// get: the count
				nCount += getCountOfTreeSize(nIndexOfBits*8+nChild);
				}
			else								// 1 == Black
				{
				// set: the octree voxel of an index
				setOctreeVoxel(nIndexOfBits*8+nChild);		// set 1 (= Black)

				// push: the child into queue
				oQueue.push(nIndexOfBits*8+nChild);
				}
			}
		}

	// read: the bitstream of PPM node
	void readBitstreamOfPPMNode(cInStream& fin, int nIndexOfBits, context** contextTreeNodes, context* contextColorDifference)
		{
		// variables: the range of the voxel data
		int nX0 = 0;
		int nY0 = 0;
		int nZ0 = 0;
		int nX1 = 0;
		int nY1 = 0;
		int nZ1 = 0;

		// variables: the colors
		int nRed = 0;
		int nGreen = 0;
		int nBlue = 0;
		static int nRedPrevious = 0;
		static int nGreenPrevious = 0;
		static int nBluePrevious = 0;

		// get: the region from an index
		getRegion(nX0, nY0, nZ0, nX1, nY1, nZ1, nIndexOfBits);

		// encode: the voxel and the color in the region
		for(int k = nZ0; k <= nZ1; k++)
			{
			for(int j = nY0; j <= nY1; j++)
				{
				for(int i = nX0; i <= nX1; i++)
					{
					// get: the index of the context
					unsigned int unIndexOfContext = getIndexOfContext(m_pVoxel, i, j, k);
					// decode: the voxel with the context
					int nVoxel = decode(contextTreeNodes[unIndexOfContext], fin);

					// decode: the color in the voxel
					if(nVoxel != 0)
						{
						int nColor = 0;

						// set: the voxel of (i, j, k)
						m_pVoxel->set(i, j, k);
						m_pColor->set(i, j, k);

						// decode: the red color
						nColor = decode(contextColorDifference, fin);

						// restore: the red color of the voxel
						if(nColor > 255)
							nColor = 255 - nColor;
						nRed = nColor+nRedPrevious;
						nRedPrevious = nRed;

						// decode: the green color
						nColor = decode(contextColorDifference, fin);

						// restore: the red color of the voxel
						if(nColor > 255)
							nColor = 255 - nColor;
						nGreen = nColor+nGreenPrevious;
						nGreenPrevious = nGreen;

						// decode: the blue color
						nColor = decode(contextColorDifference, fin);

						// restore: the red color of the voxel
						if(nColor > 255)
							nColor = 255 - nColor;
						nBlue = nColor+nBluePrevious;
						nBluePrevious = nBlue;

						// set: the color of the voxel
						m_pColor->setColor(nRed, nGreen, nBlue, i, j, k);
						}
					}
				}
			}
		}

	// make: the adjustable octree
	bool makeAdjustableOctree(CQueue& oQueue, bool bSpeedUp = false)
		{
		// variable: the boolean value of the octree
		bool bOctree = false;

		// variables: a mean color
		int nR = 0; // red
		int nG = 0; // green
		int nB = 0; // blue

		// variables: the range of the voxel data
		int nX0 = 0;
		int nY0 = 0;
		int nZ0 = 0;
		int nX1 = 0;
		int nY1 = 0;
		int nZ1 = 0;

		// check: the queue
		while(oQueue.isEmpty() != true)
			{
			// get: an index from queue
			int nIndex = oQueue.pop();

			// get: the region from an index
			getRegion(nX0, nY0, nZ0, nX1, nY1, nZ1, nIndex);

			// get: the index of the parent node
			int nIndexOfParent = (nIndex-1) / 8;

			// get: the mean color from the index of the parent node
			getMeanColor(nR, nG, nB, nIndexOfParent);

			// make: the adjustable octree from the root node
			if(bSpeedUp == true)
				bOctree = makeAdjustableOctreeWithBox(nR, nG, nB, nX0, nY0, nZ0, nX1, nY1, nZ1);	// the approximate function
			else
				bOctree = makeAdjustableOctree(nR, nG, nB, nX0, nY0, nZ0, nX1, nY1, nZ1, nIndex);	// the original function
			}

		return bOctree;
		}

	// make: the adjustable octree -> The purpose of this function is to speed up
	bool makeAdjustableOctreeWithBox(int nR, int nG, int nB, int nX0, int nY0, int nZ0, int nX1, int nY1, int nZ1)
		{
		// variables: indices
		int i = 0;
		int j = 0;
		int k = 0;

		// draw: the front surface in the box
		for(j = nY0; j <= nY1; j++)
			{
			for(i = nX0; i <= nX1; i++)
				{
				// set: the voxel of (i, j, nZ0)
				m_pVoxel->set(i, j, nZ0);
				m_pColor->set(i, j, nZ0);

				// set: the color
				m_pColor->setColor(nR, nG, nB, i, j, nZ0);
				}
			}

		// draw: four side surfaces in the box
		for(k = nZ0+1; k <= nZ1-1; k++)
			{
			// draw: the top side surface and the bottom side surface
			for(i = nX0; i <= nX1; i++)
				{
				// set: the voxel of (i, nY0, k)
				m_pVoxel->set(i, nY0, k);
				m_pColor->set(i, nY0, k);

				// set: the color
				m_pColor->setColor(nR, nG, nB, i, nY0, k);

				// set: the voxel of (i, nY1, k)
				m_pVoxel->set(i, nY1, k);
				m_pColor->set(i, nY1, k);

				// set: the color
				m_pColor->setColor(nR, nG, nB, i, nY1, k);
				}

			// draw: the left side surface and the right side surface
			for(j = nY0+1; j <= nY1-1; j++)
				{
				// set: the voxel of (nX0, j, k)
				m_pVoxel->set(nX0, j, k);
				m_pColor->set(nX0, j, k);

				// set: the color
				m_pColor->setColor(nR, nG, nB, nX0, j, k);

				// set: the voxel of (nX1, j, k)
				m_pVoxel->set(nX1, j, k);
				m_pColor->set(nX1, j, k);

				// set: the color
				m_pColor->setColor(nR, nG, nB, nX1, j, k);
				}
			}

		// draw: the rear surface in the box
		for(j = nY0; j <= nY1; j++)
			{
			for(i = nX0; i <= nX1; i++)
				{
				// set: the voxel of (i, j, k)
				m_pVoxel->set(i, j, nZ1);
				m_pColor->set(i, j, nZ1);

				// set: the color
				m_pColor->setColor(nR, nG, nB, i, j, nZ1);
				}
			}

		return true;
		}

	// make: the adjustable octree -> This function fills the box with full voxels
	bool makeAdjustableOctree(int nR, int nG, int nB, int nX0, int nY0, int nZ0, int nX1, int nY1, int nZ1, int nIndexOfBits)
		{
		// check: the index of the octree
		if(nIndexOfBits < 0 || nIndexOfBits >= m_nSizeOfOctree)
			return false; // out of range

		// check: the space of the voxel
		if(nX0 > nX1 || nY0 > nY1 || nZ0 > nZ1)
			return false; // empty
		else if((nX0 == nX1) && (nY0 == nY1) && (nZ0 == nZ1))
			{
			// set: the voxel of (nX0, nY0, nZ0)
			m_pVoxel->set(nX0, nY0, nZ0);
			m_pColor->set(nX0, nY0, nZ0);

			// set: the color
			m_pColor->setColor(nR, nG, nB, nX0, nY0, nZ0);

			// set: the octree value of (nX0, nY0, nZ0)
			// setOctreeVoxel(nIndexOfBits);	// set 1 (= Black) --> no necessity!
			}
		else // if(nX0 < nX1 || nY0 < nY1 || nZ0 < nZ1)
			{
			// get: the middle points
			int nXMid = nX0 + (nX1-nX0)/2;
			int nYMid = nY0 + (nY1-nY0)/2;
			int nZMid = nZ0 + (nZ1-nZ0)/2;

			// make: the sub tree with recursive call
			makeAdjustableOctree(nR, nG, nB, nX0, nYMid+1, nZMid+1, nXMid, nY1, nZ1, nIndexOfBits*8+1);		// 1st child
			makeAdjustableOctree(nR, nG, nB, nXMid+1, nYMid+1, nZMid+1, nX1, nY1, nZ1, nIndexOfBits*8+2);	// 2nd child
			makeAdjustableOctree(nR, nG, nB, nX0, nY0, nZMid+1, nXMid, nYMid, nZ1, nIndexOfBits*8+3);		// 3rd child
			makeAdjustableOctree(nR, nG, nB, nXMid+1, nY0, nZMid+1, nX1, nYMid, nZ1, nIndexOfBits*8+4);		// 4th child
			makeAdjustableOctree(nR, nG, nB, nX0, nYMid+1, nZ0, nXMid, nY1, nZMid, nIndexOfBits*8+5);		// 5th child
			makeAdjustableOctree(nR, nG, nB, nXMid+1, nYMid+1, nZ0, nX1, nY1, nZMid, nIndexOfBits*8+6);		// 6th child
			makeAdjustableOctree(nR, nG, nB, nX0, nY0, nZ0, nXMid, nYMid, nZMid, nIndexOfBits*8+7);			// 7th child
			makeAdjustableOctree(nR, nG, nB, nXMid+1, nY0, nZ0, nX1, nYMid, nZMid, nIndexOfBits*8+8);		// 8th child

			// set: the octree voxel of an index
			// setOctreeVoxel(nIndexOfBits);		// set 1 (= Black) --> no necessity!
			}

		return true; // black
		}

	// set: an octree voxel of an index
	void setOctreeVoxel(int nIndexOfBits)
		{
		if(nIndexOfBits < 0 || nIndexOfBits >= m_nSizeOfOctree)
			return;

		// get: the position from an index
		int nIndex= nIndexOfBits / 8;
		int nBitPos = nIndexOfBits % 8;
		unsigned char ucPos = 1 << nBitPos;

		// get: the value from the octree data
		unsigned char ucByte = m_a_ucOctree[nIndex];

		// set: the value into the octree data
		m_a_ucOctree[nIndex] = ucByte | ucPos;	// set 1
		}

	// get: an octree voxel from an index
	int getOctreeVoxel(int nIndexOfBits)
		{
		int nValue = 0;
		if(nIndexOfBits < 0 || nIndexOfBits >= m_nSizeOfOctree)
			return 0; // empty or white

		// get: the position from an index
		int nIndex= nIndexOfBits / 8;
		int nBitPos = nIndexOfBits % 8;
		unsigned char ucPos = 1 << nBitPos;

		// get: the value from the octree data
		unsigned char ucByte = m_a_ucOctree[nIndex];
		nValue = ucByte & ucPos;

		return nValue;
		}

	// set: an mean color of an index
	void setMeanColor(int nR, int nG, int nB, int nIndexOfBits)
		{
		if(nIndexOfBits < 0 || nIndexOfBits >= m_nSizeOfMeanColor)
			return;

		// get: the position from an index
		int nIndex= nIndexOfBits * 3;

		// set: the values into the mean color
		m_a_ucMeanColor[nIndex] = nR;	// red
		m_a_ucMeanColor[nIndex+1] = nG;	// green
		m_a_ucMeanColor[nIndex+2] = nB;	// blue
		}

	// get: an mean color from an index
	void getMeanColor(int& nR, int& nG, int& nB, int nIndexOfBits)
		{
		if(nIndexOfBits < 0 || nIndexOfBits >= m_nSizeOfMeanColor)
			return;

		// get: the position from an index
		int nIndex= nIndexOfBits * 3;

		// get: the values from the mean color
		nR = m_a_ucMeanColor[nIndex];	// red
		nG = m_a_ucMeanColor[nIndex+1];	// green
		nB = m_a_ucMeanColor[nIndex+2];	// blue
		}

	// set: an sop of an index
	void setSOP(int nIndexOfBits)
		{
		if(nIndexOfBits < 0 || nIndexOfBits >= m_nSizeOfSOP)
			return;

		// get: the position from an index
		int nIndex= nIndexOfBits / 8;
		int nBitPos = nIndexOfBits % 8;
		unsigned char ucPos = 1 << nBitPos;

		// get: the value from the octree data
		unsigned char ucByte = m_a_ucSOP[nIndex];

		// set: the value into the octree data
		m_a_ucSOP[nIndex] = ucByte | ucPos;	// set 1
		}

	// get: an sop from an index
	int getSOP(int nIndexOfBits)
		{
		int nValue = 0;
		if(nIndexOfBits < 0 || nIndexOfBits >= m_nSizeOfSOP)
			return 0; // Split (= default)

		// get: the position from an index
		int nIndex= nIndexOfBits / 8;
		int nBitPos = nIndexOfBits % 8;
		unsigned char ucPos = 1 << nBitPos;

		// get: the value from the sop data
		unsigned char ucByte = m_a_ucSOP[nIndex];
		nValue = ucByte & ucPos;

		return nValue;
		}

	// set: an count of tree size
	void setCountOfTreeSize(int nCount, int nIndex)
		{
		if(nIndex < 0 || nIndex >= m_nSizeOfCount)
			return;

		// set: the count
		m_a_nCountOfTreeSize[nIndex] = nCount;
		}

	// get: an count of tree size
	int getCountOfTreeSize(int nIndex)
		{
		int nCount = 0;
		if(nIndex < 0 || nIndex >= m_nSizeOfCount)
			return nCount;

		// get: the count
		nCount = m_a_nCountOfTreeSize[nIndex];

		return nCount;
		}

  private:
	// get: the level of tree
	int getLevelOfTree(void)
		{
		int nLevelOfTree = 0;

		// get: the max range among width, height, depth
		int nMaxEdge = (m_nWidth > m_nHeight) ? (m_nWidth > m_nDepth ? m_nWidth : m_nDepth) : (m_nHeight > m_nDepth ? m_nHeight : m_nDepth);

		// get: the exponent from the max edge
		nLevelOfTree = (int)ceil(log10(nMaxEdge)/log10(2));

		return nLevelOfTree;
		}

	// get: the size of tree
	int getSizeOfTree(int nLevelOfTree)
		{
		int nSize = 0;

		// calculate: the size of the octree data
		for(int i = 0; i <= nLevelOfTree; i++)
			{
			// add: the number of the children in each tree level
			int nNumberOfChildren = (int)pow(pow(2, i),3);
			nSize += nNumberOfChildren;
			}

		return nSize;
		}

	// initialize: the empty label data
	int initializeEmptyLabels(int& nTreeLevel, int nX0, int nY0, int nZ0, int nX1, int nY1, int nZ1, int nIndexOfBits)
		{
		int nCount = 0;

		// check: the range of the octree
		if(nIndexOfBits < 0 || nIndexOfBits >= m_nSizeOfOctree)
			return 0;

		// check: whether the voxel has no space
		if(nX0 > nX1 || nY0 > nY1 || nZ0 > nZ1)
			{
			return 0;
			}
		else if((nX0 == nX1) && (nY0 == nY1) && (nZ0 == nZ1))
			{
			// get: the real number of tree nodes
			nCount++;

			// set: the tree level
			nTreeLevel = 0;
			}
		else // if(nX0 < nX1 || nY0 < nY1 || nZ0 < nZ1)
			{
			// get: the real number of tree nodes
			nCount++;

			// variables: a tree level
			int nSubTreeLevel = 0;

			// get: the middle points
			int nXMid = nX0 + (nX1-nX0)/2;
			int nYMid = nY0 + (nY1-nY0)/2;
			int nZMid = nZ0 + (nZ1-nZ0)/2;

			// make: the sub trees emtpy
			nCount += initializeEmptyLabels(nSubTreeLevel, nX0, nYMid+1, nZMid+1, nXMid, nY1, nZ1, nIndexOfBits*8+1);	// 1st child
			nTreeLevel = nSubTreeLevel;
			nCount += initializeEmptyLabels(nSubTreeLevel, nXMid+1, nYMid+1, nZMid+1, nX1, nY1, nZ1, nIndexOfBits*8+2);	// 2nd child
			nTreeLevel = (nSubTreeLevel < nTreeLevel) ? nSubTreeLevel : nTreeLevel;
			nCount += initializeEmptyLabels(nSubTreeLevel, nX0, nY0, nZMid+1, nXMid, nYMid, nZ1, nIndexOfBits*8+3);		// 3rd child
			nTreeLevel = (nSubTreeLevel < nTreeLevel) ? nSubTreeLevel : nTreeLevel;
			nCount += initializeEmptyLabels(nSubTreeLevel, nXMid+1, nY0, nZMid+1, nX1, nYMid, nZ1, nIndexOfBits*8+4);	// 4th child
			nTreeLevel = (nSubTreeLevel < nTreeLevel) ? nSubTreeLevel : nTreeLevel;
			nCount += initializeEmptyLabels(nSubTreeLevel, nX0, nYMid+1, nZ0, nXMid, nY1, nZMid, nIndexOfBits*8+5);		// 5th child
			nTreeLevel = (nSubTreeLevel < nTreeLevel) ? nSubTreeLevel : nTreeLevel;
			nCount += initializeEmptyLabels(nSubTreeLevel, nXMid+1, nYMid+1, nZ0, nX1, nY1, nZMid, nIndexOfBits*8+6);	// 6th child
			nTreeLevel = (nSubTreeLevel < nTreeLevel) ? nSubTreeLevel : nTreeLevel;
			nCount += initializeEmptyLabels(nSubTreeLevel, nX0, nY0, nZ0, nXMid, nYMid, nZMid, nIndexOfBits*8+7);		// 7th child
			nTreeLevel = (nSubTreeLevel < nTreeLevel) ? nSubTreeLevel : nTreeLevel;
			nCount += initializeEmptyLabels(nSubTreeLevel, nXMid+1, nY0, nZ0, nX1, nYMid, nZMid, nIndexOfBits*8+8);		// 8th child
			nTreeLevel = (nSubTreeLevel < nTreeLevel) ? nSubTreeLevel : nTreeLevel;			

			// set: the tree level
			nTreeLevel++; // nTreeLevel <- nSubTreeLevel + 1

			// check: the tree level
			if(nTreeLevel >= m_nThresholdLevelOfPPM || nIndexOfBits == 0)
				{
				// set: the count
				setCountOfTreeSize(nCount, nIndexOfBits);
				}
			}

		return nCount;
		}

	// get: the region from an index
	void getRegion(int& nX0, int& nY0, int& nZ0, int& nX1, int& nY1, int& nZ1, int nIndexOfBits)
		{
		if(nIndexOfBits < 0 || nIndexOfBits >= m_nSizeOfOctree)
			return;

		// variables: a stack pointer, a stack and a temporary index
		int nSizeOfStack = 0;
		int* a_nStack = NULL;
		int nIndex = nIndexOfBits;

		// find: the indices and push them into the stack
		while(nIndex > 0)
			{
			// push: an index into the stack
			nSizeOfStack++;
			a_nStack = (int*)realloc((void*)a_nStack, sizeof(int)*nSizeOfStack);
			a_nStack[nSizeOfStack-1] = nIndex;

			// get: the next index
			nIndex = (nIndex - 1) / 8;
			}

		// initialize: the range of the voxel data
		nX0 = 0;
		nY0 = 0;
		nZ0 = 0;
		nX1 = m_nWidth-1;
		nY1 = m_nHeight-1;
		nZ1 = m_nDepth-1;

		// variable: a base index
		int nIndexOfBase = 0;

		// pop: the indices from the stack and find out the region
		while(nSizeOfStack > 0)
			{
			// pop: a value from the stack
			nIndex = a_nStack[nSizeOfStack-1];

			// get: the middle points
			int nXMid = nX0 + (nX1-nX0)/2;
			int nYMid = nY0 + (nY1-nY0)/2;
			int nZMid = nZ0 + (nZ1-nZ0)/2;

			// make: the sub tree emtpy
			switch(nIndex - nIndexOfBase)
				{
				case 1:
					nX1 = nXMid; nY0 = nYMid+1; nZ0 = nZMid+1;
					break;

				case 2:
					nX0 = nXMid+1; nY0 = nYMid+1; nZ0 = nZMid+1;
					break;

				case 3:
					nX1 = nXMid; nY1 = nYMid; nZ0 = nZMid+1;
					break;

				case 4:
					nX0 = nXMid+1; nY1 = nYMid; nZ0 = nZMid+1;
					break;

				case 5:
					nX1 = nXMid; nY0 = nYMid+1; nZ1 = nZMid;
					break;

				case 6:
					nX0 = nXMid+1; nY0 = nYMid+1; nZ1 = nZMid;
					break;

				case 7:
					nX1 = nXMid; nY1 = nYMid; nZ1 = nZMid;
					break;

				case 8:
					nX0 = nXMid+1; nY1 = nYMid; nZ1 = nZMid;
					break;
				}

			// move: the base index
			nIndexOfBase = nIndex*8;

			// move: the stack pointer
			nSizeOfStack--;
			}

		free(a_nStack);
		}

	// get: the index of the context
	unsigned int getIndexOfContext(CVoxel* pVoxel, int nX, int nY, int nZ)
		{
		unsigned int unIndex = 0;
		unsigned int unBit = 1;

		// set: 1st context of (nX, nY-1, nZ-1)
		if(pVoxel->get(nX, nY-1, nZ-1) != 0)
			unIndex |= unBit;

		// set: 2nd context of (nX+1, nY-1, nZ-1)
		unBit <<= 1;
		if(pVoxel->get(nX+1, nY-1, nZ-1) != 0)
			unIndex |= unBit;

		// set: 3rd context of (nX-1, nY, nZ-1)
		unBit <<= 1;
		if(pVoxel->get(nX-1, nY, nZ-1) != 0)
			unIndex |= unBit;

		// set: 4th context of (nX, nY, nZ-1)
		unBit <<= 1;
		if(pVoxel->get(nX, nY, nZ-1) != 0)
			unIndex |= unBit;

		// set: 5th context of (nX, nY+1, nZ-1)
		unBit <<= 1;
		if(pVoxel->get(nX, nY+1, nZ-1) != 0)
			unIndex |= unBit;

		// set: 6th context of (nX+1, nY+1, nZ-1)
		unBit <<= 1;
		if(pVoxel->get(nX+1, nY+1, nZ-1) != 0)
			unIndex |= unBit;

		// set: 7th context of (nX-1, nY-1, nZ)
		unBit <<= 1;
		if(pVoxel->get(nX-1, nY-1, nZ) != 0)
			unIndex |= unBit;

		// set: 8th context of (nX, nY-1, nZ)
		unBit <<= 1;
		if(pVoxel->get(nX, nY-1, nZ) != 0)
			unIndex |= unBit;

		// set: 9th context of (nX+1, nY-1, nZ)
		unBit <<= 1;
		if(pVoxel->get(nX+1, nY-1, nZ) != 0)
			unIndex |= unBit;

		// set: 10th context of (nX-1, nY, nZ)
		unBit <<= 1;
		if(pVoxel->get(nX-1, nY, nZ) != 0)
			unIndex |= unBit;

		return unIndex;
		}
	};

#endif _CADJUSTABLEOCTREE_H_
