/***********************************************************************
This software module was originally developed by Shinjun Lee
(SAMSUNG AIT) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. SAMSUNG AIT retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright ?2002.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Shinjun Lee at sjlee@sait.samsung.co.kr.
----------------------------------------------------------------------*/

// CColor.h
// Author: Shinjun Lee, Samsung AIT
// Date: 2003-11-11 ~ 2003-12-15

#ifndef _CCOLOR_H_
#define _CCOLOR_H_

#include <iostream.h>
#include <math.h>
#include <limits.h>

class CColor
	{
  private:
	class CElement
		{
	  public:
		int*			m_pDepth;			// depth
		unsigned char*	m_pColor;			// color
		int				m_nSize;			// the size of the depth/color
		int				m_nLengthOfDepth;	// the current length of the depth
		int				m_nLengthOfColor;	// the current length of the color

		CElement(void)
			{
			m_pDepth = NULL;
			m_pColor = NULL;
			m_nSize = 0;
			m_nLengthOfDepth = 0;
			m_nLengthOfColor = 0;
			}

		CElement(int nSize)
			{
			m_pDepth = (int*)calloc(nSize, sizeof(int)); // allocate: depth
			m_pColor = (unsigned char*)calloc(nSize*3, sizeof(unsigned char)); // allocate: color
			m_nSize = nSize;
			m_nLengthOfDepth = 0;
			m_nLengthOfColor = 0;
			}

		~CElement()
			{
			if(m_pDepth != NULL)
				{
				free(m_pDepth);
				m_pDepth = NULL;
				}

			if(m_pColor != NULL)
				{
				free(m_pColor);
				m_pColor = NULL;
				}
			}
		};

	CElement**	m_pElement;					// element array
	int			m_nSizeOfElement;			// memory size for element array
	int			m_nIndexOfElement;			// the current index of the element array

	int			m_nWidth;					// width
	int			m_nHeight;					// height
	int			m_nDepthNbBits;				// bits value for depth
	int			m_nDepth;					// depth

  public:
	CColor(void)
		{
		m_pElement = NULL;
		m_nSizeOfElement = 0;
		m_nIndexOfElement = -1;

		m_nWidth = 0;
		m_nHeight = 0;
		m_nDepthNbBits = 0;
		m_nDepth = 0;
		}

	~CColor()
		{
		int i = 0;

		for(i = 0; i < m_nSizeOfElement; i++)
			{
			if(m_pElement[i] != NULL)
				{
				delete m_pElement[i];
				m_pElement[i] = NULL;
				}
			}

		if(m_pElement != NULL)
			{
			delete[] m_pElement;
			m_pElement = NULL;
			}
		}

	// initialize: the voxel data with width, height, depth
	bool initialize(int nWidth, int nHeight, int nDepthNbBits)
		{
		// initialize: the resolution information
		m_nWidth = nWidth;
		m_nHeight = nHeight;
		m_nDepthNbBits = nDepthNbBits;
		m_nDepth = (int)pow(2, nDepthNbBits+1);

		// allocate: elmenet array
		m_nSizeOfElement = nWidth * nHeight;
		m_pElement = new CElement*[m_nSizeOfElement];
		if(m_pElement == NULL)
			{
			cout << "Color object allocation error." << endl;
			exit(1);
			}
		memset((void*)m_pElement, 0, sizeof(CElement*)*m_nSizeOfElement);

		return true;
		}

	// set: the size of a depth in the position of (nX, nY)
	bool setSizeOfDepth(int nX, int nY, int nSize)
		{
		if(nX < 0 || nX >= m_nWidth || nY < 0 || nY >= m_nHeight)
			return false;

		// allocate: element
		int nIndex = (nX + nY*m_nWidth);
		if(m_nIndexOfElement == -1)
			m_nIndexOfElement = nIndex;
		m_pElement[nIndex] = new CElement(nSize);

		return true;
		}

	// get: the size of a depth in the position of (nX, nY)
	int getSizeOfDepth(int nX, int nY)
		{
		int nSize = 0;
		if(nX < 0 || nX >= m_nWidth || nY < 0 || nY >= m_nHeight)
			return false;

		// get: the size
		int nIndex = (nX + nY*m_nWidth);
		if(m_pElement[nIndex] != NULL)
			nSize = m_pElement[nIndex]->m_nSize;

		return nSize;
		}

	// get: the depth data in the position of (nX, nY)
	int* getDepth(int nX, int nY)
		{
		int* pDepth = NULL;
		if(nX < 0 || nX >= m_nWidth || nY < 0 || nY >= m_nHeight)
			return pDepth;

		// get: the depth data
		int nIndex = (nX + nY*m_nWidth);
		if(m_pElement[nIndex] != NULL)
			pDepth = m_pElement[nIndex]->m_pDepth;

		return pDepth;
		}

	// get: the color data in the position of (nX, nY)
	unsigned char* getColor(int nX, int nY)
		{
		unsigned char* pColor = NULL;
		if(nX < 0 || nX >= m_nWidth || nY < 0 || nY >= m_nHeight)
			return pColor;

		// get: the color data
		int nIndex = (nX + nY*m_nWidth);
		if(m_pElement[nIndex] != NULL)
			pColor = m_pElement[nIndex]->m_pColor;

		return pColor;
		}


	// set: nZ (Z-axis) value in the position of (nX, nY)
	bool set(int nX, int nY, int nZ)
		{
		if(nX < 0 || nX >= m_nWidth || nY < 0 || nY >= m_nHeight || nZ < 0 || nZ >= m_nDepth)
			return false;

		// check: the index, size and the length of depth
		int nIndex = (nX + nY*m_nWidth);
		if(m_pElement[nIndex] == NULL)
			m_pElement[nIndex] = new CElement(1);
		int& nSize = m_pElement[nIndex]->m_nSize;
		int& nLengthOfDepth = m_pElement[nIndex]->m_nLengthOfDepth;
		if(nLengthOfDepth >= nSize)
			{
			nSize = nLengthOfDepth+1;
			m_pElement[nIndex]->m_pDepth = (int*)realloc((void*)m_pElement[nIndex]->m_pDepth, sizeof(int)*nSize); // reallocate: depth
			m_pElement[nIndex]->m_pColor = (unsigned char*)realloc((void*)m_pElement[nIndex]->m_pColor, sizeof(unsigned char)*nSize*3); // reallocate: color
			}

		// set: (nX, nY, nZ)
		m_pElement[nIndex]->m_pDepth[nLengthOfDepth] = nZ;

		// increase: the lenth of the depth
		nLengthOfDepth++;

		return true;
		}

	// get: a value from voxel data in the position of (nX, nY, nZ)
	int get(int nX, int nY, int nZ)
		{
		int nValue = 0;
		if(nX < 0 || nX >= m_nWidth || nY < 0 || nY >= m_nHeight || nZ < 0 || nZ >= m_nDepth)
			return nValue;

		// find: the value in (nX, nY, nZ)
		int nIndex = (nX + nY*m_nWidth);
		if(m_pElement[nIndex] != NULL)
			{
			int nLengthOfDepth = m_pElement[nIndex]->m_nLengthOfDepth;
			for(int i = 0; i < nLengthOfDepth; i++)
				{
				int nDepth = m_pElement[nIndex]->m_pDepth[i];
				if(nDepth == nZ)
					{
					nValue = 1;
					break;
					}
				}
			}

		return nValue;
		}

	// set: a red color value into the voxel data
	bool setRed(int nRed)
		{
		if(nRed < 0 || nRed > 255)
			return false;

		// set: red color
		int nIndex = m_nIndexOfElement;
		if(m_pElement[nIndex] == NULL)
			{
			cout << "Error: the index of the element array." << endl;
			exit(1);
			}
		int nLengthOfColor = m_pElement[nIndex]->m_nLengthOfColor;
		m_pElement[nIndex]->m_pColor[nLengthOfColor*3] = (unsigned char)nRed;

		return true;
		}

	// get: a red color value in the position of (nX, nY, nZ)
	unsigned char getRed(int nX, int nY, int nZ)
		{
		unsigned char ucRed = 0;
		if(nX < 0 || nX >= m_nWidth || nY < 0 || nY >= m_nHeight || nZ < 0 || nZ >= m_nDepth)
			return ucRed;

		// find: the red color value
		int nIndex = (nX + nY*m_nWidth);
		if(m_pElement[nIndex] != NULL)
			{
			int nLengthOfDepth = m_pElement[nIndex]->m_nLengthOfDepth;
			for(int i = 0; i < nLengthOfDepth; i++)
				{
				if(m_pElement[nIndex]->m_pDepth[i] == nZ)
					{
					ucRed = m_pElement[nIndex]->m_pColor[i*3];
					break;
					}
				else if(m_pElement[nIndex]->m_pDepth[i] > nZ)
					break;
				}
			}

		return ucRed;
		}

	// set: a green color value into the voxel data
	bool setGreen(int nGreen)
		{
		if(nGreen < 0 || nGreen > 255)
			return false;

		// set: green color
		int nIndex = m_nIndexOfElement;
		if(m_pElement[nIndex] == NULL)
			{
			cout << "Error: the index of the element array." << endl;
			exit(1);
			}
		int nLengthOfColor = m_pElement[nIndex]->m_nLengthOfColor;
		m_pElement[nIndex]->m_pColor[nLengthOfColor*3+1] = (unsigned char)nGreen;

		return true;
		}

	// get: a green color value in the position of (nX, nY, nZ)
	unsigned char getGreen(int nX, int nY, int nZ)
		{
		unsigned char ucGreen = 0;
		if(nX < 0 || nX >= m_nWidth || nY < 0 || nY >= m_nHeight || nZ < 0 || nZ >= m_nDepth)
			return ucGreen;

		// find: the green color value
		int nIndex = (nX + nY*m_nWidth);
		if(m_pElement[nIndex] != NULL)
			{
			int nLengthOfDepth = m_pElement[nIndex]->m_nLengthOfDepth;
			for(int i = 0; i < nLengthOfDepth; i++)
				{
				if(m_pElement[nIndex]->m_pDepth[i] == nZ)
					{
					ucGreen = m_pElement[nIndex]->m_pColor[i*3+1];
					break;
					}
				else if(m_pElement[nIndex]->m_pDepth[i] > nZ)
					break;
				}
			}

		return ucGreen;
		}

	// set: a blue color value into the voxel data
	bool setBlue(int nBlue)
		{
		if(nBlue < 0 || nBlue > 255)
			return false;

		// set: blue color
		int& nIndex = m_nIndexOfElement;
		if(m_pElement[nIndex] == NULL)
			{
			cout << "Error: the index of the element array." << endl;
			exit(1);
			}
		int& nLengthOfColor = m_pElement[nIndex]->m_nLengthOfColor;
		m_pElement[nIndex]->m_pColor[nLengthOfColor*3+2] = (unsigned char)nBlue;

		// increase: the lenth of the depth
		int nSize = m_pElement[nIndex]->m_nSize;
		if(++nLengthOfColor >= nSize)
			{
			nLengthOfColor = nSize;
			sortDepth(nIndex);
			while(++nIndex < m_nSizeOfElement)
				{
				if(m_pElement[nIndex] != NULL)
					break;
				}
			}

		return true;
		}

	// get: a blue color value in the position of (nX, nY, nZ)
	unsigned char getBlue(int nX, int nY, int nZ)
		{
		unsigned char ucBlue = 0;
		if(nX < 0 || nX >= m_nWidth || nY < 0 || nY >= m_nHeight || nZ < 0 || nZ >= m_nDepth)
			return ucBlue;

		// find: the blue color value
		int nIndex = (nX + nY*m_nWidth);
		if(m_pElement[nIndex] != NULL)
			{
			int nLengthOfDepth = m_pElement[nIndex]->m_nLengthOfDepth;
			for(int i = 0; i < nLengthOfDepth; i++)
				{
				if(m_pElement[nIndex]->m_pDepth[i] == nZ)
					{
					ucBlue = m_pElement[nIndex]->m_pColor[i*3+2];
					break;
					}
				else if(m_pElement[nIndex]->m_pDepth[i] > nZ)
					break;
				}
			}

		return ucBlue;
		}

	// set: the color in the position of (nX, nY, nZ)
	bool setColor(int nR, int nG, int nB, int nX, int nY, int nZ)
		{
		// check: the ranges of the position and the color
		if(nX < 0 || nX >= m_nWidth || nY < 0 || nY >= m_nHeight || nZ < 0 || nZ >= m_nDepth)
			return false;
		if(nR < 0 || nR > 255 || nG < 0 || nG > 255 || nB < 0 || nB > 255)
			return false;

		// set: the color
		int nIndex = (nX + nY*m_nWidth);
		if(m_pElement[nIndex] == NULL)
			{
			cout << "Error: the index of the element array." << endl;
			exit(1);
			}
		int nLengthOfDepth = m_pElement[nIndex]->m_nLengthOfDepth;
		for(int i = 0; i < nLengthOfDepth; i++)
			{
			if(m_pElement[nIndex]->m_pDepth[i] == nZ)
				{
				m_pElement[nIndex]->m_pColor[i*3] = (unsigned char)nR;
				m_pElement[nIndex]->m_pColor[i*3+1] = (unsigned char)nG;
				m_pElement[nIndex]->m_pColor[i*3+2] = (unsigned char)nB;
				break;
				}
			}

		return true;
		}

	// get: the color in the position of (nX, nY, nZ)
	bool getColor(int& nR, int& nG, int& nB, int nX, int nY, int nZ)
		{
		// initialize: the color values
		nR = 0;
		nG = 0;
		nB = 0;

		// check: the ranges of the position and the color
		if(nX < 0 || nX >= m_nWidth || nY < 0 || nY >= m_nHeight || nZ < 0 || nZ >= m_nDepth)
			return false;
		if(nR < 0 || nR > 255 || nG < 0 || nG > 255 || nB < 0 || nB > 255)
			return false;

		// get: the color
		int nIndex = (nX + nY*m_nWidth);
		if(m_pElement[nIndex] == NULL)
			{
			cout << "Error: the index of the element array." << endl;
			exit(1);
			}
		int nLengthOfDepth = m_pElement[nIndex]->m_nLengthOfDepth;
		for(int i = 0; i < nLengthOfDepth; i++)
			{
			if(m_pElement[nIndex]->m_pDepth[i] == nZ)
				{
				nR = m_pElement[nIndex]->m_pColor[i*3];
				nG = m_pElement[nIndex]->m_pColor[i*3+1];
				nB = m_pElement[nIndex]->m_pColor[i*3+2];
				break;
				}
			}

		return true;
		}

  private:
	// sort: depth data
	void sortDepth(int nIndex)
		{
		int nMinimum = 0;							// minimum value
		int nMinimumPrevious = -1;					// previous minimum value
		int nIndexOfMinimum = 0;					// index of the minimum value
		int& nSize = m_pElement[nIndex]->m_nSize;	// the size of the depth array
		for(int i = 0; i < nSize; i++)
			{
			nMinimum = INT_MAX;
			nIndexOfMinimum = i-1;

			// find: the minimum value
			for(int j = i; j < nSize; j++)
				{
				if(nMinimumPrevious < m_pElement[nIndex]->m_pDepth[j] && m_pElement[nIndex]->m_pDepth[j] < nMinimum)
					{
					nMinimum = m_pElement[nIndex]->m_pDepth[j];
					nIndexOfMinimum = j;
					if(nMinimum == nMinimumPrevious + 1)
						break;
					}
				}

			// change: the minimum value
			if(nIndexOfMinimum > i)
				{
				// update: the previous minimum value
				nMinimumPrevious = nMinimum;

				// the index of the minimum value
				j = nIndexOfMinimum;

				// change: depth
				int nTemp = m_pElement[nIndex]->m_pDepth[i];
				m_pElement[nIndex]->m_pDepth[i] = nMinimum;
				m_pElement[nIndex]->m_pDepth[j] = nTemp;

				// change: red color
				nTemp = (int)m_pElement[nIndex]->m_pColor[i*3];
				m_pElement[nIndex]->m_pColor[i*3] = m_pElement[nIndex]->m_pColor[j*3];
				m_pElement[nIndex]->m_pColor[j*3] = (unsigned char)nTemp;

				// change: green color
				nTemp = (int)m_pElement[nIndex]->m_pColor[i*3+1];
				m_pElement[nIndex]->m_pColor[i*3+1] = m_pElement[nIndex]->m_pColor[j*3+1];
				m_pElement[nIndex]->m_pColor[j*3+1] = (unsigned char)nTemp;

				// change: blue color
				nTemp = (int)m_pElement[nIndex]->m_pColor[i*3+2];
				m_pElement[nIndex]->m_pColor[i*3+2] = m_pElement[nIndex]->m_pColor[j*3+2];
				m_pElement[nIndex]->m_pColor[j*3+2] = (unsigned char)nTemp;
				}
			else if(nIndexOfMinimum == i)
				{
				// update: the previous minimum value
				nMinimumPrevious = nMinimum;
				}
			else if(nIndexOfMinimum < i)
				{
				// reallocate: the depth data and the color data
				nSize = i;
				m_pElement[nIndex]->m_pDepth = (int*)realloc((void*)m_pElement[nIndex]->m_pDepth, sizeof(int)*nSize);
				m_pElement[nIndex]->m_pColor = (unsigned char*)realloc((void*)m_pElement[nIndex]->m_pColor, sizeof(unsigned char)*nSize*3);
				m_pElement[nIndex]->m_nLengthOfDepth = nSize;
				m_pElement[nIndex]->m_nLengthOfColor = nSize;
				break;
				}
			}
		}
	};

#endif _CCOLOR_H_
