/***********************************************************************
This software module was originally developed by Shinjun Lee
(SAMSUNG AIT) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. SAMSUNG AIT retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright ?2002.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Shinjun Lee at sjlee@sait.samsung.co.kr.
----------------------------------------------------------------------*/

// CPTDecoder.h: PointTexture Compression Decoder
// Author: Shinjun Lee, Samsung AIT
// Date: 2004-07-15

#ifndef _CPTDECODER_H_
#define _CPTDECODER_H_

#include <stdlib.h>
#include <string.h>
#include <iostream.h>
#include <fstream.h>
#include <time.h>
#include "cCommandLineProcessor.h"
#include "CVoxel.h"
#include "CColor.h"
#include "CAdjustableOctree.h"
#include "CQueue.h"
#include "bitio.h"
#include "arith.h"
#include "stats.h"

class CPTDecoder
	{
  public:
	int					m_nWidth;							// width
	int					m_nHeight;							// height
	int					m_nDepthNbBits;						// bits value for depth
	int					m_nPercentOfDecoding;				// the percent of decoding nodes

  private:
	int					m_nPercentOfCutting;				// the percent of cutting nodes
	int					m_nNumberOfTreeNodes;				// the number of tree nodes
	bool				m_bIsExtension;						// the flag of extension
	bool				m_bBoundingBoxWRL;					// the flag of bounding box wrl
	bool				m_bSpeedUp;							// the flag of speed up

  public:
	CVoxel				m_oVoxel;							// voxel object
	CColor				m_oColor;							// color object

  private:
	CAdjustableOctree	m_oOctree;							// adjustable octree object
	CQueue				m_oQueue;							// queue for reading bitstream

 	// variables: file names
	char				m_szBitstreamFile[_MAX_PATH];		// bitstream file name
	char				m_szOutputTextFile[_MAX_PATH];		// output text file name

  public:
	CPTDecoder(void)
		{
		// initialize: the member variables
		m_nWidth = 0;
		m_nHeight = 0;
		m_nDepthNbBits = 0;
		m_nPercentOfDecoding = 100;

		m_nPercentOfCutting = 100;
		m_nNumberOfTreeNodes = 0;
		m_bIsExtension = false;
		m_bBoundingBoxWRL = false;
		m_bSpeedUp = false;

		memset((void*)m_szBitstreamFile, 0, _MAX_PATH);
		memset((void*)m_szOutputTextFile, 0, _MAX_PATH);
		}

	CPTDecoder(int argc, char* argv[])
		{
		// initialize: the member variables
		m_nWidth = 0;
		m_nHeight = 0;
		m_nDepthNbBits = 0;
		m_nPercentOfDecoding = 100;

		m_nPercentOfCutting = 0;
		m_nNumberOfTreeNodes = 0;
		m_bIsExtension = false;
		m_bBoundingBoxWRL = false;
		m_bSpeedUp = false;

		memset((void*)m_szBitstreamFile, 0, _MAX_PATH);
		memset((void*)m_szOutputTextFile, 0, _MAX_PATH);

		// command line processor
		CCommandLineProcessor oCLProcessor;
		oCLProcessor.setArguments(argc, argv);

		// get: the command line parameters
		strcpy(m_szBitstreamFile, oCLProcessor.getInputFile());
		strcpy(m_szOutputTextFile, oCLProcessor.getOutputFile());
		m_nPercentOfCutting = oCLProcessor.getPercentOfCutting();
		m_bBoundingBoxWRL = oCLProcessor.getBoundingBoxWRL();
		m_bSpeedUp = oCLProcessor.getSpeedUp();

		// print: the input values into the screen
		if(strcmp(m_szBitstreamFile, "") && strcmp(m_szOutputTextFile, ""))
			{
			// print: file name
			cout << "PointTexture decoding:" << endl;
			cout << "Input bitstream file = " << m_szBitstreamFile << endl;
			}
		else
			{
			// print: usage
			cout << "Usage: PTDecoder <Input bitstream file> <Output text file> [<Percent of cutting> | <-box> | <-s>]" << endl;
			exit(1);
			}
		}

	~CPTDecoder()
		{
		}

	// 2004-07-15, Shinjun {
	// decode: the voxel data and the color data
	void decodeWithBuffer(char* szBitstreamBuffer, int nLength = -1)
		{
		// make: the temporary file
		char* szTemporaryFile = "TemporaryFile.bin";
		ofstream fout(szTemporaryFile, ios::binary);
		fout.write(szBitstreamBuffer, nLength);
		fout.close();

		// copy: the file name
		strcpy(m_szBitstreamFile, szTemporaryFile);

		// read: the bitstream of the octree
		readBitstream();

		// make: the adjustable octree
		m_bSpeedUp = false;
		m_oOctree.makeAdjustableOctree(m_oQueue, m_bSpeedUp);

		// remove: the temporary file
		remove(szTemporaryFile);
		}

	// write:the output text file
	void write(char* szOutputTextFile, int nDefID = -1)
		{
		// variables: string buffer
		char		szBuffer[1024] = "";					// string buffer
		char*		pDest = NULL;							// pointer for string search

		if(nDefID != -1)
			{
			// find ".txt" or ".dec" and remove the string
			strcpy(m_szOutputTextFile, szOutputTextFile);
			pDest = strstr(m_szOutputTextFile, ".txt");
			if(pDest)
				*pDest = NULL;
			pDest = strstr(m_szOutputTextFile, ".dec");
			if(pDest)
				*pDest = NULL;
			// add "_ID" + nDefID + ".txt"
			strcat(m_szOutputTextFile, "_ID");
			_itoa(nDefID, szBuffer, 10);
			strcat(m_szOutputTextFile, szBuffer);
			strcat(m_szOutputTextFile, ".txt");
			}
		else
			{
			// set: the bistream file
			strcpy(m_szOutputTextFile, szOutputTextFile);
			strcat(m_szOutputTextFile, ".txt");
			}

		// write: the output text file
		m_bBoundingBoxWRL = false;
		writeTextFile(m_szOutputTextFile, m_bBoundingBoxWRL);
		}

	// set: value
	void setValues(unsigned char* pData)
		{
		// variables: pointers, resolution values, indices
		int nX_max = 0;			// maximum in X-axis
		int nY_max = 0;			// maximum in Y-axis
		int nZ_max = 0;			// maximum in Z-axis
		int nIndexOfDepth = 0;
		int nNumberOfDepth = 0;
		int nIndexOfColor = 0;
		int nNumberOfColor = 0;
		int i = 0;
		int j = 0;
		int k = 0;

		// set: the resolution values
		m_nWidth = ((int*)pData)[0];
		m_nHeight = ((int*)pData)[1];
		m_nDepthNbBits = ((int*)pData)[2];
		nX_max = m_nWidth - 1;
		nY_max = m_nHeight - 1; 
		nZ_max = (int)pow(2, m_nDepthNbBits+1)-1;
		m_oVoxel.initialize(m_nWidth, m_nHeight, m_nDepthNbBits);
		m_oColor.initialize(m_nWidth, m_nHeight, m_nDepthNbBits);

		// transfer: the depth data
		for(j = 0; j <= nY_max; j++)
			{
			for(i = 0; i <= nX_max; i++)
				{
				int nSize = ((int*)pData)[3+nIndexOfDepth];
				if(nSize > 0)
					m_oColor.setSizeOfDepth(i, j, nSize);
				nIndexOfDepth++;
				
				for(k = 0; k < nSize; k++)
					{
					int nDepth = ((int*)pData)[3+nIndexOfDepth];
					m_oVoxel.set(i, j, nDepth);	// set the voxel of (nX, nY, nValue)
					m_oColor.set(i, j, nDepth);	// set the voxel of (nX, nY, nValue)
					nIndexOfDepth++;
					}
				}
			}
		
		// transfer: the color data
		nNumberOfDepth = nIndexOfDepth;
		nIndexOfDepth = 0;
		for(j = 0; j <= nY_max; j++)
			{
			for(i = 0; i <= nX_max; i++)
				{
				int nSize = ((int*)pData)[3+nIndexOfDepth];
				nIndexOfDepth++;
				
				for(k = 0; k < nSize; k++)
					{
					nIndexOfDepth++;
					m_oColor.setRed(((unsigned char*)(pData+sizeof(int)*(3+nNumberOfDepth)))[nIndexOfColor++]);
					m_oColor.setGreen(((unsigned char*)(pData+sizeof(int)*(3+nNumberOfDepth)))[nIndexOfColor++]);
					m_oColor.setBlue(((unsigned char*)(pData+sizeof(int)*(3+nNumberOfDepth)))[nIndexOfColor++]);
					nNumberOfColor++;
					}
				}
			}

		}
	// 2004-07-15, Shinjun }

	// decode: the voxel data and the color data
	void decodeVoxelAndColor(void)
		{
		// variables: for clock
		clock_t ctStart, ctFinish;
		double dDuration = 0;
		ctStart = clock();

		// print: the output text file
		cout << "Output text file = " << m_szOutputTextFile << endl;

		// read: the bitstream of the octree
		readBitstream();

		// print: the percent of decoding
		cout << "Percent of decoding = " << m_nPercentOfDecoding << endl;
		cout << endl;

		// print: the time for reading bitstream
		ctFinish = clock();
		dDuration = (double)(ctFinish - ctStart) / CLOCKS_PER_SEC;
		cout << "(Bitstream Reading Time = " << dDuration << " seconds)" << endl;
		ctStart = clock();

		// make: the adjustable octree
		m_oOctree.makeAdjustableOctree(m_oQueue, m_bSpeedUp);

		// print: the time for building octree
		ctFinish = clock();
		dDuration = (double)(ctFinish - ctStart) / CLOCKS_PER_SEC;
		cout << "(Octree Building Time = " << dDuration << " seconds)" << endl << endl;
		ctStart = clock();

		// write: the output text file
		writeTextFile(m_szOutputTextFile, m_bBoundingBoxWRL);

		// print: the time for reading bitstream
		ctFinish = clock();
		dDuration = (double)(ctFinish - ctStart) / CLOCKS_PER_SEC;
		cout << "(Text File Writing Time = " << dDuration << " seconds)" << endl;
		ctStart = clock();
		}

	// read: the bitstream
	void readBitstream(void)
		{
		// PPM codec
		cInStream fin;
		fin.open(m_szBitstreamFile);

		// read: the bitstream of header
		readBitstreamOfHeader(fin);

		// read: the bitstream of tree nodes
		readBitstreamOfTreeNodes(fin);

		// close
		fin.close();
		}

	// write: the information file
	void writeInformation(double dTime)
		{
		// variable: the name of the information file and the file objects
		char szInformationFile[_MAX_PATH] = "";
		ifstream fin;
		ofstream fout;

		// make: the name of the information file
		strcpy(szInformationFile, m_szBitstreamFile);
		*strrchr(szInformationFile, '.') = NULL;
		strcat(szInformationFile, "_info.txt");

		// open: the information file
		fin.open(szInformationFile, ios::nocreate);
		if(fin.is_open() != 0)
			{
			fin.close();

			// write: the information file
			fout.open(szInformationFile, ios::app);
			fout << "\t";
			fout << dTime;
			fout.close();
			}
		else
			{
			fin.close();
			}
		}

  private:
	// read: the bitstream of header
	void readBitstreamOfHeader(cInStream& fin)
		{
		int nBitSize = 0;

		// get: flag of extension
		int nIsExtension = fin.getvlc(1);
		m_bIsExtension = (nIsExtension == 1) ? true : false;

		// get: width
		nBitSize = fin.getvlc(5);
		m_nWidth = fin.getvlc(nBitSize);

		// get: height
		nBitSize = fin.getvlc(5);
		m_nHeight = fin.getvlc(nBitSize);

		// get: depthNbBits
		m_nDepthNbBits = fin.getvlc(5);

		// get: percent for lossy decoding
		m_nPercentOfDecoding = fin.getvlc(7);
		}

	// read: the bitstream of tree nodes
	void readBitstreamOfTreeNodes(cInStream& fin)
		{
		// variable: index, count and general index
		int nIndex = 0;
		int nCount = 0;
		int i = 0;

		// variables: contexts for decoding
		context** contextTreeNodes = new context*[1024];	// octree
		context* contextSOP = new context;
		context* contextBOW = new context;
		context* contextRedOfRegion = new context;			// average color
		context* contextGreenOfRegion = new context;
		context* contextBlueOfRegion = new context;
		context* contextColorDifference = new context;		// color difference

		// create: context for decoding
		for(i = 0; i < 1024; i++)
			contextTreeNodes[i] = create_context_easy(2);	// b or w
		contextSOP = create_context_easy(2);
		contextBOW = create_context_easy(2);
		contextRedOfRegion = create_context_easy(256);		// 256 color
		contextGreenOfRegion = create_context_easy(256);
		contextBlueOfRegion = create_context_easy(256);
		contextColorDifference = create_context_easy(512);	// 256+256

		// initialize: the voxel and the color
		m_oVoxel.initialize(m_nWidth, m_nHeight, m_nDepthNbBits);
		m_oColor.initialize(m_nWidth, m_nHeight, m_nDepthNbBits);

		// initialize: the adjustable octree
		m_nNumberOfTreeNodes = m_oOctree.initialize(m_nWidth, m_nHeight, m_nDepthNbBits, m_oVoxel, m_oColor);

		// set: the percent of decoding
		if(m_nPercentOfCutting < m_nPercentOfDecoding)
			m_nPercentOfDecoding = m_nPercentOfCutting;

		// get: the number limit for decoding the tree nodes
		int nNumberLimit = (int)((double)m_nNumberOfTreeNodes*m_nPercentOfDecoding/100 + 0.5);

		// set: the octree value
		m_oOctree.setOctreeVoxel(0);	// 0 == root

		// push: the root value into queue
		m_oQueue.push(0);	// 0 == root

		// start: arithmetic decoding
		start_arithmetic_decode(fin);

		// read: the bitstream
		while(nCount < nNumberLimit)
			{
			// check: the queue
			if(m_oQueue.isEmpty() == true)
				break;

			// get: an index from queue
			nIndex = m_oQueue.pop();

			// get: the count
			nCount++;

			// read: the bitstream of SOP
			if(decode(contextSOP, fin) == 0)
				{
				// set: the sop
				// m_oOctree.resetSOP(i);		// Split: set 0

				// read: the bitstream of split node
				m_oOctree.readBitstreamOfSplitNode(fin, nIndex, nCount, m_oQueue, contextBOW, contextRedOfRegion, contextGreenOfRegion, contextBlueOfRegion);
				}
			else
				{
				// set: the sop
				m_oOctree.setSOP(i);			// PPM: set 1

				// read: the bitstream of PPM node
				m_oOctree.readBitstreamOfPPMNode(fin, nIndex, contextTreeNodes, contextColorDifference);

				// get: the count
				nCount += m_oOctree.getCountOfTreeSize(nIndex) - 1;
				}
			}

		// finish: arithmetic decoding
		finish_arithmetic_decode(fin);

		// delete: contexts for decoding
		delete_context(contextColorDifference);
		delete_context(contextRedOfRegion);
		delete_context(contextGreenOfRegion);
		delete_context(contextBlueOfRegion);
		delete_context(contextBOW);
		delete_context(contextSOP);
		for(i = 0; i < 1024; i++)
			delete_context(contextTreeNodes[i]);

		// free: contexts for decoding
		delete contextColorDifference;
		delete contextRedOfRegion;
		delete contextGreenOfRegion;
		delete contextBlueOfRegion;
		delete contextBOW;
		delete contextSOP;
		delete[] contextTreeNodes;
		}

	// write: the output text file
	void writeTextFile(char* szOutputTextFile, bool bBoundingBox = false)
		{
		// variables
		int nX_min = 0;									// minimum in X-axis
		int nX_max = m_nWidth-1;						// maximum in X-axis
		int nY_min = 0;									// minimum in Y-axis
		int nY_max = m_nHeight-1;						// maximum in Y-axis
		int nZ_min = 0;									// minimum in Z-axis
		int nZ_max = (int)pow(2, m_nDepthNbBits+1)-1;	// maximum in Z-axis
		int nDepthNbBits = 0;							// bits value for real depth
		int	i = 0;										// index
		int	j = 0;										// index
		int k = 0;										// index

		// find: the bounding box
		if(bBoundingBox == true)
			{
			// find: minimum in X-axis
			bool bExit = false;
			for(i = 0; i < m_nWidth; i++)
				{
				for(j = 0; j < m_nHeight; j++)
					{
					int nSize = m_oColor.getSizeOfDepth(i, j);
					if(nSize > 0)
						{
						nX_min = i;
						bExit = true;
						break;
						}
					}
				if(bExit == true)
					break;
				}
			
			// find: minimum in Y-axis
			bExit = false;
			for(j = 0; j < m_nHeight; j++)
				{
				for(i = 0; i < m_nWidth; i++)
					{
					int nSize = m_oColor.getSizeOfDepth(i, j);
					if(nSize > 0)
						{
						nY_min = j;
						bExit = true;
						break;
						}
					}
				if(bExit == true)
					break;
				}
			
			// find: maximum in X-axis
			bExit = false;
			for(i = m_nWidth-1; i >= 0; i--)
				{
				for(j = 0; j < m_nHeight; j++)
					{
					int nSize = m_oColor.getSizeOfDepth(i, j);
					if(nSize > 0)
						{
						nX_max = i;
						bExit = true;
						break;
						}
					}
				if(bExit == true)
					break;
				}
			
			// find: maximum in Y-axis
			bExit = false;
			for(j = m_nHeight-1; j >= 0; j--)
				{
				for(i = 0; i < m_nWidth; i++)
					{
					int nSize = m_oColor.getSizeOfDepth(i, j);
					if(nSize > 0)
						{
						nY_max = j;
						bExit = true;
						break;
						}
					}
				if(bExit == true)
					break;
				}

			// find: minimum and maximum in Z-axis
			nZ_min = INT_MAX;
			nZ_max = 0;
			for(j = nY_min; j <= nY_max; j++)
				{
				for(i = nX_min; i <= nX_max; i++)
					{
					int nSize = m_oColor.getSizeOfDepth(i, j);
					if(nSize > 0)
						{
						int* pDepth = m_oColor.getDepth(i, j);
						if(pDepth[0] < nZ_min)
							nZ_min = pDepth[0];
						if(pDepth[nSize-1] > nZ_max)
							nZ_max = pDepth[nSize-1];
						for(k = 1; k < nSize-1; k++)
							{
							if(pDepth[k] < nZ_min)
								nZ_min = pDepth[k];
							if(pDepth[k] > nZ_max)
								nZ_max = pDepth[k];
							}
						}
					}
				}
			
			// get: the bits value for real depth
			nDepthNbBits = (int)(log10(nZ_max - nZ_min)/log10(2));
			}

		// write: the output file
		ofstream fout(szOutputTextFile);
		fout << "#VRML V2.0 utf8" << endl;
		fout << " Background {" << endl;
		fout << "groundAngle [ ]" << endl;
		fout << "groundColor [ ]" << endl;
		fout << "backUrl [ ]" << endl;
		fout << "bottomUrl [ ]" << endl;
		fout << "frontUrl [ ]" << endl;
		fout << "leftUrl [ ]" << endl;
		fout << "rightUrl [ ]" << endl;
		fout << "topUrl [ ]" << endl;
		fout << "skyAngle [ ]" << endl;
		fout << "skyColor 0.5 0.5 0.5" << endl;
		fout << "}" << endl;
		fout << "Transform {" << endl;
		fout << "  translation 0 0 90" << endl;
		fout << "  children [" << endl;
		fout << "    DepthImage{" << endl;
		fout << "      orthographic TRUE" << endl;
		fout << "      fieldOfView 20 20" << endl;
		fout << "      farPlane 20" << endl;
		fout << "      nearPlane 1e-006" << endl;
		fout << "      diTexture PointTexture{" << endl;
		fout << "\twidth " << nX_max - nX_min + 1 << endl;
		fout << "\theight " << nY_max - nY_min + 1 << endl;
		if(bBoundingBox == true)
			fout << "\tdepthNbBits " << nDepthNbBits << endl;
		else
			fout << "\tdepthNbBits " << m_nDepthNbBits << endl;
		fout << "\tdepth [" << endl;

		// write: the depth data
		for(j = nY_min; j <= nY_max; j++)
			{
			for(i = nX_min; i <= nX_max; i++)
				{
				int nSize = m_oColor.getSizeOfDepth(i, j);
				fout << nSize << ", ";

				int* pDepth = m_oColor.getDepth(i, j);
				for(k = 0; k < nSize; k++)
					{
					if(bBoundingBox == true && nDepthNbBits < m_nDepthNbBits)
						fout << pDepth[k] - nZ_min << ", ";
					else
						fout << pDepth[k] << ", ";
					}
				fout << endl;
				}
			}

		fout << "\t]" << endl;
		fout << endl;
		fout << "\tcolor [" << endl;

		// write: the color data
		for(j = nY_min; j <= nY_max; j++)
			{
			for(i = nX_min; i <= nX_max; i++)
				{
				int nSize = m_oColor.getSizeOfDepth(i, j);

				unsigned char* pColor = m_oColor.getColor(i, j);
				for(k = 0; k < nSize; k++)
					{
					double dRed = roundOffWithDigit((double)pColor[k*3] / 255, 3);		// digit = 3
					fout << dRed << " ";

					double dGreen = roundOffWithDigit((double)pColor[k*3+1] / 255, 3);	// digit = 3
					fout << dGreen << " ";

					double dBlue = roundOffWithDigit((double)pColor[k*3+2] / 255, 3);	// digit = 3
					fout << dBlue << ", ";
					}
				if(nSize > 0)
					fout << endl;
				}
			}

		fout << "\t]" << endl;
		fout << "      }" << endl;
		fout << "    }" << endl;
		fout << "  ]" << endl;
		fout << "}" << endl;
		fout.close();

		// print: information of the resolution
		cout << "(Width * Height * Depth) = ("	<< nX_max - nX_min + 1 << " * "
												<< nY_max - nY_min + 1 << " * "
												<< nZ_max - nZ_min + 1 << ")" << endl; // <- (int)pow(2, m_nDepthNbBits+1)
		cout << "(XMin~XMax, YMin~YMax, ZMin~ZMax) = ("
			 << nX_min << "~" << nX_max << ", "
			 << nY_min << "~" << nY_max << ", "
			 << nZ_min << "~" << nZ_max << ")" << endl;
		cout << endl;
		}

	// round off: a number with digit
	double roundOffWithDigit(double dSource, int nDigit)
		{
		int nDecimal = 0;
		int nSign = 0;
		char* szBuffer = NULL;
		double dValue = 0;
		
		szBuffer = _fcvt(dSource, 10, &nDecimal, &nSign);

		if(nSign == 0)
			dValue = (double)int(dSource*pow(10, nDigit-nDecimal)+0.5)/pow(10, nDigit-nDecimal);
		else
			dValue = (double)int(dSource*pow(10, nDigit-nDecimal)-0.5)/pow(10, nDigit-nDecimal);

		return dValue;
		}
	};

#endif _CPTDECODER_H_
