/***********************************************************************
This software module was originally developed by Shinjun Lee
(SAMSUNG AIT) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. SAMSUNG AIT retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright ?2002.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Shinjun Lee at sjlee@sait.samsung.co.kr.
----------------------------------------------------------------------*/

// CQueue.h: Queue
// Author: Shinjun Lee, Samsung AIT
// Date: 2003-12-11 ~ 2003-12-16

#ifndef _CQUEUE_H_
#define _CQUEUE_H_

class CQueue
	{
  private:
	class CElement
		{
	  public:
		int			m_nValue;		// integer value for queue
		CElement*	m_pNext;		// pointer for next element

		CElement(void)
			{
			m_nValue = 0;
			m_pNext = NULL;
			}

		CElement(int nValue)
			{
			m_nValue = nValue;
			m_pNext = NULL;
			}

		~CElement()
			{
			if(m_pNext != NULL)
				{
				delete m_pNext;
				m_pNext = NULL;
				}
			}
		};

	CElement*	m_pFirst;			// pointer for first element
	CElement*	m_pLast;			// pointer for last element

	CElement*	m_pFirstChild[8];	// pointer array for first child element
	CElement*	m_pLastChild[8];	// pointer array for last child element

	int			m_nPreviousParent;	// the previous parent
	int			m_nIndexOfChild;	// the index of child

  public:
	CQueue(void)
		{
		m_pFirst = NULL;
		m_pLast = NULL;

		memset((void*)m_pFirstChild, 0, sizeof(CElement*)*8);
		memset((void*)m_pLastChild, 0, sizeof(CElement*)*8);

		m_nPreviousParent = -2;
		m_nIndexOfChild = 0;
		}

	~CQueue()
		{
		if(m_pFirst != NULL)
			{
			delete m_pFirst;
			m_pFirst = NULL;
			}

		for(int i = 0; i < 8; i++)
			{
			if(m_pFirstChild[i] != NULL)
				{
				delete m_pFirstChild[i];
				m_pFirstChild[i] = NULL;
				}
			}
		}

	// push: a value into queue
	void push(int nValue)
		{
		// set: the value into queue
		CElement* pElement = new CElement(nValue);

		// get: the parent
		int nParent = (int)floor((double)(nValue - 1) / 8);

		// compare: the parent with the previous parent
		if(nParent != m_nPreviousParent)
			{
			m_nPreviousParent = nParent;
			m_nIndexOfChild = 0;
			}
		else if(++m_nIndexOfChild > 8)
			{
			cout << "Error: the index of child is overflow." << endl;
			exit(1);
			}

		// get: the parent and the index of child
		int nIndex = m_nIndexOfChild;

		// push: the value into the children queue
		if(m_pLastChild[nIndex] != NULL)
			{
			m_pLastChild[nIndex]->m_pNext = pElement;
			m_pLastChild[nIndex] = pElement;
			}
		else if(m_pFirstChild[nIndex] == NULL)
			{
			m_pFirstChild[nIndex] = pElement;
			m_pLastChild[nIndex] = pElement;
			}
		}

	// pop: a value from queue
	int pop(void)
		{
		// variables: a return value and a pointer of an element
		int nValue = 0;
		CElement* pElement = NULL;

		if(m_pFirst != NULL)
			{
			// get: the first value from queue
			nValue = m_pFirst->m_nValue;

			// move: the pointer to the next element
			pElement = m_pFirst;
			m_pFirst = m_pFirst->m_pNext;

			// delete; the first element
			pElement->m_pNext = NULL;
			delete pElement;

			// check: the first pointer
			if(m_pFirst == NULL)
				m_pLast = NULL;
			}
		else if(m_pFirstChild[0] != NULL)
			{
			int i = 0;

			// make: the one children queue
			for(i = 1; i < 8; i++)
				{
				// link: the previous children to the next children
				if(m_pFirstChild[i] != NULL)
					m_pLastChild[i-1]->m_pNext = m_pFirstChild[i];
				else
					break;
				}

			// move: the children queue to the parent
			m_pFirst = m_pFirstChild[0];
			m_pLast = m_pLastChild[i-1];

			// clear: the children queues
			for(i = 0; i < 8; i++)
				{
				// clear: the children queue
				if(m_pFirstChild[i] != NULL)
					{
					m_pFirstChild[i] = NULL;
					m_pLastChild[i] = NULL;
					}
				else
					break;
				}

			// get: the first value from queue
			nValue = m_pFirst->m_nValue;

			// move: the pointer to the next element
			pElement = m_pFirst;
			m_pFirst = m_pFirst->m_pNext;

			// delete; the first element
			pElement->m_pNext = NULL;
			delete pElement;

			// check: the first pointer
			if(m_pFirst == NULL)
				m_pLast = NULL;
			}
		else
			{
			cout << "Error: the queue is underflow." << endl;
			exit(1);
			}

		return nValue;
		}

	// check: whether the queue is empty or not
	bool isEmpty(void)
		{
		if(m_pFirst != NULL)
			return false;
		else if(m_pFirstChild[0] != NULL)
			return false;

		return true;
		}
	};

#endif _CQUEUE_H_
