/***********************************************************************
This software module was originally developed by Shinjun Lee
(SAMSUNG AIT) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. SAMSUNG AIT retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright ?2002.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Shinjun Lee at sjlee@sait.samsung.co.kr.
----------------------------------------------------------------------*/

// CVoxel.h
// Author: Shinjun Lee, Samsung AIT
// Date: 2003-11-07 ~ 2003-12-13

#ifndef _CVOXEL_H_
#define _CVOXEL_H_

#include <iostream.h>
#include <math.h>

class CVoxel
	{
  private:
	unsigned char*	m_pVoxel;				// voxel data
	int				m_nSize;				// memory size for voxel data

	int				m_nWidth;				// width
	int				m_nHeight;				// height
	int				m_nDepthNbBits;			// bits value for depth
	int				m_nDepth;				// depth

  public:
	CVoxel(void)
		{
		m_pVoxel = NULL;
		m_nSize = 0;

		m_nWidth = 0;
		m_nHeight = 0;
		m_nDepthNbBits = 0;
		m_nDepth = 0;
		}

	~CVoxel()
		{
		if(m_pVoxel != NULL)
			{
			delete[] m_pVoxel;
			m_pVoxel = NULL;
			}
		}

	// initialize: the voxel data with width, height, depth
	bool initialize(int nWidth, int nHeight, int nDepthNbBits)
		{
		// set: information of the resolution
		m_nWidth = nWidth;
		m_nHeight = nHeight;
		m_nDepthNbBits = nDepthNbBits;
		m_nDepth = (int)pow(2, nDepthNbBits+1);

		// get: the size of voxel data
		m_nSize = nWidth * nHeight * m_nDepth / 8;

		// allocate: the voxel data
		m_pVoxel = new unsigned char[m_nSize];
		if(m_pVoxel == NULL)
			{
			cout << "Voxel object allocation error." << endl;
			exit(1);
			}
		memset((void*)m_pVoxel, 0, sizeof(unsigned char)*m_nSize);

		return true;
		}

	// set: a value into voxel data in the position of (nX, nY, nZ)
	bool set(int nX, int nY, int nZ, int nValue)
		{
		if(nX < 0 || nX >= m_nWidth || nY < 0 || nY >= m_nHeight || nZ < 0 || nZ >= m_nDepth)
			return false;

		int nPosition = nX + nY*m_nWidth + nZ*m_nWidth*m_nHeight;
		int nIndex = nPosition / 8;
		int nBitPos = nPosition % 8;
		unsigned char ucByte = m_pVoxel[nIndex];
		unsigned char ucPos = 1 << nBitPos;

		if(nValue == 0)
			{
			unsigned char sMask = ~ucPos; // <- 0xff ^ ucPos;

			m_pVoxel[nIndex] = ucByte & sMask;
			}
		else 
			{
			m_pVoxel[nIndex] = ucByte | ucPos;
			}

		return true;
		}

	// set: 1 in the voxel of (nX, nY, nZ)
	void set(int nX, int nY, int nZ)
		{
		if(nX < 0 || nX >= m_nWidth || nY < 0 || nY >= m_nHeight || nZ < 0 || nZ >= m_nDepth)
			return;

		// get: the position from (nX, nY, nZ)
		int nPosition = nX + nY*m_nWidth + nZ*m_nWidth*m_nHeight;
		int nIndex = nPosition / 8;
		int nBitPos = nPosition % 8;
		unsigned char ucPos = 1 << nBitPos;

		// get: the value from the voxel data
		unsigned char ucByte = m_pVoxel[nIndex];
		
		// set: the value into the voxel data
		m_pVoxel[nIndex] = ucByte | ucPos;
		}

	// set: 0 in the voxel of (nX, nY, nZ)
	bool reset(int nX, int nY, int nZ)
		{
		if(nX < 0 || nX >= m_nWidth || nY < 0 || nY >= m_nHeight || nZ < 0 || nZ >= m_nDepth)
			return false;

		// get: the position from (nX, nY, nZ)
		int nPosition = nX + nY*m_nWidth + nZ*m_nWidth*m_nHeight;
		int nIndex = nPosition / 8;
		int nBitPos = nPosition % 8;
		unsigned char ucPos = 1 << nBitPos;
		unsigned char sMask = ~ucPos; // <- 0xff ^ ucPos;

		// get: the value from the voxel data
		unsigned char ucByte = m_pVoxel[nIndex];

		// set: the value into the voxel data
		m_pVoxel[nIndex] = ucByte & sMask;
		}

	// get: a value from voxel data in the position of (nX, nY, nZ)
	int get(int nX, int nY, int nZ)
		{
		int nValue = 0;
		if(nX < 0 || nX >= m_nWidth || nY < 0 || nY >= m_nHeight || nZ < 0 || nZ >= m_nDepth)
			return nValue;

		int nPosition = nX + nY*m_nWidth + nZ*m_nWidth*m_nHeight;
		int nIndex = nPosition / 8;
		int nBitPos = nPosition % 8;
		unsigned char ucByte = m_pVoxel[nIndex];
		unsigned char ucPos = 1 << nBitPos;

		nValue = ucByte & ucPos;
		if(nValue > 0)
			nValue = 1;

		return nValue;
		}

	// find out: whether a region is empty or not (nX0, nY0, nZ0) ~ (nX1, nY1, nZ1)
	bool isRegionEmpty(int nX0, int nY0, int nZ0, int nX1, int nY1, int nZ1)
		{
		// check: the value of the region
		if(nX0 < 0)
			nX0 = 0;
		else if(nX0 >= m_nWidth)
			nX0 = m_nWidth-1;
		if(nY0 < 0)
			nY0 = 0;
		else if(nY0 >= m_nHeight)
			nY0 = m_nHeight-1;
		if(nZ0 < 0)
			nZ0 = 0;
		else if(nZ0 >= m_nDepth)
			nZ0 = m_nDepth-1;
		if(nX1 < 0)
			nX1 = 0;
		else if(nX1 >= m_nWidth)
			nX1 = m_nWidth-1;
		if(nY1 < 0)
			nY1 = 0;
		else if(nY1 >= m_nHeight)
			nY1 = m_nHeight-1;
		if(nZ1 < 0)
			nZ1 = 0;
		else if(nZ1 >= m_nDepth)
			nZ1 = m_nDepth-1;

		// check: the region values
		for(int k = nZ0; k <= nZ1; k++)
			for(int j = nY0; j <= nY1; j++)
				for(int i = nX0; i <= nX1; i++)
					if(get(i, j, k) != 0)
						return false;
		
		return true;
		}
	};

#endif _CVOXEL_H_
