/***********************************************************************
This software module was originally developed by Shinjun Lee
(SAMSUNG AIT) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. SAMSUNG AIT retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright ?2002.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Shinjun Lee at sjlee@sait.samsung.co.kr.
----------------------------------------------------------------------*/

#ifndef _ARITH_H_
#define _ARITH_H_

#include "bitio.h"

/* ================= USER ADJUSTABLE PARAMETERS =================== */

/* Default B_bits and F_bits */

#ifndef		B_BITS
#define		B_BITS		32
#endif

#ifndef		F_BITS
#define		F_BITS		27
#endif

/* Change these types for different precision calculations.  They may affect
 * the speed of the arithmetic operations (multiplcation, division, shift,
 * etc).
 * The way the stats module is implemented, the type of freq_value
 * must be able to accomodate f_bits+1 bits, instead of f_bits, to avoid
 * overflows.  Ie: For an f_bits of up to 31, type freq_value must be 32 bits.
 */
typedef unsigned long code_value;	  /* B_BITS of precision */
typedef unsigned long	freq_value;	  /* F_BITS+1 of precision */
typedef unsigned long	div_value;	  /* B_BITS-F_BITS of precision */


/* MAX_BITS_OUTSTANDING is a bound on bits_outstanding
 * If bits_outstanding ever gets above this number (extremely unlikely)
 * the program will abort with an error message.   (See arith.c for details).
 */
#define 	MAX_BITS_OUTSTANDING	((unsigned long)1<<31)

/* ================= END USER ADJUSTABLE PARAMETERS =================== */

/* If varying bits, variables are B_bits, F_bits, Half and Quarter,
 *	otherwise #define them
 * These variables will be read (and possibly changed) by main.c and
 *  stats.c
 */

#   define		B_bits		B_BITS
#   define		F_bits  	F_BITS

extern char *coder_desc;

/* function prototypes */
void arithmetic_encode(freq_value l, freq_value h, freq_value t, cOutStream &Out);
freq_value arithmetic_decode_target(freq_value t);
void arithmetic_decode(freq_value l, freq_value h, freq_value t, cInStream &In);

void start_arithmetic_encode(void);
void finish_arithmetic_encode(cOutStream &Out);
void start_arithmetic_decode(cInStream &In);
void finish_arithmetic_decode(cInStream &In);

#endif		/* ifndef arith.h */
