/***********************************************************************
This software module was originally developed by Shinjun Lee
(SAMSUNG AIT) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. SAMSUNG AIT retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright ?2002.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Shinjun Lee at sjlee@sait.samsung.co.kr.
----------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include "bitio.h"


///////////////////////////////////////////////////////////////////////
// cInStream 
///////////////////////////////////////////////////////////////////////
void cInStream::open(const char *filename)
{
  if( (fp = fopen(filename, "rb")) == NULL ){
	   fprintf(stdout, "Input file opening failed.\n");
		 exit(-1);
  }

	_in_buffer = 0;
	_in_bit_ptr = 0;	
	_bytes_input =0;
}

void cInStream::close()
{
	fclose(fp);
}

int cInStream::getbit()
{
	int bit;	
	if (_in_bit_ptr == 0)															  
	{																									  
		int failure = fread(&_in_buffer, sizeof(_in_buffer), 1, fp);													  
		if (failure != 1) 														  
		{																								  
			fprintf(stderr,"Bad input file - attempted "	  
										"read past end of file.\n");			
			exit(1);																			  
		}																									
		else																							
			_bytes_input++; 																

		_in_bit_ptr = (1<<(BYTE_SIZE-1));									
	}																										
	
	if (_in_buffer & _in_bit_ptr) bit = 1;
	else bit = 0;
										
	_in_bit_ptr >>= 1;																

	return bit;
}

void cInStream::flushbit()
{
	_in_bit_ptr = 0;
}

unsigned int cInStream::getvlc(int n_bits)
{
	unsigned int ui=0;
	unsigned int mask = 1<<(n_bits-1);

	for(int n=0; n<n_bits; n++){
		if(getbit()==1)
			ui = ui | mask;
		mask = mask >> 1;
	}

	return ui;
}


int cInStream::getbytecount()
{
	return _bytes_input++;
}

	
///////////////////////////////////////////////////////////////////////
// cOutStream 
///////////////////////////////////////////////////////////////////////
void cOutStream::open(const char *filename)
{
	if( (fp = fopen(filename, "wb")) == NULL ){
		fprintf(stdout, "Output file opening failed.\n");
		exit(-1);
	};
	
	_out_buffer = 0;
	_out_bits_to_go = BYTE_SIZE;
	_bytes_output = 0;
}

void cOutStream::close()
{
  if (_out_bits_to_go != BYTE_SIZE)
	{
		_bytes_output++;									// align information to MSB side and flush it
		_out_buffer <<= _out_bits_to_go;
		fwrite(&_out_buffer, sizeof(_out_buffer), 1, fp);
	}

	fclose(fp);
}

void cOutStream::putbit(int b)
{
	_out_buffer <<= 1;				        
	if (b)					                  
 		_out_buffer |= 1;			          
	_out_bits_to_go--;				        
	if (_out_bits_to_go == 0)					
	{	
		_bytes_output++;
		fwrite(&_out_buffer, sizeof(_out_buffer), 1, fp);

		_out_bits_to_go = BYTE_SIZE;		
		_out_buffer = 0;								
	}					
}

void cOutStream::flushbit()
{
	if (_out_bits_to_go != BYTE_SIZE)
		_out_buffer <<= _out_bits_to_go;

	fwrite(&_out_buffer, sizeof(_out_buffer), 1, fp);
	_out_bits_to_go = BYTE_SIZE;
	_out_buffer = 0;
}

void cOutStream::putvlc(unsigned int code, int n_bits)
{
	unsigned int mask = 1<<(n_bits-1);

	for(int n=0; n<n_bits; n++)
	{
		if(code & mask)	 putbit(1);
		else             putbit(0);

		mask = mask >> 1;
	}
}

int cOutStream::getbytecount()
{
	if(_out_bits_to_go != BYTE_SIZE)
		return _bytes_output+1;
	else
		return _bytes_output;
}
