/***********************************************************************
This software module was originally developed by Shinjun Lee
(SAMSUNG AIT) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. SAMSUNG AIT retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright ?2002.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Shinjun Lee at sjlee@sait.samsung.co.kr.
----------------------------------------------------------------------*/

#ifndef _STATS_H_
#define _STATS_H_

#include "bitio.h"

/* 
   Note:  arith.h needs to have been previously included, as it has the
   freq_value definition.  Theoretically it should have been defined here,
   but it is easier to read and modify when the frequency and code
   value definitions are placed together, as they are interrelated.
 */

/* 
 * macros to add and remove the end '1' bit of a binary number 
 * using two's complement arithmetic
 */
#define BACK(i)			((i) & ((i) - 1))	
#define FORW(i)			((i) + ((i) & - (i)))

#define NOT_KNOWN		(-1)	/* attempt to code unknown symbol */
#define TOO_MANY_SYMBOLS	(-1)	/* could not install symbol */
#define NO_MEMORY		(-2)	/* install exceeded memory */

#define STATIC			0	/* context cannot grow- no escape */
#define	DYNAMIC			1	/* context may grow- escape needed */

/* memory used per symbol is a determined by noting that the Fenwick
 * structure used to store the symbols in is doubled as it is
 * realloc'ed.  The old structures (assuming not reusable) roughly
 * double the space used.  This doubling also means that almost half the
 * space (the new half) may never be used.  Together, this means, the
 * maximum cost of a "dynamic symbol" to memory requirements may be up
 * to 4 times that of a "static symbol" (the amount of memory it
 * actually takes to store the symbol).
 * Here we define MEM_PER_SYMBOL as this worst case, so that we can always
 * stay within any user specified memory limit.
 */
#define	MEM_PER_SYMBOL		(4 * sizeof(freq_value))	

#define MIN_INCR		1	/* minimum increment value */


/* context structure used to store frequencies */
typedef struct {
    int initial_size;			/* original length of context */
    int max_length, length;		/* length of tree and current length */
    freq_value nSingletons;		/* no. symbols with frequency=1 */
    int type;				/* context may be STATIC or DYNAMIC */
    int nSymbols;			/* count of installed symbols */
    freq_value total;			/* total of all frequencies */
    freq_value *tree;			/* Fenwick's binary index tree */
    freq_value incr;			/* current increment */
    int 	most_freq_symbol;
    freq_value	most_freq_count;
    freq_value	most_freq_pos;
} context;


/* context structure for binary contexts */
typedef struct {
    freq_value c0;			/* number of zeroes */
    freq_value c1;			/* number of ones */
    freq_value incr;			/* current increment used */
} binary_context;


extern char *stats_desc;


/* function prototypes */
context *create_context(int length, int type);
context *create_context_easy(int length);
int install_symbol(context *pTree, int symbol);
int encode(context *pContext, int symbol, cOutStream &Out);
int decode(context *pContext, cInStream &In);
void purge_context(context *pContext);
/* Chang-Su Kim */
void delete_context(context *pContext);
/* Chang-Su Kim */
binary_context *create_binary_context(void);

#endif
