/***********************************************************************
This software module was originally developed by Sang Oak Woo
(SAMSUNG AIT) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. SAMSUNG AIT retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2002.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Sang Oak Woo at sowoo@sait.samsung.co.kr.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Decoder
FILE:       SAIT_AFX_STREAM_DECODER.H
AUTHOR:     Sang Oak Woo (SAMSUNG AIT)
OVERVIEW:   Decoding the AFX stream object made in SAIT.
-------------------------------------------------------------------------*/

#include "stdafx.h"
#include "defs.h"
#include "SAIT_AFX_STREAM_DECODER.h"


BOOL JWPositionInterpolatorDecoder :: Decode (
	LPBYTE pInput, int nInputLength, LPBYTE pOutput, int &rnOutputLength)
{
	m_PositionInterpolator.decodeWithBuffer((char*) pInput, nInputLength);
//	m_PositionInterpolator.write("AFXObject-Position.txt");

#if defined SAIT_CONFORMANCE_2003
	// initial value of 
		// pOutput: the address of memory allocated by DecoderImp
		// rnOutputLength: the size of pOutput
	// final value of
		// rnOutputLength: the real size (actual data size) of pOutput

	int nNumberOfKeys = m_PositionInterpolator.key.GetSize();

	// if the size of pOutput is less then the size needed, return false
	if((int)(nNumberOfKeys*sizeof(float)*4 + sizeof(int))> rnOutputLength)
	{
		rnOutputLength = (int)(nNumberOfKeys*sizeof(float)*4 + sizeof(int));
		return FALSE;
	}

	// the Format of pOutput
		// # of frames
		// all of the keys
		// all of the keyValues
	int *pOutput_NumberOfKey = (int*) pOutput;
	float *pOutput_Keys = (float*) (pOutput + sizeof(int));
	float *pOutput_KeyValues = (float*) (pOutput + sizeof(float)*nNumberOfKeys + sizeof(int));

	// Fill pOutput
	// # of frames
	pOutput_NumberOfKey[0] = nNumberOfKeys;

	for(int i=0, j=0; i<nNumberOfKeys; i++)
	{
		// all of the keys
		pOutput_Keys[i] = m_PositionInterpolator.key[i];

		// all of the keyValues
		pOutput_KeyValues[j++] = m_PositionInterpolator.keyValue[i].x;
		pOutput_KeyValues[j++] = m_PositionInterpolator.keyValue[i].y;
		pOutput_KeyValues[j++] = m_PositionInterpolator.keyValue[i].z;
	}

	rnOutputLength = nNumberOfKeys*sizeof(float)*4 + sizeof(int);
#endif SAIT_CONFORMANCE_2003

	return TRUE;
}


BOOL JWOrientationInterpolatorDecoder :: Decode (
	LPBYTE pInput, int nInputLength, LPBYTE pOutput, int &rnOutputLength)
{
	m_OrientationInterpolator.decodeWithBuffer((char*) pInput, nInputLength);
//	m_OrientationInterpolator.write("AFXObject-OrientationInterpolator.txt");

#if defined SAIT_CONFORMANCE_2003
	// initial value of 
		// pOutput: the address of memory allocated by DecoderImp
		// rnOutputLength: the size of pOutput
	// final value of
		// rnOutputLength: the real size (actual data size) of pOutput

	int nNumberOfKeys = m_OrientationInterpolator.dKey.GetSize();

	// if the size of pOutput is less then the size needed, return false
	if((int)(nNumberOfKeys*sizeof(float)*5 + sizeof(int))> rnOutputLength)
	{
		rnOutputLength = (int)(nNumberOfKeys*sizeof(float)*5 + sizeof(int));
		return FALSE;
	}

	// the Format of pOutput
		// # of frames
		// all of the keys
		// all of the keyValues
	int *pOutput_NumberOfKey = (int*) pOutput;
	float *pOutput_Keys = (float*) (pOutput + sizeof(int));
	float *pOutput_KeyValues = (float*) (pOutput + sizeof(float)*nNumberOfKeys + sizeof(int));

	// Fill pOutput
	// # of frames
	pOutput_NumberOfKey[0] = nNumberOfKeys;

	for(int i=0, j=0; i<nNumberOfKeys; i++)
	{
		// all of the keys
		pOutput_Keys[i] = m_OrientationInterpolator.dKey[i];

		// all of the keyValues
		pOutput_KeyValues[j++] = m_OrientationInterpolator.dKeyValue[i].x;
		pOutput_KeyValues[j++] = m_OrientationInterpolator.dKeyValue[i].y;
		pOutput_KeyValues[j++] = m_OrientationInterpolator.dKeyValue[i].z;
		pOutput_KeyValues[j++] = m_OrientationInterpolator.dKeyValue[i].theta;
	}

	rnOutputLength = nNumberOfKeys*sizeof(float)*5 + sizeof(int);
#endif SAIT_CONFORMANCE_2003

	return TRUE;
}

BOOL JWCoordinateInterpolatorDecoder :: Decode (
	LPBYTE pInput, int nInputLength, LPBYTE pOutput, int &rnOutputLength)
{
	m_CoordinateInterpolator.decodeWithBuffer((char*) pInput, nInputLength);
//	m_CoordinateInterpolator.write("AFXObject-CoordinateInterpolator.txt");

#if defined SAIT_CONFORMANCE_2003
	// initial value of 
		// pOutput: the address of memory allocated by DecoderImp
		// rnOutputLength: the size of pOutput
	// final value of
		// rnOutputLength: the real size (actual data size) of pOutput

	int nNumberOfKeys = m_CoordinateInterpolator.kHeader.nNumberOfKey;
	int nNumberOfCoords = m_CoordinateInterpolator.coordIKVHeader.nNumberOfCoord;

	// if the size of pOutput is less then the size needed, return false
	if((int)(2*sizeof(int) + nNumberOfKeys*sizeof(float) + 3*nNumberOfKeys*nNumberOfCoords*sizeof(float))> rnOutputLength)
		return FALSE;

	// the Format of pOutput
		// # of frames
		// all of the keys
		// all of the keyValues
	int *pOutput_NumberOfKeys = (int*) pOutput;
	int *pOutput_NumberOfCoords = (int*) (pOutput + sizeof(int));
	float *pOutput_Keys = (float*) (pOutput + 2*sizeof(int));
	float *pOutput_KeyValues = (float*) (pOutput + 2*sizeof(int) + nNumberOfKeys*sizeof(float));

	// Fill pOutput
	// # of frames
	// # of vertices
	*pOutput_NumberOfKeys = nNumberOfKeys;
	*pOutput_NumberOfCoords = nNumberOfCoords;

	// Fill key data
	for(int i = 0; i < nNumberOfKeys; i++)
		pOutput_Keys[i] = (float)m_CoordinateInterpolator.k.dKey[i];

	// Fill key value data
	for(i = 0; i < nNumberOfKeys; i++)
	{
		for(int j = 0; j < nNumberOfCoords; j++)
		{
			pOutput_KeyValues[i*3*nNumberOfCoords+j*3] = (float)m_CoordinateInterpolator.coordIKeyValue.dKeyValue[i*3*nNumberOfCoords+j*3];
			pOutput_KeyValues[i*3*nNumberOfCoords+j*3+1] = (float)m_CoordinateInterpolator.coordIKeyValue.dKeyValue[i*3*nNumberOfCoords+j*3+1];
			pOutput_KeyValues[i*3*nNumberOfCoords+j*3+2] = (float)m_CoordinateInterpolator.coordIKeyValue.dKeyValue[i*3*nNumberOfCoords+j*3+2];
		}
	}

	rnOutputLength = 2*sizeof(int) + nNumberOfKeys*sizeof(float) + 3*nNumberOfKeys*nNumberOfCoords*sizeof(float);
#endif SAIT_CONFORMANCE_2003

	return TRUE;
}


BOOL JW3DMCDecoder :: Decode (
	LPBYTE pInput, int nInputLength, LPBYTE pOutput, int &rnOutputLength)
{
	m_3DMCDecoder.DecodeWithBuffer((char*) pInput, nInputLength);
	//m_3DMCDecoder.Decode(fileName);
//	m_3DMCDecoder.write("AFXObject-OrientationInterpolator.txt");

#if defined SAIT_CONFORMANCE_2003
		
	// 3DMC  ũ⸦ .
	//  ũ⸦ rnOutputLength   rnOutputLength ,
	// rnOutputLength  ũ⸦  !
	// , return FALSE
	
	//  д´.
	bool ccw = m_3DMCDecoder.GetCcw(); // Read ccw value
	bool convex = m_3DMCDecoder.GetConvex();
	float creaseAngle = m_3DMCDecoder.GetCreaseAngle();
	bool solid = m_3DMCDecoder.GetSolid(); 
	
	int nCoord = m_3DMCDecoder.GetCoord(); 
	int nCoordIndex = m_3DMCDecoder.GetCoordIdx();
	
	int nColor = m_3DMCDecoder.GetColor();
	bool colorPerVertex = m_3DMCDecoder.GetColorPerVertex();
	int nColorIndex = m_3DMCDecoder.GetColorIndex();
	
	int nNormal = m_3DMCDecoder.GetNormal();
	bool normalPerVertex = m_3DMCDecoder.GetNormalPerVertex();
	int nNormalIndex = m_3DMCDecoder.GetNormalIndex();
	
	int nTexCoord  = m_3DMCDecoder.GetTexCoord();
	int nTexCoordIndex = m_3DMCDecoder.GetTexCoordIndex();

	// Field Values
	int    *pCoordIndex;
	float  *pCoord;
	int    *pColorIndex;
	float  *pColor;
	int    *pNormalIndex;
	float  *pNormal;
	int    *pTexCoordIndex;
	float  *pTexCoord;

	int nMemorySizeNeeded = 
		sizeof(bool)*5 +
		sizeof(int)*(8+nCoordIndex+nColorIndex+nNormalIndex+nTexCoordIndex) + 
		sizeof(float)*(1+nCoord+nColor+nNormal+nTexCoord);

	if(nMemorySizeNeeded > rnOutputLength)
	{
		rnOutputLength = nMemorySizeNeeded;
		return FALSE;
	}

	int i = 0;
	int   nLengthOfTotal = 0;

	// Headers: ccw
	bool* pCCW = (bool*) pOutput;      
    nLengthOfTotal += sizeof(bool);    // Increase Total Output size
	pCCW[0] = ccw;                     //   Ѵ.

	// Headers: convex
	bool* pConvex = (bool*)( pOutput + nLengthOfTotal );
	nLengthOfTotal += sizeof(bool);
	pConvex[0] = convex;

	// Headers: creaseAngle
	float* pCreaseAngle = (float *)(pOutput + nLengthOfTotal);
	nLengthOfTotal += sizeof(float);
	pCreaseAngle[0] = creaseAngle;

	// Headers: Solid	
    bool* pSolid = (bool *)( pOutput + nLengthOfTotal);
	nLengthOfTotal += sizeof(bool);
	pSolid[0] = solid;

	// Values: Coord (x,y,z)
    int* pNumCoord = (int*)(pOutput + nLengthOfTotal);
	nLengthOfTotal += sizeof(int);
	pNumCoord[0] = nCoord;                   // Store the number of Coord
	
	pCoord = (float *)( pOutput + nLengthOfTotal); // ŭ ũ ´.

	for(i = 0 ; i < nCoord; i++) {
		pCoord[i] = m_3DMCDecoder.pCoord[i]; // Store Coord
    }	
	nLengthOfTotal += sizeof(float)*nCoord;

	// Values: coordIndex
	int* pNumCoordIndex = (int*)(pOutput + nLengthOfTotal);
    pNumCoordIndex[0] = nCoordIndex;  // Store the number of CoordIndex
	nLengthOfTotal += sizeof(int);

	pCoordIndex = (int *)( pOutput + nLengthOfTotal);
	for(i = 0 ; i < nCoordIndex; i++) {
		pCoordIndex[i] = m_3DMCDecoder.pCoordIdx[i]; //  Ѵ.
    }
	nLengthOfTotal += sizeof(int)*nCoordIndex;
	
	// Values: Color
	int* pNumColor = (int*)(pOutput + nLengthOfTotal);
    pNumColor[0] = nColor;  // Store the number of Color
	nLengthOfTotal += sizeof(int);

   

	//  Color  Ѵٸ
	if(nColor > 0) 
	{
		// ColorPerVertex:  True or False
        bool* pColorPerVertex = (bool *)( pOutput + nLengthOfTotal);
		pColorPerVertex[0] = colorPerVertex;
		nLengthOfTotal += sizeof(bool);

		// Color Values
		pColor = (float *)( pOutput + nLengthOfTotal);
		for(i = 0 ; i < nColor; i++) {
			pColor[i] = m_3DMCDecoder.pColor[i]; //  Ѵ.
		}
		nLengthOfTotal += sizeof(float)*nColor;

		// Color Index
		pColorIndex = (int *)(pOutput + nLengthOfTotal);
		pColorIndex[0] = nColorIndex;
		nLengthOfTotal += sizeof(int);

		if(nColorIndex > 0)
		{
			int *pColorIndex = (int *)( pOutput + nLengthOfTotal);
			for(i = 0 ; i < nColorIndex; i++)
			{
				pColorIndex[i] = m_3DMCDecoder.pColorIndex[i];
			}
			nLengthOfTotal += sizeof(int)*nColorIndex;
		}
	} 	// if
	
	
	int* pNumNormal = (int*)(pOutput + nLengthOfTotal);
    pNumNormal[0] = nNormal;  // Store the number of Color
	nLengthOfTotal += sizeof(int);

	//  Normal  Ѵٸ
	if(nNormal > 0) 
	{
		// NormalPerVertex:  True or False
		bool* pNormalPerVertex = (bool *)( pOutput + nLengthOfTotal);
		pNormalPerVertex[0] = normalPerVertex;
		nLengthOfTotal += sizeof(bool);

		// Normal Values
		pNormal = (float *)( pOutput + nLengthOfTotal);
		for(i = 0 ; i < nNormal; i++) {
			pNormal[i] = m_3DMCDecoder.pNormal[i]; //  Ѵ.
		}
		nLengthOfTotal += sizeof(float)*nNormal;

		pNormalIndex = (int *)( pOutput + nLengthOfTotal);
		pNormalIndex[0] = nNormalIndex;
		nLengthOfTotal += sizeof(int);
		
		// Normal Index
		if(nNormalIndex > 0)
		{
			int *pNormalIndex = (int *)( pOutput + nLengthOfTotal);
			for(i = 0 ; i < nNormalIndex; i++)
			{
				pNormalIndex[i] = m_3DMCDecoder.pNormalIndex[i];
			}
			nLengthOfTotal += sizeof(int)*nNormalIndex;
		}
    } 	// if


	int* pNumTexCoord = (int*)(pOutput + nLengthOfTotal);
    pNumTexCoord[0] = nTexCoord;  // Store the number of TexCoord
	nLengthOfTotal += sizeof(int);

	// TexCoord
	if(nTexCoord > 0)
	{
		// Texcoord Values
		pTexCoord = (float *)( pOutput + nLengthOfTotal);
		nLengthOfTotal += sizeof(float)*nTexCoord;

		for(i = 0 ; i < nTexCoord; i++) {
			pTexCoord[i] = m_3DMCDecoder.pTexCoord[i]; //  Ѵ.
		}

		pTexCoordIndex = (int *)( pOutput + nLengthOfTotal);
		nLengthOfTotal += sizeof(int);
		pTexCoordIndex[0] = nTexCoordIndex;

		// TexCoord Index
		if(nTexCoordIndex > 0)
		{
			int *pTexCoordIndex = (int *)( pOutput + nLengthOfTotal);
			nLengthOfTotal += sizeof(int)*nTexCoordIndex;

			for(i = 0 ; i < nTexCoordIndex; i++)
			{
				pTexCoordIndex[i] = m_3DMCDecoder.pTexCoordIndex[i];
			}
		}
    } 	// if
	rnOutputLength = nLengthOfTotal;

#endif SAIT_CONFORMANCE_2003

	return TRUE;
}


// JYHAN 20021210
BOOL YDIBRDecoder :: Decode (
	LPBYTE pInput, int nInputLength, LPBYTE pOutput, int &rnOutputLength)
{
#ifdef SAIT_CONFORMANCE_2003
	if(GetOctree(&octdata) != S_OK)
		err("Can't create octree-object");

	CInStream istrm;
	SAIT_CAC::_shift = 16;
	CAC_QF codec(istrm);
	istrm.opendecWithBuffer((char*) pInput, nInputLength);

	octdata->len_octreeBitstream = istrm.Inbuf.size;
	octdata->setLenOctreeBitstreamBuf(octdata->len_octreeBitstream);

	memcpy(octdata->octreeBitstream,istrm.Inbuf.buf,octdata->len_octreeBitstream);	

	octdata->setLenOctreeBuf(octdata->len_octreeBitstream*10);
	octdata->setLenVoxelImageIndexBuf(octdata->len_octreeBitstream*10);

	if(octdata->DecodeOctree())
		err("Can't decode");

    int count = 0;

	unsigned char buff[4];
	buff[0]=(octdata->len_octree & 0xFF000000)>>24;
	buff[1]=(octdata->len_octree & 0x00FF0000)>>16;
	buff[2]=(octdata->len_octree & 0x0000FF00)>>8;
	buff[3]=(octdata->len_octree & 0x000000FF);

	memcpy(pOutput,buff,4);
    count += 4;

    for ( int i=0 ; i<octdata->len_octree ; i++ ) {
        pOutput[count++] = (unsigned char) octdata->octree[i];
    }

	buff[0]=(octdata->len_voxels & 0xFF000000)>>24;
	buff[1]=(octdata->len_voxels & 0x00FF0000)>>16;
	buff[2]=(octdata->len_voxels & 0x0000FF00)>>8;
	buff[3]=(octdata->len_voxels & 0x000000FF);

	memcpy(pOutput+4+octdata->len_octree,buff,sizeof(int));
    
    count += 4;
    for ( i=0 ; i<octdata->len_voxels ; i++ ) {
        pOutput[count++] = (unsigned char) octdata->voxelImageIndex[i];
    }
#else
/*	// TEMPARARY: the following codes are temparay.
	// it does not work with char string
	FILE* fp;
	fp = fopen("why.aac", "wb");
	fwrite((char*) pInput, sizeof(char), nInputLength, fp);
	fclose(fp);
	// TEMPARARY end
*/

	if(GetOctree(&octdata) != S_OK)
		err("Can't create octree-object");

	CInStream istrm;
	SAIT_CAC::_shift = 16;
	CAC_QF codec(istrm);
	istrm.opendecWithBuffer((char*) pInput, nInputLength);
	//istrm.opendec("why.aac");

	octdata->len_octreeBitstream = istrm.Inbuf.size;
	octdata->setLenOctreeBitstreamBuf(octdata->len_octreeBitstream);

	memcpy(octdata->octreeBitstream,istrm.Inbuf.buf,octdata->len_octreeBitstream);	

	octdata->setLenOctreeBuf(octdata->len_octreeBitstream*10);
	octdata->setLenVoxelImageIndexBuf(octdata->len_octreeBitstream*10);

	//printf("Decoding");
	if(octdata->DecodeOctree())
		err("Can't decode");
	//printf("\n");

	// Write("AFXObject-DIBR.txt");
#endif SAIT_CONFORMANCE_2003

	return TRUE;
}
// JYHAN end

void YDIBRDecoder :: Write(char* fileName)
{
	FILE* fp;
	fp = fopen(fileName, "w");

	fprintf(fp, "OctreeImage {\n\toctree [");

	for(int i=0; i<octdata->len_octree; i++)
	{
		if(i%50 == 0)
			fprintf(fp, "\n\t\t");

		fprintf(fp, " %d, ", octdata->octree[i]);
	}

	fprintf(fp, "\n\t]\n\tvoxelImage [");

	for(i=0; i<octdata->len_voxels; i++)
	{	
		if(i%50==0)
			fprintf(fp, "\n\t\t");

		fprintf(fp, " %d,", octdata->voxelImageIndex[i]);
	}
	fprintf(fp, "\n\t]\n}\n");

	fclose(fp);

	FreeOctree(&octdata);
}

// 2004-07-15, Shinjun {
BOOL PointTextureDecoder :: Decode (
	LPBYTE pInput, int nInputLength, LPBYTE pOutput, int &rnOutputLength)
{
	// variables: resolution values, indices
	int nX_max = 0;			// maximum in X-axis
	int nY_max = 0;			// maximum in Y-axis
	int nZ_max = 0;			// maximum in Z-axis
	int nIndexOfDepth = 0;
	int nIndexOfColor = 0;
	int nSizeOfMemory = 0;
	int i = 0;
	int j = 0;
	int k = 0;

	m_oPTDecoder.decodeWithBuffer((char*) pInput, nInputLength);
//	m_oPTDecoder.write("AFXObject-PT.txt");

#if defined SAIT_CONFORMANCE_2003
	// initial value of 
		// pOutput: the address of memory allocated by DecoderImp
		// rnOutputLength: the size of pOutput
	// final value of
		// rnOutputLength: the real size (actual data size) of pOutput

	// set: resolution values
	nX_max = m_oPTDecoder.m_nWidth-1;						// maximum in X-axis
	nY_max = m_oPTDecoder.m_nHeight-1;						// maximum in Y-axis
	nZ_max = (int)pow(2, m_oPTDecoder.m_nDepthNbBits+1)-1;	// maximum in Z-axis

	// if the size of pOutput is less then the size needed, return false
	nSizeOfMemory = sizeof(int)*3; // width, height, depth
	if(nSizeOfMemory > rnOutputLength)
		{
		rnOutputLength = nSizeOfMemory;
		return FALSE;
		}
	
	// set: the resoution values into the output memory
	((int*)pOutput)[0] = m_oPTDecoder.m_nWidth;
	((int*)pOutput)[1] = m_oPTDecoder.m_nHeight;
	((int*)pOutput)[2] = m_oPTDecoder.m_nDepthNbBits;

	// transfer: the depth data into the output memory
	for(j = 0; j <= nY_max; j++)
		{
		for(i = 0; i <= nX_max; i++)
			{
			int nSize = m_oPTDecoder.m_oColor.getSizeOfDepth(i, j);
			nSizeOfMemory += sizeof(int);
			if(nSizeOfMemory > rnOutputLength)
				{
				rnOutputLength = nSizeOfMemory;
				return FALSE;
				}
			((int*)pOutput)[3+nIndexOfDepth++] = nSize;

			int* pDepth = m_oPTDecoder.m_oColor.getDepth(i, j);
			for(k = 0; k < nSize; k++)
				{
				nSizeOfMemory += sizeof(int);
				if(nSizeOfMemory > rnOutputLength)
					{
					rnOutputLength = nSizeOfMemory;
					return FALSE;
					}
				((int*)pOutput)[3+nIndexOfDepth++] = pDepth[k];
				}
			}
		}

	// transfer: the color data into the output memory
	nIndexOfColor = 0;
	for(j = 0; j <= nY_max; j++)
		{
		for(i = 0; i <= nX_max; i++)
			{
			int nSize = m_oPTDecoder.m_oColor.getSizeOfDepth(i, j);
			
			unsigned char* pColor = m_oPTDecoder.m_oColor.getColor(i, j);
			for(k = 0; k < nSize; k++)
				{
				nSizeOfMemory += sizeof(unsigned char)*3;
				if(nSizeOfMemory > rnOutputLength)
					{
					rnOutputLength = nSizeOfMemory;
					return FALSE;
					}
				((unsigned char*)(pOutput+sizeof(int)*(3+nIndexOfDepth)))[nIndexOfColor++] = pColor[k*3];
				((unsigned char*)(pOutput+sizeof(int)*(3+nIndexOfDepth)))[nIndexOfColor++] = pColor[k*3+1];
				((unsigned char*)(pOutput+sizeof(int)*(3+nIndexOfDepth)))[nIndexOfColor++] = pColor[k*3+2];
				}
			}
		}

	rnOutputLength = nSizeOfMemory;
#endif SAIT_CONFORMANCE_2003

	return TRUE;
}

// 2004-07-15, Shinjun }

//ory dmlab 2007 01 03
BOOL JW3DMCExtensionDecoder :: Decode (
	LPBYTE pInput, int nInputLength, LPBYTE pOutput, int &rnOutputLength)
{
	m_3DMCExtensionDecoder.DecodeWithBuffer((char*) pInput, nInputLength);

#if defined _3DMCEXT_CONFORMANCE_
		
	bool ccw = m_3DMCExtensionDecoder.GetCcw(); // Read ccw value
	bool convex = m_3DMCExtensionDecoder.GetConvex();
	float creaseAngle = m_3DMCExtensionDecoder.GetCreaseAngle();
	bool solid = m_3DMCExtensionDecoder.GetSolid(); 
	
	int nCoord = m_3DMCExtensionDecoder.GetCoord(); 
	int nCoordIndex = m_3DMCExtensionDecoder.GetCoordIdx();
	
	int nColor = m_3DMCExtensionDecoder.GetColor();
	bool colorPerVertex = m_3DMCExtensionDecoder.GetColorPerVertex();
	int nColorIndex = m_3DMCExtensionDecoder.GetColorIndex();
	
	int nNormal = m_3DMCExtensionDecoder.GetNormal();
	bool normalPerVertex = m_3DMCExtensionDecoder.GetNormalPerVertex();
	int nNormalIndex = m_3DMCExtensionDecoder.GetNormalIndex();
	
	int nTexCoord  = m_3DMCExtensionDecoder.GetTexCoord();
	int nTexCoordIndex = m_3DMCExtensionDecoder.GetTexCoordIndex();

	// Field Values
	int    *pCoordIndex;
	float  *pCoord;
	int    *pColorIndex;
	float  *pColor;
	int    *pNormalIndex;
	float  *pNormal;
	int    *pTexCoordIndex;
	float  *pTexCoord;

	int nMemorySizeNeeded = 
		sizeof(bool)*5 +
		sizeof(int)*(8+nCoordIndex+nColorIndex+nNormalIndex+nTexCoordIndex) + 
		sizeof(float)*(1+nCoord+nColor+nNormal+nTexCoord);

	if(nMemorySizeNeeded > rnOutputLength)
	{
		rnOutputLength = nMemorySizeNeeded;
		return FALSE;
	}

	int i = 0;
	int   nLengthOfTotal = 0;

	// Headers: ccw
	bool* pCCW = (bool*) pOutput;      
    nLengthOfTotal += sizeof(bool);    // Increase Total Output size
	pCCW[0] = ccw;                     

	// Headers: convex
	bool* pConvex = (bool*)( pOutput + nLengthOfTotal );
	nLengthOfTotal += sizeof(bool);
	pConvex[0] = convex;

	// Headers: creaseAngle
	float* pCreaseAngle = (float *)(pOutput + nLengthOfTotal);
	nLengthOfTotal += sizeof(float);
	pCreaseAngle[0] = creaseAngle;

	// Headers: Solid	
    bool* pSolid = (bool *)( pOutput + nLengthOfTotal);
	nLengthOfTotal += sizeof(bool);
	pSolid[0] = solid;

	// Values: Coord (x,y,z)
    int* pNumCoord = (int*)(pOutput + nLengthOfTotal);
	nLengthOfTotal += sizeof(int);
	pNumCoord[0] = nCoord;                   // Store the number of Coord
	
	pCoord = (float *)( pOutput + nLengthOfTotal); 

	for(i = 0 ; i < nCoord; i++) {
		pCoord[i] = m_3DMCExtensionDecoder.pCoord[i]; // Store Coord
    }	
	nLengthOfTotal += sizeof(float)*nCoord;

	// Values: coordIndex
	int* pNumCoordIndex = (int*)(pOutput + nLengthOfTotal);
    pNumCoordIndex[0] = nCoordIndex;  // Store the number of CoordIndex
	nLengthOfTotal += sizeof(int);

	pCoordIndex = (int *)( pOutput + nLengthOfTotal);
	for(i = 0 ; i < nCoordIndex; i++) {
		pCoordIndex[i] = m_3DMCExtensionDecoder.pCoordIdx[i]; 
    }
	nLengthOfTotal += sizeof(int)*nCoordIndex;
	
	// Values: Color
	int* pNumColor = (int*)(pOutput + nLengthOfTotal);
    pNumColor[0] = nColor;  // Store the number of Color
	nLengthOfTotal += sizeof(int);

	if(nColor > 0) 
	{
		// ColorPerVertex:  True or False
        bool* pColorPerVertex = (bool *)( pOutput + nLengthOfTotal);
		pColorPerVertex[0] = colorPerVertex;
		nLengthOfTotal += sizeof(bool);

		// Color Values
		pColor = (float *)( pOutput + nLengthOfTotal);
		for(i = 0 ; i < nColor; i++) {
			pColor[i] = m_3DMCExtensionDecoder.pColor[i]; 
		}
		nLengthOfTotal += sizeof(float)*nColor;

		// Color Index
		pColorIndex = (int *)(pOutput + nLengthOfTotal);
		pColorIndex[0] = nColorIndex;
		nLengthOfTotal += sizeof(int);

		if(nColorIndex > 0)
		{
			int *pColorIndex = (int *)( pOutput + nLengthOfTotal);
			for(i = 0 ; i < nColorIndex; i++)
			{
				pColorIndex[i] = m_3DMCExtensionDecoder.pColorIndex[i];
			}
			nLengthOfTotal += sizeof(int)*nColorIndex;
		}
	} 	// if
	
	
	int* pNumNormal = (int*)(pOutput + nLengthOfTotal);
    pNumNormal[0] = nNormal;  // Store the number of Color
	nLengthOfTotal += sizeof(int);

	if(nNormal > 0) 
	{
		// NormalPerVertex:  True or False
		bool* pNormalPerVertex = (bool *)( pOutput + nLengthOfTotal);
		pNormalPerVertex[0] = normalPerVertex;
		nLengthOfTotal += sizeof(bool);

		// Normal Values
		pNormal = (float *)( pOutput + nLengthOfTotal);
		for(i = 0 ; i < nNormal; i++) {
			pNormal[i] = m_3DMCExtensionDecoder.pNormal[i]; 
		}
		nLengthOfTotal += sizeof(float)*nNormal;

		pNormalIndex = (int *)( pOutput + nLengthOfTotal);
		pNormalIndex[0] = nNormalIndex;
		nLengthOfTotal += sizeof(int);
		
		// Normal Index
		if(nNormalIndex > 0)
		{
			int *pNormalIndex = (int *)( pOutput + nLengthOfTotal);
			for(i = 0 ; i < nNormalIndex; i++)
			{
				pNormalIndex[i] = m_3DMCExtensionDecoder.pNormalIndex[i];
			}
			nLengthOfTotal += sizeof(int)*nNormalIndex;
		}
    } 	// if


	int* pNumTexCoord = (int*)(pOutput + nLengthOfTotal);
    pNumTexCoord[0] = nTexCoord;  // Store the number of TexCoord
	nLengthOfTotal += sizeof(int);

	// TexCoord
	if(nTexCoord > 0)
	{
		// Texcoord Values
		pTexCoord = (float *)( pOutput + nLengthOfTotal);
		nLengthOfTotal += sizeof(float)*nTexCoord;

		for(i = 0 ; i < nTexCoord; i++) {
			pTexCoord[i] = m_3DMCExtensionDecoder.pTexCoord[i]; 
		}

		pTexCoordIndex = (int *)( pOutput + nLengthOfTotal);
		nLengthOfTotal += sizeof(int);
		pTexCoordIndex[0] = nTexCoordIndex;

		// TexCoord Index
		if(nTexCoordIndex > 0)
		{
			int *pTexCoordIndex = (int *)( pOutput + nLengthOfTotal);
			nLengthOfTotal += sizeof(int)*nTexCoordIndex;

			for(i = 0 ; i < nTexCoordIndex; i++)
			{
				pTexCoordIndex[i] = m_3DMCExtensionDecoder.pTexCoordIndex[i];
			}
		}
    } 	// if
	rnOutputLength = nLengthOfTotal;

#endif _3DMCEXT_CONFORMANCE_

	return TRUE;
}
//~ory DMLAB