/***********************************************************************
This software module was originally developed by Sang Oak Woo
(SAMSUNG AIT) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. SAMSUNG AIT retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2002.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Sang Oak Woo at sowoo@sait.samsung.co.kr.
----------------------------------------------------------------------*/

#if !defined(_SAIT_AFX_STREAM_DECODER_H_)	// Sentry, use file only if it's not already included.
#define _SAIT_AFX_STREAM_DECODER_H_

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Decoder
FILE:       SAIT_AFX_STREAM_DECODER.H
AUTHOR:     Sang Oak Woo (SAMSUNG AIT)
OVERVIEW:   Decoding the AFX stream object made in SAIT.
-------------------------------------------------------------------------*/

#include "AFX.h"
#include "InterpolatorCompression/CompressedOrientationInterpolator.h"
#include "InterpolatorCompression/CompressedPositionInterpolator.h"
#include "InterpolatorCompression/CompressedCoordinateInterpolator.h"
#include "DIBR/Octree.h"	//JYHAN 20021210
#include "3DMC/Expand/mpgelib/cifsdec.h" 
// 2004-07-15, Shinjun {
#include "PTCompression/CPTDecoder.h"
// 2004-07-15, Shinjun }
//ory DMLAB 2006 12 28
#include "3DMCExtension/Expand/MpgeExtLib/cifsdec.h"
//~ory DMLAB 

class JWPositionInterpolatorDecoder : public AFXTemplate
{
public:
	virtual BOOL Setup (AFXExtDescriptor *pSpecificInfo) { return TRUE; }
	virtual BOOL Decode (LPBYTE pInput, int nInputLength, LPBYTE pOutput, int &rnOutputLength);
	virtual int GetOptimalOutputSize () { return 100000; }

private:
	CompressedPositionInterpolator m_PositionInterpolator;
};

class JWOrientationInterpolatorDecoder : public AFXTemplate
{
public:
	virtual BOOL Setup (AFXExtDescriptor *pSpecificInfo) { return TRUE; }
	virtual BOOL Decode (LPBYTE pInput, int nInputLength, LPBYTE pOutput, int &rnOutputLength);
	virtual int GetOptimalOutputSize () { return 100000; }

private:
	CompressedOrientationInterpolator m_OrientationInterpolator;
};


class JWCoordinateInterpolatorDecoder : public AFXTemplate
{
public:
	virtual BOOL Setup (AFXExtDescriptor *pSpecificInfo) { return TRUE; }
	virtual BOOL Decode (LPBYTE pInput, int nInputLength, LPBYTE pOutput, int &rnOutputLength);
	virtual int GetOptimalOutputSize () { return 200000; }

private:
	CompressedCoordinateInterpolator m_CoordinateInterpolator;
};

class YDIBRDecoder : public AFXTemplate
{
public:
	virtual BOOL Setup (AFXExtDescriptor *pSpecificInfo) { return TRUE; }
	virtual BOOL Decode (LPBYTE pInput, int nInputLength, LPBYTE pOutput, int &rnOutputLength);
	virtual int GetOptimalOutputSize () { return 200000; }

private:
	COctree *octdata;
	void hlp() { exit(1); }

	void err(const char *str){
		printf("\nError: %s\n",str);
		exit(1);
	}

	int GetOctree(COctree  ** pOctree)
	{
		*pOctree=new COctree;
		return S_OK;
	}

	int FreeOctree(COctree ** pOctree)
	{
		if(*pOctree != NULL)
			delete (*pOctree);
		*pOctree=NULL;
		return S_OK;
	}

	void Write(char* fileName);
};

class JW3DMCDecoder : public AFXTemplate
{
public:
	virtual BOOL Setup (AFXExtDescriptor *pSpecificInfo) { return TRUE; }
	virtual BOOL Decode (LPBYTE pInput, int nInputLength, LPBYTE pOutput, int &rnOutputLength);
	virtual int GetOptimalOutputSize () { return 100000; }

private:
	CIFSDecoder m_3DMCDecoder;
};

//ory DMLAB 2006 12 28
class JW3DMCExtensionDecoder : public AFXTemplate
{
public:
	virtual BOOL Setup (AFXExtDescriptor *pSpecificInfo) { return TRUE; }
	virtual BOOL Decode (LPBYTE pInput, int nInputLength, LPBYTE pOutput, int &rnOutputLength);
	virtual int GetOptimalOutputSize () { return 100000; }

private:
	CIFSDecoderExt m_3DMCExtensionDecoder;
};
//~ory DMLAB

// 2004-07-15, Shinjun {
class PointTextureDecoder : public AFXTemplate
{
public:
	virtual BOOL Setup (AFXExtDescriptor *pSpecificInfo) { return TRUE; }
	virtual BOOL Decode (LPBYTE pInput, int nInputLength, LPBYTE pOutput, int &rnOutputLength);
	virtual int GetOptimalOutputSize () { return 5000000; }

private:
	CPTDecoder m_oPTDecoder;
};
// 2004-07-15, Shinjun }

#endif