/***********************************************************************
This software module was originally developed by Sebastian Amengual
(Universidad Politecnica de Madrid, UPM) in the course of development of 
the MPEG-4 Systems (ISO/IEC 14496-1) standard. This software module is
an implementation of a part of one or more MPEG-4 Systems 
(ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems
(ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 Systems 
(ISO/IEC 14496-1) free license to this software module or modifications
thereof for use in hardware or software products claiming conformance to
the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use this 
software module in hardware or software products are advised that its 
use may infringe existing patents. The original developer of this 
software module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software module
or modifications thereof in an implementation. Copyright is not 
released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
UPM retains full right to use the code for his/her own purpose, assign 
or donate the code to a third party and to inhibit third parties from 
using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
products. This copyright notice must be included in all copies or 
derivative works. Copyright  2005
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Francisco Moran at fmb@gti.ssr.upm.es.
----------------------------------------------------------------------*/

/*----------------------------------------------------------------------
Project Player3d
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  WaveSurfDecoder
FILE:       WaveSurfDec.cpp
AUTHOR:     Sebastian Amengual (UPM)
OVERVIEW:   This file should have contained the decoder of MPEG-4's WSS 
			tool but, due to the inherent nature of the Wavelet 
			Subdivision Surfaces it is not possible to perform the 
			decoding at this point, because no information about the 
			base mesh is available here. The decoding is done in 
			Wavesurf.cpp, which is called by WaveSurfProxy.cpp.
----------------------------------------------------------------------*/

#include <fstream>
#include "ODP.h"
#include "WaveSurfDec.h"
#include "WSSDataExchange.h"
#include "sdslib/WSSNode.h"
#include "WaveSurf.h"
#include "IM1Decoders/AFX/AFX.h"


//-----------------------------------------------------------------------------
// Name: WriteToStream()
// Desc: Writes the given information into a bitstream pOutput and gives
//		 its length in len.
//-----------------------------------------------------------------------------

static bool WriteToStream(bool hasScaleCoeff, int NbBpSC, int NbBPX, int NbBPY, int NbBPZ, 
						  int Wtype, bool lift, int NumberOfLevels, float Xmax, float  Ymax,
						  float Zmax, LPBYTE pInput, int inputLength, BYTE *pOutput, int &len)
{
	WSSDataExchange dataExchange(pOutput, false, len);
  
	if (!dataExchange.StoreNoTags(12))
		return false; 
	if (!dataExchange.StoreNext(WSSDataExchange::TAG_hasScaleCoeff, sizeof(bool), &hasScaleCoeff))
		return false; 
	if (!dataExchange.StoreNext(WSSDataExchange::TAG_NbBpSC, sizeof(int), &NbBpSC))
		return false; 
	if (!dataExchange.StoreNext(WSSDataExchange::TAG_NbBPX, sizeof(int), &NbBPX))
		return false; 
	if (!dataExchange.StoreNext(WSSDataExchange::TAG_NbBPY, sizeof(int), &NbBPY))
		return false; 
	if (!dataExchange.StoreNext(WSSDataExchange::TAG_NbBPZ, sizeof(int), &NbBPZ))
		return false; 
	if (!dataExchange.StoreNext(WSSDataExchange::TAG_Wtype, sizeof(int), &Wtype))
		return false; 
	if (!dataExchange.StoreNext(WSSDataExchange::TAG_lift, sizeof(bool), &lift))
		return false; 
	if (!dataExchange.StoreNext(WSSDataExchange::TAG_NumberOfLevels, sizeof(int), &NumberOfLevels))
		return false; 
	if (!dataExchange.StoreNext(WSSDataExchange::TAG_Xmax, sizeof(float), &Xmax))
		return false; 
	if (!dataExchange.StoreNext(WSSDataExchange::TAG_Ymax, sizeof(float), &Ymax))
		return false; 
	if (!dataExchange.StoreNext(WSSDataExchange::TAG_Zmax, sizeof(float), &Zmax))
		return false; 
	if (!dataExchange.StoreNext(WSSDataExchange::TAG_stream , inputLength, pInput))
		return false; 

  len = dataExchange.Finish();
 
  return true;
}


//-----------------------------------------------------------------------------
// Name: WaveSurfDecoder()
// Desc: Constructor of the WaveSurfDecoder class. 
//-----------------------------------------------------------------------------

WaveSurfDecoder :: WaveSurfDecoder()
{
}
  

//-----------------------------------------------------------------------------
// Name: Setup()
// Desc: Extracts the decoder specific info form the given AFXExtDescriptor.
//       This information is needed in order to decode the bitstream.
//-----------------------------------------------------------------------------

BOOL WaveSurfDecoder :: Setup (	AFXExtDescriptor *pSpecificInfo)
{

	WMDecoderSpecificInfo *pInfo=(WMDecoderSpecificInfo *)pSpecificInfo;
	
		_hasScaleCoeff=pInfo->hasScaleCoeff;
		if (_hasScaleCoeff)
			_NbBpSC=pInfo->NbBpSC;
		_NbBPX=pInfo->NbBPX;
		_NbBPY=pInfo->NbBPY;
		_NbBPZ=pInfo->NbBPZ;
		_Wtype=pInfo->Wtype;
		_lift=pInfo->lift;
		_NumberOfLevels=pInfo->NumberOfLevels;
		_Xmax=pInfo->Xmax;
		_Ymax=pInfo->Ymax;
		_Zmax=pInfo->Zmax;
					
	return TRUE;
}


//-----------------------------------------------------------------------------
// Name: GetOptimalOutputSize()
// Desc: Gives an approximate value of the size of the output stream in the 
//       Decode() method. It's impossible to know here the size of this stream, 
//       because it will depend on the size of the input stream. A high value
//       is give, but it could be changed in case it isn't enough.
//-----------------------------------------------------------------------------

int WaveSurfDecoder :: GetOptimalOutputSize ()
{

	return 400000;
}


//-----------------------------------------------------------------------------
// Name: Decode()
// Desc: Just calls the WriteToStream() method and passes it all the extracted 
//       specific info and the whole bitstream, so it can be decoded in the 
//       WaveSurfProxy.
//-----------------------------------------------------------------------------

BOOL WaveSurfDecoder :: Decode (LPBYTE pInput, int nInputLength, LPBYTE pOutput, int &rnOutputLength)
{
		WriteToStream(_hasScaleCoeff, _NbBpSC, _NbBPX, _NbBPY, _NbBPZ, _Wtype, _lift, _NumberOfLevels,
			_Xmax, _Ymax, _Zmax, pInput, nInputLength, pOutput, rnOutputLength);

	return TRUE;
}