/***********************************************************************
This software module was originally developed by Yannick Maret (from the
Ecole Polytechnique Federale de Lausanne, EPFL, during a stay at the
Universidad Politecnica de Madrid, UPM) in the course of development of 
the MPEG-4 Systems (ISO/IEC 14496-1) standard. This software module is
an implementation of a part of one or more MPEG-4 Systems 
(ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems
(ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 Systems 
(ISO/IEC 14496-1) free license to this software module or modifications
thereof for use in hardware or software products claiming conformance to
the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use this 
software module in hardware or software products are advised that its 
use may infringe existing patents. The original developer of this 
software module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software module
or modifications thereof in an implementation. Copyright is not 
released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
EPFL and UPM retain full right to use the code for his/her own purpose,
assign or donate the code to a third party and to inhibit third parties 
from using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
products. This copyright notice must be included in all copies or 
derivative works. Copyright  2003-2005
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yannick Maret at Yannick.Maret@a3.epfl.ch or Francisco
Moran at fmb@gti.ssr.upm.es.
----------------------------------------------------------------------*/

/***********************************************************************
Revision 1.1: 2005/9/15

Alberto Lopez-Ceron Pinilla has modified the file to support the use of
local coordinates.
***********************************************************************/

/*-------------------------------------------------------------------------
Project Player3d
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  WaveSurfDecoder
FILE:       SSNode.cpp
AUTHOR:     Yannick Maret (EPFL, UPM)
OVERVIEW:   Implementation of the SubdivisionSurface node.
-------------------------------------------------------------------------*/

// *** INCLUDES ***
#include "SSNode.h"
#include <limits>

float pp=0.0f;

// *** CONSTRUCTOR ***
SSNode::SSNode():
subdivisionLevel(0),
_ccw(true),
_colorPerVert(true),
_convex(true),
_solid(true),
_isCoordIdSet(false),
_isStencilBuilt(false),
_pwavCoeff(NULL),
_lift(false),
//_localCoord(false),
_pcoordCallBack(NULL),
_pcoordIdCallBack(NULL)
{
}
// *** DATA ACCESS / SET ***
void SSNode::setTexCoordId(const vector<int>& r)
{   
   _texCoordId = r;
}
void SSNode::setColorId(const vector<int>& r) 
{
   _colorId = r;
}
void SSNode::setCoordId(const vector<int>& r)
{
   _isCoordIdSet = true;
   if (r.back() != -1)
      _coordId.reserve(r.size() + 1);
   _coordId = r;
   if (_coordId.back() != -1)
      _coordId.push_back(-1);
   if (subdivisionLevel == 0)
      return;
   _coordIdAux = _coordId;
   psetBaseCoordId(&_coordId, TRI);
   createTopoLUT(subdivisionLevel, false);
   _isStencilBuilt = false;
}
void SSNode::setCreaseEdgeId(const vector<int>& r)
{
   _creaseEdgeId = r;
   psetBaseCreaseEdgeId(&_creaseEdgeId);
   _isStencilBuilt = false;
}
void SSNode::setCornerVertId(const vector<int>& r)
{
   _cornerVertId = r;
   psetBaseCornerVertId(&_cornerVertId);
   _isStencilBuilt = false;
}
void SSNode::setDartVertId(const vector<int>& r)
{
   _dartVertId = r;
   psetBaseDartVertId(&_dartVertId);
   _isStencilBuilt = false;
}
void SSNode::setCreaseVertId(const vector<int>& r)
{
   _creaseVertId = r;
   psetBaseCreaseVertId(&_creaseVertId);
   _isStencilBuilt = false;
}
const vector<coord3D>* SSNode::pgetCoord() const
{
   if (subdivisionLevel != 0)
      return &_currCoord;
   else
      return pcoordNode;
}
const vector<int>*  SSNode::pgetCoordId() const
{
   if (subdivisionLevel != 0)
      return pgetFinalCoordId();
   else
      return &_coordId;
}
void SSNode::_psetWavCoef(const vector<float>* rp)
{
   _pwavCoeff = rp;
}
void SSNode::_psetLift(const bool r)
{
   _lift = r;
}
const coord3D* SSNode::pgetBBox() const
{
   return _pboundBox;
}
void SSNode::coordChangeFunc(void (*rpcallBack)())
{
   _pcoordCallBack = rpcallBack;
}
void SSNode::coordIdChangeFunc(void (*rpcallBack)())
{
   _pcoordIdCallBack = rpcallBack;
}
// *** GEOMETRY UPDATE ***
void SSNode::onCoordChange()
{
   if (subdivisionLevel != 0)
   {
      if (!_isStencilBuilt)
      {
	 //_coordId = _coordIdAux;
	 //pcoordNode = pcoordNodeAux;
		 //psetBaseCoordId(&_coordId, TRI);
         //createTopoLUT(subdivisionLevel, false);
         createTagsTables(subdivisionLevel);
         buildStencilsLUT(subdivisionLevel, (etSSsType)subdivisionType, subdivisionSubType);
         _isStencilBuilt = true;
		 buildTriStenLUT(subdivisionLevel);
         if (_pcoordIdCallBack != NULL)
            _pcoordIdCallBack();
      }
      _updateCoord();
      if (_pcoordCallBack != NULL)
         _pcoordCallBack();
   }
}
void SSNode::_updateCoord()
{
#ifdef _TIME_SSLIB_
   _startTime = clock();
#endif
   // *** STACK VARIABLE
   stencil::const_iterator i_stencil;
   unsigned old_size, lvl, j;
   float pow2;
   //vector<coord3D>::const_iterator i_coord;
   vector<float>::const_iterator i_dx, i_dy, i_dz;
   const vector<vector<stencil> >* p_stencils = pgetStencilsLUT();
   const vector<vector<stencil> >* p_triSten = pgetTriStenLUT();
   vector<stencil>::const_iterator i_stencils;
   vector<stencil>::const_iterator i_triStens;
   vector<vector<stencil> >::const_iterator i_stencilss;
   vector<vector<stencil> >::const_iterator i_triStenss;
   vector<coord3D>::iterator i_c_coord;
   vector<coord3D> tmp_coord;
   const vector<coord3D>* p_t_coord;
   coord3D lift_coeff;
   coord3D base[3];
   coord3D points[4];
   coord3D normal, normal2;
   float dx, dy, dz;

   // # FOR LIFTING
   unsigned v_idx;
   vector<ring>::const_iterator i_ring;
   ring::const_iterator i_neigh;
   // *** INITIALISATION
   _currCoord.reserve(p_stencils->back().size());
   if (isInterpolating())
   {
      // # INTERPOLATING SCHEME ARE COMPUTED ONLINE
      _currCoord = *pcoordNode;
      p_t_coord = &_currCoord;
   }
   else
   {
      // # APPROXIMTING SCHEME NEED TEMPORARY MEMORY
      tmp_coord.reserve(p_stencils->back().size());
      p_t_coord = pcoordNode;
   }
   old_size = pcoordNode->size();
   pow2 = 1.0F;

   // *** COMPUTE THE FINAL GEOMETRY ***

   for (i_stencilss = p_stencils->begin(), i_triStenss = p_triSten->begin(), lvl = 0 ; i_stencilss < p_stencils->end() ; i_stencilss++, i_triStenss++, lvl++)
   {
      // # LIFTING
      if (_lift)
      {
         i_ring = pgetRingsAtLvl(lvl + 1)->begin();
         i_c_coord = _currCoord.begin();
         for (i_ring, v_idx = 0 ; v_idx < old_size ; i_ring++, v_idx++, i_c_coord++)
         {
            lift_coeff.clear();
            for (i_neigh = i_ring->begin() ; i_neigh < i_ring->end() ; i_neigh++)
            {
               lift_coeff.x += _pwavCoeff[X][*i_neigh]*pow2;
               lift_coeff.y += _pwavCoeff[Y][*i_neigh]*pow2;
               lift_coeff.z += _pwavCoeff[Z][*i_neigh]*pow2;
            }
            lift_coeff *= 6.F*(float)sin(M_PI/3);
            lift_coeff /= (float)sin(2*M_PI/getNbAdjVert(v_idx))*8.F*getNbAdjVert(v_idx);
            *i_c_coord += lift_coeff;
         }         
      }
      // # INITIALISATION
      _currCoord.resize(i_stencilss->size());
      i_stencils = i_stencilss->begin();
      i_triStens = i_triStenss->begin();
      i_c_coord = _currCoord.begin();
      if (isInterpolating())
      {
         // there are no stencils for even vertices (keep same position)
         i_stencils += old_size;
	     i_triStens += old_size;
         i_c_coord += old_size;
      }
      // # COMPUTE THE NEW POSITIONS
      i_dx = _pwavCoeff[X].begin() + old_size;
      i_dy = _pwavCoeff[Y].begin() + old_size;
      i_dz = _pwavCoeff[Z].begin() + old_size;

      for (i_stencils, i_triStens, i_c_coord ; i_stencils < i_stencilss->end() ; i_stencils++, i_triStens++, i_c_coord++, i_dx++, i_dy++, i_dz++)
      {
        (*i_c_coord).clear();
        for (i_stencil = i_stencils->begin() ; i_stencil < i_stencils->end() ; i_stencil++)
            *i_c_coord += (*p_t_coord)[i_stencil->idx]*i_stencil->weight;     

		if (_localCoord){//added by Alberto Lpez-Cern Pinilla to support the use of local coordinates
			for (j = 0, i_stencil = i_triStens->begin(); i_stencil < i_triStens->end(); i_stencil++, j++)
				points[j] = (*p_t_coord)[i_stencil->idx];
	
			coord3D aux, aux2;
			normal = calcNormal2(&points[0], &points[1], &points[2]);
		
			if (j==4)
			{
				normal2 = calcNormal2(&points[0], &points[3], &points[1]);
				aux = normal + normal2;
			}
			else aux=normal;

			aux2 = points[1] - points[0];

			base[0] = *normalize(&aux);
			base[1] = *normalize(&aux2);
			base[2] = vecProd(&base[0], &base[1]);

			dx = (*i_dx)*base[0].x + (*i_dy)*base[1].x + (*i_dz)*base[2].x;
			dy = (*i_dx)*base[0].y + (*i_dy)*base[1].y + (*i_dz)*base[2].y;
			dz = (*i_dx)*base[0].z + (*i_dy)*base[1].z + (*i_dz)*base[2].z;
		}
		else {
			dy = (*i_dy);
			dx = (*i_dx);
			dz = (*i_dz);
		}

		//DGB:sag
		if(14.43>i_c_coord->x && 14.42<i_c_coord->x) 
			pp +=dz;
		if(-2.14>i_c_coord->x && -2.15<i_c_coord->x) 
			pp +=dz;
		

        i_c_coord->x += dx;
        i_c_coord->y += dy;
        i_c_coord->z += dz;
      }
      old_size = _currCoord.size();
      if (!isInterpolating() && i_stencilss < (p_stencils->end() - 1))
      {
         tmp_coord.swap(_currCoord);
         p_t_coord = &tmp_coord;
      }
   }
   // # BOUNDING BOX
   static numeric_limits<float> float_lims;
   _pboundBox[0].x = float_lims.max();
   _pboundBox[0].y = float_lims.max();
   _pboundBox[0].z = float_lims.max();
   _pboundBox[1].x = -float_lims.max();
   _pboundBox[1].y = -float_lims.max();
   _pboundBox[1].z = -float_lims.max();
   i_c_coord = _currCoord.begin() ;
   for (i_c_coord ; i_c_coord < _currCoord.end() ; i_c_coord++)
   {
      // x
      if (i_c_coord->x > _pboundBox[1].x) 
         _pboundBox[1].x = i_c_coord->x;
      if (i_c_coord->x < _pboundBox[0].x)
         _pboundBox[0].x = i_c_coord->x;
      // y
      if (i_c_coord->y > _pboundBox[1].y) 
         _pboundBox[1].y = i_c_coord->y;
      if (i_c_coord->y < _pboundBox[0].y)
         _pboundBox[0].y = i_c_coord->y;
      // z
      if (i_c_coord->z > _pboundBox[1].z) 
         _pboundBox[1].z = i_c_coord->z;
      if (i_c_coord->z < _pboundBox[0].z)
         _pboundBox[0].z = i_c_coord->z;
   }
#ifdef _TIME_SSLIB_
   _endTime = clock();
   _refineTime = _endTime - _startTime;
   cout << "SSNode: refine Timing (in [s]):\n";
   cout << "-------------------------------\n";
   cout << "Geometry refine       *" << _refineTime/(float)CLK_TCK << '*' << "\n";
#endif
}
// *** TEXTURE UDPATE ***
void SSNode::onTexCoordChange()
{
   if (subdivisionLevel != 0)
      _updateTex(subdivisionLevel);
}
void SSNode::_updateTex(const unsigned rNumSub)
{
   if (_currTexCoordId.size() != 0)
   {
      // *** USE CUSTOM TEX COORD INDICES
      // # STACK VARIABLES
      vector<int> _currCoord_id;
      vector<int>::iterator i_tcid;
      vector<int>::const_iterator i_curr_tcid;
      unsigned v0, v1, v2, v3, v4, v5, i, num_tc, num_tri;
      // # GET THE FINAL NUMBER OF TEXTURE COORDINATE
      num_tc = _currTexCoord.size();
      num_tri = _currTexCoordId.size()/4;
      for ( i = 0 ; i < rNumSub ; i++)
      {
         num_tc += num_tri*3;
         num_tri *= 4;
      }
      // # RESERVE EXACT MEMORY
      _currTexCoord.reserve(num_tc);
      _currTexCoordId.reserve(num_tri*4);
      _currCoord_id.reserve(num_tri*4);
      // # INTERPOLATE TEX COORD & CREATE FINAL TEX COORD ID
      num_tc = _currTexCoord.size();
      for (i = 0 ; i < rNumSub ; i++)
      {
         i_tcid = _currCoord_id.begin();
         i_curr_tcid = _currTexCoordId.begin();
         for (i_curr_tcid ; i_curr_tcid < _currTexCoordId.end() ; i_curr_tcid++)
         {
            // # get even vertices
            v0 = *(i_curr_tcid++);
            v1 = *(i_curr_tcid++);
            v2 = *(i_curr_tcid++);
            // # create odd vertices & interpolate tex coord
            // edge v0 & v1
            v3 = num_tc++;
            _currTexCoord.push_back((_currTexCoord[v0] + _currTexCoord[v1])/2);
            // edge v1 & v2
            v4 = num_tc++;
            _currTexCoord.push_back((_currTexCoord[v1] + _currTexCoord[v2])/2);
            // edge v0 & v2
            v5 = num_tc++;
            _currTexCoord.push_back((_currTexCoord[v0] + _currTexCoord[v2])/2);
            // # create the four new triangles
            (*i_tcid++) = v0; (*i_tcid++) = v3; (*i_tcid++) = v5; (*i_tcid++) = -1; // T0
            (*i_tcid++) = v1; (*i_tcid++) = v4; (*i_tcid++) = v3; (*i_tcid++) = -1; // T1
            (*i_tcid++) = v2; (*i_tcid++) = v5; (*i_tcid++) = v4; (*i_tcid++) = -1; // T2
            (*i_tcid++) = v3; (*i_tcid++) = v4; (*i_tcid++) = v5; (*i_tcid++) = -1; // T3
         }
         _currTexCoordId.swap(_currCoord_id);
         _pcurrTexCoordId = &_currTexCoordId;
      }
   }
   else
   {
      // *** USE COORD INDICES AS TEX COORD INDICES ***
      // # STACK VARIABLES
      unsigned v0, v1, v2, v3, v4, v5, i; 
      const vector<vector<int> >* p_all_cid = pgetCoordIdLUT();
      vector<int>::const_iterator i_cid;
      const vector<unsigned>* p_num_vert = pgetNbVertLUT();
      // # RESERVE EXACT MEMORY
      _currTexCoord.resize(p_num_vert->back());
      // # INTERPOLATE TEX COORD
      for (i = 1 ; i < rNumSub ; i++)
      {
         i_cid = (*p_all_cid)[i].begin();
         for (i_cid ; i_cid < (*p_all_cid)[i].end() ; i_cid++)
         {
            // # get even vertices
            v0 = *i_cid;
            i_cid += 4;
            v1 = *i_cid;
            i_cid += 4;
            v2 = *i_cid;
            i_cid += 4;
            // # get odd vertices & interpolate tex coord
            // edge v0 & v1
            v3 = *(i_cid++);
            _currTexCoord[v3] = (_currTexCoord[v0] + _currTexCoord[v1])/2;
            // edge v1 & v2
            v4 = *(i_cid++);
            _currTexCoord[v4] = (_currTexCoord[v1] + _currTexCoord[v2])/2;
            // edge v0 & v2
            v5 = *(i_cid++);
            _currTexCoord[v5] = (_currTexCoord[v0] + _currTexCoord[v2])/2;
         }
      }
      _pcurrTexCoordId = &p_all_cid->back();
   }
}
//Added by Alberto Lpez-Cern Pinilla:
coord3D* SSNode::normalize(coord3D* v)
{
	float mod;
	mod = (float)sqrt(pow(v->x,2) + pow(v->y,2) + pow(v->z,2));
	if (!(mod == 0.0F || mod == 1.0F)) *v /= mod;
	return v;
}

coord3D SSNode::vecProd(coord3D* a, coord3D* b)
{
    coord3D v;
    v.x = a->y * b->z - a->z * b->y;
    v.y = a->z * b->x - a->x * b->z;
    v.z = a->x * b->y - a->y * b->x;
    return v;
}

coord3D SSNode::calcNormal(coord3D* a, coord3D* b, coord3D* c)
{
	coord3D v;
	coord3D ba = *b - *a;
	coord3D ca = *c - *a;
	coord3D a1 = vecProd(&ba, &ca);
	v = *normalize(&a1);
	return v;
}

coord3D SSNode::calcNormal2(coord3D* a, coord3D* b, coord3D* c)
{
	coord3D ba = *b - *a;
	coord3D ca = *c - *a;
	coord3D a1 = vecProd(&ba, &ca);
	return a1;
}


float SSNode::calcK(coord3D* v)
{
	float mod = (float)sqrt(pow(v->x,2) + pow(v->y,2) + pow(v->z,2));
	return mod/2;
}

bool SSNode::areEqual(coord3D* a, coord3D* b)
{
  if ((a->x == b->x) && (a->y == b->y) && (a->z == b->z)) return true;
  else return false;
}