/***********************************************************************
This software module was originally developed by Yannick Maret (from the
Ecole Polytechnique Federale de Lausanne, EPFL, during a stay at the
Universidad Politecnica de Madrid, UPM) in the course of development of 
the MPEG-4 Systems (ISO/IEC 14496-1) standard. This software module is
an implementation of a part of one or more MPEG-4 Systems 
(ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems
(ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 Systems 
(ISO/IEC 14496-1) free license to this software module or modifications
thereof for use in hardware or software products claiming conformance to
the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use this 
software module in hardware or software products are advised that its 
use may infringe existing patents. The original developer of this 
software module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software module
or modifications thereof in an implementation. Copyright is not 
released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
EPFL and UPM retain full right to use the code for his/her own purpose,
assign or donate the code to a third party and to inhibit third parties 
from using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
products. This copyright notice must be included in all copies or 
derivative works. Copyright  2003-2005
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yannick Maret at Yannick.Maret@a3.epfl.ch or Francisco
Moran at fmb@gti.ssr.upm.es.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Player3d
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  WaveSurfDecoder
FILE:       SSNode.h
AUTHOR:     Yannick Maret (EPFL, UPM)
OVERVIEW:   Interface of the SubdivisionSurface node.
-------------------------------------------------------------------------*/

#ifndef _SUBDIVISION_SURFACE_NODE_INCLUDED_
#define _SUBDIVISION_SURFACE_NODE_INCLUDED_
// *** INCLUDES CLASS ***
#include "_SSsStencil.h"
// *** SUBDIVISION SURFACE NODE CLASS ***
class SSNode : public _SSsStencil
{
   // # MPEG-4 NORMATIVE STUFFS
public:
   void setColorId(const vector<int>&);
   void setCoordId(const vector<int>&);
   void setTexCoordId(const vector<int>&);
   void setCreaseEdgeId(const vector<int>&);
   void setCornerVertId(const vector<int>&);
   void setCreaseVertId(const vector<int>&);
   void setDartVertId(const vector<int>&);
   const vector<coord3D>* ptexCoordNode;
   const vector<coord3D>* pcoordNode;
   const vector<coord3D>* pcoordNodeAux;
   int subdivisionType;
   int subdivisionSubType;
   int subdivisionLevel;
   coord3D* normalize(coord3D*);
   coord3D vecProd(coord3D*, coord3D*);
   coord3D calcNormal(coord3D*, coord3D*, coord3D*);
   coord3D calcNormal2(coord3D*, coord3D*, coord3D*);
   float calcK(coord3D*);
   bool areEqual(coord3D*, coord3D*);
private:
   bool _ccw;
   vector<int> _colorId;
   bool _colorPerVert;
   bool _convex;
   //vector<int> _coordId;
   //vector<int> _coordIdAux;
   bool _solid;
   vector<int> _texCoordId;
   vector<int> _creaseEdgeId;
   vector<int> _dartVertId;
   vector<int> _creaseVertId;
   vector<int> _cornerVertId;
   // # THIS IMPLEMENTATION STUFF
public:
   // destruction & construction
   SSNode();
   // action to do in case of change
   void onCoordChange();
   void onTexCoordChange();
   // final data access
   const vector<coord3D>* pgetCoord() const;
   const vector<int>* pgetCoordId() const;
   const coord3D* pgetBBox() const;
   void coordChangeFunc(void(*)());
   void coordIdChangeFunc(void(*)());
   bool _localCoord;
protected:
   vector<int> _coordId;
   vector<int> _coordIdAux;
   void _psetWavCoef(const vector<float>*);
   void _psetLift(const bool);
   void _updateCoord();
   
private:
   // # internal stuff
   // what is set?
   bool _isCoordIdSet, _isStencilBuilt;
   // wavelet configuration
   bool _lift;
   // current geometry
   vector<coord3D> _currCoord;
   vector<coord3D> _tmpCoord;
   // current texture
   vector<int> _currTexCoordId;
   const vector<int>* _pcurrTexCoordId;
   vector<coord2D> _currTexCoord;
   coord3D _pboundBox[2];
   void _updateTex(const unsigned);
   const vector<float>* _pwavCoeff;
   void (*_pcoordCallBack)();
   void (*_pcoordIdCallBack)();
   
#ifdef _TIME_SSLIB_
   clock_t _startTime, _endTime;
   clock_t _refineTime;
#endif
};
#endif//ndef _SUBDIVISION_SURFACE_NODE_INCLUDED_