/***********************************************************************
This software module was originally developed by Yannick Maret (from the
Ecole Polytechnique Federale de Lausanne, EPFL, during a stay at the
Universidad Politecnica de Madrid, UPM) in the course of development of 
the MPEG-4 Systems (ISO/IEC 14496-1) standard. This software module is
an implementation of a part of one or more MPEG-4 Systems 
(ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems
(ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 Systems 
(ISO/IEC 14496-1) free license to this software module or modifications
thereof for use in hardware or software products claiming conformance to
the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use this 
software module in hardware or software products are advised that its 
use may infringe existing patents. The original developer of this 
software module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software module
or modifications thereof in an implementation. Copyright is not 
released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
EPFL and UPM retain full right to use the code for his/her own purpose,
assign or donate the code to a third party and to inhibit third parties 
from using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
products. This copyright notice must be included in all copies or 
derivative works. Copyright  2003-2005
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yannick Maret at Yannick.Maret@a3.epfl.ch or Francisco
Moran at fmb@gti.ssr.upm.es.
----------------------------------------------------------------------*/

/***********************************************************************
Revision 1.1: 2005/9/15

Alberto Lopez-Ceron Pinilla has modified the file to support different
number of bitplanes in each coordinate and to read the bit _isInBand. 
***********************************************************************/

/*-------------------------------------------------------------------------
Project Player3d
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  WaveSurfDecoder
FILE:       WSSNode.cpp
AUTHOR:     Yannick Maret (EPFL, UPM)
OVERVIEW:   Implementation of the WaveletSubdivisionSurface node.
-------------------------------------------------------------------------*/

#include "WSSNode.h"
// *** INCLUDES ***
#include <math.h>
#include <iostream>
#include <string.h>
#include <stdlib.h>
#if _SS_WSS_LIM_BANDWITH
   #include <time.h>
#endif
// *** CONSTRUCTOR ***
WSSNode::WSSNode():
frequency(1),
fieldOfView((float)M_PI/4.F),
quality(1),
_wssInit(false),
_bpCoordUpdate(false)
{
}
// *** CONFIGURATION
void WSSNode::bpCoordUpdate(const bool r)
{
    _bpCoordUpdate = r;
}
unsigned WSSNode::getMaxnbp()
{
  return maxnbp;
}
// *** MESH DECODER ***
long unsigned WSSNode::initMeshDecoder(char *rpbs)
{
   // *** STACK VARIABLE (removed by fmb as it's not used -- any longer?)
   // unsigned c;
   // *** SET STREAM
   _bsProxy.psetStream(rpbs);

   packet_ended = true;
   return _bsProxy.getNbitRead();
}
long unsigned WSSNode::bpMeshDecoder()
{
   // *** STACK VARIABLE
   unsigned c;
      
   // *** INITIALISATION
   if (packet_ended)
   {
      // *** READ THE HEADER
      _pckHeaderDecoder();
      packet_ended = false;
      j = _fbp;
      if (_isPartial)
         // # PARTIAL TRANSMISSION PREPARATION
         for (c = X ; c <= Z ; c++)
            if (_component[c])
               if (_partialIdx[c][_treeIdx] == -1)
               {
                  // # FIRST TIME THIS TREE APPEARS 'ALONE'
                  _partialSpiht[c].resize(_partialSpiht[c].size() + 1);
                  _forestSpiht[c].moveTreeTo(_treeIdx, &_partialSpiht[c].back());
                  _partialIdx[c][_treeIdx] = _partialSpiht[c].size() - 1;       
               }
   }
    
   if (j <= _lbp)
   {
      if (_isPartial)
      {
         _partPckDecoder(j);
         for (c = X ; c <= Z ; c++)
           if (j < _nbBp[c]) //this "if" was added by Alberto Lpez-Cern Pinilla to support different number of bitplanes in each coordinate
           {
             if (_component[c])
               _partialSpiht[c][_partialIdx[c][_treeIdx]].decodeNextBp();
             if (_bpCoordUpdate)
               onCoordChange();
           }
      }
      else 
      {
         // # 'ALL TREES' TRANSMISSION
         _forestPckDecoder(j);
         for (c = X ; c <= Z ; c++)
           if (j < _nbBp[c]) //this "if" was added by Alberto Lpez-Cern Pinilla to support different number of bitplanes in each coordinate
           {
             if (_component[c])
               _forestSpiht[c].decodeNextBp();
             if (_bpCoordUpdate)
               onCoordChange();
           }
      }
      j++;
   }
   else {
      packet_ended = true;
   }

   if (!_bpCoordUpdate)
      onCoordChange();
   return _bsProxy.getNbitRead();
}

long unsigned WSSNode::decodeAllBitplanes()
{
   // *** STACK VARIABLE
   unsigned c;
      
   // *** INITIALISATION
   if (packet_ended)
   {
      // *** READ THE HEADER
      _pckHeaderDecoder();
      packet_ended = false;
      j = _fbp;
      if (_isPartial)
         // # PARTIAL TRANSMISSION PREPARATION
         for (c = X ; c <= Z ; c++)
            if (_component[c])
               if (_partialIdx[c][_treeIdx] == -1)
               {
                  // # FIRST TIME THIS TREE APPEARS 'ALONE'
                  _partialSpiht[c].resize(_partialSpiht[c].size() + 1);
                  _forestSpiht[c].moveTreeTo(_treeIdx, &_partialSpiht[c].back());
                  _partialIdx[c][_treeIdx] = _partialSpiht[c].size() - 1;       
               }
   }
    
   while (j <= _lbp)
   {
      if (_isPartial)
      {
         _partPckDecoder(j);
         for (c = X ; c <= Z ; c++)
           if (j < _nbBp[c]) //this "if" was added by Alberto Lpez-Cern Pinilla to support different number of bitplanes in each coordinate
           {
             if (_component[c])
               _partialSpiht[c][_partialIdx[c][_treeIdx]].decodeNextBp();
             if (_bpCoordUpdate)
               onCoordChange();
           }
      }
      else 
      {
         // # 'ALL TREES' TRANSMISSION
         _forestPckDecoder(j);
         for (c = X ; c <= Z ; c++)
           if (j < _nbBp[c]) //this "if" was added by Alberto Lpez-Cern Pinilla to support different number of bitplanes in each coordinate
           {
             if (_component[c])
               _forestSpiht[c].decodeNextBp();
             if (_bpCoordUpdate)
               onCoordChange();
           }
      }
      j++;
   }

   packet_ended = true;
   if (!_bpCoordUpdate)
      onCoordChange();

   return _bsProxy.getNbitRead();
}

// *** CONFIGURATION DECODER ***
long unsigned WSSNode::configDecoder(char *rpbs)
{
   _bsProxy.psetStream(rpbs);
   // *** EXTRACT CONFIGURATION ***
   if (_bsProxy.getBit())
      _nbitSc = (unsigned)_bsProxy.getNbit(5);
   else
      _nbitSc = 0;
   _nbBp[X] = (unsigned)_bsProxy.getNbit(5);
   _nbBp[Y] = (unsigned)_bsProxy.getNbit(5);
   _nbBp[Z] = (unsigned)_bsProxy.getNbit(5);

   maxnbp = _nbBp[X];
   for(unsigned c = Y; c <= Z; c++)
     if (_nbBp[c] > maxnbp) maxnbp = _nbBp[c];
   unsigned w_type = (unsigned)_bsProxy.getNbit(2);
   switch (w_type)
   {
   case 0:
      _ssType = MID_POINT;
      break;
   case 1:
      _ssType = LOOP;
      break;
   case 2:
      _ssType = BUTTERFLY;
   }
   _lift = _bsProxy.getBit();
   _nbLvl = (unsigned)_bsProxy.getNbit(4);
   _max[X] = _bsProxy.getFloat();
   _max[Y] = _bsProxy.getFloat();
   _max[Z] = _bsProxy.getFloat();
#if _SS_DBG_VERBOSE_BS_READER
   cout << "_SSsBS, decr configuration:" << endl;
   cout << "------------------------------" << endl;
   cout << "_nbitSc  = " << _nbitSc << endl;
   cout << "_nbBp[X] = " << _nbBp[X] << endl;
   cout << "_nbBp[Y] = " << _nbBp[Y] << endl;
   cout << "_nbBp[Z] = " << _nbBp[Z] << endl;
   cout << "_lift    = " << _lift << endl;
   cout << "_nbLvl   = " << _nbLvl << endl;
   cout << "_max[X]  = " << _max[X] << endl;
   cout << "_max[Y]  = " << _max[Y] << endl;
   cout << "_max[Z]  = " << _max[Z] << endl;
#endif
   return _bsProxy.getNbitRead();
}
// *** PACKET HEADER DECODER ***
void WSSNode::_pckHeaderDecoder()
{
   unsigned c, base_layer;
   bool isInBand;
   isInBand = _bsProxy.getBit(); //added by Alberto Lpez-Cern Pinilla
   if(isInBand)		//If the decoder info is in band it will overwrite the other info
   {
		if (_bsProxy.getBit())
			_nbitSc = (unsigned)_bsProxy.getNbit(5);
		else
			_nbitSc = 0;
		_nbBp[X] = (unsigned)_bsProxy.getNbit(5);
		_nbBp[Y] = (unsigned)_bsProxy.getNbit(5);
		_nbBp[Z] = (unsigned)_bsProxy.getNbit(5);

		maxnbp = _nbBp[X];
		for(unsigned c = Y; c <= Z; c++)
			if (_nbBp[c] > maxnbp) maxnbp = _nbBp[c];
		unsigned w_type = (unsigned)_bsProxy.getNbit(2);
		switch (w_type)
		{
		case 0:
			_ssType = MID_POINT;
			break;
		case 1:
			_ssType = LOOP;
			break;
		case 2:
			_ssType = BUTTERFLY;
		}
		_lift = _bsProxy.getBit();
		_nbLvl = (unsigned)_bsProxy.getNbit(4);
		_max[X] = _bsProxy.getFloat();
		_max[Y] = _bsProxy.getFloat();
		_max[Z] = _bsProxy.getFloat();
   }
   base_layer = _bsProxy.getBit();
   _localCoord = _bsProxy.getBit(); 
   if (base_layer && _nbitSc != 0)
      _scaleCoeffDecoder();
   _nbitBpLength = _bsProxy.getNbit(5);
   _fbp = _bsProxy.getNbit(5);
   _lbp = _bsProxy.getNbit(5);
   for (c = X ; c <= Z ; c++)
      _component[c] = _bsProxy.getBit();
   if (_isPartial = _bsProxy.getBit())
      _treeIdx = _bsProxy.getNbit(16);



   // *** SET SSMESH INFO
   subdivisionLevel = _nbLvl;
   setCoordId(*(pbaseMesh->pgetCoordId()));
   // *** CREATE B-TABLES
   _reserveTables();
   // *** CREATE QUANTISATION, OFFSPRINGS AND INITIAL SPIHT DECODERS
   _forest.buildChildTrees((_SSsTopo*)this);
   for (c = X; c <= Z ;c++)
      if (_nbBp[c] == 0)
         _wc[c].resize(pgetNbVertLUT()->back(), 0.F);
      else
      {
         _quant[c].buildQuantTables(_max[c], _nbBp[c]);
         _wc[c].resize(pgetNbVertLUT()->back(), 0.F);
         _forestSpiht[c].configure(_b[c].begin(), &_forest, _wc[c].begin(), &_quant[c], _nbBp[c], subdivisionLevel);
         _partialIdx[c].resize(getNbEdgeAtLvl(0), -1);
         _forestSpiht[c].initForest();
      }
   // *** REGISTER WAV COEFF FOR GEOMETRY UPDATE
   pcoordNode = pbaseMesh->pgetCoord();
   subdivisionType = _ssType;
   subdivisionSubType = 0;
   _psetLift(_lift);
   _psetWavCoef(_wc);
}
// *** SCALE COEFFICIENT DECODER ***
void WSSNode::_scaleCoeffDecoder()
{
   const unsigned nb_base_vert = getNbVertAtLvl(0);
   _scaleCoeff.resize(nb_base_vert);
   for (unsigned i = 0 ; i < nb_base_vert ; i++)
      _scaleCoeff[i] = _bsProxy.getNbit(_nbitSc);
}
// *** FULL TRANSMITION BP DECODER ***
void WSSNode::_forestPckDecoder(const unsigned rbp)
{
   // *** STACK VARIABLES
   const unsigned nb_base_edges = getNbEdgeAtLvl(0);
   unsigned c, k, bp_length, i;
   // *** READ A BIT-PLANE OF THE WHOLE FOREST 
   for (i = 0 ; i < nb_base_edges ; i++)
      for (c = X ; c <= Z ;c++)
         if (_component[c])
         {
            bp_length = _bsProxy.getNbit(_nbitBpLength);
            if (bp_length)
               _b[c][i][rbp].resize(bp_length);
            for (k = 0 ; k < bp_length ; k++)
               _b[c][i][rbp][k] = _bsProxy.getBit();
         }
#if _SS_DBG_VERBOSE_BS_READER
   cout << "_SSsBS, Forest B-tables:" << endl;
   cout << "------------------------" << endl;
   for (c = X ; c <= Z ; c++)
      if (_component[c])
         for (i = 0 ; i < nb_base_edges ; i++)
         {
            cout << "b_" << c << '[' << i << "][" << rbp << "] = ";
            if (_b[c][i][rbp].size() < 60)
               for (k = 0 ; k < _b[c][i][rbp].size() ; k++)
                  cout << (unsigned)_b[c][i][rbp][k];
            else
            {
               for (k = 0 ; k < 60 ; k++)
                  cout << (unsigned)_b[c][i][rbp][k];
               cout << "...";
            }
            cout << endl;
          }
#endif
}
// *** PARTIAL BP TRANSMITION ***
void WSSNode::_partPckDecoder(const unsigned rbp)
{
   // *** STACK VARIABLES
   unsigned c, k, bp_length;
   // *** READ A BIT-PLANE OF A SINGLE TREE
   
   for (c = X ; c <= Z ;c++)
      if (_component[c])
      {
          bp_length = _bsProxy.getNbit(_nbitBpLength);
         if (bp_length)
            _b[c][_treeIdx][rbp].resize(bp_length);
         for (k = 0 ; k < bp_length ; k++)
            _b[c][_treeIdx][rbp][k] = _bsProxy.getBit();
      }
#if _SS_DBG_VERBOSE_BS_READER
   cout << "_SSsBS, partial B-tables:" << endl;
   cout << "-------------------------" << endl;
   for (c = X ; c <= Z ; c++)
      if (_component[c])
      {
         cout << "b_" << c << '[' << _treeIdx << "][" << rbp << "] = ";
      if (_b[c][_treeIdx][rbp].size() < 60)
         for (k = 0 ; k < _b[c][_treeIdx][rbp].size() ; k++)
            cout << (unsigned)_b[c][_treeIdx][rbp][k];
      else
         for (k = 0 ; k < 60 ; k++)
            cout << (unsigned)_b[c][_treeIdx][rbp][k];
      cout << endl;
      }
#endif
}
// *** B-TABLE MEMORY RESERVATION
void WSSNode::_reserveTables()
{
   const unsigned nb_base_edges = getNbEdgeAtLvl(0);
   // *** CREATE THE B-TABLES ***
   _b[X].resize(nb_base_edges);
   _b[Y].resize(nb_base_edges);
   _b[Z].resize(nb_base_edges);
   for (unsigned i = 0 ; i < nb_base_edges ; i++)
   {
      _b[X][i].resize(_nbBp[X]);
      _b[Y][i].resize(_nbBp[Y]);
      _b[Z][i].resize(_nbBp[Z]);
   }
}
// *** DETAILS COEFF VISUALISATION ***
#if _SS_DBG_VERBOSE_WAV_COEFF
void WSSNode::_displayWavCoeff()
{
   vector<unsigned> ident;
   unsigned idx, first_idx;
   first_idx = 0;
   while (_forest.getTreeIdx(first_idx) == -1)
      first_idx++;
   for (unsigned i = 0 ; i < _forest.getNbTrees() ; i++)
   {
      idx = first_idx;
      while (_forest.getTreeIdx(idx) != i)
         idx++;
      ident.clear();
      ident.push_back(i);
      _displayIdent(&ident, idx);
      cout << '(' << _wc[X][idx] << ',' << _wc[Y][idx] << ',' << _wc[Z][idx] << ')' << endl;
      _displayWavCoeffRec(idx, &ident, 2);
   }
}
#define MAX_NB_RECURSION 3
void WSSNode::_displayWavCoeffRec(const unsigned r, vector<unsigned>* rp, const unsigned rrecLvl)
{
   unsigned idx;
   for (unsigned i = 0 ; i < _forest.pgetChilds(r)->size() ; i++)
   {
      rp->push_back(i);
      idx = (*_forest.pgetChilds(r))[i];
      _displayIdent(rp, idx);
      cout << '(' << _wc[X][idx] << ',' << _wc[Y][idx] << ',' << _wc[Z][idx] << ')' << endl;
      if (!_forest.areChildsLeaves(r) && rrecLvl < MAX_NB_RECURSION)
         _displayWavCoeffRec((*_forest.pgetChilds(r))[i], rp, rrecLvl + 1);
      rp->resize(rp->size() - 1);
   }
}
void WSSNode::_displayIdent(const vector<unsigned>* rp, const unsigned ridx)
{
   unsigned i;
   cout << 'E' << (*rp)[0];
   vector<unsigned>::const_iterator i_ident = rp->begin() + 1;
   for (i = 1 ; i < rp->size() ; i++, i_ident++)
      cout << '.' << *i_ident;
   for (i = (MAX_NB_RECURSION - rp->size())*2 ; i > 0 ; i--)
      cout << ' ';
   cout << '(' << setw(3) << ridx << ") = ";
}
#endif