/***********************************************************************
This software module was originally developed by Yannick Maret (from the
Ecole Polytechnique Federale de Lausanne, EPFL, during a stay at the
Universidad Politecnica de Madrid, UPM) in the course of development of 
the MPEG-4 Systems (ISO/IEC 14496-1) standard. This software module is
an implementation of a part of one or more MPEG-4 Systems 
(ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems
(ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 Systems 
(ISO/IEC 14496-1) free license to this software module or modifications
thereof for use in hardware or software products claiming conformance to
the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use this 
software module in hardware or software products are advised that its 
use may infringe existing patents. The original developer of this 
software module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software module
or modifications thereof in an implementation. Copyright is not 
released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
EPFL and UPM retain full right to use the code for his/her own purpose,
assign or donate the code to a third party and to inhibit third parties 
from using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
products. This copyright notice must be included in all copies or 
derivative works. Copyright  2003-2005
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yannick Maret at Yannick.Maret@a3.epfl.ch or Francisco
Moran at fmb@gti.ssr.upm.es.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Player3d
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  WaveSurfDecoder
FILE:       WSSNode.h
AUTHOR:     Yannick Maret (EPFL, UPM)
OVERVIEW:   Interface of the WaveletSubdivisionSurface node.
-------------------------------------------------------------------------*/

#ifndef _WAVELET_SUBDIVISION_SURFACE_NODE_INCLUDED_
#define _WAVELET_SUBDIVISION_SURFACE_NODE_INCLUDED_
// *** INCLUDES ***
#include "SSNode.h"
#include "_SSsSpiht.h"
#include "_SSsBs.h"
#include "_SSsForest.h"
#include "_SSsQuant.h"
// *** WAVELET SSs NODE ***
class WSSNode: public SSNode
{
// *** WAVELET SSs INTERFACE
public:
   // constructor
   WSSNode();
   const SSNode* pbaseMesh;
   float frequency;
   float fieldOfView;
   int quality;
// *** BIT STREAM INTERFACE
   long unsigned configDecoder(char*);
   long unsigned initMeshDecoder(char*);
   long unsigned bpMeshDecoder();
   long unsigned decodeAllBitplanes();
// *** CONFIGURATION
   void bpCoordUpdate(const bool);
   unsigned getMaxnbp();
   // decoder configuration
   float _max[3];
   unsigned _nbBp[3], _nbitSc, _nbLvl;
   bool _lift;
   enum etSSsType _ssType;
   bool _component[3], _isPartial;
   unsigned _treeIdx, _fbp, _lbp, _nbitBpLength;
 
// *** INTERNAL STUFF
private:
#if _SS_DBG_VERBOSE_WAV_COEFF
   void _displayWavCoeff();
   void _displayIdent(const vector<unsigned>*, const unsigned);
   void _displayWavCoeffRec(const unsigned, vector<unsigned>*, const unsigned);
#endif
   // # CONFIGURATION
   bool _bpCoordUpdate;
   // # FOR SPiHT
   vector<float> _wc[3];
   _SSsQuant _quant[3];
   _SSsForest _forest;
   vector<_SSsSpiht> _partialSpiht[3];
   _SSsSpiht _forestSpiht[3];
   vector<unsigned> _partialIdx[3];
   // # BIT-STREAM READER
   // function
   void _reserveTables();
   void _pckHeaderDecoder();
   void _scaleCoeffDecoder();
   void _forestPckDecoder(const unsigned);
   void _partPckDecoder(const unsigned);
   // bitstream proxy
   _SSsBs _bsProxy;
   bool _wssInit;
   unsigned j;
   bool packet_ended;
   
   // b-tables
   vector<vector<vector<char> > > _b[3];
   // scale coeff
   vector<unsigned long> _scaleCoeff;
   bool _hasScaleCoeff;

   unsigned maxnbp;
   
};
#endif//ndef _WAVELET_SUBDIVISION_SURFACE_NODE_INCLUDED_
