/***********************************************************************
This software module was originally developed by Yannick Maret (from the
Ecole Polytechnique Federale de Lausanne, EPFL, during a stay at the
Universidad Politecnica de Madrid, UPM) in the course of development of 
the MPEG-4 Systems (ISO/IEC 14496-1) standard. This software module is
an implementation of a part of one or more MPEG-4 Systems 
(ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems
(ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 Systems 
(ISO/IEC 14496-1) free license to this software module or modifications
thereof for use in hardware or software products claiming conformance to
the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use this 
software module in hardware or software products are advised that its 
use may infringe existing patents. The original developer of this 
software module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software module
or modifications thereof in an implementation. Copyright is not 
released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
EPFL and UPM retain full right to use the code for his/her own purpose,
assign or donate the code to a third party and to inhibit third parties 
from using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
products. This copyright notice must be included in all copies or 
derivative works. Copyright  2003-2005
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yannick Maret at Yannick.Maret@a3.epfl.ch or Francisco
Moran at fmb@gti.ssr.upm.es.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Player3d
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  WaveSurfDecoder
FILE:       _SSsBs.cpp
AUTHOR:     Yannick Maret (EPFL, UPM)
OVERVIEW:   Implementation of bitstream reader/writer.
-------------------------------------------------------------------------*/

// *** INCLUDES ***
#include "_SSsBs.h"
// *** BITSTREAM READING ***
void _SSsBs::psetStream(const char* rp, const unsigned rbitMask)
{
   _nbitRead = 0;
   _bitMask = rbitMask;
   _pibitStream = rp;
}
long unsigned _SSsBs::getNbit(const unsigned r)
{
   // *** STACK VARIABLES
   long unsigned a_unsigned = 0;
   // *** r EXTRACTION
   for (unsigned i = 0 ; i < r ; i++)
   {
      a_unsigned <<= 1;
      a_unsigned += getBit();
   }
   return a_unsigned;
}
long unsigned _SSsBs::getNbitRead()
{
   return _nbitRead;
}
float _SSsBs::getFloat()
{
   // *** STACK VARIABLES
   long unsigned* p_luns;
   float a_float;
   // *** FLOAT EXTRACTION
   // !!                     !!
   // !! processor dependent !! 
   // !!                     !!
   p_luns = (long unsigned*) &a_float;
   (*p_luns) = getNbit(32);
   return a_float;
}
// *** BITSTREAM WRITING ***
void _SSsBs::setNbit(const unsigned rN, long unsigned r)
{
   // *** STACK VARIABLES
   long unsigned mask = 1 << (rN - 1);
   // *** r EXTRACTION
   for (unsigned i = 0 ; i < rN ; i++)
   {
      setBit((mask & r) != 0);
      mask >>= 1;
   }
}
void _SSsBs::setFloat(const float r)
{
   // *** STACK VARIABLES ***
   long unsigned luns;
   luns = *((long unsigned*) &r);
   setNbit(32, luns);
}
const vector<char>* _SSsBs::pgetStream()
{
   return &_obitStream;
}
void _SSsBs::clearStream()
{
   _obitStream.clear();
   _aByte = 0x00;
   _nbitWrite = 0;
   _bitMask = 0x80;
}
long unsigned _SSsBs::getNbitWritten()
{
   return _nbitWrite;
}
