/***********************************************************************
This software module was originally developed by Yannick Maret (from the
Ecole Polytechnique Federale de Lausanne, EPFL, during a stay at the
Universidad Politecnica de Madrid, UPM) in the course of development of 
the MPEG-4 Systems (ISO/IEC 14496-1) standard. This software module is
an implementation of a part of one or more MPEG-4 Systems 
(ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems
(ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 Systems 
(ISO/IEC 14496-1) free license to this software module or modifications
thereof for use in hardware or software products claiming conformance to
the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use this 
software module in hardware or software products are advised that its 
use may infringe existing patents. The original developer of this 
software module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software module
or modifications thereof in an implementation. Copyright is not 
released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
EPFL and UPM retain full right to use the code for his/her own purpose,
assign or donate the code to a third party and to inhibit third parties 
from using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
products. This copyright notice must be included in all copies or 
derivative works. Copyright  2003-2005
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yannick Maret at Yannick.Maret@a3.epfl.ch or Francisco
Moran at fmb@gti.ssr.upm.es.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Player3d
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  WaveSurfDecoder
FILE:       _SSsBs.h
AUTHOR:     Yannick Maret (EPFL, UPM)
OVERVIEW:   Interface of the bitstream reader/writer.
-------------------------------------------------------------------------*/

#ifndef _WAVELET_SUBDIVISION_SURFACE_BITSTREAM_INCLUDED_
#define _WAVELET_SUBDIVISION_SURFACE_BITSTREAM_INCLUDED_
// *** INCLUDES ***
#include "_SSsCommon.h"
// *** BITSTREAM CLASS ***
class _SSsBs
{
public:
   // *** reading 
   void psetStream(const char*, const unsigned = 0x80);
   bool getBit();  
   long unsigned getNbit(const unsigned);
   float getFloat();
   long unsigned getNbitRead();
   // *** writing
   void setBit(const bool);
   void setNbit(const unsigned, const long unsigned);
   void setFloat(const float);
   const vector<char>* pgetStream();
   void clearStream();
   long unsigned getNbitWritten();
private:
   // bistream reading
   const char* _pibitStream;
   unsigned char _bitMask;
   long unsigned _nbitRead;
   // bitstream creation
   vector<char> _obitStream;
   char _aByte;
   long unsigned _nbitWrite;
};
// *** INLINED FUNCTIONS DEFINITION
inline bool _SSsBs::getBit()
{
   bool a_bit;
   a_bit = ((*_pibitStream) & _bitMask) != 0;
   if (_bitMask == 0x01)
   {
      _bitMask = 0x80;
      _pibitStream++;
   }
   else
      _bitMask = _bitMask >> 1;
   _nbitRead++;
   return a_bit;
}
inline void _SSsBs::setBit(const bool r)
{
   if (r)
      _aByte |= _bitMask;
   if (_bitMask == 0x01)
   {
      _bitMask = 0x80;
      _obitStream.push_back(_aByte);
      _aByte = 0x00;
   }
   else
      _bitMask = _bitMask >> 1;
   _nbitWrite++;
}
#endif//ndef _WAVELET_SUBDIVISION_SURFACE_BITSTREAM_INCLUDED_