/***********************************************************************
This software module was originally developed by Yannick Maret (from the
Ecole Polytechnique Federale de Lausanne, EPFL, during a stay at the
Universidad Politecnica de Madrid, UPM) in the course of development of 
the MPEG-4 Systems (ISO/IEC 14496-1) standard. This software module is
an implementation of a part of one or more MPEG-4 Systems 
(ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems
(ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 Systems 
(ISO/IEC 14496-1) free license to this software module or modifications
thereof for use in hardware or software products claiming conformance to
the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use this 
software module in hardware or software products are advised that its 
use may infringe existing patents. The original developer of this 
software module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software module
or modifications thereof in an implementation. Copyright is not 
released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
EPFL and UPM retain full right to use the code for his/her own purpose,
assign or donate the code to a third party and to inhibit third parties 
from using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
products. This copyright notice must be included in all copies or 
derivative works. Copyright  2003-2005
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yannick Maret at Yannick.Maret@a3.epfl.ch or Francisco
Moran at fmb@gti.ssr.upm.es.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Player3d
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  WaveSurfDecoder
FILE:       _SSsCommon.h
AUTHOR:     Yannick Maret (EPFL, UPM)
OVERVIEW:   Subdvision surfaces common stuff file.
-------------------------------------------------------------------------*/

#ifndef _SSLIB_SUBDIVISION_SURFACE_COMMON_INCLUDED_
#define _SSLIB_SUBDIVISION_SURFACE_COMMON_INCLUDED_
// *** INCLUDE CONFIGURATION FILE ***
#include "_SSsConfig.h"
// *** ERROR & WARNING ***
class _SSErr{};
class BuildHullStage : public _SSErr{};
   class CoordIdEmpty : public BuildHullStage{}; // ERROR
   class BadCoordId : public BuildHullStage{}; // ERROR
   class BadCreaseEdgeId : public BuildHullStage{}; // ERROR
   class BadCreaseId : public BuildHullStage{}; // ERROR
   class BadCornerId : public BuildHullStage{}; // ERROR
   class BadDartId : public BuildHullStage{}; // ERROR
   class CreaseEdgeIndexOverrided : public BuildHullStage{}; // WARNING
class SubdivisionStageErr : public _SSErr{};
   class InBadSector : SubdivisionStageErr{}; // ERROR
   class SSsSchemeUnknow : SubdivisionStageErr{}; // ERROR
class RingBuildStageErr: public _SSErr{};
   class NonManifoldVertex : public RingBuildStageErr{}; // ERROR
   class DanglingEdge : public RingBuildStageErr {}; // ERROR
   class OrphanVertex : public RingBuildStageErr{}; // ERROR
// *** STL ***
#include <vector>
#include <math.h>
#ifdef _MSC_VER
   #if _MSC_VER < 1300 // i.e. before VC7
      // disable the warning about 'debuging name being longer than 255 char'
      #pragma warning(disable : 4786)
      // this code doesn't work with MS STL implementation prior to .NET
      #ifndef _STLP_VECTOR
         #error WITH THIS VERSION OF VISUAL STUDIO YOU NEED STL PORT (www.stlport.org)
      #else
         #ifdef WIN32
            #undef max
            #undef min
         #endif
         using namespace _STL;
     #endif
   #endif
#endif
#include <algorithm>
#include <deque>
#include <list>
#ifdef _TIME_SSLIB_
   #include <time.h>
#endif
#ifdef _SS_NEED_IO
   #include <iostream>
   #include <iomanip>
#endif
// *** GLOBAL CONSTANT ***
// make sure PI is defined
#ifndef M_PI
   const double M_PI = 3.14159265358979323846;
#endif
// custom type
const unsigned X = 0;
const unsigned Y = 1;
const unsigned Z = 2;
// *** COORD ***
// 2D
class coord2D
{
public:
   float x;
   float y;
   coord2D(){x = y = 0.F;};
   coord2D operator+(const coord2D&) const;   // vertex + vertex
   coord2D operator-(const coord2D&) const;   // vertex - vertex
   coord2D operator-() const;                 // - vertex
   coord2D operator*(const coord2D&) const;   // cross product
   coord2D operator*(const float) const;      // vertex*m
   coord2D operator/(const float) const;      // vertex/m
   coord2D operator+=(const coord2D&);        // v1 = v1 + v2
   coord2D operator-=(const coord2D&);        // v1 = v1 - v2
   coord2D operator*=(const float);           // vertex*m
   coord2D operator/=(const float);           // vertex*m
   float dot(const coord2D&) const;
   float angle(const coord2D&) const;
   float length() const;
   void clear();
};
inline void coord2D::clear()
{
   x = y = 0.F;
}
inline coord2D coord2D::operator+(const coord2D& r) const
{
   coord2D ret;
   ret.x = this->x + r.x; ret.y = this->y + r.y;
   return ret;
}
inline coord2D coord2D::operator-(const coord2D& r) const
{
   coord2D ret;
   ret.x = this->x - r.x; ret.y = this->y - r.y;
   return ret;
}
inline coord2D coord2D::operator-() const
{
   coord2D ret;
   ret.x = -this->x; ret.y = -this->y;
   return ret;
}
inline coord2D coord2D::operator*(const coord2D& r) const
{
   coord2D ret;
   ret.x = this->x*r.x; ret.y = this->y*r.y;
   return ret;
}
inline coord2D coord2D::operator*(const float r) const
{
   coord2D ret;
   ret.x = this->x*r; ret.y = this->y*r;
   return ret;
}
inline coord2D coord2D::operator/(const float r) const
{
   coord2D ret;
   ret.x = this->x/r; ret.y = this->y/r;
   return ret;
}
inline coord2D coord2D::operator+=(const coord2D& r)
{
   x += r.x; y += r.y;  
   return *this;
}
inline coord2D coord2D::operator-=(const coord2D& r)
{
   x -= r.x; y -= r.y; 
   return *this;
}
inline coord2D coord2D::operator*=(const float r)
{
   x *= r; y *= r; 
   return *this;
}
inline coord2D coord2D::operator/=(const float r)
{
   x /= r; y /= r; 
   return *this;
}
inline float coord2D::dot(const coord2D& r) const
{
   return x*r.x + y*r.y;
}

inline float coord2D::angle(const coord2D& r) const
{
   return (float)acos(this->dot(r)/(length()*r.length()));
}
inline float coord2D::length() const
{
   return (float)sqrt(dot(*this));
}
/////
// 3D
/////
class coord3D
{
public:
   float x;
   float y;
   float z;
   coord3D(){x = y = z = 0.F;};
   coord3D operator+(const coord3D&) const;   // vertex + vertex
   coord3D operator-(const coord3D&) const;   // vertex - vertex
   coord3D operator-() const;                 // - vertex
   coord3D operator*(const coord3D&) const;   // cross product
   coord3D operator*(const float) const;      // vertex*m
   coord3D operator/(const float) const;      // vertex/m
   coord3D operator+=(const coord3D&);        // v1 = v1 + v2
   coord3D operator-=(const coord3D&);        // v1 = v1 - v2
   coord3D operator*=(const float);           // vertex*m
   coord3D operator/=(const float);           // vertex/m
   float dot(const coord3D&) const;
   float angle(const coord3D&) const;
   float length() const;
   void clear();
#ifdef _SS_NEED_IO
   void display();
#endif
};
#ifdef _SS_NEED_IO
   inline void coord3D::display()
   {
      cout << '(' << setw(10) << x << ',' << setw(10) << y << ',' << setw(10) << z << ')';
   }
#endif
inline void coord3D::clear()
{
   x = y = z = 0.F;
}
inline coord3D coord3D::operator+(const coord3D& r) const
{
   coord3D ret;
   ret.x = this->x + r.x; ret.y = this->y + r.y; ret.z = this->z + r.z;
   return ret;
}
inline coord3D coord3D::operator-(const coord3D& r) const
{
   coord3D ret;
   ret.x = this->x - r.x; ret.y = this->y - r.y; ret.z = this->z - r.z;
   return ret;
}
inline coord3D coord3D::operator-() const
{
   coord3D ret;
   ret.x = -this->x; ret.y = -this->y; ret.z = -this->z;
   return ret;
}
inline coord3D coord3D::operator*(const coord3D& r) const
{
   coord3D ret;
   ret.x = this->x*r.x; ret.y = this->y*r.y; ret.z = this->z*r.z;
   return ret;
}
inline coord3D coord3D::operator*(const float r) const
{
   coord3D ret;
   ret.x = this->x*r; ret.y = this->y*r; ret.z = this->z*r;
   return ret;
}
inline coord3D coord3D::operator/(const float r) const
{
   coord3D ret;
   ret.x = this->x/r; ret.y = this->y/r; ret.z = this->z/r;
   return ret;
}
inline coord3D coord3D::operator+=(const coord3D& r)
{
   x += r.x; y += r.y; z += r.z;   
   return *this;
}
inline coord3D coord3D::operator-=(const coord3D& r)
{
   x -= r.x; y -= r.y; z -= r.z;   
   return *this;
}
inline coord3D coord3D::operator*=(const float r)
{
   x *= r; y *= r; z *= r;   
   return *this;
}
inline coord3D coord3D::operator/=(const float r)
{
   x /= r; y /= r; z /= r;   
   return *this;
}
inline float coord3D::dot(const coord3D& r) const
{
   return x*r.x + y*r.y + z*r.z;
}
inline float coord3D::angle(const coord3D& r) const
{
   return (float)acos(this->dot(r)/(length()*r.length()));
}
inline float coord3D::length() const
{
   return (float)sqrt(dot(*this));
}
#endif //_SSLIB_SUBDIVISION_SURFACE_COMMON_INCLUDED_