/***********************************************************************
This software module was originally developed by Yannick Maret (from the
Ecole Polytechnique Federale de Lausanne, EPFL, during a stay at the
Universidad Politecnica de Madrid, UPM) in the course of development of 
the MPEG-4 Systems (ISO/IEC 14496-1) standard. This software module is
an implementation of a part of one or more MPEG-4 Systems 
(ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems
(ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 Systems 
(ISO/IEC 14496-1) free license to this software module or modifications
thereof for use in hardware or software products claiming conformance to
the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use this 
software module in hardware or software products are advised that its 
use may infringe existing patents. The original developer of this 
software module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software module
or modifications thereof in an implementation. Copyright is not 
released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
EPFL and UPM retain full right to use the code for his/her own purpose,
assign or donate the code to a third party and to inhibit third parties 
from using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
products. This copyright notice must be included in all copies or 
derivative works. Copyright  2003-2005
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yannick Maret at Yannick.Maret@a3.epfl.ch or Francisco
Moran at fmb@gti.ssr.upm.es.
----------------------------------------------------------------------*/

/***********************************************************************
Revision 1.1: 2005/9/15

Alberto Lopez-Ceron Pinilla has modified the file to correct the 
orientation of some edges.
***********************************************************************/

/*-------------------------------------------------------------------------
Project Player3d
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  WaveSurfDecoder
FILE:       _SSsForest.cpp
AUTHOR:     Yannick Maret (EPFL, UPM)
OVERVIEW:   Implementation of the forest class (offsprings tree for SPiHT).
-------------------------------------------------------------------------*/

// *** INCLUDES ***
#include "_SSsForest.h"
// *** TREE BUILDING FROM SSs TOPOLOGY ***
void _SSsForest::buildChildTrees(const _SSsTopo* rpBaseTopo)
{
   // *** STACK VARIABLES
   unsigned tree_idx = 0;
   vector<int>::const_iterator it_cid;
   const vector<unsigned>* p_num_vert = rpBaseTopo->pgetNbVertLUT();
   if (p_num_vert->size() == 1)
      return;
   vector<char> included(rpBaseTopo->getNbVertAtLvl(1), false);
   _nbTrees = rpBaseTopo->getNbEdgeAtLvl(0);
#if _SS_DBG_VERBOSE_FOREST
   cout << "Tree order:";
#endif
   _treesIdx.clear();
   _treesIdx.resize(p_num_vert->back(), -1);
   _invertTreesIdx.clear();
   _invertTreesIdx.resize(rpBaseTopo->getNbEdgeAtLvl(0), -1);
   for (it_cid = rpBaseTopo->pgetCoordIdAtLvl(1)->begin() + 1;
        it_cid < rpBaseTopo->pgetCoordIdAtLvl(1)->end() - 1;
        it_cid += 4)
      if (!included[*it_cid])
      {
         included[*it_cid] = true;
         // store the corresponding tree index
         _invertTreesIdx[tree_idx] = *it_cid;
         _treesIdx[*it_cid] = tree_idx++;
#if _SS_DBG_VERBOSE_FOREST
         cout << ' ' << *it_cid;
#endif
      }
#if _SS_DBG_VERBOSE_FOREST
   cout << endl;
#endif
   if (p_num_vert->size() <= 2)
      return;
   // *** GET THE TREES ORDER)
   // *** BUILD OFFSPRING TABLES
   // # STACK VARIABLES
   unsigned i, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12, v13, v14;
   vector<tOrientation> orient;
   const vector<vector<int> >* p_all_cid = rpBaseTopo->pgetCoordIdLUT();
   _childs.clear();
   _childs.resize(*(p_num_vert->end() - 2));
   orient.clear();
   orient.resize(p_num_vert->back());
   // # BUILD THE FIRST LEVEL
   // for detail on the algorithm, see the documentation:
   // "Implementation of MPEG-4 Subdivision Surfaces Tools"
   // from Yannick Maret (LTS-EPFL, GTI-UPM)
   for (it_cid = p_all_cid->at(2).begin() ; it_cid < p_all_cid->at(2).end() ; it_cid++)
   {
      it_cid += 12;
      v6 = *(it_cid++);
      v7 = *(it_cid++);
      v8 = *it_cid;
      it_cid += 14;
      v9 = *(it_cid++);
      v10 = *(it_cid++);
      v11 = *it_cid;
      it_cid += 14;
      v12 = *(it_cid++);
      v13 = *(it_cid++);
      v14 = *it_cid;
      it_cid += 2;
      v3 = *it_cid;
      it_cid += 4;
      v4 = *it_cid;
      it_cid += 4;
      v5 = *it_cid;
      it_cid += 7;
      if (_childs[v3].empty())
      {
         _childs[v3].reserve(4);
         _childs[v3].push_back(v6);
         _childs[v3].push_back(v11);
         _childs[v3].push_back(v13);
         orient[v3] = orient[v6] = orient[v11] = orient[v13] = PL;
         _treesIdx[v6] = _treesIdx[v11] = _treesIdx[v13] = _treesIdx[v3];
      } 
      else
      {
         _childs[v3].push_back(v13);
         orient[v13] = AR;
         _treesIdx[v13] = _treesIdx[v3];
      }
      if (_childs[v4].empty())
      {
         _childs[v4].reserve(4);
         _childs[v4].push_back(v9);
         _childs[v4].push_back(v14);
         _childs[v4].push_back(v7);
         orient[v4] = orient[v9] = orient[v14] = orient[v7] = PL;
         _treesIdx[v9] = _treesIdx[v14] = _treesIdx[v7] = _treesIdx[v4];
      }
      else
      {
         _childs[v4].push_back(v7);
         orient[v7] = AR;
         _treesIdx[v7] = _treesIdx[v4];
      }
      if (_childs[v5].empty())
      {
         _childs[v5].reserve(4);
         _childs[v5].push_back(v12);
         _childs[v5].push_back(v8);
         _childs[v5].push_back(v10);
         orient[v5] = orient[v12] = orient[v8] = orient[v10] = PL;
         _treesIdx[v12] = _treesIdx[v8] = _treesIdx[v10] = _treesIdx[v5];
      }
      else
      {
         _childs[v5].push_back(v10);
         orient[v10] = AR;
         _treesIdx[v10] = _treesIdx[v5];
      }
   }
   // # BUILD THE OTHER LEVEL
   for (i = 3 ; i < p_all_cid->size() ; i++)
      for (it_cid = p_all_cid->at(i).begin() ; it_cid < p_all_cid->at(i).end() ; it_cid++)
      {
         it_cid += 12;
         v6 = *(it_cid++);
         v7 = *(it_cid++);
         v8 = *it_cid;
         it_cid += 14;
         v9 = *(it_cid++);
         v10 = *(it_cid++);
         v11 = *it_cid;
         it_cid += 14;
         v12 = *(it_cid++);
         v13 = *(it_cid++);
         v14 = *it_cid;
         it_cid += 2;
         v3 = *it_cid;
         it_cid += 4;
         v4 = *it_cid;
         it_cid += 4;
         v5 = *it_cid;
         it_cid += 7;
         if (_childs[v3].empty())
         {
            _childs[v3].reserve(4);
            _childs[v3].push_back(v6);
            _childs[v3].push_back(v11);
            _childs[v3].push_back(v13);  

            orient[v6] = orient[v11] = orient[v13] = orient[v3];
            _treesIdx[v6] = _treesIdx[v11] = _treesIdx[v13] = _treesIdx[v3];
         } 
         else
         {

               _childs[v3].push_back(v13);
               orient[v13] = AR;

            _treesIdx[v13] = _treesIdx[v3];
         }
         if (_childs[v4].empty())
         {
             _childs[v4].reserve(4);
               _childs[v4].push_back(v9);
               _childs[v4].push_back(v14);
               _childs[v4].push_back(v7); 

            orient[v9] = orient[v14] = orient[v7] = orient[v4];
            _treesIdx[v9] = _treesIdx[v14] = _treesIdx[v7] = _treesIdx[v4];
         } 
         else
         {

               _childs[v4].push_back(v7);
               orient[v7] = AR;

            _treesIdx[v7] = _treesIdx[v4];
         }
         if (_childs[v5].empty())
         {
            _childs[v5].reserve(4);

               _childs[v5].push_back(v12);
               _childs[v5].push_back(v8);
               _childs[v5].push_back(v10);  

            orient[v12] = orient[v8] = orient[v10] = orient[v5];
            _treesIdx[v12] = _treesIdx[v8] = _treesIdx[v10] = _treesIdx[v5];
         } 
         else
         {

               _childs[v5].push_back(v10);
               orient[v10] = AR;
 
            _treesIdx[v10] = _treesIdx[v5];
         }
      }
   // # BUILD THE LEAVE BRANCHES LUT TABLE
   // i.e the edges which childs have no offsprings
   _leavesBranch.resize(_childs.size(), 0);
   for (i = 0 ; i < _childs.size() ; i++)
      if (!_childs[i].empty())
         if (_childs[i][0] >= _childs.size())
            _leavesBranch[i] = 1;
   // *** DEBUG VIEW 
#if _SS_DBG_VERBOSE_FOREST
   cout << "Offsprings Tree" << endl;
   cout << "---------------" << endl;
   for (i = 0 ; i < _childs.size() ; i++)
      if (_leavesBranch[i])
         cout << setw(2) << i << '*';
      else
         cout << setw(3) << i;
   cout << endl;
   for (unsigned j = 0 ; j < 4 ; j++)
   {
      for (i = 0 ; i < _childs.size() ; i++)
         if (_childs[i].size() > j)
            cout << setw(3) << _childs[i][j];
         else
            if (j == 0)
               cout << "  B";
            else
               cout << "   ";
      cout << endl;
   }
   cout << "Tree indices of offsprings" << endl;
   cout << "--------------------------" << endl;
   for (i = 0 ; i < _treesIdx.size() ; i++)
   cout << setw(3) << i;
   cout << endl;
   for (i = 0 ; i < _treesIdx.size() ; i++)
      if (_treesIdx[i] != -1)
         cout << setw(3) << _treesIdx[i];
      else
         cout << "  B";
   cout << endl;
#endif
}

void _SSsForest::_buildChildTrees1Lvl(const _SSsTopo* rpBaseTopo)
{
   // *** STACK VARIABLES
   unsigned tree_idx = 0;
   vector<int>::const_iterator it_cid;
   vector<char> included(rpBaseTopo->getNbVertAtLvl(1), false);
   const vector<unsigned>* p_num_vert = rpBaseTopo->pgetNbVertLUT();
   // *** GET THE TREES ORDER
   _nbTrees = rpBaseTopo->getNbEdgeAtLvl(0);
   _treesIdx.clear();
   _treesIdx.resize(p_num_vert->back(), -1);
   _invertTreesIdx.clear();
   _invertTreesIdx.resize(rpBaseTopo->getNbEdgeAtLvl(0), -1);
   for (it_cid = rpBaseTopo->pgetCoordIdAtLvl(1)->begin() + 1;
        it_cid < rpBaseTopo->pgetCoordIdAtLvl(1)->end() - 1;
        it_cid += 4)
      if (!included[*it_cid])
      {
         included[*it_cid] = true;
         // store the corresponding tree index
         _invertTreesIdx[tree_idx] = *it_cid;
         _treesIdx[*it_cid] = tree_idx++;
#if _SS_DBG_VERBOSE_FOREST
         cout << ' ' << *it_cid;
#endif
      }
#if _SS_DBG_VERBOSE_FOREST
   cout << endl;
#endif
   // *** BUILD OFFSPRING TABLES
   // # STACK VARIABLES
   unsigned i, v3, v4, v5, v6, v7, v8, v9, v10, v11, v12, v13, v14;
   vector<tOrientation> orient;
   const vector<vector<int> >* p_all_cid = rpBaseTopo->pgetCoordIdLUT();
   _childs.clear();
   _childs.resize(*(p_num_vert->end() - 2));
   orient.clear();
   orient.resize(p_num_vert->back());
   // # BUILD THE FIRST LEVEL
   // for detail on the algorithm, see the documentation:
   // "Implementation of MPEG-4 Subdivision Surfaces Tools"
   // from Yannick Maret (LTS-EPFL, GTI-UPM)
   for (it_cid = p_all_cid->at(2).begin() ; it_cid < p_all_cid->at(2).end() ; it_cid++)
   {
      it_cid += 12;
      v6 = *(it_cid++);
      v7 = *(it_cid++);
      v8 = *it_cid;
      it_cid += 14;
      v9 = *(it_cid++);
      v10 = *(it_cid++);
      v11 = *it_cid;
      it_cid += 14;
      v12 = *(it_cid++);
      v13 = *(it_cid++);
      v14 = *it_cid;
      it_cid += 2;
      v3 = *it_cid;
      it_cid += 4;
      v4 = *it_cid;
      it_cid += 4;
      v5 = *it_cid;
      it_cid += 7;
      if (_childs[v3].empty())
      {
         _childs[v3].reserve(4);
         _childs[v3].push_back(v6);
         _childs[v3].push_back(v11);
         _childs[v3].push_back(v13);
         orient[v3] = orient[v6] = orient[v11] = orient[v13] = PL;
         _treesIdx[v6] = _treesIdx[v11] = _treesIdx[v13] = _treesIdx[v3];
      } 
      else
      {
         _childs[v3].push_back(v13);
         orient[v13] = AR;
         _treesIdx[v13] = _treesIdx[v3];
      }
      if (_childs[v4].empty())
      {
         _childs[v4].reserve(4);
         _childs[v4].push_back(v9);
         _childs[v4].push_back(v14);
         _childs[v4].push_back(v7);
         orient[v4] = orient[v9] = orient[v14] = orient[v7] = PL;
         _treesIdx[v9] = _treesIdx[v14] = _treesIdx[v7] = _treesIdx[v4];
      }
      else
      {
         _childs[v4].push_back(v7);
         orient[v7] = AR;
         _treesIdx[v7] = _treesIdx[v4];
      }
      if (_childs[v5].empty())
      {
         _childs[v5].reserve(4);
         _childs[v5].push_back(v12);
         _childs[v5].push_back(v8);
         _childs[v5].push_back(v10);
         orient[v5] = orient[v12] = orient[v8] = orient[v10] = PL;
         _treesIdx[v12] = _treesIdx[v8] = _treesIdx[v10] = _treesIdx[v5];
      }
      else
      {
         _childs[v5].push_back(v10);
         orient[v10] = AR;
         _treesIdx[v10] = _treesIdx[v5];
      }
   }
   // # BUILD THE OTHER LEVEL
   for (i = 3 ; i < p_all_cid->size() ; i++)
      for (it_cid = p_all_cid->at(i).begin() ; it_cid < p_all_cid->at(i).end() ; it_cid++)
      {
         it_cid += 12;
         v6 = *(it_cid++);
         v7 = *(it_cid++);
         v8 = *it_cid;
         it_cid += 14;
         v9 = *(it_cid++);
         v10 = *(it_cid++);
         v11 = *it_cid;
         it_cid += 14;
         v12 = *(it_cid++);
         v13 = *(it_cid++);
         v14 = *it_cid;
         it_cid += 2;
         v3 = *it_cid;
         it_cid += 4;
         v4 = *it_cid;
         it_cid += 4;
         v5 = *it_cid;
         it_cid += 7;
         if (_childs[v3].empty())
         {
            _childs[v3].reserve(4);
            if (orient[v3] == PL)
            {
               _childs[v3].push_back(v6);
               _childs[v3].push_back(v11);
               _childs[v3].push_back(v13);  
            }
            else
            {
               _childs[v3].push_back(v11);
               _childs[v3].push_back(v6);
               // in the case of AR orientation, V13 at position 2
               // of _childs[v3] is dummy and will remplaced by
               // it's true value when the 'correct' adjacent triangle 
               // will be proceded
               _childs[v3].push_back(v13); 
               _childs[v3].push_back(v13);
            }
            orient[v6] = orient[v11] = orient[v13] = orient[v3];
            _treesIdx[v6] = _treesIdx[v11] = _treesIdx[v13] = _treesIdx[v3];
         } 
         else
         {
            if (orient[v3] == PL)
            {
               _childs[v3].push_back(v13);
               orient[v13] = AR;
            }
            else
            {
               _childs[v3][2] = v13;
               orient[v13] = PL;
            }
            _treesIdx[v13] = _treesIdx[v3];
         }
         if (_childs[v4].empty())
         {
             _childs[v4].reserve(4);
            if (orient[v4] == PL)
            {
               _childs[v4].push_back(v9);
               _childs[v4].push_back(v14);
               _childs[v4].push_back(v7); 
            }
            else
            {
               _childs[v4].push_back(v14);
               _childs[v4].push_back(v9);
               _childs[v4].push_back(v7);            
               // in the case of AR orientation, V13 at position 2
               // of _childs[v4] is dummy and will remplaced by
               // it's true value when the 'correct' adjacent triangle 
               // will be proceded
               _childs[v4].push_back(v7);
            }
            orient[v9] = orient[v14] = orient[v7] = orient[v4];
            _treesIdx[v9] = _treesIdx[v14] = _treesIdx[v7] = _treesIdx[v4];
         } 
         else
         {
            if (orient[v4] == PL)
            {
               _childs[v4].push_back(v7);
               orient[v7] = AR;
            }
            else
            {
               _childs[v4][2] = v7;
               orient[v7] = PL;
            }
            _treesIdx[v7] = _treesIdx[v4];
         }
         if (_childs[v5].empty())
         {
            _childs[v5].reserve(4);
            if (orient[v5] == PL)
            {
               _childs[v5].push_back(v12);
               _childs[v5].push_back(v8);
               _childs[v5].push_back(v10);  
            }
            else
            {
               _childs[v5].push_back(v8);
               _childs[v5].push_back(v12);
               // in the case of AR orientation, V13 at position 2
               // of _childs[v3] is dummy and will remplaced by
               // it's true value when the 'correct' adjacent triangle 
               // will be proceded
               _childs[v5].push_back(v10); 
               _childs[v5].push_back(v10);
            }
            orient[v12] = orient[v8] = orient[v10] = orient[v5];
            _treesIdx[v12] = _treesIdx[v8] = _treesIdx[v10] = _treesIdx[v5];
         } 
         else
         {
            if (orient[v5] == PL)
            {
               _childs[v5].push_back(v10);
               orient[v10] = AR;
            }
            else
            {
               _childs[v5][2] = v10;
               orient[v10] = PL;
            }
            _treesIdx[v10] = _treesIdx[v5];
         }
      }
   // # BUILD THE LEAVE BRANCHES LUT TABLE
   // i.e the edges which childs have no offsprings
   _leavesBranch.resize(_childs.size(), 0);
   for (i = 0 ; i < _childs.size() ; i++)
      if (!_childs[i].empty())
         if (_childs[i][0] >= _childs.size())
            _leavesBranch[i] = 1;
   // *** DEBUG VIEW 
#if _SS_DBG_VERBOSE_FOREST
   cout << "Offsprings Tree" << endl;
   cout << "---------------" << endl;
   for (i = 0 ; i < _childs.size() ; i++)
      if (_leavesBranch[i])
         cout << setw(2) << i << '*';
      else
         cout << setw(3) << i;
   cout << endl;
   for (unsigned j = 0 ; j < 4 ; j++)
   {
      for (i = 0 ; i < _childs.size() ; i++)
         if (_childs[i].size() > j)
            cout << setw(3) << _childs[i][j];
         else
            if (j == 0)
               cout << "  B";
            else
               cout << "   ";
      cout << endl;
   }
   cout << "Tree indices of offsprings" << endl;
   cout << "--------------------------" << endl;
   for (i = 0 ; i < _treesIdx.size() ; i++)
   cout << setw(3) << i;
   cout << endl;
   for (i = 0 ; i < _treesIdx.size() ; i++)
      if (_treesIdx[i] != -1)
         cout << setw(3) << _treesIdx[i];
      else
         cout << "  B";
   cout << endl;
#endif
}
