/***********************************************************************
This software module was originally developed by Yannick Maret (from the
Ecole Polytechnique Federale de Lausanne, EPFL, during a stay at the
Universidad Politecnica de Madrid, UPM) in the course of development of 
the MPEG-4 Systems (ISO/IEC 14496-1) standard. This software module is
an implementation of a part of one or more MPEG-4 Systems 
(ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems
(ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 Systems 
(ISO/IEC 14496-1) free license to this software module or modifications
thereof for use in hardware or software products claiming conformance to
the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use this 
software module in hardware or software products are advised that its 
use may infringe existing patents. The original developer of this 
software module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software module
or modifications thereof in an implementation. Copyright is not 
released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
EPFL and UPM retain full right to use the code for his/her own purpose,
assign or donate the code to a third party and to inhibit third parties 
from using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
products. This copyright notice must be included in all copies or 
derivative works. Copyright  2003-2005
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yannick Maret at Yannick.Maret@a3.epfl.ch or Francisco
Moran at fmb@gti.ssr.upm.es.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Player3d
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  WaveSurfDecoder
FILE:       _SSsForest.h
AUTHOR:     Yannick Maret (EPFL, UPM)
OVERVIEW:   Interface of the forest class (offsprings tree for SPiHT).
-------------------------------------------------------------------------*/

#ifndef _SUBDIVISION_SURFACE_FOREST_INCLUDED_
#define _SUBDIVISION_SURFACE_FOREST_INCLUDED_
// *** INCLUDES ***
#include "_SSsTopo.h"
// *** OFFSPRING ORIENTATION TYPPE ***
enum tOrientation {PL, AR};
// *** FOREST CLASS ***
class _SSsForest
{
public:
   void buildChildTrees(const _SSsTopo*);
   const vector<unsigned>* pgetChilds(const unsigned) const;
   char areChildsLeaves(const unsigned) const;
   unsigned getTreeIdx(const unsigned) const;
   unsigned getBedgeIdx(const unsigned) const;
   unsigned getNbTrees() const;
private:
   void _buildChildTrees1Lvl(const _SSsTopo*);
   unsigned _nbTrees;
   vector<vector<unsigned> > _childs;
   vector<char> _leavesBranch;
   vector<unsigned> _treesIdx;
   vector<unsigned> _invertTreesIdx;
};
// *** INLINED FUNCTIONS
inline const vector<unsigned>* _SSsForest::pgetChilds(const unsigned ridx) const
{
   return &_childs[ridx];
}
inline char _SSsForest::areChildsLeaves(const unsigned ridx) const
{
   return _leavesBranch[ridx];
}
inline unsigned _SSsForest::getTreeIdx(const unsigned ridx) const
{
   return _treesIdx[ridx];
}
inline unsigned _SSsForest::getBedgeIdx(const unsigned rtidx) const
{
   return _invertTreesIdx[rtidx];
}
inline unsigned _SSsForest::getNbTrees() const
{
   return _nbTrees;
}
#endif //_SUBDIVISION_SURFACE_FOREST_INCLUDED_