/***********************************************************************
This software module was originally developed by Yannick Maret (from the
Ecole Polytechnique Federale de Lausanne, EPFL, during a stay at the
Universidad Politecnica de Madrid, UPM) in the course of development of 
the MPEG-4 Systems (ISO/IEC 14496-1) standard. This software module is
an implementation of a part of one or more MPEG-4 Systems 
(ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems
(ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 Systems 
(ISO/IEC 14496-1) free license to this software module or modifications
thereof for use in hardware or software products claiming conformance to
the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use this 
software module in hardware or software products are advised that its 
use may infringe existing patents. The original developer of this 
software module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software module
or modifications thereof in an implementation. Copyright is not 
released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
EPFL and UPM retain full right to use the code for his/her own purpose,
assign or donate the code to a third party and to inhibit third parties 
from using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
products. This copyright notice must be included in all copies or 
derivative works. Copyright  2003-2005
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yannick Maret at Yannick.Maret@a3.epfl.ch or Francisco
Moran at fmb@gti.ssr.upm.es.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Player3d
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  WaveSurfDecoder
FILE:       _SSsQuant.cpp
AUTHOR:     Yannick Maret (EPFL, UPM)
OVERVIEW:   Implementation of the quantization tables.
-------------------------------------------------------------------------*/

// *** INCLUDES ***
#include "_SSsQuant.h"
#include <math.h>
// *** TABLES BUILDING ***
void _SSsQuant::buildQuantTables(const float rmax, const unsigned n)
{
   // *** STACK VARIABLES
   unsigned i;
   double scale = rmax/(pow(2, n) - 1);
   // # QUANT LUT
   if (n > 0)
   {
      _initQuantLut.resize(n);
      for (i = 0 ; i < n ; i++)
         _initQuantLut[i] = (float)((3*pow(2, (int)i - 1) - 0.5F)*scale);
      if (n > 1)
      {
         _addQuantLut.clear();
         _addQuantLut.resize(n - 1);
         for (i = 0 ; i < n - 1 ; i++)
            _addQuantLut[i] = (float)(pow(2, (int)i - 1)*scale);
      }
   }
#if _SS_DBG_VERBOSE_QUANT
   cout << "_initQuantLut = ";
   for (i = 0 ; i < n ; i++)
      cout << _initQuantLut[i] << ' ';
   cout << endl;
   if (n > 1)
   {
      cout << "_initQuantLut = ";
      for (i = 0 ; i < n - 1 ; i++)
         cout << _initQuantLut[i] << ' ';
      cout << endl;
   }
#endif
}