/***********************************************************************
This software module was originally developed by Yannick Maret (from the
Ecole Polytechnique Federale de Lausanne, EPFL, during a stay at the
Universidad Politecnica de Madrid, UPM) in the course of development of 
the MPEG-4 Systems (ISO/IEC 14496-1) standard. This software module is
an implementation of a part of one or more MPEG-4 Systems 
(ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems
(ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 Systems 
(ISO/IEC 14496-1) free license to this software module or modifications
thereof for use in hardware or software products claiming conformance to
the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use this 
software module in hardware or software products are advised that its 
use may infringe existing patents. The original developer of this 
software module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software module
or modifications thereof in an implementation. Copyright is not 
released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
EPFL and UPM retain full right to use the code for his/her own purpose,
assign or donate the code to a third party and to inhibit third parties 
from using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
products. This copyright notice must be included in all copies or 
derivative works. Copyright  2003-2005
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yannick Maret at Yannick.Maret@a3.epfl.ch or Francisco
Moran at fmb@gti.ssr.upm.es.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Player3d
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  WaveSurfDecoder
FILE:       _SSsSpiht.cpp
AUTHOR:     Yannick Maret (EPFL, UPM)
OVERVIEW:   Implementation of the set partionning algorithm.
-------------------------------------------------------------------------*/

#include "_SSsSpiht.h"
#include <math.h>
#include <string.h>
#include <stdlib.h>
#include <fstream>
// *** SPIHT CONFIGURATION ***
void _SSsSpiht::configure(const vector<vector<char> >* rpbsT, const _SSsForest* rpforest, 
                          float* rpwc, const _SSsQuant* rpq, const unsigned rnbBp, int l)
{
   _pbsT = rpbsT;
   _pforest = rpforest;
   _pwc = rpwc;
   _pquant = rpq;
   _nbBp = rnbBp;
   _nlevel = l;
}
// *** ADD THE WHOLE FOREST IN THE SPIHT ***
void _SSsSpiht::initForest()
{
   _treesInSpiht.resize(_pforest->getNbTrees());
   for (unsigned i = 0 ; i < _pforest->getNbTrees() ; i++)
      _treesInSpiht[i] = i;
   initTrees(NULL);
}
// *** ADD ONLY A TREE IN THE SPIHT ***
void _SSsSpiht::initTrees(const vector<unsigned>* rptreeIdx)
{
   // *** STACK VARIABLE
   unsigned i;
   tlis tmp;
   // *** INITIALIZE THE LIS AN LIE LISTES 
   // for the corresponding trees and link them for this SPiHT
   _lis.clear();
   _lie.clear();
   _lse.clear();
   if (rptreeIdx != NULL)
      _treesInSpiht = *rptreeIdx;
   tmp._type = A;
   for (i = 0 ; i < _treesInSpiht.size() ; i++)
   {
      tmp._idx =  _pforest->getBedgeIdx(_treesInSpiht[i]);
      _lie.push_back(tmp._idx);
      _lis.push_back(tmp);
   }
   // # SET THE CURRENT BIT PLANE AND THE CURRENT BIT 'INDE'
   _bp = 0;
   _n = _nbBp - 1;
   // # CLEAR THE NUMBER OF SIGNIFIVANT COEFF
   _oldNbSignif = 0;
}
// *** (RE)MOVE A TREE FROM THE SPIHT ***
void _SSsSpiht::moveTreeTo(const unsigned rtree, _SSsSpiht* rpn)
{
   // *** STACK VARIABLE
   list<unsigned>::iterator i_lse, i_lse_tmp;
   list<unsigned>::iterator i_lie, i_lie_tmp;
   list<tlis>::iterator i_lis, i_lis_tmp;
   unsigned i;
   // *** REMOVE THE TREE
   i = 0;
   while (_treesInSpiht[i] != rtree)
      i++;
   _treesInSpiht.erase(&_treesInSpiht[i]);
   // *** UPDATE NEW SPHIT ENTRIES (NOT LIST)
   if (rpn != NULL)
   {
      // # pointer
      rpn->_pbsT = _pbsT;
      rpn->_pforest = _pforest;
      rpn->_pwc = _pwc;
      rpn->_pquant = _pquant;
      rpn->_nbBp = _nbBp;
      // # current values
      rpn->_bp = _bp;
      rpn->_n = _n;
      rpn->_oldNbSignif = _oldNbSignif;
      rpn->_treesInSpiht.clear();
      rpn->_treesInSpiht.push_back(rtree);
      if (!_ibT.empty())
      {
         rpn->_ibT.resize(_ibT.size());
         rpn->_ibT[rtree] = _ibT[rtree];
      }
      rpn->_nbBp = _nbBp;
   }
   // *** (RE)MOVE THE TREE ENTRIES (LIST)
   if (rpn != NULL)
      rpn->_lse.clear();
   for (i_lse = _lse.begin() ; i_lse != _lse.end() ;)
      if (_pforest->getTreeIdx(*i_lse) == rtree)
      {
         i_lse_tmp = i_lse;
         if (rpn != NULL)
            rpn->_lse.push_back(*i_lse);
         i_lse++;
         _lse.erase(i_lse_tmp);
      }
      else
         i_lse++;
   // # LIE
   if (rpn != NULL)
      rpn->_lie.clear();
   for (i_lie = _lie.begin() ; i_lie != _lie.end() ;)
      if (_pforest->getTreeIdx(*i_lie) == rtree)
      {
         i_lie_tmp = i_lie;
         if (rpn != NULL)
            rpn->_lie.push_back(*i_lie);
         i_lie++;
         _lie.erase(i_lie_tmp);
      }
      else
         i_lie++;
   // # LIS
   if (rpn != NULL)
      rpn->_lis.clear();
   for (i_lis = _lis.begin() ; i_lis != _lis.end() ;)
      if (_pforest->getTreeIdx(i_lis->_idx) == rtree)
      {
         i_lis_tmp = i_lis;
         if (rpn != NULL)
            rpn->_lis.push_back(*i_lis);
         i_lis++;
         _lis.erase(i_lis_tmp);
      }
      else
         i_lis++;
}
// *** SPiHT ALGORITHM ***
void _SSsSpiht::decodeNextBp()
{
	if (_n >= 0){
   // *** UPDATE BIT STREAM POINTER
   // # STACK VARIABLE
   unsigned i;
   if (_bp == 0)
      _ibT.resize(_pforest->getNbTrees());
   for (i = 0 ; i < _treesInSpiht.size() ; i++)
      _ibT[_treesInSpiht[i]] = _pbsT[_treesInSpiht[i]][_bp].begin();
   // *** SORTING PASS
   // # STACK VARIABLE
   const unsigned liv_size = _lie.size();
   list<unsigned>::iterator it_lie = _lie.begin();
   list<unsigned>::iterator it_lie_tmp;
   // # ALGO II, 2.1
#if _SS_DBG_VERBOSE_SPIHT
   cout << "Algo II.1, bitPlane No " << _bp << endl;
#endif
   for (i = 0 ; i < liv_size ; i++)
      // input significance of *it_lie
      if (getNextBit(*it_lie))
      {
         // input the sign of *it_lie
         if (getNextBit(*it_lie))
            _pwc[*it_lie] = _pquant->getInitVal(_n);
         else
            _pwc[*it_lie] = -_pquant->getInitVal(_n);
         _lse.push_back(*it_lie);
         it_lie_tmp = it_lie++;
         _lie.erase(it_lie_tmp);
      }
      else
         it_lie++;
#if _SS_DBG_VERBOSE_SPIHT
   cout << "After (2.1): LIV = ";
   for (it_lie = _lie.begin(), i = 0 ; i < _lie.size() ; i++, it_lie++)
      cout << *it_lie << ' ';
   cout << endl;
   cout << "             LSV = ";
   for (it_lie = _lse.begin(), i = 0 ; i < _lse.size() ; i++, it_lie++)
      cout << *it_lie << ' ';
   cout << endl;
#endif
   // # ALGO II, 2.2
  if (_nlevel != 1){
   tlis lis_tmp;
   list<tlis>::iterator it_lis = _lis.begin();
   list<tlis>::iterator it_lis_tmp;
   vector<unsigned>::const_iterator it_childs;
   unsigned idx;
   for (i = 0 ; it_lis != _lis.end() ; i++)
   {
      if (it_lis->_type == A)
      {
         // # TYPE *A* ENTRY
         idx = it_lis->_idx;
         // input significance of descent of *it_lis
         if (getNextBit(idx))
         {
            for (it_childs = _pforest->pgetChilds(idx)->begin() ; it_childs < _pforest->pgetChilds(idx)->end() ; it_childs++)
            {
               // input significance of offsprings of idx
               if (getNextBit(idx))
               {
                  _lse.push_back(*it_childs);
                  // input sign of offspring j of idx
                  if (getNextBit(*it_childs))
                     _pwc[*it_childs] = _pquant->getInitVal(_n);
                  else
                     _pwc[*it_childs] = -_pquant->getInitVal(_n);
               }
               else
                  _lie.push_back(*it_childs);
            }
            if (!_pforest->areChildsLeaves(idx))
               {
                  // add as at the end of LIS type B entry
                  lis_tmp._idx = idx;
                  lis_tmp._type = B;
                  _lis.push_back(lis_tmp);     
               }
            // supress the current entry
            // (it has been either moved or it's not more usefull)
            it_lis_tmp = it_lis;
            it_lis++;
            _lis.erase(it_lis_tmp);
         }
         else
            it_lis++;
      }
      else
      {
         // TYPE *B* ENTRY
         idx = it_lis->_idx;
         // input significance of after offspring of *it_lis
         if (getNextBit(idx))
         {
            lis_tmp._type = A;
            for (it_childs = _pforest->pgetChilds(idx)->begin() ; it_childs < _pforest->pgetChilds(idx)->end() ; it_childs++)
            {
               lis_tmp._idx = *it_childs;
               _lis.push_back(lis_tmp);
            }
            it_lis_tmp = it_lis;
            it_lis++;
            _lis.erase(it_lis_tmp);
         }
         else
            it_lis++;
      }
   }
  }
#if _SS_DBG_VERBOSE_SPIHT
   cout << "After (2.2): LIV = ";
   for (it_lie = _lie.begin(), i = 0 ; i < _lie.size() ; i++, it_lie++)
      cout << *it_lie << ' ';
   cout << endl;
   cout << "             LSV = ";
   for (it_lie = _lse.begin(), i = 0 ; i < _lse.size() ; i++, it_lie++)
      cout << *it_lie << ' ';
   cout << endl;
   cout << "             LIS = ";
   for (it_lis = _lis.begin(), i = 0 ; i < _lis.size() ; i++, it_lis++)
      cout << it_lis->_idx << (char)(it_lis->_type + 'A') << ' ';
   cout << endl;
#endif
   // *** REFINE PASS
   // # STACK VARIABLE
   list<unsigned>::const_iterator it_lsc;
   // # REFINE DETAIL COEFFICIENTS
   // (the one already significant before this sorting pass)
#if _SS_DBG_VERBOSE_SPIHT
   cout << "Algo II.3" << endl;
   cout << "Vertices being raffined = ";
   for (i = 0, it_lsc = _lse.begin() ; i < _oldNbSignif ; i++, it_lsc++)
      cout << *it_lsc << ' ';
      cout << endl;
#endif
   for (i = 0, it_lsc = _lse.begin() ; i < _oldNbSignif ; i++, it_lsc++)
      // input the (_n)-th most signifiant bit of *it_lsc
      if (getNextBit(*it_lsc) ^ (_pwc[*it_lsc] < 0))
         _pwc[*it_lsc] += _pquant->getAddiVal(_n);
      else
         _pwc[*it_lsc] -= _pquant->getAddiVal(_n);
   // # PREPARE FOR THE NET BIT PLANE
   _oldNbSignif = _lse.size();
   _bp++;
   }
   _n--;
}
