/***********************************************************************
This software module was originally developed by Yannick Maret (from the
Ecole Polytechnique Federale de Lausanne, EPFL, during a stay at the
Universidad Politecnica de Madrid, UPM) in the course of development of 
the MPEG-4 Systems (ISO/IEC 14496-1) standard. This software module is
an implementation of a part of one or more MPEG-4 Systems 
(ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems
(ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 Systems 
(ISO/IEC 14496-1) free license to this software module or modifications
thereof for use in hardware or software products claiming conformance to
the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use this 
software module in hardware or software products are advised that its 
use may infringe existing patents. The original developer of this 
software module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software module
or modifications thereof in an implementation. Copyright is not 
released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
EPFL and UPM retain full right to use the code for his/her own purpose,
assign or donate the code to a third party and to inhibit third parties 
from using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
products. This copyright notice must be included in all copies or 
derivative works. Copyright  2003-2005
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yannick Maret at Yannick.Maret@a3.epfl.ch or Francisco
Moran at fmb@gti.ssr.upm.es.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Player3d
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  WaveSurfDecoder
FILE:       _SSsSpiht.h
AUTHOR:     Yannick Maret (EPFL, UPM)
OVERVIEW:   Interface of the set partionning algorithm.
-------------------------------------------------------------------------*/

#ifndef _WAVELET_SUBDIVISION_SURFACE_SPHIT_INCLUDED_
#define _WAVELET_SUBDIVISION_SURFACE_SPHIT_INCLUDED_
// *** INCLUDES ***
#include "_SSsQuant.h"
#include "_SSsForest.h"
//#include "MarkupSTL.h"
// *** LIS TYPE ***
enum tlisType {A, B};
struct tlis
{
   unsigned _idx;
   enum tlisType _type;
};
// *** SPIHT CLASS ***
class _SSsSpiht
{
public:
   // # public INTERFACE
   // destruction & construction
   void configure(const vector<vector<char> >*, const _SSsForest*,
                  float*, const _SSsQuant*, const unsigned, int);
   // # INHERITANCE SHARED DATA & FUNCTIONS
   void decodeNextBp();
   void moveTreeTo(const unsigned, _SSsSpiht* = NULL);
   void initForest();
   void initTrees(const vector<unsigned>*);
private:
   // # PRIVATE DATA & FUNCTIONS
   // shared (between x, y & z) data
   const _SSsForest *_pforest;
   float*_pwc;
   const vector<vector<char> > *_pbsT;
   vector<vector<char>::const_iterator> _ibT;
   const _SSsQuant *_pquant;
   list<unsigned> _lse;
   list<unsigned> _lie;
   list<tlis> _lis;
   unsigned _bp;
   int _n;
   unsigned _nc;
   unsigned _nbBp;
   unsigned _oldNbSignif;
   int _nlevel;
   vector<unsigned> _treesInSpiht;
   char getNextBit(const unsigned);
};
// *** INLINED FUNCTIONS DEFINITION
inline char _SSsSpiht::getNextBit(const unsigned rvidx)
{
   return *(_ibT[_pforest->getTreeIdx(rvidx)]++);
}
#endif//ndef _WAVELET_SUBDIVISION_SURFACE_SPHIT_INCLUDED_
