/***********************************************************************
This software module was originally developed by Yannick Maret (from the
Ecole Polytechnique Federale de Lausanne, EPFL, during a stay at the
Universidad Politecnica de Madrid, UPM) in the course of development of 
the MPEG-4 Systems (ISO/IEC 14496-1) standard. This software module is
an implementation of a part of one or more MPEG-4 Systems 
(ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems
(ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 Systems 
(ISO/IEC 14496-1) free license to this software module or modifications
thereof for use in hardware or software products claiming conformance to
the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use this 
software module in hardware or software products are advised that its 
use may infringe existing patents. The original developer of this 
software module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software module
or modifications thereof in an implementation. Copyright is not 
released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
EPFL and UPM retain full right to use the code for his/her own purpose,
assign or donate the code to a third party and to inhibit third parties 
from using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
products. This copyright notice must be included in all copies or 
derivative works. Copyright  2003-2005
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yannick Maret at Yannick.Maret@a3.epfl.ch or Francisco
Moran at fmb@gti.ssr.upm.es.
----------------------------------------------------------------------*/

/***********************************************************************
Revision 1.1: 2005/9/15

Alberto Lopez-Ceron Pinilla has added a function to create a structure 
that permits the use of local coordinates. 
***********************************************************************/

/*-------------------------------------------------------------------------
Project Player3d
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  WaveSurfDecoder
FILE:       _SSsStencil.cpp
AUTHOR:     Yannick Maret (EPFL, UPM)
OVERVIEW:   Implementation of the subdivision surfaces stencils.
-------------------------------------------------------------------------*/

// *** INCLUDES ***
#include "_SSsStencil.h"
// *** DATA ACCESS ***
const vector<vector<stencil> >* _SSsStencil::pgetStencilsLUT()
{
   return &_stencilLpLUT;
}
const vector<vector<stencil> >* _SSsStencil::pgetTriStenLUT()
{
   return &_triStenLUT;
}
bool _SSsStencil::isInterpolating() const
{
   return _interpolating;
}
// *** MASK CREATION (MAIN) ***
// LOW PASS
void _SSsStencil::buildStencilsLUT(const unsigned rnbSub, const enum etSSsType rtype,
                                   const unsigned rsubType, const bool rhpLoop)
{
#if _TIME_SSLIB_
   _evenTime = _oddTime = _stencilTime = 0;
   _startTimeStencils = clock();
#endif
   // *** STACK VARIABLES
   unsigned i, j, nb_vert;
   vector<ring>::const_iterator i_orings_lut, i_nrings_lut, i_o_aux;
   const vector<vector<ring> >* p_rings_lut = pgetRingsLUT();
   const vector<unsigned>* p_nb_vert = pgetNbVertLUT();
   ring::const_iterator i_nring, i_oring;
   stencil* p_stencil;

   // *** POSITIONS UPDATE
   _stencilLpLUT.resize(rnbSub);
   //_triStenLUT.resize(rnbSub);
   switch (rtype)
   {
   case LOOP:
      _interpolating = false;
      break;
   case BUTTERFLY:
   case MID_POINT:
      _interpolating = true;
      break;
   default:
      throw SSsSchemeUnknow();
   };        
   for (_currSubLevel = 0 ; _currSubLevel < rnbSub ; _currSubLevel++)
   {
      i_orings_lut = (*p_rings_lut)[_currSubLevel].begin();
      i_nrings_lut = (*p_rings_lut)[_currSubLevel + 1].begin();
      i_o_aux = (*p_rings_lut)[_currSubLevel].begin();
      nb_vert = (*p_nb_vert)[_currSubLevel];
      _stencilLpLUT[_currSubLevel].resize((*p_nb_vert)[_currSubLevel + 1]);
      if (rhpLoop)
         _stencilHpLUT[_currSubLevel].resize((*p_nb_vert)[_currSubLevel + 1]);
      for (i = 0 ; i < nb_vert ; i++, i_orings_lut++, i_nrings_lut++)
      {
#if _TIME_SSLIB_
         _startTime = clock();
#endif
         // # EVEN VERTEX POSITION
         // create stencil
         switch (rtype)
         {
         case LOOP:
            p_stencil = &_stencilLpLUT[_currSubLevel][i];
            p_stencil->clear();
            p_stencil->reserve(7);
            _evenVertStencilLoop(p_stencil, i, i_orings_lut, rsubType);
            if (rhpLoop)
            {
               p_stencil = &_stencilHpLUT[_currSubLevel][i];
               p_stencil->clear();
               p_stencil->reserve(7);
               _evenVertStencilLoopHp(p_stencil, i, i_orings_lut, rsubType);
            }
            break;
         case BUTTERFLY:
         case MID_POINT:
            break;
         };        
         // store stencil
#if _TIME_SSLIB_
         _endTime = clock();
         _evenTime += _endTime - _startTime;
#endif      
         // # ODD VERTEX POSITION
#if _TIME_SSLIB_
         _startTime = clock();
#endif
         i_oring = i_orings_lut->begin();
         i_nring = i_nrings_lut->begin(); 
         for (j = 0 ; j < i_orings_lut->size() ; j++, i_nring++, i_oring++)
            if (*i_oring >= (int)i) // position not yet computed
            { 
               p_stencil = &_stencilLpLUT[_currSubLevel][*i_nring];
               p_stencil->clear();
	    
               // create stencil
               switch (rtype)
               {
               case LOOP:
                  p_stencil->reserve(7);
                  _oddVertStencilLoop(p_stencil, i, j, i_orings_lut);
                  break;
               case MID_POINT:
                  p_stencil->push_back(i, 0.5F);
                  p_stencil->push_back(*i_oring, 0.5F);
		  
                  break;
               case BUTTERFLY:
                  _stencilButterfly(p_stencil, i, j, &(*p_rings_lut)[_currSubLevel], rsubType == 1);
		
                  break;
               };    
            }
#if _TIME_SSLIB_
         _endTime = clock();
         _oddTime += _endTime - _startTime;
#endif
      }
   }

}
// *** MASK CREATION (LOOP) ***
// # LOOP EVEN VERTICES
inline void _SSsStencil::_evenVertStencilLoop(stencil* rpstencil, const unsigned rv0Idx, 
                                              const ring* rpring, const unsigned rbetaMode)
{
   // *** 'STACK' VARIABLE
   // static evite to create them each time!
   ring::const_iterator i_oring;
   unsigned k, ridx;
   float a, beta;
   k = getNbAdjFaces(rv0Idx);
   // *** CREATE EVEN MASK
   if (!isCreaseVert(rv0Idx))
   {
      if (!isCornerVert(rv0Idx))
      {
         // # interior (cf. [1), 6.3.3, fig.7a)
         if (rbetaMode == 0)
         {
            a = 3.F/8 + (float)cos(2*M_PI/k)/4;
            beta = (float)(5.F/8 - a*a)/k;
         }
         else
            beta = (k == 3)? (3.F/16) : (3.F/(8*k));
         for (i_oring = rpring->begin() ; i_oring < rpring->end() ; i_oring++)
            rpstencil->push_back(*i_oring, beta);    
         rpstencil->push_back(rv0Idx, 1 - k*beta);
      }
      else
         // # corner (never moved)
         rpstencil->push_back(rv0Idx, 1.F);
   }
   else
   {
      // # crease vertex
      // first crease edge
      rpstencil->push_back(rv0Idx, 3.F/4);
      rpring->setCurrRidx(ridx = 1);
      while (!isCreaseEdge(rv0Idx, ridx))
        ridx = rpring->getNextRidx(); 
      rpstencil->push_back((*rpring)[ridx], 1.F/8);
      // second crease edge
      rpring->setCurrRidx(ridx = 0);
      while (!isCreaseEdge(rv0Idx, ridx))
        ridx = rpring->getPrevRidx(); 
      rpstencil->push_back((*rpring)[ridx], 1.F/8);
   }
}
// # LOOP ODD VERTICES
inline void _SSsStencil::_oddVertStencilLoop(stencil* rpstencil, const unsigned rv0Idx,
                                             const unsigned rridx, const ring* rpring)
{
   // *** 'STACK' VARIABLES
   // static evite to create them each time!
   unsigned v1_idx, v0_k, v1_k;
   bool v0_tagextra, v1_tagextra;
   float gama, half;
   v1_idx = (*rpring)[rridx];
   // *** GET NEIGHBOURGS ID
   if (!isCreaseEdge(rv0Idx, rridx) || isDartVert(rv0Idx) || isDartVert(v1_idx))
   {
      // get the opposite vertices of the the current edge and set their weigth
      rpring->setCurrRidx(rridx);
      rpstencil->push_back((*rpring)[rpring->getPrevRidx()], 1.F/8);
      rpring->setCurrRidx(rridx);;
      rpstencil->push_back((*rpring)[rpring->getNextRidx()], 1.F/8);
      // control the extraodinarity of the tagged ends (if any)
      if (isCornerVert(rv0Idx) || isCreaseVert(rv0Idx))
         v0_tagextra = (v0_k = getCreaseDegree(_currSubLevel, rv0Idx, v1_idx)) != 3;
      else
         v0_tagextra = false;
      if (isCornerVert(v1_idx) || isCreaseVert(v1_idx))
         v1_tagextra = (v1_k = getCreaseDegree(_currSubLevel, v1_idx, rv0Idx)) != 3;
      else
         v1_tagextra = false;
      if (!v0_tagextra && !v1_tagextra)
      {
         // A) absence of tags, cf. [1), 6.3.3, fig.7c
         // C) a dart end, cf. [1), 6.3.3, fig.7c
         rpstencil->push_back(rv0Idx, 3.F/8);
         rpstencil->push_back(v1_idx, 3.F/8);
      }
      else
      {
         // D)E) one or both end point(s) is(are) tagged as corner or crease
         //      *AND* is(are) extraordinar (i.e crease_degree!=3 or valence!=4)
         half = (v0_k && v1_k) ? 0.5F : 1.0F;
         if (v0_k)
         {
            gama = 1.F/2;
            gama -= (isCreaseVert(rv0Idx))? (float)cos(M_PI/v0_k)/4 : (float)cos(M_PI/(2*v0_k))/4;
            rpstencil->push_back(rv0Idx, (3.F/4 - gama)*half);
            rpstencil->push_back(v1_idx, gama*half);
         }
         if (v1_k)
         {
            gama = 1.F/2;
            gama -= (isCreaseVert(v1_idx))? (float)cos(M_PI/v1_k)/4 : (float)cos(M_PI/(2*v1_k))/4;
            rpstencil->push_back(v1_idx, (3.F/4 - gama)*half);
            rpstencil->push_back(rv0Idx, gama*half);
         }
      }
   }
   else
   {
      // # CREASE EDGE
      // B) crease edge without any dart end, cf. [1), 6.3.3, fig.7d
      rpstencil->push_back(rv0Idx, 1.F/2);
      rpstencil->push_back(v1_idx, 1.F/2);
   }
}
void _SSsStencil::_evenVertStencilLoopHp(stencil* rpstencil, const unsigned rv0Idx,
                                   const ring* rpring, const unsigned rridx)
{

}
void _SSsStencil::_oddVertStencilLoopHp(stencil*, const unsigned, const unsigned, const ring*)
{
}

void _SSsStencil::buildTriStenLUT(unsigned nl){//added by Alberto Lpez-Cern Pinilla to support the use of local coordinates
  unsigned i, j, v0, v1, v2, v3, v4, v5;
  stencil *p_triSten;
  vector<int>::const_iterator it_cid;
  const vector<unsigned>* p_nb_vert = pgetNbVertLUT();
  const vector<vector<int> >* p_all_cid = pgetCoordIdLUT();
  _triStenLUT.resize(nl);
  for (i = 0; i < nl; i++){
    _triStenLUT[i].resize((*p_nb_vert)[i + 1]);
    for (j = 0; j < _triStenLUT[i].size(); j++){
      p_triSten = &_triStenLUT[i][j];
      p_triSten->clear();
    }
    for (it_cid = p_all_cid->at(i + 1).begin(); it_cid < p_all_cid->at(i + 1).end(); it_cid++){
      v0 = *(it_cid++);
      v3 = *(it_cid++);
      v5 = *(it_cid++);
      it_cid++;
      v1 = *(it_cid++);
      v4 = *(it_cid++);
      it_cid += 2;
      v2 = *it_cid;
      it_cid += 7;
      p_triSten = &_triStenLUT[i][v3];
      if (!(*p_triSten).empty()){
	p_triSten->push_back(v2, 1.0F);
      }
      else {
	p_triSten->push_back(v0, 1.0F);
	p_triSten->push_back(v1, 1.0F);
	p_triSten->push_back(v2, 1.0F);
      }
      p_triSten = &_triStenLUT[i][v4];
      if (!(*p_triSten).empty()) p_triSten->push_back(v0, 1.0F);
      else {
	p_triSten->push_back(v1, 1.0F);
	p_triSten->push_back(v2, 1.0F);
	p_triSten->push_back(v0, 1.0F);
      }
      p_triSten = &_triStenLUT[i][v5];
      if (!(*p_triSten).empty()) p_triSten->push_back(v1, 1.0F);
      else {
	p_triSten->push_back(v2, 1.0F);
	p_triSten->push_back(v0, 1.0F);
	p_triSten->push_back(v1, 1.0F);
      }
    }
  }
}
// *** MASK CREATION (BUTTERFLY) ***
// # BUTTERFLY MAIN
const float _M1_16 = -0.0625F;
const float  _M1_32 = -0.03125F;
const float  _9_16 = 0.5625F;
const float  _1_16 = 0.0625F;
inline void _SSsStencil::_stencilButterfly(stencil* rpstencil, const unsigned rv0Idx, const unsigned rridx,
                                           const vector<ring>* rporings, const bool rmodified)
{
   // *** STACK VARIABLES
   unsigned v1_idx, ridx_v0;
   unsigned ridx_v0_ccw[2], ridx_v0_cw[2];
   unsigned ridx_v1_ccw, ridx_v1_cw;
   v1_idx = (*rporings)[rv0Idx][rridx];
   ridx_v0 = (*rporings)[v1_idx].getPosInRing( rv0Idx);
   if (!isCreaseEdge(rv0Idx, rridx) || isDartVert(rv0Idx) || isDartVert(v1_idx))
   {
      if (!rmodified)
      {
         // *** DYN'S ORIGINAL BUTTERLFY, NOT ON CREASE
         // # STACK VARIABLES
         char v0_cw_crease, v0_ccw_crease, v1_cw_crease, v1_ccw_crease;
         // get v0 ccw neighbors
         (*rporings)[rv0Idx].setCurrRidx(rridx);
         ridx_v0_ccw[0] = (*rporings)[rv0Idx].getNextRidx();
         if (!(v0_ccw_crease = isCreaseEdge(rv0Idx, ridx_v0_ccw[0])))
            ridx_v0_ccw[1] = (*rporings)[rv0Idx].getNextRidx();
         // get v0 cw neighbors
         (*rporings)[rv0Idx].setCurrRidx(rridx);
         ridx_v0_cw[0] = (*rporings)[rv0Idx].getPrevRidx();
         if (!(v0_cw_crease = isCreaseEdge(rv0Idx, ridx_v0_cw[0])))
            ridx_v0_cw[1] = (*rporings)[rv0Idx].getPrevRidx();
         // get v1 ccw neighbors
         (*rporings)[v1_idx].setCurrRidx(ridx_v0);
         ridx_v1_ccw = (*rporings)[v1_idx].getNextRidx();
         if (!(v1_ccw_crease = isCreaseEdge(v1_idx, ridx_v1_ccw)))
            ridx_v1_ccw = (*rporings)[v1_idx].getNextRidx();
         // get v1 cw neighbors
         (*rporings)[v1_idx].setCurrRidx(ridx_v0);
         ridx_v1_cw = (*rporings)[v1_idx].getPrevRidx();
         if (!(v1_cw_crease = isCreaseEdge(v1_idx, ridx_v1_cw)))
            ridx_v1_cw = (*rporings)[v1_idx].getPrevRidx();
         // build the actual stencil
         rpstencil->push_back(rv0Idx, 0.5F);
         rpstencil->push_back(v1_idx, 0.5F);
         // chop off triangles if crease(s)
         if (!v0_ccw_crease)
           rpstencil->push_back((*rporings)[rv0Idx][ridx_v0_ccw[1]], _M1_16);
         if (!v1_cw_crease)
            rpstencil->push_back((*rporings)[v1_idx][ridx_v1_cw], _M1_16);
         if (!(v0_ccw_crease || v1_cw_crease))
            rpstencil->push_back((*rporings)[rv0Idx][ridx_v0_ccw[0]], 0.125);
         else if (!(v0_ccw_crease && v1_cw_crease))
            rpstencil->push_back((*rporings)[rv0Idx][ridx_v0_ccw[0]], _1_16);
         if (!v0_cw_crease)
            rpstencil->push_back((*rporings)[rv0Idx][ridx_v0_cw[1]], _M1_16);
         if (!v1_ccw_crease)
            rpstencil->push_back((*rporings)[v1_idx][ridx_v1_ccw], _M1_16);
         if (!(v0_cw_crease || v1_ccw_crease))
            rpstencil->push_back((*rporings)[rv0Idx][ridx_v0_cw[0]], 0.125);
         else if (!(v0_cw_crease && v1_ccw_crease))
            rpstencil->push_back((*rporings)[rv0Idx][ridx_v0_cw[0]], _1_16);
      }
      else
      {
         // *** MODIFIED BUTTERFLY, NOT ON CREASE
         // cf. "Subdivision for Modeling and Animation", Sigraph'00 course pp.72-75
         // STACK VARIABLES
         bool excrease_0, excrease_1, exintern_0, exintern_1;
         vector<unsigned> cr_lim_0, cr_lim_1;
         unsigned v0_k, v1_k;
         float half;
         // # CHECK IF NEED OF SPECIAL RULES 
         // (in case of modified butterfly scheme)
         excrease_0 = excrease_1 = exintern_0 = exintern_1 = false;
         if (isCreaseVert(rv0Idx))
         {
            getCrLimRidx(_currSubLevel, rv0Idx, rridx, cr_lim_0);
            excrease_0 = (v0_k = cr_lim_0[2]) != 4;
         }
         else
            exintern_0 = getNbAdjVert(rv0Idx) != 6;
         if (isCreaseVert(v1_idx))
         {
            getCrLimRidx(_currSubLevel, v1_idx, ridx_v0, cr_lim_1);
            excrease_1 = (v1_k = cr_lim_1[2]) != 4;
         }
         else
            exintern_1 = getNbAdjVert(v1_idx) != 6;
         if ((excrease_0 || exintern_0) && (excrease_1 || exintern_1))
            half = 0.5F;
         else
            half = 1.0F;
         // # EXTRAORDINARY NEIGHBORORS
         if (exintern_0)
            // head is extraordinary interior
            _exinternStencilButterfly(rpstencil, rv0Idx, rridx, &(*rporings)[rv0Idx], half);
         else if (excrease_0)
            // head is extraordinary crease
            _excreaseStencilButterfly(rpstencil, rv0Idx, rridx, &(*rporings)[rv0Idx], half, &cr_lim_0);
         if (exintern_1)
            // tail is extraordinary interior
            _exinternStencilButterfly(rpstencil, v1_idx, ridx_v0, &(*rporings)[v1_idx], half);
         else if (excrease_1)
            // tail is extraordinary crease
            _excreaseStencilButterfly(rpstencil, v1_idx, ridx_v0, &(*rporings)[v1_idx], half, &cr_lim_1);
         if (!exintern_0 && !exintern_1 && !excrease_0 && !excrease_1)
         {
            // # REGULAR NEIGHBORORS
            // STACK VARIABLES
            char v0_cw_crease, v0_ccw_crease, v1_cw_crease, v1_ccw_crease;
            // get v0 ccw neighbors
            (*rporings)[rv0Idx].setCurrRidx(rridx);
            ridx_v0_ccw[0] = (*rporings)[rv0Idx].getNextRidx();
            if (!(v0_ccw_crease = isCreaseEdge(rv0Idx, ridx_v0_ccw[0])))
               ridx_v0_ccw[1] = (*rporings)[rv0Idx].getNextRidx();
            // get v0 cw neighbors
            (*rporings)[rv0Idx].setCurrRidx(rridx);
            ridx_v0_cw[0] = (*rporings)[rv0Idx].getPrevRidx();
            if (!(v0_cw_crease = isCreaseEdge(rv0Idx, ridx_v0_cw[0])))
               ridx_v0_cw[1] = (*rporings)[rv0Idx].getPrevRidx();
            // get v1 ccw neighbors
            (*rporings)[v1_idx].setCurrRidx(ridx_v0);
            ridx_v1_ccw = (*rporings)[v1_idx].getNextRidx();
            if (!(v1_ccw_crease = isCreaseEdge(v1_idx, ridx_v1_ccw)))
               ridx_v1_ccw = (*rporings)[v1_idx].getNextRidx();
            // get v1 cw neighbors
            (*rporings)[v1_idx].setCurrRidx(ridx_v0);
            ridx_v1_cw = (*rporings)[v1_idx].getPrevRidx();
            if (!(v1_cw_crease = isCreaseEdge(v1_idx, ridx_v1_cw)))
               ridx_v1_cw = (*rporings)[v1_idx].getPrevRidx();
            // build stencil
            if (v0_ccw_crease && v1_cw_crease || v0_cw_crease && v1_ccw_crease)
            {
               // crease-crease rule 2
               rpstencil->push_back(rv0Idx, 0.5F);
               rpstencil->push_back(v1_idx, 0.5F);
            }
            else if (!v0_ccw_crease && !v0_cw_crease && !v1_ccw_crease && !v1_cw_crease)
            {
               // regular-interior
               rpstencil->push_back(rv0Idx, 0.5F);
               rpstencil->push_back(v1_idx, 0.5F);
               rpstencil->push_back((*rporings)[rv0Idx][ridx_v0_ccw[0]], 0.125F );
               rpstencil->push_back((*rporings)[rv0Idx][ridx_v0_ccw[1]], _M1_16);
               rpstencil->push_back((*rporings)[rv0Idx][ridx_v0_cw[0]], 0.125F);
               rpstencil->push_back((*rporings)[rv0Idx][ridx_v0_cw[1]], _M1_16);
               rpstencil->push_back((*rporings)[v1_idx][ridx_v1_ccw], _M1_16);
               rpstencil->push_back((*rporings)[v1_idx][ridx_v1_cw], _M1_16);
            }
            else
            {
               // crease-interior and crease-crease rule 1 can be built by the same algo
               float add;
               add = (v1_cw_crease - v0_ccw_crease - v0_cw_crease + v1_ccw_crease)/8.F;
               rpstencil->push_back(rv0Idx, 0.5F + add);
               rpstencil->push_back(v1_idx, 0.5F - add);
               if (!v0_ccw_crease)
                  rpstencil->push_back((*rporings)[rv0Idx][ridx_v0_ccw[1]], _M1_16*(1 + v1_ccw_crease));
               if (!v0_cw_crease)
                  rpstencil->push_back((*rporings)[rv0Idx][ridx_v0_cw[1]], _M1_16*(1 + v1_cw_crease));
               if (!v1_ccw_crease)
                  rpstencil->push_back((*rporings)[v1_idx][ridx_v1_ccw], _M1_16*(1 + v0_ccw_crease));
               if (!v1_cw_crease)
                  rpstencil->push_back((*rporings)[v1_idx][ridx_v1_cw], _M1_16*(1 + v0_cw_crease));
               add = (v0_cw_crease + v1_ccw_crease - v0_ccw_crease - v1_cw_crease)/16.F;
               if (!(v0_ccw_crease && v1_cw_crease))
                  rpstencil->push_back((*rporings)[rv0Idx][ridx_v0_ccw[0]], 0.125F + add);
               if (!(v0_cw_crease && v1_ccw_crease))
                  rpstencil->push_back((*rporings)[rv0Idx][ridx_v0_cw[0]], 0.125F - add);
            }
         }
      }
   }
   else
   {
      // *** BUTTERFLY, ON CREASE
      // # STACK VARIABLE
      unsigned cw_ridx, ccw_ridx;
      // direct ends
      rpstencil->push_back(rv0Idx, _9_16);
      rpstencil->push_back(v1_idx, _9_16);
      // # find where the crease continue (search cw)
      (*rporings)[rv0Idx].setCurrRidx(rridx);
      do
        cw_ridx = (*rporings)[rv0Idx].getPrevRidx(); 
      while (!isCreaseEdge(rv0Idx, cw_ridx));
      if (isCornerVert(rv0Idx))
      {
         // V0 IS A CORNER VERTEX
         // find where the *other* crease continue (search ccw)
         (*rporings)[rv0Idx].setCurrRidx(rridx);
         do
            ccw_ridx = (*rporings)[rv0Idx].getNextRidx(); 
         while (!isCreaseEdge(rv0Idx, ccw_ridx));
         rpstencil->push_back((*rporings)[rv0Idx][ccw_ridx], _M1_32);
         rpstencil->push_back((*rporings)[rv0Idx][cw_ridx], _M1_32);
      }
      else
         rpstencil->push_back((*rporings)[rv0Idx][cw_ridx], _M1_16);
      // # find where the crease continue (search cw)
      (*rporings)[v1_idx].setCurrRidx(ridx_v0);
      do
        cw_ridx = (*rporings)[v1_idx].getPrevRidx(); 
      while (!isCreaseEdge(v1_idx, cw_ridx));
      if (isCornerVert(v1_idx))
      {
         // V1 IS A CORNER VERTEX
         // find where the *other *crease continue (search ccw)
         (*rporings)[v1_idx].setCurrRidx(ridx_v0);
         do
            ccw_ridx = (*rporings)[v1_idx].getNextRidx(); 
         while (!isCreaseEdge(v1_idx, ccw_ridx));
         rpstencil->push_back((*rporings)[v1_idx][ccw_ridx], _M1_32);
         rpstencil->push_back((*rporings)[v1_idx][cw_ridx], _M1_32);
      }
      else
         rpstencil->push_back((*rporings)[v1_idx][cw_ridx], _M1_16);
   }
}
// # BUTTERFLY, INTERIOR VERTICES ADJACENTS TO EXTRAORDINARY INTERIOR
const float _M1_12 = -0.0833333333333F; // -1.0/12.0
const float _5_12  =  0.4166666666666F; // 5.0/12.0
inline void _SSsStencil::_exinternStencilButterfly(stencil* rpstencil, const unsigned rvIdx, const unsigned rridx,
                                                   const ring* rporings, const float rhalf)
{
   // rules for neighbors of extraordinary interior
   const unsigned k = getNbAdjVert(rvIdx);
   rpstencil->push_back(rvIdx, 0.75F*rhalf);
   if (k == 3)
   {
      rporings->setCurrRidx(rridx);
      rpstencil->push_back(rporings->getNext(), _M1_12*rhalf);
      rpstencil->push_back(rporings->getNext(), _M1_12*rhalf);
      rpstencil->push_back((*rporings)[rridx], _5_12*rhalf);
   }
   else if (k == 4)
   {
      rporings->setCurrRidx(rridx);
      rporings->getNextRidx();
      rpstencil->push_back(rporings->getNext(), -0.125F*rhalf);
      rpstencil->push_back((*rporings)[rridx], 0.375F*rhalf);
   }
   else
   {
      float weigth, inv_k = 1.F/k;
      unsigned i;
      rporings->setCurrRidx(rridx);
      rporings->getPrev();
      for (i = 0 ; i < k ; i++)
      {
         weigth = inv_k*
                  (0.25F + (float)cos(2*M_PI*i*inv_k) + 0.5F*(float)cos(4*M_PI*i*inv_k));
         rpstencil->push_back(rporings->getNext(), weigth*rhalf);
      }
   }
}
// # BUTTERFLY, INTERIOR ADJACENT TO EXTRAORDINARY CREASE
inline void _SSsStencil::_excreaseStencilButterfly(stencil* rpstencil, const unsigned rvIdx, const unsigned rridx,
                                                   const ring* rporings, const float rhalf,
                                                   const vector<unsigned>* rpLim)
{
   float weigth, theta_k;
   unsigned k = (*rpLim)[2] - 1;
   unsigned i = k - (*rpLim)[3];
   // rules for neighbors of extraordinary crease
   theta_k = (float)M_PI/(k - 1);
   weigth = -(float)sin(theta_k)*(float)sin(i*theta_k);
   weigth /= (1.F - (float)cos(theta_k))*(k - 1);
   weigth += 1.F;
   rpstencil->push_back(rvIdx, weigth*rhalf);
   weigth = -(float)sin(2*theta_k)*(float)sin(2*theta_k*i);
   weigth /= ((float)cos(theta_k) - (float)cos(2*theta_k))*4.F*(k - 1);
   weigth += (float)cos(i*theta_k)/4.F;
   rpstencil->push_back((*rporings)[(*rpLim)[0]], weigth*rhalf);
   rpstencil->push_back((*rporings)[(*rpLim)[1]], weigth*rhalf);
   rporings->setCurrRidx((*rpLim)[0]);
   for (unsigned j = 1 ; j < (*rpLim)[2] - 1 ; j++)
   {
      weigth = (float)sin(i*theta_k)*(float)sin(j*theta_k);
      weigth += (float)sin(2*i*theta_k)*(float)sin(2*j*theta_k)/2;
      weigth /= k;
      rpstencil->push_back(rporings->getNext(), weigth*rhalf);
   }
}