/***********************************************************************
This software module was originally developed by Yannick Maret (from the
Ecole Polytechnique Federale de Lausanne, EPFL, during a stay at the
Universidad Politecnica de Madrid, UPM) in the course of development of 
the MPEG-4 Systems (ISO/IEC 14496-1) standard. This software module is
an implementation of a part of one or more MPEG-4 Systems 
(ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems
(ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 Systems 
(ISO/IEC 14496-1) free license to this software module or modifications
thereof for use in hardware or software products claiming conformance to
the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use this 
software module in hardware or software products are advised that its 
use may infringe existing patents. The original developer of this 
software module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software module
or modifications thereof in an implementation. Copyright is not 
released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
EPFL and UPM retain full right to use the code for his/her own purpose,
assign or donate the code to a third party and to inhibit third parties 
from using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
products. This copyright notice must be included in all copies or 
derivative works. Copyright  2003-2005
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yannick Maret at Yannick.Maret@a3.epfl.ch or Francisco
Moran at fmb@gti.ssr.upm.es.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Player3d
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  WaveSurfDecoder
FILE:       _SSsStencil.h
AUTHOR:     Yannick Maret (EPFL, UPM)
OVERVIEW:   Interface of the subdivision surfaces stencils.
-------------------------------------------------------------------------*/

#ifndef _SUBDIVISION_SURFACE_MASKS_INCLUDED_
#define _SUBDIVISION_SURFACE_MASKS_INCLUDED_
// *** INCLUDES ***
#include "_SSsTag.h"
// *** MASK ***
struct tStencilElem
{
   float weight;
   unsigned idx;
};
class stencil: public vector<tStencilElem>
{
public:
   void push_back(const unsigned, const float);
};
inline void stencil::push_back(const unsigned ridx, const float rweight)
{
   static tStencilElem elem;
   elem.idx = ridx;
   elem.weight = rweight;
   vector<tStencilElem>::push_back(elem);
}
// *** BETA MODE ***
enum etBetaMode {LOOP_BETA, WARREN_LOOP_BETA};
// *** MASK TYPE ***
enum etSSsType {MID_POINT = 0, LOOP = 1, BUTTERFLY = 2, CATMULL_CLARK = 3};
// *** SSs MASK ***
class _SSsStencil: public _SSsTag
{
public:
   // data access
   const vector<vector<stencil> >* pgetStencilsLUT();
   const vector<vector<stencil> >* pgetTriStenLUT(); //MIO
   void buildStencilsLUT(const unsigned, const enum etSSsType,
                         const unsigned = 0, const bool = false);
   bool isInterpolating() const;
   void buildTriStenLUT(unsigned);
private:
   // # PRIVATE DATA & FUNCTIONS
   // stencil creation and gestion
   void _evenVertStencilLoop(stencil*, const unsigned, const ring*, const unsigned);
   void _oddVertStencilLoop(stencil*, const unsigned, const unsigned, const ring*);
   void _evenVertStencilLoopHp(stencil*, const unsigned, const ring*, const unsigned);
   void _oddVertStencilLoopHp(stencil*, const unsigned, const unsigned, const ring*);
   void _stencilButterfly(stencil*, const unsigned, const unsigned,
                          const vector<ring>*, const bool = false);
   void _exinternStencilButterfly(stencil*, const unsigned, const unsigned,
                                  const ring*, const float);
   void _excreaseStencilButterfly(stencil*, const unsigned, const unsigned,
                                  const ring*, const float, const vector<unsigned>*);
   vector<vector<stencil> > _stencilLpLUT;
   vector<vector<stencil> > _stencilHpLUT;
   vector<vector<stencil> > _triStenLUT; //MIO
   bool _interpolating;
   unsigned _currSubLevel;
   // timing
#if _TIME_SSLIB_
   clock_t _startTime, _endTime, _startTimeStencils, _endTimeStencils;
   clock_t _evenTime, _oddTime, _stencilTime;
   void _printTiming();
#endif
};
#endif//ndef _SUBDIVISION_SURFACE_MASKS_INCLUDED_
