/***********************************************************************
This software module was originally developed by Yannick Maret (from the
Ecole Polytechnique Federale de Lausanne, EPFL, during a stay at the
Universidad Politecnica de Madrid, UPM) in the course of development of 
the MPEG-4 Systems (ISO/IEC 14496-1) standard. This software module is
an implementation of a part of one or more MPEG-4 Systems 
(ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems
(ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 Systems 
(ISO/IEC 14496-1) free license to this software module or modifications
thereof for use in hardware or software products claiming conformance to
the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use this 
software module in hardware or software products are advised that its 
use may infringe existing patents. The original developer of this 
software module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software module
or modifications thereof in an implementation. Copyright is not 
released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
EPFL and UPM retain full right to use the code for his/her own purpose,
assign or donate the code to a third party and to inhibit third parties 
from using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
products. This copyright notice must be included in all copies or 
derivative works. Copyright  2003-2005
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yannick Maret at Yannick.Maret@a3.epfl.ch or Francisco
Moran at fmb@gti.ssr.upm.es.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Player3d
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  WaveSurfDecoder
FILE:       _SSsTag.cpp
AUTHOR:     Yannick Maret (EPFL, UPM)
OVERVIEW:   Implementation of tagged subdivision surfaces.
-------------------------------------------------------------------------*/

// *** INCLUDES ***
#include "_SSsTag.h"
// *** CONSTRUCTION/DESTRUCTION ***
_SSsTag::_SSsTag():
_pbaseCreaseEdgeId(NULL),
_pbaseDartVertId(NULL),
_pbaseCreaseVertId(NULL),
_pbaseCornerVertId(NULL)
#if _TIME_SSLIB_
   ,
   _tagsTime(0)
#endif
{
}
// *** CREATE THE TAGS TABLES ***
void _SSsTag::createTagsTables(const unsigned rNbSub)
{
#if _TIME_SSLIB_
   _startTime = clock();
#endif
   // *** STACK VARIABLES
   unsigned ring_size, i, j;
   unsigned new_nb_vert;
   unsigned nb_vert;
   vector<unsigned>::const_iterator i_new_ring;
   vector<char>::iterator i_crease_edge;
   const vector<vector<ring> >* p_all_rings = pgetRingsLUT();
   const unsigned max_nb_vert = pgetNbVertLUT()->back();
   // *** PARSE BASE MESH AND CREATE INITIAL TABLES
   _buildFirstTagTables();
   // *** CREATE CORNER AND DART VERTEX TABLES
   _cornerVert.resize(max_nb_vert, false);
   _dartVert.resize(max_nb_vert, false);
   // *** CREATE CREASE VERTEX AND EDGE TABLES
   _creaseVert.resize(max_nb_vert, false);
   _creaseEdge.resize(max_nb_vert);
   for (unsigned n = 0 ; n < rNbSub - 1 ; n++)
   {
      nb_vert = (*p_all_rings)[n].size();
      new_nb_vert = (*p_all_rings)[n + 1].size();
      // # Crease Vert
      for (i = 0; i < nb_vert ; i++)
      {
         i_new_ring = (*p_all_rings)[n + 1][i].begin();
         i_crease_edge = _creaseEdge[i].begin();
         ring_size = (*p_all_rings)[n + 1][i].size();
         for (j = 0 ; j < ring_size ; j++, i_new_ring++, i_crease_edge++)
            if (*i_new_ring >= (int)i)
               _creaseVert[*i_new_ring] = *i_crease_edge;
      }
      // # Crease Edges
#if _SS_DBG_VERBOSE_TAGS
      cout << "Creases Edges: ";
#endif
      for(i = nb_vert ; i < new_nb_vert ; i++)
      {  
         _creaseEdge[i].resize((*p_all_rings)[n + 1][i].size(), false);
         if (_creaseVert[i])
         {
            for (i_new_ring = (*p_all_rings)[n + 1][i].begin(),
                 i_crease_edge = _creaseEdge[i].begin();
                 i_new_ring < (*p_all_rings)[n + 1][i].end();
                 i_new_ring++, i_crease_edge++)
               if (*i_new_ring < (int)nb_vert)
               {
#if _SS_DBG_VERBOSE_TAGS
                  cout << '(' << i << ',' << *i_new_ring << ')' << endl;
#endif
                  *i_crease_edge = true;
               }
         }
      }
#if _SS_DBG_VERBOSE_TAGS
      _printTags(n + 1);
#endif
   }
/*#if _TIME_SSLIB_
   _endTime = clock();
   _tagsTime += _endTime - _startTime;
   cout << "_SSsTag Timings (in [s]):\n";
   cout << "-------------------------\n";
   cout << "Tags Tables Building  *" << _tagsTime/(float)CLK_TCK << '*' << endl;
#endif*/
}
// *** BUILD THE FIRST TAG TABLES ***
void _SSsTag::_buildFirstTagTables()
{  
   // *** STACK VARIABLES
   vector<int>::const_iterator i_tag;
   const unsigned nb_vert = getNbVertAtLvl(0);
   unsigned i, j;
   // *** CREASE DART & CORNER
   _dartVert.clear();
   _dartVert.resize(nb_vert, false);
   _creaseVert.clear();
   _creaseVert.resize(nb_vert, false);
   _cornerVert.clear();
   _cornerVert.resize(nb_vert, false);
   _creaseEdge.clear();
   _creaseEdge.resize(nb_vert);
   // # CREASE EDGE
   // build crease edge look-up table
   const vector<ring>*  p_rings = pgetRingsAtLvl(0);
   for (i = 0 ; i < nb_vert ; i++)
      _creaseEdge[i].resize((*p_rings)[i].size(), false);
   if (_pbaseCreaseEdgeId != NULL)
      for (i_tag = _pbaseCreaseEdgeId->begin() ; i_tag < _pbaseCreaseEdgeId->end() ; i_tag++)
         if (*i_tag != -1)
         {
            if (*(i_tag - 1) != -1)
            {
               j = (*p_rings)[*i_tag].getPosInRing(*(i_tag - 1));
               _creaseEdge[*i_tag][j] = true;
            }
            if (*(i_tag + 1) != -1)
            {
               j = (*p_rings)[*i_tag].getPosInRing(*(i_tag + 1));
               _creaseEdge[*i_tag][j] = true;
            }
         }
   // insure that boundary edge are tagged as crease
   vector<enum etVertType>::const_iterator i_type;
   for (i = 0, i_type = pgetVertTypeLUT()->begin() ; i < nb_vert ; i++, i_type++)
      if (*i_type == BOUNDARY)
      {
         _creaseEdge[i][0] = _creaseEdge[i][_creaseEdge[i].size() - 1] = true;
         _creaseVert[i] = true;
      }
   // other look-up tables building from crease edge
   vector<unsigned> nb_cedge;
   vector<unsigned>::iterator i_nb_cedge;
   nb_cedge.clear();
   nb_cedge.resize(nb_vert, 0);
   i_nb_cedge = nb_cedge.begin();
   for (i = 0 ; i < nb_vert ; i++, i_nb_cedge++)
      for (j = 0 ; j < _creaseEdge[i].size() ; j++)
         if (_creaseEdge[i][j])
            (*i_nb_cedge)++;
   i_nb_cedge = nb_cedge.begin();
   for (i = 0 ; i < nb_vert ; i++, i_nb_cedge++)
      if (*i_nb_cedge == 1)
         _dartVert[i] = true;
      else if (*i_nb_cedge == 2)
         _creaseVert[i] = true;
      else if (*i_nb_cedge >= 3)
         _cornerVert[i] = true;
   // (if not empty the following fields override the previous look-up tables)
   // # CREASE VERTEX
   // look-up table building
   if (_pbaseCreaseVertId != NULL)
      for (i_tag = _pbaseCreaseVertId->begin() ; i_tag < _pbaseCreaseVertId->end() ; i_tag++)
      {
         _creaseVert[*i_tag] = true;
         _dartVert[*i_tag] = false;
         _cornerVert[*i_tag] = false;
      }
   // # DART VERTEX
   // look-up table building
   if (_pbaseDartVertId != NULL)
      for (i_tag = _pbaseDartVertId->begin() ; i_tag < _pbaseDartVertId->end() ; i_tag++)
      {
         _creaseVert[*i_tag] = false;
         _dartVert[*i_tag] = true;
         _cornerVert[*i_tag] = false;
      }
   // # CORNER VERTEX
   // look-up table building
   if (_pbaseCornerVertId != NULL)
      for (i_tag = _pbaseCornerVertId->begin() ; i_tag < _pbaseCornerVertId->end() ; i_tag++)
      {
         _creaseVert[*i_tag] = false;
         _dartVert[*i_tag] = false;
         _cornerVert[*i_tag] = true;
      }
#if _SS_DBG_VERBOSE_TAGS
   _printTags(0);
#endif
}
// *** GET THE CREASE DEGREE OF A SECTOR ***
unsigned _SSsTag::getCreaseDegree(const unsigned rlvl, const unsigned rtagVidx, 
                                  const unsigned rotherVidx) const
{
   // *** STACK VARIABLES
   const vector<unsigned> crease_limits = getSectorLimits(rlvl, rtagVidx, rotherVidx);
   // *** RETURN RESULT
   return crease_limits[2];
}
// *** GET THE SECTOR LIMITS (VERTEX) ***
vector<unsigned> _SSsTag::getSectorLimits(const unsigned rlvl, const unsigned rcreaseVidx, 
                                          const unsigned rotherVidx) const
{
   // *** STACK VARIABLES
   const vector<unsigned>* p_oring = &(*pgetRingsAtLvl(rlvl))[rcreaseVidx];
   const unsigned ring_size = p_oring->size();
   unsigned i, v1_ridx, v0_ridx;
   bool right_sector, found_1;
   vector<unsigned> ret_val(3); // [firstId, secondId, CreaseDegree]
   // *** GET SECTOR LIMITS
   v1_ridx = 0;
   right_sector = false;
   found_1 = false;
   for (i = 0 ; i < ring_size ; i++)
   {
      if (_creaseEdge[rcreaseVidx][i])
      {
         v0_ridx = v1_ridx;
         v1_ridx = i;
         if (right_sector && !found_1)
            break;
         found_1 = true;
      }
      if ((int)rotherVidx == (*p_oring)[i])
         right_sector = true;
   }
   if (!found_1)
   {
      ret_val[2] = v1_ridx + ring_size - i;
      i = ring_size - 1;
      while (!_creaseEdge[rcreaseVidx][i])
         i--;
      ret_val[1] = (*p_oring)[i];
   }
   else
      if (i == ring_size)
      {
         ret_val[2] = v1_ridx;
         ret_val[1] = p_oring->front();
      }
      else
      {
         ret_val[2] = v1_ridx - v0_ridx;
         ret_val[1] = (*p_oring)[v0_ridx];
      }
   ret_val[0] = (*p_oring)[v1_ridx];
   return ret_val;
}
// *** GET THE SECTOR LIMITS (RING IDX) ***
void _SSsTag::getCrLimRidx(const unsigned rlvl, const unsigned rcrVidx, 
                           const unsigned rridx, vector<unsigned>& rret) const
{
   // *** STACK VARIABLES
   const ring* p_rg = pgetRingsAtLvl(rlvl, rcrVidx);
   rret.resize(4);
   rret[2] = 1;
   p_rg->setCurrRidx(rridx);
   while (!_creaseEdge[rcrVidx][rret[0] = p_rg->getPrevRidx()])
      rret[2]++;
   rret[3] = rret[2];
   p_rg->setCurrRidx(rridx);
   while (!_creaseEdge[rcrVidx][rret[1] = p_rg->getNextRidx()])
      rret[2]++;
   rret[2] += 2;
}
// *** DEBUGING FUNCTIONS ***
#if _SS_DBG_VERBOSE_TAGS
void _SSsTag::_printTags(const unsigned rlvl)
{
      unsigned j;
      cout << "        ";
      for (j = 0 ; j < 10 ; j++)
         cout << j << "  ";
      cout << '\n';
      cout << "crease :";
      unsigned nb_vert = getNbVertAtLvl(rlvl);
      for (j = 0 ; j < nb_vert ; j++)
      {
         cout << (_creaseVert[j] == 1) << "  ";
         if ((j + 1)%10 == 0)
            cout << "\n        ";
      }
      cout << '\n';
      cout << "corner :";
      for (j = 0 ; j < nb_vert ; j++)
      {
         cout << (_cornerVert[j] == 1) << "  ";
         if ((j + 1)%10 == 0)
            cout << "\n        ";
      }
      cout << '\n';
      cout << "dart   :";
      for (j = 0 ; j < nb_vert ; j++)
      {
         cout << (_dartVert[j] == 1) << "  ";
         if ((j + 1)%10 == 0)
            cout << "\n        ";
      }
      cout << '\n';
}
#endif
