/***********************************************************************
This software module was originally developed by Yannick Maret (from the
Ecole Polytechnique Federale de Lausanne, EPFL, during a stay at the
Universidad Politecnica de Madrid, UPM) in the course of development of 
the MPEG-4 Systems (ISO/IEC 14496-1) standard. This software module is
an implementation of a part of one or more MPEG-4 Systems 
(ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems
(ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 Systems 
(ISO/IEC 14496-1) free license to this software module or modifications
thereof for use in hardware or software products claiming conformance to
the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use this 
software module in hardware or software products are advised that its 
use may infringe existing patents. The original developer of this 
software module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software module
or modifications thereof in an implementation. Copyright is not 
released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
EPFL and UPM retain full right to use the code for his/her own purpose,
assign or donate the code to a third party and to inhibit third parties 
from using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
products. This copyright notice must be included in all copies or 
derivative works. Copyright  2003-2005
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yannick Maret at Yannick.Maret@a3.epfl.ch or Francisco
Moran at fmb@gti.ssr.upm.es.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Player3d
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  WaveSurfDecoder
FILE:       _SSsTag.h
AUTHOR:     Yannick Maret (EPFL, UPM)
OVERVIEW:   Interface of the tagged subdivision surfaces.
-------------------------------------------------------------------------*/

#ifndef _TAGGED_SUBDIVISION_SURFACE_INCLUDED_
#define _TAGGED_SUBDIVISION_SURFACE_INCLUDED_
// *** INCLUDE THE BASE CLASS ***
#include "_SSsTopo.h"
// *** TAGGED SSs CLASS ***
class _SSsTag: public _SSsTopo
{
public:
   // # PUBLIC INTERFACE
   // destruction & construction
   _SSsTag();
   // data setting
   void psetBaseCreaseEdgeId(const vector<int>*);
   void psetBaseCornerVertId(const vector<int>*);
   void psetBaseCreaseVertId(const vector<int>*);
   void psetBaseDartVertId(const vector<int>*);
   // main function
   void createTagsTables(const unsigned);
   // data access
   char isCreaseEdge(const unsigned, const unsigned) const;
   char isCreaseVert(const unsigned) const;
   char isDartVert(const unsigned) const;
   char isCornerVert(const unsigned) const;
   // functions
   vector<unsigned> getSectorLimits(const unsigned, const unsigned, const unsigned) const;
   unsigned getCreaseDegree(const unsigned, const unsigned, const unsigned) const;
   void getCrLimRidx(const unsigned, const unsigned, const unsigned, vector<unsigned>&) const;
private:
   // # PRIVATE DATA & FUNCTIONS
   // trick: bool vector element are stored in a bit
   // -----  and access is done trough a proxy
   //        --> very slow to access
   vector<vector<char> > _creaseEdge;
   vector<char> _dartVert, _creaseVert, _cornerVert;
   const vector<int>* _pbaseCreaseEdgeId;
   const vector<int>* _pbaseDartVertId;
   const vector<int>* _pbaseCreaseVertId;
   const vector<int>* _pbaseCornerVertId;
   //  functions
   void _buildFirstTagTables();
   // debug & timmings
#if _DEBUG_SSLIB_ && _SS_DBG_VERBOSE_TAGS
   void _printTags(const unsigned);
#endif
#if _TIME_SSLIB_
   clock_t _startTime, _endTime;
   clock_t _tagsTime;
#endif
};
// *** INLINED FUNCTIONS DEFINITION
inline void _SSsTag::psetBaseCreaseEdgeId(const vector<int>* rp)
{
   _pbaseCreaseEdgeId = rp;
}
inline void _SSsTag::psetBaseCornerVertId(const vector<int>* rp)
{
   _pbaseCornerVertId = rp;
}
inline void _SSsTag::psetBaseCreaseVertId(const vector<int>* rp)
{
   _pbaseCreaseVertId = rp;
}
inline void _SSsTag::psetBaseDartVertId(const vector<int>* rp)
{
   _pbaseDartVertId = rp;
}
inline char _SSsTag::isCreaseEdge(const unsigned rvidx, const unsigned rridx) const
{
   return _creaseEdge[rvidx][rridx];
}
inline char _SSsTag::isCreaseVert(const unsigned r) const
{
   return _creaseVert[r];
}
inline char _SSsTag::isDartVert(const unsigned r) const
{
   return _dartVert[r];
}
inline char _SSsTag::isCornerVert(const unsigned r) const
{
   return _cornerVert[r];
}
#endif//ndef _TAGGED_SUBDIVISION_SURFACE_INCLUDED_
