#ifndef _G723COMPRESS_H_
#define _G723COMPRESS_H_

#include <limits.h>
#include "debug.h"
#include "cmpmain.h"
#include "generalcompressor.h"

#define	G723_BYTES_IN_BLOCK	       							480
#define G723_SAMPLING_RATE_8K                   8000
#define DECODE_ERROR                            SHRT_MAX
#define G723_BLOCKS_IN_PACKET                   2
#define G723_53                                 0
#define G723_63                                 1 
#define G723_63_SIZE                            12  
#define G723_53_SIZE                            10
#define G723_AVERAGE_SAMPLING_RATE              800

class CG723Compress : public CGeneralCompressor
{
	public:
		CG723Compress(HINSTANCE, DWORD);
		virtual ~CG723Compress();

	private:
			VDOWAVEFORMATEX				m_wfxOutFormat; 
			BYTE									*m_lpDstBuf;
			DWORD									m_dwBufferSize;
			HINSTANCE							m_hInst;
      DWORD                 m_dwType;
      DWORD                 m_dwDstBufSize;
      BOOL                  m_bPrevVoxState;
      BOOL                  m_bDisabled;
      BOOL                  m_bMute;
      DWORD                 m_dwLastAvg;
			BOOL									m_bSendCng;

		public:
			DWORD Compress( short *lpData, DWORD dwBufSize, BOOL *bSend, DWORD *pdwNewState);
			VDOWAVEFORMATEX *Init();
			BOOL	StartCompression();
		  void	SetLevels(int ActivateLevel)        {                     }
      DWORD GetLevels(void)                     { return 0;           }
		  DWORD	GetCount(void)                      { return m_dwLastAvg; }
      void  VoxDisable(BOOL bFlag)              { m_bDisabled = bFlag;}                        
      void  Mute(BOOL bFlag)                    { m_bMute = bFlag;    }                    
      void  SetMode(DWORD dwMode)               {                     }
      DWORD GetCompressedBufferSize()           { return G723_BLOCKS_IN_PACKET*2*((m_dwType==G723_53)?G723_53_SIZE:G723_63_SIZE); }
	    DWORD GetUncompressedBufferSize()         { return G723_BYTES_IN_BLOCK*G723_BLOCKS_IN_PACKET; }
      BOOL  IsMute()                            { return m_bMute; }    


    private:
      DWORD	FrameCompress(short *lpData, BYTE *lpDst);
      DWORD Average(char *lpData, DWORD Size);
      void  CalcLastAvg(DWORD Avg);

		public:
      DWORD GetKbps()												{return 7;}


		public:
			DWORD GetAlignSize()			{return G723_BYTES_IN_BLOCK*G723_BLOCKS_IN_PACKET;}
};

class CG723Decompress : public CGeneralDecompressor
{
  private:
		VDOWAVEFORMATEX	m_wfxOutFormat; 
		BYTE						*m_lpDstBuf;
		short						m_sTreble;
		HINSTANCE				m_hInst;
    DWORD           m_dwType;
		BOOL						m_bGenerateComfortNoise;
		BYTE						m_aTypeVec[G723_BLOCKS_IN_PACKET];
		int							m_nInedx;

	public:
		CG723Decompress(HINSTANCE, DWORD);
		virtual ~CG723Decompress();

	public:
			BOOL	StartDecompression(VDOWAVEFORMATEX *wfx);
			DWORD Decompress(short *lpOutput, BYTE *lpInput, DWORD dwBufSize, DWORD *pdwOutputSize, BOOL bUseAES);
			BOOL	Init();
      DWORD GetCompressedBufferSize(DWORD dwBuffLen)										{ return 2*((m_dwType==G723_53)?G723_53_SIZE:G723_63_SIZE); }
	    DWORD GetUncompressedBufferSize(char *pBuffer, DWORD dwBuffLen)   { return 1920; }
			void	GenerateComfortNoise(char *pBuffer, DWORD dwSize);
			BOOL  IsTalkingIntervalEnded()																		{ return ( (m_aTypeVec[G723_BLOCKS_IN_PACKET-1] == 2) || (m_aTypeVec[G723_BLOCKS_IN_PACKET-1] == 3) ); }
			BOOL	IsSpeechBuffer();

};
#endif //_G723COMPRESS_H_