/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(VDOnet Corp.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. VDOnet Corp. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@vdo.net or zvil@compuserve.com.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Decoder
FILE:       G723GDec.cpp
AUTHOR:     Zvi Lifshitz (VDOnet Corp.)
OVERVIEW:   Implementation of G723Decoder (simulation - just copies
			the units)
-------------------------------------------------------------------------*/
//#include "Defs.h"
#include "defs.h" // SOLARIS PORTING - typo
#include <assert.h>
#include <limits.h>
#include "G723Dec.h"

/*-------------------------------------------------------------
The exported function that instantiates the proper decoder
-------------------------------------------------------------*/
void *CreateInstance ()
{
	return new G723Decoder;
}

/*-----------------------------------------------------------------
Find a compatible format and initialize the decoder
-----------------------------------------------------------------*/
/*
BOOL G723Decoder :: SetFormat (
	Capabilities &formatOptions,
	DecoderParams &decoderParams
)
{
	int i;

	for (i = 0; i < formatOptions.GetFormatCapabilityLength (); i++)
		if (formatOptions.GetFormatCapability (i) == MediaFormat :: PCM)
		{	
			decoderParams.m_nStreamNumber = 0;
			decoderParams.m_format = MediaFormat (MediaFormat :: PCM);
			decoderParams.m_nOptimalAUSize = 24;
			decoderParams.m_nDBSize = 256;
			decoderParams.m_nCBSize = GetOptimalOutputSize ()*20;
			decoderParams.m_bRequiresFullAUs = TRUE;
			INITdec (G723_53);
			return TRUE;
		}
	return FALSE;
} */

//PAB: this should have been updated according to Zvi's comments to:

BOOL G723Decoder :: SetFormat (
	Capabilities &formatOptions,
	DecoderParams &decoderParams
)
{
	int i;
	
	decoderParams.m_nOptimalAUSize = 24; //PAB_core50RC1+:
	decoderParams.m_nDBSize = 256; //PAB_core50RC1+:
	decoderParams.m_nCBSize = GetOptimalOutputSize ()*20; //PAB_core50RC1+:
	decoderParams.m_bRequiresFullAUs = TRUE; //PAB_core50RC1+:

	for (i = 0; i < formatOptions.GetFormatCapabilityLength (); i++)
		if (formatOptions.GetFormatCapability (i) == MediaFormat :: PCM)
		{	
			decoderParams.m_nStreamNumber = 0;
			decoderParams.m_format = MediaFormat (MediaFormat :: PCM);
//PAB_core50RC1-:			decoderParams.m_nOptimalAUSize = 24;
//PAB_core50RC1-:			decoderParams.m_nDBSize = 256;
//PAB_core50RC1-:			decoderParams.m_nCBSize = GetOptimalOutputSize ()*20;
//PAB_core50RC1-:			decoderParams.m_bRequiresFullAUs = TRUE;
// SOLARIS PORTING - Changed the following
#ifdef WIN32
			INITdec (G723_53);
#else
//			G723InitDec();
#endif
			return TRUE;
		}
	return FALSE;
}		
/*-----------------------------------------------------------------
Decompress a unit.
-----------------------------------------------------------------*/
BOOL G723Decoder :: Decode (
		LPBYTE pInput,
		int /*nInputLength*/,
		LPBYTE pOutput,
		int &rnOutputLength
)
{
	int nError;

	if (rnOutputLength < GetOptimalOutputSize ())
	{	rnOutputLength = GetOptimalOutputSize ();
		return FALSE;
	}
	
// SOLARIS PORTING
#ifdef WIN32
	nError = DecSoundFrame (pInput, pOutput);
	assert (nError != DECODE_ERROR);
	rnOutputLength = GetOptimalOutputSize ();
#else
	if( !G723Dec((char *)pOutput, (char *)pInput) )
	{
		COMPLAIN("couldn't decode G.723 frame\n");
		return FALSE;
	}

	rnOutputLength = 480;
	TRACE("G723Class::Decode: pInput %08x, pOutput %08x \n", pInput, pOutput);
#endif
	return TRUE;
}

/*-----------------------------------------------------------------
Run() retrieves units from the input stream, decode them, and dispatch
units to the output stream, till the input stream's end is exceeded.
-----------------------------------------------------------------*/
void G723Decoder :: Run ()
{
// SOLARIS PORTING
#ifdef WIN32
	SetThreadPriority (GetCurrentThread(), THREAD_PRIORITY_ABOVE_NORMAL);
#else
	Init_Decod();
	Init_Dec_Cng();
	Set_Priority(THREAD_PRIORITY_ABOVE_NORMAL);
#endif
	DecoderImp :: Run ();
}

/*-------------------------------------------------------------
In this decoder, the decoded data always strats at beginning of output
block
-------------------------------------------------------------*/
int G723Decoder :: GetDataOffset  (const LPBYTE)
{
	return 0;
}

/*-------------------------------------------------------------
In this decoder, attributes are fixed for all units
-------------------------------------------------------------*/
DWORD G723Decoder :: GetAttributes (
	ATTRIBUTE_CODE code, 
	const LPBYTE
)
{
	switch (code) {
	case AUDIO_FREQUENCY:
		return 8000;
	case BITS_PER_SAMPLE :
		return 16;
	case NUM_CHANNELS:
		return 1;
	default:
		return 0;
	}
}
