/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(VDOnet Corp.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. VDOnet Corp. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@vdo.net or zvil@compuserve.com.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Decoder
FILE:       H263GDec.cpp
AUTHOR:     Zvi Lifshitz (VDOnet Corp.)
OVERVIEW:   Implementation of H263Decoder (simulation - just copies
			the units)
-------------------------------------------------------------------------*/
#include "Defs.h"
#include "H263Dec.h"
#include "MediaStream.h"

/*-------------------------------------------------------------
The exported function that instantiates the proper decoder
-------------------------------------------------------------*/
void *CreateInstance ()
{
	return new H263Decoder;
}

/*-------------------------------------------------------------
Sets the decoder to CIF or QCIF
-------------------------------------------------------------*/
BOOL H263Decoder :: Setup (
	const BYTE *pSpecificInfo,
	int nSpecificInfoLength,
	int
)
{
	/*-------------------------------------------------------------
	Check if this is QCIF or CIF. QCIF is the default.
	-------------------------------------------------------------*/
	BOOL bCIF = FALSE;
	LPBYTE pSeparator = NULL;
	if (pSpecificInfo)
		pSeparator = (LPBYTE) memchr (pSpecificInfo, ';', nSpecificInfoLength);

	if (pSeparator && !memicmp (pSeparator + 1, "cif", 3))
		bCIF = TRUE;

	m_biDst.biSize = sizeof (BITMAPINFOHEADER);
	m_biDst.biWidth = 176 * (1 + bCIF); 
	m_biDst.biHeight = 144 * (1 + bCIF);
	m_biDst.biPlanes = 1; //always 
	m_biDst.biBitCount = 24; 
	m_biDst.biCompression = BI_RGB; //RGB_FORMAT  
	m_biDst.biSizeImage =
		m_biDst.biWidth * m_biDst.biHeight * m_biDst.biBitCount / 8; 

	m_biSrc.biSize = sizeof (BITMAPINFOHEADER);
	m_biSrc.biWidth = m_biDst.biWidth; 
	m_biSrc.biHeight = m_biDst.biHeight; 
	m_biSrc.biPlanes = 1; 
	m_biSrc.biBitCount = 24; 
	m_biSrc.biCompression = FOURCC_H263; 

	m_decodeObject.m_DecodeInst.DecFrameBuffer = NULL;
	m_decodeObject.m_DecodeInst.VDODecBuffer = NULL;
	m_decodeObject.m_RGBLow = NULL;
	m_decodeObject.m_InputFourcc = 0;
	m_decodeObject.m_OutputFourcc = 0;
	m_decodeObject.m_DoLowRes = 0;
	m_decodeObject.m_HurryUp = FALSE;
	m_decodeObject.m_pCompressedData = (LPBYTE) &m_biSrc;	// just to make it
															// non-null.
	return TRUE;
}

/*-----------------------------------------------------------------
Find a compatible format and initialize the decoder
-----------------------------------------------------------------*/
BOOL H263Decoder :: SetFormat (
	Capabilities &formatOptions,
	DecoderParams &decoderParams
)
{

//PAB: this should have been updated according to Zvi's comments to:

	decoderParams.m_nOptimalAUSize = 256; //PAB_core50RC1+:
	decoderParams.m_nDBSize = 10000; //PAB_core50RC1+:
	decoderParams.m_nCBSize = GetOptimalOutputSize () * 5; //PAB_core50RC1+:
	decoderParams.m_bRequiresFullAUs = TRUE; //PAB_core50RC1+:

   /*-------------------------------------------------------------
	If the preferred format is YUV, set the precise format descriptor
	in the BITMAPINFOHEADER structure
	-------------------------------------------------------------*/
	BOOL bResult = FALSE;
	int i;

	for (i = 0; !bResult; i++)
	{	if (i >= formatOptions.GetFormatCapabilityLength ())
			return FALSE;
		switch (formatOptions.GetFormatCapability (i).m_code) {
		case MediaFormat :: YUV:
			m_biDst.biCompression =
				formatOptions.GetFormatCapability (i).m_dwAdditionalInfo;
			bResult = TRUE;
			break;
		case MediaFormat :: RGB:
			bResult = TRUE;
			break;
		}
	}

	decoderParams.m_nStreamNumber = 0;
	decoderParams.m_format = MediaFormat :: BGR; 

	if(!InitDecodedObject (&m_biSrc, &m_biDst, &m_decodeObject))
		return FALSE;

	return TRUE;
}



/*-----------------------------------------------------------------
Calculate the output size in bytes
-----------------------------------------------------------------*/
int H263Decoder :: GetOptimalOutputSize ()
{
	return m_biDst.biWidth * m_biDst.biHeight * 3 + 1;
}

/*-------------------------------------------------------------
Decode one unit
-------------------------------------------------------------*/
BOOL H263Decoder :: Decode (
		LPBYTE pInput,
		int nInputLength,
		LPBYTE pOutput,
		int &rnOutputLength
)
{
	int nError;
    
	if (rnOutputLength < GetOptimalOutputSize ())
	{	rnOutputLength = GetOptimalOutputSize ();
		return FALSE;
	}

	m_decodeObject.m_pCompressedData = pInput;
	m_biSrc.biSizeImage = nInputLength;
	nError = DecodeFrame (
		&m_biSrc,
		&m_biDst,
		tempOutput,
		&m_decodeObject,
		FALSE,
		TRUE,
		FALSE,
		1
	);
	rnOutputLength = nError == 0 ? GetOptimalOutputSize () : 0;
	
	if (rnOutputLength > 0)
		memcpy(pOutput, tempOutput, rnOutputLength );
	return TRUE;
}

/*-------------------------------------------------------------
In this decoder, the decoded data always strats at beginning of output
block
-------------------------------------------------------------*/
int H263Decoder :: GetDataOffset  (const LPBYTE)
{
	return 0;
}

/*-------------------------------------------------------------
In this decoder, attributes are fixed for all units
-------------------------------------------------------------*/
DWORD H263Decoder :: GetAttributes (
	ATTRIBUTE_CODE code, 
	const LPBYTE
)
{
	switch (code) {
	case FRAME_WIDTH:
		return m_biDst.biWidth;
	case FRAME_HEIGHT:
		return m_biDst.biHeight;
	case PITCH:
		return 3*m_biDst.biWidth;
	case OFFSETX:
		return 0;
	case OFFSETY:
		return 0;
	default:
		return 0;
	}
}
void H263Decoder :: Run ()
{
/* //Use the DecoderImp :: Run() instead
	LPBYTE pInData, pOutData;
	int nInSize, nOutSize;
	DWORD dwTime;
	BOOL bEOU;

	while (
		(pInData =
			(LPBYTE) GetInputStream () -> FetchNext (nInSize, dwTime, bEOU, TRUE)
		) !=
		END_OF_STREAM
	)
	{
		if (IsActive ())
		{	nOutSize = GetOptimalOutputSize (),
			pOutData = (LPBYTE) GetOutputStream () -> Allocate (nOutSize, TRUE, FALSE);
			if (pOutData == NULL)
				throw;
			if (pOutData == END_OF_STREAM)
				return;
			while (!Decode (pInData, nInSize, pOutData, nOutSize))
			{	pOutData = (LPBYTE) GetOutputStream () -> Reallocate (nOutSize);
				if (pOutData == NULL)
					throw;
				if (pOutData == END_OF_STREAM)
					return;
			}
			GetOutputStream () -> Dispatch (nOutSize, dwTime);
		}
		GetInputStream () -> Release (TRUE);
	}
	m_pOutStream -> Close ();
*/
  SetThreadPriority (GetCurrentThread(), THREAD_PRIORITY_ABOVE_NORMAL);
  DecoderImp::Run();
}