/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Decoder
FILE:       H263PrevGDec.cpp
AUTHOR:     Karl Anders ygard (Telenor R&D)
OVERVIEW:   Implementation of H263PrevDecoder
-------------------------------------------------------------------------*/
//GL: compile using multithreaded run time library, not multithreadead dll.

#include "defs.h"
//#include <assert.h>
#include "H263Dec-Karl.h"
#include "H263Decoder/bitio.h"

/*-------------------------------------------------------------
The exported function that instantiates the proper decoder
-------------------------------------------------------------*/
void *CreateInstance ()
{
	return new H263Decoder;
}

/*-----------------------------------------------------------------
Find a compatible format and initialize the decoder
-----------------------------------------------------------------*/
BOOL H263Decoder :: SetFormat (
	Capabilities &formatOptions,
	DecoderParams &decoderParams
)
{
// 	fOut = fopen ("test.rgb", "wb");
//	fIn = fopen("h263-in.dat", "wb");
   	

	decoderParams.m_nStreamNumber = 0;
	decoderParams.m_format = MediaFormat :: YUV;
	
	decoderParams.m_nOptimalAUSize = 256;
	decoderParams.m_nDBSize = 10000;
	decoderParams.m_nCBSize = GetOptimalOutputSize () * 5;
	decoderParams.m_bRequiresFullAUs = TRUE;

	
	return TRUE;
}


/*-----------------------------------------------------------------
Calculate the output size in bytes
-----------------------------------------------------------------*/
int H263Decoder :: GetOptimalOutputSize ()
{
	return 176 * 144 * 3 / 2 ;
}

/*-------------------------------------------------------------
In this decoder, the decoded data always strats at beginning of output
block
-------------------------------------------------------------*/
int H263Decoder :: GetDataOffset  (const LPBYTE)
{
	return 0;
}
/*-------------------------------------------------------------
In this decoder, attributes are fixed for all units
-------------------------------------------------------------*/
DWORD H263Decoder :: GetAttributes (
	ATTRIBUTE_CODE code, 
	const LPBYTE
)
{
	switch (code) {
	case FRAME_WIDTH:
		return 176;
	case FRAME_HEIGHT:
		return 144;
	case PITCH:
		return 176;
	case OFFSETX:
		return 0;
	case OFFSETY:
		return 0;
	default:
		return 0;
	}
}
BOOL H263Decoder :: Decode (LPBYTE pInput, int nInputLength,
							LPBYTE pOutput, int &rnOutputLength)
{
	// Decode the next frame
	bitiob inbuf((unsigned char*) pInput, 0, nInputLength);
	unsigned char *frame[3];
	m_decoder.Decode(inbuf, frame);

	// If there is not enough buffer space, return FALSE
	int frameSize = m_decoder.horizontal_size * m_decoder.vertical_size;
/*	if (rnOutputLength < frameSize * 3 / 2 + (int)sizeof(DecoderImage))
	{
		rnOutputLength = frameSize * 3 / 2 + (int)sizeof(DecoderImage);
		return FALSE;
	}*/

	// Store the dimensions of the frame in the output buffer
	/*DecoderImage* pImgDetails = (DecoderImage*)pOutput;
	pImgDetails->width   = m_decoder.horizontal_size;
	pImgDetails->height  = m_decoder.vertical_size;
	pImgDetails->pitch   = m_decoder.horizontal_size;
	pImgDetails->offsetX = 0;
	pImgDetails->offsetY = 0;*/

	// Copy the output of the decoder into the output buffer
	
	memcpy(pOutput, frame[0], frameSize);
	memcpy(pOutput + frameSize, frame[1], frameSize / 4);
	memcpy(pOutput + frameSize * 5 / 4, frame[2], frameSize / 4);

	rnOutputLength = frameSize * 3 / 2 ;
	return TRUE;
}


