/************************************************************************
 *
 *  global.h, global variables for tmndecode (H.263 decoder)
 *  Copyright (C) 1995, 1996  Telenor R&D, Norway
 *        Karl Olav Lillevold <Karl.Lillevold@nta.no>
 *  
 *  Contacts: 
 *  Karl Olav Lillevold               <Karl.Lillevold@nta.no>, or
 *  Robert Danielsen                  <Robert.Danielsen@nta.no>
 *
 *  Telenor Research and Development  http://www.nta.no/brukere/DVC/
 *  P.O.Box 83                        tel.:   +47 63 84 84 00
 *  N-2007 Kjeller, Norway            fax.:   +47 63 81 00 76
 *  
 ************************************************************************/

/*
 * Disclaimer of Warranty
 *
 * These software programs are available to the user without any
 * license fee or royalty on an "as is" basis.  Telenor Research and
 * Development disclaims any and all warranties, whether express,
 * implied, or statuary, including any implied warranties or
 * merchantability or of fitness for a particular purpose.  In no
 * event shall the copyright-holder be liable for any incidental,
 * punitive, or consequential damages of any kind whatsoever arising
 * from the use of these programs.
 *
 * This disclaimer of warranty extends to the user of these programs
 * and user's customers, employees, agents, transferees, successors,
 * and assigns.
 *
 * Telenor Research and Development does not represent or warrant that
 * the programs furnished hereunder are free of infringement of any
 * third-party patents.
 *
 * Commercial implementations of H.263, including shareware, are
 * subject to royalty fees to patent holders.  Many of these patents
 * are general enough such that they are unavoidable regardless of
 * implementation design.
 * */


/*
 * based on mpeg2decode, (C) 1994, MPEG Software Simulation Group
 * and mpeg2play, (C) 1994 Stefan Eckart
 *                         <stefan@lis.e-technik.tu-muenchen.de>
 *
 */


/* choose between declaration (GLOBAL undefined)
 * and definition (GLOBAL defined)
 * GLOBAL is defined in exactly one file (tmndec.c)
 */

#include <stdio.h>
#include "bitio.h"
#include "H263.h"

#ifndef GLOBAL
#define EXTERN extern
#else
#define EXTERN
#endif

/* Some macros */
#define mmax(a, b)        ((a) > (b) ? (a) : (b))
#define mmin(a, b)        ((a) < (b) ? (a) : (b))
#define mnint(a)        ((a) < 0 ? (int)(a - 0.5) : (int)(a + 0.5))
#define sign(a)         ((a) < 0 ? -1 : 1)

/* prototypes of global functions */
/* getblk.c */
void getblock(int comp, int mode, bitiob& inbuf, short [][64], H263Dec* dec);
void get_sac_block(int comp, int mode, short [][64], bitiob& inbuf, H263Dec* dec);
/* getpic.c */
void getpicture(bitiob& inbuf, int *framenum, unsigned char *frame[3], H263Dec* dec);
void putlast(int framenum);
//void getpicturehdr(bitiob &inbuf, GlobalVars &glob);

/* getvlc.c */
int getTMNMV(bitiob& inbuf, H263Dec* dec);
int getMCBPC(bitiob& inbuf, H263Dec* dec);
int getMODB(bitiob& inbuf, H263Dec* dec);
int getMCBPCintra(bitiob& inbuf, H263Dec* dec);
int getCBPY(bitiob& inbuf, H263Dec* dec);

/* idct.c */
void idct(short *block);
void init_idct(void);

/* idctref.c */
void init_idctref(void);
void idctref(short *block);

/* tmndec.c */
void error(char *text);
void printbits(int code, int bits, int len);
#ifdef USE_TIME
void doframerate(int pb);
#endif

/* recon.c */
void reconstruct(int bx, int by, int P, int bdx, int bdy, H263Dec* dec);

/* store.c */
void storeframe(unsigned char *src[], int frame, H263Dec* dec);
void conv422to444(unsigned char *src, unsigned char *dst, H263Dec* dec);
void conv420to422(unsigned char *src, unsigned char *dst, H263Dec* dec);

/* sac.c */
int decode_a_symbol(int cumul_freq[ ], bitiob& inbuf, H263Dec* dec);
void decoder_reset(bitiob& inbuf, H263Dec* dec);



