/**********************************************************************
 *
 * Copyright  1995 Telenor Research and Development
 * All rights reserved
 *
 **********************************************************************/
/**********************************************************************
 *
 *	Name:		bitio.cc
 *	Description:	class member functions for classes bitiob
 *			and bitiof
 *
 *	Input:		
 *	Returns:	
 *	Side effects:	
 *
 *	Date: 		28.09.95
 *	Author: 	Robert Inge Danielsen  <rid@nta.no>
 *	Last modified:	20.10.95 merged bitiob.cc and bitiof.cc
 *
 **********************************************************************/

#include <assert.h>
#include <stdlib.h>
#include "bitio.h"
#include "global.h"

const unsigned int bf[33] = {
  0x00000000,0x00000001,0x00000003,0x00000007,
  0x0000000f,0x0000001f,0x0000003f,0x0000007f,
  0x000000ff,0x000001ff,0x000003ff,0x000007ff,
  0x00000fff,0x00001fff,0x00003fff,0x00007fff,
  0x0000ffff,0x0001ffff,0x0003ffff,0x0007ffff,
  0x000fffff,0x001fffff,0x003fffff,0x007fffff,
  0x00ffffff,0x01ffffff,0x03ffffff,0x07ffffff,
  0x0fffffff,0x1fffffff,0x3fffffff,0x7fffffff,
  0xffffffff
};

// buffer i/o

bitiob::bitiob(unsigned char *buffer, int in_or_ou, int bufferle)
{
  if ((in_or_out = in_or_ou) == 0)
    {				// Read from buffer
      buflen = bufferle;
      bufch = buffer;
      bitcount = 0;
      pos = 32;		// 32 bit words
      current = (((unsigned int) bufch[0]) << 24 |
		 ((unsigned int) bufch[1]) << 16 |
		 ((unsigned int) bufch[2]) << 8 |
		  (unsigned int) bufch[3]);
      next = (((unsigned int) bufch[4]) << 24 |
	      ((unsigned int) bufch[5]) << 16 |
	      ((unsigned int) bufch[6]) << 8 |
	       (unsigned int) bufch[7]);
      count = 4;
    }
  else
    {				// Write to buffer
      buflen = bufferle;
      cout << "buflen = " << buflen << " in constructor" << endl;
      bufch = buffer;
      current = 0;
      next = 0;
      pos = 32;		// 32 bit words
      count = 0;
      bitcount = 0;
    }      
}

void
bitiob::init (unsigned char *buffer, int in_or_ou, int bufferle)
{
  if ((in_or_out = in_or_ou) == 0)
    {				// Read from buffer
      buflen = bufferle;
      bufch = buffer;
      bitcount = 0;
      pos = 32;		// 32 bit words
      current = (((unsigned int) bufch[0]) << 24 |
		 ((unsigned int) bufch[1]) << 16 |
		 ((unsigned int) bufch[2]) << 8 |
		  (unsigned int) bufch[3]);
      next = (((unsigned int) bufch[4]) << 24 |
	      ((unsigned int) bufch[5]) << 16 |
	      ((unsigned int) bufch[6]) << 8 |
	       (unsigned int) bufch[7]);
      count = 4;
    }
  else
    {				// Write to buffer
      buflen = bufferle;
      cout << "buflen = " << buflen << " in constructor" << endl;
      bufch = buffer;
      current = 0;
      next = 0;
      pos = 32;		// 32 bit words
      count = 0;
      bitcount = 0;
    }      
}

void
bitiob::reset (int in_or_ou)
{
  in_or_out = in_or_ou;
  if (in_or_out == 0)
    {
      bitcount = 0;
      pos = 32;
      current = (((unsigned int) bufch[0]) << 24 |
		 ((unsigned int) bufch[1]) << 16 |
		 ((unsigned int) bufch[2]) << 8 |
		  (unsigned int) bufch[3]);
      next = (((unsigned int) bufch[4]) << 24 |
	      ((unsigned int) bufch[5]) << 16 |
	      ((unsigned int) bufch[6]) << 8 |
	       (unsigned int) bufch[7]);
      count = 4;
    }
  else
    {
      current = 0;
      next = 0;
      pos = 32;
      count = 0;
      bitcount = 0;
    }
}


unsigned int
bitiob::get(int n)
{
  int diff;
  int word;
  
  if ((diff=pos-n) >= 0)
    {
      word = (current >> diff) & bf[n];
      pos -= n;
    }
  else
    {
      diff = -diff;
      word = ((current << diff) | (next >> 32-diff)) & bf[n];
      pos = 32-diff;
      current = next;
      count+=4;

      if (buflen > count + 4)
	next = (((unsigned int) bufch[count + 0]) << 24 |
		((unsigned int) bufch[count + 1]) << 16 |
		((unsigned int) bufch[count + 2]) << 8 |
		(unsigned int) bufch[count + 3]);
      else {
	next = 0;

	if (buflen > count)
	  switch (buflen - count) {
	  default: next |= (unsigned int) bufch[count + 3];
	  case 3: next |= ((unsigned int) bufch[count + 2]) << 8;
	  case 2: next |= ((unsigned int) bufch[count + 1]) << 16;
	  case 1: next |= ((unsigned int) bufch[count + 0]) << 24;
	    break;
	  }
      }
    }
  bitcount += n;
  return word;
}


unsigned int
bitiob::peek(int n)
{
  int diff;
  int word;
  
  if ((diff=pos-n) >= 0)
    {
      word = (current >> diff) & bf[n];
    }
  else
    {
      diff = -diff;
      word = ((current << diff) | (next >> 32-diff)) & bf[n];
    }
  return word;
}


void
bitiob::put(int n, const unsigned int word)
{
  int diff;

  if ((diff=pos-n) >= 0)
    {
      current |= (word << diff);
      pos -= n;
    }
  else
    {
      diff = -diff;
      current |= (word >> diff);
      next |= (word << (32-diff));
      bufch[count + 3] =         current & 0xff;
      bufch[count + 2] =  (current >> 8) & 0xff;
      bufch[count + 1] = (current >> 16) & 0xff;
      bufch[count + 0] = (current >> 24) & 0xff;
      current = next;
      next = 0;
      pos = 32-diff;
      count+=4;
      if (count >= buflen)
	{
	  cerr << "bitiob::put(): Reached end of output buffer\n";
	}
    }
  bitcount += n;
}


void
bitiob::flush()
{
  bufch[count + 3] =         current & 0xff;
  bufch[count + 2] =  (current >> 8) & 0xff;
  bufch[count + 1] = (current >> 16) & 0xff;
  bufch[count + 0] = (current >> 24) & 0xff;
  
  current = 0;
  count = 0;
  pos = 32;
  if (count >= buflen)
    {
      cerr << "bitiob::flush() Reached end of output buffer\n";
    }
  // Do proper bitcount here
}


int
bitiob::eof()
{
  if (bitcount >= buflen << 3)
    {
      return 1;
    }
  return 0;
}

// file i/o

bitiof::bitiof(char *filename, int writ, int bufle)
{
  if (writ == 0)		
    {				// Infile
      ifile.open (filename);
      assert (ifile != 0);
      buflen = bufle;
      bufch = new unsigned char [buflen];
      bitcount = 0;
      pos = 32;		// 32 bit words
      buflim = buflen;
      end_of_file = 0;
      getbuf();
      current = (((unsigned int) bufch[0]) << 24 |
		 ((unsigned int) bufch[1]) << 16 |
		 ((unsigned int) bufch[2]) << 8 |
		  (unsigned int) bufch[3]);
      next = (((unsigned int) bufch[4]) << 24 |
	      ((unsigned int) bufch[5]) << 16 |
	      ((unsigned int) bufch[6]) << 8 |
	       (unsigned int) bufch[7]);
      count = 4;
    }
  else				
    {				// Outfile
      ofile.open (filename);
      assert(ifile != 0);
      buflen = bufle;
      bufch = new unsigned char [buflen];
      count = 0;
      bitcount = 0;
      pos = 32;		// 32 bit words
      current = 0;
      next = 0;
    }
}


void
bitiof::init (char *filename, int writ, int bufle)
{
  if (writ == 0)		
    {				// Infile
      ifile.open (filename);
      assert (ifile != 0);
      buflen = bufle;
      bufch = new unsigned char [buflen];
      bitcount = 0;
      pos = 32;		// 32 bit words
      buflim = buflen;
      end_of_file = 0;
      getbuf();
      current = (((unsigned int) bufch[0]) << 24 |
		 ((unsigned int) bufch[1]) << 16 |
		 ((unsigned int) bufch[2]) << 8 |
		  (unsigned int) bufch[3]);
      next = (((unsigned int) bufch[4]) << 24 |
	      ((unsigned int) bufch[5]) << 16 |
	      ((unsigned int) bufch[6]) << 8 |
	       (unsigned int) bufch[7]);
      count = 4;
    }
  else				
    {				// Outfile
      ofile.open (filename);
      assert(ifile != 0);
      buflen = bufle;
      bufch = new unsigned char [buflen];
      count = 0;
      bitcount = 0;
      pos = 32;		// 32 bit words
      current = 0;
      next = 0;
    }
}


bitiof::~bitiof()
{
  delete [] bufch;
}


void
bitiof::getbuf ()
{
  int i = 0;
  while (!ifile.eof () && i < buflen)
    {
      ifile.get (bufch[i++]);
    }
  if (i < buflen) buflim = i;
  //  if (i < buflen) buflim = i % 4 ? i / 4 + 1 : i / 4;
}


void
bitiof::get(int n, unsigned int& word)
{
  int diff;
  
  if ((diff=pos-n) >= 0)
    {
      word = (current >> diff) & bf[n];
      pos -= n;
    }
  else
    {
      diff = -diff;
      word = ((current << diff) | (next >> 32-diff)) & bf[n];
      pos = 32-diff;
      current = next;
      count+=4;
      if (count >= buflen)
	{
	  getbuf();
	  count = 0;
	}
      next = (((unsigned int) bufch[count + 0]) << 24 |
	      ((unsigned int) bufch[count + 1]) << 16 |
	      ((unsigned int) bufch[count + 2]) << 8 |
	       (unsigned int) bufch[count + 3]);
    }
  bitcount += n;
}


void
bitiof::peek(int n, unsigned int& word)
{
  int diff;
  
  if ((diff=pos-n) >= 0)
    {
      word = (current >> diff) & bf[n];
    }
  else
    {
      diff = -diff;
      word = ((current << diff) | (next >> 32-diff)) & bf[n];
    }
}


void
bitiof::put(int n, const unsigned int word)
{
  int diff;

  if ((diff=pos-n) >= 0)
    {
      current |= (word << diff);
      pos -= n;
    }
  else
    {
      diff = -diff;
      current |= (word >> diff);
      next |= (word << (32-diff));
      bufch[count + 3] =         current & 0xff;
      bufch[count + 2] =  (current >> 8) & 0xff;
      bufch[count + 1] = (current >> 16) & 0xff;
      bufch[count + 0] = (current >> 24) & 0xff;
      count += 4;
      current = next;
      next = 0;
      pos = 32-diff;
      if (count >= buflen)
	{
	  cout << "bitio.cc, bitiof::put(): flushing...\n";
	  flush();
	}
    }
  bitcount += n;
}


void
bitiof::flush()
{
  int i = 0;
  int max = count;
  
  while (i < max)
    {
      ofile.put (bufch[i++]);
    }
  assert (i == max);
  count = 0;
}


void
bitiof::finish()
{
  bufch[count + 3] =         current & 0xff;
  bufch[count + 2] =  (current >> 8) & 0xff;
  bufch[count + 1] = (current >> 16) & 0xff;
  bufch[count + 0] = (current >> 24) & 0xff;
  flush();
}


int
bitiof::reof()
{
  if (count >= buflim) return 1;
  else return 0;
  //  return end_of_file;
}

