/************************************************************************
*
*  getvlc.c, variable length decoding for tmndecode (H.263 decoder)
*  Copyright (C) 1995, 1996  Telenor R&D, Norway
*        Karl Olav Lillevold <Karl.Lillevold@nta.no>
*  
*  Contacts: 
*  Karl Olav Lillevold               <Karl.Lillevold@nta.no>, or
*  Robert Danielsen                  <Robert.Danielsen@nta.no>
*
*  Telenor Research and Development  http://www.nta.no/brukere/DVC/
*  P.O.Box 83                        tel.:   +47 63 84 84 00
*  N-2007 Kjeller, Norway            fax.:   +47 63 81 00 76
*  
************************************************************************/

/*
 * Disclaimer of Warranty
 *
 * These software programs are available to the user without any license
 * fee or royalty on an "as is" basis.  Telenor Research and Development
 * disclaims any and all warranties, whether express, implied, or statuary,
 * including any implied warranties or merchantability or of fitness for a
 * particular purpose.  In no event shall the copyright-holder be liable
 * for any incidental, punitive, or consequential damages of any kind
 * whatsoever arising from the use of these programs.
 *
 * This disclaimer of warranty extends to the user of these programs and
 * user's customers, employees, agents, transferees, successors, and
 * assigns.
 *
 * Telenor Research and Development does not represent or warrant that the
 * programs furnished hereunder are free of infringement of any third-party
 * patents.
 *
 * Commercial implementations of H.263, including shareware, are subject to
 * royalty fees to patent holders.  Many of these patents are general
 * enough such that they are unavoidable regardless of implementation
 * design.
 *
 */

/*
 * based on mpeg2decode, (C) 1994, MPEG Software Simulation Group
 * and mpeg2play, (C) 1994 Stefan Eckart
 *                         <stefan@lis.e-technik.tu-muenchen.de>
 */

#include <stdio.h>
#include "config.h"
#include "global.h"
#include "getvlc.h"

int getTMNMV(bitiob& inbuf, H263Dec* dec)
{
	int code;
	
	if (dec->trace)
		printf("motion_code (");
	
	if (inbuf.get(1))
	{
		if (dec->trace)
			printf("1): 0\n");
		return 0;
	}
	
	if ((code = inbuf.peek(12))>=512)
	{
		code = (code>>8) - 2;
		inbuf.get(TMNMVtab0[code].len);
		
		if (dec->trace)
		{
			printf("0");
			printbits(code+2,4,TMNMVtab0[code].len);
			printf("): %d\n", TMNMVtab0[code].val);
		}
		
		return TMNMVtab0[code].val;
	}
	
	if (code>=128)
	{
		code = (code>>2) -32;
		inbuf.get(TMNMVtab1[code].len);
		
		if (dec->trace)
		{
			printf("0");
			printbits(code+32,10,TMNMVtab1[code].len);
			printf("): %d\n",TMNMVtab1[code].val);
		}
		
		return TMNMVtab1[code].val;
	}
	
	if ((code-=5)<0)
	{
		if (!dec->quiet)
			fprintf(stderr,"Invalid motion_vector code\n");
		dec->fault=1;
		return 0;
	}
	
	inbuf.get(TMNMVtab2[code].len);
	
	if (dec->trace)
	{
		printf("0");
		printbits(code+5,12,TMNMVtab2[code].len);
		printf("): %d\n",TMNMVtab2[code].val);
	}
	
	return TMNMVtab2[code].val;
}


int getMCBPC(bitiob& inbuf, H263Dec* dec)
{
	int code;
	
	if (dec->trace)
		printf("MCBPC (");
	
	code = inbuf.peek(9);
	
	if (code == 1) {
		/* macroblock stuffing */
		if (dec->trace)
			printf("000000001): stuffing\n");
		inbuf.get(9);
		return 255;
	}
	
	if (code == 0) {
		if (!dec->quiet) 
			fprintf(stderr,"Invalid MCBPC code\n");
		dec->fault = 1;
		return 0;
	}
    
	if (code>=256)
	{
		inbuf.get(1);
		if (dec->trace)
			printf("1): %d\n",0);
		return 0;
	}
    
	inbuf.get(MCBPCtab[code].len);
	
	if (dec->trace)
	{
		printbits(code,9,MCBPCtab[code].len);
		printf("): %d\n",MCBPCtab[code].val);
	}
	
	return MCBPCtab[code].val;
}

int getMODB(bitiob& inbuf, H263Dec* dec)
{
	int code;
	int MODB;
	
	if (dec->trace)
		printf("MODB (");
	
	code = inbuf.peek(2);
	
	if (code < 2) {
		if (dec->trace)
			printf("0): MODB = 0\n");
		MODB = 0;
		inbuf.get(1);
	}
	else if (code == 2) {
		if (dec->trace)
			printf("10): MODB = 1\n");
		MODB = 1;
		inbuf.get(2);
	}
	else { /* code == 3 */
		if (dec->trace)
			printf("11): MODB = 2\n");
		MODB = 2;
		inbuf.get(2);
	}
	return MODB;
}


int getMCBPCintra(bitiob& inbuf, H263Dec* dec)
{
	int code;
	
	if (dec->trace)
		printf("MCBPCintra (");
	
	code = inbuf.peek(9);
	
	if (code == 1) {
		/* macroblock stuffing */
		if (dec->trace)
			printf("000000001): stuffing\n");
		inbuf.get(9);
		return 255;
	}
	
	if (code < 8) {
		if (!dec->quiet) 
			fprintf(stderr,"Invalid MCBPCintra code\n");
		dec->fault = 1;
		return 0;
	}
	
	code >>= 3;
    
	if (code>=32)
	{
		inbuf.get(1);
		if (dec->trace)
			printf("1): %d\n",3);
		return 3;
	}
	
	inbuf.get(MCBPCtabintra[code].len);
	
	if (dec->trace)
	{
		printbits(code,6,MCBPCtabintra[code].len);
		printf("): %d\n",MCBPCtabintra[code].val);
	}
	
	return MCBPCtabintra[code].val;
}

int getCBPY(bitiob& inbuf, H263Dec* dec)
{
	int code;
	
	if (dec->trace)
		printf("CBPY (");
	
	code = inbuf.peek(6);
	if (code < 2) {
		if (!dec->quiet) 
			fprintf(stderr,"Invalid CBPY code\n");
		dec->fault = 1;
		return -1;
	}
    
	if (code>=48)
	{
		inbuf.get(2);
		if (dec->trace)
			printf("11): %d\n",0);
		return 0;
	}
	
	inbuf.get(CBPYtab[code].len);
	
	if (dec->trace)
	{
		printbits(code,6,CBPYtab[code].len);
		printf("): %d\n",CBPYtab[code].val);
	}
	
	return CBPYtab[code].val;
}
