//This software module was originally developed by CSELT in the course of   //
//development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.             //
//This software module is an implementation of a part of one or more MPEG-4 //
//Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems        //
//(ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 Systems     //
//(ISO/IEC 14496-1) free license to this software module or modifications   //
//thereof for use in hardware or software products claiming conformance to  //
//the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use this software//
//module in hardware or software products are advised that its use may      //
//infringe existing patents.                                                //
//The original developer of this software module and his/her company, the   //
//subsequent editors and their companies, and ISO/IEC have no liability     //
//for use of this software module or modifications thereof in an            //
//implementation. Copyright is not released for non MPEG-4 Systems          //
//(ISO/IEC 14496-1) conforming products. CSELT S.pA. retains full right     //
//to use the code for his/her own purpose, assign or donate the code to a   //
//third party and to inhibit third parties from using the code for non      //
//MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright      //
//notice must be included in all copies or derivative works.                //
//Copyright  1999.                                                         //
//This work has been partly developed within the framework of the ESPRIT    //
//European Research Project  "MPEG-4 PC".                                   //
//////////////////////////////////////////////////////////////////////////////

/****************************************************************************
 *
 * File:        Decoder.cpp
 *
 * Author:      Franco Casalino, Antonio Arena
 * Created:     28.05.97
 *
 * Modified:    Matthew Leditschke, 1/7/97
 *
 * Description: JPEG Decoder class definition
 * 
 * Comments: Work derived from the Independent JPEG Group's software
 *
 ****************************************************************************/

// SOLARIS PORTING - Include defs.h for UNIX
#if defined(__UNIX) || defined(PLAYER3D)
#include "defs.h"
#endif
#include <assert.h>
// SOLARIS PORTING
#if defined(WIN32) && !defined(PLAYER3D)
#include "windows.h"
#endif // SOLARIS PORTING
#include "JPEGDec.h"
#include "jpegmemdec.h"

/*-------------------------------------------------------------
The exported function that instantiates the proper decoder
-------------------------------------------------------------*/
void *CreateInstance ()
{
	return new JPEGDecoder;
}

BOOL JPEGDecoder::Decode(LPBYTE pInput,int nInputLength,LPBYTE pOutput,int &rnOutputLength)
{
// SOLARIS PORTING - PLAYER3D requires different output
#ifndef PLAYER3D
	BOOL result = (BOOL) JpegDecodeInMemory(pInput, nInputLength,
									 pOutput,
									 &rnOutputLength, &m_width, &m_height);

	// If the width and height are zero, then an error has occurred
	// decoding the image
	if ((m_width == 0) || (m_height == 0))
		assert(0);

	return result;
#else // PLAYER3D
  TRACE("[JPEG::Decode] before W %4d H %4d \n",
    m_width, m_height);
  JpegDecodeInMemory (
    pInput,
    nInputLength,
	pOutput + 4, 
    &rnOutputLength,
	&m_width, &m_height);
  // put Width & Height in the first 4 bytes...
  pOutput[0] = (m_width >> 8) & 0xff;
  pOutput[1] = (m_width) & 0xff;
  pOutput[2] = (m_height >> 8) & 0xff;
  pOutput[3] = (m_height) & 0xff;
  TRACE("[JPEG::Decode] before W %4d H %4d \n",
    m_width, m_height);

  return TRUE;
#endif
}



int JPEGDecoder :: GetOptimalInputSize()
		{ return (15500*3); }

// SOLARIS PORTING - different return value for the 3D player
int JPEGDecoder :: GetOptimalOutputSize()
{
#ifdef PLAYER3D
	return m_height * m_width * 3 + 4;
#else
	return 0;
#endif
} // FG 23/6  //	{ return (704 * 576 * 3); }


/*-------------------------------------------------------------
In this decoder, attributes are fixed for all units
-------------------------------------------------------------*/
DWORD JPEGDecoder :: GetAttributes (
	ATTRIBUTE_CODE code, 
	const LPBYTE
)
{
	switch (code) {
	case FRAME_WIDTH:
		return m_width;
	case FRAME_HEIGHT:
		return m_height;
	case PITCH:
		return 3*m_width;
	case OFFSETX:
		return 0;
	case OFFSETY:
		return 0;
	default:
		return 0;
	}
}
/*-----------------------------------------------------------------
Find a compatible format and initialize the decoder
-----------------------------------------------------------------*/

// SOLARIS PORTING
#ifndef PLAYER3D
BOOL JPEGDecoder :: SetFormat (
	Capabilities &formatOptions,
	DecoderParams &decoderParams
)
{
	decoderParams.m_nDBSize = 10000;
	decoderParams.m_nOptimalAUSize = 0;
	decoderParams.m_nCBSize = -1; 
	decoderParams.m_bRequiresFullAUs = TRUE;

	if (formatOptions.GetFormatCapabilityLength () == 0) 
		return FALSE; 
	decoderParams.m_nStreamNumber = 0; 
	decoderParams.m_format = MediaFormat :: RGB; 

	return TRUE;
}
#endif // PLAYER3D
