/***********************************************************************
This software module was originally developed by Marcos Aviles
(Universidad Politecnica de Madrid, UPM) in the course of development of 
the MPEG-4 Systems (ISO/IEC 14496-1) standard. This software module is
an implementation of a part of one or more MPEG-4 Systems 
(ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems
(ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 Systems 
(ISO/IEC 14496-1) free license to this software module or modifications
thereof for use in hardware or software products claiming conformance to
the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use this 
software module in hardware or software products are advised that its 
use may infringe existing patents. The original developer of this 
software module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software module
or modifications thereof in an implementation. Copyright is not 
released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
UPM retains full right to use the code for his/her own purpose, assign 
or donate the code to a third party and to inhibit third parties from 
using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
products. This copyright notice must be included in all copies or 
derivative works. Copyright  2006
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Marcos Aviles at mar@gti.ssr.upm.es.
----------------------------------------------------------------------*/

#include <windows.h>
#include "jasper/jasper.h"
#include "jp2dec.h"

void * CreateInstance()
{
    return new JP2Dec();
}

JP2Dec::JP2Dec()
    : m_nOutsize(0)
{

}

BOOL JP2Dec::Decode(LPBYTE pInput, int nInputLength, LPBYTE pOutput, int &rnOutputLength)
{
    jas_stream_t *jp2_stream = NULL;
    jas_image_t *jp2_image = NULL;
    jas_matrix_t *pixels[] = { NULL, NULL, NULL, NULL };
    unsigned char *ptr;
    BOOL res = FALSE;

    // Init JasPer library
    if (::jas_init())
        goto cleanup;

    // Open the file...
    jp2_stream = ::jas_stream_memopen(reinterpret_cast<char *>(pInput), nInputLength);
    if (!jp2_stream)
        goto cleanup;

    // ... and decode it
    jp2_image = ::jas_image_decode(jp2_stream, -1, 0);
    if (!jp2_image)
    {
        ::jas_stream_close(jp2_stream);
        ::jas_cleanup();
        goto cleanup;
    }

    int cmpts[4];
    int numcmpts;
    switch (jas_clrspc_fam(jas_image_clrspc(jp2_image)))
    {
        case JAS_CLRSPC_FAM_RGB:
            cmpts[0] = ::jas_image_getcmptbytype(jp2_image, JAS_IMAGE_CT_COLOR(JAS_CLRSPC_CHANIND_RGB_R));
            cmpts[1] = ::jas_image_getcmptbytype(jp2_image, JAS_IMAGE_CT_COLOR(JAS_CLRSPC_CHANIND_RGB_G));
            cmpts[2] = ::jas_image_getcmptbytype(jp2_image, JAS_IMAGE_CT_COLOR(JAS_CLRSPC_CHANIND_RGB_B));
            if ((cmpts[0] < 0) || (cmpts[1] < 0) || (cmpts[2] < 0))
                goto cleanup;

            numcmpts = 3;
            cmpts[3] = ::jas_image_getcmptbytype(jp2_image, JAS_IMAGE_CT_COLOR(JAS_IMAGE_CT_OPACITY));
            if (cmpts[3] > 0)
                numcmpts++;
            break;

        case JAS_CLRSPC_FAM_GRAY:
            cmpts[0] = ::jas_image_getcmptbytype(jp2_image, JAS_IMAGE_CT_COLOR(JAS_CLRSPC_CHANIND_GRAY_Y));
            if (cmpts[0] < 0)
                goto cleanup;

            numcmpts = 1;
            cmpts[1] = ::jas_image_getcmptbytype(jp2_image, JAS_IMAGE_CT_COLOR(JAS_IMAGE_CT_OPACITY));
            if (cmpts[1] > 0)
                numcmpts++;
            break;

        default:
            goto cleanup;
    }

    m_nWidth = jas_image_width(jp2_image);
    m_nHeight = jas_image_height(jp2_image);
    m_nPitch = numcmpts * m_nWidth;

    int i;
    for (i = 0; i < numcmpts; i++)
        if ((jas_image_cmptwidth(jp2_image, cmpts[i]) != m_nWidth) ||
            (jas_image_cmptheight(jp2_image, cmpts[i]) != m_nHeight) ||
            (jas_image_cmpttlx(jp2_image, cmpts[i]) != 0) ||
            (jas_image_cmpttly(jp2_image, cmpts[i]) != 0) ||
            (jas_image_cmpthstep(jp2_image, cmpts[i]) != 1) ||
            (jas_image_cmptvstep(jp2_image, cmpts[i]) != 1) ||
            (jas_image_cmptsgnd(jp2_image, cmpts[i]) != FALSE))
            goto cleanup;

    m_nBPP = numcmpts;
    m_nOutsize = m_nWidth * m_nHeight * numcmpts + 4;

    if (rnOutputLength < m_nOutsize)
    {
        rnOutputLength = m_nOutsize;
        goto cleanup;
    }
    
    for (i = 0; i < numcmpts; i++)
    {
        pixels[i] = ::jas_matrix_create(1, m_nWidth);
        if (!pixels[i])
            goto cleanup;
    }
    
    ptr = pOutput + 4;
    
    int x, y;
    for (y = 0; y < m_nHeight; y++)
    {
        for (i = 0; i < numcmpts; i++)
            ::jas_image_readcmpt(jp2_image, cmpts[i], 0, y, m_nWidth, 1, pixels[i]);
        
        for (x = 0; x < m_nWidth; x++)
            for (i = 0; i < numcmpts; i++)
                ptr[x * numcmpts + i] = static_cast<unsigned char>(jas_matrix_getv(pixels[i], x));
            
            ptr += m_nPitch;
    }


    pOutput[0] = (m_nWidth >> 8) & 0xff;
    pOutput[1] = (m_nWidth) & 0xff;
    pOutput[2] = (m_nHeight >> 8) & 0xff;
    pOutput[3] = (m_nHeight) & 0xff;
/*
    pOutput[0] = (m_nWidth >> 24) & 0xff;
    pOutput[1] = (m_nWidth >> 16) & 0xff;
    pOutput[2] = (m_nWidth >> 8) & 0xff;
    pOutput[3] = (m_nWidth) & 0xff;
    pOutput[4] = (m_nHeight >> 24) & 0xff;
    pOutput[5] = (m_nHeight >> 16) & 0xff;
    pOutput[6] = (m_nHeight >> 8) & 0xff;
    pOutput[7] = (m_nHeight) & 0xff;
    pOutput[9] = (m_nBPP) & 0xff;
*/
    res = TRUE;

cleanup:
    for (i = 0; i < 4; i++)
        if (pixels[i])
            ::jas_matrix_destroy(pixels[i]);

    if (jp2_image)
        ::jas_image_destroy(jp2_image);
    if (jp2_stream)
        ::jas_stream_close(jp2_stream);
    ::jas_cleanup();

    return res;
}


int JP2Dec::GetOptimalOutputSize()
{
    return m_nOutsize;
}

DWORD JP2Dec::GetAttributes(ATTRIBUTE_CODE code, const LPBYTE)
{
    switch (code)
    {
        case FRAME_WIDTH:
            return m_nWidth;

        case FRAME_HEIGHT:
            return m_nHeight;

        case PITCH:
            return m_nBPP * m_nWidth;

        case OFFSETX:
        case OFFSETY:
            return 0;

        default:
            return 0;
    }
}

BOOL JP2Dec::SetFormat(Capabilities &formatOptions, DecoderParams &decoderParams)
{
    decoderParams.m_nDBSize = 10000;
    decoderParams.m_nOptimalAUSize = 0;
    decoderParams.m_nCBSize = -1; 
    decoderParams.m_bRequiresFullAUs = TRUE;
    
    if (!formatOptions.GetFormatCapabilityLength ()) 
        return FALSE; 
    decoderParams.m_nStreamNumber = 0; 
    
    switch (m_nBPP)
    {
        case 1:
            decoderParams.m_format = MediaFormat::GRAYSCALE;
            break;

        case 2:
            decoderParams.m_format = MediaFormat::ALPHAGRAYSCALE;
            break;

        case 3:
            decoderParams.m_format = MediaFormat::RGB;
            break;

        case 4:
            decoderParams.m_format = MediaFormat::RGBA;
            break;

        default:
            decoderParams.m_format = MediaFormat::UNKNOWN; 
    }

    return TRUE;
}