/*************************************************************************************
 
 This software module was originally developed by
 
	Dae-Sung Cho 	(dscho@sait.samsung.co.kr), Samsung AIT
 
 and edited by 

	Dae-Sung Cho	(dscho@sait.samsung.co.kr), Samsung AIT

 Those who intend to use this software module in hardware or software products
 are advised that its use may infringe existing patents and other intellectual
 property rights.
 
 We disclaim all warranties with regard to this software, including
 all implied warranties of merchantability and fitness.
 In no event, Samsung AIT shall be liable for any special, indirect or
 consequential damages or any damages whatsoever resulting from loss of
 data or profits arising out of or in connection with the use or performance
 of this software.
 
 Samsung AIT retains the full right to use the code for its own purpose, assign
 or donate the code to a third party and to inhibit third parties from using
 the code. This copyright notice must be included in all copies or derivative
 works.
 
 Copyright (c) 1997-2000 Samsung AIT all rights reserved

*************************************************************************************/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Multiplexer
FILE:       AACadts.cpp
AUTHOR:     Patrizio Bergallo Franco Casalino (CSELT)
OVERVIEW:   Implementation of ADTSStream, the class that handles AAC
input streams.
-------------------------------------------------------------------------*/

#include "defs.h"
#include "AACadts.h"
#include "assert.h"

#ifdef _MP4FF_	// standard MP4 file  format
#include "odMp4.h"
#else			// non-standard TRIFF file format
#include "odMux.h"
#endif

/*-----------------------------------------------------------------
ADTSStream constructor reads first character
-----------------------------------------------------------------*/
ADTSStream :: ADTSStream (FILE *fStream, ESDesc *pDescriptor) :
	Stream (fStream, pDescriptor)
{
	int samplingRate;
	m_nCount = 0;
	m_nFrameSize = 0; // by D.S.CHO
	m_cNext = getc ();
	m_nAU = 0;
	if (pDescriptor -> streamData != (LPCSTR) NULL && pDescriptor -> streamData [0] != 0)
		samplingRate = atof (pDescriptor -> streamData);
	else
		samplingRate = 44100; //FC
	m_dt = 1024000.0 / samplingRate;  // AU duration
}

/*-----------------------------------------------------------------
For AAC, each AU contains 1024 samples.
The duration of each AU is then related to the audio sampling rate 
of the samples: time increment = (1024/Fs)
Example: at Fs = 44100, time increment is (1024/44100) = 23 ms

-----------------------------------------------------------------*/
long ADTSStream :: GetDTS ()
{
	if (m_cNext == EOF)
		return -1;
	m_lDTS = m_nAU++ * m_dt;
	return m_lDTS;
}

/*-----------------------------------------------------------------
GetChar returns a character and reads one char ahead.  The number
of characters in an AU is determined by the data rate, which is
given in the pDescriptor -> streamData field.
-----------------------------------------------------------------*/
int ADTSStream :: GetChar ()
{
	if (m_nCount > m_nFrameSize)
		return -1;
	else
	if (m_nCount == m_nFrameSize) {		
		if (m_cNext == EOF) 
			return -1;
		getc ();
		getc ();
		a[0] = getc ();
		a[1] = getc ();
		a[2] = getc ();
		getc ();		
		m_nFrameSize = (0x001fff) & ( (a[0]<<11) | (a[1]<<3) | (a[2]>>5 & 0x07) );
		m_nFrameSize -= 7;		
		assert (m_nFrameSize > 0);
		m_nCount = 0;
		m_cNext = getc (); 		
		return -1;			
	}
	++m_nCount;
	int c = m_cNext;
	m_cNext = getc ();
	assert (c != EOF);
	return c;
}

/*----------------------------------------------------------------
Returns true if the AU referenced by the last call to GetDTS()
was a Random Access Point
-----------------------------------------------------------------*/
bool ADTSStream :: IsRAP ()
{
	return true;
}

static StreamFactory<ADTSStream> adtsFactory ("AACadts");
