
// Routines for getting decoderSpecificInfo of AAC LC from AAC ADTS file 

//#define DEBUG					nofprintf
//#define DEBUG_BUFFER1			nofprintf
//#define DEBUG_BUFFER2			nofprintf

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "aac_adts_dsi.h"

// for AudioSpecificInfo 
//#define ER_BSAC						22
#define AAC_LC						2
#define	lenAudioObjectType			5
#define lenSamplingFrequencyIndex	4
#define lenSamplingFrequency		24
#define lenChannelConfiguration		4
#define lenFrameLengthFlag	    	1
#define	lenDependsOnCoreCoder		1
#define lenExtensionFlag			1
#define	lenASIHeader				(5+4+0+4+1+1+1)
#define lenNumOfSubFrame   			5
#define lenLayer_length           	11
#define lenExtension3Flag		    1
#define lenEpConfig		            2

#define MAX_DSI_LEN					1024	    // bytes 

FILE	*fp;
unsigned int		bword, bbit; // for buffer i/o 
//long				buffer_fullness;
unsigned int		num_of_channels;
unsigned int		sampling_rate_idx;
//unsigned int		bsac_bitrate = 96000;

unsigned int sampling_rate[] = 
{
	96000, 
	88200, 
	64000, 
	48000, 
	44100, 
	32000, 
	24000, 
	22050, 
	16000, 
	12000, 
	11025, 
	8000,  
	0,
	0,
	0,
	0
};

/*
void nofprintf() {;}

void myexit(char *s)
{
    if (s)
        fprintf(stderr,"\n%s\n", s);
    exit(s ? 1 : 0);
}
*/

// writes n bits from data (msb first) to **pd if pd is not NULL otherwise does nothing
static void putbits( char **pd, unsigned int data, unsigned int n)
{
	unsigned int dbit;
	unsigned int word, bit;
	char *d;

	if (pd == NULL) {
		d = NULL;
	}
	else {
		d = *pd;
	}
    dbit = 1<<(n-1);
    word = bword;
    bit = bbit;
    while(dbit != 0) {
        if(data & dbit) {
            word |= bit;
        }
        dbit >>= 1;
        bit >>= 1;
        if(bit == 0) {
			if (d)
				*d++ = word;
            word = 0;
            bit = 0x80;
        }
    }
    bword = word;
    bbit = bit;
	if (d)
	    *pd = d;
}

int analyzeAACadtsFile ( FILE *fp )
{
	unsigned char c[2];
	fseek(fp, 2, SEEK_SET);
	fread(c, 1, 2, fp);
	sampling_rate_idx  = (c[0]>>2) & 0x0f;
	num_of_channels   = (c[0] & 0x01) | (c[1]>>6 & 0x03);
	return 0;
}
	
int getDecoderSpecificInfo(char ** decoderSpecificInfo, unsigned int *decoderSpecificInfoLen )
{
	unsigned int n;
	char *dsi, *plen;
	
	// allocate default amount of memory to decoderSpecificInfo 
	*decoderSpecificInfo = new char [MAX_DSI_LEN];
	if (*decoderSpecificInfo == NULL) {
		return 1;
	}

	// write DecoderSpecificInfo 	
	bbit = 0x80; // byte align bit stream 
	bword = 0;
	dsi = (char *) *decoderSpecificInfo;

	// tag is DecSpecificInfoTag, 14496-1, Sec 8.2.2.2, Table 1
//	putbits( &dsi, 0x05, 8 ); // ????

	// dummy sizeOfInstance, 14496-1, Sec 12.3.3 
	plen = dsi;
	putbits( &dsi, 0, 8 );

	// Audio Specific Configuration 
//	putbits( &dsi, ER_BSAC, lenAudioObjectType ); // For only ER BSAC 
	putbits( &dsi, AAC_LC, lenAudioObjectType ); //AAC  sjlee 0904
	putbits( &dsi, sampling_rate_idx, lenSamplingFrequencyIndex );
	if (sampling_rate_idx==0xf) {
		fprintf (stderr,"samplingFrquency should be provided !\n");
		return 1;
	}
	putbits( &dsi, num_of_channels, lenChannelConfiguration );
	// GA Specific Configurtaion 
	putbits( &dsi, 0, lenFrameLengthFlag );
	putbits( &dsi, 0, lenDependsOnCoreCoder );
	putbits( &dsi, 0, lenExtensionFlag );
/*
	putbits( &dsi, 1, lenNumOfSubFrame );
	putbits( &dsi, (unsigned int)AU_avgLen, lenLayer_length );
	putbits( &dsi, 0, lenExtension3Flag );
	putbits( &dsi, epConfig, lenEpConfig );
*/
	// byte align
	while (bbit != 0x80)	putbits(&dsi, 0, 1);
	
	// set exact handle size 
	n = dsi - *decoderSpecificInfo; // ?????
	*decoderSpecificInfoLen = n;

	// set length of decoderSpecificInfo part in header, encoded as sizeOfInstance 
	if (n < 128) {
		//*plen = n-2;
		*plen = n-1;		
	}
	else {
		fprintf (stderr,"size %d (> 127) of decoderSpecificInfo is not currently supported\n",n);
		return 1;
	}
	return 0;
}