/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Multiplexer
FILE:       BSAC.cpp
AUTHOR:     Dae-Sung Cho 	(dscho@sait.samsung.co.kr), Samsung AIT
OVERVIEW:   Implementation of BSACStream, the class that handles BSAC
input streams.
-------------------------------------------------------------------------*/

/*************************************************************************************
 
 This software module was originally developed by
 
	Dae-Sung Cho 	(dscho@sait.samsung.co.kr), Samsung AIT
 
 and edited by 

	Dae-Sung Cho	(dscho@sait.samsung.co.kr), Samsung AIT

 Those who intend to use this software module in hardware or software products
 are advised that its use may infringe existing patents and other intellectual
 property rights.
 
 We disclaim all warranties with regard to this software, including
 all implied warranties of merchantability and fitness.
 In no event, Samsung AIT shall be liable for any special, indirect or
 consequential damages or any damages whatsoever resulting from loss of
 data or profits arising out of or in connection with the use or performance
 of this software.
 
 Samsung AIT retains the full right to use the code for its own purpose, assign
 or donate the code to a third party and to inhibit third parties from using
 the code. This copyright notice must be included in all copies or derivative
 works.
 
 Copyright (c) 1997-2000 Samsung AIT all rights reserved

*************************************************************************************/


#include "defs.h" // by D.S.CHO NEW
#include "BSAC.h"
#include "bsac_parser.h"
#include "assert.h"
#ifdef _MP4FF_	// standard MP4 file  format
#include "odMp4.h"
#else			// non-standard TRIFF file format
#include "odMux.h"
#endif

int	fs_tbl[12] = {
	96000, 88200, 64000, 48000, 44100, 32000, 24000, 22050, 16000, 12000, 11025, 8000
};

/*-----------------------------------------------------------------
BSACStream constructor reads first character
-----------------------------------------------------------------*/
BSACStream :: BSACStream (FILE *fStream, ESDesc *pDescriptor) :	
	Stream (fStream, pDescriptor),
	m_lFrames(0),
	m_bEOF(false),
	m_bHeader(true),	// by D.S.CHO NEW2
	m_pAUBuffer(NULL),
	m_lAULength(0),
	m_lIndex(0)
{
	unsigned char *bsac_header;
	m_pParser = new CBSACParser(fStream);
	if (m_pParser->parseFrameHeader() == EOF) {
		fprintf(stderr,"Error: Incorrect Header Parsing!!!!\n");
		exit(1);
	}
	if(m_pParser->getOutFrameLength()!=1) {
		fprintf(stderr,"Error: Incorrect Header Parsing!!!!\n");
		exit(0);
	}
	bsac_header = m_pParser->getOutFrameBuffer();

	int sample_freq_index = (bsac_header[0]>>1) & 0x0f;
	m_nNumChannel = (bsac_header[0]>>5) & 0x07;

// by D.S.CHO NEW
	if (pDescriptor == NULL) {	
		fprintf (stderr,"No info for stream \n");
		exit(0);
	}	
	if (pDescriptor -> streamData != (LPCSTR) NULL &&  pDescriptor -> streamData [0] != 0)
	{
		m_nSamplingFrequency = atof (pDescriptor -> streamData);
	} 
	else
//~by D.S.CHO NEW  	
	m_nSamplingFrequency = fs_tbl[sample_freq_index];
	m_header = bsac_header[0];
	// first AU access
	m_pAUBuffer = GetAUBuffer();
	m_lAULength = GetAULength();
	m_lIndex = 0;
}

BSACStream :: ~BSACStream ()
{
	delete m_pParser;
}

long BSACStream :: GetDTS ()
{
	if (m_bEOF)
		return -1;
	m_lDTS = (long)(((float)m_lFrames * 1024 * 1000 / m_nSamplingFrequency));
	m_lFrames++;
	return m_lDTS;
}

/*-----------------------------------------------------------------
GetChar returns a character and reads one char ahead.  The number
of characters in an AU is determined by the data rate, which is
given in the pDescriptor -> streamData field.
-----------------------------------------------------------------*/
int BSACStream :: GetChar ()
{
	if (m_bEOF)
		return -1;

// by D.S.CHO NEW2
// temporal - this part will be removed since the global header is also in the DecoderSpecificInfo in ES descriptor
// by D.S.CHO SS
/*
	if (m_bHeader) {
		m_bHeader = false;
		return m_header;
	} 
*/
//~by D.S.CHO SS
//~temporal - this part will be removed since the global header is also in the DecoderSpecificInfo in ES descriptor
//~by D.S.CHO NEW2
	if(m_lIndex == m_lAULength) {
		m_pAUBuffer = GetAUBuffer();
		m_lAULength = GetAULength();
		m_lIndex = 0;
		return -1;
	}	
	if (m_bEOF) 
		return -1;

	int c = m_pAUBuffer[m_lIndex++];
	return c;
}

/*----------------------------------------------------------------
Returns true if the AU referenced by the last call to GetDTS()
was a Random Access Point
-----------------------------------------------------------------*/
bool BSACStream :: IsRAP ()
{
	return true;
}

// by D.S.CHO 
unsigned char* BSACStream :: GetAUBuffer()
{
	m_pParser->initParser(); // NEW
	if( m_pParser->parseFrame() == EOF) {
		m_bEOF = true;
	} 
	return m_pParser->getOutFrameBuffer();	
}

long BSACStream :: GetAULength()
{
	return ((long)m_pParser->getOutFrameLength());
}
// ~by D.S.CHO

static StreamFactory<BSACStream> bifsFactory ("BSAC");	

