/*************************************************************************************
 
 This software module was originally developed by
 
	Dae-Sung Cho 	(dscho@sait.samsung.co.kr), Samsung AIT
 
 and edited by 

	Dae-Sung Cho	(dscho@sait.samsung.co.kr), Samsung AIT

 Those who intend to use this software module in hardware or software products
 are advised that its use may infringe existing patents and other intellectual
 property rights.
 
 We disclaim all warranties with regard to this software, including
 all implied warranties of merchantability and fitness.
 In no event, Samsung AIT shall be liable for any special, indirect or
 consequential damages or any damages whatsoever resulting from loss of
 data or profits arising out of or in connection with the use or performance
 of this software.
 
 Samsung AIT retains the full right to use the code for its own purpose, assign
 or donate the code to a third party and to inhibit third parties from using
 the code. This copyright notice must be included in all copies or derivative
 works.
 
 Copyright (c) 1997-2000 Samsung AIT all rights reserved

*************************************************************************************/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  BifsEnc
FILE:       BSAC.cpp
AUTHOR:     Dae-Sung Cho 	(dscho@sait.samsung.co.kr), Samsung AIT
OVERVIEW:   Implementation of BSACStream, the class that handles BSAC
input streams.
-------------------------------------------------------------------------*/

#include "defs.h" 
#include "BSAC_dsi.h"

/*-----------------------------------------------------------------
BSACStream constructor reads first character
-----------------------------------------------------------------*/
BSACStreamDSI :: BSACStreamDSI (FILE *fStream) :	
	StreamDSI (fStream),
	m_bGetDecSpecificInfo (false),
	m_lDsiLength (-1)
{
	m_pParser = new CBSACParser(fStream);
	if (m_pParser == NULL) {
		fprintf (stderr," No infor for CBSACParser objects \n");
		exit(0);
	}
}

BSACStreamDSI :: ~BSACStreamDSI ()
{
	if (m_pParser != NULL) {
		delete m_pParser;
		m_pParser = NULL;
	}	
}

unsigned char* BSACStreamDSI :: GetDecSpecificInfo ()
{
	if (m_bGetDecSpecificInfo==false) 
	{
		m_bGetDecSpecificInfo=true;		
		m_pParser->initParser(); // NEW
		if (m_pParser->parseFrameHeader() == EOF) {
			fprintf(stderr,"Error: Incorrect Header Parsing!!!!\n");
			exit(1);
		}		
		m_pDecSpecificInfo = m_pParser->getOutFrameBuffer ();
		m_lDsiLength = m_pParser->getOutFrameLength ();
		if(m_lDsiLength!=1) 
		{
			fprintf(stderr,"Error: Incorrect Header Parsing!!!!\n");
			exit(0);
		}	
	}
	return m_pDecSpecificInfo;		
}

long BSACStreamDSI :: GetDecSpecificInfoLen ()
{
	if (m_bGetDecSpecificInfo) 
	{
		return m_lDsiLength;
	} else 
	{
		return -1;
	}
}	

static StreamDsiFactory<BSACStreamDSI> BSACFactory ("BSAC");