/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

/*************************************************************************************
 
 This software module was originally developed by
 
	Dae-Sung Cho 	(dscho@sait.samsung.co.kr), Samsung AIT
 
 and edited by 

	Dae-Sung Cho	(dscho@sait.samsung.co.kr), Samsung AIT

 Those who intend to use this software module in hardware or software products
 are advised that its use may infringe existing patents and other intellectual
 property rights.
 
 We disclaim all warranties with regard to this software, including
 all implied warranties of merchantability and fitness.
 In no event, Samsung AIT shall be liable for any special, indirect or
 consequential damages or any damages whatsoever resulting from loss of
 data or profits arising out of or in connection with the use or performance
 of this software.
 
 Samsung AIT retains the full right to use the code for its own purpose, assign
 or donate the code to a third party and to inhibit third parties from using
 the code. This copyright notice must be included in all copies or derivative
 works.
 
 Copyright (c) 1997-2000 Samsung AIT all rights reserved

*************************************************************************************/

#include "defs.h" // by D.S.CHO NEW
#include "MP4VIDEO.h"
#ifdef _MP4FF_	// standard MP4 file format
#include "odMp4.h"
#else			// non-standard TRIFF file format
#include "odMux.h"
#endif

//FILE *fp;// by D.S.CHO

/*-----------------------------------------------------------------
MP4VIDEOStream constructor reads first 3 character
-----------------------------------------------------------------*/
MP4VIDEOStream :: MP4VIDEOStream (FILE *fStream, ESDesc *pDescriptor) :
	Stream (fStream, pDescriptor), m_bRAP (true),
// by D.S.CHO
	m_bNextRAP (true),
	m_lFrames (0),	
	m_bEOF(false),
	m_bIsSessionHeader(true),
	m_bScalability (false),
	m_lHierarchyType (0),
	m_pAUBuffer(NULL),
	m_lAULength(0),
	m_lIndex(0)
// ~by D.S.CHO
{
// by D.S.CHO
	/*
	m_cNext [0] = getc ();
	m_cNext [1] = getc ();
	m_cNext [2] = getc ();
	m_cNext [3] = getc ();
	m_firstStartCode = TRUE;
	m_returnedStartCodeLastCall = FALSE;
	*/
//~by D.S.CHO

// by D.S.CHO 
	if (pDescriptor == NULL) {	
		fprintf (stderr,"No info for stream \n");
		exit(0);
	}
//~by D.S.CHO 

	// The stream data specifies the frame rate to use (default to 15)
	if ((pDescriptor -> streamData == (LPCSTR)NULL) || (pDescriptor -> streamData[0] == 0)) 
		m_frameRate = 15;
	else
	{
		m_frameRate = atof(pDescriptor -> streamData); 
		if (m_frameRate < 0.0001)
			throw ExceptionInvalidData (
				"The MPEG4 video frame rate must be greater than 0.\n"
			);
	}
	
// by D.S.CHO
	// first AU access
	m_pParser = new CMp4VideoParser(fStream);	
	if (m_pParser == NULL) {
		fprintf (stderr," No infor for CMp4VideoParser objects \n");
		exit(0);
	}
	m_pAnalyzer = new CMp4VideoAnalyzer;	
	if (m_pAnalyzer == NULL) {
		fprintf (stderr," No infor for CMp4VideoAnalyzer objects \n");
		exit(0);		
	}
//fp = fopen ("vmux.cmp", "wb");
	m_pAUBuffer = GetAUBuffer();
	m_lAULength = GetAULength();
	m_lIndex = 0;

	m_pAnalyzer -> AnalyzeVopHeader (m_pAUBuffer, m_lAULength);
	m_lCTS = m_pAnalyzer -> getVopTime ();
	if (m_pAnalyzer -> getVopPredType () == IVOP)
		m_bNextRAP = true;
	else
		m_bNextRAP = false;

//~by D.S.CHO
}


// by D.S.CHO
MP4VIDEOStream :: ~MP4VIDEOStream ()
{
	if (m_pParser != NULL) {
		delete m_pParser;
		m_pParser = NULL;
	}
	if (m_pAnalyzer != NULL) {
		delete m_pAnalyzer;
		m_pAnalyzer = NULL;
	}
//fclose(fp);
}
//~by D.S.CHO

/*-----------------------------------------------------------------
Time stamps are calculated according to the rate given by the
pDescriptor -> streamData field
-----------------------------------------------------------------*/
long MP4VIDEOStream :: GetDTS (long)
{
// by D.S.CHO 
	//if (m_cNext [0] == EOF)
	//	return -1;
	if (m_bEOF)
		return -1;
//~by D.S.CHO 
	m_lDTS = (long)((float)(m_lFrames * 1000) / m_frameRate);
	++m_lFrames;
	return m_lDTS;
}

/*-----------------------------------------------------------------
Time stamps are calculated according to the time stampe in the
bitstream of the video header 
-----------------------------------------------------------------*/
// by D.S.CHO ???
long MP4VIDEOStream :: GetCTS ()
{
	if (m_bEOF)
		return -1;
	return m_lCTS;
}
//~by D.S.CHO

/*-------------------------------------------------------------
In this version only the first AU is RAP
-------------------------------------------------------------*/
bool MP4VIDEOStream :: IsRAP ()
{
	bool bResult = m_bRAP;
// by D.S.CHO
	//m_bRAP = false;
	m_bRAP = m_bNextRAP;	
//~by D.S.CHO
	return bResult;
}

/*-----------------------------------------------------------------
GetChar returns a character and reads one next char. We are always
4 chars ahead, which are used to check whether this is end of AU.
-----------------------------------------------------------------*/
int MP4VIDEOStream :: GetChar ()
{
// by D.S.CHO
	if (m_bEOF)
		return -1;
	if(m_lIndex == m_lAULength) {
		m_pAUBuffer = GetAUBuffer();
		m_lAULength = GetAULength();
		m_lIndex = 0;
		if (!m_bEOF) {
			m_pAnalyzer -> AnalyzeVopHeader (m_pAUBuffer, m_lAULength);
			m_lCTS = m_pAnalyzer -> getVopTime ();
			if (m_pAnalyzer -> getVopPredType () == IVOP)
				m_bNextRAP = true;
			else
				m_bNextRAP = false;
		}
		return -1;
	}
	if (m_bEOF) 
		return -1;
	int c = m_pAUBuffer[m_lIndex++];
	return c;
	/*
	if (m_cNext[0] == EOF)
		return -1;
	if (!m_returnedStartCodeLastCall)
	{
		if ((m_cNext[0] == 0) && (m_cNext[1] == 0) &&
			(m_cNext[2] == 1) && (m_cNext[3] == 0x73))
		{
			// Ignore the first start code
			if (m_firstStartCode)
				m_firstStartCode = FALSE;
			else
			{
				m_returnedStartCodeLastCall = TRUE;
				return -1;
			}
		}
	}
	int c = m_cNext [0];
	m_cNext [0] = m_cNext [1];
	m_cNext [1] = m_cNext [2];
	m_cNext [2] = m_cNext [3];
	m_cNext [3] = getc ();
	m_returnedStartCodeLastCall = FALSE;

	return c;
	*/	
//~by D.S.CHO	
}

// by D.S.CHO 
unsigned char* MP4VIDEOStream :: GetAUBuffer()
{
	m_pParser->initParser(); 
	if(m_bIsSessionHeader) {
		if (m_pParser->parseFrameHeader() == EOF) {
			fprintf(stderr,"Error: Incorrect Header Parsing!!!!\n");
			exit(1);
		}
		m_bIsSessionHeader = false;
		unsigned char *pAUBuffer = m_pParser->getOutFrameBuffer ();
		long lAULength = m_pParser->getOutFrameLength ();
		m_pAnalyzer -> AnalyzeVolHeader (pAUBuffer, lAULength);
		m_bScalability = (m_pAnalyzer -> getVolType ()) ? true : false;
		m_lHierarchyType = m_pAnalyzer -> getHierarchyType();
// temporal - this line will be removed since the above VOL info. is also in the DecoderSpecificInfo in ES descriptor
		//return pAUBuffer;		// by D.S.CHO SS
		m_pParser->initParser(); // by D.S.CHO SS
//fprintf (stderr,"vol length=%ld\n", lAULength); getchar();
//fwrite (pAUBuffer,1,lAULength,fp);
//~temporal - this line will be removed since the above VOL info. is also in the DecoderSpecificInfo in ES descriptor
	}
	if( m_pParser->parseFrame() == EOF) {
		m_bEOF = true;
	} 
//fwrite (m_pParser->getOutFrameBuffer(),1,m_pParser->getOutFrameLength(),fp);
//fprintf (stderr,"length=%ld\n", m_pParser->getOutFrameLength()); getchar();

	return m_pParser->getOutFrameBuffer();	
}
 
long MP4VIDEOStream :: GetAULength()
{
	return ((long)m_pParser->getOutFrameLength());	
}
// ~by D.S.CHO

#ifdef _SAIT_3DTG_MP4VIDEO_
static StreamFactory<MP4VIDEOStream> bifsFactory ("MPEG4-Video");
#endif _SAIT_3DTG_MP4VIDEO_