/*************************************************************************************

 This software module was originally developed by

        Dae-Sung Cho    (dscho@sait.samsung.co.kr), Samsung AIT

 and edited by

        Dae-Sung Cho    (dscho@sait.samsung.co.kr), Samsung AIT

 Those who intend to use this software module in hardware or software products
 are advised that its use may infringe existing patents and other intellectual
 property rights.

 We disclaim all warranties with regard to this software, including
 all implied warranties of merchantability and fitness.
 In no event, Samsung AIT shall be liable for any special, indirect or
 consequential damages or any damages whatsoever resulting from loss of
 data or profits arising out of or in connection with the use or performance
 of this software.

 Samsung AIT retains the full right to use the code for its own purpose, assign
 or donate the code to a third party and to inhibit third parties from using
 the code. This copyright notice must be included in all copies or derivative
 works.

 Copyright (c) 1997-2000 Samsung AIT all rights reserved


 Required module :

 Authors :

 Changes :
        date            programmers     comment

*************************************************************************************/
#include <stdio.h>
#include <memory.h>

#include "defs.h" // by D.S.CHO NEW
#include "bitstream_i.h"

#define true	1
#define false	0


CInBitstream::CInBitstream ()
{
	m_inType = NOT_DEFINED;
	m_fptr = NULL;
	m_pBuffer = NULL;
}

CInBitstream::CInBitstream (FILE *file)
{
  int		i;

  m_fptr = file;
  m_incnt = 0;
  m_rdptr = m_rdbfr + BFR_SIZE;
  m_bitcnt = 0;
  
  m_rdbfr_bitcnt = 0; // by D.S.CHO 
  m_bEof = false; // by D.S.CHO
  m_bEndBits = false; // by D.S.CHO

  for (i = 0; i < 3*INBFR_BYTES; i++)
    {
      m_inbfr[i] = 0;
    }

  m_inType = FILE_IN; // by D.S.CHO NEW
}

// by D.S.CHO NEW
CInBitstream::CInBitstream (unsigned char *buffer, int length)
{
  int		i;

  m_fptr = NULL;
  m_incnt = 0;
  m_rdptr = m_rdbfr + BFR_SIZE;
  m_bitcnt = 0;
  
  m_rdbfr_bitcnt = 0; // by D.S.CHO 
  m_bEof = false; // by D.S.CHO
  m_bEndBits = false; // by D.S.CHO

  for (i = 0; i < 3*INBFR_BYTES; i++)
    {
      m_inbfr[i] = 0;
    }

  m_inType = BUFFER_IN; 
  m_nLength = length;
  m_nRest = length;
  m_nIndex = 0;
  m_pBuffer = new unsigned char [length];
  if (m_pBuffer==NULL) {
	  fprintf(stderr,"Error: Memorry allocation for m_pBuffer !!!\n");
	  exit(0);
  }
  memcpy (m_pBuffer, buffer, length);	 
}
// ~by D.S.CHO NEW

CInBitstream::~CInBitstream ()
{
  if (m_inType == FILE_IN) {	// by D.S.CHO NEW	
	  if (m_fptr != NULL) {
		fclose (m_fptr);
		m_fptr = NULL;
	  }
// by D.S.CHO NEW
  } else if(m_inType == BUFFER_IN) {
	  if (m_pBuffer != NULL) {
		delete m_pBuffer;
		m_pBuffer = NULL;
	  }
  }
// ~by D.S.CHO NEW
}

/***********************************************************CommentBegin******
 *
 * -- bitstreamOpen -- Initialises a bitstream structure
 *
 * Author :		
 *	Paulo Nunes (IST) - Paulo.Nunes@lx.it.pt
 *
 * Created :		
 *	1-Mar-1996
 *
 * Purpose :		
 *	Initialisation of a bitstream structure.
 * 
 * Arguments in : 	
 *	Char filename,	 The name of the file containing the bitstream data.
 *
 * Arguments in/out :	
 *	
 *
 * Arguments out :	
 *	
 *
 * Return values :	
 *	A poiter to a bitstream structure.
 *
 * Side effects :	
 *	
 *
 * Description :	
 *	Allocates memory for the bitstream structure, opens the bitstream
 *	file and initialises the bitstream structure.
 *
 * See also :
 *	
 *
 * Modified :
 *	
 *
 ***********************************************************CommentEnd********/
void CInBitstream::bitstreamOpen (FILE *file)
{
  int		i;

  m_fptr = file;
  m_incnt = 0;
  m_rdptr = m_rdbfr + BFR_SIZE;
  m_bitcnt = 0;
  m_rdbfr_bitcnt = 0; // by D.S.CHO 
  m_bEof = false; // by D.S.CHO 
  m_bEndBits = false; // by D.S.CHO 

  for (i = 0; i < 3*INBFR_BYTES; i++)
    {
      m_inbfr[i] = 0;
    }

  m_inType = FILE_IN; // by D.S.CHO NEW
}				

// by D.S.CHO NEW
void CInBitstream::bitstreamOpen (unsigned char *buffer, int length)
{
  int		i;

  m_fptr = NULL;
  m_incnt = 0;
  m_rdptr = m_rdbfr + BFR_SIZE;
  m_bitcnt = 0;
  
  m_rdbfr_bitcnt = 0; // by D.S.CHO 
  m_bEof = false; // by D.S.CHO
  m_bEndBits = false; // by D.S.CHO

  for (i = 0; i < 3*INBFR_BYTES; i++)
    {
      m_inbfr[i] = 0;
    }

  m_inType = BUFFER_IN; 
  m_nLength = length;
  m_nRest = length;
  m_nIndex = 0;
  m_pBuffer = new unsigned char [length];
  if (m_pBuffer==NULL) {
	  fprintf(stderr,"Error: Memorry allocation for m_pBuffer !!!\n");
	  exit(0);
  }
  memcpy (m_pBuffer, buffer, length);	 
}
// ~by D.S.CHO NEW

/***********************************************************CommentBegin******
 *
 * -- bitstreamClose -- Frees a bitstream structure
 *
 * Author :		
 *	Paulo Nunes (IST) - Paulo.Nunes@lx.it.pt
 *
 * Created :		
 *	1-Mar-1996
 *
 * Purpose :		
 *	Freeing a bitstream structure.
 * 
 * Arguments in : 	
 *	bitstream * stream,	input bitstream.
 *
 * Arguments in/out :	
 *	
 *
 * Arguments out :	
 *	
 *
 * Return values :	
 *	
 *
 * Side effects :	
 *	
 *
 * Description :	
 *	Closes the file associated with the bitstream structure and frees
 *	the structure. 
 *
 * See also :
 *	
 *
 * Modified :		
 *	
 *
 ***********************************************************CommentEnd********/

void CInBitstream::bitstreamClose ()
{
  if (m_inType == FILE_IN) {	// by D.S.CHO NEW	
	  if (m_fptr != NULL) {
		fclose (m_fptr);
		m_fptr = NULL;
	  }
// by D.S.CHO NEW
  } else if(m_inType == BUFFER_IN) {
	  if (m_pBuffer != NULL) {
		delete m_pBuffer;
		m_pBuffer = NULL;
	  }
  }
// ~by D.S.CHO NEW
}				

/***********************************************************CommentBegin******
 *
 * -- bitstreamFillBuffer -- Fills a bitstream structure
 *
 * Author :		
 *	Paulo Nunes (IST) - Paulo.Nunes@lx.it.pt
 *
 * Created :		
 *	1-Mar-1996
 *
 * Purpose :		
 *	To read a piece of the bitstream from the bitstream file to the
 *	bitstream buffer. 
 *	
 * 
 * Arguments in : 	
 *	Bitstream * stream,	input bitstream.
 *
 * Arguments in/out :	
 *	
 *
 * Arguments out :	
 *	
 *
 * Return values :	
 *	
 *
 * Side effects :	
 *	
 *
 * Description :	
 *	
 *
 * See also :
 *	
 *
 * Modified :		 Noel Brady/Luis Ducla-Soares: Changed termination of bitstream
 *	
 *
 ***********************************************************CommentEnd********/

void CInBitstream::bitstreamFillBuffer ()
{
  int                 l, i;

  for (i = 0; i < INBFR_BYTES; i++)
    {
      m_inbfr[i] = m_inbfr[i + 2 * INBFR_BYTES];
    }

  if (m_rdptr >= m_rdbfr + BFR_SIZE)
    {
// by D.S.CHO NEW
		if(m_inType == FILE_IN)	{
//~by D.S.CHO NEW
		  l = fread ((char *)m_rdbfr, sizeof (unsigned char), BFR_SIZE, m_fptr);
		  m_rdptr = m_rdbfr;
		  if (l < BFR_SIZE)
		  {
			 if (l < 0)
				l = 0;

			 m_rdbfr[l++] = 0;
			 m_rdbfr[l++] = 0;
			 m_rdbfr[l++] = 1;
			 m_rdbfr[l++] = 0;

			 m_bEof = true; // by D.S.CHO 
			 m_rdbfr_bitcnt = l*8; // by D.S.CHO 
		  }      
// by D.S.CHO NEW
		} else if (m_inType == BUFFER_IN) {
		  if (m_nRest >= BFR_SIZE) {
			 memcpy (m_rdbfr, m_pBuffer + m_nIndex, BFR_SIZE);
  		     m_rdptr = m_rdbfr;
			 m_nIndex += BFR_SIZE;
			 m_nRest -= BFR_SIZE;
		  } else {
			 if(m_nRest!=0)
				memcpy (m_rdbfr, m_pBuffer + m_nIndex, m_nRest);
			 m_rdptr = m_rdbfr;
			 l = m_nRest;
			 m_rdbfr[l++] = 0;
			 m_rdbfr[l++] = 0;
			 m_rdbfr[l++] = 1;
			 m_rdbfr[l++] = 0;

			 m_bEof = true; // by D.S.CHO 
			 m_rdbfr_bitcnt = l*8; // by D.S.CHO 			
		  }
		}
// ~by D.S.CHO NEW
    }

  for (l = 0; l < 2*INBFR_BYTES; l++)
    m_inbfr[l + INBFR_BYTES] = m_rdptr[l];

  m_rdptr += 2*INBFR_BYTES;
  m_incnt += 2*INBFR_BYTES*8;

  // by D.S.CHO 
  if(m_bEof) {
    if(m_rdbfr_bitcnt <= 0)
       m_bEndBits = true;	
    else 
       m_bEndBits = false;	
  } else {
    m_bEndBits = false;
  }
  // ~by D.S.CHO 
}	


/***********************************************************CommentBegin******
 *
 * -- bitstreamShowBits -- Shows the next "nbits" of the bitstream
 *
 * Author :		
 *	Paulo Nunes (IST) - Paulo.Nunes@lx.it.pt
 *
 * Created :		
 *	1-Mar-1996
 *
 * Purpose :		
 *	To see the next "nbits" bitstream bits without advancing the read
 *	pointer. 
 *
 * 
 * Arguments in : 	
 *	Bitstream * stream,	input bitstream.
 *	Int nbits,		number of bits to see.

 *
 * Arguments in/out :	
 *	
 *
 * Arguments out :	
 *	
 *
 * Return values :	
 *	The value of the "nbits" right adjusted in one UInt.
 *
 * Side effects :	
 *	
 *
 * Description :	
 *	This function allows to see the next "nbits" after the current
 *	position of the bitstream pointer without advancing the pointer. It
 *	returns "nbits" bitstream bits right aligned in one UInt (see
 *	example)  
 *  			  ______ 
 *  			  |    | 
 *  			  v    v
 *  	bitstream -> 0000001000100111101 
 *  			  ^
 *  			  |
 *  	bitstreamShowBits(bitstream, 6) = 17
 *  	(i.e. 00000000000000000000000000010001) 
 *  					^    ^ 
 *  					|    |
 *  	
 *
 * See also :
 *	
 *
 * Modified :		
 *	10-May-96 Paulo Nunes: change to allow reading words up to 32 bits.
 *
 ***********************************************************CommentEnd********/

unsigned int CInBitstream::bitstreamShowBits (int nbits)
{
  unsigned char              *v;
  unsigned int                b, b1;
  int                 c;

 /* to mask the n least significant bits of an integer */
  static unsigned int         msk[33] =
  {
    0x00000000, 0x00000001, 0x00000003, 0x00000007,
    0x0000000f, 0x0000001f, 0x0000003f, 0x0000007f,
    0x000000ff, 0x000001ff, 0x000003ff, 0x000007ff,
    0x00000fff, 0x00001fff, 0x00003fff, 0x00007fff,
    0x0000ffff, 0x0001ffff, 0x0003ffff, 0x0007ffff,
    0x000fffff, 0x001fffff, 0x003fffff, 0x007fffff,
    0x00ffffff, 0x01ffffff, 0x03ffffff, 0x07ffffff,
    0x0fffffff, 0x1fffffff, 0x3fffffff, 0x7fffffff,
    0xffffffff
  };

  if (nbits > 8 * sizeof (unsigned int))
    {
      fprintf (stderr, "ERROR: number of bits greater than size of UInt.\n");
      exit (1);
    }
  if (m_incnt < nbits) 
    {
       bitstreamFillBuffer ();// by D.S.CHO 
    }

  if(m_bEndBits) return 0; // by D.S.CHO 

  v = m_inbfr + ((INBFR_TOTAL_BITS - m_incnt) >> 3);
  b = (v[0] << 24) | (v[1] << 16) | (v[2] << 8) | v[3];
  c = ((m_incnt - 1) & 7) + 25;

  if (c < nbits)
    {
      b1 = v[4];
      return ((b << (nbits - c)) | (b1 >> (8 - (nbits - c)))) & msk[nbits];
    }
  else
    {
      return (b >> (c - nbits)) & msk[nbits];
    }
}	

/***********************************************************CommentBegin******
 *
 * -- bitstreamLookAheadBit -- shows the value of the next "n"th bit in the bitstream
 *
 * Author :		
 *	Paulo Nunes (IST) - Paulo.Nunes@lx.it.pt
 *
 * Created :		
 *	1-Mar-1996
 *
 * Purpose :		
 *	To see the value of the "n"th bit in the bitstream without advancing the read
 *	pointer. 
 *
 * 
 * Arguments in : 	
 *	Bitstream * stream,	input bitstream.
 *	Int nth_bit,			the relative bit position.
 *
 *
 * Arguments in/out :	
 *	
 *
 * Arguments out :	
 *	
 *
 * Return values :	
 *	The value of the "n"th bit in the bitstream relatively to the
 *	current position.
 *
 * Side effects :	
 *	
 *
 * Description :	
 *		Returns the value of the n-th bit in the bitstream.
 *		The next bit in the bitstream can be obtained by 
 *		bitstreamLookAheadBit(bitstream,1); The use of nth_bit=0
 *		is not allowed. The bitstream pointer is not advanced and
 *		nth_bit can be in the range [1,64].
 *  	
 *
 * See also :
 *	
 *
 * Modified :		Noel Brady.
 *
 *
 ***********************************************************CommentEnd********/

unsigned int CInBitstream::bitstreamLookAheadBit (int nth_bit)
{
  unsigned char              *v;
  unsigned int                b, offset;
  int                 c;

  if (nth_bit > 8 * INBFR_BYTES || nth_bit < 1)
    {
      fprintf (stderr, "ERROR(bitstreamLookAheadBit): Parameter out of range.\n");
      exit (1);
    }
  if (m_incnt < nth_bit)
    bitstreamFillBuffer (); // by D.S.CHO

  if(m_bEndBits) return 0;  // by D.S.CHO 

  offset = INBFR_TOTAL_BITS - m_incnt + nth_bit - 1;

  v = m_inbfr + (offset >> 3);
  b = v[0];
  c = 7 - (offset & 7);

  return (b >> c) & 1;
}


/***********************************************************CommentBegin******
 *
 * -- bitstreamFlushBits -- Advances the bitstream read pointer
 *
 * Author :		
 *	Paulo Nunes (IST) - Paulo.Nunes@lx.it.pt
 *
 * Created :		
 *	1-Mar-1996
 *
 * Purpose :		
 *	To advance the bitstream read pointer.
 * 
 * Arguments in : 	
 *	Bitstream * stream,	input bitstream; 
 *	Int nbits,		number of bits positions to advance the
 *	                        pointer. 
 *
 *
 * Arguments in/out :	
 *	
 *
 * Arguments out :	
 *	
 *
 * Return values :	
 *	
 *
 * Side effects :	
 *	
 *
 * Description :	
 *	Advances "nbits" the bitstream read pointer and calls the function
 *	bitstreamFillBuffer to reload the bitstream buffer whenever
 *	necessary. 
 *
 *
 * See also :
 *	bitstreamFillBuffer.
 *
 * Modified :		
 *	
 *
 ***********************************************************CommentEnd********/

void CInBitstream::bitstreamFlushBits (int nbits)
{
  m_bitcnt += nbits;
  m_incnt -= nbits;

  if(m_bEof)		  	// by D.S.CHO 
      m_rdbfr_bitcnt -= nbits;	// by D.S.CHO 
  if (m_incnt < 0)
    bitstreamFillBuffer ();// by D.S.CHO 
} 


/***********************************************************CommentBegin******
 *
 * -- bitstreamReadBits -- Reads the next next "nbits" from the bitstream
 *
 * Author :		
 *	Paulo Nunes (IST) - Paulo.Nunes@lx.it.pt
 *
 * Created :		
 *	1-Mar-1996
 *
 * Purpose :		
 *	To read the next "nbits" from the bitstream.
 * 
 * Arguments in : 	
 *	Bitstream * stream,	input bitstream.
 *	Int nbits,		number of bits to read.
 *
 *
 * Arguments in/out :	
 *	
 *
 * Arguments out :	
 *	
 *
 * Return values :	
 *	
 *
 * Side effects :	
 *	
 *
 * Description :	
 *	Reads the next "nbits" of the bitstream and advances the bitsttream
 *	read pointer. 
 *
 * See also :
 *	bitstreamFlushBits.
 *
 * Modified :		
 *	
 *
 ***********************************************************CommentEnd********/

unsigned int CInBitstream::bitstreamReadBits (int nbits)
{
  unsigned int                l;
  if (nbits==0) return 0;	// by D.S.CHO NEW
  l = bitstreamShowBits (nbits); // by D.S.CHO 
  bitstreamFlushBits (nbits);

  return l;
} 



/***********************************************************CommentBegin******
 *
 * -- getBitstreamPosition -- Get the position of the next input on the file
 *                associated with the bitstream
 *
 * Author :
 *      Hisashi Saiga (SHARP)
 *
 * Created :
 *   25-FEB-1997
 *
 *
 * Purpose :
 *   Get the position of the next input on the file
 *   associated with the bitstream
 *
 *
 * Arguments in :
 *   int      offset0     offset
 *
 * Arguments in/out :
 *   long      *stream    the bitstream
 *
 *
 * Arguments out :
 *
 *
 * Return values :
 *
 *
 * Side effects :
 *
 *
 * Description :
 *
 *
 * See also :
 *
 *
 * Modified :
 *
 *
 ***********************************************************CommentEnd********/

int CInBitstream::getBitstreamPosition(int *offset)
{
   FILE   *fp;

   fp = m_fptr;
   *offset = ftell( fp );
   /* I don't know what value ftell() returns on error */
   return( 0 );
}

/***********************************************************CommentBegin******
 *
 * -- seekBitstream -- Seek the file associated with the bitstream
 *
 * Author :
 *      Hisashi Saiga (SHARP)
 *
 * Created :
 *   25-FEB-1997
 *
 * Purpose :
 *   Seek the file associated with the bitstream
 *
 * Arguments in :
 *   long      offset0      offset
 *   Bitstream   *stream    a bitstream that contains the original status
 * Arguments in/out :
 *
 *
 * Arguments out :
 *
 *
 * Return values :
 *
 *
 * Side effects :
 *
 *
 * Description :
 *
 *
 * See also :
 *
 *
 * Modified :
 *
 *
 ***********************************************************CommentEnd********/

int CInBitstream::seekBitstream(int offset)
{
   FILE   *fp;

   fp = m_fptr;
   if ( fseek( fp , ( long )offset , 0 ) == -1 ) return( 1 );
   else return( 0 );
}

/***********************************************************CommentBegin******
 *
 * -- clone_bitstream -- Clones a bitstream???
 *
 * Author :
 *   SHARP???
 *
 * Created :
 *   ???
 *
 * Purpose :
 *  ???
 *
 ***********************************************************CommentEnd********/
 
void CInBitstream::clone_bitstream(CInBitstream *stream0, CInBitstream *&stream1)
{
   //stream = ( Bitstream * )malloc ( sizeof( CInBitstream ) );
   stream1 = new CInBitstream();
   memcpy( stream1 , stream0 , sizeof( CInBitstream ) );
   /* Do not stop here; there is some more to do. */
   stream1 -> m_rdptr = stream0->m_rdbfr + ( stream0->m_rdptr - &( stream0->m_rdbfr[ 0 ] ) );   
}

/***********************************************************CommentBegin******
 *
 * -- bitstreamByteAlign -- Advances the bitstream read pointer until it is
 *                          byte aligned
 *
 * Author :		
 *	Luis Ducla-Soares (IST) - lds@lx.it.pt
 *
 * Created :		
 *	14-Jan-1997
 *
 * Purpose :		
 *	To advance the bitstream read pointer until it is byte aligned.
 * 
 * Arguments in : 	
 *	Bitstream * stream,	input bitstream; 
 *
 *
 * Arguments in/out :	
 *	
 *
 * Arguments out :	
 *	
 *
 * Return values :	
 *	Returns n_stuffed, the number of bits stuffed that were skipped.
 *
 * Side effects :	
 *	
 *
 * Description :	
 *	Advances the bitstream read pointer until the number of bits left in
 *      the buffer is a multiple of 8. Since the function
 *      bitstreamFillBuffer always reads bytes (multiples of 8 bits), this
 *      is the same as saying that the read pointer is byte aligned.
 *     
 *
 *
 * See also :
 *
 * Modified :		
 *	27.04.97 Luis Ducla-Soares: modified function to deal with
 *                                  bitstuffing as per VM6.0
 *	16.11.98 Sven Brandau: check correct value of stuffing bits
 ***********************************************************CommentEnd********/

unsigned int CInBitstream::bitstreamByteAlign ()
{
//  unsigned int n_stuffed, v, should_bits;
  unsigned int n_stuffed;
  
  n_stuffed = m_incnt % 8;
//  if (n_stuffed == 0)
//   n_stuffed = 8;
 
//  v = bitstreamShowBits(n_stuffed);// by D.S.CHO 

  if(m_bEndBits)	return 0; // by D.S.CHO

// by D.S.CHO 
/*
  should_bits = (1<<(n_stuffed-1)) -1;
  
  if (v!=should_bits) {
    fprintf( stderr, "stuffing bits not correct" );
    exit(-1);
  }
*/
//~by D.S.CHO 
  bitstreamFlushBits(n_stuffed);// by D.S.CHO 

  if(m_bEndBits)	return 0; // by D.S.CHO 
 
  return n_stuffed;
} 


/***********************************************************CommentBegin******
 *
 * -- bitstreamShowBitsByteAlign -- Shows the next "nbits" of the bitstream
 *
 * Author :		
 *	Luis Ducla Soares (IST) - lds@lx.it.pt
 *
 * Created :		
 *	28-April-1997
 *
 * Purpose :		
 *	To see the next "nbits" bitstream bits (byte aligned) without advancing the read
 *	pointer. 
 *
 * 
 * Arguments in : 	
 *	Bitstream * stream,	input bitstream.
 *	Int nbits,		number of bits to see after the byte alignment

 *
 * Arguments in/out :	
 *	
 *
 * Arguments out :	
 *	
 *
 * Return values :	
 *	The value of the "nbits" right adjusted in one UInt.
 *
 * Side effects :	
 *	
 *
 * Description :	
 *	This function allows to see the next "nbits" after the current
 *	position of the bitstream pointer without advancing the pointer. It
 *	returns "nbits" bitstream bits right aligned in one UInt (see
 *	example)  
 *  			  ______ 
 *  			  |    | 
 *  			  v    v
 *  	bitstream -> 0000001000100111101 
 *  			  ^
 *  			  |
 *  	bitstreamShowBits(bitstream, 6) = 17
 *  	(i.e. 00000000000000000000000000010001) 
 *  					^    ^ 
 *  					|    |
 *  	
 *
 * See also :
 *	
 *
 * Modified :		
 *      23.05.97 Luis Ducla-Soares: changed it to deal with the larger input
 *                                  buffer added by Paulo.
 *
 ***********************************************************CommentEnd********/

unsigned int CInBitstream::bitstreamShowBitsByteAlign (int nbits)
{
  unsigned char              *v;
  unsigned int                b;

  unsigned int n_stuffed;
  const unsigned int byte = 8;

 /* to mask the n least significant bits of an integer */
  static unsigned int         msk[33] =
  {
    0x00000000, 0x00000001, 0x00000003, 0x00000007,
    0x0000000f, 0x0000001f, 0x0000003f, 0x0000007f,
    0x000000ff, 0x000001ff, 0x000003ff, 0x000007ff,
    0x00000fff, 0x00001fff, 0x00003fff, 0x00007fff,
    0x0000ffff, 0x0001ffff, 0x0003ffff, 0x0007ffff,
    0x000fffff, 0x001fffff, 0x003fffff, 0x007fffff,
    0x00ffffff, 0x01ffffff, 0x03ffffff, 0x07ffffff,
    0x0fffffff, 0x1fffffff, 0x3fffffff, 0x7fffffff,
    0xffffffff
  };

  if (nbits > 8 * sizeof (unsigned int))
    {
      fprintf (stderr, "ERROR: number of bits greater than size of UInt.\n");
      exit (1);
    }

  n_stuffed = m_incnt % byte;

  if (n_stuffed == 0)    /* to enable the VM5.1 stuffing mode, remove the  comments */
    n_stuffed = byte;

//  if (m_incnt < (nbits + n_stuffed)) // by D.S.CHO NEW
  if (m_incnt < (nbits + (int)n_stuffed)) // by D.S.CHO NEW
    bitstreamFillBuffer (); // by D.S.CHO 

  if (m_bEndBits)   return 0; // by D.S.CHO

  v = m_inbfr + ((INBFR_TOTAL_BITS - m_incnt + n_stuffed) >> 3);
  b = (v[0] << 24) | (v[1] << 16) | (v[2] << 8) | v[3];

  return ((b >> (32 - nbits)) & msk[nbits]);
}	

/***********************************************************CommentBegin******
 *
 * -- checkBitStuffing -- Shows the next "nbits" of the bitstream
 *
 * Author :		
 *	Luis Ducla Soares (IST) - lds@lx.it.pt
 *
 * Created :		
 *	28-Apr-1997
 *
 * Purpose :		
 *	To see the next "nbits" bitstream bits (byte aligned) without advancing the read
 *	pointer. 
 *
 * 
 * Arguments in : 	
 *	Bitstream * stream,	input bitstream.
 *	Int nbits,		number of bits to see after the byte alignment

 *
 * Arguments in/out :	
 *	
 *
 * Arguments out :	
 *	
 *
 * Return values :	
 *	The value of the "nbits" right adjusted in one UInt.
 *
 * Side effects :	
 *	
 *
 * Description :	
 *	This function allows to see the next "nbits" after the current
 *	position of the bitstream pointer without advancing the pointer. It
 *	returns "nbits" bitstream bits right aligned in one UInt (see
 *	example)  
 *  			  ______ 
 *  			  |    | 
 *  			  v    v
 *  	bitstream -> 0000001000100111101 
 *  			  ^
 *  			  |
 *  	bitstreamShowBits(bitstream, 6) = 17
 *  	(i.e. 00000000000000000000000000010001) 
 *  					^    ^ 
 *  					|    |
 *  	
 *
 * See also :
 *	
 *
 * Modified :		
 *
 ***********************************************************CommentEnd********/

int CInBitstream::checkBitStuffing()
{
  unsigned int n_stuffed;
  const unsigned int byte = 8;
  
  static unsigned int     bit_stuffing[9] =
  {
    0x00, 0x00, 0x01, 0x03,
    0x07, 0x0f, 0x1f, 0x3f,
    0x7f
  };

  n_stuffed = m_incnt % byte;

  if (n_stuffed == 0)    /* to enable the VM5.1 stuffing mode, remove the  comments */
    n_stuffed = byte;

  if (bitstreamShowBits(n_stuffed) != bit_stuffing[n_stuffed])
    return 0;
  else
    return 1;
} 

/***********************************************************CommentBegin******
 *
 * -- bitstreamShowBitsAfterX -- Shows "nbits" of the bitstream starting after
 *                               the next xbits.
 *
 * Author :		
 *	Luis Ducla-Soares (IST) - lds@lx.it.pt
 *
 * Created :		
 *	23-May-1997
 *
 * Purpose :		
 *	To see "nbits" of the bitstream bits after the next xbits, without advancing the read
 *	pointer. 
 *
 * 
 * Arguments in : 	
 *	Bitstream * stream,	input bitstream.
 *	Int nbits,		number of bits to see.

 *
 * Arguments in/out :	
 *	
 *
 * Arguments out :	
 *	
 *
 * Return values :	
 *	The value of the "nbits" right adjusted in one UInt.
 *
 * Side effects :	
 *	
 *
 * Description :	
 *	This function allows to see "nbits" after the next xbits after current
 *	position of the bitstream pointer without advancing the pointer. It
 *	returns "nbits" bitstream bits right aligned in one UInt (see
 *	example)  
 *  			  ______ 
 *  			  |    | 
 *  			  v    v
 *   bitstream -> 0000001000100111101 
 *  	         	^
 *  			|
 *                   pointer
 *
 *
 *  	bitstreamShowBits(bitstream, 2, 6) = 9
 *  	(i.e. 00000000000000000000000000001001) 
 *  					^    ^ 
 *  					|    |
 *  	
 *
 * See also : bitstreamShowBits
 *	
 *
 * Modified :		
 *
 ***********************************************************CommentEnd********/

unsigned int CInBitstream::bitstreamShowBitsAfterX (int xbits, int nbits)
{
  unsigned char              *v;
  unsigned int                b, b1;
  int                 c;

 /* to mask the n least significant bits of an integer */
  static unsigned int         msk[33] =
  {
    0x00000000, 0x00000001, 0x00000003, 0x00000007,
    0x0000000f, 0x0000001f, 0x0000003f, 0x0000007f,
    0x000000ff, 0x000001ff, 0x000003ff, 0x000007ff,
    0x00000fff, 0x00001fff, 0x00003fff, 0x00007fff,
    0x0000ffff, 0x0001ffff, 0x0003ffff, 0x0007ffff,
    0x000fffff, 0x001fffff, 0x003fffff, 0x007fffff,
    0x00ffffff, 0x01ffffff, 0x03ffffff, 0x07ffffff,
    0x0fffffff, 0x1fffffff, 0x3fffffff, 0x7fffffff,
    0xffffffff
  };

  if (nbits > 8 * sizeof (unsigned int))
    {
      fprintf (stderr, "ERROR: number of bits greater than size of UInt.\n");
      exit (1);
    }

  if (xbits + nbits >= 64)
    {
      fprintf (stderr, "ERROR: tried to read ahead by more than 64 bits!\n");
      exit(1);
    }

  if (m_incnt < nbits)
    bitstreamFillBuffer (); // by D.S.CHO 

  if (m_bEndBits)     return 0; // by D.S.CHO

  v = m_inbfr + ((INBFR_TOTAL_BITS - m_incnt + xbits) >> 3);
  b = (v[0] << 24) | (v[1] << 16) | (v[2] << 8) | v[3];
  c = ((m_incnt - xbits - 1) & 7) + 25;

  if (c < nbits)
    {
      b1 = v[4];
      return ((b << (nbits - c)) | (b1 >> (8 - (nbits - c)))) & msk[nbits];
    }
  else
    {
      return (b >> (c - nbits)) & msk[nbits];
    }
}			


/***********************************************************CommentBegin******
 *
 * -- H263BitstreamByteAlign --
 *
 * Authors :		
 *      Gunn K Klungsoeyr (Telenor R&D/ ACTS MoMuSys)
 *
 * Created :		
 *	14-Jan-1998
 *
 * Purpose :	H.263-style Bytealigning of startcodes
 *
 * Arguments in : 	
 *
 * Arguments in/out :	
 *	Bitstream *stream,	input bitstream
 *
 * Arguments out :	
 *
 * Return values :	
 *
 * Side effects :	
 *
 * Description :	
 *
 * See also :
 *
 * Modified : 
 *	
 *
 ***********************************************************CommentEnd********/

#if 0
unsigned int CInBitstream::H263BitstreamByteAlign ()
{
  
  unsigned int n_stuffed;
  const unsigned int byte = 8;
  
  n_stuffed = m_incnt % byte;
  m_bitcnt += n_stuffed;
  m_incnt -= n_stuffed;

  return n_stuffed;
}
#endif

/***********************************************************CommentBegin******
 *
 * -- H263BitstreamShowBitsByteAlign --
 *
 * Author : Aasmund Sandvand  (Telenor R&D/ ACTS MoMuSys)		
 *	
 * Created : 23-Jun-1998			
 *
 * Purpose :		
 *	To see the next bytealigned (H.263 style) "nbits" bits in the bitstream
 *      without advancing the read pointer. 
 *
 * Arguments in : 	
 *	Bitstream * stream,	input bitstream.
 *	Int nbits,		number of bits to see after the byte alignment
 *
 * Arguments in/out :	
 *	
 * Arguments out :	
 *	
 * Return values :	
 *	The value of the "nbits" right adjusted in one UInt.
 *
 * Side effects :	
 *	
 * Description :
 *  	
 * See also :
 *	
 * Modified :		
 *    
 ***********************************************************CommentEnd********/

#if 0
unsigned int CInBitstream::H263BitstreamShowBitsByteAlign (int nbits)
{
  unsigned char              *v;
  unsigned int                b;

  unsigned int n_stuffed;
  const unsigned int byte = 8;

 /* to mask the n least significant bits of an integer */
  static unsigned int         msk[33] =
  {
    0x00000000, 0x00000001, 0x00000003, 0x00000007,
    0x0000000f, 0x0000001f, 0x0000003f, 0x0000007f,
    0x000000ff, 0x000001ff, 0x000003ff, 0x000007ff,
    0x00000fff, 0x00001fff, 0x00003fff, 0x00007fff,
    0x0000ffff, 0x0001ffff, 0x0003ffff, 0x0007ffff,
    0x000fffff, 0x001fffff, 0x003fffff, 0x007fffff,
    0x00ffffff, 0x01ffffff, 0x03ffffff, 0x07ffffff,
    0x0fffffff, 0x1fffffff, 0x3fffffff, 0x7fffffff,
    0xffffffff
  };
  
  if (nbits > 8 * sizeof (unsigned int))
    {
      fprintf (stderr, "ERROR: number of bits greater than size of UInt.\n");
      exit (1);
    }

  n_stuffed = m_incnt % byte;

  if (m_incnt < (nbits + n_stuffed))
    bitstreamFillBuffer ();

  if (m_bEndBits)    return 0; // by D.S.CHO 

  v = m_inbfr + ((INBFR_TOTAL_BITS - m_incnt + n_stuffed) >> 3);
  b = (v[0] << 24) | (v[1] << 16) | (v[2] << 8) | v[3];

  return ((b >> (32 - nbits)) & msk[nbits]);
}			
#endif


