/*******************************************************************************
 
This software module was originally developed by
 
        Dae-Sung Cho (dscho@saitgw.sait.samsung.co.kr), Samsung AIT
 
Those who intend to use this software module in hardware or software products
are advised that its use may infringe existing patents and other intellectual
property rights.
 
We disclaim all warranties with regard to this software, including
all implied warranties of merchantability and fitness.
In no event, Samsung AIT shall be liable for any special, indirect or
consequential damages or any damages whatsoever resulting from loss of
data or profits arising out of or in connection with the use or performance
of this software.
 
Samsung AIT retains the full right to use the code for its own purpose, assign
or donate the code to a third party and to inhibit third parties from using
the code. This copyright notice must be included in all copies or derivative
works.
 
Copyright (c) 1997, 1998 Samsung AIT all rights reserved
 
*********************************************************************************/

#include "defs.h" // by D.S.CHO NEW

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <assert.h>

#include "bitstream_o.h"

COutBitstream::COutBitstream()
{
	m_buffer = (unsigned char *)calloc(DEFAULT_BUFSIZE,sizeof(unsigned char)); //
	if(m_buffer==NULL)	{
		fprintf(stderr,"ERROR(calloc): memory allocation error.\n");
		exit(1);
	}
	m_bufsize = DEFAULT_BUFSIZE;
	m_bufcnt = 0;
	m_bytebuf = 0;
	m_bitnum = 8;
}

COutBitstream::~COutBitstream()
{
	if (m_buffer != NULL)	
	{
		free(m_buffer);
		m_buffer = NULL;
	}
}

//Packet *packetOpen()
void COutBitstream::bitstreamOpen()
{
	m_buffer = (unsigned char *)calloc(DEFAULT_BUFSIZE,sizeof(unsigned char)); //
	if(m_buffer==NULL)	{
		fprintf(stderr,"ERROR(calloc): memory allocation error.\n");
		exit(1);
	}
	m_bufsize = DEFAULT_BUFSIZE;
	m_bufcnt = 0;
	m_bytebuf = 0;
	m_bitnum = 8;
}

//void packetClose(Packet *pPacket)
void COutBitstream::bitstreamClose()
{
	if (m_buffer != NULL)	
	{
		free(m_buffer);
		m_buffer = NULL;
	}
}

//void packetInit(Packet *pPacket)
void COutBitstream::bitstreamInit()
{
	m_bufcnt = 0;
	m_bytebuf = 0;
	m_bitnum = 8;
}

//int putBits2Packet(int iValue, int iBits, Packet *pPacket)
int COutBitstream::bitstreamPutBits(int iValue, int iBits)
{
	int m;

	if (iBits<=0) return 0;	// by D.S.CHO NEW
	assert(m_bitnum!=0);
	for (m = 1 << (iBits - 1); m > 0; m >>= 1) {
        	m_bytebuf <<= 1;
        	if (iValue & m) m_bytebuf |= 1;
        	if (!(--(m_bitnum))) {
				if(m_bufcnt >= m_bufsize) {
					m_buffer = (unsigned char *)realloc(m_buffer, m_bufsize+BFR_INC);
					if(m_buffer==NULL) {
      						fprintf(stderr,"ERROR(malloc): memory allocation error.\n");
      						exit(1);
					}
					m_bufsize += BFR_INC;
				}
				m_buffer[m_bufcnt] = (unsigned char)m_bytebuf;
                m_bufcnt ++;
				m_bytebuf = 0;
				m_bitnum = 8;
        	}
	}
 
	return iBits;
}

//void flushBits2Packet(Packet *pPacket)
void COutBitstream::bitstreamFlush()
{
	assert(m_bitnum!=0);
	if(m_bitnum!=8){
		m_bytebuf <<= m_bitnum;
		if(m_bufcnt >= m_bufsize) {
			m_buffer = (unsigned char *)realloc(m_buffer, m_bufsize+BFR_INC);
			if(m_buffer==NULL) {
      				fprintf(stderr,"ERROR(malloc): memory allocation error.\n");
      				exit(1);
			}
			m_bufsize += BFR_INC;
		}
		m_buffer[m_bufcnt] = (unsigned char)m_bytebuf;
 		m_bufcnt ++;
		m_bytebuf = 0;
        m_bitnum = 8;
  	}
}

//int getPacketSize(Packet *pPacket)
int COutBitstream::getBitstreamLength()
{
	return (m_bufcnt);
}

//char *getPacketBuffer(Packet *pPacket)
unsigned char* COutBitstream::getBitstreamBuffer()
{
	return (m_buffer);
}

void COutBitstream::bitstreamWrite(FILE *fp)
{
	fwrite(m_buffer, sizeof(unsigned char), m_bufcnt, fp);
}

void COutBitstream::bitstreamAppend(COutBitstream *appendedBits)	// NEW
{	
	unsigned char *pBuffer = appendedBits->getBitstreamBuffer();
	int nLength = appendedBits->getBitstreamLength();
	int nLastBits = appendedBits->getLastBitsNumber();
	for(int i=0; i<nLength; i++) 
		bitstreamPutBits(pBuffer[i], 8);
	if(nLastBits!=0) {
		//printf("nLastBits!=0   ----------------------------------------\n");
		appendedBits->bitstreamFlush();
		int iValue = appendedBits->getLastByteValue();
		bitstreamPutBits(iValue, nLastBits);
	}
}

int COutBitstream::getLastBitsNumber()	// NEW
{
	return (8-m_bitnum);
}

int COutBitstream::getLastByteValue()		// NEW
{
	return m_bytebuf;
}
