/*************************************************************************************
 
 This software module was originally developed by
 
	Dae-Sung Cho 	(dscho@sait.samsung.co.kr), Samsung AIT
 
 and edited by 

	Dae-Sung Cho	(dscho@sait.samsung.co.kr), Samsung AIT

 Those who intend to use this software module in hardware or software products
 are advised that its use may infringe existing patents and other intellectual
 property rights.
 
 We disclaim all warranties with regard to this software, including
 all implied warranties of merchantability and fitness.
 In no event, Samsung AIT shall be liable for any special, indirect or
 consequential damages or any damages whatsoever resulting from loss of
 data or profits arising out of or in connection with the use or performance
 of this software.
 
 Samsung AIT retains the full right to use the code for its own purpose, assign
 or donate the code to a third party and to inhibit third parties from using
 the code. This copyright notice must be included in all copies or derivative
 works.
 
 Copyright (c) 1997-2000 Samsung AIT all rights reserved
 

 Required module :
 
 Authors : 

 Changes :
    	date      	programmers     comment

*************************************************************************************/
#include "defs.h" // by D.S.CHO NEW

#include <stdio.h>
#include <stdlib.h>
#include <assert.h>

#include "bsac_parser.h"

CBSACParser::CBSACParser()
{
	m_pInBitstream = new CInBitstream;
	m_pOutBitstream = new COutBitstream;
}

// by D.S.CHO SS
/*
CBSACParser::CBSACParser(char *infile)
{
	m_pInBitstream = new CInBitstream(infile);
	m_pOutBitstream = new COutBitstream;
}
*/
//~by D.S.CHO SS

CBSACParser::CBSACParser(FILE *fp)
{
	m_pInBitstream = new CInBitstream(fp);
	m_pOutBitstream = new COutBitstream;
}


CBSACParser::~CBSACParser()
{
	if (m_pInBitstream!=NULL) 
		delete m_pInBitstream;
	if (m_pOutBitstream!=NULL)
		delete m_pOutBitstream;
}

void CBSACParser::openParser()
{
	if (m_pInBitstream==NULL) 
		m_pInBitstream = new CInBitstream;
	if (m_pOutBitstream==NULL)
		m_pOutBitstream = new COutBitstream;
}

// by D.S.CHO SS
/*
void CBSACParser::openParser(char *infile)
{
	if (m_pInBitstream==NULL) 
		m_pInBitstream = new CInBitstream;
	m_pInBitstream -> bitstreamOpen(infile);

	if (m_pOutBitstream==NULL)
		m_pOutBitstream = new COutBitstream;
}
*/
//~by D.S.CHO SS

void CBSACParser::closeParser() 
{
	if (m_pInBitstream!=NULL) 
		delete m_pInBitstream;
	if (m_pOutBitstream!=NULL)
		delete m_pOutBitstream;
}

int CBSACParser::parseFrameHeader()
{
	unsigned int uiValue;
	bool isEOF = false;
	unsigned char bsac_header;

	if(m_pInBitstream->getInputFile() == NULL)
		return EOF;

	uiValue = m_pInBitstream->bitstreamReadBits(NUMBITS_ONE_BYTE);
	isEOF = m_pInBitstream->isBitstreamEnd();
	if(isEOF) 	return EOF;
	m_pOutBitstream->bitstreamPutBits((int)uiValue,NUMBITS_ONE_BYTE);
	m_pOutBitstream->bitstreamFlush(); 

	bsac_header = (unsigned char) (0xFF & uiValue);
	m_nNumChannel = (bsac_header>>5) & 0x07;

	return 0;
}

int CBSACParser::parseFrame()
{
	unsigned int uiValue;
	int iFrameLength;
	bool isEOF = false;

	if(m_pInBitstream->getInputFile() == NULL)
		return EOF;
	uiValue = m_pInBitstream->bitstreamShowBits(NUMBITS_TWO_BYTES);
	isEOF = m_pInBitstream->isBitstreamEnd();
	if (isEOF)	return EOF;
	if(m_nNumChannel == 1) {
		iFrameLength = (int)((uiValue & 0xFFFF) >> 6);	// first 10 bits mean the frame length	??????
	} else {
		iFrameLength = (int)((uiValue & 0xFFFF) >> 5);	// first 11 bits mean the frame length
	}
	if (iFrameLength==0) return EOF;
	for(int i=0; i<iFrameLength; i++) {
		uiValue = m_pInBitstream->bitstreamReadBits(NUMBITS_ONE_BYTE);
		isEOF = m_pInBitstream->isBitstreamEnd();
		if (isEOF)	return EOF;
		m_pOutBitstream->bitstreamPutBits((int)uiValue, NUMBITS_ONE_BYTE);
	}
	m_pOutBitstream->bitstreamFlush(); 
	return 0;
}



