/*************************************************************************************

 This software module was originally developed by

        Dae-Sung Cho    (dscho@sait.samsung.co.kr), Samsung AIT

 and edited by

        Dae-Sung Cho    (dscho@sait.samsung.co.kr), Samsung AIT

 Those who intend to use this software module in hardware or software products
 are advised that its use may infringe existing patents and other intellectual
 property rights.

 We disclaim all warranties with regard to this software, including
 all implied warranties of merchantability and fitness.
 In no event, Samsung AIT shall be liable for any special, indirect or
 consequential damages or any damages whatsoever resulting from loss of
 data or profits arising out of or in connection with the use or performance
 of this software.

 Samsung AIT retains the full right to use the code for its own purpose, assign
 or donate the code to a third party and to inhibit third parties from using
 the code. This copyright notice must be included in all copies or derivative
 works.

 Copyright (c) 1997-2000 Samsung AIT all rights reserved


 Required module :

 Authors :

 Changes :
        date            programmers     comment

*************************************************************************************/

#include <stdio.h>
#include <memory.h>

#include "defs.h" // by D.S.CHO NEW
// by D.S.CHO 
#include "assert.h"
#include "bitstream_i.h"
#include "mp4video_codehead.h"
#include "mp4video_analyzer.h"
//~by D.S.CHO 

CMp4VideoAnalyzer::CMp4VideoAnalyzer () :
	m_t (0), 
	m_tPastRef (0), 
	m_tFutureRef (0),
	m_bCodedFutureRef (true),
	m_tModuloBaseDecd (0), 
	m_tModuloBaseDisp (0),
	m_uiSprite (0)
{
	m_vopmd.iVopConstantAlphaValue = 255;
}

CMp4VideoAnalyzer::~CMp4VideoAnalyzer ()
{
	if (m_pInBitstream!=NULL) {
		delete m_pInBitstream;
		m_pInBitstream = NULL;
	}
}

void CMp4VideoAnalyzer::AnalyzeVolHeader (unsigned char *pStream, int nLength)
{
	// new of bitstream buffer
	m_pInBitstream = new CInBitstream(pStream, nLength);	

	if (m_pInBitstream->bitstreamShowBits(NUMBITS_SHORT_HEADER_START_CODE) == SHORT_VIDEO_START_MARKER)  
	{ 
#ifdef _DSCHO_DBG_
		fprintf (stderr, "\nBitstream with short header format detected\n"); 
#endif
		m_bvideo_plane_with_short_header = true;
		m_pInBitstream -> bitstreamReadBits(22);
		video_plane_with_short_header();
	}
	else {

		m_bvideo_plane_with_short_header = false;
#ifdef _DSCHO_DBG_
		fprintf(stderr,"\nBitstream without short headers detected\n");
#endif
		decodeVOHead ();
#ifdef _DSCHO_DBG_
		fprintf (stderr, "VO %d...\n", m_uivoid);
#endif
		decodeVOLHead ();
	} 

	if(m_volmd.volType == ENHN_LAYER) { // check scalability type
		m_volmd.bSpatialScalability = false;
		if(m_volmd.iHierarchyType == 0 || m_volmd.bShapeOnly) {
			m_volmd.bSpatialScalability = true;
		}
		else { 	
			m_volmd.bSpatialScalability = false;
		}
// by D.S.CHO SS
		m_volmd.bTemporalScalability = false;
		if(m_volmd.iHierarchyType == 1) {
			m_volmd.bTemporalScalability = true;
		}
		else { 	
			m_volmd.bTemporalScalability = false;
		}
//~by D.S.CHO SS
	}
	// buffers for Temporal Scalabe Decoding	End	 Sharp(1998-02-10)
	m_iClockRateScale = 1; // added by Sharp (98/6/26)

	// Set sprite_transmit_mode to STOP	for the duration of VOL if (fSptUsage () == 0),
	// and later set to PIECE by decode_init_sprite () if (fSptUsage () == 1)
	m_vopmd.SpriteXmitMode = STOP;
	
	// delete of bitstream buffer
	if (m_pInBitstream!=NULL) {
		delete m_pInBitstream;
		m_pInBitstream = NULL;
	}
}

int CMp4VideoAnalyzer::AnalyzeVopHeader (unsigned char *pStream, int nLength)
{
	// new of bitstream buffer
	m_pInBitstream = new CInBitstream(pStream, nLength);	

	if(!m_bvideo_plane_with_short_header) 
	{
		//sprite piece should not come here
		assert ((m_vopmd.SpriteXmitMode == STOP) || ( m_vopmd.SpriteXmitMode == PAUSE));
	
		if (findStartCode () == EOF)
			return EOF;
	
		bool bCoded = decodeVOPHead (); // set the bounding box here
#ifdef _DSCHO_DBG_
		fprintf(stderr,"\t Time ... %d (%lf) sec\n", m_t, m_t / (double)m_volmd.iClockRate);
#endif
		if(bCoded == false)
			fprintf(stderr,"\tNot coded.\n");
	} else {
		int ret = decodeH263Head ();
		return ret;
	}

	// delete of bitstream buffer
	if (m_pInBitstream!=NULL) {
		delete m_pInBitstream;
		m_pInBitstream = NULL;
	}

	return 1;
}

int CMp4VideoAnalyzer::decodeH263Head ()
{
	static bool first_time = true;

	if (!first_time) 
	{
		while ( m_pInBitstream -> bitstreamShowBits(NUMBITS_SHORT_HEADER_START_CODE) != SHORT_VIDEO_START_MARKER)
		{
			if(m_pInBitstream->isBitstreamEnd())  // [FDS] 
				return EOF;
			m_pInBitstream -> bitstreamReadBits(1);
		}
		m_pInBitstream -> bitstreamReadBits(22);
		video_plane_with_short_header(); 
	}
	else
		first_time = false;

	m_bUseGOV=false; 
	m_bLinkisBroken=false;
	m_vopmd.iRoundingControl=0;
	m_vopmd.iintraDcSwitchThr=0; 
	m_vopmd.binterlace=false;
	m_vopmd.bAlternateScan=false;
	m_t=1; 
// by D.S.CHO SS
/*
	m_vopmd.mvInfoForward.uiFCode=1; 
	m_vopmd.mvInfoForward.uiScaleFactor = 1 << (m_vopmd.mvInfoForward.uiFCode - 1);
	m_vopmd.mvInfoForward.uiRange = 16 << m_vopmd.mvInfoForward.uiFCode;
	//	Added for error resilient mode by Toshiba(1998-1-16)
	m_vopmd.mvInfoBackward.uiFCode = 1;
	//	End Toshiba(1998-1-16)
*/
	m_vopmd.uiVopFcodeForward = 1;
	m_vopmd.uiVopFcodeBackward = 1;
//~by D.S.CHO SS
	m_vopmd.bShapeCodingType=1;

	// set time stamps for Base/Temporal-Enhc/Spatial-Enhc Layer	Modified by Sharp(1998-02-10)
	if(m_volmd.volType == BASE_LAYER) {
		if(m_vopmd.vopPredType==IVOP || m_vopmd.vopPredType==PVOP) {
			m_tPastRef = m_tFutureRef; 
			m_tFutureRef = m_t; 
		}
		
	}
	// set time stamps for Base/Temporal-Enhc/Spatial-Enhc Layer	End 	    Sharp(1998-02-10)

	// select reference frames for Base/Temporal-Enhc/Spatial-Enhc Layer	End	    Sharp(1998-02-10)

	switch(m_vopmd.vopPredType)
	{
	case IVOP:
#ifdef _DSCHO_DBG_		
		fprintf(stderr,"\tIVOP");
#endif
/*Added by SONY (98/03/30)*/
		if(m_bLinkisBroken == true && m_bUseGOV == true)        m_bLinkisBroken = false;
/*Added by SONY (98/03/30) END*/
		break;
	case PVOP:
#ifdef _DSCHO_DBG_
		fprintf(stderr,"\tPVOP (reference: t=%d)",m_tPastRef);
#endif
		break;
	default:
		break;
	}
#ifdef _DSCHO_DBG_
	fprintf(stderr,"\n");
#endif
/*Added by SONY (98/03/30)*/
	if(m_bLinkisBroken == true && m_bUseGOV == true)
		fprintf(stderr,"WARNING: broken_link = 1  --- Output image must be broken.\n");
/*Added by SONY (98/03/30) End*/

	return true; 
}

void CMp4VideoAnalyzer::video_plane_with_short_header()
{
	/*unsigned uiTemporalReference = */m_pInBitstream -> bitstreamReadBits (8); // by D.S.CHO NEW
	m_pInBitstream -> bitstreamReadBits(5);
	unsigned uiSourceFormat = m_pInBitstream -> bitstreamReadBits (3);

	if (uiSourceFormat==1) {
		//fprintf(stderr,"Sub-QCIF, 128x96, 8 macroblocks/gob, 6 gobs in vop\n");
		uiNumGobsInVop=6;
		uiNumMacroblocksInGob=8;
		m_volmd.iVolWidth=m_ivolWidth=128;
		m_volmd.iVolHeight=m_ivolHeight=96;
	} else if (uiSourceFormat==2) {
		//fprintf(stderr,"QCIF, 176x144, 11 macroblocks/gob, 9 gobs in vop\n");
		uiNumGobsInVop=9;
		uiNumMacroblocksInGob=11;
		m_volmd.iVolWidth=m_ivolWidth=176;
		m_volmd.iVolHeight=m_ivolHeight=144;
	} else if (uiSourceFormat==3) {
		//fprintf(stderr,"CIF, 352x288, 22 macroblocks/gob, 18 gobs in vop\n");
		uiNumGobsInVop=18;
		uiNumMacroblocksInGob=22;
		m_volmd.iVolWidth=m_ivolWidth=352;
		m_volmd.iVolHeight=m_ivolHeight=288;
	} else if (uiSourceFormat==4) {
		//fprintf(stderr,"4CIF, 704x576, 88 macroblocks/gob, 18 gobs in vop\n");
		uiNumGobsInVop=18;
		uiNumMacroblocksInGob=88;
		m_volmd.iVolWidth=m_ivolWidth=704;
		m_volmd.iVolHeight=m_ivolHeight=576;
	} else if (uiSourceFormat==5) {
		//fprintf(stderr,"16CIF, 1408x1152, 352 macroblocks/gob, 18 gobs in vop\n");
		uiNumGobsInVop=18;
		uiNumMacroblocksInGob=352;
		m_volmd.iVolWidth=m_ivolWidth=1408;
		m_volmd.iVolHeight=m_ivolHeight=1152;
	} else {
		fprintf(stderr,"Wrong Source Format in video_plane_with_short_header()\n");
		exit (0);
	}
	unsigned uiPictureCodingType = m_pInBitstream -> bitstreamReadBits(1);
	if (uiPictureCodingType==0) 
		m_vopmd.vopPredType=IVOP;
	else
		m_vopmd.vopPredType=PVOP;

	m_pInBitstream -> bitstreamReadBits(4);
	unsigned uiVopQuant = m_pInBitstream -> bitstreamReadBits(5);
	//fprintf(stderr,"vop_quant (0..31)             %d\n",uiVopQuant);
	m_vopmd.intStepI=uiVopQuant; 
	m_vopmd.intStep=uiVopQuant; // idem
	m_pInBitstream -> bitstreamReadBits(1);
	do {
		uiPei = m_pInBitstream -> bitstreamReadBits(1);
		//fprintf(stderr,"pei gelezen %d\n",uiPei);
		if (uiPei==1) {			
			m_pInBitstream -> bitstreamReadBits(8);
		}
// [bezig]
	} while (uiPei==1);

	m_uivoid = 1; 
	m_volmd.iClockRate = 30; 
	m_iNumBitsTimeIncr = 4; 
	m_volmd.bShapeOnly=false; 
	m_volmd.fAUsage = RECTANGLE; 
	m_volmd.bAdvPredDisable = true; 
	m_uiSprite = false; 
	m_volmd.bNot8Bit=false; 
	m_volmd.uiQuantPrecision=5;
	m_volmd.nBits=8;
	m_volmd.fQuantizer=Q_H263; 
	m_volmd.bDataPartitioning=false;
	m_volmd.bReversibleVlc=false;
	m_volmd.volType=BASE_LAYER;
	m_volmd.ihor_sampling_factor_n=1;
	m_volmd.ihor_sampling_factor_m=1;
	m_volmd.iver_sampling_factor_n=1;
	m_volmd.iver_sampling_factor_m=1;
	m_volmd.bDeblockFilterDisable=true;
	
// NEWPRED
	m_volmd.bNewpredEnable=false;
// ~NEWPRED
}

int CMp4VideoAnalyzer::findStartCode()
{
	// ensure byte alignment
//	m_pInBitstream->flush ();	// by D.S.CHO SS

	int bUserData;
	do {
		bUserData = 0;
		while(m_pInBitstream->bitstreamShowBits(NUMBITS_START_CODE_PREFIX)!=START_CODE_PREFIX)
		{
			m_pInBitstream->bitstreamReadBits(8);
			if(m_pInBitstream->isBitstreamEnd())
				return EOF;
		}
		m_pInBitstream->bitstreamReadBits(NUMBITS_START_CODE_PREFIX);
		if(m_pInBitstream->bitstreamShowBits(NUMBITS_START_CODE_SUFFIX)==USER_DATA_START_CODE)
			bUserData = 1;
	} while(bUserData);

	return 0;
}

void CMp4VideoAnalyzer::decodeVOHead ()
{
	findStartCode();
 	// Added for data partitioning mode By Toshiba(1998-1-16:DP+RVLC)(for Skip Session_Start_Code)
 	if( m_pInBitstream->bitstreamShowBits(NUMBITS_START_CODE_SUFFIX)==SESSION_START_CODE){
 		m_pInBitstream -> bitstreamReadBits (NUMBITS_START_CODE_SUFFIX);
 		if (findStartCode () == EOF)
 			assert(false);
 	}
 	// End Toshiba(1998-1-16:DP+RVLC)
	unsigned uiVoStartCode = m_pInBitstream -> bitstreamReadBits (NUMBITS_VO_START_CODE);
	assert(uiVoStartCode == VO_START_CODE);

	m_uivoid = m_pInBitstream -> bitstreamReadBits (NUMBITS_VO_ID);
}

void CMp4VideoAnalyzer::decodeVOLHead ()
{
	findStartCode();
	unsigned uiVolStartCode = m_pInBitstream -> bitstreamReadBits (NUMBITS_VOL_START_CODE);
	assert(uiVolStartCode == VOL_START_CODE);
	/*unsigned uiVOLId =*/ m_pInBitstream -> bitstreamReadBits (NUMBITS_VOL_ID); // by D.S.CHO NEW
	/*unsigned uiRandom =*/ m_pInBitstream->bitstreamReadBits (1);			// VOL_Random_Access // by D.S.CHO NEW
	/*unsigned uiOLType =*/ m_pInBitstream -> bitstreamReadBits(8);		// VOL_type_indication // by D.S.CHO NEW
	unsigned uiOLI = m_pInBitstream -> bitstreamReadBits (1);		// VOL_Is_Object_Layer_Identifier, useless flag for now
	if(uiOLI)
	{
		m_volmd.uiVerID = m_pInBitstream -> bitstreamReadBits (4);	// video_oject_layer_verid
		m_pInBitstream -> bitstreamReadBits (3);					// video_oject_layer_priority
	}
	else{
		m_volmd.uiVerID = 1;
	}
	
	unsigned uiAspect = m_pInBitstream -> bitstreamReadBits (4);
	if(uiAspect==15) // extended PAR
	{
		/*unsigned iParWidth =*/ m_pInBitstream -> bitstreamReadBits (8); // by D.S.CHO NEW
		/*unsigned iParHeight =*/ m_pInBitstream -> bitstreamReadBits (8); //  by D.S.CHO NEW
	}

	unsigned uiMark;
	unsigned uiCTP = m_pInBitstream -> bitstreamReadBits (1);		//VOL_Control_Parameter, useless flag for now
	if(uiCTP)
	{
		/*unsigned uiChromaFormat =*/ m_pInBitstream -> bitstreamReadBits (2); // by D.S.CHO NEW
		/*unsigned uiLowDelay =*/ m_pInBitstream -> bitstreamReadBits (1); // by D.S.CHO NEW
		unsigned uiVBVParams = m_pInBitstream -> bitstreamReadBits (1);
		
		if(uiVBVParams)
		{
			/*unsigned uiFirstHalfBitRate =*/ m_pInBitstream -> bitstreamReadBits (15); // by D.S.CHO NEW
			uiMark = m_pInBitstream -> bitstreamReadBits (1);
			assert(uiMark==1);
			/*unsigned uiLatterHalfBitRate =*/ m_pInBitstream -> bitstreamReadBits (15); // by D.S.CHO NEW
			uiMark = m_pInBitstream -> bitstreamReadBits (1);
			assert(uiMark==1);
			/*unsigned uiFirstHalfVbvBufferSize =*/ m_pInBitstream -> bitstreamReadBits (15); // by D.S.CHO NEW
			uiMark = m_pInBitstream -> bitstreamReadBits (1);
			assert(uiMark==1);
			/*unsigned uiLatterHalfVbvBufferSize =*/ m_pInBitstream -> bitstreamReadBits (3); // by D.S.CHO NEW
			/*unsigned uiFirstHalfVbvBufferOccupany =*/ m_pInBitstream -> bitstreamReadBits (11); // by D.S.CHO NEW
			uiMark = m_pInBitstream -> bitstreamReadBits (1);
			assert(uiMark==1);
			/*unsigned uiLatterHalfVbvBufferOccupany =*/ m_pInBitstream -> bitstreamReadBits (15); // by D.S.CHO NEW
			uiMark = m_pInBitstream -> bitstreamReadBits (1);
			assert(uiMark==1);
		}
	}

  	unsigned uiAUsage = m_pInBitstream -> bitstreamReadBits (NUMBITS_VOL_SHAPE);

// MAC (SB) 1-Dec-99
	if (uiAUsage==3) { // gray scale
		if (m_volmd.uiVerID!=1) {
			m_volmd.iAlphaShapeExtension = m_pInBitstream -> bitstreamReadBits (4);
//			m_volmd.iAuxCompCount =  CVideoObject::getAuxCompCount(m_volmd.iAlphaShapeExtension);	// by D.S.CHO SS
		} else {
			m_volmd.iAuxCompCount = 1;
		}
	} else
		m_volmd.iAuxCompCount = 0;
//~MAC

	uiMark = m_pInBitstream -> bitstreamReadBits (1);
	assert(uiMark==1);
	m_volmd.iClockRate = m_pInBitstream -> bitstreamReadBits (NUMBITS_TIME_RESOLUTION);
#ifdef _DSCHO_DBG_
	//cout << m_volmd.iClockRate << "\n";
	fprintf (stderr,"iClockRate = %d\n", m_volmd.iClockRate);
#endif
	uiMark = m_pInBitstream -> bitstreamReadBits (1);
	assert(uiMark==1);
	int iClockRate = m_volmd.iClockRate - 1;
	assert (iClockRate < 65536);
	if(iClockRate>0)
	{
		for (m_iNumBitsTimeIncr = 1; m_iNumBitsTimeIncr < NUMBITS_TIME_RESOLUTION; m_iNumBitsTimeIncr++) {	
			if (iClockRate == 1)			
				break;
			iClockRate = (iClockRate >> 1);
		}
	}
	else
		m_iNumBitsTimeIncr = 0;

	unsigned uiFixFrameRate = m_pInBitstream -> bitstreamReadBits (1);
	if(uiFixFrameRate)
	{
		unsigned uiFixedVOPTimeIncrement = 0;
		if(m_iNumBitsTimeIncr!=0)
			uiFixedVOPTimeIncrement = m_pInBitstream -> bitstreamReadBits (m_iNumBitsTimeIncr);
		// not used
	}

	if(uiAUsage==2)  // shape-only mode
	{
//OBSS_SAIT_991015
		if(m_volmd.uiVerID == 2) {
			m_volmd.volType = (m_pInBitstream -> bitstreamReadBits (1) == 0) ? BASE_LAYER : ENHN_LAYER;
			m_volmd.ihor_sampling_factor_n = 1;
			m_volmd.ihor_sampling_factor_m = 1;
			m_volmd.iver_sampling_factor_n = 1;
			m_volmd.iver_sampling_factor_m = 1;	
			m_volmd.ihor_sampling_factor_n_shape = 1;
			m_volmd.ihor_sampling_factor_m_shape = 1;
			m_volmd.iver_sampling_factor_n_shape = 1;
			m_volmd.iver_sampling_factor_m_shape = 1;	
			if (m_volmd.volType == ENHN_LAYER)	{
				m_volmd.ihor_sampling_factor_n_shape = m_pInBitstream->bitstreamReadBits (5);
				m_volmd.ihor_sampling_factor_m_shape = m_pInBitstream->bitstreamReadBits (5);
				m_volmd.iver_sampling_factor_n_shape = m_pInBitstream->bitstreamReadBits (5);
				m_volmd.iver_sampling_factor_m_shape = m_pInBitstream->bitstreamReadBits (5);
				m_volmd.ihor_sampling_factor_n = m_volmd.ihor_sampling_factor_n_shape;
				m_volmd.ihor_sampling_factor_m = m_volmd.ihor_sampling_factor_m_shape;
				m_volmd.iver_sampling_factor_n = m_volmd.iver_sampling_factor_n_shape;
				m_volmd.iver_sampling_factor_m = m_volmd.iver_sampling_factor_m_shape;	
			}
		}
//~OBSS_SAIT_991015
		/*unsigned uiResyncMarkerDisable =*/ m_pInBitstream -> bitstreamReadBits (1); // by D.S.CHO NEW

		// default to some values - probably not all needed
		m_volmd.bShapeOnly=true;
		m_volmd.fAUsage=ONE_BIT;
		m_volmd.bAdvPredDisable = 0;
		m_volmd.fQuantizer = Q_H263;
//OBSS_SAIT_991015
		m_volmd.bSadctDisable = 1;
		m_volmd.bNewpredEnable = 0;
		m_volmd.bQuarterSample = 0;
//~OBSS_SAIT_991015
		m_volmd.bDeblockFilterDisable = true;		
		m_uiSprite = 0;
		m_volmd.bNot8Bit = 0;
		m_volmd.bComplexityEstimationDisable = 1;
		m_volmd.bDataPartitioning = 0;
		m_volmd.bReversibleVlc = false;
		m_volmd.bDeblockFilterDisable = true;
		return;
	}

	m_volmd.bShapeOnly=false;
	if(uiAUsage==3)
		uiAUsage=2;
	m_volmd.fAUsage = (AlphaUsage) uiAUsage;
	if (m_volmd.fAUsage == RECTANGLE) {
		unsigned uiMarker = m_pInBitstream -> bitstreamReadBits (1);
		assert(uiMarker==1);
		m_volmd.iVolWidth = m_ivolWidth = m_pInBitstream -> bitstreamReadBits (NUMBITS_VOP_WIDTH);
		uiMarker  = m_pInBitstream -> bitstreamReadBits (1);
		assert(uiMarker==1);
		m_volmd.iVolHeight = m_ivolHeight = m_pInBitstream -> bitstreamReadBits (NUMBITS_VOP_HEIGHT);
		uiMarker  = m_pInBitstream -> bitstreamReadBits (1);
		assert(uiMarker==1);
	}

	m_vopmd.binterlace = (m_pInBitstream -> bitstreamReadBits (1)) ? true : false; // interlace (was vop flag)
	m_volmd.bAdvPredDisable = (m_pInBitstream -> bitstreamReadBits (1)) ? true : false;  //VOL_obmc_Disable

	// decode sprite info
// GMC
	if(m_volmd.uiVerID == 1)
// ~GMC
		m_uiSprite = m_pInBitstream -> bitstreamReadBits (1);
// GMC
	else if(m_volmd.uiVerID == 2)
		m_uiSprite = m_pInBitstream -> bitstreamReadBits (2);
// ~GMC
	if (m_uiSprite == 1) { // sprite information
		/*int isprite_hdim =*/ m_pInBitstream -> bitstreamReadBits (NUMBITS_SPRITE_HDIM); // by D.S.CHO NEW
		int iMarker = m_pInBitstream -> bitstreamReadBits (MARKER_BIT);
		assert (iMarker == 1);
		/*int isprite_vdim =*/ m_pInBitstream -> bitstreamReadBits (NUMBITS_SPRITE_VDIM); // by D.S.CHO NEW
		iMarker = m_pInBitstream -> bitstreamReadBits (MARKER_BIT);
		assert (iMarker == 1);
		int isprite_left_edge = (m_pInBitstream -> bitstreamReadBits (1) == 0) ?
		    m_pInBitstream->bitstreamReadBits (NUMBITS_SPRITE_LEFT_EDGE - 1) : ((int)m_pInBitstream->bitstreamReadBits (NUMBITS_SPRITE_LEFT_EDGE - 1) - (1 << 12));
		assert(isprite_left_edge%2 == 0);
		iMarker = m_pInBitstream -> bitstreamReadBits (MARKER_BIT);
		assert (iMarker == 1);
		int isprite_top_edge = (m_pInBitstream -> bitstreamReadBits (1) == 0) ?
		    m_pInBitstream->bitstreamReadBits (NUMBITS_SPRITE_TOP_EDGE - 1) : ((int)m_pInBitstream->bitstreamReadBits (NUMBITS_SPRITE_LEFT_EDGE - 1) - (1 << 12));
		assert(isprite_top_edge%2 == 0);
		iMarker = m_pInBitstream -> bitstreamReadBits (MARKER_BIT);
		assert (iMarker == 1);
/*		
		m_rctSpt.left = isprite_left_edge;
		m_rctSpt.right = isprite_left_edge + isprite_hdim;
		m_rctSpt.top = isprite_top_edge;
		m_rctSpt.bottom = isprite_top_edge + isprite_vdim;
		m_rctSpt.width = isprite_hdim;
        	m_rctSptPieceY = m_rctSpt; //initialization; will be overwritten by first vop header
*/
// GMC
	}
	if (m_uiSprite == 1 || m_uiSprite == 2) { // sprite information
		m_iNumOfPnts = m_pInBitstream -> bitstreamReadBits (NUMBITS_NUM_SPRITE_POINTS);
		if(m_uiSprite == 2)
			assert (m_iNumOfPnts == 0 ||
				m_iNumOfPnts == 1 ||
				m_iNumOfPnts == 2 ||
				m_iNumOfPnts == 3);
		m_uiWarpingAccuracy = m_pInBitstream -> bitstreamReadBits (NUMBITS_WARPING_ACCURACY);
		/*unsigned uiLightChange =*/ m_pInBitstream -> bitstreamReadBits (1); // by D.S.CHO NEW
	}
	if (m_uiSprite == 1) { // sprite information
		unsigned uisptMode = m_pInBitstream -> bitstreamReadBits (1);  // Low_latency_sprite_enable
		if (uisptMode)
			m_sptMode = LOW_LATENCY ;
		else			
			m_sptMode = BASIC_SPRITE ;
	}

// HHI Suehring 991022
// HHI Schueuer: sadct
    if (m_volmd.fAUsage != RECTANGLE ) {
		if (m_volmd.uiVerID == 1)
			m_volmd.bSadctDisable = true;
		else
			m_volmd.bSadctDisable = (m_pInBitstream -> bitstreamReadBits(1)) ? true : false;
	}
// end HHI
// end HHI Suehring 991022

	m_volmd.bNot8Bit = (m_pInBitstream -> bitstreamReadBits(1)) ? true : false;
	if (m_volmd.bNot8Bit) {
		m_volmd.uiQuantPrecision = (unsigned) m_pInBitstream -> bitstreamReadBits (4);
		m_volmd.nBits = (unsigned) m_pInBitstream -> bitstreamReadBits (4);
		assert(m_volmd.nBits>3);
	} else {
		m_volmd.uiQuantPrecision = 5;
		m_volmd.nBits = 8;
	}

	if (m_volmd.fAUsage == EIGHT_BIT)
	{
		m_volmd.bNoGrayQuantUpdate = (m_pInBitstream -> bitstreamReadBits (1)) ? true : false;
		/*unsigned uiCompMethod =*/ m_pInBitstream -> bitstreamReadBits (1); // by D.S.CHO NEW
		/*unsigned uiLinearComp =*/ m_pInBitstream -> bitstreamReadBits (1); // by D.S.CHO NEW
	}

	m_volmd.fQuantizer = (Quantizer) m_pInBitstream -> bitstreamReadBits (1); 
	if (m_volmd.fQuantizer == Q_MPEG) {
		m_volmd.bLoadintraMatrix = (m_pInBitstream -> bitstreamReadBits (1)) ? true : false;
		if (m_volmd.bLoadintraMatrix) {
			unsigned i = 0;
			int iElem;
			do {
				iElem = m_pInBitstream -> bitstreamReadBits (NUMBITS_QMATRIX);
				m_volmd.rgiintraQuantizerMatrix [grgiStandardZigzag[i]] = iElem;	
			} while (iElem != 0 && ++i < BLOCK_SQUARE_SIZE);
			for (unsigned j = i; j < BLOCK_SQUARE_SIZE; j++) {
				m_volmd.rgiintraQuantizerMatrix [grgiStandardZigzag[j]] = m_volmd.rgiintraQuantizerMatrix [grgiStandardZigzag[i - 1]]; // by D.S.CHO SS
			}
		}
		else {
			memcpy (m_volmd.rgiintraQuantizerMatrix, rgiDefaultIntraQMatrix, BLOCK_SQUARE_SIZE * sizeof (int));	
		}
		m_volmd.bLoadinterMatrix = (m_pInBitstream -> bitstreamReadBits (1)) ? true : false;
		if (m_volmd.bLoadinterMatrix) {
			unsigned i = 0;
			int iElem;
			do {
				iElem = m_pInBitstream -> bitstreamReadBits (NUMBITS_QMATRIX);
				m_volmd.rgiinterQuantizerMatrix [grgiStandardZigzag[i]] = iElem;	// by D.S.CHO SS
			} while (iElem != 0 && ++i < BLOCK_SQUARE_SIZE);
			for (unsigned j = i; j < BLOCK_SQUARE_SIZE; j++) {
				m_volmd.rgiinterQuantizerMatrix [grgiStandardZigzag[j]] = m_volmd.rgiinterQuantizerMatrix [grgiStandardZigzag[i - 1]];	// by D.S.CHO SS
			}
		}
		else {
			memcpy (m_volmd.rgiinterQuantizerMatrix, rgiDefaultInterQMatrix, BLOCK_SQUARE_SIZE * sizeof (int));
		}
		if (m_volmd.fAUsage == EIGHT_BIT) {      
      		for(int iAuxComp=0; iAuxComp<m_volmd.iAuxCompCount; iAuxComp++ ) { // MAC (SB) 2-Dec-99
				m_volmd.bLoadintraMatrixAlpha = (m_pInBitstream -> bitstreamReadBits (1)) ? true : false;
        		if (m_volmd.bLoadintraMatrixAlpha) {
          			for (unsigned i = 0; i < BLOCK_SQUARE_SIZE; i++)	{
            			int iElem = m_pInBitstream -> bitstreamReadBits (NUMBITS_QMATRIX);
            			if (iElem != 0)
              				m_volmd.rgiintraQuantizerMatrixAlpha[iAuxComp] [grgiStandardZigzag[i]] = iElem;
            			else
              				m_volmd.rgiintraQuantizerMatrixAlpha[iAuxComp] [i] = m_volmd.rgiintraQuantizerMatrixAlpha[iAuxComp] [grgiStandardZigzag[i - 1]];
          			}
        		}
        		else {
#ifdef _FOR_GSSP_
          // memcpy (m_volmd.rgiintraQuantizerMatrixAlpha, rgiDefaultintraQMatrix, BLOCK_SQUARE_SIZE * sizeof (int));
          // use rgiDefaultintraQMatrixAlpha instead of rgiDefaultintraQMatrix (both defined in ./sys/global.hpp), mwi
          			memcpy (m_volmd.rgiintraQuantizerMatrixAlpha[iAuxComp], rgiDefaultintraQMatrixAlpha, BLOCK_SQUARE_SIZE * sizeof (int));
#else
          			for (unsigned i = 0; i < BLOCK_SQUARE_SIZE; i++)
            			m_volmd.rgiintraQuantizerMatrixAlpha[iAuxComp] [i] = 16;
#endif
        		}
				m_volmd.bLoadinterMatrixAlpha = (m_pInBitstream -> bitstreamReadBits (1)) ? true : false;
        		if (m_volmd.bLoadinterMatrixAlpha) {
          			for (unsigned i = 0; i < BLOCK_SQUARE_SIZE; i++) {
            			int iElem = m_pInBitstream -> bitstreamReadBits (NUMBITS_QMATRIX);
            			if (iElem != 0)
              				m_volmd.rgiinterQuantizerMatrixAlpha[iAuxComp] [grgiStandardZigzag[i]] = iElem;
            			else
              				m_volmd.rgiinterQuantizerMatrixAlpha[iAuxComp] [i] = m_volmd.rgiinterQuantizerMatrixAlpha[iAuxComp] [grgiStandardZigzag[i - 1]];
          			}
        		}
        		else {
#ifdef _FOR_GSSP_
          // memcpy (m_volmd.rgiinterQuantizerMatrixAlpha, rgiDefaultinterQMatrix, BLOCK_SQUARE_SIZE * sizeof (int));
          // use rgiDefaultinterQMatrixAlpha instead of rgiDefaultinterQMatrix (both defined in ./sys/global.hpp), mwi
          			memcpy (m_volmd.rgiinterQuantizerMatrixAlpha[iAuxComp], rgiDefaultinterQMatrixAlpha, BLOCK_SQUARE_SIZE * sizeof (int));
#else
          			for (unsigned i = 0; i < BLOCK_SQUARE_SIZE; i++)
            			m_volmd.rgiinterQuantizerMatrixAlpha[iAuxComp] [i] = 16;
#endif
        		}
      		}
		}
	}

    if(m_volmd.uiVerID == 1)
		m_volmd.bQuarterSample = 0; //Quarter sample	
    else if(m_volmd.uiVerID == 2)
		m_volmd.bQuarterSample = (m_pInBitstream -> bitstreamReadBits (1)) ? true : false; //Quarter sample	

// START: Complexity Estimation syntax support - Marc Mongenet (EPFL) - 15 Jun 1998
	m_volmd.bComplexityEstimationDisable = (m_pInBitstream -> bitstreamReadBits (1)) ? true : false;
	if (!m_volmd.bComplexityEstimationDisable) {
		m_volmd.iEstimationMethod = m_pInBitstream -> bitstreamReadBits (2);
		// START: Complexity Estimation syntax support - Update version 2 - Massimo Ravasi (EPFL) - 5 Nov 1999
		if ( (m_volmd.iEstimationMethod != 0) && (m_volmd.iEstimationMethod != 1)) {
		//// Replaced line: if (m_volmd.iEstimationMethod != 0) {
		// END: Complexity Estimation syntax support - Update version 2
			fprintf (stderr, "ERROR: Unknown complexity estimation method number %d.\n", m_volmd.iEstimationMethod);
			exit (1);
		}
		m_volmd.bShapeComplexityEstimationDisable = (m_pInBitstream -> bitstreamReadBits (1)) ? true : false;
		if (! m_volmd.bShapeComplexityEstimationDisable) {
			m_volmd.bOpaque = (m_pInBitstream -> bitstreamReadBits (1)) ? true : false;
			m_volmd.bTransparent = (m_pInBitstream -> bitstreamReadBits (1)) ? true : false;
			m_volmd.bintraCAE = (m_pInBitstream -> bitstreamReadBits (1)) ? true : false;
			m_volmd.binterCAE = (m_pInBitstream -> bitstreamReadBits (1)) ? true : false;
			m_volmd.bNoUpdate = (m_pInBitstream -> bitstreamReadBits (1)) ? true : false;
			m_volmd.bUpsampling = (m_pInBitstream -> bitstreamReadBits (1)) ? true : false;
			if (!(m_volmd.bOpaque ||
				  m_volmd.bTransparent ||
				  m_volmd.bintraCAE ||
				  m_volmd.binterCAE ||
				  m_volmd.bNoUpdate ||
				  m_volmd.bUpsampling)) {
// by D.S.CHO SS
				//fatal_error("Shape complexity estimation is enabled,\nbut no corresponding flag is enabled.");
				fprintf (stderr,"Shape complexity estimation is enabled,\nbut no corresponding flag is enabled.");
				exit (0);
//~by D.S.CHO SS
			}
		}
		else
			m_volmd.bOpaque =
			m_volmd.bTransparent =
			m_volmd.bintraCAE =
			m_volmd.binterCAE =
			m_volmd.bNoUpdate =
			m_volmd.bUpsampling = false;

		m_volmd.bTextureComplexityEstimationSet1Disable = (m_pInBitstream -> bitstreamReadBits (1)) ? true : false;
// GMC
		if(m_uiSprite == 2) assert(m_volmd.bTextureComplexityEstimationSet1Disable == true);
// ~GMC
		if (! m_volmd.bTextureComplexityEstimationSet1Disable) {
			m_volmd.bintraBlocks = (m_pInBitstream -> bitstreamReadBits (1)) ? true : false;
			m_volmd.binterBlocks = (m_pInBitstream -> bitstreamReadBits (1)) ? true : false;
			m_volmd.binter4vBlocks = (m_pInBitstream -> bitstreamReadBits (1)) ? true : false;
			m_volmd.bNotCodedBlocks = (m_pInBitstream -> bitstreamReadBits (1)) ? true : false;
			if (!(m_volmd.bintraBlocks ||
				  m_volmd.binterBlocks ||
				  m_volmd.binter4vBlocks ||
				  m_volmd.bNotCodedBlocks)) {
// by D.S.CHO SS
				//fatal_error("Texture complexity estimation set 1 is enabled,\nbut no corresponding flag is enabled.");
				fprintf(stderr,"Texture complexity estimation set 1 is enabled,\nbut no corresponding flag is enabled.");
				exit (0);
//~by D.S.CHO SS
			}
		}
		else
			m_volmd.bintraBlocks =
			m_volmd.binterBlocks =
			m_volmd.binter4vBlocks =
			m_volmd.bNotCodedBlocks = false;
		
		uiMark = m_pInBitstream -> bitstreamReadBits (1);
		assert (uiMark == 1);

		m_volmd.bTextureComplexityEstimationSet2Disable =( m_pInBitstream -> bitstreamReadBits (1)) ? true : false;
		if (! m_volmd.bTextureComplexityEstimationSet2Disable) {
			m_volmd.bDCTCoefs = (m_pInBitstream -> bitstreamReadBits (1)) ? true : false;
			m_volmd.bDCTLines = (m_pInBitstream -> bitstreamReadBits (1)) ? true : false;
			m_volmd.bVLCSymbols = (m_pInBitstream -> bitstreamReadBits (1)) ? true : false;
			m_volmd.bVLCBits = (m_pInBitstream -> bitstreamReadBits (1)) ? true : false;
			if (!(m_volmd.bDCTCoefs ||
				  m_volmd.bDCTLines ||
				  m_volmd.bVLCSymbols ||
				  m_volmd.bVLCBits)) {
// by D.S.CHO SS
				//fatal_error("Texture complexity estimation set 2 is enabled,\nbut no corresponding flag is enabled.");
				fprintf(stderr,"Texture complexity estimation set 2 is enabled,\nbut no corresponding flag is enabled.");
				exit (0);
//~by D.S.CHO SS
			}
		}
		else
			m_volmd.bDCTCoefs =
			m_volmd.bDCTLines =
			m_volmd.bVLCSymbols =
			m_volmd.bVLCBits = false;

		m_volmd.bMotionCompensationComplexityDisable = (m_pInBitstream -> bitstreamReadBits (1)) ? true : false;
		if (! m_volmd.bMotionCompensationComplexityDisable) {
			m_volmd.bAPM = (m_pInBitstream -> bitstreamReadBits (1)) ? true : false;
			m_volmd.bNPM = (m_pInBitstream -> bitstreamReadBits (1)) ? true : false;
			m_volmd.binterpolateMCQ = (m_pInBitstream -> bitstreamReadBits (1)) ? true : false;
			m_volmd.bForwBackMCQ = (m_pInBitstream -> bitstreamReadBits (1)) ? true : false;
			m_volmd.bHalfpel2 = (m_pInBitstream -> bitstreamReadBits (1)) ? true : false;
			m_volmd.bHalfpel4 = (m_pInBitstream -> bitstreamReadBits (1)) ? true : false;
			if (!(m_volmd.bAPM ||
				  m_volmd.bNPM ||
				  m_volmd.binterpolateMCQ ||
				  m_volmd.bForwBackMCQ ||
				  m_volmd.bHalfpel2 ||
				  m_volmd.bHalfpel4)) {
// by D.S.CHO SS
				//fatal_error("Motion complexity estimation is enabled,\nbut no corresponding flag is enabled.");
				fprintf(stderr,"Motion complexity estimation is enabled,\nbut no corresponding flag is enabled.");
				exit (0);
//~by D.S.CHO SS
			}
		}
		else
			m_volmd.bAPM =
			m_volmd.bNPM =
			m_volmd.binterpolateMCQ =
			m_volmd.bForwBackMCQ =
			m_volmd.bHalfpel2 =
			m_volmd.bHalfpel4 = false;
		
		uiMark = m_pInBitstream -> bitstreamReadBits (1);
		assert (uiMark == 1);

		// START: Complexity Estimation syntax support - Update version 2 - Massimo Ravasi (EPFL) - 5 Nov 1999
		if (m_volmd.iEstimationMethod == 1) {
			m_volmd.bVersion2ComplexityEstimationDisable = (m_pInBitstream -> bitstreamReadBits (1)) ? true : false;
			if (! m_volmd.bVersion2ComplexityEstimationDisable) {
				m_volmd.bSadct = (m_pInBitstream -> bitstreamReadBits (1)) ? true : false;
				m_volmd.bQuarterpel = (m_pInBitstream -> bitstreamReadBits (1)) ? true : false;
				if (!(m_volmd.bSadct ||
					  m_volmd.bQuarterpel)) {
// by D.S.CHO SS
					//fatal_error("Version 2 complexity estimation is enabled,\nbut no corresponding flag is enabled.");
					fprintf(stderr,"Version 2 complexity estimation is enabled,\nbut no corresponding flag is enabled.");
					exit (0);
//~by D.S.CHO SS
				}
			}
			else
				m_volmd.bSadct =
				m_volmd.bQuarterpel = false;
		
		} else {
			m_volmd.bVersion2ComplexityEstimationDisable = true;
			m_volmd.bSadct =
			m_volmd.bQuarterpel = false;
		}
		// END: Complexity Estimation syntax support - Update version 2
		
		// START: Complexity Estimation syntax support - Update version 2 - Massimo Ravasi (EPFL) - 11 Nov 1999
		// Main complexity estimation flag test
		if (m_volmd.bShapeComplexityEstimationDisable &&
		    m_volmd.bTextureComplexityEstimationSet1Disable &&
		    m_volmd.bTextureComplexityEstimationSet2Disable &&
		    m_volmd.bMotionCompensationComplexityDisable &&
			m_volmd.bVersion2ComplexityEstimationDisable) {
// by D.S.CHO SS
			//fatal_error("Complexity estimation is enabled,\nbut no correponding flag is enabled.");
			fprintf(stderr,"Complexity estimation is enabled,\nbut no correponding flag is enabled.");
			exit (0);
//~by D.S.CHO SS
		}
		// END: Complexity Estimation syntax support - Update version 2
	}
	// END: Complexity Estimation syntax support

	/*unsigned uiResyncMarkerDisable = */m_pInBitstream -> bitstreamReadBits (1);

	//	Modified by Toshiba(1998-4-7)
	m_volmd.bDataPartitioning = (m_pInBitstream -> bitstreamReadBits (1)) ? true : false;
	if( m_volmd.bDataPartitioning )
		m_volmd.bReversibleVlc = (m_pInBitstream -> bitstreamReadBits (1)) ? true : false;
	else
		m_volmd.bReversibleVlc = false;
	//	End Toshiba

// NEWPRED
    if(m_volmd.uiVerID == 1)
		m_volmd.bNewpredEnable = false;
    else if(m_volmd.uiVerID != 1)
		m_volmd.bNewpredEnable = (m_pInBitstream -> bitstreamReadBits (1)) ? true : false;

	if (m_volmd.bNewpredEnable) {
		assert (m_volmd.fAUsage == RECTANGLE);
		assert (m_vopmd.binterlace == 0);
		assert (m_uiSprite == 0);
		m_volmd.iRequestedBackwardMessegeType = m_pInBitstream -> bitstreamReadBits (2);
		m_volmd.bNewpredSegmentType = (m_pInBitstream -> bitstreamReadBits (1)) ? true : false;

		// generate NEWPRED object
//		g_pNewPredDec = new CNewPredDecoder();	// by D.S.CHO SS
	}
// ~NEWPRED
// RRV insertion
	if(m_volmd.uiVerID != 1)
	{
		m_volmd.breduced_resolution_vop_enable = (m_pInBitstream -> bitstreamReadBits (1)) ? true : false;
	}
	else
	{
		m_volmd.breduced_resolution_vop_enable = 0;
	}
//	m_vopmd.RRVmode.iOnOff	= m_volmd.breduced_resolution_vop_enable;	// by D.S.CHO SS
// ~RRV

	//wchen: wd changes
	m_volmd.volType = (m_pInBitstream -> bitstreamReadBits (1) == 0) ? BASE_LAYER : ENHN_LAYER;

// NEWPRED
	if (m_volmd.bNewpredEnable)
		assert (m_volmd.volType == BASE_LAYER);
// ~NEWPRED

// GMC
	if(m_uiSprite == 2) assert(m_volmd.volType == BASE_LAYER);
// ~GMC
	m_volmd.ihor_sampling_factor_n = 1;
	m_volmd.ihor_sampling_factor_m = 1;
	m_volmd.iver_sampling_factor_n = 1;
	m_volmd.iver_sampling_factor_m = 1;	
//OBSS_SAIT_991015
	m_volmd.ihor_sampling_factor_n_shape = 1;
	m_volmd.ihor_sampling_factor_m_shape = 1;
	m_volmd.iver_sampling_factor_n_shape = 1;
	m_volmd.iver_sampling_factor_m_shape = 1;	
//~OBSS_SAIT_991015
	if (m_volmd.volType == ENHN_LAYER)	{
//#ifdef _Scalable_SONY_
		m_volmd.iHierarchyType = m_pInBitstream->bitstreamReadBits (1); // 
#ifdef	_DSCHO_DBG_	// by D.S.CHO SS
		if(m_volmd.iHierarchyType == 0)
			fprintf(stdout,"Hierarchy_Type == 0 (Spatial scalability)\n");
		else if(m_volmd.iHierarchyType == 1)
			fprintf(stdout,"Hierarchy_type == 1 (Temporal scalability)\n");
#endif	// _by D.S.CHO SS
//#endif _Scalable_SONY_
		m_pInBitstream->bitstreamReadBits (4);						// ref_layer_id
		m_pInBitstream->bitstreamReadBits (1);						// ref_layer_samping_director
		m_volmd.ihor_sampling_factor_n = m_pInBitstream->bitstreamReadBits (5);
		m_volmd.ihor_sampling_factor_m = m_pInBitstream->bitstreamReadBits (5);
		m_volmd.iver_sampling_factor_n = m_pInBitstream->bitstreamReadBits (5);
		m_volmd.iver_sampling_factor_m = m_pInBitstream->bitstreamReadBits (5);
		m_volmd.iEnhnType     = m_pInBitstream->bitstreamReadBits (1);			//enhancement_type
//OBSS_SAIT_991015
		if (m_volmd.fAUsage == ONE_BIT && m_volmd.iHierarchyType == 0) {			
			m_volmd.iuseRefShape = m_pInBitstream->bitstreamReadBits (1);						// use_ref_shape
			m_volmd.iuseRefTexture = m_pInBitstream->bitstreamReadBits (1);						// use_ref_texture
			m_volmd.ihor_sampling_factor_n_shape = m_pInBitstream->bitstreamReadBits (5);
			m_volmd.ihor_sampling_factor_m_shape = m_pInBitstream->bitstreamReadBits (5);
			m_volmd.iver_sampling_factor_n_shape = m_pInBitstream->bitstreamReadBits (5);
			m_volmd.iver_sampling_factor_m_shape = m_pInBitstream->bitstreamReadBits (5);
		}
//~OBSS_SAIT_991015		
	}

	m_volmd.bDeblockFilterDisable = true;							//no deblocking filter
}

bool CMp4VideoAnalyzer::decodeVOPHead ()
{
	// Start code
	unsigned uiVopStartCode = m_pInBitstream -> bitstreamReadBits (NUMBITS_VOP_START_CODE);
	if (uiVopStartCode == GOV_START_CODE) {
/*Added by SONY (98/03/30)*/
		m_bUseGOV = true;
		m_bLinkisBroken = false;
/*Added by SONY (98/03/30) End */
		int timecode;
		timecode = m_pInBitstream -> bitstreamReadBits (5) * 3600;
		timecode += m_pInBitstream -> bitstreamReadBits (6) * 60;
		m_pInBitstream -> bitstreamReadBits (1);
		timecode += m_pInBitstream -> bitstreamReadBits (6);

		m_tModuloBaseDecd = timecode;
		m_tModuloBaseDisp = timecode;

#ifdef _DSCHO_DBG_
		//cout << "GOV Header (t=" << timecode << ")\n\n";
		fprintf (stderr,"GOV Header (t= %d)\n\n", timecode);
#endif
		int closed_gov = m_pInBitstream -> bitstreamReadBits (1);
		int broken_link = m_pInBitstream -> bitstreamReadBits (1);
/*modified by SONY (98/03/30)*/
		if ((closed_gov == 0)&&(broken_link == 1))
			m_bLinkisBroken = true;
/*modified by SONY (98/03/30) End*/
		
		findStartCode();
		uiVopStartCode = m_pInBitstream -> bitstreamReadBits (NUMBITS_VOP_START_CODE);
	}
//980212
	assert(uiVopStartCode == VOP_START_CODE);
	////////////////////////////////////////////////////////////////////////////////////////////////
	m_vopmd.vopPredType = (VOPpredType) m_pInBitstream -> bitstreamReadBits (NUMBITS_VOP_PRED_TYPE);
	////////////////////////////////////////////////////////////////////////////////////////////////
// NEWPRED
	if (m_volmd.bNewpredEnable)
		assert (m_vopmd.vopPredType != BVOP);
// ~NEWPRED

	// Time reference and VOP_pred_type
	int iModuloInc = 0;
	while (m_pInBitstream -> bitstreamReadBits (1) != 0)
		iModuloInc++;
	Time tCurrSec = iModuloInc + ((m_vopmd.vopPredType != BVOP ||
				  (m_vopmd.vopPredType == BVOP && m_volmd.volType == ENHN_LAYER ))?
				m_tModuloBaseDecd : m_tModuloBaseDisp);
	//	Added for error resilient mode by Toshiba(1997-11-14)
	unsigned uiMarker = m_pInBitstream -> bitstreamReadBits (1);
	assert(uiMarker == 1);
	//	End Toshiba(1997-11-14)
	Time tVopIncr = 0;
	if(m_iNumBitsTimeIncr!=0)
		tVopIncr = m_pInBitstream -> bitstreamReadBits (m_iNumBitsTimeIncr);
	uiMarker = m_pInBitstream->bitstreamReadBits (1); // marker bit
	assert(uiMarker == 1);
	m_tOldModuloBaseDecd = m_tModuloBaseDecd;
	m_tOldModuloBaseDisp = m_tModuloBaseDisp;
	if (m_vopmd.vopPredType != BVOP ||
		(m_vopmd.vopPredType == BVOP && m_volmd.volType == ENHN_LAYER ))
	{
		m_tModuloBaseDisp = m_tModuloBaseDecd;		//update most recently displayed time base
		m_tModuloBaseDecd = tCurrSec;
	}
	///////////////////////////////////////////////////////////////////////////////////
    m_t = tCurrSec*m_volmd.iClockRate*m_iClockRateScale + tVopIncr*m_iClockRateScale;
	///////////////////////////////////////////////////////////////////////////////////
	if (m_pInBitstream->bitstreamReadBits (1) == 0)		{		//vop_coded == false
		m_vopmd.binterlace = false;	//wchen: temporary solution
		return false;
	}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// by D.S.CHO SS 
#ifdef	_DSCHO_DBG_
//~by D.S.CHO SS

// NEWPRED
	if (m_volmd.bNewpredEnable) {
		m_vopmd.m_iVopID = m_pInBitstream -> bitstreamReadBits(m_vopmd.m_iNumBitsVopID);
		m_vopmd.m_iVopID4Prediction_Indication = m_pInBitstream -> bitstreamReadBits(NUMBITS_VOP_ID_FOR_PREDICTION_INDICATION);
		if( m_vopmd.m_iVopID4Prediction_Indication )
			m_vopmd.m_iVopID4Prediction = m_pInBitstream -> bitstreamReadBits(m_vopmd.m_iNumBitsVopID);
		m_pInBitstream -> bitstreamReadBits(MARKER_BIT);	
/*
		g_pNewPredDec->GetRef(
				NP_VOP_HEADER,
				m_vopmd.vopPredType,
				m_vopmd.m_iVopID,	
				m_vopmd.m_iVopID4Prediction_Indication,
				m_vopmd.m_iVopID4Prediction
		);
*/
	}
// ~NEWPRED

	if ((m_vopmd.vopPredType == PVOP  || (m_uiSprite == 2 && m_vopmd.vopPredType == SPRITE)) && m_volmd.bShapeOnly==false) // GMC
		m_vopmd.iRoundingControl = m_pInBitstream->bitstreamReadBits (1); //"VOP_Rounding_Type"
	else
		m_vopmd.iRoundingControl = 0;

// RRV insertion
	if((m_volmd.breduced_resolution_vop_enable == 1)&&(m_volmd.fAUsage == RECTANGLE)&&
	   ((m_vopmd.vopPredType == PVOP)||(m_vopmd.vopPredType == IVOP)))
	{
	  	int iRRVOnOff = m_pInBitstream->bitstreamReadBits (1);
	}	
	else
	{
		int iRRVOnOff = 0;
	}
// ~RRV
	if (m_volmd.fAUsage != RECTANGLE) {
// Begin: modified by Hughes	  4/9/98
		if (!(m_uiSprite == 1 && m_vopmd.vopPredType == IVOP)) {
// End: modified by Hughes	  4/9/98
			int width = m_pInBitstream -> bitstreamReadBits (NUMBITS_VOP_WIDTH);
			int marker;
			marker = m_pInBitstream -> bitstreamReadBits (1); // marker bit
			assert(marker==1);
			int height = m_pInBitstream -> bitstreamReadBits (NUMBITS_VOP_HEIGHT);
			marker = m_pInBitstream -> bitstreamReadBits (1); // marker bit
			assert(marker==1);
			//wchen: cd changed to 2s complement
			int left = (m_pInBitstream -> bitstreamReadBits (1) == 0) ?
					m_pInBitstream->bitstreamReadBits (NUMBITS_VOP_HORIZONTAL_SPA_REF - 1) : 
					((int)m_pInBitstream->bitstreamReadBits (NUMBITS_VOP_HORIZONTAL_SPA_REF - 1) - (1 << (NUMBITS_VOP_HORIZONTAL_SPA_REF - 1)));
			marker = m_pInBitstream -> bitstreamReadBits (1); // marker bit
			assert(marker==1);
			int top = (m_pInBitstream -> bitstreamReadBits (1) == 0) ?
				   m_pInBitstream->bitstreamReadBits (NUMBITS_VOP_VERTICAL_SPA_REF - 1) : 
				   ((int)m_pInBitstream->bitstreamReadBits (NUMBITS_VOP_VERTICAL_SPA_REF - 1) - (1 << (NUMBITS_VOP_VERTICAL_SPA_REF - 1)));
			assert(((left | top)&1)==0); // must be even pix unit
			marker = m_pInBitstream -> bitstreamReadBits (1); // marker bit added Nov 10, swinder
			assert(marker==1);
/*
			m_rctCurrVOPY = CRct (left, top, left + width, top + height);
			m_rctCurrVOPUV = m_rctCurrVOPY.downSampleBy2 ();
*/
		}

		int BGComposition; // by D.S.CHO SS
		if ( m_volmd.volType == ENHN_LAYER && m_volmd.iEnhnType != 0 ){ // change added for Norio Ito
			BGComposition = m_pInBitstream -> bitstreamReadBits (1); // modified by Sharp (98/3/24)
		}

		m_volmd.bNoCrChange = (m_pInBitstream -> bitstreamReadBits (1)) ? true : false;	//VOP_CR_Change_Disable
		int iVopConstantAlpha = m_pInBitstream -> bitstreamReadBits (1);
		if(iVopConstantAlpha==1)
			m_vopmd.iVopConstantAlphaValue = m_pInBitstream -> bitstreamReadBits (8);
		else
			m_vopmd.iVopConstantAlphaValue = 255;
		m_vopmd.bShapeCodingType = (m_vopmd.vopPredType == IVOP) ? 0 : 1; //	Added error resilient mode by Toshiba(1998-1-16)
	}

	// START: Complexity Estimation syntax support - Marc Mongenet (EPFL) - 15 Jun 1998

	if (!m_volmd.bComplexityEstimationDisable) {

		// START: Complexity Estimation syntax support - Update version 2 - Massimo Ravasi (EPFL) - 5 Nov 1999
		if (  (m_volmd.iEstimationMethod != 0) && (m_volmd.iEstimationMethod != 1) ) {
		//// Replaced line: if (m_volmd.iEstimationMethod != 0) {
		// END: Complexity Estimation syntax support - Update version 2
			fprintf (stderr, "ERROR: Unknown estimation method number %d.\n", m_volmd.iEstimationMethod);
			exit (1);
		}
		
		if (m_vopmd.vopPredType == IVOP ||
			m_vopmd.vopPredType == PVOP ||
			m_vopmd.vopPredType == BVOP) {

			if (m_volmd.bOpaque) {
				printf ("dcecs_opaque = %d\n", m_vopmd.iOpaque = m_pInBitstream -> bitstreamReadBits (8));
				if (m_vopmd.iOpaque == 0) {
					fprintf (stderr, "ERROR: Illegal null value for 'opaque' complexity estimation.\n");
					exit (1);
				}
			}
			if (m_volmd.bTransparent) {
				printf ("dcecs_transparent = %d\n", m_vopmd.iTransparent = m_pInBitstream -> bitstreamReadBits (8));
				if (m_vopmd.iTransparent == 0) {
					fprintf (stderr, "ERROR: Illegal null value for 'transparent' complexity estimation.\n");
					exit (1);
				}
			}
			if (m_volmd.bintraCAE) {
				printf ("dcecs_intra_cae = %d\n", m_vopmd.iintraCAE = m_pInBitstream -> bitstreamReadBits (8));
				if (m_vopmd.iintraCAE == 0) {
					fprintf (stderr, "ERROR: Illegal null value for 'intra_cae' complexity estimation.\n");
					exit (1);
				}
			}
			if (m_volmd.binterCAE) {
				printf ("dcecs_inter_cae = %d\n", m_vopmd.iinterCAE = m_pInBitstream -> bitstreamReadBits (8));
				if (m_vopmd.iinterCAE == 0) {
					fprintf (stderr, "ERROR: Illegal null value for 'inter_cae' complexity estimation.\n");
					exit (1);
				}
			}
			if (m_volmd.bNoUpdate) {
				printf ("dcecs_no_update = %d\n", m_vopmd.iNoUpdate = m_pInBitstream -> bitstreamReadBits (8));
				if (m_vopmd.iNoUpdate == 0) {
					fprintf (stderr, "ERROR: Illegal null value for 'no_update' complexity estimation.\n");
					exit (1);
				}
			}
			if (m_volmd.bUpsampling) {
				printf ("dcecs_upsampling = %d\n", m_vopmd.iUpsampling = m_pInBitstream -> bitstreamReadBits (8));
				if (m_vopmd.iUpsampling == 0) {
					fprintf (stderr, "ERROR: Illegal null value for 'upsampling' complexity estimation.\n");
					exit (1);
				}
			}
		}

		if (m_volmd.bintraBlocks) {
			printf ("dcecs_intra_blocks = %d\n", m_vopmd.iintraBlocks = m_pInBitstream -> bitstreamReadBits (8));
			if (m_vopmd.iintraBlocks == 0) {
					fprintf (stderr, "ERROR: Illegal null value for 'intra_blocks' complexity estimation.\n");
					exit (1);
				}
		}
		if (m_volmd.bNotCodedBlocks) {
			printf ("dcecs_not_coded_blocks = %d\n", m_vopmd.iNotCodedBlocks = m_pInBitstream -> bitstreamReadBits (8));
			if (m_vopmd.iNotCodedBlocks == 0) {
					fprintf (stderr, "ERROR: Illegal null value for 'not_coded_blocks' complexity estimation.\n");
					exit (1);
				}
		}
		if (m_volmd.bDCTCoefs) {
			printf ("dcecs_dct_coefs = %d\n", m_vopmd.iDCTCoefs = m_pInBitstream -> bitstreamReadBits (8));
			if (m_vopmd.iDCTCoefs == 0) {
					fprintf (stderr, "ERROR: Illegal null value for 'dct_coefs' complexity estimation.\n");
					exit (1);
				}
		}
		if (m_volmd.bDCTLines) {
			printf ("dcecs_dct_lines = %d\n", m_vopmd.iDCTLines = m_pInBitstream -> bitstreamReadBits (8));
			if (m_vopmd.iDCTLines == 0) {
					fprintf (stderr, "ERROR: Illegal null value for 'dct_lines' complexity estimation.\n");
					exit (1);
				}
		}
		if (m_volmd.bVLCSymbols) {
			printf ("dcecs_vlc_symbols = %d\n", m_vopmd.iVLCSymbols = m_pInBitstream -> bitstreamReadBits (8));
			if (m_vopmd.iVLCSymbols == 0) {
					fprintf (stderr, "ERROR: Illegal null value for 'vlc_symbols' complexity estimation.\n");
					exit (1);
				}
		}
		if (m_volmd.bVLCBits) {
			printf ("dcecs_vlc_bits = %d\n", m_vopmd.iVLCBits = m_pInBitstream -> bitstreamReadBits (4));
			if (m_vopmd.iVLCBits == 0) {
					fprintf (stderr, "ERROR: Illegal null value for 'vlc_bits' complexity estimation.\n");
					exit (1);
				}
		}

		if (m_vopmd.vopPredType == PVOP ||
			m_vopmd.vopPredType == BVOP ||
			// START: Complexity Estimation syntax support - Update version 2 - Massimo Ravasi (EPFL) - 11 Nov 1999
  			((m_vopmd.vopPredType) == SPRITE && (m_uiSprite == 1)) ) {
			// END: Complexity Estimation syntax support - Update version 2
			if (m_volmd.binterBlocks) {
				printf ("dcecs_inter_blocks = %d\n", m_vopmd.iinterBlocks = m_pInBitstream -> bitstreamReadBits (8));
				if (m_vopmd.iinterBlocks == 0) {
					fprintf (stderr, "ERROR: Illegal null value for 'inter_blocks' complexity estimation.\n");
					exit (1);
				}
			}
			if (m_volmd.binter4vBlocks) {
				printf ("dcecs_inter4v_blocks = %d\n", m_vopmd.iinter4vBlocks = m_pInBitstream -> bitstreamReadBits (8));
				if (m_vopmd.iinter4vBlocks == 0) {
					fprintf (stderr, "ERROR: Illegal null value for 'inter4v_blocks' complexity estimation.\n");
					exit (1);
				}
			}
			if (m_volmd.bAPM) {
				printf ("dcecs_apm = %d\n", m_vopmd.iAPM = m_pInBitstream -> bitstreamReadBits (8));
				if (m_vopmd.iAPM == 0) {
					fprintf (stderr, "ERROR: Illegal null value for 'apm' complexity estimation.\n");
					exit (1);
				}
			}
			if (m_volmd.bNPM) {
				printf ("dcecs_npm = %d\n", m_vopmd.iNPM = m_pInBitstream -> bitstreamReadBits (8));
				if (m_vopmd.iNPM == 0) {
					fprintf (stderr, "ERROR: Illegal null value for 'npm' complexity estimation.\n");
					exit (1);
				}
			}
			if (m_volmd.bForwBackMCQ) {
				printf ("dcecs_forw_back_mc_q = %d\n", m_vopmd.iForwBackMCQ = m_pInBitstream -> bitstreamReadBits (8));
				if (m_vopmd.iForwBackMCQ == 0) {
					fprintf (stderr, "ERROR: Illegal null value for 'forw_back_mc_q' complexity estimation.\n");
					exit (1);
				}
			}
			if (m_volmd.bHalfpel2) {
				printf ("dcecs_halfpel2 = %d\n", m_vopmd.iHalfpel2 = m_pInBitstream -> bitstreamReadBits (8));
				if (m_vopmd.iHalfpel2 == 0) {
					fprintf (stderr, "ERROR: Illegal null value for 'halfpel2' complexity estimation.\n");
					exit (1);
				}
			}
			if (m_volmd.bHalfpel4) {
				printf ("dcecs_halfpel4 = %d\n", m_vopmd.iHalfpel4 = m_pInBitstream -> bitstreamReadBits (8));
				if (m_vopmd.iHalfpel4 == 0) {
					fprintf (stderr, "ERROR: Illegal null value for 'halfpel4' complexity estimation.\n");
					exit (1);
				}
			}
		}
		if (m_vopmd.vopPredType == BVOP ||
			// START: Complexity Estimation syntax support - Update version 2 - Massimo Ravasi (EPFL) - 11 Nov 1999
  			((m_vopmd.vopPredType) == SPRITE && (m_uiSprite == 1)) ) {
			// END: Complexity Estimation syntax support - Update version 2
			if (m_volmd.binterpolateMCQ) {
				printf ("dcecs_interpolate_mc_q = %d\n", m_vopmd.iinterpolateMCQ = m_pInBitstream -> bitstreamReadBits (8));
				if (m_vopmd.iinterpolateMCQ == 0) {
					fprintf (stderr, "ERROR: Illegal null value for 'interpolate_mc_q' complexity estimation.\n");
					exit (1);
				}
			}
		}
		// START: Complexity Estimation syntax support - Update version 2 - Massimo Ravasi (EPFL) - 5 Nov 1999
		if (m_vopmd.vopPredType == IVOP ||
			m_vopmd.vopPredType == PVOP ||
			m_vopmd.vopPredType == BVOP) {
			if(m_volmd.bSadct) {
				printf ("dcecs_bSadct = %d\n", m_vopmd.iSadct = m_pInBitstream -> bitstreamReadBits (8));
				if (m_vopmd.iSadct == 0) {
					fprintf (stderr, "ERROR: Illegal null value for 'sadct' complexity estimation.\n");
					exit (1);
				}
			}
		}

		if (m_vopmd.vopPredType == PVOP ||
			m_vopmd.vopPredType == BVOP ||
			// START: Complexity Estimation syntax support - Update version 2 - Massimo Ravasi (EPFL) - 11 Nov 1999
  			((m_vopmd.vopPredType) == SPRITE && (m_uiSprite == 1)) ) {
			// END: Complexity Estimation syntax support - Update version 2
			if(m_volmd.bQuarterpel) {
				printf ("dcecs_bQuarterpel = %d\n", m_vopmd.iQuarterpel = m_pInBitstream -> bitstreamReadBits (8));
				if (m_vopmd.iQuarterpel == 0) {
					fprintf (stderr, "ERROR: Illegal null value for 'quarterpel' complexity estimation.\n");
					exit (1);
				}
			}
		}
		// END: Complexity Estimation syntax support - Update version 2
	}
	// END: Complexity Estimation syntax support

	if(m_volmd.bShapeOnly==true)
	{
		m_vopmd.intStep = 10; // probably not needed 
		m_vopmd.intStepI = 10;
		m_vopmd.intStepB = 10;
/*
		m_vopmd.mvInfoForward.uiFCode = m_vopmd.mvInfoBackward.uiFCode = 1;	//	Modified error resilient mode by Toshiba (1998-1-16)
*/
		m_vopmd.uiVopFcodeForward = m_vopmd.uiVopFcodeBackward = 1;

		m_vopmd.binterlace = false;	//wchen: temporary solution
//OBSS_SAIT_991015
		if(m_volmd.volType == ENHN_LAYER && m_vopmd.vopPredType == PVOP)
			m_vopmd.iRefSelectCode = 3;
		else if(m_volmd.volType == ENHN_LAYER && m_vopmd.vopPredType == BVOP)	
			m_vopmd.iRefSelectCode = 0;
//~OBSS_SAIT_991015
		return true;
	}
	m_vopmd.iintraDcSwitchThr = m_pInBitstream->bitstreamReadBits (3);

// intERLACE
	if (m_vopmd.binterlace) {
		m_vopmd.bTopFieldFirst = (m_pInBitstream -> bitstreamReadBits (1)) ? true : false;
        m_vopmd.bAlternateScan = (m_pInBitstream -> bitstreamReadBits (1)) ? true : false;
		assert(m_volmd.volType == BASE_LAYER);
	} else
// ~intERLACE
    {
        m_vopmd.bAlternateScan = false;
    }

// GMC
	if (m_uiSprite == 2 && m_vopmd.vopPredType == SPRITE) {
		//if(m_iNumOfPnts > 0)
		//	decodeWarpPoints ();
	}
// ~GMC

	if (m_vopmd.vopPredType == IVOP)	{ 
		m_vopmd.intStepI = m_vopmd.intStep = m_pInBitstream -> bitstreamReadBits (m_volmd.uiQuantPrecision);	//also assign intStep to be safe
		m_vopmd.uiVopFcodeForward = m_vopmd.uiVopFcodeBackward = 1;		//	Modified error resilient mode by Toshiba(1998-1-16) // by D.S.CHO SS
		if(m_volmd.fAUsage == EIGHT_BIT)
			for(int iAuxComp=0; iAuxComp<m_volmd.iAuxCompCount; iAuxComp++ ) // MAC (SB) 2-Dec-99
				m_vopmd.intStepIAlpha[iAuxComp] = m_pInBitstream -> bitstreamReadBits (NUMBITS_VOP_ALPHA_QUANTIZER);
	}
	else if (m_vopmd.vopPredType == PVOP || (m_uiSprite == 2 && m_vopmd.vopPredType == SPRITE)) { // GMC
		m_vopmd.intStep = m_pInBitstream -> bitstreamReadBits (m_volmd.uiQuantPrecision);
		if(m_volmd.fAUsage == EIGHT_BIT)
			for(int iAuxComp=0; iAuxComp<m_volmd.iAuxCompCount; iAuxComp++ ) // MAC (SB) 2-Dec-99
				m_vopmd.intStepPAlpha[iAuxComp] = m_pInBitstream -> bitstreamReadBits (NUMBITS_VOP_ALPHA_QUANTIZER);

		m_vopmd.uiVopFcodeForward = m_pInBitstream -> bitstreamReadBits (NUMBITS_VOP_FCODE);
	}
	else if (m_vopmd.vopPredType == BVOP) {
		m_vopmd.intStepB = m_vopmd.intStep = m_pInBitstream -> bitstreamReadBits (m_volmd.uiQuantPrecision); //also assign intStep to be safe

		if(m_volmd.fAUsage == EIGHT_BIT)
			for(int iAuxComp=0; iAuxComp<m_volmd.iAuxCompCount; iAuxComp++ )  // MAC (SB) 2-Dec-99
				m_vopmd.intStepBAlpha[iAuxComp] = m_pInBitstream -> bitstreamReadBits (NUMBITS_VOP_ALPHA_QUANTIZER);
		
		m_vopmd.uiVopFcodeForward = m_pInBitstream -> bitstreamReadBits (NUMBITS_VOP_FCODE);	// by D.S.CHO SS
		m_vopmd.uiVopFcodeBackward = m_pInBitstream -> bitstreamReadBits (NUMBITS_VOP_FCODE);	// by D.S.CHO SS
	}
	//	Added for error resilient mode by Toshiba(1997-11-14)
	if ( m_volmd.volType == BASE_LAYER ) // added by Sharp (98/4/13)
        if (m_volmd.fAUsage != RECTANGLE && m_vopmd.vopPredType != IVOP	&& m_uiSprite != 1)
		{	
			m_vopmd.bShapeCodingType = (m_pInBitstream -> bitstreamReadBits (1)) ? true : false;
        }
	// End Toshiba(1997-11-14)
	if(m_volmd.volType == ENHN_LAYER) {					//sony
		m_vopmd.bShapeCodingType = (m_vopmd.vopPredType == IVOP?0:1); // FIX for C2-1, added 
		if( m_volmd.iEnhnType != 0 ){  // change for Norio Ito
			m_vopmd.iLoadBakShape = m_pInBitstream -> bitstreamReadBits (1); // load_backward_shape
			if(m_vopmd.iLoadBakShape){
/*
				CVOPU8YUVBA* pvopcCurr = new CVOPU8YUVBA (*(rgpbfShape [0]->pvopcReconCurr()));
				copyVOPU8YUVBA(rgpbfShape [1]->m_pvopcRefQ1, pvopcCurr);
					// previous backward shape is saved to current forward shape
				rgpbfShape [1]->m_rctCurrVOPY.left   = rgpbfShape [0]->m_rctCurrVOPY.left;
				rgpbfShape [1]->m_rctCurrVOPY.right  = rgpbfShape [0]->m_rctCurrVOPY.right;
				rgpbfShape [1]->m_rctCurrVOPY.top    = rgpbfShape [0]->m_rctCurrVOPY.top;
				rgpbfShape [1]->m_rctCurrVOPY.bottom = rgpbfShape [0]->m_rctCurrVOPY.bottom;
*/
				int width = m_pInBitstream -> bitstreamReadBits (NUMBITS_VOP_WIDTH); 
				assert (width % MB_SIZE == 0); // has to be multiples of MB_SIZE
				unsigned uiMark = m_pInBitstream -> bitstreamReadBits (1);
				assert (uiMark == 1);
				int height = m_pInBitstream -> bitstreamReadBits (NUMBITS_VOP_HEIGHT); 
				assert (height % MB_SIZE == 0); // has to be multiples of MB_SIZE
				uiMark = m_pInBitstream -> bitstreamReadBits (1);
				assert (uiMark == 1);
				width = ((width+15)>>4)<<4; // not needed if the asserts are present
				height = ((height+15)>>4)<<4;
				int iSign = (m_pInBitstream -> bitstreamReadBits (1) == 1)? -1 : 1;
				int left = iSign * m_pInBitstream -> bitstreamReadBits (NUMBITS_VOP_HORIZONTAL_SPA_REF - 1);
				int marker = m_pInBitstream -> bitstreamReadBits (1); // marker bit
				iSign = (m_pInBitstream -> bitstreamReadBits (1) == 1)? -1 : 1;
				int top = iSign * m_pInBitstream -> bitstreamReadBits (NUMBITS_VOP_VERTICAL_SPA_REF - 1);
/*			
				rgpbfShape[0]->m_rctCurrVOPY = CRct (left, top, left + width, top + height);
				rgpbfShape[0]->m_rctCurrVOPUV = rgpbfShape[0]->m_rctCurrVOPY.downSampleBy2 ();
*/
				// decode backward shape
/*
				rgpbfShape[0]->setRefStartingPointers ();
				rgpbfShape[0]->compute_bfShapeMembers (); // clear m_pvopcRefQ1
				rgpbfShape[0]->resetBYPlane ();// clear BY of RefQ1 (katata)
				rgpbfShape[0]->m_volmd.bShapeOnly = true;
				rgpbfShape[0]->m_volmd.bNoCrChange = m_volmd.bNoCrChange; // set CR change disable(Oct. 9 1997)
				rgpbfShape[0]->m_vopmd.binterlace = false;
				rgpbfShape[0]->decodeIVOP_WithShape ();
*/
				m_vopmd.iLoadForShape = m_pInBitstream -> bitstreamReadBits (1); // load_forward_shape
				if(m_vopmd.iLoadForShape){
					width = m_pInBitstream -> bitstreamReadBits (NUMBITS_VOP_WIDTH); 
					assert (width % MB_SIZE == 0); // has to be multiples of MB_SIZE
					uiMark = m_pInBitstream -> bitstreamReadBits (1);
					assert (uiMark == 1);
					height = m_pInBitstream -> bitstreamReadBits (NUMBITS_VOP_HEIGHT); 
					assert (height % MB_SIZE == 0); // has to be multiples of MB_SIZE
					uiMark = m_pInBitstream -> bitstreamReadBits (1);
					assert (uiMark == 1);
					width = ((width+15)>>4)<<4; // not needed if the asserts are present
					height = ((height+15)>>4)<<4;
					iSign = (m_pInBitstream -> bitstreamReadBits (1) == 1)? -1 : 1;
					left = iSign * m_pInBitstream -> bitstreamReadBits (NUMBITS_VOP_HORIZONTAL_SPA_REF - 1);
					marker = m_pInBitstream -> bitstreamReadBits (1); // marker bit
					iSign = (m_pInBitstream -> bitstreamReadBits (1) == 1)? -1 : 1;
					top = iSign * m_pInBitstream -> bitstreamReadBits (NUMBITS_VOP_VERTICAL_SPA_REF - 1);
/*		
					rgpbfShape[1]->m_rctCurrVOPY = CRct (left, top, left + width, top + height);
					rgpbfShape[1]->m_rctCurrVOPUV = rgpbfShape[1]->m_rctCurrVOPY.downSampleBy2 ();
*/
					// decode forward shape
/*
					rgpbfShape[1]->setRefStartingPointers ();
					rgpbfShape[1]->compute_bfShapeMembers (); // clear m_pvopcRefQ1
					rgpbfShape[1]->resetBYPlane ();// clear BY of RefQ1 (katata)
					rgpbfShape[1]->m_volmd.bShapeOnly = true;
					rgpbfShape[1]->m_volmd.bNoCrChange = m_volmd.bNoCrChange; // set CR change disable(Oct. 9 1997)
					rgpbfShape[1]->m_vopmd.binterlace = false;
					rgpbfShape[1]->decodeIVOP_WithShape ();
*/
				}
			} // end of "if(m_vopmd.iLoadBakShape)"
			else
				m_vopmd.iLoadForShape = 0; // no forward shape when backward shape is not decoded
		}
		else {
			m_vopmd.iLoadForShape = 0;
			m_vopmd.iLoadBakShape = 0;
		}
        m_vopmd.iRefSelectCode = m_pInBitstream ->bitstreamReadBits (2) ;
	}

// by D.S.CHO SS 
#endif	// #ifdef	_DSCHO_DBG_
//~by D.S.CHO SS
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	return true;
}