/*************************************************************************************

 This software module was originally developed by

        Dae-Sung Cho    (dscho@sait.samsung.co.kr), Samsung AIT

 and edited by

        Dae-Sung Cho    (dscho@sait.samsung.co.kr), Samsung AIT

 Those who intend to use this software module in hardware or software products
 are advised that its use may infringe existing patents and other intellectual
 property rights.

 We disclaim all warranties with regard to this software, including
 all implied warranties of merchantability and fitness.
 In no event, Samsung AIT shall be liable for any special, indirect or
 consequential damages or any damages whatsoever resulting from loss of
 data or profits arising out of or in connection with the use or performance
 of this software.

 Samsung AIT retains the full right to use the code for its own purpose, assign
 or donate the code to a third party and to inhibit third parties from using
 the code. This copyright notice must be included in all copies or derivative
 works.

 Copyright (c) 1997-2000 Samsung AIT all rights reserved


 Required module :

 Authors :

 Changes :
        date            programmers     comment

*************************************************************************************/
#if !defined(_MP4VIDEO_ANALYZER_H_)
#define _MP4VIDEO_ANALYZER_H_

#include "bitstream_i.h"
#include "mp4video_codehead.h"	// by D.S.CHO SS

#if !defined (MAX_MAC)
#define MAX_MAC 10
#endif
#if !defined (BLOCK_SQUARE_SIZE)
#define BLOCK_SQUARE_SIZE 64
#endif
#if !defined (MB_SIZE)
#define MB_SIZE 16
#endif

typedef int Time;

typedef enum {BASE_LAYER, ENHN_LAYER} VOLtype; // will be generlized later
typedef enum AlphaUsage {RECTANGLE, ONE_BIT, EIGHT_BIT} AlphaUsage;
typedef enum {Q_H263, Q_MPEG} Quantizer; 
typedef enum {IVOP, PVOP, BVOP, SPRITE, UNKNOWNVOPTYPE} VOPpredType;
typedef enum {STOP, PIECE, UPDATE, PAUSE, NEXT} SptXmitMode;
typedef enum {BASIC_SPRITE, LOW_LATENCY, PIECE_OBJECT, PIECE_UPDATE} SptMode;	// basic sprite, and low-latency (object only, update only, intermingled) 

//MPEG quantizer matrix
static int rgiDefaultIntraQMatrix [BLOCK_SQUARE_SIZE] = {
	8,	17,	18,	19,	21,	23,	25,	27,
	17,	18,	19,	21,	23,	25,	27,	28,
	20,	21,	22,	23,	24,	26,	28,	30,
	21,	22,	23,	24,	26,	28,	30,	32,
	22,	23,	24,	26,	28,	30,	32,	35,
	23,	24,	26,	28,	30,	32,	35,	38,
	25,	26,	28,	30,	32,	35,	38,	41,
	27,	28,	30,	32,	35,	38,	41,	45
};

static int rgiDefaultInterQMatrix [BLOCK_SQUARE_SIZE] = {
	16,	17,	18, 19, 20,	21,	22,	23,
	17,	18,	19,	20,	21,	22,	23,	24,
	18,	19,	20,	21,	22,	23,	24,	25,
	19,	20,	21,	22,	23,	24,	26,	27,
	20,	21,	22,	23,	25,	26,	27,	28,
	21,	22,	23,	24,	26,	27,	28,	30,
	22,	23,	24,	26,	27,	28,	30,	31,
	23,	24,	25,	27,	28,	30,	31,	33
};

static int rgiDefaultIntraQMatrixAlpha [BLOCK_SQUARE_SIZE] = {
	8,	17,	18,	19,	21,	23,	25,	27,
	17,	18,	19,	21,	23,	25,	27,	28,
	20,	21,	22,	23,	24,	26,	28,	30,
	21,	22,	23,	24,	26,	28,	30,	32,
	22,	23,	24,	26,	28,	30,	32,	35,
	23,	24,	26,	28,	30,	32,	35,	38,
	25,	26,	28,	30,	32,	35,	38,	41,
	27,	28,	30,	32,	35,	38,	41,	45
};

static int rgiDefaultInterQMatrixAlpha [BLOCK_SQUARE_SIZE] = {
	16,	17,	18, 19, 20,	21,	22,	23,
	17,	18,	19,	20,	21,	22,	23,	24,
	18,	19,	20,	21,	22,	23,	24,	25,
	19,	20,	21,	22,	23,	24,	26,	27,
	20,	21,	22,	23,	25,	26,	27,	28,
	21,	22,	23,	24,	26,	27,	28,	30,
	22,	23,	24,	26,	27,	28,	30,	31,
	23,	24,	25,	27,	28,	30,	31,	33
};

static int grgiStandardZigzag [BLOCK_SQUARE_SIZE] = {
	0, 1, 8, 16, 9, 2, 3, 10, 
	17, 24, 32, 25, 18, 11, 4, 5, 
	12, 19, 26, 33, 40, 48, 41, 34, 
	27, 20, 13, 6, 7, 14, 21, 28, 
	35, 42, 49, 56, 57, 50, 43, 36, 
	29, 22, 15, 23, 30, 37, 44, 51, 
	58, 59, 52, 45, 38, 31, 39, 46, 
	53, 60, 61, 54, 47, 55, 62, 63
};

typedef struct VOLMode				 // VideoObjectLayer Mode
{
	unsigned	uiVerID;		 // Version identification // GMC
	VOLtype	volType;		 // what type of VOL

	// NBIT: nbit information
	bool 	bNot8Bit;
	unsigned 	uiQuantPrecision;
	unsigned 	nBits;

	// time info
	int	iClockRate;			 //rate of clock used to count vops in Hz

	// shape info
	AlphaUsage	fAUsage;		 //binary or gray level alpha; or no alpha (rectangle VO)
	int     iAlphaShapeExtension;
	int     iAuxCompCount;
	bool	bShapeOnly;			 // code only the shape
	bool	bNoCrChange;			 //nobinary shape size conversion
	
// by D.S.CHO SS
	int		iVolWidth;
	int		iVolHeight;
//~by D.S.CHO SS	
	// motion info
	bool	bAdvPredDisable;		// No OBMC, (8x8 in the future).
  	bool    bQuarterSample;         	// Quarter sample
	bool	bRoundingControlDisable;

// NEWPRED
	bool	bNewpredEnable;
	int 	iRequestedBackwardMessegeType;
	bool	bNewpredSegmentType;
// ~NEWPRED

	bool	bDataPartitioning;		// data partitioning
	bool	bReversibleVlc;			// reversible VLC

	// texture coding info
	Quantizer fQuantizer;			// either H.263 or MPEG
	bool 	bLoadintraMatrix;			// flag indicating whether to load intra Q-matrix
	int 	rgiintraQuantizerMatrix [BLOCK_SQUARE_SIZE]; // intra Q-Matrix
	bool 	bLoadinterMatrix;			// flag indicating whether to load inter Q-matrix
	int 	rgiinterQuantizerMatrix [BLOCK_SQUARE_SIZE]; // inter Q-Matrix
	bool 	bLoadintraMatrixAlpha;			// flag indicating whether to load intra Q-matrix
	int  	rgiintraQuantizerMatrixAlpha[MAX_MAC][BLOCK_SQUARE_SIZE];
	bool 	bLoadinterMatrixAlpha;			// flag indicating whether to load inter Q-matrix
	int  	rgiinterQuantizerMatrixAlpha[MAX_MAC][BLOCK_SQUARE_SIZE];
	bool 	bDeblockFilterDisable;		// apply deblocking filter or not.
	bool 	bNoGrayQuantUpdate;		// decouple gray quant and dquant

	// HHI Klaas Schueuer sadct flag
	bool	bSadctDisable;
	// end HHI 

	// START: Complexity Estimation syntax support - Marc Mongenet (EPFL) - 15 Jun 1998
	bool 	bComplexityEstimationDisable;
	int 	iEstimationMethod;
	bool 	bShapeComplexityEstimationDisable;
	bool 	bOpaque;
	bool 	bTransparent;
	bool 	bintraCAE;
	bool 	binterCAE;
	bool 	bNoUpdate;
	bool 	bUpsampling;
	bool 	bTextureComplexityEstimationSet1Disable;
	bool 	bintraBlocks;
	bool 	binterBlocks;
	bool 	binter4vBlocks;
	bool 	bNotCodedBlocks;
	bool 	bTextureComplexityEstimationSet2Disable;
	bool 	bDCTCoefs;
	bool 	bDCTLines;
	bool 	bVLCSymbols;
	bool 	bVLCBits;
	bool 	bMotionCompensationComplexityDisable;
	bool 	bAPM;
	bool 	bNPM;
	bool 	binterpolateMCQ;
	bool 	bForwBackMCQ;
	bool 	bHalfpel2;
	bool 	bHalfpel4;
	// START: Complexity Estimation syntax support - Update version 2 - Massimo Ravasi (EPFL) - 5 Nov 1999
	bool 	bVersion2ComplexityEstimationDisable;
	bool 	bSadct;
	bool 	bQuarterpel;
	// END: Complexity Estimation syntax support - Update version 2

	// START: Vol Control Parameters
	unsigned uiVolControlParameters;
	unsigned uiChromaFormat;
	unsigned uiLowDelay;
	unsigned uiVBVParams;
	unsigned uiBitRate;
	unsigned uiVbvBufferSize;
	unsigned uiVbvBufferOccupany;
	// END: Vol Control Parameters

	// scalability info
	int 	iHierarchyType;
	int 	iEnhnType;					//enhancement type
	int 	ihor_sampling_factor_m ;
	int 	ihor_sampling_factor_n ;
	int 	iver_sampling_factor_m ;
	int 	iver_sampling_factor_n ;

   	// object based spatial scalability
	int 	iuseRefShape;	
	int 	iuseRefTexture;
	int 	ihor_sampling_factor_m_shape ;
	int 	ihor_sampling_factor_n_shape ;
	int 	iver_sampling_factor_m_shape ;
	int 	iver_sampling_factor_n_shape ;

	bool 	bSpatialScalability;	
	bool 	bTemporalScalability;

	// RRV
	bool	breduced_resolution_vop_enable;
} VOLMode;

typedef struct VOPMode // VideoObjectPlane Mode
{
	// user specify, per VOP
	int intStepI;	// I-VOP stepsize for DCT
	int intStep;	// P-VOP stepsize for DCT
	int intStepB;	// B-VOP stepsize for DCT
	
  	int intStepIAlpha[MAX_MAC];	// I-VOP stepsize for DCT alpha
	int intStepPAlpha[MAX_MAC];	// P-VOP stepsize for DCT alpha
	int intStepBAlpha[MAX_MAC];	// B-VOP stepsize for DCT alpha

	VOPpredType vopPredType; //whether IVOP, PVOP, BVOP, or Sprite
	int 	iintraDcSwitchThr;	 //threshold to code intraDC as with AC VLCs
	int 	iRoundingControl;	 //rounding control
	int 	iVopConstantAlphaValue;   // for binary or grayscale shape pk val

	// Complexity Estimation syntax support - Marc Mongenet (EPFL) - 15 Jun 1998
	int 	iOpaque;
	int 	iTransparent;
	int 	iintraCAE;
	int 	iinterCAE;
	int 	iNoUpdate;
	int 	iUpsampling;
	int 	iintraBlocks;
	int 	iinterBlocks;
	int 	iinter4vBlocks;
	int 	iNotCodedBlocks;
	int 	iDCTCoefs;
	int 	iDCTLines;
	int 	iVLCSymbols;
	int 	iVLCBits;
	int 	iAPM;
	int 	iNPM;
	int 	iinterpolateMCQ;
	int 	iForwBackMCQ;
	int 	iHalfpel2;
	int 	iHalfpel4;
	// START: Complexity Estimation syntax support - Update version 2 - Massimo Ravasi (EPFL) - 5 Nov 1999
	int 	iSadct;
	int 	iQuarterpel;
	// END: Complexity Estimation syntax support - Update version 2

	// motion search info
// by D.S.CHO SS
	unsigned uiVopFcodeForward;
	unsigned uiVopFcodeBackward;
//~by D.S.CHO SS

	bool 	binterlace;						// interlace coding flag
	bool 	bTopFieldFirst;					// Top field first
    bool 	bAlternateScan;                    // Alternate Scan

	// for scalability
	int 	iRefSelectCode;
	int 	iLoadForShape; // load_forward_shape
	int 	iLoadBakShape; // load_backward_shape
	bool 	bShapeCodingType; // vop_shape_coding_type (0:intra, 1:inter): Added for error resilient mode by Toshiba(1997-11-14)
	SptXmitMode SpriteXmitMode;	// sprite transmit mode 

// NEWPRED
	int		m_iVopID;
	int		m_iNumBitsVopID;
	int		m_iVopID4Prediction_Indication;
	int		m_iVopID4Prediction;
// ~NEWPRED

} VOPMode;

class CMp4VideoAnalyzer 
{
public:
	// Constructors
	~CMp4VideoAnalyzer ();
	CMp4VideoAnalyzer ();
	
	// Operations ///////////////////////////////////////////////////////
	void AnalyzeVolHeader (unsigned char *pStream, int nLength);	// by D.S.CHO SS
	int AnalyzeVopHeader (unsigned char *pStream, int nLength);		// by D.S.CHO SS

	int findStartCode();

	// VO and VOL routines
	void decodeVOHead ();
	void decodeVOLHead ();

	// VOP routines
	bool decodeVOPHead ();
	int decodeH263Head ();		// by D.S.CHO SS
	void video_plane_with_short_header();

	// I/O ///////////////////////////////////////////////////////////////
	const VOLMode& volmd () const {return m_volmd;}	// by D.S.CHO SS
	const VOPMode& vopmd () const {return m_vopmd;}	// by D.S.CHO SS

	int getVolType () { return m_volmd.volType; } // 0 - BASE_LAYER, 1 - ENHN_LAYER
	int getHierarchyType() { return m_volmd.iHierarchyType; } // by D.S.CHO SS
	Time getVopTime () { return 1000*m_t/m_volmd.iClockRate; } // by D.S.CHO SS	
	int getVopTimeIncrement() { return m_t; }	// by D.S.CHO SS
	int getVopPredType () { return m_vopmd.vopPredType; }

private:
	CInBitstream *m_pInBitstream;
	
	VOLMode	m_volmd;
	VOPMode m_vopmd;
	unsigned m_uivoid; 			// VOP ID

	bool m_bvideo_plane_with_short_header;	// by D.S.CHO SS

	int m_ivolWidth;			// by D.S.CHO SS
	int m_ivolHeight;			// by D.S.CHO SS
	
	Time m_t; 					// current time
	Time m_tPastRef; 			// time of reference frame in past (for P/B)
	Time m_tFutureRef; 			// time of reference frame in future (for B)
	bool m_bCodedFutureRef;
	bool m_bCodedBaseRef;		// OBSS_SAIT_991015

	int m_iNumBitsTimeIncr;
	Time m_tDistanceBetwIPVOP; 	//no. of frms betw. I/PVOP = PeriodOfPVOP except for at irregular period (end of sequence)
	Time m_tPrevIorPVOPCounter; //frm. no of previous encoded I/PVOP
	Time m_tModuloBaseDisp;		//of the most recently displayed I/Pvop
	Time m_tModuloBaseDecd;		//of the most recently decoded I/Pvop
	
	bool m_bUseGOV ;
	bool m_bLinkisBroken;

	Time m_tOldModuloBaseDisp;			  //of the most recently displayed I/Pvop
	Time m_tOldModuloBaseDecd;			  //of the most recently decoded I/Pvop

	int m_iClockRateScale; // added by Sharp (98/6/26)

	int m_coded;

	// Added for short headers by KPN (1998-02-07, DS)
	unsigned uiPei; // [FDS]
	unsigned uiGobNumber;
	unsigned uiNumGobsInVop;
	unsigned uiNumMacroblocksInGob;
	unsigned uiGobHeaderEmpty;
	// Added for short headers by KPN - END

	unsigned m_uiSprite; // whether this is a sprite VOP: 1 - yes; other - no
	unsigned m_uiWarpingAccuracy; // accuracy for sprite warping
	int m_iNumOfPnts; // for sprite warping
	SptMode m_sptMode;  // sprite reconstruction mode : 0 -- basic sprite , 1 -- Object piece only, 2 -- Update piece only, 3 -- intermingled	
	SptXmitMode m_oldSptXmitMode ;	
};

#endif	 // #if !defined(_MP4VIDEO_ANALYZER_H_)