/*************************************************************************************
 
 This software module was originally developed by
 
	Dae-Sung Cho 	(dscho@sait.samsung.co.kr), Samsung AIT
 
 and edited by 

	Dae-Sung Cho	(dscho@sait.samsung.co.kr), Samsung AIT

 Those who intend to use this software module in hardware or software products
 are advised that its use may infringe existing patents and other intellectual
 property rights.
 
 We disclaim all warranties with regard to this software, including
 all implied warranties of merchantability and fitness.
 In no event, Samsung AIT shall be liable for any special, indirect or
 consequential damages or any damages whatsoever resulting from loss of
 data or profits arising out of or in connection with the use or performance
 of this software.
 
 Samsung AIT retains the full right to use the code for its own purpose, assign
 or donate the code to a third party and to inhibit third parties from using
 the code. This copyright notice must be included in all copies or derivative
 works.
 
 Copyright (c) 1997-2000 Samsung AIT all rights reserved
 

 Required module :
 
 Authors : 

 Changes :
    	date      	programmers     comment

*************************************************************************************/

#include "defs.h" // by D.S.CHO NEW

#include <stdio.h>
#include <stdlib.h>
#include <assert.h>

#include "mp4video_codehead.h"	// by D.S.CHO SS
#include "mp4video_parser.h"

CMp4VideoParser::CMp4VideoParser()
{
	m_pInBitstream = new CInBitstream;
	m_pOutBitstream = new COutBitstream;
}

// by D.S.CHO SS
/*
CMp4VideoParser::CMp4VideoParser(char *infile)
{
	m_pInBitstream = new CInBitstream(infile);
	m_pOutBitstream = new COutBitstream;
}
*/
//~by D.S.CHO SS

CMp4VideoParser::CMp4VideoParser(FILE *fStream)
{
	m_pInBitstream = new CInBitstream(fStream);
	m_pOutBitstream = new COutBitstream;
}


CMp4VideoParser::~CMp4VideoParser()
{
	if (m_pInBitstream!=NULL) 
		delete m_pInBitstream;
	if (m_pOutBitstream!=NULL)
		delete m_pOutBitstream;
}

void CMp4VideoParser::openParser()
{
	if (m_pInBitstream==NULL) 
		m_pInBitstream = new CInBitstream;
	if (m_pOutBitstream==NULL)
		m_pOutBitstream = new COutBitstream;
}

// by D.S.CHO SS
/*
void CMp4VideoParser::openParser(char *infile)
{
	if (m_pInBitstream==NULL) 
		m_pInBitstream = new CInBitstream;
	m_pInBitstream -> bitstreamOpen(infile);

	if (m_pOutBitstream==NULL)
		m_pOutBitstream = new COutBitstream;
}
*/
//~by D.S.CHO SS

void CMp4VideoParser::openParser(FILE *fStream)
{
	if (m_pInBitstream==NULL) 
		m_pInBitstream = new CInBitstream;
	m_pInBitstream -> bitstreamOpen(fStream);

	if (m_pOutBitstream==NULL)
		m_pOutBitstream = new COutBitstream;
}

void CMp4VideoParser::closeParser() 
{
	if (m_pInBitstream!=NULL) 
		delete m_pInBitstream;
	if (m_pOutBitstream!=NULL)
		delete m_pOutBitstream;
}

int CMp4VideoParser::parseFrameHeader()
{
	unsigned int uiValue;
	bool isEOF = false;

	if(m_pInBitstream->getInputFile() == NULL)
		return EOF;

	/* get bitstream buffer for VO header data */
	if (m_pInBitstream->bitstreamShowBits(NUMBITS_SHORT_HEADER_START_CODE) == SHORT_VIDEO_START_MARKER) 
	{ 
		/* find short Mp4Video header */
		isEOF = m_pInBitstream->isBitstreamEnd();
		if(isEOF) 	return EOF;

		uiValue = m_pInBitstream->bitstreamReadBits(22);
		isEOF = m_pInBitstream->isBitstreamEnd();
		if(isEOF) 	return EOF;
		m_pOutBitstream->bitstreamPutBits(uiValue, 22);

		if( findLastStartCode() == EOF) 
			return EOF;
	} else {
		/* find vidoe_object_start_code */
		isEOF = m_pInBitstream->isBitstreamEnd();
		if(isEOF) 	return EOF;

		if (findFirstStartCode()==EOF) 
			return EOF;

		/* sesion start_code skipping */
		if (m_pInBitstream->bitstreamShowBits(NUMBITS_START_CODE_SUFFIX)==SESSION_START_CODE)
		{
			isEOF = m_pInBitstream->isBitstreamEnd();
			if(isEOF) 	return EOF;

			uiValue = m_pInBitstream->bitstreamReadBits(NUMBITS_START_CODE_SUFFIX);
			isEOF = m_pInBitstream->isBitstreamEnd();
			if(isEOF) 	return EOF;
			m_pOutBitstream->bitstreamPutBits(uiValue, NUMBITS_START_CODE_SUFFIX);
			
			if (findNextStartCode() == NULL) 
				return EOF;
		}
		isEOF = m_pInBitstream->isBitstreamEnd(); 
		if(isEOF) 	return EOF;

		/* find vidoe_object_start_code */
		uiValue = m_pInBitstream->bitstreamReadBits(NUMBITS_VO_START_CODE);
		isEOF = m_pInBitstream->isBitstreamEnd(); 
		if (isEOF) 	return EOF;
		if (uiValue != VO_START_CODE) return EOF;

		m_pOutBitstream->bitstreamPutBits(uiValue, NUMBITS_VO_START_CODE);
		uiValue = m_pInBitstream->bitstreamReadBits(NUMBITS_VO_ID); 
		isEOF = m_pInBitstream->isBitstreamEnd();
		if(isEOF) 	return EOF;		
		m_pOutBitstream->bitstreamPutBits(uiValue, NUMBITS_VO_ID);

		/* find vidoe_object_layer_start_code */
		if( findNextStartCode() == EOF) 
			return EOF;
		uiValue = m_pInBitstream->bitstreamReadBits(NUMBITS_VOL_START_CODE);
		isEOF = m_pInBitstream->isBitstreamEnd(); 
		if(isEOF)	return EOF;
		if(uiValue != VOL_START_CODE)	return EOF;
		m_pOutBitstream->bitstreamPutBits(uiValue, NUMBITS_VOL_START_CODE);

		uiValue = m_pInBitstream->bitstreamReadBits(NUMBITS_VOL_ID);
		isEOF = m_pInBitstream->isBitstreamEnd(); 
		if(isEOF)	return EOF;		
		m_pOutBitstream->bitstreamPutBits(uiValue, NUMBITS_VOL_ID);

		if (findLastStartCode()==EOF) 
			return EOF;
	}	
	m_pOutBitstream->bitstreamFlush(); 

	return 0;
}

int CMp4VideoParser::parseFrame()
{
	unsigned int uiValue;
	bool isEOF = false;

	if(m_pInBitstream->getInputFile() == NULL)
		return EOF;

	/* get bitstream buffer for VOL header data */
	if (findFirstStartCode()==EOF) 
		return EOF;
	uiValue = m_pInBitstream->bitstreamReadBits(NUMBITS_VOP_START_CODE);
	isEOF = m_pInBitstream->isBitstreamEnd(); 
	if (isEOF)	return EOF;
	if(uiValue != VOP_START_CODE && uiValue != GOV_START_CODE) 		return EOF;

	m_pOutBitstream->bitstreamPutBits((int)uiValue, NUMBITS_VOP_START_CODE);
	if (uiValue == GOV_START_CODE) {
		uiValue = m_pInBitstream->bitstreamReadBits(20);
		isEOF = m_pInBitstream->isBitstreamEnd(); 
		if (isEOF)		return EOF;
		m_pOutBitstream->bitstreamPutBits((int)uiValue, 20);

		if (findNextStartCode()==EOF) 
			return EOF;
		uiValue = m_pInBitstream->bitstreamReadBits(NUMBITS_VOP_START_CODE);
		isEOF = m_pInBitstream->isBitstreamEnd(); 
		if (isEOF)	return EOF;
		if(uiValue != VOP_START_CODE) 	   	return EOF;
		m_pOutBitstream->bitstreamPutBits((int)uiValue, NUMBITS_VOP_START_CODE);
	}
	if (findLastStartCode()==EOF) 
		return EOF;
	m_pOutBitstream->bitstreamFlush(); 

	return 0;
}

/* find first start code and store start code prefixe to packet buffer */
int CMp4VideoParser::findFirstStartCode()
{
	unsigned int uiValue;
	bool isEOF = false;
	int bUserData;
//	int includeBody=0; // by D.S.CHO NEW

	/* ensure byte alignment */

	do {
		bUserData = 0;
		while (m_pInBitstream->bitstreamShowBits(NUMBITS_START_CODE_PREFIX)!=START_CODE_PREFIX) 
		{
			isEOF = m_pInBitstream->isBitstreamEnd(); 
			if(isEOF)	return EOF;

			m_pInBitstream->bitstreamReadBits(8);
			isEOF = m_pInBitstream->isBitstreamEnd(); 
			if(isEOF)	return EOF;
		}
		isEOF = m_pInBitstream->isBitstreamEnd(); 
		if(isEOF)	return EOF;
		if(m_pInBitstream->bitstreamShowBits(NUMBITS_START_CODE)==0x000001B2) {
			isEOF = m_pInBitstream->isBitstreamEnd(); 
			if(isEOF)	return EOF;
			m_pInBitstream->bitstreamReadBits(NUMBITS_START_CODE);
			isEOF = m_pInBitstream->isBitstreamEnd(); 
			if(isEOF)	return EOF;
			bUserData = 1;
		} else {
			isEOF = m_pInBitstream->isBitstreamEnd(); 
			if(isEOF)	return EOF;
			uiValue = m_pInBitstream->bitstreamReadBits(NUMBITS_START_CODE_PREFIX);
			isEOF = m_pInBitstream->isBitstreamEnd(); 
			if(isEOF)	return EOF;
			m_pOutBitstream->bitstreamPutBits((int)uiValue, NUMBITS_START_CODE_PREFIX);
		}
		isEOF = m_pInBitstream->isBitstreamEnd(); 
		if(isEOF)	return EOF;
	} while(bUserData);

	return 0;
}

/* find next start code and store all search data and start code prefixe to packet buffer */
int CMp4VideoParser::findNextStartCode()
{
	unsigned int uiValue;
	bool isEOF = false;
	int bUserData;

	/* ensure byte alignment */

	do {
		bUserData = 0;
		while (m_pInBitstream->bitstreamShowBits(NUMBITS_START_CODE_PREFIX)!=START_CODE_PREFIX)
		{
			isEOF = m_pInBitstream->isBitstreamEnd(); 
			if(isEOF)	return EOF;
			uiValue = m_pInBitstream->bitstreamReadBits(8);
			isEOF = m_pInBitstream->isBitstreamEnd(); 
			if(isEOF)	return EOF;
			m_pOutBitstream->bitstreamPutBits((int)uiValue, 8);
		}
		isEOF = m_pInBitstream->isBitstreamEnd(); 
		if(isEOF)	return EOF;
		uiValue = m_pInBitstream->bitstreamReadBits(NUMBITS_START_CODE_PREFIX);
		isEOF = m_pInBitstream->isBitstreamEnd();
		if(isEOF)	return EOF;
		m_pOutBitstream->bitstreamPutBits((int)uiValue, NUMBITS_START_CODE_PREFIX);
		if(m_pInBitstream->bitstreamShowBits(NUMBITS_START_CODE_SUFFIX)==USER_DATA_START_CODE)
			bUserData = 1;
		isEOF = m_pInBitstream->isBitstreamEnd(); 
		if(isEOF)	return EOF;
	} while(bUserData);

	return 0;
}

/* find next start code and store all search data and start code prefixe to packet buffer */
int CMp4VideoParser::findLastStartCode()
{
	unsigned int uiValue;
	bool isEOF = false;
	int bUserData;

	/* ensure byte alignment */

	do {
		bUserData = 0;
		while (m_pInBitstream->bitstreamShowBits(NUMBITS_START_CODE_PREFIX)!=START_CODE_PREFIX)
		{
			isEOF = m_pInBitstream->isBitstreamEnd(); 
			if(isEOF)	return EOF;
			uiValue = m_pInBitstream->bitstreamReadBits(8);
			isEOF = m_pInBitstream->isBitstreamEnd(); 
			if(isEOF)	return EOF;
			m_pOutBitstream->bitstreamPutBits((int)uiValue, 8);
		}
		isEOF = m_pInBitstream->isBitstreamEnd(); 
		if (isEOF)	return EOF;
		if(m_pInBitstream->bitstreamShowBits(NUMBITS_START_CODE)==0x000001B2) {
			isEOF = m_pInBitstream->isBitstreamEnd(); 
			if(isEOF)	return EOF;
			uiValue = m_pInBitstream->bitstreamReadBits(NUMBITS_START_CODE);
			isEOF = m_pInBitstream->isBitstreamEnd(); 
			if(isEOF)	return EOF;

			m_pOutBitstream->bitstreamPutBits((int)uiValue, NUMBITS_START_CODE);
			bUserData = 1;
		}
		isEOF = m_pInBitstream->isBitstreamEnd(); 
		if (isEOF)	return EOF;
	} while(bUserData);

	return 0;
}

