/////////////////////////////////////////////////////////////////////////////////////
// 
// This software module was originally developed by
// 
//    Dae-Sung Cho 	(dscho@sait.samsung.co.kr), Samsung AIT
// 
// and edited by 
//
//	  Dae-Sung Cho	(dscho@sait.samsung.co.kr), Samsung AIT
//
// Those who intend to use this software module in hardware or software products
// are advised that its use may infringe existing patents and other intellectual
// property rights.
// 
// We disclaim all warranties with regard to this software, including
// all implied warranties of merchantability and fitness.
// In no event, Samsung AIT shall be liable for any special, indirect or
// consequential damages or any damages whatsoever resulting from loss of
// data or profits arising out of or in connection with the use or performance
// of this software.
// 
// Samsung AIT retains the full right to use the code for its own purpose, assign
// or donate the code to a third party and to inhibit third parties from using
// the code. This copyright notice must be included in all copies or derivative
// works.
// 
// Copyright (c) 1997-2000 Samsung AIT all rights reserved
// 
//
// Required module :
// 
// Authors : 
//
// Changes :
//    	date      	programmers     comment
//
//
/////////////////////////////////////////////////////////////////////////////////////

#include "Defs.h"
#include "Mp4videoDec.h"
#include "Stream.h"
#include <mmsystem.h>
#ifdef _SAIT_3DTG_MP4VIDEO_
#include <fstream.h>
#endif _SAIT_3DTG_MP4VIDEO_


#if 1
	int count=0;
	DWORD prev;
	DWORD cur;
	unsigned int totalTime=0;
#endif

/*-------------------------------------------------------------
The exported function that instantiates the proper decoder
-------------------------------------------------------------*/
#ifdef _DSCHO_DBG_ 
FILE *fp; 
#endif 

void *CreateInstance ()
{
	return new Mp4videoDecoder;
}

Mp4videoDecoder :: Mp4videoDecoder () : 
	DecoderImp ()
{	
	m_pVideoDecoder = new CVideoDecoder;
#ifdef _DSCHO_DBG_
	fp = fopen ("video_dec.txt", "w");
#endif
	m_bIsFirstDecodedData = true; 
}

Mp4videoDecoder :: ~Mp4videoDecoder()
{
	if(m_pVideoDecoder) 
	{
		m_pVideoDecoder->videoDecoderEnd();
		delete m_pVideoDecoder;
		m_pVideoDecoder = NULL;
	}	
	if (m_pOutput) {
		delete m_pOutput;
		m_pOutput = NULL;
		m_pOutputSize = 0;
	}
#ifdef _DSCHO_DBG_
	if (fp) { 
		fclose(fp);	
		fp = NULL;
	}
#endif
}
FILE *hyeDebug;
/*-------------------------------------------------------------
Sets the decoder to CIF or QCIF
-------------------------------------------------------------*/
BOOL Mp4videoDecoder :: Setup (
	const BYTE *pSpecificInfo,
	int nSpecificInfoLength,
	int
)
{

#ifdef _HYE_
	hyeDebug = fopen("hye.txt", "wa"); 
#endif
	// currently only support base layer
#ifdef _DSCHO_DBG_
	fprintf(fp,"Mp4videoDecoder :: Setup (nSpecificInfoLength=%d)\n", nSpecificInfoLength); 
#endif
	if (pSpecificInfo==NULL || nSpecificInfoLength==0) {
		return FALSE;
	}

	int iDisplayWidth = 352; // default value
	int iDisplayHeight = 288; // default value

	m_pVideoDecoder->videoDecoderStart(NULL,iDisplayWidth,iDisplayHeight,(char *)pSpecificInfo,nSpecificInfoLength); // change void return to BOOL return????

	m_bitmapheader.biSize = sizeof (BITMAPINFOHEADER);
	m_bitmapheader.biWidth = iDisplayWidth; 
	m_bitmapheader.biHeight = iDisplayHeight;
	m_bitmapheader.biPlanes = 1; //always 
	m_bitmapheader.biBitCount = 24; 
	m_bitmapheader.biCompression = BI_RGB; //RGB_FORMAT  
	m_bitmapheader.biSizeImage =
		m_bitmapheader.biWidth * m_bitmapheader.biHeight * m_bitmapheader.biBitCount / 8; 
#ifdef _DSCHO_DBG_
	fprintf (fp,"width=%d height=%d\n", iDisplayWidth, iDisplayHeight);
#endif
// by d.s.cho
	//m_bIsAlpha = m_pVideoDecoder->isAlphaPlane (); // need modify ??? 
	m_bIsAlpha = m_pVideoDecoder->isShapeUsed (); // need modify ??? 
//~by d.s.cho
	m_nFrameBitsPerPixel = m_pVideoDecoder->getFrameBitsPerPixel (); // need modify ???
#ifdef _DSCHO_DBG_
	fprintf (fp,"m_bIsAlpha=%d,m_nFrameBitsPerPixel=%d\n", m_bIsAlpha?1:0, m_nFrameBitsPerPixel);
#endif

	m_pOutput = new unsigned char [m_bitmapheader.biWidth*m_bitmapheader.biHeight*3+1];
	m_pOutputSize = m_bitmapheader.biWidth*m_bitmapheader.biHeight*3;

	m_nOffSetX = m_nOffSetY = 0; 
	Yuv2RgbInit(); 

#ifdef _DSCHO_DBG_
	fprintf (fp,"Setup Success!!!\n");
#endif
															// non-null.
	return TRUE;
}

/*-----------------------------------------------------------------
Find a compatible format and initialize the decoder
-----------------------------------------------------------------*/
BOOL Mp4videoDecoder :: SetFormat (
	Capabilities &formatOptions,
	DecoderParams &decoderParams
)
{
//PAB: this should have been updated according to Zvi's comments to:
	decoderParams.m_nOptimalAUSize = 256; //PAB_core50RC1+:
	decoderParams.m_nDBSize = 81920; //PAB_core50RC1+:
	decoderParams.m_nCBSize = GetOptimalOutputSize () * 5; //PAB_core50RC1+:
	decoderParams.m_bRequiresFullAUs = TRUE; //PAB_core50RC1+:

   /*-------------------------------------------------------------
	If the preferred format is YUV, set the precise format descriptor
	in the BITMAPINFOHEADER structure
	-------------------------------------------------------------*/
	BOOL bResult = FALSE;
	int i;

// by d.s.cho
#if 0
	for (i = 0; !bResult; i++)
	{	if (i >= formatOptions.GetFormatCapabilityLength ())
			return FALSE;
		switch (formatOptions.GetFormatCapability (i).m_code) {
		case MediaFormat :: YUV:
			m_bitmapheader.biCompression = formatOptions.GetFormatCapability (i).m_dwAdditionalInfo;
			bResult = TRUE;
			break;
		case MediaFormat :: RGB:
			bResult = TRUE;
			break;
		}
	}
	decoderParams.m_nStreamNumber = 0;
	decoderParams.m_format = formatOptions.GetFormatCapability (--i);
	if (decoderParams.m_format.m_code == MediaFormat :: RGB)
		decoderParams.m_format.m_code=MediaFormat :: BGR;
#else
	// for core profile
	for (i = 0; !bResult; i++)
	{	if (i >= formatOptions.GetFormatCapabilityLength ())
			return FALSE;
		if (!m_bIsAlpha) 
		{	if (formatOptions.GetFormatCapability (i).m_code == MediaFormat :: YUV) 
			{	m_bitmapheader.biCompression = formatOptions.GetFormatCapability (i).m_dwAdditionalInfo;
				bResult = TRUE;
				break;
			}			
		} else 
		{	if (formatOptions.GetFormatCapability (i).m_code == MediaFormat :: YUVA) 
			{	m_bitmapheader.biCompression = formatOptions.GetFormatCapability (i).m_dwAdditionalInfo;
				bResult = TRUE;
				break;
			}
		}
	}
	decoderParams.m_nStreamNumber = 0;
	decoderParams.m_format = formatOptions.GetFormatCapability (i);
	if (decoderParams.m_format.m_code == MediaFormat :: RGB)
		decoderParams.m_format.m_code=MediaFormat :: BGR;

#endif
//~by d.s.cho

	return TRUE;
}

/*-----------------------------------------------------------------
Calculate the output size in bytes
-----------------------------------------------------------------*/
int Mp4videoDecoder :: GetOptimalOutputSize ()
{
#ifndef _SAIT_3DTG_MP4VIDEO_
	return m_bitmapheader.biWidth * m_bitmapheader.biHeight * 3 + 1;
#else _SAIT_3DTG_MP4VIDEO_
	return sizeof(int)*5 + m_bitmapheader.biWidth * m_bitmapheader.biHeight * 3 * 2;
#endif _SAIT_3DTG_MP4VIDEO_
}

#ifdef _SAIT_3DTG_MP4VIDEO_
/*-----------------------------------------------------------------
Calculate the frame size in bytes
-----------------------------------------------------------------*/
int Mp4videoDecoder :: GetYUVFrameSize ()
{
	return m_bitmapheader.biWidth * m_bitmapheader.biHeight * 3 / 2; // for yuv
}
#endif _SAIT_3DTG_MP4VIDEO_

/*-------------------------------------------------------------
Decode one unit
-------------------------------------------------------------*/
BOOL Mp4videoDecoder :: Decode (
		LPBYTE pInput,
		int nInputLength,
		LPBYTE pOutput,
		int &rnOutputLength
)
{
#ifndef _SAIT_3DTG_MP4VIDEO_
	if (rnOutputLength < GetOptimalOutputSize ())
	{	rnOutputLength = GetOptimalOutputSize ();
		return FALSE;
	}
#else _SAIT_3DTG_MP4VIDEO_
	if (rnOutputLength < GetYUVFrameSize ())
	{
		rnOutputLength = GetYUVFrameSize ();
		return FALSE;
	}
#endif _SAIT_3DTG_MP4VIDEO_

	bool changeRec; 
		                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                 
	changeRec = m_pVideoDecoder->videoDecoder((char *)pInput, nInputLength); // change void return to BOOL return ??????

	if(m_bIsFirstDecodedData) { m_bIsFirstDecodedData = false; return false; }

	unsigned char *pxlYframe = m_pVideoDecoder->getYframe();
	unsigned char *pxlUframe = m_pVideoDecoder->getUframe();
	unsigned char *pxlVframe = m_pVideoDecoder->getVframe();
	unsigned char *pxlBYframe = m_pVideoDecoder->getBYframe();
//	pxlBYframe = m_pVideoDecoder->getBYframe(); //hye

//////////hye
	int vopwidth, vopheight; 

	if(changeRec){
		
		m_pVideoDecoder->GetVopPos(&vopwidth, &vopheight); 

#ifdef _HYE_
//	fprintf(hyeDebug,"%d %d \n", vopwidth,vopheight  ); 
#endif
	
		m_bitmapheader.biWidth = vopwidth; 
		m_bitmapheader.biHeight = vopheight; 

	
	
	}
////////////~hye

	int iWidth = m_bitmapheader.biWidth;
	int iHeight = m_bitmapheader.biHeight;
	int iWidthUV = iWidth>>1;
	int iHeightUV = iHeight>>1;
//	int iValY, iValU, iValV, iValBY;
//	int iw, ih, iw1, ih1, iw2, ih2; 
//	int iWidthRGB = iWidth*3;
//	int l, m, n;


// by d.s.cho
#if 0

	// for RGB, RGBA
	if(m_bIsAlpha)
	{
		
		for(ih=0; ih<iHeight; ih+=2) {
			for(iw=0; iw<iWidth; iw+=2) {
				ih2 = ih>>1;
				iw2 = iw>>1;

				iValU = *(pxlUframe+ih2*iWidthUV+iw2);
				iValV = *(pxlVframe+ih2*iWidthUV+iw2);

				n= Cr_r_tab[iValV];
				l= (int) RIGHT_SHIFT((Cb_g_tab[iValU] + Cr_g_tab[iValV]), SCALEBITS);
				m= Cb_b_tab[iValU];

				for(ih1=ih; ih1<ih+2; ih1++) {
					for(iw1=iw; iw1<iw+2; iw1++) {
						iValY = *(pxlYframe+ih1*iWidth+iw1);
						iValBY = *(pxlBYframe+ih1*iWidth+iw1);
						if(iValBY != 0)
						{
							*(m_pOutput+ih1*iWidthRGB+iw1*3  ) = clp[iValY + m];
							*(m_pOutput+ih1*iWidthRGB+iw1*3+1) = clp[iValY + l];
							*(m_pOutput+ih1*iWidthRGB+iw1*3+2) = clp[iValY + n];
						}
						else
						{
							*(m_pOutput+ih1*iWidthRGB+iw1*3  ) = 0;
							*(m_pOutput+ih1*iWidthRGB+iw1*3+1) = 0;
							*(m_pOutput+ih1*iWidthRGB+iw1*3+2) = 0; 
						}
					}
				}
			}
		}

	}
	else
	{
		for(ih=0; ih<iHeight; ih+=2) {
			for(iw=0; iw<iWidth; iw+=2) {
				ih2 = ih>>1;
				iw2 = iw>>1;

				iValU = *(pxlUframe+ih2*iWidthUV+iw2);
				iValV = *(pxlVframe+ih2*iWidthUV+iw2);

				n= Cr_r_tab[iValV];
				l= (int) RIGHT_SHIFT((Cb_g_tab[iValU] + Cr_g_tab[iValV]), SCALEBITS);
				m= Cb_b_tab[iValU];

				for(ih1=ih; ih1<ih+2; ih1++) {
					for(iw1=iw; iw1<iw+2; iw1++) {
						iValY = *(pxlYframe+ih1*iWidth+iw1);
						*(m_pOutput+ih1*iWidthRGB+iw1*3  ) = clp[iValY + m];
						*(m_pOutput+ih1*iWidthRGB+iw1*3+1) = clp[iValY + l];
						*(m_pOutput+ih1*iWidthRGB+iw1*3+2) = clp[iValY + n];
					}
				}
			}
		}	
	}
#else
	// for YUV, YUVA
//	if(!m_bIsAlpha)
	{	memcpy(m_pOutput, pxlYframe, iWidth*iHeight );
		memcpy(m_pOutput+iWidth*iHeight, pxlUframe, iWidthUV*iHeightUV );
		memcpy(m_pOutput+iWidth*iHeight*5/4, pxlVframe, iWidthUV*iHeightUV );
		if(m_bIsAlpha)
			memcpy(m_pOutput+iWidth*iHeight*3/2, pxlBYframe, iWidth*iHeight );
	}
/*	else
	{	bool isOpaque;
		int offsetU,offsetV,offsetA;

		offsetU = iWidth*iHeight;
		offsetV = iWidth*iHeight*5/4;
		offsetA = iWidth*iHeight*3/2;

		for(ih=0; ih<iHeight; ih+=2) {
			for(iw=0; iw<iWidth; iw+=2) {
				ih2 = ih>>1;
				iw2 = iw>>1;

				// check transparency of chroma signals
				isOpaque = false;
				for(ih1=ih; ih1<ih+2; ih1++) {
					for(iw1=iw; iw1<iw+2; iw1++) {
						iValBY = *(pxlBYframe+ih1*iWidth+iw1);
						if (iValBY) {
							isOpaque = true;
							break;
						}
					}
					if (isOpaque) break;
				}
				if (isOpaque) {
					iValU = *(pxlUframe+ih2*iWidthUV+iw2);
					iValV = *(pxlVframe+ih2*iWidthUV+iw2);
					*(m_pOutput + offsetU + ih2*iWidthUV+iw2) = iValU;
					*(m_pOutput + offsetV + ih2*iWidthUV+iw2) = iValV;
				} else {
					*(m_pOutput + offsetU + ih2*iWidthUV+iw2) = 128;
					*(m_pOutput + offsetV + ih2*iWidthUV+iw2) = 128;
				}
				for(ih1=ih; ih1<ih+2; ih1++) {
					for(iw1=iw; iw1<iw+2; iw1++) {						
						iValBY = *(pxlBYframe+ih1*iWidth+iw1);
						if(iValBY != 0)
						{
							iValY = *(pxlYframe+ih1*iWidth+iw1);
							*(m_pOutput+ih1*iWidth+iw1) = iValY;
						}
						else
						{
							*(m_pOutput+ih1*iWidth+iw1) = 0;
						}
					}
				}
			}
		}
		memcpy(m_pOutput+offsetA, pxlBYframe, iWidth*iHeight );
	} */
#endif
//~by d.s.cho
	
	if (rnOutputLength > 0)
		memcpy(pOutput, m_pOutput, rnOutputLength );
	
	return TRUE;
}

/*-------------------------------------------------------------
In this decoder, the decoded data always strats at beginning of output
block
-------------------------------------------------------------*/
int Mp4videoDecoder :: GetDataOffset  (const LPBYTE)
{
	return 0;
}

/*-------------------------------------------------------------
In this decoder, attributes are fixed for all units
-------------------------------------------------------------*/
DWORD Mp4videoDecoder :: GetAttributes (
	ATTRIBUTE_CODE code, 
	const LPBYTE
)
{

	switch (code) {
	case FRAME_WIDTH:
		return m_bitmapheader.biWidth; 
	case FRAME_HEIGHT:
		return m_bitmapheader.biHeight; 
	case PITCH:
// by d.s.cho
#if 0
		return 3*m_bitmapheader.biWidth; // for RGB,RGBA
#else
		return m_bitmapheader.biWidth; // for YUV,YUVA
#endif
//~by d.s.cho
	case OFFSETX:
		return m_nOffSetX;
	case OFFSETY:
		return m_nOffSetY;
	case FRAME_BITS_PER_PIXEL: // bits per pixel in a video frame
		return m_nFrameBitsPerPixel; // ???
	case FRAME_PIXEL_FORMAT: // pixel format in a video frame
		return 0; // ???
	case IS_ALPHA: // is this video frame an alpha mask?
		return (m_bIsAlpha ? 1 : 0);
//	case  ALPHA_POINTER: //hye new
//		return (unsigned long) pxlBYframe; 

	default:
		return 0;
	}
}

void Mp4videoDecoder :: Run ()
{
	LPBYTE pInData, pOutData;
	int nInSize, nOutSize;
	DWORD dwTime;
	BOOL bEOU;
#ifdef _DSCHO_DBG_
	int diffTime, prevTime=-1;	
#endif
#ifdef _SAIT_3DTG_MP4VIDEO_
	// header information
	int nNumberOfFrame = 0;
	int nWidth = m_bitmapheader.biWidth;
	int nHeight = m_bitmapheader.biHeight;
	int nNumberOfChannels = 3;
	int nBytePerPixel = 3;

	// frame information
	int nVideoSize = sizeof(int)*5; // for nNumberOfFrame, width, height, #channel, BytePerPixel
	int nYUVFrameSize = nWidth * nHeight * 3 / 2;
	int nFrameSize = nWidth * nHeight * 3;
	int i = 0;

	// video buffer
	LPBYTE pOutVideo = NULL;
	LPBYTE pOutFrame = (LPBYTE)malloc(nFrameSize);
	if(pOutFrame == NULL)
		throw;
	nOutSize = 0;
#endif _SAIT_3DTG_MP4VIDEO_

	while (
		(pInData =
			(LPBYTE) GetInputStream () -> FetchNext (nInSize, dwTime, bEOU, TRUE)
		) !=
		END_OF_STREAM
	)
	{

		if (IsActive ())
		{
#ifndef _SAIT_3DTG_MP4VIDEO_
			nOutSize = GetOptimalOutputSize (),
			pOutData = (LPBYTE) GetOutputStream () -> Allocate (nOutSize, TRUE, FALSE);
			if (pOutData == NULL)
				throw;
			if (pOutData == END_OF_STREAM)
				return;

#else _SAIT_3DTG_MP4VIDEO_
			nVideoSize += nYUVFrameSize;
			pOutVideo = (LPBYTE)realloc(pOutVideo, nVideoSize);
			if (pOutVideo == NULL)
				throw;
			pOutData = (LPBYTE) GetOutputStream () -> Allocate (nVideoSize, TRUE, FALSE);
			if (pOutData == NULL)
			{
				nNumberOfFrame--;
				break;
			}
			if (pOutData == END_OF_STREAM)
				return;
			i = nNumberOfFrame++;
#endif _SAIT_3DTG_MP4VIDEO_

#ifdef _DSCHO_DBG_
			if (prevTime == -1) 
			{
				prevTime = (int)dwTime;
				fprintf (fp,"FetchNext: dwTime=%d nInSize=%d => nOutSize=%d\n",dwTime, nInSize, nOutSize);
				fflush(fp);
			} else 
			{
				diffTime = (int)dwTime - prevTime;			
				fprintf (fp,"FetchNext: dwTime=%d nInSize=%d => nOutSize=%d",dwTime, nInSize, nOutSize);
				if (diffTime > 100)
					fprintf (fp, "*********\n");
				else 
					fprintf (fp, "\n");
				fflush(fp);
				prevTime = (int)dwTime;
			}
#endif
#ifndef _SAIT_3DTG_MP4VIDEO_
			while (!Decode (pInData, nInSize, pOutData, nOutSize))
			{	pOutData = (LPBYTE) GetOutputStream () -> Reallocate (nOutSize);
				if (pOutData == NULL)
					throw;
				if (pOutData == END_OF_STREAM)
					return;
			}

			GetOutputStream () -> Dispatch (nOutSize, dwTime);

#else _SAIT_3DTG_MP4VIDEO_
			while (!Decode (pInData, nInSize, pOutFrame, nFrameSize))
			{
				pOutFrame = (LPBYTE)realloc(pOutFrame, nFrameSize);
				if (pOutFrame == NULL)
					throw;
			}

			// header information
			((int*)pOutVideo)[0] = nNumberOfFrame;
			if(i == 0)
			{
				((int*)pOutVideo)[1] = nWidth;
				((int*)pOutVideo)[2] = nHeight;
				((int*)pOutVideo)[3] = nNumberOfChannels;
				((int*)pOutVideo)[4] = nBytePerPixel;
			}

			// copy decoded yuv data into video data
			memcpy(pOutVideo+sizeof(int)*5 + i*nYUVFrameSize, pOutFrame, nYUVFrameSize);

			// dispatch video data
			memcpy(pOutData, pOutVideo, nVideoSize);
			GetOutputStream () -> Dispatch (nVideoSize, dwTime);
#endif _SAIT_3DTG_MP4VIDEO_
		}
		GetInputStream () -> Release (TRUE);
		Sleep(0);
	}

#ifdef _SAIT_3DTG_MP4VIDEO_
	/*//
	ofstream fout;
	fout.open("YUV-3D.bin", ios::binary);
	fout.write(pOutVideo+sizeof(int)*5, nNumberOfFrame*nYUVFrameSize);
	fout.close();

	fout.open("Bitstream-3D.bin", ios::binary);
	fout.write(pOutData, nVideoSize);
	fout.close();
	//*/

	free(pOutFrame);
	free(pOutVideo);

	GetOutputStream () -> Dispatch (0, dwTime);
	GetOutputStream () -> Close ();
#endif _SAIT_3DTG_MP4VIDEO_

//	GetOutputStream () -> Dispatch (0, dwTime);


//	GetOutputStream () -> Close ();
}

void Mp4videoDecoder::Yuv2RgbInit()
{
	int		i;
	long	x;

	for(i = 0, x = -CENTERJSAMPLE; i <= MAXJSAMPLE; i++, x++)
	{
		Cr_r_tab[i] = (int)RIGHT_SHIFT((FIX(1.40200) * x + ONE_HALF), SCALEBITS);
		Cb_b_tab[i] = (int)RIGHT_SHIFT((FIX(1.77200) * x + ONE_HALF), SCALEBITS);
		Cr_g_tab[i] = (- FIX(0.71414)) * x;
		Cb_g_tab[i] = (- FIX(0.34414)) * x + ONE_HALF;
	}
	clp = clip+384;
	for (i= -384; i<640; i++)
		clp[i] = (i<0) ? 0 : ((i>255) ? 255 : i);
}
