/////////////////////////////////////////////////////////////////////////////////////
// 
// This software module was originally developed by
// 
//    Dae-Sung Cho 	(dscho@sait.samsung.co.kr), Samsung AIT
// 
// and edited by 
//
//	  Dae-Sung Cho	(dscho@sait.samsung.co.kr), Samsung AIT
//
// Those who intend to use this software module in hardware or software products
// are advised that its use may infringe existing patents and other intellectual
// property rights.
// 
// We disclaim all warranties with regard to this software, including
// all implied warranties of merchantability and fitness.
// In no event, Samsung AIT shall be liable for any special, indirect or
// consequential damages or any damages whatsoever resulting from loss of
// data or profits arising out of or in connection with the use or performance
// of this software.
// 
// Samsung AIT retains the full right to use the code for its own purpose, assign
// or donate the code to a third party and to inhibit third parties from using
// the code. This copyright notice must be included in all copies or derivative
// works.
// 
// Copyright (c) 1997-2000 Samsung AIT all rights reserved
// 
//
// Required module :
// 
// Authors : 
//
// Changes :
//    	date      	programmers     comment
//
//
/////////////////////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include <stdlib.h>
#include <fstream.h>
#include <strstrea.h> // by D.S.CHO (2000/05/30)

#include "typeapi.h"
#include "codehead.h"
#include "mode.hpp"
#include "vopses.hpp"
#include "entropy/bitstrm.hpp"
#include "tps_enhcbuf.hpp"
#include "decoder/enhcbufdec.hpp"
#include "decoder/vopsedec.hpp"

//#include "dataStruct.hpp" // by D.S.CHO

#ifndef __GLOBAL_VAR_
#define __GLOBAL_VAR_
#endif

#include "global.hpp"

#ifdef __MFC_
#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif
#define new DEBUG_NEW
#endif // __MFC_


#include "video_decoder.h" // by D.S.CHO (2000/05/30)

#define MAX_TIME	99999			// for bitstreams stopped at BVOP

#ifdef _HYE_
	FILE *hyeDebug2 = fopen("hye2.txt", "wa"); 
#endif
// Main Routine
	
void CVideoDecoder::videoDecoderStart(char *chrBitsFile, int &iDisplayWidth, int &iDisplayHeight, char *pBitstream, long lBitstreamLength,Bool bTScalability)
{

	Bool bScalability = 0;
	m_bScalable = FALSE;
	m_bSpatialScalable = FALSE;

	strstreambuf *pistrm = NULL;
	pistrm = new strstreambuf (pBitstream,(int)lBitstreamLength);
	
	Int iFrmWidth=iDisplayWidth;
	Int iFrmHeight=iDisplayHeight;
	Bool bSpatialScalable = FALSE;
	Bool bTemporalScalability = TRUE;
	bScalability = bTScalability;

	m_main_short_video_header = FALSE; // Added by KPN for short headers [FDS]

	if (bScalability == FALSE)
	{
		m_pvodec [BASE_LAYER] = new CVideoObjectDecoder (chrBitsFile, iFrmWidth, iFrmHeight, &bSpatialScalable, &m_main_short_video_header, pistrm);
		iDisplayWidth = iFrmWidth;
		iDisplayHeight = iFrmHeight;

#ifdef _HYE_
//	fprintf(hyeDebug2, "%d %d\n",iDisplayWidth, iDisplayHeight ); 
#endif
		CRct rctDisplay (0, 0, iFrmWidth, iFrmHeight); 
	//	CRct rctDisplay (0, 0, 0, 0); //hye
		m_rctDisplay = rctDisplay;
		m_iEnhanceLayer = FALSE;
	}
	else
	{
		m_pvodec[ENHN_LAYER]	= new CVideoObjectDecoder (chrBitsFile, iFrmWidth, iFrmHeight, &bSpatialScalable, &m_main_short_video_header, pistrm);
		
		m_bSpatialScalable = bSpatialScalable;
		m_bScalable = TRUE;
		
		iDisplayWidth=iFrmWidth; 
		iDisplayHeight=iFrmHeight;
		if (m_bSpatialScalable == TRUE)
		{
			(m_pvodec[ENHN_LAYER]->getvolmd()).bSpatialScalability	= bSpatialScalable;	//OBSSFIX_1:1
			(m_pvodec[ENHN_LAYER]->getvolmd()).iFrmWidth_SS			= iFrmWidth;		//OBSSFIX_1:1
			(m_pvodec[ENHN_LAYER]->getvolmd()).iFrmHeight_SS			= iFrmHeight;	//OBSSFIX_1:1
		}
		
		CRct trctDisplay_SSenh(0, 0, iFrmWidth, iFrmHeight);
		m_rctDisplay_SSenh = trctDisplay_SSenh;									//Added

		// KJH
		int voltype = (m_pvodec[ENHN_LAYER]->getvolmd()).volType; 
		if(voltype==1) // temporal S
		{
			// for back/forward shape
			//KJHKJHm_pvodec[ENHN_LAYER]->rgpbfShape[0] = new CVideoObjectDecoder ( iFrmWidth, iFrmHeight );
			//m_pvodec[ENHN_LAYER]->rgpbfShape[1] = new CVideoObjectDecoder ( iFrmWidth, iFrmHeight );
			m_pvodec[BASE_LAYER]->setClockRateScale( m_pvodec[ENHN_LAYER] );
			// copy pointers
			//m_pvodec[ENHN_LAYER]->copyTobfShape ();
		}
		m_iEnhanceLayer = TRUE;
		
	}
	//assert(!bSpatialScalable); 

	//KJH
	if(m_pxlYframe != NULL)
	{
		delete [] m_pxlYframe;
	}
	m_pxlYframe = new unsigned char [iDisplayWidth*iDisplayHeight+1];

	if(m_pxlUframe != NULL)
	{
		delete [] m_pxlUframe;
	}
	m_pxlUframe = new unsigned char [iDisplayWidth*iDisplayHeight/4+1];

	if(m_pxlVframe != NULL)
	{
		delete [] m_pxlVframe;
	}
	m_pxlVframe = new unsigned char [iDisplayWidth*iDisplayHeight/4+1];

	if(m_pxlBYframe != NULL)
	{
		delete [] m_pxlBYframe;
	}
	m_pxlBYframe = new unsigned char [iDisplayWidth*iDisplayHeight+1];
	//~KJH
	

	m_iEof = 1;
	m_nFrames = 0;
	m_bCachedRefFrame = FALSE;
	m_bCachedRefFrameCoded = FALSE;
	return;
}

///////HYE
bool CVideoDecoder::resetBuf()
{
	bool bRctChange = false; 
	if(m_pvodec[BASE_LAYER]->volmd().fAUsage != RECTANGLE){
		Int largerWidth=0, largerHeight=0; 
		Int vopleft, voptop,vopwidth,vopheight;
	
		
		GetVopPos( &vopleft, &voptop, &vopwidth, &vopheight); 
		
		if(m_rctDisplay.width < vopleft + vopwidth){
			largerWidth = vopleft + vopwidth ;
			bRctChange = true; 
		}else largerWidth = m_rctDisplay.width; 
		if( m_rctDisplay.height() < voptop + vopheight){
			largerHeight = voptop + vopheight; 			
			bRctChange = true; 
		} else largerHeight = m_rctDisplay.height(); 
		
		if(bRctChange){
			CRct rctDisplay (0, 0, largerWidth, largerHeight);
			//CRct rctDisplay  = m_pvodec[BASE_LAYER]->GetCurVOPY();
			m_rctDisplay = rctDisplay; 
				if(m_pxlYframe != NULL)
	{
		delete [] m_pxlYframe;
	}
	m_pxlYframe = new unsigned char [largerWidth*largerHeight+1];

	if(m_pxlUframe != NULL)
	{
		delete [] m_pxlUframe;
	}
	m_pxlUframe = new unsigned char [largerWidth*largerHeight/4+1];

	if(m_pxlVframe != NULL)
	{
		delete [] m_pxlVframe;
	}
	m_pxlVframe = new unsigned char [largerWidth*largerHeight/4+1];

	if(m_pxlBYframe != NULL)
	{
		delete [] m_pxlBYframe;
	}
	m_pxlBYframe = new unsigned char [largerWidth*largerHeight+1];

		
		}
	} 
	return bRctChange; 

}
///////////~HYE
bool CVideoDecoder::videoDecoder(char *pBitstream, long lBitstreamLength,Bool bTScalability)
{
	Bool bScalability;
	Bool bTemporalScalability;

	bScalability = bTScalability;
	bTemporalScalability = !m_bSpatialScalable;

	strstreambuf *pistrm = NULL;
	bool bRctChange = false; 



//	if(pBitstream!=NULL) { //hye
		pistrm = new strstreambuf (pBitstream,(int)lBitstreamLength);
//	} 

	if ( !bScalability ){
	
		UInt nBits = m_pvodec [BASE_LAYER]->volmd ().nBits;
		const CVOPU8YUVBA* pvopcQuant;
			if (m_main_short_video_header) // Added by KPN for short headers
			{
				m_iEof = m_pvodec [BASE_LAYER] -> h263_decode ();
			} else {
				m_iEof = m_pvodec [BASE_LAYER] -> decode (NULL,pistrm);
				bRctChange = resetBuf(); //hye
			}
			if (m_iEof != EOF) 
				m_nFrames++; 

			if(m_bSpatialScalable == FALSE && m_bScalable == TRUE)
				m_pvodec[BASE_LAYER] -> updateBuffVOPsBase (m_pvodec[ENHN_LAYER]);

			if(m_pvodec [BASE_LAYER] -> fSptUsage () == 1) 
			{
				// sprite
				if(m_iEof != EOF)
				{
					pvopcQuant = m_pvodec [BASE_LAYER]->pvopcReconCurr();
					getDisplayFrame (pvopcQuant, m_pvodec[BASE_LAYER]->volmd().fAUsage, m_rctDisplay); // by D.S.CHO (2000/05/30)
				}
			}
			else
			{
				if(m_iEof == EOF)
				{
					// dump final cached frame if present
					if(m_bCachedRefFrame)
					{
						m_bCachedRefFrame = FALSE;
						if(m_bCachedRefFrameCoded)
						{
							pvopcQuant = m_pvodec [BASE_LAYER]->pvopcRefQLater(); // future reference
							getDisplayFrame (pvopcQuant, m_pvodec[BASE_LAYER]->volmd().fAUsage, m_rctDisplay); // by D.S.CHO (2000/05/30)
						}
					}
				}
				else
				{
					// dump if bvop
					if(m_pvodec[BASE_LAYER]->vopmd().vopPredType == BVOP)
					{
						// BVOP
						if(m_iEof != FALSE)
						{
							pvopcQuant = m_pvodec [BASE_LAYER]->pvopcReconCurr(); // current vop
							getDisplayFrame (pvopcQuant, m_pvodec[BASE_LAYER]->volmd().fAUsage, m_rctDisplay); // by D.S.CHO (2000/05/30)
						}
					}
					else
					{
						// not a BVOP, so dump any previous cached frame
						if(m_bCachedRefFrame)
						{
							m_bCachedRefFrame = FALSE;
							if(m_bCachedRefFrameCoded)
							{
								pvopcQuant = m_pvodec [BASE_LAYER]->pvopcRefQPrev(); // past reference
								getDisplayFrame (pvopcQuant, m_pvodec[BASE_LAYER]->volmd().fAUsage, m_rctDisplay);// by D.S.CHO (2000/05/30)
							}
						}

						// cache current reference
						m_bCachedRefFrame = TRUE;
						m_bCachedRefFrameCoded = (m_iEof != FALSE);
					}
				}
			}

	}
	else if( bScalability == TRUE && bTemporalScalability == TRUE )
	{ // loop for temporal scalability



		int voltype = (m_pvodec[ENHN_LAYER]->getvolmd()).volType; 
		assert(voltype==bScalability); 

		bTemporalScalability = (Bool)(m_pvodec[ENHN_LAYER]->getvolmd ()).iHierarchyType; // by D.S.CHO (2000/05/30)
		assert(bTemporalScalability == 1);
		
		m_pvodec[ENHN_LAYER] -> decode (NULL,pistrm);

		//KJH - used???
		//m_pvodec[ENHN_LAYER] -> bufferB1flush ();
		//~KJH

		getDisplayFrame (m_pvodec[ENHN_LAYER]->pvopcReconCurr(), m_pvodec[ENHN_LAYER]->volmd().fAUsage, m_rctDisplay_SSenh);

		/* temporarily not using
		if (m_pvodec [ENHN_LAYER] -> volmd ().fAUsage != RECTANGLE) {
			// for background composition
			if(m_pvodec [ENHN_LAYER] -> volmd ().iEnhnType != 0) { // change for Norio Ito
				// should be changed to background_composition flag later.
				printf("============== background composition (2)\n");
				pvodec [ENHN_LAYER] -> BackgroundComposition(argv, bScalability, rctDisplay.width, rctDisplay.height (), pfReconYUV [ENHN_LAYER]); // modified by Sharp (98/10/26)
			}
			// for back/forward shape
			if(m_pvodec [ENHN_LAYER] -> vopmd ().iLoadBakShape) {
				printf("---------- output backward shape\n");
				const CVOPU8YUVBA* pvopcQuant = pvodec [ENHN_LAYER]->rgpbfShape[0]->pvopcReconCurr ();
				pvopcQuant->getPlane (BY_PLANE)->dump (pfTestSeg, rctDisplay);
			}
			if(pvodec [ENHN_LAYER] -> vopmd ().iLoadForShape) {
				printf("---------- output forward shape\n");
				const CVOPU8YUVBA* pvopcQuant = pvodec [ENHN_LAYER]->rgpbfShape[1]->pvopcReconCurr ();
				pvopcQuant->getPlane (BY_PLANE)->dump (pfTestSeg, rctDisplay);
			}
		}*/
	}
	else if( bScalability == TRUE && bTemporalScalability == FALSE ) // Spatial Scalability
	{


		int voltype = (m_pvodec[ENHN_LAYER]->getvolmd()).volType; 
		assert(voltype==bScalability); 

		bTemporalScalability = (Bool)(m_pvodec[ENHN_LAYER]->getvolmd ()).iHierarchyType; // by D.S.CHO (2000/05/30)
		assert(!bTemporalScalability);
		
		const CVOPU8YUVBA* pvopcQuant;// to be modifed to optimization
		
		pvopcQuant			= m_pvodec[BASE_LAYER]->pvopcReconCurr();	//Added by SONY 98/11/25
		m_iBASEVOP_time		= m_pvodec[BASE_LAYER]->show_current_time();	
		m_iBASEVOP_PredType	= m_pvodec[BASE_LAYER]->vopmd().vopPredType;
		

		//These static parameters will be used in the case BVOP is available in the BASE_LAYER (for spatial scalability)
		static CVOPU8YUVBA* pBASE_stackVOP			= NULL; //Temporal buffer of BASELAYER reconstructed image
		static Int			iBase_stackTime			= -1;
		static CRct			pBase_stack_rctBase;

		m_pvodec[ENHN_LAYER]->modify_filepointer (pistrm);
		Int m_iENHNVOP_time = m_pvodec[ENHN_LAYER]->senseTime();

		static ShapeMode*		pBase_stack_Baseshpmd	= NULL;
		static ShapeMode*		pBase_tmp_Baseshpmd		= NULL;
		static CMotionVector*	pBase_stack_mvBaseBY	= NULL;
		static CMotionVector*	pBase_tmp_mvBaseBY		= NULL;
		static Int				iBase_stack_x, iBase_stack_y;
		Bool					BGComposition			= FALSE;	//for OBSS partial enhancement mode

		if (m_iBASEVOP_time!=-1 )//KJHKJHKJHKJHKJH&& m_iBASEVOP_time==m_iENHNVOP_time)
		{
			//decode VOP using currentry decoded Base layer image
			// if base layer is non coded, pvopcQuant=NULL, but enh layer should also be non coded.
			m_iEof = m_pvodec [ENHN_LAYER]->decode (pvopcQuant);

			// for background composition (base layer(background) + partial enhancement layer)
			BGComposition = FALSE;
//			if(	pvodec [ENHN_LAYER] -> volmd().iHierarchyType == 0 && 
//				pvodec [ENHN_LAYER] -> volmd().iEnhnType == 1 && 
//				pvodec [ENHN_LAYER] -> volmd().iuseRefShape == 0 ) 
//				BGComposition = pvodec [ENHN_LAYER]->BackgroundComposition(rctDisplay_SSenh->width, rctDisplay_SSenh->height (), pfReconYUV[ENHN_LAYER], pfReconSeg[ENHN_LAYER]); 

			if(m_iEof!=EOF)	
				m_nFrames++; // include enhancement layer
//			if(iEof==FALSE)	dumpNonCodedFrame(pfReconYUV[ENHN_LAYER], pfReconSeg[ENHN_LAYER],*rctDisplay_SSenh, nBits);
			
			if (m_iEof != EOF && m_iEof!=FALSE )
			{
				//for OBSS partial enhancement mode
				if(!(m_pvodec[ENHN_LAYER]->volmd().iHierarchyType==0 && m_pvodec[ENHN_LAYER]->volmd().iEnhnType==1 && m_pvodec[ENHN_LAYER]->volmd().iuseRefShape==0 && BGComposition))
				{	
					pvopcQuant = m_pvodec[ENHN_LAYER]->pvopcReconCurr(); // past reference
					getDisplayFrame (pvopcQuant, m_pvodec[ENHN_LAYER]->volmd().fAUsage, m_rctDisplay_SSenh); // by D.S.CHO (2000/05/30)
				}
			}
		}
	}
	return bRctChange;
}

void CVideoDecoder::videoDecoderEnd(Bool bTScalability)
{
	if (!bTScalability)			delete m_pvodec [BASE_LAYER];
	else						delete m_pvodec [ENHN_LAYER];
	
	if(m_pxlYframe != NULL)
	{
		delete [] m_pxlYframe;
		m_pxlYframe = NULL;
	}

	if(m_pxlUframe != NULL)
	{
		delete [] m_pxlUframe;
		m_pxlUframe = NULL;
	}
	if(m_pxlVframe != NULL)
	{
		delete [] m_pxlVframe;
		m_pxlVframe = NULL;
	}
	if(m_pxlBYframe != NULL)
	{
		delete [] m_pxlBYframe;
		m_pxlBYframe = NULL;
	}

	return;
}

int CVideoDecoder::getDecConfigInfoBitstreams (char *pInBitstream, long lInBitsLength, char *&pOutBitstream, long &lOutBitsOffset, long &lOutBitsLength)
{
	// construction of bitstream buffer
	strstreambuf *pistrmbuf = NULL;
	ifstream* pistrm = NULL;
	CInBitStream* pbitstrmIn = NULL;

	if(pInBitstream==NULL) 
		return EOF;
	pistrmbuf = new strstreambuf (pInBitstream,(int)lInBitsLength);
	if(pistrmbuf==NULL) 
		return EOF;
	pistrm = (ifstream *)new istream (pistrmbuf);
	pbitstrmIn = new CInBitStream (*pistrm);

	// get bitstream buffer for VO header data 
	long lBufStart=0;
	long lBufEnd=0;

	if (pbitstrmIn->peekBits(NUMBITS_SHORT_HEADER_START_CODE) == SHORT_VIDEO_START_MARKER) { 
		// find short video header ///////////////////////////////////////
		pbitstrmIn -> getBits(22);
		lBufEnd = findStartCode(pbitstrmIn);
		if(lBufEnd==EOF)
			return EOF;
		lBufEnd --;//
	} else {
		// find vidoe_object_start_code //////////////////////////////////
		lBufStart = findStartCode(pbitstrmIn);
		if(lBufStart==EOF)
			return EOF;

 		//	Added for data partitioning mode By Toshiba(1998-1-16:DP+RVLC)(for Skip Session_Start_Code)
 		if( pbitstrmIn->peekBits(NUMBITS_START_CODE_SUFFIX)==SESSION_START_CODE){
 			pbitstrmIn -> getBits (NUMBITS_START_CODE_SUFFIX);
 			if (findStartCode (pbitstrmIn) == EOF)
 				return EOF; //assert(FALSE); // by D.S.CHO (2000/05/30)
 		}
 		//	End Toshiba(1998-1-16:DP+RVLC)

		// find vidoe_object_start_code //////////////////////////////////
		UInt uiVoStartCode = pbitstrmIn -> getBits (NUMBITS_VO_START_CODE);
		assert(uiVoStartCode == VO_START_CODE);
		UInt uiVOId = pbitstrmIn -> getBits (NUMBITS_VO_ID);
	
		// find vidoe_object_layer_start_code
 		if (findStartCode (pbitstrmIn) == EOF)
			return EOF;
 		UInt uiVolStartCode = pbitstrmIn -> getBits (NUMBITS_VOL_START_CODE);
		assert(uiVolStartCode == VOL_START_CODE);
		UInt uiVOLId = pbitstrmIn -> getBits (NUMBITS_VOL_ID);

		lBufEnd = findStartCode(pbitstrmIn);	
		if(lBufEnd == EOF)
			return EOF;
		lBufEnd --;//
	}
	pOutBitstream = pInBitstream + lBufStart;
	lOutBitsOffset = lBufStart;
	lOutBitsLength = lBufEnd - lBufStart + 1;

	delete pistrm;
	delete pistrmbuf;
	delete pbitstrmIn;

	return 0;
}

int CVideoDecoder::getVOLheaderBitstreams (char *pInBitstream, long lInBitsLength, char *&pOutBitstream, long &lOutBitsOffset, long &lOutBitsLength)
{
	// construction of bitstream buffer
	strstreambuf *pistrmbuf = NULL;
	ifstream* pistrm = NULL;
	CInBitStream* pbitstrmIn = NULL;

	if(pInBitstream==NULL) 
		return EOF;
	pistrmbuf = new strstreambuf (pInBitstream,(int)lInBitsLength);
	if(pistrmbuf==NULL) 
		return EOF;
	pistrm = (ifstream *)new istream (pistrmbuf);
	pbitstrmIn = new CInBitStream (*pistrm);

	// get bitstream buffer for VOL header data 
	long lBufStart=0;
	long lBufEnd=0;

	lBufStart = findStartCode(pbitstrmIn);
	if (lBufStart==EOF)
		return EOF;
	UInt uiVolStartCode = pbitstrmIn -> getBits (NUMBITS_VOL_START_CODE);
	assert(uiVolStartCode == VOL_START_CODE);
	UInt uiVOLId = pbitstrmIn -> getBits (NUMBITS_VOL_ID);
	lBufEnd = findStartCode(pbitstrmIn);
	if (lBufEnd==EOF)
		return EOF;
	lBufEnd --;//

	pOutBitstream = pInBitstream + lBufStart;
	lOutBitsOffset = lBufStart;
	lOutBitsLength = lBufEnd - lBufStart + 1;

	delete pistrm;
	delete pistrmbuf;
	delete pbitstrmIn;

	return 0;
}

int CVideoDecoder::getOneVopBitstreams (char *pInBitstream, long lInBitsLength, char *&pOutBitstream, long &lOutBitsOffset, long &lOutBitsLength)
{
	// construction of bitstream buffer
	strstreambuf *pistrmbuf = NULL;
	ifstream* pistrm = NULL;
	CInBitStream* pbitstrmIn = NULL;

	if(pInBitstream==NULL) 
		return EOF;
	pistrmbuf = new strstreambuf (pInBitstream,(int)lInBitsLength);
	if(pistrmbuf==NULL) 
		return EOF;
	pistrm = (ifstream *)new istream (pistrmbuf);
	pbitstrmIn = new CInBitStream (*pistrm);

	// get bitstream buffer for VOL header data 
	long lBufStart=0;
	long lBufEnd=0;

	lBufStart = findStartCode(pbitstrmIn);
	if(lBufStart == EOF)
		return EOF;
	UInt uiVopStartCode = pbitstrmIn -> getBits (NUMBITS_VOP_START_CODE);
	if (uiVopStartCode == GOV_START_CODE) {
		pbitstrmIn -> getBits (20);
		if (findStartCode(pbitstrmIn) == EOF)
			return EOF;
		uiVopStartCode = pbitstrmIn -> getBits (NUMBITS_VOP_START_CODE);
	}
	assert(uiVopStartCode == VOP_START_CODE);

	lBufEnd = findStartCode(pbitstrmIn);
	if (lBufEnd==EOF) 
		return EOF;
	lBufEnd --;//

	pOutBitstream = pInBitstream + lBufStart;
	lOutBitsOffset = lBufStart;
	lOutBitsLength = lBufEnd - lBufStart + 1;

	delete pistrm;
	delete pistrmbuf;
	delete pbitstrmIn;

	return 0;
}

long CVideoDecoder::findStartCode(CInBitStream* pbitstrmIn)
{
	// ensure byte alignment
	pbitstrmIn->flush ();

	int bUserData;
	long lBufOffset=0;	//by D.S.CHO (2000/05/30)
	do {
		bUserData = 0;
		while(pbitstrmIn->peekBits(NUMBITS_START_CODE_PREFIX)!=START_CODE_PREFIX)
		{
			pbitstrmIn->getBits(8);
			if(pbitstrmIn->eof()==EOF)
				return EOF;
		}
		lBufOffset = pbitstrmIn->getCounter() / 8; // by D.S.CHO (2000/05/30)
		pbitstrmIn->getBits(NUMBITS_START_CODE_PREFIX);
		if(pbitstrmIn->peekBits(NUMBITS_START_CODE_SUFFIX)==USER_DATA_START_CODE)
			bUserData = 1;
	} while(bUserData);

	return lBufOffset; //return 0; // by D.S.CHO (2000/05/30)
}

void CVideoDecoder::getDisplayFrame(const CVOPU8YUVBA* pvopcQuant, AlphaUsage fAUsage, CRct& rct)
{
	pvopcQuant->getPlane (Y_PLANE)->dump (m_pxlYframe, rct);
    pvopcQuant->getPlane (U_PLANE)->dump (m_pxlUframe, rct / 2);
    pvopcQuant->getPlane (V_PLANE)->dump (m_pxlVframe, rct / 2);
	if(fAUsage != RECTANGLE)
		pvopcQuant->getPlane (BY_PLANE)->dump (m_pxlBYframe, rct);

	return;
} 


Bool CVideoDecoder::getScalabilityType(void)
{
	// 1 - Spatial
	// 0 - Temporal

	return m_bSpatialScalable;
}
