/*************************************************************************

This software module was originally developed by 

	Ming-Chieh Lee (mingcl@microsoft.com), Microsoft Corporation
	Wei-ge Chen (wchen@microsoft.com), Microsoft Corporation
	Bruce Lin (blin@microsoft.com), Microsoft Corporation
	Chuang Gu (chuanggu@microsoft.com), Microsoft Corporation
	(date: March, 1996)
and edited by
	Wei Wu (weiwu@stallion.risc.rockwell.com) Rockwell Science Center
and also edited by
    Mathias Wien (wien@ient.rwth-aachen.de) RWTH Aachen / Robert BOSCH GmbH

in the course of development of the MPEG-4 Video (ISO/IEC 14496-2). 
This software module is an implementation of a part of one or more MPEG-4 Video tools 
as specified by the MPEG-4 Video. 
ISO/IEC gives users of the MPEG-4 Video free license to this software module or modifications 
thereof for use in hardware or software products claiming conformance to the MPEG-4 Video. 
Those intending to use this software module in hardware or software products are advised that its use may infringe existing patents. 
The original developer of this software module and his/her company, 
the subsequent editors and their companies, 
and ISO/IEC have no liability for use of this software module or modifications thereof in an implementation. 
Copyright is not released for non MPEG-4 Video conforming products. 
Microsoft retains full right to use the code for his/her own purpose, 
assign or donate the code to a third party and to inhibit third parties from using the code for non <MPEG standard> conforming products. 
This copyright notice must be included in all copies or derivative works. 

Copyright (c) 1996, 1997.

Module Name:

	vopSes.cpp

Abstract:

	Base class for the encoder for one VOP session.

Revision History:
    

*************************************************************************/
/////////////////////////////////////////////////////////////////////////////////////
// 
// This software module was originally developed by
// 
//    Dae-Sung Cho 	(dscho@sait.samsung.co.kr), Samsung AIT
// 
// and editted by 
//
//	  Dae-Sung Cho	(dscho@sait.samsung.co.kr), Samsung AIT
//
// Those who intend to use this software module in hardware or software products
// are advised that its use may infringe existing patents and other intellectual
// property rights.
// 
// We disclaim all warranties with regard to this software, including
// all implied warranties of merchantability and fitness.
// In no event, Samsung AIT shall be liable for any special, indirect or
// consequential damages or any damages whatsoever resulting from loss of
// data or profits arising out of or in connection with the use or performance
// of this software.
// 
// Samsung AIT retains the full right to use the code for its own purpose, assign
// or donate the code to a third party and to inhibit third parties from using
// the code. This copyright notice must be included in all copies or derivative
// works.
// 
// Copyright (c) 1997-2000 Samsung AIT all rights reserved
// 
//
// Required module :
// 
// Authors : 
//
// Changes :
//    	date      	programmers     comment

//
/////////////////////////////////////////////////////////////////////////////////////


#include <stdio.h>

#include "typeapi.h"
#include "dct.hpp"

#ifdef __MFC_
#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

#define new DEBUG_NEW				   
#endif // __MFC_

CBlockDCT::~CBlockDCT ()
{
/* NBIT: change
	m_rgchClipTbl -= 512;
*/
	m_rgchClipTbl -= (1<<(m_nBits+1));
	delete [] m_rgchClipTbl;
}

/* NBIT
CBlockDCT::CBlockDCT ()
*/
CBlockDCT::CBlockDCT (UInt nBits) : m_nBits(nBits)
{
	Int ClipTblSize = 1<<(m_nBits+2); // NBIT
	Int offset = ClipTblSize/2; // NBIT
	Int maxVal = (1<<m_nBits)-1; // NBIT

/* NBIT: change
	m_rgchClipTbl = new PixelC [1024];
	m_rgchClipTbl += 512;
	Int i;
	for (i = -512; i < 512; i++)	{
		if (i < 0)
			m_rgchClipTbl [i] = 0;
		else if (i >= 0 && i < 256)
			m_rgchClipTbl [i] = i;
		else 
			m_rgchClipTbl [i] = 255;
	}
*/
	m_rgchClipTbl = new PixelC [ClipTblSize];
	m_rgchClipTbl += offset;
	Int i;
	for (i = -offset; i < offset; i++)	{
		if (i < 0)
			m_rgchClipTbl [i] = 0;
		else if (i >= 0 && i <= maxVal)
			m_rgchClipTbl [i] = i;
		else 
			m_rgchClipTbl [i] = maxVal;
	}

	/*	m_c0 = 0.7071068F; //mwi
	m_c1 = 0.4903926F;
	m_c2 = 0.4619398F;
	m_c3 = 0.4157348F;
	m_c4 = 0.3535534F;
	m_c5 = 0.2777851F;
	m_c6 = 0.1913417F;
	m_c7 = 0.0975452F; */
	m_c0 = 0.7071068;
	m_c1 = 0.4903926;
	m_c2 = 0.4619398;
	m_c3 = 0.4157348;
	m_c4 = 0.3535534;
	m_c5 = 0.2777851;
	m_c6 = 0.1913417;
	m_c7 = 0.0975452; 



}

Void CBlockDCT::apply (const PixelC* rgchSrc, Int nColSrc, Int* rgiDst, Int nColDst)
{
	// transform here
	const PixelC* pchRowSrc = rgchSrc;
	for (CoordI iRow = 0; iRow < BLOCK_SIZE; iRow++)	{
		xformRow (pchRowSrc, iRow);
		pchRowSrc += nColSrc;
	}

	Int* piColDst = rgiDst;
	for (CoordI iCol = 0; iCol < BLOCK_SIZE; iCol++)	{
		xformColumn (piColDst, iCol, nColDst);
		piColDst++;
	}
}

Void CBlockDCT::apply (const Int* rgiSrc, Int nColSrc, PixelC* rgchDst, Int nColDst)
{
	// transform here
	const PixelI* piRowSrc = rgiSrc;
	for (CoordI iRow = 0; iRow < BLOCK_SIZE; iRow++)	{
		xformRow (piRowSrc, iRow);
		piRowSrc += nColSrc;
	}

	PixelC* pchColDst = rgchDst;
	for (CoordI iCol = 0; iCol < BLOCK_SIZE; iCol++)	{
		xformColumn (pchColDst, iCol, nColDst);
		pchColDst++;
	}
}

Void CBlockDCT::apply (const Int* rgiSrc, Int nColSrc, Int* rgiDst, Int nColDst)
{
	// transform here
	const PixelI* piRowSrc = rgiSrc;
	for (CoordI iRow = 0; iRow < BLOCK_SIZE; iRow++)	{
		xformRow (piRowSrc, iRow);
		piRowSrc += nColSrc;
	}

	Int* piColDst = rgiDst;
	for (CoordI iCol = 0; iCol < BLOCK_SIZE; iCol++)	{
		xformColumn (piColDst, iCol, nColDst);
		piColDst++;
	}
}

Void CBlockDCT::xformRow (const PixelC* ppxlcRowSrc, CoordI i)
{
	UInt j;
	for (j = 0; j < BLOCK_SIZE; j++) 
		m_rgfltBuf1 [j] = (Float) *ppxlcRowSrc++;

	oneDimensionalDCT ();

	for (j = 0; j < BLOCK_SIZE; j++) 
		m_rgfltAfterRowXform [i] [j] = m_rgfltAfter1dXform [j];
}

Void CBlockDCT::xformRow (const PixelI* ppxlfRowSrc, CoordI i)
{
	UInt j;
	for (j = 0; j < BLOCK_SIZE; j++) 
		m_rgfltBuf1 [j] = (Float) *ppxlfRowSrc++;

	oneDimensionalDCT ();

	for (j = 0; j < BLOCK_SIZE; j++) 
		m_rgfltAfterRowXform [i] [j] = m_rgfltAfter1dXform [j];
}

Void CBlockDCT::xformColumn (PixelC* ppxlcColDst, CoordI i, Int nColDst)
{
	UInt j;
	for (j = 0; j < BLOCK_SIZE; j++)	
		m_rgfltBuf1[j] = m_rgfltAfterRowXform[j][i];
	
	oneDimensionalDCT ();

	for (j = 0; j < BLOCK_SIZE; j++)	{ 
		Int iValue = 
				(m_rgfltAfter1dXform[j] >= 0) ? (PixelI) (m_rgfltAfter1dXform[j] + .5) :
											(PixelI) (m_rgfltAfter1dXform[j] - .5);
		*ppxlcColDst = m_rgchClipTbl [iValue];
		ppxlcColDst += nColDst;
	}
}

Void CBlockDCT::xformColumn (PixelI* ppxliColDst, CoordI i, Int nColDst)
{
	UInt j;
	for (j = 0; j < BLOCK_SIZE; j++)	
		m_rgfltBuf1[j] = m_rgfltAfterRowXform[j][i];

	oneDimensionalDCT ();

	for (j = 0; j < BLOCK_SIZE; j++)	{ 
		PixelI vl = 
			(m_rgfltAfter1dXform[j] >= 0) ? (PixelI) (m_rgfltAfter1dXform[j] + .5) :
											(PixelI) (m_rgfltAfter1dXform[j] - .5);
		*ppxliColDst = vl;
		ppxliColDst += nColDst;
	}
}



/* NBIT: change
CFwdBlockDCT::CFwdBlockDCT () : CBlockDCT ()
*/
CFwdBlockDCT::CFwdBlockDCT (UInt nBits) : CBlockDCT (nBits)
{
// by DSCHO on 2000/01/25 for Forward DCT ////////////////////////////////////////////////////////////////
#ifdef	_DSCHO_20000211_
	m_iLog_scale = 10;
	m_iLog_scale_1 = 11;
	m_iLog_scale_2 = 12;
	m_iLog_scale__1 = 9;
	m_iLog_scale__2 = 8;

	m_iLog_scale_22 = 20;
	m_iLog_scale_22_1 = 21;
	m_iLog_scale_22_2 = 22;
	m_iLog_scale_22__1 = 19;
	m_iLog_scale_22__2 = 18;

	m_iF0_no_sqrt = 0x2d4;
	m_iF0 = 0x400;
	m_iF1 = 0x2c6;
	m_iF2 = 0x29d;
	m_iF3 = 0x25a;
	m_iF4 = 0x200;
	m_iF5 = 0x192;
	m_iF6 = 0x115;
	m_iF7 = 0x8d;

#endif
// ~by DSCHO on 2000/01/25 for Forward DCT ////////////////////////////////////////////////////////////////
}

Void CFwdBlockDCT::oneDimensionalDCT ()
{
	Int j, j1;
	for (j = 0; j < 4; j++) {
		j1 = 7 - j;
		m_rgfltBuf2[j] = m_rgfltBuf1[j] + m_rgfltBuf1[j1];
		m_rgfltBuf2[j1] = m_rgfltBuf1[j] - m_rgfltBuf1[j1];
	}

	m_rgfltBuf1[0] = m_rgfltBuf2[0] + m_rgfltBuf2[3];
	m_rgfltBuf1[1] = m_rgfltBuf2[1] + m_rgfltBuf2[2];
	m_rgfltBuf1[2] = m_rgfltBuf2[1] - m_rgfltBuf2[2];
	m_rgfltBuf1[3] = m_rgfltBuf2[0] - m_rgfltBuf2[3];
	m_rgfltBuf1[4] = m_rgfltBuf2[4];
	m_rgfltBuf1[5] = (m_rgfltBuf2[6] - m_rgfltBuf2[5]) * m_c0;
	m_rgfltBuf1[6] = (m_rgfltBuf2[6] + m_rgfltBuf2[5]) * m_c0;
	m_rgfltBuf1[7] = m_rgfltBuf2[7];
	m_rgfltAfter1dXform[0] = (m_rgfltBuf1[0] + m_rgfltBuf1[1]) * m_c4;
	m_rgfltAfter1dXform[4] = (m_rgfltBuf1[0] - m_rgfltBuf1[1]) * m_c4;
	m_rgfltAfter1dXform[2] = m_rgfltBuf1[2] * m_c6 + m_rgfltBuf1[3] * m_c2;
	m_rgfltAfter1dXform[6] = m_rgfltBuf1[3] * m_c6 - m_rgfltBuf1[2] * m_c2;

	m_rgfltBuf2[4] = m_rgfltBuf1[4] + m_rgfltBuf1[5];
	m_rgfltBuf2[7] = m_rgfltBuf1[7] + m_rgfltBuf1[6];
		m_rgfltBuf2[5] = m_rgfltBuf1[4] - m_rgfltBuf1[5];
	m_rgfltBuf2[6] = m_rgfltBuf1[7] - m_rgfltBuf1[6];
	m_rgfltAfter1dXform[1] = m_rgfltBuf2[4] * m_c7 + m_rgfltBuf2[7] * m_c1;
	m_rgfltAfter1dXform[5] = m_rgfltBuf2[5] * m_c3 + m_rgfltBuf2[6] * m_c5;
	m_rgfltAfter1dXform[7] = m_rgfltBuf2[7] * m_c7 - m_rgfltBuf2[4] * m_c1;
	m_rgfltAfter1dXform[3] = m_rgfltBuf2[6] * m_c3 - m_rgfltBuf2[5] * m_c5;
}

// by DSCHO on 2000/01/25 for IDCT ////////////////////////////////////////////////////////////////
#ifdef	_DSCHO_20000211_
CInvBlockDCT::~CInvBlockDCT ()
{
	assert(m_nBits==8);
	m_rgnClipTbl -= 512;
	delete [] m_rgnClipTbl;
}
#endif
// ~by DSCHO on 2000/01/25 for IDCT ////////////////////////////////////////////////////////////////

/* NBIT: change
CInvBlockDCT::CInvBlockDCT () : CBlockDCT ()
*/
CInvBlockDCT::CInvBlockDCT (UInt nBits) : CBlockDCT (nBits)
{
// by DSCHO on 2000/01/19 for IDCT ////////////////////////////////////////////////////////////////
#ifdef	_DSCHO_20000211_
	assert(m_nBits==8);

	m_rgnClipTbl = new Short [1024];
	m_rgnClipTbl = m_rgnClipTbl+512;
	Int i;
	for (i= -512; i<512; i++)
		m_rgnClipTbl[i] = (i<-256) ? -256 : ((i>255) ? 255 : i);	


	m_iW1 = 2841; /* 2048*sqrt(2)*cos(1*pi/16) */
	m_iW2 = 2676; /* 2048*sqrt(2)*cos(2*pi/16) */
	m_iW3 = 2408; /* 2048*sqrt(2)*cos(3*pi/16) */
	m_iW5 = 1609; /* 2048*sqrt(2)*cos(5*pi/16) */
	m_iW6 = 1108; /* 2048*sqrt(2)*cos(6*pi/16) */
	m_iW7 = 565;  /* 2048*sqrt(2)*cos(7*pi/16) */
#endif
// ~by DSCHO on 2000/01/19 for IDCT ////////////////////////////////////////////////////////////////

}

Void CInvBlockDCT::oneDimensionalDCT ()
{
  /*	Float flt1 = m_rgfltBuf1[1] * m_c7 - m_rgfltBuf1[7] * m_c1; // Double, because m_rgfltBuf1,2 are doubles, mwi
	Float flt2 = m_rgfltBuf1[7] * m_c7 + m_rgfltBuf1[1] * m_c1;
	Float flt3 = m_rgfltBuf1[5] * m_c3 - m_rgfltBuf1[3] * m_c5;
	Float flt4 = m_rgfltBuf1[3] * m_c3 + m_rgfltBuf1[5] * m_c5; */
	Double flt1 = m_rgfltBuf1[1] * m_c7 - m_rgfltBuf1[7] * m_c1;
	Double flt2 = m_rgfltBuf1[7] * m_c7 + m_rgfltBuf1[1] * m_c1;
	Double flt3 = m_rgfltBuf1[5] * m_c3 - m_rgfltBuf1[3] * m_c5;
	Double flt4 = m_rgfltBuf1[3] * m_c3 + m_rgfltBuf1[5] * m_c5;

	m_rgfltBuf2[0] = (m_rgfltBuf1[0] + m_rgfltBuf1[4]) * m_c4;
	m_rgfltBuf2[1] = (m_rgfltBuf1[0] - m_rgfltBuf1[4]) * m_c4;
	m_rgfltBuf2[2] = m_rgfltBuf1[2] * m_c6 - m_rgfltBuf1[6] * m_c2;
	m_rgfltBuf2[3] = m_rgfltBuf1[6] * m_c6 + m_rgfltBuf1[2] * m_c2;
	m_rgfltBuf1[4] = flt1 + flt3;
	m_rgfltBuf2[5] = flt1 - flt3;
	m_rgfltBuf2[6] = flt2 - flt4;
	m_rgfltBuf1[7] = flt2 + flt4;
    
	m_rgfltBuf1[5] = (m_rgfltBuf2[6] - m_rgfltBuf2[5]) * m_c0;
	m_rgfltBuf1[6] = (m_rgfltBuf2[6] + m_rgfltBuf2[5]) * m_c0;
	m_rgfltBuf1[0] = m_rgfltBuf2[0] + m_rgfltBuf2[3];
	m_rgfltBuf1[1] = m_rgfltBuf2[1] + m_rgfltBuf2[2];
	m_rgfltBuf1[2] = m_rgfltBuf2[1] - m_rgfltBuf2[2];
	m_rgfltBuf1[3] = m_rgfltBuf2[0] - m_rgfltBuf2[3];

	Int j, j1;
	for (j = 0; j < 4; j++) {
		j1 = 7 - j;
		m_rgfltAfter1dXform[j] = m_rgfltBuf1[j] + m_rgfltBuf1[j1];
		m_rgfltAfter1dXform[j1] = m_rgfltBuf1[j] - m_rgfltBuf1[j1];
	}
}

// by DSCHO on 2000/01/24 for Forward DCT //////////////////////////////////////////////////////////////

#ifdef	_DSCHO_20000211_
Void CFwdBlockDCT::applyForwardDCT (const PixelC* rgchSrc, Int nColSrc, Int* rgiDst, Int nColDst)
{
	// transform here
	CoordI	iCol, iRow;
	const PixelC* pchRowSrc = rgchSrc;
	Int iIndex = 0;
	for (iCol = 0; iCol < BLOCK_SIZE; iCol++, pchRowSrc += nColSrc)	{
		for (iRow = 0; iRow < BLOCK_SIZE; iRow++)	{
			m_rgiFltBuf[iIndex] = (Int) pchRowSrc[iRow];
			iIndex++;			
		}
	}

	forwardDCT (m_rgiFltBuf);

	Int* piColDst = rgiDst;
	iIndex = 0;
	for (iCol = 0; iCol < BLOCK_SIZE; iCol++, piColDst += nColDst)	{
		for (iRow = 0; iRow < BLOCK_SIZE; iRow++)	{
			piColDst[iRow] = m_rgiFltBuf[iIndex];
			iIndex++;
		}
	}
}

Void CFwdBlockDCT::applyForwardDCT (const Int* rgiSrc, Int nColSrc, PixelC* rgchDst, Int nColDst)
{
	// transform here
	CoordI	iCol, iRow;
	Int iIndex = 0;
	const PixelI* piRowSrc = rgiSrc;
	for (iCol = 0; iCol < BLOCK_SIZE; iCol++, piRowSrc += nColSrc)	{
		for (iRow = 0; iRow < BLOCK_SIZE; iRow++)	{
			m_rgiFltBuf[iIndex] = (Int) piRowSrc[iRow];
			iIndex++;
		}
	}

	forwardDCT (m_rgiFltBuf);

	PixelC* pchColDst = rgchDst;
	iIndex = 0;
	for (iCol = 0; iCol < BLOCK_SIZE; iCol++, pchColDst += nColDst)	{
		for (iRow = 0; iRow < BLOCK_SIZE; iRow++)	{
			Int iValue = m_rgiFltBuf[iIndex];	// modified on 2000/01/24
			pchColDst[iRow] = m_rgchClipTbl[iValue];	// modified on 2000/01/24
			iIndex ++;
		}
	}
}

Void CFwdBlockDCT::applyForwardDCT (const Int* rgiSrc, Int nColSrc, Int* rgiDst, Int nColDst)
{
	// transform here
	CoordI iCol, iRow;
	Int iIndex = 0;
	const PixelI* piRowSrc = rgiSrc;
	for (iCol = 0; iCol < BLOCK_SIZE; iCol++, piRowSrc += nColSrc)	{
		for (iRow = 0; iRow < BLOCK_SIZE; iRow++)	{
			m_rgiFltBuf[iIndex] = (Int) piRowSrc[iRow];
			iIndex++;
		}
	}

	forwardDCT (m_rgiFltBuf);

	Int* piColDst = rgiDst;
	iIndex = 0;
	for (iCol = 0; iCol < BLOCK_SIZE; iCol++, piColDst += nColDst)	{
		for (iRow = 0; iRow < BLOCK_SIZE; iRow++)	{
			piColDst[iRow] = m_rgiFltBuf[iIndex];
			iIndex++;
		}
	}
}

//////////////////////////////////////////////////////////////////////////////
//
// -- forwardDCT -- integer foward DCT (8x8)
//
// Author :		
//	Dae-Sung Cho (dscho@sait.samsung.co.kr)	
//
// Created :		
//	2000-01-25
//
// Purpose :		
//  a fast, not so accurate integer implementation of the forward DCT (Discrete Cosine Transform).
// 
// Arguments in : 	
//  none
//
// Arguments in/out :	
//	Int			*data	: input block data and output coefficients
//
// Arguments out :	
//	none
//
// Return values :	
//	none
//
// Side effects :	
//	none
//
// Description :	
// Does dct on an 8x8 block, does zigzag-scanning of
//        coefficients
//
// See also :
//
// Modified :		
//
//////////////////////////////////////////////////////////////////////////////

Void CFwdBlockDCT::forwardDCT (Int *block)
{
   forwardDCTRow(block);
   forwardDCTRow(block + 8);
   forwardDCTRow(block + 16);
   forwardDCTRow(block + 24);
   forwardDCTRow(block + 32);
   forwardDCTRow(block + 40);
   forwardDCTRow(block + 48);
   forwardDCTRow(block + 56);

   forwardDCTCol(block);
   forwardDCTCol(block + 1);
   forwardDCTCol(block + 2);
   forwardDCTCol(block + 3);
   forwardDCTCol(block + 4);
   forwardDCTCol(block + 5);
   forwardDCTCol(block + 6);
   forwardDCTCol(block + 7);
}

Void CFwdBlockDCT::forwardDCTRow (Int *d)
{
    Int temp;
    Int b0, b1, b2, b3, b4, b5, b6, b7;
    Int b10, b11, b12, b13, b14, b15, b16, b17;
	
    b10 = d[0] + d[7];
    b11 = d[1] + d[6];
    b12 = d[2] + d[5];
    b13 = d[3] + d[4];

    b16 = d[1] - d[6];
    b15 = d[2] - d[5];

    b4 = (d[3] - d[4]);
    b7 = (d[0] - d[7]);

    b0 = b10 + b13;
    b3 = b10 - b13;

    b1 = b11 + b12;
    b2 = b11 - b12;

    b5 = (b16 - b15)*m_iF0_no_sqrt >> m_iLog_scale;
    b6 = (b16 + b15)*m_iF0_no_sqrt >> m_iLog_scale;

    d[0] = (b0 + b1) << m_iLog_scale__1;
    d[4] = (b0 - b1) << m_iLog_scale__1;

    temp =  m_iF6*(b2 + b3);
    d[2] = (temp + (m_iF2 - m_iF6)* b3);
    d[6] = (temp - (m_iF2 + m_iF6)* b2);

    b14 = b4 + b5;
    b15 = b4 - b5;

    b17 = b7 + b6;
    b16 = b7 - b6;

    temp = m_iF7 * ( b14 + b17 );
    d[1] = (temp + (m_iF1 - m_iF7 )*b17 );
    d[7] = (temp - (m_iF1 + m_iF7 )*b14 );

    temp = m_iF3*( b15 + b16 );
    d[5] = (temp + ( m_iF5 - m_iF3 )*b16 );
    d[3] = (temp - ( m_iF5 + m_iF3 )*b15 );
}

Void CFwdBlockDCT::forwardDCTCol (Int *d)
{
    Int temp;
    Int b0, b1, b2, b3, b4, b5, b6, b7;
    Int b10, b11, b12, b13, b14, b15, b16, b17;

    b10 = d[0<<3] + d[7<<3]; 
    b11 = d[1<<3] + d[6<<3];
    b12 = d[2<<3] + d[5<<3];
    b13 = d[3<<3] + d[4<<3];

    b16 = d[1<<3] - d[6<<3];
    b15 = d[2<<3] - d[5<<3];

    b4 = (d[3<<3] - d[4<<3]);
    b7 = (d[0<<3] - d[7<<3]);

    b0 = b10 + b13;
    b3 = b10 - b13;

    b1 = b11 + b12;
    b2 = b11 - b12;

    b5 = (b16 - b15)*m_iF0_no_sqrt >> m_iLog_scale;
    b6 = (b16 + b15)*m_iF0_no_sqrt >> m_iLog_scale;

    d[8*0] = (b0 + b1) >> m_iLog_scale_2;
    d[8*4] = (b0 - b1) >> m_iLog_scale_2;

    temp = m_iF6*(b2 + b3);
    d[8*2] = (temp + (m_iF2 - m_iF6)*b3) >> m_iLog_scale_22_1;
    d[8*6] = (temp - (m_iF2 + m_iF6)*b2) >> m_iLog_scale_22_1;

    b14 = b4 + b5;
    b15 = b4 - b5;

    b17 = b7 + b6;
    b16 = b7 - b6;

    temp = m_iF7 * ( b14 + b17 );
    d[8*1] = (temp + ( m_iF1 - m_iF7 )* b17) >> m_iLog_scale_22_1;
    d[8*7] = (temp - ( m_iF1 + m_iF7 )* b14) >> m_iLog_scale_22_1;

    temp = m_iF3 * ( b15 + b16 );
    d[8*5] = (temp + ( m_iF5 - m_iF3 ) * b16) >> m_iLog_scale_22_1;
    d[8*3] = (temp - ( m_iF5 + m_iF3 ) * b15) >> m_iLog_scale_22_1;
}
// ~by DSCHO on 2000/01/24 for Forward DCT //////////////////////////////////////////////////////////////

// by DSCHO on 2000/02/10 for Forward DCT ////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//
// -- applySelectiveForwardDCT -- integer foward DCT (8x8)
//
// Author :		
//	Dae-Sung Cho (dscho@sait.samsung.co.kr)	
//
// Created :		
//	2000-02-10
//
// Purpose :		
//  a fast, not so accurate integer implementation of the forward DCT (Discrete Cosine Transform).
// 
// Arguments in : 	
//  none
//
// Arguments in/out :	
//	Int			*data	: input block data and output coefficients
//
// Arguments out :	
//	none
//
// Return values :	
//	none
//
// Side effects :	
//	none
//
// Description :	
// Does dct on an 8x8 block, does zigzag-scanning of
//        coefficients
//
// See also :
//
// Modified :		
//
//////////////////////////////////////////////////////////////////////////////
Bool CFwdBlockDCT::applySelectiveForwardDCT (const Int* rgiSrc, Int nColSrc, Int* rgiDst, Int nColDst, Quantizer fQuantizer, Int iQP, Bool bDefaultQuantMatrix)
{
	// transform here
	CoordI iCol, iRow;
	Int iSAE = 0;
	Int iIndex = 0;
	Bool bDctSkipped = FALSE;

	const PixelI* piRowSrc = rgiSrc;
	for (iCol = 0; iCol < BLOCK_SIZE; iCol++, piRowSrc += nColSrc)	{
		for (iRow = 0; iRow < BLOCK_SIZE; iRow++)	{
			m_rgiFltBuf[iIndex] = (Int) piRowSrc[iRow];
			iSAE += ABS(m_rgiFltBuf[iIndex]);
			iIndex++;
		}
	}

	if (fQuantizer == Q_H263) {
		if (iSAE < 17 * iQP) {
			bDctSkipped = TRUE;
			return bDctSkipped;
		}
	    if (iSAE < 22 * iQP) forwardDCT2 (m_rgiFltBuf);
	    else if (iSAE < 25 * iQP) forwardDCT3 (m_rgiFltBuf);
	    else if (iSAE < 27 * iQP) forwardDCT4 (m_rgiFltBuf);
	    else if (iSAE < 32 * iQP) forwardDCT5 (m_rgiFltBuf);
	    else if (iSAE < 37 * iQP) forwardDCT6 (m_rgiFltBuf);
	    else if (iSAE < 45 * iQP) forwardDCT7 (m_rgiFltBuf);
	    else forwardDCT (m_rgiFltBuf);
	} else {
		if (bDefaultQuantMatrix==FALSE) {
			if (iSAE < 14 * iQP) {
				bDctSkipped = TRUE;
				return bDctSkipped;
			}
			if (iSAE < 20 * iQP) forwardDCT2 (m_rgiFltBuf);
			else if (iSAE < 23 * iQP) forwardDCT3 (m_rgiFltBuf);
			else if (iSAE < 27 * iQP) forwardDCT4 (m_rgiFltBuf);
			else if (iSAE < 34 * iQP) forwardDCT5 (m_rgiFltBuf);
			else if (iSAE < 41 * iQP) forwardDCT6 (m_rgiFltBuf);
			else if (iSAE < 51 * iQP) forwardDCT7 (m_rgiFltBuf);
			else forwardDCT (m_rgiFltBuf);
		} else {
			forwardDCT (m_rgiFltBuf);
		}
	}

	Int* piColDst = rgiDst;
	iIndex = 0;
	for (iCol = 0; iCol < BLOCK_SIZE; iCol++, piColDst += nColDst)	{
		for (iRow = 0; iRow < BLOCK_SIZE; iRow++)	{
			piColDst[iRow] = m_rgiFltBuf[iIndex];
			iIndex++;
		}
	}
	return bDctSkipped;
}

// 2 * 2 DCT calling module 
// by DSCHO_20000209
Void CFwdBlockDCT::forwardDCT2(Int *block)
{
// by DSCHO_20000209
//   forwardDCTRow2(block);
//   forwardDCTRow2(block + 16);
//   forwardDCTRow2(block + 32);
//   forwardDCTRow2(block + 48);
   forwardDCTRow2(block);
   forwardDCTRow2(block + 8);
   forwardDCTRow2(block + 16);
   forwardDCTRow2(block + 24);
   forwardDCTRow2(block + 32);
   forwardDCTRow2(block + 40);
   forwardDCTRow2(block + 48);
   forwardDCTRow2(block + 56);
// ~by DSCHO_20000209

   forwardDCTCol2(block);
// by DSCHO_20000209
//   forwardDCTCol2(block + 1);
   forwardDCTCol1(block + 1);
// ~by DSCHO_20000209
}
// ~by DSCHO_20000209

// DCT module having xform values only in DC coeff & 1st,2nd AC zigzag bands
Void CFwdBlockDCT::forwardDCT3(Int *block)
{
// by DSCHO_20000209
//   forwardDCTRow3(block);
//   forwardDCTRow3(block + 16);
//   forwardDCTRow3(block + 32);
//   forwardDCTRow3(block + 48);
   forwardDCTRow3(block);
   forwardDCTRow3(block + 8);
   forwardDCTRow3(block + 16);
   forwardDCTRow3(block + 24);
   forwardDCTRow3(block + 32);
   forwardDCTRow3(block + 40);
   forwardDCTRow3(block + 48);
   forwardDCTRow3(block + 56);
// ~by DSCHO_20000209

   forwardDCTCol3(block);
// by DSCHO_20000209
   //forwardDCTCol3(block + 1);
   //forwardDCTCol3(block + 2);
   forwardDCTCol2(block + 1);
   forwardDCTCol1(block + 2);
// ~by DSCHO_20000209
}

// DCT module having xform values only in DC coeff & 1st,2nd,3rd AC zigzag bands
Void CFwdBlockDCT::forwardDCT4(Int *block)
{
   forwardDCTRow4(block);
   forwardDCTRow4(block + 8);
   forwardDCTRow4(block + 16);
   forwardDCTRow4(block + 24);
   forwardDCTRow4(block + 32);
   forwardDCTRow4(block + 40);
   forwardDCTRow4(block + 48);
   forwardDCTRow4(block + 56);

   forwardDCTCol4(block);
// by DSCHO_20000209
   //forwardDCTCol4(block + 1);
   //forwardDCTCol4(block + 2);
   //forwardDCTCol4(block + 3);
   forwardDCTCol3(block + 1);
   forwardDCTCol2(block + 2);
   forwardDCTCol1(block + 3);
// ~by DSCHO_20000209
}

// DCT module having xform values only in DC coeff & 1st,2nd,3rd,4th AC zigzag bands
Void CFwdBlockDCT::forwardDCT5(Int *block)
{
   forwardDCTRow5(block);
   forwardDCTRow5(block + 8);
   forwardDCTRow5(block + 16);
   forwardDCTRow5(block + 24);
   forwardDCTRow5(block + 32);
   forwardDCTRow5(block + 40);
   forwardDCTRow5(block + 48);
   forwardDCTRow5(block + 56);

   forwardDCTCol5(block);
// by DSCHO_20000209
   //forwardDCTCol5(block + 1);
   //forwardDCTCol5(block + 2);
   //forwardDCTCol5(block + 3);
   //forwardDCTCol5(block + 4);
   forwardDCTCol4(block + 1);
   forwardDCTCol3(block + 2);
   forwardDCTCol2(block + 3);
   forwardDCTCol1(block + 4);
// ~by DSCHO_20000209
}

// DCT module having xform values only in DC coeff & 1st,2nd,3rd,4th,5th AC zigzag bands
Void CFwdBlockDCT::forwardDCT6(Int *block)
{
   forwardDCTRow6(block);
   forwardDCTRow6(block + 8);
   forwardDCTRow6(block + 16);
   forwardDCTRow6(block + 24);
   forwardDCTRow6(block + 32);
   forwardDCTRow6(block + 40);
   forwardDCTRow6(block + 48);
   forwardDCTRow6(block + 56);

   forwardDCTCol6(block);
// by DSCHO_20000209
   //forwardDCTCol6(block + 1);
   //forwardDCTCol6(block + 2);
   //forwardDCTCol6(block + 3);
   //forwardDCTCol6(block + 4);
   //forwardDCTCol6(block + 5);
   forwardDCTCol5(block + 1);
   forwardDCTCol4(block + 2);
   forwardDCTCol3(block + 3);
   forwardDCTCol2(block + 4);
   forwardDCTCol1(block + 5);
// ~by DSCHO_20000209
}

// DCT module having xform values only in DC coeff & 1st,2nd,3rd,4th,5th AC zigzag bands
Void CFwdBlockDCT::forwardDCT7(Int *block)
{
   forwardDCTRow7(block);
   forwardDCTRow7(block + 8);
   forwardDCTRow7(block + 16);
   forwardDCTRow7(block + 24);
   forwardDCTRow7(block + 32);
   forwardDCTRow7(block + 40);
   forwardDCTRow7(block + 48);
   forwardDCTRow7(block + 56);

   forwardDCTCol7(block);
// by DSCHO_20000209
   //forwardDCTCol7(block + 1);
   //forwardDCTCol7(block + 2);
   //forwardDCTCol7(block + 3);
   //forwardDCTCol7(block + 4);
   //forwardDCTCol7(block + 5);
   //forwardDCTCol7(block + 6);
   forwardDCTCol6(block + 1);
   forwardDCTCol5(block + 2);
   forwardDCTCol4(block + 3);
   forwardDCTCol3(block + 4);
   forwardDCTCol2(block + 5);
   forwardDCTCol1(block + 6);
// ~by DSCHO_20000209
}
// -------------  1 * 1 DCT --------------------
Void CFwdBlockDCT::forwardDCTRow1(Int *d)
{
    d[0] = (d[0] + d[1] + d[2] + d[3] + d[4] + d[5] + d[6]) << m_iLog_scale__1;
	d[1] = d[2] = d[3] = d[4] = d[5] = d[6] = d[7] = 0;
}

Void CFwdBlockDCT::forwardDCTCol1(Int *d)
{
    //d[8*0] = (d[8*0]+d[8*1]+d[8*2]+d[8*3]+d[8*4]+d[8*5]+d[8*6]+d[8*7]) >> m_iLog_scale_2;
    //d[8*1] = d[8*2] = d[8*3] = d[8*4] = d[8*5] = d[8*6] = d[8*7] = 0;
	d[0] = (d[0] + d[8] + d[16] + d[24] + d[32] + d[40] + d[48] + d[56]) >> m_iLog_scale_2;
	d[8] = d[16] = d[24] = d[32] = d[40] = d[48] = d[56] = 0;
}

// -------------  2 * 2 DCT --------------------
Void CFwdBlockDCT::forwardDCTRow2(Int *d)
{
// by DSCHO_20000209
/*
    Int b0, b1, b2, b3, b4, b5, b6, b7;

    b0 = d[0] + d[1];
    b1 = d[2] + d[3];
    b2 = d[4] + d[5];
    b3 = d[6] + d[7];

    b4 = b0 + b3;
    b5 = b1 + b2;
    b6 = b1 - b2;
    b7 = b0 - b3;

    d[0] = (b4 + b5);
    d[1] = ( b6*m_iF6 + m_iF2*b7 ) >> m_iLog_scale__1;

    d += 8;
    
    b0 = d[0] + d[1];
    b1 = d[2] + d[3];
    b2 = d[4] + d[5];
    b3 = d[6] + d[7];

    b4 = b0 + b3;
    b5 = b1 + b2;
    b6 = b1 - b2;
    b7 = b0 - b3;

    d[0] = (b4 + b5);
    d[1] = ( b6*m_iF6 + m_iF2*b7 ) >> m_iLog_scale__1;
*/
//    Int b0, b1, b2, b3, b4, b5, b6, b7;
//    Int b10, b11, b12, b13, b14, b15, b16, b17;
    Int b0, b1, b4, b5, b6, b7;
    Int b10, b11, b12, b13, b15, b16;
	
    b10 = d[0] + d[7]; 
    b11 = d[1] + d[6];
    b12 = d[2] + d[5];
    b13 = d[3] + d[4];

    b16 = d[1] - d[6];
    b15 = d[2] - d[5];

    b4 = (d[3] - d[4]);
    b7 = (d[0] - d[7]);

    b0 = b10 + b13;
    b1 = b11 + b12;
//    b2 = b11 - b12;
//    b3 = b10 - b13;

    b5 = (b16 - b15)*m_iF0_no_sqrt >> m_iLog_scale;
    b6 = (b16 + b15)*m_iF0_no_sqrt >> m_iLog_scale;

	d[0] = (b0 + b1) << m_iLog_scale__1;
    
//    b14 = b4 + b5;
//    b17 = b7 + b6;
//    b15 = b4 - b5;
//    b16 = b7 - b6;

    d[1] = m_iF7*(b4+b5) + m_iF1*(b7+b6); // d[1] = m_iF7*b14 + m_iF1*b17;

	d[2] = d[3] = d[4] = d[5] = d[6] = d[7] = 0;
// ~by DSCHO_20000209
}

Void CFwdBlockDCT::forwardDCTCol2(Int *d)
{
// by DSCHO_20000209
/*
    Int b0, b1, b2, b3, b4, b5, b6, b7;

    b0 = d[0<<3] + d[1<<3];
    b1 = d[2<<3] + d[3<<3];
    b2 = d[4<<3] + d[5<<3];
    b3 = d[6<<3] + d[7<<3];

    b4 = b0 + b3;
    b5 = b1 + b2;

    b7 = b0 - b3;
    b6 = b1 - b2;

    d[0<<3] = (b4 + b5) >> 3;
    d[1<<3] = ( b6* m_iF6 + m_iF2*b7) >> m_iLog_scale_2;

    d[2<<3] = d[3<<3] = d[4<<3] = d[5<<3] = d[6<<3] = d[7<<3] = 0;
*/
//    Int b0, b1, b2, b3, b4, b5, b6, b7;
//    Int b10, b11, b12, b13, b14, b15, b16, b17;
    Int b0, b1, b4, b5, b6, b7;
    Int b10, b11, b12, b13, b15, b16;

    b10 = d[0] + d[56]; //b10 = d[0<<3] + d[7<<3]; 
    b11 = d[8] + d[48]; //b11 = d[1<<3] + d[6<<3];
    b12 = d[16] + d[40]; //b12 = d[2<<3] + d[5<<3];
    b13 = d[24] + d[32]; //b13 = d[3<<3] + d[4<<3];

    b16 = d[8] - d[48]; //b16 = d[1<<3] - d[6<<3];
    b15 = d[16] - d[40]; //b15 = d[2<<3] - d[5<<3];

    b4 = d[24] - d[32]; //b4 = (d[3<<3] - d[4<<3]);
    b7 = d[0] - d[56]; //b7 = (d[0<<3] - d[7<<3]);

    b0 = b10 + b13;
    b1 = b11 + b12;
//    b2 = b11 - b12;
//    b3 = b10 - b13;

    b5 = (b16 - b15)*m_iF0_no_sqrt >> m_iLog_scale;
    b6 = (b16 + b15)*m_iF0_no_sqrt >> m_iLog_scale;

    d[0] = (b0 + b1) >> m_iLog_scale_2; // for d[8*0]

//    b14 = b4 + b5;
//    b17 = b7 + b6;
//    b15 = b4 - b5;
//    b16 = b7 - b6;

    d[8] = (m_iF7*(b4+b5) + m_iF1*(b7+b6)) >> m_iLog_scale_22_1; // d[8] = (m_iF7*b14 + m_iF1*b17) >> m_iLog_scale_22_1; // for d[8*1]

	d[16] = 0; // for d[8*2]
	d[24] = 0; // for d[8*3]
	d[32] = 0; // for d[8*4]
    d[40] = 0; // for d[8*5]
    d[48] = 0; // for d[8*6]
    d[56] = 0; // for d[8*7]
// ~by DSCHO_20000209
}

// -------------  3 * 3 DCT --------------------
Void CFwdBlockDCT::forwardDCTRow3(Int *d)
{
// by DSCHO_20000209
/*
    Int b0, b1, b2, b3, b4, b5, b6, b7;

    b0 = d[0] + d[1];
    b1 = d[2] + d[3];
    b2 = d[4] + d[5];
    b3 = d[6] + d[7];

    b4 = b0 + b3;
    b5 = b1 + b2;
    b6 = b1 - b2;
    b7 = b0 - b3;

    d[0] = (b4 + b5);
    d[2] = (b4 - b5);
    d[1] = ( b6*m_iF6 + m_iF2*b7 ) >> m_iLog_scale__1;

    d += 8;
    
    b0 = d[0] + d[1];
    b1 = d[2] + d[3];
    b2 = d[4] + d[5];
    b3 = d[6] + d[7];

    b4 = b0 + b3;
    b5 = b1 + b2;
    b6 = b1 - b2;
    b7 = b0 - b3;

    d[0] = (b4 + b5);
    d[2] = (b4 - b5);
    d[1] = ( b6*m_iF6 + m_iF2*b7 ) >> m_iLog_scale__1;
*/
    Int b0, b1, b2, b3, b4, b5, b6, b7;
//    Int b10, b11, b12, b13, b14, b15, b16, b17;
    Int b10, b11, b12, b13, b15, b16;
	
    b10 = d[0] + d[7]; 
    b11 = d[1] + d[6];
    b12 = d[2] + d[5];
    b13 = d[3] + d[4];

    b16 = d[1] - d[6];
    b15 = d[2] - d[5];

    b4 = (d[3] - d[4]);
    b7 = (d[0] - d[7]);

    b0 = b10 + b13;
    b3 = b10 - b13;
    b1 = b11 + b12;
    b2 = b11 - b12;

    b5 = (b16 - b15)*m_iF0_no_sqrt >> m_iLog_scale;
    b6 = (b16 + b15)*m_iF0_no_sqrt >> m_iLog_scale;

	d[0] = (b0 + b1) << m_iLog_scale__1;
    d[2] = m_iF6*b2 + m_iF2*b3;
    
//    b14 = b4 + b5;
//    b17 = b7 + b6;
//    b15 = b4 - b5;
//    b16 = b7 - b6;
    d[1] = m_iF7*(b4+b5) + m_iF1*(b7+b6); //d[1] = m_iF7*b14 + m_iF1*b17;

	d[3] = d[4] = d[5] = d[6] = d[7] = 0;
// ~by DSCHO_20000209
}

Void CFwdBlockDCT::forwardDCTCol3(Int *d)
{
// by DSCHO_20000209
/*
    Int b0, b1, b2, b3, b4, b5, b6, b7;

    b0 = d[0<<3] + d[1<<3];
    b1 = d[2<<3] + d[3<<3];
    b2 = d[4<<3] + d[5<<3];
    b3 = d[6<<3] + d[7<<3];

    b4 = b0 + b3;
    b5 = b1 + b2;
    b6 = b1 - b2;
    b7 = b0 - b3;

    d[0<<3] = (b4 + b5) >> 3;
    d[2<<3] = (b4 - b5) >> 3;
    d[1<<3] = ( b6* m_iF6 + m_iF2*b7) >> m_iLog_scale_2;

    d[3<<3] = d[4<<3] = d[5<<3] = d[6<<3] = d[7<<3] = 0;
*/
    Int b0, b1, b2, b3, b4, b5, b6, b7;
//    Int b10, b11, b12, b13, b14, b15, b16, b17;
    Int b10, b11, b12, b13, b15, b16;

    b10 = d[0] + d[56]; //b10 = d[0<<3] + d[7<<3]; 
    b11 = d[8] + d[48]; //b11 = d[1<<3] + d[6<<3];
    b12 = d[16] + d[40]; //b12 = d[2<<3] + d[5<<3];
    b13 = d[24] + d[32]; //b13 = d[3<<3] + d[4<<3];

    b16 = d[8] - d[48]; //b16 = d[1<<3] - d[6<<3];
    b15 = d[16] - d[40]; //b15 = d[2<<3] - d[5<<3];

    b4 = d[24] - d[32]; //b4 = (d[3<<3] - d[4<<3]);
    b7 = d[0] - d[56]; //b7 = (d[0<<3] - d[7<<3]);

    b0 = b10 + b13;
    b3 = b10 - b13;

    b1 = b11 + b12;
    b2 = b11 - b12;

    b5 = (b16 - b15)*m_iF0_no_sqrt >> m_iLog_scale;
    b6 = (b16 + b15)*m_iF0_no_sqrt >> m_iLog_scale;

    d[0] = (b0 + b1) >> m_iLog_scale_2; // for d[8*0]
    d[16] = (m_iF6*b2 + m_iF2*b3) >> m_iLog_scale_22_1; // for d[8*2]

//    b14 = b4 + b5;
//    b17 = b7 + b6;
//    b15 = b4 - b5;
//    b16 = b7 - b6;
    d[8] = (m_iF7*(b4+b5) + m_iF1*(b7+b6)) >> m_iLog_scale_22_1; // d[8] = (m_iF7*b14 + m_iF1*b17) >> m_iLog_scale_22_1; // for d[8*1]

	d[24] = 0; // for d[8*3]
	d[32] = 0; // for d[8*4]
    d[40] = 0; // for d[8*5]
    d[48] = 0; // for d[8*6]
    d[56] = 0; // for d[8*7]
// ~by DSCHO_20000209
}

// -------------------- 4 * 4 DCT --------------------
Void CFwdBlockDCT::forwardDCTRow4(Int *d)
{
// by DSCHO_20000209
/*
    Int temp;
    Int b0, b1, b2, b3, b4, b5, b6, b7;

    b0 = d[0] + d[1];
    b1 = d[2] + d[3];
    b2 = d[4] + d[5];
    b3 = d[6] + d[7];

    b4 = b0 + b3;
    b5 = b1 + b2;
    b6 = b1 - b2;
    b7 = b0 - b3;

    d[0] = (b4 + b5) << m_iLog_scale__1;
    d[2] = (b4 - b5) << m_iLog_scale__1;

    temp = ( b6 + b7 ) * m_iF6 ;
    d[1] =  temp + (m_iF2 - m_iF6)*b7;
    d[3] =  temp - (m_iF2 + m_iF6)*b6;
*/
    Int b0, b1, b2, b3, b4, b5, b6, b7;
    Int b10, b11, b12, b13, b14, b15, b16, b17;
	
    b10 = d[0] + d[7]; 
    b11 = d[1] + d[6];
    b12 = d[2] + d[5];
    b13 = d[3] + d[4];

    b16 = d[1] - d[6];
    b15 = d[2] - d[5];

    b4 = (d[3] - d[4]);
    b7 = (d[0] - d[7]);

    b0 = b10 + b13;
    b3 = b10 - b13;
    b1 = b11 + b12;
    b2 = b11 - b12;

    b5 = (b16 - b15)*m_iF0_no_sqrt >> m_iLog_scale;
    b6 = (b16 + b15)*m_iF0_no_sqrt >> m_iLog_scale;

	d[0] = (b0 + b1) << m_iLog_scale__1;
    d[2] = m_iF6*b2 + m_iF2*b3;
    
    b14 = b4 + b5;
    b17 = b7 + b6;
    b15 = b4 - b5;
    b16 = b7 - b6;

    d[1] = m_iF7*b14 + m_iF1*b17;
	d[3] = m_iF3*b16 - m_iF5*b15;

	d[4] = d[5] = d[6] = d[7] = 0;
// ~by DSCHO_20000209
}

Void CFwdBlockDCT::forwardDCTCol4(Int *d)
{
// by DSCHO_20000209
/*
    Int temp;
    Int b0, b1, b2, b3, b4, b5, b6, b7;

    b0 = d[0<<3] + d[1<<3];
    b1 = d[2<<3] + d[3<<3];
    b2 = d[4<<3] + d[5<<3];
    b3 = d[6<<3] + d[7<<3];

    b4 = b0 + b3;
    b5 = b1 + b2;
    b6 = b1 - b2;
    b7 = b0 - b3;

    d[0<<3] = ((b4 + b5)) >> m_iLog_scale_2;
    d[2<<3] = ((b4 - b5)) >> m_iLog_scale_2;

    temp = ( b6 + b7 ) * m_iF6;
    d[1<<3] = ( temp + (m_iF2 - m_iF6)*b7 ) >> m_iLog_scale_22_1;
    d[3<<3] = ( temp - (m_iF2 + m_iF6)*b6 ) >> m_iLog_scale_22_1;
    
    d[4<<3] = d[5<<3] = d[6<<3] = d[7<<3] = 0;
*/
    Int b0, b1, b2, b3, b4, b5, b6, b7;
    Int b10, b11, b12, b13, b14, b15, b16, b17;

    b10 = d[0] + d[56]; //b10 = d[0<<3] + d[7<<3]; 
    b11 = d[8] + d[48]; //b11 = d[1<<3] + d[6<<3];
    b12 = d[16] + d[40]; //b12 = d[2<<3] + d[5<<3];
    b13 = d[24] + d[32]; //b13 = d[3<<3] + d[4<<3];

    b16 = d[8] - d[48]; //b16 = d[1<<3] - d[6<<3];
    b15 = d[16] - d[40]; //b15 = d[2<<3] - d[5<<3];

    b4 = d[24] - d[32]; //b4 = (d[3<<3] - d[4<<3]);
    b7 = d[0] - d[56]; //b7 = (d[0<<3] - d[7<<3]);

    b0 = b10 + b13;
    b3 = b10 - b13;

    b1 = b11 + b12;
    b2 = b11 - b12;

    b5 = (b16 - b15)*m_iF0_no_sqrt >> m_iLog_scale;
    b6 = (b16 + b15)*m_iF0_no_sqrt >> m_iLog_scale;

    d[0] = (b0 + b1) >> m_iLog_scale_2; // for d[8*0]
    d[16] = (m_iF6*b2 + m_iF2*b3) >> m_iLog_scale_22_1; // for d[8*2]

    b14 = b4 + b5;
    b17 = b7 + b6;
    b15 = b4 - b5;
    b16 = b7 - b6;

    d[8] = (m_iF7*b14 + m_iF1*b17) >> m_iLog_scale_22_1; // for d[8*1]
	d[24] = (m_iF3*b16 - m_iF5*b15) >> m_iLog_scale_22_1; // for d[8*3]

	d[32] = 0; // for d[8*4]
    d[40] = 0; // for d[8*5]
    d[48] = 0; // for d[8*6]
    d[56] = 0; // for d[8*7]
// ~by DSCHO_20000209
}

// -------------------- 5 * 5 DCT --------------------
Void CFwdBlockDCT::forwardDCTRow5(Int *d)
{
//    Int temp; // by DSCHO_20000209
    Int b0, b1, b2, b3, b4, b5, b6, b7;
    Int b10, b11, b12, b13, b14, b15, b16, b17;
	
    b10 = d[0] + d[7];
    b11 = d[1] + d[6];
    b12 = d[2] + d[5];
    b13 = d[3] + d[4];

    b16 = d[1] - d[6];
    b15 = d[2] - d[5];

    b4 = (d[3] - d[4]);
    b7 = (d[0] - d[7]);

    b0 = b10 + b13;
    b3 = b10 - b13;

    b1 = b11 + b12;
    b2 = b11 - b12;

    b5 = (b16 - b15)*m_iF0_no_sqrt >> m_iLog_scale;
    b6 = (b16 + b15)*m_iF0_no_sqrt >> m_iLog_scale;

    d[0] = (b0 + b1) << m_iLog_scale__1;
    d[4] = (b0 - b1) << m_iLog_scale__1;
    d[2] = (m_iF6*b2 + m_iF2*b3);

    b14 = b4 + b5;
    b17 = b7 + b6;
    b15 = b4 - b5;
    b16 = b7 - b6;

    d[1] = (m_iF7*b14 + m_iF1*b17);
// by DSCHO_20000209
    //d[3] = (b16 - b15) << m_iLog_scale__1; //  d[3] = m_iF3*(b16) - (m_iF5)*b15;
	d[3] = m_iF3*b16 - m_iF5*b15;
	d[5] = d[6] = d[7] = 0;
// by DSCHO_20000209
}

Void CFwdBlockDCT::forwardDCTCol5(Int *d)
{
//    Int temp;// by DSCHO_20000209
    Int b0, b1, b2, b3, b4, b5, b6, b7;
    Int b10, b11, b12, b13, b14, b15, b16, b17;

    b10 = d[0] + d[56]; //b10 = d[0<<3] + d[7<<3]; 
    b11 = d[8] + d[48]; //b11 = d[1<<3] + d[6<<3];
    b12 = d[16] + d[40]; //b12 = d[2<<3] + d[5<<3];
    b13 = d[24] + d[32]; //b13 = d[3<<3] + d[4<<3];

    b16 = d[8] - d[48]; //b16 = d[1<<3] - d[6<<3];
    b15 = d[16] - d[40]; //b15 = d[2<<3] - d[5<<3];

    b4 = d[24] - d[32]; //b4 = (d[3<<3] - d[4<<3]);
    b7 = d[0] - d[56]; //b7 = (d[0<<3] - d[7<<3]);

    b0 = b10 + b13;
    b3 = b10 - b13;

    b1 = b11 + b12;
    b2 = b11 - b12;

    b5 = (b16 - b15)*m_iF0_no_sqrt >> m_iLog_scale;
    b6 = (b16 + b15)*m_iF0_no_sqrt >> m_iLog_scale;

    d[0] = (b0 + b1) >> m_iLog_scale_2; // for d[8*0]
    d[32] = (b0 - b1) >> m_iLog_scale_2; // for d[8*4]

    d[16] = (m_iF6*b2 + m_iF2*b3) >> m_iLog_scale_22_1; // for d[8*2]

    b14 = b4 + b5;
    b17 = b7 + b6;
    b15 = b4 - b5;
    b16 = b7 - b6;

    d[8] = (m_iF7*b14 + m_iF1*b17) >> m_iLog_scale_22_1; // for d[8*1]

// by DSCHO_20000209
    // d[8*3] = (b16 - b15) >> m_iLog_scale_2; //  d[8*3] = m_iF3*(b16) - (m_iF5)*b15 >> m_iLog_scale_22_1;
	d[24] = (m_iF3*b16 - m_iF5*b15) >> m_iLog_scale_22_1; // for d[8*3]
// ~by DSCHO_20000209

    d[40] = 0; // for d[8*5]
    d[48] = 0; // for d[8*6]
    d[56] = 0; // for d[8*7]
}

// -------------------- 6 * 6 DCT row / col --------------------
Void CFwdBlockDCT::forwardDCTRow6(Int *d)
{
    Int temp;
    Int b0, b1, b2, b3, b4, b5, b6, b7;
    Int b10, b11, b12, b13, b14, b15, b16, b17;
	
    b10 = d[0] + d[7];
    b11 = d[1] + d[6];
    b12 = d[2] + d[5];
    b13 = d[3] + d[4];

    b16 = d[1] - d[6];
    b15 = d[2] - d[5];

    b4 = (d[3] - d[4]);
    b7 = (d[0] - d[7]);

    b0 = b10 + b13;
    b3 = b10 - b13;

    b1 = b11 + b12;
    b2 = b11 - b12;

    b5 = (b16 - b15)*m_iF0_no_sqrt >> m_iLog_scale;
    b6 = (b16 + b15)*m_iF0_no_sqrt >> m_iLog_scale;

    d[0] = (b0 + b1) << m_iLog_scale__1;
    d[4] = (b0 - b1) << m_iLog_scale__1;

    d[2] = (m_iF6*b2 + m_iF2*b3);
    d[6] = 0; //

    b14 = b4 + b5;
    b15 = b4 - b5;

    b17 = b7 + b6;
    b16 = b7 - b6;

    d[1] = (m_iF7*b14 + m_iF1*b17);
    d[7] = 0; //

    temp = m_iF3*( b15 + b16 );
    d[5] = (temp + ( m_iF5 - m_iF3 )*b16 );
    d[3] = (temp - ( m_iF5 + m_iF3 )*b15 );
}

Void CFwdBlockDCT::forwardDCTCol6(Int *d)
{
    Int temp;
    Int b0, b1, b2, b3, b4, b5, b6, b7;
    Int b10, b11, b12, b13, b14, b15, b16, b17;

    b10 = d[0] + d[56];		//b10 = d[0<<3] + d[7<<3]; 
    b11 = d[8] + d[48];		//b11 = d[1<<3] + d[6<<3];
    b12 = d[16] + d[40];	//b12 = d[2<<3] + d[5<<3];
    b13 = d[24] + d[32];	//b13 = d[3<<3] + d[4<<3];

    b16 = d[8] - d[48];		//b16 = d[1<<3] - d[6<<3];
    b15 = d[16] - d[40];	//b15 = d[2<<3] - d[5<<3];

    b4 = d[24] - d[32];		//b4 = (d[3<<3] - d[4<<3]);
    b7 = d[0] - d[56];		//b7 = (d[0<<3] - d[7<<3]);

    b0 = b10 + b13;
    b3 = b10 - b13;

    b1 = b11 + b12;
    b2 = b11 - b12;

    b5 = (b16 - b15)*m_iF0_no_sqrt >> m_iLog_scale;
    b6 = (b16 + b15)*m_iF0_no_sqrt >> m_iLog_scale;

    d[0] = (b0 + b1) >> m_iLog_scale_2;		// for d[8*0]
    d[32] = (b0 - b1) >> m_iLog_scale_2;	// for d[8*4]

    d[16] = (m_iF6*b2 + m_iF2* b3) >> m_iLog_scale_22_1;	// for d[8*2]
    d[48] = 0;												// for d[8*6]

    b14 = b4 + b5;
    b15 = b4 - b5;

    b17 = b7 + b6;
    b16 = b7 - b6;

    d[8] = (m_iF7*b14 + m_iF1*b17) >> m_iLog_scale_22_1;	// for d[8*1]
    d[56] = 0;												// for d[8*7]

    temp = m_iF3 * ( b15 + b16 );
    d[40] = (temp + ( m_iF5 - m_iF3 ) * b16) >> m_iLog_scale_22_1; // for d[8*5]
    d[24] = (temp - ( m_iF5 + m_iF3 ) * b15) >> m_iLog_scale_22_1; // for d[8*3]
}

// -------------------- 7 * 7 DCT row / col --------------------
Void CFwdBlockDCT::forwardDCTRow7(Int *d)
{
    Int temp;
    Int b0, b1, b2, b3, b4, b5, b6, b7;
    Int b10, b11, b12, b13, b14, b15, b16, b17;
	
    b10 = d[0] + d[7];
    b11 = d[1] + d[6];
    b12 = d[2] + d[5];
    b13 = d[3] + d[4];

    b16 = d[1] - d[6];
    b15 = d[2] - d[5];

    b4 = (d[3] - d[4]);
    b7 = (d[0] - d[7]);

    b0 = b10 + b13;
    b3 = b10 - b13;

    b1 = b11 + b12;
    b2 = b11 - b12;

    b5 = (b16 - b15)*m_iF0_no_sqrt >> m_iLog_scale;
    b6 = (b16 + b15)*m_iF0_no_sqrt >> m_iLog_scale;

    d[0] = (b0 + b1) << m_iLog_scale__1;
    d[4] = (b0 - b1) << m_iLog_scale__1;

    temp =  m_iF6*(b2 + b3);
    d[2] = (temp + (m_iF2 - m_iF6)* b3);
    d[6] = (temp - (m_iF2 + m_iF6)* b2);

    b14 = b4 + b5;
    b15 = b4 - b5;

    b17 = b7 + b6;
    b16 = b7 - b6;

    d[1] = (m_iF7*b14 + m_iF1*b17);
    d[7] = 0; //

    temp = m_iF3*( b15 + b16 );
    d[5] = (temp + ( m_iF5 - m_iF3 )*b16 );
    d[3] = (temp - ( m_iF5 + m_iF3 )*b15 );
}

Void CFwdBlockDCT::forwardDCTCol7(Int *d)
{
    Int temp;
    Int b0, b1, b2, b3, b4, b5, b6, b7;
    Int b10, b11, b12, b13, b14, b15, b16, b17;

    b10 = d[0] + d[56];		//b10 = d[0<<3] + d[7<<3]; 
    b11 = d[8] + d[48];		//b11 = d[1<<3] + d[6<<3];
    b12 = d[16] + d[40];	//b12 = d[2<<3] + d[5<<3];
    b13 = d[24] + d[32];	//b13 = d[3<<3] + d[4<<3];

    b16 = d[8] - d[48];		//b16 = d[1<<3] - d[6<<3];
    b15 = d[16] - d[40];	//b15 = d[2<<3] - d[5<<3];

    b4 = d[24] - d[32];		//b4 = (d[3<<3] - d[4<<3]);
    b7 = d[0] - d[56];		//b7 = (d[0<<3] - d[7<<3]);

    b0 = b10 + b13;
    b3 = b10 - b13;

    b1 = b11 + b12;
    b2 = b11 - b12;

    b5 = (b16 - b15)*m_iF0_no_sqrt >> m_iLog_scale;
    b6 = (b16 + b15)*m_iF0_no_sqrt >> m_iLog_scale;

    d[0] = (b0 + b1) >> m_iLog_scale_2;		// for d[8*0]
    d[32] = (b0 - b1) >> m_iLog_scale_2;	// for d[8*4]

    temp = m_iF6*(b2 + b3);
    d[16] = (temp + (m_iF2 - m_iF6)*b3) >> m_iLog_scale_22_1;	// for d[8*2]
    d[48] = (temp - (m_iF2 + m_iF6)*b2) >> m_iLog_scale_22_1;	// for d[8*6]

    b14 = b4 + b5;
    b15 = b4 - b5;

    b17 = b7 + b6;
    b16 = b7 - b6;

    d[8] = (m_iF7*b14 + m_iF1*b17) >> m_iLog_scale_22_1;	// for d[8*1]
    d[56] = 0; // for d[8*7]

    temp = m_iF3 * ( b15 + b16 );
    d[40] = (temp + ( m_iF5 - m_iF3 ) * b16) >> m_iLog_scale_22_1;	// for d[8*5]
    d[24] = (temp - ( m_iF5 + m_iF3 ) * b15) >> m_iLog_scale_22_1;	// for d[8*3]
}
// ~by DSCHO on 2000/02/10 for Forward DCT ////////////////////////////////////////////////////////////////

// by DSCHO on 2000/01/19 for IDCT ////////////////////////////////////////////////////////////////
Void CInvBlockDCT::applyInverseDCT (const PixelC* rgchSrc, Int nColSrc, Int* rgiDst, Int nColDst)
{
	CoordI	iCol, iRow;
	Int iIndex = 0;
	Int iNCoeffs = 0; 
	Int iLast;	
	const PixelC* pchRowSrc = rgchSrc;

	for (iCol = 0; iCol < BLOCK_SIZE; iCol++, pchRowSrc += nColSrc)	{
		for (iRow = 0; iRow < BLOCK_SIZE; iRow++)	{
			Int iValue = m_rgiFltBuf[iIndex] = (Int) pchRowSrc[iRow];
			if (iValue != 0) {
				iLast = standarZigzagScanIndexUntil5(iIndex);
				if(iLast > iNCoeffs) iNCoeffs = iLast;
			}
			iIndex++;
		}
	}
	inverseDCT (m_rgiFltBuf, iNCoeffs);

	Int* piColDst = rgiDst;
	iIndex = 0;
	for (iCol = 0; iCol < BLOCK_SIZE; iCol++, piColDst += nColDst)	{
		for (iRow = 0; iRow < BLOCK_SIZE; iRow++)	{
			piColDst[iRow] = m_rgiFltBuf[iIndex];
			iIndex++;
		}
	}
}

Void CInvBlockDCT::applyInverseDCT (const Int* rgiSrc, Int nColSrc, PixelC* rgchDst, Int nColDst)
{
	CoordI	iCol, iRow;
	Int iIndex = 0;
	Int iNCoeffs = 0; 
	Int iLast;	
	const PixelI* piRowSrc = rgiSrc;
	for (iCol = 0; iCol < BLOCK_SIZE; iCol++, piRowSrc += nColSrc)	{
		for (iRow = 0; iRow < BLOCK_SIZE; iRow++)	{
			Int iValue = m_rgiFltBuf[iIndex] = (Int) piRowSrc[iRow];
			if (iValue != 0) {
				iLast = standarZigzagScanIndexUntil5(iIndex);
				if(iLast > iNCoeffs) iNCoeffs = iLast;
			}
			iIndex++;
		}
	}
	inverseDCT (m_rgiFltBuf, iNCoeffs);

	PixelC* pchColDst = rgchDst;
	iIndex = 0;
	for (iCol = 0; iCol < BLOCK_SIZE; iCol++, pchColDst += nColDst)	{
		for (iRow = 0; iRow < BLOCK_SIZE; iRow++)	{
			Int iValue = m_rgiFltBuf[iIndex];
			pchColDst[iRow] = m_rgchClipTbl[iValue];
			iIndex ++;
		}
	}
}

Void CInvBlockDCT::applyInverseDCT (const Int* rgiSrc, Int nColSrc, Int* rgiDst, Int nColDst)
{
	CoordI iCol, iRow;
	Int iIndex = 0;
	Int iNCoeffs = 0; 
	Int iLast;	
	const PixelI* piRowSrc = rgiSrc;
	for (iCol = 0; iCol < BLOCK_SIZE; iCol++, piRowSrc += nColSrc)	{
		for (iRow = 0; iRow < BLOCK_SIZE; iRow++)	{
			Int iValue = m_rgiFltBuf[iIndex] = (Int) piRowSrc[iRow];
			if (iValue != 0) {
				iLast = standarZigzagScanIndexUntil5(iIndex);
				if(iLast > iNCoeffs) iNCoeffs = iLast;
			}
			iIndex++;
		}
	}
	inverseDCT (m_rgiFltBuf, iNCoeffs);

	Int* piColDst = rgiDst;
	iIndex = 0;
	for (iCol = 0; iCol < BLOCK_SIZE; iCol++, piColDst += nColDst)	{
		for (iRow = 0; iRow < BLOCK_SIZE; iRow++)	{
			piColDst[iRow] = m_rgiFltBuf[iIndex];
			iIndex++;
		}
	}
}

Int	CInvBlockDCT::standarZigzagScanIndexUntil5(Int index) 
{
	Int iNCoeffs;
	if(index==0)		iNCoeffs = 0;
	else if(index==1)	iNCoeffs = 1;
	else if(index==8)	iNCoeffs = 2;
	else if(index==16)	iNCoeffs = 3;
	else if(index==9)	iNCoeffs = 4;
	else if(index==2)	iNCoeffs = 5;
	else				iNCoeffs = 6;
	return	iNCoeffs;
}

//////////////////////////////////////////////////////////////////////////////
//
// -- inverseDCT -- integer inverse DCT
//
// Author :		
//	Dae-Sung Cho (dscho@sait.samsung.co.kr)	
//
// Created :		
//	2000-01-24
//
// Purpose :		
//  a fast integer implementation of the inverse DCT (Discrete Cosine Transform).
// 
// Arguments in : 	
//  none
//
// Arguments in/out :	
//	Int			*data	: input coefficient data and output block data
//
// Arguments out :	
//	none
//
// Return values :	
//	none
//
// Side effects :	
//	none
//
// Description :	
//	inverse two dimensional DCT, Chen-Wang algorithm       

//	32-bit integer arithmetic (8 bit coefficients) 
//  11 mults, 29 adds per DCT
//                                      sE, 18.8.91
//
// this code assumes >> to be a two's-complement arithmetic
// right shift: (-2)>>1 == -1 , (-3)>>1 == -2
//
// See also :
//	(cf. IEEE ASSP-32, pp. 803-816, Aug. 1984)
//
// Modified :		
//
//////////////////////////////////////////////////////////////////////////////

Void CInvBlockDCT::inverseDCT(Int *coeff, Int ncoeffs)
{
//  Int i, j;

/* if column dct is performed before row dct, it will be better . */

  if(ncoeffs == 0) 
  {
      onlyDC(coeff); 
  }
  else if (ncoeffs == 1)
  {
      inverseDCTRow2(coeff+8*0);
   
      inverseDCTCol1(coeff+0);
      inverseDCTCol1(coeff+1);
      inverseDCTCol1(coeff+2);
      inverseDCTCol1(coeff+3);
      inverseDCTCol1(coeff+4);
      inverseDCTCol1(coeff+5);
      inverseDCTCol1(coeff+6);
      inverseDCTCol1(coeff+7);
   }
   else if (ncoeffs == 2)
   {
      inverseDCTRow2(coeff+8*0);
      inverseDCTRow1(coeff+8*1);
   
      inverseDCTCol2(coeff+0);
      inverseDCTCol2(coeff+1);
      inverseDCTCol2(coeff+2);
      inverseDCTCol2(coeff+3);
      inverseDCTCol2(coeff+4);
      inverseDCTCol2(coeff+5);
      inverseDCTCol2(coeff+6);
      inverseDCTCol2(coeff+7);
   }
   else if  ( ncoeffs <= 5 )
   {
      inverseDCTRow3(coeff+8*0);
      inverseDCTRow2(coeff+8*1);
      inverseDCTRow1(coeff+8*2);
   
      inverseDCTCol3(coeff+0);
      inverseDCTCol3(coeff+1);
      inverseDCTCol3(coeff+2);
      inverseDCTCol3(coeff+3);
      inverseDCTCol3(coeff+4);
      inverseDCTCol3(coeff+5);
      inverseDCTCol3(coeff+6);
      inverseDCTCol3(coeff+7);
   }
   else 
   {
      inverseDCTRow(coeff+8*0);
      inverseDCTRow(coeff+8*1);
      inverseDCTRow(coeff+8*2);
      inverseDCTRow(coeff+8*3);
      inverseDCTRow(coeff+8*4);
      inverseDCTRow(coeff+8*5);
      inverseDCTRow(coeff+8*6);
      inverseDCTRow(coeff+8*7);

      inverseDCTCol(coeff+0);
      inverseDCTCol(coeff+1);
      inverseDCTCol(coeff+2);
      inverseDCTCol(coeff+3);
      inverseDCTCol(coeff+4);
      inverseDCTCol(coeff+5);
      inverseDCTCol(coeff+6);
      inverseDCTCol(coeff+7);
   }
}

Void CInvBlockDCT::onlyDC(Int *bk)
{
    Int i, mean;

    mean = (bk[0] + 4 ) >> 3;

    for( i =0; i < 16; i ++)
    {
       bk[0] = mean;
       bk[1] = mean;
       bk[2] = mean;
       bk[3] = mean;

       bk += 4;
    }
}

/* row (horizontal) inverseDCT
 *
 *           7                       pi         1
 * dst[k] = sum c[l] * src[l] * cos( -- * ( k + - ) * l )
 *          l=0                      8          2
 *
 * where: c[0]    = 128
 *        c[1..7] = 128*sqrt(2)
 */

Void CInvBlockDCT::inverseDCTRow(Int *blk)
{
  Int x0, x1, x2, x3, x4, x5, x6, x7, x8;

  /* Intcut */

  if (!((x1 = blk[4]<<11) | (x2 = blk[6]) | (x3 = blk[2]) |
        (x4 = blk[1]) | (x5 = blk[7]) | (x6 = blk[5]) | (x7 = blk[3])))
  {
    blk[0]=blk[1]=blk[2]=blk[3]=blk[4]=blk[5]=blk[6]=blk[7]=blk[0]<<3;
    return;
  }

  x0 = (blk[0]<<11) + 128; /* for proper rounding in the fourth stage */

  /* first stage */
  if( x4 | x5 )
  {
  x8 = m_iW7*(x4+x5);
  x4 = x8 + (m_iW1-m_iW7)*x4;
  x5 = x8 - (m_iW1+m_iW7)*x5;
  } else x4 = x5 =0;

  if( x6 | x7 )
  {
  x8 = m_iW3*(x6+x7);
  x6 = x8 - (m_iW3-m_iW5)*x6;
  x7 = x8 - (m_iW3+m_iW5)*x7;
  } else x6 = x7 = 0;
  
  /* second stage */
  x8 = x0 + x1;
  x0 -= x1;
  
  if( x2 | x3)
  {
  x1 = m_iW6*(x3+x2);
  x2 = x1 - (m_iW2+m_iW6)*x2;
  x3 = x1 + (m_iW2-m_iW6)*x3;
  } else x2 =x3 =0;

  x1 = x4 + x6;
  x4 -= x6;
  x6 = x5 + x7;
  x5 -= x7;
  
  /* third stage */
  x7 = x8 + x3;
  x8 -= x3;
  x3 = x0 + x2;
  x0 -= x2;
  
 if( x4 | x5 )
 {
  x2 = (181*(x4+x5)+128)>>8;
  x4 = (181*(x4-x5)+128)>>8;
 } else x2 = x4 =0;
  
  /* fourth stage */
  blk[0] = (x7+x1)>>8;
  blk[1] = (x3+x2)>>8;
  blk[2] = (x0+x4)>>8;
  blk[3] = (x8+x6)>>8;
  blk[4] = (x8-x6)>>8;
  blk[5] = (x0-x4)>>8;
  blk[6] = (x3-x2)>>8;
  blk[7] = (x7-x1)>>8;
}

/* column (vertical) inverseDCT
 *
 *             7                         pi         1
 * dst[8*k] = sum c[l] * src[8*l] * cos( -- * ( k + - ) * l )
 *            l=0                        8          2
 *
 * where: c[0]    = 1/1024
 *        c[1..7] = (1/1024)*sqrt(2)
 */

Void CInvBlockDCT::inverseDCTCol(Int *blk)
{
  Int x0, x1, x2, x3, x4, x5, x6, x7, x8;

  /* Intcut */

  if (!((x1 = (blk[32]<<8)) | (x2 = blk[48]) | (x3 = blk[16]) |
        (x4 = blk[8]) | (x5 = blk[56]) | (x6 = blk[40]) | (x7 = blk[24])))
  {
    blk[0]=blk[8]=blk[16]=blk[24]=blk[32]=blk[40]=blk[48]=blk[56]=
      m_rgnClipTbl[(blk[0]+32)>>6];
    return;
  }

  x0 = (blk[8*0]<<8) + 8192;

  /* first stage */
  if( x4 | x5)
  {
  x8 = m_iW7*(x4+x5) + 4;
  x4 = (x8+(m_iW1-m_iW7)*x4)>>3;
  x5 = (x8-(m_iW1+m_iW7)*x5)>>3;
  } else x4 =x5 =0;

  if(x6 | x7 )
  {
  x8 = m_iW3*(x6+x7) + 4;
  x6 = (x8-(m_iW3-m_iW5)*x6)>>3;
  x7 = (x8-(m_iW3+m_iW5)*x7)>>3;
  } else x6 = x7 =0;
  
  /* second stage */
  x8 = x0 + x1;
  x0 -= x1;

  if( x2 | x3 )
  {
  x1 = m_iW6*(x3+x2) + 4;
  x2 = (x1-(m_iW2+m_iW6)*x2)>>3;
  x3 = (x1+(m_iW2-m_iW6)*x3)>>3;
  } else x2 = x3 = 0;

  x1 = x4 + x6;
  x4 -= x6;
  x6 = x5 + x7;
  x5 -= x7;
  
  /* third stage */
  x7 = x8 + x3;
  x8 -= x3;
  x3 = x0 + x2;
  x0 -= x2;
  x2 = (181*(x4+x5)+128)>>8;
  x4 = (181*(x4-x5)+128)>>8;
  
  /* fourth stage */
  blk[0] = m_rgnClipTbl[(x7+x1)>>14];
  blk[8] = m_rgnClipTbl[(x3+x2)>>14];
  blk[16] = m_rgnClipTbl[(x0+x4)>>14];
  blk[24] = m_rgnClipTbl[(x8+x6)>>14];
  blk[32] = m_rgnClipTbl[(x8-x6)>>14];
  blk[40] = m_rgnClipTbl[(x0-x4)>>14];
  blk[48] = m_rgnClipTbl[(x3-x2)>>14];
  blk[56] = m_rgnClipTbl[(x7-x1)>>14];
}

// --------------- only one ac coef. -------------

Void CInvBlockDCT::inverseDCTRow1(Int *blk)
{
  blk[0]=blk[1]=blk[2]=blk[3]=blk[4]=blk[5]=blk[6]=blk[7]=blk[0]<<3;
}

Void CInvBlockDCT::inverseDCTCol1(Int *blk)
{
    blk[0]=blk[8]=blk[16]=blk[24]=blk[32]=blk[40]=blk[48]=blk[56]=
      m_rgnClipTbl[(blk[0]+32)>>6];
}


// ------------- only two ac coef. ------------- 

Void CInvBlockDCT::inverseDCTRow2(Int *blk)
{
  Int x0, x1, x2, x4, x5;

  if (!(x4 = blk[1]))
  {
    blk[0]=blk[1]=blk[2]=blk[3]=blk[4]=blk[5]=blk[6]=blk[7]=blk[0]<<3;
    return;
  }

  x0 = (blk[0]<<11) + 128; 

  x5 = m_iW7*x4;
  x1 = x4 = x5 + (m_iW1-m_iW7)*x4;

  x2 = (181*(x4+x5)+128)>>8;
  x4 = (181*(x4-x5)+128)>>8;
  
  /* fourth stage */
  blk[0] = (x0+x1)>>8;
  blk[1] = (x0+x2)>>8;
  blk[2] = (x0+x4)>>8;
  blk[3] = (x0+x5)>>8;
  blk[4] = (x0-x5)>>8;
  blk[5] = (x0-x4)>>8;
  blk[6] = (x0-x2)>>8;
  blk[7] = (x0-x1)>>8;
}

Void CInvBlockDCT::inverseDCTCol2(Int *blk)
{
  Int x0, x1, x2, x4, x5, x8;

  if (!(x4 = blk[8*1]))
  {
    blk[0]=blk[8]=blk[16]=blk[24]=blk[32]=blk[40]=blk[48]=blk[56]=
      m_rgnClipTbl[(blk[0]+32)>>6];
    return;
  }

  x0 = (blk[0]<<8) + 8192;

  x8 = m_iW7*(x4) + 4;
  x1 = x4 = (x8+(m_iW1-m_iW7)*x4)>>3;
  x5 = x8>>3;

  x2 = (181*(x4+x5)+128)>>8;
  x4 = (181*(x4-x5)+128)>>8;
  
  /* fourth stage */
  blk[8*0] = m_rgnClipTbl[(x0+x1)>>14];
  blk[8*1] = m_rgnClipTbl[(x0+x2)>>14];
  blk[8*2] = m_rgnClipTbl[(x0+x4)>>14];
  blk[8*3] = m_rgnClipTbl[(x0+x5)>>14];
  blk[8*4] = m_rgnClipTbl[(x0-x5)>>14];
  blk[8*5] = m_rgnClipTbl[(x0-x4)>>14];
  blk[8*6] = m_rgnClipTbl[(x0-x2)>>14];
  blk[8*7] = m_rgnClipTbl[(x0-x1)>>14];
}
//  ------------- only three ac coef. ------------- 

Void CInvBlockDCT::inverseDCTRow3(Int *blk)
{
  Int x0, x1, x2, x3, x4, x5, x6, x7, x8;

  if (!((x3 = blk[2]) | (x4 = blk[1])))
  {
    blk[0]=blk[1]=blk[2]=blk[3]=blk[4]=blk[5]=blk[6]=blk[7]=blk[0]<<3;
    return;
  }

  x0 = x8 = (blk[0]<<11) + 128; /* for proper rounding in the fourth stage */

  /* first stage */
  x5 = x6 = m_iW7*x4;
  x1 = x4 = m_iW1*x4;
  
  /* second stage */
  x2 = m_iW6*x3;
  x3 = m_iW2*x3;
  
  /* third stage */
  x7 = x8 + x3;
  x8 -= x3;
  x3 = x0 + x2;
  x0 -= x2;
  x2 = (181*(x4+x5)+128)>>8;
  x4 = (181*(x4-x5)+128)>>8;
  
  /* fourth stage */
  blk[0] = (x7+x1)>>8;
  blk[1] = (x3+x2)>>8;
  blk[2] = (x0+x4)>>8;
  blk[3] = (x8+x6)>>8;
  blk[4] = (x8-x6)>>8;
  blk[5] = (x0-x4)>>8;
  blk[6] = (x3-x2)>>8;
  blk[7] = (x7-x1)>>8;
}

Void CInvBlockDCT::inverseDCTCol3(Int *blk)
{
  Int x0, x1, x2, x3, x4, x5, x6, x7, x8;

  /* Intcut */
  if (!((x3 = blk[8*2]) | (x4 = blk[8*1])))
  {
    blk[0]=blk[8]=blk[16]=blk[24]=blk[32]=blk[40]=blk[48]=blk[56]=
      m_rgnClipTbl[(blk[8*0]+32)>>6];
    return;
  }

  x0 = x8 = (blk[0]<<8) + 8192;

  /* first stage */
  x5 = x6 = (m_iW7*x4 + 4)>>3;
  x1 = x4 = (m_iW1*x4 + 4)>>3;
  
  /* second stage */
  x2 = (m_iW6*x3 + 4)>>3;
  x3 = (m_iW2*x3 + 4)>>3;

  /* third stage */
  x7 = x8 + x3;
  x8 -= x3;
  x3 = x0 + x2;
  x0 -= x2;
  x2 = (181*(x4+x5)+128)>>8;
  x4 = (181*(x4-x5)+128)>>8;
  
  /* fourth stage */
  blk[ 0] = m_rgnClipTbl[(x7+x1)>>14];
  blk[ 8] = m_rgnClipTbl[(x3+x2)>>14];
  blk[16] = m_rgnClipTbl[(x0+x4)>>14];
  blk[24] = m_rgnClipTbl[(x8+x6)>>14];
  blk[32] = m_rgnClipTbl[(x8-x6)>>14];
  blk[40] = m_rgnClipTbl[(x0-x4)>>14];
  blk[48] = m_rgnClipTbl[(x3-x2)>>14];
  blk[56] = m_rgnClipTbl[(x7-x1)>>14];
}

#endif
// ~by DSCHO on 2000/01/19 for IDCT ////////////////////////////////////////////////////////////////