/*
	This software module was originally developed by ENST in the course of
development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.
	This software module is an implementation of a part of one or more MPEG-4
Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1) 
standard. ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license 
to this software module or modifications thereof for use in hardware or software 
products claiming conformance to the MPEG-4 Systems (ISO/IEC 14496-1). 
Those intending to use this software module in hardware or software products are 
advised that its use may infringe existing patents.
	The original developer of this software module and his/her company, the 
subsequent editors and their companies, and ISO/IEC have no liability for use of 
this software module or modifications thereof in an implementation. Copyright is 
not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
	ENST retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using the code 
for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright    
notice must be included in all copies or derivative works.              
						Copyright  2002.
*/

//////////////////////////////////////////////////////////////////////////////
//		File: PNGDec.cpp
//
//		Author: Jean Le Feuvre ENST 2002
//
//		Content: IM1 wrapper for LibPNG
//
//////////////////////////////////////////////////////////////////////////////

#include <assert.h>
#include "windows.h"
#include "PNGDec.h"
#include "zconf.h"
#include "zlib.h"
#include "pngconf.h"
#include "png.h"

void *CreateInstance ()
{
	return new PNGDec;
}

static void user_read_data(png_structp png_ptr, png_bytep data, png_size_t length)
{
	PNGDec* obj = (PNGDec *)png_ptr->io_ptr;
	if (obj->ReadData(data, length) != length) 
		png_error(png_ptr, "Read Error");
}

unsigned int PNGDec::ReadData(BYTE *ptr, unsigned int length)
{
	if (m_CurrentPos+length>m_InputLength) 
		return 0;
	memcpy(ptr, m_pInput+m_CurrentPos, length);
	m_CurrentPos += length;
	return length;
}

static void user_error_fn(png_structp png_ptr,png_const_charp error_msg)
{
	longjmp(png_ptr->jmpbuf, 1);
}

BOOL PNGDec::Decode(LPBYTE pInput,int nInputLength,LPBYTE pOutput,int &rnOutputLength)
{
	png_struct *png_ptr;
	png_info *info_ptr;

	if (nInputLength<8) return FALSE;
	int is_png = !png_sig_cmp(pInput, 0, 8);

	m_pInput = pInput;
	m_InputLength = nInputLength;
	m_CurrentPos = 0;

	png_ptr = png_create_read_struct(PNG_LIBPNG_VER_STRING, (png_voidp) this, NULL, NULL);
	if (!png_ptr) return FALSE;

	info_ptr = png_create_info_struct(png_ptr);
	if (info_ptr == NULL) {
		png_destroy_read_struct(&png_ptr, (png_infopp)NULL, (png_infopp)NULL);
		return FALSE;
	}
	if (setjmp(png_ptr->jmpbuf)) {
		png_destroy_read_struct(&png_ptr, &info_ptr, (png_infopp)NULL);
		return FALSE;
	}
    png_set_read_fn(png_ptr, this, (png_rw_ptr) user_read_data);
	png_set_error_fn(png_ptr, this, (png_error_ptr) user_error_fn, NULL);

	// read the file information 
	png_read_info(png_ptr, info_ptr);

	m_BPP = info_ptr->pixel_depth / 8;
	m_Width = info_ptr->width;
	m_Height = info_ptr->height;

#if defined SAIT_CONFORMANCE_2003
	//m_Outsize = m_Width*m_Height*m_BPP+9;	
	m_Outsize = m_Width*m_Height*m_BPP+10;	//16bits
	unsigned char m_channel = info_ptr->channels;
#else 
	m_Outsize = m_Width*m_Height*m_BPP;
#endif SAIT_CONFORMANCE_2003

	if (rnOutputLength < (int) m_Outsize) {	
		rnOutputLength = (int) m_Outsize;
		png_destroy_read_struct(&png_ptr, &info_ptr, (png_infopp)NULL);
		return FALSE;
	}

	//finally read
	unsigned int stride = png_get_rowbytes(png_ptr, info_ptr);
	png_byte **rows = (png_bytepp) malloc(sizeof(png_bytep)*m_Height);
	for (int i=0; i< (int) m_Height; i++) {
#if defined SAIT_CONFORMANCE_2003
		//rows[i] = pOutput + i*stride + 9;
		rows[i] = pOutput + i*stride + 10;	//16bits
#else		
		rows[i] = pOutput + i*stride;
#endif SAIT_CONFORMANCE_2003		
	}
	png_read_image(png_ptr, rows);
    png_read_end(png_ptr, NULL);


#if defined SAIT_CONFORMANCE_2003
	// put Width & Height in the first 9 bytes...
	//width -> 32bit
	pOutput[0] = (m_Width >> 24) & 0xff;
	pOutput[1] = (m_Width >> 16) & 0xff;
	pOutput[2] = (m_Width >> 8) & 0xff;
	pOutput[3] = (m_Width) & 0xff;
	
	//height -> 32 bit
	pOutput[4] = (m_Height >> 24) & 0xff;
	pOutput[5] = (m_Height >> 16) & 0xff;
	pOutput[6] = (m_Height >> 8) & 0xff;
	pOutput[7] = (m_Height) & 0xff;
	// channel -> 8 bit
	pOutput[8] = (m_channel) & 0xff;
	pOutput[9] = (m_BPP) & 0xff;	//16bits

	// put Width & Height in the first 4 bytes...

	/*
	// test
	FILE *fp;
	fp = fopen("test.raw", "w");
	//	for(int kk=0;kk<m_BPP;kk++)
		for(int ii=0;ii<m_Width;ii++)
			for(int jj=0;jj<m_Height;jj++)
			{
				fprintf(fp, "%c", *(pOutput + 9 + ii*m_Height + jj));
				//fflush(fp);
			}
			// + kk*m_Width*m_Height));
	fclose(fp); 
	*/
#endif SAIT_CONFORMANCE_2003 

	free(rows);

	m_pInput = NULL;
	m_InputLength = m_CurrentPos = 0;

	png_destroy_read_struct(&png_ptr, &info_ptr, (png_infopp) NULL);
	return TRUE;
}


int PNGDec :: GetOptimalOutputSize()
{
	if (!m_Outsize) return 0; 
	return (m_Outsize);
}

DWORD PNGDec :: GetAttributes (ATTRIBUTE_CODE code, const LPBYTE)
{
	switch (code) {
	case FRAME_WIDTH:
		return m_Width;
	case FRAME_HEIGHT:
		return m_Height;
	case PITCH:
		return m_BPP*m_Width;
	case OFFSETX:
		return 0;
	case OFFSETY:
		return 0;
	default:
		return 0;
	}
}


BOOL PNGDec :: SetFormat (Capabilities &formatOptions, DecoderParams &decoderParams)
{
	decoderParams.m_nDBSize = 10000;
	decoderParams.m_nOptimalAUSize = 0;
	decoderParams.m_nCBSize = -1; 
	decoderParams.m_bRequiresFullAUs = TRUE;

	if (formatOptions.GetFormatCapabilityLength () == 0) 
		return FALSE; 
	decoderParams.m_nStreamNumber = 0; 

	switch (m_BPP) {
	case 1:
		decoderParams.m_format = MediaFormat :: GRAYSCALE; 
		break;
	case 2:
		decoderParams.m_format = MediaFormat :: ALPHAGRAYSCALE; 
		break;
	case 3:
		decoderParams.m_format = MediaFormat :: RGB; 
		break;
	case 4:
		decoderParams.m_format = MediaFormat :: RGBA; 
		break;
	//we're not init 
	default:
		decoderParams.m_format = MediaFormat :: UNKNOWN; 
		break;
	}
	return TRUE;
}

