/*
	This software module was originally developed by ENST in the course of
development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.
	This software module is an implementation of a part of one or more MPEG-4
Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1) 
standard. ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license 
to this software module or modifications thereof for use in hardware or software 
products claiming conformance to the MPEG-4 Systems (ISO/IEC 14496-1). 
Those intending to use this software module in hardware or software products are 
advised that its use may infringe existing patents.
	The original developer of this software module and his/her company, the 
subsequent editors and their companies, and ISO/IEC have no liability for use of 
this software module or modifications thereof in an implementation. Copyright is 
not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
	ENST retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using the code 
for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright    
notice must be included in all copies or derivative works.              
						Copyright  2003.
*/

///////////////////////////////////////////////////////////////////////////
// File: faad_dec.cpp
//
// Authors: Jean Le Feuvre ENST 2003
//
///////////////////////////////////////////////////////////////////////////

#include "defs.h"
#include "faad_dec.h"
#include <assert.h>
#include "MediaStream.h"

/*-------------------------------------------------------------
The exported function that instantiates the proper decoder
-------------------------------------------------------------*/
void *CreateInstance ()
{
	return new FAAD_Dec;
}


FAAD_Dec::FAAD_Dec() : DecoderImp()
{
	m_pCodec = NULL;
	m_SampleRate = m_NumChannels = m_NumSamples = m_ESNum = 0;
	m_OutSize = 0;
}

FAAD_Dec::~FAAD_Dec()
{
	if (m_pCodec) faacDecClose(m_pCodec);
}

BOOL FAAD_Dec::Setup(const BYTE *pSpecificInfo, int nSpecificInfoLength, int nESNumber)
{
	if (!pSpecificInfo || !nSpecificInfoLength) return FALSE;
	if (m_pCodec) faacDecClose(m_pCodec);


	m_pCodec = faacDecOpen();
	if (!m_pCodec) return FALSE;

	faacDecInit2(m_pCodec, (unsigned char *) pSpecificInfo, nSpecificInfoLength, &m_SampleRate, &m_NumChannels);
	//usual number of samples in a decoded AAC frame
	m_NumSamples = 1024;

	//we only handle 16bits per samples.
	m_OutSize = m_NumSamples * m_NumChannels * 2;
	m_ESNum = nESNumber;
	return TRUE;
}

int FAAD_Dec::GetOptimalOutputSize()
{
	return m_OutSize;
}



BOOL FAAD_Dec :: SetFormat(Capabilities &formatOptions,DecoderParams &decoderParams)
{
	decoderParams.m_format = MediaFormat :: PCM;
	decoderParams.m_nOptimalAUSize = 256;
	decoderParams.m_nDBSize = 10000;
	//work with 10 AUs in composition memory
	decoderParams.m_nCBSize = 20*GetOptimalOutputSize();
	decoderParams.m_bRequiresFullAUs = TRUE;
	decoderParams.m_nStreamNumber = m_ESNum;
	return TRUE;
}		

BOOL FAAD_Dec::Decode( LPBYTE AUptr, int /*AUlen*/, LPBYTE PUptr, int &PUlen )
{
	void *buf;

	if (PUlen < m_OutSize) {
		PUlen = m_OutSize;
		return FALSE;
	}

	buf = faacDecDecode(m_pCodec, &m_Info, AUptr);
	//core cannot handle empty CUs (it locks...) so write an empty frame. the API may change one of these days...
	if (!m_Info.samples || !buf || !m_Info.bytesconsumed || m_Info.error>0) {
		::memset(PUptr, m_OutSize, 0 );
		PUlen = m_OutSize;
	} else { 
		if (sizeof(short)*m_Info.samples > PUlen) {
			PUlen = sizeof(short)*m_Info.samples; 
			return FALSE;
		}
		memcpy(PUptr, buf, sizeof(short)*m_Info.samples);
		PUlen = sizeof(short)*m_Info.samples;
	}
	return TRUE;
}



/*-------------------------------------------------------------
In this decoder, attributes are fixed for all units
-------------------------------------------------------------*/
DWORD FAAD_Dec :: GetAttributes(ATTRIBUTE_CODE code, const LPBYTE)
{
	switch (code) {
	case AUDIO_FREQUENCY:
		return m_SampleRate;
	case BITS_PER_SAMPLE :
		return 16;
	case NUM_CHANNELS:
		return m_NumChannels;
	default:
		return 0;
	}
}

