/*
	This software module was originally developed by ENST in the course of
development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.
	This software module is an implementation of a part of one or more MPEG-4
Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1) 
standard. ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license 
to this software module or modifications thereof for use in hardware or software 
products claiming conformance to the MPEG-4 Systems (ISO/IEC 14496-1). 
Those intending to use this software module in hardware or software products are 
advised that its use may infringe existing patents.
	The original developer of this software module and his/her company, the 
subsequent editors and their companies, and ISO/IEC have no liability for use of 
this software module or modifications thereof in an implementation. Copyright is 
not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
	ENST retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using the code 
for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright    
notice must be included in all copies or derivative works.              
						Copyright  2003.
*/

///////////////////////////////////////////////////////////////////////////
// File: mad_dec.cpp
//
// Authors: Jean Le Feuvre ENST 2003
//
///////////////////////////////////////////////////////////////////////////

#include "defs.h"
#include "mad_dec.h"
#include <assert.h>
#include "MediaStream.h"

/*-------------------------------------------------------------
The exported function that instantiates the proper decoder
-------------------------------------------------------------*/
void *CreateInstance ()
{
	return new MAD_Dec;
}


MAD_Dec::MAD_Dec() : DecoderImp()
{
	m_SampleRate = m_NumChannels = m_NumSamples = m_ESNum = 0;
	m_OutSize = 0;
	m_len = 0;
	m_bFirst = TRUE;

	mad_stream_init(&stream);
	mad_frame_init(&frame);
	mad_synth_init(&synth);

}

MAD_Dec::~MAD_Dec()
{
	mad_stream_finish(&stream);
	mad_frame_finish(&frame);
	mad_synth_finish(&synth);
}

BOOL MAD_Dec::Setup(const BYTE *pSpecificInfo, int nSpecificInfoLength, int nESNumber)
{
	//we only handle 16bits per samples.
	m_OutSize = 1152 * 2 * 2;
	m_ESNum = nESNumber;
	return TRUE;
}

int MAD_Dec::GetOptimalOutputSize()
{
	return m_OutSize;
}



BOOL MAD_Dec :: SetFormat(Capabilities &formatOptions,DecoderParams &decoderParams)
{
	decoderParams.m_format = MediaFormat :: PCM;
	decoderParams.m_nOptimalAUSize = 256;
	decoderParams.m_nDBSize = 10000;
	//work with 10 AUs in composition memory
	decoderParams.m_nCBSize = 10*GetOptimalOutputSize();
	decoderParams.m_bRequiresFullAUs = TRUE;
	decoderParams.m_nStreamNumber = m_ESNum;
	return TRUE;
}		

/*cf minimad.c*/
static inline signed int scale(mad_fixed_t sample)
{
  sample += (1L << (MAD_F_FRACBITS - 16));
  if (sample >= MAD_F_ONE)
    sample = MAD_F_ONE - 1;
  else if (sample < -MAD_F_ONE)
    sample = -MAD_F_ONE;

  return sample >> (MAD_F_FRACBITS + 1 - 16);
}


BOOL MAD_Dec::Decode( LPBYTE AUptr, int AUlen, LPBYTE PUptr, int &PUlen)
{
	char *ptr = (char *) PUptr;
	int num;
		
	if (PUlen < m_OutSize) {
		PUlen = m_OutSize;
		return FALSE;
	}
	//store first frame and insert silence - this is a bug in mad that screws up byte offset 
	//in stream/frame side info when working with aligned frames, so we force init with 2 AUs...
	if (m_bFirst) {
		m_bFirst = FALSE;
		memcpy(m_buffer, AUptr, AUlen);
		m_len = AUlen;
		goto err;
	}
	memcpy(m_buffer + m_len, AUptr, AUlen);
	m_len += AUlen;

	mad_stream_buffer(&stream, m_buffer, m_len);


	if (mad_frame_decode(&frame, &stream) == -1) {
		memcpy(m_buffer, AUptr, AUlen);
		m_len = AUlen;
		goto err;
	}

	//move non used data to begining of buffer
	if (stream.next_frame) {
		m_len = &m_buffer[m_len] - stream.next_frame;
	    memmove(m_buffer, stream.next_frame, m_len);
	}

	//store config	
	if (!m_SampleRate) {
		m_SampleRate = synth.pcm.samplerate;
		m_NumChannels = synth.pcm.channels;
	}
	//write PCM output
	mad_synth_frame(&synth, &frame);
	num = synth.pcm.length;
	mad_fixed_t *left_ch, *right_ch;
	left_ch = synth.pcm.samples[0];
	right_ch = synth.pcm.samples[1];
	PUlen = 0;
	while (num--) {
		signed int sample;
		sample = scale(*left_ch++);

		*ptr = (sample >> 0) & 0xff;
		ptr++;
		*ptr = (sample >> 8) & 0xff;
		ptr++;
		PUlen += 2;

		if (m_NumChannels == 2) {
			sample = scale(*right_ch++);
			*ptr = (sample >> 0) & 0xff;
			ptr++;
			*ptr = (sample >> 8) & 0xff;
			ptr++;
			PUlen += 2;
		}
	}
	return TRUE;

err:
	memset(PUptr, 0, PUlen);
	return TRUE;
}



/*-------------------------------------------------------------
In this decoder, attributes are fixed for all units
-------------------------------------------------------------*/
DWORD MAD_Dec :: GetAttributes(ATTRIBUTE_CODE code, const LPBYTE)
{
	switch (code) {
	case AUDIO_FREQUENCY:
		return m_SampleRate;
	case BITS_PER_SAMPLE :
		return 16;
	case NUM_CHANNELS:
		return m_NumChannels;
	default:
		return 0;
	}
}

