/*
	This software module was originally developed by ENST in the course of
development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.
	This software module is an implementation of a part of one or more MPEG-4
Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1) 
standard. ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license 
to this software module or modifications thereof for use in hardware or software 
products claiming conformance to the MPEG-4 Systems (ISO/IEC 14496-1). 
Those intending to use this software module in hardware or software products are 
advised that its use may infringe existing patents.
	The original developer of this software module and his/her company, the 
subsequent editors and their companies, and ISO/IEC have no liability for use of 
this software module or modifications thereof in an implementation. Copyright is 
not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
	ENST retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using the code 
for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright    
notice must be included in all copies or derivative works.              
						Copyright  2003.
*/

///////////////////////////////////////////////////////////////////////////
// File: xvid_dec.cpp
//
// Authors: Jean Le Feuvre ENST 2003
//
///////////////////////////////////////////////////////////////////////////

#include <windows.h>
#include "xvid_dec.h"
#include "MediaStream.h"

//set to 1 to perform YUV to RGB in the codec, 0 to let the player handle it
//note that it is usually better to use the player conversion since YUV2RGB time on droped frames is then saved
#define BGR_OUTPUT	0

void yuv2rgb_init(void);
void yuv2rgb_24(unsigned char *dst, int dst_stride, unsigned char *y_src, unsigned char *u_src, unsigned char * v_src, 
				 int y_stride, int uv_stride, int width, int height);



void *CreateInstance ()
{
	return new XVIDDec;
}


static int isInit = 0;

XVIDDec :: XVIDDec () : DecoderImp () 
{
	m_pCodec = NULL;
	m_pConv = NULL;
	if (!isInit) {
		XVID_INIT_PARAM init;
		init.api_version = 0;
		init.core_build = 0;
		/*let the codec decide */
		init.cpu_flags = XVID_CPU_CHKONLY;
		xvid_init(NULL, 0, &init, NULL);
		xvid_init(NULL, 0, &init, NULL);
		
		isInit = 1;
	}
}

XVIDDec :: ~XVIDDec()
{
	if (m_pCodec) xvid_decore(m_pCodec, XVID_DEC_DESTROY, NULL, NULL);
	if (m_pConv) free(m_pConv);
}


BOOL XVIDDec :: Setup ( const BYTE *pSpecificInfo, int nSpecificInfoLength, int EsID)
{
	if (!pSpecificInfo || !nSpecificInfoLength) return FALSE;

	XVID_DEC_FRAME frame;
	memset(&frame, 0, sizeof(XVID_DEC_FRAME));
	frame.bitstream = (void *) pSpecificInfo;
	frame.length = nSpecificInfoLength;
	XVID_DEC_PARAM par;

	if (my_xvid_configure(&frame, &par) != XVID_ERR_OK) return FALSE;
	if (xvid_decore(NULL, XVID_DEC_CREATE, &par, NULL) != XVID_ERR_OK) return FALSE;
	m_Width = par.width;
	m_Height = par.height;
	m_pCodec = par.handle;

	//and init 
	xvid_decore(m_pCodec, XVID_DEC_DECODE, &frame, NULL);

	m_EsNum = EsID;	

#if BGR_OUTPUT
	m_OutSize = m_Width * m_Height * 3;
	/*don't use xvid colorspace conversion it is broken in mmx (memory overwrite)*/
	m_pConv = (unsigned char *) malloc(sizeof(char) * m_Width * m_Height * 5 / 2);
#else
	m_OutSize = m_Width * m_Height * 5 / 2;
#endif

	return TRUE;
}

/*-----------------------------------------------------------------
Find a compatible format and initialize the decoder
-----------------------------------------------------------------*/
BOOL XVIDDec :: SetFormat (Capabilities &formatOptions, DecoderParams &decoderParams)
{
	decoderParams.m_nOptimalAUSize = 256;
	decoderParams.m_nDBSize = 10000;
	//number of CU to cache
	decoderParams.m_nCBSize = GetOptimalOutputSize () * 8; 
	decoderParams.m_bRequiresFullAUs = TRUE;
	decoderParams.m_nStreamNumber = m_EsNum;

#if BGR_OUTPUT
	decoderParams.m_format = MediaFormat::BGR;
#else
	decoderParams.m_format = MediaFormat::YUV;
#endif
	return TRUE;
}



/*-----------------------------------------------------------------
Calculate the output size in bytes
-----------------------------------------------------------------*/
int XVIDDec :: GetOptimalOutputSize ()
{
	return m_OutSize;
}

/*-------------------------------------------------------------
Decode one unit
-------------------------------------------------------------*/
BOOL XVIDDec::Decode(LPBYTE pInput, int nInputLength, LPBYTE pOutput, int &rnOutputLength)
{
    //check we are given, enough space
	if (rnOutputLength < m_OutSize) {
		rnOutputLength = m_OutSize;
		return FALSE;
	}

	XVID_DEC_FRAME frame;
	memset(&frame, 0, sizeof(XVID_DEC_FRAME));
	frame.bitstream = (void *) pInput;
	frame.length = nInputLength;

	frame.colorspace = XVID_CSP_I420;
	frame.stride = m_Width;
#if BGR_OUTPUT
	frame.image = (void *) m_pConv;
#else
	frame.image = (void *) pOutput;
#endif
	int ret = xvid_decore(m_pCodec, XVID_DEC_DECODE, &frame, NULL);
	assert(ret==XVID_ERR_OK);

#if BGR_OUTPUT
	yuv2rgb_24(pOutput, m_Width, m_pConv, m_pConv + m_Width * m_Height, m_pConv + 5 * m_Width*m_Height / 4, 
				 m_Width, m_Width/2, m_Width, m_Height);
#endif
	
	rnOutputLength = m_OutSize;
	return TRUE;
}

/*-------------------------------------------------------------
In this decoder, the decoded data always strats at beginning of output
block
-------------------------------------------------------------*/
int XVIDDec :: GetDataOffset  (const LPBYTE)
{
	return 0;
}

/*-------------------------------------------------------------
In this decoder, attributes are fixed for all units
-------------------------------------------------------------*/
DWORD XVIDDec :: GetAttributes (
	ATTRIBUTE_CODE code, 
	const LPBYTE
)
{
	switch (code) {
	case FRAME_WIDTH:
		return m_Width;
	case FRAME_HEIGHT:
		return m_Height;
	//pitch is image width
	case PITCH:
#if BGR_OUTPUT
		return m_Width*3;
#else
		return m_Width;
#endif
	case OFFSETX:
		return 0;
	case OFFSETY:
		return 0;
	default:
		return 0;
	}
}

