/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

#if !defined(_IPMPImp_h)	// Sentry, use file only if it's not already included.
#define _IPMPImp_h

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  IPMP
FILE:       IPMPImp.h
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Definition of IPMPManagerImp, a base class implementation of
			IPMP sub-system objects
-------------------------------------------------------------------------*/
#include "IPMP.h"
#include "ztl.h"

class IPMPManagerImp : public IPMPManager
{
private:
	/*-------------------------------------------------------------------
	Interface methods
	-------------------------------------------------------------*/
	virtual void Start ();
	virtual void Stop ();
	virtual void QueryInterface (const char*, const void*, void*& r_pSink) {r_pSink = NULL;}
	virtual const char* GetInfo () {return NULL;}
	virtual void SetIPMPStream (
		MediaStream *pStream,
		int,					// nProfileIndication,
		const BYTE *,			// pSpecificInfo,
		int						// nSpecificInfoLength
	)
	{
		m_pIPMPStream = pStream;
	}
	virtual bool Init () {return true;}
	virtual bool IsMyType (int) const {return true;}
	virtual bool GetAccess () {return true;}

protected:
	/*-------------------------------------------------------------
	Protected functions to be used by derived objects
	-------------------------------------------------------------*/
	virtual void Run ();
	virtual void Terminate () {}
	int StartIPMPThread () {return m_ipmpThread.Begin ();}
	void WaitForThread () {m_ipmpThread.Wait ();}

	//added by Andrea Varesio
	void *m_pAssociatedDecoder;


private:
	/*-------------------------------------------------------------
	Private methods, some may be overloaded
	-------------------------------------------------------------*/
	void IPMPThread ();
	virtual bool Decrypt (
		const BYTE *,		//pInput,
		int,				//nInputLength,
		BYTE *,				//pOutput,
		int &,				//rnOutputLength,
		const BYTE *,		//pIPMP,
		int					//nIPMPLength
	) {return false;}

public:
	/*-------------------------------------------------------------
	Constructor and destructor
	-------------------------------------------------------------*/
	IPMPManagerImp ();
	virtual ~IPMPManagerImp ();

	//added by Andrea Varesio
	void SetAssociatedDecoder(void *pDecoder)  {m_pAssociatedDecoder = pDecoder;}

private:
	/*-------------------------------------------------------------
	Member variables
	-------------------------------------------------------------*/
	MediaStream *m_pIPMPStream;
	bool m_bActive;
	ZThread<IPMPManagerImp> m_ipmpThread;

};

#endif                                      // _IPMPImp_h sentry.
