/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  IPMP
FILE:       IPMPNull.cpp
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Implementation of IPMPNull (prototype of IPMP system)
-------------------------------------------------------------------------*/
#include "defs.h"
#include "IPMPNull.h"

/*-------------------------------------------------------------
The exported function that instantiates the proper IPMPManager object
-------------------------------------------------------------*/
void *CreateInstance ()
{
	return new IPMPNull;
}

/*-------------------------------------------------------------
IPMPNull constructor. A place holder.
-------------------------------------------------------------*/
IPMPNull :: IPMPNull ()
{
	printf ("IPMPNull is used\n");
}

/*-------------------------------------------------------------
IPMPNull destructor. A place holder.
-------------------------------------------------------------*/
IPMPNull :: ~IPMPNull ()
{
}

/*-------------------------------------------------------------
Decrypt does null decryption, i.e. it copies the content unit to the
output buffer
-------------------------------------------------------------*/
bool IPMPNull :: Decrypt (
	const BYTE *pInput,
	int nInputLength,
	BYTE *pOutput,
	int &rnOutputLength,
	const BYTE * /*pIPMP*/,
	int /*nIPMPLength*/
)
{
	bool bResult = false;
	if (rnOutputLength >= nInputLength)
	{	memcpy (pOutput, pInput, nInputLength);
		bResult = true;
	}
	rnOutputLength = nInputLength;
	return bResult;
}

/*-------------------------------------------------------------
A place holder for Init method
-------------------------------------------------------------*/
bool IPMPNull :: Init ()
{
	return true;
}

/*-------------------------------------------------------------
A place holder for GetAccess method
-------------------------------------------------------------*/
bool IPMPNull :: GetAccess ()
{
	return true;
}

/*-------------------------------------------------------------
A place holder for SetDescriptor method
-------------------------------------------------------------*/
int IPMPNull :: SetDescriptor (
	int nID,
	WORD wIPMPS_Type,
	const BYTE *pIPMP_data,
	int nDataLength
)
{
	printf (
		"IPMP_Descriptor handled. ID: %d, Type: %d, Data: %.*s\n",
		nID,
		wIPMPS_Type,
		nDataLength,
		pIPMP_data
	);
	return 1;
}

/*-------------------------------------------------------------
A place holder for RemoveDescriptor method
-------------------------------------------------------------*/
void IPMPNull :: RemoveDescriptor (int nID)
{
	printf ("IPMP_Descriptor removed. ID: %d\n", nID);
}
