
//	This software module was originally developed by Massimo Balestri (TILAB)	
//	in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1) standard. 
//	This software module is an implementation of a part of one or more MPEG-4 
//	Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 
//	14496-1) standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	EPFL retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2002.


// BaseCommand.h


#ifndef	BASE_COMMAND_H
#define	BASE_COMMAND_H

#include "ExpandableBaseClass.h"
#include "namespace.h"
BEGIN_MOSES_NAMESPACE
						
//----------- list of tags for classes extending the BaseCommand ---------//
#define Forbidden									0x00
#define	ObjectDescrUpdateTag						0x01
#define	ObjectDescrRemoveTag						0x02
#define	ES_DescrUpdateTag							0x03
#define ES_DescrRemoveTag							0x04	
#define	IPMP_DescrUpdateTag							0x05
#define	IPMP_DescrRemoveTag							0x06
#define	ES_DescrRemoveRefTag						0x07
#define ObjectDescrExecuteTag						0x08
#define IPMP_ToolDescrUpdateTag						0x09
#define IPMP_ToolDescrRemoveTag						0x0A


/** 
 *  @class BaseCommand
 *  @brief This is the base class for all MPEG-4 Commands.
 *
 *  @author Massimo Balestri
 *  @date 2002
 */
class BaseCommand: public ExpandableBaseClass
{
private:
	BaseCommand();		// do not allow creating object of this type from external code.

protected:

	/**
	 * creates an object of a specified type, identified by the tag.
	 * @param tag the object_id of the instance to be created.
	 */
	BaseCommand(byte tag);	

public:

	/**
	 * destructor.
	 */
	virtual ~BaseCommand();		

	/**
	 * get the total size of the class,when converted into a bitstream. 
	 * This size does NOT include the object_id and the length of the base class. 
	 * @return the size in bytes. 
	 */
	virtual unsigned int sizeOfInstance() const;

	/**
	 * Creates an instance from a binary message. The caller must delete the object when
	 * not used anymore.
	 * @param message the binary message
	 */
	static BaseCommand * createMessage(const byte * message);

};

END_MOSES_NAMESPACE

#endif

