
//	This software module was originally developed by Massimo Balestri (TILAB)	
//	in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1) standard. 
//	This software module is an implementation of a part of one or more MPEG-4 
//	Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 
//	14496-1) standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	TILAB retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2002.


// BaseDescriptor.h


#ifndef	BASE_DESCRIPTOR_H
#define	BASE_DESCRIPTOR_H

#include "ExpandableBaseClass.h"
#include "namespace.h"		

BEGIN_MOSES_NAMESPACE
				
//----------- list of tags for classes extending the BaseDescriptor ---------//
#define Forbidden									0x00
#define ObjectDescrTag								0x01
#define InitialObjectDescrTag						0x02
#define ES_DescrTag									0x03
#define DecoderConfigDescrTag						0x04
#define DecSpecificInfoTag							0x05
#define SLConfigDescrTag							0x06
#define ContentIdentDescrTag						0x07
#define SupplContentIdentDescrTag					0x08
#define IPI_DescrPointerTag							0x09
#define IPMP_DescrPointerTag						0x0A
#define IPMP_DescrTag								0x0B
#define QoS_DescrTag								0x0C
#define RegistrationDescrTag						0x0D
#define ES_ID_IncTag								0x0E
#define ES_ID_RefTag								0x0F
#define MP4_IOD_Tag									0x10
#define MP4_OD_Tag									0x11
#define IPI_DescrPointerRefTag						0x12
#define ExtendedProfileLevelDescrTag				0x13
#define profileLevelIndicationIndexDescrTag			0x14
#define ContentClassificationDescrTag				0x40
#define KeyWordDescrTag								0x41
#define RatingDescrTag								0x42
#define LanguageDescrTag							0x43
#define ShortTextualDescrTag						0x44
#define ExpandedTextualDescrTag						0x45
#define ContentCreatorNameDescrTag					0x46
#define ContentCreationDateDescrTag					0x47
#define OCICreatorNameDescrTag						0x48
#define OCICreationDateDescrTag						0x49
#define SmpteCameraPositionDescrTag					0x4A
#define IPMPToolES_DecoderConfigTag					0x05

// new tags defined in IPMPX
#define IPMP_ToolListDescrTag						0x60
#define IPMP_ToolTag								0x61
//#define IPMP_ToolParamDescrTag					0x62  // not in the spec anymore (16/9/2003)

/** 
 *  @class BaseDescriptor
 *  @brief This is the base class for all MPEG-4 descriptors.
 *
 *  @author Massimo Balestri
 *  @date 2002
 */
class BaseDescriptor: public ExpandableBaseClass
{
private:
	BaseDescriptor();		// do not allow creating object of this type from external code.

protected:

	/**
	 * creates an object of a specified type, identified by the tag.
	 * @param tag the object_id of the instance to be created.
	 */
	BaseDescriptor(byte tag);	

public:

	/**
	 * destructor.
	 */
	virtual ~BaseDescriptor();		

	/**
	 * get the total size of the class,when converted into a bitstream. 
	 * This size does NOT include the object_id and the length of the base class. 
	 * @return the size in bytes. 
	 */
	virtual unsigned int sizeOfInstance()  const;

	/**
	 * Creates an instance from a binary message. The caller must delete the object when
	 * not used anymore.
	 * @param message the binary message
	 */
	static BaseDescriptor * createMessage(const byte * message);

};

END_MOSES_NAMESPACE

#endif

