
//	This software module was originally developed by Massimo Balestri 
//  (Telecom Italia Lab S.p.A.) in the course of development 
//	of the MPEG-4 Systems (ISO/IEC 14496-1) standard. This software module 
//	is an implementation of a part of one or more MPEG-4 Systems (ISO/IEC 
//	14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1) 
//	standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	Telecom Italia Lab S.p.A retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2000. 


#include "ExpandableBaseClass.h"
#include <assert.h>

USING_MOSES_NAMESPACE


ExpandableBaseClass::ExpandableBaseClass():
    object_id(0)					// forbidden - used in classes that have no tag
{
	syntaxOk = true;
}

ExpandableBaseClass::ExpandableBaseClass(byte objectId):
	object_id(objectId)				// this must be assigned by the derived classes
{		
	syntaxOk = true;
}


ExpandableBaseClass::~ExpandableBaseClass()
{
}



int ExpandableBaseClass::fromBuffer(const byte* buf)
{
	setReadBuffer(buf);

	if (object_id)
	{
		if (object_id == buf[0])				// check the object Id
		{
			skipReading(8);						// the object_id is OK, so skip it.
		}
		else
		{
			syntaxOk = false;			// this is the wrong class to read this bitstream!
			assert(false);				// trap the error in debug mode
			return 0;
		}
	}
	
	skipReading( len.fromBuffer( getReadPtr() ));	// read the length
	return getReadCounter();
}



int ExpandableBaseClass::toBuffer(byte* buf) 
{
	if (object_id)
		put(object_id, buf, 0, 8);				//writes IPMP_MessageType in the first position of the buffer.
	else
		setWriteBuffer(buf);

	len.length = sizeOfInstance();				// copy the size into the Length class
	skipWriting( len.toBuffer(getWritePtr()) ); // skip what is already written by len.toBuffer
	return getWriteCounter();
}



byte ExpandableBaseClass::getObjectId() const
{
	return object_id;
}


bool ExpandableBaseClass::checkSyntax() const
{
	if (!syntaxOk) 
		return false;
	
	return len.checkSyntax();
}

unsigned int ExpandableBaseClass::getSizeOfInstance() const
{
	return len.length;
}

void ExpandableBaseClass::setSizeOfInstance()
{
	len.length = sizeOfInstance();				// copy the computed size into the Length member var
}

unsigned int ExpandableBaseClass::getSize() const
{
	unsigned int size = sizeOfInstance();	// compute the size of the payload

	Length mylen;							// temporary length 
	mylen.length = size;					// assign current size

	if (object_id)
		size++;								// object_id 

	size += mylen.getSize();				// add the lenght size (1..4 bytes)
	return size;				
}


void ExpandableBaseClass::setObjectId(byte tag)
{
	object_id = tag;
}
