
#ifndef __HEXUTILS_H__
#define __HEXUTILS_H__

#include <ctype.h>

/**
 * @class HexUtils
 * Class containing static methods to convert from hex to binary and vice versa.
 * @author Massimo Balestri
 * @date 2003
 */
class HexUtils
{
private:
	static char tohex(int v)
	{
		return (char) (v + ((v <= 0x09) ? '0' : ('A' - 0x0A)));
	}
	
	static void btoa(unsigned char b, char * pa)
	{
		pa[0] = tohex((b >> 4));
		pa[1] = tohex((b & 0x0F));
	}

	static int fromhex(int v)
	{
		return (v - ((v <= '9') ? '0' : ('A' - 0x0A)));
	}
	
	static unsigned char atob(const char * pa)
	{
		return (unsigned char) ((fromhex(toupper(pa[0])) << 4) | fromhex(toupper(pa[1])));
	}

public:
	
	/**
	 * Convert from hexadecimal to binary the specified number of bytes.
	 * @param buf the binary buffer
	 * @param size the number of bytes to convert
	 * @param str the hexadecimal string of characters
	 * @return the number of bytes converted if successful, a negative number otherwise.
	 */
	static int fromhex(unsigned char * buf, unsigned int size, const char * str)
	{
		if (str == NULL)
			return -1;

		for (unsigned int i = 0; i < size; i++) 
		{
			buf[i] = atob(str);
			str += 2;
		}

		return size;
	}
	
	/**
	 * Convert from binary to hexadecimal the specified number of bytes.
	 * @param buf the binary buffer
	 * @param size the number of bytes to convert
	 * @param str the hexadecimal string of characters
	 * @return the number of bytes converted if successful, a negative number otherwise.
	 */
	static int tohex(const unsigned char * buf, unsigned int size, char * str)
	{
		if (str == NULL)
			return -1;
		
		for (unsigned int i = 0; i<size; i++)
		{
			btoa(buf[i], str + 2*i);
		}
		
		str[2*size] = 0;	// add a terminating null character

		return size;
	}

};

#endif
