
//	This software module was originally developed by James King (CRL), 
//  Leon Sucharov (CRL) and edited by Massimo Balestri (Telecom
//	Italia Lab S.p.A.) in the course of development of the MPEG-4 Systems 
//	(ISO/IEC 14496-1) standard. This software module is an implementation 
//	of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified 
//	by the MPEG-4 Systems (ISO/IEC 14496-1) standard. 
////
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	CRL retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2000. 
//
//
//
//	Version developed by James King (CRL), Leon Sucharov (CRL) on behalf of MOSES
//	More information on MOSES can be found at http://www.crl.co.uk/projects/moses/


// IPMPToolES_DecoderConfig.cpp: implementation of the IPMPToolES_DecoderConfig class.
//
//////////////////////////////////////////////////////////////////////

#include "IPMPToolES_DecoderConfig.h"
#include <memory.h>

USING_MOSES_NAMESPACE

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

IPMPToolES_DecoderConfig::IPMPToolES_DecoderConfig()
:DecoderSpecificInfo( )
{
	hasFormatID = false;
	hasPackageID = false;
}

IPMPToolES_DecoderConfig::~IPMPToolES_DecoderConfig()
{

}

void IPMPToolES_DecoderConfig::setTool_Format_ID( unsigned int nFormat )
{
	hasFormatID = true;
	Tool_Format_ID = nFormat ;
}

void IPMPToolES_DecoderConfig::setTool_Package_ID( unsigned short nID )
{
	hasPackageID = true;
	Tool_Package_ID = nID ;
}

unsigned int IPMPToolES_DecoderConfig::getTool_Format_ID( )
{
	return Tool_Format_ID ;
}

unsigned short IPMPToolES_DecoderConfig::getTool_Package_ID( )
{
	return Tool_Package_ID ;
}

int IPMPToolES_DecoderConfig::toBuffer( byte *buf )
{
	BaseDescriptor::toBuffer( buf );
	putObject(toolID);					// IPMP_ToolID

	put(hasFormatID, 1);					// flags
	put(hasPackageID, 1);
	put(RESERVED, 6);						// reserved

	if (hasFormatID)
		put( Tool_Format_ID, 32 );			// Tool_Format_ID

	if (hasPackageID)
		put( Tool_Package_ID, 16 );			// Tool_Package_ID

	return getWriteCounter() ;
}

int IPMPToolES_DecoderConfig::fromBuffer( const byte* buf)
{
	BaseDescriptor::fromBuffer( buf ) ;
	getObject(toolID);

	hasFormatID = (get(1) == 1);
	hasPackageID = (get(1) == 1);
	skipReading(6);						// reserved

	if (hasFormatID)
		Tool_Format_ID = get(32);

	if (hasPackageID)
		Tool_Package_ID = (unsigned short) get( 16 );

	return getReadCounter();
}

unsigned int IPMPToolES_DecoderConfig::sizeOfInstance() const
{
	unsigned int size = BaseDescriptor::sizeOfInstance() + 17;	// base + tool_id + flags

	if (hasFormatID)
		size += 4;

	if (hasPackageID)
		size += 2;
	
	return size;
}

