
//	This software module was originally developed by James King (CRL), 
//  Leon Sucharov (CRL) and edited by Massimo Balestri (Telecom
//	Italia Lab S.p.A.) in the course of development of the MPEG-4 Systems 
//	(ISO/IEC 14496-1) standard. This software module is an implementation 
//	of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified 
//	by the MPEG-4 Systems (ISO/IEC 14496-1) standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	CRL retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2000. 
//
//
//
//	Version developed by James King (CRL), Leon Sucharov (CRL) on behalf of MOSES
//	More information on MOSES can be found at http://www.crl.co.uk/projects/moses/


// IPMPToolES_DecoderConfig.h: interface for the IPMPToolES_DecoderConfig class.
//
//////////////////////////////////////////////////////////////////////

#ifndef __IPMPTOOLES_DECODERCONFIG_H__
#define __IPMPTOOLES_DECODERCONFIG_H__

#include "DecoderSpecificInfo.h"
#include "IPMP_ToolId.h"
#include "namespace.h"

BEGIN_MOSES_NAMESPACE

/**
 * @class IPMPToolES_DecoderConfig
 * This class carries information for IPMP tool Elementary Stream decoders.
 */
class IPMPToolES_DecoderConfig : public DecoderSpecificInfo  
{
private:
	bool hasFormatID;
	bool hasPackageID;
	unsigned int Tool_Format_ID ;
	unsigned short Tool_Package_ID ;

public:
	IPMP_ToolId toolID;					///< the 128 bit ID if the tool carried in this elementary stream.

	IPMPToolES_DecoderConfig();						///< constructor
	virtual ~IPMPToolES_DecoderConfig();			///< destructor

	/**
	 * set the Tool format ID. This is 0x01 for structurally described tools.
	 * Otherwise, the Tool_Format_ID indicates the binary representation of the Tool
	 * and is assigned by a registration authority.
	 * @param nFormat the format ID
	 */
	void setTool_Format_ID( unsigned int nFormat ) ;
	
	/**
	 * set the tool package id, which indicates the package of the Tool, e.g. CAB, Zip, TAR.
	 * Values are assigned by a registration authority.
	 * @param nID the package ID
	 */
	void setTool_Package_ID( unsigned short nID ) ;

	/**
	 * get the Tool format ID. This is 0x01 for structurally described tools.
	 * Otherwise, the Tool_Format_ID indicates the binary representation of the Tool
	 * and is assigned by a registration authority.
	 * @return the format ID
	 */
	unsigned int getTool_Format_ID( );

	/**
	 * get the tool package id, which indicates the package of the Tool, e.g. CAB, Zip, TAR.
	 * Values are assigned by a registration authority.
	 * @return the package ID
	 */
	unsigned short getTool_Package_ID( );

	virtual int fromBuffer(const byte* buf);		// reimplemented 
	virtual int toBuffer(byte* buf);				// reimplemented
	virtual unsigned int sizeOfInstance() const;	// reimplemented
};

END_MOSES_NAMESPACE

#endif 
