
//	This software module was originally developed by Antonio Romeo (EPFL)	
//  and modified by Massimo Balestri (TILAB)
//	in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1) standard. 
//	This software module is an implementation of a part of one or more MPEG-4 
//	Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 
//	14496-1) standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	EPFL retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2002.

// IPMP_AudioWatermarkingInit.cpp

#include "IPMP_AudioWatermarkingInit.h"
#include <stdlib.h>
#include <memory.h>
#include <assert.h>

USING_MOSES_NAMESPACE

IPMP_AudioWatermarkingInit::IPMP_AudioWatermarkingInit():
	IPMP_Data_BaseClass(IPMP_AudioWatermarkingInit_tag)		// set the correct tag
{
	// init all variables to default vaules
	init() ;
}	

IPMP_AudioWatermarkingInit::IPMP_AudioWatermarkingInit(const IPMP_AudioWatermarkingInit &obj):
	IPMP_Data_BaseClass(obj)		// set the correct tag
{
	inputFormat		= obj.inputFormat;
	requiredOp		= obj.requiredOp;
	b_hasOpaqueData	= obj.b_hasOpaqueData;
	nChannels		= obj.nChannels;
	bitPerSample	= obj.bitPerSample;
	frequency		= obj.frequency;
	wmRecipientId	= obj.wmRecipientId;
	wmPayloadLen	= 0;
	wmPayload		= NULL;

	if (obj.wmPayloadLen > 0)
		setwmPayload(obj.wmPayload, obj.wmPayloadLen);

	opaqueDataSize	= 0;
	opaqueData		= NULL;	

	if (obj.opaqueDataSize > 0)
		setOpaqueData(obj.opaqueData, obj.opaqueDataSize);
}


IPMP_AudioWatermarkingInit::~IPMP_AudioWatermarkingInit() 
{
	clearUp() ;
}

void IPMP_AudioWatermarkingInit::clearUp()
{

	if(wmPayloadLen)
		delete[] wmPayload;

	if(b_hasOpaqueData)
		delete[] opaqueData;

}

void IPMP_AudioWatermarkingInit::init() 
{
	inputFormat		= PCM;
	requiredOp		= EXTRACT_WM;
	b_hasOpaqueData	= false;
	nChannels		= 0;
	bitPerSample	= 0;
	frequency		= 0;
	wmPayloadLen	= 0;
	wmPayload		= NULL;
	wmRecipientId	= 0;		// 0 = the terminal
	opaqueDataSize	= 0;
	opaqueData		= NULL;	
}


word32 IPMP_AudioWatermarkingInit::getnChannels() const
{
	if (inputFormat == PCM)
		return nChannels;
	else 
		return 0;
}


word32 IPMP_AudioWatermarkingInit::getbitPerSample() const
{
	if (inputFormat == PCM)
		return bitPerSample;
	else 
		return 0;
}

word32 IPMP_AudioWatermarkingInit::getfrequency() const
{
	if (inputFormat == PCM)
		return frequency;
	else 
		return 0;
}

enum IPMP_AudioWatermarkingInit::RequiredOpTable IPMP_AudioWatermarkingInit::getRequiredOp() const
{
	return (enum RequiredOpTable) requiredOp;
}

word32 IPMP_AudioWatermarkingInit::getwmPayloadLen() const
{
	if ((requiredOp == INSERT_WM) || (requiredOp == REMARK_WM))
		return wmPayloadLen;
	else 
		return 0;
}


byte* IPMP_AudioWatermarkingInit::getwmPayload() const
{
	if ((requiredOp == INSERT_WM) || (requiredOp == REMARK_WM))
		return wmPayload;
	else 
		return 0;
}


bool IPMP_AudioWatermarkingInit::setwmPayload(const byte* buf, word32 len)
{
	requiredOp = 0;
	wmPayloadLen = len;					// set the new length

	if (wmPayload != NULL)				
	{
		delete[] wmPayload;				// free the memory allocated by the old payload
		wmPayload = NULL;
	}

	if(wmPayloadLen > 0)
	{
		wmPayload = new byte [wmPayloadLen];		// allocate memory
		if (wmPayload != NULL)
			memcpy(wmPayload, buf, wmPayloadLen);	// set the new payload
		else 
			return false; 
	}
	return true;
}

word32 IPMP_AudioWatermarkingInit::getwmRecipientId() const
{
	if ((requiredOp == EXTRACT_WM) || (requiredOp == DETECT_COMPRESSION))
		return wmRecipientId;
	else 
		return 0;
}

bool IPMP_AudioWatermarkingInit::hasOpaqueData() const
{
	return hasOpaqueData;
}


word32 IPMP_AudioWatermarkingInit::getOpaqueDataSize() const
{
	if (b_hasOpaqueData)
		return opaqueDataSize;
	else 
		return 0;
}


bool   IPMP_AudioWatermarkingInit::setOpaqueData(const byte* buf, word32 len)
{
					// set the new length

	if (b_hasOpaqueData != NULL)				
	{
		delete[] opaqueData;				// free the memory allocated by the old payload
		opaqueData = NULL;
	}
	b_hasOpaqueData = true;					// now there is some opaque data!
	opaqueDataSize = len;
	if(opaqueDataSize > 0)
	{
		opaqueData = new byte [opaqueDataSize];			// allocate memory
		if (opaqueData != NULL)
			memcpy(opaqueData, buf, opaqueDataSize);	// set the new payload
		else 
			return false; 
	}
	else
		b_hasOpaqueData = false;				// if (size == 0) there is no data!

	return true;
}

byte* IPMP_AudioWatermarkingInit::getOpaqueData() const
{
	if (b_hasOpaqueData)
		return opaqueData;
	else 
		return 0;
}

int IPMP_AudioWatermarkingInit::fromBuffer(const byte * buf)
{
	clearUp(); 
	int nbits = IPMP_Data_BaseClass::fromBuffer(buf);
	assert((nbits % 8) == 0);
	const byte * wmbuf = buf + (nbits/8);			// place pointer to the start of the wm_init 

	inputFormat = get(wmbuf, 0, 8);					// bit address: 0
	requiredOp = get(wmbuf, 8, 4);					// bit address: 8
	b_hasOpaqueData = (1 == get(wmbuf, 12, 1));		// bit address: 12
	long bitindex = 16;								// now, use the bit index.

	if (inputFormat == PCM)
	{
		nChannels = get(wmbuf, bitindex, 8);		// get number of channels
		bitindex +=8;
		bitPerSample = get(wmbuf, bitindex, 8);		// get the num of bit per sample
		bitindex +=8;
		frequency = get(wmbuf, bitindex, 32);		// get the frequency
		bitindex +=32;
	}

	if ((requiredOp == INSERT_WM) || (requiredOp == REMARK_WM))
	{
		word32 len = get(wmbuf, bitindex, 16);		// get the wm payload length
		bitindex += 16;

		setwmPayload(wmbuf + (bitindex/8), len);	// set the new payload 
		bitindex += (len * 8);
	}else{
		wmPayload = NULL ;

	}

	if((requiredOp == EXTRACT_WM) || (requiredOp == DETECT_COMPRESSION))
	{
		wmRecipientId = get(wmbuf, bitindex, 16);
		bitindex += 16;
	}

	if(b_hasOpaqueData)
	{
		word32 size = get(wmbuf, bitindex, 16);		// get the wm payload length
		bitindex += 16;

		setOpaqueData(wmbuf + (bitindex/8), size);
		bitindex += (size * 8);
	}

	return (nbits + bitindex);
}


int IPMP_AudioWatermarkingInit::toBuffer(byte * buf)
{
	int nbits = IPMP_Data_BaseClass::toBuffer(buf);
	assert((nbits % 8) == 0);
	byte * wmbuf = buf + (nbits/8);			// place pointer to the start of the wm_init 

	put(inputFormat, wmbuf, 0, 8); 
	put(requiredOp, wmbuf, 8, 4);
	put(b_hasOpaqueData, wmbuf, 12, 1);
	put(0, wmbuf, 13, 3);							// reserved bits 
	long bitindex = 16;								// now, use the bit index.

	if (inputFormat == PCM)
	{
		put(nChannels, wmbuf, bitindex, 8);			// number of channels
		bitindex +=8;
		put(bitPerSample, wmbuf, bitindex, 8);		// num of bit per sample
		bitindex +=8;
		put(frequency, wmbuf, bitindex, 32);		// frequency
		bitindex +=32;
	}

	if ((requiredOp == INSERT_WM) || (requiredOp == REMARK_WM))
	{
		put(wmPayloadLen,wmbuf, bitindex, 16);		// wm payload length
		bitindex += 16;

		memcpy(wmbuf + (bitindex/8), getwmPayload(), wmPayloadLen);
		bitindex += (wmPayloadLen * 8);
	}

	if((requiredOp == EXTRACT_WM) || (requiredOp == DETECT_COMPRESSION))
	{
		put(wmRecipientId, wmbuf, bitindex, 16);
		bitindex += 16;
	}

	if(b_hasOpaqueData)
	{
		put(opaqueDataSize, wmbuf, bitindex, 16);
		bitindex += 16;
		
		memcpy(wmbuf + (bitindex/8), getOpaqueData(), opaqueDataSize);
		bitindex += (opaqueDataSize * 8);
	}

	return (nbits + bitindex);
}

// compute size, in BYTES!
unsigned int IPMP_AudioWatermarkingInit::sizeOfInstance() const
{
	unsigned int size = IPMP_Data_BaseClass::sizeOfInstance();		// size of the base class 

	// now compute the size of this instance.
	
	size += 2;														// flags
	
	if (inputFormat == PCM)
		size += 6;													// PCM params
	
	if ((requiredOp == INSERT_WM) || (requiredOp == REMARK_WM))
		size += (2 + wmPayloadLen);

	if((requiredOp == EXTRACT_WM) || (requiredOp == DETECT_COMPRESSION))
		size += 2;

	if(b_hasOpaqueData)
		size += (2 + opaqueDataSize);

	return size;
}


bool IPMP_AudioWatermarkingInit::setInputFormat(InputFormatTable format, word32 nchannels, word32 bitpersample, word32 freq)
{
	inputFormat = format;
	nChannels = nchannels;
	bitPerSample = bitpersample;
	frequency = freq;
	return true;
}

bool IPMP_AudioWatermarkingInit::insertWatermark(const byte *payload, word32 size)
{
	requiredOp = INSERT_WM;
	return setwmPayload(payload, size); 
}

bool IPMP_AudioWatermarkingInit::remarkWatermark(const byte *payload, word32 size)
{
	requiredOp = REMARK_WM;
	return setwmPayload(payload, size); 
}

bool IPMP_AudioWatermarkingInit::extractWatermark(word32 recipientId)
{
	requiredOp = EXTRACT_WM;
	wmRecipientId = recipientId;
	return true;
}

bool IPMP_AudioWatermarkingInit::detectCompression(word32 recipientId)
{
	requiredOp = DETECT_COMPRESSION;
	wmRecipientId = recipientId;
	return true;
}


enum IPMP_AudioWatermarkingInit::InputFormatTable IPMP_AudioWatermarkingInit::getInputFormat() const
{
	return (enum InputFormatTable) inputFormat;
}
