
//	This software module was originally developed by Claudio Alberti (EPFL)	
//	in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1) standard. 
//	This software module is an implementation of a part of one or more MPEG-4 
//	Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 
//	14496-1) standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	EPFL retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2002.

// IPMP_ConnectTool.h

#ifndef	IPMP_CONNECTTOOL_H
#define	IPMP_CONNECTTOOL_H

#include "IPMP_Data_BaseClass.h"
#include "IPMP_Descriptor.h"
#include "namespace.h"

BEGIN_MOSES_NAMESPACE

/** @class IPMP_ConnectTool
 *  This message allows a tool to request the Terminal to create a connection to a tool identified in the toolDescriptor.
 *	Scope of connected tool shall be the same as the requesting tool.  
 *  The Terminal shall link the new Tool Context to the Message router, before it responds to this message.
 *  @author Claudio Alberti, Massimo Balestri
 *  @date 2002
 */
class IPMP_ConnectTool: public IPMP_Data_BaseClass 
{

private:

	IPMP_Descriptor ipmpDescriptor;

public:
	/**
	 * constructor.
	 */
	IPMP_ConnectTool();	

	/**
	 * destructor.
	 */
	virtual ~IPMP_ConnectTool();		
	virtual int toBuffer(byte * buf);				// reimplemented	
	virtual int fromBuffer(const byte * buf);		// reimplemented
	virtual unsigned int sizeOfInstance() const;	// reimplemented
	
	/**
	 * Get the IPMP_Descriptor contained in this messge.
	 * @return the IPMP descriptor
	 */
	const IPMP_Descriptor& getIPMP_Descriptor() const;
	
	/**
	 * Set the IPMP descriptor of the Tool to be connected
	 * @param desc pointer to a serialized IPMP descriptor
	 */
	void setIPMP_Descriptor( byte *desc ) ;

	/**
	 * Set the IPMP descriptor of the Tool to be connected
	 * @param desc reference to the IPMP descriptor
	 */
	void setIPMP_Descriptor( IPMP_Descriptor& desc );
};

END_MOSES_NAMESPACE

#endif
