
//	This software module was originally developed by Liao Xuan (PSL)
//	in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1) standard. 
//	This software module is an implementation of a part of one or more MPEG-4 
//	Systems (ISO/IEC 14496-1) deices as specified by the MPEG-4 Systems (ISO/IEC 
//	14496-1) standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	PSL retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2003.


// IPMP_ContentTransfer.cpp

#include "IPMP_ContentTransfer.h"
#include <stdlib.h>
#include <memory.h>
#include <assert.h>
#include <string.h>
#include <conio.h> 
#include "common.h"

USING_MOSES_NAMESPACE

IPMP_ContentTransfer::IPMP_ContentTransfer():
	IPMP_Data_BaseClass(IPMP_ContentTransfer_tag)	// set the correct tag
{
	 init();                                        // initialize the object
}	

IPMP_ContentTransfer::IPMP_ContentTransfer(IPMP_ContentTransfer & obj):
	IPMP_Data_BaseClass(obj),
	Payload(obj.Payload)
{
	Sequence = obj.Sequence;
	ContentID = NULL;
	setContentID(obj.getContentID());
}


IPMP_ContentTransfer::~IPMP_ContentTransfer() 
{
	delete[] ContentID; 	
}

void IPMP_ContentTransfer::init() 
{
	// initialize the object
	ContentID = NULL;
	Sequence = 0;
	Payload.clear();
}

word32 IPMP_ContentTransfer::getSequence() const
{
	return Sequence;
}

word32 IPMP_ContentTransfer::getPayloadSize() const
{
	return Payload.getDataSize();
}


byte* IPMP_ContentTransfer::getPayload() const
{
	byte *p = Payload.getData() ; 
	return p ;
}


byte * IPMP_ContentTransfer::getContentID() const
{
	return ContentID;                           // return the pointer to ContentID
}


void IPMP_ContentTransfer::setContentID(const byte * contentID)
{
	if (ContentID != NULL)
		delete[] ContentID;                     // clear the old ContentID

	ContentID = new byte[16];
	memcpy(ContentID, contentID, 16) ;          // dump the new ContentID
}

void IPMP_ContentTransfer::setSequence(word32 sequence)
{
	Sequence = sequence;
}

void IPMP_ContentTransfer::setPayload(const byte* buf, word32 size)
{
	Payload.setData( buf, size ) ;             // set the payload data
//	PayloadSize = Payload.getDataSize();       // set the payload size
}


int IPMP_ContentTransfer::fromBuffer(const byte * buf)
{
	int nbits = IPMP_Data_BaseClass::fromBuffer(buf);  // read the tag and class size
    assert((nbits % 8) == 0);

	setContentID(getReadPtr());	                       // read the ContentID, 128 bits			
	nbits+=128;

	Sequence = get(buf, nbits, 8);		               // read the sequence number, 8 bits				
    nbits+=8;

	word32 PayloadSize = get(buf, nbits, 32);	       // read the payload size, 32 bits 
	nbits+=32;
    if (PayloadSize)
	{
		setPayload(buf + (nbits/8), PayloadSize);	   // read the payload, 8bits*PayloadSize 
		nbits +=  (PayloadSize * 8) ;
    }
	

	return nbits;
}


int IPMP_ContentTransfer::toBuffer(byte * buf)
{
	int nbits = IPMP_Data_BaseClass::toBuffer(buf);    // write the tag and class size

	memcpy(buf+(nbits/8), ContentID, 16);              // write the ContentID, 128 bits
	nbits+=128 ;

	put(Sequence, buf, nbits, 8);                      // write the payload sequence, 8 bits
	nbits+=8;

	word32 PayloadSize = getPayloadSize();
	
	put(PayloadSize, buf, nbits, 32);                  // write the payload size, 32 bits
	nbits+=32;

	memcpy(buf + (nbits/8), getPayload(), PayloadSize);// write the payload, 8 bits * payloadsize
	nbits += (PayloadSize * 8); 

	return nbits;
}

// in BYTES!
unsigned int IPMP_ContentTransfer::sizeOfInstance() const
{
	unsigned int size = IPMP_Data_BaseClass::sizeOfInstance();	// size of the base class 
	
	size+= (21 + getPayloadSize());    // (16bytes ContentID + 1byte Sequence + 4bytes PayloadSize) + PayloadSize											

	return size;                  // return the size in byte
}

