//	This software module was originally developed by Liao Xuan (PSL)
//	in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1) standard. 
//	This software module is an implementation of a part of one or more MPEG-4 
//	Systems (ISO/IEC 14496-1) deices as specified by the MPEG-4 Systems (ISO/IEC 
//	14496-1) standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	PSL retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2003.

// IPMP_ContentTransfer.h

#ifndef	IPMP_CONTENTTRANSFER_H
#define	IPMP_CONTENTTRANSFER_H

#include "IPMP_Data_BaseClass.h"
#include "namespace.h"
#include "IPMP_ByteArray.h"




BEGIN_MOSES_NAMESPACE

/** 
 *  @class IPMP_ContentTransfer
 *  This message is sent from a Device to another Device to transfer a requested content with the specified ContentID.
 *
 *  @author Liao Xuan
 *  @date 2003
 */ 
class IPMP_ContentTransfer: public IPMP_Data_BaseClass 
{
private:
	byte * ContentID;					        // 128 bits
	word32 Sequence;                            // 8 bits
    IPMP_ByteArray Payload;					    // 8 bits * arraysize

	void init() ;                               // initialize object

public:
	
	IPMP_ContentTransfer();									///< constructor.
	IPMP_ContentTransfer(IPMP_ContentTransfer & obj);		///< copy constructor
	virtual ~IPMP_ContentTransfer();						///< destructor.	
	virtual int toBuffer(byte * buf);						// reimplemented
	virtual int fromBuffer(const byte * buf);				// reimplemented
	virtual unsigned int sizeOfInstance() const;			// reimplemented

	/**
	 * Set the Payload. 
	 * @param buf the pointer to payload buffer
	 * @param size payload size in bytes
	 */
    void setPayload(const byte* buf, word32 size);

	/**
	 * Set the Content ID. 
	 * @param contentID the Content ID pointer of the requested content.
	 */
	void setContentID(const byte * contentID );

	/**
	 * Get the content ID. 
	 * @return ContentID the Content ID pointer of the requested content
	 */
	byte * getContentID() const;

    /**
	 * Set the Sequence. 
	 * @param seq indicate the sequence number of this payload
	 */
	void setSequence( word32 seq );

	/**
	 * Get the Sequence. 
	 * @return the sequence number of this payload
	 */
	word32 getSequence() const;


	/**
	 * Get the size in bytes of the content payload. 
	 * @return PayloadSize the size of this payload
	 */
	word32 getPayloadSize() const;

	/**
	 * Get the content payload. 
	 * @return a pointer to the Payload
	 */
	byte * getPayload() const;



};

END_MOSES_NAMESPACE

#endif
