
//	This software module was originally developed by Joao Trindade  (ADETTI)
//  and modified by Massimo Balestri (TILAB)	
//	in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1) standard. 
//	This software module is an implementation of a part of one or more MPEG-4 
//	Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 
//	14496-1) standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	ADETTI retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2002.
 

// IPMP_Data_BaseClass.cpp

#include "IPMP_Data_BaseClass.h"
#include "IPMP_UserDefinedBase.h"
#include "common.h"
#include <stdlib.h>
#include <assert.h>

USING_MOSES_NAMESPACE

IPMP_Data_BaseClass::IPMP_Data_BaseClass(byte tag):
	ExpandableBaseClass(tag),
	Version(VERSION),
	dataID(0)
{
}

IPMP_Data_BaseClass::IPMP_Data_BaseClass(const IPMP_Data_BaseClass& msg):
	ExpandableBaseClass(msg)
{	
	Version = msg.Version;
	dataID = msg.dataID;
}

IPMP_Data_BaseClass::~IPMP_Data_BaseClass() 
{

}

bool IPMP_Data_BaseClass::checkSyntax() const
{
	if (Version != 0x01)
		return false;					// syntax version must be 0x01 for IPMP_Data_BaseClass messages.
	return ExpandableBaseClass::checkSyntax();
}


void IPMP_Data_BaseClass::setVersion(byte ver)
{
	Version = ver;
}

byte IPMP_Data_BaseClass::getVersion() const
{
	return Version;
}


int IPMP_Data_BaseClass::fromBuffer(const byte * buf)
{

	int nbits = ExpandableBaseClass::fromBuffer(buf);	// read tag and size
	Version = (unsigned char) get(buf, nbits, 8);		// read next 8 bits
	nbits +=8 ;
	dataID = get(buf, nbits, 32) ;
	nbits += 32 ;
	return nbits ;
}


int IPMP_Data_BaseClass::toBuffer(byte * buf)
{
	int nbits = ExpandableBaseClass::toBuffer(buf);		// write tag and size
	put(Version, buf, nbits, 8);						// write into next 8 bits
	nbits+=8 ;
	put( dataID, buf, nbits, 32 ) ;
	nbits+=32 ;
	return nbits ;
}

unsigned int IPMP_Data_BaseClass::sizeOfInstance() const
{
	return 5 ;				// "Version (1 byte) + dataID (4 bytes)"
}

IPMP_Data_BaseClass * IPMP_Data_BaseClass::createMessage(byte tag)
{
	// IF THIS IS A IPMP SYSTEM SPECIFIC MESSAGE THEN...
	if( tag >= 0xD0 ){
		return IPMP_UserDefinedBase::createMessage(tag);
	}

	IPMP_Data_BaseClass * pNewMessage = NULL;

    switch (tag)
    {
    case IPMP_OpaqueData_tag:
        pNewMessage=new IPMP_OpaqueData();
        break;
    
    case IPMP_AudioWatermarkingInit_tag:
        pNewMessage=new IPMP_AudioWatermarkingInit();
        break;
    
    case IPMP_VideoWatermarkingInit_tag:
        pNewMessage=new IPMP_VideoWatermarkingInit();
        break;
    
    case IPMP_SelectiveDecryptionInit_tag:
        pNewMessage=new IPMP_SelectiveDecryptionInit();
        break;

	case IPMP_KeyData_tag:
		pNewMessage = new IPMP_KeyData();
		break;

    case IPMP_SendAudioWatermark_tag:
        pNewMessage=new IPMP_SendAudioWatermark();
        break;
		
	case IPMP_SendVideoWatermark_tag:
		pNewMessage=new IPMP_SendVideoWatermark();
		break;

	case IPMP_RightsData_tag:
		pNewMessage = new IPMP_RightsData();
		break;

	case IPMP_Secure_Container_tag:
		pNewMessage = new IPMP_SecureContainer();
		break;	

	case IPMP_AddToolNotificationListener_tag:
		pNewMessage = new IPMP_AddToolNotificationListener();
		break;

	case IPMP_RemoveToolNotificationListener_tag:
		pNewMessage = new IPMP_RemoveToolNotificationListener();
		break;

    case IPMP_InitAuthentication_tag:
        pNewMessage=new IPMP_InitAuthentication();
        break;		
		
	case IPMP_MutualAuthentication_tag:
		pNewMessage = new IPMP_MutualAuthentication();
		break;
		
	case IPMP_UserQuery_tag:
		pNewMessage = new IPMP_UserQuery();
		break;
		
	case IPMP_UserQueryResponse_tag:
		pNewMessage = new IPMP_UserQueryResponse();
		break;
		
	case IPMP_ToolParamCapabilitiesQuery_tag:
		pNewMessage = new IPMP_ToolParamCapabilitiesQuery();
		break;
		
	case IPMP_ToolParamCapabilitiesResponse_tag:
		pNewMessage = new IPMP_ToolParamCapabilitiesResponse();
		break;
		
	case IPMP_GetTools_tag:
		pNewMessage = new IPMP_GetTools();
		break;
		
	case IPMP_GetToolsResponse_tag:
		pNewMessage = new IPMP_GetToolsResponse();
		break;
		
	case IPMP_GetToolContext_tag:
		pNewMessage = new IPMP_GetToolContext();
		break;
		
	case IPMP_GetToolContextResponse_tag:
		pNewMessage = new IPMP_GetToolContextResponse();
		break;
		
	case IPMP_ConnectTool_tag:
		pNewMessage = new IPMP_ConnectTool();
		break;
		
	case IPMP_DisconnectTool_tag:
		pNewMessage = new IPMP_DisconnectTool();
		break;
		
	case IPMP_NotifyToolEvent_tag:
		pNewMessage = new IPMP_NotifyToolEvent();
		break;
		
	case IPMP_CanProcess_tag:
		pNewMessage = new IPMP_CanProcess();
		break;	
	
	case IPMP_TrustSecurityMetadata_tag:
		pNewMessage = new IPMP_TrustSecurityMetadata();
		break;	

	case IPMP_ParametricDescription_tag:
		pNewMessage = new IPMP_ParametricDescription();
		break;

	case IPMP_ToolAPI_Config_tag:
		pNewMessage = new IPMP_ToolAPI_Config();
		break;

	// inter-device messages	
	case IPMP_RequestContent_tag:
		pNewMessage = new IPMP_RequestContent;
		break;
		
	case IPMP_ResponseToContentRequest_tag:
		pNewMessage = new IPMP_ResponseToContentRequest;
		break;

	case IPMP_ContentTransfer_tag:
		pNewMessage = new IPMP_ContentTransfer;
		break;

	case IPMP_RequestTool_tag:
		pNewMessage = new IPMP_RequestTool;
		break;

	case IPMP_ResponseToToolRequest_tag:
		pNewMessage = new IPMP_ResponseToToolRequest;
		break;

	case IPMP_DeviceID_Notification_tag:
		pNewMessage = new IPMP_DeviceID_Notification;
		break;
		
	default: break;
    }

	// ENSURE A MESSAGE IS CREATED OTHERWISE IT IS NOT CURRENTLY HANDLED
	assert(pNewMessage);

	return pNewMessage;
}


IPMP_Data_BaseClass * IPMP_Data_BaseClass::createMessage(const byte * message, unsigned int & nBitRead )
{
	IPMP_Data_BaseClass * pNewMessage = createMessage(message[0]);
	
	if (pNewMessage != NULL)
	{
		nBitRead = pNewMessage->fromBuffer(message);
	}
	return pNewMessage;
}

IPMP_Data_BaseClass * IPMP_Data_BaseClass::createMessage(IPMP_Data_BaseClass &object)
{
	unsigned int size = object.getSize();
	byte * buffer = new byte[size + 1];			
	object.toBuffer(buffer);						// dump data into a buffer
	unsigned int nbitsRead;
	IPMP_Data_BaseClass * newobj = createMessage(buffer, nbitsRead);
	delete[] buffer;
	return newobj;
}

void IPMP_Data_BaseClass::setDataId(unsigned int id)
{
	dataID = id;
}

unsigned int IPMP_Data_BaseClass::getDataId() const
{
	return dataID;
}

