
//	This software module was originally developed by Claudio Alberti (EPFL)	
//  and modified by Massimo Balestri (TILAB)
//	in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1) standard. 
//	This software module is an implementation of a part of one or more MPEG-4 
//	Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 
//	14496-1) standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	EPFL retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2002.


// IPMP_Data_BaseClass.h


#ifndef	IPMP_DATA_BASECLASS_H
#define	IPMP_DATA_BASECLASS_H

#include "ExpandableBaseClass.h"
#include "namespace.h"

BEGIN_MOSES_NAMESPACE


#define VERSION		0x01		
						
//----------- list of tags for classes extending the IPMP_Data_BaseClass ---------//
#define Forbidden									0x00
#define IPMP_OpaqueData_tag							0x01
#define IPMP_AudioWatermarkingInit_tag				0x02
#define IPMP_VideoWatermarkingInit_tag				0x03
#define IPMP_SelectiveDecryptionInit_tag			0x04
#define IPMP_KeyData_tag							0x05
#define IPMP_SendAudioWatermark_tag					0x06
#define IPMP_SendVideoWatermark_tag					0x07
#define IPMP_RightsData_tag							0x08
#define IPMP_Secure_Container_tag					0x09
#define IPMP_AddToolNotificationListener_tag		0x0A
#define IPMP_RemoveToolNotificationListener_tag		0x0B
#define IPMP_InitAuthentication_tag					0x0C
#define IPMP_MutualAuthentication_tag				0x0D
#define IPMP_UserQuery_tag							0x0E
#define IPMP_UserQueryResponse_tag					0x0F
#define IPMP_ParametricDescription_tag				0x10	
#define IPMP_ToolParamCapabilitiesQuery_tag			0x11
#define IPMP_ToolParamCapabilitiesResponse_tag		0x12
#define IPMP_GetTools_tag							0x13
#define IPMP_GetToolsResponse_tag					0x14
#define IPMP_GetToolContext_tag						0x15
#define IPMP_GetToolContextResponse_tag				0x16
#define IPMP_ConnectTool_tag						0x17
#define IPMP_DisconnectTool_tag						0x18
#define IPMP_NotifyToolEvent_tag					0x19
#define IPMP_CanProcess_tag							0x1A
#define IPMP_TrustSecurityMetadata_tag				0x1B
#define IPMP_ToolAPI_Config_tag						0x1C	


//Tags for device messages extending IPMP_Data_BaseClass
#define IPMP_RequestContent_tag				        0x1D
#define IPMP_ResponseToContentRequest_tag			0x1E
#define IPMP_ContentTransfer_tag				    0x1F
#define IPMP_RequestTool_tag				        0x20
#define IPMP_ResponseToToolRequest_tag				0x21
#define IPMP_DeviceID_Notification_tag              0x22
/** 
 * @class IPMP_Data_BaseClass
 * This class is used to carry IPMP data in the bitstream or from one Tool to another.
 * When the IPMP data is in the bitstream, it can be carried in IPMP_Descriptor's
 * or IPMP_Message's. When the IPMP data is exchanged between Tools, it is carried in 
 * IPMP_MessageFromTool.
 *
 * @see IPMP_Descriptor, IPMP_Message, IPMP_MessageFromTool.
 * @author Claudio Alberti, Massimo Balestri
 * @date 2002
 */
class IPMP_Data_BaseClass: public ExpandableBaseClass
{
private:
	byte Version;				// the version of syntax used - must be 0x01
	unsigned int dataID ;		// the ID of the message
	IPMP_Data_BaseClass();		// do not allow creating object of this type from external code.

protected:
	/**
	 * method to set the version of syntax used (for IPMP Extension this must be 0x01)
	 * @param ver the version number to set
	 */
	void setVersion(byte ver);

	/**
	 * creates an object of a specified type, identified by the tag.
	 * @param tag the object_id of the instance to be created.
	 */
	IPMP_Data_BaseClass(byte tag);	
	
	/**
	 *	Copy constructor.
	 *	Creates a copy of the object given as input parameter
	 *  @param msg the object to copy
	 */
	IPMP_Data_BaseClass(const IPMP_Data_BaseClass& msg);

public:
	virtual ~IPMP_Data_BaseClass();						///< destructor
	virtual int toBuffer(byte * buf);					// reimplemented
	virtual int fromBuffer(const byte * buf);			// reimplemented
	virtual bool checkSyntax() const;					// reimplemented
	virtual unsigned int sizeOfInstance() const;		// reimplemented

	/**
	 * function that should be overloaded by the derived class if that message class is MOSES specific
	 * @return pointer to a new copy of this class.
	 */

	/**
	 * method to get access to the version of syntax of this message (must be 0x01).
	 * @return the syntax version number
	 */
	byte getVersion() const;

	/**
	 * This static "factory" method creates and initialize an object derived 
	 * from this class from the binary message given in input.
	 * The caller must delete the object when not used anymore.
	 * @param message the binary message from which the object is created.
	 * @param nBitRead returns the number of bits read from "message".
	 * @return a pointer to an initialized object constructed from the bitstream
	 */
	static IPMP_Data_BaseClass * createMessage(const byte * message, unsigned int & nBitRead);
	
	/**
	 * This static "factory" method creates an empy object derived 
	 * from this class using the tag given in input.
	 * The caller must delete the object when not used anymore.
	 * @param tag the object tag
	 * @return a pointer to an empty object with the same tag
	 */
	static IPMP_Data_BaseClass * createMessage(byte tag);

	/**
	 * This static "factory" method creates and initialize an object derived 
	 * from this class from the object in input.
	 * The caller must delete the object when not used anymore.
	 * @param object the object to clone
	 * @return a pointer to an initialized object of the same type
	 */
	static IPMP_Data_BaseClass * createMessage(IPMP_Data_BaseClass &object);
	
	/**
	 * Returns the Data ID associated to this message.
	 * This is a unique identifier that can be used to match queries and replies in 
	 * a communication session between two tools.
	 * @return the data ID
	 */
	unsigned int getDataId() const;

	/**
	 * Set the data ID of this message.
	 * This is a unique identifier that can be used to match queries and replies in 
	 * a communication session between two tools.
	 * @param dataId the value to be assigned to the data id field of the message.
	 */
	void setDataId(unsigned int dataId);

};

END_MOSES_NAMESPACE

#endif

