
//	This software module was originally developed by James King (CRL), 
//  Leon Sucharov (CRL) and edited by Massimo Balestri (Telecom
//	Italia Lab S.p.A.) in the course of development of the MPEG-4 Systems 
//	(ISO/IEC 14496-1) standard. This software module is an implementation 
//	of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified 
//	by the MPEG-4 Systems (ISO/IEC 14496-1) standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	CRL retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2000. 
//
//
//
//	Version developed by James King (CRL), Leon Sucharov (CRL) on behalf of MOSES
//	More information on MOSES can be found at http://www.crl.co.uk/projects/moses/
//
//
// IPMP_DescriptorPointer.h: interface for the IPMP_DescriptorPointer class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_IPMP_DESCRIPTORPOINTER_H__9DBBACFB_4167_4D64_BC20_1B2B83D44F10__INCLUDED_)
#define AFX_IPMP_DESCRIPTORPOINTER_H__9DBBACFB_4167_4D64_BC20_1B2B83D44F10__INCLUDED_

#include "BaseDescriptor.h"
#include "namespace.h"

BEGIN_MOSES_NAMESPACE

/** 
 *  @class IPMP_DescriptorPointer
 *  The IPMP_DescriptorPointer appears in the ipmpDescPtr section of an OD or ESD structures. 
 *  The presence of this descriptor in an object descriptor indicates that all streams referred to by embedded ES_Descriptors are subject to protection and management by the IPMP Tool specified in the referenced IPMP_Descriptor.
 *  The presence of this descriptor in an ES_Descriptor indicates that the stream associated with this descriptor is subject to protection and management by the IPMP Tool specified in the referenced IPMP_Descriptor or IPMP_Descriptor.
 *  IPMP_ES_ID is the id of an IPMP stream that may carry messages intended to the tool pointed to by IPMP_DescriptorID. In case more than one IPMP stream is needed to feed the IPMP tool, several IPMP_DescriptorPointer can be given with 
 *  the same IPMP_DescriptorID and different IPMP_ES_ID. If the IPMP_ES_ID is null, it means the IPMP tool does not require an IPMP stream. A value of 2^16-1 for IPMP_ES_ID indicates that this field should be ignored, meaning that the tool 
 *  pointed to by IPMP_DescriptorID may receive messages from any IPMP stream within the presentation.
 *  
 */
class IPMP_DescriptorPointer : public BaseDescriptor  
{
public:
enum DescrTypeEnum {
	HOOKS_TYPE		= 0,		///< indicates that this descriptor contains "hooks" data
	IPMPX_TYPE		= 1,		///< indicates that this descriptor contains IPMPX data
};

private:
	unsigned int	IPMP_DescriptorID;
	unsigned int	IPMP_ES_ID ;
	DescrTypeEnum		descrType;		// identifies the type of this IPMP descriptor pointer
public:

	IPMP_DescriptorPointer();			///< constructor
	virtual ~IPMP_DescriptorPointer();	///< destructor
	virtual int toBuffer(byte * buf);				// reimplemented (same documentation)		
	virtual int fromBuffer(const byte* buf);		// reimplemented
	virtual unsigned int sizeOfInstance() const;	// reimplemented

	/**
	 * Get the current type of the descriptor.
	 * @return an enum indicating the type. Can be IPMPX_TYPE, HOOKS_TYPE.
	 */
	DescrTypeEnum getType() const;
	
	/**
	 * set the ID of the descriptor referenced by this "pointer" message.
	 * This ID identifies uniquely an IPMP_Descriptor within its name scope. 
	 * Values of 0x0000 and 0xFFFF are forbidden. 
	 * @param id the descriptor ID 
	 */	
	void setIPMP_DescriptorID(unsigned short id);

	/**
	 * get the ID of the descriptor referenced by this "pointer" message.
	 * This ID identifies uniquely an IPMP_Descriptor within its name scope. 
	 * @return the descriptor ID 
	 */	
	unsigned short getIPMP_DescriptorID() const;
	
	/**
	 * get the IPMP_ES_ID. 
	 * This is the id of an IPMP stream that may carry messages intended to the tool 
	 * pointed to by IPMP_DescriptorID. In case more than one IPMP stream is needed 
	 * to feed the IPMP tool, several IPMP_DescriptorPointer can be given with the same 
	 * IPMP_DescriptorID and different IPMP_ES_ID. If the IPMP_ES_ID is null, it means 
	 * the IPMP tool does not require an IPMP stream. A value of 2^16-1 for IPMP_ES_ID 
	 * indicates that this field should be ignored, meaning that the tool pointed to 
	 * by IPMP_DescriptorID may receive messages from any IPMP stream within the presentation.
	 * @return the IPMP_ES_ID
	 */	
	unsigned short getIPMP_ES_ID();
	
	/**
	 * set the IPMP_ES_ID. 
	 * This is the id of an IPMP stream that may carry messages intended to the tool 
	 * pointed to by IPMP_DescriptorID. In case more than one IPMP stream is needed 
	 * to feed the IPMP tool, several IPMP_DescriptorPointer can be given with the same 
	 * IPMP_DescriptorID and different IPMP_ES_ID. If the IPMP_ES_ID is null, it means 
	 * the IPMP tool does not require an IPMP stream. A value of 2^16-1 for IPMP_ES_ID 
	 * indicates that this field should be ignored, meaning that the tool pointed to 
	 * by IPMP_DescriptorID may receive messages from any IPMP stream within the presentation.
	 * @param es_id the IPMP_ES_ID
	 */	
	void setIPMP_ES_ID(unsigned short es_id);
};

END_MOSES_NAMESPACE
#endif // !defined(AFX_IPMP_DESCRIPTORPOINTER_H__9DBBACFB_4167_4D64_BC20_1B2B83D44F10__INCLUDED_)
