
//	This software module was originally developed by Massimo Balestri (TILAB)
//	in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1) standard. 
//	This software module is an implementation of a part of one or more MPEG-4 
//	Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 
//	14496-1) standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	TILAB retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2002.


// IPMP_ToolDescriptorUpdate.cpp

#include "IPMP_DescriptorUpdate.h"
#include "IPMP_Descriptor.h"
#include <assert.h>
#include <stdlib.h>
#include <memory.h>

USING_MOSES_NAMESPACE

// constructor
IPMP_DescriptorUpdate::IPMP_DescriptorUpdate():
	BaseCommand(IPMP_ToolDescrUpdateTag)			// set the correct tag
{
	ipmp_ToolDescr = NULL;
	num = 0;
}	

// copy constructor
IPMP_DescriptorUpdate::IPMP_DescriptorUpdate(const IPMP_DescriptorUpdate &obj):
	BaseCommand(obj)			// set the correct tag
{
	ipmp_ToolDescr = NULL;
	num = 0;
	for (unsigned int i = 0; i < obj.getDescriptorNum(); i++)
		addDescriptor( *obj.getDescriptor(i));
}

IPMP_DescriptorUpdate::~IPMP_DescriptorUpdate() 
{
	cleanup();			// delete all
}


int IPMP_DescriptorUpdate::fromBuffer(const byte * buf)
{
	BaseCommand::fromBuffer(buf);			// read the base class data and set "buf" as read buffer for this class.

	unsigned int size = getSizeOfInstance();	// get the size as read from the bitstream
	unsigned int currentsize = 0;

	cleanup();
	ipmp_ToolDescr = new IPMP_Descriptor * [256];

	while (currentsize < size)
	{
		ipmp_ToolDescr[num] = new IPMP_Descriptor();
		skipReading( ipmp_ToolDescr[num]->fromBuffer(getReadPtr()) );
		num++;
		currentsize = getReadCounter() / 8;
	}
	return getReadCounter();				// return the number of read bits
}


int IPMP_DescriptorUpdate::toBuffer(byte * buf)
{
	BaseCommand::toBuffer(buf);			// write the base descriptor and set "buf" as read buffer for this class

	for (unsigned int i=0; i<num; i++)
	{
		skipWriting( ipmp_ToolDescr[i]->toBuffer(getWritePtr()) );
	}
	return getWriteCounter();				// return the number of bits written into the buffer
}



unsigned int IPMP_DescriptorUpdate::sizeOfInstance() const
{
	unsigned int size = 0;
	for (unsigned int i=0; i<num; i++)
	{
		size += ipmp_ToolDescr[i]->getSize();
	}

	return (BaseCommand::sizeOfInstance() + size);		// size of the base class + size of this class
}


void IPMP_DescriptorUpdate::cleanup()
{
	if (num > 0)
	{
		for (unsigned int i=0; i<num; i++)
			delete ipmp_ToolDescr[i];

		delete[] ipmp_ToolDescr;
	}
	ipmp_ToolDescr = NULL;
	num = 0;
}

bool IPMP_DescriptorUpdate::addDescriptor(const IPMP_Descriptor & descr)
{
	if (num == 0)
		ipmp_ToolDescr = new IPMP_Descriptor * [256];

	if (num > 255)
		return false;

	ipmp_ToolDescr[num++] = new IPMP_Descriptor(descr);		// use copy constructor
	return true;
}


IPMP_Descriptor * IPMP_DescriptorUpdate::getDescriptor(unsigned int index) const
{
	if (index < num)
		return ipmp_ToolDescr[index];
	else
		return NULL;
}

unsigned int IPMP_DescriptorUpdate::getDescriptorNum() const
{
	return num;
}


