
//	This software module was originally developed by Massimo Balestri (TILAB)
//	in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1) standard. 
//	This software module is an implementation of a part of one or more MPEG-4 
//	Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 
//	14496-1) standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	Telecom Italia Lab retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2002.

// IPMP_ToolDescriptorUpdate.h

#ifndef	__IPMP_DESCRIPTORUPDATE_H__
#define	__IPMP_DESCRIPTORUPDATE_H__

#include "BaseCommand.h"
#include "namespace.h"
BEGIN_MOSES_NAMESPACE


class IPMP_Descriptor;

/** 
 *  @class IPMP_DescriptorUpdate
 *  @brief this class conveys a list of new or updated IPMP_Descriptor's
 *  
 *  @author Massimo Balestri
 *  @date 2002
 */
class IPMP_DescriptorUpdate: public BaseCommand 
{
private:
	IPMP_Descriptor** ipmp_ToolDescr;				// the vector of pointers to the IPMP Tool descriptors
	unsigned int num;								// number of elements in the list

	void cleanup();									// free memory and restore the initial situation

public:
	IPMP_DescriptorUpdate();									///< constructor	
	IPMP_DescriptorUpdate(const IPMP_DescriptorUpdate & obj);	///< copy constructor
	virtual ~IPMP_DescriptorUpdate();							///< destructor	
	virtual int toBuffer(byte * buf);				// reimplemented	
	virtual int fromBuffer(const byte * buf);		// reimplemented
	virtual unsigned int sizeOfInstance() const;	// reimplemented

	/**
	 * add a descriptor to the descriptor list
	 * (the descriptor is added at the end of the list).
	 * @param descr the descriptor to be copied into the list. 
	 */
	bool addDescriptor(const IPMP_Descriptor & descr);
	
	/**
	 * Get the number of IPMP_Descriptor's contained in this object.
	 * @return the descriptor count.
	 */
	unsigned int getDescriptorNum() const;
	
	/**
	 * Get the n-th IPMP_Descriptor from the descriptor list.
	 * @param index the index of the IPMP_Descriptor in the list.
	 * @return if index is in range, a pointer to the IPMP_Descriptor; NULL otherwise.
	 */
	IPMP_Descriptor * getDescriptor(unsigned int index) const;

};

END_MOSES_NAMESPACE

#endif
