//	This software module was originally developed by Liao Xuan (PSL)
//	in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1) standard. 
//	This software module is an implementation of a part of one or more MPEG-4 
//	Systems (ISO/IEC 14496-1) deices as specified by the MPEG-4 Systems (ISO/IEC 
//	14496-1) standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	PSL retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2003.

// IPMP_DeviceID_Notification.cpp

#include "IPMP_DeviceID_Notification.h"
#include <memory.h>
#include "common.h"
#include <stdlib.h>
#include <assert.h>
#include <string.h>
#include <conio.h> 


USING_MOSES_NAMESPACE

IPMP_DeviceID_Notification::IPMP_DeviceID_Notification()
:IPMP_Data_BaseClass( IPMP_DeviceID_Notification_tag )
{
	// initialize IPMP_DomainID pointer
    IPMP_DomainID = NULL;
}	

IPMP_DeviceID_Notification::IPMP_DeviceID_Notification(const IPMP_DeviceID_Notification & obj):
	IPMP_Data_BaseClass( obj ),
	OpaqueData(obj.OpaqueData)
{
    IPMP_DomainID = NULL;
	setDomainID(obj.getDomainID());
}

IPMP_DeviceID_Notification::~IPMP_DeviceID_Notification() 
{
	delete[] IPMP_DomainID;
}
	
void IPMP_DeviceID_Notification::setDomainID(const byte * domainID )
{
	if (IPMP_DomainID != NULL)
		delete[] IPMP_DomainID;             // clear the old IPMP_DomainID

	IPMP_DomainID = new byte[16];
	memcpy(IPMP_DomainID, domainID, 16) ;   // dump the new IPMP_DomainID, 16 bytes
}

byte * IPMP_DeviceID_Notification::getDomainID() const
{
	return IPMP_DomainID ;                  // return the pointer to IPMP_DomainID
}

int IPMP_DeviceID_Notification::fromBuffer(const byte * buf)
{
	int nbits = IPMP_Data_BaseClass::fromBuffer(buf);    // read the base class and return bit number read
	assert((nbits % 8) == 0);

    setDomainID(getReadPtr());                           // read 128 bits DomainID				
	nbits+=128;
	
	int nCount = OpaqueData.fromBuffer(buf+(nbits/8));   // read the opaquedata and return the size in bit

	return nbits+nCount ;
}


int IPMP_DeviceID_Notification::toBuffer(byte * buf)     
{
	int nbits = IPMP_Data_BaseClass::toBuffer(buf);      // write the base class and return bit number read
	assert((nbits % 8) == 0);

	memcpy(buf+(nbits/8), IPMP_DomainID, 16);            // write 128 bits DomainID
	nbits+=128 ;
	
	int nCount = OpaqueData.toBuffer(buf+(nbits/8));     // write the opaquedata and return the size in bit

	return nbits+nCount ;
}

// in Byte!!
unsigned int IPMP_DeviceID_Notification::sizeOfInstance() const
{
	int nSize = IPMP_Data_BaseClass::sizeOfInstance() ;  // size of the base class

	nSize += (16 + OpaqueData.getSize()) ; // 16 bytes domainID + opaquedata size

	return nSize ;  // size in byte
}

void IPMP_DeviceID_Notification::setOpaqueData(const byte *buf, unsigned int size)
{
	OpaqueData.setData(buf, size);
}

const byte* IPMP_DeviceID_Notification::getOpaqueData() const
{
	byte *p = OpaqueData.getData() ;
	return p ;  // return the pointer to the opaque data
}
