//	This software module was originally developed by Liao Xuan (PSL)
//	in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1) standard. 
//	This software module is an implementation of a part of one or more MPEG-4 
//	Systems (ISO/IEC 14496-1) deices as specified by the MPEG-4 Systems (ISO/IEC 
//	14496-1) standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	PSL retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2003.

// IPMP_DeviceID_Notification.h

#ifndef	IPMP_DEVICEIDNOTIFICATION_H
#define	IPMP_DEVICEIDNOTIFICATION_H

#include "IPMP_Data_BaseClass.h"
#include "namespace.h"
#include "IPMP_ByteArray.h"


BEGIN_MOSES_NAMESPACE

/** 
 *  @class IPMP_DeviceID_Notification
 *  This message is sent from a Device to broadcast its existence to other device connected.
 *  The recipient Device replies with a IPMP_DeviceID_Notification message.
 *
 *  @author Liao Xuan
 *  @date 2003
 */
class IPMP_DeviceID_Notification: public IPMP_Data_BaseClass 
{
private:
	byte * IPMP_DomainID;					                // 128 bits
	IPMP_ByteArray OpaqueData;     
	
public:
	
	IPMP_DeviceID_Notification();						    ///< constructor.
	IPMP_DeviceID_Notification(const IPMP_DeviceID_Notification & obj);		  ///< copy constructor.
	virtual ~IPMP_DeviceID_Notification();					///< destructor.	
	virtual int toBuffer(byte * buf);						// reimplemented
	virtual int fromBuffer(const byte * buf);				// reimplemented
	virtual unsigned int sizeOfInstance() const;			// reimplemented

	/**
	 * set the Domain ID. (128 bits)
	 * @param domainID pointer the DomainID's pointer of the domain which the requesting Device(request generator) belongs to.
	 */
	void setDomainID(const byte *  domainID );

	/**
	 * Get the Domain ID.(128 bits) 
	 * @return DomainID pointer the DomainID's pointer of the domain which the requesting Device (request generator) belongs to.
	 */ 
	byte * getDomainID() const;

	/**
	 * method for setting the opaque data.
	 * @param buf opaque data pointer. 
	 * @param size opaque data size. 
	 */
	void setOpaqueData(const byte *buf, unsigned int size);

	/**
	 * method for getting the opaque data.
	 * @return pointer to opaque data. 
	 */
	const byte * getOpaqueData() const;

};

END_MOSES_NAMESPACE

#endif
