//	This software module was originally developed by Liao Xuan (PSL)
//	in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1) standard. 
//	This software module is an implementation of a part of one or more MPEG-4 
//	Systems (ISO/IEC 14496-1) devices as specified by the MPEG-4 Systems (ISO/IEC 
//	14496-1) standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	PSL retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2003.



// IPMP_DeviceMessageBase.cpp

#include "IPMP_DeviceMessageBase.h"
#include "common.h" 
#include "IPMP_MessageFromDevice.h"

#include <stdlib.h>
#include <memory.h>
#include <assert.h>
#include <string.h>
#include <conio.h> 
#include "common.h"


USING_MOSES_NAMESPACE

IPMP_DeviceMessageBase::IPMP_DeviceMessageBase(byte tag)
:   ExpandableBaseClass(tag)
{
	// initialize the object
	Version = NEWVERSION;	// version for Device to Device messages should be 0x02;
	sender_deviceID = NULL;
	recipient_deviceID = NULL;
}

IPMP_DeviceMessageBase::~IPMP_DeviceMessageBase() 
{
	delete sender_deviceID;
	delete recipient_deviceID;
}

IPMP_DeviceMessageBase::IPMP_DeviceMessageBase(const IPMP_DeviceMessageBase & msg)
:	ExpandableBaseClass(msg)
{	
	Version = msg.Version;
	sender_deviceID = NULL;
	recipient_deviceID = NULL;
	setSender(msg.getSender());
	setRecipient(msg.getRecipient());
}

void IPMP_DeviceMessageBase::setVersion(word32 ver)
{
	Version = ver;
}

word32 IPMP_DeviceMessageBase::getVersion()
{
	return Version;
}

bool IPMP_DeviceMessageBase::checkSyntax() const
{ 
	if (Version != 0x02) // syntax version must be 0x02 for IPMP_DeviceMessageBase messages.
		return false;	
	return ExpandableBaseClass::checkSyntax();
}
 
int IPMP_DeviceMessageBase::fromBuffer(const byte * buf)
{
	int nbits = ExpandableBaseClass::fromBuffer(buf);	// read tag and size
	Version = get(buf, nbits, 8);						// read version
	nbits += 8;

	setSender(getReadPtr());	                        // read Sender id, size is 128bits			
	nbits+=128;
    
	setRecipient(getReadPtr()+16);	                 	// read Recipient id,, size is 128bits						
	nbits+=128;

	return nbits;
}

int IPMP_DeviceMessageBase::toBuffer(byte * buf)
{
	int nbits = ExpandableBaseClass::toBuffer(buf);		// write tag and size
	put( Version, buf, nbits, 8 ) ;
	nbits += 8 ;                                        // write version
    
	memcpy(buf+(nbits/8), sender_deviceID, 16);         // write sender device ID, 128 bits
	nbits+=128 ;

    memcpy(buf+(nbits/8), recipient_deviceID, 16);      // write recipient device ID, 28 bits
	nbits+=128 ;

	return nbits;
}

unsigned int IPMP_DeviceMessageBase::sizeOfInstance() const
{
	return 33;   // 1 byte (version) + 16 bytes (senderID) + 16 bytes (recipientID)
}

byte * IPMP_DeviceMessageBase::getSender() const
{
	return sender_deviceID;
}

void IPMP_DeviceMessageBase::setSender(const byte * newsender_deviceID)
{
	if (sender_deviceID != NULL)
		delete[] sender_deviceID;                        // clear old sender_deviceID

	sender_deviceID = new byte[16];
	memcpy(sender_deviceID, newsender_deviceID, 16) ;    // set new sender_deviceID
}

byte * IPMP_DeviceMessageBase::getRecipient() const
{
	return recipient_deviceID;
}

void IPMP_DeviceMessageBase::setRecipient(const byte * newrecipient_deviceID)
{
	if (recipient_deviceID != NULL)
		delete[] recipient_deviceID;                       // clear old recipient_deviceID

	recipient_deviceID = new byte[16];
	memcpy(recipient_deviceID, newrecipient_deviceID, 16); // set new recipient_deviceID
}


// ------ factory methods ---------
 
IPMP_DeviceMessageBase * IPMP_DeviceMessageBase::createMessage(byte tag)
{
	
	IPMP_DeviceMessageBase * pNewMessage = NULL; // initialize object pointers

    if (tag == IPMP_MessageFromDevice_tag) 
		pNewMessage=new IPMP_MessageFromDevice;  // get object pointers according to tag

	return pNewMessage;
}

IPMP_DeviceMessageBase * IPMP_DeviceMessageBase::createMessage(const byte * message, unsigned int & nBitRead )
{
	IPMP_DeviceMessageBase * pNewMessage = createMessage(message[0]); // create object from the binary message
	
	if (pNewMessage != NULL)
	{
		nBitRead = pNewMessage->fromBuffer(message);  // get the No of bits read from "message"
	}
	return pNewMessage;  // return the initialized object' pointer
}

IPMP_DeviceMessageBase * IPMP_DeviceMessageBase::createMessage(IPMP_DeviceMessageBase &object)
{
	unsigned int size = object.getSize();  // get the object size
	byte * buffer = new byte[size];        // initialize buffer for the new object	
	object.toBuffer(buffer);			   // dump object data into the buffer
	unsigned int nbitsRead;
	IPMP_DeviceMessageBase * newobj = createMessage(buffer, nbitsRead); // create a cloned new object in buffer and return message size
	delete[] buffer;                       // clear the buffer used
	return newobj;                         // return cloned object pointer
}

