
//	This software module was originally developed by Liao Xuan (PSL)
//	in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1) standard. 
//	This software module is an implementation of a part of one or more MPEG-4 
//	Systems (ISO/IEC 14496-1) devices as specified by the MPEG-4 Systems (ISO/IEC 
//	14496-1) standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	PSL retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2003.


// IPMP_DeviceMessageBase.h


#ifndef	IPMP_DEVICEMESSAGEBASE_H
#define	IPMP_DEVICEMESSAGEBASE_H

#include "ExpandableBaseClass.h"
#include "namespace.h"
#include "IPMP_ByteArray.h"


BEGIN_MOSES_NAMESPACE


#define NEWVERSION		                            0x02		
						
//----------- list of tags for messages extending the IPMP_DeviceMessageBase ---------//
#define Forbidden									0x00
#define IPMP_MessageFromDevice_tag					0x01



//----------- list of tags for device messages extending the IPMP_Data_BaseClass ---------//
/*
#define IPMP_RequestContent_tag					    0x1D
#define IPMP_ResponseToContentRequest_tag			0x1E
#define IPMP_ContentTransfer_tag			        0x1F
#define IPMP_RequestTool_tag			            0x20
#define IPMP_ResponseToToolRequest_tag			    0x21
#define IPMP_DeviceID_Notification_tag              0x22
*/


/** 
 * @class IPMP_DeviceMessageBase
 * This class is for IPMP Devices to IPMP Devices Messages
 * 
 * @author Liao Xuan, Panasonic Singapore Laboratories
 * @date Aug 2003
 */
class IPMP_DeviceMessageBase: public ExpandableBaseClass
{ 
private:
	word32 Version;				// the version of syntax used - must be 0x02
	byte * sender_deviceID;		// the 128 bit ID of the sender device
	byte * recipient_deviceID;	// the 128 bit ID of the destination device 
	
	IPMP_DeviceMessageBase();   // make this an abstract class, do not allow creating object 
	                            // of this type from external code.

protected:
	/**
	 * method to set the version of syntax used (for IPMP Extension Annex,
	 * this must be 0x02)
	 * @param ver the version number to set
	 */
	void setVersion(word32 ver);

	/**
	 * creates an object of a specified type, identified by the tag.
	 * @param tag the object_id of the instance to be created.
	 */
	IPMP_DeviceMessageBase(byte tag);	
	
	/**
	 *	Copy constructor.
	 *	Creates a copy of the object given as input parameter
	 *  @param msg the object to copy
	 */
	IPMP_DeviceMessageBase::IPMP_DeviceMessageBase(const IPMP_DeviceMessageBase& msg);

public:
	virtual ~IPMP_DeviceMessageBase();					///<destructor
	virtual int toBuffer(byte * buf);					// reimplemented
	virtual int fromBuffer(const byte * buf);			// reimplemented
	virtual bool checkSyntax() const;					// reimplemented
	virtual unsigned int sizeOfInstance() const;		// reimplemented

	/**
	 * method to get access to the version of syntax of this message (must be 0x02).
	 * @return the syntax version number
	 */
	word32 getVersion();

	/**
	 * Get the context id of the sender the originator of the message.  
	 * 
	 * @return the context id of the sender Device.
	 */
	byte * getSender() const;

	/**
	 * Set the context id of the sender the originator of the message.  
	 *
	 * @param sender the context id of the sender Device
	 */
	void setSender(const byte *  sender);

	/**
	 * Get the the context ID of the intended recipient of the message.  
	 * 
	 * @return the context id of the destination Device.
	 */
	 byte *  getRecipient() const;
	
	/**
	 * Set the the context ID of the intended recipient of the message.  
	 * 
	 * @return the context id of the destination Device.
	 */
	void setRecipient(const byte *  recipient);


	/**
	 * This static "factory" method creates and initialize an object derived 
	 * from this class from the binary message given in input.
	 * The caller must delete the object when not used anymore.
	 * @param message the binary message from which the object is created.
	 * @param nBitRead returns the number of bits read from "message".
	 * @return a pointer to an initialized object constructed from the bitstream
	 */
	static IPMP_DeviceMessageBase * createMessage(const byte * message, unsigned int & nBitRead);
	
	/**
	 * This static "factory" method creates an empy object derived 
	 * from this class using the tag given in input.
	 * The caller must delete the object when not used anymore.
	 * @param tag the object tag
	 * @return a pointer to an empty object with the same tag
	 */
	static IPMP_DeviceMessageBase * createMessage(byte tag);

	/**
	 * This static "factory" method creates and initialize an object derived 
	 * from this class from the object in input.
	 * The caller must delete the object when not used anymore.
	 * @param object the object to clone
	 * @return a pointer to an initialized object of the same type
	 */
	static IPMP_DeviceMessageBase * createMessage(IPMP_DeviceMessageBase &object);
};

END_MOSES_NAMESPACE

#endif

