
//	This software module was originally developed by Claudio Alberti (EPFL)	
//	in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1) standard. 
//	This software module is an implementation of a part of one or more MPEG-4 
//	Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 
//	14496-1) standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	EPFL retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2002.


// IPMP_GetToolContext.cpp

#include "IPMP_GetToolContext.h"
#include <memory.h>

USING_MOSES_NAMESPACE

IPMP_GetToolContext::IPMP_GetToolContext()
:IPMP_Data_BaseClass( IPMP_GetToolContext_tag ),
 hasToolDescriptorID(0),
 scope(0),
 IPMP_ToolDescriptorID(0)
{
}	

// copy constructor.
IPMP_GetToolContext::IPMP_GetToolContext(const IPMP_GetToolContext & obj)	
:IPMP_Data_BaseClass( obj )
{

	if (obj.hasIPMP_ToolDescriptorID())
		setToolDescriptorID( obj.getToolDescriptorID() );
	else
	{
		hasToolDescriptorID = 0;
		IPMP_ToolDescriptorID = 0;
	}
	setScope( obj.getScope() );
}


IPMP_GetToolContext::~IPMP_GetToolContext() 
{

}

bool IPMP_GetToolContext::hasIPMP_ToolDescriptorID() const
{
	return (hasToolDescriptorID != 0);
}

void IPMP_GetToolContext::setScope( enum Scope myscope )
{
	scope = myscope ;
}

enum IPMP_GetToolContext::Scope IPMP_GetToolContext::getScope() const
{
	return (enum Scope) scope ;
}
	
void IPMP_GetToolContext::setToolDescriptorID( unsigned short toolID )
{
	hasToolDescriptorID = true ;
	IPMP_ToolDescriptorID = toolID ;
}

unsigned short IPMP_GetToolContext::getToolDescriptorID() const
{
	return (unsigned short) IPMP_ToolDescriptorID ;
}

int IPMP_GetToolContext::fromBuffer(const byte * buf)
{
	int nbits = IPMP_Data_BaseClass::fromBuffer(buf);
	
	scope = get( buf, nbits, 3 ) ;				// scope 3 bits
	nbits+=3 ;

	hasToolDescriptorID = get( buf, nbits, 1 ) ;	// hasToolDescriptorID 1 bit
	nbits+=1 ;

	get( buf, nbits, 4 ) ;						// res 4 bits
	nbits+=4 ;

	if( hasToolDescriptorID )
	{
		IPMP_ToolDescriptorID = get(buf, nbits, 16 ) ; // 16 bits for id
		nbits+=16 ;
	}

	return nbits ;
}


int IPMP_GetToolContext::toBuffer(byte * buf)
{
	int nbits = IPMP_Data_BaseClass::toBuffer(buf);
	
	put( scope, buf, nbits, 3 ) ;				// scope 3 bits
	nbits+=3 ;

	put( hasToolDescriptorID, buf, nbits, 1 ) ;	// hasToolDescriptorID 1 bit
	nbits+=1 ;

	byte res = 0 ;								// res 4 bits
	put( res, buf, nbits, 4 ) ;
	nbits+=4 ;

	if( hasToolDescriptorID )					
	{
		put( IPMP_ToolDescriptorID, buf, nbits, 16 ) ;	// 16 bits for id
		nbits+=16 ;
	}

	return nbits ;
	
}

unsigned int IPMP_GetToolContext::sizeOfInstance() const
{
	int nsize = IPMP_Data_BaseClass::sizeOfInstance() + 1 ;
	if( hasToolDescriptorID )					
		nsize+=2 ;
	return nsize ;
}
