
//	This software module was originally developed by Claudio Alberti (EPFL)	
//	in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1) standard. 
//	This software module is an implementation of a part of one or more MPEG-4 
//	Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 
//	14496-1) standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	EPFL retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2002.

// IPMP_GetToolContext.h

#ifndef	IPMP_GETTOOLCONTEXT_H
#define	IPMP_GETTOOLCONTEXT_H

#include "IPMP_Data_BaseClass.h"
#include "namespace.h"


BEGIN_MOSES_NAMESPACE

/** 
 *  @class IPMP_GetToolContext
 *  This message is sent from a Tool to the Terminal to find a particular tool context within a specified scope.
 *  The Terminal replies with a IPMP_GetToolContextResponse message.
 *
 *  @author Claudio Alberti, Massimo Balestri
 *  @date 2002
 */
class IPMP_GetToolContext: public IPMP_Data_BaseClass 
{
private:
	unsigned int scope ;					// 3 bits
	unsigned int hasToolDescriptorID;		// 1 bit
	unsigned int IPMP_ToolDescriptorID ;	// 16 bits

public:
	enum Scope {
		SCOPE_ESD	= 0x0,		///< limit the search to all tools protecting the same elementary stream the tool who sends the message is protecting.
		SCOPE_OD	= 0x1,		///< limit the search to all tools declared in the same OD the tool who sends the message is declared in (at the OD level or the ESD level).
		SCOPE_LOCAL = 0x2		///< limit the search to all tools declared in the same name scope the tool who sends the message is declared in.
	};	
	
	IPMP_GetToolContext();									///< constructor.
	IPMP_GetToolContext(const IPMP_GetToolContext & obj);	///< copy constructor.
	virtual ~IPMP_GetToolContext();							///< destructor.	
	virtual int toBuffer(byte * buf);						// reimplemented
	virtual int fromBuffer(const byte * buf);				// reimplemented
	virtual unsigned int sizeOfInstance() const;			// reimplemented

	/**
	 * Set the scope. This can be SCOPE_ESD, SCOPE_OD, SCOPE_LOCAL.
	 * @param scope the scope of the tool.
	 */
	void setScope( enum Scope scope ) ;
	
	/**
	 * Set the tool descriptor ID. 
	 * @param toolID the Tool descriptor ID of the tool.
	 */
	void setToolDescriptorID( unsigned short toolID ) ;

	/**
	 * Get the scope. This can be SCOPE_ESD, SCOPE_OD, SCOPE_LOCAL.
	 * @return the scope of the Tool.
	 */
	enum Scope getScope() const ;
	
	/**
	 * Get the tool descriptor ID. 
	 * @return toolID the Tool descriptor ID of the tool.
	 */
	unsigned short getToolDescriptorID() const;

	/**
	 * Informs about the presence of a Tool descriptor ID in the message.
	 * return true if the Tool descriptor ID is present.
	 */
	bool hasIPMP_ToolDescriptorID() const;
};

END_MOSES_NAMESPACE

#endif
