
//	This software module was originally developed by Claudio Alberti (EPFL)	
//	in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1) standard. 
//	This software module is an implementation of a part of one or more MPEG-4 
//	Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 
//	14496-1) standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	EPFL retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2002.


// IPMP_GetToolContextResponse.cpp

#include "IPMP_GetToolContextResponse.h"
#include <memory.h>


USING_MOSES_NAMESPACE

IPMP_GetToolContextResponse::IPMP_GetToolContextResponse()
:IPMP_Data_BaseClass(IPMP_GetToolContextResponse_tag),
 IPMP_ToolContextID(0),
 OD_ID(0),
 ESD_ID(0),
 hasESD_ID(false)
{
}	


IPMP_GetToolContextResponse::~IPMP_GetToolContextResponse() 
{

}


void IPMP_GetToolContextResponse::setESD_ID(word32 id)
{
	ESD_ID = id ;
	hasESD_ID = true ;
}


word32 IPMP_GetToolContextResponse::getESD_ID() const
{
	return ESD_ID ;
}

	
void IPMP_GetToolContextResponse::setOD_ID(short id)
{
	OD_ID = id ;
}


unsigned int IPMP_GetToolContextResponse::getOD_ID() const
{
	return OD_ID ;
}


void IPMP_GetToolContextResponse::setIPMP_ToolContextID(word32 id)
{
	IPMP_ToolContextID = id ;
}


word32 IPMP_GetToolContextResponse::getIPMP_ToolContextID() const
{
	return IPMP_ToolContextID ;
}


int IPMP_GetToolContextResponse::fromBuffer(const byte * buf)
{
	int nbits = IPMP_Data_BaseClass::fromBuffer(buf);
	
	hasESD_ID = get( buf, nbits, 1 ) ;					// 1 bit
	nbits+=1 ;

	get( buf, nbits, 5 ) ;								// 5 bits
	nbits+=5 ;

	OD_ID = get( buf, nbits, 10 ) ;						// 10 bits
	nbits+=10 ;

	if( hasESD_ID )
	{
		ESD_ID = get( buf, nbits, 16 ) ;				// 16 bits
		nbits+=16 ;
	}

	IPMP_ToolContextID = get( buf, nbits, 32 ) ;		// 32 bits
	nbits+=32 ;

	return nbits;
}


int IPMP_GetToolContextResponse::toBuffer(byte * buf)
{
	int nbits = IPMP_Data_BaseClass::toBuffer(buf);
	
	put( hasESD_ID, buf, nbits, 1 ) ;					// 1 bit
	nbits+=1 ;

	byte res = 0 ;
	put( res, buf, nbits, 5 ) ;								// 5 bits
	nbits+=5 ;

	put( OD_ID, buf, nbits, 10 ) ;						// 10 bits
	nbits+=10 ;

	if( hasESD_ID )
	{
		put( ESD_ID, buf, nbits, 16 ) ;				// 16 bits
		nbits+=16 ;
	}

	put( IPMP_ToolContextID, buf, nbits, 32 ) ;		// 32 bits
	nbits+=32 ;
	
	return nbits ;
	
}

unsigned int IPMP_GetToolContextResponse::sizeOfInstance() const
{
	int nsize = IPMP_Data_BaseClass::sizeOfInstance() + 6 ;
	if( hasESD_ID )					
		nsize+=2 ;
	return nsize ;
}
