
//	This software module was originally developed by Claudio Alberti (EPFL)	
//	in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1) standard. 
//	This software module is an implementation of a part of one or more MPEG-4 
//	Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 
//	14496-1) standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	EPFL retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2002.

// IPMP_GetToolContextResponse.h

#ifndef	IPMP_GETTOOLCONTEXTRESPONSE_H
#define	IPMP_GETTOOLCONTEXTRESPONSE_H

#include "IPMP_Data_BaseClass.h"
#include "namespace.h"

BEGIN_MOSES_NAMESPACE

/** @class IPMP_GetToolContextResponse
 *  Sent from the Terminal to a tool that has required to find a particular tool context within a specified scope.
 *
 *  @author Claudio Alberti
 *  @date 2002
 */
class IPMP_GetToolContextResponse: public IPMP_Data_BaseClass 
{
private:
	word32 hasESD_ID ;					// 1  bit
										// 5  bit reserved
	word32 OD_ID;						// 10 bit
	word32 ESD_ID;						// 16 bit
	word32 IPMP_ToolContextID;			// 32 bit 
										// 64 bit total  ( without ver and tag )
public:
	IPMP_GetToolContextResponse();					///< constructor
	virtual ~IPMP_GetToolContextResponse();			///< destructor
	virtual int toBuffer(byte * buf);				// reimplemented		
	virtual int fromBuffer(const byte * buf);		// reimplemented
	virtual unsigned int sizeOfInstance() const;	// reimplemented
	
	/**
	 * Set the ESD_ID of this message.
	 * @param id the ID of the ESD where the Tool is declared.
	 */
	void setESD_ID(word32 id);

	/**
	 * Get the ESD_ID of this message.
	 * @return the ID of the ESD where the Tool is declared.
	 */
	word32 getESD_ID() const;
	
	/**
	 * Set the OD_ID of this message.
	 * @param id the ID of the OD where the Tool is declared.
	 */
	void setOD_ID(short id);

	/**
	 * Set the OD_ID of this message.
	 * @return the ID of the OD where the Tool is declared.
	 */
	unsigned int getOD_ID() const;

	/**
	 * Set the Context ID assigned to the required Tool.
	 * A NULL value indicates that no tool has been found in the specified context.
	 * @param id the context id of the Tool.
	 */
	void setIPMP_ToolContextID(word32 id) ;

	/**
	 * Get the Context ID assigned to the required Tool.
	 * A NULL value indicates that no tool has been found in the specified context.
	 */
	word32 getIPMP_ToolContextID() const;
};

END_MOSES_NAMESPACE

#endif
