
//	This software module was originally developed by Claudio Alberti (EPFL)	
//  and edited by Massimo Balestri (Telecom Italial Lab)
//	in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1) standard. 
//	This software module is an implementation of a part of one or more MPEG-4 
//	Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 
//	14496-1) standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	EPFL retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2002.


// IPMP_GetToolsResponse.cpp

#include "IPMP_GetToolsResponse.h"
#include <memory.h>
#include <string.h>

USING_MOSES_NAMESPACE

IPMP_GetToolsResponse::IPMP_GetToolsResponse()
	:IPMP_Data_BaseClass(IPMP_GetToolsResponse_tag)
{
}	

IPMP_GetToolsResponse::IPMP_GetToolsResponse(const IPMP_GetToolsResponse & obj)
	:IPMP_Data_BaseClass(obj),
	tools(obj.tools)
{
}

IPMP_GetToolsResponse::~IPMP_GetToolsResponse() 
{
}

int IPMP_GetToolsResponse::fromBuffer(const byte * buf)
{
	IPMP_Data_BaseClass::fromBuffer(buf);
	
	tools.clear();									// remove all old objects 
	int buffersize = getSizeOfInstance();			// get the size of the buffer
	while( (getReadCounter() / 8) < buffersize )
	{
		IPMP_Tool * newtool = (IPMP_Tool *) BaseDescriptor::createMessage( getReadPtr() );
		tools.addObject(*newtool);
		skipReading( newtool->getSize() * 8);
		delete newtool;
	}

	return getReadCounter();
}


int IPMP_GetToolsResponse::toBuffer(byte * buf)
{
	IPMP_Data_BaseClass::toBuffer(buf);

	for (unsigned int i=0; i<tools.getObjectNum(); i++)
		skipWriting( tools.getObject(i).toBuffer( getWritePtr() ));
	
	return getWriteCounter();
}


unsigned int IPMP_GetToolsResponse::sizeOfInstance() const
{
	unsigned int size = IPMP_Data_BaseClass::sizeOfInstance();
	
	for (unsigned int i=0; i < tools.getObjectNum(); i++)
		size += tools.getObject(i).getSize();

	return size;

}
