
//	This software module was originally developed by ADETTI in the course of
//	development of the MPEG-4 Systems (ISO/IEC 14496-1) standard. 
//	This software module is an implementation of a part of one or more MPEG-4 
//	Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 
//	14496-1) standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	ADETTI retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2002.


// IPMP_InitAuthentication.h

#ifndef	IPMP_INITAUTHENTICATION_H
#define	IPMP_INITAUTHENTICATION_H

#include "IPMP_Data_BaseClass.h"

#include "namespace.h"

BEGIN_MOSES_NAMESPACE

// Types of authentication.
//                 0x00      // Forbidden
#define NO_AUTH    0x01      // No Authentication Required
#define NO_VER_SEC 0x02      // No ID verify, Do secure channel
#define VER_NO_SEC 0x03      // Do ID verify, No secure channel
#define VER_SEC    0x04      // Do ID verify, Do secure channel
//                 0x05-0xFE // ISO Reserved
//                 0xFF      // Forbidden

/** @class IPMP_InitAuthentication
 *  Message that initiates a mutual authentication process.
 *
 *  @author ADETTI
 *  @date 2002
 */
class IPMP_InitAuthentication: public IPMP_Data_BaseClass 
{
private:
	word32 context;
	word32 authType;

public:
	/**
	 * Constructor.
	 */
	IPMP_InitAuthentication();					
	/**
	 * Destructor.
	 */
	virtual ~IPMP_InitAuthentication();		
	/**
	 * Method for writing the message using the data of the class.
	 * @param buf storage location for the byte sequence. 
	 */
	virtual int toBuffer(byte * buf);		
	/**
	 * Method for reading the contents of the message.
	 * @param buf storage location of the byte sequence. 
	 */
	virtual int fromBuffer(const byte * buf);

	/**
	 * Get the total size of the class, when converted into a bitstream. 
	 * This size does NOT include the object_id and the length of the base
	 * class. 
	 * @return the size in bytes. 
	 */
	virtual unsigned int sizeOfInstance() const;

	/**
	 * Set the 16-bit Context ID of the logical instance of the Tool with
	 * which mutual authentication is to be performed.
	 * @param context the 16-bit Context ID
	 */
	void setContext(word32 context);
	/**
	 * Get the 16-bit Context ID of the logical instance of the Tool with
	 * which mutual authentication is to be performed.
	 * @return the 16-bit Context ID
	 */
	word32 getContext() const;

	/**
	 * Set the type of authentication to perform.
	 * @param authType can be one of the following values:
	 *                 <table>
	 *                     <tr>
	 *                         <td><b>Value of AuthType</b></td>
	 *                         <td><b>Semantic Meaning</b></td>
	 *                     </tr>
	 *                     <tr>
	 *                         <td>0x00</td>
	 *                         <td>Forbidden</td>
	 *                     </tr>
	 *                     <tr>
	 *                         <td>0x01</td>
	 *                         <td>No Authentication Required</td>
	 *                     </tr>
	 *                     <tr>
	 *                         <td>0x02</td>
	 *                         <td>No ID verify, Do secure channel</td>
	 *                     </tr>
	 *                     <tr>
	 *                         <td>0x03</td>
	 *                         <td>Do ID verify, No secure channel</td>
	 *                     </tr>
	 *                     <tr>
	 *                         <td>0x04</td>
	 *                         <td>Do ID verify, Do secure channel</td>
	 *                     </tr>
	 *                     <tr>
	 *                         <td>0x05-0xFE</td>
	 *                         <td>ISO Reserved</td>
	 *                     </tr>
	 *                     <tr>
	 *                         <td>0xFF</td>
	 *                         <td>Forbidden</td>
	 *                     </tr>
	 *                 </table>
	 */
	void setAuthType(word32 authType);
	/**
	 * Get the type of authentication to perform.
	 * @return can be one of the values referred for #setAuthType.
	 */
	word32 getAuthType() const;
};

END_MOSES_NAMESPACE

#endif

