
//	This software module was originally developed by Claudio Alberti (EPFL)	
//	in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1) standard. 
//	This software module is an implementation of a part of one or more MPEG-4 
//	Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 
//	14496-1) standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	EPFL retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2002.


// IPMP_KeyData.cpp

#include "IPMP_KeyData.h"
#include <memory.h>

USING_MOSES_NAMESPACE


IPMP_KeyData::IPMP_KeyData()
:	IPMP_Data_BaseClass( IPMP_KeyData_tag )
{
	init() ;	
}

IPMP_KeyData::~IPMP_KeyData() 
{

}

void IPMP_KeyData::setStartDTS(byte* data)
{
	hasStartDTS = true;
	memcpy(startDTS, data, 8);
}

void IPMP_KeyData::setStartPacketID(word32 data)
{
	hasStartPacketID = true;
	startPacketID = data;
}

void IPMP_KeyData::setExpireDTS(byte *data)
{
	hasExpireDTS = true;
	memcpy(expireDTS, data, 8);
}
	
void IPMP_KeyData::setExpirePacketID(word32 data)
{
	hasExpirePacketID = true;
	expirePacketID = data;
}

bool IPMP_KeyData::getStartDTS(byte *&data) const
{
	if(hasStartDTS)
		memcpy(data, startDTS, 8);
	else return false;

	return true;
}

bool IPMP_KeyData::getStartPacketID(word32 &data) const
{
	if(hasStartPacketID)
	{
		data = startPacketID;
		return true;
	}
	else 
		return false;
}

bool IPMP_KeyData::getExpireDTS(byte *&data) const
{
	if(hasExpireDTS)
		memcpy(data, expireDTS, 8);
	else return false;

	return true;
}

bool IPMP_KeyData::getExpirePacketID(word32 &data) const
{
	if(hasExpirePacketID)
	{
		data = expirePacketID;
		return true;
	}
	else 
		return false;
}

void IPMP_KeyData::setKeyBody(byte *key, unsigned int keyDim)
{
	keyBody.setData(key, keyDim);
}

const IPMP_ByteArray& IPMP_KeyData::getkeyBody() const
{
	return keyBody;
}


void IPMP_KeyData::setOpaqueData(byte *od, unsigned int odDim)
{
	opaqueData.setData(od, odDim);
}

const IPMP_ByteArray& IPMP_KeyData::getOpaqueData() const
{
	return opaqueData;
}

int IPMP_KeyData::toBuffer(byte * buf)
{
	IPMP_Data_BaseClass::toBuffer( buf ) ;

	skipWriting( keyBody.toBuffer( getWritePtr())) ;

	put( hasStartDTS, 1 ) ;
	put( hasStartPacketID, 1 ) ;
	put( hasExpireDTS, 1 ) ;
	put( hasExpirePacketID, 1 ) ;
	put( 0, 4 ) ;						// reserved

	if( hasStartDTS )
		putBuffer(startDTS, 64 ) ;

	if( hasStartPacketID )
		put(startPacketID, 32);

	if( hasExpireDTS )
		putBuffer( expireDTS, 64) ;

	if( hasExpirePacketID )
		put(expirePacketID, 32 ) ;

	skipWriting( opaqueData.toBuffer( getWritePtr()) ) ;

	return getWriteCounter() ;
}


int IPMP_KeyData::fromBuffer(const byte * buf)
{
	IPMP_Data_BaseClass::fromBuffer( buf ) ;
	init();

	skipReading( keyBody.fromBuffer( getReadPtr())) ;

	hasStartDTS			= (get(1) == 1);
	hasStartPacketID	= (get(1) == 1);
	hasExpireDTS		= (get(1) == 1);
	hasExpirePacketID	= (get(1) == 1);
	
	skipReading(4) ;						// reserved

	if( hasStartDTS )
		getBuffer(startDTS, 64 ) ;

	if( hasStartPacketID )
		startPacketID = get(32);

	if( hasExpireDTS )
		getBuffer( expireDTS, 64);

	if( hasExpirePacketID )
		expirePacketID = get(32);

	skipReading( opaqueData.fromBuffer( getReadPtr()) ) ;

	return getReadCounter() ;
}

unsigned int IPMP_KeyData::sizeOfInstance() const
{
	unsigned int nbytes = IPMP_Data_BaseClass::sizeOfInstance() + 1 ;
	nbytes += keyBody.getSize() ;
	
	if( hasStartDTS )
		nbytes += 8 ;
	
	if( hasStartPacketID )
		nbytes += 4 ;

	if( hasExpireDTS )
		nbytes += 8 ;

	if( hasExpirePacketID )
		nbytes += 4 ;

	nbytes += opaqueData.getSize() ;
	
	return nbytes ;
}

void IPMP_KeyData::init()
{
	hasStartDTS = hasStartPacketID = hasExpireDTS = hasExpirePacketID = false ;
	startPacketID = 0;
	expirePacketID = 0;
	memset(startDTS, 0, 8);
	memset(expireDTS, 0, 8);
}
