
//	This software module was originally developed by Claudio Alberti (EPFL)	
//	in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1) standard. 
//	This software module is an implementation of a part of one or more MPEG-4 
//	Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 
//	14496-1) standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	EPFL retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2002.

// IPMP_KeyData.h

#ifndef	IPMP_KEYDATA_H
#define	IPMP_KEYDATA_H

#include "IPMP_Data_BaseClass.h"
#include "IPMP_ByteArray.h"
#include "namespace.h"

BEGIN_MOSES_NAMESPACE

/** 
 *  @class IPMP_KeyData
 *  This message carries key and synchronization information for decryption Tools.
 *	It could possibly be carried in IPMP_Message's in IPMP Streams. 
 *  This facilitates the switching of keys in a decryption session.
 *  @author Claudio Alberti
 *  @date 2002
 */
class IPMP_KeyData: public IPMP_Data_BaseClass 
{
private:
	IPMP_ByteArray keyBody;
	bool hasStartDTS;
	bool hasStartPacketID;
	bool hasExpireDTS;
	bool hasExpirePacketID;

	byte startDTS[8];
	word32 startPacketID;

	byte expireDTS[8];
	word32 expirePacketID;

	IPMP_ByteArray opaqueData;
	
	void init() ;					// init member vars

public:
	IPMP_KeyData();					///< constructor
	virtual ~IPMP_KeyData();		///< destructor		
	virtual int toBuffer(byte * buf);						// reimplemented
	virtual int fromBuffer(const byte * buf);				// reimplemented
	virtual unsigned int sizeOfInstance() const;			// reimplemented

	/**
	 * Set the StartDTS.
	 * StartDTS is the decoding time stamp of the first access unit for which the contained key is valid.
	 * @param data value to be set
	 */
	void setStartDTS(byte *data);

	/**
	 * Set the StartPacketID.
	 * StartPacketID is the packet ID of the first access unit for which the contained key is valid.
	 * @param data value to be set
	 */
	void setStartPacketID(word32 data);

	/**
	 * Set the ExpireDTS.
	 * ExpireDTS is the decoding time stamp of the first access unit for which the contained key is no longer valid.
	 * @param data value to be set
	 */
	void setExpireDTS(byte *data);

	/**
	 * Set the ExpirePacketID.
	 * ExpirePacketID is the packet ID of the first access unit for which the contained key is no longer valid.
	 * @param data value to be set 
	 */
	void setExpirePacketID(word32 data);

	/**
	 * Get the StartDTS.
	 * StartDTS is the decoding time stamp of the first access unit for which the contained key is valid.
	 * @param data value to be returned if present
	 * @return true if successful. 
	 */
	bool getStartDTS(byte *&data) const;

	/**
	 * Get the StartPacketID.
	 * StartPacketID is the packet ID of the first access unit for which the contained key is valid.
	 * @param data value to be returned if present
	 * @return true if successful. 
	 */
	bool getStartPacketID(word32 &data) const;

	/**
	 * Get the ExpireDTS.
	 * ExpireDTS is the decoding time stamp of the first access unit for which the contained key is no longer valid.
	 * @param data value to be returned if present
	 * @return true if successful. 
	 */
	bool getExpireDTS(byte *&data) const;

	/**
	 * Get the ExpirePacketID.
	 * ExpirePacketID is the packet ID of the first access unit for which the contained key is no longer valid.
	 * @param data value to be returned if present
	 * @return true if successful. 
	 */
	bool getExpirePacketID(word32 &data) const;

	/**
	 * method for setting the key body.
	 * @param key key value. 
	 * @param keyDim key size. 
	 */
	void setKeyBody(byte *key, unsigned int keyDim);

	/**
	 * method for getting the key body.
	 * @return key value. 
	 */
	const IPMP_ByteArray& getkeyBody() const;

	/**
	 * method for setting the opaque data.
	 * @param od opaque data value. 
	 * @param odDim opaque data size. 
	 */
	void setOpaqueData(byte *od, unsigned int odDim);

	/**
	 * method for getting the opaque data.
	 * @return opaque data. 
	 */
	const IPMP_ByteArray& getOpaqueData() const;
};

END_MOSES_NAMESPACE

#endif
