//	This software module was originally developed by Liao Xuan (PSL)
//	in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1) standard. 
//	This software module is an implementation of a part of one or more MPEG-4 
//	Systems (ISO/IEC 14496-1) devices as specified by the MPEG-4 Systems (ISO/IEC 
//	14496-1) standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	PSL retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2003.


#include "IPMP_MessageFromDevice.h"
#include "IPMP_Data_BaseClass.h"

#include <memory.h>
#include <stdlib.h>
#include <assert.h>

USING_MOSES_NAMESPACE

IPMP_MessageFromDevice::IPMP_MessageFromDevice()
:IPMP_DeviceMessageBase(IPMP_MessageFromDevice_tag)
{
	// initialize the message array in IPMP_MessageFromDevice
	m_Messages = new IPMP_Data_BaseClass * [ 255] ;
	memset( m_Messages, 0, sizeof(IPMP_Data_BaseClass*) * 255 ) ;
	m_nCount = 0 ;
}

IPMP_MessageFromDevice::IPMP_MessageFromDevice(const IPMP_MessageFromDevice & msg)
:IPMP_DeviceMessageBase(msg)
{
	m_Messages = new IPMP_Data_BaseClass * [ 255 ] ;
	memset( m_Messages, 0, sizeof(IPMP_Data_BaseClass*) * 255 ) ;
	m_nCount = 0 ;
	for( int x=0; x < msg.getNumOfMessages(); x++)
	{
		addMessage(* msg.getMessage(x));
	}
}

IPMP_MessageFromDevice::~IPMP_MessageFromDevice() 
{
	clearUp() ;
	delete[] m_Messages ;
}


void IPMP_MessageFromDevice::clearUp()
{
	for( int x=0;x<m_nCount;x++)
	{
		if( m_Messages[ x ] )
		{
			delete m_Messages[ x ] ;
		}
	}
	m_nCount = 0 ;
}

int IPMP_MessageFromDevice::fromBuffer(const byte * buf)
{ 
	/**********************Refer to MOSETest******************************
	IPMP_DeviceMessageBase::fromBuffer(buf) ;	// read tag and size
	clearUp() ;

	int bitsize = 8 * getSizeOfInstance() ;
	while( getReadCounter() < bitsize )
	{
		unsigned int nBitRead;
		m_Messages[m_nCount++] = IPMP_Data_BaseClass::createMessage( getReadPtr(), nBitRead ) ;
		skipReading(nBitRead);
	}
	return getReadCounter();
    ********************************************************************/
	/*********************by liao xuan **********************************/
	int nbits = IPMP_DeviceMessageBase::fromBuffer(buf) ;	// read tag and size
	assert((nbits % 8) == 0);
	clearUp() ; // delete existing messages in buffer

	int bitsize = 8 * getSizeOfInstance() ; // size of the whole message
	
	while( nbits < bitsize )
	{
		unsigned int nBitRead;
		m_Messages[m_nCount++] = IPMP_Data_BaseClass::createMessage(buf+nbits/8, nBitRead ) ;
        nbits += nBitRead ;// add the size of every message in IPMP_DeviceMessageBase respectively
	}

	return nbits;
	/********************************************************************/
}


int IPMP_MessageFromDevice::toBuffer(byte * buf)
{
	/*********************by liao xuan **********************************/
    int nbits = IPMP_DeviceMessageBase::toBuffer(buf);	// write tag and size
	for( int x=0;x<m_nCount;x++)
	{
		int nCount =0;
		nCount += m_Messages[x]->toBuffer(buf+(nbits/8)); // add every message's size respectively
        nbits += nCount;
	}

	return nbits; 
    /********************************************************************/

	/**********************Refer to MOSETest******************************
	IPMP_DeviceMessageBase::toBuffer(buf);		// write tag and size
	for( int x=0;x<m_nCount;x++)
	{
		skipWriting( m_Messages[x]->toBuffer( getWritePtr() )) ;
	}
	return getWriteCounter();
    ********************************************************************/
}

unsigned int IPMP_MessageFromDevice::sizeOfInstance() const
{
	int nbytes = IPMP_DeviceMessageBase::sizeOfInstance() ;
	for( int x=0;x<m_nCount;x++)
	{
		nbytes += (m_Messages[x]->getSize()) ;
	}

	return nbytes;
}


IPMP_Data_BaseClass* IPMP_MessageFromDevice::getMessage( int nIndex ) const
{
	if( nIndex < m_nCount )
	{
		return m_Messages[ nIndex ] ;
	}
	else 
		return NULL;
}


bool IPMP_MessageFromDevice::addMessage( IPMP_Data_BaseClass &msg )
{
	if( m_nCount < 255 )
	{
		m_Messages[ m_nCount++ ] = IPMP_Data_BaseClass::createMessage(msg);
		return true ;
	}
	else
		return false ;
}


int IPMP_MessageFromDevice::getNumOfMessages() const
{
	return m_nCount ;// Number of messages carried inside
}
