//	This software module was originally developed by Massimo Balestri (TILAB)	
//	in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1) standard. 
//	This software module is an implementation of a part of one or more MPEG-4 
//	Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 
//	14496-1) standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	ADETTI retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2002.



#include "IPMP_MessageFromTool.h"
#include "IPMP_Data_BaseClass.h"

#include <string.h>
#include <stdlib.h>

USING_MOSES_NAMESPACE

IPMP_MessageFromTool::IPMP_MessageFromTool()
:IPMP_ToolMessageBase(IPMP_MessageFromTool_tag)
{
	m_Messages = new IPMP_Data_BaseClass * [ 256 ] ;
	memset( m_Messages, 0, sizeof(IPMP_Data_BaseClass*) * 256 ) ;
	m_nCount = 0 ;
}

IPMP_MessageFromTool::IPMP_MessageFromTool(const IPMP_MessageFromTool & msg)
:IPMP_ToolMessageBase(msg)
{
	m_Messages = new IPMP_Data_BaseClass * [ 256 ] ;
	memset( m_Messages, 0, sizeof(IPMP_Data_BaseClass*) * 256 ) ;
	m_nCount = 0 ;
	for( int x=0; x < msg.getNumOfMessages(); x++)
	{
		addMessage(* msg.getMessage(x));
	}
}

IPMP_MessageFromTool::~IPMP_MessageFromTool() 
{
	clearUp() ;
	delete[] m_Messages ;
}


void IPMP_MessageFromTool::clearUp()
{
	for( int x=0;x<m_nCount;x++)
	{
		if( m_Messages[ x ] )
		{
			delete m_Messages[ x ] ;
		}
	}
	m_nCount = 0 ;
}

int IPMP_MessageFromTool::fromBuffer(const byte * in)
{ 
	IPMP_ToolMessageBase::fromBuffer(in);	// read tag and size
	clearUp();

	long eom = getReadCounter() + 8 * (getSizeOfInstance() - IPMP_ToolMessageBase::sizeOfInstance());	// end of message

	while( getReadCounter() < eom )
	{
		unsigned int nBitRead;
		m_Messages[m_nCount++] = IPMP_Data_BaseClass::createMessage(getReadPtr(), nBitRead );
		skipReading(nBitRead);
	}
	
	return getReadCounter();
}


int IPMP_MessageFromTool::toBuffer(byte * out)
{
	IPMP_ToolMessageBase::toBuffer(out);		// write tag and size
	for( int x=0;x<m_nCount;x++)
	{
		putObject(*m_Messages[x]);
	}
	return getWriteCounter();
}

unsigned int IPMP_MessageFromTool::sizeOfInstance() const
{
	int nbytes = IPMP_ToolMessageBase::sizeOfInstance() ;
	for( int x=0;x<m_nCount;x++)
	{
		nbytes += m_Messages[x]->getSize() ;
	}

	return nbytes;
}


IPMP_Data_BaseClass* IPMP_MessageFromTool::getMessage( int nIndex ) const
{
	if( nIndex < m_nCount )
	{
		return m_Messages[ nIndex ] ;
	}
	else 
		return NULL;
}


bool IPMP_MessageFromTool::addMessage( IPMP_Data_BaseClass &msg )
{
	if( m_nCount < 255 )
	{
		m_Messages[ m_nCount++ ] = IPMP_Data_BaseClass::createMessage(msg);
		return true ;
	}
	else
		return false ;
}


int IPMP_MessageFromTool::getNumOfMessages() const
{
	return m_nCount ;
}
