
//	This software module was originally developed by Massimo Balestri (TILAB)
//	in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1) standard. 
//	This software module is an implementation of a part of one or more MPEG-4 
//	Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 
//	14496-1) standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	Telecom Italia Lab retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2002.


#ifndef	IPMP_MESSAGEFROMTOOL_H
#define	IPMP_MESSAGEFROMTOOL_H


#include "IPMP_ToolMessageBase.h"
#include "namespace.h"

BEGIN_MOSES_NAMESPACE


class IPMP_Data_BaseClass ;

/** 
 *  @class IPMP_MessageFromTool
 *  This class defines a container of multiple messages that can be 
 *  sent to one IPMP Tool instance, identified by its context Id (recipient).
 *  
 *  @author Massimo Balestri
 *  @date 2002
 */
class IPMP_MessageFromTool: public IPMP_ToolMessageBase
{
private:
	int m_nCount ;
	IPMP_Data_BaseClass **m_Messages ;

	void clearUp() ;						// clean all

public:	
	IPMP_MessageFromTool();										///< constructor
	IPMP_MessageFromTool(const IPMP_MessageFromTool & msg);		///< copy constructor
	virtual ~IPMP_MessageFromTool();							///< destructor
	virtual int toBuffer(byte * buf);				// reimplemented
	virtual int fromBuffer(const byte * buf);		// reimplemented
	virtual unsigned int sizeOfInstance() const;	// reimplemented

	/**
	 * Get the n-th message in the list. 
	 * @param nIndex the index of the message.
	 * @return a pointer to a message object.
	 */
	IPMP_Data_BaseClass *getMessage( int nIndex ) const ;

	/**
	 * Add a message to the list of messages contained into this object.
	 * @param msg the message object to be added.
	 */
	bool addMessage( IPMP_Data_BaseClass &msg ) ;

	/**
	 * Get the number of messages contained in this object.
	 * @return the number of messages.
	 */
	int getNumOfMessages() const;
};

END_MOSES_NAMESPACE

#endif