
//	This software module was originally developed by ADETTI in the course of
//  development of the MPEG-4 Systems (ISO/IEC 14496-1) standard. 
//	This software module is an implementation of a part of one or more MPEG-4 
//	Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 
//	14496-1) standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	ADETTI retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2002.

// IPMP_MutualAuthentication.cpp

#include "IPMP_MutualAuthentication.h"
#include "IPMP_ByteArray.h"
#include "TrustSecurityMetadata.h"

#include <assert.h>
#include <string.h>

/////////////////////////////////////////////////////////////////////////////
//
// IPMP_MutualAuthentication
//

USING_MOSES_NAMESPACE

IPMP_MutualAuthentication::IPMP_MutualAuthentication()
	:	IPMP_Data_BaseClass(IPMP_MutualAuthentication_tag)
{
	requestNegotiation = false;
	successNegotiation = false;
	failedNegotiation = false;
	inclAuthenticationData = false;
	inclAuthCodes = false;
	nCandidate = 0;
	ppCandidateAlgorithms = 0;
	nAlgorithms = 0;
	ppAgreedAlgorithms = 0;
	pAuthenticationData = 0;
	pAuthCodes = 0;
}

IPMP_MutualAuthentication::~IPMP_MutualAuthentication()
{
	clear();
}

IPMP_MutualAuthentication::IPMP_MutualAuthentication(const IPMP_MutualAuthentication & ma)
:	IPMP_Data_BaseClass(ma)
{
	requestNegotiation = ma.requestNegotiation;		// copy flags
	successNegotiation = ma.successNegotiation;
	failedNegotiation = ma.failedNegotiation;
	inclAuthenticationData = ma.inclAuthenticationData;
	inclAuthCodes = ma.inclAuthCodes;
	nCandidate = 0;
	ppCandidateAlgorithms = 0;
	nAlgorithms = 0;
	ppAgreedAlgorithms = 0;
	pAuthenticationData = 0;
	pAuthCodes = 0;

	if (requestNegotiation)
	{
		setRequestNegotiation(ma.getCandidateAlgorithms(), ma.getNumCandidateAlgorithms());
	}
	if (successNegotiation)
	{
		setSuccessNegotiation(ma.getAgreedAlgorithms(), ma.getNumAgreedAlgorithms());
	}
	if (inclAuthenticationData)
	{
		setAuthenticationData(ma.getAuthenticationData(), ma.getAuthenticationDataSize());
	}
	if (inclAuthCodes)
	{
		setAuthCodes( ma.getAuthCodes() );
	}
}


void IPMP_MutualAuthentication::clear()
{
	requestNegotiation = false;
	successNegotiation = false;
	failedNegotiation = false;
	inclAuthenticationData = false;
	inclAuthCodes = false;
	if (ppCandidateAlgorithms != 0) {
		for (word32 nc = 0; nc < nCandidate; ++nc) {
			delete ppCandidateAlgorithms[nc];
		}
		delete [] ppCandidateAlgorithms;
		ppCandidateAlgorithms = 0;
	}
	nCandidate = 0;
	if (ppAgreedAlgorithms != 0) {
		for (word32 na = 0; na < nAlgorithms; ++na) {
			delete ppAgreedAlgorithms[na];
		}
		delete [] ppAgreedAlgorithms;
		ppAgreedAlgorithms = 0;
	}
	nAlgorithms = 0;
	delete pAuthenticationData;
	pAuthenticationData = 0;
	delete [] pAuthCodes;
	pAuthCodes = 0;
}

int IPMP_MutualAuthentication::toBuffer(byte * buf)
{
	int nbits = IPMP_Data_BaseClass::toBuffer(buf);
	assert((nbits % 8) == 0);

	byte * mabuf = buf + (nbits/8);
	long bitindex = 0;

	// control bits
	put(requestNegotiation, mabuf, bitindex, 1);
	bitindex += 1;
	put(successNegotiation, mabuf, bitindex, 1);
	bitindex += 1;
	put(failedNegotiation, mabuf, bitindex, 1);
	bitindex += 1;
	put(inclAuthenticationData, mabuf, bitindex, 1);
	bitindex += 1;
	put(inclAuthCodes, mabuf, bitindex, 1);
	bitindex += 1;

	// reserved
	put(0, mabuf, bitindex, 3);
	bitindex += 3;

	if (requestNegotiation) {
		assert(nCandidate == 0 || ppCandidateAlgorithms != 0);
		assert(nCandidate < 256);	// must be 8 bit
		// nCandidate
		put(nCandidate, mabuf, bitindex, 8);
		bitindex += 8;
		// candidate algorithms
		for (word32 nc = 0; nc < nCandidate; ++nc) {
			assert(ppCandidateAlgorithms[nc] != 0);
			bitindex += ppCandidateAlgorithms[nc]->toBuffer(&mabuf[bitindex / 8]);
		}
	}

	if (successNegotiation) {
		assert(nAlgorithms == 0 || ppAgreedAlgorithms != 0);
		assert(nAlgorithms < 256);	// must be 8 bit
		// nAlgorithms
		put(nAlgorithms, mabuf, bitindex, 8);
		bitindex += 8;
		// agreed algorithms
		for (word32 na = 0; na < nAlgorithms; ++na) {
			assert(ppAgreedAlgorithms[na] != 0);
			bitindex += ppAgreedAlgorithms[na]->toBuffer(&mabuf[bitindex / 8]);
		}
	}

	if (inclAuthenticationData) {
		assert( pAuthenticationData != 0);
		bitindex += pAuthenticationData->toBuffer(&mabuf[bitindex / 8]);
	}

	if (inclAuthCodes) {
		assert(pAuthCodes != 0);
		bitindex += pAuthCodes->toBuffer(&mabuf[bitindex / 8]);
	}

	return (nbits + bitindex);
}

int IPMP_MutualAuthentication::fromBuffer(const byte * buf)
{
	clear();

	int nbits = IPMP_Data_BaseClass::fromBuffer(buf);
	assert((nbits % 8) == 0);
	
	const byte * mabuf = buf + (nbits/8);
	long bitindex = 0;

	// control bits
	requestNegotiation = (get(mabuf, bitindex, 1) != 0);
	bitindex += 1;
	successNegotiation = (get(mabuf, bitindex, 1) != 0);
	bitindex += 1;
	failedNegotiation = (get(mabuf, bitindex, 1) != 0);
	bitindex += 1;
	inclAuthenticationData = (get(mabuf, bitindex, 1) != 0);
	bitindex += 1;
	inclAuthCodes = (get(mabuf, bitindex, 1) != 0);
	bitindex += 1;

	// reserved
	get(mabuf, bitindex, 3);
	bitindex += 3;

	if (requestNegotiation) {
		// nCandidate
		nCandidate = get(mabuf, bitindex, 8);
		bitindex += 8;
		// candidate algorithms
		if (nCandidate > 0) {
			ppCandidateAlgorithms = new AlgorithmDescriptor *[nCandidate];
			assert(ppCandidateAlgorithms != 0);
			for (word32 nc = 0; nc < nCandidate; ++nc) {
				ppCandidateAlgorithms[nc] = new AlgorithmDescriptor;
				assert(ppCandidateAlgorithms[nc] != 0);
				bitindex += ppCandidateAlgorithms[nc]->fromBuffer(&mabuf[bitindex / 8]);
			}
		}
	}

	if (successNegotiation) {
		// nAlgorithms
		nAlgorithms = get(mabuf, bitindex, 8);
		bitindex += 8;
		// agreed algorithms
		if (nAlgorithms > 0) {
			ppAgreedAlgorithms = new AlgorithmDescriptor *[nAlgorithms];
			assert(ppAgreedAlgorithms != 0);
			for (word32 na = 0; na < nAlgorithms; ++na) {
				ppAgreedAlgorithms[na] = new AlgorithmDescriptor;
				assert(ppAgreedAlgorithms[na] != 0);
				bitindex += ppAgreedAlgorithms[na]->fromBuffer(&mabuf[bitindex / 8]);
			}
		}
	}

	if (inclAuthenticationData) {
		if (pAuthenticationData != 0)
			delete pAuthenticationData;

		pAuthenticationData = new IPMP_ByteArray();

		assert( pAuthenticationData != 0);
		bitindex += pAuthenticationData->fromBuffer(&mabuf[bitindex / 8]);
	}

	if (inclAuthCodes) {
		delete pAuthCodes;
		pAuthCodes = new AuthCodes;
		assert(pAuthCodes != 0);
		bitindex += pAuthCodes->fromBuffer(&mabuf[bitindex / 8]);
	}

	return (nbits + bitindex);
}


// compute size, in BYTES!
unsigned int IPMP_MutualAuthentication::sizeOfInstance() const
{
	unsigned int size = IPMP_Data_BaseClass::sizeOfInstance();
	size += 1; // control bits
	if (requestNegotiation) {
		assert(nCandidate == 0 || ppCandidateAlgorithms != 0);
		size += 1; // nCandidate
		// candidate algorithms
		for (word32 nc = 0; nc < nCandidate; ++nc) {
			assert(ppCandidateAlgorithms[nc] != 0);
			size += ppCandidateAlgorithms[nc]->getSize();
		}
	}
	if (successNegotiation) {
		assert(nAlgorithms == 0 || ppAgreedAlgorithms != 0);
		size += 1; // nAlgorithms
		// candidate algorithms
		for (word32 nc = 0; nc < nAlgorithms; ++nc) {
			assert(ppAgreedAlgorithms[nc] != 0);
			size += ppAgreedAlgorithms[nc]->getSize();
		}
	}
	if (inclAuthenticationData) {
		assert(pAuthenticationData != 0);
		size += pAuthenticationData->getSize();
	}
	if (inclAuthCodes) {
		assert(pAuthCodes != 0);
		size += pAuthCodes->getSize();
	}
	return size;
}

bool IPMP_MutualAuthentication::getRequestNegotiation() const
{
	return requestNegotiation;
}

void IPMP_MutualAuthentication::setRequestNegotiation(AlgorithmDescriptor * * ppCandidateAlgorithms, word32 nCandidate)
{
	word32 nc;
	
	if (this->ppCandidateAlgorithms != 0) {
		for (nc = 0; nc < this->nCandidate; ++nc) {
			delete this->ppCandidateAlgorithms[nc];
		}
		delete [] this->ppCandidateAlgorithms;
	}

	if (ppCandidateAlgorithms != 0 && nCandidate > 0) {

		this->nCandidate = nCandidate;
		this->ppCandidateAlgorithms = new AlgorithmDescriptor *[nCandidate];
		
		AlgorithmDescriptor * pAD;
		for (nc = 0; nc < nCandidate; ++nc) {
			if (ppCandidateAlgorithms[nc] != 0) {
				pAD = this->ppCandidateAlgorithms[nc] = new AlgorithmDescriptor(*(ppCandidateAlgorithms[nc]));
			} else {
				pAD = this->ppCandidateAlgorithms[nc] = 0;
			}
		}

	} else {
		this->nCandidate = 0;
		this->ppCandidateAlgorithms = 0;
	}

	requestNegotiation = true;
}

void IPMP_MutualAuthentication::clearRequestNegotiation()
{
	word32 nc;
	
	if (ppCandidateAlgorithms != 0) {
		for (nc = 0; nc < nCandidate; ++nc) {
			delete ppCandidateAlgorithms[nc];
		}
		delete [] ppCandidateAlgorithms;
	}

	nCandidate = 0;
	ppCandidateAlgorithms = 0;

	requestNegotiation = false;
}

word32 IPMP_MutualAuthentication::getNumCandidateAlgorithms() const
{
	return nCandidate;
}

AlgorithmDescriptor * * IPMP_MutualAuthentication::getCandidateAlgorithms() const
{
	return ppCandidateAlgorithms;
}


bool IPMP_MutualAuthentication::getSuccessNegotiation() const
{
	return successNegotiation;
}

void IPMP_MutualAuthentication::setSuccessNegotiation(AlgorithmDescriptor * * ppAgreedAlgorithms, word32 nAlgorithms)
{
	word32 na;
	
	if (this->ppAgreedAlgorithms != 0) {
		for (na = 0; na < this->nAlgorithms; ++na) {
			delete this->ppAgreedAlgorithms[na];
		}
		delete [] this->ppAgreedAlgorithms;
	}
	
	if (ppAgreedAlgorithms != 0) {

		this->nAlgorithms = nAlgorithms;
		this->ppAgreedAlgorithms = new AlgorithmDescriptor *[nAlgorithms];
		
		AlgorithmDescriptor * pAD;
		for (na = 0; na < nAlgorithms; ++na) {
			if (ppAgreedAlgorithms[na] != 0) {
				pAD = this->ppAgreedAlgorithms[na] = new AlgorithmDescriptor(*(ppAgreedAlgorithms[na]));
			} else {
				pAD = this->ppAgreedAlgorithms[na] = 0;
			}
		}

	} else {
		this->nAlgorithms = 0;
		this->ppAgreedAlgorithms = 0;
	}

	successNegotiation = true;
}

void IPMP_MutualAuthentication::clearSuccessNegotiation()
{
	word32 na;
	
	if (ppAgreedAlgorithms != 0) {
		for (na = 0; na < nAlgorithms; ++na) {
			delete ppAgreedAlgorithms[na];
		}
		delete [] ppAgreedAlgorithms;
	}
	
	nAlgorithms = 0;
	ppAgreedAlgorithms = 0;

	successNegotiation = false;
}

word32 IPMP_MutualAuthentication::getNumAgreedAlgorithms() const
{
	return nAlgorithms;
}

AlgorithmDescriptor * * IPMP_MutualAuthentication::getAgreedAlgorithms() const
{
	return ppAgreedAlgorithms;
}

bool IPMP_MutualAuthentication::getFailedNegotiation() const
{
	return failedNegotiation;
}

void IPMP_MutualAuthentication::setFailedNegotiation()
{
	failedNegotiation = true;
}

void IPMP_MutualAuthentication::clearFailedNegotiation()
{
	failedNegotiation = false;
}

bool IPMP_MutualAuthentication::hasAuthenticationData() const
{
	return inclAuthenticationData;
}

void IPMP_MutualAuthentication::setAuthenticationData(const byte * pData, word32 dataSize)
{
	delete pAuthenticationData;

	pAuthenticationData = new IPMP_ByteArray();
	assert(pAuthenticationData != 0);
	pAuthenticationData->setData(pData, dataSize);
	inclAuthenticationData = true;
}

void IPMP_MutualAuthentication::clearAuthenticationData()
{
	delete pAuthenticationData;
	pAuthenticationData = 0;
	inclAuthenticationData = false;
}

byte * IPMP_MutualAuthentication::getAuthenticationData() const
{
	assert(pAuthenticationData);
	return pAuthenticationData->getData();
}

word32 IPMP_MutualAuthentication::getAuthenticationDataSize() const
{
	assert(pAuthenticationData);
	return pAuthenticationData->getDataSize();
}


bool IPMP_MutualAuthentication::hasAuthCodes() const
{
	return inclAuthCodes;
}

void IPMP_MutualAuthentication::setAuthCodes(AuthCodes * authCodes)
{
	assert(authCodes != 0);		// must be valid
	
	if (authCodes == 0) 
		return;						// cannot set as valid AuthCodes

	delete pAuthCodes;	

	pAuthCodes = new AuthCodes(*authCodes);
	inclAuthCodes = true;
}

AuthCodes * IPMP_MutualAuthentication::getAuthCodes() const
{
	return pAuthCodes;
}


/////////////////////////////////////////////////////////////////////////////
//
// BaseAuthenticationDescriptor
//

BaseAuthenticationDescriptor::BaseAuthenticationDescriptor(byte tag)
	: ExpandableBaseClass(tag)
{

}

BaseAuthenticationDescriptor::~BaseAuthenticationDescriptor()
{

}

void BaseAuthenticationDescriptor::clear()
{

}

int BaseAuthenticationDescriptor::toBuffer(byte * buf)
{
	int nbits = ExpandableBaseClass::toBuffer(buf);

	return nbits;
}

int BaseAuthenticationDescriptor::fromBuffer(const byte * buf)
{
	int nbits = ExpandableBaseClass::fromBuffer(buf);

	return nbits;
}

unsigned int BaseAuthenticationDescriptor::sizeOfInstance() const
{
	return 0;
}

/////////////////////////////////////////////////////////////////////////////
//
// AlgorithmDescriptor
//

AlgorithmDescriptor::AlgorithmDescriptor(byte tag)
	: BaseAuthenticationDescriptor(tag)
{
	isRegistered = false;
	regAuthId = 0;
	pSpecAlgoId = 0;
	pOpaqueData = 0;
}

AlgorithmDescriptor::AlgorithmDescriptor()
	: BaseAuthenticationDescriptor(IPMP_AlgorithmDescr_tag)
{
	isRegistered = false;
	regAuthId = 0;
	pSpecAlgoId = 0;
	pOpaqueData = 0;
}

AlgorithmDescriptor::~AlgorithmDescriptor()
{
	clear();
}

// clear all internal data
void AlgorithmDescriptor::clear()
{
	BaseAuthenticationDescriptor::clear();

	isRegistered = false;
	regAuthId = 0;
	delete pSpecAlgoId;
	pSpecAlgoId = 0;
	delete pOpaqueData;
	pOpaqueData = 0;
}

AlgorithmDescriptor::AlgorithmDescriptor(const AlgorithmDescriptor & ad)
	: BaseAuthenticationDescriptor(const_cast<AlgorithmDescriptor &>(ad).getObjectId())
{
	isRegistered = ad.isRegistered;
	regAuthId = ad.regAuthId;
	if (ad.pSpecAlgoId != 0 ) {
		pSpecAlgoId = new IPMP_ByteArray(*ad.pSpecAlgoId);
	} else {
		pSpecAlgoId = 0;
	}
	if (ad.pOpaqueData != 0) {
		pOpaqueData = new IPMP_ByteArray(*ad.pOpaqueData);
	} else {
		pOpaqueData = 0;
	}
}

bool AlgorithmDescriptor::operator==(const AlgorithmDescriptor & a_ad) const
{
	return equals(a_ad);
}

bool AlgorithmDescriptor::equals(const AlgorithmDescriptor & a_ad) const
{
	if (&a_ad == this) return true;
	if (a_ad.isRegistered == isRegistered) 
	{
		if (isRegistered) 
		{
			if (a_ad.regAuthId == regAuthId) 
				return true;
		} 
		else 
			return *pSpecAlgoId == *a_ad.pSpecAlgoId;
	}
	return false;
}

int AlgorithmDescriptor::toBuffer(byte * buf)
{
	BaseAuthenticationDescriptor::toBuffer(buf);

	put(isRegistered, 1);			// control bits
	put(0, 7);						// reserved
	
	if (isRegistered) 		
		put(regAuthId, 16);			// regAuthId
	else
		putObject(*pSpecAlgoId);		// specAlgoID

	assert(pOpaqueData != 0);		// opaqueData
	putObject(*pOpaqueData);

	return getWriteCounter();
}

int AlgorithmDescriptor::fromBuffer(const byte * buf)
{
	clear();

	BaseAuthenticationDescriptor::fromBuffer(buf);
	
	isRegistered = (get(1) != 0);	// control bits
	get(7);							// reserved

	if (isRegistered) {
		regAuthId = get(16);		// regAuthId
	} else {
		pSpecAlgoId = new IPMP_ByteArray();
		getObject(*pSpecAlgoId);		// specAlgoId
	}

	// opaqueData
	pOpaqueData = new IPMP_ByteArray();
	getObject(*pOpaqueData);			// read opaque data

	return (getReadCounter());
}


unsigned int AlgorithmDescriptor::sizeOfInstance() const
{
	unsigned int size = BaseAuthenticationDescriptor::sizeOfInstance();

	// control bits
	size += 1;

	if (isRegistered) {		
		size += 2;							// regAuthId
	} else {
		assert(pSpecAlgoId != 0);
		size += pSpecAlgoId->getSize();		// specAlgoID
	}

	// opaqueData
	assert(pOpaqueData != 0);
	size += pOpaqueData->getSize();

	return size;
}

bool AlgorithmDescriptor::getIsRegistered() const
{
	return isRegistered;
}

void AlgorithmDescriptor::setRegAuthId(word32 regAuthId)
{
	delete pSpecAlgoId;
	pSpecAlgoId = 0;

	this->regAuthId = regAuthId;

	isRegistered = true;
}

word32 AlgorithmDescriptor::getRegAuthId() const
{
	return regAuthId;
}

void AlgorithmDescriptor::setSpecAlgoId(const byte * pSpecAlgoId, word32 specAlgoIdSize)
{
	regAuthId = 0;

	delete this->pSpecAlgoId;
	if (pSpecAlgoId != 0 && specAlgoIdSize > 0) {
		this->pSpecAlgoId = new IPMP_ByteArray();
		assert(this->pSpecAlgoId != 0);
		this->pSpecAlgoId->setData(pSpecAlgoId, specAlgoIdSize);
	} else {
		this->pSpecAlgoId = 0;
	}

	isRegistered = false;
}

word32 AlgorithmDescriptor::getSpecAlgoIdSize() const
{
	return pSpecAlgoId->getDataSize();
}

byte * AlgorithmDescriptor::getSpecAlgoId() const
{
	return pSpecAlgoId->getData();
}

void AlgorithmDescriptor::setOpaqueData(const byte * pOpaqueData, word32 opaqueDataSize)
{
	delete this->pOpaqueData;
	if (pOpaqueData != 0 && opaqueDataSize > 0) {
		this->pOpaqueData = new IPMP_ByteArray();
		assert(this->pOpaqueData != 0);
		this->pOpaqueData->setData(pOpaqueData, opaqueDataSize);
	} else {
		this->pOpaqueData = 0;
	}
}

void AlgorithmDescriptor::clearOpaqueData()
{
	delete pOpaqueData;
	pOpaqueData = 0;
}

word32 AlgorithmDescriptor::getOpaqueDataSize() const
{
	return pOpaqueData->getDataSize();
}

byte * AlgorithmDescriptor::getOpaqueData() const
{
	return pOpaqueData->getData();
}
	
/////////////////////////////////////////////////////////////////////////////
//
// KeyDescriptor
//

KeyDescriptor::KeyDescriptor(byte tag)
	: BaseAuthenticationDescriptor(tag)
{
	keyBodySize = 0;
	pKeyBody = 0;
}

KeyDescriptor::KeyDescriptor()
	: BaseAuthenticationDescriptor(IPMP_KeyDescr_tag)
{
	keyBodySize = 0;
	pKeyBody = 0;
}

KeyDescriptor::KeyDescriptor(const KeyDescriptor & kd)
	: BaseAuthenticationDescriptor(kd)
{
	if (kd.keyBodySize > 0 && pKeyBody != 0) {
		keyBodySize = kd.keyBodySize;
		pKeyBody = new byte[keyBodySize];
		memcpy(pKeyBody, kd.pKeyBody, keyBodySize);
	} else {
		keyBodySize = 0;
		pKeyBody = 0;
	}
}

KeyDescriptor::~KeyDescriptor()
{
	clear();
}

void KeyDescriptor::clear()
{
	delete [] pKeyBody;
	pKeyBody = 0;
	keyBodySize = 0;
}

bool KeyDescriptor::equals(const KeyDescriptor & kd) const
{
	if ( kd.getObjectId() == this->getObjectId() ) 
	{
		// Both instances are KeyDescriptors;
		if (kd.keyBodySize == keyBodySize) 
		{
			if (memcmp(kd.pKeyBody, pKeyBody, keyBodySize) == 0) 
			{
				// and have the same key.
				return true;
			}
		}
	}
	return false;
}

int KeyDescriptor::toBuffer(byte * buf)
{
	BaseAuthenticationDescriptor::toBuffer(buf);

	// keyBody
	assert(keyBodySize == 0 || pKeyBody != 0);
	for (word32 kb_idx = 0; kb_idx < keyBodySize; ++kb_idx) {
		put(pKeyBody[kb_idx], 8);
	}

	return (getWriteCounter());
}

int KeyDescriptor::fromBuffer(const byte * buf)
{
	clear();

    BaseAuthenticationDescriptor::fromBuffer(buf);
	
	// keyBody
	keyBodySize = getSizeOfInstance();
	if (keyBodySize > 0) {
		pKeyBody = new byte[keyBodySize];
		assert(pKeyBody != 0);
		for (word32 kb_idx = 0; kb_idx < keyBodySize; ++kb_idx) {
			pKeyBody[kb_idx] = (byte) get(8);
		}
	}

	return (getReadCounter());
}


unsigned int KeyDescriptor::sizeOfInstance() const
{
	unsigned int size = BaseAuthenticationDescriptor::sizeOfInstance();
	
	// keyBodySize
	assert(keyBodySize == 0 || pKeyBody != 0);
	size += keyBodySize;

	return size;
}

void KeyDescriptor::setKeyBody(const byte * pKeyBody, word32 keyBodySize)
{
	delete [] this->pKeyBody;
	if (pKeyBody != 0 && keyBodySize > 0) {
		this->pKeyBody = new byte[keyBodySize];
		assert(this->pKeyBody != 0);
		memcpy(this->pKeyBody, pKeyBody, keyBodySize);
		this->keyBodySize = keyBodySize;
	} else {
		this->pKeyBody = 0;
		this->keyBodySize = 0;
	}
}

word32 KeyDescriptor::getKeyBodySize() const
{
	return keyBodySize;
}

byte * KeyDescriptor::getKeyBody() const
{
	return pKeyBody;
}

/////////////////////////////////////////////////////////////////////////////
//
// AuthCodes
//

AuthCodes::AuthCodes()
{
	type = 0;
	nCertificate = 0;
	certType = 0;
	ppCertificates = 0;
	pPublicKey = 0;
	pOpaque = 0;
	pAuthCodes = 0;
	pTrustData = 0;
}

AuthCodes::~AuthCodes()
{
	clear();
}

AuthCodes::AuthCodes(const AuthCodes & ac)
{
	type = ac.type;
	certType = ac.certType;
	nCertificate = ac.nCertificate;

	assert(ac.nCertificate == 0 || ac.ppCertificates != 0);
	if (ac.nCertificate != 0) {
		ppCertificates = new Certificate *[nCertificate];
		for (word32 nc = 0; nc < nCertificate; ++nc) {
			assert(ac.ppCertificates[nc] != 0);
			ppCertificates[nc] = new Certificate(*ac.ppCertificates[nc]);
			assert(ppCertificates[nc] != 0);
		}
	} else {
		nCertificate = 0;
		ppCertificates = 0;
	}

	if (ac.pPublicKey != 0) {
		pPublicKey = new KeyDescriptor(*ac.pPublicKey);
	} else {
		pPublicKey = 0;
	}

	if (ac.pOpaque != 0) {
		pOpaque = new IPMP_ByteArray(*ac.pOpaque);		// call the copy constructor
	} else {
		pOpaque = 0;
	}

	assert(ac.pTrustData != 0);
	pTrustData = new IPMP_TrustSecurityMetadata(*ac.pTrustData);

	assert(ac.pAuthCodes != 0);
	pAuthCodes = new IPMP_ByteArray(*ac.pAuthCodes);	// call the copy constructor
}

void AuthCodes::clear()
{
	type = 0;
	certType = 0;
	if (ppCertificates != 0) {
		for (word32 nc = 0; nc < nCertificate; ++nc) {
			delete ppCertificates[nc];
		}
		delete [] ppCertificates;
		ppCertificates = 0;
	}
	nCertificate = 0;
	delete pPublicKey;
	pPublicKey = 0;
	delete pOpaque;
	pOpaque = 0;
	delete pTrustData;
	pTrustData = 0;
	delete pAuthCodes;
	pAuthCodes = 0;
}

int AuthCodes::toBuffer(byte * buf)
{
	byte * acbuf = buf;
	long bitindex = 0;

	// type
	put(type, acbuf, bitindex, 8);
	bitindex += 8;

	if (type == 0x01) {
		assert(nCertificate == 0 || ppCertificates != 0);
		// nCertificate
		put(nCertificate, acbuf, bitindex, 32);
		bitindex += 32;
		// certType
		put(certType, acbuf, bitindex, 32);
		bitindex += 32;
		// ppCertificates
		for (word32 nc = 0; nc < nCertificate; ++nc) {
			assert(ppCertificates[nc] != 0);
			bitindex += ppCertificates[nc]->toBuffer(&acbuf[bitindex / 8]);
		}
	} else if (type == 0x02) {
		// publicKey
		assert(pPublicKey != 0);
		bitindex += pPublicKey->toBuffer(&acbuf[bitindex / 8]);
	} else if (type == 0xFE) {
		// opaque
		assert(pOpaque != 0);
		bitindex += pOpaque->toBuffer(&acbuf[bitindex / 8]);		
	}

	// trust data
	assert(pTrustData != 0);
	bitindex += pTrustData->toBuffer(&acbuf[bitindex / 8]);

	// authCodes
	assert(pAuthCodes != 0);
	bitindex += pAuthCodes->toBuffer(&acbuf[bitindex / 8]);

	return bitindex;
}

int AuthCodes::fromBuffer(const byte * buf)
{
	clear();

	const byte * acbuf = buf;
	long bitindex = 0;

	// type
	type = get(acbuf, bitindex, 8);
	bitindex += 8;

	if (type == 0x01) {
		// nCertificate
		nCertificate = get(acbuf, bitindex, 32);
		bitindex += 32;
		// certType
		certType = get(acbuf, bitindex, 32);
		bitindex += 32;
		// ppCertificates
		if (nCertificate > 0) {
			ppCertificates = new Certificate *[nCertificate];
			assert(ppCertificates != 0);
			for (word32 nc = 0; nc < nCertificate; ++nc) {
				ppCertificates[nc] = new Certificate;
				assert(ppCertificates[nc] != 0);
				bitindex += ppCertificates[nc]->fromBuffer(&acbuf[bitindex / 8]);
			}
		}
	} else if (type == 0x02) {
		// publicKey
		pPublicKey = new KeyDescriptor;
		assert(pPublicKey != 0);
		bitindex += pPublicKey->fromBuffer(&acbuf[bitindex / 8]);
	} else if (type == 0xFE) {
		// opaque
		delete pOpaque;
		pOpaque = new IPMP_ByteArray();
		assert(pOpaque != 0);
		bitindex += pOpaque->fromBuffer(&acbuf[bitindex / 8]);
	}

	// trust data
	delete pTrustData;
	pTrustData = new IPMP_TrustSecurityMetadata();
	assert(pTrustData != 0);
	bitindex += pTrustData->fromBuffer(&acbuf[bitindex / 8]);

	// authCodes
	delete pAuthCodes;
	pAuthCodes = new IPMP_ByteArray();
	bitindex += pAuthCodes->fromBuffer(&acbuf[bitindex / 8]);

	return bitindex;
}
	
bool AuthCodes::checkSyntax() const
{
	return true; // not implemented yet
}

unsigned int AuthCodes::getSize() const 
{
	unsigned int size = 0;

	// type
	size += 1;

	if (type == 0x01) {
		assert(nCertificate == 0 || ppCertificates != 0);
		// nCertificate
		size += 4;
		// certType
		size += 4;
		// ppCertificates
		for (word32 nc = 0; nc < nCertificate; ++nc) {
			assert(ppCertificates[nc] != 0);
			size += ppCertificates[nc]->getSize();
		}
	} else if (type == 0x02) {
		// publicKey
		assert(pPublicKey != 0);
		size += pPublicKey->getSize();		
	} else if (type == 0xFE) {
		// opaque
		assert(pOpaque != 0);
		size += pOpaque->getSize();
	}

	// trust data
	size += pTrustData->getSize();

	// authCodes
	size += pAuthCodes->getSize();

	return size;
}

word32 AuthCodes::getType() const
{
	return type;
}

void AuthCodes::clearType()
{
	word32 nc;

	// clear previous 0x01 data (if existing)
	if (this->ppCertificates != 0) {
		for (nc = 0; nc < this->nCertificate; ++ nc) {
			delete this->ppCertificates[nc];
		}
		delete [] this->ppCertificates;
		this->ppCertificates = 0;
	}
	this->nCertificate = 0;
	this->certType = 0;
	// clear previous 0x02 data (if existing)
	delete pPublicKey;
	pPublicKey = 0;
	// clear previous 0xFE data (if existing)
	delete pOpaque;
	pOpaque = 0;

	type = 0x00;
}

void AuthCodes::setCertificates(word32 certType, Certificate * * ppCertificates, word32 nCertificate)
{
	assert(nCertificate == 0 || ppCertificates != 0);

	word32 nc;

	clearType();

	// set new data
	this->certType = certType;
	this->nCertificate = nCertificate;
	if (nCertificate > 0) {
		this->ppCertificates = new Certificate *[nCertificate];
		assert(this->ppCertificates != 0);
		for (nc = 0; nc < nCertificate; ++nc) {
			assert(ppCertificates[nc]);
			this->ppCertificates[nc] = new Certificate(*(ppCertificates[nc]));
			assert(this->ppCertificates[nc] != 0);
		}
	}

	// set appropriate type
	type = 0x01;
}

word32 AuthCodes::getCertificateType() const
{
	return certType;
}

word32 AuthCodes::getNumCertificates() const
{
	return nCertificate;
}

Certificate * * AuthCodes::getCertificates() const
{
	return ppCertificates;
}

void AuthCodes::setPublicKey(KeyDescriptor * pPublicKey)
{
	clearType();

	if (pPublicKey != 0) {
		this->pPublicKey = new KeyDescriptor(*pPublicKey);
	}

	type = 0x02;
}

KeyDescriptor * AuthCodes::getPublicKey() const
{
	return pPublicKey;
}

void AuthCodes::setOpaque(const byte * opaque, word32 opaqueSize)
{
	clearType();

	if (opaqueSize > 0) {
		type = 0xFE;
		pOpaque = new IPMP_ByteArray();
		assert(pOpaque != 0);
		pOpaque->setData(opaque, opaqueSize);
	} 
}

word32 AuthCodes::getOpaqueSize() const
{
	return pOpaque->getSize();
}

byte * AuthCodes::getOpaque() const
{
	return pOpaque->getData();
}

void AuthCodes::setAuthCodes(const byte * authCodes, word32 authCodesSize, IPMP_TrustSecurityMetadata * trustData)
{
	assert(authCodes != 0);			// must be valid
	assert(authCodesSize > 0);		// must be valid
	assert(trustData != 0);			// must be valid

	delete pAuthCodes;
	delete pTrustData;

	pAuthCodes = new IPMP_ByteArray();
	assert(pAuthCodes != 0);
	pAuthCodes->setData(authCodes, authCodesSize);

	pTrustData = new IPMP_TrustSecurityMetadata(*trustData);
}

void AuthCodes::clearAuthCodes()
{
	clearType();
	
	delete pAuthCodes;
	pAuthCodes = 0;
}

word32 AuthCodes::getAuthCodesSize() const
{
	assert(pAuthCodes);
	return pAuthCodes->getSize();
}

byte * AuthCodes::getAuthCodes() const
{
	assert(pAuthCodes);
	return pAuthCodes->getData();
}


IPMP_TrustSecurityMetadata * AuthCodes::getTrustData() const
{
	return pTrustData;
}


