
//	This software module was originally developed by ADETTI in the course of
//	development of the MPEG-4 Systems (ISO/IEC 14496-1) standard. 
//	This software module is an implementation of a part of one or more MPEG-4 
//	Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 
//	14496-1) standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	ADETTI retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2002.

// IPMP_MutualAuthentication.h

#ifndef	IPMP_MUTUALAUTHENTICATION_H
#define	IPMP_MUTUALAUTHENTICATION_H

#include "IPMP_Data_BaseClass.h"

#include "namespace.h"

BEGIN_MOSES_NAMESPACE

class AlgorithmDescriptor;
class KeyDescriptor;
class AuthCodes;
class IPMP_ByteArray;
class IPMP_TrustSecurityMetadata;

typedef IPMP_ByteArray Certificate;			// rename ByteArray as "Certificate" indicating a ByteArray holding a Certificate

/** @class IPMP_MutualAuthentication
 *  Messages exchanged during mutual authentication process.
 *  An instance of IPMP_Mutual_Authentication is generated by and exchanged
 *  between IPMP tools, and IPMP Tools and a Terminal for the purpose of mutual
 *  authentication.
 *  @author ADETTI
 *  @date 2002
 */
class IPMP_MutualAuthentication : public IPMP_Data_BaseClass
{
private:
	bool requestNegotiation;
	bool successNegotiation;
	bool failedNegotiation;
	bool inclAuthenticationData;
	bool inclAuthCodes;

	// if requestNegotiation;
	word32 nCandidate;
	AlgorithmDescriptor * * ppCandidateAlgorithms;

	// if successNegotiation;
	word32 nAlgorithms;
	AlgorithmDescriptor * * ppAgreedAlgorithms;

	// if inclAuthenticationData
	IPMP_ByteArray * pAuthenticationData;

	// if inclAuthCodes;
	AuthCodes * pAuthCodes;

	
	// clear all internal data
	virtual void clear();

public:
	/**
	 * Constructor.
	 */
	IPMP_MutualAuthentication();					
	
	/**
	 * Destructor.
	 */
	virtual ~IPMP_MutualAuthentication();		
	
	/**
	 * Copy constructor
	 */
	IPMP_MutualAuthentication(const IPMP_MutualAuthentication & ma);

	virtual int toBuffer(byte * buf);				// reimplemented methods
	virtual int fromBuffer(const byte * buf);
	virtual unsigned int sizeOfInstance() const;

	/**
	 * Sets the list of #AlgorithmDescriptor for the algorithm negotiation.
	 * @param ppCandidateAlgorithms address of an array of pointers to 
	 *                              #AlgorithmDescriptor.
	 * @param nCandidate number of pointers to #AlgorithmDescriptor in the
	 *                   array pointed by ppCandidateAlgorithms.
	 */
	void setRequestNegotiation(AlgorithmDescriptor * * ppCandidateAlgorithms, word32 nCandidate);
	
	/**
	 * Clears the requestNegociation bit and related fields.
	 */
	void clearRequestNegotiation();
	
	/**
	 * Check the requestNegociation bit.
	 * Must be called before #getNumCandidateAlgorithms or
	 * #getCandidateAlgorithms.
	 * @return <i>true</i> if the requestNegotiation bit is set to 1.
	 */
	bool getRequestNegotiation() const;
	
	/**
	 * Get the number of candidate #AlgorithmDescriptor stored in the
	 * message.
	 * First call #getRequestNegotiation to know if this information
	 * is available.
	 * @return the number of #AlgorithmDescriptor present.
	 */
	word32 getNumCandidateAlgorithms() const;
	
	/**
	 * Get the array of pointers to candidate #AlgorithmDescriptor present
	 * in the message.
	 * First call #getRequestNegotiation to know if this information
	 * is available.
	 * @return the array of pointers to candidate #AlgorithmDescriptor.
	 */
	AlgorithmDescriptor * * getCandidateAlgorithms() const;

	/**
	 * Sets the list of agreed #AlgorithmDescriptor.
	 * @param ppAgreedAlgorithms address of an array of pointers to 
	 *                           #AlgorithmDescriptor.
	 * @param nAlgorithms number of pointers to #AlgorithmDescriptor in the
	 *                array pointed by ppAgreedAlgorithms.
	 */
	void setSuccessNegotiation(AlgorithmDescriptor * * ppAgreedAlgorithms, word32 nAlgorithms);
	
	/**
	 * Clears the successNegociation bit and related fields.
	 */
	void clearSuccessNegotiation();
	
	/**
	 * Check the successNegociation bit.
	 * Must be called before #getNumAgreedAlgorithms or
	 * #getAgreedAlgorithms.
	 * @return <i>true</i> if the successNegotiation bit is set to 1.
	 */
	bool getSuccessNegotiation() const;
	
	/**
	 * Get the number of agreed #AlgorithmDescriptor stored in the message.
	 * First call #getSuccessNegotiation to know if this information
	 * is available.
	 * @return the number of #AlgorithmDescriptor present.
	 */
	word32 getNumAgreedAlgorithms() const;
	
	/**
	 * Get the array of pointers to agreed #AlgorithmDescriptor present
	 * in the message.
	 * First call #getSuccessNegotiation to know if this information
	 * is available.
	 * @return the array of pointers to agreed #AlgorithmDescriptor.
	 */
	AlgorithmDescriptor * * getAgreedAlgorithms() const;

	/**
	 * Set the failedNegociation bit.
	 */
	void setFailedNegotiation();
	
	/**
	 * Clear the failedNegotiation bit.
	 */
	void clearFailedNegotiation();
	
	/**
	 * Get the value of the failedNegociation bit.
	 * @return <i>true</i> if failedNegotiation is 1.
	 */
	bool getFailedNegotiation() const;

	/**
	 * Set authentication data.
	 * @param pAuthenticationData start address of authentication data.
	 * @param authenticationDataSize size (in bytes) of authentication data.
	 */
	void setAuthenticationData(const byte * pAuthenticationData, word32 authenticationDataSize);
	
	/**
	 * Clears the inclAuthenticationData bit and related fields.
	 */
	void clearAuthenticationData();
	
	/**
	 * Check the inclAuthenticationData bit.
	 * Must be called before #getAuthenticationData or
	 * #getAuthenticationDataSize.
	 * @return <i>true</i> if the inclAuthenticationData bit is set to 1.
	 */
	bool hasAuthenticationData() const;
	
	/**
	 * Get pointer to authentication data.
	 * First call #hasAuthenticationData to know if this information
	 * is available.
	 * @return pointer to authentication data.
	 */
	byte * getAuthenticationData() const;
	
	/**
	 * Get size of authentication data.
	 * First call #hasAuthenticationData to know if this information
	 * is available.
	 * @return size of authentication data.
	 */
	word32 getAuthenticationDataSize() const;

	/**
	 * Set authentication codes for the message.
	 * @param pAuthCodes authentication codes for the message.
	 */
	void setAuthCodes(AuthCodes * pAuthCodes);
	
	/**
	 * Clear inclAuthCodes bit and related fields.
	 */
	void clearAuthCodes();
	
	/**
	 * Get the value of inclAuthCodes bit.
	 * Must be called before #getAuthCodes.
	 * @return <i>true</i> if inclAuthCodes is 1.
	 */
	bool hasAuthCodes() const;
	
	/**
	 * Get the authentication codes from the message.
	 * First call #hasAuthCodes to know if this information is available.
	 * @return authentication codes for the message.
	 */
	AuthCodes * getAuthCodes() const;
};

// defines for BaseAuthenticationDescriptor

#define IPMP_AlgorithmDescr_tag		0x01
#define IPMP_KeyDescr_tag			0x02


/** @class BaseAuthenticationDescriptor
 *  The base for authentication descriptors.
 *  @author ADETTI
 *  @date 2002
 */
class BaseAuthenticationDescriptor : public ExpandableBaseClass
{
protected:
	/**
	 * clear all internal data
	 */
	virtual void clear();

	/**
	 * Constructor.
	 * To be called by derived classes only.
	 * @param tag the type of descriptor, according to the table:
	 *            <table>
	 *                <tr><td>0x00</td><td>Forbidden</td></tr>
	 *                <tr><td>0x01</td><td>IPMP_AlgorithmDescr_tag</td></tr>
	 *                <tr><td>0x02</td><td>IPMP_KeyDescr_tag</td></tr>
	 *                <tr><td>0x03-0x7F</td><td>ISO Reserved</td></tr>
	 *                <tr><td>0x7F-0xFE</td><td>User defined</td></tr>
	 *                <tr><td>0xFF</td><td>Forbidden</td></tr>
	 *            </table>
	 */
	BaseAuthenticationDescriptor(byte tag);
public:
	/**
	 * Destructor.
	 */
	virtual ~BaseAuthenticationDescriptor();
	
	/**
	 * Method for writing the message using the data of the class.
	 * @param buf storage location for the byte sequence. 
	 */
	virtual int toBuffer(byte * buf);
	/**
	 * Method for reading the contents of the message.
	 * @param buf storage location of the byte sequence. 
	 */
	virtual int fromBuffer(const byte * buf);

	/**
	 * Get the total size of the class, when converted into a bitstream. 
	 * This size does NOT include the object_id and the length of the base
	 * class. 
	 * @return the size in bytes. 
	 */
	virtual unsigned int sizeOfInstance() const;
};

/** @class AlgorithmDescriptor
 *  This class is for specifying an identifier of an arbitrary algorithm.
 *  @author ADETTI
 *  @date 2002
 */
class AlgorithmDescriptor : public BaseAuthenticationDescriptor
{
//protected:
	bool isRegistered;
	word32 regAuthId;
	IPMP_ByteArray * pSpecAlgoId;
	IPMP_ByteArray * pOpaqueData;

	// clear all internal data
	virtual void clear();

	/**
	 * Constructor.
	 * To be used by derived classes only.
	 * @param tag the type of descriptor, according to the table:
	 *            <table>
	 *                <tr><td>0x00</td><td>Forbidden</td></tr>
	 *                <tr><td>0x01</td><td>IPMP_AlgorithmDescr_tag</td></tr>
	 *                <tr><td>0x02</td><td>IPMP_KeyDescr_tag</td></tr>
	 *                <tr><td>0x03-0x7F</td><td>ISO Reserved</td></tr>
	 *                <tr><td>0x7F-0xFE</td><td>User defined</td></tr>
	 *                <tr><td>0xFF</td><td>Forbidden</td></tr>
	 *            </table>
	 */
	AlgorithmDescriptor(byte tag);

	virtual bool equals(const AlgorithmDescriptor & ad) const;

public:
	/**
	 * Constructor.
	 */
	AlgorithmDescriptor();
	/**
	 * Destructor.
	 */
	virtual ~AlgorithmDescriptor();

	/**
	 * Copy constructor.
	 */
	AlgorithmDescriptor(const AlgorithmDescriptor &);

	/**	
	 * The operator== function.
	 * @param	ad	a parameter of type const AlgorithmDescriptor &
	 * @return	true if the two object are equal
	 */
	bool operator==(const AlgorithmDescriptor & ad) const;

	/**
	 * Method for writing the message using the data of the class.
	 * @param buf storage location for the byte sequence. 
	 */
	virtual int toBuffer(byte * buf);		
	/**
	 * Method for reading the contents of the message.
	 * @param buf storage location of the byte sequence. 
	 */
	virtual int fromBuffer(const byte * buf);
	
	/**
	 * Get the total size of the class, when converted into a bitstream. 
	 * This size does NOT include the object_id and the length of the base
	 * class. 
	 * @return the size in bytes. 
	 */
	virtual unsigned int sizeOfInstance() const;

	/**
	 * Check if ID included is a normative identifier of the algorithm or
	 * an ID assigned by a registration authority.
	 * Must be called before #getRegAuthId, #getSpecAlgoId or
	 * #getSpecAlgoIdSize.
	 * @return <i>true</i> if the ID has been assigned by a registration
	 *                     authority.
	 */
	bool getIsRegistered() const;

	/**
	 * Set the 16-bit ID assigned by a registration authority to this
	 * algorithm.
	 * @param regAuthId the 16-bit ID.
	 */
	void setRegAuthId(word32 regAuthId);
	/**
	 * Get the 16-bit ID assigned by a registration authority to this
	 * algorithm.
	 * First call #getIsRegistered to know if this information is available.
	 * @return the 16-bit ID.
	 */
	word32 getRegAuthId() const;
	
	/**
	 * <p>Set a normative ID according to the following table:</p>
	 *
	 *     <table>
	 *         <tr>
	 *             <td><b>SpecAlgoID&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</b></td>
	 *             <td><b>Symbol&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</b></td>
	 *             <td><b>Content</b></td>
	 *             <td><b>Confidentiality</b></td>
	 *             <td><b>Integrity</b></td>
	 *         </tr>
	 *         <tr>
	 *             <td>0x0001</td>
	 *             <td>DH-G-H-x-y</td>
	 *             <td>Diffie-Hellman Key Agreement Scheme on a base group
	 *                 G. The parameter setting of the both parties is
	 *                 specified in certificates.</td>
	 *             <td>No</td>
	 *             <td>Yes</td>
	 *         </tr>
	 *         <tr>
	 *             <td>0x0002</td>
	 *             <td>EDH-G-H-x-y</td>
	 *             <td>Ephemeral Diffie-Hellman Key Agreement Scheme on a
	 *                 base group G. Both parties are assumed to agree on
	 *                 the base group G, and each party is supposed to
	 *                 generate ephemeral parameters (a public key pair)
	 *                 for the purpose of authentication and the parameters
	 *                 are submitted being signed by the party.  A
	 *                 certificate of the party is used for the opponent
	 *                 party to verify the signature.</td>
	 *             <td>No</td>
	 *             <td>Yes</td>
	 *         </tr>
	 *         <tr>
	 *             <td>0x0003</td>
	 *             <td>DH-G-E-H-x-y-z</td>
	 *             <td>DH-G-H-x-y being used in combination with a
	 *                 symmetric key cipher algorithm E.</td>
	 *             <td>Yes</td>
	 *             <td>Yes</td>
	 *         </tr>
	 *         <tr>
	 *             <td>0x0004</td>
	 *             <td>EDH-G-E-H-x-y-z</td>
	 *             <td>EDH-G-H-x-y being used in combination with a
	 *                 symmetric key cipher algorithm E.</td>
	 *             <td>Yes</td>
	 *             <td>Yes</td>
	 *         </tr>
	 *         <tr>
	 *             <td>0x0005</td>
	 *             <td>RSA-OAEP-H-x</td>
	 *             <td>Needham-Schroeder Scheme deploying RSA-based OAEP as
	 *                 the public key encryption.</td>
	 *             <td>No</td>
	 *             <td>Yes</td>
	 *         </tr>
	 *         <tr>
	 *             <td>0x0006</td>
	 *             <td>RSA-OAEP-E-H-x-z</td>
	 *             <td>RSA-OAEP-H-x being used in combination with a
	 *                 symmetric key cipher algorithm E.</td>
	 *             <td>Yes</td>
	 *             <td>No</td>
	 *         </tr>
	 *         <tr>
	 *             <td>0x0007</td>
	 *             <td>SCHNRR-G-x-y</td>
	 *             <td>Schnorr Identification Scheme on a base group G.</td>
	 *             <td>No</td>
	 *             <td>No</td>
	 *         </tr>
	 *         <tr>
	 *             <td>0x0008-0x00FF</td>
	 *             <td>ISO Reserved</td>
	 *             <td>&nbsp;</td>
	 *             <td>&nbsp;</td>
	 *             <td>&nbsp;</td>
	 *         </tr>
	 *         <tr>
	 *             <td>0x0100-0x01FE</td>
	 *             <td>User Defined</td>
	 *             <td>&nbsp;</td>
	 *             <td>&nbsp;</td>
	 *             <td>&nbsp;</td>
	 *         </tr>
	 *         <tr>
	 *             <td>0x01FF-0xFFFF</td>
	 *             <td>Forbidden</td>
	 *             <td>For use by registration authority</td>
	 *             <td>&nbsp;</td>
	 *             <td>&nbsp;</td>
	 *         </tr>
	 *     </table>
	 *     
	 * <p>In the above table, the significance of the variables is as
	 *    follows:</p>
	 *
	 *     <table>
	 *         <tr>
	 *             <td><b>Variable</b>
	 *             </td><td><b>Definition</b></td>
	 *         </tr>
	 *         <tr>
	 *             <td>G</td>
	 *             <td>Specifies the type of a base group
	 *                 (e.g. a sub-groups of an elliptic curve)</td>
	 *         </tr>
	 *         <tr>
	 *             <td>E</td>
	 *             <td>Specifies a symmetric key cipher algorithm to
	 *                 realize confidentiality of messages.</td>
	 *         </tr>
	 *         <tr>
	 *             <td>H</td>
	 *             <td>Specifies a hash function to realize message
	 *                 integrity.</td>
	 *         </tr>
	 *         <tr>
	 *             <td>x</td>
	 *             <td>Specifies a size of a base group (e.g. the length in
	 *                 bits of the base field of an elliptic curve).</td>
	 *         </tr>
	 *         <tr>
	 *             <td>y</td>
	 *             <td>Specifies a size of a base group (e.g. the length in
	 *                 bits of the order of a base group).</td>
	 *         </tr>
	 *         <tr>
	 *             <td>z</td>
	 *             <td>Specifies the length in bits of keys of a symmetric
	 *                 key cipher to be used in conjunction.</td>
	 *         </tr>
	 *     </table>
	 * @note The actual cryptographic scheme is constructed on a proper
	 *       sub-group instead of being constructed on an entire elliptic
	 *       curve or an entire multiplicative group of a finite field.
	 *       Further, the order of the base sub-group shall be a prime
	 *       number, and its length is specified to be y bits. This value
	 *       of y influences the strength of the scheme against the
	 *       Pohlig-Hellman method, which finds out solutions to DLP on
	 *       arbitrary groups.
	 * @param pSpecAlgoId normative identifier of the algorithm.
	 * @param specAlgoIdSize size of the normative identifier.
	 */
	void setSpecAlgoId(const byte * pSpecAlgoId, word32 specAlgoIdSize);
	/**
	 * Get the size of SpecAlgoId.
	 * First call #getIsRegistered to know if this information is available.
	 * @return the size of SpecAlgoId.
	 */
	word32 getSpecAlgoIdSize() const;
	/**
	 * Get the SpecAlgoId.
	 * First call #getIsRegistered to know if this information is available.
	 * @return pointer to the start of SpecAlgoId.
	 */
	byte * getSpecAlgoId() const;

	/**
	 * Set opaque data.
	 * @param pOpaqueData opaque data.
	 * @param opaqueDataSize opaque data size.
	 */
	void setOpaqueData(const byte * pOpaqueData, word32 opaqueDataSize);
	/**
	 * Clear opaque data.
	 */
	void clearOpaqueData();
	/**
	 * Set opaque data size.
	 * @return opaque data size.
	 */
	word32 getOpaqueDataSize() const;
	/**
	 * Get opaque data.
	 * @return pointer to start of opaque data.
	 */
	byte * getOpaqueData() const;
};

/** @class KeyDescriptor
 *  This class is for specifying a cryptographic algorithm and a key conforming to the algorithm.
 *  @author ADETTI
 *  @date 2002
 */
class KeyDescriptor : public BaseAuthenticationDescriptor
{
private:
	word32 keyBodySize;
	byte * pKeyBody;

protected:

	/**
	 * clear all internal data
	 */
	virtual void clear();

	/**
	 * Constructor.
	 * To be used by derived classes only.
	 * @param tag the type of descriptor, according to the table:
	 *            <table>
	 *                <tr><td>0x00</td><td>Forbidden</td></tr>
	 *                <tr><td>0x01</td><td>IPMP_AlgorithmDescr_tag</td></tr>
	 *                <tr><td>0x02</td><td>IPMP_KeyDescr_tag</td></tr>
	 *                <tr><td>0x03-0x7F</td><td>ISO Reserved</td></tr>
	 *                <tr><td>0x7F-0xFE</td><td>User defined</td></tr>
	 *                <tr><td>0xFF</td><td>Forbidden</td></tr>
	 *            </table>
	 */
	KeyDescriptor(byte tag);

	/**
	 * Compare this object with another, and return true if they contain the same data.
	 * @return true if the two objects contain the same data.
	 */
	virtual bool equals(const KeyDescriptor & ad) const;

public:
	/**
	 * Constructor.
	 */
	KeyDescriptor();
	/**
	 * Destructor.
	 */
	virtual ~KeyDescriptor();

	/**
	 * Copy constructor.
	 */
	KeyDescriptor(const KeyDescriptor &);

	virtual int toBuffer(byte * buf);				// reimplemented methods
	virtual int fromBuffer(const byte * buf);
	virtual unsigned int sizeOfInstance() const;

	/**
	 * Set the value of the key.
	 * @param pKeyBody the key
	 * @param keyBodySize the size of the key
	 */
	void setKeyBody(const byte * pKeyBody, word32 keyBodySize);
	/**
	 * Get the size of the key.
	 * @return the size of the key
	 */
	word32 getKeyBodySize() const;
	/**
	 * Get the value of the key.
	 * @return pointer to the start of the key
	 */
	byte * getKeyBody() const;
};



/** @class AuthCodes
 *  Authentication codes for #IPMP_MutualAuthentication messages.
 *  @author ADETTI
 *  @date 2002
 */
class AuthCodes : public BitField
{
private:
	word32 type;					// the type of authentication code
	word32 nCertificate;			// type == 0x01
	word32 certType;
	Certificate * * ppCertificates;
	KeyDescriptor * pPublicKey;		// type == 0x02
	IPMP_ByteArray * pOpaque;		// type == 0xFE
	IPMP_TrustSecurityMetadata * pTrustData;	// always present
	IPMP_ByteArray * pAuthCodes;				// always present

	// clear all internal data
	virtual void clear();

public:
	/**
	 * Constructor.
	 */
	AuthCodes();
	/**
	 * Destructor.
	 */
	virtual ~AuthCodes();

	/**
	 * Copy constructor.
	 */
	AuthCodes(const AuthCodes &);

	virtual int toBuffer(byte * buf);				// reimplemented methods	
	virtual int fromBuffer(const byte * buf);
	virtual bool checkSyntax() const;
	virtual unsigned int getSize() const;

	/**
	 * Get the type of the authentication codes. Semantics of each value for this variable is defined as follows:
	 * 
	 *     <table>
	 *         <tr>
	 *             <td><b>Value&nbsp;of&nbsp;Type</b></td>
	 *             <td><b>Semantics</b></td>
	 *         </tr>
	 *         <tr>
	 *             <td>0x00</td>
	 *             <td>No information regarding the type is explicitly
	 *                 specified.</td>
	 *         </tr>
	 *         <tr>
	 *             <td>0x01</td>
	 *             <td>The value specified in the variable authCodes is
	 *                 digital signature. One or more SPKI certificates
	 *                 or X.509 V3 certificates, depending on the value
	 *                 of certType are specified as a method to verify
	 *                 the signature in the variable certificates[].</td>
	 *         </tr>
	 *         <tr>
	 *             <td>0x02</td>
	 *             <td>The value specified in the variable authCodes is
	 *                 a digital signature. At the same time, a public
	 *                 key accompanied by algorithm specification is
	 *                 specified as a method to verify the signature.</td>
	 *         </tr>
	 *         <tr>
	 *             <td>0x03-0xDF</td>
	 *             <td>ISO Reserved</td>
	 *         </tr>
	 *         <tr>
	 *             <td>0xE0-0xFD</td>
	 *             <td>User Private</td>
	 *         </tr>
	 *         <tr>
	 *             <td>0xFE</td>
	 *             <td>Application-dependent information regarding the
	 *                 type is specified in the variable opaque.</td>
	 *         </tr>
	 *         <tr>
	 *             <td>0xFF</td>
	 *             <td>Forbidden</td>
	 *         </tr>
	 *     </table>
	 *
	 * Must be called before #getCertificateType, #getCertificates,
	 * #getNumCertificates, #getPublicKey, #getOpaqueSize and #getOpaque.
	 * @return the type of authentication codes
	 */
	word32 getType() const;

	/**
	 * Clear type information.
	 */
	void clearType();

	/**
	 * Set certificates and set type to 0x01.
	 * @param certType the type of certificates
	 * @param ppCertificates array of pointers to Certificate
	 * @param nCertificate number of certificates in ppCertificates
	 */
	void setCertificates(word32 certType, Certificate * * ppCertificates, word32 nCertificate);
	/**
	 * Get the type of certificates stored in this object.
	 * First call #getType to know if this information is available.
	 * @return type of certificates
	 */
	word32 getCertificateType() const;
	/**
	 * Get the number of certificates stored in this object.
	 * First call #getType to know if this information is available.
	 * @return number of certificates
	 */
	word32 getNumCertificates() const;
	/**
	 * Get the certificates stored in this object.
	 * First call #getType to know if this information is available.
	 * @return array of certificates
	 */
	Certificate * * getCertificates() const;

	/**
	 * Set the public key and set type to 0x02.
	 * @param pPublicKey public key
	 */
	void setPublicKey(KeyDescriptor * pPublicKey);
	/**
	 * Get the public key.
	 * First call #getType to know if this information is available.
	 * @return public key
	 */
	KeyDescriptor * getPublicKey() const;

	/**
	 * Set opaque data and set type to 0xFE.
	 * @param pOpaque opaque data
	 * @param opaqueSize size of opaque data
	 */
	void setOpaque(const byte * pOpaque, word32 opaqueSize);
	
	/**
	 * Get size of opaque data.
	 * First call #getType to know if this information is available.
	 * @return size of opaque data
	 */
	word32 getOpaqueSize() const;
	
	/**
	 * Get opaque data.
	 * First call #getType to know if this information is available.
	 * @return pointer to start of opaque data
	 */
	byte * getOpaque() const;

	/**
	 * Set authentication codes data.
	 * @param pAuthCodes pointer to start of authentication codes data
	 * @param authCodesSize size of authentication codes data
	 * @param pTrustData trust and security metadata for the message.
	 */
	void setAuthCodes(const byte * pAuthCodes, word32 authCodesSize, IPMP_TrustSecurityMetadata * pTrustData);
	
	/**
	 * Clear authentication codes data.
	 */
	void clearAuthCodes();
	
	/**
	 * Get the size of authentication codes data.
	 * @return size of authentication codes data
	 */
	word32 getAuthCodesSize() const;

	/**
	 * Get authentication codes data.
	 * @return pointer to start of authentication codes data
	 */
	byte * getAuthCodes() const;

	/**
	 * Get the trust and security metadata from the message.
	 * First call #hasAuthCodes to know if this information is available.
	 * @return trust information.
	 */
	IPMP_TrustSecurityMetadata * getTrustData() const;
};

END_MOSES_NAMESPACE

#endif // IPMP_MUTUALAUTHENTICATION_H

