
//	This software module was originally developed by Claudio Alberti (EPFL)
//  and modified by Massimo Balestri (TILAB)	
//	in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1) standard. 
//	This software module is an implementation of a part of one or more MPEG-4 
//	Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 
//	14496-1) standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	EPFL retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2002.

// IPMP_OpaqueData.h

#ifndef	IPMP_OPAQUEDATA_H
#define	IPMP_OPAQUEDATA_H

#include "IPMP_Data_BaseClass.h"
#include "IPMP_ByteArray.h"
#include "namespace.h"

BEGIN_MOSES_NAMESPACE

/** @class IPMP_OpaqueData
 *  This class is used for carriage of opaque data.
 *
 *  @author Claudio Alberti, Massimo Balestri
 *  @date 2002
 */
class IPMP_OpaqueData: public IPMP_Data_BaseClass 
{
private:
	IPMP_ByteArray opaqueData;		///< the private data container

public:
	IPMP_OpaqueData();				///< constructor.
	virtual ~IPMP_OpaqueData();		///< destructor.
	virtual int toBuffer(byte * buf);				// reimplemented
	virtual int fromBuffer(const byte * buf);		// reimplemented
	virtual unsigned int sizeOfInstance() const;	// reimplemented

	/**
	 * Returns the size of the opaque data
	 * @returns size of opaque data
	 */
	int getOpaqueDataSize() const;

	/**
	 * Returns the opaque data as an IPMP_ByteArray reference
	 * @returns reference to IPMP_ByteArray
	 */
	const IPMP_ByteArray& getOpaqueData() const;

	/**
	 * Sets the opaque data 
	 * @param od constant reference to IPMP_ByteArray that is copied into this opaque
	 * data class
	 */
	void setOpaqueData(const IPMP_ByteArray& od);

	/**
	 * Sets the opaque data 
	 * @param buf constant pointer to data that is copied into this opaque
	 * data class
	 * @param length length of the buffer in bytes
	 */
	void setOpaqueData(const byte* buf, unsigned int length){ opaqueData.setData(buf, length); };
};

END_MOSES_NAMESPACE

#endif
