
//	This software module was originally developed by Massimo Balestri (TILAB)	
//	in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1) standard. 
//	This software module is an implementation of a part of one or more MPEG-4 
//	Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 
//	14496-1) standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	Telecom Italia Lab retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2002.


#include "IPMP_ParametricDescription.h"
#include <memory.h>

USING_MOSES_NAMESPACE

/////////////////////// class Description //////////////////////////

Description::Description()
{
}

Description::~Description() 
{
}


int Description::toBuffer(byte * buf)
{
	setWriteBuffer(buf);
	skipWriting(toolClass.toBuffer( getWritePtr())) ;
	skipWriting(toolSubClass.toBuffer( getWritePtr())) ;
	skipWriting(toolTypeData.toBuffer( getWritePtr())) ;
	skipWriting(toolType.toBuffer( getWritePtr())) ;
	skipWriting(addedData.toBuffer( getWritePtr())) ;
	return getWriteCounter() ;
}


int Description::fromBuffer(const byte * buf)
{
	setReadBuffer(buf);
	skipReading(toolClass.fromBuffer( getReadPtr()));
	skipReading(toolSubClass.fromBuffer( getReadPtr()));
	skipReading(toolTypeData.fromBuffer( getReadPtr()));
	skipReading(toolType.fromBuffer( getReadPtr()));
	skipReading(addedData.fromBuffer( getReadPtr()));
	return getReadCounter() ;
}

unsigned int Description::getSize() const
{
	int nbytes = toolClass.getSize();
	nbytes += toolSubClass.getSize();
	nbytes += toolTypeData.getSize();
	nbytes += toolType.getSize();
	nbytes += addedData.getSize();

	return nbytes ;
}

bool Description::checkSyntax() const
{
	return true;
}

/////////////////////// class IPMP_ParametricDescription //////////////////////////

IPMP_ParametricDescription::IPMP_ParametricDescription()
:	IPMP_Data_BaseClass( IPMP_ParametricDescription_tag )
{
	majorVersion = 0;
	minorVersion = 0;
}

IPMP_ParametricDescription::~IPMP_ParametricDescription() 
{
}


int IPMP_ParametricDescription::toBuffer(byte * buf)
{
	IPMP_Data_BaseClass::toBuffer( buf ) ;

	skipWriting(descriptionComment.toBuffer( getWritePtr())) ;
	put(majorVersion, 8);
	put(minorVersion, 8);
	unsigned int numOfDescriptions = descriptions.getObjectNum();
	put(numOfDescriptions, 32);
	for (unsigned int i=0; i < numOfDescriptions; i++)
	{
		skipWriting(descriptions.getObject(i).toBuffer( getWritePtr() ));
	}
	return getWriteCounter() ;
}


int IPMP_ParametricDescription::fromBuffer(const byte * buf)
{
	IPMP_Data_BaseClass::fromBuffer( buf ) ;

	skipReading( descriptionComment.fromBuffer( getReadPtr())) ;
	majorVersion = (byte) get(8);
	minorVersion = (byte) get(8);

	descriptions.clear();		
	unsigned int numOfDescriptions = get(32);
	for (unsigned int i=0; i < numOfDescriptions; i++)
	{
		Description desc;
		skipReading(desc.fromBuffer( getReadPtr() ));
		descriptions.addObject(desc);
	}

	return getReadCounter() ;
}

unsigned int IPMP_ParametricDescription::sizeOfInstance() const
{
	int nbytes = IPMP_Data_BaseClass::sizeOfInstance() ;

	nbytes += descriptionComment.getSize() + 2 + 4;		// majorVersion + MinorVersion + numOfDescriptions

	for (unsigned int i=0; i<descriptions.getObjectNum(); i++)
	{
		nbytes += descriptions.getObject(i).getSize();
	}
	
	return nbytes ;
}

