
//	This software module was originally developed by Massimo Balestri (TILAB)	
//	in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1) standard. 
//	This software module is an implementation of a part of one or more MPEG-4 
//	Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 
//	14496-1) standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	Telecom Italia Lab retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2002.

// IPMP_ParametricDescription

#ifndef	__IPMP_PARAMETRICDESCRIPTION_H__
#define	__IPMP_PARAMETRICDESCRIPTION_H__

#include "IPMP_Data_BaseClass.h"
#include "IPMP_ByteArray.h"
#include "Container.h"
#include "namespace.h"

BEGIN_MOSES_NAMESPACE

/**
 * @class Description
 * Class containing one parametrical description of an IPMP Tool.\
 * @see class IPMP_ParametricDescription
 * @author Massimo Balestri
 * @date 2003
 */
class Description: public BitField
{
public:
	IPMP_ByteArray toolClass;		///< class of the parametrically described tool, e.g. decryption
	IPMP_ByteArray toolSubClass;	///< sub-class of the parametrically described tool, e.g. AES
	IPMP_ByteArray toolTypeData;	///< specific type data to describe a particular type of tool, e.g. block length
	IPMP_ByteArray toolType;		///< value of the type data above, e.g. 128
	IPMP_ByteArray addedData;		///< any aditional data which may help to further describe the tool.

public:
	Description();					///< constructor
	virtual ~Description();			///< destructor		
	virtual int toBuffer(byte * buf);						// reimplemented
	virtual int fromBuffer(const byte * buf);				// reimplemented
	virtual bool checkSyntax() const;						// reimplemented
	virtual unsigned int getSize() const;					// reimplemented
};

/** 
 *  @class IPMP_ParametricDescription
 *  Using a parametric description, it is possible to describe what type of IPMP Tool
 *  is required in order to be able to play the content, instead of using fixed Tool IDs.
 *  For example, it is possible to specify that an AES tool, with block size of 128 bits
 *  is required to decrypt the video stream. An IPMP Terminal, upon receiving such description,
 *  can choose an optimised AES Tool from the list of available tools.
 *  @author Massimo Balestri
 *  @date 2003
 */
class IPMP_ParametricDescription: public IPMP_Data_BaseClass 
{
public:
	IPMP_ByteArray descriptionComment;				///< an optional comment
	byte majorVersion;								///< the major version of the required Tool
	byte minorVersion;								///< the minor version of the required Tool
	Container<Description> descriptions;			///< the list of parametric descriptions

public:
	IPMP_ParametricDescription();					///< constructor
	virtual ~IPMP_ParametricDescription();			///< destructor		
	virtual int toBuffer(byte * buf);						// reimplemented
	virtual int fromBuffer(const byte * buf);				// reimplemented
	virtual unsigned int sizeOfInstance() const;			// reimplemented
};

END_MOSES_NAMESPACE

#endif
