//	This software module was originally developed by Liao Xuan (PSL)
//	in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1) standard. 
//	This software module is an implementation of a part of one or more MPEG-4 
//	Systems (ISO/IEC 14496-1) deices as specified by the MPEG-4 Systems (ISO/IEC 
//	14496-1) standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	PSL retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2003.

// IPMP_RequestContent.cpp

#include "IPMP_RequestContent.h"
#include <memory.h>
#include <stdlib.h>
#include <assert.h>
#include <string.h>
#include <conio.h> 
#include "common.h"

USING_MOSES_NAMESPACE

IPMP_RequestContent::IPMP_RequestContent()
:IPMP_Data_BaseClass( IPMP_RequestContent_tag )
{
	//initialize message 
	ContentID = NULL;
	IPMP_DomainID = NULL;
}	

IPMP_RequestContent::~IPMP_RequestContent() 
{
	delete[] ContentID;
	delete[] IPMP_DomainID;

}

// copy constructor, added by MB
IPMP_RequestContent::IPMP_RequestContent(const IPMP_RequestContent &obj):
	IPMP_Data_BaseClass( obj ),
	opaqueData(obj.opaqueData)
{
	ContentID = NULL;
	IPMP_DomainID = NULL;
	setContentID(obj.getContentID());
	setDomainID(obj.getDomainID());
}
	
void IPMP_RequestContent::setContentID( const byte * contentID )
{
	if (ContentID != NULL)
		delete[] ContentID;                  // clear the old ContentID

	ContentID = new byte[16];
	memcpy(ContentID, contentID, 16) ;       // dump the new ContentID
}

byte * IPMP_RequestContent::getContentID() const
{
	return ContentID ;
}

void IPMP_RequestContent::setDomainID(const byte * domainID )
{
		if (IPMP_DomainID != NULL)           // clear the old IPMP_DomainID
		delete[] IPMP_DomainID;

	IPMP_DomainID = new byte[16];
	memcpy(IPMP_DomainID, domainID , 16) ;   // dump the new IPMP_DomainID
}

byte * IPMP_RequestContent::getDomainID() const
{
	return IPMP_DomainID ;
}

int IPMP_RequestContent::fromBuffer(const byte * buf)
{
	int nbits = IPMP_Data_BaseClass::fromBuffer(buf);  // tag and class size
	assert((nbits % 8) == 0);

    setContentID(getReadPtr()); // read 128 bits content id				
	nbits+=128;

	bool contentFlag = true;    // flag to indicate if ContentID is 0;
    byte * content = ContentID;
	for (int n=0; n<16; n++)
	{
        if (content[n]!=0)
			contentFlag = false;
    }
    
	int nCount = 0;
	if(contentFlag)
    {
		nCount = opaqueData.fromBuffer(buf+(nbits/8));  // Number of bits of opaqueData
	}

    setDomainID(getReadPtr()+16+nCount/8);				
	nbits+=128;	                                        // read 128 bits IPMP_DomainID

	return nbits+nCount ;
}


int IPMP_RequestContent::toBuffer(byte * buf)
{
	int nbits = IPMP_Data_BaseClass::toBuffer(buf);     // tag and class size
	assert((nbits % 8) == 0);
	
	memcpy(buf+(nbits/8), ContentID, 16);               // write ContentID 128 bits
	nbits+=128 ;			
	
	bool contentFlag = true;           // flag to indicate if ContentID is 0;
    byte * content = ContentID;
	for (int n=0; n<16; n++)
	{
        if (content[n]!=0)
			contentFlag = false;
    }

	int nCount = 0;
	// if ContentID is 0
	if(contentFlag)
    {
		nCount = opaqueData.toBuffer(buf+(nbits/8));    // write opaqueData
    }

	memcpy((buf+(nbits+nCount)/8), IPMP_DomainID, 16);  // write the domain ID 128 bits
	nbits+=128 ;			

	return nbits+nCount;
}

unsigned int IPMP_RequestContent::sizeOfInstance() const
{
	int nSize = IPMP_Data_BaseClass::sizeOfInstance() + 32 ; // 32bytes = 16bytes ContentID + 16bytes DomainID

    bool contentFlag = true;            // flag to indicate if ContentID is 0;
    byte * content = ContentID;
	for (int n=0; n<16; n++)
	{
        if (content[n]!=0)
			contentFlag = false;
    }
    
	// if Content ID is 0
	if(contentFlag)
    {
		nSize += opaqueData.getSize() ; // add the opaque data size in Bytes
    }
	
	return nSize ;                      // return the size in bytes
}

void IPMP_RequestContent::setOpaqueData(const byte *pOpaqueData, unsigned int opaqueDataSize)
{
	opaqueData.setData(pOpaqueData, opaqueDataSize);
}

const byte * IPMP_RequestContent::getOpaqueData() const
{
	byte *p = opaqueData.getData() ;
	return p ;
}
