//	This software module was originally developed by Liao Xuan (PSL)
//	in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1) standard. 
//	This software module is an implementation of a part of one or more MPEG-4 
//	Systems (ISO/IEC 14496-1) deices as specified by the MPEG-4 Systems (ISO/IEC 
//	14496-1) standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	PSL retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2003.

// IPMP_RequestContent.h

#ifndef	IPMP_REQUESTCONTENT_H
#define	IPMP_REQUESTCONTENT_H

#include "IPMP_Data_BaseClass.h"
#include "namespace.h"
#include "IPMP_ByteArray.h"


BEGIN_MOSES_NAMESPACE

/** 
 *  @class IPMP_RequestContent
 *  This message is sent from a Device to another Device to find a particular content with a specified ContentID.
 *  The recipient Device replies with a IPMP_ResponseToContentRequest message.
 *
 *  @author Liao Xuan
 *  @date 2003
 */
class IPMP_RequestContent: public IPMP_Data_BaseClass 
{
private:
	byte * ContentID;					                     // 128 bits content id
	
	//if Content ID is 0
	IPMP_ByteArray opaqueData;     
	
	byte * IPMP_DomainID;					                 // 128 bits domain id

public:
	
	IPMP_RequestContent();									///< constructor.
	virtual ~IPMP_RequestContent();							///< destructor.	
	IPMP_RequestContent(const IPMP_RequestContent & obj);	///< copy constructor
	virtual int toBuffer(byte * buf);						// reimplemented
	virtual int fromBuffer(const byte * buf);				// reimplemented
	virtual unsigned int sizeOfInstance() const;			// reimplemented

	
	/**
	 * Set the Content ID. 
	 * @param contentID the Content ID (128bits) of the requested content.
	 */
	void setContentID(const byte *  contentID );

	/**
	 * set the Domain ID. 
	 * @param domainID the ID (128bits) of the domain which the requesting Device(request generator) belongs to.
	 */
	void setDomainID(const byte *  domainID );


	/**
	 * Get the content ID. 
	 * @return ContentID the Content ID of the requested content
	 */
	byte * getContentID() const;

	/**
	 * Get the Domain ID. 
	 * @return DomainID the Domain ID of the domain which the requesting Device (request generator) belongs to.
	 */ 
	byte * getDomainID() const;

	/**
	 * method for setting the opaque data.
	 * @param pOpaqueData opaque data value. 
	 * @param opaqueDataSize opaque data size. 
	 */
	void setOpaqueData(const byte *pOpaqueData, unsigned int opaqueDataSize);

	/**
	 * method for getting the opaque data.
	 * @return opaque data. 
	 */
	const byte * getOpaqueData() const;

};

END_MOSES_NAMESPACE

#endif
