//	This software module was originally developed by Liao Xuan (PSL)
//	in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1) standard. 
//	This software module is an implementation of a part of one or more MPEG-4 
//	Systems (ISO/IEC 14496-1) deices as specified by the MPEG-4 Systems (ISO/IEC 
//	14496-1) standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	PSL retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2003.

// IPMP_RequestTool.cpp

#include "IPMP_RequestTool.h"
#include <memory.h>
#include <stdlib.h>
#include <assert.h>
#include <string.h>
#include <conio.h> 
#include "common.h"

USING_MOSES_NAMESPACE

IPMP_RequestTool::IPMP_RequestTool()
:IPMP_Data_BaseClass( IPMP_RequestTool_tag )
{
	// initialize the object
    ToolID = NULL;
	IPMP_DomainID = NULL;
}	

IPMP_RequestTool::IPMP_RequestTool(const IPMP_RequestTool & obj)
:IPMP_Data_BaseClass( obj ),
 opaqueData(obj.opaqueData)
{
    ToolID = NULL;
	IPMP_DomainID = NULL;
	setToolID(obj.getToolID());
	setDomainID(obj.getDomainID());
}

IPMP_RequestTool::~IPMP_RequestTool() 
{
	delete[] ToolID;
	delete[] IPMP_DomainID;
}
	
void IPMP_RequestTool::setToolID( const byte * toolID )
{
	if (ToolID != NULL)                          // clear the old ToolID
		delete[] ToolID;

	ToolID = new byte[16];
	memcpy(ToolID, toolID, 16) ;                 // dump the new ToolID
}

byte *IPMP_RequestTool::getToolID() const
{
	return ToolID ;
}

void IPMP_RequestTool::setDomainID( const byte * domainID )
{
	if (IPMP_DomainID != NULL)                   // clear the old IPMP_DomainID
		delete[] IPMP_DomainID;

	IPMP_DomainID = new byte[16];
	memcpy(IPMP_DomainID, domainID , 16) ;       // dump the new IPMP_DomainID
}

byte * IPMP_RequestTool::getDomainID() const
{
	return IPMP_DomainID ;
}

int IPMP_RequestTool::fromBuffer(const byte * buf)
{
	int nbits = IPMP_Data_BaseClass::fromBuffer(buf);  // read the base class
	assert((nbits % 8) == 0);

    setToolID(getReadPtr());                           // read the ToolID				
	nbits+=128;

	setDomainID(getReadPtr()+16);		               // read the DomainID		
	nbits+=128;
	
	int nCount = opaqueData.fromBuffer(buf+(nbits/8)); // read the opaque data and get opaque data size in Bit

	return nbits+nCount ;
}


int IPMP_RequestTool::toBuffer(byte * buf)
{
	int nbits = IPMP_Data_BaseClass::toBuffer(buf);    // write the base class
	assert((nbits % 8) == 0);
	
	memcpy(buf+(nbits/8), ToolID, 16);                 // write ContentID, 128 bits
	nbits+=128 ;

    memcpy(buf+(nbits/8), IPMP_DomainID, 16);          // write the DomainID, 128 bits
	nbits+=128 ;

	int nCount = opaqueData.toBuffer(buf+(nbits/8));   // write the opaque data and 
	                                                   // get the opaque data size in Bit

	return nbits+nCount ;
}

unsigned int IPMP_RequestTool::sizeOfInstance() const
{
	int nSize = IPMP_Data_BaseClass::sizeOfInstance() + 32 ; // size of base class + 16bytes ToolID + 16bytes DomainID

	nSize += opaqueData.getSize() ;                          // add opaque data size
	
	return nSize ;
}

void IPMP_RequestTool::setOpaqueData(const byte *pOpaqueData, unsigned int opaqueDataSize)
{
	opaqueData.setData(pOpaqueData, opaqueDataSize);
}

const byte* IPMP_RequestTool::getOpaqueData() const
{
	byte *p = opaqueData.getData() ;
	return p ;  // return the pointer to opaquedata
}
