//	This software module was originally developed by Liao Xuan (PSL)
//	in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1) standard. 
//	This software module is an implementation of a part of one or more MPEG-4 
//	Systems (ISO/IEC 14496-1) devices as specified by the MPEG-4 Systems (ISO/IEC 
//	14496-1) standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	PSL retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2003.


// IPMP_ResponseToContentRequest.cpp

#include "IPMP_ResponseToContentRequest.h"
#include <stdlib.h>
#include <memory.h>
#include <assert.h>

USING_MOSES_NAMESPACE

IPMP_ResponseToContentRequest::IPMP_ResponseToContentRequest()
	:	IPMP_Data_BaseClass(IPMP_ResponseToContentRequest_tag)
{

}	

IPMP_ResponseToContentRequest::~IPMP_ResponseToContentRequest() 
{

}

unsigned int IPMP_ResponseToContentRequest::sizeOfInstance() const
{
	unsigned int nSize = IPMP_Data_BaseClass::sizeOfInstance();

	nSize += 1; // 2bits response + 6bits reserve = 1 byte
	
	return nSize;  // return the size in Byte
}

int IPMP_ResponseToContentRequest::fromBuffer(const byte * buf)
{
	int nbits = IPMP_Data_BaseClass::fromBuffer(buf);   // read tag and class size
	response = get( buf, nbits, 2 ) ;                   // read response, 2 bits	
	nbits+=2 ;

	get( buf, nbits, 6 ) ;								// read reserved data, 6 bits
	nbits+=6 ;

	return nbits;                    
}

int IPMP_ResponseToContentRequest::toBuffer(byte * buf)
{
	int nbits = IPMP_Data_BaseClass::toBuffer(buf);     // write tag and class size
	
	put( response, buf, nbits, 2 );					    // write response, 2 bits
	nbits+=2 ;

	byte reserve = 0 ;
	put( reserve, buf, nbits, 6 );						// write reserved data, 6 bits
	nbits+=6 ;

	return nbits;
	
}

void IPMP_ResponseToContentRequest::setResponse(unsigned int resp)
{
	response = resp;
}

unsigned int IPMP_ResponseToContentRequest::getResponse() const
{
	return response;                                    // 2 bits response
}



