//	This software module was originally developed by Liao Xuan (PSL)
//	in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1) standard. 
//	This software module is an implementation of a part of one or more MPEG-4 
//	Systems (ISO/IEC 14496-1) devices as specified by the MPEG-4 Systems (ISO/IEC 
//	14496-1) standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	PSL retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2003.


// IPMP_ResponseToToolRequest.cpp

#include "IPMP_ResponseToToolRequest.h"
#include <stdlib.h>
#include <memory.h>
#include <assert.h>
#include "common.h"

USING_MOSES_NAMESPACE

IPMP_ResponseToToolRequest::IPMP_ResponseToToolRequest()
	:	IPMP_Data_BaseClass(IPMP_ResponseToToolRequest_tag)
{
	// intialize the object
	ToolID = NULL;
	response = 0;
}	

IPMP_ResponseToToolRequest::IPMP_ResponseToToolRequest(const IPMP_ResponseToToolRequest &obj):
	IPMP_Data_BaseClass(obj),
	response(obj.response),
	Payload(obj.Payload),
	ToolDescription(obj.ToolDescription)
{
	ToolID = NULL;
	setToolID(obj.getToolID());
}

IPMP_ResponseToToolRequest::~IPMP_ResponseToToolRequest() 
{ 
	delete[] ToolID;
} 

// in Bytes !!
unsigned int IPMP_ResponseToToolRequest::sizeOfInstance() const
{
	unsigned int nSize = IPMP_Data_BaseClass::sizeOfInstance();
	nSize += 17;    // (128bits ToolID + 2bits response + 6bits reserve)/8 =17bytes
	
    //f the reponse is "approved" (0b11/0x03)
	if( response == 0x03 )
		nSize += (6 + getToolDescriptionSize() + getPayloadSize() ); // 4bytes PayloadSize + 2bytes ToolDescriptionSize
	                                                       // + size of Payload + size of ToolDescription
	
	return nSize;
}

void IPMP_ResponseToToolRequest::setToolID(const byte * toolID )
{
	if (ToolID != NULL)                                    // clear the old ToolID
		delete[] ToolID;

	ToolID = new byte[16];                                 // dump the new ToolID, 128 bits(16bytes)
	memcpy(ToolID, toolID, 16) ;
}

byte * IPMP_ResponseToToolRequest::getToolID() const
{
	return ToolID ;
}

word32 IPMP_ResponseToToolRequest::getPayloadSize() const
{
	return Payload.getDataSize();
}


byte* IPMP_ResponseToToolRequest::getPayload() const
{
	byte *p = Payload.getData() ;
	return p ;                                      // return the pointer to the Payload
}

void IPMP_ResponseToToolRequest::setPayload(const byte* buf, word32 size)
{
	Payload.setData( buf, size ) ;                  // set the payload data
}

word32 IPMP_ResponseToToolRequest::getToolDescriptionSize() const
{
	return ToolDescription.getDataSize();                     
}

byte* IPMP_ResponseToToolRequest::getToolDescription() const
{
	byte *p = ToolDescription.getData() ; 
	return p ;                                     // return the pointer to the ToolDescription
}

void IPMP_ResponseToToolRequest::setToolDescription(const byte* buf, word32 size)
{
	ToolDescription.setData( buf, size ) ;                // set the ToolDescription data
}

 
int IPMP_ResponseToToolRequest::fromBuffer(const byte * buf)
{
	int nbits = IPMP_Data_BaseClass::fromBuffer(buf);     // read the base class
	assert((nbits % 8) == 0);
                  
	setToolID(getReadPtr());				              // read and set the ToolID, 128 bits
	nbits+=128 ;

	response = get( buf, nbits, 2 ) ;                     // read the response, 2 bits	
	nbits+=2 ;

	get( buf, nbits, 6 ) ;								  // read the reserved data, 6 bits
	nbits+=6 ;

	// if the response is "approved"- 0x03 (0b11)
	if (response == 0x03)
	{
		word32 PayloadSize = get(buf, nbits, 32);	              // read the PayloadSize,32 bits      
	    nbits+=32;
        if (PayloadSize)
		{
			setPayload(buf + (nbits/8), PayloadSize);	  // read and set the payload
			nbits +=  (PayloadSize * 8) ;
		}
        
		
        word32 ToolDescriptionSize = get(buf, nbits, 16);	      // read the ToolDescriptionSize, 16 bits      
	    nbits+=16;
		if (ToolDescriptionSize)
		{
			setToolDescription(buf + (nbits/8), ToolDescriptionSize); // read and set the new ToolDescription 
			nbits +=  (ToolDescriptionSize * 8) ;
		}
	}

	return nbits;
}


int IPMP_ResponseToToolRequest::toBuffer(byte * buf)
{

	int nbits = IPMP_Data_BaseClass::toBuffer(buf);       // write the base class 
	
	memcpy(buf+(nbits/8), ToolID, 16);                    // write the ToolID, 128 bits			
	nbits+=128 ;

	put( response, buf, nbits, 2 );					      // write the response, 2 bits
	nbits+=2 ;

	byte reserve = 0 ;
	put( reserve, buf, nbits, 6 );						   // set and write reserved data 0, 6 bits 
	nbits+=6 ;
    
	// if tool request is "approved", response = 0x03 (0b11)
	if (response == 0x03)
    {
		word32 PayloadSize = getPayloadSize();

		put(PayloadSize, buf, nbits, 32);                  // write the PayloadSize, 32 bits
		nbits+=32;

		memcpy(buf + (nbits/8), getPayload(), PayloadSize);// write the Payload
		nbits += (PayloadSize * 8);
		
		word32 ToolDescriptionSize = getToolDescriptionSize();

        put(ToolDescriptionSize, buf, nbits, 16);          // write the ToolDescriptionSize, 16 bits
		nbits+=16;

		memcpy(buf + (nbits/8), getToolDescription(), ToolDescriptionSize); // write the ToolDescription
		nbits += (ToolDescriptionSize * 8);
    }
	return nbits; 
}

void IPMP_ResponseToToolRequest::setResponse(word32 resp)
{
	response = resp;
}

word32 IPMP_ResponseToToolRequest::getResponse() const
{
	return response;    // 2 bits
}



