
//	This software module was originally developed by Liao Xuan (PSL)
//	in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1) standard. 
//	This software module is an implementation of a part of one or more MPEG-4 
//	Systems (ISO/IEC 14496-1) deices as specified by the MPEG-4 Systems (ISO/IEC 
//	14496-1) standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	PSL retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2003.


// IPMP_ResponseToToolRequest.h

#ifndef	IPMP_RESPONSETOTOOLREQUEST_H
#define	IPMP_RESPONSETOTOOLREQUEST_H

#include "IPMP_Data_BaseClass.h"
#include "IPMP_ByteArray.h"
#include "namespace.h"

BEGIN_MOSES_NAMESPACE

/** @class IPMP_ResponseToToolRequest
 *  Response from the recipient device to the sender device regarding to the Tool request
 *
 *  @author Liao Xuan
 *  @date 2003
 */
class IPMP_ResponseToToolRequest: public IPMP_Data_BaseClass 
{
private:
    byte * ToolID;                              // 128 bits
	word32 response ;					        // 2 bits
	
	//if(response==0b11)
    IPMP_ByteArray Payload;					    // 8 bits * arraysize
    IPMP_ByteArray ToolDescription;			    // 8 bits * arraysize

	

public:
	IPMP_ResponseToToolRequest();				///< constructor			
	IPMP_ResponseToToolRequest(const IPMP_ResponseToToolRequest &obj);				///< copy constructor			
	virtual ~IPMP_ResponseToToolRequest();		///< destructor
	virtual int toBuffer(byte * buf);		    // reimplemented
	virtual int fromBuffer(const byte * buf);	// reimplemented
	virtual unsigned int sizeOfInstance() const;// reimplemented


	/**
	 * Set the Payload. 
	 * @param buf the pointer to payload buffer
	 * @param size payload size in bytes
	 */
    void setPayload(const byte* buf, word32 size);


	/** 
	 * Set the Tool Description. 
	 * @param buf the pointer to ToolDescription buffer
	 * @param size Tool Description size in bytes
	 */
	void setToolDescription(const byte* buf, word32 size);

	/**
	 * Set the Tool ID. 
	 * @param toolID pointer the ContentID's pointer (128bits) of the requested content.
	 */
	void setToolID(const byte * toolID );

	/**
	 * Get the tool ID. 
	 * @return ToolID pointer the ToolID's pointer of the requested tool
	 */
	byte * getToolID() const;

	/**
	 * Set the response (2 bits)
	 *     0b00: Reserved
	 *     0b01: No Such content
	 *     0b10: Prohibit Copy/Move
	 *     0b11: Approved
	 */
	void setResponse( word32 );
	
	/**
	 * Get the value of the response bits.
	 * @return the value of the response bits
	 */
    word32 getResponse() const;

	/**
	 * Get the size in bytes of the tool payload. 
	 * @return PayloadSize the size of this payload
	 */
	word32 getPayloadSize() const;

	/**
	 * Get the tool payload. 
	 * @return the pointer to the Payload
	 */
	byte * getPayload() const;

	/**
	 * Get the size in bytes of the tool description. 
	 * @return ToolDescriptionSize the size of this tool description
	 */
	word32 getToolDescriptionSize() const;

	/**
	 * Get the tool description. 
	 * @return the pointer to the tool description
	 */
	byte * getToolDescription() const;

};

END_MOSES_NAMESPACE

#endif

