
//	This software module was originally developed by Claudio Alberti (EPFL)	
//	in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1) standard. 
//	This software module is an implementation of a part of one or more MPEG-4 
//	Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 
//	14496-1) standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	EPFL retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2002.

// IPMP_RightsData.h

#ifndef	IPMP_RIGHTSDATA_H
#define	IPMP_RIGHTSDATA_H

#include "IPMP_Data_BaseClass.h"
#include "IPMP_ByteArray.h"
#include "namespace.h"

BEGIN_MOSES_NAMESPACE

/** 
 *  @class IPMP_RightsData
 *  The IPMP_RightsData contains rights information and can be carried in either 
 *  the IPMP_Descriptor or IPMP_Message's forming an IPMP Stream.
 *  This facilitates the carriage of rights information in content stream.
 *
 *  @author Claudio Alberti
 *  @date 2002
 */
class IPMP_RightsData: public IPMP_Data_BaseClass 
{
private:

	IPMP_ByteArray rightsInfo;


public:
	/**
	 * constructor.
	 */
	IPMP_RightsData();	
	
	/**
	 * destructor.
	 */
	virtual ~IPMP_RightsData();		
	virtual int toBuffer(byte * buf);		
	virtual int fromBuffer(const byte * buf);
	virtual unsigned int sizeOfInstance() const;
	
	/**
	 * Set the rights information.
	 * This contains the details of usage rights information.
	 * @param rights rights info. 
	 * @param rightsDim rights info size. 
	 */
	void setRightsInfo(byte *rights, unsigned int rightsDim);

	/**
	 * Get the rights information.
	 * This contains the details of usage rights information.
	 * @return rights info value. 
	 */
	const IPMP_ByteArray& getRightsInfo() const;
};

END_MOSES_NAMESPACE

#endif
