
//	This software module was originally developed by Claudio Alberti (EPFL)	
//	in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1) standard. 
//	This software module is an implementation of a part of one or more MPEG-4 
//	Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 
//	14496-1) standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	EPFL retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2002.

// IPMP_SendAudioWatermark.h

#ifndef	IPMP_SENDAUDIOWATERMARK_H
#define	IPMP_SENDAUDIOWATERMARK_H

#include "IPMP_Data_BaseClass.h"
#include "IPMP_ByteArray.h"
#include "namespace.h"

BEGIN_MOSES_NAMESPACE

/** @class IPMP_SendAudioWatermark
 *  An audio watermarking tool that has been required 
 *  to perform payload extraction will construct this IPMP data.
 *  
 *  @author CRL
 *  @date 2002
 */
class IPMP_SendAudioWatermark: public IPMP_Data_BaseClass 
{
private:
	bool hasOpaqueData;

	IPMP_ByteArray payload;
	IPMP_ByteArray opaqueData;

	word32 wm_status ;
	word32 compression_status ;

public:
	/**
	 *	Table describing the result of the watermark check.
	 */
	enum  WmStatusTable {
		WM_PAYLOAD		= 0,	///< watermarking was present in the audio stream, payload is carried in the message.
		WM_NOPAYLOAD	= 1,	///< watermarking was present in the audio stream, no payload is carried in the message.
		NO_WM			= 2,	///< watermarking was not present in the audio stream.
		WM_UNKNOWN		= 3		///< the Watermarking Tool was unable to detect whether watermarking was present in the audio stream or not.
	};

	/**
	 * Table describing the result of the compression check.
	 */
	enum  CompressionStatusTable {
		COMPRESSION		= 0,	///< the audio stream was compressed
		NO_COMPRESSION  = 1,	///< the audio stream was not compressed
		COMP_UNKNOWN	= 2		///< the Watermarking Tool was unable to detect if the audio stream was compressed
	};

	IPMP_SendAudioWatermark();					///< constructor
	virtual ~IPMP_SendAudioWatermark();			///< destructor

	virtual int toBuffer(byte * buf);			// reimplemented methods
	virtual int fromBuffer(const byte * buf);
	virtual unsigned int sizeOfInstance() const;

	/**
	 * Set the watermarking payload extracted from the the audio content.
	 * @param buf the buffer containing the watermarking payload.
	 * @param len the length in bytes of the payload.
	 */
	void setPayload( const byte * buf, word32 len ) ;

	/**
	 * Set the opaque data. This is normally done by the watermarking tool.
	 * @param buf the buffer containing opaque data.
	 * @param len the length in bytes of the opaque data.
	 */
	void setOpaqueData ( const byte * buf, word32 len ) ;

	/**
	 * Set the result of the watermark check.
	 * @param status the result of the watermark check.
	 */
	void setWMStatus( enum WmStatusTable status ) ;

	/**
	 * Set the result of the compression check.
	 * @param status the result of the compression check.
	 */
	void setCompressionStatus( enum CompressionStatusTable status ) ;
	
	/**
	 * Get the watermarking payload extracted from the audio content.
	 * @return a pointer to the payload.
	 */
	const byte * getPayload() const;

	/**
	 * Get the size of the watermarking payload.
	 * @return the size in bytes.
	 */
	word32 getPayloadSize() const;

	/**
	 * Get the opaque data.
	 * @return a pointer to the opaque data.
	 */
	const byte * getOpaqueData() const;

	/**
	 * Get the size of the opaque data.
	 * @return the size in bytes.
	 */
	word32 getOpaqueDataSize() const;

	/**
	 * Get the result of the watermark check.
	 * @return the result of the watermark check.
	 */
	enum WmStatusTable getWMStatus() const;

	/**
	 * Get the result of the compression check.
	 * @return the result of the compression check.
	 */
	enum CompressionStatusTable getCompressionStatus() const;
};

END_MOSES_NAMESPACE

#endif

