
//	This software module was originally developed by Massimo Balestri (TILAB)
//	in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1) standard. 
//	This software module is an implementation of a part of one or more MPEG-4 
//	Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 
//	14496-1) standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	Telecom Italia Lab retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2002.


// IPMP_Tool.cpp

#include "IPMP_Tool.h"
#include <stdlib.h>
#include <string.h>
#include <assert.h>

USING_MOSES_NAMESPACE

IPMP_Tool::IPMP_Tool():
	BaseDescriptor(IPMP_ToolTag),			// set the correct tag
	toolParamDesc()							// init the byte array
{
	init() ;		// initialize all member vars
}	

IPMP_Tool::~IPMP_Tool() 
{	
}

void IPMP_Tool::init() 
{
	m_isAltGroup=0;
	m_isParametric=0;
}

int IPMP_Tool::fromBuffer(const byte * in)
{
	BaseDescriptor::fromBuffer(in);		// read the base descriptor
	long eom = getReadCounter() + 8 * (getSizeOfInstance() - BaseDescriptor::sizeOfInstance());			// mark end of message

	getObject(toolID);					// read the IPMP tool ID
	
	m_isAltGroup = get(1);				// flags
	m_isParametric = get(1);
	skipReading(6);						// reserved

	specificToolID.clear();				// cleanup old data - if any.
	
	if(m_isAltGroup)
	{
		unsigned int numAlternates = get(8);

		for (unsigned int i=0; i<numAlternates; i++)
		{
			IPMP_ToolId id;
			getObject(id);
			specificToolID.addObject(id);
		}
	}

	if(m_isParametric)
		getObject(toolParamDesc);

	toolURL.clear();				// cleanup old data - if any.

	while (getReadCounter() < eom)
	{
		IPMP_ByteArray myUrl;
		getObject(myUrl);
		toolURL.addObject(myUrl);
	}

	return getReadCounter();
}

int IPMP_Tool::toBuffer(byte * out)
{
	BaseDescriptor::toBuffer(out);		// read the base descriptor and set "buf" as read buffer for this class.

	putObject(toolID);
	put(m_isAltGroup, 1); 
	put(m_isParametric, 1); 
	put(RESERVED, 6);

	if(m_isAltGroup)
	{
		unsigned int numAlternates = specificToolID.getObjectNum();

		put(numAlternates, 8); 
		for (unsigned int i=0; i<numAlternates; i++){
			putObject(specificToolID.getObject(i));
		}
	}

	if(m_isParametric)
	{
		putObject(toolParamDesc);
	}

	for (unsigned int i=0;i<toolURL.getObjectNum();i++){
		putObject(toolURL.getObject(i));
	}

	return getWriteCounter();
}

void IPMP_Tool::addSpecificToolID(const IPMP_ToolId & toolId)
{
	m_isAltGroup = 1;
	specificToolID.addObject(toolId);
}


void IPMP_Tool::setToolParamDesc( const IPMP_ParametricDescription & descr )
{
	m_isParametric = true;
	toolParamDesc = descr;
}

void IPMP_Tool::addToolURL( byte *desc, int length )
{
	IPMP_ByteArray ba;
	ba.setData(desc, length);

	toolURL.addObject(ba);
}

byte * IPMP_Tool::getToolURL(unsigned int index) const
{
	if (index <toolURL.getObjectNum())
		return toolURL.getObject(index).getData();
	else 
		return NULL;
}

unsigned int IPMP_Tool::getToolURLsize(unsigned int index) const
{
	if (index <toolURL.getObjectNum())
		return toolURL.getObject(index).getDataSize();
	else 
		return 0;
}

const IPMP_ParametricDescription & IPMP_Tool::getToolParamDesc() const
{
	return toolParamDesc;
}

unsigned int IPMP_Tool::getToolParamDescSize() const
{
	return toolParamDesc.sizeOfInstance();
}


IPMP_ToolId & IPMP_Tool::getSpecificToolID(unsigned int index) const
{
	return specificToolID.getObject(index);
}

unsigned int IPMP_Tool::sizeOfInstance() const
{
	int size = BaseDescriptor::sizeOfInstance() ;
	size+=16 ;
	size+=1 ;

	if(m_isAltGroup)
	{
		size+=1 ;
		size+=(16 * specificToolID.getObjectNum());
	}
	if(m_isParametric)
	{
		size+=toolParamDesc.getSize();
	}
	for (unsigned int i=0;i < toolURL.getObjectNum();i++){
		size += toolURL.getObject(i).getSize();
	}

	return size;
}

bool IPMP_Tool::isAltGroup() const
{
	return (m_isAltGroup != 0);
}

bool IPMP_Tool::isParametric() const
{
	return (m_isParametric != 0);
}

unsigned int IPMP_Tool::getNumAlternates() const 
{
	return specificToolID.getObjectNum();
}

unsigned int IPMP_Tool::getNumURLs() const
{
	return toolURL.getObjectNum();
}

