
//	This software module was originally developed by Massimo Balestri 
//	(Telecom Italia Lab S.p.A.), in the course of development 
//	of the MPEG-4 Systems (ISO/IEC 14496-1) standard. This software module 
//	is an implementation of a part of one or more MPEG-4 Systems (ISO/IEC 
//	14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1) 
//	standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	Telecom Italia Lab S.p.A retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2000. 


#include <stdio.h>
#include <stdlib.h>
#include <memory.h>
#include <assert.h>
#include "IPMP_ToolAPI_Config.h"

USING_MOSES_NAMESPACE


////////////////////////////////////////////////////////////
/////    class IPMP_ToolAPI_Config
////////////////////////////////////////////////////////////
IPMP_ToolAPI_Config::IPMP_ToolAPI_Config():
	IPMP_Data_BaseClass(IPMP_ToolAPI_Config_tag)
{
	has_Instantiation_API_ID = false;
	has_Messaging_API_ID = false;
}

IPMP_ToolAPI_Config::~IPMP_ToolAPI_Config()
{
}

void IPMP_ToolAPI_Config::clear()
{
	has_Instantiation_API_ID = false;
	has_Messaging_API_ID = false;
	opaqueData.clear();
}


void IPMP_ToolAPI_Config::setOpaqueData(const IPMP_ByteArray & myOpaqueData)
{
	opaqueData = myOpaqueData;
}




int IPMP_ToolAPI_Config::fromBuffer(const byte *buf)
{
	IPMP_Data_BaseClass::fromBuffer(buf);	
	has_Instantiation_API_ID = (get(1) == 1);	// flags
	has_Messaging_API_ID = (get(1) == 1);
	skipReading(6);								// reserved
	
	if (has_Instantiation_API_ID)
		instantiation_API_ID = get(32);			// instantiation API ID

	if (has_Messaging_API_ID)
		messaging_API_ID = get(32);				// messaging API ID

	getObject(opaqueData);						// opaque data

	return getReadCounter();			// return the num of bits
}


int IPMP_ToolAPI_Config::toBuffer(byte *buf)
{
	IPMP_Data_BaseClass::toBuffer(buf);	
	put(has_Instantiation_API_ID, 1);	// flags
	put(has_Messaging_API_ID, 1);
	put(RESERVED, 6);
	
	if (has_Instantiation_API_ID)
		put(instantiation_API_ID, 32);	// instantiation API ID

	if (has_Messaging_API_ID)
		put(messaging_API_ID, 32);		// messaging API ID

	putObject(opaqueData);				// opaque data

	return getWriteCounter();			// return the num of bits
}


unsigned int IPMP_ToolAPI_Config::sizeOfInstance() const
{
	unsigned int nbytes = IPMP_Data_BaseClass::sizeOfInstance();	// base class size
	nbytes += 1;							// flags
	if (has_Instantiation_API_ID)
		nbytes += 4;						// instantiation API ID

	if (has_Messaging_API_ID)
		nbytes += 4;						// messaging API ID

	nbytes += opaqueData.getSize();
	
	return nbytes;
}



void IPMP_ToolAPI_Config::setInstantiationID(unsigned int id)
{
	has_Instantiation_API_ID = true;
	instantiation_API_ID = id;
}

void IPMP_ToolAPI_Config::setMessagingID(unsigned int id)
{
	has_Messaging_API_ID = true;
	messaging_API_ID = id;
}

bool IPMP_ToolAPI_Config::hasMessagingID() const
{
	return has_Messaging_API_ID;
}

unsigned int IPMP_ToolAPI_Config::getMessagingID() const
{
	if (hasMessagingID())
		return messaging_API_ID;
	else
		return 0;
}

bool IPMP_ToolAPI_Config::hasInstantiationID() const
{
	return has_Instantiation_API_ID;
}


unsigned int IPMP_ToolAPI_Config::getInstantiationID() const
{
	if (hasInstantiationID())
		return instantiation_API_ID;
	else 
		return 0;
}
