
//	This software module was originally developed by Massimo Balestri 
//	(Telecom Italia S.p.A.) in the course of development 
//	of the MPEG-4 Systems (ISO/IEC 14496-1) standard. This software module 
//	is an implementation of a part of one or more MPEG-4 Systems (ISO/IEC 
//	14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1) 
//	standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	Telecom Italia S.p.A retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2000. 



#ifndef __IPMP_TOOLAPI_CONFIG__
#define __IPMP_TOOLAPI_CONFIG__
#include "namespace.h"

#include "IPMP_Data_BaseClass.h"
#include "IPMP_ByteArray.h"

BEGIN_MOSES_NAMESPACE

/**
 * @class IPMP_ToolAPI_Config
 * This class stores information about the IPMP Tools instantiation and the messaging API.
 * @author Massimo Balestri
 * @date Oct. 2003
 */
class IPMP_ToolAPI_Config : public IPMP_Data_BaseClass
{
private:
	bool has_Instantiation_API_ID;
	bool has_Messaging_API_ID;
	word32 instantiation_API_ID;
	word32 messaging_API_ID;
	IPMP_ByteArray opaqueData;

public:
	IPMP_ToolAPI_Config();											///< constructor
	~IPMP_ToolAPI_Config();											///< destructor

	virtual int fromBuffer(const byte *buf);				// reimplemented
	virtual int toBuffer(byte *buf);						// reimplemented
	virtual unsigned int sizeOfInstance() const;					// reimplemented

	/**
	 * Removes any data, and free any allocated memory.
	 */
	void clear();

	/**
	 * Sets the opaque data
	 * @param myOpaqueData the byte array containing the opaque data.
	 */
	void setOpaqueData(const IPMP_ByteArray & myOpaqueData);

	/**
	 * Set the messaging API ID
	 * @param id the ID assigned by a registration authority as messaging API identifier.
	 */
	void setMessagingID(unsigned int id);
	
	/**
	 * Check if this message contains a messaging API identifier.
	 * @return true if the message contains a messaging API ID, assigned by a registration authority.
	 */
	bool hasMessagingID() const;
	
	/**
	 * Get the Messaging API identifier.
	 * @return the messaging API ID, or zero if this message does not contain a messanging API ID.
	 */
	unsigned int getMessagingID() const;

	/**
	 * Set the Instantiation API identifier.
	 * @param id the API ID, assigned by a registration authority.
	 */
	void setInstantiationID(unsigned int id);

	/**
	 * Check is this message contains an instantiation API identifier.
	 * @return true if the message contains an instantiation API ID, assigned by a registration authority.
	 */
	bool hasInstantiationID() const;
	
	/**
	 * Get the Instantiation API identifier.
	 * @return the Instantiation API ID, or zero if this message does not contain an Instantiation API ID.
	 */
	unsigned int getInstantiationID() const;
};

END_MOSES_NAMESPACE

#endif
