
//	This software module was originally developed by James King (Central
//	Research Laboratories Ltd) and edited by Filippo Chiariglione (Central 
//	Research Laboratories Ltd) and Massimo Balestri (Telecom Italia S.p.A.)
//  in the course of development of the MPEG-4 Systems 
//	(ISO/IEC 14496-1) standard. This software module is an implementation 
//	of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified 
//	by the MPEG-4 Systems (ISO/IEC 14496-1) standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	Central Research Laboratories Ltd retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2000. 





// IPMP_ToolES_AU.cpp: implementation of the IPMP_ToolES_AU class.
//
//////////////////////////////////////////////////////////////////////

#include "IPMP_ToolES_AU.h"
#include <memory.h>
#include <assert.h>

USING_MOSES_NAMESPACE


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

IPMP_ToolES_AU::IPMP_ToolES_AU()
{
	isSigned = false;
	sizeofTool = 0;
	Tool_Body = NULL;
	for(int f=0;f<16;f++)
		Verifying_Tool_Id[f] = 0;
}

IPMP_ToolES_AU::IPMP_ToolES_AU(const IPMP_ToolES_AU & obj):
	certificateList(obj.certificateList),
	IPMP_Tool_Signature(obj.IPMP_Tool_Signature)
{
	isSigned = obj.isSigned;
	sizeofTool = obj.sizeofTool;
	Tool_Body = new byte[sizeofTool];
	memcpy(Tool_Body, obj.Tool_Body, sizeofTool);
	memcpy(Verifying_Tool_Id, obj.Verifying_Tool_Id, 16);
}


IPMP_ToolES_AU::~IPMP_ToolES_AU()
{
	clearUp();
}

int IPMP_ToolES_AU::toBuffer(byte * buf)
{
	setWriteBuffer(buf);
	put( isSigned, 1);
	put( RESERVED, 7);

	if( isSigned ){
		putObject(IPMP_Tool_Signature);
		unsigned int numCerts = certificateList.getObjectNum();
		put (numCerts, 16);											// must be less that 2^16 

		for(unsigned int f=0;f<numCerts;f++){
			put( certificateList.getObject(f).certType, 8);			// certType
			putObject(certificateList.getObject(f).certificate);	// certificate
		}
		for(f=0;f<16;f++){
			put( Verifying_Tool_Id[f], 8 );
		}
	}
	put( sizeofTool, 32);
	
	for(unsigned long f=0;f<sizeofTool;f++)
		put( Tool_Body[f], 8); 
	
	return getWriteCounter();
}

int IPMP_ToolES_AU::fromBuffer(const byte * buf)
{
	clearUp();
	setReadBuffer(buf);
	isSigned = (get(1) == 1);
	skipReading(7);						// reserved

	if( isSigned ){
		getObject(IPMP_Tool_Signature);
		word32 numCerts = get(16);											// numCerts

		for(unsigned int f=0;f<numCerts;f++){
			CertStruct cs;
			cs.certType = (byte) get(8);			// certType
			getObject(cs.certificate);		// certificate
			certificateList.addObject(cs);
		}

		for(f=0;f<16;f++){
			Verifying_Tool_Id[f] = (byte) get(8);
		}
	}

	sizeofTool = get(32);
	Tool_Body = new byte [sizeofTool];
	for(unsigned long f=0;f<sizeofTool;f++)
		Tool_Body[f] = (byte) get(8); 
	
	return getReadCounter();
}


bool IPMP_ToolES_AU::checkSyntax() const
{
	return ((sizeofTool > 0) && (Tool_Body !=  NULL));		// must contain a Tool
}

unsigned int IPMP_ToolES_AU::getSize() const 
{
	unsigned int nSize = 1 ;							// isSigned

	if(isSigned)
	{
		nSize += IPMP_Tool_Signature.getSize();			// IPMP_Tool_Signature
		nSize+=2;										// numCerts
		for(unsigned int f=0;f<certificateList.getObjectNum();f++)
		{
			nSize += 1;														// CertType
			nSize += certificateList.getObject(f).certificate.getSize();	// certificate	
		}
		nSize += 16;										// Verifying_Tool_Id
	}
	nSize += 4;											// sizeofTool
	nSize += (sizeofTool);								// Tool_Body

	return nSize ;
}


void IPMP_ToolES_AU::clearUp()
{
	delete [] Tool_Body;
	Tool_Body = NULL;

	certificateList.clear();
}

unsigned int IPMP_ToolES_AU::getSizeofTool() const
{
	return sizeofTool;
}

unsigned char * IPMP_ToolES_AU::getToolBody() const
{
	return Tool_Body;
}

unsigned int IPMP_ToolES_AU::getNumCertificates()
{
	return certificateList.getObjectNum();
}

Certificate* IPMP_ToolES_AU::getCertificate(unsigned int index)
{
	return & certificateList.getObject(index).certificate;
}

unsigned int IPMP_ToolES_AU::getCertType(unsigned int index)
{
	return certificateList.getObject(index).certType;
}

unsigned int IPMP_ToolES_AU::getVerifyingToolID(unsigned int index)
{
	return Verifying_Tool_Id[index];
}

bool IPMP_ToolES_AU::addCertificate(byte myType, Certificate *pcert)
{
	assert(pcert != NULL);

	CertStruct cs ;
	
	cs.certType = myType;
	cs.certificate = *pcert;

	int ret = certificateList.addObject(cs);

	return (ret != -1);
}

void IPMP_ToolES_AU::setToolBody(const byte *buf, unsigned int size)
{
	if(Tool_Body != NULL)
		delete [] Tool_Body;
	sizeofTool = size;
	Tool_Body = new byte[size];
	memcpy(Tool_Body, buf, size);
}


void IPMP_ToolES_AU::setVerifyingToolId(const byte *ptr)
{
	memcpy(Verifying_Tool_Id, ptr, 16);
}


void IPMP_ToolES_AU::setIPMP_Tool_Signature(byte *buf, unsigned int len)
{
	isSigned = true;	
	IPMP_Tool_Signature.setData(buf, len);
}


byte * IPMP_ToolES_AU::getIPMP_Tool_Signature()
{
	if (isSigned)
		return IPMP_Tool_Signature.getData();
	else
		return NULL;
}

unsigned int IPMP_ToolES_AU::getIPMP_Tool_SignatureLen()
{
	if (isSigned)
		return IPMP_Tool_Signature.getDataSize();
	else
		return 0;
}
