
//	This software module was originally developed by James King (Central
//	Research Laboratories Ltd) and edited by Filippo Chiariglione (Central 
//	Research Laboratories Ltd) and Massimo Balestri (Telecom Italia S.p.A.)
//	in the course of development of the MPEG-4 Systems 
//	(ISO/IEC 14496-1) standard. This software module is an implementation 
//	of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified 
//	by the MPEG-4 Systems (ISO/IEC 14496-1) standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	Central Research Laboratories Ltd retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2000. 





// IPMP_ToolES_AU.h: interface for the ByteStream class.

#ifndef __IPMP_TOOLES_AU__
#define __IPMP_TOOLES_AU__// IPMP_ToolES_AU.h: interface for the IPMP_ToolES_AU class.
//
//////////////////////////////////////////////////////////////////////

#include "IPMP_Data_BaseClass.h"
#include "IPMP_MutualAuthentication.h"
#include "common.h"
#include "namespace.h"

BEGIN_MOSES_NAMESPACE

/**
 *  @class IPMP_ToolES_AU
 *  This class defines an Access Unit which carries IPMP Tools in the bitstream.
 *  A concatenation of such access units form an IPMP Tool Elementary Stream. Such streams are
 *  identified by the StreamType identifier "IPMPToolStream".
 *  
 *  @author Massimo Balestri
 *  @date 2003 
 */
class IPMP_ToolES_AU : public BitField
{
private:
	struct CertStruct {
		byte certType;
		Certificate certificate;
	};

	bool	isSigned;
	IPMP_ByteArray	IPMP_Tool_Signature;
	byte Verifying_Tool_Id[16];	// 128 BIT

	word32 sizeofTool;			// 32 BIT
	byte* Tool_Body;			// 8 BIT POINTER
	Container<CertStruct> certificateList ;				// contains the list of certificates associated to the Tool

public:

	/**
	* constructor.
	*/
	IPMP_ToolES_AU();

	/**
	* copy constructor.
	*/
	IPMP_ToolES_AU(const IPMP_ToolES_AU & obj);

	/**
	* destructor.
	*/
	virtual ~IPMP_ToolES_AU();
	
	/**
	* Sets the IPMP tool signature
	* @param buf pointer to the signature buffer
	* @param len length of the signature buffer
	*/
	void setIPMP_Tool_Signature(byte *buf, unsigned int len);
	
	/**
	* Gets the IPMP tool signature
	* @return a pointer to the tool signature
	*/
	byte * IPMP_ToolES_AU::getIPMP_Tool_Signature();

	/**
	* Gets the length in bytes of the IPMP tool signature
	* @return the signature size in bytes
	*/
	unsigned int getIPMP_Tool_SignatureLen();
	
	/**
	* Sets the verifying tool ID
	* @param ptr pointer to the tool id buffer
	*/
	void setVerifyingToolId(const byte *ptr);

	/**
	* Sets the tool body
	* @param buf pointer to the tool body buffer
	* @param size size of the buffer
	*/
	void setToolBody(const byte* buf, unsigned int size);

	/**
	* Attaches a certificate to this message
	* @param certType the type of the Certificate
	* @param pcert pointer to a certificate
	*/
	bool addCertificate(byte certType, Certificate* pcert);

	/**
	* Get the type of the n-th certificate in the certificate list.
	* @param index the index in the list of certificates
	* @return the type of the n-th certificate
	*/
	unsigned int getCertType(unsigned int index);

	/**
	* Gets a byte from the verifying tool ID
	* @param Index index into the tool id array 
	* @returns an 8 bit value at the index specified
	*/
	unsigned int getVerifyingToolID(unsigned int Index);

	/**
	* Gets the number of certificates
	* @return number of certificates currently stored in the certificate list
	*/
	unsigned int getNumCertificates();

	/**
	* Get a certificate 
	* @param Index into the certificate array
	* @return pointer to the specified certificate. Returns NULL if the index value is invalid
	*/
	Certificate* getCertificate(unsigned int Index);

	/**
	* Gets the ToolBody all in once
	* @return a pointer to the tool body. The size of the tool body can be obtained calling getSizeofTool().
	*/
	unsigned char * getToolBody() const;

	/**
	* Gets the size of this tool data
	* @return tool data size
	*/
	unsigned int getSizeofTool() const;

	/**
	* method to free memory
	*/
	void clearUp();

	/**
	 * method for writing the message using the data of the class.
	 * @param buf storage location for the byte sequence. 
	 */
	int toBuffer(byte * buf);

	/**
	 * method for reading the contents of the message.
	 * @param buf message storage location for the byte sequence. 
	 */
	int fromBuffer(const byte * buf);

	virtual bool checkSyntax() const;			// reimplemented
	virtual unsigned int getSize() const;		// reimplemented

};

END_MOSES_NAMESPACE

#endif // !defined(AFX_IPMP_TOOLES_AU_H__2E95BAD2_889B_482B_A8CB_4F7B26C3C2E1__INCLUDED_)
