
//	This software module was originally developed by Massimo Balestri 
//	(Telecom Italia Lab S.p.A.) in the course of development 
//	of the MPEG-4 Systems (ISO/IEC 14496-1) standard. This software module 
//	is an implementation of a part of one or more MPEG-4 Systems (ISO/IEC 
//	14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1) 
//	standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	Telecom Italia Lab S.p.A retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2000. 


#include "IPMP_ToolId.h"
#include <stdio.h>
#include <string.h>
#include "HexUtils.h"

USING_MOSES_NAMESPACE

//////////////////////////////////////////////////////////////////////
// class IPMP_ToolId
//////////////////////////////////////////////////////////////////////
IPMP_ToolId::IPMP_ToolId()
{
	Data1 = 0;
	Data2 = 0;
	Data3 = 0;
	memset(Data4, 0, sizeof(Data4));
}

IPMP_ToolId::IPMP_ToolId(const char *string)
{
	fromString(string);
}

IPMP_ToolId::~IPMP_ToolId()
{
}

// write something like "6B29FC40-CA47-1067-B31D-00DD010662DA"
void IPMP_ToolId::toString(char * string) const
{
	char substr1[24], substr2[32];
	HexUtils::tohex(Data4, 2, substr1);
	HexUtils::tohex(Data4 + 2, 6, substr2);

	sprintf(string, "%08lX-%04hX-%04hX-%s-%s", 
		Data1, Data2, Data3, substr1, substr2);
}

// read something like "6B29FC40-CA47-1067-B31D-00DD010662DA"
void IPMP_ToolId::fromString(const char * string) 
{
	sscanf(string, "%08lX-%04hX-%04hX", 
		&Data1, &Data2, &Data3);
	
	const char * ptr = string + 19;
	HexUtils::fromhex(Data4, 2, ptr);		// read first two bytes of Data4

	ptr = string + 24;
	HexUtils::fromhex(Data4 + 2, 6, ptr);	// read last six bytes of Data4
}

int IPMP_ToolId::fromBuffer(const byte * in)
{
	setReadBuffer(in);
	Data1 = get(32);
	Data2 = (unsigned short) get(16);
	Data3 = (unsigned short) get(16);
	getBuffer(Data4, 64);
	return getReadCounter();
}

int IPMP_ToolId::toBuffer(byte * out)
{
	setWriteBuffer(out);
	put(Data1, 32);
	put(Data2, 16);
	put(Data3, 16);
	putBuffer((byte *) Data4, 64);
	return getWriteCounter();
}

// return the size of this class, in bytes
unsigned int IPMP_ToolId::getSize() const 
{
	return 16;			// 128 bit = 16 bytes
}
	
bool IPMP_ToolId::checkSyntax() const
{
	return true;
}

void IPMP_ToolId::GetID(unsigned char * id) const
{
	memcpy(id, &Data1, 4);
	memcpy(id+4, &Data2, 2);
	memcpy(id+6, &Data3, 2);
	memcpy(id+8, Data4, 8);
}

void IPMP_ToolId::SetID(const unsigned char * id)
{
	memcpy(&Data1, id, 4);
	memcpy(&Data2, id+4, 2);
	memcpy(&Data3, id+6, 2);
	memcpy(Data4, id+8, 8);
}
