
//	This software module was originally developed by Massimo Balestri 
//	(Telecom Italia Lab S.p.A.) in the course of development 
//	of the MPEG-4 Systems (ISO/IEC 14496-1) standard. This software module 
//	is an implementation of a part of one or more MPEG-4 Systems (ISO/IEC 
//	14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1) 
//	standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	Telecom Italia Lab S.p.A retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2000. 


#ifndef __IPMP_TOOLID_H__
#define __IPMP_TOOLID_H__
#include "BitField.h"
#include "namespace.h"

BEGIN_MOSES_NAMESPACE

/**
 * @class IPMP_ToolId
 * This class stores the 128-bit unique ID of an IPMP Tool.
 * The ID can be used as a generic string of bytes or a MS GUID.
 * A GUID is a 128-bit value consisting of one group of 8 hexadecimal digits, followed by three groups of 4 hexadecimal digits each, followed by one group of 12 hexadecimal digits. 
 * The following example shows the groupings of hexadecimal digits in a GUID:  6B29FC40-CA47-1067-B31D-00DD010662DA
 * GUIDs are the Microsoft implementation of the distributed computing environment (DCE) universally unique identifier (UUID).
 * Use "uuidgen -s" to generate GUIDs on a PC with Visual Studio installed (you must run VCVARS32.BAT before running uuidgen). 
 * @author Massimo Balestri
 * @date 2003 
 */
class IPMP_ToolId : public BitField
{
public:
	unsigned long  Data1;		///< Specifies the first 8 hexadecimal digits of the GUID.
	unsigned short Data2;		///< Specifies the first group of 4 hexadecimal digits.
	unsigned short Data3;		///< Specifies the second group of 4 hexadecimal digits.
	unsigned char  Data4[8];	///< Specifies an array of 8 bytes. The first 2 bytes contain the third group of 4 hexadecimal digits. The remaining 6 bytes contain the final 12 hexadecimal digits. 

public:
	IPMP_ToolId();									///< default constructor
	virtual ~IPMP_ToolId();							///< destructor
	virtual int toBuffer(byte * buf);				// reimplemented
	virtual int fromBuffer(const byte * buf);		// reimplemented
	unsigned int getSize() const;					// reimplemented
	virtual bool checkSyntax() const;				// reimplemented

	/**
	 * Alternate constructor.
	 * This constructor assign the GUID from a string like the following:  6B29FC40-CA47-1067-B31D-00DD010662DA.
	 * @param string the string to read
	 */
	IPMP_ToolId(const char * string);

	/**
	 * Convert the GUID to a string like the following:  6B29FC40-CA47-1067-B31D-00DD010662DA.
	 * @param string the string to write
	 */
	void toString(char * string) const;

	/**
	 * Assign the GUID from a string like the following:  6B29FC40-CA47-1067-B31D-00DD010662DA.
	 * @param string the string to read
	 */
	void fromString(const char * string);
	
	/**
	 * Get the ID as a string of bytes.
	 * Maintained for backward compatibility.
	 * @param id pointer to a block of memory of at least 16 bytes, where the ID will be returned.
	 */
	void GetID(unsigned char *id) const;

	/**
	 * Get the ID as a string of bytes.
	 * Maintained for backward compatibility.
	 * @param id pointer to a block of memory of at least 16 bytes, containing the ID.
	 */
	void SetID(const unsigned char * id);
};

END_MOSES_NAMESPACE

#endif
