//	This software module was originally developed by Massimo Balestri (TILAB)	
//	in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1) standard. 
//	This software module is an implementation of a part of one or more MPEG-4 
//	Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 
//	14496-1) standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	ADETTI retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2002.


// IPMP_ToolMessageBase.cpp

#include "IPMP_ToolMessageBase.h"
#include "common.h"
#include <stdlib.h>

USING_MOSES_NAMESPACE

IPMP_ToolMessageBase::IPMP_ToolMessageBase(byte tag)
:   ExpandableBaseClass(tag),
	Version(VERSION),
	sender(0),
	recipient(0)
{
	
}

IPMP_ToolMessageBase::~IPMP_ToolMessageBase() 
{

}

IPMP_ToolMessageBase::IPMP_ToolMessageBase(const IPMP_ToolMessageBase & msg)
:	ExpandableBaseClass(msg)
{	
	Version = msg.Version;
	sender = msg.sender;
	recipient = msg.recipient;
}

void IPMP_ToolMessageBase::setVersion(byte ver)
{
	Version = ver;
}

byte IPMP_ToolMessageBase::getVersion()
{
	return (byte) Version;
}

bool IPMP_ToolMessageBase::checkSyntax() const
{
	if (Version != 0x01)
		return false;					// syntax version must be 0x01 for IPMP_ToolMessageBase messages.
	return ExpandableBaseClass::checkSyntax();
}

int IPMP_ToolMessageBase::fromBuffer(const byte * buf)
{
	int nbits = ExpandableBaseClass::fromBuffer(buf);	// read tag and size
	Version = get(buf, nbits, 8);						// read version id
	nbits += 8;
	sender = get(buf, nbits, 32);						// read sender id
	nbits += 32;
	recipient = get(buf, nbits, 32);					// read recipient id
	nbits += 32;
	return nbits;
}


int IPMP_ToolMessageBase::toBuffer(byte * buf)
{
	int nbits = ExpandableBaseClass::toBuffer(buf);		// write tag and size
	put( Version, buf, nbits, 8 ) ;
	nbits+=8 ;
	put(sender, buf, nbits, 32);						// write sender id
	nbits += 32;
	put(recipient, buf, nbits, 32);					// write recipient id
	nbits += 32;
	return nbits;
}

unsigned int IPMP_ToolMessageBase::sizeOfInstance() const
{
	return 9;						//  (9 bytes)
}

// ----- access methods ----


word32 IPMP_ToolMessageBase::getSender()
{
	return sender;
}

void IPMP_ToolMessageBase::setSender(word32 newsender)
{
	sender = newsender;
}

word32 IPMP_ToolMessageBase::getRecipient()
{
	return recipient;
}

void IPMP_ToolMessageBase::setRecipient(word32 newrecipient)
{
	recipient = newrecipient;
}

// ------ factory methods ---------

IPMP_ToolMessageBase * IPMP_ToolMessageBase::createMessage(byte tag)
{
	IPMP_ToolMessageBase * pNewMessage = NULL;

    switch (tag)
    {
    case IPMP_MessageFromBitstream_tag:
        pNewMessage=new IPMP_MessageFromBitstream;
        break;
    
    case IPMP_DescriptorFromBitstream_tag:
        pNewMessage=new IPMP_DescriptorFromBitstream;
        break;
    
    case IPMP_MessageFromTool_tag:
        pNewMessage=new IPMP_MessageFromTool;
        break;

	default: break;
    }
	return pNewMessage;
}


IPMP_ToolMessageBase * IPMP_ToolMessageBase::createMessage(const byte * message, unsigned int & nBitRead )
{
	IPMP_ToolMessageBase * pNewMessage = createMessage(message[0]);
	
	if (pNewMessage != NULL)
	{
		nBitRead = pNewMessage->fromBuffer(message);
	}
	return pNewMessage;
}

IPMP_ToolMessageBase * IPMP_ToolMessageBase::createMessage(IPMP_ToolMessageBase &object)
{
	unsigned int size = object.getSize();
	byte * buffer = new byte[size];			
	object.toBuffer(buffer);						// dump data into a buffer
	unsigned int nbitsRead;
	IPMP_ToolMessageBase * newobj = createMessage(buffer, nbitsRead);
	delete[] buffer;
	return newobj;
}

