
//	This software module was originally developed by Massimo Balestri (TILAB)
//	in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1) standard. 
//	This software module is an implementation of a part of one or more MPEG-4 
//	Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 
//	14496-1) standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	EPFL retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2002.


// IPMP_ToolMessageBase.h


#ifndef	IPMP_TOOLMESSAGEBASE_H
#define	IPMP_TOOLMESSAGEBASE_H

#include "ExpandableBaseClass.h"
#include "namespace.h"

BEGIN_MOSES_NAMESPACE

#define VERSION		0x01		

					
//----------- list of tags for classes extending the IPMP_ToolMessageBase ---------//
#define Forbidden									0x00
#define IPMP_MessageFromBitstream_tag				0x01
#define IPMP_DescriptorFromBitstream_tag			0x02
#define IPMP_MessageFromTool_tag					0x03


/** 
 *  @class IPMP_ToolMessageBase
 *  @brief This class is used to carry IPMP Tool messages.
 *  This is the base class for all messages with tags defined above.
 *
 *  @author Massimo Balestri
 *  @date 2002
 */
class IPMP_ToolMessageBase: public ExpandableBaseClass
{
private:
	word32 Version;		// the version of syntax used - must be 0x01
	word32 sender;				// the sender of the message
	word32 recipient;			// the recipient of the message

	IPMP_ToolMessageBase();		// make this an abstract class

protected:
	/**
	 * method to set the version of syntax used (for IPMP Extension messages this must be 0x01)
	 * @param ver the version number to set
	 */
	void setVersion(byte ver);

	/**
	 * creates an object of a specified type, identified by the tag.
	 * @param tag the object_id of the instance to be created. 
	 */
	IPMP_ToolMessageBase(byte tag);	
	
	/**
	 *	Copy constructor.
	 *	Creates a copy of the object given as input parameter.
	 *  @param msg the object to copy
	 */
	IPMP_ToolMessageBase::IPMP_ToolMessageBase(const IPMP_ToolMessageBase & msg);

public:

	virtual ~IPMP_ToolMessageBase();					///< destructor	
	virtual int toBuffer(byte * buf);					// reimplemented
	virtual int fromBuffer(const byte * buf);			// reimplemented
	virtual bool checkSyntax() const;					// reimplemented
	virtual unsigned int sizeOfInstance() const;		// reimplemented

	/**
	 * Get the version of syntax of this message (must be 0x01).
	 * @return the syntax version number
	 */
	byte getVersion();

	/**
	 * Get the context id of the sender the originator of the message.  
	 * The value 0x00 is reserved for the terminal.
	 * @return the context id of the sender Tool.
	 */
	word32 getSender();

	/**
	 * Set the context id of the sender the originator of the message.  
	 * The value 0x00 is reserved for the terminal.
	 *
	 * @param sender the context id of the sender Tool
	 */
	void setSender(word32 sender);

	/**
	 * Get the the context ID of the intended recipient of the message.  
	 * The value 0x00 is reserved for the terminal.
	 * @return the context id of the destination Tool.
	 */
	word32 getRecipient();
	
	/**
	 * Set the the context ID of the intended recipient of the message.  
	 * The value 0x00 is reserved for the terminal.
	 * @return the context id of the destination Tool.
	 */
	void setRecipient(word32 recipient);

	/**
	 * This static "factory" method creates and initialize an object derived 
	 * from this class from the binary message given in input.
	 * The caller must delete the object when not used anymore.
	 * @param message the binary message from which the object is created.
	 * @param nBitRead returns the number of bits read from "message".
	 * @return a pointer to an initialized object constructed from the bitstream
	 */
	static IPMP_ToolMessageBase * createMessage(const byte * message, unsigned int & nBitRead);
	
	/**
	 * This static "factory" method creates an empy object derived 
	 * from this class using the tag given in input.
	 * The caller must delete the object when not used anymore.
	 * @param tag the object tag
	 * @return a pointer to an empty object with the same tag
	 */
	static IPMP_ToolMessageBase * createMessage(byte tag);

	/**
	 * This static "factory" method creates and initialize an object derived 
	 * from this class from the object in input.
	 * The caller must delete the object when not used anymore.
	 * @param object the object to clone
	 * @return a pointer to an initialized object of the same type
	 */
	static IPMP_ToolMessageBase * createMessage(IPMP_ToolMessageBase &object);
};

END_MOSES_NAMESPACE

#endif