
//	This software module was originally developed by Claudio Alberti (EPFL)	
//	in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1) standard. 
//	This software module is an implementation of a part of one or more MPEG-4 
//	Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 
//	14496-1) standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	EPFL retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2002.


// IPMP_ToolNotificationListener.cpp

#include "IPMP_ToolNotificationListener.h"
#include <memory.h>
#include <string.h>

USING_MOSES_NAMESPACE

IPMP_AddToolNotificationListener::IPMP_AddToolNotificationListener()
	:	eventTypeCount(0),
		scope(Constants::SCOPE_LOCAL),
		IPMP_Data_BaseClass( IPMP_AddToolNotificationListener_tag )
{
}



IPMP_AddToolNotificationListener::~IPMP_AddToolNotificationListener() 
{
}

bool IPMP_AddToolNotificationListener::addEventType(Constants::EventType et)
{
	
	for (unsigned int i=0; i<eventTypeCount; i++)
	{
		if (et == eventArray[i])			// check if the event is already in the list
			return false;
	}

	if (eventTypeCount < 255)
	{
		eventArray[eventTypeCount++] = (byte) et ;		// insert the event in the list
	}
	else
		return false;

	return true;
}

bool IPMP_AddToolNotificationListener::getEventType(Constants::EventType &et, unsigned int index) const
{
	if (index >= eventTypeCount)				// check if the index is OK
		return false;

	et = (Constants::EventType) eventArray[index];					// return the event type

	return true;
}


unsigned int IPMP_AddToolNotificationListener::getEventTypeCount() const
{
	return eventTypeCount;
}

void IPMP_AddToolNotificationListener::setScope(Constants::ScopeType newscope)
{
	scope = newscope;
}

Constants::ScopeType IPMP_AddToolNotificationListener::getScope() const
{
	return scope;
}

int IPMP_AddToolNotificationListener::fromBuffer(const byte * buf)
{
	IPMP_Data_BaseClass::fromBuffer(buf);		//read the base class and set "buf" as read buffer for this class.
	
	scope = (Constants::ScopeType) get(3);								// read the scope
	skipReading(5);											// skip 5 reserved bits
	eventTypeCount = get(8);								// read the counter
	getBuffer(eventArray, 8 * eventTypeCount);				// get the vector of event types
	return getReadCounter();								// return the number of bits read so far
}


int IPMP_AddToolNotificationListener::toBuffer(byte * buf)
{
	IPMP_Data_BaseClass::toBuffer(buf);

	put(scope, 3);
	put(0, 5);		// reserved
	put(eventTypeCount, 8);
	putBuffer(eventArray, 8 * eventTypeCount);
	return getWriteCounter();
}

unsigned int IPMP_AddToolNotificationListener::sizeOfInstance() const
{
	return IPMP_Data_BaseClass::sizeOfInstance() + eventTypeCount + 2 ;
}




///////////////////////// IPMP_RemoveToolNotificationListener ////////////////////////////////////

IPMP_RemoveToolNotificationListener::IPMP_RemoveToolNotificationListener()
	:	eventTypeCount(0),
		IPMP_Data_BaseClass( IPMP_RemoveToolNotificationListener_tag )
{
}


IPMP_RemoveToolNotificationListener::~IPMP_RemoveToolNotificationListener() 
{
}

bool IPMP_RemoveToolNotificationListener::addEventType(Constants::EventType et)
{
 	
	for (unsigned int i=0; i<eventTypeCount; i++)
 	{
 		if (et == eventArray[i])			// check if the event is already in the list
			return false;
	}
 
 	if (eventTypeCount < 255)
 	{
		eventArray[eventTypeCount++] = (byte) et ;		// insert the event in the list
 	}
 	else
 		return false;

 	return true;
}

bool IPMP_RemoveToolNotificationListener::getEventType(Constants::EventType &et, unsigned int index) const
{
	if (index >= eventTypeCount)				// check if the index is OK
		return false;

	et = (Constants::EventType) eventArray[index];					// return the event type

	return true;
}


unsigned int IPMP_RemoveToolNotificationListener::getEventTypeCount() const
{
	return eventTypeCount;
}


int IPMP_RemoveToolNotificationListener::fromBuffer(const byte * buf)
{
	IPMP_Data_BaseClass::fromBuffer(buf);					// read the base class and set "buf" as read buffer for this class.
	
	eventTypeCount = get(8);								// read the counter
	getBuffer(eventArray, 8 * eventTypeCount);				// get the vector of event types
	return getReadCounter();								// return the number of bits read so far
}


int IPMP_RemoveToolNotificationListener::toBuffer(byte * buf)
{
	IPMP_Data_BaseClass::toBuffer(buf);

	put(eventTypeCount, 8);
	putBuffer(eventArray, 8 * eventTypeCount);
	return getWriteCounter();
}

unsigned int IPMP_RemoveToolNotificationListener::sizeOfInstance() const
{
	return IPMP_Data_BaseClass::sizeOfInstance() + eventTypeCount + 1 ;
}


//////////////////////////////////// IPMP_NotifyToolEvent ////////////////////////////////////

IPMP_NotifyToolEvent::IPMP_NotifyToolEvent():
	m_hasID(false),
	m_eventType(Constants::EVENT_USER_DEFINED_BASE),
	m_toolContextID(0),
	IPMP_Data_BaseClass( IPMP_NotifyToolEvent_tag )
{
}


IPMP_NotifyToolEvent::~IPMP_NotifyToolEvent() 
{
}

Constants::EventType IPMP_NotifyToolEvent::getEventType() const
{
	return m_eventType;
}

void IPMP_NotifyToolEvent::setEventType(Constants::EventType et)
{
	m_eventType = et;
}

bool IPMP_NotifyToolEvent::hasID() const
{
	return m_hasID;
}

void IPMP_NotifyToolEvent::setScope(unsigned int OD_ID, unsigned int ESD_ID)
{
	m_hasID = true;
	m_OD_ID = OD_ID;
	m_ESD_ID = ESD_ID;
}

bool IPMP_NotifyToolEvent::getScope(unsigned int &OD_ID, unsigned int &ESD_ID) const
{
	if (hasID())
	{
		OD_ID = m_OD_ID;
		ESD_ID = m_ESD_ID;
		return true;
	}
	else 
		return false;
}

void IPMP_NotifyToolEvent::setToolContextID(unsigned int id)
{
	m_toolContextID = id;
}

unsigned int IPMP_NotifyToolEvent::getToolContextID() const
{
	return m_toolContextID ;
}

int IPMP_NotifyToolEvent::fromBuffer(const byte * buf)
{
	IPMP_Data_BaseClass::fromBuffer(buf);					// read the base class and set "buf" as read buffer for this class.
	
	m_hasID = ( 1 == get(1));								// hadID flag (1 bit)
	skipReading(7);											// skip next 7 reserved bits
	if (m_hasID)
	{
		m_OD_ID = get(10);
		skipReading(6);
		m_ESD_ID = get(16);
	}
	m_eventType = (Constants::EventType) get(8);
	m_toolContextID = get(32);
	return getReadCounter();								// return the number of bits read so far
}


int IPMP_NotifyToolEvent::toBuffer(byte * buf)
{
	IPMP_Data_BaseClass::toBuffer(buf);

	if (m_hasID)
	{
		put(1,1);
		put(0,7);
	}
	else
		put(0,8);

	if (m_hasID)
	{
		put(m_OD_ID, 10);
		put(0, 6);
		put(m_ESD_ID, 16);
	}
	put(m_eventType, 8);
	put(m_toolContextID, 32);
	return getWriteCounter();
}

unsigned int IPMP_NotifyToolEvent::sizeOfInstance() const
{
	unsigned int tot = IPMP_Data_BaseClass::sizeOfInstance() + 6 ;
	
	if (m_hasID)
		return (tot + 4);
	else
		return tot;
}


