
//	This software module was originally developed by Claudio Alberti (EPFL)	
//	in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1) standard. 
//	This software module is an implementation of a part of one or more MPEG-4 
//	Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 
//	14496-1) standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	EPFL retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2002.

// IPMP_ToolParamCapabilitiesResponse.h

#ifndef	IPMP_TOOLPARAMCAPABILITIESRESPONSE_H
#define	IPMP_TOOLPARAMCAPABILITIESRESPONSE_H

#include "IPMP_Data_BaseClass.h"
#include "namespace.h"

BEGIN_MOSES_NAMESPACE

/** @class IPMP_ToolParamCapabilitiesResponse
 *  This message is the response to the above parametric capabilities query and simply returns 
 *  a boolean value as to whether or not the parametric description is supported by the tool.
 *  
 *  @author Claudio Alberti
 *  @date 2002
 */
class IPMP_ToolParamCapabilitiesResponse: public IPMP_Data_BaseClass 
{
private:
	bool capabilitiesSupported;

public:
	/**
	 * constructor.
	 */
	IPMP_ToolParamCapabilitiesResponse();	
	
	/**
	 * destructor.
	 */
	virtual ~IPMP_ToolParamCapabilitiesResponse();		

	virtual int toBuffer(byte * buf);					// reimplemented
	virtual int fromBuffer(const byte * buf);			// reimplemented
	virtual unsigned int sizeOfInstance() const;		// reimplemented
	
	/**
	 * Indicate if the required capabilities are supported.
	 * @param cs the cs flag: true means that the capabilities are supported.
	 */
	void setCapabilitiesSupported(bool cs);
	
	/**
	 * Indicate if the required capabilities are supported.
	 * @return the cs flag: if true the capabilities are supported.
	 */
	bool areCapabilitiesSupported() const;

};

END_MOSES_NAMESPACE

#endif
