// IPMP_UserDefinedBase.h: interface for the IPMP_UserDefinedBase class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_IPMP_USERDEFINEDBASE1_H__09FDD790_6128_4B8C_949F_DD100F6F89B5__INCLUDED_)
#define AFX_IPMP_USERDEFINEDBASE1_H__09FDD790_6128_4B8C_949F_DD100F6F89B5__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "IPMP_Data_BaseClass.h"

BEGIN_MOSES_NAMESPACE


/**	
 * @class IPMP_UserDefinedBase
 * This class is the base class for all user-defined messages deriving from IPMP_Data_BaseClass
 * @author Massimo Balestri
 * @date 2003
 */
class IPMP_UserDefinedBase : public IPMP_Data_BaseClass  
{
protected:
	/**
	 * creates an object of a specified type, identified by the tag.
	 * @param tag the object_id of the instance to be created.
	 */
	IPMP_UserDefinedBase(byte tag);	
	
	/**
	 *	Copy constructor.
	 *	Creates a copy of the object given as input parameter
	 *  @param msg the object to copy
	 */
	IPMP_UserDefinedBase(const IPMP_UserDefinedBase& msg);

public:
	virtual ~IPMP_UserDefinedBase();					///< destructor

	/**
	 * This static "factory" method creates and initialize an object derived 
	 * from this class from the binary message given in input.
	 * The caller must delete the object when not used anymore.
	 * @param message the binary message from which the object is created.
	 * @param nBitRead returns the number of bits read from "message".
	 * @return a pointer to an initialized object constructed from the bitstream
	 */
	static IPMP_Data_BaseClass * createMessage(const byte * message, unsigned int & nBitRead);
	
	/**
	 * This static "factory" method creates an empy object derived 
	 * from this class using the tag given in input.
	 * The caller must delete the object when not used anymore.
	 * @param tag the object tag
	 * @return a pointer to an empty object with the same tag
	 */
	static IPMP_Data_BaseClass * createMessage(byte tag);

	/**
	 * This static "factory" method creates and initialize an object derived 
	 * from this class from the object in input.
	 * The caller must delete the object when not used anymore.
	 * @param object the object to clone
	 * @return a pointer to an initialized object of the same type
	 */
	static IPMP_Data_BaseClass * createMessage(IPMP_Data_BaseClass &object);
};

END_MOSES_NAMESPACE

#endif // !defined(AFX_IPMP_USERDEFINEDBASE1_H__09FDD790_6128_4B8C_949F_DD100F6F89B5__INCLUDED_)
