
//	This software module was originally developed by ADETTI  and modified by
//  Massimo Balestri in the course of
//	development of the MPEG-4 Systems (ISO/IEC 14496-1) standard. 
//	This software module is an implementation of a part of one or more MPEG-4 
//	Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 
//	14496-1) standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	ADETTI retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2002.


// IPMP_UserQuery.cpp

#include "IPMP_UserQuery.h"

#include <string.h>
#include <assert.h>

USING_MOSES_NAMESPACE

//////////////////////////////////////////////////////////////////////////////
// 
// IPMP_UserQuery - note: the default copy constructor is OK for this class.
//					So don't define a c.c. !
// 

IPMP_UserQuery::IPMP_UserQuery()
	:	IPMP_Data_BaseClass(IPMP_UserQuery_tag)
{
	inclDisplayTitle = false;
	inclDisplayText = false;
	needReplyText = false;
	inclOptionSelect = false;
	inclSMIL = false;
}

IPMP_UserQuery::~IPMP_UserQuery()
{
	clear();
}

void IPMP_UserQuery::clear()
{
	inclSMIL = false;
	inclOptionSelect = false;
	needReplyText = false;
	inclDisplayText = false;
	inclDisplayTitle = false;
	userQueryText.clear();
}

int IPMP_UserQuery::toBuffer(byte * buf)
{
	IPMP_Data_BaseClass::toBuffer(buf);

	put(inclDisplayTitle, 1);		// control bits
	put(inclDisplayText, 1);
	put(needReplyText, 1);
	put(inclOptionSelect, 1);
	put(inclSMIL, 1);
	put(0, 3);						// reserved

	unsigned int numOfAltText = userQueryText.getObjectNum();
	put(numOfAltText, 24);		// numOfAltText

	// user query text
	for (word32 nat = 0; nat < numOfAltText; ++nat) {
		skipWriting( userQueryText.getObject(nat).toBuffer(getWritePtr()));
	}

	return (getWriteCounter());
}

int IPMP_UserQuery::fromBuffer(const byte * buf)
{
	clear();

	IPMP_Data_BaseClass::fromBuffer(buf);
	inclDisplayTitle = (get(1) != 0);
	inclDisplayText = (get(1) != 0);
	needReplyText = (get( 1) != 0);
	inclOptionSelect = (get(1) != 0);
	inclSMIL = (get(1) != 0);

	get( 3);

	unsigned int numOfAltText = get(24);		// numOfAltText
								
	for (word32 nat = 0; nat < numOfAltText; ++nat) 	// user query text
	{
		UserQueryText query(inclDisplayTitle, inclDisplayText, needReplyText, inclOptionSelect, inclSMIL);
		skipReading( query.fromBuffer(getReadPtr()) );
		userQueryText.addObject(query);
	}

	return (getReadCounter());
}
	
bool IPMP_UserQuery::checkSyntax() const
{
	return IPMP_Data_BaseClass::checkSyntax();
}

// compute size, in BYTES!
unsigned int IPMP_UserQuery::sizeOfInstance() const
{
	unsigned int size = IPMP_Data_BaseClass::sizeOfInstance();

	size += 1;						// control bits
	size += 3;						// numOfAltText
	for (word32 nat = 0; nat < userQueryText.getObjectNum(); ++nat) 	// user query text
	{
		size += userQueryText.getObject(nat).getSize();
	}
	return size;
}

void IPMP_UserQuery::setInclDisplayTitle()
{
	inclDisplayTitle = true;
}

void IPMP_UserQuery::clearInclDisplayTitle()
{
	inclDisplayTitle = false;
}

bool IPMP_UserQuery::getInclDisplayTitle() const
{
	return inclDisplayTitle;
}

void IPMP_UserQuery::setInclDisplayText()
{
	inclDisplayText = true;
}

void IPMP_UserQuery::clearInclDisplayText()
{
	inclDisplayText = false;
}

bool IPMP_UserQuery::getInclDisplayText() const
{
	return inclDisplayText;
}

void IPMP_UserQuery::setNeedReplyText()
{
	needReplyText = true;
}

void IPMP_UserQuery::clearNeedReplyText()
{
	needReplyText = false;
}

bool IPMP_UserQuery::getNeedReplyText() const
{
	return needReplyText;
}

void IPMP_UserQuery::setInclOptionSelect()
{
	inclOptionSelect = true;
}

void IPMP_UserQuery::clearInclOptionSelect()
{
	inclOptionSelect = false;
}

bool IPMP_UserQuery::getInclOptionSelect() const
{
	return inclOptionSelect;
}

void IPMP_UserQuery::setInclSMIL()
{
	inclSMIL = true;
}

void IPMP_UserQuery::clearInclSMIL()
{
	inclSMIL = false;
}

bool IPMP_UserQuery::getInclSMIL() const
{
	return inclSMIL;
}

//////////////////////////////////////////////////////////////////////////////
// 
// UserQueryText
// 
UserQueryText::UserQueryText(bool _inclDisplayTitle,
		bool _inclDisplayText,
		bool _needReplyText,
		bool _inclOptionSelect,
		bool _inclSMIL)
{
	inclDisplayTitle = _inclDisplayTitle;
	inclDisplayText = _inclDisplayText;
	needReplyText = _needReplyText;
	inclOptionSelect = _inclOptionSelect;
	inclSMIL = _inclSMIL;
	isReferenced = false;
}


UserQueryText::~UserQueryText()
{
}


int UserQueryText::toBuffer(byte * buf)
{
	setWriteBuffer(buf);
	put(languageCode, 24);			// 24 bits

	if (inclDisplayTitle)
		skipWriting( titleText.toBuffer(getWritePtr()) );

	if (inclDisplayText)
	{
		word32 numOfDisplayText = displayText.getObjectNum();
		put(numOfDisplayText, 8);
		for (word32 i = 0; i < numOfDisplayText; ++i) 
		{
			skipWriting(displayText.getObject(i).toBuffer(getWritePtr()) );		
		}
	}
	
	if (needReplyText) 
	{
		word32 numOfPromptText = promptText.getObjectNum();
		put(8, numOfPromptText);
		for (word32 i = 0; i < numOfPromptText; ++i) 
		{
			skipWriting( promptText.getObject(i).toBuffer(getWritePtr()) );
		}
	}		
	
	if (inclOptionSelect) 
	{
		word32 numOfOptions = option.getObjectNum();
		put(8, numOfOptions);
		for (word32 i = 0; i < numOfOptions; ++i) 
		{
			skipWriting( option.getObject(i).toBuffer(getWritePtr()) );
		}
	}
	
	if (inclSMIL) 
	{
		put(isReferenced, 1);
		put(0, 7);								// reserved

		if (isReferenced)
			skipWriting( SMIL_URL.toBuffer(getWritePtr()) );
		else
			skipWriting( SMIL.toBuffer(getWritePtr()) );
	}
	return getWriteCounter();
}

int UserQueryText::fromBuffer(const byte * buf)
{
	setReadBuffer(buf);
	languageCode = get(24);			// 24 bits

	if (inclDisplayTitle)
		skipReading( titleText.fromBuffer(getReadPtr()) );

	if (inclDisplayText)
	{
		word32 numOfDisplayText = get(8);
		for (word32 i = 0; i < numOfDisplayText; ++i) 
		{
			DTArray dta;
			skipReading( dta.fromBuffer(getReadPtr()) );
			displayText.addObject(dta);
		}
	}
	
	if (needReplyText) 
	{
		word32 numOfPromptText = get(8);
		for (word32 i = 0; i < numOfPromptText; ++i) 
		{
			QTArray qta;
			skipReading( qta.fromBuffer(getReadPtr()) );
			promptText.addObject(qta);
		}
	}		
	
	if (inclOptionSelect) 
	{
		word32 numOfOptions = get(8);
		for (word32 i = 0; i < numOfOptions; ++i) 
		{
			OptionArray oa;
			skipReading( oa.fromBuffer(getReadPtr()) );
			option.addObject(oa);
		}
	}
	
	if (inclSMIL) 
	{
		isReferenced = (get(1) == 1);
		get(7);									// reserved
		if (isReferenced)
			skipReading( SMIL_URL.fromBuffer(getReadPtr()) );
		else
			skipReading( SMIL.fromBuffer(getReadPtr()) );
	}
	return getReadCounter();
}


bool UserQueryText::checkSyntax() const
{
	return true; // not implemented yet
}

// get the size in bytes
unsigned int UserQueryText::getSize() const 
{
	unsigned int size = 3;		// languageCode

	if (inclDisplayTitle) 
		size += titleText.getSize();

	if (inclDisplayText)
	{
		size += 1;				// numOfDisplayText
		for (word32 i = 0; i < displayText.getObjectNum(); ++i) 
			size += displayText.getObject(i).getSize();
	}

	if (needReplyText) 
	{
		size += 1;				// numOfPromptText
		for (word32 i = 0; i < promptText.getObjectNum(); ++i)
			size += promptText.getObject(i).getSize();
	}

	if (inclOptionSelect) 
	{	
		size += 1;			// numOfOptions
		for (word32 i = 0; i < option.getObjectNum(); ++i) 
			size += option.getObject(i).getSize();
	}

	if (inclSMIL) 
	{	
		size += 1;		// control bits
		if (isReferenced) 
			size += SMIL_URL.getSize();
		else 
			size += SMIL.getSize();		
	}

	return size;
}

void UserQueryText::setLanguageCode(word32 languageCode)
{
	this->languageCode = languageCode;
}

void UserQueryText::clearLanguageCode()
{
	languageCode = 0;
}

word32 UserQueryText::getLanguageCode() const
{
	return languageCode;
}


void UserQueryText::setSMIL_URL(const IPMP_ByteArray &smilUrl)
{
	inclSMIL = true;
	isReferenced = true;
	SMIL_URL = smilUrl;
}

const IPMP_ByteArray & UserQueryText::getSMIL_URL() const
{
	return SMIL_URL;
}

void UserQueryText::setSMIL(const IPMP_ByteArray &smil)
{
	inclSMIL = true;
	isReferenced = false;
	SMIL = smil;
}

const IPMP_ByteArray & UserQueryText::getSMIL() const
{
	return SMIL;
}

bool UserQueryText::hasSMIL() const
{
	return (inclSMIL && (! isReferenced));
}

bool UserQueryText::hasSMIL_URL() const
{
	return (inclSMIL && isReferenced);
}


//////////////////////////////////////////////////////////////////////////////
// 
// DTArray
// 

DTArray::DTArray()
{
	id = 0;
}

DTArray::~DTArray()
{
	clear();
}

DTArray::DTArray(const DTArray & dt):
	displayText(dt.displayText)
{
	id = dt.id;
}

void DTArray::clear()
{
	id = 0;
	displayText.clear();
}

int DTArray::toBuffer(byte * buf)
{
	setWriteBuffer(buf);
	put(id, 16);						// id
	putObject(displayText);			// display text
	return getWriteCounter();
}

int DTArray::fromBuffer(const byte * buf)
{
	clear();
	setReadBuffer(buf);
	id = get(16);						// id
	getObject(displayText);			// display text
	return getReadCounter();
}


bool DTArray::checkSyntax() const
{
	return true; // not implemented yet
}

unsigned int DTArray::getSize() const
{
	unsigned int size = 2;	// id
	size += displayText.getSize();
	return size;
}

void DTArray::setId(word32 id)
{
	this->id = id;
}

void DTArray::clearId()
{
	id = 0;
}

word32 DTArray::getId() const
{
	return id;
}

void DTArray::setDisplayText(byte * displayText, word32 displayTextSize)
{
	assert(displayTextSize == 0 || displayText != 0);
	this->displayText.setData(displayText, displayTextSize);
}

void DTArray::clearDisplayText()
{
	displayText.clear();
}

byte * DTArray::getDisplayText() const
{
	return displayText.getData();
}

word32 DTArray::getDisplayTextSize() const
{
	return displayText.getDataSize();
}

//////////////////////////////////////////////////////////////////////////////
// 
// QTArray
// 

QTArray::QTArray()
{
	id = 0;
	subId = 0;
	isHidden = false;
}

QTArray::~QTArray()
{
	clear();
}

QTArray::QTArray(const QTArray & qt):
	promptText(qt.promptText)

{
	id = qt.id;
	subId = qt.subId;
	isHidden = qt.isHidden;
}

void QTArray::clear()
{
	id = 0;
	subId = 0;
	isHidden = false;
}

int QTArray::toBuffer(byte * buf)
{
	setWriteBuffer(buf);
	put(id, 16);							// id
	put(subId, 16);							// subId
	put(isHidden, 1);						// isHidden
	put(RESERVED, 7);						// reserved
	putObject(promptText);					// prompt text
	return getWriteCounter();
}

int QTArray::fromBuffer(const byte * buf)
{
	promptText.clear();
	setReadBuffer(buf);
	id = get(16);							// id
	subId = get(16);						// subId
	isHidden = (get(1) == 1);				// isHidden
	skipReading(7);							// reserved
	getObject(promptText);					// promptText
	return getReadCounter();
}

bool QTArray::checkSyntax()  const
{
	return true; // not implemented yet
}

unsigned int QTArray::getSize() const 
{
	unsigned int size = 0;
	size += 2;	// id
	size += 2;	// subId
	size += 1;	// control bits
	size += promptText.getSize();	// prompt text
	return size;
}

void QTArray::setId(word32 id)
{
	this->id = id;
}

void QTArray::clearId()
{
	id = 0;
}

word32 QTArray::getId() const
{
	return id;
}

void QTArray::setSubId(word32 subId)
{
	this->subId = subId;
}

void QTArray::clearSubId()
{
	subId = 0;
}

word32 QTArray::getSubId() const
{
	return subId;
}

void QTArray::setPromptText(byte * promptText, word32 promptTextSize)
{
	assert(promptTextSize == 0 || promptText != 0);
	this->promptText.setData(promptText, promptTextSize);
}

void QTArray::clearPromptText()
{
	promptText.clear();
}

byte * QTArray::getPromptText() const
{
	return promptText.getData();
}

word32 QTArray::getPromptTextSize() const
{
	return promptText.getDataSize();
}

void QTArray::setIsHidden()
{
	this->isHidden = isHidden;
}

void QTArray::clearIsHidden()
{
	isHidden = 0;
}

word32 QTArray::getIsHidden() const
{
	return isHidden;
}

//////////////////////////////////////////////////////////////////////////////
// 
// OptionArray
// 

OptionArray::OptionArray()
{
	isExclusive = false;
	id = 0;
	subId = 0;
}

OptionArray::~OptionArray()
{
	clear();
}

OptionArray::OptionArray(const OptionArray & qt):
	promptText(qt.promptText)
{
	isExclusive = qt.isExclusive;
	id = qt.id;
	subId = qt.subId;
}

void OptionArray::clear()
{
	isExclusive = false;
	id = 0;
	subId = 0;
	promptText.clear();
}

int OptionArray::toBuffer(byte * buf)
{
	setWriteBuffer(buf);
	put(id, 16);				// id
	put(subId, 16);				// subId	
	put(isExclusive, 1);		// isExclusive
	put(RESERVED, 7);			// reserved
	putObject(promptText);		// promptText
	return getWriteCounter();
}

int OptionArray::fromBuffer(const byte * buf)
{
	clear();
	setReadBuffer(buf);
	id = get(16);				// id
	subId = get(16);			// subId	
	isExclusive = (get(1) == 1);	// isExclusive
	skipReading(7);				// reserved
	getObject(promptText);		// promptText
	return getReadCounter();
}

bool OptionArray::checkSyntax() const
{
	return true; // not implemented yet
}

unsigned int OptionArray::getSize() const 
{
	unsigned int size = 0;
	size += 2;						// id
	size += 2;						// subId
	size += 1;						// control bits
	size += promptText.getSize();	// prompt text
	return size;
}

void OptionArray::setIsExclusive()
{
	isExclusive = true;
}

void OptionArray::clearIsExclusive()
{
	isExclusive = false;
}

bool OptionArray::getIsExclusive() const
{
	return isExclusive;
}

void OptionArray::setId(word32 id)
{
	this->id = id;
}

void OptionArray::clearId()
{
	id = 0;
}

word32 OptionArray::getId() const
{
	return id;
}

void OptionArray::setSubId(word32 subId)
{
	this->subId = subId;
}

void OptionArray::clearSubId()
{
	subId = 0;
}

word32 OptionArray::getSubId() const
{
	return subId;
}

void OptionArray::setPromptText(byte * promptText, word32 promptTextSize)
{
	assert(promptTextSize == 0 || promptText != 0);
	this->promptText.setData(promptText, promptTextSize);
}

void OptionArray::clearPromptText()
{
	promptText.clear();
}

byte * OptionArray::getPromptText() const
{
	return promptText.getData();
}

word32 OptionArray::getPromptTextSize() const
{
	return promptText.getDataSize();
}


