
//	This software module was originally developed by ADETTI and modified by 
//  Massimo Balestri (Telecom Italia Lab) in the course
//	of development of the MPEG-4 Systems (ISO/IEC 14496-1) standard. 
//	This software module is an implementation of a part of one or more MPEG-4 
//	Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 
//	14496-1) standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	ADETTI retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2002.


// IPMP_UserQuery.h

#ifndef	IPMP_USERQUERY_H
#define	IPMP_USERQUERY_H

#include "IPMP_Data_BaseClass.h"
#include "IPMP_ByteArray.h"
#include "Container.h"
#include "namespace.h"

BEGIN_MOSES_NAMESPACE

class UserQueryText;
class DTArray;
class QTArray;
class OptionArray;

/** @class IPMP_UserQuery
 *  Used to query the user for information.
 *
 *  @author ADETTI
 *  @date 2002
 */
class IPMP_UserQuery : public IPMP_Data_BaseClass 
{
private:
	bool inclDisplayTitle;
	bool inclDisplayText;
	bool needReplyText;
	bool inclOptionSelect;
	bool inclSMIL;

	virtual void clear();						// clear all internal data

public:	
	Container<UserQueryText> userQueryText;			///< list of alternate text for each language

	IPMP_UserQuery();									///< constructor					
	virtual ~IPMP_UserQuery();							///< destructor
	virtual int toBuffer(byte * buf);					// reimplemented
	virtual int fromBuffer(const byte * buf);			// reimplemented
	virtual bool checkSyntax() const;					// reimplemented
	virtual unsigned int sizeOfInstance() const;		// reimplemented


	/**
	 * Set inclDisplayTitle bit.
	 * This implies that UserQueryText entries must include a title.
	 */
	void setInclDisplayTitle();
	
	/**
	 * Clear inclDisplayTitle bit.
	 * This implies that UserQueryText entries must not include a title.
	 */
	void clearInclDisplayTitle();
	
	/**
	 * Get inclDisplayTitle bit.
	 */
	bool getInclDisplayTitle() const;
	
	/**
	 * Set inclDisplayText bit.
	 * This implies that UserQueryText entries must include text to display.
	 */
	void setInclDisplayText();
	
	/**
	 * Clear inclDisplayText bit.
	 * This implies that UserQueryText entries must not include text to display.
	 */
	void clearInclDisplayText();
	
	/**
	 * Get inclDisplayText bit.
	 */
	bool getInclDisplayText() const;
	
	/**
	 * Set needReplyText bit.
	 * This implies that UserQueryText entries must include prompt text.
	 */
	void setNeedReplyText();
	
	/**
	 * Clear needReplyText bit.
	 * This implies that UserQueryText entries must not include prompt text.
	 */
	void clearNeedReplyText();
	
	/**
	 * Get needReplyText bit.
	 */
	bool getNeedReplyText() const;
	
	/**
	 * Set inclOptionSelect bit.
	 * This implies that UserQueryText entries must include a set of options.
	 */
	void setInclOptionSelect();
	
	/**
	 * Clear inclOptionSelect bit.
	 * This implies that UserQueryText entries must not include a set of options.
	 */
	void clearInclOptionSelect();
	
	/**
	 * Get inclOptionSelect bit.
	 */
	bool getInclOptionSelect() const;
	
	/**
	 * Set inclSMIL bit.
	 * This implies that UserQueryText entries must include a SMIL.
	 */
	void setInclSMIL();
	
	/**
	 * Clear inclSMIL bit.
	 * This implies that UserQueryText entries must not include a SMIL.
	 */
	void clearInclSMIL();
	
	/**
	 * Get inclSMIL bit.
	 * This implies that UserQueryText entries must include a SMIL.
	 */
	bool getInclSMIL() const;
};

/** 
 *  @class UserQueryText
 *  One alternative query to be presented to the user.
 *
 *  @author ADETTI
 *  @date 2002
 */
class UserQueryText : public BitField
{
private:
	bool inclDisplayTitle;
	bool inclDisplayText;
	bool needReplyText;
	bool inclOptionSelect;
	bool inclSMIL;
	bool isReferenced;

	IPMP_ByteArray SMIL_URL;			// The SMIL URL to display  - this must be private to let the class know the correct value of the isReferenced flag!
	IPMP_ByteArray SMIL;				// The SMIL file to display - this must be private to let the class know the correct value of the isReferenced flag!

	word32 languageCode;			// always present

	UserQueryText();				// defaul constructor cannot be used - this class need to know the flags in advance!
	
public:
	IPMP_ByteArray titleText;			///< The Title text to display
	Container<DTArray> displayText;		///< The text to display
	Container<QTArray> promptText;		///< The prompt text to display
	Container<OptionArray> option;		///< The option to display

	/**
	 * Constructor.
	 * @param inclDisplayTitle	indicates if UserQueryText entries must include a title.
	 * @param inclDisplayText indicates if UserQueryText entries must include text to display.
	 * @param needReplyText indicates if UserQueryText entries must include prompt text.
	 * @param inclOptionSelect indicates if UserQueryText entries must include a set of options.
	 * @param inclSMIL indicates if UserQueryText entries must include a SMIL.
	 */
	UserQueryText(bool inclDisplayTitle,
		bool inclDisplayText,
		bool needReplyText,
		bool inclOptionSelect,
		bool inclSMIL);

	/**
	 * Destructor.
	 */
	~UserQueryText();

	/**
	 * Copy constructor.
	 */

	virtual int toBuffer(byte * buf);				// reimplemented methods
	virtual int fromBuffer(const byte * buf);
	virtual bool checkSyntax() const;
	virtual unsigned int getSize() const;

	/**
	 * Set the language code for this UserQueryText instance.
	 * Language code is a 3 character value according to ISO 639-2:1998.
	 * @param languageCode 3 character language code
	 */
	void setLanguageCode(word32 languageCode);
	
	/**
	 * Clear the language code for this UserQueryText instance.
	 */
	void clearLanguageCode();
	
	/**
	 * Get the language code for this UserQueryText instance.
	 * Language code is a 3 character value according to ISO 639-2:1998.
	 * @return 3 character language code
	 */
	word32 getLanguageCode() const;	
	
	/**
	 * Set the fully qualified location of SMIL file to be displayed.
	 * @param smilUrl URL of the SMIL file to be displayed
	 */
	void setSMIL_URL(const IPMP_ByteArray & smilUrl);
	
	/**
	 * Get the fully qualified location of SMIL file to be displayed.
	 * @return URL of the SMIL file to be displayed
	 */
	const IPMP_ByteArray & getSMIL_URL() const;

	/**
	 * Set the SMIL file to be displayed.
	 * @param smil file to be displayed
	 */
	void setSMIL(const IPMP_ByteArray & smil);
	
	/**
	 * Get the SMIL file to be displayed.
	 * @return file to be displayed
	 */
	const IPMP_ByteArray & getSMIL() const;

	/**
	 * Check if SMIL information is available.
	 * @return <i>true</i> if available
	 */
	bool hasSMIL() const;

	/**
	 * Check if SMIL_URL information is available.
	 * @return <i>true</i> if available
	 */
	bool hasSMIL_URL() const;


};

/** @class DTArray
 *  Carries text to be displayed to the user.
 *
 *  @author ADETTI
 *  @date 2002
 */
class DTArray : public BitField
{
private:
	word32 id;
	IPMP_ByteArray displayText;

	// clear all internal data
	virtual void clear();

public:
	/**
	 * Constructor.
	 */
	DTArray();
	/**
	 * Destructor.
	 */
	~DTArray();

	/**
	 * Copy constructor.
	 */
	DTArray(const DTArray &);

	/**
	 * Method for writing the message using the data of the class.
	 * @param buf storage location for the byte sequence. 
	 */
	virtual int toBuffer(byte * buf);		
	/**
	 * Method for reading the contents of the message.
	 * @param buf storage location of the byte sequence. 
	 */
	virtual int fromBuffer(const byte * buf);

	/**
	 * Checks that the syntax is correct. This can be used before sending
	 * the message through an interface.
	 * @return <i>true</i> if successful
	 */
	virtual bool checkSyntax() const;

	/**
	 * Get the size in bytes of the binary message generated by the
	 * toBuffer method.
	 * @return the size (in bytes) of this class.
	 */
	virtual unsigned int getSize() const ;

	/**
	 * Set the 16 bit serial number to be associated with the contained
	 * data or to associate contained data with other data.
	 * @param id a 16 bit identifier
	 */
	void setId(word32 id);
	/**
	 * Clear the 16 bit identifier.
	 */
	void clearId();
	/**
	 * Get the 16 bit serial number associated with the contained
	 * data or to associating contained data with other data.
	 * @return the 16 bit identifier.
	 */
	word32 getId() const;

	/**
	 * Set the text to be displayed to the user.
	 * @param displayText text to be displayed to the user
	 * @param displayTextSize length of the text string
	 */
	void setDisplayText(byte * displayText, word32 displayTextSize);
	/**
	 * Clear the text to be displayed to the user.
	 */
	void clearDisplayText();
	/**
	 * Get the text to be displayed to the user.
	 * @return the text to be displayed to the user
	 */
	byte * getDisplayText() const;
	/**
	 * Get the size of the text to be displayed to the user.
	 * @return the size of the text to be displayed to the user
	 */
	word32 getDisplayTextSize() const;
};

/** @class QTArray
 *  Carries a prompt to be displayed to the user.
 *
 *  @author ADETTI
 *  @date 2002
 */
class QTArray : public BitField
{
private:
	word32 id;
	word32 subId;
	bool   isHidden;
	IPMP_ByteArray promptText;

	// clear all internal data
	virtual void clear();

public:
	/**
	 * Constructor.
	 */
	QTArray();
	/**
	 * Destructor.
	 */
	~QTArray();

	/**
	 * Copy constructor.
	 */
	QTArray(const QTArray &);

	/**
	 * Method for writing the message using the data of the class.
	 * @param buf storage location for the byte sequence. 
	 */
	virtual int toBuffer(byte * buf);		
	/**
	 * Method for reading the contents of the message.
	 * @param buf storage location of the byte sequence. 
	 */
	virtual int fromBuffer(const byte * buf);

	/**
	 * Checks that the syntax is correct. This can be used before sending
	 * the message through an interface.
	 * @return <i>true</i> if successful
	 */
	virtual bool checkSyntax() const;

	/**
	 * Get the size in bytes of the binary message generated by the
	 * toBuffer method.
	 * @return the size (in bytes) of this class.
	 */
	virtual unsigned int getSize() const ;

	/**
	 * Set the 16 bit serial number to be associated with the contained
	 * data or to associate contained data with other data.
	 * @param id a 16 bit identifier
	 */
	void setId(word32 id);
	/**
	 * Clear (i.e. set to 0) the 16 bit identifier.
	 */
	void clearId();
	/**
	 * Get the 16 bit serial number associated with the contained
	 * data or to associating contained data with other data.
	 * @return the 16 bit identifier.
	 */
	word32 getId() const;

	/**
	 * Set the 16 bit serial number to associate items within an
	 * id group.
	 * @param subId a 16 bit sub-identifier
	 */
	void setSubId(word32 subId);
	/**
	 * Clear (i.e. set to 0) the 16 bit sub-identifier.
	 */
	void clearSubId();
	/**
	 * Get the 16 bit serial number that associates items within
	 * the id group.
	 * @return the 16 bit sub-identifier
	 */
	word32 getSubId() const;

	/**
	 * Set the prompt to be displayed to the user.
	 * @param promptText prompt to be displayed to the user
	 * @param promptTextSize length of the prompt string
	 */
	void setPromptText(byte * promptText, word32 promptTextSize);
	/**
	 * Clear the prompt to be displayed to the user.
	 */
	void clearPromptText();
	/**
	 * Get the prompt to be displayed to the user.
	 * @return the prompt to be displayed to the user
	 */
	byte * getPromptText() const;
	/**
	 * Get the size of the prompt to be displayed to the user.
	 * @return the size of the prompt to be displayed to the user
	 */
	word32 getPromptTextSize() const;

	/**
	 * Set the isHidden bit. When this bit is set, the answer typed
	 * by the user must not be displayed in clear. (Useful when
	 * asking for passwords.)
	 */
	void setIsHidden();
	/**
	 * Clear the isHidden bit. The answer typed by the user may
	 * be displayed in clear.
	 */
	void clearIsHidden();
	/**
	 * Get the value of the isHidden bit. If <i>true</i>, then the
	 * answer typed by the user must not be displayed in clear.
	 */
	word32 getIsHidden() const;
};

/** @class OptionArray
 *  Carries an option that may be selected by the user.
 *
 *  @author ADETTI
 *  @date 2002
 */
class OptionArray : public BitField
{
private:
	bool   isExclusive;
	word32 id;
	word32 subId;
	IPMP_ByteArray promptText;

	// clear all internal data
	virtual void clear();

public:
	/**
	 * Constructor.
	 */
	OptionArray();
	/**
	 * Destructor.
	 */
	~OptionArray();

	/**
	 * Copy constructor.
	 */
	OptionArray(const OptionArray &);

	/**
	 * Method for writing the message using the data of the class.
	 * @param buf storage location for the byte sequence. 
	 */
	virtual int toBuffer(byte * buf);		
	/**
	 * Method for reading the contents of the message.
	 * @param buf storage location of the byte sequence. 
	 */
	virtual int fromBuffer(const byte * buf);

	/**
	 * Checks that the syntax is correct. This can be used before sending
	 * the message through an interface.
	 * @return <i>true</i> if successful
	 */
	virtual bool checkSyntax() const;

	/**
	 * Get the size in bytes of the binary message generated by the
	 * toBuffer method.
	 * @return the size (in bytes) of this class.
	 */
	virtual unsigned int getSize() const ;

	/**
	 * Set the isExclusive bit. This bit is set to indicate that this
	 * is a mutually exclusive condition of the option selector.
	 */
	void setIsExclusive();
	/**
	 * Clear the isExclusive bit. 
	 */
	void clearIsExclusive();
	/**
	 * Get the value of the isExclusive bit. This bit is set to
	 * indicate that this is a mutually exclusive condition of
	 * the option selector.
	 * @return <i>true</i> if the option is mutually exclusive
	 */
	bool getIsExclusive() const;

	/**
	 * Set the 16 bit serial number to be associated with the contained
	 * data or to associate contained data with other data.
	 * @param id a 16 bit identifier
	 */
	void setId(word32 id);
	/**
	 * Clear (i.e. set to 0) the 16 bit identifier.
	 */
	void clearId();
	/**
	 * Get the 16 bit serial number associated with the contained
	 * data or to associating contained data with other data.
	 * @return the 16 bit identifier.
	 */
	word32 getId() const;

	/**
	 * Set the 16 bit serial number to associate items within an
	 * id group.
	 * @param subId a 16 bit sub-identifier
	 */
	void setSubId(word32 subId);
	/**
	 * Clear (i.e. set to 0) the 16 bit sub-identifier.
	 */
	void clearSubId();
	/**
	 * Get the 16 bit serial number that associates items within
	 * the id group.
	 * @return the 16 bit sub-identifier
	 */
	word32 getSubId() const;

	/**
	 * Set the prompt to be displayed to the user.
	 * @param promptText prompt to be displayed to the user
	 * @param promptTextSize length of the prompt string
	 */
	void setPromptText(byte * promptText, word32 promptTextSize);
	/**
	 * Clear the prompt to be displayed to the user.
	 */
	void clearPromptText();
	/**
	 * Get the prompt to be displayed to the user.
	 * @return the prompt to be displayed to the user
	 */
	byte * getPromptText() const;
	/**
	 * Get the size of the prompt to be displayed to the user.
	 * @return the size of the prompt to be displayed to the user
	 */
	word32 getPromptTextSize() const;
};

END_MOSES_NAMESPACE

#endif

