
//	This software module was originally developed by ADETTI in the course of
//	development of the MPEG-4 Systems (ISO/IEC 14496-1) standard. 
//	This software module is an implementation of a part of one or more MPEG-4 
//	Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 
//	14496-1) standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	ADETTI retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2002.


// IPMP_UserQueryResponse.cpp

#include "IPMP_UserQueryResponse.h"

#include <string.h>
#include <assert.h>

USING_MOSES_NAMESPACE

//////////////////////////////////////////////////////////////////////////////
// 
// IPMP_UserQueryResponse
// 

IPMP_UserQueryResponse::IPMP_UserQueryResponse()
	:	IPMP_Data_BaseClass(IPMP_UserQueryResponse_tag)
{
	inclReplyText = false;
	inclOptionSelect = false;
	languageCode = 0;
	numOfOptions = 0;
	pOptionResults = 0;
}

IPMP_UserQueryResponse::~IPMP_UserQueryResponse()
{
	clear();
}

void IPMP_UserQueryResponse::clear()
{
	inclReplyText = false;
	inclOptionSelect = false;

	languageCode = 0;

	replyText.clear();
	clearOptionResults();
}

int IPMP_UserQueryResponse::toBuffer(byte * buf)
{
	IPMP_Data_BaseClass::toBuffer(buf);

	put(inclReplyText, 1);			// control bits
	put(inclOptionSelect, 1);
	put(0, 6);						// reserved
	put(languageCode, 24);

	if (inclReplyText) {
		unsigned int numOfReplyText = replyText.getObjectNum();
		put(numOfReplyText, 8);
		for (word32 i = 0; i < numOfReplyText; ++i) {
			skipWriting(replyText.getObject(i).toBuffer(getWritePtr()));
		}
	}

	if (inclOptionSelect) {
		assert(numOfOptions == 0 || pOptionResults != 0);
		// numOfOptions
		put(numOfOptions, 16);
		// option results
		for (word32 i = 0; i < numOfOptions; ++i) {
			put(pOptionResults[i] ? 1 : 0, 1);
		}
	}

	return (getWriteCounter());
}

int IPMP_UserQueryResponse::fromBuffer(const byte * buf)
{
	clear();

	IPMP_Data_BaseClass::fromBuffer(buf);
	
	inclReplyText = (get(1) != 0);	// control bits
	inclOptionSelect = (get(1) != 0);
	get(6);							// reserved	
	languageCode = get(24);			// language code

	if (inclReplyText) {
		unsigned int numOfReplyText = get(8);		// numOfReplyText
		for (word32 i = 0; i < numOfReplyText; ++i) {
			QTArray rta;
			skipReading( rta.fromBuffer(getReadPtr()) );
			replyText.addObject(rta);
		}
	}

	if (inclOptionSelect) {
		// numOfOptions
		numOfOptions = get(16);
		if (numOfOptions != 0) {
			// option results
			pOptionResults = new bool[numOfOptions];
			assert(pOptionResults != 0);
			for (word32 i = 0; i < numOfOptions; ++i) {
				pOptionResults[i] = (get(1) != 0);
			}
		}
	}

	return (getReadCounter());
}


// compute size, in BYTES!
unsigned int IPMP_UserQueryResponse::sizeOfInstance() const
{
	unsigned int size = IPMP_Data_BaseClass::sizeOfInstance();

	// control bits
	size += 1;

	// language code
	size += 3;

	if (inclReplyText) {
		size += 1;		// numOfReplyText
		for (word32 i = 0; i < replyText.getObjectNum(); ++i) {
			size += replyText.getObject(i).getSize();
		}
	}

	if (inclOptionSelect) {
		assert(numOfOptions == 0 || pOptionResults != 0);
		// numOfOptions
		size += 2;
		// option results
		size += ((numOfOptions + 7) / 8);
	}

	return size;
}

void IPMP_UserQueryResponse::setLanguageCode(word32 languageCode)
{
	this->languageCode = languageCode;
}

void IPMP_UserQueryResponse::clearLanguageCode()
{
	languageCode = 0;
}

word32 IPMP_UserQueryResponse::getLanguageCode() const
{
	return languageCode;
}


bool IPMP_UserQueryResponse::hasReplyText() const
{
	return inclReplyText;
}


void IPMP_UserQueryResponse::setOptionResults(bool * pOptionResults, word32 numOfOptions)
{
	assert(numOfOptions == 0 || pOptionResults != 0);
	clearOptionResults();

	this->numOfOptions = numOfOptions;
	if (numOfOptions) {
		this->pOptionResults = new bool[numOfOptions];
		memcpy(this->pOptionResults, pOptionResults, numOfOptions);
	}
}

void IPMP_UserQueryResponse::clearOptionResults()
{
	delete [] pOptionResults;
	pOptionResults = 0;
	numOfOptions = 0;
	inclOptionSelect = false;
}

bool IPMP_UserQueryResponse::hasOptionResults() const
{
	return inclOptionSelect;
}

bool * IPMP_UserQueryResponse::getOptionResults() const
{
	return pOptionResults;
}

word32 IPMP_UserQueryResponse::getNumOfOptions() const
{
	return numOfOptions;
}

