
//	This software module was originally developed by ADETTI in the course of
//	development of the MPEG-4 Systems (ISO/IEC 14496-1) standard. 
//	This software module is an implementation of a part of one or more MPEG-4 
//	Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 
//	14496-1) standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	ADETTI retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2002.


// IPMP_UserQueryResponse.h

#ifndef	IPMP_USERQUERYRESPONSE_H
#define	IPMP_USERQUERYRESPONSE_H

#include "IPMP_Data_BaseClass.h"
#include "IPMP_UserQuery.h"
#include "Container.h"
#include "namespace.h"

BEGIN_MOSES_NAMESPACE


/** @class IPMP_UserQueryResponse
 *  Carries an user's answer to an IPMP_UserQuery.
 *
 *  @author ADETTI
 *  @date 2002
 */
class IPMP_UserQueryResponse: public IPMP_Data_BaseClass 
{
private:
	bool inclReplyText;
	bool inclOptionSelect;
	word32 languageCode;
	word32 numOfOptions;
	bool * pOptionResults;	// if inclOptionSelect

	virtual void clear();	// clear all internal data

public:
	Container<QTArray> replyText;		///< list of reply text

	/**
	 * Constructor.
	 */
	IPMP_UserQueryResponse();
	
	/**
	 * Destructor.
	 */
	virtual ~IPMP_UserQueryResponse();
	
	virtual int toBuffer(byte * buf);		
	virtual int fromBuffer(const byte * buf);
	virtual unsigned int sizeOfInstance() const;

	/**
	 * Set the language code for this IPMP_UserQueryResponse instance.
	 * Language code is a 3 character value according to ISO 639-2:1998.
	 * @param languageCode 3 character language code
	 */
	void setLanguageCode(word32 languageCode);
	/**
	 * Clear the language code for this IPMP_UserQueryResponse instance.
	 */
	void clearLanguageCode();
	/**
	 * Get the language code for this IPMP_UserQueryResponse instance.
	 * Language code is a 3 character value according to ISO 639-2:1998.
	 * @return 3 character language code
	 */
	word32 getLanguageCode() const;

	/**
	 * Check if reply text information is available.
	 * @return <i>true</i> if available
	 */
	bool hasReplyText() const;

	/**
	 * Set the array of option results.
	 * @param pOptionResults array with option results
	 * @param numOfOptions length of the pOptionResult array
	 */
	void setOptionResults(bool * pOptionResults, word32 numOfOptions);
	
	/**
	 * Clear the array of option results.
	 */
	void clearOptionResults();
	
	/**
	 * Check if option results information is available.
	 * @return <i>true</i> if available
	 */
	bool hasOptionResults() const;
	
	/**
	 * Get the array of option results.
	 * First call #hasOptionResults to know if this information is available.
	 * @return the array of option results
	 */
	bool * getOptionResults() const;
	
	/**
	 * Get the length of the array of option results.
	 * First call #hasOptionResults to know if this information is available.
	 * @return the length of the array of option results
	 */
	word32 getNumOfOptions() const;
};


END_MOSES_NAMESPACE

#endif

