
//	This software module was originally developed by Claudio Alberti (EPFL)	
//	in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1) standard. 
//	This software module is an implementation of a part of one or more MPEG-4 
//	Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 
//	14496-1) standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	EPFL retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2002.


// IPMP_VideoWatermarkingInit.cpp

#include "IPMP_VideoWatermarkingInit.h"
#include <stdlib.h>
#include <memory.h>
#include <assert.h>

USING_MOSES_NAMESPACE

IPMP_VideoWatermarkingInit::IPMP_VideoWatermarkingInit():
	IPMP_Data_BaseClass(IPMP_VideoWatermarkingInit_tag)			// set the correct tag
{

	// init all variables to default vaules

	inputFormat		= YUV;
	requiredOp		= EXTRACT_WM;
	b_hasOpaqueData	= false;
	frame_horizontal_size = 0;
	frame_vertical_size	  = 0;
	chroma_format	= 0;
	wmPayloadLen	= 0;
	wmPayload		= NULL;
	wmRecipientId	= 0;		// 0 = the terminal
	opaqueDataSize	= 0;
	opaqueData		= NULL;
}	

IPMP_VideoWatermarkingInit::IPMP_VideoWatermarkingInit(const IPMP_VideoWatermarkingInit &msg)
:	IPMP_Data_BaseClass(msg)
{
	inputFormat		= msg.inputFormat;
	requiredOp		= msg.requiredOp;
	b_hasOpaqueData	= msg.b_hasOpaqueData;
	frame_horizontal_size = msg.frame_horizontal_size;
	frame_vertical_size	  = msg.frame_vertical_size;
	chroma_format	= msg.chroma_format;

	wmRecipientId	= msg.wmRecipientId;
	wmPayloadLen	= 0;
	wmPayload		= NULL;

	if (msg.wmPayloadLen > 0)
		setwmPayload(msg.wmPayload, msg.wmPayloadLen);

	opaqueDataSize	= 0;
	opaqueData		= NULL;	

	if (msg.opaqueDataSize > 0)
		setOpaqueData(msg.opaqueData, msg.opaqueDataSize);
}

IPMP_VideoWatermarkingInit::~IPMP_VideoWatermarkingInit() 
{

	if(wmPayload)
		delete[] wmPayload;

	if(opaqueData)
		delete[] opaqueData;

}


word32 IPMP_VideoWatermarkingInit::getHorizontalSize() const
{
	if (inputFormat == YUV)
		return frame_horizontal_size;
	else
		return 0;
}

word32 IPMP_VideoWatermarkingInit::getVerticalSize() const
{
	if (inputFormat == YUV)
		return frame_vertical_size;
	else
		return 0;
}

enum IPMP_VideoWatermarkingInit::ChromaFormatTable IPMP_VideoWatermarkingInit::getChromaFormat() const
{
	if (inputFormat == YUV)
		return (ChromaFormatTable) chroma_format;
	else
		return FORBIDDEN;
}

IPMP_VideoWatermarkingInit::RequiredOpTable IPMP_VideoWatermarkingInit::getRequiredOp() const
{
	return (RequiredOpTable) requiredOp;
}

word32 IPMP_VideoWatermarkingInit::getwmPayloadLen() const
{
	if ((requiredOp == INSERT_WM) || (requiredOp == REMARK_WM))
		return wmPayloadLen;
	else 
		return 0;
}


byte* IPMP_VideoWatermarkingInit::getwmPayload() const
{
	if ((requiredOp == INSERT_WM) || (requiredOp == REMARK_WM))
		return wmPayload;
	else 
		return 0;
}


bool IPMP_VideoWatermarkingInit::setwmPayload(const byte* buf, word32 len)
{
	wmPayloadLen = len;					// set the new length

	if (wmPayload != NULL)				
	{
		delete[] wmPayload;				// free the memory allocated by the old payload
		wmPayload = NULL;
	}

	if(wmPayloadLen > 0)
	{
		wmPayload = new byte [wmPayloadLen];		// allocate memory
		if (wmPayload != NULL)
			memcpy(wmPayload, buf, wmPayloadLen);	// set the new payload
		else 
			return false; 
	}
	return true;
}

word32 IPMP_VideoWatermarkingInit::getwmRecipientId() const
{
	if ((requiredOp == EXTRACT_WM) || (requiredOp == DETECT_COMPRESSION))
		return wmRecipientId;
	else 
		return 0;
}

bool IPMP_VideoWatermarkingInit::hasOpaqueData() const
{
	return hasOpaqueData;
}


word32 IPMP_VideoWatermarkingInit::getOpaqueDataSize() const
{
	if (b_hasOpaqueData)
		return opaqueDataSize;
	else 
		return 0;
}


bool   IPMP_VideoWatermarkingInit::setOpaqueData(const byte* buf, word32 len)
{
	b_hasOpaqueData = true;					// now there is some opaque data!
	opaqueDataSize = len;					// set the new length

	if (opaqueData != NULL)				
	{
		delete[] opaqueData;				// free the memory allocated by the old payload
		opaqueData = NULL;
	}

	if(opaqueDataSize > 0)
	{
		opaqueData = new byte [opaqueDataSize];			// allocate memory
		if (opaqueData != NULL)
			memcpy(opaqueData, buf, opaqueDataSize);	// set the new payload
		else 
			return false; 
	}
	else
		b_hasOpaqueData = false;				// if (size == 0) there is no data!

	return true;
}

byte* IPMP_VideoWatermarkingInit::getOpaqueData() const
{
	if (b_hasOpaqueData)
		return opaqueData;
	else 
		return 0;
}

int IPMP_VideoWatermarkingInit::fromBuffer(const byte * buf)
{
	int nbits = IPMP_Data_BaseClass::fromBuffer(buf);
	assert((nbits % 8) == 0);
	const byte * wmbuf = buf + (nbits/8);			// place pointer to the start of the wm_init 

	inputFormat = get(wmbuf, 0, 8);					// bit address: 0
	requiredOp = get(wmbuf, 8, 4);					// bit address: 8
	b_hasOpaqueData = (1 == get(wmbuf, 12, 1));		// bit address: 12
	long bitindex = 16;								// now, use the bit index.

	if (inputFormat == YUV)
	{
		frame_horizontal_size = get(wmbuf, bitindex, 16);		// get horiz. size
		bitindex +=16;
		frame_vertical_size = get(wmbuf, bitindex, 16);			// get the vertical size
		bitindex +=16;
		chroma_format = get(wmbuf, bitindex, 8);				// get the chrominance format
		bitindex +=8;
	}

	if ((requiredOp == INSERT_WM) || (requiredOp == REMARK_WM))
	{
		word32 len = get(wmbuf, bitindex, 16);		// get the wm payload length
		bitindex += 16;

		setwmPayload(wmbuf + (bitindex/8), len);	// set the new payload 
		bitindex += (len * 8);
	}

	if((requiredOp == EXTRACT_WM) || (requiredOp == DETECT_COMPRESSION))
	{
		wmRecipientId = get(wmbuf, bitindex, 16);
		bitindex += 16;
	}

	if(b_hasOpaqueData)
	{
		word32 size = get(wmbuf, bitindex, 16);
		bitindex += 16;

		setOpaqueData(wmbuf + (bitindex/8), size);
		bitindex += (size * 8);
	}

	return (nbits + bitindex);
}


int IPMP_VideoWatermarkingInit::toBuffer(byte * buf)
{
	int nbits = IPMP_Data_BaseClass::toBuffer(buf);
	assert((nbits % 8) == 0);
	byte * wmbuf = buf + (nbits/8);			// place pointer to the start of the wm_init 

	put(inputFormat, wmbuf, 0, 8); 
	put(requiredOp, wmbuf, 8, 4);
	put(b_hasOpaqueData, wmbuf, 12, 1);
	put(0, wmbuf, 13, 3);							// reserved bits 
	long bitindex = 16;								// now, use the bit index.

	if (inputFormat == YUV)
	{
		put(frame_horizontal_size, wmbuf, bitindex, 16);
		bitindex +=16;
		put(frame_vertical_size, wmbuf, bitindex, 16);
		bitindex +=16;
		put(chroma_format, wmbuf, bitindex, 8);	
		bitindex +=8;
	}

	if ((requiredOp == INSERT_WM) || (requiredOp == REMARK_WM))
	{
		put(wmPayloadLen,wmbuf, bitindex, 16);		// wm payload length
		bitindex += 16;
		memcpy(wmbuf + (bitindex/8), getwmPayload(), wmPayloadLen);
		bitindex += (wmPayloadLen * 8);
	}

	if((requiredOp == EXTRACT_WM) || (requiredOp == DETECT_COMPRESSION))
	{
		put(wmRecipientId, wmbuf, bitindex, 16);
		bitindex += 16;
	}

	if(b_hasOpaqueData)
	{
		put(opaqueDataSize, wmbuf, bitindex, 16);
		bitindex += 16;
		memcpy(wmbuf + (bitindex/8), getOpaqueData(), opaqueDataSize);
		bitindex += (opaqueDataSize * 8);
	}

	return (nbits + bitindex);
}

// compute size, in BYTES!
unsigned int IPMP_VideoWatermarkingInit::sizeOfInstance() const
{
	unsigned int size = IPMP_Data_BaseClass::sizeOfInstance();		// size of the base class 

	// now compute the size of this instance.
	
	size += 2;														// flags
	
	if (inputFormat == YUV)
		size += 5;													// PCM params
	
	if ((requiredOp == INSERT_WM) || (requiredOp == REMARK_WM))
		size += (2 + wmPayloadLen);

	if((requiredOp == EXTRACT_WM) || (requiredOp == DETECT_COMPRESSION))
		size += 2;

	if(b_hasOpaqueData)
		size += (2 + opaqueDataSize);

	return size;
}

bool IPMP_VideoWatermarkingInit::setInputFormat(enum InputFormatTable format, word32 horizontal, word32 vertical, word32 chroma)
{
	inputFormat = format;
	frame_horizontal_size = horizontal;
	frame_vertical_size = vertical;
	chroma_format = chroma;
	return true;
}

bool IPMP_VideoWatermarkingInit::insertWatermark(const byte *payload, word32 size)
{
	requiredOp = INSERT_WM;
	return setwmPayload(payload, size); 
}

bool IPMP_VideoWatermarkingInit::remarkWatermark(const byte *payload, word32 size)
{
	requiredOp = REMARK_WM;
	return setwmPayload(payload, size); 
}

bool IPMP_VideoWatermarkingInit::extractWatermark(word32 recipientId)
{
	requiredOp = EXTRACT_WM;
	wmRecipientId = recipientId;
	return true;
}

bool IPMP_VideoWatermarkingInit::detectCompression(word32 recipientId)
{
	requiredOp = DETECT_COMPRESSION;
	wmRecipientId = recipientId;
	return true;
}

