
//	This software module was originally developed by Massimo Balestri 
//	(Telecom Italia Lab S.p.A.) in the course of development 
//	of the MPEG-4 Systems (ISO/IEC 14496-1) standard. This software module 
//	is an implementation of a part of one or more MPEG-4 Systems (ISO/IEC 
//	14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1) 
//	standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	Telecom Italia Lab S.p.A retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2000. 


#include "Length.h"

USING_MOSES_NAMESPACE


//////////////////////////////////////////////////////////////////////
// class Length
//////////////////////////////////////////////////////////////////////
Length::Length()
{
	length = 0;
	isCorrect = true;
}

Length::~Length()
{
}

int Length::fromBuffer(const unsigned char *buf)
{
	word32 cbit;

	int counter = 0;
	length = 0;
	do 
	{
		cbit = get(buf, counter*8, 1);
		length = (length << 7) | get(buf, counter*8 + 1, 7);
		counter++;
	}
	while ( cbit && (counter <= 4));

	isCorrect = (cbit == 0);		// the last bit must be == 0
	
	return (counter*8);
}

int Length::toBuffer(unsigned char *buf)
{
	int loops;

	if (length > 0x1fffff)
		loops = 4;
	else if (length > 0x3fff)
		loops = 3;
	else if (length > 0x7f)
		loops = 2;
	else
		loops = 1;

	int bitcounter = 0;

	for (int i=loops-1; i>=0; i--)
	{
		if (i == 0)									// is this the last byte?
			put(0, buf, bitcounter, 1);				// the last control bit is always = zero.
		else
			put(1, buf, bitcounter, 1);				// all other control bits are = one.

		bitcounter++;
		put(length >> (7*i), buf, bitcounter, 7);		// write the 7 bits.
		bitcounter +=7;
	}

	return bitcounter;
}

// return the size of this class
unsigned int Length::getSize() const 
{
	if (length > 0x1fffff)
		return 4;
	else if (length > 0x3fff)
		return 3;
	else if (length > 0x7f)
		return 2;
	else
		return 1;
}
	
bool Length::checkSyntax() const
{
	return isCorrect;
}

