
//	This software module was originally developed by Claudio Alberti (EPFL)	
//  and modified by Massimo Balestri (Telecom Italia Lab)
//	in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1) standard. 
//	This software module is an implementation of a part of one or more MPEG-4 
//	Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 
//	14496-1) standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	EPFL retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2002.


// TrustSecurityMetadata.cpp

#include "TrustSecurityMetadata.h"
#include <memory.h>
#include <string.h>
#include <assert.h>

USING_MOSES_NAMESPACE

DateClass::DateClass()
{
	memset(Date, 0, 5);
}


DateClass::~DateClass()
{

}


void DateClass::setDate(const DateType &buf)
{
	memcpy(Date, buf, 5 );
}


const DateType & DateClass::getDate() const
{
	return Date;
}


int DateClass::fromBuffer(const byte *buf)
{
	memcpy(Date, buf, 5);
	return (5 * 8);
}


int DateClass::toBuffer(byte *buf)
{
	memcpy(buf, Date, 5);
	return (5 * 8);
}

unsigned int DateClass::getSize() const
{
	return 5;
}

bool DateClass::checkSyntax() const
{
	return true;
}



TrustSpecification::TrustSpecification()
:	hasCCBasedTrustMetadata(false),
	attackerProfile(0),
	trustedDuration(0)
{
	
}	



TrustSpecification::~TrustSpecification() 
{
	
}


bool TrustSpecification::checkSyntax() const
{
	return true;
}


const DateClass & TrustSpecification::getStartDate() const
{
	return startDate;
}


word32 TrustSpecification::getAttackerProfile() const
{	
	return attackerProfile ;
}


word32 TrustSpecification::getTrustedDuration() const
{
	return trustedDuration ;
}

void TrustSpecification::setTrustMetadata(const DateType & date, word32 ap, word32 td)
{
	startDate.setDate(date);
	attackerProfile = ap;
	trustedDuration = td;
	hasCCBasedTrustMetadata = false;
}


void TrustSpecification::setCCTrustMetadata(const byte *data, unsigned int size)
{
	hasCCBasedTrustMetadata = true;
	CCTrustMetadata.setData(data, size);
}


int TrustSpecification::fromBuffer(const byte * buf)
{
	setReadBuffer(buf);
	hasCCBasedTrustMetadata = (1 == get(1));	// 1 bit
	skipReading(7);								// 7 reserved bits
	if(!hasCCBasedTrustMetadata)
	{
		skipReading( startDate.fromBuffer( getReadPtr() ));		// start date
		attackerProfile = get(2);								// 2 bits
		skipReading(6);											// 6 reserved bits
		trustedDuration = get(32);								// 32 bits
	}
	else
		skipReading(CCTrustMetadata.fromBuffer( getReadPtr() ));

	return getReadCounter();
}


int TrustSpecification::toBuffer(byte * buf)
{
	setWriteBuffer(buf);
	put(hasCCBasedTrustMetadata, 1);
	put(0, 7);
	if(!hasCCBasedTrustMetadata)
	{
		skipWriting( startDate.toBuffer( getWritePtr() ));		// start date
		put(attackerProfile, 2);								// 2 bits
		put(0, 6);												// 6 reserved bits
		put(trustedDuration, 32);								// 32 bits
	}
	else
		skipWriting(CCTrustMetadata.toBuffer( getWritePtr() ));

	return getWriteCounter();
}


unsigned int TrustSpecification::getSize() const
{
	unsigned int size = 1;		// hasCCBasedTrustMetadata

	if(!hasCCBasedTrustMetadata)
	{
		size += 5 +		//startDate
				1 +		//attackerProfile
				4;		//trustedDuration
	} else 
	{
		size += CCTrustMetadata.getSize();
	}
	return size;
}



TrustedTool::TrustedTool()
{
	memset(toolID, 0, 16);
}


TrustedTool::~TrustedTool() 
{
}


void TrustedTool::setToolID(const ToolIdType & myToolId)
{
	memcpy(toolID, myToolId, 16);
}

const ToolIdType & TrustedTool::getToolID() const
{
	return toolID;
}

void TrustedTool::setAuditDate(const DateClass & date)
{	
	auditDate = date;
}

const DateClass & TrustedTool::getAuditDate() const
{
	return auditDate;
}

bool TrustedTool::checkSyntax() const 
{
	return true;
}

int TrustedTool::fromBuffer(const byte * buf)
{
	setReadBuffer(buf);
	getBuffer(toolID, 128);									// toolID
	skipReading(auditDate.fromBuffer(getReadPtr()));		// auditDate
	unsigned int numTrustSpecification = get(16);			// numTrustSpecification

	for(word32 i = 0; i < numTrustSpecification; i++)		// trust specification
	{
		TrustSpecification ts;
		ts.fromBuffer( getReadPtr() );
		trustSpecification.addObject( ts );
		skipReading( ts.getSize() * 8);
	}
	return (getReadCounter());
}


int TrustedTool::toBuffer(byte * buf)
{
	setWriteBuffer(buf);
	putBuffer(toolID, 128);									// toolID
	skipWriting(auditDate.toBuffer(getWritePtr()));			// auditDate
	unsigned int n = trustSpecification.getObjectNum();			
	put( n, 16);											// numTrustSpecification

	for(word32 i = 0; i < n; i++)							// trust specification
	{
		skipWriting( trustSpecification.getObject(i).toBuffer( getWritePtr()));
	}
	return (getWriteCounter());
}


unsigned int TrustedTool::getSize() const
{

	unsigned int size = 16 +	//toolID
						5 +		//auditDate
						2;		//numTrustSpecification

	for(word32 i = 0; i < trustSpecification.getObjectNum(); i++)
		size += trustSpecification.getObject(i).getSize();

	return size;						
}


IPMP_TrustSecurityMetadata::IPMP_TrustSecurityMetadata() 
:	IPMP_Data_BaseClass(IPMP_TrustSecurityMetadata_tag)
{
	
}

IPMP_TrustSecurityMetadata::IPMP_TrustSecurityMetadata(const IPMP_TrustSecurityMetadata & tsm)
:IPMP_Data_BaseClass(tsm),
trustedTool(tsm.trustedTool)
{
}


IPMP_TrustSecurityMetadata::~IPMP_TrustSecurityMetadata() 
{
	
}


int IPMP_TrustSecurityMetadata::fromBuffer(const byte * buf)
{
	IPMP_Data_BaseClass::fromBuffer(buf);
	
	word32 numTrustedTools = get(16);
	for(word32 i = 0; i < numTrustedTools; i++)
	{
		TrustedTool tt;
		tt.fromBuffer( getReadPtr() );
		trustedTool.addObject( tt );
		skipReading( tt.getSize() * 8);
	}
	return getReadCounter();
}


int IPMP_TrustSecurityMetadata::toBuffer(byte * buf)
{
	IPMP_Data_BaseClass::toBuffer(buf);

	word32 numTrustedTools = trustedTool.getObjectNum();			
	put( numTrustedTools, 16);							// numTrustedTools

	for(word32 i = 0; i < numTrustedTools; i++)	
	{
		skipWriting( trustedTool.getObject(i).toBuffer( getWritePtr()));
	}
	return (getWriteCounter());
}


unsigned int IPMP_TrustSecurityMetadata::sizeOfInstance() const
{
	unsigned int size = IPMP_Data_BaseClass::sizeOfInstance();

	size += 2;				// numTrustedTools (2 bytes)	

	for(word32 i = 0; i < trustedTool.getObjectNum(); i++)
		size += trustedTool.getObject(i).getSize();

	return size;									
}



