
//	This software module was originally developed by Massimo Balestri (TILAB)	
//	in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1) standard. 
//	This software module is an implementation of a part of one or more MPEG-4 
//	Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 
//	14496-1) standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	Telecom Italia Lab retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2002.


// MOSEStest.cpp : executes all the MOSES regression tests. 
//				   Generates files containing IPMPX messages in binary format.
#include "common.h"
#include "assert.h"

#include "IPMPToolES_DecoderConfig.h"
#include "IPMP_ToolId.h"
#include "IPMP_ToolES_AU.h"

#include <stdio.h>
#include <string.h>


USING_MOSES_NAMESPACE

// global vars
int counter;				// test counter
int errcounter;				// error counter

//////////////////////////////////////////////////////////
// serialize the object into a file, then read the file //
// into another object of the same class				//
//////////////////////////////////////////////////////////
void test(BitField & msg, char * fname, BitField &msgout)
{
	printf("Test # %02d: %s - creating file:   ", counter + 1, fname); 
	if (msg.toFile(fname))
		printf("passed.\n");
	else
	{
		printf("failed.\n");
		errcounter++;
	}
	
	// now read from the file

	printf("Test # %02d: %s - reading file:    ", counter + 1, fname); 
	if (msgout.fromFile(fname))
		printf("passed.\n");
	else
	{
		printf("failed.\n");
		errcounter++;
	}
	
	counter++;
}

void IPMP_ToolIdTest()
{
	IPMP_ToolId msg("b9919653-56aa-467d-8bff-3c2541878312");
	IPMP_ToolId msg2;

	test(msg, "IPMP_ToolId.bin", msg2);				// test
	assert(msg == msg2);
	IPMP_ToolId msg3 = msg;							// test copy constructor
	assert(msg3 == msg);

	IPMP_ToolId msg4;								// test reading the ID from a string
	msg4.fromString("b9919653-56aa-467d-8bff-3c2541878312");
	assert(msg4 == msg);
}

void IPMP_ByteArrayTest()	
{
	IPMP_ByteArray msg,msg2;
	msg.setData((byte *) "store this data", 15);
	test(msg, "IPMP_ByteArray.bin", msg2);			// test
	assert(msg == msg2);
	IPMP_ByteArray msg3 = msg;						// test also the copy constructor 
	assert( msg3 == msg);
}

void IPMP_ToolTest()
{
	IPMP_Tool msg, msgout;							// create the object under test
	msg.toolID.fromString("b9919653-56aa-467d-8bff-3c2541878312");
	IPMP_ToolId myid;
	myid.fromString("C9488377-73B8-AF32-14AA-3c2541878312");
	msg.addSpecificToolID(myid);
	myid.fromString("D6102365-73B8-AF32-14AA-3c2541878312");
	msg.addSpecificToolID(myid);
	byte url1[]="http://WWW.Wibble.com";
	byte url2[]="http://www.moses.org/";
	msg.addToolURL(url1,sizeof(url1));
	msg.addToolURL(url2,sizeof(url2));

	test(msg, "IPMP_Tool.bin", msgout);		// test the class serialization
	assert(msg == msgout);

	IPMP_Tool msg2 = msgout;				// test also the copy constructor
	assert(msg2 == msgout);					// do briefly all the above tests

	// IPMP_ToolListDescriptor
	
	IPMP_ToolListDescriptor msg3;

	msg3.ipmpTool.addObject(msg);
	msg3.ipmpTool.addObject(msg2);

	IPMP_ToolListDescriptor msg4;
	test(msg3, "IPMP_ToolListDescriptor.bin", msg4);	// test the class serialization

	IPMP_ToolListDescriptor msg5 = msg4;	// test the copy constructor
	assert(msg5 == msg3);		

	// IPMP_GetToolsResponse

	IPMP_GetToolsResponse msg6, msg7;
	msg6.setDataId(0x12341234);
	msg6.tools.addObject(msg);			// add three times the same IPMP_Tool
	msg6.tools.addObject(msg);
	msg6.tools.addObject(msg);
	test(msg6,"IPMP_GetToolsResponse.bin", msg7);
	assert(msg6 == msg7);
	IPMP_GetToolsResponse msg8 = msg6;					// test copy constructor
	assert(msg8 == msg6);
}

void DecoderSpecificInfoTest()
{
	DecoderSpecificInfo msg, msg2;
	test(msg, "DecoderSpecificInfo.bin", msg2);			// test
	DecoderSpecificInfo msg3 = msg2;	// test the copy constructor
	assert(msg3 == msg2);
}



void IPMPToolES_DecoderConfigTest()
{
	IPMPToolES_DecoderConfig msg;
	msg.toolID.fromString("b9919653-56aa-467d-8bff-3c2541878312");
	msg.setTool_Format_ID(0xf0a1c8ba);
	msg.setTool_Package_ID(0xe0c3);

	IPMPToolES_DecoderConfig msg2;
	test(msg, "IPMPToolES_DecoderConfig.bin", msg2);
	IPMPToolES_DecoderConfig msg3 = msg2;		// test copy constructor
	assert(msg == msg3);

}

void IPMP_DescriptorPointerTest()
{
	IPMP_DescriptorPointer msg;
	msg.setIPMP_DescriptorID(0x3402);
	msg.setIPMP_ES_ID(0x235a);

	IPMP_DescriptorPointer msg2;
	test(msg, "IPMP_DescriptorPointer.bin", msg2);
	IPMP_DescriptorPointer msg3 = msg2;		// test copy constructor
	assert(msg == msg3);
}

void IPMP_DescriptorHooksTest()
{
	IPMP_Descriptor msg;
	byte mydata[] = { 0xAA, 0xBB, 0xCC, 0xDD, 0xEE, 0xFF };
	bool b = msg.setHooksType(15, 0x34AB, mydata, sizeof(mydata));
	assert(b);
	IPMP_Descriptor msg2;
	test(msg, "IPMP_Descriptor_1.bin", msg2);
	IPMP_Descriptor msg3 = msg2;		// test copy constructor
	assert(msg == msg3);
}

void IPMP_DescriptorUrlTest()
{
	IPMP_Descriptor msg;
	bool b = msg.setUrlType(8, "http://wwww.mydomain.org/exampleurl.html");
	assert(b);
	IPMP_Descriptor msg2;
	test(msg, "IPMP_Descriptor_2.bin", msg2);
	IPMP_Descriptor msg3 = msg2;		// test copy constructor
	assert(msg == msg3);
}

void IPMP_DescriptorIpmpxTest()
{
	IPMP_Descriptor msg;
	IPMP_ToolId mytoolid("C9488377-73B8-AF32-14AA-3c2541878312");
	byte plainID[16];
	mytoolid.GetID(plainID);
	bool b = msg.setIPMPXType(0xFAFA, plainID, 0x01, 0xff);
	assert(b);
	
	IPMP_AudioWatermarkingInit msg2;			// Use this message to test that messages can be stored in the IPMPX_data array
	msg2.setInputFormat(IPMP_AudioWatermarkingInit::PCM, 2, 16, 44100);
	msg2.extractWatermark(0x1D);			
	msg.addIPMPXMessage(msg2);						// add three times the same message
	msg.addIPMPXMessage(msg2);
	msg.addIPMPXMessage(msg2);

	IPMP_Descriptor msg3;
	test(msg, "IPMP_Descriptor_3.bin", msg3);	// test
	assert(msg == msg3);						// compare messages
	IPMP_Descriptor msg4 = msg3;				// test copy constructor
	assert(msg == msg4);			// compare messages

// IPMP_DescriptorUpdate

	IPMP_DescriptorUpdate msg5;
	msg5.addDescriptor(msg);
	IPMP_DescriptorUpdate msg6;
	test(msg5, "IPMP_DescriptorUpdate.bin", msg6);
	IPMP_DescriptorUpdate msg7 = msg6;			// test copy constructor
	assert(msg5 == msg7);
}


void IPMP_MessageUrlTest()
{
	IPMP_Message msg, msg2;
	bool b = msg.setUrlType("http://www.trythis.com/");
	assert(b);
	test(msg,"IPMP_Message_1.bin", msg2);		// test
	assert(msg == msg2);
	IPMP_Message msg3 = msg2;					// test copy constructor
	assert(msg3 == msg2);
}

void IPMP_MessageHooksTest()
{
	IPMP_Message msg, msg2;
	byte mydata[] = { 0xAA, 0xBB, 0xCC, 0xDD, 0xEE, 0xFF };
	bool b = msg.setHooksType(0x3A17, mydata, sizeof(mydata));
	assert(b);
	test(msg,"IPMP_Message_2.bin", msg2);		// test
	assert(msg == msg2);
	IPMP_Message msg3 = msg2;					// test copy constructor
	assert(msg3 == msg2);
}

void IPMP_MessageIpmpxTest()
{
	IPMP_Message msg, msg2;
	bool b = msg.setIPMPXType(0xAAAA);		// set descriptor_ID
	assert(b);

	IPMP_AudioWatermarkingInit wminit;			// Use this message to test that messages can be stored in the IPMPX_data array
	wminit.setInputFormat(IPMP_AudioWatermarkingInit::PCM, 2, 16, 44100);
	wminit.extractWatermark(0x1D);			
	msg.addIPMPXMessage(wminit);						// add three times the same message
	msg.addIPMPXMessage(wminit);
	msg.addIPMPXMessage(wminit);

	test(msg,"IPMP_Message_3.bin", msg2);		// test
	assert(msg == msg2);
	IPMP_Message msg3 = msg2;					// test copy constructor
	assert(msg3 == msg2);
}

void IPMP_GetToolsTest()
{
	IPMP_GetTools msg, msg2;
	msg.setDataId(0x12341234);
	test(msg,"IPMP_GetTools.bin", msg2);		// test
	assert(msg == msg2);
	IPMP_GetTools msg3 = msg2;					// test copy constructor
	assert(msg3 == msg2);
}

void IPMP_GetToolContextTest()
{
	IPMP_GetToolContext msg, msg2;
	msg.setDataId(0xaabbccdd);					
	msg.setScope( IPMP_GetToolContext::SCOPE_LOCAL);
	msg.setToolDescriptorID(0xF0A0);
	test(msg,"IPMP_GetToolContext.bin", msg2);		// test
	assert(msg == msg2);
	IPMP_GetToolContext msg3 = msg2;					// test copy constructor
	assert(msg3 == msg2);
}

void IPMP_GetToolContextResponseTest()
{
	IPMP_GetToolContextResponse msg, msg2;
	msg.setDataId(0xaabbccdd);
	msg.setESD_ID(12720);
	msg.setOD_ID(33);
	msg.setIPMP_ToolContextID(32);
	test(msg,"IPMP_GetToolContextResponse.bin", msg2);		// test
	assert(msg == msg2);
	IPMP_GetToolContextResponse msg3 = msg2;					// test copy constructor
	assert(msg3 == msg2);
}

void IPMP_ConnectToolTest()
{
	IPMP_ConnectTool msg, msg2;
	IPMP_Descriptor td;
	td.setUrlType(0xCA, "http://www.Inside_ConnectTool.org");
	msg.setIPMP_Descriptor(td);
	test(msg,"IPMP_ConnectTool.bin", msg2);						// test
	assert(msg == msg2);
	IPMP_ConnectTool msg3 = msg2;								// test copy constructor
	assert(msg3 == msg2);
}

void IPMP_DisconnectToolTest()
{
	IPMP_DisconnectTool msg, msg2;
	msg.setDataId(1234);
	msg.setIPMP_ToolContextID(3656);
	test(msg,"IPMP_DisconnectTool.bin", msg2);					// test
	assert(msg == msg2);
	IPMP_DisconnectTool msg3 = msg2;								// test copy constructor
	assert(msg3 == msg2);
}

void IPMP_AddToolNotificationListenerTest()
{
	IPMP_AddToolNotificationListener msg, msg2;
	msg.setDataId(273847);
	msg.setScope(Constants::SCOPE_ESD);
	msg.addEventType(Constants::EVENT_CONNECTED);
	msg.addEventType(Constants::EVENT_CONNECTIONFAILED);
	msg.addEventType(Constants::EVENT_WATERMARKDETECTED);
	test( msg, "IPMP_AddToolNotificationListener.bin", msg2 );	// test
	assert(msg == msg2);
	IPMP_AddToolNotificationListener msg3 = msg2;				// test copy constructor
	assert(msg3 == msg2);

	assert(msg3.getScope() == Constants::SCOPE_ESD);						// additional tests
	assert(msg3.getEventTypeCount() == 3);
	Constants::EventType et;
	assert(msg3.getEventType(et, 0) == true);
	assert(et == Constants::EVENT_CONNECTED);
	assert(msg3.getEventType(et, 1) == true);
	assert(et == Constants::EVENT_CONNECTIONFAILED);
	assert(msg3.getEventType(et, 2) == true);
	assert(et == Constants::EVENT_WATERMARKDETECTED);
	assert(msg3.getEventType(et, 3) == false);
}


void IPMP_RemoveToolNotificationListenerTest()
{
	IPMP_RemoveToolNotificationListener msg, msg2;
	msg.addEventType(Constants::EVENT_CONNECTED);
	msg.addEventType(Constants::EVENT_CONNECTIONFAILED);
	msg.addEventType(Constants::EVENT_WATERMARKDETECTED);
	test( msg, "IPMP_RemoveToolNotificationListener.bin", msg2 );		// test
	assert(msg == msg2);
	IPMP_RemoveToolNotificationListener msg3 = msg2;				// test copy constructor
	assert(msg3 == msg2);

	assert(msg3.getEventTypeCount() == 3);
	Constants::EventType et;
	assert(msg3.getEventType(et, 0) == true);
	assert(et == Constants::EVENT_CONNECTED);
	assert(msg3.getEventType(et, 1) == true);
	assert(et == Constants::EVENT_CONNECTIONFAILED);
	assert(msg3.getEventType(et, 2) == true);
	assert(et == Constants::EVENT_WATERMARKDETECTED);
	assert(msg3.getEventType(et, 3) == false);
}

void IPMP_NotifyToolEventTest()
{
	IPMP_NotifyToolEvent msg, msg2;					// initialize
	msg.setScope(0x6A, 0x4F1A);
	msg.setEventType(Constants::EVENT_CONNECTED);
	msg.setToolContextID(0x6A2834FE);
	test(msg, "IPMP_NotifyToolEvent.bin", msg2);		// test
	assert(msg == msg2);
	IPMP_NotifyToolEvent msg3 = msg2;				// test copy constructor
	assert(msg3 == msg2);

	unsigned int myOD_ID, myESD_ID;
	assert(msg.getScope(myOD_ID, myESD_ID));	// additional test
	assert(myOD_ID == 0x6A);
	assert(myESD_ID == 0x4F1A);
	assert(msg.getEventType() == Constants::EVENT_CONNECTED);
	assert(msg.getToolContextID() == 0x6A2834FE);
}

void IPMP_opaqueDataTest()
{
	IPMP_OpaqueData msg, msg2;									// create the object under test
	byte myData[] = {1,2,3,4,5,6,7,8,9,10,11,12,13,14,15};		// private data
	msg.setOpaqueData(myData, sizeof(myData));
	test(msg, "IPMP_OpaqueData.bin", msg2);
	assert(msg == msg2);
	IPMP_OpaqueData msg3 = msg2;				// test copy constructor
	assert(msg3 == msg2);
}

void IPMP_KeyDataTest()
{

	IPMP_KeyData msg, msg2 ;
	msg.setDataId(0xffa0);
	byte key[16] =  {1,2,3,4,5,6,7,8,9,10,11,12,13,14,15, 16};
	msg.setKeyBody(key, sizeof(key) ) ;

	word32 startPcktID = 19288374;
	byte expDTS[] = {1,2,3,4,5,6,7,8};

	msg.setStartPacketID(startPcktID);
	msg.setExpireDTS(expDTS);

	byte opaque[] = {1,0,3,0,5,0,7,0,9,0,1,2,0,4,5,6 };
	msg.setOpaqueData(opaque, sizeof(opaque) ) ;

	test( msg, "IPMP_KeyData.bin", msg2 ) ;
	assert(msg == msg2);
	IPMP_KeyData msg3 = msg2;				// test copy constructor
	assert(msg3 == msg2);

}

void IPMP_RightsDataTest()
{

	IPMP_RightsData msg, msg2 ;
	char *rights = "Play - No Copy - 30 days Trial" ;
	msg.setRightsInfo( (byte*)rights, strlen(rights) ) ;

	test( msg, "IPMP_RightsData.bin", msg2 ) ;
	assert(msg == msg2);
	IPMP_RightsData msg3 = msg2;				// test copy constructor
	assert(msg3 == msg2);

}

void IPMP_TrustSecurityMetadataTest()
{
	IPMP_TrustSecurityMetadata msg, msg2;

	IPMP_ToolId myguid("b9919653-56aa-467d-8bff-3c2541878312");
	byte myid[16];
	myguid.GetID(myid);

	DateType dummyDate = {5, 4, 3, 2, 1};
	DateClass mydate;
	mydate.setDate(dummyDate);


	msg.setDataId(12341234);

	TrustSpecification ts;					// set-up the trust specification
	ts.setTrustMetadata(dummyDate, 1, 99);

	TrustedTool tt;							// set up the trusted tool
	tt.setToolID(myid);
	tt.setAuditDate(mydate);
	tt.trustSpecification.addObject(ts);

	msg.trustedTool.addObject(tt);			// add all trusted tools to the list
	msg.trustedTool.addObject(tt);			// add all trusted tools to the list
	msg.trustedTool.addObject(tt);			// add all trusted tools to the list

	test(msg, "IPMP_TrustSecurityMetadata.bin", msg2);
	assert(msg == msg2);
	IPMP_TrustSecurityMetadata msg3 = msg2;				// test copy constructor
	assert(msg3 == msg2);


// IPMP_SecureContainer - uses the above message as payload

	
	IPMP_SecureContainer sc, sc2;
	sc.setDataId(237);
	sc.setProtectedMsg(msg);		// use TrustSecurityMetadata as payload
	IPMP_ByteArray myMAC;

	myMAC.setData(dummyDate, sizeof(dummyDate));	// use the dummy date as MAC - just for test
	sc.setMAC(myMAC);
	test(sc, "IPMP_SecureContainer.bin", sc2);
	assert(sc == sc2);
	IPMP_SecureContainer sc3 = sc2;				// test copy constructor
	assert(sc3 == sc2);
}

void IPMP_MessageFromBitstreamTest()
{
	// must use IPMP_Message as payload
	IPMP_Message msg, msg2;
	bool b = msg.setUrlType("http://www.trythis.com/");
	assert(b);
	test(msg,"IPMP_Message_1.bin", msg2);		// test
	assert(msg == msg2);
	IPMP_Message msg3 = msg2;					// test copy constructor
	assert(msg3 == msg2);

	IPMP_MessageFromBitstream mfb, mfb2;
	mfb.setRecipient(32);
	mfb.setSender(122);
	mfb.message.addObject(msg);				// add IPMP_Message
	test(mfb, "IPMP_MessageFromBitstream.bin", mfb2);
	assert(mfb == mfb2);
	IPMP_MessageFromBitstream mfb3 = mfb2;				// test copy constructor
	assert(mfb3 == mfb2);
}

void IPMP_DescriptorFromBitstreamTest()
{
	IPMP_DescriptorFromBitstream dfb, dfb2;
	dfb.setSender(123);
	dfb.setRecipient(321);
	dfb.toolDescriptor.setUrlType(0x22,"http://www.mywebsite.com/");
	test(dfb, "IPMP_DescriptorFromBitstream.bin", dfb2);
	assert(dfb == dfb2);
	IPMP_DescriptorFromBitstream dfb3 = dfb2;				// test copy constructor
	assert(dfb3 == dfb2);
}

void IPMP_MessageFromToolTest()
{
	IPMP_MessageFromTool msg, msg2;
	msg.setRecipient(24);
	msg.setSender(32);

	IPMP_RightsData rd;			// use Rights data as payload
	char *rights = "Play - No Copy - 30 days Trial" ;
	rd.setRightsInfo( (byte*)rights, strlen(rights) ) ;
	msg.addMessage(rd);
	msg.addMessage(rd);				// add two times

	test(msg, "IPMP_MessageFromTool.bin", msg2);		// test
	assert(msg == msg2);
	IPMP_MessageFromTool msg3 = msg2;					// test copy constructor
	assert(msg3 == msg2);
}

void IPMP_CanProcessTest()
{
	IPMP_CanProcess msg, msg2;
	msg.setCanProcess();
	test(msg, "IPMP_CanProcess.bin", msg2);			// test 
	assert(msg == msg2);
	IPMP_CanProcess msg3 = msg2;					// test copy constructor
	assert(msg3 == msg2);
}

void IPMP_UserQueryTest()
{
	IPMP_UserQuery msg, msg2 ;
	msg.setDataId(223);

	msg.setInclSMIL() ;

	char * mySmil = "SMILE while you SMIL";
	char * mySmilUrl = "mySMIL_URL.com";

	IPMP_ByteArray smil;
	smil.setData((byte *) mySmil, strlen(mySmil));

	UserQueryText uqt(false, false, false, false, true);
	uqt.setLanguageCode(37);
	uqt.setSMIL(smil);
	msg.userQueryText.addObject(uqt);		// add text
	
	uqt.setLanguageCode(29);
	IPMP_ByteArray smilUrl;
	smilUrl.setData((byte *) mySmilUrl, strlen(mySmilUrl));

	uqt.setSMIL_URL(smilUrl);

	msg.userQueryText.addObject(uqt);		// add URL
	
	test( msg, "IPMP_UserQuery.bin", msg2 ) ;				// test
	assert(msg == msg2);
	IPMP_UserQuery msg3 = msg2;					// test copy constructor
	assert(msg3 == msg2);
}

void IPMP_UserQueryResponseTest()
{
	IPMP_UserQueryResponse msg, msg2;
	msg.setDataId(333);
	QTArray myReply;
	myReply.setPromptText((byte *) "See you tomorrow", strlen("See you tomorrow"));
	myReply.setId(324);
	myReply.setSubId(78);
	msg.replyText.addObject(myReply);
	test( msg, "IPMP_UserQueryResponse.bin", msg2 ) ;				// test
	assert(msg == msg2);
	IPMP_UserQueryResponse msg3 = msg2;					// test copy constructor
	assert(msg3 == msg2);

}

void IPMP_InitAuthenticationTest()
{
	IPMP_InitAuthentication msg, msg2;
	msg.setAuthType(34);
	msg.setDataId(28847);
	msg.setContext(994588);
	test( msg, "IPMP_InitAuthentication.bin", msg2 ) ;				// test
	assert(msg == msg2);
	IPMP_InitAuthentication msg3 = msg2;					// test copy constructor
	assert(msg3 == msg2);
}

void IPMP_MutualAuthenticationTest()
{
	char * fname = NULL;
	/* Test negotiation message */
	AlgorithmDescriptor * ppCandidates[2];
	/* First candidate uses an id assigned by a registration authority. */
	AlgorithmDescriptor * pCandidate0 = new AlgorithmDescriptor;
	pCandidate0->setRegAuthId(0x1234);
	pCandidate0->setOpaqueData((const byte *)"CANDIDATE0", 11);
	ppCandidates[0] = pCandidate0;
	/* Second candidate uses a normative identifier. */
	AlgorithmDescriptor * pCandidate1 = new AlgorithmDescriptor;
	unsigned short specAlgoId = 0x01AA;
	pCandidate1->setSpecAlgoId((const byte *)&specAlgoId, 2);
	pCandidate1->setOpaqueData((const byte *)"CANDIDATE1", 11);
	ppCandidates[1] = pCandidate1;
	/* Set */
	IPMP_MutualAuthentication msg1, msgout1;
	msg1.setRequestNegotiation(ppCandidates, 2);
	delete pCandidate1;
	fname = "IPMP_MutualAuthentication_1.bin";
	test(msg1, fname, msgout1);
	assert(msg1 == msgout1);
	IPMP_MutualAuthentication msgcopy1 = msg1;					// test copy constructor
	assert(msgcopy1 == msg1);

	/* Test failed negotiation */
	IPMP_MutualAuthentication msg2, msgout2;
	msg2.setFailedNegotiation();
	test(msg2, "IPMP_MutualAuthentication_2.bin", msgout2);
	assert(msg2 == msgout2);
	IPMP_MutualAuthentication msgcopy2 = msg2;					// test copy constructor
	assert(msgcopy2 == msg2);

	/* Test success negotiation */
	AlgorithmDescriptor * ppAgreed[1];
	ppAgreed[0] = pCandidate0;
	/* Set */
	IPMP_MutualAuthentication msg3, msgout3;
	msg3.setSuccessNegotiation(ppAgreed, 1);
	delete pCandidate0;
	test(msg3, "IPMP_MutualAuthentication_3.bin", msgout3);
	assert(msg3 == msgout3);
	IPMP_MutualAuthentication msgcopy3 = msg3;					// test copy constructor
	assert(msgcopy3 == msg3);

	/* Test authentication data */
	IPMP_MutualAuthentication msg4, msgout4;
	msg4.setAuthenticationData((const byte *)"TEST DATA", 10);
	test(msg4, "IPMP_MutualAuthentication_4.bin", msgout4);
	assert(msg4 == msgout4);
	IPMP_MutualAuthentication msgcopy4 = msg4;					// test copy constructor
	assert(msgcopy4 == msg4);

	/* Test authentication codes */
	AuthCodes * pAuthCodes;
	IPMP_TrustSecurityMetadata trustData;
	// generate the trust security metadata
	ToolIdType IPMP_ToolId = {0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15};
	DateType dummyDate = {5, 4, 3, 2, 1};
	DateClass mydate;
	mydate.setDate(dummyDate);
	trustData.setDataId(12341234);

	TrustSpecification ts;					// set-up the trust specification
	ts.setTrustMetadata(dummyDate, 1, 99);

	TrustedTool tt;							// set up the trusted tool
	tt.setToolID(IPMP_ToolId);
	tt.setAuditDate(mydate);
	tt.trustSpecification.addObject(ts);

	trustData.trustedTool.addObject(tt);			// add the trusted tool to the list

	/* Test authentication codes (type = 0x00) */
	IPMP_MutualAuthentication msg5, msgout5;
	pAuthCodes = new AuthCodes;
	pAuthCodes->setAuthCodes((const byte *)"AUTH CODES 0x00", 16, &trustData);
	msg5.setAuthCodes(pAuthCodes);
	delete pAuthCodes; pAuthCodes = 0;
	test(msg5, "IPMP_MutualAuthentication_5.bin", msgout5);
	assert(msg5 == msgout5);
	IPMP_MutualAuthentication msgcopy5 = msg5;					// test copy constructor
	assert(msgcopy5 == msg5);

	/* Test authentication codes (type = 0x01) */
	Certificate * ppCertificates[2];
	/* Create certificates */
	Certificate * pCertificate0 = new Certificate;
	pCertificate0->setData((const byte *)"CERTIFICATE 0", 14);
	ppCertificates[0] = pCertificate0;
	Certificate * pCertificate1 = new Certificate;
	pCertificate1->setData((const byte *)"CERTIFICATE 1", 14);
	ppCertificates[1] = pCertificate1;
	/* Set */
	IPMP_MutualAuthentication msg6, msgout6;
	pAuthCodes = new AuthCodes;
	pAuthCodes->setAuthCodes((const byte *)"AUTH CODES 0x01", 16, &trustData);
	pAuthCodes->setCertificates(0, ppCertificates, 2);
	delete pCertificate0;
	delete pCertificate1;
	msg6.setAuthCodes(pAuthCodes);
	delete pAuthCodes; pAuthCodes = 0;
	test(msg6, "IPMP_MutualAuthentication_6.bin", msgout6);
	assert(msg6 == msgout6);
	IPMP_MutualAuthentication msgcopy6 = msg6;					// test copy constructor
	assert(msgcopy6 == msg6);

	/* Test authentication codes (type = 0x02) */
	IPMP_MutualAuthentication msg7, msgout7;
	pAuthCodes = new AuthCodes;
	pAuthCodes->setAuthCodes((const byte *)"AUTH CODES 0x02", 16, &trustData);
	msg7.setAuthCodes(pAuthCodes);
	delete pAuthCodes; pAuthCodes = 0;
	test(msg7, "IPMP_MutualAuthentication_7.bin", msgout7);
	assert(msg7 == msgout7);
	IPMP_MutualAuthentication msgcopy7 = msg7;					// test copy constructor
	assert(msgcopy7 == msg7);

	/* Test authentication codes (type = 0xFE) */
	IPMP_MutualAuthentication msg8, msgout8;
	pAuthCodes = new AuthCodes;
	pAuthCodes->setAuthCodes((const byte *)"AUTH CODES 0xFE", 16, &trustData);
	msg8.setAuthCodes(pAuthCodes);
	delete pAuthCodes; pAuthCodes = 0;
	test(msg8, "IPMP_MutualAuthentication_8.bin", msgout8);
	assert(msg8 == msgout8);
	IPMP_MutualAuthentication msgcopy8 = msg8;					// test copy constructor
	assert(msgcopy8 == msg8);
}

void IPMP_SelectiveDecryptionInitCiTest()					// content independent decryption
{
	IPMP_SelectiveDecryptionInit msg, msgout;
	msg.setDataId(3847);
	msg.setMediaTypeExtension(IPMP_SelectiveDecryptionInit::ISO_IEC);
	msg.setMediaTypeIndication(IPMP_SelectiveDecryptionInit::MPEG1_AUDIO);
	msg.setProfileLevelIndication(3);
	msg.setCompliance(IPMP_SelectiveDecryptionInit::FULLY_COMPLIANT);

	BufInfoStruct myBufInfo;
	byte mycipherID[16] = { 2,4,3,6,5,8,2,5,9,7,6,4,7,4,3,6 };
	myBufInfo.setBlockCipher(
		mycipherID,									// cipher ID
		BufInfoStruct::VIDEO_PACKET,				// sync boundary
		BufInfoStruct::CTR,							// mode
		16,											// block size
		16);										// key size

	msg.bufInfoStruct.addObject(myBufInfo);
	unsigned short myRLE[] = {0x09af,0x003f};
	msg.setDecryptionSegments(myRLE, 2);
	test(msg,"IPMP_SelectiveDecryptionInit_1.bin", msgout);		// do the test
	assert(msg == msgout);
	IPMP_SelectiveDecryptionInit msgcopy = msg;					// test copy constructor
	assert(msgcopy == msg);	
}

void IPMP_SelectiveDecryptionInitCsTest()				// content specific selective decryption
{
	IPMP_SelectiveDecryptionInit msg, msgout;
	msg.setDataId(3847);
	msg.setMediaTypeExtension(IPMP_SelectiveDecryptionInit::ISO_IEC);
	msg.setMediaTypeIndication(IPMP_SelectiveDecryptionInit::MPEG1_AUDIO);
	msg.setProfileLevelIndication(3);			// MP3 ?
	msg.setCompliance(IPMP_SelectiveDecryptionInit::FULLY_COMPLIANT);

	FieldStruct & x = msg.addContentSpecificField();
	x.field_Id = FieldStruct::DC_COEFFICIENT;
	x.field_Scope = 0;
	x.buf = 73;
	unsigned short mymap[3] = {25,15,41};
	unsigned short myshuffle[] = {46828, 56000};
	IPMP_ByteArray myshuffletable;
	myshuffletable.setData((byte * ) myshuffle, sizeof(myshuffle));
	x.setMappingTable(mymap, 3);
	x.setShuffleTable(myshuffletable);
	test(msg,"IPMP_SelectiveDecryptionInit_2.bin", msgout);		// do the test
	assert(msg == msgout);
	IPMP_SelectiveDecryptionInit msgcopy = msg;					// test copy constructor
	assert(msgcopy == msg);	
}

void IPMP_AudioWatermarkingInitTest()
{
	IPMP_AudioWatermarkingInit msg, msgout;
	msg.setDataId(0x01020304);
	byte payload[] = { 0xAB, 0xCD, 0xEF, 0xAA };
	msg.setInputFormat(IPMP_AudioWatermarkingInit::PCM, 2, 16, 44100);
	msg.insertWatermark(payload, sizeof(payload));
	test(msg,"IPMP_AudioWatermarkingInit.bin", msgout);
	assert(msg == msgout);
	IPMP_AudioWatermarkingInit msgcopy = msg;  // test copy constructor
	assert( msg == msgcopy);
}	

void IPMP_SendAudioWatermarkTest()
{
	IPMP_SendAudioWatermark msg, msgout;
	msg.setDataId(456);
	msg.setWMStatus(IPMP_SendAudioWatermark::WM_PAYLOAD);
	byte mypayload[] ={ 33, 15, 27 };
	msg.setPayload(mypayload, sizeof(mypayload));
	msg.setCompressionStatus(IPMP_SendAudioWatermark::COMPRESSION);
	msg.setOpaqueData(mypayload, sizeof(mypayload));
	test(msg,"IPMP_SendAudioWatermark.bin", msgout);
	assert(msg == msgout);
	IPMP_SendAudioWatermark msgcopy = msg;  // test copy constructor
	assert( msg == msgcopy);
}

void IPMP_VideoWatermarkingInitTest()
{
	IPMP_VideoWatermarkingInit msg, msgout;
	msg.setInputFormat(IPMP_VideoWatermarkingInit::YUV,720,576, IPMP_VideoWatermarkingInit::CHROMA_420);
	byte myPayload[] = {0xf1, 0xf2,0xf3, 0xf4};
	msg.insertWatermark(myPayload, sizeof(myPayload));
	test(msg,"IPMP_VideoWatermarkingInit.bin", msgout);
	assert(msg == msgout);
	IPMP_VideoWatermarkingInit msgcopy = msg;  // test copy constructor
	assert( msg == msgcopy);
}

void IPMP_SendVideoWatermarkTest()
{
	IPMP_SendVideoWatermark msg, msgout;
	msg.setWMStatus(IPMP_SendVideoWatermark::WM_NOPAYLOAD);
	test(msg,"IPMP_SendVideoWatermark.bin", msgout);
	assert(msg == msgout);
	IPMP_SendVideoWatermark msgcopy = msg;  // test copy constructor
	assert( msg == msgcopy);
}

void IPMP_ToolParamCapabilitiesQueryTest()
{
	IPMP_ToolParamCapabilitiesQuery msg, msgout;
	msg.setDataId(356);
	const char descr[] = "my Tool rocks";
	msg.toolParamDesc.descriptionComment.setData((unsigned char *) descr, strlen(descr));
	msg.toolParamDesc.majorVersion = 2;
	msg.toolParamDesc.minorVersion = 19;
	test(msg,"IPMP_ToolParamCapabilitiesQuery.bin", msgout);
	assert(msg == msgout);
	IPMP_ToolParamCapabilitiesQuery msgcopy = msg;  // test copy constructor
	assert( msg == msgcopy);
}

void IPMP_ToolParamCapabilitiesResponseTest()
{
	IPMP_ToolParamCapabilitiesResponse msg, msgout;
	msg.setDataId(356);
	msg.setCapabilitiesSupported(true);
	test(msg,"IPMP_ToolParamCapabilitiesResponse.bin", msgout);		// test
	assert(msg == msgout);
	IPMP_ToolParamCapabilitiesResponse msgcopy = msg;  // test copy constructor
	assert( msg == msgcopy);
}

void IPMP_ParametricDescriptionTest()
{
	IPMP_ParametricDescription msg, msgout;
	msg.setDataId(18);
	Description desc;
	desc.toolClass.setData((byte *)"decryption", 10);
	desc.toolSubClass.setData((byte *)"AES", 3);
	desc.toolTypeData.setData((byte *)"Block_size", 10);
	desc.toolType.setData((byte *)"128",3);
	msg.majorVersion = 1;
	msg.minorVersion = 0;
	msg.descriptions.addObject(desc);
	test(msg,"IPMP_ParametricDescription.bin", msgout);		// test
	assert(msg == msgout);
	IPMP_ParametricDescription msgcopy = msg;  // test copy constructor
	assert(msg == msgcopy);
}

void IPMP_ToolES_AUTest()
{
	IPMP_ToolES_AU msg, msg2;
	byte myToolBody[] = {11, 13, 54,22, 18, 78, 3, 44, 85, 133, 14};	// dummy data - just for testing 
	byte mySignature[] = {0xa0, 0xa1, 0xa2, 0xa3};						// dummy signature
	byte myCert[] = {0xCC, 0xDD, 0xEE, 0xFF};							// dummy certificate
	Certificate cert;
	cert.setData(myCert, sizeof(myCert));
	byte myVerifToolId[16] = {1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16};	// dummy tool ID

	msg.setToolBody(myToolBody, sizeof(myToolBody));
	msg.setIPMP_Tool_Signature(mySignature, sizeof(mySignature));
	msg.setVerifyingToolId(myVerifToolId);
	msg.addCertificate(0x01, &cert);
	msg.addCertificate(0x01, &cert);
	
	test(msg, "IPMP_ToolES_AU.bin", msg2);								// do the test

	IPMP_ToolES_AU msg3 = msg;		// test also the copy constructor 
	assert( msg3 == msg);
}

//---------------- Tests for Inter-device message---------------//
//---------------- Added By Liao Xuan --------------------------//

void IPMP_ResponseToContentRequestTest()  
{
	IPMP_ResponseToContentRequest msg, msg2;
	msg.setResponse(3);
	msg.setDataId(8);

	test(msg,"IPMP_ResponseToContentRequest.bin", msg2);		// test
	assert(msg==msg2);
	IPMP_ResponseToContentRequest msg3 = msg2;					// test copy constructor
	assert(msg3==msg2);
}

void IPMP_ContentTransferTest()
{
	IPMP_ContentTransfer msg, msg2;
	msg.setDataId(32);

	byte contentID[] = {0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15};	
	msg.setContentID(contentID);

    msg.setSequence(3);

    byte payload[] ={ 1, 3, 2,4,8 };
	msg.setPayload(payload, sizeof(payload));

	test(msg,"IPMP_ContentTransfer.bin", msg2);	    	// test
	assert(msg==msg2);
	IPMP_ContentTransfer msg3 = msg2;					// test copy constructor
	assert(msg3==msg2);
}

void IPMP_RequestToolTest() 
{
	IPMP_RequestTool msg, msg2;

	byte toolID[] = {0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15};	
	msg.setToolID(toolID);

	byte domainID[] = {0,0,0,0,0,0,0,0,0,0,0,0,0,3,2,1};	
	msg.setDomainID(domainID);


	byte opaque[] = {1,2,3,4,5};
	msg.setOpaqueData(opaque, sizeof(opaque) ) ;

	test(msg,"IPMP_RequestTool.bin", msg2);	
	msg2.toFile("IPMP_RequestTool_out.bin"); // test

	assert(msg==msg2);
	IPMP_RequestTool msg3 = msg2;					// test copy constructor
	assert(msg3==msg2);

}

void IPMP_ResponseToToolRequestTest()
{
	IPMP_ResponseToToolRequest msg, msg2;
	//msg.setToolID(10); 
	byte toolID[] = {0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15};	
	msg.setToolID(toolID);

	msg.setResponse(3);

    byte payload[] ={ 3, 1, 2 };
	msg.setPayload(payload, sizeof(payload));

	byte toolDescription[] ={ 1, 2 };
	msg.setToolDescription(toolDescription, sizeof(toolDescription));
    
	test(msg,"IPMP_ResponseToToolRequest.bin", msg2);		// test
	assert(msg==msg2);

	IPMP_ResponseToToolRequest msg3 = msg2;					// test copy constructor
	assert(msg3==msg2);
}

void IPMP_DeviceID_NotificationTest()
{
	IPMP_DeviceID_Notification msg, msg2;

	byte domainID[] = {0,1,2,0,0,0,0,0,0,0,0,0,0,0,0,15};	
	msg.setDomainID(domainID);

	byte opaque[] = {1,2,3};
	msg.setOpaqueData(opaque, sizeof(opaque) ) ;

	test(msg,"IPMP_DeviceID_Notification.bin", msg2);
	msg2.toFile("IPMP_DeviceID_Notification_out.bin")   ;      // test
	assert(msg== msg2);
	IPMP_DeviceID_Notification msg3 = msg2;					// test copy constructor
	assert(msg3== msg2);
}


void IPMP_RequestContentTest ()
{
	IPMP_RequestContent msg, msg2;
    byte contentID[] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};	
	msg.setContentID(contentID);
	msg.setDataId(3);

    byte domainID[] = {15,15,15,15,15,15,15,15,0,0,0,0,0,0,0,15};	
	msg.setDomainID(domainID);

	byte opaque[] = {1,2,3};
	msg.setOpaqueData(opaque, sizeof(opaque) ) ;

	test(msg,"IPMP_RequestContent.bin", msg2);		// test
	assert(msg==msg2);

	IPMP_RequestContent msg3 = msg2;
	msg3.toFile("IPMP_RequestContent_copy.bin") ;     // test copy constructor
	assert(msg3==msg2);
}

void IPMP_MessageFromDeviceTest()
{
	IPMP_MessageFromDevice msg, msg2;
    byte sender[] = {1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0};	
	msg.setSender(sender);

	byte recipient[] = {1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0};	
	msg.setRecipient(recipient);
	
    // add message rd 
	IPMP_RightsData rd;			// use Rights data as payload
	char *rights = "Play - No Copy - 30 days Trial" ;
	rd.setRightsInfo( (byte*)rights, strlen(rights) ) ;
	msg.addMessage(rd);

	// add message dn twice
    IPMP_DeviceID_Notification dn;

	byte domainID[] = {0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15};	
	dn.setDomainID(domainID);

	byte opaque[] = {1,2,3,4,5};
	dn.setOpaqueData(opaque, sizeof(opaque) ) ;
	msg.addMessage(dn);			// add message
	msg.addMessage(dn);

	// add message rc
	IPMP_RequestContent rc;
    byte contentID[] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};	
	rc.setContentID(contentID);
	byte domainID2[] = {0,1,2,3,4,5,6,7,8,0,0,0,0,13,14,15};	
	rc.setDomainID(domainID2);
	rc.setDataId(3);
	byte opaque2[] = {1,2,3};
	rc.setOpaqueData(opaque2, sizeof(opaque2) ) ;
    msg.addMessage(rc);		

	test(msg, "IPMP_MessageFromDevice.bin", msg2);		 
	assert(msg==msg2);
	IPMP_MessageFromDevice msg3 = msg2;	// test copy constructor
	assert(msg3==msg2);
}

//
// M A I N - do all the regression tests.
//
int main()
{	
	counter = 0;
	errcounter = 0;
	
	IPMP_ToolIdTest();
	IPMP_ByteArrayTest();
	IPMP_ToolTest();
	DecoderSpecificInfoTest();
	IPMPToolES_DecoderConfigTest();
	IPMP_DescriptorPointerTest();
	IPMP_DescriptorHooksTest();
	IPMP_DescriptorUrlTest();
	IPMP_DescriptorIpmpxTest();
	IPMP_MessageUrlTest();
	IPMP_MessageHooksTest();
	IPMP_MessageIpmpxTest();
	IPMP_GetToolsTest();
	IPMP_GetToolContextTest();
	IPMP_GetToolContextResponseTest();
	IPMP_ConnectToolTest();
	IPMP_DisconnectToolTest();
	IPMP_AddToolNotificationListenerTest();
	IPMP_RemoveToolNotificationListenerTest();
	IPMP_NotifyToolEventTest();
	IPMP_opaqueDataTest();
	IPMP_KeyDataTest();
	IPMP_RightsDataTest();
	IPMP_TrustSecurityMetadataTest();
	IPMP_MessageFromBitstreamTest();
	IPMP_DescriptorFromBitstreamTest();
	IPMP_MessageFromToolTest();
	IPMP_CanProcessTest();
	IPMP_UserQueryTest();
	IPMP_UserQueryResponseTest();
	IPMP_InitAuthenticationTest();
	IPMP_MutualAuthenticationTest();
	IPMP_SelectiveDecryptionInitCiTest();
	IPMP_SelectiveDecryptionInitCsTest();
	IPMP_AudioWatermarkingInitTest();
	IPMP_SendAudioWatermarkTest();
	IPMP_VideoWatermarkingInitTest();
	IPMP_SendVideoWatermarkTest();
	IPMP_ToolParamCapabilitiesQueryTest();
	IPMP_ToolParamCapabilitiesResponseTest();
	IPMP_ParametricDescriptionTest();
	IPMP_ToolES_AUTest();

	// testing inter-device message
	IPMP_MessageFromDeviceTest();
	IPMP_RequestContentTest();
	IPMP_ResponseToContentRequestTest(); 
	IPMP_ContentTransferTest();
	IPMP_RequestToolTest();
	IPMP_ResponseToToolRequestTest();
	IPMP_DeviceID_NotificationTest();
	
	printf("tests done = %d, failed = %d\n", counter, errcounter);
	return 0;
}



