//	This software module was originally developed by Craig A. Schultz 
//	(AccessTicket Systems Inc.)
//
//	in the course of development of the MPEG-4 Systems 
//	(ISO/IEC 14496-1) standard. This software module is an implementation 
//	of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified 
//	by the MPEG-4 Systems (ISO/IEC 14496-1) standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	AccessTicket Systems Inc. retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2000. 

#ifndef _CONTROLPOINTOBJECT_H_
#define _CONTROLPOINTOBJECT_H_

#include "Filter.h"
#include "IPMPX_Tool.h"
#include "MediaStream.h"

class SequencedTool
{
public:
	SDLInt<8> sequenceID;
	IPMPX_Tool * ptrTool;
	IPMPX_Tool * ptrNextTool;
};

class IPMPServicesFull;

class ControlPointObject : public Filter
						, public IPMPX_Tool
{
public:
	ControlPointObject();
	~ControlPointObject();

	// Filter interface methods.
	void Start ();

	void Stop ();

	void QueryInterface (
		const char* sTag,
		const void* pSink,
		void*& r_pSink )  {r_pSink = NULL;}

	const char* GetInfo() {return NULL;}

	virtual void SetInputStream (MediaStream *pStream)
	{
		m_pInStream = pStream;
	}

	void SetOutputStream (MediaStream *pStream)
	{
		m_pOutStream = pStream;
	}

	MediaStream *GetInputStream () {return m_pInStream;}

	MediaStream *GetOutputStream () {return m_pOutStream;}

	int GetOptimalOutputSize ( int inputSize ) { return inputSize; }

	// IPMPX_Tool interface methods
	// by Liao Xuan
	//bool ReceiveMessage( ToolMessage* ) {return false;}
	bool ReceiveMessage( IPMP_ToolMessageBase* ) {return false;}
	// by Liao Xuan

	bool ProcessData (
		LPBYTE,		// pInput,
		int,		// nInputLength,
		DWORD		// dwTime;
	);

	bool AddTool( IPMPX_Tool * ptrTool, SDLInt<8> sequenceID );
	bool RemoveTool( IPMPX_Tool * ptrTool );
	void SetControlPoint( SDLInt<8> controlPointCode )
	{	IPMP_ControlPointCode = controlPointCode;
	}
	SDLInt<8> GetControlPointCode()
	{	return IPMP_ControlPointCode;
	}

private: // Methods
	void Run ();
	void Terminate () {}
	bool IsActive () {return m_bActive;}
	int StartControlPointObjectThread () {return m_MyThread.Begin ();}
	void WaitForThread () {m_MyThread.Wait ();}

private:
	// Protected interface data.
	SDLInt<8> IPMP_ControlPointCode;
	MediaStream *m_pInStream, *m_pOutStream;
	IPMPX_Tool * m_pFirstTool;
	ZArray<SequencedTool*>	tools;
	bool m_bActive;
	ZMutex accessLock;
	ZThread<ControlPointObject> m_MyThread;

private:
	// Helper methods
	bool GetToolFromID( const SDLInt<8> sequenceID, SequencedTool * &ptrEntry, int & index );
	bool GetToolFromPtr( const IPMPX_Tool * ptrTool, SequencedTool * &ptrEntry, int & index  );
	bool GetConnectedToolFromID( const SDLInt<8> sequenceID, SequencedTool * &ptrEntry, int & index );
	bool GetConnectedToolFromPtr( const IPMPX_Tool * ptrTool, SequencedTool * &ptrEntry, int & index  );
	void SetUpTools();
	bool GetLastTool( SequencedTool * &ptrEntry );
	bool GetPreviousTool( const int currentSequenceID, SequencedTool * &ptrEntry  );
	void DisconnectAllTools();
};

#endif	//  _CONTROLPOINTOBJECT_H_

