//	This software module was originally developed by Craig A. Schultz 
//	(AccessTicket Systems Inc.)
//
//	in the course of development of the MPEG-4 Systems 
//	(ISO/IEC 14496-1) standard. This software module is an implementation 
//	of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified 
//	by the MPEG-4 Systems (ISO/IEC 14496-1) standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	AccessTicket Systems Inc. retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2000. 

#include "defs.h"
#include "IPMPXFull.h"
#include "IPMPXinterface.h"
#include "ControlPointObject.h"
#include "IPMP_ToolES_StreamDump.h"
#include "IPMP_StreamDump.h"

typedef IPMPX_Tool* (*CREATEFUNCPTR)();

/*-------------------------------------------------------------
The exported function that instantiates the proper IPMPManager object
-------------------------------------------------------------*/
extern "C" void *CreateInstance ()
{
	return new IPMPXFull;
}

/*-------------------------------------------------------------
Instantiate an IPMPServices object
@param szServiceURL the URL identifying this IPMP Service
@param toolList the list of tools contained in the inital Tool list
@return an IPMPServices object containing the Tool Manager and the Message Router
-------------------------------------------------------------*/
IPMPServices *IPMPXFull :: CreateIPMPServices (
	const char *szServiceURL,
	IPMP_ToolListDescriptor* toolList
)
{
	IPMPServices * service = new IPMPServicesFull;
	service->ReceiveIPMP_ToolListDescriptor(toolList);
	return ((IPMPServices*)service);
}

/*-------------------------------------------------------------
Delete an IPMPServices object
@param pIPMPServices the pointer to the IPMPServices to be removed
@return true if successful
-------------------------------------------------------------*/
bool IPMPXFull :: RemoveIPMPServices(IPMPServices *pIPMPServices)
{
	((IPMPServicesFull*)pIPMPServices)->Die();
	// by Liao Xuan
	if (pIPMPServices!=NULL)
	{
		delete pIPMPServices;
		pIPMPServices = NULL;
	}
	// by Liao Xuan
	return true;
}

IPMPServicesFull::IPMPServicesFull()
	: usedContextIDs		(0)
	, objectsCreated		(0)
{
	m_MessageDistributor.SetParent( this );

	/*************** Move by Liao Xuan **********************/
	/* Move to IPMPServicesFull::ProcessIPMPDescriptor()
	/* Should process tool inicialization after creating IPMPTool from Tool ES  */
	//Init_AvailableIPMPTools();
	/*************** End: Move by Liao Xuan *****************/
}

/**
 * Called by the MR to indicate a tool is no longer needed.
 * @param toolPtr the pointer to the tool to be disconnected
 * @return true if successful
 */
bool IPMPServicesFull::DisconnectTool( void* toolPtr)
{
	for ( int i = availableTools.GetLength(); --i >= 0; ){
		for ( int j = availableTools[i]->instancedTools.GetLength(); --j >= 0; ){
			//if ( availableTools[i]->instancedTools[j] == toolPtr ){
			if ( toolPtr !=NULL && availableTools[i]->instancedTools[j] == toolPtr){
				// by Liao Xuan, cause memory release errors, to do
				//delete (IPMPX_Tool*)toolPtr;
				//delete toolPtr;
				toolPtr = NULL;
				availableTools[i]->instancedTools[j] = NULL;

				objectsCreated--;
				availableTools[i]->instancedTools.DeleteFrom(j);
				return true;
			}
		}
	}
    return FALSE;
}

/**
 * Called by the MR to request a tool be instantiated.
 * @param OD_id the id of the Object Descriptor in which scope the tool will run
 * @param pESD a pointer to the ESD of the Elementary Stream protected by the tool
 * @param toolDescriptor the descriptor carrying initialization information for the tool
 * @return the pointer to the tool
 */
void* IPMPServicesFull::ConnectTool(ES_Descriptor *pESD, IPMP_Descriptor* descriptor)
{
    ZLargeInt<128>	nIPMPToolID;
	IPMPToolEntry*	toolEntry = NULL;
	int				toolEntryIndex(0);

	nIPMPToolID = descriptor->IPMP_ToolID.GetValue();

	if(!FindTool( descriptor->IPMP_ToolID, toolEntry, toolEntryIndex ))  //can not find the specified tool id
		return NULL;

	CREATEFUNCPTR CreateInstance = (CREATEFUNCPTR)::GetProcAddress(toolEntry->m_pIPMP_ToolDLL_Instance,"CreateInstance");
	if (CreateInstance!=NULL)
	{
		//now, call CreateInstace() to create an instance
		IPMPX_Tool* tool = NULL;
		tool = CreateInstance();
		objectsCreated++;
		toolEntry->instancedTools += tool;
		return (void*)tool;
	}
    
	return NULL;
}

/**
 * Called by the Terminal to pass a Tool elementary stream
 * @param tool_ES the interface to the media stream implemented in IM1
 * @param tool_ESD the relevant Elementary Stream Descriptor
 * @return true if successful
 */
bool IPMPServicesFull::ReceiveToolES(MediaStream * tool_ES, ES_Descriptor * tool_ESD)
{
	if( !tool_ES || !tool_ESD)
		return false;

	//Generation of saved file name (randomly generated)
	char nameTemplate[11] = "ipmpXXXXXX";
	std::string toolFileName;
	char *result;
	const char* ext = ".dll";
	
	result = _mktemp(nameTemplate);
	if(result == NULL){
		printf("Error in generating temp file name . \n");
		return false;
	}

	toolFileName = (std::string)(result);
	toolFileName.append(ext);
	//end of generation of saved file name
	
	IPMP_ToolES_StreamDump * toolESdump = new IPMP_ToolES_StreamDump( tool_ESD
																	, tool_ES
																	, this
																	, toolFileName );
	objectsCreated++;
	toolESdump->Start();
	toolESHandlers += toolESdump;

	return true;
}

/**
 * Called by the Terminal when a Tool elementary stream is removed
 * @param pESD a pointer to the relevant Elementary Stream Descriptor
 * @return true if successful
 */
bool IPMPServicesFull::RemoveToolES(ES_Descriptor *pESD)
{
	return true;
}

/**
 * Called by the Terminal to pass the tool list.
 * @param toolList the list of tools as defined in MPEG-4 IPMP Extension
 * @return true if successful
 */
bool IPMPServicesFull::ReceiveIPMP_ToolListDescriptor( IPMP_ToolListDescriptor* toolList )
{
	/****** Add by Liao Xuan***********************************/
	/* to show IPMP Tool List information         */
	char info[300];
	int j=0;
	j+= sprintf (info+j,"IPMP_ToolList Received: \n");
	/****** End: Add by Liao Xuan******************************/
	
	if ( toolList == NULL )
		return false;

	bool bErrorFlag;
	bErrorFlag = false;

	/****** Change by Liao Xuan********************************/
	/* to get the length of                                   */
	/* SDLClassArray<class IPMP_Tool, 0, 255> ipmpTools;      */
	//for ( int i = 0; i < toolList->GetLength(); i++ )
	for ( int i = 0; i < toolList->ipmpTools.GetLength(); i++ )
	/****** End: Change by Liao Xuan***************************/
	{   
		// if meet error, set flag, continue parse tools	
		if (! Parse_IPMPTool(toolList->ipmpTools[i]))
			bErrorFlag = true;

	/****** Add by Liao Xuan***********************************/
	/* to show IPMP Tool List Information information         */
		j+=sprintf(info+j,"Tool ID: %d,%d,%d,%d \n",
		(int)toolList->ipmpTools[i]->IPMP_ToolID.GetValue().GetWord(0),
		(int)toolList->ipmpTools[i]->IPMP_ToolID.GetValue().GetWord(1),
		(int)toolList->ipmpTools[i]->IPMP_ToolID.GetValue().GetWord(2),
		(int)toolList->ipmpTools[i]->IPMP_ToolID.GetValue().GetWord(3));
	}
    MessageBox(NULL,info,"IPMP Tool List Information",MB_OK);
	/****** End: Add by Liao Xuan******************************/
    return (! bErrorFlag);
}

/************************** Rewrite by Liao Xuan **************************/
	/**
	 * messaging interface.
	 * @param size the size of the message
	 * @param message one of the messages defined in MPEG-4 IPMP Extension, or a user-defined message
	 * @return true if successful
	 */
/* bool IPMPServicesFull::ReceiveMessage(ToolMessage* msg)
{	
	for ( int i = msg->IPMP_MessageFromBitstream.GetLength(); --i >= 0; )
	 	ProcessMessage( msg
					  , (IPMP_Data_BaseClass*)msg->IPMP_MessageFromBitstream[i] );

	return true;
}*/
/**
* messaging interface.
* @param size the size of the message
* @param message one of the messages defined in MPEG-4 IPMP Extension, or a user-defined message
* @return true if successful
*/
bool IPMPServicesFull::ReceiveMessage( IPMP_ToolMessageBase* msg )
{	
	switch (msg->tag)
	{
		case(0x01)://IPMP_MessageFromBitstream		
			return false;
		case(0x02)://IPMP_DescriptorFromBitstream
			return false;
		case(0x03)://IPMP_MessageFromTool
		{
			for (int i = ((IPMP_MessageFromTool*)msg)->messages.GetLength(); --i >= 0; )
				ProcessMessage( ((IPMP_MessageFromTool*)msg), (IPMP_Data_BaseClass*)((IPMP_MessageFromTool*)msg)->messages[i] );
			return true;
		}
	}
	return false;
}
/**************************End: Rewrite by Liao Xuan *********************/


/**
 * Called by the Terminal to process an OD
 * @param pOD pointer to the object descriptor to be processed by the MR
 * @return true if successful
 */
bool IPMPServicesFull::ProcessObjectDescriptor( ObjectDescriptor* pOD )
{
	/*******************************************************
	 *	1.  Cycle through ES_Descriptor and add to OD entry
	 *	2.  Add OD entry to "ZOD_Descriptors"
	 */

	ODEntry*					entryOD = NULL;
	ESDEntry*					entryES = NULL;
	IPMP_DescriptorEntry*		descriptorEntry;
	bool						foundESD(false);
	ZArray<__int16>				descriptorIDs;
	
	// Find ES_Descriptor
	// For now, just add the descriptor IDs and any tool descriptors 
	// that are identified at either the OD level or at the ESD level, 
	// later, during the processing of ESDs
	// memory pointers will be added.
	entryOD = new ODEntry;
	objectsCreated++;
	entryOD->OD_ID = pOD->ObjectDescriptorID;
	entryOD->ptrOD = pOD;

	for ( int i = pOD->esDescr.GetLength(); --i >= 0; )
	{	foundESD = true;
		entryES = new ESDEntry;
		objectsCreated++;
		entryES->ES_ID = pOD->esDescr[i]->ES_ID;
		entryES->OD_ID = entryOD->OD_ID;
		//Check for and fill in tool descriptor data.
		// At the OD level.
		for ( int j = pOD->ipmpDescrPtr.GetLength(); --j >= 0; )
		{	
			/*********** changed by Liao Xuan *******************************/
			/* should check if IPMP_DescriptorID == 0xff, instead of
			/* IPMP_DescriptorIDEx											*/
			//if ( pOD->ipmpDescrPtr[j]->IPMP_DescriptorIDEx == 0xff )
			if ( pOD->ipmpDescrPtr[j]->IPMP_DescriptorID == 0xff )
			/*********** changed by Liao Xuan *******************************/
			{	// First check to see if it has the same IPMP_DescriptorIDEx as one already received.
				// If so, just create an additional IPMPStreamRecipient and fill in the information
				// and add it to this object's m_IPMPStreamRecipients list.
				if ( descriptorIDs.Find( pOD->ipmpDescrPtr[j]->IPMP_DescriptorIDEx ) == -1 )
				{	// It is a new IPMP_DescriptorPointer
					descriptorIDs += pOD->ipmpDescrPtr[j]->IPMP_DescriptorIDEx;

					descriptorEntry = new IPMP_DescriptorEntry;

					descriptorEntry->IPMP_DescriptorIDEx = pOD->ipmpDescrPtr[j]->IPMP_DescriptorIDEx;
					descriptorEntry->referencedAt = 1;
					descriptorEntry->ipmpStreams += pOD->ipmpDescrPtr[j]->IPMP_ES_ID;
					entryES->descriptors += descriptorEntry;
				}
				else 
				{	// It is a duplicate and its IPMP_ES_ID needs to be added to the correct
					// descriptorEntry.
					for (int k = entryES->descriptors.GetLength(); --k >= 0; )
						if ( entryES->descriptors[k]->IPMP_DescriptorIDEx 
							== pOD->ipmpDescrPtr[j]->IPMP_DescriptorIDEx )
						{
							entryES->descriptors[k]->ipmpStreams += pOD->ipmpDescrPtr[j]->IPMP_ES_ID;
						}
				}
			}
			else
			{	// Its a "Hooks" descriptor pointer. Ignored for now.
			}
		}
		// Now at the ESD level.
		descriptorIDs.Clear();
		for ( int l = pOD->esDescr[i]->ipmpDescrPtr.GetLength(); --l >= 0; )
		{	ES_Descriptor * ptrESD = pOD->esDescr[i];
			if ( descriptorIDs.Find( pOD->esDescr[i]->ipmpDescrPtr[l]->IPMP_DescriptorIDEx ) == -1 )
			{	// It is a new IPMP_DescriptorPointer
				descriptorIDs += pOD->esDescr[i]->ipmpDescrPtr[l]->IPMP_DescriptorIDEx;

				descriptorEntry = new IPMP_DescriptorEntry;

				descriptorEntry->IPMP_DescriptorIDEx = pOD->esDescr[i]->ipmpDescrPtr[l]->IPMP_DescriptorIDEx;
				descriptorEntry->referencedAt = 0;
				descriptorEntry->ipmpStreams += pOD->esDescr[i]->ipmpDescrPtr[l]->IPMP_ES_ID;
				entryES->descriptors += descriptorEntry;
			}
			else
			{	// It is a duplicate and its IPMP_ES_ID needs to be added to the correct
				// descriptorEntry.
				for (int m = entryES->descriptors.GetLength(); --m >= 0; )
					if ( entryES->descriptors[m]->IPMP_DescriptorIDEx 
						== pOD->ipmpDescrPtr[j]->IPMP_DescriptorIDEx )
					{
						entryES->descriptors[m]->ipmpStreams += pOD->esDescr[i]->ipmpDescrPtr[j]->IPMP_ES_ID;
					}
			}
		}
		// Add entryESD to entryOD's entryES array.
		entryOD->entryES += entryES;
	}
	if ( !foundESD )
		return false;

	ZOD_Descriptors += entryOD;

	return true;
}

/**
* Called by the Terminal to inform the MR that an OD is not valid anymore.
* @param pOD pointer to the OD to be removed
* @return true if successful
*/
bool IPMPServicesFull::RemoveObjectDescriptor( ObjectDescriptor* pOD )
{
	/*******************************************************
	 *  1. Find in and remove from ZOD_Descriptors list.
	 */
	ODEntry* entry = NULL;
	int index;
	if ( GiveMeODFromPtr( pOD, entry,index ) )
	{	
		ShutDownODtools( entry );
		if(entry!=NULL)
		{
			delete entry;
			entry = NULL;
		}
		objectsCreated--;
		ZOD_Descriptors.DeleteFrom(index);
		return true;
	}

	return false;
}

	/**
	 * Called by the Terminal to process an IPMPToolDescriptor.
	 * @param descriptor the IPMPToolDescriptor to be processed by the MR
	 * @return true if successful
	 * NOTE: this descriptor object will be deleted after calling this method. The IPMPX
	 * system shall take this into account and copy it into its own memory space if necessary
	 */
bool
IPMPServicesFull::ProcessIPMPDescriptor(IPMP_Descriptor* descriptor)
{
	/************** Add by Liao Xuan, to show IPMP information ******/
	char info[300];
	sprintf (
		info,
		"\n\nIPMP_Descriptor Handled: \nDescriptor ID: %d, \nDescriptor IDEx: %d, \nTool_ID: %d,%d,%d,%d, \nControlPoint: %d, \nSequenceCode: %d, \nURL: %s, \nIPMPData: %s\n",
		(int)descriptor->IPMP_DescriptorID,
		(int)descriptor->IPMP_DescriptorIDEx,
		(int)descriptor->IPMP_ToolID.GetValue().GetWord(0),
		(int)descriptor->IPMP_ToolID.GetValue().GetWord(1),
		(int)descriptor->IPMP_ToolID.GetValue().GetWord(2),
		(int)descriptor->IPMP_ToolID.GetValue().GetWord(3),
		(int)descriptor->controlPointCode,
		(int)descriptor->sequenceCode,
		(LPCSTR)descriptor->URLString,
		(LPCSTR)descriptor->IPMP_data
	);
	MessageBox(NULL,info,"IPMP Descriptor Information",MB_OK);
	/***********End: Add by Liao Xuan, to show IPMP information *****/

	/**************** Move by Liao Xuan *****************************/
	/* Move from IPMPServicesFull::IPMPServicesFull()
	/* Should process tool inicialization after creating IPMPTool from Tool ES  */	
	Init_AvailableIPMPTools();
	//MessageBox(NULL, "All IPMP tools initialized !", "IPMP Tool Initialization", MB_OK);
	/**************** End: Move by Liao Xuan ************************/
	/*******************************************************
	 *  1. Check if pointer is NULL.
	 *  2. Add pointer to Descriptors.
	 *  3. Check "Is_Initialize", if not set deliver to required tool.
	 */
	if (descriptor == NULL)
		return false;

	IPMP_DescriptorEntry entry( descriptor->IPMP_DescriptorIDEx
							  , descriptor );
	SDLInt<16>			descriptorIDEx;
	IPMP_Descriptor*	ptrDescr = NULL;
	int					index;

	GiveMeDescriptorFromID( descriptor->IPMP_DescriptorIDEx, ptrDescr, index );
	if ( ptrDescr != NULL )
	{	// A descriptor with the same ID exists, replace it.
		ZDescriptors.DeleteFrom(index);
		ZDescriptors += entry;

		// Deliver to indicated tools.
		// If the tool isn't in the address list no descriptor will be sent.
		
		/************ Change by Liao Xuan ************************/
		IPMP_DescriptorFromBitstream  msg;
		msg.sender = 0x00;
		//msg.IPMP_DescriptorFromBitstream += entry.ptrIPMP_Descriptor;
		msg.toolDescriptor += entry.ptrIPMP_Descriptor;
		//msg.toolDescriptor = entry.ptrIPMP_Descriptor;
		/************ End: Change by Liao Xuan *******************/

		for (int m = ZAddressList.GetLength(); --m >= 0; )
		{	if ( ZAddressList[m].IPMP_ToolID.GetValue() == descriptor->IPMP_ToolID.GetValue() )
			{	msg.receiver = ZAddressList[m].contextID;
				((IPMPTool_Interface*)ZAddressList[m].ptrTool)->ReceiveMessage(&msg);
			}
		}
	}
	else
	{	
		// Add the descriptor
		ZDescriptors += entry;
		// Find entryESs that need the tool descriptor.
		for ( int i = ZOD_Descriptors.GetLength(); --i >= 0; )
		{	for ( int j = ZOD_Descriptors[i]->entryES.GetLength(); --j >= 0; )
			{	for ( int k = ZOD_Descriptors[i]->entryES[j]->descriptors.GetLength(); --k >= 0; )
				{	if ( descriptor->IPMP_DescriptorIDEx == ZOD_Descriptors[i]->entryES[j]->descriptors[k]->IPMP_DescriptorIDEx )
					{	ZOD_Descriptors[i]->entryES[j]->descriptors[k]->ptrIPMP_Descriptor = descriptor;
						if ( ZOD_Descriptors[i]->entryES[j]->ptrESD != NULL )
						{	// The tool descriptor is needed and the ESD needing it has already been processed.
							// If the ESD needing it has not yet been processed, the Terminal will not know what
							// to do with the tools it may receive.
							ZOD_Descriptors[i]->entryES[j]->descriptors[k]->ptrIPMP_Descriptor = descriptor;
							if ( !InstanceTool( ZOD_Descriptors[i]->entryES[j]->descriptors[k]
											  , ZOD_Descriptors[i]->entryES[j] ) )
							{	// This may be redundant or not needed but is done just in case.
								ZOD_Descriptors[i]->entryES[j]->ptrESD->GoNoGo(false);
							}
						}
					}
				}
			}
		}
	}
	return true;
}

/**
 * Called by the Terminal to inform the MR that an IPMPToolDescriptor is not valid anymore.
 * @param descriptor pointer to the IPMPToolDescriptor to be removed
 * @return true if successful
 */
bool IPMPServicesFull::RemoveIPMPDescriptor(IPMP_Descriptor* descriptor)
{
	/************************************************************
	 *  This method seems to be not needed.  It is not yet known
	 *	what action to take when a tool descriptor is removed.
	 */
	SDLInt<16> ID;
	int index;
	if ( GiveMeDescriptorFromPtr( descriptor, ID, index ) )
	{	
		ZDescriptors.DeleteFrom(index);
		return true;
	}
	return false;
}

/**
 * Called by the Terminal to process an Elementary Stream Descriptor.
 * @param pESD pointer to the Elementary Stream Descriptor to be processed by the MR
 * @return true if successful
 */
enum MR_Interface::ACCESS_PERMISSION
IPMPServicesFull::ProcessESDescriptor(ES_Descriptor* pESD)
{
	/*******************************************************
	 *	1. Fill in OD information
	 *		1. Get ESDEntry with same ID.
	 *		2. Insert pESD into ES_Entry.
	 *		3. Check for IPMP_Descriptor pointers in the ESD.
	 *			If found ;
	 *				1. Cycle through ESD IPMP_Descriptor pointers.
	 *					Create filters connect their "ReceiveMessage" interfaces
	 *					and wait for them to all report to go ahead with processing
	 *					before actually passing any of the filters to the Terminal.
	 *					NOTE : If not all tool Descriptors are found, return "DENIED".
	 *				2. Return "PROTECTED".
	 *	2.	If tool descriptors were not found,
	 *		return "ALLOWED".
	 *
	 *	NOTE : If IPMP_Descriptor has not yet been received, return PROTECTED and then 
	 *			continue processing when IPMP_Descriptor is received.
	 */
	
	ESDEntry*					entryESD = NULL;
	int							index(0);
	bool						descriptorsFound(false);

	if ( !GiveMeESDFromID( pESD->ES_ID, entryESD,index ) )
		return DENIED;
	// Fill in the ESD memory pointer.
	entryESD->ptrESD = pESD;

	// Check for protection of this ES.
	if ( entryESD->descriptors.GetLength() )
	{	descriptorsFound = true;
		for (int i = entryESD->descriptors.GetLength(); --i >= 0;)
		{	if ( entryESD->descriptors[i]->ptrIPMP_Descriptor != NULL )
			{	// It has already been received.

				// Go ahead and instantiate the tool because it will not have happened until
				// the ESD is processed even if the tool descriptor was found ealier.
				if ( !InstanceTool( entryESD->descriptors[i]
								  , entryESD ) )
					return DENIED;
			}
			else
			{	// Look for it anyway because the OD this ESD was found in may have been 
				// delivered after the tool descriptor was.
				for ( int j = ZDescriptors.GetLength(); --j >= 0; )
				{	if ( entryESD->descriptors[i]->IPMP_DescriptorIDEx
						== ZDescriptors[j].IPMP_DescriptorIDEx )
					{	entryESD->descriptors[i]->ptrIPMP_Descriptor = ZDescriptors[j].ptrIPMP_Descriptor;
						// Get tool and initialize its interface.
						if ( !InstanceTool( entryESD->descriptors[i]
										  , entryESD ) )
							return DENIED;
					}
				}
			}
		}
	}

	if ( !descriptorsFound )
		return ALLOWED;

	return PROTECTED;
}
	
/**
 * Called by the Terminal to inform the MR that an ESDescriptor is not valid anymore.
 * @param pESD pointer to the Elementary Stream Descriptor to be removed 
 * @return true if successful
 */
bool
IPMPServicesFull::RemoveESDescriptor( ES_Descriptor* pESD )
{
	/*******************************************************
	 *  1. Get ESD entry.
	 *	2. Shut down ESD.
	 *	3. Get OD entry and remove ESD entry.
	 */

	ESDEntry*	entryESD = NULL;
	int			indexESDEntry(0);
	ODEntry*	entryOD = NULL;
	int			indexODEntry(0);

	if ( !GiveMeESDFromPtr( pESD, entryESD, indexESDEntry ) )
		return false;

	ShutDownEStools( entryESD );

	// Remove the ESD entry from the OD entry.
	GiveMeODFromID( entryESD->OD_ID, entryOD, indexODEntry );
	entryOD->entryES.DeleteFrom(indexESDEntry);
	
	/**** Add by Liao Xuan ********/
	if(entryESD!=NULL)
	{
		delete entryESD;
		entryESD = NULL;
	}
	if (entryOD)
	{
		delete entryOD;
		entryOD = NULL;
	}
	/**** End: Add by Liao Xuan ***/

	objectsCreated--;

	return true;
}

/**
 * Called by the Terminal to create a sink for an IPMP stream.
 * @param pOD a pointer to the Object Descriptor where the ipmp stream is declared
 * @param pESD a pointer to the relevant Elementary Stream Descriptor
 * @param ipmpStream the MediaStream object handling the IPMP stream
 * @return true if successful
 */
bool IPMPServicesFull::SetUpIPMPStream(ObjectDescriptor *pOD, ES_Descriptor* pESD, MediaStream* ipmpStream)
{	
	IPMP_StreamDump * pStrmDmp = new IPMP_StreamDump ( pOD
													 , pESD
													 , ipmpStream
													 , this );
	objectsCreated++;

	IPMPStreamDumpEntry entry;
	entry.pOD = pOD;
	entry.pESD = pESD;
	entry.ipmpStream = ipmpStream;
	entry.ipmpStreamDump = pStrmDmp;

	ZIPMPStreamDumps += entry;

	pStrmDmp->Start();

	return true;
}

/**
 * Called by the Terminal to destroy a sink for an IPMP stream, that was previously created by the Terminal
 * using the SetUpIPMPStream method.
 * @param pESD a pointer to the relevant Elementary Stream Descriptor
 * @return true if successful
 */
bool IPMPServicesFull::RemoveIPMPStream(ES_Descriptor *pESD)
{
	// Find Stream dump
	for ( int i = ZIPMPStreamDumps.GetLength(); --i >= 0; )
	{	if ( ZIPMPStreamDumps[i].pESD = pESD )
		{	
			ZIPMPStreamDumps[i].ipmpStreamDump->Stop();
			/**** Add by Liao Xuan ********/
			if(ZIPMPStreamDumps[i].ipmpStreamDump!=NULL)
			{
				delete ZIPMPStreamDumps[i].ipmpStreamDump;
				ZIPMPStreamDumps[i].ipmpStreamDump = NULL;
			}
			/**** End: Add by Liao Xuan ***/
			objectsCreated--;
			ZIPMPStreamDumps.DeleteFrom(i);
			return true;
		}
	}
	return false;
}

//read tool information from windows registry, to initialize toollistmap.
//see ToolManager.reg for details.
//windows (terminal) should pre-store all tools information including toolID and location
//into windows registry.
bool IPMPServicesFull::Init_AvailableIPMPTools()
{
    struct _finddata_t c_file;
    long hFile;

    /* Find first .c file in current directory */

    if( (hFile = _findfirst( "*.dll", &c_file )) == -1L ){
       printf( "No tool files in current directory!\n" );
	   return false;
	}
	else
	{
		CheckDllForTool( c_file );
	}
	// Search through the rest of the dll files in the directory.
	while ( _findnext( hFile, &c_file ) == 0 ){
		CheckDllForTool( c_file );
	}
    return true;
}

void
IPMPServicesFull::CheckDllForTool( _finddata_t &fileInfo )
{
	int file(0), result(0), numRead(0), amountProcessed(0), IDSize(0), intSize(0);
	unsigned char ipmpToolTag[8], sizeData[2];

	// Read the dll file looking for "IPMPTOOL"
	// in the last 8 bytes of the dll file.
	file = _open( fileInfo.name, _O_BINARY | _O_RDONLY, _S_IREAD );
	if( file ==-1 )
		printf( "dll file could not be opened for reading!\n" );
	else{
		result = _lseek( file, -8L, SEEK_END);

		numRead = _read( file, (void*)&ipmpToolTag, 8 );
		if ( !memcmp( ipmpToolTag
					, "IPMPTOOL"
					, 8 ) ){
		// Its an IPMP tool!
		// Remaining data is as follows: From end to beginning so that the certificates
		// are included in the signature as well.
		// 1. Size of all ID data		[2 bytes]  Note : Minus this size plus tool tag.
		// 2. "SIGN" tag
		// 3. signatureSize.			[2 bytes]
		// 4. Signature. [signatureSize bytes]  Signature applies to all preceeding
		//		data including certificates, the Tool ID or parametric description
		//		as well as the tool body itself.
		// 5. "CERT" tag
		// 6. Number of Certificates.	[2 bytes]
		// 7. Size of Certificate.		[2 bytes]
		// 8. Certificate				[size of Certificate]
		//	Note :  from #9 onward,the listed data is found in sequence from the actual end
		//			of the original dll.
		// The following, 9, 10 and 11 are for tools identified by a 128 bit tool ID.
		// 9. "TLID" tag
		// 10. sizeOfID [2 bytes]
		// 11. Tool ID [sizeOfID bytes]
		// The following, 12, 13 and 14 are for tools identified parametrically.
		// 12. "PARA" tag
		// 13. numOfClasses [2 bytes]
		// 14. Parametric description(s).
		//		a. "CLSS" tag
		//		b. sizeOfClass		[2 bytes]
		//		c. Class			[sizeOfClass bytes]
		//		d. "SBCL" tag
		//		e. sizeOfSubClass	[2 bytes]
		//		f. Sub Class		[sizeOfSubClass bytes]
		//		g. "DATA" tag
		//		h. sizeOfData		[2 bytes]
		//		i. Type Data		[sizeOfData bytes]
		//		j. "ADDD" tag
		//		k. numOfAddData		[2 bytes]
		//		l. sizeOfAddData	[2 bytes]
		//		m. additional data	[sizeOfAddData bytes]
			result = _lseek( file, -10L, SEEK_END);

			numRead = _read( file, (void*)&sizeData, 2 );
			memcpy( &IDSize
				  , sizeData
				  , 2 );

			result = _lseek( file, -(IDSize + 2 + 8), SEEK_END);
			
			unsigned char * readData = new unsigned char[IDSize];
			objectsCreated++;
			numRead = _read( file, (void*)readData, IDSize );

			// Check for signature
			if ( !memcmp( readData + IDSize - 4
						, "SIGN"
						, 4 ) ){
				// Contains signature.
				amountProcessed += 4;
				memcpy ( &intSize
					   , readData + IDSize - amountProcessed
					   , 2 );
				amountProcessed += 2;
				// To Do : The completion of the signature verification.
			}
			// Find the Tool ID or parametric description.
			if ( ! memcmp( readData
						 , "TLID"
						 , 4 ) )
			{
				// Contains a tool ID
				amountProcessed = 4;
				memcpy ( &intSize
					   , readData + amountProcessed
					   , 2 );
				amountProcessed += 2;
				unsigned char* toolIdData = new unsigned char[intSize + 1];
				objectsCreated++;
				memcpy ( toolIdData
					   , readData + amountProcessed
					   , intSize );
				amountProcessed += intSize;

				*(toolIdData + intSize) = '\0';

				char value[20];
				char* curPos;
				int lengthParsed = 0;
				ZLargeInt<128> tmpToolID(0,0);
				//the tool ID is stored as xxx.xxx.xxx.xxx
				for (int i =0 ; i<4 ; i++)
				{
					strcpy(value, (char*)toolIdData + lengthParsed);
					curPos = strstr(value, ".");
					if (curPos!=NULL)
					{
						*curPos = '\0';
						lengthParsed += curPos - value + 1;
						tmpToolID <<= i*32;
						tmpToolID += atol(value);
					}
					else
					{
						tmpToolID <<= i*32;
						tmpToolID += atol(value);
						break;
					}
				}
				char info1[200];
				sprintf(info1,"IPMP Tool %d,%d,%d,%d found",
					tmpToolID.GetWord(0), 
					tmpToolID.GetWord(1), 
					tmpToolID.GetWord(2), 
					tmpToolID.GetWord(3));
				//MessageBox(NULL, info1, "IPMP", MB_OK);

				//by Liao Xuan
				if(toolIdData!=NULL)
				{
					delete[] toolIdData;
					toolIdData = NULL;
				}
				// by Liao Xuan
				objectsCreated--;

				// Create an IPMPToolEntry object and fill it in
				// then add it to the availableTools array.
				IPMPToolEntry * toolEntry = new IPMPToolEntry;
				objectsCreated++;
				toolEntry->m_nPhysical_IPMPToolID.InsertElement(tmpToolID);
				toolEntry->m_pIPMP_ToolDLL_Instance = LoadLibrary( fileInfo.name );
				availableTools += toolEntry;
			}
			else{
				// Must contain a parametric description but for now, not processed.
			}
			//by liao Xuan
			if (readData!=NULL)
			{
				delete[] readData;
				readData = NULL;
			}
			//by liao Xuan
			objectsCreated--;
		}
	}
}

bool IPMPServicesFull::DestroyLists(){

	for ( int i = availableTools.GetLength(); --i >= 0; )
	{	
		// add by Liao Xuan
		if(availableTools[i]!=NULL)
		{
			delete availableTools[i];
			availableTools[i] = NULL;
		}
		// add by Liao Xuan
		objectsCreated--;
		availableTools.DeleteFrom(i);
	}

	for ( int l = toolESHandlers.GetLength(); --l >= 0; )
	{	
		// add by Liao Xuan
		if(toolESHandlers[l]!=NULL)
		{
			delete toolESHandlers[l];
			toolESHandlers[l] = NULL;
		}
		// add by Liao Xuan
		objectsCreated--;
		toolESHandlers.DeleteFrom(l);
	}

	for ( int j = ZRequestedDescriptors.GetLength(); --j >= 0; )
	{	for ( int k = ZDescriptors.GetLength(); --k >= 0; )
		{	
			if ( ZDescriptors[k].IPMP_DescriptorIDEx == ZRequestedDescriptors[j]->IPMP_DescriptorIDEx )
			 	ZDescriptors.DeleteFrom(k);
		}
		// add by Liao Xuan
		if(ZRequestedDescriptors[j]!=NULL)
		{
			delete ZRequestedDescriptors[j];
			ZRequestedDescriptors[j]= NULL;
		}
		// add by Liao Xuan

		ZRequestedDescriptors.DeleteFrom(j);
	}

	for ( int k = ZIPMPStreamDumps.GetLength(); --k >=0; )
	{	
		//by liao xuan
		if (ZIPMPStreamDumps[k].ipmpStreamDump!=NULL)
		{
			delete ZIPMPStreamDumps[k].ipmpStreamDump;
			ZIPMPStreamDumps[k].ipmpStreamDump = NULL;
		}
		//add by liao xuan

		objectsCreated--;
		ZIPMPStreamDumps.DeleteFrom(k);
	}

	return true;
}

bool IPMPServicesFull::Parse_IPMPTool(IPMP_Tool* pIPMPTool)
{
	int i, nAltNum;
	SDLLongInt<128> nAltIPMPToolID;
	bool bFindAltFlag;
	IPMPToolEntry* toolEntry = NULL;
	int toolEntryIndex(0);
	   
	//if isAltGroup is specified
	if((int)pIPMPTool->isAltGroup ){
	  
		bFindAltFlag = false;
		nAltNum = (int)pIPMPTool->numAlternates;
		i = 0;
		//we find the first alternate tool occured in the availableTools 
		while(i<nAltNum && !bFindAltFlag){
	      
			nAltIPMPToolID.InsertElement(int(pIPMPTool->Alt_IPMP_ToolIDs.GetElement(i)));
			i++;

		    if( FindTool( nAltIPMPToolID, toolEntry, toolEntryIndex ) )
			{
				toolEntry->m_nPresentation_IPMPToolID = pIPMPTool->IPMP_ToolID;
			    bFindAltFlag = true;
				break;
			}
		}

		if(!bFindAltFlag) 
			goto toolNotFound;

		return true;
	}
    
	if((int)pIPMPTool->isParametric){
		//TODO
		return false;
	}

	// neither alternative nor parametric, check wehther the tool exists in m_ToolObjMap
	// remember, m_ToolObjMap is initialized upon Startup by reading the registry
	if( FindTool( pIPMPTool->IPMP_ToolID, toolEntry, toolEntryIndex ) )
	{
		return true;
	}
	else //not found, retrieve it now.
	{
		if (pIPMPTool->ToolURLs.GetLength()>0)
		{
			//for testing purpose, I only use the first URL provided
			bool retreivalResult = false;
			int i=0;
			while (i<pIPMPTool->ToolURLs.GetLength())
			{
				retreivalResult = RetrieveMissingTool(pIPMPTool->IPMP_ToolID.GetValue(), (LPCSTR)pIPMPTool->ToolURLs[i++]);
				if (retreivalResult)
				{
					return true;
				}
			}
			goto toolNotFound;
			//m_IPMPToolObjMap.insert(TOOLLISTMAP::value_type(toolObj.m_nPresentation_IPMPToolID, toolObj));
		}
		else
			goto toolNotFound;

	}

toolNotFound:
	char tmp[200];
	sprintf(tmp,"Sorry, tool with ID %d,%d,%d,%d is not found in the local IPMP terminal",
		(int)pIPMPTool->IPMP_ToolID.GetValue().GetWord(0),
		(int)pIPMPTool->IPMP_ToolID.GetValue().GetWord(1),
		(int)pIPMPTool->IPMP_ToolID.GetValue().GetWord(2),
		(int)pIPMPTool->IPMP_ToolID.GetValue().GetWord(3)
		);
//	MessageBox(NULL,tmp,"Notice!",MB_OK);
	return false;
}


bool
IPMPServicesFull::RetrieveMissingTool (ZLargeInt<128> tool_ID, const char* Location)
{
	//put your code here to retrieve missing tools
	//remember to add retrieved result to m_IPMPToolObjMap!!!
	char info[200];
	sprintf(info,"Retrieving missing tool with ID %d,%d,%d,%d from %s",
		tool_ID.GetWord(0), 
		tool_ID.GetWord(1), 
		tool_ID.GetWord(2), 
		tool_ID.GetWord(3), 
		Location);
//	MessageBox(NULL,info,"missing tools",MB_OK);
	return false;
}

bool IPMPServicesFull::FindTool( const SDLLongInt<128> actualID, IPMPToolEntry* &toolEntry, int &index )
{
/*	char info1[200];
	sprintf(info1,"%d,%d,%d,%d",
		actualID.GetWord(0), 
		actualID.GetWord(1), 
		actualID.GetWord(2), 
		actualID.GetWord(3));
*/
	for ( int i = availableTools.GetLength(); --i >= 0; ){
/*		char info2[200];
		sprintf(info2,"%d,%d,%d,%d",
			availableTools[i]->m_nPhysical_IPMPToolID.GetWord(0), 
			availableTools[i]->m_nPhysical_IPMPToolID.GetWord(1), 
			availableTools[i]->m_nPhysical_IPMPToolID.GetWord(2), 
			availableTools[i]->m_nPhysical_IPMPToolID.GetWord(3));
*/
		SDLLongInt<128> second = availableTools[i]->m_nPhysical_IPMPToolID;
		if ( availableTools[i]->m_nPhysical_IPMPToolID.GetValue() == actualID.GetValue() ){
			toolEntry = availableTools[i];
			index = i;
			return true;
		}
	}
	return false;
}

bool
IPMPServicesFull::InstanceTool( IPMP_DescriptorEntry*	descriptorEntry
							  , ESDEntry*					entryESD)
{	// Create entry container for tool and set it's parent ESD ID.
	// Then add tool descriptor pointer to existing tool descriptor entry in the entryES.
	// Get tool and initialize its interface.
	IPMPX_Tool	*	tool = NULL;

	tool = (IPMPX_Tool*)ConnectTool( NULL, descriptorEntry->ptrIPMP_Descriptor );

	if ( tool == NULL )
	{	// Shut down all the tools for this ESD and return false.
		ShutDownEStools ( entryESD );
		return false;
	}

	ToolEntry	*	entryTool = new ToolEntry;
	objectsCreated++;

	entryTool->address.ptrTool = tool;
	entryTool->address.contextID = ++usedContextIDs;
	entryTool->address.descriptorIDEx = descriptorEntry->IPMP_DescriptorIDEx;
	entryTool->ES_ID = entryESD->ES_ID;
	entryTool->address.IPMP_ToolID = descriptorEntry->ptrIPMP_Descriptor->IPMP_ToolID;
	entryTool->toolDescrEntry = descriptorEntry;

	entryESD->tools += entryTool;

	ZAddressList += entryTool->address;

	// Initialize its interface by sending it a message with its contextID.
	User_Initialize payload;

	/********** Change by Liao Xuan **************************/
	//ToolMessage msg;
	IPMP_MessageFromTool msg;

	payload.messageInterface.InsertElement((int)(IPMPTool_Interface*)&m_MessageDistributor);
	payload.toolDescriptor.InsertElement((int)descriptorEntry->ptrIPMP_Descriptor);
	//msg.IPMP_MessageFromBitstream += &payload;
	msg.messages += &payload;
	/********** End: Change by Liao Xuan *********************/

	msg.sender = 0;
	msg.receiver = usedContextIDs;
	m_MessageDistributor.ReceiveMessage(&msg);

	return true;
}

bool
IPMPServicesFull::ShutDownEStools( ESDEntry*  entryESD ){
	// First, shut down ControlPointObjects
	for ( int i = entryESD->controlPointObjects.GetLength(); --i >= 0; )
	{	//entryESD->controlPointObjects[i]->Stop();
	 	entryESD->ptrESD->ReleaseFilter( entryESD->controlPointObjects[i] );

		// add by liao xuan
		if(entryESD->controlPointObjects[i])
		{
			delete entryESD->controlPointObjects[i];
			entryESD->controlPointObjects[i] = NULL;
		}
		// add by Liao Xuan

		entryESD->controlPointObjects.DeleteFrom(i);
	}
	for ( int j = entryESD->tools.GetLength(); --j >=0; )
	{	DisconnectTool( entryESD->tools[j]->address.ptrTool );
		ZAddressList -= entryESD->tools[j]->address;
		
		// add by Liao Xuan
		if (entryESD->tools[j]!=NULL)
		{
			delete entryESD->tools[j];
			entryESD->tools[j]= NULL;
		}
		// add by Liao Xuan

		objectsCreated--;
		entryESD->tools.DeleteFrom(j);
	}
	// Delete entryESD's Descriptors
	for ( int k = entryESD->descriptors.GetLength(); --k >= 0; )
		entryESD->descriptors.DeleteFrom(k);

	return true;
}

bool
IPMPServicesFull::ShutDownODtools( ODEntry * entryOD ){
	for ( int i = entryOD->entryES.GetLength(); --i >= 0; )
	{	
		for ( int j = entryOD->entryES[i]->tools.GetLength(); --j >= 0; )
			ShutDownEStools( entryOD->entryES[i] );
		// add by liao xuan
		if(entryOD->entryES[i]!=NULL)
		{
			delete entryOD->entryES[i];
			entryOD->entryES[i]= NULL;
		}
		// add by liao xuan
		objectsCreated--;
		entryOD->entryES.DeleteFrom(i);
	}

	return true;
}

bool IPMPServicesFull::GiveMeODFromID( SDLInt<10>& OD_ID
								, ODEntry* &returnEntry
								, int& index )
{	for ( int i = ZOD_Descriptors.GetLength(); --i >= 0; )
		if ( ZOD_Descriptors[i]->OD_ID == OD_ID )
		{	returnEntry = ZOD_Descriptors[i];
			index = i;
			return true;
		}
	return false;
}

bool 
IPMPServicesFull::GiveMeODFromPtr( const ObjectDescriptor * ptrOD
									   , ODEntry* &returnEntry
									   , int& index )
{	for ( int i = ZOD_Descriptors.GetLength(); --i >= 0; )
		if ( ZOD_Descriptors[i]->ptrOD == ptrOD )
		{	returnEntry = ZOD_Descriptors[i];
			index = i;
			return true;
		}
	return false;
}

bool IPMPServicesFull::GiveMeESDFromID( SDLInt<16>& ES_ID
								 , ESDEntry* &returnEntry
								 , int& index )
{	for ( int i = ZOD_Descriptors.GetLength(); --i >= 0; )
	 	for ( int j = ZOD_Descriptors[i]->entryES.GetLength(); --j >= 0; )
		 	if ( ZOD_Descriptors[i]->entryES[j]->ES_ID == ES_ID )
			{	returnEntry = ZOD_Descriptors[i]->entryES[j];
				index = j;
				return true;
			}
	return false;
}

bool IPMPServicesFull::GiveMeESDFromPtr( const ES_Descriptor* ptrESD
								  , ESDEntry* &returnEntry
								  , int& index )
{	
	ES_Descriptor* ptrTest = NULL;

	for ( int i = ZOD_Descriptors.GetLength(); --i >= 0; )
	 	for ( int j = ZOD_Descriptors[i]->entryES.GetLength(); --j >= 0; )
		{ 	ptrTest = ZOD_Descriptors[i]->entryES[j]->ptrESD;
			if ( ptrTest == ptrESD )
			{	returnEntry = ZOD_Descriptors[i]->entryES[j];
				index = j;
				return true;
			}
		}

	return false;
}

bool IPMPServicesFull::GiveMeFilterFromID( SDLInt<32>& contextID
										, ToolEntry* &returnEntry
										, int& index )
{	for ( int i = ZOD_Descriptors.GetLength(); --i >= 0; )
		for ( int j = ZOD_Descriptors[i]->entryES.GetLength(); --j >= 0; )
			for ( int k = ZOD_Descriptors[i]->entryES[j]->tools.GetLength(); --k >= 0; )
				if ( ZOD_Descriptors[i]->entryES[j]->tools[k]->address.contextID == contextID )
				{	returnEntry = ZOD_Descriptors[i]->entryES[j]->tools[k];
					index = k;
					return true;
				}
	return false;
}

bool IPMPServicesFull::GiveMeFilterFromPtr( const IPMPX_Tool* ptrTool
										, ToolEntry* &returnEntry
										, int& index )
{
	for ( int i = ZOD_Descriptors.GetLength(); --i >= 0; )
	 	for ( int j = ZOD_Descriptors[i]->entryES.GetLength(); --j >= 0; )
		 	for ( int k = ZOD_Descriptors[i]->entryES[j]->tools.GetLength(); --k >= 0; )
			 	if ( ZOD_Descriptors[i]->entryES[j]->tools[k]->address.ptrTool == ptrTool )
				{	returnEntry = ZOD_Descriptors[i]->entryES[j]->tools[k];
					index = k;
					return true;
				}
	return false;
}

bool IPMPServicesFull::GiveMeDescriptorFromID( SDLInt<16>& descriptorIDEx
											, IPMP_Descriptor* &ptrDescr
											, int& index )
{
	for ( int i = ZDescriptors.GetLength(); --i >= 0; )
		if ( ZDescriptors[i].IPMP_DescriptorIDEx == descriptorIDEx )
		{	ptrDescr = ZDescriptors[i].ptrIPMP_Descriptor;
			index = i;
			return true;
		}
	return false;
}

bool IPMPServicesFull::GiveMeDescriptorFromPtr( const IPMP_Descriptor* ptrDescr
											, SDLInt<16>& descriptorIDEx
											, int& index )
{
	for ( int i = ZDescriptors.GetLength(); --i >= 0; )
		if ( ZDescriptors[i].ptrIPMP_Descriptor == ptrDescr )
		{	descriptorIDEx = ZDescriptors[i].IPMP_DescriptorIDEx;
			index = i;
			return true;
		}
	return false;
}

void IPMPServicesFull::GetControlPointObject( ESDEntry* entryESD
										 , const SDLInt<8> controlPoint
										 , ControlPointObject* &controlPointObject )
{	for ( int i = entryESD->controlPointObjects.GetLength(); --i >= 0; )
	{	if ( entryESD->controlPointObjects[i]->GetControlPointCode() == controlPoint )
		{	controlPointObject = entryESD->controlPointObjects[i];
			return;
		}
	}

	// One was not found so create one.
	ControlPointObject * newObject = new ControlPointObject;
	newObject->SetControlPoint(controlPoint);
	entryESD->controlPointObjects += newObject;

	controlPointObject = newObject;
}

bool IPMPServicesFull::GiveMeControlPointObject( ESDEntry* entryESD
											, const SDLInt<8> controlPoint
											, ControlPointObject* &controlPointObject )
{	for ( int i = entryESD->controlPointObjects.GetLength(); --i >= 0; )
	{	if ( entryESD->controlPointObjects[i]->GetControlPointCode() == controlPoint )
		{	controlPointObject = entryESD->controlPointObjects[i];
			return true;
		}
	}
	return false;
}

/*************** Change by Liao Xuan ********************************/
//bool IPMPServicesFull::ProcessMessage ( ToolMessage*	base
//										, IPMP_Data_BaseClass*	msg	)
bool IPMPServicesFull::ProcessMessage (	IPMP_MessageFromTool*	base
										,IPMP_Data_BaseClass*	msg	)
/*************** End: Change by Liao Xuan ***************************/
{
	ToolEntry* sendingTool = NULL;
	int			indexSendingTool(0);
	GiveMeFilterFromID( base->sender, sendingTool, indexSendingTool );

	switch (msg->GetTag()){
	case(TAG_IPMP_Secure_Container):
			return false;
	case(TAG_IPMP_AddToolNotificationListener):
		{	for ( int x = ((IPMP_AddToolNotificationListener*)msg)->eventType.GetLength(); --x >= 0; )
				sendingTool->events.eventTypes += ((IPMP_AddToolNotificationListener*)msg)->eventType.GetElement(x);

			switch (((IPMP_AddToolNotificationListener*)msg)->scope){
			case(PRESENTATION):
				{	// Cycle through all ESDs and add their IDs.
					for ( int i = ZOD_Descriptors.GetLength(); --i >= 0; )
					 	for ( int j = ZOD_Descriptors[i]->entryES.GetLength(); --j >= 0; ) 
							sendingTool->events.ESD_ID += ZOD_Descriptors[i]->entryES[j]->ES_ID;
				}
			case(MY_OD):
				{	// Get containing OD and cycle through its 
					// ESDs and add their IDs.
					ESDEntry*	entryESD = NULL;
					int			indexESD(0);
					ODEntry*		entryOD = NULL;
					int			indexOD(0);

					GiveMeESDFromID( sendingTool->ES_ID, entryESD, indexESD );
					GiveMeODFromID( entryESD->OD_ID, entryOD, indexOD );
					for ( int i = entryOD->entryES.GetLength(); --i >= 0; )
						sendingTool->events.ESD_ID += entryOD->entryES[i]->ES_ID;
				}
			case(MY_ESD):
				{	// Get containing ESD and add its ID.
					ESDEntry*	entryESD = NULL;
					int			indexESD(0);
					GiveMeESDFromID( sendingTool->ES_ID, entryESD, indexESD );

					sendingTool->events.ESD_ID += entryESD->ES_ID;
				}
			};
			return false;
		}
	case(TAG_IPMP_RemoveToolNotificationListener):
		{	bool eventsFound(false);
			for ( int i = ((IPMP_RemoveToolNotificationListener*)msg)->eventType.GetLength(); i >=0; )
			{	eventsFound = true;
				sendingTool->events.eventTypes -= ((IPMP_RemoveToolNotificationListener*)msg)->eventType[i];
			}

			if ( !eventsFound )
			{	sendingTool->events.eventTypes.Clear();
				sendingTool->events.ESD_ID.Clear();																			  
			}
			else if ( !sendingTool->events.eventTypes.GetLength() )
				sendingTool->events.ESD_ID.Clear();

			return true;
		}
	case(TAG_IPMP_InitAuthentication):
			return false;
	case(TAG_IPMP_MutualAuthentication):
			return false;
	case(TAG_IPMP_UserQuery):
			return false;
	case(TAG_IPMP_GetTools):
			return false;
	case(TAG_IPMP_GetToolContext):
		{ 	
			/****************** Hide by Liao Xuan **************/
			/* this message is out of date, to implement later */ 
			
			/*
			IPMP_GetToolContextResponse	payload;
			ToolMessage					toolMsg;
			toolMsg.sender = 0x00;
			toolMsg.receiver = base->sender;

			bool						anyFound(false);

			if ( ((IPMP_GetToolContext*)msg)->hasDescriptor )
			{	//  Search through all tools in all ODs and their contained ESDs
				for ( int i = ZOD_Descriptors.GetLength(); --i >= 0; )
					for ( int j = ZOD_Descriptors[i]->entryES.GetLength(); --j >= 0; )
						for ( int k = ZOD_Descriptors[i]->entryES[j]->tools.GetLength(); --k >= 0; )
							if ( ZOD_Descriptors[i]->entryES[j]->tools[k]->toolDescrEntry->IPMP_DescriptorIDEx
								== ((IPMP_GetToolContext*)msg)->descriptorIDEx )
							{	anyFound = true;
								payload.toolContextID = ZOD_Descriptors[i]->entryES[j]->tools[k]->address.contextID;
								toolMsg.IPMP_MessageFromBitstream += &payload;
							}
			}
			else if ( ((IPMP_GetToolContext*)msg)->hasOD_ID )
			{	ODEntry*	entryOD = NULL;
				int			indexOD(0);
				GiveMeODFromID( ((IPMP_GetToolContext*)msg)->OD_ID, entryOD, indexOD );

				for ( int i = entryOD->ptrOD->ipmpDescrPtr.GetLength(); --i >= 0; )
					//  Cycle through the contained ESs' tools.
					for ( int j = entryOD->entryES.GetLength(); --j >= 0; )
						for ( int k = entryOD->entryES[j]->tools.GetLength(); --k >= 0; )
						{	anyFound = true;
							payload.toolContextID = entryOD->entryES[j]->tools[k]->address.contextID;
							toolMsg.IPMP_MessageFromBitstream += &payload;
						}
			}
			else if ( ((IPMP_GetToolContext*)msg)->hasES_ID )
			{	ESDEntry*	entryESD = NULL;
				int			indexESD(0);
				GiveMeESDFromID( ((IPMP_GetToolContext*)msg)->ES_ID, entryESD, indexESD );

				for ( int i = entryESD->ptrESD->ipmpDescrPtr.GetLength(); --i >= 0; )
					//  Cycle through the tools to find the matching tools.
					for ( int j = entryESD->tools.GetLength(); --j >= 0; )
					{	anyFound = true;
						payload.toolContextID = entryESD->tools[j]->address.contextID;
						toolMsg.IPMP_MessageFromBitstream += &payload;
					}
			}

			if ( !anyFound )
			{	payload.toolContextID = -1;  // Implementation specific indication of no tools found.
				toolMsg.IPMP_MessageFromBitstream += &payload;
			}
			m_MessageDistributor.ReceiveMessage(&toolMsg);

			return true; */
			/****************** End: Hide by Liao Xuan **************/
		}
	case(TAG_IPMP_ConnectTool):
		{	
			// Copy and add input IPMP_Descriptor to ZRequestedDescriptors
			IPMP_Descriptor	* descriptor = new IPMP_Descriptor;
			descriptor->IPMP_DescriptorIDEx = ((IPMP_ConnectTool*)msg)->descriptor[0].IPMP_DescriptorIDEx;
			descriptor->IPMP_ToolID =	((IPMP_ConnectTool*)msg)->descriptor[0].IPMP_ToolID;
			descriptor->URLString	=	((IPMP_ConnectTool*)msg)->descriptor[0].URLString;
			descriptor->controlPointCode = ((IPMP_ConnectTool*)msg)->descriptor[0].controlPointCode;
			descriptor->sequenceCode = ((IPMP_ConnectTool*)msg)->descriptor[0].sequenceCode;
			for ( int i = ((IPMP_ConnectTool*)msg)->descriptor[0].IPMPX_Data.GetLength(); --i >= 0; )
			{
				descriptor->IPMPX_Data += ((IPMP_Data_BaseClass*)((IPMP_ConnectTool*)msg)->descriptor[0].IPMPX_Data[i])->Copy();
			}

			ZRequestedDescriptors += descriptor;

			IPMP_DescriptorEntry * entry = new IPMP_DescriptorEntry;
			entry->IPMP_DescriptorIDEx = descriptor->IPMP_DescriptorIDEx;
			entry->ptrIPMP_Descriptor = descriptor;
			entry->ipmpStreams += 0x00;  //  Says this tool will not receive IPMP stream data.

			/*  1. Find scope of requesting tool to find where protection was defined.
			 *		If the scope of the requesting tool is OD, tools will be created for
			 *		all contained streams.  If ESD, then only one tool in that stream.
			 *  2. Add request notification info(s) to requesting tool's entry.
			 *  3. Instance tool(s).
			 *	4. Initialize its interface, done in the InstanceTool() method.
			 *	5. Send the new tool its tool descriptor and notify requesting tool,
			 *		done in the User_InterfaceInitialized::ProcessMessage() method.
			 *	6. When the tool sends a "go" message, and all the other tools for that stream
			 *		are "go", connect the tool. 
			 */

			ESDEntry*		entryESD = NULL;
			int				indexESD(0);
			ODEntry*		entryOD = NULL;
			int				indexOD(0);
			int				newContextID(0);

			// Needed in case a connection fails.
			IPMP_NotifyToolEvent	notifyPayload;
			notifyPayload.eventType = CONNECTIONFAILED;

			/***********Change by Liao Xuan *************************************/
			/* do not use 'msg' again, it has been used in						*/
			/*bool IPMPServicesFull::ProcessMessage(IPMP_MessageFromTool*base,  */
			/*						IPMP_Data_BaseClass*	msg	)				*/
			
			//ToolMessage				msg;
			IPMP_MessageFromTool		newmsg;
			newmsg.sender = 0x00;
			newmsg.receiver = base->sender;
			//msg.IPMP_MessageFromBitstream += &notifyPayload;
			newmsg.messages += &notifyPayload;

			GiveMeESDFromID( sendingTool->ES_ID, entryESD, indexESD );

			if ( sendingTool->toolDescrEntry->referencedAt == 1 )
			{	// Protected at the OD level.
				// Get the ESD and then the OD and start going through all
				// the contained ESDs and create tools.
				GiveMeODFromID( entryESD->OD_ID, entryOD, indexOD );
				for ( int i = entryOD->entryES.GetLength(); --i >= 0; )
				{	sendingTool->requestedToolsContextIDs += newContextID = usedContextIDs + 1;
					sendingTool->events.eventTypes += CONNECTED;
					sendingTool->events.ESD_ID += entryESD->ES_ID;
					entry->referencedAt = 1;
					if ( !InstanceTool( entry, entryOD->entryES[i] ) )
					{	// Notify tool that the "connection" failed.
						sendingTool->requestedToolsContextIDs.DeleteFrom( sendingTool->requestedToolsContextIDs.Find(newContextID) );
						//m_MessageDistributor.ReceiveMessage(&msg);
						m_MessageDistributor.ReceiveMessage(&newmsg);
					}
				}
			}
			else{	// Protected at the ESD level only.
				sendingTool->requestedToolsContextIDs += newContextID = usedContextIDs + 1;
				sendingTool->events.eventTypes += CONNECTED;
				sendingTool->events.ESD_ID += entryESD->ES_ID;
				entry->referencedAt = 0;
				if ( !InstanceTool( entry, entryESD ) )
				{	// Notify tool that the "connection" failed.
					sendingTool->requestedToolsContextIDs.DeleteFrom( sendingTool->requestedToolsContextIDs.Find(newContextID) );
					//m_MessageDistributor.ReceiveMessage(&msg);
					m_MessageDistributor.ReceiveMessage(&newmsg);
				}
			}
			/***********End: Change by Liao Xuan ***************************/

			return true;
		}
	case(TAG_IPMP_DisconnectTool):
		{	ToolEntry*		killedTool = NULL;
			int				indexKilledTool(0);
			ESDEntry*		entryESD = NULL;
			int				indexESD(0);

			GiveMeFilterFromID( ((IPMP_DisconnectTool*)msg)->toolContextID, killedTool, indexKilledTool );
//			killedTool->address.ptrTool->Stop();

			//  Notify tools of disconnection
			//  Technically one should disconnect the tool first such that it would be possible to
			//  report a disconnection failure but since it is impossible for it to fail, this
			//  gives tools a chance to communicate with the tool to be disconnected if need be
			//  before it is actually disconnected.
			bool					notified(false);
			ToolEntry*				searchEntry = NULL;
			int						indexSearch(0);
			IPMP_NotifyToolEvent	payload;

			for (int i = ZAddressList.GetLength(); --i >= 0; )
			{	GiveMeFilterFromID(  ZAddressList[i].contextID, searchEntry, indexSearch );
				if ( searchEntry->events.eventTypes.Find(DISCONNECTED)
					&& searchEntry->events.ESD_ID.Find(sendingTool->ES_ID) )
				{	
					//  Notify tool of event.

					/***********Change by Liao Xuan *************************************/
					/* do not use 'msg' again, it has been used in						*/
					/*bool IPMPServicesFull::ProcessMessage(IPMP_MessageFromTool*base,  */
					/*						IPMP_Data_BaseClass*	msg	)				*/
					
					//ToolMessage				msg;
					IPMP_MessageFromTool		newmsg;
					
					//msg.sender = 0x00;
					//msg.receiver = searchEntry->address.contextID;
					newmsg.sender = 0x00;
					newmsg.receiver = searchEntry->address.contextID;

					payload.eventType = DISCONNECTED;
					payload.toolContextID = killedTool->address.contextID;
					
					//msg.IPMP_MessageFromBitstream += (IPMP_Data_BaseClass*)&payload;
					newmsg.messages += (IPMP_Data_BaseClass*)&payload;
					
					//m_MessageDistributor.ReceiveMessage(&msg);
					m_MessageDistributor.ReceiveMessage(&newmsg);
					if ( searchEntry->address.contextID == base->sender )
						notified = true;
					/***********End: Change by Liao Xuan *********************/
				}
			}
			if ( !notified )
			{		
				/***********Change by Liao Xuan *************************************/
				/* do not use 'msg' again, it has been used in						*/
				/*bool IPMPServicesFull::ProcessMessage(IPMP_MessageFromTool*base,  */
				/*						IPMP_Data_BaseClass*	msg	)				*/
					
				//ToolMessage				msg;
				IPMP_MessageFromTool		newmsg;
	
				//msg.sender = 0x00;
				//msg.receiver = sendingTool->address.contextID;
				newmsg.sender = 0x00;
				newmsg.receiver = sendingTool->address.contextID;

				payload.eventType = DISCONNECTED;
				payload.toolContextID = killedTool->address.contextID;

				//msg.IPMP_MessageFromBitstream += (IPMP_Data_BaseClass*)&payload;
				newmsg.messages += (IPMP_Data_BaseClass*)&payload;

				m_MessageDistributor.ReceiveMessage(&newmsg);
				/***********End: Change by Liao Xuan ******************************/
			}

			//  Now the tool is actually disconnected.
			// 1. Get the containing ESD.
			GiveMeESDFromID( killedTool->ES_ID, entryESD, indexESD );

			// 2. Check to see if the tool is actually connected.
			if ( killedTool->connected )
			{	// 1. Find the tool's control point object.
				ControlPointObject* controlPointObject = NULL;
				GiveMeControlPointObject( entryESD
										, killedTool->toolDescrEntry->ptrIPMP_Descriptor->controlPointCode
										, controlPointObject );
				if ( controlPointObject != NULL )
					controlPointObject->RemoveTool( killedTool->address.ptrTool );
			}
			
			DisconnectTool( killedTool->address.ptrTool );

			ZAddressList -= killedTool->address;
			entryESD->tools.DeleteFrom(indexKilledTool);

			return true;
		}
	case(TAG_IPMP_CanProcess):
		{	/* Find the tool.
			 *	1. Get the ToolEntry, set it's "go" member to that in the message.
			 *	2. If changed to "go" = true check the rest of the tools for the stream
			 *		If all tools have signalled "go" then connect all the tools to the Terminal.
			 *	3. If changed to "go" = false, signal same to Terminal
			 */
			/********** added by Liao Xuan, Feb16 2004 **********/
			// flag to indicate if all tools for ESD have be instantiated
			bool			allToolReady(false); 
			/********** End: added by Liao Xuan, Feb16 2004 *****/
			bool			allGo(true);
			ESDEntry*		entryESD = NULL;
			int				indexESD(0);

			sendingTool->go = ((IPMP_CanProcess*)msg)->canProcess;
			GiveMeESDFromID( sendingTool->ES_ID, entryESD, indexESD );

			if ( sendingTool->go )
			{	// Get the ESD
				// Cycle through all the filters and check if all are set to "go".
				for ( int i = entryESD->tools.GetLength(); --i >= 0; )
					if ( entryESD->tools[i]->go != true )
					{	allGo = false;
						break;
					}
				/********** added by Liao Xuan, Feb16 2004 **********************/
				/*---------------------------------------------------------------
				check if all tools held in IPMP_Descriptors for this ESD have 
				been instantiated and added into its entryESD->tools
				----------------------------------------------------------------*/
				if(entryESD->tools.GetLength()<entryESD->descriptors.GetLength())
				{	
					allToolReady = false;
				}
				else 
					allToolReady = true;
				/********** End: added by Liao Xuan, Feb16 2004 *****************/
			}
			else
			{
				entryESD->ptrESD->GoNoGo( false );
				/******* Add by Liao Xuan ********/
				allGo = false;
				/********End: Add by Liao Xuan****/
			}
			if ( allGo )
			{
				// Cycle through filters, check that they are at a control point
				// and if so, pass them to a ControlPointObject and set
				// their "connected" flag.
				// Then, cycle through the ControlPointObjects and pass them to the Terminal.
				// Then tell the Terminal that the ES is ready to go.
				ControlPointObject * controlPointObject = NULL;
				SDLInt<8> controlPoint(0);

				/************* Change by Liao Xuan *********************/
				/* just process the ControlPoint of the latest tool    */
				//for ( int j = entryESD->tools.GetLength(); --j >= 0; )
				int j = entryESD->tools.GetLength()-1;
				{	controlPoint = entryESD->tools[j]->toolDescrEntry->ptrIPMP_Descriptor->controlPointCode;
					if ( controlPoint != 0x00 ) 
					{	// Get required ControlPointObject.
						GetControlPointObject( entryESD
											 , controlPoint
											 , controlPointObject );
						controlPointObject->AddTool( entryESD->tools[j]->address.ptrTool
												   , entryESD->tools[j]->toolDescrEntry->ptrIPMP_Descriptor->sequenceCode );
						entryESD->tools[j]->connected = true;
						
						// change and move from below codes	
						int controlPointsfound(0);
						for ( int k = 0; k < 3; k++ )
						/**********Changed by liao xuan, Feb16 2004 ************************/
						/*----------------------------------------------------------------- 
						only all tools for the ESD have been instantiated, then SetFilter
						for these tools one by one
						------------------------------------------------------------------*/
						//{	if ( GiveMeControlPointObject ( entryESD
						{	if ( allToolReady && GiveMeControlPointObject ( entryESD
						/**********End: Changed by liao xuan, Feb16 2004 *******************/
														  , k + 1
														  , controlPointObject ) )
							{
								bool success = entryESD->ptrESD->SetFilter( controlPointObject
																	,k );
								if(success)
								{
									char info[100];
									sprintf(info, "Set a filter at control point %d",k + 1 );
									//MessageBox(NULL,info, "Set Filter", MB_OK );
								}
							}
						}
						// End: change and move from below codes	
					}
				}
				// change and move to the above place 
				/*
				int controlPointsfound(0);
				for ( int k = 0; k < 3; k++ )
				{	if ( GiveMeControlPointObject ( entryESD
												  , k + 1
												  , controlPointObject ) )
					{
						bool success = entryESD->ptrESD->SetFilter( controlPointObject
																  , controlPointsfound++ + k );
//						if ( success )
//							entryESD->controlPointObjects[k]->Start();
					}
				}
				*/
				// End: change and move to the above place 
				
				/**********Changed by liao xuan, Feb16 2004 ************************/
				/*------------------------------------------------------------------
				ready for opening this stream only after all tools can be go
				and all tools have been instantiated
				------------------------------------------------------------------*/
				//entryESD->ptrESD->GoNoGo( true );
				if(allGo&&allToolReady)
					entryESD->ptrESD->GoNoGo( true );
				/**********End: Changed by liao xuan, Feb16 2004 *******************/

			}

			return true;
		}
	case(TAG_User_Initialized):
		{
			OutputDebugString ("IPMPXFull:TAG_User_Initialized");
			ToolEntry*		searchEntry = NULL;
			int				indexSearch(0);

			// Notify tools requesting notification.
			for (int i = ZAddressList.GetLength(); --i >= 0; )
			{	GiveMeFilterFromID(  ZAddressList[i].contextID, searchEntry, indexSearch );
				if ( searchEntry->events.eventTypes.Find(CONNECTED) >= 0
					&& searchEntry->events.ESD_ID.Find(sendingTool->ES_ID) >= 0 
					/********* Add by Liao Xuan **************************************/
					/* to check if the new tool(sender)immediately follows the     
					/* tool which requests IPMP_NotifyToolEvent.
					/* Without this line, once there exists one tool requesting
					/* IPMP_NotifyToolEvent in ZAddressList, IPMPXFull will send 
					/* IPMP_NotifyToolEvent again and again when every new tool is initialized */
					&& ZAddressList[i].contextID == (base->sender-1)
					/********* End: Add by Liao Xuan *********************************/

					//&& searchEntry->toolDescrEntry->ptrIPMP_Descriptor->sequenceCode == senderSequenceCode
					)
				{	//  Notify tool of event.
					
					/***********Change by Liao Xuan ********************/
					/* do not use 'msg' again, it has been used in						*/
					/*bool IPMPServicesFull::ProcessMessage(IPMP_MessageFromTool*base,  */
					/*						IPMP_Data_BaseClass*	msg	)				*/
					
					//ToolMessage				msg;
					IPMP_MessageFromTool		newmsg;

					IPMP_NotifyToolEvent		payload;
					//msg.sender = 0x00;
					//msg.receiver = searchEntry->address.contextID;
					newmsg.sender = 0x00;
					newmsg.receiver = searchEntry->address.contextID;

					payload.eventType = CONNECTED;
					payload.toolContextID = sendingTool->address.contextID;

					//msg.IPMP_MessageFromBitstream += (IPMP_Data_BaseClass*)&payload;
					newmsg.messages += (IPMP_Data_BaseClass*)&payload;
					
					//m_MessageDistributor.ReceiveMessage(&msg);
					m_MessageDistributor.ReceiveMessage(&newmsg);
					/***********End: Change by Liao Xuan ***************/


					/*********** Add by Liao Xuan ***************************/
					/* send a message to the connected tool, inform it the	*/
					/* ContextID of the tool who requested connection		*/
					/* with it                                              */
					
					// Notify the new connected tool of event.
					IPMP_MessageFromTool newmsg2;
					IPMP_NotifyToolEvent payload2;
					newmsg2.sender = 0x00;
					newmsg2.receiver = sendingTool->address.contextID;

					payload2.eventType = CONNECTED;
					payload2.toolContextID = searchEntry->address.contextID;
					newmsg2.messages += (IPMP_Data_BaseClass*)&payload2;
				
					m_MessageDistributor.ReceiveMessage(&newmsg2);
					/*********** End: Add by Liao Xuan ***********************/
				}
			}
			return true;
		}
	}
	return false;
};
