//	This software module was originally developed by Craig A. Schultz 
//	(AccessTicket Systems Inc.)
//
//	in the course of development of the MPEG-4 Systems 
//	(ISO/IEC 14496-1) standard. This software module is an implementation 
//	of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified 
//	by the MPEG-4 Systems (ISO/IEC 14496-1) standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	AccessTicket Systems Inc. retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2000. 

#ifndef _IPMPFULL_H_
#define _IPMPFULL_H_

#include <stdio.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <io.h>

#include "MR_Lists.h"
#include "MessageDistributor.h"

class IPMPX_Tool;

class IPMPToolEntry{
   public:
	   //usually, m_nPresentation_IPMPToolID is the same as m_nToolManger_IPMPToolID
	   //except in the case of the isAltGroup or isParametric 
	   SDLLongInt<128>		m_nPresentation_IPMPToolID; //toolid from content
	   SDLLongInt<128>		m_nPhysical_IPMPToolID; //physical toolid maintained by tool manager
	   HINSTANCE			m_pIPMP_ToolDLL_Instance;//pointer to loaded DLL instance
	   ZArray<IPMPX_Tool*>	instancedTools;
};

class IPMPXFull : public IPMPSystem
{
private:
	/**
	 * Instantiate an IPMPServices object
	 * @param szServiceURL the URL identifying this IPMP Service
	 * @param toolList the list of tools contained in the inital Tool list
	 * @return an IPMPServices object containing the Tool Manager and the Message Router
	 */
	virtual IPMPServices *CreateIPMPServices (const char *szServiceURL, IPMP_ToolListDescriptor* toolList);

	/** 
	 * Delete an IPMPServices object
	 * @param pIPMPServices the pointer to the IPMPServices to be removed
	 * @return true if successful
	 */
	virtual bool RemoveIPMPServices(IPMPServices *pIPMPServices);
};

class IPMPServicesFull : public IPMPServices
{
	friend class IPMP_StreamDump;
	friend class IPMP_ToolES_StreamDump;
	friend class MessageDistributor;;

public:
	IPMPServicesFull();

	void Die()
	{	DestroyLists();
	}

	/**
	 * setting message router's pointer, so that the TM can use it to access MR's member functions
	 * @param tmPointer the pointer to the Message Router
	 * @return true if successful
	 */
	bool SetMRPointer( MR_Interface * tmPointer) {return true;}

	/**
	 * Called by the MR to indicate a tool is no longer needed.
	 * @param toolPtr the pointer to the tool to be disconnected
	 * @return true if successful
	 */
	bool DisconnectTool( void* toolPtr);

	/**
	 * Called by the MR to request a tool be instantiated.
	 * @param OD_id the id of the Object Descriptor in which scope the tool will run
	 * @param pESD a pointer to the ESD of the Elementary Stream protected by the tool
	 * @param toolDescriptor the descriptor carrying initialization information for the tool
	 * @return the pointer to the tool
	 */
	void* ConnectTool(ES_Descriptor *pESD, IPMP_Descriptor* descriptor);

	/**
	 * Called by the Terminal to pass a Tool elementary stream
	 * @param tool_ES the interface to the media stream implemented in IM1
	 * @param tool_ESD the relevant Elementary Stream Descriptor
	 * @return true if successful
	 */
	bool ReceiveToolES(MediaStream *tool_ES, ES_Descriptor *tool_ESD);

	/**
	 * Called by the Terminal when a Tool elementary stream is removed
	 * @param pESD a pointer to the relevant Elementary Stream Descriptor
	 * @return true if successful
	 */
	bool RemoveToolES(ES_Descriptor *pESD);

	/**
	 * Called by the Terminal to pass the tool list.
	 * @param toolList the list of tools as defined in MPEG-4 IPMP Extension
	 * @return true if successful
	 */
	bool ReceiveIPMP_ToolListDescriptor( IPMP_ToolListDescriptor* toolList );

	/**
	 * messaging interface.
	 * @param size the size of the message
	 * @param message one of the messages defined in MPEG-4 IPMP Extension, or a user-defined message
	 * @return true if successful
	 */
	// by Liao Xuan
	//bool ReceiveMessage( ToolMessage* );
	bool ReceiveMessage( IPMP_ToolMessageBase*);
	// by Liao Xuan


	/**
	 * Setting tool manager's pointer, so that the MR can use it to access TM's member functions
	 * @param tmPointer a pointer to an object derived from TM_interface
	 * @return true if successful
	 */
	bool SetTMPointer( TM_Interface * tmPointer) {return true;}

	/**
	 * Called by the Terminal to process an OD
	 * @param pOD pointer to the object descriptor to be processed by the MR
	 * @return true if successful
	 */
	bool ProcessObjectDescriptor( ObjectDescriptor* pOD );

	/**
	 * Called by the Terminal to inform the MR that an OD is not valid anymore.
	 * @param pOD pointer to the OD to be removed
	 * @return true if successful
	 */
	bool RemoveObjectDescriptor( ObjectDescriptor* pOD );

	/**
	 * Called by the Terminal to process an IPMPToolDescriptor.
	 * @param descriptor the IPMPToolDescriptor to be processed by the MR
	 * @return true if successful
	 * NOTE: this descriptor object will be deleted after calling this method. The IPMPX
	 * system shall take this into account and copy it into its own memory space if necessary
	 */
	bool ProcessIPMPDescriptor(IPMP_Descriptor* descriptor);

	/**
	 * Called by the Terminal to inform the MR that an IPMPToolDescriptor is not valid anymore.
	 * @param descriptor pointer to the IPMPToolDescriptor to be removed
	 * @return true if successful
	 */
	bool RemoveIPMPDescriptor(IPMP_Descriptor* descriptor);

	/**
	 * Called by the Terminal to process an Elementary Stream Descriptor.
	 * @param pESD pointer to the Elementary Stream Descriptor to be processed by the MR
	 * @return true if successful
	 */
	enum ACCESS_PERMISSION ProcessESDescriptor(ES_Descriptor* pESD);
	
	/**
	 * Called by the Terminal to inform the MR that an ESDescriptor is not valid anymore.
	 * @param pESD pointer to the Elementary Stream Descriptor to be removed 
	 * @return true if successful
	 */
	bool RemoveESDescriptor( ES_Descriptor* pESD );

	/**
	 * Called by the Terminal to create a sink for an IPMP stream.
	 * @param pOD a pointer to the Object Descriptor where the ipmp stream is declared
	 * @param pESD a pointer to the relevant Elementary Stream Descriptor
	 * @param ipmpStream the MediaStream object handling the IPMP stream
	 * @return true if successful
	 */
	bool SetUpIPMPStream(ObjectDescriptor *pOD, ES_Descriptor* pESD, MediaStream* ipmpStream);

	/**
	 * Called by the Terminal to destroy a sink for an IPMP stream, that was previously created by the Terminal
	 * using the SetUpIPMPStream method.
	 * @param pESD a pointer to the relevant Elementary Stream Descriptor
	 * @return true if successful
	 */
	bool RemoveIPMPStream(ES_Descriptor *pESD);

private: //member variables

	// Tool Manager
	ZArray<IPMPToolEntry*>			availableTools;
	ZArray<IPMP_ToolES_StreamDump*>	toolESHandlers;

	// Message Router
	ZArray<IPMP_DescriptorEntry>		ZDescriptors;
	ZArray<IPMP_Descriptor*>			ZRequestedDescriptors;
	ZArray<ODEntry*>					ZOD_Descriptors;
	ZArray<IPMPStreamDumpEntry>			ZIPMPStreamDumps;
	ZArray<AddressEntry>				ZAddressList;
	__int16								usedContextIDs;
	MessageDistributor					m_MessageDistributor;

	// For testing
	int objectsCreated;
private:
	/****************************************************************/
	/*    Utility Operations, Tool Manager                          */
	/****************************************************************/
	bool  Init_AvailableIPMPTools();

	void  CheckDllForTool( _finddata_t &fileInfo );

	bool  DestroyLists();

	bool  Parse_IPMPTool(IPMP_Tool* pIPMPTool);

	bool  RetrieveMissingTool (ZLargeInt<128>, const char* Location);

	bool  FindTool( const SDLLongInt<128> actualID, IPMPToolEntry* &toolEntry, int &index );

	/****************************************************************/
	/*    Utility Operations, Message Router                        */
	/****************************************************************/
	bool InstanceTool( IPMP_DescriptorEntry*	descriptorEntry
					 , ESDEntry*				entryESD );

	bool ShutDownEStools( ESDEntry * entryESD );
	bool ShutDownODtools( ODEntry * entryOD );

	bool GiveMeODFromID( SDLInt<10>& OD_ID
					   , ODEntry* &returnEntry
					   , int& index );
	bool GiveMeODFromPtr( const ObjectDescriptor * ptrOD
						, ODEntry* &returnEntry
						, int& index );

	bool GiveMeESDFromID( SDLInt<16>& ES_ID
						, ESDEntry* &returnEntry
						, int& index );
	bool GiveMeESDFromPtr( const ES_Descriptor* ptrESD
						 , ESDEntry* &returnEntry
						 , int& index );

	bool GiveMeFilterFromID( SDLInt<32>& contextID
						   , ToolEntry* &returnEntry
						   , int& index );
	bool GiveMeFilterFromPtr( const IPMPX_Tool* ptrTool
							, ToolEntry* &returnEntry
							, int& index );

	bool GiveMeDescriptorFromID( SDLInt<16>& descriptorID
							   , IPMP_Descriptor* &ptrDescr
							   , int& index );
	bool GiveMeDescriptorFromPtr( const IPMP_Descriptor* ptrDescr
								, SDLInt<16>& DescriptorID
								, int& index );

	// Creates a new ControlPointObject if one is not found.
	void GetControlPointObject( ESDEntry* entryESD
							  , const SDLInt<8> controlPoint
							  , ControlPointObject* &controlPointObject );

	bool GiveMeControlPointObject( ESDEntry* entryESD
								 , const SDLInt<8> controlPoint
								 , ControlPointObject* &controlPointObject );

	// by Liao Xuan
	//bool ProcessMessage ( ToolMessage			*	base
						//, IPMP_Data_BaseClass	*	msg	);
	bool ProcessMessage ( IPMP_MessageFromTool*	base
						, IPMP_Data_BaseClass*	msg	);
	// by Liao Xuan


};

#endif  // _IPMPFULL_H_
