//	This software module was originally developed by Craig A. Schultz 
//	(AccessTicket Systems Inc.)
//
//	in the course of development of the MPEG-4 Systems 
//	(ISO/IEC 14496-1) standard. This software module is an implementation 
//	of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified 
//	by the MPEG-4 Systems (ISO/IEC 14496-1) standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
// 
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	AccessTicket Systems Inc. retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2000. 

#include "IPMPXFull.h"
#include "IPMP_StreamDump.h"
#include "IPMP_Messages.h"
#include "ODCoder.h"
#include "IPMPX_Tool.h"

/****************************************************************************
 *  IPMP_StreamDump::IPMP_StreamDump()					ref:IPMP_StreamDump.h
 *
 *  Constructor
 */
IPMP_StreamDump::IPMP_StreamDump( ObjectDescriptor	*	pOD
								, ES_Descriptor		*	pESD
								, MediaStream		*	ipmpStream 
								, IPMPServicesFull	*	pParent )
	: m_IPMPStreamDumpThread	(IPMP_StreamDumpThread, this)
	, m_pOD						(pOD)
	, m_pESD					(pESD)
	, m_pIPMPStream				(ipmpStream)
	, m_pParent					(pParent)
	, m_bActive					(false)
{
}

/****************************************************************************
 *  IPMP_StreamDump::~IPMP_StreamDump()									   ref:MR_Class.h
 *
 *  Destructor
 */
IPMP_StreamDump::~IPMP_StreamDump()
{
	ASSERT (m_IPMPStreamDumpThread.GetThread () == NULL);
}

/*-----------------------------------------------------------------
Stop() closes the stream
-----------------------------------------------------------------*/
void
IPMP_StreamDump:: Stop ()
{
	m_bActive = false;
	if (m_pIPMPStream)
		m_pIPMPStream -> Terminate ();
	WaitForThread ();
}

/*-----------------------------------------------------------------
Start() opens the streams and creates the IPMP
thread.
-----------------------------------------------------------------*/
void IPMP_StreamDump :: Start ()
{
	ASSERT (m_IPMPStreamDumpThread.GetThread () == NULL);
	m_bActive = true;
	if (m_pIPMPStream)
		m_pIPMPStream -> Start ();
	if (StartIPMPStreamDumpThread () == -1)
    	throw;
}

/*-----------------------------------------------------------------
Run() retrieves units from the input stream, synchronizes them with units
from the IPMP stream, decrypt the input units, and dispatches the decrypted
units to the output stream, till the input stream's end is exceeded.
-----------------------------------------------------------------*/
void IPMP_StreamDump :: Run ()
{
	LPBYTE pInData;
	int nInSize;
	DWORD dwTime;
	BOOL bEOU;
	ODCoder	coder(NULL, NULL, 0, 1);

	ToolEntry*  toolEntry = NULL;
	int			index(0);
	bool		bSendIt(false);
				
	//MessageBox(NULL, "Dumping IPMP Stream Began! ", "IPMP Stream", MB_OK);
	while ( m_bActive &&
			(pInData = (LPBYTE) m_pIPMPStream -> FetchNext (nInSize, dwTime, bEOU, TRUE) 
			) != END_OF_STREAM )
	{
		if (m_bActive)
		{	
			/*-------------------------------------------------------------
			Process the unit
			-------------------------------------------------------------*/
			coder.Reset( pInData, nInSize );
		
			IPMP_Message update;

			update.EncDec( &coder, DECODE );

			if ( update.IPMPS_Type == 0 ){
				// The actual update is at a URL
			}

			/********* Changed by Liao Xuan, to latest Spec ********/
			//else if ( update.IPMPS_Type == 0 )
			else if ( update.IPMPS_Type == 0xffff )
			/********* Changed by Liao Xuan, to latest Spe ********/
			{	// This is an IPMPX update.
				// by Liao Xuan
				//ToolMessage msg;
				IPMP_MessageFromBitstream  msg;
				//msg.IPMP_Messages += &update;
				msg.messages += &update;
				// by Liao Xuan

				// Search through instantiated tools to find IPMP_DescriptorIDs the same as
				// in the update.
				for ( int j = m_pParent->ZAddressList.GetLength(); --j >= 0; )
				{
				 	// by Liao Xuan
					//if ( m_pParent->ZAddressList[j].descriptorIDEx == update.IPMP_DescriptorID )
					if ( m_pParent->ZAddressList[j].descriptorIDEx == update.IPMP_DescriptorIDEx )
					// by Liao Xuan
					{	bSendIt = false;
						m_pParent->GiveMeFilterFromPtr( m_pParent->ZAddressList[j].ptrTool
													  , toolEntry
													  , index );
						for	(int k = toolEntry->toolDescrEntry->ipmpStreams.GetLength(); --k >= 0; )
						{
							/*********Hide by Liao Xuan *************************************/
							/* can not extract right ED_ID from ipmpStreams now, to do		*/
							//if ( toolEntry->toolDescrEntry->ipmpStreams[k] == m_pESD->ES_ID
								//|| toolEntry->toolDescrEntry->ipmpStreams[k] == 0xfff )
							/*********End: Hide by Liao Xuan ********************************/
							bSendIt = true;
						}
						if ( bSendIt )
					 		m_pParent->ZAddressList[j].ptrTool->ReceiveMessage(&msg);
					}//if
				}//for
			}//else if
			else
			{
				// The update is for "Hooks" tools.
			}
		}//if
		m_pIPMPStream -> Release (TRUE);
	}//while
}

/*-----------------------------------------------------------------
The IPMP thread function calls the virtual Run() function and 
release the flag semaphore when finishes.
-----------------------------------------------------------------*/
void IPMP_StreamDump :: IPMP_StreamDumpThread ()
{
	if (Init ())
		Run ();
	Terminate ();
}
