//	This software module was originally developed by Craig A. Schultz 
//	(AccessTicket Systems Inc.)
//
//	in the course of development of the MPEG-4 Systems 
//	(ISO/IEC 14496-1) standard. This software module is an implementation 
//	of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified 
//	by the MPEG-4 Systems (ISO/IEC 14496-1) standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	AccessTicket Systems Inc. retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2000. 

#include "IPMPXFull.h"
#include "IPMP_ToolES_StreamDump.h"
#include "IPMP_Messages.h"
#include "ODCoder.h"

/****************************************************************************
 *  IPMP_ToolES_StreamDump::IPMP_ToolES_StreamDump()	ref:IPMP_ToolES_StreamDump.h
 *
 *  Constructor
 */
IPMP_ToolES_StreamDump::IPMP_ToolES_StreamDump( ES_Descriptor		*	pESD
											  , MediaStream			*	ipmpToolStream 
											  , IPMPServicesFull	*	pParent 
											  , std::string			&	fileName )
	: m_IPMP_ToolES_StreamDumpThread	(IPMP_ToolES_StreamDumpThread, this)
	, toolFileName						(fileName)
	, m_pESD							(pESD)
	, m_pIPMP_ToolES_Stream				(ipmpToolStream)
	, m_pParent							(pParent)
	, m_bActive							(false)
{
}

/****************************************************************************
 *  IPMP_ToolES_StreamDump::~IPMP_ToolES_StreamDump()	ref:IPMP_ToolES_StreamDump.h
 *
 *  Destructor
 */
IPMP_ToolES_StreamDump::~IPMP_ToolES_StreamDump()
{
	ASSERT (m_IPMP_ToolES_StreamDumpThread.GetThread () == NULL);
}

/*-----------------------------------------------------------------
Stop() closes the stream
-----------------------------------------------------------------*/
void
IPMP_ToolES_StreamDump:: Stop ()
{
	m_bActive = false;
	if (m_pIPMP_ToolES_Stream)
		m_pIPMP_ToolES_Stream -> Terminate ();
	WaitForThread ();
}

/*-----------------------------------------------------------------
Start() opens the streams and creates the IPMP
thread.
-----------------------------------------------------------------*/
void IPMP_ToolES_StreamDump :: Start ()
{
	ASSERT (m_IPMP_ToolES_StreamDumpThread.GetThread () == NULL);
	m_bActive = true;
	if (m_pIPMP_ToolES_Stream)
		m_pIPMP_ToolES_Stream -> Start ();
	if (StartIPMPToolStreamDumpThread () == -1)
    	throw;
}

/*-----------------------------------------------------------------
Run() retrieves units from the input stream, synchronizes them with units
from the IPMP stream, decrypt the input units, and dispatches the decrypted
units to the output stream, till the input stream's end is exceeded.
-----------------------------------------------------------------*/
void IPMP_ToolES_StreamDump :: Run ()
{
	LPBYTE pInData;
	int nInSize;
	DWORD dwTime;
	BOOL bEOU;
	IPMP_ToolES_AU toolAU;
	ODCoder	coder(NULL, NULL, 0, 1);

	if ( m_bActive &&
		 (pInData = (LPBYTE) m_pIPMP_ToolES_Stream -> FetchNext (nInSize, dwTime, bEOU, TRUE) 
		 ) != END_OF_STREAM )
	{	
		/*-------------------------------------------------------------
		Process the unit
		-------------------------------------------------------------*/
		
		/****************** Change by Liao Xuan**********************/
		/* make use of IPMPX Messages Libraries to create tool file */  
		/* from IPMP ToolES stream												*/
		//coder.Reset( pInData, nInSize );
		//toolAU.EncDec( &coder, DECODE );
		toolAU.fromBuffer(pInData );
		IPMPToolES_DecoderConfig *	IPMPinfo = NULL;

		int fs;
		fs = _open(toolFileName.c_str(), _O_BINARY | _O_CREAT | _O_WRONLY, _S_IREAD | _S_IWRITE);
	    //_write(fs, LPCBYTE(toolAU.Tool_Body), toolAU.Tool_Body.GetLength());
		_write(fs, toolAU.Tool_Body, toolAU.sizeOfTool);
		_close(fs);
 
		char debug[100];
		sprintf(debug,"IPMP Tool file %s assembled from Tool ES\n",toolFileName.c_str());
		MessageBox(NULL,debug,"IPMP Tool Assembly", MB_OK);
		/*******************End : Change by Liao Xuan *****************/
		
		/************** Hide by Liao Xuan *******************************/
		/* can not load the dll using info from IPMPToolES_DecoderConfig*/
		/* which causing thread syncronization error, to do				*/
		/*	
		IPMPToolEntry * toolEntry = new IPMPToolEntry;
		IPMPinfo = (IPMPToolES_DecoderConfig*)&(m_pESD->decConfigDescr[1].decSpecificInfo[1]);
		toolEntry->m_nPhysical_IPMPToolID = IPMPinfo->IPMP_ToolID;
		toolEntry->m_pIPMP_ToolDLL_Instance = LoadLibrary( toolFileName.c_str() );
		m_pParent->availableTools += toolEntry;
		m_pIPMP_ToolES_Stream -> Release (TRUE);*/
	}
}

/*-----------------------------------------------------------------
The IPMP thread function calls the virtual Run() function and 
release the flag semaphore when finishes.
-----------------------------------------------------------------*/
void IPMP_ToolES_StreamDump :: IPMP_ToolES_StreamDumpThread ()
{
	if (Init ())
		Run ();
	Terminate ();
}
