//	This software module was originally developed by Craig A. Schultz 
//	(AccessTicket Systems Inc.)
//
//	in the course of development of the MPEG-4 Systems 
//	(ISO/IEC 14496-1) standard. This software module is an implementation 
//	of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified 
//	by the MPEG-4 Systems (ISO/IEC 14496-1) standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	AccessTicket Systems Inc. retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2000. 

#ifndef _MR_LIST_H_
#define _MR_LIST_H_

#include "IPMP_Messages.h"

class ControlPointObject;
class IPMP_StreamDump;
class IPMPX_Tool;
class MediaStream;

/**
  *	 Needed information for a given Tool.
  */
typedef SDLInt<16>	IPMP_ES_ID; // 0x0 = None, 0x01 - 0xfffe = IPMP_ES_ID, 0xffff = broadcast

class IPMP_DescriptorEntry{
public:
	IPMP_DescriptorEntry () { IPMP_DescriptorIDEx = 0;
							  ptrIPMP_Descriptor = NULL;
							  referencedAt = -1; }

	IPMP_DescriptorEntry ( SDLInt<16>		ID
						 , IPMP_Descriptor*	ptr
						 , int				referenceLevel = -1 )
	{							 
		IPMP_DescriptorIDEx = ID;
		ptrIPMP_Descriptor = ptr;
		referencedAt = referenceLevel;
	}

	SDLInt<16>				IPMP_DescriptorIDEx;
	IPMP_Descriptor*		ptrIPMP_Descriptor;
	int						referencedAt; // 0 = ESD level, 1 = OD level.
	ZArray<IPMP_ES_ID>		ipmpStreams;
};

/**
  *	 Needed information for a given Tool.
  */
class EventEntry{
public:
	ZArray<__int16>	ESD_ID;
	ZArray<__int8>	eventTypes; // EventTypes
};


/**
  *	 Needed information for a given Tool.
  */
class AddressEntry{
public:
	AddressEntry() 
		: contextID(0)
		, ptrTool(NULL)
		, descriptorIDEx(0)
	{}

	bool operator == (const AddressEntry &entry) const
	{
		return contextID == entry.contextID 
			&& ptrTool == entry.ptrTool
			&& IPMP_ToolID.GetValue() == entry.IPMP_ToolID.GetValue()
			&& descriptorIDEx == entry.descriptorIDEx;
	}

	SDLLongInt<128>		IPMP_ToolID;
	SDLInt<32>			contextID;
	IPMPX_Tool*			ptrTool;
	SDLInt<16>			descriptorIDEx;
	
};

/**
  *	 Needed information for a given Filter.
  */
class ToolEntry{
public:
	ToolEntry() { ES_ID		= 0;
				  connected	= false;
				  go		= false; }

	AddressEntry				address;
	SDLInt<16>					ES_ID;
	EventEntry					events;
	IPMP_DescriptorEntry*	toolDescrEntry;
	ZArray<int>					requestedToolsContextIDs;
	bool						connected;
	bool						go;
};

/**
  *	 Needed information for a given ESD.
  */
class ESDEntry{
public:
	ESDEntry() { ES_ID = 0;
				 ptrESD = NULL; 
				 OD_ID = 0; }

	SDLInt<16>							ES_ID;
	ES_Descriptor*						ptrESD;
	SDLInt<10>							OD_ID;
	ZArray<IPMP_DescriptorEntry*>		descriptors;
	ZArray<ControlPointObject*>			controlPointObjects;
	ZArray<ToolEntry*>					tools;
	ZArray<void*>						ptr;
};

/**
  *	 Needed information for a given OD.
  */
class ODEntry{
public:
	ODEntry() { OD_ID = 0;
				ptrOD = NULL; }

	SDLInt<10>			OD_ID;
	ObjectDescriptor*	ptrOD;
	ZArray<ESDEntry*>	entryES;
};

class IPMPStreamDumpEntry{
public:
	IPMPStreamDumpEntry () { pOD = NULL;
							 pESD = NULL;
							 ipmpStream = NULL; 
							 ipmpStreamDump = NULL; }

	ObjectDescriptor	*	pOD;
	ES_Descriptor		*	pESD;
	MediaStream			*	ipmpStream;
	IPMP_StreamDump		*	ipmpStreamDump;
};

#endif // _MR_LIST_H_