/** 
 *  @file	IPMP_DESTool.cpp
 *  @brief	implementation of a IPMP DES Decryption Tool
 *  @author Liao Xuan, PSL
 *  @date	Feb 2004
 */

// Primary files for IPMP DES Decryption Tool
//
// IPMP_DESTool.cpp		The entire implementation
// IPMP_DESTool.h			Definition of functions and varables
// DESDLL.dll				128bit DES Encryption&Decryption DLL 
// IPMP_Messages.cpp		The entire implementation for IPMP messages 
// IPMP_Messages.h			Definition of IPMP messages
// IPMP_MessagesEncDec		The implementation for encoding/decoding IPMP messages 
// ...

// Base classes used: IPMPX_Tool

#include "defs.h"
#include "IPMP_Messages.h"
#include "IPMP_DESTool.h"
#include <stdio.h>
#include <malloc.h>
#include <io.h>
#include <sys/stat.h>
#include <fcntl.h>


/*-------------------------------------------------------------
The exported function that instantiates the proper IPMPManager object
-------------------------------------------------------------*/
extern "C" void* CreateInstance ()
{
	/************** Add by Liao Xuan, to show IPMP information***********/
	MessageBox(NULL, "IPMP DES Tool Loaded", "IPMP Tool", MB_OK);
	/************** End: Add by Liao Xuan, to show IPMP information******/
	return new IPMP_DESTool;
}

IPMP_DESTool::IPMP_DESTool ()
	: m_pMRInterface		(NULL)
	, myContextID			(0)
	, m_pMyIPMPDescriptor	(NULL)
{
	IPMPX_Tool::IPMPX_Tool();
#if defined(DES)
	DES_Keys  = (char*) malloc(16*200*sizeof(char));
	indexDecryAU = 0;
	indexKey = 0 ;
	// load DES dll
	hInstDes = LoadLibrary("desdll.dll");
	if(hInstDes == NULL)
	{
		MessageBox(NULL, "\nError: fail to load DES dll", "Error", MB_OK);
		exit(-1);
	}
#endif// DES
}

IPMP_DESTool::~IPMP_DESTool()
{
#if defined(DES)
	FreeLibrary(hInstDes);
#endif// DES
}

bool IPMP_DESTool::ReceiveMessage( IPMP_ToolMessageBase* msg )
{
	switch (msg->tag)
	{
		case(0x01): //IPMP_MessageFromBitstream
		{
			for ( int i = ((IPMP_MessageFromBitstream*)msg)->messages.GetLength(); --i >= 0; )
				ProcessBitstreamMessage( ((IPMP_MessageFromBitstream*)msg), (IPMP_Message*)((IPMP_MessageFromBitstream*)msg)->messages[i] );
			return true;
		}
		case(0x02)://IPMP_DescriptorFromBitstream
		{
			for ( int i = ((IPMP_DescriptorFromBitstream*)msg)->toolDescriptor.GetLength(); --i >= 0; )
				IPMP_Descriptor descriptor = *(((IPMP_DescriptorFromBitstream*)msg)->toolDescriptor[i]);
			return true;
		}
		case(0x03)://IPMP_MessageFromTool
		{
			for ( int i = ((IPMP_MessageFromTool*)msg)->messages.GetLength(); --i >= 0; )
				ProcessMessage( ((IPMP_MessageFromTool*)msg), (IPMP_Data_BaseClass*)((IPMP_MessageFromTool*)msg)->messages[i] );
			return true;
		}
	}
	return false;
}

//function for processing IPMP_Messgaes from bitstreams	
bool IPMP_DESTool::ProcessBitstreamMessage( 
												IPMP_MessageFromBitstream* base, 
												IPMP_Message* msg)
{
#if defined(DES)
	int nIPMPData = ((IPMP_Message*)msg)->IPMP_ExtendedData.GetLength();// number of IPMPX data carried in this IPMP_Message
	for (int i=0; i < nIPMPData; i++)
	{
		switch (((IPMP_Message*)msg)->IPMP_ExtendedData[i]->GetTag())
		{
			case(TAG_IPMP_CanProcess):
			{
				MessageBox(NULL, "DESTool Received CanProcess Message", "IPMP Stream Update", MB_OK);
				return true;
			}
			case(TAG_IPMP_KeyData):
			{
				IPMP_KeyData * keyData = new IPMP_KeyData;
				keyData = (IPMP_KeyData*)((IPMP_Message*)msg)->IPMP_ExtendedData[i];
				int startDTS = (int)keyData->startDTS;
				int exprDTS =  (int)keyData->expireDTS;

				//get key body from IPMP_KeyData
				int keylen = ((IPMP_KeyData*)keyData)->keyBody->array.GetLength();
				const char* keyPtr = (LPCSTR)(((IPMP_KeyData*)keyData)->keyBody->array);
				char key[16];
				if(keylen==16 && indexKey<200 && DES_Keys ) // put 10 DES key into DES_Key[]
				{
					memcpy(DES_Keys+16*indexKey,keyPtr,keylen);
					memcpy(key,keyPtr,16);
					indexKey++;
					char buffer[255]; 
					sprintf (buffer,"\n------------------------------------------\n No.%d Key received through IPMPStream----\n KeyBody: %s StartDTS: %d  Exprire DTS: %d\n-------------------------------------------\n",
							indexKey, keyPtr,startDTS, exprDTS );
					OutputDebugString (buffer);
					return true;
				}//if
				else
				{
					return false;
				}
			}//case
			default:
				return false;
		}//switch
	}//for
#endif //DES
	return false;
}

#if defined(DES)
	//for DES decryption
	typedef int (*EncFi)(char*, char*, char*);
	typedef int (*DecFi)(char*, char*, char*);
	typedef int (*DESPrototype)(char*, int, int, char*, char*);
#endif // DES

bool IPMP_DESTool::ProcessData (	LPBYTE	pInput
									,int	nInputLength 
									, DWORD	dwTime )
{
#if defined(DES)//decrypt AU received
	if(!pInput)
	{
		MessageBox(NULL, "input stream for ProcessData() is NULL", "Error", MB_OK);
		return false;
	}

	// DES decryption
	if(indexDecryAU<200) // decrypt the first 200 AU
	{
		// if not received the key, wait for a while
		while(indexDecryAU>=indexKey)
			Sleep(10);

		char tmp[16];  
		if (nInputLength>=16)
		{
			memcpy(tmp, pInput, 16);

			DESPrototype Decryption;
			if(hInstDes == NULL)
			{
				printf("\nError: Failed to load DES DLL\n");
				return false;
			}
			else {
				Decryption = (DESPrototype)GetProcAddress(hInstDes,"DES_Decryption");
				if(!Decryption)
				{
					printf("\nError: fail to load DES functions\n");
					return false;
				}
			}

			int nKeyLength=16;
			char key[16]; 
			memcpy(key,DES_Keys+indexDecryAU*16,16);
		
			Decryption(key,nKeyLength,nKeyLength,tmp,tmp);
			memcpy(pInput,tmp,16);
			indexDecryAU++;

			char msg[200];  
			sprintf(msg,"\n*****************************************\nNo.%d Audio AU decrypted--- \n AU length: %d. DTS: %d\n*****************************************\n",
					indexDecryAU, nInputLength,dwTime);
			OutputDebugString(msg);
			FILE *temp = fopen("IPMP.log","a+");
			fwrite(msg, strlen(msg),1,temp);
			fclose(temp);
		}
	}
#endif // DES

	if ( m_pNextTool == NULL )
		return false;

	return m_pNextTool->ProcessData( pInput, nInputLength, dwTime );
}


bool IPMP_DESTool::ProcessMessage ( IPMP_MessageFromTool	*	base
									, IPMP_Data_BaseClass	*	msg)
{
	OutputDebugString("IPMP_DESTool:: Begin: ProcessMessage()");

	switch (msg->GetTag()){
	case(TAG_IPMP_Secure_Container):
			return false;
	case(TAG_IPMP_InitAuthentication):
			return false;
	case(TAG_IPMP_MutualAuthentication):
			return false;
	case(TAG_IPMP_UserQueryResponse):
			return false;
	case(TAG_IPMP_ToolParamCapabilitiesQuery):
			return false;
	case(TAG_IPMP_GetToolsResponse):
			return false;
	case(TAG_IPMP_GetToolContextResponse):
			return false;
	case(TAG_IPMP_NotifyToolEvent):
			return false;
	case(TAG_User_Initialize):
		{
			myContextID = base->receiver;
			memcpy( &m_pMRInterface
				  , LPCBYTE(((User_Initialize*)msg)->messageInterface)
				  , sizeof(void*) );

			IPMP_MessageFromTool		newMsg;

			IPMP_CanProcess				payload;
			User_Initialized			anotherPayload;

			newMsg.receiver = 0x00;
			newMsg.sender = myContextID;

			memcpy( &m_pMyIPMPDescriptor
				  , LPCBYTE(((User_Initialize*)msg)->toolDescriptor)
				  , sizeof(void*) );

			// Read the contents of the IPMP_data field.
			for ( int ui = m_pMyIPMPDescriptor->IPMPX_Data.GetLength(); --ui >= 0; )
			{
				switch (m_pMyIPMPDescriptor->IPMPX_Data[ui]->GetTag()){
				case (TAG_IPMP_OpaqueData):
					{
						IPMP_OpaqueData * data = (IPMP_OpaqueData*)m_pMyIPMPDescriptor->IPMPX_Data[ui];
					}
				}
			}  
			payload.canProcess = true;

			newMsg.messages += &payload;
			newMsg.messages += &anotherPayload;

			m_pMRInterface->ReceiveMessage(&newMsg);
			return true;
		}
		case(TAG_IPMP_CanProcess):
			return false;
		default:
				return false;
	}
	return false;
}
