/** 
 *  @file	IPMP_DESTool.h
 *  @brief	implementation of a IPMP DES Decryption Tool
 *  @author Liao Xuan, PSL
 *  @date	Feb 2004
 */

// Primary files for IPMP DES Decryption Tool
//
// IPMP_DESTool.cpp		The entire implementation
// IPMP_DESTool.h			Definition of functions and varables
// DESDLL.dll				128bit DES Encryption&Decryption DLL 
// IPMP_Messages.cpp		The entire implementation for IPMP messages 
// IPMP_Messages.h			Definition of IPMP messages
// IPMP_MessagesEncDec		The implementation for encoding/decoding IPMP messages 
// ...

// Base classes used: IPMPX_Tool


#ifndef _IPMP_DESTOOL_H_
#define _IPMP_DESTOOL_H_



#include "IPMPX_Tool.h"

/** 
 * @class IPMP_DESTool
 * Class which receive key data from IPMP Stream for decypte 
 * audio stream its own size in bytes explicitly.
 * @author Liao Xuan
 * @date Feb 2004
*/
class IPMP_DESTool : public IPMPX_Tool
{
public:
	IPMP_DESTool();
	~IPMP_DESTool();

	// IPMPTool_Interface interface methods.
	bool ReceiveMessage( IPMP_ToolMessageBase*);

	//to support Messages from Bitstream			
	bool IPMP_DESTool::ProcessBitstreamMessage( 
						IPMP_MessageFromBitstream* base, 
						IPMP_Message* msg);

	// Interface methods added by IPMPX_Tool
	bool ProcessData (
		LPBYTE,		// pInput,
		int,		// rnInputLength
		DWORD		// dwTime
		);		

private:
	// Helper method
	bool ProcessMessage ( IPMP_MessageFromTool*	base
						, IPMP_Data_BaseClass*	msg	);

private:
	// Member data needed during tool functioning.
	IPMPTool_Interface	*	m_pMRInterface;
	__int32					myContextID;
	IPMP_Descriptor		*	m_pMyIPMPDescriptor;
	//for DES decryption
	HINSTANCE			hInstDes;
	char*				DES_Keys;//contain 200 DES keys
	int					indexAU;// index of the current AU received
	int					indexDecryAU; // index of the current decrypted AU
	int					indexKey; // index of the current DESkey received from IPMP Stream

};

#endif	// _IPMPX_TOOL_H_