//	This software module was originally developed by Craig A. Schultz 
//	(AccessTicket Systems Inc.)
//
//	in the course of development of the MPEG-4 Systems 
//	(ISO/IEC 14496-1) standard. This software module is an implementation 
//	of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified 
//	by the MPEG-4 Systems (ISO/IEC 14496-1) standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	AccessTicket Systems Inc. retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2000. 

// IPMP_MessagesEncDec.cpp

#include "defs.h"
#include "IPMP_Messages.h"


// by Liao Xuan
void IPMP_ToolListDescriptor :: DescEncDec (ODCoder *pCoder, OPTYPE op)
{
	ipmpTools.EncDec (pCoder, this, op);
}
// by Liao Xuan


void IPMP_Tool :: DescEncDec (ODCoder *pCoder, OPTYPE op)
{
	/********* Change by Liao Xuan **********************/
	/* according to ISO/IEC 14496-1:2001/FDAM 3:2003	*/
	
	IPMP_ToolID.EncDec (pCoder, this, op);// moved 
	isAltGroup.EncDec (pCoder, this, op);
	isParametric.EncDec (pCoder, this, op);
	reserved.EncDec (pCoder, this, op);

	//IPMP_ToolID.EncDec (pCoder, this, op);// moved 
	/********* End: Change by Liao Xuan *****************/

	if (isAltGroup) {
		numAlternates.EncDec (pCoder, this, op);
		Alt_IPMP_ToolIDs.EncDecArray(pCoder, this, op, numAlternates);
	} 
	if (isParametric) {
		toolParamDescr.EncDec (pCoder, this, op);
	}
	ToolURLs.EncDec (pCoder, this, op);
}

void IPMPToolES_DecoderConfig:: DescEncDec (ODCoder *pCoder, OPTYPE op)
{
	/*************** Change by Liao Xuan **************************/
	IPMP_ToolID.EncDec (pCoder, this, op);
	hasFormatID.EncDec (pCoder, this, op);
	hasPackageID.EncDec (pCoder, this, op);
	reserved.EncDec (pCoder, this, op);

	if (hasFormatID)
		Tool_Format_ID.EncDec (pCoder, this, op);
	if (hasPackageID)
		Tool_Package_ID.EncDec (pCoder, this, op);

	opaqueData.EncDec (pCoder, this, op);
	/*************** End: Change by Liao Xuan **********************/
}

/********** Added by Liao Xuan,UK NB Comments *******/
void IPMP_OpaqueData :: DescEncDec (ODCoder *pCoder, OPTYPE op)
{	
	Version.EncDec (pCoder, this, op);
	dataID.EncDec (pCoder, this, op);
	opaqueData.EncDec (pCoder, this, op);
}
/********** Added by Liao Xuan,UK NB Comments *******/

void IPMP_AudioWatermarkingInit :: DescEncDec (ODCoder *pCoder, OPTYPE op)
{	
	Version.EncDec (pCoder, this, op);
	dataID.EncDec (pCoder, this, op);
	inputFormat.EncDec (pCoder, this, op);
	requiredOp.EncDec (pCoder, this, op);
	hasOpaqueData.EncDec (pCoder, this, op);
	reserved.EncDec (pCoder, this, op);
	
	if (inputFormat == PCM)
	{	
		nChannels.EncDec (pCoder, this, op);
		bitPerSample.EncDec (pCoder, this, op);
		frequency.EncDec (pCoder, this, op);
	}

	if ((requiredOp == INSERT_WM)||(requiredOp == REMARK_WM)) 
	{
		/********** Changed by Liao Xuan,UK NB Comments *******/
		wmPayloadLen.EncDec (pCoder, this, op);//add
		//wmPayload.EncDec (pCoder, this, op);//changed
		wmPayload.EncDecArray (pCoder, this, op, wmPayloadLen);
		/********** End: Changed by Liao Xuan,UK NB Comments **/
	}
	
	if ((requiredOp == EXTRACT_WM)||(requiredOp == DETECT_COMPRESSION))
		wmRecipientId.EncDec (pCoder, this, op);
	
	if (hasOpaqueData)
	{
		/********** Changed by Liao Xuan,UK NB Comments *******/
		opaqueDataSize.EncDec (pCoder, this, op);//added
		//opaqueData.EncDec (pCoder, this, op);//changed
		opaqueData.EncDecArray (pCoder, this, op, opaqueDataSize);
		/********** End: Changed by Liao Xuan,UK NB Comments **/
	}
}

void IPMP_VideoWatermarkingInit :: DescEncDec (ODCoder *pCoder, OPTYPE op)
{	Version.EncDec (pCoder, this, op);
	dataID.EncDec (pCoder, this, op);
	inputFormat.EncDec (pCoder, this, op);
	requiredOp.EncDec (pCoder, this, op);
	hasOpaqueData.EncDec (pCoder, this, op);
	reserved.EncDec (pCoder, this, op);
	
	if (inputFormat == YUV)
	{	
		frame_horizontal_size.EncDec (pCoder, this, op);
		frame_vertical_size.EncDec (pCoder, this, op);
		chroma_format.EncDec (pCoder, this, op);
	}

	if ((requiredOp == INSERT_WM)||(requiredOp == REMARK_WM))
	{
		/********** Changed by Liao Xuan,UK NB Comments *******/
		wmPayloadLen.EncDec (pCoder, this, op);//add
		//wmPayload.EncDec (pCoder, this, op);//changed
		wmPayload.EncDecArray (pCoder, this, op, wmPayloadLen);
		/********** End: Changed by Liao Xuan,UK NB Comments **/
	}
	
	if ((requiredOp == EXTRACT_WM) ||(requiredOp == DETECT_COMPRESSION))
		wmRecipientId.EncDec (pCoder, this, op);
	
	if (hasOpaqueData)
	{
		/********** Changed by Liao Xuan,UK NB Comments *******/
		opaqueDataSize.EncDec (pCoder, this, op);//added
		//opaqueData.EncDec (pCoder, this, op);//changed
		opaqueData.EncDecArray (pCoder, this, op, opaqueDataSize);
		/********** End: Changed by Liao Xuan,UK NB Comments **/
	}
}

void BufferInfoStructure::DescEncDec (ODCoder *pCoder, OPTYPE op)
{	cipher_Id.EncDec (pCoder, this, op);
	syncBoundary.EncDec (pCoder, this, op);
	isBlock.EncDec (pCoder, this, op);
	reserved.EncDec (pCoder, this, op);

	if ( isBlock )
	{	mode.EncDec (pCoder, this, op);
		blockSize.EncDec (pCoder, this, op);
		keySize.EncDec (pCoder, this, op);
	} else
		streamCipherSpecificInitInfo.EncDec (pCoder, this, op);
}

void FieldStructure::DescEncDec (ODCoder *pCoder, OPTYPE op)
{	field_Id.EncDec (pCoder, this, op);
	field_Scope.EncDec (pCoder, this, op);
	reserved.EncDec (pCoder, this, op);
	buf.EncDec (pCoder, this, op);
	isMapped.EncDec (pCoder, this, op);
	reserved1.EncDec (pCoder, this, op);

	if ( isMapped )
	{	sendMapTable.EncDec (pCoder, this, op);
		isShuffled.EncDec (pCoder, this, op);
		reserved2.EncDec (pCoder, this, op);

		if ( sendMapTable )
		{
			//by Liao Xuan -- ISO/IEC 14496-1:2001/FDAM 3:2003
			sizMapTable.EncDec (pCoder, this, op);//add
			mappingTable.EncDecArray (pCoder, this, op, sizMapTable);//changed
			//End: by Liao Xuan -- ISO/IEC 14496-1:2001/FDAM 3:2003
		}
		if ( isShuffled )
			shuffleSpecificInfo.EncDec (pCoder, this, op);
	}
}

/******* Changed by Liao Xuan -- ISO/IEC 14496-1:2001/FDAM 3:2003 */
void IPMP_SelectiveDecryptionInit :: DescEncDec (ODCoder *pCoder, OPTYPE op)
{	
	Version.EncDec (pCoder, this, op);
	dataID.EncDec (pCoder, this, op);
	mediaTypeExtension.EncDec (pCoder, this, op);
	mediaTypeIndication.EncDec (pCoder, this, op);
	profileLevelIndication.EncDec (pCoder, this, op);
	compliance.EncDec (pCoder, this, op);
	numBufs.EncDec (pCoder, this, op);//added
	bufferInfo.EncDecArray (pCoder, this, op,numBufs);//changed
	isContentSpecific.EncDec (pCoder, this, op);
	reserved.EncDec (pCoder, this, op);

	if ( isContentSpecific )
	{
		numFields.EncDec (pCoder, this, op);//add
		fields.EncDecArray (pCoder, this, op, numFields);//changed
	}
	else
	{
		//decryptionMask.EncDec (pCoder, this, op);
		nSegments.EncDec (pCoder, this, op);//added
		RLE_Data.EncDecArray (pCoder, this, op, nSegments);//added
	}
	
}
/*****End: Changed by Liao Xuan -- ISO/IEC 14496-1:2001/FDAM 3:2003 */


void IPMP_KeyData :: DescEncDec (ODCoder *pCoder, OPTYPE op)
{	
	Version.EncDec (pCoder, this, op);
	dataID.EncDec (pCoder, this, op);
	keyBody.EncDec(pCoder,this,op);
	hasStartDTS.EncDec (pCoder, this, op);
	hasStartPacketID.EncDec (pCoder, this, op);
	hasExpireDTS.EncDec (pCoder, this, op);
	hasExpirePacketID.EncDec (pCoder, this, op);
	reserved.EncDec (pCoder, this, op);

	if ( hasStartDTS )
		startDTS.EncDec (pCoder, this, op);
	if ( hasStartPacketID )
		startPacketID.EncDec(pCoder,this,op);
	if ( hasExpireDTS )
		expireDTS.EncDec(pCoder,this,op);
	if ( hasExpirePacketID )
		expirePacketID.EncDec(pCoder,this,op);

	opaqueData.EncDec(pCoder,this,op);
}

void IPMP_SendAudioWatermark :: DescEncDec (ODCoder *pCoder, OPTYPE op)
{	Version.EncDec (pCoder, this, op);
	dataID.EncDec (pCoder, this, op);
	wm_status.EncDec(pCoder,this,op);
	compression_status.EncDec(pCoder,this,op);
	hasOpaqueData.EncDec(pCoder,this,op);
	reserved.EncDec(pCoder,this,op);
	
	if (wm_status == WM_PAYLOAD)
		payload.EncDec(pCoder,this,op);

	if	(hasOpaqueData)
		opaqueData.EncDec(pCoder,this,op);
}

void IPMP_SendVideoWatermark :: DescEncDec (ODCoder *pCoder, OPTYPE op)
{	Version.EncDec (pCoder, this, op);
	dataID.EncDec (pCoder, this, op);
	wm_status.EncDec(pCoder,this,op);
	compression_status.EncDec(pCoder,this,op);
	hasOpaqueData.EncDec(pCoder,this,op);
	reserved.EncDec(pCoder,this,op);
	
	if (wm_status == WM_PAYLOAD)
		payload.EncDec(pCoder,this,op);

	if	(hasOpaqueData)
		opaqueData.EncDec(pCoder,this,op);
}

/********** Added by Liao Xuan,UK NB Comments **********/
void IPMP_RightsData :: DescEncDec (ODCoder *pCoder, OPTYPE op)
{	
	Version.EncDec (pCoder, this, op);
	dataID.EncDec (pCoder, this, op);
	rightsInfo.EncDec (pCoder, this, op);
}
/********** End: Added by Liao Xuan,UK NB Comments *****/


/********** Added by Filippo Chiariglione,UK NB Comments **********/
void IPMP_CanProcess :: DescEncDec (ODCoder *pCoder, OPTYPE op)
{	
	Version.EncDec (pCoder, this, op);
	dataID.EncDec (pCoder, this, op);
	canProcess.EncDec (pCoder, this, op);
	Reserved.EncDec (pCoder, this, op);
}

void IPMP_ConnectTool :: DescEncDec (ODCoder *pCoder, OPTYPE op)
{	
	Version.EncDec (pCoder, this, op);
	dataID.EncDec (pCoder, this, op);
	descriptor.EncDec (pCoder, this, op);
}

void IPMP_NotifyToolEvent :: DescEncDec (ODCoder *pCoder, OPTYPE op)
{	
	Version.EncDec (pCoder, this, op);
	dataID.EncDec (pCoder, this, op);
	hasID.EncDec (pCoder, this, op);
	reserved.EncDec (pCoder, this, op);

	if (hasID)
	{
		OD_ID.EncDec (pCoder, this, op);
		reserved2.EncDec (pCoder, this, op);
		ESD_ID.EncDec (pCoder, this, op);
	}
	
	eventType.EncDec (pCoder, this, op);
	toolContextID.EncDec (pCoder, this, op);
}
/********** End: Added by Filippo Chiariglione,UK NB Comments *****/


void IPMP_Secure_Container :: DescEncDec (ODCoder *pCoder, OPTYPE op)
{	
	Version.EncDec (pCoder, this, op);
	dataID.EncDec (pCoder, this, op);
	hasEncryption.EncDec (pCoder, this, op);
	hasMAC.EncDec(pCoder,this,op);
	isMACEncrypted.EncDec (pCoder, this, op);
	Reserved.EncDec (pCoder, this, op);

	/************ Change by Liao Xuan ************************/
	/* according to ISO/IEC 14496-1:2001/FDAM 3:2003         */

	//if (hasEncryption || isMACEncrypted)
		//encryptedData.EncDec (pCoder, this, op);
	//if (hasMAC)
		//MAC.EncDec (pCoder, this, op);

	if (hasEncryption)
	{
		encryptedData.EncDec (pCoder, this, op);
		if (hasMAC && !isMACEncrypted)
			MAC.EncDec (pCoder, this, op);
	}
	else
	{
		protectedMsg.EncDec (pCoder, this, op);

		if(hasMAC)
			MAC.EncDec (pCoder, this, op);
	}
	/************ End: Change by Liao Xuan *********************/
}

/************ Add by Liao Xuan **********************************/
void AlgorithmDescriptor:: DescEncDec (ODCoder *pCoder, OPTYPE op)
{
	isRegistered.EncDec (pCoder, this, op);
	reserved.EncDec (pCoder, this, op);
	if (isRegistered) {
		regAuthID.EncDec (pCoder, this, op);
	}
	else {
		specAlgoID.EncDec (pCoder, this, op);
	}
	OpaqueData.EncDec (pCoder, this, op);
}

// for use in IPMP_TrustSecurityMetadata
void TrustSpecification:: DescEncDec (ODCoder *pCoder, OPTYPE op)
{
	hasCCBasedTrustMetadata.EncDec (pCoder, this, op);
	reserved.EncDec (pCoder, this, op);
	if (!hasCCBasedTrustMetadata) 
	{
		startDate.EncDec (pCoder, this, op);
		attackerProfile.EncDec (pCoder, this, op);
		reserved2.EncDec (pCoder, this, op);
		trustedDuration.EncDec (pCoder, this, op);
	} 
	else 
	{
		CCTrustMetadata.EncDec (pCoder, this, op);
	}
}

// for use in IPMP_TrustSecurityMetadata
void TrustedTool:: DescEncDec (ODCoder *pCoder, OPTYPE op)
{
	toolID.EncDec (pCoder, this, op);	
	AuditDate.EncDec (pCoder, this, op);
	numTrustSpecification.EncDec (pCoder, this, op);
	trustSpecification.EncDecArray (pCoder, this, op, numTrustSpecification);
}

void IPMP_TrustSecurityMetadata:: DescEncDec (ODCoder *pCoder, OPTYPE op)
{
	Version.EncDec (pCoder, this, op);	
	dataID.EncDec (pCoder, this, op);
	numTrustedTools.EncDec (pCoder, this, op);
	trustedtools.EncDecArray (pCoder, this, op, numTrustedTools);
}

void IPMP_InitAuthentication :: DescEncDec (ODCoder *pCoder, OPTYPE op)
{	
	Version.EncDec (pCoder, this, op);
	dataID.EncDec (pCoder, this, op);
	contextID.EncDec(pCoder,this,op);
	authType.EncDec(pCoder,this,op);
}
/************ End: Add by Liao Xuan *******************************/


/************* Change by Liao Xuan ********************************/
/*
void IPMP_MutualAuthentication :: DescEncDec (ODCoder *pCoder, OPTYPE op)
{	Version.EncDec (pCoder, this, op);
	dataID.EncDec (pCoder, this, op);
	requestNegotiation.EncDec (pCoder, this, op);
	successNegotiation.EncDec (pCoder, this, op);
	failedNegotiation.EncDec (pCoder, this, op);
	inclAuthenticationData.EncDec (pCoder, this, op);
	inclAuthCodes.EncDec (pCoder, this, op);
	reserved.EncDec (pCoder, this, op);

	if (requestNegotiation)
	 	candidateAlgorithms.EncDec (pCoder, this, op);
	 
	if (successNegotiation)
		agreedAlgorithms.EncDec (pCoder, this, op);

	if (inclAuthenticationData)
		AuthenticationData.EncDec (pCoder, this, op);

	if (inclAuthCodes)
	{	type.EncDec (pCoder, this, op);
		
		if (type == 0x01)
		{	certType.EncDec (pCoder, this, op);
			certificates.EncDec (pCoder, this, op);
		}
		else if (type == 0x02)
			publicKey.EncDec (pCoder, this, op);
		else if (type == 0xFE)
			opaque.EncDec (pCoder, this, op);

		authCodes.EncDec (pCoder, this, op);
	}
}
*/
void IPMP_MutualAuthentication :: DescEncDec (ODCoder *pCoder, OPTYPE op)
{	
	Version.EncDec (pCoder, this, op);
	dataID.EncDec (pCoder, this, op);
	requestNegotiation.EncDec (pCoder, this, op);
	successNegotiation.EncDec (pCoder, this, op);
	failedNegotiation.EncDec (pCoder, this, op);
	inclAuthenticationData.EncDec (pCoder, this, op);
	inclAuthCodes.EncDec (pCoder, this, op);
	reserved.EncDec (pCoder, this, op);

	if (requestNegotiation)
	{
		nCandidateAlgorithm.EncDec (pCoder, this, op);//added
	 	candidateAlgorithms.EncDecArray (pCoder, this, op, nCandidateAlgorithm);//changed
	}
	 
	if (successNegotiation)
	{
		nAgreedAlgorithm.EncDec (pCoder, this, op);//added
		agreedAlgorithms.EncDecArray(pCoder, this, op, nAgreedAlgorithm);//changed
	}

	if (inclAuthenticationData)
		AuthenticationData.EncDec (pCoder, this, op);

	if (inclAuthCodes)
	{	
		type.EncDec (pCoder, this, op);
		
		if (type == 0x01)
		{	
			nCertificate.EncDec (pCoder, this, op);//added
			certType.EncDec (pCoder, this, op);
			certificates.EncDecArray (pCoder, this, op, nCertificate);
		}
		else if (type == 0x02)
			publicKey.EncDec (pCoder, this, op);
		else if (type == 0xFE)
			opaque.EncDec (pCoder, this, op);

		trustData.EncDec (pCoder, this, op);//added
		authCodes.EncDec (pCoder, this, op);
	}
}
/************* End: Change by Liao Xuan ***************************/


/************* Add by Liao Xuan ***********************************/
void IPMP_UserQueryResponse:: DescEncDec (ODCoder *pCoder, OPTYPE op)
{
	Version.EncDec (pCoder, this, op);
	dataID.EncDec (pCoder, this, op);
	inclReplyText.EncDec (pCoder, this, op);
    inclOptionSelect.EncDec (pCoder, this, op);
    reserved.EncDec (pCoder, this, op);
    languageCode.EncDec (pCoder, this, op);
    if  (inclReplyText) 
	{
		nReplyText.EncDec (pCoder, this, op);
        replyText.EncDecArray (pCoder, this, op, nReplyText);
    }
    if  (inclOptionSelect) 
	{
        numOfOptions.EncDec (pCoder, this, op);
  		for (int i=0; i<numOfOptions; i++)
  		{
			optionResults[i].EncDec (pCoder, this, op);
			reserved2[i].EncDec (pCoder, this, op);
  		}
    }
}
/************* End: Add by Liao Xuan ******************************/


void IPMP_GetToolContext :: DescEncDec (ODCoder *pCoder, OPTYPE op)
{	
	Version.EncDec (pCoder, this, op);
	dataID.EncDec (pCoder, this, op);
	/******** Change by Liao Xuan ************************/
	/*  according to ISO/IEC 14496-1:2001/FDAM 3:2003    */
	/*
	hasDescriptor.EncDec (pCoder, this, op);
	hasES_ID.EncDec (pCoder, this, op);
	hasOD_ID.EncDec (pCoder, this, op);
	Reserved.EncDec (pCoder, this, op);

	if (hasDescriptor)
		descriptorIDEx.EncDec (pCoder, this, op);

	if (hasES_ID)
		ES_ID.EncDec (pCoder, this, op);

	if (hasOD_ID)
		OD_ID.EncDec (pCoder, this, op);
	*/
	scope.EncDec (pCoder, this, op);
	hasIPMPDescriptorIDEx.EncDec (pCoder, this, op);
	reserved.EncDec (pCoder, this, op);
	if (hasIPMPDescriptorIDEx)
		IPMP_DescriptorIDEx.EncDec (pCoder, this, op);
	/******** End: Change by Liao Xuan *******************/
	
}


/************* Add by Liao Xuan ***********************************/
void IPMP_GetToolContextResponse :: DescEncDec (ODCoder *pCoder, OPTYPE op)
{
	Version.EncDec (pCoder, this, op);
	dataID.EncDec (pCoder, this, op);
	hasESD_ID.EncDec (pCoder, this, op);
    reserved.EncDec (pCoder, this, op);
    OD_ID.EncDec (pCoder, this, op);
    if (hasESD_ID)
       ESD_ID.EncDec (pCoder, this, op);
    toolContextID.EncDec (pCoder, this, op);

}
/************* End: Add by Liao Xuan ******************************/


void IPMP_Message :: DescEncDec (ODCoder *pCoder, OPTYPE op)
{	
	IPMPS_Type.EncDec (pCoder, this, op);

	if (IPMPS_Type == 0)
		URLString.EncDec(pCoder,this,op);

	//else if (IPMPS_Type == 1)
	else if (IPMPS_Type == 0xffff)
	{	
		//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003
		//IPMP_DescriptorID.EncDec(pCoder,this,op);
		IPMP_DescriptorIDEx.EncDec(pCoder,this,op);
		//End: by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003
	
		IPMP_ExtendedData.EncDec(pCoder,this,op);
	}
	else
		IPMP_data.EncDec(pCoder,this,op);
}

/******* Hide by Liao Xuan *****************************************/
/* using another implementation to decode it, binary messsage ******/
/* 
void IPMP_ToolES_AU :: DescEncDec (ODCoder *pCoder, OPTYPE op)
{	
	isSigned.EncDec (pCoder, this, op);
	reserved.EncDec (pCoder, this, op);

	if (isSigned)
	{	
		IPMP_Tool_Signature.EncDec(pCoder,this,op);
	
		//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003
		numCerts.EncDec(pCoder,this,op);//added
		for(int i=0; i<numCerts; i++)
		{	
			CertType[i].EncDec(pCoder,this,op);//added
			certificates[i].EncDec(pCoder,this,op);
		}
		verifying_Tool_ID.EncDec(pCoder,this,op);
	}
	sizeOfTool.EncDec(pCoder,this,op);//added
	//End: by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003

	Tool_Body.EncDecArray(pCoder,this,op,sizeOfTool);
}
*/
/*******End: Hide by Liao Xuan ************************************/

/************* Add by Liao Xuan ***********************************/
void IPMP_ToolAPI_Config :: DescEncDec (ODCoder *pCoder, OPTYPE op)
{	
	Version.EncDec(pCoder,this,op);
	dataID.EncDec(pCoder,this,op);
	has_Instantiation_API_ID.EncDec(pCoder,this,op);
	has_Messaging_API_ID.EncDec(pCoder,this,op);
	reserved.EncDec(pCoder,this,op);
	if (has_Instantiation_API_ID)
		Instantiation_API_ID.EncDec(pCoder,this,op);
	if (has_Messaging_API_ID)
		Messaging_API_ID.EncDec(pCoder,this,op);
	OpaqueData.EncDec(pCoder,this,op);
}

void IPMP_AddToolNotificationListener:: DescEncDec (ODCoder *pCoder, OPTYPE op)
{	
	Version.EncDec(pCoder,this,op);
	dataID.EncDec(pCoder,this,op);
	scope.EncDec(pCoder,this,op);
	reserved.EncDec(pCoder,this,op);
	eventTypeCount.EncDec(pCoder,this,op);
	eventType.EncDecArray(pCoder,this,op,eventTypeCount);
}

void IPMP_RemoveToolNotificationListener:: DescEncDec (ODCoder *pCoder, OPTYPE op)
{	
	Version.EncDec(pCoder,this,op);
	dataID.EncDec(pCoder,this,op);
	eventTypeCount.EncDec(pCoder,this,op);
	eventType.EncDecArray(pCoder,this,op,eventTypeCount);
}

void AltText:: DescEncDec (ODCoder *pCoder, OPTYPE op)
{	
	languageCode.EncDec(pCoder,this,op);
	if (inclDisplayTitle)
		titleText.EncDec(pCoder,this,op);
	if(inclDisplayText)
	{
		nDisplayText.EncDec(pCoder,this,op);
		displayText.EncDecArray(pCoder,this,op,nDisplayText);
	}
	if(needReplyText)
	{
		nPromptText.EncDec(pCoder,this,op);
		promptText.EncDecArray(pCoder,this,op,nPromptText);
	}
	if(inclOptionSelect)
	{
		nOption.EncDec(pCoder,this,op);
		option.EncDecArray(pCoder,this,op,nPromptText);
	}
	if(inclSMIL)
	{
		isReferenced.EncDec(pCoder,this,op);
		reserved.EncDec(pCoder,this,op);
		if(isReferenced)
			SMIL_URL.EncDec(pCoder,this,op);
		else 
			SMIL.EncDec(pCoder,this,op);
	}
}

void IPMP_UserQuery:: DescEncDec (ODCoder *pCoder, OPTYPE op)
{	
	Version.EncDec(pCoder,this,op);
	dataID.EncDec(pCoder,this,op);
	inclDisplayTitle.EncDec(pCoder,this,op);
    inclDisplayText.EncDec(pCoder,this,op);
    needReplyText.EncDec(pCoder,this,op);
    inclOptionSelect.EncDec(pCoder,this,op);
    inclSMIL.EncDec(pCoder,this,op);
   	reserved.EncDec(pCoder,this,op);
	numOfAltText.EncDec(pCoder,this,op);
	// set flag values for AltTexts before EncDec them
	for(int i=0;i<numOfAltText;i++)
	{
		query[i]->SetFlagValues(inclDisplayTitle,inclDisplayText,
								needReplyText,inclOptionSelect,
								inclSMIL);
	}
	query.EncDecArray(pCoder,this,op,numOfAltText);
}

void IPMP_ParamtericDescription:: DescEncDec (ODCoder *pCoder, OPTYPE op)
{
	Version.EncDec(pCoder,this,op);
	dataID.EncDec(pCoder,this,op);
	descriptionComment.EncDec(pCoder,this,op);
	majorVersion.EncDec(pCoder,this,op);
	minorVersion.EncDec(pCoder,this,op);
	numOfDescriptions.EncDec(pCoder,this,op);
	description.EncDecArray(pCoder,this,op,numOfDescriptions);
}

void User_Initialize:: DescEncDec (ODCoder *pCoder, OPTYPE op)
{	
	Version.EncDec(pCoder,this,op);
	dataID.EncDec(pCoder,this,op);
	messageInterface.EncDec(pCoder,this,op);
	toolDescriptor.EncDec(pCoder,this,op);
}

/************* End: Add by Liao Xuan ******************************/