//	This software module was originally developed by ContentGuard, Inc
//  and later on modified by Filippo Chiariglione (Microsoft Research Asia)
//
//	in the course of development of the MPEG-4 Systems 
//	(ISO/IEC 14496-1) standard. This software module is an implementation 
//	of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified 
//	by the MPEG-4 Systems (ISO/IEC 14496-1) standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	ContentGuard Inc and Microsoft Research Asia retains full right to use
//  the code for its own purpose, assign or donate the code to a third party and 
//  to inhibit third parties from using the code for non MPEG-4 Systems 
//  (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2000. 


#include "CGUtil.h"
#include <string>
#include <time.h>

using namespace MSXML2;
using namespace std;

//-------------------------------------------------------------------------
//--FUNCTION ReportError
//-------------------------------------------------------------------------
//--This function is used for reporting parsing errors which occur during the
//  loading of xml into a IXMLDOMDocument2Ptr object.
//--Parameters:  
//     wostream &wout 					-> a wide output stream to which the error report will be output to.
//     IXMLDOMParseError *pXMLError 	-> contains the error info.
//-------------------------------------------------------------------------
HRESULT CGUtil::ReportError(wstring* pwstrErr, MSXML2::IXMLDOMParseError *pXMLError)
{
    long	line;
	long	linePos;
    LONG	errorCode;
	BSTR	pBReason	= NULL;
    HRESULT hr;

	if(NULL == pXMLError)
	{
		return E_INVALIDARG;
	}//END IF

	hr = pXMLError->get_line(&line);
	if (SUCCEEDED(hr))
	{
		hr = pXMLError->get_linepos(&linePos);
		if (SUCCEEDED(hr))
		{
			hr = pXMLError->get_errorCode(&errorCode);
			if (SUCCEEDED(hr))
			{
				hr = pXMLError->get_reason(&pBReason);
				if (SUCCEEDED(hr))
				{
					(*pwstrErr) += pBReason;
					if (line > 0)
					{
						(*pwstrErr).append(L"Error on line ");
						char num1 [10];
						_itoa(line, num1, 10);
						BSTR bs = _com_util::ConvertStringToBSTR(num1);
						(*pwstrErr).append(bs);
						(*pwstrErr).append(L", position ");
						_itoa(linePos, num1, 10);
						bs = _com_util::ConvertStringToBSTR(num1);
						(*pwstrErr).append(bs);
						(*pwstrErr).append(L".");
					}//END IF
				}//END IF
			}//END IF
		}//END IF
	}//END IF

	if(NULL != pBReason)
		SysFreeString(pBReason);
	//END IF

    return hr;
}//END FUNCTION ReportError

//-------------------------------------------------------------------------
//--FUNCTION compare
//-------------------------------------------------------------------------
//--This function takes two strings containing ISO date and time, converts them
//  to time_t representation, and returns the difference between the two time_t
//  objects through ptrdCompareResult (wstrDateTime1 - wstrDateTime2).
//--The input strings must be in the following format:
//   YYYY-MM-DDTHH:MM:SS
//--The incoming ptrdCompareResult must point to a valid double (it should be
//  allocated by the caller).
//--Example: 2002-10-17T12:57:23 (Oct 10th, 2002, 12:57 with 23 seconds pm)
//-------------------------------------------------------------------------
HRESULT CGUtil::compare(wstring wstrDateTime1, wstring wstrDateTime2, double *ptrdCompareResult)
{
	tm		tmDateTime1;		
	tm		tmDateTime2;   
	HRESULT	hr;


	if(NULL == ptrdCompareResult)
	{
		return E_INVALIDARG;
	}//END IF

	hr = convertDateTime(wstrDateTime1, &tmDateTime1);

	if(S_OK != hr)
	{
		return hr;
	}//END IF

	hr = convertDateTime(wstrDateTime2, &tmDateTime2);

	if(S_OK != hr)
	{
		return hr;
	}//END IF

	*ptrdCompareResult = difftime(mktime(&tmDateTime1), mktime(&tmDateTime2)); 

	return S_OK;
}//END FUNCTION compare



//-------------------------------------------------------------------------
//--FUNCTION convertDateTime
//-------------------------------------------------------------------------
//--This function takes a string containing an ISO date and time and returns 
//  the corresponding tm representation of it.
//--The input string must be in the following format:
//   YYYY-MM-DDTHH:MM:SS
//--The incoming ptrtmOutTime must point to a valid tm (it should be allocated
//  by the caller).
//--Example: 2002-10-17T12:57:23 (Oct 10th, 2002, 12:57 with 23 seconds pm)
//-------------------------------------------------------------------------
HRESULT CGUtil::convertDateTime(wstring wstrISODateTime, tm *ptrtmOutTime)
{ 
  const int	nExactlength = 19;
	wstring		wstrDate;
	wstring		wstrTime;
	//tm				tmOutTime;


	//--LETS MAKE SURE THAT THE STRING IS THE EXACT SIZE SPECIFIED BY OUR 
	//  SUPPORTED FORMAT YYYY-MM-DDTHH:MM:SS.
	if(wstrISODateTime.length() != nExactlength || NULL == ptrtmOutTime)
	{
		return E_INVALIDARG;
	}//END IF	

	ptrtmOutTime->tm_yday = 0;
	ptrtmOutTime->tm_wday = 0;
	ptrtmOutTime->tm_isdst = -1;

	//--Lets extract the numberical values from the string.
	//-- Y Y Y Y - M M - D D T  H  H  :  M  M  :  S  S.
	//-- 0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18

	ptrtmOutTime->tm_year	= _wtoi(wstrISODateTime.substr(0, 4).c_str()) - 1900;
	ptrtmOutTime->tm_mon	= _wtoi(wstrISODateTime.substr(5, 2).c_str()) - 1;
	ptrtmOutTime->tm_mday	= _wtoi(wstrISODateTime.substr(8, 2).c_str());

	ptrtmOutTime->tm_hour	= _wtoi(wstrISODateTime.substr(11, 2).c_str());
	ptrtmOutTime->tm_min	= _wtoi(wstrISODateTime.substr(14, 2).c_str());
	ptrtmOutTime->tm_sec	= _wtoi(wstrISODateTime.substr(17, 2).c_str());

	return S_OK;
}//END FUNCTION convertDateTime

