//	This software module was originally developed by Filippo Chiariglione
//	(Microsoft Research Asia) and Craig A. Schultz (AccessTicket Systems Inc.)
//  
//	in the course of development of the MPEG-4 Systems 
//	(ISO/IEC 14496-1) standard. This software module is an implementation 
//	of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified 
//	by the MPEG-4 Systems (ISO/IEC 14496-1) standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	Microsoft Research Asia and AccessTicket Systems Inc. retains full right to 
//	use the code for its own purpose, assign or donate the code 
//	to a third party and to inhibit third parties from using the code for non 
//	MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2000. 

#ifndef _IPMP_RELTOOL_H_
#define _IPMP_RELTOOL_H_

#include "IPMPX_Tool.h"
#include "SecurityLib.h"


#define WIN32_LEAN_AND_MEAN	

#pragma warning(disable: 4786)

#include <stdio.h>
#include <iostream>
#include <string>
#include <vector>
#include <stdexcept>
#include <atlbase.h>
#include <time.h>
#include <sys/types.h>
#include <sys/timeb.h>



#import <msxml4.dll>


class IPMP_RELTool : public IPMPX_Tool
{
public:
	IPMP_RELTool();
	~IPMP_RELTool();

	// IPMPTool_Interface interface methods.
	// by Liao Xuan
	//bool ReceiveMessage( ToolMessage* );
	bool ReceiveMessage( IPMP_ToolMessageBase*);
	// by Liao Xuan

	// Interface methods added by IPMPX_Tool
	bool ProcessData (
		LPBYTE,		// pInput,
		int,		// rnInputLength
		DWORD		// dwTime
		);		

private:
	// Helper method
	// by Liao Xuan
	//bool ProcessMessage ( ToolMessage			*	base
						//, IPMP_Data_BaseClass	*	msg	);
	bool ProcessMessage ( IPMP_MessageFromTool*	base
						, IPMP_Data_BaseClass*	msg	);
	// by Liao Xuan
	
	/**
	* This is the method used for validation request and conditions.
	*/ 
	bool ProcessREL(BSTR query);
	
	/**
	* This method is called by ProcessRel for extracting from license & query their items
	* 
	* @param license a pointer to an IPMP_RightsData message containing in the 
	* "rightsInfo" field the pointer to a string containing the license.
	* @param query a pointer to an IPMP_RightsData message containing in the 
	* "rightsInfo" field the pointer to a string containing the query.
	* @param pwstrLicenseGroup will hold the REL licenseGroup extracted from 
	* the license file. 
	* @param pwstrUserModulus will hold the modulus (for a keyHolder) extracted from 
	* the XML query file.
	* @param pwstrUserExponent will hold the exponent (for a keyHolder) extracted from 
	* the XML query file.
	* @param pwstrRELRight will hold the REL right extracted from XML query file.
	* @param pwstrResourceLocationUri will hold the REL resource created using the 
	* resourceUri from the query file.
	* @param pwstrTimetoCheck will hold the reference time (used as the time of 
	* exercise) extracted from the XML query file.	  	  
	* @param pwErrorString will contain an error string in case of failure.
	*
	* @return S_OK if correctly extract items,-1 or E_FAIL if otherwise
	*/
	HRESULT ExtractRequestItems(BSTR query,					
								wstring *pwstrLicenseGroup,				
								wstring *pwstrUserModulus,
								wstring *pwstrUserExponent,
								wstring *pwstrRELRight,				
								wstring *pwstrResourceLocationUri,				  	  
								wstring *pwstrTimetoCheck,
								wstring *pwErrorString);

	/**
	* Called by the ProcessREL method, writes in pwIssuers all the Issuers found.
	* 
	* @param pwIssuers the wstring on which the found Issuers will be written
	* @param pvecIssuers 
	*/
	HRESULT AddIssuers(wstring* pwIssuers, vector<wstring>* pvecIssuers);
	BSTR GetRELString(IPMP_RightsData* msg);

private:
	// Member data needed during tool functioning.
	SecurityLib*			m_pSecurity;	//used for Mutual Authentication and processing IPMP_Secure_Container msgs
	char*					m_pData;	//used for containing opaque data to send to other tools
	IPMPTool_Interface	*	m_pMRInterface;
	__int32					m_myContextID;
	__int32					m_MasterToolContextID;
	IPMP_Descriptor	*		m_pMyIPMPDescriptor;
	BSTR					m_sLicense;
	bool					m_bLicenseArrived;
};

#endif	// _IPMPX_TOOL_H_