//	This software module was originally developed by ContentGuard Inc
//	and later on modified by Filippo Chiariglione (Microsoft Research Asia)
//  
//	in the course of development of the MPEG-4 Systems 
//	(ISO/IEC 14496-1) standard. This software module is an implementation 
//	of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified 
//	by the MPEG-4 Systems (ISO/IEC 14496-1) standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	ContentGuard Inc and Microsoft Research Asia  retains full right to 
//	use the code for its own purpose, assign or donate the code 
//	to a third party and to inhibit third parties from using the code for non 
//	MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2000. 



#include <assert.h>
#include <iostream>
#include "RELLicenseAuthorization.h"
#include "CGUtil.h"

using namespace MSXML2;
using namespace std;

const wstring NS_XRML_CORE			= L"http://www.xrml.org/schema/2002/05/xrml2core";
const wstring NS_XRML_SX			= L"http://www.xrml.org/schema/2002/05/xrml2sx";
const wstring NS_MX					= L"urn:mpeg:mpeg21:2002:01-REL-NS";
const wstring NS_DSIG				= L"http://www.w3.org/2000/09/xmldsig#";
const wstring NS_XSI				= L"http://www.w3.org/2001/XMLSchema-instance";
const wstring LICENSE_GROUP			= L"licenseGroup";
const wstring KEY_HOLDER			= L"keyHolder";
const wstring DIGITAL_RESOURCE		= L"digitalResource";
const wstring VALIDITY_INTERVAL		= L"validityInterval";
const wstring GRANT					= L"grant";
const wstring ISSUER				= L"issuer";
const wstring PLAY					= L"play";
const wstring LICENSE_PART_ID_REF	= L"licensePartIdRef";
const wstring VAR_REF				= L"varRef";
const wstring XSI_TYPE				= L"xsi:type";


// XPATH expressions used in the program
const wstring	XPATH_ALL_CHILDREN(L"child::*");
const wstring	XPATH_RESOURCE_URI(L"self::*[local-name()=\"digitalResource\"][namespace-uri()=\"http://www.xrml.org/schema/2002/05/xrml2core\"]/child::*[local-name()=\"nonSecureIndirect\"][namespace-uri()=\"http://www.xrml.org/schema/2002/05/xrml2core\"]/@URI");
const wstring	XPATH_ALL_ATTRIBUTES(L"descendant-or-self::*/attribute::*");
const wstring	XPATH_ALL_LICENSES(L"child::*[local-name()=\"license\"][namespace-uri()=\"http://www.xrml.org/schema/2002/05/xrml2core\"]");
const wstring	XPATH_KEY_HOLDER_MODULUS(L"child::*[local-name()=\"info\"][namespace-uri()=\"http://www.xrml.org/schema/2002/05/xrml2core\"]/child::*[local-name()=\"KeyValue\"][namespace-uri()=\"http://www.w3.org/2000/09/xmldsig#\"]/child::*[local-name()=\"RSAKeyValue\"][namespace-uri()=\"http://www.w3.org/2000/09/xmldsig#\"]/child::*[local-name()=\"Modulus\"][namespace-uri()=\"http://www.w3.org/2000/09/xmldsig#\"]");
const wstring	XPATH_KEY_HOLDER_EXPONENT(L"child::*[local-name()=\"info\"][namespace-uri()=\"http://www.xrml.org/schema/2002/05/xrml2core\"]/child::*[local-name()=\"KeyValue\"][namespace-uri()=\"http://www.w3.org/2000/09/xmldsig#\"]/child::*[local-name()=\"RSAKeyValue\"][namespace-uri()=\"http://www.w3.org/2000/09/xmldsig#\"]/child::*[local-name()=\"Exponent\"][namespace-uri()=\"http://www.w3.org/2000/09/xmldsig#\"]");
const wstring	XPATH_LICENSE_ISSUERS(L"child::*[local-name()=\"issuer\"][namespace-uri()=\"http://www.xrml.org/schema/2002/05/xrml2core\"]");
const wstring	XPATH_VI_NOT_AFTER(L"child::*[local-name()=\"notAfter\"][namespace-uri()=\"http://www.xrml.org/schema/2002/05/xrml2core\"]");
const wstring	XPATH_VI_NOT_BEFORE(L"child::*[local-name()=\"notBefore\"][namespace-uri()=\"http://www.xrml.org/schema/2002/05/xrml2core\"]");
const wstring	XPATH_ISSUER_MODULUS(L"self::*[local-name()=\"issuer\"]/child::*[local-name()=\"Signature\"][namespace-uri()=\"http://www.w3.org/2000/09/xmldsig#\"]/child::*[local-name()=\"KeyInfo\"][namespace-uri()=\"http://www.w3.org/2000/09/xmldsig#\"]/child::*[local-name()=\"KeyValue\"][namespace-uri()=\"http://www.w3.org/2000/09/xmldsig#\"]/child::*[local-name()=\"RSAKeyValue\"][namespace-uri()=\"http://www.w3.org/2000/09/xmldsig#\"]/child::*[local-name()=\"Modulus\"][namespace-uri()=\"http://www.w3.org/2000/09/xmldsig#\"]");
const wstring	XPATH_ISSUER_EXPONENT(L"self::*[local-name()=\"issuer\"]/child::*[local-name()=\"Signature\"][namespace-uri()=\"http://www.w3.org/2000/09/xmldsig#\"]/child::*[local-name()=\"KeyInfo\"][namespace-uri()=\"http://www.w3.org/2000/09/xmldsig#\"]/child::*[local-name()=\"KeyValue\"][namespace-uri()=\"http://www.w3.org/2000/09/xmldsig#\"]/child::*[local-name()=\"RSAKeyValue\"][namespace-uri()=\"http://www.w3.org/2000/09/xmldsig#\"]/child::*[local-name()=\"Exponent\"][namespace-uri()=\"http://www.w3.org/2000/09/xmldsig#\"]");


//------------------------------------------------------------------------------------------
//--FUNCTION CRELLicenseAuthorization
//------------------------------------------------------------------------------------------
// This is the constructor of the class.
// see RELLicenseAuthorization.h for the class definition
//------------------------------------------------------------------------------------------
CRELLicenseAuthorization::CRELLicenseAuthorization()
{ 
	m_bValidateOnParse = VARIANT_FALSE;
}//END CONSTRUCTOR

//------------------------------------------------------------------------------------------
//--FUNCTION attributesAreSupported
//------------------------------------------------------------------------------------------
//--This function checks the REL document we are working with to see if there are
//  any unsupported attributes being used. If there are, the program will exit reporting the 
//	error occurred in the wErr parameter.
//--It takes an IXMLDOMNodePtr containing an XML node as input.
//------------------------------------------------------------------------------------------

HRESULT CRELLicenseAuthorization::attributesAreSupported(MSXML2::IXMLDOMNodePtr pxmlNode, wstring* wErr)
{
	MSXML2::IXMLDOMNodeListPtr	plstAttributeNodeListPtr	= NULL;	
	MSXML2::IXMLDOMNodePtr		pCurrAttributeNode			= NULL;
	HRESULT						hr							= S_OK;

	wErr->assign(L"");
	// Lets check to see is a valid DOM node pointer was sent
	if(NULL == pxmlNode)
	{
		wErr->append(L"A NULL node pointer was passed in to function attributesAreSupported(...).\r\n");
		return E_INVALIDARG;
	}//END IF

	// Grab all the attributes of all the nodes in the document
	plstAttributeNodeListPtr = pxmlNode->selectNodes(XPATH_ALL_ATTRIBUTES.c_str());

	//If we got any attributes lets make sure there aren't any unsupported ones. 
	if(NULL != plstAttributeNodeListPtr)
	{
		// Loop through all the attributes in all the elements in the licenseGroup
		for (int i = 0; i < plstAttributeNodeListPtr->length; i++ )
		{
			pCurrAttributeNode = plstAttributeNodeListPtr->nextNode ();
			
			// Test for xsi:type attribute type
			if (0 == XSI_TYPE.compare(pCurrAttributeNode->nodeName)) 
			{
				wErr->append(L"Encountered unsupported xsi:type attribute type.\r\n");
				hr = S_FALSE;		
			}
			// Test for licensePartIdRef attribute type
			else if(0 == LICENSE_PART_ID_REF.compare(pCurrAttributeNode->nodeName))
			{
				wErr->append(L"Encountered unsupported licensePartIdRef attribute type.\r\n");
				hr = S_FALSE;
			}
			// Test for varRef attribute type
			else if (0 == VAR_REF.compare(pCurrAttributeNode->nodeName)) 
			{
				wErr->append(L"Encountered unsupported varRef attribute type.\r\n");
				hr = S_FALSE;
			}//END IF
		}//END FOR LOOP
		
	}//END IF

	return hr;

}//END FUNCTION attributesAreSupported

//------------------------------------------------------------------------------------------
//--FUNCTION extractRequestItems
//------------------------------------------------------------------------------------------
//--This function takes in four strings containing REL for the input license document we are
//  searching for and our query items: the keyHolder, the right, and the resource.
//--It returns a IXMLDOMNodePtr object containing the licenseGroup from the input license
//  document and five strings containing the items we are searching our license document for: 
//  the keyHolder Modulus, the keyHolder Exponent, the right base name, the right URI, and 
//  the resource location URI, plus one containing possible errors occurred. 
//--Parameters:
//		wstring wstrRELDoc							->REL from the input license document
//		wstring wstrRELKeyHolder					->REL for the keyHolder to search for
//		wstring wstrRELRight						->REL for the right to search for
//		wstring wstrRELResource						->REL for the resource to search for
//		IXMLDOMNodePtr *pRELLicenseGroup			->returns the licenseGroup extracted from wstrRELDoc
//		wstring *pwstrKeyHolderModulus				->returns modulus extracted from wstrRELKeyHolder
//		wstring *pwstrKeyHolderExponent				->returns exponent extracted from wstrRELKeyHolder
//		wstring *pwstrRightBaseName					->returns right base name extracted from wstrRELRight
//		wstring *pwstrRightUri						->returns right URI extracted from wstrRELRight
//		wstring *pwstrResourceLocationUri			->returns resource URI extracted from wstrRELResource
//		wstring* wstrError							->returns possible errors occurred
//--All return parameters must be allocated by the caller.
//------------------------------------------------------------------------------------------
HRESULT CRELLicenseAuthorization::extractRequestItems(
			wstring wstrRELDoc, 
			wstring wstrRELKeyHolder, 
			wstring wstrRELRight, 
			wstring wstrRELResource,
			MSXML2::IXMLDOMNodePtr *pRELLicenseGroup,
			wstring *pwstrKeyHolderModulus,
			wstring *pwstrKeyHolderExponent,
			wstring *pwstrRightBaseName, 
			wstring *pwstrRightUri, 
			wstring *pwstrResourceLocationUri,
			wstring* wstrError)
{
	MSXML2::IXMLDOMParseError  *pXMLError		= NULL;	// For XML errors
	MSXML2::IXMLDOMDocument2Ptr pRELDoc		= NULL;		// For the keyHolder we're trying to evaluate
	MSXML2::IXMLDOMDocument2Ptr pInputKeyHolder	= NULL;	// For the keyHolder we're trying to evaluate
	MSXML2::IXMLDOMDocument2Ptr pInputRight		= NULL;	// For the right we're trying to evaluate
	MSXML2::IXMLDOMDocument2Ptr pInputResource	= NULL;	// For the resource we're trying to evaluate
	MSXML2::IXMLDOMNodePtr		pxmlNodeTemp	= NULL;	// Temp node			

	HRESULT						hr;
	LONG						nErrorCode = E_FAIL;	// For error codes

	wstrError->assign(L"");
	// Lets make sure we got valid pointers for our output data.
	if(	NULL == pRELLicenseGroup || 
		NULL == pwstrKeyHolderModulus || 
		NULL == pwstrKeyHolderExponent || 
		NULL == pwstrRightBaseName || 
		NULL == pwstrRightUri || 
		NULL == pwstrResourceLocationUri)
	{
		wstrError->append(L"Not valid pointers for output data \r\n");
		return E_INVALIDARG;
	}//END IF


	// Load the keyHolder we're trying to get a match on into an XMLDomDocument
	hr = pInputKeyHolder.CreateInstance (__uuidof(MSXML2::DOMDocument40));
	pInputKeyHolder->async = VARIANT_FALSE; // default is true
	pInputKeyHolder->validateOnParse = m_bValidateOnParse;

    if(pInputKeyHolder->loadXML(wstrRELKeyHolder.c_str())!=VARIANT_TRUE)
    {
	    wstrError->append(L"RELLicenseAuthorization: Failed to create keyHolder DOM \r\n");
	    hr = pInputKeyHolder->get_parseError(&pXMLError);
		if(NULL != pXMLError)
		{
			hr = pXMLError->get_errorCode(&nErrorCode);
			if (nErrorCode != 0)
			{
				wstring wtmp;
				hr = CGUtil::ReportError(&wtmp, pXMLError);
				wstrError->append(wtmp);
			}//END IF
			pXMLError->Release();
		}//END IF
	    return E_FAIL;
    }//END IF

	pxmlNodeTemp = pInputKeyHolder->documentElement->selectSingleNode(XPATH_KEY_HOLDER_MODULUS.c_str());
	if(NULL == pxmlNodeTemp)
	{
	    wstrError->append(L"Unsupported keyHolder.  Could not find modulus.\r\n");
		return E_FAIL;
	}//END IF
	*pwstrKeyHolderModulus	= pxmlNodeTemp->text;

	pxmlNodeTemp = pInputKeyHolder->documentElement->selectSingleNode(XPATH_KEY_HOLDER_EXPONENT.c_str());
	if(NULL == pxmlNodeTemp)
	{
		wstrError->append(L"Unsupported keyHolder.  Could not find exponent.\r\n");
		return E_FAIL;
	}//END IF
	*pwstrKeyHolderExponent = pxmlNodeTemp->text;

	// Load the right we're trying to test for into an XMLDomDocument
	hr = pInputRight.CreateInstance (__uuidof(MSXML2::DOMDocument40));
	pInputRight->async = VARIANT_FALSE; // default is true
	pInputRight->validateOnParse = m_bValidateOnParse;


    if(pInputRight->loadXML (wstrRELRight.c_str())!=VARIANT_TRUE)
    {
	    wstrError->append(L"Failed to create right DOM \r\n");
	    hr = pInputRight->get_parseError(&pXMLError);
		if(NULL != pXMLError)
		{
			hr = pXMLError->get_errorCode(&nErrorCode);
			if (nErrorCode != 0)
			{
				wstring wtmp;
				hr = CGUtil::ReportError(&wtmp, pXMLError);
				wstrError->append(wtmp);
			}//END IF
			pXMLError->Release();
		}//END IF
	    return E_FAIL;
    }//END IF

	//--Since the document loaded fine we know we have a document element.  Lets
	//  get the name of the element and its uri.
	*pwstrRightBaseName	= pInputRight->documentElement->baseName;
	*pwstrRightUri		= pInputRight->documentElement->namespaceURI;
	// Lets make sure it is an mx play right.
	if(PLAY != *pwstrRightBaseName || NS_MX != *pwstrRightUri)
	{
		wstrError->append(L"Encountered unsupported query right (expecting mx play).\r\n");
	    return E_FAIL;		
	}//END IF


	// Load the resource we're trying to test for into an XMLDomDocument
	hr = pInputResource.CreateInstance (__uuidof(MSXML2::DOMDocument40));
	pInputResource->async = VARIANT_FALSE; // default is true
	pInputResource->validateOnParse = m_bValidateOnParse;

    if(pInputResource->loadXML (wstrRELResource.c_str())!=VARIANT_TRUE)
    {
	    wstrError->append(L"Failed to create right DOM \r\n");
	    hr = pInputResource->get_parseError(&pXMLError);
		if(NULL != pXMLError)
		{
			hr = pXMLError->get_errorCode(&nErrorCode);
			if (nErrorCode != 0)
			{
				wstring wtmp;
				hr = CGUtil::ReportError(&wtmp, pXMLError);
				wstrError->append(wtmp);
			}//END IF
			pXMLError->Release();
		}//END IF
	    return E_FAIL;
    }//END IF

	pxmlNodeTemp = pInputResource->documentElement->selectSingleNode (XPATH_RESOURCE_URI.c_str());
	if(NULL == pxmlNodeTemp)
	{
		wstrError->append(L"Encountered unsupported resource (expecting a digitalResource).\r\n");
	    return E_FAIL;	
	}//END IF

	*pwstrResourceLocationUri = pxmlNodeTemp->text;	

	//
	// Load the licenseGroup string into an XMLDOMDocument
	//

	hr = pRELDoc.CreateInstance (__uuidof(MSXML2::DOMDocument40));
	pRELDoc->async = VARIANT_FALSE; // default is true
	pRELDoc->validateOnParse = m_bValidateOnParse;
		
    if(pRELDoc->loadXML (wstrRELDoc.c_str())!=VARIANT_TRUE)
    {
	    wstrError->append(L"Failed to create LicenseGroup DOM \r\n");
	    hr = pRELDoc->get_parseError(&pXMLError);
		if(NULL != pXMLError)
		{
			hr = pXMLError->get_errorCode(&nErrorCode);
			if (nErrorCode != 0)
			{
				wstring wtmp;
				hr = CGUtil::ReportError(&wtmp, pXMLError);
				wstrError->append(wtmp);
			}//END IF
			pXMLError->Release();
		}//END IF
	    return E_FAIL;
    }//END IF
	*pRELLicenseGroup = pRELDoc->documentElement;

	//--Check to see if the REL doc had an element and make sure it is an XrML2core licenseGroup
    if(NULL == *pRELLicenseGroup || 
	   0 != LICENSE_GROUP.compare((*pRELLicenseGroup)->baseName) || 
	   0 != NS_XRML_CORE.compare((*pRELLicenseGroup)->namespaceURI))
	{
		wstrError->append(L"Encountered unsupported root element (expecting a licenseGroup).\r\n");
		return E_FAIL;
	}//END IF

	return S_OK;
}//END FUNCTION extractRequestItems


//------------------------------------------------------------------------------------------
//--FUNCTION ValidateRequest
//------------------------------------------------------------------------------------------
//--This function takes in an REL license file, checks to see if there are any matching grants 
//  against the supplied query items (keyholder, right, and resource). It returns vectors of 
//  conditions for any conditions found in the matching grants, and vectors of issuers for the 
//  issuers of the matching grants, plus a wstring documenting what has been found.
//--Parameters:
//		wstring wstrRELLicenseGroup			->REL from the input license document
//		wstring wstrRELKeyHolder			->REL for the keyHolder to search for
//		wstring wstrRELRight				->REL for the right to search for
//		wstring wstrRELResource				->REL for the resource to search for
//		vector<wstring>** ppvecConditions	->Returns a pointer to a vector of conditions. 
//		vector<wstring>** ppvecIssuers		->Returns a pointer to a vector of issuers 
//		wstring* wReport					->Returns a string reporting what has been found in the request
//--The vectors are initialized within the function ValidateRequest
//--The function returns the following HRESULT values:
//	S_OK	if at least one matching grant was found
//	S_FALSE if no matching grants were found
//	S_FAIL  if an error condition was encountered (an unsupported attribute for example)
//------------------------------------------------------------------------------------------
HRESULT CRELLicenseAuthorization::ValidateRequest(wstring wstrRELLicenseGroup, wstring wstrRELKeyHolder,
												  wstring wstrRELRight, wstring wstrRELResource, 
												  vector<wstring>** ppvecConditions, 
												  vector<wstring>** ppvecIssuers, wstring* wReport)
{

	MSXML2::IXMLDOMDocument2Ptr pRELDoc;			// For the REL Document
	wstring				wstrRequestKeyHolderModulus;
	wstring				wstrRequestKeyHolderExponent;
	wstring				wstrRequestRightBaseName;
	wstring				wstrRequestRightUri;
	wstring				wstrRequestResourceLocationUri;
	wstring				wstrCurrCondition;
	bool				bAtLeastOneMatch				= false; // Used at the end to determine HRESULT return value
	HRESULT				hr								= S_OK;


	MSXML2::IXMLDOMNodePtr pRELLicenseGroup;				// For the licenseGroup
	MSXML2::IXMLDOMNodeListPtr plstLicenseNodes;			// Used to loop through the children of the licenseGroup
	MSXML2::IXMLDOMNodeListPtr plstLicenseNodeChildren;		// Used to loop through the children of the current license node
	MSXML2::IXMLDOMNodePtr pCurrLicNode = NULL;				// Temp for the current license node
	MSXML2::IXMLDOMNodePtr pCurrLicChildNode = NULL;		// Temp for the current license child node
	MSXML2::IXMLDOMNodePtr pKeyHolderNode = NULL;			// Will hold the keyHolder node from the grant we're searching
	MSXML2::IXMLDOMNodePtr pIssuerNode = NULL;				// Will hold the issuer node for the license that validates
	MSXML2::IXMLDOMNodeListPtr plstIssuerNodes;				// Used to loop through the issuers of a grant
	MSXML2::IXMLDOMNodePtr pCurrIssuer = NULL;				// Temp for the current issuer
	MSXML2::IXMLDOMNodePtr pXMLNodeTemp = NULL;				// Temp for node.

	*ppvecConditions			= new vector<wstring>();	// Pointer to a vector of conditions we will be returning
	*ppvecIssuers				= new vector<wstring>();	// Pointer to a vector of issuers we will be returning
				
	//
	// Load the items we're querying the licenseGroup for into their respective DOM trees, or extract content some other way
	//
	hr = extractRequestItems(wstrRELLicenseGroup,wstrRELKeyHolder, wstrRELRight, 
		wstrRELResource, (MSXML2::IXMLDOMNodePtr *) &pRELLicenseGroup, 
			&wstrRequestKeyHolderModulus, &wstrRequestKeyHolderExponent,
			&wstrRequestRightBaseName, &wstrRequestRightUri, 
			&wstrRequestResourceLocationUri, wReport);

    if(S_OK != hr)
	{
	  return hr;
	}//END IF

	//
	// Test the licenseGroup for various unsuported items
	//
	wstring wtmp;
	hr = attributesAreSupported(pRELLicenseGroup, &wtmp);
	wReport->append(wtmp);
	if(S_OK != hr)
	{
	  return hr;
	}//END IF

	// 
	// Grab the licenses we will be working with out of the licenseGroup that was passed in to us
	//

	// Grab all the license nodes in the licenseGroup
	plstLicenseNodes = pRELLicenseGroup->selectNodes(XPATH_ALL_LICENSES.c_str());

	//Lets see if we have any licenses to look at.
	if(NULL == plstLicenseNodes)
	{
		wReport->append(L"ValidateRequest: License not valid \r\n");
		return S_FALSE;
    }//END IF

	// Loop through all the licenses in the licenseGroup
	// We will exit upon success
	for (int nLicCounter = 0; nLicCounter < plstLicenseNodes->length; nLicCounter++)
	{
		wReport->append(L"\r\n");
		wReport->append(L"\r\n");
		wReport->append(L"ENTERING LICENSE... \r\n");
		pCurrLicNode = plstLicenseNodes->nextNode ();
		
		// Grab all the children of the current license
		plstLicenseNodeChildren = pCurrLicNode->selectNodes(XPATH_ALL_CHILDREN.c_str());

		// If this license does not contain any children (i.e. grants) lets go to the next one.
		if(NULL == plstLicenseNodeChildren)
		{
			continue;
		}//END IF

		// Grab all the issuers of this grant
		plstIssuerNodes = pCurrLicNode->selectNodes (XPATH_LICENSE_ISSUERS.c_str());

		for (int licChildCounter = 0; licChildCounter != plstLicenseNodeChildren->length; licChildCounter++)
		{
			pCurrLicChildNode = plstLicenseNodeChildren->nextNode ();

			// Lets check to see if this child (of the license) is an xrml2core grant.
			if (0 != GRANT.compare(pCurrLicChildNode->baseName) || 0 != NS_XRML_CORE.compare(pCurrLicChildNode->namespaceURI)) 
			{
				// We only support grants whose parent is license so lets try the next one.
				continue;
			}//END IF

			// Ok, we found a grant node whose parent is a license node

			wstring wtmp1;
			hr = isMatchingGrant(pCurrLicChildNode, wstrRequestKeyHolderModulus, 
						wstrRequestKeyHolderExponent, wstrRequestRightBaseName,
						wstrRequestRightUri, wstrRequestResourceLocationUri, &wstrCurrCondition, &wtmp1);
			wReport->append(wtmp1);

			if(S_OK == hr)
			{
				wReport->append(L"-->Found a match\r\n");

				//If there were no issuers in the license, lets push the condition onto the
				//condition vector and lets push an empty string in the issuer vector. 
				if(NULL == plstIssuerNodes)
				{
					// Lets put a place holder for the issuer to keep in sync with the condition vector.
					(*ppvecIssuers)->push_back(L"");

					// Lets push in the condition for the current match.
					(*ppvecConditions)->push_back(wstrCurrCondition);
				}
				else
				{
					for (int issuerCounter = 0; issuerCounter < plstIssuerNodes->length; issuerCounter++)
					{
						pCurrIssuer		= plstIssuerNodes->nextNode ();

						// Attach all the issuers to our issuer vector
						(*ppvecIssuers)->push_back(static_cast<wstring>(pCurrIssuer->xml));

						// For each issuer, we need to associate a condition
						(*ppvecConditions)->push_back(wstrCurrCondition);


						wReport->append(L"-->Issuer #");
						
						char num1 [10];
						_itoa(issuerCounter, num1, 10);
						BSTR bs = _com_util::ConvertStringToBSTR(num1);
						wReport->append(bs);
						wReport->append(L":");
						wReport->append(L"\r\n");

						// Lets printout the issuer modulus.  
						pXMLNodeTemp = pCurrIssuer->selectSingleNode (XPATH_ISSUER_MODULUS.c_str());
						wReport->append(L"           Modulus=[");
						if (NULL == pXMLNodeTemp)
						{
							wReport->append(L"Missing modulus.");
						}
						else 
						{
							wReport->append(pXMLNodeTemp->text);
						}
						wReport->append(L"]\r\n");

						// Lets printout the issuer exponent.
						pXMLNodeTemp = pCurrIssuer->selectSingleNode (XPATH_ISSUER_EXPONENT.c_str());
						wReport->append(L"           Exponent=[");
						if (NULL == pXMLNodeTemp)
						{
							wReport->append(L"Missing exponent.");
						}
						else
						{
							wReport->append(pXMLNodeTemp->text);
						}
						wReport->append(L"]\r\n");
					}//END FOR LOOP
				}//END IF

				bAtLeastOneMatch = true;
			}//END IF

			// Reset the issuer iterator before going to the next grant(if we have issuers).
			if(NULL != plstIssuerNodes)
			{
				hr = plstIssuerNodes->reset();
			}//END IF
		}//END FOR LOOP (NEXT LICENSE CHILD)
	
		pCurrLicChildNode = NULL;
		plstLicenseNodeChildren = NULL;

	}//END FOR LOOP


	if (bAtLeastOneMatch)
	{
		return S_OK;
	}
	else
	{
		return S_FALSE;		
	}//END IF
	
}//END FUNCTION 

//------------------------------------------------------------------------------------------
//--FUNCTION isMatchingGrant
//------------------------------------------------------------------------------------------
//--This function is passed a grant DOM node and several items to check for; if all of them 
//  match within the grant, then the grant is considered a match. In addition, if a validityInterval
//  condition is found, it is returned. The paremeter wReport will return a string documenting what 
//	has been found.
//  The items to match against are a keyHolder modulus and exponent, a right, and a resource URI.
//--Parameters:
//		IXMLDOMNodePtr	pxmlNodeGrant	->Current DOM tree grant node we're checking
//		wstring	wstrRequestKeyHolderModulus		->keyHolder modulus to check for
//		wstring	wstrRequestKeyHolderExponent	->keyHolder exponent to check for
//		wstring wstrRequestRightBaseName		->right to check for
//		wstring wstrRequestRightUri				->right URI to check for
//		wstring	wstrRequestResourceLocationUri	->resource URI to check for
//		wstring	*pwstrCurrCondition				->returns validityInterval condition if one was found
//		wstring* wReport						->returns a string describing what has been found
//------------------------------------------------------------------------------------------
HRESULT CRELLicenseAuthorization::isMatchingGrant(
						MSXML2::IXMLDOMNodePtr	pxmlNodeGrant, 
						wstring			wstrRequestKeyHolderModulus, 
						wstring			wstrRequestKeyHolderExponent,
						wstring			wstrRequestRightBaseName,
						wstring			wstrRequestRightUri,
						wstring			wstrRequestResourceLocationUri,
						wstring			*pwstrCurrCondition,
						wstring*		wReport)
{
	MSXML2::IXMLDOMNodeListPtr	plstGrantChildrenNodes;	// Used to loop through the children of the current grant node
	MSXML2::IXMLDOMNodePtr		pCurrGrantChildNode = NULL;		// Temp for the current grant child node
	MSXML2::IXMLDOMNodePtr		pDSIGModulusNode = NULL;			// Will hold the dsig:modulus node from the grant we're searching
	MSXML2::IXMLDOMNodePtr		pDSIGExponentNode = NULL;		// Will hold the dsig:exponent node from the grant we're searching
	MSXML2::IXMLDOMNodePtr		pResourceURINode = NULL;			// Will hold the URI of the digitalResource/nonSecureIndirect node
	bool						bKeyHolderMatch = false;					// Used inside each grant to track keyHolder match
	bool						bRightMatch = false;						// Used inside each grant to track right match
	bool						bResourceMatch = false;						// Used inside each grant to track resource match

	wReport->assign(L"");
	// Lets make sure that we got a valid pointer.
	if(NULL == pxmlNodeGrant)
	{
		wReport->append(L"Error: pxmlNodeGrant = NULL \r\n");
		return E_INVALIDARG;
	}//END IF

	// Grab all the children of the current grant node
	wReport->append(L"\r\n");
	wReport->append(L"ENTERING grant\r\n");
	plstGrantChildrenNodes = pxmlNodeGrant->selectNodes(XPATH_ALL_CHILDREN.c_str());

	// Lets check to see if this grant is empty (if the license was validated this would have been tested).
	if (NULL == plstGrantChildrenNodes) 
	{
		wReport->append(L"Error: plstGrantChildrenNodes = NULL \r\n");
		return S_FALSE;;
	}//END IF

	for (int nGrantChildCounter = 0; nGrantChildCounter < plstGrantChildrenNodes->length; nGrantChildCounter++)
	{
		pCurrGrantChildNode = plstGrantChildrenNodes->nextNode ();

		// Test the children of the grant
		wstring wstrGrantChildName	= pCurrGrantChildNode->baseName;
		wstring wstrGrantChildUri	= pCurrGrantChildNode->namespaceURI;

		if (KEY_HOLDER == wstrGrantChildName && NS_XRML_CORE == wstrGrantChildUri)
		{
			// Since we have a keyholder lets get the modulus and the exponent
			pDSIGModulusNode = pCurrGrantChildNode->selectSingleNode(XPATH_KEY_HOLDER_MODULUS.c_str());
			pDSIGExponentNode = pCurrGrantChildNode->selectSingleNode(XPATH_KEY_HOLDER_EXPONENT.c_str());


			// Lets see if the keyHolder had the modulus and exponent and
			// that they matched those of the request keyHolder.
			if (NULL != pDSIGModulusNode &&
				NULL != pDSIGExponentNode &&																								
				(0 == wstrRequestKeyHolderModulus.compare(pDSIGModulusNode->text)) &&
				(0 == wstrRequestKeyHolderExponent.compare(pDSIGExponentNode->text)) ) 
			{
				bKeyHolderMatch = true;
				wReport->append(L"-->keyholder match\r\n");
			}
			else
			{
				bKeyHolderMatch = false;
				wReport->append(L"-->NO keyholder match\r\n");
			}//END IF

		}
		else if (wstrGrantChildName == wstrRequestRightBaseName && wstrGrantChildUri == wstrRequestRightUri)
		{
			if (nGrantChildCounter == 0)
			{
				// if the right was the first child, then there was no keyHolder, so grant is for all
				bKeyHolderMatch = true;
				wReport->append(L"-->keyholder match\r\n");
			}//END IF

			// We have a right
			bRightMatch = true;
			wReport->append(L"-->right match \r\n");
		}
		else if (DIGITAL_RESOURCE == wstrGrantChildName && NS_XRML_CORE == wstrGrantChildUri)
		{
			// We have a resource
			pResourceURINode = pCurrGrantChildNode->selectSingleNode(XPATH_RESOURCE_URI.c_str());

			//If no URI was specified we do not have a match.
			if(NULL != pResourceURINode)
			{
				if(0 == wstrRequestResourceLocationUri.compare(pResourceURINode->text))
				{
					bResourceMatch = true;
					wReport->append(L"-->resource match\r\n");
				}//END IF
			}//END IF

		}
		else if (VALIDITY_INTERVAL == wstrGrantChildName && NS_XRML_CORE == wstrGrantChildUri)
		{
			// We have a time condition, we want to return this
			wReport->append(L"-->we got a condition\r\n");

			*pwstrCurrCondition	= pCurrGrantChildNode->xml;
		}
		else
		{
			// We ran into an element that we are not aware of.
			wReport->append(L"-->Encountered an unsupported element [");
			wReport->append(pCurrGrantChildNode->nodeName);
			wReport->append(L"]. Skipping this grant.\r\n");
			return E_FAIL;		
		}//END IF

	}//END FOR LOOP (NEXT GRANT CHILD)

		// Once we've met all our checks & examined all children of the grant node, grab the issuer
	return ((bKeyHolderMatch && bRightMatch && bResourceMatch) ? S_OK : S_FALSE);

}//END FUNCTION isMatchingGrant

//------------------------------------------------------------------------------------------
//--FUNCTION ValidateCondition
//------------------------------------------------------------------------------------------
//--This function is passed in a string containing an REL validityInterval condition (wstrCondition), 
//  and a string containing an ISO date time (wstrReferenceTime) to check against the condition.
//  It checks to see if the condition has been met. wReport is a return string containing the 
//	result of this operation.
//------------------------------------------------------------------------------------------
HRESULT CRELLicenseAuthorization::ValidateCondition(wstring wstrCondition, wstring wstrReferenceTime, wstring* wReport)
{
	
	
	MSXML2::IXMLDOMDocument2Ptr pValidityInterval;	// For the licenseGroup
	MSXML2::IXMLDOMParseError  *pXMLError = NULL;	// For XML errors
	LONG				nErrorCode = E_FAIL;		// For error codes
	HRESULT				hr = S_OK;
	double				dCompareResult;

	wReport->assign(L"");
	//
	// Load the condition string into an XMLDOMDocument
	//
	hr = pValidityInterval.CreateInstance (__uuidof(MSXML2::DOMDocument40));
	pValidityInterval->async = VARIANT_FALSE; // default is true
	pValidityInterval->validateOnParse = m_bValidateOnParse;
		
    if(pValidityInterval->loadXML (wstrCondition.c_str())!=VARIANT_TRUE)
    {
	    wReport->append(L"Failed to create validityInterval DOM \r\n");
	    hr = pValidityInterval->get_parseError(&pXMLError);
		if(NULL != pXMLError)
		{
			hr = pXMLError->get_errorCode(&nErrorCode);
			if (nErrorCode != 0)
			{
				wstring wtmp;
				hr = CGUtil::ReportError(&wtmp, pXMLError);
				wReport->append(wtmp);
			}//END IF
			pXMLError->Release();
		}//END IF
	    return false;
    }//END IF
	
	// Check the condition
	MSXML2::IXMLDOMNodePtr pNotBeforeNode	= NULL;			// Will hold the validityInterval/notBefore node
	MSXML2::IXMLDOMNodePtr pNotAfterNode	= NULL;			// Will hold the validityInterval/notAfter node

	// Lets make sure we were given a xrml2core validityInterval.
	if(	0 != VALIDITY_INTERVAL.compare(pValidityInterval->documentElement->baseName) ||
		0 != NS_XRML_CORE.compare(pValidityInterval->documentElement->namespaceURI))
	{
		wReport->append(L"Encountered unsupported condition (expecting a validityInterval).\r\n");
		return E_FAIL;
	}//END IF

	pNotBeforeNode = pValidityInterval->documentElement->selectSingleNode (XPATH_VI_NOT_BEFORE.c_str());
	pNotAfterNode = pValidityInterval->documentElement->selectSingleNode (XPATH_VI_NOT_AFTER.c_str());

	// Check the notBefore condition is it exists
	if (pNotBeforeNode != NULL)
	{
		// True if first parameter >= second parameter

		// See if wstrReferenceTime >= notBeforeTime
		hr = CGUtil::compare(wstrReferenceTime, (wstring)pNotBeforeNode->text, &dCompareResult);
		
		// Check to see if we were able to compare (The time format could have been wrong.)
		if (S_OK != hr)
		{
			wReport->append(L"Invalid time format.\r\n");
			return hr;
		}//END IF

		if (dCompareResult < 0.0)
		{
			return S_FALSE;
		}//END IF

	}//END IF
	
	// Check the notAfter condition if it exists
	if (pNotAfterNode != NULL)
	{
		// True if first parameter >= second parameter
		hr = CGUtil::compare(wstrReferenceTime, (wstring)pNotAfterNode->text, &dCompareResult);

		// Check to see if we were able to compare (The time format could have been wrong.)
		if (S_OK != hr)
		{
			wReport->append(L"Invalid time format.\r\n");
			return hr;
		}//END IF

		if (dCompareResult > 0.0)
		{
			return S_FALSE;
		}//END IF
	}//END IF
	

	// If we got here, we either had no time constraints, or the ones that existed passed OK.
	return S_OK;
}//END FUNCTION ValidateCondition


//------------------------------------------------------------------------------------------
//--FUNCTION setValidationFlag
//------------------------------------------------------------------------------------------
//--This function is used to turn on document validation by the MSXML parser at the time the
//  XML is loaded into DOM.
//--It takes a boolean set to either true or false as a parameter.
//------------------------------------------------------------------------------------------
void CRELLicenseAuthorization::setValidationFlag(bool bValidate)
{
	m_bValidateOnParse = (bValidate ? VARIANT_TRUE: VARIANT_FALSE);
}//END FUNCTION setValidationFlag

//------------------------------------------------------------------------------------------
//--FUNCTION releaseMem
//------------------------------------------------------------------------------------------
//--This function is used to delete the vectors containing the conditions and issuers.
//  It takes as a parameter a pointer.
//------------------------------------------------------------------------------------------
HRESULT CRELLicenseAuthorization::releaseMem(void *ptr)
{
	if(NULL != ptr)
	{
		delete ptr;
		// by Liao Xuan
		ptr = NULL;
	}//END IF

	return S_OK;
}//END FUNCTION releaseMem
