//	This software module was originally developed by ContentGuard Inc
//	and later on modified by Filippo Chiariglione (Microsoft Research Asia)
//  
//	in the course of development of the MPEG-4 Systems 
//	(ISO/IEC 14496-1) standard. This software module is an implementation 
//	of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified 
//	by the MPEG-4 Systems (ISO/IEC 14496-1) standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	ContentGuard Inc and Microsoft Research Asia retains full right to 
//	use the code for its own purpose, assign or donate the code 
//	to a third party and to inhibit third parties from using the code for non 
//	MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2000. 



#ifndef GUARD_RELLICENSEAUTHORIZATION_h
#define GUARD_RELLICENSEAUTHORIZATION_h

#pragma warning (disable : 4786) 

#include <string>
#include <vector>

using namespace std;

#import <msxml4.dll>


class IXMLDOMNodePtr;
class IXMLDOMDocument2ptr;

class CRELLicenseAuthorization {
public:
	
	//------------------------------------------------------------------------------------------
	//--FUNCTION CRELLicenseAuthorization
	//------------------------------------------------------------------------------------------
	// This is the constructor of the class.
	//------------------------------------------------------------------------------------------
	CRELLicenseAuthorization();

	//------------------------------------------------------------------------------------------
	//--FUNCTION ValidateRequest
	//------------------------------------------------------------------------------------------
	//--This function takes in an REL license file, checks to see if there are any matching grants 
	//  against the supplied query items (keyholder, right, and resource). It returns vectors of 
	//  conditions for any conditions found in the matching grants, and vectors of issuers for the 
	//  issuers of the matching grants, plus a string detailing the outcome of the validation process
	//--Parameters:
	//		wstring wstrRELLicenseGroup			->REL from the input license document
	//		wstring wstrRELKeyHolder			->REL for the keyHolder to search for
	//		wstring wstrRELRight				->REL for the right to search for
	//		wstring wstrRELResource				->REL for the resource to search for
	//		vector<wstring>** ppvecConditions	->Returns a pointer to a vector of conditions. 
	//		vector<wstring>** ppvecIssuers		->Returns a pointer to a vector of issuers 
	//		wstring* wReport					->Returns a string reporting info on what happened
	//											  while validating the request
	//--The vectors are initialized within the function ValidateRequest
	//--The function returns the following HRESULT values:
	//	S_OK	if at least one matching grant was found
	//	S_FALSE if no matching grants were found
	//	S_FAIL  if an error condition was encountered (an unsupported attribute for example)
	//------------------------------------------------------------------------------------------
	HRESULT ValidateRequest(
		wstring wstrRELLicenseGroup, 
		wstring wstrRELKeyHolder, 
		wstring wstrRELRight, 
		wstring wstrRELResource, 
		vector<wstring>** ppvecConditions, 
		vector<wstring>** ppvecIssuers,
		wstring* wReport);
	
	//------------------------------------------------------------------------------------------
	//--FUNCTION ValidateCondition
	//------------------------------------------------------------------------------------------
	//--This function is passed in a string containing an REL validityInterval condition (wstrCondition), 
	//  and a string containing an ISO date time (wstrReferenceTime) to check against the condition.
	//  It checks to see if the condition has been met. It also returns a string detailing the 
	//	outcome of the validation process.
	//------------------------------------------------------------------------------------------
	HRESULT ValidateCondition(wstring wstrCondition, wstring wstrReferenceTime, wstring* wReport);

	//------------------------------------------------------------------------------------------
	//--FUNCTION setValidationFlag
	//------------------------------------------------------------------------------------------
	//--This function is used to turn on document validation by the MSXML parser at the time the
	//  XML is loaded into DOM.
	//--It takes a boolean set to either true or false as a parameter.
	//------------------------------------------------------------------------------------------
	void setValidationFlag(bool bValidate);

	//------------------------------------------------------------------------------------------
	//--FUNCTION releaseMem
	//------------------------------------------------------------------------------------------
	//--This function is used to delete the vectors containing the conditions and issuers.
	//  It takes as a parameter a pointer.
	//------------------------------------------------------------------------------------------
	HRESULT releaseMem(void *);
private:
	VARIANT_BOOL	m_bValidateOnParse;

	//------------------------------------------------------------------------------------------
	//--FUNCTION attributesAreSupported
	//------------------------------------------------------------------------------------------
	//--This function checks the REL document we are working with to see if there are
	//  any unsupported attributes being used. If there are, the program will exit reporting
	//	possible errors on the string wError.
	//--It takes an IXMLDOMNodePtr containing an XML node as input.
	//------------------------------------------------------------------------------------------
	HRESULT attributesAreSupported(MSXML2::IXMLDOMNodePtr pxmlNode, wstring* wError);

	//------------------------------------------------------------------------------------------
	//--FUNCTION extractRequestItems
	//------------------------------------------------------------------------------------------
	//--This function takes in four strings containing REL for the input license document we are
	//  searching for and our query items: the keyHolder, the right, and the resource.
	//--It returns a IXMLDOMNodePtr object containing the licenseGroup from the input license
	//  document and five strings containing the items we are searching our license document for: 
	//  the keyHolder Modulus, the keyHolder Exponent, the right base name, the right URI, and 
	//  the resource location URI, plus a string reporting possible errors occurred inthe validation process. 
	//--Parameters:
	//		wstring wstrRELDoc							->REL from the input license document
	//		wstring wstrRELKeyHolder					->REL for the keyHolder to search for
	//		wstring wstrRELRight						->REL for the right to search for
	//		wstring wstrRELResource						->REL for the resource to search for
	//		IXMLDOMNodePtr *pRELLicenseGroup			->returns the licenseGroup extracted from wstrRELDoc
	//		wstring *pwstrKeyHolderModulus				->returns modulus extracted from wstrRELKeyHolder
	//		wstring *pwstrKeyHolderExponent				->returns exponent extracted from wstrRELKeyHolder
	//		wstring *pwstrRightBaseName					->returns right base name extracted from wstrRELRight
	//		wstring *pwstrRightUri						->returns right URI extracted from wstrRELRight
	//		wstring *pwstrResourceLocationUri			->returns resource URI extracted from wstrRELResource
	//		wstring* wstrError							->string reporting possible errors occurred
	//													  in the validation process.
	//--All return parameters must be allocated by the caller.
	//------------------------------------------------------------------------------------------
	HRESULT extractRequestItems(
				wstring wstrRELDoc, 
				wstring wstrRELKeyHolder, 
				wstring wstrRELRight, 
				wstring wstrRELResource,
				MSXML2::IXMLDOMNodePtr *pRELLicenseGroup,
				wstring *pwstrKeyHolderModulus,
				wstring *pwstrKeyHolderExponent,
				wstring *pwstrRightBaseName, 
				wstring *pwstrRightUri, 
				wstring *pwstrResourceLocation,
				wstring* wstrError);

	//------------------------------------------------------------------------------------------
	//--FUNCTION isMatchingGrant
	//------------------------------------------------------------------------------------------
	//--This function is passed a grant DOM node and several items to check for; if all of them 
	//  match within the grant, then the grant is considered a match. In addition, if a validityInterval
	//  condition is found, it is returned. wRepost contains a string detailing the outcome of the process
	//  The items to match against are a keyHolder modulus and exponent, a right, and a resource URI.
	//--Parameters:
	//		IXMLDOMNodePtr	pxmlNodeGrant			->Current DOM tree grant node we're checking
	//		wstring	wstrRequestKeyHolderModulus		->keyHolder modulus to check for
	//		wstring	wstrRequestKeyHolderExponent	->keyHolder exponent to check for
	//		wstring wstrRequestRightBaseName		->right to check for
	//		wstring wstrRequestRightUri				->right URI to check for
	//		wstring	wstrRequestResourceLocationUri	->resource URI to check for
	//		wstring	*pwstrCurrCondition				->returns validityInterval condition if one was found
	//		wstring* wReport						->returns a string detailing the outcome of the process
	//------------------------------------------------------------------------------------------
	HRESULT isMatchingGrant(
				MSXML2::IXMLDOMNodePtr	pxmlNodeGrant, 
				wstring					wstrRequestKeyHolderModulus, 
				wstring					wstrRequestKeyHolderExponent,
				wstring					wstrRequestRightBaseName,
				wstring					wstrRequestRightUri,
				wstring					wstrRequestResourceLocationUri,
				wstring					*pwstrCurrCondition,
				wstring*				wReport);
};

#endif

