//	This software module was originally developed by Massimo Balestri 
//	(Telecom Italia Lab S.p.A.), and edited by
//	Zvi, Craig, Ji Ming
//
//	in the course of development of the MPEG-4 Systems 
//	(ISO/IEC 14496-1) standard. This software module is an implementation 
//	of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified 
//	by the MPEG-4 Systems (ISO/IEC 14496-1) standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	Telecom Italia Lab S.p.A retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2000. 



// Required Interfaces among IM1 Terminal, Message Router, IPMP Tool Manager, IPMP Tool:


#ifndef	IM1_INTERFACE_H
#define	IM1_INTERFACE_H

//#include "ODP.h"

class MediaStream;
class TM_Interface;
class IPMP_Descriptor;
class Filter;
class IPMP_ToolListDescriptor;
class IPMPServices;
class ES_Descriptor;
class ObjectDescriptor;
//class ToolMessage;
class IPMP_ToolMessageBase;
/**
 * Top level object of the IPMPX DLL
 * Instantiated into an IM1 terminal by the following code :
 *
 *	static ZRegistry registry ("IPMPX\\System");
 *	m_pIPMPX = (IPMPSystem *) registry.CreateInstance ("IPMPSystem", m_hIPMPX);
 * The pathname of an IPMPX DLL must be defined in the registry as the value of the key
 *  HKEY_CURRENT_USER\Software\MPEG-4\Im1\IPMPX\IPMPSystem 
 *
 * The IPMPX DLL must include the following code :
 *	extern "C" IPMPSystem* CreateInstance ()
 *	{
 *		return new IPMPSystem
 *	}
 */
class IPMPSystem
{
public:

	/**
	 * Instantiate an IPMPServices object
	 * @param szServiceURL the URL identifying this IPMP Service
	 * @param toolList the list of tools contained in the inital Tool list
	 * @return an IPMPServices object containing the Tool Manager and the Message Router
	 */
	virtual IPMPServices *CreateIPMPServices (const char *szServiceURL, IPMP_ToolListDescriptor* toolList) = 0;

	/** 
	 * Delete an IPMPServices object
	 * @param pIPMPServices the pointer to the IPMPServices to be removed
	 * @return true if successful
	 */
	virtual bool RemoveIPMPServices(IPMPServices *pIPMPServices) = 0;
};


/**
  * IPMP Tool interface.
  * Abstract class to be implemented by any IPMP Tool.
  */
class IPMPTool_Interface
{
public:
	/**
	 * messaging interface.
	 * @param size the size of the message
	 * @param message one of the messages defined in MPEG-4 IPMP Extension, or a user-defined message
	 * @return true if successful
	 */
	// by Liao Xuan
	//virtual bool ReceiveMessage(ToolMessage*) = 0;
	virtual bool ReceiveMessage(IPMP_ToolMessageBase* msg) = 0;
};


/**
  *	Message Router interface.
  * Abstract class to be implemented by the Message Router.
  */
class MR_Interface : public IPMPTool_Interface
{
public:
	/**
	 * This constants enumerates the possible access permission of an Elementary Stream
	 */
	enum ACCESS_PERMISSION
	{
		ALLOWED = 1,			///< terminal can proceed processing the stream
		DENIED = 2,				///< don't process and discard the stream
		PROTECTED = 3			///< wait for a GO
	};
 

	/**
	 * Setting tool manager's pointer, so that the MR can use it to access TM's member functions
	 * @param tmPointer a pointer to an object derived from TM_interface
	 * @return true if successful
	 */
	virtual bool SetTMPointer( TM_Interface * tmPointer) = 0;


	/**
	 * Called by the Terminal to process an OD
	 * @param pOD pointer to the object descriptor to be processed by the MR
	 * @return true if successful
	 */
	virtual bool ProcessObjectDescriptor( ObjectDescriptor* pOD ) = 0;

	/**
	 * Called by the Terminal to inform the MR that an OD is not valid anymore.
	 * @param pOD pointer to the OD to be removed
	 * @return true if successful
	 */
	virtual bool RemoveObjectDescriptor( ObjectDescriptor* pOD ) = 0;

	/**
	 * Called by the Terminal to process an IPMPToolDescriptor.
	 * @param descriptor the IPMPToolDescriptor to be processed by the MR
	 * @return true if successful
	 * NOTE: this descriptor object will be deleted after calling this method. The IPMPX
	 * system shall take this into account and copy it into its own memory space if necessary
	 */
	virtual bool ProcessIPMPDescriptor(IPMP_Descriptor* descriptor) =0;

	/**
	 * Called by the Terminal to inform the MR that an IPMPToolDescriptor is not valid anymore.
	 * @param descriptor pointer to the IPMPToolDescriptor to be removed
	 * @return true if successful
	 */
	virtual bool RemoveIPMPDescriptor(IPMP_Descriptor* descriptor) = 0;

	/**
	 * Called by the Terminal to process an Elementary Stream Descriptor.
	 * @param pESD pointer to the Elementary Stream Descriptor to be processed by the MR
	 * @return true if successful
	 */
	virtual enum ACCESS_PERMISSION ProcessESDescriptor(ES_Descriptor* pESD) = 0;
	

	/**
	 * Called by the Terminal to inform the MR that an ESDescriptor is not valid anymore.
	 * @param pESD pointer to the Elementary Stream Descriptor to be removed 
	 * @return true if successful
	 */
	virtual bool RemoveESDescriptor( ES_Descriptor* pESD ) = 0;

	/**
	 * Called by the Terminal to create a sink for an IPMP stream.
	 * @param pOD a pointer to the Object Descriptor where the ipmp stream is declared
	 * @param pESD a pointer to the relevant Elementary Stream Descriptor
	 * @param ipmpStream the MediaStream object handling the IPMP stream
	 * @return true if successful
	 */
	virtual bool SetUpIPMPStream(ObjectDescriptor *pOD, ES_Descriptor* pESD, MediaStream* ipmpStream) = 0;

	/**
	 * Called by the Terminal to destroy a sink for an IPMP stream, that was previously created by the Terminal
	 * using the SetUpIPMPStream method.
	 * @param pESD a pointer to the relevant Elementary Stream Descriptor
	 * @return true if successful
	 */
	virtual bool RemoveIPMPStream(ES_Descriptor *pESD) = 0;

};

/**
 *	IPMP Tool Manager interface.
 *  Abstract class to be implemented by the Tool Manager.
 */
class TM_Interface
{
public:
	/**
	 * setting message router's pointer, so that the TM can use it to access MR's member functions
	 * @param tmPointer the pointer to the Message Router
	 * @return true if successful
	 */
	virtual bool SetMRPointer( MR_Interface * tmPointer) = 0;

	/**
	 * Called by the MR to indicate a tool is no longer needed.
	 * @param toolPtr the pointer to the tool to be disconnected
	 * @return true if successful
	 */
	virtual bool DisconnectTool( void* toolPtr) = 0;

	/**
	 * Called by the MR to request a tool be instantiated.
	 * @param OD_id the id of the Object Descriptor in which scope the tool will run
	 * @param pESD a pointer to the ESD of the Elementary Stream protected by the tool
	 * @param toolDescriptor the descriptor carrying initialization information for the tool
	 * @return the pointer to the tool
	 */
	virtual void* ConnectTool(ES_Descriptor *pESD, IPMP_Descriptor* toolDescriptor) = 0;


	/**
	 * Called by the Terminal to pass a Tool elementary stream
	 * @param tool_ES the interface to the media stream implemented in IM1
	 * @param tool_ESD the relevant Elementary Stream Descriptor
	 * @return true if successful
	 */
	virtual bool ReceiveToolES(MediaStream *tool_ES, ES_Descriptor *tool_ESD) = 0;

	/**
	 * Called by the Terminal when a Tool elementary stream is removed
	 * @param pESD a pointer to the relevant Elementary Stream Descriptor
	 * @return true if successful
	 */
	virtual bool RemoveToolES(ES_Descriptor *pESD) = 0;

	/**
	 * Called by the Terminal to pass the tool list.
	 * @param toolList the list of tools as defined in MPEG-4 IPMP Extension
	 * @return true if successful
	 */
	virtual bool ReceiveIPMP_ToolListDescriptor( IPMP_ToolListDescriptor* toolList ) = 0;
};

/**
 * container class to hold the ToolManager and MessageRouter classes;
 * the Terminal will do all its interactions with IPMP through the single "IPMPServices" class.  
 * There is one instance of this class for each MPEG-4 "service", since each "service" defines a namescope
 * for ES id's.
 * This class iherits all public methods of TM_Interface and MR_Interface
 */
class IPMPServices : public TM_Interface, public MR_Interface
{
	// all methods are inherited from TM_Interface and MR_Interface
};


#endif
// SOLARIS PORTING - Added a new line

