//	This software module was originally developed by Craig A. Schultz 
//	(AccessTicket Systems Inc.)
//
//	in the course of development of the MPEG-4 Systems 
//	(ISO/IEC 14496-1) standard. This software module is an implementation 
//	of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified 
//	by the MPEG-4 Systems (ISO/IEC 14496-1) standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	AccessTicket Systems Inc. retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2000. 

#ifndef _IPMP_MESSAGES_H_
#define _IPMP_MESSAGES_H_

#include "IPMPXinterface.h"
#include "IPMP_MessagesTags.h"
#include "odtags.h"
//#include "odp.h"
#include "ODP.h" // SOLARIS PORTING - typo
#include "CrSdl.h"

/****** added by Liao Xuan *********************/
/*to support binary stream implementation*******/
#include "BitField.h"
#include "IPMP_ByteArray.h"
/****** End :add by Liao Xuan ******************/

enum EventTypes{
	CONNECTED,
	CONNECTIONFAILED,
	DISCONNECTED,
	DISCONNECTIONFAILED,
	//WATERMARKDETECTED,
	WATERMARKDETECTED // SOLARIS PORTING - took off ,
};

enum WatermarkOps{
	INSERT_WM,
	EXTRACT_WM,
	REMARK_WM,
	DETECT_COMPRESSION
};

enum WatermarkPayload{
	WM_PAYLOAD,
	WM_NOPAYLOAD,
	NO_WM,
	WM_UNKNOWN
};

enum WatermarkCompression{
	COMPRESSION,
	NO_COMPRESSION,
	COMP_UNKNOWN
};

// The watermark format IDs should be maintained
// by an RA but are given here for testing purposes
// only.
enum WatermarkFormats{
	/**Changed by Liao Xuan,UK NB Comments *******/
	//PCM,
	PCM=1,
	/**End:Changed by Liao Xuan,UK NB Comments ***/
	YUV
};

enum EventScope{
	PRESENTATION,
	MY_OD,
	//MY_ESD,
	MY_ESD // SOLARIS PORTING - took off ,
};


class ByteArray : public SDLDescriptor
{ 
	OD_DECLARE_DESCRIPTOR
public:
	ByteArray()
		: SDLDescriptor(true)
    {  }

    // Member
	SDLArray<8>  array;
	public:
	// Copy
	ByteArray* Copy()
	{	
		ByteArray* copy = new ByteArray;
		copy->array = array;
		return copy;
	}
};

class ParametricDescriptor : public SDLDescriptor
{ OD_DECLARE_DESCRIPTOR
public:
    ParametricDescriptor()
      : SDLDescriptor(true)
     {  }

    // Member
	//by Liao Xuan-- according to ISO/IEC 14496-1:2001/FDAM 3:2003
	//ByteArray				Class;
	//ByteArray				subClass;
	//ByteArray				typeData;
	//ByteArray				type;
	//ByteArray				addedData;
	SDLClass<class ByteArray>	Class;
	SDLClass<class ByteArray>	subClass;
	SDLClass<class ByteArray>	typeData;
	SDLClass<class ByteArray>	type;
	SDLClass<class ByteArray>	addedData;
	//by Liao Xuan-- according to ISO/IEC 14496-1:2001/FDAM 3:2003
};

class IPMP_ParamtericDescription : public IPMP_Data_BaseClass
{ OD_DECLARE_DESCRIPTOR
public:
    // Member
	SDLClass<class ByteArray>	descriptionComment;
    SDLInt<8>	majorVersion;
    SDLInt<8>	minorVersion;
    //by Liao Xuan-- according to ISO/IEC 14496-1:2001/FDAM 3:2003
    //SDLInt<16>  numOfDescriptions;
	SDLInt<32>  numOfDescriptions;
    //SDLClassArray<ByteArray,1,255>  description;
	SDLClassArray<class ParametricDescriptor,1,255>  description;
	//by Liao Xuan-- according to ISO/IEC 14496-1:2001/FDAM 3:2003


   	// Copy
	IPMP_Data_BaseClass* Copy()
	{	IPMP_ParamtericDescription* copy = new IPMP_ParamtericDescription;

		copy->Version = Version;
		copy->dataID = dataID;
		copy->descriptionComment = descriptionComment;
		copy->majorVersion = majorVersion;
		copy->minorVersion = minorVersion;
		copy->numOfDescriptions = numOfDescriptions;
		copy->description = description;
		
		return copy;
	}
// add by Liao Xuan
private:
	virtual void DescEncDec (ODCoder *pODCoder, OPTYPE op);
// add by Liao Xuan
};

class IPMP_ToolListDescriptor : public BaseDescriptor
{
	OD_DECLARE_DESCRIPTOR
public:
	SDLClassArray<class IPMP_Tool, 0, 255> ipmpTools;
	
// by Liao Xuan
private:
	virtual void DescEncDec (ODCoder *pODCoder, OPTYPE op);
// by Liao Xuan
};

class IPMP_Tool : public BaseDescriptor
{
	OD_DECLARE_DESCRIPTOR	//this is not a descriptor class, but we want to use SDL anyway

public:
	// Members
	SDLLongInt<128>								IPMP_ToolID;
	SDLInt<1>									isAltGroup;
	SDLInt<1>									isParametric;
	SDLInt<6>									reserved;	//=0b0000.00
	SDLInt<8>									numAlternates;
	SDLArray<128>								Alt_IPMP_ToolIDs;
	//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003
	//SDLClassArray<IPMP_ParamtericDescription,0,255>	toolParamDescr;
	SDLClass<class IPMP_ParamtericDescription>	toolParamDescr;

	SDLClassArray<class ByteArray,0,255>		ToolURLs;
	//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003

private:
	virtual void DescEncDec (ODCoder *pODCoder, OPTYPE op);
};

class IPMPToolES_DecoderConfig: public DecoderSpecificInfo
{
	OD_DECLARE_DESCRIPTOR

public:
	SDLLongInt<128> IPMP_ToolID;
	//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003
	SDLInt<1> hasFormatID;//added
	SDLInt<1> hasPackageID;//added
	SDLInt<6> reserved;	//=0b0000.00
	//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003
	
	// if hasFormatID == true
	SDLInt<32> Tool_Format_ID;
	// if hasPackageID == true
	SDLInt<16> Tool_Package_ID;
	
	//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003
	SDLClass<class ByteArray>	opaqueData;	
private:
	virtual void DescEncDec (ODCoder *pODCoder, OPTYPE op);
//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003
};

// Messages from bitstream, as well as other tools.
class IPMP_OpaqueData : public IPMP_Data_BaseClass
{	
	OD_DECLARE_DESCRIPTOR
public:
	// Members
	//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003
	SDLClass<class ByteArray>		opaqueData;
	//SDLArray<8>	opaqueData;
	//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003

public:
	// Copy
	IPMP_Data_BaseClass* Copy()
	{	
		IPMP_OpaqueData* copy = new IPMP_OpaqueData;
		copy->Version = Version;
		copy->dataID = dataID;
		copy->opaqueData = opaqueData;
		
		return copy;
	}
/********** Added by Liao Xuan,UK NB Comments *******/
private:
	virtual void DescEncDec (ODCoder *pODCoder, OPTYPE op);
/********** End:Added by Liao Xuan,UK NB Comments****/

};

class IPMP_AudioWatermarkingInit : public IPMP_Data_BaseClass
{	OD_DECLARE_DESCRIPTOR
public:
	// Members
	SDLInt<8>	inputFormat;
	SDLInt<4>	requiredOp;
	SDLInt<1>	hasOpaqueData;
	SDLInt<3>	reserved;
	SDLInt<8>	nChannels;
	SDLInt<8>	bitPerSample;
	SDLInt<32>	frequency;
	//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003
	SDLInt<16>	wmPayloadLen;//added
	SDLArray<8>	wmPayload;
	SDLInt<16>	wmRecipientId;
	SDLInt<16>	opaqueDataSize;//added
	//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003
	SDLArray<8>	opaqueData;

	// Copy
	IPMP_Data_BaseClass* Copy()
	{	
		IPMP_AudioWatermarkingInit* copy = new IPMP_AudioWatermarkingInit;

		copy->Version = Version;
		copy->dataID = dataID;
		copy->inputFormat = inputFormat;
		copy->requiredOp = requiredOp;
		copy->hasOpaqueData = hasOpaqueData;
		copy->nChannels = nChannels;
		copy->bitPerSample = bitPerSample;
		copy->frequency = frequency;
		//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003
		copy->wmPayloadLen = wmPayloadLen;//added
		copy->wmPayload = wmPayload;
		copy->wmRecipientId = wmRecipientId;
		copy->opaqueDataSize = opaqueDataSize;//added
		//End:by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003
		copy->opaqueData = opaqueData;

		return copy;
	}

private:
	virtual void DescEncDec (ODCoder *pODCoder, OPTYPE op);

};

class IPMP_VideoWatermarkingInit : public IPMP_Data_BaseClass
{	OD_DECLARE_DESCRIPTOR
public:
	// Members
	SDLInt<8>	inputFormat;
	SDLInt<4>	requiredOp;
	SDLInt<1>	hasOpaqueData;
	SDLInt<3>	reserved;
	SDLInt<16>	frame_horizontal_size;
	SDLInt<16>	frame_vertical_size;
	SDLInt<8>	chroma_format;
	//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003
	SDLInt<16>	wmPayloadLen;//added
	SDLArray<8>	wmPayload;
	SDLInt<16>	wmRecipientId;
	SDLInt<16>	opaqueDataSize;//added
	//End: by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003
	SDLArray<8>	opaqueData;

	// Copy
	IPMP_Data_BaseClass* Copy()
	{	IPMP_VideoWatermarkingInit* copy = new IPMP_VideoWatermarkingInit;

		copy->Version = Version;
		copy->dataID = dataID;
		copy->inputFormat = inputFormat;
		copy->requiredOp = requiredOp;
		copy->hasOpaqueData = hasOpaqueData;
		copy->frame_horizontal_size = frame_horizontal_size;
		copy->frame_vertical_size = frame_vertical_size;
		copy->chroma_format = chroma_format;
		//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003
		copy->wmPayloadLen = wmPayloadLen;//added
		copy->wmPayload = wmPayload;
		copy->wmRecipientId = wmRecipientId;
		SDLInt<16>	opaqueDataSize;//added
		//End: by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003
		copy->opaqueData = opaqueData;
	
		return copy;
	}

private:
	virtual void DescEncDec (ODCoder *pODCoder, OPTYPE op);

};

class BufferInfoStructure : public SDLDescriptor
{	
	OD_DECLARE_DESCRIPTOR
public:
	BufferInfoStructure() 
		: SDLDescriptor(true)
	{  }
	
	// Members
	SDLInt<128>		cipher_Id; 
	SDLInt<8>		syncBoundary;
	SDLInt<1>		isBlock;
	SDLInt<7>		reserved;
	SDLInt<8>		mode;  
	SDLInt<16>		blockSize;
	SDLInt<16>		keySize;
	//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003
	//SDLArray<8>		streamCipherSpecificInitInfo;
	SDLClass<class ByteArray>	streamCipherSpecificInitInfo;
	//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003

private:
	virtual void DescEncDec (ODCoder *pODCoder, OPTYPE op);

};

class FieldStructure : public SDLDescriptor
{	OD_DECLARE_DESCRIPTOR
public:
	FieldStructure() 
		: SDLDescriptor(true)
	{  }
	
	// Members
	SDLInt<8>		field_Id; 
	SDLInt<3>		field_Scope;
	SDLInt<5>		reserved;
	SDLInt<8>		buf; 
	SDLInt<1>		isMapped;
	SDLInt<7>		reserved1;
	SDLInt<1>		sendMapTable;
	SDLInt<1>		isShuffled;
	SDLInt<6>		reserved2;
	//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003
	SDLInt<16>		sizMapTable;// added
	//SDLArray<8>	mappingTable;
	SDLArray<16>	mappingTable;
	//SDLArray<8>		shuffleSpecificInfo;
	SDLClass<class ByteArray>	shuffleSpecificInfo;
	//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003

private:
	virtual void DescEncDec (ODCoder *pODCoder, OPTYPE op);

};

class IPMP_SelectiveDecryptionInit : public IPMP_Data_BaseClass
{	OD_DECLARE_DESCRIPTOR
public:
	// Members
	SDLInt<8>											mediaTypeExtension; 
	SDLInt<8>											mediaTypeIndication;
	SDLInt<8>											profileLevelIndication; 
	SDLInt<8>											compliance;
	SDLInt<8>											numBufs;// added by Liao Xuan
	SDLClassArray<class BufferInfoStructure, 0, 255>	bufferInfo;
	SDLInt<1>											isContentSpecific;
	SDLInt<7>											reserved;
	//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003
	SDLInt<8>											numFields;// added

	SDLClassArray<class FieldStructure, 0, 255>			fields;
	//SDLArray<8>										decryptionMask;
	SDLInt<16>											nSegments;// added
	SDLArray<16>										RLE_Data;// added
	//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003
	
	// Copy
	IPMP_Data_BaseClass* Copy()
	{	
		
		IPMP_SelectiveDecryptionInit* copy = new IPMP_SelectiveDecryptionInit;

		copy->Version = Version;
		copy->dataID = dataID;
		copy->mediaTypeExtension = mediaTypeExtension;
		copy->mediaTypeIndication = mediaTypeIndication;
		copy->profileLevelIndication = profileLevelIndication;
		copy->compliance = compliance;
		copy->bufferInfo = bufferInfo;
		copy->isContentSpecific = isContentSpecific;
		//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003
		copy->numFields = numFields;// added
		copy->fields = fields;
		copy->nSegments = nSegments;// added
		copy->RLE_Data = RLE_Data;// added
		//copy->decryptionMask = decryptionMask;
		//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003	
		return copy;
	}

private:
	virtual void DescEncDec (ODCoder *pODCoder, OPTYPE op);

};

class IPMP_KeyData : public IPMP_Data_BaseClass
{	OD_DECLARE_DESCRIPTOR
public:
	// Members
	/********** Change by Liao Xuan -- SO/IEC 14496-1:2001/FDAM 3:2003 */
	//SDLString			keyBody;
	SDLClass<class ByteArray>	keyBody;
	SDLInt<1>			hasStartDTS;
	SDLInt<1>			hasStartPacketID;
	SDLInt<1>			hasExpireDTS;
	SDLInt<1>			hasExpirePacketID;
	SDLInt<4>			reserved;  // 0b0000
	SDLInt<64>			startDTS;
	SDLInt<32>			startPacketID;
	SDLInt<64>			expireDTS;
	SDLInt<32>			expirePacketID;
	//SDLArray<8>		opaqueData;
	SDLClass<class ByteArray>	opaqueData;
	/*End: Change by Liao Xuan ****************************************/

	// Copy
	IPMP_Data_BaseClass* Copy()
	{	
		IPMP_KeyData* copy = new IPMP_KeyData;
		
		copy->Version = Version;
		copy->dataID = dataID;
		copy->keyBody = keyBody;
		copy->hasStartDTS = hasStartDTS;
		copy->hasStartPacketID = hasStartPacketID;
		copy->hasExpireDTS = hasExpireDTS;
		copy->hasExpirePacketID = hasExpirePacketID;
		copy->startDTS = startDTS;
		copy->startPacketID = startPacketID;
		copy->expireDTS = expireDTS;
		copy->expirePacketID = expirePacketID;
		copy->opaqueData = opaqueData;
		
		return copy;
	}

private:
	virtual void DescEncDec (ODCoder *pODCoder, OPTYPE op);

};

class IPMP_SendAudioWatermark : public IPMP_Data_BaseClass
{	
	OD_DECLARE_DESCRIPTOR
public:
	// Members
	SDLInt<2>	wm_status;
	SDLInt<2>	compression_status;
	SDLInt<1>	hasOpaqueData;
	SDLInt<3>	reserved;
	//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003	
	SDLClass<class ByteArray>	payload;
	SDLClass<class ByteArray>	opaqueData;
	//SDLArray<8> payload;
	//SDLArray<8> opaqueData;
	//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003

	// Copy
	IPMP_Data_BaseClass* Copy()
	{	
		IPMP_SendAudioWatermark* copy = new IPMP_SendAudioWatermark;
		
		copy->Version = Version;
		copy->dataID = dataID;
		copy->wm_status = wm_status;
		copy->compression_status = compression_status;
		copy->hasOpaqueData = hasOpaqueData;
		copy->payload = payload;
		copy->opaqueData = opaqueData;

		return copy;
	}

private:
	virtual void DescEncDec (ODCoder *pODCoder, OPTYPE op);

};

class IPMP_SendVideoWatermark : public IPMP_Data_BaseClass
{	OD_DECLARE_DESCRIPTOR
public:
	// Members
	SDLInt<2>	wm_status;
	SDLInt<2>	compression_status;
	SDLInt<1>	hasOpaqueData;
	SDLInt<3>	reserved;
	//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003	
	SDLClass<class ByteArray>	payload;
	SDLClass<class ByteArray>	opaqueData;
	//SDLArray<8> payload;
	//SDLArray<8> opaqueData;
	//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003

	// Copy
	IPMP_Data_BaseClass* Copy()
	{	IPMP_SendVideoWatermark* copy = new IPMP_SendVideoWatermark;
		
		copy->Version = Version;
		copy->dataID = dataID;
		copy->wm_status = wm_status;
		copy->compression_status = compression_status;
		copy->hasOpaqueData = hasOpaqueData;
		copy->payload = payload;
		copy->opaqueData = opaqueData;

		return copy;
	}

private:
	virtual void DescEncDec (ODCoder *pODCoder, OPTYPE op);

};

class IPMP_RightsData : public IPMP_Data_BaseClass
{	OD_DECLARE_DESCRIPTOR
public:
	// Members
	//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003	
	//SDLArray<8>						rightsInfo;
	SDLClass<class ByteArray>				rightsInfo;
	//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003	

	// Copy
	IPMP_Data_BaseClass* Copy()
	{	
		IPMP_RightsData* copy = new IPMP_RightsData;
		copy->Version = Version;
		copy->dataID = dataID;
		copy->rightsInfo = rightsInfo;
		return copy;
	}
private:
	virtual void DescEncDec (ODCoder *pODCoder, OPTYPE op);

};

class IPMP_Secure_Container : public IPMP_Data_BaseClass
{	OD_DECLARE_DESCRIPTOR
public:
	// Members
	SDLInt<1>			hasEncryption;
	SDLInt<1>			hasMAC;
	SDLInt<1>			isMACEncrypted;
	//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003	
	//SDLInt<1>			Reserved; // 0b0
	SDLInt<5>			Reserved; // 0b0.0000
	//SDLArray<8>			encryptedData;
	//SDLArray<8>			MAC;
	SDLClass<class ByteArray>			encryptedData;
	SDLClass<class ByteArray>			MAC;
	SDLClass<class IPMP_Data_BaseClass>	protectedMsg; // added 
	//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003
	
	// Copy
	IPMP_Data_BaseClass* Copy()
	{	IPMP_Secure_Container* copy = new IPMP_Secure_Container;
		
		copy->Version = Version;
		copy->dataID = dataID;
		copy->hasEncryption = hasEncryption;
		copy->hasMAC = hasMAC;
		copy->isMACEncrypted = isMACEncrypted;
		copy->encryptedData = encryptedData;
		copy->MAC = MAC;
		//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003
		copy->protectedMsg = protectedMsg;
		//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003
	
		return copy;
	}

private:
	virtual void DescEncDec (ODCoder *pODCoder, OPTYPE op);

};

class IPMP_AddToolNotificationListener : public IPMP_Data_BaseClass
{	OD_DECLARE_DESCRIPTOR
public:
	// Members
	//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003
	//SDLInt<8>	scope; // EventScope
	SDLInt<3>	scope; //EventScope
	SDLInt<5>	reserved; // added reserve 0b0000.0
	SDLInt<8>	eventTypeCount;// added
	SDLArray<8> eventType;// added
	//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003

	// Copy
	IPMP_Data_BaseClass* Copy()
	{	
		IPMP_AddToolNotificationListener* copy = new IPMP_AddToolNotificationListener;
		
		copy->Version = Version;
		copy->dataID = dataID;
		copy->scope = scope;
		//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003
		copy->reserved = reserved;//added
		copy->eventTypeCount = eventTypeCount;//added
		//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003
		copy->eventType = eventType;
		
		return copy;
	}
private:
	virtual void DescEncDec (ODCoder *pODCoder, OPTYPE op);
};

class IPMP_RemoveToolNotificationListener : public IPMP_Data_BaseClass
{	OD_DECLARE_DESCRIPTOR
public:
	// Members
	//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003
	SDLInt<8>	eventTypeCount;//added
	//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003
	SDLArray<8>	eventType; //EventTypes

	// Copy
	IPMP_Data_BaseClass* Copy()
	{	
		IPMP_RemoveToolNotificationListener* copy = new IPMP_RemoveToolNotificationListener;
		
		copy->Version = Version;
		copy->dataID = dataID;
		copy->eventType = eventType;
		//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003
		copy->eventTypeCount = eventTypeCount;
		//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003
		return copy;
	}
private:
	virtual void DescEncDec (ODCoder *pODCoder, OPTYPE op);
};

class IPMP_InitAuthentication : public IPMP_Data_BaseClass
{	OD_DECLARE_DESCRIPTOR
public:
	// Members
	SDLInt<32>			contextID;
	//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003
	//SDLInt<32>		authType;
	SDLInt<8>			authType;
	//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003

	// Copy
	IPMP_Data_BaseClass* Copy()
	{	
		IPMP_InitAuthentication* copy = new IPMP_InitAuthentication;
		
		copy->Version = Version;
		copy->dataID = dataID;
		copy->contextID = contextID;
		copy->authType = authType;
		
		return copy;
	}
private:
	virtual void DescEncDec (ODCoder *pODCoder, OPTYPE op);

};
/*
// For use in IPMP_MutualAuthentication
class AlgorithmDescriptor : public SDLDescriptor
{	OD_DECLARE_DESCRIPTOR
public:
	AlgorithmDescriptor() 
		: SDLDescriptor(true)
	{  }
	

	SDLInt<32>	regAuthID;
	SDLArray<8>	specAlgoID;
	SDLArray<8>	OpaqueData;
};

// For use in IPMP_MutualAuthentication
class KeyDescriptor : public AlgorithmDescriptor
{	OD_DECLARE_DESCRIPTOR
public:
	KeyDescriptor() 
	{ AlgorithmDescriptor(); }
	
	SDLArray<8>			keyBody;
};

// For use in IPMP_MutualAuthentication
class Certificate : public SDLDescriptor
{	OD_DECLARE_DESCRIPTOR
public:
	Certificate() 
		: SDLDescriptor(true)
	{  }
	
	SDLArray<8>	certificate;
};

class IPMP_MutualAuthentication : public IPMP_Data_BaseClass
{	OD_DECLARE_DESCRIPTOR
public:
	// Members
	SDLInt<1>									requestNegotiation;
	SDLInt<1>									successNegotiation;
	SDLInt<1>									failedNegotiation;
	SDLInt<1>									inclAuthenticationData;
	SDLInt<1>									inclAuthCodes;
	SDLInt<3>									reserved;
	SDLClassArray<class AlgorithmDescriptor, 0, 255>	candidateAlgorithms;
	SDLClassArray<class AlgorithmDescriptor, 0, 255>	agreedAlgorithms;
	SDLArray<8>									AuthenticationData;
	SDLInt<32>									type;
	SDLInt<32>									certType;
	SDLClassArray<class Certificate, 0, 255>			certificates;
	SDLClass<class KeyDescriptor, 0>			publicKey;
	SDLArray<8>									opaque;
	SDLArray<8>									authCodes;

	// Copy
	IPMP_Data_BaseClass* Copy()
	{	IPMP_MutualAuthentication* copy = new IPMP_MutualAuthentication;
		
		copy->Version = Version;
		copy->dataID = dataID;
		copy->requestNegotiation	= requestNegotiation;
		copy->successNegotiation	= successNegotiation;
		copy->failedNegotiation	= failedNegotiation;
		copy->inclAuthenticationData	= inclAuthenticationData;
		copy->inclAuthCodes	= inclAuthCodes;
		copy->candidateAlgorithms	= candidateAlgorithms;
		copy->agreedAlgorithms = agreedAlgorithms;
		copy->AuthenticationData = AuthenticationData;
		copy->certType = certType;
		copy->certificates = certificates;
		copy->publicKey = publicKey;
		copy->opaque = opaque;
		copy->authCodes = authCodes;
		
		return copy;
	}

private:
	virtual void DescEncDec (ODCoder *pODCoder, OPTYPE op);

};
*/
// by Liao Xuan
// new added class, base class of  AlgorithmDescriptor and KeyDescriptor
class BaseAuthenticationDescriptor : public SDLDescriptor
{	
public:
	BaseAuthenticationDescriptor()
	: SDLDescriptor(true)
	{  }
	// Members
};

// For use in IPMP_MutualAuthentication
//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003
//class AlgorithmDescriptor : public SDLDescriptor
class AlgorithmDescriptor : public BaseAuthenticationDescriptor
{	
	OD_DECLARE_DESCRIPTOR
public:
	
	AlgorithmDescriptor() 
	{ BaseAuthenticationDescriptor(); }
	
	SDLInt<1>	isRegistered;//added
	SDLInt<7>	reserved ;// added, 0b0000.000;
	
	//SDLInt<32>				regAuthID;
	SDLInt<16>					regAuthID;
	//SDLArray<8>				specAlgoID;
	SDLClass<class ByteArray>	specAlgoID;
	//SDLArray<8>				OpaqueData;
	SDLClass<class ByteArray>	OpaqueData;

private:
	virtual void DescEncDec (ODCoder *pODCoder, OPTYPE op);// added
//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003
};

// For use in IPMP_MutualAuthentication
//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003
//class KeyDescriptor : public AlgorithmDescriptor
class KeyDescriptor : public BaseAuthenticationDescriptor
{	OD_DECLARE_DESCRIPTOR
public:

	//KeyDescriptor() 
	//{ AlgorithmDescriptor(); }
	KeyDescriptor() 
	{ BaseAuthenticationDescriptor(); }

	SDLArray<8>			keyBody;
};
//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003

// new added class, For use in IPMP_TrustSecurityMetadata
class DateClass : public SDLDescriptor
{	
	OD_DECLARE_DESCRIPTOR
public:
	DateClass() 
		: SDLDescriptor(true)
	{  }
	SDLInt<40>		Date;
};


// new added class, For use in IPMP_TrustSecurityMetadata
class TrustSpecification : public SDLDescriptor
{
	OD_DECLARE_DESCRIPTOR
public:
	TrustSpecification() 
		: SDLDescriptor(true)
	{  }
	SDLInt<1>						hasCCBasedTrustMetadata;
	SDLInt<7>						reserved;
	SDLClass<class DateClass>		startDate; // added 
	SDLInt<2>						attackerProfile;
	SDLInt<6>						reserved2;
	SDLInt<32>						trustedDuration;
	SDLClass<class ByteArray>		CCTrustMetadata;
private:
	virtual void DescEncDec (ODCoder *pODCoder, OPTYPE op);// added
};

class TrustedTool : public SDLDescriptor
{
	OD_DECLARE_DESCRIPTOR
public:
	TrustedTool() 
		: SDLDescriptor(true)
	{  }
	SDLInt<128>						toolID;
	SDLClass<class DateClass>		AuditDate; // added 
	SDLInt<16>						numTrustSpecification;
	SDLClassArray<class TrustSpecification, 0, 255> trustSpecification;

private:
	virtual void DescEncDec (ODCoder *pODCoder, OPTYPE op);// added
};

//new added class, For use in IPMP_MutualAuthentication
class IPMP_TrustSecurityMetadata : public IPMP_Data_BaseClass
{
	OD_DECLARE_DESCRIPTOR
public:
	SDLInt<16>	numTrustedTools;
	SDLClassArray<class TrustedTool, 0, 255>	trustedtools;

	// Copy
	IPMP_Data_BaseClass* Copy()
	{	
		IPMP_TrustSecurityMetadata* copy = new IPMP_TrustSecurityMetadata;
		copy->Version = Version;
		copy->dataID = dataID;
		copy->numTrustedTools = numTrustedTools;
		copy->trustedtools = trustedtools;

		return copy;
	}
private:
	virtual void DescEncDec (ODCoder *pODCoder, OPTYPE op);// added
};
//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003


// For use in IPMP_MutualAuthentication
class Certificate : public SDLDescriptor
{	OD_DECLARE_DESCRIPTOR
public:
	Certificate() 
		: SDLDescriptor(true)
	{  }
	//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003
	//SDLArray<8>				certificate;
	SDLClass<class ByteArray>	certificate;
	//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003

};

class IPMP_MutualAuthentication : public IPMP_Data_BaseClass
{	
	OD_DECLARE_DESCRIPTOR
public:
	// Members
	SDLInt<1>									requestNegotiation;
	SDLInt<1>									successNegotiation;
	SDLInt<1>									failedNegotiation;
	SDLInt<1>									inclAuthenticationData;
	SDLInt<1>									inclAuthCodes;
	SDLInt<3>									reserved;
	
	//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003
	SDLInt<8>									nCandidateAlgorithm;//added
	SDLClassArray<class AlgorithmDescriptor, 0, 255>	candidateAlgorithms;
	SDLInt<8>									nAgreedAlgorithm;//added
	SDLClassArray<class AlgorithmDescriptor, 0, 255>	agreedAlgorithms;
	SDLArray<8>									AuthenticationData;
	//SDLInt<32>								type;
	SDLInt<8>									type;
	SDLInt<32>									nCertificate;//added
	SDLInt<32>									certType;
	SDLClassArray<class Certificate, 0, 255>	certificates;
	SDLClass<class KeyDescriptor	>			publicKey;
	//SDLArray<8>								opaque;
	SDLClass<class ByteArray>					opaque;
	SDLClass<class IPMP_TrustSecurityMetadata>	trustData;//added
	//SDLArray<8>								authCodes;
	SDLClass<class ByteArray>					authCodes;
	//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003
	
	// Copy
	IPMP_Data_BaseClass* Copy()
	{	
		//_asm{int 3};
		IPMP_MutualAuthentication* copy = new IPMP_MutualAuthentication;
		
		copy->Version = Version;
		copy->dataID = dataID;
		copy->requestNegotiation	= requestNegotiation;
		copy->successNegotiation	= successNegotiation;
		copy->failedNegotiation	= failedNegotiation;
		copy->inclAuthenticationData	= inclAuthenticationData;
		copy->inclAuthCodes	= inclAuthCodes;
		
		//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003
		copy->nCandidateAlgorithm = nCandidateAlgorithm;//added
		copy->candidateAlgorithms	= candidateAlgorithms;
		copy->nAgreedAlgorithm = nAgreedAlgorithm;//added
		//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003
		
		copy->agreedAlgorithms = agreedAlgorithms;
		copy->AuthenticationData = AuthenticationData;
		copy->type = type;//added
		copy->nCertificate = nCertificate;//added
		copy->certType = certType;
		copy->certificates = certificates;
		copy->publicKey = publicKey;
		copy->opaque = opaque;
		copy->trustData = trustData;//added
		//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003
		copy->authCodes = authCodes;
		
		return copy;
	}

private:
	virtual void DescEncDec (ODCoder *pODCoder, OPTYPE op);

};

// For use in IPMP_UserQuery 
// Display text
class DTArray : public SDLDescriptor
{	
	OD_DECLARE_DESCRIPTOR
public:
	DTArray() 
		: SDLDescriptor(true)
	{  }
	
	//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003
	//SDLInt<32>				ID;
	SDLInt<16>					ID;
	//SDLString					displayText;
	SDLClass<class ByteArray>	displayText;
	//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003
};

//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003
// no used any more
/*
// For use in AltText
// Reply text
class RTArray : public SDLDescriptor
{	
	OD_DECLARE_DESCRIPTOR
public:
	RTArray() 
		: SDLDescriptor(true)
	{  }
	
	SDLInt<32>		ID;
	SDLInt<32>		SubID;
	SDLString		promptText;
};
*/
//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003


// For use in IPMP_UserQuery 
// Option selection
class OptionArray : public SDLDescriptor
{	OD_DECLARE_DESCRIPTOR
public:
	OptionArray() 
		: SDLDescriptor(true)
	{  }
	
		//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003
	//SDLInt<32>	ID;
	SDLInt<16>		ID;
	//SDLInt<32>	SubID;
	SDLInt<16>		SubID;
	SDLInt<1>		isExclusive;
	SDLInt<7>		reserved;//added
	//SDLString		promptText;
	SDLClass<class ByteArray>		promptText;
	//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003
};


// For use in IPMP_UserQuery
// Alternate text
class AltText : public SDLDescriptor
{	
	OD_DECLARE_DESCRIPTOR
public:
	AltText()
		: SDLDescriptor(true)
	{  }
	
	void SetFlagValues(	SDLInt<1> _inclDisplayTitle,
					SDLInt<1> _inclDisplayText,
					SDLInt<1> _needReplyText,
					SDLInt<1> _inclOptionSelect,
					SDLInt<1> _inclSMIL)
	{  	
		inclDisplayTitle = _inclDisplayTitle;
		inclDisplayText = _inclDisplayText;
		needReplyText = _needReplyText;
		inclOptionSelect = _inclOptionSelect;
		inclSMIL = _inclSMIL;
		isReferenced = false;
	}
	
	//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003
	SDLInt<1>								inclDisplayTitle;//added
    SDLInt<1>        						inclDisplayText;//added
    SDLInt<1>        						needReplyText;//added
    SDLInt<1>        						inclOptionSelect;//added
    SDLInt<1>        						inclSMIL;//added
	//SDLInt<32>							languageCode;
	SDLInt<24>								languageCode;
	SDLClass<class DTArray>					titleText;//added
	//SDLInt<32>							charCode;
	SDLInt<8>								nDisplayText;//added
	//SDLString								displayTitle;
	SDLClassArray<class DTArray, 0, 255>	displayText;
	SDLInt<8>								nPromptText;//added
	SDLClassArray<class QTArray, 0, 255>	promptText;//added
	//SDLClassArray<class RTArray, 0, 255>	replyText;
	SDLInt<8>								nOption;//added
	//SDLClassArray<class OptionArray, 0, 255>	optionSelect;
	SDLClassArray<class OptionArray, 0, 255>	option;
	SDLInt<1>								isReferenced;//added
	SDLInt<7>								reserved;//added
	//SDLArray<8>							SMILurl;
	SDLClass<class ByteArray>				SMIL_URL;
	//SDLString								SMIL;
	SDLClass<class ByteArray>				SMIL;
	//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003
	
private:
	virtual void DescEncDec (ODCoder *pODCoder, OPTYPE op);// added by Liao Xuan
};


//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003
// added class, For use in IPMP_UserQuery 
class QTArray : public SDLDescriptor
{	
	OD_DECLARE_DESCRIPTOR
public:
	QTArray() 
		: SDLDescriptor(true)
	{  }

	SDLInt<16>					ID;
	SDLInt<16>					SubID;
	SDLInt<1>					isHidden;
	SDLInt<7>					reserved;// 0b0000.000
	SDLClass<class ByteArray>	promptText;
};
//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003

class IPMP_UserQuery : public IPMP_Data_BaseClass
{	
	OD_DECLARE_DESCRIPTOR
public:
	// Members
	//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003
	SDLInt<1>			inclDisplayTitle;//added
    SDLInt<1>        	inclDisplayText;//added
    SDLInt<1>        	needReplyText;//added
    SDLInt<1>        	inclOptionSelect;//added
    SDLInt<1>        	inclSMIL;//added
    SDLInt<3>  			reserved; //added,  0b000
    SDLInt<24>			numOfAltText;//added
	
	//SDLClassArray<class AltText, 0, 255>	query;	
	SDLClassArray<class AltText, 0, 0xffff>	query;	
	//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003

	// Copy
	IPMP_Data_BaseClass* Copy()
	{	
		IPMP_UserQuery* copy = new IPMP_UserQuery;
		
		copy->Version = Version;
		copy->dataID = dataID;
		//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003
		copy->inclDisplayTitle = inclDisplayTitle;//added
      	copy->inclDisplayText = inclDisplayText;//added
        copy->needReplyText = needReplyText;//added
       	copy->inclOptionSelect = inclOptionSelect;//added
        copy->inclSMIL = inclSMIL;//added
   		copy->reserved = reserved; //added,  0b000
		copy->numOfAltText = numOfAltText;//added
		//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003

		copy->query = query;
		
		return copy;
	}
private:
	virtual void DescEncDec (ODCoder *pODCoder, OPTYPE op);// added by Liao Xuan
};

class IPMP_UserQueryResponse : public IPMP_Data_BaseClass
{	OD_DECLARE_DESCRIPTOR
public:
	// Members
	//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003
	SDLInt<1>			inclReplyText;//added
    SDLInt<1>			inclOptionSelect;//added
    SDLInt<6>			reserved ;////added, 0b0000.00
	//SDLInt<32>		languageCode;
	SDLInt<24>			languageCode;
	//SDLInt<32>		charCode;
	SDLInt<8>			nReplyText ;//added
	//SDLClassArray<class RTArray, 0, 255>	replyText;
	SDLClassArray<class QTArray, 0, 255>	replyText;
	//SDLInt<32>		numOfOptions;
	SDLInt<16>			numOfOptions;
	//SDLString			optionResults;
	SDLArray<1>*		optionResults;
	SDLArray<7>*		reserved2;//added

	// Copy
	IPMP_Data_BaseClass* Copy()
	{	IPMP_UserQueryResponse* copy = new IPMP_UserQueryResponse;
		
		copy->Version = Version;
		copy->dataID = dataID;
		/*
		copy->languageCode = languageCode;
		copy->charCode = charCode;
		copy->replyText = replyText;
		copy->numOfOptions = numOfOptions;
		copy->optionResults = optionResults;
		*/
		copy->inclReplyText		= inclReplyText;//added
		copy->inclOptionSelect	= inclOptionSelect;//added
		copy->reserved			= reserved;//added, 0b0000.00
		copy->languageCode		= languageCode;
		copy->nReplyText		= nReplyText;//added
		copy->replyText			= replyText;
		copy->numOfOptions      = numOfOptions;
		copy->optionResults		= optionResults;
		copy->reserved2			= reserved2;//added
		//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003
		return copy;
	}
private:
	virtual void DescEncDec (ODCoder *pODCoder, OPTYPE op);// added by Liao Xuan
};

class IPMP_ToolParamCapabilitiesQuery : public IPMP_Data_BaseClass
{	OD_DECLARE_DESCRIPTOR
public:
	// Members
	//SDLString			toolParamDesc;
	SDLClass<class IPMP_ParamtericDescription> toolParamDesc;

	// Copy
	IPMP_Data_BaseClass* Copy()
	{	
		
		IPMP_ToolParamCapabilitiesQuery* copy = new IPMP_ToolParamCapabilitiesQuery;
		
		copy->Version = Version;
		copy->dataID = dataID;
		copy->toolParamDesc = toolParamDesc;
		
		return copy;
	}
};

class IPMP_ToolParamCapabilitiesResponse : public IPMP_Data_BaseClass
{	OD_DECLARE_DESCRIPTOR
public:
	// Members
	SDLInt<1>			capabilitiesSupported;
	SDLInt<7>			Reserved; // 0b0000.000

	// Copy
	IPMP_Data_BaseClass* Copy()
	{	IPMP_ToolParamCapabilitiesResponse* copy = new IPMP_ToolParamCapabilitiesResponse;
		
		copy->Version = Version;
		copy->dataID = dataID;
		copy->capabilitiesSupported = capabilitiesSupported;
		
		return copy;
	}
};

class IPMP_GetTools : public IPMP_Data_BaseClass
{	OD_DECLARE_DESCRIPTOR
public:
	// Members

	// Copy
	IPMP_Data_BaseClass* Copy()
	{	IPMP_GetTools* copy = new IPMP_GetTools;
		
		copy->Version = Version;
		copy->dataID = dataID;

		return copy;
	}

};

class IPMP_GetToolsResponse : public IPMP_Data_BaseClass
{	OD_DECLARE_DESCRIPTOR
public:
	// Members
	//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003
	//SDLLongInt<128>	IPMP_ToolID;			
	SDLClassArray<class IPMP_Tool, 0, 255>	tools;
	//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003
		

	// Copy
	IPMP_Data_BaseClass* Copy()
	{	
		IPMP_GetToolsResponse* copy = new IPMP_GetToolsResponse;
		copy->Version = Version;
		copy->dataID = dataID;
		//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003
		//copy->IPMP_ToolID = IPMP_ToolID;
		copy->tools = tools;
		//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003
		return copy;
	}
};

class IPMP_GetToolContext : public IPMP_Data_BaseClass
{	OD_DECLARE_DESCRIPTOR
public:
	// Members
	//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003
	/*
	SDLInt<1>			hasDescriptor;
	SDLInt<1>			hasES_ID;
	SDLInt<1>			hasOD_ID;
	SDLInt<5>			Reserved; // 0b0000.0
	SDLInt<16>			descriptorIDEx;
	SDLInt<16>			ES_ID;
	SDLInt<10>			OD_ID;
	*/
	SDLInt<3>			scope;
	SDLInt<1>			hasIPMPDescriptorIDEx;
	SDLInt<4>			reserved;
	SDLInt<16>			IPMP_DescriptorIDEx; // 0b0000.0
	//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003
		

	// Copy
	IPMP_Data_BaseClass* Copy()
	{	IPMP_GetToolContext* copy = new IPMP_GetToolContext;
		
		copy->Version = Version;
		copy->dataID = dataID;
		//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003
		/*
		copy->hasDescriptor = hasDescriptor;
		copy->hasES_ID = hasES_ID;
		copy->hasOD_ID = hasOD_ID;
		copy->descriptorIDEx = descriptorIDEx;
		copy->ES_ID = ES_ID;
		copy->OD_ID = OD_ID;
		*/
		copy->scope = scope;
		copy->hasIPMPDescriptorIDEx = hasIPMPDescriptorIDEx ;
		copy->reserved = reserved;
		copy->IPMP_DescriptorIDEx = IPMP_DescriptorIDEx; 
		//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003
	
		return copy;
	}

private:
	virtual void DescEncDec (ODCoder *pODCoder, OPTYPE op);

};

class IPMP_GetToolContextResponse : public IPMP_Data_BaseClass
{	OD_DECLARE_DESCRIPTOR
public:
	// Members
	//added by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003
	SDLInt<1>		hasESD_ID;//added
    SDLInt<5>		reserved;//added
    SDLInt<10>		OD_ID;//added
    SDLInt<16>		ESD_ID;//added
	//addded by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003
	SDLInt<32>		toolContextID;

	// Copy
	IPMP_Data_BaseClass* Copy()
	{	
		IPMP_GetToolContextResponse* copy = new IPMP_GetToolContextResponse;
		copy->Version = Version;
		copy->dataID = dataID;
		//added by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003
		copy->hasESD_ID = hasESD_ID;//added
		copy->reserved = reserved;//added
		copy->OD_ID = OD_ID;//added
		copy->ESD_ID = ESD_ID;//added
		//addded by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003
		copy->toolContextID = toolContextID;
		
		return copy;
	}
private:
	virtual void DescEncDec (ODCoder *pODCoder, OPTYPE op);//added by Liao Xuan
};

class IPMP_ConnectTool : public IPMP_Data_BaseClass
{	OD_DECLARE_DESCRIPTOR
public:
	// Members
	SDLClass<class IPMP_Descriptor> descriptor;

	// Copy
	IPMP_Data_BaseClass* Copy()
	{	IPMP_ConnectTool* copy = new IPMP_ConnectTool;
		
		copy->Version = Version;
		copy->dataID = dataID;
		copy->descriptor = descriptor;

		return copy;
	}
private:
	virtual void DescEncDec (ODCoder *pODCoder, OPTYPE op);//added by Filippo Chiariglione
};

class IPMP_DisconnectTool : public IPMP_Data_BaseClass
{	OD_DECLARE_DESCRIPTOR
public:
	// Members
	SDLInt<32>				toolContextID;

	// Copy
	IPMP_Data_BaseClass* Copy()
	{	IPMP_DisconnectTool* copy = new IPMP_DisconnectTool;
		
		copy->Version = Version;
		copy->dataID = dataID;
		copy->toolContextID = toolContextID;
		
		return copy;
	}
};

class IPMP_NotifyToolEvent : public IPMP_Data_BaseClass
{	OD_DECLARE_DESCRIPTOR
public:
	// Members
	//addded by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003
	SDLInt<1>		hasID;//added
	SDLInt<7>		reserved;//added ,0b0000.000
	SDLInt<10>		OD_ID;//added
	SDLInt<6>		reserved2; //added, 0b0000.00
	SDLInt<16>		ESD_ID;//added
	//addded by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003
	SDLInt<8>		eventType; //EventTypes
	SDLInt<32>		toolContextID;

	// Copy
	IPMP_Data_BaseClass* Copy()
	{	IPMP_NotifyToolEvent* copy = new IPMP_NotifyToolEvent;
		
		copy->Version = Version;
		copy->dataID = dataID;
		//addded by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003
		copy->hasID = hasID;
		copy->reserved = reserved;
		copy->OD_ID = OD_ID;
		copy->reserved2 = reserved2;
		copy->ESD_ID = ESD_ID;
		//addded by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003
		copy->eventType = eventType;
		copy->toolContextID = toolContextID;
		return copy;
	}
private:
	virtual void DescEncDec (ODCoder *pODCoder, OPTYPE op);
};

class IPMP_CanProcess : public IPMP_Data_BaseClass
{	OD_DECLARE_DESCRIPTOR
public:
	// Members
	SDLInt<1>		canProcess;
	SDLInt<7>		Reserved;	// 0b0000.000

	// Copy
	IPMP_Data_BaseClass* Copy()
	{	IPMP_CanProcess* copy = new IPMP_CanProcess;
		
		copy->Version = Version;
		copy->dataID = dataID;
		copy->canProcess = canProcess;
		
		return copy;
	}
private:
	virtual void DescEncDec (ODCoder *pODCoder, OPTYPE op);

};

class User_Initialize : public IPMP_Data_BaseClass
{	
	OD_DECLARE_DESCRIPTOR
public:
	// Members
	SDLArray<sizeof(void*)*8>	messageInterface;
	SDLArray<sizeof(void*)*8>	toolDescriptor;

	// Copy
	IPMP_Data_BaseClass* Copy()
	{	
		User_Initialize* copy = new User_Initialize;
		
		copy->Version = Version;
		copy->dataID = dataID;
		copy->messageInterface = messageInterface;
		copy->toolDescriptor = toolDescriptor;
		
		return copy;
	}
private:
	virtual void DescEncDec (ODCoder *pODCoder, OPTYPE op);
};

class User_Initialized : public IPMP_Data_BaseClass
{	OD_DECLARE_DESCRIPTOR
public:
	// Members

	// Copy
	IPMP_Data_BaseClass* Copy()
	{	
		User_Initialized* copy = new User_Initialized;
		
		copy->Version = Version;
		copy->dataID = dataID;

		return copy;
	}
};

class IPMP_Message : public SDLDescriptor
{	
	OD_DECLARE_DESCRIPTOR
public:
	IPMP_Message() 
		: SDLDescriptor(true)
	{  }

	// Members
//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003
	SDLInt<16>									IPMPS_Type;
	//SDLString									URLString;
	SDLArray<8>									URLString;
	//SDLInt<16>								IPMP_DescriptorID;
	SDLInt<16>									IPMP_DescriptorIDEx;
	SDLClassArray<class IPMP_Data_BaseClass,0, 255>	IPMP_ExtendedData;
	//SDLString									IPMP_data;
	SDLArray<8>									IPMP_data;
	
	// Copy
	IPMP_Message* Copy()
	{	IPMP_Message * copy = new IPMP_Message;
		
		copy->IPMPS_Type = IPMPS_Type;
		copy->URLString = URLString;
		//copy->IPMP_DescriptorID = IPMP_DescriptorID;
		copy->IPMP_DescriptorIDEx = IPMP_DescriptorIDEx;
		copy->IPMP_ExtendedData = IPMP_ExtendedData;
		copy->IPMP_data = IPMP_data;
//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003
		return copy;
	}

private:
	virtual void DescEncDec (ODCoder *pODCoder, OPTYPE op);

};

/************** Changed by Liao Xuan ********************************************/
/*-------------------------------------------------------------------------------
to support implementation of binary message, because tool body
usually has too big size for the current Decoder to decode at real-time,
causing system error always. But binary implmentation has nuch higher efficiency
--------------------------------------------------------------------------------*/ 
//class IPMP_ToolES_AU : public SDLDescriptor
class IPMP_ToolES_AU : public BitField
{	//OD_DECLARE_DESCRIPTOR
public:
	//SDLInt<1>							isSigned;
	int									isSigned;
	//SDLInt<7>							reserved;
	int									reserved;
	//SDLString							IPMP_Tool_Signature;
	IPMP_ByteArray						IPMP_Tool_Signature;
	//SDLInt<16>						numCerts;//added
	int									numCerts;
	//SDLInt<8>*						CertType;//added
	int*								CertType;//added
	//SDLClassArray<class Certificate, 0, 255>			certificates;
	IPMP_ByteArray*						certificates;
	//SDLArray<128>						verifying_Tool_ID;
	byte Verifying_Tool_Id[16];
	//SDLInt<32>						sizeOfTool;//added
	int									sizeOfTool;	
	//SDLString							Tool_Body;
	byte*								Tool_Body;
	
	~IPMP_ToolES_AU()
	{
	}

	int toBuffer(byte * buf)
	{
		setWriteBuffer(buf);
		put( isSigned, 1);
		put( reserved, 7);

		if( isSigned ){
			putObject(IPMP_Tool_Signature);
			put (numCerts, 16);		// must be less that 2^16 
			for(int f=0;f<numCerts;f++){
				put( CertType[f], 8);		// certType
				putObject(certificates[f]);	// certificate
			}
			for(f=0;f<16;f++){
				put( Verifying_Tool_Id[f], 8 );
			}
		}
		
		put( sizeOfTool, 32);
		
		for(long f=0;f<sizeOfTool;f++)
			put( Tool_Body[f], 8); 
		
		return getWriteCounter();
	}

	int fromBuffer(const byte * buf)
	{
		setReadBuffer(buf);
		isSigned = (get(1) == 1);
		skipReading(7);							// reserved

		if( isSigned ){
			getObject(IPMP_Tool_Signature);
			numCerts = get(16);					// numCerts

			for(int f=0;f<numCerts;f++){
				//CertStruct cs;
				CertType[f] = (byte) get(8);	// certType
				getObject(certificates[f]);		// certificates
			}

			for(f=0;f<16;f++){
				Verifying_Tool_Id[f] = (byte) get(8);
			}
		}
		sizeOfTool = get(32);
		Tool_Body = new byte [sizeOfTool];
		for(long f=0;f<sizeOfTool;f++)
			Tool_Body[f] = (byte) get(8); 
		
		return getReadCounter();
	}

	unsigned int getSize() const 
	{
		unsigned int nSize = 1 ;							// isSigned

		if(isSigned)
		{
			nSize += IPMP_Tool_Signature.getSize();			// IPMP_Tool_Signature
			nSize+=2;										// numCerts
			for(int f=0;f<numCerts;f++)
			{
				nSize += 1;									// CertType
				nSize += certificates[f].getSize();			// certificate	
			}
			nSize += 16;									// Verifying_Tool_Id
		}
		nSize += 4;											// sizeOfTool
		nSize += (sizeOfTool);								// Tool_Body
		return nSize ;
	}
	bool IPMP_ToolES_AU::checkSyntax() const
	{
		return ((sizeOfTool > 0) && (Tool_Body !=  NULL));		// must contain a Tool
	}

//private:
	//virtual void DescEncDec (ODCoder *pODCoder, OPTYPE op);
};
/**************End: Changed by Liao Xuan ***************************************/


/*
// Tool message class
class ToolMessage{
public:
	ToolMessage() { version = 0x01;
					bCopied = false; }
	~ToolMessage()
	{	if ( bCopied )
		{	for ( int i = 0; i < IPMP_MessageFromBitstream.GetLength(); i++ )
				delete IPMP_MessageFromBitstream[i];
			for ( int j = 0; j < IPMP_Messages.GetLength(); j++ )
				delete IPMP_Messages[j];
		}
	}

	// Copy
	ToolMessage* Copy()
	{	ToolMessage* copy = new ToolMessage;
		copy->bCopied = true;
		copy->version = version;
		copy->msgID = msgID;
		copy->sender = sender;
		copy->receiver = receiver;

		for ( int i = 0; i < IPMP_DescriptorFromBitstream.GetLength(); i++ )
			copy->IPMP_DescriptorFromBitstream += IPMP_DescriptorFromBitstream[i];

		for ( int j = 0; j < IPMP_MessageFromBitstream.GetLength(); j++ )
			copy->IPMP_MessageFromBitstream += IPMP_MessageFromBitstream[j]->Copy();

		for ( int k = 0; k < IPMP_Messages.GetLength(); k++ )
			copy->IPMP_Messages += IPMP_Messages[k]->Copy();
		
		return copy;
	};

	SDLInt<32>						version;
	SDLInt<32>						msgID;
	SDLInt<32>						sender;
	SDLInt<32>						receiver;
	ZArray<IPMP_Descriptor*>		IPMP_DescriptorFromBitstream;
	ZArray<IPMP_Data_BaseClass*>	IPMP_MessageFromBitstream;
	ZArray<IPMP_Message*>			IPMP_Messages;

private:
	bool bCopied;

};
*/

// by Liao Xuan--- tool message base
/**************** Changed by Liao Xuan, May19 2004 ******************/
//class IPMP_ToolMessageBase
//{

class IPMP_ToolMessageBase: public BaseDescriptor
{
	OD_DECLARE_ABSTRACT_DESCRIPTOR
/**************** Changed by Liao Xuan, May19 2004 ******************/
public:
	IPMP_ToolMessageBase() 
	{
		Version = 0x01;
		tag		= 0x00;
	}
/*
	~IPMP_ToolMessageBase()
	{
	}
	*/
	virtual IPMP_ToolMessageBase* Copy()= 0;


	// Members
	SDLInt<8> 	tag;
	SDLInt<8> 	Version;
	SDLInt<32>	sender;
	SDLInt<32>	receiver;
};

// by Liao Xuan--- tool messages
class IPMP_MessageFromBitstream : public IPMP_ToolMessageBase
{
public :
	ZArray<IPMP_Message*>			messages;

	IPMP_MessageFromBitstream() 
	{
		Version = 0x01;
		tag		= 0x01;
	}
/*
	~IPMP_MessageFromBitstream()
	{	
		for ( int i = 0; i < messages.GetLength(); i++ )
		{
			if (messages[i]!=NULL)
			{
				delete messages[i];
				messages[i] = NULL;
			}
		}
	}
	*/

	
	// Copy
	IPMP_ToolMessageBase* Copy()
	{	
		IPMP_MessageFromBitstream* copy = new IPMP_MessageFromBitstream;
		copy->tag		= tag;
		copy->Version	= Version;
		copy->sender	= sender;
		copy->receiver	= receiver;
		for ( int i = 0; i < messages.GetLength(); i++ )
			copy->messages += messages[i]->Copy();
		return copy;
	}
};

class IPMP_DescriptorFromBitstream : public IPMP_ToolMessageBase
{
public :
	ZArray<IPMP_Descriptor*>		toolDescriptor;
	//IPMP_Descriptor*				toolDescriptor;

	IPMP_DescriptorFromBitstream() 
	{
		Version = 0x01;
		tag		= 0x02;
	}
/*	
	~IPMP_DescriptorFromBitstream()
	{	
		for ( int i = 0; i < toolDescriptor.GetLength(); i++ )
		{
			if (toolDescriptor[i]!=NULL)
			{
				delete toolDescriptor[i];
				toolDescriptor[i] = NULL;
			}
		}
	}
	*/
	
	// Copy
	IPMP_ToolMessageBase* Copy()
	{	
		IPMP_DescriptorFromBitstream* copy = new IPMP_DescriptorFromBitstream;
		copy->tag		= tag;
		copy->Version	= Version;
		copy->sender	= sender;
		copy->receiver	= receiver;
		for ( int i = 0; i < toolDescriptor.GetLength(); i++ )
			copy->toolDescriptor += toolDescriptor[i];
		//copy->toolDescriptor = toolDescriptor;
		return copy;
	}
};


class IPMP_MessageFromTool : public IPMP_ToolMessageBase
{
public :
	ZArray<IPMP_Data_BaseClass*>	messages;

	IPMP_MessageFromTool() 
	{
		Version = 0x01;
		tag		= 0x03;
	}
/*	
	~IPMP_MessageFromTool()
	{	
		for ( int i = 0; i < messages.GetLength(); i++ )
		{
			if (messages[i]!=NULL)
			{
				delete messages[i];
				messages[i] = NULL;
			}
		}
	}
	*/
	
	// Copy
	IPMP_ToolMessageBase* Copy()
	{	
		IPMP_MessageFromTool * copy = new IPMP_MessageFromTool ;
		copy->Version	= Version;
		copy->sender	= sender;
		copy->receiver	= receiver;
		for ( int i = 0; i < messages.GetLength(); i++ )
			copy->messages += messages[i]->Copy();
		return copy;
	}
};
// by Liao Xuan---tool messages


//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003
// new added class 
class IPMP_ToolAPI_Config :  public IPMP_Data_BaseClass
{
	OD_DECLARE_DESCRIPTOR
public:
	// Members
	SDLInt<1>					has_Instantiation_API_ID;
	SDLInt<1>					has_Messaging_API_ID;
	SDLInt<6>					reserved ;// 0b0000.00
	SDLInt<32>					Instantiation_API_ID;
	SDLInt<32>	 				Messaging_API_ID;
	SDLClass<class ByteArray>	OpaqueData;

	// Copy
	IPMP_Data_BaseClass* Copy()
	{	
		IPMP_ToolAPI_Config* copy = new IPMP_ToolAPI_Config;
		
		copy->Version = Version;
		copy->dataID = dataID;
		copy->has_Instantiation_API_ID = has_Instantiation_API_ID;
		copy->has_Messaging_API_ID = has_Messaging_API_ID;
		copy->reserved = reserved;
		copy->Instantiation_API_ID = Instantiation_API_ID;
		copy->Messaging_API_ID = Messaging_API_ID;
		copy->OpaqueData = OpaqueData;
		return copy;
	}
private:
	virtual void DescEncDec (ODCoder *pODCoder, OPTYPE op);//added by Liao Xuan
};
//by Liao Xuan -- according to ISO/IEC 14496-1:2001/FDAM 3:2003


/********************************Added by Liao Xuan************/
/* Inter-device messages                                      */
/**************************************************************/
// Inter-device messages base
class IPMP_DeviceMessageBase
{ 
public:
	IPMP_DeviceMessageBase()
	{	
		Version = 0x02; 
		tag = 0x00;
	}

    // Member
	SDLInt<8>		tag;
    SDLInt<8>		Version;
	SDLInt<128>		sender_deviceID;
	SDLInt<128>		recipient_deviceID;

	// Copy
	virtual IPMP_DeviceMessageBase* Copy()	= 0;
};

// Inter-device messages
class IPMP_MessageFromDevice: public IPMP_DeviceMessageBase
{ 
public:
    // Member
	IPMP_MessageFromDevice()	
	{	
		Version = 0x02; 
		tag = 0x01;
	}
    SDLClassArray<class IPMP_Data_BaseClass, 1, 255>		message;

	// Copy
	IPMP_DeviceMessageBase* Copy()
	{	
		IPMP_MessageFromDevice* copy = new IPMP_MessageFromDevice;
		copy->tag					= tag;
		copy->Version				= Version;
		copy->sender_deviceID		= sender_deviceID;
		copy->recipient_deviceID	= recipient_deviceID;
		copy->message				= message;
		return copy;
	}
};

class IPMP_RequestContent : public IPMP_Data_BaseClass
{
public:
	//members
	SDLInt<128> ContentID;
	ByteArray	opaqueData;
	SDLInt<128> IPMP_DomainID;
	
	IPMP_Data_BaseClass* Copy()
	{	
		IPMP_RequestContent* copy = new IPMP_RequestContent;
		copy->Version = Version;
		copy->dataID = dataID;
		copy->ContentID = ContentID;
		copy->opaqueData = opaqueData;
		copy->IPMP_DomainID = IPMP_DomainID;
		return copy;
	}
};

class IPMP_ResponseToContentRequest: public IPMP_Data_BaseClass 
{
public:
	//members
	SDLInt<2> response;
	SDLInt<6> reserved;//0b0000.00
	
	IPMP_Data_BaseClass* Copy()
	{	
		IPMP_ResponseToContentRequest * copy = new IPMP_ResponseToContentRequest;
		copy->Version = Version;
		copy->dataID = dataID;
		copy->response = response;
		copy->reserved = reserved;
		return copy;
	}
};

class IPMP_ContentTransfer: public IPMP_Data_BaseClass 
{
public:
	//members
	SDLInt<128> ContentID;
	SDLInt<8>	Sequence;
	SDLInt<32> 	PayloadSize;
	SDLArray<8>	Payload;
	
	IPMP_Data_BaseClass* Copy()
	{	
		IPMP_ContentTransfer * copy = new IPMP_ContentTransfer;
		copy->Version = Version;
		copy->dataID = dataID;
		copy->ContentID = ContentID;
		copy->Sequence = Sequence;
		copy->PayloadSize = PayloadSize;
		copy->Payload = Payload;
		return copy;
	}
};

class IPMP_RequestTool: public IPMP_Data_BaseClass 
{
public:
	//members
	SDLInt<128> ToolID;
	SDLInt<128>	IPMP_DomainID;
	ByteArray	opaqueData;
	
	IPMP_Data_BaseClass* Copy()
	{	
		IPMP_RequestTool * copy = new IPMP_RequestTool;
		copy->Version = Version;
		copy->dataID = dataID;
		copy->ToolID = ToolID;
		copy->IPMP_DomainID = IPMP_DomainID;
		copy->opaqueData = opaqueData;
		return copy;
	}
};

class IPMP_ResponseToToolRequest: public IPMP_Data_BaseClass 
{
public:
	//members
	SDLInt<128> ToolID;
	SDLInt<2>	response;
	SDLInt<6>	reserved;// 0b0000.00;
	ByteArray	opaqueData;
	SDLInt<32> 	PayloadSize;
	SDLArray<8>	Payload;
	SDLInt<16>	ToolDescriptionSize;
	SDLArray<8> ToolDescription;
	
	IPMP_Data_BaseClass* Copy()
	{	
		IPMP_ResponseToToolRequest * copy = new IPMP_ResponseToToolRequest;
		copy->Version = Version;
		copy->dataID = dataID;
		copy->ToolID = ToolID;
		copy->response = response;
		copy->reserved = reserved;
		copy->opaqueData = opaqueData;
		copy->PayloadSize = PayloadSize;
		copy->Payload = Payload;
		copy->ToolDescriptionSize = ToolDescriptionSize;
		copy->ToolDescription = ToolDescription;
		return copy;
	}
};

class IPMP_DeviceID_Notification: public IPMP_Data_BaseClass 
{
public:
	//members
	SDLInt<128>	IPMP_DomainID;
	ByteArray	opaqueData;
	
	IPMP_Data_BaseClass* Copy()
	{	
		IPMP_DeviceID_Notification * copy = new IPMP_DeviceID_Notification;
		copy->Version = Version;
		copy->dataID = dataID;
		copy->IPMP_DomainID = IPMP_DomainID;
		copy->opaqueData = opaqueData;
		return copy;
	}
};
/********************************Added by Liao Xuan************/
/* End of Inter-device messages                               */
/**************************************************************/

#endif	//  _IPMP_MESSAGES_H_

