//	This software module was originally developed by Craig A. Schultz 
//	(AccessTicket Systems Inc.)
//
//	in the course of development of the MPEG-4 Systems 
//	(ISO/IEC 14496-1) standard. This software module is an implementation 
//	of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as specified 
//	by the MPEG-4 Systems (ISO/IEC 14496-1) standard. 
//
//	ISO/IEC gives users of the  the MPEG-4 Systems (ISO/IEC 14496-1) free 
//	license to this software module or modifications thereof for use in hardware 
//	or software products claiming conformance to the MPEG-4 Systems (ISO/IEC 
//	14496-1). 
//
//	Those intending to use this software module in hardware or software 
//	products are advised that its use may infringe existing patents. 
//
//	The original developer of this software module and his/her company, the 
//	subsequent editors and their companies, and ISO/IEC have no liability for 
//	use of this software module or modifications thereof. 
//
//	Copyright is not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
//	products. 
//
//	AccessTicket Systems Inc. retains full right to use the code for its own purpose, 
//	assign or donate the code to a third party and to inhibit third parties from 
//	using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products.
// 
//	This copyright notice must be included in all copies or derivative works. 
//
//	Copyright (c) 2000. 

#ifndef _SECURITYLIB_H_
#define _SECURITYLIB_H_

#include "IPMP_Messages.h"
#include "MessageDistributor.h"

class SecurityLib
{
public:
	MessageDistributor* msgDistributor;

	// Constructor
	SecurityLib( MessageDistributor* msgDistributor );

	// Constructor
	SecurityLib(); 

	// Destructor
	~SecurityLib();

	// Class Methods

	// Takes a message of type IPMP_MutualAuthenticationInit as input and internally 
	// takes the necessary action which in this case is the beginning of a mutual 
	// authentication process with the required tool as indicated within the message.
	// NOTE : any generation or sending of messages will be done internally.
	bool ProcessMutAuthInit( IPMP_Data_BaseClass* // msg
	);
	
	// Takes a message of type IPMP_MutualAuthentication as well as the context ID
	// of the tool it was received from as input and internally takes the necessary action
	// which in this case is the continuing of the mutual authentication process with
	// the required tool.
	// NOTE : any generation or sending of messages will be done internally.
	bool ProcessMutAuthMsg( IPMP_Data_BaseClass* // msg
						  , SDLInt<32> // contextID
	);

	// Send a encrypted message and the contextID
	// NOTE : any generation or sending of messages will be done internally.
	bool SendSecureMessage(IPMP_Data_BaseClass* msg , SDLInt<32> contextID);

	// Takes a message of type IPMP_Secure_Container as well as the context ID
	// of the tool it was received from as input and internally takes the necessary action
	// which in this case is using the shared secret derived during the mutual authentication
	// process to either verify or decrypt the contained message.  The output is a properly
	// constructed message object of the correct type.
	// NOTE : A return of NULL indicates failure to properly construct or verify the 
	// contained message.
	IPMP_Data_BaseClass* ProcessSecureContainer( IPMP_Secure_Container* // msg
											   , SDLInt<32> // contextID
	);

	// Takes a message derived from IPMP_Data_BaseClass as well as the context ID
	// of the tool the result should be sent to as input and internally takes the necessary action
	// which in this case is using the shared secret derived during the mutual authentication
	// process to either encrypt, add a MAC, or both.  The output is then is internally sent
	// to the required tool.
	// NOTE : A return of "false" indicates failure to properly construct or deliver the 
	// message.
	bool ProcessSecureContainer( IPMP_Data_BaseClass* // msg
							   , SDLInt<32> // contextID
	);
};

#endif //  _SECURITYLIB_H_

