/*------------------------------------------------------------------
	 2000 CSELT - Centro Studi e Laboratori Telecomunicazioni.
	All rights reserved.
------------------------------------------------------------------*/


#ifndef _DAI_stats_h_
#define _DAI_stats_h_

// Microsoft Visual Studio .NET changed behavior
// Michael Stepping
#if ( _MSC_VER >= 1300 || __UNIX )
using namespace std;
#include <iostream> // neue Schreibweise
#else
#include "iostream.h"
#endif
#include "ztl.h"

typedef enum tagSessionStats
{
	NONE			= 0x00,
	TOTALBYTES		= 0x01,
	TOTALPAYLOAD	= 0x02,
	AVGRATE			= 0x03,
	PEAKRATE		= 0x04,
	FMXPACKETS		= 0x05,
	IPPACKETS		= 0x06
} SESSION_STATS;

class DAISessionStatElement
{
protected:
	SESSION_STATS	eTag;
public:
	DAISessionStatElement() : eTag(NONE) {}
	virtual ~DAISessionStatElement() {}
	virtual void Print(ostream& s) const {s<<"UNDEFINED Print()!!"<<endl;}
	SESSION_STATS GetTag() {return eTag;}
	bool ValidTag()
	{
		bool bRes = false;
		switch(eTag)
		{
		case NONE:
		case TOTALBYTES:
		case TOTALPAYLOAD:
		case AVGRATE:
		case PEAKRATE:
		case FMXPACKETS:
		case IPPACKETS:
			bRes = true;
			break;
		default:
			bRes = false;
			break;
		}
		return bRes;
	}
friend inline ostream& operator<<(ostream& s, const DAISessionStatElement& m) ;
};

inline ostream& operator<<(ostream& s, const SESSION_STATS& m)
{
	switch(m)
	{
	case NONE:
		s<<"NONE";
		break;
	case TOTALBYTES:
		s<<"TOTALBYTES";
		break;
	case TOTALPAYLOAD:
		s<<"TOTALPAYLOAD";
		break;
	case AVGRATE:
		s<<"AVGRATE";
		break;
	case PEAKRATE:
		s<<"PEAKRATE";
		break;
	case FMXPACKETS:
		s<<"FMXPACKETS";
		break;
	case IPPACKETS:
		s<<"IPPACKETS";
		break;
	default:
		s<<"undefined";
		break;
	}
	return s;
}


inline ostream& operator<<(ostream& s, const DAISessionStatElement& m)
{
//	s << "Tag = " << m.eTag << endl;
	m.Print(s);
	return s;
}

class DAISessionStatElement_totalBytes : public DAISessionStatElement
{
public:
	unsigned __int64	nTotalBytes;
public:
	DAISessionStatElement_totalBytes() : nTotalBytes(0) {eTag = TOTALBYTES;}
	void Print(ostream& s) const
	{
		s << "Total Bytes Received (cut off to 32bit) = "<< (DWORD)nTotalBytes << endl;
	}
};

class DAISessionStatElement_totalPayload : public DAISessionStatElement
{
public:
	unsigned __int64	nTotalPayload;
public:
	DAISessionStatElement_totalPayload() : nTotalPayload(0) {eTag = TOTALPAYLOAD;}
	void Print(ostream& s) const
	{
		s << "Total Payload Bytes (cut off to 32bit) = "<< (DWORD)nTotalPayload << endl;
	}
};

class DAISessionStatElement_avgRate : public DAISessionStatElement
{
public:
	DWORD	avgRate;	// bit per second (average for an observation window of the last interval)
public:
	DAISessionStatElement_avgRate() : avgRate(0) {eTag = AVGRATE;}
	void Print(ostream& s) const
	{
		s << "Average Rate = "<< avgRate << " [bit/s]"<< endl;
	}
};

class DAISessionStatElement_peakRate : public DAISessionStatElement
{
public:
	DWORD	peakRate;	// bit per second (peak for an observation window of an interval)
public:
	DAISessionStatElement_peakRate() : peakRate(0) {eTag = PEAKRATE;}
	void Print(ostream& s) const
	{
		s << "Peak Rate = "<< peakRate << " [bit/s]" << endl;
	}
};

class DAISessionStatElement_ipPackets : public DAISessionStatElement
{
public:
	DWORD	nIpPackets;	// number of ip packets received
public:
	DAISessionStatElement_ipPackets() : nIpPackets(0) {eTag = IPPACKETS;}
	void Print(ostream& s) const
	{
		s << "IP Packets received = "<< nIpPackets << endl;
	}
};

class DAISessionStatElement_fmxPackets : public DAISessionStatElement
{
public:
	DWORD	nFmxPackets;	// number of FMX packets received
public:
	DAISessionStatElement_fmxPackets() : nFmxPackets(0) {eTag = FMXPACKETS;}
	void Print(ostream& s) const
	{
		s << "FlexMux Packets received = "<< nFmxPackets << endl;
	}
};


typedef ZArray<SESSION_STATS>	DAISessionStatTagParam;
typedef ZArray<DAISessionStatElement*>	DAISessionStatParam;


#endif // _DAI_stats_h_
