/***********************************************************************
    This software module  was  originally  developed by Zvi  Lifshitz
    (Triton R&D Ltd.) and edited by Mauro Rossi (CSELT), George
    Tselikis (Xbind Inc.) and Michael Stepping (FTK)
    in the course of development of the MPEG-4 DMIF (ISO/IEC 14496-6)
    standard.  This software module is an implementation of a part of
    one or more MPEG-4  DMIF  (ISO/IEC 14496-6)  tools  as  specified
    by  the  MPEG-4 DMIF (ISO/IEC 14496-6)  standard.  ISO/IEC  gives
    users of  the  MPEG-4  DMIF  (ISO/IEC 14496-6)  free  license  to
    this software module or modifications thereof for use in hardware
    or software products claiming  conformance  to  the  MPEG-4  DMIF 
    (ISO/IEC 14496-6). Those intending to use this software module in
    hardware or software  products  are  advised  that  its  use  may
    infringe   existing  patents.  The  original  developer  of  this
    software module and his/her company, the subsequent  editors  and
    their  companies,  and  ISO/IEC have no liability for use of this
    software module or modifications thereof  in  an  implementation.
    Copyright  is not released for non MPEG-4 DMIF  (ISO/IEC 14496-6)
    conforming  products. Triton R&D Ltd.
    retains full  right to use the code for  its  own purpose, assign
    or donate the code to a third party and to inhibit third  parties
    from  using  the  code  for  non MPEG-4 DMIF (ISO/IEC 14496-6)
    conforming products. This copyright notice must  be  included  in
    all copies or derivative works. Copyright  1998.
************************************************************************/

/**************************************************************************
 * Project Im1Player
 * INTERNATIONAL STANDARD ORGANIZATION
 * ISO/IEC JTC1/SC29/WG11
 * 
 * SUBSYSTEM: DMIF
 * FILE:      DAIserver.h
 * AUTHOR:    Mauro Rossi (CSELT)
 * OVERVIEW:  Definitions for the DMIF-Application Interface.
 * 
 **************************************************************************/

#ifndef _DAIserver_h_
#define _DAIserver_h_

#include "DAI.h"

// The following C++ class provides an abstract view of the functions that the DMIF implementation calls in order 
// to inform the application about the result of a requested service (functions with "_Cnf" extension) or about services 
// requested by the peer entity (functions with "_Ind" extension). This class defines the methods that a DMIF 
// implementation (DMIF_Filter derived class) expects to be available from the application. The application should 
// define an Application_Sink derived class and implement the DAI functions with "_Ind" or "_Cnf" extension. 
// Therefore, in this case the function call direction is DMIF->Application.
/*class Application_Sink_clientserver : public Application_Sink
{
public:
	virtual void DAI_ServiceAttach_Ind(
		const char*		service_name,
		const UINT		nServiceId,
		const BYTE*		uuData,
		const u_short	uuData_Length
	) = 0;
	virtual void DAI_ServiceDetach_Ind(
		const UINT		nServiceId,
		const REASON	eReason
	) = 0;
	virtual void DAI_ChannelAdd_Ind(
		const UINT		nServiceId,
		const DAIChannelAddParam& inParam
	) = 0;
	virtual void DAI_ChannelDelete_Ind(
		const UINT		nServiceId,
		const DAIChannelDeleteParam& inParam
	) = 0;
	virtual void DAI_UserCommand_Ind(
		const UINT		nServiceId,
		const DAIUserCmdParam& inParam
	) = 0; 
	virtual	void DAI_UserCommandAck_Ind(
		const UINT		nServiceId,
		const DAIUserCmdParam& inParam
	) = 0;
};*/

// The following C++ class provides an abstract view of the functions that the application should call in order to
// request a DMIF service (functions with "_Req" extension) or to respond to a DMIF service requested by the peer entity 
// (functions with "_Rsp" extension). This class defines the methods that an application expects to be available from 
// a DMIF implementation (DMIF_Filter derived class). A DMIF_Filter derived class is the one that implements
// the DAI functions with "_Req" or "_Rsp" extension. Therefore, in this case the function call direction is 
// Application->DMIF.
class DMIF_Filter_clientserver : public DMIF_Filter 
{
public:
	// Function for dll server registration
	virtual void DAI_TransmuxAdd_Rsp(
		const UINT		nServiceId,
		const DAIChannelAddParam& outParam
	) = 0;
	virtual void RegistService(
		const LPCSTR	sURLstring,
		RESPONSE& eResponse
	) = 0;
};
#endif  //_DAIserver_h_
