/***********************************************************************
    This software module  was  originally  developed by Zvi  Lifshitz
    (Triton R&D Ltd.) and edited by Mauro Rossi (CSELT), George
    Tselikis (Xbind Inc.) and Michael Stepping (FTK)
    in the course of development of the MPEG-4 DMIF (ISO/IEC 14496-6)
    standard.  This software module is an implementation of a part of
    one or more MPEG-4  DMIF  (ISO/IEC 14496-6)  tools  as  specified
    by  the  MPEG-4 DMIF (ISO/IEC 14496-6)  standard.  ISO/IEC  gives
    users of  the  MPEG-4  DMIF  (ISO/IEC 14496-6)  free  license  to
    this software module or modifications thereof for use in hardware
    or software products claiming  conformance  to  the  MPEG-4  DMIF 
    (ISO/IEC 14496-6). Those intending to use this software module in
    hardware or software  products  are  advised  that  its  use  may
    infringe   existing  patents.  The  original  developer  of  this
    software module and his/her company, the subsequent  editors  and
    their  companies,  and  ISO/IEC have no liability for use of this
    software module or modifications thereof  in  an  implementation.
    Copyright  is not released for non MPEG-4 DMIF  (ISO/IEC 14496-6)
    conforming  products. Triton R&D Ltd.
    retains full  right to use the code for  its  own purpose, assign
    or donate the code to a third party and to inhibit third  parties
    from  using  the  code  for  non MPEG-4 DMIF (ISO/IEC 14496-6)
    conforming products. This copyright notice must  be  included  in
    all copies or derivative works. Copyright  1998.
************************************************************************/

/**************************************************************************
 * Project Im1Player
 * INTERNATIONAL STANDARD ORGANIZATION
 * ISO/IEC JTC1/SC29/WG11
 * 
 * SUBSYSTEM: DMIF
 * FILE:      DAItools.h
 * AUTHOR:    Mauro Rossi (CSELT)
 * OVERVIEW:  Definitions for the DMIF-Application Interface.
 * 
 **************************************************************************/

#ifndef _DAItools_h_
#define _DAItools_h_

#include "DAItypes.h"
// ZArrays for definition of Loops
#include "ztl.h"

/*-------------------------------------------------------------
Structs to pass information in the interface
-------------------------------------------------------------*/

typedef struct 
{
	RESPONSE	m_eResponse;
	WORD		m_nESid;
	UINT		m_nChid;
	int			m_nFmxInfo;
} STREAMMAPELEM,*LPSTREAMMAPELEM;

/*-------------------------------------------------------------
Class for passing parameters in DAI_Channel_Add_* inside the loop
-------------------------------------------------------------*/
class DAIChannelAddElement
{
  public:

	UINT		m_nChid;
	UINT		m_nTmpId;
	// direction for channel content. Usually from point of view of 
	// client/MPEG-4 Terminal
	DIRECTION	m_eDirection;

	RESPONSE	m_eResponse;
	QOSSTREAM	m_pQoSStream;
	// SLConfigDescriptor is provisionally exposed as a separate parameter
	BYTE*		m_pAppInfo;
	USHORT		m_nAppInfoLen;

	//@
//	FLEXMUX_TYPE		m_eFmxType;
	WORD				m_eFmxType;//FLEXMUX_TYPE is "used internally of DMIF" see FlexMuxTool.h
	LPSTREAMMAPELEM		m_pStreamMap;
	DWORD				m_nStreamMapLength;
	//@

	// user-user data.
	BYTE*		m_pData;
	USHORT		m_nDataLen;
	
  public: // methods	
	BOOL operator == (DAIChannelAddElement const &el) const 
	{
		return (m_nChid == el.m_nChid);
	}
	
	DAIChannelAddElement() : 
		m_nChid(0), 
		m_nTmpId(0), 
		m_pData(NULL), 
		m_nDataLen(0), 
		m_eDirection( DOWNSTREAM ),
		m_pQoSStream( NULL ),
		m_pAppInfo( NULL ),
		m_nAppInfoLen( 0 ),
		m_eResponse ( RESPONSE_OK ),
		m_eFmxType ( 0x00/*UNDEFINED_FLEXMUX*/ ),
		m_pStreamMap ( NULL ),
		m_nStreamMapLength ( 0 )
	{}
	
	DAIChannelAddElement(const DAIChannelAddElement& o)//copy constructor
	{
		m_nChid	= o.m_nChid;
		m_nTmpId	= o.m_nTmpId;
		m_eDirection= o.m_eDirection;
		m_eResponse	= o.m_eResponse;
		m_pQoSStream= o.m_pQoSStream;
		m_nAppInfoLen= o.m_nAppInfoLen;
		if ( m_nAppInfoLen > 0 )
		{
			m_pAppInfo		= new BYTE[o.m_nAppInfoLen];
			memcpy(m_pAppInfo,o.m_pAppInfo,o.m_nAppInfoLen);
		}
		else
			m_pAppInfo = NULL ;
		m_nDataLen	= o.m_nDataLen;
		if ( m_nDataLen > 0 )
		{
			m_pData		= new BYTE[o.m_nDataLen];
			memcpy(m_pData,o.m_pData,o.m_nDataLen);
		}
		else
			m_pData = NULL ;
		m_eFmxType	= o.m_eFmxType;
		m_nStreamMapLength = o.m_nStreamMapLength;
		m_pStreamMap = o.m_pStreamMap;
/*		if(m_nStreamMapLength)
		{
			m_pStreamMap = new STREAMMAPELEM[m_nStreamMapLength];
			memcpy( (char*)&m_pStreamMap,(char*)&(o.m_pStreamMap),m_nStreamMapLength*sizeof(STREAMMAPELEM) );
		}
		else
			m_pStreamMap = NULL;*/
	}

	DAIChannelAddElement& operator=(const DAIChannelAddElement& r)
	{
		m_nChid	= r.m_nChid;
		m_nTmpId	= r.m_nTmpId;
		m_eDirection	= r.m_eDirection;
		m_eResponse	= r.m_eResponse;
		m_pQoSStream= r.m_pQoSStream;
		m_nAppInfoLen= r.m_nAppInfoLen;
		if ( m_pAppInfo )
			delete [] m_pAppInfo ;
		if ( m_nAppInfoLen > 0 )
		{
			m_pAppInfo		= new BYTE[r.m_nAppInfoLen];
			memcpy(m_pAppInfo,r.m_pAppInfo,m_nAppInfoLen);
		}
		else
			m_pAppInfo = NULL ;
		m_nDataLen	= r.m_nDataLen;
		if ( m_pData )
			delete [] m_pData ;
		if ( m_nDataLen > 0 )
		{
			m_pData		= new BYTE[r.m_nDataLen];
			memcpy(m_pData,r.m_pData,m_nDataLen);
		}
		else
			m_pData = NULL ;
		m_eFmxType	= r.m_eFmxType;
		m_nStreamMapLength = r.m_nStreamMapLength;
		m_pStreamMap = r.m_pStreamMap;
/*		if(m_nStreamMapLength)
		{
			m_pStreamMap = new STREAMMAPELEM[m_nStreamMapLength];
			memcpy( (char*)&m_pStreamMap,(char*)&(r.m_pStreamMap),m_nStreamMapLength*sizeof(STREAMMAPELEM) );
		}
		else
			m_pStreamMap = NULL;*/
		return ( *this ) ;
	}
	
};


/*-------------------------------------------------------------
Class for passing parameters in DAI_Channel_Delete_* inside the loop
-------------------------------------------------------------*/
class DAIChannelDeleteElement
{
  public:
	UINT			m_nChid;
	UINT			m_nTmpId;	// used to delete channels not yet confirmed
	REASON			m_eReason;
	RESPONSE		m_eResponse;
	BOOL operator == (DAIChannelDeleteElement const &el) const {return el.m_nChid == m_nChid;}

	DAIChannelDeleteElement& operator=(const DAIChannelDeleteElement& r)
	{
		m_nChid = r.m_nChid;
		m_nTmpId = r.m_nTmpId;
		m_eReason = r.m_eReason;
		m_eResponse = r.m_eResponse;
		return ( *this ) ;
	}
};

/*-------------------------------------------------------------
Class for passing a loop of parameters in DAI_Channel_Add_*
-------------------------------------------------------------*/
typedef ZArray<DAIChannelAddElement>	DAIChannelAddParam;
/*-------------------------------------------------------------
Class for passing a loop of parameters in DAI_Channel_Delete_*
-------------------------------------------------------------*/
typedef ZArray<DAIChannelDeleteElement>	DAIChannelDeleteParam;

/*-------------------------------------------------------------
Class for passing all parameters in DAI_User_Command_*
-------------------------------------------------------------*/
class DAIUserCmdParam
{
public:
	int				m_nCmdId;
	int				m_eResponse;
	char*			m_pData;
	int				m_nDataLen;
	UINT*			m_pChid;
	int				m_nChLen;

public:
	DAIUserCmdParam() : m_nCmdId(0),m_eResponse(RESPONSE_OK),m_pData(NULL),m_nDataLen(0),m_pChid(NULL),m_nChLen(0) 
	{}
	DAIUserCmdParam(const DAIUserCmdParam& o)
	{
		m_nCmdId= o.m_nCmdId;
		m_eResponse= o.m_eResponse;
		m_nChLen= o.m_nChLen;
		if(m_nChLen > 0)
		{
			m_pChid = new UINT[m_nChLen];
			memcpy((char*)m_pChid,(char*)o.m_pChid,sizeof(UINT)*m_nChLen);
		}
		else
		{
			m_pChid = NULL;
		}
		m_nDataLen	= o.m_nDataLen;
		m_pData	= new char[o.m_nDataLen];
		memcpy(m_pData,o.m_pData,o.m_nDataLen);
	}
	DAIUserCmdParam& operator=(const DAIUserCmdParam& r)
	{
		m_nCmdId	= r.m_nCmdId;
		m_eResponse	= r.m_eResponse;
		if(m_nChLen > 0)
			delete[] m_pChid;
		m_nChLen= r.m_nChLen;
		if(m_nChLen > 0)
		{
			m_pChid = new UINT[m_nChLen];
			memcpy((char*)m_pChid,(char*)r.m_pChid,sizeof(UINT)*m_nChLen);
		}
		else
		{
			m_pChid = NULL;
		}
		m_nDataLen	= r.m_nDataLen;
		if ( m_pData )
			delete[] m_pData ;
		m_pData = new char[r.m_nDataLen];
		memcpy(m_pData,r.m_pData,r.m_nDataLen);
		return ( *this ) ;
	}
	/*-------------------------------------------------------------
	Zvi: remove this. Interface shall have no assumptions on the nature of m_pData,
	m_pChid. They might point to static memory. It makes no sense to force the use of
	SetData and AddChid. The application may set the fields directly like it does with
	other structures.
	-------------------------------------------------------------*/
/*	~DAIUserCmdParam()
	{
		if ( m_pData )
			delete[] m_pData;
		if(m_nChLen > 0)
			delete[] m_pChid;
	}
*/
	void SetData(int len,const BYTE* pData)
	{
		if ( m_pData )
			delete[] m_pData ;
		m_nDataLen = len;
		if(len == 0)
		{
			m_pData = NULL;
		}
		else
		{
			m_pData = new char[len];
			memcpy(m_pData,pData,len);
		}
	}
	void AddChid(UINT chid)
	{
		UINT* newChidArray = new UINT[m_nChLen+1];
		if(m_nChLen>0)
			memcpy((char*)newChidArray,(char*)m_pChid,sizeof(UINT)*(m_nChLen));
		newChidArray[m_nChLen] = chid;
		if(m_nChLen > 0)
			delete[] m_pChid;
		m_pChid = newChidArray;
		m_nChLen++;
	}
	/*JLF: added quick mem cleanup...*/
	void Reset() {
		if (m_pData) delete [] m_pData;
		m_pData = NULL;
		m_nDataLen = 0;
		if (m_pChid) delete m_pChid;
		m_pChid = NULL;
		m_nChLen = 0;
	}
};

#endif  //_DAItools_h_
