/***********************************************************************
    This software module  was  originally  developed by Zvi  Lifshitz
    (Triton R&D Ltd.) and edited by Mauro Rossi (CSELT), George
    Tselikis (Xbind Inc.) and Michael Stepping (FTK)
    in the course of development of the MPEG-4 DMIF (ISO/IEC 14496-6)
    standard.  This software module is an implementation of a part of
    one or more MPEG-4  DMIF  (ISO/IEC 14496-6)  tools  as  specified
    by  the  MPEG-4 DMIF (ISO/IEC 14496-6)  standard.  ISO/IEC  gives
    users of  the  MPEG-4  DMIF  (ISO/IEC 14496-6)  free  license  to
    this software module or modifications thereof for use in hardware
    or software products claiming  conformance  to  the  MPEG-4  DMIF
    (ISO/IEC 14496-6). Those intending to use this software module in
    hardware or software  products  are  advised  that  its  use  may
    infringe   existing  patents.  The  original  developer  of  this
    software module and his/her company, the subsequent  editors  and
    their  companies,  and  ISO/IEC have no liability for use of this
    software module or modifications thereof  in  an  implementation.
    Copyright  is not released for non MPEG-4 DMIF  (ISO/IEC 14496-6)
    conforming  products. Triton R&D Ltd.
    retains full  right to use the code for  its  own purpose, assign
    or donate the code to a third party and to inhibit third  parties
    from  using  the  code  for  non MPEG-4 DMIF (ISO/IEC 14496-6)
    conforming products. This copyright notice must  be  included  in
    all copies or derivative works. Copyright  1998.
************************************************************************/

/**************************************************************************
 * Project Im1Player
 * INTERNATIONAL STANDARD ORGANIZATION
 * ISO/IEC JTC1/SC29/WG11
 *
 * SUBSYSTEM: DMIF
 * FILE:      DAItypes.h
 * AUTHOR:    Mauro Rossi (CSELT)
 * OVERVIEW:  Definitions for the DMIF-Application Interface.
 *
 **************************************************************************/

#ifndef _DAItypes_h_
#define _DAItypes_h_

//#ifndef _WINSOCKAPI_
#ifndef _WINSOCK2API_
#ifndef __unix /*rba*/
#define ntohs(n) (((n)%256)*256 + (n)/256)
#define htons(n) (((n)%256)*256 + (n)/256)
#endif
#endif
//#endif

// SOLARIS PORTING - taking off BYTE - defined in defs.h
#ifdef WIN32
typedef unsigned char	BYTE;
#endif

#ifdef WIN32 /*rba*/
typedef unsigned short	u_short;
#elif __unix
typedef unsigned short	USHORT;
#endif
typedef unsigned int	UINT;

// Response parameter
typedef enum tagResponseParameter
{
	RESPONSE_OK								= 0x0000,
	RESPONSE_SERVICEDLL_NOT_FOUND			= 0x0001,
	RESPONSE_SERVICEDLL_ERROR				= 0x0002,
	RESPONSE_URL_NOT_RECOGNIZED				= 0x0003,
	RESPONSE_CHANNEL_NOT_FOUND				= 0x0004,
	RESPONSE_RESOURCES_EXHAUSTED			= 0x0005,
	RESPONSE_REMOTE_ERROR					= 0x0006,
	RESPONSE_URL_NOT_FOUND					= 0x0007,
	RESPONSE_END_OF_DATA					= 0x0008,
	RESPONSE_FUNCTION_NOT_SUPPORTED			= 0x0009,
	RESPONSE_CONNECTION_DOWN				= 0x000A,
	RESPONSE_CHANNEL_DELETED				= 0x000B,
	RESPONSE_FUNCTION_ABORTED				= 0x000C,
	RESPONSE_SERVICE_DOES_NOT_EXIST			= 0x000D,
	RESPONSE_SIGNALLING_ERROR				= 0x000E,
	RESPONSE_INVALID_DIRECTION				= 0x000F,
	RESPONSE_WINDOWS_SOCKETS_NOT_FOUND		= 0x0010,
	RESPONSE_CORRUPTED_MESSAGE_RECEIVED		= 0x0011,
	RESPONSE_CANNOT_CREATE_THREAD			= 0x0012,
	RESPONSE_REGISTRATION_ERROR				= 0x0013,
	RESPONSE_WRONG_RESOURCE_IDENTIFYER		= 0x0014,
	RESPONSE_TIMEOUT						= 0x0015,
	RESPONSE_ESID_NOT_FOUND					= 0x0016,
	RESPONSE_MUXED							= 0x0017,

	RESPONSE_SESSION_SETUP_ERROR			= 0x0030,
	RESPONSE_SESSION_RELEASE_ERROR			= 0x0031,
	RESPONSE_SERVICE_ATTACH_ERROR			= 0x0032,
	RESPONSE_SERVICE_DETACH_ERROR			= 0x0033,
	RESPONSE_CHANNEL_ADD_ERROR				= 0x0034,
	RESPONSE_CHANNEL_DELETE_ERROR			= 0x0035,
	RESPONSE_USER_COMMAND_ACK_ERROR			= 0x0036,

	RESPONSE_DATA_RECEIVE_ERROR				= 0x0040,
	RESPONSE_DATA_SEND_ERROR				= 0x0041,

	RESPONSE_gethostname_FAILED				= 0x0050,
	RESPONSE_gethostbyname_FAILED			= 0x0051,
	RESPONSE_socket_FAILED					= 0x0052,
	RESPONSE_connect_FAILED					= 0x0053,

	RESPONSE_LOCAL_DAEMON_DOWN				= 0x0060,
	RESPONSE_REMOTE_DAEMON_DOWN				= 0x0061,

	RESPONSE_REQUESTED_FILE_NOT_FOUND		= 0x0070,
	RESPONSE_DISK_FULL						= 0x0071,
	RESPONSE_IO_ERROR						= 0x0072,

	RESPONSE_UNKNOWN_ERROR					= 0xFFFF
} RESPONSE;

// Reason parameter
typedef enum tagReasonParameter
{
	REASON_OK								= 0x0000,
	REASON_REMOTE_ERROR						= 0x0001,
	REASON_REMOTE_INITIATED					= 0x0002,
	REASON_SESSION_CLOSED					= 0x0003,
	REASON_UNKNOWN_ERROR					= 0xFFFF
} REASON;

// Direction parameter for transport content
typedef enum tagDIRECTION
{
	UNSPECIFIED				= 0x00,
	DOWNSTREAM				= 0x01,
	UPSTREAM				= 0x02,
	BIDIRECTIONAL			= 0x03
} DIRECTION;

// This parameter indicates correctness of the data
typedef enum tagDATA
{
	DATA_EOF				= -1, // [Michael Stepping 2001] end of stream transmission
	DATA_OK					= 0x00,	// the PDU is a segment of an Access Unit
	DATA_UNBOUNDED			= 0x01,	// the PDU is not aligned with AU boundaries
	DATA_CORRUPTED			= 0x80	// some transmission error has been detected
} DATA_STATUS;

// The QoS parameters are streamed.
// MPEG-4 terminal has to negotiate the parameters with DMIF.
// Inside DMIF the Stream is decoded and used. Look at "QoSLoop"
// The exchange format is a SYSTEMS descriptor called qosDescriptor
// Inside a function call it has to be decoded before the
// function calls returns.
typedef LPCBYTE QOSSTREAM ;


#endif  //_DAItypes_h_
