/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

#if !defined(_Decoder_h)	// Sentry, use file only if it's not already included.
#define _Decoder_h

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Decoder
FILE:       Decoder.h
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Declaration of Decoder, an interface for Decoder objects.
-------------------------------------------------------------------------*/
#include "Filter.h"
#include "Capabilities.h"

#ifdef MPGJ
#include "../core/Executiv.h"	// MPEGJ MERGE - RLS
#endif

class MediaStream;
class Executive;
class Root;

enum PIXEL_FORMAT {
	PF_RGB   = 0x00000001,
	PF_BGR   = 0x00000002,
	PF_YUV   = 0x00000004,
	PF_VUY   = 0x00000008,
	PF_PREA  = 0x00000010,
	//PF_POSTA = 0x00000020,
	PF_POSTA = 0x00000020 // SOLARIS PORTING - Took off trailing ','
};

typedef enum {
		FRAME_WIDTH,
		FRAME_HEIGHT,
		PITCH,
		OFFSETX,
		OFFSETY,
		AUDIO_FREQUENCY,
		BITS_PER_SAMPLE,
		NUM_CHANNELS,
		FRAME_PIXMAP_WIDTH,
		FRAME_PIXMAP_HEIGHT,
		FRAME_BITS_PER_PIXEL,
		FRAME_PIXEL_FORMAT,
		AUDIO_CU_DURATION,
		IS_ALPHA
		// [ed: what else do we need?]
} ATTRIBUTE_CODE;

/*-------------------------------------------------------------
DecoderParams is used in the call to SetFormat
-------------------------------------------------------------*/
struct DecoderParams
{
	int m_nStreamNumber;
	MediaFormat m_format;
	int m_nOptimalAUSize;
	int m_nDBSize;
	int m_nCBSize;
	bool m_bRequiresFullAUs;
	bool m_bBackChannel;

	DecoderParams () :
		m_nStreamNumber (0), m_format (MediaFormat::NONE), m_nOptimalAUSize (256),
		m_nDBSize (1024), m_nCBSize (1024),
		m_bRequiresFullAUs (true), m_bBackChannel (false)
	{}
};

/*-------------------------------------------------------------------
declare the interface object
-------------------------------------------------------------------*/
class Decoder : public Filter
{
public:
	virtual BOOL Setup (
		const BYTE *pSpecificInfo,
		int nSpecificInfoLength,
		int	nESNumber
	) = 0;
	virtual BOOL SetFormat (
		Capabilities &formatOptions,
		DecoderParams &decoderParams
	) = 0;
	virtual int GetOptimalOutputSize () = 0;
	virtual void SetInputStream (int nESNumber, MediaStream *pStream) = 0;
	virtual void SetOutputStream (int nStreamNumber, MediaStream *pStream) = 0;
	virtual void SetStream (
		MediaStream *pStream,
		Executive *pExecutive,
		Root *pNode
	) = 0;
	virtual int GetDataOffset  (const LPBYTE pFrame) = 0;
	virtual DWORD GetAttributes (
		ATTRIBUTE_CODE code,
		const LPBYTE pFrame
	) = 0;

	#ifdef MPGJ
	virtual void SetExecutive(Executive *e) = 0; //PA
	int decoderType;// PA
	#endif
	int stop; //PA
};

#endif                                      // _Decoder_h sentry.
