/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

#if !defined(_Filter_h)	// Sentry, use file only if it's not already included.
#define _Filter_h

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Decoder/ IPMP systems
FILE:       Filter.h
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Declaration of Filter, an base interface for Decoder
			and IPMP objects.
-------------------------------------------------------------------------*/

class MediaStream;

class Filter
{
public:
	Filter () : m_hDLL (NULL) {}
	virtual ~Filter () {}
	virtual void Start () = 0;
	virtual void Stop () = 0;
	virtual void QueryInterface (
		const char* sTag,
		const void* pSink,
		void*& r_pSink
	) = 0;
	virtual const char* GetInfo() = 0;
	virtual void SetInputStream (MediaStream *pStream)
	{
		m_pInStream = pStream;
	};
	virtual void SetOutputStream (MediaStream *pStream)
	{
		m_pOutStream = pStream;
	}
	virtual MediaStream *GetInputStream () {return m_pInStream;}
	virtual MediaStream *GetOutputStream () {return m_pOutStream;}

	/*-------------------------------------------------------------
	The following method should be overloaded by decoders who accept multiple input
	streams. The decoder should replace input stream 'pOld' with inputStream 'pNew'
	-------------------------------------------------------------*/
	virtual void ReplaceInputStream (MediaStream * /*pOld */, MediaStream *pNew)
	{
		m_pInStream = pNew;
	}

	HMODULE GetLibrary () const {return m_hDLL;}
	void SetLibrary (HMODULE hDLL) {m_hDLL = hDLL;}

protected:
	virtual void Terminate () = 0;
	MediaStream *m_pInStream, *m_pOutStream;

private:
	HMODULE m_hDLL;
};

extern "C" void *CreateInstance ();

#endif                                      // _Filter_h sentry.
