/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

#if !defined(_defs_h)	// Sentry, use file only if it's not already included.
#define _defs_h

#if !defined (WIN32) && !defined(__UNIX)
#error specify Operating System: WIN32 or __UNIX
#endif
#if defined(__UNIX) && !defined(__SOLARIS) && !defined(__LINUX)
#error specifiy __UNIX concrete: use __SOLARIS or __LINUX
#endif

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  all
FILE:       defs.h
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Platform independent type definitions
-------------------------------------------------------------------------*/
#ifdef BIFSCAR
#define BIFSDEC
#endif

#ifdef SERVER
#define MUX
#endif

#ifdef MUX
#define BIFSENC
#endif

#ifndef BIFSENC
#define BIFSDEC
#endif

#if defined(BIFSDEC) && !defined(BIFSCAR)
#define PLAYER
#endif

#if defined(BIFSENC) && !defined(MUX) && !defined (BIFSCAR)
#define ENCODER
#endif

#if defined(BIFSENC) && !defined(TRACE_ENCODING)
#define TRACE_ENCODING
#endif

#ifdef BIFSENC
#define PREDICTIVE
#endif

typedef const unsigned char *LPCBYTE;

#ifdef WIN32

	#pragma warning(disable: 4201)  // nonstandard extension used : nameless struct/union
	#pragma warning(disable: 4514)  // unreferenced inline function has been removed
	#pragma warning(disable: 4226)  // nonstandard extension used : 'pascal' is an obsolete keyword
	#pragma warning(disable: 4244)  // conversion from 'const int' to 'float', possible loss of data
	#pragma warning(disable: 4355)  // 'this' : used in base member initializer list
	#pragma warning(disable: 4710)  // function not expanded
	#pragma warning(disable: 4800)  // forcing value to bool 'true' or 'false'

	#define STOREINT(n, pBuf) (* (int *) (pBuf) = n)
	#define LOADINT(pBuf) (* (int *) (pBuf))
	#define STOREWORD(n, pBuf) (* (WORD *) (pBuf) = n)
	#define LOADWORD(pBuf) (* (WORD *) (pBuf))
	#define STOREDWORD(n, pBuf) (* (DWORD *) (pBuf) = n)
	#define LOADDWORD(pBuf) (* (DWORD *) (pBuf))

	#if defined(BUILD_CSELT_APP) || defined(OSMOSE) || defined(PLAYER3D)
		#ifndef STAND_ALONE
	//PA		#include "StdAfx.h"
		#endif
		#include <afxwin.h>             // MFC core and standard components
		#include <afxext.h>             // MFC extensions
		#include "mmsystem.h"
		#ifndef PLAYER3D
		// MPEGJ porting - Commented because of afxeol.inl compile
			#include <afxdisp.h>            // MFC OLE automation classes
		#endif

		#ifndef _AFX_NO_AFXCMN_SUPPORT
			#include <afxcmn.h>             // MFC support for Windows Common Controls
		#endif	// _AFX_NO_AFXCMN_SUPPORT

	#else	//defined(BUILD_CSELT_APP) || defined(OSMOSE) || defined(PLAYER3D)

		#include <windows.h>       /* Windows include file */
		#include <stdio.h>

	#endif	// defined(BUILD_CSELT_APP) || defined(OSMOSE) || defined(PLAYER3D)

	#define COMPLAIN(a) OutputDebugString(a)
	#define M_PI 3.1415926535
	//addedGL
	#define ALMOST_ZERO 1e-10

#endif	// WIN32

#ifndef TRACE

	#ifdef __UNIX
		#include <stdarg.h>
		#include <stdio.h>
	#endif

	inline void TRACE (const char* a, ...)
	{
		va_list marker;
		char buf[1024];

		va_start(marker, a);
		vsprintf(buf, a, marker);
	#if defined( WIN32 )
		OutputDebugString(buf);
	#elif defined (__UNIX)
		printf(buf);
	#endif
		va_end(marker);
	}
#endif	// TRACE

//e.g. __UNIX
#ifdef __UNIX

	#include <string.h>
	#include <stdarg.h>
	#include <stdlib.h>
	#include <stdio.h>
	//#include <iostream.h>
	#include <iostream>
	#include <errno.h>
	#include <unistd.h> //usleep()

	#include <dlfcn.h>
	#include <netinet/in.h>

	#include <ctype.h> // SOLARIS PORTING - for tolower

	// Stepping: added types for Linux
	// 8Bit
	typedef char *LPSTR;
	typedef char *LPTSTR;
	typedef const char* LPCSTR;
	typedef const char* LPCTSTR;
	typedef char TCHAR;
	typedef unsigned char BYTE;
	typedef BYTE* LPBYTE;
	typedef const BYTE* LPCBYTE;
	// SOLARIS PORTING
	typedef unsigned char UCHAR ;

	//16Bit
	typedef unsigned short WORD;
	typedef unsigned short UINT16;

	//32Bit
	typedef unsigned int DWORD;
	typedef signed int INT32;
	typedef unsigned int UINT32;
	typedef int LONG;

	//64Bit
	typedef long LONG64;
	typedef long __int64;
	typedef unsigned long ULONG64;

	//divers
	#ifndef HANDLE
		#ifdef __SOLARIS
			typedef unsigned int HANDLE;
		#endif
		#ifdef __LINUX
			typedef void* HANDLE;
		#endif
	#endif
	typedef void* HINSTANCE;
	typedef void* HMODULE;
	typedef unsigned int HKEY;
	typedef unsigned int UINT;
	typedef struct tagRect {int left, top, width, height;} RECT;

	#define INFINITE 0xFFFFFFFF
	#define ERROR_SUCCESS	0

	#ifndef BOOL
		//typedef bool BOOL;
		#define BOOL bool // SOLARIS PORTING
	#endif	// BOOL

	#ifndef TRUE
		#define TRUE true
		#define FALSE false
	#endif	// TRUE

	#define min(a,b) ((a)>(b)?(b):(a))
	#define max(a,b) ((a)<(b)?(b):(a))

	#define LOBYTE(w)           ((BYTE)(w))
	#define HIBYTE(w)           ((BYTE)(((WORD)(w) >> 8) & 0xFF))
	#define LOWORD(l)           ((WORD)(l))
	#define HIWORD(l)           ((WORD)(((DWORD)(l) >> 16) & 0xFFFF))
	#define MAKEWORD(a, b)      ((WORD)(((BYTE)(a)) | ((WORD)((BYTE)(b))) << 8))
	#define MAKELONG(a, b)      ((LONG)(((WORD)(a)) | ((DWORD)((WORD)(b))) << 16))

#ifdef __SOLARIS
	#define STOREWORD(n, pBuf) \
		((pBuf) [0] = LOBYTE (n), (pBuf) [1] = HIBYTE (n))

	// SOLARIS PORTING - Corrected these
	#define STOREDWORD(n, pBuf) \
		(	STOREWORD (LOWORD (n), (pBuf) ), \
			STOREWORD (HIWORD (n), (pBuf) + sizeof (WORD) ) \
		)

	#define STOREINT(n, pBuf) STOREDWORD (n, pBuf)
	#define LOADWORD(pBuf) MAKEWORD ((pBuf) [0], (pBuf) [1])
	#define LOADDWORD(pBuf) \
		MAKELONG (LOADWORD (pBuf), LOADWORD ((pBuf) + sizeof (WORD)))
	#define LOADINT(pBuf) LOADDWORD (pBuf)
#endif
#ifdef __LINUX
	#define STOREINT(n, pBuf) (* (int *) (pBuf) = n)
	#define LOADINT(pBuf) (* (int *) (pBuf))
	#define STOREWORD(n, pBuf) (* (WORD *) (pBuf) = n)
	#define LOADWORD(pBuf) (* (WORD *) (pBuf))
	#define STOREDWORD(n, pBuf) (* (DWORD *) (pBuf) = n)
	#define LOADDWORD(pBuf) (* (DWORD *) (pBuf))
#endif


	#define COMPLAIN(a) printf(a)

	#define strnicmp strncasecmp
	#define _strnicmp strncasecmp
	#define stricmp strcasecmp

	#define wsprintf sprintf
	#define Sleep( time ) usleep( time * 1000 )

	#define PASCAL

	#ifdef __LINUX // SOLARIS PORTING
		#include <linux/limits.h>
	#endif
	#define _MAX_PATH	PATH_MAX

	#ifdef __UNIX

		//needed to throw error reports
		class CUserException
		{
		};

	#endif

	// SOLARIS PORTING

	class HWND{
	};

	#ifndef LPVOID
		typedef void *LPVOID;
	#endif

	// SOLARIS PORTING - Custom Header files
	#ifndef _strdup
		#define _strdup(val1) strdup(val1)
	#endif

	#ifndef FreeLibrary
		#define FreeLibrary(val1) delete(val1)
	#endif

	#ifndef atanf
		#define atanf(val) atan(val)
	#endif

	#ifndef acosf
		#define acosf(val) acos(val)
	#endif

	#ifndef cosf
		#define cosf(val) cos(val)
	#endif

	#ifndef sinf
		#define sinf(val) sin(val)
	#endif

	#ifndef tanf
		#define tanf(val) tan(val)
	#endif

	#ifndef sqrtf
		#define sqrtf(val) sqrt(val)
	#endif

	#include "time_def.h"
	#include "thread_utils.h"

#endif	// ndef __UNIX

#ifndef ASSERT
#if defined (_DEBUG) || defined (DEBUG)
#define ASSERT(n) assert(n)
#else
#define ASSERT(n)
#endif

#endif

#ifndef VERIFY

#if defined (_DEBUG) || defined (DEBUG)
#define VERIFY(n) assert(n)
#else
#define VERIFY(n) n
#endif
#endif

//ML - added the next two macros
#define min3(a,b,c) ((a)<(b)?min(a,c):min(b,c))
#define max3(a,b,c) ((a)>(b)?max(a,c):max(b,c))

#endif                                      // _defs_h sentry.
