/***********************************************************************
    This software module  was  originally  developed by Guido
	Franceschini (CSELT)
    in the course of development of the MPEG-4 DMIF (ISO/IEC 14496-6)
    standard.  This software module is an implementation of a part of
    one or more MPEG-4  DMIF  (ISO/IEC 14496-6)  tools  as  specified
    by  the  MPEG-4 DMIF (ISO/IEC 14496-6)  standard.  ISO/IEC  gives
    users of  the  MPEG-4  DMIF  (ISO/IEC 14496-6)  free  license  to
    this software module or modifications thereof for use in hardware
    or software products claiming  conformance  to  the  MPEG-4  DMIF
    (ISO/IEC 14496-6). Those intending to use this software module in
    hardware or software  products  are  advised  that  its  use  may
    infringe   existing  patents.  The  original  developer  of  this
    software module and his/her company, the subsequent  editors  and
    their  companies,  and  ISO/IEC have no liability for use of this
    software module or modifications thereof  in  an  implementation.
    Copyright  is not released for non MPEG-4 DMIF  (ISO/IEC 14496-6)
    conforming  products. CSELT
    retain  full  right to use the code for their own purpose, assign
    or donate the code to a third party and to inhibit third  parties
    from  using  the  code  for  non  MPEG-4  DMIF  (ISO/IEC 14496-6)
    conforming products. This copyright notice must  be  included  in
    all copies or derivative works. Copyright  1998.
************************************************************************/

// Contact: guido.franceschini@cselt.it

/**************************************************************************
 * Project Im1Player
 * INTERNATIONAL STANDARD ORGANIZATION
 * ISO/IEC JTC1/SC29/WG11
 *
 * SUBSYSTEM: DMIF
 * FILE:      sl_mpeg4.cpp
 * AUTHOR:    Guido Franceschini (CSELT)
 * OVERVIEW:  Implementation of the Sync Layer packetizer and depacketizer
 *            for the MPEG4 Sync Layer.
 *
 **************************************************************************/

#ifndef MPG4DMIF_H
#define MPG4DMIF_H

struct SLConfigDesc
{
	unsigned short predefined;
	bool useAccessUnitStartFlag;
	bool useAccessUnitEndFlag;
	bool useRandomAccessPointFlag;
	bool hasRandomAccessUnitsOnlyFlag;
	bool usePaddingFlag;
	bool useTimeStampsFlag;
	bool useIdleFlag;
	bool durationFlag;
	unsigned long timeStampResolution;
	unsigned long OCRResolution;
	unsigned short timeStampLength;	// must be  64
	unsigned short OCRLength;		// must be  64
	unsigned short AU_Length;		// must be  32
	unsigned short instantBitrateLength;	// must be ?????
	unsigned short degradationPriorityLength;
	unsigned short AU_seqNumLength; // must be  16
	unsigned short packetSeqNumLength; // must be  16
//	unsigned short reserved;
	unsigned long timeScale;
	unsigned short accessUnitDuration;
	unsigned short compositionUnitDuration;
	__int64 startDecodingTimeStamp;
	__int64 startCompositionTimeStamp;
//	SDLAlignedInt<1> OCRstreamFlag;
//	unsigned short reserved_2;
//	SDLInt<16> OCR_ES_Id;
};

struct SLHeader
{
	bool accessUnitStartFlag;
	bool accessUnitEndFlag;
	bool OCRflag;
	bool idleFlag;
	bool paddingFlag;
	unsigned short paddingBits;
	unsigned short packetSequenceNumber;
	__int64 objectClockReference;
	bool randomAccessPointFlag;
	unsigned short AU_sequenceNumber;
	bool decodingTimeStampFlag;
	bool compositionTimeStampFlag;
	bool instantBitrateFlag;
	__int64 decodingTimeStamp;
	__int64 compositionTimeStamp;
	unsigned short accessUnitLength;
	unsigned long instantBitrate;	// instantBitRateLength is ?????
	bool degradationPriorityFlag;
	unsigned short degradationPriority;
};

struct SegmentInfo
{
	bool accessUnitStartFlag;
	bool accessUnitEndFlag;
//	bool OCRflag;
//	bool idleFlag;
//	bool paddingFlag;
//	unsigned short paddingBits;
//	unsigned short packetSequenceNumber;
//	__int64 objectClockReference;
	bool randomAccessPointFlag;
	unsigned short AU_sequenceNumber;
	bool decodingTimeStampFlag;
	bool compositionTimeStampFlag;
	bool instantBitrateFlag;
	__int64 decodingTimeStamp;
	__int64 compositionTimeStamp;
	unsigned short accessUnitLength;
	unsigned long instantBitrate;	// instantBitRateLength is ?????
	unsigned short degradationPriority;
	unsigned short lsType;
	unsigned long transmissionTimeStamp;
};

extern void
SL_Packetize	(const struct SLConfigDesc* pConfig, const struct SLHeader* pHeader, const BYTE* pPDU, const int nPDULen, int* pnHeaderLen);

extern int
SL_Depacketize	(const struct SLConfigDesc* pConfig, const struct SLHeader* pHeader, const BYTE* pPDU, const int nPDULen, int* pnHeaderLen);

#endif // MPG4DMIF_H
