/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) and extended by Michael Stepping (FernUni Hagen)
and Taher Nasched (FernUni Hagen)in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

// -*- C++ -*-

#if !defined(_ztl_h)	// Sentry, use file only if it's not already included.
#define _ztl_h

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  all
FILE:       ztl.h
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   A template library for data containers.  Currently includes:

ZArray<type>
ZMap<type>
ZString
ZSemaphore
ZMutex
ZThread
Some general utility functions
-------------------------------------------------------------------------*/

#include "defs.h"
#include "limits.h"
#include "assert.h"

// Michael Stepping
// shift some methods to inlining in release version
// 	define it always, (for non WIN32 systems)
	#define INLINE
#ifdef WIN32
	// some methods can be inlined
	#ifdef NDEBUG
	#undef INLINE
	#define _ENABLE_INLINE_EXPANSION
	#define INLINE inline
	#endif
#endif




/*-----------------------------------------------------------------
Array class
-----------------------------------------------------------------*/
class ZArrayBase
{
	enum {GROW_BY = 16};

	int m_nElementSize;
	int m_nLength, m_nSize;

protected:
	void *m_pBuffer;

	ZArrayBase (int nElementSize) :
		m_nElementSize (nElementSize),
		m_nLength (0),
		m_nSize (0),
		m_pBuffer (NULL)
	{}
	~ZArrayBase () {if (m_pBuffer) free (m_pBuffer);}
	void Clear () {m_nLength = 0;}
	void Stress (int nSize);
	void Shrink (int nSize);
	int Increase ()
	{
		int nResult = m_nLength;
		MakeRoomAt (m_nLength);
		return nResult;
	}

	void MakeRoomAt (int i);
	void DeleteFrom (int i);
	void DeleteLast () {if (m_nLength > 0) DeleteFrom (m_nLength - 1);}
	void *Search (class ZCriterion *pCriterion);
	void Pump (ZArrayBase &rArray);

public:

  /// Get number of elements

	int GetLength () const {return m_nLength;}
};


/*-----------------------------------------------------------------
Parameterised array class
-----------------------------------------------------------------*/
template <class T> class ZArray : public ZArrayBase
{
public:

	/*-----------------------------------------------------------------
	constructor
	-----------------------------------------------------------------*/
	ZArray () : ZArrayBase (sizeof (T)) {}

	/*-----------------------------------------------------------------
	destructor.
	-----------------------------------------------------------------*/
	~ZArray ()
	{
		int i;

		/*-----------------------------------------------------------------
		The following trick will make sure the destructors for all
		elements are called.
		Caution: this cannot be used if the default constructor for T
		allocates any resources!!
		-----------------------------------------------------------------*/
		for (i = 0; i < GetLength (); i++)
		{	T defT;

			memcpy (&defT, (T*) m_pBuffer + i, sizeof (T));
		}
	}

	/*-----------------------------------------------------------------
	Access the buffer.
	-----------------------------------------------------------------*/
	T* GetBuffer () { return (T*) m_pBuffer; }; // VUB


	/*-----------------------------------------------------------------
	Clear all.
	-----------------------------------------------------------------*/
	void Clear  ()
	{
		int i;

		/*-----------------------------------------------------------------
		The following trick will make sure the destructors for all
		elements are called.
		-----------------------------------------------------------------*/
		for (i = 0; i < GetLength (); i++)
		{	T defT;

			memcpy (&defT, (T*) m_pBuffer + i, sizeof (T));
		}
		ZArrayBase :: Clear ();
	}

	/*-----------------------------------------------------------------
	type conversion
	-----------------------------------------------------------------*/
	operator const T* () const {return (T*) m_pBuffer;}

	/*-----------------------------------------------------------------
	Expand the array, fill all new entries with default object
	-----------------------------------------------------------------*/
	void Expand (int i)
	{
		int j = GetLength ();

		if (i >= j)
		{	T defT;
			T &refT = defT;		// required to avoid compiler warning in release mode

			Stress (i);
			for (;j <= i; j++)
				memcpy ((T*) m_pBuffer + j, &refT, sizeof (T));
		}
	}

	/*-----------------------------------------------------------------
	Set a fixed size
	-----------------------------------------------------------------*/
	void SetSize (int i)
	{
		int j = GetLength ();

		if (i > j)
		{
      // expand
      T defT;
			T &refT = defT;		// required to avoid compiler warning in release mode

			Stress (i);
			for (;j <= i; j++)
				memcpy ((T*) m_pBuffer + j, &refT, sizeof (T));
    }
    else if (i < j)
    {
		  /*-----------------------------------------------------------------
		  The following trick will make sure the destructors for all
		  elements are called.
		  Caution: this cannot be used if the default constructor for T
		  allocates any resources!!
		  -----------------------------------------------------------------*/
		  for (int k = i; k < j; k++)
		  {	T defT;

			  memcpy (&defT, (T*) m_pBuffer + k, sizeof (T));
		  }
      // shrink
      Shrink (i);
		}
	}

	/*-----------------------------------------------------------------
	set element
	-----------------------------------------------------------------*/
	T &SetAt (int i, const T t)
	{
		Expand (i);
		return ((T*) m_pBuffer) [i] = t;
	}

	/*-----------------------------------------------------------------
	Delete element at position i
	-----------------------------------------------------------------*/
	void DeleteFrom (int i)
	{
		T defT;

		/*-----------------------------------------------------------------
		The following trick will make sure the destructor of the element
		is called.
		-----------------------------------------------------------------*/
		memcpy (&defT, (T*) m_pBuffer + i, sizeof (T));
		ZArrayBase :: DeleteFrom (i);
	}

	/*-----------------------------------------------------------------
	get element
	-----------------------------------------------------------------*/
	T &ElementAt (int i) const
	{
		if (i >= GetLength ())
			assert (0);
		return ((T*) m_pBuffer) [i];
	}


	/*-----------------------------------------------------------------
	get element
	-----------------------------------------------------------------*/
	T &operator [] (int i)
	{
		if (i >= GetLength ())
			assert (0);
		return ((T*) m_pBuffer) [i];
	}

	T &operator [] (int i) const
	{
		return ((T*) m_pBuffer) [i];
	}

	/*-----------------------------------------------------------------
	get element, expand if necessary
	-----------------------------------------------------------------*/
	T &ElementAt (int i)
	{
		Expand (i);
		return ((T*) m_pBuffer) [i];
	}

	/*-----------------------------------------------------------------
	Find an element and return its index
	-----------------------------------------------------------------*/
	int Find (const T &t) const
	{
		for (int i = GetLength (); --i >= 0;)
			if ((*this) [i] == t)
				return i;
		return -1;
	}

	/*-----------------------------------------------------------------
	Add an element to the end of the array
	-----------------------------------------------------------------*/
	T &operator += (const T &t)
	{
		int n = Increase ();
		T defT;
		T &refT = defT;		// required to avoid compiler warning in release mode

		memcpy ((T*) m_pBuffer + n, &refT, sizeof (T));
		((T*) m_pBuffer) [n] = t;
		return ((T*) m_pBuffer) [n];
	}

	/*-----------------------------------------------------------------
	Find an element and delete it from the array
	-----------------------------------------------------------------*/
	BOOL operator -= (const T &t)
	{
		int i = Find (t);

		if (i >= 0)
		{
			DeleteFrom (i);
			return TRUE;
		}
		else
			return FALSE;
	}

	T &InsertAt (int i, const T &t)
	{
		if (i >= GetLength ())
			Expand (i);
		else
		{
			MakeRoomAt (i);

			// MBS 032803: removed for release mode. Useless since MakeRoomAt sets value to 0

			T defT;
			T &refT = defT;		// required to avoid compiler warning in release mode
			memcpy ((T*) m_pBuffer + i, &refT, sizeof (T));
			//memcpy ((T*) m_pBuffer + i, &defT, sizeof (T));

		}
		return ((T*) m_pBuffer) [i] = t;
	}

	/*-----------------------------------------------------------------
	operator ++ appends one default T at the end of the array and returns
	a reference to it
	-----------------------------------------------------------------*/
	T &operator ++ ()
	{
		const int n = Increase ();
		// MBS 032803: removed for release mode. Useless since Increase sets value to 0
		T defT;

		T &refT = defT;		// required to avoid compiler warning in release mode
		memcpy ((T*) m_pBuffer + n, &refT, sizeof (T));
		//memcpy ((T*) m_pBuffer + n, &defT, sizeof (T));

		return ((T*) m_pBuffer) [n];
	}

	T *Search (class ZCriterion *pCriterion)
	{
		return (T*) ZArrayBase :: Search (pCriterion);
	}

	/*-----------------------------------------------------------------
	Copy the contents of another array. The other array is left empty.
	-----------------------------------------------------------------*/
	ZArray<T> &operator <<= (ZArray<T> &array)
	{
		Clear ();
		Pump (array);
		return *this;
	}

  // JMR - Start Update for SB 10/24/99: added for Karl's code
	/*-------------------------------------------------------
	ZArray :: operator = (const ZArray& rSource) - copy assignment
	-------------------------------------------------------*/
	ZArray<T>& operator = (const ZArray<T>& rSource)
	{
		Clear(); // clear THIS
		// copy each element from SOURCE to THIS
		for (int i = 0; i < rSource.GetLength(); i++)
			*this += rSource [i];
		return *this;
	}

  // JMR - End Update for SB

};

/*-----------------------------------------------------------------
ZCriterion is a base class for objects which have a virtual function,
IsOK(), which receives a pointer to NULL, casts it to pointer to
some object, and checks whether the object meets some criterion
-----------------------------------------------------------------*/
class ZCriterion
{
public:
	virtual BOOL IsOK (void *object) const = 0;
};

/*-------------------------------------------------------------
ZMapBase. A base class for ZMap - the template class that maps integer indice to objects
-------------------------------------------------------------*/
class ZMapBase
{
public:
	int GetLength () const {return m_nLength;}
	bool Exists (int index) const {return FindIndex (index) >= 0;}

protected:
	ZMapBase (int nElementSize);
	~ZMapBase ();
	int FindIndex (int index) const;
	int InsertIndex (int index);
	int DeleteIndex (int index);
	char *GetLocation (int index) const {return m_pBuffer + index * m_nElementSize;}
	bool HasValue (int i) const {return m_pIndice [i] != -1;}
	int GetSize () const {return m_nSize;}

private:
	int *m_pIndice;
	char *m_pBuffer;
	int m_nElementSize, m_nSize, m_nLength, m_nMask, m_nShift;
};

/*-------------------------------------------------------------
ZMap class
Implement a hashed table that maps integer indice to objects.
-----------------------------------------------------------------*/
template <class T> class ZMap : public ZMapBase
{
public:
	/*-------------------------------------------------------------
	Exception object thrown when an object is not found
	-------------------------------------------------------------*/
	class NotFound {};

	/*-----------------------------------------------------------------
	constructor
	-----------------------------------------------------------------*/
	ZMap () : ZMapBase (sizeof (T)) {}

	/*-----------------------------------------------------------------
	destructor.
	-----------------------------------------------------------------*/
	~ZMap ()
	{
		/*-----------------------------------------------------------------
		The following trick will make sure the destructors for all
		elements are called.
		Caution: this cannot be used if the default constructor for T
		allocates any resources!!
		-----------------------------------------------------------------*/
		int i;
		for (i = 0; i < GetSize (); i++)
		{
			if (HasValue (i))
			{	T defT;
				memcpy (&defT, GetLocation (i), sizeof (T));
			}
		}
	}

	/*-------------------------------------------------------------
	Find an element in the map
	-------------------------------------------------------------*/
	const T &operator [] (int index) const
	{
		int i = FindIndex (index);
		if (i < 0)
			throw NotFound ();
		return *(T*) GetLocation (i);
	}

	/*-----------------------------------------------------------------
	Insert an element into the map
	-----------------------------------------------------------------*/
	T &InsertAt (int index, const T &t)
	{
		int i = FindIndex (index);
		if (i < 0)
		{
			i = InsertIndex (index);
			// MBS 032803: removed for release mode. Useless.
			/*
			T defT;
			memcpy (GetLocation (i), &defT, sizeof (T));
			*/
		}
		return *(T*) GetLocation (i) = t;
	}

	/*-----------------------------------------------------------------
	Delete an element
	-----------------------------------------------------------------*/
	void DeleteFrom (int index)
	{
		int i = DeleteIndex (index);
		if (i >= 0)
		{	/*-----------------------------------------------------------------
			The following trick will make sure the destructor of the element
			is called.
			-----------------------------------------------------------------*/
			T defT;
			memcpy (&defT, GetLocation (i), sizeof (T));
		}
	}
};

/*-----------------------------------------------------------------
String class
-----------------------------------------------------------------*/
class ZString
{
	enum {GROW_BY = 16};

	int m_nLength, m_nSize;
	unsigned char *m_pBuffer;


private:
	void Stress (int i, BOOL bFill = FALSE);

public:

	/*-----------------------------------------------------------------
	constructor
	-----------------------------------------------------------------*/
	ZString (LPCBYTE szStr, int nLength);
	ZString (LPCSTR szStr = NULL);
	ZString (const ZString &str);

	~ZString ()
	{	if (m_pBuffer)
			free (m_pBuffer);
	}

	ZString &operator = (float f)
	{
		Stress (20);
		sprintf ((char*) m_pBuffer, "%f", f);
		Cut (0, strlen ((char*) m_pBuffer));
		return *this;
	}
	void Cut (int nFrom, int nLength = -1);
	ZString Substr (int nFrom, int nLength = -1) const;
	int Find (int c, int iOffset = 0) const;
	int ReverseFind (int c, int iOffset = 0) const;
	operator LPCSTR () const {return m_pBuffer ? (LPCSTR) m_pBuffer : "";}
	operator unsigned char * () const {return m_pBuffer;}
	unsigned char &operator [] (int i);
	unsigned char operator [] (int i) const {return m_pBuffer [i];}
	ZString &operator += (char c);
	ZString &operator += (char *szStr);
	ZString &operator = (const ZString &str);
	ZString &operator = (LPCSTR szStr);
	BOOL operator == (const ZString &str) const;
	BOOL operator == (LPCSTR sToCompare) const;
	BOOL operator != (LPCSTR sToCompare) const;
	unsigned char InsertAt (int i, char c, int n = 1);
	const ZString &InsertAt (int i, const ZString &str);
	char DeleteFrom (int i);
	int GetLength () const {return m_nLength;}
	ZString &operator += (const ZString &str);
	ZString operator + (const ZString &str) const;
	ZString &Copy (LPCSTR szSource, int nMax = INT_MAX);
	ZString &Assign (LPCBYTE pData, int nLength);
	int FRead (FILE *pFile, int nSize);
	int FWrite (FILE *pFile);
	int Format(const char* szFormatString,...);
	void Split(char Splitter, ZString& sLeft, ZString& sRight) const;
	int AsInt() const;
};

#ifdef WIN32

/*----------------------        Win32       -------------------------------
ZSemaphore, ZMutex
------------------------        Win32       -----------------------------*/

class ZSemaphore
{
	HANDLE m_hSemaph;
public:
	// constructor
	ZSemaphore (long lMaxCount = 1, long lInitialCount = -1);

	//destructor, does not signall waiting Semaphores
	~ZSemaphore ()
		{ ::CloseHandle (m_hSemaph);}

	DWORD P (UINT uTimeout = INFINITE)
		{return ::WaitForSingleObject (m_hSemaph, uTimeout);}

	/* signal Semaphore, sometimes a lot of at once (nSignalled)
		0: error, 1: success */
	BOOL V( long nSignalled = 1 )
		{return ::ReleaseSemaphore (m_hSemaph, nSignalled, NULL);}

	// try to bypass a semaphore, return value: if not signalled, true
	BOOL Try ();

	/* really necessary to retrieve internal handles? Michael Stepping
		it should run without this method */
	HANDLE GetHandle ()
		{return m_hSemaph;}
};

class ZMutex
{
	HANDLE m_hMutex;

public:
	ZMutex() {m_hMutex = CreateMutex (NULL, FALSE, NULL);}
	~ZMutex() {CloseHandle (m_hMutex);}

	DWORD P(UINT uTimeout = INFINITE)
		{return WaitForSingleObject (m_hMutex, uTimeout);}

	BOOL V()
		{return ReleaseMutex (m_hMutex);}

	BOOL Try();

#ifdef _DEBUG
	void Trace(LPCSTR szOp);
#endif
};

#elif defined(__UNIX) // WIN32
/*----------------------        UNIX        -----------------------------
ZSemaphore, ZMutex
------------------------        UNIX        -----------------------------*/


#include <pthread.h>

#define WAIT_TIMEOUT ((DWORD)0x00000102L)
#define WAIT_OBJECT_0 TRUE

class ZSemaphore
{
	pthread_cond_t  m_Cond;
	pthread_mutex_t m_Mutex;
  // internal behaviour:
	// now values below 0 indicate a Wait.
	// 0 and above means an accesible resource
	int m_nCount;

public:
	// external behaviour:
	// 1 and above: pass
	// 0 and below: stop
	// initial value 1: pass
	ZSemaphore (long lMaxCount = 1, long lInitialCount = -1);
	~ZSemaphore ();
	DWORD P (UINT uTimeout = INFINITE);
	BOOL V ( long nSignalled = 1 ) ;
	BOOL Try ();
};

class ZMutex
{
	pthread_mutex_t m_Mutex;
	/** SOLARIS PORTING Housekeeping for pthread to avoid double locks */

	pthread_t m_LockingThread;

public:
	ZMutex ()
	{
		if (::pthread_mutex_init (&m_Mutex, NULL))
			perror ("pthread_mutex_init");
	}

	~ZMutex ()
	{
		if (::pthread_mutex_destroy (&m_Mutex))
			perror ("pthread_mutex_destroy");
	}

	// SOLARIS PORTING
	DWORD P ()
	{
	 	if (m_LockingThread != pthread_self())
			if (::pthread_mutex_lock (&m_Mutex))
			{	perror ("pthread_mutex_lock");
				return FALSE;
			}
			else {
				m_LockingThread = pthread_self();
				return TRUE;
			}
		else
			return TRUE;
	}

	BOOL V ()
	{
		if (::pthread_mutex_unlock (&m_Mutex))
		{	perror ("pthread_mutex_unlock");
			return FALSE;
		}
		else {
			m_LockingThread = NULL;
			return TRUE;
		}
	}

	BOOL Try ()
	{
		switch (::pthread_mutex_trylock(&m_Mutex))
		{
			case 0:
				return TRUE;
			case EBUSY:
				return FALSE;
			default:
				perror("pthread_mutex_trylock");
		}
	}
};


#else		// __UNIX
/*----------------------        SGI        -----------------------------
ZSemaphore, ZMutex
------------------------        SGI        -----------------------------*/

#include <ulocks.h>

class ZSGIArena
{
public:
	static usptr_t *arena()
	{
		if (!m_pArena)
		{
			::usconfig(CONF_INITSIZE, 1024 * 1024);
			::usconfig(CONF_INITUSERS, 32);
			::usconfig(CONF_ARENATYPE, US_SHAREDONLY);
			if (!(m_pArena = ::usinit("/dev/zero")))
				::perror("usinit");
		}
		return m_pArena;
	}

private:
	static usptr_t *m_pArena;
};


class ZSemaphore
{
	usema_t *m_pSema;

public:
	ZSemaphore (long lMaxCount = 1, long lInitialCount = -1);
	~ZSemaphore () {::usfreesema(m_pSema, ZSGIArena::arena());}
	DWORD P ()
	{	if (::uspsema(m_pSema) < 0)
		{	::perror("uspsema");
			return FALSE;
		}
		else
			return TRUE;
	}

	BOOL V ()
	{	if (::usvsema(m_pSema) < 0)
		{	::perror("usvsema");
			return FALSE;
		}
		else
			return TRUE;
	}
	BOOL Try () {return ::uscpsema(m_pSema);}
};

class ZMutex
{
	ulock_t m_Mutex;

public:
	ZMutex ()
	{	if (!(m_Mutex = usnewlock(ZSGIArena::arena())))
			::perror("usnewlock");
	}
	~ZMutex () {usfreelock (m_Mutex, ZSGIArena::arena());}
	DWORD P ();
    BOOL V ()
	{	if (usunsetlock(m_Mutex))
		{	::perror("usunsetlock");
			return FALSE;
		}
		else
			return TRUE;
	}

	BOOL Try ();
};

#endif //WIN32 -> __UNIX


/*-----------------------------------------------------------------
A template class which encapsultes thread control
-----------------------------------------------------------------*/

#ifdef WIN32
#include <process.h>
#else
typedef enum
{
	THREAD_PRIORITY_ABOVE_NORMAL,
	THREAD_PRIORITY_BELOW_NORMAL,
	THREAD_PRIORITY_NORMAL
} ZThreadPriority;

#ifndef THREAD_PRIORITY_HIGHEST
#define THREAD_PRIORITY_HIGHEST THREAD_PRIORITY_ABOVE_NORMAL
#endif

#endif

template <class T> class ZThread
{
public:
	typedef void (T :: *ThreadFunction) ();

	ZThread (
		ThreadFunction function,
		T *object,
		int nPriority = THREAD_PRIORITY_NORMAL
	) :
		m_pObject (object),
		m_function (function),
		m_nPriority (nPriority),
		m_smpThreadRunning (1, 0),
		m_hThread (0)
	{}

	~ZThread ()
	{
#ifdef WIN32
		if (m_hThread != NULL)
			::CloseHandle (m_hThread);
#elif defined (__UNIX)
		int TODO;
#endif
	}

	int Begin ()
	{
#ifdef WIN32
		m_hThread =
			(HANDLE) _beginthreadex (NULL, 0, (unsigned (__stdcall*) (void *)) ThreadProc, this, 0, (unsigned*)&m_threadID);
#elif defined (__UNIX)
		m_hThread = NULL ;
		pthread_t thread;
		int nError =
			::pthread_create (
				&thread,
				NULL,
				(void *(*)(void*)) ThreadProc,
				this
			);
		// successfull
		if ( nError == 0 )
			m_hThread = (HANDLE) thread ;

		// add set priority class
#else
		m_hThread = ::sproc (ThreadProc, PR_SADDR | PR_SFDS, this);
#endif

		/*-----------------------------------------------------------------
		Wait till thread retrieves values and run (if we don't wait the
		object might be deleted before the thread has the chance to
		retrieve the values
		-----------------------------------------------------------------*/

// SOLARIS PORTING - adding ifdef's for Solaris
#ifdef WIN32
		if (m_hThread != NULL)
#else
		if (nError == 0)
#endif
			m_smpThreadRunning.P ();
		else
			return -1;

		return (int) m_hThread;
	}

	void Wait(DWORD dwTimeout = INFINITE)
	{
#ifdef WIN32
		WaitForSingleObject (m_hThread, dwTimeout);
		CloseHandle (m_hThread);
#endif
#ifdef __UNIX
		void *result = NULL;
		pthread_join(m_hThread,&result);

#endif
		m_hThread = NULL;
	}
	HANDLE GetThread () const {return m_hThread;}
	void SetThreadID (DWORD threadID) {m_threadID = threadID;}

private:
	T *m_pObject;
	ThreadFunction m_function;
	ZSemaphore m_smpThreadRunning;
	int m_nPriority;
	HANDLE m_hThread;
	DWORD m_threadID;

	static void ThreadProc (ZThread<T> *pInstance)
	{
		T *pObject = pInstance -> m_pObject;
		ThreadFunction function = pInstance -> m_function;
#ifdef WIN32
		pInstance ->SetThreadID (GetCurrentThreadId ());
		SetThreadPriority (GetCurrentThread(), pInstance -> m_nPriority);
#endif
		pInstance -> m_smpThreadRunning.V ();
		(*pObject.*function) ();
	}
};


/*-------------------------------------------------------------
A class of very large integers
-------------------------------------------------------------*/
class ZLargeInteger
{
protected:
	ZArray<DWORD> number;
	int length;
	DWORD m_dwCarry;

public:
	DWORD GetCarry () {return m_dwCarry;}
	ZLargeInteger pwr (const ZLargeInteger &exp);
	DWORD GetWord (int i) const {return number [length - i - 1];}
	operator int () const {return number [0];}
	void RemainderOf (const ZLargeInteger n);
	void DivideBy (const ZLargeInteger n);
	void MultiplyBy (const ZLargeInteger n);
	void SubtractFrom (const ZLargeInteger n);
	bool IsZero () const;
	void AddOnto (const ZLargeInteger n);
	int GetSize () const {return length * sizeof (DWORD) * 8;}
	ZLargeInteger (DWORD nLow = 0, DWORD nHigh = 0, int nBits = sizeof (DWORD) * 8);
	ZLargeInteger (const ZLargeInteger &n);
	bool operator == (const ZLargeInteger &n) const;
	bool operator != (const ZLargeInteger &n) const  {return !(*this == n);}
	bool operator > (const ZLargeInteger &n) const;
	bool operator < (const ZLargeInteger &n) const;
	bool operator >= (const ZLargeInteger &n) const  {return !(*this < n);}
	bool operator <= (const ZLargeInteger &n) const {return !(*this > n);}
	ZLargeInteger &operator <<= (unsigned n);
	ZLargeInteger &operator >>= (unsigned n);
	const ZLargeInteger operator + (const ZLargeInteger &n) const
	{
		ZLargeInteger temp (0, 0, max (GetSize (), n.GetSize ()));
		temp = *this;
		temp.AddOnto (n);
		return temp;
	}
	const ZLargeInteger operator - (const ZLargeInteger &n) const
	{
		ZLargeInteger temp (0, 0, max (GetSize (), n.GetSize ()));
		temp = *this;
		temp.SubtractFrom (n);
		return temp;
	}
	const ZLargeInteger operator * (const ZLargeInteger &n) const
	{
		ZLargeInteger temp (0, 0, max (GetSize (), n.GetSize ()));
		temp = *this;
		temp.MultiplyBy (n);
		return temp;
	}
	const ZLargeInteger operator / (const ZLargeInteger &n) const
	{
		ZLargeInteger temp (0, 0, max (GetSize (), n.GetSize ()));
		temp = *this;
		temp.DivideBy (n);
		return temp;
	}
	const ZLargeInteger operator % (const ZLargeInteger &n) const
	{
		ZLargeInteger temp (0, 0, max (GetSize (), n.GetSize ()));
		temp = *this;
		temp.RemainderOf (n);
		return temp;
	}

protected:
	void Assign (DWORD nLow);
	void Assign (const ZLargeInteger &n)
	{
		number = n.number;
	}
};

/*-------------------------------------------------------------
A template for very large integers
-------------------------------------------------------------*/
// SOLARIS PORTING
#ifdef WIN32
template <int nBits = 64> class ZLargeInt : public ZLargeInteger
#else
template <int nBits> class ZLargeInt : public ZLargeInteger
#endif
{
public:
	ZLargeInt (const ZLargeInteger &n) : ZLargeInteger (n) {}
	ZLargeInt (DWORD nLow = 0, DWORD nHigh = 0) :
		ZLargeInteger (nLow, nHigh, nBits)
	{}

	ZLargeInteger &operator = (const ZLargeInteger &n)
	{

		Assign (n);
		return *this;
	}
	ZLargeInteger &operator = (DWORD n)
	{
		Assign (n);
		return *this;
	}
/*	ZLargeInt<nBits> &operator = (const ZLargeInt<nBits> &n)
	{
		Assign (n);
		return *this;
	}*/
	ZLargeInt<nBits> &operator += (const ZLargeInteger &n)
	{
		AddOnto (n);
		return *this;
	}
	ZLargeInt<nBits> &operator -= (const ZLargeInteger &n)
	{
		SubtractFrom (n);
		return *this;
	}
	ZLargeInt<nBits> &operator *= (const ZLargeInteger &n)
	{
		MultiplyBy (n);
		return *this;
	}
	ZLargeInt<nBits> &operator /= (const ZLargeInteger &n)
	{
		DivideBy (n);
		return *this;
	}
	ZLargeInt<nBits> &operator %= (const ZLargeInteger &n)
	{
		RemainderOf (n);
		return *this;
	}
	const ZLargeInteger operator % (const ZLargeInteger &n) const
	{
		ZLargeInteger temp (0, 0, max (GetSize (), n.GetSize ()));
		temp = *this;
		temp.RemainderOf (n);
		return temp;
	}
};



/*-------------------------------------------------------------
A class which encapsulates registry access
-------------------------------------------------------------*/
#ifdef WIN32

class ZRegistry
{
	HKEY m_hSection;

public:
	BOOL Enumerate (int index, LPSTR szBuffer, int nBufferLen);
	ZRegistry (LPCSTR szSection, BOOL bCreate = FALSE, HKEY hKey = HKEY_CURRENT_USER, LPCSTR szRoot = "Software\\MPEG-4\\IM1\\");
	~ZRegistry ();
	void *CreateInstance (LPCSTR szKey, HINSTANCE &hDLL) const;
	BOOL QueryValue(LPCSTR szName,LPBYTE szData,DWORD& nDataLen) const;
};

#elif (__UNIX)

#define HKEY_CLASSES_ROOT           (( HKEY ) 0x80000000 )
#define HKEY_CURRENT_USER           (( HKEY ) 0x80000001 )
#define HKEY_LOCAL_MACHINE          (( HKEY ) 0x80000002 )
#define HKEY_USERS                  (( HKEY ) 0x80000003 )
#define HKEY_PERFORMANCE_DATA       (( HKEY ) 0x80000004 )
#define HKEY_CURRENT_CONFIG         (( HKEY ) 0x80000005 )
#define HKEY_DYN_DATA               (( HKEY ) 0x80000006 )

#define REGFILE "./registry.txt"
#define TEMPFILE "./registry.tmp"
#define BUFFERSIZE 1024

class ZRegistry
{
	FILE *m_hFile;
	LPSTR m_pszRegPath;
	LPSTR m_pszSectionValue;
	LPSTR m_pRegFile;

public:
	LPCSTR IdentifyKeyValue(HKEY hKey);
	BOOL Enumerate (int index, LPSTR szBuffer, int nBufferLen);
	ZRegistry (LPCSTR szSection, BOOL bCreate = FALSE, HKEY hKey = HKEY_CURRENT_USER, LPCSTR szRoot = "Software\\MPEG-4\\IM1\\", LPCSTR RegFile = REGFILE);
	~ZRegistry ();
	void *CreateInstance (LPCSTR szKey, HINSTANCE &hDLL);
	BOOL QueryValue(LPCSTR szName,LPBYTE szData,DWORD& nDataLen);
	BOOL SetValue(LPCSTR szSection,LPCSTR szValue,BOOL Create);

private:
	BOOL RootExists(LPCSTR szRoot);
	BOOL CreateRoot(LPCSTR szRoot);
	BOOL SectionExists(LPCSTR szSection);
	BOOL CreateSection(LPCSTR szRoot,LPCSTR szSection);
	void RemoveNewLine(LPSTR StrBuffer);
	void Init(LPCSTR szSection, BOOL bCreate, HKEY hKey, LPCSTR szRoot, LPCSTR RegFile);
};


BOOL WritePrivateProfileString(
  LPCSTR lpAppName,  // section name
  LPCSTR lpKeyName,  // key name
  LPCSTR lpString,   // string to add
  LPCSTR lpFileName  // initialization file
);

int GetPrivateProfileInt(
  LPCSTR lpAppName,        // section name
  LPCSTR lpKeyName,        // key name
  int nDefault,        // default string
  LPCSTR lpFileName        // initialization file name
);

DWORD GetPrivateProfileString(
  LPCSTR lpAppName,        // section name
  LPCSTR lpKeyName,        // key name
  LPCSTR lpDefault,        // default string
  LPSTR lpReturnedString,  // destination buffer
  DWORD nSize,              // size of destination buffer
  LPCSTR lpFileName        // initialization file name
);

#endif

/*-----------------------------------------------------------------
Utilities
-----------------------------------------------------------------*/
int BitFieldWidth (int nMax);
BOOL ZFileExists (LPCSTR szFile);
ZString ComposeFileName (LPCSTR szFileName, LPCSTR szFileType);
int ZExpandURL (LPCSTR szRootURL, LPCSTR szRelURL, LPSTR szAbsURL, int nSize);
void ZBin2Base64 (const ZString &stFrom, ZString &stTo);
#ifdef __UNIX
char * itoa(int value, char *string, int radix);
#endif // __UNIX

#ifdef _ENABLE_INLINE_EXPANSION
#include "ztl.inl"
#endif


#endif // _ztl_h sentry.
