/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) and extended by Michael Stepping (FernUni Hagen)
and Taher Nasched (FernUni Hagen) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  all
FILE:       ztl.inl
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Implementation of the template library for data containers.
Currently includes:


This file contains the inline-methods for Release versions of the
file ztl.cpp. Necessary to do this, because the code has to be available
while inline expansion occurs.

if INLINE is defined, this file is included by ztl.h
otherwise it is included by ztl.cpp
-------------------------------------------------------------------------*/


/*-----------------------------------------------------------------
Make room for new element at position i
-----------------------------------------------------------------*/
INLINE void ZArrayBase :: MakeRoomAt (int i)
{
	Stress (m_nLength);
	if (m_nLength > i + 1)
		memmove (
			((char *) m_pBuffer) + (i + 1) * m_nElementSize,
			((char *) m_pBuffer) + i * m_nElementSize,
			(m_nLength - i - 1) * m_nElementSize
		);
	memset (((char *) m_pBuffer) + i * m_nElementSize, 0, m_nElementSize);
}

/*-----------------------------------------------------------------
Delete element at position i
-----------------------------------------------------------------*/
INLINE void ZArrayBase :: DeleteFrom (int i)
{
	if (m_nLength > i + 1)
		memmove (
			((char *) m_pBuffer) + i * m_nElementSize,
			((char *) m_pBuffer) + (i + 1) * m_nElementSize,
			(m_nLength - i - 1) * m_nElementSize
		);
	--m_nLength;
}


/*-------------------------------------------------------------
Find an index in a ZMap table
-------------------------------------------------------------*/
INLINE int ZMapBase :: FindIndex (int index) const
{
	int i = (index ^ (index >> m_nShift)) & m_nMask;
	int n = m_nSize;
	for (n = m_nSize; n > 0; n--)
	{	if (m_pIndice [i] == index)
			return i;
		if (i == 0)
			i = m_nSize;
		--i;
	}
	return -1;
}





/*-----------------------------------------------------------------
Return a reference to position i. Make sure string is large enough
-----------------------------------------------------------------*/
INLINE unsigned char & ZString :: operator [] (int i)
{
	Stress (i, TRUE);
	return m_pBuffer [i];
}

/*-----------------------------------------------------------------
ZString assignment
-----------------------------------------------------------------*/
INLINE ZString &ZString :: operator = (const ZString &str)
{
	if (str.m_pBuffer == NULL)
	{	if (m_pBuffer)
			free (m_pBuffer);
		m_pBuffer = NULL;
		m_nLength = m_nSize = 0;
	}
	else
	{
		m_nLength = str.m_nLength ;
		Stress (m_nLength);
		memcpy (m_pBuffer, str.m_pBuffer, m_nLength + 1);
	}
	return *this;
}

/*-----------------------------------------------------------------
ZString assignment
-----------------------------------------------------------------*/
INLINE ZString &ZString :: operator = (LPCSTR szStr)
{
	if (szStr == NULL)
	{	if (m_pBuffer)
			free (m_pBuffer);
		m_pBuffer = NULL;
		m_nLength = m_nSize = 0;
	}
	else
	{	Stress (m_nLength = strlen (szStr));
		strcpy ((LPSTR) m_pBuffer, szStr);
	}
	return *this;
}

/*-----------------------------------------------------------------
Append one character to a string
-----------------------------------------------------------------*/
INLINE ZString &ZString :: operator += (char c)
{
	Stress (m_nLength + 1);
	m_pBuffer [m_nLength++] = c;
	m_pBuffer [m_nLength] = 0;
	return *this;
}

/*-----------------------------------------------------------------
Concatenate two strings
-----------------------------------------------------------------*/
INLINE ZString &ZString :: operator += (char *szStr)
{
	int n = strlen (szStr);
	Stress (m_nLength + n);
	memcpy (m_pBuffer + m_nLength, szStr, n);
	m_pBuffer [m_nLength += n] = 0;
	return *this;
}

/*-----------------------------------------------------------------
Concatenate two strings
-----------------------------------------------------------------*/
INLINE ZString &ZString :: operator += (const ZString &str)
{
	Stress (m_nLength + str.m_nLength);
	memcpy (m_pBuffer + m_nLength, str.m_pBuffer, str.m_nLength);
	m_pBuffer [m_nLength += str.m_nLength] = 0;
	return *this;
}

/*-----------------------------------------------------------------
Concatenate two strings
-----------------------------------------------------------------*/
INLINE ZString ZString :: operator + (const ZString &str) const
{
	return ZString (*this) += str;
}



/*-----------------------------------------------------------------
Delete a character from position i. Returns the deleted character
-----------------------------------------------------------------*/
INLINE char ZString :: DeleteFrom (int i)
{
	char cResult = m_pBuffer [i];

	memmove (m_pBuffer + i, m_pBuffer + i + 1, m_nLength - i);
	--m_nLength;
	return cResult;
}

/*-----------------------------------------------------------------
Compare two strings
-----------------------------------------------------------------*/
INLINE BOOL ZString :: operator == (const ZString &str) const
{
	return m_nLength == str.m_nLength && !memcmp ((BYTE *) ((*this).m_pBuffer), (BYTE *)(str.m_pBuffer), m_nLength);
}

INLINE BOOL ZString :: operator == (LPCSTR sToCompare) const
{
	return ( m_nLength == (int) strlen(sToCompare) ) && ( !memcmp(m_pBuffer,sToCompare,m_nLength) );
}

INLINE BOOL ZString :: operator != (LPCSTR sToCompare) const
{
	return (m_nLength != (int) strlen(sToCompare)) || (memcmp(m_pBuffer,sToCompare,m_nLength));
}

/*-----------------------------------------------------------------
Converts String
-----------------------------------------------------------------*/
INLINE int ZString :: AsInt() const
{
	return (atoi((LPCSTR) m_pBuffer));
}


/*-----------------------------------------------------------------
Copy an array of bytes to the object.
-----------------------------------------------------------------*/
INLINE ZString & ZString :: Assign (LPCBYTE pData, int nLength)
{
	m_nLength = nLength;
	Stress (m_nLength);
	memcpy (m_pBuffer, pData, m_nLength);
	m_pBuffer [m_nLength] = 0;
	return *this;
}

/*-------------------------------------------------------------
Read a string from a file
-------------------------------------------------------------*/
INLINE int ZString :: FRead (FILE *pFile, int nSize)
{
	Stress (nSize);
	m_nLength = fread (m_pBuffer, 1, nSize, pFile);
	m_pBuffer [m_nLength] = 0;
	return m_nLength;
}

/*-------------------------------------------------------------
Write a string into a file
-------------------------------------------------------------*/
INLINE int ZString :: FWrite (FILE *pFile)
{
	return fwrite (m_pBuffer, 1, m_nLength, pFile);
}

#ifdef WIN32

/*----------------------        Win32       -------------------------------
ZSemaphore constructor. Create unnamed non-inherited semaphore.
If lInitialCount is -1, it is set to lMaxCount.
------------------------        Win32       -----------------------------*/
INLINE ZSemaphore :: ZSemaphore (long lMaxCount, long lInitialCount)
{
	if (lInitialCount < 0)
		lInitialCount = lMaxCount;
	m_hSemaph = CreateSemaphore (NULL, lInitialCount, lMaxCount, NULL);
}

/*-------------------------------------------------------------------------
Try to reserve the semphore. Return immediately if semaphore already
reserved. Return TRUE if succeeded.
-------------------------------------------------------------------------*/
INLINE BOOL ZSemaphore :: Try ()
{
	return WaitForSingleObject (m_hSemaph, 0) != WAIT_TIMEOUT;
}

/*-------------------------------------------------------------------------
Try to reserve the mutex. Return immediately if mutex already
reserved. Return TRUE if succeeded.
-------------------------------------------------------------------------*/
INLINE BOOL ZMutex :: Try ()
{
	return WaitForSingleObject (m_hMutex, 0) != WAIT_TIMEOUT;
}



/*----------------------        UNIX        -----------------------------
ZSemaphore constructor. Create unnamed non-inherited semaphore.
If lInitialCount is -1, it is set to lMaxCount.
------------------------        UNIX        -----------------------------*/

//#elif defined(PTHREADS)
#elif defined(__UNIX)


/*----------------------        SGI        -----------------------------
ZSemaphore, ZMutex
------------------------        SGI        -----------------------------*/

#else //elifdef __UNIX


#endif //WIN32 -> __UNIX



/*-----------------------------------------------------------------
A function that calculates how many bits are required to store a number
which is less than nMax
-----------------------------------------------------------------*/
INLINE int BitFieldWidth (int nMax)
{
	int nResult = 0;
	--nMax;
	while (nMax > 0)
	{	++nResult;
		nMax >>= 1;
	}
	return nResult;
}


/*-------------------------------------------------------------
ZLargeInteger constructor
-------------------------------------------------------------*/
INLINE ZLargeInteger :: ZLargeInteger (DWORD nLow, DWORD nHigh, int nBits) :
	length (nBits / (sizeof (DWORD) * 8))
{
	if (length > 0)
		number.SetAt (0, nLow);
	if (length > 1)
		number.SetAt (1, nHigh);
	for (int i = 2; i < length; i++)
		number.SetAt (i, 0);
}

INLINE ZLargeInteger :: ZLargeInteger (const ZLargeInteger &n)
{
	length = n.length;
	number = n.number;
}

/*-------------------------------------------------------------
Assignment of an int to ZLargeInteger
-------------------------------------------------------------*/
INLINE void ZLargeInteger :: Assign (DWORD nLow)
{
	number.SetAt (0, nLow);
	for (int i = 1; i < length; i++)
		number.SetAt (i, 0);
}

/*-------------------------------------------------------------
Compare ZLargeIntegers
-------------------------------------------------------------*/
INLINE bool ZLargeInteger :: operator == (const ZLargeInteger &n) const
{
	for (int l = max (number.GetLength (), n.number.GetLength ()); --l >= 0;)
	{	DWORD n1 = l >= number.GetLength () ? 0 : number [l];
		DWORD n2 = l >= n.number.GetLength () ? 0 : n.number [l];
		if (n1 != n2)
			return false;
	}
	return true;
}

/*bool ZLargeInteger :: operator == (DWORD n) const
{
	ZLargeInteger l(n);
	return *this == l;
}*/

INLINE bool ZLargeInteger :: operator > (const ZLargeInteger &n) const
{
	for (int l = max (number.GetLength (), n.number.GetLength ()); --l >= 0;)
	{	DWORD n1 = l >= number.GetLength () ? 0 : number [l];
		DWORD n2 = l >= n.number.GetLength () ? 0 : n.number [l];
		if (n1 > n2)
			return true;
		else if (n1 < n2)
			return false;
	}
	return false;
}

INLINE bool ZLargeInteger :: operator < (const ZLargeInteger &n) const
{
	for (int l = max (number.GetLength (), n.number.GetLength ()); --l >= 0;)
	{	DWORD n1 = l >= number.GetLength () ? 0 : number [l];
		DWORD n2 = l >= n.number.GetLength () ? 0 : n.number [l];
		if (n1 < n2)
			return true;
		else if (n1 > n2)
			return false;
	}
	return false;
}


/*-------------------------------------------------------------
Verifies that a certain file exists
-------------------------------------------------------------*/

