//////////////////////////////////////////////////////////////////////////////////////////////////
//    ______ _____ _____ ___  ______   
//   /     /  ___/  ___/   ///_   _/ 
//  /  /--/___  /  ___/   /// / / /   
// /_____/_____/_____/_____/_/_/_/
// 
// -----------------------------------------------------------
// PROJECT:	Decoder MPEG2 audio/video/system full software
// FILE NAME:	M2DecoderPlus.h
// CONTENT:		Header file della Libreria statica M2DecoderPlus.lib
// AUTHORS:		Marco Vecchietti, Alfredo Rinaudo, Paolo Panicciari, Giovanni Caire
// RELEASE:		0.7	
// MODIFIED:	09/03/1999
// 
//////////////////////////////////////////////////////////////////////////////////////////////////


 #ifndef __M2DECODERPLUS_INCLUDED__
	#define __M2DECODERPLUS_INCLUDED__
 #endif


///////////////////////////////////////////////////////////////////////////////////////////////////
// Definizione messaggi di errore metodi della classe M2DecoderPlus
///////////////////////////////////////////////////////////////////////////////////////////////////

 #ifndef __wtypes_h__
	#include <wtypes.h>
 #endif

 #define LIBRARY_VERSION    "M2DecoderPlus  ver: 0.7  Date: 09.03.1999"

 #define MAX_CHAR	256					// Numero di caratteri delle stringhe

 #define DX3_VER	0x00040003			// Versione DirectX 3
 #define DX5_VER	0x00040005			// Versione DirectX 5

 #define NOCHROMAKEY -1					// Chroma key non utilizzato


//------ definizione dei PID come stabilito da MPEG-2 e DVB ---------------------------
#define PAT_PID				0x00
#define CAT_PID				0x01
#define NIT_PID				0x10
#define SDT_PID				0x11		// stesso pid anche per la BAT
#define EIT_PID				0x12
#define RST_PID				0x13
#define TDT_PID				0x14		// stesso pid anche per la TOT	

#define PID_MASK			0x1FFF

//------ definizione degli stream id come stabilito da MPEG-2 -------------------------
#define MPEG1_Video_Stream		0x01
#define MPEG2_Video_Stream		0x02
#define MPEG1_Audio_Stream		0x03
#define MPEG2_Audio_Stream		0x04
#define Private_Data_Stream		0x06

//------ definizione dei flag di indicazione di errore nel payload dati ---------------
 #define TRANSPORT_ERROR_INDICATOR		0x80	// indicates an error occurred at physical payer 
 #define PAYLOAD_UNIT_START				0x40	// a PES or a table section starts here
												// from 0x20 to 0x02 : to be defined
 #define CONTINUITY_COUNTER_BREAK		0x01	// a break in the c.c. occurred
 #define PTS_MARKED_AU					0x02

//------ definizioni degli errori ritornati dai metodi della classe -------------------
 #define _FACM2D  0x636
 #define MAKE_M2DHRESULT(code)  MAKE_HRESULT(1, _FACM2D, code)


 #define M2D_OK						0                                   // Metodo eseguito con successo

 	                                                                    // Condizioni di errore nell'esecuzione dei metodi

 #define M2D_INVALIDPARAMETER		MAKE_M2DHRESULT(10)                 // Valori dei parametri passati ad un metodo non corretti
 #define M2D_INITMUTEX				MAKE_M2DHRESULT(20)                 // Inizializzazione semaforo mutua esclusione struttura M2DECDISPLAYINFO
 #define M2D_PLAYEVENTSETUP			MAKE_M2DHRESULT(30)	                // Inizializzazione eventi di sincronizzazione del M2DecoderPlus
 #define M2D_NODIRECTX				MAKE_M2DHRESULT(40)					// Nessuna versione di directX riscontrata (o versione inferiore alla 3)
 #define M2D_FILENAMEUNDEFINED      MAKE_M2DHRESULT(50)                 // Nome del file contenente lo stream non definito
 #define M2D_WRONGIPADDR	        MAKE_M2DHRESULT(60)                 // Indirizzo IP del server non definito o non corretto
 #define M2D_WRONGIPPORT	        MAKE_M2DHRESULT(70)                 // Indirizzo della porta non definito o non corretto
 #define M2D_VIDEOHANDLEUNDEFINED	MAKE_M2DHRESULT(80)					// Handle della finestra di presentazione non definito
 #define M2D_BRIGHTNESSVALUE		MAKE_M2DHRESULT(90)					// Valore della luminosita' non compreso nell'intervallo didefinizione
 #define M2D_VOLUMEVALUE			MAKE_M2DHRESULT(100)				// Valore del volume non compreso nell'intervallo didefinizione
 #define M2D_BALANCEVALUE			MAKE_M2DHRESULT(110)				// Valore del bilanciamento audio non compreso nell'intervallo di
																		// definizione
 #define M2D_NOMMX					MAKE_M2DHRESULT(120)				// Architettura Intel MMX non presente
 #define M2D_NOTCLOSED              MAKE_M2DHRESULT(130)                // Chiamata al metodo M2DecInit prima di aver terminato il decoder tramite 
                                                                        // il metodo M2DecClose

 #define M2D_DEMUXINIT              MAKE_M2DHRESULT(140)                // Inizializzazione del thread Demux
 #define M2D_NETINIT                MAKE_M2DHRESULT(150)                // Inizializzazione del trhead Net
 #define M2D_NULLCLOSE              MAKE_M2DHRESULT(160)                // Operazione di chiusura non eseguita, il decoder e' stato gia' terminato
 #define M2D_VIDEOINIT			    MAKE_M2DHRESULT(170)                // Inizializzazione del thread Video
 #define M2D_NULLVIDEOSTART         MAKE_M2DHRESULT(180)                // Chiamata al metodo M2DecStartVideoDecoder prima di aver terminato il 
                                                                        // decoder video 
 #define M2D_AUDIOINIT			    MAKE_M2DHRESULT(190)                // Inizializzazione del thread Audio
 #define M2D_NULLAUDIOSTART         MAKE_M2DHRESULT(200)                // Chiamata al metodo M2DecStartAudioDecoder prima di aver terminato il 
                                                                        // decoder audio 
 #define M2D_DATAINIT			    MAKE_M2DHRESULT(210)                // Inizializzazione del thread Dati privati
 #define M2D_NULLDATASTART          MAKE_M2DHRESULT(220)                // Chiamata al metodo M2DecStartDataDecoder prima di aver terminato il 
                                                                        // decoder dati privati
 #define M2D_SECFILTERINIT		    MAKE_M2DHRESULT(230)                // Inizializzazione del thread Section filter
 #define M2D_NULLSECFILTERSTART     MAKE_M2DHRESULT(240)                // Chiamata al metodo M2DecStartDataDecoder prima di aver terminato il 
                                                                        // decoder section filter

 #define M2D_NOSECTIONDATA		    MAKE_M2DHRESULT(250)                // Section Data non disponibile

 #define M2D_NULLVIDEOSTOP		    MAKE_M2DHRESULT(260)                // Terminazione non eseguita del video decoder
 #define M2D_NULLAUDIOSTOP		    MAKE_M2DHRESULT(270)                // Terminazione non eseguita dell'audio decoder
 #define M2D_NULLDATASTOP		    MAKE_M2DHRESULT(280)                // Terminazione non eseguita del data decoder
 #define M2D_NULLSECTIONFILTERSTOP  MAKE_M2DHRESULT(290)                // Terminazione non esguita del section filter
 #define M2D_NULLDEMUXSTOP		    MAKE_M2DHRESULT(300)                // Terminazione non eseguita del demux
 #define M2D_NULLNETSTOP		    MAKE_M2DHRESULT(310)                // Terminazione non eseguita del sottosistema di rete
 #define M2D_NODEMUX		        MAKE_M2DHRESULT(320)                // Thread demux non ancora creato
 #define M2D_BUFFERTOOSHORT		    MAKE_M2DHRESULT(330)                // la Access Unit e' piu' grande del buffer utente
 #define M2D_AUDIOEVENT				MAKE_M2DHRESULT(340)                // Creazione fallita evento inizializzazione audio
 #define M2D_AUDIOALLOCERROR		MAKE_M2DHRESULT(350)                // Fallita allocazione dinamica interfaccia audio
 #define M2D_VIDEOEVENT				MAKE_M2DHRESULT(360)                // Creazione fallita evento inizializzazione video
 #define M2D_VIDEOALLOCERROR		MAKE_M2DHRESULT(370)                // Fallita allocazione dinamica interfaccia video
 #define M2D_FILENOTFOUND			MAKE_M2DHRESULT(380)                // File inesistente
 #define M2D_DATAEVENT	 			MAKE_M2DHRESULT(390)                // Creazione fallita evento inizializzazione dati privati
 #define M2D_SFEVENT	 			MAKE_M2DHRESULT(400)                // Creazione fallita evento inizializzazione section filter
 
                                                                        // Condizioni di errore connesse alla classe DirectDraw

 #define M2D_DDCREATION				MAKE_M2DHRESULT(800)                // Creazione istanza della classe DirectDraw
 #define M2D_DDBLTNOYUVTORGBCONV	MAKE_M2DHRESULT(810)                // La scheda video non supporta la conversione YUV-RGB 
																		// utilizzando una superficie bitmap in hardware
 #define M2D_DDPRIMARYSURFCREATION	MAKE_M2DHRESULT(820)                // Creazione della superficie primaria (buffer video di presentazione)
 #define M2D_DDWORKSURFCREATION		MAKE_M2DHRESULT(830)                // Creazione della superficie secondaria
 #define M2D_DDCLIPPERCREATION		MAKE_M2DHRESULT(840)                // Creazione oggetto di clipping
 #define M2D_DDSURFACEUNDEFINED		MAKE_M2DHRESULT(850)                // Superfici DirectDraw non definite
 #define M2D_DDCLEARWORKSURF		MAKE_M2DHRESULT(860)                // Reset del contenuto della superficie secondaria
 #define M2D_DDOVRNOTMODIFIED		MAKE_M2DHRESULT(870)                // Flag Overlay non modificabile
 #define M2D_DDOVRNOYUVTORGBCONV	MAKE_M2DHRESULT(880)                // La scheda video non supporta la conversione YUV-RGB 
																		// in hardware utilizzando una superficie overlay
 #define M2D_DDNOOVERLAY			MAKE_M2DHRESULT(890)                // La scheda video non supporta superfici di tipo overlay

 #define M2D_DDNOCHROMAKEY			MAKE_M2DHRESULT(891)                // Chroma key non definito

                                                                         // Condizioni di errore connesse alla classe DirectSound

 #define M2D_DSCREATION				MAKE_M2DHRESULT(900)                // Creazione istanza della classe DirectSound
 #define M2D_DSPRIMBUFFERCREATION   MAKE_M2DHRESULT(910)                // Creazione buffer audio primario
 #define M2D_DSSECBUFFERCREATION	MAKE_M2DHRESULT(920)                // Creazione buffer audio secondario
 #define M2D_DSEVENTCREATION		MAKE_M2DHRESULT(930)                // Creazione eventi di notifica posizione di lettura del buffer secondario

 #define M2D_ERROR					MAKE_M2DHRESULT(1000)				// Errore generico



///////////////////////////////////////////////////////////////////////////////////////////////////
// Definizione della struttura M2DECINITINFO contenente le informazioni necessarie 
// per l'inizializzazione dell'oggetto M2DecoderPlus
///////////////////////////////////////////////////////////////////////////////////////////////////

typedef struct m2decinitinfo
{
	int  length;						// Lunghezza in byte della struttura
	char localFileName[MAX_CHAR];		// Nome del file locale contenente i dati MPEG. Se e` "" --> i dati MPEG saranno letti da rete
	char serverIPAddress[MAX_CHAR];		// Indirizzo IP (a.b.c.d) del server da cui ricevere i dati MPEG
	int  serverIPPort;					// Porta sulla quale leggere i dati MPEG
	int  Protocol;						// Indicazione sul tipo di protocollo usato per la trasmissione dei dati MPEG (non usato)
	HWND hWndVideo;						// Handle della finestra dove sarranno visualizzati i dati MPEG decodificati
} M2DECINITINFO, *PM2DECINITINFO;


///////////////////////////////////////////////////////////////////////////////////////////////////
// Definizione della struttura M2DECDISPLAYINFO contenente le informazioni necessarie
// per identificare il rettangolo nel quale fare il display dei dati MPEG decodificati
///////////////////////////////////////////////////////////////////////////////////////////////////

typedef struct m2decdisplayinfo
{
	int          length;				// Lunghezza in byte della struttura
	unsigned int dx;					// Ascissa del vertice superiore sinistro del rettangolo di display
	unsigned int dy;					// Ordinata del vertice superiore sinistro del rettangolo di display
	unsigned int hSize;					// Dimensione orizzontale del rettangolo di display
	unsigned int vSize;					// Dimensione verticale del rettangolo di display
	unsigned int chromaKey;				// Colore di chroma key espresso nel formato R,G,B con dinamica [0, 255]
										// per ciascun canale
} M2DECDISPLAYINFO, *PM2DECDISPLAYINFO;



///////////////////////////////////////////////////////////////////////////////////////////////////
// parametri del metodo M2DecStartSectionFilter
///////////////////////////////////////////////////////////////////////////////////////////////////

typedef struct m2decfilterinfo
{
    int             length;
    int             PID;
    unsigned char   posMask[12];
    unsigned char   posValue[12];
    unsigned char   negMask[12];
    unsigned char   negValue[12];
    int             posLen;
    int             negLen;
} M2DECFILTERINFO, * PM2DECFILTERINFO;



///////////////////////////////////////////////////////////////////////////////////////////////////
// parametri del metodo M2DecGetNewSection
///////////////////////////////////////////////////////////////////////////////////////////////////

typedef struct m2decsectiondata
{
    int             length;
	int				sectionLength;
    unsigned char   data[4096];
} M2DECSECTIONDATA, * PM2DECSECTIONDATA;


///////////////////////////////////////////////////////////////////////////////////////////////////
// parametri del metodo M2DecGetPrivateData
///////////////////////////////////////////////////////////////////////////////////////////////////
typedef struct m2decprivatedata
{
    int             length;
	unsigned int	maxPayloadLen;
	unsigned int	realPayloadLen;
	unsigned char * pBuffer;
	__int64			newPTS;
	char			flags;

} M2DECPRIVATEDATA, * PM2DECPRIVATEDATA;




 
///////////////////////////////////////////////////////////////////////////////////////////////////
// Definizione della classe M2DecoderPlus
///////////////////////////////////////////////////////////////////////////////////////////////////

class M2DecoderPlus
{
																	// METODI
  public:
				 M2DecoderPlus();										// Costruttore della Classe M2DecoderPlus
				~M2DecoderPlus();										// Distruttore della Classe M2DecoderPlus

    void        M2DecGetLibraryVersion(char *);                     // Versione della libreria M2DecoderPlus
 
	HRESULT		M2DecInit(PM2DECINITINFO);							// Inizializza l'oggetto decoder
	HRESULT		M2DecSetDisplay(PM2DECDISPLAYINFO);					// Definisce/modifica il rettangolo di display
	HRESULT		M2DecPlay(int);										// Attiva il processo di decodifica
	HRESULT		M2DecPause();										// Interrompe il processo di decodifica
	HRESULT		M2DecResume();										// Riprende il processo di decodifica 
	HRESULT		M2DecClose();										// Termina il processo di decodifica e resetta il decoder
	HRESULT		M2DecRepaint();										// Forza un repaint della finestra video. Va chiamato quando occorre 
																	// rinfrescare la finestra video (o perche` una zona ritorna visibile 
																	// dopo che un'altra finestra sovrapposta viene mossa, o perche` la 
																	// finestra video stessa viene modificata) in stato di PAUSE.

    HRESULT     M2DecStartAudioDecoder(int);                        // Crea il decoder audio
    HRESULT     M2DecStartVideoDecoder(int);                        // Crea il decoder video
    HRESULT     M2DecStartDataDecoder(int);                         // Crea il decoder dati privati
    HRESULT     M2DecStartSectionFilter(PM2DECFILTERINFO);          // Crea il section filter

    HRESULT     M2DecStopAudioDecoder();                            // Termina il decoder audio
    HRESULT     M2DecStopVideoDecoder();                            // Termina il decoder video
    HRESULT     M2DecStopDataDecoder();                             // Termina il decoder dati privati
    HRESULT     M2DecStopSectionFilter();                           // Termina il section filter

    HRESULT     M2DecGetNewSection(PM2DECSECTIONDATA);  
	HRESULT		M2DecGetPrivateData(PM2DECPRIVATEDATA);

	HRESULT		M2DecSetBrightness(int);							// Imposta il livello di brightness del video decodificato
	HRESULT		M2DecSetVolume(int);								// Imposta il livello del volume dell'audio decodificato
	HRESULT		M2DecSetBalance(int);								// Imposta il bilanciamento stereo dell'audio decodificato

	HRESULT		M2DecSetOverlay(BOOL);								// Imposta il flag di utilizzo dell'overlay

	HRESULT		M2DecGetChromaKey(unsigned int *) ;					// Rilascia il valore di chroma key nel formato RGB
	HRESULT		M2DecGetDisplayArea(RECT *);						// Legge il contenuto della struttura di display e rilascia il rettangolo di 
																	// visualizzazione
    HRESULT		M2DecGetLocalFileName(char *);						// Rilascia il nome dello stream locale
    HRESULT		M2DecGetServerIPAddress(char *);					// Rilascia l'indirizzo IP del server
    HRESULT		M2DecGetServerIPPort(int *);						// Rilascia la porta da cui leggere i dati
	HRESULT		M2DecGetHandleVideoWindow(HWND *);					// Rilascia l'handle della finestra di presnetazione

	HRESULT		M2DecGetBrightness(int *);							// Rilascia il livello di brightness del video decodificato
	HRESULT		M2DecGetVolume(int *);								// Rilascia il livello del volume dell'audio decodificato
	HRESULT		M2DecGetBalance(int *);								// Rilascia il bilanciamento stereo dell'audio decodificato

	BOOL		M2DecGetOverlay();									// Rilascia il valore del flag per l'uso dell'overlay
	HRESULT		M2DecTestMMX(void);									// Verifica la presenza dell'architettura MMX

	void		M2DecGetError(char *, HRESULT, char *);				// Ritorna l'errore associato al relativo valore HRESULT

  protected:

	HRESULT		M2DecInitSyncEvents(void);							// Inizializza eventi di sincronizzazione nella creazione dei thread
	void		M2DecCloseSyncEvents(void);							// Termina gli eventi di sincronizzazione nella creazione dei thread

    HRESULT     M2DecStartDemuxSubSystem();                         // Crea il sottosistema di demultiplexing    
    HRESULT     M2DecStartNetSubSystem();                           // Crea il sottosistema di rete    
    HRESULT     M2DecStopDemuxSubSystem();                          // Termina il sottosistema di demultiplexing
    HRESULT     M2DecStopNetSubSystem();                            // Termina il sottosistema di rete    
																	// VARIABILI MEMBRO
  public:
	unsigned long	  DXversion;									// Versione DirectX installata

    BOOL              bAudioActive;                                 // stato del decoder audio
    BOOL              bVideoActive;                                 // stato del decoder video
    BOOL              bDataActive;                                  // stato del decoder dati privati
    BOOL              bSectionFilterActive;                         // stato del Section Filter
    BOOL              bDemuxActive;                                 // stato del Demux
    BOOL              bNetActive;                                   // stato del sottosistema di rete

  protected:
	M2DECINITINFO	  initInfo;										// Struttura dati di inizializzazione del decoder
    M2DECDISPLAYINFO  displayInfo;									// Struttura dati di definizione impostazioni di visualizzazione

	HANDLE			  hMutexInitInfo;								// handle semaforo di mutua esclusione per lettura/scrittura della struttura
	HANDLE			  hMutexDisplayInfo;							// handle semaforo di mutua esclusione per lettura/scrittura della struttura

	int				  videoCtrlBrigthness;							// Valore del livello di luminiosit video
	int				  audioCtrlVolume;								// Valore del volume
	int				  audioCtrlBalance;								// Valore del bilanciamneto audio L/R

	HANDLE			  hDemuxSubSystem;								// Handle thread sottosistema demultiplexer
	HANDLE			  hAudioSubSystem;								// Handle thread sottosistema deocder audio
	HANDLE			  hVideoSubSystem;								// Handle thread sottosistema decoder video
	HANDLE			  hDataSubSystem;								// Handle thread sottosistema decoder dati privati
	HANDLE			  hNetSubSystem;								// Handle thread sottositema di rete
    HANDLE            hSectionFilter;                               // Handle thread section filter

	BOOL			  bOverlay;										// Flag per l'utillizzo dell'overlay

	DWORD			  demuxSubSystemID;								// ID sottosistema demultiplexer
	DWORD			  audioSubSystemID;								// ID sottosistema decoder audio
	DWORD			  videoSubSystemID;								// ID sottosistema decoder video
	DWORD			  dataSubSystemID;								// ID sottosistema decoder dati privati
	DWORD			  netSubSystemID;								// ID sottosistema di rete
    DWORD             sectionFilterID;                              // ID section filter
};






