// This software module was originally developed 
// by Stefano Battista (STMicroelectronics) 
// in the course
// of development of the MPEG-4 Systems standard (ISO/IEC 14496-1).  This
// software module is an implementation of a part of one or more MPEG-4
// Systems tools as specified by the MPEG-4 Systems standard.  ISO/IEC
// gives users of the MPEG-4 Systems standard free license to this software
// module or modifications thereof for use in hardware or software products
// claiming conformance to the MPEG-4 Systems standard.  Those intending to
// use this software module in hardware or software products are advised
// that its use may infringe existing patents.  The original developer of
// this software module and his/her company, the subsequent editors and
// their companies, and ISO/IEC have no liability for use of this software
// module or modifications thereof in an implementation.  Copyright is not
// released for non MPEG-4 Systems conforming products.  The original
// developer retains full right to use the code for his/her own purpose,
// assign or donate the code to a third party and to inhibit third parties
// from using the code for non MPEG-4 Systems conforming products.  This
// copyright notice must be included in all copies or derivative works.
//
// Copyright 1999.

#include <assert.h>

#include "defs.h"
#include "thread_utils.h"

#include "Mesh2dClass.h"

/*******************************************************************************
the entry point to the decoder DLL
*******************************************************************************/

void * CreateInstance()
{
  TRACE("[Mesh2d] CreateInstance \n");

  return new Mesh2dClass;
}

/*******************************************************************************
Ctor & Dtor
*******************************************************************************/

Mesh2dClass::Mesh2dClass() :
  DecoderImp()
{
  mpoDecoder = new MeshDecoder;
}

Mesh2dClass::~Mesh2dClass()
{
  if (mpoDecoder != NULL)
    delete mpoDecoder;
}

/*-----------------------------------------------------------------
Find a compatible format and initialize the decoder
-----------------------------------------------------------------*/

int Mesh2dClass :: SetFormat (Capabilities *)
{
  return MediaFormat :: MESH;
}

MediaFormat Mesh2dClass :: GetFormat ()
{
  return MediaFormat :: MESH;
}

/*-----------------------------------------------------------------
Calculate the output size in bytes
-----------------------------------------------------------------*/

/* virtual */
bool Mesh2dClass::Init()
{
  TRACE("[Mesh2d::Init] \n");

  return TRUE;
}

/* virtual */
void Mesh2dClass::Run()
{
  TRACE("[Mesh2d::Run] \n");

// WIN-SOL release
#ifdef WIN32
  SetThreadPriority(GetCurrentThread(), THREAD_PRIORITY_ABOVE_NORMAL);
#else
  Set_Priority(THREAD_PRIORITY_ABOVE_NORMAL);
#endif // WIN32

  DecoderImp::Run();
}

BOOL Mesh2dClass :: Decode (
		LPBYTE pucInput,
		int iInputLength,
		LPBYTE pucOutput,
		int & riOutputLength)
{
  TRACE("[Mesh2d::Decode] buffer %08x \n",
    pucInput);

  int iType = 0;

  mpoDecoder->decodeMesh(iType);

  /*** >>>>
  decode(
    (unsigned char*) pucInput, 
    0, 
    iInputLength, 
    &moFopData, 
    &moBopData, 
    sizeof(FOP_DATA) + sizeof(BOP_DATA), 
    &uiPts);
  <<<< ***/

  TRACE("[FBA::Decode] decoded %4d bytes \n", iInputLength);

	// memcpy(pucOutput, &moFopData, GetOptimalOutputSize ());

	riOutputLength = GetOptimalOutputSize (); 
	return TRUE;
}

