/*************************************************************************

This software module was originally developed by 

    Ming-Chieh Lee (Microsoft Corporation)
    Wei-ge Chen (Microsoft Corporation)
    Bruce Lin (Microsoft Corporation)
    Chuang Gu (Microsoft Corporation)

and edited by

    Peter van Beek (University of Rochester)

in the course of development of the MPEG-4 Video (ISO/IEC 14496-2). 
This software module is an implementation of a part of one or more 
MPEG-4 Video (ISO/IEC 14496-2) tools as specified by MPEG-4 Video
(ISO/IEC 14496-2). ISO/IEC gives users of the MPEG-4 Video (ISO/IEC 
14496-2) free license to this software module or modifications thereof 
for use in hardware or software products claiming conformance to the 
MPEG-4 Video (ISO/IEC 14496-2). 
Those intending to use this software module in hardware or software 
products are advised that its use may infringe existing patents. The 
original developer of this software module and his/her company, the 
subsequent editors and their companies, and ISO/IEC have no liability 
for use of this software module or modifications thereof in an 
implementation. 
Copyright is not released for non MPEG-4 Video (ISO/IEC 14496-2) 
conforming products. Microsoft Corporation retains full right to use 
the code for his/her own purpose, assign or donate the code to a third 
party and to inhibit third parties from using the code for non MPEG 
Video (ISO/IEC 14496-2) conforming products. 
This copyright notice must be included in all copies or derivative works. 

Copyright (c) 1996, 1997.

Module Name:
	
	bitstrm.cpp

Abstract:

	Classes for bitstream I/O.

Revision History:

	Peter Van Beek, Tue Aug 26 16:04:07 1997
	- added flush() to CInBitStream

	Peter Van Beek, Thu Oct 16 16:24:59 1997
	- rewritten CInBitStream for working with input BYTE buffer,
	  instead of a file.

	Peter Van Beek, Tue Apr  7 11:44:24 1998
	- corrected COutBitStream::flush() (added some statements) 

	Peter Van Beek, Thu Apr  9 18:42:50 1998
	- deleted some trace methods

*************************************************************************/

#include <iostream.h>
#include <stdio.h>
#include <stdlib.h>

#include "basic.h"
#include "bitstrm.h"

#ifdef __MFC_
#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

#define new DEBUG_NEW				   
#endif // __MFC_

// get the num-bit field of x starting from position p
UInt getbit (UInt data, UInt position, UInt num)
{
	return ((data >> (position + 1 - num)) & ~(~0 << num));
}

// print num bits starting from startPos
Void print_bit (UInt x, UInt num, UInt startPos) 
{
	for (Int k = 0; k <= (Int) (num - startPos); k++) {
		UInt y = getbit (x, num, 1);
		printf ("%u ", y);
		x = x << 1;
	}
	printf ("\n");
}

Char CInBitStream::getBitsC (Int iNOfBits)
{
    assert (iNOfBits <= 8);
    assert (iNOfBits >= 0);
	return (Char)  getBits ((UInt) iNOfBits);
}

UInt CInBitStream::getBits (UInt numBits)
{
	assert (numBits <= 32 && numBits >= 0);
	if (numBits == 0) return 0;
	
	UInt retData = 0;
	if (m_uNumOfBitsInBuffer >= numBits) {	// don't need to read from stream
		retData = getbit (m_chDecBuffer, 7, numBits);
		m_chDecBuffer = m_chDecBuffer << numBits;
		m_uNumOfBitsInBuffer -= numBits;
        m_lCounter += numBits;
	}
	else { // ready, but need to handle the leftover part
		numBits -= m_uNumOfBitsInBuffer; // left-over unhandled bits
		retData = getbit (m_chDecBuffer, 7, m_uNumOfBitsInBuffer) << numBits;
        m_lCounter += m_uNumOfBitsInBuffer;
		if (eof() == EOF) 
			return 0;
#ifdef BUFFER_IO
		m_chDecBuffer = *m_pInStream++;
#else
        m_chDecBuffer = m_pInStream -> get ();
#endif
		m_uNumOfBitsInBuffer = 8;
		retData += getBits (numBits);
	}
	return retData;

}

Void CInBitStream::getBits (Char *pBits, Int lNOfBits)
{
    assert (lNOfBits <= 8);
    assert (lNOfBits >= 0);

    while(lNOfBits>0)
    {
        if(lNOfBits>8)
        {
            *pBits=(Char) getBitsC(8);
            lNOfBits-=8;
            pBits++;
        }
        else
        {
            *pBits=(Char) getBitsC ((UInt) lNOfBits);
            break;
        }
    }
}


#ifdef BUFFER_IO
// attach to the input buffer
Void CInBitStream::attach (BYTE *inStream, UInt size, Int iBitPosition)
{
    assert(iBitPosition<8);
    assert(iBitPosition>=0);
    m_iBitPosition=iBitPosition;
	m_lCounter = 0;
    m_pInStream=inStream;
    m_pInStreamStart=inStream;
    m_iBuffer=0x00ff&(*m_pInStreamStart);
	m_uStreamSize = size;
	m_chDecBuffer = 0; 
	m_uNumOfBitsInBuffer = 0;
}
#else
Void CInBitStream::attach (istream &inStream, Int iBitPosition)
{
    assert(iBitPosition<8);
    assert(iBitPosition>=0);
    m_iBitPosition=iBitPosition;
    m_pInStream=&inStream;
    m_iBuffer=0x00ff&m_pInStream->peek();
}
#endif

//!! Peter Van Beek, Tue Aug 26 16:02:21 1997
// flush the input buffer to the next byte
Void CInBitStream::flush ()
{
	assert (m_uNumOfBitsInBuffer != 8);
	m_lCounter += m_uNumOfBitsInBuffer;
	m_uNumOfBitsInBuffer = 0;
}


COutBitStream::COutBitStream(ostream &outStream, Int iBitPosition, ostream* pstrmTrace) : 
	m_pstrmTrace (pstrmTrace),
	m_chEncBuffer (0),
	m_uEncNumEmptyBits (8)
{
	attach (outStream, iBitPosition);
}
 

Void COutBitStream::putBitsC (Char cBits,Int iNOfBits)
{
	putBits ((Int) cBits, (UInt) iNOfBits); 
}

Void COutBitStream::putBits (Int data, UInt numBits, const Char* rgchSymbolName)
{	
	assert (numBits < 100);				//usually not that large
	if (numBits == 0) return;

	if (m_pstrmTrace != NULL && rgchSymbolName != NULL)	{
		Char* rgchBinaryForm = new Char [numBits + 1];
		assert (rgchBinaryForm != NULL);
		m_pstrmTrace->width (20);
		(*m_pstrmTrace) << rgchSymbolName << ": ";
		Int iMask = 0xFFFFFFFF;
		iMask = iMask << numBits;
		iMask = ~iMask;
		Int iCleanData = data & iMask;
		//_itoa (iCleanData, rgchBinaryForm, 2);	// win32 only
		Int iBit;
		iMask = 0x00000001;
		for (iBit = (Int) numBits - 1; iBit >= 0; iBit--)	{
			rgchBinaryForm [iBit] = ((iCleanData & iMask) == 0) ? '0' : '1';
			iMask = iMask << 1;
		}
		rgchBinaryForm [numBits] = '\0';
		m_pstrmTrace->width (numBits);
		m_pstrmTrace->fill ('0');
		(*m_pstrmTrace) << rgchBinaryForm;
		m_pstrmTrace->fill (' ');
		(*m_pstrmTrace) << " @" << m_lCounter << '\n';
		m_pstrmTrace->flush ();
		delete rgchBinaryForm;
	}

	if (m_uEncNumEmptyBits > numBits) {	// not ready to put the data to buffer since it's not full
		m_uEncNumEmptyBits -= numBits;
		Char mskData = (0xFF >> (8 - numBits)) & data; 
		mskData = mskData << m_uEncNumEmptyBits;
		m_chEncBuffer = m_chEncBuffer ^ mskData;
        m_lCounter += numBits;
	}
	else if (m_uEncNumEmptyBits == numBits) { // ready
		Char mskData = (0xFF >> (8 - numBits)) & data; 
		m_chEncBuffer = m_chEncBuffer ^ mskData;
	    m_pOutStream -> put (m_chEncBuffer);
		m_chEncBuffer = 0;
		m_uEncNumEmptyBits = 8;
        m_lCounter += numBits;
	}
	else { // ready, but need to handle the leftover part
		UInt temp = getbit (data, numBits - 1, m_uEncNumEmptyBits);
		numBits -= m_uEncNumEmptyBits; // length of unhandled part
		m_chEncBuffer = m_chEncBuffer ^ temp;
        m_lCounter += m_uEncNumEmptyBits;
	    m_pOutStream -> put (m_chEncBuffer);
		m_chEncBuffer = 0;
		m_uEncNumEmptyBits = 8;
		data = data ^ (temp << numBits);
		putBits (data, numBits);
	}
}

#ifdef BUFFER_IO
Int CInBitStream::eof () const
{
	if (m_pInStream >= m_pInStreamStart+m_uStreamSize)
		return EOF;
	else
		return 0;
}
#else
Int CInBitStream::eof () const
{
	return m_pInStream->peek ();
}
#endif

#ifdef BUFFER_IO
Int CInBitStream::peekBits (const UInt numBits) const
{
	assert (numBits <= 32);
	UInt iBitsToRet;
	Int nBitsToPeek = numBits - m_uNumOfBitsInBuffer;
	if (nBitsToPeek <= 0)							// already enough bits 
		iBitsToRet = getbit (m_chDecBuffer, 7, numBits);
	else {
		BYTE *pStream = m_pInStream;				// auxiliary pointer
		iBitsToRet = getbit (m_chDecBuffer, 7, m_uNumOfBitsInBuffer);
		for (; nBitsToPeek > 0; nBitsToPeek -= 8) {
			if (eof() == EOF) 						// at end of buffer?
				return EOF;
			int chNext = *pStream++;	   			// get the next char
			Int iNext = chNext & 0x000000FF;		// clean the upper bits 
			if (nBitsToPeek < 8) {					// fractional char
				iNext = iNext >> (8 - nBitsToPeek);
				iBitsToRet = iNext | (iBitsToRet << nBitsToPeek);
			}
			else {
				iBitsToRet = iBitsToRet << 8;		// full char
				iBitsToRet |= iNext;
			}
		}
	}
	return iBitsToRet;
}
#else
Int CInBitStream::peekBits (const UInt numBits) const
{
	assert (numBits <= 32);
	UInt iBitsToRet;
	Int nBitsToPeek = numBits - m_uNumOfBitsInBuffer;
	streamoff strmoff = 0;
	if (nBitsToPeek <= 0)
		iBitsToRet = getbit (m_chDecBuffer, 7, numBits);
	else	{
		iBitsToRet = getbit (m_chDecBuffer, 7, m_uNumOfBitsInBuffer);
		for (; nBitsToPeek > 0; nBitsToPeek -= 8) {
			int chNext = m_pInStream->peek();				//peek the next ch
			m_pInStream->seekg ((streamoff) 1, ios::cur);	//adv. strm ptr
			strmoff++;
			if (chNext == EOF)
				return EOF;
			Int iNext = chNext & 0x000000FF;			//clean the upper bits 
			if (nBitsToPeek < 8){					   	//fractional char
				iNext = iNext >> (8 - nBitsToPeek);
				iBitsToRet = iNext | (iBitsToRet << nBitsToPeek);
			}
			else	{
				iBitsToRet = iBitsToRet << 8;			//full char
				iBitsToRet |= iNext;
			}
		}
	}			
	m_pInStream->seekg (-1 * strmoff, ios::cur);		//reset the strm ptr pos
	return iBitsToRet;		
}
#endif


Void COutBitStream::putBits (Char *pBits, Int lNOfBits)
{
    assert(lNOfBits>=0);
    while(lNOfBits>0)
    {
        if(lNOfBits>8)
        {
            putBitsC((Int)(*pBits),8);
            lNOfBits-=8;
            pBits++;
        }
        else
        {
            putBitsC((Int)(*pBits),(UInt)lNOfBits);
            break;
        }
    }
}

// flush current byte to output stream (stuff with zeros) and update state
Void COutBitStream::flush()
{
	if (m_uEncNumEmptyBits != 8)	{
		m_pOutStream -> put (m_chEncBuffer);
		// Peter Van Beek, Tue Apr  7 11:38:46 1998
        m_lCounter += m_uEncNumEmptyBits;
		m_chEncBuffer = 0;
		m_uEncNumEmptyBits = 8;
	}
}

Void COutBitStream::attach (ostream &outStream, Int iBitPosition)
{
    assert (iBitPosition < 8);
    assert (iBitPosition >= 0);
    m_iBitPosition = iBitPosition;
    m_pOutStream = &outStream;
}

Void COutBitStream::trace (Int iValue, const Char* rgchSymbolName)
{
	if (m_pstrmTrace != NULL)	{
		m_pstrmTrace->width (20);
		(*m_pstrmTrace) << rgchSymbolName << "= ";
		(*m_pstrmTrace) << iValue << "\n";
		m_pstrmTrace->flush ();
	}
}

Void COutBitStream::trace (UInt uiValue, const Char* rgchSymbolName)
{
	if (m_pstrmTrace != NULL)	{
		m_pstrmTrace->width (20);
		(*m_pstrmTrace) << rgchSymbolName << "= ";
		(*m_pstrmTrace) << uiValue << "\n";
		m_pstrmTrace->flush ();
	}
}

Void COutBitStream::trace (Float fltValue, const Char* rgchSymbolName)
{
	if (m_pstrmTrace != NULL)	{
		m_pstrmTrace->width (20);
		(*m_pstrmTrace) << rgchSymbolName << "= ";
		(*m_pstrmTrace) << fltValue << "\n";
		m_pstrmTrace->flush ();
	}
}

Void COutBitStream::trace (Double dblValue, const Char* rgchSymbolName)
{
	if (m_pstrmTrace != NULL)	{
		m_pstrmTrace->width (20);
		(*m_pstrmTrace) << rgchSymbolName << "= ";
		(*m_pstrmTrace) << dblValue << "\n";
		m_pstrmTrace->flush ();
	}
}

