/*************************************************************************

This software module was originally developed by 

    Ming-Chieh Lee (Microsoft Corporation)
    Wei-ge Chen (Microsoft Corporation)
    Bruce Lin (Microsoft Corporation)
    Chuang Gu (Microsoft Corporation)

and edited by

    Peter van Beek (University of Rochester)

in the course of development of the MPEG-4 Video (ISO/IEC 14496-2). 
This software module is an implementation of a part of one or more 
MPEG-4 Video (ISO/IEC 14496-2) tools as specified by MPEG-4 Video
(ISO/IEC 14496-2). ISO/IEC gives users of the MPEG-4 Video (ISO/IEC 
14496-2) free license to this software module or modifications thereof 
for use in hardware or software products claiming conformance to the 
MPEG-4 Video (ISO/IEC 14496-2). 
Those intending to use this software module in hardware or software 
products are advised that its use may infringe existing patents. The 
original developer of this software module and his/her company, the 
subsequent editors and their companies, and ISO/IEC have no liability 
for use of this software module or modifications thereof in an 
implementation. 
Copyright is not released for non MPEG-4 Video (ISO/IEC 14496-2) 
conforming products. Microsoft Corporation retains full right to use 
the code for his/her own purpose, assign or donate the code to a third 
party and to inhibit third parties from using the code for non MPEG 
Video (ISO/IEC 14496-2) conforming products. 
This copyright notice must be included in all copies or derivative works. 

Copyright (c) 1996, 1997.

Module Name:
	
	bitstrm.hpp

Abstract:

	Classes for bitstream I/O.

Revision History:

	Peter van Beek,
	- renamed bitstrm.h

	Peter Van Beek, Tue Aug 26 16:04:07 1997
	- added flush() to CInBitStream

	Peter Van Beek, Thu Oct 16 16:24:59 1997
	- rewritten CInBitStream for working with input BYTE buffer,
	  instead of a file.

	Peter Van Beek, Thu Mar 19 16:19:33 1998
	- added getStreamPtr() to CInBitStream

	Peter Van Beek, Thu Apr  9 18:42:50 1998
	- deleted some trace methods from COutBitStream

	Peter Van Beek, Wed May  6 12:53:46 1998
	- added getNumBitsInBuf() to CInBitStream

*************************************************************************/

#ifndef _BITSTREAM_HPP_
#define _BITSTREAM_HPP_

class istream;
class ostream;

// define BUFFER_IO to base the CInBitStream on a pointer to a BYTE 
// array (buffer I/O); instead of a pointer to an istream (file I/O)
#define BUFFER_IO

#ifdef BUFFER_IO
#define BYTE 	unsigned char
#endif


Class CIOBitStream
{
public:
	// Operations

    //get current bit position within current byte
	Int getPosition () { return m_iBitPosition; }
    //get counter of processed bits
	Int getCounter () { return m_lCounter; }
    //set counter of processed bits
	Void getCounter (Int lCounter) { m_lCounter = lCounter; }

protected:
    Int m_lCounter;
    Int m_iBitPosition;
    UInt m_iBuffer;

	// Constructor
    CIOBitStream () {m_iBitPosition=0; m_iBuffer=0; m_lCounter=0;}
};


Class CInBitStream : public CIOBitStream
{
public:
	// Constructors

    // create unattached bitstream    
	CInBitStream () {m_chDecBuffer = 0; m_uNumOfBitsInBuffer = 0; m_uStreamSize = 0;}
#ifdef BUFFER_IO
	// create bitstream and attach to inStream
	CInBitStream (BYTE *inStream, UInt size, Int iBitPosition = 0)
	{attach (inStream, size, iBitPosition); m_chDecBuffer = 0; m_uNumOfBitsInBuffer = 0;}
#else
	// create bitstream and attach to inStream
	CInBitStream (istream &inStream, Int iBitPosition = 0)
	{attach (inStream, iBitPosition); m_chDecBuffer = 0; m_uNumOfBitsInBuffer = 0;}
#endif
	// Attributes

	// stream is EOF
	Int eof () const;

	// Operations

    // return bits from the stream
	UInt getBits (UInt numBits);
    // peek bits from the stream
	Int peekBits (const UInt numBits) const;
    // get bits and write to the buffer
	Void getBits (Char *pBits, Int lNOfBits);
#ifdef BUFFER_IO
    // attach to the inStream
	Void attach (BYTE *inStream, UInt size, Int iBitPosition = 0);
	// return ptr to stream
	BYTE *getStreamPtr()    { return m_pInStream; } 
	UInt getNumBitsInBuf()  { return m_uNumOfBitsInBuffer; }
#else
    // attach to the inStream
	Void attach (istream &inStream, Int iBitPosition = 0);
#endif
	Void flush ();

private:
#ifdef BUFFER_IO
	BYTE *m_pInStreamStart;			// ptr to start of input buffer
	BYTE *m_pInStream;				// ptr to current position in input buffer
#else
    istream *m_pInStream;
#endif
    Char getBitsC (Int iNOfBits);
	UInt m_uNumOfBitsInBuffer;
	Char m_chDecBuffer;
	UInt m_uStreamSize;				// size of input buffer in bytes
};


Class COutBitStream : public CIOBitStream
{
public:
	// Constructors

    // create unattached bitstream
	COutBitStream () {m_chEncBuffer = 0; m_uEncNumEmptyBits = 8;}
    // create bitstream and attach to outStream
	COutBitStream (ostream &outStream, Int iBitPosition = 0, ostream* pstrmTrace = NULL); 

	// Operations

    // put bits into the stream
	Void putBits (Int data, UInt numBits, const Char* rgchSymbolName = NULL);
    // put bits from the array into the stream
	Void putBits (Char *pBits, Int lNOfBits);
    // attach to the outStream
	Void attach (ostream &OutStream, Int iBitPosition=0);
    // flush remaing bits in the COutBitStream into the attached ostream
	Void flush ();
	// dump a Int variable into the trace stream
	Void trace (Int iValue, const Char* rgchSymbolName);
	// dump a UInt variable into the trace stream
	Void trace (UInt iValue, const Char* rgchSymbolName);
	// dump a Float variable into the trace stream
	Void trace (Float fltValue, const Char* rgchSymbolName);
	// dump a Double variable into the trace stream
	Void trace (Double dblValue, const Char* rgchSymbolName);

//	Void setFile (FILE* pfEnc) {m_pfEnc = pfEnc;}

private:
    ostream *m_pOutStream;
    ostream *m_pstrmTrace;		// stream for trace file
    Void putBitsC (Char cBits, Int iNOfBits);
	Char m_chEncBuffer;
	UInt m_uEncNumEmptyBits;
//	FILE* m_pfEnc;
};

#endif

