/**********************************************************************/
/*                                                                    */
/* This software module was originally developed by                   */
/*                                                                    */
/* A. Tanju Erdem (Eastman Kodak Company)                             */
/* Candemir Toklu (University of Rochester)                           */
/*                                                                    */
/* in the course of development of MPEG-4 Visual (ISO/IEC 14496-2).   */
/* This software module is an implementation of a part of one or      */
/* more MPEG-4 Visual (ISO/IEC 14496-2) tools as specified by the     */
/* MPEG-4 Visual (ISO/IEC 14496-2). ISO/IEC gives users of MPEG-4     */
/* Visual (ISO/IEC 14496-2) free license to this software module or   */
/* modifications thereof for use in hardware or software products     */
/* claiming conformance to MPEG-4 Visual (ISO/IEC 14496-2). Those     */
/* intending to use this software module in hardware or software      */
/* products are advised that its use may infringe existing patents.   */
/* The original developer of this software module and his/her         */
/* company, the subsequent editors and their companies, and ISO/IEC   */
/* have no liability for use of this software module or modifications */ 
/* thereof in an implementation. Copyright is not released for non    */
/* MPEG-4 Visual (ISO/IEC 14496-2) conforming products.               */
/* Eastman Kodak Company and the University of Rochester retain full  */
/* right to use the code for their own purposes, assign or donate the */
/* code to a third party and to inhibit third parties from using the  */
/* code for non MPEG-4 Visual (ISO/IEC 14496-2) conforming products.  */
/* This copyright notice must be included in all copies or derivative */
/* works.                                                             */
/*                                                                    */
/* Copyright (c) 1994-1997                                            */
/*                                                                    */
/**********************************************************************/

/*+*********************************************************************
   NAME         linkClass.cpp
   AUTHOR       A. Tanju Erdem - Eastman Kodak Company and Candemir Toklu - 
                University of Rochester.
   DATE         September 1994
   UPDATE       14 August 1996 by Candemir Toklu
   DESCRIPTION  Member functions for Point2dLink class are defined in this
                file.

   UPDATE		Peter Van Beek, Tue Feb 17 12:44:05 1998
   				Added 'motion' and 'mark' fields.
				Added 'setPoint()' method.

   SYNOPSIS     Include files : 
                                
                #include "linkClass.h"
                #include "general.h"
                               
*********************************************************************-*/
#include "linkClass.h"
#include "general.h"

//------------------------------------------------------------------------

Pt2dLink::Pt2dLink()  // A link
{
  here = Point2d();
  base = 0.0;  
  mult = 1.0;  
  motion = Point2d();
  mark = 0;
  next = NULL;
  prev = NULL;
  sizeLink = 0;
  owner = NULL;
}

Pt2dLink::Pt2dLink(const Point2d& p)  // A link
{
  here = p;
  base = 0.0;  
  mult = 1.0;  
  motion = Point2d();
  mark = 0;
  next = NULL;
  prev = NULL;
  sizeLink = 0;
  owner = NULL;
}

Pt2dLink::~Pt2dLink()
{
  if( sizeLink)
  {
    delete[] owner;  // De-allocate the pointers to polygon references
  }
}

Poly2dRef* Pt2dLink::address( int indx)
{
  if( sizeLink == 0) {
    cout << "\n  Error: Cannot index to a link that has no owners!" << endl;
    exit( 1);
  }
  else if( indx < 0 || indx >= sizeLink) {
    cout << "\n  Error: Index value is out of range!" << endl;
    exit( 1);
  }
  else {
    Poly2dRef* tmpRef;
    tmpRef = owner[indx];
    return tmpRef;
  }
  return(NULL);
}
