/**********************************************************************/
/*                                                                    */
/* This software module was originally developed by                   */
/*                                                                    */
/* A. Tanju Erdem (Eastman Kodak Company)                             */
/* Candemir Toklu (University of Rochester)                           */
/*                                                                    */
/* in the course of development of MPEG-4 Visual (ISO/IEC 14496-2).   */
/* This software module is an implementation of a part of one or      */
/* more MPEG-4 Visual (ISO/IEC 14496-2) tools as specified by the     */
/* MPEG-4 Visual (ISO/IEC 14496-2). ISO/IEC gives users of MPEG-4     */
/* Visual (ISO/IEC 14496-2) free license to this software module or   */
/* modifications thereof for use in hardware or software products     */
/* claiming conformance to MPEG-4 Visual (ISO/IEC 14496-2). Those     */
/* intending to use this software module in hardware or software      */
/* products are advised that its use may infringe existing patents.   */
/* The original developer of this software module and his/her         */
/* company, the subsequent editors and their companies, and ISO/IEC   */
/* have no liability for use of this software module or modifications */ 
/* thereof in an implementation. Copyright is not released for non    */
/* MPEG-4 Visual (ISO/IEC 14496-2) conforming products.               */
/* Eastman Kodak Company and the University of Rochester retain full  */
/* right to use the code for their own purposes, assign or donate the */
/* code to a third party and to inhibit third parties from using the  */
/* code for non MPEG-4 Visual (ISO/IEC 14496-2) conforming products.  */
/* This copyright notice must be included in all copies or derivative */
/* works.                                                             */
/*                                                                    */
/* Copyright (c) 1994-1997                                            */
/*                                                                    */
/**********************************************************************/

/*+*********************************************************************
   NAME         linkClass.h
   AUTHOR       A. Tanju Erdem - Eastman Kodak Company and Candemir Toklu - 
                University of Rochester.
   DATE         September 1994
   UPDATE       14 August 1996 by Candemir Toklu
   DESCRIPTION  The Point2dLink class defines a Point2d link. A link is a 
                point with pointers to the previous and next neighboring 
				points. Only class Pt2dList functions can create and 
				destroy Pt2dLink objects. Otherwise one should only declare 
				pointers to Pt2dLink objects. Its private members are links
				to the next and previous points in the list, the location
				of the point, the number of the polygons that share the
				point as a corner and pointers to these polygons. Last two
				private members are meaningful when the Point2dLinks are
				used to create a list of the nodes of a mesh structure.
   UPDATE		Peter Van Beek, Tue Feb 17 12:44:05 1998
   				Added 'motion' and 'mark' fields.
				Added 'setPoint()' method.

   SYNOPSIS     Include files : 
                                
                #include <iostream.h>
                #include <fstream.h>
                #include <math.h>
                #include <time.h>
                #include "pointClass.h"

*********************************************************************-*/
#include <iostream.h>
#include <fstream.h>
#include <math.h>
#include <time.h>
#include "pointClass.h"

class Poly2dRef;

#ifndef POINT_LINK_CLASS
#define POINT_LINK_CLASS

//------------------------------------------------------------------------

// A link is a point with pointers to the previous and next neighboring points.
// Only class Pt2dList functions can create and destroy Pt2dLink objects.
// Otherwise one should only declare pointers to Pt2dLink objects.
class Pt2dLink {
    friend class Pt2dList;
    friend class Poly2dTess;
  public:
    Pt2dLink();
    Pt2dLink( const Point2d&);
    ~Pt2dLink();
    inline Point2d getPoint();
	inline void setPoint(const Point2d&);
    inline int getSize();
    Poly2dRef* address( int);
    double base, mult; // C.T.
	Point2d motion;	   // Motion vector of the point, PvB
	char mark;		   // Marker of this point, PvB
  private:
    Point2d here;  // Coordinates of the point
    Pt2dLink *next, *prev;  // Pointers to the next and previous point-links in a list
// The following two data members are defined to be used in a tesselation;
// they are not to be used in a stand-alone point-list. 
// While a tesselation is responsible for allocating the memory that "owner" points at,
// the point-list is responsible for de-allocating it. 
    int sizeLink;  // Number of owners (can be 0)
    Poly2dRef **owner;  // Pointer to an array of sizeLink pointers to owner polygons
};

Point2d Pt2dLink::getPoint()
{
   return here;
}

void Pt2dLink::setPoint(const Point2d& p)
{
	here = p;
}

int Pt2dLink::getSize()
{
   return sizeLink;
}

#endif
