/**********************************************************************/
/*                                                                    */
/* This software module was originally developed by                   */
/*                                                                    */
/* A. Tanju Erdem (Eastman Kodak Company)                             */
/* Candemir Toklu (University of Rochester)                           */
/*                                                                    */
/* in the course of development of MPEG-4 Visual (ISO/IEC 14496-2).   */
/* This software module is an implementation of a part of one or      */
/* more MPEG-4 Visual (ISO/IEC 14496-2) tools as specified by the     */
/* MPEG-4 Visual (ISO/IEC 14496-2). ISO/IEC gives users of MPEG-4     */
/* Visual (ISO/IEC 14496-2) free license to this software module or   */
/* modifications thereof for use in hardware or software products     */
/* claiming conformance to MPEG-4 Visual (ISO/IEC 14496-2). Those     */
/* intending to use this software module in hardware or software      */
/* products are advised that its use may infringe existing patents.   */
/* The original developer of this software module and his/her         */
/* company, the subsequent editors and their companies, and ISO/IEC   */
/* have no liability for use of this software module or modifications */ 
/* thereof in an implementation. Copyright is not released for non    */
/* MPEG-4 Visual (ISO/IEC 14496-2) conforming products.               */
/* Eastman Kodak Company and the University of Rochester retain full  */
/* right to use the code for their own purposes, assign or donate the */
/* code to a third party and to inhibit third parties from using the  */
/* code for non MPEG-4 Visual (ISO/IEC 14496-2) conforming products.  */
/* This copyright notice must be included in all copies or derivative */
/* works.                                                             */
/*                                                                    */
/* Copyright (c) 1994-1997                                            */
/*                                                                    */
/**********************************************************************/

/*+*********************************************************************
   NAME         listClass.cpp
   AUTHOR       A. Tanju Erdem - Eastman Kodak Company and Candemir Toklu - 
                University of Rochester.
   DATE         September 1994
   UPDATE       14 August 1996 by Candemir Toklu
   DESCRIPTION  Member functions for Pt2dList class are defined in this
                file.

   SYNOPSIS     Include files : 
                                
                #include "listClass.h"
                #include "general.h"
                               
*********************************************************************-*/
#include "listClass.h"
#include "general.h"

//------------------------------------------------------------------------
//
//        Last point   First point   Second point
//
// ... <---> (prev) <---> (list) <---> (next) <---> ...
//                    ^
//                    |
//
//            append/insert here
//


Pt2dList::Pt2dList()  // Empty list
{
  list = NULL;
  size = 0;
  plast = NULL;
  ilast = 0;
}

Pt2dList::Pt2dList(const Point2d& p)  // A list with a single link
{
  list = new Pt2dLink( p);  // Start the list with point p
  list->next = list->prev = list;  // List is closed
  size = 1;
  plast = list;
  ilast = 0;
}

Pt2dList::Pt2dList(const Pt2dList& from)  // Initialization with another list
{
  if( from.size == 0) {
    list = NULL;
    size = 0;
    plast = NULL;
    ilast = 0;
  }
  else {
    Pt2dLink *tmpLink = from.list;
    list = new Pt2dLink( tmpLink->here);  // Copy the first point in the list
    list->next = list->prev = list;  // Close the list
    Pt2dLink *newLink;
    for( int i = 1; i < from.size; i++) {
      tmpLink = tmpLink->next;
      newLink = new Pt2dLink( tmpLink->here);
      newLink->prev = list->prev;
      newLink->next = list;
      list->prev->next = newLink;
      list->prev = newLink;
    }
    size = from.size;
    plast = list;
    ilast = 0;
  }
}

Pt2dList::~Pt2dList()
{
  print();
  empty();
}

Pt2dList& Pt2dList::empty()
{
  Pt2dLink *tmpLink;
  for( int i = 0; i < size; i++) {
    tmpLink = list->prev;
    tmpLink->prev->next = tmpLink->next;
    tmpLink->next->prev = tmpLink->prev;
    delete tmpLink;  // Remove the current last point-link from the list
  }
  list = NULL;
  plast = NULL;
  size = 0;
  ilast = 0;
  return *this;
}

Pt2dList& Pt2dList::operator=(const Pt2dList& from)  // Setting equal to a list
{
  if( this == &from) return *this;  // Cannot copy a list onto itself

  Pt2dLink *tmpLink;
  int i;

// First, destruct the old list
  for( i = 0; i < size; i++) {
    tmpLink = list->prev;
    tmpLink->prev->next = tmpLink->next;
    tmpLink->next->prev = tmpLink->prev;
    delete tmpLink;  // Remove the current last point-link from the list
  }

// Then, construct a new list that is identical to the given list
  if( from.size == 0) {
    list = NULL;
    size = 0;
    plast = NULL;
    ilast = 0;
  }
  else {
    tmpLink = from.list;
    list = new Pt2dLink( tmpLink->here);  // Copy the first point in the list
    list->next = list->prev = list;  // Close the list
    Pt2dLink *newLink;
    for( i = 1; i < from.size; i++) {
      tmpLink = tmpLink->next;
      newLink = new Pt2dLink( tmpLink->here);
      newLink->prev = list->prev;
      newLink->next = list;
      list->prev->next = newLink;
      list->prev = newLink;
    }
    size = from.size;
    plast = list;
    ilast = 0;
  }
  return *this;
}

Pt2dList& Pt2dList::insert( const Point2d& p)
{
  if( size == 0) {
    list = new Pt2dLink( p);  // Start the list with point p
    list->next = list->prev = list;  // List is closed
    size = 1;
  }
  else {
    Pt2dLink *tmpLink = list;
    do {
      if( tmpLink->here == p) {
        cout << "\n  Warning: Cannot insert " << p << " to the list. "
             << "The point is already in the list." << endl;
        return *this;
      }
      tmpLink = tmpLink->next;
    } while( tmpLink != list);

// Add point p to the beginning of the list
    tmpLink = new Pt2dLink( p); 
    tmpLink->next = list;
    tmpLink->prev = list->prev;
    list->prev->next = tmpLink;
    list->prev = tmpLink;
    list = tmpLink;
    size++;
  }
  plast = list;
  ilast = 0;
  return *this;  
}

Pt2dList& Pt2dList::append( Point2d p)
{
  if( size == 0) {
    list = new Pt2dLink( p);  // Start the list with point p
    list->next = list->prev = list;  // List is closed
    size = 1;
  }
  else {
    Pt2dLink *tmpLink = plast;
    int i = ilast;
    do {
      if( tmpLink->here == p)
      {  // Cannot add to the list; the point is already in the list
        plast = tmpLink;
        ilast = (i+size)%size;
        return *this;
      }
      tmpLink = tmpLink->prev;
      i--;
    } while( tmpLink != plast);

// Add point p to the end of the list
    tmpLink = new Pt2dLink( p); 
    tmpLink->prev = list->prev;
    tmpLink->next = list;
    list->prev->next = tmpLink;
    list->prev = tmpLink;
    size++;
  }
  plast = list->prev;
  ilast = size-1;
  return *this;  
}

Pt2dLink* Pt2dList::add( Point2d p)
{
  if( size == 0) {
    list = new Pt2dLink( p);  // Start the list with point p
    list->next = list->prev = list;  // List is closed
    size = 1;
  }
  else {
    Pt2dLink *tmpLink = plast;
    int i = ilast;
    do {
      if( tmpLink->here == p)
      {  // Cannot add to the list; the point is already in the list
        plast = tmpLink;
        ilast = (i+size)%size;
        return tmpLink;
      }
      tmpLink = tmpLink->prev;
      i--;
    } while( tmpLink != plast);

// Add point p to the end of the list
    tmpLink = new Pt2dLink( p); 
    tmpLink->prev = list->prev;
    tmpLink->next = list;
    list->prev->next = tmpLink;
    list->prev = tmpLink;
    size++;
  }
  plast = list->prev;
  ilast = size-1;
  return list->prev;  // Return the address of the last added point-link
}

int Pt2dList::grow( Point2d p)
{
  if( size == 0) {
    list = new Pt2dLink( p);  // Start the list with point p
    list->next = list->prev = list;  // List is closed
    size = 1;
  }
  else {
    Pt2dLink *tmpLink = plast;
    int i = ilast;
    do {
      if( tmpLink->here == p)
      {
        plast = tmpLink;
        ilast = (i+size)%size;
        return 0;  // Return 0 indicating that the point is already in the list
      }
      tmpLink = tmpLink->prev;
      i--;
    } while( tmpLink != plast);

// Add point p to the end of the list
    tmpLink = new Pt2dLink( p); 
    tmpLink->prev = list->prev;
    tmpLink->next = list;
    list->prev->next = tmpLink;
    list->prev = tmpLink;
    size++;
  }
  plast = list->prev;
  ilast = size-1;
  return 1;  // Return 1 indicating that the point is new to the list
}

Pt2dList& Pt2dList::remove( const Point2d& p)
{
  if( size == 0) {  // List is empty
        cout << "\n  Warning: Cannot remove " << p << " from the list."
             << "\n           The list is empty." << endl;
  }
  else {
    int indx = 0;

    Pt2dLink *tmpLink = list;
    while( tmpLink->here != p && indx < size) {
      indx++;
      tmpLink = tmpLink->next;
    }
    if( indx == size) {       // Point p is not found in the list
        cout << "\n  Warning: Cannot remove " << p << " from the list."
             << "\n           The point does not belong to the list." << endl;
    }
    else {
      if( size == 1) {
        list = NULL;
        plast = NULL;
        ilast = 0;
      }
      else {
        tmpLink->prev->next = tmpLink->next;
        tmpLink->next->prev = tmpLink->prev;
        if( indx == 0) list = tmpLink->next;
        plast = tmpLink->next;
        ilast = indx;
      }
      delete tmpLink;  // Remove the point link from the list
      size--;     
    }
  }
  return *this;  
}

Point2d& Pt2dList::operator[]( int indx)
{
  if( size == 0) {
    cout << "\n  Error: Cannot index to an empty list!" << endl;
    exit( 1);
  }
  else if( indx < 0 || indx >= size) {
    cout << "\n  Error: Index value is out of range!" << endl;
    exit( 1);
  }

  {
    int i;
    Pt2dLink *tmpLink = plast;
    if( indx > ilast) for( i = 0; i < indx-ilast; i++) tmpLink = tmpLink->next;
    else              for( i = 0; i < ilast-indx; i++) tmpLink = tmpLink->prev;
    plast = tmpLink;
    ilast = indx;
    return( tmpLink->here);
  }
}

int Pt2dList::where( Pt2dLink* pLink) const
{
  if( size == 0) {
    return -1;  // Point p is not a member of the list (the list is empty)
  }
  else {
    int indx = 0;

    Pt2dLink *tmpLink = list;
    while( tmpLink != pLink && indx < size) {
      indx++;
      tmpLink = tmpLink->next;
    }
    if( indx == size) {       // Point p is not found in the list
      return -1;
    }
    else {
      return indx;   // Return the location of the point in the list
    }
  }
}

int Pt2dList::where( Point2d& p) const
{
  if( size == 0) {
    return -1;  // Point p is not a member of the list (the list is empty)
  }
  else {
    int indx = 0;

    Pt2dLink *tmpLink = list;
    while( tmpLink->here != p && indx < size) {
      indx++;
      tmpLink = tmpLink->next;
    }
    if( indx == size) {       // Point p is not found in the list
      return -1;
    }
    else {
      return indx;   // Return the location of the point in the list
    }
  }
}


void Pt2dList::markLinks(const char m)
{
	if (size != 0) {
		Pt2dLink *tmpLink = list;
		do {
			tmpLink->mark = m;
			tmpLink = tmpLink->next;
		} while (tmpLink != list);
	}
}


void Pt2dList::print()
{
  if( size == 0) {
    cout << "\n  The list is empty." << endl;
  }
  else {
    Pt2dLink *tmpLink = list;
    for( int i = 0; i < size; i++, tmpLink = tmpLink->next) {
      cout << "\nPoint # " << i;
      tmpLink->here.print();
    }
  }
}

int Pt2dList::read(istream& fp)
{
  double tmpX, tmpY;
  int tmpSize;

// First, destruct the old list
  //this->~Pt2dList();
  empty();
  list = NULL;
  size = 0;
  plast = NULL;
  ilast = 0;

// Then, construct a new list that is identical to the input list
  fp>>tmpSize;
  if(!fp) return 0;
  if( tmpSize <= 0) {
    cout << "\n  Error: Input size cannot be negative!" << endl;
    return 0;
  }
  for( int i = 0; i < tmpSize; i++)
  {
    fp>>tmpX;
    if(!fp) return 0;
    fp>>tmpY;
    if(!fp) return 0;
    append(point( tmpX, tmpY));
  }

  return 1;
}

int Pt2dList::write(ostream& fp)
{
  if( size == 0) {
    cout << "\n  Warning: The list is empty." << endl;
    fp<<size;
    if(!fp) return 0;
  }
  else {
    Pt2dLink *tmpLink = list;
    fp<<size<<"\n";
    if(!fp) return 0;
    for( int i = 0; i < size; i++, tmpLink = tmpLink->next) {
      fp<<tmpLink->here.X()<<" ";
      if(!fp) return 0;
      fp<<tmpLink->here.Y()<<"\n";
      if(!fp) return 0;
    }
  }
  return 1;
}

Pt2dLink* Pt2dList::address( int indx)
{
  if( size == 0) {
    cout << "\n  Error: Cannot index to an empty list!" << endl;
    exit( 1);
  }
  else if( indx < 0 || indx >= size) {
    cout << "\n  Error: Index value is out of range!" << endl;
    exit( 1);
  }
  else {
    int i;
    Pt2dLink *tmpLink = plast;
    if( indx > ilast) for( i = 0; i < indx-ilast; i++) tmpLink = tmpLink->next;
    else              for( i = 0; i < ilast-indx; i++) tmpLink = tmpLink->prev;
    plast = tmpLink;
    ilast = indx;
    return tmpLink;
  }
  return(NULL);
}

Pt2dLink* Pt2dList::address( const Point2d& p)
{
  if( size == 0) {
    cout << "\n  Error: List is empty!" << endl;
    exit( 1);
  }
  else {
    Pt2dLink *tmpLink = list;
    if( tmpLink->here == p) return tmpLink;
    do{
       if( tmpLink->here == p) break;
       else tmpLink = tmpLink->next;
    }while( tmpLink != list);
    if( tmpLink == list)
    {
      cout << "\n Point is not in the list!" << endl;
      exit(1);
    }
    return tmpLink;
  }
  return(NULL);
}

