/**********************************************************************/
/*                                                                    */
/* This software module was originally developed by                   */
/*                                                                    */
/* A. Tanju Erdem (Eastman Kodak Company)                             */
/* Candemir Toklu (University of Rochester)                           */
/*                                                                    */
/* in the course of development of MPEG-4 Visual (ISO/IEC 14496-2).   */
/* This software module is an implementation of a part of one or      */
/* more MPEG-4 Visual (ISO/IEC 14496-2) tools as specified by the     */
/* MPEG-4 Visual (ISO/IEC 14496-2). ISO/IEC gives users of MPEG-4     */
/* Visual (ISO/IEC 14496-2) free license to this software module or   */
/* modifications thereof for use in hardware or software products     */
/* claiming conformance to MPEG-4 Visual (ISO/IEC 14496-2). Those     */
/* intending to use this software module in hardware or software      */
/* products are advised that its use may infringe existing patents.   */
/* The original developer of this software module and his/her         */
/* company, the subsequent editors and their companies, and ISO/IEC   */
/* have no liability for use of this software module or modifications */ 
/* thereof in an implementation. Copyright is not released for non    */
/* MPEG-4 Visual (ISO/IEC 14496-2) conforming products.               */
/* Eastman Kodak Company and the University of Rochester retain full  */
/* right to use the code for their own purposes, assign or donate the */
/* code to a third party and to inhibit third parties from using the  */
/* code for non MPEG-4 Visual (ISO/IEC 14496-2) conforming products.  */
/* This copyright notice must be included in all copies or derivative */
/* works.                                                             */
/*                                                                    */
/* Copyright (c) 1994-1997                                            */
/*                                                                    */
/**********************************************************************/

/*+*********************************************************************
   NAME         listClass.h
   AUTHOR       A. Tanju Erdem - Eastman Kodak Company and Candemir Toklu - 
                University of Rochester.
   DATE         September 1994
   UPDATE       14 August 1996 by Candemir Toklu
   DESCRIPTION  The Pt2dList class defines a link list of Point2dLink's, i.e.
                Point2d's. A list (i.e., Pt2dList) is a closed sequence of 
				points with links to the "previous" and "next" points. 
				The last point in the list points to the first point in the 
				list (hence the closedness). The links are defined to be 
				two-directional so that adding and removing points to/from 
				the list is straightforward. The points in a list are 
				distinct however they are not ordered in any particular way. 
				A list may be empty.


   SYNOPSIS     Include files : 
                                
                #include <iostream.h>
                #include <fstream.h>
                #include <math.h>
                #include <time.h>
                #include "linkClass.h"
                #include "pointClass.h"

*********************************************************************-*/
#include <iostream.h>
#include <fstream.h>
#include <math.h>
#include <time.h>
#include "linkClass.h"
#include "pointClass.h"

#ifndef POINT_LIST_CLASS
#define POINT_LIST_CLASS

//------------------------------------------------------------------------

class Pt2dList {
    friend class Poly2dTess;
  public:
    Pt2dList();
    Pt2dList( const Point2d&);
    Pt2dList( const Pt2dList&);
    ~Pt2dList();
    Pt2dList& empty();
    Pt2dList& operator=( const Pt2dList&);
    Pt2dList& insert( const Point2d&);  // Add to the beginning
    Pt2dList& append( Point2d);  // Add to the end
    Pt2dLink* add( Point2d);  // Returns the address if already in the list
    int grow( Point2d);  // Returns 1 if new or 0 if already in the list
    Pt2dList& remove( const Point2d&);  // Remove from the list
    Point2d& operator[](int);
    int where( Pt2dLink*) const;  // Returns -1 if cannot find
    int where( Point2d&) const;  // Returns -1 if cannot find
	void markLinks(const char);
    inline int getSize();
    void print();
    int read( istream& fp);
    int write( ostream& fp);
  protected:
    Pt2dLink *list;  // Pointer to the first point-link (link number 0)
    int size;  // Number of points in the list
    Pt2dLink *plast;  // Pointer to the last-accessed point-link
    int ilast;  // Index to the last-accessed point-link
  private:
    Pt2dLink* address( int);
    Pt2dLink* address( const Point2d&);
};

int Pt2dList::getSize()
{
  return size;
}

#endif
