/**********************************************************************/
/*                                                                    */
/* This software module was originally developed by                   */
/*                                                                    */
/* Peter van Beek (University of Rochester)                           */
/*                                                                    */
/* in the course of development of MPEG-4 Visual (ISO/IEC 14496-2).   */
/* This software module is an implementation of a part of one or      */
/* more MPEG-4 Visual (ISO/IEC 14496-2) tools as specified by the     */
/* MPEG-4 Visual (ISO/IEC 14496-2). ISO/IEC gives users of MPEG-4     */
/* Visual (ISO/IEC 14496-2) free license to this software module or   */
/* modifications thereof for use in hardware or software products     */
/* claiming conformance to MPEG-4 Visual (ISO/IEC 14496-2). Those     */
/* intending to use this software module in hardware or software      */
/* products are advised that its use may infringe existing patents.   */
/* The original developer of this software module and his/her         */
/* company, the subsequent editors and their companies, and ISO/IEC   */
/* have no liability for use of this software module or modifications */ 
/* thereof in an implementation. Copyright is not released for non    */
/* MPEG-4 Visual (ISO/IEC 14496-2) conforming products.               */
/* The University of Rochester retains full right to use the code for */
/* their own purposes, assign or donate the code to a third party and */
/* to inhibit third parties from using the code for non MPEG-4 Visual */
/* (ISO/IEC 14496-2) conforming products.                             */
/* This copyright notice must be included in all copies or derivative */
/* works.                                                             */
/*                                                                    */
/* Copyright (c) 1997, 1998                                           */
/*                                                                    */
/**********************************************************************/

/*+***********************************************************-*-C++-*-*
   NAME         mesh_decode.h
   AUTHOR/DATE  Peter Van Beek, Fri May  1 17:40:22 1998
   DESCRIPTION  Mesh decoder definitions.
   SYNOPSIS     
*********************************************************************-*/

#ifndef	MESH_DECODER_DEF
#define	MESH_DECODER_DEF

#include "pointClass.h"					// Point2d
#include "polyTessClass.h"				// Poly2dTess
//#include "basic.h"
//#include "bitstrm.h"					// CInBitStream
//#include "huffman.h"					// CEntropyDecoderSet


/************** DEFINITIONS ************************************************/

// Define I-MOP and P-MOP types
#define I_MOP						1
#define P_MOP						0

// Define byte
#ifndef BYTE
#define BYTE 	unsigned char
#endif

/************** CLASS DEFINITIONS ******************************************/

// forward class definitions
class CInBitStream;
class CEntropyDecoderSet;


// MeshDecoder class definition
class MeshDecoder
{
  public:
	// Constructor 
	MeshDecoder();

	// Constructor that attaches given input buffer to the decoder
	MeshDecoder(BYTE *pInBuffer, const int inSize);

	// Destructor
	~MeshDecoder();

	// Attach a given input buffer
	void attachBuf(BYTE *pInBuffer, const int inSize, const int bitPos);

	// Decode one Mesh Object Plane (MOP) from a bit stream 'inStream'
	int decodeMesh (int &mType);

	// Get attributes
	int getStartStrm()     { return startStrm; }
	int getEndStrm()       { return endStrm; }
	BYTE *getStreamPtr(); 
	UINT getBitsLeft(); 

	// The current (decoded) mesh
	Poly2dTess meshCurr;

	// Mesh bounding box attributes
	double xmin, xmax, ymin, ymax;

  private:
	// Decode geometry and boundary of Delaunay-type mesh (I-MOP)
	void decDelaunayMesh ();
	// Decode geometry and boundary of Uniform-type mesh (I-MOP)
	void decUniformMesh ();
	// Decode motion of previous to current mesh (P-MOP)
	void decMeshMotion ();

	// Decode a mesh node motion vector
	Point2d decodeMV (Point2d &mv);
	// Skip stuffing bits until next start code
	int findStartCode();
	// Skip stuffing bits until byte-aligned
	int nextByteAlign();
	// Compute mesh bounding box attibutes
	void setBBox();
	// Compute index of most top left node on the boundary of the mesh
	int getTopLeftIndex();

	// Boolean to check if start code has been read
	int startStrm;
	// Boolean to check if end code has been read
	int endStrm;

	// Bitstream and Huffman entropy decoding pointers
	CInBitStream* bitstrmIn;
	CEntropyDecoderSet* entrdecSet;

	// Number of boundary mesh nodes
	int nBoun;
	// Array with indices of boundary mesh nodes
	int *bounIndex;

	// motion vector range code (Fcode)
	int  fCode; 
	// motion vector scale factor
	UINT scaleFactor;
	// motion range in half pixel units
	int  motionRange;

};


#endif // MESH_DECODER_DEF

// End of file
