/**********************************************************************/
/*                                                                    */
/* This software module was originally developed by                   */
/*                                                                    */
/* A. Tanju Erdem (Eastman Kodak Company)                             */
/* Candemir Toklu (University of Rochester)                           */
/*                                                                    */
/* in the course of development of MPEG-4 Visual (ISO/IEC 14496-2).   */
/* This software module is an implementation of a part of one or      */
/* more MPEG-4 Visual (ISO/IEC 14496-2) tools as specified by the     */
/* MPEG-4 Visual (ISO/IEC 14496-2). ISO/IEC gives users of MPEG-4     */
/* Visual (ISO/IEC 14496-2) free license to this software module or   */
/* modifications thereof for use in hardware or software products     */
/* claiming conformance to MPEG-4 Visual (ISO/IEC 14496-2). Those     */
/* intending to use this software module in hardware or software      */
/* products are advised that its use may infringe existing patents.   */
/* The original developer of this software module and his/her         */
/* company, the subsequent editors and their companies, and ISO/IEC   */
/* have no liability for use of this software module or modifications */ 
/* thereof in an implementation. Copyright is not released for non    */
/* MPEG-4 Visual (ISO/IEC 14496-2) conforming products.               */
/* Eastman Kodak Company and the University of Rochester retain full  */
/* right to use the code for their own purposes, assign or donate the */
/* code to a third party and to inhibit third parties from using the  */
/* code for non MPEG-4 Visual (ISO/IEC 14496-2) conforming products.  */
/* This copyright notice must be included in all copies or derivative */
/* works.                                                             */
/*                                                                    */
/* Copyright (c) 1994-1997                                            */
/*                                                                    */
/**********************************************************************/

/*+*********************************************************************
   NAME         polyRefClass.cpp
   AUTHOR       A. Tanju Erdem - Eastman Kodak Company and Candemir Toklu - 
                University of Rochester.
   DATE         September 1994
   UPDATE       14 August 1996 by Candemir Toklu
   DESCRIPTION  Member functions for Poly2dRef class are defined in this
                file.

   UPDATE		Peter Van Beek, Tue Feb 17 12:44:05 1998
   				Added 'mark' field.

   SYNOPSIS     Include files : 
                                
                #include "polyRefClass.h"
                #include "polyClass.h"
                               
*********************************************************************-*/
#include "polyRefClass.h"
#include "polyClass.h"


//------------------------------------------------------------------------
//------------------------------------------------------------------------

Poly2dRef::Poly2dRef()  // A link
{
  sizeRef = 0;
  corner = NULL;
//  neighbor = NULL;
  next = prev = NULL;
  mark = 0;
}

Poly2dRef::Poly2dRef( int sides)  // A link with references
{
  sizeRef = sides;
  next = prev = NULL;
  corner = (Pt2dLink**)new long[ sizeRef];
//  neighbor = (Poly2dRef**)new long[ sizeRef];
  for( int i = 0; i < sizeRef; i++)
  {
    corner[i] = NULL;
//    neighbor[i] = NULL;
  }
  mark = 0;
}

Poly2dRef::~Poly2dRef()  // Delete the pointers to corners and neighbors.
{
  if( sizeRef)
  {
    delete[] corner;
//    delete[] neighbor;
  }
}

Pt2dLink* Poly2dRef::address( int indx)
{
  if( sizeRef == 0) {
    cout << "\n  Error: Cannot index to a reference that has no corners!" 
         << endl;
    exit( 1);
  }
  else if( indx < 0 || indx >= sizeRef) {
    cout << "\n  Error: Index value is out of range!" << endl;
    exit( 1);
  }
  else {
    Pt2dLink* tmpLink;
    tmpLink = corner[indx];
    return tmpLink;
  }
  return(NULL);
}

Poly2d Poly2dRef::polygon()
{
   if( sizeRef == 0) 
   {
      cout << "\n Empty polygon is returned the reference has no corners."
           << endl;
   }

   Poly2d p( sizeRef);
   for( int i = 0; i < sizeRef; i++)
   {
      p[i] = corner[i]->getPoint();
   }
   return p;   
}

