/**********************************************************************/
/*                                                                    */
/* This software module was originally developed by                   */
/*                                                                    */
/* A. Tanju Erdem (Eastman Kodak Company)                             */
/* Candemir Toklu (University of Rochester)                           */
/*                                                                    */
/* in the course of development of MPEG-4 Visual (ISO/IEC 14496-2).   */
/* This software module is an implementation of a part of one or      */
/* more MPEG-4 Visual (ISO/IEC 14496-2) tools as specified by the     */
/* MPEG-4 Visual (ISO/IEC 14496-2). ISO/IEC gives users of MPEG-4     */
/* Visual (ISO/IEC 14496-2) free license to this software module or   */
/* modifications thereof for use in hardware or software products     */
/* claiming conformance to MPEG-4 Visual (ISO/IEC 14496-2). Those     */
/* intending to use this software module in hardware or software      */
/* products are advised that its use may infringe existing patents.   */
/* The original developer of this software module and his/her         */
/* company, the subsequent editors and their companies, and ISO/IEC   */
/* have no liability for use of this software module or modifications */ 
/* thereof in an implementation. Copyright is not released for non    */
/* MPEG-4 Visual (ISO/IEC 14496-2) conforming products.               */
/* Eastman Kodak Company and the University of Rochester retain full  */
/* right to use the code for their own purposes, assign or donate the */
/* code to a third party and to inhibit third parties from using the  */
/* code for non MPEG-4 Visual (ISO/IEC 14496-2) conforming products.  */
/* This copyright notice must be included in all copies or derivative */
/* works.                                                             */
/*                                                                    */
/* Copyright (c) 1994-1997                                            */
/*                                                                    */
/**********************************************************************/

/*+*********************************************************************
   NAME         polyRefClass.h
   AUTHOR       A. Tanju Erdem - Eastman Kodak Company and Candemir Toklu - 
                University of Rochester.
   DATE         September 1994
   UPDATE       14 August 1996 by Candemir Toklu
   DESCRIPTION  The Poly2dRef class defines a polygon link. A polygon link 
                is a polygon with pointers to the previous and next
                neighboring polygons, the number of the corners, and
                pointers to the references of the corner locations.

   UPDATE		Peter Van Beek, Tue Feb 17 12:44:05 1998
   				Added 'mark' field.

   SYNOPSIS     Include files : 
                                
                #include <iostream.h>
                #include <fstream.h>
                #include <math.h>
                #include <time.h>
                #include "linkClass.h"

*********************************************************************-*/
#include <iostream.h>
#include <fstream.h>
#include <math.h>
#include <time.h>
#include "linkClass.h"

class Poly2d;
class Poly2dTess;

#ifndef POLY_LINK_CLASS
#define POLY_LINK_CLASS

/* A reference-to-a-triangle points to the corner points of the triangle
// and its neighboring triangles. Two triangles are said to be neighbors 
// if they share a side.
//
//         0
//         +
//         |\
//         | \                +'s indicate the corner points.
//         |  \
//   0 <-- |   \ --> 2      -->'s point to the neighboring triangles.
//         |    \
//         |     \
//         |      \
//       1 +-------+ 2
//             |
//             V
//
//             1
//
// In the figure above, the indexing of the corner points is arbitrary;
// however, the neighboring triangle on side [i,i+1] is always assigned 
// the index i. Note that i+1 on the preceding line is modulo 3.
//
// A generalization of a reference-to-a-triangle is a reference-to-a-polygon.
// Only class Poly2dTess functions can create and destroy Poly2dRef objects.
// Otherwise one should only declare pointers to Poly2dRef objects.
*/

class Poly2dRef {
    friend class Poly2dTess;
  public:
    Poly2dRef();
    Poly2dRef( int);
    ~Poly2dRef();
    inline int getSize();
    Pt2dLink* address( int);
    Poly2d polygon();
	char mark;			// Marker of this polygon, PvB
  private:
    int sizeRef;  // The number of corners/neighbors.
    Pt2dLink** corner;  // Pointer to an array of sizeRef pointers to corner points.
    Poly2dRef *next, *prev;  // Pointers to previous and next links in the list.
// The following data member currently is not used
//    Poly2dRef** neighbor;  // Pointer to an array of sizeRef pointers to neighbors.
};

int Poly2dRef::getSize()
{
  return sizeRef;
}

#endif
