/**********************************************************************/
/*                                                                    */
/* This software module was originally developed by                   */
/*                                                                    */
/* A. Tanju Erdem (Eastman Kodak Company)                             */
/* Candemir Toklu (University of Rochester)                           */
/*                                                                    */
/* and edited by                                                      */
/*                                                                    */
/* Peter van Beek (University of Rochester)                           */
/*                                                                    */
/* in the course of development of MPEG-4 Visual (ISO/IEC 14496-2).   */
/* This software module is an implementation of a part of one or      */
/* more MPEG-4 Visual (ISO/IEC 14496-2) tools as specified by the     */
/* MPEG-4 Visual (ISO/IEC 14496-2). ISO/IEC gives users of MPEG-4     */
/* Visual (ISO/IEC 14496-2) free license to this software module or   */
/* modifications thereof for use in hardware or software products     */
/* claiming conformance to MPEG-4 Visual (ISO/IEC 14496-2). Those     */
/* intending to use this software module in hardware or software      */
/* products are advised that its use may infringe existing patents.   */
/* The original developer of this software module and his/her         */
/* company, the subsequent editors and their companies, and ISO/IEC   */
/* have no liability for use of this software module or modifications */ 
/* thereof in an implementation. Copyright is not released for non    */
/* MPEG-4 Visual (ISO/IEC 14496-2) conforming products.               */
/* Eastman Kodak Company and the University of Rochester retain full  */
/* right to use the code for their own purposes, assign or donate the */
/* code to a third party and to inhibit third parties from using the  */
/* code for non MPEG-4 Visual (ISO/IEC 14496-2) conforming products.  */
/* This copyright notice must be included in all copies or derivative */
/* works.                                                             */
/*                                                                    */
/* Copyright (c) 1994-1997                                            */
/*                                                                    */
/**********************************************************************/

/*+*********************************************************************
   NAME         polyTessClass.h
   AUTHOR       A. Tanju Erdem - Eastman Kodak Company and Candemir Toklu - 
                University of Rochester.
   DATE         September 1994
   UPDATE       14 August 1996 by Candemir Toklu
   				- various
   				Peter Van Beek, Fri May  1 16:08:26 1998 
				- added quantize method
   DESCRIPTION  Poly2dTess defines a tesselation, i.e. mesh. A mesh is formed
                from a Pt2dList and a link list of Poly2dRef. Pt2dList holds
                the locations of the nodes of the mesh in a linked fashion.
                The member function tess is a pointer to the first Poly2dRef
                in the patch list. Patches of the mesh and the nodes of
                the mesh are linked to each other thru corner, owner
                relationships.

   SYNOPSIS     Include files : 
                                
                #include <iostream.h>
                #include <fstream.h>
                #include <math.h>
                #include <time.h>
                #include "polyClass.h"
                #include "pointClass.h"
                #include "polyRefClass.h"
                #include "listClass.h"

*********************************************************************-*/

#include <iostream.h>
#include <fstream.h>
#include <math.h>
#include <time.h>
#include "polyClass.h"
#include "pointClass.h"
#include "polyRefClass.h"
#include "listClass.h"

#ifndef POLY_TESS_CLASS
#define POLY_TESS_CLASS


//------------------------------------------------------------------------
// Tesselation of polygons.
class Poly2dTess {
//    friend void polyTess( const Pt2dList& plist, Poly2dTess& ptess);
//    friend Poly2dRef* ( const Point2d& p, Poly2dTess& ttess);
    friend Poly2dTess rectTess( int width, int height);
    friend Poly2dTess divInQuad( Poly2d&, int, int);
    friend Poly2dTess divInTriangle( Poly2d&, int, int, int);
  public:
    Poly2dTess();
    Poly2dTess( const Poly2d&);
    Poly2dTess( const Poly2dTess&);
    ~Poly2dTess();
    Poly2dTess& destroy();
    Poly2dTess& operator=( const Poly2dTess&);
    Poly2dTess& append( Poly2d&);  // Add to the list
//    Poly2dTess& remove( Poly2d&);  // Remove from the list
//    Poly2d& operator[](int);
    Poly2dTess& move( double, double);
    Poly2dTess& move( const Point2d&);
    Poly2dTess& scale( double, double);
	Poly2dTess& quantize( double);		// Quantize coordinates
    Point2d& point(int);
    Point2d& point(int, double&, double&); // C.T.
    Poly2d polygon(int);
    Poly2dTess& reOrderOwners();
	void markRefs(const char);
	void markLinks(const char);
    int where( Pt2dLink*) const;  // Index of a point whose address is given
    int where( Poly2dRef*) const;  // Index of a reference whose address is given
    int where( Point2d&) const; // Index of a point in the point list.
    Pt2dLink* addressC( const Point2d);  // Address of a point if it exits.
    Pt2dLink* addressC( int);  // Address of a point whose index is given
    Poly2dRef* addressR( int);  // Address of a reference whose index is given
    int getSizeRef( int);  // Size of the indexed reference (i.e., # of sides)
    int getSizeLink( int);  // Size of the indexed link (i.e., # of owners)
    inline int getSizePoints();
    inline int getSize();
	Pt2dList neighbors (Point2d&, int);
    int read( ifstream&);
    int write( ofstream&);
    int read(char []);
    int write(char []);
    void print();
	Poly2dTess& makeUniform(int nhori, int nvert, double hsize, double vsize, int splitcode); // Construct a uniform triangular mesh
  private:
    int sizeTess;
    Poly2dRef* tess;  // Provides access to pointers to the member points
    Pt2dList points;  // Contains pointers to the member polygons
};

//--------------------------------------------------------------------------
int Poly2dTess::getSizePoints()
{
  return points.getSize();
}

//--------------------------------------------------------------------------
int Poly2dTess::getSize()
{
  return sizeTess;
}

#endif
