/**********************************************************************/
/*                                                                    */
/* This software module was originally developed by                   */
/*                                                                    */
/* Peter van Beek (University of Rochester)                           */
/*                                                                    */
/* in the course of development of MPEG-4 Visual (ISO/IEC 14496-2).   */
/* This software module is an implementation of a part of one or      */
/* more MPEG-4 Visual (ISO/IEC 14496-2) tools as specified by the     */
/* MPEG-4 Visual (ISO/IEC 14496-2). ISO/IEC gives users of MPEG-4     */
/* Visual (ISO/IEC 14496-2) free license to this software module or   */
/* modifications thereof for use in hardware or software products     */
/* claiming conformance to MPEG-4 Visual (ISO/IEC 14496-2). Those     */
/* intending to use this software module in hardware or software      */
/* products are advised that its use may infringe existing patents.   */
/* The original developer of this software module and his/her         */
/* company, the subsequent editors and their companies, and ISO/IEC   */
/* have no liability for use of this software module or modifications */ 
/* thereof in an implementation. Copyright is not released for non    */
/* MPEG-4 Visual (ISO/IEC 14496-2) conforming products.               */
/* The University of Rochester retains full right to use the code for */
/* their own purposes, assign or donate the code to a third party and */
/* to inhibit third parties from using the code for non MPEG-4 Visual */
/* (ISO/IEC 14496-2) conforming products.                             */
/* This copyright notice must be included in all copies or derivative */
/* works.                                                             */
/*                                                                    */
/* Copyright (c) 1997                                                 */
/*                                                                    */
/**********************************************************************/

/*+*********************************************************************
   NAME         queueClass.cpp
   AUTHOR       Peter Van Beek, Sat Mar  1 14:31:35 1997
   DESCRIPTION  Queue of triplets of dnode pointersintegers implemented by 
   				singly linked list. Queue elements are handled on a first-
				in-first-out basis.

   SYNOPSIS     #include "queueClass.h"

*********************************************************************-*/

#include <iostream.h>
#include "queueClass.h"
#include "linkClass.h"


// constructor
TripletQueue::TripletQueue()
{	
	first = NULL; last = NULL;
}

// destructor
TripletQueue::~TripletQueue()
{	
	TripletQueueElem *elem = first;

	while (elem != NULL) {
		elem = first->next;
		delete (first);
		first = elem;
	}
	first = NULL; last = NULL;
}

// copy constructor
TripletQueue::TripletQueue (const TripletQueue& queue)
{
	if (queue.first == NULL  ||  queue.last == NULL) {
		first = NULL; last = NULL;
	}
	else {
		TripletQueueElem *elem = queue.first;
		TripletQueueElem *newelem = new TripletQueueElem;

		// first copy the first element
		newelem->v1 = elem->v1;
		newelem->v2 = elem->v2;
		newelem->v3 = elem->v3;
		newelem->next = NULL;
		first = last = newelem;

		while (elem->next != NULL) {
			elem = elem->next;
			newelem = new TripletQueueElem;
			newelem->v1 = elem->v1;
			newelem->v2 = elem->v2;
			newelem->v3 = elem->v3;
			newelem->next = NULL;
			last->next = newelem;
			last = newelem;
		}
	}
}

// assignment operator
TripletQueue& TripletQueue::operator= (const TripletQueue& queue)
{
	// protect against copying to itself
	if (this == &queue) return (*this);

	TripletQueueElem *elem = first;

	// destruct old queue first
	while (elem != NULL) {
		elem = first->next;
		delete (first);
		first = elem;
	}
	first = NULL; last = NULL;

	// now copy other queue
	if (queue.first != NULL  &&  queue.last != NULL) {
		elem = queue.first;
		TripletQueueElem *newelem = new TripletQueueElem;

		// first copy the first element
		newelem->v1 = elem->v1;
		newelem->v2 = elem->v2;
		newelem->v3 = elem->v3;
		newelem->next = NULL;
		first = last = newelem;

		while (elem->next != NULL) {
			elem = elem->next;
			newelem = new TripletQueueElem;
			newelem->v1 = elem->v1;
			newelem->v2 = elem->v2;
			newelem->v3 = elem->v3;
			newelem->next = NULL;
			last->next = newelem;
			last = newelem;
		}
	}

	return (*this);
}

// append an element with pointer triplet at the end of the queue
TripletQueue& TripletQueue::append (Pt2dLink *i1, Pt2dLink *i2, Pt2dLink *i3)
{
	TripletQueueElem *newelem = new TripletQueueElem;

	// fill new element
	newelem->v1 = i1;
	newelem->v2 = i2;
	newelem->v3 = i3;
	newelem->next = NULL;

	if (first == NULL  ||  last == NULL) {
		// list was empty
		first = last = newelem;
	}
	else {
		// add to the linked list
		last->next = newelem;
		last = newelem;
	}
	return (*this);
}

// remove and return the element at the front of the queue
TripletQueue& TripletQueue::remove(TripletQueueElem& elem)
{
	if (first == NULL  ||  last == NULL) {
		cout << "TripletQueue: queue is empty!" << endl;
		exit(1);
	}
	else {
		TripletQueueElem *oldelem = first;
		elem.v1 = oldelem->v1;
		elem.v2 = oldelem->v2;
		elem.v3 = oldelem->v3;
		elem.next = NULL;
		first = first->next;
		if (first == NULL) last = NULL;
		delete oldelem;
	}
	return (*this);
}

// remove and return the element at the front of the queue
TripletQueue& TripletQueue::remove(Pt2dLink *&p1, Pt2dLink *&p2, Pt2dLink *&p3)
{
	if (first == NULL  ||  last == NULL) {
		cout << "TripletQueue: queue is empty!" << endl;
		exit(1);
	}
	else {
		TripletQueueElem *oldelem = first;
		p1 = (Pt2dLink *) oldelem->v1;
		p2 = (Pt2dLink *) oldelem->v2;
		p3 = (Pt2dLink *) oldelem->v3;
		first = first->next;
		if (first == NULL) last = NULL;
		delete oldelem;
	}
	return (*this);
}

// check if queue is empty or not
int TripletQueue::empty ()
{
	if (first == NULL  ||  last == NULL) 
		return (1);
	else
		return (0);
}

// print contents of elements of the queue
void TripletQueue::print()
{
	TripletQueueElem *elem = first;
	while (elem != NULL) {
		cout << elem->v1 << " " << elem->v2 << " " << elem->v3 << endl;
		elem = elem->next;
	}
}

// end of file
