/**********************************************************************/
/*                                                                    */
/* This software module was originally developed by                   */
/*                                                                    */
/* Peter van Beek (University of Rochester)                           */
/*                                                                    */
/* in the course of development of MPEG-4 Visual (ISO/IEC 14496-2).   */
/* This software module is an implementation of a part of one or      */
/* more MPEG-4 Visual (ISO/IEC 14496-2) tools as specified by the     */
/* MPEG-4 Visual (ISO/IEC 14496-2). ISO/IEC gives users of MPEG-4     */
/* Visual (ISO/IEC 14496-2) free license to this software module or   */
/* modifications thereof for use in hardware or software products     */
/* claiming conformance to MPEG-4 Visual (ISO/IEC 14496-2). Those     */
/* intending to use this software module in hardware or software      */
/* products are advised that its use may infringe existing patents.   */
/* The original developer of this software module and his/her         */
/* company, the subsequent editors and their companies, and ISO/IEC   */
/* have no liability for use of this software module or modifications */ 
/* thereof in an implementation. Copyright is not released for non    */
/* MPEG-4 Visual (ISO/IEC 14496-2) conforming products.               */
/* The University of Rochester retains full right to use the code for */
/* their own purposes, assign or donate the code to a third party and */
/* to inhibit third parties from using the code for non MPEG-4 Visual */
/* (ISO/IEC 14496-2) conforming products.                             */
/* This copyright notice must be included in all copies or derivative */
/* works.                                                             */
/*                                                                    */
/* Copyright (c) 1997                                                 */
/*                                                                    */
/**********************************************************************/

/*+*********************************************************************
   NAME         queueClass.h
   AUTHOR       Peter Van Beek, University of Rochester
   DATE			Sat Mar  1 14:31:35 1997
   DESCRIPTION  Queue of triplets of pointers implemented by singly 
   				linked list. Queue elements are handled on a first-in-
				first-out basis.

   SYNOPSIS     
                                
***************************************************************************-*/

#include <iostream.h>
#include "linkClass.h"

#ifndef TRI_QUEUE_CLASS
#define TRI_QUEUE_CLASS

//------------------------------------------------------------------------

// Queue element class 
// Contains triplet of node pointers and pointer to next element
class TripletQueueElem {
friend class TripletQueue;
  public:
	// triplet of integers
	Pt2dLink *v1, *v2, *v3;
  private:
	// pointer to next element
	TripletQueueElem *next;
};


// Triplet Queue class
// Queue is implemented with singly linked list
class TripletQueue {
  public:
	// normal constructor
	TripletQueue ();
	// copy constructor
	TripletQueue (const TripletQueue&);
	// assignment operator
	TripletQueue& operator= (const TripletQueue&);
	// destructor
	~TripletQueue ();
	// append new element at the end of the queue
	TripletQueue& append (Pt2dLink *, Pt2dLink *, Pt2dLink *);
	// remove element from front of the queue and return it by reference
	TripletQueue& remove (TripletQueueElem&);
	// remove element from front of the queue and return its data
	TripletQueue& remove(Pt2dLink *&p1, Pt2dLink *&p2, Pt2dLink *&p3);
	// is the queue empty?
	int empty ();
	// print elements of the queue
    void print ();

  public:
	// pointers to first and last elements in linked list
	TripletQueueElem *first, *last;
};


#endif
