/**********************************************************************/
/*                                                                    */
/* This software module was originally developed by                   */
/*                                                                    */
/* A. Patti (University of Rochester)                                 */
/* J. Riek (University of Rochester)                                  */
/*                                                                    */
/* and edited by                                                      */
/*                                                                    */
/* Peter van Beek (University of Rochester)                           */
/*                                                                    */
/* in the course of development of MPEG-4 Visual (ISO/IEC 14496-2).   */
/* This software module is an implementation of a part of one or      */
/* more MPEG-4 Visual (ISO/IEC 14496-2) tools as specified by the     */
/* MPEG-4 Visual (ISO/IEC 14496-2). ISO/IEC gives users of MPEG-4     */
/* Visual (ISO/IEC 14496-2) free license to this software module or   */
/* modifications thereof for use in hardware or software products     */
/* claiming conformance to MPEG-4 Visual (ISO/IEC 14496-2). Those     */
/* intending to use this software module in hardware or software      */
/* products are advised that its use may infringe existing patents.   */
/* The original developer of this software module and his/her         */
/* company, the subsequent editors and their companies, and ISO/IEC   */
/* have no liability for use of this software module or modifications */ 
/* thereof in an implementation. Copyright is not released for non    */
/* MPEG-4 Visual (ISO/IEC 14496-2) conforming products.               */
/* The University of Rochester retain full right to use the code for  */
/* their own purposes, assign or donate the code to a third party and */
/* to inhibit third parties from using the code for non MPEG-4 Visual */
/* (ISO/IEC 14496-2) conforming products.                             */
/* This copyright notice must be included in all copies or derivative */
/* works.                                                             */
/*                                                                    */
/* Copyright (c) 1993-1997                                            */
/*                                                                    */
/**********************************************************************/

/*+*********************************************************************
   NAME         stringClass.cpp
   AUTHORS      A. Patti and J. Riek, University of Rochester
   DATE         July 7, 1993
   UPDATE       Peter Van Beek, Tue Sep  9 10:29:49 1997
   				Inserted many checks against NULL pointers. 
   DESCRIPTION  Member functions of the string class.

   SYNOPSIS     

   ASSUMPTIONS  -  
                -  
**********************************************************************-*/

#include <string.h>
#include <stdio.h>
#include "stringClass.h"

char   string::fill_char='0';
int    string::fill_width=1;
int    string::precision=2;

// Define an ostream inserter to print out the string.
ostream& operator<<(ostream& os,const string& str)
{
	if(os.opfx()){
		os<<str.data;
		os.osfx();
	}
	return os;
}


// Define an istream extractor to read in strings.
istream& operator>>(istream& is,string& str)
{
	char buff[200];
	//if(is.ipfx())
//	{
	is>>buff;
	if(!str.getmem(strlen(buff))) exit(1);
	strcpy(str.data,buff);
//	}
	return is;
}


// Copy part of a string e.g. a(0,a.length()-2) returns a shortened by 1.
string string::operator()(int start,int end)
{
	if(end<start) 
	{
		cerr<<"Cannot copy from "<<start<<" to "<<end<<"."<<endl;
		return *this;
	}
	if(start<0) start=0;
	if(end>len) end=len;
	string tmp(end-start+1);
	strncpy(tmp.data,data+start,(end-start)+1);
	return tmp;
}


// Default constructor with no arguments - sets length 0 and data Null.
string::string()
{
	len=0;data=NULL;
}



// Copy constructor - used when a function returns a string by value.
string::string(const string& str)
{
	len=0;data=NULL;
	if (str.len!=0) {					// if there is data
		if (!getmem(str.len)) exit(1);
		strcpy(data,str.data);
	}
}



// Constructor with string initializer
string::string(char *chr)
{
	len=0;data=NULL;
	if (chr!=NULL) {					// if there is data
		if (!getmem(strlen(chr))) exit(1);
		strcpy(data,chr);
	}
}



// Constructor that provides an initial size for the string. Not too
// useful for the outside world, but good for operators.
string::string(int num)
{
	int i;
	len=0;data=NULL;
	if (!getmem(num)) exit(1);
	for(i=0;i<len;i++) data[i]=fill_char;
	data[len]='\0';
}


// Set precision to num decimal places in the range [1,19].
int string::set_precision(int num)
{
	if(num>0 && num<20) {precision=num;return 1;}
	cerr<<"Illegal value for precision: precision not changed."<<endl;
	return 0;
}


// Set the width of the numbers to be at least num characters.
// If the number width is greater than num, then all the characters
// will be printed. If the number width is less than num, then the
// number will be preceded by (num - number width) fill_char's.
int string::set_width(int num)
{
	if(num>0 && num<20) {fill_width=num;return 1;}
	cerr<<"Illegal value for width: width not changed."<<endl;
	return 0;
}



// Destructor - deletes the allocated memory if any.
string::~string()
{
    if (data!=NULL) 
	{ 
		delete data; 
		data=NULL;
	}
	len=0;
}


// Allocate memory if needed.
int string::getmem(int num)
{
	if (data!=NULL && len>=num) 
	{									// no need to allocate
		len=num;
		return 1;
	}
	if (data!=NULL) 
	{									// destroy old data
		delete data; 
		data=NULL; 
	}
	len=num;
	data = new char[len+1];				// allocate memory
	if (data!=NULL) return 1;
	else exit(1);
	return(0);
}


// Copy a string using the = operator
string& string::operator=(const string& str)
{
	if (this != &str)					// prevent auto-assignment
		return operator=(str.data);		// copy string data
	else
		return *this;					// return self
}


// Put a character string into a string using the = operator.
string& string::operator=(const char *chr)
{
	if (chr==NULL){						// no data
		cerr<<"Trying to assign with NULL string pointer."<<endl;
		return *this;					// do nothing
	}
	else {
		if(!getmem(strlen(chr))) exit(1);
		if(len==0) data[0]='\0';
		else strcpy(data,chr);
		return *this;
	}
}


// Put a character into a string using the = operator.
string& string::operator=(const char chr)
{
	if(!getmem(1)) exit(1);
	data[0]=chr;
	data[1]='\0';
	return *this;
}


// Put a floating point number into a string
string& string::operator=(const double dub)
{
	char num[30];
	int w;
	sprintf(num,"%*.*f%n",fill_width,precision,dub,&w);
	if(!getmem(w)) exit(1);

	for(w=0;num[w]==' ';w++) num[w]=fill_char;

	strcpy(data,num);
	return *this;
}


// Put an integer number into a string
string& string::operator=(const int dub)
{
	char num[30];
	int w;
	sprintf(num,"%*d%n",fill_width,dub,&w);
	if(!getmem(w)) exit(1);

	for(w=0;num[w]==' ';w++) num[w]=fill_char;

	strcpy(data,num);
	return *this;
}


// Add a string to the string
string& string::operator+=(const string& str)
{
	return operator+=(str.data);
}


// Add a character array to the string
string& string::operator+=(const char *chr)
{
	if (chr==NULL){						// no data
		cerr<<"Trying to append with NULL string pointer."<<endl;
		return *this;					// do nothing
	}
	else {
		char *tmp;
		tmp = new char[len+1];
		if(data) strcpy(tmp,data);		// save temporarily
		if(!getmem(len+strlen(chr))) exit(1);
		strcpy(data,tmp);				// return old data
		strcat(data,chr);				// append new data
		return *this;
	}
}


// Add a character to the string
string& string::operator+=(const char chr)
{
	char *tmp;
	tmp = new char[len+1];
	if(data) strcpy(tmp,data);
	if(!getmem(len+1)) exit(1);
	strcpy(data,tmp);
	data[len-1]=chr;
	data[len]='\0';
	return *this;
}


// Add a floating point number to the string
string& string::operator+=(const double dub)
{
	char *tmp;
	tmp = new char[len+1];
	if(data) strcpy(tmp,data);
	char num[30];
	int w;
	sprintf(num,"%*.*f%n",fill_width,precision,dub,&w);
	if(!getmem(len+w)) exit(1);

	for(w=0;num[w]==' ';w++) num[w]=fill_char;

	strcpy(data,tmp);
	strcat(data,num);
	return *this;
}


// Add an integer number to the string
string& string::operator+=(const int dub)
{
	char *tmp;
	tmp = new char[len+1];
	if(data) strcpy(tmp,data);
	int w;
	char num[30];
	sprintf(num,"%*d%n",fill_width,dub,&w);

	if(!getmem(len+w)) exit(1);

	for(w=0;num[w]==' ';w++) num[w]=fill_char;

	strcpy(data,tmp);
	strcat(data,num);
	return *this;
}


// Return concatenation of the string and another one
string string::operator+(const string& str)
{
	return operator+(str.data);
}


// Return concatenation of the string and a character array
string string::operator+(const char *chr)
{
	if (chr==NULL){						// no data
		string tmp(*this);
		return tmp;						// return copy of self
	}
	else {
		string tmp(len+strlen(chr));
		if(data) strcpy(tmp.data,data);
		strcat(tmp.data,chr);
		return tmp;
	}
}


// Return concatenation of a character array and a string
string operator+(const char *chr,const string& str)
{
	if (chr==NULL) {					// no data
		string tmp(str.len);
		if(str.data) strcpy(tmp.data,str.data);
		return tmp;						// return copy of str
	}
	else {
		string tmp(str.len+strlen(chr));
		strcpy(tmp.data,chr);
		if(str.data) strcat(tmp.data,str.data);
		return tmp;
	}
}


// Return concatenation of a character and a string
string operator+(const char chr,const string& str)
{
	string tmp(str.len+1);
	tmp.data[0]=chr;
	if(str.data) strcpy(tmp.data+1,str.data);
	return tmp;
}


// Return concatenation of a character and a number
string operator+(const double dub,const string& str)
{
	char num[30];
	int w;

	sprintf(num,"%*.*f%n",string::fill_width,string::precision,dub,&w);

	string tmp(str.len+w);

	for(w=0;num[w]==' ';w++) num[w]=string::fill_char;

	strcpy(tmp.data,num);
	if(str.data) strcat(tmp.data,str.data);
	return tmp;
}


// Return concatenation of a character and a number
string operator+(const int dub,const string& str)
{
	int w;
	char num[30];
	sprintf(num,"%*d%n",string::fill_width,dub,&w);

	string tmp(str.len+w);

	for(w=0;num[w]==' ';w++) num[w]=string::fill_char;

	strcpy(tmp.data,num);
	if(str.data) strcat(tmp.data,str.data);
	return tmp;
}


// Return concatenation of the string and a character
string string::operator+(const char chr)
{
	string tmp(len+1);
	if(data) strcpy(tmp.data,data);
	tmp.data[len]=chr;
	tmp.data[len+1]='\0';
	return tmp;
}


// Return concatenation of the string and a number
string string::operator+(const double dub)
{
	char num[30];
	int w;
	sprintf(num,"%*.*f%n",fill_width,precision,dub,&w);

	string tmp(len+w);

	for(w=0;num[w]==' ';w++) num[w]=fill_char;

	if(data) strcpy(tmp.data,data);
	strcat(tmp.data,num);
	return tmp;
}


// Return concatenation of the string and a number
string string::operator+(const int dub)
{
	int w;
	char num[30];
	sprintf(num,"%*d%n",fill_width,dub,&w);

	string tmp(len+w);

	for(w=0;num[w]==' ';w++) num[w]=fill_char;

	if(data) strcpy(tmp.data,data);
	strcat(tmp.data,num);
	return tmp;
}


// Return position of a first occurrence of a character in the string
int string::operator[](char chr)
{
	char *tmp;
	tmp=strchr(data,chr);
	if(!tmp) return -1;
	else return (int)(tmp-data);
}


// Return position of first occurrence of a character array in the string
int string::operator[](char *chr)
{
	if (chr==NULL) {
		cerr<<"Trying to index with NULL string pointer."<<endl;
		return -1;
	}
	else {
		char *tmp;
		tmp=strstr(data,chr);
		if(!tmp) return -1;
		else return (int)(tmp-data);
	}
}


// Repeat the string a number of times
string& string::operator*=(int num)
{
	if(num<1) {cerr<<"Can't make a nonpositive number of copies";exit(1);}

	char *tmp;
	int i,l;

	tmp=new char[len+1];
	if(!tmp) exit(1);
	strcpy(tmp,data);
	l=len;
	if(!getmem(len*num)) exit(1);
	for(i=0;i<num;i++) strcpy(data+i*l,tmp);
	return *this;
}


// Return the repeated string
string string::operator*(int num)
{
	if(num<1) {cerr<<"Can't make a nonpositive number of copies";exit(1);}

	string tmp;
	int i;

	if(!tmp.getmem(len*num)) exit(1);

	for(i=0;i<num;i++) strcpy(tmp.data+i*len,data);
	return tmp;
}


// String equality operator
int string::operator==(char *chr)
{
	if (chr==NULL) {
		cerr<<"Trying to compare with NULL string pointer."<<endl;
		return -1;
	}
	else {
		if(!strcmp(data,chr)) return 1;
		else return 0;
	}
}


// String in-equality operator
int string::operator!=(char *chr)
{
	if (chr==NULL){
		cerr<<"Trying to compare with NULL string pointer."<<endl;
		return -1;
	}
	else {
		if(!strcmp(data,chr)) return 0;
		else return 1;
	}
}


// String comparison operator
int string::operator<(char *chr)
{
	if (chr==NULL){
		cerr<<"Trying to compare with NULL string pointer."<<endl;
		return -1;
	}
	else {
		if(strcmp(data,chr)<0) return 1;
		else return 0;
	}
}


// String comparison operator
int string::operator>(char *chr)
{
	if (chr==NULL){
		cerr<<"Trying to compare with NULL string pointer."<<endl;
		return -1;
	}
	else {
		if(strcmp(data,chr)>0) return 1;
		else return 0;
	}
}


// String comparison operator
int string::operator>=(char *chr)
{
	if (chr==NULL){
		cerr<<"Trying to compare with NULL string pointer."<<endl;
		return -1;
	}
	else {
		if(strcmp(data,chr)>=0) return 1;
		else return 0;
	}
}


// String comparison operator
int string::operator<=(char *chr)
{
	if (chr==NULL){
		cerr<<"Trying to compare with NULL string pointer."<<endl;
		return -1;
	}
	else {
		if(strcmp(data,chr)<=0) return 1;
		else return 0;
	}
}


// String equality operator
int string::operator==(const string& str)
{
	return operator==(str.data);
}


// String in-equality operator
int string::operator!=(const string& str)
{
	return operator!=(str.data);
}


// String comparison operator
int string::operator<(const string& str)
{
	return operator<(str.data);
}


// String comparison operator
int string::operator>(const string& str)
{
	return operator>(str.data);
}


// String comparison operator
int string::operator<=(const string& str)
{
	return operator<=(str.data);
}


// String comparison operator
int string::operator>=(const string& str)
{
	return operator>=(str.data);
}


// end of file
