/**********************************************************************/
/*                                                                    */
/* This software module was originally developed by                   */
/*                                                                    */
/* A. Patti (University of Rochester)                                 */
/* J. Riek (University of Rochester)                                  */
/*                                                                    */
/* and edited by                                                      */
/*                                                                    */
/* Peter van Beek (University of Rochester)                           */
/*                                                                    */
/* in the course of development of MPEG-4 Visual (ISO/IEC 14496-2).   */
/* This software module is an implementation of a part of one or      */
/* more MPEG-4 Visual (ISO/IEC 14496-2) tools as specified by the     */
/* MPEG-4 Visual (ISO/IEC 14496-2). ISO/IEC gives users of MPEG-4     */
/* Visual (ISO/IEC 14496-2) free license to this software module or   */
/* modifications thereof for use in hardware or software products     */
/* claiming conformance to MPEG-4 Visual (ISO/IEC 14496-2). Those     */
/* intending to use this software module in hardware or software      */
/* products are advised that its use may infringe existing patents.   */
/* The original developer of this software module and his/her         */
/* company, the subsequent editors and their companies, and ISO/IEC   */
/* have no liability for use of this software module or modifications */ 
/* thereof in an implementation. Copyright is not released for non    */
/* MPEG-4 Visual (ISO/IEC 14496-2) conforming products.               */
/* The University of Rochester retain full right to use the code for  */
/* their own purposes, assign or donate the code to a third party and */
/* to inhibit third parties from using the code for non MPEG-4 Visual */
/* (ISO/IEC 14496-2) conforming products.                             */
/* This copyright notice must be included in all copies or derivative */
/* works.                                                             */
/*                                                                    */
/* Copyright (c) 1993-1997                                            */
/*                                                                    */
/**********************************************************************/

/*+*********************************************************************
   NAME         stringClass.h
   AUTHORS      A. Patti and J. Riek, University of Rochester
   DATE         July 7, 1993
   UPDATE       Peter Van Beek, Fri Mar  7 13:11:40 1997
   				Peter Van Beek, Tue Sep  9 10:29:49 1997
   DESCRIPTION  Header file for string class (see also stringClass.cpp).
**********************************************************************-*/

#ifndef STRING_CLASS
#define STRING_CLASS

#include<string.h>
#include<fstream.h>
#include<stdlib.h>


/*************************************************************************/
// Class string is a string storage and manipulation class

class string 
{
  public:
	char *data;							// stores the character string
	int len;							// the length of the string
	static char fill_char;				// character to fill empty spaces
	static int precision;
	static int fill_width;
  public:
	string();
	string(int num);
	string(char *chr);
	string(const string& str);
	~string();
	string operator()(int start,int end);
	int getmem(int num);
	int length() {return len;}
	char& operator[](int indx) {return data[indx];}
	int operator[](char chr);
	int operator[](char *chr);
	operator char*() {return data;}
	int set_precision(int num);
	int set_width(int num);
	int set_fill_char(char chr);
	string& operator=(const string& str);
	string& operator=(const char *chr);
	string& operator=(const char chr);
	string& operator=(const double dub);
	string& operator=(const int dub);
	string& operator+=(const string& str);
	string& operator+=(const char *chr);
	string& operator+=(const char chr);
	string& operator+=(const double dub);
	string& operator+=(const int dub);
	string& operator*=(int dub);
	string operator+(const string& str);
	string operator+(const char *chr);
	string operator+(const double dub);
	string operator+(const int dub);
	string operator*(int dub);
	string operator+(const char chr);
	int operator==(const string& str);
	int operator!=(const string& str);
	int operator<(const string& str);
	int operator>(const string& str);
	int operator<=(const string& str);
	int operator>=(const string& str);
	int operator==(char *chr);
	int operator!=(char *chr);
	int operator<(char *chr);
	int operator>(char *chr);
	int operator<=(char *chr);
	int operator>=(char *chr);

	friend ostream& operator<<(ostream& os,const string& str);
	friend istream& operator>>(istream& is,string& str);
	friend string operator+(const char *chr,const string& str);
	friend string operator+(const char chr,const string& str);
	friend string operator+(const double dub,const string& str);
	friend string operator+(const int dub,const string& str);

}; // class string

#endif // STRING_CLASS
