// mmiDoc.cpp : implementation of the CMmiDoc class
//

#include "stdafx.h"
#include <string.h>
#include <GL/gl.h>

#include "mmi.h"
#include "mmiDoc.h"
#include "mmiView.h"
#include "WorldInfoDlg.h"

#include "AL.h"

#include "Executive3d.h"
#include "opengl/base/OpenGLApplication.h"
#include "opengl/base/OpenGLPresenter.h"
#include "opengl/base/OpenGLVisualRenderer.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CMmiDoc

IMPLEMENT_DYNCREATE(CMmiDoc, CDocument)

BEGIN_MESSAGE_MAP(CMmiDoc, CDocument)
	//{{AFX_MSG_MAP(CMmiDoc)
	ON_COMMAND(ID_FILE_SIMULATE, OnFileBIFSSimulate)
	ON_UPDATE_COMMAND_UI(ID_EDIT_UNDO_MOVEMENT, OnUpdateEditUndoMovement)
	ON_COMMAND(ID_EDIT_UNDO_MOVEMENT, OnEditUndoMovement)
	ON_COMMAND(ID_FILE_CLOSE, OnFileClose)
	ON_COMMAND(ID_REMOTE_OPEN, OnRemoteOpen)
	ON_COMMAND(ID_FILE_RELOAD, OnFileReload)
	ON_UPDATE_COMMAND_UI(ID_FILE_RELOAD, OnUpdateFileReload)
	ON_COMMAND(ID_VIEW_WORLD_INFO, OnViewWorldInfo)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMmiDoc construction/destruction

CMmiDoc::CMmiDoc()
{
	m_nReturn = 0;
	mpoApp = NULL;
	mpoExec = NULL;
	mpoPres = NULL;
	TRACE("[Doc::Ctor] navigation FALSE \n");
	m_bNavigation = FALSE;
	m_bSequenceComplete = FALSE;
	m_fileOpened = FALSE;
}

CMmiDoc::~CMmiDoc()
{
  TRACE("[Doc::Dtor] \n");
}

BOOL CMmiDoc::OnNewDocument()
{
	if (!CDocument::OnNewDocument())
		return FALSE;

	// TODO: add reinitialization code here
	// (SDI documents will reuse this document)
	return TRUE;
}



/////////////////////////////////////////////////////////////////////////////
// CMmiDoc serialization

void CMmiDoc::Serialize(CArchive& ar)
{
	if (ar.IsStoring())
	{
		// TODO: add storing code here
	}
	else
	{
		// TODO: add loading code here
	}
}

/////////////////////////////////////////////////////////////////////////////
// CMmiDoc diagnostics

#ifdef _DEBUG
void CMmiDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CMmiDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CMmiDoc commands
void CMmiDoc::Finish(int /* nReason */)
{
  TRACE("[Doc::Finish] begin \n");

  if (mpoExec != NULL) 
	mpoExec->Stop();

  TRACE("[Doc::Finish] end \n");
}


/////////////////////////////////////////////////////////////////////////////

void CMmiDoc::DeleteContents() 
{
  TRACE("[Doc::DeleteContents] begin \n");

	// TODO: Add your specialized code here and/or call the base class
	if (!m_bFinished) {
		if (mpoExec)
			mpoExec->Stop();
		m_bFinished = TRUE;
	}
		
	CDocument::DeleteContents();

  TRACE("[Doc::DeleteContents] end \n");
}


/////////////////////////////////////////////////////////////////////////////

/*!
  \brief Load in a scene.

  This function is called to load in a scene given the URL of the
  scene.  If a scene is already open, it is first closed before the
  new one is opened.  The scene is started by calling
  Executive::Start().

  This function is called from CMmiApp::OpenDocumentFile() to open
  a file, and CmiDoc::OnRemoteOpen() to open a URL.

  \param lpszPathName  the URL of the scene to open.

  \return TRUE if the scene was loaded OK, otherwise FALSE.
*/

BOOL CMmiDoc::Restart(LPCTSTR lpszPathName) 
{
  TRACE("[Doc::Restart] begin: FileOpened %1d FileName %s \n", 
    m_fileOpened, lpszPathName);
  
  if (m_fileOpened)
    CloseDocument();
  
  if (lpszPathName) {
    
    TRACE("[Doc::Restart] navigation TRUE \n");
    m_bNavigation = TRUE;
    m_bFinished = FALSE;
    
    //		LPSTR szType = "file:";
    //		strcat(szType,lpszPathName);
    //		mpoExec->Start(szType);
    mpoExec->Start(lpszPathName);
    
    POSITION pos = GetFirstViewPosition();
    CMmiView* pFirstView = (CMmiView*)GetNextView( pos );
    if (pFirstView->m_nTimer == NULL)
      pFirstView->m_nTimer = pFirstView->SetTimer(1,40,NULL);
    
    UpdateAllViews(NULL);

	//ML - added this line because the core no longer uses
	//     Application::OnConnect()
	m_fileOpened = TRUE;

    TRACE("[Doc::Restart] returning with SUCCESS \n"); 
    return TRUE;
  }
  else {
    TRACE("[Doc::Restart] returning with FAILURE \n"); 
    return FALSE;
  }
}

/*******************************************************************************
DOC CLOSE DOCUMENT
*******************************************************************************/

void CMmiDoc::OnCloseDocument() 
{
  TRACE("[Doc::OnCloseDoc] begin \n");

  // TODO: Add your specialized code here and/or call the base class
	CloseDocument();
	CDocument::OnCloseDocument();

  TRACE("[Doc::OnCloseDoc] end \n");
}

void CMmiDoc::CloseDocument()
{
  TRACE("[Doc::CloseDoc] begin \n");

  // Only proceed a file is actually open
	if (m_fileOpened)
	{
		//To get the first view in the list of views:

		POSITION pos = GetFirstViewPosition();
		CMmiView* pFirstView = (CMmiView*)GetNextView( pos );
		m_smpFileOpen.V();
		BOOL bdead = pFirstView->KillTimer( pFirstView->m_nTimer );
		if (bdead)
			pFirstView->m_nTimer = NULL;

		// Stop the executive running this session
		mpoExec->Stop();
		m_fileOpened = FALSE;

		// Wait for the application to be notified that the file
		// has been closed
		m_smpFileOpen.P();
		m_smpFileOpen.V();

		//ML - added this line (to clear the screen)
		UpdateAllViews(NULL);
	}

  TRACE("[Doc::CloseDoc] end \n");
}

/*******************************************************************************
DOC 
*******************************************************************************/

void CMmiDoc::OnFileBIFSSimulate() 
{
	// TODO: Add your command handler code here
		m_bFinished = FALSE;
		UpdateAllViews(NULL);

}

void CMmiDoc::CreateExecutive(HWND hWnd)
{
  TRACE("[Doc::CreateExec] begin \n");

  m_pViewWnd = hWnd;

  // SB #### added creation of Application
  mpoApp = new OpenGLApplication(hWnd);

#ifdef _DEBUG
  FILE * poFileLog = fopen("ExecutiveLog.Txt", "w");
	mpoExec = new Executive3d(this, mpoApp, hWnd, 
    poFileLog, TRACE_PARSE);
#else
	mpoExec = new Executive3d(this, mpoApp, hWnd);
#endif // _DEBUG

  //ML - removed the following lines.  Now make Executive::Executive()
  // create the correct presenter.
/*
  // SB 99/02/24 hack: since Exec creates Pres its way,
  // I delete and re-create Pres here...
  if (mpoPres != NULL) delete mpoPres;
  mpoPres = new OpenGLPresenter(mpoExec, hWnd);

  mpoExec->SetPresenter(mpoPres);
*/
  //ML - added this line instead
  mpoPres = (OpenGLPresenter*)mpoExec->GetPresenter();

  TRACE("[Doc::CreateExec] end \n");
}

void CMmiDoc::DestroyExecutive(HWND /* hWnd */)
{
  TRACE("[Doc::DestroyExec] begin \n");

  delete mpoPres;

  if (mpoExec->GetTraceFile() != NULL)
    fclose(mpoExec->GetTraceFile());
  // delete mpoExec;

  delete mpoApp;

  TRACE("[Doc::DestroyExec] end \n");
}


//ML - it appears that the core code no longer calls this function to signal
//     that the scene has been loaded.

// SB @@@@ changed parameter
// void CMmiDoc::OnConnect(CONNECTION_RESULT result)
//ML - changed RESPONSE_PARAMETER to RESPONSE
void CMmiDoc::OnConnect(RESPONSE result)
{
	// if (result == RESPONSE_UNKNOWN_ERROR)	{
	if (result == RESPONSE_OK)	{
		m_bSequenceComplete = FALSE;
		m_fileOpened = TRUE;
		m_smpFileOpen.P();
	}
}

// SB @@@@ changed parameter
// void CMmiDoc::OnFinish(SERVICE_TERMINATION reason)
//ML - changed RESPONSE_PARAMETER to RESPONSE
void CMmiDoc::OnFinish(RESPONSE reason)
{
  TRACE("[Doc::OnFinish] begin \n");

	m_bFinished = TRUE;
	m_nReturn = reason;

  TRACE("[Doc::OnFinish] end \n");
}

void CMmiDoc::OnUpdateEditUndoMovement(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
  OpenGLVisualRenderer* pRenderer = 
    (OpenGLVisualRenderer*) 
    ((Presenter *) mpoExec->GetPresenter())->
    GetVisualRenderer();

	if (mpoExec->GetPresenter()->GetScene() != NULL) {
		if (pRenderer->AllowUndo()) {
			pCmdUI->Enable(TRUE);
		}
		else
			pCmdUI->Enable(FALSE);
	}
	else
			pCmdUI->Enable(FALSE);
	
}



void CMmiDoc::OnEditUndoMovement() 
{
	// TODO: Add your command handler code here
	// TODO: Add your command update UI handler code here
  OpenGLVisualRenderer* pRenderer = 
    (OpenGLVisualRenderer*) 
    ((Presenter *) mpoExec->GetPresenter())->
    GetVisualRenderer();

  pRenderer->UndoTransformation();
}

void CMmiDoc::OnError(NON_FATAL_ERROR error)
{
  switch (error) {
  case INVALID_STREAM_TYPE:
    COMPLAIN("Non-fatal error: Invalid stream type");
    break;

  case STREAM_TYPE_COLLISION:
    COMPLAIN("Non-fatal error: Stream type collision");
    break;

  case NO_BIFS_STREAM:
    COMPLAIN("Non-fatal error: No BIFS stream");
    break;

  case SCENE_PARSING_ERROR:
    COMPLAIN("Non-fatal error: Scene parsing error");
    break;

  case OD_PARSING_ERROR:
    COMPLAIN("Non-fatal error: OD parsing error");
    break;

  case OD_UPDATE_NOT_SUPPORTED:
    COMPLAIN("Non-fatal error: OD update not supported");
    break;

  case ODID_COLLISION:
    COMPLAIN("Non-fatal error: ODID collision");
    break;

  case UNKNOWN_ODID:
    COMPLAIN("Non-fatal error: Unknown ODID");
    break;

  case UPDATE_ESD_NOT_SUPPORTED:
    COMPLAIN("Non-fatal error: Update ESD not supported");
    break;

  case REMOVE_ESD_NOT_SUPPORTED:
    COMPLAIN("Non-fatal error: Remove ESD not supported");
    break;

  case UNKNOWN_DECODER_TYPE:
    COMPLAIN("Non-fatal error: Unknown decoder type");
    break;
    
  case INCOMPATIBLE_DECODER_TYPE:
    COMPLAIN("Non-fatal error: Incompatible decoder type");
    break;

  case NO_STREAM:
    COMPLAIN("Non-fatal error: No stream");
    break;

  case NO_STREAM_TYPE:
    COMPLAIN("Non-fatal error: No stream type");
    break;

  /*** SB #### commented out 99/02/19
  case REMOTE_OD_NOT_SUPPORTED:
    COMPLAIN("Non-fatal error: Remote OD not supported");
    break;
  ***/

  case UNKNOWN_URL_PROTOCOL:
    COMPLAIN("Non-fatal error: Unknown URL protocol");
    break;

  case BUFFER_TOO_SMALL:
    COMPLAIN("Non-fatal error: Buffer too small");
    break;
  }

}


/////////////////////////////////////////////////////////////////////////////

void CMmiDoc::OnFileClose() 
{
  TRACE("[Doc::OnFileClose] begin \n");

  // TODO: Add your command handler code here
	CloseDocument();

  TRACE("[Doc::OnFileClose] end \n");
}


/////////////////////////////////////////////////////////////////////////////

void CMmiDoc::OnRemoteOpen() 
{
	// TODO: Code fr Befehlsbehandlungsroutine hier einfgen
	m_RemoteDlg.PresetInput ("http://");
	if (m_RemoteDlg.DoModal() == IDOK) {
		Restart (m_RemoteDlg.GetInput());
	}	
}


/////////////////////////////////////////////////////////////////////////////

/*!
  \brief Update the File->Reload menu entry.

  This will disable the menu if there is no scene currently loaded.
*/

void CMmiDoc::OnUpdateFileReload(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_fileOpened);
}


/////////////////////////////////////////////////////////////////////////////

/*!
  \brief Reload the current scene.

*/

void CMmiDoc::OnFileReload() 
{
	if (m_fileOpened)
	{
		CWaitCursor cursor;
		Restart(GetPathName());
	}
}


/////////////////////////////////////////////////////////////////////////////

//ML - added this function

/*!
  \brief Show the contents of the WorldInfo node in the scene.
*/

void CMmiDoc::OnViewWorldInfo() 
{
	if (mpoExec == NULL)
		return;

	Presenter* pPresenter = (Presenter*)mpoExec->GetPresenter();
	if (pPresenter == NULL)
		return;

	const CString* pTitle = pPresenter->GetWorldInfoTitle();
	if (pTitle == NULL)
	{
		AfxMessageBox("The current scene does not contain a WorldInfo node.");
	}
	else
	{
		WorldInfoDlg dialog;
		dialog.m_title = *pTitle;

		const CString* pInfo = pPresenter->GetWorldInfoInfo();
		if (pInfo)
			dialog.m_info = *pInfo;

		dialog.DoModal();
	}
} /* CMmiDoc::OnViewWorldInfo */


/////////////////////////////////////////////////////////////////////////////
