// mmiView.cpp : implementation of the CMmiView class
//

#include "stdafx.h"
#include "OpenGl/OpenGl.h"

#include "mmi.h"

#include "mmiDoc.h"
#include "mmiView.h"

#include "Executive3d.h"

#include "Im1defs.h"
#include "opengl/base/OpenGLPresenter.h"
#include "opengl/base/OpenGLVisualRenderer.h"

// #include "bindable\Viewpoint.h"
//#include "Nodes.h"

//#include <GL/gl.h>
//#include <GL/glu.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


/////////////////////////////////////////////////////////////////////////////
// CMmiView

IMPLEMENT_DYNCREATE(CMmiView, CView)

BEGIN_MESSAGE_MAP(CMmiView, CView)
//{{AFX_MSG_MAP(CMmiView)
ON_WM_CREATE()
ON_WM_SIZE()
ON_WM_DESTROY()
ON_WM_ERASEBKGND()
ON_WM_TIMER()
ON_WM_KEYDOWN()
ON_WM_LBUTTONDOWN()
ON_WM_LBUTTONUP()
ON_WM_MOUSEMOVE()
ON_COMMAND(ID_EDIT_ZBUFFER, OnEditZbuffer)
ON_COMMAND(ID_EDIT_Cscreen, OnEDITCscreen)
ON_COMMAND(ID_EDIT_ABUFFER, OnEditAbuffer)
ON_COMMAND(ID_EDIT_HWSHADOW, OnEditHWShadow)
ON_WM_RBUTTONUP()
ON_WM_RBUTTONDOWN()
ON_COMMAND(ID_EDIT_SELECTION, OnEditSelection)
ON_COMMAND(ID_EDIT_SGRAPH, OnEditSgraph)
ON_COMMAND(ID_VIEW_BBOX, OnViewBbox)
ON_COMMAND(ID_VIEW_SGMEM, OnViewSgmem)
ON_COMMAND(ID_VIEW_WIREFRAME, OnViewWireframe)
ON_COMMAND(ID_EDIT_FASTTEXT, OnEditFasttext)
ON_COMMAND(ID_EDIT_NICETEXT, OnEditNicetext)
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMmiView construction/destruction

CMmiView::CMmiView() : 
m_bZbuffer(TRUE),
m_bSgraph(FALSE),
m_bCscreen(TRUE),
m_bSelection(FALSE),
m_bBBox(FALSE),
m_bWFrame(FALSE),
m_bAbuffer(FALSE),
m_bHWshadow(FALSE),
m_bSGMem(FALSE),
m_bFastTexture(FALSE),
m_bNiceTexture(FALSE),
m_ElapsedTime( ~(DWORD)0 ) 

{
  // TODO: add construction code here
  m_pDC=NULL;
  m_pOldPalette = NULL;
  modifierState = 0;
  userEvent = EVENT_LEFT_BUTTON_UP;
  TRACE("[CView::Ctor] \n");	
}

CMmiView::~CMmiView()
{
  TRACE("[CView::Dtor] \n");	
}

BOOL CMmiView::PreCreateWindow(CREATESTRUCT& cs)
{
  // TODO: Modify the Window class or styles here by modifying
  //  the CREATESTRUCT cs
		cs.cy = 100;
    cs.cx = 100;
    cs.style |= WS_CLIPSIBLINGS | WS_CLIPCHILDREN;
    
    return CView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////

/*!
  \brief Redraw the view.

  This function is called by the framework when this view needs to be
  redrawn.  The drawing of the scene is done by calling
  Executive3d::Resize().
*/

//ML - added the code to redraw the scene when the executive is not active.

void CMmiView::OnDraw(CDC* pDC)
{
	if (GetDocument()->mpoExec == NULL)
		return;

	// If the executive is currently not active, just redraw the scene.
	Executive3d* pExec = GetDocument()->mpoExec;
	if (pExec->IsActive())
		pExec->ReSize(m_nWidth, m_nHeight);	
	else
		pDC->FillSolidRect(0, 0, m_nWidth, m_nHeight, RGB(0,0,0));
}


/////////////////////////////////////////////////////////////////////////////
// CMmiView diagnostics

#ifdef _DEBUG
void CMmiView::AssertValid() const
{
  CView::AssertValid();
}

void CMmiView::Dump(CDumpContext& dc) const
{
  CView::Dump(dc);
}

CMmiDoc* CMmiView::GetDocument() // non-debug version is inline
{
  ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CMmiDoc)));
  return (CMmiDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CMmiView message handlers

/*******************************************************************************
  CREATE VIEW
*******************************************************************************/

int CMmiView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
  TRACE("[View::OnCreate] begin \n");

  if (CView::OnCreate(lpCreateStruct) == -1)
    return -1;
  
  // TODO: Add your specialized creation code here
  HGLRC hrc;
  m_pDC=new CClientDC(this);
  
  PIXELFORMATDESCRIPTOR pfd = {
    sizeof(PIXELFORMATDESCRIPTOR),  /* size */
      1,                              /* version */
      PFD_SUPPORT_OPENGL |
      PFD_DRAW_TO_WINDOW |
      PFD_DOUBLEBUFFER,               /* support double-buffering */
      PFD_TYPE_RGBA,                  /* color type */
      16,                             /* prefered color depth */
      0, 0, 0, 0, 0, 0,               /* color bits (ignored) */
      0,                              /* no alpha buffer */
      0,                              /* alpha bits (ignored) */
      0,                              /* no accumulation buffer */
      0, 0, 0, 0,                     /* accum bits (ignored) */
      16,                             /* depth buffer */
      0,                              /* no stencil buffer */
      0,                              /* no auxiliary buffers */
      PFD_MAIN_PLANE,                 /* main layer */
      0,                              /* reserved */
      0, 0, 0,                        /* no layer, visible, damage masks */
  };
  
  int pixelformat = ChoosePixelFormat(m_pDC->GetSafeHdc(), &pfd);
  
  if (pixelformat == 0) {
    MessageBox("Could not choose pixelformat\n");
    return -1;
  }
  
  if (SetPixelFormat(m_pDC->GetSafeHdc(), pixelformat, &pfd)==FALSE) {
    MessageBox("Could not set pixelformat\n");
    return -1;
  }
  
  ::DescribePixelFormat(m_pDC->GetSafeHdc(),
    pixelformat, pfd.nSize, &pfd);
  
  if (!(pfd.dwFlags & PFD_GENERIC_FORMAT))
    TRACE("We have a device index\n");
  else
    TRACE("We have a generic format\n");
  
  if (pfd.dwFlags & PFD_NEED_PALETTE)
    CreateRGBPalette();
  
  hrc = wglCreateContext(m_pDC->GetSafeHdc());
  
  if (!hrc) {
    MessageBox("Could not create OpenGL render context");
    return -1;
  }
  
  wglMakeCurrent(m_pDC->GetSafeHdc(), hrc);
  
  ConfigureOpenGL();
  
  //const unsigned char *vendor   = glGetString( GL_VENDOR );

  // Now we initialize the animation code
  m_StartTime = ::timeGetTime(); // get time in ms
  m_previousElapsedTime = m_StartTime;
  
  TRACE("[View::OnCreate] end \n");

  return 0;
}

void CMmiView::OnInitialUpdate() 
{
  TRACE("[View::OnInitialUpdate] begin \n");

  CView::OnInitialUpdate();
  
  ConfigureOpenGL();
  // TODO: Add your specialized code here and/or call the base class
  //The first time this is called we need to create the Executive

  TRACE("[View::OnInitialUpdate] before Doc::CreateExec, SetTimer \n");
  if (GetDocument()->mpoExec==NULL) {
    GetDocument()->CreateExecutive(m_hWnd);
    
    //Create timer to update the view regularly
    m_nTimer = SetTimer(1,40,NULL);
  }
  

  TRACE("[View::OnInitialUpdate] end \n");
}

/*******************************************************************************
  REPAINT VIEW every TIMER milliseconds
*******************************************************************************/

void CMmiView::OnTimer(UINT nIDEvent) 
{
  // TODO: Add your message handler code here and/or call default
  PaintScene();
  
  CView::OnTimer(nIDEvent);
  MSG msg;
  while(::PeekMessage(&msg,m_hWnd,WM_TIMER,WM_TIMER,PM_REMOVE));
}

void CMmiView::PaintScene()
{
  static bBusy=FALSE;
  PAINTSTRUCT ps;
  
  if (bBusy) {
    TRACE ("Busy\n");
    return;
  }
  bBusy=TRUE;
  DWORD timeBefore, timeAfter;
  timeBefore=GetTickCount();
  if (m_nTimer != NULL)
  {
    GetDocument()->mpoExec->Paint(&ps.rcPaint);
    // SB $$$$ CHANNELS !!!
    // GetDocument()->mpoExec->RequestChannels();
  }
  timeAfter=GetTickCount();
  // SB %%%%
  // TRACE("[View::PaintScene] Paint time: %d ms\n", (timeAfter - timeBefore));
  
  bBusy=FALSE;
  /*
  PAINTSTRUCT ps;
  GetDocument()->m_pExecutive->Paint(&ps.rcPaint);
  PostMessage(WM_PAINT,0,0);
  */
}

/*******************************************************************************
  DESTROY VIEW
*******************************************************************************/

void CMmiView::OnDestroy() 
{
  TRACE("[View::OnDestroy] begin \n");

  HGLRC	hrc;
  
  //Not a good solution but there is a problem with Executive Stop fn.
  TRACE("[View::OnDestroy] before KillTimer, Doc::DestroyExec \n");

  KillTimer(m_nTimer);
  GetDocument()->m_bFinished = TRUE;
  GetDocument()->Finish(0);
  
  if (GetDocument()->mpoExec != NULL) {
    GetDocument()->DestroyExecutive(m_hWnd);
  }

  TRACE("[View::OnDestroy] after Doc::DestroyExec \n");

  hrc = ::wglGetCurrentContext();
  
  ::wglMakeCurrent(NULL,  NULL);
  
  if (hrc)
    ::wglDeleteContext(hrc);
  
  if (m_pOldPalette)
    m_pDC->SelectPalette(m_pOldPalette, FALSE);
  
  if (m_pDC)
    delete m_pDC;
  
  CView::OnDestroy();

  TRACE("[View::OnDestroy] end \n");
}

/*******************************************************************************
  KEY EVENTS
*******************************************************************************/

void CMmiView::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
  // TODO: Add your message handler code here and/or call default
  static float step=0.5f;
  //BOOL ctrl = 0x8000 & GetKeyState( VK_CONTROL );
  BOOL shift= 0x8000 & GetKeyState( VK_SHIFT );
  
  if (GetDocument()->m_bFinished) {
    CView::OnKeyDown(nChar, nRepCnt, nFlags);
    return;
  }
  
  OpenGLVisualRenderer* pRenderer = 
    (OpenGLVisualRenderer*) GetDocument()->mpoPres->GetVisualRenderer();
  Viewpoint* View = pRenderer->GetActiveViewPoint();
  
  if (View != NULL) {
    
    Vec3f WhereIAm = View->position /* GetPosition() */ ;
    Rotation WhereIAmLooking = View->orientation /* GetOrientation() */ ;
    
    switch ( nChar ) {
    case VK_UP: // up arrow
      WhereIAm.z+=step*(WhereIAmLooking.z*-cos(WhereIAmLooking.theta));
      WhereIAm.y+=step*(WhereIAmLooking.y * sin(WhereIAmLooking.theta));
      WhereIAm.x+=step*(WhereIAmLooking.x*-sin(WhereIAmLooking.theta));
      break;
    case VK_DOWN: // down arrow
      WhereIAm.z-=step*(WhereIAmLooking.z*-cos(WhereIAmLooking.theta));
      WhereIAm.y-=step*(WhereIAmLooking.y * sin(WhereIAmLooking.theta));
      WhereIAm.x-=step*(WhereIAmLooking.x*-sin(WhereIAmLooking.theta));
      break;
    case VK_LEFT: // left arrow
      if (shift) {
        WhereIAmLooking.theta+=step*10;
      }
      else 
        WhereIAm.x-=step;
      break;
    case VK_RIGHT: // right arrow
      if (shift) {
        WhereIAmLooking.theta-=step*10;
      }
      else
        WhereIAm.x+=step;
      break;
      
    case VK_ESCAPE:	// close down the app
      GetParent()->PostMessage(WM_CLOSE);
      return;
    default:
      return;
    }
    View->orientation = /* SetOrientation */ WhereIAmLooking;
    View->position = /* SetPosition */ WhereIAm;
    pRenderer->SetActiveViewpoint(View);
  }
}

/*******************************************************************************
  MOUSE EVENTS
*******************************************************************************/

void CMmiView::OnLButtonDown(UINT nFlags, CPoint point) 
{
  // TODO: Add your message handler code here and/or call default
  OpenGLVisualRenderer* pRenderer = 
    (OpenGLVisualRenderer*) GetDocument()->mpoPres->GetVisualRenderer();
  
  // TRACE("[View::LButtonDown] selection %1d navigation %1d \n", m_bSelection, GetDocument()->m_bNavigation);
  if (!m_bSelection)
  {
    SetCapture();
    userEvent = EVENT_LEFT_BUTTON_DOWN;
    if (GetDocument()->m_bNavigation)
    {
      // TRACE("[View::LButtonDown] point %4d %4d \n", point.x, point.y);
      // GetDocument()->m_pExecutive->OnMouseEvent(userEvent, point.x, point.y, modifierState);
      GetDocument()->mpoExec->OnUserEvent(userEvent, point.x, point.y, modifierState);
    }
  }
  else
  {
    pRenderer->LBDownAction(nFlags,point);
  }
  CView::OnLButtonDown(nFlags, point);
}

void CMmiView::OnLButtonUp(UINT nFlags, CPoint point) 
{
  // TODO: Add your message handler code here and/or call default
  OpenGLVisualRenderer* pRenderer = 
    (OpenGLVisualRenderer*) GetDocument()->mpoPres->GetVisualRenderer();
  
  // TRACE("[View::LButtonUp] selection %1d navigation %1d \n", m_bSelection, GetDocument()->m_bNavigation);
  if (!m_bSelection)
  {
    ReleaseCapture();
    userEvent = EVENT_LEFT_BUTTON_UP;
    if (GetDocument()->m_bNavigation)
    {
      // TRACE("[View::LButtonUp] point %4d %4d \n", point.x, point.y);
      // GetDocument()->m_pExecutive->OnMouseEvent(userEvent, point.x, point.y, modifierState);
      GetDocument()->mpoExec->OnUserEvent(userEvent, point.x, point.y, modifierState);
    }
  }
  else
  {
    pRenderer->LBUpAction(nFlags,point);
  }
  
  CView::OnLButtonUp(nFlags, point);
}

void CMmiView::OnRButtonUp(UINT nFlags, CPoint point) 
{
  // TODO: Add your message handler code here and/or call default
  
  ReleaseCapture();
  userEvent = EVENT_RIGHT_BUTTON_UP;
  // TRACE("[View::RButtonUp] navigation %1d \n", GetDocument()->m_bNavigation);
  if (GetDocument()->m_bNavigation)
  {
    // TRACE("[View::RButtonUp] point %4d %4d \n", point.x, point.y);
    // GetDocument()->m_pExecutive->OnMouseEvent(userEvent, point.x, point.y, modifierState);
    GetDocument()->mpoExec->OnUserEvent(userEvent, point.x, point.y, modifierState);
  }
  CView::OnRButtonUp(nFlags, point);
}

void CMmiView::OnRButtonDown(UINT nFlags, CPoint point) 
{
  // TODO: Add your message handler code here and/or call default
  
  SetCapture();
  userEvent = EVENT_RIGHT_BUTTON_DOWN;
  // TRACE("[View::RButtonDown] navigation %1d \n", GetDocument()->m_bNavigation);
  if (GetDocument()->m_bNavigation)
  {
    // TRACE("[View::RButtonDown] point %4d %4d \n", point.x, point.y);
    // GetDocument()->m_pExecutive->OnMouseEvent(userEvent, point.x, point.y, modifierState);
    GetDocument()->mpoExec->OnUserEvent(userEvent, point.x, point.y, modifierState);
  }
  CView::OnRButtonDown(nFlags, point);
}

void CMmiView::OnMouseMove(UINT nFlags, CPoint point) 
{
  CMmiDoc* pDoc = GetDocument();
  OpenGLVisualRenderer* pRenderer = 
    (OpenGLVisualRenderer*) pDoc->mpoPres->GetVisualRenderer();
  
  if (!m_bSelection) {
    if (pDoc->m_bNavigation && userEvent != NULL)
    {
      // pDoc->m_pExecutive->OnMouseEvent(userEvent, point.x, point.y, modifierState);
      pDoc->mpoExec->OnUserEvent(userEvent, point.x, point.y, modifierState);
    }
    pRenderer->MouseMovement(nFlags,point);
  }
  else {
    pRenderer->MouseMovement(nFlags,point);
  }
  
  CView::OnMouseMove(nFlags, point);
}

/*******************************************************************************
  OTHER VIEW STUFF
*******************************************************************************/

void CMmiView::CreateRGBPalette()
{
  PIXELFORMATDESCRIPTOR pfd;
  int n = GetPixelFormat(m_pDC->GetSafeHdc());
  DescribePixelFormat(m_pDC->GetSafeHdc(), n, sizeof(PIXELFORMATDESCRIPTOR), &pfd);
  
  LOGPALETTE* pPal = (LOGPALETTE*)malloc(sizeof(LOGPALETTE) + 256 * sizeof(PALETTEENTRY));
  pPal->palVersion = 0x300;
  pPal->palNumEntries = 256;
  
  ASSERT(pfd.cColorBits == 8);
  n = 1 << pfd.cColorBits;
  unsigned char byRedMask = (1 << pfd.cRedBits) - 1;
  unsigned char byGreenMask = (1 << pfd.cGreenBits) - 1;
  unsigned char byBlueMask = (1 << pfd.cBlueBits) - 1;
  for (int i = 0; i < n; ++i) {
    pPal->palPalEntry[i].peRed   = ((((i >> pfd.cRedShift) & byRedMask) * 255) / byRedMask);
    pPal->palPalEntry[i].peGreen = ((((i >> pfd.cGreenShift) & byGreenMask) * 255) / byGreenMask);
    pPal->palPalEntry[i].peBlue  = ((((i >> pfd.cBlueShift) & byBlueMask) * 255) / byBlueMask);
    pPal->palPalEntry[i].peFlags = 0;
  }
  m_cPalette.CreatePalette(pPal);
  free(pPal);
  
  m_pOldPalette = m_pDC->SelectPalette(&m_cPalette, FALSE);
  m_pDC->RealizePalette();
}

void CMmiView::ConfigureOpenGL()
{
#if defined(_DEBUG)
//  TRACE("[View::ConfigureOpenGl] %s\n%s\n%s\n%s\n",
//	   ::glGetString(GL_VENDOR),
//     ::glGetString(GL_RENDERER),
//     ::glGetString(GL_VERSION),
//     ::glGetString(GL_EXTENSIONS));
#endif
  
  ::glEnable(GL_NORMALIZE);
  
  ::glDepthFunc(GL_LEQUAL);
  ::glAlphaFunc(GL_GREATER, 0.0f);
  ::glEnable(GL_ALPHA_TEST);
  
#if defined(FASTER_PUSSYCAT)
  ::glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_FASTEST);
  ::glDisable(GL_DITHER);
#endif
}

void CMmiView::OnSize(UINT nType, int cx, int cy) 
{
  CView::OnSize(nType, cx, cy);
  
  // TODO: Add your message handler code here
  m_nWidth = cx;
  m_nHeight = cy;
  if (GetDocument()->mpoExec==NULL) return;
  
  GetDocument()->mpoExec->ReSize(cx, cy);	
  
}

BOOL CMmiView::OnEraseBkgnd(CDC* /*pDC*/) 
{
  return TRUE;
}

void CMmiView::OnEditZbuffer() 
{
  CMenu* viewmenu = AfxGetMainWnd()->GetMenu();
  viewmenu = viewmenu->GetSubMenu(1);

  OpenGLVisualRenderer* pRenderer = 
    (OpenGLVisualRenderer*) GetDocument()->mpoPres->GetVisualRenderer();
  
  if (m_bZbuffer) {
    m_bZbuffer = FALSE;
    viewmenu->CheckMenuItem(ID_EDIT_ZBUFFER, MF_UNCHECKED);
  }
  else {
    m_bZbuffer = TRUE;
    viewmenu->CheckMenuItem(ID_EDIT_ZBUFFER, MF_CHECKED);
  }
  pRenderer->UseZBuffer(m_bZbuffer);	
  
}

void CMmiView::OnEditAbuffer() 
{
  CMenu* viewmenu = AfxGetMainWnd()->GetMenu();
  viewmenu = viewmenu->GetSubMenu(1);

  OpenGLVisualRenderer* pRenderer = 
    (OpenGLVisualRenderer*) GetDocument()->mpoPres->GetVisualRenderer();
  
  if (m_bAbuffer) {
    m_bAbuffer = FALSE;
    viewmenu->CheckMenuItem(ID_EDIT_ABUFFER, MF_UNCHECKED);
  }
  else {
    m_bAbuffer = TRUE;
    viewmenu->CheckMenuItem(ID_EDIT_ABUFFER, MF_CHECKED);
  }
  pRenderer->UseABuffer(m_bAbuffer);	
  
}


void CMmiView::OnEditHWShadow() 
{
  CMenu* viewmenu = AfxGetMainWnd()->GetMenu();
  viewmenu = viewmenu->GetSubMenu(1);

  OpenGLVisualRenderer* pRenderer = 
    (OpenGLVisualRenderer*) GetDocument()->mpoPres->GetVisualRenderer();
  
  if (m_bHWshadow) {
    m_bHWshadow = FALSE;
    viewmenu->CheckMenuItem(ID_EDIT_HWSHADOW, MF_UNCHECKED);
  }
  else {
    m_bHWshadow = TRUE;
    viewmenu->CheckMenuItem(ID_EDIT_HWSHADOW, MF_CHECKED);
  }
  pRenderer->UseHWShadow(m_bHWshadow);	
  
}


void CMmiView::OnEDITCscreen() 
{
  // TODO: Add your command handler code here
  CMenu* viewmenu = AfxGetMainWnd()->GetMenu();
  viewmenu = viewmenu->GetSubMenu(1);

  OpenGLVisualRenderer* pRenderer = 
    (OpenGLVisualRenderer*) GetDocument()->mpoPres->GetVisualRenderer();
  
  if (m_bCscreen) {
    m_bCscreen = FALSE;
    viewmenu->CheckMenuItem(ID_EDIT_Cscreen, MF_UNCHECKED);
  }
  else {
    m_bCscreen = TRUE;
    viewmenu->CheckMenuItem(ID_EDIT_Cscreen, MF_CHECKED);
  }
  pRenderer->ClearScreen(m_bCscreen);	
  
}


void CMmiView::OnEditSelection() 
{
  // TODO: Add your command handler code here
  CMenu* viewmenu = AfxGetMainWnd()->GetMenu();
  viewmenu = viewmenu->GetSubMenu(1);

//  OpenGLVisualRenderer* pRenderer = 
//    (OpenGLVisualRenderer*) GetDocument()->mpoPres->GetVisualRenderer();
  
  if (m_bSelection) {
    m_bSelection = FALSE;
    viewmenu->CheckMenuItem(ID_EDIT_SELECTION, MF_UNCHECKED);
  }
  else {
    m_bSelection = TRUE;
    viewmenu->CheckMenuItem(ID_EDIT_SELECTION, MF_CHECKED);
  }
}

void CMmiView::OnEditSgraph() 
{
  // TODO: Add your command handler code here
  CMenu* viewmenu = AfxGetMainWnd()->GetMenu();
  viewmenu = viewmenu->GetSubMenu(2);

  OpenGLVisualRenderer* pRenderer = 
    (OpenGLVisualRenderer*) GetDocument()->mpoPres->GetVisualRenderer();
  
  if (m_bSgraph) {
    m_bSgraph = FALSE;
    viewmenu->CheckMenuItem(ID_EDIT_SGRAPH, MF_UNCHECKED);
  }
  else {
    if (m_bSGMem) {
      m_bSGMem = FALSE;
      viewmenu->CheckMenuItem(ID_VIEW_SGMEM, MF_UNCHECKED);
    }
    m_bSgraph = TRUE;
    viewmenu->CheckMenuItem(ID_EDIT_SGRAPH, MF_CHECKED);
  }
  pRenderer->DrawSceneGraph((int)m_bSgraph);	
}

//////////////////////////////////////////////////////////////////////////////
// FramesPerSecond
// fetch frame rate calculations
int CMmiView::FramesPerSecond( void )
{
  double eTime = ElapsedTimeinMSSinceLastRender();
  
  if ( 0 == (int)eTime )
    return 0;
  
  return (int)(1000/(int)eTime);
}

void CMmiView::OnViewBbox() 
{
  // TODO: Add your command handler code here
  CMenu* viewmenu = AfxGetMainWnd()->GetMenu();
  viewmenu = viewmenu->GetSubMenu(2);

  OpenGLVisualRenderer* pRenderer = 
    (OpenGLVisualRenderer*) GetDocument()->mpoPres->GetVisualRenderer();
  
  if (m_bBBox) {
    m_bBBox = FALSE;
    viewmenu->CheckMenuItem(ID_VIEW_BBOX, MF_UNCHECKED);
  }
  else {
    m_bBBox = TRUE;
    viewmenu->CheckMenuItem(ID_VIEW_BBOX, MF_CHECKED);
  }
  pRenderer->DrawBBoxes((int)m_bBBox);
  
}

void CMmiView::OnViewSgmem() 
{
  // TODO: Add your command handler code here
  CMenu* viewmenu = AfxGetMainWnd()->GetMenu();
  viewmenu = viewmenu->GetSubMenu(2);

  OpenGLVisualRenderer* pRenderer = 
    (OpenGLVisualRenderer*) GetDocument()->mpoPres->GetVisualRenderer();
  
  if (m_bSGMem) {
    m_bSGMem = FALSE;
    viewmenu->CheckMenuItem(ID_VIEW_SGMEM, MF_UNCHECKED);
    pRenderer->DrawSceneGraph(0);
  }
  else {
    if (m_bSgraph) {
      m_bSgraph = FALSE;
      viewmenu->CheckMenuItem(ID_EDIT_SGRAPH, MF_UNCHECKED);
    }
    m_bSGMem = TRUE;
    viewmenu->CheckMenuItem(ID_VIEW_SGMEM, MF_CHECKED);
    pRenderer->DrawSceneGraph(-1);
  }
  
}

void CMmiView::OnViewWireframe() 
{
  // TODO: Add your command handler code here
  CMenu* viewmenu = AfxGetMainWnd()->GetMenu();
  viewmenu = viewmenu->GetSubMenu(2);

//  OpenGLVisualRenderer* pRenderer = 
//    (OpenGLVisualRenderer*) GetDocument()->mpoPres->GetVisualRenderer();
  
  if (m_bWFrame) {
    m_bWFrame = FALSE;
    viewmenu->CheckMenuItem(ID_VIEW_WIREFRAME, MF_UNCHECKED);
    ::glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
    ::glEnable(GL_CULL_FACE);
  }
  else {
    m_bWFrame = TRUE;
    viewmenu->CheckMenuItem(ID_VIEW_WIREFRAME, MF_CHECKED);
    ::glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
    ::glDisable(GL_CULL_FACE);
  }
}

void CMmiView::OnEditFasttext() 
{
  // TODO: Add your command handler code here
  CMenu* viewmenu = AfxGetMainWnd()->GetMenu();
  viewmenu = viewmenu->GetSubMenu(1);

  OpenGLVisualRenderer* pRenderer = 
    (OpenGLVisualRenderer*) GetDocument()->mpoPres->GetVisualRenderer();
  
  if (m_bFastTexture) {
    m_bFastTexture = FALSE;
    viewmenu->CheckMenuItem(ID_EDIT_FASTTEXT, MF_UNCHECKED);
  }
  else {
    if (m_bNiceTexture) {
      m_bNiceTexture = FALSE;
      viewmenu->CheckMenuItem(ID_EDIT_NICETEXT, MF_UNCHECKED);
    }
    m_bFastTexture = TRUE;
    viewmenu->CheckMenuItem(ID_EDIT_FASTTEXT, MF_CHECKED);
    ::glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_FASTEST);
    pRenderer->SetTextureMagFunc(GL_NEAREST);
    pRenderer->SetTextureMinFunc(GL_NEAREST);
  }
}

void CMmiView::OnEditNicetext() 
{
  // TODO: Add your command handler code here
  CMenu* viewmenu = AfxGetMainWnd()->GetMenu();
  viewmenu = viewmenu->GetSubMenu(1);

  OpenGLVisualRenderer* pRenderer = 
    (OpenGLVisualRenderer*) GetDocument()->mpoPres->GetVisualRenderer();
  
  if (m_bNiceTexture) {
    m_bNiceTexture = FALSE;
    viewmenu->CheckMenuItem(ID_EDIT_NICETEXT, MF_UNCHECKED);
    ::glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_FASTEST);
    pRenderer->SetTextureMagFunc(GL_NEAREST);
    pRenderer->SetTextureMinFunc(GL_NEAREST);
  }
  else {
    if (m_bFastTexture) {
      m_bFastTexture = FALSE;
      viewmenu->CheckMenuItem(ID_EDIT_FASTTEXT, MF_UNCHECKED);
    }
    m_bNiceTexture = TRUE;
    viewmenu->CheckMenuItem(ID_EDIT_NICETEXT, MF_CHECKED);
    ::glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
    pRenderer->SetTextureMagFunc(GL_LINEAR);
    pRenderer->SetTextureMinFunc(GL_LINEAR);
  }
}
