/***********************************************************************
This software module was originally developed by Marius Preda
(INT ARTEMIS) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. INT retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003.
************************************************************************

For more information or to receive an up to date version of this module,
you may mail Matius Preda at marius.preda@int-evry.fr
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Multiplexer
FILE:       BBAStream.cpp
AUTHOR:     Marius Preda (INT)
OVERVIEW:   Implementation of the BBA Stream
-------------------------------------------------------------------------*/

#include "defs.h"
#include "BBAStream.h"
#include "../IM1Decoders/AFX/BBA/BBADecoderLib/BBADecoderLib.h"



/////////////////////////////////////////////////////////////////////////


BBAStream :: BBAStream (FILE *fStream, ESDesc *pDescriptor) :
	Stream (fStream, pDescriptor), m_bRAP (true),
	m_bNextRAP (true),
	m_lFrames (0),	
	m_bEOF(false),
	m_bIsSessionHeader(true),
	m_pAUBuffer(NULL),
	m_lAULength(0),
	m_lIndex(0)
{

	m_frameRate = 15; // this value should come from the BBA stream
	
	
	m_bbaDecoderLib.initStream(fStream);
	m_pAUBuffer = (unsigned char*)malloc(10000);
	int l,cm;
	m_bbaDecoderLib.bba_parsing(m_pAUBuffer, &l, 0,&cm);
	codingMethod=cm;
	m_lAULength=l;
	m_lIndex = 0;
}


BBAStream :: ~BBAStream ()
{
}

long BBAStream :: GetDTS (long)
{
	if (m_bEOF)
		return -1;
	m_lDTS = (long)((float)(m_lFrames * 1000) / m_frameRate);
	++m_lFrames;
	if (codingMethod) m_lFrames+=15;
	return m_lDTS;
}

long BBAStream :: GetCTS ()
{
	if (m_bEOF)
		return -1;
	m_lCTS = m_lDTS;
	return m_lCTS;
}

bool BBAStream :: IsRAP ()
{
	bool bResult = m_bRAP;
	m_bRAP = m_bNextRAP;	
	return bResult;
}

int BBAStream :: GetChar ()
{
	int l;
	int cm;
	if (m_bEOF)
		return -1;
	if(m_lIndex == m_lAULength) {
		if (m_bbaDecoderLib.bba_parsing(m_pAUBuffer, &l, 1, &cm)){
			m_bEOF = true;
		}
		int a=m_lFrames;
		m_lAULength=l;
		m_lIndex = 0;
		return -1;
	}
	if (m_bEOF) 
		return -1;
	int c = m_pAUBuffer[m_lIndex++];
	return c;
}



////////////////////////////////////////////////////////////////////////////////////////////////


static StreamFactory<BBAStream> BBAFactory("BBA");


