/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Multiplexer
FILE:       G723.cpp
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Implementation of G723Stream, the class that handles G.723
input streams.
-------------------------------------------------------------------------*/
#include "defs.h"
#include "G723.h"
#include "assert.h"
#include "odMp4.h"

/*-----------------------------------------------------------------
G723Stream constructor reads first character
-----------------------------------------------------------------*/
G723Stream :: G723Stream (FILE *fStream, ESDesc *pDescriptor) :
	Stream (fStream, pDescriptor)
{
	m_cNext = getc ();
	m_lTime = 0;

	// The stream data specifies the data rate of the audio, either
	// 5.3 or 6.3 Kbits/sec.  The data rate determines the frame size.
	m_nFrameSize = 20;
	if (pDescriptor -> streamData != (LPCSTR) NULL &&
		pDescriptor -> streamData [0] != 0
	)
	{
		double dRate = atof (pDescriptor -> streamData);
		if (dRate == 6.3)
			m_nFrameSize = 24;
		else if (dRate != 5.3)
			throw ExceptionInvalidData (
				"G723 data rate must be either 5.3 or 6.3.\n"
			);
	}
}

/*-----------------------------------------------------------------
Time stamps are increments of 30 ms
-----------------------------------------------------------------*/
long G723Stream :: GetDTS (long)
{
	if (m_cNext == EOF)
		return -1;
	m_lDTS = m_lTime;
	m_lTime += 30;
	m_nCount = 0;
	return m_lDTS;
}

/*-------------------------------------------------------------
Although every G723 unit is RAP, this needs to be changed in the broadcast
case so that only AUs which have CTS will be RAP
-------------------------------------------------------------*/
bool G723Stream :: IsRAP ()
{
	return true;
}

/*-----------------------------------------------------------------
GetChar returns a character and reads one char ahead.  The number
of characters in an AU is determined by the data rate, which is
given in the szStreamData field.
-----------------------------------------------------------------*/
int G723Stream :: GetChar ()
{
	if (m_nCount >= m_nFrameSize)
		return -1;
	++m_nCount;
	int c = m_cNext;
	m_cNext = getc ();
	assert (c != EOF);
	return c;
}

static StreamFactory<G723Stream> g723Factory ("G723");