/***********************************************************************
This software module was originally developed by Marcos Aviles
(Universidad Politecnica de Madrid, UPM) in the course of development of 
the MPEG-4 Systems (ISO/IEC 14496-1) standard. This software module is
an implementation of a part of one or more MPEG-4 Systems 
(ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems
(ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 Systems 
(ISO/IEC 14496-1) free license to this software module or modifications
thereof for use in hardware or software products claiming conformance to
the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use this 
software module in hardware or software products are advised that its 
use may infringe existing patents. The original developer of this 
software module and his/her company, the subsequent editors and their
companies, and ISO/IEC have no liability for use of this software module
or modifications thereof in an implementation. Copyright is not 
released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
UPM retains full right to use the code for his/her own purpose, assign 
or donate the code to a third party and to inhibit third parties from 
using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
products. This copyright notice must be included in all copies or 
derivative works. Copyright  2006
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Marcos Aviles at mar@gti.ssr.upm.es.
----------------------------------------------------------------------*/

#include "defs.h"
#include "JP2Stream.h"

JP2Stream::JP2Stream(FILE *fStream, ESDesc *pDescriptor)
    : Stream (fStream, pDescriptor), m_bRAP(true), m_constant(TRUE), m_nextTime(0), m_intervalTime(1)
{

}

/*--------------------------------------------------------------
Time stamped BIFS file are made of access units, each preceded 
by 4 bytes of time stamp and 1 byte of AU length.
--------------------------------------------------------------*/
long JP2Stream::GetDTS(long)
{
    float fTime = 0;
    
    // Return -1 if the end of the file has been reached
    if (m_lDTS == -1)
        return -1;
    
    if (m_constant)
    {
        fTime = m_nextTime;
        m_nextTime += m_intervalTime;
    }
    
    // Convert the time to milliseconds
    return (m_lDTS = static_cast<long>(fTime * 1000.0f));
}

/*--------------------------------------------------------------
In this version only the first AU is RAP
--------------------------------------------------------------*/
bool JP2Stream::IsRAP()
{
    bool bResult = m_bRAP;
    m_bRAP = false;
    return bResult;
}

/*--------------------------------------------------------------
GetChar returns a character and read one char ahead
--------------------------------------------------------------*/
int JP2Stream::GetChar()
{
    int ch = getc();
    if (ch == -1)
        m_lDTS = -1;
    
    return ch;
}

static StreamFactory<JP2Stream> jp2Factory("JPEG2000");
