/***********************************************************************
This software module was originally developed by Ioan Alexandru Salomie
(Vrije Universiteit Brussel VUB) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. VUB retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Ioan Alexandru Salomie at iasalomi@etro.vub.ac.be.
----------------------------------------------------------------------*/

#include "defs.h"
#include "MG.h"

/*-----------------------------------------------------------------
MGStream constructor
-----------------------------------------------------------------*/
MGStream :: MGStream (FILE *fStream, ESDesc *pDescriptor) :
	Stream (fStream, pDescriptor), m_bRAP (true)
{
	m_prevChar[0] = 0;
	m_prevChar[1] = 0;
	m_readAhead = FALSE;

	//This variables should be used in further versions (when 1 MG file != 1 AU)
	m_constant = TRUE;
	m_nextTime = 0;
	m_intervalTime = 1;
}

MGStream :: ~MGStream ()
{
}

/*-----------------------------------------------------------------
Time stamps are either calculated using a fixed increment, or they
are read from a file.
-----------------------------------------------------------------*/
long MGStream :: GetDTS ()
{
	float fTime = 0;

	// Return -1 if the end of the file has been reached
	if (m_prevChar[0] == EOF)
		return -1;

	if (m_constant)
	{
		fTime = m_nextTime;
		m_nextTime += m_intervalTime;
	}

	// Convert the time to milliseconds
	return (m_lDTS = (long)(fTime * 1000.0));
}

/*-------------------------------------------------------------
Only the first AU is RAP
-------------------------------------------------------------*/
bool MGStream :: IsRAP ()
{
	bool bResult = m_bRAP;
	m_bRAP = false;
	return bResult;
}

/*-----------------------------------------------------------------
GetChar returns a character and reads one next char. We are always
3 chars ahead, which are used to check whether this is end of AU.
-----------------------------------------------------------------*/
int MGStream :: GetChar ()
{
	// If the previous two characters read were the end code of a
	// picture, then return -1.
	if ((m_prevChar[1] == 0xFF) && (m_prevChar[0] == 0xD9))
	{
		m_prevChar[0] = 0; 
		m_prevChar[1] = 0;

		// Before returning -1, we need to check if reading the
		// next byte while reach the end of the file
		m_nextChar = getc();
		m_readAhead = TRUE;
		if (m_nextChar == EOF)
			m_prevChar[0] = m_nextChar;

		return -1;
	}

	// Otherwise read in a new character, and update the list
	// of the past two characters read.
	int c = (m_readAhead) ? m_nextChar : getc();
	m_readAhead= FALSE;
	m_prevChar[1] = m_prevChar[0];
	m_prevChar[0] = c;

	return c;
}

static StreamFactory<MGStream> mgFactory ("MeshGrid");