/***********************************************************************
This software module was originally developed by Jean Le Feuvre
(avipix, llc) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. avipix, llc retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2000.
************************************************************************/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  MP4 File Writer
FILE:       MP4Dump.cpp
AUTHOR:     Jean Le Feuvre (avipix, llp, jean@avipix.com), December 2000
OVERVIEW:   the MP4 File Dumper
-------------------------------------------------------------------------*/

#include "MP4Dump.h"
// SOLARIS PORTING - defining the _MAX_PATH
#ifndef WIN32

#ifndef _MAX_PATH
#define _MAX_PATH PATH_MAX
#endif

#endif

MP4Dump::MP4Dump(ZString FileName, BOOL dumpRequest, BOOL forceDump)
{
	m_sFileName = FileName;
	m_bDump = dumpRequest;
	m_bForceDump = forceDump;

}

MP4Dump::~MP4Dump()
{

}

void MP4Dump::MP4ErrToText(MP4Err e)
{
	switch (e) {
	case MP4FileNotFoundErr:
		printf("Error: File not found\n");
		return;
	case MP4BadParamErr:
		printf("Error: Bad Parameter\n");
		return;
	case MP4NoMemoryErr:
		printf("Error: Out of memory\n");
		return;
	case MP4IOErr:
		printf("Error: Input/Output exception\n");
		return;
	case MP4NoLargeAtomSupportErr:
		printf("Error: No Large Atom Support in this lib\n");
		return;
	case MP4BadDataErr:
		printf("Error: corrupted file\n");
		return;
	case MP4VersionNotSupportedErr:
		printf("Error: File Version Not Supported\n");
		return;
	case MP4InvalidMediaErr:
		printf("Error: Invalid Media found\n");
		return;
	case MP4DataEntryTypeNotSupportedErr:
		printf("Error: Media Entry Type not supported\n");
		return;
	case MP4NoQTAtomErr:
		printf("Error: Non-compliant Atom found\n");
		return;
	case MP4NotImplementedErr:
		printf("Error: Not implemented in this lib\n");
		return;
	}
}



void MP4Dump::Dump()
{
	MP4Err e = MP4NoErr;
	u8 ODPL, ScenePL, AudioPL, VisualPL, GraphicsPL;
	u64 duration;
	u32 timeScale, nbTracks, ref, streamType;
	int i = 0, j = 0;
	MP4Handle urlH, urnH;
	MP4NewHandle(0, &urlH);
	MP4NewHandle(0, &urnH);

	//first create the trace file
	int nNameLength = m_sFileName.GetLength ();
	LPCSTR szExt = strrchr (m_sFileName, '.');

	//if no ext, add .mp4 (default)
	if (szExt && !strchr (szExt, '\\')) {
		nNameLength -= strlen (szExt);
	} else {
		m_sFileName += ".mp4";
	}
	//open our file
	e = MP4OpenMovieFile(&m_pMovie, (LPCSTR)m_sFileName, MP4OpenMovieNormal);
	if (e) {
		MP4ErrToText(e);
		return;
	}

	m_sFileName.Cut (nNameLength);

	szExt = strrchr(m_sFileName, '\\');
	if (szExt) {
		m_sFileName.InsertAt(strlen(m_sFileName) - strlen(szExt) + 1, "M4D_");
		m_sTraceName = m_sFileName;
	} else {
		m_sTraceName = "M4D_";
		m_sTraceName += m_sFileName;
	}
	m_sTraceName += ".trc";

	m_fTrace = fopen(m_sTraceName, "wt");
	if (m_fTrace == NULL) {
		printf("Error opening trace file %s...\n", m_sTraceName);
		goto bail;
	}
	
	//some movie info
	e = MP4GetMovieProfilesAndLevels(m_pMovie, &ODPL, &ScenePL, &AudioPL, &VisualPL, &GraphicsPL);
	switch (e) {
	case MP4HasRootOD:
		fprintf(m_fTrace, "This file is an exchange file (no IOD)\n\n");
		break;
	case MP4NoErr:
		//Profiles and Levels
		fprintf(m_fTrace, "This file is a presentation - Profiles and Levels:\n");
		fprintf(m_fTrace, "\tOD profile = %d\n", ODPL);
		fprintf(m_fTrace, "\tScene profile = %d\n", ScenePL);
		fprintf(m_fTrace, "\tAudio profile = %d\n", AudioPL);
		fprintf(m_fTrace, "\tVisual profile = %d\n", VisualPL);
		fprintf(m_fTrace, "\tGraphics profile = %d\n\n", GraphicsPL);
		break;
	default:
		goto bail;
	}

	//Movie Time
	e = MP4GetMovieDuration(m_pMovie, &duration);	if (e) goto bail;
	e = MP4GetMovieTimeScale(m_pMovie, &timeScale);	if (e) goto bail;
	fprintf (m_fTrace, "Movie Timing: Time scale %d - Duration %d \n", timeScale, (u32) duration);

	e = MP4GetMovieTrackCount(m_pMovie, &nbTracks);	if (e) goto bail;
	fprintf (m_fTrace, "Number Of Tracks: %d\n", nbTracks);


	//Track specific Infos
	for (i = 1 ; i <= nbTracks ; i++ ) {
		u32 id = 0, mediaTScale = 0, nbDataRef = 0, idRef = 0;
		u32 nbSamp = 0;
		u32 refType, refAtt;
		MP4Track track;
		MP4Media media;
		u64 mediaDuration = 0;

		printf("Reading track %d...\n", i);
		//Track ID and Time stuff
		e = MP4GetMovieIndTrack(m_pMovie, i, &track);	if (e) goto bail;
		e = MP4GetTrackID(track, &id);	if (e) goto bail;
		fprintf(m_fTrace, "\n************ Track #%d - ES_ID %d ************\n", i, id);

		e = MP4GetTrackDuration(track, &duration);	if (e) goto bail;
		fprintf(m_fTrace, "\tDuration: %d\n", (u32) duration);

		e = MP4GetTrackEnabled(track, &timeScale);	if (e) goto bail;
		if (timeScale) {
			fprintf(m_fTrace, "\tThis track is enabled\n\n");
		} else {
			fprintf(m_fTrace, "\tThis track is disabled\n\n");
		}

		//Track Stream References
		e = MP4GetTrackReferenceCount(track, MP4StreamDependencyReferenceType, &ref);	if (e) goto bail;
		fprintf(m_fTrace, "\tStream Dependency References: #%d", ref);
		for (j = 0 ; j < ref ; j++) {
			MP4Track tmpTrack;
			e = MP4GetTrackReference(track, MP4StreamDependencyReferenceType, j+1, &tmpTrack);	if (e) goto bail;
			e = MP4GetTrackID(tmpTrack, &idRef);	if (e) goto bail;
			fprintf(m_fTrace, " - Track ID %d", idRef);
		}
		fprintf(m_fTrace, "\n");
		
		//Track OD References
		e = MP4GetTrackReferenceCount(track, MP4ODTrackReferenceType, &ref);	if (e) goto bail;
		fprintf(m_fTrace, "\tOD Track References: #%d", ref);
		for (j = 0 ; j < ref ; j++) {
			MP4Track tmpTrack;
			e = MP4GetTrackReference(track, MP4ODTrackReferenceType, j+1, &tmpTrack);	if (e) goto bail;
			e = MP4GetTrackID(tmpTrack, &idRef);	if (e) goto bail;
			fprintf(m_fTrace, " - Track ID %d", idRef);
		}
		fprintf(m_fTrace, "\n");

		//Track Sync References
		e = MP4GetTrackReferenceCount(track, MP4SyncTrackReferenceType, &ref);	if (e) goto bail;
		fprintf(m_fTrace, "\tOCR References: #%d", ref);
		for (j = 0 ; j < ref ; j++) {
			MP4Track tmpTrack;
			e = MP4GetTrackReference(track, MP4SyncTrackReferenceType, j+1, &tmpTrack);	if (e) goto bail;
			e = MP4GetTrackID(tmpTrack, &idRef);	if (e) goto bail;
			fprintf(m_fTrace, " - Track ID %d", idRef);
		}
		fprintf(m_fTrace, "\n");

		//Track Hint References
		e = MP4GetTrackReferenceCount(track, MP4HintTrackReferenceType, &ref);	if (e) goto bail;
		fprintf(m_fTrace, "\tHint Track References: #%d", ref);
		for (j = 0 ; j < ref ; j++) {
			MP4Track tmpTrack;
			e = MP4GetTrackReference(track, MP4HintTrackReferenceType, j+1, &tmpTrack);	if (e) goto bail;
			e = MP4GetTrackID(tmpTrack, &idRef);	if (e) goto bail;
			fprintf(m_fTrace, " - Track ID %d", idRef);
		}
		fprintf(m_fTrace, "\n");




		//Checking the Media
		fprintf(m_fTrace, "\n\n\tMedia Info:\n");
	

		e = MP4GetTrackMedia(track, &media);	if (e) goto bail;
		e = MP4GetMediaDuration(media, &mediaDuration);	if (e) goto bail;
		e = MP4GetMediaTimeScale(media, &mediaTScale);	if (e) goto bail;
		e = MP4GetMediaSampleCount(media, &nbSamp);	if (e) goto bail;
		e = MP4GetMediaDataRefCount(media, &nbDataRef);	if (e) goto bail;

		//Media description
		fprintf(m_fTrace, "\t\tDuration: %d\n", (u32) mediaDuration);
		fprintf(m_fTrace, "\t\tTime Scale: %d\n", mediaTScale);
		fprintf(m_fTrace, "\t\tNb Samples: %d\n", nbSamp);
		fprintf(m_fTrace, "\t\tNb Datas References: %d\n", nbDataRef);

		for (j = 0 ; j < nbDataRef ; j++ ) {
			e = MP4GetMediaDataReference(media, j+1, urlH, urnH, &refType, &refAtt); if (e) goto bail;
			fprintf(m_fTrace, "\t\t\tRef #%d: ", j+1);
			if (refAtt & MP4DataRefSelfReferenceMask) {
				fprintf(m_fTrace, "self-contained data reference\n");
			} else if (refType == MP4URLDataReferenceType) {
				fprintf(m_fTrace, "URL: %s\n", *urlH);
			} else if (refType == MP4URNDataReferenceType) {
				MP4GetHandleSize(urlH, &refAtt);
				if (refAtt) {
					fprintf(m_fTrace, "URL: %s - ", *urlH);
				}
				fprintf(m_fTrace, "URN: %s\n", *urnH);
			}
			//check the data ref
			e = MP4CheckMediaDataRef(media, j+1);
			if (e == MP4FileNotFoundErr) {
				fprintf(m_fTrace, "\t\t\tReferenced file not found!\n");
			} else if (e != MP4NoErr) {
				goto bail;
			}
		}

		//here we work blindly till err...
		j = 0;
		while (1) {
			u32 strType = 0, objType = 0, bufSize = 0, upStream = 0, maxRate = 0, avgRate = 0, decSize = 0;
			MP4SetHandleSize(urlH, 0);
			e = MP4GetMediaDecoderInformation(media, j+1, &objType, &strType, &bufSize, &upStream, &maxRate, &avgRate, urlH);
			if (e == MP4BadParamErr) break;
			if (e) goto bail;
			streamType = strType;
			MP4GetHandleSize(urlH, &decSize);
			fprintf(m_fTrace, "\n\t\tSample Description #%d :\n\t\t\tObject Type Indication: %d\n\t\t\tStream Type: %d\n\t\t\tBuffer Size: %d\n\t\t\tUpstream Flag: %d\n\t\t\tMaximum Bitrate: %d\n\t\t\tAverage Bitrate: %d\n\t\t\tDecoder Specific Info size: %d\n"
				 , j+1
				 , objType
				 , strType
				 , bufSize
				 , upStream
				 , maxRate
				 , avgRate
				 , decSize);
			j++;
		}
		e = MP4NoErr;

		//Do we have to dump our track ??
		if (m_bDump) {
			DumpTrack(track, streamType);
		}
		
	}


bail:
	fclose(m_fTrace);
	MP4ErrToText(e);
	MP4DisposeHandle(urlH);
	MP4DisposeHandle(urnH);
	MP4DisposeMovie(m_pMovie);
}


//streamType is used to format TS system streams
void MP4Dump::DumpTrack(MP4Track track, unsigned int streamType)
{
	MP4Err e = MP4NoErr;
	MP4TrackReader reader;
	u32 trackID;
	FILE *fTrack;
	FILE *fAU;
	char sFile[_MAX_PATH];
	MP4Handle AU;
	int iNbAU = 0;
	u32 Size, SampleFlags, CTS, DTS, duration;

	MP4GetTrackID(track, &trackID);

	//get the names if requested
	sFile[0] = 0;
	if (!m_bForceDump) {
		printf("Enter a file name for exported track %d (type d for default name, s to skip)\n", trackID);
		scanf("%s", sFile);
	}
	//d = default
	if (!strcmp(sFile, "") || !strcmp(sFile, "d") ) {
		sprintf(sFile, "M4D_%s_%d.raw", (LPCSTR) m_sFileName, trackID);
	}
	//s = skip
	if (!strcmp(sFile, "s")) {
		fTrack = NULL;
	} else {
		if ((fTrack = fopen (sFile, "wb")) == NULL) {
			printf("Error opening media file %s\n", sFile);
			return;
		}
	}
	BOOL bUseBifsFormat = FALSE;
	switch (streamType) {
		//BIFS, OD and IPMP streams are extracted with TimeStamp and AUSize to be reused...
	case 0x01:
	case 0x03:
	case 0x07:
		bUseBifsFormat = TRUE;
		break;
	default:
		printf("Export track using System format? y:n\n");
		scanf("%s", sFile);
		if (!strcmp(sFile, "y"))
			bUseBifsFormat = TRUE;
		break;
	}

	sFile[0] = 0;
	if (!m_bForceDump) {
		printf("Enter a file name for track %d timing (type d for default name, s to skip)\n", trackID);
		scanf("%s", sFile);
	}
	if (!strcmp(sFile, "") || !strcmp(sFile, "d") ) {
		sprintf(sFile, "M4D_%s_%d.aud", (LPCSTR) m_sFileName, trackID);
	}
	//s = skip
	if (!strcmp(sFile, "s")) {
		fAU = NULL;
	} else {
		if ((fAU = fopen (sFile, "wt")) == NULL) {
			printf("Error opening description file %s\n", sFile);
			return;
		}
	}
	e = MP4CreateTrackReader( track, &reader );	if (e) goto bail;
	e = MP4NewHandle(0, &AU);	if (e) goto bail;

	while (!e) {
		e = MP4TrackReaderGetNextAccessUnitWithDuration(reader, AU, &Size, &SampleFlags, &CTS, &DTS, &duration);
		if (!e) {
			iNbAU++;
			if (fTrack) {
				if (bUseBifsFormat) {
					//the macros are in defs.h
					//first write the TimeStamp
					bool RAP = ( (SampleFlags & MP4MediaSampleNotSync) == 0 ) ? 1 : 0;
					char c = (RAP ? 1 : 0)<<7 | (HIBYTE (HIWORD (DTS)) & 0x7F);
					fputc (c, fTrack);
					fputc (LOBYTE (HIWORD (DTS)), fTrack);
					fputc (HIBYTE (LOWORD (DTS)), fTrack);
					fputc (LOBYTE (LOWORD (DTS)), fTrack);
					//then the AULength
					fputc (HIBYTE (HIWORD (Size)), fTrack);
					fputc (LOBYTE (HIWORD (Size)), fTrack);
					fputc (HIBYTE (LOWORD (Size)), fTrack);
					fputc (LOBYTE (LOWORD (Size)), fTrack);
				}
				fwrite( (void *) *AU, Size, 1, fTrack);
			}
			if ( (SampleFlags & MP4MediaSampleNotSync) == 0) {
				SampleFlags = 1;
			} else {
				SampleFlags = 0;
			}
			if (fAU)
				fprintf( fAU
					, "AU #%d: Size %d Bytes - RAP: %s - CTS %d - DTS %d - Duration %d\n"
					, iNbAU
					, Size
					, ( SampleFlags ? "Yes" : "No" )
					, CTS
					, DTS
					, duration
					);
		}
	}

bail:
	if (fTrack)	fclose(fTrack);
	if (fAU) fclose(fAU);
	MP4DisposeHandle(AU);
	MP4DisposeTrackReader(reader);
	return;
}
