/***********************************************************************
This software module was originally developed by Jean Le Feuvre
(avipix, llc) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. avipix, llc retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2000.
************************************************************************/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  MP4 File Writer
FILE:       MP4Enc.cpp
AUTHOR:     Jean Le Feuvre (avipix, llp, jean@avipix.com), December 2000
OVERVIEW:   parsing of the script file, movie writing
-------------------------------------------------------------------------*/

#include "defs.h"
#include "MP4Enc.h"
#include "Token.h"
#include "MuxODEnc.h"
// SOLARIS PORTING - Comment the next line
//#include "ODIPMPX.h"
#include "IPMP_Messages.h"
#include "CrSdl.h"

//a quick macro to put a desc in a MP4Handle
#define DESC_TO_HANDLE \
		coder.Align ();	\
		int idescSize = coder.GetDataLength();	\
		err = MP4SetHandleSize(descriptorH, idescSize);	\
		if (err) return;	\
		BYTE *p = (BYTE*) *descriptorH;	\
		BYTE *buffer = (BYTE *) calloc(idescSize, 1);		\
		memcpy(buffer, coder.GetData(), idescSize);	\
		for (int z=0; z < idescSize; z++) {		\
			*p++ = buffer[z];				\
		}									\
		free(buffer);

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

MP4Enc::MP4Enc(LPCSTR stMP4Name, BOOL bInterleaved, BOOL bMaximize)
{
	m_stMP4Movie = (char *) stMP4Name;
	m_pMovie = NULL;
	m_bInterleaved = bInterleaved;
	m_bMaximize = bMaximize;
}

MP4Enc::~MP4Enc()
{

}

/*-----------------------------------------------
Parses the script file
-------------------------------------------------*/
void MP4Enc:: Start (FILE *fList, int flags, FILE *trace)
{

	MuxODEncoder odEncoder (NULL, fList, "MP4", TRACE_NONE);

	// We separate ODs and IODs to insert the IOD streams in the movie's IOD
	SDLClassArray<ObjectDescriptor, 1, 255> ods;
	SDLClassArray<InitialObjectDescriptor, 1, 255> IODs;

	MP4Err err;
	MP4Handle descriptorH;
	BOOL bError = FALSE;
	int i, j;
	ZArray<int> esIDs;

	/*-------------------------------------------------------------
	ES_Descriptor is overloaded with a special version used by the Mux
	-------------------------------------------------------------*/
	OVERLOAD_OD_CLASS (ES_DescriptorMP4);

	SDLClassArray<ES_DescriptorMP4, 0, 255> esds;

	try
	{
		/*-------------------------------------------------------------
		Parse the initial object descritpor, if exists, and encode
		it at the beginning of the file.
		-------------------------------------------------------------*/

		if (stricmp (nextToken.GetString (), "InitialObjectDescriptor") == 0)
 		// SOLARIS PORTING - added args
 		//{	SDLClass<InitialObjectDescriptor> IOD;
 		{	SDLClass<InitialObjectDescriptor, 1> IOD;
			IOD.Scan ();
			if (IOD != NULL_DESC)
			{
				IODs += IOD;
				IOD = NULL_DESC;
			}
		}

		/*-----------------------------------------------------------------
		Parse the rest of the object descriptors
		-----------------------------------------------------------------*/
		while (nextToken != TID_EOF)
		{
 			// SOLARIS PORTING - added args
 			//SDLClass<ObjectDescriptor> od;
 			SDLClass<ObjectDescriptor, 1> od;
			od.Scan ();
			if (od != NULL_DESC)
			{	ods += od;
				od = NULL_DESC;
			}
		}
	}
	catch (...)
	{
		printf ("Script file error. Hit enter to continue\x7\n");
		getchar ();
		return;
	}

	if (IODs.GetLength() > 1) {
		printf ("Error: only one IOD must be specified for a Presentation File.\nHit enter to continue\x7\n");
		getchar();
		return;
	}

	//We still need the coder to encode the other desc...
	ODCoder coder (NULL, NULL, "VariousDesc", TRACE_NONE, 0);
	MP4NewHandle(0, &descriptorH);

	//we have an IOD, create presentation file
	if (IODs.GetLength()) {
		isExchangeFile = 0;
		InitialObjectDescriptor *IOD = IODs [0];
		err = NewMovie(IOD->ObjectDescriptorID,
								IOD->ODProfileLevelIndication,
								IOD->sceneProfileLevelIndication,
								IOD->audioProfileLevelIndication,
								IOD->visualProfileLevelIndication,
								IOD->graphicsProfileLevelIndication,
								(char *) (LPCSTR) IOD->URLstring,
								isExchangeFile);
		if (err) return;
		if (trace)
			fprintf(trace, "Creating Presentation MP4 file...\n");

		//Add all OCIdescriptorPtr to our movie
		for (i = 0; i < IOD->ociDescr.GetLength(); i++) {
			coder.Reset();
			IOD->ociDescr[i]->EncDec(&coder, CALC_LENGTH, NULL);
			IOD->ociDescr[i]->EncDec(&coder, ENCODE, NULL);
			DESC_TO_HANDLE;
			if (MP4AddDescToMovieIOD(m_pMovie, descriptorH)) {
				MP4DisposeHandle(descriptorH);
				return;
			}
		}

		//Add all IPMPdescriptorPtr to our movie
		for (i = 0; i < IOD->ipmpDescrPtr.GetLength(); i++) {
			coder.Reset();
			IOD->ipmpDescrPtr[i]->EncDec(&coder, CALC_LENGTH, NULL);
			IOD->ipmpDescrPtr[i]->EncDec(&coder, ENCODE, NULL);
			DESC_TO_HANDLE;
			if (MP4AddDescToMovieIOD(m_pMovie, descriptorH)) {
				MP4DisposeHandle(descriptorH);
				return;
			}
		}

		/************ Add by Liao Xuan ******************************/
		/* according to ISO/IEC 14496-1:2001/FDAM 3:2003            */     
		//Add all IPMP_Descriptor to our movie
		for (i = 0; i < IOD->ipmpDescr.GetLength(); i++) {
			coder.Reset();
			IOD->ipmpDescr[i]->EncDec(&coder, CALC_LENGTH, NULL);
			IOD->ipmpDescr[i]->EncDec(&coder, ENCODE, NULL);
			DESC_TO_HANDLE;
			if (MP4AddDescToMovieIOD(m_pMovie, descriptorH)) {
				MP4DisposeHandle(descriptorH);
				return;
			}
		}
		//Add all IPMP_ToolListDescriptor to our movie
		if (IOD->toolListDescr)
		{
			coder.Reset();
			IOD->toolListDescr->EncDec(&coder, CALC_LENGTH, NULL);
			IOD->toolListDescr->EncDec(&coder, ENCODE, NULL);
			DESC_TO_HANDLE;
			if (MP4AddDescToMovieIOD(m_pMovie, descriptorH)) {
				MP4DisposeHandle(descriptorH);
				return;
			}
		}
		/************ End: Add by Liao Xuan ******************************/

		//Add all EXTdescriptorPtr to our movie
		for (i = 0; i < IOD->extDescr.GetLength(); i++) {
			coder.Reset();
			IOD->extDescr[i]->EncDec(&coder, CALC_LENGTH, NULL);
			IOD->extDescr[i]->EncDec(&coder, ENCODE, NULL);
			DESC_TO_HANDLE;
			if (MP4AddDescToMovieIOD(m_pMovie, descriptorH)) {
				MP4DisposeHandle(descriptorH);
				return;
			}
		}

	} else {
		if (ods.GetLength() != 1) {
			printf ("Error: only one OD must be specified for a Exchange File.\nHit enter to continue\x7\n");
			getchar();
			return;
		}
		isExchangeFile = 1;
		err = NewMovie(ods[0]->ObjectDescriptorID, 0, 0, 0, 0, 0, (char *) (LPCSTR) ods[0]->URLstring, isExchangeFile);
		if (err) return;
		if (trace)
			fprintf(trace, "Creating Exchange MP4 file...\n");

		//Add all OCIdescriptorPtr to our movie
		for (i = 0; i < ods[0]->ociDescr.GetLength(); i++) {
			coder.Reset();
			ods[0]->ociDescr[i]->EncDec(&coder, CALC_LENGTH, NULL);
			ods[0]->ociDescr[i]->EncDec(&coder, ENCODE, NULL);
			DESC_TO_HANDLE;
			if (MP4AddDescToMovieIOD(m_pMovie, descriptorH)) {
				MP4DisposeHandle(descriptorH);
				return;
			}
		}

		//Add all IPMPdescriptorPtr to our movie
		for (i = 0; i < ods[0]->ipmpDescrPtr.GetLength(); i++) {
			coder.Reset();
			ods[0]->ipmpDescrPtr[i]->EncDec(&coder, CALC_LENGTH, NULL);
			ods[0]->ipmpDescrPtr[i]->EncDec(&coder, ENCODE, NULL);
			DESC_TO_HANDLE;
			if (MP4AddDescToMovieIOD(m_pMovie, descriptorH)) {
				MP4DisposeHandle(descriptorH);
				return;
			}
		}
	
		//Add all EXTdescriptorPtr to our movie
		for (i = 0; i < ods[0]->extDescr.GetLength(); i++) {
			coder.Reset();
			ods[0]->extDescr[i]->EncDec(&coder, CALC_LENGTH, NULL);
			ods[0]->extDescr[i]->EncDec(&coder, ENCODE, NULL);
			DESC_TO_HANDLE;
			if (MP4AddDescToMovieIOD(m_pMovie, descriptorH)) {
				MP4DisposeHandle(descriptorH);
				return;
			}
		}
	}

	/*-----------------------------------------------------------------
	Traverse the esDescrs list. For	each descriptor which points to a file
	that shall be added to the movie, open that file.
	Register the streams with the Movie.
	-----------------------------------------------------------------*/

	if (!isExchangeFile) {
		/*-------------------------------------------------------------
		Open the IOD streams
		--------------------------------------------------------------*/
		InitialObjectDescriptor *IOD = IODs [0];

		for (j = 0; j < IOD->esDescr.GetLength ();) {
			ES_DescriptorMP4 *pESD = (ES_DescriptorMP4 *) IOD->esDescr [j];
			pESD->m_bIsInIOD = TRUE;
			pESD->SetMovie(m_pMovie);

			switch ( pESD->OpenFile(fList, flags)) {
				//Stream couldn't be opened: if URLString, it's OK, otherwise pb...
			case -1:
				//UI streams don't use data
/* Commenting Stream Type == 9 MPEGJ
				if (pESD->decConfigDescr->streamType == 9) {
					esds += pESD;
					j++;
				} else if ((LPCSTR) pESD->URLstring && strlen( (LPCSTR) pESD->URLstring) ) {*/
				if ((LPCSTR) pESD->URLstring && strlen( (LPCSTR) pESD->URLstring) ) {
					esds += pESD;
					j++;
				} else {
					printf("ESD %d: file %s not found...\n", (int) pESD->ES_ID, (LPCSTR) pESD->GetFileName());
					bError = TRUE;
					goto has_error;
				}
				break;
			default:
				esds += pESD;
				j++;
				break;
			}
		}
	}

	/*-------------------------------------------------------------
	Open the other streams
	--------------------------------------------------------------*/
	for (i = 0; i < ods.GetLength (); i++) {
		//if we have a URL in this OD, it is handle by
		//1) the OD stream if present	(presentation)
		//2) the IOD of the movie		(exchange)
		//so we don't need to worry about this case
		for (j = 0; j < ods [i] -> esDescr.GetLength (); ) {
			ES_DescriptorMP4 *pESD = (ES_DescriptorMP4 *) ods[i]->esDescr[j];
			if (isExchangeFile) {
				pESD->m_bIsInIOD = TRUE;
			} else {
				pESD->m_bIsInIOD = FALSE;
			}
			pESD->SetMovie(m_pMovie);

			switch (pESD->OpenFile (fList, flags) ) {
				//Stream couldn't be opened: if URLString, it's OK, otherwise pb...
			case -1:
				//UI streams don't use data
/* Commenting MPEGJ Omission
				if (pESD->decConfigDescr->streamType == 9) {
					esds += pESD;
					j++;
				} else if ((LPCSTR) pESD->URLstring && strlen( (LPCSTR) pESD->URLstring) ) {*/
				if ((LPCSTR) pESD->URLstring && strlen( (LPCSTR) pESD->URLstring) ) {
					esds += pESD;
					j++;
				} else {
					printf("ESD %d: file %s not found...\n", (int) pESD->ES_ID, (LPCSTR) pESD->GetFileName());
					bError = TRUE;
					goto has_error;
				}
				break;
			//Add the track
			default:
				esds += pESD;
				j++;
				break;
			}
		}
	}

	//Error while opening -> EXIT
has_error:
	if (bError) {
		printf ("Hit Enter to continue\x07\n");
		getchar ();
		return;
	}

	int minGroupID = 1000;
	//Now we have all the tracks - stream association in the list.
	//Let's create the tracks and the media
	for (i = 0; i<esds.GetLength(); i++) {
		esds[i]->CreateTrack();		//create a track
		esds[i]->CreateMedia();		//then a media in the track
		esds[i]->InitWrite();		//then init the media

		//if remote esd or UI stream terminate track and remove from active list
		if ( ((LPCSTR) esds[i]->URLstring && strlen( (LPCSTR) esds[i]->URLstring) )
			) {
/* Commenting MPEGJ omission
			|| (esds[i]->decConfigDescr->streamType == 9) ) { */
			if (trace)
				fprintf(trace, "Track %d done\n", (int) esds[i]->ES_ID);

			esds[i]->WriteBlankSample();
			esds[i]->TerminateWrite();
			esds.DeleteFrom(i);
			i--;
			continue;
		}

		esds[i]->GetAU();			//init the reading of the first AU for the track
		//get the ID of the first stream group
		if (esds[i]->muxInfo->GroupID < minGroupID)
			minGroupID = esds[i]->muxInfo->GroupID;

		if (trace)
			fprintf(trace, "Track %d initialized...\n", (int) esds[i]->ES_ID);
	}

	u32 stDTS = -1;
	int iNoDesc;
	//this var is used to switch the groups
	int iDescInGroup = 0;
	ES_DescriptorMP4 *esd;

	if (!m_bInterleaved) {
		while (esds.GetLength()) {
			esd = esds[0];
			err = esd->WriteSample(&stDTS); if (err) goto bail;
			if (esd->GetDTS() == -1) {
				//if end of stream, finish write and remove stream from our list
				esd->TerminateWrite();
				esds.DeleteFrom(0);
			} else {
				//else let's get another AU from this stream
				esd->GetAU();
			}
		}
	} else {
		iNoDesc = 0;
		//And now the magic: store the file with a nice interleaving for self-contained files
		while (esds.GetLength()) {
			iDescInGroup = 0;
			esd = NULL;
			stDTS = -1;	//warning: it's u32, so -1 is pretty big...
			//find the track with the lowest DTS in this group
			for (i = 0; i < esds.GetLength(); i++) {
				if ( (esds[i]->GetDTS() < stDTS)
					&& (esds[i]->muxInfo->GroupID == minGroupID) ) {

					stDTS = esds[i]->GetDTS();
					esd = esds[i];
					iNoDesc = i;
					iDescInGroup = 1;
				}
			}
			if (iDescInGroup) {
				//yes, we do have desc in group
				err = esd->WriteSample(&stDTS); if (err) goto bail;
				if (trace)
					fprintf(trace, "Group %d - Track %d: Writing AU with DTS %d\n", minGroupID, (int)esd->ES_ID, (int) stDTS);

				if (esd->GetDTS() == -1) {
					//if end of stream, finish write and remove stream from our list
					esd->TerminateWrite();
					if (trace) fprintf(trace, "\t\tTrack %d done....\n", (int) esd->ES_ID);
					esds.DeleteFrom(iNoDesc);
				} else {
					//else let's get another AU from this stream
					esd->GetAU();
				}
			} else {
				//no, we do not have desc in group: minGroup ++
				minGroupID ++;
				if (trace)
					fprintf(trace, "End of Group reached.\n\nSwitching to next group [ID: %d]\n", minGroupID);
			}
		}
	}
	//All tracks have been added-> set dependencies and save the movie to a file
	if (m_pMovie != NULL) {
		if (m_bMaximize) MaximizeDuration();

		//Write the movie to a file and free the handle
		if (trace) fprintf(trace, "Writing Movie to file ...");
		err = (MP4Err) MP4WriteMovieToFile( m_pMovie, m_stMP4Movie); if (err) goto bail;
		err = (MP4Err) MP4DisposeMovie(m_pMovie); if (err) goto bail;
		if (trace) fprintf(trace, " done.\n");
	}
	return;

bail:
	printf("Has Error\n");
	return;
}


//ExchangeFile: if != 0, we'll create an exchange MP4 file, eg with an OD instead of IOD
// SOLARIS PORTING - changed the return type to BOOL
BOOL MP4Enc::NewMovie(u32 OD_ID, u8 OD_PL, u8 scene_PL, u8 audio_PL, u8 visual_PL, u8 praph_PL, char *url, u8 ExchangeFile)
{
	MP4Err err = MP4NoErr;

	err = MP4NewMovieExt(&m_pMovie, OD_ID, OD_PL, scene_PL, audio_PL, visual_PL, praph_PL , url, ExchangeFile);
	if (err) return err;
	return MP4SetMovieTimeScale(m_pMovie, DEF_MOVIE_TIMESCALE);
}


void MP4Enc::MaximizeDuration()
{
	u32 numTracks, i, timescale;
	u64 duration, newDur;
	double maxDur, tmp;
	MP4Track track;
	MP4Media media;
	maxDur = 0.0;
	MP4GetMovieTrackCount(m_pMovie, &numTracks);

	if (numTracks==1) return;

	for (i=0; i<numTracks; i++) {
		MP4GetMovieIndTrack(m_pMovie, i+1, &track);
		MP4GetTrackMedia(track, &media);
		MP4GetMediaDuration(media, &duration);
		MP4GetMediaTimeScale(media, &timescale);
		tmp = duration*1000;
		tmp/= timescale;
		if (tmp>maxDur) maxDur = tmp;
	}

	for (i=0; i<numTracks; i++) {
		MP4GetMovieIndTrack(m_pMovie, i+1, &track);
		MP4GetTrackMedia(track, &media);
		MP4GetMediaDuration(media, &duration);
		MP4GetMediaTimeScale(media, &timescale);
		newDur = maxDur*timescale/1000;

		if (newDur<=duration) continue;

		MP4OffsetMediaLastSampleDuration(media, newDur-duration);
	}

}