#include "MP4OSMacros.h"


/********************************************************\
|*														*|
|*			DEFINES AND CONSTANTS - MP4LIB				*|
|*														*|
\********************************************************/

#define MP4ObjectDescriptorHandlerType    MP4_FOUR_CHAR_CODE( 'o', 'd', 's', 'm' )
#define MP4ClockReferenceHandlerType      MP4_FOUR_CHAR_CODE( 'c', 'r', 's', 'm' )
#define MP4SceneDescriptionHandlerType    MP4_FOUR_CHAR_CODE( 's', 'd', 's', 'm' )
#define MP4VisualHandlerType              MP4_FOUR_CHAR_CODE( 'v', 'i', 'd', 'e' )
#define MP4AudioHandlerType               MP4_FOUR_CHAR_CODE( 's', 'o', 'u', 'n' )
#define MP4MPEG7HandlerType               MP4_FOUR_CHAR_CODE( 'm', '7', 's', 'm' )
#define MP4OCIHandlerType                 MP4_FOUR_CHAR_CODE( 'o', 'c', 's', 'm' )
#define MP4IPMPHandlerType                MP4_FOUR_CHAR_CODE( 'i', 'p', 's', 'm' )
#define MP4MPEGJHandlerType               MP4_FOUR_CHAR_CODE( 'm', 'j', 's', 'm' )
#define MP4HintHandlerType                MP4_FOUR_CHAR_CODE( 'h', 'i', 'n', 't' )

enum
{
	MP4NewTrackIsVisual  = (1 << 1),
	MP4NewTrackIsAudio   = (1 << 2),
	MP4NewTrackIsPrivate = (1 << 8)
};

enum
{
	MP4OpenMovieNormal  = 0,
	MP4OpenMovieDebug   = (1 << 0)
};

/* flags for GetMediaSample */
enum
{
	MP4MediaSampleNotSync = (1 << 0),
	MP4MediaSampleHasCTSOffset  = (1 << 1)
};

/* flags for NextInterestingTime */
enum
{
	MP4NextTimeSearchForward = 0,
	MP4NextTimeSearchBackward = -1,
	MP4NextTimeMediaSample = (1 << 0),
	MP4NextTimeMediaEdit   = (1 << 1),
	MP4NextTimeTrackEdit   = (1 << 2),
	MP4NextTimeSyncSample  = (1 << 3),
	MP4NextTimeEdgeOK      = (1 << 4)
};

enum
{
	MP4HintTrackReferenceType        = MP4_FOUR_CHAR_CODE( 'h', 'i', 'n', 't' ),
	MP4StreamDependencyReferenceType = MP4_FOUR_CHAR_CODE( 'd', 'p', 'n', 'd' ),
	MP4ODTrackReferenceType          = MP4_FOUR_CHAR_CODE( 'm', 'p', 'o', 'd' )
};


typedef int MP4Err;

#define MP4EOF                    1
#define MP4NoErr                  0
#define MP4FileNotFoundErr        -1
#define MP4BadParamErr            -2
#define MP4NoMemoryErr            -3
#define MP4IOErr                  -4
#define MP4NoLargeAtomSupportErr  -5
#define MP4BadDataErr             -6
#define MP4VersionNotSupportedErr -7
#define MP4InvalidMediaErr        -8
#define MP4DataEntryTypeNotSupportedErr -100
#define MP4NoQTAtomErr                  -500
#define MP4NotImplementedErr            -1000




/********************************************************\
|*														*|
|*			EXPORTED CLASSES - MP4LIB					*|
|*														*|
\********************************************************/

class MP4Handle  
{
public:
	MP4_EXPORT MP4Handle();
	MP4_EXPORT MP4Handle(u32 handleSize);
	MP4_EXPORT virtual ~MP4Handle();

	//Methods
public:
	MP4_EXPORT u32 GetHandleSize();
	MP4_EXPORT MP4Err SetHandleSize(u32 requestedSize);
	MP4_EXPORT MP4Err SetData(char *mydata, u32 mydataSize);
	MP4_EXPORT MP4Err SetData(char *mydata, u32 mydataSize, u32 offset);
	MP4_EXPORT char *GetData();

	//variables
private:
	char *m_pData;
	u32  m_lSignature;
	u32  m_lSize;
	u32  m_lAllocatedSize;

};

class MP4_EXPORT SLConfigSettings  
{
public:
	SLConfigSettings();
	virtual ~SLConfigSettings();

	//Variables
public:
	u32 predefined;
	u32 useAccessUnitStartFlag;
	u32 useAccessUnitEndFlag;
	u32 useRandomAccessPointFlag;
	u32 useRandomAccessUnitsOnlyFlag;
	u32 usePaddingFlag;
	u32 useTimestampsFlag;
	u32 useIdleFlag;
	u32 durationFlag;
	u32 timestampResolution;
	u32 OCRResolution;
	u32 timestampLength;
	u32 OCRLength;
	u32 AULength;
	u32 instantBitrateLength;
	u32 degradationPriorityLength;
	u32 AUSeqNumLength;
	u32 packetSeqNumLength;
	u32 timeScale;
	u32 AUDuration;
	u32 CUDuration;
	u64	startDTS;
	u64 startCTS;
#ifdef PRECOR1
	u32 OCRESID;
#endif
};

class MP4UserData
{
	//Methods
public:
	MP4_EXPORT AddUserData(MP4Handle *dataH, u32 userDataType, u32 *outIndex );
	MP4_EXPORT GetUserDataEntryCount(u32 userDataType, u32 *outCount);
	MP4_EXPORT GetIndUserDataType(u32 typeIndex, u32 *outType );
	MP4_EXPORT GetUserDataTypeCount(u32 *outCount );
	MP4_EXPORT GetUserDataItem(MP4Handle *dataH, u32 userDataType, u32 itemIndex );
};

class MP4Track;
class Movie;

class MP4_EXPORT MP4Movie
{
public:
	MP4Movie(u32 theinitialODID,
		   u8 theOD_profileAndLevel, 
		   u8 thescene_profileAndLevel,
		   u8 theaudio_profileAndLevel, 
		   u8 thevisual_profileAndLevel,
		   u8 thegraphics_profileAndLevel);
	MP4Movie(MP4Handle *movieH, u32 newMovieFlags);
	MP4Movie(const char *movieURL, int openMovieFlags);
	virtual ~MP4Movie();

	/**************
	*   Methods   *
    **************/
public:
	MP4Err	PutMovieIntoHandle(MP4Handle **movieH);
	MP4Err	WriteMovieToFile(char *filename);
	u32		GetMovieTrackCount();
	u32		GetMovieTimeScale();
	u64		GetMovieDuration();
	MP4Err	GetMovieUserData(MP4UserData **);
	u8		GetMovieIODInlineProfileFlag();
	MP4Err	SetMovieIODInlineProfileFlag(u8 theFlag);
	MP4Err	GetMovieProfilesAndLevels(u8 *outOD,
										u8 *outScene,
										u8 *outAudio, 
										u8 *outVisual, 
										u8 *outGraphics);
	MP4Err	GetMovieInitialObjectDescriptor(MP4Handle **outDescriptorH );
	MP4Err	GetMovieInitialObjectDescriptorUsingSLConfig(SLConfigSettings *slconfig, MP4Handle **outDescriptorH );
	MP4Err	GetMovieIndTrack(u32 trackIndex, MP4Track **outTrack);
	MP4Err	GetMovieTrack(u32 trackID, MP4Track **outTrack );
	MP4Err	SetMovieTimeScale(u32 timeScale);
	MP4Err	NewMovieTrack(u32 newTrackFlags, MP4Track **outTrack );
	MP4Err	NewMovieTrackWithID(u32 newTrackFlags, u32 newTrackID, MP4Track **outTrack );
	MP4Err	AddTrackES_IDToIOD(MP4Track *theTrack);

private:
	Movie *theMovie;
};


class MP4Media
{
	//Methods
public:
	MP4_EXPORT MP4Err	AddMediaSampleReference(u64 dataOffset, 
									u32 sampleCount,
									MP4Handle *durationsH, 
									MP4Handle *sizesH, 
									MP4Handle *sampleEntryH,
									MP4Handle *decodingOffsetsH, 
									MP4Handle *syncSamplesH );
	MP4_EXPORT MP4Err	AddMediaSamples(MP4Handle *sampleH, 
									u32 sampleCount,
									MP4Handle *durationsH, 
									MP4Handle *sizesH, 
									MP4Handle *sampleEntryH,
									MP4Handle *decodingOffsetsH, 
									MP4Handle *syncSamplesH);
	MP4_EXPORT MP4Err	SetMediaLanguage(char *threeCharCode);
	MP4_EXPORT MP4Err	GetMediaLanguage(char *outThreeCharCode);
	MP4_EXPORT u32		GetMediaTimeScale();
	MP4_EXPORT u64		GetMediaDuration();
	MP4_EXPORT MP4Err	GetMediaTrack(MP4Track **);
	MP4_EXPORT u32		GetMediaSampleCount();
	MP4_EXPORT MP4Err	NewSampleDescription(MP4Track *track,
										MP4Handle **sampleDescriptionH,
										u32 dataReferenceIndex,
										u32 objectTypeIndication,
										u32 streamType,
										u32 decoderBufferSize,
										u32 maxBitrate,
										u32 avgBitrate,
										MP4Handle *decoderSpecificInfoH);
	MP4_EXPORT MP4Err	NewSampleDescriptionWithOCRAssociation( MP4Track *track,
										MP4Handle **sampleDescriptionH,
                                        u32 dataReferenceIndex,
										u32 objectTypeIndication,
										u32 streamType,
										u32 decoderBufferSize,
										u32 maxBitrate,
										u32 avgBitrate,
										MP4Handle *decoderSpecificInfoH,
										u32 theOCRESID);
	MP4_EXPORT MP4Err	GetMediaSampleDescription(u32 index,
										MP4Handle **outDescriptionH, 
										u32 *outDataReferenceIndex);
	MP4_EXPORT MP4Err	GetMediaDecoderConfig(u32 sampleDescIndex, MP4Handle **decoderConfigH);
	MP4_EXPORT MP4Err	GetMediaDecoderType(u32 sampleDescIndex, 
										u32 *outObjectType,
										u32 *outStreamType,
										u32 *outBufferSize,
										MP4Handle **specificInfoH);
	MP4_EXPORT MP4Err	GetMediaDecoderInformation(u32 sampleDescIndex, 
										u32 *outObjectType, 
										u32 *outStreamType,
										u32 *outBufferSize,
										u32 *outUpstream,
										u32 *outMaxBitrate,
										u32 *outAvgBitrate,
										MP4Handle **specificInfoH);
	MP4_EXPORT u32		GetMediaDataRefCount();
	MP4_EXPORT MP4Err	CheckMediaDataReferences();
	MP4_EXPORT MP4Err	AddMediaDataReference(u32 *outReferenceIndex,
										MP4Handle *urlHandle, 
										MP4Handle *urnHandle);
	MP4_EXPORT MP4Err	GetMediaDataReference(u32 index,
										MP4Handle **referenceH,
										MP4Handle *urnH,
										u32 *outReferenceType,
										u32 *outReferenceAttributes);

	MP4_EXPORT MP4Err	GetMediaHandlerDescription(u32 *outType, MP4Handle **outName);
	MP4_EXPORT MP4Err	GetMediaNextInterestingTime(u32 interestingTimeFlags,
										u64 searchFromTime,
										u32 searchDirection,
										u64 *outInterestingTime,
										u64 *outInterestingDuration);
	MP4_EXPORT MP4Err	GetMediaSampleDescIndex(u64 desiredTime, u32 *outSampleDescriptionIndex );
	MP4_EXPORT MP4Err	GetMediaSample(MP4Handle **outSample,
										u32 *outSize,
										u64 desiredDecodingTime, 
										u64 *outDecodingTime, 
										u64 *outCompositionTime, 
										u64 *outDuration,
										MP4Handle **outSampleDescription, 
										u32 *outSampleDescriptionIndex,
										u32 *outSampleFlags);
	MP4_EXPORT MP4Err	GetIndMediaSample(u32 sampleNumber,
										MP4Handle **outSample,
										u32 *outSize,
										u64 *outDTS,
										s32 *outCTSOffset,
										u64 *outDuration,
										u32 *outSampleFlags,
										u32 *outSampleDescIndex);
	MP4_EXPORT MP4Err	GetIndMediaSampleReference(u32 sampleNumber,
										u32 *outOffset,
										u32 *outSize,
										u32 *outDuration,
										u32 *outSampleFlags,
										u32 *outSampleDescIndex,
										MP4Handle **sampleDesc);
	MP4_EXPORT MP4Err	MediaTimeToSampleNum(u64 mediaTime, 
										u32 *outSampleNum, 
										u64 *outSampleCTS, 
										u64 *outSampleDTS, 
										s32 *outSampleDuration);
	MP4_EXPORT MP4Err	SampleNumToMediaTime(u32 sampleNum, 
										u64 *outSampleCTS, 
										u64 *outSampleDTS, 
										s32 *outSampleDuration);


};



class MP4Track
{
	//Methods
public:
	MP4_EXPORT u32		GetTrackID();
	MP4_EXPORT MP4Err	SetTrackEnabled(u32 enabled);
	MP4_EXPORT MP4Err	GetTrackEnabled(u32 *outEnabled );
	MP4_EXPORT MP4Err	AddTrackReferenceWithID(u32 dependsOnID, 
										 u32 dependencyType, 
										 u32 *outReferenceIndex );
	MP4_EXPORT MP4Err	AddTrackReference(MP4Track *dependsOn, 
								   u32 dependencyType, 
								   u32 *outReferenceIndex);
	MP4_EXPORT MP4Err	AddTrackToMovieIOD();
	MP4_EXPORT MP4Err	GetTrackDuration(u64 *outDuration );
	MP4_EXPORT MP4Err	GetTrackReference(u32 referenceType, 
									u32 referenceIndex, 
									MP4Track **);
	MP4_EXPORT MP4Err	GetTrackReferenceCount(u32 referenceType, u32 *outReferenceCount );
	MP4_EXPORT MP4Err	GetTrackMovie(MP4Movie **outMovie);
	MP4_EXPORT MP4Err	GetTrackMedia(MP4Media **);
	MP4_EXPORT MP4Err	GetTrackUserData(MP4UserData **);
	MP4_EXPORT MP4Err	NewTrackMedia(MP4Media **outMedia, 
								u32 mediaType, 
								u32 timeScale, 
								MP4Handle *dataReference);
	MP4_EXPORT MP4Err	InsertMediaIntoTrack(s32 trackStartTime, 
									  u32 mediaStartTime, 
									  u64 mediaDuration, 
									  s32 mediaRate);
	MP4_EXPORT MP4Err	GetTrackOffset(u32 *outMovieOffsetTime);
	MP4_EXPORT MP4Err	SetTrackOffset(u32 movieOffsetTime);
	MP4_EXPORT MP4Err	TrackTimeToMediaTime(u64 inTrackTime, s64 *outMediaTime);
};


class MP4TrackReader
{
public:
	MP4_EXPORT MP4TrackReader(MP4Track *theTrack);
	MP4_EXPORT virtual ~MP4TrackReader();

	//Methods
public:
	MP4_EXPORT MP4Err GetCurrentDecoderConfig(MP4Handle **decoderConfigH);
	MP4_EXPORT MP4Err GetNextAccessUnit(MP4Handle **outAccessUnit,
										 u32 *outSize, 
										 u32 *outSampleFlags,
										 u32 *outCTS, 
										 u32 *outDTS);
	MP4_EXPORT MP4Err GetNextAccessUnitWithDuration(MP4Handle **outAccessUnit,
													 u32 *outSize,
													 u32 *outSampleFlags,
													 u32 *outCTS, 
													 u32 *outDTS, 
													 u32 *outDuration);
	MP4_EXPORT MP4Err SetSLConfig(SLConfigSettings *slConfig);
	MP4_EXPORT MP4Err GetNextPacket(MP4Handle **outSample, u32 *outSize);

};