/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1999.
************************************************************************/
#include "defs.h"
#include "MPEGJ.h"
#include "odMux.h"
#include <stdlib.h>
#include <math.h>

bool flag = true;
FILE* fptr;

/*-----------------------------------------------------------------
MPEGJStream constructor reads first 3 character
-----------------------------------------------------------------*/
MPEGJStream :: MPEGJStream (FILE *fStream, ESDesc *pDescriptor) :
	Stream (fStream, pDescriptor), m_infoFile(NULL), m_lCurrentAUSize(-1), m_lDTS(-1)
{

	// Handle the error when the pDescriptor -> streamData field is not given
	if ((pDescriptor -> streamData == NULL) || (pDescriptor -> streamData[0] == 0)) {
		static char szBuffer [150];
		sprintf(szBuffer, "MPEGJ: Error opening '%s'.\n", pDescriptor -> streamData);
		throw ExceptionInvalidData (szBuffer);
	}
	// Otherwise the pDescriptor -> streamData is the name of a file
	else {
		m_infoFile = fopen(pDescriptor -> streamData, "r");
		if (m_infoFile == NULL) {
			static char szBuffer [150];
			sprintf(szBuffer, "MPEGJ: Error opening '%s'.\n", pDescriptor -> streamData);
			throw ExceptionInvalidData (szBuffer);
		}
	}

	if(getNextAUInfo() == EOF) {
		static char szBuffer [150];
		sprintf(szBuffer, "MPEGJ: Error reading '%s'.\n", pDescriptor -> streamData);
		throw ExceptionInvalidData (szBuffer);
	}

	getNextAUInfo();
}
	
int MPEGJStream::getNextAUInfo() 
{
	int res = fscanf(m_infoFile, "%d %d", &m_lCurrentAUSize, &m_lDTS);
	if(res != EOF && (m_lCurrentAUSize ==-1 || m_lDTS ==-1))
		throw ExceptionInvalidData ("MPEGJ: Error reading info file");
	return res;
}

/*-----------------------------------------------------------------
Time stamps are calculated according to the rate given by the
pDescriptor -> streamData field
-----------------------------------------------------------------*/
long MPEGJStream :: GetDTS (long)
{
	if (m_cNext == EOF && getNextAUInfo() == EOF)
		return -1;
	
	return m_lDTS;
}

/*-----------------------------------------------------------------
GetChar returns a character and reads one next char. We are always
3 chars ahead, which are used to check whether this is end of AU.
-----------------------------------------------------------------*/
int MPEGJStream :: GetChar ()
{
	if(m_lCurrentAUSize-- > 0) {
		m_cNext = getc ();
		if(m_cNext == EOF)
			throw ExceptionInvalidData ("MPEGJ: Invalid MPEGJ ES info file");
	}
	else
		m_cNext = EOF;
	return m_cNext;
}

static StreamFactory<MPEGJStream> mpegjFactory ("MPEGJ");