/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Multiplexer
FILE:       hint.cpp
AUTHOR:     Guido Franceschini (CSELT)
OVERVIEW:   Implementation of NHnt, the class that handles input streams
which have been preparsed according to the NHnt format 
-------------------------------------------------------------------------*/
#include "defs.h"
#include "nhnt.h"
#include "assert.h"
#include "odMP4.h"
#include "endian.h"

/*-----------------------------------------------------------------
NHntStream constructor reads first character
-----------------------------------------------------------------*/
NHntStream :: NHntStream (FILE *fStream, ESDesc *pDescriptor) :
	Stream (fStream, pDescriptor)
{
	m_nScale = 1000;
	m_nCount = 0;
	m_nTotalCount = 0;
	m_nTotalNHnt = 0;
	m_nTotalNHntSize = 0;
	m_fNHnt = NULL;
	char *szFileNHnt = new char[strlen(pDescriptor->fileName)+5];
	char *last = strrchr (pDescriptor->fileName, '.');
	strcpy(szFileNHnt, pDescriptor->fileName);
	strcpy(szFileNHnt + (last - pDescriptor->fileName), ".nhnt");
	if ((m_fNHnt = fopen (szFileNHnt, "rb")) == NULL)
	{
		printf ("cannot open NHnt file for %s\n", pDescriptor->fileName);
		exit(1);
	}
	char version[5];
	version[4] = '\0';
	fread (version, 4, 1, m_fNHnt);
	if (strcmp (version, "NHnt") != 0)
	{
		printf ("NHnt file %s is of unknown version\n", szFileNHnt);
		exit(1);
	}
	fread (&m_header, sizeof(m_header), 1, m_fNHnt);
	m_header.first = ntohl(m_header.first);
	m_header.bufferSizeDB = ntohl(m_header.bufferSizeDB);
	m_header.maxBitrate = ntohl(m_header.maxBitrate);
	m_header.avgBitrate = ntohl(m_header.avgBitrate);
	m_header.timeScale = ntohl(m_header.timeScale);
	m_nScale = m_header.timeScale;

	fread (&m_nhnt, sizeof(m_nhnt), 1, m_fNHnt);
	// convert from network byte order
	m_nhnt.first = ntohl(m_nhnt.first);
	m_nhnt.offset = ntohl(m_nhnt.offset);
	m_nhnt.cts = ntohl(m_nhnt.cts);
	m_nhnt.dts = ntohl(m_nhnt.dts);
	if (seek( (long) m_nhnt.offset) == -1) {
		printf ("NHnt file %s starts with invalid offset %d corresponding to cts %d\n", szFileNHnt, m_nhnt.offset, m_nhnt.cts);
		exit(1);
	}
	m_cNext = getc ();
}

int NHntStream :: GetScale ()
{
	return m_nScale;
}

long NHntStream :: GetDTS ()
{
	m_nCount = 0;
	m_lCTS = m_nhnt.cts;
	m_nFrameSize = m_nhnt.first >> 8;
	if (m_nFrameSize == 0)
		return -1;
	assert (m_nTotalNHntSize == m_nTotalCount);
	m_nTotalNHntSize += m_nFrameSize;
	return m_nhnt.dts;
}

long NHntStream :: GetCTS ()
{
	return m_lCTS;
}

/*-----------------------------------------------------------------
GetChar returns a character and reads one char ahead.
-----------------------------------------------------------------*/
int NHntStream :: GetChar ()
{
	if (m_nCount >= m_nFrameSize) {
		if (fread (&m_nhnt, sizeof(m_nhnt), 1, m_fNHnt) <1)
			m_nhnt.first &= 0x000000ff; // set length = 0
		else
		{	// convert from network byte order
			m_nhnt.first = ntohl(m_nhnt.first);
			m_nhnt.offset = ntohl(m_nhnt.offset);
			m_nhnt.cts = ntohl(m_nhnt.cts);
			m_nhnt.dts = ntohl(m_nhnt.dts);
			++m_nTotalNHnt;
			if (seek( (long) m_nhnt.offset) == -1) {
				printf ("NHnt file contains invalid offset %d corresponding to cts %d\n", m_nhnt.offset, m_nhnt.cts);
				exit(1);
			}
			m_cNext = getc ();
		}
		return -1;
	}
	++m_nCount;
	++m_nTotalCount;
	int c = m_cNext;
	m_cNext = getc ();
	assert (c != EOF);
	return c;
}

/*----------------------------------------------------------------
Returns true if the AU referenced by the last call to GetDTS()
was a Random Access Point
-----------------------------------------------------------------*/
bool NHntStream :: IsRAP ()
{
	return (m_nhnt.first & (0x00000080));
}

static StreamFactory<NHntStream> bifsFactory ("NHnt");
