/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Multiplexer
FILE:       TSBifs.cpp
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Implementation of TSBifsStream, the class that handles 
			time stamped BIFS streams.
-------------------------------------------------------------------------*/
#include "defs.h"
#include "TSBifs.h"

/*-----------------------------------------------------------------
Time stamped BIFS file are made of access units, each preceded by
4 bytes of time stamp and 1 byte of AU length.
-----------------------------------------------------------------*/
long TSBifsStream :: GetDTS (long)
{
	int c1 = getc ();
	if (c1 == EOF)
		return -1;

	/*JLF 03/03. first bit is RAP indicator*/
	if (m_bUseV2) {
		m_bRAP = c1>>7;
		c1 = c1 & 0x7F;
	}

	int c2 = getc ();
	int c3 = getc ();
	int c4 = getc ();
	int c5 = getc ();
	int c6 = getc ();
	int c7 = getc ();
	int c8 = getc ();
	m_nAULength = MAKELONG (MAKEWORD (c8, c7), MAKEWORD (c6, c5));
	m_lDTS = MAKELONG (MAKEWORD (c4, c3), MAKEWORD (c2, c1));
	return m_lDTS;
}

/*-------------------------------------------------------------
In version 1 only the first AU is RAP, in v2 RAP is indicated in the bitstream
-------------------------------------------------------------*/
bool TSBifsStream :: IsRAP ()
{
	bool bResult = m_bRAP;
	m_bRAP = false;
	return bResult;
}


/*-----------------------------------------------------------------
GetChar returns a character and read one char ahead
-----------------------------------------------------------------*/
int TSBifsStream :: GetChar ()
{
	if (m_nAULength-- == 0)
		return -1;
	return getc ();
}

static StreamFactory<TSBifsStream> bifsFactory ("BIFS");
static StreamFactory<TSBifsStreamV2> bifsFactory2 ("BIFS2");
static StreamFactory<TSBifsStream> odFactory ("BIFS");