/***********************************************************************
This software module was originally developed by Jean Le Feuvre
(avipix, llc) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. avipix, llc retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2000.
************************************************************************/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  MP4 File Writer
FILE:       esMP4.cpp
AUTHOR:     Jean Le Feuvre (avipix, llc, jean@avipix.com), December 2000
OVERVIEW:   odMP4: the ElementaryStream implementation for the MP4 file format
-------------------------------------------------------------------------*/



#include "defs.h"
#include "esMP4.h"
#include "MuxStream.h"

extern BOOL g_bErrors;


#define DESC_TO_HANDLE \
		m_coder.Align ();	\
		idescSize = m_coder.GetDataLength();	\
		err = MP4SetHandleSize(descriptorH, idescSize);	\
		if (err) return err;	\
		p = (BYTE*) *descriptorH;	\
		buffer = (BYTE *) calloc(idescSize, 1);		\
		memcpy(buffer, m_coder.GetData(), idescSize);	\
		for (int z=0; z < idescSize; z++) {		\
			*p++ = buffer[z];				\
		}									\
		free(buffer);
		

/*-------------------------------------------------------------
ES_DescriptorMux constructor
-------------------------------------------------------------*/
ES_DescriptorMP4 :: ES_DescriptorMP4 () :
	m_fStream (NULL),
	m_pStream (NULL),
	m_lTS (-1),
	m_nPayloadLength (0),
	m_bStart (TRUE),
	m_bEnd (FALSE),
	m_bEndOfAU (FALSE),
	m_coder (NULL, NULL, "DecoderSpecificInfos", TRACE_NONE, 0)
{
	MP4NewHandle(0, &m_pSample);
	m_pSampleEntry = NULL;
	m_bIsFirst = TRUE;
	m_iInitTS = 0;

/***********Added by Liao Xuan *************/
#if defined(DES)
	indexEncryAU = 0;
/*
//load DES library
	hInstDes = LoadLibrary("desdll.dll");
	if(hInstDes == NULL)
	{
		printf("\nError: fail to load DES dll\n");
		exit(-1);
	}	
*/
#endif //DES
/***********End: Added by Liao Xuan ********/
}

/*-----------------------------------------------------------------
ES_DescriptorMP4 destructor. Close the stream file
-----------------------------------------------------------------*/
ES_DescriptorMP4 :: ~ES_DescriptorMP4 ()
{
	MP4DisposeHandle(m_pSample);
	if (m_pStream)
		delete m_pStream;
	if (m_fStream)
		fclose (m_fStream);

/***********Added by Liao Xuan *************/
//free library
#if defined(DES)
	if(hInstDes) FreeLibrary(hInstDes);
#endif //DES	
/***********End: Added by Liao Xuan ********/
}

/*-----------------------------------------------------------------
Array of factory objects to create proper Stream object
-----------------------------------------------------------------*/
StreamFactoryBase *ES_DescriptorMP4 :: m_pFactories [40];
int ES_DescriptorMP4 :: m_nFactoriesAvailable;

/*-----------------------------------------------------------------
Register a StreamFactory object
-----------------------------------------------------------------*/
void ES_DescriptorMP4 :: RegisterFactory (StreamFactoryBase *pFactory)
{
	assert (m_nFactoriesAvailable < 40);
	m_pFactories [m_nFactoriesAvailable++] = pFactory;
}

/*-----------------------------------------------------------------
Open the stream file, and create a proper Stream object
-----------------------------------------------------------------*/
int ES_DescriptorMP4 :: OpenFile (FILE *fList, int flags)
{
	/*-----------------------------------------------------------------
	Open the stream file
	-----------------------------------------------------------------*/
	if (muxInfo == NULL_DESC) {
		if (!decConfigDescr || (decConfigDescr->streamType != 9)) 
			printf ("No Mux info for stream no %d\n", (int) ES_ID);
		
		return -1;
	}
	if (muxInfo -> fileName.GetLength () == 0)
		return 0;
	m_fStream = fopen (muxInfo -> fileName, "rb");
	if (!m_fStream)
	{	printf ("Cannot open %s\n", (LPCSTR) muxInfo -> fileName);
		return -1;
	}

	/*-----------------------------------------------------------------
	Instantiate a proper Stream object
	-----------------------------------------------------------------*/
	if (decConfigDescr == NULL_DESC)
	{	printf (
			"%s: decConfigDescr not available\n",
			(LPCSTR) muxInfo -> fileName
		);
		return -1;
	}


	//No hint file precised, use stream
	LPCSTR szStreamType = muxInfo -> streamFormat;
	for (int i = m_nFactoriesAvailable; --i >= 0;)
		if (m_pFactories [i] -> IsTypeOf (szStreamType)
		)
		{	ESDesc *pDesc = new ESDesc;
			pDesc -> startTime = muxInfo -> startTime;
			pDesc -> fileName = muxInfo -> fileName;
			pDesc -> streamType = muxInfo -> streamFormat;
			pDesc -> streamData = muxInfo -> streamAttr;
			pDesc -> repeatRate = muxInfo -> repeatRate;
			pDesc -> sessionLength = muxInfo -> sessionLength;
			pDesc -> maxAhead = muxInfo -> maxAhead;
			m_pStream = m_pFactories [i] -> Create (m_fStream, pDesc);
			break;
		}
	if (!m_pStream)
	{	printf (
			"%s: Unknown decoder type %s\n",
			(LPCSTR) muxInfo -> fileName,
			(LPCSTR) szStreamType
		);
		return -1;
	}

	// Stream Track in the movie
	m_coder.SetTraceFile (fList, muxInfo -> streamFormat, flags);
	m_lTS = -1;

	//If the file is empty, return.
	if (GetDTS() == -1) {
		printf("Error! Stream file %s is empty !\n", (LPCSTR)muxInfo->fileName);
		return -1;
	}
	
	return 1;
}

/*-----------------------------------------------------------------
Ask the stream for next time stamp
-----------------------------------------------------------------*/
long ES_DescriptorMP4 :: GetDTS (long lNow)
{
	if (m_lTS < 0)
		m_lTS = m_pStream -> GetDTS (lNow);
	return m_lTS;
}

int ES_DescriptorMP4 :: GetStreamType()
{
	return (int) decConfigDescr -> streamType;
}

int ES_DescriptorMP4 :: GetScale ()
{
	return m_pStream -> GetScale();
}

/*------------------------------------------------------------
Create the sample description (ESD) and add the descriptors
------------------------------------------------------------*/
MP4Err ES_DescriptorMP4::InitWrite()
{
	MP4Handle descriptorH;	
	MP4Handle descH;
	int idescSize = 0;
	BYTE *p, *buffer;
	u32 OCRId;
	int priority, i;
	MP4Err err = MP4NoErr;
	u32 uDRefID;
	//set to one, this is the default value for NewSampleEntry
	m_bSelfContained = TRUE;
	uDRefID = 1;


	
	/*-------------------------------------------------------------
	Get the SLConfigDescr
	-------------------------------------------------------------*/
	//AC:20021009 - SL extension : esMP4 : AbstractSLConfig  -- begin
	/*SLConfigDescriptor*/AbstractSLConfig configDefault;	
	m_pSLConfig = slConfigDescr;
	//AC:20021009 - SL extension : esMP4 : AbstractSLConfig  -- end
	if (m_pSLConfig == NULL)
		m_pSLConfig = &configDefault;

	m_MaxDuration = 0;
	
	
	//create a data reference in the media for future use
	if (muxInfo && muxInfo->UseDataReference) {
		m_bSelfContained = FALSE;
		//Add a reference to the file 
		MP4Handle URLHandle;
		err = MP4NewHandle(muxInfo->fileName.GetLength() + 1, &URLHandle); 
		if (err) return err;
		memcpy( *URLHandle, (LPCSTR) muxInfo->fileName, muxInfo->fileName.GetLength() + 1);
		err = MP4AddMediaDataReference(m_Media, &uDRefID, URLHandle, NULL); 
		if (err) return err;
		MP4DisposeHandle(URLHandle);
	}

	MP4NewHandle(0, &descriptorH);

	//Get the SpecificInfo
	if (decConfigDescr->decSpecificInfo != NULL_DESC) {
		//First, encode the DecSpecInfo
		decConfigDescr->decSpecificInfo -> EncDec (&m_coder, CALC_LENGTH, NULL);
		decConfigDescr->decSpecificInfo -> EncDec (&m_coder, ENCODE, NULL);
		DESC_TO_HANDLE;
		descH = descriptorH;
	} else {
		descH = NULL;
	}

	//Check for Object Clock References
	ES_Descriptor *pESD = (ES_Descriptor *) this;
	OCRId = pESD->OCR_ES_Id;
	
	priority = pESD->streamPriority;

	err = MP4NewHandle(0, &m_pSampleEntry); if (err) goto bail;
	
	//Create the sample description
	err = MP4NewSampleDescriptionWithOCRAssociation(m_Track, m_pSampleEntry,
						uDRefID,
						decConfigDescr -> objectTypeIndication, 
					 	decConfigDescr -> streamType,
						decConfigDescr -> bufferSizeDB,
					 	decConfigDescr -> maxBitrate,
						decConfigDescr -> avgBitrate,
						descH,
						OCRId); 
	if (err) goto bail;
	
	//set the priority
	err = MP4SetSampleDescriptionPriority(m_pSampleEntry, priority);
	if (err) goto bail;

	m_sampEnt = m_pSampleEntry;

	
	//add all the descriptors needed
	//first IPMP
	for (i=0; i<pESD->ipmpDescrPtr.GetLength(); i++) {
		//First, encode the DecSpecInfo
		m_coder.Reset();
		pESD->ipmpDescrPtr[i]->EncDec (&m_coder, CALC_LENGTH, NULL);
		pESD->ipmpDescrPtr[i]->EncDec (&m_coder, ENCODE, NULL);
		DESC_TO_HANDLE;
		//add the desc to the SampleEntry
		MP4AddDescToSampleDescription(m_pSampleEntry, descriptorH);
	}
	//then EXT
	for (i=0; i<pESD->extDescr.GetLength(); i++) {
		//First, encode the DecSpecInfo
		m_coder.Reset();
		pESD->extDescr[i]->EncDec (&m_coder, CALC_LENGTH, NULL);
		pESD->extDescr[i]->EncDec (&m_coder, ENCODE, NULL);
		DESC_TO_HANDLE;
		//add the desc to the SampleEntry
		MP4AddDescToSampleDescription(m_pSampleEntry, descriptorH);
	}
	//then IPIDataSet
	for (i=0; i<pESD->ipIDS.GetLength(); i++) {
		//First, encode the DecSpecInfo
		m_coder.Reset();
		pESD->ipIDS[i]->EncDec (&m_coder, CALC_LENGTH, NULL);
		pESD->ipIDS[i]->EncDec (&m_coder, ENCODE, NULL);
		DESC_TO_HANDLE;
		//add the desc to the SampleEntry
		MP4AddDescToSampleDescription(m_pSampleEntry, descriptorH);
	}
	//then language
	for (i=0; i<pESD->langDescr.GetLength(); i++) {
		//First, encode the DecSpecInfo
		m_coder.Reset();
		pESD->langDescr[i]->EncDec (&m_coder, CALC_LENGTH, NULL);
		pESD->langDescr[i]->EncDec (&m_coder, ENCODE, NULL);
		DESC_TO_HANDLE;
		//add the desc to the SampleEntry
		MP4AddDescToSampleDescription(m_pSampleEntry, descriptorH);
	}
	//then IPI_DescPtr
	if (pESD->ipiPtr != NULL_DESC) {
		//First, encode the DecSpecInfo
		m_coder.Reset();
		pESD->ipiPtr->EncDec (&m_coder, CALC_LENGTH, NULL);
		pESD->ipiPtr->EncDec (&m_coder, ENCODE, NULL);
		DESC_TO_HANDLE;
		//add the desc to the SampleEntry
		MP4AddDescToSampleDescription(m_pSampleEntry, descriptorH);
	}
	//then QoS
	if (pESD->qosDescr != NULL_DESC) {
		//First, encode the DecSpecInfo
		m_coder.Reset();
		pESD->qosDescr->EncDec(&m_coder, CALC_LENGTH, NULL);
		pESD->qosDescr->EncDec(&m_coder, ENCODE, NULL);
		DESC_TO_HANDLE;
		//add the desc to the SampleEntry
		MP4AddDescToSampleDescription(m_pSampleEntry, descriptorH);
	}
	//and Reg
	if (pESD->regDescr != NULL_DESC) {
		//First, encode the DecSpecInfo
		m_coder.Reset();
		pESD->regDescr->EncDec(&m_coder, CALC_LENGTH, NULL);
		pESD->regDescr->EncDec(&m_coder, ENCODE, NULL);
		DESC_TO_HANDLE;
		//add the desc to the SampleEntry
		MP4AddDescToSampleDescription(m_pSampleEntry, descriptorH);
	}


bail:
	if (descriptorH) MP4DisposeHandle(descriptorH);
	return err;
}




/*------------------------------------------------------------
Insert the stream in the media using StreamFactory 
to calculate the DTS and CTS of each AU
------------------------------------------------------------*/
MP4Err ES_DescriptorMP4::WriteSample(u32 *outDTS)
{
	MP4Err err;
	u32 duration;
	u32 DTSNext, DTS, CTS;
	float sc, tmp;
	MP4Handle sampleSize;
	MP4Handle sampleSync;
	MP4Handle sampleDuration;
	MP4Handle compositionOffset, compOff;

	err = MP4NoErr;
	err = MP4NewHandle(sizeof(u32), &sampleSync); if (err) goto bail;
	err = MP4NewHandle(sizeof(u32), &sampleDuration); if (err) goto bail;
	err = MP4NewHandle(sizeof(u32), &sampleSize);if (err) goto bail;
	err = MP4NewHandle(sizeof(u32), &compositionOffset);if (err) goto bail;


	sc = GetScale() / 1000;
	tmp = GetDTS();
	tmp *= m_pSLConfig->timeStampResolution / 1000.;
	DTS = tmp / sc;
	tmp = m_pStream -> GetCTS();
	tmp *= m_pSLConfig->timeStampResolution / 1000.;
	CTS = tmp / sc;

	//Get the composition offset ( = CTS - DTS) if needed
	if ( m_pSLConfig->useTimeStampsFlag && (m_pStream->GetCTS() >= 0) ) {		
		*(u32 *) *compositionOffset = ( CTS > DTS ) ? CTS - DTS : 0;
		compOff = compositionOffset;
	} else {
		compOff = NULL;
	}
	//store initial offset
	if (m_bIsFirst) {
		if (DTS) m_iInitTS = DTS;
		m_bIsFirst = FALSE;
	}

	*outDTS = DTS;
	//Is this AU a Random Access Point ??
	//If yes, then set the syncSample flag...
	if (m_pStream->IsRAP()) {
		err = MP4SetHandleSize(sampleSync, sizeof(u32)); 
		if (err) goto bail;
		*(u32 *) *sampleSync = 1;
	} else {
		err = MP4SetHandleSize(sampleSync, 0); if (err) goto bail;
	}

	// Get the duration of the AU (ie DTSNext - DTS)
	// First, reset TS to go to the next AU
	m_lTS = -1;

	duration = 0;
	tmp = GetDTS ();
	tmp *= m_pSLConfig->timeStampResolution / 1000. ;
	DTSNext = tmp / sc;

	if (GetDTS() == -1) {
		//Last AU -> duration = maxDuration
		duration = m_MaxDuration;
	} else {
		if (DTSNext > DTS) {
			duration = DTSNext - DTS;
			if (m_MaxDuration < DTSNext - DTS) {
				m_MaxDuration = DTSNext - DTS;
			}
		} 
	}
	if (!duration)
		duration = DEF_SAMPLE_DURATION * m_pSLConfig->timeStampResolution;

	*(u32 *) *sampleSize = m_nAULength;
	*(u32 *) *sampleDuration = duration;

	//and here's the trick for DataRef
	if (m_bSelfContained) {
		err = MP4AddMediaSamples(m_Media, 
								m_pSample, 
								1, 
								sampleDuration, 
								sampleSize, 
								m_sampEnt, 
								compOff, 
								sampleSync);	
		if (err) goto bail;
	} else {
		//first get the offset in the ref file: current pos - sizeof sample
		u64 end = ftell(m_fStream);
		err = MP4AddMediaSampleReference(m_Media, 
								end - m_nAULength,
								1,
								sampleDuration, 
								sampleSize, 
								m_sampEnt, 
								compOff, 
								sampleSync);
		if (err) goto bail;
	}

	//We use the same description entry for each sample
	if (m_sampEnt) {
		m_sampEnt = NULL;
	}


	//If the associated file is empty :(
	if (!m_nAULength) {
		err = MP4EOF;
	}

bail:
	MP4DisposeHandle(sampleSync);
	MP4DisposeHandle(sampleDuration);
	MP4DisposeHandle(sampleSize);
	MP4DisposeHandle(compositionOffset);
	return err;
}

MP4Err ES_DescriptorMP4::WriteBlankSample()
{
	return MP4AddSampleEntry(m_Media, m_sampEnt);
}

/*-----------------------------------------------------------------
Write one access unit to the sampleHandle
-----------------------------------------------------------------*/
void ES_DescriptorMP4::GetAU()
{
	BYTE packet[DEF_PACKET_SIZE];
	int nLength, nDataLength;

	m_nAULength = 0;
	m_bEndOfAU = false;
	int i;
	BYTE *bufferPrev = NULL, *buffer = NULL, *p;
	int prevSize = 0;

	//if Data Ref, just get the size of the au.
	if (!m_bSelfContained) {
		while (! m_bEndOfAU ) {
			int c = m_pStream->GetChar();
			m_bEndOfAU = c < 0;
			if (! m_bEndOfAU) {
				m_nAULength ++;
			}
		}
		return;
	}

	//else get the AU and store it...
	while (!m_bEndOfAU) {
		m_bEndOfAU = BuildPacket (packet, 0, nLength, nDataLength);
		assert(nLength>=0);
		m_nAULength += nDataLength;

		buffer = (BYTE *) calloc(prevSize+nLength, 1);
		memcpy(buffer, bufferPrev, prevSize);
		memcpy(buffer+prevSize, packet, nLength);
		if (bufferPrev) free(bufferPrev);
		prevSize += nLength;
		bufferPrev = (BYTE *) calloc(prevSize, 1);
		memcpy(bufferPrev, buffer, prevSize);
		free(buffer);
	}

	MP4SetHandleSize(m_pSample, prevSize);
	m_nAULength = prevSize;
	p = (BYTE*) *m_pSample;

	for (i=0; i<prevSize; i++) {
		*p++ = bufferPrev[i];
	}
	free(bufferPrev);

	/*-------------------------------------------------------------
	Make sure requested buffer length can hold all access units
	-------------------------------------------------------------*/
	if (m_nAULength > decConfigDescr -> bufferSizeDB) {
		printf("WARNING: access units too big: decoding buffer overflow"
			". File: %s\n",
			(LPCSTR) muxInfo -> fileName
		);
		g_bErrors = TRUE;
	}
	
}

/*-----------------------------------------------------------------
Fill the packet with AU datas. Return TRUE if the last packet of  
an AU was generated.
Note: We don't use any Sync Layer stuff here (MP4 File Format has its
own predefined ( == 2) SLConfig and does not handle any other one)
-----------------------------------------------------------------*/

/*********** Added by Liao Xuan **********************************/
//for encryption
#if defined(DES)
	typedef int (*EncFi)(char*, char*, char*);
	typedef int (*DecFi)(char*, char*, char*);
	typedef int (*DESPrototype)(char*, int, int, char*, char*);
	extern int	nAudioEncrypt;
	extern char DES_Keys[16*200];
#endif
/*********** End: Added by Liao Xuan *****************************/


BOOL ES_DescriptorMP4 :: BuildPacket (
	LPBYTE pPacket,
	int nMaxSize,
	int &rnLength, 
	int &rnDataLength
)
{
	if (!nMaxSize)
		nMaxSize = DEF_PACKET_SIZE;

	/*-----------------------------------------------------------------
	Fill the packet and return if end of AU or packet is full
	-----------------------------------------------------------------*/

	while (!m_bEnd && m_nPayloadLength <= nMaxSize)
	{	int c = m_pStream -> GetChar ();
		m_bEnd = c < 0;
		if (!m_bEnd) {
			m_payload [m_nPayloadLength++] = c;
		}
	}
/*********** Added by Liao Xuan **********************************/
//for audio DES encryption
#if defined(DES)
	// only encrypt 200 AUs 
	if (nAudioEncrypt>0 && indexEncryAU<200 &&((int)ES_ID)==2113 && m_nPayloadLength>=16)
	{	
		if (nAudioEncrypt==1)
		{
			char tmp[16];
			memcpy(tmp,m_payload,16);
			//HINSTANCE hInst;
		
			//load DES library
			hInstDes = LoadLibrary("desdll.dll");
			if(hInstDes == NULL)
			{
				printf("\nError: fail to load DES dll\n");
				exit(-1);
			}	
			DESPrototype Encryption;
			Encryption = (DESPrototype)GetProcAddress(hInstDes,"DES_Encryption");
		
			if(!Encryption) {
				printf("\nError: fail to load DES functions\n");
				exit(-1);
			}
			//16 byte DES key, hard code for testing
			int nKeyLength=16;
			char key[16];
			memcpy(key, DES_Keys+indexEncryAU*16,16);
			Encryption(key,nKeyLength,nKeyLength,tmp,tmp);
		
			//write back
			memcpy(m_payload, tmp, 16);
			indexEncryAU++;
		}//if
		else
			printf("Error: wrong Audio Encrypt Type");	
	}//if
#endif // DES
/*********** End: Added by Liao Xuan *****************************/



	/*-----------------------------------------------------------------
	Store the packet.
	-----------------------------------------------------------------*/
	int nPacketLength = nMaxSize - m_coder.GetDataLength ();

	if (nPacketLength <= 0) 
	{
		rnLength = -1;
		rnDataLength = 0;
		nPacketLength = 0;
	}
	else
	{
		if (nPacketLength > m_nPayloadLength)
			nPacketLength = m_nPayloadLength;

		int i = 0;

		memcpy (pPacket + i, m_payload, nPacketLength);
		rnLength = i += nPacketLength;
		rnDataLength = nPacketLength;

	/*-----------------------------------------------------------------
	The data that was not stored is shifted to the beginning of
	the buffer
	-----------------------------------------------------------------*/
		m_nPayloadLength -= nPacketLength;
		if (m_nPayloadLength > 0)
			memcpy (m_payload, m_payload + nPacketLength, m_nPayloadLength);
	}

	if (m_bEnd && m_nPayloadLength == 0)
	{	
		m_bStart = TRUE;
		m_bEnd = FALSE;
		return TRUE;
	}
	else
	{	
		if (nPacketLength > 0)
			m_bStart = FALSE;
		return FALSE;
	}
}


u32 ES_DescriptorMP4::GetHandlerType(u32 streamType)
{
	switch (streamType) {
	case 0x01 :
		return MP4ObjectDescriptorHandlerType;

	case 0x02:
		return MP4ClockReferenceHandlerType;

	case 0x03:
		return MP4SceneDescriptionHandlerType;

	case 0x04:
		return MP4VisualHandlerType;

	case 0x05:
		return MP4AudioHandlerType;

	case 0x06:
		return MP4MPEG7HandlerType;

	case 0x07:
		return MP4IPMPHandlerType;

	case 0x08:
		return MP4OCIHandlerType;

	case 0x09:
		return MP4MPEGJHandlerType;

	default:
		return MP4HintHandlerType;
	}
}




MP4Err ES_DescriptorMP4::CreateTrack()
{
	return MP4NewMovieTrackWithID(m_pMovie, GetTrackType(decConfigDescr -> streamType), 
					(u32) ES_ID, 
					&m_Track);
}

MP4Err ES_DescriptorMP4::CreateMedia()
{
	u32 timeScale = 0;
	MP4Err err = MP4NoErr;

	/*-------------------------------------------------------------
	Get the SLConfigDescr
	-------------------------------------------------------------*/
	//AC:20021009 - SL extension : esMP4 : AbstractSLConfig  -- begin
	/*SLConfigDescriptor*/AbstractSLConfig configDefault;	
	AbstractSLConfig *pSLConfig = slConfigDescr;
	//SLConfigDescriptor configDefault;
	//SLConfigDescriptor *pSLConfig = slConfigDescr;
	//AC:20021009 - SL extension : esMP4 : AbstractSLConfig  -- end
	if (pSLConfig == NULL)
		pSLConfig = &configDefault;

	//In the API, media timeScale == timestampResolution (default DEF_TRACK_TIMESCALE)
	timeScale = (u32) pSLConfig->timeStampResolution;
	if (!timeScale) {
		timeScale = DEF_TRACK_TIMESCALE;
	}
	
	//Add a movie container to that track (it may be an URL)

	err = MP4NewTrackMedia(m_Track, &m_Media, 
			GetHandlerType(decConfigDescr -> streamType),
			timeScale, 
			NULL); if (err) return err;
	return MP4SetMediaLanguage(m_Media, "und");


}

u32 ES_DescriptorMP4::GetTrackType(u32 streamType)
{
	switch (streamType) {
	case 0x04:	//Visual is Visual
		return MP4NewTrackIsVisual;

	case 0x05:
		return MP4NewTrackIsAudio;

	default:
		return MP4NewTrackIsPrivate;
	}
}


MP4Err ES_DescriptorMP4::TerminateWrite()
{
	MP4Err err = MP4NoErr;
	u64 mediaDuration;
	/*************************************************************
	Insert media in the track
	if bIsInIOD, this stream is in the initial Object descriptor 
	-> we have to add it
	Then set the track enable
	**************************************************************/

	//EndMediaEdit -> compute the duration
	err = MP4EndMediaEdits(m_Media); if (err) return err;
	err = MP4GetMediaDuration(m_Media, &mediaDuration ); if (err) return err;
	err = MP4InsertMediaIntoTrack( m_Track, muxInfo ? muxInfo->startTime : 0, 0, mediaDuration, 1 ); if (err) return err;

	if (m_bIsInIOD) {
		err = MP4AddTrackToMovieIOD(m_Track); 
		if (err) return err;
	}
	if (m_pSampleEntry) MP4DisposeHandle(m_pSampleEntry);

	MP4SetTrackEnabled( m_Track, 1); 

	if (m_iInitTS) {
		return MP4SetTrackOffset(m_Track, m_iInitTS);
	}
	return err;
}	



/*-----------------------------------------------------------------
The StreamFactoryBase constructor registers a factory object
-----------------------------------------------------------------*/
StreamFactoryBase :: StreamFactoryBase (LPCSTR szType)
{
	m_szType = szType;
	ES_DescriptorMP4 :: RegisterFactory (this);
}

/*-----------------------------------------------------------------
Check if a stream factory object is of a required type
-----------------------------------------------------------------*/
BOOL StreamFactoryBase :: IsTypeOf (LPCSTR szType)
{
	return
		(	szType != NULL && m_szType != NULL &&
			!stricmp (szType, m_szType)
		);
}



