/***********************************************************************
This software module was originally developed by Jean Le Feuvre
(avipix, llc) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. avipix, llc retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2000.
************************************************************************/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  MP4 File Writer
FILE:       main.cpp
AUTHOR:     Jean Le Feuvre (avipix, llc, jean@avipix.com), December 2000
OVERVIEW:   main: main() function for the MP4 File Writer
-------------------------------------------------------------------------*/

// SOLARIS PORTING - include direct.h for WIN32
#ifdef WIN32
#include <direct.h>
#else // WIN32

#ifndef _chdir
#define _chdir(val) chdir(val)
#endif

#ifndef DeleteFile
#define DeleteFile(val) unlink(val)
#endif

#endif // WIN32

#include "defs.h"
#define MAIN
#include "MP4Enc.h"
#include "MP4Dump.h"
#include "Token.h"
#include "SysCoder.h"
#include "VrmlPars.h"


/***********Added by Liao Xuan *************/
#if defined(DES)
#include<stdlib.h>
#include<time.h>
#endif //DES	
/***********End: Added by Liao Xuan ********/

BOOL g_bErrors;
Token nextToken;

/************ Added by Liao Xuan ****************************/
#if defined(DES)
int nAudioEncrypt;
char DES_Keys[16*200];//contain 200 DES keys
#endif //DES
/***************End: Added by Liao Xuan **********************/

void PrintUsage()
{
	printf ("MP4Enc [-w -i -m [-t] || -r [-d -f] ] inputfile\n"
			"Writing Options:\n"
			"-w: write an MP4 file: *.script->*.mp4\n"
			"-m: set all track duration to max duration\n"
			"-i: write the samples interleaved\n"
			"-t: interleaving detailed output in M4E_*.trc\n"
			"\nReading Options:\n"
			"-r: read an MP4 file and print info: *.mp4->M4D_*.trc\n"
			"-d: dump each track to files (name at prompt)\n"
			"-f: forces track dump in files:\n"
			"\tM4D_*_[TrackID].trc: SampleDescription and AU timing,\n"
			"\tM4D_*_[TrackID].raw: stream raw data\n"
			/*******Added by Liao Xuan ********/
			"\nEncryption Algorithm:\n"
			"--audio: Audio encryption\n"
			"-des: DES algorithm\n"
			/*******End: added by Liao Xuan ***/
			"\nMP4Enc v3.2 - Written by Jean Le Feuvre - (c) 2000 - 2003\n");

}

int main (int argc, char **argv)
{
	FILE *fList, *fTrace, *fout;
	int nNameLength;
	ZString stMovieName ("");
	ZString stTraceName ("");
	ZString stFileName("");
	ZString stPathName("");
/***********Added by Liao Xuan *************/
#if defined(DES)
	ZString strParam("");
#endif //DES	
/***********End: Added by Liao Xuan ********/
	int i;
	char *arg;
	BOOL bTraceActive = FALSE;
	int ReadMode = -1;
	BOOL dumpRequest = FALSE;
	BOOL forceDump = FALSE;
	BOOL bInter = FALSE;
	BOOL bMaxDur = FALSE;

	fout = NULL;

/************ Added by Liao Xuan ****************************/
#if defined(DES)
	//Default algorithm is None
	nAudioEncrypt=-1;
	char *mp4FileName=new char[255];
#endif //DES
/***************End: Added by Liao Xuan **********************/

	//parse our agrs
	for (i = 1; i < argc ; i++) {
		arg = argv[i];
		if (arg[0] != '-') 
		{
			stPathName = arg;
			break;
		}		
		//parse the flags
		switch (arg[1]) {
		case 'w':
			ReadMode = 0;
			break;
		case 's':
			fout = freopen("stdout.txt", "wt", stdout);
			break;
		case 'r':
			ReadMode = 1;
			break;
		case 't':
			bTraceActive = TRUE;
			break;
		case 'd':
			dumpRequest = TRUE;
			break;
		case 'f':
			forceDump = TRUE;
			break;
		case 'i':
			bInter = TRUE;
			break;
		case 'm':
			bMaxDur = TRUE;
			break;
/************ Added by Liao Xuan ****************************/
#if defined(DES)
		case '-':	//for encryption.
			strParam=argv[i];
			if(strcmp(strParam,"--audio")==0)
			{
				//audio encryption specified
				i++;	//next argument should be the audio encryption algorithm
				strParam=argv[i];
					if(strcmp(strParam,"-des")==0)
					{
						nAudioEncrypt=1;
						break;	//break for next loop
					}
				PrintUsage();
				return -1;
			}
			PrintUsage();
			return -1;	
#endif //DES 
/***************End: Added by Liao Xuan **********************/		
		default:
			PrintUsage();
			return -1;
		}//switch
	}//for

	nNameLength = stPathName.GetLength ();
	stFileName = strrchr (stPathName, '\\');
	if (stFileName.GetLength()) {
		stPathName.Cut(stPathName.GetLength() - stFileName.GetLength() + 1);
		_chdir(stPathName);
		stFileName.DeleteFrom(0);
	} else {
		stFileName = stPathName;
	}
	

	if (!nNameLength) {
		printf("Missing file name\n\n");
		PrintUsage();
		return 0;
	}

	//Quick hack to set to write if -w if not set
	if (ReadMode == -1) {
		if (dumpRequest || forceDump) {
			PrintUsage();
			return 0;
		}
		ReadMode = 0;
	} else if (ReadMode == 1) {
		//Dump the file
		MP4Dump dumper(stFileName, dumpRequest, forceDump);
		dumper.Dump();
		return 0;
	}
	

	/*-----------------------------------------------------------------
	Source : ".scr", output ".mp4" and list is ".trc"
	-----------------------------------------------------------------*/

	nNameLength = stFileName.GetLength();
	LPCSTR szExt = strrchr(stFileName, '.');
	if (szExt && !strchr (szExt, '\\'))
		nNameLength -= strlen (szExt);
	else
		stFileName += ".scr";

	VRMLParser parser;
	if (!parser.Init (stFileName))
	{	printf ("Cannot open %s\n", (LPCSTR) stFileName);
		parser.Shutdown ();
		return -1;		
	}
/************ Added by Liao Xuan ****************************/
#if defined(DES)
	if(nAudioEncrypt==1) 
	{
		//	read DES key from DESkey.txt
		FILE *fpDESkey = fopen("DESkeysFile.txt","r"); //should be the first time to open the key file
		if(!fpDESkey)
		{
			printf ("DESkeysFile failed to open");
			return -1;
		}
		fread(DES_Keys, sizeof(DES_Keys),1,fpDESkey);
		fclose(fpDESkey);
		char buffer[4000];
		sprintf (buffer,"DES KEYs: \n%s\n",DES_Keys);
		OutputDebugString (buffer);
	}
#endif //DES
/***************End: Added by Liao Xuan **********************/

	stFileName.Cut (nNameLength);
	stFileName += ".mp4";
	stMovieName = stFileName;

	stTraceName = "M4E_";
	stFileName.Cut (nNameLength);
	stFileName += ".trc";
	stTraceName  += stFileName;

	stFileName.Cut (nNameLength);
	stFileName += ".trc";
	if ((fList = fopen (stFileName, "w")) == NULL)
	{	printf ("Cannot open %s\n", (LPCSTR) stFileName);
		return -1;
	}
	
	fTrace = NULL;
	if (bTraceActive && bInter) {
		fTrace = fopen((LPCSTR) stTraceName, "wt");
		if (fTrace == NULL) 
			printf("Warning: cannot open trace file...\n");
	}

	printf("Writing file %s...\n", (LPCSTR) stMovieName);
	//Init text parsing
	nextToken.Init (&parser, stFileName, fList);
	//Start encoding
	MP4Enc((LPCSTR) stMovieName, bInter, bMaxDur).Start (fList, TRACE_NONE, fTrace);

	fclose(fList);

	printf ("...done\n");

	if (g_bErrors) {
		printf ("Hit any key to continue\n");
		getchar ();
		return -1;
	}

	if (fTrace) fclose(fTrace);
	if (fout) fclose(fout);
	DeleteFile("stdout.txt");
	return 0;
}