/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

#if !defined(_odMux_h)  // Sentry, use file only if it's not already included.
#define _odMux_h

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Multiplexer
FILE:       odMP4.h
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Derived special versions of OD descriptors, for the use of 
			the Mux tool.
-------------------------------------------------------------------------*/


/*------------------------------------------
File: odMP4.h
Author: Jean Le Feuvre, E-Vue Inc, March 2000

Content: track writer
------------------------------------------*/

#include "EncOD.h"
#include "ODCoder.h"
#include "MP4Defs.h"


#include "MP4OSMacros.h"
#include "MP4Movies.h"

class ES_DescriptorMP4 : public ES_DescriptorEnc
{
	/*--------------------
	Methods
	--------------------*/
public:
	ES_DescriptorMP4 ();
	~ES_DescriptorMP4 ();

	//Opening of stream file
	int OpenFile (FILE *fList, int flags);
	int GetScale ();	// Guido 4/8/00
	long GetDTS (long lNow = 0);
	static void RegisterFactory (class StreamFactoryBase *pFactory);

	//MP4 functions
	BOOL AddTrackToMovie(MP4Movie Movie, BOOL bIsInIOD);

	int GetStreamType();
	MP4Err CreateTrack();
	MP4Err CreateMedia();
	MP4Err WriteToTrack(BOOL bInIOD);

private:
	MP4Err InsertStreamInMedia();

	//Writing with StreamFactory
	MP4Err WriteUsingStreamFactory(SLConfigDescriptor *pSLConfig);
	BOOL BuildPacket (LPBYTE pPacket,int nMaxSize,int &rnLength, int &rnDataLength);
	void Store();
	u32 GetHandlerType(u32 streamType);
	u32 GetTrackType(u32 streamType);

	/*--------------------
	Members Variables
	--------------------*/
private:
	//Stream Factory 
	class Stream *m_pStream;
	FILE *m_fStream;
	static class StreamFactoryBase *m_pFactories [40];
	static int m_nFactoriesAvailable;

	//Building AUs 
	long m_lTS;
	int m_nScale;	// Guido 4/8/00
	int m_nPayloadLength;
	bool m_bStart, m_bEnd;
	BYTE m_payload [DEF_PACKET_SIZE];
	int m_nAULength;
	BOOL m_bEndOfAU;

	//We still need the coder to encode the DecoderSpecificInfo
	ODCoder m_coder;

	//MP4 
	MP4Track m_Track;
	MP4Media m_Media;
	MP4Movie m_pMovie;
	MP4Handle m_pSampleEntry;
	MP4Handle m_pSample;

};

#endif	// _odMux_h sentry