/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************
/*-------------------------------------------------------------------------
MPEGJ 
AUTHORS:    Viswanathan Swaminathan - vishy@eng.sun.com 
			Robert Schmidt(RLS)
			Praveen Alavilli(PA) 
			YUNTAI KYONG(YT) - yuntaikyong@ieee.org
-------------------------------------------------------------------------*/

#include "defs.h"
#include "MPEGJDec.h"
#include "Stream.h"
#include <jni.h>
#include <assert.h>
#include "defs.h"
#include <fstream.h>

//Bindu
#include "Executiv.h"
#include "AL.h"
#include "../Mpgj/sources/JVMSupport.h"
#include "../Mpgj/sources/DMIFbridge.h"

// SOLARIS PORTING
#ifdef WIN32
#include "trace.h"
#include <direct.h>
#endif

#include <math.h>
#include <iostream.h>
#include <stdio.h>
#include <stdlib.h>


#include "Root.h"
#include "StreamCo.h"
#include "Inline.h"
#include "ODImp.h"


#include "../mpegj-player/MPDecoderInterface.h"

// SISF-MERGE - PA
#include "../Mpgj/SI_api_Native/SFilterBridge.h"

// BB: Necessary for the SF startfunction.
SommitSfInterface l_cpp = NULL;
void CstartFilter( int, int, SommitSfInterface , int, unsigned char * , unsigned char * , unsigned char *, unsigned char *, int, int);
void CstopFilter( int );
// END SISF-Merge

//BINDU
FILE* bintraceFile;

#ifdef SIA
// SOLARIS PORTING
#ifdef WIN32
extern DWORD WINAPI Myfunc(ALManager *foo); 
#endif // WIN32
#endif

//BINDU

/*-------------------------------------------------------------
The exported function that instantiates the proper decoder
-------------------------------------------------------------*/
void *CreateInstance ()
{
	return new MPEGJDecoder;
}


/*-------------------------------------------------------------
Sets the decoder to CIF or QCIF
-------------------------------------------------------------*/
BOOL MPEGJDecoder :: Setup (
	const BYTE *pSpecificInfo,
	int nSpecificInfoLength,
	int 
)
{
	return TRUE;
	nSpecificInfoLength;
	pSpecificInfo;
}

/*-----------------------------------------------------------------
Find a compatible format and initialize the decoder
-----------------------------------------------------------------*/
BOOL MPEGJDecoder :: SetFormat (
	Capabilities &formatOptions,
	DecoderParams &decoderParams
)
{
	decoderParams.m_format.m_code = MediaFormat :: MPEGJ;
	decoderParams.m_nOptimalAUSize = 256;
	decoderParams.m_nDBSize = 10000; // PA - MPEGJ
	decoderParams.m_nCBSize = GetOptimalOutputSize () * 5;
	return TRUE;
	formatOptions;
}

/*-----------------------------------------------------------------
Calculate the output size in bytes
-----------------------------------------------------------------*/
int MPEGJDecoder :: GetOptimalOutputSize ()
{
	return 100000;
}


/*-----------------------------------------------------------------
GetByte() retrieves a new byte from the source file
-----------------------------------------------------------------*/
int MPEGJDecoder :: GetByte ()
{
	
	m_nLastByte = *m_pData++;
	m_nLastBit = 8;		// The whole byte is consumed, unless this
						// function is called from GetBits, which will
						// reset m_nLastBit to zero.
	++m_nCount;
	return m_nLastByte;	
}


/*-----------------------------------------------------------------
GetBits () retrieve nBits bits from the byte stream, returning them as 
an int.
-----------------------------------------------------------------*/
int MPEGJDecoder :: GetBits(int nBits)
{
		int nResult = 0;

	while (nBits-- > 0)
	{
		/*-----------------------------------------------------------------
		Get next byte, if necessary
		-----------------------------------------------------------------*/
		if (m_nLastBit >= 8)
		{	m_nLastByte = GetByte ();
			m_nLastBit = 0;
		}

		/*-----------------------------------------------------------------
		Retrieve the first bit of the byte, shift the byte so next time
		second bit will be the first
		-----------------------------------------------------------------*/
		nResult <<= 1;
		nResult |= (m_nLastByte & 0x80) >> 7;
		m_nLastByte <<= 1;
		++m_nLastBit;
	}
	return nResult;
}


/* Return one character */
int MPEGJDecoder :: GetChar()
{
	return GetBits(8);
}



/****** thread procedures to throw events onto the Java side - PA ******/

/* global referance for attaching current thread to the java virtual Machine */

/* local function for function table ptr */
int MythrowEvent(MPJTerminalDispatchTable* fnPtr , JNIEnv* this_env ) {
	static jclass mycls;
	static jmethodID mymid, mymid2;
	static jstring myjstr;
	static jobjectArray myargs;


	/* load the classes if this is the first time the thread is running */
	if( fnPtr->isFirstTime == 1 ) {
		// load the class to call main method in testMain class
		mycls = this_env -> FindClass("org/iso/mpeg/mpegjRefImpl/EventSupport");
		if (mycls == 0) 
		{
			TRACE("MPEGJ decoder: Can't find EventSupport class\n");
			assert(0);
		}
		this_env -> ExceptionDescribe();
		mymid = this_env->GetStaticMethodID(mycls, "rendererDispatcher", "(I)V");
		if (mymid == 0) 
		{
			TRACE("MPEGJ decoder : Can't find main method in ClassLoader.. \n");
			assert(0);
		}
		this_env -> ExceptionDescribe();
		//*** YT START (144)
		// signature is changed from (IJ)V to (IJJ)V (one more long type)
		// mymid2 = this_env->GetStaticMethodID(mycls, "decoderDispatcher", "(IJ)V");
		mymid2 = this_env->GetStaticMethodID(mycls, "decoderDispatcher", "(IJJ)V");
		//*** YT END (144)
		if (mymid2 == 0) 
		{
			TRACE("MPEGJ decoder : Can't find main method in ClassLoader.. \n");
			assert(0);
		}
		this_env -> ExceptionDescribe();
		
		fnPtr->isFirstTime = 0;

	} // end of if first time

	jint condition;
	jlong decoderOD;
	
	jlong decoderESID; // YT (126)
	while ( fnPtr->dEQ != NULL ) {			
		//decoder events
		condition = fnPtr->dEQ->condition;
		decoderOD = fnPtr->dEQ->ODID;
		//** YT START (127)
		// We are interested in ESID also
		decoderESID = fnPtr->dEQ->ESID;
		// call event dispatch function with additional parameter( ESID )
		(this_env)->CallStaticVoidMethod( mycls, mymid2, condition,decoderOD,decoderESID);
		//** YT END (127)
		this_env->ExceptionDescribe();
		fnPtr->dEQ = fnPtr->dEQ->next;
	}

	while( fnPtr->rEQ != NULL ) {		
		// renderer events
		condition = fnPtr->rEQ->condition;				
		(this_env)->CallStaticVoidMethod( mycls, mymid, condition);
		this_env->ExceptionDescribe();		
		fnPtr->rEQ = fnPtr->rEQ->next;
	}
	return 1; 
}

// SOLARIS PORTING
#ifdef WIN32
DWORD WINAPI ThreadProc(LPVOID lpParameter)
{
	((MPEGJDecoder*)lpParameter)->eventThread();
	return 0;
}
#else
void ThreadProc(MPEGJDecoder * dec)
{
    dec->eventThread();
}
#endif // WIN32




// thread to throw events from the player
void MPEGJDecoder::eventThread() 
{
	MPJTerminalDispatchTablePtr di = MPJGetTerminalDispatchTable(); 
	JavaVM *pJVM = m_jvmSupport.getJVM();
	// handle the function table which will be used to check whether the JVM is started or not and 
	// to throw events once JVM is started

	jint res;
	static JNIEnv *this_env;
	while ( di->isJVMStarted != 1 ) Sleep(100);
	
	if( di->isJVMStarted == 1 ) { // if jvm is started then dispatch the events
// PORTING to Java 2
//		res = pJVM->AttachCurrentThread( &this_env, NULL);
		res = pJVM->AttachCurrentThread( (void **)&this_env, NULL);
		if( res < 0 ) {
// SOLARIS PORTING
#ifdef WIN32
			AfxMessageBox("Attach thread failed");
#endif
			return ;
		}		
		/* infinite loop - but doesn't take the CPU until ThreadResume is called from some where else */
		di->isStartedDispatching = 1;		 
		while(di->isJVMStarted) { 				
			di->throwEvent(di , this_env);	
// SOLARIS PORTING - Pending
#ifdef WIN32
			SuspendThread(di->hThrow);				
#else
			thr_suspend( *(thread_t *)(di->hThrow) );
#endif
		} // end of while - never comes here  
		pJVM->DetachCurrentThread();		
	}
}
// end of events thread related code


void MPEGJDecoder::startJavaVirtualMachine()
{
	char currentDir[256];
	getcwd(currentDir, 256);
	chdir(currentDir);
	
	// Add to the CLASSPATH some java libraries needed by the player
	char additionalClassPath[1000]; // contains the new CLASSPATH 
											 // to be passed to StartJVM
	strcpy(additionalClassPath, ""); // init
// WIN-SOL Release - Take off the additional classpath
// SOLARIS PORTING
#ifdef WIN321
	//strcat(additionalClassPath, currentDir);
	//strcat(additionalClassPath, "\\..\\..\\..\\MPEGJ\\java;");
	
	// SISF-Merge - PA
	// BB: Functional testplatform added to classpath for SF support.
	//strcat(additionalClassPath, currentDir);
	//strcat(additionalClassPath, "\\..\\..\\MPGJ\\java;");
	//strcat(additionalClassPath, currentDir);
	//strcat(additionalClassPath, "\\..\\..\\MPGJ\\func_test_platf;");
	// end SISF Merge
    additionalClassPath[0] = '\0';
#else
    additionalClassPath[0] = '\0';
#endif
	
// SOLARIS PORTING
#ifdef WIN32
	char additionalPath[_MAX_PATH];  // contains the new PATH to be passed 
									 // to StartJVM
#else
    char additionalPath[PATH_MAX];
#endif

// WIN-SOL Release - Take off the additional path
// SOLARIS PORTING
#ifdef WIN321
	//strcpy(additionalPath, ""); // init
	//strcat(additionalPath,".;");
	
	//	strcat(additionalPath, currentDir);
	//	strcat(additionalPath, "..\\..\\Debug;");
	//strcat(additionalPath, currentDir);
	//strcat(additionalPath, "..\\..\\Mpgj\\lib;");
	//strcat(additionalPath, currentDir);
	//strcat(additionalPath, "..\\..\\Debug;");
    additionalPath[0] = '\0';
#else
    additionalPath[0] = '\0';
#endif
	 
	m_jvmSupport.startJVM(additionalPath, additionalClassPath);
	
	// We don't need additional classpath or path .. YT
	additionalClassPath[0] = '\0';
	additionalPath[0] = '\0';
	JNIEnv *pEnv = m_jvmSupport.getEnv();
		
	// load the class to intialize the multicaster queues
	jclass mc = pEnv -> FindClass("org/iso/mpeg/mpegjRefImpl/EventSupport");
	if (mc == 0) {
		TRACE("MPEGJ decoder: Can't find Eventsupport class\n");
		assert(0);
	}
	pEnv -> ExceptionDescribe();
	
	jmethodID mm = pEnv->GetStaticMethodID(mc, "initializeMulticaster", "(I)V");
	if (mm == 0) {
		TRACE("MPEGJ decoder : Can't find initiliazeMulticaster method in EventSupport Class.. \n");
		assert(0);
	}
	pEnv -> ExceptionDescribe();

	/***** for getting the ObjectDescriptor Id of this 
	decoder and number of decoders currently running - PA */
	InlineImp* m_pInline;
	m_pInline = m_pExecutive->GetInline();
	int len = m_pInline->m_streamAssociation.GetLength();	
	
	// now we know that there are len # of decoders
	// start queues for them on the java side
	jint num = len;
	pEnv->CallStaticVoidMethod( mc, mm, num);
	jthrowable obj1;
// SOLARIS PORTING
#ifdef WIN32
	if( ( obj1 = pEnv->ExceptionOccurred() ) != NULL )
		AfxMessageBox("Exception occured while : calling initializeMulticaster method");		
#endif

	/*** PA ****/

// SOLARIS PORTING - Ignore SectionFilterImp
#ifdef WIN32
	// SISF - Merge - PA
	// BB: initialize the SectionFilterImp library with a valid Java virtual machine
	initSectionFilterImpLibrary(&m_jvmSupport);
	//End SISF Merge
#endif

	// Find the MPEGJ loader in the system from the class path.
	m_cMPEGJClassLoader = pEnv -> FindClass("org/iso/mpeg/mpegjRefImpl/MPEGJClassLoader");
		
	jthrowable obj;
// SOLARIS PORTING
#ifdef WIN32
	if( ( obj = pEnv->ExceptionOccurred() ) != NULL )
		AfxMessageBox("Exception occured while : loading mpegjclassloader class");
#endif

	if (m_cMPEGJClassLoader == 0) {
		TRACE("MPEGJ decoder: Can't find MPEGJClassLoader class\n");
		assert(0);
	}

// SOLARIS PORTING
#ifdef WIN32
	if( ( obj = pEnv->ExceptionOccurred() ) != NULL )
		AfxMessageBox("Exception occured while: loading loadnow method ");
#endif

	m_midLoadZIP = pEnv->GetStaticMethodID(m_cMPEGJClassLoader, "loadZipArchive", "([B)V");
	if (m_midLoadZIP == 0) {
		TRACE("MPEGJ decoder : Can't find loadZipArchive method in ClassLoader.. \n");
		assert(0);
	}
}

/*-------------------------------------------------------------
Decode one unit
-------------------------------------------------------------*/
BOOL MPEGJDecoder :: Decode (
		LPBYTE pInput,
		int nInputLength,
		LPBYTE pOutput,
		int &rnOutputLength
)
{
	rnOutputLength = 0;
	pOutput = NULL;
	// At this point you have all the header information.....
	// Process the header info. and check if all the reqClasses are already
	// loaded. If not wait for them to be loaded before you load this class.

	// every unit that comes in to be decoded has a header
	JavaStreamHeader javaStrHeader;
	/*
	char temp[256];
	sprintf(temp,"%d",nInputLength);
	AfxMessageBox(temp);
	*/
	ProcessHeader(pInput, *&nInputLength, javaStrHeader);
	/*
	sprintf(temp,"%d",nInputLength);
	AfxMessageBox(temp);
	*/
	JNIEnv *pEnv = m_jvmSupport.getEnv();

//	SOLARIS PORTING - commenting if non-WIN32
#ifdef WIN32
	// SISF -Merge - PA
	// BB: Added the initialisation of the section filter engine.
	setStartPunt(CstartFilter);
	setStopPunt(CstopFilter);
	//End -SISF Merge
#endif


	/*
	here the JVM has been created. Now invoke the MPEGJClassLoader and args passed to 
	it is the bytearray coming in as input to decode. - IF the isClassFlag is set.
	if it isClassFlag = 0 then it is just object data and needs to be instantiated.
	*/
	
	// if isClassFlag is set - 2 cases further to be considered - isPackaged and 
	// notPackaged. In either case - find the loadNow method of the MPEGJClassLoader
	// first.
	// Note - here we may be unnecessarily be doing a FindClass("MPEGJClassLoader") 
	// multiple times - set a flag to do it only for the first AU carrying "class 
	// data" ?? 

	if(javaStrHeader.isClassFlag) {
		if(javaStrHeader.isPackaged) { 
			// This is the "if packaged" loop - in this case : load all classes in the 
			// reqClasses list. loadNow returns a class. Pass this as argument to 
			// function "RunIfMPEGlet" whcih will do what the name suggests - executes
			// the bytecode if it impelments the interface MPEGlet. all the classes should
			// be loaded after the Start Loading time stamp and before the Load By
			// Time stamp associated with the access unit carrying the packaged file.
			// Now: as it stands - i am hardcoding the name of the file AtlanticDemo and
			// checking if it is an MPEGlet and running it. This is because the MPEGJClass
			// Loader still needs to implement the reqClasses list as containing all the
			// classes archived in the zip file. 
			// actually what needs to be done is:

			// for (int k=0; k< javaStrHeader -> numReqClassses; k++)
			// {
			//		1. - load class and on success, i.e. if the loader
			//           does not return null then add it to the listOfLoadedClasses 
			//      2. - invoke RunIfMPEGlet.startMPEGlet on the loaded class 
			//  }

			// for now with knowledge of what is archived in the zip file, since Atlantic
			// Demo implements MPEGlet its name is hard coded and this piece of byte code
			// is executed.
			jsize size = nInputLength;
			jbyteArray data = pEnv->NewByteArray(size);
			ExceptionMessageBox("Error in MPEGJ: Failed to create java byte array");
					
			jbyte *pBAInput = new jbyte[size];
			memcpy(pBAInput, m_pData, size);
			pEnv->SetByteArrayRegion(data, (jsize) 0, (jsize) size, pBAInput);
			pEnv->CallStaticObjectMethod(m_cMPEGJClassLoader, m_midLoadZIP, data);
			ExceptionMessageBox("Error in MPEGJ: Failed in calling loadZip method of MPEGJClassLoader");
		}
		else { 
			// Non-Packaged case
	
			//Name of the class which has been streamed. Stored in the header
			jstring jstr = pEnv->NewStringUTF((javaStrHeader.classID).ID); 
			ExceptionMessageBox("Error in MPEGJ: Creating new jstring");
			
			jsize size = nInputLength;
			jbyteArray data = pEnv -> NewByteArray(size);
			ExceptionMessageBox("Error in MPEGJ: Creating new jbyte array");
					
			jbyte *mypInput = new jbyte[size];
			memcpy(mypInput, m_pData, size);
			pEnv->SetByteArrayRegion(data, (jsize) 0, (jsize) size, mypInput);

			ExceptionMessageBox("Setting jbyte array region");
			

			// have already been loaded. Then load the class if the time is right...
			pEnv->CallStaticObjectMethod(m_cMPEGJClassLoader,m_midLoadNow,data,jstr);
		}/* end of the else  non-packaged case  */
	}/* end of if(isClass) */

	else {  
		// not isClass case 
		// do what needs to be done in case of an object.
		// i.e. instantiate an object. 
	}

	return TRUE;
}

/*-------------------------------------------------------------
In this decoder, the decoded data always starts at beginning of output
block
-------------------------------------------------------------*/
int MPEGJDecoder :: GetDataOffset  (const LPBYTE)
{
	return 0;
}

/*-------------------------------------------------------------
In this decoder, attributes are fixed for all units
-------------------------------------------------------------*/
DWORD MPEGJDecoder :: GetAttributes (
	ATTRIBUTE_CODE code, 
	const LPBYTE)
{
	code;
	return 0;
}


void MPEGJDecoder :: Run ()
{
	LPBYTE pInData, pOutData = NULL;
	int nInSize, nOutSize, rnStatus;
	DWORD dwTime;
	BOOL bEOU;

	m_pExecutive = GetExecutive();
	assert(m_pExecutive);
	m_pInline = m_pExecutive->GetInline();
	assert(m_pInline);
	
	/*** NOTE *** This function no longer exists in the Executive class.
	 * I am assigning a nul value to the pointer to facilitate compilation.
	 * This will have to be addressed to get the MPEGJ Decoder working
	 * again - RLS
	 */ 
	m_pALM = m_pExecutive->GetALManager ();
	assert(m_pALM);
	initLib(m_pALM);
	
	Decoder* dec;
	for( int i=m_pInline->m_streamAssociation.GetLength()-1; i >= 0; i--) {		
		for(int j = m_pInline->m_streamAssociation[i].m_pOD->esDescr.GetLength()-1;j>=0;j--) {
			dec =  ((ES_DescriptorImp *)(m_pInline->m_streamAssociation[i].m_pOD->esDescr[j]))->GetDecoder();
			if( this == dec ) {
				ESID = m_pInline->m_streamAssociation[i].m_pOD->esDescr[j]->ES_ID;
				ODID = m_pInline->m_streamAssociation[i].m_pOD->ObjectDescriptorID; 
			}
		}
	}

	// MPEGJ PORTING - Use FetchNext rather than Fetch
	// Consistent with DecoderImp::Run model
   	while (
		(pInData =
			(LPBYTE) GetInputStream () -> FetchNext (nInSize, dwTime, bEOU, TRUE)
			) != END_OF_STREAM ) {
		if(pInData == NULL)
			break;
		if (IsActive () && pInData) {
			// If this is the first AU - then start the JVM 
			if(m_bFirstMPEGJAU) {
				startJavaVirtualMachine();
				throwEvent(START_EVENT);
			}
			Decode (pInData, nInSize, pOutData, nOutSize);
			
			/***  intialize table pointers and status varibles - PA */
			if(m_bFirstMPEGJAU) {
				di = MPJGetTerminalDispatchTable();
				di->throwEvent = MythrowEvent;
				di->isJVMStarted = 1;
				di->isFirstTime = 1;
				di->isStartedDispatching = 0;
				DWORD dw;
// SOLARIS PORTING - Pending
#ifdef WIN32
				di->hThrow = CreateThread(NULL, 0, ThreadProc, (LPVOID) this, 0,&dw);
#else
// Changing to Solaris threads
// For the ease of thr_suspend and thr_continue
//		pthread_t thread;
//		int m_hThread = 
//			::pthread_create (
//				&thread,
//				NULL,
//				(void *(*)(void*)) ThreadProc,
//				this
//			);
		int m_hThread =
			::thr_create( NULL, NULL,
				(void *(*)(void*)) ThreadProc,
				this, NULL, &m_thread );
		di->hThrow = (void *)(&m_thread);
#endif
				m_bFirstMPEGJAU = false;
			}
		}
	}
	throwEvent(END_EVENT);
	//m_jvmSupport.stopJVM();
}

void MPEGJDecoder::throwEvent(EVENT_CODE code)
{	
	 /**** get the instance of the function pointer table. - PA ****/ 
    di = MPJGetTerminalDispatchTable(); 
	// set the status variable 
   	if( di != NULL ) {
    	// go till end of list and add the new event there
	    if( di->dEQ == NULL )
			di->dEQ = new decoderEventQueue(code, ODID, ESID); // YT (129)					
		else {
			decoderEventQueue* temp;
			temp = di->dEQ;
			while ( temp->next != NULL )
				temp = temp->next;
			temp->next = new decoderEventQueue(code, ODID, ESID);	// YT (129)					
		}
		
	    if( di->isStartedDispatching == 1) {
		    // signal the thread to throw the event 
// SOLARIS PORTING
#ifdef WIN32
		   	ResumeThread(di->hThrow);		   
#else
			thr_continue( *(thread_t *)(di->hThrow) );
#endif
		}
	}
}

void MPEGJDecoder::ProcessHeader(LPBYTE pInput, int &nInputLength, JavaStreamHeader& javaStrHeader)
{
	// First handle the header bits coming in and then hand the rest of the 
	// packet bits to the MPEGJClassLoader.....

	int nHeaderLen = 32;	// bits

	m_pData = pInput;
	m_nLastBit = 8;		// Will make the next bit to be retrieved from
						// a new byte.
	ResetByteCount();	// needed for counting header bytes

	// Start storing the bits from the incoming stream into the header;
	javaStrHeader.version = GetBits(2);
	javaStrHeader.isClassFlag = GetBits(1);
	javaStrHeader.numReqClasses = GetBits(13);
	javaStrHeader.isPackaged = GetBits(1);
	javaStrHeader.compressionScheme = GetBits(3);
	javaStrHeader.reserved = GetBits(12);
	
	int nIDlen = (javaStrHeader.classID).length = GetBits(16);
	nHeaderLen += 16;
	(javaStrHeader.classID).ID = new char[nIDlen+1];

	 // Getting the actual ID one character at a time
	// PORTING to CC 5.3
	int j = 0;
//	for( int j = 0; j< nIDlen ;j++)
	for( j = 0; j< nIDlen ;j++)
		(javaStrHeader.classID). ID[j] = GetChar();
	
	(javaStrHeader.classID). ID[nIDlen] = '\0';
	nHeaderLen += 8*nIDlen;
	/*	
	dLenpadbits stores the length of the padding bits as the 
	JavaClassID is 32 bit aligned. 
	*/
    double dLenpadbits = ceil(( 16 + (8* (double) nIDlen))/32);
    dLenpadbits = (dLenpadbits*32) - ( 16 + (8* nIDlen));
    GetBits(dLenpadbits);	// Get and just discard the padded bits
    nHeaderLen += dLenpadbits;

	if(!javaStrHeader.numReqClasses) { 
		nInputLength -= nHeaderLen/8;
		return;
	}

	else {

		javaStrHeader.reqClassID = new JavaClassID[javaStrHeader.numReqClasses];
		for(j = 0; j < javaStrHeader.numReqClasses ; j++) {
			int nReqIDlen = (javaStrHeader.reqClassID[j]).length = GetBits(16);
			nHeaderLen += 16;
			(javaStrHeader.reqClassID[j]).ID = new char[nReqIDlen+1];
			for( int k=0; k< nReqIDlen; k++)
				(javaStrHeader.reqClassID[j]).ID[k] = GetChar();
			(javaStrHeader.reqClassID[j]).ID[nReqIDlen] = '\0';
			nHeaderLen +=(8 * nReqIDlen);

			dLenpadbits = ceil(( 16 + (8* nReqIDlen))/32);
			dLenpadbits = (dLenpadbits *32) - ( 16 + (8* nReqIDlen));
			// Discard the padded bits
			GetBits(dLenpadbits);
			nHeaderLen+= dLenpadbits;
		}
		nInputLength -= nHeaderLen/8; 
		return;	
	}
}


void MPEGJDecoder::ExceptionMessageBox(char* s)
{
	if((m_jvmSupport.getEnv())->ExceptionOccurred()) {
		(m_jvmSupport.getEnv())->ExceptionDescribe();
		char msg[256] = "Exception Occurred while";
// SOLARIS PORTING
#ifdef WIN32
		AfxMessageBox(strcat(msg,s));
#else
        printf("%s\n", strcat(msg, s) );
#endif
	}
}
