/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************
/*-------------------------------------------------------------------------
MPEGJ 
AUTHORS:    Viswanathan Swaminathan - vishy@eng.sun.com 
			Robert Schmidt(RLS)
			Praveen Alavilli(PA) 
			YUNTAI KYONG(YT) - yuntaikyong@ieee.org
			BINDU 
-------------------------------------------------------------------------*/

#if !defined(_mpegjdec_h)	
#define _mpegjdec_h
#include "DecoderImp.h"

// SOLARIS PORTING
#ifdef WIN32
#include "DecInstW.h"
#include "declib.h"
#endif // WIN32

#include "jni.h"
#include "../Mpgj/sources/JVMSupport.h"

#include "../mpegj-player/MPEGJTerminalInterface.h" //fn. table

struct JavaClassID {
	JavaClassID() : length(0), ID(NULL) {}
	~JavaClassID() { if(ID) delete [] ID; }
	int length;
	char *ID;

};

	
class JavaStreamHeader{
public:
	int version;
	BOOL isClassFlag;
	int numReqClasses;
	BOOL isPackaged;
	int compressionScheme;
	int reserved;
	JavaClassID classID;
	JavaClassID *reqClassID;
public:
	JavaStreamHeader() //Default constructor
		: version(0), isClassFlag(1), numReqClasses(0),
			isPackaged(0), compressionScheme(0),reqClassID(NULL)
	{}

	~JavaStreamHeader() { if(reqClassID) delete [] reqClassID; }
};

typedef enum { START_EVENT = 4, END_EVENT = 5 } EVENT_CODE;

class MPEGJDecoder : public DecoderImp
{
// SOLARIS PORTING
#ifdef WIN32
	friend DWORD WINAPI ThreadProc(LPVOID lpParameter);
#else
    friend void ThreadProc(MPEGJDecoder * dec);
#endif
private:
	
	virtual BOOL Setup (
		const BYTE *pSpecificInfo,
		int nSpecificInfoLength,
		int	nESNumber
	);
	virtual BOOL SetFormat (
		Capabilities &formatOptions,
		DecoderParams &decoderParams
	);
	virtual int GetOptimalOutputSize ();

	void throwEvent(EVENT_CODE code);
	void eventThread();
	
	void ExceptionMessageBox(char*);

	JVMSupport m_jvmSupport;
	Executive *m_pExecutive;
	ALManager *m_pALM;
	InlineImp *m_pInline;
	
	jclass m_cMPEGJClassLoader;
	jmethodID m_midLoadNow;
	jmethodID m_midLoadZIP;
	bool m_bFirstMPEGJAU;
	void startJavaVirtualMachine();
// SOLARIS PORTING - Using thread_t
#ifndef WIN32
	thread_t m_thread;
#endif
/////////////   HEADER ////////////////////////
public:
	int m_nLastByte, m_nLastBit, m_nCount;
	LPBYTE m_pData;
	void ResetByteCount () {m_nCount = 0;}
	int GetByteCount () {return m_nCount;}
	int GetBits(int nBits);
	int GetByte();
	int GetChar();
	void ProcessHeader(LPBYTE pInput, int &nInputLength, JavaStreamHeader& javaStrHeader);
	MPJTerminalDispatchTablePtr di; // PA

/////////////   HEADER ////////////////////////

private:

	virtual BOOL Decode (
		LPBYTE pInput,
		int nInputLength,
		LPBYTE pOutput,
		int &rnOutputLength
	);
	virtual int GetDataOffset  (const LPBYTE pFrame);
	virtual DWORD GetAttributes (
		ATTRIBUTE_CODE code, 
		const LPBYTE pFrame
	);

public :

	MPEGJDecoder () : DecoderImp ()
	{	
		di = MPJGetTerminalDispatchTable(); // PA
		m_bFirstMPEGJAU = true;
	}

	//ML added this
	~MPEGJDecoder()
	{
		di->isJVMStarted = 1;
	}
	void  Run ();
};

#endif                                      // _mpegjdec_h sentry.
