/*******************************************************************************
* Copyright(c) Philips Consumer Electronics B.V. 1998                         
* All rights reserved. Reproduction in whole or in part is                  
* prohibited without the written consent of the copyright holder.            
*                                                                            
*  Project             : Bali
*  File name   $Logfile: $
*       Last save $Date: 2003/12/24 00:53:12 $
*             $Revision: 1.1.1.1 $
*          Last $Author: mikeseven $
*  Creation Date       : 15-11-1998
*  First Author        : Bas van den Heuvel
*                        NatLab - Eindhoven
* 
*  Change history      :
*  ---------------
*
* $Log: SommitSectionFilter.java,v $
* Revision 1.1.1.1  2003/12/24 00:53:12  mikeseven
* Initial release of the CVS repository for MPEG-4 Systems code
* after the code was corrupted due to reorganization and human 
* errors.
*
* Revision 1.1  2003/04/03 04:29:17  mikeseven
* Initial release of Player3D and MPEG-J for all MPEG-4 reference softwares players with samples.
* For installation, follow instructions in docs/Player3D_setup.doc.
*
* Many thanks to Prashanth Gujjeti and Viswanathan Swaminathan from Sun Microsystems.
*
* -- Mikael Bourges-Sevenier (Mindego)
*
* Revision 1.1  2002/08/23 16:47:21  pg138550
* Changes for MPEG-J. NOT FINAL
*
* Revision 1.2  2000/07/21 20:36:55  alavilli
* adding missing files in the previous checkin for SISF
*
* Revision 1.1.2.1  1999/07/09 14:16:35  depetris
* Aggiornati i file per lo sviluppo e il rilascio a MPEG-J.
* Integra la versione 2.5 del core.
*
* Revision 1.2  1999-05-04 17:51:45+02  vdheuvel
* *** empty log message ***
*
* Revision 1.1  1998-12-14 14:05:52+01  vdheuvel
* Sommit section filter definition
*
*
*******************************************************************************/

package sommit.sectfilt;
import  bali.support.Debug;
import  bali.support.Assert;

/****************
The class represents the section filter interface connecting
a Java application with the CSELT SOMMIT demonstrator.
Date: 19-11-1998
CSELT  : Massimo Balestri    mailto:Massico.Balestri@cselt.it
Philips: Bas van den Heuvel  mailto:vdheuvel@natlab.research.philips.com
*/
public class SommitSectionFilter
{
    static{
//	sommit.InitJavaSystem.init();
    }
    /**************
    Return the number of section filters available to Java.
    */
    public final static int getNumberOfFilters()
    {   return NFILTER;
    }
    
    /**************
    Start a filter action on the indicated filter.
    A section filter is set using a mask and value for negative and positive
    filtering. The following algorithm is to be used:
    A section fits the filter parameters when:
      sec : array of unsigned bytes representing this section. sec[0] is the table_id
            field
      mask and value arrays: array of unsigned bytes with a length 12.
      LL = MIN( 12, sec.length )
      (Ai : 0<=i<LL: (  posMask[i]&posValue[i] == posMask[i]&sec[i]
                     && negMask[i]&negValue[i] != negMask[i]&sec[i]
                     ))
    @param index  The index of the section filter to be started. When the section
                  filter is already active it will stop and restart using this
                  section filter parameters.
    @param id     An identifier provided by the application that identifies this
                  filter action (session id).
    @param l      The class that will receive the sections filtered on this filter
                  session.
    @param PID    The PID to filter upon.
    @param posMask  The mask used for positive filtering.
    @param posValue The value used for positive filtering.
    @param negMask  The mask used for negative filtering, unless null.
    @param negValue The value used for negative filtering, unless null.
    */
    public final static void startFilter
        ( int index
        , int id
        , SommitSfInterface l
        , int PID
        , byte[] posMask
        , byte[] posValue
        , byte[] negMask
        , byte[] negValue
        )
    {    
        /*DBG*/ Debug.msg( Debug.SOMSF, Debug.MEDIUM,"SommitSectionFilter.setFilter"+index );
        /*DBG*/ Assert.assert( posMask.length == posValue.length );
        /*DBG*/ Assert.assert( negMask.length == negValue.length );
        initialise();
        synchronized( _lock )
        {
            FilterData fd = new FilterData( id, l, PID, posMask, posValue, negMask, negValue );
            _arFilter[ index ] = fd ;
        }
    }

    /**************
    Start a filter action on the indicated filter.
    A section filter is set using a mask and value for positive
    filtering. The following algorithm is to be used:
    A section fits the filter parameters when:
      sec : array of unsigned bytes representing this section. sec[0] is the table_id
            field
      mask and value arrays: array of unsigned bytes with a length 12.
      LL = MIN( 12, sec.length )
      (Ai : 0<=i<LL: (  posMask[i]&posValue[i] == posMask[i]&sec[i]
                     ))
    @param index  The index of the section filter to be started. When the section
                  filter is already active it will stop and restart using this
                  section filter parameters.
    @param id     An identifier provided by the application that identifies this
                  filter action (session id).
    @param l      The class that will receive the sections filtered on this filter
                  session.
    @param PID    The PID to filter upon.
    @param posMask  The mask used for positive filtering.
    @param posValue The value used for positive filtering.
    */
    public final static void startFilter
        ( int index
        , int id
        , SommitSfInterface l
        , int PID
        , byte[] posMask
        , byte[] posValue
        )
    {    
        /*DBG*/ Debug.msg( Debug.SOMSF, Debug.MEDIUM,"SommitSectionFilter.setFilter"+index );
        /*DBG*/ Assert.assert( posMask.length == posValue.length );
        initialise();
        synchronized( _lock )
        {
            FilterData fd = new FilterData( id, l, PID, posMask, posValue, null, null );
            _arFilter[ index ] = fd ;
        }
    }

    /**************
    Stop the section filter indicated with index.
    */
    public final static void stopFilter( int index )
    {
        /*DBG*/ Debug.msg( Debug.SOMSF, Debug.MEDIUM,"SommitSectionFilter.clearFilter "+index);
       initialise();
       synchronized( _lock )
       {
        /*DBG*/ Debug.msg( Debug.SOMSF, Debug.MEDIUM,"SommitSectionFilter.clearFilter "+index+"inside _lock");
            _arFilter[ index ] = null;
       }
    }
   
    //////////////// IMPLEMENTATION ////////////////
    private static        Object       _lock = new Object();
    private static        FilterData[] _arFilter = null;
    private static final  int          NFILTER = 1; 

   /**
    A new section is present in the TS, check if it fits any of the filters
   */
   static public void newSection( int PID, byte[] sec )
   {
     /*DBG*/ Debug.msg( Debug.SOMSF, Debug.IMPL,"SommitSectionFilter.newSection "+PID);
     initialise();
     SommitSfInterface list = null;
     int               id = 0;
     synchronized( _lock )
     {
        for ( int i=0; i<NFILTER; i++ )
        {
          if ( _arFilter[i] != null )
          { if ( checkSection( PID, sec, _arFilter[i] ) )
            {  
                 /*DBG*/ Debug.msg( Debug.SOMSF, Debug.MEDIUM,"SommitSectionFilter.newSection section filtered");
                 list = _arFilter[i].listener;
                 id   = _arFilter[i].identifier;
            }
          }
        }
     }
     // calling listener outside of locked area to prevent deadlock in HwSfController.
     if ( list != null )
     {
        list.newSection( id, sec );
     }
   }

    /**************
    Check if section fits this filter (fd).
    */
    private static boolean checkSection( int PID, byte[] secData, FilterData fd )
    {
        
        boolean res     = true;
        if ( fd.PID == PID )
        { 
            int i;    
            i=0;
            while ( res && i<fd.posMask.length && i<secData.length)
            {
                res = ( (secData[i]&fd.posMask[i]) == (fd.posValue[i]&fd.posMask[i]) );
                i++;
            }
            i=0;
            while ( fd.negMask!=null && res && i<fd.negMask.length && i<secData.length )
            {   
                res = ( (secData[i]&fd.negMask[i]) != (fd.posMask[i]&fd.negValue[i]) );
                i++;
            }
            /*DBG*/ Debug.msg( Debug.SOMSF, Debug.MEDIUM,"SommitSectionFilter.checkSection "+PID+" "+res);  
        }
        else
        {
            res = false;
        }
        return res;
    }
    
    /**************
    Initialise the section filter.
    */
    private static void initialise()
    {
         synchronized( _lock )
         {
           if ( _arFilter == null )
           {
                /*DBG*/ Debug.msg( Debug.SOMSF, Debug.HIGH,"SommitSectionFilter.initialise");  
                _arFilter = new FilterData[NFILTER];
                for ( int i=0; i<NFILTER; i++ )
                {   _arFilter[i] = null;
                }
           }
         }
    }
}
