/*
* Copyright(c) Philips Consumer Electronics B.V. 1998                         
* All rights reserved. Reproduction in whole or in part is                  
* prohibited without the written consent of the copyright holder.            
*                                                                            
*  Project             : Bali
*  Module              : bali.si 
*  File name   $Logfile: /func_test_platf2/bali/si/cache/SITableCacheManager.java $
*       Last save $Date: 2003/12/24 00:53:01 $
*             $Revision: 1.1.1.1 $
*          Last $Author: mikeseven $
*  Creation Date       : 28/10/98
*  First Author        : S.A.F.A. van den Heuvel
*                        NatLab - Eindhoven
* 
*                                                                          
*  Change history      :
*  ---------------
*
* $Log: SICacheManager.java,v $
* Revision 1.1.1.1  2003/12/24 00:53:01  mikeseven
* Initial release of the CVS repository for MPEG-4 Systems code
* after the code was corrupted due to reorganization and human 
* errors.
*
* Revision 1.1  2003/04/03 04:29:17  mikeseven
* Initial release of Player3D and MPEG-J for all MPEG-4 reference softwares players with samples.
* For installation, follow instructions in docs/Player3D_setup.doc.
*
* Many thanks to Prashanth Gujjeti and Viswanathan Swaminathan from Sun Microsystems.
*
* -- Mikael Bourges-Sevenier (Mindego)
*
* Revision 1.1  2002/08/23 16:47:21  pg138550
* Changes for MPEG-J. NOT FINAL
*
* Revision 1.1  2000/06/06 18:43:02  alavilli
* After SISF integration into the CORE4 code
*
* Revision 1.1.2.1  1999/07/09 14:16:37  depetris
* Aggiornati i file per lo sviluppo e il rilascio a MPEG-J.
* Integra la versione 2.5 del core.
*
* Revision 1.3  1999-04-01 13:31:48+02  vdheuvel
* Added tuning support. After a tuning action the default cache settings
* are reset.
*
* Revision 1.2  1999-03-29 16:53:57+02  vdheuvel
* Added caching of TDT.
*
* Revision 1.1  1999-02-12 15:24:06+01  vdheuvel
* 39 march 1999, start of archive.
* Middleware packages DAVIC API implementation.
*
 * 
 * 1     9-11-98 13:58 Vdheuvel
 * Creation
*/

package bali.cache;

import org.davic.mpeg.dvb.DvbTransportStream;
import bali.component.TransportStreamSelection; 
import bali.table.TableIdentifier;
import  bali.hw.tuner.HwTunerListener;
import  bali.hw.tuner.HwTunerEvent;
import  bali.hw.tuner.HwTuner;

/*****************
This class is responsible for setting and maintaining the cache 
policy. 
*/
class SICacheManager
    implements HwTunerListener
{
    /****************
    */    
   	static synchronized SICacheManager getInstance()
   	{
        if ( _instance == null )
        {
            _instance = new SICacheManager();
        }
   	    return _instance;
   	}

    ///////////////// PRIVATE ///////////////////
    private static SICacheManager _instance = null;
    private static DvbTransportStream _ts;
        
    /****************
    */    
    private SICacheManager()
    {
        HwTuner.getInstance().addTunerListener( this );
        _ts = TransportStreamSelection.getInstance().getCurrentTs();
        startCaching();
    }
    
    /****************
    */    
    private void startCaching()
    {        
        setNITCacheStrategy();
        setSDTCacheStrategy();
        setEITpfCacheStrategy();
        setPATCacheStrategy();
        setTDTCacheStrategy();
    }

    /****************
    */    
    private void stopCaching()
    {        
        unsetNITCacheStrategy();
        unsetSDTCacheStrategy();
        unsetEITpfCacheStrategy();
        unsetPATCacheStrategy();
        unsetTDTCacheStrategy();
    }

    /****************
    */    
    private void setNITCacheStrategy()
    {
        // cache actual network
        TableIdentifier ti = new TableIdentifier( TableIdentifier.NIT, _ts.getNetworkId() );
        SITableCache.startCaching( ti, SITableCache.CACHE );
    }

    /****************
    */    
    private void unsetNITCacheStrategy()
    {
        // cache actual network
        TableIdentifier ti = new TableIdentifier( TableIdentifier.NIT, _ts.getNetworkId() );
        SITableCache.stopCaching( ti, SITableCache.CACHE );
    }

    /****************
    */    
    private void setSDTCacheStrategy()
    {
        // cache SDT actual
        TableIdentifier ti = 
            new TableIdentifier( TableIdentifier.SDT, _ts.getOriginalNetworkId(), _ts.getTransportStreamId() );
        SITableCache.startCaching( ti, SITableCache.CACHE );
    }

    /****************
    */    
    private void unsetSDTCacheStrategy()
    {
        // cache SDT actual
        TableIdentifier ti = 
            new TableIdentifier( TableIdentifier.SDT, _ts.getOriginalNetworkId(), _ts.getTransportStreamId() );
        SITableCache.stopCaching( ti, SITableCache.CACHE );
    }

    /****************
    */    
    private void setEITpfCacheStrategy()
    {
        SITableCache.setMaxUnselected( TableIdentifier.EITpf, 4 );
    }

    /****************
    */    
    private void unsetEITpfCacheStrategy()
    {
    }

    /****************
    */    
    private void setPATCacheStrategy()
    {
        // cache PAT
        TableIdentifier ti = 
            new TableIdentifier( TableIdentifier.PAT, _ts.getOriginalNetworkId(), _ts.getTransportStreamId() );
        SITableCache.startCaching( ti, SITableCache.CACHE );
    }

    /****************
    */    
    private void unsetPATCacheStrategy()
    {
        // cache PAT
        TableIdentifier ti = 
            new TableIdentifier( TableIdentifier.PAT, _ts.getOriginalNetworkId(), _ts.getTransportStreamId() );
        SITableCache.startCaching( ti, SITableCache.CACHE );
    }

    /****************
    */    
    private void setTDTCacheStrategy()
    {
        // cache TDT
        TableIdentifier ti = 
            new TableIdentifier( TableIdentifier.TDT, _ts.getOriginalNetworkId(), _ts.getTransportStreamId() );
        SITableCache.startCaching( ti, SITableCache.CACHE );
    }

    /****************
    */    
    private void unsetTDTCacheStrategy()
    {
        // cache TDT
        TableIdentifier ti = 
            new TableIdentifier( TableIdentifier.TDT, _ts.getOriginalNetworkId(), _ts.getTransportStreamId() );
        SITableCache.stopCaching( ti, SITableCache.CACHE );
    }

    /******************
    The transport stream selection has been changed.
    */
    public void newHwTunerEvent( HwTunerEvent event )
    {
        if ( event.getType() == event.TUNING )
        {
            stopCaching();
        }            
        if ( event.getType() == event.TUNING_OVER )
        {
            _ts = TransportStreamSelection.getInstance().getCurrentTs();
            startCaching();
        }
    }
}