/*
* Copyright(c) Philips Consumer Electronics B.V. 1998                         
* All rights reserved. Reproduction in whole or in part is                  
* prohibited without the written consent of the copyright holder.            
*                                                                            
*  Project             : Bali
*  File name   $Logfile: $
*       Last save $Date: 2003/12/24 00:53:01 $
*             $Revision: 1.1.1.1 $
*          Last $Author: mikeseven $
*  Creation Date       : 09/02/99
*  First Author        : S.A.F.A. van den Heuvel
*                        NatLab - Eindhoven
* 
*                                                                          
*  Change history      :
*  ---------------
*
* $Log: SITableCache.java,v $
* Revision 1.1.1.1  2003/12/24 00:53:01  mikeseven
* Initial release of the CVS repository for MPEG-4 Systems code
* after the code was corrupted due to reorganization and human 
* errors.
*
* Revision 1.1  2003/04/03 04:29:18  mikeseven
* Initial release of Player3D and MPEG-J for all MPEG-4 reference softwares players with samples.
* For installation, follow instructions in docs/Player3D_setup.doc.
*
* Many thanks to Prashanth Gujjeti and Viswanathan Swaminathan from Sun Microsystems.
*
* -- Mikael Bourges-Sevenier (Mindego)
*
* Revision 1.1  2002/08/23 16:47:21  pg138550
* Changes for MPEG-J. NOT FINAL
*
* Revision 1.1  2000/06/06 18:43:02  alavilli
* After SISF integration into the CORE4 code
*
* Revision 1.1.2.1  1999/07/09 14:16:37  depetris
* Aggiornati i file per lo sviluppo e il rilascio a MPEG-J.
* Integra la versione 2.5 del core.
*
* Revision 1.2  1999-04-15 09:53:18+02  vdheuvel
* Files changed to hook the system up to the TM demultipler and the
* TM hardware. Changes include, swapping the java.util.Date class
* for the bali.tm.Date class (to prevent using 64 bit floats). Changing
* the HW sect filt used by bali.hw.SectFilt.
*
* Revision 1.1  1999-02-23 20:43:26+01  vdheuvel
* 39 march 1999, start of archive.
* Middleware packages DAVIC API implementation.
*
*/

package bali.cache;
import  bali.table.TableIdentifier;
import  bali.table.SITable;
import  bali.table.SITableInfo;
import  bali.table.NIT;
import  bali.table.SDT;
import  bali.table.BAT;
import  bali.table.TOT;
import  bali.table.TDT;
import  bali.table.PAT;
import  bali.table.PMT;
import  bali.table.EITpf;
import  bali.table.EITsch;
import  bali.support.Debug;
import  bali.support.Assert;
/*DATE_NORMAL*/ import  java.util.Date;
//DATE_TM*/     import  bali.tm.Date;
import  java.util.Vector;
import  java.util.Enumeration;
import  org.davic.mpeg.dvb.DvbTransportStream;

/*****************
This is the base class of all Table caches. A SICache caches the data
of one specific DVB or MPEG defined table. For each table a cache is 
defined. This allows a different cache strategy to be implemented 
for each table. <p>
When caching tables three different cache lvl are defined: CACHE, MONITOR
and APPLICATION: <p>
CACHE: the cache management of the implementation wants to cache this information.
MONITOR: this table is currently monitored by the application<p>
Each table so indicated will be cached. When starting and stopping caching
of tables wildcards may not be used (value -1).
Wildcards may be used when accessing cached information, not in indicating
the tables to be cached.
*/
public class SITableCache
{
	/** Cache lvl for cache management tables.  */	  
	   public static final int CACHE       = 1;
	/** Cache lvl for monitored tables. */	          
	   public static final int MONITOR     = 2;

	/***********
	Get the SITableCache cache.
	*/
	private synchronized static void initialise()
	{
        if ( _arSITableCache == null )
        {
            /*DBG*/ Debug.msg( Debug.CACHE, Debug.LOW,"Cache initialised" );
            _arSITableCache = new SITableCache[TableIdentifier.MAX_TABLE];
            for ( int i=0; i< _arSITableCache.length; i++ )
            {
                _arSITableCache[i] = new SITableCache( i );
            }
           	SICacheManager.getInstance();
        }
	}
	
    /***********
	Start caching of the indicated table on the indicated lvl.
	@param ti       TableIdentifier indicating the table to be cached.
	@param cacheLvl The cache lvl to be activated.
	*/
	public static void startCaching( TableIdentifier ti, int cacheLvl )
	{
        /*DBG*/ Debug.msg( Debug.CACHE, Debug.LOW,"static startCaching" );
	    initialise();
        /*DBG*/ Assert.assert( !ti.holdsWildCard() );
	    _arSITableCache[ti.getTableType()].nsStartCaching( ti, cacheLvl ) ;
	}

    /***********
	Set the maximum unselected elements in this cache.
	Wildcards are not allowed.
	@param table    Table field from TableIdentifier indicating the cache to be effected.
	@param maxUnsel The maximum unselected elements in this cache.
	*/
	public static void setMaxUnselected( int table, int maxUnsel )
	{
        /*DBG*/ Debug.msg( Debug.CACHE, Debug.LOW,"static setMaxUnselected" );
	    initialise();
	    _arSITableCache[table].nsSetMaxUnselected( maxUnsel ) ;
    }

	/***********
	Stop caching of the indicated table on the indicated lvl.
	Wildcards are not allowed.
	@param ti       TableIdentifier indicating the table to be cached.
	@param cacheLvl The cache lvl to be deactivated.
	*/
	public static void stopCaching( TableIdentifier ti, int cacheLvl )
	{
        /*DBG*/ Debug.msg( Debug.CACHE, Debug.LOW,"static stopCaching" );
	    initialise();
        /*DBG*/ Assert.assert( !ti.holdsWildCard() );
	    _arSITableCache[ti.getTableType()].nsStopCaching( ti, cacheLvl ) ;
    }
    
	/***********
	Access the cache and obtain the indicated table.
	Wildcards not are allowed.
	@param  ti   TableIdentifier indicating the table to be obtained.
	@returns     The required table or null when not present.      
	*/
	public static SITable getSITable( TableIdentifier ti )
	{
        /*DBG*/ Debug.msg( Debug.CACHE, Debug.LOW,"static getSITable" );
	    initialise();
        /*DBG*/ Assert.assert( !ti.holdsWildCard() );
        SITable res = _arSITableCache[ti.getTableType()].nsGetSITable( ti );
        /*DBG*/ Debug.msg( Debug.CACHE, Debug.LOW,"static getSITable"+ res );
	    return ( res );
    }
    
	/***********
	Access the cache and obtain the tables indicated by ti.
	Wildcards are allowed.
	@param  ti   TableIdentifier indicating the table to be obtained.
	@returns     The required table or null when not present.      
	*/
	public static SITable[] getSITables( TableIdentifier ti )
	{
        /*DBG*/ Debug.msg( Debug.CACHE, Debug.LOW,"static getSITables" );
	    initialise();
	    return ( _arSITableCache[ti.getTableType()].nsGetSITables( ti ) );
    }

	/***********
	Indicate to the cache a new table has been filtered. The table
	will return the parsed table (when no changes were detected,
	a new SITable is created using the old SITable data and a
	new SITableInfo object).
	*/
	public static SITable newTable( SITable newTable )
	{
        /*DBG*/ Debug.msg( Debug.CACHE, Debug.LOW,"static newSITable" );
	    initialise();
	    return _arSITableCache[ newTable.getTableIdentifier().getTableType() ].nsNewTable( newTable );
	}
    
    ////////////////////// STATIC PRIVATE ////////////////////////////
    private static SITableCache[] _arSITableCache = null;

    /////////////////////// NON STATIC /////////////////////////////

    /*******************
    @param tt Table type of this cache.
    @see TableIdentifier
    */
    private SITableCache( int tt)
    {
	    /*DBG*/Debug.msg( Debug.CACHE, "SITableCache init "+tt );
        _tableType      = tt;
        _vec = new Vector();
        _maxUnsel = 0;
    }
    
	/***********
	Indicate to the cache a new table has been filtered. The table
	will return the parsed table (when no changes were detected,
	a new SITable is created using the old SITable data and a
	new SITableInfo object).
	*/
	private SITable nsNewTable( SITable newTable )
	{
	    /*DBG*/Debug.msg( Debug.CACHE, "SITableCache("+ _tableType+").newTable " );
		SITable         res   = (SITable)newTable;
		TableIdentifier newTi = newTable.getTableIdentifier();
	    /*DBG*/Assert.assert( newTi.getTableType() == _tableType );
		
		synchronized ( _vec )
		{
        	SICacheElement el = getCacheElement( newTi );
            if ( el != null )
            {	// this table is present in cache
        	    /*DBG*/Debug.msg( Debug.CACHE, Debug.IMPL, "SITableCache("+ _tableType+").newTable update table" ); 
            	SITable oldTable = (SITable)el.getSITable();
            	if ( oldTable != null )
            	{   // a table is present in cache, check if new one is the same
        	    	if ( oldTable.isSameTable( newTable ) )
                	{	// old table is same version reloaded
                		res = updateTable( newTi, oldTable, newTable );
            	    }
            	}
            	// update cache
                el.setSITable( res );
            }
            else
            {   // table not in cache
        	    /*DBG*/Debug.msg( Debug.CACHE, Debug.IMPL, "SITableCache("+ _tableType+").newTable new table" ); 
                el = new SICacheElement( newTable.getTableIdentifier() );
                el.setSITable( newTable );
                _vec.insertElementAt( el, 0 );
                cleanVector();
   	        }
        }
	    return res;
	}
	

    /***********
	Start caching of the indicated table on the indicated lvl.
	If ti holds wild cards, all matching elements are handled.
	@param ti       TableIdentifier indicating the table to be cached.
	@param cacheLvl The cache lvl to be activated.
	*/
	private void nsStartCaching( TableIdentifier ti, int cacheLvl )
	{
	    /*DBG*/Debug.msg( Debug.CACHE, "SITableCache.nsStartCaching "+ _tableType );
	    synchronized( _vec )
        {
      	    SICacheElement el = getCacheElement( ti );
       	    if ( el == null )
   	        {   // add new element
   	            el = new SICacheElement( ti );
   	            _vec.addElement( el );
   	        }
       	    // change element setting
   	        el.setCacheLvl( cacheLvl, true );
	    }
    }

    /***********
	Set the maximum unselected elements in this cache.
	@param ti       TableIdentifier indicating the table to be cached.
	@param maxUnsel The maximum unselected elements in this cache.
	*/
	private void nsSetMaxUnselected( int maxUnsel )
	{
	    /*DBG*/Debug.msg( Debug.CACHE, "SITableCache.nsSetMaxUnselected "+ _tableType );
        _maxUnsel = maxUnsel;
        cleanVector();
    }
    
	/***********
	Stop caching of the indicated table on the indicated lvl.
	If the ti holds wildcards, all matching cache elements are
	handled.
	@param ti       TableIdentifier indicating the table to be cached.
	@param cacheLvl The cache lvl to be deactivated.
	*/
	private void nsStopCaching( TableIdentifier ti, int cacheLvl )
	{
	    /*DBG*/Debug.msg( Debug.CACHE, "SITableCache.nsStopCaching "+ _tableType );
	    synchronized( _vec )
        {
       	    SICacheElement el = getCacheElement( ti );
    	    if ( el == null )
	        {   // change cache lvl
    	        el.setCacheLvl( cacheLvl, false );
	        }
        }
        cleanVector();
    }

	/***********
	Access the cache and obtain the indicated table.
	@param  ti   TableIdentifier indicating the table to be obtained.
	@returns     The required table or null when not present.      
	*/
	private SITable nsGetSITable( TableIdentifier ti )
	{
	    SICacheElement el = getCacheElement( ti );
	    /*DBG*/Debug.msg( Debug.CACHE, "SITableCache("+_tableType+").nsGetSITable "+ el );
	    if ( el != null )
	    {
    		return (SITable)el.getSITable();
    	}
    	else
    	{
    	    return null;
    	}
    }

	/***********
	Access the cache and obtain all indicated tables.
	@param  ti   TableIdentifier indicating the table to be obtained.
	@returns     The required table or null when not present.      
	*/
	private SITable[] nsGetSITables( TableIdentifier ti )
	{
	    /*DBG*/Debug.msg( Debug.CACHE, "SITableCache("+_tableType+").nsGetSITables " );
        SICacheElement  el;
        int[] arIdReq     = ti.getIdentifiers();
        int[] arIdEl;    
        Vector    vecRes  = new Vector();
        synchronized( _vec )
        {
            Enumeration e = _vec.elements();
            while ( e.hasMoreElements() )
            {
                el = (SICacheElement)e.nextElement();
                arIdEl = el.getTableIdentifier().getIdentifiers();
                boolean ok = true;
                int i=0;
                while( ok && i<arIdEl.length && i<arIdReq.length )
                {
                    ok = (  arIdEl[i] == arIdReq[i]
                         || arIdReq[i] == -1 
                         );
                    i++;
                }
                if ( ok )
                {
                    vecRes.addElement( el.getSITable() );
                }
            }
	    }
        SITable[] res = createSITableArray( vecRes.size() );
        vecRes.copyInto( res );
	    /*DBG*/Debug.msg( Debug.CACHE, "       "+vecRes.size() );
    	return res;
    }

    ////////////////////// NON STATIC PRIVATE /////////////////////
    private int                 _tableType;
    private int    _maxUnsel;   // the maximum number unselected elements in cache.
    private Vector _vec;

    /************************
    */
    private SITable[] createSITableArray( int arLength )
    {   
        SITable[] res = null;
        switch( _tableType )
        {
        case TableIdentifier.NIT:
       		res = new NIT[ arLength ];
	        break;
        case TableIdentifier.SDT:
       		res = new SDT[ arLength ];
            break;
        case TableIdentifier.TDT:
       		res = new TDT[ arLength ];
            break;
        case TableIdentifier.TOT:
       		res = new TOT[ arLength ];
            break;
        case TableIdentifier.PAT:
       		res = new PAT[ arLength ];
            break;
        case TableIdentifier.PMT:
       		res = new PMT[ arLength ];
            break;
        case TableIdentifier.EITpf:
       		res = new EITpf[ arLength ];
            break;
        case TableIdentifier.EITsch:
       		res = new EITsch[ arLength ];
            break;
        default:
            /*DBG*/Assert.assert( true );
        }
        return res;
    }

    /************************
    Updates filter time and source ts of cached table.
    */
    private SITable updateTable( TableIdentifier ti, SITable oldTable, SITable newTable )
    {
        SITable res = null;
        SITableInfo info = newTable.getSITableInfo();
        switch( ti.getTableType() )
        {
        case TableIdentifier.NIT:
       		res = new NIT( (NIT)oldTable, info.getSourceTs(), info.getUpdateTime() );
	        break;
        case TableIdentifier.SDT:
       		res = new SDT( (SDT)oldTable, info.getSourceTs(), info.getUpdateTime() );
            break;
        case TableIdentifier.TDT:
       		res = new TDT( (TDT)oldTable, info.getSourceTs(), info.getUpdateTime() );
            break;
        case TableIdentifier.TOT:
       		res = new TOT( (TOT)oldTable, info.getSourceTs(), info.getUpdateTime() );
            break;
        case TableIdentifier.PAT:
       		res = new PAT( (PAT)oldTable, info.getSourceTs(), info.getUpdateTime() );
            break;
        case TableIdentifier.PMT:
       		res = new PMT( (PMT)oldTable, info.getSourceTs(), info.getUpdateTime() );
            break;
        case TableIdentifier.EITpf:
       		res = new EITpf( (EITpf)oldTable, info.getSourceTs(), info.getUpdateTime() );
            break;
        case TableIdentifier.EITsch:
       		res = new EITsch( (EITsch)oldTable, info.getSourceTs(), info.getUpdateTime() );
            break;
        default:
            /*DBG*/Assert.assert( true );
        }
        return res;
    }

    /**************
    Retrieve the indicated cache element from the cache, null when not present.
    */
    private SICacheElement getCacheElement( TableIdentifier ti )
    {
        SICacheElement  res = null;
        boolean         ready = false;

        /*DBG*/ Assert.assert( !ti.holdsWildCard() );
       
        synchronized( _vec )
        {
            Enumeration e = _vec.elements();
            while ( e.hasMoreElements() && !ready )
            {
                res = (SICacheElement)e.nextElement();
                {
                    ready = ( ti.equals( res.getTableIdentifier() ) );
                }
            }
        }
        if ( ready )
        {   return res;
        }
        else
        {   return null;
        }
    }

    /**************
    This method checks the cache for tables that no longer need
    to be cached. Depending on the cache strategy these tables
    and other tables can be removed.
    */
    private void cleanVector()
    {
        SICacheElement  res;
        int             i;
        int             unref = 0;
        
        synchronized( _vec )
        {
            try
            {   i=0;
                while ( true )
                {   res = (SICacheElement)_vec.elementAt(i);
                    if ( !res.isSelected() )
                    {   // obsolete element found, count
                        if ( unref > _maxUnsel )
                        {
                            // element index in vec --: donnot increase index.
                            _vec.removeElementAt( i );
                        }
                        else
                        {
                            unref++;
                        }
                    }

                    i++;
                }
            }
            catch(  ArrayIndexOutOfBoundsException e )
            {   // last element has been read, clean up ready
            }
        }
    }

	/***********
	Enable/diable caching of the indicated cache lvl on all elements
	matching the id[].
	@see SICache
	*/
	private void setCacheLvl( int id[], int cacheLvl, boolean active )
	{
        synchronized( _vec )
        {
            Enumeration e = _vec.elements();
            while ( e.hasMoreElements() )
            {
                SICacheElement res = (SICacheElement)e.nextElement();
                int[] arId1 = res.getTableIdentifier().getIdentifiers();
                int[] arId2 = id;
                for( int i=0; i<arId1.length && i<arId2.length; i++ )
                {
                    if ( ( arId1[i] == arId2[i] 
                         || arId2[i] == -1  // wildcard will only effect non wildcard entries.
                         )
                         &&
                         arId1[i] != -1 // wildcard will only effect non wildcard entries.
                       )
                    {   // match
                        res.setCacheLvl( cacheLvl, active );
                    }
                }
            }
            cleanVector();
	    }
	}

}