/*******************************************************************************
* Copyright(c) Philips Consumer Electronics B.V. 1998                         
* All rights reserved. Reproduction in whole or in part is                  
* prohibited without the written consent of the copyright holder.            
*                                                                            
*  Project             : Bali
*  File name   $Logfile: /g4/JavaCode/org/davic/mpeg/dvb/ComponentMaster.java $
*       Last save $Date: 2003/12/24 00:53:01 $
*             $Revision: 1.1.1.1 $
*          Last $Author: mikeseven $
*  Creation Date       : 1/1/98
*  First Author        : H.v.Woerkom
*                        NatLab - Eindhoven
* 
*  Description         : ComponentMaster
*                                                                          
*  Change history      :
*  ---------------
*
* $Log: ComponentMaster.java,v $
* Revision 1.1.1.1  2003/12/24 00:53:01  mikeseven
* Initial release of the CVS repository for MPEG-4 Systems code
* after the code was corrupted due to reorganization and human 
* errors.
*
* Revision 1.1  2003/04/03 04:29:18  mikeseven
* Initial release of Player3D and MPEG-J for all MPEG-4 reference softwares players with samples.
* For installation, follow instructions in docs/Player3D_setup.doc.
*
* Many thanks to Prashanth Gujjeti and Viswanathan Swaminathan from Sun Microsystems.
*
* -- Mikael Bourges-Sevenier (Mindego)
*
* Revision 1.1  2002/08/23 16:47:21  pg138550
* Changes for MPEG-J. NOT FINAL
*
* Revision 1.1  2000/06/06 18:43:02  alavilli
* After SISF integration into the CORE4 code
*
* Revision 1.1.2.1  1999/07/09 14:16:38  depetris
* Aggiornati i file per lo sviluppo e il rilascio a MPEG-J.
* Integra la versione 2.5 del core.
*
* Revision 1.3  1999-04-02 14:15:22+02  vdheuvel
* *** empty log message ***
*
* Revision 1.2  1999-03-29 16:26:31+02  vdheuvel
* Remove the service based methods.
*
* Revision 1.1  1999-02-19 16:40:20+01  vdheuvel
* 39 march 1999, start of archive.
* Middleware packages DAVIC API implementation.
*
* 5     6-07-98 16:45 Woerkom
* Adapted to Java support module
* 
* 4     12-06-98 16:46 Woerkom
* 
* 3     10-04-98 13:32 Woerkom
* Component API implementation finished
* 
* 2     7-04-98 14:51 Woerkom
* First implementation
* 
* 1     7-04-98 13:10 Woerkom
* First implementation
* 
* 2     19-02-98 16:16 Woerkom
* Adapted headers with keywords
*
*******************************************************************************/

package bali.component;
import bali.support.Debug;
import java.util.Enumeration;
import java.util.Vector;
import org.davic.mpeg.*;
import org.davic.mpeg.dvb.*;

/*************
This class holds all instances of the different org.davic.mpeg.dvb.* and
prevents the occurance of dual object instances.
*/
public class ComponentMaster
{
    /********************
    */
	ComponentMaster()
	{
		_transportStreams = new Vector();
	}

    /********************
    Get the ComponentMaster (singleton design pattern).
    */
	public synchronized static ComponentMaster getInstance()
	{
		if (_instance == null)
		{
			_instance = new ComponentMaster();
		}
		return _instance;
	}

    /********************
    Create a new TransportStream object or access an existing one.
    */
	public synchronized DvbTransportStream getTransportStream( int originalNetworkId, int networkId, int transportStreamId )
	{
        /*DBG*/Debug.msg( Debug.COMP, "ComponentMaster getTransportStream : "+originalNetworkId+" "+networkId+" "+transportStreamId );
		Enumeration e = _transportStreams.elements();
		DvbTransportStream ts = null;
		DvbTransportStream ts2 = null;
		while (e.hasMoreElements() && ts == null)
		{
			ts2 = (DvbTransportStream) e.nextElement();
			if (ts2.getOriginalNetworkId() == originalNetworkId &&
				ts2.getNetworkId() == networkId &&
				ts2.getTransportStreamId() == transportStreamId)
			{
				ts = ts2;
			}
		}

		if (ts == null)
		{
			ts = new DvbTransportStream( originalNetworkId, networkId, transportStreamId, this );
			if (ts != null)
			{
				_transportStreams.addElement( ts );
			}
		}

		return ts;
	}


    ///////////////////// IMPLEMENTATION ///////////////////////
	private static ComponentMaster _instance = null;
	private Vector _transportStreams;
}
