/*
* Copyright(c) Philips Consumer Electronics B.V. 1998                         
* All rights reserved. Reproduction in whole or in part is                  
* prohibited without the written consent of the copyright holder.            
*                                                                            
*  Project             : Bali
*  Module              : bali.si 
*  File name   $Logfile: /func_test_platf2/bali/si/scheduler/RetrieveListener.java $
*       Last save $Date: 2003/12/24 00:53:01 $
*             $Revision: 1.1.1.1 $
*          Last $Author: mikeseven $
*  Creation Date       : 28/10/98
*  First Author        : S.A.F.A. van den Heuvel
*                        NatLab - Eindhoven
* 
*                                                                          
*  Change history      :
*  ---------------
*
* $Log: TransportStreamSelection.java,v $
* Revision 1.1.1.1  2003/12/24 00:53:01  mikeseven
* Initial release of the CVS repository for MPEG-4 Systems code
* after the code was corrupted due to reorganization and human 
* errors.
*
* Revision 1.1  2003/04/03 04:29:18  mikeseven
* Initial release of Player3D and MPEG-J for all MPEG-4 reference softwares players with samples.
* For installation, follow instructions in docs/Player3D_setup.doc.
*
* Many thanks to Prashanth Gujjeti and Viswanathan Swaminathan from Sun Microsystems.
*
* -- Mikael Bourges-Sevenier (Mindego)
*
* Revision 1.1  2002/08/23 16:47:21  pg138550
* Changes for MPEG-J. NOT FINAL
*
* Revision 1.1  2000/06/06 18:43:02  alavilli
* After SISF integration into the CORE4 code
*
* Revision 1.1.2.1  1999/07/09 14:16:38  depetris
* Aggiornati i file per lo sviluppo e il rilascio a MPEG-J.
* Integra la versione 2.5 del core.
*
* Revision 1.4  1999-04-15 09:53:18+02  vdheuvel
* Files changed to hook the system up to the TM demultipler and the
* TM hardware. Changes include, swapping the java.util.Date class
* for the bali.tm.Date class (to prevent using 64 bit floats). Changing
* the HW sect filt used by bali.hw.SectFilt.
*
* Revision 1.3  1999-04-01 12:51:51+02  vdheuvel
* Updated tuning dependency; only reload current TS when TUNING_OVER event has been detected.
*
* Revision 1.2  1999-03-29 16:51:28+02  vdheuvel
* Added transport stream change support.
*
* Revision 1.1  1999-02-12 12:40:54+01  vdheuvel
* 39 march 1999, start of archive.
* Middleware packages DAVIC API implementation.
*
 * 
 * 1     9-11-98 13:59 Vdheuvel
 * Creation
*/

package bali.component;
import  org.davic.mpeg.dvb.DvbTransportStream;
import  bali.support.Debug;
import  bali.support.SeriousError;
import  bali.table.SDT;
import  bali.table.NIT;
import  bali.table.InvalidTableException;
import  bali.cache.SITableCache;
import  bali.hw.sectfilt.HwSection;
import  bali.hw.sectfilt.HwSfController;
import  bali.hw.sectfilt.HwSfData;
import  bali.hw.sectfilt.HwSfEvent;
import  bali.hw.sectfilt.HwSfResourceManager;
import  bali.hw.sectfilt.HwSfResourceEvent;
import  bali.hw.sectfilt.HwSfResourceListener;
import  bali.hw.sectfilt.ReleaseSfEnhancer;
import  bali.hw.sectfilt.TableSfEnhancer;
import  bali.hw.sectfilt.SerializerSfEnhancer;
import  bali.hw.tuner.HwTunerListener;
import  bali.hw.tuner.HwTunerEvent;
import  bali.hw.tuner.HwTuner;
/*DATE_NORMAL*/ import  java.util.Date;
//DATE_TM*/     import  bali.tm.Date;

/*********************
This class stores the current transport stream. At start-up and each time
the transport stream selection changes, the SDT and NIT is filtered to 
determine the current transport stream.
*/
public class TransportStreamSelection
    implements HwSfResourceListener, HwTunerListener
{
    /****************
    Returns the current TS selection.
    */
    public synchronized DvbTransportStream getCurrentTs()
    {
        /*DBG*/Debug.msg(Debug.COMP, "TransportStreamSelection.getCurrentTs");
        if ( _ts == null )
        {
            /*DBG*/Debug.msg(Debug.COMP, "TransportStreamSelection filter SDT and NIT to determine CurrentTs");
            determineCurrentTs();
        }
        return _ts;
    }
    
    /******************
    Constructor.
    */
    TransportStreamSelection()
    {   
        HwTuner.getInstance().addTunerListener( this );
    }

    /******************
    */
    public synchronized static TransportStreamSelection getInstance()
    {
        if ( _instance == null )
        {
            _instance = new TransportStreamSelection();
        }
        return _instance;
    }
    
    ///////////////// IMPLEMENTATION ////////////////
    private static TransportStreamSelection _instance = null;
    private DvbTransportStream       _ts  = null;
    private Object                   _resLock = new Object();

    /******************
    */
    private void determineCurrentTs()
    {
        int nw_id = -1;
        int ts_id = -1;
        int or_nw_id = -1;
        SerializerSfEnhancer se_nit= null;
        SerializerSfEnhancer se_sdt= null;
        TableSfEnhancer      te_nit = null;
        TableSfEnhancer      te_sdt = null;
        HwSection[]          arSec_nit = null;
        HwSection[]          arSec_sdt = null;
        HwSfResourceManager  man = HwSfResourceManager.getInstance();
        
        man.addHwSfResourceListener( this );
        while ( nw_id == -1 && ts_id == -1 )
        {
            // filter NIT
            if ( nw_id == -1 )
            {
                /*DBG*/Debug.msg(Debug.COMP, Debug.LOW, "   filter NIT");
                se_nit = new SerializerSfEnhancer(); 
                te_nit = new TableSfEnhancer( new ReleaseSfEnhancer( se_nit ) );
                HwSfController sfc = null;
                sfc = man.claimSectionFilter( HwSfResourceManager.CTSAPI, te_nit );
                synchronized( _resLock )
                {
                    while ( sfc == null )
                    {
                        /*DBG*/Debug.msg(Debug.COMP, Debug.LOW,"TransportStreamSelection wait for resource");
                        try  // wait for a section filter to become available
                        {   _resLock.wait();
                        }
                        catch ( InterruptedException e )
                        {}
                        /*DBG*/Debug.msg(Debug.COMP, Debug.LOW,"TransportStreamSelection reclaim resource");
                        sfc = man.claimSectionFilter( HwSfResourceManager.CTSAPI, te_nit );
                    }
                }
                HwSfData sfd = new HwSfData( NIT.PID, NIT.TABLE_ID_ACTUAL );
                sfc.startSectionFilter( sfd );
            }
            // filter SDT
            if ( ts_id == -1 )
            {
                /*DBG*/Debug.msg(Debug.COMP, Debug.LOW, "   filter SDT");
                se_sdt = new SerializerSfEnhancer(); 
                te_sdt = new TableSfEnhancer( new ReleaseSfEnhancer( se_sdt ) );
                HwSfController sfc = null;
                sfc = man.claimSectionFilter( HwSfResourceManager.CTSAPI, te_sdt );
                synchronized( _resLock )
                {
                    while ( sfc == null )
                    {
                        /*DBG*/Debug.msg(Debug.COMP, Debug.LOW,"TransportStreamSelection wait for resource");
                        try  // wait for a section filter to become available
                        {   _resLock.wait();
                        }
                        catch ( InterruptedException e )
                        {}
                        /*DBG*/Debug.msg(Debug.COMP, Debug.LOW,"TransportStreamSelection reclaim resource");
                        sfc = man.claimSectionFilter( HwSfResourceManager.CTSAPI, te_sdt );
                    }
                }
                HwSfData sfd = new HwSfData( SDT.PID, SDT.TABLE_ID_ACTUAL );
                sfc.startSectionFilter( sfd );
            }

            /*DBG*/Debug.msg(Debug.COMP, Debug.LOW, "   filters set, check results");
              // check results
            if ( nw_id == -1 )
            {   // filtering done
                if ( se_nit.getNextHwSfEvent().getResult() == HwSfEvent.SUCCESS )
                {
                    /*DBG*/Debug.msg(Debug.COMP, "TransportStreamSelection.getCurrentTs NIT filtered");
                    arSec_nit = te_nit.getSections();
                    nw_id = arSec_nit[0].table_id_extension();
                }
                else
                {
                    /*DBG*/Debug.msg(Debug.COMP, Debug.LOW,"TransportStreamSelection error occurred");
                }
            }
            if ( ts_id == -1 )
            {   // filtering done
                if ( se_sdt.getNextHwSfEvent().getResult() == HwSfEvent.SUCCESS )
                {
                    /*DBG*/Debug.msg(Debug.COMP, "TransportStreamSelection.getCurrentTs SDT filtered");
                    arSec_sdt = te_sdt.getSections();
                    ts_id = arSec_sdt[0].table_id_extension();
                    or_nw_id = arSec_sdt[0].get16bits(8);
                }
                else
                {
                    /*DBG*/Debug.msg(Debug.COMP, Debug.LOW,"TransportStreamSelection error occurred");
                }
            }
        }
        _ts = (ComponentMaster.getInstance()).getTransportStream( or_nw_id, nw_id, ts_id );
        man.removeHwSfResourceListener( this );

        try
        {
            Date now = new Date();        
            SITableCache.newTable( new SDT( arSec_sdt, _ts, now ) );
            SITableCache.newTable( new NIT( arSec_nit, _ts, now ) );
        }
        catch ( InvalidTableException e )
        {   throw new SeriousError();
        }
    }       

    /******************
    A section filter resource has been claimed or released.
    */
    public void newHwSfResourceEvent( HwSfResourceEvent e)
    {
        /*DBG*/Debug.msg(Debug.COMP, Debug.LOW, "TransportStreamSelection resource event");
        if ( e.getResult() == e.RELEASED )
        {
            /*DBG*/Debug.msg(Debug.COMP, Debug.LOW,"TransportStreamSelection resource RELEASED event");
            synchronized( _resLock )
            {
                _resLock.notifyAll();
            }
        }
        else
            /*DBG*/Debug.msg(Debug.COMP, Debug.LOW,"TransportStreamSelection resource CLAIMED event");
    }

    /******************
    The transport stream selection has been changed.
    */
    public void newHwTunerEvent( HwTunerEvent event )
    {
        if ( event.getType() == event.TUNING_OVER )
        {
            determineCurrentTs();
        }
    }

}

