/*
* Copyright(c) Philips Consumer Electronics B.V. 1998                         
* All rights reserved. Reproduction in whole or in part is                  
* prohibited without the written consent of the copyright holder.            
*                                                                            
*  Project             : Bali
*  Module              : bali.si 
*  File name   $Logfile: /func_test_platf2/bali/si/scheduler/RetrieveListener.java $
*       Last save $Date: 2003/12/24 00:53:01 $
*             $Revision: 1.1.1.1 $
*          Last $Author: mikeseven $
*  Creation Date       : 28/10/98
*  First Author        : S.A.F.A. van den Heuvel
*                        NatLab - Eindhoven
* 
*                                                                          
*  Change history      :
*  ---------------
*
* $Log: TransportStreamTime.java,v $
* Revision 1.1.1.1  2003/12/24 00:53:01  mikeseven
* Initial release of the CVS repository for MPEG-4 Systems code
* after the code was corrupted due to reorganization and human 
* errors.
*
* Revision 1.1  2003/04/03 04:29:18  mikeseven
* Initial release of Player3D and MPEG-J for all MPEG-4 reference softwares players with samples.
* For installation, follow instructions in docs/Player3D_setup.doc.
*
* Many thanks to Prashanth Gujjeti and Viswanathan Swaminathan from Sun Microsystems.
*
* -- Mikael Bourges-Sevenier (Mindego)
*
* Revision 1.1  2002/08/23 16:47:21  pg138550
* Changes for MPEG-J. NOT FINAL
*
* Revision 1.1  2000/06/06 18:43:02  alavilli
* After SISF integration into the CORE4 code
*
* Revision 1.1.2.1  1999/07/09 14:16:39  depetris
* Aggiornati i file per lo sviluppo e il rilascio a MPEG-J.
* Integra la versione 2.5 del core.
*
* Revision 1.3  1999-04-15 09:53:18+02  vdheuvel
* Files changed to hook the system up to the TM demultipler and the
* TM hardware. Changes include, swapping the java.util.Date class
* for the bali.tm.Date class (to prevent using 64 bit floats). Changing
* the HW sect filt used by bali.hw.SectFilt.
*
* Revision 1.2  1999-03-29 16:52:52+02  vdheuvel
* Filtered TDT is offered to the cache.
*
* Revision 1.1  1998-12-01 16:57:30+01  vdheuvel
* 39 march 1999, start of archive.
* Middleware packages DAVIC API implementation.
*
 * 
 * 1     9-11-98 13:59 Vdheuvel
 * Creation
*/

package bali.component;
import  org.davic.mpeg.dvb.DvbTransportStream;
import  bali.support.Debug;
import  bali.support.SeriousError;
import  bali.table.TDT;
import  bali.table.TOT;
import  bali.table.InvalidTableException;
import  bali.cache.SITableCache;
import  bali.hw.sectfilt.HwSection;
import  bali.hw.sectfilt.HwSfController;
import  bali.hw.sectfilt.HwSfData;
import  bali.hw.sectfilt.HwSfEvent;
import  bali.hw.sectfilt.HwSfResourceManager;
import  bali.hw.sectfilt.HwSfResourceEvent;
import  bali.hw.sectfilt.HwSfResourceListener;
import  bali.hw.sectfilt.ReleaseSfEnhancer;
import  bali.hw.sectfilt.SingleSfEnhancer;
import  bali.hw.sectfilt.SerializerSfEnhancer;
/*DATE_NORMAL*/ import  java.util.Date;
//DATE_TM*/     import  bali.tm.Date;

/*********************
Within DVB-SI the current time is stored in the TOT and TDT tables. These tables
are send at regurly intervals in the TS. In principle all transport streams should
represent the current UTC time so synchronizing the internal clock with the TDT or
TOT does not have to be done on a regurlar basis.
When using stored, looping transport streams like in a demo or test situation, the
time in the TS does not represent the current UTC time and will reset with every loop
of the transport stream. This class provides an access point to the stream time (the
time as represented in the transport stream).
*/
public class TransportStreamTime
    implements HwSfResourceListener
{
    /****************
    Return the current time
    */
    public synchronized Date getTsTime()
    {
        /*DBG*/Debug.msg(Debug.COMP, "TransportStreamTime.getLastTsTime");
        if ( _time == null )
        {
            /*DBG*/Debug.msg(Debug.COMP, "TransportStreamSelection filter T?T to determine ts time");
            determineCurrentTsTime();
        }
        return _time;
    }
    
    /******************
    */
    TransportStreamTime()
    {
    }

    /******************
    */
    public synchronized static TransportStreamTime getInstance()
    {
        if ( _instance == null )
        {
            _instance = new TransportStreamTime();
        }
        return _instance;
    }
    
    ///////////////// IMPLEMENTATION ////////////////
    private static TransportStreamTime _instance = null;
    private Date                       _time     = null;
    private Object                     _resLock  = new Object();

    /******************
    */
    private void determineCurrentTsTime()
    {
        HwSfResourceManager man = HwSfResourceManager.getInstance();

        // when resources are not available a signal is given when they become available
        man.addHwSfResourceListener( this );
        HwSection sec=null; 

        while ( _time  == null )
        {
            
            // filter TDT
            /*DBG*/Debug.msg(Debug.COMP, Debug.LOW, "   filter TDT");
            SerializerSfEnhancer se = new SerializerSfEnhancer(); 
            SingleSfEnhancer   te   = new SingleSfEnhancer( new ReleaseSfEnhancer( se ));

            HwSfController sfc = null;
            sfc = man.claimSectionFilter( HwSfResourceManager.CTSAPI, te );
            synchronized( _resLock )
            {
                while ( sfc == null )
                {
                    /*DBG*/Debug.msg(Debug.COMP, Debug.LOW,"TransportStreamTime wait for resource");
                    try  // wait for a section filter to become available
                    {   _resLock.wait();
                    }
                    catch ( InterruptedException e )
                    {}
                    /*DBG*/Debug.msg(Debug.COMP, Debug.LOW,"TransportStreamTime reclaim resource");
                    sfc = man.claimSectionFilter( HwSfResourceManager.CTSAPI, te );
                }
            }
            HwSfData sfd   = new HwSfData( TDT.PID, TDT.TABLE_ID );
            sfc.startSectionFilter( sfd );

            // check results
            if ( se.getNextHwSfEvent().getResult() == HwSfEvent.SUCCESS )
            {
                /*DBG*/Debug.msg(Debug.COMP, "TransportStreamTime.getCurrentTs TDT filtered");
                sec = te.getSection();
                _time = sec.getUTCTime( 3 );
            }
            else
            {
                /*DBG*/Debug.msg(Debug.COMP, Debug.LOW,"TransportStreamTime filter error occurred");
            }
        }
        // stop listening to section filter resource changes
        man.removeHwSfResourceListener( this );

        // store TDT in cache.
        
        try
        {
            Date now = new Date();    
            HwSection[] arSec = new HwSection[1];
            arSec[0] = sec;
            SITableCache.newTable( new TDT( arSec, TransportStreamSelection.getInstance().getCurrentTs(), now ) );
        }
        catch ( InvalidTableException e )
        {   throw new SeriousError();
        }

    }       

    /******************
    */
    public void newHwSfResourceEvent( HwSfResourceEvent e)
    {
        /*DBG*/Debug.msg(Debug.COMP, Debug.LOW, "TransportStreamSelection resource event");
        if ( e.getResult() == e.RELEASED )
        {
            /*DBG*/Debug.msg(Debug.COMP, Debug.LOW,"TransportStreamSelection resource RELEASED event");
            synchronized( _resLock )
            {
                _resLock.notifyAll();
            }
        }
        else
            /*DBG*/Debug.msg(Debug.COMP, Debug.LOW,"TransportStreamSelection resource CLAIMED event");
    }


}

