/*
* Copyright(c) Philips Consumer Electronics B.V. 1998                         
* All rights reserved. Reproduction in whole or in part is                  
* prohibited without the written consent of the copyright holder.            
*                                                                            
*  Project             : Bali
*  File name   $Logfile: $
*       Last save $Date: 2003/12/24 00:53:02 $
*             $Revision: 1.1.1.1 $
*          Last $Author: mikeseven $
*  Creation Date       : 12-11-98
*  First Author        : S.A.F.A. van den Heuvel
*                        NatLab - Eindhoven
* 
*                                                                          
*  Change history      :
*  ---------------
*
* $Log: HwSection.java,v $
* Revision 1.1.1.1  2003/12/24 00:53:02  mikeseven
* Initial release of the CVS repository for MPEG-4 Systems code
* after the code was corrupted due to reorganization and human 
* errors.
*
* Revision 1.1  2003/04/03 04:29:18  mikeseven
* Initial release of Player3D and MPEG-J for all MPEG-4 reference softwares players with samples.
* For installation, follow instructions in docs/Player3D_setup.doc.
*
* Many thanks to Prashanth Gujjeti and Viswanathan Swaminathan from Sun Microsystems.
*
* -- Mikael Bourges-Sevenier (Mindego)
*
* Revision 1.1  2002/08/23 16:47:21  pg138550
* Changes for MPEG-J. NOT FINAL
*
* Revision 1.1  2000/06/06 18:43:03  alavilli
* After SISF integration into the CORE4 code
*
* Revision 1.1.2.1  1999/07/09 14:16:39  depetris
* Aggiornati i file per lo sviluppo e il rilascio a MPEG-J.
* Integra la versione 2.5 del core.
*
* Revision 1.1  1998-11-23 17:14:50+01  vdheuvel
* 39 march 1999, start of archive.
* Middleware packages DAVIC API implementation.
*
 * 
 * 1     9-11-98 13:59 Vdheuvel
 * Copy from org.davic.mpeg.sections
*/
package bali.hw.sectfilt;
import  bali.support.TablePart;

/** This class describes a single section as filtered from an MPEG transport stream. */
public class HwSection 
    extends TablePart
{
	/** Constructor */

	public HwSection( byte[] data )
	{
	    super( data );
	}

	/** This method returns all the data in the Section object. 
	*/
	public byte[] getData()
	{
		return _ar;
	}

	/** This method returns that part of the filtered data requested. 
	*/
	public byte[] getData(int offset, int length) 
	{
		byte[] temp = new byte[length];

		System.arraycopy( _ar, offset, temp, 0, length);
		return temp;
	}

	/** This method returns one byte from the filtered data. 
    */
	public byte getByteAt(int index) 
	{
		return _ar[index];
	}

	/** This method returns the corresponding field from an MPEG-2 section header. 
	*/
	public int table_id()
	{
		return get8bits( 0 );
	}

	/** This method returns the corresponding field from an MPEG-2 section header. 
	*/
	public boolean section_syntax_indicator()
	{
		if ( (get8bits(1) & 0x80) == 0x80)
			return true;
		return false;
	}

	/** This method returns the corresponding field from an MPEG-2 section header. 
	*/
	public boolean private_indicator() 
    {
    	if ((get8bits(1) & 0x40) == 0x40)
			return true;
		return false;
	}

	/** This method returns the corresponding field from an MPEG-2 section header. 
	*/
	public int section_length() 
	{
		return ((get8bits(1) & 0x0f) << 8) + get8bits(2);
	}

	/** This method returns the corresponding field from an MPEG-2 section header. 
	*/
	public int table_id_extension() 
	{
		return (get8bits(3) << 8) + get8bits(4);
	}

	/** This method returns the corresponding field from an MPEG-2 section header. 
	*/
	public short version_number()
	{
		return (short) ((get8bits(5) >> 1) & 0x1f);
	}

	/** This method returns the corresponding field from an MPEG-2 section header. 
	*/
	public boolean current_next_indicator()
	{
		if ((get8bits(5) & 0x01) == 0x01)
			return true;
		return false;
	}

	/** This method returns the corresponding field from an MPEG-2 section header. 
	*/
	public int section_number()
	{
		return get8bits(6);
	}

	/** This method returns the corresponding field from an MPEG-2 section header. 
	*/
	public int last_section_number()
	{
		return get8bits(7);
	}


}
