/*
* Copyright(c) Philips Consumer Electronics B.V. 1998                         
* All rights reserved. Reproduction in whole or in part is                  
* prohibited without the written consent of the copyright holder.            
*                                                                            
*  Project             : Bali
*  File name   $Logfile: /func_test_platf2/bali/si/scheduler/RetrieveEvent.java $
*       Last save $Date: 2003/12/24 00:53:02 $
*             $Revision: 1.1.1.1 $
*          Last $Author: mikeseven $
*  Creation Date       : 17/11/98
*  First Author        : S.A.F.A. van den Heuvel
*                        NatLab - Eindhoven
* 
*                                                                          
*  Change history      :
*  ---------------
*
* $Log: HwSfController.java,v $
* Revision 1.1.1.1  2003/12/24 00:53:02  mikeseven
* Initial release of the CVS repository for MPEG-4 Systems code
* after the code was corrupted due to reorganization and human 
* errors.
*
* Revision 1.1  2003/04/03 04:29:18  mikeseven
* Initial release of Player3D and MPEG-J for all MPEG-4 reference softwares players with samples.
* For installation, follow instructions in docs/Player3D_setup.doc.
*
* Many thanks to Prashanth Gujjeti and Viswanathan Swaminathan from Sun Microsystems.
*
* -- Mikael Bourges-Sevenier (Mindego)
*
* Revision 1.1  2002/08/23 16:47:21  pg138550
* Changes for MPEG-J. NOT FINAL
*
* Revision 1.1  2000/06/06 18:43:03  alavilli
* After SISF integration into the CORE4 code
*
* Revision 1.1.2.1  1999/07/09 14:16:40  depetris
* Aggiornati i file per lo sviluppo e il rilascio a MPEG-J.
* Integra la versione 2.5 del core.
*
* Revision 1.8  1999-05-04 14:40:46+02  vdheuvel
* Changes required after first Bouquet tests
*
* Revision 1.7  1999-04-26 14:21:44+02  vdheuvel
* *** empty log message ***
*
* Revision 1.6  1999-04-23 11:31:49+02  vdheuvel
* *** empty log message ***
*
* Revision 1.5  1999-04-19 15:57:26+02  vdheuvel
* Hook the platform to the TM STB.
*
* Revision 1.4  1999-04-15 14:02:12+02  vdheuvel
* Files changed to hook the system up to the TM demultipler and the
* TM hardware. Changes include, swapping the java.util.Date class
* for the bali.tm.Date class (to prevent using 64 bit floats). Changing
* the HW sect filt used by bali.hw.SectFilt.
*
* Revision 1.3  1999-04-14 11:50:38+02  vdheuvel
* Changed simulation platform and HwSectFilt module from sommit
* to tm section filter.
*
* Revision 1.2  1999-04-01 14:07:04+02  vdheuvel
* Added support for ts changes.
*
* Revision 1.1  1999-02-04 16:33:36+01  vdheuvel
* 39 march 1999, start of archive.
* Middleware packages DAVIC API implementation.
*
*/

package bali.hw.sectfilt;
import  bali.support.Debug;
import  bali.support.ActiveEventSender;
/*SOMMIT*/import  sommit.sectfilt.SommitSfInterface;
/*SOMMIT*/import  sommit.sectfilt.SommitSectionFilter;
//TM-HW*/import prl.mpeg.SectFiltListener;
//TM-HW*/import prl.mpeg.TMSectFilt;
//TM-SIM*/import  bali.tm.sectfilt.SectFiltListener;
//TM-SIM*/import  bali.tm.sectfilt.TMSectFilt;

/*********************
Instances of this class represent a claimed section filter. As long
as this section filter instance holds the section filter resource
it can be used to start and stop section filter actions.
When the resource is reclaimed, the current filter job is stopped
and a REVOKED event is send. Hereafter, any attempt to restart without
reclaiming will result in a REVOKED event.
*/
public class HwSfController
//TM*/     implements SectFiltListener 
/*SOMMIT*/ implements SommitSfInterface
{
    /*********************
    Release the section filter resource of this SectionFilter instance.
    */
    public synchronized void release()
    {
        /*DBG*/Debug.msg(Debug.HWSF, Debug.IMPL, "HwSfController.release" );
        if ( _sfd != null )
        {   // filtering
            stopFiltering();
        }
        if ( _resAvailable )
        {
            HwSfResourceManager.getInstance().releaseSfResource( this );
            _resAvailable = false;
        }
    }

    /*********************
    Stop the current filter action. If no filter action is current this call
    is ignored.
    */
    public synchronized void stopFiltering( )
    {
        /*DBG*/Debug.msg(Debug.HWSF,"HwSfController.stopFiltering" );
        if ( _sfd != null )
        {
            sendEvent( new HwSfEvent( this, HwSfEvent.CANCELLED, null, _sfd ) );
            stopFilter();
        }
    }

    /*********************
    Start filtering of any sections using the indicated section filter
    data.
    */
    public synchronized void startSectionFilter( HwSfData sfd )
    {
        /*DBG*/Debug.msg(Debug.HWSF,"HwSfController.startSectionFilter" );
        if ( !_tuning )
        {   // no tuning active
            if ( _resAvailable  )
            {
                _sfd = sfd;
/*SOMMIT*/      if ( sfd.getNegFilterDef() == null )
/*SOMMIT*/      {
/*SOMMIT*/          /*DBG*/Debug.msg(Debug.HWSF, Debug.IMPL, "HwSfController.startSectionFilter pos only" );
/*SOMMIT*/          SommitSectionFilter.startFilter
/*SOMMIT*/              ( _sfNo, _filterId
/*SOMMIT*/              , this
/*SOMMIT*/              , sfd.getPID()
/*SOMMIT*/              , sfd.getPosFilterMask()              
/*SOMMIT*/              , sfd.getPosFilterDef()              
/*SOMMIT*/              );
/*SOMMIT*/      }
/*SOMMIT*/      else
/*SOMMIT*/      {
/*SOMMIT*/          /*DBG*/Debug.msg(Debug.HWSF,Debug.IMPL,"HwSfController.startSectionFilter pos&nef" );
/*SOMMIT*/          SommitSectionFilter.startFilter
/*SOMMIT*/              ( _sfNo, _filterId
/*SOMMIT*/              , this
/*SOMMIT*/              , sfd.getPID()
/*SOMMIT*/              , sfd.getPosFilterMask()              
/*SOMMIT*/              , sfd.getPosFilterDef()              
/*SOMMIT*/              , sfd.getNegFilterMask()              
/*SOMMIT*/              , sfd.getNegFilterDef()              
/*SOMMIT*/              );
/*SOMMIT*/      }

//TM*/          /*DBG*/Debug.msg(Debug.HWSF,Debug.HIGH,"TMSf sf("+_sfNo+") "+sfd.getPID()+" "+sfd.getTableId()+" "+sfd.getTableIdExtension() );
//TM*/          TMSectFilt.setFilter
//TM*/                  ( _filterId, _sfNo
//TM*/                  , sfd.getPID()
//TM*/                  , sfd.getTableId()
//TM*/                  , sfd.getTableIdExtension()
//TM*/                  , this 
//TM*/                  );
            }
            else
            {   // section filter revoked
                sendEvent( new HwSfEvent( this, HwSfEvent.RES_REVOKED, null, sfd ) );
            }
        }
        else
        {   // section filter revoked
            sendEvent( new HwSfEvent( this, HwSfEvent.TS_CHANGE, null, sfd ) );
        }
    }

    /*********************
    Indicates the filter resource of this section filter have been removed.
    */
    synchronized void revoke()
    {
        /*DBG*/Debug.msg(Debug.HWSF, Debug.IMPL, "HwSfController.revoke" );
        if ( _sfd != null )
        {   // filtering
            stopFiltering();
        }
        sendEvent( new HwSfEvent( this, HwSfEvent.RES_REVOKED, null, _sfd ) );
    }

    /*********************
    Indicates the filter resource of this section filter tuning has occured.
    */
    synchronized void tsChanged( boolean tuningOver )
    {
        /*DBG*/Debug.msg(Debug.HWSF, Debug.HIGH, "HwSfController.tsChanged( "+tuningOver+" )" );
        if ( !tuningOver )
        {
            if ( _sfd != null )
            {   // filtering
                stopFilter();
            }
            sendEvent( new HwSfEvent( this, HwSfEvent.TS_CHANGE, null, _sfd ) );
            _tuning = true;
        }
        else
        {
            _tuning = false;
        }
    }

    /*********************
    */
    public boolean resourceClaimed()
    {
        /*DBG*/Debug.msg(Debug.HWSF, Debug.IMPL, "HwSfController.resourceClaimed" );
        return ( _resAvailable );
    }

    /*********************
    */
    HwSfController( int sfNo, HwSfListener list )
    {
        /*DBG*/Debug.msg(Debug.HWSF, Debug.IMPL, "HwSfController " );
        _list = list;
        _sfNo = sfNo;
        _resAvailable = true;
        _tuning       = false;
    }
    
    
    ////////////////// IMPLEMENTATION //////////////////
    private int           _sfNo ;
    private boolean       _resAvailable;
    private HwSfListener  _list;
    private HwSfData      _sfd = null;
    private static volatile int      _filterId = 0;
    private boolean       _tuning;

    /****************
    Send an event to _list.
    */
    private void sendEvent( HwSfEvent event)
    {
        /*DBG*/Debug.msg(Debug.HWSF, Debug.IMPL, "HwSfController sendEvent "+event );
       	HwSfListenerInterfaceAdapter la =
    	   new HwSfListenerInterfaceAdapter( _list );
    	ActiveEventSender.getInstance( ActiveEventSender.HW_SF ).sendEvent( la, event );
    }

    private void stopFilter()
    {
        /*DBG*/Debug.msg(Debug.HWSF, Debug.IMPL, "HwSfController stopFilter" );
        if ( _sfd != null )
        {
/*SOMMIT*/  SommitSectionFilter.stopFilter( _sfNo );
//TM*/      TMSectFilt.clearFilter( _sfNo );
            _sfd = null;
            _filterId = getNextFilterId();
            _sfNo = -1;
        }
    }

    /*********************
    Section filter has filtered a new section on the indicated id by the TM section filter.
    */   
//TM*/public synchronized void iStore( int id, byte[] sec )
//TM*/{
//TM*/  Debug.msg(Debug.HWSF, Debug.IMPL, "HwSfController newSection" );
//TM*/  int res = 0;
//TM*/  HwSfEvent event;
//TM*/  HwSfListener l = _list;
//TM*/  HwSection hwSec = new HwSection( sec );
//TM*/  
//TM*/  synchronized( this )
//TM*/  {
//TM*/  	if ( id == _filterId && _resAvailable && _sfd!=null )
//TM*/      {   // section for this filter action
//TM*/          if ( _sfd.checkSection( hwSec ) )
//TM*/          {   // correct section
//TM*/              sendEvent( new HwSfEvent( this, HwSfEvent.NEW_SECTION, hwSec, _sfd ) );
//TM*/          }
//TM*/  	}
//TM*/  }
//TM*/}

    /*********************
    Section filter has filtered a new section on the indicated id by the sommit section filter.
    */   
/*SOMMIT*/public void newSection( int id, byte[] sec )
/*SOMMIT*/{
/*SOMMIT*/  /*DBG*/Debug.msg(Debug.HWSF, Debug.IMPL, "HwSfController newSection" );
/*SOMMIT*/  int res = 0;
/*SOMMIT*/  HwSfEvent event;
/*SOMMIT*/  HwSfListener l = _list;
/*SOMMIT*/  HwSection hwSec = new HwSection( sec );
/*SOMMIT*/    
/*SOMMIT*/  synchronized( this )
/*SOMMIT*/  {
/*SOMMIT*/  	if ( id == _filterId && _resAvailable && _sfd!=null )
/*SOMMIT*/      {   // section for this filter action
/*SOMMIT*/          if ( _sfd.checkSection( hwSec ) )
/*SOMMIT*/          {   // correct section
/*SOMMIT*/              sendEvent( new HwSfEvent( this, HwSfEvent.NEW_SECTION, hwSec, _sfd ) );
/*SOMMIT*/          }
/*SOMMIT*/  	}
/*SOMMIT*/  }
/*SOMMIT*/}

    /****************
    */
    private synchronized int getNextFilterId()
    {
        /*DBG*/Debug.msg(Debug.HWSF, Debug.IMPL, "HwSfController getNextFilterId" );
        _filterId ++;
        if ( _filterId == Integer.MAX_VALUE )
        {
        	_filterId  = Integer.MIN_VALUE;
        }
        return _filterId;
    }
}