/*******************************************************************************
* Copyright(c) Philips Consumer Electronics B.V. 1998                         
* All rights reserved. Reproduction in whole or in part is                  
* prohibited without the written consent of the copyright holder.            
*                                                                            
*  Project             : Bali
*  File name   $Logfile: $
*       Last save $Date: 2003/12/24 00:53:02 $
*             $Revision: 1.1.1.1 $
*          Last $Author: mikeseven $
*  Creation Date       : 16-11-1998
*  First Author        : Bas van den Heuvel
*                        NatLab - Eindhoven
* 
*  Change history      :
*  ---------------
*
* $Log: HwSfData.java,v $
* Revision 1.1.1.1  2003/12/24 00:53:02  mikeseven
* Initial release of the CVS repository for MPEG-4 Systems code
* after the code was corrupted due to reorganization and human 
* errors.
*
* Revision 1.1  2003/04/03 04:29:18  mikeseven
* Initial release of Player3D and MPEG-J for all MPEG-4 reference softwares players with samples.
* For installation, follow instructions in docs/Player3D_setup.doc.
*
* Many thanks to Prashanth Gujjeti and Viswanathan Swaminathan from Sun Microsystems.
*
* -- Mikael Bourges-Sevenier (Mindego)
*
* Revision 1.1  2002/08/23 16:47:21  pg138550
* Changes for MPEG-J. NOT FINAL
*
* Revision 1.1  2000/06/06 18:43:03  alavilli
* After SISF integration into the CORE4 code
*
* Revision 1.1.2.1  1999/07/09 14:16:40  depetris
* Aggiornati i file per lo sviluppo e il rilascio a MPEG-J.
* Integra la versione 2.5 del core.
*
* Revision 1.3  1999-05-04 17:49:12+02  vdheuvel
* Changes required after first Bouquet tests
*
* Revision 1.1  1998-12-04 15:08:50+01  vdheuvel
* HwSfData version for the sommit HW section filter.
*
* Revision 1.1  1998-12-04 15:08:50+01  vdheuvel
* 39 march 1999, start of archive.
* Middleware packages DAVIC API implementation.
*
*
*******************************************************************************/

package bali.hw.sectfilt;
import  bali.support.Util;
import  bali.support.Debug;

/***************
This class allows filtering of the individual sections. Each time a section is
filtered an event will send indicating this.
*/
public class HwSfData
{
    /*********************
	Start filtering for sections on with the indicated filter 
	parameters. 
	For all values except PID -1 is treated like a wildcard.
	@param PID           The PID to filter upon.
    */
    public HwSfData( int PID )
    {
        /*DBG*/ Debug.msg( Debug.SOMSF, Debug.HIGH,"HwSfData "+PID );
        _PID                = PID;
        _posFilterDef  = new byte[FILTER_LENGTH];
        _posFilterMask = new byte[FILTER_LENGTH];
        _negFilterDef  = null;
        _negFilterMask = null;
        
	    for ( int i=0; i<FILTER_LENGTH; i++)
        {
            _posFilterDef[i]  = (byte)0xFF;
            _posFilterMask[i] = (byte)0;
        }   
    }
    
    /*********************
	Start filtering for sections on with the indicated filter 
	parameters.
	For all values except PID -1 is treated like a wildcard.
	@param PID           The PID to filter upon.
	@param table_id      The table_id value of to be filtered sections.
    */
    public HwSfData( int PID , int table_id  )
	{
 	    this( PID );
        /*DBG*/ Debug.msg( Debug.SOMSF, Debug.HIGH,"HwSfData "+PID+" "+table_id );
	    if ( table_id!=-1 )
	    {
            _posFilterDef[0]  = (byte)table_id;
    	    _posFilterMask[0] = (byte)0xFF;
    	}
	}

    /*********************
	Start filtering for sections on with the indicated filter 
	parameters.
	For all values except PID -1 is treated like a wildcard.
	@param PID           The PID to filter upon.
	@param table_id      The table_id value of to be filtered sections.
	@param table_id_ext  The table_id_extension value of to be filtered sections.
    */
    public HwSfData( int PID 
                  , int table_id
                  , int table_id_ext
                  )
	{
	    this( PID, table_id );
        /*DBG*/ Debug.msg( Debug.SOMSF, Debug.HIGH,"HwSfData "+PID+" "+table_id+" "+table_id_ext );
	    if ( table_id_ext!=-1 )
	    {
    	    _posFilterDef[3]  = (byte)((table_id_ext >>8) & 0xFF);
    	    _posFilterMask[3] = (byte)0xFF;
    	    _posFilterDef[4]  = (byte)(table_id_ext & 0xFF);
    	    _posFilterMask[4] = (byte)0xFF;
    	}
	}

	
	/********************
	Start filtering for sections on with the indicated filter 
	parameters.
	For all values except PID -1 is treated like a wildcard.
	@param PID           The PID to filter upon.
	@param table_id      The table_id value of to be filtered sections.
	@param table_id_ext  The table_id_extension value of to be filtered sections.
	@param payl1         The value of the first 2 bytes of the payload of any to filtered sections.
	@param payl2         The value of the second 2 bytes of the payload of any to filtered sections.
	*/
    public HwSfData( int PID 
                  , int table_id
                  , int table_id_ext
                  , int payl1
                  , int payl2
                  )
    {
        this( PID, table_id, table_id_ext );
        /*DBG*/ Debug.msg( Debug.SOMSF, Debug.HIGH,"HwSfData "+PID+" "+table_id+" "+table_id_ext+" "+payl1+" "+payl2 );
	    if ( payl1!=-1 )
	    {
            _posFilterDef[8]   = (byte)((payl1 >>8) & 0xFF);
            _posFilterMask[8]  = (byte)0xFF;
            _posFilterDef[9]   = (byte)(payl1 & 0xFF);
            _posFilterMask[9]  = (byte)0xFF;
        }
	    if ( payl2!=-1 )
	    {
    	    _posFilterDef[10]  = (byte)((payl2 >>8) & 0xFF);
    	    _posFilterMask[10] = (byte)0xFF;
    	    _posFilterDef[11]  = (byte)(payl2 & 0xFF);
            _posFilterMask[11] = (byte)0xFF;
        }
    }
    
	/*********************
	Start filtering for sections on with the indicated filter 
	parameters.
	For all values except PID -1 is treated like a wildcard.
	@param PID           The PID to filter upon.
	@param table_id      The table_id value of to be filtered sections.
	@param table_id_ext  The table_id_extension value of to be filtered sections.
	@param version_no    The version number any sections may not have.
	@param payl1         The value of the first 2 bytes of the payload of any to filtered sections.
	@param payl2         The value of the second 2 bytes of the payload of any to filtered sections.
	*/
    public HwSfData( int PID 
                  , int table_id
                  , int table_id_ext
                  , int version_no
                  , int payl1
                  , int payl2
                  )
	{
        this( PID, table_id, table_id_ext, payl1, payl2 );

	    if ( version_no!=-1 )
	    {
            _negFilterDef  = new byte[FILTER_LENGTH];
            _negFilterMask = new byte[FILTER_LENGTH];
        	for ( int i=0; i<FILTER_LENGTH; i++)
            {
                _negFilterDef[i]  = (byte)0xFF;
                _negFilterMask[i] = (byte)0;
            }   
    	    _negFilterDef[5]   = (byte)(version_no<<1);
    	    _negFilterMask[6]  = (byte)0x3E;
        }
	}
  
	/*********************
	Start filtering for sections on with the indicated filter 
	parameters.
	@param PID           The PID to filter upon.
    @param offset defines the offset within the section which the first byte of the posFilterDef, 
    posFilterMask, negFilterDef and negFilterMask arrays is intended to match. The offset must be 
    less than 31 as described in DAVIC part 10, section 115.3. The offset must be equal to or
    greater than 3.
    @param posFilterDef defines values to match for bits in the section, as defined in clause H7.
    @param posFilterMask defines which bits in the section are to be compared against the values specified 
    in the posFilterDef parameter, as defined in clause H7.
    @param negFilterDef defines values to match for bits in the section, as defined in clause H7.
    @param negFilterMask defines which bits in the section are to be compared against the values specified 
    in the negFilterDef parameter, as defined in clause H7.
	*/
    public HwSfData( int PID 
                  , int table_id
                  , int offset
                  , byte posFilterDef[]
                  , byte posFilterMask[]
                  , byte negFilterDef[]
                  , byte negFilterMask[]
                  )
    {
        /*DBG*/ Debug.msg( Debug.SOMSF, Debug.HIGH,"HwSfData "+PID );
        int i;
        _PID                = PID;
        // init filter
        int posLength = Math.max( posFilterDef.length, posFilterMask.length ) + offset;
        int negLength = Math.max( negFilterDef.length, negFilterMask.length ) + offset;
        int maxLength = Math.max( negLength, posLength );
        _posFilterDef  = new byte[maxLength];
        _posFilterMask = new byte[maxLength];
	    for ( i=0; i<posLength; i++)
        {
            if ( i < offset )
            {
                _posFilterDef[i]  = (byte)0xFF;
                _posFilterMask[i] = (byte)0;
                _negFilterDef[i]  = (byte)0xFF;
                _negFilterMask[i] = (byte)0;
            }
            else
            {
                _posFilterDef[i]  = posFilterDef[i-offset];
                _posFilterMask[i] = negFilterMask[i-offset];
                _negFilterDef[i]  = posFilterDef[i-offset];
                _negFilterMask[i] = negFilterMask[i-offset];
            }
        }

    }

    //////////////////// IMPLEMENTATION //////////////////////////////

	/***************
	Return PID
	*/
	int getPID()
	{
		return _PID;
	}	

	/***************
	Return posFilterDef.
	*/
	byte[] getPosFilterDef()
	{
		return _posFilterDef;
	}	

	/***************
	Return posFilterMask.
	*/
	byte[] getPosFilterMask()
	{
		return _posFilterMask;
	}	

	/***************
	Return posFilterDef.
	*/
	byte[] getNegFilterDef()
	{
		return _negFilterDef;
	}	

	/***************
	Return posFilterMask.
	*/
	byte[] getNegFilterMask()
	{
		return _negFilterMask;
	}	

	/***************
	Check if the indicated section matches the filter parameters.
	*/
	boolean checkSection( HwSection sec )
	{
	    boolean res = true;
	    byte[] secData = sec.getData();
        // check positive filter
        int i=0;
//        while ( res && i<lgth )
//        {
//            if ( i<fd.posMask.length )
//            {   res = ( (secData[i]&fd.posMask[i]) == (fd.posValue[i]&fd.posMask[i]) );
//            }
//            /*DBG*/ Debug.msg( Debug.SOMSF, Debug.LOW,"SommitSectionFilter.checkSection pos["+i+"] "+res);  
//            if (  res && i<fd.negMask.length )
//            {   res = ( (secData[i]&fd.negMask[i]) != (fd.posMask[i]&fd.negValue[i]) );
//            }
//            /*DBG*/ Debug.msg( Debug.SOMSF, Debug.LOW,"SommitSectionFilter.checkSection neg["+i+"] "+res);  
//            i++;
//        }
	    return res;
	}
	
	
    /////////////// PRIVATE /////////////////
    private final static int FILTER_LENGTH = 12;
    private int _PID;
    private byte _posFilterDef[]  ;
    private byte _posFilterMask[] ;
    private byte _negFilterDef[]  ;
    private byte _negFilterMask[] ;
}