/*
* Copyright(c) Philips Consumer Electronics B.V. 1998                         
* All rights reserved. Reproduction in whole or in part is                  
* prohibited without the written consent of the copyright holder.            
*                                                                            
*  Project             : Bali
*  File name   $Logfile: /func_test_platf2/bali/si/scheduler/RetrieveEvent.java $
*       Last save $Date: 2003/12/24 00:53:02 $
*             $Revision: 1.1.1.1 $
*          Last $Author: mikeseven $
*  Creation Date       : 17/11/98
*  First Author        : S.A.F.A. van den Heuvel
*                        NatLab - Eindhoven
* 
*                                                                          
*  Change history      :
*  ---------------
*
* $Log: HwSfResourceManager.java,v $
* Revision 1.1.1.1  2003/12/24 00:53:02  mikeseven
* Initial release of the CVS repository for MPEG-4 Systems code
* after the code was corrupted due to reorganization and human 
* errors.
*
* Revision 1.1  2003/04/03 04:29:21  mikeseven
* Initial release of Player3D and MPEG-J for all MPEG-4 reference softwares players with samples.
* For installation, follow instructions in docs/Player3D_setup.doc.
*
* Many thanks to Prashanth Gujjeti and Viswanathan Swaminathan from Sun Microsystems.
*
* -- Mikael Bourges-Sevenier (Mindego)
*
* Revision 1.1  2002/08/23 16:47:21  pg138550
* Changes for MPEG-J. NOT FINAL
*
* Revision 1.1  2000/06/06 18:43:04  alavilli
* After SISF integration into the CORE4 code
*
* Revision 1.1.2.1  1999/07/09 14:16:42  depetris
* Aggiornati i file per lo sviluppo e il rilascio a MPEG-J.
* Integra la versione 2.5 del core.
*
* Revision 1.5  1999-05-04 17:53:18+02  vdheuvel
* Changes required after first Bouquet tests
*
* Revision 1.4  1999-04-26 11:51:34+02  vdheuvel
* *** empty log message ***
*
* Revision 1.3  1999-04-23 17:56:50+02  vdheuvel
* Bug fix: number of section filter. Make distinction between SOMMIT
* and TM.
*
* Revision 1.2  1999-04-01 14:23:07+02  vdheuvel
* Added support for ts changes.
*
* Revision 1.1  1998-12-14 18:08:48+01  vdheuvel
* 39 march 1999, start of archive.
* Middleware packages DAVIC API implementation.
*
*/

package bali.hw.sectfilt;
import  bali.support.Debug;
import  bali.support.ActiveEventSender;
/*SOMMIT*/import  sommit.sectfilt.SommitSectionFilter;
//TM-HW*/import prl.mpeg.TMSectFilt;
//TM-SIM*/import  bali.tm.sectfilt.TMSectFilt;
import  bali.hw.tuner.HwTunerListener;
import  bali.hw.tuner.HwTunerEvent;
import  bali.hw.tuner.HwTuner;

/*********************
This class manages the available section filter resources for the different
java packages. It allows claiming and releasing of resources. It is also
possible to claim and start a filter action directly. The distribution of 
section filters is done accoriding to a priority system. The section filter 
call with the highest priority may use the section filter. 
The class supports filtering of single sections, tables and sets of sections.
Each filter action will only stop when the required data has been filtered, the
resource is revoked or released or when stopFiltering is called.
When a tuning action is detected, all section filters are stopped and cannot
be restarted until the tuning action has been completed.
*/
public class HwSfResourceManager
    implements HwTunerListener
{
    // constant that define the different priority levels for the different
    // users
    /**Highest priority*/               public final static int HIGH    = 5;
    /**Priority of the Current Ts API*/ public final static int CTSAPI  = 4;
    /**Priority of the SI API*/         public final static int SIAPI   = 3;
    /**Priority of the SF API (high)*/  public final static int SFAPI_H = 2;
    /**Priority of the SF API (low)*/   public final static int SFAPI_L = 1;
    /**Lowest priority*/                public final static int LOW     = 0;

    /*********************
    Get the instance of the section filter manager.
    */
    public synchronized static HwSfResourceManager getInstance()
    {
        if ( _instance == null )
        {
            _instance = new HwSfResourceManager();
        }
        return _instance;
    }
    
    /*********************
    Claim a section filter.
    @param priority The priority of this claim.
    @param list     The listener that will receive any event regarding this section filter.
    @return         The claimed section filter (or null when none is available).
    */
    public HwSfController claimSectionFilter( int priority, HwSfListener list )
    {
        HwSfController sf = null;
        synchronized ( _arSFData )
        {
            /*DBG*/Debug.msg(Debug.HWSF, "Locate available section filter" );
            int indexLowestPriority = 0;
            int i=0;
            while ( i<_NSECFILT && _arSFData[i].sf!=null )
            {   if ( _arSFData[indexLowestPriority].sf == null )
                {   indexLowestPriority = i;
                }
                // locate lowest priority
                if ( _arSFData[i].priority < _arSFData[indexLowestPriority].priority )
                {   indexLowestPriority=i;
                }
                i++;
            }
            
            // interpret results
            if ( i!=_NSECFILT )
            {   // free section filter found
                /*DBG*/Debug.msg(Debug.HWSF, Debug.IMPL,"Free section filter "+i );
            }
            else
            {   // no free section filter, locate sf with lower priority
                if ( _arSFData[indexLowestPriority].priority < priority )
                {   // sf with lower priority present, revoke section filter
                    /*DBG*/Debug.msg(Debug.HWSF, "Revoke section filter :"+indexLowestPriority );
                    i = indexLowestPriority;
                	ActiveEventSender.getInstance( ActiveEventSender.HW_SF ).sendEvent( _listHand, new HwSfResourceEvent( _arSFData[i].sf, HwSfResourceEvent.RELEASED ) );
                    _arSFData[indexLowestPriority].sf.revoke();
                }
                else
                {   // no available section filter
                    return  null;
                }
            }

            // i indicates a available section filter, return new
            /*DBG*/Debug.msg(Debug.HWSF, "Section filter "+i+" claimed." );
            _arSFData[i].priority = priority;
            sf                    = new HwSfController( i, list );
            _arSFData[i].sf       = sf;
            ActiveEventSender.getInstance( ActiveEventSender.HW_SF ).sendEvent( _listHand, new HwSfResourceEvent( _arSFData[i].sf, HwSfResourceEvent.CLAIMED ) );
        }
        return sf;
    }


    /*********************
    Add a listener of section filter resource change events.
    */
    public void addHwSfResourceListener( HwSfResourceListener list )
    {
        /*DBG*/Debug.msg(Debug.HWSF, "addHwSfResourceListener "+list );
        _listHand.addListener( list );
    }    

    /*********************
    Remove a listener of section filter resource change events.
    */
    public void removeHwSfResourceListener( HwSfResourceListener list )
    {
        /*DBG*/Debug.msg(Debug.HWSF, "removeHwSfResourceListener "+list );
        _listHand.removeListener( list );
    }    
    
    /********************
    */
    void releaseSfResource( HwSfController sf )
    {
        /*DBG*/Debug.msg(Debug.HWSF,"Release SfResource" );
        synchronized( _arSFData )
        {
            int i=0;
            boolean ready = false;
            while ( i<_NSECFILT && !ready )
            {   if (sf==_arSFData[i].sf)
                {
                    ready = true;
                	ActiveEventSender.getInstance( ActiveEventSender.HW_SF ).sendEvent( _listHand, new HwSfResourceEvent( _arSFData[i].sf, HwSfResourceEvent.RELEASED ) );
                    _arSFData[i].sf       = null;
                    _arSFData[i].priority = -1; 
                }
                i++;
            }
            
        }
    }
    
    /********************
    */
    HwSfResourceManager()
    {
        /*DBG*/Debug.msg(Debug.HWSF, Debug.HIGH,"Init HwSfResourceManager" );
/*SOMMIT*/_NSECFILT = SommitSectionFilter.getNumberOfFilters();
//TM*/    _NSECFILT = 16;
        /*DBG*/Debug.msg(Debug.HWSF, Debug.HIGH,"Init HwSfResourceManager (after)" );
        _listHand = new HwSfResourceListenerHandler();
        _arSFData = new SFData[ _NSECFILT ];
        for ( int i=0; i<_NSECFILT; i++ )
        {   _arSFData[i] = new SFData();
            _arSFData[i].sf       = null;
            _arSFData[i].priority = -1; 
        }
    }

    /////////////////////////// IMPLEMENTATION //////////////////////////
    private static HwSfResourceManager   _instance = null;
    private HwSfResourceListenerHandler  _listHand;
    private static int                   _NSECFILT;              // total number of section filters
    private SFData[]                     _arSFData = null;       // array of section filter related data
    private static int                   _filterId = 0;          // section filter action identifier counter.


    /****************
    */
    private synchronized static int getFilterId()
    {
        _filterId ++;
        if ( _filterId == Integer.MAX_VALUE )
        {
        	_filterId  = Integer.MIN_VALUE;
        }
        return _filterId;
    }

    /******************
    The transport stream selection has been changed.
    */
    public void newHwTunerEvent( HwTunerEvent event )
    {
        synchronized( _arSFData )
        {
            for ( int i=0; i<_arSFData.length; i++ )
            {
                _arSFData[i].sf.tsChanged( event.getType() == event.TUNING_OVER );
            }
        }
    }

}

// Local torage class
class SFData
{
    HwSfController  sf; // when null this section filter is available
    int             priority; 
}