/*
* Copyright(c) Philips Consumer Electronics B.V. 1998                         
* All rights reserved. Reproduction in whole or in part is                  
* prohibited without the written consent of the copyright holder.            
*                                                                            
*  Project             : Bali
*  Module              : bali.si 
*  File name   $Logfile: /func_test_platf2/bali/si/scheduler/TableSectionFilterData.java $
*       Last save $Date: 2003/12/24 00:53:02 $
*             $Revision: 1.1.1.1 $
*          Last $Author: mikeseven $
*  Creation Date       : 28/10/98
*  First Author        : S.A.F.A. van den Heuvel
*                        NatLab - Eindhoven
* 
*                                                                          
*  Change history      :
*  ---------------
*
* $Log: TableData.java,v $
* Revision 1.1.1.1  2003/12/24 00:53:02  mikeseven
* Initial release of the CVS repository for MPEG-4 Systems code
* after the code was corrupted due to reorganization and human 
* errors.
*
* Revision 1.1  2003/04/03 04:29:22  mikeseven
* Initial release of Player3D and MPEG-J for all MPEG-4 reference softwares players with samples.
* For installation, follow instructions in docs/Player3D_setup.doc.
*
* Many thanks to Prashanth Gujjeti and Viswanathan Swaminathan from Sun Microsystems.
*
* -- Mikael Bourges-Sevenier (Mindego)
*
* Revision 1.1  2002/08/23 16:47:21  pg138550
* Changes for MPEG-J. NOT FINAL
*
* Revision 1.1  2000/06/06 18:43:04  alavilli
* After SISF integration into the CORE4 code
*
* Revision 1.1.2.1  1999/07/09 14:16:43  depetris
* Aggiornati i file per lo sviluppo e il rilascio a MPEG-J.
* Integra la versione 2.5 del core.
*
* Revision 1.1  1998-12-11 11:48:02+01  vdheuvel
* 39 march 1999, start of archive.
* Middleware packages DAVIC API implementation.
*
 * 
 * 1     9-11-98 13:59 Vdheuvel
 * Creation
*/

package bali.hw.sectfilt;
import  bali.support.Debug;
import  bali.support.Util;

/*******************
This class represents one the section filters available to the retrieve controller.
*/
public class TableData
{
	/**************
	May only be called with FilterData on a complete section table format.
	*/
	public TableData( )
	{
        /*DBG*/Debug.msg( Debug.HWSF_LE, Debug.IMPL, "TableData ");
      	_arSec = null;
      	_version = -1;
	}
	
	/**************
	Return true when all data for this section filter has been filtered.
	*/
	public synchronized boolean isDataComplete()
	{
        /*DBG*/Debug.msg(Debug.HWSF_LE, Debug.LOW, "TableData.isDataComplete" );
		if (  _arSec != null )
		{
			boolean res = true;
			for ( int i=0; i< _arSec.length; i++ )
			{
				res = res && ( _arSec[i]!=null );
			}
			return res;
		}
		else
		{	return false;
		}
	}

	/**************
	A new section has been filtered matching the filter parameters.
	When a version change has been detected the method will return false and
	a call to the method versionChangeOccured will return true;
	*/
	public synchronized void newSection( HwSection sec )
	{
        /*DBG*/Debug.msg(Debug.HWSF_LE, "TableData.newSection" );
		if ( _arSec == null )
		{	// first section on this table.
            /*DBG*/Debug.msg(Debug.HWSF_LE, Debug.IMPL, "TableData.newSection first section, table length "+(sec.last_section_number()+1) );
			_arSec = new HwSection[ sec.last_section_number()+1 ];
			for ( int i=0; i<sec.last_section_number();i++ )
			{	_arSec[i] = null;
			}
			_version = sec.version_number();
		}
		
		if ( sec.version_number() != _version )
		{	// version change detected
			_versionChangeOccured = true;
		}
		else if ( sec.section_number() < _arSec.length )
        {   /*DBG*/Debug.msg(Debug.HWSF_LE, Debug.IMPL, "TableData.newSection  sn "+sec.section_number() );
		    _arSec[ sec.section_number() ] = sec;
		}
	}
	
	/**************
	Return true when a version change has occured.
	*/
	public synchronized boolean versionChangeOccured()
	{
        /*DBG*/Debug.msg( Debug.HWSF_LE, Debug.IMPL, "TableData versionChangeOccured");
		return _versionChangeOccured;
	}
	
	/**************
	Return the filtered data.
	*/
	public synchronized HwSection[] getSections()
	{
        /*DBG*/Debug.msg(Debug.HWSF_LE, "TableData.getSections "+ _arSec.length );
	    if ( _arSec == null )
	    {
            _arSec = new HwSection[0];   
	    }
		return _arSec;
	}

	//////////////// PRIVATE ///////////////
	private HwSection[] _arSec = null;
	private int       _version;
	private boolean   _versionChangeOccured;
	
}







