/*
* Copyright(c) Philips Consumer Electronics B.V. 1998                         
* All rights reserved. Reproduction in whole or in part is                  
* prohibited without the written consent of the copyright holder.            
*                                                                            
*  Project             : Bali
*  Module              : bali.si 
*  File name   $Logfile: /func_test_platf2/bali/si/scheduler/TableSectionFilterData.java $
*       Last save $Date: 2003/12/24 00:53:02 $
*             $Revision: 1.1.1.1 $
*          Last $Author: mikeseven $
*  Creation Date       : 28/10/98
*  First Author        : S.A.F.A. van den Heuvel
*                        NatLab - Eindhoven
* 
*                                                                          
*  Change history      :
*  ---------------
*
* $Log: TableSfEnhancer.java,v $
* Revision 1.1.1.1  2003/12/24 00:53:02  mikeseven
* Initial release of the CVS repository for MPEG-4 Systems code
* after the code was corrupted due to reorganization and human 
* errors.
*
* Revision 1.1  2003/04/03 04:29:22  mikeseven
* Initial release of Player3D and MPEG-J for all MPEG-4 reference softwares players with samples.
* For installation, follow instructions in docs/Player3D_setup.doc.
*
* Many thanks to Prashanth Gujjeti and Viswanathan Swaminathan from Sun Microsystems.
*
* -- Mikael Bourges-Sevenier (Mindego)
*
* Revision 1.1  2002/08/23 16:47:21  pg138550
* Changes for MPEG-J. NOT FINAL
*
* Revision 1.1  2000/06/06 18:43:05  alavilli
* After SISF integration into the CORE4 code
*
* Revision 1.1.2.1  1999/07/09 14:16:44  depetris
* Aggiornati i file per lo sviluppo e il rilascio a MPEG-J.
* Integra la versione 2.5 del core.
*
* Revision 1.1  1998-12-11 11:48:02+01  vdheuvel
* 39 march 1999, start of archive.
* Middleware packages DAVIC API implementation.
*
 * 
 * 1     9-11-98 13:59 Vdheuvel
 * Creation
*/

package bali.hw.sectfilt;
import  bali.support.Debug;
import  bali.support.Util;

/*******************
This HwSfEnhancer stops and sends HwSfEvent::SUCCESS when a complete section table has been filtered.
Any other event will result in stopping the filter and passing the event. When filtering has
been stopped, all following events are ignored.
A section table is defined as:
- all sections have the same version number
- all sections between 0 and last_section_number are present.
- all sections are confirm the filter settings.

*/
public class TableSfEnhancer
    implements HwSfListener
{
	/**************
	*/
	public TableSfEnhancer( HwSfListener list )
	{
        /*DBG*/Debug.msg( Debug.HWSF_LE, Debug.IMPL, "TableSfEnhancer ");
        _finished = false;
      	_list    = list;
      	_td      = new TableData();
	}
	
	/**************
	A new HwSfEvent, process the event and resend it.
	*/
    public void newHwSfEvent( HwSfEvent e)
    {
        /*DBG*/Debug.msg( Debug.HWSF_LE, Debug.IMPL, "TableSfEnhancer newHwSfEvent");
        if ( !_finished )
        {
            /*DBG*/Debug.msg(Debug.HWSF_LE, "TableSfEnhancer new event");
            HwSfController sfc = (HwSfController)e.getSource();
            switch ( e.getResult() )
            {
            case HwSfEvent.NEW_SECTION:
                /*DBG*/Debug.msg(Debug.HWSF_LE, "TableSfEnhancer new section filtered");
                _td.newSection( e.getHwSection() );
                if ( _td.isDataComplete() )
                {
                    /*DBG*/Debug.msg(Debug.HWSF_LE, "TableSfEnhancer table complete");
                    _finished = true;
                    sfc.stopFiltering();
                    _list.newHwSfEvent( new HwSfEvent( e.getSource(), HwSfEvent.SUCCESS, null, e.getHwSfData() ) );
                }
                if ( _td.versionChangeOccured() )
                {
                    /*DBG*/Debug.msg(Debug.HWSF_LE, "TableSfEnhancer version change");
                    _finished = true;
                    sfc.stopFiltering();
                    _list.newHwSfEvent( new HwSfEvent( e.getSource(), HwSfEvent.VERSION_CHANGE, null, e.getHwSfData() ) );
                }
                break;
            default:
                /*DBG*/Debug.msg(Debug.HWSF_LE, "TableSfEnhancer. error event, error occured, stop filter");
                _finished = true;
                sfc.release();
                _list.newHwSfEvent( e );
                break;
            }
        }
        else
        {
            /*DBG*/Debug.msg(Debug.HWSF_LE, "TableSfEnhancer Already finished, event ignored.");
        }
    }
    
	/**************
	Return true when all data for this section filter has been filtered.
	*/
	public synchronized boolean isDataComplete()
	{
        /*DBG*/Debug.msg(Debug.HWSF_LE, "TableSfEnhancer.isDataComplete" );
        return _td.isDataComplete();
	}

	/**************
	Return true when a version change has occured.
	*/
	public synchronized boolean versionChangeOccured()
	{
        /*DBG*/Debug.msg( Debug.HWSF_LE, Debug.IMPL, "TableSfEnhancer versionChangeOccured");
		return _td.versionChangeOccured();
	}
	
	/**************
	Return the filtered data.
	*/
	public synchronized HwSection[] getSections()
	{
        /*DBG*/Debug.msg(Debug.HWSF_LE, "TableSfEnhancer.getSections " );
		return _td.getSections();
	}

	
	//////////////// PRIVATE ///////////////
	private HwSfListener _list;
    private boolean      _finished;
    private TableData    _td;
}







