/*
* Copyright(c) Philips Consumer Electronics B.V. 1998
* All rights reserved. Reproduction in whole or in part is
* prohibited without the written consent of the copyright holder.
*
*  Project             : Bali
*  Module              : bali.si
*  File name   $Logfile: $
*       Last save $Date: 2003/12/24 00:53:02 $
*             $Revision: 1.1.1.1 $
*          Last $Author: mikeseven $
*  Creation Date       : 08/02/199
*  First Author        : S.A.F.A. van den Heuvel
*                        NatLab - Eindhoven
*
*
*  Change history      :
*  ---------------
*
* $Log: BasicMonitor.java,v $
* Revision 1.1.1.1  2003/12/24 00:53:02  mikeseven
* Initial release of the CVS repository for MPEG-4 Systems code
* after the code was corrupted due to reorganization and human 
* errors.
*
* Revision 1.1  2003/04/03 04:29:22  mikeseven
* Initial release of Player3D and MPEG-J for all MPEG-4 reference softwares players with samples.
* For installation, follow instructions in docs/Player3D_setup.doc.
*
* Many thanks to Prashanth Gujjeti and Viswanathan Swaminathan from Sun Microsystems.
*
* -- Mikael Bourges-Sevenier (Mindego)
*
* Revision 1.1  2002/08/23 16:47:22  pg138550
* Changes for MPEG-J. NOT FINAL
*
* Revision 1.1  2000/06/06 18:43:05  alavilli
* After SISF integration into the CORE4 code
*
* Revision 1.1.2.1  1999/07/09 14:16:45  depetris
* Aggiornati i file per lo sviluppo e il rilascio a MPEG-J.
* Integra la versione 2.5 del core.
*
* Revision 1.2  1999-04-27 15:08:40+02  vdheuvel
* The old/new table mechanism has been extended to BasicMonitor.
*
* Revision 1.1  1999-02-24 20:26:24+01  vdheuvel
* 39 march 1999, start of archive.
* Middleware packages DAVIC API implementation.
*
*/

package bali.retrieve;
import  bali.support.Debug;
import  bali.support.Assert;
import  bali.cache.SITableCache;
import  bali.table.SITable;
import  bali.table.TableIdentifier;
import  java.util.Vector;
import  java.util.Enumeration;

/*********************
This class provide is the backbone of all monitors. It holds
the list of all active monitors and is responsible for active
filtering and cache controle.

@see MonitorEvent;
@see MonitorListener;
*/
public class BasicMonitor
{
    ////////////////// STATIC /////////////////////////
    
    /********************
    The indicated monitor starts monitoring of table ti.
    */
    static BasicMonitor startBasicMonitor( Monitor mon, TableIdentifier ti )
    {   
        BasicMonitor res = null;
        Vector vecBasMon = getBasicMonitorVector( ti.getTableType() );
        synchronized ( vecBasMon )
        {
            res = getBasicMonitor( vecBasMon, ti );
            if ( res == null )
            {   res = new BasicMonitor( ti );
            }
            vecBasMon.addElement( res );
            res.addMonitor( mon );
        }
        return res;
    }

    /********************
    The indicated monitor stops monitoring of table ti.
    */
    static void stopBasicMonitor( Monitor mon, TableIdentifier ti )
    {   
        Vector vecBasMon = getBasicMonitorVector( ti.getTableType() );
        synchronized ( vecBasMon )
        {
            BasicMonitor res = getBasicMonitor( vecBasMon, ti );
            if ( res != null )
            {
                res.removeMonitor( mon );
                if ( !res.isActive() )
                {   // monitor stopped remove BasicMonitor.
                    getBasicMonitorVector( ti.getTableType() ).removeElement( res );
                }
            }
        }
    }

    /**********************
    Indicate to the Monitors of this table a new table has been
    filtered.
    */
    static void newTableFiltered( SITable siTable )
    {
        /*DBG*/Debug.msg( Debug.MONITOR, "static BasicMonitor newTableFiltered");
        TableIdentifier ti = siTable.getTableIdentifier();
        Vector vecBasMon = getBasicMonitorVector( ti.getTableType() );
        synchronized ( vecBasMon )
        {
            BasicMonitor bm = getBasicMonitor( vecBasMon, ti );
            if ( bm != null )
            {
                bm.newTable( siTable );
            }
        }
    }

    ///////////////// STATIC PRIVATE ////////////////////////
    /* 
    The static part of this class stores the different Monitors. When
    a new table has been filtered, this static part is called and the
    different monitors on the table type are notified of the new table.
    */
    private static Vector[] _arVecMon = null;
    
    /**********************
    Get the Vector holding monitors of the indicated table.
    @return Vector of Monitors on this table.
    */
    private static BasicMonitor getBasicMonitor( Vector vecBasMon, TableIdentifier ti )
    {
        Enumeration  e   = vecBasMon.elements();
        BasicMonitor res = null;
        while ( e.hasMoreElements() &&  res == null )
        {
            BasicMonitor dummy = (BasicMonitor)e.nextElement();
            if ( dummy._ti.equals( ti ) )
            {   res = dummy;
            }
        }
        
        return res;
    }
    
    /**********************
    Get the Vector holding monitors of the indicated table.
    @return Vector of Monitors on this table.
    */
    private synchronized static Vector getBasicMonitorVector( int table )
    {
        if ( _arVecMon == null )
        {   _arVecMon = new Vector[ TableIdentifier.MAX_VALUE ];
        }
        /*DBG*/Assert.assert(table>=0);
        /*DBG*/Assert.assert(table<TableIdentifier.MAX_VALUE);
        if ( _arVecMon[table] == null )
        {   _arVecMon[table] = new Vector( 0,1 );
        }
        return _arVecMon[ table ];
    }
    

    ////////////////// NON STATIC /////////////////////////

    /*************************
    @param table The table to be monitored.
    */
    private BasicMonitor( TableIdentifier table )
    {
        /*DBG*/Debug.msg( Debug.MONITOR,"BasicMonitor("+table.getTableType()+") new on SITable "+table);
        SITableCache.startCaching( table, SITableCache.MONITOR );
        _ti        = table;
        _siTable   = SITableCache.getSITable( table );
        _monFriend = new MonitorFriend(this);
        _vecMon    = new Vector( 0, 1 );
        _active    = true;
        filterTable();
    }

    /*************************
    */
    private synchronized void stopBasicMonitor()
    {
        /*DBG*/Debug.msg( Debug.MONITOR, "BasicMonitor("+_ti.getTableType()+") stopBasicMonitor");
        _active = false;
        _rm.cancelRetrieve();
        SITableCache.stopCaching( _ti, SITableCache.MONITOR );
    }

    /*************************
    Access the last version of this SITable. When no table is
    available, null is returned.
    */
    SITable getLatestSITable()
    {
        /*DBG*/Debug.msg( Debug.MONITOR, "BasicMonitor("+_ti.getTableType()+") getLatestSITable");
        return _siTable;
    }
    /*************
    A new table of this table type has been filtered.
    */
    private void newTable( SITable table )
    {
        /*DBG*/Debug.msg( Debug.MONITOR, Debug.LOW, "BasicMonitor("+_ti.getTableType()+") newTable");
        if ( _ti.equals( table.getTableIdentifier() ) )
        {   // new table filtered
            /*DBG*/Debug.msg( Debug.MONITOR, "BasicMonitor("+_ti.getTableType()+") newTable send Event");
            synchronized( this )
            {
                sendEvent( _siTable, table );
                _siTable = table;
            }
        }
    }
    
    /*************
    Send a MonitorEvent to the listener.
    */
    private synchronized void sendEvent( SITable oldTable, SITable newTable )
    {
        /*DBG*/Debug.msg( Debug.MONITOR, "BasicMonitor("+_ti.getTableType()+") sendEvent");
        if ( _active )
        {   Enumeration e = _vecMon.elements();
            while ( e.hasMoreElements() )
            {   ((Monitor)e.nextElement()).newTable( oldTable, newTable );
            }
        }
    }

    /*************************
    Adds a monitor to the list of monitors on this table.
    */
    private synchronized void addMonitor( Monitor mon )
    {
        /*DBG*/Debug.msg( Debug.MONITOR, "BasicMonitor("+_ti.getTableType()+") addMonitor");
        _vecMon.addElement( mon );
    }

    /*************************
    Removes a monitor from the list of monitors on this table.
    */
    private synchronized void removeMonitor( Monitor mon )
    {
        /*DBG*/Debug.msg( Debug.MONITOR, "BasicMonitor("+_ti.getTableType()+") removeMonitor");
        _vecMon.removeElement( mon );
        if ( _vecMon.isEmpty() )
        {
            // stop monitoring
            stopBasicMonitor();
        }
    }

    /*************************
    */
    private boolean isActive()
    {
        /*DBG*/Debug.msg( Debug.MONITOR, "BasicMonitor("+_ti.getTableType()+") isActive "+_active);
        return _active;
    }
    
        /**********************
    Start a new filter action
    */
    private void filterTable()
    {
        /*DBG*/Debug.msg( Debug.MONITOR,  Debug.LOW,"BasicMonitor("+_ti.getTableType()+") filterTable");
        synchronized( this )
        {
            if ( _active )
            {   // still active
                _rm = RetrievalMediator.retrieveSITable( _monFriend, _ti );
                _rm.setMonitor( true );
            }
        }
    }

    /**********************
    Called by RetrieveMediator to indicate a new table has been filtered
    by this Monitor.
    */
    void newRetrievalEvent(RetrievalEvent e)    
    {
        /*DBG*/Debug.msg( Debug.MONITOR,  Debug.LOW,"BasicMonitor("+_ti.getTableType()+") newRetrievalEvent");
        filterTable();
    }

    ////////////////// IMPLEMENTATION ///////////////////////
    private TableIdentifier      _ti;
    private SITable              _siTable;
    private RetrievalMediator    _rm;
    private MonitorFriend        _monFriend;
    private boolean              _active;
    private Vector               _vecMon;

}




