/*
* Copyright(c) Philips Consumer Electronics B.V. 1998                         
* All rights reserved. Reproduction in whole or in part is                  
* prohibited without the written consent of the copyright holder.            
*                                                                            
*  Project             : Bali
*  Module              : bali.si 
*  File name   $Logfile: /func_test_platf2/bali/si/scheduler/TableSectionFilterData.java $
*       Last save $Date: 2003/12/24 00:53:02 $
*             $Revision: 1.1.1.1 $
*          Last $Author: mikeseven $
*  Creation Date       : 28/10/98
*  First Author        : S.A.F.A. van den Heuvel
*                        NatLab - Eindhoven
* 
*                                                                          
*  Change history      :
*  ---------------
*
* $Log: EITschSubTableData.java,v $
* Revision 1.1.1.1  2003/12/24 00:53:02  mikeseven
* Initial release of the CVS repository for MPEG-4 Systems code
* after the code was corrupted due to reorganization and human 
* errors.
*
* Revision 1.1  2003/04/03 04:29:22  mikeseven
* Initial release of Player3D and MPEG-J for all MPEG-4 reference softwares players with samples.
* For installation, follow instructions in docs/Player3D_setup.doc.
*
* Many thanks to Prashanth Gujjeti and Viswanathan Swaminathan from Sun Microsystems.
*
* -- Mikael Bourges-Sevenier (Mindego)
*
* Revision 1.1  2002/08/23 16:47:22  pg138550
* Changes for MPEG-J. NOT FINAL
*
* Revision 1.1  2000/06/06 18:43:05  alavilli
* After SISF integration into the CORE4 code
*
* Revision 1.1.2.1  1999/07/09 14:16:46  depetris
* Aggiornati i file per lo sviluppo e il rilascio a MPEG-J.
* Integra la versione 2.5 del core.
*
* Revision 1.1  1998-12-11 11:06:42+01  vdheuvel
* 39 march 1999, start of archive.
* Middleware packages DAVIC API implementation.
*
 * 
 * 1     9-11-98 13:59 Vdheuvel
 * Creation
*/

package bali.retrieve;
import  bali.support.Debug;
import  bali.support.Util;
import  bali.hw.sectfilt.HwSection;

/*******************
This class stores a EITsch sub table: all sections in one segment of one table_id.
*/
 class EITschSubTableData
{
	/**************
	May only be called with FilterData on a complete section table format.
	*/
    EITschSubTableData( )
	{
        /*DBG*/Debug.msg( Debug.RETRIEVE, Debug.IMPL, "EITschSubTableData ");
      	_arSec = null;
      	_version = -1;
     	_startSfNo = -1;
    }
	

	/**************
	Return true when all data for this section filter has been filtered.
	*/
	 synchronized boolean isDataComplete()
	{
        /*DBG*/Debug.msg(Debug.RETRIEVE, "EITschSubTableData.isDataComplete" );
		if (  _arSec != null )
		{
			boolean res = true;
			for ( int i=0; (i<_arSec.length) && res; i++ )
			{
				res = res && ( _arSec[i]!=null );
			}
			return res;
		}
		else
		{	return false;
		}
	}

	/**************
	A new section has been filtered matching the filter parameters.
	*/
	 synchronized void newSection( HwSection sec )
	{
        /*DBG*/Debug.msg(Debug.RETRIEVE, "EitSchSubTableData.newSection" );
		if ( _arSec == null )
		{	// first section on this table.
		    _startSfNo    = ((sec.section_number()>>3)<<3);
		    int tableLength = getSegmentLastSectionNumber( sec ) - _startSfNo + 1;
            
            /*DBG*/Debug.msg(Debug.RETRIEVE, Debug.LOW, "EitSchSubTableData.newSection first section, table length "+tableLength );
            
			_arSec = new HwSection[ tableLength ];
			for ( int i=0; i<_arSec.length; i++ )
			{	_arSec[i] = null;
			}
			_version = sec.version_number();
		}
        /*DBG*/Debug.msg(Debug.RETRIEVE, Debug.IMPL, "EitSchSubTableData "+sec.section_number()+" "+_startSfNo );
		
		if ( sec.version_number() != _version )
		{	// version change detected
            /*DBG*/Debug.msg(Debug.RETRIEVE, Debug.IMPL, "EitSchSubTableData version change" );
			_versionChangeOccured = true;
		}
		else if ( sec.section_number()-_startSfNo < _arSec.length )
        {   /*DBG*/Debug.msg(Debug.RETRIEVE, Debug.IMPL, "EitSchSubTableData.newSection  sn "+sec.section_number() );
		    _arSec[ sec.section_number()-_startSfNo ] = sec;
		}
	}
	
	/**************
	Return true when a version change has occured.
	*/
    synchronized boolean versionChangeOccured()
	{
        /*DBG*/Debug.msg( Debug.RETRIEVE, Debug.IMPL, "EITschSubTableData versionChangeOccured");
		return _versionChangeOccured;
	}
	
	/**************
	Return the filtered data.
	*/
	 synchronized HwSection[] getSections()
	{
        /*DBG*/Debug.msg(Debug.RETRIEVE, "EitSchSubTableData.getSections "+ _arSec.length );
	    if ( _arSec == null )
	    {
            _arSec = new HwSection[0];   
	    }
		return _arSec;
	}

    /***************
    */
    private int getSegmentLastSectionNumber( HwSection sec )
    {
        /*DBG*/Debug.msg( Debug.RETRIEVE, Debug.IMPL, "EITschSubTableData getSegmentLastSectionNumber");
        return (int)sec.get8bits(12);
    }
	
	//////////////// PRIVATE ///////////////
	private HwSection[]  _arSec = null;
	private int          _version;
	private boolean      _finished = false;
    private int          _startSfNo;
    private boolean      _versionChangeOccured = false;
}







