/*
* Copyright(c) Philips Consumer Electronics B.V. 1998                         
* All rights reserved. Reproduction in whole or in part is                  
* prohibited without the written consent of the copyright holder.            
*                                                                            
*  Project             : Bali
*  Module              : bali.si 
*  File name   $Logfile: /func_test_platf2/bali/si/scheduler/TableSectionFilterData.java $
*       Last save $Date: 2003/12/24 00:53:02 $
*             $Revision: 1.1.1.1 $
*          Last $Author: mikeseven $
*  Creation Date       : 28/10/98
*  First Author        : S.A.F.A. van den Heuvel
*                        NatLab - Eindhoven
* 
*                                                                          
*  Change history      :
*  ---------------
*
* $Log: EITschTableSetEnhancer.java,v $
* Revision 1.1.1.1  2003/12/24 00:53:02  mikeseven
* Initial release of the CVS repository for MPEG-4 Systems code
* after the code was corrupted due to reorganization and human 
* errors.
*
* Revision 1.1  2003/04/03 04:29:22  mikeseven
* Initial release of Player3D and MPEG-J for all MPEG-4 reference softwares players with samples.
* For installation, follow instructions in docs/Player3D_setup.doc.
*
* Many thanks to Prashanth Gujjeti and Viswanathan Swaminathan from Sun Microsystems.
*
* -- Mikael Bourges-Sevenier (Mindego)
*
* Revision 1.1  2002/08/23 16:47:22  pg138550
* Changes for MPEG-J. NOT FINAL
*
* Revision 1.1  2000/06/06 18:43:05  alavilli
* After SISF integration into the CORE4 code
*
* Revision 1.1.2.1  1999/07/09 14:16:46  depetris
* Aggiornati i file per lo sviluppo e il rilascio a MPEG-J.
* Integra la versione 2.5 del core.
*
* Revision 1.1  1998-12-11 15:19:10+01  vdheuvel
* 39 march 1999, start of archive.
* Middleware packages DAVIC API implementation.
*
 * 
 * 1     9-11-98 13:59 Vdheuvel
 * Creation
*/

package bali.retrieve;
import  bali.support.Debug;
import  bali.support.Assert;
import  bali.support.Util;
import  bali.hw.sectfilt.HwSfListener;
import  bali.hw.sectfilt.HwSfController;
import  bali.hw.sectfilt.HwSfEvent;
import  bali.hw.sectfilt.HwSection;
import  java.util.Vector;
import  java.util.Enumeration;

/*******************
This class represents one the section filters available to the retrieve controller.
*/
class EITschTableSetEnhancer
    implements HwSfListener
{
	/**************
	*/
    EITschTableSetEnhancer( HwSfListener list, int startPeriod, int endPeriod )
	{
        /*DBG*/Debug.msg(Debug.HWSF_LE, Debug.IMPL, "EITschTableSetEnhancer constructor");
      	_list              = list;
      	_vecTd             = new Vector();
        _finished          = false;
      	_startPeriod       = startPeriod;
      	_endPeriod         = endPeriod;
        _lastPeriod        = -1;
	}

	/**************
	A new HwSfEvent, process the event and resend it.
	*/
    public void newHwSfEvent( HwSfEvent event)
    {
        /*DBG*/Debug.msg(Debug.HWSF_LE, Debug.IMPL, "EITschTableSetEnhancer newHwSfEvent");
        if ( !_finished )
        {
            HwSfController sfc = (HwSfController)event.getSource();
            switch ( event.getResult() )
            {
            case HwSfEvent.NEW_SECTION:
                // check for correct period
                HwSection sec    = (HwSection)event.getHwSection();
                /*DBG*/Debug.msg(Debug.HWSF_LE, Debug.LOW, "EITschTableSetEnhancer NEW_SECTION "+sec.get16bits(10)+" "+sec.get16bits(8)+" "+sec.table_id_extension()+" "+ getPeriod(sec));
                
                if ( _lastPeriod == -1 )
                {
                    _lastPeriod = (sec.last_section_number()>>3);
                }

                int       period = getPeriod( sec );
                if ( _startPeriod<=period && period<=_endPeriod )
                {   // target section found
                    Enumeration e = _vecTd.elements();
                    EITschSubTableData td = null;
                    HwSection[] arSec;
                    boolean found = false;
                    while ( e.hasMoreElements() && !found )
                    {
                        td = (EITschSubTableData)e.nextElement();
                        arSec = td.getSections();
                        if ( arSec.length>0 )
                        {
                            found = tablesAreEqual( arSec[0], sec );
                        }
                    }
                    if ( !found )
                    {
                        /*DBG*/Debug.msg(Debug.HWSF_LE, Debug.LOW, "EITschTableSetEnhancer create EITschSubTableData");
                        td = new EITschSubTableData( );
                        _vecTd.addElement( td );
                    }
                    td.newSection( sec );
                }
                
                if ( isDataComplete() )
                {   // all data filtered
                    /*DBG*/Debug.msg(Debug.HWSF_LE, "EITschTableSetEnhancer data complete");
                    _finished = true;
                    sfc.stopFiltering();
                    /*DBG*/Debug.msg(Debug.HWSF_LE, Debug.IMPL, "EITschTableSetEnhancer filter stopped");
                    _list.newHwSfEvent( new HwSfEvent( event.getSource(), HwSfEvent.SUCCESS, null, event.getHwSfData() ) );
                }
                break;

            default:
                /*DBG*/Debug.msg(Debug.HWSF_LE, "EITschTableSetEnhancer new event, error occured, stop filter");
                _finished = true;
                sfc.stopFiltering();
                _list.newHwSfEvent( event );
                break;
            }
            
        }
        else
        {
            /*DBG*/Debug.msg(Debug.HWSF_LE, "EITschTableSetEnhancer Already finished, event ignored.");
        }
    }

	/**************
	Return true when all data for this section filter has been filtered.
	*/
	private synchronized boolean isDataComplete()
	{
        /*DBG*/Debug.msg(Debug.HWSF_LE, Debug.IMPL, "EITschTableSetEnhancer isDataComplete");
	    /*DBG*/ Assert.assert( _lastPeriod != -1 );
        
        if ( _lastPeriod < _startPeriod )
        {   // no data in requested periods in this EIT schedule
    	    /*DBG*/Debug.msg(Debug.HWSF_LE, Debug.IMPL, "EITschTableSetEnhancer.isDataComplete lastPeriod < startPeriod" );
            return true;
        }
                
        int Nperiod = Math.min( _endPeriod, _lastPeriod ) - _startPeriod + 1 ;
  	    /*DBG*/Debug.msg(Debug.HWSF_LE, Debug.IMPL, "EITschTableSetEnhancer.isDataComplete Nperiod "+Nperiod+" "+_vecTd.size() );
        if ( _vecTd.size() == Nperiod )
        {
            Enumeration e = _vecTd.elements();
            boolean     res = true;
            while ( e.hasMoreElements() && res )
            {   
                res &= ((EITschSubTableData)e.nextElement()).isDataComplete();
            }
            return res;
        }
        else
        {   return false;
        }
	}

    /****************
    */
     synchronized Enumeration getTableData()
    {
        /*DBG*/Debug.msg(Debug.HWSF_LE, Debug.IMPL, "EITschTableSetEnhancer getTableData");
        return _vecTd.elements();
    }

	
	//////////////// PRIVATE ///////////////
	private Vector       _vecTd = null;
	private HwSfListener _list;
	private boolean      _finished = false;
    private int          _startPeriod;
    private int          _endPeriod;
    private int          _lastPeriod; // last period in this EIT schule

    private boolean tablesAreEqual( HwSection sec1, HwSection sec2 )
    {
        return (  (sec1.table_id_extension() == sec2.table_id_extension() ) // service_id
               && (sec1.get16bits(10)        == sec2.get16bits(10) )        // or_nw_id
               && (sec1.get16bits(8)         == sec2.get16bits(8) )         // ts_id
               && (getPeriod(sec1)           == getPeriod(sec2) )           // period
               );
    }
    
    private int getPeriod( HwSection sec )
    {
//        return ( (int)(sec.section_number()/8) );
          return ( (int)(sec.section_number()>>3) );
    }
}








