/*
* Copyright(c) Philips Consumer Electronics B.V. 1998
* All rights reserved. Reproduction in whole or in part is
* prohibited without the written consent of the copyright holder.
*
*  Project             : Bali
*  Module              : bali.si
*  File name   $Logfile: $
*       Last save $Date: 2003/12/24 00:53:02 $
*             $Revision: 1.1.1.1 $
*          Last $Author: mikeseven $
*  Creation Date       : 08/02/199
*  First Author        : S.A.F.A. van den Heuvel
*                        NatLab - Eindhoven
*
*
*  Change history      :
*  ---------------
*
* $Log: Monitor.java,v $
* Revision 1.1.1.1  2003/12/24 00:53:02  mikeseven
* Initial release of the CVS repository for MPEG-4 Systems code
* after the code was corrupted due to reorganization and human 
* errors.
*
* Revision 1.1  2003/04/03 04:29:22  mikeseven
* Initial release of Player3D and MPEG-J for all MPEG-4 reference softwares players with samples.
* For installation, follow instructions in docs/Player3D_setup.doc.
*
* Many thanks to Prashanth Gujjeti and Viswanathan Swaminathan from Sun Microsystems.
*
* -- Mikael Bourges-Sevenier (Mindego)
*
* Revision 1.1  2002/08/23 16:47:22  pg138550
* Changes for MPEG-J. NOT FINAL
*
* Revision 1.1  2000/06/06 18:43:06  alavilli
* After SISF integration into the CORE4 code
*
* Revision 1.1.2.1  1999/07/09 14:16:46  depetris
* Aggiornati i file per lo sviluppo e il rilascio a MPEG-J.
* Integra la versione 2.5 del core.
*
* Revision 1.2  1999-04-28 10:07:08+02  vdheuvel
* The old/new table mechanism has been extended to BasicMonitor.
*
* Revision 1.1  1999-02-23 09:00:48+01  vdheuvel
* 39 march 1999, start of archive.
* Middleware packages DAVIC API implementation.
*
*/

package bali.retrieve;
import  bali.support.Debug;
import  bali.support.Assert;
import  bali.support.ActiveEventSender;
import  bali.table.TableIdentifier;
import  bali.table.SITable;

/*********************
This class provides a high level access point for monitoring of SI tables.
The class offers a set of static methods, each targeted at a SI-table. These
methods return Monitor instances. The Monitor can be used to identify the 
MonitorEvents and to cancel the monitor process.

@see MonitorEvent;
@see MonitorListener;
*/
public class Monitor
{
    ///////////////////// NON STATIC ////////////////////////////////

    /*************************
    @param list The listener to be notified, null when no listener needs to be notified.
    */
    public Monitor( MonitorListener list, TableIdentifier table )
    {
        /*DBG*/Debug.msg( Debug.MONITOR, Debug.HIGH, "Monitor SITable ("+list+" , "+table+" )");
        if ( list != null )
        {   _listAdapt = new MonitorListenerInterfaceAdapter( list );
        }
        _ti           = table;
        _basicMonitor = BasicMonitor.startBasicMonitor( this, table );
        _siTable      = null;
        _changed      = false;
        _siTable      = _basicMonitor.getLatestSITable();
    }

    /*************************
    Access the last version of this SITable. When no table is
    available, null is returned.
    */
    public SITable getLatestSITable()
    {
        /*DBG*/Debug.msg( Debug.MONITOR, Debug.LOW, "Monitor getLatestSITable");
        return _siTable;
    }

    /*************************
    Return true when the table has changed since the last time this method
    was called.
    */
    public boolean checkForTableChange()
    {
        boolean res;
        synchronized( this )
        {
        /*DBG*/Debug.msg( Debug.MONITOR, Debug.LOW, "Monitor checkForTableChange "+_changed);
            res = _changed;
            _changed = false;
        }
        return res;
    }
    

    /*************************
    Stop monitoring of this table.
    */
    public void stopMonitor()
    {
        /*DBG*/Debug.msg( Debug.MONITOR, Debug.HIGH, "Monitor stopMonitor");
        synchronized( this )
        {
            BasicMonitor.stopBasicMonitor( this, _ti );
            _listAdapt = null;
        }
    }

    /////////////////////////////////////////////////////////
    
    /*************
    A new table of this table type has been filtered.
    */
    void newTable( SITable oldTable, SITable newTable )
    {
        /*DBG*/Debug.msg( Debug.MONITOR, "Monitor("+_ti.getTableType()+") newTable "+newTable+" "+oldTable);
        if ( _ti.equals( newTable.getTableIdentifier() ) )
        {   // new table filtered
            synchronized( this )
            {
                if ( _siTable != null )
                {   if ( !_siTable.isSameTable( newTable ) )
                    {   _siTable = newTable;
                        sendEvent( oldTable, newTable );
                    }
                }
                else
                {   _siTable = newTable;
                    sendEvent( oldTable, newTable );
                }
            }
        }
    }
    
    /*************
    Send a MonitorEvent to the listener.
    */
    private void sendEvent( SITable oldTable, SITable newTable )
    {
        /*DBG*/Debug.msg( Debug.MONITOR, Debug.HIGH, "Monitor("+_ti.getTableType()+").sendEvent ");
        _changed = true;
        if ( _listAdapt != null )
        {
            ActiveEventSender.getInstance( ActiveEventSender.SI_INT ).
                sendEvent( _listAdapt, new MonitorEvent( this, oldTable, newTable ));
        }
    }

    
    ////////////////// IMPLEMENTATION ///////////////////////
    private SITable      _siTable;
    private MonitorListenerInterfaceAdapter _listAdapt = null;
    private BasicMonitor _basicMonitor;
    private TableIdentifier _ti;
    private boolean         _changed;

}




