/*
* Copyright(c) Philips Consumer Electronics B.V. 1998                         
* All rights reserved. Reproduction in whole or in part is                  
* prohibited without the written consent of the copyright holder.            
*                                                                            
*  Project             : Bali
*  Module              : bali.si 
*  File name   $Logfile: /func_test_platf2/bali/si/RetrieveMediatorEvent.java $
*       Last save $Date: 2003/12/24 00:53:02 $
*             $Revision: 1.1.1.1 $
*          Last $Author: mikeseven $
*  Creation Date       : 28/10/98
*  First Author        : S.A.F.A. van den Heuvel
*                        NatLab - Eindhoven
* 
*                                                                          
*  Change history      :
*  ---------------
*
* $Log: RetrievalEvent.java,v $
* Revision 1.1.1.1  2003/12/24 00:53:02  mikeseven
* Initial release of the CVS repository for MPEG-4 Systems code
* after the code was corrupted due to reorganization and human 
* errors.
*
* Revision 1.1  2003/04/03 04:29:23  mikeseven
* Initial release of Player3D and MPEG-J for all MPEG-4 reference softwares players with samples.
* For installation, follow instructions in docs/Player3D_setup.doc.
*
* Many thanks to Prashanth Gujjeti and Viswanathan Swaminathan from Sun Microsystems.
*
* -- Mikael Bourges-Sevenier (Mindego)
*
* Revision 1.1  2002/08/23 16:47:23  pg138550
* Changes for MPEG-J. NOT FINAL
*
* Revision 1.1  2000/06/06 18:43:07  alavilli
* After SISF integration into the CORE4 code
*
* Revision 1.1.2.1  1999/07/09 14:16:49  depetris
* Aggiornati i file per lo sviluppo e il rilascio a MPEG-J.
* Integra la versione 2.5 del core.
*
* Revision 1.1  1998-12-11 11:03:10+01  vdheuvel
* 39 march 1999, start of archive.
* Middleware packages DAVIC API implementation.
*
 * 
 * 1     9-11-98 13:59 Vdheuvel
 * Creation
*/

package bali.retrieve;
import  java.util.EventObject;
import  bali.hw.sectfilt.HwSfEvent;
import  bali.table.SITable;
import  bali.support.Debug;

/*********************
 This event is sent when a retrieve action has been completed.

 @see RetrieveMediator
 @see RetrieveMediatorListener
*/
public class RetrievalEvent
	extends EventObject
{
	/** The sections were successfuly filtered.  */
	    public final static int SUCCESS        = HwSfEvent.SUCCESS;
	/** A version number change occurred while filtering. */
    	public final static int VERSION_CHANGE = HwSfEvent.VERSION_CHANGE;
    /** Filtering stopped due to a time out. */
	    public final static int TIME_OUT       = 100;
    /** Filtering stopped because the source TS changed. */
    	public final static int TS_CHANGE      = HwSfEvent.TS_CHANGE;
    /** This filter action was canceled. */
    	public final static int CANCELLED      = HwSfEvent.CANCELLED;
    /** This filter action was canceled due to loss of section filter resource. */
    	public final static int RES_REVOKED    = HwSfEvent.RES_REVOKED;
    /** This filter action could not start due to lack of resources. */
    	public final static int NO_RES_AVAILABLE = 101;

	/*******************
	*/
	RetrievalEvent( Object source, int result )
	{ 
		super( source ); 
        /*DBG*/Debug.msg(Debug.RETRIEVE, Debug.IMPL, "RetrievalEvent constructor");
		_result  = result;
		_table   = null;
		_arTable = null;
	}

	/*******************
	*/
	RetrievalEvent( Object source, int result, SITable table )
	{ 
		super( source ); 
        /*DBG*/Debug.msg(Debug.RETRIEVE, Debug.IMPL, "RetrievalEvent constructor");
		_result  = result;
		_table   = table;
		_arTable = null;
	}

	/*******************
	*/
	RetrievalEvent( Object source, int result, SITable[] arTable )
	{ 
		super( source ); 
        /*DBG*/Debug.msg(Debug.RETRIEVE, Debug.IMPL, "RetrievalEvent constructor");
		_result  = result;
		_arTable = arTable;
		_table   = null;
	}
	
	/*******************
	Return the result of this retrieve action.
	*/
	public int getResult()
	{
        /*DBG*/Debug.msg(Debug.RETRIEVE, Debug.IMPL, "RetrievalEvent getResult");
		return _result;
	}

	/*******************
	Return the filtered table, only valid when result is SUCCESS and the
	corresponding filter operation didn't filter a set of tables. In all
	other cases null is returned.
	*/
	public SITable getSITable()
	{
        /*DBG*/Debug.msg(Debug.RETRIEVE, Debug.IMPL, "RetrievalEvent getSITable");
		return _table;
	}

	/*******************
	Return the filtered table, only valid when result is SUCCESS and the
	corresponding filter operation filtered a set of tables. In all
	other cases null is returned.
	*/
	public SITable[] getSITables()
	{
        /*DBG*/Debug.msg(Debug.RETRIEVE, Debug.IMPL, "RetrievalEvent getSITables");
		return _arTable;
	}

	////////////////////////// PRIVATE /////////////////////////////////
	private int 	    _result;
	private SITable     _table;
	private SITable[]   _arTable;
}

