/*
* Copyright(c) Philips Consumer Electronics B.V. 1998
* All rights reserved. Reproduction in whole or in part is
* prohibited without the written consent of the copyright holder.
*
*  Project             : Bali
*  Module              : bali.si
*  File name   $Logfile: /func_test_platf2/bali/si/RetrieveMediatorListener.java $
*       Last save $Date: 2003/12/24 00:53:03 $
*             $Revision: 1.1.1.1 $
*          Last $Author: mikeseven $
*  Creation Date       : 28/10/98
*  First Author        : S.A.F.A. van den Heuvel
*                        NatLab - Eindhoven
*
*
*  Change history      :
*  ---------------
*
* $Log: RetrievalMediator.java,v $
* Revision 1.1.1.1  2003/12/24 00:53:03  mikeseven
* Initial release of the CVS repository for MPEG-4 Systems code
* after the code was corrupted due to reorganization and human 
* errors.
*
* Revision 1.1  2003/04/03 04:29:23  mikeseven
* Initial release of Player3D and MPEG-J for all MPEG-4 reference softwares players with samples.
* For installation, follow instructions in docs/Player3D_setup.doc.
*
* Many thanks to Prashanth Gujjeti and Viswanathan Swaminathan from Sun Microsystems.
*
* -- Mikael Bourges-Sevenier (Mindego)
*
* Revision 1.1  2002/08/23 16:47:23  pg138550
* Changes for MPEG-J. NOT FINAL
*
* Revision 1.1  2000/06/06 18:43:07  alavilli
* After SISF integration into the CORE4 code
*
* Revision 1.1.2.1  1999/07/09 14:16:50  depetris
* Aggiornati i file per lo sviluppo e il rilascio a MPEG-J.
* Integra la versione 2.5 del core.
*
* Revision 1.7  1999-05-05 10:45:09+02  vdheuvel
* Changes required after first Bouquet tests
*
* Revision 1.6  1999-04-20 15:15:59+02  vdheuvel
* *** empty log message ***
*
* Revision 1.5  1999-04-16 11:08:42+02  vdheuvel
* Hook the platform to the TM STB.
*
* Revision 1.4  1999-04-15 09:53:18+02  vdheuvel
* Files changed to hook the system up to the TM demultipler and the
* TM hardware. Changes include, swapping the java.util.Date class
* for the bali.tm.Date class (to prevent using 64 bit floats). Changing
* the HW sect filt used by bali.hw.SectFilt.
*
* Revision 1.3  1999-04-01 14:55:48+02  vdheuvel
* *** empty log message ***
*
* Revision 1.2  1999-03-29 14:17:59+02  vdheuvel
* Removed WatchDog import.
*
* Revision 1.1  1999-02-23 20:43:08+01  vdheuvel
* 39 march 1999, start of archive.
* Middleware packages DAVIC API implementation.
*
 *
 * 1     9-11-98 13:59 Vdheuvel
 * Creation
*/

package bali.retrieve;
import  bali.component.TransportStreamSelection;
import  bali.support.Debug;
import  bali.support.Assert;
import  bali.support.ActiveEventSender;
import  bali.cache.SITableCache;
import  bali.table.SITable;
import  bali.table.NIT;
import  bali.table.SDT;
import  bali.table.BAT;
import  bali.table.TOT;
import  bali.table.TDT;
import  bali.table.PAT;
import  bali.table.PMT;
import  bali.table.EITpf;
import  bali.table.EITsch;
import  bali.table.TableIdentifier;
import  bali.table.InvalidTableException;
import  bali.hw.sectfilt.TableSfEnhancer;
import  bali.hw.sectfilt.SingleSfEnhancer;
import  bali.hw.sectfilt.HwSfController;
import  bali.hw.sectfilt.HwSfListener;
import  bali.hw.sectfilt.HwSfResourceManager;
import  bali.hw.sectfilt.HwSfData;
import  bali.hw.sectfilt.HwSection;
import  bali.hw.sectfilt.TableData;
import  org.davic.mpeg.dvb.DvbTransportStream;
/*DATE_NORMAL*/ import  java.util.Date;
//DATE_TM*/     import  bali.tm.Date;
import  java.util.Vector;
import  java.util.Enumeration;

/*********************
This class provides a high level access point for retrieval of SI tables.
The class offers a set of static methods, each targeted at a SI-table. These
methods return RetrieveMediator instances. The RetrieveMediator can be used
to identify the RetrieveMediatorEvents and to cancel the retrieve
action.
The class also provides a getRetrieveMediatorEvent method. This
method will block until the RetrieveMediatorEvent of this class is
created or return the sent event.
Each time a retrieve action is excecuted the table is offered to
the cache and the monitor modules.

@see RetrieveMediatorEvent
@see RetrieveMediatorListener
*/
public class RetrievalMediator
{
    /*******************
    Filter the indicated table (not EITsch, SERVICE, PMT ). The table must have a version number 
    different from the indicated one.
    @param list The listener to be notified, null: no event is sent.
    */
    static RetrievalMediator retrieveSITable( RetrievalListener list, TableIdentifier ti, int versionNo )
    {
        RetrievalMediator ret = null;
        int[] id = ti.getIdentifiers();
        switch( ti.getTableType() )
        {
        case TableIdentifier.NIT:
            ret = RetrievalMediator.retrieveNIT( list, id[0] );
            break;
        case TableIdentifier.SDT:
            ret = RetrievalMediator.retrieveSDT( list, id[0], id[1] );
            break;
        case TableIdentifier.TDT:
            ret = RetrievalMediator.retrieveTDT( list );
            break;
        case TableIdentifier.TOT:
            ret = RetrievalMediator.retrieveTOT( list );
            break;
        case TableIdentifier.PAT:
            ret = RetrievalMediator.retrievePAT( list );
            break;
        case TableIdentifier.PMT:
            synchronized( _patLock )
            {
                if ( _patMonitor == null )
                {   _patMonitor = PATMonitor.getInstance();
                }
            }
            PAT pat = (PAT)_patMonitor.getLatestSITable();
            int pid = pat.getProgramPid( id[2] );
            pid = ( pid==-2 ? 0: pid );
            ret = RetrievalMediator.retrievePMT( list, pid, id[0] );
            break;
        case TableIdentifier.EITpf:
            ret = RetrievalMediator.retrieveEITpf( list, id[0], id[1], id[2] );
            break;
        case TableIdentifier.EITsch:
     //       ret = RetrievalMediator.retrieveEITsch( list, id[0], id[1], id[2] );
            /*DBG*/ Assert.assert( true );
            break;
        case TableIdentifier.EITsch_ST:
            int segment = EITsch.getSegmentNo( id[3] );
            ret = RetrievalMediator.retrieveEITsch
                   ( list, id[0], id[1], id[2]
                   , EITsch.getTableIdOffset( id[3] )
                   , EITsch.getStartSectionNo( segment )
                   , EITsch.getEndSectionNo( segment ) 
                   );
            break;
        default:
            /*DBG*/Assert.assert( true );
        }
        return ret;
    }

    /*******************
    Filter the indicated table (not EITsch, SERVICE, PMT ).
    @param list The listener to be notified, null: no event is sent.
    */
    static public RetrievalMediator retrieveSITable( RetrievalListener list, TableIdentifier ti )
    {
        RetrievalMediator ret = null;
        int[] id = ti.getIdentifiers();
        switch( ti.getTableType() )
        {
        case TableIdentifier.NIT:
            ret = RetrievalMediator.retrieveNIT( list, id[0] );
            break;
        case TableIdentifier.SDT:
            ret = RetrievalMediator.retrieveSDT( list, id[0], id[1] );
            break;
        case TableIdentifier.TDT:
            ret = RetrievalMediator.retrieveTDT( list );
            break;
        case TableIdentifier.TOT:
            ret = RetrievalMediator.retrieveTOT( list );
            break;
        case TableIdentifier.PAT:
            ret = RetrievalMediator.retrievePAT( list );
            break;
        case TableIdentifier.PMT:
            synchronized( _patLock )
            {
                if ( _patMonitor == null )
                {   _patMonitor = PATMonitor.getInstance();
                }
            }
            PAT pat = (PAT)_patMonitor.getLatestSITable();
            int pid = pat.getProgramPid( id[2] );
            pid = ( pid==-2 ? 0: pid );
            ret = RetrievalMediator.retrievePMT( list, pid, id[0] );
            break;
        case TableIdentifier.EITpf:
            ret = RetrievalMediator.retrieveEITpf( list, id[0], id[1], id[2] );
            break;
        case TableIdentifier.EITsch:
     //       ret = RetrievalMediator.retrieveEITsch( list, id[0], id[1], id[2] );
            /*DBG*/ Assert.assert( true );
            break;
        case TableIdentifier.EITsch_ST:
            int segment = EITsch.getSegmentNo( id[3] );
            ret = RetrievalMediator.retrieveEITsch
                   ( list, id[0], id[1], id[2]
                   , EITsch.getTableIdOffset( id[3] )
                   , EITsch.getStartSectionNo( segment )
                   , EITsch.getEndSectionNo( segment ) 
                   );
            break;
        default:
            /*DBG*/Assert.assert( true );
        }
        return ret;
    }
    
    /*******************
    Filter the PAT of the current transport stream.
    @param list The listener to be notified, null: no event is sent.
    */
    static public RetrievalMediator retrievePAT( RetrievalListener list )
    {
        /*DBG*/Debug.msg( Debug.RETRIEVE, Debug.HIGH, "RetrievalMediator retrievePAT");
        RetrievalMediator rm    = new RetrievalMediator( list, RetrievalMediator.TABLE, TableIdentifier.PAT );
        HwSfData         sfd    = new HwSfData( PAT.PID, PAT.TABLE_ID );
        Retrieve         ar     = new Retrieve( PAT_TIMEOUT, sfd, rm );
        TableSfEnhancer  te     = new TableSfEnhancer( ar );
        retrieve( rm, ar, te );
        return rm;
    }

    /*******************
    Filter the PMT of the indicated program in the current transport stream.
    @param list The listener to be notified, null: no event is sent.
    */
    static public RetrievalMediator retrievePMT( RetrievalListener list, int pid, int serv_id )
    {
        /*DBG*/Debug.msg( Debug.RETRIEVE, Debug.HIGH, "RetrievalMediator retrievePMT "+pid+" "+serv_id);
        RetrievalMediator rm    = new RetrievalMediator( list, RetrievalMediator.TABLE, TableIdentifier.PMT );
        HwSfData         sfd    = new HwSfData( pid, PMT.TABLE_ID, serv_id );
        Retrieve         ar     = new Retrieve( PMT_TIMEOUT, sfd, rm );
        TableSfEnhancer  te     = new TableSfEnhancer( ar );
        retrieve( rm, ar, te );
        return rm;

    }

    /*******************
    Filter all SDT in the stream
    @param list The listener to be notified, null: no event is sent.
    */
    static public RetrievalMediator retrieveAllSDTother( RetrievalListener list )
    {
        /*DBG*/Debug.msg( Debug.RETRIEVE, Debug.HIGH, "RetrievalMediator retrieveAllSDT");
        RetrievalMediator rm    = new RetrievalMediator( list, RetrievalMediator.TABLE_SET, TableIdentifier.SDT );
        HwSfData         sfd    = new HwSfData( SDT.PID, SDT.TABLE_ID_OTHER );
        Retrieve         ar     = new Retrieve( SDT_TIMEOUT, sfd, rm );
        SDTTableSetEnhancer  te = new SDTTableSetEnhancer( ar );
        retrieve( rm, ar, te );
        return rm;
    }

    /*******************
    Filter a SDT.
    @param list The listener to be notified, null: no event is sent.
    */
    static public RetrievalMediator retrieveSDT( RetrievalListener list, int or_nw_id, int ts_id )
    {
        /*DBG*/Debug.msg( Debug.RETRIEVE, Debug.HIGH, "RetrievalMediator retrieveSDT "+or_nw_id+" "+ts_id);
        DvbTransportStream ts = TransportStreamSelection.getInstance().getCurrentTs();
        HwSfData           sfd;

        // check SDT origin
        if ( (ts.getTransportStreamId()==ts_id) && (ts.getOriginalNetworkId()==or_nw_id ) )
        {   // SDT actual
            sfd    = new HwSfData( SDT.PID, SDT.TABLE_ID_ACTUAL, ts_id );
        }
        else
        {   // SDT other
            sfd    = new HwSfData( SDT.PID, SDT.TABLE_ID_OTHER, ts_id );
        }
        RetrievalMediator rm    = new RetrievalMediator( list, RetrievalMediator.TABLE, TableIdentifier.SDT );
        Retrieve         ar     = new Retrieve( SDT_TIMEOUT, sfd, rm );
        TableSfEnhancer  te     = new TableSfEnhancer( ar );
        retrieve( rm, ar, te );
        return rm;
    }

    /*******************
    Filter the SDT actual.
    @param list The listener to be notified, null: no event is sent.
    */
    static public RetrievalMediator retrieveSDTActual( RetrievalListener list )
    {
        /*DBG*/Debug.msg( Debug.RETRIEVE, Debug.HIGH, "RetrievalMediator retrieveSDTActual");
        RetrievalMediator rm    = new RetrievalMediator( list, RetrievalMediator.TABLE, TableIdentifier.SDT );
        HwSfData         sfd    = new HwSfData( SDT.PID, SDT.TABLE_ID_ACTUAL );
        Retrieve         ar     = new Retrieve( SDT_TIMEOUT, sfd, rm );
        TableSfEnhancer  te     = new TableSfEnhancer( ar );
        retrieve( rm, ar, te );
        return rm;
    }

    /*******************
    Filter the EITpf of the indicated service.
    @param list The listener to be notified, null: no event is sent.
    */
    static public RetrievalMediator retrieveEITpf( RetrievalListener list, int or_nw_id, int ts_id, int serv_id )
    {
        /*DBG*/Debug.msg( Debug.RETRIEVE, Debug.HIGH, "RetrievalMediator retrieveEITpf "+or_nw_id+" "+ts_id+" "+serv_id);
        DvbTransportStream ts = TransportStreamSelection.getInstance().getCurrentTs();
        HwSfData         sfd;

        if ( (ts.getTransportStreamId()==ts_id) && (ts.getOriginalNetworkId()==or_nw_id ) )
        {   // EITpf actual
            sfd = new HwSfData( EITpf.PID, EITpf.TABLE_ID_EITpf_ACTUAL, serv_id, -1, ts_id, or_nw_id );
        }
        else
        {   // EITpf other
            sfd = new HwSfData( EITpf.PID, EITpf.TABLE_ID_EITpf_OTHER, serv_id, -1, ts_id, or_nw_id );
        }

        RetrievalMediator rm    = new RetrievalMediator( list, RetrievalMediator.TABLE, TableIdentifier.EITpf );
        Retrieve         ar     = new Retrieve( EITpf_TIMEOUT, sfd, rm );
        TableSfEnhancer  te     = new TableSfEnhancer( ar );
        retrieve( rm, ar, te );
        return rm;
    }

    /*******************
    Filter the EITsch periods of the indicated service from the indicated sub table.
    @param list The listener to be notified, null: no event is sent.
    */
    static public RetrievalMediator retrieveEITsch( RetrievalListener list, int or_nw_id, int ts_id, int serv_id, int tableIdOffset, int startPeriod, int endPeriod )
    {
        /*DBG*/Debug.msg( Debug.RETRIEVE, Debug.HIGH, "RetrievalMediator retrieveEITsch "+or_nw_id+" "+ts_id+" "+serv_id+" "+tableIdOffset+" "+startPeriod+" "+endPeriod);
        DvbTransportStream ts = TransportStreamSelection.getInstance().getCurrentTs();
        HwSfData         sfd;

        if ( (ts.getTransportStreamId()==ts_id) && (ts.getOriginalNetworkId()==or_nw_id ) )
        {   // EITsch actual
            sfd = new HwSfData( EITsch.PID, EITsch.TABLE_ID_EITsch_ACTUAL_START+tableIdOffset, serv_id, -1, ts_id, or_nw_id );
        }
        else
        {   // EITsch other
            sfd = new HwSfData( EITsch.PID, EITsch.TABLE_ID_EITsch_OTHER_START+tableIdOffset, serv_id, -1, ts_id, or_nw_id );
        }

        RetrievalMediator rm    = new RetrievalMediator( list, RetrievalMediator.TABLE_EITsch, TableIdentifier.EITsch );
        Retrieve         ar     = new Retrieve( EITsch_TIMEOUT, sfd, rm );
        EITschTableSetEnhancer  te     = new EITschTableSetEnhancer( ar, startPeriod, endPeriod );
        retrieve( rm, ar, te );
        return rm;
    }

    /*******************
    Filter all NIT other in the stream
    @param list The listener to be notified, null: no event is sent.
    */
    static public RetrievalMediator retrieveAllNITother( RetrievalListener list )
    {
        /*DBG*/Debug.msg( Debug.RETRIEVE, Debug.HIGH, "RetrievalMediator retrieveAllNITother");
        RetrievalMediator rm    = new RetrievalMediator( list, RetrievalMediator.TABLE_SET, TableIdentifier.NIT );
        HwSfData         sfd    = new HwSfData( NIT.PID, NIT.TABLE_ID_OTHER );
        Retrieve         ar     = new Retrieve( NIT_TIMEOUT, sfd, rm );
        NITTableSetEnhancer  te = new NITTableSetEnhancer( ar );
        retrieve( rm, ar, te );
        return rm;
    }

    /*******************
    Filter a NIT.
    @param list The listener to be notified, null: no event is sent.
    */
    static public RetrievalMediator retrieveNIT( RetrievalListener list, int nw_id )
    {
        /*DBG*/Debug.msg( Debug.RETRIEVE, Debug.HIGH, "RetrievalMediator retrieveNIT "+nw_id );
        DvbTransportStream ts = TransportStreamSelection.getInstance().getCurrentTs();
        HwSfData           sfd;

        // check NIT origin
        if ( ts.getNetworkId()==nw_id )
        {   // NIT actual
            sfd    = new HwSfData( NIT.PID, NIT.TABLE_ID_ACTUAL, nw_id );
        }
        else
        {   // NIT other
            sfd    = new HwSfData( NIT.PID, NIT.TABLE_ID_OTHER );
        }
        RetrievalMediator rm    = new RetrievalMediator( list, RetrievalMediator.TABLE, TableIdentifier.NIT );
        Retrieve         ar     = new Retrieve( NIT_TIMEOUT, sfd, rm );
        TableSfEnhancer  te     = new TableSfEnhancer( ar );
        retrieve( rm, ar, te );
        return rm;
    }
    
    /*******************
    Filter the NIT actual.
    @param list The listener to be notified, null: no event is sent.
    */
    static public RetrievalMediator retrieveNITActual( RetrievalListener list )
    {
        /*DBG*/Debug.msg( Debug.RETRIEVE, Debug.HIGH, "RetrievalMediator retrieveNITActual");
        RetrievalMediator rm    = new RetrievalMediator( list, RetrievalMediator.TABLE, TableIdentifier.NIT );
        HwSfData         sfd    = new HwSfData( NIT.PID, NIT.TABLE_ID_ACTUAL );
        Retrieve         ar     = new Retrieve( NIT_TIMEOUT, sfd, rm );
        TableSfEnhancer  te     = new TableSfEnhancer( ar );
        retrieve( rm, ar, te );
        return rm;
    }

    /*******************
    Filter all BAT in the stream
    @param list The listener to be notified, null: no event is sent.
    */
    static public RetrievalMediator retrieveAllBAT( RetrievalListener list )
    {
        /*DBG*/Debug.msg( Debug.RETRIEVE, Debug.HIGH, "RetrievalMediator retrieveAllBAT");
        RetrievalMediator rm    = new RetrievalMediator( list, RetrievalMediator.TABLE_SET, TableIdentifier.BAT );
        HwSfData         sfd    = new HwSfData( BAT.PID, BAT.TABLE_ID );
        Retrieve         ar     = new Retrieve( BAT_TIMEOUT, sfd, rm );
        NITTableSetEnhancer  te = new NITTableSetEnhancer( ar );
        retrieve( rm, ar, te );
        return rm;
    }


    /*******************
    Filter a BAT.
    @param list The listener to be notified, null: no event is sent.
    */
    static public RetrievalMediator retrieveBAT( RetrievalListener list, int bq_id )
    {
        /*DBG*/Debug.msg( Debug.RETRIEVE, Debug.HIGH, "RetrievalMediator retrieveBAT "+bq_id );

        HwSfData          sfd   = new HwSfData( BAT.PID, BAT.TABLE_ID, bq_id );
        RetrievalMediator rm    = new RetrievalMediator( list, RetrievalMediator.TABLE, TableIdentifier.BAT );
        Retrieve          ar    = new Retrieve( BAT_TIMEOUT, sfd, rm );
        TableSfEnhancer   te    = new TableSfEnhancer( ar );
        retrieve( rm, ar, te );
        return rm;
    }

    /*******************
    Filter the TDT.
    @param list The listener to be notified, null: no event is sent.
    */
    static public RetrievalMediator retrieveTDT( RetrievalListener list )
    {
        /*DBG*/Debug.msg( Debug.RETRIEVE, Debug.HIGH, "RetrievalMediator retrieveTDT " );

        HwSfData          sfd   = new HwSfData( TDT.PID, TDT.TABLE_ID );
        RetrievalMediator rm    = new RetrievalMediator( list, RetrievalMediator.SINGLE, TableIdentifier.TDT );
        Retrieve          ar    = new Retrieve( TDT_TIMEOUT, sfd, rm );
        SingleSfEnhancer   te   = new SingleSfEnhancer( ar );
        retrieve( rm, ar, te );
        return rm;
    }

    /*******************
    Filter the TOT.
    @param list The listener to be notified, null: no event is sent.
    */
    static public RetrievalMediator retrieveTOT( RetrievalListener list )
    {
        /*DBG*/Debug.msg( Debug.RETRIEVE, Debug.HIGH, "RetrievalMediator retrieveTOT " );

        HwSfData          sfd   = new HwSfData( TOT.PID, TOT.TABLE_ID );
        RetrievalMediator rm    = new RetrievalMediator( list, RetrievalMediator.SINGLE, TableIdentifier.TOT );
        Retrieve          ar    = new Retrieve( TOT_TIMEOUT, sfd, rm );
        SingleSfEnhancer   te   = new SingleSfEnhancer( ar );
        retrieve( rm, ar, te );
        return rm;
    }

    ///////////////////// NON STATIC ////////////////////////////////

    /*************************
    */
    RetrievalMediator( RetrievalListener list, int type, int table )
    {
        /*DBG*/Debug.msg( Debug.RETRIEVE, Debug.IMPL, "RetrievalMediator ");
        _retr = null;
        _list = list;
        _filterType = type;
        _tableType = table;
    }

    /*************************
    */
    public void cancelRetrieve()
    {
        /*DBG*/Debug.msg( Debug.RETRIEVE, Debug.IMPL, "RetrievalMediator cancelRetrieve");
        /*DBG*/ Assert.assert( _retr != null );
        RetrieveScheduler.getInstance().cancelRetrieve( _retr );
    }

    /*************************
    */
    public RetrievalEvent getRetrievalEvent()
    {
        /*DBG*/Debug.msg( Debug.RETRIEVE, Debug.IMPL, "RetrievalMediator getRetrievalEvent");
        synchronized( this )
    	{   while( _event == null )
            {
                try
                {   this.wait();
                }
                catch ( InterruptedException e )
                {}
            }
        }
        return _event;
    }

    /*************************
    @param result The filter result as in RetrievalEvent.
    */
    void activeRetrievalResult( int result, Retrieve ret )
    {
        /*DBG*/Debug.msg( Debug.RETRIEVE, Debug.IMPL, "RetrievalMediator activeRetrievalResult");
        /*DBG*/Debug.msg( Debug.RETRIEVE, "RetrievalMediator retrieve finished" );
        synchronized( this )
    	{
    	    try
    	    {
                SITable table;
                switch ( result )
                {
                case RetrievalEvent.SUCCESS :
                    switch( _filterType )
                    {
                    case TABLE:
                        TableSfEnhancer te = (TableSfEnhancer)ret.getHwSfListener();
                        table = getSITable( te.getSections(), ret.getTsSource(), ret.getFilterTime() );
                        _event = new RetrievalEvent( this, result, table );
                        break;
                    case SINGLE:
                        SingleSfEnhancer se = (SingleSfEnhancer)ret.getHwSfListener();
                        HwSection[] arSec = new HwSection[1];
                        arSec[0] = se.getSection();
                        table = getSITable( arSec, ret.getTsSource(), ret.getFilterTime() );
                        _event = new RetrievalEvent( this, result, table );
                        break;
                    case TABLE_EITsch:
                        /*DBG*/Debug.msg( Debug.RETRIEVE, Debug.LOW, "RetrievalMediator EITsch retrieve finished with SUCCESS" );
                        EITschTableSetEnhancer  tse  = (EITschTableSetEnhancer)ret.getHwSfListener();
                        Enumeration       e    = tse.getTableData();
                        Vector            vec  = new Vector(1);
                        while ( e.hasMoreElements() )
                        {
                            EITschSubTableData td = (EITschSubTableData)e.nextElement();
                            if ( td.isDataComplete() )
                            {
                                vec.addElement( getSITable( td.getSections(), ret.getTsSource(), ret.getFilterTime() ) );
                            }
                        }
                   		SITable[] arTable = new SITable[ vec.size() ];
                		vec.copyInto( arTable );
                		_event = new RetrievalEvent( this, result, arTable );
                        break;
                    default:
                        /*DBG*/Assert.assert( false );
                        break;
                    }
                    break;
                case RetrievalEvent.TIME_OUT :
                    if ( _filterType == TABLE_SET )
                    {
                        TableSetSfEnhancer  tse  = (TableSetSfEnhancer)ret.getHwSfListener();
                        Enumeration       e    = tse.getTableData();
                        Vector            vec  = new Vector(1);
                        while ( e.hasMoreElements() )
                        {
                            TableData td = (TableData)e.nextElement();
                            if ( td.isDataComplete() )
                            {
                                vec.addElement( getSITable( td.getSections(), ret.getTsSource(), ret.getFilterTime() ) );
                            }
                        }
                   		SITable[] arTable = new SITable[ vec.size() ];
                		vec.copyInto( arTable );
                		_event = new RetrievalEvent( this, result, arTable );
                        break;
                    }
                default :
                    _event = new RetrievalEvent( this, result );
                    break;
                }
            }
            catch ( InvalidTableException e )
            {   _event = new RetrievalEvent( this, RetrievalEvent.TIME_OUT );
            }    
            sendEvent( _event );
            this.notifyAll();

        }
    }

    /******************
    The RetrieveListener stack has been created. Set the Retrieve this RetrieveMediator
    works with.
    */
    void setRetrieve( Retrieve retr )
    {
        /*DBG*/Debug.msg( Debug.RETRIEVE, Debug.IMPL, "RetrievalMediator setRetrieve");
        _retr = retr;
    }

    /******************
    When this is monitor initiated retrieve, this can be indicated here (retrieve must be known).
    */
    void setMonitor( boolean monitor )
    {
        /*DBG*/Debug.msg( Debug.RETRIEVE, Debug.IMPL, "RetrievalMediator setMonitor( "+monitor+")");
        _retr.setMonitor( monitor );
    }

    //////////// IMPLEMENTATION /////////////////
    /** perminent monitor on PAT required to filter PMT using TableIdentifier.*/
    private static Monitor _patMonitor = null;
    private static Object  _patLock = new Object();

    /*******************
    */
    private static void retrieve( RetrievalMediator rm, Retrieve ar, HwSfListener te )
    {
        /*DBG*/Debug.msg( Debug.RETRIEVE, Debug.IMPL, "RetrievalMediator retrieve");
        rm.setRetrieve( ar );
        ar.setHwSfListener( te );
        RetrieveScheduler.getInstance().scheduleRetrieve( ar );
    }

    /*******************
    */
    private void sendEvent( RetrievalEvent event )
    {
        /*DBG*/Debug.msg( Debug.RETRIEVE, Debug.IMPL, "RetrievalMediator sendEvent");
        if ( _list != null )
        {
            ActiveEventSender.getInstance( ActiveEventSender.SI_INT ).
                sendEvent( new RetrievalListenerInterfaceAdapter(_list), event );
        }
    }

    /*******************
    */
    private SITable getSITable( HwSection arSec[], DvbTransportStream ts, Date filterTime )
        throws InvalidTableException
    {
        /*DBG*/Debug.msg( Debug.RETRIEVE, Debug.LOW, "RetrievalMediator create table" );
        SITable ret=null;
        try
        {
            switch( _tableType )
            {
            case TableIdentifier.BAT:
                ret = new BAT( arSec, ts, filterTime );
                break;
            case TableIdentifier.NIT:
                ret = new NIT( arSec, ts, filterTime );
                break;
            case TableIdentifier.SDT:
                ret = new SDT( arSec, ts, filterTime );
                break;
            case TableIdentifier.TDT:
                ret = new TDT( arSec, ts, filterTime );
                break;
            case TableIdentifier.TOT:
                ret = new TOT( arSec, ts, filterTime );
                break;
            case TableIdentifier.PAT:
                ret = new PAT( arSec, ts, filterTime );
                break;
            case TableIdentifier.PMT:
                ret = new PMT( arSec, ts, filterTime );
                break;
            case TableIdentifier.EITpf:
                ret = new EITpf( arSec, ts, filterTime );
                break;
            case TableIdentifier.EITsch:
                ret = new EITsch( arSec, ts, filterTime );
                break;
            default:
                /*DBG*/Assert.assert( false );
            }
            /*DBG*/Debug.msg( Debug.RETRIEVE, Debug.LOW, "RetrievalMediator offer table to cache");
            SITableCache.newTable( ret );
            /*DBG*/Debug.msg( Debug.RETRIEVE, Debug.LOW, "RetrievalMediator offer table to monitor");
            BasicMonitor.newTableFiltered( ret );
        }
        catch (InvalidTableException e)        
        {   // incorrect table, ignore
            /*DBG*/Debug.msg( Debug.RETRIEVE, "RetrievalMediator incorrect table filtered" );
            throw e;
        }
        return ret;
    }


    //////TIMER definitions ///////////
    private static final long BAT_TIMEOUT   = 15000;
    private static final long NIT_TIMEOUT   = 15000;
    private static final long SDT_TIMEOUT   = 3000;
    private static final long EITpf_TIMEOUT = 3000;
    private static final long EITsch_TIMEOUT = 15000;
    private static final long PAT_TIMEOUT   = 1000;
    private static final long PMT_TIMEOUT   = 1000;
    private static final long TDT_TIMEOUT   = 60000;
    private static final long TOT_TIMEOUT   = 60000;

    private static final int  TABLE     = 1;
    private static final int  SINGLE    = 2;
    private static final int  TABLE_SET = 3;
    private static final int  TABLE_EITsch = 4;

    private RetrievalEvent    _event = null;
    private RetrievalListener _list;
    private Retrieve          _retr;
    private int               _filterType;
    private int               _tableType;
}

