/*
* Copyright(c) Philips Consumer Electronics B.V. 1998                         
* All rights reserved. Reproduction in whole or in part is                  
* prohibited without the written consent of the copyright holder.            
*                                                                            
*  Project             : Bali
*  Module              : bali.si 
*  File name   $Logfile: /func_test_platf2/bali/si/scheduler/TableSectionFilterData.java $
*       Last save $Date: 2003/12/24 00:53:03 $
*             $Revision: 1.1.1.1 $
*          Last $Author: mikeseven $
*  Creation Date       : 28/10/98
*  First Author        : S.A.F.A. van den Heuvel
*                        NatLab - Eindhoven
* 
*                                                                          
*  Change history      :
*  ---------------
*
* $Log: Retrieve.java,v $
* Revision 1.1.1.1  2003/12/24 00:53:03  mikeseven
* Initial release of the CVS repository for MPEG-4 Systems code
* after the code was corrupted due to reorganization and human 
* errors.
*
* Revision 1.1  2003/04/03 04:29:23  mikeseven
* Initial release of Player3D and MPEG-J for all MPEG-4 reference softwares players with samples.
* For installation, follow instructions in docs/Player3D_setup.doc.
*
* Many thanks to Prashanth Gujjeti and Viswanathan Swaminathan from Sun Microsystems.
*
* -- Mikael Bourges-Sevenier (Mindego)
*
* Revision 1.1  2002/08/23 16:47:23  pg138550
* Changes for MPEG-J. NOT FINAL
*
* Revision 1.1  2000/06/06 18:43:07  alavilli
* After SISF integration into the CORE4 code
*
* Revision 1.1.2.1  1999/07/09 14:16:50  depetris
* Aggiornati i file per lo sviluppo e il rilascio a MPEG-J.
* Integra la versione 2.5 del core.
*
* Revision 1.4  1999-04-19 15:36:16+02  vdheuvel
* Hook the platform to the TM STB.
*
* Revision 1.3  1999-04-15 09:53:18+02  vdheuvel
* Files changed to hook the system up to the TM demultipler and the
* TM hardware. Changes include, swapping the java.util.Date class
* for the bali.tm.Date class (to prevent using 64 bit floats). Changing
* the HW sect filt used by bali.hw.SectFilt.
*
* Revision 1.2  1999-03-29 14:18:20+02  vdheuvel
* Removed WatchDog import.
*
* Revision 1.1  1999-02-16 14:10:14+01  vdheuvel
* 39 march 1999, start of archive.
* Middleware packages DAVIC API implementation.
*
 * 
 * 1     9-11-98 13:59 Vdheuvel
 * Creation
*/

package bali.retrieve;
import  bali.support.Debug;
import  bali.support.Assert;
import  bali.support.Util;
import  bali.support.ActiveEventSender;
import  bali.support.TimerListener;
import  bali.support.TimerEvent;
import  bali.hw.sectfilt.HwSfEvent;
import  bali.hw.sectfilt.HwSfListener;
import  bali.hw.sectfilt.HwSfController;
import  bali.hw.sectfilt.HwSfData;
import  bali.component.TransportStreamSelection;
import  org.davic.mpeg.dvb.DvbTransportStream;
/*DATE_NORMAL*/ import  java.util.Date;
//DATE_TM*/     import  bali.tm.Date;

/*******************
This HwSfEnhancer translates the HW section filter event model to the retrieve
send model. A time out functionality is added as well storing time of completion
and transport stream source.
Because the section filter needs to be stopped when filtering has been completed, 
the HwSfController has to be indicated to instances of this class before filtering
commences. This class is to be stored in a scheduler and only executed when the
resources become available. To allow this scheduler to only store one object, the
HwSfData is stored allong with other data entries.
This combination of factors is provided by starting the filter process using
the startRetrieve method on this class.
Filtering is stopped and the resource released when: a TIME_OUT occurs and when 
a SUCCESS or any other event is received. The result of the filter action is notified
to the respective RetrieveMediator class.
*/
 class Retrieve
    implements HwSfListener, TimerListener
{
	/**************
	Create a new ActiveRetrieve object. The RetrieveListener is required because the bottom
	RetrieveListener is required to start the section filter.
	@param timeOut The time in milliseconds filtering may continue before a time out occurs.
	@param sfd     The filter settings for the section filter.
	@param rm      The RetrieveMediator to be notified when this retrieve id completed.
	@param list    The RetrieveListener to be indicated to the HwSfController when the section
	               filter is started.
	*/
    Retrieve( long timeOut, HwSfData sfd, RetrievalMediator rm )
	{
        /*DBG*/Debug.msg( Debug.RETRIEVE, Debug.IMPL, "Retrieve ");
        _finished = false;
        _sfd      = sfd;
        _rm       = rm;
        _sf       = null;
        _to       = timeOut;
        _ts       = null;
        _list     = null;
        _monitor  = false;
        // prevent deadlock in RetrieveScheduler when CurrentTS has not been determined yet.
        // in the case of limited section filter resources.
   	    _ts = TransportStreamSelection.getInstance().getCurrentTs();
	}
	
	/**************
	Indicate the filter process has been started. This method may only be called once.
	@param sf The HwSfController to filter with.
	*/
	void startRetrieve( HwSfController sf )
	{
        /*DBG*/Debug.msg( Debug.RETRIEVE, Debug.IMPL, "Retrieve startRetrieve");
    	synchronized ( this )
    	{   
    		if ( !_finished ) // revoked before section filter started
    	    {
        	    /*DBG*/Assert.assert( sf!=null && _sfd!=null );
        	    _sf = sf;
                /*DBG*/Debug.msg( Debug.RETRIEVE, Debug.IMPL, "Retrieve startRetrieve entering SF code");
        	    _sf.startSectionFilter( _sfd );
                /*DBG*/Debug.msg( Debug.RETRIEVE, Debug.IMPL, "Retrieve startRetrieve SF started");
        	    // store current transport stream selection
        	    //  dangers: when current ts is not yet kwown the claiming of
        	    //           resources of the TransportStreamSelection will revoke this resource
        	    _ts = TransportStreamSelection.getInstance().getCurrentTs();
        	    ActiveEventSender.getInstance( ActiveEventSender.HW_SF).setTimer( this, _to, this );
        	}
        }
	}
	
	/**************
	Stop the filter process when active and release the resource.
	*/
	void stopRetrieve()
	{
        /*DBG*/Debug.msg( Debug.RETRIEVE, Debug.IMPL, "Retrieve stopRetrieve");
    	synchronized ( this )
    	{   
    	    /*DBG*/Assert.assert( _sf!=null && _sfd!=null );
            if ( !_finished )
            {
                releaseResourceAndStop();
        	    // timer on same thread as section events prevents timer events
        	    // overtaking section events
        	    ActiveEventSender.getInstance( ActiveEventSender.HW_SF).cancelTimer( this );
    	    }
        }
	}
	
	/*****************
	*/
	void setHwSfListener( HwSfListener list )
	{
	    _list = list;
	}

	/*****************
	*/
	HwSfListener getHwSfListener()
	{
        /*DBG*/Debug.msg( Debug.RETRIEVE, Debug.LOW, "Retrieve. getHwSfListener "+_list );
	    /*DBG*/Assert.assert( _list != null );
	    return _list;
	}

	/*****************
	*/
	DvbTransportStream getTsSource()
	{
        /*DBG*/Debug.msg( Debug.RETRIEVE, Debug.IMPL, "Retrieve getTsSource");
	    return _ts;
	}
	
	/*****************
	*/
	Date getFilterTime()
	{
        /*DBG*/Debug.msg( Debug.RETRIEVE, Debug.IMPL, "Retrieve getFilterTime");
	    return _time;
	}
	
	/////////////////////// IMPLEMENTATION ///////////////////////
	
	/**************
	A new HwSfEvent.
	*/
    public void newHwSfEvent( HwSfEvent e)
    {
        /*DBG*/Debug.msg( Debug.RETRIEVE, Debug.IMPL, "Retrieve newHwSfEvent");
    	synchronized ( this )
    	{   
            /*DBG*/Debug.msg( Debug.RETRIEVE, Debug.LOW, "Retrieve.newHwSfEvent " );
            if ( !_finished )
            {
                releaseResourceAndStop();
        	    _time = new Date();
        	    // timer on same thread as section events prevents timer events
        	    // overtaking section events
        	    ActiveEventSender.getInstance( ActiveEventSender.HW_SF).cancelTimer( this );
                _rm.activeRetrievalResult( e.getResult(), this );
    	    }
        }
    }
    
	/****************
	The time out perid has passed.
	*/
    public void postTimerEvent(TimerEvent event)	
    {
        /*DBG*/Debug.msg( Debug.RETRIEVE, Debug.IMPL, "Retrieve postTimerEvent");
    	synchronized ( this )
    	{   /*DBG*/Debug.msg( Debug.RETRIEVE, Debug.LOW, "Retrieve. time out " );
            if ( !_finished )
            {
                releaseResourceAndStop();
                _rm.activeRetrievalResult( RetrievalEvent.TIME_OUT, this );
        	}
        }
    }
	
	/***************************
	When monitor is set to true, this retrieve is a monitor retrieve
	with lower priority.
	*/
	void setMonitor( boolean monitor )
	{
        /*DBG*/Debug.msg( Debug.RETRIEVE, Debug.IMPL, "Retrieve setMonitor( "+monitor+" )");
	    _monitor= true;
	}

	/***************************
	When monitor is set to true, this retrieve is a monitor retrieve
	with lower priority.
	*/
	boolean isMonitor()
	{
	    return _monitor;
	}
	
	//////////////// PRIVATE ///////////////
    private boolean            _finished;
    private long               _to;
    private HwSfData           _sfd;
    private HwSfController     _sf;
    private RetrievalMediator  _rm;
    private DvbTransportStream _ts;
    private Date               _time;
    private HwSfListener       _list;
    private boolean            _monitor;
    
    
    
    private void releaseResourceAndStop()
    {
        if ( _sf != null )
        {   _sf.release();
        }
   	    _finished = true;
        RetrieveScheduler.getInstance().activeRetrieveReady( this );
    }
}







