/*
* Copyright(c) Philips Consumer Electronics B.V. 1998                         
* All rights reserved. Reproduction in whole or in part is                  
* prohibited without the written consent of the copyright holder.            
*                                                                            
*  Project             : Bali
*  Module              : bali.si 
*  File name   $Logfile: /func_test_platf2/bali/si/scheduler/RetrieveScheduler.java $
*       Last save $Date: 2003/12/24 00:53:03 $
*             $Revision: 1.1.1.1 $
*          Last $Author: mikeseven $
*  Creation Date       : 28/10/98
*  First Author        : S.A.F.A. van den Heuvel
*                        NatLab - Eindhoven
* 
*                                                                          
*  Change history      :
*  ---------------
*
* $Log: RetrieveScheduler.java,v $
* Revision 1.1.1.1  2003/12/24 00:53:03  mikeseven
* Initial release of the CVS repository for MPEG-4 Systems code
* after the code was corrupted due to reorganization and human 
* errors.
*
* Revision 1.1  2003/04/03 04:29:23  mikeseven
* Initial release of Player3D and MPEG-J for all MPEG-4 reference softwares players with samples.
* For installation, follow instructions in docs/Player3D_setup.doc.
*
* Many thanks to Prashanth Gujjeti and Viswanathan Swaminathan from Sun Microsystems.
*
* -- Mikael Bourges-Sevenier (Mindego)
*
* Revision 1.1  2002/08/23 16:47:23  pg138550
* Changes for MPEG-J. NOT FINAL
*
* Revision 1.1  2000/06/06 18:43:08  alavilli
* After SISF integration into the CORE4 code
*
* Revision 1.1.2.1  1999/07/09 14:16:51  depetris
* Aggiornati i file per lo sviluppo e il rilascio a MPEG-J.
* Integra la versione 2.5 del core.
*
* Revision 1.2  1999-03-29 14:18:41+02  vdheuvel
* Removed WatchDog reference.
*
* Revision 1.1  1999-02-16 13:53:36+01  vdheuvel
* 39 march 1999, start of archive.
* Middleware packages DAVIC API implementation.
*
 * 
 * 1     9-11-98 13:59 Vdheuvel
 * Creation
*/

package bali.retrieve;
import  bali.support.Debug;
import  bali.support.ActiveEventSender;
import  bali.hw.sectfilt.HwSfResourceManager;
import  bali.hw.sectfilt.HwSfController;
import  bali.hw.sectfilt.HwSfResourceListener;
import  bali.hw.sectfilt.HwSfResourceEvent;
import  bali.hw.sectfilt.HwSfData;
import  java.util.Vector;
import  java.util.Enumeration;
import  org.davic.mpeg.dvb.DvbTransportStream;

/*********************
This class allows schedulung of the DVB-SI API retrieve actions.
It stores the different retrieve actions and starts them as soon
as the resources become available. When the number of scheduled
tables becomes to large a LACK_OF_RESOURCES event is sent and the
retrieve is dismissed.
*/
class RetrieveScheduler
    implements HwSfResourceListener
{
	/*******************
	Return the RetrieveScheduler.
	*/
	static synchronized  RetrieveScheduler getInstance()
	{
		if ( _instance == null )
		{
    	    /*DBG*/Debug.msg( Debug.RETRIEVE, "RetrieveScheduler created");
			_instance = new RetrieveScheduler();
		}
		return _instance;
	}
	
	/*******************
	Retrieve the a section indicated by the FilterData.
	@param ar The active retrieve to be scheduled.
	*/
	void scheduleRetrieve( Retrieve ar )
	{
	    /*DBG*/Debug.msg( Debug.RETRIEVE,"RetrieveScheduler.scheduleRetrieve");
       	_vecRet.addElement( ar );
		schedule();
	}

	/*******************
	Cancel the indicated retrieve action.
	@param retrieveId The id of the retrieve action.
	*/
    void cancelRetrieve( Retrieve ar )
	{
        /*DBG*/Debug.msg( Debug.RETRIEVE, Debug.IMPL, "RetrieveScheduler cancelRetrieve");
	    synchronized( _vecRet )
    	{  
	        if ( _vecFilter.removeElement( ar ) )
	        {   ar.stopRetrieve();
	        }
    	    _vecRet.removeElement( ar );
    	}
   	    schedule();
	}
	
	////////////// IMPLEMENTATION //////////////////////
	private static RetrieveScheduler _instance = null;
	private static final int MAX_FILTERS = 4; // the maximum number of section filters to be claimed by the retrieveScheduler
	private static final int MAX_ACT_RET = 4; // the maximum number of scheduled active retrieves
    private Vector _vecRet;     // stores the active retrieves
    private Vector _vecFilter;  // the list of currently active filters
	
	/*******************
	A section filter has been claimed or released.
	*/
	RetrieveScheduler()
	{
        /*DBG*/Debug.msg( Debug.RETRIEVE, Debug.IMPL, "RetrieveScheduler ");
	    // create active retrieve vector
	    _vecRet     = new Vector(1);
	    _vecFilter = new Vector( MAX_FILTERS );
		// receive resource status events from section filter
	    HwSfResourceManager.getInstance().addHwSfResourceListener( this );
	}

	/*******************
	A section filter has been claimed or released.
	*/
	public void newHwSfResourceEvent( HwSfResourceEvent e)
	{
        /*DBG*/Debug.msg( Debug.RETRIEVE, Debug.IMPL, "RetrieveScheduler newHwSfResourceEvent");
	    boolean doSchedule = false;
	    doSchedule =  (  e.getResult()==HwSfResourceEvent.RELEASED
	                  && _vecRet.size() < MAX_FILTERS
	                  );   
	    /*DBG*/Debug.msg( Debug.RETRIEVE, Debug.LOW, "RetrieveScheduler newResourceEvent "+doSchedule);
	    if ( doSchedule )
	    {   schedule();
	    }
	}
	

	/*******************
	ActieveRetrieval indicates filter action has finished.
	*/
	void activeRetrieveReady( Retrieve ar )
	{
	    /*DBG*/Debug.msg( Debug.RETRIEVE, "RetrieveScheduler.activeRetrieveReady");
	    synchronized( _vecRet )
    	{  
	        if ( _vecFilter.removeElement( ar ) )
	        {   ar.stopRetrieve();
	        }
    	}
   	    _vecRet.removeElement( ar );
   	    schedule();
	}
	
	/*******************
	*/
	private void schedule()
	{
        /*DBG*/Debug.msg( Debug.RETRIEVE, Debug.IMPL, "RetrieveScheduler schedule");
	    synchronized( _vecRet )
    	{  
            /*DBG*/Debug.msg( Debug.RETRIEVE, "RetrieveScheduler.schedule");
           
    		boolean ready = false;
    		Retrieve ar;
    				
    		while (  !ready 
    		      && _vecFilter.size() < MAX_FILTERS 
    		      )
    		{
    		    ar = getNextRetrieve();
//            	if ( !_vecRet.isEmpty() )
              	if ( ar != null )
            	{   
                    /*DBG*/Debug.msg( Debug.RETRIEVE, Debug.IMPL, "RetrieveScheduler.schedule, retrieve found");
//            	    ar = (Retrieve)_vecRet.firstElement();
                    HwSfController sfc = 
                        HwSfResourceManager.getInstance().
                            claimSectionFilter( HwSfResourceManager.SIAPI, ar.getHwSfListener() );
                    if ( sfc != null )
                    {   // section filter available
                       /*DBG*/Debug.msg( Debug.RETRIEVE, "RetrieveScheduler start "+ar);
                        ar.startRetrieve( sfc );
                        _vecFilter.addElement( ar );
                        _vecRet.removeElement( ar );
                    }
                    else
                    {   // no section filter resource available
                        ready = true;
                    }
    			}
    			else
    			{   // no retrieve available
    			   ready = true;
                }
    		}
    	}
    }
   
    private Retrieve getNextRetrieve()
    {
        Retrieve ret = null;
        Retrieve mon = null;
        synchronized( _vecRet )
        {
            Enumeration e = _vecRet.elements();
            Retrieve    dummy;
            while ( e.hasMoreElements() && ret == null )
            {
                dummy = (Retrieve)e.nextElement();
                if ( dummy.isMonitor() && mon == null )
                {   mon = dummy;
                }
                else
                {   ret = dummy;
                }
            }
        }
        /*DBG*/Debug.msg( Debug.RETRIEVE, Debug.IMPL, "RetrieveScheduler.getNextRetrieve "+ret+" "+mon);
      	return ( ret!=null ? ret: mon );
    }

}


