/*******************************************************************************
* Copyright(c) Philips Consumer Electronics B.V. 1998                         
* All rights reserved. Reproduction in whole or in part is                  
* prohibited without the written consent of the copyright holder.            
*                                                                            
*  Project             : Bali
*  File name   $Logfile: /func_test_platf2/bali/support/ActiveEventSender.java $
*       Last save $Date: 2003/12/24 00:53:03 $
*             $Revision: 1.1.1.1 $
*          Last $Author: mikeseven $
*  Creation Date       : 1/1/98
*  First Author        : H.v.Woerkom
*                        NatLab - Eindhoven
* 
*  Description         : ActiveEventSender
*                                                                          
*  Change history      :
*  ---------------
*
* $Log: ActiveEventSender.java,v $
* Revision 1.1.1.1  2003/12/24 00:53:03  mikeseven
* Initial release of the CVS repository for MPEG-4 Systems code
* after the code was corrupted due to reorganization and human 
* errors.
*
* Revision 1.1  2003/04/03 04:29:24  mikeseven
* Initial release of Player3D and MPEG-J for all MPEG-4 reference softwares players with samples.
* For installation, follow instructions in docs/Player3D_setup.doc.
*
* Many thanks to Prashanth Gujjeti and Viswanathan Swaminathan from Sun Microsystems.
*
* -- Mikael Bourges-Sevenier (Mindego)
*
* Revision 1.1  2002/08/23 16:47:24  pg138550
* Changes for MPEG-J. NOT FINAL
*
* Revision 1.1  2000/06/06 18:43:08  alavilli
* After SISF integration into the CORE4 code
*
* Revision 1.1.2.1  1999/07/09 14:16:52  depetris
* Aggiornati i file per lo sviluppo e il rilascio a MPEG-J.
* Integra la versione 2.5 del core.
*
* Revision 1.4  1999-04-19 15:39:38+02  vdheuvel
* Hook the platform to the TM STB.
*
* Revision 1.3  1999-04-15 13:03:50+02  vdheuvel
* Files changed to hook the system up to the TM demultipler and the
* TM hardware. Changes include, swapping the java.util.Date class
* for the bali.tm.Date class (to prevent using 64 bit floats). Changing
* the HW sect filt used by bali.hw.SectFilt.
*
* Revision 1.2  1999-04-06 14:40:52+02  vdheuvel
* Add tuner API external constant.
*
* Revision 1.1  1999-03-26 14:58:05+01  vdheuvel
* 39 march 1999, start of archive.
* Middleware packages DAVIC API implementation.
*
 * 
 * 1     9-11-98 14:00 Vdheuvel
 * Creation
 * 
 * 2     26-03-98 15:03 Woerkom
 * After renaming operation
 * 
 * 1     26-03-98 14:19 Woerkom
 * 
 * 5     26-03-98 11:34 Woerkom
 * Added ListenerHandlerInterface + postEvent
 * 
 * 4     10-03-98 10:32 Woerkom
 * Stable SI API (passed stress test, although memory management errors in
 * the C runtime library still occur), only EITscheduled not tested.
 * 
 * 3     4-03-98 11:46 Woerkom
 * At this moment, the only bugs seem to be G+4 problems. The
 * ResourceClients still need a revision.
 * 
 * 2     19-02-98 16:16 Woerkom
 * Adapted headers with keywords
*
*******************************************************************************/

package bali.support;
import java.util.EventObject;
import java.util.Vector;
import java.util.Enumeration;
/*DATE_NORMAL*/ import  java.util.Date;
//DATE_TM*/     import  bali.tm.Date;

/**
This class is responsible for sending events according to the Listener
design pattern.
The class uses the ListenerInterfaceAdapter definition to obtain the listeners the
an event has to be sent to.
<p>
This class can also be used to send notifications (object.notifyAll will be
called). This functionality is not needed by any API at this moment.
<p>
The class also allows timer to be set. When the timer period has expired an
event will be sent to the TimerListener registrated with the event.
@see ListenerInterfaceAdapter 
*/
public class ActiveEventSender
	implements Runnable
{

	/** The ActiveEventSender used for application events in the DVB-SI API */
	public final static int SI_APP = 0;

	/** The ActiveEventSender used for application events in the TUNING API */
	public final static int TUNING_APP = 0;

	/** The ActiveEventSender used for internal events in the DVB-SI API */
	public final static int SI_INT = 1;

	/** The ActiveEventSender used for internal events in the section filter API */
	public final static int SF_INT = 2;

	/** The ActiveEventSender used for internal events in the HW section filter API */
	public final static int HW_SF = 2;

	/** The ActiveEventSender used for the watchdog */
	public final static int DEBUG = 3;

	/** The ActiveEventSender used for external resource events of the section filter API */
	public final static int SFAPI_EXT_RES = 4;

	/**	The maximum number of ActiveEventSenders. */ 
	public final static int MAXIMUM_SENDERS = 5;

	/*******************
	Get the indicated ActiveEventSender.
	*/
	public static synchronized ActiveEventSender getInstance(int i)
	{
  	    /*DBG*/Debug.msg( Debug.SUPPORT, Debug.IMPL, "ActiveEventSender getInstance( "+i+" )" );
		if (i < 0 || i >= MAXIMUM_SENDERS)
		{
			return null;
		}
		if (senders == null)
		{
			senders = new ActiveEventSender[MAXIMUM_SENDERS];
		}
		if (senders[i] == null)
		{
    	    /*DBG*/Debug.msg( Debug.SUPPORT, "create ActiveEventSender "+i );
			senders[i] = new ActiveEventSender();
			Thread tr = new Thread(senders[i]);
			tr.setName("AES"+i);
			tr.setDaemon( true );
			if ( i == HW_SF )
			{   tr.setPriority( tr.MAX_PRIORITY );
			}
			tr.start();
		}
  	    /*DBG*/Debug.msg( Debug.SUPPORT, Debug.IMPL, "ActiveEventSender getInstance( "+i+" ) = "+senders[i] );
		return senders[i];
	}

	/*****************
	Send event using the indicated ListenerInterfaceAdapter.
	*/
	public synchronized void sendEvent( ListenerInterfaceAdapter  l, EventObject event )
	{
	    /*DBG*/Debug.msg( Debug.SUPPORT, "ActiveEventSender: Queuing :"+event );
	    /*DBG*/Assert.assert( l!=null && event!=null );
		_listenerQueue.addElement( l );
		_eventQueue.addElement( event );
		notify();
	}

	/*****************
	Send a notifyAll on the indicated object.
	*/
	public synchronized void sendNotification( Object o )
	{
	    /*DBG*/Debug.msg( Debug.SUPPORT, "ActiveEventSender: Queueing notify ("+_notifyQueue.size()+" in)");
		if (!_notifyQueue.contains( o ))
		{
			_notifyQueue.addElement( o );
			notify();
		}
	}

	/***********************
	Set a timer that will send a event to listener when time out period timeOut
	has past.
	@param listener The listener receiving the TimerEvent.
	@param timeOut  The time out period.
	@param timerRef The reference of this timer to used for identification and cancelling (null not allowed).
	*/
	public synchronized void setTimer( TimerListener listener, long timeOut, Object timerRef )
	{
	    /*DBG*/Debug.msg( Debug.SUPPORT, "ActiveEventSender.setTimer :"+timerRef+" , "+timeOut );
		
		Enumeration toe = _timeOutQueue.elements();
		
		Date      timeOutDate = new Date( (new Date()).getTime() + timeOut );
		boolean   ready = false;
		Date      d = null;
		int       index = 0;
		
		while ( toe.hasMoreElements() && !ready )
		{
		   d = (Date)toe.nextElement();
		   ready = timeOutDate.before(d);
		   //DBG System.out.println("ready"+ready );
		   if ( !ready )
		   		index++;
		}
		
		
		//DBG System.out.println("index"+index );
		_timerListenerQueue.insertElementAt( listener, index ) ;
		_timeOutQueue.insertElementAt( timeOutDate, index ) ;
        _timerRefQueue.insertElementAt( timerRef, index ) ;
		
		notify();
		
	}

	/***********************
	Cancel all timers using the indicated timer reference.
	*/
	public synchronized void cancelTimer( Object timerRef )
	{
  	    /*DBG*/Debug.msg( Debug.SUPPORT, Debug.MEDIUM, "ActiveEventSender cancelTimer "+timerRef );
		int       index = 0;

		while ( _timerRefQueue.contains( timerRef ) )
		{
			index = _timerRefQueue.indexOf( timerRef );
			_timerListenerQueue.removeElementAt(  index ) ;
			_timeOutQueue.removeElementAt(  index ) ;
       		_timerRefQueue.removeElementAt(  index ) ;
		}
		notify();
	}

	/*****************
	*/
	public void run()
	{
		boolean Ready = false;

		while ( !Ready )
		{
      	    /*DBG*/Debug.msg( Debug.SUPPORT, Debug.IMPL, "ActiveEventSender loop1" );
			checkTimers();

			while( eventsLeft() )
			{
				processNextEvent();
			}
			
			while( notificationsLeft() )
			{
				processNextNotification();
			}
			
			checkTimers();
			long st = getSleepTime();
			synchronized ( this )
			{
				// a (eventor notify) notify could have occured just before 
				// the synchronized statement and we wouldn't know (and wait for the next
				// notify)! So we check here.
				if ( !eventsLeft() && !notificationsLeft() )
				{
					try
					{
						if ( st < 0 )
						{	wait( );
						}
						else
						{	wait( st );
						}
					}
					catch( java.lang.InterruptedException e )
					{}
				}
			}
		}
	}
	
	/*******************
	*/
	ActiveEventSender()
	{
		_listenerQueue = new Vector();
		_eventQueue    = new Vector();
		_notifyQueue   = new Vector();
        _timerListenerQueue = new Vector();
        _timerRefQueue      = new Vector();
		_timeOutQueue       = new Vector();
	}
	
	///////////////////// PRIVATE //////////////////////////////
	// event 
	private Vector	_listenerQueue;
	private Vector  _eventQueue;
	// notifications
	private Vector  _notifyQueue;
	// timers
	private Vector  _timerListenerQueue;
	private Vector  _timerRefQueue;
	private Vector  _timeOutQueue;
	private static int _timerId = Integer.MIN_VALUE;
	private static ActiveEventSender senders[] = null;

	
	/***************
	Send next event from the timer queue.
	*/
	private void sendFirstTimerEvent()
	{
		TimerListener l    =null;
		TimerEvent    event=null;

   	    /*DBG*/Debug.msg( Debug.SUPPORT, Debug.IMPL, "ActiveEventSender sendFirstTimerEvent" );
		synchronized( this )
		{
			l = (TimerListener)_timerListenerQueue.firstElement();
			_timerListenerQueue.removeElementAt( 0 );
			
			event = new TimerEvent( this, _timerRefQueue.firstElement() );
			_timerRefQueue.removeElementAt( 0 );
			
			_timeOutQueue.removeElementAt( 0 );

			
		}
        /*DBG*/Debug.msg( Debug.SUPPORT, "ActiveEventSender. send Timer event to "+l+" at"+new Date() );
    	sendEvent( new TimerListenerInterfaceAdapter( l ), event );
	}
	
	/*******************
	Check if there are events to be send.
	*/
	private synchronized void checkTimers()
	{
   	    /*DBG*/Debug.msg( Debug.SUPPORT, Debug.IMPL, "ActiveEventSender checkTimers" );
		Date d = new Date();
		Date first;
		boolean ready = false;
		
		//DBG*/ Debug.msg("checkTimers at : "+new Date() );

		while ( !ready )
		{
			if ( !_timeOutQueue.isEmpty() )
			{ 				
				first = (Date)_timeOutQueue.firstElement();
				if ( d.after( first ) )
				{	sendFirstTimerEvent();
				}
				else
				{	ready = true;
				}
				d = new Date();
			}
			else
			{	ready = true;
			}
		}
	}

	/*******************
	Calculate the time until the next time out.
	*/
	private synchronized long getSleepTime()
	{
		long res = -1;
		
		if ( !_timeOutQueue.isEmpty() )
		{
			long now = (new Date()).getTime();
			res = ((Date)_timeOutQueue.firstElement()).getTime() - now;
            /*DBG*/Debug.msg( Debug.SUPPORT, Debug.LOW, "ActiveEventSender getSleepTime : "+res );
			if ( res <= 0 )
			{	// Timer to has fired.
                sendFirstTimerEvent();
				res = getSleepTime();
			}
		}
		return res;
	}


	/*******************
	*/
	private void processNextEvent()
	{
	    /*DBG*/Debug.msg( Debug.SUPPORT, "ActiveEventSender.processNextEvent" );
		ListenerInterfaceAdapter l=null;
		EventObject event=null;
	
		synchronized( this )
		{
			l = (ListenerInterfaceAdapter)_listenerQueue.firstElement();
			_listenerQueue.removeElement( l );
			event = (EventObject)_eventQueue.firstElement();
			_eventQueue.removeElement( event );
		}
	    /*DBG*/Debug.msg( Debug.SUPPORT, "ActiveEventSender.processNextEvent sending Event "+event );
		l.notifyListeners( event );
	    /*DBG*/Debug.msg( Debug.SUPPORT, "ActiveEventSender.processNextEvent sending Event ready" );
	}
	
	/*******************
	*/
	private void processNextNotification()
	{
		Object o = null;
		int x;
		
		synchronized( this )
		{
			o = _notifyQueue.firstElement();
			_notifyQueue.removeElement( o );
			x = _notifyQueue.size();
		}
	    /*DBG*/Debug.msg( Debug.SUPPORT, "ActiveEventSender.Notifying ("+x+" left)");
		synchronized( o )
		{
			o.notifyAll();
		}
	}

	/*******************
	*/
	private synchronized boolean eventsLeft()
	{
		return !_listenerQueue.isEmpty();
	}

	/*******************
	*/
	private synchronized boolean notificationsLeft()
	{
		return !_notifyQueue.isEmpty();
	}


}
