/*******************************************************************************
* Copyright(c) Philips Consumer Electronics B.V. 1998                         
* All rights reserved. Reproduction in whole or in part is                  
* prohibited without the written consent of the copyright holder.            
*                                                                            
*  Project             : Bali
*  File name   $Logfile: $
*       Last save $Date: 2003/12/24 00:53:04 $
*             $Revision: 1.1.1.1 $
*          Last $Author: mikeseven $
*  Creation Date       : 15-11-1998
*  First Author        : Bas van den Heuvel
*                        NatLab - Eindhoven
* 
*  Change history      :
*  ---------------
*
* $Log: Debug.java,v $
* Revision 1.1.1.1  2003/12/24 00:53:04  mikeseven
* Initial release of the CVS repository for MPEG-4 Systems code
* after the code was corrupted due to reorganization and human 
* errors.
*
* Revision 1.1  2003/04/03 04:29:24  mikeseven
* Initial release of Player3D and MPEG-J for all MPEG-4 reference softwares players with samples.
* For installation, follow instructions in docs/Player3D_setup.doc.
*
* Many thanks to Prashanth Gujjeti and Viswanathan Swaminathan from Sun Microsystems.
*
* -- Mikael Bourges-Sevenier (Mindego)
*
* Revision 1.1  2002/08/23 16:47:24  pg138550
* Changes for MPEG-J. NOT FINAL
*
* Revision 1.1  2000/06/06 18:43:09  alavilli
* After SISF integration into the CORE4 code
*
* Revision 1.1.2.1  1999/07/09 14:16:53  depetris
* Aggiornati i file per lo sviluppo e il rilascio a MPEG-J.
* Integra la versione 2.5 del core.
*
* Revision 1.1  1999-02-23 09:00:16+01  vdheuvel
* 39 march 1999, start of archive.
* Middleware packages DAVIC API implementation.
*
*
*******************************************************************************/

package bali.support;

//SIA
import java.io.*;
import java.awt.*;
//End SIA

/**
This class prints debug messages.
*/
public class Debug
{
    // The constants defining the different debug units.
    // when a constant is added this should also be added to the switch statement
    // in the C++ implementation of the native method.
   
    // The following constants represent debug units that should only be used
    public  final static int SI      = 1;
    public  final static int SF      = 2;  // section filter API
    public  final static int HWSF    = 3;  // HWSF abstraction
    public  final static int SUPPORT = 4;
    public  final static int COMP    = 5;  // Component module
    public  final static int TABLE   = 6;  // Table definition module
    public  final static int RETRIEVE= 7;  // Retrieve abstraction module
    public  final static int CACHE   = 8;  // Cache module
    public  final static int SOMSF   = 9;  // sommit hardware sf interface
    public  final static int WD      = 10; // watchdog
    public  final static int HWSF_LE = 11; // Hardware section filter listener enhancers
    public  final static int TUNING  = 12; // Tuning API
    public  final static int MPEG    = 13; // MPEG component API
    public  final static int MONITOR = 14; // MPEG component API
    private final static int LAST    = 15;
    //levels
    public final static int ERROR = 5;
    public final static int HIGH  = 4;
    public final static int MEDIUM= 3;
    public final static int LOW   = 2;
    public final static int IMPL  = 1;
    

//SIA
// out: it represents the output file stream in wich the debug messages wiil be written
	static FileOutputStream out;

// This block initializes the out file
	static{
	  try {
  		out = new FileOutputStream("C:\\temp\\LogDebug.txt");
	   } catch (java.io.IOException ex) {
		Frame f = new Frame("IO EXCEPTION");
		f.resize(100,100);
		f.show();
	   }
	}

	Debug(){}
//End SIA

    /**
    Prints a debug message using the G+4 debug mechanism. The message is printed
    using the indicated debug message and level.
    @param dbgUnit  The debug unit the message is sent on.
    @param dbgLevel The level of the debug message (0<=dbgLevel<=5).
    @param msg   The message to be printed.
    */
    public static synchronized void msg(int dbgUnit, int dbgLvl, String msg)
    {
        init();
        if ( dbgLvl >= _arDbgLvl[dbgUnit]  )
        {   
            if ( _printThread )
            {
                System.out.println( Thread.currentThread()+" DBG: "+dbgUnit+", lvl: "+dbgLvl+" : "+msg);
//SIA
	  	try {
			String message = new String(Thread.currentThread()+"DBG: "+dbgUnit+" , lvl: "+dbgLvl+" : " + msg  +"\n");
			out.write(message.getBytes());
	   	} catch (java.io.IOException ex) {
			Frame f = new Frame("IO EXCEPTION");
			f.resize(100,100);
			f.show();
	   	}
///End SIA
            }
            else
            {
                System.out.println( " DBG: "+dbgUnit+", lvl: "+dbgLvl+" : "+msg);
//SIA
	  	try {
			String message = new String("DBG: "+dbgUnit+" , lvl: "+dbgLvl+" : " + msg  +"\n");
			out.write(message.getBytes());
			out.flush();
	   	} catch (java.io.IOException ex) {
			Frame f = new Frame("IO EXCEPTION");
			f.resize(100,100);
			f.show();
	   	}
///End SIA
            }
        }
    }

    /**
    Prints a debug message using the debug mechanism. The message is printed
    using the indicated debug message and NORMAL level.
    @param dbgUnit  The debug unit the message is sent on.
    @param msg   The message to be printed.
    */
    public static synchronized void msg(int dbgUnit, String msg)
    {
        init();
        msg( dbgUnit, MEDIUM, msg);
    }
    
    /**
    Sets the indicated DbgUnit to only print messages of a lvl higher or equal than
    indicated.
    @param dbgUnit  The debug uni
    @param lvl      The  minimal message lvl
    */
    public static synchronized void setDbgLvl(int dbgUnit, int lvl )
    {
        init();
        Assert.assert( dbgUnit>0 );
        Assert.assert( dbgUnit<LAST );
        _arDbgLvl[dbgUnit] = lvl;
    }
    
    /**
    When PrintThread is set to true, the Thread printing the message is included in the message.
    */
    public static synchronized void setPrintThread( boolean p )
    {
        init();
        _printThread = p;
    }
    
    ///////////////// IMPLEMENTATION /////////////
    private static int _arDbgLvl[] =null;
    private static Object _lock = new Object();
    private static boolean _printThread;

    private static void init()
    {   
        synchronized( _lock )
        {
            if ( _arDbgLvl == null )
            {
                _arDbgLvl = new int[LAST];
                _printThread = false;
                for ( int i=0; i<LAST; i++ )
                {   _arDbgLvl[i] = LAST;
                }
            }
        }
    }
    
}