/*******************************************************************************
* Copyright(c) Philips Consumer Electronics B.V. 1998                         
* All rights reserved. Reproduction in whole or in part is                  
* prohibited without the written consent of the copyright holder.            
*                                                                            
*  Project             : Bali
*  Module              : bali (support package)
*  File name   $Logfile: /func_test_platf2/bali/support/ListenerHandler.java $
*       Last save $Date: 2003/12/24 00:53:04 $
*             $Revision: 1.1.1.1 $
*          Last $Author: mikeseven $
*  Creation Date       : 1/1/98
*  First Author        : H.v.Woerkom
*                        NatLab - Eindhoven
* 
*  Description         : ListenerHandler
*                                                                          
*  Change history      :
*  ---------------
*
* $Log: ListenerHandler.java,v $
* Revision 1.1.1.1  2003/12/24 00:53:04  mikeseven
* Initial release of the CVS repository for MPEG-4 Systems code
* after the code was corrupted due to reorganization and human 
* errors.
*
* Revision 1.1  2003/04/03 04:29:24  mikeseven
* Initial release of Player3D and MPEG-J for all MPEG-4 reference softwares players with samples.
* For installation, follow instructions in docs/Player3D_setup.doc.
*
* Many thanks to Prashanth Gujjeti and Viswanathan Swaminathan from Sun Microsystems.
*
* -- Mikael Bourges-Sevenier (Mindego)
*
* Revision 1.1  2002/08/23 16:47:24  pg138550
* Changes for MPEG-J. NOT FINAL
*
* Revision 1.1  2000/06/06 18:43:09  alavilli
* After SISF integration into the CORE4 code
*
* Revision 1.1.2.1  1999/07/09 14:16:53  depetris
* Aggiornati i file per lo sviluppo e il rilascio a MPEG-J.
* Integra la versione 2.5 del core.
*
* Revision 1.1  1998-12-09 17:26:50+01  vdheuvel
* 39 march 1999, start of archive.
* Middleware packages DAVIC API implementation.
*
 * 
 * 1     9-11-98 14:00 Vdheuvel
 * Creation
 * 
 * 7     26-03-98 15:03 Woerkom
 * After renaming operation
 * 
 * 6     26-03-98 14:19 Woerkom
 * 
 * 5     26-03-98 11:34 Woerkom
 * Added ListenerHandlerInterface + postEvent
 * 
 * 4     19-02-98 16:16 Woerkom
 * Adapted headers with keywords
*
*******************************************************************************/

package bali.support;

import java.util.Vector;
import java.util.EventListener;
import java.util.EventObject;

/******************************
Utility object to handle the Listeners used for a SITable update
*/

public abstract class ListenerHandler
	implements ListenerInterfaceAdapter
{
	public ListenerHandler()
	{
		_listeners = new Vector();
	}
	
	public synchronized boolean isEmpty()
	{
		return _listeners.isEmpty() ;
	}

	public synchronized int size()
	{
		return _listeners.size();
	}

	public synchronized void addListener( EventListener l )
	{
		_listeners.addElement( l );
	}
	
	public synchronized boolean removeListener( EventListener l )
	{
		return _listeners.removeElement( l );
	}
	
	public synchronized void removeListeners()
	{
		_listeners.removeAllElements( );
	}

// We now provide an (empty) implementation that can be overridden.
// Ideally only postEvent is overridden, but notifyListeners could
// also be overridden.
	public void notifyListeners( EventObject event )
	{
		EventListener listeners[];

		synchronized (this) {
			listeners = new EventListener[_listeners.size()];
			_listeners.copyInto( (Object[]) listeners );
		}
		for (int i = 0; i < listeners.length; i++) {
			postEvent( listeners[i], (EventObject) event);
		}
	}
	
	public void postEvent( EventListener listener, EventObject event )
	{
		// do the equivalent of listener.post(event)
	}
	
	/////////////// PRIVATE //////////////////
	protected Vector	_listeners;
}