/*
* Copyright(c) Philips Consumer Electronics B.V. 1998                         
* All rights reserved. Reproduction in whole or in part is                  
* prohibited without the written consent of the copyright holder.            
*                                                                            
*  Project             : Bali
*  File name   $Logfile: /func_test_platf2/bali/si/table/TablePart.java $
*       Last save $Date: 2003/12/24 00:53:04 $
*             $Revision: 1.1.1.1 $
*          Last $Author: mikeseven $
*  Creation Date       : 28/10/98
*  First Author        : S.A.F.A. van den Heuvel
*                        NatLab - Eindhoven
* 
*                                                                          
*  Change history      :
*  ---------------
*
* $Log: TablePart.java,v $
* Revision 1.1.1.1  2003/12/24 00:53:04  mikeseven
* Initial release of the CVS repository for MPEG-4 Systems code
* after the code was corrupted due to reorganization and human 
* errors.
*
* Revision 1.1  2003/04/03 04:29:24  mikeseven
* Initial release of Player3D and MPEG-J for all MPEG-4 reference softwares players with samples.
* For installation, follow instructions in docs/Player3D_setup.doc.
*
* Many thanks to Prashanth Gujjeti and Viswanathan Swaminathan from Sun Microsystems.
*
* -- Mikael Bourges-Sevenier (Mindego)
*
* Revision 1.1  2002/08/23 16:47:24  pg138550
* Changes for MPEG-J. NOT FINAL
*
* Revision 1.1  2000/06/06 18:43:09  alavilli
* After SISF integration into the CORE4 code
*
* Revision 1.1.2.1  1999/07/09 14:16:55  depetris
* Aggiornati i file per lo sviluppo e il rilascio a MPEG-J.
* Integra la versione 2.5 del core.
*
* Revision 1.3  1999-04-26 14:53:26+02  vdheuvel
* When a TablePart error occurs a IndexOutOfBoundsException is
* thrown instead of the SeriousError.
*
* Revision 1.2  1999-04-15 09:40:42+02  vdheuvel
* Files changed to hook the system up to the TM demultipler and the
* TM hardware. Changes include, swapping the java.util.Date class
* for the bali.tm.Date class (to prevent using 64 bit floats). Changing
* the HW sect filt used by bali.hw.SectFilt.
*
* Revision 1.1  1998-12-08 09:53:14+01  vdheuvel
* 39 march 1999, start of archive.
* Middleware packages DAVIC API implementation.
*
 * 
 * 2     10-11-98 18:06 Vdheuvel
 * Changed getUTCTime to use the jdk1.1 Calendar class.
 * 
 * 1     9-11-98 13:59 Vdheuvel
 * Creation
*/

package bali.support;
/*DATE_NORMAL*/ import java.util.TimeZone;
/*DATE_NORMAL*/ import java.util.Calendar;
/*DATE_NORMAL*/ import  java.util.Date;
//DATE_TM*/     import  bali.tm.Date;

/**************************
This class allows an interpreter to access a part of an array as
a seperate part.
*/
public class TablePart
{
    /********************
    Create a new TablePart using another TablePart.
    */
    public TablePart( TablePart tp )
    {
        _ar = tp._ar;
        _start = tp._start;
        _length = tp._length;
		internalCheck();
    }

   /********************
    Create a new TablePart using a part of another TablePart.
    */
    public TablePart( TablePart tp, int start, int length )
    {
        _ar = tp._ar;
        _start = tp._start+start;
        _length = length;
		internalCheck();
    }

    /********************
    Create a new TablePart using the indicated array.
    */
    public TablePart( byte[] ar )
    {
        _ar = ar;
        _start =0;
        _length = ar.length;
    }

    /********************
    Create a new TablePart using a part of the indicated array.
    */
    public TablePart( byte[] ar, int start, int length )
    {
        _ar = ar;
        _start = start;
        _length = length;
		internalCheck();
    }

	static private char hexByte[] = { '0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F' };
	
    /********************
    Check if this is a valid TablePart, if not valid, SeriousError/IndexOutOfBoundsException is thrown.
    TBD may change SeriousError/IndexOutOfBoundsException to an Exception
    */
	private void internalCheck()
	{
		if (_start > _ar.length) {
			System.out.print( "TablePart: Illegal part, start > array length, start = " + _start + " length = "
					+ _length + " ar[" + _ar.length + "] =" );
			for (int i = 0; i < _ar.length; i++) {
				System.out.print( " " + hexByte[(_ar[i] >> 4) & 0x0f] + hexByte[_ar[i] & 0x0f] );
			}
			System.out.println( "" );
//			throw new SeriousError( "TablePart" );
    		throw new IndexOutOfBoundsException( "TablePart" );
		} else if (_start + _length > _ar.length) {
			System.out.print( "TablePart: Illegal part, start + length > array length, start = " + _start + " length = "
					+ _length + " ar[" + _ar.length + "] =" );
			for (int i = 0; i < _ar.length; i++) {
				System.out.print( " " + hexByte[(_ar[i] >> 4) & 0x0f] + hexByte[_ar[i] & 0x0f] );
			}
			System.out.println( "" );
//			throw new SeriousError( "TablePart" );
    		throw new IndexOutOfBoundsException( "TablePart" );
		}
	}
	
    /********************
    Return the length of this table part.
    */
    public int getLength()
    {
        return _length;
    }

    /********************
    Return the byte at the indicated index.
    */
    public byte getByte( int index )
    {
        return _ar[ _start+index ];
    }

    /********************
    Return subarray of this table part starting at index and length bytes long.
    */
    public byte[] getBytes( int index, int length )
    {
        byte[] res = new byte[ length ];
        System.arraycopy( _ar, _start+index, res, 0, length );
        return res;
    }

    /********************
    Return an unsigned value of 8 bits on the indicated index.
    */
    public short get8bits( int index )
    {
        return (short)(getByte( index )& 0x00ff);
    }

    /********************
    Return an unsigned value of 12 bits on the indicated index.
    */
    public int get12bits( int index )
    {
        int res;

        res  = get8bits( index ) << 8;
        res |= get8bits( index+1 );
        res &= 0x0FFF ;
        
        return res;
    }

    /********************
    Return an unsigned value of 13 bits on the indicated index, the 5 most significant 
    bits are ignored.
    */
    public int get13bits( int index )
    {
        int res;

        res  = get8bits( index ) << 8;
        res |= get8bits( index+1 );
        res &= 0x1FFF ;
        
        return res;
    }

    /********************
    Return an unsigned value of 16 bits on the indicated index.
    */
    public int get16bits( int index )
    {
        int res;

        res  = get8bits( index ) << 8;
        res |= get8bits( index+1 );
        
        return res;
    }

    /********************
    Return an unsigned value of 24 bits on the indicated index.
    */
    public int get24bits( int index )
    {
        int res;

		res  = get16bits( index ) <<8;
        res |= get8bits( index+2 );
        
        return res;
    }

    /********************
    Return a Date object holding the UTC time coded from index and forward.
    */
	public Date getUTCTime( int index )
	{
	    //DBG*/ for (int i=0;i<5;i++)
	    //DBG*/   System.out.println(" "+ i +" " +Integer.toHexString( get8bits( index+i ) ));

		int m,y,d;
		int mjd = get16bits( index );
		// calculate year, month and day
		{
			int yy  = (int)( (mjd - 15078.2) /365.25);
			int mm  = (int)( (mjd - 14956.1 - (int)(yy*365.25) )/30.6001 );
			    d   = mjd - 14956 - (int)(yy*365.25) - (int)(mm*30.6001);
			if ( mm==14 || mm==15 )
			{
				y = yy+1;
				m = mm -1 - 12;
			}
			else
			{
				y = yy;
				m = mm - 1 ;
			}
		}
		//DBG*/ System.out.println("  year "+y+" month "+m+" day "+d );
	    int h,mi,s;
		// calculate houre, minutes and seconds
		{
		    short val;
		    val = get8bits( index+2 );
			h   = ( ((val&0xF0)>>4)*10 )
			    + (val&0x0F);	// hours (MSB)
		    val = get8bits( index+3 );
			mi  = ( ((val&0xF0)>>4)*10 )
			    + (val&0x0F);	// minutes (MSB)
		    val = get8bits( index+4 );
			s   = ( ((val&0xF0)>>4)*10 )
			    + (val&0x0F);	// seconds (MSB)
		}
		//DBG*/ System.out.println("  houres "+h+" min "+mi+" sec "+s );

/*DATE_NORMAL*/ Calendar cal = Calendar.getInstance( TimeZone.getTimeZone("UTC") );
/*DATE_NORMAL*/ cal.clear();
/*DATE_NORMAL*/ cal.set( 1900+y,m,d,h,mi,s);
/*DATE_NORMAL*/ return ( cal.getTime() );

//DATE_TM*/     return new Date( 1900+y,m,d,h,mi,s);

	}
	
	/****************************
	Returns the BCDTime at the given index in milli seconds
	*/
	public long	getBCDTime( int index )
	{
	    int h,mi,s;
		// calculate houre, minutes and seconds
		{
		    short val;
		    val = get8bits( index );
			h   = ( ((val&0xF0)>>4)*10 )
			    + (val&0x0F);	// hours (MSB)
		    val = get8bits( index+1 );
			mi  = ( ((val&0xF0)>>4)*10 )
			    + (val&0x0F);	// minutes (MSB)
		    val = get8bits( index+2 );
			s   = ( ((val&0xF0)>>4)*10 )
			    + (val&0x0F);	// seconds (MSB)
		}
		//DBG*/ System.out.println("  houres "+h+" min "+mi+" sec "+s );
		return( ((h*60 + mi)*60 + s)*1000 );
	}


    /********************
    True when the two tables hold the same data.
    */
	public boolean isEqual( TablePart tp )
	{
		boolean res = true;
		
		if ( tp != this )
		{
			res = ( tp._length == _length );
		
			int i=0;
			while ( res && i < _length && res )
			{
				res = ( tp._ar[i] == _ar[i] );
				i++;
			}
		}
		return res;
	}

    ////////////////// PRIVATE //////////////////////
    protected byte[] _ar;
    protected int    _start;
    protected int    _length;
}
