/*
* Copyright(c) Philips Consumer Electronics B.V. 1998                         
* All rights reserved. Reproduction in whole or in part is                  
* prohibited without the written consent of the copyright holder.            
*                                                                            
*  Project             : Bali
*  Module              : bali 
*  File name   $Logfile: /func_test_platf2/bali/si/table/BAT.java $
*       Last save $Date: 2003/12/24 00:53:04 $
*             $Revision: 1.1.1.1 $
*          Last $Author: mikeseven $
*  Creation Date       : 28/10/98
*  First Author        : S.A.F.A. van den Heuvel
*                        NatLab - Eindhoven
* 
*                                                                          
*  Change history      :
*  ---------------
*
* $Log: BAT.java,v $
* Revision 1.1.1.1  2003/12/24 00:53:04  mikeseven
* Initial release of the CVS repository for MPEG-4 Systems code
* after the code was corrupted due to reorganization and human 
* errors.
*
* Revision 1.1  2003/04/03 04:29:25  mikeseven
* Initial release of Player3D and MPEG-J for all MPEG-4 reference softwares players with samples.
* For installation, follow instructions in docs/Player3D_setup.doc.
*
* Many thanks to Prashanth Gujjeti and Viswanathan Swaminathan from Sun Microsystems.
*
* -- Mikael Bourges-Sevenier (Mindego)
*
* Revision 1.1  2002/08/23 16:47:24  pg138550
* Changes for MPEG-J. NOT FINAL
*
* Revision 1.1  2000/06/06 18:43:10  alavilli
* After SISF integration into the CORE4 code
*
* Revision 1.1.2.1  1999/07/09 14:16:57  depetris
* Aggiornati i file per lo sviluppo e il rilascio a MPEG-J.
* Integra la versione 2.5 del core.
*
* Revision 1.3  1999-05-05 14:26:35+02  vdheuvel
* Changes required after first Bouquet tests
*
* Revision 1.2  1999-04-15 09:53:18+02  vdheuvel
* Files changed to hook the system up to the TM demultipler and the
* TM hardware. Changes include, swapping the java.util.Date class
* for the bali.tm.Date class (to prevent using 64 bit floats). Changing
* the HW sect filt used by bali.hw.SectFilt.
*
* Revision 1.1  1999-02-08 17:19:24+01  vdheuvel
* 39 march 1999, start of archive.
* Middleware packages DAVIC API implementation.
*
 * 
 * 1     10-11-98 9:49 Vdheuvel
 * Table representation of a Bouquet Association Table
 * 
 * 1     9-11-98 13:59 Vdheuvel
 * Creation
*/
package bali.table;

import java.util.Vector;
import java.util.Enumeration;
/*DATE_NORMAL*/ import  java.util.Date;
//DATE_TM*/     import  bali.tm.Date;
import bali.hw.sectfilt.HwSection;
import bali.support.Debug;
import bali.support.TablePart;
import org.davic.mpeg.dvb.DvbTransportStream;

/****************************************
This class parses the filtered BAT tables and allows access to the different BAT components.
*/
public class BAT
    extends SITable
{
	/** */ public final static int PID = 0x11; 
	/** */ public final static int TABLE_ID = 0x4A; 
	/** */ public final static int BOUQUET_NAME_DESCRIPTOR_TAG  = 0x47; 
	/** */ public final static int SERVICE_LIST_DESCRIPTOR_TAG  = 0x41; 

	/***************************************
	*/
	public BAT( BAT table, DvbTransportStream ts, Date filterTime )
	{
		super( table
			 , new SITableInfo( table._ti.getVersion(), table._ti.fromActual(), ts, filterTime )
             );
        _descriptorList = table._descriptorList;
        _TSList         = table._TSList;
        _updateTime     = table._updateTime;
        _bouquetName    = table._bouquetName;
	}

	/***************************************
	*/
    public BAT( HwSection[] arSec, DvbTransportStream ts, Date filterTime )
        throws InvalidTableException
    {
    	super( arSec
    	     , new SITableInfo( arSec[0].version_number(), true, ts, filterTime )
    	     );
    	try     
    	{     
            /*DBG*/ Debug.msg( Debug.TABLE, "parse BAT");
            
    		// data stored in an array of TablePart
            
            _TSList = new Vector();
    	    _descriptorList = new DescriptorList();
    
    		// parse all sections of this table.
    		for ( int i=0; i<_arSec.length; i++ )
    		{	// parse section
    	    	int descLen;
    
    	        descLen = _arSec[i].get12bits( 8 );
    	        _descriptorList = new DescriptorList( _descriptorList, new TablePart( _arSec[i], 10, descLen ) );
    
    			int   tsLength = _arSec[i].get12bits( descLen+10 );
    	        int   tl;
        	    int   index = descLen+12;
    
    
    	        while ( index < tsLength+descLen+10 )
        	    {
            	    tl = 6 + _arSec[i].get12bits( index + 4 );
    	            _TSList.addElement( new TS( new TablePart( _arSec[i], index, tl ), _ti ) );
    	            index += tl;
        	    }
        	    
        	    parseStrings();
            }// end parsing sections
        } 
        catch( IndexOutOfBoundsException e )
        {
            throw new InvalidTableException( );
        }    
    }

    /*****************
    Returns the TableIdentifier of this BAT.
    */
    public TableIdentifier getTableIdentifier()
    {
        synchronized( this )
        {
            if ( _tableIdentifier == null )
            {
                int[] arInt = new int[1];
                arInt[0] = getBouquetId();
                _tableIdentifier = new TableIdentifier( TableIdentifier.BAT, arInt );
            }
        }
        return _tableIdentifier;
    }


	/***************************************
	Return the descriptors of the main loop of the NIT table.
	*/
    public DescriptorList getDescriptorList()
    {
		return _descriptorList;
    }
    
	/***************************************
	Return the network_id of this NIT.
	*/
    public int getBouquetId()
    {
        return _arSec[0].table_id_extension();
   }

	/***************************************
	Return the network_name of this NIT ("" of none existend).
	*/
    public String getBouquetName()
    {
        return _bouquetName.getString();
    }

	/***************************************
	Return the short network_name of this NIT ("" of none existend).
	*/
    public String getShortBouquetName()
    {
        return _bouquetName.getShortString();
    }

	/***************************************
	Return a list of TableIdentifiers identifying the services indicated by
	this BAT
	*/
	public TableIdentifier[] getServices()
	{
	    Vector      vec = new Vector(0,1);
		Enumeration e   = _TSList.elements();
        Descriptor  d; 
        int         dLength;
        int         id[]= new int[3];
		TS          ts  = null;
		while ( e.hasMoreElements() )
		{
		    ts = (TS)e.nextElement();
            d  = ts.getDescriptorList().getDescriptor( SERVICE_LIST_DESCRIPTOR_TAG );
            if ( d!= null )
            {   // descriptor found
                id[0] = ts.getOriginalNetworkId();
                id[1] = ts.getTransportStreamId();
                dLength = d.get8bits( 1 );
                int i=0;
                while ( i<dLength )
                {
                    id[2] = d.get16bits( 2 + i*3 );
                    vec.addElement( new TableIdentifier( TableIdentifier.SERVICE, id ) );
                    i+=3;
                }
            }
		}
		TableIdentifier[] res = new TableIdentifier[ vec.size() ];
		vec.copyInto( res );
		return res;
	}
	
	/***************************************
	Return the TransportStreams in this NIT
	*/
	public TS[] getTSs()
	{
		TS[] ar = new TS[ _TSList.size() ];
		_TSList.copyInto( ar );
		return ar;
	}
	
	/***************************************
	Return the TransportStream in this NIT, null when not present
	*/
	public TS getTS( int or_nw_id, int ts_id )
	{
		Enumeration e = _TSList.elements();
		TS ts = null;
		boolean found = false;
		while ( e.hasMoreElements() && !found )
		{
			ts = (TS)e.nextElement();
			if (  ts_id    == ts.getTransportStreamId() 
			   && or_nw_id == ts.getOriginalNetworkId() )
			{
				found = true;
			}
		}
		if ( found )
			return ts;
		else
			return null;
	}


	/***************************************
	Return the number of TransportStreams in this NIT
	*/
    public int getNumberOfTransportStreams()
	{
		return _TSList.size();
	}

	
	/////////////////////////////// PRIVATE //////////////////////////
	private DescriptorList 	_descriptorList;
	private Vector 			_TSList;
	private Date			_updateTime;
	private DvbString       _bouquetName;

	/************************
	*/
	private void parseStrings()
	{
        //DBG*/ System.out.println("Serv.parseStrings" );
        Descriptor d; 
        d = _descriptorList.getDescriptor( BOUQUET_NAME_DESCRIPTOR_TAG );

        if ( d != null )
        {
			int start = 2;
			int end   = start  + d.get8bits( 1 );

            //DBG*/ System.out.println("   start"+start+" end: "+end );
			
			_bouquetName = new DvbString( d.getContent(), start, end );
		}                
		else
		{
			_bouquetName = new DvbString();
		}
	}


}
