/*
* Copyright(c) Philips Consumer Electronics B.V. 1998                         
* All rights reserved. Reproduction in whole or in part is                  
* prohibited without the written consent of the copyright holder.            
*                                                                            
*  Project             : Bali
*  File name   $Logfile: /func_test_platf2/bali/si/table/DescriptorList.java $
*       Last save $Date: 2003/12/24 00:53:04 $
*             $Revision: 1.1.1.1 $
*          Last $Author: mikeseven $
*  Creation Date       : 28/10/98
*  First Author        : S.A.F.A. van den Heuvel
*                        NatLab - Eindhoven
* 
*                                                                          
*  Change history      :
*  ---------------
*
* $Log: DescriptorList.java,v $
* Revision 1.1.1.1  2003/12/24 00:53:04  mikeseven
* Initial release of the CVS repository for MPEG-4 Systems code
* after the code was corrupted due to reorganization and human 
* errors.
*
* Revision 1.1  2003/04/03 04:29:25  mikeseven
* Initial release of Player3D and MPEG-J for all MPEG-4 reference softwares players with samples.
* For installation, follow instructions in docs/Player3D_setup.doc.
*
* Many thanks to Prashanth Gujjeti and Viswanathan Swaminathan from Sun Microsystems.
*
* -- Mikael Bourges-Sevenier (Mindego)
*
* Revision 1.1  2002/08/23 16:47:24  pg138550
* Changes for MPEG-J. NOT FINAL
*
* Revision 1.1  2000/06/06 18:43:10  alavilli
* After SISF integration into the CORE4 code
*
* Revision 1.1.2.1  1999/07/09 14:16:58  depetris
* Aggiornati i file per lo sviluppo e il rilascio a MPEG-J.
* Integra la versione 2.5 del core.
*
* Revision 1.1  1998-11-25 17:17:40+01  vdheuvel
* 39 march 1999, start of archive.
* Middleware packages DAVIC API implementation.
*
 * 
 * 1     9-11-98 13:59 Vdheuvel
 * Creation
*/

package bali.table;

import java.util.Enumeration;
import java.util.Vector;
import bali.support.TablePart;

public class  DescriptorList
{
	/*******************************
	Creates an empty Descriptor list object
	*/
    DescriptorList( )
	{
		_descrVec = new java.util.Vector();
		_tp = null;
	}

	/*******************************
	Adds descriptors to an excisting Descriptor list object
	*/
    DescriptorList( DescriptorList descrList, TablePart tp )
	{
		_descrVec = descrList._descrVec;
     	_makeDescriptorList( tp );
     	_tp = tp;
	}

    /*******************************
    Creates a new descriptor list object
    */
    DescriptorList( TablePart tp )
    {
       	_descrVec = new java.util.Vector();
     	_makeDescriptorList( tp );
     	_tp = tp;
    }
    
    /*******************************
    This method returns a list of all tags stored in this descriptor list
    */
    public short[] getDescriptorTags()
    {
        Enumeration e = _descrVec.elements();
        short dt[]    = new short[ _descrVec.size() ];

        Descriptor d;
        int i=0;
        while ( e.hasMoreElements() )
        {
            d = (Descriptor)e.nextElement();
            dt[i] = d.getTag();
        }
        return dt;
    }

    /*********************************
    Returns an Enumeration of all descriptors in the list.
    */
    public Enumeration getDescriptors()
    {
        return _descrVec.elements();
    }

    /*********************************
    Returns an the number of descriptors in the list.
    */
    public int getNumberOfDescriptors()
    {
        return _descrVec.size();
    }
    
    /*********************************
    Returns the first instance of the indicated descriptor.
    When the descriptor is not found, null is returned.
    */
    public Descriptor getDescriptor( int tag )
    {
		Enumeration e = _descrVec.elements();
        
        Descriptor d=null;
        boolean found = false;
        int i=0;
        while ( e.hasMoreElements() && !found )
        {
            d = (Descriptor)e.nextElement();
            found = ( d.getTag() == tag );
        }
        return d;
    }
    

    //////////////////////////// PRIVATE /////////////////////////////
    private Vector _descrVec;
    private TablePart        _tp;

    /********************************
    This static function parses the indicated array from start to length and stores
    all detected descriptors in the Vector 
    */
    private void _makeDescriptorList( TablePart tp )
    {
        int   length = tp.getLength();
        int   index = 0;
        int   dl;
        while ( index < length )
        {
			// DBG System.out.println( "_makeDescriptorList: index " + index + " length " + length );
            dl = tp.get8bits( index+1 );
            _descrVec.addElement( new Descriptor( new TablePart( tp, index, dl + 2 ) ) );
            index += dl+2;
        }
    }
}
