/*
* Copyright(c) Philips Consumer Electronics B.V. 1998                         
* All rights reserved. Reproduction in whole or in part is                  
* prohibited without the written consent of the copyright holder.            
*                                                                            
*  Project             : Bali
*  File name   $Logfile: /func_test_platf2/bali/si/table/EIT.java $
*       Last save $Date: 2003/12/24 00:53:04 $
*             $Revision: 1.1.1.1 $
*          Last $Author: mikeseven $
*  Creation Date       : 28/10/98
*  First Author        : S.A.F.A. van den Heuvel
*                        NatLab - Eindhoven
* 
*                                                                          
*  Change history      :
*  ---------------
*
* $Log: EIT.java,v $
* Revision 1.1.1.1  2003/12/24 00:53:04  mikeseven
* Initial release of the CVS repository for MPEG-4 Systems code
* after the code was corrupted due to reorganization and human 
* errors.
*
* Revision 1.1  2003/04/03 04:29:25  mikeseven
* Initial release of Player3D and MPEG-J for all MPEG-4 reference softwares players with samples.
* For installation, follow instructions in docs/Player3D_setup.doc.
*
* Many thanks to Prashanth Gujjeti and Viswanathan Swaminathan from Sun Microsystems.
*
* -- Mikael Bourges-Sevenier (Mindego)
*
* Revision 1.1  2002/08/23 16:47:24  pg138550
* Changes for MPEG-J. NOT FINAL
*
* Revision 1.1  2000/06/06 18:43:11  alavilli
* After SISF integration into the CORE4 code
*
* Revision 1.1.2.1  1999/07/09 14:16:58  depetris
* Aggiornati i file per lo sviluppo e il rilascio a MPEG-J.
* Integra la versione 2.5 del core.
*
* Revision 1.2  1999-04-15 09:53:18+02  vdheuvel
* Files changed to hook the system up to the TM demultipler and the
* TM hardware. Changes include, swapping the java.util.Date class
* for the bali.tm.Date class (to prevent using 64 bit floats). Changing
* the HW sect filt used by bali.hw.SectFilt.
*
* Revision 1.1  1999-02-08 17:21:22+01  vdheuvel
* 39 march 1999, start of archive.
* Middleware packages DAVIC API implementation.
*
 * 
 * 3     13-11-98 10:10 Vdheuvel
 * 
 * 2     10-11-98 18:07 Vdheuvel
 * Added access to last_section_number and segement_last_section_number
 * field (required for EITsch).
 * 
 * 1     9-11-98 13:59 Vdheuvel
 * Creation
*/
package bali.table;

import java.util.Vector;
import java.util.Enumeration;
/*DATE_NORMAL*/ import  java.util.Date;
//DATE_TM*/     import  bali.tm.Date;
import bali.support.Util;
import bali.support.Debug;
import bali.support.TablePart;
import bali.hw.sectfilt.HwSection;
import org.davic.mpeg.dvb.DvbTransportStream;


/****************************************
This class parses the filtered EIT sections and allows access to the different EIT components.
The main difference with the SDT and NIT objects is the SDT and NIT objects parse tables. This
object parses sections, the sections are not necessaraly ordered into tables. 
This is done because the EIT schedule and present following table ae ordered by their section_number.
@see NIT
@see SDT
*/
public abstract class EIT
    extends SITable
{
	/** */ public final static int PID = 0x12; 
	/** */ public final static int TABLE_ID_EITpf_ACTUAL = 0x4E; 
	/** */ public final static int TABLE_ID_EITpf_OTHER  = 0x4F; 
	/** */ public final static int TABLE_ID_EITsch_ACTUAL_START = 0x50; 
	/** */ public final static int TABLE_ID_EITsch_ACTUAL_END   = 0x5F; 
	/** */ public final static int TABLE_ID_EITsch_OTHER_START  = 0x60; 
	/** */ public final static int TABLE_ID_EITsch_OTHER_END    = 0x6F; 

	/***************************************
	*/
	public EIT( EIT eit, DvbTransportStream ts, Date filterTime )
	{
		super( eit
		     , new SITableInfo( eit._ti.getVersion(), eit._ti.fromActual(), ts, filterTime )
		     );
		_vecEitEvent   = eit._vecEitEvent;
        _lastSectionNo = eit._lastSectionNo;

        int size = _vecEitEvent.size();
        for ( int i=0; i<size; i++ )
        {
            ((EitEvent)_vecEitEvent.elementAt( i ) )._ti =_ti;
        }

	}

	/***************************************
	*/
  	public EIT( HwSection[] arSec, DvbTransportStream ts, Date filterTime )
  	    throws InvalidTableException
	{
    	super( arSec 
    	     , new SITableInfo( arSec[0].version_number()
    			              , (  (arSec[0].table_id() == TABLE_ID_EITpf_ACTUAL )
    		                    || (arSec[0].table_id() >= TABLE_ID_EITsch_ACTUAL_START )
    		                    || (arSec[0].table_id() <= TABLE_ID_EITsch_ACTUAL_END )
    		                    )
        	                  , ts
            	              , filterTime 
            	              )
             );
        try
        {
            /*DBG*/ Debug.msg( Debug.TABLE, "parse EIT");
    
            _lastSectionNo = arSec[0].last_section_number();
            _vecEitEvent = new Vector();
    
    		// parse the section list in ft
    		for (int i=0;i<_arSec.length;i++)
    		{
    			int   EventLength = _arSec[i].getLength() - 4;	// - CRC
        	    int   index = 14;
       		  	int   el;
    	
    	        while ( index < EventLength )
       		    {
            	    el = 12 + _arSec[i].get12bits( index + 10 );
    	   	        _vecEitEvent.addElement( new EitEvent( new TablePart( _arSec[i], index, el ), _ti ) );
    	       	    index += el;
    	       	}
            }// end parsing section
        }
        catch( IndexOutOfBoundsException e )
        {
            throw new InvalidTableException( );
        }
    }

    /*****************
    Returns the TableIdentifier of this EIT.
    */
    public abstract TableIdentifier getTableIdentifier();

	/***************************************
	Return the service_id of this EIT.
	*/
    public int getServiceId()
    {
        return (int)_arSec[0].get16bits(3);
    }

	/***************************************
	Return the ts_id of this EIT.
	*/
    public int getTransportStreamId()
    {
        return (int)_arSec[0].get16bits(8);
    }

	/***************************************
	Return the or_nw_id of this EIT.
	*/
    public int getOriginalNetworkId()
    {
        return (int)_arSec[0].get16bits(10);
    }

	/***************************************
	Return the last section number of this EIT.
	*/
    public int getLastSectionNo()
    {
        return _lastSectionNo;
    }

	/***************************************
	Return the segment last section number of this EIT.
	*/
    public int getSegmentLastSectionNumber()
    {
        return (int)_arSec[0].get8bits(12);
    }

	/***************************************
	Return the segment last table id of this EIT.
	*/
    public int getLastTableId()
    {
        return (int)_arSec[0].get8bits(13);
    }

	/***************************************
	Returns the events part of this EIT.
	*/
	public EitEvent[] getEitEvents()
	{
		EitEvent[] ar =  new EitEvent[ _vecEitEvent.size() ];
		_vecEitEvent.copyInto( ar );
		return ar;
	}

	/***************************************
	Returns the event with the indicated event_id in this EIT (null when not present)
	*/
	public EitEvent getEitEvent( int event_id )
	{
		Enumeration e = _vecEitEvent.elements();
		EitEvent ret = null;
		boolean ready = false;
		while ( !ready && e.hasMoreElements() )
		{
			ret = (EitEvent)e.nextElement();
			ready = (ret.getEventId() == event_id );
		}
		if ( ready )
			return ret;
		else
			return null;
	}

	/////////////////////////////// PRIVATE //////////////////////////
	private Vector 	_vecEitEvent;
	private int     _lastSectionNo;
}
