/*
* Copyright(c) Philips Consumer Electronics B.V. 1998                         
* All rights reserved. Reproduction in whole or in part is                  
* prohibited without the written consent of the copyright holder.            
*                                                                            
*  Project             : Bali
*  File name   $Logfile: /func_test_platf2/bali/si/table/EITpf.java $
*       Last save $Date: 2003/12/24 00:53:04 $
*             $Revision: 1.1.1.1 $
*          Last $Author: mikeseven $
*  Creation Date       : 28/10/98
*  First Author        : S.A.F.A. van den Heuvel
*                        NatLab - Eindhoven
* 
*                                                                          
*  Change history      :
*  ---------------
*
* $Log: EITpf.java,v $
* Revision 1.1.1.1  2003/12/24 00:53:04  mikeseven
* Initial release of the CVS repository for MPEG-4 Systems code
* after the code was corrupted due to reorganization and human 
* errors.
*
* Revision 1.1  2003/04/03 04:29:25  mikeseven
* Initial release of Player3D and MPEG-J for all MPEG-4 reference softwares players with samples.
* For installation, follow instructions in docs/Player3D_setup.doc.
*
* Many thanks to Prashanth Gujjeti and Viswanathan Swaminathan from Sun Microsystems.
*
* -- Mikael Bourges-Sevenier (Mindego)
*
* Revision 1.1  2002/08/23 16:47:24  pg138550
* Changes for MPEG-J. NOT FINAL
*
* Revision 1.1  2000/06/06 18:43:11  alavilli
* After SISF integration into the CORE4 code
*
* Revision 1.1.2.1  1999/07/09 14:16:59  depetris
* Aggiornati i file per lo sviluppo e il rilascio a MPEG-J.
* Integra la versione 2.5 del core.
*
* Revision 1.2  1999-04-15 09:53:18+02  vdheuvel
* Files changed to hook the system up to the TM demultipler and the
* TM hardware. Changes include, swapping the java.util.Date class
* for the bali.tm.Date class (to prevent using 64 bit floats). Changing
* the HW sect filt used by bali.hw.SectFilt.
*
* Revision 1.1  1999-02-08 17:23:26+01  vdheuvel
* 39 march 1999, start of archive.
* Middleware packages DAVIC API implementation.
*
 * 
 * 1     9-11-98 13:59 Vdheuvel
 * Creation
*/
package bali.table;

import java.util.Vector;
import java.util.Enumeration;
/*DATE_NORMAL*/ import  java.util.Date;
//DATE_TM*/     import  bali.tm.Date;
import bali.hw.sectfilt.HwSection;
import bali.support.Debug;
import bali.support.TablePart;
import org.davic.mpeg.dvb.DvbTransportStream;

/****************************************
This class parses the filtered EIT present following sections and allows access to the  present
and following event
The length of a EITpf is always two sections, The first section holds the present event, the
second section the following event.
@see NIT
@see SDT
@see EIT
*/
public class EITpf
	extends EIT
{
    
	/***************************************
	*/
	public EITpf( EITpf eitpf, DvbTransportStream ts, Date filterTime )
	{
		super( eitpf, ts, filterTime );
		_present   = eitpf._present;
		_following = eitpf._following;
	}

	/***************************************
	*/
	public EITpf( HwSection[] arSec, DvbTransportStream ts, Date filterTime )
	    throws InvalidTableException
    {
    	super( arSec, ts, filterTime );


        try
        {
            /*DBG*/ Debug.msg( Debug.TABLE, "parse EITpf");
        
    		EitEvent[] ar = this.getEitEvents();
    		int eventNo =0;
    		// If the first section is empty, there is no present event [ETR 211] 
            /*DBG*/ Debug.msg( Debug.TABLE, Debug.LOW, "present length "+arSec[0].section_length());
    		if ( arSec[0].section_length() > 15 ) 
    		{   _present = ar[eventNo];
    		    eventNo++;
    		}
    		// If the second section is empty, no following event is present (ETR 211)
    		
    		if ( arSec.length > 1 )
    		{   /*DBG*/ Debug.msg( Debug.TABLE, Debug.LOW, "following length "+arSec[1].section_length());
    		    if ( arSec[1].section_length() > 15 ) 
            	{   
            		_following = ar[eventNo];
            	}
            }
        }
        catch( IndexOutOfBoundsException e )
        {
            throw new InvalidTableException( );
        }
    }

    /*****************
    Returns the TableIdentifier of this EITpf.
    */
    public TableIdentifier getTableIdentifier()
    {
        synchronized( this )
        {
            if ( _tableIdentifier == null )
            {
                int[] arInt = new int[3];
                arInt[0] = getOriginalNetworkId();
                arInt[1] = getTransportStreamId();
                arInt[2] = getServiceId();
                _tableIdentifier = new TableIdentifier( TableIdentifier.EITpf, arInt );
            }
        }
        return _tableIdentifier;
    }

	/***************************************
	Returns the present EitEvent of this EITpf.
	If no present event is present null is returned.
	*/
	public EitEvent getPresentEitEvent()
	{
		return _present;
	}

	/***************************************
	Returns the following EitEvent of this EITpf.
	If no following event is present null is returned.
	*/
	public EitEvent getFollowingEitEvent()
	{
		return _following;
	}

	/////////////////////////////// PRIVATE //////////////////////////
	private EitEvent _present   = null;
	private EitEvent _following = null;
}
