/*
* Copyright(c) Philips Consumer Electronics B.V. 1998                         
* All rights reserved. Reproduction in whole or in part is                  
* prohibited without the written consent of the copyright holder.            
*                                                                            
*  Project             : Bali
*  File name   $Logfile: /func_test_platf2/bali/si/table/EITpf.java $
*       Last save $Date: 2003/12/24 00:53:04 $
*             $Revision: 1.1.1.1 $
*          Last $Author: mikeseven $
*  Creation Date       : 28/10/98
*  First Author        : S.A.F.A. van den Heuvel
*                        NatLab - Eindhoven
* 
*                                                                          
*  Change history      :
*  ---------------
*
* $Log: EITsch.java,v $
* Revision 1.1.1.1  2003/12/24 00:53:04  mikeseven
* Initial release of the CVS repository for MPEG-4 Systems code
* after the code was corrupted due to reorganization and human 
* errors.
*
* Revision 1.1  2003/04/03 04:29:25  mikeseven
* Initial release of Player3D and MPEG-J for all MPEG-4 reference softwares players with samples.
* For installation, follow instructions in docs/Player3D_setup.doc.
*
* Many thanks to Prashanth Gujjeti and Viswanathan Swaminathan from Sun Microsystems.
*
* -- Mikael Bourges-Sevenier (Mindego)
*
* Revision 1.1  2002/08/23 16:47:25  pg138550
* Changes for MPEG-J. NOT FINAL
*
* Revision 1.1  2000/06/06 18:43:11  alavilli
* After SISF integration into the CORE4 code
*
* Revision 1.1.2.1  1999/07/09 14:16:59  depetris
* Aggiornati i file per lo sviluppo e il rilascio a MPEG-J.
* Integra la versione 2.5 del core.
*
* Revision 1.2  1999-04-15 09:44:58+02  vdheuvel
* Files changed to hook the system up to the TM demultipler and the
* TM hardware. Changes include, swapping the java.util.Date class
* for the bali.tm.Date class (to prevent using 64 bit floats). Changing
* the HW sect filt used by bali.hw.SectFilt.
*
* Revision 1.1  1999-02-09 11:15:06+01  vdheuvel
* 39 march 1999, start of archive.
* Middleware packages DAVIC API implementation.
*
 * 
 * 1     9-11-98 13:59 Vdheuvel
 * Creation
*/
package bali.table;

import java.util.Vector;
import java.util.Enumeration;
/*DATE_NORMAL*/ import  java.util.Date;
//DATE_TM*/     import  bali.tm.Date;
import bali.hw.sectfilt.HwSection;
import bali.support.Debug;
import bali.support.Assert;
import bali.support.TablePart;
import org.davic.mpeg.dvb.DvbTransportStream;

/****************************************
This class parses the filtered EIT schedule sub table. A EITsch sub table is defined as
one segment of one table of the EIT schedule.
@see EIT
@see EitEvent
*/
public class EITsch
	extends EIT
{
    
	/***************************************
	*/
	public EITsch( EITsch eit, DvbTransportStream ts, Date filterTime )
	{
		super( eit, ts, filterTime );
	}

	/***************************************
	*/
	public EITsch( HwSection[] arSec, DvbTransportStream ts, Date filterTime )
	    throws InvalidTableException
    {
    	super( arSec, ts, filterTime );
        /*DBG*/ Debug.msg( Debug.TABLE, "parse EITsch");
    }

    /*****************
    Returns the TableIdentifier of this EITsch.
    */
    public TableIdentifier getTableIdentifier()
    {
        synchronized( this )
        {
            if ( _tableIdentifier == null )
            {
                int[] arInt = new int[3];
                arInt[0] = getOriginalNetworkId();
                arInt[1] = getTransportStreamId();
                arInt[2] = getServiceId();
                _tableIdentifier = new TableIdentifier( TableIdentifier.EITsch, arInt );
            }
        }
        return _tableIdentifier;
    }

    /***************************************
    */
    public static int numberOfPeriodsPastMidnight( Date midnight, Date time )
    {   
        int  period = 3*60*60*1000;
        long delta  = ( time.getTime() - midnight.getTime() );
        int Nperiod = (int)( delta / period );
        return Nperiod;
    }

    /***************************************
    Returns the table if offset of this period. In the first table,
    the offset is 0, in the second table, the offset is 1, ... .
    */
    public static int getTableIdOffset( int period )
    {   
        int off = (int)( period / 32 ); 
        off = ( ( off>0xF) ? 0xf: off  );
        return off;
    }

    /***************************************
    Returns the segment no of the period in its table.
    */
    public static int getSegmentNo( int period )
    {   
        int no = period - getTableIdOffset( period );
        /*DBG*/ Assert.assert( no<0 );
        /*DBG*/ Assert.assert( no>32);
        return no;
    }
    
    /***************************************
    Return start section no of the segment.
    */
    public static int getStartSectionNo( int segment )
    {
        int start = segment*8;
        start = ( (start>(256-8)) ? 256: start );
        return start;
    }
    /***************************************
    Return end section no of the segment.
    */
    public static int getEndSectionNo( int segment )
    {
        int end = segment*8+7;
        end = ( (end>(256-8)) ? 256: end );
        return end;
    }
}
