/*
* Copyright(c) Philips Consumer Electronics B.V. 1998                         
* All rights reserved. Reproduction in whole or in part is                  
* prohibited without the written consent of the copyright holder.            
*                                                                            
*  Project             : Bali
*  File name   $Logfile: /func_test_platf2/bali/si/table/EitEvent.java $
*       Last save $Date: 2003/12/24 00:53:04 $
*             $Revision: 1.1.1.1 $
*          Last $Author: mikeseven $
*  Creation Date       : 28/10/98
*  First Author        : S.A.F.A. van den Heuvel
*                        NatLab - Eindhoven
* 
*                                                                          
*  Change history      :
*  ---------------
*
* $Log: EitEvent.java,v $
* Revision 1.1.1.1  2003/12/24 00:53:04  mikeseven
* Initial release of the CVS repository for MPEG-4 Systems code
* after the code was corrupted due to reorganization and human 
* errors.
*
* Revision 1.1  2003/04/03 04:29:26  mikeseven
* Initial release of Player3D and MPEG-J for all MPEG-4 reference softwares players with samples.
* For installation, follow instructions in docs/Player3D_setup.doc.
*
* Many thanks to Prashanth Gujjeti and Viswanathan Swaminathan from Sun Microsystems.
*
* -- Mikael Bourges-Sevenier (Mindego)
*
* Revision 1.1  2002/08/23 16:47:25  pg138550
* Changes for MPEG-J. NOT FINAL
*
* Revision 1.1  2000/06/06 18:43:11  alavilli
* After SISF integration into the CORE4 code
*
* Revision 1.1.2.1  1999/07/09 14:17:00  depetris
* Aggiornati i file per lo sviluppo e il rilascio a MPEG-J.
* Integra la versione 2.5 del core.
*
* Revision 1.2  1999-04-15 09:53:18+02  vdheuvel
* Files changed to hook the system up to the TM demultipler and the
* TM hardware. Changes include, swapping the java.util.Date class
* for the bali.tm.Date class (to prevent using 64 bit floats). Changing
* the HW sect filt used by bali.hw.SectFilt.
*
* Revision 1.1  1998-12-09 17:28:00+01  vdheuvel
* 39 march 1999, start of archive.
* Middleware packages DAVIC API implementation.
*
 * 
 * 1     9-11-98 13:59 Vdheuvel
 * Creation
*/
package bali.table;

import java.util.Vector;
import java.util.Enumeration;
/*DATE_NORMAL*/ import  java.util.Date;
//DATE_TM*/     import  bali.tm.Date;
import bali.support.Util;
import bali.support.Debug;
import bali.support.TablePart;

/******************************
*/
public class EitEvent
    extends FilteredSITable
{
	/** */ public final static int SHORT_EVENT_DESCRIPTOR_TAG  = 0x4D; 
	/** */ public final static int CONTENT_DESCRIPTOR_TAG      = 0x54; 

	/******************************
	*/
	EitEvent( TablePart tp, SITableInfo ti )
	{
	    super( ti );
		_tablePart = tp;

        int descLen = _tablePart.get12bits( 10 );
        _descriptorList = new DescriptorList( new TablePart( _tablePart, 12, descLen ) );
        parseStrings();
	}

	/******************************
	*/
	public int getEventId()
	{
		return _tablePart.get16bits( 0 );
	}
	

	/******************************
	*/
	public Date getStartTime()
	{
		return _tablePart.getUTCTime( 2 );
	}

	/******************************
	*/
	public long getDuration()
	{
		return _tablePart.getBCDTime( 7 );
	}

	/******************************
	*/
	public int getRunningStatus()
	{
		int res;

		res = _tablePart.getByte( 10 )&0xE0;
		res = res >> 5;
		
		return res;
	}

	/******************************
	*/
	public boolean getFreeCaMode()
	{
		int res;

		res = _tablePart.getByte( 10 )&0x10;
		res = res >> 4;
		
		return (res==1);
	}

	/******************************
	*/
	public String getEventName()
	{
		return _eventName.getString();
	}

	/******************************
	*/
	public String getShortEventName()
	{
		return _eventName.getShortString();
	}

	/******************************
	*/
	public String getEventDescription()
	{
		return _description.getString();
	}

	/******************************
	*/
	public String getShortEventDescription()
	{
		return _description.getShortString();
	}

	/******************************
	*/
	public byte[] getLevel1ContentNibbles()
	{
		byte[] res;
		
        Descriptor d; 
        d = _descriptorList.getDescriptor( CONTENT_DESCRIPTOR_TAG );
        if ( d!=null )
        {
			int Nentries = d.get8bits(1) >>1;  // divide by 2
			res = new byte[ Nentries ];
			for ( int i=0; i<Nentries ; i++ )
			{
				res[i] = (byte)((d.get8bits( 2 + i*2 ) & 0xF0) >>4);
			}
		}
        else
        {
        	res = new byte[0];
        }
		
		return res;
	}
	
	/******************************
	*/
	public byte[] getContentNibbles()
	{
		byte[] res;
		
        Descriptor d; 
        d = _descriptorList.getDescriptor( CONTENT_DESCRIPTOR_TAG );
		if ( d!=null )
		{
			int Nentries = d.get8bits(1) >>1;  // divide by 2
			res = new byte[ Nentries ];
			for ( int i=0; i<Nentries ; i++ )
			{
				res[i] = (byte) ( d.get8bits( 2 + i*2 ) );
			}
		}
		else
		{
			res = new byte[0];		}
		
		return ( res );
	}
	
	/******************************
	*/
	public DescriptorList getDescriptorList()
    {
		return _descriptorList;
    }

    //////////////////////// PRIVATE ///////////////////////////////////
	private DescriptorList 	_descriptorList;
	private TablePart 		_tablePart;
	private DvbString       _eventName;
	private DvbString       _description;
	
	/************************
	*/
	private void parseStrings()
	{
        //DBG*/ System.out.println("Serv.parseStrings" );
        Descriptor d; 
        d = _descriptorList.getDescriptor( SHORT_EVENT_DESCRIPTOR_TAG );

        if ( d != null )
        {
			int nameStart = 6;
			int nameEnd   = nameStart + d.get8bits( nameStart-1 ) ;
			int descStart = nameEnd + 1;
			int descEnd   = descStart + d.get8bits( descStart-1 ) ;
			
			_eventName   = new DvbString( d.getContent(), nameStart, nameEnd );
			_description = new DvbString( d.getContent(), descStart, descEnd );
		}
		else
		{
			_eventName   = new DvbString();
			_description = new DvbString();
		}
 	}

}

