/*
* Copyright(c) Philips Consumer Electronics B.V. 1998                         
* All rights reserved. Reproduction in whole or in part is                  
* prohibited without the written consent of the copyright holder.            
*                                                                            
*  Project             : Bali
*  File name   $Logfile: /func_test_platf2/bali/si/table/NIT.java $
*       Last save $Date: 2003/12/24 00:53:04 $
*             $Revision: 1.1.1.1 $
*          Last $Author: mikeseven $
*  Creation Date       : 28/10/98
*  First Author        : S.A.F.A. van den Heuvel
*                        NatLab - Eindhoven
* 
*                                                                          
*  Change history      :
*  ---------------
*
* $Log: NIT.java,v $
* Revision 1.1.1.1  2003/12/24 00:53:04  mikeseven
* Initial release of the CVS repository for MPEG-4 Systems code
* after the code was corrupted due to reorganization and human 
* errors.
*
* Revision 1.1  2003/04/03 04:29:26  mikeseven
* Initial release of Player3D and MPEG-J for all MPEG-4 reference softwares players with samples.
* For installation, follow instructions in docs/Player3D_setup.doc.
*
* Many thanks to Prashanth Gujjeti and Viswanathan Swaminathan from Sun Microsystems.
*
* -- Mikael Bourges-Sevenier (Mindego)
*
* Revision 1.1  2002/08/23 16:47:25  pg138550
* Changes for MPEG-J. NOT FINAL
*
* Revision 1.1  2000/06/06 18:43:11  alavilli
* After SISF integration into the CORE4 code
*
* Revision 1.1.2.1  1999/07/09 14:17:01  depetris
* Aggiornati i file per lo sviluppo e il rilascio a MPEG-J.
* Integra la versione 2.5 del core.
*
* Revision 1.2  1999-04-15 09:53:18+02  vdheuvel
* Files changed to hook the system up to the TM demultipler and the
* TM hardware. Changes include, swapping the java.util.Date class
* for the bali.tm.Date class (to prevent using 64 bit floats). Changing
* the HW sect filt used by bali.hw.SectFilt.
*
* Revision 1.1  1999-02-08 17:24:30+01  vdheuvel
* 39 march 1999, start of archive.
* Middleware packages DAVIC API implementation.
*
 * 
 * 1     9-11-98 13:59 Vdheuvel
 * Creation
*/
package bali.table;

import java.util.Vector;
import java.util.Enumeration;
/*DATE_NORMAL*/ import  java.util.Date;
//DATE_TM*/     import  bali.tm.Date;
import bali.support.SeriousError;
import bali.support.Util;
import bali.support.Debug;
import bali.support.TablePart;
import bali.hw.sectfilt.HwSection;
import org.davic.mpeg.dvb.DvbTransportStream;

/****************************************
This class parses the filtered NIT tables and allows access to the different NIT components.
*/
public class NIT
    extends SITable
{
	/** */ public final static int PID = 0x10; 
	/** */ public final static int TABLE_ID_ACTUAL = 0x40; 
	/** */ public final static int TABLE_ID_OTHER  = 0x41; 
	/** */ public final static int NETWORK_NAME_DESCRIPTOR_TAG  = 0x40; 

	/***************************************
	*/
	public NIT( NIT table, DvbTransportStream ts, Date filterTime )
	{
		super( table
    	     , new SITableInfo( table._ti
        	                  , ts
            	              , filterTime ) 
             );
        _descriptorList = table._descriptorList;
        _TSList         = table._TSList;
        _networkName    = table._networkName;
		
		Enumeration e = _TSList.elements();
		while ( e.hasMoreElements() )
		{
			((TS)e.nextElement())._ti = _ti;
		}
	}

	/***************************************
	*/
	public NIT( HwSection[] arSec, DvbTransportStream ts, Date filterTime )
	    throws InvalidTableException
    {
    	super( arSec, 
    	     new SITableInfo( arSec[0].version_number()
    			            , arSec[0].table_id() == TABLE_ID_ACTUAL
        	                , ts
            	            , filterTime ) 
             );
        try
        {
            /*DBG*/ Debug.msg( Debug.TABLE, "parse NIT");
    
    		// data stored in an array of TablePart
            
            _TSList = new Vector();
    	    _descriptorList = new DescriptorList();
    
    		// parse all sections of this table.
    		for ( int i=0; i<_arSec.length; i++ )
    		{	// parse section
    	    	int descLen;
    
    	        descLen = _arSec[i].get12bits( 8 );
    	        _descriptorList = new DescriptorList( _descriptorList, new TablePart( _arSec[i], 10, descLen ) );
    
    			int   tsLength = _arSec[i].get12bits( descLen+10 );
    	        int   tl;
        	    int   index = descLen+12;
    
    
    	        while ( index < tsLength+descLen+10 )
        	    {
            	    tl = 6 + _arSec[i].get12bits( index + 4 );
    	            _TSList.addElement( new TS( new TablePart( _arSec[i], index, tl ), _ti ) );
    	            index += tl;
        	    }
            }// end parsing sections
            
            parseStrings();
        }
        catch( IndexOutOfBoundsException e )
        {
            throw new InvalidTableException( );
        }
    }

    /*****************
    Returns the TableIdentifier of this NIT.
    */
    public TableIdentifier getTableIdentifier()
    {
        synchronized( this )
        {
            if ( _tableIdentifier == null )
            {
                int[] arInt = new int[1];
                arInt[0] = getNetworkId();
                _tableIdentifier = new TableIdentifier( TableIdentifier.NIT, arInt );
            }
        }
        return _tableIdentifier;
    }

	/***************************************
	Return the descriptors of the main loop of the NIT table.
	*/
    public DescriptorList getDescriptorList()
    {
		return _descriptorList;
    }
    
	/***************************************
	Return the network_id of this NIT.
	*/
    public int getNetworkId()
    {
        return _arSec[0].table_id_extension();
    }

	/***************************************
	Return the network_name of this NIT ("" of none existend).
	*/
    public String getNetworkName()
    {
        return _networkName.getString();
    }

	/***************************************
	Return the short network_name of this NIT ("" of none existend).
	*/
    public String getShortNetworkName()
    {
        return _networkName.getShortString();
    }

	/***************************************
	Return the TransportStreams in this NIT
	*/
	public TS[] getTSs()
	{
		TS[] ar = new TS[ _TSList.size() ];
		_TSList.copyInto( ar );
		return ar;
	}
	
	/***************************************
	Return the TransportStream in this NIT, null when not present
	*/
	public TS getTS( int or_nw_id, int ts_id )
	{
		Enumeration e = _TSList.elements();
		TS ts = null;
		boolean found = false;
		while ( e.hasMoreElements() && !found )
		{
			ts = (TS)e.nextElement();
			if (  ts_id    == ts.getTransportStreamId() 
			   && or_nw_id == ts.getOriginalNetworkId() )
			{
				found = true;
			}
		}
		if ( found )
			return ts;
		else
			return null;
	}


	/***************************************
	Return the number of TransportStreams in this NIT
	*/
    public int getNumberOfTransportStreams()
	{
		return _TSList.size();
	}

	
	/////////////////////////////// PRIVATE //////////////////////////
	private DescriptorList 	_descriptorList;
	private Vector 			_TSList;
	private DvbString       _networkName;

	/************************
	*/
	private void parseStrings()
	{
        //DBG*/ System.out.println("Serv.parseStrings" );
        Descriptor d; 
        d = _descriptorList.getDescriptor( NETWORK_NAME_DESCRIPTOR_TAG );

        if ( d != null )
        {
			int start = 2;
			int end   = start  + d.get8bits( 1 ) ;

            //DBG*/ System.out.println("   start"+start+" end: "+end );
			
			_networkName = new DvbString( d.getContent(), start, end );
		}                
		else
		{
			_networkName = new DvbString();
		}
	}

}
