/*
* Copyright(c) Philips Consumer Electronics B.V. 1998                         
* All rights reserved. Reproduction in whole or in part is                  
* prohibited without the written consent of the copyright holder.            
*                                                                            
*  Project             : Bali
*  Module              : bali 
*  File name   $Logfile: /func_test_platf2/bali/si/table/PAT.java $
*       Last save $Date: 2003/12/24 00:53:04 $
*             $Revision: 1.1.1.1 $
*          Last $Author: mikeseven $
*  Creation Date       : 28/10/98
*  First Author        : S.A.F.A. van den Heuvel
*                        NatLab - Eindhoven
* 
*                                                                          
*  Change history      :
*  ---------------
*
* $Log: PAT.java,v $
* Revision 1.1.1.1  2003/12/24 00:53:04  mikeseven
* Initial release of the CVS repository for MPEG-4 Systems code
* after the code was corrupted due to reorganization and human 
* errors.
*
* Revision 1.1  2003/04/03 04:29:26  mikeseven
* Initial release of Player3D and MPEG-J for all MPEG-4 reference softwares players with samples.
* For installation, follow instructions in docs/Player3D_setup.doc.
*
* Many thanks to Prashanth Gujjeti and Viswanathan Swaminathan from Sun Microsystems.
*
* -- Mikael Bourges-Sevenier (Mindego)
*
* Revision 1.1  2002/08/23 16:47:25  pg138550
* Changes for MPEG-J. NOT FINAL
*
* Revision 1.1  2000/06/06 18:43:12  alavilli
* After SISF integration into the CORE4 code
*
* Revision 1.1.2.1  1999/07/09 14:17:01  depetris
* Aggiornati i file per lo sviluppo e il rilascio a MPEG-J.
* Integra la versione 2.5 del core.
*
* Revision 1.2  1999-04-15 09:53:18+02  vdheuvel
* Files changed to hook the system up to the TM demultipler and the
* TM hardware. Changes include, swapping the java.util.Date class
* for the bali.tm.Date class (to prevent using 64 bit floats). Changing
* the HW sect filt used by bali.hw.SectFilt.
*
* Revision 1.1  1999-02-12 15:13:08+01  vdheuvel
* 39 march 1999, start of archive.
* Middleware packages DAVIC API implementation.
*
 * 
 * 1     9-11-98 13:59 Vdheuvel
 * Creation
*/
package bali.table;

import java.util.Vector;
import java.util.Enumeration;
/*DATE_NORMAL*/ import  java.util.Date;
//DATE_TM*/     import  bali.tm.Date;
import bali.support.SeriousError;
import bali.support.Util;
import bali.support.Debug;
import bali.hw.sectfilt.HwSection;
import org.davic.mpeg.dvb.DvbTransportStream;

/****************************************
This class parses the filtered PAT tables and allows access to the different PAT components.
*/
public class PAT
    extends SITable
{
	/** */ public final static int PID = 0x0; 
	/** */ public final static int TABLE_ID = 0x0; 

	/***************************************
	*/
	public PAT( PAT table, DvbTransportStream ts, Date filterTime )
	{
		super( table, new SITableInfo( table._ti.getVersion()
             , table._ti.fromActual()
             , ts, filterTime )
             );
        /*DVB*/Debug.msg( Debug.TABLE, "PAT: copy constructor");
		_ts = table._ts;
        _vecProgramList = table._vecProgramList;
        int size = _vecProgramList.size();
        for ( int i=0; i<size; i++ )
        {
            ((Program)_vecProgramList.elementAt( i ) )._ti =_ti;
        }
	}

	/***************************************
	*/
	public PAT( HwSection[] arSec, DvbTransportStream ts, Date filterTime )
        throws InvalidTableException
    {
    	super( arSec, new SITableInfo( arSec[0].version_number(), true, ts, filterTime ) );

        try
        {
            /*DBG*/ Debug.msg( Debug.TABLE, "parse PAT");
    		// data stored in an array of TablePart
            
            _vecProgramList  = new Vector();
            _ts = ts;
    		// parse all sections of this table.
    		for ( int i=0; i<_arSec.length; i++ )
    		{	// parse section
    			int   Length = _arSec[i].getLength()-4;
    	        int   index = 8;
    	        int   no, pid;
    
    	        while ( index < Length )
        	    {
        	    	no  = _arSec[i].get16bits( index );
        	    	pid = (_arSec[i].get16bits( index+2 )) & 0x1FFF;
        	        _vecProgramList.addElement( new Program( no, pid, _ti ));
            	    index += 4;
    	        }
            }// end parsing sections
        }
        catch( IndexOutOfBoundsException e )
        {
            throw new InvalidTableException( );
        }
    }

    /*****************
    Returns the TableIdentifier of this PAT.
    */
    public TableIdentifier getTableIdentifier()
    {
        synchronized( this )
        {
            if ( _tableIdentifier == null )
            {
                int[] arInt = new int[2];
                arInt[0] = getOriginalNetworkId();
                arInt[1] = getTransportStreamId();
                _tableIdentifier = new TableIdentifier( TableIdentifier.PAT, arInt );
            }
        }
        return _tableIdentifier;
    }

	/***************************************
	Return the or_nw_id of this PAT.
	*/
    public int getOriginalNetworkId()
    {
 		return _ts.getOriginalNetworkId();
    }

	/***************************************
	Return the ts_id of this PAT.
	*/
    public int getTransportStreamId()
    {
 		return _ts.getTransportStreamId();
    }

	/***************************************
	Returns the programs part of this PAT.
	*/
	public Enumeration getPrograms()
	{
		return _vecProgramList.elements();
	}
	
	/***************************************
	Returns the pid of the PMT of indicated program.
	When the program is not in this PAT, -2 is returned.
	*/
	public int getProgramPid( int programNo )
	{
	    boolean     ready = false;
		Enumeration e = _vecProgramList.elements();
		Program     pr = null;
		
		while ( !ready && e.hasMoreElements() )
		{
		    pr = (Program)e.nextElement();
		    //DGB*/System.out.println("PAT "+pr.getNo()+" "+pr.getPID() );
		    if ( pr.getNo() == programNo )
		    {   ready = true;
		    }
		}
		if ( ready )
		{   return pr.getPID() ;
		}
		else
		{   return -2;
		}
	}
	
	/////////////////////////////// PRIVATE //////////////////////////
	private Vector 	_vecProgramList;
	private DvbTransportStream _ts;
	
}
