/*
* Copyright(c) Philips Consumer Electronics B.V. 1998                         
* All rights reserved. Reproduction in whole or in part is                  
* prohibited without the written consent of the copyright holder.            
*                                                                            
*  Project             : Bali
*  File name   $Logfile: /func_test_platf2/bali/si/table/PMT.java $
*       Last save $Date: 2003/12/24 00:53:04 $
*             $Revision: 1.1.1.1 $
*          Last $Author: mikeseven $
*  Creation Date       : 28/10/98
*  First Author        : S.A.F.A. van den Heuvel
*                        NatLab - Eindhoven
* 
*                                                                          
*  Change history      :
*  ---------------
*
* $Log: PMT.java,v $
* Revision 1.1.1.1  2003/12/24 00:53:04  mikeseven
* Initial release of the CVS repository for MPEG-4 Systems code
* after the code was corrupted due to reorganization and human 
* errors.
*
* Revision 1.1  2003/04/03 04:29:26  mikeseven
* Initial release of Player3D and MPEG-J for all MPEG-4 reference softwares players with samples.
* For installation, follow instructions in docs/Player3D_setup.doc.
*
* Many thanks to Prashanth Gujjeti and Viswanathan Swaminathan from Sun Microsystems.
*
* -- Mikael Bourges-Sevenier (Mindego)
*
* Revision 1.1  2002/08/23 16:47:25  pg138550
* Changes for MPEG-J. NOT FINAL
*
* Revision 1.1  2000/06/06 18:43:12  alavilli
* After SISF integration into the CORE4 code
*
* Revision 1.1.2.1  1999/07/09 14:17:02  depetris
* Aggiornati i file per lo sviluppo e il rilascio a MPEG-J.
* Integra la versione 2.5 del core.
*
* Revision 1.2  1999-04-15 09:53:18+02  vdheuvel
* Files changed to hook the system up to the TM demultipler and the
* TM hardware. Changes include, swapping the java.util.Date class
* for the bali.tm.Date class (to prevent using 64 bit floats). Changing
* the HW sect filt used by bali.hw.SectFilt.
*
* Revision 1.1  1999-02-15 17:36:30+01  vdheuvel
* 39 march 1999, start of archive.
* Middleware packages DAVIC API implementation.
*
 * 
 * 1     9-11-98 13:59 Vdheuvel
 * Creation
*/
package bali.table;

import java.util.Vector;
import java.util.Enumeration;
/*DATE_NORMAL*/ import  java.util.Date;
//DATE_TM*/     import  bali.tm.Date;
import bali.support.Util;
import bali.support.Debug;
import bali.support.TablePart;
import bali.hw.sectfilt.HwSection;
import org.davic.mpeg.TransportStream;
import org.davic.mpeg.dvb.DvbTransportStream;

/****************************************
This class parses the filtered PMT tables and allows access to the different PMT components.
*/
public class PMT
    extends SITable
{
	/** */ public final static int TABLE_ID = 0x2; 

	/***************************************
	*/
	public PMT( PMT table, DvbTransportStream ts, Date filterTime )
	{
		super( table
		     , new SITableInfo( table._ti.getVersion(), table._ti.fromActual(), ts, filterTime )
		     );
        _descriptorList = table._descriptorList;
        _vecES          = table._vecES;
        _ts             = table._ts;
        
        int size = _vecES.size();
        for ( int i=0; i<size; i++ )
        {
            ((ES)_vecES.elementAt( i ) )._ti =_ti;
        }
	}

	/***************************************
	*/
	public PMT( HwSection[] arSec, DvbTransportStream ts, Date filterTime )
        throws InvalidTableException    
    {
    	super( arSec
    	     , new SITableInfo( arSec[0].version_number(), true, ts, filterTime )
             );
        try
        {
            /*DBG*/ Debug.msg( Debug.TABLE, "parse PMT");
    
    		// data stored in an array of TablePart
    
            _vecES = new Vector();
    	    _descriptorList = new DescriptorList();
    
    		// parse all sections of this table.
    		for ( int i=0; i<_arSec.length; i++ )
    		{	// parse section
    	    	int descLen;
    			int esl;
    
    	        descLen = _arSec[i].get12bits( 10 );
    	        _descriptorList = new DescriptorList( _descriptorList, new TablePart( _arSec[i], 12, descLen ) );
    
    			int   ESlength = _arSec[i].getLength() - 4;
    	        int   tl;
        	    int   index = descLen+12;
    
    
    	        while ( index < ESlength )
        	    {
            	    esl = 5 + _arSec[i].get12bits( index + 3 );
    	            _vecES.addElement( new ES( new TablePart( _arSec[i], index, esl ), _ti ) );
    	            index += esl;
        	    }
            }// end parsing sections
            _ts = ts;
        }
        catch( IndexOutOfBoundsException e )
        {
            throw new InvalidTableException( );
        }
    }

    /*****************
    Returns the TableIdentifier of this PMT.
    */
    public TableIdentifier getTableIdentifier()
    {
        synchronized( this )
        {
            if ( _tableIdentifier == null )
            {
                int[] arInt = new int[3];
                arInt[0] = getOriginalNetworkId();
                arInt[1] = getTransportStreamId();
                arInt[2] = getProgramNo();
                _tableIdentifier = new TableIdentifier( TableIdentifier.PMT, arInt );
            }
        }
        return _tableIdentifier;
    }

	/***************************************
	Return the descriptors of the main loop of the PMT table.
	*/
	public DescriptorList getDescriptorList()
    {
		return _descriptorList;
    }
    
	/***************************************
	Return the program_no of this PMT.
	*/
    public int getProgramNo()
    {
        return _arSec[0].table_id_extension();
    }

	/***************************************
	Return the PCR PID of this PMT.
	*/
    public int getPCR_PID()
    {
        return _arSec[0].get13bits(8);
    }

	/***************************************
	Return the or_nw_id of this PAT.
	*/
    public int getOriginalNetworkId()
    {
 		return _ts.getOriginalNetworkId();
    }

	/***************************************
	Return the ts_id of this PAT.
	*/
    public int getTransportStreamId()
    {
 		return _ts.getTransportStreamId();
    }


	/***************************************
	*/
	public ES[] getESs()
	{
		ES[] ar = new ES[ _vecES.size() ];
		_vecES.copyInto( ar );
		return ar;
	}
	
	
	/////////////////////////////// PRIVATE //////////////////////////
	private DescriptorList 	_descriptorList;
	private Vector 			_vecES;
	private DvbTransportStream _ts;
}
