/*
* Copyright(c) Philips Consumer Electronics B.V. 1998                         
* All rights reserved. Reproduction in whole or in part is                  
* prohibited without the written consent of the copyright holder.            
*                                                                            
*  Project             : Bali
*  File name   $Logfile: /func_test_platf2/bali/si/table/SDT.java $
*       Last save $Date: 2003/12/24 00:53:05 $
*             $Revision: 1.1.1.1 $
*          Last $Author: mikeseven $
*  Creation Date       : 28/10/98
*  First Author        : S.A.F.A. van den Heuvel
*                        NatLab - Eindhoven
* 
*                                                                          
*  Change history      :
*  ---------------
*
* $Log: SDT.java,v $
* Revision 1.1.1.1  2003/12/24 00:53:05  mikeseven
* Initial release of the CVS repository for MPEG-4 Systems code
* after the code was corrupted due to reorganization and human 
* errors.
*
* Revision 1.1  2003/04/03 04:29:26  mikeseven
* Initial release of Player3D and MPEG-J for all MPEG-4 reference softwares players with samples.
* For installation, follow instructions in docs/Player3D_setup.doc.
*
* Many thanks to Prashanth Gujjeti and Viswanathan Swaminathan from Sun Microsystems.
*
* -- Mikael Bourges-Sevenier (Mindego)
*
* Revision 1.1  2002/08/23 16:47:25  pg138550
* Changes for MPEG-J. NOT FINAL
*
* Revision 1.1  2000/06/06 18:43:12  alavilli
* After SISF integration into the CORE4 code
*
* Revision 1.1.2.1  1999/07/09 14:17:03  depetris
* Aggiornati i file per lo sviluppo e il rilascio a MPEG-J.
* Integra la versione 2.5 del core.
*
* Revision 1.2  1999-04-15 09:53:18+02  vdheuvel
* Files changed to hook the system up to the TM demultipler and the
* TM hardware. Changes include, swapping the java.util.Date class
* for the bali.tm.Date class (to prevent using 64 bit floats). Changing
* the HW sect filt used by bali.hw.SectFilt.
*
* Revision 1.1  1999-02-08 17:26:54+01  vdheuvel
* 39 march 1999, start of archive.
* Middleware packages DAVIC API implementation.
*
 * 
 * 1     9-11-98 13:59 Vdheuvel
 * Creation
*/
package bali.table;
import java.util.Vector;
/*DATE_NORMAL*/ import  java.util.Date;
//DATE_TM*/     import  bali.tm.Date;
import java.util.Enumeration;
import bali.hw.sectfilt.HwSection;
import bali.support.Debug;
import bali.support.TablePart;
import org.davic.mpeg.dvb.DvbTransportStream;

/****************************************
This class parses the filtered SDT tables and allows access to the different SDT components.
*/
public class SDT
    extends SITable
{
	/** */ public final static int PID = 0x11; 
	/** */ public final static int TABLE_ID_ACTUAL         = 0x42; 
	/** */ public final static int TABLE_ID_OTHER          = 0x46; 

	/***************************************
	*/
	public SDT( SDT sdt, DvbTransportStream ts, Date filterTime )
	{
		super( sdt, new SITableInfo( sdt._ti, ts, filterTime ) );
		_descriptorList = sdt._descriptorList;
		_ServiceList    = sdt._ServiceList;
		
	    Enumeration e = _ServiceList.elements();
		while ( e.hasMoreElements() )
		{
			((Serv)e.nextElement())._ti = _ti;
		}
	}

	/***************************************
	*/
	public SDT( HwSection[] arSec, DvbTransportStream ts, Date filterTime )
	    throws InvalidTableException
    {
    	super( arSec, new SITableInfo( arSec[0].version_number()
    			                 , arSec[0].table_id() == TABLE_ID_ACTUAL
        	                     , ts
            	                 , filterTime )
             );
        try
        {
            /*DBG*/ Debug.msg( Debug.TABLE, "parse SDT");
            	
            _ServiceList = new Vector();
    
    		// parse all sections of this table.
    		for ( int i=0; i<_arSec.length; i++ )
    		{	// parse section
    			int   ServiceLength = _arSec[i].getLength() - 4;	// - CRC
    	        int   index = 11;
        	    int   sl;
    
    	        while ( index < ServiceLength )
        	    {
    	            sl = 5 + _arSec[i].get12bits( index + 3 );
                    //DBG*/ System.out.println( "SDT: Service found of length " + sl + "/" + (ServiceLength - index) );
        	        _ServiceList.addElement( new Serv( new TablePart( _arSec[i], index, sl ), _ti ) );
            	    index += sl;
    	        }
            }// end parsing sections
        } 
        catch ( IndexOutOfBoundsException e)
        {
            throw new InvalidTableException();
        }

    }

    /*****************
    Returns the TableIdentifier of this SDT.
    */
    public TableIdentifier getTableIdentifier()
    {
        synchronized( this )
        {
            if ( _tableIdentifier == null )
            {
                int[] arInt = new int[2];
                arInt[0] = getOriginalNetworkId();
                arInt[1] = getTransportStreamId();
                _tableIdentifier = new TableIdentifier( TableIdentifier.SDT, arInt );
            }
        }
        return _tableIdentifier;
    }

	/***************************************
	Return the ts_id of this SDT.
	*/
    public int getTransportStreamId()
    {
       return (int)_arSec[0].table_id_extension();
    }

	/***************************************
	Return the or_nw_id of this NIT.
	*/
    public int getOriginalNetworkId()
    {
        return _arSec[0].get16bits( 8 );
    }

	/***************************************
	Returns the services part of this SDT.
	*/
	public Serv[] getServices()
	{
		Serv[] ar = new Serv[ _ServiceList.size() ];
		_ServiceList.copyInto( ar );
		return ar;
	}

	/***************************************
	Returns the indicated service in this SDT (null when not present).
	*/
	public Serv getService( int service_id)
	{
		Enumeration e = _ServiceList.elements();
		Serv res = null;
		boolean found = false;
		
		while ( e.hasMoreElements() && !found )
		{
			res = (Serv)e.nextElement();
			found = ( res.getServiceId() == service_id );
		}
		
		if ( found )
			return res;
		else
			return null;
	}

	/////////////////////////////// PRIVATE //////////////////////////
	private DescriptorList 	_descriptorList;
	private Vector 			_ServiceList;
}
