/*
* Copyright(c) Philips Consumer Electronics B.V. 1998                         
* All rights reserved. Reproduction in whole or in part is                  
* prohibited without the written consent of the copyright holder.            
*                                                                            
*  Project             : Bali
*  File name   $Logfile: /func_test_platf2/bali/si/table/SITable.java $
*       Last save $Date: 2003/12/24 00:53:05 $
*             $Revision: 1.1.1.1 $
*          Last $Author: mikeseven $
*  Creation Date       : 28/10/98
*  First Author        : S.A.F.A. van den Heuvel
*                        NatLab - Eindhoven
* 
*                                                                          
*  Change history      :
*  ---------------
*
* $Log: SITable.java,v $
* Revision 1.1.1.1  2003/12/24 00:53:05  mikeseven
* Initial release of the CVS repository for MPEG-4 Systems code
* after the code was corrupted due to reorganization and human 
* errors.
*
* Revision 1.1  2003/04/03 04:29:26  mikeseven
* Initial release of Player3D and MPEG-J for all MPEG-4 reference softwares players with samples.
* For installation, follow instructions in docs/Player3D_setup.doc.
*
* Many thanks to Prashanth Gujjeti and Viswanathan Swaminathan from Sun Microsystems.
*
* -- Mikael Bourges-Sevenier (Mindego)
*
* Revision 1.1  2002/08/23 16:47:25  pg138550
* Changes for MPEG-J. NOT FINAL
*
* Revision 1.1  2000/06/06 18:43:12  alavilli
* After SISF integration into the CORE4 code
*
* Revision 1.1.2.1  1999/07/09 14:17:03  depetris
* Aggiornati i file per lo sviluppo e il rilascio a MPEG-J.
* Integra la versione 2.5 del core.
*
* Revision 1.1  1999-02-19 17:08:44+01  vdheuvel
* 39 march 1999, start of archive.
* Middleware packages DAVIC API implementation.
*
 * 
 * 2     10-11-98 15:54 Vdheuvel
 * Bug fix in TDT and TOT parsing. Changed SITable to support empty
 * constructor.
 * 
 * 1     9-11-98 13:59 Vdheuvel
 * Creation
*/

package bali.table;
import  bali.hw.sectfilt.HwSection;
import  bali.support.SeriousError;
import  bali.support.Debug;
import  bali.support.Assert;

/******************************
The base class of all tables.
*/
public abstract class SITable
    extends FilteredSITable
{
	/*****************
	*/
	SITable( SITable table, SITableInfo ti )
    {
        super( ti );
        /*DBG*/ Debug.msg( Debug.TABLE, "SITable copy constructor" );
      	_arSec = table._arSec;
      	_tableIdentifier = null;
	}

	/*****************
	*/
	SITable( HwSection[] arSec, SITableInfo ti )
	    throws InvalidTableException
    {
        super( ti );
        try
        {        
            /*DBG*/ Debug.msg( Debug.TABLE, "SITable.constructor" );
          	_arSec = arSec;
          	_tableIdentifier = null;
        }
      	catch( IndexOutOfBoundsException e )
        {
            throw new InvalidTableException( );
        }
	}
	
	/*****************
	Check if the offered table is another representation of this table
	(same version number and both actual/other).
	*/
	public boolean isSameTable( SITable table )
	{
	    /*DBG*/Assert.assert( table!= null, "table == null " );
		return (  (_ti.getVersion()==table._ti.getVersion()) 
		       && (_ti.fromActual()==table._ti.fromActual())
		       ); 
	}

    /*****************
    Returns the TableIdentifier of this SITable.
    */
    public abstract TableIdentifier getTableIdentifier();
    
	//////////////// PRIVATE ///////////////
	protected HwSection[]     _arSec;
	protected TableIdentifier _tableIdentifier;
}