/*
* Copyright(c) Philips Consumer Electronics B.V. 1998                         
* All rights reserved. Reproduction in whole or in part is                  
* prohibited without the written consent of the copyright holder.            
*                                                                            
*  Project             : Bali
*  File name   $Logfile: /func_test_platf2/bali/si/table/Serv.java $
*       Last save $Date: 2003/12/24 00:53:05 $
*             $Revision: 1.1.1.1 $
*          Last $Author: mikeseven $
*  Creation Date       : 28/10/98
*  First Author        : S.A.F.A. van den Heuvel
*                        NatLab - Eindhoven
* 
*                                                                          
*  Change history      :
*  ---------------
*
* $Log: Serv.java,v $
* Revision 1.1.1.1  2003/12/24 00:53:05  mikeseven
* Initial release of the CVS repository for MPEG-4 Systems code
* after the code was corrupted due to reorganization and human 
* errors.
*
* Revision 1.1  2003/04/03 04:29:27  mikeseven
* Initial release of Player3D and MPEG-J for all MPEG-4 reference softwares players with samples.
* For installation, follow instructions in docs/Player3D_setup.doc.
*
* Many thanks to Prashanth Gujjeti and Viswanathan Swaminathan from Sun Microsystems.
*
* -- Mikael Bourges-Sevenier (Mindego)
*
* Revision 1.1  2002/08/23 16:47:25  pg138550
* Changes for MPEG-J. NOT FINAL
*
* Revision 1.1  2000/06/06 18:43:12  alavilli
* After SISF integration into the CORE4 code
*
* Revision 1.1.2.1  1999/07/09 14:17:04  depetris
* Aggiornati i file per lo sviluppo e il rilascio a MPEG-J.
* Integra la versione 2.5 del core.
*
* Revision 1.2  1999-04-23 14:04:41+02  vdheuvel
* Fixed bug in calculation of service provider name.
*
* Revision 1.1  1998-12-08 13:44:52+01  vdheuvel
* 39 march 1999, start of archive.
* Middleware packages DAVIC API implementation.
*
 * 
 * 1     9-11-98 13:59 Vdheuvel
 * Creation
*/
package bali.table;

import java.util.Vector;
import java.util.Enumeration;
import bali.support.Util;
import bali.support.Debug;
import bali.support.TablePart;

/******************************
This class represents the service part of an SDT.
*/
public class Serv
    extends FilteredSITable
{
	/** */ public final static int SERVICE_DESCRIPTOR_TAG  = 0x48; 

	/******************************
	*/
	Serv( TablePart tp, SITableInfo ti)
	{
	    super( ti );
		_tablePart = tp;

        int descLen = _tablePart.get12bits( 3 );
        _descriptorList = new DescriptorList( new TablePart( _tablePart, 5, descLen ) );
		parseStrings();
	}

	/******************************
	*/
	public int getServiceId()
	{
		return _tablePart.get16bits( 0 );
	}
	
	/***************************************
	Return the service name of this service. Only
	the latin ascii table is supported.
	*/
	public String getServiceName()
	{
		return _serviceName.getString();
	}

	/***************************************
	*/
	public String getShortServiceName()
	{
		return _serviceName.getShortString();
	}

	/***************************************
	*/
	public String getServiceProvider()
	{
		return _serviceProviderName.getString();
	}

	/***************************************
	*/
	public String getShortServiceProvider()
	{
		return _serviceProviderName.getShortString();
	}


	/******************************
	*/
	public boolean getEitScheduleFlag()
	{
		int res;

		res = _tablePart.getByte( 2 )&0x02;
		res = res >> 1;

		return ( res == 1 );
	}

	/******************************
	*/
	public boolean getEitPresentFollowingFlag()
	{
		int res;

		res = _tablePart.getByte( 2 )&0x01;

		return (res==1)
		;
	}

	/******************************
	*/
	public int getRunningStatus()
	{
		int res;

		res = _tablePart.getByte( 3 )&0xE0;
		res = res >> 5;
		
		return res;
	}

	/******************************
	*/
	public boolean getFreeCaMode()
	{
	int res;

		res = _tablePart.getByte( 3 )&0x10;
		res = res >> 4;
		
		return (res==1);
	}


	/******************************
	*/
	public DescriptorList getDescriptorList()
    {
		return _descriptorList;
    }

    //////////////////////// PRIVATE ///////////////////////////////////
	private DescriptorList 	_descriptorList;
	private TablePart 		_tablePart;
	private DvbString       _serviceName;
	private DvbString       _serviceProviderName;
	
	/************************
	*/
	private void parseStrings()
	{
        //DBG*/ System.out.println("Serv.parseStrings" );
        Descriptor d; 
        d = _descriptorList.getDescriptor( SERVICE_DESCRIPTOR_TAG );

        if ( d != null )
        {
			int spnStart  = 4;
			int spnLength = d.get8bits( 3 ) ;
			int snStart   = spnStart + spnLength + 1;
			int snLength  = d.get8bits( snStart-1 ) ;
			
			_serviceName = new DvbString( d.getContent(), snStart, snStart+snLength );
			_serviceProviderName = new DvbString( d.getContent(), spnStart, spnStart+spnLength );
		}                
		else
		{
			_serviceName = new DvbString( );
			_serviceProviderName = new DvbString( );
		}
	}
}

