/*
* Copyright(c) Philips Consumer Electronics B.V. 1998                         
* All rights reserved. Reproduction in whole or in part is                  
* prohibited without the written consent of the copyright holder.            
*                                                                            
*  Project             : Bali
*  Module              : bali 
*  File name   $Logfile: /func_test_platf2/bali/si/table/TableIdentifier.java $
*       Last save $Date: 2003/12/24 00:53:05 $
*             $Revision: 1.1.1.1 $
*          Last $Author: mikeseven $
*  Creation Date       : 28/10/98
*  First Author        : S.A.F.A. van den Heuvel
*                        NatLab - Eindhoven
* 
*                                                                          
*  Change history      :
*  ---------------
*
* $Log: TableIdentifier.java,v $
* Revision 1.1.1.1  2003/12/24 00:53:05  mikeseven
* Initial release of the CVS repository for MPEG-4 Systems code
* after the code was corrupted due to reorganization and human 
* errors.
*
* Revision 1.1  2003/04/03 04:29:27  mikeseven
* Initial release of Player3D and MPEG-J for all MPEG-4 reference softwares players with samples.
* For installation, follow instructions in docs/Player3D_setup.doc.
*
* Many thanks to Prashanth Gujjeti and Viswanathan Swaminathan from Sun Microsystems.
*
* -- Mikael Bourges-Sevenier (Mindego)
*
* Revision 1.1  2002/08/23 16:47:25  pg138550
* Changes for MPEG-J. NOT FINAL
*
* Revision 1.1  2000/06/06 18:43:13  alavilli
* After SISF integration into the CORE4 code
*
* Revision 1.1.2.1  1999/07/09 14:17:06  depetris
* Aggiornati i file per lo sviluppo e il rilascio a MPEG-J.
* Integra la versione 2.5 del core.
*
* Revision 1.3  1999-05-05 14:13:05+02  vdheuvel
* Changes required after first Bouquet tests
*
* Revision 1.2  1999-04-15 09:53:18+02  vdheuvel
* Files changed to hook the system up to the TM demultipler and the
* TM hardware. Changes include, swapping the java.util.Date class
* for the bali.tm.Date class (to prevent using 64 bit floats). Changing
* the HW sect filt used by bali.hw.SectFilt.
*
* Revision 1.1  1999-02-19 17:56:26+01  vdheuvel
* 39 march 1999, start of archive.
* Middleware packages DAVIC API implementation.
*
 * 
 * 2     10-11-98 10:16 Vdheuvel
 * Added  SERVICE to the tableType list
 * 
 * 1     9-11-98 13:59 Vdheuvel
 * Creation
*/

package bali.table;
import bali.support.Assert;
import bali.support.Debug;
/*DATE_NORMAL*/ import  java.util.Date;
//DATE_TM*/     import  bali.tm.Date;
import java.lang.Object;

/*****************
This class holds holds the identiers inditifiing a DVB-SI element.
A class is defined by its tableType and an array of identifiers (id[]). 
The identifier array has a different meaning depending on the 
tableType. Allowed values for the identifiers are -1..MAX_INT. The value
-1 can be used as a wildcard.
The ordning of the identifiers is 0: most generic (original_transport_stream_id).
<p> Tables
<p> NIT     nw_id
<p> BAT     bouquet_id
<p> SDT     or_nw_id, ts_id
<p> EITpf   or_nw_id, ts_id, service_id
<p> EITsch  or_nw_id, ts_id, service_id
<p> TDT     or_nw_id, ts_id
<p> TOT     or_nw_id, ts_id
<p> RST     or_nw_id, ts_id
<p> PAT     or_nw_id, ts_id
<p> PMT     or_nw_id, ts_id, service_id
<p> TSDT    or_nw_id, ts_id
<p> SubTables 
<p> SERVICE   or_nw_id, ts_id, service_id
<p> EITsch_ST or_nw_id, ts_id, service_id, period_no
*/
public class TableIdentifier
    extends Object
{
	public final static int NIT       =  0;
	public final static int BAT       =  1;
	public final static int SDT       =  2;
	public final static int EITpf     =  3;
	public final static int EITsch    =  4;
	public final static int TDT       =  5;
	public final static int TOT       =  6;
	public final static int RST       =  7;
	public final static int PAT       =  8;
	public final static int PMT       =  9;
	public final static int TSDT      = 10;
	/** Max table value*/public final static int MAX_TABLE = 10; 
	public final static int SERVICE   = 11;
	public final static int EITsch_ST = 12;
	/** Max value*/public final static int MAX_VALUE = 12; 
	
	/************************
	Return the table_type of this table.
	*/
	public int getTableType()
	{
		return _type;
	}
	
	/************************
	Return the identifiers indentifying this table.
	*/
	public int[] getIdentifiers()
	{
		return _id;
	}
	
	/************************
	*/
	public TableIdentifier( int tableType, int[] id)
	{
		_type = tableType;
		_id   = new int[id.length];
		System.arraycopy( id, 0, _id, 0, id.length );
		check();
	}

	/************************
	*/
	public TableIdentifier( int tableType, int id0)
	{
		_type = tableType;
		_id   = new int[1];
		_id[0]= id0;
		check();
	}

	/************************
	*/
	public TableIdentifier( int tableType, int id0, int id1)
	{
		_type = tableType;
		_id   = new int[2];
		_id[0]= id0;
		_id[1]= id1;
		check();
	}

	/************************
	*/
	public TableIdentifier( int tableType, int id0, int id1, int id2)
	{
		_type = tableType;
		_id   = new int[3];
		_id[0]= id0;
		_id[1]= id1;
		_id[2]= id2;
		check();
	}

	/************************
	*/
	public TableIdentifier( int tableType, int id0, int id1, int id2, int id3)
	{
		_type = tableType;
		_id   = new int[4];
		_id[0]= id0;
		_id[1]= id1;
		_id[2]= id2;
		_id[3]= id3;
		check();
	}

	/************************
	*/
    public boolean equals( java.lang.Object obj)
    {
  		/*DBG*/ Debug.msg( Debug.TABLE, "TableIdentifier.equals");
        boolean res;
        TableIdentifier ti = (TableIdentifier)obj;
        res = ( (_type == ti._type) && (_id.length == ti._id.length) );
        int i=0;
        while( res && i<_id.length )
        {
            res = ( _id[i]==ti._id[i] );
            i++;
        }
        return res;
    }

	/***************
	Returns true when this cache element is a wildcard.
	*/
    public boolean holdsWildCard()
    {
        boolean res = false;
        int i = 0;
        while( !res && i<_id.length )
        {
            res = (_id[i]==-1);
            i++;
        }
        return res;
    }

	/////////// PRIVATE /////////////////////
	private int _type;
	private int _id[];
	
	private void check()
	{
  		/*DBG*/ Assert.assert( _id.length >= 0, "check TableIdentifier length >=0 "+_id.length );
        switch( _type )
        {
        case NIT:
      		/*DBG*/ Assert.assert( _id.length == 1, "NIT Length != 1" );
	        break;
        case SDT:
      		/*DBG*/ Assert.assert( _id.length == 2, "SDT Length != 2" );
            break;
        case TDT:
      		/*DBG*/ Assert.assert( _id.length == 2, "TDT Length != 2" );
            break;
        case TOT:
      		/*DBG*/ Assert.assert( _id.length == 2, "TOT Length != 2" );
            break;
        case TSDT:
      		/*DBG*/ Assert.assert( _id.length == 2, "TSDT Length != 2" );
            break;
        case RST:
      		/*DBG*/ Assert.assert( _id.length == 2, "RST Length != 2" );
            break;
        case PAT:
      		/*DBG*/ Assert.assert( _id.length == 2, "PAT Length != 2" );
            break;
        case PMT:
      		/*DBG*/ Assert.assert( _id.length == 3, "PMT Length != 3" );
            break;
        case EITpf:
      		/*DBG*/ Assert.assert( _id.length == 3, "EITpf Length != 3" );
            break;
        case EITsch:
      		/*DBG*/ Assert.assert( _id.length == 3, "EITsch Length != 3" );
            break;
        case EITsch_ST:
      		/*DBG*/ Assert.assert( _id.length == 4, "EITsch_STLength != 4" );
            break;
        case SERVICE:
      		/*DBG*/ Assert.assert( _id.length == 3, "SERVICE Length != 3" );
            break;
        default:
            /*DBG*/Assert.assert( true, "incorrect table" );
        }
        
        for ( int i=0; i<_id.length; i++ )
        {
            /*DBG*/Assert.assert( _id[i] >= -1, "value < -1"  );
        }
	}
}