package bali.testframe.sender;

import java.io.*;
import java.util.Vector;
import bali.hw.sectfilt.HwSection;
import bali.testframe.ServerBase;

/*********************************************************************
Creates a object that sends a section repeatingly between a start
and end time at a given frequency.
*/
public class MultiRepeater
  implements java.lang.Runnable
{
  private ServerBase s;
  private long       NextTime;
  private long       EndTime;
  private long       Interval;
  private int        pid;
  //private bali.network.Section section;
  private Vector sections;
  private int sectionNr;
  private boolean				loop;
	private FileInputStream fileIn;
//	private StreamTokenizer in;

	private void loadSections(String fn)
	{
		boolean contin = true;

		try {
			fileIn = new FileInputStream(fn);

			while (contin) {
				byte header[] = new byte[3];
				byte rest[];
				byte total[];
				int sl;

				if (fileIn.available() < header.length) {
					break;
				}
				fileIn.read(header);
				sl = (header[1] & 0x0f) * 256;
				if (header[2] < 0) {
					sl += 256 + header[2];
				} else {
					sl += header[2];
				}
				if (fileIn.available() < sl) {
					break;
				}

				rest = new byte[sl];
				fileIn.read(rest);
				total = new byte[sl+header.length];

				System.arraycopy( header, 0, total, 0, header.length );
				System.arraycopy( rest, 0, total, header.length, sl );

//				System.out.println( "Read section of length " + sl );

				HwSection section = new HwSection(total);
				sections.addElement(section);
			}
		} catch (IOException e) {
		    System.out.println(this.getClass()+" : IO exception "+fn);
		}
	}

  /******************************************************************
  Create a new Repeater sending section sec to the Server server
  */
  public MultiRepeater	( ServerBase         server
	   				, double                        frequency
			   		, int	                        pid
			   		, String fn
			   		)
  {
    NextTime  = 0;
    Interval  = (long)( 1/frequency *1000 );
    s         = server;
    this.pid  = pid;
    sections  = new Vector();
	loop      = true;
	sectionNr = 0;
	loadSections(fn);
//    System.out.println("Repeater : freq : " + frequency);
  }
	  
  /*****************************************************************
  Create a new Repeater sending sections sec to the Server server 
  between start and endtime.
  */
  public MultiRepeater ( ServerBase	server
    			   , long						starttime 
		           , long						endtime
				   , double						frequency
				   , int						pid
				   , String						fn
				   )
  {
    NextTime  = starttime*1000;
    EndTime   = endtime*1000;
    Interval  = (long)( 1/frequency *1000 );
    s         = server;
    this.pid  = pid;
    sections  = new Vector();
	loop      = false;
	sectionNr = 0;
	loadSections(fn);
//    System.out.println("Repeater : start: " + starttime +" end: " + endtime +
//                                 " freq : " + frequency);
  }
	
  /****************************************************************
  Start the repeater
  */
  public void run()
  {
    long time = System.currentTimeMillis();
    NextTime += time;
    EndTime  += time;
    try
    {
      // waiting to start of proces
      time = System.currentTimeMillis();
      while( (loop || EndTime > time) && sections.size() > 0 )
      {
     	if ( NextTime > time )
        { Thread.currentThread().sleep( NextTime - time );
        }
		for (sectionNr = 0; sectionNr < sections.size(); sectionNr++) {
			s.send( pid, (HwSection) sections.elementAt(sectionNr) );
		}
		NextTime += Interval;
        time = System.currentTimeMillis();
      }
      System.out.println("Repeater ready");
    }
    catch( java.lang.InterruptedException e )
    {
      System.out.println("exception " + e);
    }
  }
}
