// StP -- created on Fri Oct 09 09:23:18 1998 for sommitcc@kerne from system DVBSI

/***********************************************************************************
 * project Java for Sommit :                                                       *
 * API SI-EPG                                                                      *
 * Author : Jean-Philippe Barazeur                                                 *
 * CCETT  : 09/10/98                                                               *
 ***********************************************************************************/


 /***********************************************************************************/
 /*                                                                                 */
 /*                          API SI - EPG                                           */
 /*                                                                                 */
 /***********************************************************************************/

 /*
 APISiEPg is the main class of SI/EPG API.
 APISiEPG provides Navigation resources to easily handle Events and Services
 4 steps have to be executed, to naviguate in better conditions:

 1 - Call Create(), to create the database
 2 - Call InitDatabase(x) method, to retrieve all information on services and Events,
     present into the Stream attached to the current Database
 3 - Call ListServices(x) method, to construct the Service Navigation list.
     Now you can handle services, using Next/PrevService() methods.
 4 - Call an Event List method, to construct the Event Nzvigation list. Now you can
     easily handle Events(and their services) using Next / PrevEvent methods()

*/
package sommit.APISiEpg;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;
import java.util.Date;

//import org.davic.net.dvb.si;   API SI - Davic 1.3
import sommit.davic13.*;
// stp class definition 8
public class APISiEpg
{
// stp class members
    Vector _Alarms;                       // list of Alarm objects
    int _CurrentEventIndex;               // index on _EventNavigation List
    int _CurrentServiceIndex;             // index on _ServiceNavigation List
    SIDatabase[] _Database;               // Database array (each element is a stream)
    Vector _EventNavigation;              // List of common Events (depends on User's choices)
    Hashtable _FollowingEventsTable;      // Link between Services and Following Events
                                          // Key : Integer - ServiceId / Value : Integer - EventId
    String _Language;                     // User Language preferences
    Vector _ListOfBouquets;               // list of SIBouquets objects
    Hashtable _ListOfEvents;              // Link between a SIEvent object and it EventId
                                          // Key : Integer - EventId / Value : SIEvent
    Hashtable _ListOfServices;            // Link between a SIService object and it ServiceId
                                          // Key : Integer - ServiceId / Value : SIService
    Vector _PrefServices;                 // List of User favourites services ( Vector of ServiceId)
    Hashtable _PresentEventsTable;        // Link between Services and Present Events
                                          // Key : Integer - ServiceId / Value : Integer - EventId
    Hashtable _ScheduleEventsTable;       // Link between Services and Schedule Events
                                          // Key : Integer - ServiceId / Value : Vector of EventId(Integer)
    Vector _ServiceNavigation;            // List of common Services (depends on User's choices)
    SIDatabase _StreamGuide;              // Current Database ( a specific element of _Database)

    SITime    _Time;                      // Actual TDT value

    boolean   _ViewOnlyPrefServices = false; // for preferences ....
    int       _DeltaGMT = 0;              // difference between GMT hour and Local Hour


    private final static short MaxServices = 150; // Constants to dim Events an Services Lists !!!
    private final static short MaxEvents   = 150;

/***********************************************************************************************/
/*                                       Methods                                               */
/***********************************************************************************************/

/*---------------------------------------------------------------------------------------------*/
/* Method       :   AffectCurrentEventIndex                                                    */
/* Description  :   Sets _CurrentEventIndex                                                    */
/* Parameters   :   EventIndex(int) : the new value                                            */
/* Return Value :   false if index is out of range / true otherwise                            */
/*---------------------------------------------------------------------------------------------*/

public boolean AffectCurrentEventIndex(int EventIndex)
{
    boolean result = false;

    if ((EventIndex < GetNbEvents()) && (EventIndex>=0))
    {
      _CurrentEventIndex = EventIndex;

      // when _CurrentEventIndex is modified, _CurrentServiceIdex must be modified too
      String EventServId   = (String)_EventNavigation.elementAt(_CurrentEventIndex);
      SIEvent anEvent   = (SIEvent)_ListOfEvents.get(EventServId);
      Integer ServiceId =  new Integer(anEvent.getServiceId());
      int ServiceIndex  =  _ServiceNavigation.indexOf(ServiceId);

      result = AffectCurrentServiceIndex(ServiceIndex);
    }

    return result;
}

/*---------------------------------------------------------------------------------------------*/
/* Method       :   AffectCurrentServiceIndex                                                  */
/* Description  :   Sets _CurrentServiceIndex                                                  */
/* Parameters   :   ServiceIndex(int) : the new value                                          */
/* Return Value :   false if index is out of range / true otherwise                            */
/*---------------------------------------------------------------------------------------------*/

public boolean AffectCurrentServiceIndex(int ServiceIndex)
{

   boolean result = false;

    if ((ServiceIndex < GetNbServices()) && (ServiceIndex>=0))
    {
      _CurrentServiceIndex = ServiceIndex;
      result = true;
    }

    return result;
}

/*---------------------------------------------------------------------------------------------*/
/* Method       :   Create                                                                     */
/* Description  :   Create the Database                                                        */
/* Parameters   :   N/A                                                                        */
/* Return Value :   false if no Database /           true otherwise                            */
/*---------------------------------------------------------------------------------------------*/

public boolean Create()
{
    boolean result;

    _Database = SIDatabase.getSIDatabase();

    if (_Database != null)
    {
       _StreamGuide = _Database[0];
      result = true;
      if (_Alarms == null)
      {
        _Alarms = new Vector(5,1);  // initial capacity 5 - increment capacity 1
      }

      _Language = "FRA";  // Why not ????

      try
     {
        _Time = _StreamGuide.retrieveSITimeFromTDT();


     }
     catch(RetrieveFailedException e)
     {
        _Time = new SITime();

     }


    }
    else
    {
      result = false;
    }

    return result;

}
/*---------------------------------------------------------------------------------------------*/
/* Method       :   CreateAlarm                                                                */
/* Description  :   Create an Alarm object and Add it to the _Alarms vector                    */
/* Parameters   :   TypeOfAlarm(int) : 0 - Record mode / 1 - Notify mode                       */
/*                                     2 - Record & Notify mode                                */
/* Return Value :   N/A                                                                        */
/*---------------------------------------------------------------------------------------------*/

public void CreateAlarm(int TypeOfAlarm)
{

    String EventServId =(String)_EventNavigation.elementAt(_CurrentEventIndex);
    SIEvent anEvent =(SIEvent)_ListOfEvents.get(EventServId);

    Alarm anAlarm = new Alarm(anEvent,TypeOfAlarm);
    anEvent.addListener(anAlarm);

    _Alarms.addElement(anAlarm);

}
/*---------------------------------------------------------------------------------------------*/
/* Method       :   CreateAlarm                                                                */
/* Description  :   Create an Alarm object and Add it to the _Alarms vector                    */
/* Parameters   :   EventIndex(int) : the Event Index, to be attached to Alarm Object          */
/*                  TypeOfAlarm(int) : see after                                               */
/* Return Value :   false if index is out of range / true otherwise                            */
/*---------------------------------------------------------------------------------------------*/

public boolean CreateAlarm(int EventIndex, int TypeOfAlarm)
{
    boolean result = AffectCurrentEventIndex(EventIndex);

    if (result == true)
    {
       CreateAlarm(TypeOfAlarm);
    }

    return result;

}

/*---------------------------------------------------------------------------------------------*/
/* Method       :   FilterEventsByParentalRating                                               */
/* Description  :   Filter _EventNavigation by parental rating, and by Services (Favourites Services) */
/* Parameters   :   Min Age: the legal minimum age                                             */
/* Return Value :   number of elements in _EventNavigation                                     */
/*---------------------------------------------------------------------------------------------*/

public int FilterEventsByParentalRating(int MinAge,boolean LessThan)
{
    // first step, Events by Content ...
    Integer RefServiceId,ServiceId;
    String  EventServId;
    Integer EventId;
    SIEvent anEvent;

    if ((_EventNavigation == null) || (_EventNavigation.isEmpty()))
    {
        return 0;
    }

    Vector myVector = new Vector(10,1);
    Enumeration EventIterator;
    short[] DescTab = new short[1];
    DescTab[0] = DescriptorTag.PARENTAL_RATING;
    SIIterator ParentalIterator;

    EventIterator = _EventNavigation.elements();

    while (EventIterator.hasMoreElements())
    {
        int result;
        EventServId = (String)EventIterator.nextElement();
        anEvent =(SIEvent)_ListOfEvents.get(EventServId);
        try
        {
            ParentalIterator = anEvent.retrieveDescriptors(DescTab);
            // only one ParentalIterator element must be present
            // modifications SIIterator il faut enlever _Enum
            if (ParentalIterator != null)
            {
              result = DescParental.GetRating((Descriptor)ParentalIterator._Enum.nextElement(),_Language);
              EventId = new Integer(anEvent.getEventId());
              ServiceId = new Integer(anEvent.getServiceId());
              EventServId = EventId.toString() + " " + ServiceId.toString();

              if ((result <= MinAge) && (LessThan == true))
              {
                myVector.addElement(EventServId);
              }

              if ((result >= MinAge) && (LessThan == false))
              {
                myVector.addElement(EventServId);
              }
            }
        }
        catch (RetrieveFailedException e)
        {
        }
    }

    _EventNavigation = null;
    _EventNavigation = new Vector(10,1);

    // second step, events by pref services, if necessary
    if (_PrefServices != null)
    {
        for (int i = 0; i < _PrefServices.size(); i++)
        {
            RefServiceId = (Integer)_PrefServices.elementAt(i);
            EventIterator = myVector.elements();

            while(EventIterator.hasMoreElements())
            {
                EventServId = (String)EventIterator.nextElement();
                anEvent = (SIEvent)_ListOfEvents.get(EventServId);
                if (anEvent.getServiceId() == RefServiceId.intValue())
                {
                    _EventNavigation.addElement(EventServId);
                    myVector.removeElement(EventServId);
                }
            }
        }
     }

     if (_ViewOnlyPrefServices == false)
     {
        EventIterator = myVector.elements();

        while (EventIterator.hasMoreElements())
        {
            EventServId = (String)EventIterator.nextElement();
            _EventNavigation.addElement(EventServId);
        }

     }

    int NbEvents = GetNbEvents();

    if (NbEvents>0);
    {
        AffectCurrentEventIndex(0);
    }

    return NbEvents;

}


/*---------------------------------------------------------------------------------------------*/
/* Method       :   FlushDatabase                                                              */
/* Description  :   Remove all the SIObjects from the current  Database                        */
/* Parameters   :   none                                                                       */
/* Return Value :   false if _StreamGuide is already null / true otherwise                     */
/*---------------------------------------------------------------------------------------------*/
public boolean FlushDatabase()
{
    boolean result = false;

    if (_StreamGuide != null)
    {

        // only _Alarms is safed

        _CurrentEventIndex = 0;
        _CurrentServiceIndex = 0;
        _EventNavigation = null;
        _ServiceNavigation = null;
        _ListOfBouquets = null;
        _ListOfEvents = null;
        _ListOfServices = null;
        _FollowingEventsTable = null;
        _PresentEventsTable = null;
        _ScheduleEventsTable = null;
        result = true;
    }

    return result;

}

/*---------------------------------------------------------------------------------------------*/
/* Method       :   GetAlarmDate(int AlarmIndex)                                               */
/* Description  :   returns the Local Start Time, for the selected alarm                       */
/* Parameters   :   AlarmIndex(between 0 and GetNbAlarms()                                     */
/* Return Value :   Date value (locale string) or null if an error occurs                      */
/*---------------------------------------------------------------------------------------------*/

public String GetAlarmDate(int AlarmIndex)
{
    String result = null;

    if ((AlarmIndex>=0) && (AlarmIndex<GetNbAlarms()))
    {
        Alarm anAlarm =(Alarm)_Alarms.elementAt(AlarmIndex);
        
        Date myDate = new Date(anAlarm.GetStartDate());
        myDate.setHours(myDate.getHours()+_DeltaGMT);
        result = myDate.toGMTString();
    }

    return result;
}

/*---------------------------------------------------------------------------------------------*/
/* Method       :   GetAudioPID()                                                              */
/* Description  :   returns the AudioPID for the current service                               */
/* Parameters   :   none                                                                       */
/* Return Value :   PID Value / 0 if no AudioPID was found                                     */
/*---------------------------------------------------------------------------------------------*/

public int GetAudioPID()
{
    int result = 0;

    SIIterator PMTIterator;
    Integer ServiceId =(Integer) _ServiceNavigation.elementAt(_CurrentServiceIndex);
    SIService aService = (SIService) _ListOfServices.get(ServiceId);
    PMTElementaryStream ES;
    boolean hasAudioES = false;

    try
    {
      // retrieve the PMT Elementary streams in  the actual TS
      PMTIterator = aService.retrievePMTElementaryStreams(null);
      // null because we don't need any information on PMT Descriptors

      // modifications SIIterator : il faut enlever le _Enum
      while ((hasAudioES == false) && (PMTIterator._Enum.hasMoreElements()))
      {
          ES = (PMTElementaryStream)PMTIterator._Enum.nextElement();

          if ((ES.getStreamType() == PMTStreamType.MPEG2_AUDIO) ||
              (ES.getStreamType() == PMTStreamType.MPEG1_AUDIO ))
              {
                hasAudioES = true;
                result = ES.getElementaryPID();
              }
      }

    }
    catch (RetrieveFailedException e)
    {
        // troubles in retrieving PMT Elementary streams
        result = 0;
    }

    return result;
}

/*---------------------------------------------------------------------------------------------*/
/* Method       :   GetAudioPID                                                                */
/* Description  :   returns the AudioPID for the selected service                              */
/* Parameters   :   ServiceIndex(int) : the selected service on _ServiceNavigation             */
/* Return Value :   PID Value / 0 if no AudioPID was found                                     */
/*---------------------------------------------------------------------------------------------*/

public int GetAudioPID(int ServiceIndex)
{
    boolean res = AffectCurrentServiceIndex(ServiceIndex);
    int result = 0;

    if (res)
    {
        result = GetAudioPID();
    }

    return result;
}

/*---------------------------------------------------------------------------------------------*/
/* Method       :   GetBouquetName                                                             */
/* Description  :   returns the Bouquet Name for the selected Bouquet                          */
/* Parameters   :   BouquetIndex(int) : the selected bouquet on _ListOfBouquets                */
/* Return Value :   Bouquet Name / or null if an error occurs (no descriptor or Bad index)     */
/*---------------------------------------------------------------------------------------------*/

public String GetBouquetName(int BouquetIndex)
{
    String result = null;

    if ((BouquetIndex<GetNbBouquets()) && (BouquetIndex>=0))
    {

        SIBouquet myBouquet = (SIBouquet)_ListOfBouquets.elementAt(BouquetIndex);

        result = myBouquet.getBouquetName();

    }

    return result;

}

/*---------------------------------------------------------------------------------------------*/
/* Method       :   GetComponentInfo                                                           */
/* Description  :   returns Information on Component Descriptor for the current Event          */
/* Parameters   :   none                                                                       */
/* Return Value :   Component results / or null if an error occurs (no Component descriptor    */
/*---------------------------------------------------------------------------------------------*/

public String GetComponentInfo()
{
   String result = null;

   String EventServId = (String)_EventNavigation.elementAt(_CurrentEventIndex);

   SIEvent anEvent = (SIEvent)_ListOfEvents.get(EventServId);
   SIIterator CompIterator;
   short[] descTab = new short[1];
   descTab[0] = DescriptorTag.COMPONENT;

   try
   {
      CompIterator = anEvent.retrieveDescriptors(descTab);
      // only one Descriptor must compose CompIterator

      // modifications SIIterator il faut enlever _Enum
      if (CompIterator != null)
      {
         result = DescComponent.GetComponent((Descriptor)CompIterator._Enum.nextElement());
      }
   }
   catch (RetrieveFailedException e)
   {
      result = null;
   }

   return result;
}

/*---------------------------------------------------------------------------------------------*/
/* Method       :   GetComponentInfo                                                           */
/* Description  :   returns Information on Component Descriptor for the selected Event         */
/* Parameters   :   EventIndex (int) Selected Event in _EventNavigation table                  */
/* Return Value :   Component results / or null if an error occurs (no Component descriptor)   */
/*---------------------------------------------------------------------------------------------*/

public String GetComponentInfo(int EventIndex)
{
    String result = null;

    boolean res = AffectCurrentEventIndex(EventIndex);

    if (res)
    {
       result = GetComponentInfo();
    }

    return result;
}

/*---------------------------------------------------------------------------------------------*/
/* Method       :   GetContent                                                                 */
/* Description  :   returns Information on Content Descriptor for the current Event            */
/* Parameters   :   none                                                                       */
/* Return Value :   Content results / or null if an error occurs (no Content descriptor)       */
/*---------------------------------------------------------------------------------------------*/

public String GetContent()
{
   String result = null;

   String EventServId = (String)_EventNavigation.elementAt(_CurrentEventIndex);

   SIEvent anEvent = (SIEvent)_ListOfEvents.get(EventServId);
   SIIterator CompIterator;
   short[] descTab = new short[1];
   descTab[0] = DescriptorTag.CONTENT;

   try
   {
      CompIterator = anEvent.retrieveDescriptors(descTab);
      // only one Descriptor must compose CompIterator
      // modifications SIIterator il faut enlever _Enum
      if (CompIterator != null)
      {
         result = DescContent.GetContent((Descriptor)CompIterator._Enum.nextElement());
      }
   }
   catch (RetrieveFailedException e)
   {
      result = null;
   }

   return result;
}

/*---------------------------------------------------------------------------------------------*/
/* Method       :   GetContent                                                                 */
/* Description  :   returns Information on Content Descriptor for the selected Event           */
/* Parameters   :   EventIndex(int) Selected event on EventNavigation table                    */
/* Return Value :   Content results / or null if an error occurs (no Content or bad index)     */
/*---------------------------------------------------------------------------------------------*/

public String GetContent(int EventIndex)
{
    String result = null;

    boolean res = AffectCurrentEventIndex(EventIndex);

    if (res)
    {
       result = GetContent();
    }

    return result;
}


/*---------------------------------------------------------------------------------------------*/
/* Method       :   GetCurrentEventIndex                                                       */
/* Description  :   returns _CurrentEventIndex value                                           */
/* Parameters   :   none                                                                       */
/* Return Value :   _CurrentEventIndex value (int)                                             */
/*---------------------------------------------------------------------------------------------*/

public int GetCurrentEventIndex()
{
    return _CurrentEventIndex;
}

/*---------------------------------------------------------------------------------------------*/
/* Method       :   GetCurrentServiceIndex                                                     */
/* Description  :   returns _CurrentServiceIndex value                                         */
/* Parameters   :   none                                                                       */
/* Return Value :   _CurrentServiceIndex value (int)                                           */
/*---------------------------------------------------------------------------------------------*/

public int GetCurrentServiceIndex()
{
    return _CurrentServiceIndex;
}

/*---------------------------------------------------------------------------------------------*/
/* Method       :   GetCurrentStreamId                                                         */
/* Description  :   returns StreamId field for the current service                             */
/* Parameters   :   none                                                                       */
/* Return Value :   StreamId value (int)                                                       */
/*---------------------------------------------------------------------------------------------*/

public int GetCurrentStreamId()
{
    Integer ServiceId = (Integer)_ServiceNavigation.elementAt(_CurrentServiceIndex);
    SIService aService = (SIService)_ListOfServices.get(ServiceId);

    return aService.getTransportStreamId();
}


/*---------------------------------------------------------------------------------------------*/
/* Method       :   GetDate                                                                    */
/* Description  :   returns _Time Date                                                         */
/* Parameters   :   none                                                                       */
/* Return Value :   a Date object ( LOCAL DATE !!!!! NO GMT DATE)                              */
/*---------------------------------------------------------------------------------------------*/

public Date GetDate()
{

    if (_Time != null)
    {
        Date myDate = new Date(_Time.getUTCTime().toGMTString());
        myDate.setHours(myDate.getHours()+_DeltaGMT);
        return myDate;
    }
    else
    {
        return null;
    }
}

/*---------------------------------------------------------------------------------------------*/
/* Method       :   GetDeltaGMT                                                                */
/* Description  :   returns _DeltaGMT                                                          */
/* Parameters   :   none                                                                       */
/* Return Value :   an int : local hour = GMT hour + DeltaGMT                                  */
/*---------------------------------------------------------------------------------------------*/

public int GetDeltaGMT()
{
    return _DeltaGMT;
}

/*---------------------------------------------------------------------------------------------*/
/* Method       :   GetDuration                                                                */
/* Description  :   returns Duration time field for the current Event                          */
/* Parameters   :   none                                                                       */
/* Return Value :   a String which contains the Duration Time ("00:15")                        */
/*---------------------------------------------------------------------------------------------*/

public String GetDuration()
{

    String EventServId = (String)_EventNavigation.elementAt(_CurrentEventIndex);
    SIEvent anEvent = (SIEvent)_ListOfEvents.get(EventServId);

    long milli = anEvent.getDuration();
    int seconds = (int)(milli / 1000);
    Date Duration = new Date();

    Duration.setHours(0);
    Duration.setMinutes(0);
    Duration.setSeconds(seconds);

    String result = Duration.getHours()+":"+Duration.getMinutes();

    return result;

}

/*---------------------------------------------------------------------------------------------*/
/* Method       :   GetDuration                                                                */
/* Description  :   returns Duration time field for the selected Event                         */
/* Parameters   :   EventIndex (int) Event in _EventNavigation Table                           */
/* Return Value :   a String which contains the Duration Time ("00:15")                        */
/*---------------------------------------------------------------------------------------------*/

public String GetDuration(int EventIndex)
{
    String result = null;

    boolean res = AffectCurrentEventIndex(EventIndex);

    if (res)
    {
        result = GetDuration();
    }

    return result;

}

/*---------------------------------------------------------------------------------------------*/
/* Method       :   GetEITPresentFollowingFlag                                                 */
/* Description  :   returns the EIT P/F Flag field of the current Service                      */
/* Parameters   :   none                                                                       */
/* Return Value :   returns false if no Present / Following Event, linked to the current       */
/*                  service, is present in the stream, and true otherwise                      */
/*---------------------------------------------------------------------------------------------*/

public boolean GetEITPresentFollowingFlag()
{
   boolean flag = false;

   Integer ServiceId = (Integer)_ServiceNavigation.elementAt(_CurrentServiceIndex);
   SIService aService = (SIService)_ListOfServices.get(ServiceId);

   try
   {
       flag = aService.retrieveEITPresentFollowingFlag();
   }
   catch (RetrieveFailedException e)
   {
      flag = false;
   }

   return flag;

}

/*---------------------------------------------------------------------------------------------*/
/* Method       :   GetEITPresentFollowingFlag                                                 */
/* Description  :   returns the EIT P/F Flag field of the selected Service                     */
/* Parameters   :   ServiceIndex(int) : Service in _ServiceNavigation table                    */
/* Return Value :   returns false if no Present / Following Event, linked to the selected      */
/*                  service, is present in the stream, and true otherwise                      */
/*                  false if ServiceIndex is out of range !!                                   */
/*---------------------------------------------------------------------------------------------*/

public boolean GetEITPresentFollowingFlag(int ServiceIndex)
{

    boolean result = false;

    boolean res = AffectCurrentServiceIndex(ServiceIndex);

    if (res)
    {
        result = GetEITPresentFollowingFlag();
    }

    return result;
}

/*---------------------------------------------------------------------------------------------*/
/* Method       :   GetEITScheduleFlag                                                         */
/* Description  :   returns the EIT S Flag field of the current Service                        */
/* Parameters   :   none                                                                       */
/* Return Value :   returns false if no Schedule Event, linked to the current                  */
/*                  service, is present in the stream, and true otherwise                      */
/*---------------------------------------------------------------------------------------------*/

public boolean GetEITScheduleFlag()
{
   boolean flag = false;

   Integer ServiceId = (Integer)_ServiceNavigation.elementAt(_CurrentServiceIndex);
   SIService aService = (SIService)_ListOfServices.get(ServiceId);

   try
   {
       flag = aService.retrieveEITScheduleFlag();
   }
   catch (RetrieveFailedException e)
   {
      flag = false;
   }

   return flag;
}

/*---------------------------------------------------------------------------------------------*/
/* Method       :   GetEITScheduleFlag                                                         */
/* Description  :   returns the EIT S Flag field of the current Service                        */
/* Parameters   :   ServiceIndex(int) : Service in _ServiceNavigation table                    */
/* Return Value :   returns false if no Schedule Event, linked to the current                  */
/*                  service, is present in the stream, and true otherwise                      */
/*                  false if ServiceIndex is out of range !!                                   */
/*---------------------------------------------------------------------------------------------*/

public boolean GetEITScheduleFlag(int ServiceIndex)
{
    boolean result = false;

    boolean res = AffectCurrentServiceIndex(ServiceIndex);

    if (res)
    {
        result = GetEITScheduleFlag();
    }

    return result;
}


/*---------------------------------------------------------------------------------------------*/
/* Method       :   GetEventId                                                                 */
/* Description  :   returns the EventId field of the current Event                             */
/* Parameters   :   none                                                                       */
/* Return Value :   returns an int value                                                       */
/*---------------------------------------------------------------------------------------------*/

public int GetEventId()
{
    String EventServId = (String)_EventNavigation.elementAt(_CurrentEventIndex);
    SIEvent anEvent = (SIEvent)_ListOfEvents.get(EventServId);

    return anEvent.getEventId();
}

/*---------------------------------------------------------------------------------------------*/
/* Method       :   GetEventId                                                                 */
/* Description  :   returns the EventId field of the selected Event                            */
/* Parameters   :   EventIndex(int) Event in _EvnetNavigation Table                            */
/* Return Value :   returns the EventId value / returns 0 if bad Index                         */
/*---------------------------------------------------------------------------------------------*/

public int GetEventId(int EventIndex)
{
    int result = 0;
    boolean res = AffectCurrentEventIndex(EventIndex);

    if (res)
    {
       result = GetEventId();
    }

    return result;
}


/*---------------------------------------------------------------------------------------------*/
/* Method       :   GetEventName                                                               */
/* Description  :   returns the Event Name of the current Event                                */
/* Parameters   :   none                                                                       */
/* Return Value :   returns a String which contains the Event Name                             */
/*---------------------------------------------------------------------------------------------*/

public String GetEventName()
{
    String EventServId = (String)_EventNavigation.elementAt(_CurrentEventIndex);
    SIEvent anEvent = (SIEvent)_ListOfEvents.get(EventServId);

    return anEvent.getEventName();

}

/*---------------------------------------------------------------------------------------------*/
/* Method       :   GetEventName                                                               */
/* Description  :   returns the Event Name of the Selected Event                               */
/* Parameters   :   EventIndex(int) , on _EventNavigation table                                */
/* Return Value :   returns a String which contains the Event Name(may be null if bad index)   */
/*---------------------------------------------------------------------------------------------*/

public String GetEventName(int EventIndex)
{
    String result = null;
    boolean res = AffectCurrentEventIndex(EventIndex);

    if (res)
    {
        result = GetEventName();
    }

    return result;
}
/*---------------------------------------------------------------------------------------------*/
/* Method       :   GetExtendedInfo                                                            */
/* Description  :   returns information on Extended descriptor, for the current Event          */
/* Parameters   :   none                                                                       */
/* Return Value :   returns a string which contains info                                       */
/*                  returns null if no Extended Descriptor was found                           */
/*---------------------------------------------------------------------------------------------*/

public String GetExtendedInfo()
{

    String result = null;

    String EventServId = (String)_EventNavigation.elementAt(_CurrentEventIndex);
    SIEvent anEvent = (SIEvent)_ListOfEvents.get(EventServId);
    SIIterator ExtendedIterator;
    short[] DescTab = new short[1];
    DescTab[0] = DescriptorTag.EXTENDED_EVENT;

    try
    {
        ExtendedIterator = anEvent.retrieveDescriptors(DescTab);
        // only one ExtendedIterator element must be present
        // modifications SIIterator il faut enlever _Enum
        if (ExtendedIterator != null)
        {
           result = DescExtendedDescription.GetDescription((Descriptor)ExtendedIterator._Enum.nextElement());
        }
    }
    catch (RetrieveFailedException e)
    {
      result = null;
    }

    return result;

}

/*---------------------------------------------------------------------------------------------*/
/* Method       :   GetExtendedInfo                                                            */
/* Description  :   returns information on Extended descriptor, for the selected Event         */
/* Parameters   :   EventIndex(int) Event in _EventNavigation table                            */
/* Return Value :   returns a string which contains info                                       */
/*                  returns null if no Extended Descriptor was found or bad index              */
/*---------------------------------------------------------------------------------------------*/

public String GetExtendedInfo(int EventIndex)
{
    String result = null;
    boolean res = AffectCurrentEventIndex(EventIndex);

    if (res)
    {
      result = GetExtendedInfo();
    }

    return result;
}

/*---------------------------------------------------------------------------------------------*/
/* Method       :   GetLanguage                                                                */
/* Description  :   returns _Language value                                                    */
/* Parameters   :   none                                                                       */
/* Return Value :   returns a string which contains _Language preferences                      */
/*---------------------------------------------------------------------------------------------*/

public String GetLanguage()
{
    return _Language;
}

/*---------------------------------------------------------------------------------------------*/
/* Method       :   GetNbAlarms                                                                */
/* Description  :   returns the number of elements for _Alarms                                 */
/* Parameters   :   none                                                                       */
/* Return Value :   number of Alarms objects                                                   */
/*---------------------------------------------------------------------------------------------*/

public int GetNbAlarms()
{
    return _Alarms.size();
}

/*---------------------------------------------------------------------------------------------*/
/* Method       :   GetNbBouquets                                                              */
/* Description  :   returns the number of elements for _ListOfBouquets                         */
/* Parameters   :   none                                                                       */
/* Return Value :   number of SIBouquet objects                                                */
/*---------------------------------------------------------------------------------------------*/

public int GetNbBouquets()
{
    return _ListOfBouquets.size();
}

/*---------------------------------------------------------------------------------------------*/
/* Method       :   GetNbEvents                                                                */
/* Description  :   returns the number of elements for _EventNavigation                        */
/* Parameters   :   none                                                                       */
/* Return Value :   number of Events stored in _EventNavigation                                */
/*---------------------------------------------------------------------------------------------*/

public int GetNbEvents()
{
    return _EventNavigation.size();
}

/*---------------------------------------------------------------------------------------------*/
/* Method       :   GetNbServices                                                              */
/* Description  :   returns the number of elements for _ServiceNavigation                      */
/* Parameters   :   none                                                                       */
/* Return Value :   number of Services stored in _ServiceNavigation                            */
/*---------------------------------------------------------------------------------------------*/

public int GetNbServices()
{
    return _ServiceNavigation.size();
}

/*---------------------------------------------------------------------------------------------*/
/* Method       :   GetNbStreams                                                               */
/* Description  :   returns the number of elements for _Database                               */
/* Parameters   :   none                                                                       */
/* Return Value :   number of streams                                                          */
/*---------------------------------------------------------------------------------------------*/

public int GetNbStreams()
{
    return _Database.length;
}

/*---------------------------------------------------------------------------------------------*/
/* Method       :   GetParentalRating                                                          */
/* Description  :   returns the minimum Age to follow the current Event                        */
/* Parameters   :   none                                                                       */
/* Return Value :   returns an int value, which corresponds to the minimum age (7->7 year olds)*/
/*                  returns 0 if there's no ParentalRatong Descriptor                          */
/*---------------------------------------------------------------------------------------------*/

public int GetParentalRating()
{
    int result = 0;

    String  EventServId = (String)_EventNavigation.elementAt(_CurrentEventIndex);
    SIEvent anEvent = (SIEvent)_ListOfEvents.get(EventServId);
    SIIterator ParentalIterator;
    short[] DescTab = new short[1];
    DescTab[0] = DescriptorTag.PARENTAL_RATING;

    try
    {
        ParentalIterator = anEvent.retrieveDescriptors(DescTab);
        // only one ParentalIterator element must be present
        // modifications SIIterator il faut enlever _Enum
        if (ParentalIterator != null)
        {
           result = DescParental.GetRating((Descriptor)ParentalIterator._Enum.nextElement(),_Language);
        }
    }
    catch (RetrieveFailedException e)
    {
      result = 0;
    }

    return result;
}

/*---------------------------------------------------------------------------------------------*/
/* Method       :   GetParentalRating                                                          */
/* Description  :   returns the minimum Age to follow the selected Event                       */
/* Parameters   :   EventIndex(int) Event in _EventNavigation                                  */
/* Return Value :   returns an int value, which corresponds to the minimum age (7->7 year olds)*/
/*                  returns 0 if there's no ParentalRating Descriptor / or Bad index           */
/*---------------------------------------------------------------------------------------------*/

public int GetParentalRating(int EventIndex)
{
    int result = 0;
    boolean res = AffectCurrentEventIndex(EventIndex);

    if (res)
    {
      result = GetParentalRating();
    }

    return result;
}


/*---------------------------------------------------------------------------------------------*/
/* Method       :   GetPrefServices()                                                          */
/* Description  :   returns the _PrefService vector  (Ordered User favourites services)        */
/* Parameters   :   none                                                                       */
/* Return Value :   returns _PrefServices ( may be null)                                       */
/*---------------------------------------------------------------------------------------------*/

public Vector GetPrefServices()
{
    return _PrefServices;
}


/*---------------------------------------------------------------------------------------------*/
/* Method       :   GetRunningStatus()                                                         */
/* Description  :   returns the Running Status  for the currernt Event                         */
/* Parameters   :   none                                                                       */
/* Return Value :   returns a string which contains Running status                             */
/*---------------------------------------------------------------------------------------------*/

public String GetRunningStatus()
{
    String result   = null;
    String EventServId = (String)_EventNavigation.elementAt(_CurrentEventIndex);
    SIEvent anEvent = (SIEvent)_ListOfEvents.get(EventServId);

    byte status = anEvent.getRunningStatus();

    switch (status)
    {
        case 0 : result = "undefined";
                 break;
        case 1 : result = "not running";
                 break;
        case 2 : result = "starts in few seconds";
                 break;
        case 3 : result = "pausing";
                 break;
        case 4 : result = "running";
                 break;
        default: result = "reserved for future use";
                 break;
    }
    return result;
}

/*---------------------------------------------------------------------------------------------*/
/* Method       :   GetRunningStatus()                                                         */
/* Description  :   returns the Running Status  for the selected Event                         */
/* Parameters   :   EventIndex(int) Event in _EventNavigation table                            */
/* Return Value :   returns a string which contains Running status or null for bad index       */
/*---------------------------------------------------------------------------------------------*/

public String GetRunningStatus(int EventIndex)
{
    String result = null;
    boolean res = AffectCurrentEventIndex(EventIndex);

    if (res)
    {
        result = GetRunningStatus();
    }

    return result;
}


/*---------------------------------------------------------------------------------------------*/
/* Method       :   GetServiceId()                                                             */
/* Description  :   returns the ServiceId value  for the current Service                       */
/* Parameters   :   none                                                                       */
/* Return Value :   returns an int                                                             */
/*---------------------------------------------------------------------------------------------*/

public int GetServiceId()
{
     Integer myInt = (Integer)_ServiceNavigation.elementAt(_CurrentServiceIndex);

     return myInt.intValue();
}

/*---------------------------------------------------------------------------------------------*/
/* Method       :   GetServiceId()                                                             */
/* Description  :   returns the ServiceId value  for the selected Service                      */
/* Parameters   :   ServiceIndex(int) : index on _ServiceNavigation                            */
/* Return Value :   returns an int or 0 for a bad index                                        */
/*---------------------------------------------------------------------------------------------*/

public int GetServiceId(int ServiceIndex)
{
    int result = 0;
    boolean res = AffectCurrentServiceIndex(ServiceIndex);

    if (res)
    {
        result = GetServiceId();
    }

    return result;
}


/*---------------------------------------------------------------------------------------------*/
/* Method       :   GetServiceName()                                                           */
/* Description  :   returns the ServiceName value  for the current Service                     */
/* Parameters   :   none                                                                       */
/* Return Value :   returns a  String which contains the Service Name                          */
/*---------------------------------------------------------------------------------------------*/

public String GetServiceName()
{

   Integer ServiceId = (Integer)_ServiceNavigation.elementAt(_CurrentServiceIndex);
   SIService aService = (SIService)_ListOfServices.get(ServiceId);

   return aService.getServiceName();

}

/*---------------------------------------------------------------------------------------------*/
/* Method       :   GetServiceName()                                                           */
/* Description  :   returns the ServiceName value  for the selected Service                    */
/* Parameters   :   Service Index - index on _ServiceNavigation table                          */
/* Return Value :   returns a String which contains the Service Name(may be null for bad index)*/
/*---------------------------------------------------------------------------------------------*/

public String GetServiceName(int ServiceIndex)
{
   String result = null;
   boolean res = AffectCurrentServiceIndex(ServiceIndex);

   if (res)
   {
      result = GetServiceName();
   }

   return result;
}

/*---------------------------------------------------------------------------------------------*/
/* Method       :   GetServiceProviderName()                                                   */
/* Description  :   returns the ServiceProviderName value  for the current Service             */
/* Parameters   :   none                                                                       */
/* Return Value :   returns a  String which contains the Service Provider Name                 */
/*---------------------------------------------------------------------------------------------*/

public String GetServiceProviderName()
{
   Integer ServiceId = (Integer)_ServiceNavigation.elementAt(_CurrentServiceIndex);
   SIService aService = (SIService)_ListOfServices.get(ServiceId);

   return aService.getServiceProviderName();
}

/*---------------------------------------------------------------------------------------------*/
/* Method       :   GetServiceProviderName()                                                   */
/* Description  :   returns the ServiceProviderName value  for the selected Service            */
/* Parameters   :   ServiceIndex:(int) index on _ServiceNavigation table                       */
/* Return Value :   returns a  String which contains the Service Provider Name                 */
/*---------------------------------------------------------------------------------------------*/

public String GetServiceProviderName(int ServiceIndex)
{

   String result = null;
   boolean res = AffectCurrentServiceIndex(ServiceIndex);

   if (res)
   {
      result = GetServiceProviderName();
   }

   return result;
}


/*---------------------------------------------------------------------------------------------*/
/* Method       :   GetServiceTypeString()                                                           */
/* Description  :   returns the Service Type for the current Service                           */
/* Parameters   :   none                                                                       */
/* Return Value :   returns a  String which contains the Service Type (ex "FM Radio" )         */
/*                  return value may be null if no Service Descriptor was found                */
/*---------------------------------------------------------------------------------------------*/

public String GetServiceTypeString()
{

    String result = null;
    Integer ServiceId = (Integer)_ServiceNavigation.elementAt(_CurrentServiceIndex);
    SIService aService= (SIService)_ListOfServices.get(ServiceId);

    SIIterator ServIterator;
    short[] DescTab = new short[1];

    DescTab[0] = DescriptorTag.SERVICE;

    try
    {
        ServIterator = aService.retrieveDescriptors(DescTab);
        // only one element must compose ServIterator
        // modifications SIIterator il faut enlever _Enum

        if (ServIterator != null)
        {
            result = DescService.GetType((Descriptor)ServIterator._Enum.nextElement());
        }

    }
    catch (RetrieveFailedException e)
    {
        result = null;
    }

    return result;

}

/*---------------------------------------------------------------------------------------------*/
/* Method       :   GetServiceTypeString()                                                           */
/* Description  :   returns the Service Type for the selected Service                          */
/* Parameters   :   ServiceIndex(int) index on _ServiceNavigation table                        */
/* Return Value :   returns a  String which contains the Service Type (ex "FM Radio" )         */
/*                  return value may be null if no Service Descriptor was found, or bad index  */
/*---------------------------------------------------------------------------------------------*/

public String GetServiceTypeString(int ServiceIndex)
{
    String result = null;
    boolean res = AffectCurrentServiceIndex(ServiceIndex);

    if (res)
    {
        result = GetServiceTypeString();
    }

    return result;

}

/*---------------------------------------------------------------------------------------------*/
/* Method       :   GetServiceType()                                                           */
/* Description  :   returns the Service Type for the current Service                           */
/* Parameters   :   none                                                                       */
/* Return Value :   returns an int which contains the Service Type                             */
/*                  return value may be -1 if no Service Descriptor was found                  */
/*---------------------------------------------------------------------------------------------*/

public int GetServiceType()
{
     GetServiceTypeString();
     return (int)DescService._type;
}

/*---------------------------------------------------------------------------------------------*/
/* Method       :   GetServiceType()                                                           */
/* Description  :   returns the Service Type for the selected Service                          */
/* Parameters   :   ServiceIndex ....                                                          */
/* Return Value :   returns an int which contains the Service Type                             */
/*                  return value may be -1 if no Service Descriptor was found                  */
/*---------------------------------------------------------------------------------------------*/

public int GetServiceType(int ServiceIndex)
{
     if (GetServiceTypeString(ServiceIndex) != null)
     {
        return (int)DescService._type;
     }
     else
     {
        return -1;
     }
}


/*---------------------------------------------------------------------------------------------*/
/* Method       :   GetShortDescription()                                                      */
/* Description  :   returns the Short Description  for the current Event                       */
/* Parameters   :   none                                                                       */
/* Return Value :   returns a  String which contains the Event Short Description               */
/*---------------------------------------------------------------------------------------------*/

public String GetShortDescription()
{
    String EventServId = (String)_EventNavigation.elementAt(_CurrentEventIndex);
    SIEvent anEvent = (SIEvent)_ListOfEvents.get(EventServId);

    return anEvent.getShortDescription();
}

/*---------------------------------------------------------------------------------------------*/
/* Method       :   GetShortDescription()                                                      */
/* Description  :   returns the Short Description  for the selected Event                      */
/* Parameters   :   Event Index(int) on _EventNavigation table                                 */
/* Return Value :   returns a  String which contains the Event Short Description               */
/*---------------------------------------------------------------------------------------------*/

public String GetShortDescription(int EventIndex)
{

  String result = null;
  boolean res = AffectCurrentEventIndex(EventIndex);

  if (res)
  {
      result = GetShortDescription();
  }

  return result;

}


/*---------------------------------------------------------------------------------------------*/
/* Method       :   GetStartTime                                                                    */
/* Description  :   returns Start time  field for the current Event                            */
/* Parameters   :   none                                                                       */
/* Return Value :   a String which contains the Time & Date ("jj/mm/aa 23:10")                                 */
/*---------------------------------------------------------------------------------------------*/

public String GetStartTime()
{
    String EventServId = (String)_EventNavigation.elementAt(_CurrentEventIndex);
    SIEvent anEvent = (SIEvent)_ListOfEvents.get(EventServId);

    Date myDate = new Date (anEvent.getStartTime().toGMTString());
    myDate.setHours(myDate.getHours()+_DeltaGMT); // for local time

    String result = myDate.toLocaleString();

    return result;
}




/*---------------------------------------------------------------------------------------------*/
/* Method       :   GetStartTime                                                               */
/* Description  :   returns Start time  field for the selected Event                           */
/* Parameters   :   EventIndex(int) Event in _EventNavigation table                            */
/* Return Value :   a String which contains the Date & Time ("23:10") or null if bad Index     */
/*---------------------------------------------------------------------------------------------*/

public String GetStartTime(int EventIndex)
{
    String result = null;

    boolean res = AffectCurrentEventIndex(EventIndex);

    if (res)
    {
        result = GetStartTime();
    }

    return result;
}
/*---------------------------------------------------------------------------------------------*/
/* Method       :   GetStopTime                                                                */
/* Description  :   returns Stop time   for the current Event                                  */
/* Parameters   :   none                                                                       */
/* Return Value :   a String which contains the Date & Time ("jj/mm/aa hh:mm")                                 */
/*---------------------------------------------------------------------------------------------*/

public String GetStopTime()
{
    String EventServId = (String)_EventNavigation.elementAt(_CurrentEventIndex);
    SIEvent anEvent = (SIEvent)_ListOfEvents.get(EventServId);

    Date myDate = new Date(anEvent.getStartTime().toGMTString());
    long milli = anEvent.getDuration();
    int minutes =(int)(milli / 60000);
    minutes = minutes + myDate.getMinutes();

    myDate.setMinutes(minutes);
    myDate.setHours(myDate.getHours()+_DeltaGMT);  // for a local time aspect

    String result = myDate.toLocaleString();

    return result;
}

/*---------------------------------------------------------------------------------------------*/
/* Method       :   GetStopTime                                                                */
/* Description  :   returns Stop time  field for the selected Event                            */
/* Parameters   :   EventIndex(int) Event in _EventNavigation table                            */
/* Return Value :   a String which contains the Date & Time (jj/mm/aa "23:10") or null if bad Index*/
/*---------------------------------------------------------------------------------------------*/

public String GetStopTime(int EventIndex)
{
    String result = null;

    boolean res = AffectCurrentEventIndex(EventIndex);

    if (res)
    {
        result = GetStopTime();
    }

    return result;
}

/*---------------------------------------------------------------------------------------------*/
/* Method       :   GetVideoPID()                                                              */
/* Description  :   returns the VideoPID for the current service                               */
/* Parameters   :   none                                                                       */
/* Return Value :   PID Value / 0 if no VideoPID was found                                     */
/*---------------------------------------------------------------------------------------------*/

public int GetVideoPID()
{
    int result = 0;

    SIIterator PMTIterator;
    Integer ServiceId =(Integer) _ServiceNavigation.elementAt(_CurrentServiceIndex);
    SIService aService = (SIService) _ListOfServices.get(ServiceId);
    PMTElementaryStream ES;
    boolean hasVideoES = false;

    try
    {
      // retrieve the PMT Elementary streams in  the actual TS
      PMTIterator = aService.retrievePMTElementaryStreams(null);
      // null because we don't need any information on PMT Descriptors
      // modifications SIIterator il faut enlever _Enum

      while ((hasVideoES == false) && (PMTIterator._Enum.hasMoreElements()))
      {
          ES = (PMTElementaryStream)PMTIterator._Enum.nextElement();

          if ((ES.getStreamType() == PMTStreamType.MPEG2_VIDEO) ||
              (ES.getStreamType() == PMTStreamType.MPEG1_VIDEO))
              {
                hasVideoES = true;
                result = ES.getElementaryPID();
              }
      }

    }
    catch (RetrieveFailedException e)
    {
        // troubles in retrieving PMT Elementary streams
        result = 0;
    }

    return result;
}

/*---------------------------------------------------------------------------------------------*/
/* Method       :   GetVideoPID                                                                */
/* Description  :   returns the VideoPID for the selected service                              */
/* Parameters   :   ServiceIndex(int) : the selected service on _ServiceNavigation             */
/* Return Value :   PID Value / 0 if no VideoPID was found                                     */
/*---------------------------------------------------------------------------------------------*/

public int GetVideoPID(int ServiceIndex)
{
    boolean res = AffectCurrentServiceIndex(ServiceIndex);
    int result = 0;

    if (res)
    {
        result = GetVideoPID();
    }

    return result;
}

/*---------------------------------------------------------------------------------------------*/
/* Method       :   GetViewOnlyPrefServices()                                                  */
/* Description  :   returns _ViewOnlyPrefServices                                              */
/* Parameters   :   none                                                                       */
/* Return Value :   true  / false                                                              */
/*---------------------------------------------------------------------------------------------*/

public boolean GetViewOnlyPrefServices()
{
    return _ViewOnlyPrefServices;
}

/*---------------------------------------------------------------------------------------------*/
/* Method       :   InitDatabase                                                               */
/* Description  :   Initializes the current Database, creating all lists and tabs              */
/* Parameters   :   DataIndex(int): the selected database                                      */
/* Return Value :   true if all is ok / false if an error occurs                               */
/*---------------------------------------------------------------------------------------------*/

public boolean InitDatabase(int DataIndex)
{
    SIService   aService;
    SIBouquet   aBouquet;
    SIEvent     anEvent;
    SIIterator  ServIterator;
    SIIterator  EventIterator;
    SIIterator  BouquetIterator;

    Integer     EventId;
    Integer     ServiceId;
    String      EventServId;
    short[]     DescTab = new short[1];
    DescTab[0] = -1;   // we want to retrieve all descriptors

    Date StartDate;   // start of EIT-S Extraction
    Date StopDate;    // end of EIT-S Extraction


    // First Step : Flush the current database
     if (_StreamGuide != null)
     {
        FlushDatabase();
     }

     // second Step, connect the Database....
     if ((DataIndex <0) || (DataIndex>= GetNbStreams()))
     {
        return false;
     }

     _StreamGuide = _Database[DataIndex];



     StartDate = new Date(_Time.getUTCTime().toGMTString());                  // by default StartDate is equals to _Time
     StopDate  = new Date(_Time.getUTCTime().toGMTString());


     int StartHour =(int)((StartDate.getHours()+3) / 3);
     StartHour = StartHour * 3;

     StartDate.setHours(StartHour);
     StopDate.setHours(StartHour+3);   // by default StopDate covers only one days (from StartScheduleDate+3)



     // third Step, retrieve all SIServices
     try
     {
         ServIterator = _StreamGuide.retrieveSIServices(-1,-1,-1,DescTab);
         _ListOfServices = new Hashtable(MaxServices);
         _ListOfEvents   = new Hashtable(10*MaxEvents);
         _FollowingEventsTable = new Hashtable(MaxEvents);
         _PresentEventsTable  = new Hashtable(MaxEvents);
         _ScheduleEventsTable = new Hashtable(2*MaxEvents);

         // modifications SIIterator , enlever _Enum
         while (ServIterator._Enum.hasMoreElements())
         {
             aService = (SIService)ServIterator._Enum.nextElement(); // recup of SIService
             ServiceId = new Integer(aService.getServiceId());
             _ListOfServices.put(ServiceId,aService);   // add to the List

             // fourth step, retrieve SIEvents

             try
             {
                // EIT - P
                anEvent = aService.retrievePresentSIEvent(DescTab);
                EventId = new Integer(anEvent.getEventId());
                EventServId = EventId.toString() + " " + ServiceId.toString();
                _ListOfEvents.put(EventServId,anEvent);
                _PresentEventsTable.put(ServiceId,EventServId);
             }
             catch(RetrieveFailedException e)
             {
               // no EIT P for the current Service
             }

             try
             {
                //EIT - F
                anEvent = aService.retrieveFollowingSIEvent(DescTab);
                EventId = new Integer(anEvent.getEventId());
                EventServId =EventId.toString() + " " + ServiceId.toString();
                _ListOfEvents.put(EventServId,anEvent);

                _FollowingEventsTable.put(ServiceId,EventServId);
             }
             catch(RetrieveFailedException e)
             {
               // no EIT F for the current service
             }

             try
             {
                //EIT - S


                EventIterator = aService.retrieveScheduledSIEvents(DescTab,StartDate,StopDate);


                Vector EITSVector = new Vector(5,1);

                  // modifications SIIterator , enlever _Enum

                  while(EventIterator._Enum.hasMoreElements())
                  {
                    anEvent = (SIEvent)EventIterator._Enum.nextElement();
                    EventId = new Integer(anEvent.getEventId());
                    EventServId=EventId.toString() + " " + ServiceId.toString();
                    _ListOfEvents.put(EventServId,anEvent);

                    EITSVector.addElement(EventServId);
                  }
                  _ScheduleEventsTable.put(ServiceId,EITSVector);
                  EITSVector = null;
                }

             catch(SIException e)
             {
                // no EIT S
             }
         } // end of while(servIterator.hasMoreElements();

     }
     catch (RetrieveFailedException e)
     {
         FlushDatabase();  // no services
         return false;
     }

     // fifth step : retrieve all bouquets
     try
     {
        _ListOfBouquets = new Vector(10,1);
        BouquetIterator = _StreamGuide.retrieveSIBouquets(-1,DescTab);

        // modifications SIIterator , enlever _Enum

        while (BouquetIterator._Enum.hasMoreElements())
        {
            aBouquet = (SIBouquet)BouquetIterator._Enum.nextElement();
            _ListOfBouquets.addElement(aBouquet);
        }
     }
     catch(RetrieveFailedException e)
     {
          _ListOfBouquets = null;
     }


     return true;

}

/*---------------------------------------------------------------------------------------------*/
/* Method       :   ListAlarms                                                                 */
/* Description  :   fills _EventNavigation with Events attached to Alarms objects              */
/* Parameters   :   none                                                                       */
/* Return Value :   number of elements in _EventNavigation                                     */
/*---------------------------------------------------------------------------------------------*/

public int ListAlarms()
{
    _EventNavigation = null;
    _EventNavigation = new Vector(10,1);

    Enumeration AlarmIterator = _Alarms.elements();
    Alarm anAlarm;
    SIEvent anEvent;

    while (AlarmIterator.hasMoreElements())
    {
       anAlarm = (Alarm)AlarmIterator.nextElement();
       anEvent = anAlarm.GetEvent();
       String EventServId = new String();
       EventServId+= anEvent.getEventId()+ " " + anEvent.getServiceId();

       if (_ListOfEvents.containsKey(EventServId) == false)
       {
          _ListOfEvents.put(EventServId,anEvent);
       }

       _EventNavigation.addElement(EventServId);
    }
    int NbEvents = GetNbEvents();

    if (NbEvents>0);
    {
        AffectCurrentEventIndex(0);
    }

    return NbEvents;
}


/*---------------------------------------------------------------------------------------------*/
/* Method       :   ListEventsAt                                                               */
/* Description  :   List all events for the selected Date                                      */
/*                  Be careful, EIT-S are removed,and then are retrieved (Start = Date - 3h /  */
/*                  Stop = Date + 3h). _ScheduleEventTable is reseted and then filled          */
/* Parameters   :   WhishedDate , a Date Object which corresponds to the selected Date         */
/*                  PAY ATTENTION, WhishedDate is a LOCAL TIME, not A GMT TIME                 */
/* Return Value :   number of elements in _EventNavigation                                     */
/*---------------------------------------------------------------------------------------------*/

public int ListEventsAt(Date WhishedDate)
{



    // first step delete all EIT_S;
    _EventNavigation = null;
    _EventNavigation = new Vector(10,1);

    Enumeration Enum = _ScheduleEventsTable.elements();
    Vector myEITSVect;
    String  EventServId;
    Integer EventId;
    Integer ServiceId;

    SIEvent anEvent;
    short[] DescTab = new short[1];
    DescTab[0] = -1;

    while (Enum.hasMoreElements())
    {
        myEITSVect =(Vector)Enum.nextElement();


        for (Enumeration EITSEnum = myEITSVect.elements(); EITSEnum.hasMoreElements();)
        {
            EventServId = (String)EITSEnum.nextElement();
            _ListOfEvents.remove(EventServId);

        }
    }
    _ScheduleEventsTable = null;
    _ScheduleEventsTable = new Hashtable(2*MaxEvents);

    // 2nd step, retrieve EIT-S at the whished Date
    // we retrieve EIT-S in Whished Hour +-3
    Date CopyOfWhishedDate = new Date(WhishedDate.toGMTString());

    CopyOfWhishedDate.setHours(WhishedDate.getHours()-_DeltaGMT);  // transfo LOCAL TIME to GMT Time

    Date StartDate = new Date(CopyOfWhishedDate.toGMTString());

    int StartHour = (int)((StartDate.getHours()) / 3);  // for
    StartHour = StartHour * 3;

    Date StopDate = new Date(CopyOfWhishedDate.toGMTString());
    StopDate.setMinutes(0);
    StopDate.setSeconds(0);

    if (StartHour >= 3)
    {
      StartHour = (StartHour -3);
      StopDate.setHours(StartHour+6);
    }
    else
    {
      StopDate.setHours(StartHour+3);
    }

    StartDate.setHours(StartHour);
    StartDate.setMinutes(0);
    StartDate.setSeconds(0);


    Enumeration EnumService = _ListOfServices.elements();
    SIService aService;

    while (EnumService.hasMoreElements())
    {
      aService = (SIService)EnumService.nextElement();

      try
      {
            //EIT - S

          SIIterator EventIterator = aService.retrieveScheduledSIEvents(DescTab,StartDate,StopDate);

          Vector EITSVector = new Vector(5,1);

          // modifications SIIterator , enlever _Enum

          while(EventIterator._Enum.hasMoreElements())
          {
             anEvent = (SIEvent)EventIterator._Enum.nextElement();
             EventId = new Integer(anEvent.getEventId());
             ServiceId = new Integer(anEvent.getServiceId());
             EventServId = EventId.toString()+" "+ServiceId.toString();
             _ListOfEvents.put(EventServId,anEvent);
             EITSVector.addElement(EventServId);

          }
           ServiceId = new Integer(aService.getServiceId());
          _ScheduleEventsTable.put(ServiceId,EITSVector);
          EITSVector = null;
        } // end try

        catch(SIException e)
        {
               // no EIT S
        }
       } // End while Services

       // fourth Step : fills _EventNavigation...

       Enumeration WhishedEvents = _ListOfEvents.elements();

       Vector VectResult = new Vector(10,1);

       while (WhishedEvents.hasMoreElements())
       {

           anEvent = (SIEvent)WhishedEvents.nextElement();

           try
           {
              StartDate = new Date(anEvent.getStartTime().toGMTString());

              StopDate =  new Date(anEvent.getStartTime().toGMTString());


              long milli = anEvent.getDuration();
              int min =(int)(milli / 60000);
              StopDate.setMinutes(StopDate.getMinutes()+min);
              if ((StartDate.before(CopyOfWhishedDate)) && (StopDate.after(CopyOfWhishedDate)))
              {
                VectResult.addElement(anEvent);
              }
            }
            catch (NullPointerException npe)
            {
                // an error might occurs on StopDate and StartDate
            }
       }


       // last step, sort by preferences ......
        if (_PrefServices != null)
        {
            for (int i=0; i<_PrefServices.size();i++)
          {
            Integer RefServiceId =(Integer)_PrefServices.elementAt(i);
            boolean hasCorrectServiceId = false;
            Enumeration myEnum = VectResult.elements();
            while ((myEnum.hasMoreElements()) && hasCorrectServiceId == false)
            {
                anEvent = (SIEvent)myEnum.nextElement();
                ServiceId = new Integer(anEvent.getServiceId());

                if (ServiceId.equals(RefServiceId))
                {
                    EventId = new Integer(anEvent.getEventId());

                    EventServId = EventId.toString() + " " + ServiceId.toString();
                    _EventNavigation.addElement(EventServId);
                    VectResult.removeElement(anEvent);
                   hasCorrectServiceId = true;
                }
            }// end while
          }  // end for

        } // end if

    // Other events are added after ....

    if (_ViewOnlyPrefServices == false)
   {
     Enumeration EventIterator = VectResult.elements();
     while (EventIterator.hasMoreElements())
     {
       anEvent = (SIEvent)EventIterator.nextElement();
       EventId = new Integer(anEvent.getEventId());
       ServiceId = new Integer(anEvent.getServiceId());
       EventServId = EventId.toString() + " " + ServiceId.toString();
       if (_EventNavigation.contains(EventServId) == false)
       {
          _EventNavigation.addElement(EventServId);
       }
     }
   }
    int NbEvents = GetNbEvents();

    if (NbEvents>0);
    {
        AffectCurrentEventIndex(0);
    }

    return NbEvents;

}

/*---------------------------------------------------------------------------------------------*/
/* Method       :   ListEventsAtAndContent                                                     */
/* Description  :   List all events for the selected Date and the selected Content             */
/*                  for further information, see ListEventsAt and ListEventsByContent          */
/* Parameters   :   myDate , a Date Object which corresponds to the selected Date              */
/*                  PAY ATTENTION: myDate is a LOCAL TIME, Not a GMT Time                      */
/*                  Content : int (content value)                                              */
/* Return Value :   number of elements in _EventNavigation                                     */
/*---------------------------------------------------------------------------------------------*/

public int ListEventsAtAndContent(Date myDate,int Content)
{

    ListEventsAt(myDate);
    Vector tempVector =new Vector(10,1);

    for (Enumeration e = _EventNavigation.elements(); e.hasMoreElements();)
    {
        String Key = (String)e.nextElement();
        SIEvent anEvent =(SIEvent)_ListOfEvents.get(Key);
        tempVector.addElement(anEvent);
    }

    if (tempVector.isEmpty())
    {
        return 0;
    }

    // first step, Events by Content ...
    Integer RefServiceId,ServiceId;
    String  EventServId;
    Integer EventId;
    SIEvent anEvent;


    _EventNavigation = null;
    _EventNavigation = new Vector(10,1);

    Vector myVector = new Vector(10,1);
    Enumeration EventIterator;

    EventIterator = tempVector.elements();
    short[] descTab = new short[1];
    descTab[0] = DescriptorTag.CONTENT;
    SIIterator CompIterator;
    int result;

    while (EventIterator.hasMoreElements())
    {
        anEvent = (SIEvent)EventIterator.nextElement();

        try
        {
            CompIterator = anEvent.retrieveDescriptors(descTab);
           // only one Descriptor must compose CompIterator
            // modifications SIIterator il faut enlever _Enum
          if (CompIterator != null)
          {
             result = DescContent.GetContentInt((Descriptor)CompIterator._Enum.nextElement());

             if ((Content & 0x000F) == 0)  // all Nibbles 2
             {
                result = (int)(result & 0x00F0);
             }

             if (result == Content)
             {
                EventId = new Integer(anEvent.getEventId());
                ServiceId = new Integer(anEvent.getServiceId());
                EventServId = EventId.toString() + " " + ServiceId.toString();
                myVector.addElement(EventServId);
             }

          }
        }
        catch(RetrieveFailedException rfe)
        {
        }
   }


    // second step, events by pref services, if necessary
    if (_PrefServices != null)
    {
        for (int i = 0; i < _PrefServices.size(); i++)
        {
            RefServiceId = (Integer)_PrefServices.elementAt(i);
            EventIterator = myVector.elements();

            while(EventIterator.hasMoreElements())
            {
                EventServId = (String)EventIterator.nextElement();
                anEvent = (SIEvent)_ListOfEvents.get(EventServId);
                if (anEvent.getServiceId() == RefServiceId.intValue())
                {
                    _EventNavigation.addElement(EventServId);
                    myVector.removeElement(EventServId);
                }
            }
        }
     }

     if (_ViewOnlyPrefServices == false)
     {
        EventIterator = myVector.elements();

        while (EventIterator.hasMoreElements())
        {
            EventServId = (String)EventIterator.nextElement();
            _EventNavigation.addElement(EventServId);
        }

     }

    int NbEvents = GetNbEvents();

    if (NbEvents>0);
    {
        AffectCurrentEventIndex(0);
    }

    return NbEvents;

}






/*---------------------------------------------------------------------------------------------*/
/* Method       :   ListEventsByContent                                                        */
/* Description  :   sort Event by content, and by Services (Favourites Services)               */
/* Parameters   :   Content(int) : Hexadecimal value : 0xUV with U = first Nibble,             */
/*                  V = second Nibble      , if V = 0 list all first nibbles                   */
/* Return Value :   number of elements in _EventNavigation                                     */
/*---------------------------------------------------------------------------------------------*/

public int ListEventsByContent(int Content)
{
    // first step, Events by Content ...
    Integer RefServiceId,ServiceId;
    String  EventServId;
    Integer EventId;
    SIEvent anEvent;


    _EventNavigation = null;
    _EventNavigation = new Vector(10,1);

    Vector myVector = new Vector(10,1);
    Enumeration EventIterator;

    EventIterator = _ListOfEvents.elements();
    short[] descTab = new short[1];
    descTab[0] = DescriptorTag.CONTENT;
    SIIterator CompIterator;
    int result;

    while (EventIterator.hasMoreElements())
    {
        anEvent = (SIEvent)EventIterator.nextElement();

        try
        {
            CompIterator = anEvent.retrieveDescriptors(descTab);
           // only one Descriptor must compose CompIterator
            // modifications SIIterator il faut enlever _Enum
          if (CompIterator != null)
          {
             result = DescContent.GetContentInt((Descriptor)CompIterator._Enum.nextElement());

             if ((Content & 0x000F) == 0)  // all Nibbles 2
             {
                result = (int)(result & 0x00F0);
             }

             if (result == Content)
             {
                EventId = new Integer(anEvent.getEventId());
                ServiceId = new Integer(anEvent.getServiceId());
                EventServId = EventId.toString() + " " + ServiceId.toString();
                myVector.addElement(EventServId);
             }

          }
        }
        catch(RetrieveFailedException rfe)
        {
        }
   }


    // second step, events by pref services, if necessary
    if (_PrefServices != null)
    {
        for (int i = 0; i < _PrefServices.size(); i++)
        {
            RefServiceId = (Integer)_PrefServices.elementAt(i);
            EventIterator = myVector.elements();

            while(EventIterator.hasMoreElements())
            {
                EventServId = (String)EventIterator.nextElement();
                anEvent = (SIEvent)_ListOfEvents.get(EventServId);
                if (anEvent.getServiceId() == RefServiceId.intValue())
                {
                    _EventNavigation.addElement(EventServId);
                    myVector.removeElement(EventServId);
                }
            }
        }
     }

     if (_ViewOnlyPrefServices == false)
     {
        EventIterator = myVector.elements();

        while (EventIterator.hasMoreElements())
        {
            EventServId = (String)EventIterator.nextElement();
            _EventNavigation.addElement(EventServId);
        }

     }

    int NbEvents = GetNbEvents();

    if (NbEvents>0);
    {
        AffectCurrentEventIndex(0);
    }

    return NbEvents;

}

/*---------------------------------------------------------------------------------------------*/
/* Method       :   ListFollowingEvents                                                        */
/* Description  :   fills _EventNavigation with Events sorted by Favourites Service            */
/* Parameters   :   none                                                                       */
/* Return Value :   number of elements in _EventNavigation                                     */
/*---------------------------------------------------------------------------------------------*/

public int ListFollowingEvents()
{
    _EventNavigation = null;
    _EventNavigation = new Vector(10,1);

    Enumeration EventIterator;
    Integer RefServiceId;
    String EventServId;

    if (_PrefServices != null)
    {
        for (int i=0; i<_PrefServices.size();i++)
        {
            RefServiceId =(Integer)_PrefServices.elementAt(i);
            if (_FollowingEventsTable.containsKey(RefServiceId))
            {
               EventServId =(String)_FollowingEventsTable.get(RefServiceId);
               _EventNavigation.addElement(EventServId);
            }
        }
    }
    // Other events are added after ....
    if (_ViewOnlyPrefServices == false)
    {

      EventIterator = _FollowingEventsTable.elements();
      while (EventIterator.hasMoreElements())
      {
        EventServId = (String)EventIterator.nextElement();

        if (_EventNavigation.contains(EventServId) == false)
        {
          _EventNavigation.addElement(EventServId);
        }
      }
    }

    int NbEvents = GetNbEvents();

    if (NbEvents>0);
    {
        AffectCurrentEventIndex(0);
    }

    return NbEvents;

}

/*---------------------------------------------------------------------------------------------*/
/* Method       :   ListPresentEvents                                                          */
/* Description  :   fills _EventNavigation with Events sorted by Favourites Service            */
/* Parameters   :   none                                                                       */
/* Return Value :   number of elements in _EventNavigation                                     */
/*---------------------------------------------------------------------------------------------*/

public int ListPresentEvents()
{
    _EventNavigation = null;
    _EventNavigation = new Vector(10,1);

    Enumeration EventIterator;
    Integer RefServiceId;
    String EventServId;

    if (_PrefServices != null)
    {
        for (int i=0; i<_PrefServices.size();i++)
        {
            RefServiceId =(Integer)_PrefServices.elementAt(i);
            if (_PresentEventsTable.containsKey(RefServiceId))
            {
               EventServId =(String)_PresentEventsTable.get(RefServiceId);
               _EventNavigation.addElement(EventServId);
            }
        }
    }

    if (_ViewOnlyPrefServices == false)
    {

      // Other events are added after ....
      EventIterator = _PresentEventsTable.elements();
      while (EventIterator.hasMoreElements())
      {
        EventServId = (String)EventIterator.nextElement();

        if (_EventNavigation.contains(EventServId) == false)
        {
            _EventNavigation.addElement(EventServId);
        }
      }
    }

    int NbEvents = GetNbEvents();

    if (NbEvents>0);
    {
        AffectCurrentEventIndex(0);
    }

    return NbEvents;

}

/*---------------------------------------------------------------------------------------------*/
/* Method       :   ListScheduleEvents                                                         */
/* Description  :   fills _EventNavigation with Schedule Events sorted by Favourites Service   */
/* Parameters   :   none                                                                       */
/* Return Value :   number of elements in _EventNavigation                                     */
/*---------------------------------------------------------------------------------------------*/

public int ListScheduleEvents()
{
    _EventNavigation = null;
    _EventNavigation = new Vector(10,1);

    Enumeration VectorIterator;
    Enumeration  EventIterator;
    Integer RefServiceId;
    String  EventServId;
    Vector  SchedEvents;


    if (_PrefServices != null)
    {
        for (int i=0; i<_PrefServices.size();i++)
        {
            RefServiceId =(Integer)_PrefServices.elementAt(i);

            if (_ScheduleEventsTable.containsKey(RefServiceId))
            {
               SchedEvents =(Vector)_ScheduleEventsTable.get(RefServiceId);
               EventIterator = SchedEvents.elements();

               while (EventIterator.hasMoreElements())
               {
                 EventServId = (String)EventIterator.nextElement();
                  _EventNavigation.addElement(EventServId);
               }
            }
        }
    }

    if (_ViewOnlyPrefServices == false)
    {

      VectorIterator = _ScheduleEventsTable.elements();
      while (VectorIterator.hasMoreElements())
      {
        SchedEvents = (Vector)VectorIterator.nextElement();
        EventIterator = SchedEvents.elements();

        while (EventIterator.hasMoreElements())
        {
            EventServId = (String)EventIterator.nextElement();
            if (_EventNavigation.contains(EventServId) == false)
            {
               _EventNavigation.addElement(EventServId);
            }
        }
      }
    }

    int NbEvents = GetNbEvents();

    if (NbEvents>0);
    {
        AffectCurrentEventIndex(0);
    }

    return NbEvents;



}



/*---------------------------------------------------------------------------------------------*/
/* Method       :   ListAllScheduleEventsBetweenAndContent                                                     */
/* Description  :   fills _EventNavigation with Schedule Events sorted by Favourites Service   */
/*                  and sorted by Content                                                      */
/* Parameters   :   Content                                                                    */
/* Return Value :   number of elements in _EventNavigation                                     */
/*---------------------------------------------------------------------------------------------*/

public int ListAllScheduleEventsBetweenAndContent(Date Start, Date Stop,int Content)
{
    ListAllScheduleEventsBetween(Start,Stop);

    Vector tempVector =new Vector(10,1);

    for (Enumeration e = _EventNavigation.elements(); e.hasMoreElements();)
    {
        String Key = (String)e.nextElement();
        SIEvent anEvent =(SIEvent)_ListOfEvents.get(Key);
        tempVector.addElement(anEvent);
    }

    if (tempVector.isEmpty())
    {
        return 0;
    }

    // first step, Events by Content ...
    Integer RefServiceId,ServiceId;
    String  EventServId;
    Integer EventId;
    SIEvent anEvent;


    _EventNavigation = null;
    _EventNavigation = new Vector(10,1);

    Vector myVector = new Vector(10,1);
    Enumeration EventIterator;

    EventIterator = tempVector.elements();
    short[] descTab = new short[1];
    descTab[0] = DescriptorTag.CONTENT;
    SIIterator CompIterator;
    int result;

    while (EventIterator.hasMoreElements())
    {
        anEvent = (SIEvent)EventIterator.nextElement();

        try
        {
            CompIterator = anEvent.retrieveDescriptors(descTab);
           // only one Descriptor must compose CompIterator
            // modifications SIIterator il faut enlever _Enum
          if (CompIterator != null)
          {
             result = DescContent.GetContentInt((Descriptor)CompIterator._Enum.nextElement());

             if ((Content & 0x000F) == 0)  // all Nibbles 2
             {
                result = (int)(result & 0x00F0);
             }

             if (result == Content)
             {
                EventId = new Integer(anEvent.getEventId());
                ServiceId = new Integer(anEvent.getServiceId());
                EventServId = EventId.toString() + " " + ServiceId.toString();
                myVector.addElement(EventServId);
             }

          }
        }
        catch(RetrieveFailedException rfe)
        {
        }
   }


    // second step, events by pref services, if necessary
    if (_PrefServices != null)
    {
        for (int i = 0; i < _PrefServices.size(); i++)
        {
            RefServiceId = (Integer)_PrefServices.elementAt(i);
            EventIterator = myVector.elements();

            while(EventIterator.hasMoreElements())
            {
                EventServId = (String)EventIterator.nextElement();
                anEvent = (SIEvent)_ListOfEvents.get(EventServId);
                if (anEvent.getServiceId() == RefServiceId.intValue())
                {
                    _EventNavigation.addElement(EventServId);
                    myVector.removeElement(EventServId);
                }
            }
        }
     }

     if (_ViewOnlyPrefServices == false)
     {
        EventIterator = myVector.elements();

        while (EventIterator.hasMoreElements())
        {
            EventServId = (String)EventIterator.nextElement();
            _EventNavigation.addElement(EventServId);
        }

     }

    int NbEvents = GetNbEvents();

    if (NbEvents>0);
    {
        AffectCurrentEventIndex(0);
    }

    return NbEvents;

}

/*---------------------------------------------------------------------------------------------*/
/* Method       :   ListServiceEvents                                                          */
/* Description  :   fills _EventNavigation with all the Events attached to the current Service */
/* Parameters   :   none                                                                       */
/* Return Value :   number of elements in _EventNavigation                                     */
/*---------------------------------------------------------------------------------------------*/

public int ListServiceEvents()
{
    _EventNavigation = null;
    _EventNavigation = new Vector(10,1);

    String EventServId;
    Integer ServiceId = (Integer)_ServiceNavigation.elementAt(_CurrentServiceIndex);

    if (_PresentEventsTable.containsKey(ServiceId))
    {
        EventServId =(String)_PresentEventsTable.get(ServiceId);
        _EventNavigation.addElement(EventServId);
    }

    if (_FollowingEventsTable.containsKey(ServiceId))
    {
        EventServId =(String)_FollowingEventsTable.get(ServiceId);
        _EventNavigation.addElement(EventServId);
    }

    if (_ScheduleEventsTable.containsKey(ServiceId))
    {
        Vector myVect = (Vector)_ScheduleEventsTable.get(ServiceId);
        Enumeration EventIterator = myVect.elements();

        while (EventIterator.hasMoreElements())
        {
            EventServId = (String)EventIterator.nextElement();
            _EventNavigation.addElement(EventServId);
        }
    }

    int NbEvents = GetNbEvents();

    if (NbEvents>0);
    {
        AffectCurrentEventIndex(0);
    }

    return NbEvents;
}

/*---------------------------------------------------------------------------------------------*/
/* Method       :   ListServiceEvents                                                          */
/* Description  :   fills _EventNavigation with all the Events attached to the selected Service*/
/* Parameters   :   ServiceIndex(int): index on _ServiceNavigation Table                       */
/* Return Value :   number of elements in _EventNavigation or 0 if bad Index                   */
/*---------------------------------------------------------------------------------------------*/

public int ListServiceEvents(int ServiceIndex)
{
    int result = 0;
    boolean res = AffectCurrentServiceIndex(ServiceIndex);

    if (res)
    {
        result = ListServiceEvents();
    }

    return result;

}


/*---------------------------------------------------------------------------------------------*/
/* Method       :   ListServiceScheduleEventsBetween                                           */
/* Description  :   fills _EventNavigation with all the Schedule Events attached to the selected*/
/*                  Service, and will start between StartTime and StopTime                     */
/* Parameters   :   StartDate / StopDate: index on _ServiceNavigation Table                    */
/*                  PAY ATTENTION !!! StartDate and StopDate are in a LOCAL TIME FORMAT !!!    */
/* Return Value :   number of elements in _EventNavigation or 0 if bad Index                   */
/*---------------------------------------------------------------------------------------------*/


public int ListServiceScheduleEventsBetween(Date StartDate,Date StopDate)
{
    _EventNavigation = null;
    _EventNavigation = new Vector(10,1);
    Integer ServiceId = (Integer)_ServiceNavigation.elementAt(_CurrentServiceIndex);

    String EventServId;
    Vector EITSVector;
    SIEvent anEvent;
    SIService aService =(SIService)_ListOfServices.get(ServiceId);
    short[] DescTab = new short[1];
    DescTab[0] = -1;

    try
    {
    if (aService.retrieveEITScheduleFlag() == true)
    {

      // first step => delete old EIT schedule....(if necessary)

      if (_ScheduleEventsTable.containsKey(ServiceId))
      {
         EITSVector = (Vector)_ScheduleEventsTable.get(ServiceId);
         for (Enumeration en = EITSVector.elements(); en.hasMoreElements();)
         {
            EventServId = (String)en.nextElement();

            if (_ListOfEvents.containsKey(EventServId))
            {
                _ListOfEvents.remove(EventServId);
            }
         }

         _ScheduleEventsTable.remove(ServiceId);
      }

     // second Step => retrieve correct EIT Schedule
     Date CopyOfStartDate = new Date(StartDate.toGMTString());
     Date CopyOfStopDate = new Date(StopDate.toGMTString());
     CopyOfStartDate.setHours(CopyOfStartDate.getHours()- _DeltaGMT); // transpose LOCAL HOUR to GMT hour
     CopyOfStopDate.setHours(CopyOfStopDate.getHours()- _DeltaGMT); // transpose LOCAL HOUR to GMT hour
     Date myStopDate = new Date(CopyOfStopDate.toGMTString());
     myStopDate.setHours(myStopDate.getHours()+3);

     SIIterator myIT;
    try
    {
     myIT = aService.retrieveScheduledSIEvents(DescTab,CopyOfStartDate,myStopDate);
     if (myIT != null)
     {
        EITSVector = new Vector(10,1);

        while (myIT._Enum.hasMoreElements())
        {
            anEvent = (SIEvent)myIT._Enum.nextElement();
            EventServId = new String();
            EventServId += anEvent.getEventId()+" "+anEvent.getServiceId();
            _ListOfEvents.put(EventServId,anEvent);
            EITSVector.addElement(EventServId);
            try
            {
              Date EventStartDate = anEvent.getStartTime();
            if ((EventStartDate.after(CopyOfStartDate)) && (EventStartDate.before(CopyOfStopDate)))
            {
              _EventNavigation.addElement(EventServId);
            }

            }
            catch (NullPointerException npe)
            {
            }
        }       // end while

        _ScheduleEventsTable.put(ServiceId,EITSVector);

       } // end myIt != null
      }
     catch (SIException e)
     {
     }
    }// end EIT_S present

    } // end try
    catch (RetrieveFailedException ex)
    {
    }

    int NbEvents = GetNbEvents();

    if (NbEvents>0);
    {
        AffectCurrentEventIndex(0);
    }

    return NbEvents;
}


/*---------------------------------------------------------------------------------------------*/
/* Method       :   ListServiceScheduleEventsBetween                                           */
/* Description  :   fills _EventNavigation with all the Schedule Events attached to the selected*/
/*                  Service, and will start between StartTime and StopTime                     */
/* Parameters   :   ServiceIndex(int) new value for the index                                  */
/*                  StartDate / StopDate: index on _ServiceNavigation Table                    */
/*                  PAY ATTENTION !!! StartDate and StopDate are in a LOCAL TIME FORMAT !!!    */
/* Return Value :   number of elements in _EventNavigation or 0 if bad Index                   */
/*---------------------------------------------------------------------------------------------*/

public int ListServiceScheduleEventsBetween(int ServiceIndex,Date StartDate, Date StopDate)
{
    int result = 0;
    boolean res = AffectCurrentServiceIndex(ServiceIndex);

    if (res)
    {
        result = ListServiceScheduleEventsBetween(StartDate,StopDate);
    }

    return result;

}

/*--------------------------------------------------------------------------------------------*/
/* Method       :   ListAllScheduleEventsBetween                                              */
/* Description  :   fills _EventNavigation with all the Schedule Events                       */
/*                  between StartTime and StopTime                                            */
/* Parameters   :                                                                             */
/*                  StartDate / StopDate: Date                                                */
/*                  PAY ATTENTION !!! StartDate and StopDate are in a LOCAL TIME FORMAT !!!   */
/* Return Value :   number of elements in _EventNavigation                                    */
/*--------------------------------------------------------------------------------------------*/

public int ListAllScheduleEventsBetween(Date StartDate, Date StopDate)
{

   Vector result = new Vector(10,1);
   // first step, destroy ScheduleTableEvents and ListOfEvents
   Enumeration Enum = _ScheduleEventsTable.elements();

   while (Enum.hasMoreElements())
    {
        Vector myEITSVect =(Vector)Enum.nextElement();


        for (Enumeration EITSEnum = myEITSVect.elements(); EITSEnum.hasMoreElements();)
        {
            String EventServId = (String)EITSEnum.nextElement();
            _ListOfEvents.remove(EventServId);

        }
    }
    _ScheduleEventsTable = null;
    _ScheduleEventsTable = new Hashtable(2*MaxEvents);


   // end of remove operation ...

   for(int i=0;i<_ServiceNavigation.size();i++)
   {
       ListServiceScheduleEventsBetween(i,StartDate,StopDate);
       Enumeration EventIT = _EventNavigation.elements();

       while (EventIT.hasMoreElements())
       {
          String EventServId = (String)EventIT.nextElement();
          result.addElement(EventServId);
       }
   }

   _EventNavigation = null;
   _EventNavigation = new Vector(10,1);

    for (Enumeration en = result.elements(); en.hasMoreElements();)
    {
        String ESId = (String)en.nextElement();
        _EventNavigation.addElement(ESId);
    }

    int NbEvents = GetNbEvents();

    if (NbEvents>0);
    {
        AffectCurrentEventIndex(0);
    }

    return NbEvents;
    
}



/*---------------------------------------------------------------------------------------------*/
/* Method       :   ListServices                                                               */
/* Description  :   fills _ServiceNavigation with all The Services                             */
/* Parameters   :   none                                                                       */
/* Return Value :   number of elements in _ServiceNavigation                                   */
/*---------------------------------------------------------------------------------------------*/

public int ListServices()
{

  _ServiceNavigation = null;
  _ServiceNavigation = new Vector(10,1);
  Enumeration ServIterator;
  Integer ServiceId, RefServiceId;

  if (_PrefServices != null)
  {
      for(int i=0; i<_PrefServices.size();i++)
      {

          RefServiceId =(Integer)_PrefServices.elementAt(i);
          if (_ListOfServices.containsKey(RefServiceId))
          {
              _ServiceNavigation.addElement(RefServiceId);
          }
      }
  }

  if (_ViewOnlyPrefServices == false)
  {
    ServIterator =  _ListOfServices.keys();


    while (ServIterator.hasMoreElements())
    {
        ServiceId = (Integer)ServIterator.nextElement();

        if (_ServiceNavigation.contains(ServiceId) == false)
        {
           _ServiceNavigation.addElement(ServiceId);
        }
    }
  }

  int NbServices = GetNbServices();
  if (NbServices >0)
  {
    AffectCurrentServiceIndex(0);
  }

  return NbServices;

}


/*---------------------------------------------------------------------------------------------*/
/* Method       :   NextEvent                                                                  */
/* Description  :   increases _CurrentEventIndex, If possible !!!                              */
/* Parameters   :   none                                                                       */
/* Return Value :   false when the index is out of range / true otherwhise                     */
/*---------------------------------------------------------------------------------------------*/

public boolean NextEvent()
{
    boolean result = false;
    int Limit = GetNbEvents();
    int NewPos = _CurrentEventIndex + 1;

    if (NewPos<Limit)
    {
        result = AffectCurrentEventIndex(NewPos);
    }

    return result;

}


/*---------------------------------------------------------------------------------------------*/
/* Method       :   NextService                                                                */
/* Description  :   increases _CurrentServiceIndex, If possible !!!                            */
/* Parameters   :   none                                                                       */
/* Return Value :   false when the index is out of range / true otherwhise                     */
/*---------------------------------------------------------------------------------------------*/

public boolean NextService()
{
    boolean result = false;
    int Limit = GetNbServices();
    int NewPos = _CurrentServiceIndex + 1;

    if (NewPos<Limit)
    {
        result = AffectCurrentServiceIndex(NewPos);
    }

    return result;
}

/*---------------------------------------------------------------------------------------------*/
/* Method       :   PrevEvent                                                                  */
/* Description  :   decreases _CurrentEventIndex, If possible !!!                              */
/* Parameters   :   none                                                                       */
/* Return Value :   false when the index is out of range / true otherwhise                     */
/*---------------------------------------------------------------------------------------------*/

public boolean PrevEvent()
{
  boolean result = false;
  int NewPos = _CurrentEventIndex - 1;

  if (NewPos>=0)
  {
      result = AffectCurrentEventIndex(NewPos);
  }

  return result;
}

/*---------------------------------------------------------------------------------------------*/
/* Method       :   PrevService                                                                */
/* Description  :   decreases _CurrentServiceIndex, If possible !!!                            */
/* Parameters   :   none                                                                       */
/* Return Value :   false when the index is out of range / true otherwhise                     */
/*---------------------------------------------------------------------------------------------*/

public boolean PrevService()
{

  boolean result = false;
  int NewPos = _CurrentServiceIndex - 1;

  if (NewPos>=0)
  {
      result = AffectCurrentServiceIndex(NewPos);
  }

  return result;
}

/*---------------------------------------------------------------------------------------------*/
/* Method       :   RemoveAlarm                                                                */
/* Description  :   Remove an AlarmObject from _Alarms If possible !!!                         */
/* Parameters   :   AlarmIndex(int) Index of _Alarms vector                                    */
/* Return Value :   false when the index is out of range / true otherwhise                     */
/*---------------------------------------------------------------------------------------------*/

public boolean RemoveAlarm(int AlarmIndex)
{
   if ((AlarmIndex<0) || (AlarmIndex>=GetNbAlarms()))
   {
      return false;
   }

   Alarm anAlarm = (Alarm)_Alarms.elementAt(AlarmIndex);

   SIEvent anEvent =anAlarm.GetEvent();

   // first step --> delete the EventListener ....)

   if (anEvent == null)
   {
      return false;
   }

   anEvent.removeListener(anAlarm);

   // second step --> remove it from _Alarm;

   _Alarms.removeElement(anAlarm);

   return true;

}

/*---------------------------------------------------------------------------------------------*/
/* Method       :   SetDeltaGMT                                                                */
/* Description  :   modifies _DeltaGMT                                                         */
/* Parameters   :   NewDelta(int) : local hour = GMT hour + DeltaGMT                           */
/* Return Value :   none                                                                       */
/*---------------------------------------------------------------------------------------------*/

public void SetDeltaGMT(int NewDelta)
{
    _DeltaGMT = NewDelta;
}


/*---------------------------------------------------------------------------------------------*/
/* Method       :   SetLanguage                                                                */
/* Description  :   Sets _Language (an ISO 369 Language Code)                                  */
/* Parameters   :   Language: new value of _Language                                           */
/* Return Value :   N/A                                                                        */
/*---------------------------------------------------------------------------------------------*/


public void SetLanguage(String Language)
{
    _Language = Language;
}


/*---------------------------------------------------------------------------------------------*/
/* Method       :   SetPrefServices                                                            */
/* Description  :   Sets _PrefServices Vector                                                  */
/* Parameters   :   Vector: new value of _PrefServices                                         */
/* Return Value :   N/A                                                                        */
/*---------------------------------------------------------------------------------------------*/

public void SetPrefServices(Vector FavouriteServices)
{
    if ((FavouriteServices== null) || (FavouriteServices.isEmpty()))
    {
        _PrefServices = null;
    }
    else
    {
      _PrefServices = FavouriteServices;
    }
}


/*---------------------------------------------------------------------------------------------*/
/* Method       :   SetViewOnlyPrefServices()                                                  */
/* Description  :   modifies _ViewOnlyPrefServices                                             */
/* Parameters   :   newValue, a boolean                                                        */
/* Return Value :   none                                                                       */
/*---------------------------------------------------------------------------------------------*/

public void SetViewOnlyPrefServices(boolean newValue)
{
    _ViewOnlyPrefServices = newValue;
}

/********* for debug info ********/
public void endProgram()
{
    _StreamGuide.QuitProgram();
}

// stp class members end
}
// stp class definition end



