/*
* Copyright(c) Philips Consumer Electronics B.V. 1998                         
* All rights reserved. Reproduction in whole or in part is                  
* prohibited without the written consent of the copyright holder.            
*                                                                            
*  Project             : Bali
*  Module              : bali.si 
*  File name   $Logfile: /func_test_platf2/org/davic/mpeg/TransportStream.java $
*       Last save $Date: 2003/12/24 00:53:09 $
*             $Revision: 1.1.1.1 $
*          Last $Author: mikeseven $
*  Creation Date       : 09/11/98
*  First Author        : S.A.F.A. van den Heuvel
*                        NatLab - Eindhoven
* 
*                                                                          
*  Change history      :
*  ---------------
*
* $Log: Service.java,v $
* Revision 1.1.1.1  2003/12/24 00:53:09  mikeseven
* Initial release of the CVS repository for MPEG-4 Systems code
* after the code was corrupted due to reorganization and human 
* errors.
*
* Revision 1.1  2003/04/03 04:29:30  mikeseven
* Initial release of Player3D and MPEG-J for all MPEG-4 reference softwares players with samples.
* For installation, follow instructions in docs/Player3D_setup.doc.
*
* Many thanks to Prashanth Gujjeti and Viswanathan Swaminathan from Sun Microsystems.
*
* -- Mikael Bourges-Sevenier (Mindego)
*
* Revision 1.1  2002/08/23 16:47:26  pg138550
* Changes for MPEG-J. NOT FINAL
*
* Revision 1.1  2000/06/06 18:43:17  alavilli
* After SISF integration into the CORE4 code
*
* Revision 1.1.2.1  1999/07/09 14:17:17  depetris
* Aggiornati i file per lo sviluppo e il rilascio a MPEG-J.
* Integra la versione 2.5 del core.
*
* Revision 1.4  1999-04-15 09:53:18+02  vdheuvel
* Files changed to hook the system up to the TM demultipler and the
* TM hardware. Changes include, swapping the java.util.Date class
* for the bali.tm.Date class (to prevent using 64 bit floats). Changing
* the HW sect filt used by bali.hw.SectFilt.
*
* Revision 1.3  1999-04-02 14:21:35+02  vdheuvel
* *** empty log message ***
*
* Revision 1.2  1999-04-01 17:20:50+02  vdheuvel
* Added getPcrPID to support of JMF. This method is NOT part
* of the DAVIC APIs.
*
* Revision 1.1  1999-02-24 21:11:00+01  vdheuvel
* start of archive, 29 march 1999
* DAVIC API implementation
*
*/

package org.davic.mpeg;
import  org.davic.mpeg.dvb.DvbTransportStream;
import  org.davic.mpeg.dvb.DvbElementaryStream;
import  bali.retrieve.Monitor;
import  bali.retrieve.RetrievalEvent;
import  bali.retrieve.RetrievalMediator;
import  bali.table.TableIdentifier;
import  bali.component.TransportStreamSelection;
import  bali.component.ComponentMaster;
import  bali.table.PMT;
import  bali.table.ES;
/*DATE_NORMAL*/ import  java.util.Date;
//DATE_TM*/     import  bali.tm.Date;
import  bali.support.Debug;
/**
  * This class is used to represent a Service within an MPEG Transport Stream.
  *
  */

public class Service
{

	/**
	 * @return a reference to the TransportStream object to which this Service belongs.
	 */
	public TransportStream getTransportStream()
	{
        /*DBG*/Debug.msg( Debug.MPEG, "Service getTransportStream");
	  return _ts;
	}

	/**
	 * @return the service_id (or equivalently the program_number) of this service.
	 */
	public int getServiceId()
	{
        /*DBG*/Debug.msg( Debug.MPEG, "Service getServiceId");
	  return _servId;
	}

	/**
	  * @param pid the value of MPEG-2 Transport Stream packets 
	  * that carry the elementary stream.
	  *
	  * @return a reference to the ElementaryStream object that 
	  * represents the Elementary Stream carried by packets with 
	  * the specified PID. Null is returned if the specified PID 
	  * is not present within this service or if  no Elementary 
	  * Stream is carried by the specified PID or if the required 
	  * information is not available.
	  *
	  * @version updated to DAVIC 1.3.1
	  *
	  */
	public ElementaryStream retrieveElementaryStream(int pid)
	{
        /*DBG*/Debug.msg( Debug.MPEG, "Service retrieveElementaryStream");
        ElementaryStream res;
        synchronized( _lock )
        {
            res = getES( _arEs, pid );
        }
    	return res;
	}

	/**
	  * @return the array of all ElementaryStream objects present within this service.
	  * When the required information is not available null is returned.
	  * @version updated to DAVIC 1.3.1
	  */

	public ElementaryStream[] retrieveElementaryStreams()
	{
        /*DBG*/Debug.msg( Debug.MPEG, "Service retrieveElementaryStreams");
   	    DvbElementaryStream[] res = null;
	    synchronized( _lock )
	    {
    	    updateElementaryStreams();
    	    if ( _pmt != null )
    	    {
        	    res = new DvbElementaryStream[ _arEs.length ];
        	    System.arraycopy( _arEs, 0, res, 0, _arEs.length );
        	}
        }
    	return res;
	}

    /*******************
    BALI implementation specific method returning the pcr PID of this service.
    If the pmt of the service is not accessible or the pmt is not part of
    the current transport stream, -2 is returned.
    */
    public int getPcrPid()
    {
        int res = -2;
        synchronized( _lock )
        {
    	    updateElementaryStreams();
            if ( _pmt!=null )
            {
                res = _pmt.getPCR_PID();
            }
        }
        return res;
    }

    ////////////// IMPLEMENTATION ///////////////////////
    private DvbTransportStream    _ts;
    private int                   _servId;
    private PMT                   _pmt;
    private DvbElementaryStream[] _arEs;
    private Object  _lock = new Object();
    
    Service( DvbTransportStream ts, int serviceId )
    {
        /*DBG*/Debug.msg( Debug.MPEG, "Service creation "+serviceId);
        _ts     = ts;
        _servId = serviceId;
        _arEs   = new DvbElementaryStream[0];
        _pmt    = null;
    }
    
    private void updateElementaryStreams()
    {
        /*DBG*/Debug.msg( Debug.MPEG, "Service "+_servId+" updateElementaryStreams");
	    synchronized( _lock )
	    {
	        DvbTransportStream ts = TransportStreamSelection.getInstance().getCurrentTs();
    	    if (  ts.equals( _ts ) )
    	    {   // current TS obtain and monitor PMT.
    	        if ( _pmt==null )
        	    {   // table changed or just filtered.
        	        PMT pmt = getPMT();
        	        if ( pmt != null )
        	        {
                	    if ( _pmt == null )
                	    {   _pmt = pmt;
                	        setESs( pmt );
                	    }
                	    else if ( pmt.isSameTable( _pmt ) )
                	    {
                	        _pmt = pmt;
                	    }
                	    else
                	    {
                	        setESs( pmt );
                	    }
        	        }
        	    }
        	}
    	    else
    	    {   _arEs = null;
    	    }
    	}
    }
    
    private DvbElementaryStream getES( DvbElementaryStream[] arEs, int pid )
    {
        DvbElementaryStream res =null;
        int i=0;
        while ( i<arEs.length && res == null )
        {   i++;
            res = ( !(arEs[i].getPID()==pid) ?arEs[i] : null );
        }
        return res;
    }
    
    private PMT getPMT()
    {  
        /*DBG*/Debug.msg( Debug.MPEG, "Service startMonitor 2, filter PMT");
        PMT pmt = null;
        TableIdentifier ti = 
            new TableIdentifier( TableIdentifier.PMT, _ts.getOriginalNetworkId(), _ts.getTransportStreamId(), _servId );
        // this is current ts, filter PMT.
        RetrievalMediator rm;
        RetrievalEvent event;
        do
        {
            rm = RetrievalMediator.retrieveSITable( null, ti );
            event = rm.getRetrievalEvent();
        }
        while (  event.getResult() != RetrievalEvent.SUCCESS &&
                 event.getResult() != RetrievalEvent.TIME_OUT 
              );
              
        if ( event.getResult() == RetrievalEvent.SUCCESS )
        {   pmt = (PMT)event.getSITable();
        }
        /*DBG*/Debug.msg( Debug.MPEG, Debug.MEDIUM, "Service getPMT "+pmt);
        
        return pmt;
    }

    /*******************
    */
    private void setESs( PMT pmt )
    {
        ES[] arEsPmt = pmt.getESs();
        DvbElementaryStream[] arEsTmp = new DvbElementaryStream[ arEsPmt.length ];
        DvbElementaryStream dummy;
        Object  cm = ComponentMaster.getInstance();
        for ( int i=0; i<arEsPmt.length; i++ )
        {
            dummy = getES( _arEs, arEsPmt[i].getPID() );
            if ( dummy==null )
            {   dummy = new DvbElementaryStream( this, arEsPmt[i].getPID(), arEsPmt[i].getStreamType(), cm );
            }
            arEsTmp[i] = dummy;
        }
        _arEs = arEsTmp;
    }

}


