/*
* Copyright(c) Philips Consumer Electronics B.V. 1998                         
* All rights reserved. Reproduction in whole or in part is                  
* prohibited without the written consent of the copyright holder.            
*                                                                            
*  Project             : Bali
*  Module              : bali.si 
*  File name   $Logfile: /func_test_platf2/org/davic/mpeg/TransportStream.java $
*       Last save $Date: 2003/12/24 00:53:09 $
*             $Revision: 1.1.1.1 $
*          Last $Author: mikeseven $
*  Creation Date       : 09/11/98
*  First Author        : S.A.F.A. van den Heuvel
*                        NatLab - Eindhoven
* 
*                                                                          
*  Change history      :
*  ---------------
*
* $Log: TransportStream.java,v $
* Revision 1.1.1.1  2003/12/24 00:53:09  mikeseven
* Initial release of the CVS repository for MPEG-4 Systems code
* after the code was corrupted due to reorganization and human 
* errors.
*
* Revision 1.1  2003/04/03 04:29:30  mikeseven
* Initial release of Player3D and MPEG-J for all MPEG-4 reference softwares players with samples.
* For installation, follow instructions in docs/Player3D_setup.doc.
*
* Many thanks to Prashanth Gujjeti and Viswanathan Swaminathan from Sun Microsystems.
*
* -- Mikael Bourges-Sevenier (Mindego)
*
* Revision 1.1  2002/08/23 16:47:26  pg138550
* Changes for MPEG-J. NOT FINAL
*
* Revision 1.1  2000/06/06 18:43:17  alavilli
* After SISF integration into the CORE4 code
*
* Revision 1.1.2.1  1999/07/09 14:17:17  depetris
* Aggiornati i file per lo sviluppo e il rilascio a MPEG-J.
* Integra la versione 2.5 del core.
*
* Revision 1.2  1999-04-15 09:53:18+02  vdheuvel
* Files changed to hook the system up to the TM demultipler and the
* TM hardware. Changes include, swapping the java.util.Date class
* for the bali.tm.Date class (to prevent using 64 bit floats). Changing
* the HW sect filt used by bali.hw.SectFilt.
*
* Revision 1.1  1999-02-19 17:49:02+01  vdheuvel
* start of archive, 29 march 1999
* DAVIC API implementation
*
 * 
 * 2     9-11-98 17:20 Vdheuvel
 * Added source safe headers.
*/

package org.davic.mpeg;
import  org.davic.mpeg.dvb.DvbTransportStream;
import  bali.support.NotImplementedError;
import  bali.support.Debug;
import  bali.retrieve.Monitor;
import  bali.retrieve.RetrievalEvent;
import  bali.retrieve.RetrievalMediator;
import  bali.table.TableIdentifier;
import  bali.component.TransportStreamSelection;
import  bali.table.SDT;
import  bali.table.Serv;
/*DATE_NORMAL*/ import  java.util.Date;
//DATE_TM*/     import  bali.tm.Date;

/**
 * This class represents an MPEG-2 Transport Stream with its 
 * associated Service Information (SI) as known to a decoder.
 * <p>
 * A TransportStream object models a specific transport stream that can be 
 * accessed through a specific network interface. This implies that a Transport
 * Stream object has implicitly a connection to a specific network interface. 
 * Thus, if two or more network interfaces can deliver the same transport 
 * stream, this will be modeled by two or more separate TransportStream objects. 
 * A network interface does not need to be tuned to a transport stream if an 
 * application wants to use the corresponding TransportStream object.
 * <p>
 * If the corresponding network interface is currently tuned to the 
 * TransportStream, then an application can query the TransportStream for the 
 * services it contains, and the services for which elementary streams it 
 * contains. If the corresponding network interface is not currently tuned to 
 * the TransportStream, then the application can query the TransportStream for
 * the services it contains. If the STB has cached the required information 
 * it can return it, otherwise it should return null. If an application queries
 * a Service object for elementary stream information and the corresponding 
 * TransportStream is not currently tuned to, then the Service object should 
 * return null.
 * <p>
 * If an application has two references to a TransportStream object and those 
 * TransportStream objects model the same transport stream coming from the same
 * network interface, then the equals() method (inherited from java.lang.Object) 
 * return true when comparing both TransportStream objects. The references 
 * themselves are not necessarily the same, although they may be.
 * <p>
 * Note: If an application wants to know to which network interface a 
 * TransportStream object is connected to, it should use the Tuning API if it 
 * is available.
 * <p>
 * @version updated to DAVIC 1.3.1
 */

public abstract class TransportStream 
{
	protected TransportStream( int or_nw_id, int ts_id )
	{
        /*DBG*/Debug.msg( Debug.MPEG, "TransportStream creation "+or_nw_id+" "+ts_id);
	    // TODO, protect with security exception
	    _ts_id = ts_id;
	    _or_nw_id = or_nw_id;
	    _sdtMonitor = null;
	    _arService  = new Service[0];
	    _sdt        = null;
	}

	/**
	  * @return the transport_stream_id of this transport stream.
	  */
	public int getTransportStreamId()
	{
        /*DBG*/Debug.msg( Debug.MPEG, Debug.IMPL, "TransportStream getTransportStreamId");
	  return _ts_id;
	}

	/**
	  * @param serviceId the id of the requested service within this transport stream
	  * @return a reference to the service object that represents 
	  * the service from which this MPEG-2 TS is accessed. 
	  * If the required information is not available or the indicated 
	  * service does not exist, null is returned.
	  *
	  * @version updated to DAVIC 1.3.1
	  */
	public Service retrieveService(int serviceId)
	{
        /*DBG*/Debug.msg( Debug.MPEG, "TransportStream retrieveService");
	    Service res;
	    synchronized ( _lock )
	    {   
    	    updateServices();
    	    res = getService( _arService, serviceId );
	    }   
     	return res;
	}

	/**
	  * @return the array of all service objects belonging to this transport stream.  
	  * When the required information is not available null is returned.
	  * @version updated to DAVIC 1.3.1
	  */
	public Service[] retrieveServices()
	{
        /*DBG*/Debug.msg( Debug.MPEG, "TransportStream retrieveServices");
   	    Service[] res = null;
	    synchronized( _lock )
	    {
    	    updateServices();
    	    if ( _sdt != null )
    	    {
        	    res = new Service[ _arService.length ];
        	    System.arraycopy( _arService, 0, res, 0, _arService.length );
        	}
        }
    	return res;
	}

	/**
	  * @return the time the information contained in the transport
	  * stream object was last updated from a live transport stream. 
	  * If the transport stream is currently live, it returns null.
	  */
	public Date getUpdateTime()
	{
        /*DBG*/Debug.msg( Debug.MPEG, "TransportStream getUpdateTime");
	    Date res = null;
	    if ( this == TransportStreamSelection.getInstance().getCurrentTs() )
	    {   return null;
	    }
	    // SDT actual
	    SDT sdt = getSDT();
	    if (  sdt == null )
	    {   res = new Date( 0 );
	    }
	    else
	    {
	        res = sdt.getSITableInfo().getUpdateTime() ;
	    }
    	return res;
	}

    /**
      * Compare two Transport streams
      */
    public boolean equals( Object obj )
    {
        /*DBG*/Debug.msg( Debug.MPEG, "TransportStream equals");
        return ( ((TransportStream)obj)._ts_id == _ts_id );
    }

    /////////////// PRIVATE ///////////////
    private int       _ts_id;
    private int       _or_nw_id;
    private SDT       _sdt;
    private Service[] _arService;
    // monitor SDT for list of services.
    private Monitor   _sdtMonitor;
    private Object    _lock = new Object();
    
    private void updateServices()
    {
        /*DBG*/Debug.msg( Debug.MPEG, Debug.MEDIUM, "TransportStream updateServices");
	    synchronized( _lock )
	    {
    	    if ( _sdtMonitor ==null )
    	    {   /*DBG*/Debug.msg( Debug.MPEG, Debug.LOW, "TransportStream updateServices create Monitor");
                TableIdentifier ti = new TableIdentifier( TableIdentifier.SDT, _or_nw_id, _ts_id );
    	        _sdtMonitor = new Monitor( null, ti );
    	    }
    	    
	        if ( _sdtMonitor.checkForTableChange() || _sdt == null )
    	    {   /*DBG*/Debug.msg( Debug.MPEG, Debug.LOW, "TransportStream updateServices new Table filtered");
        	    SDT sdt = getSDT();

        	    if ( sdt != null )
        	    {
        	        if ( _sdt == null )
        	        {
                        /*DBG*/Debug.msg( Debug.MPEG, Debug.LOW, "TransportStream updateServices Set first SDT");
        	            _sdt = sdt;
        	            setServices( sdt );
        	        } 
        	        else if ( sdt.isSameTable( _sdt ) )
            	    {
            	        _sdt = sdt;
            	    }
            	    else
            	    {   setServices( sdt );
            	    }
            	}
            }
    	}
    }
    
    private Service getService( Service[] arService, int serviceId )
    {
        Service res =null;
        if ( arService != null )
        {
            int i=0;
            while ( i<arService.length && res == null )
            {   i++;
                res = ( !(arService[i].getServiceId()==serviceId) ?arService[i] : null );
            }
        }
        return res;
    }
    
    
    private SDT getSDT()
    {  
        /*DBG*/Debug.msg( Debug.MPEG, Debug.MEDIUM, "TransportStream getSDT");
        SDT sdt = (SDT)_sdtMonitor.getLatestSITable();
        /*DBG*/Debug.msg( Debug.MPEG, Debug.MEDIUM, "TransportStream getSDT, monitor: "+sdt);

        if ( sdt == null )
        {   // table not present
            // filter new SDT
        
            RetrievalMediator rm;
            RetrievalEvent event;
            do
            {
                rm = RetrievalMediator.retrieveSDT(null, _or_nw_id, _ts_id );
                event = rm.getRetrievalEvent();
                sdt = (SDT)_sdtMonitor.getLatestSITable();
            }
            while (  sdt==null || event.getResult()==RetrievalEvent.TIME_OUT );
            // stop when data has been filtered or is not available in current
            // transport stream
        }
        
        return sdt;
    }
    
    /*****************
    */
    public void finalize()
    {
        _sdtMonitor.stopMonitor();
    }

    /*******************
    */
    private void setServices( SDT sdt )
    {
        Serv[] arServ = sdt.getServices();
        Service[] arService = new Service[ arServ.length ];
        Service dummy;
        for ( int i=0; i<arServ.length; i++ )
        {
            dummy = getService( _arService, arServ[i].getServiceId() );
            if ( dummy==null )
            {   dummy = new Service( (DvbTransportStream)this, arServ[i].getServiceId() );
            }
            arService[i] = dummy;
        }
        _arService = arService;
   }
}









