/*
* Copyright(c) Philips Consumer Electronics B.V. 1998                         
* All rights reserved. Reproduction in whole or in part is                  
* prohibited without the written consent of the copyright holder.            
*                                                                            
*  Project             : Bali
*  Module              : bali.si 
*  File name   $Logfile: /func_test_platf2/org/davic/net/dvb/DvbLocator.java $
*       Last save $Date: 2003/12/24 00:53:09 $
*             $Revision: 1.1.1.1 $
*          Last $Author: mikeseven $
*  Creation Date       : 09/11/98
*  First Author        : S.A.F.A. van den Heuvel
*                        NatLab - Eindhoven
* 
*                                                                          
*  Change history      :
*  ---------------
*
* $Log: DvbLocator.java,v $
* Revision 1.1.1.1  2003/12/24 00:53:09  mikeseven
* Initial release of the CVS repository for MPEG-4 Systems code
* after the code was corrupted due to reorganization and human 
* errors.
*
* Revision 1.1  2003/04/03 04:29:31  mikeseven
* Initial release of Player3D and MPEG-J for all MPEG-4 reference softwares players with samples.
* For installation, follow instructions in docs/Player3D_setup.doc.
*
* Many thanks to Prashanth Gujjeti and Viswanathan Swaminathan from Sun Microsystems.
*
* -- Mikael Bourges-Sevenier (Mindego)
*
* Revision 1.1  2002/08/23 16:47:27  pg138550
* Changes for MPEG-J. NOT FINAL
*
* Revision 1.1  2000/07/21 21:05:02  alavilli
* adding missing files in the previous checkin for SISF
*
* Revision 1.1.2.1  1999/07/09 14:17:19  depetris
* Aggiornati i file per lo sviluppo e il rilascio a MPEG-J.
* Integra la versione 2.5 del core.
*
* Revision 1.3  1999-04-28 11:17:36+02  vdheuvel
* Before event '.' is replaced by ';'.
*
* Revision 1.2  1999-04-15 16:54:42+02  vdheuvel
* Bug fix, url no hold the numbers in hex instead of decimals.
*
* Revision 1.1  1998-11-09 18:12:44+01  vdheuvel
* start of archive, 29 march 1999
* DAVIC API implementation
*
 * 
 * 2     9-11-98 17:12 Vdheuvel
 * Implemented all methods.
*/

package org.davic.net.dvb;

/** DVB Locator that encapsulates a DVB URL into an object
  */

public class DvbLocator extends org.davic.net.Locator {

  public DvbLocator() {
    super();
  }

  /** Constructor for the DVB locator 
    * @param url URL string
    */
  public DvbLocator(String url) {
    super(url);
  }

  /** Constructor for the DVB locator of the form "dvb://<onid>.<tsid>" 
    * @param onid original network identifier
    * @param tsid transport stream identifier
    */
  public DvbLocator(int onid, int tsid) {
    super( "dvb://"
         + Integer.toHexString(onid)
         + "."
         +Integer.toHexString(tsid) 
         );
  }

  /** Constructor for the DVB locator of the form "dvb://<onid>.<tsid>.<serviceid>" 
    * @param onid original network identifier
    * @param tsid transport stream identifier
    * @param serviceid service identifier
    */
  public DvbLocator(int onid, int tsid, int serviceid) {
    super( "dvb://"
         + Integer.toHexString(onid)
         + "."
         + Integer.toHexString(tsid) 
         + "."
         + Integer.toHexString(serviceid) 
         );
  }

  /** Constructor for the DVB locator of the form 
    * "dvb://<onid>.<tsid>.<serviceid>;<eventid>" 
    * @param onid original network identifier
    * @param tsid transport stream identifier
    * @param serviceid service identifier
    * @param eventid event identifier
    */
  public DvbLocator(int onid, int tsid, int serviceid, int eventid) {
    super( "dvb://"
         + Integer.toHexString(onid)
         + "."
         + Integer.toHexString(tsid) 
         + "."
         + Integer.toHexString(serviceid) 
         + ";"
         + Integer.toHexString(eventid) 
         );
  }

  /** Constructor for the DVB locator of the form 
    * "dvb://<onid>.<tsid>.<serviceid>;<eventid>.<componenttag>"
    * or "dvb://<onid>.<tsid>.<serviceid>.<componenttag>" 
    * @param onid original network identifier
    * @param tsid transport stream identifier
    * @param serviceid service identifier
    * @param eventid event identifier (if -1, the locator does not include an event id)
    * @param componenttag component tag
    */
  public DvbLocator(int onid, int tsid, int serviceid, int eventid, int componenttag) 
  {
    super( "dvb://"
         + Integer.toHexString(onid)
         + "."
         + Integer.toHexString(tsid) 
         + "."
         + Integer.toHexString(serviceid) 
         + (eventid!=-1 ? ";"+Integer.toHexString(eventid)+"." : ";" )
         + Integer.toHexString(componenttag) 
         );
  }

  ////////////// IMPLEMENTATION ////////////////

}

