/*
* Copyright(c) Philips Consumer Electronics B.V. 1998                         
* All rights reserved. Reproduction in whole or in part is                  
* prohibited without the written consent of the copyright holder.            
*                                                                            
*  Project             : Bali
*  File name   $Logfile: /func_test_platf2/org/davic/net/dvb/si/Descriptor.java $
*       Last save $Date: 2003/12/24 00:53:09 $
*             $Revision: 1.1.1.1 $
*          Last $Author: mikeseven $
*  Creation Date       : 28/10/98
*  First Author        : S.A.F.A. van den Heuvel
*                        NatLab - Eindhoven
* 
*                                                                          
*  Change history      :
*  ---------------
*
* $Log: Descriptor.java,v $
* Revision 1.1.1.1  2003/12/24 00:53:09  mikeseven
* Initial release of the CVS repository for MPEG-4 Systems code
* after the code was corrupted due to reorganization and human 
* errors.
*
* Revision 1.1  2003/04/03 04:29:31  mikeseven
* Initial release of Player3D and MPEG-J for all MPEG-4 reference softwares players with samples.
* For installation, follow instructions in docs/Player3D_setup.doc.
*
* Many thanks to Prashanth Gujjeti and Viswanathan Swaminathan from Sun Microsystems.
*
* -- Mikael Bourges-Sevenier (Mindego)
*
* Revision 1.1  2002/08/23 16:47:27  pg138550
* Changes for MPEG-J. NOT FINAL
*
* Revision 1.1  2000/06/06 18:43:18  alavilli
* After SISF integration into the CORE4 code
*
* Revision 1.1.2.1  1999/07/09 14:17:20  depetris
* Aggiornati i file per lo sviluppo e il rilascio a MPEG-J.
* Integra la versione 2.5 del core.
*
* Revision 1.1  1998-12-10 12:57:08+01  vdheuvel
* start of archive, 29 march 1999
* DAVIC API implementation
*
 * 
 * 1     9-11-98 14:01 Vdheuvel
 * Creation
 * 
 * 1     9-11-98 13:56 Vdheuvel
 * Creation
*/

package org.davic.net.dvb.si;
import  bali.support.Util;

/**
 * This class represents a descriptor within a sub-table.
 * <p>
 * A descriptor consist of three fields: a tag, a contentLength and the content.
 * <p>
 * The tag uniquely identifies the descriptor type. The content length indicates the number of bytes
 * in the content. The content consists of an array of bytes of length content length. The data
 * represented by the content is descriptor type dependent.
 * 
 * @see DescriptorTag
 * @since DAVIC1.3
*/

public class Descriptor{


/**
 * Get the descriptor tag
 *
 * @return The descriptor tag (the most common values are defined in the DescriporTag interface)
 * @see    DescriptorTag
*/
public short getTag()
{
    return _desc.getTag();
}

/**
 * This method returns the length of the descriptor content as 
 * coded in the length field of this descriptor.
 * 
 * @return The length of the descriptor content.
*/
public short getContentLength()
{
    return (short)( _desc.get8bits( 1 ) );
}

/**
 * Get a particular byte within the descriptor content
 * 
 * @return The required byte
 * @exception java.lang.IndexOutOfBoundsException index<0  or index>ContentLength
*/
public byte getByteAt(
	int index
) throws 
	java.lang.IndexOutOfBoundsException
{
    return _desc.getByte( index );
}

/**
 * Get a copy of the content of this descriptor (everything after the length field).
 *
 * @return a copy of the content of the descriptor
*/
public byte[] getContent(
)
{
    byte arB[] = new byte[ _desc.getDescLength() ];
    byte descCont[] = _desc.getContent();
    System.arraycopy( descCont, 2, arB, 0, arB.length );
    return arB;
}

////////////////////// PRIVATE /////////////////////////
private bali.table.Descriptor _desc;

/************************
*/
Descriptor( bali.table.Descriptor desc ) 
{
    _desc = desc;
}

}
