/*
* Copyright(c) Philips Consumer Electronics B.V. 1998
* All rights reserved. Reproduction in whole or in part is
* prohibited without the written consent of the copyright holder.
*
*  Project             : Bali
*  File name   $Logfile: $
*       Last save $Date: 2003/12/24 00:53:09 $
*             $Revision: 1.1.1.1 $
*          Last $Author: mikeseven $
*  Creation Date       : 17/03/98
*  First Author        : S.A.F.A. van den Heuvel
*                        NatLab - Eindhoven
*
*
*  Change history      :
*  ---------------
*
* $Log: EitEventMonitorHandler.java,v $
* Revision 1.1.1.1  2003/12/24 00:53:09  mikeseven
* Initial release of the CVS repository for MPEG-4 Systems code
* after the code was corrupted due to reorganization and human 
* errors.
*
* Revision 1.1  2003/04/03 04:29:31  mikeseven
* Initial release of Player3D and MPEG-J for all MPEG-4 reference softwares players with samples.
* For installation, follow instructions in docs/Player3D_setup.doc.
*
* Many thanks to Prashanth Gujjeti and Viswanathan Swaminathan from Sun Microsystems.
*
* -- Mikael Bourges-Sevenier (Mindego)
*
* Revision 1.1  2002/08/23 16:47:27  pg138550
* Changes for MPEG-J. NOT FINAL
*
* Revision 1.1  2000/06/06 18:43:18  alavilli
* After SISF integration into the CORE4 code
*
* Revision 1.1.2.1  1999/07/09 14:17:20  depetris
* Aggiornati i file per lo sviluppo e il rilascio a MPEG-J.
* Integra la versione 2.5 del core.
*
* Revision 1.3  1999-04-28 10:27:16+02  vdheuvel
* Fixed and test processing of monitor events.
*
* Revision 1.2  1999-04-26 14:16:56+02  vdheuvel
* The 'oldTable' in a Monitor event can be null.
*
* Revision 1.1  1999-03-18 13:55:05+01  vdheuvel
* start of archive, 29 march 1999
* DAVIC API implementation
*
*/

package org.davic.net.dvb.si;
import  bali.retrieve.Monitor;
import  bali.retrieve.MonitorListener;
import  bali.retrieve.MonitorEvent;
import  bali.table.TableIdentifier;
import  bali.table.SITable;
import  bali.table.EIT;
import  bali.table.EitEvent;
import  bali.support.ActiveEventSender;
import  bali.support.ListenerHandler;
import  bali.support.Debug;
import  java.util.EventListener;
import  java.util.EventObject;

/**********************
This class is responsible for monitoring changes in SI tables and 
sending events to the listeners.
*/
class EitEventMonitorHandler
    extends MonitorHandler
{
    /********************
    */
//    EitEventMonitorHandler( SIInformation source, TableIdentifier ti, int eventId )
    EitEventMonitorHandler( SIInformation source, EIT eit, TableIdentifier ti, int eventId )
    {   
//        super( source, eit.getTableIdentifier()i );
        super( source, ti );
        _eventId = eventId;
        _eit = eit;
    }
    

    /********************
    New Monitor event.
    */
    public void newMonitorEvent(MonitorEvent event)
    {
        SIEventObject siEvent = null;
        EIT eit;
        eit = (EIT)event.getOldTable();
//        EitEvent eitEvOld = ( eit!=null ? eit.getEitEvent( _eventId ) : null );
        EitEvent eitEvOld = ( eit!=null ? eit.getEitEvent( _eventId ) : _eit.getEitEvent( _eventId ) );
        eit = (EIT)event.getNewTable();
        EitEvent eitEvNew = ( eit!=null ? eit.getEitEvent( _eventId ) : null );
        /*DBG*/Debug.msg(Debug.SI,Debug.MEDIUM,"newMonitorEvent "+event+" old "+eitEvOld+" new "+eitEvNew );

        if ( eitEvNew == null )
        {   // event does not exist any more
            /*DBG*/Debug.msg(Debug.SI,Debug.MEDIUM,"   Running status changed" );
            siEvent = new SIEventObject( _source, (byte)-1, SIEventObjectType.SOURCE_OBJECT_KILLED );
        }
        else if ( eitEvOld != null )
        {   
            if ( eitEvOld.getRunningStatus() != eitEvNew.getRunningStatus() )
            {   // running status has changed
                siEvent = 
                    new SIEventObject( _source, (byte)eitEvNew.getRunningStatus(), SIEventObjectType.RUNNING_STATUS_CHANGED );            
            }
        }
        
        if ( siEvent == null )
        {   // just a version number change
            siEvent = 
                new SIEventObject( _source, (byte)-1, SIEventObjectType.WARNING_ON_INFORMATION_VALIDITY );
        }

        ActiveEventSender.getInstance( ActiveEventSender.SI_APP ).sendEvent( _listHand, siEvent );
    }

    ////////////////// PRIVATE //////////////////
    private int _eventId;
    private EIT _eit;
}