/*
* Copyright(c) Philips Consumer Electronics B.V. 1998
* All rights reserved. Reproduction in whole or in part is
* prohibited without the written consent of the copyright holder.
*
*  Project             : Bali
*  File name   $Logfile: $
*       Last save $Date: 2003/12/24 00:53:09 $
*             $Revision: 1.1.1.1 $
*          Last $Author: mikeseven $
*  Creation Date       : 17/03/98
*  First Author        : S.A.F.A. van den Heuvel
*                        NatLab - Eindhoven
*
*
*  Change history      :
*  ---------------
*
* $Log: MonitorHandler.java,v $
* Revision 1.1.1.1  2003/12/24 00:53:09  mikeseven
* Initial release of the CVS repository for MPEG-4 Systems code
* after the code was corrupted due to reorganization and human 
* errors.
*
* Revision 1.1  2003/04/03 04:29:31  mikeseven
* Initial release of Player3D and MPEG-J for all MPEG-4 reference softwares players with samples.
* For installation, follow instructions in docs/Player3D_setup.doc.
*
* Many thanks to Prashanth Gujjeti and Viswanathan Swaminathan from Sun Microsystems.
*
* -- Mikael Bourges-Sevenier (Mindego)
*
* Revision 1.1  2002/08/23 16:47:27  pg138550
* Changes for MPEG-J. NOT FINAL
*
* Revision 1.1  2000/06/06 18:43:18  alavilli
* After SISF integration into the CORE4 code
*
* Revision 1.1.2.1  1999/07/09 14:17:21  depetris
* Aggiornati i file per lo sviluppo e il rilascio a MPEG-J.
* Integra la versione 2.5 del core.
*
* Revision 1.1  1999-03-18 10:51:56+01  vdheuvel
* start of archive, 29 march 1999
* DAVIC API implementation
*
*/

package org.davic.net.dvb.si;
import  bali.retrieve.Monitor;
import  bali.retrieve.MonitorListener;
import  bali.retrieve.MonitorEvent;
import  bali.table.TableIdentifier;
import  bali.table.SITable;
import  bali.support.ActiveEventSender;
import  bali.support.ListenerHandler;
import  java.util.EventListener;
import  java.util.EventObject;

/**********************
This class is responsible for monitoring changes in SI tables and 
sending events to the listeners.
*/
class MonitorHandler
    implements MonitorListener
{
    /********************
    */
    MonitorHandler( SIInformation source, TableIdentifier ti )
    {
        _source   = source;
        _listHand = new SIEventListenerHandler();
        _mon      = null;
        _ti       = ti;
    }
    
    /********************
    */
    synchronized void addListener( SIEventListener list )
    {
        if ( _listHand.isEmpty() )
        {   // first listener
            _mon = new Monitor( this, _ti );
        }
        _listHand.addListener( list );
    }

    /********************
    New Monitor event.
    */
    public void newMonitorEvent(MonitorEvent event)
    {
        SIEventObject siEvent = 
            new SIEventObject( _source, (byte)-1, SIEventObjectType.WARNING_ON_INFORMATION_VALIDITY );
        ActiveEventSender.getInstance( ActiveEventSender.SI_APP ).sendEvent( _listHand, siEvent );
    }

    /********************
    */
    synchronized void removeListener( SIEventListener list )
    {
        _listHand.removeListener( list );
        if ( _listHand.isEmpty() )
        {   // last listener
            _mon.stopMonitor();
            _mon = null;
        }
    }
    
    ////////////////// PRIVATE //////////////////
    protected SIInformation          _source;
    protected SIEventListenerHandler _listHand;
    protected Monitor                _mon;
    protected TableIdentifier        _ti;
}