/*
* Copyright(c) Philips Consumer Electronics B.V. 1998
* All rights reserved. Reproduction in whole or in part is
* prohibited without the written consent of the copyright holder.
*
*  Project             : Bali
*  File name   $Logfile: /func_test_platf2/org/davic/net/dvb/si/PMTElementaryStream.java $
*       Last save $Date: 2003/12/24 00:53:09 $
*             $Revision: 1.1.1.1 $
*          Last $Author: mikeseven $
*  Creation Date       : 05/11/98
*  First Author        : S.A.F.A. van den Heuvel
*                        NatLab - Eindhoven
*
*
*  Change history      :
*  ---------------
*
* $Log: PMTElementaryStream.java,v $
* Revision 1.1.1.1  2003/12/24 00:53:09  mikeseven
* Initial release of the CVS repository for MPEG-4 Systems code
* after the code was corrupted due to reorganization and human 
* errors.
*
* Revision 1.1  2003/04/03 04:29:31  mikeseven
* Initial release of Player3D and MPEG-J for all MPEG-4 reference softwares players with samples.
* For installation, follow instructions in docs/Player3D_setup.doc.
*
* Many thanks to Prashanth Gujjeti and Viswanathan Swaminathan from Sun Microsystems.
*
* -- Mikael Bourges-Sevenier (Mindego)
*
* Revision 1.1  2002/08/23 16:47:27  pg138550
* Changes for MPEG-J. NOT FINAL
*
* Revision 1.1  2000/06/06 18:43:19  alavilli
* After SISF integration into the CORE4 code
*
* Revision 1.1.2.1  1999/07/09 14:17:22  depetris
* Aggiornati i file per lo sviluppo e il rilascio a MPEG-J.
* Integra la versione 2.5 del core.
*
* Revision 1.3  1999-04-15 09:53:18+02  vdheuvel
* Files changed to hook the system up to the TM demultipler and the
* TM hardware. Changes include, swapping the java.util.Date class
* for the bali.tm.Date class (to prevent using 64 bit floats). Changing
* the HW sect filt used by bali.hw.SectFilt.
*
* Revision 1.2  1999-04-08 12:37:36+02  vdheuvel
* Bug fix, getDescriptors returned PMT descriptors instead of ES descriptors.
* Also removed dependencies on PMT class.
*
* Revision 1.1  1998-11-30 12:36:42+01  vdheuvel
* start of archive, 29 march 1999
* DAVIC API implementation
*
 * 
 * 1     9-11-98 14:01 Vdheuvel
 * Creation
*/
package org.davic.net.dvb.si;
import  bali.table.PMT;
import  bali.table.ES;
import  bali.table.PAT;
import  bali.table.SITableInfo;
import  bali.retrieve.RetrievalMediator;
import  bali.retrieve.RetrievalEvent;
import  java.util.Vector;
import  java.util.Enumeration;
/*DATE_NORMAL*/ import  java.util.Date;
//DATE_TM*/     import  bali.tm.Date;

/**
 * This class represents an elementary stream of a service.
 * <p>
 * For each service there is a PMT describing the elementary streams of the service. 
 * This object represents one such elementary stream.
 * Each PMTElementaryStream object is uniquely defined by the combination of the identifiers
 * original_network_id, transport_stream_id, service_id, elementary_PID.
 *
 * @see SIService
 * @see PMTStreamType
 * @since DAVIC1.3
*/

public class PMTElementaryStream
	implements SIInformation
{

    /** 
     * This method implements SIInformation.retrieveDescriptors (first prototype).
     * @see SIInformation
     * @see SIInformation#retrieveDescriptors
    */
    public SIIterator retrieveDescriptors( ) 
        throws RetrieveFailedException
    {
       	return ( new DescriptorIterator( _es.getDescriptorList() ) );
    }
    
    /** 
     * This method implements SIInformation.retrieveDescriptors (second prototype).
     * @see SIInformation
     * @see SIInformation#retrieveDescriptors
    */
    public SIIterator retrieveDescriptors(
    	short[] someDescriptorTags
    ) throws
    	RetrieveFailedException
    {
    	return ( new DescriptorIterator( _es.getDescriptorList(), someDescriptorTags ) );
    }
    
    /** 
     * This method implements SIInformation.retrieveDescriptorTags.
     * @see SIInformation
     * @see SIInformation#retrieveDescriptorTags
    */
    public short[] retrieveDescriptorTags(
    ) throws
    	RetrieveFailedException
    {
        return(  _es.getDescriptorList().getDescriptorTags() );
    }
    
    /**
     * This method implements SIInformation.getSIDatabase.
     * @see SIInformation
     * @see SIInformation#getSIDatabase
    */
    public SIDatabase getSIDatabase( )
    {
   		return ( _sid );
    }
    
    /**
     * This method implements SIInformation.getUpdateTime.
     * @see SIInformation
     * @see SIInformation#getUpdateTime
    */
    public Date getUpdateTime(  )
    {
   		return ( _es.getSITableInfo().getUpdateTime() );
    }
    
    /**
     * This method implements SIInformation.fromActual.
     * @see SIInformation
     * @see SIInformation#fromActual
    */
    public boolean fromActual(   )
    {
   		return ( _es.getSITableInfo().fromActual() );
    }
    
    /**
     * This method implements SIInformation.getDataSource.
     * @see SIInformation
     * @see SIInformation#getDataSource
    */
    public org.davic.mpeg.TransportStream getDataSource( )
    {
   		return ( _es.getSITableInfo().getSourceTs() );
    }
    
    /**
     * Get the URL adressing the elementary stream.
     * @return The URL describing this elementary stream.
    */
    public java.lang.String getURL(
    )
    {
		String str;
		str = "dvb://"
		    + Integer.toHexString( _orNwId )
		    + "."
		    + Integer.toHexString( _tsId )
		    + "."
		    + Integer.toHexString( _progNo )
		    + "."
		    + Integer.toHexString( _es.getComponentTag() )
		    ;
		return ( str );
    }
    
    /**
     * Get the stream type of this elemetary stream.
     * @return The stream type (some of the possible values are defined in the PMTStreamType interface).
     * @see PMTStreamType
    */
    public byte getStreamType( )
    {
        return (byte)_es.getStreamType();
    }
    
    /**
     * Get the elementary PID.
     * @return The PID the data of elementary stream is sent on in the transport stream.
    */
    
    public short getElementaryPID(
    )
    {
        return (short)_es.getPID();
    }

    ///////////// IMPLEMENTATION //////////////////////////
    private Object     _pmtLock;
	private ES         _es;
	private SIDatabase _sid;
	private int        _orNwId;
	private int        _tsId;
	private int        _progNo;

    /*******************
    */
    PMTElementaryStream( SIDatabase sid, PMT pmt, ES es )
	{
		_sid      = sid;
		_pmtLock  = new Object();
		_orNwId   = pmt.getOriginalNetworkId();
		_tsId     = pmt.getTransportStreamId();
		_progNo   = pmt.getProgramNo();
		_es       = es;
	}

}
