/*
* Copyright(c) Philips Consumer Electronics B.V. 1998
* All rights reserved. Reproduction in whole or in part is
* prohibited without the written consent of the copyright holder.
*
*  Project             : Bali
*  File name   $Logfile: /func_test_platf2/org/davic/net/dvb/si/RetrieveProxy.java $
*       Last save $Date: 2003/12/24 00:53:09 $
*             $Revision: 1.1.1.1 $
*          Last $Author: mikeseven $
*  Creation Date       : 05/11/98
*  First Author        : S.A.F.A. van den Heuvel
*                        NatLab - Eindhoven
*
*
*  Change history      :
*  ---------------
*
* $Log: RetrieveProxy.java,v $
* Revision 1.1.1.1  2003/12/24 00:53:09  mikeseven
* Initial release of the CVS repository for MPEG-4 Systems code
* after the code was corrupted due to reorganization and human 
* errors.
*
* Revision 1.1  2003/04/03 04:29:32  mikeseven
* Initial release of Player3D and MPEG-J for all MPEG-4 reference softwares players with samples.
* For installation, follow instructions in docs/Player3D_setup.doc.
*
* Many thanks to Prashanth Gujjeti and Viswanathan Swaminathan from Sun Microsystems.
*
* -- Mikael Bourges-Sevenier (Mindego)
*
* Revision 1.1  2002/08/23 16:47:27  pg138550
* Changes for MPEG-J. NOT FINAL
*
* Revision 1.1  2000/06/06 18:43:19  alavilli
* After SISF integration into the CORE4 code
*
* Revision 1.1.2.1  1999/07/09 14:17:23  depetris
* Aggiornati i file per lo sviluppo e il rilascio a MPEG-J.
* Integra la versione 2.5 del core.
*
* Revision 1.1  1999-02-12 12:05:01+01  vdheuvel
* start of archive, 29 march 1999
* DAVIC API implementation
*
 * 
 * 1     9-11-98 14:01 Vdheuvel
 * Creation
*/
package org.davic.net.dvb.si;
import  org.davic.mpeg.dvb.DvbTransportStream;
import  bali.table.SDT;
import  bali.table.NIT;
import  bali.table.TS;
import  bali.table.EITpf;
import  bali.table.PAT;
import  bali.table.PMT;
import  bali.table.SITableInfo;
import  bali.table.TableIdentifier;
import  bali.cache.SITableCache;
import  bali.retrieve.RetrievalMediator;
import  bali.retrieve.RetrievalEvent;
import  bali.support.Debug;
import  bali.support.Assert;
import  bali.component.TransportStreamSelection;
import  java.util.Vector;
import  java.util.Enumeration;
/******************
This is support class providing static methods accessing the main tables in the
system.
*/
class RetrieveProxy
{
    /************
    */
    static NIT filterNITActual()
        throws RetrieveFailedException
    {
        /*DBG*/Debug.msg(Debug.SI, "RetrieveProxy.filterNITActual" );
        NIT                res;
        DvbTransportStream ts = getCurrentTs();
        TableIdentifier    ti = new TableIdentifier( TableIdentifier.NIT, ts.getNetworkId() );
        NIT                nit_cache  = (NIT)SITableCache.getSITable( ti );
        NIT                nit_stream = null;
        
        if ( nit_cache == null || !nit_cache.getSITableInfo().fromActual() )
        {   // table not present or not an 'actual' table
            // filter new NIT actual
        
            RetrievalMediator rm;
            RetrievalEvent event;
            do
            {
                /*DBG*/Debug.msg(Debug.SI, Debug.LOW, "RetrieveProxy    access stream" );
                rm = RetrievalMediator.retrieveNITActual(null);
                event = rm.getRetrievalEvent();
                if ( event.getResult() == RetrievalEvent.SUCCESS )
        	    {
                    /*DBG*/Debug.msg(Debug.SI, Debug.LOW, "RetrieveProxy    success" );
                    nit_stream = (NIT)event.getSITable();
                }
            }
            while (  event.getResult() != RetrievalEvent.SUCCESS 
                  && event.getResult() != RetrievalEvent.TIME_OUT  );
        }
        // filtering and cache access done, indicate results
        if ( nit_stream == null && nit_cache == null )
        {
            throw new TimeOutException();
        }
        
        
        if ( nit_stream!=null )
        {   res = nit_stream ;
        }
        else
        {   res = nit_cache ;
        }
        
        // return result
        return res;
    }
    
    /**************
    Return indicated NIT or null if not present
    */
    static NIT filterNIT( int nw_id)
        throws RetrieveFailedException
    {
        NIT             res = null;
        TableIdentifier ti = new TableIdentifier( TableIdentifier.NIT, nw_id );
        NIT             nit_cache  = (NIT)SITableCache.getSITable( ti );
        NIT             nit_stream = null;
    
        if ( nit_cache == null  )
        {   // table not present in cache
    
            RetrievalEvent event;
            do
            {
                RetrievalMediator rm;
                rm = RetrievalMediator.retrieveNIT( null, nw_id );
                event = rm.getRetrievalEvent();
                if ( event.getResult() == RetrievalEvent.SUCCESS )
        	    {
                    nit_stream = (NIT)event.getSITable();
                }
            }
            while (  event.getResult() != RetrievalEvent.SUCCESS 
                  && event.getResult() != RetrievalEvent.TIME_OUT  );
                  
            if ( nit_stream != null )
            {   res = nit_stream;
            }
        }
        else
        {   res = nit_cache;
        }
        return res;
    }
    
    /************
    */
    static SDT filterSDTActual()
        throws RetrieveFailedException
    {
        /*DBG*/Debug.msg(Debug.SI, "RetrieveProxy.filterSDTActual" );
        SDT                res;
        DvbTransportStream ts = getCurrentTs();
        TableIdentifier    ti = new TableIdentifier( TableIdentifier.SDT, ts.getOriginalNetworkId(), ts.getTransportStreamId() );
        SDT                sdt_cache  = (SDT)SITableCache.getSITable( ti );
        SDT                sdt_stream = null;
        
        if ( sdt_cache == null || !sdt_cache.getSITableInfo().fromActual() )
        {   // table not present or not an 'actual' table
            // filter new NIT actual
        
            RetrievalMediator rm;
            RetrievalEvent event;
            do
            {
                 /*DBG*/Debug.msg(Debug.SI, "  access stream" );
                rm = RetrievalMediator.retrieveSDTActual(null);
                event = rm.getRetrievalEvent();
                if ( event.getResult() == RetrievalEvent.SUCCESS )
        	    {
                    sdt_stream = (SDT)event.getSITable();
                }
            }
            while (  event.getResult() != RetrievalEvent.SUCCESS 
                  && event.getResult() != RetrievalEvent.TIME_OUT  );
        }
        // filtering and cache access done, indicate results
        if ( sdt_stream == null && sdt_cache == null )
        {
            throw new TimeOutException();
        }
        
        // create output Vector
        if ( sdt_stream!=null )
        {   res = sdt_stream ;
        }
        else
        {   res = sdt_cache ;
        }
        
        // return result
        return res;
    }

    /************
    */
    static SDT filterSDT( int or_nw_id, int ts_id )
        throws RetrieveFailedException
    {
        SDT                res;
        TableIdentifier    ti = new TableIdentifier( TableIdentifier.SDT, or_nw_id, ts_id );
        SDT                sdt_cache  = (SDT)SITableCache.getSITable( ti );
        SDT                sdt_stream = null;
        
        if ( sdt_cache == null )
        {   // table not present or not an 'actual' table
            // filter new NIT actual
        
            RetrievalMediator rm;
            RetrievalEvent event;
            do
            {
                rm = RetrievalMediator.retrieveSDT(null, or_nw_id, ts_id);
                event = rm.getRetrievalEvent();
                if ( event.getResult() == RetrievalEvent.SUCCESS )
        	    {
                    sdt_stream = (SDT)event.getSITable();
                }
            }
            while (  event.getResult() != RetrievalEvent.SUCCESS 
                  && event.getResult() != RetrievalEvent.TIME_OUT  );
        }
        // filtering and cache access done, indicate results
        if ( sdt_stream == null && sdt_cache == null )
        {
            throw new TimeOutException();
        }
        
        // create output Vector
        if ( sdt_stream!=null )
        {   res = sdt_stream ;
        }
        else
        {   res = sdt_cache ;
        }
        
        // return result
        return res;
    }

    /************
    */
    static EITpf filterEITpf( int or_nw_id, int ts_id, int serv_id )
        throws RetrieveFailedException
    {
        EITpf              res;
        TableIdentifier    ti = new TableIdentifier( TableIdentifier.EITpf, or_nw_id, ts_id, serv_id );
        EITpf              eit_cache  = (EITpf)SITableCache.getSITable( ti );
        EITpf              eit_stream = null;
        
        if ( eit_cache == null )
        {   // table not present or not an 'actual' table
            // filter new table
        
            RetrievalMediator rm;
            RetrievalEvent event;
            do
            {
                rm = RetrievalMediator.retrieveEITpf(null, or_nw_id, ts_id, serv_id );
                event = rm.getRetrievalEvent();
                if ( event.getResult() == RetrievalEvent.SUCCESS )
        	    {
                    eit_stream = (EITpf)event.getSITable();
                }
            }
            while (  event.getResult() != RetrievalEvent.SUCCESS 
                  && event.getResult() != RetrievalEvent.TIME_OUT  );
        }
        // filtering and cache access done, indicate results
        if ( eit_stream == null && eit_cache == null )
        {
            throw new TimeOutException();
        }
        
        // create output Vector
        if ( eit_stream!=null )
        {   res = eit_stream ;
        }
        else
        {   res = eit_cache ;
        }
        
        // return result
        return res;
    }

    /************
    */
    static PAT filterPAT( int or_nw_id, int ts_id )
        throws RetrieveFailedException
    {
        DvbTransportStream ts = getCurrentTs();
        PAT                res;
        TableIdentifier    ti = new TableIdentifier( TableIdentifier.PAT, ts.getOriginalNetworkId(), ts.getTransportStreamId() );
        PAT                pat_cache  = (PAT)SITableCache.getSITable( ti );
        PAT                pat_stream = null;
        
        if ( pat_cache == null )
        {   // table not present
            // filter new table
        
            RetrievalMediator rm;
            RetrievalEvent event;
            do
            {
                rm = RetrievalMediator.retrievePAT(null );
                event = rm.getRetrievalEvent();
                if ( event.getResult() == RetrievalEvent.SUCCESS )
        	    {
                    pat_stream = (PAT)event.getSITable();
                }
            }
            while (  event.getResult() != RetrievalEvent.SUCCESS 
                  && event.getResult() != RetrievalEvent.TIME_OUT  );
        }
        // filtering and cache access done, indicate results
        if ( pat_stream == null && pat_cache == null )
        {
            throw new TimeOutException();
        }
        
        // create output Vector
        if ( pat_stream!=null )
        {   res = pat_stream ;
        }
        else
        {   res = pat_cache ;
        }
        
        // return result
        return res;
    }

    /************
    */
    static PMT filterPMT( int or_nw_id, int ts_id, int serv_id )
        throws RetrieveFailedException
    {
        DvbTransportStream ts = getCurrentTs();
        PAT                pat = filterPAT( or_nw_id, ts_id );
        PMT                res;
        TableIdentifier    ti = new TableIdentifier( TableIdentifier.PMT, or_nw_id, ts_id, serv_id );
        PMT                pmt_cache  = (PMT)SITableCache.getSITable( ti );
        PMT                pmt_stream = null;
        
        if ( pmt_cache == null )
        {   // table not present
            // filter new table
            /*DBG*/Debug.msg(Debug.SI, "Filter PMT");
            RetrievalMediator rm;
            RetrievalEvent event;
            do
            {
                /*DBG*/Debug.msg(Debug.SI, "Filter PMT : pid "+pat.getProgramPid(serv_id));
                rm = RetrievalMediator.retrievePMT( null, pat.getProgramPid(serv_id), serv_id );
                event = rm.getRetrievalEvent();
                if ( event.getResult() == RetrievalEvent.SUCCESS )
        	    {
                    pmt_stream = (PMT)event.getSITable();
                }
            }
            while (  event.getResult() != RetrievalEvent.SUCCESS 
                  && event.getResult() != RetrievalEvent.TIME_OUT  );
        }
        // filtering and cache access done, indicate results
        if ( pmt_stream == null && pmt_cache == null )
        {
            throw new TimeOutException();
        }
        
        // create output Vector
        if ( pmt_stream!=null )
        {   res = pmt_stream ;
        }
        else
        {   res = pmt_cache ;
        }
        
        // return result
        return res;
    }

    /****************
    */
    private static DvbTransportStream getCurrentTs()
    {
        return TransportStreamSelection.getInstance().getCurrentTs();
    }


}