/*
* Copyright(c) Philips Consumer Electronics B.V. 1998
* All rights reserved. Reproduction in whole or in part is
* prohibited without the written consent of the copyright holder.
*
*  Project             : Bali
*  File name   $Logfile: $
*       Last save $Date: 2003/12/24 00:53:09 $
*             $Revision: 1.1.1.1 $
*          Last $Author: mikeseven $
*  Creation Date       : 03/12/98
*  First Author        : S.A.F.A. van den Heuvel
*                        NatLab - Eindhoven
*
*
*  Change history      :
*  ---------------
*
* $Log: SIBouquet.java,v $
* Revision 1.1.1.1  2003/12/24 00:53:09  mikeseven
* Initial release of the CVS repository for MPEG-4 Systems code
* after the code was corrupted due to reorganization and human 
* errors.
*
* Revision 1.1  2003/04/03 04:29:32  mikeseven
* Initial release of Player3D and MPEG-J for all MPEG-4 reference softwares players with samples.
* For installation, follow instructions in docs/Player3D_setup.doc.
*
* Many thanks to Prashanth Gujjeti and Viswanathan Swaminathan from Sun Microsystems.
*
* -- Mikael Bourges-Sevenier (Mindego)
*
* Revision 1.1  2002/08/23 16:47:27  pg138550
* Changes for MPEG-J. NOT FINAL
*
* Revision 1.1  2000/06/06 18:43:19  alavilli
* After SISF integration into the CORE4 code
*
* Revision 1.1.2.1  1999/07/09 14:17:23  depetris
* Aggiornati i file per lo sviluppo e il rilascio a MPEG-J.
* Integra la versione 2.5 del core.
*
* Revision 1.3  1999-05-05 14:29:41+02  vdheuvel
* Changes required after first Bouquet tests.
*
* Revision 1.2  1999-04-15 09:53:18+02  vdheuvel
* Files changed to hook the system up to the TM demultipler and the
* TM hardware. Changes include, swapping the java.util.Date class
* for the bali.tm.Date class (to prevent using 64 bit floats). Changing
* the HW sect filt used by bali.hw.SectFilt.
*
* Revision 1.1  1999-03-18 12:33:48+01  vdheuvel
* start of archive, 29 march 1999
* DAVIC API implementation
*
*/

package org.davic.net.dvb.si;
import  bali.table.SDT;
import  bali.table.Serv;
import  bali.table.BAT;
import  bali.table.TS;
import  bali.table.SITableInfo;
import  bali.table.TableIdentifier;
import  bali.retrieve.RetrievalMediator;
import  bali.retrieve.RetrievalEvent;
import  bali.support.Debug;
import  java.util.Vector;
import  java.util.Enumeration;
/*DATE_NORMAL*/ import  java.util.Date;
//DATE_TM*/     import  bali.tm.Date;

/**
 * This class (together with the SITransportStream class) represents a sub-table of the Bouquet 
 * Association Table (BAT) describing a particular bouquet. Objects of this class are trackable.
 * <p>
 * Each SIBouquet object is uniquely defined by the identifier bouquet_id.
 *
 * @since DAVIC1.3
*/

public class SIBouquet
	implements
		SIInformation
{

    
    /** 
     * This method implements SIInformation.retrieveDescriptors (first prototype).
     * @see SIInformation
     * @see SIInformation#retrieveDescriptors
    */
    public SIIterator retrieveDescriptors(
    ) throws
    	RetrieveFailedException
    {
        synchronized ( _batLock )
        {
        	return ( new DescriptorIterator( _bat.getDescriptorList() ) );
        }
    }
    
    /** 
     * This method implements SIInformation.retrieveDescriptors (second prototype).
     * @see SIInformation
     * @see SIInformation#retrieveDescriptors
    */
    public SIIterator retrieveDescriptors(
    	short[] someDescriptorTags
    ) throws
    	RetrieveFailedException
    {
        synchronized ( _batLock )
        {
    		return ( new DescriptorIterator( _bat.getDescriptorList(), someDescriptorTags ) );
        }
    }
    
    /** 
     * This method implements SIInformation.retrieveDescriptorTags.
     * @see SIInformation
     * @see SIInformation#retrieveDescriptorTags
    */
    public short[] retrieveDescriptorTags(
    ) throws
    	RetrieveFailedException
    {
        synchronized ( _batLock )
        {
            return(  _bat.getDescriptorList().getDescriptorTags() );
        }
    }
    
    /**
     * This method implements SIInformation.getSIDatabase.
     * @see SIInformation
     * @see SIInformation#getSIDatabase
    */
    public SIDatabase getSIDatabase(
    )
    {
        synchronized ( _batLock )
        {
    		return ( _sid );
        }
    }
    
    /**
     * This method implements SIInformation.getUpdateTime.
     * @see SIInformation
     * @see SIInformation#getUpdateTime
    */
    public Date getUpdateTime(
    )
    {
        synchronized ( _batLock )
        {
    		return ( _bat.getSITableInfo().getUpdateTime() );
    	}
    }
    
    /**
     * This method implements SIInformation.fromActual.
     * @see SIInformation
     * @see SIInformation#fromActual
    */
    public boolean fromActual(
    )
    {
        synchronized ( _batLock )
        {
    		return ( _bat.getSITableInfo().fromActual() );
    	}
    }
    
    /**
     * This method implements SIInformation.getDataSource.
     * @see SIInformation
     * @see SIInformation#getDataSource
    */
    public org.davic.mpeg.TransportStream getDataSource(
    )
    {
        synchronized ( _batLock )
        {
    		return ( _bat.getSITableInfo().getSourceTs() );
    	}
    }
    
    /**
     * Add a listener.
     * @param aListener The object to be notified of events concerning this object.
     * @see SIEventListener
     * @see SIEventObject
    */
    public void addListener(
    	SIEventListener aListener
    )
    {
        _monHand.addListener( aListener );
    }
    
    /**
     * Remove a listener
     * @param aListener The object no longer interested in events concerning this object.
     * @see SIEventListener
     * @see SIEventObject
    */
    public void removeListener(
    	SIEventListener aListener
    )
    {
        _monHand.removeListener( aListener );
    }
    
    /**
     * Update the information contained in the object from the actual transport stream of this SIDatabase.
     *
     * @exception TimeOutException         The required information could not be filtered within the time out period.
     * @exception LackOfResourcesException The resources required to obtain the information are not available at this time.
     * @exception NotImplementedException  This feature is not implemented on this platform.
    */
    public void retrieveUpdate(
    ) throws
    	RetrieveFailedException
    {
		// goal: obtain new version of the BAT of this transport stream network
		// 1 Start filtering the BAT.
		// 2 When filter failed, return cached information

        BAT old_bat = null;
        BAT new_bat = null;
        synchronized ( _batLock )
        {
            old_bat = _bat;
        }

        synchronized( _retrieveUpdateLock )
        {
            RetrievalEvent event;
            boolean ready = false;
            RetrievalMediator rm;
            
            do
            {
                rm = RetrievalMediator.retrieveBAT(null, old_bat.getBouquetId() );
                event = rm.getRetrievalEvent();
                if ( event.getResult() == RetrievalEvent.SUCCESS )
        	    {
                    new_bat = (BAT)event.getSITable();
                }
            }
            while (  event.getResult() != RetrievalEvent.SUCCESS 
                  && event.getResult() != RetrievalEvent.TIME_OUT  );

            // check BAT cache
            // TODO add code when BAT cache is added
        }
        // retrieveUpdate finished, process results
        if ( new_bat != null )
        {   // update _nit of this object
            synchronized( _batLock )
            {   _bat = new_bat;
            }
        }
        else
        {   throw new TimeOutException();
        }
    }
    
    /**
     * Get the identification.
     * @return The bouquet identification of this bouquet.
    */
    public int getBouquetId(
    )
    {
        synchronized ( _batLock )
        {
            return ( _bat.getBouquetId() );
    	}
    }
    
    /** 
     * This method returns the name of the bouquet represented by this SIBouquet object. 
     * The name is extracted from the bouquet_name_descriptor or optionally from the 
     * multilingual_bouquet_name_descriptor. When this information is not available "" is returned. 
     * All control characters as defined in ETR 211 are ignored. For each character the DVB-SI 8 bit character 
     * code is mapped to the appropriate Unicode representation
     * @return The bouquet name of this bouquet.
    */
    public java.lang.String getBouquetName(
    )
    {
        synchronized ( _batLock )
        {
            return ( _bat.getBouquetName() );
    	}
    }
    
    /** 
     * This method returns the short name (ETR 211) of the bouquet represented by this SIBouquet object 
     * without emphasis marks. The name is extracted from the bouquet_name_descriptor or optionally 
     * from the multilingual_bouquet_name_descriptor. 
     * When this information is not available "" is returned. For each character the DVB-SI 8 bit character 
     * code is mapped to the appropriate Unicode representation.
     * @return The short bouquet name of this bouquet.
    */
    public String getShortBouquetName()
    {
        synchronized ( _batLock )
        {
            return ( _bat.getShortBouquetName() );
    	}
    }
    
    /**
     * Retrieve information associated with transport streams belonging to the bouquet.
     * @param someDescriptorTags A list of hints for descriptors (identified by their tags) the application is interested in. If the array contains -1 as its one and only element, The application is interested in all descriptors. If someDescriptorTags is null, the application is not interested in descriptors. All non applicable tag values are ignored.
     * @return                   A set of SITransportStream objects of the transport stream holding services of this bouquet (may be empty).
     * @exception TimeOutException         The required information could not be filtered within the time out period.
     * @exception LackOfResourcesException The resources required to obtain the information are not available at this time.
     * @exception NotImplementedException  This feature is not implemented on this platform.
     * @see SITransportStream
     * @see SIIterator
     * @see DescriptorTag
    */
    public SIIterator retrieveSITransportStreams(
    	short[] someDescriptorTags
    ) throws
    	RetrieveFailedException
    {
		TS[] arTs;
		synchronized ( _batLock )
		{
			arTs = _bat.getTSs();
		}

		Vector vec =  new Vector(1,1);

		for ( int i=0; i<arTs.length; i++ )
		{
			vec.addElement( new SITransportStream( _sid, arTs[i] ) );
		}

		return ( new SIIteratorImpl( vec ) );
    }
    
    /**
     * Retrieve information associated with services belonging to the bouquet.
     * @param someDescriptorTags A list of hints for descriptors (identified by their tags) the application is interested in. If the array contains -1 as its one and only element, the application is interested in all descriptors. If someDescriptorTags is null, the application is not interested in descriptors. All non applicable tag values are ignored.
     * @return                   A set of SIService objects representing the services in this bouquet (may be empty).
     * @exception TimeOutException         The required information could not be filtered within the time out period.
     * @exception LackOfResourcesException The resources required to obtain the information are not available at this time.
     * @exception NotImplementedException  This feature is not implemented on this platform.
     * @see SIService
     * @see SIIterator
     * @see DescriptorTag
    */
    public SIIterator retrieveSIServices(
    	short[] someDescriptorTags
    ) throws
    	RetrieveFailedException
    {
        Vector vec;
		TableIdentifier[] arServ;
		synchronized ( _batLock )
		{
			arServ = _bat.getServices();
		}
		
		// filter all requested SDT. When the SDT of one of the services could not
		// be filtered, throw TimeOutException.
		SDT sdt;
		Serv service;
		int[] arId1;
		vec  =  new Vector( arServ.length );
		for ( int i=0; i<arServ.length; i++ )
        {   
    		if ( arServ[i] != null )
    		{   // service has not been filtered yet, filter sdt and obtain service

    		    arId1 = arServ[i].getIdentifiers();
                /*DBG*/ Debug.msg( Debug.SI, Debug.IMPL, "SIBouquet,"+i+" filter SDT("+arId1[0]+","+arId1[1]+")" );
                try
                {
        		    sdt = RetrieveProxy.filterSDT( arId1[0], arId1[1] );
    
        		    // check all services of this Bouquet if they fit a service in this SDT.
        		    for( int j=0; j<arServ.length; j++ )
        		    {   
                        /*DBG*/ Debug.msg( Debug.SI, Debug.IMPL, "SIBouquet, check SDT("+arId1[0]+","+arId1[1]+") " );
        		        if (  arServ[j]!=null )
        		        {   // this service has not yet been processed.
            		        int[] arId2 = arServ[j].getIdentifiers();
                            /*DBG*/ Debug.msg( Debug.SI, Debug.IMPL, "SIBouquet, check SDT("+arId1[0]+","+arId1[1]+") for service("+j+") "+arId2[0]+","+arId2[1]+","+arId2[2] );
        		            if (  arId2[0] == sdt.getOriginalNetworkId()
        		               && arId2[1] == sdt.getTransportStreamId()
        		               )
                		    {   // Services from the same source ts.
                                /*DBG*/ Debug.msg( Debug.SI, Debug.IMPL, "SIBouquet, service in correct ts found, check for service "+arId2[2] );
                		        service = sdt.getService( arId2[2] );
                		        if ( service!=null )
                		        {   // service found, add to result and remove service from list.
                                    /*DBG*/ Debug.msg( Debug.SI, Debug.IMPL, "SIBouquet,"+j+" service found " );
                		            vec.addElement( new SIService( _sid, sdt, service ));
            	        		    arServ[j] = null;
                		        }
                		    }
        		        }
        		    }
        		}
        		catch ( RetrieveFailedException e )
        		{}    
            }
            if ( vec.size() != arServ.length )
            {   // not all services filtered
                throw new TimeOutException( vec );
            }
		}
		return new SIIteratorImpl( vec );
    }
    ////////////////// PACKAGE /////////////////////
    /*******************
    */
    SIBouquet( SIDatabase sid, BAT bat ) 		
    {
        _bat      = bat;
		_sid      = sid;
		_batLock  = new Object();
		_retrieveUpdateLock = new Object();
		_monHand  = new MonitorHandler( this, bat.getTableIdentifier() );
	}

    
    ////////////////// IMPLEMENTATION //////////////
    private SIDatabase _sid;
    private BAT        _bat;
    private Object     _batLock;
    private Object     _retrieveUpdateLock;
	private MonitorHandler _monHand;
}
