/*
* Copyright(c) Philips Consumer Electronics B.V. 1998
* All rights reserved. Reproduction in whole or in part is
* prohibited without the written consent of the copyright holder.
*
*  Project             : Bali
*  File name   $Logfile: /func_test_platf2/org/davic/net/dvb/si/SIDatabase.java $
*       Last save $Date: 2003/12/24 00:53:10 $
*             $Revision: 1.1.1.1 $
*          Last $Author: mikeseven $
*  Creation Date       : 05/11/98
*  First Author        : S.A.F.A. van den Heuvel
*                        NatLab - Eindhoven
*
*
*  Change history      :
*  ---------------
*
* $Log: SIDatabase.java,v $
* Revision 1.1.1.1  2003/12/24 00:53:10  mikeseven
* Initial release of the CVS repository for MPEG-4 Systems code
* after the code was corrupted due to reorganization and human 
* errors.
*
* Revision 1.1  2003/04/03 04:29:32  mikeseven
* Initial release of Player3D and MPEG-J for all MPEG-4 reference softwares players with samples.
* For installation, follow instructions in docs/Player3D_setup.doc.
*
* Many thanks to Prashanth Gujjeti and Viswanathan Swaminathan from Sun Microsystems.
*
* -- Mikael Bourges-Sevenier (Mindego)
*
* Revision 1.1  2002/08/23 16:47:27  pg138550
* Changes for MPEG-J. NOT FINAL
*
* Revision 1.1  2000/06/06 18:43:19  alavilli
* After SISF integration into the CORE4 code
*
* Revision 1.1.2.1  1999/07/09 14:17:23  depetris
* Aggiornati i file per lo sviluppo e il rilascio a MPEG-J.
* Integra la versione 2.5 del core.
*
* Revision 1.2  1999-05-05 10:49:51+02  vdheuvel
* Changes required after first Bouquet tests.
*
* Revision 1.1  1999-02-12 15:08:40+01  vdheuvel
* start of archive, 29 march 1999
* DAVIC API implementation
*
 * 
 * 3     10-11-98 15:54 Vdheuvel
 * bug fix in retrieveSITime*.
 * 
 * 2     10-11-98 11:21 Vdheuvel
 * Added SITime implementation.
 * 
 * 1     9-11-98 14:01 Vdheuvel
 * Creation
 * 
 * 1     9-11-98 13:56 Vdheuvel
 * Creation
*/
package org.davic.net.dvb.si;
import  org.davic.mpeg.dvb.DvbTransportStream;
import  bali.component.TransportStreamSelection;
import  bali.support.Assert;
import  bali.support.Debug;
import  bali.retrieve.RetrievalMediator;
import  bali.retrieve.RetrievalEvent;
import  bali.cache.SITableCache;
import  bali.table.SITable;
import  bali.table.TableIdentifier;
import  bali.table.BAT;
import  bali.table.SDT;
import  bali.table.Serv;
import  bali.table.NIT;
import  bali.table.TS;
import  bali.table.TDT;
import  bali.table.TOT;
import  java.util.Vector;
import  java.util.Enumeration;

/**
 * This class represents the root of the SI information hierarchy. There is one SIDatabase per network 
 * interface. In a system with a single network interface there is only one SIDatabase object.
 * @since DAVIC1.3
*/

public class SIDatabase
{
    /**
     * Return an array of SIDatabase objects (one object per network interface).
     * In a system with one network interface, the length of this array will be one.
     * The network interface of each SIDatabase is used as data source for all new 
     * data accessed by this SIDatabase or SIInformation instances obtained from it.
     * <p>
     * This is the first method to be called to access the DVB-SI API. The returned SIDatabase objects
     * provide the access point to the DVB-SI information.
     * @return An array of SIDatabase objects, one per network interface.
    */
    public synchronized static SIDatabase[] getSIDatabase()
    {
        if ( _arSid == null )
        {   // this version only has one SIDatabase
            _arSid = new SIDatabase[1];
            _arSid[0] = new SIDatabase();
        }
        return _arSid;
    }
    
    /**
     * Retrieve information associated with bouquets.
     * A bouquet can be specified by its identification (when bouquetId is set to -1, 
     * all bouquets are retrieved)
     * 
     * @param bouquetId          Identification of the bouquet to be retrieved.
     * @param someDescriptorTags A list of hints for descriptors (identified by their tags) the application is interested in. If the array contains -1 as its one and only element, the application is interested in all descriptors. If someDescriptorTags is null, the application is not interested in descriptors. All non applicable tag values are ignored
     * @return                   A set of SIBouquet objects (may be empty). 
     * @exception TimeOutException         The required information could not be filtered within the time out period.
     * @exception NotImplementedException  This feature is not implemented on this platform. 
     * @exception LackOfResourcesException The resources required to obtain the information are not available at this time.
     * 
     * @see SIBouquet
     * @see SIIterator
     * @see DescriptorTag
    */
    public SIIterator retrieveSIBouquets(
    	int bouquetId,
    	short[] someDescriptorTags
    ) throws
    	RetrieveFailedException
    {   
        Vector vec = new Vector( 1, 1 );
        if ( bouquetId != -1 )
        {
            RetrievalEvent event;
            do
            {
                RetrievalMediator rm;
                rm = RetrievalMediator.retrieveBAT( null, bouquetId );
                event = rm.getRetrievalEvent();
            }
            while (  event.getResult() != RetrievalEvent.SUCCESS 
                  && event.getResult() != RetrievalEvent.TIME_OUT  );
   
            vec.addElement( new SIBouquet( this, (BAT)event.getSITable()) );
        }
        else
        {   // wildcard
            SITable[] ar_filter;
            RetrievalEvent event;
            do
            {
                RetrievalMediator rm;
                rm = RetrievalMediator.retrieveAllBAT( null );
                event = rm.getRetrievalEvent();
            }
            while (  event.getResult() != RetrievalEvent.SUCCESS 
                  && event.getResult() != RetrievalEvent.TIME_OUT  );
            
            ar_filter = event.getSITables();
            /*DBG*/ Debug.msg( Debug.SI, Debug.IMPL, "SIDatabase.retrieveSIBouquets #bouq "+ar_filter.length );
            for ( int i=0; i<ar_filter.length; i++ )
            {
                vec.addElement( new SIBouquet( this, (BAT)ar_filter[i]) );
            }
        }
        return new SIIteratorImpl( vec );
    }
    
    /**
     * Retrieve information associated with the actual network.
     * The actual network is the network carrying the transport stream currently 
     * selected by the network interface connected to this SIDatabase.
     * 
     * @param someDescriptorTags A list of hints for descriptors (identified by their tags) the application is interested in. If the array contains -1 as its one and only element, the application is interested in all descriptors. If someDescriptorTags is null, the application is not interested in descriptors. 
     *                           All non applicable tag values are ignored
     * @return    The SINetwork object representing the actual network
     * @exception TimeOutException         The required information could not be filtered within the time out period.
     * @exception NotImplementedException  This feature is not implemented on this platform.
     * @exception LackOfResourcesException The resources required to obtain the information are not available at this time.
     * 
     * @see SINetwork
     * @see DescriptorTag
    */
    public SINetwork retrieveActualSINetwork(
    	short[] someDescriptorTags
    ) throws
    	RetrieveFailedException
    {  
        return new SINetwork( this, RetrieveProxy.filterNITActual() );
    }
    
    /**
     * Retrieve information associated with networks.
     * A network can be specified by its identification (when networkId is set to -1, all networks are retrieved)
     * 
     * @param networkId          Identification of the network to be retrieved
     * @param someDescriptorTags A list of hints for descriptors (identified by their tags) the application is interested in. If the array contains -1 as its one and only element, the application is interested in all descriptors. If someDescriptorTags is null, the application is not interested in descriptors. All non applicable tag values are ignored.
     * @return                   A set of SINetwork objects (may be empty).
     * @exception TimeOutException         The required information could not be filtered within the time out period.
     * @exception NotImplementedException  This feature is not implemented on this platform.
     * @exception LackOfResourcesException The resources required to obtain the information are not available at this time.
     * @see SINetwork
     * @see SIIterator
     * @see DescriptorTag
    */
    public SIIterator retrieveSINetworks(
    	int networkId,
    	short[] someDescriptorTags
    ) throws
    	RetrieveFailedException
    {  
        /*DBG*/ Debug.msg( Debug.SI, "SIDatabase.retrieveSINetworks ("+networkId+")" );
        // three different situations can occur:
        // 1 requested NIT is NIT actual
        // 2 requested NIT is not NIT actual
        // 3 all NIT's are requested
        //     NIT actual plus any in cache
        DvbTransportStream ts  = getCurrentTs();
        Vector             vec = new Vector(1,1);

        if      ( networkId == -1 )
        {   // wildcard NIT retrieval
            /*DBG*/ Debug.msg( Debug.SI, Debug.IMPL, "SIDatabase wildcard NIT retrieval" );
            // 1 add NIT actual, pass on any exceptions
            vec.addElement( retrieveActualSINetwork( null ) );
            
            // filter all NIT other
            SITable[] ar_filter;
            {
                RetrievalEvent event;
                do
                {
                    RetrievalMediator rm;
                    rm = RetrievalMediator.retrieveAllNITother( null );
                    event = rm.getRetrievalEvent();
                }
                while (  event.getResult() != RetrievalEvent.SUCCESS 
                      && event.getResult() != RetrievalEvent.TIME_OUT  );
                
                ar_filter = event.getSITables();
            }
            
            /*DBG*/ Debug.msg( Debug.SI, Debug.IMPL, "SIDatabase.retrieveSINetworks mark1" );
            // add filtered NIT's
            for ( int i=0; i< ar_filter.length; i++ )
            {
                NIT nit = (NIT)ar_filter[i];
                if ( nit.getNetworkId() != ts.getNetworkId() )
                {   vec.addElement( new SINetwork( this, nit ) );
                }
            }
                        
            /*DBG*/ Debug.msg( Debug.SI, Debug.IMPL, "SIDatabase.retrieveSINetworks mark2" );
            // add all cached NITother not allready filtered
            SITable[] ar_sitable = 
              SITableCache.getSITables( new TableIdentifier( TableIdentifier.NIT, -1 ) );
//            NITCache  nc      = (NITCache)NITCache.getInstance();
//            NIT[]     ar_nit  = nc.getAllNITs();
            int       j;
            boolean   found;
            /*DBG*/ Debug.msg( Debug.SI, Debug.IMPL, "SIDatabase.retrieveSINetworks mark3" );
            for ( int i=0; i< ar_sitable.length; i++ )
            {
                NIT nit = (NIT)ar_sitable[i];
                j = 0;
                found = false;
                while ( !found && j<ar_filter.length )
                {
                    found = (((NIT)ar_filter[j]).getNetworkId() == nit.getNetworkId());
                    j++;
                }
                if (  nit.getNetworkId() != ts.getNetworkId() 
                   && !found 
                   )
                {   vec.addElement( new SINetwork( this, nit ) );
                }
            }
        }
        else if ( networkId == ts.getNetworkId() )
        {   // NIT actual
            SINetwork res = retrieveActualSINetwork( null );
            vec.addElement( res );
        }
        else 
        {   // NIT other
            NIT res = RetrieveProxy.filterNIT( networkId );
            if ( res == null )
            {   // filter did not succeed
                throw new TimeOutException();
            }
            vec.addElement( new SINetwork( this, res ) );
        }
        // result in vec or exception thrown
        // create SIIterator
        return new SIIteratorImpl( vec );
    }
    
    /**
     * Retrieve information associated with the actual transport stream.
     * The actual transport stream is the transport stream currently selected 
     * by the network interface connected to this SIDatabase.
     *
     * @param someDescriptorTags A list of hints for descriptors (identified by their tags) the application is interested in. If the array contains -1 as its one and only element, the application is interested in all descriptors. If someDescriptorTags is null, the application is not interested in descriptors. All non applicable tag values are ignored.
     * @return                   The SITransportStream object representing the actual transport stream.
     * @exception InformationSourceNoMoreAvailableException The actual transport stream is no longer described in the NIT actual.
     * @exception TimeOutException         The required information could not be filtered within the time out period.
     * @exception NotImplementedException  This feature is not implemented on this platform.
     * @exception LackOfResourcesException The resources required to obtain the information are not available at this time.
     * @see SITransportStream
     * @see DescriptorTag
    */
    public SITransportStream retrieveActualSITransportStream(
    	short[] someDescriptorTags
    ) throws
    	RetrieveFailedException
    {
        DvbTransportStream ts  = getCurrentTs();
        
        // access SINetwork of actual network
        // pass on any exceptions
        NIT nit     = RetrieveProxy.filterNITActual();
        TS  nit_ts  = nit.getTS( ts.getOriginalNetworkId(), ts.getTransportStreamId() );
        
        if ( nit_ts == null )
        {   // TS not present in this network
            throw new InformationSourceNoMoreAvailableException();
        }
        
        return new SITransportStream( this, nit_ts );
    }
    
    /**
     * Retrieve information associated with transport streams.
     * The required transport streams can be specified by their identification (-1 means 'any' for 
     * originalNetworkId and tranportStreamId). When the same transport stream exists in more than one network,
     * a separate object will be returned for each occurrence.
     * 
     * @param originalNetworkId  Identification of the transport streams to be retrieved : original network identification
     * @param transportStreamId  Identification of the transport streams to be retrieved : transport stream identification
     * @param someDescriptorTags A list of hints for descriptors (identified by their tags) the application is interested in. If the array contains -1 as its one and only element, the application is interested in all descriptors. If someDescriptorTags is null, the application is not interested in descriptors. All non applicable tag values are ignored.
     * @return                   A set of SITransportStream objects (may be empty).
     * @exception TimeOutException         The required information could not be filtered within the time out period.
     * @exception NotImplementedException  This feature is not implemented on this platform.
     * @exception LackOfResourcesException The resources required to obtain the information are not available at this time.
     * @see SITransportStream
     * @see SIIterator
     * @see DescriptorTag
    */
    public SIIterator retrieveSITransportStreams(
    	int originalNetworkId,
    	int transportStreamId,
    	short[] someDescriptorTags
    ) throws
    	RetrieveFailedException
    {
        // When wildcards are present:
        // 1 Filter all networks, this always includes the NITactual 
        //   (see retrieveNITs) (all exceptions are passed on)
        // 2 Create SITransportStream objects for every TS in each NIT
        //   matching the wild cards
        // No wildcards present the same process should be followed.
        int  i,j;
        NIT  nit;  
        TS[] arTs;
        Vector  vec = new Vector();
        
        // TSs in actual network
        nit = RetrieveProxy.filterNITActual();
        arTs = nit.getTSs();
        for ( i=0; i<arTs.length; i++ )
        {
            if (  ( arTs[i].getOriginalNetworkId() == originalNetworkId || originalNetworkId == -1 )
               && ( arTs[i].getTransportStreamId() == transportStreamId || transportStreamId == -1 )
               )
            {
                vec.addElement( new SITransportStream( this, arTs[i] ) );
            }
        }
        // TSs in other network
        NIT[] ar_nit = 
          (NIT[])SITableCache.getSITables( new TableIdentifier( TableIdentifier.NIT, -1 ) );
//        NITCache  nc      = (NITCache)NITCache.getInstance();
//        NIT[]     ar_nit  = nc.getAllNITs();
        for ( i=0; i<ar_nit.length; i++ )
        {
            arTs = ar_nit[i].getTSs();
            for ( j=0; j<arTs.length; j++ )
            {   
                if (  ( arTs[i].getOriginalNetworkId() == originalNetworkId || originalNetworkId == -1 )
                   && ( arTs[i].getTransportStreamId() == transportStreamId || transportStreamId == -1 )
                   )
                {
                    vec.addElement( new SITransportStream( this, arTs[j] ) );
                }
            }
        }
        
        return new SIIteratorImpl( vec );
    }
    
    /**
     * Retrieve information associated with the actual services.
     * The actual services are the services in the transport stream 
     * currently selected by the network interface connected to this SIDatabase.
     * 
     * @param someDescriptorTags A list of hints for descriptors (identified by their tags) the application is interested in. If the array contains -1 as its one and only element, the application is interested in all descriptors. If someDescriptorTags is null, the application is not interested in descriptors. All non applicable tag values are ignored.
     * @return                   The SIService objects representing the services in the actual  transport stream.
     * @exception TimeOutException         The required information could not be filtered within the time out period.
     * @exception NotImplementedException  This feature is not implemented on this platform.
     * @exception LackOfResourcesException The resources required to obtain the information are not available at this time.
     * @see SIService
     * @see SIIterator
     * @see DescriptorTag
    */
    public SIIterator retrieveActualSIServices(
    	short[] someDescriptorTags
    ) throws
    	RetrieveFailedException
    {
        Vector vec = new Vector();
        SDT sdt = RetrieveProxy.filterSDTActual();
        Serv[] arServ = sdt.getServices();
        for ( int i=0; i<arServ.length; i++ )
        {
            vec.addElement( new SIService( this, sdt, arServ[i] ) );
        }
        return new SIIteratorImpl( vec );
    }
    
    /**
     * Retrieve information associated with services.
     * The required services can be specified by their identification (-1 means 'any' for originalNetworkId, 
     * tranportStreamId and serviceId)
     * 
     * @param originalNetworkId  Identification of the services to be retrieved : original network identification
     * @param transportStreamId  Identification of the services to be retrieved : transport stream identification
     * @param serviceId          Identification of the services to be retrieved : service identification
     * @param someDescriptorTags A list of hints for descriptors (identified by their tags) the application is interested in. If the array contains -1 as its one and only element, the application is interested in all descriptors. If someDescriptorTags is null, the application is not interested in descriptors. All non applicable tag values are ignored.
     * @return                   A set of SIService objects (may be empty).
     * @exception TimeOutException         The required information could not be filtered within the time out period.
     * @exception NotImplementedException  This feature is not implemented on this platform.
     * @exception LackOfResourcesException The resources required to obtain the information are not available at this time.
     * @see SIService
     * @see SIIterator
     * @see DescriptorTag
    */
    public SIIterator retrieveSIServices(
    	int originalNetworkId,
    	int transportStreamId,
    	int serviceId,
    	short[] someDescriptorTags
    ) throws
    	RetrieveFailedException
    {
        // no WildCard
        //   Actual service
        //   Other service
        // Wildcard
        //   Get services from SDT actual and try to filter any SDT other (cache or stream)
        //   Add any services from other SDTs
        
        /*DBG*/ Debug.msg( Debug.SI, "SIDatabase.retrieveSIServices ("+originalNetworkId+", "+transportStreamId+", "+serviceId+")" );
        DvbTransportStream ts  = getCurrentTs();
        Vector             vec = new Vector();
        if   (  originalNetworkId == ts.getOriginalNetworkId() 
             && transportStreamId == ts.getTransportStreamId() 
             )
        {   // SDT actual
            /*DBG*/ Debug.msg( Debug.SI, Debug.LOW, "SIDatabase.retrieveSIServices actual ts" );
            SDT    sdt    = RetrieveProxy.filterSDTActual();
            Serv[] arServ = sdt.getServices();
            for ( int i=0; i< arServ.length; i++ )
            {
                if ( arServ[i].getServiceId() == serviceId || serviceId == -1 )
                {
                    vec.addElement( new SIService( this, sdt, arServ[i] ) );
                }
            }
            if ( vec.size() == 0 )
            {   throw new TimeOutException();
            }
        }
        else if (  originalNetworkId != -1
                && transportStreamId != -1
                )
        {   // SDT other
            /*DBG*/ Debug.msg( Debug.SI, Debug.LOW, "SIDatabase.retrieveSIServices other ts" );
            SDT    sdt    = RetrieveProxy.filterSDT( originalNetworkId, transportStreamId );
            Serv[] arServ = sdt.getServices();
            for ( int i=0; i< arServ.length; i++ )
            {
                if ( arServ[i].getServiceId() == serviceId || serviceId == -1 )
                {
                    vec.addElement( new SIService( this, sdt, arServ[i] ) );
                }
            }
            if ( vec.size() == 0 )
            {   throw new TimeOutException();
            }
        }
        else
        {
            /*DBG*/ Debug.msg( Debug.SI, Debug.LOW, "SIDatabase.retrieveSIServices wildcard in ts" );
            // wildcard in SDT selection
            Vector vecSDT = new Vector( 1, 1 );
            int i,j;
            
            // filter all SDT other
            SITable[] ar_filter;
            {
                RetrievalEvent event;
                do
                {
                    RetrievalMediator rm;
                    rm = RetrievalMediator.retrieveAllSDTother( null );
                    event = rm.getRetrievalEvent();
                }
                while (  event.getResult() != RetrievalEvent.SUCCESS 
                      && event.getResult() != RetrievalEvent.TIME_OUT  );
                
                ar_filter = event.getSITables();
            }
            /*DBG*/ Debug.msg( Debug.SI, Debug.LOW, "SIDatabase.retrieveSIServices filtered "+ar_filter.length+" SDT" );
            // obtain all cached SDT
            SDT[] ar_sdt = 
              (SDT[])SITableCache.getSITables( new TableIdentifier( TableIdentifier.SDT, originalNetworkId, transportStreamId ) );
//            SDTCache sdtCache = (SDTCache)SDTCache.getInstance();
//            SDT[]    ar_sdt   = sdtCache.getSDTs( originalNetworkId, transportStreamId );
            /*DBG*/ Debug.msg( Debug.SI, Debug.LOW, "SIDatabase.retrieveSIServices "+ar_filter.length+" SDT in cache" );

            // add filtered SDT's fitting the profile
            for ( i=0; i< ar_filter.length; i++ )
            {
                SDT sdt = (SDT)ar_filter[i];
                if   (  ( originalNetworkId == sdt.getOriginalNetworkId() || originalNetworkId==-1 )
                     && ( transportStreamId == sdt.getTransportStreamId() || transportStreamId==-1 ) 
                     )
                {
                    vecSDT.addElement( sdt );
                }
            }
            
            // add all cached but not previously added SDT's
            boolean   found;
            for ( i=0; i< ar_sdt.length; i++ )
            {
                SDT sdt = (SDT)ar_sdt[i];
                j = 0;
                found = false;
                while ( !found && j<ar_filter.length )
                {   // found == true when sdt already proccessed
                    SDT sdt_filter = (SDT)ar_filter[j];
                    found   = (  ( sdt_filter.getOriginalNetworkId() == sdt.getOriginalNetworkId() )
                              && ( sdt_filter.getTransportStreamId() == sdt.getTransportStreamId() ) 
                              );
                    j++;
                }
                
                if   (  ( originalNetworkId == sdt.getOriginalNetworkId() || originalNetworkId==-1 )
                     && ( transportStreamId == sdt.getTransportStreamId() || transportStreamId==-1 ) 
                     && !found
                     )
                {
                    vecSDT.addElement( sdt );
                }
            }
            
            // vecSDT holds all required SDT's, create Service vector
               
            // All SDT filtered, check for services
            Enumeration e = vecSDT.elements();
            SDT sdt;
            while ( e.hasMoreElements() )
            {
                sdt = (SDT)e.nextElement();
                Serv[] arServ = sdt.getServices();
                for ( j=0; j< arServ.length; j++ )
                {
                    if ( arServ[j].getServiceId() == serviceId || serviceId == -1 )
                    {
                        vec.addElement( new SIService( this, sdt, arServ[j] ) );
                    }
                }
            }
        }
        return new SIIteratorImpl( vec );
    }
    
    /**
     * Retrieve information associated with time from the Time and Date Table (TDT) from the actual transport 
     * stream.
     * @return    A SITime object representing the TDT table in the actual transport stream.
     * @exception TimeOutException         The required information could not be filtered within the time out period.
     * @exception NotImplementedException  This feature is not implemented on this platform.
     * @exception LackOfResourcesException The resources required to obtain the information are not available at this time.
     * @see SITime
    */
    public SITime retrieveSITimeFromTDT(
    ) throws
    	RetrieveFailedException
    {
        TDT  tdt=null;
        RetrievalEvent event;
        do
        {
            RetrievalMediator rm;
            rm = RetrievalMediator.retrieveTDT( null );
            event = rm.getRetrievalEvent();
            if ( event.getResult() == RetrievalEvent.SUCCESS )
    	    {
                tdt = (TDT)event.getSITable();
            }
        }
        while (  event.getResult() != RetrievalEvent.SUCCESS 
              && event.getResult() != RetrievalEvent.TIME_OUT  );
       
        if ( event.getResult() == RetrievalEvent.TIME_OUT )
        {
            throw new TimeOutException();
        }
        return new SITime( this, tdt );
    }
    
    /**
     * Retrieve information associated with time from the Time Offset Table (TOT) from the actual transport stream.
     * The time information will be accompanied with an offset information
     *
     * @param someDescriptorTags A list of hints for descriptors (identified by their tags) the application is interested in. If the array contains -1 as its one and only element, the application is interested in all descriptors. If someDescriptorTags is null, the application is not interested in descriptors. All non applicable tag values are ignored.
     * @return                   A SITime object representing the TOT table in the actual transport stream.
     * @exception TimeOutException         The required information could not be filtered within the time out period.
     * @exception NotImplementedException  This feature is not implemented on this platform.
     * @exception LackOfResourcesException The resources required to obtain the information are not available at this time.
     * @see SITime
    */
    public SITime retrieveSITimeFromTOT(
    	short[] someDescriptorTags
    ) throws
    	RetrieveFailedException
    {
        TOT  tot=null;
        RetrievalEvent event;
        do
        {
            RetrievalMediator rm;
            rm = RetrievalMediator.retrieveTOT( null );
            event = rm.getRetrievalEvent();
            if ( event.getResult() == RetrievalEvent.SUCCESS )
    	    {
                tot = (TOT)event.getSITable();
            }
        }
        while (  event.getResult() != RetrievalEvent.SUCCESS 
              && event.getResult() != RetrievalEvent.TIME_OUT  );
       
        if ( event.getResult() == RetrievalEvent.TIME_OUT )
        {
            throw new TimeOutException();
        }
        return new SITime( this, tot );
    }
    
    /**
     * Retrieve the SITransportStreamDescription object representing the information of the TSDT
     * table in the actual transport stream of this SIDatabase object.
     * @param someDescriptorTags A list of hints for descriptors (identified by their tags) the application is interested in. If the array contains -1 as its one and only element, the application is interested in all descriptors. If someDescriptorTags is null, the application is not interested in descriptors. All non applicable tag values are ignored.
     * @return                   The SITransportStreamDescription of the actual transport stream.
     * @exception TimeOutException         The required information could not be filtered within the time out period.
     * @exception NotImplementedException  This feature is not implemented on this platform.
     * @exception LackOfResourcesException The resources required to obtain the information are not available at this time.
     * @see SITransportStreamDescription
     * @see DescriptorTag
    */
    public SITransportStreamDescription retrieveSITransportStreamDescription(
    	short[] someDescriptorTags
    ) throws
    	RetrieveFailedException
    {
        // retrieve with retrieve code
        throw new NotImplementedException();
    }

    /////////////////// IMPLEMENTATION ////////////////////////
    private static SIDatabase[] _arSid = null;

    /****************
    */    
    private SIDatabase()
    {
    }
    
    /****************
    */
    private static DvbTransportStream getCurrentTs()
    {
        return TransportStreamSelection.getInstance().getCurrentTs();
    }




}
