/*
* Copyright(c) Philips Consumer Electronics B.V. 1998
* All rights reserved. Reproduction in whole or in part is
* prohibited without the written consent of the copyright holder.
*
*  Project             : Bali
*  File name   $Logfile: /func_test_platf2/org/davic/net/dvb/si/SIEvent.java $
*       Last save $Date: 2003/12/24 00:53:10 $
*             $Revision: 1.1.1.1 $
*          Last $Author: mikeseven $
*  Creation Date       : 05/11/98
*  First Author        : S.A.F.A. van den Heuvel
*                        NatLab - Eindhoven
*
*
*  Change history      :
*  ---------------
*
* $Log: SIEvent.java,v $
* Revision 1.1.1.1  2003/12/24 00:53:10  mikeseven
* Initial release of the CVS repository for MPEG-4 Systems code
* after the code was corrupted due to reorganization and human 
* errors.
*
* Revision 1.1  2003/04/03 04:29:32  mikeseven
* Initial release of Player3D and MPEG-J for all MPEG-4 reference softwares players with samples.
* For installation, follow instructions in docs/Player3D_setup.doc.
*
* Many thanks to Prashanth Gujjeti and Viswanathan Swaminathan from Sun Microsystems.
*
* -- Mikael Bourges-Sevenier (Mindego)
*
* Revision 1.1  2002/08/23 16:47:27  pg138550
* Changes for MPEG-J. NOT FINAL
*
* Revision 1.1  2000/06/06 18:43:19  alavilli
* After SISF integration into the CORE4 code
*
* Revision 1.1.2.1  1999/07/09 14:17:24  depetris
* Aggiornati i file per lo sviluppo e il rilascio a MPEG-J.
* Integra la versione 2.5 del core.
*
* Revision 1.4  1999-04-28 10:15:08+02  vdheuvel
* Adapted to new constructor of XxxMonitorHandler
*
* Revision 1.3  1999-04-23 19:13:44+02  vdheuvel
* Adapted new interface of bali.tm.Date
*
* Revision 1.2  1999-04-15 10:01:00+02  vdheuvel
* Files changed to hook the system up to the TM demultipler and the
* TM hardware. Changes include, swapping the java.util.Date class
* for the bali.tm.Date class (to prevent using 64 bit floats). Changing
* the HW sect filt used by bali.hw.SectFilt.
*
* Revision 1.1  1999-03-18 14:05:08+01  vdheuvel
* start of archive, 29 march 1999
* DAVIC API implementation
*
 * 
 * 2     9-11-98 17:05 Vdheuvel
 * Updated retrieveUpdate and other class local retrieve methods.
 * 
 * 1     9-11-98 14:01 Vdheuvel
 * Creation
 * 
 * 1     9-11-98 13:56 Vdheuvel
 * Creation
*/

package org.davic.net.dvb.si;
import bali.cache.SITableCache;
import  bali.table.SITable;
import  bali.table.TableIdentifier;
import bali.table.EITsch;
import bali.table.EITpf;
import bali.table.EIT;
import bali.table.EitEvent;
import bali.retrieve.RetrievalMediator;
import bali.retrieve.RetrievalEvent;
import bali.component.TransportStreamTime;
import bali.support.Debug;
/*DATE_NORMAL*/ import java.util.TimeZone;
/*DATE_NORMAL*/ import java.util.Calendar;
/*DATE_NORMAL*/ import  java.util.Date;
//DATE_TM*/     import  bali.tm.Date;

/**
 * This class represents a particular event within a service. 
 * <p>
 * An object of this class represents either an event obtained from the EIT-pf or from the EIT-s. 
 * An SIEvent object filtered from the EIT-s will never be reused as the SIEvent object filtered from 
 * the EIT-pf and vice versa.
 * Each SIEvent object is uniquely defined by the combination of the identifiers
 * original_network_id, transport_stream_id, service_id, event_id.
 * Objects of this type are trackable.
 * @see SIService
 * @since DAVIC1.3
*/

public class SIEvent
	implements SIInformation
{
    
    /** 
     * This method implements SIInformation.retrieveDescriptors (first prototype).
     * @see SIInformation
     * @see SIInformation#retrieveDescriptors
    */
    public SIIterator retrieveDescriptors() 
        throws RetrieveFailedException
    {
        synchronized ( _eitLock )
        {
        	return ( new DescriptorIterator( _ev.getDescriptorList() ) );
        }
    }
    
    /** 
     * This method implements SIInformation.retrieveDescriptors (second prototype).
     * @see SIInformation
     * @see SIInformation#retrieveDescriptors
    */
    public SIIterator retrieveDescriptors( short[] someDescriptorTags )
        throws RetrieveFailedException
    {
        synchronized ( _eitLock )
        {
    		return ( new DescriptorIterator( _ev.getDescriptorList(), someDescriptorTags ) );
        }
    }
    
    /** 
     * This method implements SIInformation.retrieveDescriptorTags.
     * @see SIInformation
     * @see SIInformation#retrieveDescriptorTags
    */
    public short[] retrieveDescriptorTags( )
        throws RetrieveFailedException
    {
        synchronized ( _eitLock )
        {
            return(  _ev.getDescriptorList().getDescriptorTags() );
        }
    }
    
    /**
     * This method implements SIInformation.getSIDatabase.
     * @see SIInformation
     * @see SIInformation#getSIDatabase
    */
    public SIDatabase getSIDatabase( )
    {
        synchronized ( _eitLock )
        {
    		return ( _sid );
        }
    }
    
    /**
     * This method implements SIInformation.getUpdateTime.
     * @see SIInformation
     * @see SIInformation#getUpdateTime
    */
    public Date getUpdateTime()
    {
        synchronized ( _eitLock )
        {
    		return ( _eit.getSITableInfo().getUpdateTime() );
    	}
    }
    
    /**
     * This method implements SIInformation.fromActual.
     * @see SIInformation
     * @see SIInformation#fromActual
    */
    public boolean fromActual( )
    {
        synchronized ( _eitLock )
        {
    		return ( _eit.getSITableInfo().fromActual() );
    	}
    }
    
    /**
     * This method implements SIInformation.getDataSource.
     * @see SIInformation
     * @see SIInformation#getDataSource
    */
    public org.davic.mpeg.TransportStream getDataSource( )
    {
        synchronized ( _eitLock )
        {
    		return ( _eit.getSITableInfo().getSourceTs() );
    	}
    }
    
    /**
     * Add a listener.
     * @param aListener The object to be notified of events concerning this object.
     * @see SIEventListener
     * @see SIEventObject
    */
    public void addListener(
    	SIEventListener aListener
    )
    {
        _monHand.addListener( aListener );
    }
    
    /**
     * Remove a listener
     * @param aListener The object no longer interested in events concerning this object.
     * @see SIEventListener
     * @see SIEventObject
    */
    public void removeListener(
    	SIEventListener aListener
    )
    {
        _monHand.removeListener( aListener );
    }
    
    /**
     * Update the information contained in the object from the actual transport stream of this SIDatabase.
     * @exception InformationSourceNoMoreAvailableException This event is no longer present in the source EIT table (present/following or schedule).
     * @exception TimeOutException         The required information could not be filtered within the time out period.
     * @exception LackOfResourcesException The resources required to obtain the information are not available at this time.
     * @exception NotImplementedException  This feature is not implemented on this platform.
    */
    public void retrieveUpdate(
    ) throws
    	RetrieveFailedException
    {
        EIT      old_eit = null;
        EitEvent old_ev  = null;
        EIT      new_eit = null;
        EitEvent new_ev  = null;
        
        synchronized( _eitLock )
        {
            old_eit = _eit;
            old_ev  = _ev;
        }
        
        synchronized( _retrieveUpdateLock )
        {
            if ( _isPf )
            {   // update from EIT present following
                RetrievalMediator rm;
                RetrievalEvent event;
                do
                {
                    rm = RetrievalMediator.retrieveEITpf(null, old_eit.getOriginalNetworkId()
                                                       , old_eit.getTransportStreamId() 
                                                       , old_eit.getServiceId()
                                                       );
                    event = rm.getRetrievalEvent();
                    if ( event.getResult() == RetrievalEvent.SUCCESS )
            	    {
                        new_eit = (EITpf)event.getSITable();
                    }
                }
                while (  event.getResult() != RetrievalEvent.SUCCESS 
                      && event.getResult() != RetrievalEvent.TIME_OUT  );

                if ( new_eit == null )
                {   // no new eit filtered
                    new_eit = 
                      (EITpf)SITableCache.
                        getSITable( new TableIdentifier( TableIdentifier.EITpf
                                                        , old_eit.getOriginalNetworkId()
                                                        , old_eit.getTransportStreamId() 
                                                        , old_eit.getServiceId()
                                                        )
                                   );
//                    EITpfCache nc = (EITpfCache)EITpfCache.getInstance();
//                    new_eit = nc.getEITpf( old_eit.getOriginalNetworkId()
//                                         , old_eit.getTransportStreamId() 
//                                         , old_eit.getServiceId()
//                                         );
                }
            }
            else
            {   // update from EIT schedule
                Date midnigth = getMidnigth();
                int period = EITsch.numberOfPeriodsPastMidnight( midnigth, old_ev.getStartTime() );

                int deltaTableId = (int)( period / 32 ); // table id modifier start of time period
                // modify start and endPeriod to table_id
                period -= deltaTableId*32;
                RetrievalMediator rm ;
                RetrievalEvent event;
                do
                {
                    rm = RetrievalMediator.retrieveEITsch
                      ( null
                      , getOriginalNetworkId()
                      , getTransportStreamId()
                      , getServiceId()
                      , deltaTableId, period, period
                      );
                    event = rm.getRetrievalEvent();
                    if ( event.getResult() == RetrievalEvent.SUCCESS )
            	    {
                        new_eit = (EITpf)event.getSITable();
                    }
                }
                while (  event.getResult() != RetrievalEvent.SUCCESS 
                      && event.getResult() != RetrievalEvent.TIME_OUT  );
            }
        }
        
        // retrieve update complete, process results
        // retrieveUpdate finished, process results
        if ( new_eit != null )
        {   // locate this service
            new_ev = new_eit.getEitEvent( old_ev.getEventId() );
            if ( new_ev != null )
            {   // reset _eit of this object
                synchronized( _eitLock )
                {   _eit = new_eit;
                    _ev  = new_ev;
                }
            }
            else
            {   // EIT is there, service not present; service killed
                throw new InformationSourceNoMoreAvailableException();
            }
        }
        else
        {   throw new TimeOutException();
        }
    }
    
    /**
     * Get the URL of this event.
     * @return The URL describing this event.
    */
    public java.lang.String getURL(
    )
    {
		String str;
		str = "dvb://"
		    + Integer.toHexString( getOriginalNetworkId() )
		    + "."
		    + Integer.toHexString( getTransportStreamId() )
		    + "."
		    + Integer.toHexString( getServiceId() )
		    + "."
		    + Integer.toHexString( getEventId() )
		    ;
		return ( str );
    }
    
    /**
     * Get the original network identification identifier.
     * @return The original network identification.
    */
    public int getOriginalNetworkId()
    {
        synchronized( _eitLock )
        {   return _eit.getOriginalNetworkId();
        }
    }
    
    /**
     * Get the transport stream identification identifier.
     * @return The transport stream identification.
    */
    public int getTransportStreamId()
    {
        synchronized( _eitLock )
        {   return _eit.getTransportStreamId();
        }
    }
    
    /**
     * Get the service identification identifier.
     * @return The service identification.
    */
    public int getServiceId()
    {
        synchronized( _eitLock )
        {   return _eit.getServiceId();
        }
    }
    
    /**
     * Get the event identification.
     * @return The event identification.
    */
    public int getEventId( )
    {
        synchronized( _eitLock )
        {   return _ev.getEventId();
        }
    }
    
    /**
     * Get the start time of this event in UTC time.
     * @return The start time of this event.
    */
    public Date getStartTime(  )
    {
        synchronized( _eitLock )
        {   return _ev.getStartTime();
        }
    }
    
    /**
     * Get the duration of this event.
     * @return The duration in milliseconds.
    */
    public long getDuration()
    {
        synchronized( _eitLock )
        {   return _ev.getDuration();
        }
    }
    
    /**
     * Get the running status of this event.
     * @return The running status (the possible values are defined in the SIRunningStatus interface).
     * @see SIRunningStatus
    */
    public byte getRunningStatus(   )
    {
        synchronized( _eitLock )
        {   return (byte)_ev.getRunningStatus();
        }
    }
    
    /**
     * Get the free_CA_mode value for this event, false indicates none of the component streams of this event
     * are scrambled.
     * @return The free_CA_mode value.
    */
    public boolean getFreeCAMode( )
    {
        synchronized( _eitLock )
        {   return _ev.getFreeCaMode();
        }
    }
    
    /**
     * This method returns the name of the event represented by this SIEvent object. 
     * The name is extracted from a short_event_descriptor. When this information is not available "" is returned.
     * All control characters as defined in ETR 211 are ignored. For each character the DVB-SI 8 bit character
     * code is mapped to the appropriate Unicode representation.
     * @return The event name of this event.
    */
    public java.lang.String getEventName( )
    {
        synchronized( _eitLock )
        {   return _ev.getEventName();
        }
    }
    
    /**
     * This method returns the short event name (ETR 211) of the event represented by this SIEvent object 
     * without emphasis marks. The name is extracted from a short_event_descriptor. When this 
     * information is not available "" is returned. For each character the DVB-SI 8 bit character code is mapped 
     * to the appropriate Unicode representation.
     * @return The short event name of this event.
    */
    public String getShortEventName()
    {
        synchronized( _eitLock )
        {   return _ev.getShortEventName();
        }
    }
     
    /**
     * This method returns the description of the event represented by this SIEvent object. 
     * The description is extracted from a short_event_descriptor. When this information is not available,
     * "" is returned. For each character the DVB-SI 8 bit character code is mapped to the appropriate 
     * Unicode representation
     * @return The short description of this event.
    */
    public java.lang.String getShortDescription(
    )
    {
        synchronized( _eitLock )
        {   return _ev.getEventDescription();
        }
    }
    
    /**
     * This method returns the level 1 content nibbles of this event. This information is extracted from 
     * the content_descriptor. If this descriptor is not present an empty array is returned (array with length 0).
     * The return value is an array, each array element describes one content nibble. In each nibble the data
     * occupies the four least significant bits of the returned bytes.
     * @return All level 1 content nibbles related to the event.
    */
    public byte[] getLevel1ContentNibbles() 
    {
        synchronized( _eitLock )
        {   return _ev.getLevel1ContentNibbles();
        }
    }
    
    /**
     * This method returns the content nibbles related to the event. This information is extracted from the 
     * content_descriptor. If this descriptor is not present an empty array is returned (array with length 0). 
     * The return value is an array, each array element describes one content nibble. In each nibble the 
     * level 1 content nibbles occupy the four least significant bits of the returned bytes, 
     * level 2 content nibbles the four most significan bits.
     * @return The content nibbles related to the event; level 1 content nibbles occupy the first four bits of the returned bytes, level 2 content nibbles the last four bits.
    */
    public byte[] getContentNibbles( ) 
    {
        synchronized( _eitLock )
        {   return _ev.getContentNibbles();
        }
    }
    
    /**
     * This method retrieves the SIService object representing the service the event, 
     * represented by this SIEvent, is part of.
     * @param someDescriptorTags A list of hints for descriptors (identified by their tags) the application is interested in. If the array contains -1 as its one and only element, the application is interested in all descriptors. If someDescriptorTags is null, the application is not interested in descriptors. All non applicable tag values are ignored
     * @return                   The SIService object representing the service containing the event.
     * @exception InformationSourceNoMoreAvailableException The SDT of the transport stream of this event no longer describes the required service.
     * @exception TimeOutException         The required information could not be filtered within the time out period.
     * @exception NotImplementedException  This feature is not implemented on this platform.
     * @exception LackOfResourcesException The resources required to obtain the information are not available at this time.
     * @see SIService
     * @see DescriptorTag
    */
    public SIService retrieveSIService( short[] someDescriptorTags ) 
        throws RetrieveFailedException
    {
        SIService res = null;
        SIIterator it = 
            _sid.retrieveSIServices( _eit.getOriginalNetworkId()
                                   , _eit.getTransportStreamId()
                                   , _eit.getServiceId()
                                   , someDescriptorTags );
        if ( it.hasMoreElements() )
        {
            res = (SIService)it.nextElement();
        }
        else
        {   // service not present
            throw new InformationSourceNoMoreAvailableException();
        }
        return res;
    }

    ////////////// IMPLEMENTATION //////////////////

    private Object     _retrieveUpdateLock = new Object();
    private Object     _eitLock = new Object();
    private EIT        _eit;
    private EitEvent   _ev;    
    private	SIDatabase _sid;
    private boolean    _isPf;
    private EitEventMonitorHandler _monHand;
    
    /**************
    */
    SIEvent( SIDatabase sid, EIT eit, EitEvent ev, boolean isPf )
    {
        _sid = sid;
        _eit = eit;
        _ev  = ev;
        _isPf= isPf;
        if ( isPf )
        {
            _monHand = new EitEventMonitorHandler( this, eit, eit.getTableIdentifier(), ev.getEventId() );
        }
        else
        {
            int[] arId = eit.getTableIdentifier().getIdentifiers();
            Date midnigth = getMidnigth();
            int period = EITsch.numberOfPeriodsPastMidnight( midnigth, ev.getStartTime() );
            _monHand = new EitEventMonitorHandler( this, eit, new TableIdentifier( TableIdentifier.EITsch_ST, arId[0], arId[1], arId[2], period ), ev.getEventId() );
        }
    }

    /*******************
    Calculates the last UTC midnight in the time of the Transport Stream.
    */
    private Date getMidnigth()
    {
        // calculate last midnight time
        Date d = TransportStreamTime.getInstance().getTsTime();

/*DATE_NORMAL*/ Calendar cal = Calendar.getInstance( TimeZone.getTimeZone("UTC") );
/*DATE_NORMAL*/ cal.clear();
/*DATE_NORMAL*/ cal.setTime( TransportStreamTime.getInstance().getTsTime() );
/*DATE_NORMAL*/ // reset clock to last midnight
/*DATE_NORMAL*/ cal.set( Calendar.HOUR_OF_DAY, 0 );
/*DATE_NORMAL*/ cal.set( Calendar.MINUTE, 0 );
/*DATE_NORMAL*/ cal.set( Calendar.SECOND, 0 );
/*DATE_NORMAL*/ Date midnigth =  cal.getTime();

//DATE_TM*/      Date midnigth =  new Date( d.getYear(),d.getMonth(),d.getDay(),0/*houres*/,0/*minutes*/,0/*seconds*/);

        /*DBG*/Debug.msg( Debug.SI, Debug.IMPL, "EIT schedule midnigth "+midnigth );

        return midnigth;
    }
}
