/*
* Copyright(c) Philips Consumer Electronics B.V. 1998                         
* All rights reserved. Reproduction in whole or in part is                  
* prohibited without the written consent of the copyright holder.            
*                                                                            
*  Project             : Bali
*  File name   $Logfile: $
*       Last save $Date: 2003/12/24 00:53:10 $
*             $Revision: 1.1.1.1 $
*          Last $Author: mikeseven $
*  Creation Date       : 28/10/98
*  First Author        : S.A.F.A. van den Heuvel
*                        NatLab - Eindhoven
* 
*                                                                          
*  Change history      :
*  ---------------
*
* $Log: SIInformation.java,v $
* Revision 1.1.1.1  2003/12/24 00:53:10  mikeseven
* Initial release of the CVS repository for MPEG-4 Systems code
* after the code was corrupted due to reorganization and human 
* errors.
*
* Revision 1.1  2003/04/03 04:29:32  mikeseven
* Initial release of Player3D and MPEG-J for all MPEG-4 reference softwares players with samples.
* For installation, follow instructions in docs/Player3D_setup.doc.
*
* Many thanks to Prashanth Gujjeti and Viswanathan Swaminathan from Sun Microsystems.
*
* -- Mikael Bourges-Sevenier (Mindego)
*
* Revision 1.1  2002/08/23 16:47:27  pg138550
* Changes for MPEG-J. NOT FINAL
*
* Revision 1.1  2000/06/06 18:43:20  alavilli
* After SISF integration into the CORE4 code
*
* Revision 1.1.2.1  1999/07/09 14:17:25  depetris
* Aggiornati i file per lo sviluppo e il rilascio a MPEG-J.
* Integra la versione 2.5 del core.
*
* Revision 1.2  1999-04-15 09:53:18+02  vdheuvel
* Files changed to hook the system up to the TM demultipler and the
* TM hardware. Changes include, swapping the java.util.Date class
* for the bali.tm.Date class (to prevent using 64 bit floats). Changing
* the HW sect filt used by bali.hw.SectFilt.
*
*
*/

package org.davic.net.dvb.si;
/*DATE_NORMAL*/ import  java.util.Date;
//DATE_TM*/     import  bali.tm.Date;

/**
 * This interface groups the common features of SIBouquet, SINetwork, SITransportStream, SIService, 
 * SIEvent, SITime and PMTElementaryStream.
 * <p>
 * Each SIInformation instance represents a sub-table (part). Any method accessing descriptors will 
 * retrieve descriptors from the same sub-table version as the SIInformation instance. 
 * When this data is no longer available, an InformationSourceNoMoreAvailableException is thrown.
 * @see SIBouquet
 * @see SINetwork
 * @see SITransportStream
 * @see SIService
 * @see SIEvent
 * @see SITime
 * @see PMTElementaryStream
 * @since DAVIC1.3
*/

public interface SIInformation{

/**
 * This method retrieves all descriptors in the order the descriptors are broadcast.
 * Depending on the caching features and policy of the underlying implementation, this request can 
 * initiate an actual access to the stream.
 * @return    The Descriptor objects from the sub-table (part) of this SIInformation instance in the order they are broadcast.
 * @exception InformationSourceNoMoreAvailableException The version number of the sub-table has changed or data represented by this SIInformation instance is no longer present in the sub-table.
 * @exception TimeOutException         The required information could not be filtered within the time out period.
 * @exception LackOfResourcesException The resources required to obtain the information are not available at this time.
 * @exception NotImplementedException  This feature is not implemented on this platform.
 * @see Descriptor
 * @see SIIterator
*/
public SIIterator retrieveDescriptors(
) throws
	RetrieveFailedException;

/**
 * Retrieve a set of descriptors. This method retrieves all or a set of descriptors in the order the 
 * descriptors are broadcast. Depending on the caching features and policy of the underlying 
 * implementation, this request can initiate an actual access to the stream.
 * <p>
 * If the list of tags is a subset of the one hinted to the underlying implementation (in the request 
 * which created the object on which the method is called), this is likely to increase the efficiency 
 * of the (optional) caching mechanism
 * 
 * @param someDescriptorTags A list of tags for descriptors (identified by their tags) the application is interested in. If the array contains -1 as its one and only element, the application is interested in all descriptors. If someDescriptorTags is null, the application is not interested in descriptors. All non applicable tag values are ignored.
 * @return                   A set of Descriptor objects from the sub-table (part) of this SIInformation instance holding tags as indicated in someDescriptorTags
 * @exception InformationSourceNoMoreAvailableException The version number of the sub-table has changed or data represented by this SIInformation instance is no longer present in the sub-table.
 * @exception TimeOutException         The required information could not be filtered within the time out period.
 * @exception LackOfResourcesException The resources required to obtain the information are not available at this time.
 * @exception NotImplementedException  This feature is not implemented on this platform.
 * @see Descriptor
 * @see SIIterator
 * @see DescriptorTag
*/
public SIIterator retrieveDescriptors(
	short[] someDescriptorTags
) throws
	RetrieveFailedException;

/**
 * Retrieve the tags of all descriptors that are actually broadcast for that object.
 * The tags are returned in the same order as the descriptors are broadcast.
 * This method returns also the tags of descriptors that were not hinted at and that are not 
 * necessarily present in the cache. 
 * @return    The tags of the descriptors actually broadcast for the object (identified by their tags).
 * @exception InformationSourceNoMoreAvailableException The version number of the sub-table has changed or data represented by this SIInformation instance is no longer present in sub-table.
 * @exception TimeOutException         The required information could not be filtered within the time out period.
 * @exception LackOfResourcesException The resources required to obtain the information are not available at this time.
 * @exception NotImplementedException  This feature is not implemented on this platform.
 * @see DescriptorTag
*/
public short[] retrieveDescriptorTags(
) throws
	RetrieveFailedException;

/**
 * Return the root of the hierarchy the object that implements this interface belongs to.
 * @return The root of the hierarchy.
*/
public SIDatabase getSIDatabase(
);

/**
 * Return the time when the information contained in the object that implements this interface was last 
 * updated.
 *
 * @return The date of the last update.
*/
public Date getUpdateTime(
);

/**
 * Return true when the information contained in the object that implements this interface was filtered 
 * from an 'actual' table or from a table with no 'actual/other' distinction.
 * @return <code>true</code> if the information comes from an 'actual' table or from a table with no 'actual/other' 
           distinction
*/
public boolean fromActual(
);

/**
 * Return the org.davic.mpeg.TransportStream object the information contained in the object that implements that interface was filtered from
 *
 * @return The <code>org.davic.mpeg.TransportStream</code> object the information was filtered from.
 * @see    org.davic.mpeg.TransportStream
*/
public org.davic.mpeg.TransportStream getDataSource(
);
}
