/*
* Copyright(c) Philips Consumer Electronics B.V. 1998
* All rights reserved. Reproduction in whole or in part is
* prohibited without the written consent of the copyright holder.
*
*  Project             : Bali
*  File name   $Logfile: /func_test_platf2/org/davic/net/dvb/si/SINetwork.java $
*       Last save $Date: 2003/12/24 00:53:10 $
*             $Revision: 1.1.1.1 $
*          Last $Author: mikeseven $
*  Creation Date       : 05/11/98
*  First Author        : S.A.F.A. van den Heuvel
*                        NatLab - Eindhoven
*
*
*  Change history      :
*  ---------------
*
* $Log: SINetwork.java,v $
* Revision 1.1.1.1  2003/12/24 00:53:10  mikeseven
* Initial release of the CVS repository for MPEG-4 Systems code
* after the code was corrupted due to reorganization and human 
* errors.
*
* Revision 1.1  2003/04/03 04:29:32  mikeseven
* Initial release of Player3D and MPEG-J for all MPEG-4 reference softwares players with samples.
* For installation, follow instructions in docs/Player3D_setup.doc.
*
* Many thanks to Prashanth Gujjeti and Viswanathan Swaminathan from Sun Microsystems.
*
* -- Mikael Bourges-Sevenier (Mindego)
*
* Revision 1.1  2002/08/23 16:47:27  pg138550
* Changes for MPEG-J. NOT FINAL
*
* Revision 1.1  2000/06/06 18:43:20  alavilli
* After SISF integration into the CORE4 code
*
* Revision 1.1.2.1  1999/07/09 14:17:26  depetris
* Aggiornati i file per lo sviluppo e il rilascio a MPEG-J.
* Integra la versione 2.5 del core.
*
* Revision 1.2  1999-04-15 09:53:18+02  vdheuvel
* Files changed to hook the system up to the TM demultipler and the
* TM hardware. Changes include, swapping the java.util.Date class
* for the bali.tm.Date class (to prevent using 64 bit floats). Changing
* the HW sect filt used by bali.hw.SectFilt.
*
* Revision 1.1  1999-03-18 12:31:58+01  vdheuvel
* start of archive, 29 march 1999
* DAVIC API implementation
*
 * 
 * 2     9-11-98 17:05 Vdheuvel
 * Updated retrieveUpdate and other class local retrieve methods.
 * 
 * 1     9-11-98 14:01 Vdheuvel
 * Creation
 * 
 * 1     9-11-98 13:56 Vdheuvel
 * Creation
*/
package org.davic.net.dvb.si;
import  bali.table.NIT;
import  bali.table.TS;
import  bali.table.SITableInfo;
import  bali.table.SITable;
import  bali.cache.SITableCache;
import  bali.table.TableIdentifier;
import  bali.retrieve.RetrievalMediator;
import  bali.retrieve.RetrievalEvent;
import  java.util.Vector;
import  java.util.Enumeration;
/*DATE_NORMAL*/ import  java.util.Date;
//DATE_TM*/     import  bali.tm.Date;

/**
 * This class (together with the SITransportStream class) represents a sub-table of the Network Information
 * Table (NIT) describing a particular network. Objects of this class are trackable.
 * <p>
 * Each SINetwork object is uniquely defined by the identifier network_id.
 * @see SITransportStream
 * @since DAVIC1.3
*/

public class SINetwork
	implements
		SIInformation
{
    /**
     * This method implements SIInformation.retrieveDescriptors (first prototype).
     * @see SIInformation
     * @see SIInformation#retrieveDescriptors
    */
    public SIIterator retrieveDescriptors(
    ) throws
    	RetrieveFailedException
    {
        synchronized ( _nitLock )
        {
        	return ( new DescriptorIterator( _nit.getDescriptorList() ) );
        }
    }

    /**
     * This method implements SIInformation.retrieveDescriptors (second prototype).
     * @see SIInformation
     * @see SIInformation#retrieveDescriptors
    */
    public SIIterator retrieveDescriptors( short[] someDescriptorTags )
        throws RetrieveFailedException
    {
        synchronized ( _nitLock )
        {
    		return ( new DescriptorIterator( _nit.getDescriptorList(), someDescriptorTags ) );
        }
    }

    /**
     * This method implements SIInformation.retrieveDescriptorTags.
     * @see SIInformation
     * @see SIInformation#retrieveDescriptorTags
    */
    public short[] retrieveDescriptorTags()
        throws RetrieveFailedException
    {
        synchronized ( _nitLock )
        {
            return(  _nit.getDescriptorList().getDescriptorTags() );
        }
    }

    /**
     * This method implements SIInformation.getSIDatabase.
     * @see SIInformation
     * @see SIInformation#getSIDatabase
    */
    public SIDatabase getSIDatabase()
    {
        synchronized ( _nitLock )
        {
    		return ( _sid );
        }
    }

    /**
     * This method implements SIInformation.getUpdateTime.
     * @see SIInformation
     * @see SIInformation#getUpdateTime
    */
    public Date getUpdateTime( )
    {
        synchronized ( _nitLock )
        {
    		return ( _nit.getSITableInfo().getUpdateTime() );
    	}
    }

    /**
     * This method implements SIInformation.fromActual.
     * @see SIInformation
     * @see SIInformation#fromActual
    */
    public boolean fromActual( )
    {
        synchronized ( _nitLock )
        {
    		return ( _nit.getSITableInfo().fromActual() );
    	}
    }

    /**
     * This method implements SIInformation.getDataSource.
     * @see SIInformation
     * @see SIInformation#getDataSource
    */
    public org.davic.mpeg.TransportStream getDataSource( )
    {
        synchronized ( _nitLock )
        {
    		return ( _nit.getSITableInfo().getSourceTs() );
    	}
    }

    /**
     * Add a listener.
     * @param aListener The object to be notified of events concerning this object.
     * @see SIEventListener
     * @see SIEventObject
    */
    public void addListener( SIEventListener aListener )
    {   
        _monHand.addListener( aListener );
    }

    /**
     * Remove a listener
     * @param aListener The object no longer interested in events concerning this object.
     * @see SIEventListener
     * @see SIEventObject
    */
    public void removeListener( SIEventListener aListener )
    {
        _monHand.removeListener( aListener );
    }

    /**
     * Update the information contained in the object from the actual transport stream of this SIDatabase.
     * @exception TimeOutException         The required information could not be filtered within the time out period.
     * @exception NotImplementedException  This feature is not implemented on this platform.
     * @exception LackOfResourcesException The resources required to obtain the information are not available at this time.
    */
    public void retrieveUpdate(
    ) throws
    	RetrieveFailedException
    {
		// goal: obtain new version of the NIT of this transport stream network
		// 1 Start filtering the NIT.
		// 2 When filter failed, return cached information

        NIT old_nit = null;
        NIT new_nit = null;
        synchronized ( _nitLock )
        {
            old_nit = _nit;
        }

        synchronized( _retrieveUpdateLock )
        {
            RetrievalEvent event;
            boolean ready = false;
            RetrievalMediator rm;
            
            do
            {
                rm = RetrievalMediator.retrieveNIT(null, old_nit.getNetworkId() );
                event = rm.getRetrievalEvent();
                if ( event.getResult() == RetrievalEvent.SUCCESS )
        	    {
                    new_nit = (NIT)event.getSITable();
                }
            }
            while (  event.getResult() != RetrievalEvent.SUCCESS 
                  && event.getResult() != RetrievalEvent.TIME_OUT  );


            if ( new_nit == null )
            {   // no new table filtered, try cache
                new_nit = (NIT)SITableCache.
                   getSITable( new TableIdentifier( TableIdentifier.NIT, old_nit.getNetworkId() ) );
//                NITCache nc = (NITCache)NITCache.getInstance();
//                new_nit = nc.getNIT( old_nit.getNetworkId() );
            }
        }
        // retrieveUpdate finished, process results
        if ( new_nit != null )
        {   // update _nit of this object
            synchronized( _nitLock )
            {   _nit = new_nit;
            }
        }
        else
        {   throw new TimeOutException();
        }
    }

    /**
     * Get the identification of this network.
     * @return The network identification identifier.
    */
    public int getNetworkId()
    {
        synchronized ( _nitLock )
        {
    		return ( _nit.getNetworkId() );
    	}
    }

    /**
     * This method returns the name of the network represented by this SINetwork object. The name is
     * extracted from the network_name_descriptor or optionally from the multilingual_network_name_descriptor.
     * When this information is not available "" is returned. All control characters as defined in ETR 211
     * are ignored. For each character the DVB-SI 8 bit character code is mapped to the appropriate
     * Unicode representation.
     * @return The network name of this network.
    */
    public java.lang.String getNetworkName(
    )
    {
        synchronized ( _nitLock )
        {
    		return ( _nit.getNetworkName() );
    	}
    }

    /**
     * This method returns the short name (ETR 211) of the network represented by this SINetwork object
     * without emphasis marks. The name is extracted from the network_name_descriptor or optionally from
     * the multilingual_network_name_descriptor. When this information is not available "" is returned.
     * For each character the DVB-SI 8 bit character code is mapped to the appropriate Unicode representation.
     * @return The short network name of this network.
    */
    public String getShortNetworkName()
    {
        synchronized ( _nitLock )
        {
    		return ( _nit.getShortNetworkName() );
    	}
    }

    /**
     * Retrieve information associated with transport streams carried via the network.
     * @param someDescriptorTags A list of hints for descriptors (identified by their tags) the application is interested in. If the array contains -1 as its one and only element, the application is interested in all descriptors. If someDescriptorTags is null, the application is not interested in descriptors. All non applicable tag values are ignored
     * @return The SITransportStream objects representing the transport streams of this network (may be empty).
     * @exception TimeOutException         The required information could not be filtered within the time out period.
     * @exception NotImplementedException  This feature is not implemented on this platform.
     * @exception LackOfResourcesException The resources required to obtain the information are not available at this time.
     * @see SITransportStream
     * @see SIIterator
     * @see DescriptorTag
    */
    public SIIterator retrieveSITransportStreams( short[] someDescriptorTags )
        throws RetrieveFailedException
    {
		TS[] arTs;
		synchronized ( _nitLock )
		{
			arTs = _nit.getTSs();
		}

		Vector vec =  new Vector();

		for ( int i=0; i<arTs.length; i++ )
		{
			vec.addElement( new SITransportStream( _sid, arTs[i] ) );
		}

		return ( new SIIteratorImpl( vec ) );
    }

    ///////////////// IMPLEMENTATION ////////////////////////////////////
    private Object     _nitLock;
    private Object     _retrieveUpdateLock;
	private NIT        _nit;
	private SIDatabase _sid;
	private MonitorHandler _monHand;

    /*******************
    */
    	SINetwork( SIDatabase sid, NIT nit )
	{
		_nit      = nit;
		_sid      = sid;
		_nitLock  = new Object();
		_retrieveUpdateLock = new Object();
		_monHand  = new MonitorHandler( this, nit.getTableIdentifier() );
	}



}
