/*
* Copyright(c) Philips Consumer Electronics B.V. 1998
* All rights reserved. Reproduction in whole or in part is
* prohibited without the written consent of the copyright holder.
*
*  Project             : Bali
*  File name   $Logfile: /func_test_platf2/org/davic/net/dvb/si/SIService.java $
*       Last save $Date: 2003/12/24 00:53:11 $
*             $Revision: 1.1.1.1 $
*          Last $Author: mikeseven $
*  Creation Date       : 05/11/98
*  First Author        : S.A.F.A. van den Heuvel
*                        NatLab - Eindhoven
*
*
*  Change history      :
*  ---------------
*
* $Log: SIService.java,v $
* Revision 1.1.1.1  2003/12/24 00:53:11  mikeseven
* Initial release of the CVS repository for MPEG-4 Systems code
* after the code was corrupted due to reorganization and human 
* errors.
*
* Revision 1.1  2003/04/03 04:29:33  mikeseven
* Initial release of Player3D and MPEG-J for all MPEG-4 reference softwares players with samples.
* For installation, follow instructions in docs/Player3D_setup.doc.
*
* Many thanks to Prashanth Gujjeti and Viswanathan Swaminathan from Sun Microsystems.
*
* -- Mikael Bourges-Sevenier (Mindego)
*
* Revision 1.1  2002/08/23 16:47:28  pg138550
* Changes for MPEG-J. NOT FINAL
*
* Revision 1.1  2000/06/06 18:43:21  alavilli
* After SISF integration into the CORE4 code
*
* Revision 1.1.2.1  1999/07/09 14:17:27  depetris
* Aggiornati i file per lo sviluppo e il rilascio a MPEG-J.
* Integra la versione 2.5 del core.
*
* Revision 1.4  1999-04-27 15:14:54+02  vdheuvel
* Adapted to new constructor of XxxMonitorHandler
*
* Revision 1.3  1999-04-23 19:11:21+02  vdheuvel
* Adapted new interface of bali.tm.Date
*
* Revision 1.2  1999-04-15 10:03:00+02  vdheuvel
* Files changed to hook the system up to the TM demultipler and the
* TM hardware. Changes include, swapping the java.util.Date class
* for the bali.tm.Date class (to prevent using 64 bit floats). Changing
* the HW sect filt used by bali.hw.SectFilt.
*
* Revision 1.1  1999-03-18 12:43:41+01  vdheuvel
* start of archive, 29 march 1999
* DAVIC API implementation
*
 * 
 * 2     9-11-98 17:05 Vdheuvel
 * Updated retrieveUpdate and other class local retrieve methods.
 * 
 * 1     9-11-98 14:01 Vdheuvel
 * Creation
 * 
 * 1     9-11-98 13:56 Vdheuvel
 * Creation
*/
package org.davic.net.dvb.si;
import  bali.component.TransportStreamTime;
import  bali.table.SITable;
import  bali.table.EITpf;
import  bali.table.EITsch;
import  bali.table.EitEvent;
import  bali.table.SDT;
import  bali.table.Serv;
import  bali.table.PMT;
import  bali.table.ES;
import  bali.table.SITableInfo;
import  bali.table.SITable;
import  bali.table.TableIdentifier;
import  bali.cache.SITableCache;
import  bali.retrieve.RetrievalMediator;
import  bali.retrieve.RetrievalEvent;
import  bali.support.Debug;
import  java.util.Vector;
import  java.util.Enumeration;
import  java.util.TimeZone;
import  java.util.Calendar;
/*DATE_NORMAL*/ import  java.util.Date;
//DATE_TM*/     import  bali.tm.Date;

/**
 * This class represents a particular service carried by a transport stream. Objects of this type are 
 * trackable. 
 * <p>
 * Each SIService object is uniquely identified by the combination of the following identifiers: 
 *   original_network_id, transport_stream_id, service_id.
 * @since DAVIC1.3
*/

public class SIService
	implements SIInformation
{
    /** 
     * This method implements SIInformation.retrieveDescriptors (first prototype).
     * @see SIInformation
     * @see SIInformation#retrieveDescriptors
    */
    public SIIterator retrieveDescriptors(
    ) throws
    	RetrieveFailedException
    {
        synchronized ( _sdtLock )
        {
        	return ( new DescriptorIterator( _srv.getDescriptorList() ) );
        }
    }
    
    /** 
     * This method implements SIInformation.retrieveDescriptors (second prototype).
     * @see SIInformation
     * @see SIInformation#retrieveDescriptors
    */
    public SIIterator retrieveDescriptors(
    	short[] someDescriptorTags
    ) throws
    	RetrieveFailedException
    {
        synchronized ( _sdtLock )
        {
    		return ( new DescriptorIterator( _srv.getDescriptorList(), someDescriptorTags ) );
        }
    }
    
    /** 
     * This method implements SIInformation.retrieveDescriptorTags.
     * @see SIInformation
     * @see SIInformation#retrieveDescriptorTags
    */
    public short[] retrieveDescriptorTags()
        throws RetrieveFailedException
    {
        synchronized ( _sdtLock )
        {
            return(  _srv.getDescriptorList().getDescriptorTags() );
        }
    }

    
    /**
     * This method implements SIInformation.getSIDatabase.
     * @see SIInformation
     * @see SIInformation#getSIDatabase
    */
    public SIDatabase getSIDatabase()
    {
        synchronized ( _sdtLock )
        {
    		return ( _sid );
        }
    }
    
    /**
     * This method implements SIInformation.getUpdateTime.
     * @see SIInformation
     * @see SIInformation#getUpdateTime
    */
    public Date getUpdateTime( )
    {
        synchronized ( _sdtLock )
        {
    		return ( _sdt.getSITableInfo().getUpdateTime() );
    	}
    }
    
    /**
     * This method implements SIInformation.fromActual.
     * @see SIInformation
     * @see SIInformation#fromActual
    */
    public boolean fromActual( )
    {
        synchronized ( _sdtLock )
        {
    		return ( _sdt.getSITableInfo().fromActual() );
    	}
    }
    
    /**
     * This method implements SIInformation.getDataSource.
     * @see SIInformation
     * @see SIInformation#getDataSource
    */
    public org.davic.mpeg.TransportStream getDataSource( )
    {
        synchronized ( _sdtLock )
        {
    		return ( _sdt.getSITableInfo().getSourceTs() );
    	}
    }

    
    /**
     * Add a listener.
     * @param aListener The object to be notified of events concerning this object.
     * @see SIEventListener
     * @see SIEventObject
    */
    public void addListener( SIEventListener aListener )
    {
        _servMonHand.addListener( aListener );
    }
    
    /**
     * Remove a listener
     * @param aListener The object no longer interested in events concerning this object.
     * @see SIEventListener
     * @see SIEventObject
    */
    public void removeListener( SIEventListener aListener )
    {
        _servMonHand.removeListener( aListener );
    }
    
    /**
     * Update the information contained in the object from the actual transport stream of this SIDatabase.
     * @exception InformationSourceNoMoreAvailableException The service represented by this SIService object no longer exists in the SDT.
     * @exception TimeOutException         The required information could not be filtered within the time out period.
     * @exception NotImplementedException  This feature is not implemented on this platform.
     * @exception LackOfResourcesException The resources required to obtain the information are not available at this time.
    */
    public void retrieveUpdate(
    ) throws
    	RetrieveFailedException
    {
		// goal: obtain new version of the SDT of this service
		// 1 Start filtering the SDT.
		// 2 When filter failed, return cached information

        /*DBG*/Debug.msg(Debug.SI,Debug.HIGH,"SIService.retrieveUpdate" );
        SDT  old_sdt = null;
        Serv old_srv = null;
        SDT  new_sdt = null;
        Serv new_srv = null;
        synchronized ( _sdtLock )
        {
            old_sdt = _sdt;
            old_srv = _srv;
        }

        synchronized( _retrieveUpdateLock )
        {
            RetrievalEvent event;
            RetrievalMediator rm;
            
            do
            {
                rm = RetrievalMediator.retrieveSDT( null, old_sdt.getOriginalNetworkId()
                                                  , old_sdt.getTransportStreamId() );

                event = rm.getRetrievalEvent();
                if ( event.getResult() == RetrievalEvent.SUCCESS )
        	    {
                    new_sdt = (SDT)event.getSITable();
                }
            }
            while (  event.getResult() != RetrievalEvent.SUCCESS 
                  && event.getResult() != RetrievalEvent.TIME_OUT  );

            if ( new_sdt == null )
            {   // no new table filtered, update from cache
                new_sdt =
                    (SDT)SITableCache.getSITable( new TableIdentifier( TableIdentifier.SDT
                                                                     , old_sdt.getOriginalNetworkId()
                                                                     , old_sdt.getTransportStreamId() 
                                                                     )
                                                );
            }
            
        } // end  of _retrieveUpdateLock

        // retrieveUpdate finished, process results
        if ( new_sdt != null )
        {   // locate this service
            new_srv = new_sdt.getService( old_srv.getServiceId() );
            if ( new_srv != null )
            {   // reset _sdt of this object
                synchronized( _sdtLock )
                {   _sdt = new_sdt;
                    _srv = new_srv;
                }
            }
            else
            {   // SDT is there, service not present; service killed
                throw new InformationSourceNoMoreAvailableException();
            }
        }
        else
        {   throw new TimeOutException();
        }
    }

    
    /**
     * Get the URL addressing the service.
     * @return The URL of this service.
    */
    public java.lang.String getURL( )
    {
		String str;
		str = "dvb://"
		    + Integer.toHexString( getOriginalNetworkId() )
		    + "."
		    + Integer.toHexString( getTransportStreamId() )
		    + "."
		    + Integer.toHexString( getServiceId() )
		    ;
		return ( str );
    }
    
    /**
     * Get the original network identification.
     * @return The original network identification identifier.
    */
    public int getOriginalNetworkId()
    {
        synchronized( _sdtLock )
        {   return _sdt.getOriginalNetworkId();
        }
    }
    
    /**
     * Get the transport stream identification.
     * @return The transport stream identification identifier.
    */
    public int getTransportStreamId()
    {
        synchronized( _sdtLock )
        {   return _sdt.getTransportStreamId();
        }
    }
    
    /**
     * Get the service identification.
     * @return The service identification identifier.
    */
    public int getServiceId()
    {
        synchronized( _sdtLock )
        {   return _srv.getServiceId();
        }
    }
    
    /** 
     * This method returns the name of the service represented by this SIService object. The name is 
     * extracted from the service_descriptor or optionally from the multilingual_service_name_descriptor. 
     * If this descriptor is not present "" is returned. All control characters as defined in ETR 211 
     * are ignored. For each character the DVB-SI 8 bit character code is mapped to the appropriate 
     * Unicode representation.
     * @return The name of this service.
    */
    public String getServiceName()
    {
        synchronized( _sdtLock )
        {   return _srv.getServiceName();
        }
    }
    
    /** 
     * This method returns the short name (ETR 211) of the service represented by this SIService object 
     * without emphasis marks. The name is extracted from the service_descriptor or optionally 
     * from the multilingual_service_name_descriptor. When this information is not available "" is returned.
     * For each character the DVB-SI 8 bit character code is mapped to the appropriate Unicode representation.
     * @return The short name of this service.
    */
    public String getShortServiceName()
    {
        synchronized( _sdtLock )
        {   return _srv.getShortServiceName();
        }
    }
    
    /** 
     * This method returns the service provider name of the service of by this SIService object. 
     * The service provider name is extracted from the service_descriptor or optionally from the 
     * multilingual_service_name_descriptor. If this descriptor is not present "" is returned.
     * All control characters as defined in ETR 211 are ignored. For each character the DVB-SI 8 bit 
     * character code is mapped to the appropriate Unicode representation.
     * @return The service provider name of this service.
    */
    public String getServiceProviderName()
    {
        synchronized( _sdtLock )
        {   return _srv.getServiceProvider();
        }
    }
    
    /** 
     * This method returns the short name (ETR 211) of the service provider of by this SIService object 
     * without emphasis marks. The name is extracted from the service_descriptor or optionally from the 
     * multilingual_service_name_descriptor. When this information is not available "" is returned.
     * For each character the DVB-SI 8 bit character code is mapped to the appropriate Unicode representation.
     * @return The short service provider name of this service.
    */
    public String getShortServiceProviderName()
    {
        synchronized( _sdtLock )
        {   return _srv.getShortServiceProvider();
        }
    }
    
    /**
     * Retrieve the EIT_schedule_flag value, true indicates this services has scheduled event information.
     * @return The EIT_schedule_flag value.
     * @exception InformationSourceNoMoreAvailableException The information could not be accessed because the version of the service represented by this SIService object has changed or this service no longer exists in the SDT.
     * @exception TimeOutException         The required information could not be filtered within the time out period.
     * @exception NotImplementedException  This feature is not implemented on this platform.
     * @exception LackOfResourcesException The resources required to obtain the information are not available at this time.
    */
    public boolean retrieveEITScheduleFlag() 
        throws RetrieveFailedException
    {
        synchronized( _sdtLock )
        {   return _srv.getEitScheduleFlag();
        }
    }
    
    /**
     * Retrieve the EIT_present_following_flag value, true indicates this service has present and/or following 
     * event information.
     * @return The EIT_present_following_flag value.
     * @exception InformationSourceNoMoreAvailableException The information could not be accessed because the version of the service represented by this SIService object has changed or this service no longer exists in the SDT.
     * @exception TimeOutException         The required information could not be filtered within the time out period.
     * @exception NotImplementedException  This feature is not implemented on this platform.
     * @exception LackOfResourcesException The resources required to obtain the information are not available at this time.
    */
    public boolean retrieveEITPresentFollowingFlag() 
        throws 	RetrieveFailedException
    {
        synchronized( _sdtLock )
        {   return _srv.getEitPresentFollowingFlag();
        }
    }
    
    /**
     * Retrieve the running status of this service.
     * @return The running status (the possible values are defined in the SIRunningStatus interface)
     * @exception InformationSourceNoMoreAvailableException The information could not be accessed because the version of the service represented by this SIService object has changed or this service no longer exists in the SDT.
     * @exception TimeOutException         The required information could not be filtered within the time out period.
     * @exception NotImplementedException  This feature is not implemented on this platform.
     * @exception LackOfResourcesException The resources required to obtain the information are not available at this time.
     * @see SIRunningStatus
    */
    public byte retrieveRunningStatus() 
        throws RetrieveFailedException
    {
        synchronized( _sdtLock )
        {   return (byte)_srv.getRunningStatus();
        }
    }
    
    /**
     * Retrieve the free_CA_mode value of this service, false indicates none of the components 
     * of this service are scrambled.
     * @return The free_CA_mode value of this service.
     * @exception InformationSourceNoMoreAvailableException The information could not be accessed because the version of the service represented by this SIService object has changed or this service no longer exists in the SDT.
     * @exception TimeOutException         The required information could not be filtered within the time out period.
     * @exception NotImplementedException  This feature is not implemented on this platform.
     * @exception LackOfResourcesException The resources required to obtain the information are not available at this time.
    */
    public boolean retrieveFreeCAMode() 
        throws RetrieveFailedException
    {
        synchronized( _sdtLock )
        {   return _srv.getFreeCaMode();
        }
    }
    
    /**
     * Retrieve information associated with the transport stream carrying this service.
     * @param someDescriptorTags A list of hints for descriptors (identified by their tags) the application is interested in. If the array contains -1 as its one and only element, the application is interested in all descriptors. If someDescriptorTags is null, the application is not interested in descriptors. All non applicable tag values are ignored.
     * @return                   The SITransportStream object representing the transport stream carrying this service.
     * @exception InformationSourceNoMoreAvailableException The information could not be accessed because transport stream carrying this service is no longer present in a sdt.
     * @exception TimeOutException         The required information could not be filtered within the time out period.
     * @exception NotImplementedException  This feature is not implemented on this platform.
     * @exception LackOfResourcesException The resources required to obtain the information are not available at this time.
     * @see SITransportStream
     * @see DescriptorTag
    */
    public SITransportStream retrieveSITransportStream(
    	short[] someDescriptorTags
    ) throws
    	RetrieveFailedException
    {
        SITransportStream res = null;
        SIIterator it = 
            _sid.retrieveSITransportStreams( _sdt.getOriginalNetworkId()
                                           , _sdt.getTransportStreamId()
                                           , someDescriptorTags );
        if ( it.hasMoreElements() )
        {
            res = (SITransportStream)it.nextElement();
        }
        else
        {
            throw new InformationSourceNoMoreAvailableException();
        }
        return res;
    }
    
    /**
     * Retrieve information associated with the present event from the EIT-present/following.
     * If the present event is not present in the EIT-present/following, an 
     * InformationSourceNoMoreAvailableException is thrown.
     * @param someDescriptorTags A list of hints for descriptors (identified by their tags) the application is interested in. If the array contains -1 as its one and only element, the application is interested in all descriptors. If someDescriptorTags is null, the application is not interested in descriptors. All non applicable tag values are ignored.
     * @return                   The SIEvent object representing the present event within the service.
     * @exception InformationSourceNoMoreAvailableException The information could not be accessed because EIT of this service holds no present event.
     * @exception TimeOutException         The required information could not be filtered within the time out period.
     * @exception NotImplementedException  This feature is not implemented on this platform.
     * @exception LackOfResourcesException The resources required to obtain the information are not available at this time.
     * @see SIEvent
     * @see DescriptorTag
    */
    public SIEvent retrievePresentSIEvent(
    	short[]someDescriptorTags
    ) throws
    	RetrieveFailedException
    {
        EITpf eit = RetrieveProxy.filterEITpf( _sdt.getOriginalNetworkId(), _sdt.getTransportStreamId(), _srv.getServiceId() );
        EitEvent ev = eit.getPresentEitEvent();
        if ( ev == null )
        {
            throw new InformationSourceNoMoreAvailableException();
        }
        
        return new SIEvent( _sid, eit, ev, true );
    }
    
    /**
     * Retrieve information associated with the following event from the EIT-present/following. 
     * If the following event is not present in the EIT-present/following, an 
     * InformationSourceNoMoreAvailableException is thrown.
     * @param someDescriptorTags A list of hints for descriptors (identified by their tags) the application is interested in. If the array contains -1 as its one and only element, the application is interested in all descriptors. If someDescriptorTags is null, the application is not interested in descriptors. All non applicable tag values are ignored.
     * @return                   The SIEvent object representing the following event within the service.
     * @exception InformationSourceNoMoreAvailableException The information could not be accessed because EIT of this service holds no following event.
     * @exception TimeOutException         The required information could not be filtered within the time out period.
     * @exception NotImplementedException  This feature is not implemented on this platform.
     * @exception LackOfResourcesException The resources required to obtain the information are not available at this time.
     * @see SIEvent
     * @see DescriptorTag
    */
    public SIEvent retrieveFollowingSIEvent(
    	short[]someDescriptorTags
    ) throws
    	RetrieveFailedException
    {
        EITpf eit = RetrieveProxy.filterEITpf( _sdt.getOriginalNetworkId(), _sdt.getTransportStreamId(), _srv.getServiceId() );
        EitEvent ev = eit.getFollowingEitEvent();
        if ( ev == null )
        {
            throw new InformationSourceNoMoreAvailableException();
        }
        
        return new SIEvent( _sid, eit,ev, true );
    }
    
    /** 
     * Retrieve information associated with the scheduled events within the service for a requested period from
     * the EIT-schedule. The events are presented in the order they are present in the EIT-schedule.
     * @param someDescriptorTags A list of hints for descriptors (identified by their tags) the application is interested in. If the array contains -1 as its one and only element, the application is interested in all descriptors. If someDescriptorTags is null, the application is not interested in descriptors. All non applicable tag values are ignored.
     * @param startTime          The beginning of the required period in UTC time.
     * @param endTime            The end of the required period in UTC time.
     * @return                   A set of SIEvent objects representing the scheduled events within the service for the requested period (may be empty).
     * @exception TimeOutException         The required information could not be filtered within the time out period.
     * @exception NotImplementedException  This feature is not implemented on this platform.
     * @exception LackOfResourcesException The resources required to obtain the information are not available at this time.
     * @exception InvalidPeriodException   When no valid period is indicated.
     * @see SIEvent
     * @see SIIterator
     * @see DescriptorTag
    */
    public SIIterator retrieveScheduledSIEvents(
    	short[] someDescriptorTags,
    	Date startTime,
    	Date endTime
    )
    throws
    	SIException
    {
        if ( endTime.before(startTime) )
        {   throw new InvalidPeriodException();
        }

        Vector vec = new Vector( 1,1);
        Date midnigth = getMidnigth();
        
        // calculate number 3 hour periods past since midnight for start and end time
        int startPeriod = EITsch.numberOfPeriodsPastMidnight( midnigth, startTime ); 
        int endPeriod   = EITsch.numberOfPeriodsPastMidnight( midnigth, endTime );
        
        /*DBG*/Debug.msg( Debug.SI, "EIT schedule from "+startPeriod+" to "+endPeriod);
        
        int startDeltaTableId = (int)( startPeriod / 32 ); // table id modifier start of time period
        int endDeltaTableId   = (int)( endPeriod / 32 );   // table id modifier end of time period
        endDeltaTableId = ( ( endDeltaTableId>0xF) ? 0xf: endDeltaTableId  );
        // modify start and endPeriod to table_id
        startPeriod -= startDeltaTableId*32;
        endPeriod   -= endDeltaTableId*32;
        // calculate the number of EITsch table to filter.
        int nTable = endDeltaTableId-startDeltaTableId +1;
        
        
        RetrievalMediator[] arRm = new RetrievalMediator[ nTable ];
        RetrievalEvent[]    arEvent = new RetrievalEvent[ nTable ];
        
        // start filtering of EITsch tables.
        int off, start, end, i;

        for ( i=0; i<nTable; i++ )
        {   arEvent[i] = null;
        }
        
        do
        {
            for ( i=0; i<nTable; i++ )
            {
                if ( arEvent[i] == null )
                {
                    off = startDeltaTableId+i;
                    start = ( ( off == startDeltaTableId ) ? startPeriod: 0  );
                    end   = ( ( off == endDeltaTableId )   ? endPeriod  : 32 );
        
                    arRm[i] = RetrievalMediator.retrieveEITsch
                          ( null
                          , getOriginalNetworkId()
                          , getTransportStreamId()
                          , getServiceId()
                          , off, start, end
                          );
                }
            }
        }
        while ( !retrieveSetFinished( arRm, arEvent) );
        
        /*DBG*/Debug.msg( Debug.SI, "EIT schedule filtered");

        // if all RetrieveMediators return TIME_OUT, throw a TimeOutException
        boolean time_out = true;

        // get results (after retrieve start to allow parallel filtering
        RetrievalEvent event;
        for ( i=0; i<arRm.length; i++ )
        {   
            // get filter result
            event = arRm[i].getRetrievalEvent();
            if ( event.getResult() == RetrievalEvent.SUCCESS )
       	    {   // when no success, ignore, part of the requested table may not be available
                SITable[] arEit = event.getSITables();
                for ( int j=0; j< arEit.length; j++ )
                {
                    EITsch eit = (EITsch)arEit[j];
                    EitEvent[] arEitEvent = eit.getEitEvents();
                    for ( int k=0; k<arEitEvent.length; k++ )
                    {
                        Date d = arEitEvent[k].getStartTime();
                        if ( startTime.before(d) &&  endTime.after(d) )
                        {
                            vec.addElement( new SIEvent( _sid, eit, arEitEvent[k], false ) );
                        }
                    }
                }
            }
            // only one table without a time out is enough.
            time_out &= ( event.getResult() == RetrievalEvent.TIME_OUT );
        }
        if ( time_out )
        {   throw new TimeOutException();
        }
        
        return new SIIteratorImpl( vec );
    }
    
    /**
     * Retrieve information associated with the elementary streams which composed the service from the Program 
     * Map Table (PMT).
     * @param someDescriptorTags A list of hints for descriptors (identified by their tags) the application is interested in. If the array contains -1 as its one and only element, the application is interested in all descriptors. If someDescriptorTags is null, the application is not interested in descriptors. All non applicable tag values are ignored.
     * @return                   A set of PMTElementaryStream objects (may be empty).
     * @exception TimeOutException          The required information could not be filtered within the time out period. This exception is also thrown when required the transport stream is not accessible at this moment and the information of this PMT is not available in cache. 
     * @exception NotImplementedException  This feature is not implemented on this platform.
     * @exception LackOfResourcesException The resources required to obtain the information are not available at this time.
     * @see PMTElementaryStream
     * @see SIIterator
    */
    public SIIterator retrievePMTElementaryStreams(
    	short[] somePMTDescriptorTags
    )
    throws
    	RetrieveFailedException
    {
        PMT pmt = retrievePMT();
        Vector vec  = new Vector();
        ES[]   arES = pmt.getESs();
        for ( int i=0; i<arES.length; i++ )
        {
            vec.addElement( new PMTElementaryStream( _sid, _pmt, arES[i] ));
        }
        return new SIIteratorImpl( vec );
    }
    
    /**
     * This method retrieves either all PMT descriptors.
     * @return    A set of Descriptor objects representing the descriptors in the main loop of the PMT in the order they are broadcast.
     * @exception TimeOutException          The required information could not be filtered within the time out period. This exception is also thrown when required the transport stream is not accessible at this moment and the information of this PMT is not available in cache. 
     * @exception NotImplementedException  This feature is not implemented on this platform.
     * @exception LackOfResourcesException The resources required to obtain the information are not available at this time.
     * @see Descriptor
     * @see SIIterator
    */
    public SIIterator retrievePMTDescriptors(
    ) throws
    	RetrieveFailedException
    {
        PMT pmt = retrievePMT();
   		return ( new DescriptorIterator( pmt.getDescriptorList() ) );
    }
    
    /**
     * Retrieve a set of PMT descriptors from the main loop of the PMT associated with this service.
     * @param somePMTDescriptorTags A list of hints for descriptors (identified by their tags) the application is interested in. If the array contains -1 as its one and only element, the application is interested in all descriptors. If someDescriptorTags is null, the application is not interested in descriptors. All non applicable tag values are ignored.
     * @return                      A set of Descriptor objects with a tag value as indicated (may be empty).
     * @exception TimeOutException          The required information could not be filtered within the time out period. This exception is also thrown when required the transport stream is not accessible at this moment and the information of this PMT is not available in cache. 
     * @exception NotImplementedException  This feature is not implemented on this platform.
     * @exception LackOfResourcesException The resources required to obtain the information are not available at this time.
     * @see Descriptor
     * @see SIIterator
    */
    public SIIterator retrievePMTDescriptors(
    	short[] somePMTDescriptorTags
    ) throws
    	RetrieveFailedException
    {
        PMT pmt = retrievePMT();
   		return ( new DescriptorIterator( pmt.getDescriptorList(), somePMTDescriptorTags ) );
    }
    
    /**
     * Retrieve the tags of all PMT descriptors that are actually broadcast for the service in the order
     * they are broadcast.
     * @return The list of PMT descriptors actually broadcast for the service (identified by their tags).
     * @exception TimeOutException          The required information could not be filtered within the time out period. This exception is also thrown when required the transport stream is not accessible at this moment and the information of this PMT is not available in cache. 
     * @exception NotImplementedException  This feature is not implemented on this platform.
     * @exception LackOfResourcesException The resources required to obtain the information are not available at this time.
    */
    public short[] retrievePMTDescriptorTags(
    ) throws
    	RetrieveFailedException
    {
        PMT pmt = retrievePMT();
        return(  pmt.getDescriptorList().getDescriptorTags() );
    }
    
    /**
     * Add a listener on the PMT associated with the Service represented by this SIService object.
     * @param aListener The object interested in changes in the PMT.
     * @see SIEventListener
     * @see SIEventObject
    */
    public void addPMTListener(
    	SIEventListener aListener
    )
    {
        _pmtMonHand.addListener( aListener );
    }
    
    /**
     * Remove a listener on the PMT associated with the Service represented by this SIService object.
     * @param aListener The object no longer interested in changes in the PMT.
     * @see SIEventListener
     * @see SIEventObject
    */
    public void removePMTListener(
    	SIEventListener aListener
    )
    {
        _pmtMonHand.removeListener( aListener );
    }

    ///////////////// IMPLEMENTATION ////////////////////
    private Object     _pmtLock;
    private Object     _sdtLock;
    private Object     _retrieveUpdateLock;
	private SDT        _sdt;
	private Serv       _srv;
    private PMT        _pmt = null;
	private SIDatabase _sid;
	private ServiceMonitorHandler _servMonHand;
	private MonitorHandler        _pmtMonHand;

    /*******************
    */
    SIService( SIDatabase sid, SDT sdt, Serv srv )
	{
		_sid      = sid;
		_sdtLock  = new Object();
		_pmtLock  = new Object();
		_retrieveUpdateLock = new Object();
		_sdt      = sdt;
		_srv      = srv;
		_servMonHand = new ServiceMonitorHandler( this, sdt, _srv.getServiceId());
		_pmtMonHand  = new MonitorHandler( this,  new TableIdentifier( TableIdentifier.PMT, sdt.getOriginalNetworkId(), sdt.getTransportStreamId(), _srv.getServiceId() ));
	}

    /*******************
    Returns PMT of this service or throws an RetrieveFailedException. 
    The filtered PMT is returned to improve liveliness.
    */
    private PMT retrievePMT()
    	throws RetrieveFailedException
    {
        /*DBG*/Debug.msg( Debug.SI, "SIService retrievePMT");
        synchronized ( _pmtLock )
        {
            if ( _pmt== null )
            {
                _pmt = RetrieveProxy.filterPMT( getOriginalNetworkId(), getTransportStreamId(), getServiceId() );
            }
        }
        return _pmt;
    }

    /*******************
    Calculates the last UTC midnight in the time of the Transport Stream.
    */
    private Date getMidnigth()
    {
        // calculate last midnight time
        Date d = TransportStreamTime.getInstance().getTsTime();

/*DATE_NORMAL*/ Calendar cal = Calendar.getInstance( TimeZone.getTimeZone("UTC") );
/*DATE_NORMAL*/ cal.clear();
/*DATE_NORMAL*/ cal.setTime( TransportStreamTime.getInstance().getTsTime() );
/*DATE_NORMAL*/ // reset clock to last midnight
/*DATE_NORMAL*/ cal.set( Calendar.HOUR_OF_DAY, 0 );
/*DATE_NORMAL*/ cal.set( Calendar.MINUTE, 0 );
/*DATE_NORMAL*/ cal.set( Calendar.SECOND, 0 );
/*DATE_NORMAL*/ Date midnigth =  cal.getTime();

//DATE_TM*/ Date midnigth =  new Date( d.getYear(),d.getMonth(),d.getDay(),0/*houres*/,0/*minutes*/,0/*seconds*/);

        /*DBG*/Debug.msg( Debug.SI, Debug.IMPL, "EIT schedule midnigth "+midnigth );
        
        return midnigth;
    }
    
    /*****************
    */
    private boolean retrieveSetFinished( RetrievalMediator[] arRm, RetrievalEvent[] arEvent )
    {
        boolean ret=true;
        for ( int i=0; i<arEvent.length; i++ )
        {
            if ( arEvent[i]==null )
            {
                arEvent[i] = arRm[i].getRetrievalEvent();
            }
            if ( !(  arEvent[i].getResult() == RetrievalEvent.SUCCESS 
                  || arEvent[i].getResult() == RetrievalEvent.TIME_OUT 
                  ) 
               )
            {   ret = false;
                arEvent[i] = null;
            }
        }
        /*DBG*/Debug.msg( Debug.SI, Debug.IMPL, "EIT retrieveSetFinished "+ret);
        return ret;
    }
}
