/*
* Copyright(c) Philips Consumer Electronics B.V. 1998
* All rights reserved. Reproduction in whole or in part is
* prohibited without the written consent of the copyright holder.
*
*  Project             : Bali
*  File name   $Logfile: /func_test_platf2/org/davic/net/dvb/si/SITime.java $
*       Last save $Date: 2003/12/24 00:53:11 $
*             $Revision: 1.1.1.1 $
*          Last $Author: mikeseven $
*  Creation Date       : 09/11/98
*  First Author        : S.A.F.A. van den Heuvel
*                        NatLab - Eindhoven
*
*
*  Change history      :
*  ---------------
*
* $Log: SITime.java,v $
* Revision 1.1.1.1  2003/12/24 00:53:11  mikeseven
* Initial release of the CVS repository for MPEG-4 Systems code
* after the code was corrupted due to reorganization and human 
* errors.
*
* Revision 1.1  2003/04/03 04:29:33  mikeseven
* Initial release of Player3D and MPEG-J for all MPEG-4 reference softwares players with samples.
* For installation, follow instructions in docs/Player3D_setup.doc.
*
* Many thanks to Prashanth Gujjeti and Viswanathan Swaminathan from Sun Microsystems.
*
* -- Mikael Bourges-Sevenier (Mindego)
*
* Revision 1.1  2002/08/23 16:47:28  pg138550
* Changes for MPEG-J. NOT FINAL
*
* Revision 1.1  2000/06/06 18:43:21  alavilli
* After SISF integration into the CORE4 code
*
* Revision 1.1.2.1  1999/07/09 14:17:27  depetris
* Aggiornati i file per lo sviluppo e il rilascio a MPEG-J.
* Integra la versione 2.5 del core.
*
* Revision 1.2  1999-04-15 10:05:32+02  vdheuvel
* Files changed to hook the system up to the TM demultipler and the
* TM hardware. Changes include, swapping the java.util.Date class
* for the bali.tm.Date class (to prevent using 64 bit floats). Changing
* the HW sect filt used by bali.hw.SectFilt.
*
* Revision 1.1  1998-12-04 16:23:50+01  vdheuvel
* start of archive, 29 march 1999
* DAVIC API implementation
*
 * 
 * 3     10-11-98 15:55 Vdheuvel
 * Bug fix.
 * 
 * 2     10-11-98 11:22 Vdheuvel
 * Added implementation.
*/

package org.davic.net.dvb.si;
import  bali.table.TDT;
import  bali.table.TOT;
import  bali.support.Debug;
/*DATE_NORMAL*/ import  java.util.Date;
//DATE_TM*/     import  bali.tm.Date;

/**
 * This class represents the Time and Date Table (TDT) and the (optional) Time Offset Table (TOT).
 * When this SITime object represents a TDT table, the retrieveDescriptors and 
 * retrieveDescriptorTags methods will return empty SIIterators because no descriptors can 
 * be filtered from a TDT.
 * Note: because of the low update frequency of the TDT and TOT a filter action 
 * can only time out after 1 minute.
 * @see SIDatabase
 * @since DAVIC1.3
*/

public class SITime
	implements SIInformation{
    
    /** 
     * This method implements SIInformation.retrieveDescriptors (first prototype).
     * @see SIInformation
     * @see SIInformation#retrieveDescriptors
    */
    public SIIterator retrieveDescriptors(
    ) throws
    	RetrieveFailedException
    {
        DescriptorIterator res;
        if ( _tot != null )
        {
        	res = new DescriptorIterator( _tot.getDescriptorList() ) ;
        }
        else
        {
        	res = new DescriptorIterator() ;
        }
        return res;
    }
    
    /** 
     * This method implements SIInformation.retrieveDescriptors (second prototype).
     * @see SIInformation
     * @see SIInformation#retrieveDescriptors
    */
    public SIIterator retrieveDescriptors(
    	short[] someDescriptorTags
    ) throws
    	RetrieveFailedException
    {
        DescriptorIterator res;
        if ( _tot != null )
        {
        	res = new DescriptorIterator( _tot.getDescriptorList(), someDescriptorTags ) ;
        }
        else
        {
        	res = new DescriptorIterator() ;
        }
        return res;
    }
    
    /** 
     * This method implements SIInformation.retrieveDescriptorTags.
     * @see SIInformation
     * @see SIInformation#retrieveDescriptorTags
    */
    public short[] retrieveDescriptorTags(
    ) throws
    	RetrieveFailedException
    {
        short[] res;
        if ( _tot != null )
        {
        	res = _tot.getDescriptorList().getDescriptorTags() ;
        }
        else
        {
        	res = new short[0] ;
        }
        return res;
    }
    
    /**
     * This method implements SIInformation.getSIDatabase.
     * @see SIInformation
     * @see SIInformation#getSIDatabase
    */
    public SIDatabase getSIDatabase(
    )
    {
        return _sid;
    }
    
    /**
     * This method implements SIInformation.getUpdateTime.
     * @see SIInformation
     * @see SIInformation#getUpdateTime
    */
    public Date getUpdateTime(
    )
    {
        if ( _tot != null )
        {	return ( _tot.getSITableInfo().getUpdateTime() );
        }
        else
        {	return ( _tdt.getSITableInfo().getUpdateTime() );
        }
    }
    
    /**
     * This method implements SIInformation.fromActual.
     * @see SIInformation
     * @see SIInformation#fromActual
    */
    public boolean fromActual(
    )
    {
        return (true);
    }
    
    /**
     * This method implements SIInformation.getDataSource.
     * @see SIInformation
     * @see SIInformation#getDataSource
    */
    public org.davic.mpeg.TransportStream getDataSource(
    )
    {
        if ( _tot != null )
        {	return ( _tot.getSITableInfo().getSourceTs() );
        }
        else
        {	return ( _tdt.getSITableInfo().getSourceTs() );
        }
    }
    
    /**
     * Get the UTC time as coded in the TDT or TOT table this object represents.
     * @return The UTC as coded in the TDT or TOT table.
    */
    public Date getUTCTime(
    )
    {
        if ( _tot != null )
        {	return ( _tot.getUTCTime() );
        }
        else
        {	return ( _tdt.getUTCTime() );
        }
    }

    //////////////// IMPLEMENTATION //////////////////////////////
    private TDT        _tdt=null;
    private TOT        _tot=null;
    private SIDatabase _sid ;

    /*********************
    */
    SITime( SIDatabase sid, TDT tdt )
    {
        /*DBG*/Debug.msg( Debug.SI, "SITime : created with tdt "+tdt );
        _tdt = tdt;
    }

    /*********************
    */
    SITime( SIDatabase sid, TOT tot )
    {
        /*DBG*/Debug.msg( Debug.SI, "SITime : created with tot "+tot );
        _tot = tot;
    }
}
