/*
* Copyright(c) Philips Consumer Electronics B.V. 1998                         
* All rights reserved. Reproduction in whole or in part is                  
* prohibited without the written consent of the copyright holder.            
*                                                                            
*  Project             : Bali
*  File name   $Logfile: /func_test_platf2/org/davic/net/dvb/si/SITransportStream.java $
*       Last save $Date: 2003/12/24 00:53:11 $
*             $Revision: 1.1.1.1 $
*          Last $Author: mikeseven $
*  Creation Date       : 05/11/98
*  First Author        : S.A.F.A. van den Heuvel
*                        NatLab - Eindhoven
* 
*                                                                          
*  Change history      :
*  ---------------
*
* $Log: SITransportStream.java,v $
* Revision 1.1.1.1  2003/12/24 00:53:11  mikeseven
* Initial release of the CVS repository for MPEG-4 Systems code
* after the code was corrupted due to reorganization and human 
* errors.
*
* Revision 1.1  2003/04/03 04:29:33  mikeseven
* Initial release of Player3D and MPEG-J for all MPEG-4 reference softwares players with samples.
* For installation, follow instructions in docs/Player3D_setup.doc.
*
* Many thanks to Prashanth Gujjeti and Viswanathan Swaminathan from Sun Microsystems.
*
* -- Mikael Bourges-Sevenier (Mindego)
*
* Revision 1.1  2002/08/23 16:47:28  pg138550
* Changes for MPEG-J. NOT FINAL
*
* Revision 1.1  2000/06/06 18:43:21  alavilli
* After SISF integration into the CORE4 code
*
* Revision 1.1.2.1  1999/07/09 14:17:28  depetris
* Aggiornati i file per lo sviluppo e il rilascio a MPEG-J.
* Integra la versione 2.5 del core.
*
* Revision 1.3  1999-04-23 11:58:10+02  vdheuvel
* Repaired getServices bug.
*
* Revision 1.2  1999-04-15 09:53:18+02  vdheuvel
* Files changed to hook the system up to the TM demultipler and the
* TM hardware. Changes include, swapping the java.util.Date class
* for the bali.tm.Date class (to prevent using 64 bit floats). Changing
* the HW sect filt used by bali.hw.SectFilt.
*
* Revision 1.1  1998-12-04 10:43:18+01  vdheuvel
* start of archive, 29 march 1999
* DAVIC API implementation
*
 * 
 * 2     9-11-98 17:05 Vdheuvel
 * Updated retrieveUpdate and other class local retrieve methods.
 * 
 * 1     9-11-98 14:01 Vdheuvel
 * Creation
*/

package org.davic.net.dvb.si;
import  bali.table.NIT;
import  bali.table.TS;
/*DATE_NORMAL*/ import  java.util.Date;
//DATE_TM*/     import  bali.tm.Date;

/**
 * This class represents a particular transport stream belonging to a bouquet or carried via a network.
 * A SITranportStream object retrieved from instances of the classes SIDatabase, SINetwork, or SIService
 * represent a part of the NIT table; in this case all descriptor accessing method target the NIT. 
 * A SITransportStream object retrieved from an instance of SIBouquet represents a part of the BAT; in
 * this case all descriptor accessing methods target the BAT. 
 * @since DAVIC1.3
*/

public class SITransportStream
	implements SIInformation
{
    /** 
    * This method implements SIInformation.retrieveDescriptors (first prototype).
    * @see SIInformation
    * @see SIInformation#retrieveDescriptors
    */
    public SIIterator retrieveDescriptors() 
        throws RetrieveFailedException
    {
		return ( new DescriptorIterator( _ts.getDescriptorList() ) );
    }
    
    /** 
    * This method implements SIInformation.retrieveDescriptors (second prototype).
    * @see SIInformation
    * @see SIInformation#retrieveDescriptors
    */
    public SIIterator retrieveDescriptors( short[] someDescriptorTags ) 
        throws RetrieveFailedException
    {
		return ( new DescriptorIterator( _ts.getDescriptorList(), someDescriptorTags ) );
    }
    
    /** 
    * This method implements SIInformation.retrieveDescriptorTags.
    * @see SIInformation
    * @see SIInformation#retrieveDescriptorTags
    */
    public short[] retrieveDescriptorTags() 
        throws RetrieveFailedException
    {
   		return ( _ts.getDescriptorList().getDescriptorTags() );
    }
    
    /**
    * This method implements SIInformation.getSIDatabase.
    * @see SIInformation
    * @see SIInformation#getSIDatabase
    */
    public SIDatabase getSIDatabase()
    {
        return _sid;
    }
    
    /**
    * This method implements SIInformation.getUpdateTime.
    * @see SIInformation
    * @see SIInformation#getUpdateTime
    */
    public Date getUpdateTime()
    {
   		return ( _ts.getSITableInfo().getUpdateTime() );
    }
    
    /**
    * This method implements SIInformation.fromActual.
    * @see SIInformation
    * @see SIInformation#fromActual
    */
    public boolean fromActual()
    {
   		return ( _ts.getSITableInfo().fromActual() );
    }
    
    /**
    * This method implements SIInformation.getDataSource.
    * @see SIInformation
    * @see SIInformation#getDataSource
    */
    public org.davic.mpeg.TransportStream getDataSource()
    {
   		return ( _ts.getSITableInfo().getSourceTs() );
    }
    
    /**
    * Get the URL addressing the transport stream.
    * @return The URL of this transport stream.
    */
    public java.lang.String getURL()
    {
		String str;
		str = "dvb://"
		    + Integer.toHexString( getOriginalNetworkId() )
		    + "."
		    + Integer.toHexString( getTransportStreamId() )
		    ;
		return ( str );
    }
    
    /**
    * Get the original network identification.
    * @return The original network identification identifier.
    */
    public int getOriginalNetworkId()
    {
		return (short)_ts.getOriginalNetworkId();
    }
    
    /**
     * Get the transport stream identification.
     * @return The transport stream identification identifier.
    */
    public int getTransportStreamId()
    {
		return (short)_ts.getTransportStreamId();
    }
    
    /**
     * Retrieve information associated with services carried via the transport stream.
     * @param someDescriptorTags A list of hints for descriptors (identified by their tags) the application is interested in. If the array contains -1 as its one and only element, the application is interested in all descriptors. If someDescriptorTags is null, the application is not interested in descriptors. All non applicable tag values are ignored.
     * @return                   The SIService objects representing the services in this transport stream (may be empty).
     * @exception TimeOutException         The required information could not be filtered within the time out period.
     * @exception NotImplementedException  This feature is not implemented on this platform.
     * @exception LackOfResourcesException The resources required to obtain the information are not available at this time.
     * @see SIService
     * @see SIIterator
     * @see DescriptorTag
    */
    public SIIterator retrieveSIServices( short[] someDescriptorTags
    ) throws
    	RetrieveFailedException
    {
        return _sid.retrieveSIServices( _ts.getOriginalNetworkId()
                                      , _ts.getTransportStreamId()
                                      , -1
                                      , someDescriptorTags );
    }
    
    
    /**
     * Retrieve the SITransportStreamDescription object representing the information of the TSDT
     * table of this transport stream.
     * @param someDescriptorTags A list of hints for descriptors (identified by their tags) the application is interested in. If the array contains -1 as its one and only element, the application is interested in all descriptors. If someDescriptorTags is null, the application is not interested in descriptors. All non applicable tag values are ignored.
     * @return                   The SITransportStreamDescription of the actual transport stream.
     * @exception TimeOutException         The required information could not be filtered within the time out period. This exception is also thrown when this transport stream is not accessible at this moment and the information of this TSDT is not available in cache.
     * @exception NotImplementedException  This feature is not implemented on this platform.
     * @exception LackOfResourcesException The resources required to obtain the information are not available at this time.
     * @see SITransportStreamDescription
     * @see DescriptorTag
    */
    public SITransportStreamDescription retrieveSITransportStreamDescription(
    	short[] someDescriptorTags
    ) throws
    	RetrieveFailedException
    {
       // TODO retrieve the required information.
        throw new NotImplementedException();
     }

	//////////////////////////// IMPLEMENTATION ///////////////////////
	SIDatabase	_sid;
	TS		   	_ts;
	
	/*****************
	*/
	SITransportStream( SIDatabase sid, TS ts )
	{
		_sid = sid;
		_ts  = ts;
	}
}
