/*
* Copyright(c) Philips Consumer Electronics B.V. 1998
* All rights reserved. Reproduction in whole or in part is
* prohibited without the written consent of the copyright holder.
*
*  Project             : Bali
*  Module              : bali.si
*  File name   $Logfile: /func_test_platf2/bali/si/scheduler/RetrieveListener.java $
*       Last save $Date: 2003/12/24 00:53:11 $
*             $Revision: 1.1.1.1 $
*          Last $Author: mikeseven $
*  Creation Date       : 7/12/98
*  First Author        : S.A.F.A. van den Heuvel
*                        NatLab - Eindhoven
*
*
*  Change history      :
*  ---------------
*
* $Log: NetworkInterface.java,v $
* Revision 1.1.1.1  2003/12/24 00:53:11  mikeseven
* Initial release of the CVS repository for MPEG-4 Systems code
* after the code was corrupted due to reorganization and human 
* errors.
*
* Revision 1.1  2003/04/03 04:29:33  mikeseven
* Initial release of Player3D and MPEG-J for all MPEG-4 reference softwares players with samples.
* For installation, follow instructions in docs/Player3D_setup.doc.
*
* Many thanks to Prashanth Gujjeti and Viswanathan Swaminathan from Sun Microsystems.
*
* -- Mikael Bourges-Sevenier (Mindego)
*
* Revision 1.1  2002/08/23 16:47:28  pg138550
* Changes for MPEG-J. NOT FINAL
*
* Revision 1.1  2000/06/06 18:43:22  alavilli
* After SISF integration into the CORE4 code
*
* Revision 1.1.2.1  1999/07/09 14:17:29  depetris
* Aggiornati i file per lo sviluppo e il rilascio a MPEG-J.
* Integra la versione 2.5 del core.
*
* Revision 1.1  1999-04-14 10:35:36+02  vdheuvel
* Creation.
*
 *
 * 1     9-11-98 13:59 Vdheuvel
 * Creation
*/

package org.davic.net.tuning;
import  org.davic.resources.*;
import  org.davic.mpeg.TransportStream;
import  org.davic.mpeg.dvb.DvbTransportStream;
import  bali.component.TransportStreamSelection;
import  bali.component.ComponentMaster;
import  bali.table.NIT;
import  bali.table.TS;
import  bali.table.TableIdentifier;
import  bali.cache.SITableCache;
import  bali.retrieve.RetrievalMediator;
import  bali.retrieve.RetrievalEvent;
import  bali.retrieve.Monitor;
import  bali.support.Debug;
import  bali.support.ActiveEventSender;
import  bali.hw.tuner.HwTunerEvent;
import  bali.hw.tuner.HwTuner;

/** Network interface class represents a physical network interface
  * that can be used for receiving broadcast transport streams.
  */
public class NetworkInterface
{
    /* For javadoc to hide the non-public constructor. */
    NetworkInterface()
    {
        Debug.msg( Debug.TUNING, "NetworkInterface created");
        
        // monitor NIT
        DvbTransportStream ts = TransportStreamSelection.getInstance().getCurrentTs();
        TableIdentifier ti    = new TableIdentifier( TableIdentifier.NIT, ts.getNetworkId() );
        _nitMonitor           = new Monitor( null, ti );
        _listHand             = new NetworkInterfaceListenerHandler();
        _friend               = new NetworkInterfaceFriend( this );
        HwTuner.getInstance().addTunerListener( _friend );
    }



    /** Returns the transport stream to which the network
    * Interface is currently tuned. Returns null if
    * if the network interface is not currently tuned
    * to a transport stream, e.g. because it is performing
    * a tune action.
    * @return Transport stream to which the network interface
    *         is currently tuned
    */
    public TransportStream getCurrentTransportStream() 
    {
        return TransportStreamSelection.getInstance().getCurrentTs();
    }



    /** Returns the URL of the transport stream to which the network
    * interface is connected. Returns null if the network
    * interface is not currently tuned to a transport stream.
    * @return URL of the transport stream to which the network interface
    *         is tuned
    */

    public String getURL() 
    {
        DvbTransportStream ts = TransportStreamSelection.getInstance().getCurrentTs();
		String str;
		str = "dvb://"
		    + Integer.toHexString( ts.getOriginalNetworkId() )
		    + "."
		    + Integer.toHexString( ts.getTransportStreamId() )
		    ;
		return ( str );
    }


    /** Returns true if this network interface is reserved, otherwise
    * returns false
    * @return true, if the network interface is reserved, otherwise false
    */

    public synchronized boolean isReserved() 
    {
        return true;
    }



    /** Returns true if this network interface is a local.
    * (i.e. embedded in the receiver)
    * @return true, if the network interface is local, otherwise false
    */
    public boolean isLocal() 
    {
        return true;
    }



    /** Lists the known transport streams that are accessible through
    * this network interface. If there are no such streams, returns
    * an array with length of zero.
    * @return array of transport streams accassible through this network
    *         interface
    */
    public TransportStream[] listAccessibleTransportStreams() 
    {
        NIT nit = getNITActual();
        
        TS[] arTs = nit.getTSs();
        DvbTransportStream[] res = new DvbTransportStream[ arTs.length ];
        ComponentMaster      cm  = ComponentMaster.getInstance();   
        
        for ( int i=0; i<arTs.length; i++ )
        {
            res[i] = cm.getTransportStream
                ( arTs[i].getOriginalNetworkId()
                , nit.getNetworkId()
                , arTs[i].getTransportStreamId()
                );
        }
        // return result
        return res;

    }



    /** This method returns the type of the delivery system
    * that this network interface is connected to.
    * @return delivery system type
    */

    public int getDeliverySystemType() 
    {
        return DeliverySystemType.SATELLITE_DELIVERY_SYSTEM;
    }



    /** Adds a listener for network interface events
    * @param listener listener object to be registered to receive network interface
    *          events
    */

    public void addNetworkInterfaceListener(NetworkInterfaceListener listener) 
    {
        _listHand.addListener( listener );
    }



    /** Removes a registered listener
    * @param listener listener object to be removed so that it will not receive
    *        network interface events any more
    */
    public void removeNetworkInterfaceListener(NetworkInterfaceListener listener) 
    {
        _listHand.removeListener( listener );
    }

    //////////////////////// IMPLEMENTATION ///////////////////////
    private Monitor                         _nitMonitor;
    private NetworkInterfaceListenerHandler _listHand;
    private NetworkInterfaceFriend          _friend;

    /****************
    */
    private NIT getNITActual()
    {
        NIT nit = (NIT)_nitMonitor.getLatestSITable();
        
        if ( nit == null || !nit.getSITableInfo().fromActual() )
        {   // table not present or not an 'actual' table
            // filter new NIT actual
        
            RetrievalMediator rm;
            RetrievalEvent event;
            do
            {
                rm = RetrievalMediator.retrieveNITActual(null);
                event = rm.getRetrievalEvent();
                if ( event.getResult() == RetrievalEvent.SUCCESS )
        	    {
                    nit = (NIT)event.getSITable();
                }
            }
            while (  nit==null );
        }
        
        return nit;
    }

    /*************
    */
    void newHwTunerEvent( HwTunerEvent tuningEvent )
    {
        NetworkInterfaceEvent event = null;
        if ( tuningEvent.getType() == tuningEvent.TUNING )
        {
            event = new NetworkInterfaceTuningEvent( this );
        }
        else
        {
            event = new NetworkInterfaceTuningOverEvent( this );
        }
        ActiveEventSender.getInstance( ActiveEventSender.TUNING_APP ).
            sendEvent( _listHand, event );
    }

}


