package org.davic.net.tuning;

import org.davic.resources.*;

import org.davic.mpeg.TransportStream;



/** NetworkInterfaceController represents a controller that 

  * can be used for tuning a network interface. 

  * The application may create a network interface controller

  * object and try to reserve the capability to tune a network 

  * interface.

  * The capability to tune a network interface is a resource

  * and the network interface controller acts as a resource

  * proxy for this resource.

  */

  

public class NetworkInterfaceController implements ResourceProxy {



  /** Creates a NetworkInterfaceController

    * @param rc The ResourceClient that the controller is associated with

    */

  public NetworkInterfaceController(ResourceClient rc) 

  {

    _resClient = rc;

  }



  /** Tunes asynchronously to the given stream (specified

    * by a URL). <p>

    * This method causes the NetworkInterfaceTuningEvent and the

    * NetworkInterfaceTuningOverEvent to be sent to the listeners

    * of this NetworkInterface. The reason

    * code in the event tells if the tuning succeeded 

    * or failed. After a failed tuning action, the network

    * interface may be tuned to any transport stream or

    * be left in a state where it is not tuned to any 

    * transport stream.

    * @param url The URL pointing to the transport stream to tune to

    * @exception StreamNotFoundException raised if the specified URL

    *                                    does not point to any known 

    *                                    transport stream or the currently

    *                                    reserved NetworkInterface cannot

    *                                    tune to the specified transport stream

    * @exception IncorrectURLException raised if the syntax of the URL is 

    *                                  incorrect or it is not of valid type 

    *                                  that references a broadcast trasnport

    *                                  stream

    * @exception NotOwnerException if no network interface is reserved at 

    *                              the moment

    */	

  public synchronized void tune(String url) 

    throws NetworkInterfaceException {

    }

  

  /** Tunes asynchronously to the given transport stream. <p>

    * This method causes the NetworkInterfaceTuningEvent and the

    * NetworkInterfaceTuningOverEvent to be sent to the listeners

    * of this NetworkInterface. The reason

    * code in the event tells if the tuning succeeded 

    * or failed. After a failed tuning action, the network

    * interface may be tuned to any transport stream or

    * be left in a state where it is not tuned to any 

    * transport stream.

    * 

    * @param ts Transport stream object to which to tune to 

    * @exception StreamNotFoundException raised if the specified

    *                                    transport stream is not associated

    *                                    with the currently reserved 

    *                                    network interface

    * @exception NotOwnerException raised if no network interface is 

    *                              reserved at the moment

    */

  public synchronized void tune(TransportStream ts) 

    throws NetworkInterfaceException 

    {

        throw new NotOwnerException();

    }

  

  /** Tries to reserve exclusively the control

    * over the specified network interface. <p>

    * If the reservation succeeds, a NetworkInterfaceReservedEvent is

    * sent to the listeners of the NetworkInterfaceManager.

    * If this NetworkInterfaceController has already

    * reserved another NetworkInterface, then it will

    * either release that NetworkInterface and reserve

    * the specified one, or throw an exception.

    * If the specified NetworkInterface has already been

    * reserved by this NetworkInterfaceController, then

    * this method does nothing.

    * @param ni Network Interface to be reserved

    * @param requestData Used by the Resource Notification

    *                    API in the requestRelease method

    *                    of the ResourceClient interface.

    *                    The usage of this parameter is optional and

    *                    a null reference may be supplied.

    * @exception NoFreeInterfaceException raised if the requested

    *                                     network interface can not 

    *                                     be raised

    */

  public synchronized void reserve(NetworkInterface ni,

				   Object requestData)

    throws NetworkInterfaceException 

    {

        throw new NoFreeInterfaceException();

    }

  

  /** Tries to reserve exclusively the control over a network

    * interface that can receive the transport stream

    * specified by the 'url' parameter. <p>

    * The specific network interface is selected by the method

    * implementation. <p>

    * If the reservation succeeds, a

    * NetworkInterfaceReservedEvent is sent to the listeners of the

    * NetworkInterfaceManager.

    * If this NetworkInterfaceController has already

    * reserved another NetworkInterface, then it will

    * either release that NetworkInterface and reserve

    * an appropriate one, or throw an exception.

    * If NetworkInterfaceController has already reserved

    * a NetworkInterface that is able to tune to the

    * specified transport stream, then this method does

    * nothing.

    * @param url URL that points to a transport stream that 

    *                the reserved network interface should be able to 

    *                tune to

    * @param requestData Used by the Resource Notification

    *                    API in the requestRelease method

    *                    of the ResourceClient interface.

    *                    The usage of this parameter is optional and

    *                    a null reference may be supplied.

    * @exception NoFreeInterfaceException raised if a network interface

    *                                     can not be reserved

    * @exception StreamNotFoundException raised if the specified URL

    *                                    does not point to any known 

    *                                    transport stream 

    * @exception IncorrectURLException raised if the syntax of the URL is 

    *                                  incorrect or it is not of valid type 

    *                                  that references a broadcast trasnport

    *                                  stream

    * 

    */

  public synchronized void reserveFor(String url,

				      Object requestData)

    throws NetworkInterfaceException 

    {

        throw new NoFreeInterfaceException();

    }

  

  /** Releases the tuner. <p>

    * This method causes a NetworkInterfaceReleasedEvent to be

    * sent to the listeners of the NetworkInterfaceManager.

    * @exception NotOwnerException raised if the controller does not 

    *                              currently have a network interface

    *                              reserved

    */

  public synchronized void release() 

    throws NetworkInterfaceException 

    {

        throw new NotOwnerException();

    }

  

  /** Returns the network interface associated with this	

    * controller.

    * @return the network interface associated with this controller or 

    * null if no network interface has been reserved.

    */

  public NetworkInterface getNetworkInterface()

  {

    return null;

  }

  

  /** Returns the resource client that is associated with

    * this NetworkInterfaceController.

    * This method implements getClient method of 

    * org.davic.resources.ResourceProxy.

    * @return the resource client associated with this controller

    */

  public ResourceClient getClient() 

  {

    return _resClient ;

  }



    ///////////////////////////// IMPLEMENTATION /////////////////////

    private ResourceClient _resClient;

}

