/*
* Copyright(c) Philips Consumer Electronics B.V. 1998                         
* All rights reserved. Reproduction in whole or in part is                  
* prohibited without the written consent of the copyright holder.            
*                                                                            
*  Project             : Bali
*  Module              : bali.si 
*  File name   $Logfile: /func_test_platf2/bali/si/scheduler/RetrieveListener.java $
*       Last save $Date: 2003/12/24 00:53:11 $
*             $Revision: 1.1.1.1 $
*          Last $Author: mikeseven $
*  Creation Date       : 7/12/98
*  First Author        : S.A.F.A. van den Heuvel
*                        NatLab - Eindhoven
* 
*                                                                          
*  Change history      :
*  ---------------
*
* $Log: NetworkInterfaceManager.java,v $
* Revision 1.1.1.1  2003/12/24 00:53:11  mikeseven
* Initial release of the CVS repository for MPEG-4 Systems code
* after the code was corrupted due to reorganization and human 
* errors.
*
* Revision 1.1  2003/04/03 04:29:34  mikeseven
* Initial release of Player3D and MPEG-J for all MPEG-4 reference softwares players with samples.
* For installation, follow instructions in docs/Player3D_setup.doc.
*
* Many thanks to Prashanth Gujjeti and Viswanathan Swaminathan from Sun Microsystems.
*
* -- Mikael Bourges-Sevenier (Mindego)
*
* Revision 1.1  2002/08/23 16:47:28  pg138550
* Changes for MPEG-J. NOT FINAL
*
* Revision 1.1  2000/06/06 18:43:22  alavilli
* After SISF integration into the CORE4 code
*
* Revision 1.1.2.1  1999/07/09 14:17:31  depetris
* Aggiornati i file per lo sviluppo e il rilascio a MPEG-J.
* Integra la versione 2.5 del core.
*
* Revision 1.1  1999-04-14 10:35:36+02  vdheuvel
* Creation.
*
 * 
 * 1     9-11-98 13:59 Vdheuvel
 * Creation
*/


package org.davic.net.tuning;

import org.davic.resources.*;
import org.davic.mpeg.TransportStream; 
import bali.support.Debug;


/** Network interface manager is an object that keeps track
  * of broadcast network interfaces that are connected to the 
  * receiver. <p>
  * There is only one instance of the network interface manager
  * in a receiver and this can be retrieved using the getInstance 
  * method.
  */

public class NetworkInterfaceManager implements ResourceServer 
{

    /* For javadoc to hide the non-public constructor. */
    private NetworkInterfaceManager() 
    {
        Debug.msg( Debug.TUNING, "NetworkInterfaceManager created");
        _ni = new NetworkInterface();
    }
    
    
    
    /** Returns the instance of the NetworkInterfaceManager
    * @return network interface manager 
    */
    public static NetworkInterfaceManager getInstance() 
    {
        synchronized ( _lock )
        {
            if ( _instance == null )
            {
                _instance =  new NetworkInterfaceManager();
            }
        }
        return _instance;
    }
    
    
    
    
    
    /** Returns all network interfaces. <p>
    * If there are no network interfaces, returns
    * an array with the length of zero.
    * @return an array containing all network interfaces
    */
    
    public NetworkInterface[] getNetworkInterfaces() 
    {
        NetworkInterface[] ar = new NetworkInterface[1];
        ar[0] = _ni;
        return ar;
    }
    
    
    
    /** Returns the NetworkInterface with which this 
    * TransportStream object is associated. It does 
    * neither tune nor reserve the
    * NetworkInterface.
    * @param ts Transport stream object
    * @return network interface that is associated with the transport stream
    */
    public NetworkInterface getNetworkInterface(TransportStream ts) 
    {
        if ( _ni.getCurrentTransportStream() == ts )
        {
            return _ni;
        }
        else
        {
            return null;
        }
    }
    
    
    
    /** Registers a resource status listener to receive resource status events
    * @param listener listener to be registered
    */
    public void addResourceStatusEventListener(ResourceStatusListener listener) 
    {
        // ignore, no tuning functionality present so no events are send
    }

  

    /** Removes the registration of a registered listener so that 
    * it will not receive resource status events any more
    * @param listener listener whose registration is to be removed
    */
    public void removeResourceStatusEventListener(ResourceStatusListener listener) 
    {
        // ignore, no tuning functionality present so no events are send
    }


    /////////////// IMPLEMENTATION //////////////////////
    private static NetworkInterfaceManager _instance = null;
    private static Object _lock = new Object();
    private NetworkInterface _ni;
 

}




