/*
* Copyright(c) Philips Consumer Electronics B.V. 1998
* All rights reserved. Reproduction in whole or in part is
* prohibited without the written consent of the copyright holder.
*
*  Project             : Bali
*  Module              : bali.si
*  File name   $Logfile: /func_test_platf2/bali/si/scheduler/RetrieveListener.java $
*       Last save $Date: 2003/12/24 00:53:11 $
*             $Revision: 1.1.1.1 $
*          Last $Author: mikeseven $
*  Creation Date       : 7/12/98
*  First Author        : S.A.F.A. van den Heuvel
*                        NatLab - Eindhoven
*
*
*  Change history      :
*  ---------------
*
* $Log: StreamTable.java,v $
* Revision 1.1.1.1  2003/12/24 00:53:11  mikeseven
* Initial release of the CVS repository for MPEG-4 Systems code
* after the code was corrupted due to reorganization and human 
* errors.
*
* Revision 1.1  2003/04/03 04:29:35  mikeseven
* Initial release of Player3D and MPEG-J for all MPEG-4 reference softwares players with samples.
* For installation, follow instructions in docs/Player3D_setup.doc.
*
* Many thanks to Prashanth Gujjeti and Viswanathan Swaminathan from Sun Microsystems.
*
* -- Mikael Bourges-Sevenier (Mindego)
*
* Revision 1.1  2002/08/23 16:47:28  pg138550
* Changes for MPEG-J. NOT FINAL
*
* Revision 1.1  2000/06/06 18:43:24  alavilli
* After SISF integration into the CORE4 code
*
* Revision 1.1.2.1  1999/07/09 14:17:33  depetris
* Aggiornati i file per lo sviluppo e il rilascio a MPEG-J.
* Integra la versione 2.5 del core.
*
* Revision 1.1  1999-04-14 10:35:38+02  vdheuvel
* Creation.
*
*/

package org.davic.net.tuning;
import org.davic.mpeg.dvb.DvbTransportStream;
import org.davic.mpeg.TransportStream;
import java.util.Vector;
import java.net.MalformedURLException;
import bali.support.DavicURL;

/** A stream table containing information about transport streams
  * known to the receiver
  */
public class StreamTable {
    /** Returns the transport streams that match the URL. <p>
    * The URL must uniquely identify the transport stream (i.e.
    * for DVB networks, it must specify the orig_network_id and the 
    * transport_stream_id). If the URL is more specific than
    * just identifying the transport stream, any more specific part of it 
    * will be disregarded. <p>
    * Since the same transport stream may be received
    * via multiple networks and via multiple network interfaces, 
    * this function returns an array of all the possible 
    * transport stream objects that can be 
    * used for receiving this transport stream.
    * @param url A URL that points to a broadcast transport stream
    * @exception IncorrectURLException raised if the syntax of the URL is 
    *                                  incorrect or it is not of valid type 
    *                                  that references a broadcast trasnport
    *                                  stream
    * @return array of transport streams
    */
    public static TransportStream[] getTransportStreams(String url) 
    throws NetworkInterfaceException 
    {
        DavicURL davUrl;
        try
        {
            davUrl = new DavicURL( url );
        }
        catch ( MalformedURLException e )
        {
            throw new IncorrectURLException();
        }
        Vector vec = new Vector(1,1);
        int i,j;
        int orNwId = davUrl.getOriginalNetworkId();
        int tsId   = davUrl.getTransportStreamId();
        
        NetworkInterface[] arNi = NetworkInterfaceManager.getInstance().getNetworkInterfaces();
        for ( i=0; i< arNi.length; i++ )
        {
            TransportStream[] arTs  = arNi[i].listAccessibleTransportStreams();
            for ( j=0; j<arTs.length; j++ )
            {
                if (  ( orNwId == ((DvbTransportStream)arTs[i]).getOriginalNetworkId() )
                   && ( tsId == arTs[i].getTransportStreamId() || tsId == -1 )
                   )
                {
                    vec.addElement( arTs[i] );
                }
            }
        }
        TransportStream[] arTs = new TransportStream[ vec.size() ];
        vec.copyInto( arTs );
        return arTs;
    }

	

    /** Returns all known transport streams on all network
    * interfaces as an array of URLs.
    * @return array of URLs pointing to known transport streams
    */
    public static String[] listTransportStreams() 
    {
        NetworkInterface[] arNI = NetworkInterfaceManager.getInstance().getNetworkInterfaces();
        Vector vecURL = new Vector();
        
        // go through all network interfaces
        for ( int i=0; i<arNI.length; i++ )
        {	
          // access the transport streams of this network interface
          TransportStream[] arTS = arNI[i].listAccessibleTransportStreams() ;
          for ( int j = 0; j<arTS.length; j++ )
          {
            //  create a URL for each transport stream and add to Vector
            String str = new String();
            DvbTransportStream ts = (DvbTransportStream)arTS[j];
            
        	str = "dvb://"
        	    + Integer.toHexString( ts.getOriginalNetworkId() )
        	    + "."
        	    + Integer.toHexString( ts.getTransportStreamId() )
        	    ;
            vecURL.addElement( str );
          }
        }
        String[] arURL = new String[ vecURL.size() ];
        vecURL.copyInto( arURL );
        
        return arURL;

    }

}
