/*
* Copyright(c) Philips Consumer Electronics B.V. 1998                         
* All rights reserved. Reproduction in whole or in part is                  
* prohibited without the written consent of the copyright holder.            
*                                                                            
*  Project             : Bali
*  File name   $Logfile: $
*       Last save $Date: 2003/12/24 00:53:13 $
*             $Revision: 1.1.1.1 $
*          Last $Author: mikeseven $
*  Creation Date       : early 1998
*  First Author        : H. v. Woerkom
*                        NatLab - Eindhoven
* 
*                                                                          
*  Change history      :
*  ---------------
* $Log: SIBouquetUI.java,v $
* Revision 1.1.1.1  2003/12/24 00:53:13  mikeseven
* Initial release of the CVS repository for MPEG-4 Systems code
* after the code was corrupted due to reorganization and human 
* errors.
*
* Revision 1.1  2003/04/03 04:29:39  mikeseven
* Initial release of Player3D and MPEG-J for all MPEG-4 reference softwares players with samples.
* For installation, follow instructions in docs/Player3D_setup.doc.
*
* Many thanks to Prashanth Gujjeti and Viswanathan Swaminathan from Sun Microsystems.
*
* -- Mikael Bourges-Sevenier (Mindego)
*
* Revision 1.1  2002/08/23 16:47:31  pg138550
* Changes for MPEG-J. NOT FINAL
*
* Revision 1.2  2000/07/21 20:39:01  alavilli
* adding missing files in the previous checkin for SISF
*
* Revision 1.1.2.1  1999/07/09 14:27:29  depetris
* Aggiornati i file per lo sviluppo e il rilascio a MPEG-J.
* Integra la versione 2.5 del core.
*
* Revision 1.1  1999-05-05 14:58:49+02  vdheuvel
* *** empty log message ***
*
* Revision 1.3  1999-04-23 13:22:33+02  vdheuvel
* Debug of errors due to FTP2 hook-up.
* Added new functionality:
*   - display of strings
*   - diplay of descriptor contents
*   - optimised filter order
*
* Revision 1.2  1999-04-23 10:52:47+02  vdheuvel
* Adapted to the new ftp.
*
*
*/

package test_awt;

import java.awt.*;
import org.davic.net.dvb.si.*;

public class SIBouquetUI extends Panel {
	public List _serviceList;
	public List _tsList;
	public Button exit;
	public Button refresh;
	public Button descriptors;
	public Frame frame;

	public SIBouquetUI( SIBouquet t ) {
		_siBouquet = t;

		setLayout(new FlowLayout());

		add(new Label("TSs"));
		_tsList = new MyList(7,false,250);
		add(_tsList);

		add(new Label("Services"));
		_serviceList = new MyList(7,false,250);
		add(_serviceList);

		descriptors = new Button("Descriptors");
		add(descriptors);

		refresh = new Button("Refresh");
		add(refresh);

		exit = new Button("Exit");
		add(exit);

		validate();

		frame = new Frame("SIBouquet " + _siBouquet.getBouquetId()  );
		frame.setLayout(new BorderLayout());
		frame.add("North",this);
		frame.pack();
		frame.show();

		updateAll();
	}

	public void updateAll() {
		SIIterator iter;
		int i = 0;
		
		try {
			iter = _siBouquet.retrieveSITransportStreams( null );
		} catch (SIException e) {
			return;
		}
		_tsList.removeAll();
		_tss = new SITransportStream[iter.numberOfRemainingObjects()];
		while (iter.hasMoreElements()) {
			SITransportStream se = (SITransportStream) iter.nextElement();
			
			_tss[i++] = se;
			_tsList.addItem("ts " + se.getOriginalNetworkId() + ", " + se.getTransportStreamId() );
		}

        i=0;
		try {
			iter = _siBouquet.retrieveSIServices( null );
		} catch ( TimeOutException te ) {
		    iter = te.getFilteredData();
		} catch (SIException e) {
			return;
		}
		_serviceList.removeAll();
		_services = new SIService[iter.numberOfRemainingObjects()];
		while (iter.hasMoreElements()) {
			SIService se = (SIService) iter.nextElement();
			
			_services[i++] = se;
			_serviceList.addItem("service " + se.getOriginalNetworkId() + ", " + se.getTransportStreamId() + ", " + se.getServiceId() );
		}
		validate();
	}

	public boolean handleEvent(Event e) {
		if (e.target == exit) {
			if (e.id == Event.ACTION_EVENT) {
				frame.dispose();
				frame = null;
			}
		} else if (e.target == _serviceList) {
			if (e.id == Event.ACTION_EVENT) { // double-click
				int s = _serviceList.getSelectedIndex();

				if (s != -1) {
					new SIServiceUI( _services[s] );
				}
			}
		} else if (e.target == _tsList) {
			if (e.id == Event.ACTION_EVENT) { // double-click
				int s = _tsList.getSelectedIndex();

				if (s != -1) {
					new SITransportStreamUI( _tss[s] );
				}
			}
		} else if (e.target == refresh) {
			if (e.id == Event.ACTION_EVENT) {
				updateAll();
			}
		} else if (e.target == descriptors) {
			if (e.id == Event.ACTION_EVENT) {
				try {
					new DescriptorListUI( frame.getTitle(), _siBouquet.retrieveDescriptors() );
				} catch (SIException ex) {
				}
			}
		}
		return super.handleEvent(e);
	}

	private SIBouquet _siBouquet;
	private SIService _services[];
	private SITransportStream _tss[];
}
