/********************************************************************
 This software module was originally developed by Luca Deri (Finsiel 
- I - SOMMIT Consortium) and edited by Roberto Becchini (SIA - I - 
SOMMIT Consortium), Barbara Silano (SIA - I - SOMMIT Consortium), 
Gianluca De Petris (CSELT - I - SOMMIT Consortium) in the course of 
development of the MPEG-4 Systems (ISO/IEC 14496-1). This software 
module is an implementation of part of one or more MPEG-4 Systems 
(ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 
14496-1). ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1) 
free license to this software module or modifications thereof for use 
in hardware or software products claiming conformance to the MPEG-4 
Systems (ISO/IEC 14496-1). Those intending to use this software 
module in hardware or software products are advised that its use may 
infringe existing patents. THE ORIGINAL DEVELOPER OF THIS SOFTWARE 
MODULE AND HIS/HER COMPANY, THE SUBSEQUENT EDITORS AND THEIR 
COMPANIES, AND ISO/IEC HAVE NO LIABILITY FOR USE OF THIS SOFTWARE 
MODULE OR MODIFICATIONS THEREOF. No license to this software module 
is granted for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
products. The SOMMIT Consortium retains full right to use the 
software module for his/her own purpose, assign or donate the 
software module to a third party and to inhibit third parties from 
using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
products. This copyright notice must be included in all copies or 
derivative works of the software module. Copyright 1999
********************************************************************/
#include "CPUmonitor.h"

/* SOLARIS PORTING */
#ifdef WIN32
#include <windows.h>
#include <stdio.h>
#include <fcntl.h>
#include <io.h>
#include <share.h>
#include <string.h>
#include <time.h>
#include <errno.h>
#include <sys/stat.h>

#define INITIAL_SIZE         51200
#define EXTEND_SIZE          25600
#define REGKEY_PERF          "software\\microsoft\\windows nt\\currentversion\\perflib"
#define REGSUBKEY_COUNTERS   "Counters"
#define PROCESS_COUNTER      "process"

#define WIN_95_PERFKEY           "PerfStats\\StatData"
#define WIN_95_PERFKEY_START     "PerfStats\\StartStat"
#define WIN_95_PERFKEY_STOP      "PerfStats\\StopStat"
#define WIN_95_PERFKEY_START_SRV "PerfStats\\StartSrv"
#define WIN_95_PERFKEY_STOP_SRV  "PerfStats\\StopSrv"
#define WIN_95_CPU_USAGE         "KERNEL\\CPUUsage"


static LONGLONG value, value_old, actTime, prevTime;
static short samplerInitialised = 0;

CHAR szSubKey[1024];
LPBYTE tmpBuf, buf = NULL;
DWORD                        rc;
HKEY                         hKeyNames;
DWORD                        dwType;
DWORD                        dwSize;
LANGID                       lid;
LPSTR                        p;
LPSTR                        p2;
PPERF_DATA_BLOCK             pPerf;
PPERF_OBJECT_TYPE            pObj;
PPERF_INSTANCE_DEFINITION    pInst;
PPERF_COUNTER_BLOCK          pCounter;
PPERF_COUNTER_DEFINITION     pCounterDef;
int                          totalMemory=0, counter=0, actualLoad;
short                        runWin95 = 0;
float secs;
long diffValue, diffSecs;

int readWin95CPULoad(); // Forward Declaration
void manageWin95CPUSampling(int startSampling);

////////////////////////////////////////////////////////////

int initSampler() {

  samplerInitialised = 1;

  /* Look for the list of counters.  Always use the neutral
     English version, regardless of the local language.  We
     are looking for some particular keys, and we are always
     going to do our looking in English.  We are not going
     to show the user the counter names, so there is no need
     to go find the corresponding name in the local language.
     */
  lid = MAKELANGID( LANG_ENGLISH, SUBLANG_NEUTRAL );
  sprintf(szSubKey, "%s\\%03x", REGKEY_PERF, lid);
  rc = RegOpenKeyEx( HKEY_LOCAL_MACHINE,
                     szSubKey,
                     0,
                     KEY_READ,
                     &hKeyNames
		     );
  if (rc != ERROR_SUCCESS)
    {
      RegCloseKey(hKeyNames);
      // printf("GetTaskList(): error (%lu) in RegOpenKeyEx()", rc);
	  runWin95 = 1;
	  manageWin95CPUSampling(1); // Start Win95 Sampling
      return(-1);
    }

  runWin95 = 0;

  /* get the buffer size for the counter names */
  rc = RegQueryValueEx(hKeyNames,
		       REGSUBKEY_COUNTERS,
		       NULL,
		       &dwType,
		       NULL,
		       &dwSize);

  if (rc != ERROR_SUCCESS)
    {
      RegCloseKey(hKeyNames);
      // printf("GetTaskList(): error (%lu) in RegQueryValueEx() - 1", rc);
      return(-1);
    }

  /* allocate the counter names buffer */
  buf = (LPBYTE) malloc(dwSize);
  if (buf == NULL)
    {
      RegCloseKey(hKeyNames);
      return(-1);
    }
  memset(buf, 0, dwSize);

  /* read the counter names from the registry */
  rc = RegQueryValueEx( hKeyNames,
                        REGSUBKEY_COUNTERS,
                        NULL,
                        &dwType,
                        buf,
                        &dwSize
			);

  if (rc != ERROR_SUCCESS)
    {
      free(buf);
      RegCloseKey(hKeyNames);
      // printf("GetTaskList(): error (%lu) in RegQueryValueEx() - 2", rc);
      return(-1);
    }

  p = buf;
  while (*p) {
    if (p > buf) {
      for(p2=p-2; isdigit(*p2); p2--) ;
    }

    //printf("%s\n", p);
    if (stricmp(p, PROCESS_COUNTER) == 0) {
      /* look backwards for the counter number */
      for(p2=p-2; isdigit(*p2); p2--) ;
      strcpy(szSubKey, p2+1);
    }

    p += (strlen(p) + 1);  /* next string */
  }

  free(buf);   /* free the counter names buffer */

  /* allocate the initial buffer for the performance data */
  dwSize = INITIAL_SIZE;
  buf = malloc(dwSize);
  if (buf == NULL)
    {
      RegCloseKey(hKeyNames);
      return(-1);
    }

  memset(buf, 0, dwSize);

  return(0);
}


/* *************************************************** */

void terminateSampler() {
  if (buf)  free(buf);

  if(!runWin95) {
	  RegCloseKey(hKeyNames);
	  RegCloseKey(HKEY_PERFORMANCE_DATA);
  } else
	manageWin95CPUSampling(0); // Stop Win95 Sampling

}

/* *************************************************** */

void manageWin95CPUSampling(int startSampling)
{
  char theBuffer[4096];

  if(startSampling) {
    	rc = RegOpenKeyEx(HKEY_DYN_DATA,
                       WIN_95_PERFKEY_START,
                       0,
                       KEY_READ,
                       &hKeyNames);

  } else {
    	rc = RegOpenKeyEx(HKEY_DYN_DATA,
                       WIN_95_PERFKEY_STOP,
                       0,
                       KEY_READ,
                       &hKeyNames);
  }

  if (rc != ERROR_SUCCESS)
    {
      RegCloseKey(hKeyNames);
      return;
    }

	rc = RegQueryValueEx(hKeyNames,
		 WIN_95_CPU_USAGE,
		 NULL,
		 &dwType,
		 NULL,
		 &dwSize);

	rc = RegQueryValueEx(hKeyNames,
		 WIN_95_CPU_USAGE,
		 NULL,
		 &dwType,
		 theBuffer,
		 &dwSize);

   if (rc != ERROR_SUCCESS)
    {
      RegCloseKey(hKeyNames);
      return;
    }

  RegCloseKey(hKeyNames);
}

/* *************************************************** */

int readWin95CPULoad()
{
  DWORD actLoad;

	//printf("Hello\n");

	rc = RegOpenKeyEx(HKEY_DYN_DATA,
                   WIN_95_PERFKEY,
                   0,
                   KEY_READ,
                   &hKeyNames);

  if (rc != ERROR_SUCCESS)
    {
      RegCloseKey(hKeyNames);
      return(-1);
    }

 	dwType = 3;
    dwSize = 4;

	rc = RegQueryValueEx(hKeyNames,
			 WIN_95_CPU_USAGE,
			 NULL,
			 &dwType,
			 (unsigned char*)&actLoad,
			 &dwSize);

 if (rc != ERROR_SUCCESS)
    {
	  //printf("RegQueryValueEx() error %lu", rc);
      RegCloseKey(hKeyNames);
      return(-1);
    }

  RegCloseKey(hKeyNames);

  //printf("Load: %d\n", (int)actLoad);

  return((int)actLoad);
}

/* *************************************************** */

int readCPULoad()
{
  if(!samplerInitialised)
    initSampler();

  if(runWin95)
	return(readWin95CPULoad());

  while (TRUE) {
    rc = RegQueryValueEx(HKEY_PERFORMANCE_DATA,
			 szSubKey,
			 NULL,
			 &dwType,
			 buf,
			 &dwSize);

    pPerf = (PPERF_DATA_BLOCK) buf;

    /* check for success and valid perf data block signature */
    if ((rc == ERROR_SUCCESS) &&
	(dwSize > 0) &&
	(pPerf)->Signature[0] == (WCHAR)'P' &&
	(pPerf)->Signature[1] == (WCHAR)'E' &&
	(pPerf)->Signature[2] == (WCHAR)'R' &&
	(pPerf)->Signature[3] == (WCHAR)'F' ) {
      break;
    }

    /* if buffer is not big enough, reallocate and try again */
    if (rc == ERROR_MORE_DATA)
      {
	dwSize += EXTEND_SIZE;
	tmpBuf = realloc(buf, dwSize);
	if (tmpBuf == NULL)
	  {
	    free(buf);
	    RegCloseKey(hKeyNames);
	    return(-1);
	  }
	buf = tmpBuf;
	memset(buf, 0, dwSize);
      }
    else
      {
	free(buf);
	RegCloseKey(hKeyNames);
	RegCloseKey(HKEY_PERFORMANCE_DATA);
	// printf("GetTaskList(): error (%lu) in RegQueryValueEx() - HKEY_PERFORMANCE_DATA", rc);
	return(-1);
      }
  }


  /* set the perf_object_type pointer */
  pObj = (PPERF_OBJECT_TYPE) ((DWORD)pPerf + pPerf->HeaderLength);


  pInst = (PPERF_INSTANCE_DEFINITION) ((DWORD)pObj + pObj->DefinitionLength);

  //////////////////////////////////////////////////////////////////

    pCounterDef = (PPERF_COUNTER_DEFINITION) ((DWORD)pObj + pObj->HeaderLength);
    pInst = (PPERF_INSTANCE_DEFINITION) ((DWORD)pObj + pObj->DefinitionLength);

    pCounter = (PPERF_COUNTER_BLOCK) ((DWORD)pInst + pInst->ByteLength);
    value_old = value;

    value = *((LONGLONG UNALIGNED *)((PBYTE)pCounter+4));

    prevTime = actTime;
    actTime  = pObj->PerfTime.LowPart ;
    secs = ((float)(actTime-prevTime))/10000000;

    diffValue = (long)((value-value_old)/10000);
    diffSecs  = (long)((actTime-prevTime)/10000);
    if(diffSecs == 0) diffSecs = 1; /* Avoid divisions by zero */
    actualLoad = 100-(int)(diffValue*100)/diffSecs;

    return(actualLoad);
}

/* ********************************* */

long getTotalSystemMemory() {
  MEMORYSTATUS lpBuffer;

  GlobalMemoryStatus(&lpBuffer);
  return(/*lpBuffer.dwTotalPageFile*/lpBuffer.dwTotalPhys);
}

/* ********************************* */

long getFreeSystemMemory() {
  MEMORYSTATUS lpBuffer;

  GlobalMemoryStatus(&lpBuffer);
  return(/*lpBuffer.dwAvailPageFile*/lpBuffer.dwAvailPhys);
}
#else // WIN32

long readCPULoad()
{
    return 0;
}

long getTotalSystemMemory()
{
    return 1000000;
}

long getFreeSystemMemory()
{
    return 500000;
}

#endif // WIN32


////////////////////////////////////////////////////////////////////
// SOLARIS PORTING - Changed the signature of the method to jclass instead of 
// jobject
JNIEXPORT jshort JNICALL Java_org_iso_mpeg_mpegj_resource_CPUmonitor_getCPUload(JNIEnv *env,
								 jclass obj)
{
	return(readCPULoad());
}

////////////////////////////////////////////////////////////////////

JNIEXPORT jlong JNICALL Java_org_iso_mpeg_mpegj_resource_CPUmonitor_getFreeSystemMemory
(JNIEnv *env, jclass cl) {
  return(getFreeSystemMemory());
}

////////////////////////////////////////////////////////////////////

JNIEXPORT jlong JNICALL Java_org_iso_mpeg_mpegj_resource_CPUmonitor_getAvailSystemMemory
(JNIEnv *env, jclass cl) {
  return(getTotalSystemMemory());
}

////////////////////////////////////////////////////////////////////

int main(int argc, char* argv[]) {
	return((int)getFreeSystemMemory());
}
