/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Multiplexer
FILE:       AAC.cpp
AUTHOR:     Patrizio Bergallo Franco Casalino (CSELT)
OVERVIEW:   Implementation of AACStream, the class that handles AAC
input streams.
-------------------------------------------------------------------------*/
#include "defs.h"
#include "AAC.h"
#include "assert.h"
#include "odMux.h"

/*-----------------------------------------------------------------
AACStream constructor reads first character
-----------------------------------------------------------------*/
AACStream :: AACStream (FILE *fStream, ESDesc *pDescriptor) :
	Stream (fStream, pDescriptor)
{
	int samplingRate;
	m_nCount = 0;
	m_cNext = getc ();
	m_nAU = 0;
  if (pDescriptor -> streamData != (LPCSTR) NULL &&
		  pDescriptor -> streamData [0] != 0)
		samplingRate = atof (pDescriptor -> streamData);
  else
	  samplingRate = 44100; //FC
  m_dt = 1024000.0 / samplingRate;  // AU duration
	

//	stream = fopen ("outmux.txt", "w");

}

/*-----------------------------------------------------------------
For AAC, each AU contains 1024 samples.
The duration of each AU is then related to the audio sampling rate 
of the samples: time increment = (1024/Fs)
Example: at Fs = 44100, time increment is (1024/44100) = 23 ms

-----------------------------------------------------------------*/
long AACStream :: GetDTS ()
{
  if (m_cNext == EOF)
		return -1;
  m_lDTS = m_nAU++ * m_dt;
	m_nCount = 0;
	do
	{
    a[0] = m_cNext;
	  a[1] = getc ();
	  a[2] = getc ();
	  a[3] = getc ();
	  m_cNext = getc ();
	  if (m_cNext == EOF)
		  return -1;
	  m_nFrameSize = ((a[0]<<24) | (a[1]<<16) |(a[2]<<8) | (a[3]));
	}
	while (m_nFrameSize == 0);
	// fprintf(stream, " Dim =  %d \n", m_nFrameSize );
	assert (m_nFrameSize != 0);
  return m_lDTS;
}

/*-----------------------------------------------------------------
GetChar returns a character and reads one char ahead.  The number
of characters in an AU is determined by the data rate, which is
given in the pDescriptor -> streamData field.
-----------------------------------------------------------------*/
int AACStream :: GetChar ()
{
	if (m_nCount >= m_nFrameSize)
		return -1;
	++m_nCount;
	int c = m_cNext;
	m_cNext = getc ();
	assert (c != EOF);
	return c;
}

/*----------------------------------------------------------------
Returns true if the AU referenced by the last call to GetDTS()
was a Random Access Point
-----------------------------------------------------------------*/
bool AACStream :: IsRAP ()
{
	return true;
}

static StreamFactory<AACStream> bifsFactory ("AAC");
