// AACFHGheader.cpp: Implementierung der Klasse AACFHGheader.
//
//////////////////////////////////////////////////////////////////////

#include "defs.h"
#include "HeaderInfo\AACFHGheader.h"

//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////
/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Multiplexer
FILE:       AACFHGheader
AUTHOR:     Christian Koechling (Robert Bosch GmbH)
OVERVIEW:   to extract the additional header info embedded in the stream
-------------------------------------------------------------------------*/
AACFHGheader::AACFHGheader(int numofBits):BitVector(numofBits)
{
	
}

AACFHGheader::~AACFHGheader()
{

}


void AACFHGheader::decodeFHGFrameInfo()
{
  /*
    bits    name                    :       value
    ------------------------------------------------------------------
    16      frame_length    :       frame bytes + header length in Bytes 
    8       ID                      :       0xb7 
    8       header_length   :       11 Bytes (first time),  sonst 6 Bytes
    6       blocks in frame :       0x1
    2       channel_config  :       0x1
    4       sampling frequ. :       index
    1       seq_counter_present     0x0
    1       time_stamp_present      0x0
    1       last_frame_len          0x0
    1       misc_info_present       0x0
    ***** end of standard header ******
    add this only in the first frame:
    4       variable_ID             0x1
    4       variable_length         0x5
    32      variable_bitrate                long value (only integer numbers)
    -------------------------------------------
   = 48 for normal frames == 6Bytes  first frame 11! */

/*
  1.) 2Bits       channel_config  :       0x1  =3D>2Bits fuer die
> NumOfChan., bedeutet das 1=3D1Kanal 2=3DStereo?


0x1 =single channel element = center front speaker = single channel=mono
0x2 =channel pair_element = left, right front speaker = dual channe=
l = 3D stereo
Der Rest (0x3) ist f=FCr Multikanal-Matrizierung reserviert. 

Also die Zuordnung der Samplingfrequenz auf 4 Bit ist:
0x0	96000
0x1	88200
0x2	64000
0x3	48000
0x4	44100
0x5	32000 - wird vom SHARC-AAC Encoder benutzt
0x6	24000
0x7	22050
0x8	16000 - wird vom SHARC-AAC Encoder benutzt
0x9	12000
0xa	11025
0xb	8000	 - wird vom SHARC-AAC Encoder benutzt
0xc - 0xf 	reserviert

  */
  //cout <<"lenOfBytesLevel="<<lenOfBytesLevel << endl;
  if(lenOfBytesLevel == lenOfBytes)
  {
     
	 frame_length = getBits(0, 16);
//	 cout <<"frame_length="<<frame_length << endl;
	 BYTE ID = getBits(16,8);
	 header_length = getBits(24,8);
//	 cout <<"header_length="<<header_length << endl;
	 blocks_in_frame = getBits(32,6);
	 NumOfChan = getBits(38,2);
//     cout <<"NumOfChan="<<NumOfChan << endl;
	 sampling_frequ = SamplingRate[getBits(40,4)];
//	 cout <<"sampling_frequ="<<sampling_frequ<< endl;
	 BitsPerChan = 16;
	/*
	The AAC writes always 2048*NumOfChannels Bytes to the output buffer
	input frame len is variable
	length in ms is 2048
	*/
	pcmoutlen_byte = 1024 * BitsPerChan/8  * NumOfChan;
	pcmoutlen_ms = 1024*1000.0/sampling_frequ;
//	cout <<"pcmoutlen_ms="<<pcmoutlen_ms<< endl;
  }
}