/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Multiplexer
FILE:       AAC.cpp
AUTHOR:     Patrizio Bergallo (CSELT) Franco Casalino (CSELT)
OVERVIEW:   Implementation of AACStream, the class that handles AAC
input streams.
-------------------------------------------------------------------------*/
#include "defs.h"
#include "AACfhg.h"
#include "assert.h"
#include "odMux.h"

/*-----------------------------------------------------------------
AACFHGStream constructor 
-----------------------------------------------------------------*/
AACFHGStream :: AACFHGStream (FILE *fStream, ESDesc *pDescriptor) :
	Stream (fStream, pDescriptor)
{
		m_lTime = 0;
		m_nCount = 0;
		m_lDTS = 0;
	  contauH = 0;
	  contauF = 0;
	  first = 1;
}

/*-----------------------------------------------------------------
GetTimeStamp loads the next frame into the buffer
for the GetChar function.
Time stamps are increments of FHGheader->pcmoutlen_ms
-----------------------------------------------------------------*/
/*-----------------------------------------------------------------
For AAC, each AU contains 1024 samples.
The duration of each AU is then related to the audio sampling rate 
of the samples: time increment = (1024/Fs)
Example: at Fs = 44100, time increment is (1024/44100) = 23 ms

-----------------------------------------------------------------*/

long AACFHGStream::GetDTS()
{
	unsigned int i;

	// read and store the next frame 

	m_nFrameLen = 0;

	for (i = 0 ; i < 6 ; i++)
	{	m_anBuffer[i] = getc();
	   if (m_anBuffer[i] == EOF)
	   {    contauH = 0;
	        contauF = 0;
			return -1;
	   }
	}

	if (m_anBuffer[3] == 11)
	 {  
		for (i = 6 ; i < 11 ; i++)
		{	m_anBuffer[i] = getc();
			if (m_anBuffer[i] == EOF)
			{    contauH = 0;
			     contauF = 0;
				 return -1;
			}
		}

		 // read and decode the header 

		m_pFHGheader = new AACFHGheader(88); 

		for (i = 0 ; i < 11 ; i++)
			m_pFHGheader->setByte(m_anBuffer[i]);

		m_pFHGheader->decodeFHGFrameInfo();

		// read the remaining bytes
		for (i = 0 ; i < (m_pFHGheader->frame_length - 11) ; i++)
				m_anBuffer[i + 11] = getc();

		m_nFrameLen = m_pFHGheader->frame_length;

		if (first)
		{
			m_dt = 1024000.0 / m_pFHGheader->sampling_frequ;
			first =0;
		}

		delete m_pFHGheader;
		m_bRAP = 1;
		contauH++;
	 } 
	 else {
		// read and decode the header
		m_pFHGheader = new AACFHGheader(48);

		for(i = 0 ; i < 6 ; ++i)
			m_pFHGheader->setByte(m_anBuffer[i]);

		m_pFHGheader->decodeFHGFrameInfo(); 
 
		// read the remaining bytes
     
		for (i = 0 ; i < (m_pFHGheader->frame_length - 6) ; i++)
				m_anBuffer[i + 6] = getc();

		m_nFrameLen = m_pFHGheader->frame_length;

		// frame time lenght

		m_nFrameTime = m_pFHGheader->pcmoutlen_ms;

		if (first)
		{
			m_dt = 1024000.0 / m_pFHGheader->sampling_frequ;
			first =0;
		}

		delete m_pFHGheader;
		m_bRAP = 0;
		contauF++;
	 }

	// time stamp

	if (m_bRAP == 0) 
	{
	   // m_lDTS += m_nFrameTime;
       m_lDTS = contauF * m_dt;

	}

	// index of next byte for the GetChar function.
	m_nCount = 0;

   // printf("Number of AU header = %d  Number of AU pay load = %d  Time Stamp = %d \n", contauH, contauF, m_lDTS);

	return m_lDTS;
}

/*-----------------------------------------------------------------
GetChar returns the next byte of the frame (from the loaded buffer)
or -1 if the frame is ended.
-----------------------------------------------------------------*/
int AACFHGStream::GetChar()
{
	return (m_nCount >= m_nFrameLen) ? -1 : m_anBuffer[m_nCount++];
}

/*----------------------------------------------------------------
Returns true if the AU referenced by the last call to GetDTS()
was a Random Access Point
-----------------------------------------------------------------*/
bool AACFHGStream :: IsRAP ()
{
	return m_bRAP;
}

static StreamFactory<AACFHGStream> bifsFactory ("AACfhg");
