/*-------------------------------------------------------------------------
Project Mux
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Mux
FILE:       AbstractSLManager.cpp
AUTHOR:     Alexandre Cotarmanac'h Echevarria
			France Telecom R&D
E-MAIL:		alexandre.cotarmanach@francetelecom.com
OVERVIEW:   Declaration of abstract sl manager.
			Classes implementing this manager set the actual slExtensionParams
			in the packet Header through SetExtensionValues
-------------------------------------------------------------------------*/
#include "defs.h"
#include "AbstractSLManager.h"
#include "SLExtensionPlugin.h"
#include "ztl.h"

static AbstractSLManager * szManager=NULL;

static HINSTANCE   szLibrary		=NULL;


bool CreateSLManager(LPCSTR slManagerName,bool bDebug)
{
	szManager=NULL;
	unsigned int res=false;			//ERROR
// SOLARIS PORTING
#ifdef WIN32
	HINSTANCE instance = LoadLibrary(slManagerName);
#else
	typedef void * (*InstantiationFunction) ();
	HINSTANCE instance = dlopen(slManagerName, RTLD_LAZY);
#endif
	szLibrary = instance;
	if (instance != NULL) 
	{
// SOLARIS PORTING
#ifdef WIN32
		FARPROC proc_address = GetProcAddress(instance, "_Init@4");
#else
	InstantiationFunction proc_address = 
		(InstantiationFunction) dlsym(instance, "CreateInstance");
#endif
		if (proc_address != NULL)
		{
			InitialisePlugin initialise= (InitialisePlugin) proc_address;
			SLManagerShell * tmp = new SLManagerShell();
			tmp->m_plugin.debug  = bDebug;
			szManager=tmp;
			(*initialise)( &tmp->m_plugin );			
			res=true;				//OK
		}
		else
		{
			FreeLibrary(instance);
		}
	}

	return res;
}

AbstractSLManager * GetSLManager()
{
	return szManager;
}

void				CloseSLManager()
{
	if (GetSLManager()) delete GetSLManager();
	if (szLibrary) FreeLibrary(szLibrary);	
}
