/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Mikael Bourges-Sevenier at mikael.bourgessevenier@cnet.francetelecom.fr.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Multiplexer
FILE:       Animation.cpp
AUTHOR:     Mikael Bourges-Sevenier (France Telecom CNET/DIH/ATI)
OVERVIEW:   Implementation of AnimationStream, the class that handles Animation
input streams.
-------------------------------------------------------------------------*/
#include "defs.h"
#include "Animation.h"
#include "odMux.h"

/*-----------------------------------------------------------------
AnimationStream constructor reads first 3 character
-----------------------------------------------------------------*/
AnimationStream :: AnimationStream (FILE *fStream, ESDesc *pDescriptor) :
	Stream (fStream, pDescriptor), m_bRAP (true)
{
  int i;

  m_lDTS=0;
  for(i=24;i>=0;i-=8)
    m_lDTS |= ((long) getc())<<i;
  
  m_lSize=0;
  for(i=24;i>=0;i-=8)
    m_lSize |= ((long) getc())<<i;

  m_lFrames = 0;
  m_bStart = TRUE;
  m_lByteCount=0;

  //fprintf(stderr,"Reading AU %ld DTS %ld size %ld          \n",m_lFrames,m_lDTS,m_lSize);

  // The stream data specifies the frame rate to use (default to 15)
  // should be given in slConfigDescr.compositionUnitDuration
  if (pDescriptor -> streamData == NULL || pDescriptor -> streamData [0] == 0)
    m_fFrameRate = 15;
  else
    {
      m_fFrameRate = atof (pDescriptor -> streamData);
      if (m_fFrameRate < 0.0001)
	throw ExceptionInvalidData (
				    "Animation frame rate must be greater than 0.\n"
				    );
    }
}

/*-----------------------------------------------------------------
Time stamps are calculated according to the rate given by the
pDescriptor -> streamData field
-----------------------------------------------------------------*/
long AnimationStream :: GetDTS (long)
{
  return m_lDTS;
}

/*-------------------------------------------------------------
In this version only the first AU is RAP
-------------------------------------------------------------*/
bool AnimationStream :: IsRAP ()
{
	bool bResult = m_bRAP;
	m_bRAP = false;
	return bResult;
}

/*-----------------------------------------------------------------
GetChar returns a character and reads one next char. We are always
4 chars ahead, which are used to check whether this is end of AU.
-----------------------------------------------------------------*/
int AnimationStream :: GetChar ()
{
  if(m_cNext==EOF) return -1;

  if(m_lByteCount == m_lSize) {
    int i;
    
    m_lDTS=0;
    for(i=24;i>=0;i-=8)
      m_lDTS |= ((long) getc())<<i;
	     
    m_lSize=0;
    for(i=24;i>=0;i-=8)
      m_lSize |= ((long) getc())<<i;
    ++m_lFrames;
    m_lByteCount=0;
    //fprintf(stderr,"Reading AU %ld DTS %ld size %ld          \n",m_lFrames,m_lDTS,m_lSize);
    return -1;
  }

  m_cNext = getc ();
  m_lByteCount++;
  m_bStart = FALSE;
  return m_cNext;
}

static StreamFactory<AnimationStream> bifsFactory ("BIFSANIM");
