// This software module was originally developed by Karl Anders ygard
// (Telenor R&D) in the course of development of the MPEG-4 Systems
// standard (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// For more information or to receive an up to date version of this module,
// mail karlo@nta.no or check <http://televr.fou.telenor.no/compositor/>.
//
// Copyright 1997.

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Multiplexer
FILE:       FBAStream.cpp
AUTHOR:     Karl Anders ygard
OVERVIEW:   Implementation of FBAStream, the class that handles FBA streams
-------------------------------------------------------------------------*/

#include "defs.h"
#include "FBAStream.h"

#define CPLUSPLUS
extern "C" {
  #include "fbadecoder/sim.h"
}

/*-----------------------------------------------------------------
FBAStream constructor reads first 3 character
-----------------------------------------------------------------*/
// FBAStream :: FBAStream (
//    FILE * fStream, LPCSTR szStreamData) :
//	Stream (fStream),
FBAStream :: FBAStream (
    FILE *fStream, ESDesc *pDescriptor) :
	Stream (fStream, pDescriptor),
	m_nBitsLeft(0),
	m_nCurrentBit(0)
{
        size_t length;

	::fseek(fStream, 0L, SEEK_END);
	length = ::ftell(fStream);
	::fseek(fStream, 0L, SEEK_SET);
	
	m_pBuffer = (unsigned char*) ::malloc(length);

	if (m_pBuffer)
	        if (::fread(m_pBuffer, 1, length, fStream) == length)
		        m_nBufferSize = length * 8;
		else
		        printf("couldn't read file in FBAStream :: FBAStream ()\n");

	m_lFrames = 0;
	GetHeader ();

	// The stream data specifies the frame rate to use (default to 15)
	// if (szStreamData == NULL || szStreamData [0] == 0)
	if (pDescriptor -> streamData == (LPCSTR) NULL ||
		pDescriptor -> streamData [0] == 0)
		m_fFrameRate = 30;
	else
	{
		m_fFrameRate = atof (pDescriptor -> streamData);
		// m_fFrameRate = atof (szStreamData);
		if (m_fFrameRate < 0.0001)
			throw ExceptionInvalidData (
				"FBA frame rate must be greater than 0.\n"
			);
	}
}

/*-----------------------------------------------------------------
Get header
-----------------------------------------------------------------*/
void FBAStream :: GetHeader ()
{
  unsigned int PTS = 0;
  FOP_DATA fop_data;
  BOP_DATA bop_data;
  
  if (m_nBufferSize - m_nCurrentBit >= 32) {
    m_nBitsLeft = fba_decoder(
      m_pBuffer + m_nCurrentBit / 8, 
      m_nCurrentBit % 8, 
      (m_nBufferSize - m_nCurrentBit) / 8, 
      &fop_data, 
      &bop_data, 
      sizeof(FOP_DATA) + sizeof(BOP_DATA), 
      &PTS);
    //m_nBufferSize -= m_nBitsLeft;
  }
}

/*-----------------------------------------------------------------
Time stamps are calculated according to the rate given by the
szStreamData field
-----------------------------------------------------------------*/
long FBAStream :: GetDTS ()
{
	if (m_nBitsLeft <= 0)
		return -1;

	long lTime = (long) ((float) (m_lFrames * 1000) / m_fFrameRate);
	++m_lFrames;
	return lTime;
}

/*-----------------------------------------------------------------
GetBits returns the requested bits from the bitstream.
-----------------------------------------------------------------*/
inline int FBAStream :: GetBits (int nBits)
{
  int nResult = 0;

  for (; nBits > 0; --nBits) {
    nResult <<= 1;
    if (m_nBitsLeft > 0) {
      nResult |= (m_pBuffer[m_nCurrentBit / 8] >> (7 - (m_nCurrentBit % 8))) & 0x1;
      --m_nBitsLeft;
      ++m_nCurrentBit;
    }
#if 1 // hack to get FBA streams to work 
    else
      if (m_nBitsLeft > -16) {
	nResult |= (m_pBuffer[(m_nCurrentBit - m_nBitsLeft)/ 8] >> (7 - ((m_nCurrentBit - m_nBitsLeft) % 8))) & 0x1;
	--m_nBitsLeft;
      }
#endif
  }

  return nResult;
}

/*-----------------------------------------------------------------
GetChar returns a character and reads one next char.
-----------------------------------------------------------------*/
int FBAStream :: GetChar ()
{
  if (m_nBitsLeft <= -16) {
  // if (m_nBitsLeft <= 0) {
    GetHeader ();
    return -1;
  } 
  else
  {
    return GetBits(8);
  }
}

// SB 98/03/18
/* virtual */ 
long FBAStream::GetCTS()
{
  return 0;
}

/* virtual */ 
bool FBAStream::IsRAP()
{
  return FALSE;
}

static StreamFactory<FBAStream> bifsFactory ("FBA");
