/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Multiplexer
FILE:       H263.cpp
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Implementation of H263Stream, the class that handles H263
input streams.
-------------------------------------------------------------------------*/
#include "defs.h"
#include "H263.h"
#include "odMux.h"

/*-----------------------------------------------------------------
H263Stream constructor reads first 3 character
-----------------------------------------------------------------*/
H263Stream :: H263Stream (FILE *fStream, ESDesc *pDescriptor) :
	Stream (fStream, pDescriptor)
{
	for (int i = 0; i < 5; i++)
		m_cNext [i] = getc ();
	m_lFrames = 0;
	m_bStart = TRUE;

	// The stream data specifies the frame rate to use (default to 15)
	if (pDescriptor -> streamData == (LPCSTR) NULL ||
		pDescriptor -> streamData [0] == 0)
		m_fFrameRate = 15;
	else
	{
		m_fFrameRate = atof (pDescriptor -> streamData);
		if (m_fFrameRate < 0.0001)
			throw ExceptionInvalidData (
				"H263 frame rate must be greater than 0.\n"
			);
	}
}

/*-----------------------------------------------------------------
Time stamps are calculated according to the rate given by the
pDescriptor -> streamData field
-----------------------------------------------------------------*/
long H263Stream :: GetDTS (long)
{
	if (m_cNext [0] == EOF)
		return -1;
	m_lDTS = (long) ((float) (m_lFrames * 1000) / m_fFrameRate);
	++m_lFrames;
	m_bRAP = !(m_cNext [4] & 2);
	return m_lDTS;
}

/*-----------------------------------------------------------------
GetChar returns a character and reads one next char. We are always
3 chars ahead, which are used to check whether this is end of AU.
-----------------------------------------------------------------*/
int H263Stream :: GetChar ()
{
	if (!m_bStart &&
		(	m_cNext [0] == EOF ||
			m_cNext [0] == 0 && m_cNext [1] == 0 &&
			(m_cNext [2] & 0x8c) == 0x80
		)
	)
	{	m_bStart = TRUE;
		return -1;
	}
	int c = m_cNext [0];
	for (int i = 0; i < 4; i++)
		m_cNext [i] = m_cNext [i + 1];
	m_cNext [4] = getc ();
	m_bStart = FALSE;
	return c;
}

static StreamFactory<H263Stream> h263Factory ("H263");