/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Multiplexer
FILE:       Bitvector.cpp
AUTHOR:     Christian Koechling (Robert Bosch GmbH)
OVERVIEW:   to extract the additional header info embedded in the stream
            helper class
-------------------------------------------------------------------------*/

#include "defs.h"
#include <assert.h>
#include "BitVector.h"

#include <iostream.h>
#include <stdio.h>


BitVector::BitVector(int numofBits)
{
   
    int i;

    lenOfBits = numofBits;
    lenOfBytesLevel = 0;
    lenOfBytes = numofBits/8;
    if(numofBits%8 > 0)
        ++lenOfBytes;

    //cout << "num of Bits : "<< numofBits<<endl;
	//cout << "array len : "<< lenOfBytes<<endl;
    array = new BYTE [lenOfBytes];
    for(i = 0; i< lenOfBytes; ++i)
        array[i] = 0x0;
    
}

BitVector::~BitVector()
{
    delete(array);
}

int BitVector::getlen_bytes()
{
	return lenOfBytes;
}
int BitVector::getlen_bits()
{
	return  lenOfBits;
}
int BitVector::getlen_BytesLevel()
{
	return lenOfBytesLevel;
}
int BitVector::getBit(int offset)
{
  int byteNum;
  int byteoffset;
  unsigned int value;
  
  
  //which byte contains the demanded bit
  byteNum = offset/8;
  byteoffset = offset%8;
  if(byteoffset >0)
      assert(byteNum+1 <= lenOfBytes);
  else
      assert(byteNum <= lenOfBytes);  
  value =(int) array[byteNum];
  value = (value & hexval[byteoffset]) >> offset;
 
  
 
  return value; //which is 0 or 1;
  
}

int BitVector::getBits(int len)
{
	return getBits(0, len);
}

// reads high byte first then lowbyte
// possible to make it more generic afterwards
// offset in and len in bytes not in Bits!
int BitVector::getByteshl(int byteNum, int len)
{
  unsigned int value;
  if(len == 1)
  {
    value = (unsigned int) array[byteNum];
  }
  else if(len == 2)
  {
    value = ((array[byteNum]) <<8) | (array[byteNum+1]); 
  }
  else if(len == 4)
  {
    value = ((array[byteNum]) <<24) |((array[byteNum+1]) <<16) |((array[byteNum+2]) <<8) |(array[byteNum+3]);
  }
  else
  {
      assert(0>1); //not yet implemented thus error message   
  }

  return value;


}
// reads high byte first then lowbyte
// possible to make it more generic afterwards
// offset in and len in bytes not in Bits!
int BitVector::getByteslh(int byteNum, int len)
{

  unsigned int value;
  if(len == 1)
  {
    value = (unsigned int) array[byteNum];
  }
  else if(len == 2)
  {
    value = ((array[byteNum+1]) <<8) | (array[byteNum]); 
  }
  else if(len == 4)
  {
    value = ((array[byteNum+3]) <<24) |((array[byteNum+2]) <<16) |((array[byteNum+1]) <<8) |(array[byteNum]);
  }
  else
  {
      assert(0>1); //not yet implemented thus error message   
  }

   return value;
}
int BitVector::getBits(int offset, int len)
{
  int byteNum;
  int leninByte;
  int byteoffset;
  unsigned int value;
  int hexval;

  
  byteNum = offset/8;
  byteoffset = offset%8;
  
  //assert(len<=48); //only 2 Bytes allowed here!
				   // please extend it to your demand
  assert(len>0);

  
  leninByte = len/8;
  if(len%8 >0) 
    ++leninByte;

  
  //info in first Byte only
  if(len<=8)
  {
    value = (unsigned int) array[byteNum];
	
    value = value & hexframes[len*8-byteoffset-len]; 
	value = value >> (8-(byteoffset+len));
  }
  else if(len == 16)
  {
    value = (array[byteNum]) <<8;
    value = value | array[byteNum+1]; 
  }
  else if(len == 32)
  {
    value = ((array[byteNum]) <<24) |((array[byteNum+1]) <<16) |((array[byteNum+2]) <<8) |(array[byteNum+3]);
  }
  else
  {
      assert(0>1); //not yet implemented thus error message   
  }


  return value;
      

}
//  1 : took a byte
//  0 : did not take the byte
// -1 :	load level reached - byte taken
int BitVector::setByte(BYTE abyte)
{
  //cout << "in SetByte"<< endl;
  //cout << "lenOfBytesLevel ="<<lenOfBytesLevel<< endl;
  if(lenOfBytesLevel+1<lenOfBytes)
  {
    array[lenOfBytesLevel++] = abyte;
	//cout << "abyte= "<<abyte<<endl;
	//printf("hex abyte = %x\n",abyte);
	//printf("Dec abyte = %i\n",abyte);
	
  }
  else if (lenOfBytesLevel+1 == lenOfBytes)
  {
    array[lenOfBytesLevel++] = abyte;
    return -1;
  }
  else
	return 0;
  return 1;
}

void BitVector::resetCounter()
{
    lenOfBytesLevel = 0;

}

void BitVector::resetAll()
{
  lenOfBytesLevel = 0;
  for(int i = 0; i< lenOfBytes; ++i)
        array[i] = 0x0; 
}

