/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Multiplexer
FILE:       TSBifs.cpp
AUTHOR:     David
OVERVIEW:   Implementation of IPMPToolStream, the class that handles 
			time stamped BIFS streams.
-------------------------------------------------------------------------*/
#include "defs.h"
#include "IPMPToolStream.h"
#include "assert.h"
#include "odMux.h"

/*-----------------------------------------------------------------
IPMP tools are not timestamped.
-----------------------------------------------------------------*/
IPMPToolStream :: IPMPToolStream (FILE *fStream, ESDesc *pDescriptor) :
	Stream (fStream, pDescriptor), m_bRAP (true), ip(fStream)
{
		fseek(fStream, 0, SEEK_END);
		filesize.longsize = ftell(fStream);
		fseek(fStream, 0, SEEK_SET);
		hdrIndex = 0;
}


long IPMPToolStream :: GetTimeStamp ()
{
	return feof(ip)? -1 : 0;
}

long IPMPToolStream :: GetDTS (long)
{
	return GetCTS();
}

long IPMPToolStream :: GetCTS ()
{
	return feof(ip)? -1 : 0;
}

/*-------------------------------------------------------------
In this version only the first AU is RAP
-------------------------------------------------------------*/
bool IPMPToolStream :: IsRAP ()
{
	bool bResult = m_bRAP;
	m_bRAP = false;
	return bResult;
}


/*-----------------------------------------------------------------
GetChar returns a character and read one char ahead
-----------------------------------------------------------------*/
int IPMPToolStream :: GetChar ()
{
	size_t eof;
	unsigned char c;
	switch (hdrIndex) {
		case 0:
			//hdrIndex++;
			//return 0x7f;	//no signature, reserved = 0x7f
		case 1:
		case 2:
		case 3:
		case 4:
			//c = filesize.size[4-hdrIndex];	//[hdrIndex - 1];
			//hdrIndex++;
			//return c;
		default:
			//return getc();
			eof = fread(&c, 1, 1, ip);
			return eof==0? -1:c;
	}
}



static StreamFactory<IPMPToolStream> bifsFactory ("IPMPToolES");
//static StreamFactory<IPMPToolStream> odFactory ("IPMPTool");