/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

#include "defs.h"
#include "Mpg4Vid.h"
#include "odMux.h"

/*-----------------------------------------------------------------
Mpg4VidStream constructor reads first 3 character
-----------------------------------------------------------------*/
Mpg4VidStream :: Mpg4VidStream (FILE *fStream, ESDesc *pDescriptor) :
	Stream (fStream, pDescriptor), m_bRAP (true)
{
	m_cNext [0] = getc ();
	m_cNext [1] = getc ();
	m_cNext [2] = getc ();
	m_cNext [3] = getc ();
	m_lFrames = 0;
	m_firstStartCode = TRUE;
	m_returnedStartCodeLastCall = FALSE;

	// The stream data specifies the frame rate to use (default to 15)
	if ((pDescriptor -> streamData == NULL) || (pDescriptor -> streamData[0] == 0))
		m_frameRate = 15;
	else
	{
		m_frameRate = atof(pDescriptor -> streamData);
		if (m_frameRate < 0.0001)
			throw ExceptionInvalidData (
				"The H263 frame rate must be greater than 0.\n"
			);
	}
}

/*-----------------------------------------------------------------
Time stamps are calculated according to the rate given by the
pDescriptor -> streamData field
-----------------------------------------------------------------*/
long Mpg4VidStream :: GetDTS (long)
{
	if (m_cNext [0] == EOF)
		return -1;
	m_lDTS = (long)((float)(m_lFrames * 1000) / m_frameRate);
	++m_lFrames;
	return m_lDTS;
}

/*-------------------------------------------------------------
In this version only the first AU is RAP
-------------------------------------------------------------*/
bool Mpg4VidStream :: IsRAP ()
{
	bool bResult = m_bRAP;
	m_bRAP = false;
	return bResult;
}

/*-----------------------------------------------------------------
GetChar returns a character and reads one next char. We are always
4 chars ahead, which are used to check whether this is end of AU.
-----------------------------------------------------------------*/
int Mpg4VidStream :: GetChar ()
{
	if (m_cNext[0] == EOF)
		return -1;

	if (!m_returnedStartCodeLastCall)
	{
		if ((m_cNext[0] == 0) && (m_cNext[1] == 0) &&
			(m_cNext[2] == 1) && (m_cNext[3] == 0x73))
		{
			// Ignore the first start code
			if (m_firstStartCode)
				m_firstStartCode = FALSE;
			else
			{
				m_returnedStartCodeLastCall = TRUE;
				return -1;
			}
		}
	}

	int c = m_cNext [0];
	m_cNext [0] = m_cNext [1];
	m_cNext [1] = m_cNext [2];
	m_cNext [2] = m_cNext [3];
	m_cNext [3] = getc ();
	m_returnedStartCodeLastCall = FALSE;

	return c;
}

#ifndef _SAIT_3DTG_MP4VIDEO_
static StreamFactory<Mpg4VidStream> bifsFactory ("MPEG4-Video");
#endif _SAIT_3DTG_MP4VIDEO_