/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Multiplexer
FILE:       Mux.cpp
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.) 
						Muxcode implementation by Carsten Herpel (THOMSON multimedia)
OVERVIEW:   Implementation of Mux, the object that performs the multiplexing
-------------------------------------------------------------------------*/

#include "defs.h"
#include "Mux.h"
#include "odMux.h"
#include "Token.h"
#include "MuxODEnc.h"
//AC:20021009 - SL extension : include  -- begin
#include "AbstractSLManager.h"
//AC:20021009 - SL extension : include  -- end

void Mux :: Start (FILE *fList, int flags, LPCSTR szRootFile,LPCSTR slManager)
{
	MuxODEncoder odEncoder (m_fTRIF, fList, "TRIF", flags);
	SDLClassArray<ObjectDescriptor, 1, 255> ods;
	//AC:20021009 - SL extension : CreateSLManager  -- begin
	bool bDebug = (flags & (TRACE_VERBOSE | TRACE_HEADERS))!=0;
	CreateSLManager(slManager,bDebug   /*"DummySLManager.dll"*/);
	//AC:20021009 - SL extension : CreateSLManager  -- end
	/*-------------------------------------------------------------
	SLHeaders are traced if flags contains TRACE_HEADERS
	-------------------------------------------------------------*/
	if (flags & TRACE_HEADERS)
		flags = TRACE_VERBOSE;
	else
		flags = TRACE_NONE;

	/*-------------------------------------------------------------
	ES_Descriptor is overloaded with a special version used by the Mux
	-------------------------------------------------------------*/		
	OVERLOAD_OD_CLASS (ES_DescriptorMux);

	try
	{
		/*-------------------------------------------------------------
		Parse the initial object descritpor, if exists, and encode
		it at the beginning of the file.
		-------------------------------------------------------------*/
		if (stricmp (nextToken.GetString (), "InitialObjectDescriptor") == 0)
 		//{	SDLClass<InitialObjectDescriptor> od;
 		{	SDLClass<InitialObjectDescriptor, 1> od; // SOLARIS PORTING
			od.Scan ();
			if (od != NULL_DESC)
			{	ods += od;
				od -> EncDec (&odEncoder, CALC_LENGTH, NULL);
				od -> EncDec (&odEncoder, ENCODE, NULL);
				odEncoder.Align ();
				od = NULL_DESC;
			}
		}
		odEncoder.Write (true);

		/*-----------------------------------------------------------------
		Parse the rest of the object descriptors
		-----------------------------------------------------------------*/
		while (nextToken != TID_EOF)
		{
 			// SOLARIS PORTING - adding args
 			//SDLClass<ObjectDescriptor> od;
 			SDLClass<ObjectDescriptor, 1> od;
			od.Scan ();
			if (od != NULL_DESC)
			{	ods += od;
				od = NULL_DESC;
			}
		}
	}
	catch (...)
	{
		printf ("Script file error. Hit enter to continue\x7\n");
		getchar ();
		return;
	}
	//AC:20021009 - SL extension : init with ods  -- begin
	GetSLManager()->Init(ods);
	//AC:20021009 - SL extension : init with ods  -- end
	/*-----------------------------------------------------------------
	Traverse the esDescrs list assign a flexmux code (FMC) to
	each descriptor which points to a file that shall be multiplexed, and
	open that file. Other esDescrs are deleted.
	-----------------------------------------------------------------*/
	int i, j;
	ZArray<int> esIDs;
	BOOL bError = FALSE;
	MuxCodeTable *pMuxCodeTable = NULL;

	for (i = 0; i < ods.GetLength ();)
	{	for (j = 0; j < ods [i] -> esDescr.GetLength ();)
			/*-----------------------------------------------------------------
			The function OpenFile() assing an FMC to an esDescr,
			and opens the associated file. It returns 1 if sucessful,
			0 if the descriptor is not assicated with a file, and -1
			if could not open the file
			-----------------------------------------------------------------*/
			switch (
				((ES_DescriptorMux *) ods [i] -> esDescr [j]) ->
					OpenFile (esIDs.GetLength (), m_fTRIF, fList, flags, szRootFile)
			) {
			case 1:
				esIDs += ods [i] -> esDescr [j] -> ES_ID;
				if (ods [i] -> esDescr [j] -> muxCodeTable != NULL) {
					if (pMuxCodeTable) {
						printf("Only one mux code table in mux script file allowed!\n");
						bError = TRUE;
					}
					else {
						pMuxCodeTable = ods [i] -> esDescr [j] -> muxCodeTable;
					}
				}
				m_streamTable += (ES_DescriptorMux *) ods [i] -> esDescr [j];
				j++;
				break;
			case -1:
				bError = TRUE;
			case 0:
				ods [i] -> esDescr.DeleteFrom (j);
			}

		if (ods [i] -> esDescr.GetLength () == 0)
			ods.DeleteFrom (i);
		else
			i++;
	}

	/*-----------------------------------------------------------------
	Terminate here if had an error while opening the files
	-----------------------------------------------------------------*/
	if (bError)
	{	printf ("Hit Enter to continue\x07\n");
		fputc (0, m_fTRIF);
		getchar ();
		return;
	}

	/*-----------------------------------------------------------------
	Write the stream map table
	-----------------------------------------------------------------*/
	fputc (esIDs.GetLength (), m_fTRIF);
	for (i = 0; i < esIDs.GetLength (); i++)
	{	fputc (HIBYTE (esIDs [i]), m_fTRIF);
		fputc (LOBYTE (esIDs [i]), m_fTRIF);
	}

	
	/*-----------------------------------------------------------------
	Prepare the muxCode table
	-----------------------------------------------------------------*/
	if (pMuxCodeTable) {
		bError = PrepareMuxCodeTable((MuxCodeTable &)*pMuxCodeTable);
		if (bError)
		{	printf ("Hit Enter to continue\x07\n");
			getchar ();
			return;
		}
	}
	/*-----------------------------------------------------------------
	Multiplexing is done by finding the AU with the lowest time stamp and 
	appending it to the muxed file. We iterate till all files reach EOF
	-----------------------------------------------------------------*/
	while (ods.GetLength () > 0)
	{
		ES_DescriptorMux *pDesc = NULL;
		long lLowestTime = INT_MAX;

		for (i = 0; i < ods.GetLength (); i++)
		{	for (j = 0; j < ods [i] -> esDescr.GetLength (); j++)
			{	ES_DescriptorMux *p =
					(ES_DescriptorMux *) ods [i] -> esDescr [j];
				// Guido 4/8/00 Take into account scale in Hint files
				// Normalize the LTS value in ms
				long lTS = p -> GetDTS () * 1000 / (float)(p -> GetScale ());
				/********** Changed by Liao Xuan,UK NB Comments *********/
				/* Mux goes into an endless loop						*/
				//if (lTS == -1)				// means EOF
				if (p -> GetDTS () == -1)		// means EOF
				/********** End:Changed by Liao Xuan,UK NB Comments *****/
				{	
					ods [i] -> esDescr.DeleteFrom (j);
					break;
				}
				lTS += p -> GetStartTime ();
				if (lTS < lLowestTime)
				{	pDesc = p;
					lLowestTime = lTS;
				}
			}

			if (ods [i] -> esDescr.GetLength () == 0)
			{	ods.DeleteFrom (i);
				break;
			}
			if (pDesc == NULL)
				break;
		}

		/*-----------------------------------------------------------------
		Store one AU of the proper stream (unless one of the streams
		reached EOF)
		Store one AU of each muxCoded stream in case of MuxCode IF they are
		close in time.
		If no valid MuxCodeTableElement available : use simple mode.
		(Note that a given stream/FMC may use both MuxCode and simple mode FlexMux)
		-----------------------------------------------------------------*/
		if (pDesc != NULL) {
			int muxCode = (int)pDesc -> muxInfo ->muxCode;
			int maxDeltaT = (int)pDesc -> muxInfo ->maxDeltaT;

			if (muxCode == -1) {
				pDesc -> Store (false);
			}
			else
			{
				if (m_muxCodeTable.GetLength() > muxCode) // muxcode available
				{
					LinearMuxCodeTableElement &mcte = m_muxCodeTable[muxCode];
					BOOL bError = false;
					/* ---------------------------------------------------------------
					Check whether all relevant streams are present & have AUs 
					that are close in time
					----------------------------------------------------------------*/
					for (int i = 0; i < mcte.slots.GetLength(); i++)
					{
						int fmc = mcte.slots[i].flexMuxChannel;
						if (fmc >= m_streamTable.GetLength()) { // stream not present
							bError = true;
							break;
						}
						ES_DescriptorMux *p = m_streamTable[fmc];
						// Guido 4/8/00 Take into account scale in Hint files
						// Normalize the lTS value in ms
						long lTS = p -> GetDTS () * 1000 / (float)(p -> GetScale ());
						/********** Changed by Liao Xuan,UK NB Comments *********/
						/* Mux goes into an endless loop						*/
						//if (lTS == -1)				// means EOF
						if (p -> GetDTS () == -1)		// means EOF
						/********** End:Changed by Liao Xuan,UK NB Comments *****/
						{
							bError = true;
							break;
						}
						lTS += p -> GetStartTime ();
						if (lTS > lLowestTime + maxDeltaT) {
							bError = true;
							break;
						}
					}
					if (bError) { // muxcode does not work
						pDesc -> Store (false);
					}
					else // use the muxcode
					{
						BYTE packet[258];
						int nLength = 3;
						BOOL bRemain = false;
						BOOL bDone = false;
						packet[0] = muxCode + 240;
						packet[2] = (mcte.version<<4)|0x0F;

						do		// iterate until one AU of each stream is sent
						{
							int fmc = -1;
							int slotSize = -1;
							int nActBytes = 0;
// Guido Franceschini
// Lines marked GF 23/8/99 have the goal of avoiding the definition and usage of
// the IsEndOfAU() method, which was introduced with core3.2, and caused changes in
// the mux plug-ins. These changes can now be undone.
							int nCompletedAU = -1;	// GF 23/8/99
							ES_DescriptorMux *p = NULL;
							bRemain = false;
							for (int i = 0; i < mcte.slots.GetLength(); i++)
							{
								fmc = mcte.slots[i].flexMuxChannel;
								slotSize = mcte.slots[i].numberOfBytes;
								p = m_streamTable[fmc];
//								bDone = (slotSize > 258-nLength) || p -> isEndOfAU();	// GF 23/8/99
								bDone = (slotSize > 258-nLength) || (i == nCompletedAU);// GF 23/8/99
								nActBytes = 0;
						
								if (bDone)
									break;

								BOOL bDoneAU = p -> StoreSlot(&packet[nLength], slotSize,nActBytes);
								bRemain |= !bDoneAU;
								if (bDoneAU && nCompletedAU == -1)	// GF 23/8/99
									nCompletedAU = i;				// GF 23/8/99
								if (nActBytes>0) {
									nLength += nActBytes;
								}
								if (slotSize != nActBytes)
								{
									nActBytes = 0;
									bDone = true;
									break;
								}
							}
							if (bDone || !bRemain) {
								if (nLength > 3) { // put length & send muxcode packet
									packet[1] = nLength - 3;
									fwrite (packet, nLength, 1, m_fTRIF);
								}
								if (nActBytes>0 && slotSize != nActBytes) { // send remainder of current stream p
									packet[nLength-2] = fmc;
									packet[nLength-1] = nActBytes;
									fwrite (&packet[nLength-2], nActBytes+2, 1, m_fTRIF);
								}
								for (int j = 0; j < mcte.slots.GetLength(); j++)
								{	// and remainder of other access units, if any
									int fmc = mcte.slots[j].flexMuxChannel;
									ES_DescriptorMux *p = m_streamTable[fmc];
									p -> Store(true);
								}
								if (bDone)
									bRemain = false;
							}
						} while (bRemain);
					}
				}
			}
		}
	}

	//AC:20021009 - SL extension : CloseSLManager  -- begin
	CloseSLManager();
	//AC:20021009 - SL extension : CloseSLManager  -- end
}

/*-------------------------------------------------------------------------
Convert the recursive mux code table entries into linear lists of slots
------------------------------------------------------------------------- */

BOOL Mux :: PrepareMuxCodeTable(MuxCodeTable &muxCodeTable)
{
	FILE *fMct = NULL;

	if ((fMct = fopen (m_stMctFileName, "w")) == NULL)
	{	printf ("Cannot open %s\n", (LPCSTR) m_stMctFileName);
		return -1;
	}

	fprintf(fMct," %d //elementCount [0..16]\n",muxCodeTable.elements.GetLength());
	for (int i = 0; i < muxCodeTable.elements.GetLength(); i++)
	{
		int muxCode = (int)muxCodeTable.elements[i]->muxCode;
		int nSubStrucLength = (int)muxCodeTable.elements[i]->slots.GetLength();
		int nSubStrucCount = (int)muxCodeTable.elements[i]->subStrucCount;
		LinearMuxCodeTableElement &pMcte = m_muxCodeTable.ElementAt(muxCode);

		pMcte.version = (int)muxCodeTable.elements[i]->version;

		if (nSubStrucLength != nSubStrucCount) {
			printf ("MuxCodeTable substructureCount mismatch value=%d != actual=%d\n",nSubStrucCount,nSubStrucLength);
			return -1;
		}

		fprintf(fMct," %d //length [0..255]\n",(int)muxCodeTable.elements[i]->length);
		fprintf(fMct," %d //muxCode [0..15]\n",muxCode);
		fprintf(fMct," %d //version [0..15]\n",(int)muxCodeTable.elements[i]->version);
		fprintf(fMct," %d //substructureCount [1..255]\n",nSubStrucCount);

		for (int iSub = 0; iSub <  nSubStrucCount; iSub++)
		{
			int nSlotCount = (int)muxCodeTable.elements[i]->slots[iSub]->slotCount;
			int nSlotLen1 =  muxCodeTable.elements[i]->slots[iSub]->flexMuxChannel.GetLength();
			int nSlotLen2 = muxCodeTable.elements[i]->slots[iSub]->numberOfBytes.GetLength();
			int nRepetitionCount = (int)muxCodeTable.elements[i]->slots[iSub]->repetitionCount;

			if (nSlotCount != nSlotLen1 || nSlotCount != nSlotLen2) {
				printf ("MuxCodeTable slotCount mismatch value=%d != #fmc=%d #bytes=%d\n",nSlotCount,nSlotLen1,nSlotLen2);
				return -1;
			}

			fprintf(fMct," %d //slotCount [1..31]\n",nSlotCount);
			fprintf(fMct," %d //repetitionCount [1..7]\n",nRepetitionCount);

			for (int iRep = 0; iRep < nRepetitionCount; iRep++)
			{
				BOOL bErr = false;
				for (int iSlot = 0; iSlot < nSlotCount; iSlot++)
				{
					McteSlot aSlot;
					aSlot.numberOfBytes = (int)muxCodeTable.elements[i]->slots[iSub]->numberOfBytes[iSlot];
					int Esid = (int)muxCodeTable.elements[i]->slots[iSub]->flexMuxChannel[iSlot];
					aSlot.flexMuxChannel = FindFmcForEsid(Esid);

					if (aSlot.flexMuxChannel < 0) {
						printf ("MuxCodeTable invalid ES_ID %d\n",Esid);
						bErr = 1;
					}

					pMcte.slots += aSlot;
					if (iRep == 0)
					{
						fprintf(fMct," %d //fmxChannel [1..255]\n",aSlot.flexMuxChannel);
						fprintf(fMct," %d //numberOfBytes [1..255]\n",aSlot.numberOfBytes);
					}
				}
				if (bErr)
						return -1;
			}
		}
	}
	fclose(fMct);
	return 0;
}

int Mux :: FindFmcForEsid(int Esid)
{
	for (int i = 0; i < m_streamTable.GetLength(); i++) {
		if (m_streamTable[i] -> ES_ID == Esid)
			return i;
	}
	return -1;
}
