/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

#if !defined(_Mux_h)	// Sentry, use file only if it's not already included.
#define _Mux_h

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Multiplexer
FILE:       Mux.h
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Declaration of Mux, the object that performs the multiplexing
-------------------------------------------------------------------------*/
#include "ztl.h"
#include "ODP.h"
#include "odMux.h"

struct McteSlot
{
	int flexMuxChannel;
	int numberOfBytes;
public:
	McteSlot() : flexMuxChannel(-1), numberOfBytes(-1)
	{}
};
typedef ZArray<McteSlot> McteSlotTable;

struct LinearMuxCodeTableElement
{
	int version;
	McteSlotTable slots;
public:
	LinearMuxCodeTableElement() : version(-1), slots()
	{}
};
typedef	ZArray<LinearMuxCodeTableElement> LinearMuxCodeTable;

typedef ZArray<ES_DescriptorMux*> StreamTable;


class Mux
{
	FILE *m_fTRIF, *m_fMct;
	LinearMuxCodeTable m_muxCodeTable;
	StreamTable m_streamTable;
	ZString m_stMctFileName;

public:
	Mux (FILE *fTRIF, LPCSTR stMctFileName) : m_fTRIF (fTRIF), m_stMctFileName (stMctFileName) {}
//AC:20021009 - SL extension -- begin
	void Start (FILE *fList, int flags, LPCSTR szRootFile,LPCSTR slManager);
//AC:20021009 - SL extension -- end
private:
	BOOL PrepareMuxCodeTable(MuxCodeTable &muxCodeTable);
	int FindFmcForEsid(int Esid);
};
	
#endif                                      // _Mux_h sentry.
