/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

#if !defined(_MuxStream_h)  // Sentry, use file only if it's not already included.
#define _MuxStream_h

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Multiplexer
FILE:       MuxStream.h
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Declaration of Stream, the base class for objects that handles
input streams.
-------------------------------------------------------------------------*/
#include "Factory.h"

#undef getc

/*-------------------------------------------------------------
Structure that contains info about the stream
-------------------------------------------------------------*/
struct ESDesc
{
	double startTime;
	LPCSTR fileName;
	LPCSTR streamType;
	LPCSTR streamData;
	int repeatRate;
	int sessionLength;
	int maxAhead;
};	

/*-------------------------------------------------------------
The Stream object
-------------------------------------------------------------*/
class Stream
{
	FILE *m_fStream;
	ESDesc *m_pDescriptor;

public:
	Stream (FILE *fStream, ESDesc *pDescriptor) :
		m_fStream (fStream), m_pDescriptor (pDescriptor)
	{}
	virtual ~Stream () {delete m_pDescriptor;}
	virtual int GetScale () { return 1000; }	// Guido 4/8/00
	virtual long GetDTS (long) { return GetDTS(); }
	virtual long GetDTS () { return 0; }
	virtual long GetCTS () = 0;
	virtual int GetChar () = 0;
	virtual int GetPaddingBits () {return 0;}
	virtual bool IsRAP () = 0;
	virtual bool IsRepeated () {return false;}

protected:
	int getc () {return fgetc (m_fStream);}
	int seek (long offset) { return fseek (m_fStream, offset, SEEK_SET); }
	ESDesc *GetDescriptor () {return m_pDescriptor;}
	FILE *GetFile () {return m_fStream;}
};

/*-----------------------------------------------------------------
Definiton of an execption for the case streamData is invalid
-----------------------------------------------------------------*/
struct ExceptionInvalidData
{
	LPCSTR m_szError;
	ExceptionInvalidData (LPCSTR szError) : m_szError (szError) {}
};

#endif	// _MuxStream_h sentry
// SOLARIS PORTING - Added a new line
