/*-------------------------------------------------------------------------
Project Mux
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Mux
FILE:       SLDummyPlugin.cpp
AUTHOR:     Alexandre Cotarmanac'h Echevarria
			France Telecom R&D
E-MAIL:		alexandre.cotarmanach@francetelecom.com
OVERVIEW:   Declaration of an SL Manager plugin.
-------------------------------------------------------------------------*/
#define PLUGIN_EXPORTS

#include "../SLExtensionPlugin.h"
#include "../odMux.h"
#include "../Core/odtags.h"

#include "SLDummyPlugin.h"

#ifdef _DEBUG
#include<stdio.h>
#define LOG(X) printf X;
#define LOGERROR(X) printf X;
#define FX "[SLDummyPlugin] "
#define LOGLN LOG((FX "------------------------------------------------\n"))
#define DO(X)
#else
#include<stdio.h>
#define LOG(X)	;
#define LOGERROR(X) printf X;
#define FX 
#define LOGLN LOG(X)
#define DO(X)
#endif


struct ESDInfo
{
	int ESID;
	int TimeStampLength;
	int NbPacket;
};

static bool										 sbInit=false;
static bool										 sbDebug=false;
/*-----------------------------------------------------------------
On stocke tout de sorte  ne pas tre embt par la gestion du
tableau des ods qui peut subir des effacements. Le cas limite o
l'on fait rfrence  un flux qui vient d'tre ferm n'est pas
improbable.
-----------------------------------------------------------------*/
static ZArray<struct ESDInfo>					 sESDs;


bool GetInfoOn		(int inESID, int &outTimeStampLength, int &outNbPacket, bool bUpdate=false)
{
	int		nESD	=0;
	bool	bFound	=false;

	for (int i=0; i< sESDs.GetLength(); i++)
	{

		int lvESID = sESDs[i].ESID;
		if (lvESID==inESID)
		{
			bFound=true;
			outNbPacket			= sESDs[i].NbPacket;
			outTimeStampLength	= sESDs[i].TimeStampLength;
			if (bUpdate) sESDs[i].NbPacket++;
			break;
		}
		nESD++;
	}
	return bFound;
}
/*
void ShowAll()
{
	for (int i=0; i< sESDs.GetLength(); i++)
	{
		printf("%d %d %d \n",sESDs[i].ESID,sESDs[i].NbPacket,sESDs[i].TimeStampLength);
	}
}
*/

bool Initialise    (unsigned int handle, SDLClassArray<ObjectDescriptor, 1, 255> & ods)
{
	if (!sbInit)
	{
		sbInit=true;
		LOGLN;
		LOG((FX "Initialising SLDummyPlugin\n"));		
		LOGLN;
		int nODs = ods.GetLength();
		for (int i=0; i< nODs; i++)
		{						
			LOG((FX "\tObjectDescriptor : %d\n",(int)(ods[i]->ObjectDescriptorID)));
			for (int j=0; j< ods[i]->esDescr.GetLength();j++)
			{							
				LOG((FX "\t\tAdding ES Descriptor : %d\n",(int)(ods[i]->esDescr[j]->ES_ID)));
				LOG((FX "\t\t\tdependsOnESID : %d\n",(int)(ods[i]->esDescr[j]->dependsOn_ES_ID)));
				LOG((FX "\t\t\ttimeStampLength : %d\n",(int) ods[i]->esDescr[j]->slConfigDescr->timeStampLength));
				struct ESDInfo structInfo;
				structInfo.ESID=(int)(ods[i]->esDescr[j]->ES_ID);
				structInfo.TimeStampLength=(int)(ods[i]->esDescr[j]->slConfigDescr->timeStampLength);
				structInfo.NbPacket=0;
				sESDs+= structInfo;
			}
		}
	}
	else
	{
		LOGLN;
		LOGERROR((FX "[ERROR] Dll has already been initialised\n"));
		LOGLN;
		throw;
	}
	return !sbInit;
}
bool Set		 (unsigned int handle,ES_DescriptorMux * esDescriptor, SL_PacketHeader * packetHeader)
{
	LOG((FX "Building SLPacket for ESID : %d ",(int)esDescriptor->ES_ID));
	int packetNumber,timeStampLength;
	GetInfoOn((int)esDescriptor->ES_ID,timeStampLength,packetNumber,true);
	if (esDescriptor->slConfigDescr->GetTag() == TAG_ExtSLConfigDescr)
	{
		LOG(("packet %d  (Extension)\n",packetNumber));
		AbstractSLConfig * tempSlConfig = (AbstractSLConfig *)esDescriptor->slConfigDescr;
		ExtSLConfigDescr * slConfig		= (ExtSLConfigDescr *)tempSlConfig;
		/*-----------------------------------------------------------------
		Go through the descriptorpointers and markers and add 
		the marker to the packet Header.
		-----------------------------------------------------------------*/
		for (int i=0; i<slConfig->slextDescr.GetLength();i++)
		{
			LOG((FX "\t storing %s",slConfig->slextDescr[i]->GetName()));
			switch (slConfig->slextDescr[i]->GetTag())
			{
			/*-----------------------------------------------------------------
			TAG_MarkerDescriptor : we simply write the packet number.
			-----------------------------------------------------------------*/
			case TAG_MarkerDescriptor:
			{
				MarkerDescriptor * markerDescr = (MarkerDescriptor *)slConfig->slextDescr[i];
				packetHeader->slExtensionsSizes.InsertElement((int)markerDescr->markerLength,i);
				packetHeader->slExtensions.InsertElement(packetNumber,i);
				LOG(("value(%d)=%d",(int)packetHeader->slExtensionsSizes.GetElement(i),(int)packetHeader->slExtensions.GetElement(i)));
				break;
			}
			/*-----------------------------------------------------------------
			TAG_DescriptorPointer : 
			-----------------------------------------------------------------*/
			case TAG_DependencyPointer:
			{
				bool bStrangeESID=false;
				DependencyPointer * depPointer = (DependencyPointer *)slConfig->slextDescr[i];
				//0] find ESID we depend on.
				int mESID=-1;
				if ((BOOL)depPointer->hasESID)
				{
					mESID=(int)depPointer->ESID;	
				}
				else
				{
					mESID=(int)esDescriptor->dependsOn_ES_ID;
					if (mESID==0)
					{
						bStrangeESID=true;
						LOG(("\n"));
						LOG((FX "\t Potentially something is wrong here...\n\tdependsOn_ES_ID==0 and hasESID==false\n"));
					}
				}
				//1] check length and write it...
				int lvLength = (int)depPointer->dependencyLength;
				if ((BOOL)depPointer->mode==0)
				{
					int lvDecodingTimeStampLength, lvNull;
					if (GetInfoOn(mESID,lvDecodingTimeStampLength,lvNull))
					{
						if (lvDecodingTimeStampLength!=lvLength)
						{
							LOG(("\n"));
							LOGERROR((FX "\t ERROR : invalid length...\n"));
							LOG((FX "\t lvDecodingTimeStampLength(%d)!=dependencyLength(%d)\n",lvDecodingTimeStampLength,lvLength));
						}
					}
					else
					{
						LOG(("\n"));
						LOGERROR((FX "\t ERROR : invalid ESID...\n"));
						if (bStrangeESID) LOG((FX "\t dependsOn_ES_ID==0 and hasESID==false\n"));
						LOG((FX "\t Could not find %d\n",mESID));
					}
				}
				packetHeader->slExtensionsSizes.InsertElement(lvLength,i);					//SDLArray
				packetHeader->slExtensions.InsertElement((int)1,i);							//default value.

				LOG(("value(%d)=%d",(int)packetHeader->slExtensionsSizes.GetElement(i),(int)packetHeader->slExtensions.GetElement(i)));
				break;
			}
			default:				
			{
				LOG((FX "\t unknown descriptor (skipping...)\n"));
				packetHeader->slExtensionsSizes.InsertElement((int)0,i);
				packetHeader->slExtensions.InsertElement((int)0,i);
			}
				break;
			}
			LOG(("\n"));
		}		
	}
	else
	{
		LOG(("\n"));
	}
	return true;
}




bool Delete   (unsigned int handle)
{
	LOG((FX "Delete\n"));
	sESDs.Clear();
	return true;
}


void DLL_CALLCONV Init(SLPlugin * pPlugin)
{
	sbDebug					= pPlugin->debug;
	pPlugin->handle			= 0;
	pPlugin->initFunction	= Initialise;
	pPlugin->setFunction	= Set;
	pPlugin->deleteFunction	= Delete;
}

