/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Multiplexer
FILE:       main.cpp
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   main() function for the off-line Mux
-------------------------------------------------------------------------*/
#include "defs.h"
#define MAIN
#include "Token.h"
#include "Mux.h"
#include "SysCoder.h"
#include "VrmlPars.h"

/***********Added by Liao Xuan *************/
#if defined(DES)

#include<stdlib.h>
#include<time.h>
#include <direct.h>

#ifndef _chdir
#define _chdir(val) chdir(val)
#endif

#endif //DES	
/***********End: Added by Liao Xuan ********/

Token nextToken;
BOOL g_bErrors;

/************ Added by Liao Xuan ****************************/
#if defined(DES)
int nAudioEncrypt;
char DES_Keys[16*200];//contain 200 DES keys

// print usage indication
void PrintUsage()
{
	printf ("Mux [--audio -des] inputfile\n"
			"Writing Options:\n"
			"-i: write the samples interleaved\n"
			"\nEncryption Algorithm:\n"
			"--audio: Audio encryption\n"
			"-des: DES algorithm\n"
			"\nMP4Enc v3.2 - Written by Jean Le Feuvre - (c) 2000 - 2003\n");
}
#endif //DES
/***********End: Added by Liao Xuan *************************/

int main (int argc, char **argv)
{
	FILE *fTRIF, *fList;
	int nNameLength;

	if (argc < 2)
	{	printf ("Missing file name\n");
		return -1;
	}

/***********Added by Liao Xuan *************/
#if defined(DES)
	ZString stPathName("");
	char *arg;
	ZString strParam("");
	nAudioEncrypt=-1;//Default algorithm is None

	//parse our agrs
	for (int i = 1; i < argc ; i++) {
		arg = argv[i];
		if (arg[0] != '-') 
		{
			stPathName = arg;
			break;
		}		
		//parse the flags
		switch (arg[1]) {
		case '-':	//for encryption.
			strParam=argv[i];
			if(strcmp(strParam,"--audio")==0)
			{
				//audio encryption specified
				i++;	//next argument should be the audio encryption algorithm
				strParam=argv[i];
					if(strcmp(strParam,"-des")==0)
					{
						nAudioEncrypt=1;
						break;	//break for next loop
					}
				PrintUsage();
				return -1;
			}
			PrintUsage();
			return -1;	
		default:
			PrintUsage();
			return -1;
		}//switch
	}//for


	
#endif //DES	
/***********End: Added by Liao Xuan ********/

	/*-----------------------------------------------------------------
	Source file name extension is "scr", output extension is "trif" and
	list file is "trc"
	-----------------------------------------------------------------*/

/****************Changed by Liao Xuan ************************/	
#if defined(DES)
	// process PathName and FileName of Argu
	ZString stFileName ("");
	nNameLength = stPathName.GetLength ();
	stFileName = strrchr (stPathName, '\\');
	if (stFileName.GetLength()) {
		stPathName.Cut(stPathName.GetLength() - stFileName.GetLength() + 1);
		_chdir(stPathName);
		stFileName.DeleteFrom(0);
	} else {
		stFileName = stPathName;
	}

#else
	ZString stFileName (*++argv) ;
#endif
/*****************End: Changed by Liao Xuan *******************/

	nNameLength = stFileName.GetLength ();
	LPCSTR szExt = strrchr (stFileName, '.');
	if (szExt && !strchr (szExt, '\\'))
		nNameLength -= strlen (szExt);
	else
		stFileName += ".scr";

	VRMLParser parser;
	if (!parser.Init (stFileName))
	{	printf ("Cannot open %s\n", (LPCSTR) stFileName);
		parser.Shutdown ();
		return -1;		
	}

/************ Added by Liao Xuan ****************************/
#if defined(DES)
	if(nAudioEncrypt==1) 
	{
		//	read DES key from DESkey.txt
		FILE *fpDESkey = fopen("DESkeysFile.txt","r"); //should be the first time to open the key file
		if(!fpDESkey)
		{
			printf ("DESkeysFile failed to open");
			return -1;
		}
		fread(DES_Keys, sizeof(DES_Keys),1,fpDESkey);
		fclose(fpDESkey);
		char buffer[4000];
		sprintf (buffer,"DES KEYs: \n%s\n",DES_Keys);
		OutputDebugString (buffer);
	}
#endif //DES
/***************End: Added by Liao Xuan **********************/

	stFileName.Cut (nNameLength);
	stFileName += ".trif";
	if ((fTRIF = fopen (stFileName, "wb")) == NULL)
	{	printf ("Cannot open %s\n", (LPCSTR) stFileName);
		return -1;
	}
	stFileName.Cut (nNameLength);
	stFileName += ".trc";
	if ((fList = fopen (stFileName, "w")) == NULL)
	{	printf ("Cannot open %s\n", (LPCSTR) stFileName);
		return -1;
	}

	stFileName.Cut (nNameLength);
	stFileName += ".trif.mct";  // must be last in list of files.
	//AC:20021009 - SL extension -- begin
	/*-------------------------------------------------------------
	Parse the SL extension manager dll
	-dll DLLName
	-------------------------------------------------------------*/
	char slManager[1024];
	strcpy(slManager,"DummySLManager.dll");
	if (argc >=3)
	{
		if (stricmp((*++argv),"-dll")==0)
		{
			strcpy(slManager,*++argv);
			argc --;
			argc --;			
		}
		else
		{
			--argv;	//could not find the dll
		}		
	}
	printf ("[SL Extension] Using  %s\n", slManager);
	//AC:20021009 - SL extension -- end
	/*-------------------------------------------------------------
	Parse the trace argument
	-------------------------------------------------------------*/
	int flags =  TRACE_NONE;
	if (argc >= 3 && (*++argv)[0] == '-')
	{	int c;
		for (int i = 1; (c = (*argv) [i]) != 0; i++)
			switch (tolower (c)) {
			case 'v':
				flags |= TRACE_VERBOSE;
				break;
			case 'h':
				flags |= TRACE_HEADERS;
				break;
			}
	}

	/*-----------------------------------------------------------------
	The Multiplex work is done by the Mux object
	-----------------------------------------------------------------*/
	nextToken.Init (&parser, stFileName, fList);
	Mux (fTRIF, (LPCSTR) stFileName).Start (fList, flags, stFileName,slManager);
	parser.Shutdown ();
	fclose (fList);
	fclose (fTRIF);
	if (g_bErrors)
	{	printf ("Hit any key to continue\n");
		getchar ();
		return -1;
	}
	return 0;
}
