/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Decoder
FILE:       NullDec.cpp
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Implementation of NullDecoder - an object that servers as
			a template for devlopment of decoder plug-ins.
-------------------------------------------------------------------------*/
//#include "Defs.h"
#include "defs.h" //SOLARIS PORTING - typo
#include "NullDec.h"

static char dumpFileName [] = "Stream0.dmp";

/*-------------------------------------------------------------
The exported function that instantiates the proper decoder
-------------------------------------------------------------*/
//MBS 110498: changed returned type to "void *"
void *CreateInstance ()
{
	return (void *) new NullDecoder;
}

/*-------------------------------------------------------------
Sets the decoder to CIF or QCIF
-------------------------------------------------------------*/
BOOL NullDecoder :: Setup (
	const BYTE *,			// pSpecificInfo,
	int,					// nSpecificInfoLength,
	int
)
{
	++dumpFileName [6];
	m_fDump = fopen (dumpFileName, "wb");
	return TRUE;
}

/*-----------------------------------------------------------------
Find a compatible format and initialize the decoder.
In this template we just pick up the first format.
-----------------------------------------------------------------*/
BOOL NullDecoder :: SetFormat (
	Capabilities &formatOptions,
	DecoderParams &decoderParams
)
{
	if (formatOptions.GetFormatCapabilityLength () == 0)
		return FALSE;
	decoderParams.m_nStreamNumber = 0;
	decoderParams.m_format = formatOptions.GetFormatCapability (0);
	decoderParams.m_nOptimalAUSize = 256;
	decoderParams.m_nDBSize = 1024;
	decoderParams.m_nCBSize = 100000;
	decoderParams.m_bRequiresFullAUs = TRUE;
	return TRUE;
}

/*-----------------------------------------------------------------
Calculate the output size in bytes
-----------------------------------------------------------------*/
int NullDecoder :: GetOptimalOutputSize ()
{
	return 256;
}

/*-------------------------------------------------------------
Decode one unit.
This template just copy input buffer to output buffer
-------------------------------------------------------------*/
BOOL NullDecoder :: Decode (
		LPBYTE pInput,
		int nInputLength,
		LPBYTE pOutput,
		int &rnOutputLength
)
{
	if (rnOutputLength < nInputLength)
	{	rnOutputLength = nInputLength;
		return FALSE;
	}

	memcpy (pOutput, pInput, nInputLength);
	return TRUE;
}

/*-------------------------------------------------------------
In this decoder, the decoded data always strats at beginning of output
block
-------------------------------------------------------------*/
int NullDecoder :: GetDataOffset  (const LPBYTE)
{
	return 0;
}

/*-------------------------------------------------------------
In this decoder, attributes are fixed for all units
-------------------------------------------------------------*/
DWORD NullDecoder :: GetAttributes (
	ATTRIBUTE_CODE,		// code, 
	const LPBYTE
)
{
	return 0;
}

void NullDecoder :: Terminate ()
{
	fclose (m_fDump);
}

/*-----------------------------------------------------------------
Run() retrieves units from the input stream, decode them, and dispatch
units to the output stream, till the input stream's end is exceeded.
-----------------------------------------------------------------*/
void NullDecoder :: Run ()
{
	LPBYTE pInData, pOutData;
	int nInSize, nOutSize;
	DWORD dwTime;
	BOOL bEOU;

	while (
		m_bActive &&
		(pInData =
			(LPBYTE) GetInputStream () -> FetchNext (nInSize, dwTime, bEOU, TRUE)
		) !=
		END_OF_STREAM
	)
	{
		if (m_bActive)
		{	nOutSize = GetOptimalOutputSize (),
			pOutData = (LPBYTE) GetOutputStream () -> Allocate (nOutSize, TRUE, FALSE);
			if (pOutData == NULL)
				throw;
			if (pOutData == END_OF_STREAM)
				return;
			fputc (HIBYTE (HIWORD (dwTime)), m_fDump);
			fputc (LOBYTE (HIWORD (dwTime)), m_fDump);
			fputc (HIBYTE (LOWORD (dwTime)), m_fDump);
			fputc (LOBYTE (LOWORD (dwTime)), m_fDump);
			fputc (HIBYTE (HIWORD (nInSize)), m_fDump);
			fputc (LOBYTE (HIWORD (nInSize)), m_fDump);
			fputc (HIBYTE (LOWORD (nInSize)), m_fDump);
			fputc (LOBYTE (LOWORD (nInSize)), m_fDump);
			fwrite (pInData, nInSize, 1, m_fDump);
			while (!Decode (pInData, nInSize, pOutData, nOutSize))
			{	pOutData = (LPBYTE) GetOutputStream () -> Reallocate (nOutSize);
				if (pOutData == NULL)
					throw;
				if (pOutData == END_OF_STREAM)
					return;
			}
			GetOutputStream () -> Dispatch (nOutSize, dwTime);
		}
		GetInputStream () -> Release (TRUE);
	}
	GetOutputStream () -> Close ();
}
