// This software module was originally developed by Marius PREDA
// (INT/ARTEMIS) in the course of development of the MPEG-4 Systems
// standard (ISO/IEC 14496-1). This current version includes approach 
// developed by Vctor Fernndez-Carbajales Caete (GTI-EPS-UAM/OLGA). 
// This software module is an implementation of a part of one or more MPEG-4
// Systems tools as specified by the MPEG-4 Systems standard.  ISO/IEC gives
// users of the MPEG-4 Systems standard free license to this software module
// or modifications thereof for use in hardware or software products claiming
// conformance to the MPEG-4 Systems standard.  Those intending to use this
// software module in hardware or software products are advised that its use
// may infringe existing patents.  The original developer of this software 
// module and his/her company, the subsequent editors and their companies, and
// ISO/IEC have no liability for use of this software module or modifications
// thereof in an implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The author retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// For more information or to receive an up to date version of this module,
// send a mail to Marius PREDA marius.preda@int-evry.fr
//
// Copyright 1997, 1998.

#include "../../OpenGl.h"
#include "SBBoneProxy.h"
#include "../../base/OpenGLVisualRenderer.h"

#include <vector>

using namespace std;	


// Render object
BOOL  SBBoneProxy::RenderFromStream(MediaStream* poStream, int id, DWORD dwTime)
{
  return TRUE;
}

BOOL  SBBoneProxy::RenderFromBuffer(SFString *pBuffer, int id)
{
  return TRUE;
}

// Render object description

/* virtual */ void
SBBoneProxy::RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD /* dwTime */)
{
  RenderDescriptionBox(pRenderer, "SBBone");
}

// Render object


BOOL /*SBBoneProxy::Render(const Matrix&, const CullFlag&, OpenGLVisualRenderer* pRenderer, DWORD dwTime)*/
SBBoneProxy::Render(const Matrix& rTransformation, const CullFlag& rCullFlag,
		  OpenGLVisualRenderer* pRenderer, DWORD dwTime)
{
    return MediaObjectProxy::Render(rTransformation, rCullFlag,
			     pRenderer, dwTime);
}

// Traverse scene graph, checking for intersection

/* virtual */ BOOL
SBBoneProxy::Select(OpenGLVisualRenderer* pRenderer, DWORD dwTime, unsigned int& depth)
{
  
#ifdef _DEBUG
  if (pRenderer->CheckGLError("SBBoneProxy:"))
    TRACE("SBBoneProxy::Render() [%08x]: ", this);
#endif

  return FALSE;
}


Rotation SBBoneProxy::GetRotationBoneSpace(){
	return m_rotationBoneSpace;
}

Rotation SBBoneProxy::GetRotationAbsolute(){
	return m_rotationAbsolute;
}

Rotation SBBoneProxy::GetRotation(){
	return m_rotation;
}

Vec3f SBBoneProxy::GetTranslation(){
	return m_translation;
}
Vec3f SBBoneProxy::GetTranslationBoneSpace(){
	return m_translationBoneSpace;
}
Vec3f SBBoneProxy::GetTranslationAbsolute(){
	return m_translationAbsolute;
}

Matrix SBBoneProxy::GetTranformMatrix(){
	return m_transformMatrix;
}


void SBBoneProxy::calculateState()
{

	m_transformMatrix.LoadIdentityTransform();
  // check if the bone was not touched by any active animation
//  if(m_accumulatedWeight == 0.0f)
//  {
    // set the bone to the initial skeleton state
    //m_translation = GetSBBone()->translation;
    //m_rotation = GetSBBone()->rotation;

	computeLocalTransform();

//  }

  // get parent bone id
  int parentId;
  int a=GetSBBone()->GetParent()->GetNodeCode();
  int b=GetNode()->GetNodeCode();
  
  if (a!=b){
    // no parent, this means absolute state == relative state
    //m_translationAbsolute = GetSBBone()->translation;
    //m_rotationAbsolute = GetSBBone()->rotation;
	m_transformMatrix = m_transformMatrixLocal;
  }
  else
  {
    // get the parent bone
    SBBone *pParent = (SBBone *)GetSBBone()->GetParent();
	SBBoneProxy *pBP=(SBBoneProxy *)(pParent->GetRenderingObject());

    // transform relative state with the absolute state of the parent
/*    m_translationAbsolute = m_translation;
    m_translationAbsolute *= pBP->GetRotationAbsolute();
    m_translationAbsolute += pBP->GetTranslationAbsolute();

	//m_rotation.theta+=0.1;
	//m_rotation.x=1;
    m_rotationAbsolute = m_rotation;
    m_rotationAbsolute *= pBP->GetRotationAbsolute();*/
	m_transformMatrix = pBP->GetTranformMatrix() * m_transformMatrixLocal;
  }


  // calculate the bone space transformation
  //m_translationBoneSpace = GetSBBone()->translation; // read from animation data
  //m_translationBoneSpace *= m_rotationAbsolute;
  //m_translationBoneSpace += m_translationAbsolute;

  //m_rotationBoneSpace = GetSBBone()->rotation;// read from animation data
  //m_rotationBoneSpace *= m_rotationAbsolute;

  //m_transformMatrix.Rotate(m_rotationAbsolute); 
//  m_transformMatrix.Translate(m_translationAbsolute);

//  int id=GetSBBone()->boneID;

  // calculate all child bones
  int noChilds = this->GetNode()->GetChildCount();
  for(int ch=0;ch<noChilds;ch++)  {
	  if (this->GetNode()->GetChild(ch)->GetNodeCode() == this ->GetNode()->GetNodeCode()){
		SBBoneProxy *pBPch=(SBBoneProxy *)(this->GetNode()->GetChild(ch)->GetRenderingObject());
		pBPch->calculateState();
	  }
  }
  
}

void SBBoneProxy::computeLocalTransform()
{
	// MP :: clean this code !

	if (isUpdated){
		SBBone *pB=GetSBBone();
		Vec3f center=pB->center.GetValue();
		Vec3f translation=pB->translation.GetValue();
		Rotation rotation=pB->rotation.GetValue();
		Vec3f scale=pB->scale.GetValue();
		Rotation scaleOrientation=pB->scaleOrientation.GetValue();

	//	FILE *myfile=fopen("bonefile.txt","a");
	//	this->printState(myfile);
	//	fclose(myfile);

		Matrix m1(center+translation);
		m_transformMatrixLocal=m1;
		Matrix m2(rotation);
		Matrix m12=m_transformMatrixLocal*m2;
		m_transformMatrixLocal=m12;
		// Victor Begin {
		Matrix m3(scale,scaleOrientation);
		Matrix m13=m_transformMatrixLocal*m3;
		m_transformMatrixLocal=m13;
		// } Victor End

			 // no scale for the moment
//		 if (!scaleFactor.isDefault()) {
//			m = RotationAxisMatrix(scaleOrientation.Axis(),scaleOrientation.Angle()) * m;
//			m = ScaleMatrix(scaleFactor) * m;
//			m = RotationAxisMatrix(scaleOrientation.Axis(),-scaleOrientation.Angle()) * m;
//		 }
		// Victor Begin {
		Vec3f ct(-center.x,-center.y,-center.z);
		Matrix m4(ct);
		Matrix m14=m_transformMatrixLocal*m4;
		m_transformMatrixLocal=m14;
		// } Victor End
		isUpdated=false;
	}	
}

void SBBoneProxy::printState(FILE *fl)
{
#ifdef _DEBUG
	SBBone *pB=GetSBBone();
	Vec3f tran=pB->translation;
	fprintf(fl, "Bone Id:%d, translation:%f %f %f",pB->boneID.GetValue(),tran.x, tran.y, tran.z);

	fprintf(fl, "Bone Id:%d, \nTransform Absolute:\n",pB->boneID.GetValue());
	m_transformMatrix.Print(fl);
	fprintf(fl, "Bone Id:%d, \nTransform Relative:\n",pB->boneID.GetValue());
	m_transformMatrixLocal.Print(fl);
	// calculate all child bones
	int noChilds = this->GetNode()->GetChildCount();
	for(int ch=0;ch<noChilds;ch++)  {
	  if (this->GetNode()->GetChild(ch)->GetNodeCode() == this ->GetNode()->GetNodeCode()){
		SBBoneProxy *pBPch=(SBBoneProxy *)(this->GetNode()->GetChild(ch)->GetRenderingObject());
		pBPch->printState(fl);
	  }
  }
#endif
}

