// This software module was originally developed by Marius Preda (INT)
// in the course of development of the MPEG-4 
// Systems standard (ISO/IEC 14496-1).  This software module is an 
// implementation of a part of one or more MPEG-4 Systems tools as specified by 
// the MPEG-4 Systems standard.  ISO/IEC gives users of the MPEG-4 Systems 
// standard free license to this software module or modifications thereof for 
// use in hardware or software products claiming conformance to the MPEG-4 
// Systems standard.  Those intending to use this software module in hardware or 
// software products are advised that its use may infringe existing patents.  
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use 
// of this software module or modifications thereof in an implementation.  
// Copyright is not released for non MPEG-4 Systems conforming products.  The 
// original developer retains full right to use the code for his/her own 
// purpose, assign or donate the code to a third party and to inhibit third 
// parties from using the code for non MPEG-4 Systems conforming products.  This 
// copyright notice must be included in all copies or derivative works.
//
// Copyright 2003.


#include "OpenGl/OpenGl.h"
#include "Presenter.h"
#include "SBSkinnedModelProxy.h"
#include "SBBoneProxy.h"
#include "SBSegmentProxy.h"
#include "Stream.h"

#include "opengl/base/OpenGLVisualRenderer.h"
#include "opengl/geometry/ShapeProxy.h"
#include "opengl/geometry/GeometryProxy.h"
#include "opengl/appearance/AppearanceProxy.h"


class Geometry;

/*
SBSkinnedModelProxy::SBSkinnedModelProxy(SBSkinnedModel * pNS)
  : GeometryProxy(pNS),m_nTimeStamp(-1)
{
}

SBSkinnedModelProxy::~SBSkinnedModelProxy()
{
}
*/

/* virtual */ void
SBSkinnedModelProxy::RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD /* dwTime */)
{
  RenderDescriptionBox(pRenderer, "SBSkinnedModel");
}

// Render object

BOOL SBSkinnedModelProxy::Render(const Matrix& rTransformation, const CullFlag& rCullFlag,
		  OpenGLVisualRenderer* pRenderer, DWORD dwTime)
{
  
	if (!init) {
		GetInitialSkin();
		init=true;
	}

	SBSkinnedModel* poSBSM = GetSBSkinnedModel();

	UpdateVertices(); // Temporary here but normally MUST be called by the decoder


	Coordinate *pCoordinate = (Coordinate *)poSBSM->skinCoord.GetNode();
	Normal *pNormal = (Normal *)poSBSM->skinNormal.GetNode();
	const MF<Vec3f> &points  = pCoordinate->point;
	const MF<Vec3f> &vectors  = pNormal->vector;
	
	int noSP = poSBSM->skin.GetLength();

	int shp;
	for (shp=0;shp<noSP;shp++){
		Shape *pShp = (Shape*) poSBSM->skin[shp];
		ShapeProxy *pShpProxy = (ShapeProxy *) (pShp->GetRenderingObject());
		Geometry* pGeometry=(Geometry*)pShpProxy->GetGeometry();
		IndexedFaceSet *pISF=(IndexedFaceSet *)pGeometry;
		pISF->SetUpdateTime();
		pShpProxy->CullOrRender(rTransformation, rCullFlag,	pRenderer, dwTime);  


	}
	
	
	int noSeg=poSBSM->segments.GetLength();

	ZArray<SBSegmentProxy *> nodes;
	for (int i=0;i<noSeg;i++){
		SBSegment* pSg = poSBSM->segments[i];
		SBSegmentProxy * pProxy = (SBSegmentProxy *) (pSg->GetRenderingObject());
		nodes.InsertAt(nodes.GetLength(), pProxy);
  }

  for (int j = 0; j < nodes.GetLength(); ++j)
    nodes[j]->CullOrRender(rTransformation, rCullFlag,
			   pRenderer, dwTime);
  

  return NodeProxy::Render(rTransformation, rCullFlag,
			     pRenderer, dwTime);
}

/*
BOOL SBSkinnedModelProxy::Render(const Matrix& rTransformation, const CullFlag& rCullFlag,
		  OpenGLVisualRenderer* pRenderer, DWORD dwTime)
{
  
	if (!init) {
		GetInitialSkin();
		init=true;
	}

	SBSkinnedModel* poSBSM = GetSBSkinnedModel();

	UpdateVertices(); // Temporary here but normally MUST be called by the decoder

	glPushAttrib(GL_POLYGON_BIT);
	
	Coordinate *pCoordinate = (Coordinate *)poSBSM->skinCoord.GetNode();
	Normal *pNormal = (Normal *)poSBSM->skinNormal.GetNode();
	const MF<Vec3f> &points  = pCoordinate->point;
	const MF<Vec3f> &vectors  = pNormal->vector;
	
	int noSP = poSBSM->skin.GetLength();
	for (int shp=0;shp<noSP;shp++){
		Shape *pShp = (Shape*) poSBSM->skin[shp];
		ShapeProxy *pShpProxy = (ShapeProxy *) (pShp->GetRenderingObject());
		MediaObject* pGeometry=pShpProxy->GetGeometry();
		IndexedFaceSet *pIFS=(IndexedFaceSet*) pGeometry;
		Appearance* pAppearance = pShpProxy->GetAppearance();
		
		((AppearanceProxy*) pAppearance->GetRenderingObject())->Enable(pRenderer, dwTime);
		
		const MF<int>  &indices = pIFS->coordIndex;
		int noVertexInTriangles=indices.GetLength();
		for (int k = 0; k < noVertexInTriangles;){
			glBegin(GL_POLYGON);
			for (int j = 0; j < 3; j++){
				int index = indices[k++];
				if (pNormal) glNormal3fv(vectors[index]);
				//			glTexCoord2fv (p+2*vert);
				glVertex3fv(points[index]);
			}
			k++;  // skip the -1
			glEnd();
		}
		((AppearanceProxy*) pAppearance->GetRenderingObject())->Disable(pRenderer, dwTime);
	}
	
	glPopAttrib();
	
	return NodeProxy::Render(rTransformation, rCullFlag,
			     pRenderer, dwTime);
}
*/
/* virtual */ void
SBSkinnedModelProxy::Select(
		const Matrix& rTransformation, 
		const CullFlag& rCullFlag,
		OpenGLVisualRenderer* pRenderer, 
		DWORD dwTime,
		MediaObjectProxy*& intersectingObject,
		ZArray<MediaObjectProxy*>& activatedSensors, 
		unsigned int& depth,
		const ZArray<MediaObjectProxy*>& localSensors)
{

	SBSkinnedModel* poSBSM = GetSBSkinnedModel();
  int noSP = poSBSM->skin.GetLength();

  ZArray<MediaObjectProxy *> nodes;
  ZArray<MediaObjectProxy *> sensors;

  for (int i=0;i<noSP;i++){
	MediaObject * pObject = (Shape*) poSBSM->skin[i];
	MediaObjectProxy *pProxy = (MediaObjectProxy *) (pObject->GetRenderingObject());
	nodes.InsertAt(nodes.GetLength(), pProxy);
	if (pProxy->IsPointingDeviceSensor())
      sensors += pProxy;
  }
	

  for (int j = 0; j < nodes.GetLength(); ++j){
		nodes[j]->CullOrSelect(rTransformation, rCullFlag,pRenderer, dwTime,intersectingObject,      activatedSensors,       depth,	  localSensors);
  }
}


void SBSkinnedModelProxy::UpdateVertices()
{

	// Update the SBSkinnedModel vertices 
	// with respect to the Bones transforms
	// initial data (vertices Coord) are stored in m_vectorVertex

	SBSkinnedModel* poSBSM = GetSBSkinnedModel();
	
	MF<Vec3f> &vertices=GetVerticesCoord();
	int noVertices=vertices.GetLength();


	// first compute the bone transforms in world coordinate //


	SBBone *pB = (SBBone*) poSBSM->skeleton[0];
	SBBoneProxy *pBP = (SBBoneProxy *) (pB->GetRenderingObject());
	pBP->calculateState();
	


	// update each vertex wrt all the bones it is connected to
	int vertexId;
	for(vertexId = 0; vertexId < noVertices; ++vertexId){
		Vertex& vertex = m_vectorVertex[vertexId];
	    float x = 0.0f;
		float y = 0.0f;
		float z = 0.0f;
		// blend together all vertex influences
		int influenceId;
		int influenceCount=(int)vertex.vectorInfluence.size();
		for(influenceId = 0; influenceId < influenceCount; ++influenceId){
			Influence& influence = vertex.vectorInfluence[influenceId];
		
			// get the bone of the influence vertex
			SBBone *pBone = (SBBone*) poSBSM->bones[influence.boneId];
			SBBoneProxy *pBP = (SBBoneProxy *) (pBone->GetRenderingObject());

			// transform vertex with current state of the bone

		  	Matrix m=pBP->GetTranformMatrix();
			Vec3f v(vertex.position.x,vertex.position.y,vertex.position.z);
			v *= m;

			//v += pBP->getTranslationBoneSpace(); for physics in future version of the standard
		
			x += influence.weight * v.x;
			y += influence.weight * v.y;
			z += influence.weight * v.z;

		}
		
		Vec3f localVertex(x,y,z);		
		vertices.SetAt(vertexId,localVertex);
		
	}

}

void SBSkinnedModelProxy::setVertex(int vertexId, float x,float y,float z)
{
	SBSkinnedModel* poSBSM = GetSBSkinnedModel();

	Vec3f localVertex;
	localVertex.x=x;
	localVertex.y=y;
	localVertex.z=z;

	
	poSBSM->skinCoord->point.SetAt(vertexId,localVertex);	
	
}


void SBSkinnedModelProxy::GetInitialSkin()
{

	// Get the SBSkinnedModel initial vertices and 
	// keep a copy for later animation 
	
	// output: build the m_vectorVertex
	// that contains vertices coordinates, and bonnes influence for each vertex


	SBSkinnedModel* poSBSM = GetSBSkinnedModel();
	const MF<Vec3f>& vertices=GetVerticesCoord();
	const MF<Vec3f>& normales=GetVerticesNormals();
	int noVertices=poSBSM->skinCoord->point.GetLength();
	m_vectorVertex.reserve(noVertices);
	m_vectorVertex.resize(noVertices);

	// get bones//

	int noBones = poSBSM->bones.GetLength();
	
	ZArray<SBBone *> nodes;
	int b,i;
	
	for (b=0;b<noBones;b++){
		SBBone * pObject = (SBBone*) poSBSM->bones[b];
		nodes.InsertAt(nodes.GetLength(), pObject);
	}

	for (i=0;i<noVertices;i++)
		m_vectorVertex[i].influenceCount=0;
	
	for (b = 0; b < nodes.GetLength(); ++b){
	  const MF<int>& indices = nodes[b]->skinCoordIndex;
	  for (int j=0;j<indices.GetLength();j++)
		m_vectorVertex[indices[j]].influenceCount++;
	}

	for (i=0;i<noVertices;i++)
		m_vectorVertex[i].vectorInfluence.reserve(m_vectorVertex[i].influenceCount);

	for (b = 0; b < nodes.GetLength(); ++b){
		const MF<int>& indices = nodes[b]->skinCoordIndex;
		const MF<float>& weights = nodes[b]->skinCoordWeight;
		for (int j=0;j<indices.GetLength();j++){
			Influence I;
			I.boneId=b;
			I.weight=weights[j];
			m_vectorVertex[indices[j]].vectorInfluence.push_back(I);
		}
	}


	for (i=0;i<noVertices;i++){
		m_vectorVertex[i].position.x=vertices[i].x;
		m_vectorVertex[i].position.y=vertices[i].y;
		m_vectorVertex[i].position.z=vertices[i].z;

}


}

bool SBSkinnedModelProxy::setVertex(int vertexId, const Vertex& vertex)
{
  if((vertexId < 0) || (vertexId >= (int)m_vectorVertex.size())) return false;

  m_vectorVertex[vertexId] = vertex;

  return true;
}

/*
std::vector<Vertex>& SBSkinnedModelProxy::getVectorVertex()
{
  return m_vectorVertex;
}
*/

void SBSkinnedModelProxy::UpdateSBSegments()
{
	SBSkinnedModel* poSBSM = GetSBSkinnedModel();
	int noSeg=poSBSM->segments.GetLength();
	for (int i=0;i<noSeg;i++)
		poSBSM ->segments[i]->OnUpdate();

	// get the parent for each segment
	// get its transform
	// affect the segment by this transform
}
