// This software module was originally developed by Sang Oak Woo (SASUNG AIT),
// James D.K. Kim (SAMSUNG, AIT), Kevin Barron (KPN Research), 
// Mike Coleman (Five Bats Research), Edward Cooke (Teltec Ireland), In-Dai Ha 
// (Hyundai Electronics Industries, Co.), Hae-Kwan Kim (Hyundai Electronics 
// Industries, Co.), Matthew Leditschke (CSELT), Zvi Lifshitz (VDOnet Corp.), 
// Karl Anders ygard (Telenor R&D), in the course of development of the MPEG-4 
// Systems standard (ISO/IEC 14496-1).  This software module is an 
// implementation of a part of one or more MPEG-4 Systems tools as specified by 
// the MPEG-4 Systems standard.  ISO/IEC gives users of the MPEG-4 Systems 
// standard free license to this software module or modifications thereof for 
// use in hardware or software products claiming conformance to the MPEG-4 
// Systems standard.  Those intending to use this software module in hardware or 
// software products are advised that its use may infringe existing patents.  
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use 
// of this software module or modifications thereof in an implementation.  
// Copyright is not released for non MPEG-4 Systems conforming products.  The 
// original developer retains full right to use the code for his/her own 
// purpose, assign or donate the code to a third party and to inhibit third 
// parties from using the code for non MPEG-4 Systems conforming products.  This 
// copyright notice must be included in all copies or derivative works.
//
// Copyright 1997.

// SOWOO 2003/06/21

#include "OpenGl/OpenGl.h"
#include "Presenter.h"
#include "BitWrapperProxy.h"
#include "Stream.h"

#include "AfxStreamProxy.h" // VUB
#include "AfxMediaObjectProxy.h" // VUB
#include "Im1Decoders/AFX/AFX.h"  // VUB
#include "CompressedInterpolatorProxy.h" // VUB - Samsung
#include "AfxMediaObjectProxy.h" // VUB - Samsung
#include "../interpolators/PositionInterpolatorProxy.h"
#include "../interpolators/PositionInterpolatorProxy.h"
#include "../interpolators/OrientationInterpolatorProxy.h"
#include "../interpolators/CoordinateInterpolatorProxy.h"
#include "../geometry/IndexedFaceSetProxy.h" // by JHAHN 20030627
#include "DIBR/OctreeImageProxy.h"
// 2004-07-15, Shinjun {
#include "DIBR/PointTextureProxy.h"
// 2004-07-15, Shinjun }
#include "../AFX/WaveSurfProxy.h"	// PG (FT)

void BitWrapperProxy::SetDecoderData(AfxStreamProxy *pProxy)
{
  pProxy->SetupDecoderData(this);
}

BOOL BitWrapperProxy::Render(const Matrix& rTransformation, const CullFlag& rCullFlag,
                             OpenGLVisualRenderer* pRenderer, DWORD dwTime)
{
  BitWrapper* poBitWrapper = GetBitWrapper();
  NodeField* pNode = (NodeField*) poBitWrapper->GetField(0);
  MediaObject* pMO = ((SFGenericNode*) pNode)->GetNode();		// BitWrapper.node



  switch (pMO->GetNodeCode())
  {
  case CODE_IndexedFaceSet:
  case CODE_MeshGrid:
  case CODE_WaveletSubdivisionSurface:
    {
      AfxStreamProxy *pProxy = (AfxStreamProxy *)pMO->GetRenderingObject();
      if (pProxy == NULL) return FALSE;

	  //ory DMLAB 2006 12
   	  pProxy->setType(poBitWrapper->type.GetValue());
	  //~ory DMLAB
	  
      if (m_bIsDecodingNeeded)
      {
        // ouband or inband decoding ?
        if (poBitWrapper->GetURL().GetLength() > 0)
        {
          // bitstream from URL
          SetDecoderData(pProxy);
          Start();
        
          MediaStream* poStream = poBitWrapper->GetStream();
          if (pProxy->RenderFromStream(poStream, pMO->GetID(), dwTime))
          {
            m_bIsDecodingNeeded = false;
        
            MediaStream* poStream = poBitWrapper->GetStream();
            poStream->Release();
          }
        }
        else
        {
          // bitstream from buffer
          // SFString* pBuffer = (SFString*) poBitWrapper->GetField(3);
          pProxy->RenderFromBuffer(&poBitWrapper->buffer, pMO->GetID());
        
          m_bIsDecodingNeeded = false;
        }
      } 
    }
    break;
  case CODE_PositionInterpolator:
  case CODE_OrientationInterpolator:
  case CODE_CoordinateInterpolator:
    {
      CompressedInterpolatorProxy *pProxy = (CompressedInterpolatorProxy *)pMO->GetRenderingObject();
      if (pProxy == NULL) return FALSE;
    
      if (m_bIsDecodingNeeded)
      {
        // ouband or inband decoding ?
        if (poBitWrapper->GetURL().GetLength() > 0)
        {
          // bitstream from URL
          Start();
        
          MediaStream* poStream = poBitWrapper->GetStream();
          if (pProxy->RenderFromStream(poStream, pMO->GetID(), dwTime))
          {
            m_bIsDecodingNeeded = false;
        
            MediaStream* poStream = poBitWrapper->GetStream();
            poStream->Release();
          }
        }
        else
        {
          // bitstream from buffer
          SFString* pBuffer = (SFString*) poBitWrapper->GetField(3);
          pProxy->RenderFromBuffer(pBuffer, pMO->GetID());
        
          m_bIsDecodingNeeded = false;
        }
      }
    }
    break;
  case CODE_OctreeImage:
    {
      AFXMediaObjectProxy *pProxy = (AFXMediaObjectProxy *)pMO->GetRenderingObject();
      if (pProxy == NULL) return FALSE;

      if (m_bIsDecodingNeeded)
      {
        // ouband or inband decoding ?
        if (poBitWrapper->GetURL().GetLength() > 0)
        {
          // bitstream from URL
          Start();
        
          MediaStream* poStream = poBitWrapper->GetStream();
          if (pProxy->RenderFromStream(poStream, pMO->GetID(), dwTime))
          {
            m_bIsDecodingNeeded = false;
        
            MediaStream* poStream = poBitWrapper->GetStream();
            poStream->Release();
          }
        }
        else
        {
          // bitstream from buffer
          SFString* pBuffer = (SFString*) poBitWrapper->GetField(3);
          pProxy->RenderFromBuffer(pBuffer, pMO->GetID());
        
          m_bIsDecodingNeeded = false;
        }
      }
      else
      {
        pProxy->Render(rTransformation, rCullFlag, pRenderer, dwTime);
      }
    }
    break;
// 2004-07-15, Shinjun {
  case CODE_PointTexture:
    {
	  static bool bURL = false;
	  PointTextureProxy *pProxy = (PointTextureProxy *)pMO->GetRenderingObject();
      if (pProxy == NULL) return FALSE;
    
      if (m_bIsDecodingNeeded)
      {
        // ouband or inband decoding ?
        if (poBitWrapper->GetURL().GetLength() > 0)
        {
          // bitstream from URL
          Start();
        
          MediaStream* poStream = poBitWrapper->GetStream();
          if (pProxy->RenderFromStream(poStream, pMO->GetID(), dwTime))
          {
            m_bIsDecodingNeeded = false;
        
            MediaStream* poStream = poBitWrapper->GetStream();
            poStream->Release();
          }
		  bURL = true;
        }
        else
        {
          // bitstream from buffer
          SFString* pBuffer = (SFString*) poBitWrapper->GetField(3);
          pProxy->RenderFromBuffer(pBuffer, pMO->GetID());
        
          m_bIsDecodingNeeded = false;
		  bURL = false;
        }
      }
	else
      {
	    if(bURL == true)
			pProxy->Render(rTransformation, rCullFlag, pRenderer, dwTime);
      }
    }
    break;
// 2004-07-15, Shinjun }
  case CODE_FootPrintSet:
    {
      AfxStreamProxy *pProxy = (AfxStreamProxy *)pMO->GetRenderingObject();
      if (pProxy == NULL) return FALSE;
    
      if (m_bIsDecodingNeeded)
      {
        // ouband or inband decoding ?
        if (poBitWrapper->GetURL().GetLength() > 0)
        {
          // bitstream from URL
          SetDecoderData(pProxy);
          Start();
        
          MediaStream* poStream = poBitWrapper->GetStream();
          if (pProxy->RenderFromStream(poStream, pMO->GetID(), dwTime))
          {
            m_bIsDecodingNeeded = false;
        
            MediaStream* poStream = poBitWrapper->GetStream();
            poStream->Release();
          }
        }
        else
        {
          // bitstream from buffer
          SFString* pBuffer = (SFString*) poBitWrapper->GetField(3);
          pProxy->RenderFromBuffer(&poBitWrapper->buffer, pMO->GetID());
        
          m_bIsDecodingNeeded = false;
        }
      } 
    }
    break;
  default: ASSERT(0);
  }

  return TRUE;
}
