// -*- c++ -*-

// This software module was originally developed by Karl Anders ygard
// (Telenor R&D) in the course of development of the MPEG-4 Systems
// standard (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// For more information or to receive an up to date version of this module,
// mail karlo@nta.no or check <http://televr.fou.telenor.no/compositor/>.
//
// Copyright 1997, 1998. 

#include "../../OpenGl.h"
#include "GL/gl.h"
#include "stream.h"
#include "DepthImageProxy.h"
#include "SimpleTextureProxy.h"
#include "../../appearance/ImageTextureProxy.h"
#include "PointTextureProxy.h"
#include "glext.h"
#include "../../base/OpenGLVisualRenderer.h"

// Render object description

/* virtual */ void
DepthImageProxy::RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD /* dwTime */)
{
    RenderDescriptionBox(pRenderer, "DepthImage");
}

// Render object

// Make GL_LIST
enum { COLOR, DEPTH };			

int DepthImageProxy::MakeGLList(void){

    DepthImage * pDepthImage = GetDepthImage();

    if(pDepthImage->diTexture.GetNode() == NULL) {
        return m_nListMade = -1;
    }
    
    int NodeCode = pDepthImage->diTexture.GetNode()->GetNodeCode();
    InitSplatParametersExtensions();

    if ( NodeCode == CODE_SimpleTexture ) {
        NodeField* pNode = (NodeField*) pDepthImage->GetField(0);
        SimpleTexture * pST = (SimpleTexture *) ((SFGenericNode*) pNode)->GetNode();
        SimpleTextureProxy * pSTP = (SimpleTextureProxy *) pST->GetRenderingObject();
        pSTP->m_pDepthImageProxy = this;


        if( IsListMade() == 0 ) { 
		    m_nListMade = 1;
			
            int SubNodeCode = pST->texture.GetNode()->GetNodeCode();

			if(SubNodeCode == CODE_ImageTexture)
			{ 
                m_nGlListID = glGenLists(1);
			    glNewList(m_nGlListID, GL_COMPILE);
			    pSTP->CreateGlList(pDepthImage->orthographic.GetValue()?-1:pDepthImage->nearPlane, pDepthImage->orthographic.GetValue()?-1:pDepthImage->farPlane,0);				                
			    glEndList();
		    }
			else if(SubNodeCode == CODE_MovieTexture)
			{
				int DepthNumberOfFrame, ColorNumberOfFrame;				
				DepthNumberOfFrame = ColorNumberOfFrame = 0;

				pSTP->SetAnim(TRUE);

				NodeField* pImgNode0 = (NodeField*) pST->GetField(0);   // depth
				NodeField* pImgNode1 = (NodeField*) pST->GetField(1);   // texture

				//Depth Data fetch
				MovieTexture* pIMO = (MovieTexture*)(((SFGenericNode*)pImgNode0)->GetNode());
				MovieTextureProxy* pImgProxy0 = (MovieTextureProxy*) pIMO->GetRenderingObject();															
				pSTP->GetImageInfoFromMovieTexture(pImgProxy0,DEPTH);

				//Depth Frame number
				DepthNumberOfFrame = pSTP->m_nNumberOfFrame;

				//Color Data fetch
				MovieTexture* pIM1 = (MovieTexture*)(( (SFGenericNode*)pImgNode1)->GetNode());
				MovieTextureProxy* pImgProxy1 = (MovieTextureProxy*) pIM1->GetRenderingObject();
				pSTP->GetImageInfoFromMovieTexture(pImgProxy1,COLOR);

				//Color Frame number
				ColorNumberOfFrame = pSTP->m_nNumberOfFrame;

				if ( !pSTP->m_nColorVideoFetched || !pSTP->m_nDepthVideoFetched ) {
					m_nListMade = 0;
					return m_nListMade;
				}	
				
				if(DepthNumberOfFrame < ColorNumberOfFrame)
					m_nNumberOfFrame = DepthNumberOfFrame;
				else
					m_nNumberOfFrame = ColorNumberOfFrame;
				
				int listnum;					
				
				m_pLists = (int*)realloc((void*)m_pLists, m_nNumberOfFrame*sizeof(int));
											
				listnum = glGenLists(m_nNumberOfFrame);								
				for(int x=0;x<m_nNumberOfFrame;x++){
					m_pLists[x] = listnum + x;					
					
					glNewList(m_pLists[x], GL_COMPILE);
					pSTP->CreateGlList(pDepthImage->orthographic.GetValue()?-1:pDepthImage->nearPlane, pDepthImage->orthographic.GetValue()?-1:pDepthImage->farPlane,x);
					glEndList();				
				}
			}
        }
	    return m_nListMade;
    }
    else if ( NodeCode == CODE_PointTexture ) {
        PointTexture * pPT = (PointTexture *) pDepthImage->diTexture.GetNode();
        PointTextureProxy * pPTP = (PointTextureProxy *) pPT->GetRenderingObject();
        pPTP->m_pDepthImageProxy = this;

        NodeField * pNF0= (NodeField *) pPT->GetField(2);
        NodeField * pNF1 = (NodeField *) pPT->GetField(3);
        NodeField * pNF2= (NodeField *) pPT->GetField(4);

        pPTP->SetDepthBitPerPixel( ((SF<int>*) pNF0)->GetValue()+1 );
		pPTP->SetMaxDepth( pow(2, pPTP->GetDepthBitPerPixel())-1 );
		pPTP->SetHeight( ((SF<int>*) pNF1)->GetValue() );
        pPTP->SetWidth( ((SF<int>*) pNF2)->GetValue() );

        if( IsListMade() == 0 ) { 
		    
			if(m_bGLEXTInit)	
			{
                m_nGlListID = glGenLists(1);
			    glNewList(m_nGlListID, GL_COMPILE);			    
                pPTP->CreateGlList(pDepthImage->nearPlane, pDepthImage->farPlane); 
			    glEndList();
		    }
        }
	    return m_nListMade = 1;
    }

    return m_nListMade = -1;
}

PFNGLPOINTPARAMETERFEXTPROC glPointParameterfEXT=NULL;
PFNGLPOINTPARAMETERFVEXTPROC glPointParameterfvEXT=NULL;

bool DepthImageProxy::InitSplatParametersExtensions()
{
	if(m_bGLEXTInit) return true;
	glPointParameterfEXT = (PFNGLPOINTPARAMETERFEXTPROC)wglGetProcAddress("glPointParameterfEXT");
	glPointParameterfvEXT = (PFNGLPOINTPARAMETERFVEXTPROC)wglGetProcAddress("glPointParameterfvEXT");
	return m_bGLEXTInit=(glPointParameterfEXT != NULL && glPointParameterfvEXT != NULL);
}

void DepthImageProxy::CalculateSplatParametes(OpenGLVisualRenderer* pRenderer, float model_size, float& size, float& minsize, float&  maxsize, float& a, float& b, float& c, float& thresh)
{
	minsize = 1e-100;
	maxsize = 200;
	size = 1;
	a = 0;
	b = 0;

	int x,y;
	float pdiameter = model_size;
	
	x = pRenderer->GetViewportWidth();
	y = pRenderer->GetViewportHeight(); 

	size = 1; //pdiameter;  
	
	double ang;
	if(pRenderer->GetActiveViewPoint() != NULL)
		ang = pRenderer->GetActiveViewPoint()->fieldOfView;	
	else
		ang = 45;
	
	double tn = tan(ang /2 );
	double c_root =  2 * tn / (y * pdiameter);
	c = (float)(c_root * c_root); // FIXME: 2.2 is an euristic parameter
}

void DepthImageProxy::SetSplatGLParameters(float size, float minsize, float maxsize, float a, float b, float c, float thresh)
{
	float att[3];
	att[0] = a;
	att[1] = b;
	att[2] = c;
	glPointSize(size);	

	glPointParameterfvEXT(GL_DISTANCE_ATTENUATION_EXT, att);
	glPointParameterfEXT(GL_POINT_SIZE_MIN_EXT, minsize);
	glPointParameterfEXT(GL_POINT_SIZE_MAX_EXT, maxsize);
	glPointParameterfEXT(GL_POINT_FADE_THRESHOLD_SIZE_EXT, thresh);
}

/* virtual */ BOOL
DepthImageProxy::Render(const Matrix& rTransformation, const CullFlag& rCullFace, OpenGLVisualRenderer* pRenderer, DWORD dwTime)
{
    DepthImage * pDepthImage = GetDepthImage();
	
    if(pDepthImage->diTexture.GetNode() == NULL) {
        return FALSE;
    }

    // Get parameters in DepthImage node
    double nearPlane = pDepthImage->nearPlane;
    double farPlane = pDepthImage->farPlane;

    Vec2f FOV = pDepthImage->fieldOfView.GetValue();
    double FOVx = FOV.x;
    double FOVy = FOV.y;
    Vec3f position = pDepthImage->position.GetValue();
    Rotation orientation = pDepthImage->orientation.GetValue();
    Bool ortho = pDepthImage->orthographic.GetValue();
    BOOL orthographic = bool(ortho);

    // Initial transform according to the transformation information in DepthImage
    glPushMatrix();

    // Temporary routine for rotation  -- In Kyu Park 2003.0701
    //glRotated( m_dRotateAngle, 0, 1, 0 );        // Simple rotation test
    //m_dRotateAngle += 1;

	glTranslatef(position.x, position.y, position.z);
	glRotatef(orientation.theta*180/M_PI, orientation.x, orientation.y, orientation.z);
       
    if( orthographic ){
		glTranslatef(0,0,-nearPlane);
		glScalef(FOVx, FOVy, farPlane-nearPlane);
    } else {
		glTranslatef(0, 0, -nearPlane);
		double x = tan(FOVx/2)*farPlane*2;
		double y = tan(FOVy/2)*farPlane*2;
		double z = farPlane-nearPlane;
		glScalef(x,y,z);
	}

	glEnable(GL_POINT_SMOOTH);
    
    if( m_nListMade == 0 ) {
        MakeGLList();
    }
    else if ( m_nListMade == -1 ) {
        glPopMatrix();
        return FALSE;
    }

    if( m_nListMade == 0 ) {
        glPopMatrix();
        return FALSE;            // Not fetched yet. So try to fetch again.
    }

    int NodeCode = pDepthImage->diTexture.GetNode()->GetNodeCode();

	int width, height, depth;
	bool anim = false;			

	if( NodeCode == CODE_SimpleTexture ) {
        SimpleTexture * pST = (SimpleTexture *) pDepthImage->diTexture.GetNode();
        SimpleTextureProxy * pSTP = (SimpleTextureProxy *) pST->GetRenderingObject();

        depth = pSTP->GetMaxDepth();
        height = pSTP->GetHeight();
        width = pSTP->GetWidth();

		anim = pSTP->GetAnim();
		if(anim){
			m_nCurrentListIdx = m_nCurrentListIdx % m_nNumberOfFrame;		
		}
    } else if( NodeCode == CODE_PointTexture ) {
        PointTexture * pPT = (PointTexture *) pDepthImage->diTexture.GetNode();
        PointTextureProxy * pPTP = (PointTextureProxy *) pPT->GetRenderingObject();

        depth = pPTP->GetMaxDepth();
        height = pPTP->GetHeight();
        width = pPTP->GetWidth();
    }
    else {
        depth = 256;
        height = 256;
        width = 256;
    }

	if(orthographic && m_bGLEXTInit){
		
		float size, minsize, maxsize, a, b, c, thresh;
		float xdiameter = FOVx / width ;
		float ydiameter = FOVy / height ;
		int dmax = depth-1;
		float zdiameter = (farPlane - nearPlane) / dmax; // FIXME: this is only true for 8-bit bitmaps
		float diameter = max(xdiameter, max(ydiameter, zdiameter));

        CalculateSplatParametes(pRenderer, diameter, size, minsize, maxsize, a, b, c, thresh);

        if( NodeCode == CODE_SimpleTexture ){
			c/=2;
        } else if( NodeCode == CODE_PointTexture ){
			c/=3;
		}
		SetSplatGLParameters(size, minsize, maxsize, a, b, c, thresh);
	}

	if(anim)
	{
		if(glIsList(m_pLists[m_nCurrentListIdx]))
		{
			glCallList(m_pLists[m_nCurrentListIdx]);				
			m_nCurrentListIdx += 1;								
		}	
	}else{
		glCallList(m_nGlListID);
	}	

	glPopMatrix();

    return 0;
}

void DepthImageProxy::DrawAxis(void)
{
    glDisable(GL_LIGHTING);
	glBegin(GL_LINES);
		glColor3f( 1, 0, 0 );
		glVertex3f( 0, 0, 0 );
		glVertex3f( 10, 0, 0 );

		glColor3f( 0, 1, 0 );
		glVertex3f( 0, 0, 0 );
		glVertex3f( 0, 10, 0 );

		glColor3f( 0, 0, 1 );
		glVertex3f( 0, 0, 0 );
		glVertex3f( 0, 0, 10 );
		
	glEnd();
    glEnable(GL_LIGHTING);
}

// Traverse scene graph, checking for intersection

/* virtual */ BOOL
DepthImageProxy::Select(OpenGLVisualRenderer* pRenderer, DWORD dwTime, unsigned int& depth)
{
    // I am not sure this would work really 2003.06.24 In Kyu Park

  GLuint selectBuffer[4];

  ::glSelectBuffer(4, selectBuffer);
  ::glRenderMode(GL_SELECT);

  if (::glRenderMode(GL_RENDER) > 0)
    if (depth > selectBuffer[1]) {
      depth = selectBuffer[1];

      return TRUE;
    }

#ifdef _DEBUG
  if (pRenderer->CheckGLError("DepthImageProxy:"))
    TRACE("DepthImageProxy::Render() [%08x]: ", this);
#endif

  return FALSE;
}

