/*\   Programmed by Alexander Zhirkov azh@graphics.cs.msu.su
\*/

//---------------------------------------------------------------------------
#ifndef _IBRArray_H_
#define _IBRArray_H_
//---------------------------------------------------------------------------

#include <assert.h>
#include <math.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#include "IBRCommon.h"

template <class T> class IBRArray
{
protected:
	T *__D;
    int __A,__I;
    bool __S; // can automate increasing
public:
	IBRArray(int a){__I=0;__D=0;__A=-1; __S=true; Alloc(a);};
	IBRArray(){__I=0;__D=0; __A=-1; __S=true; Alloc(1); };
	void FromPointer(T *g,int a)
    {
    	Assert(g,"<IBRArray::FromPointer> null pointer");
    	Assert(a>=0,"<IBRArray::FromPointer> bad pointer size="+a);
    	AllocSet(a);
        iforinc(i,a)
            __D[i]=g[i];
    }
	T* Pointer()
    {
        return &__D[0];
    }
    void AutoSize(bool s){ __S=s;};
    ~IBRArray(){delete []__D;};
    IBRArray<T> This(){return *this;}
    void Copy(IBRArray<T> &g)
    {
		Alloc(g.Size());
        iforinc(i,__A)
            Elem(i)=g[i];
        SetPoint(g.All());
	}
    int All(){return __I;}
    int Size(){return __A;}
    void Reset(){__I=0;};
	void Minimize(){__I=0; Alloc(1);}

    void SetPoint(int i)
    {
    	Assert(i>=0 && i<=__A,"<IBRArray::SetPoint> bad point i="+i);
        __I=i;
    }
    void AllocSet(int a)
    {
    	Alloc(a);
        SetPoint(a);
    }
    void AllocSetIfNeed(int a)
    {
    	AllocIfNeed(a);
        SetPoint(a);
    }
    void AllocIfNeed(int a)
    {
     	Assert(a>=0,"<IBRArray::AllocIfNeed> bad allocation size= "+a);
        if(a<=__A)
        	return;
        Alloc(a);
    }
    void Alloc(int a)
    {
     	Assert(a>=0,"<IBRArray::Alloc> bad allocation size= "+a);
        if(a==__A)
        	return;
    	if(__A>0) delete []__D;
        __A=a;
        __D=new T[__A];
     	Exit(__D,"<IBRArray::Alloc> allocation error");
    }
    int Add(T e)
    {
    	if(__I<__A)	__D[__I]=e;
        else 	ReAlloc(__A*2+1),__D[__I]=e;
        __I++;
        return __I-1;
    }
    int Add()
    {
    	if(__I>=__A)  ReAlloc(__A*2+1);
        __I++;
        return __I-1;
    }
    int MultiAdd(int n)
    {
		Assert(n>=0,"<MultiAdd> negative number of elemetns to add");
    	if(__I+n>__A)  ReAlloc(mx(__A*2,__I+n));
        __I+=n;
        return __I-1;
    }
	void Beg(){__I=-1;}
	int Next(){
        __I++;
        Assert(__I>=0,
        	"<IBRArray::operator[]> bad array index "+__I);
    	if(__I>=__A) return 0;
	    return 1;}
    T& Cur(){
    	return Elem(__I);}
    T& CurNext()
    {
        int q=Next();
        Assert(q,"<IBRArray::CurNext> overflow map");
		return Elem(__I-1);
    }
    T& Last(){
    	return Elem(__I-1);}
	void DeleteElem(int n)
    {
    	AssertIndexText(n,__A,"<IBRArray::DeleteElem>");
        int a=__A-1;
    	T *d=new T[a];
        Exit(d,"<IBRArray::ReAlloc> can't allocate swap data");
        int i=0;
        for(;i<n;i++)
        	d[i]=__D[i];
        for(;i<a;i++)
        	d[i]=__D[i+1];
        Alloc(a);
        for(i=0;i<__A;i++)
        	__D[i]=d[i];
        delete []d;
    }
	void SetAll(int v)
    {
        iforinc(i,__A)
        	__D[i]=v;
    }
    void ReAlloc(int a)
    {
       	Exit(__S,"IBRArray overflow !");
     	Assert(a>=0,"<IBRArray::ReAlloc> bad allocation size= "+a);
        if(a<__A)
        	return;
    	T *d=new T[a];
        Exit(d,"<IBRArray::ReAlloc> can't allocate swap data");
        int i;
        for(i=0;i<__A;i++)
        	d[i]=__D[i];
        Alloc(a);
        for(i=0;i<__A;i++)
        	__D[i]=d[i];
        delete []d;
    }
    T& Elem(int i)
    {
    	Assert(__D,"<IBRArray::operator[]> Array not initialized");
        Assert(i>=0 && i<__A,
        	"<IBRArray::operator[]> Array index "+i+" not in range 0.."+(__A-1));
	    return __D[i];
    }
    T& ElemAdd(int i)
    {
     	Assert(i>=0,"<IBRArray::AddElem> i<0");
        if(i>=__A)
	    	ReAlloc(i+1);
        return __D[i];
    }
    T& operator[](int i)
    {
    	return Elem(i);
    }
};
//---------------------- ONE DIMENSIONAL ARRAYS --------------------------------
struct IBRArrayInt1D:
	public IBRArray<int>
{
	void ClearInt1D(int c)
    {
    	iforinc(i,All()) Elem(i)=c;
    }
};
struct IBRArrayFloat1D:
	public IBRArray<flt>
{
	void ClearFloat1D(flt c)
    {
    	iforinc(i,All()) Elem(i)=c;
    }
};
class IBRArrayChar:
	public IBRArray<char>
{
public:
	char* Chars(){
    	Assert(__D,"<IBRArrayChar::Char> Array not initialized");
    	return __D;}
    void FromChars(char* s){
        AllocSet(strlen(s)+1);
        strcpy(Chars(),s);}
    void FWrite(FILE *f)
    {
        Assert(f,"<IBRArrayChar::FWrite> bad file stream");
        fwrite(&Elem(0), 1, All(), f);
    }
    void FRead(FILE *f,int a)
    {
        Assert(f,"<IBRArrayChar::FRead> bad file stream");
        AllocSet(a);
        fread(&Elem(0), 1, a, f);
    }
    void FWrite(chr* t)
    {
        FILE *f=fopen(t,"wb");
        Exit(f,"can create "+t);
        FWrite(f);
        fclose(f);
    }
    void FRead(chr* t)
    {
        FILE *f=fopen(t,"rb");
        Exit(f,"can't read "+t);
        int a=f_size(f);
        FRead(f,a);
        fclose(f);
    }
    int IsEqual(char* s)
    {
        iforinc(i,All())
        {
            if(Elem(i)!=s[i])
            	return 0;
            if(!Elem(i))
            	return 1;
        }
        return 1;
    }
    
};
//---------------------- TWO DIMENSIONAL ARRAYS --------------------------------
struct IBRArrayInt2D:
	public IBRArray<IBRArrayInt1D>
{
	void ClearInt2D(int c){
    	iforinc(i,All()) Elem(i).ClearInt1D(c);}
	void Alloc2D(int d1,int d2){
    	Alloc(d1);
        iforinc(i,d1) Elem(i).Alloc(d2);}
	void AllocSet2D(int d1,int d2){
    	AllocSet(d1);
        iforinc(i,d1) Elem(i).AllocSet(d2);}
};
struct IBRArrayFloat2D:
	public IBRArray<IBRArrayFloat1D>
{
	void ClearFloat2D(flt c){
    	iforinc(i,All()) Elem(i).ClearFloat1D(c);}
	void Alloc2D(int d1,int d2){
    	Alloc(d1);
        iforinc(i,d1) Elem(i).Alloc(d2);}
	void AllocSet2D(int d1,int d2){
    	AllocSet(d1);
        iforinc(i,d1) Elem(i).AllocSet(d2);}
    void FromPointer2D(flt *p,int v,int h)
    {
        AllocSet2D(v,h);
        iforinc(i,v)
        	Elem(i).FromPointer(&p[i*h],h);
    }
};
struct ListIBRArrayChar:
	public IBRArray<IBRArrayChar>
{
	void ClearList(char *s){
    	iforinc(i,All()) Elem(i).FromChars(s);}
    void Clone(ListIBRArrayChar &g){
		AllocSet(g.All());
    	iforinc(i,All()) Elem(i).FromChars(g[i].Chars());}
	void FromFile(chr *t)
	{
		FILE *f=fopen(t,"r");
    	Exit(f,"can't open "+t);
	    chr s[256];
    	int a;
		f_line255r(f,s);
		for(a=0;!feof(f);a++)
			f_line255r(f,s);
    	fseek(f,0,0); // seek to begin
	    AllocSet(a);
    	iforinc(i,a)
	    {
			f_line255r(f,s);
	        Elem(i).FromChars(s);
    	}
	    fclose(f);
	};
};
//---------------------- >TWO DIMENSIONAL ARRAYS -------------------------------
struct IBRArrayInt3D:
	public IBRArray<IBRArrayInt2D>
{
  	void ClearInt3D(int c=0){
    	iforinc(i,All()) Elem(i).ClearInt2D(c);}
	void Alloc3D(int d1,int d2,int d3){
    	Alloc(d1);
        iforinc(i,d1) Elem(i).Alloc2D(d2,d3);}
	void AllocSet3D(int d1,int d2,int d3){
    	AllocSet(d1);
        iforinc(i,d1) Elem(i).AllocSet2D(d2,d3);}
};
struct IBRArrayFloat3D:
	public IBRArray<IBRArrayFloat2D>
{
	void ClearFloat2D(flt c){
    	iforinc(i,All()) Elem(i).ClearFloat2D(c);}
	void Alloc3D(int d1,int d2,int d3){
    	Alloc(d1);
        iforinc(i,d1) Elem(i).Alloc2D(d2,d3);}
	void AllocSet3D(int d1,int d2,int d3){
    	AllocSet(d1);
        iforinc(i,d1) Elem(i).AllocSet2D(d2,d3);}
};
struct IBRArrayInt4D:
	public IBRArray<IBRArrayInt3D>
{
	void AllocSet4D(int d1,int d2,int d3,int d4){
    	AllocSet(d1);
        for(int i=0;i<d1;i++) Elem(i).AllocSet3D(d2,d3,d4);}
};

//---------------------------------------------------------------------------

#endif

