// -*- c++ -*-

// This software module was originally developed by Karl Anders ygard
// (Telenor R&D) in the course of development of the MPEG-4 Systems
// standard (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// For more information or to receive an up to date version of this module,
// mail karlo@nta.no or check <http://televr.fou.telenor.no/compositor/>.
//
// Copyright 1997, 1998.

#include "../../OpenGl.h"
#include "OctreeImageProxy.h"
#include "../../base/OpenGLVisualRenderer.h"
#include "../../IM1Decoders/AFX/jwafxstreamdecoder.h"
#include "Stream.h"

#include "SimpleTextureProxy.h"
#include "IBRCommon.h"

enum { COLOR, DEPTH };
vec3d vec3d_Null=vec3d(0.,0.,0.);
vec3d vec3d_Identity=vec3d(1.,1.,1.);
vec3d vec3d_X=vec3d(1.,1.,1.);


// Render object
BOOL  OctreeImageProxy::RenderFromStream(MediaStream* poStream, int id, DWORD dwTime)
{
  BYTE * pData = NULL;
  int nSize = 0;
  BOOL bEOU;
  int nStatus;
  BOOL bSkip = TRUE;
  BOOL bWait = FALSE;
  long lDelta = 0;
  BOOL bDontClose = TRUE;
  DWORD rdvTime;
  
  pData = (unsigned char *) poStream->Fetch(
    nSize, rdvTime, bEOU, nStatus, bSkip, bWait, lDelta, bDontClose);
  
  if (pData)
  {
    YAFXOctreeDecoder * pDecoder = new YAFXOctreeDecoder;
  
    // Set OctreeImage contents
    pDecoder->SetValues(pData);
  
    // Test
    pDecoder->Write("AFX-OctreeImageURL.debug.vrml", id);
    delete pDecoder;

    Enable(pData);

    return TRUE;
  }

  return FALSE;
}

BOOL  OctreeImageProxy::RenderFromBuffer(SFString *pBuffer, int id)
{
  YAFXOctreeDecoder * pDecoder = new YAFXOctreeDecoder;
  
  int nSize = 0;
  BYTE *pData = NULL;
  //Decode the bitstream
  pDecoder->Decode((LPBYTE) pBuffer->GetValue(), pBuffer->GetLength(), (LPBYTE) pData, nSize);
  
  // Test
  pDecoder->Write("AFX-OctreeImageBuffer.debug.vrml", id);

  Enable(pData);

  return TRUE;
}

// Render object description

/* virtual */ void
OctreeImageProxy::RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD /* dwTime */)
{
  RenderDescriptionBox(pRenderer, "SimpleTexture");
}

// Render object
Vec3f Vec3f_Null = Vec3f(0.,0.,0.);

int OctreeImageProxy::MakeGLList(void)
{
    OctreeImage * pOctreeImage = GetOctreeImage();

	if(pOctreeImage == NULL) {
        return m_nListMade = -1;
    }  
	
	MFNode<class DepthImage>*depthimages =  (MFNode<class DepthImage>*)pOctreeImage->GetField(0);
	NumOfImages = depthimages->GetLength();	

	if(m_nListMade == 0) {
	
		m_nFetchedImage = 0;
		for(int i=0;i<NumOfImages;i++){
		
			DepthImage *p = depthimages->operator [](i);			
			DepthImageProxy * pDI = (DepthImageProxy *) p->GetRenderingObject();
			pDI->InitSplatParametersExtensions();
			m_bGLEXTInit = pDI->m_bGLEXTInit;

			SFDepthTextureNode *depthtexture = (SFDepthTextureNode *)p->GetField(0);
			SimpleTexture *st = (SimpleTexture *)depthtexture->GetNode();
			SimpleTextureProxy *stp = (SimpleTextureProxy *)st->GetRenderingObject();

			NodeField* pImgNode1 = (NodeField*) st->GetField(1);   // texture    	

			m_nListMade = 1;

			int SubNodeCode = st->texture.GetNode()->GetNodeCode();
			if(SubNodeCode == CODE_MovieTexture)
			{
				m_nAnim = TRUE;
					
				MovieTexture* pIM1 = (MovieTexture*)(((SFGenericNode*)pImgNode1)->GetNode());
				MovieTextureProxy* pImgProxy1 = (MovieTextureProxy*) pIM1->GetRenderingObject();
				
				if(!stp->m_nColorVideoFetched)
				{
					stp->GetImageInfoFromMovieTexture(pImgProxy1,COLOR);					
					m_nNumberOfFrame = stp->m_nNumberOfFrame;
				}
				
				if(!stp->m_nColorVideoFetched){
					m_nListMade = 0;
					return m_nListMade;
				}		
				else {
					stp->SwapVideoRows();
					m_nFetchedImage++;
				}
				
				if(m_nFetchedImage == NumOfImages) {
					m_nListMade = 1;					
				}
			}
			else if(SubNodeCode == CODE_ImageTexture)
			{
				ImageTexture* pIM1 = (ImageTexture*)(((SFGenericNode*)pImgNode1)->GetNode());
				ImageTextureProxy* pImgProxy1 = (ImageTextureProxy*) pIM1->GetRenderingObject();
				
				if(!stp->m_nColorFetched)
				{
					stp->GetImageInfoFromImageTexture(pImgProxy1, COLOR);
				}

				if(!stp->m_nColorFetched) 
				{
					m_nListMade = 0;
					return FALSE;
				}		
				else {
					stp->SwapImageRows();
					m_nFetchedImage++;
				}
				
				if(m_nFetchedImage == NumOfImages) {
					m_nListMade = 1;					
				}
			}
		}
	}
		

	OctreeHeight=(int)((double)1.000000001+log(OctreeResolutionField-1)/log(2));

	m_bTBVO =  VoxelImageIndexFieldLength > 0;

	int listnum;

	if(m_bTBVO)
	{	//TBVO
		if(m_nAnim)
		{
			m_pLists = (int*)realloc((void*)m_pLists, m_nNumberOfFrame*sizeof(int));
			listnum = glGenLists(m_nNumberOfFrame);
						
			for(int x=0;x<m_nNumberOfFrame;x++)
			{
				m_pLists[x] = listnum + x;										
				glNewList(m_pLists[x], GL_COMPILE);

				MakeTBVOTopology();			
			    ToLeafs(0);       
			    if(m_bGLEXTInit){
					ToPointsTBVO(x);
				}
			    glEndList();				
			}
		}
		else
		{
			MakeTBVOTopology();			
						
		    ToLeafs(0);              
		    if(m_bGLEXTInit){
				m_nGlListID = glGenLists(1);
				glNewList(m_nGlListID, GL_COMPILE);
				ToPointsTBVO(0);
				glEndList();
			}			
		}			
	}
	else{	//BVO
		if(m_nAnim)
		{
			m_pLists = (int*)realloc((void*)m_pLists, m_nNumberOfFrame*sizeof(int));
			listnum = glGenLists(m_nNumberOfFrame);
						
			for(int x=0;x<m_nNumberOfFrame;x++)
			{
				m_pLists[x] = listnum + x;	
				
				glNewList(m_pLists[x], GL_COMPILE);

				MakeBVOData(x);  
				ToPointsBVO(x);

				glEndList();
			}
		}
		else
		{
			MakeBVOData(0);   
          
			if(m_bGLEXTInit){
				m_nGlListID = glGenLists(1);
                glNewList(m_nGlListID, GL_COMPILE);
                ToPointsBVO(0);
                glEndList();
			}
        }
	}
	
	return 1;
}

int OctreeImageProxy::MakeBVOData(int idx)
{
    OctreeImage * pOctreeImage = GetOctreeImage();
  	MFNode<class DepthImage>*depthimages =  (MFNode<class DepthImage>*)pOctreeImage->GetField(0);
	NumOfImages = depthimages->GetLength();	

	m_cur_frame = 0;
    int a=0,i,w,h;          
      
    m_cms.AllocSet(NumOfImages);
      
    for( i = 0; i < NumOfImages; i++ )
    {
			DepthImage *p = depthimages->operator [](i);
			SFDepthTextureNode *depthtexture = (SFDepthTextureNode *)p->GetField(0);
			SimpleTexture *st = (SimpleTexture *)depthtexture->GetNode();
			SimpleTextureProxy *stp = (SimpleTextureProxy *)st->GetRenderingObject();	

           w = stp->GetWidth();
		   h = stp->GetHeight();

		   Vec3f position = p->position;	
		   Rotation orientation = p->orientation;
		   Vec2f FOV = p->fieldOfView;

           m_cms[i].Set(
                vec3d( 
                     (double)(position.x), 
                     (double)(position.y), 
                     (double)(position.z) ),
                vec3d( 
                     (double)(orientation.x), 
                     (double)(orientation.y), 
                     (double)(orientation.z) ),
                orientation.theta,
                p->nearPlane,p->farPlane,
				FOV.x, FOV.y,
                w,h
                );
    }
      
    {
           if( !UtilityBVO_BreadthFirstToVoxels( bvo_ind ) ) return 0;
                          
           UtilityCameraImages_VoxelsToGeneralizedVoxels( 0 , idx );
           
           ArrayDouble1D camdistr;
           vec3d light_n = vec3d_Null; 
           unt light_c = 180|(127<<8)|(240<<16)|(0<<24);
           
           camdistr.AllocSet(m_cms.All());
           camdistr.ClearDouble1D(1);
           
           frames.Add();          
		   PointsCC &pnts = frames[frames.All()-1]();		   
		   VoxelImageIndexFieldLength = m_vxls.All();
		   
           pnts.AllocSet(m_vxls.All());
           forinc(i,m_vxls.All())
                pnts[i].P = m_vxls[i];
           
           UtilityGeneralizedVoxels_FillColorPoints(
                camdistr,   // in: cameras distribution
                light_c,    // in: light color RGBA
                light_n,    // in: light normal (1.0 - diffuse color component)
                pnts        // out: points XYZRGBA
                );
    }
	return 1;
}

void OctreeImageProxy::MakeTBVOTopology()
{
	OctreeImage * pOctreeImage = GetOctreeImage();
	MFNode<class DepthImage>*depthimages =  (MFNode<class DepthImage>*)pOctreeImage->GetField(0);

	TexturesTopology = new TBVOTexturePar[NumOfImages];

	for(int i=0;i<NumOfImages;i++){	
		DepthImage *p = depthimages->operator [](i);				

		SFDepthTextureNode *depthtexture = (SFDepthTextureNode *)p->GetField(0);
		SimpleTexture *st = (SimpleTexture *)depthtexture->GetNode();
		SimpleTextureProxy *stp = (SimpleTextureProxy *)st->GetRenderingObject();		

		TexturesTopology[i].W = stp->GetWidth();
		TexturesTopology[i].H = stp->GetHeight();

		TexturesTopology[i].bytesperscan = stp->GetWidth() * stp->GetColorBytePerPixel(); 
		TexturesTopology[i].bytesperpixel = stp->GetColorBytePerPixel(); 
  
		Vec3f position = p->position;				
		TexturesTopology[i].O[0] = position.x*OctreeResolutionField;
		TexturesTopology[i].O[1] = position.y*OctreeResolutionField;
		TexturesTopology[i].O[2] = position.z*OctreeResolutionField;

		Rotation orientation = p->orientation;
		TexturesTopology[i].FromOrientation(orientation);
		TexturesTopology[i].O[0] += OctreeResolutionField/2;
		TexturesTopology[i].O[1] += OctreeResolutionField/2;
		TexturesTopology[i].O[2] += OctreeResolutionField/2;					
	}
}

void OctreeImageProxy::ToLeafs(int cl)  // cl- coarse level
{
	int k,i,j,t,b,n;
	int aoct,aup,iup,idown;
	TBVOLeaf *bufup,*bufdown,*buftemp;	

	aoct = OctreeFieldLength + VoxelImageIndexFieldLength;	
	
	Leafs = new TBVOLeaf[aoct*16];
	bufup  = &Leafs[0];
	bufdown = &Leafs[aoct*10];
	
	n = OctreeHeight - cl + 1;
	
	aup = 1;
	bufup[0].Set(0,0,0,0);	

	for(k=0;k<n;k++){
		idown = 0;		
		for(iup=0;iup<aup;iup++){
			if(k != OctreeHeight)	b = (*pOctreeField)[m_cur_byte_octree++];
			t = bufup[iup].T;
			if(!t)	t = (*pVoxelImageIndexField)[m_cur_byte_img_ind++];
			if(k == OctreeHeight){
				bufup[iup].T = t;
			}else{
				for(j=0;j<8;j++)
					if(b&(1<<j))
						bufdown[idown++].Set(
						(bufup[iup].X<<1)|(j&1),
						(bufup[iup].Y<<1)|((j&2)>>1),
						(bufup[iup].Z<<1)|((j&4)>>2),
						t);
			}
		}			

		buftemp = bufup, bufup = bufdown, bufdown = buftemp;
		t = aup; aup = idown; idown = t;
	}

	//ofstream fout;
	signed short x, y, z;

	//fout.open("leaf3d.txt");
	for(i=0;i<idown;i++){
		Leafs[i]=bufdown[i];	
		
		//fout << Leafs[i].X <<","<<Leafs[i].Y <<","<< Leafs[i].Z <<endl;		
	}
	//fout.close();
	
	TotalNumOfLeafs = idown;
}

void OctreeImageProxy::ToPointsTBVO(int idx) 
{
	int dx,dy,dz;

    OctreeImage * pOctreeImage = GetOctreeImage();
	
	if(!m_bGLEXTInit){	
		dx = OctreeResolutionField/double(NGRAD);
		dy = OctreeResolutionField/double(NGRAD);
		dz = OctreeResolutionField/double(NGRAD);
		double diag = min(min(dx,dy),dz)/(1.24);
		////////////////////////////////////
		GLdouble model_matrix[16];
		GLdouble proj_matrix[16];
		GLint viewport[4];
		
		glGetDoublev(GL_MODELVIEW_MATRIX, model_matrix);
		glGetDoublev(GL_PROJECTION_MATRIX, proj_matrix);
		glGetIntegerv(GL_VIEWPORT, viewport);

		int x_,y_,z_;
		for(x_=0;x_<=NGRAD;x_++){
			for(y_=0;y_<=NGRAD;y_++){
				for(z_=0;z_<=NGRAD;z_++){
					double xx,yy,zz;
					xx = x_ * dx - OctreeResolutionField / 2;
					yy = y_ * dy - OctreeResolutionField / 2;
					zz = z_ * dz - OctreeResolutionField / 2;

					gluProject(
						xx, yy, zz,
						model_matrix, proj_matrix,  viewport,
						&prj[x_][y_][z_][0],
						&prj[x_][y_][z_][1],
						&prj[x_][y_][z_][2]);
				}
			}
		}
		
#define Q(a) ((a)*(a))
#define MAX(a,b) ((a)>(b)?(a):(b))
#define LEN(a,b) (Q(a[0]-b[0])+Q(a[1]-b[1]))
			for(z_=0;z_<NGRAD;z_++){
				for(y_=NGRAD-1;y_>=0;y_--){
					for(x_=0;x_<NGRAD;x_++){
						double len[4];
						len[0] = LEN(prj[x_+0][y_+0][z_+0],prj[x_+1][y_+1][z_+1]);
						len[1] = LEN(prj[x_+1][y_+0][z_+0],prj[x_+0][y_+1][z_+1]);
						len[2] = LEN(prj[x_+0][y_+1][z_+0],prj[x_+1][y_+0][z_+1]);
						len[3] = LEN(prj[x_+0][y_+0][z_+1],prj[x_+1][y_+1][z_+0]);
						double d = sqrt(MAX(MAX(len[0],len[1]),MAX(len[2],len[3])))/diag;
						splatsize[x_][y_][z_] = d + 0.6;
						if(splatsize[x_][y_][z_]<1) splatsize[x_][y_][z_] = 1 ;
					}
				}
			}
#undef Q
#undef MAX
#undef LEN
	}
	/////////////////////////////////
	
	int i,t;
	int cr,cg,cb;
	int splt = 1;
	
	if(!m_bGLEXTInit)	glPointSize(splt);
	
	glBegin(GL_POINTS);

	MFNode<class DepthImage>*depthimages =  (MFNode<class DepthImage>*)pOctreeImage->GetField(0);

	for(i=0;i<TotalNumOfLeafs;i++)
	{
		TBVOLeaf& lf = Leafs[i];
		t = lf.T;
		
		if(t){
			TBVOTexturePar& tpar = TexturesTopology[t-1];
			
			DepthImage *p = depthimages->operator [](t-1);
			SFDepthTextureNode *depthtexture = (SFDepthTextureNode *)p->GetField(0);
			SimpleTexture *st = (SimpleTexture *)depthtexture->GetNode();
			SimpleTextureProxy *stp = (SimpleTextureProxy *)st->GetRenderingObject();

			int pos = TransXYZToUV(tpar,lf.X,lf.Y,lf.Z);

			if(m_nAnim)
			{
				int MovieBufIdx = idx*stp->m_nRGBFrameSize + pos;

				cr = stp->m_pOTMovieColorBuf[MovieBufIdx];
				cg = stp->m_pOTMovieColorBuf[MovieBufIdx+1];
				cb = stp->m_pOTMovieColorBuf[MovieBufIdx+2];				
			}
			else
			{				
				cr = stp->m_pOTColorBuf[pos+0];
				cg = stp->m_pOTColorBuf[pos+1];
				cb = stp->m_pOTColorBuf[pos+2];			
			}			
		}

		if(!m_bGLEXTInit){
			int &splt_ = splatsize[int(lf.X/dx)][int(lf.Y/dy)][int(lf.Z/dz)];
			if(splt != splt_){
				splt = splt_;
				glEnd();
				glPointSize(splt);
				glBegin(GL_POINTS);
			}
		}
		glColor3ub(cr,cg,cb);	
		glVertex3d(lf.X-(OctreeResolutionField>>1),lf.Y-(OctreeResolutionField>>1),lf.Z-(OctreeResolutionField>>1));				
	}

	glEnd();
}

int OctreeImageProxy::TransXYZToUV(const TBVOTexturePar &tp, int x,int y,int z)
{
	int pt[3];
	pt[0] = x; pt[1] = y; pt[2] = z;
	pt[0] -= tp.O[0];
	pt[1] -= tp.O[1];
	pt[2] -= tp.O[2];

	int xx = tp.Is ? -1-pt[tp.I]:pt[tp.I];
	int yy = tp.Js ? -1-pt[tp.J]:pt[tp.J];
	xx += tp.W/2;
	yy += tp.H/2;

	int res;
	res = xx*tp.bytesperpixel+yy*tp.bytesperscan;	

	return res;
}

void OctreeImageProxy::ToPointsBVO(int idx)
{
	int dx,dy,dz;
	if(!m_bGLEXTInit){
		dx=OctreeResolutionField/double(NGRAD);
		dy=OctreeResolutionField/double(NGRAD);
		dz=OctreeResolutionField/double(NGRAD);
		double diag=min(min(dx,dy),dz)/(1.24);
		////////////////////////////////////

		GLdouble model_matrix[16];
		GLdouble proj_matrix[16];
		GLint viewport[4];
		
		glGetDoublev(GL_MODELVIEW_MATRIX, model_matrix);
		glGetDoublev(GL_PROJECTION_MATRIX, proj_matrix);
		glGetIntegerv(GL_VIEWPORT, viewport);
		int x_,y_,z_;
		for(x_=0;x_<=NGRAD;x_++){
			for(y_=0;y_<=NGRAD;y_++){
				for(z_=0;z_<=NGRAD;z_++){
					double xx,yy,zz;
					xx=x_*dx-OctreeResolutionField/2;
					yy=y_*dy-OctreeResolutionField/2;
					zz=z_*dz-OctreeResolutionField/2;
					gluProject(
						xx,yy,zz,
						model_matrix, proj_matrix,  viewport,
						&prj[x_][y_][z_][0],
						&prj[x_][y_][z_][1],
						&prj[x_][y_][z_][2]);
				}
			}
		}
			
#define Q(a) ((a)*(a))
#define MAX(a,b) ((a)>(b)?(a):(b))
#define LEN(a,b) (Q(a[0]-b[0])+Q(a[1]-b[1]))
			for(z_=0;z_<NGRAD;z_++){
				for(y_=NGRAD-1;y_>=0;y_--){
					for(x_=0;x_<NGRAD;x_++){
						double len[4];
						len[0]=LEN(prj[x_+0][y_+0][z_+0],prj[x_+1][y_+1][z_+1]);
						len[1]=LEN(prj[x_+1][y_+0][z_+0],prj[x_+0][y_+1][z_+1]);
						len[2]=LEN(prj[x_+0][y_+1][z_+0],prj[x_+1][y_+0][z_+1]);
						len[3]=LEN(prj[x_+0][y_+0][z_+1],prj[x_+1][y_+1][z_+0]);
						double d=sqrt(MAX(MAX(len[0],len[1]),MAX(len[2],len[3])))/diag;
						splatsize[x_][y_][z_]=d+0.6;
						if(splatsize[x_][y_][z_]<1) splatsize[x_][y_][z_]=1;
					}
				}
			}
#undef Q
#undef MAX
#undef LEN
	}
	
 	PointsCC &pnts = frames[ idx ]();
	if(idx > m_nNumberOfFrame)	idx = 0;
	int i;
	int a = pnts.All();
	
	int splt=1;
	if(!m_bGLEXTInit) glPointSize(splt);

	glBegin(GL_POINTS);
	forinc(i,a)
	{
		PointCC& pnt=pnts[i];
		if(!m_bGLEXTInit){
			int &splt_=splatsize[int(pnt.P[0]/dx)][int(pnt.P[1]/dy)][int(pnt.P[2]/dz)];
			if(splt!=splt_){
				splt=splt_;
				glEnd();
				glPointSize(splt);
				glBegin(GL_POINTS);
			}
		}

#       define B(c,n) ( (((unsigned int)(c)) >> (n)) & 0xff )

		int cr = B( pnt.C,  0 );
		int cg = B( pnt.C,  8 );
		int cb = B( pnt.C, 16 );
		glColor3ub(cr,cg,cb);

		int x = pnt.P[0]-(OctreeResolutionField>>1);
		int y = pnt.P[1]-(OctreeResolutionField>>1);
		int z = pnt.P[2]-(OctreeResolutionField>>1);
		glVertex3d(x,y,z);
	}
	glEnd();
}

void OctreeImageProxy::TBVOTexturePar::rot(double x, double y, double z, double w, double v[3])
{
	double X,Y,Z,W;
	X = x*sin(w/2);
	Y = y*sin(w/2);
	Z = -z*sin(w/2);
	W = cos(w/2);
    double xx      = X * X;
    double xy      = X * Y;
    double xz      = X * Z;
    double xw      = X * W;

    double yy      = Y * Y;
    double yz      = Y * Z;
    double yw      = Y * W;

    double zz      = Z * Z;
    double zw      = Z * W;
	double mat[4*4];

    mat[0]  = 1 - 2 * ( yy + zz );
    mat[1]  =     2 * ( xy - zw );
    mat[2]  =     2 * ( xz + yw );

    mat[4]  =     2 * ( xy + zw );
    mat[5]  = 1 - 2 * ( xx + zz );
    mat[6]  =     2 * ( yz - xw );

    mat[8]  =     2 * ( xz - yw );
    mat[9]  =     2 * ( yz + xw );
    mat[10] = 1 - 2 * ( xx + yy );

    mat[3]  = mat[7] = mat[11] = mat[12] = mat[13] = mat[14] = 0;
    mat[15] = 1;

	double vv[4]; 
	vv[0]=v[0]; vv[1]=v[1]; vv[2]=-v[2]; vv[3]=1;
	int i,j;
	double vv_[4];
	for(i=0;i<4;i++){
		vv_[i]=0;
		for(j=0;j<4;j++){
			vv_[i]+=mat[j*4+i]*vv[j];
		}
	}
	for(i=0;i<4;i++){
		vv_[i]/=vv_[3];
	}
	v[0]=vv_[0];
	v[1]=vv_[1];
	v[2]=-vv_[2];
}


void OctreeImageProxy::TBVOTexturePar::FromOrientation(const Rotation &rot_)
{
	int i;
	float x = rot_.x;
	float y = rot_.y;
	float z = rot_.z;
	float alpha = rot_.theta;
	double d = sqrt(x*x+y*y+z*z);
	x /= d;
	y /= d;
	z /= d;

	U[0] = 1; U[1] = 0; U[2] = 0;
	V[0] = 0; V[1] = 1; V[2] = 0;
	rot(x,y,z,alpha,U);
	rot(x,y,z,alpha,V);

	for(i=0;i<3;i++){
		if(fabs(U[i])<0.1) continue;
		I=i;
		Is=U[i]<0;
	}
	for(i=0;i<3;i++){
		if(fabs(V[i])<0.1) continue;
		J=i;
		Js=V[i]<0;
	}
}

/* virtual */ BOOL
OctreeImageProxy::Render(const Matrix&, const CullFlag&, OpenGLVisualRenderer* pRenderer, DWORD)
{
    // Octree Rendering Initialization by In Kyu Park 2003.07.04

    // 0: images                FIELD MFNode<class DepthImage> images;
    // 1: octree                FIELD MF<int> octree;
    // 2: octreeResolution      FIELD SF<int> octreeResolution;
    // 3: voxelImageIndex       FIELD MF<int> voxelImageIndex;

    OctreeImage * pOctreeImage = GetOctreeImage();

	if(pOctreeImage == NULL){
		return FALSE;
	}

	pDepthImageField = (MFNode<class DepthImage> *) pOctreeImage->GetField(0);
	(MF<int> *)pOctreeField = (MF<int> *) pOctreeImage->GetField(1);
    pOctreeResolutionField = (SF<int> *) pOctreeImage->GetField(2);
    pVoxelImageIndexField = (MF<int> *) pOctreeImage->GetField(3);    

	OctreeResolutionField = pOctreeResolutionField->GetValue();
    OctreeFieldLength = pOctreeField->GetLength();    
    VoxelImageIndexFieldLength = pVoxelImageIndexField->GetLength();

	glPushMatrix();
	glDisable(GL_LIGHTING);
	glDisable(GL_AUTO_NORMAL);
	glDisable(GL_BLEND);
	
	glEnable(GL_POINT_SMOOTH);

    if( m_nListMade == 0 ) {
        MakeGLList();
    }
    else if ( m_nListMade == -1 ) {
        glPopMatrix();
        return FALSE;
    }

    if( m_nListMade == 0 ) {
        glPopMatrix();
        return FALSE;            // Not fetched yet. So try to fetch again.
    }

    GLfloat diameter_ = 1.0/OctreeResolutionField;

	glScalef(diameter_,diameter_,diameter_);

	if(m_bGLEXTInit){
		float size,minsize,maxsize,a,b,c,thresh;

		MFNode<class DepthImage>*depthimages = (MFNode<class DepthImage>*)pOctreeImage->GetField(0);
		DepthImage *p = depthimages->operator [](0);			
		DepthImageProxy * pDI = (DepthImageProxy *) p->GetRenderingObject();

		pDI->CalculateSplatParametes(pRenderer, diameter_, size, minsize, maxsize, a, b, c, thresh);
		c /= 4.2;
		pDI->SetSplatGLParameters(size, minsize, maxsize, a, b, c, thresh);
		
		if(m_nAnim){
			m_nGLidx = m_nGLidx % m_nNumberOfFrame;

			if(glIsList(m_pLists[m_nGLidx]))
			{
				glCallList(m_pLists[m_nGLidx]);				
				m_nGLidx += 1;	
				Sleep(100);
			}	
		}else{
			glCallList(m_nGlListID);
		}
	}	

	glPopMatrix();
	glEnable(GL_LIGHTING);
	
    return TRUE;
}

// Traverse scene graph, checking for intersection

/* virtual */ BOOL
OctreeImageProxy::Select(OpenGLVisualRenderer* pRenderer, DWORD dwTime, unsigned int& depth)
{
    // I am not sure this would work really 2003.06.24 In Kyu Park
  
#ifdef _DEBUG
  if (pRenderer->CheckGLError("OctreeImageProxy:"))
    TRACE("OctreeImageProxy::Render() [%08x]: ", this);
#endif

  return FALSE;
}

void OctreeImageProxy::Enable (unsigned char *pData)
{
    OctreeImage * pOctreeImage = GetOctreeImage();

	int len_octree, len_voxels;
	unsigned char* octree;
	unsigned char* voxels;
	unsigned char buff[4];

	memcpy(buff,pData,4);
	len_octree=0;
	len_octree=buff[0]<<24;
	len_octree|=buff[1]<<16;
	len_octree|=buff[2]<<8;
	len_octree|=buff[3];

	memcpy(buff,pData+4+len_octree,4);
	len_voxels=0;
	len_voxels=buff[0]<<24;
	len_voxels|=buff[1]<<16;
	len_voxels|=buff[2]<<8;
	len_voxels|=buff[3];

	octree=new unsigned char[len_octree];
	memcpy(octree, pData+4,len_octree);

	voxels=new unsigned char[len_voxels];
	memcpy(voxels,pData+8+len_octree,len_voxels);

    // From here, by In Kyu Park 20030705

    MF<int> * pOctreeField = (MF<int> *) pOctreeImage->GetField(1);
    MF<int> * pVoxelImageIndexField = (MF<int> *) pOctreeImage->GetField(3);    

    for ( int i=0 ; i<len_octree ; i++ ) {
        pOctreeField->SetAt(i, octree[i]);
    }
    for ( i=0 ; i<len_voxels ; i++ ) {
        pVoxelImageIndexField->SetAt(i, voxels[i]);
    }
}

// =============================================================================
// ================== BVO ======================================================
// =============================================================================
#define DEPTHRASTER 65536

vec3d vec3d_From_vec3i(vec3i iv)
{
     vec3d v;
     iforinc(i,3)
          v[i]=iv[i];
     return v;
}
unt ToIntLightNormal3D(vec3d n)
{
     chr xyz[3];
     iforinc(i,3)
          xyz[i]=64.*n[i];
     return *((unt*)xyz)&16777215;
}
unt RGBA_ReverseBytes(unt q)
{
     unt r=q;
     uhr *u=(uhr*)&q;
     uhr *u1=(uhr*)&r;
     u1[0]=u[2];
     u1[2]=u[0];
     return r;
}
unt ToIntNormal3D(vec3d n)
{
     n.Norm();
     chr xyz[3];
     iforinc(i,3)
          xyz[i]=127.*n[i];
     return *((unt*)xyz)&16777215;
}
double ScolarMul(vec3d v1,vec3d v2)
{
     double r=0;
     for(int i=v1.size()-1;i>=0;i--)
          r+=v1[i]*v2[i];
     return r;
}
int ScolarMul(vec3i v1,vec3i v2)
{
     int r=0;
     for(int i=v1.size()-1;i>=0;i--)
          r+=v1[i]*v2[i];
     return r;
}
vec3d operator *(double c,vec3d v){ v*=c; return v; }
vec3d operator /(vec3d v,double c){ v/=c; return v; }
vec3d operator -(vec3d v1,vec3d v2){ v1-=v2; return v1; }
vec3d operator +(vec3d v1,vec3d v2){ v1+=v2; return v1; }
unt ColorPhongLight(
                    unt c,unt n,    // 0..255, -127..127
                    unt c0,unt n0)  // 0..127, -127..-63..63..127
{
     chr *xyz_n=(chr*)&n;
     chr *xyz_n0=(chr*)&n0;
     uhr *rgbt_c=(uhr*)&c;
     uhr *rgbt_c0=(uhr*)&c0;
     int s=(int)xyz_n[0]*(int)xyz_n0[0]+
          (int)xyz_n[1]*(int)xyz_n0[1]+
          (int)xyz_n[2]*(int)xyz_n0[2];
     
     if(!s) return c;
     
     Assert(abs(s)<16384,"<ColorPhongLight> Too large light vector !!");
     
     s/=64;
     
     if(s<0) s=0;
     
     s=s*s/128;
     
     unt r=(unt)((unt)s*rgbt_c0[0]+180*rgbt_c[2]); r>>=8;
     unt g=(unt)((unt)s*rgbt_c0[1]+180*rgbt_c[1]); g>>=8;
     unt b=(unt)((unt)s*rgbt_c0[2]+180*rgbt_c[0]); b>>=8;
     unt t=(unt)((unt)s*rgbt_c0[3])*4; t>>=8;
     
     if(r>255) r=255;
     if(g>255) g=255;
     if(b>255) b=255;
     if(t>255) t=255;
     
     return r|(g<<8)|(b<<16)|(t<<24); 
     
     int x=xyz_n[0]+128,
          y=xyz_n[1]+128,
          z=xyz_n[2]+128;
     return x|(y<<8)|(z<<16);
}
int PackedNormal2DToAbsGrad(unt nrm)
{
     wrd *w=(wrd*)&nrm;
     int inrm=nrm;
     if(inrm==-1) return nrm;
     return (w[0]<0?-w[0]:w[0])+(w[1]<0?-w[1]:w[1]);
}
vec3d vec3d_RotateRelativeVector(vec3d v, vec3d v1, dbl ang)
{
     dbl X,Y,Z,W,x,y,z,w;
     vec3d v_out;
     
     vec3d tmp;
     
     tmp=v1;
     
     x=v1[0],y=v1[1],z=v1[2],w=ang;
     X=x*sin(0.5*w);
     Y=y*sin(0.5*w);
     Z=-z*sin(0.5*w);
     W=cos(0.5*w);
     dbl xx=X*X,xy=X*Y,xz=X*Z,
          xw=X*W,yy=Y*Y,yz=Y*Z,
          yw=Y*W,zz=Z*Z,zw=Z*W;
     
     dbl mat[4*4];
     mat[0]  = 1 - 2 * ( yy + zz );	mat[1]  =     2 * ( xy - zw );	mat[2]  =     2 * ( xz + yw );
     mat[4]  =     2 * ( xy + zw );	mat[5]  = 1 - 2 * ( xx + zz );	mat[6]  =     2 * ( yz - xw );
     mat[8]  =     2 * ( xz - yw );  mat[9]  =     2 * ( yz + xw );  mat[10] = 1 - 2 * ( xx + yy );
     mat[3]  = mat[7] = mat[11] = mat[12] = mat[13] = mat[14] = 0;
     mat[15] = 1;
     
     dbl vv[4];
     vv[0]=v[0]; vv[1]=v[1]; vv[2]=-v[2]; vv[3]=1;
     int i,j;
     double vv_[4];
     for(i=0;i<4;i++)
     {
          vv_[i]=0;
          for(j=0;j<4;j++)
               vv_[i]+=mat[j*4+i]*vv[j];
     }
     
     for(i=0;i<4;i++)
          vv_[i]/=vv_[3];
     
     v_out[0]=vv_[0];
     v_out[1]=vv_[1];
     v_out[2]=-vv_[2];
     
     return v_out;
};
////////////////////////////////////////////////////////////////////////////////
//---------------------------------------------------------------------------
// bytes    // words  // dwords
// E: int23                       0           0         0
// R,G,B,A : unsigned bytes       4,5,6,7     2-3       1
// U,V     : signed words         8,10        4,5       2
// D       : signed word23 bit    12          6-7       3
void UtilityMix_E_RGBA_UV_D  // energy,rgba,uv,d
(
 int* in_int,
 int  all,
 unt  &out_rgba,
 unt  &out_uv,
 int  &out_d
 )
{
     unt se,e,r,g,b,a;
     int u,v,d;
     se=r=g=b=a=u=v=d=0;
     int i;
     uhr *in_uhr=(uhr*)in_int;
     wrd *in_wrd=(wrd*)in_int;
     
     forinc(i,all)
          se+=in_int[i<<2];
     
     unt se_=0;
     
     forinc(i,all)
     {
          e=(in_int[i*4]*256)/se;
          r+=e*in_uhr[i*16+4];
          g+=e*in_uhr[i*16+5];
          b+=e*in_uhr[i*16+6];
          a+=e*in_uhr[i*16+7];
          u+=e*in_wrd[i*8+4];
          v+=e*in_wrd[i*8+5];
          se_+=e;
          d+=e*in_int[i*4+3];
     }
     
     r>>=8,
          g>>=8,
          b>>=8,
          a>>=8,
          u/=256,
          v/=256,
          d/=256;
     
     out_rgba=r|(g<<8)|(b<<16)|(a<<24);
     
     urd uv[2]={u,v};
     out_uv=uv[0]|(uv[1]<<16);
     out_d=d;
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
void UtilityGeneralizedVoxels_ToColors
(
 ArrayInt1D &gvl,            // in: generalized voxel list
 ArrayDouble1D &camdistr,    // in: cameras distribution
 unt light_c,                // in: light color RGBA
 vec3d light_n,              // in: light normal (1.0 - diffuse color component)
 ArrayInt1D &cpnts          // out: colors RGBA
 )
{
     int k=0,i,a,nrm;
     unt e,c;
     unt int_light_n=ToIntLightNormal3D(light_n);
     
     int apnts = gvl[k++],acam=camdistr.All();
     
     cpnts.AllocSet(apnts);
     
     ArrayInt1D idistr;
     idistr.AllocSet(acam);
     
     Statistic s;
     s.CopyFrom(camdistr);
     s.ScaleToRange(0,255);
     
     forinc(i,acam)
          idistr[i]=mx(s[i],1);
     
     forinc(i,apnts)
     {
          a=gvl[k++];
          nrm=gvl[k++];
          
          if(!a) continue;
          
          if(a==1)
          {
               cpnts[i] = ColorPhongLight(gvl[k+1],nrm,light_c,int_light_n);
               k+=2;
               continue;
          }
          
          unt r=0,g=0,b=0,t=0,ae=0;
          
          iforinc(j,a)
          {
               e=idistr[gvl[k]&65535]*(gvl[k]>>16);
               c=gvl[k+1];
               k+=2;
               if(!e) continue;
               ae+=e;
               r+=e*(c&255);
               g+=e*((c>>8)&255);
               b+=e*((c>>16)&255);
               t+=e*((c>>24)&255);
          }
          
          if(!ae)
          {
               Assert(0,"Zero distribution !!!");
               continue;
          }
          
          r/=ae,g/=ae,b/=ae,t/=ae;
          cpnts[i] = ColorPhongLight(((unt)r|(g<<8)|(b<<16)|(t<<24)),nrm,light_c,int_light_n);
     }
}
//-- FILTERING ----------------------------------------------------------------------------------------- 


void UtilityDepthImage_DepthFiltering
(
	int *dth,
	int *out,
	int w,int h,
	int threshold
)
{
	int wh=w*h;
	if(!wh) return;
	int x,y,ix,iy,d0,d1,d2;
	int *d;
	int iscanborder,s,q,islargesmooth;
	int w2=w*2,w3=w*3,w4=w*4,w5=w*5,xy;

	memcpydword(out,dth,w*h);

	for(iy=1;iy<h-5;iy+=3)
	for(ix=1;ix<w-5;ix+=3)
	{
		d=&dth[ix+iy*w];

		d1=d2=-1;

		q=d[0]; if(q>=0) {if(d1<0) d1=d2=q; if(q<d1) d1=q; if(q>d2) d2=q;}
		q=d[2]; if(q>=0) {if(d1<0) d1=d2=q; if(q<d1) d1=q; if(q>d2) d2=q;}
		q=d[4]; if(q>=0) {if(d1<0) d1=d2=q; if(q<d1) d1=q; if(q>d2) d2=q; } d+=w2;
		q=d[0]; if(q>=0) {if(d1<0) d1=d2=q; if(q<d1) d1=q; if(q>d2) d2=q; }
		q=d[2]; if(q>=0) {if(d1<0) d1=d2=q; if(q<d1) d1=q; if(q>d2) d2=q; }
		q=d[4]; if(q>=0) {if(d1<0) d1=d2=q; if(q<d1) d1=q; if(q>d2) d2=q; } d+=w2;
		q=d[0]; if(q>=0) {if(d1<0) d1=d2=q; if(q<d1) d1=q; if(q>d2) d2=q;}
		q=d[2]; if(q>=0) {if(d1<0) d1=d2=q; if(q<d1) d1=q; if(q>d2) d2=q;}
		q=d[4]; if(q>=0) {if(d1<0) d1=d2=q; if(q<d1) d1=q; if(q>d2) d2=q;}

        if(d1==-1)
            continue;

		if(d2-d1<threshold) iscanborder=0;
		else 			    iscanborder=1;

        if(d2-d1<(threshold>>1))   islargesmooth=1;
        else                       islargesmooth=0;

		d0=(d2+d1)>>1;

		for(y=iy+1;y<iy+4;y++)
		for(x=ix+1;x<ix+4;x++)
		{
			register int l=w;

			xy=x+y*l;
			d=&dth[xy];

            if(d[-l-1]<0 || d[-l  ]<0 || d[-l+1]<0 ||
               d[  -1]<0 || d[   0]<0 || d[   1]<0 ||
               d[ l-1]<0 || d[ l  ]<0 || d[ l+1]<0)
            {
                continue;
            }

			if(iscanborder)
			{
				s=0;
                s+=d[   0]<d0?0:1;
				s+=d[   1]<d0?0:1;
				s+=d[  -1]<d0?0:1;
				s+=d[ l  ]<d0?0:1;
				s+=d[-l  ]<d0?0:1;
				s+=d[ l+1]<d0?0:1;
				s+=d[ l+1]<d0?0:1;
				s+=d[ l-1]<d0?0:1;
				s+=d[-l-1]<d0?0:1;
				if(s!=0 && s!=9)
					continue;
			}

            if(!islargesmooth)
			out[xy]=((unt)d[-l-1] +(d[-l]<<1)+d[-l]+
						 (d[-1]<<1)+(d[ 0]<<2)+(d[1]<<1)+
						  d[l-1] +(d[ l]<<1)+d[l+1])>>4;
            else
            {
                if(d[-w2-1]>0 && d[-w2]>0 && d[-w2+1]>0 &&
                   d[-w-2]>0 && d[-2]>0 && d[w+2]>0 &&
                   d[w2-1]>0 && d[w2]>0 && d[w2+1]>0)
                {
        			out[xy]=((unt)
                        d[-w2-1]+d[-w2]+d[-w2+1]+d[-w-2]+d[-2]+d[w+2]+d[w2-1]+d[w2]+d[w2+1]+
                        2*d[-l-1] +3*d[-l]+2*d[-l]+
    	    			3*d[-1]+3*d[0]+3*d[1]+
		        		2*d[l-1] +3*d[ l]+2*d[l+1])>>5;
                }
                else
    			out[xy]=((unt)d[-l-1] +(d[-l])+d[-l]+
	    				  	 (d[-1])+(d[1])+
		    			  	  d[l-1] +(d[ l])+d[l+1])>>3;
            }
		}
   }
}

void UtilityDepthImage_MakeNormals(int *dth,unt *nrm,int w,int h)
{
	int wh=w*h;
    if(!wh) return;
    int i,ix,iy;
    int *d;
    int w2=w*2,w3=w*3,w4=w*4,w5=w*5;
          
    memsetdword((int*)nrm,-1,w*h);
          
    for(iy=1;iy<h-1;iy++)
		for(ix=1;ix<w-1;ix++)
        {
			d=&dth[i=ix+iy*w];
				if(d<0)
                {
					nrm[i]=-1;
                    continue;
                }
                int dx=0,dy=0;
                
                if(d[-1]>=0 || d[1]>=0)
                {
                     if(d[-1]>=0 && d[1]>=0)
                          dx=(d[1]-d[-1])/2;
                     else
                          if(d[-1]>=0) dx=(d[0]-d[-1]);
                          else         dx=(d[1]-d[ 0]);
                }
                if(d[-w]>=0 || d[w]>=0)
                {
                     if(d[-w]>=0 && d[w]>=0)
                          dy=(d[w]-d[-w])/2;
                     else
                          if(d[-w]>=0) dy=(d[0]-d[-w]);
                          else         dy=(d[w]-d[ 0]);
                }
                
                wrd _dx=dx,_dy=dy;
                if(dx!=_dx)
                {
                     if(dx<-32767) _dx=-32767;
                     if(dx>32767) _dx=32767;
                }
                if(dy!=_dy)
                {
                     if(dx<-32767) _dx=-32767;
                     if(dx>32767) _dx=32767;
                }
                
                wrd dxy[2]={_dx,_dy};
                nrm[i]= *((unt*)&dxy[0]);
           }
}

void Statistic::ScaleToRange(dbl r1,dbl r2)
{
     MinMaxCalculate();
     ReScaleRange(MIN,MAX,r1,r2);
}

void Statistic::ReScaleRange(dbl a1,dbl a2,dbl b1,dbl b2)
{
     Assert(a1<=a2 && b1<=b2,"<Statistic::ReScaleRange> Bad source values !");
     if(a1==a2)
     {
          ClearDouble1D(0.5*(b2+b1));
          return;
     }
     Addition(-a1);
     Mul(1./(a2-a1));
     Mul(b2-b1);
     Addition(b1);
     InBound(b1,b2);
}
void Statistic::MinMaxCalculate()
{
     Exit(All(),"no statistic to calculate min/max");
     MIN=MAX=Elem(0);
     IMIN=IMAX=0;
     iforinc(i,All())
     {
          dbl e=Elem(i);
          if(e<MIN) MIN=e,IMIN=i;
          if(e>MAX) MAX=e,IMAX=i;
     }
}
void Statistic::InBound(dbl v1,dbl v2)
{
     iforinc(i,All())
     {
          dbl &q=Elem(i);
          if(q<v1) q=v1; else
               if(q>v2) q=v2;
     }
}
void Statistic::Mul(dbl k)
{
     iforinc(i,All())
          Elem(i)*=k;
}
void Statistic::Addition(dbl k)
{
     iforinc(i,All())
          Elem(i)+=k;
}

vec3d COctreeBVOCamera::Trans(vec3d p){
     return Pos+vec3d_RotateRelativeVector(p,QuadVec,QuadAng);
}
vec3i COctreeBVOCamera::Discrete(vec3d p)
{
     vec3i v;
     v[0]=round((double)(p[0]/RealWidth+0.5)*Width);
     v[1]=round((double)(-p[1]/RealHeight+0.5)*Height);
     v[2]=round(-(double)((p[2]-RealNear)*((dbl)DEPTHRASTER)/(RealFar-RealNear)));
     return v;
}

vec3d COctreeBVOCamera::TransInverse(vec3d p)
{
     return vec3d_RotateRelativeVector(p-Pos,QuadVec,-QuadAng);
}

void CameraImages::PutVoxel( // fill voxel projection in each DI
 vec3d p, // point in 3d space relative DI transforms
 dbl d    // voxel side size
 )
{
     iforinc(i,All())
          Elem(i).PutVoxel(p,d);
}

void CameraImages::ToVDVoxel // return number of writed dwords
(
 vec3d v,          // point in 3d space relative DI transforms
 ArrayInt1D &gvl,  // generalized voxels list
 dbl voxelside,
 int revbytes
 )
{
     int j,i,n,q,c;
     dbl r,zm;
     vec3d nrm=vec3d_Null;
     
#define MINALLOWEDENERGY (8.0)
     
     j=gvl.Add(0); // GV: ALL
     gvl.Add(0);   // GV: Normal
     
     forinc(n,100) // logarithmic level
     {
          zm=(dbl)((int)1<<n);
          
          forinc(i,All())
          {
               COctreeBVOCamera &cam=Elem(i).Cam;
               //vec3d v1=cam.Trans(v);
               int i0;
               
               r=Elem(i).GetVoxel
                    (
                    v, // point in 3d space relative DI camera transform
                    n,  //
                    i0,  // out: index to best point
                    voxelside*zm
                    );
               
               if(Elem(i).NormalImage[n][i0]==-1)
                    r+=4;
               
               if(r>=0 && r<MINALLOWEDENERGY)
               {
                    q=((dbl)256./(2.+r));
                    gvl.Add((q<<16)|i);   // E,C
                    AssertIndexText(i0,Elem(i).ColorImage[n].All(),"<CameraImages::ToVDVoxel> Bad voxel coordinate "+
                         i0+" at "+n +" mipmap");
                    c=Elem(i).ColorImage[n][i0];
                    if(revbytes) c=RGBA_ReverseBytes(c);
                    gvl.Add(c); // C
                    
                    nrm+=((dbl)q)*(Elem(i).GetAbsWeightedNormal(n,i0,voxelside));
                    
                    gvl[j]++;
               }
          }
          
          if(gvl[j]) break;
          
          Assert(n!=50,"<CameraImages::ToVDVoxel> can't find appropriate color for voxel !!!");
     }
     
     gvl[j+1]=ToIntNormal3D(nrm);
}
vec3d CameraImage::GetAbsWeightedNormal(int mm,int i0,dbl voxelside)
{
     vec3d n;
     unt nrm=NormalImage[mm][i0];
     wrd *w=(wrd*)&nrm;
     if(nrm==(unt)-1) return vec3d_Null;
     Assert(nrm!=((unt)-1),"<Camera::GetAbsWeightedNormal> can't make normal from empty");
     dbl u=w[0],v=-w[1];
     dbl k=1./(voxelside*((dbl)DEPTHRASTER));
     
     u*=k,v*=k;
     n.Set(u,v,1.0);
     dbl r=u*u+v*v;
     n=Cam.Trans(n)-Cam.Trans(vec3d_Null);
     n.Norm(2./(1.0+r));
     return n;
}
dbl CameraImage::GetVoxel // return square error
(
 vec3d p, // point in 3d space relative DI camera transform
 int mm,
 int &i0, // out: index to best point
 dbl voxelsize
 )
{
     int x0,y0,z0,x1,y1,x2,y2,z;
     if(!VoxelProjection(  p,mm,  x0,y0,z0,  x1,y1, x2,y2, voxelsize ))
          return -1;
     
     dbl rs=0,r;
     
     i0=x0+y0*Cam.Width;
     
     z=DepthImage[mm][i0];

     r=z-z0;
     rs=fabs(r);
     
     // RASTER -> REAL
     rs/=(dbl)DEPTHRASTER;
     // REAL -> VOXEL
     rs/=voxelsize;
     
     return rs;
}
int CameraImage::VoxelProjection // return 0 if not in camera range
(
    vec3d p,
    int mm,
    int &x0,int &y0,int &z0,
    int &x1,int &y1,
    int &x2,int &y2,
    dbl voxelsize
)
{
    dbl dv=0.5*voxelsize;
    vec3d px=p+vec3d(dv, 0., 0.);
    vec3d py=p+vec3d(0., dv, 0.);
    vec3d pz=p+vec3d(0., 0., dv);

    p=Cam.TransInverse(p);
    px=Cam.TransInverse(px)-p;
    py=Cam.TransInverse(py)-p;
    pz=Cam.TransInverse(pz)-p;

    dbl dx=fmx(fabs(px[0]),fabs(py[0]),fabs(pz[0]));
    dbl dy=fmx(fabs(px[1]),fabs(py[1]),fabs(pz[1]));

    vec3d p1=p+vec3d(-dx,-dy,p[2]);
    vec3d p2=p+vec3d(dx,dy,p[2]);

    int w=Cam.Width>>mm,h=Cam.Height>>mm;

    Assert(Cam.Height>=0 && Cam.Width>=0,"<CameraImage::GetVoxel> Bad camera");

    if(mm>=N || !w || !h) return 0;

    vec3i v=Cam.Discrete(p),
          v1=Cam.Discrete(p1),
          v2=Cam.Discrete(p2);
    x0=v[0],y0=v[1],z0=v[2];
    x1=v1[0],y1=v1[1];
    x2=v2[0],y2=v2[1];

    if(x1>x2) swap(x1,x2);
    if(y1>y2) swap(y1,y2);

    x0>>=mm,y0>>=mm;
    x1>>=mm,y1>>=mm;
    x2>>=mm,y2>>=mm;

    if(x0<0 || y0<0 || x0>=w || y0>=h || z0<0 || z0>DEPTHRASTER)
        return 0;

    if(x1<0) x1=0;
    if(y1<0) y1=0;
    if(x2>=w) x2=w-1;
    if(y2>=h) y2=h-1;

    return 1;
}


void CameraImage::PutVoxel
(
    vec3d p,
    dbl voxelsize
)
{
    int x0,y0,z0,x1,y1,x2,y2,x,y;
    if(!VoxelProjection(  p,0,  x0,y0,z0, x1,y1, x2,y2, voxelsize ))
        return;

    BinaryImage[0][x0+y0*Cam.Width]=-1;
    for(y=y1;y<=y2;y++)
    for(x=x1;x<=x2;x++)
    {
        int &d=DepthImage[0][x+y*Cam.Width];
        if(d<0 || z0<d)
        {
            d=z0;
        }
    }
}

void CameraImage::IntegrityFromCamera()
{
     Assert(Cam.Width>=0 && Cam.Height>=0 && Cam.Width*Cam.Height>=0,
          "<CameraImage::IntegrityFromCamera> Bad Camera");
     int w=Cam.Width,h=Cam.Height,wh=Cam.Width*Cam.Height,i;
     N=lg2roundup(mx(w,h));
     ColorImage.AllocSet2D(N,wh);
     DepthImage.AllocSet2D(N,wh);
     NormalImage.AllocSet2D(N,wh);
     BinaryImage.AllocSet2D(N,wh);
     
     forinc(i,N)
     {
          ColorImage[i].AllocSet(w*h),
               DepthImage[i].AllocSet(w*h),
               NormalImage[i].AllocSet(w*h),
               BinaryImage[i].AllocSet(w*h),
               h=(h+1)>>1;
     }
     ColorImage.ClearInt2D((unt)(128)|(128<<8)|(128<<16)|(255<<24)),
          DepthImage.ClearInt2D(-1),
          NormalImage.ClearInt2D(-1);
     BinaryImage.ClearInt2D(0);
}

void CameraImage::IntegrityFromCamera(COctreeBVOCamera &cam)
{
     Cam.CopyFrom(cam);
     IntegrityFromCamera();
}

void CameraImage::BuildDepthNormalImageMipmaps(dbl voxelsize)
{
    int w=Cam.Width,h=Cam.Height,
        i,i2,j,jj,x,y,x2,y2,
        q,ix,iy,x1,y1,h1,w1,h2,w2,
        dmn;

    dbl dq;

    int ae,e9[9];
    unt c9[9],c;
    unt nrm9[9],nrm;
    int d9[9],d;

    unt cnd[200],acnd; // color,normal,depth

    //---- Fitlering and upquantization ------------------------------------------------------------

    ArrayInt1D Temp;
    Temp.CopyFrom(DepthImage[0]);


    #define THRESHOLD 5

    int thr=((dbl)voxelsize*THRESHOLD*DEPTHRASTER);

    UtilityDepthImage_DepthFiltering
    (
        Temp.Pointer(),
        DepthImage[0].Pointer(),
        w,h,
        thr
    );

    UtilityDepthImage_MakeNormals
    (
        DepthImage[0].Pointer(),
        (unt*)NormalImage[0].Pointer(),
        w,h
    );

    //---- Mip-Mapping -----------------------------------------------------------------------------

    #define MAXDELTAMIP 2
    int mxdeltamip=((dbl)voxelsize*MAXDELTAMIP*DEPTHRASTER);

    h1=h,w1=w;
    iforinc(n,N)
    if(n)
    {
        w2=w1,h2=h1;
        w1=(w1+1)>>1,h1=(h1+1)>>1;
        forinc2(y1,x1,h1,w1)
        {
            i=x1+y1*w;
            x2=x1<<1,y2=y1<<1;
            i2=x2+y2*w;

            j=0,dmn=-1;
            ae=0;
            forinc2(ix,iy,3,3)
            {
                x=x2+ix-1,y=y2+iy-1;
                if(x<0 || y<0 || x>=w2 || y>=h2)
                    continue;
                q=x+y*w;
                if(!BinaryImage[n-1][q])
                    continue;
                c=ColorImage[n-1][q];
                d=DepthImage[n-1][q];
                if(d<0)
                    continue;
                nrm=NormalImage[n-1][q];
                if(nrm==-1)
                    continue;
                e9[j]=PackedNormal2DToAbsGrad(nrm);
                c9[j]=c,
                nrm9[j]=nrm;
                d9[j]=d;
                j++;
                if(dmn<0 || d<dmn)
                    dmn=d;
            }

            if(dmn<0)
            {
                ColorImage [n][i]=(unt)128|(128<<8)|(128<<16);
                NormalImage[n][i]=-1;
                DepthImage [n][i]=-1;
                BinaryImage[n][i]=0;
                continue;
            }

            acnd=0;
            forinc(jj,j)
            if(d9[jj]-dmn<MAXDELTAMIP)
            {
                dq=e9[jj];//dq*=dq;dq/=4;
                cnd[acnd++]=((dbl)262144/((dbl)2.0+dq));
                cnd[acnd++]=c9[jj];
                cnd[acnd++]=nrm9[jj];
                cnd[acnd++]=d9[jj];
            }

            Assert(acnd,"<CameraImage::BuildDepthNormalImageMipmaps> a set of {color,normal,depth} is empty !");
            BinaryImage[n][i]=-1;

            unt c,nrm;
            int d;
            UtilityMix_E_RGBA_UV_D  // energy,rgba,uv,d
            (
                (int*)cnd,
                acnd>>2,
                c,nrm,d
            );
            ColorImage [n][i]=c;
            NormalImage[n][i]=nrm;
            DepthImage [n][i]=d;
        }
    }

    h1=h,w1=w;
}

int OctreeImageProxy::UtilityBVO_BreadthFirstToVoxels( unsigned int &bvo_ind )
{
	 if(OctreeFieldLength <= bvo_ind ) return 0;    	
	
	 unsigned char temp;
	 int cnt = 0;		
	 int i = 0;

     IBRArray<vec3i> tmp;
     vec3i v;
     int j0=0,a1=1,a,j,k,b;
     
     tmp.Add(vec3i(0,0,0));

     for( i = 0; i < OctreeHeight; i++ )
     {
          a=a1;
          a1=0;
          for( j = 0; j < a; j++ )
          {               
			   b = (*pOctreeField)[bvo_ind++];
               
               v = tmp[j0+j];
               for( k = 0; k < 8; k++ )
                    if(b&(1<<k)){
                         tmp.Add(vec3i((v[0]<<1)|(k&1),(v[1]<<1)|((k>>1)&1),(v[2]<<1)|(k>>2)));
                         a1++;
                    }
          }
          j0 += a;
     }

     m_vxls.AllocSet(a1);
     for( i = 0; i < a1; i++ )
          m_vxls[i] = tmp[j0+i];
     return 1;
}	

void OctreeImageProxy::UtilityCameraImages_VoxelsToGeneralizedVoxels( 
     int revbytes , int idx
)
{
    int i,j,ix,iy;
    CameraImages cimgs;

    //--- copy images, allocate normals and fills depths to -1

    OctreeImage * pOctreeImage = GetOctreeImage(); 
	MFNode<class DepthImage>*depthimages =  (MFNode<class DepthImage>*)pOctreeImage->GetField(0);

    cimgs.AllocSet( NumOfImages );
    for ( i = 0; i < NumOfImages; i++ )
    {
		 DepthImage *p = depthimages->operator [](i);
		 SFDepthTextureNode *depthtexture = (SFDepthTextureNode *)p->GetField(0);
		 SimpleTexture *st = (SimpleTexture *)depthtexture->GetNode();
		 SimpleTextureProxy *stp = (SimpleTextureProxy *)st->GetRenderingObject();

         cimgs[i].IntegrityFromCamera(m_cms[i]);         
		 int w = stp->GetWidth();
		 int h = stp->GetHeight();

         int wh = w*h;
         cimgs[i].ColorImage[0].AllocSet(wh);		          
		 int bps = stp->GetWidth()*stp->GetColorBytePerPixel();         
		 int bpp = stp->GetColorBytePerPixel();

         int x,y,q;
         int wy;
		 unsigned char *b;		 

		 if(m_nAnim){			 
			 int MovieBufIdx = idx*wh*bpp;

              for( y = 0; y < h; y++ ){
                   wy = w * y;        
				  				   
				   b = &(stp->m_pOTMovieColorBuf)[MovieBufIdx+bps*(h-y-1)];				   
			
                   for( x = 0, q = 0; x < w; x++, q+=bpp )
                   {
                        uhr b1[4]={b[q+2],b[q+1],b[q+0],0};
                        cimgs[i].ColorImage[0][ x + wy ] = *((int*)b1);						
                   }
              }
         }else{
			  unsigned char * texture = stp->m_pOTColorBuf;

              for( y = 0; y < h; y++ ){
                   wy = w * y;
				   b = texture + bps*(h-y-1);				   
				   
                   for( x = 0, q = 0; x < w; x++, q+=bpp )
                   {
                        uhr b1[4]={b[q+2],b[q+1],b[q+0],0};
                        cimgs[i].ColorImage[0][ x + wy ] = *((int*)b1);
                   }
              }
         }
    }
    double dv=1./((double)OctreeResolutionField);
    vec3d v_center=vec3d(0.5+dv/2,0.5+dv/2,0.5+dv/2),v;
    
    for( i = 0; i < m_vxls.All(); i++ )
    {
         v = vec3d_From_vec3i(m_vxls[i]);
         v /= (double)OctreeResolutionField;
         v -= v_center;
         cimgs.PutVoxel(v,dv);
    }
        
    for( i = 0; i < NumOfImages; i++ )
         cimgs[i].BuildDepthNormalImageMipmaps(dv);

    m_gvl.Reset();
    m_gvl.Add(m_vxls.All());
    for( i = 0; i < m_vxls.All(); i++ )
    {
        v=vec3d_From_vec3i(m_vxls[i]);
        v/=(double)OctreeResolutionField;
        v-=v_center;
        cimgs.ToVDVoxel( // return number of writed dwords
            v,
            m_gvl,
            dv,
            revbytes);  // generalized voxels
    }

}
void OctreeImageProxy::UtilityGeneralizedVoxels_FillColorPoints(
 ArrayDouble1D &camdistr,    // in: cameras distribution
 unsigned int light_c,       // in: light color RGBA
 vec3d light_n,              // in: light normal (1.0 - diffuse color component)
 PointsCC &pnts              // in/out: fill colors
 )
{
     ArrayInt1D cpnts;
     UtilityGeneralizedVoxels_ToColors
          (
          m_gvl,  // in: generalized voxel list
          camdistr, // in: cameras distribution
          light_c,      // in: light color RGBA
          light_n,    // in: light normal (1.0 - diffuse color component)
          cpnts   // out: colors RGBA
          );
     iforinc(i,pnts.All())
          pnts[i].C = RGBA_ReverseBytes(cpnts[i]);
}

		