// -*- c++ -*-

// This software module was originally developed by In Kyu Park
// (SAMSUNG AIT) in the course of development of the MPEG-4 AFX
// standard (ISO/IEC 14496-16).  This software module is an implementation
// of a part of one or more MPEG-4 AFX tools as specified by the MPEG-4
// AFX standard.  ISO/IEC gives users of the MPEG-4 AFX standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 AFX
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// AFX conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// Copyright 2003
#ifdef SAIT_CONFORMANCE_2003

#ifndef __OCTREEIMAGEPROXY_H__
#define __OCTREEIMAGEPROXY_H__

#include "../AFXMediaObjectProxy.h"
#include "IBRArray.h"

#define NGRAD 8

class OctreeImage;
class OpenGLVisualRenderer;

template < class tp, int sz >
struct VecNd
{
	VecNd(){
		This()=0;}
	VecNd(tp x,...)
	{
		va_list ap;
		va_start(ap,x);
		C[0]=x;
		for(int i=1;i<sz;i++)
			C[i]=va_arg(ap,tp);
		va_end(ap);
	}
	VecNd& operator=(VecNd v){
		for(int i=0;i<sz;i++)
			C[i]=v[i];return *this; }
	VecNd& operator=(tp c){
		for(int i=0;i<sz;i++)
			C[i]=c;return *this; }
	tp& operator [](int i){
		Assert(i>=0 && i<sz,"<VecND> []: "+i+" V "+sz);
		return C[i];}
	VecNd& This(){return *this;}
	virtual void Set(tp x,...)
	{
		va_list ap;
		va_start(ap,x);
		C[0]=x;
		for(int i=1;i<sz;i++)
			C[i]=va_arg(ap,tp);
		va_end(ap);
	}
	
	void operator +=(VecNd v){ for(int i=sz-1;i>=0;i--) C[i]+=v[i];}
	void operator -=(VecNd v){ for(int i=sz-1;i>=0;i--) C[i]-=v[i];}
	void operator *=(tp c){  for(int i=sz-1;i>=0;i--) C[i]*=c;}
	void operator /=(tp c){  if(c) for(int i=sz-1;i>=0;i--) C[i]/=c;}
	
	VecNd& Norm(){
		tp r=Len();
		if(r) This()/=r;
		return *this;}
	int size() const { return sz; }
	VecNd& Norm(tp r){
		VecNd v=This();
		tp r0=v.Len();
		if(r0)	This()=(r*v)/r0;
		return *this;}
	tp Len(){
		return sqrt(LenSqr());}
	tp LenSqr(){
		tp r;
		r=ScolarMul(This(),This());
		Assert(r>=0,"<VecNd::Len>");
		return r;}
	
	friend VecNd operator +(VecNd,VecNd);
	friend VecNd operator -(VecNd,VecNd);
	friend VecNd operator *(tp,VecNd);
	friend VecNd operator /(VecNd,tp);
	
protected:
	tp C[sz];
};

typedef VecNd< double, 3 > vec3d;
typedef VecNd< int, 3 >    vec3i;

struct ArrayInt1D:
public IBRArray<int>
{
	void ClearInt1D(int c=0){
		iforinc(i,All()) Elem(i)=c;}
	void CopyFrom(ArrayInt1D &g){
		AllocSet(g.Size());
		iforinc(i,__A)
			Elem(i)=g[i];
		SetPoint(g.All());}
};
struct ArrayInt2D:
public IBRArray<ArrayInt1D>
{
	void AllocSet2D(int d1,int d2){
		AllocSet(d1);
		iforinc(i,d1) Elem(i).AllocSet(d2);}
	void ClearInt2D(int c){
		iforinc(i,All()) Elem(i).ClearInt1D(c);}
};
struct ArrayDouble1D:
public IBRArray<double>
{
	ArrayDouble1D():IBRArray<double>(){};
	ArrayDouble1D(int a):IBRArray<double>(a){};
	
	void CopyFrom(ArrayDouble1D &g){
		AllocSet(g.Size());
		iforinc(i,__A)
			Elem(i)=g[i];
		SetPoint(g.All());}
	
	void ClearDouble1D(double c=0){
		iforinc(i,All()) Elem(i)=c;}
};


struct COctreeBVOCamera
{
	int IsOrth;
	double RealWidth,RealHeight,RealNear,RealFar;
	int Width,Height;
	
	vec3d Pos;
	vec3d QuadVec;
	dbl QuadAng;
	
	void Set( vec3d pos, vec3d quadvec, double quadand, double nr, double fr,
		double rw, double rh,
		int w,int h,int isorth=1)
	{
		Pos=pos,QuadVec=quadvec,QuadAng=quadand;
		RealFar=fr,RealNear=nr,RealWidth=rw,RealHeight=rh,Width=w,Height=h,IsOrth=isorth;
		Assert(RealNear<RealFar,"<COctreeBVOCamera::Set> Integrity error");
	}
	void CopyTo(COctreeBVOCamera& cam){
		cam.Set(Pos,QuadVec,QuadAng,RealNear,RealFar,RealWidth,RealHeight,Width,Height,IsOrth);}
	void CopyFrom(COctreeBVOCamera& cam){
		cam.CopyTo(*this);}
	
	vec3i Discrete(vec3d p);
	
	
	vec3d Trans(vec3d p);
	vec3d TransInverse(vec3d);
};


class CameraImage
{
public:	
	// array2D: [mipmap][xy]
	ArrayInt2D  ColorImage;
	ArrayInt2D  DepthImage;
	ArrayInt2D  NormalImage;
	ArrayInt2D  BinaryImage; // is current pixel really used ?
	
	COctreeBVOCamera Cam;
	int N;
	
	
	void IntegrityFromCamera();
	void IntegrityFromCamera(COctreeBVOCamera &cam);
	
	void PutVoxel( // fill voxel projection in each DI
		vec3d p,   // center of voxel
		double voxelsize);  // voxel side size
	
	double GetVoxel( // return square error
		vec3d p, // point in 3d space relative DI camera transform
		int mm,
		int& i0, // out: index to best point
		double voxelsize
		);
	
	vec3d GetAbsWeightedNormal( int mm, int i0, double voxelsize );
	
	void BuildDepthNormalImageMipmaps( double voxelsize );
	
private:
	int VoxelProjection( // return 0 if not in camera range
		vec3d p,
		int mm,
		int &x0,int &y0,int &z0,
		int &x1,int &y1,
		int &x2,int &y2,
		double voxelsize
		);
};
class CameraImages: public IBRArray<CameraImage>
{
public:
	void PutVoxel( // fill voxel projection in each DI
		vec3d p, // point in 3d space relative DI transforms
		double d);  // voxel side size
	
	void ToVDVoxel( // return number of writed dwords ?! err
		vec3d v,     // point in 3d space relative DI transforms
		ArrayInt1D &gvl, // generalized voxels list
		double d,  // voxel side size
		int revbytes);
};

struct PointXYZRGBA
{
	vec3i P;
	int C;
	PointXYZRGBA(vec3i p,int c):P(p),C(c){};
	PointXYZRGBA():P(vec3i(0,0,0)),C(0){};
	PointXYZRGBA(int x,int y,int z,int c)      :P(vec3i(x,y,z)),C(c){};
	PointXYZRGBA(int x,int y,int z,int c,int a):P(vec3i(x,y,z)),C(0){SetColorAlpha(c,a);};
	PointXYZRGBA(vec3i p,int c,int a)          :P(p),C(0){SetColorAlpha(c,a);};
	void Set(vec3i p,int c){P=p;C=c;};
	int SetColorAlpha(int c,int a){ return (C=(unsigned int)(c&0x00FFFFFF)|((unsigned int)(a&0xFF)<<24)); };
	PointXYZRGBA& operator=(PointXYZRGBA pnt){ P=pnt.P;C=pnt.C; return *this;};
	unsigned char& Alpha(){ return ((unsigned char*)&C)[3]; }
	int  Color(){ return C&0x00FFFFFF; }
	unsigned char& Cam(){ return ((unsigned char*)&C)[3]; }
};

typedef PointXYZRGBA PointCC; // coordinate + color

typedef IBRArray<PointXYZRGBA> PointsXYZRGBA;
typedef PointsXYZRGBA PointsCC; // coordinate + color
struct PPointsCC
{
	mutable PointsCC *pnts;
	PPointsCC(){
		pnts = new PointsCC;
	}
	PPointsCC( const PPointsCC &p )
		:pnts(NULL)
	{
		*this = p;
	}
	const PPointsCC& operator = ( const PPointsCC &p ){
		delete pnts;
		pnts = p.pnts;
		p.pnts = NULL;
		return *this;
	}
	~PPointsCC(){
		delete pnts;
	}
	PointsCC& operator ()(void){
		return *pnts;
	}
};

class Statistic:
public ArrayDouble1D
{
public:
	int
		IMIN,IMAX; // number of max and min elements
	dbl M,         // expectation value (MeanValue)
		S,         // summa
		MIN,MAX,   // min,max
		RMS,       // root square mean value
		RMSE,      // sqrt(sqr)
		RMS2,      // square mean value
		RMSE2,     // sqr
		ME;        // mean error
	
	Statistic():ArrayDouble1D(){}
	Statistic(int a):ArrayDouble1D(a){}
	
	void Mul(dbl k);
	void Addition(dbl k);
	void MinMaxCalculate();
	void ReScaleRange(dbl a1,dbl a2,dbl b1,dbl b2);
	void InBound(dbl,dbl);
	void ScaleToRange(dbl r1=0.0,dbl r2=1.0);
};


/// OctreeImage renderer node

class OctreeImageProxy
  : public AFXMediaObjectProxy
{
protected:
	int OctreeHeight;  // height of octree

public:
	int m_nListMade;   // -1 : Null node (Cannot define), 0 : Not defined, 1 : Already defined
	int IsListMade() {return m_nListMade;}

	BOOL m_bGLEXTInit;
   	int m_nGlListID;
    int m_nFetchedImage;
	int NumOfImageTexture;
	
	unsigned char * m_pColorMap;
	//JOOYEON_ANIMATION
	bool m_nAnim;
	int *m_pLists;	
	int m_nGLidx, m_nNumberOfFrame;
	//-

private:
   	struct TBVOTexturePar
	{
		double O[3];
		double U[3];
		double V[3];
		
		int W; //width
		int H; //height
		int bytesperscan;
		int bytesperpixel;
		int I,J; //texture coordinates
		int Is,Js;
		void rot(double x, double y, double z, double w, double v[3]);
		void FromOrientation(const Rotation &);
	};
	
	struct TBVOLeaf
	{
		signed short X,Y,Z;  // coordinate
		unsigned short T;      // texture number		
		TBVOLeaf(){};
		TBVOLeaf(int x,int y,int z,int t){Set(x,y,z,t);}
		void Set(signed short x,signed short y,signed short z,signed short t){X=x;Y=y;Z=z;T=t;}
	};
	
	TBVOTexturePar *TexturesTopology;
	TBVOLeaf *Leafs;
	
	GLdouble prj[NGRAD+1][NGRAD+1][NGRAD+1][3];
	GLint	splatsize[NGRAD][NGRAD][NGRAD];

	int m_bTBVO;
	int m_iGlListID;
	int TotalNumOfLeafs;		

//2003.10.7 J.Y.HAN
	void ToPoints(int idx){ m_bTBVO ? ToPointsTBVO(idx):ToPointsBVO(idx);}
	void ToPointsTBVO(int idx);
//-
	unsigned long m_cur_byte;
	int animation;
	int m_prev_byte_octree;
	int m_prev_byte_img_ind;
	int m_cur_byte_octree;
	int m_cur_byte_img_ind;	
	unsigned int bvo_ind;

	// for BVO
	IBRArray<COctreeBVOCamera> m_cms;
	IBRArray<vec3i>            m_vxls;
	ArrayInt1D                 m_gvl;	
	IBRArray<PPointsCC> frames;
	unsigned int m_cur_frame;

	int UtilityBVO_BreadthFirstToVoxels( unsigned int &bvo_ind );
	void UtilityCameraImages_VoxelsToGeneralizedVoxels( int revbytes , int idx );
	void OctreeImageProxy::UtilityGeneralizedVoxels_FillColorPoints(
		ArrayDouble1D &camdistr,    // in: cameras distribution
		unsigned int light_c,       // in: light color RGBA
		vec3d light_n,              // in: light normal (1.0 - diffuse color component)
		PointsCC &pnts              // in/out: fill colors
		);
	void ToPointsBVO(int idx);		//2003.10.7 J.Y.HAN
	//-for BVO

public:
	MFNode<class DepthImage> * pDepthImageField;
    MF<int> * pOctreeField;
    SF<int> * pOctreeResolutionField;
    MF<int> * pVoxelImageIndexField;   

	int OctreeResolutionField;
	int NumOfImages;
    int OctreeFieldLength;
    int VoxelImageIndexFieldLength;

	int MakeGLList(void);
	int TransXYZToUV(const TBVOTexturePar &tp, int x,int y,int z);
	void ToLeafs(int cl);
	int isTBVO(){ return !!pVoxelImageIndexField; }
	void MakeTBVOTopology(void);
	int MakeBVOData(int idx);
  /// Constructor

	OctreeImageProxy(OctreeImage* pOctreeImage);

    virtual ~OctreeImageProxy()
    {		
		if(TexturesTopology != NULL)
			delete [] TexturesTopology;

		if(Leafs != NULL)
			delete [] Leafs;		
    }

  /// Render object
  virtual BOOL  RenderFromStream(MediaStream* poStream, int id, DWORD dwTime);
  virtual BOOL  RenderFromBuffer(SFString *buffer, int id);

  /// Render object description
  virtual void		RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD dwTime);

  /// Decode AFX bitstream

  void Enable (unsigned char *pData);

  /// Render object
  virtual BOOL Render(const Matrix&, const CullFlag&, OpenGLVisualRenderer* pRenderer, DWORD);
  
  /// Traverse scene graph, checking for intersection
  virtual BOOL		Select(OpenGLVisualRenderer* pRenderer, DWORD dwTime, unsigned int& depth);

  // Stefano Battista
  // moved methods from Box to OctreeImageProxy
public:
  OctreeImage * GetOctreeImage() const
  { return (OctreeImage *) GetMediaObject(); };
  /// Get size of box
  /// Get bounding box
};

#include "OctreeImageProxy.i"

#endif // __OCTREEIMAGEPROXY_H__

#endif //SAIT_CONFORMANCE_2003
