// -*- c++ -*-

// This software module was originally developed by In Kyu Park
// (SAMSUNG AIT) in the course of development of the MPEG-4 AFX
// standard (ISO/IEC 14496-16).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// For more information or to receive an up to date version of this module,
// mail karlo@nta.no or check <http://televr.fou.telenor.no/compositor/>.
//
// Copyright 1997, 1998.

#ifdef SAIT_CONFORMANCE_2003

#include "../../OpenGl.h"
#include "PointTextureProxy.h"
#include "../../base/OpenGLVisualRenderer.h"
// 2004-07-15, Shinjun {
#include "../../IM1Decoders/AFX/jwafxstreamdecoder.h"
#include "Stream.h"
// 2004-07-15, Shinjun }

// Render object description
int PointTextureProxy::CreateGlList(double nearPlane, double farPlane, bool bRender) 
{
    PointTexture * pT = GetPointTexture();
    MF<Color3f> * pColorField = (MF<Color3f> *) pT->GetField(0);
    MF<int> * pDepthField = (MF<int> *) pT->GetField(1);

    int nColor = pColorField->GetLength();

	m_dNearPlane = nearPlane;
	m_dFarPlane = farPlane;

	glPushMatrix();
	glDisable(GL_LIGHTING);
	glDisable(GL_AUTO_NORMAL);
	glDisable(GL_BLEND);
	glEnable(GL_SMOOTH);

	if(bRender == false)
		glScalef(1.0/m_nWidth,1.0/m_nHeight,1.0/m_nMaxDepth);
	else
		glScalef(15.0/m_nWidth,15.0/m_nHeight,15.0/m_nMaxDepth);
	double dx,dy,dz;
	if(!m_pDepthImageProxy->m_bGLEXTInit){
		dx=m_nWidth/double(NGRAD);
		dy=m_nHeight/double(NGRAD);
		dz=m_nMaxDepth/double(NGRAD);
        
        double diag=min(min(dx,dy),dz)/(1.24);
		////////////////////////////////////

		GLdouble model_matrix[16];
		GLdouble proj_matrix[16];
		GLint viewport[4];
		
		glGetDoublev(GL_MODELVIEW_MATRIX, model_matrix);
		glGetDoublev(GL_PROJECTION_MATRIX, proj_matrix);
		glGetIntegerv(GL_VIEWPORT, viewport);
		int x_,y_,z_;
		for(x_=0;x_<=NGRAD;x_++)
			for(y_=0;y_<=NGRAD;y_++)
				for(z_=0;z_<=NGRAD;z_++){
					gluProject(
						x_*dx-m_nWidth/2,y_*dy-m_nHeight/2,z_*dz-(m_nMaxDepth),
						model_matrix, proj_matrix,  viewport,
						&m_pProjection[x_][y_][z_][0],
						&m_pProjection[x_][y_][z_][1],
						&m_pProjection[x_][y_][z_][2]);
				}
#define Q(a) ((a)*(a))
#define MAX(a,b) ((a)>(b)?(a):(b))
#define LEN(a,b) (Q(a[0]-b[0])+Q(a[1]-b[1]))
				for(z_=0;z_<NGRAD;z_++){
					for(y_=NGRAD-1;y_>=0;y_--){
						for(x_=0;x_<NGRAD;x_++){
							double len[4];
							len[0]=LEN( m_pProjection[x_+0][y_+0][z_+0], m_pProjection[x_+1][y_+1][z_+1]);
							len[1]=LEN( m_pProjection[x_+1][y_+0][z_+0], m_pProjection[x_+0][y_+1][z_+1]);
							len[2]=LEN( m_pProjection[x_+0][y_+1][z_+0], m_pProjection[x_+1][y_+0][z_+1]);
							len[3]=LEN( m_pProjection[x_+0][y_+0][z_+1], m_pProjection[x_+1][y_+1][z_+0]);
							double d=sqrt(MAX(MAX(len[0],len[1]), MAX(len[2],len[3])))/diag;
							m_pSplatSize[x_][y_][z_]=d+0.6;
							if(m_pSplatSize[x_][y_][z_]<1) m_pSplatSize[x_][y_][z_]=1;
						}
					}
				}
#undef Q
#undef MAX
#undef LEN

	}
	int di,ci;
	unsigned int i;
	int x=0,y=0;
	int splt=1;

	if(!m_pDepthImageProxy->m_bGLEXTInit) glPointSize(splt);

	glBegin(GL_POINTS);

	for( ci=0, di=0; ci<nColor; ){
        unsigned long k = (*pDepthField)[di]; 
        di++;
		for(i=0;i<k;i++){
            glColor3ub((*pColorField)[ci].r*255, (*pColorField)[ci].g*255, (*pColorField)[ci].b*255 );
            ci++;

            double z = (*pDepthField)[di]; 
            di++;
			
            if( z < 0 ) {
				z+=(double(1<<30))*4;
			}
			double xx = 0;
			double yy = 0; 
			double zz = 0;
			xx = x-m_nWidth/2;
			yy = y-m_nHeight/2;
			zz = z-(m_nMaxDepth);
			//*//
			if(bRender == true)
				zz += (double)m_nMaxDepth/2;
			//*/
			if(nearPlane!=-1 && farPlane!=-1)
				{
				double ddd=(farPlane-((double)z/(m_nMaxDepth))*(farPlane-nearPlane))/farPlane;
				xx*=ddd;
				yy*=ddd;
				}
			
			if(!m_pDepthImageProxy->m_bGLEXTInit){
				int &splt_=m_pSplatSize[int((xx+m_nWidth/2)/dx)][int((yy+m_nHeight/2)/dy)][int((zz+(m_nMaxDepth))/dz)];
				if(splt!=splt_){
					splt=splt_;
					glEnd();
					glPointSize(4);
					glBegin(GL_POINTS);
				}
			}
			glPointSize(splt);             // Should be removed later   2003.06.27 In Kyu Park
			glVertex3d(xx, yy, zz);
		}
		x++;
		if(x==m_nWidth) {x=0; y++;}
	}
	glEnd();

	/*//
            glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
            int mfac = 1;
            float x_side = m_nWidth / 2.0f;
            float y_side = m_nWidth / 2.0f;
            float z_side = m_nMaxDepth;
            x_side *= mfac;
            y_side *= mfac;
            z_side *= mfac;

            glBegin(GL_QUADS);
            glDisable(GL_LIGHTING);
            glColor3f(1, 1, 0);
            glNormal3f(0.0f, 0.0f, -1.0f);
            glVertex3f( x_side, -y_side, -z_side); // 1
            glVertex3f(-x_side, -y_side, -z_side); // 2
            glVertex3f(-x_side,  y_side, -z_side); // 3
            glVertex3f( x_side,  y_side, -z_side); // 4

            glNormal3f(0.0f, 1.0f, 0.0f);
            glVertex3f(-x_side,  y_side,  0);      // 5
            glVertex3f( x_side,  y_side,  0);      // 6
            glVertex3f( x_side,  y_side, -z_side); // 4
            glVertex3f(-x_side,  y_side, -z_side); // 3

            glNormal3f(0.0f, 0.0f, 1.0f);
            glVertex3f(-x_side, -y_side,  0);       // 7
            glVertex3f( x_side, -y_side,  0);       // 8
            glVertex3f( x_side,  y_side,  0);       // 6
            glVertex3f(-x_side,  y_side,  0);       // 5

            glNormal3f(-1.0f, 0.0f, 0.0f);           
            glVertex3f(-x_side, -y_side, -z_side);  // 2
            glVertex3f(-x_side, -y_side,  0);       // 7
            glVertex3f(-x_side,  y_side,  0);       // 5
            glVertex3f(-x_side,  y_side, -z_side);  // 3

            glNormal3f(0.0f, -1.0f, 0.0f);
            glVertex3f( x_side, -y_side,  0);       // 8 
            glVertex3f(-x_side, -y_side,  0);       // 7
            glVertex3f(-x_side, -y_side, -z_side);  // 2
            glVertex3f( x_side, -y_side, -z_side);  // 1

            glNormal3f(1.0f, 0.0f, 0.0f);
            glVertex3f( x_side, -y_side,  0);       // 8 
            glVertex3f( x_side, -y_side, -z_side);  // 1 
            glVertex3f( x_side,  y_side, -z_side);  // 4
            glVertex3f( x_side,  y_side,  0);       // 6
            glEnd();
            glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
	//*/

	glPopMatrix();
	glEnable(GL_LIGHTING);

    return 1;
}

// 2004-07-15, Shinjun {
/* virtual */ BOOL
PointTextureProxy::Render(const Matrix&, const CullFlag&, OpenGLVisualRenderer* pRenderer, DWORD)
{
// variables: fields, indices
	MF<Color3f>* pColorField = NULL;
    MF<int>* pDepthField = NULL;
    SF<int>* pDepthNbBits = NULL;
    SF<int>* pHeight = NULL;
    SF<int>* pWidth = NULL;
	Color3f oRGB;
	int nX_max = 0;			// maximum in X-axis
	int nY_max = 0;			// maximum in Y-axis
	int nZ_max = 0;			// maximum in Z-axis
	double dX = 0;
	double dY = 0;
	double dZ = 0;
	int splt = 1;
	int nIndexOfDepth = 0;
	int nIndexOfColor = 0;
	int i = 0;
	int j = 0;
	int k = 0;

    PointTexture *pPointTexture = GetPointTexture();

	if(pPointTexture == NULL)
		return FALSE;

	pColorField = (MF<Color3f>*)pPointTexture->GetField(0);
    pDepthField = (MF<int>*)pPointTexture->GetField(1);
    pDepthNbBits = (SF<int>*)pPointTexture->GetField(2);
    pHeight = (SF<int>*)pPointTexture->GetField(3);
    pWidth = (SF<int>*)pPointTexture->GetField(4);

	nX_max = *pWidth-1;							// maximum in X-axis
	nY_max = *pHeight-1;						// maximum in Y-axis
	nZ_max = (int)pow(2, *pDepthNbBits+1)-1;	// maximum in Z-axis

	m_nWidth = *pWidth;
	m_nHeight = *pHeight;
	m_nMaxDepth = nZ_max;

	CreateGlList(m_dNearPlane, m_dFarPlane, true);

	/*//
	dX = nX_max/double(NGRAD);
	dY = nY_max/double(NGRAD);
	dZ = nZ_max/double(NGRAD);

	glPushMatrix();
	glDisable(GL_LIGHTING);
	glDisable(GL_AUTO_NORMAL);
	glDisable(GL_BLEND);
	glEnable(GL_SMOOTH);
	glScalef(1.0/nX_max, 1.0/nY_max, 1.0/nZ_max);

	// render: the depth data
	for(j = 0; j <= nY_max; j++)
		{
		for(i = 0; i <= nX_max; i++)
			{
			int nSize = (*pDepthField)[nIndexOfDepth++];

			for(k = 0; k < nSize; k++)
				{
	            glColor3ub((*pColorField)[nIndexOfColor].r*255, (*pColorField)[nIndexOfColor].g*255, (*pColorField)[nIndexOfColor].b*255);
				nIndexOfColor++;

				double dZ = (double)(*pDepthField)[nIndexOfDepth++];
				if(dZ < 0)
					dZ += (double(1<<30))*4;
				
				double xx=i-nX_max/2;
				double yy=j-nY_max/2;
				double zz=dZ-(nZ_max);
				double ddd=(m_dFarPlane-((double)dZ/(nZ_max))*(m_dFarPlane-m_dNearPlane))/m_dFarPlane;
				xx*=ddd;
				yy*=ddd;
				if(!m_pDepthImageProxy->m_bGLEXTInit){
					int &splt_=m_pSplatSize[int((xx+nX_max/2)/dX)][int((yy+nY_max/2)/dY)][int((zz+nZ_max)/dZ)];
					if(splt!=splt_){
						splt=splt_;
						glEnd();
						glPointSize(4);
						glBegin(GL_POINTS);
						}
					}
				glPointSize(splt);             // Should be removed later   2003.06.27 In Kyu Park
				glVertex3d(xx, yy, zz);
				}
			}
		}

	glEnd();
	glPopMatrix();
	glEnable(GL_LIGHTING);
	//*/

    return TRUE;
}

BOOL PointTextureProxy::RenderFromStream(MediaStream* poStream, int id, DWORD dwTime)
	{
	BYTE * pData = NULL;
	int nSize = 0;
	BOOL bEOU;
	int nStatus;
	BOOL bSkip = TRUE;
	BOOL bWait = FALSE;
	long lDelta = 0;
	BOOL bDontClose = TRUE;
	DWORD rdvTime;
	
	pData = (unsigned char *) poStream->Fetch(
		nSize, rdvTime, bEOU, nStatus, bSkip, bWait, lDelta, bDontClose);

	if(pData)
		{
		AFXPointTextureDecoder * pDecoder = new AFXPointTextureDecoder;
		
		// Set OctreeImage contents
		pDecoder->SetValues(pData);
		
		// Test
		pDecoder->Write("AFX-PointTextureURL.debug.vrml", id);
		
		Enable(pData);
		
		return TRUE;
		}
	return FALSE;
	}

BOOL PointTextureProxy::RenderFromBuffer(SFString *pBuffer, int id)
	{
	BYTE * pData = NULL;
	int nSize = 0;
	
	AFXPointTextureDecoder* pDecoder = new AFXPointTextureDecoder;
	
	// Decode bitstream
	pDecoder->Decode((LPBYTE) pBuffer->GetValue(), pBuffer->GetLength(), (LPBYTE) pData, nSize);
	
	// Test
	pDecoder->Write("AFX-PointTextureBuffer.debug.vrml", id);
	
	// Enable
	Enable(pData);

	return TRUE;
	}

void PointTextureProxy::Enable (unsigned char *pData)
	{
	// variables: fields, pointers, resolution values, indices
	MF<Color3f>* pColor = NULL;
    MF<int>* pDepth = NULL;
    SF<int>* pDepthNbBits = NULL;
    SF<int>* pHeight = NULL;
    SF<int>* pWidth = NULL;
	Color3f oRGB;
	int nX_max = 0;			// maximum in X-axis
	int nY_max = 0;			// maximum in Y-axis
	int nZ_max = 0;			// maximum in Z-axis
	int nIndexOfDepth = 0;
	int nNumberOfDepth = 0;
	int nIndexOfColor = 0;
	int nNumberOfColor = 0;
	int i = 0;
	int j = 0;
	int k = 0;

	PointTexture* pPointTexture = (PointTexture*)GetMediaObject();

	if(pPointTexture == NULL)
		return;

	pColor = (MF<Color3f>*)pPointTexture->GetField(0);
    pDepth = (MF<int>*)pPointTexture->GetField(1);
    pDepthNbBits = (SF<int>*)pPointTexture->GetField(2);
    pHeight = (SF<int>*)pPointTexture->GetField(3);
    pWidth = (SF<int>*)pPointTexture->GetField(4);

	// set: the resolution values
	*pWidth = ((int*)pData)[0];
	*pHeight = ((int*)pData)[1];
	*pDepthNbBits = ((int*)pData)[2];
	nX_max = *pWidth - 1;
	nY_max = *pHeight - 1; 
	nZ_max = (int)pow(2, *pDepthNbBits+1)-1;

	// transfer: the depth data
	for(j = 0; j <= nY_max; j++)
		{
		for(i = 0; i <= nX_max; i++)
			{
			int nSize = ((int*)pData)[3+nIndexOfDepth];
			pDepth->SetAt(nIndexOfDepth, nSize);
			nIndexOfDepth++;

			for(k = 0; k < nSize; k++)
				{
				int nDepth = ((int*)pData)[3+nIndexOfDepth];
				pDepth->SetAt(nIndexOfDepth, nDepth);
				nIndexOfDepth++;
				}
			}
		}

	// transfer: the color data
	nNumberOfDepth = nIndexOfDepth;
	nIndexOfDepth = 0;
	for(j = 0; j <= nY_max; j++)
		{
		for(i = 0; i <= nX_max; i++)
			{
			int nSize = ((int*)pData)[3+nIndexOfDepth];
			nIndexOfDepth++;

			for(k = 0; k < nSize; k++)
				{
				nIndexOfDepth++;
				oRGB.r = (double)((unsigned char*)(pData+sizeof(int)*(3+nNumberOfDepth)))[nIndexOfColor++] / 255;
				oRGB.g = (double)((unsigned char*)(pData+sizeof(int)*(3+nNumberOfDepth)))[nIndexOfColor++] / 255;
				oRGB.b = (double)((unsigned char*)(pData+sizeof(int)*(3+nNumberOfDepth)))[nIndexOfColor++] / 255;
				pColor->SetAt(nNumberOfColor, oRGB);
				nNumberOfColor++;
				}
			}
		}
	}

// 2004-07-15, Shinjun }

/* virtual */ void
PointTextureProxy::RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD /* dwTime */)
{
    RenderDescriptionBox(pRenderer, "SimpleTexture"); // 2004-07-15, Shinjun: "SimpleTexture" -> "PointTexture"
}

#endif // SAIT_CONFORMANCE_2003
