// -*- c++ -*-

// This software module was originally developed by In Kyu Park
// (SAMSUNG AIT) in the course of development of the MPEG-4 AFX
// standard (ISO/IEC 14496-16).  This software module is an implementation
// of a part of one or more MPEG-4 AFX tools as specified by the MPEG-4
// AFX standard.  ISO/IEC gives users of the MPEG-4 AFX standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 AFX
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// AFX conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// Copyright 2003
#ifdef SAIT_CONFORMANCE_2003

#ifndef __POINTTEXTUREPROXY_H__
#define __POINTTEXTUREPROXY_H__

#include "../../Geometry/GeometryProxy.h"
#include "DepthImageProxy.h"
#define NGRAD 8

class PointTexture;
class OpenGLVisualRenderer;

/// PointTexture renderer node

class PointTextureProxy
  : public MediaObjectProxy
{
protected:

    int m_nWidth;
    int m_nHeight;
    int m_nDepthBitPerPixel;
    unsigned int m_nMaxDepth;

	double m_pProjection[NGRAD+1][NGRAD+1][NGRAD+1][3];
	int	m_pSplatSize[NGRAD][NGRAD][NGRAD];
public:
    DepthImageProxy * m_pDepthImageProxy;

public:
    /// Utilites
    int GetWidth(void) { return m_nWidth; };
    int GetHeight(void) { return m_nHeight; };
    int GetDepthBitPerPixel(void) { return m_nDepthBitPerPixel; };
    int GetMaxDepth(void) { return m_nMaxDepth; };

    void SetWidth(int Width) { m_nWidth = Width; };
    void SetHeight(int Height) { m_nHeight = Height; };
    void SetDepthBitPerPixel(int Depth) { m_nDepthBitPerPixel = Depth; };
    void SetMaxDepth(int Depth) { m_nMaxDepth = Depth; };

    /// From Bluxxun
    // 2004-07-15, Shinjun {
	int			CreateGlList(double nearPlane, double farPlane, bool bRender = false);
	// 2004-07-15, Shinjun {

  /// Constructor

    PointTextureProxy(PointTexture* pPointTexture);

  /// Render object description

  virtual void		RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD dwTime);

  // Stefano Battista
  // moved methods from Box to PointTextureProxy
public:
  PointTexture * GetPointTexture() const
  { return (PointTexture *) GetMediaObject(); };
  /// Get size of box
  /// Get bounding box

  // 2004-07-15, Shinjun {
  virtual BOOL RenderFromStream(MediaStream* poStream, int id, DWORD dwTime);
  virtual BOOL RenderFromBuffer(SFString *buffer, int id);
  void Enable (unsigned char *pData);

  /// Render object
  virtual BOOL Render(const Matrix&, const CullFlag&, OpenGLVisualRenderer* pRenderer, DWORD);
  double m_dNearPlane;
  double m_dFarPlane;
  // 2004-07-15, Shinjun }
};

#include "PointTextureProxy.i"

#endif // __POINTTEXTUREPROXY_H__

#endif // SAIT_CONFORMANCE_2003
