// -*- c++ -*-

// This software module was originally developed by In Kyu Park
// (SAMSUNG AIT) in the course of development of the MPEG-4 AFX
// standard (ISO/IEC 14496-16).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// For more information or to receive an up to date version of this module,
// mail karlo@nta.no or check <http://televr.fou.telenor.no/compositor/>.
//
// Copyright 1997, 1998.

#ifdef SAIT_CONFORMANCE_2003

#include "../../OpenGl.h"
#include "SimpleTextureProxy.h"
#include "../../appearance/ImageTextureProxy.h"
#include "../../base/OpenGLVisualRenderer.h"
#include "Stream.h"

//2003.10.7 J.Y.HAN
#include "../../appearance/MovieTextureProxy.h"		
enum { COLOR, DEPTH };
//-

BOOL SimpleTextureProxy::CreateGlList(double nearPlane, double farPlane,int idx)	//2003.10.7 J.Y.HAN
{
//2003.10.7 J.Y.HAN
	if(!m_nAnim)
	{
		////////////////////// Get the stream and set parameters ///////////////////////
		SimpleTexture * pST = GetSimpleTexture();
    
		NodeField* pImgNode0 = (NodeField*) pST->GetField(0);   // depth
		NodeField* pImgNode1 = (NodeField*) pST->GetField(1);   // texture
    
		ImageTexture* pIMO = (ImageTexture*)(((SFGenericNode*)pImgNode0)->GetNode());
		ImageTextureProxy* pImgProxy0 = (ImageTextureProxy*) pIMO->GetRenderingObject();
		GetImageInfoFromImageTexture(pImgProxy0, DEPTH);

		ImageTexture* pIM1 = (ImageTexture*)(((SFGenericNode*)pImgNode1)->GetNode());
		ImageTextureProxy* pImgProxy1 = (ImageTextureProxy*) pIM1->GetRenderingObject();
		GetImageInfoFromImageTexture(pImgProxy1, COLOR);

		if ( !m_nDepthFetched || !m_nColorFetched ) {
			m_pDepthImageProxy->m_nListMade = 0;
			return FALSE;
		}		
	}
	
	bool useGLExt=m_pDepthImageProxy->m_bGLEXTInit;            // New code by In Kyu Park
	//-
    
	glPushMatrix();
	glDisable(GL_LIGHTING);
	glDisable(GL_AUTO_NORMAL);
	glDisable(GL_BLEND);
	glEnable(GL_SMOOTH);
	
	int &w = m_nWidth;
	int &h = m_nHeight;
	glScalef(1.0/w,1.0/h,1.0/(m_nMaxDepth));
	double dx,dy,dz;
	if(!useGLExt) {
		dx=w/double(NGRAD);
		dy=h/double(NGRAD);
		dz=m_nMaxDepth/double(NGRAD);
		//amount splat on diag
		double diag=min(min(dx,dy),dz)/(1.24);
	
		{//preprocessing
			GLdouble model_matrix[16];
			GLdouble proj_matrix[16];
			GLint viewport[4];
			
			glGetDoublev(GL_MODELVIEW_MATRIX, model_matrix);
			glGetDoublev(GL_PROJECTION_MATRIX, proj_matrix);
			glGetIntegerv(GL_VIEWPORT, viewport);
			int x_,y_,z_;
			for(x_=0;x_<=NGRAD;x_++){
				for(y_=0;y_<=NGRAD;y_++){
					for(z_=0;z_<=NGRAD;z_++){
						double xx,yy,zz;
						xx=x_*dx-w/2;
						yy=y_*dy-h/2;
						zz=z_*dz-m_nMaxDepth;
						gluProject(
							xx,yy,zz,
							model_matrix, proj_matrix,  viewport,
							&m_pProjection[x_][y_][z_][0],
							&m_pProjection[x_][y_][z_][1],
							&m_pProjection[x_][y_][z_][2]);
					}
				}
			}
			
#define Q(a) ((a)*(a))
#define MAX(a,b) ((a)>(b)?(a):(b))
#define LEN(a,b) (Q(a[0]-b[0])+Q(a[1]-b[1]))
			for(z_=0;z_<NGRAD;z_++){
				for(y_=NGRAD-1;y_>=0;y_--){
					for(x_=0;x_<NGRAD;x_++){
						double len[4];
						len[0]=LEN(m_pProjection[x_+0][y_+0][z_+0], m_pProjection[x_+1][y_+1][z_+1]);
						len[1]=LEN(m_pProjection[x_+1][y_+0][z_+0], m_pProjection[x_+0][y_+1][z_+1]);
						len[2]=LEN(m_pProjection[x_+0][y_+1][z_+0], m_pProjection[x_+1][y_+0][z_+1]);
						len[3]=LEN(m_pProjection[x_+0][y_+0][z_+1], m_pProjection[x_+1][y_+1][z_+0]);
						double d=sqrt(MAX(MAX(len[0],len[1]),MAX(len[2],len[3])))/diag;
						m_pSplatSize[x_][y_][z_]=d+0.6;
						if(m_pSplatSize[x_][y_][z_]<1) m_pSplatSize[x_][y_][z_]=1;
					}
				}
			}
#undef Q
#undef MAX
#undef LEN
		}
	}

//2003.10.7 J.Y.HAN
	int DepthImageSize = m_nWidth*m_nHeight*m_nDepthBytePerPixel;
	int ColorImageSize = m_nWidth*m_nHeight*m_nColorBytePerPixel;

	unsigned char * texture;
	unsigned char * depth;

	BOOL IsColorFetched,IsDepthFetched;

	int z_limit = 0;

	if(m_nAnim)
	{
		texture = m_pMovieColorBuf + idx*ColorImageSize;
		depth   = m_pMovieDepthBuf + idx*DepthImageSize;
		IsColorFetched = m_nColorVideoFetched;
		IsDepthFetched = m_nDepthVideoFetched;

		z_limit = 0x000F0F10;
	}
	else{
		texture = m_pColorBuf;
		depth   = m_pDepthBuf;
		IsColorFetched = m_nColorFetched;
		IsDepthFetched = m_nDepthFetched;

		z_limit = 0;
	}
	
	if( !IsDepthFetched && m_nColorBytePerPixel != 4 ) return -1;     
//-

	int splt=1;
	int splt_;
    if(!useGLExt) {
        glPointSize(splt);
    }

	glBegin(GL_POINTS);
    
    for(int y=0;y<h;y++) {
		for(int x=0;x<w;x++) {
			double z;
			
			if(IsDepthFetched)	//2003.10.7 J.Y.HAN
			{
			if(m_nDepthBytePerPixel==4){
					z=	double(depth[y*m_nWidth*m_nDepthBytePerPixel+x*m_nDepthBytePerPixel+3]<<24)+
						double(depth[y*m_nWidth*m_nDepthBytePerPixel+x*m_nDepthBytePerPixel+2]<<16)+
						double(depth[y*m_nWidth*m_nDepthBytePerPixel+x*m_nDepthBytePerPixel+1]<<8)+
						double(depth[y*m_nWidth*m_nDepthBytePerPixel+x*m_nDepthBytePerPixel+0]);
				} else
				if(m_nDepthBytePerPixel==3){
					z=	double(depth[y*m_nWidth*m_nDepthBytePerPixel+x*m_nDepthBytePerPixel]<<16)+
					 	double(depth[y*m_nWidth*m_nDepthBytePerPixel+x*m_nDepthBytePerPixel+1]<<8)+
						double(depth[y*m_nWidth*m_nDepthBytePerPixel+x*m_nDepthBytePerPixel+2]);
				} else
				if(m_nDepthBytePerPixel==2){
					z=	double(depth[y*m_nWidth*m_nDepthBytePerPixel+x*m_nDepthBytePerPixel]<<8)+
						double(depth[y*m_nWidth*m_nDepthBytePerPixel+x*m_nDepthBytePerPixel+1]);	//16bits
				} else
				if(m_nDepthBytePerPixel==1){
					z=depth[y*m_nWidth*m_nDepthBytePerPixel+x];
                } else return -1;
			}else{    
				z=texture[y*m_nWidth*m_nColorBytePerPixel+x*m_nColorBytePerPixel+3];
			}		
			
			if((m_nAnim) && (z <= 0x202020))			continue;
			else if((!m_nAnim) && (z==0))				continue;

			glColor3ub(
				texture[y*m_nWidth*m_nColorBytePerPixel+x*m_nColorBytePerPixel],
				texture[y*m_nWidth*m_nColorBytePerPixel+x*m_nColorBytePerPixel+1],
				texture[y*m_nWidth*m_nColorBytePerPixel+x*m_nColorBytePerPixel+2]);
			double xx = x - w/2;
			double yy = -(y - h/2);     
            double zz= z - m_nMaxDepth;
           
			if(nearPlane!=-1 && farPlane!=-1){
				double ddd=(farPlane-((double)z/m_nMaxDepth)*(farPlane-nearPlane))/farPlane;
				xx*=ddd;
				yy*=ddd;
			}
            
			if(!useGLExt) {
				splt_=m_pSplatSize[int((xx+w/2)/dx)][int((yy+h/2)/dy)][int((zz+m_nMaxDepth)/dz)];
				if(nearPlane!=-1 && farPlane!=-1){splt_=splt_*(-(double)zz)/(m_nMaxDepth)+0.5;}
				if(splt!=splt_) {
					splt=splt_; 
					glEnd();
					glPointSize(splt);
					glBegin(GL_POINTS);
				}
			}
			glVertex3d(xx, yy, zz);
		}
    }

	glEnd();


	/*//
            glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
            int mfac = 1;
            float x_side = w / 2.0f;
            float y_side = h / 2.0f;
            float z_side = m_nMaxDepth;
            x_side *= mfac;
            y_side *= mfac;
            z_side *= mfac;

            glBegin(GL_QUADS);
            glDisable(GL_LIGHTING);
            glColor3f(0, 0, 1);
            glNormal3f(0.0f, 0.0f, -1.0f);
            glVertex3f( x_side, -y_side, -z_side); // 1
            glVertex3f(-x_side, -y_side, -z_side); // 2
            glVertex3f(-x_side,  y_side, -z_side); // 3
            glVertex3f( x_side,  y_side, -z_side); // 4

            glNormal3f(0.0f, 1.0f, 0.0f);
            glVertex3f(-x_side,  y_side,  0);      // 5
            glVertex3f( x_side,  y_side,  0);      // 6
            glVertex3f( x_side,  y_side, -z_side); // 4
            glVertex3f(-x_side,  y_side, -z_side); // 3

            glNormal3f(0.0f, 0.0f, 1.0f);
            glVertex3f(-x_side, -y_side,  0);       // 7
            glVertex3f( x_side, -y_side,  0);       // 8
            glVertex3f( x_side,  y_side,  0);       // 6
            glVertex3f(-x_side,  y_side,  0);       // 5

            glNormal3f(-1.0f, 0.0f, 0.0f);           
            glVertex3f(-x_side, -y_side, -z_side);  // 2
            glVertex3f(-x_side, -y_side,  0);       // 7
            glVertex3f(-x_side,  y_side,  0);       // 5
            glVertex3f(-x_side,  y_side, -z_side);  // 3

            glNormal3f(0.0f, -1.0f, 0.0f);
            glVertex3f( x_side, -y_side,  0);       // 8 
            glVertex3f(-x_side, -y_side,  0);       // 7
            glVertex3f(-x_side, -y_side, -z_side);  // 2
            glVertex3f( x_side, -y_side, -z_side);  // 1

            glNormal3f(1.0f, 0.0f, 0.0f);
            glVertex3f( x_side, -y_side,  0);       // 8 
            glVertex3f( x_side, -y_side, -z_side);  // 1 
            glVertex3f( x_side,  y_side, -z_side);  // 4
            glVertex3f( x_side,  y_side,  0);       // 6
            glEnd();
            glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
	//*/

	glPopMatrix();
	glEnable(GL_LIGHTING);

	return TRUE;
}

// Render object description

/* virtual */ void
SimpleTextureProxy::RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD /* dwTime */)
{
    RenderDescriptionBox(pRenderer, "SimpleTexture");
}

BOOL SimpleTextureProxy::GetImageInfoFromImageTexture(ImageTextureProxy * pImgProxy, int nImageType)
{
    ImageTexture* pIMO = pImgProxy->GetImageTexture();
    MediaStream * poStream = (MediaStream *) pIMO->GetStream();

    if (pImgProxy->GetFirst()) {
        pImgProxy->Start();
        pIMO->GetExecutive()->RequestChannels();
        pImgProxy->SetFirst(FALSE);
    }

    if (poStream) {
        int nSize;
        DWORD dwTime;  // Originally commented 2003.06.26 In Kyu Park
        BOOL bEOU;
        int nStatus;
        BOOL bSkip;
        BOOL bWait;
        long lDelta;
        BOOL bDontClose;
        int nRecurrent;

        bSkip = TRUE;
        bWait = FALSE;
        lDelta = 0;
        bDontClose = TRUE;

        unsigned char * pData = (unsigned char *) poStream->Fetch(nSize, dwTime, bEOU, nStatus, bSkip, bWait, lDelta, bDontClose);

        if (pData != END_OF_STREAM) {
            if (pData) {
                if ( nImageType == COLOR ) {
                    m_nColorFetched = true;
                    m_nWidth = (((unsigned short) pData[0]) << 8) | pData[1];
                    m_nHeight = (((unsigned short) pData[2]) << 8) | pData[3];
                    m_pColorBuf = pData + 4;
                    int nMediaFormat = pIMO->GetStreamFormat();
                    m_nColorBytePerPixel = nMediaFormat == MediaFormat::RGBA ? 4 : 3;
                    if ( m_nDepthFetched ) {
                        poStream->Release(TRUE);
                    }
                }
                else if ( nImageType == DEPTH ) {
                    m_nDepthFetched = true;
                    
                    // For PNG
					m_nWidth = (((int)pData[0]) << 24) | (((int)pData[1]) << 16) 
								| (((int)pData[2]) << 8) | pData[3];
					m_nHeight = (((int)pData[4]) << 24) | (((int)pData[5]) << 16) 
								| (((int)pData[6]) << 8) | pData[7];

//2003.10.7 J.Y.HAN
					m_pDepthBuf = pData + 10;		
					int _color = pData[8]; 													

					switch(_color)
					{
						case 1: 
							m_nDepthBytePerPixel = pData[9]; 
							break;					
						
						case 3:
							m_nDepthBytePerPixel = 3;
							break;

						default:
							break;
					}								                    								
					m_nMaxDepth = pow(2, m_nDepthBytePerPixel*8)-1;	
//-
                    if ( m_nColorFetched ) {
                        poStream->Release(TRUE);
                    }
                }
                else {
                    return FALSE;
                }                    
            }
        }
    }
 
    return TRUE;
}

#include <fstream.h>
extern unsigned char* gpucClipZero;
#define	Clip(x)	*(gpucClipZero + (x))

extern void ConvertYUVtripleToRGB(int y, int u, int v, int& r, int& g, int& b);
extern void ConvertYUV2RGB(int nWidth, int nHeight, unsigned char* pYUVFrame, unsigned char* pRGBFrame);
extern void ConvertYUV2RGB2(int nWidth, int nHeight, unsigned char* pYUVFrame, unsigned char* pRGBFrame);

#include "istream.h"

BOOL SimpleTextureProxy::GetImageInfoFromMovieTexture(//NodeField* pImgNode,
MovieTextureProxy *pImgProxy,
int nImageType)
{
  MovieTexture* pIMO = pImgProxy->GetMovieTexture();
  MediaStream* pStream = (MediaStream *) pIMO->GetStream();
  
  if (pStream)						
  {
    pImgProxy->TestActive();
    if (pImgProxy->GetActive())		
    {
      int nSize;
      DWORD dwTime;
      BOOL bEOU;
      int nStatus;
      BOOL bSkip;
      BOOL bWait;
      long lDelta;
      BOOL bDontClose;
      BOOL bRecurrent;      

      // SB @@@@ I am calling Fetch without using nRecurrent!!!
      // what is th edefault? guess: FALSE
      bSkip = TRUE;
      bWait = FALSE;
      lDelta = 0;
      bDontClose = TRUE;
      unsigned char * pData = (unsigned char *) pStream->Fetch(
        nSize, dwTime, bEOU, nStatus, bSkip, bWait, lDelta, bDontClose);
      
      if (pData != END_OF_STREAM)		
	  {	
        if (pData)          
		 {
			if ( nImageType == COLOR ) {		
				if(m_nColorVideoStatus == 0)
					m_nColorVideoStatus = 1;	
			}			
			else if ( nImageType == DEPTH ) {
                if(m_nDepthVideoStatus == 0)
					m_nDepthVideoStatus = 1;
			}
			else	return FALSE;
			// return value
			BOOL bRet = false;

			// check nStatus
			if(nStatus != 0)
				return false;

			// header information
			m_nNumberOfFrame = ((int*)pData)[0];
			if(m_nNumberOfFrame < m_nPreviousFrame)
			{
				// header information
				m_nWidth = ((int*)pData)[1];
				m_nHeight = ((int*)pData)[2];
				m_nNumberOfChannels = ((int*)pData)[3];
				m_nBytePerPixel = ((int*)pData)[4];
				
				// frame information
				m_nRGBFrameSize = m_nWidth * m_nHeight * 3;
				m_nYUVFrameSize = m_nWidth * m_nHeight * 3 / 2;
				m_nCurrentFrame = 0;
				m_nPreviousFrame = 0;

				// video buffer
				if(m_pVideo != NULL)
					free(m_pVideo);
				m_pVideo = NULL;

				if ( nImageType == COLOR ) {
					m_pMovieColorBuf = NULL;
				}
				if( nImageType == DEPTH ) {
					m_pMovieDepthBuf = NULL;
				}
			}
			else if(m_nNumberOfFrame > m_nPreviousFrame)
			{
				// check frame
				if(m_nCurrentFrame+1 >= m_nNumberOfFrame)
					m_nCurrentFrame = m_nNumberOfFrame-1;
				else if(m_nCurrentFrame+1 < m_nNumberOfFrame)
					m_nCurrentFrame++;
			}
			else
				return false;

			// video buffer
			m_pVideo = (unsigned char*)realloc((void*)m_pVideo, m_nNumberOfFrame*m_nYUVFrameSize);
			memcpy(m_pVideo + m_nPreviousFrame*m_nYUVFrameSize,
					pData+sizeof(int)*5 + m_nPreviousFrame*m_nYUVFrameSize,
					(m_nNumberOfFrame-m_nPreviousFrame)*m_nYUVFrameSize);

			// convert yuv to rgb
			unsigned char* pRGBFrame = (unsigned char*)malloc(m_nRGBFrameSize);

			if ( nImageType == COLOR ) {
				m_pMovieColorBuf = (unsigned char*)realloc((void*)m_pMovieColorBuf, m_nNumberOfFrame*m_nRGBFrameSize);
			
				for(int i = m_nPreviousFrame; i < m_nNumberOfFrame; i++)
				{
					ConvertYUV2RGB(m_nWidth, m_nHeight, m_pVideo + i*m_nYUVFrameSize, pRGBFrame);
					memcpy(m_pMovieColorBuf + i*m_nRGBFrameSize, pRGBFrame, m_nRGBFrameSize);
				}
			}
			else if ( nImageType == DEPTH ) {
				m_pMovieDepthBuf = (unsigned char*)realloc((void*)m_pMovieDepthBuf, m_nNumberOfFrame*m_nRGBFrameSize);

			

				for(int i = m_nPreviousFrame; i < m_nNumberOfFrame; i++)
				{
					ConvertYUV2RGB(m_nWidth, m_nHeight, m_pVideo + i*m_nYUVFrameSize, pRGBFrame);
					memcpy(m_pMovieDepthBuf + i*m_nRGBFrameSize, pRGBFrame, m_nRGBFrameSize);					
				}
			}
	
			/*// dump
			ofstream fout_frame;
			ofstream fout_rgb;
			if(m_bDump == false)
			{
				fout_frame.open("FrameCount.txt");
				fout_frame << "number of frame : current frame" << endl;

				fout_rgb.open("RGB-3D.bin", ios::binary);
				m_bDump = true;
			}
			else
			{
				fout_frame.open("FrameCount.txt", ios::app);
				fout_rgb.open("RGB-3D.bin", ios::binary | ios::app);
			}
			fout_frame << m_nNumberOfFrame << " : ";
			fout_frame << m_nCurrentFrame << " (in)" << endl;
			fout_frame.close();

			for(int i = 0; i < m_nNumberOfFrame; i++)
			{
				ConvertYUV2RGB2(m_nWidth, m_nHeight, m_pVideo + i*m_nYUVFrameSize, pRGBFrame);
				fout_rgb.write(pRGBFrame, m_nRGBFrameSize);
			}
			fout_rgb.close();
			//*/
			
			// free
			free(pRGBFrame);
			pStream->Release(TRUE);	
		} 
      }
      else							
      {
		if ( nImageType == COLOR ) {
			if(m_nColorVideoStatus == 1){
				m_nColorVideoFetched = TRUE;
				m_nColorVideoStatus = 2;
				m_nColorBytePerPixel = m_nBytePerPixel;
			}
		}			
		else if ( nImageType == DEPTH ) {
			if(m_nDepthVideoStatus == 1){
				m_nDepthVideoFetched = TRUE;
				m_nDepthVideoStatus = 2;
				m_nDepthBytePerPixel = m_nBytePerPixel;
				m_nMaxDepth = pow(2, m_nDepthBytePerPixel*8)-1;	
			}				
		}		 
      }
    }
}
  
  return TRUE;
}

void SimpleTextureProxy::SwapImageRows()
{
	int bps, isize;

	m_pOTColorBuf = NULL;
	bps = m_nColorBytePerPixel*m_nWidth;
	isize = bps*m_nHeight;

	m_pOTColorBuf = (unsigned char*)malloc(isize);
					
	for(int y=0;y<m_nHeight;y++)
	{						
		memcpy(m_pOTColorBuf+y*bps , m_pColorBuf+(m_nHeight-y-1)*bps, bps);
	}	
}

void SimpleTextureProxy::SwapVideoRows()
{
	int bps, isize, videosize;

	m_pOTColorBuf = NULL;
	bps = m_nBytePerPixel*m_nWidth;
	isize = bps*m_nHeight;
	videosize = isize*m_nNumberOfFrame;

	m_pOTMovieColorBuf = (unsigned char*)malloc(videosize);
	
	for(int f=0;f<m_nNumberOfFrame;f++)
	{
		for(int y=0;y<m_nHeight;y++)
		{						
			memcpy(m_pOTMovieColorBuf+f*isize+y*bps , m_pMovieColorBuf+f*isize+(m_nHeight-y-1)*bps, bps);
		}	
	}
}
#endif // SAIT_CONFORMANCE_2003
