// -*- Mode: c++; tab-width: 4; c-basic-offset: 4; -*-

// This software module was adapted by Christopher Lee (MERL) 
// from DIBR/DepthImageProxy.h by Karl Anders Ãygard (Telenor R&D),
// and PointShop3D 2.0 by Richard Keiser, Oliver Knoll, Mark Pauly,
// and Matthias Zwicker (Computer Graphics Lab, ETH Zurich),
// in the course of development of the MPEG-4 Systems standard
// (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// Copyright 2004.
//
// PointShop3D is available separately, under the GNU General Public License,
//  at http://graphics.ethz.ch/pointshop3d/index.html

#ifdef SAIT_CONFORMANCE_2003

#ifndef __DEPTHIMAGEPROXYV2_H__
#define __DEPTHIMAGEPROXYV2_H__

#include "DIBR2-common.h"

#include "PS3D/Datatypes/MyDataTypes.h"
#include "PS3D/DIBR2-compat.h"
#include "PS3D/Surfel.h"

#include "../../Geometry/GeometryProxy.h"

class DepthImageV2;
class OpenGLVisualRenderer;
class GCamera;

#include "PS3D/Renderer/FrameBuffer.h"

/// DepthImageV2 renderer node

class DepthImageProxyV2:public MediaObjectProxy
{
  protected:
	SrfContext* surfelContext;
	FrameBuffer* frameBuffer; // Frame buffer
	SrfHandle lightSourceHandle;
	BOOL checkedNodes;
	BOOL prevNodeIsDepthImage;
	BOOL nextNodeIsDepthImage;

	SurfelIterator* m_surfelIterator;

  public:
	DepthImageProxyV2* prevDepthImageProxy;
	//	double m_dRotateAngle;

  public:
	int m_nCurrentListIdx, m_nNumberOfFrame;
	int m_nFetchedImage;

  public:
	void DepthImageProxyV2::DrawAxis (void);

	/// Constructor
  	DepthImageProxyV2 (DepthImageV2 * pDepthImage);

	/// Render object
	virtual BOOL Render (const Matrix &, const CullFlag &,
						 OpenGLVisualRenderer * pRenderer, DWORD);

	/// Traverse scene graph, checking for intersection
	virtual BOOL Select (OpenGLVisualRenderer * pRenderer, DWORD dwTime,
						 unsigned int &depth);

	/// Render object description
	virtual void RenderDescription (OpenGLVisualRenderer * pRenderer,
									DWORD dwTime);

  public:
	DepthImageV2 * GetDepthImage () const {
		return (DepthImageV2 *) GetMediaObject ();
	};

	SrfContext*  GetSurfelContext () { return surfelContext; }
	FrameBuffer* GetFrameBuffer () { return frameBuffer; }
  protected:
	void checkPrevAndNextNodes ();
};

#include "DepthImageProxyV2.i"

#endif // __DEPTHIMAGEPROXYV2_H__

#endif // SAIT_CONFORMANCE_2003
