// -*- Mode: c++; tab-width: 4; c-basic-offset: 4; -*-
// This software module was adapted, with permission, by Christopher Lee
// (Mitsubishi Electric Research Laboratories) from PointShop3D 2.0 by
//  Richard Keiser, Oliver Knoll, Mark Pauly, and Matthias Zwicker
// (Computer Graphics Lab, ETH Zurich).
// It was adapted in the course of development of the MPEG-4 Systems standard
// (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// Copyright 2004.
//
// PointShop3D is available separately, under the GNU General Public License,
//  at http://graphics.ethz.ch/pointshop3d/index.html

#ifndef __DIBR2_COMPAT_H__
#define __DIBR2_COMPAT_H__

#include <OpenGL.h>
#include "../DIBR2-common.h"

#define NULL 0
//#define QRgb Color3f

// QRGB seems to correspond to Gl: GL_UNSIGNED_INT_8_8_8_8_REV with GL_BGRA
typedef unsigned int QRgb;
#define _UNI(X)   ((unsigned int)(X))
#define QRGBA(R,G,B,A) _UNI((_UNI(A) << 24) | (_UNI(B) << 16) | (_UNI(G) << 8) | _UNI(R))
#define QRGB(R,G,B) QRGBA(R,G,B,0xFF)
#define qRed(RGB)   _UNI((0x000000FF & _UNI(RGB)) >>  0)
#define qGreen(RGB) _UNI((0x0000FF00 & _UNI(RGB)) >>  8)
#define qBlue(RGB)  _UNI((0x00FF0000 & _UNI(RGB)) >> 16)

typedef struct
{
	size_t width;
	size_t height;
} QSize;

#include "../../../../Core/ztypes.h"

#define CORE_API /**/
#define UTILITIES_API /**/
#define RENDERER_API /**/
#define DATATYPES_API /**/
typedef unsigned int uint;

inline Color3f
qRgb (unsigned char r, unsigned char g, unsigned char b)
{
	return Color3f (((double) r) / 255.0, ((double) g) / 255.0,
					((double) b) / 255.0);
}


#endif /* __DIBR2_COMPAT_H__ */
