// -*- Mode: c++; tab-width: 4; c-basic-offset: 4; -*-
// This software module was adapted, with permission, by Christopher Lee
// (Mitsubishi Electric Research Laboratories) from PointShop3D 2.0 by
//  Richard Keiser, Oliver Knoll, Mark Pauly, and Matthias Zwicker
// (Computer Graphics Lab, ETH Zurich).
// It was adapted in the course of development of the MPEG-4 Systems standard
// (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// Copyright 2004.
//
// PointShop3D is available separately, under the GNU General Public License,
//  at http://graphics.ethz.ch/pointshop3d/index.html

#ifndef __MYDATATYPES_H_
#define __MYDATATYPES_H_



/**
 * This namespace provides some useful custom data types, such as <code>Vector3f</code> and
 * <code>SymmetricMatrix3f</code>.
 *
 * @version 1.2
 * @author Oliver Knoll
 */
namespace MyDataTypes
{


	/**
	 * Symmetric matrix of the form:
	 * <pre>
	 * /     \
	 * | a b |
	 * | b c |
	 * \     /
	 * </pre>
	 */
	typedef struct symmetricMatrix
	{

		float a;
		float b;
		float c;

	} SymmetricMatrix3f;

	/**
	 * A 4x4 transformation matrix which stores <code>float</code> values.
	 */
	typedef float TransformationMatrix16f[16];

	/**
	 * A <code>struct</code> containing two transformation matrices - translation (and scale) and rotation - which
	 * define the camera position.
	 */
	typedef struct cameraPosition
	{
		TransformationMatrix16f scaleTranslationMatrix, rotationMatrix;
	} CameraPosition;

	/**
	 * A struct containing U, V coordinates of a texture.
	 */
	typedef struct textureCoordinate
	{
		float u, v;
	} TextureCoordinate;

	/**
	 * A <code>struct</code> which defines the view frustum.
	 */
	typedef struct viewFrustum
	{

		float fieldOfView, aspectRatio, nearPlane, farPlane;

	} ViewFrustum;

	/**
	 * A struct storing the red <code>r</code>, green <code>g</code> and blue <code>b</code>
	 * color components as <code>float</code> values.
	 */
	typedef struct rgbTriple
	{
		float r;
		float g;
		float b;
	} RGBTriple;

	/**
	 * A struct which stores the minimum and maximum coordinates of a bounding cube.
	 */
	typedef struct boundingCube
	{

		int minX, maxX, minY, maxY, minZ, maxZ;

	} BoundingCube;

	/**
	 * Splat data structure describing splat primitives that were rendered to the frame buffer.
	 * The renderer writes this data in an implementation specific fashion.
	 */
	typedef struct splat
	{
		float z;
		float conic[4];			// conic matrix for rasterizing the ellipse
		int bbox[4];			// bounding box for rasterizing the ellipse (pixel coordinates)
	} Splat;

};



#endif // __MYDATATYPES_H_

// Some Emacs-Hints -- please don't remove:
//
//  Local Variables:
//  mode:C++
//  tab-width:4
//  End:
