// -*- Mode: c++; tab-width: 4; c-basic-offset: 4; -*-
// This software module was adapted, with permission, by Christopher Lee
// (Mitsubishi Electric Research Laboratories) from PointShop3D 2.0 by
//  Richard Keiser, Oliver Knoll, Mark Pauly, and Matthias Zwicker
// (Computer Graphics Lab, ETH Zurich).
// It was adapted in the course of development of the MPEG-4 Systems standard
// (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// Copyright 2004.
//
// PointShop3D is available separately, under the GNU General Public License,
//  at http://graphics.ethz.ch/pointshop3d/index.html

#ifndef __VECTOR2D_H_
#define __VECTOR2D_H_

#include <math.h>

#include "../DIBR2-compat.h"
//#include "DataTypesDLL.h"

////////////////////////////////////////////////////////////////////////////
//
// 2D vector
//

class DATATYPES_API Vector2D
{

  public:

	/////////////////////////////////////////////////////////////////
	// constructors
	/////////////////////////////////////////////////////////////////   
	inline Vector2D (void)
	{
	};


	/////////////////////////////////////////////////////////////////

	inline Vector2D (float p0, float p1)
	{
		p[0] = p0;
		p[1] = p1;
	};

	virtual ~ Vector2D () {
	}


	/////////////////////////////////////////////////////////////////

	inline Vector2D (float *pp)
	{
		p[0] = pp[0];
		p[1] = pp[1];
	};


	/////////////////////////////////////////////////////////////////
	// overloaded operators
	/////////////////////////////////////////////////////////////////

	inline float &operator[] (int Index)
	{
		return (p[Index]);
	};


	/////////////////////////////////////////////////////////////////

	inline const float &operator[] (int Index) const
	{
		return (p[Index]);
	};


	/////////////////////////////////////////////////////////////////

	inline Vector2D & operator= (const Vector2D & P)
	{
		p[0] = P[0];
		p[1] = P[1];
		return (*this);
	};

	/////////////////////////////////////////////////////////////////

	inline Vector2D & operator+= (Vector2D P)
	{
		p[0] += P[0];
		p[1] += P[1];
		return (*this);
	};


	/////////////////////////////////////////////////////////////////

	inline Vector2D & operator-= (Vector2D P)
	{
		p[0] -= P[0];
		p[1] -= P[1];
		return (*this);
	};


	/////////////////////////////////////////////////////////////////

	inline Vector2D & operator*= (Vector2D P)
	{
		p[0] *= P[0];
		p[1] *= P[1];
		return (*this);
	};


	/////////////////////////////////////////////////////////////////

	inline Vector2D & operator*= (float s)
	{
		p[0] *= s;
		p[1] *= s;
		return (*this);
	};


	/////////////////////////////////////////////////////////////////

	inline Vector2D & operator/= (Vector2D P)
	{
		p[0] /= P[0];
		p[1] /= P[1];
		return (*this);
	};

	/////////////////////////////////////////////////////////////////

	inline Vector2D & operator/= (float s)
	{
		p[0] /= s;
		p[1] /= s;
		return (*this);
	};


	/////////////////////////////////////////////////////////////////

	inline Vector2D operator+ (const Vector2D P) const
	{
		Vector2D res;
		  res[0] = p[0] + P[0];
		  res[1] = p[1] + P[1];
		  return (res);
	};


	/////////////////////////////////////////////////////////////////

	inline Vector2D operator- (const Vector2D P) const
	{
		Vector2D res;
		  res[0] = p[0] - P[0];
		  res[1] = p[1] - P[1];
		  return (res);
	};


	/////////////////////////////////////////////////////////////////

	inline Vector2D operator- () const
	{
		Vector2D res;
		  res[0] = -p[0];
		  res[1] = -p[1];
		  return (res);
	};

	/////////////////////////////////////////////////////////////////

	inline Vector2D operator* (const Vector2D P) const
	{
		Vector2D res;
		  res[0] = p[0] * P[0];
		  res[1] = p[1] * P[1];
		  return (res);
	};

	/////////////////////////////////////////////////////////////////

	inline Vector2D operator* (float s) const
	{
		Vector2D res;
		  res[0] = p[0] * s;
		  res[1] = p[1] * s;
		  return (res);
	};


	/////////////////////////////////////////////////////////////////

	inline Vector2D operator/ (const Vector2D P) const
	{
		Vector2D res;
		  res[0] = p[0] / P[0];
		  res[1] = p[1] / P[1];
		  return (res);
	};

	/////////////////////////////////////////////////////////////////

	inline Vector2D operator/ (float s) const
	{
		Vector2D res;
		  res[0] = p[0] / s;
		  res[1] = p[1] / s;
		  return (res);
	};



	/////////////////////////////////////////////////////////////////

	inline bool operator == (const Vector2D & a) const
	{
		return (p[0] == a[0] && p[1] == a[1]);
	};


	/////////////////////////////////////////////////////////////////

	inline bool operator != (const Vector2D & a) const
	{
		return (p[0] != a[0] || p[1] != a[1]);
	};


	/////////////////////////////////////////////////////////////////

	inline bool operator < (const Vector2D & a) const
	{
		return (p[0] < a[0] && p[1] < a[1]);
	};


	/////////////////////////////////////////////////////////////////

	inline bool operator >= (const Vector2D & a) const
	{
		return (p[0] >= a[0] && p[1] >= a[1]);
	};


	/////////////////////////////////////////////////////////////////
	inline float *source (void)
	{
		return p;
	};


	/////////////////////////////////////////////////////////////////

	inline Vector2D & makeZero ()
	{
		p[0] = p[1] = 0;

		return (*this);
	}






	/////////////////////////////////////////////////////////////////

	inline float getLength () const
	{
		return (float) sqrt (p[0] * p[0] + p[1] * p[1]);
	};


	/////////////////////////////////////////////////////////////////
	// normalize and return length

	inline float normalize (void)
	{
		float length = getLength ();
		if (length == 0.0f)
			return 0;

		float rezLength = 1.0f / length;
		p[0] *= rezLength;
		p[1] *= rezLength;
		return length;
	};

	/////////////////////////////////////////////////////////////////
	// vector between the two points

	inline void fromTo (Vector2D & P1, Vector2D & P2)
	{
		p[0] = P2[0] - P1[0];
		p[1] = P2[1] - P1[1];
	};


	/////////////////////////////////////////////////////////////////

	static inline Vector2D scalarMult (float s, const Vector2D & P)
	{
		return (P * s);
	}


	static inline float dotProduct (const Vector2D & a, const Vector2D & b)
	{
		return (a[0] * b[0] + a[1] * b[1]);
	}


	static inline float squaredLength (const Vector2D & v)
	{
		return (dotProduct (v, v));
	}


	static inline float squaredDistance (const Vector2D & v1,
										 const Vector2D & v2)
	{
		Vector2D tmp = v1 - v2;
		return (squaredLength (tmp));
	}


	static inline float distance (const Vector2D & v1, const Vector2D & v2)
	{
		Vector2D tmp = v1 - v2;
		return (tmp.getLength ());
	}





	static inline Vector2D crossProduct (const Vector2D & a,
										 const Vector2D & b)
	{
		Vector2D result;

		result[0] = a[1] * b[0] - a[0] * b[1];
		result[1] = a[0] * b[1] - a[1] * b[0];

		return (result);
	}


	/////////////////////////////////////////////////////////////////

	static inline Vector2D projectOntoVector (Vector2D & v1, Vector2D & v2)
	{
		return scalarMult (dotProduct (v1, v2), v2);
	}




  protected:

	float p[2];

};


////////////////////////////////////////////////////////////////////////////

inline Vector2D
operator * (float s, const Vector2D & P)
{
	return (P * s);
}


#endif // __VECTOR2D_H_

// Some Emacs-Hints -- please don't remove:
//
//  Local Variables:
//  mode:C++
//  tab-width:4
//  End:
