// -*- Mode: c++; tab-width: 4; c-basic-offset: 4; -*-
// This software module was adapted, with permission, by Christopher Lee
// (Mitsubishi Electric Research Laboratories) from PointShop3D 2.0 by
//  Richard Keiser, Oliver Knoll, Mark Pauly, and Matthias Zwicker
// (Computer Graphics Lab, ETH Zurich).
// It was adapted in the course of development of the MPEG-4 Systems standard
// (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// Copyright 2004.
//
// PointShop3D is available separately, under the GNU General Public License,
//  at http://graphics.ethz.ch/pointshop3d/index.html


#ifndef LIGHTSOURCE_H
#define LIGHTSOURCE_H

// Variable and type declarations
typedef enum lsrType
{
	LSR_DIRECTIONAL = 0,
	LSR_POINT = 1,
	LSR_SPOT = 2
} LSrType;

typedef struct lightSource
{
	LSrType type;				// type of lightsource

	float x;					// position or direction of light, depending on type
	float y;
	float z;

	float Ir;					// red intensity
	float Ig;					// green intensity
	float Ib;					// blue intensity

	float Ar;					// ambient red intensity
	float Ag;					// ambient green intensity
	float Ab;					// ambient blue intensity
} LightSource;


// Function prototypes
LightSource *LSrNew ();
void LSrFree (LightSource * lsr);
void LSrCopy (LightSource * dst, LightSource * src);


// Macros
#define LSRGET_X(_lsr) (_lsr->x)
#define LSRGET_Y(_lsr) (_lsr->y)
#define LSRGET_Z(_lsr) (_lsr->z)
#define LSRGET_IR(_lsr) (_lsr->Ir)
#define LSRGET_IG(_lsr) (_lsr->Ig)
#define LSRGET_IB(_lsr) (_lsr->Ib)
#define LSRGET_AR(_lsr) (_lsr->Ar)
#define LSRGET_AG(_lsr) (_lsr->Ag)
#define LSRGET_AB(_lsr) (_lsr->Ab)


#endif // LIGHTSOURCE_H

// Some Emacs-Hints -- please don't remove:
//
//  Local Variables:
//  mode:C++
//  tab-width:4
//  End:
