// -*- Mode: c++; tab-width: 4; c-basic-offset: 4; -*-
// This software module was adapted, with permission, by Christopher Lee
// (Mitsubishi Electric Research Laboratires) from PointShop3D 2.0 by
//  Richard Keiser, Oliver Knoll, Mark Pauly, and Matthias Zwicker
// (Computer Graphics Lab, ETH Zurich).
// It was adapted in the course of development of the MPEG-4 Systems standard
// (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// Copyright 2004.
//
// PointShop3D is available separately, under the GNU General Public License,
//  at http://graphics.ethz.ch/pointshop3d/index.html


//-------------------------------------------------------------------------
// Warper 
//
// Warper.h 
//-------------------------------------------------------------------------

#ifndef WARPER_H
#define WARPER_H

#include <vector>
#include "../OctreeRenderer_SurfSplat/ConfigHeaders/ZBuffer.h"

#define CULL_UNKNOWN 0
#define CULL_INSIDE 1
#define CULL_OUTSIDE 2


// Variable and type declarations
// viewing transformation
typedef enum wrpOptions
{
	WRP_LODCOLORS = 1,
	WRP_SKIPSAMPLES = 2,
	WRP_ELLIPTICALSURFELS = 4
} WrpOptions;

typedef struct transformation
{
	float rotation[9];			// model-view rotation
	float translation[3];		// model-view translation
	float invRotation[9];		// inverse model-view rotation (used for block clipping)
	float invTranslation[3];	// inverse model-view rotation (used for block clipping)
	float normalsRotation[9];	// rotation matrix to transform normals
	float scaling;				// uniform model-view scaling
} Transformation;

// view frustum
typedef struct frustum
{
	float xP, yP;				// x,y-extent of the view frustum at z=1
	float xC, yC;				// x,y-center point of the view frustum at z=1
	float nearplane, farplane;	// camera space z-values of near and far clipping planes
	float v[24];				// vertices of the view frustum in camera space
	float M[9];					// view frustum normalization transformation (used for clipping)
} Frustum;

// viewport
typedef struct psp_viewport
{
	// note: the viewport has the origin at its lower left corner
	int xS, yS;					// size of the viewport in screen pixels
} PSPViewport;

// warper
typedef struct warper
{
	Transformation transformation;
	Frustum frustum;
	PSPViewport viewport;
	ZBuffer *zbf;

	int options;
} Warper;


// Function prototypes
Warper *WrpNew (ZBuffer * zbf);
void WrpFree (Warper * wrp);

void WrpSetFrustumAsymmetric (Warper * wrp, float r, float l, float t,
							  float b, float n, float f);
void WrpSetFrustum (Warper * wrp, float fofv, float aspect, float nearP,
					float farP);
void WrpSetFrustum_obsolete (Warper * wrp, float f, float x, float y);
void WrpSetViewport (Warper * wrp, int xsize, int ysize);

void WrpClearZBuffer (Warper * wrp);
void WrpResetObjectCount (Warper * wrp);

//void WrpWarpToZBufferSOT(Warper *r, SurfelCollection *slt, int bbox[4]);
//void WrpWarpToZBufferSOT (Warper * r, const std::vector < Surfel * >*slt,
//						  int nSurfels, int bbox[4]);
void WrpWarpToZBufferSOT (Warper * r, SurfelIterator* surfelIterator,
						  int bbox[4]);
//void WrpTestWarpToZBufferSOT(Warper *r, SurfelCollection *slt);
void WrpSetTrafo (Warper * r, const float trafo[16]);

void WrpEnable (Warper * wrp, WrpOptions o);
void WrpDisable (Warper * wrp, WrpOptions o);


#endif // WARPER_H

// Some Emacs-Hints -- please don't remove:
//
//  Local Variables:
//  mode:C++
//  tab-width:4
//  End:
