// -*- Mode: c++; tab-width: 4; c-basic-offset: 4; -*-
// This software module was adapted, with permission, by Christopher Lee
// (Mitsubishi Electric Research Laboratories) from PointShop3D 2.0 by
//  Richard Keiser, Oliver Knoll, Mark Pauly, and Matthias Zwicker
// (Computer Graphics Lab, ETH Zurich).
// It was adapted in the course of development of the MPEG-4 Systems standard
// (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// Copyright 2004.
//
// PointShop3D is available separately, under the GNU General Public License,
//  at http://graphics.ethz.ch/pointshop3d/index.html

// note: this code is pasted into the diverse WrpWarpBlock_*() functions!


			// note: if A[bp][8] == 0 use non-optimized incremental calculations
if (A_bp_8 > -NONOPT_TOL && A_bp_8 < NONOPT_TOL) {
	while (IITROWS) {
		p = p_start_bp;
		q = q_start_bp;
		r = r_start_bp;

		IITFIRSTCOL (r, p, q, dx_r_bp, dx_p_bp, dx_q_bp);
		while (IITCOLS) {
			nsamples = IITNSAMPLES;

			// some loop unrolling for the most frequent special case
			if (nsamples == 1) {
				// warp sample
				sample = SITGETCURRENT (sit);
				SITNEXT (sit);

				sample_depth = SMPGETDEPTH (sample, d);

#ifdef DEMO_CODE
				if (wrp_options & WRP_LODCOLORS) {
					// LOD debugging: set a shade index according to block level
					SMPSETWARPEDLEVEL (sample, l);
				}
#endif
#ifdef DEBUG_CDE_CLIP
				if (dbg_outside)
					sample->dbgshadeindex = 1;
				else
					sample->dbgshadeindex = 0;
#endif

				// transform normal to camera coordinates
				n_i = SMPGETNORMAL (sample);
				nx_o = wrp_nrm_table[n_i].x;
				ny_o = wrp_nrm_table[n_i].y;
				nz_o = wrp_nrm_table[n_i].z;

				n[0] =
					normalsA[0] * nx_o + normalsA[1] * ny_o +
					normalsA[2] * nz_o;
				n[1] =
					normalsA[3] * nx_o + normalsA[4] * ny_o +
					normalsA[5] * nz_o;
				n[2] =
					normalsA[6] * nx_o + normalsA[7] * ny_o +
					normalsA[8] * nz_o;

				// non optimized incremental calculations
				z_c = r + A_bp_8 * sample_depth;
				r_z_c = 1 / z_c;
				xImg = (p + A2_vp_sx_bp * sample_depth) * r_z_c + vp_tsx_bp;
				yImg = (q + A5_vp_sy_bp * sample_depth) * r_z_c + vp_tsy_bp;

				// put it into the z-buffer
				// note: no clip testing required
				// caution: this function (or macro) relies on global variables!
				wrp_zbf_splatsurfel (wrp_zbf, xImg, yImg, z_c, n, sample, l,
									 stoo_scaling, otoc_scaling, vp_sx, vp_sy,
									 vp_tx[0], vp_ty[0]);
			} else {
				while (nsamples) {
					// warp sample
					sample = SITGETCURRENT (sit);
					SITNEXT (sit);

					sample_depth = SMPGETDEPTH (sample, d);

#ifdef DEMO_CODE
					if (wrp_options & WRP_LODCOLORS) {
						// LOD debugging: set a shade index according to block level
						SMPSETWARPEDLEVEL (sample, l);
					}
#endif
#ifdef DEBUG_CDE_CLIP
					if (dbg_outside)
						sample->dbgshadeindex = 1;
					else
						sample->dbgshadeindex = 0;
#endif

					// transform normal to camera coordinates
					n_i = SMPGETNORMAL (sample);
					nx_o = wrp_nrm_table[n_i].x;
					ny_o = wrp_nrm_table[n_i].y;
					nz_o = wrp_nrm_table[n_i].z;

					n[0] =
						normalsA[0] * nx_o + normalsA[1] * ny_o +
						normalsA[2] * nz_o;
					n[1] =
						normalsA[3] * nx_o + normalsA[4] * ny_o +
						normalsA[5] * nz_o;
					n[2] =
						normalsA[6] * nx_o + normalsA[7] * ny_o +
						normalsA[8] * nz_o;

					// non optimized incremental calculations
					z_c = r + A_bp_8 * sample_depth;
					r_z_c = 1 / z_c;
					xImg =
						(p + A2_vp_sx_bp * sample_depth) * r_z_c + vp_tsx_bp;
					yImg =
						(q + A5_vp_sy_bp * sample_depth) * r_z_c + vp_tsy_bp;

					// put it into the z-buffer
					// note: no clip testing required
					// caution: this function (or macro) relies on global variables!
					wrp_zbf_splatsurfel (wrp_zbf, xImg, yImg, z_c, n, sample,
										 l, stoo_scaling, otoc_scaling, vp_sx,
										 vp_sy, vp_tx[0], vp_ty[0]);

					nsamples--;
				}
			}

			// increment col variables
			IITNEXTCOL (r, p, q, dx_r_bp, dx_p_bp, dx_q_bp);
		}
		// increment row variables
		IITNEXTROW (r_start_bp, p_start_bp, q_start_bp, dy_r_bp, dy_p_bp,
					dy_q_bp);
	}

} else {
	while (IITROWS) {
		p = p_start_bp;
		q = q_start_bp;
		r = r_start_bp;

		IITFIRSTCOL (r, p, q, dx_r_bp, dx_p_bp, dx_q_bp);
		while (IITCOLS) {

			nsamples = IITNSAMPLES;

			// some loop unrolling for the most frequent special case
			if (nsamples == 1) {
				// warp sample
				sample = SITGETCURRENT (sit);
				SITNEXT (sit);

				sample_depth = SMPGETDEPTH (sample, d);

#ifdef DEMO_CODE
				if (wrp_options & WRP_LODCOLORS) {
					// LOD debugging: set a shade index according to block level
					SMPSETWARPEDLEVEL (sample, l);
				}
#endif
#ifdef DEBUG_CDE_CLIP
				if (dbg_outside)
					sample->dbgshadeindex = 1;
				else
					sample->dbgshadeindex = 0;
#endif

				// transform normal to camera coordinates
				n_i = SMPGETNORMAL (sample);
				nx_o = wrp_nrm_table[n_i].x;
				ny_o = wrp_nrm_table[n_i].y;
				nz_o = wrp_nrm_table[n_i].z;

				n[0] =
					normalsA[0] * nx_o + normalsA[1] * ny_o +
					normalsA[2] * nz_o;
				n[1] =
					normalsA[3] * nx_o + normalsA[4] * ny_o +
					normalsA[5] * nz_o;
				n[2] =
					normalsA[6] * nx_o + normalsA[7] * ny_o +
					normalsA[8] * nz_o;

				// use optimized incremental calculations
				z_c = r + A_bp_8 * sample_depth;
				r_z_c = 1 / z_c;
				xImg = p * r_z_c + vp_tsx_bp;
				yImg = q * r_z_c + vp_tsy_bp;

				// put it into the z-buffer
				// note: no clip testing required
				// caution: this function (or macro) relies on global variables!
				wrp_zbf_splatsurfel (wrp_zbf, xImg, yImg, z_c, n, sample, l,
									 stoo_scaling, otoc_scaling, vp_sx, vp_sy,
									 vp_tx[0], vp_ty[0]);
			} else {
				while (nsamples) {
					// warp sample
					sample = SITGETCURRENT (sit);
					SITNEXT (sit);

					sample_depth = SMPGETDEPTH (sample, d);

#ifdef DEMO_CODE
					if (wrp_options & WRP_LODCOLORS) {
						// LOD debugging: set a shade index according to block level
						SMPSETWARPEDLEVEL (sample, l);
					}
#endif
#ifdef DEBUG_CDE_CLIP
					if (dbg_outside)
						sample->dbgshadeindex = 1;
					else
						sample->dbgshadeindex = 0;
#endif

					// transform normal to camera coordinates
					n_i = SMPGETNORMAL (sample);
					nx_o = wrp_nrm_table[n_i].x;
					ny_o = wrp_nrm_table[n_i].y;
					nz_o = wrp_nrm_table[n_i].z;

					n[0] =
						normalsA[0] * nx_o + normalsA[1] * ny_o +
						normalsA[2] * nz_o;
					n[1] =
						normalsA[3] * nx_o + normalsA[4] * ny_o +
						normalsA[5] * nz_o;
					n[2] =
						normalsA[6] * nx_o + normalsA[7] * ny_o +
						normalsA[8] * nz_o;

					// use optimized incremental calculations
					z_c = r + A_bp_8 * sample_depth;
					r_z_c = 1 / z_c;
					xImg = p * r_z_c + vp_tsx_bp;
					yImg = q * r_z_c + vp_tsy_bp;

					// put it into the z-buffer
					// note: no clip testing required
					// caution: this function (or macro) relies on global variables!
					wrp_zbf_splatsurfel (wrp_zbf, xImg, yImg, z_c, n, sample,
										 l, stoo_scaling, otoc_scaling, vp_sx,
										 vp_sy, vp_tx[0], vp_ty[0]);

					nsamples--;
				}
			}

			// increment col variables
			IITNEXTCOL (r, p, q, dx_r_bp, dx_p_bp, dx_q_bp);
		}
		// increment row variables
		IITNEXTROW (r_start_bp, p_start_bp, q_start_bp, dy_r_bp, dy_p_bp,
					dy_q_bp);
	}
}


// Some Emacs-Hints -- please don't remove:
//
//  Local Variables:
//  mode:C++
//  tab-width:4
//  End:
