// -*- Mode: c++; tab-width: 4; c-basic-offset: 4; -*-

// This software module was adapted by Christopher Lee (MERL) 
// from DIBR/PointTextureProxy.cpp by In Kyu Park (SAMSUNG AIT)
// and PointShop3D 2.0 by Richard Keiser, Oliver Knoll, Mark Pauly,
// and Matthias Zwicker (Computer Graphics Lab, ETH Zurich),
// in the course of development of the MPEG-4 Systems standard
// (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// Copyright 2004.
//
// PointShop3D is available separately, under the GNU General Public License,
//  at http://graphics.ethz.ch/pointshop3d/index.html

#ifdef SAIT_CONFORMANCE_2003

#include "../../OpenGl.h"
#include "PointTextureProxyV2.h"
#include "../../base/OpenGLVisualRenderer.h"

#include "PS3D/SurfelStruct.h"

enum {
	NODE_COLOR,
	NODE_DEPTH,
	NODE_DEPTH_NB_BITS,
	NODE_HEIGHT,
	NODE_NORMAL,
	NODE_SPLAT_U,
	NODE_SPLAT_V,
	NODE_WIDTH
};

class PointTextureSurfelIterator : public SurfelIterator {
	int m_nColor;
	int m_ci;
	SurfelStruct* m_ss;
public:
	PointTextureSurfelIterator (PointTextureProxyV2* proxy,
								int nColor,
								double nearPlane,
								double farPlane,
								MF < Color3f > *pColorField,
								MF < int > *pDepthField,
								Normal* normal,
								MF<Vec3f>* splatU,
								MF<Vec3f>* splatV);
	BOOL getNextSurfelStruct (SurfelStruct* out);
	~PointTextureSurfelIterator () {
		delete [] m_ss;
	}
	void reset() { m_ci = 0; }
};

PointTextureSurfelIterator::PointTextureSurfelIterator (
		PointTextureProxyV2* proxy,
		int nColor,
		double nearPlane,
		double farPlane,
		MF<Color3f> *pColorField,
		MF<int> *pDepthField,
		Normal* normal,
		MF<Vec3f>* splatU,
		MF<Vec3f>* splatV)
{
	m_nColor = nColor;
	m_ci = 0;

	// For now, store them all in a big memory buffer...
	m_ss = new SurfelStruct[nColor];

	int di, ci;
	int x = 0, y = 0;

	double width = proxy->GetWidth();
	double height = proxy->GetHeight();
	double depth = proxy->GetMaxDepth();

	for (ci = 0, di = 0; ci < nColor;) {
		unsigned long k = (*pDepthField)[di];
		di++;
		for (int i = 0; i < k; ci++, i++) {
			m_ss[ci].diffuseColor =
				QRGB((*pColorField)[ci].b * 255,  // Looks backward,
					 (*pColorField)[ci].g * 255,  //  but this is
					 (*pColorField)[ci].r * 255); //  what works....

			double z = (*pDepthField)[di];
			di++;

			if (z < 0) {
				z += (double (1 << 30)) *4;
			}
			double xx = (((double)x) - (((double)width) / 2))/width;
			double yy = (((double)y) - (((double)height)/ 2))/height;
			double zz = (((double)z) - ((double)depth))/depth;
			if (nearPlane != -1 && farPlane != -1) {
				double ddd=(farPlane-((double)z/depth)
							*(farPlane-nearPlane))/farPlane;
				xx *= ddd;
				yy *= ddd;
			}
			m_ss[ci].position[0] = xx;
			m_ss[ci].position[1] = yy;
			m_ss[ci].position[2] = zz;

			// Matthias email specified the following
			m_ss[ci].ambientColor  = QRGB(255,255,255);
			m_ss[ci].specularColor = QRGB(255,255,255);
			m_ss[ci].ambientCoefficient  = 1.0;
			m_ss[ci].diffuseCoefficient  = 0.1;
			m_ss[ci].specularCoefficient = 0.1;

			//TODO -- what should be the radius of the surfel?
	
			m_ss[ci].radius  = 0.7; // ???
			if ( normal && splatU && splatV ) {
				const Vec3f pNormal = normal->vector[ci];
				Vec3f pSplatU = (*splatU)[ci];
				Vec3f pSplatV = (*splatV)[ci];
					m_ss[ci].normal  = Vector3D(pNormal.x/width,
												pNormal.y/height,
												pNormal.z/depth);
					m_ss[ci].axisOne = Vector3D(pSplatU.x/width,
												pSplatU.y/height,
												pSplatU.z/depth);
					m_ss[ci].axisTwo = Vector3D(pSplatV.x/width,
												pSplatV.y/height,
												pSplatV.z/depth);
			} else {
				m_ss[ci].normal  = Vector3D(0.0, 0.0, 1.0);
				m_ss[ci].axisOne = Vector3D(0.5/width, 0.0, 0.0);
				m_ss[ci].axisTwo = Vector3D(0.0, 0.5/height, 0.0);
			}
		}
		x++;
		if (x == width) {
			x = 0;
			y++;
		}
	}
}

BOOL
PointTextureSurfelIterator::getNextSurfelStruct (SurfelStruct* out)
{
	if ( m_ci >= m_nColor ) {
		return FALSE;
	}
	memcpy (out, &m_ss[m_ci], sizeof(struct _SurfelStruct));
	m_ci += 1;
	return TRUE;
}

SurfelIterator*
PointTextureProxyV2::getSurfelIterator (int nearPlane, int farPlane) {
	PointTextureV2 *pT = GetPointTexture ();
	MF <Color3f> *pColorField = (MF<Color3f>*) pT->GetField (NODE_COLOR);
	MF <int>*pDepthField = (MF<int>*) pT->GetField (NODE_DEPTH);
	SFNode <class Normal>* normalNode = (SFNode <class Normal>*) (pT->GetField (NODE_NORMAL));
	Normal* normal = (Normal*)normalNode->GetNode();
	MF <Vec3f> *pSplatU = (MF<Vec3f>*) pT->GetField (NODE_SPLAT_U);
	MF <Vec3f> *pSplatV = (MF<Vec3f>*) pT->GetField (NODE_SPLAT_V);
	int nColor = pColorField->GetLength ();
	return new PointTextureSurfelIterator (this, nColor,
										   nearPlane, farPlane,
										   pColorField, pDepthField,
										   normal,
										   pSplatU, pSplatV);
}

/* virtual */ void
PointTextureProxyV2::RenderDescription (OpenGLVisualRenderer * pRenderer,
										DWORD /* dwTime */ )
{
	RenderDescriptionBox (pRenderer, "PointTextureV2");
}

#endif // SAIT_CONFORMANCE_2003
