// -*- Mode: c++; tab-width: 4; c-basic-offset: 4; -*-

// This software module was adapted by Christopher Lee (MERL) 
// from DIBR/SimpleTextureProxy.cpp by In Kyu Park (SAMSUNG AIT)
// and PointShop3D 2.0 by Richard Keiser, Oliver Knoll, Mark Pauly,
// and Matthias Zwicker (Computer Graphics Lab, ETH Zurich),
// in the course of development of the MPEG-4 Systems standard
// (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// Copyright 2004.
//
// PointShop3D is available separately, under the GNU General Public License,
//  at http://graphics.ethz.ch/pointshop3d/index.html

#ifdef SAIT_CONFORMANCE_2003

#include "../../OpenGl.h"
#include "SimpleTextureProxyV2.h"
#include "../../appearance/ImageTextureProxy.h"
#include "../../base/OpenGLVisualRenderer.h"
#include "Stream.h"

#define DUMP_POSITIONS 0

//2003.10.7 J.Y.HAN
#include "../../appearance/MovieTextureProxy.h"
//-

enum
{ NODE_DEPTH, NODE_NORMAL, NODE_SPLATU, NODE_SPLATV, NODE_TEXTURE };


#include <math.h>
#ifndef M_LOG_10
# define M_LOG_10 2.302585092994046
#endif
#include "PS3D/SurfelStruct.h"

void
SimpleTextureProxyV2::loadSimpleTextureImage (int nNode) {
	SimpleTextureV2 *pST = GetSimpleTexture ();
	NodeField *pImgNode = (NodeField *) pST->GetField (nNode);
	ImageTexture *pIM =
		(ImageTexture *) (((SFGenericNode *) pImgNode)->GetNode ());

	if ( ! pIM ) {
		// Backward compatibility
		if ( nNode == NODE_NORMAL
			 || nNode == NODE_SPLATU
			 || nNode == NODE_SPLATV
			 || nNode == NODE_NORMAL ) {
			m_nBufferFetched[nNode] = TRUE;
			return;
		}
	}
	ImageTextureProxy *pImgProxy =
		(ImageTextureProxy *) pIM->GetRenderingObject ();
	GetImageInfoFromImageTexture (pImgProxy, nNode);
}

BOOL
SimpleTextureProxyV2::GetImage (int FrameIdx)
{

	if (!m_nAnim) {
		//////////////// Get the stream and set parameters /////////////////

		loadSimpleTextureImage(NODE_DEPTH);
		loadSimpleTextureImage(NODE_TEXTURE);
		loadSimpleTextureImage(NODE_NORMAL);
		loadSimpleTextureImage(NODE_SPLATU);
		loadSimpleTextureImage(NODE_SPLATV);
		// normal, splatU, splatV

		if (   !m_nBufferFetched[NODE_DEPTH]
			|| !m_nBufferFetched[NODE_TEXTURE]
			|| !m_nBufferFetched[NODE_NORMAL]
			|| !m_nBufferFetched[NODE_SPLATU]
			|| !m_nBufferFetched[NODE_SPLATV]) {
			return FALSE;
		}

		texture = m_pColorBuf;
		depth = m_pDepthBuf;
		normal = m_pNormalBuf;
		splatU = m_pSplatUBuf;
		splatV = m_pSplatVBuf;
		IsColorFetched = TRUE;
		IsDepthFetched = TRUE;
	
	} else {
		int DepthImageSize = m_nWidth * m_nHeight * m_nDepthBytePerPixel;
		int ColorImageSize = m_nWidth * m_nHeight * m_nColorBytePerPixel;

		texture = m_pMovieColorBuf + FrameIdx * ColorImageSize;
		depth = m_pMovieDepthBuf + FrameIdx * DepthImageSize;
		IsColorFetched = m_nColorVideoFetched;
		IsDepthFetched = m_nDepthVideoFetched;
	}

	if (!IsDepthFetched && m_nColorBytePerPixel != 4)
		return FALSE;

	return TRUE;
}


class SimpleTextureSurfelIterator : public SurfelIterator {
	int m_nIdx;
	int m_nNumSurfels;
	int m_nNearPlane;
	int m_nFarPlane;
	SimpleTextureProxyV2* m_pSTP;
public:
	SimpleTextureSurfelIterator (SimpleTextureProxyV2* pSTP,
								 int numSurfels,
								 int nearPlane,
								 int farPlane);
	BOOL getNextSurfelStruct (SurfelStruct* out);
	void reset () { m_nIdx = 0; }
#if DUMP_POSITIONS
	~SimpleTextureSurfelIterator () {
		if (positionFile) { fclose(positionFile); }
	}
private:
	FILE* positionFile;
#endif
};

static _fileCount =  0;

SimpleTextureSurfelIterator::SimpleTextureSurfelIterator (SimpleTextureProxyV2* proxy,
															int numSurfels,
															int nearPlane,
															int farPlane)
{
	m_nIdx = 0;
	m_nNumSurfels = numSurfels;
	m_pSTP = proxy;
	m_nNearPlane = nearPlane;
	m_nFarPlane = farPlane;
#if DUMP_POSITIONS
	positionFile = NULL;
	if ( _fileCount < 6 ) {
		char buffer[256];
		sprintf (buffer, "position-list-img-%03d.txt", _fileCount++);
		positionFile = fopen (buffer, "w");
	}
#endif
}


BOOL SimpleTextureSurfelIterator::getNextSurfelStruct (SurfelStruct* sout)
{
	while ( m_nIdx < m_nNumSurfels ) {
		int ret = m_pSTP->GetSurfelStruct (sout, m_nIdx, m_nNearPlane, m_nFarPlane);
		m_nIdx += 1;
		switch ( ret ) {
		case -1: return FALSE; // Error
		case  1:
#if DUMP_POSITIONS
			if ( positionFile ) {
				fprintf (positionFile, "%g %g %g\n",
						sout->position[0],
						sout->position[1],
						sout->position[2]);
	}
#endif

			return TRUE;  // Got surfel
		case  0: break; // didn't get one this time, try next index.
		}
	}
	return FALSE;
}

SurfelIterator*
SimpleTextureProxyV2::getSurfelIterator (int nearPlane, int farPlane) {
	return new SimpleTextureSurfelIterator (this, m_nWidth*m_nHeight,
											nearPlane, farPlane);
}


#define BYTE2SCALAR(B) ( (2.0/255.0) * (((int)(B))-128)  )



Vector3D
descaleSplat(unsigned char* xyz, double slope, double offset,
			 BOOL reverse)
{
	double sx = BYTE2SCALAR(xyz[0]);  // (0 .. 255) --> (-1.0 ... 1.0)
	double sy = BYTE2SCALAR(xyz[1]);
	double sz = BYTE2SCALAR(xyz[2]);
	double slen = sqrt (sx*sx + sy*sy + sz*sz);
	double newLen = slen*slope+offset;

	double x = sx*newLen/slen;
	double y = sy*newLen/slen;
	double z = sz*newLen/slen;
	
	return (reverse ? Vector3D(-x,-y,-z) : Vector3D(x,y,z));
}


/* Returns: 1 means got surfel, 0 means didn't get surfel, -1 means error */
int
SimpleTextureProxyV2::GetSurfelStruct (SurfelStruct* surfelStruct, int SurfelIdx,
									   int nearPlane, int farPlane)
{
	double z;

	if (IsDepthFetched)	{
		unsigned int byte0 = SurfelIdx*m_nDepthBytePerPixel;
		if (m_nDepthBytePerPixel == 4) {
			z = (double) (  (depth[byte0 + 0] << 24)
						  + (depth[byte0 + 1] << 16)
						  + (depth[byte0 + 2] << 8)
						  + (depth[byte0 + 3]));
		} else if (m_nDepthBytePerPixel == 3) {
			z = (double) (  (depth[byte0 + 0] << 16)
						  + (depth[byte0 + 1] << 8)
						  + (depth[byte0 + 2]));
		} else if (m_nDepthBytePerPixel == 2) {
			z = (double) (  (depth[byte0 + 0] << 8)
						  + (depth[byte0 + 1]));
		} else if (m_nDepthBytePerPixel == 1) {
			z = depth[SurfelIdx];
		} else
			return -1;
	} else {
		z = texture[SurfelIdx * m_nColorBytePerPixel + 3];
	}

	if ((m_nAnim) && (z <= 0x202020))
		return 0;
	else if ((!m_nAnim) && (z == 0))
		return 0;

	int iy = SurfelIdx / m_nWidth;
	int ix = SurfelIdx % m_nWidth;

	surfelStruct->position[0] = ((double)(ix - m_nWidth/2)) / m_nWidth;
	surfelStruct->position[1] = ((double)(m_nHeight/2 - iy)) / m_nHeight;
	surfelStruct->position[2] = ((double)(z - m_nMaxDepth)) / m_nMaxDepth;

	if (nearPlane != -1 && farPlane != -1) {
		double ddd =
			(farPlane -
			 ((double) z / m_nMaxDepth) * (farPlane -
												   nearPlane)) / farPlane;
		surfelStruct->position[0] = surfelStruct->position[0] * ddd;
		surfelStruct->position[1] = surfelStruct->position[1] * ddd;
	}

	{
		unsigned char r = texture[SurfelIdx*m_nColorBytePerPixel+2];
		unsigned char g = texture[SurfelIdx*m_nColorBytePerPixel+1];
		unsigned char b = texture[SurfelIdx*m_nColorBytePerPixel+0];
		
		surfelStruct->diffuseColor  = QRGB(r,g,b);
	}
	surfelStruct->ambientColor  = QRGB(255,255,255);
	surfelStruct->specularColor = QRGB(255,255,255);
	surfelStruct->ambientCoefficient  = 1.0;
	surfelStruct->diffuseCoefficient  = 0.1;
	surfelStruct->specularCoefficient = 0.1;

	surfelStruct->radius  = 0.7; // ???

	if ( normal && splatU && splatV ) {
		
		unsigned char* pNormal = &normal[SurfelIdx*m_nNormalBytePerPixel];
		unsigned char* pSplatU = &splatU[SurfelIdx*m_nSplatUBytePerPixel];
		unsigned char* pSplatV = &splatV[SurfelIdx*m_nSplatVBytePerPixel];

		if ( ! m_nUVScalingComputed ) {
			SimpleTextureV2* pST = (SimpleTextureV2*) GetMediaObject();
			DepthImageV2* pDI = (DepthImageV2*) pST->GetParent(0);
			Vec2f nodeSplatMinMax =
				((SF<Vec2f>*)pDI->GetField(7))->GetValue ();
			double splatMin = nodeSplatMinMax.x;
			double splatMax = nodeSplatMinMax.y;
			
			m_descaleOffset = splatMin+splatMin-splatMax;
			m_descaleSlope  = 2.0 * (splatMax-splatMin);

			m_nUVScalingComputed = TRUE;
		}
		surfelStruct->normal  = Vector3D(BYTE2SCALAR(pNormal[0]),
										 BYTE2SCALAR(pNormal[1]),
										 BYTE2SCALAR(pNormal[2]));
		surfelStruct->normal.normalize();
		BOOL reverse = FALSE;
		if ( surfelStruct->normal[2] < 0 ) {
			// Reverse backward surfels
			surfelStruct->normal *= -1.0;
			reverse = TRUE;
		}
		surfelStruct->axisOne =
			descaleSplat(pSplatU, m_descaleSlope, m_descaleOffset, reverse);
		surfelStruct->axisTwo =
			descaleSplat(pSplatV, m_descaleSlope, m_descaleOffset, reverse);
	} else {
		surfelStruct->normal  = Vector3D(0.0, 0.0, 1.0);
		surfelStruct->axisOne = Vector3D(1.0/m_nWidth, 0.0, 0.0);
		surfelStruct->axisTwo = Vector3D(0.0, 1.0/m_nHeight, 0.0);
	}

	return 1;
}

// Render object description

/* virtual */ void
SimpleTextureProxyV2::RenderDescription (OpenGLVisualRenderer * pRenderer,
										 DWORD /* dwTime */ )
{
	RenderDescriptionBox (pRenderer, "SimpleTextureV2");
}

BOOL
SimpleTextureProxyV2::GetImageInfoFromImageTexture (ImageTextureProxy *
													pImgProxy, int nImageType)
{
	ImageTexture *pIMO = pImgProxy->GetImageTexture ();
	MediaStream *poStream = (MediaStream *) pIMO->GetStream ();

	if (pImgProxy->GetFirst ()) {
		pImgProxy->Start ();
		pIMO->GetExecutive ()->RequestChannels ();
		pImgProxy->SetFirst (FALSE);
	}

	if (poStream) {
		int nSize;
		DWORD dwTime;			// Originally commented 2003.06.26 In Kyu Park
		BOOL bEOU;
		int nStatus;
		BOOL bSkip;
		BOOL bWait;
		long lDelta;
		BOOL bDontClose;

		bSkip = TRUE;
		bWait = FALSE;
		lDelta = 0;
		bDontClose = TRUE;

		unsigned char *pData =
			(unsigned char *) poStream->Fetch (nSize, dwTime, bEOU, nStatus,
											   bSkip, bWait, lDelta,
											   bDontClose);

		if (pData != END_OF_STREAM) {
			if (pData) {
				if (nImageType == NODE_TEXTURE) {
					m_nBufferFetched[NODE_TEXTURE] = true;
					m_nWidth = (((unsigned short) pData[0]) << 8) | pData[1];
					m_nHeight = (((unsigned short) pData[2]) << 8) | pData[3];
					m_pColorBuf = pData + 4;
					int nMediaFormat = pIMO->GetStreamFormat ();
					m_nColorBytePerPixel =
						nMediaFormat == MediaFormat::RGBA ? 4 : 3;
				} else if (nImageType == NODE_DEPTH) {
					m_nBufferFetched[NODE_DEPTH] = true;

					// For PNG
					m_nWidth =
						(((int) pData[0]) << 24) | (((int) pData[1]) << 16)
						| (((int) pData[2]) << 8) | pData[3];
					m_nHeight =
						(((int) pData[4]) << 24) | (((int) pData[5]) << 16)
						| (((int) pData[6]) << 8) | pData[7];

//2003.10.7 J.Y.HAN
					m_pDepthBuf = pData + 10;
					int _color = pData[8];

					switch (_color) {
					case 1:
						m_nDepthBytePerPixel = pData[9];
						break;

					case 3:
						m_nDepthBytePerPixel = 3;
						break;

					default:
						break;
					}
					m_nMaxDepth = pow (2, m_nDepthBytePerPixel * 8) - 1;
//-
				} else if (nImageType == NODE_NORMAL) {
					m_nBufferFetched[NODE_NORMAL] = TRUE;
					m_pNormalBuf = pData + 10;
					int _color = pData[8]; // For PNG
					switch (_color) {
					case 3: m_nNormalBytePerPixel = 3; break;
					case 4:	m_nNormalBytePerPixel = 4; break;
					default: break;
					}
				} else if (nImageType == NODE_SPLATU) {
					m_nBufferFetched[NODE_SPLATU] = TRUE;
					m_pSplatUBuf = pData + 10;
					int _color = pData[8]; // For PNG
					switch (_color) {
					case 3: m_nSplatUBytePerPixel = 3; break;
					case 4:	m_nSplatUBytePerPixel = 4; break;
					default: break;
					}
				} else if (nImageType == NODE_SPLATV) {
					m_nBufferFetched[NODE_SPLATV] = TRUE;
					m_pSplatVBuf = pData + 10;
					int _color = pData[8]; // For PNG
					switch (_color) {
					case 3: m_nSplatVBytePerPixel = 3; break;
					case 4:	m_nSplatVBytePerPixel = 4; break;
					default: break;
					}
				} else {
					return FALSE;
				}
				if ( m_nBufferFetched[NODE_DEPTH]
					 && m_nBufferFetched[NODE_TEXTURE]
					 && m_nBufferFetched[NODE_NORMAL]
					 && m_nBufferFetched[NODE_SPLATU]
					 && m_nBufferFetched[NODE_SPLATV] ) {
					poStream->Release (TRUE);
				}
			}
		}
	}

	return TRUE;
}

#include <fstream.h>
extern unsigned char *gpucClipZero;
#define	Clip(x)	*(gpucClipZero + (x))

extern void ConvertYUVtripleToRGB (int y, int u, int v, int &r, int &g,
								   int &b);
extern void ConvertYUV2RGB (int nWidth, int nHeight, unsigned char *pYUVFrame,
							unsigned char *pRGBFrame);
extern void ConvertYUV2RGB2 (int nWidth, int nHeight,
							 unsigned char *pYUVFrame,
							 unsigned char *pRGBFrame);

#include "istream.h"

BOOL
SimpleTextureProxyV2::GetImageInfoFromMovieTexture (	//NodeField* pImgNode,
													   MovieTextureProxy *
													   pImgProxy,
													   int nImageType)
{
	MovieTexture *pIMO = pImgProxy->GetMovieTexture ();
	MediaStream *pStream = (MediaStream *) pIMO->GetStream ();

	if (pStream) {
		pImgProxy->TestActive ();
		if (pImgProxy->GetActive ()) {
			int nSize;
			DWORD dwTime;
			BOOL bEOU;
			int nStatus;
			BOOL bSkip;
			BOOL bWait;
			long lDelta;
			BOOL bDontClose;

			// SB @@@@ I am calling Fetch without using nRecurrent!!!
			// what is th edefault? guess: FALSE
			bSkip = TRUE;
			bWait = FALSE;
			lDelta = 0;
			bDontClose = TRUE;
			unsigned char *pData =
				(unsigned char *) pStream->Fetch (nSize, dwTime, bEOU,
												  nStatus, bSkip, bWait,
												  lDelta, bDontClose);

			if (pData != END_OF_STREAM) {
				if (pData) {
					if (nImageType == NODE_TEXTURE) {
						if (m_nColorVideoStatus == 0)
							m_nColorVideoStatus = 1;
					} else if (nImageType == NODE_DEPTH) {
						if (m_nDepthVideoStatus == 0)
							m_nDepthVideoStatus = 1;
					} else
						return FALSE;

					// check nStatus
					if (nStatus != 0)
						return false;

					// header information
					m_nNumberOfFrame = ((int *) pData)[0];
					if (m_nNumberOfFrame < m_nPreviousFrame) {
						// header information
						m_nWidth = ((int *) pData)[1];
						m_nHeight = ((int *) pData)[2];
						m_nNumberOfChannels = ((int *) pData)[3];
						m_nBytePerPixel = ((int *) pData)[4];

						// frame information
						m_nRGBFrameSize = m_nWidth * m_nHeight * 3;
						m_nYUVFrameSize = m_nWidth * m_nHeight * 3 / 2;
						m_nCurrentFrame = 0;
						m_nPreviousFrame = 0;

						// video buffer
						if (m_pVideo != NULL)
							free (m_pVideo);
						m_pVideo = NULL;

						if (nImageType == NODE_TEXTURE) {
							m_pMovieColorBuf = NULL;
						}
						if (nImageType == NODE_DEPTH) {
							m_pMovieDepthBuf = NULL;
						}
					} else if (m_nNumberOfFrame > m_nPreviousFrame) {
						// check frame
						if (m_nCurrentFrame + 1 >= m_nNumberOfFrame)
							m_nCurrentFrame = m_nNumberOfFrame - 1;
						else if (m_nCurrentFrame + 1 < m_nNumberOfFrame)
							m_nCurrentFrame++;
					} else
						return false;

					// video buffer
					m_pVideo =
						(unsigned char *) realloc ((void *) m_pVideo,
												   m_nNumberOfFrame *
												   m_nYUVFrameSize);
					memcpy (m_pVideo + m_nPreviousFrame * m_nYUVFrameSize,
							pData + sizeof (int) * 5 +
							m_nPreviousFrame * m_nYUVFrameSize,
							(m_nNumberOfFrame -
							 m_nPreviousFrame) * m_nYUVFrameSize);

					// convert yuv to rgb
					unsigned char *pRGBFrame =
						(unsigned char *) malloc (m_nRGBFrameSize);

					if (nImageType == NODE_TEXTURE) {
						m_pMovieColorBuf =
							(unsigned char *) realloc ((void *)
													   m_pMovieColorBuf,
													   m_nNumberOfFrame *
													   m_nRGBFrameSize);

						for (int i = m_nPreviousFrame; i < m_nNumberOfFrame;
							 i++) {
							ConvertYUV2RGB (m_nWidth, m_nHeight,
											m_pVideo + i * m_nYUVFrameSize,
											pRGBFrame);
							memcpy (m_pMovieColorBuf + i * m_nRGBFrameSize,
									pRGBFrame, m_nRGBFrameSize);
						}
					} else if (nImageType == NODE_DEPTH) {
						m_pMovieDepthBuf =
							(unsigned char *) realloc ((void *)
													   m_pMovieDepthBuf,
													   m_nNumberOfFrame *
													   m_nRGBFrameSize);



						for (int i = m_nPreviousFrame; i < m_nNumberOfFrame;
							 i++) {
							ConvertYUV2RGB (m_nWidth, m_nHeight,
											m_pVideo + i * m_nYUVFrameSize,
											pRGBFrame);
							memcpy (m_pMovieDepthBuf + i * m_nRGBFrameSize,
									pRGBFrame, m_nRGBFrameSize);
						}
					}
					free (pRGBFrame);
					pStream->Release (TRUE);
				}
			} else {
				if (nImageType == NODE_TEXTURE) {
					if (m_nColorVideoStatus == 1) {
						m_nColorVideoFetched = TRUE;
						m_nColorVideoStatus = 2;
						m_nColorBytePerPixel = m_nBytePerPixel;
					}
				} else if (nImageType == NODE_DEPTH) {
					if (m_nDepthVideoStatus == 1) {
						m_nDepthVideoFetched = TRUE;
						m_nDepthVideoStatus = 2;
						m_nDepthBytePerPixel = m_nBytePerPixel;
						m_nMaxDepth = pow (2, m_nDepthBytePerPixel * 8) - 1;
					}
				}
			}
		}
	}

	return TRUE;
}

void
SimpleTextureProxyV2::SwapImageRows ()
{
	int bps, isize;

	m_pOTColorBuf = NULL;
	bps = m_nColorBytePerPixel * m_nWidth;
	isize = bps * m_nHeight;

	m_pOTColorBuf = (unsigned char *) malloc (isize);

	for (int y = 0; y < m_nHeight; y++) {
		memcpy (m_pOTColorBuf + y * bps,
				m_pColorBuf + (m_nHeight - y - 1) * bps, bps);
	}
}

void
SimpleTextureProxyV2::SwapVideoRows ()
{
	int bps, isize, videosize;

	m_pOTColorBuf = NULL;
	bps = m_nBytePerPixel * m_nWidth;
	isize = bps * m_nHeight;
	videosize = isize * m_nNumberOfFrame;

	m_pOTMovieColorBuf = (unsigned char *) malloc (videosize);

	for (int f = 0; f < m_nNumberOfFrame; f++) {
		for (int y = 0; y < m_nHeight; y++) {
			memcpy (m_pOTMovieColorBuf + f * isize + y * bps,
					m_pMovieColorBuf + f * isize + (m_nHeight - y - 1) * bps,
					bps);
		}
	}
}
#endif // SAIT_CONFORMANCE_2003
