// -*- Mode: c++; tab-width: 4; c-basic-offset: 4;-*-

// This software module was adapted by Christopher Lee (MERL) 
// from DIBR/SimpleTextureProxy.h by In Kyu Park (SAMSUNG AIT)
// and PointShop3D 2.0 by Richard Keiser, Oliver Knoll, Mark Pauly,
// and Matthias Zwicker (Computer Graphics Lab, ETH Zurich),
// in the course of development of the MPEG-4 Systems standard
// (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// Copyright 2004.
//
// PointShop3D is available separately, under the GNU General Public License,
//  at http://graphics.ethz.ch/pointshop3d/index.html

#ifdef SAIT_CONFORMANCE_2003

#ifndef __SIMPLETEXTUREPROXYV2_H__
#define __SIMPLETEXTUREPROXYV2_H__

#include "DIBR2-common.h"

#include "../../Geometry/GeometryProxy.h"
#include "DepthImageProxyV2.h"

class SimpleTextureV2;
class ImageTextureProxy;
class OpenGLVisualRenderer;
class MediaObjectProxy;

class MovieTextureProxy;		//2003.10.7 J.Y.HAN

/// SimpleTextureV2 renderer node

class SimpleTextureProxyV2:public MediaObjectProxy
{
  protected:

	int m_nWidth;
	int m_nHeight;
	int m_nDepthBytePerPixel;
	unsigned int m_nMaxDepth;
	int m_nColorBytePerPixel;
	int m_nNormalBytePerPixel;
	int m_nSplatUBytePerPixel;
	int m_nSplatVBytePerPixel;
	bool m_nAnim;
	unsigned char *m_pColorBuf;
	unsigned char *m_pDepthBuf;
	unsigned char *m_pNormalBuf;
	unsigned char *m_pSplatUBuf;
	unsigned char *m_pSplatVBuf;

	unsigned char *texture;
	unsigned char *depth;
	unsigned char *normal;
	unsigned char *splatU;
	unsigned char *splatV;

	BOOL IsColorFetched, IsDepthFetched;

  public:
//2003.10.7 J.Y.HAN
	unsigned char *m_pMovieColorBuf;
	unsigned char *m_pMovieDepthBuf;
	unsigned char *m_pOTColorBuf;	//Octree Image Buffer
	unsigned char *m_pOTMovieColorBuf;

	// header information
	int m_nNumberOfFrame;
	int m_nNumberOfChannels;
	int m_nBytePerPixel;

	// frame information
	int m_nRGBFrameSize;
	int m_nYUVFrameSize;
	int m_nCurrentFrame;
	int m_nPreviousFrame;
	bool m_bDump;

	int DepthPreviousFrame;
	int ColorPreviousFrame;

	int ColorImageSize;
	int DepthImageSize;

	// video buffer
	unsigned char *m_pVideo;

	int m_nColorVideoStatus, m_nDepthVideoStatus;
	BOOL m_nColorVideoFetched, m_nDepthVideoFetched;
	BOOL m_nBufferFetched[5];
//-
	BOOL m_nUVScalingComputed;
	double m_descaleOffset, m_descaleSlope;

  public:

	/// Utilites
	int GetWidth (void) { return m_nWidth; };
	int GetHeight (void) { return m_nHeight; };
	int GetDepthBytePerPixel (void) { return m_nDepthBytePerPixel; };
	int GetColorBytePerPixel (void) { return m_nColorBytePerPixel; };
	int GetMaxDepth (void) { return m_nMaxDepth; };
	bool GetAnim (void) { return m_nAnim; };
	unsigned char *GetColorBuf () { return m_pColorBuf; }

	void SetWidth (int Width) { m_nWidth = Width; };
	void SetHeight (int Height) { m_nHeight = Height; };
	void SetDepthBytePerPixel (int Depth) { m_nDepthBytePerPixel = Depth; };
	void SetColorBytePerPixel (int Color) { m_nColorBytePerPixel = Color; };
	void SetAnim (bool Anim) { m_nAnim = Anim; };
	int SetMaxDepth (int MaxDepth) { m_nMaxDepth = MaxDepth; };

	BOOL GetImageInfoFromImageTexture (ImageTextureProxy *
									   pImageTextureProxey, int nImageType);
//2003.10.7 J.Y.HAN
	BOOL GetImageInfoFromMovieTexture (MovieTextureProxy *
									   pMovieTextureProxey, int nImageType);
	BOOL GetImage (int FrameIdx);
	SurfelIterator* getSurfelIterator (int nearPlane, int farPlane);
	int GetSurfelStruct (SurfelStruct* surfelStruct, int SurfelIdx,
						 int nearPlane, int farPlane);


	void SwapImageRows ();
	void SwapVideoRows ();
//-     
	/// Constructor
	SimpleTextureProxyV2 (SimpleTextureV2 * pSimpleTexture);

	~SimpleTextureProxyV2 () {
		if (m_pMovieColorBuf != NULL)
			free (m_pMovieColorBuf);
		if (m_pMovieDepthBuf != NULL)
			free (m_pMovieDepthBuf);

		if (m_pOTColorBuf != NULL)
			free (m_pOTColorBuf);
		if (m_pOTMovieColorBuf != NULL)
			free (m_pOTMovieColorBuf);
		//TODO -- kill old nodes here
	}

	/// Render object description
	virtual void RenderDescription (OpenGLVisualRenderer * pRenderer,
									DWORD dwTime);

  public:
	SimpleTextureV2 * GetSimpleTexture () const	{
		return (SimpleTextureV2 *) GetMediaObject ();
	};

  private:
	void loadSimpleTextureImage (int nNode);
};

#include "SimpleTextureProxyV2.i"

#endif // __SIMPLETEXTUREPROXYV2_H__

#endif // SAIT_CONFORMANCE_2003

