/********************************************************************************/
/* This software module was developped by										*/

/*	   Royan Jerome , jerome.royan@francetelecom.com							*/
/*                   France Telecom R&D, France		*/

/* in the course of development of the MPEG-4 standard (Part 16 AFX).			*/
/* This software module is an implementation of a part of one or				*/
/* more MPEG-4 tools as specified by the										*/
/* MPEG-4 standard. ISO/IEC gives users of the MPEG-4 standard					*/
/* free license to this software module or modifications thereof				*/
/* for use in hardware or software products claiming conformance				*/
/* to the MPEG-4 standard. Those intending to use this software					*/
/* module in hardware or software products are advised that its					*/
/* use may infringe existing patents. The original developer of					*/
/* this software module and his company, the subsequent							*/
/* editors and their companies, and ISO/IEC have no liability					*/
/* for use of this software module or modifications thereof in					*/
/* an implementation. Copyright is not released for non							*/
/* MPEG-4 standard conforming products. INT ARTEMIS								*/
/* retain full right to use the code for their own purposes,					*/
/* assign or donate the code to a third party and to inhibit					*/
/* third parties from using the code for non MPEG-4 standard					*/
/* conforming products. This copyright notice must be included					*/
/* in all copies or derivative works.    										*/
/* Copyright (c)2002 - 2005														*/
/********************************************************************************/

// FootPrintSetProxy.cpp: implementation of the FootPrintSetProxy class.
// dependences :
//////////////////////////////////////////////////////////////////////

#include "BuildingPartProxy.h"
#include "FacadeProxy.h"
#include "RoofProxy.h"
#include "IM1Decoders/AFX/AFX.h"
//#include "IM1Decoders/AFX/FootPrintDecoder.h"

#include "opengl/group/GroupProxy.h"
#include "opengl/group/SwitchProxy.h"
#include "opengl/group/InlineProxy.h"
#include "opengl/geometry/ShapeProxy.h"
//#include "opengl/geometry/TransformProxy.h"
#include "opengl/base/OpenGLVisualRenderer.h"
#include "opengl/base/OpenGLPresenter.h"
#include "Presenter.h"
#include "Inline.h"
/*
#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif
*/


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////


BuildingPartProxy::BuildingPartProxy(BuildingPart* buildingPart)
  : AfxStreamProxy(buildingPart)
{
  m_bInit = false;
  m_lastTime = 0;
//  m_bIs3DReconstructed=false;
}

BuildingPartProxy::BuildingPartProxy(BuildingPart* buildingPart, FPBuilding* fpBuilding)
  : AfxStreamProxy(buildingPart)
{
	fpBuilding->SetMediaObject((MediaObject*)GetFootPrintNode());
	m_pBuilding=fpBuilding;
	m_bInit = false;
	m_lastTime = 0;
//  m_bIs3DReconstructed=false;
	for (int num_fac=0; num_fac<buildingPart->facades.GetLength(); num_fac++)
	{
		FacadeProxy* facadeProxy=new FacadeProxy((Facade*)buildingPart->facades.GetObject(num_fac), (FPFacade*)fpBuilding->GetFacades()->ElementAt(num_fac));
		buildingPart->facades.GetObject(num_fac)->SetRenderingObject(facadeProxy);
	}
	for (int num_roof=0; num_roof<buildingPart->roofs.GetLength(); num_roof++)
	{
		RoofProxy* roofProxy=new RoofProxy((Roof*)buildingPart->roofs.GetObject(num_roof), (FPRoof*)fpBuilding->GetRoofs()->ElementAt(num_roof));
		buildingPart->roofs.GetObject(num_roof)->SetRenderingObject(roofProxy);
	}
}

BuildingPartProxy::~BuildingPartProxy(void)
{

}

BOOL BuildingPartProxy::Select(OpenGLVisualRenderer* pRenderer, DWORD dwTime, unsigned int& depth)
{
  GLuint selectBuffer[4];

  ::glSelectBuffer(4, selectBuffer);
  ::glRenderMode(GL_SELECT);

  Render(pRenderer, dwTime);

  if (::glRenderMode(GL_RENDER) > 0)
    if (depth >= selectBuffer[1]) {
      depth = selectBuffer[1];

      return TRUE;
    }

  return FALSE;
}

BOOL BuildingPartProxy::RenderFromStream(MediaStream* poStream, int id, DWORD dwTime)
{
  // it is called by BitWrapper
  // it should retrieve from the decoder the data and initialise the fields

  // first time get the decoder specific info from the stream

  // get the stream
 
  // get the decoded data buffer
 
	//Copy the decoded mesh into the composition buffer
	//DataExchange();
	

	
	// update time

	  return TRUE;
}

BOOL BuildingPartProxy::RenderFromBuffer(SFString *buffer, int id)
{
  
  return TRUE;
}

BOOL BuildingPartProxy::Render(DWORD dwTime)
{
	return TRUE;
}

BOOL BuildingPartProxy::Render(OpenGLVisualRenderer* pRenderer, DWORD dwTime)
{
  // called when the node is encoded with BIFS
 // Render(dwTime);

	glPushAttrib(GL_POLYGON_BIT);

	glEnable(GL_LIGHTING);
	glCullFace(GL_FRONT_AND_BACK);

	if (m_pBuilding->GetNbSwapNodes()==0)
	{
		if (m_pBuilding->State3D()==COMPUTED)
		{
			BuildingPart* node=(BuildingPart*)m_pBuilding->GetMediaObject(); 
			
			// Render Facades
			for (int num_fac=0; num_fac<node->facades.GetLength(); num_fac++)
				((FacadeProxy*)node->facades.GetObject(num_fac)->GetRenderingObject())->Render(pRenderer,dwTime);

			// Render Roofs
			for (int num_roof=0; num_roof<node->roofs.GetLength(); num_roof++)
				((RoofProxy*)node->roofs.GetObject(num_roof)->GetRenderingObject())->Render(pRenderer,dwTime);
		}

	}

	glDisable(GL_LIGHTING);

	glPopAttrib();
  return TRUE; 
}

void BuildingPartProxy::RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD dwTime)
{
  BuildingPart* pBuildingPart = (BuildingPart*) GetMediaObject();
  GLfloat
	  color[] = { 0.0f, 1.0f, 0.0f, 0.75f};

  glColor4fv(color);

  RenderDescriptionBox(pRenderer, "BuildingPart");
}

