/********************************************************************************/
/* This software module was developped by										*/

/*	   Royan Jerme , jerome.royan@francetelecom.com								*/
/*                   France Telecom R&D, France		*/

/* in the course of development of the MPEG-4 standard (Part 16 AFX).			*/
/* This software module is an implementation of a part of one or				*/
/* more MPEG-4 tools as specified by the										*/
/* MPEG-4 standard. ISO/IEC gives users of the MPEG-4 standard					*/
/* free license to this software module or modifications thereof				*/
/* for use in hardware or software products claiming conformance				*/
/* to the MPEG-4 standard. Those intending to use this software					*/
/* module in hardware or software products are advised that its					*/
/* use may infringe existing patents. The original developer of					*/
/* this software module and his company, the subsequent							*/
/* editors and their companies, and ISO/IEC have no liability					*/
/* for use of this software module or modifications thereof in					*/
/* an implementation. Copyright is not released for non							*/
/* MPEG-4 standard conforming products. INT ARTEMIS								*/
/* retain full right to use the code for their own purposes,					*/
/* assign or donate the code to a third party and to inhibit					*/
/* third parties from using the code for non MPEG-4 standard					*/
/* conforming products. This copyright notice must be included					*/
/* in all copies or derivative works.    										*/
/* Copyright (c)2002 - 2005														*/
/********************************************************************************/

// FootPrintSetProxy.cpp: implementation of the FootPrintSetProxy class.
// dependences :
//////////////////////////////////////////////////////////////////////


#include "FootPrintProxy.h"
#include "BuildingPartProxy.h"
#include "FacadeProxy.h"
#include "RoofProxy.h"
#include "FootPrintSetProxy.h"


#include "opengl/AFX/BitWrapperProxy.h"
#include "IM1Decoders/AFX/AFX.h"
#include "IM1Decoders/AFX/FootPrintDecoder.h"

#include "opengl/group/SwitchProxy.h"
#include "opengl/base/OpenGLVisualRenderer.h"
#include "opengl/base/OpenGLPresenter.h"
#include "Presenter.h"


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

FootPrintSetProxy::FootPrintSetProxy(FootPrintSet* footprintset)
  : AfxStreamProxy(footprintset)
{
  m_pFPSet = new FPSet();
  m_pFPSet->SetMediaObject((MediaObject*)GetFootPrintSetNode());
  m_bInit = false;
  m_lastTime = 0;
}

FootPrintSetProxy::~FootPrintSetProxy(void)
{
	delete m_pFPSet;
	m_pFPSet=NULL;	
}


BOOL FootPrintSetProxy::Select(OpenGLVisualRenderer* pRenderer, DWORD dwTime, unsigned int& depth)
{
  GLuint selectBuffer[4];

  ::glSelectBuffer(4, selectBuffer);
  ::glRenderMode(GL_SELECT);

  Render(pRenderer, dwTime);

  if (::glRenderMode(GL_RENDER) > 0)
    if (depth >= selectBuffer[1]) {
      depth = selectBuffer[1];

      return TRUE;
    }

  return FALSE;
}

BOOL FootPrintSetProxy::RenderFromStream(MediaStream* poStream, int id, DWORD dwTime)
{
  // it is called by BitWrapper
  // it should retrieve from the decoder the data and initialise the fields

  FootPrintSet *pObject = GetFootPrintSetNode();
  BitWrapper *pParent = (BitWrapper *)pObject->GetParent();
  ASSERT(pParent != NULL);

  // first time get the decoder specific info from the stream
  if (!m_bInit)
  {
	m_bInit = true;
  }

  // get the stream
  MediaStream * pStream = (MediaStream *)pParent->GetStream();
  ASSERT(pStream != NULL);

  // get the decoded data buffer
  int rnSize, rnStatus;
  DWORD rdvTime;
  BOOL rbEOU;
	BYTE * pOutput = (BYTE *)pStream->Fetch(rnSize, rdvTime, rbEOU, rnStatus);


	//Copy the decoded mesh into the composition buffer
	//DataExchange();
	
	// update time
	if (rdvTime > m_lastTime) {
		m_lastTime = rdvTime;
		return TRUE;
	}

	  return FALSE;
}

BOOL FootPrintSetProxy::RenderFromBuffer(SFString *buffer, int id)
{
  FPDecoderLib FPDec;
  FPDec.SetFPSet(m_pFPSet);
  FPDec.DecodeFromBuffer (buffer->GetValue(),buffer->GetLength());
  return TRUE;
}

BOOL FootPrintSetProxy::Render(DWORD dwTime)
{
	FootPrintSet* pFPSet = (FootPrintSet*) GetMediaObject();

	if (m_pFPSet->ActiveNodes().GetLength()!=0)
	{
		int i;

		// Get the viewpoint position
		Viewpoint* pActiveViewpoint = GetMediaObject()->GetRoot()->GetBoundViewpoint();
		SF<Vec3f>& position = *(SF<Vec3f>*) pActiveViewpoint->GetField(4);
		Vec3f* viewpoint=new Vec3f(position.GetValue().x,position.GetValue().y,position.GetValue().z);

		// Test the node to be add or reomve from the scene graph due to the viewpoint position
		ZArray<FP*> NodesToModify;
		ZArray<int> NodesState;
		m_pFPSet->TestActiveNodes(viewpoint,200, NodesToModify, NodesState);
		m_pFPSet->TestCollapsableNodes(viewpoint,150, NodesToModify, NodesState);

		for (i=0; i<NodesToModify.GetLength(); i++)
		{
			FP* footprint=NodesToModify[i];
			if (NodesState[i])		// If the node has to be add to the scene graph
			{
				int index=footprint->Index();
				FootPrint* node=(FootPrint*)footprint->GetMediaObject();
				if (node==NULL)
				{
					node=new FootPrint();
					footprint->SetMediaObject(node);
					node->index=footprint->Index();

					Vec2f** vertices=footprint->FootPrint();
					unsigned int* nbVerticesByRing=footprint->NbVerticesByRing();
					int nodeType;
					IndexedLineSet2D* ils=(IndexedLineSet2D*)node->footprint.GetNode();
					ils->coord.SetObject(new Coordinate(), (MediaObject*)&(node->footprint));
					
					Coordinate* CRD=(Coordinate*)ils->coord.GetNode();
					ZArray<Vec2f>* AR=(ZArray<Vec2f>*)(&(CRD->point.GetArray()));

					MF<int> CRDIND = (MF<int>)ils->coordIndex;
					ZArray<int>* CAR = (ZArray<int>*)(&(CRDIND.GetArray()));

					for (int j=0; j<footprint->NbRings(); j++)
					{
						for (unsigned int k=0; k<nbVerticesByRing[j]; k++)
						{
							(*AR)+=Vec2f(vertices[j][k].x,vertices[j][k].y);
							(*CAR)+=k;
						}
						(*CAR)+=-1;
					}
					switch (footprint->Type())
					{
						case FOOTPRINT :
						{
							FootPrintProxy* fpProxy=new FootPrintProxy((FootPrint*)node, footprint);
							node->SetRenderingObject(fpProxy);
							break;
						}
						case BUILDING :
						{
							BuildingPartProxy* fpProxy=new BuildingPartProxy((BuildingPart*)node, (FPBuilding*)footprint);
							node->SetRenderingObject(fpProxy);
							break;
						}
					}
				}

				MediaObject* obj=GetMediaObject();
				GetMediaObject()->AddChild(footprint->GetMediaObject());

			}
			else		// If the node has to be remove from the scene graph
			{
				int index=footprint->Index();
				if (footprint!=m_pFPSet->Root())
				{
					GetMediaObject()->RemoveChild(footprint->GetMediaObject());
					GetMediaObject()->DeleteChild(footprint->GetMediaObject());
					
				}
			}
		}

		delete viewpoint;
	}
	
	return TRUE;
}

BOOL FootPrintSetProxy::Render(OpenGLVisualRenderer* pRenderer, DWORD dwTime)
{
  // called when the node is encoded with BIFS

	FootPrintSet* pFPSet = (FootPrintSet*) GetMediaObject();

	ZArray<ZString>* swapNodesToSwitchOn=m_pFPSet->SwapNodesToSwitchOn();
	ZArray<ZString>* swapNodesToSwitchOff=m_pFPSet->SwapNodesToSwitchOff();

	Root* pSceneRoot = NULL;
	Switch* pSwitch = NULL;
	OpenGLPresenter* pPresenter = pRenderer->GetPresenter();
	if (pPresenter != NULL)
	{
		pSceneRoot = pPresenter->GetScene();
	}
	
	for (int numNode=0; numNode<swapNodesToSwitchOn->GetLength(); numNode++)
	{
		if (pSceneRoot != NULL)
		{
			pSwitch=(Switch*)pSceneRoot->FindNode(LPCSTR((*swapNodesToSwitchOn)[numNode]));
		}

		if (pSwitch!=NULL)
		{		
			pSwitch->whichChoice=0;
		}

	}
	swapNodesToSwitchOn->Clear();

	for (numNode=0; numNode<swapNodesToSwitchOff->GetLength(); numNode++)
	{
		if (pSceneRoot != NULL)
		{
			pSwitch=(Switch*)pSceneRoot->FindNode(LPCSTR((*swapNodesToSwitchOff)[numNode]));
		}
		if (pSwitch!=NULL)
		{		
			pSwitch->whichChoice=-1;
		}
	}
	swapNodesToSwitchOff->Clear();

	if (m_pFPSet->ActiveNodes().GetLength()!=0)
	{
		int i;

		// Get the viewpoint position
		Viewpoint* pActiveViewpoint = GetMediaObject()->GetRoot()->GetBoundViewpoint();
		SF<Vec3f>& position = *(SF<Vec3f>*) pActiveViewpoint->GetField(4);
		Vec3f* viewpoint=new Vec3f(position.GetValue().x,position.GetValue().y,position.GetValue().z);

		// Test the node to be add or reomve from the scene graph due to the viewpoint position
		ZArray<FP*> NodesToModify;
		ZArray<int> NodesState;
		m_pFPSet->TestActiveNodes(viewpoint,200, NodesToModify, NodesState);
		m_pFPSet->TestCollapsableNodes(viewpoint,150, NodesToModify, NodesState);

		for (i=0; i<NodesToModify.GetLength(); i++)
		{
			FP* footprint=NodesToModify[i];
			if (NodesState[i]==1)		// If the node has to be add to the scene graph
			{
				int index=footprint->Index();
				switch (footprint->Type())
				{
					case FOOTPRINT :
					{
						FootPrint* node=(FootPrint*)footprint->GetMediaObject();
						if (node==NULL)
						{
							node=CreateFootPrintNode(footprint);
						}
						MediaObject* obj=GetMediaObject();
						GetMediaObject()->AddChild(node);
						break;
					}
					case BUILDING :
					{
						BuildingPart* node=(BuildingPart*)footprint->GetMediaObject();
						if (node==NULL)
						{
							node=CreateBuildingPartNode((FPBuilding*)footprint);
						}
						MediaObject* obj=GetMediaObject();
						GetMediaObject()->AddChild(node);
						break;
					}
				}
			}
			else if (NodesState[i]==2)		// If the node has to be remove from the scene graph
			{
				int index=footprint->Index();
				if (footprint!=m_pFPSet->Root())
				{
					GetMediaObject()->RemoveChild(footprint->GetMediaObject());
				}
			}

		}

		delete viewpoint;
	}
	
	
	ZArray<FootPrintProxy *> nodes;
  

	// Rendering of all footprints (all children of the FootPrintSet
	int nbchildren=pFPSet->GetChildrenCount();
	if (nbchildren!=0)
	{		
		for (int j=0; j<nbchildren; j++)
		{
			FootPrint* pObject=(FootPrint*)pFPSet->GetChild(j);
			((FootPrintProxy *) (pObject->GetRenderingObject()))->Render(pRenderer,dwTime);
		}
	}

  return TRUE;
}

void FootPrintSetProxy::RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD dwTime)
{
  FootPrintSet* pFootPrintSet = (FootPrintSet*) GetMediaObject();
  GLfloat color[] = { 0.5f, 0.0f, 1.0f, 0.75f};

  glColor4fv(color);
  ::glPushMatrix();
  
  ::glScalef(1.0f, 1.0f, 1.0f);
  ::glTranslatef(0.0f, -2.0f, 0.0f);

  RenderDescriptionBox(pRenderer, "FootPrintSet");

  ::glTranslatef(0.0f, -1.0f, 0.0f);

  ZString text("FootPrint (");
  char nbchildren[10];
  _itoa( pFootPrintSet->GetChildrenCount(),nbchildren,10);
  text+=nbchildren;
  text+=")";
  

  RenderDescriptionBox(pRenderer,(LPCSTR)text);
  
  ::glPopMatrix();
  ::glPopMatrix();

 }

void FootPrintSetProxy::SetupDecoderData(BitWrapperProxy *pParent)
{
  FootPrintSet *pObject = GetFootPrintSetNode();

  BitWrapper* poBitWrapper = pParent->GetBitWrapper();
  AFXDecoder *pDecoder = (AFXDecoder *)poBitWrapper->GetDecoder();
  ASSERT(pDecoder != NULL);
  FootPrintDecoder *fpeDec = (FootPrintDecoder *)pDecoder->GetDecoder();
  ASSERT(fpeDec != NULL);

  fpeDec->SetRenderingObject(this);
}

bool FootPrintSetProxy::AddFPs(ZArray<FP*> &fps, int parentIndex)
{
	m_pFPSet->AddFootPrints(fps,parentIndex);
	FP* parent;
	if (parentIndex!=-1)
		parent=m_pFPSet->GetFootPrint(parentIndex);
	else
		parent=m_pFPSet->Root();


	return true;
}

bool FootPrintSetProxy::AddFPNode(FP* fp)
{
	Vec2f** footprint=fp->FootPrint();
	int NbRings=fp->NbRings();
	unsigned int* nbVerticesByRing=fp->NbVerticesByRing();
	for (int i=0; i<NbRings; i++)
	{		
		IndexedLineSet2D* il2D=new IndexedLineSet2D();
		SFNode<class Coordinate2D> coord=il2D->coord;
		Coordinate* CRD = (Coordinate*)coord.GetNode();
		ZArray<Vec3f>* AR = (ZArray<Vec3f>*)(&(CRD->point.GetArray()));

		Vec2f* ring=footprint[i];
		
		for (int j=0; j<nbVerticesByRing[i]; j++)
		{
			Vec3f* vertex=new Vec3f(ring[j].x, ring[j].y, 0.0);
			(*AR)+=*vertex;
		}
	}
	
	return true;
}

bool FootPrintSetProxy::RemoveFPNode(FP*)
{

	return true;
}

FootPrint* FootPrintSetProxy::CreateFootPrintNode(FP* footprint)
{
	FootPrint* node=new FootPrint();
	footprint->SetMediaObject(node);
	node->index=footprint->Index();

	Vec2f** vertices=footprint->FootPrint();
	unsigned int* nbVerticesByRing=footprint->NbVerticesByRing();

	int nodeType;

    IndexedLineSet2D* poIndexedLineSet2D = new IndexedLineSet2D;
    IndexedLineSet2DProxy* poIndexedLineSet2DProxy = new IndexedLineSet2DProxy(poIndexedLineSet2D);
	poIndexedLineSet2D->SetRenderingObject(poIndexedLineSet2DProxy);
    
	Coordinate2D * poCoordinate = new Coordinate2D;

	MF<Vec2f> coordinates;
    MF<int> indices;
	
	int coordInd=0;
	for (int j=0; j<footprint->NbRings(); j++)
	{
		int coordIndStart=coordInd;
		for (unsigned int k=0; k<nbVerticesByRing[j]; k++, coordInd++)
		{
			coordinates.Append(Vec2f(vertices[j][k].x,vertices[j][k].y));
			poIndexedLineSet2D->coordIndex.Append(coordInd);
		}
		poIndexedLineSet2D->coordIndex.Append(coordIndStart);
		poIndexedLineSet2D->coordIndex.Append(-1);
	}
	poCoordinate->point = coordinates;
	poIndexedLineSet2D->coord.SetObject(poCoordinate, poIndexedLineSet2D);

	node->footprint.SetObject((MediaObject*)poIndexedLineSet2D, (MediaObject*)node);

	FootPrintProxy* fpProxy=new FootPrintProxy((FootPrint*)node, footprint);
	node->SetRenderingObject(fpProxy);
	
	return node;
}

BuildingPart* FootPrintSetProxy::CreateBuildingPartNode(FPBuilding* building)
{
	BuildingPart* node=new BuildingPart();
	building->SetMediaObject(node);

	node->index=building->Index();

	Vec2f** vertices=building->FootPrint();
	unsigned int* nbVerticesByRing=building->NbVerticesByRing();

	IndexedLineSet2D* ils=new IndexedLineSet2D();
	node->footprint.SetObject((MediaObject*)ils, (MediaObject*)node);
	int nodeType;

	ils->coord.SetObject(new Coordinate(), (MediaObject*)ils);
	
	Coordinate* CRD=(Coordinate*)ils->coord.GetNode();
	ZArray<Vec2f>* AR=(ZArray<Vec2f>*)(&(CRD->point.GetArray()));

	MF<int> CRDIND = (MF<int>)ils->coordIndex;
	ZArray<int>* CAR = (ZArray<int>*)(&(CRDIND.GetArray()));

	for (int j=0; j<building->NbRings(); j++)
	{
		for (unsigned int k=0; k<nbVerticesByRing[j]; k++)
		{
			(*AR)+=Vec2f(vertices[j][k].x,vertices[j][k].y);
			(*CAR)+=k;
		}
		(*CAR)+=-1;
	}

	node->buildingIndex=building->Index();
	node->height=building->Height();
	node->altitude=building->Altitude();

	ZArray<FPFacade*>* fpFacades=building->GetFacades();
	for (int num_fac=0; num_fac<fpFacades->GetLength(); num_fac++)
	{
		node->facades.InsertAt(num_fac,CreateFacadeNode((*fpFacades)[num_fac]), node);
	}

	ZArray<FPRoof*>* fpRoofs=building->GetRoofs();
	for (int num_roof=0; num_roof<fpRoofs->GetLength(); num_roof++)
	{
		node->roofs.InsertAt(num_roof,CreateRoofNode((*fpRoofs)[num_roof]), node);
	}

	BuildingPartProxy* bpProxy=new BuildingPartProxy(node, building);
	node->SetRenderingObject(bpProxy);

	return node;
}

Facade* FootPrintSetProxy::CreateFacadeNode(FPFacade* facade)
{
	int i,j;
	
	Facade* node=new Facade();
	facade->SetMediaObject(node);

	node->WidthRatio=facade->GetWidthRatio();
	node->XScale=facade->GetXScale();
	node->YScale=facade->GetYScale();
	node->XPosition=facade->GetXPosition();
	node->YPosition=facade->GetYPosition();
	node->XRepeatInterval=facade->GetXRepeatInterval();
	node->YRepeatInterval=facade->GetYRepeatInterval();
	node->Repeat=facade->GetRepeat();	
	node->NbStories=facade->GetNbStories();


	ZArray<ZArray<FPFacade*>* >* facades=facade->GetElements();
	if (facades!=NULL)
	{	
		int nbcells=0;
		for (i=0; i<facades->GetLength(); i++)
		{
			ZArray<FPFacade*>* story=facades->ElementAt(i);
			int nbCellsByStory=story->GetLength();
			node->NbFacadeCellsByStorey.GetArray()+=nbCellsByStory;
			for (j=0; j<nbCellsByStory; j++)
			{
				node->FacadeCellsArray.InsertAt(nbcells++,CreateFacadeNode(story->ElementAt(j)),node);
			}
		}
		ZArray<float>* storiesHeight=facade->GetStoriesHeight();
		for (i=0; i<storiesHeight->GetLength(); i++)
			node->StoryHeight.GetArray()+=storiesHeight->ElementAt(i);
	}
	
	FacadeProxy* facadeProxy=new FacadeProxy(node, facade);
	node->SetRenderingObject(facadeProxy);

	return node;
}

Roof* FootPrintSetProxy::CreateRoofNode(FPRoof* roof)
{
	Roof* node=new Roof();
	roof->SetMediaObject(node);

	node->Type=roof->GetType();
	node->Height=roof->GetHeight();
	node->SlopeAngle.GetArray()+=roof->GetAngle();
	node->EaveProjection=roof->GetEaveProjection();
	node->EdgeSupportIndex=roof->GetWallSupport();
	node->IsGenericTexture=roof->GetTexIsGeneric();
	node->TextureXScale=roof->GetTexXScale();
	node->TextureYScale=roof->GetTexYScale();
	node->TextureXPosition=roof->GetTexXPosition();
	node->TextureYPosition=roof->GetTexYPosition();
	node->TextureRotation=roof->GetTexRotation();

	RoofProxy* roofProxy=new RoofProxy(node, roof);
	node->SetRenderingObject(roofProxy);
	return node;

}



