/***********************************************************************
This software module was originally developed by Ioan Alexandru Salomie
(Vrije Universiteit Brussel VUB) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. VUB retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Ioan Alexandru Salomie at iasalomi@etro.vub.ac.be.
----------------------------------------------------------------------*/

#ifndef __MESHGRIDPROXY_H__
#define __MESHGRIDPROXY_H__

#include "AfxStreamProxy.h"

#include "Nodes.h"
#include "..\Im1Decoders\Afx\MshGrd\BasicTypes.h"

// class MeshGrid;
class CMGMesh;
class CMGGrid;
class BitWrapperProxy;
class MeshGridDecoderSpecificInfo;

/// MeshGrid renderer node

class MeshGridProxy : public AfxStreamProxy
{
public:
  /// Constructor
	MeshGridProxy(MeshGrid* pMG);

  /// Destructor
	~MeshGridProxy();

  /// Decode object
  bool Decode(MeshGrid *pMG);

  /// Render object
  virtual BOOL  Render(OpenGLVisualRenderer* pRenderer, DWORD dwTime);
  virtual BOOL  RenderFromStream(MediaStream* poStream, int id, DWORD dwTime);
  virtual BOOL  RenderFromBuffer(SFString *buffer, int id);
  virtual BOOL  Render(DWORD dwTime);

  /// Update the mesh
  bool OnHierarchicalLevelChanged(MeshGrid *pMG);
  bool OnGridCoordChanged(MeshGrid *pMG);
  bool OnLevelChanged(MeshGrid *pMG);
  bool OnFilterChanged(MeshGrid *pMG);
  bool OnVertexOffsetChanged(MeshGrid *pMG);

  /// Traverse scene graph, checking for intersection
  virtual BOOL  Select(OpenGLVisualRenderer* pRenderer, DWORD dwTime, unsigned int& depth);

  // moved methods from MeshGrid to MeshGridProxy
  MeshGrid * GetMeshGrid() const { return (MeshGrid *) GetMediaObject(); }

  /// Get bounding box
  void GetBbox(Vec3f& rBboxSize, Vec3f& rBboxCenter) const;
  void UpdateBbox();

  void  SetDecoded(bool flag) { m_bDecoded = flag; };
  bool  IsDecoded() { return m_bDecoded; };
  void  SetFieldsInit(bool flag) { m_bInit = flag; };
  bool  AreFieldsInit() const { return m_bInit; };
  void  SetDecodeFailed(bool flag) { m_failedDecode = flag; };
  bool  IsDecodeFailed() const { return m_failedDecode; };

private:
  // get data from stream
  bool GetGridCoord(void *pData, int size);
  bool GetVertCoord(void *pData, int size);
  bool GetTriangles(void *pData, int size);
  bool GetNormals(void *pData, int size);
  bool GetOffsets(void *pData, int size);
  bool GetNoVertices(void *pData, int size);
  void CheckTextureIndices();

  // get data from mesh and grid objects
  void GetTriangles();
  void GetNormals();
  void GetOffsets();
  void GetNoVertices();

  void GetDisplayResolution();

  // initialise and destroy mesh and grid objects
  bool CreateMGObjects();
  void DeleteMGObjects();

  // value of the node fields
  void FillNodeFields();

protected:
  bool  m_bInit;
  bool  m_bDecoded;
  bool  m_failedDecode;

  int   m_lastTime;
  int   m_lastUpdateMG;
  int   m_lastUpdateGrid;
  int   m_level;

  int	  * m_indices;
  CMGMesh * m_pMesh;
  CMGGrid * m_pGrid;
  MeshGridDecoderSpecificInfo *m_pInfo;

  int m_noTriangles;
};

#endif // __MESHGRIDPROXY_H__
