// This software module was originally developed by Marius Preda (INT)
// in the course of development of the MPEG-4 
// Systems standard (ISO/IEC 14496-1).  This software module is an 
// implementation of a part of one or more MPEG-4 Systems tools as specified by 
// the MPEG-4 Systems standard.  ISO/IEC gives users of the MPEG-4 Systems 
// standard free license to this software module or modifications thereof for 
// use in hardware or software products claiming conformance to the MPEG-4 
// Systems standard.  Those intending to use this software module in hardware or 
// software products are advised that its use may infringe existing patents.  
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use 
// of this software module or modifications thereof in an implementation.  
// Copyright is not released for non MPEG-4 Systems conforming products.  The 
// original developer retains full right to use the code for his/her own 
// purpose, assign or donate the code to a third party and to inhibit third 
// parties from using the code for non MPEG-4 Systems conforming products.  This 
// copyright notice must be included in all copies or derivative works.
//
// Copyright 2003.



#include "OpenGl/OpenGl.h"
#include "Presenter.h"
#include "MorphShapeProxy.h"
#include "opengl/geometry/ShapeProxy.h"
#include "opengl/geometry/GeometryProxy.h"
#include "opengl/appearance/AppearanceProxy.h"
#include "opengl/geometry/IndexedFaceSetProxy.h"

MorphShapeProxy::MorphShapeProxy(MorphShape* pMorphShape)
  : NodeProxy(pMorphShape), init(false)
{
  
}



void MorphShapeProxy::RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD )
{
  RenderDescriptionBox(pRenderer, "MorphShape");
}

// Render object
/*
MF<Vec3f> &MorphShapeProxy::GetBaseShapeCoord() const{ 
		    MorphShape* poMS = GetMorphShape();
			Shape *pShp = (Shape*) poMS->baseShape;
			ShapeProxy *pShpProxy = (ShapeProxy *) (pShp->GetRenderingObject());
			MediaObject* pGeometry=pShpProxy->GetGeometry();
			IndexedFaceSet *pIFSbase=(IndexedFaceSet*) pGeometry;
			Coordinate *pCoordinate = (Coordinate *)pIFSbase->coord.GetNode();
			return pCoordinate->point;
	}
*/
BOOL MorphShapeProxy::Render(const Matrix& rTransformation, const CullFlag& rCullFlag,
		  OpenGLVisualRenderer* pRenderer, DWORD dwTime)
{
	// [MBS] 120304 commented this buggy code
	MorphShape* poMS = GetMorphShape();
	if (!init){
		InitMorphing();
	}

	if (!wellFormated) return false;


  // update the base mesh accordignaly
  // IFS implementation
  // interpolation for vertices coordinate ; it is assumed that coord field has the same dimention for base and targets shapes

	RetrieveWeights(); // the morphing weights are retrived from the animation stream
	
	UpdateVerticesAndNormals(); // animate

	if (pShapeBase){
		ShapeProxy *pShP=(ShapeProxy *)pShapeBase->GetRenderingObject();
		pShP->CullOrRender(rTransformation, rCullFlag,pRenderer, dwTime);

		Appearance* pAppearance = pShP->GetAppearance();
		glPushAttrib(GL_POLYGON_BIT);
			((AppearanceProxy*) pAppearance->GetRenderingObject())->Enable(pRenderer, dwTime);
				Render(dwTime);
			((AppearanceProxy*) pAppearance->GetRenderingObject())->Disable(pRenderer, dwTime);
		glPopAttrib();

		ZArray<MediaObjectProxy *> nodes;
		MediaObject * pObject = (MediaObject*) pShapeBase;
		pObject->SetUpdateTime();
		MediaObjectProxy *pProxy = (MediaObjectProxy *) (pObject->GetRenderingObject());
		nodes.InsertAt(nodes.GetLength(), pProxy);
		nodes[0]->CullOrRender(
								rTransformation, 
								rCullFlag,
								pRenderer, 
								dwTime);  
	}
	
	return MediaObjectProxy::Render(rTransformation, rCullFlag,pRenderer, dwTime);
}

void
MorphShapeProxy::Select(
		const Matrix& rTransformation, 
		const CullFlag& rCullFlag,
		OpenGLVisualRenderer* pRenderer, 
		DWORD dwTime,
		MediaObjectProxy*& intersectingObject,
		ZArray<MediaObjectProxy*>& activatedSensors, 
		unsigned int& depth,
		const ZArray<MediaObjectProxy*>& localSensors)
{

  /* [MBS 120304 commented this buggy code
	MorphShape* poMS = GetMorphShape();
  int noTargets = poMS->weights.GetLength();

  ZArray<MediaObjectProxy *> nodes;
  ZArray<MediaObjectProxy *> sensors;

	MediaObject * pObject = (Shape*) poMS->baseShape;
	MediaObjectProxy *pProxy = (MediaObjectProxy *) (pObject->GetRenderingObject());
	nodes.InsertAt(nodes.GetLength(), pProxy);
	if (pProxy->IsPointingDeviceSensor())
      sensors += pProxy;
	
	nodes[0]->CullOrSelect(rTransformation, rCullFlag,pRenderer, dwTime,intersectingObject,      activatedSensors,       depth,	  localSensors);
 */ 
}

void
MorphShapeProxy::UpdateVerticesAndNormals()
{

/// base Shape
	MorphShape* poMS = GetMorphShape();
	ShapeProxy *pShpProxy = (ShapeProxy *) (pShapeBase->GetRenderingObject());

	MediaObject* pGeometry=pShpProxy->GetGeometry();
	Appearance* pAppearance = pShpProxy->GetAppearance();

	pIFSbase=(IndexedFaceSet*) pGeometry;
	Coordinate *pCoordinate = (Coordinate *)pIFSbase->coord.GetNode();
	Normal *pNormal = (Normal *)pIFSbase->normal.GetNode();
	MF<Vec3f> &pointsBase  = pCoordinate->point;
	MF<Vec3f> &normalsBase  = pNormal->vector;


	int a=pointsBase.GetLength();
	float x,y,z;
	int vtx, tgs,nms;
	for (vtx=0;vtx<a;vtx++){
		x=baseVertices[vtx].x; y=baseVertices[vtx].y; z=baseVertices[vtx].z;
		for (tgs=0;tgs<noTargets;tgs++){
			x+=m_weights[tgs]*(targetVertices[tgs][vtx].x-baseVertices[vtx].x);
			y+=m_weights[tgs]*(targetVertices[tgs][vtx].y-baseVertices[vtx].y);
			z+=m_weights[tgs]*(targetVertices[tgs][vtx].z-baseVertices[vtx].z);
		}
		Vec3f localVertex(x,y,z);		
		pointsBase.SetAt(vtx,localVertex);
	}
	for (nms=0;nms<a;nms++){
		x=baseNormals[vtx].x; y=baseNormals[vtx].y; z=baseNormals[vtx].z;
		for (tgs=0;tgs<noTargets;tgs++){
			x+=m_weights[tgs]*(targetNormals[tgs][vtx].x-baseNormals[vtx].x);
			y+=m_weights[tgs]*(targetNormals[tgs][vtx].y-baseNormals[vtx].y);
			z+=m_weights[tgs]*(targetNormals[tgs][vtx].z-baseNormals[vtx].z);
		}
		Vec3f localNormal(x,y,z);		
		normalsBase.SetAt(vtx,localNormal);
	}
/*
	pShp->SetUpdateTime();
	pShp->OnUpdate();*/

	pIFSbase->SetUpdateTime();
}


BOOL MorphShapeProxy::Render(DWORD dwTime)
{
return true;
}

BOOL MorphShapeProxy::InitMorphing()
{
	MorphShape* poMS = GetMorphShape();

	noTargets = poMS->targetShapes.GetLength();
	noWeights = poMS->weights.GetLength();
	if (noTargets != noWeights) wellFormated=false;
	else wellFormated=true;
	pShapeBase = GetMorphBaseShape();
	ShapeProxy *pShpProxy = (ShapeProxy *) (pShapeBase->GetRenderingObject());
	MediaObject* pGeometry=pShpProxy->GetGeometry();
	IndexedFaceSet *pIFSbase=(IndexedFaceSet*) pGeometry;
	Coordinate *pCoordinate = (Coordinate *)pIFSbase->coord.GetNode();
	Normal *pNormal = (Normal *)pIFSbase->normal.GetNode();
	noVertices=pCoordinate->point.GetLength();
	noNormals=pNormal->vector.GetLength();
	init =true;
	// copy the base mesh vertices
	int i,j;

	baseVertices=(Vec3f *)malloc(noVertices *sizeof Vec3f);
	for(i=0;i<noVertices;i++)
		baseVertices[i]=(Vec3f)(pCoordinate->point[i]);
	
	baseNormals=(Vec3f *)malloc(noNormals *sizeof Vec3f);
	for(i=0;i<noNormals;i++)
		baseNormals[i]=(Vec3f)(pNormal->vector[i]);

	targetVertices=(Vec3f **)malloc(noTargets*sizeof (Vec3f*));
	targetNormals=(Vec3f **)malloc(noTargets*sizeof (Vec3f*));
	for (i=0;i<noTargets;i++){
		targetVertices[i]=(Vec3f *)malloc(noVertices *sizeof (Vec3f));
		targetNormals[i]=(Vec3f *)malloc(noNormals *sizeof (Vec3f));
		Shape *pShape = GetMorphTargetShape(i);
		pShpProxy = (ShapeProxy *) (pShape->GetRenderingObject());
		pGeometry=pShpProxy->GetGeometry();
		IndexedFaceSet *pIFS=(IndexedFaceSet*) pGeometry;
		pCoordinate = (Coordinate *)pIFS->coord.GetNode();
		pNormal = (Normal *)pIFS->normal.GetNode();
		for (j=0;j<noVertices;j++)
			targetVertices[i][j]=(Vec3f)(pCoordinate->point[j]);
		for(j=0;j<noNormals;j++)
			targetNormals[i][j]=(Vec3f)(pNormal->vector[i]);
	}

	MF<float> &weigh  = poMS->weights;
	m_weights = (float*)malloc(noTargets*sizeof (float));
	for (i=0;i<noTargets;i++)
		m_weights[i]=weigh[i];

	// check only the vertices and normals that change in targets and base
	
	return true;
}

BOOL MorphShapeProxy::RetrieveWeights(){

	int i;
	MorphShape *pMS=(MorphShape *)GetMorphShape();
	for (i=0;i<noTargets;i++){
		m_weights[i]=pMS->weights[i];
	}

	return true;
}

