/***********************************************************************
This software module was originally developed by Mikael Bourges-Sevenier
(Mindego Inc.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Mindego Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Mikael Bourges-Sevenier at mikael@mindego.com.
----------------------------------------------------------------------*/

#ifndef __NonLinearDeformerProxy_h__
#define __NonLinearDeformerProxy_h__

#include "../geometry/GeometryProxy.h"

class SubWrap;
class IndexedFaceSetProxy;

class NonLinearDeformerProxy : public GeometryProxy
{
private:
	bool _needUpdate,_hasIFS;
	long m_nTimeStamp;

protected:
	IndexedFaceSetProxy *result;
	MF<Vec3f> *coords;
	MF<Vec3f> *normals;

public:
  /// Constructor
	NonLinearDeformerProxy(NonLinearDeformer* pss);

  /// Destructor
	~NonLinearDeformerProxy();

  /// Render object
  virtual BOOL		Render(OpenGLVisualRenderer* pRenderer, DWORD dwTime);

  /// Traverse scene graph, checking for intersection
  virtual BOOL		Select(OpenGLVisualRenderer* pRenderer, DWORD dwTime, unsigned int& depth);

	/// Render object description
  virtual void		RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD dwTime);

  /// Get bounding box
  void GetBbox(Vec3f& rBboxSize, Vec3f& rBboxCenter) const;

  /// Get pointer to related NonLinearDeformer object
  inline NonLinearDeformer * GetNonLinearDeformer() const
  { 
		return (NonLinearDeformer *) GetMediaObject(); 
	}

  /// Returns the shared IndexedFaceSetProxy that contains the transformed vertices and normals.
	inline IndexedFaceSetProxy *GetIFSProxy()
	{
		return result;
	}

protected:
	/// Updates the IFSProxy object if input fields have changed
	BOOL Update(DWORD dwTime);

	/// taper along z-axis
	void taper();

	/// twist along z-axis
	void twist();

	/// isotropic bend a centerline parallel to y-axis
	void bend();

	void findKeys(float z, float zm, float dz, const MF<float>& exts, int *p0, int *p1);
};

#endif // 