/***********************************************************************
This software module was originally developed by Mindego Inc.
(mikael@mindego.com) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. VUB retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2002
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Mindego Inc. (mikael@mindego.com).
----------------------------------------------------------------------*/

#include "NodeFld.h"
#include "Nurbs.h"

static float CBknots[]={0,0,0,0,1,1,1,1};
static float QBknots[]={0,0,0,1,1,1};


Nurbs::Nurbs() :
  npoints(0),
  knots(NULL),nknots(0),weights(NULL),nweights(0),
  type(0),p(0)
{
  n=left=right=NULL;
}

Nurbs::~Nurbs() 
{
  reset();
}

void Nurbs::reset() 
{
  // why should I need to comment that???
	//if(n!=NULL) delete[] n;
  //if(left!=NULL) delete[] left;
  //if(right!=NULL) delete[] right;
  n=left=right=NULL;
}

bool Nurbs::check() 
{
  int tmpp=nknots-npoints-1;
  if(tmpp<=0 || tmpp>=nknots-1) return FALSE;
  if(nweights>0 && npoints != nweights) return FALSE;
  
  if(tmpp != p) { // reset basis function calc
    reset();
    p=tmpp;
  }
  return TRUE;
}

bool Nurbs::init(int type)
{
  if(type==CUBIC_BEZIER) {
    npoints=4;
    nknots=8;
    knots=CBknots;
  }
  else  if(type==QUADRATIC_BEZIER) {
    npoints=3;
    nknots=6;
    knots=QBknots;
  }
  nweights=0;
	weights=NULL;
  this->type=type;
  return check();
}

bool Nurbs::init(int npts, int nk, float *knot, int nw, float *weight)
{
  npoints=npts;
  knots=knot;
  nknots=nk;
  weights=weight;
  nweights=nw;
  type=GENERIC;

  return check();
}

void Nurbs::basisFuns(int span, float t)
{
  float saved,temp;

  if(!n) {
    n=new float[p+1];
    left=new float[p+1];
    right=new float[p+1];
  }
  
  n[0]=1;
  for(int j=1;j<=p;j++) {
    left[j]=t-knots[span+1-j];
    right[j]=knots[span+j]-t;
    saved=0;
    for(int r=0;r<j;r++) {
      temp=n[r]/(right[r+1]+left[j-r]);
      n[r]=saved+right[r+1]*temp;
      saved=left[j-r]*temp;
    }
    n[j]=saved;
  }
}

int Nurbs::findSpan(float u)
{
  int span;
  if(u==knots[npoints]) 
    span=npoints-1;
  else {
    for(span=p;span<nknots-p;span++)
      if(u<knots[span]) break;
    span--;
  }
  return span;
}

void Nurbs::getPoint(int span, int offset, const MF<Vec3f> &pts, Vec3f &v)
{
  Vec3f pi(0,0,0);
  float w,wi;

  w=0;
  for(int i=0;i<=p;i++) {
    pi=pts[offset+span-p+i];
    if(nweights>0) {
      wi=weights[span-p+i];
      pi *= wi;
      w += n[i]*wi;
    }
    v.x += n[i]*pi.x;
    v.y += n[i]*pi.y;
    v.z += n[i]*pi.z;
  }
  if(nweights>0) {
		v.x /= w;
		v.y /= w;
		v.z /= w;
	}
}

void Nurbs::getPoint(int span, int offset, const MF<Vec2f> &pts, Vec2f &v)
{
  Vec2f pi(0,0);
  float w,wi;

  w=0;
  for(int i=0;i<=p;i++) {
    pi=pts[offset+span-p+i];
    if(nweights>0) {
      wi=weights[span-p+i];
      pi *= wi;
      w += n[i]*wi;
    }
    v.x += n[i]*pi.x;
    v.y += n[i]*pi.y;
  }
  if(nweights>0) {
		v.x /= w;
		v.y /= w;
	}
}
