/***********************************************************************
This software module was originally developed by Mindego Inc.
(mikael@mindego.com) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. VUB retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2002
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Mindego Inc. (mikael@mindego.com).
----------------------------------------------------------------------*/

#ifndef  _NURBS_
#define  _NURBS_

class MF;

/*
  Define NURBS geometry algorithms.

  Note:
    knots, points, weights are allocated elsewhere and passed to this class.
    So the caller is responsible to delete them, not this class.
*/
// built-in curves
class Nurbs {
public:
  enum {
    GENERIC =0,
    CUBIC_BEZIER,
    QUADRATIC_BEZIER,

    SAMPLE_DEFAULT=40
  };

  Nurbs();
  ~Nurbs();

public:
  // check consistency of parameters, returns FALSE if error
  bool check();

  // set sampling of curve
  int getNumSamples() const { return SAMPLE_DEFAULT; }
  bool init(int type=QUADRATIC_BEZIER);
  bool init(int npts, int nknots, float *knot, int nweights, float *weight);
  void basisFuns(int span, float t);
  int findSpan(float u);
  void getPoint(int span, int offset, const MF<Vec3f> &pts, Vec3f &v);
  void getPoint(int span, int offset, const MF<Vec2f> &pts, Vec2f &v);

public:
  // NURBS parameters
  float *knots, *weights; // knots and weights
  int nknots, nweights;
  int npoints;

protected:
  int p; // degree (determined automatically)
  int type;
  float *n,*left,*right; // basis functions calculation temp arrays
  void reset();

public:
  // quaternion splines
};

#endif // _NURBS_
