/***********************************************************************
This software module was originally developed by Mikael Bourges-Sevenier
(Mindego Inc.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Mindego Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Mikael Bourges-Sevenier at mikael@mindego.com.
----------------------------------------------------------------------*/

#ifndef __NurbsCurve2DProxy_h__
#define __NurbsCurve2DProxy_h__

#include "../geometry/GeometryProxy.h"

class NurbsCurve2DProxy : public GeometryProxy
{
	bool _needUpdate;
	long m_nTimeStamp;
  bool isIndexed; // FALSE by default
	GLUnurbsObj *theNurb;

public:
  /// Constructor
	NurbsCurve2DProxy(NurbsCurve2D* pnc);

  /// Destructor
	~NurbsCurve2DProxy();

  /// Render object
  virtual BOOL		Render(OpenGLVisualRenderer* pRenderer, DWORD dwTime);

  /// Traverse scene graph, checking for intersection
  virtual BOOL		Select(OpenGLVisualRenderer* pRenderer, DWORD dwTime, unsigned int& depth);

	/// Render object description
  virtual void		RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD dwTime);

  /// Get bounding box
  void GetBbox(Vec3f& rBboxSize, Vec3f& rBboxCenter) const;

  /// Get pointer to related NurbsCurve2D object
  inline NurbsCurve2D * GetNurbsCurve2D() const
  { 
		return (NurbsCurve2D *) GetMediaObject(); 
	}

  inline Color * GetColor()
  { 
		return (Color*)(MediaObject*)( GetNurbsCurve2D()->color); 
	}

  /// Get coordinates
  inline const MF<Vec3f>& GetControlPoint() const
  { 
		return GetNurbsCurve2D()->controlPoint; 
	}

  /// Set color indices
  inline void SetColorIndex(/* const */ MF<int>& rColorIndex)
  { 
		GetNurbsCurve2D()->colorIndex = rColorIndex;
    GetNurbsCurve2D()->SetUpdateTime(); 
	}

  /// Get color indices
  inline const MF<int>& GetColorIndex() const
  { 
		return GetNurbsCurve2D()->colorIndex; 
	}

  /// Get color per vertex
  inline BOOL GetColorPerVertex() const
  { 
		return (Bool) GetNurbsCurve2D()->colorPerVertex; 
	}

};

#endif // __NurbsCurve2DProxy_h__