/***********************************************************************
This software module was originally developed by Mikael Bourges-Sevenier
(Mindego Inc.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Mindego Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Mikael Bourges-Sevenier at mikael@mindego.com.
----------------------------------------------------------------------*/
#include "OpenGl/OpenGl.h"
#include "PositionInterpolator4DProxy.h"

PositionInterpolator4DProxy::PositionInterpolator4DProxy(MediaObject * poNode) :
	InterpolatorNodeProxy(poNode)
{}

inline BOOL PositionInterpolator4DProxy::SetValue(PositionInterpolator4D * poPI, int iIndex)
{
  poPI->value_changed = poPI->keyValue[iIndex];
  return TRUE;
}

BOOL PositionInterpolator4DProxy::Evaluate()
{
  PositionInterpolator4D * poPI = (PositionInterpolator4D *) GetMediaObject();

  // Check if we have any keys
  if (poPI->key.GetLength() > 0)
  {
    // If we only have one key
    if (poPI->key.GetLength() == 1)
    {
      return SetValue(poPI, 0);
    }
    else // more keys
    {
      // If set_fraction is smaller than smallest key
      if (poPI->set_fraction <= poPI->key[0])
      {
        return SetValue(poPI, 0);
      }
      else
      {
        // If set_fraction is larger than largest key
        if (poPI->set_fraction >= poPI->key[poPI->key.GetLength() - 1])
        {
          return SetValue(poPI, poPI->key.GetLength() - 1);
        }
        else
        {
          // Search the key space for fraction
          for (int i = poPI->key.GetLength() - 2; i >= 0 ; --i)
          {
            if (poPI->key[i] == poPI->set_fraction)
            {
              // Found exact match
              return SetValue(poPI, i);
            }
            else
            {
              if (poPI->key[i] < poPI->set_fraction)
              {
                // Didn't find exact match; must interpolate
                return InterpolateValue( 
                  i, i + 1, 
                  ((poPI->key[i + 1] - poPI->set_fraction) / 
                  (poPI->key[i + 1] - poPI->key[i])) );
              }
            }
          } // for
        }
      }
    } // one or more keys
  } // some key

  return FALSE;
}

BOOL PositionInterpolator4DProxy::InterpolateValue(
    int nMinIndex, int nMaxIndex, float fFraction)
{
  PositionInterpolator4D * poPI = (PositionInterpolator4D *) GetMediaObject();

  Vec4f tempPosition;

  assert(poPI->keyValue.GetLength() == poPI->key.GetLength());

  // Interpolate positions

  tempPosition.x = (poPI->keyValue[nMaxIndex].x + (poPI->keyValue[nMinIndex].x - poPI->keyValue[nMaxIndex].x) * fFraction);
  tempPosition.y = (poPI->keyValue[nMaxIndex].y + (poPI->keyValue[nMinIndex].y - poPI->keyValue[nMaxIndex].y) * fFraction);
  tempPosition.z = (poPI->keyValue[nMaxIndex].z + (poPI->keyValue[nMinIndex].z - poPI->keyValue[nMaxIndex].z) * fFraction);
  tempPosition.q = (poPI->keyValue[nMaxIndex].q + (poPI->keyValue[nMinIndex].q - poPI->keyValue[nMaxIndex].q) * fFraction);
  
  poPI->value_changed = tempPosition;

  return TRUE;
}
