/***********************************************************************
This software module was originally developed by Dr. Michael Steliaros,
Superscape (UK) Ltd., in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Superscape (UK) Ltd. retains full
right to use the code for his/her own purpose, assign or donate the code
to a third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2001
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Dr. Michael Steliaros at msteliaros@superscape.com
----------------------------------------------------------------------*/

#include "OpenGl/OpenGl.h"
#include "ProceduralTextureProxy.h"
#include "opengl/base/OpenGLVisualRenderer.h"

/// Constructor

ProceduralTextureProxy::ProceduralTextureProxy(ProceduralTexture* pProceduralTexture) :
	TextureNodeProxy(pProceduralTexture),
	m_nTimeStamp(-1)
{
	m_pPlasma = new CssPlasma();
	assert(m_pPlasma);
	m_bDirty = true;
}

/// Render object description

/* virtual */ void
ProceduralTextureProxy::RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD /* dwTime */)
{
	RenderDescriptionBox(pRenderer, "ProceduralTexture");
}

/// Enable texture

/* virtual */ BOOL
ProceduralTextureProxy::Enable(OpenGLVisualRenderer* pRenderer, DWORD dwTime)
{
	ProceduralTexture* pTexture = (ProceduralTexture*) GetMediaObject();
	const Image& rImage = GetImage();

	if (rImage.height && rImage.width && rImage.numComponents && rImage.pixels)
	{
		if (long(pTexture->GetUpdateTime()) > m_nTimeStamp)
		{
			m_nTimeStamp = (long) dwTime;

			return TextureNodeProxy::Enable(pRenderer, rImage.numComponents == 4 ? RGBA : RGB, rImage.width, rImage.height, rImage.pixels);
		}
		else
		{
			return TextureNodeProxy::Enable(pRenderer);
		}
	}
	else
	{
		return TRUE;
	}
}

Image ProceduralTextureProxy::GetImage()
{
	if(m_bDirty)
	{
		SetaSmooth((Bool)GetaSmooth());
		SetbSmooth((Bool)GetbSmooth());
		SetxSmooth((Bool)GetxSmooth());
		SetySmooth((Bool)GetySmooth());
		SetWidth(GetWidth());
		SetHeight(GetHeight());
		SetSeed(GetSeed());
		SetType(GetType());
		SetRoughness(GetRoughness());
		SetDistortion(GetDistortion());
		SetCellWidth(GetCellWidth());
		SetCellHeight(GetCellHeight());
		SetaWarpmap(GetaWarpmap());
		SetbWarpmap(GetbWarpmap());
		SetxWarpmap(GetxWarpmap());
		SetyWarpmap(GetyWarpmap());
		SetaWeights(GetaWeights());
		SetbWeights(GetbWeights());
		SetColor(GetColor());
		m_bDirty = false;
	}

	Image& rImage = (Image&)GetProceduralTexture()->image_changed;
	const SssImageData& plasma = m_pPlasma->GetImageData();

	rImage.width = plasma.m_Width;
	rImage.height = plasma.m_Height;
	rImage.numComponents = 3;
	rImage.pixels = (unsigned char*)plasma.m_pColors;

	return rImage;
}

void ProceduralTextureProxy::SetWarpmap(int warpmap, const MF<Vec2f>& src, MF<Vec2f>& dst)
{
	int len = src.GetLength();

	m_pPlasma->SetNumWarpKnots(warpmap, len);

	for(int i = 0; i < len; i++)
	{
		const Vec2f& e = src[i];
		m_pPlasma->SetWarp(warpmap, 0, i, e.x);
		m_pPlasma->SetWarp(warpmap, 1, i, e.y);
	}

	// The MF<> assignment operator does not work as expected when source & destination are the same - only do it if they are different
	if(&src != &dst)
		dst = src;
}

void ProceduralTextureProxy::SetaWeights(const MF<float>& aWeights)
{
	int len = aWeights.GetLength();
	assert(len == 16);

	for(int i = 0; i < len; i++)
		m_pPlasma->SetAWeight(i, aWeights[i]);

	// The MF<> assignment operator does not work as expected when source & destination are the same - only do it if they are different
	if(&aWeights != &GetProceduralTexture()->aWeights)
		GetProceduralTexture()->aWeights = aWeights;

	GetProceduralTexture()->SetUpdateTime(); 
	m_bDirty = true;
}

void ProceduralTextureProxy::SetbWeights(const MF<float>& bWeights)
{
	int len = bWeights.GetLength();
	assert(len == 16);

	for(int i = 0; i < len; i++)
		m_pPlasma->SetBWeight(i, bWeights[i]);

	// The MF<> assignment operator does not work as expected when source & destination are the same - only do it if they are different
	if(&bWeights != &GetProceduralTexture()->bWeights)
		GetProceduralTexture()->bWeights = bWeights;

	GetProceduralTexture()->SetUpdateTime(); 
	m_bDirty = true;
}

void ProceduralTextureProxy::SetColor(const MF<Color3f>& color)
{
	int len = color.GetLength();
	assert(len == 4);

	for(int i = 0; i < len; i++)
	{
		const Color3f& col = color[i];
		m_pPlasma->Setcol(i, CssColor(col.r, col.g, col.b));
	}

	// The MF<> assignment operator does not work as expected when source & destination are the same - only do it if they are different
	if(&color != &GetProceduralTexture()->color)
		GetProceduralTexture()->color = color;

	GetProceduralTexture()->SetUpdateTime(); 
	m_bDirty = true;
}
