/***********************************************************************
This software module was originally developed by Dr. Michael Steliaros,
Superscape (UK) Ltd., in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Superscape (UK) Ltd. retains full
right to use the code for his/her own purpose, assign or donate the code
to a third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2001
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Dr. Michael Steliaros at msteliaros@superscape.com
----------------------------------------------------------------------*/

#ifndef __PROCEDURALTEXTUREPROXY_H__
#define __PROCEDURALTEXTUREPROXY_H__

#include "../appearance/TextureNodeProxy.h"
#include "ss/imagenode.h"

class ProceduralTexture;

/// Pixel texture renderer class

class ProceduralTextureProxy : public TextureNodeProxy
{
	public:

		/// Constructor

		ProceduralTextureProxy(ProceduralTexture* pProceduralTexture);

		/// Destructor

		~ProceduralTextureProxy()
		{
			delete(m_pPlasma);
		}

		/// Render object description

		virtual void RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD dwTime);

		/// Enable texture

		virtual BOOL Enable(OpenGLVisualRenderer* pRenderer, DWORD dwTime);

		/// Disable texture

		virtual BOOL Disable(OpenGLVisualRenderer* pRenderer, DWORD /* dwTime */)
		{
			const Image& rImage = GetImage();

			if (rImage.height && rImage.width && rImage.numComponents && rImage.pixels)
				return TextureNodeProxy::Disable(pRenderer);
			else
				return TRUE;
		}

	public:
		inline ProceduralTexture * GetProceduralTexture()
		{ return (ProceduralTexture *) GetMediaObject(); }

		Image GetImage();

		inline BOOL GetaSmooth()
		{ return (Bool) GetProceduralTexture()->aSmooth; }

		inline void SetaSmooth(SF<Bool> aSmooth)
		{
			GetProceduralTexture()->aSmooth = aSmooth;
			m_pPlasma->SetWarpSmooth(2, (Bool)aSmooth);
		    GetProceduralTexture()->SetUpdateTime(); 
			m_bDirty = true;
		}

		inline BOOL GetbSmooth()
		{ return (Bool) GetProceduralTexture()->bSmooth; }

		inline void SetbSmooth(SF<Bool> bSmooth)
		{
			GetProceduralTexture()->bSmooth = bSmooth;
			m_pPlasma->SetWarpSmooth(3, (Bool)bSmooth);
		    GetProceduralTexture()->SetUpdateTime(); 
			m_bDirty = true;
		}

		inline BOOL GetxSmooth()
		{ return (Bool) GetProceduralTexture()->xSmooth; }

		inline void SetxSmooth(SF<Bool> xSmooth)
		{
			GetProceduralTexture()->xSmooth = xSmooth;
			m_pPlasma->SetWarpSmooth(0, (Bool)xSmooth);
		    GetProceduralTexture()->SetUpdateTime(); 
			m_bDirty = true;
		}

		inline BOOL GetySmooth()
		{ return (Bool) GetProceduralTexture()->ySmooth; }

		inline void SetySmooth(SF<Bool> ySmooth)
		{
			GetProceduralTexture()->ySmooth = ySmooth;
			m_pPlasma->SetWarpSmooth(1, (Bool)ySmooth);
		    GetProceduralTexture()->SetUpdateTime(); 
			m_bDirty = true;
		}

		inline int GetWidth()
		{ return GetProceduralTexture()->width; }

		inline void SetWidth(SF<int> width)
		{
			GetProceduralTexture()->width = width;
			m_pPlasma->SetcSubdivHoriz(width);
		    GetProceduralTexture()->SetUpdateTime(); 
			m_bDirty = true;
		}

		inline int GetHeight()
		{ return GetProceduralTexture()->height; }

		inline void SetHeight(SF<int> height)
		{
			GetProceduralTexture()->height = height;
			m_pPlasma->SetcSubdivVert(height);
		    GetProceduralTexture()->SetUpdateTime(); 
			m_bDirty = true;
		}

		inline int GetSeed()
		{ return GetProceduralTexture()->seed; }

		inline void SetSeed(SF<int> seed)
		{
			GetProceduralTexture()->seed = seed;
			m_pPlasma->SetSeed(seed);
		    GetProceduralTexture()->SetUpdateTime(); 
			m_bDirty = true;
		}

		inline int GetType()
		{ return GetProceduralTexture()->type; }

		inline void SetType(SF<int> type)
		{
			GetProceduralTexture()->type = type;
			m_pPlasma->SetPlasmaType(type);
		    GetProceduralTexture()->SetUpdateTime(); 
			m_bDirty = true;
		}

		inline int GetRoughness()
		{ return GetProceduralTexture()->roughness; }

		inline void SetRoughness(SF<int> roughness)
		{
			GetProceduralTexture()->roughness = roughness;
			m_pPlasma->SetcRoughness(roughness);
		    GetProceduralTexture()->SetUpdateTime(); 
			m_bDirty = true;
		}

		inline float GetDistortion()
		{ return GetProceduralTexture()->distortion; }

		inline void SetDistortion(SF<float> distortion)
		{
			GetProceduralTexture()->distortion = distortion;
			m_pPlasma->SetDistortion(distortion);
		    GetProceduralTexture()->SetUpdateTime(); 
			m_bDirty = true;
		}

		inline int GetCellWidth()
		{ return GetProceduralTexture()->cellWidth; }

		inline void SetCellWidth(SF<int> cellWidth)
		{
			GetProceduralTexture()->cellWidth = cellWidth;
			m_pPlasma->SetCellWidth(cellWidth);
		    GetProceduralTexture()->SetUpdateTime(); 
			m_bDirty = true;
		}

		inline int GetCellHeight()
		{ return GetProceduralTexture()->cellHeight; }

		inline void SetCellHeight(SF<int> cellHeight)
		{
			GetProceduralTexture()->cellHeight = cellHeight;
			m_pPlasma->SetCellHeight(cellHeight);
		    GetProceduralTexture()->SetUpdateTime(); 
			m_bDirty = true;
		}

		inline const MF<Vec2f>& GetaWarpmap()
		{ return GetProceduralTexture()->aWarpmap; }

		inline void SetaWarpmap(const MF<Vec2f>& aWarpmap)
		{
			SetWarpmap(2, aWarpmap, GetProceduralTexture()->aWarpmap);
			GetProceduralTexture()->SetUpdateTime(); 
			m_bDirty = true;
		}

		inline const MF<Vec2f>& GetbWarpmap()
		{ return GetProceduralTexture()->bWarpmap; }

		inline void SetbWarpmap(const MF<Vec2f>& bWarpmap)
		{
			SetWarpmap(3, bWarpmap, GetProceduralTexture()->bWarpmap);
			GetProceduralTexture()->SetUpdateTime(); 
			m_bDirty = true;
		}

		inline const MF<Vec2f>& GetxWarpmap()
		{ return GetProceduralTexture()->xWarpmap; }

		inline void SetxWarpmap(const MF<Vec2f>& xWarpmap)
		{
			SetWarpmap(0, xWarpmap, GetProceduralTexture()->xWarpmap);
			GetProceduralTexture()->SetUpdateTime(); 
			m_bDirty = true;
		}

		inline const MF<Vec2f>& GetyWarpmap()
		{ return GetProceduralTexture()->yWarpmap; }

		inline void SetyWarpmap(const MF<Vec2f>& yWarpmap)
		{
			SetWarpmap(1, yWarpmap, GetProceduralTexture()->yWarpmap);
			GetProceduralTexture()->SetUpdateTime(); 
			m_bDirty = true;
		}

		inline const MF<float>& GetaWeights()
		{ return GetProceduralTexture()->aWeights; }

		void SetaWeights(const MF<float>& aWeights);

		inline const MF<float>& GetbWeights()
		{ return GetProceduralTexture()->bWeights; }

		void SetbWeights(const MF<float>& bWeights);

		inline const MF<Color3f>& GetColor()
		{ return GetProceduralTexture()->color; }

		void SetColor(const MF<Color3f>& color);

	private:
		void SetWarpmap(int warpmap, const MF<Vec2f>& src, MF<Vec2f>& dst);

	private:
		// Plasma class; this does all the work
		CssPlasma*		m_pPlasma;
		bool			m_bDirty;

		// Time stamp of display list
		long			m_nTimeStamp;
};

#endif // __PROCEDURALTEXTUREPROXY_H__
