// This software module was originally developed by Marius PREDA (INT)
// in the course of development of the MPEG-4 
// Systems standard (ISO/IEC 14496-1).  This software module is an 
// implementation of a part of one or more MPEG-4 Systems tools as specified by 
// the MPEG-4 Systems standard.  ISO/IEC gives users of the MPEG-4 Systems 
// standard free license to this software module or modifications thereof for 
// use in hardware or software products claiming conformance to the MPEG-4 
// Systems standard.  Those intending to use this software module in hardware or 
// software products are advised that its use may infringe existing patents.  
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use 
// of this software module or modifications thereof in an implementation.  
// Copyright is not released for non MPEG-4 Systems conforming products.  The 
// original developer retains full right to use the code for his/her own 
// purpose, assign or donate the code to a third party and to inhibit third 
// parties from using the code for non MPEG-4 Systems conforming products.  This 
// copyright notice must be included in all copies or derivative works.
//
// Copyright 2003.


#ifndef __SBVCANIMATIONV2PROXY_H__
#define __SBVCANIMATIONV2PROXY_H__

#include "opengl/base/NodeProxy.h"
#include "Core/Nodes.h"
#include "IM1Decoders\AFX\BBA\BBADecoderLib\BBADecoderLib.h"
#include "IM1Decoders\AFX\BBA\BBADecoderLib\BBATypeDefs.h"

class AFXTemplate;
class SBVCAnimationV2;
class OpenGLVisualRenderer;
class MediaStream;


class SBVCAnimationV2Proxy
  : public	NodeProxy
{
public:

  /// Constructor
	SBVCAnimationV2Proxy(SBVCAnimationV2* pSBVCAnimationV2);

public:

	bool GetIsDecodingNeeded() { return m_bIsDecodingNeeded; }
	void SetIsDecodingNeeded(bool bIsNeeded) { m_bIsDecodingNeeded = bIsNeeded; }
	bool IsLinkInit() { return m_bLinkInit; }
	void SetLinkInit(bool flag) { m_bLinkInit = flag; }
	void MakeLink();
	virtual void		RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD dwTime);
	virtual BOOL  RenderFromStream(MediaStream* poStream, int id, DWORD dwTime);

	virtual void  SetDecoder(AFXTemplate *pDecoder){
		m_pDecoder = pDecoder;
	};

  /// Render object
protected :
	virtual BOOL Render(const Matrix&, const CullFlag&, OpenGLVisualRenderer* pRenderer, DWORD) ;
	virtual void Select(const Matrix&, const CullFlag&, OpenGLVisualRenderer* pRenderer, DWORD dwTime, MediaObjectProxy*& intersectingObject,ZArray<MediaObjectProxy*>& activatedSensors, unsigned int& depth,const ZArray<MediaObjectProxy*>& localSensors);


public:
	inline SBVCAnimationV2 * GetSBVCAnimationV2() {
		return (SBVCAnimationV2 *) GetMediaObject();
	}

	void Start()
	{
#ifndef BIFSENC
    if (!GetSBVCAnimationV2()->IsStreamOpen())
    {
		  m_bActive = TRUE;
		  GetSBVCAnimationV2()->OpenStream();
      GetSBVCAnimationV2()->GetExecutive()->RequestChannels();
    }
#endif
	};

private:
	bool m_bLinkInit;
	bool m_bIsDecodingNeeded;	// it indicates if decoding of node or buffer field is needed
	bool  m_bInit;
	DWORD m_lastTime;

	AFXTemplate *m_pDecoder;

	sb_object_plane_t_decoded *m_bbaopdec;

	int keyFrame;
	int noBones;
	int noMuscles;
	int noMorphs;
	CBone *firstBone; // not used ... to be deleted
	CMuscle *firstMuscle;
	CMorph *firstMorph;
	CBone *currentBone;
	CMuscle *currentMuscle;
	CMorph *currentMorph;
	int *currentMorphData;

public:
	bool GetKeyFrame (void *pData, int size);
	bool GetNoBones (void *pData, int size);
	bool GetNoMuscles (void *pData, int size);
	bool GetNoMorphs (void *pData, int size);
	bool GetBones (void *pData, int size);
	bool GetMuscles (void *pData, int size);
	bool GetMorphsData (void *pData, int size);
	void InitBonesMuscles();
	void InitMorphs();
	bool UpdateBoneTransform (CBone *cb);
	bool UpdateMuscleTransform (CMuscle *cb);
	bool UpdateMorphTransform (int *cmd);

};

#include "SBVCAnimationV2Proxy.i"

#endif // __SBVCANIMATIONPROXY_H__

