/***********************************************************************
This software module was originally developed by Mikael Bourges-Sevenier
(Mindego Inc.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Mindego Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Mikael Bourges-Sevenier at mikael@mindego.com.
----------------------------------------------------------------------*/

#include "OpenGl/OpenGl.h"
#include "ScalarAnimatorProxy.h"

// Constructor, only calls the base constructor
ScalarAnimatorProxy::ScalarAnimatorProxy(MediaObject * poNode) :
	InterpolatorNodeProxy(poNode)
{}

inline BOOL ScalarAnimatorProxy::SetValue(ScalarAnimator * pPA, int iIndex)
{
  pPA->value_changed = pPA->keyValue[iIndex];
  return TRUE;
}

BOOL ScalarAnimatorProxy::Evaluate()
{
  ScalarAnimator * pPA = (ScalarAnimator *) GetMediaObject();

  // Check if we have any keys
  if (pPA->key.GetLength() > 0)
  {
    // If we only have one key
    if (pPA->key.GetLength() == 1)
    {
      return SetValue(pPA, 0);
    }
    else // more keys
    {
      // If set_fraction is smaller than smallest key
      if (pPA->set_fraction <= pPA->key[0])
      {
        return SetValue(pPA, 0);
      }
      else
      {
        // If set_fraction is larger than largest key
        if (pPA->set_fraction >= pPA->key[pPA->key.GetLength() - 1])
        {
          return SetValue(pPA, pPA->key.GetLength() - 1);
        }
        else
        {
          // Search the key space for fraction
          for (int i = pPA->key.GetLength() - 2; i >= 0 ; --i)
          {
            if (pPA->key[i] == pPA->set_fraction)
            {
              // Found exact match
              return SetValue(pPA, i);
            }
            else
            {
              if (pPA->key[i] < pPA->set_fraction)
              {
                // Didn't find exact match; must interpolate
                return InterpolateValue( i, i + 1, ((pPA->key[i + 1] - pPA->set_fraction) / (pPA->key[i + 1] - pPA->key[i])) );
              }
            }
          } // for
        }
      }
    } // one or more keys
  } // some key

  return FALSE;
}

BOOL ScalarAnimatorProxy::InterpolateValue(int nMinIndex, int nMaxIndex, float fFraction)
{
  ScalarAnimator * pPA = (ScalarAnimator *) GetMediaObject();

  float temp;

  // Interpolate positions

	
  // set value
  pPA->value_changed = temp;

  return TRUE;
}
