/***********************************************************************
This software module was originally developed by Mikael Bourges-Sevenier
(Mindego Inc.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Mindego Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Mikael Bourges-Sevenier at mikael@mindego.com.
----------------------------------------------------------------------*/

#ifndef __SubdivisionSurfaceProxy_h__
#define __SubdivisionSurfaceProxy_h__

#include "../geometry/GeometryProxy.h"
#include "ss/subdivisionsurfacexloop.h"

class SubWrap;

class SubdivisionSurfaceProxy : public GeometryProxy
{
	SubdivisionSurfaceXLoop* m_pSSXL; // Extended Loop class that does all the work
	SubWrap* _sw; // underlying subdivision surface object
	bool _needUpdate;
	int *f; // faces
	float *v; // vertices
	float *vn; // vertex normals
	float *p; // texture params
	long m_nTimeStamp;
	
public:
	/// Constructor
	SubdivisionSurfaceProxy(SubdivisionSurface* pss);
	
	/// Destructor
	~SubdivisionSurfaceProxy();
	
	/// Render object
	virtual BOOL		Render(OpenGLVisualRenderer* pRenderer, DWORD dwTime);
	
	/// Traverse scene graph, checking for intersection
	virtual BOOL		Select(OpenGLVisualRenderer* pRenderer, DWORD dwTime, unsigned int& depth);
	
	/// Render object description
	virtual void		RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD dwTime);
	
	/// Get bounding box
	void GetBbox(Vec3f& rBboxSize, Vec3f& rBboxCenter) const;
	
	/// Get pointer to related SubdivisionSurface object
	inline SubdivisionSurface * GetSubdivisionSurface() const
	{ 
		return (SubdivisionSurface *) GetMediaObject(); 
	}
	
	inline Color * GetColor()
	{ 
		return (Color*)(MediaObject*)( GetSubdivisionSurface()->color); 
	}
	
	/// Get coordinates
	inline Coordinate* GetCoord()
	{ 
		return GetSubdivisionSurface()->coord; 
	}
	
	/// Get texture coordinates
	inline TextureCoordinate* GetTexCoord()
	{ 
		return (TextureCoordinate*)GetSubdivisionSurface()->texCoord.GetNode(); //KM 
	}
	
	/// Set coordinate indices
	inline void SetCoordIndex(/* const */ MF<int>& rCoordIndex)
	{ 
		GetSubdivisionSurface()->coordIndex = rCoordIndex;
		GetSubdivisionSurface()->SetUpdateTime();
	}
	
	/// Get coordinate indices
	inline const MF<int>& GetCoordIndex() const
	{ 
		return GetSubdivisionSurface()->coordIndex; 
	}
	
	/// Set color indices
	inline void SetColorIndex(/* const */ MF<int>& rColorIndex)
	{ 
		GetSubdivisionSurface()->colorIndex = rColorIndex;
		GetSubdivisionSurface()->SetUpdateTime(); 
	}
	
	/// Get color indices
	inline const MF<int>& GetColorIndex() const
	{ 
		return GetSubdivisionSurface()->colorIndex; 
	}
	
	/// Set texture coordinate indices
	inline void SetTexCoordIndex(/* const */ MF<int>& rTexCoordIndex)
	{ 
		GetSubdivisionSurface()->texCoordIndex = rTexCoordIndex;
		GetSubdivisionSurface()->SetUpdateTime(); 
	}
	
	/// Get texture coordinate indices
	inline const MF<int>& GetTexCoordIndex() const
	{ 
		return GetSubdivisionSurface()->texCoordIndex; 
	}
	
	/// Get color per vertex
	inline BOOL GetColorPerVertex() const
	{ 
		return (Bool) GetSubdivisionSurface()->colorPerVertex; 
	}
	
	/// Get whether object is solid
	inline BOOL GetSolid() const
	{ 
		return (Bool) GetSubdivisionSurface()->solid; 
	}
	
	/// Get whether polygons are counter-clockwise
	inline BOOL GetCCW() const
	{ 
		return (Bool) GetSubdivisionSurface()->ccw; 
	}
	
	/// Get the invisibleEdgeIndex list
	inline const MF<int>& GetInvisibleEdgeIndex() const
	{ 
		return GetSubdivisionSurface()->invisibleEdgeIndex; 
	}

private:
	static SubdivisionSurfaceXLoop* BuildSSXL(SubdivisionSurface* pSS);
};

#endif // __SubdivisionSurfaceProxy_h__