#ifndef _ADAPTIVE_REFINEMENT_
#define _ADAPTIVE_REFINEMENT_


#include <afxtempl.h>

//***************************************************
// Class MyHashTable
//***************************************************

//Size of the table (higher it is, more memory is spent 
//but less computing time is required)
#define TABLE_SIZE 1024L

class MyHashTable
    {
    public:
        
        MyHashTable();
        ~MyHashTable();
        int  GetMidPt(int Pt1, int Pt2);        
        void InsertMidPt(int Pt1, int Pt2, int MidPt);

    private:
        //Struct that associates two vertex index 
        //with their midpoint index
        struct NewVertexInd
            {
            int Pt1;
            int Pt2;
            int MidPt;
            };

        //Struct that contains a list of NewVertexInd pointer
        struct NewVertexList
            {
            int size;
            NewVertexInd** NVPtr;
            };

        //Table of NewVertexList
        NewVertexList* m_NVTable[TABLE_SIZE][TABLE_SIZE];
    };

//***************************************************
// Class AdaptiveRefiner
//***************************************************

//Adaptive refinement parameters
#define DOT_TOLERANCE           0.8     //Minimum dotproduct between two normals

#define MAX_SUBDIVISION_LEVEL   3       //Maximum number of recursive subdivisions
                                        //for each triangle

#define MIN_DISTANCE            0.01    //Minimum distance between two points that
                                        //have to be subdivided

#define STEP                    0.02    //Step used to project points on the surface

//Flags
#define NOT_FLAT                0
#define FLAT                    1

class AdaptiveRefiner
    {
    public:
        AdaptiveRefiner(Volume_Tree_Node* VTNode) 
        {
            m_TheNode        = VTNode;
            m_ComputedVertex = 0;
            m_ColorTable     = 0;
        }

        ~AdaptiveRefiner() {}

        void Refine(Shapejb *Shape, ZArray<Vec3f> *ColorTable = 0);

    private:
        bool IsFlat(VERTEX* Pt1, VERTEX* Pt2, Point* Pt1Color, Point* Pt2Color) const;
        int  MidPoint(bool IsFlat, const int Pt1Ind, const int Pt2Ind);
        void ProjectToSurface(const double Normal[3], double MidPt[3]) const;
        void CalculateNormal(const double MidPt[3], double NewNormal[3], float MidPtColor[3]);
        void SubdivideNewTriangles(int IndOfNextVert, int Level);        
        
        CArray<VERTEX,VERTEX>   m_VertexList;
        CArray<int,int>         m_IndexList;        
        Volume_Tree_Node        *m_TheNode;        
        MyHashTable             *m_ComputedVertex;
        ZArray<Vec3f>              *m_ColorTable;
    };

double  DotProd(const POINT_J& Pt1, const POINT_J& Pt2);

#endif //_ADAPTIVE_REFINEMENT_