/*=============================================================================



==============================================================================*/
#ifndef _BBOX_h
#define _BBOX_h

/******************************************************************************


Notes:

Changes:

Todo :

******************************************************************************/


#include "stdafx.h"
#include <float.h>
#include <math.h>

#include "point.h"





#define BBOX_MAX FLT_MAX 
#define BBOX_MIN FLT_MIN


/*
! Bounding box
holds a axis aligned extent as two points
*/

class BBox {
public:
	//! box points
	Point mi,ma;

//! constructor, set to empty
	BBox() : mi(BBOX_MAX),ma(BBOX_MIN) {};

	//! constructor, set to single point
	BBox(const Point &p) : mi(p),ma(p) {} ;

#if 0
	//! constructor, set by two to single point
	BBox(const Point &pMin,const Point &pMax) : mi(pMin),ma(pMax) {} ;


	//! constructor, set to BBox
	BBox(const BBox &b) { mi=b.mi,ma=b.ma; }


   	//! Set Bounding Box to single point
	inline void Set(const Point &p) { mi.Set(p); ma.Set(p); }


    //! Set Bounding Box to new size, ma is recomputed
	inline void SetSize(const Point &p) { ma = mi + p; }

    //! Set Bounding Box to other bbox
	inline void Set(const BBox &b) { mi=b.mi,ma=b.ma; }

    //! Set Bounding Box to  bbox in center, size notation, size must be positive 
	inline void SetCenterSize(const Point &center,const Point &size) { 
		Point s2 = size * 0.5;
		mi = center - s2;
		ma = center + s2;
	}

	//! Set Bounding Box to Emtpy State
	inline void SetEmpty() { mi.Set(BBOX_MAX); ma.Set(BBOX_MIN); }

	//! Is Bounding box empty ?
	inline bool Empty() const { return(ma.x < mi.x); }
	inline bool IsEmpty() const { return(ma.x < mi.x); }

	//! return size of box
	inline Point Size() const { return(ma-mi); }


	//! scale box about center 
	void ScaleCenter(const Point &scale) 
			{ Point s(Size());
			  Point center(mi+s * 0.5);
			  s *= scale*0.5;
			  mi.Set(center-s);
			  ma.Set(center+s);
			 }
	 

	//! return center of box
	inline Point Center() const { return(mi + (ma-mi)*0.5); }

     //! Union this with Point p 

	inline BBox& Union(const Point &p) { p.MinMax(mi,ma); return(*this); }

    //! Union this with BBox b
	inline 
	BBox& Union(const BBox &b) { b.mi.MinMax(mi,ma);
								b.ma.MinMax(mi,ma);
								return(*this);
							  }
#else
		//! Set Bounding Box to Emtpy State
	inline void SetEmpty() { mi.Set(BBOX_MAX); ma.Set(BBOX_MIN); }

	//! Is Bounding box empty ?
	inline bool IsEmpty() const { return(ma.x < mi.x); }
		inline 
	BBox& Union(const BBox &b) { b.mi.MinMax(mi,ma);
								b.ma.MinMax(mi,ma);
								return(*this);
							  }
#endif

    //! is Point p inside ?
    inline bool Inside(const Point &p) const {
				return( (p.x>=mi.x) && (p.x <=ma.x)
						&& (p.y>=mi.y) && (p.y <= ma.y)
						&& (p.z>=mi.z) && (p.z <= ma.z));

	}


	//! Transform boundingbox by Matrix
	friend BBox operator* (const SL_Matrix&, const BBox& b);

	//! Transform boundingbox by Matrix
	BBox& operator *= (const SL_Matrix &m);


};


#endif //_BBOX_h


