/********************************************************
*
*	$Author: mig $
*	$JustDate:  4/17/02 $
*	$Revision: 1.1 $
*	$Archive: /afx/Solid/DensityTLS.h $
*
*********************************************************/
#ifndef _DensityTLS_
#define _DensityTLS_


/********************************************************************************/
//
//  This file regroups most of the density tools including DensityMatrix and 
//  DensityArray Class.
//
//  Author : Eric Baillargeon, SGDL Systme Inc.
//  Date   : November 28th, 2001
//
//*******************************************************************************/


#include "point.h"

class Volume_Tree_Node;


const int   K_INT_INTERIOR_INDEX        = 1;  
const int   K_INT_SKIN_INDEX            = 0;
const int   K_INT_DEFAULTSKIN		    = 1;   
const int   K_INT_DEFAULTINTERIOR       = 2;

// Enum for Bounding use
enum eBounds{
    
    eBounds_MaxX = 0,
    eBounds_MinX,
    eBounds_MaxY,
    eBounds_MinY,
    eBounds_MaxZ,
    eBounds_MinZ,
    eBounds_NumberOf,
	eBounds_PAD = 0xffff
};


///////////////////////////////////////////////////////////////////////////////
//
//
//                          Density Array Class
//
//
//     
//          
//////////////////////////////////////////////////////////////////////////////
class DensityArray 
{

// Public Members
public:
   // DensityArray(const GvMFInt32& UserList);    
    
    DensityArray()                                              //Default Constructor
    {        
        NumberOfDensity = 0;
        values          = NULL;
    }
    DensityArray(int NumberOf, int* ArrayOf)                    //Construct a densityarray from a int*
    {
        NumberOfDensity = NumberOf;

		if( NumberOf != 0) {
        values = new int[NumberOfDensity];
        for (int i = 0 ; i < NumberOfDensity; i++)
        {
            values[i] = ArrayOf[i];
        }
		}
		else
		{
			values = NULL;
		}
    }

    
    DensityArray(const DensityArray &objet)                     // copieur
    {
        NumberOfDensity = objet.NumberOfDensity;
		if( NumberOfDensity != 0 && objet.values)
        {
        
            values          = new int[NumberOfDensity];
            for (int i = 0; i < NumberOfDensity; i++)
            {
                values[i] = objet.values[i];
            }
        }
        else
        {
			NumberOfDensity = 0;
            values = NULL;
        }
    }
    
    ~DensityArray()                                             //Default Destructor
    {
        if (values)
            delete[] values;
    }
 //   DensityArray(const MF<int>& UserList);                    //Construct a DensityArray from a GvMFInt32

    DensityArray& operator=(DensityArray& a)                    //Operator = for densityArray
    {
        if(values)
        {
            delete[] values;
        }
        NumberOfDensity = a.NumberOfDensity;
        values = new int[NumberOfDensity];
        for (int i = 0 ; i < NumberOfDensity; i++)
        {
            values[i] = a.values[i];
        }
        return a;
    }
    bool operator==(DensityArray& a)                            //Operator == for a DensityArray
    {
        bool IsEqual= false;
        if (a.NumberOfDensity == NumberOfDensity)
        {   
            IsEqual = true;
            for (int i = 0; i < NumberOfDensity; i++)
            {
                if (a.values[i] != values[i])
                    break;
                    return false; 
            }
        }
        return IsEqual;
    }    
    
    void Sort();                                                //Sort a DensityArray
    void SetDefault(int numberOf,int defaultValue);             //Set the DensityArray to default value

// Public Variables
public:  
    
    int     NumberOfDensity;                                    //How many densities
    int*    values;                                             //The value of each densities
       
};

///////////////////////////////////////////////////////////////////////////////
//
//
//                          Density Matrix Class
//
//
//
//
//////////////////////////////////////////////////////////////////////////////

const int Max_Division = 100;
class DensityMatrix {
      
// Protected members
protected:
    //SL - This flag is used optimise the check density when the user
    //haven't specified any.
    bool            ShowAllDensities;
    
// Public variables
public:
    
    int             (*MyPoints_density)[Max_Division][Max_Division];            //A matrix for stocking density info
    bool            (*MyPoints_HasBeenVisited)[Max_Division][Max_Division];     //A Matrix for stocking already visited density info
    int             Max_i;                                                      //Maximum [i][][] of array
    int             Max_j;                                                      //Maximum [][j][] of array
    int             Max_k;                                                      //Maximum [][][k] of array
    int             Current_i;                                                  //The current [i][][] index
    int             Current_j;                                                  //The current [][j][] index
    int             Current_k;                                                  //The current [][][k] index
    int             division;                                                   //The lenght between two density point
    int             dimension[3];                                               //How many density points per axis
    double          MinMaxArray[6];                                             //Array with min max of good densities per axis
    DensityArray    currentDensity;                                             //Density array with current evaluating densities
    DensityArray    VolumeDensityList;                                          //Density array with all Volume detected densities     
    int             MaxDensity;                                                 //The highest detected densities

    // The BoundingBox Volume corners
    Point C1;                                                                   //the (-x,-y,z) most density point.

// Public Members
public:
   //Set the points index 
   DensityMatrix();                                                            //Constructor
   ~DensityMatrix();   
   
   void  Init(Volume_Tree_Node &The_Node, DensityArray &UserDensArray);                                    //Init a density Matrix
   
   void  EvaluateDensity(Volume_Tree_Node&, double dx, double dy, double dz);  //Evaluate the density for a given point
   void  LookForInside(int& i, int& j, int& k);                                //Return the index of an inside point depending on the currentDensity
   void  GetInsidePoint(int& i, int& j, int& k);                               //LookForInside then FlagNeighbours of this point (if there is one)    
   void  FlagNeighbours(int i, int j, int k);                                  //Flag all neighbours of one detected good density point
                                                                                //this prevent from polygonising more than one time
                                                                                //the same volume
   int   CheckForSolidPoint(int i, int j, int k,
                            unsigned char *SolidPoint);
    
   void  CalculateC1(double dx, double dy, double dz);                         //Calculate the the (-x,-y,z) most density point.
   void  ResetToUnflag();                                                      //Reset all the denisty Point to not visited.  
   void  CheckForMinMax(int i, int j, int k);                                  //Update the MinMaxArray with this point.
   bool  GetBboxAssociatedGrid(float& grid);                                   //Change the Jule grid if volume between K_VolumeGridHighLimit and K_VolumeGridLowLimit
	bool  CheckDensity(int resulEval);
   };	


float minFloat(float a, float b);                                               //Minimum of two float  
int maxInt(int a, int);                                                         //Maximum of two int
void ValidateDensities(int A_INT_Densities[], int *UserDensities);  //Correct an invalid density


#endif  //_DensityTLS_
