// -*- c++ -*-
/***********************************************************************
This software module was originally developed by Alain Mignot (Mignot Informatique Graphique Inc)
 in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. (Mignot Informatique Graphique Inc.) retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2004
************************************************************************/


#include "OpenGl/OpenGl.h"
#include "ImplicitProxy.h"
#include "opengl/base/OpenGLVisualRenderer.h"



// support classes



// Constructor

ImplicitProxy::ImplicitProxy(Implicit* pImplicit)
  : GeometryProxy(pImplicit)
{

    grid  = Global_Grid;
	created = FALSE;
		m_nTimeStamp = -1;

//AM 1    densMatrix.dimension[0] = 0;      
//AM 1    densMatrix.dimension[1] = 0;
//AM 1    densMatrix.dimension[2] = 0;
    
//    TypeOfMapping.set_("Planar"); //AM
//   TransformMatrix.Identity();	 

}

// Destructor

ImplicitProxy::~ImplicitProxy()
{
  // ici destruction

}



Volume_Tree_Node *ImplicitProxy::GetVolumeTreeNode()
{    
    int CoeffNumber = 0;
    int NodeDensities[2];


	ValidateDensities(NodeDensities, GetDensities());

    CoeffNumber = GetCoefNumber();
	
    
    if(CoeffNumber == K_PlanarNumberOfCoeff)
    {
        //If the user has defined the implicit function as a first degree
        //function, convert it to a quadric.
		float *c;
		c = GetCoef();
   //     MF<float> TempCoeff = { 0, 0, 0, 0, 0, 0, c[0], c[1], c[2], c[3] };

// AM a faire        SetCoef(TempCoeff);
		CoeffNumber = 10;
    }


	Volume_Tree_Node *to_return = new SolidNode( GetCoef() ,
											   CoeffNumber,
											   GetSolid(),
											   GetbboxSize(),
                                             NodeDensities);


	if ( GetDual() )
		to_return = new Volume_Tree_Unary_Op(SGDLNot, to_return, GetbboxSize());

	return to_return;    
}




// Render object description

void
ImplicitProxy::RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD /* dwTime */)
{
  // a voir pour color
	GLfloat color[] = { 0.0f, 0.0f, 1.0f, 0.75f};

  
  glColor4fv(color);

  RenderDescriptionBox(pRenderer, "Implicit");
}

// Render object

BOOL
ImplicitProxy::Render(OpenGLVisualRenderer* pRenderer, DWORD dwTime)
{
 
 	Implicit *pImplicit=GetImplicit();

	_needUpdate=long(pImplicit->GetUpdateTime()) > m_nTimeStamp ;

	if (_needUpdate || !created)
	{
		m_nTimeStamp=(long) dwTime;
       
        Evaluate();  
	}
    
 
//
// faire le rendu  partir due la representation cre dans Evaluate()
	Matrix rMatrix;

	SolidTree->Render( rMatrix);

  return TRUE;
}


// Traverse scene graph, checking for intersection

BOOL
ImplicitProxy::Select(OpenGLVisualRenderer* pRenderer, DWORD dwTime, unsigned int& depth)
{
  GLuint selectBuffer[4];

  ::glSelectBuffer(4, selectBuffer);
  ::glRenderMode(GL_SELECT);

  Render(pRenderer, dwTime);

  if (::glRenderMode(GL_RENDER) > 0)
    if (depth >= selectBuffer[1]) {
      depth = selectBuffer[1];

      return TRUE;
    }

#ifdef _DEBUG
  if (pRenderer->CheckGLError("ImplicitProxy:"))
    TRACE("ImplicitProxy::Render() [%08x]: ", this);
#endif

  return FALSE;
}

// en provenance de GvImplicit

int ImplicitProxy::Evaluate(){
    


    int sz;
    sz = GetCoefNumber();  
    
    // ici Texture

    if ( (sz == 10) || (sz == 35) || (sz == 4))
    { 
		

        SolidTree = GetVolumeTreeNode();
		
         densMatrix.dimension[0] = GetbboxSize().x + 1;
        densMatrix.dimension[1] = GetbboxSize().y + 1;
        densMatrix.dimension[2] = GetbboxSize().z + 1;
        
        //Add a link to the tree
  //      SolidTree->Ref();

        SolidTree->SetGraphic(NewDLzK_coe(densMatrix,
                            GetGrid(),
                            *SolidTree,
                            TextureURL,
                            &TypeOfMapping,
                            TransformMatrix,
                            DensityArray(),
                            true));
        

    //    SolidTree->UnRef();


    }

    else
    {
// AM        Reporter.Error("Bad coefficients number in planar, quadric or quartic , must be 4, 10 or 35 respectively");
    }
      
	created = true;
	
    return (1);
}

// is p outside the box ? 

Bool ImplicitProxy::isOutside(Vec3f &p)
{

    return (!SolidTree->isInside(p.x,p.y,p.z));


	return FALSE;
}
