/***********************************************************************
This software module was originally developed by Chloe Mignot,Eric Baillargeon 
and Eric Paquet, (SGDL Systme Inc. ) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. (SGDL Systme Inc.) retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2001
************************************************************************/

#include "stdafx.h"
#include <assert.h>
#include "Point.h"
#include "Polygonise.h"
#include "SolidGraphic.h"
#include "z2_coo.h"



#include "VolumeTreeNode\VolumeTreeNode.h"
#include "VolumeTreeNode\SolidTools.h"
#include "z2_coo.h"
#include "DensityTLS.h"
#include "AdaptiveRefinement.h"

//#define         IndexedFace 
#define         NoGoodVolume         0

static bool     FileOpen           = FALSE;
const  int      K_InvertOfInterval = 4;
const  int      safe               = 300;
static eAxis    Direction          = eAxisX;
static double   eps                = 0.05;

#include <assert.h>

SolidGraphic *NewDLzK_coe(DensityMatrix& densMatrix, float grid, Volume_Tree_Node& The_Node,
                     SFString TextureURL, SFString* TypeOfMapping,const SL_Matrix& TransformMatrix, DensityArray UserDensArray , 
                     bool manage_normals, bool manage_color)
{   
    
#ifdef IndexedFace
    if (!FileOpen)
{             
        remove("template.debug.vrml");
        FILE* fp = fopen("template.debug.vrml","a");    
        fprintf(fp, "%s \n", "#VRML V2.0 utf8" );
        fclose(fp);
        FileOpen = TRUE;
    }
    
#endif
    
    // Local variables
    Shapejb*  shape[100]; 
    int bounds[3];
	double box[3];    

    int         i                 = 0;
    int         cpt               = 0; 
    int         nbvertex          = 0;
    int         nbface            = 0;
    int         compteur          = 0;
    double      maxpos            = 0;
    double      maxneg            = 0;    
    ZArray<Vec3f>  **ColorTable      = NULL;
    ZArray<Vec3f>  *ShapeIColor      = NULL;
    
    //This object is used to refine the polygons
    //depending on the normals of the implicit shape (SL)
    AdaptiveRefiner *AdaptRef     = NULL;
    SolidGraphic *Sgraphic = NULL;

    // Init the densityMatrix
    densMatrix.Init(The_Node, UserDensArray);
    bounds[0]   = (int)((densMatrix.dimension[0]/grid) * 2);
	bounds[1]   = (int)((densMatrix.dimension[1]/grid) * 2);
	bounds[2]   = (int)((densMatrix.dimension[2]/grid) * 2);
        
    box[0]  = densMatrix.dimension[0];
	box[1]  = densMatrix.dimension[1];
	box[2]  = densMatrix.dimension[2];

    int _i(0),_j(0),_k(0);

    //Polygonise the volume
    do 
    {        
        densMatrix.GetInsidePoint(_i,_j,_k);
        if (densMatrix.GetBboxAssociatedGrid(grid))
        {           
            //Recalculate the bounds and box in case of a [K_VolumeGridLowLimit, K_VolumeGridHighLimit] volume
            bounds[0]   = (int)((densMatrix.dimension[0]/grid) * 2);
	        bounds[1]   = (int)((densMatrix.dimension[1]/grid) * 2);
	        bounds[2]   = (int)((densMatrix.dimension[2]/grid) * 2);
	        
            box[0]  = densMatrix.dimension[0];
	        box[1]  = densMatrix.dimension[1];
	        box[2]  = densMatrix.dimension[2];
        }

        if (_i > -1)
        {
            Vec3f p;
            p.x = densMatrix.C1.x + (_i * ((float)densMatrix.dimension[0] / (float)densMatrix.division));
            p.y = densMatrix.C1.y + (_j * ((float)densMatrix.dimension[1] / (float)densMatrix.division));
            p.z = densMatrix.C1.z - (_k * ((float)densMatrix.dimension[2] / (float)densMatrix.division));

            shape[cpt] = new Shapejb(bounds,grid,box); 
            shape[cpt]->solidshape(The_Node, p);
            
            nbvertex += shape[cpt]->Svertices()->count;

            if ( shape[cpt] != NULL )
                cpt++;
        }       
    }  

    while(_i != -1);
  
    if ((cpt == NoGoodVolume) || (nbvertex == 0))
    { 
        // Here the equations is no good nor the grid | densitymatrix size too large.
        return (NULL);
    }    
    
    if (manage_color)
        ColorTable = new ZArray<Vec3f>* [cpt];

    for (i = 0; i < cpt; i++)
    {
        if (manage_color)
        {
            ColorTable[i] = new ZArray<Vec3f>();
			ColorTable[i]->SetSize((shape[i]->Svertices())->count);
            ShapeIColor = ColorTable[i];             
        }
        
        ComputeNormalsAndColors(&The_Node, shape[i]->Svertices(), ShapeIColor, manage_normals);
    }
    
#if 0 // temporaire

    //This part is used to refine the triangles according
    //to local curvature of the implicit surface (SL).
    if ((MAX_SUBDIVISION_LEVEL != 0) && manage_normals)
    {
        AdaptRef = new AdaptiveRefiner(&The_Node);        

        for (int i = 0; i < cpt; i++)
        {
            if (manage_color)
                ShapeIColor = ColorTable[i];
            
            AdaptRef->Refine(shape[i], ShapeIColor);
        }
            
        delete AdaptRef;
    }
    
#endif
	Sgraphic = new SolidGraphic( shape, cpt, ColorTable);

    


     if (manage_color)
        delete [] ColorTable;

    //return the graphic
    return(Sgraphic);   		
}

//***************************************************
//	
//  ComputeNormalsAndColors()
//
//	Author  : Simon Lavallee, SGDL Systems
//	Date    : March 27, 2002
//
//  Input   : Volume_Tree_Node *TheNode      -> Ptr to the volume information.
//            VERTICES         *Verts        -> Ptr to the vertices produced by
//                                              the JB polygonizer.
//            bool             GetNormals    -> Flag to tell if we want the normals or 
//                                              just the color
//  Outpout : PointArray       *ColorTable   -> Ptr to the array containing the color
//                                              of each vertex.
//
//  Desc.   : This function get the normal for 
//            each vertex produced by the JB polygonizer and
//            also fill the color table for each vertex.
//***************************************************
void ComputeNormalsAndColors(Volume_Tree_Node *TheNode, VERTICES *Verts, ZArray<Vec3f> *ColorTable, bool GetNormals)
{
    VERTEX *VertexPtr    = Verts->ptr;
    int    VertexNb      = Verts->count;

    double Distance      = 0.0;

    double NormalLength  = 0.0;
    double Normal[3]     = {0.0, 0.0, 0.0};    
    bool   NormalIsValid = false;

    float  Color[3]      = {-1.0, -1.0, -1.0};
    float  *ColorPtr     = NULL;
    
    for (int i = 0; i < VertexNb; i++)
    {
        if (ColorTable)
        {
            //Try to get the color
            TheNode->GetColor(VertexPtr[i].position.x,
                              VertexPtr[i].position.y,
                              VertexPtr[i].position.z,
                              Color);
            
            if (Undefined_Color(Color))
                //If we haven't been able to get the color, keep a reference
                //to get the color when evaluating the normal.
                ColorPtr = Color;
        }        

        if (GetNormals || ColorPtr)
        {   
            //If we want the normals or we've not been able to get the color.
            //This could be optimised since all the normal calculation is done,
            //without knowing if we just want the color...
            NormalIsValid = TheNode->EvaluateGradient(VertexPtr[i].position.x,
                                                      VertexPtr[i].position.y,
                                                      VertexPtr[i].position.z,                                           
                                                      Distance, Normal, ColorPtr);
        }
        
        if (GetNormals)
        {
            //Ensure we got a normals
            assert(NormalIsValid);
            
            NormalLength  = sqrt(Normal[0]*Normal[0] +
                                 Normal[1]*Normal[1] +
                                 Normal[2]*Normal[2]);
            
            if (NormalLength != 0)
            {
                //Normalize
                Normal[0] /= NormalLength;
                Normal[1] /= NormalLength;
                Normal[2] /= NormalLength;
            }
            else
                assert(NormalLength != 0);

            
            //TBM **** This check should be done inside ****************
            //******** Volume_Tree_Node::EvaluateGradient() ************
            if (!TheNode->ShowAllDensities)
            {
                if (!TheNode->isInside(VertexPtr[i].position.x - Normal[0]*(fabs(Distance) + EPSILON*0.01),
                                       VertexPtr[i].position.y - Normal[1]*(fabs(Distance) + EPSILON*0.01),
                                       VertexPtr[i].position.z - Normal[2]*(fabs(Distance) + EPSILON*0.01)) &&
                     TheNode->isInsideWithoutDensityCheck(VertexPtr[i].position.x - Normal[0]*(fabs(Distance) + EPSILON*0.01),
                                                          VertexPtr[i].position.y - Normal[1]*(fabs(Distance) + EPSILON*0.01),
                                                          VertexPtr[i].position.z - Normal[2]*(fabs(Distance) + EPSILON*0.01)))
                {
                    Normal[0] *= -1;
                    Normal[1] *= -1;
                    Normal[2] *= -1;
                }
            }
            //TBM *** END **********************************************

            //Put the calculated normal in the vertex list
            VertexPtr[i].normal.x = Normal[0];
            VertexPtr[i].normal.y = Normal[1];
            VertexPtr[i].normal.z = Normal[2];
        }

        if (ColorTable)
        {
            //Fill the color table
   //         (*ColorTable)[i].Set(Color);
			(*ColorTable)[i].x = Color[0];
			(*ColorTable)[i].y = Color[1];
			(*ColorTable)[i].z = Color[2];

            //Reset the color (for the next iter...)
            Color[0] = -1;
            Color[1] = -1;
            Color[2] = -1;
            ColorPtr = NULL;
        }
    }
}
#ifdef AMTEXTURE

/********************************************************************\
    Function:  WriteIndexedFaceFile   
                This function write a indexed face set VRML file with all the mesh info.
    Input:     
    Output:    

    Author:     Eric Baillargeon SGDL Systme Inc.
    Created:    2001/11/19	

    
*********************************************************************/

void WriteIndexedFaceFile(ZArray<Vec3f> &_p, int vi, int nbvertex, GVertexI* verts, PointArray &color_table,const SL_Matrix& TransformMatrix)
{

    //To be optimized ... 
    FILE* fp = fopen("template.debug.vrml","a"); 

    fprintf(fp, "%s \n", "Shape { ");
    fprintf(fp, "%s \n", "appearance DEF MainApp Appearance{ material Material{ambientIntensity 0.02");
    fprintf(fp, "%s \n ", "diffuseColor 0.5 1.3 0.5"  );   
    fprintf(fp, "%s \n ", "}  }"  );
    fprintf(fp, "%s \n ", "geometry IndexedFaceSet {  creaseAngle 3.14   coord Coordinate {");
    fprintf(fp, "%s \n ", "point["  );
    
             
    for(int p1 = 0 ; p1 < nbvertex; p1++)
    {
        if (!TransformMatrix.IsIdentity())
        {
          _p[p1] =  TransformMatrix * _p[p1];
        }
  
        fprintf(fp, "%f  %f  %f, \n ",_p[p1].x,_p[p1].y,_p[p1].z);
    }

    fprintf(fp, "%s ", "] } coordIndex [");
    for (int pp = 0 ; pp < vi; pp++)
    {
        static int boucle = 0;
        fprintf(fp, "%d ", verts[pp].v);
        boucle++;
        if (boucle > 2)
        {
            fprintf(fp, "%d \n" , -1);
            boucle = 0;
        }
        
    }
  
    if (color_table.Length() > 0)
    {
    
        fprintf(fp, "%s \n", " ] color Color { ");
        fprintf(fp, "%s \n", " color [");
        for (int ppp = 0 ; ppp < nbvertex; ppp++)
        {        
            fprintf(fp, "%f , %f , %f, \n ", color_table[ppp].x, color_table[ppp].y, color_table[ppp].z);        
        }
        fprintf(fp, "%s ", "] } ");

        fprintf(fp, "%s \n", " colorPerVertex TRUE");
        fprintf(fp, "%s \n", " colorIndex [");
    }

    for (int pl = 0 ; pl < vi; pl++)
    {
        static int boucle = 0;
        fprintf(fp, "%d ", verts[pl].v);
        boucle++;
        if (boucle > 2)
        {
            fprintf(fp, "%d \n" , -1);
            boucle = 0;
        }
        
    }

    fprintf(fp, "%s ", "] } }");
    fclose(fp);
}

/********************************************************************\
    Function:  ComputeTextureCoordinate              	
    Input:     PointArray &params
    Output:    
    
    Author:     Eric Baillargeon SGDL Systme Inc.
    Created:    2001/11/19	
    
    
*********************************************************************/
    
void ComputeTextureCoordinate(PointArray &params, 
                              int nbvertex, 
                              PointArray &_p, 
                              GShellI *o,
                              SFString* TypeOfMapping,
                              Volume_Tree_Node& The_Node)
{
    double TexBounds[6];  
    double coeff[10];
    double  tdx,tdy,tdz;    
    GetBounds(nbvertex, _p, TexBounds);        
    BBox bbox;
    Point Center, Size;
     
    Size.x = (TexBounds[eBounds_MaxX] - TexBounds[eBounds_MinX]);
    Size.y = (TexBounds[eBounds_MaxY] - TexBounds[eBounds_MinY]);
    Size.z = (TexBounds[eBounds_MaxZ] - TexBounds[eBounds_MinZ]);
    
    Center.x = TexBounds[eBounds_MinX] +  (Size.x / 2.0f);
    Center.y = TexBounds[eBounds_MinY] +  (Size.y / 2.0f);
    Center.z = TexBounds[eBounds_MinZ] +  (Size.z / 2.0f);   
    bbox.SetCenterSize(Center,Size);

    eMapping index = GetMappingIndex(TypeOfMapping);
    int l5 = 0;
    switch(index) 
    {
        case eMapping_Spheric:
            ComputeSphericParams(bbox, o->v, params);        
    	    break;
        case eMapping_Cylindric:
            ComputeCylindricParams(bbox, o->v, params);
    	    break;
        case eMapping_Box:
            ComputeBoxParams(bbox, o->v, params);
    	    break;
        case eMapping_Gradient:
            
            getMinimalBounds(The_Node, nbvertex, _p, TexBounds); 
            
            tdx = (TexBounds[eBounds_MaxX] - TexBounds[eBounds_MinX]) / 2.0f;
            tdy = (TexBounds[eBounds_MaxY] - TexBounds[eBounds_MinY]) / 2.0f;
            tdz = (TexBounds[eBounds_MaxZ] - TexBounds[eBounds_MinZ]) / 2.0f;

            ConstructCylinder(TexBounds,coeff);
            for (l5; l5 < nbvertex; l5++)
            { 
        
                Point g = GetTextureCoordinate(The_Node, _p[l5].x ,_p[l5].y , _p[l5].z, coeff, Direction);         
                switch(Direction) 
                {
                case eAxisX:
                    {

                            double a = 1 - (acos(g.y/ tdy)) /(2*PI);
       
                            if (g.z > 0.0){
                                a = 1 - a;
                            }
                            
                            params[l5].Set(1 - a,(1 - (g.x + tdx)/(2*tdx)) , 0.0);    

                    }
        
    	            break;
                case eAxisY:
                    {
        

                            double a = 1 - (acos(g.x/ tdx)) /(2*PI);
       
                            if (g.z > 0.0){
                                a = 1 - a;
                            }
                            
                            params[l5].Set(1 - a,(1 - (g.y + tdy)/(2*tdy)) , 0.0);      

	        
                    }
    	            break;
                case eAxisZ:
                    {

                            double  a = 1 - (acos(g.x/ tdx)) /(2*PI);
       
                            if (g.y > 0.0){
                                a = 1 - a;
                            }      
                                                            
                            params[l5].Set(1 - a,(1 - (g.z + tdz)/(2*tdz)) , 0.0);                                 
            
                    }
                    break;
                default:
                    break;
                }       
                
            }

            break;
            
        default:
            break;
    }
}

#endif
/********************************************************************\
    Function:  GetMappingIndex
              	
    Input:     TypeOfMapping  :   Type of mapping (spheric, cylindric, Box)
    Output:    eMapping       :   The enum index. 
    
    Author:     Eric Baillargeon SGDL Systme Inc.
    Created:    2001/11/19	

		
*********************************************************************/
eMapping GetMappingIndex(SFString* TypeOfMapping)
{
    
#if 0 // AM
	const ZString* mapping = LPCSTR TypeOfMapping;
	ZString f,spheric,plane,cylindric,gradient;
    f = mapping;
    spheric = new ZString("Spherical");
    plane = new ZString("Planar");
    cylindric = new ZString("Cylindrical");
    gradient =  new ZString("Gradient");
#else

    const char *f = LPCSTR(TypeOfMapping );
    const char *spheric = "Spherical";
    const char *plane = "Planar";
    const char *cylindric = "Cylindrical";
    const char *gradient =  "Gradient";
#endif
    
    
    if (f == spheric)
    {
        return eMapping_Spheric;
    }        
    else if (f == plane)
    {
        return eMapping_Box;
    }
    else if (f == cylindric)
    {
        return eMapping_Cylindric;
    }
    else if (f == gradient)
    {
        return eMapping_Gradient;
    }

    return eMapping_Box;
}


#ifdef AMTEXTURE
/********************************************************************\
    Function:   getMinimalBounds 
                Get minimal Bounds for gradient mapping purpose
              	
    Input:      Volume_Tree_Node& The_Node, int nbvertex , PointArray _pVertex, double Bounds[6]
    Output:     
    
    Author:     Eric Baillargeon SGDL Systme Inc.
    Created:    2001/11/06	

		
*********************************************************************/
void getMinimalBounds(Volume_Tree_Node& The_Node, int nbvertex , PointArray _pVertex, double Bounds[6])
{
    GetBounds( nbvertex ,_pVertex, Bounds);
    double max = 0;

    // Migrate the bounds value from iso-value 0 to iso-value 1.
    // Desactivated for now ...  Do not remove nor change ..... 
    /* double b1 = Bounds[eBounds_MaxX]; 
    double b2 = Bounds[eBounds_MinX]; 
    double b3 = Bounds[eBounds_MaxY];   
    double b4 = Bounds[eBounds_MinY];   
    double b5 = Bounds[eBounds_MaxZ];   
    double b6 = Bounds[eBounds_MinZ]; 
   */

    /*b1 = Migrate(The_Node,_pVertex[px].x, _pVertex[px].y,_pVertex[px].z, 1, 0);    
    b2 = Migrate(The_Node,_pVertex[nx].x, _pVertex[nx].y,_pVertex[nx].z, 1, 0); 
    b3 = Migrate(The_Node,_pVertex[py].x, _pVertex[py].y,_pVertex[py].z, 1, 1); 
    b4 = Migrate(The_Node,_pVertex[ny].x, _pVertex[ny].y,_pVertex[ny].z, 1, 1); 
    b5 = Migrate(The_Node,_pVertex[pz].x, _pVertex[pz].y,_pVertex[pz].z, 1, 2); 
    b6 = Migrate(The_Node,_pVertex[nz].x, _pVertex[nz].y,_pVertex[nz].z, 1, 2);*/

   /* Bounds[eBounds_MaxX] = Migrate(The_Node,_pVertex[px].x, _pVertex[px].y,_pVertex[px].z, 1, 0);
    Bounds[eBounds_MinX] = Migrate(The_Node,_pVertex[nx].x, _pVertex[nx].y,_pVertex[nx].z, 1, 0);
    Bounds[eBounds_MaxY] = Migrate(The_Node,_pVertex[py].x, _pVertex[py].y,_pVertex[py].z, 1, 1);
    Bounds[eBounds_MinY] = Migrate(The_Node,_pVertex[ny].x, _pVertex[ny].y,_pVertex[ny].z, 1, 1);
    Bounds[eBounds_MaxZ] = Migrate(The_Node,_pVertex[pz].x, _pVertex[pz].y,_pVertex[pz].z, 1, 2);
    Bounds[eBounds_MinZ] = Migrate(The_Node,_pVertex[nz].x, _pVertex[nz].y,_pVertex[nz].z, 1, 2);*/

    double dx = (Bounds[eBounds_MaxX] - Bounds[eBounds_MinX]) / 2.0;   
    double dy = (Bounds[eBounds_MaxY] - Bounds[eBounds_MinY]) / 2.0;
    double dz = (Bounds[eBounds_MaxZ] - Bounds[eBounds_MinZ]) / 2.0;
    Direction = getAxisDirection(dx,dy,dz);
    for(int j = 0 ; j < eBounds_NumberOf; j++)
    {    
        if (fabs(Bounds[j] > max))
        max  = fabs(Bounds[j]);
    }
        Bounds[eBounds_MaxX] = (Bounds[eBounds_MaxX]/(fabs(Bounds[eBounds_MaxX] )) *max);
        Bounds[eBounds_MinX] = (Bounds[eBounds_MinX]/(fabs(Bounds[eBounds_MinX] )) *max);
        Bounds[eBounds_MaxY] = (Bounds[eBounds_MaxY]/(fabs(Bounds[eBounds_MaxY] )) *max);
        Bounds[eBounds_MinY] = (Bounds[eBounds_MinY]/(fabs(Bounds[eBounds_MinY] )) *max);
        Bounds[eBounds_MaxZ] = (Bounds[eBounds_MaxZ]/(fabs(Bounds[eBounds_MaxZ] )) *max);
        Bounds[eBounds_MinZ] = (Bounds[eBounds_MinZ]/(fabs(Bounds[eBounds_MinZ] )) *max);

    for (int k = 0; k < eBounds_NumberOf; k++)
    {
        if (Bounds[k] >= 0)
        {
            Bounds[k] += 1.0;
        }
        else
        {
            Bounds[k] -= 1.0;
        }
    }
}

/********************************************************************\
    Function:   GetBounds 
                Get minimal Bounds for non gradient mapping purpose
              	
    Input:      int nbvertex , PointArray _pVertex, double Bounds[]
    Output:     
    
    Author:     Eric Baillargeon SGDL Systme Inc.
    Created:    2001/11/06	

		
*********************************************************************/
void GetBounds( int nbvertex , PointArray _pVertex, double Bounds[])
{
    Bounds[0] = _pVertex[0].x;
    Bounds[1] = _pVertex[0].x;
    Bounds[2] = _pVertex[0].y;
    Bounds[3] = _pVertex[0].y;
    Bounds[4] = _pVertex[0].z;
    Bounds[5] = _pVertex[0].z;
    for (int i = 0 ; i < nbvertex; i++)
    {
        if (_pVertex[i].x > Bounds[eBounds_MaxX] ){
            Bounds[eBounds_MaxX] = _pVertex[i].x;
            
        }

        if (_pVertex[i].x < Bounds[eBounds_MinX] ){
            Bounds[eBounds_MinX] = _pVertex[i].x;
            
        }

        if (_pVertex[i].y > Bounds[eBounds_MaxY] ){
            Bounds[eBounds_MaxY] = _pVertex[i].y;
            
        }

        if (_pVertex[i].y < Bounds[eBounds_MinY] ){
            Bounds[eBounds_MinY] = _pVertex[i].y;
            
        }

        if (_pVertex[i].z > Bounds[eBounds_MaxZ] ){
            Bounds[eBounds_MaxZ] = _pVertex[i].z;

    }

        if (_pVertex[i].z < Bounds[eBounds_MinZ] ){
            Bounds[eBounds_MinZ] = _pVertex[i].z;
            
        }
    }
}

/********************************************************************\
    Function:   Migrate
              	Not use right now ...
    Input:     
    Output:     
    
    Author:     Eric Baillargeon SGDL Systme Inc.
    Created:    2001/11/07	

		
*********************************************************************/
double Migrate(Volume_Tree_Node& The_Node,float x,float y, float z,int isoValue,int index)
{
    Point p;
    p.x = x;
    p.y = y;
    p.z = z;
    Point gradient;
    double TmpGradient[3];
    double value;    
    double *coeff = The_Node.GetCoeff(x, y, z);
    int    nCoeff = The_Node.GetNumOfCoeff(x, y, z);

    if (nCoeff == 10)
    {
        value = DLz2_coeR(coeff, x, y, z,0);
        Dlz2_Gradient(coeff, x, y, z, TmpGradient);            
    }
        
    else
    {
        value = DLz4_coeR(coeff, x, y, z, 0);
        Dlz4_Gradient(coeff, x, y, z, TmpGradient);
    }
    
    gradient.Set(TmpGradient[0], TmpGradient[1], TmpGradient[2]);
    gradient.Normalize();    
    
    do 
    {
        if (nCoeff == 10)
        {
        value = DLz2_coeR(coeff,p.x,p.y,p.z,0);

        }
        else
        {
            value = DLz4_coeR(coeff,p.x,p.y,p.z,0);
        }
        p += 0.01*gradient;
    }
    while(value > -2.0);

    switch(index) {
    case 0:
        return p.x;
    	break;
    case 1:
        return p.y;
    	break;
    case 2:
        return p.z;
        break;
    default:
        return 0.0;
    }

}

#endif
/********************************************************************\
    Function:  ConstructCylinder
              	
    Input:     Bounds[6] : The bounds of the volume
    Output:     
    
    Author:     Eric Baillargeon SGDL Systme Inc.
    Created:    2001/11/07	

		
*********************************************************************/
void ConstructCylinder(double Bounds[6], double coeff[10])
{
    double dx = (Bounds[eBounds_MaxX] - Bounds[eBounds_MinX]) / 2.0;   
    double dy = (Bounds[eBounds_MaxY] - Bounds[eBounds_MinY]) / 2.0;
    double dz = (Bounds[eBounds_MaxZ] - Bounds[eBounds_MinZ]) / 2.0;
    double ControlPoints[24];

   
    switch(Direction) {

        // x oriented Axis
    case eAxisX:
        {        
            ControlPoints[0] =  0; 
            ControlPoints[1] =  1 * dy;
            ControlPoints[2] =  0;
            ControlPoints[3] =  1;

            ControlPoints[4] =  0;
            ControlPoints[5] =  -1 * dy;
            ControlPoints[6] =  0;
            ControlPoints[7] =  1;

            ControlPoints[8] =  1 * dx;
            ControlPoints[9] =  0;
            ControlPoints[10] = 0;
            ControlPoints[11] = 0; 

            ControlPoints[12] = 0;
            ControlPoints[13] = 0;
            ControlPoints[14] = 1 * dz;
            ControlPoints[15] = 0;

            ControlPoints[16] = 1 * dx;
            ControlPoints[17] = 0; 
            ControlPoints[18] = 0;
            ControlPoints[19] = 0;
            
            ControlPoints[20] = 0;
            ControlPoints[21] = 0;
            ControlPoints[22] = 1 * dz;
            ControlPoints[23] = 1; 
        }
    	break;
        // y oriented Axis
    case eAxisY:
        {
            ControlPoints[0] =  -1 * dx; 
            ControlPoints[1] =  0;
            ControlPoints[2] =  0;
            ControlPoints[3] =  1;

            ControlPoints[4] =  1 * dx;
            ControlPoints[5] =  0;
            ControlPoints[6] =  0;
            ControlPoints[7] =  1;

            ControlPoints[8] =  0;
            ControlPoints[9] =  1 * dy;
            ControlPoints[10] = 0;
            ControlPoints[11] = 0;
            
            ControlPoints[12] = 0;
            ControlPoints[13] = 0;
            ControlPoints[14] = 1 * dz;
            ControlPoints[15] = 0;

            ControlPoints[16] = 0;
            ControlPoints[17] = 1 * dy;
            ControlPoints[18] = 0;
            ControlPoints[19] = 0;
            
            ControlPoints[20] = 0 ;
            ControlPoints[21] = 0 ;
            ControlPoints[22] = 1 * dz;
            ControlPoints[23] = 1; 
        }
    	break;

        // z oriented Axos
    case eAxisZ :
        {
            ControlPoints[0] =  -1 * dx; 
            ControlPoints[1] =  0;
            ControlPoints[2] =  0;
            ControlPoints[3] =  1;

            ControlPoints[4] =  1 * dx;
            ControlPoints[5] =  0;
            ControlPoints[6] =  0;
            ControlPoints[7] =  1;

            ControlPoints[8] =  0;
            ControlPoints[9] =  0;
            ControlPoints[10] = 1 * dz;
            ControlPoints[11] = 0;
            
            ControlPoints[12] = 0;
            ControlPoints[13] = 1 * dy;
            ControlPoints[14] = 0;
            ControlPoints[15] = 0;

            ControlPoints[16] = 0;
            ControlPoints[17] = 0;
            ControlPoints[18] = 1 * dz;
            ControlPoints[19] = 0;
            
            ControlPoints[20] = 0 ;
            ControlPoints[21] = 1 * dy;
            ControlPoints[22] = 0;
            ControlPoints[23] = 1; 
        }
        break;
        
    default:
        break;
    }
            
    SGDL_z2_coo( coeff, ControlPoints);	
}

/********************************************************************\
    Function: getAxisDirection  
              This function return the Greater direction Axis.
              	
    Input:  (dx,dy,dz)   
    Output: eAxis     
    
    Author:     Eric Baillargeon SGDL Systme Inc.
    Created:    2001/11/07	

		
*********************************************************************/
eAxis getAxisDirection(double dx,double dy, double dz)
{
    eAxis to_return;
    double max = maxDouble(dx,dy,dz);
    if (max == (float) dx)
        to_return = eAxisX;
    if (max == (float) dy)
        to_return = eAxisY;
    if (max == (float) dz)
        to_return = eAxisZ;
	return to_return;
}

/********************************************************************\
    Function: maxDouble
              	
    Input:  double a, double b, double c
    Output: The maximum of (a,b,c);     
    
    Author:     Eric Baillargeon SGDL Systme Inc.
    Created:    2001/11/07	

		
*********************************************************************/
double maxDouble(double a, double b, double c)
{
    double max = 0;
    max = (a  > max) ? (a) : (max);
    max = (b  > max) ? (b) : (max);
    max = (c  > max) ? (c) : (max);
    return max;

}

/********************************************************************\
    Function:   ConstructSphere
              	
    Input:      Bounds[6] : The bounds of the volume
    Output:     
    
    Author:     Eric Baillargeon SGDL Systme Inc.
    Created:    2001/11/07	

		
*********************************************************************/
void ConstructSphere(double Bounds[6], double coeff[10])
{
    double dx = (Bounds[eBounds_MaxX] - Bounds[eBounds_MinX]) / 2.0;   
    double dy = (Bounds[eBounds_MaxY] - Bounds[eBounds_MinY]) / 2.0;
    double dz = (Bounds[eBounds_MaxZ] - Bounds[eBounds_MinZ]) / 2.0;
    
    double ControlPoints[24];

    ControlPoints[0] = -1 * dx; 
    ControlPoints[1] =  0 * dy;
    ControlPoints[2] =  0 * dz;
    ControlPoints[3] =  1;
    ControlPoints[4] =  1 * dx;
    ControlPoints[5] =  0 * dy;
    ControlPoints[6] =  0 * dz;
    ControlPoints[7] =  1;
    ControlPoints[8] =  0 * dx;
    ControlPoints[9] =  1 * dy;
    ControlPoints[10] = 0 * dz;
    ControlPoints[11] = 0; 
    ControlPoints[12] = 0 * dx;
    ControlPoints[13] = 0 * dy;
    ControlPoints[14] = 1 * dz;
    ControlPoints[15] = 0;
    ControlPoints[16] = 0 * dx;
    ControlPoints[17] = 1 * dy;
    ControlPoints[18] = 0 * dz;
    ControlPoints[19] = 1;
    ControlPoints[20] = 0 * dx;
    ControlPoints[21] = 0 * dy;
    ControlPoints[22] = 1 * dz;
    ControlPoints[23] = 1; 
    SGDL_z2_coo( coeff, ControlPoints);    
}

/********************************************************************\
    Function:   MagneticField
                Compute the resulting of two forces.
              	
    Input:     F0 : Force 1
               F1 : Force 2
               K0 : Power of force 1
               K1 : Power of force 2
    Output:     
    
    Author:     Eric Baillargeon SGDL Systme Inc.
    Created:    2001/11/06	

		
*********************************************************************/
Point MagneticField(Point F0, Point F1, float k0, float k1)
{
    Point p = (0.2)*F0 + (0.8)*F1;
    p.Normalize();
    return (p);
}


/********************************************************************\
    Function:  ShortestPath
                Find the shortest path between a point (middle of a cylinder for exemple)
                and a parametric surfaces
              	
    Input:     Point p , Point center
    Output:    a Point representing the path
    
    Author:     Eric Baillargeon SGDL Systme Inc.
    Created:    2001/11/06	

		
*********************************************************************/
Point ShortestPath(Point p, Point center)
{
    Point direction;
    Point Diff = p - center;
    direction = Diff.Normalize();

    return direction;
}

/********************************************************************\
    Function:  GetTextureCoordinate
               Find gradient texture mapping coordinate
              	
    Input:    
    Output:    
    
    Author:     Eric Baillargeon SGDL Systme Inc.
    Created:    2001/11/06	

		
*********************************************************************/
Point GetTextureCoordinate(Volume_Tree_Node& The_Node,double x, double y, double z, double coeff[10], eAxis axis)
{       
    double * NodeCoeff = The_Node.GetCoeff(x,y,z);
    int      numOfCoeff = The_Node.GetNumOfCoeff(x,y,z);
    Point current;
   
    Point previous;

    previous.x = current.x = x;
    previous.y = current.y = y;
    previous.z = current.z = z;    
   
    Point F0;
    Point F1;
    double TempGradient[3];
    float k0 = 0;
    float k1 = 0;  
    Point center(0,0,0);

    switch(axis) 
    {
    case eAxisX:
        center.x = current.x;
    	break;
    case eAxisY:
         center.y = current.y;
    	break;
    case eAxisZ:
         center.z = current.z;
    	break;
    default:
        break;
    }    
    
    if (numOfCoeff == 10)
    {
          
        Dlz2_Gradient(NodeCoeff, x, y, z, TempGradient);
    }
    else
    {      
        Dlz4_Gradient(NodeCoeff, x, y, z, TempGradient);
    }
  
    F0.Set(TempGradient[0], TempGradient[1], TempGradient[2]);

    F1    = ShortestPath(current,center);

    Point field = MagneticField(F0,F1,k0,k1);
    field.Normalize();
    int cpt = 0;
 

    // The 10000 limit is to prevent a point from never crossing the parametric surface
    while(cpt < 10000)
    {        
    
        current += (0.01)*field;
        double Fx = DLz2_coeR(coeff,current.x,current.y,current.z,false);
        double FxP= DLz2_coeR(coeff,previous.x,previous.y,previous.z,false); 
        if (Fx < 0 )
        {   
                   
           float range = (-Fx) / ((-Fx) + FxP);
            
            current -=  range *(current - previous);                      
                               
            return current;
        }  
        
        if (numOfCoeff == 10)
        {
            Dlz2_Gradient(NodeCoeff, x, y, z, TempGradient);
        }

        else
        {          
            Dlz4_Gradient(NodeCoeff, x, y, z, TempGradient);
        }

        F0.Set(TempGradient[0], TempGradient[1], TempGradient[2]);
       
        switch(axis) 
        {
        case eAxisX:
             center.x = current.x;
             
                break;
        case eAxisY:
             center.y = current.y;
            
    	    break;
        case eAxisZ:
             center.z = current.z;
             
    	    break;
        default:
            break;
        }
       
        F1    = ShortestPath(current,center);
        
        field = MagneticField(F0,F1,k0,k1); 
        previous = current;
        cpt++;
    }
  
    return current;
 
}

/****************************************************************************
 * C code from the article
 * "An Implicit Surface Polygonizer"

 * by Jules Bloomenthal, jbloom@beauty.gmu.edu
 * in "Graphics Gems IV", Academic Press, 1994
 */

/* implicit.cpp
 *     an implicit surface polygonizer, translated from Mesa
 *     applications should call polygonize()
 *
 * Authored by Jules Bloomenthal, Xerox PARC.
 * Copyright (c) Xerox Corporation, 1991.  All rights reserved.
 * Permission is granted to reproduce, use and distribute this code for
 * any and all purposes, provided that this notice appears in all copies.
 ****************************************************************************/


bool finish=false;
double boundingbox[3];

int tri[MAX_NUMB_TRI]; /* global pour blaxxun*/

int dual = 0;

static char * desa[500000];
static int    notdesa[500000];

long int cpt = 0;

static int numberOfCoefficient = 0;

static INTLISTS *cubetable[256]; // mis ici pour liberation memoire


// This function return 0 outside, 1 on the surface and 2 inside the volume define by an implicit quadric
// surface
int DLz2_coe (const double c[10], double x, double y, double z)
{
	double value = (	
	x*(x*c[0] + y*c[1] + z*c[3] + c[6]) +
	y*(         y*c[2] + z*c[4] + c[7]) +
	z*(                  z*c[5] + c[8]) +
	  (                           c[9]));

    // This will be change for a [ eps, -eps] interval when vectorising the skin
    if (value > 0)
    {
        return 2; 
    }
    if (value < 0)
    {
        return 0;
    }
    else
    {
        return 1;
    }	  
}

// This function return 0 outside, 1 on the surface and 2 inside the volume define by an implicit quartic
// surface
int DLz4_coe (const double c[K_Max_Coefficient], double x, double y, double z)
{
	double value = ( x*x*x*( c[0] *x + c[1] *y + c[5] *z + c[15]) +
		    y*y*y*( c[3] *x + c[4] *y + c[8] *z + c[18]) +
		    z*z*z*( c[12]*x + c[13]*y + c[14]*z + c[24]) +
		    
		    x* (x*(y*(y*c[2] + z*c[6] + c[16]) +
			   z*(         z*c[9] + c[19]) +
					        c[25])   +
			y*( y*( z*c[7] + c[17]) +
			    z*( z*c[10]+ c[20]) +
			    c[26])			 +
			z*( z*c[22] + c[28])		 +
			c[31])					 +
		    y* (y* ( z*( z*c[11] + c[21]) + c[27]) +
		        z* ( z*c[23] + c[29])		   +
		       c[32])					 +
		    z* (z*c[30] + c[33])			 +
		    c[34]);

    // This will be change for a [ eps, -eps] interval when vectorising the skin
    if (value > 0)
    {
        return 2; 
    }
    if (value < 0)
    {
        return 0;
    }
    else
    {
        return 1;
    }	  
}


#ifdef SGIGFX /**************************************************************/

#include "gl.h"

/* triangle: called by polygonize() for each triangle; set SGI lines */
int triangle (int i1, int i2, int i3, VERTICES vertices)
{
    float v[3];
    int i, ids[3];
    ids[0] = i1;
    ids[1] = i2;
    ids[2] = i3;
    bgnclosedline();
    for (i = 0; i < 3; i++) {
	POINT_J *p = &vertices.ptr[ids[i]].position;
	v[0] = p->x; v[1] = p->y; v[2] = p->z;
	v3f(v);
    }
    endclosedline();
    return 1;
}

#else /***********************************************************************/

/* triangle: called by polygonize() for each triangle; write to stdout */
int triangle (int i1, int i2, int i3, VERTICES vertices)
{
	int j=0;
  /* alain: sert  quoi?  
int ids[3];
    ids[0] = i1;
    ids[1] = i2;
    ids[2] = i3;
    for (int i = 0; i < 3; i++) {
	POINT_J *p = &vertices.ptr[ids[i]].position;
	v[0] = p->x; v[1] = p->y; v[2] = p->z;
	}
	*/
    
	//store the vertice information
    gvertices = vertices;
	j=gntris*3;
    gntris++;
		// alain: test against MAXNBTRIS
		if( j > (MAX_NUMB_TRI  ) ) {
			fprintf(stderr, "> MAXNBTRIS \n");
			exit(0);
		}
 
	tri[j++]=i1; tri[j++]=i2; tri[j++]=i3;

    return 1;

}

												
Shapejb* Shapejb::solidshape(Volume_Tree_Node &The_Node,	Vec3f &start)							
{
	gvertices.count=0;
	gvertices.ptr=NULL;

	gntris = 0;    
	boundingbox[0]=b_box[0];
	boundingbox[1]=b_box[1];
	boundingbox[2]=b_box[2];
	char * test = polygonize(The_Node, size, bounds, start.x,start.y,start.z, triangle, NOTET);
    if (test)
    {
        return NULL;
    }
    cpt = 0;
    nbtris=gntris;	
	int nbv = nbtris*3; // toujours des triangles
    //memcpy(trivert, tri, MAX_NUMB_TRI*sizeof(int));
	if( nbv > MAX_NUMB_TRI) nbv = MAX_NUMB_TRI;
	memcpy(trivert, tri, nbv*sizeof(int));

    svertices=gvertices;
	return this;
}



#endif /**********************************************************************/


/**** An Implicit Surface Polygonizer ****/


/* polygonize: polygonize the implicit surface function
 *   arguments are:
 *	 double function (x, y, z)
 *		 double x, y, z (an arbitrary 3D POINT_J)
 *	     the implicit surface function
 *	     return negative for inside, positive for outside
 *	 double size
 *	     width of the partitioning cube
 *	 int bounds
 *	     max. range of cubes (+/- on the three axes) from first cube
 *	 double x, y, z
 *	     coordinates of a starting POINT_J on or near the surface
 *	     may be defaulted to 0., 0., 0.
 *	 int triproc (i1, i2, i3, vertices)
 *		 int i1, i2, i3 (indices into the vertex array)
 *		 VERTICES vertices (the vertex array, indexed from 0)
 *	     called for each triangle
 *	     the triangle coordinates are (for i = i1, i2, i3):
 *		 vertices.ptr[i].position.x, .y, and .z
 *	     vertices are ccw when viewed from the out (positive) side
 *		 in a left-handed coordinate system
 *	     vertex normals POINT_J outwards
 *	     return 1 to continue, 0 to abort
 *	 int mode
 *	     TET: decompose cube and polygonize six tetrahedra
 *	     NOTET: polygonize cube directly
 *   returns error or NULL
 */
char *polygonize (Volume_Tree_Node &The_Node/*//**EP**(*function)(double[],double,double,double, int))*/,
				  double size,
				  int bounds[],
				  float &x,
				  float &y,
				  float &z,
				  int ((*triproc)(int,int,int,VERTICES)),
				  int mode)
{
    PROCESS p;
    int n, noabort;
    CORNER *setcorner(PROCESS *, int, int, int);
    TEST in, out, find(int, PROCESS*, double , double , double);
	const int length=2*(2*bounds[0])*(2*bounds[1])*(2*bounds[2]);

	p.VTNode = &The_Node;
    //**EP**
	p.triproc = triproc;
    p.size = size;
    for(int i=0; i<3;i++)p.bounds[i] = bounds[i];
    p.delta = size/(double)(RES*RES);
	
    /* allocate hash tables and build cube polygon table: */
    p.centers = (CENTERLIST **) mycalloc(HASHSIZE,sizeof(CENTERLIST *));
    p.corners = (CORNERLIST **) mycalloc(HASHSIZE,sizeof(CORNERLIST *));
    p.edges =	(EDGELIST   **) mycalloc(2*HASHSIZE,sizeof(EDGELIST *));    
    makecubetable();

    /* find POINT_J on surface, beginning search at (x, y, z): */
    srand(1);
    in = find(1, &p, x, y, z);
    out = find(0, &p, x, y, z);
    if (!in.ok || !out.ok) return "can't find starting POINT_J";
    converge(&in.p, &out.p, in.value, The_Node, &p.start,p.coe, p.dual);
	x=p.start.x; y=p.start.y; z=p.start.z;
	
    /* push initial cube on stack: */
    p.cubes = (CUBES *) mycalloc(1, sizeof(CUBES)); /* list of 1 */
    p.cubes->cube.i = p.cubes->cube.j = p.cubes->cube.k = 0;
    p.cubes->next = NULL;

    /* set corners of initial cube: */
    for (n = 0; n < 8; n++)
		p.cubes->cube.corners[n] = setcorner(&p, BIT(n,2), BIT(n,1), BIT(n,0));

    p.vertices.count = p.vertices.max = 0; /* no vertices yet */
    p.vertices.ptr = NULL;

    setcenter(p.centers, 0, 0, 0);

    while (p.cubes != NULL) { /* process active cubes till none left */
	CUBE c;
	CUBES *temp = p.cubes;
	c = p.cubes->cube;
	noabort = mode == TET?
   /* either decompose into tetrahedra and polygonize: */
    dotet(&c, LBN, LTN, RBN, LBF, &p) &&
    dotet(&c, RTN, LTN, LBF, RBN, &p) &&
    dotet(&c, RTN, LTN, LTF, LBF, &p) &&
    dotet(&c, RTN, RBN, LBF, RBF, &p) &&
    dotet(&c, RTN, LBF, LTF, RBF, &p) &&
    dotet(&c, RTN, LTF, RTF, RBF, &p)
   :
   /* or polygonize the cube directly: */
    docube(&c, &p);
	if (! noabort) return "aborted";

	/* pop current cube from stack */
	p.cubes = p.cubes->next;
	
	/* test six face directions, maybe add to stack: */
	finish=false;
	int ti = c.i;
    int tj = c.j;
    int tk = c.k;

	testface(ti -1, tj, tk, &c, L, LBN, LBF, LTN, LTF, &p);
	testface(ti +1, tj, tk, &c, R, RBN, RBF, RTN, RTF, &p);
	testface(ti, tj-1, tk, &c, B, LBN, LBF, RBN, RBF, &p);
	testface(ti, tj+1, tk, &c, T, LTN, LTF, RTN, RTF, &p);
	testface(ti, tj, tk-1, &c, N, LBN, LTN, RBN, RTN, &p);
	testface(ti, tj, tk+1, &c, F, LBF, LTF, RBF, RTF, &p);
    
    }

    //Removed and destroyed .. 
    //free((char *) p.cubes);

    for (int ind = 0 ; ind < cpt ; ind++)
    {       
        if (notdesa[ind] != 1)
        {
            delete(desa[ind]);
            desa[ind] = NULL;
        }
    }
    cpt = 0;
    for (int j = 0; j < 255; j++)
    {          
        cubetable[j] = NULL;
    }

    return NULL;
}


/* testface: given cube at lattice (i, j, k), and four corners of face,
 * if surface crosses face, compute other four corners of adjacent cube
 * and add new_j cube to cube stack */
void testface (int i, int j, int k, CUBE *old, int face, int c1, int c2, int c3, int c4, PROCESS *p)
{
	//chloe
	bool empty=false;
	CUBE new_j;
    CUBES *oldcubes = p->cubes;
    CORNER *setcorner(PROCESS *p, int i, int j, int k);
    static int facebit[6] = {2, 2, 1, 1, 0, 0};
	int n, pos = old->corners[c1]->value > 0.0 ? 1 : 0, bit = facebit[face];

    /* test if no surface crossing, cube out of bounds, or already visited: */
	//if(finish)return;
    if ((old->corners[c2]->value > 0) == pos &&
	(old->corners[c3]->value > 0) == pos &&
	(old->corners[c4]->value > 0) == pos) return;
    if (abs(i) > p->bounds[0] || abs(j) > p->bounds[1] || abs(k) > p->bounds[2])
		return;
	if (setcenter(p->centers, i, j, k)) return;

    /* create new_j cube: */
    new_j.i = i;
    new_j.j = j;
    new_j.k = k;
    for (n = 0; n < 8; n++) new_j.corners[n] = NULL;
    new_j.corners[FLIP(c1, bit)] = old->corners[c1];
    new_j.corners[FLIP(c2, bit)] = old->corners[c2];
    new_j.corners[FLIP(c3, bit)] = old->corners[c3];
    new_j.corners[FLIP(c4, bit)] = old->corners[c4];
    for (n = 0; n < 8; n++)
	if (new_j.corners[n] == NULL)
	    new_j.corners[n] = setcorner(p, i+BIT(n,2), j+BIT(n,1), k+BIT(n,0));

    /*add cube to top of stack: */
    p->cubes = (CUBES *) mycalloc(1, sizeof(CUBES));
    p->cubes->cube = new_j;
    p->cubes->next = oldcubes;
}


/* setcorner: return corner with the given lattice location
   set (and cache) its function value */
CORNER *setcorner (PROCESS *p, int i, int j, int k)
{
    /* for speed, do corner value caching here */
    CORNER *c = (CORNER *) mycalloc(1, sizeof(CORNER));
    int index = HASH(i, j, k);
    CORNERLIST *l = p->corners[index];
	
	c->i = i; c->x = p->start.x + ((double)i - 0.5) * p->size;
    c->j = j; c->y = p->start.y + ((double)j - 0.5) * p->size;
    c->k = k; c->z = p->start.z + ((double)k - 0.5) * p->size;

    for (; l != NULL; l = l->next)
	if (l->i == i && l->j == j && l->k == k) {
	    c->value = l->value;
	    return c;
	    }
    l = (CORNERLIST *) mycalloc(1, sizeof(CORNERLIST));
    l->i = i; l->j = j; l->k = k;
    //**EP** l->value = c->value = p->function(p->coe,c->x, c->y, c->z, p->dual);
	l->value = c->value = (double) p->VTNode->isInside(c->x, c->y, c->z);
    l->next = p->corners[index];
    p->corners[index] = l;
   	  
    return c;
}

TEST find (int sign, PROCESS *p, double x, double y, double z)
{
    int i;
    TEST test;

	//Initialize to the current point
	test.p.x = x;
    test.p.y = y;
    test.p.z = z;

    double range = p->size;
    test.ok = 1;
    for (i = 0; i < 10000; i++) 
	{
		test.value = p->VTNode->isInside(test.p.x, test.p.y, test.p.z);
        if (sign == (test.value > 0.0)) return test;

		test.p.x = x + range * (RAND() - 0.5);
        test.p.y = y + range * (RAND() - 0.5);
        test.p.z = z + range * (RAND() - 0.5);

        range = range * 1.0005; /* slowly expand search outwards */
    }
    test.ok = 0;
    return test;
}

/**** Tetrahedral Polygonization ****/

/* dotet: triangulate the tetrahedron
 * b, c, d should appear clockwise when viewed from a
 * return 0 if client aborts, 1 otherwise */

int dotet (CUBE *cube, int c1, int c2, int c3, int c4, PROCESS *p)
{
    CORNER *a = cube->corners[c1];
    CORNER *b = cube->corners[c2];
    CORNER *c = cube->corners[c3];
    CORNER *d = cube->corners[c4];
    int index = 0, apos, bpos, cpos, dpos, e1, e2, e3, e4, e5, e6;
    if (apos = (a->value > 0.0)) index += 8;
    if (bpos = (b->value > 0.0)) index += 4;
    if (cpos = (c->value > 0.0)) index += 2;
    if (dpos = (d->value > 0.0)) index += 1;
    /* index is now 4-bit number representing one of the 16 possible cases */
    if (apos != bpos) e1 = vertid(a, b, p);
    if (apos != cpos) e2 = vertid(a, c, p);
    if (apos != dpos) e3 = vertid(a, d, p);
    if (bpos != cpos) e4 = vertid(b, c, p);
    if (bpos != dpos) e5 = vertid(b, d, p);
    if (cpos != dpos) e6 = vertid(c, d, p);
    /* 14 productive tetrahedral cases (0000 and 1111 do not yield polygons */
    switch (index) {
	case 1:	 return p->triproc(e5, e6, e3, p->vertices);
	case 2:	 return p->triproc(e2, e6, e4, p->vertices);
	case 3:	 return p->triproc(e3, e5, e4, p->vertices) &&
			p->triproc(e3, e4, e2, p->vertices);
	case 4:	 return p->triproc(e1, e4, e5, p->vertices);
	case 5:	 return p->triproc(e3, e1, e4, p->vertices) &&
			p->triproc(e3, e4, e6, p->vertices);
	case 6:	 return p->triproc(e1, e2, e6, p->vertices) &&
			p->triproc(e1, e6, e5, p->vertices);
	case 7:	 return p->triproc(e1, e2, e3, p->vertices);
	case 8:	 return p->triproc(e1, e3, e2, p->vertices);
	case 9:	 return p->triproc(e1, e5, e6, p->vertices) &&
			p->triproc(e1, e6, e2, p->vertices);
	case 10: return p->triproc(e1, e3, e6, p->vertices) &&
			p->triproc(e1, e6, e4, p->vertices);
	case 11: return p->triproc(e1, e5, e4, p->vertices);
	case 12: return p->triproc(e3, e2, e4, p->vertices) &&
			p->triproc(e3, e4, e5, p->vertices);
	case 13: return p->triproc(e6, e2, e4, p->vertices);
	case 14: return p->triproc(e5, e3, e6, p->vertices);
    }
    return 1;
}


/**** Cubical Polygonization (optional) ****/


#define LB	0  /* left bottom edge	*/
#define LT	1  /* left top edge	*/
#define LN	2  /* left near edge	*/
#define LF	3  /* left far edge	*/
#define RB	4  /* right bottom edge */
#define RT_J	5  /* right top edge	*/
#define RN	6  /* right near edge	*/
#define RF	7  /* right far edge	*/
#define BN	8  /* bottom near edge	*/
#define BF	9  /* bottom far edge	*/
#define TN	10 /* top near edge	*/
#define TF	11 /* top far edge	*/

//static INTLISTS *cubetable[256];

/*			edge: LB, LT, LN, LF, RB, RT_J, RN, RF, BN, BF, TN, TF */
static int corner1[12]	   = {LBN,LTN,LBN,LBF,RBN,RTN,RBN,RBF,LBN,LBF,LTN,LTF};
static int corner2[12]	   = {LBF,LTF,LTN,LTF,RBF,RTF,RTN,RTF,RBN,RBF,RTN,RTF};
static int leftface[12]	   = {B,  L,  L,  F,  R,  T,  N,  R,  N,  B,  T,  F};
			     /* face on left when going corner1 to corner2 */
static int rightface[12]   = {L,  T,  N,  L,  B,  R,  R,  F,  B,  F,  N,  T};
			     /* face on right when going corner1 to corner2 */


/* docube: triangulate the cube directly, without decomposition */

int docube (CUBE *cube, PROCESS *p)
{
    INTLISTS *polys;
    int i, index = 0;
    for (i = 0; i < 8; i++) if (cube->corners[i]->value > 0.0) index += (1<<i);
    for (polys = cubetable[index]; polys; polys = polys->next) {
	INTLIST *edges;
	int a = -1, b = -1, count = 0;
	for (edges = polys->list; edges; edges = edges->next) {
	    CORNER *c1 = cube->corners[corner1[edges->i]];
	    CORNER *c2 = cube->corners[corner2[edges->i]];
	    int c = vertid(c1, c2, p);
	    if (++count > 2 && ! p->triproc(a, b, c, p->vertices)) return 0;
	    if (count < 3) a = b;
	    b = c;
	}
    }
    return 1;
}


/* nextcwedge: return next clockwise edge from given edge around given face */

int nextcwedge (int edge, int face)
{
    switch (edge) {
	case LB: return (face == L)? LF : BN;
	case LT: return (face == L)? LN : TF;
	case LN: return (face == L)? LB : TN;
	case LF: return (face == L)? LT : BF;
	case RB: return (face == R)? RN : BF;
	case RT_J: return (face == R)? RF : TN;
	case RN: return (face == R)? RT_J : BN;
	case RF: return (face == R)? RB : TF;
	case BN: return (face == B)? RB : LN;
	case BF: return (face == B)? LB : RF;
	case TN: return (face == T)? LT : RN;
	case TF: return (face == T)? RT_J : LF;
	default : return 0;
    }
}


/* otherface: return face adjoining edge that is not the given face */

int otherface (int edge, int face)
{
    int other = leftface[edge];
    return face == other? rightface[edge] : other;
}


/* makecubetable: create the 256 entry table for cubical polygonization */

void makecubetable ()
{
    int i, e, c, done[12], pos[8];
    for (i = 0; i < 256; i++) {
	for (e = 0; e < 12; e++) done[e] = 0;
	for (c = 0; c < 8; c++) pos[c] = BIT(i, c);
	for (e = 0; e < 12; e++)
	    if (!done[e] && (pos[corner1[e]] != pos[corner2[e]])) {
		INTLIST *ints = 0;
 		INTLISTS *lists = (INTLISTS *) mycalloc(1, sizeof(INTLISTS));
		int start = e, edge = e;
		/* get face that is to right of edge from pos to neg corner: */
		int face = pos[corner1[e]]? rightface[e] : leftface[e];
		while (1) {
		    edge = nextcwedge(edge, face);
		    done[edge] = 1;
		    if (pos[corner1[edge]] != pos[corner2[edge]]) {
			INTLIST *tmp = ints;
			ints = (INTLIST *) mycalloc(1, sizeof(INTLIST));
			ints->i = edge;
			ints->next = tmp; /* add edge to head of list */
			if (edge == start) break;
			face = otherface(edge, face);
		    }
		}
		lists->list = ints; /* add ints to head of table entry */
		lists->next = cubetable[i];
		cubetable[i] = lists;       
	    }
    }
}


/**** Storage ****/


/* calloc: return successful calloc or exit program */

char *mycalloc (int nitems, int nbytes)
{        
     char *pt;
     char *ptr = (char*) calloc(nitems,nbytes);
	 pt = ptr;
     desa[cpt] = ptr;
     cpt++;
	 for ( int i = 0 ; i< nbytes;i++)*pt++ = 0;
 
     if (ptr != NULL) return ptr;
     fprintf(stderr, "can't calloc %d bytes\n", nitems*nbytes);
     exit(1);
}


/* setcenter: set (i,j,k) entry of table[]
 * return 1 if already set; otherwise, set and return 0 */

int setcenter(CENTERLIST *table[], int i, int j, int k)
{
    int index = HASH(i, j, k);
    CENTERLIST *new_j, *l, *q = table[index];
    for (l = q; l != NULL; l = l->next)
	if (l->i == i && l->j == j && l->k == k) return 1;
    new_j = (CENTERLIST *) mycalloc(1, sizeof(CENTERLIST));
    new_j->i = i; new_j->j = j; new_j->k = k; new_j->next = q;
    table[index] = new_j;
    return 0;
}


/* setedge: set vertex id for edge */

void setedge (EDGELIST *table[], int i1, int j1, int k1, int i2, int j2, int k2, int vid)
{
    unsigned int index;
    EDGELIST *new_j;
    if (i1>i2 || (i1==i2 && (j1>j2 || (j1==j2 && k1>k2)))) {
	int t=i1; i1=i2; i2=t; t=j1; j1=j2; j2=t; t=k1; k1=k2; k2=t;
    }
    index = HASH(i1, j1, k1) + HASH(i2, j2, k2);
    new_j = (EDGELIST *) mycalloc(1, sizeof(EDGELIST));
    new_j->i1 = i1; new_j->j1 = j1; new_j->k1 = k1;
    new_j->i2 = i2; new_j->j2 = j2; new_j->k2 = k2;
    new_j->vid = vid;
    new_j->next = table[index];
    table[index] = new_j;
}


/* getedge: return vertex id for edge; return -1 if not set */

int getedge (EDGELIST *table[], int i1, int j1, int k1, int i2, int j2, int k2)
{
    EDGELIST *q;
    if (i1>i2 || (i1==i2 && (j1>j2 || (j1==j2 && k1>k2)))) {
	int t=i1; i1=i2; i2=t; t=j1; j1=j2; j2=t; t=k1; k1=k2; k2=t;
    };
    q = table[HASH(i1, j1, k1)+HASH(i2, j2, k2)];
    for (; q != NULL; q = q->next)
	if (q->i1 == i1 && q->j1 == j1 && q->k1 == k1 &&
	    q->i2 == i2 && q->j2 == j2 && q->k2 == k2)
	    return q->vid;
    return -1;
}


/**** Vertices ****/


/* vertid: return index for vertex on edge:
 * c1->value and c2->value are presumed of different sign
 * return saved index if any; else compute vertex and save */

int vertid (CORNER *c1, CORNER *c2, PROCESS *p)
{
    VERTEX v;
    POINT_J a, b;
    int vid = getedge(p->edges, c1->i, c1->j, c1->k, c2->i, c2->j, c2->k);
    if (vid != -1) return vid;			     /* previously computed */
    a.x = c1->x; a.y = c1->y; a.z = c1->z;
    b.x = c2->x; b.y = c2->y; b.z = c2->z;
	//**EP** converge(&a, &b, c1->value, p->function, &v.position,p->coe, p->dual); /* position */
    converge(&a, &b, c1->value, *(p->VTNode), &v.position,p->coe, p->dual); /* position */
    //vnormal(&v.position, p, &v.normal);			   /* normal */
    addtovertices(&p->vertices, v);			   /* save vertex */
    vid = p->vertices.count-1;
    setedge(p->edges, c1->i, c1->j, c1->k, c2->i, c2->j, c2->k, vid);
    return vid;
}


/* addtovertices: add v to sequence of vertices */

void addtovertices (VERTICES *vertices, VERTEX v)
{
    if (vertices->count == vertices->max) {
	int i;
	VERTEX *new_j;
	vertices->max = vertices->count == 0 ? 10 : 2*vertices->count;
    int k = cpt;
	new_j = (VERTEX *) mycalloc((unsigned) vertices->max, sizeof(VERTEX));
    for (int f = k; f < cpt; f++)
    {
        notdesa[f] = 1;
    }

	for (i = 0; i < vertices->count; i++) new_j[i] = vertices->ptr[i];
	if (vertices->ptr != NULL) free((char *) vertices->ptr);
	vertices->ptr = new_j;
    }
    vertices->ptr[vertices->count++] = v;
    
}

/* vnormal: compute unit length surface normal at POINT_J */

void vnormal (POINT_J *pts, PROCESS *p, POINT_J *v)
{
    //We do nothing here because the normals will be calculated later... (SL)
    //double f = (double) p->VTNode->isInside(pts->x, pts->y, pts->z);
    //v->x = (double)p->VTNode->isInside(pts->x+p->delta, pts->y, pts->z)-f;
    //v->y = (double) p->VTNode->isInside(pts->x, pts->y+p->delta, pts->z)-f;
    //v->z = (double) p->VTNode->isInside(pts->x, pts->y, pts->z+p->delta)-f;
    //f = sqrt(v->x*v->x + v->y*v->y + v->z*v->z);
    //if (f != 0.0) {v->x /= f; v->y /= f; v->z /= f;}
}

/* converge: from two POINT_Js of differing sign, converge to zero crossing */

void converge (POINT_J *p1,
			   POINT_J *p2,
			   double v,
			   const Volume_Tree_Node &The_Node/*//**EP**(double[],double,double,double, int))*/,
			   POINT_J *p,
			   double c[K_Max_Coefficient],
			   int dual)
{
    int i = 0;
    POINT_J pos, neg;
    if (v == 0) {
        pos.x = p2->x; pos.y = p2->y; pos.z = p2->z;
        neg.x = p1->x; neg.y = p1->y; neg.z = p1->z;

    }
    else {
        pos.x = p1->x; pos.y = p1->y; pos.z = p1->z;
        neg.x = p2->x; neg.y = p2->y; neg.z = p2->z;
    }
	
    while (1) {
		double tempx = 0.5*(pos.x + neg.x);
		double tempy = 0.5*(pos.y + neg.y);
		double tempz = 0.5*(pos.z + neg.z);
        
        if ((tempx == p->x &&
			tempy == p->y &&
			tempz == p->z && i > 0) || i > 100
			/*i++ == RES*/)
        {
			//to return only inside point
			p->x = pos.x;
			p->y = pos.y;
			p->z = pos.z;
            return;
            
        }
		i++;
		p->x = tempx;
		p->y = tempy;
		p->z = tempz;
        if ( (The_Node.isInside(p->x, p->y, p->z)))
             {pos.x = p->x; pos.y = p->y; pos.z = p->z;}
        else {neg.x = p->x; neg.y = p->y; neg.z = p->z;}
    }
}

Shapejb::Shapejb(int bo[3],double s,double bbox[3])
{		
		for(int j=0; j<3; j++)bounds[j]=bo[j];
		for(int k=0; k<3; k++)b_box[k]=bbox[k];
		size=s;
		nbtris=0;
        svertices.ptr = NULL;
}









