#ifndef _POLYGONISE_H_
#define _POLYGONISE_H_

#include "DensityTLS.h"
#include "sl_matrix.h"
#include <ztl.h>

#include "core/NodeFld.h"



class Volume_Tree_Node;
class DensityMatrix;
class SolidGraphic;
class Point;

const  int      nbvertbyface       = 3;

//Enum for axis direction
/************************

	2001/12/06  
	Eric Baillargeon

*************************/

enum eAxis{
    
    eAxisX = 0,
    eAxisY,
    eAxisZ,
    eAxis_NumberOf,
	eAxis_PAD = 0xffff
};

//Enum for negative or positive direction
/************************

	2001/12/06  
	Eric Baillargeon

*************************/
enum eDirection{

    eDirection_Pos = 0,
    eDirection_Neg,
    eDirection_NumberOf,
	eDirectionPAD = 0xffff
};

//Enum for Type of mapping
/************************

	2001/12/06  
	Eric Baillargeon

*************************/
enum eMapping{

    eMapping_Spheric = 0,
    eMapping_Box,
    eMapping_Cylindric,
    eMapping_Gradient,
    eMapping_NumberOf,
	eMapping_PAD = 0xffff,
};

/*********************************************************************
 * C code from the article
 * "An Implicit Surface Polygonizer"
 * by Jules Bloomenthal, jbloom@beauty.gmu.edu
 * in "Graphics Gems IV", Academic Press, 1994
 */

/* implicit.c
 *     an implicit surface polygonizer, translated from Mesa
 *     applications should call polygonize()
 *
 * to compile a test program for ASCII output:
 *     cc implicit.c -o implicit -lm
 *
 * to compile a test program for display on an SGI workstation:
 *     cc -DSGIGFX implicit.c -o implicit -lgl_s -lm
 *
 * Authored by Jules Bloomenthal, Xerox PARC.
 * Copyright (c) Xerox Corporation, 1991.  All rights reserved.
 * Permission is granted to reproduce, use and distribute this code for
 * any and all purposes, provided that this notice appears in all copies. 
/*************************************************************************/

/* Alain: added the dualop with variable dual*/

//#ifdef _OGL
// for glRender
//#include "glrender.h"
//#endif

// for	ComputePrimitiveList
//#include "tomesh.h"
//#include "gkatmai.h"

//#include <math.h>
//#include <stdio.h>
//#include <sys/types.h>

#define TET	0  /* use tetrahedral decomposition */
#define NOTET	1  /* no tetrahedral decomposition  */


#define VRML 1
#define MAX_NUMB_TRI 1000000 // 500000

#define RES	100 /* # converge iterations    */

const int  L=	0 ; /* left direction:	-x, -i */
const int  R=	1 ; /* right direction:	+x, +i */
const int  B=	2 ; /* bottom direction: -y, -j */
const int  T=	3 ; /* top direction:	+y, +j */
const int  N=	4 ; /* near direction:	-z, -k */
const int  F=	5  ;/* far direction:	+z, +k */

const int  LBN=	0  ;/* left bottom near corner  */
const int  LBF=	1 ; /* left bottom far corner   */
const int  LTN=	2 ; /* left top near corner     */
const int  LTF= 3 ; /* left top far corner      */
const int  RBN=	4 ; /* right bottom near corner */
const int  RBF=	5 ; /* right bottom far corner  */
const int  RTN=	6 ; /* right top near corner    */
const int  RTF=	7  ;/* right top far corner     */

/* the LBN corner of cube (i, j, k), corresponds with location
 * (start.x+(i-.5)*size, start.y+(j-.5)*size, start.z+(k-.5)*size) */

#define RAND()	    ((rand()&32767)/32767.)    /* random number between 0 and 1 */
#define HASHBIT	    (5)
#define HASHSIZE    (size_t)(1<<(3*HASHBIT))   /* hash table size (32768) */
#define MASK	    ((1<<HASHBIT)-1)
#define HASH(i,j,k) ((((((i)&MASK)<<HASHBIT)|((j)&MASK))<<HASHBIT)|((k)&MASK))
#define BIT(i, bit) (((i)>>(bit))&1)
#define FLIP(i,bit) ((i)^1<<(bit)) /* flip the given bit of i */

typedef struct point_j {		   /* a three-dimensional POINT_J */
    double x, y, z;		   /* its coordinates */
} POINT_J;

typedef struct test {		   /* test the function for a signed value */
    POINT_J p;			   /* location of test */
    double value;		   /* function value at p */
    int ok;			   /* if value is of correct sign */
} TEST;

typedef struct vertex {		   /* surface vertex */
    POINT_J position, normal;	   /* position and surface normal */
} VERTEX;

typedef struct vertices {	   /* list of vertices in polygonization */
    int count, max;		   /* # vertices, max # allowed */
    VERTEX *ptr;		   /* dynamically allocated */
} VERTICES;

typedef struct corner {		   /* corner of a cube */
    int i, j, k;		   /* (i, j, k) is index within lattice */
    double x, y, z, value;	   /* location and function value */
} CORNER;

typedef struct cube {		   /* partitioning cell (cube) */
    int i, j, k;		   /* lattice location of cube */
    CORNER *corners[8];		   /* eight corners */
} CUBE;

typedef struct cubes {		   /* linked list of cubes acting as stack */
    CUBE cube;			   /* a single cube */
    struct cubes *next;		   /* remaining elements */
} CUBES;

typedef struct centerlist {	   /* list of cube locations */
    int i, j, k;		   /* cube location */
    struct centerlist *next;	   /* remaining elements */
} CENTERLIST;

typedef struct cornerlist {	   /* list of corners */
    int i, j, k;		   /* corner id */
    double value;		   /* corner value */
    struct cornerlist *next;	   /* remaining elements */
} CORNERLIST;

typedef struct edgelist {	   /* list of edges */
    int i1, j1, k1, i2, j2, k2;	   /* edge corner ids */
    int vid;			   /* vertex id */
    struct edgelist *next;	   /* remaining elements */
} EDGELIST;

typedef struct intlist {	   /* list of integers */
    int i;			   /* an integer */
    struct intlist *next;	   /* remaining elements */
} INTLIST;

typedef struct intlists {	   /* list of list of integers */
    INTLIST *list;		   /* a list of integers */
    struct intlists *next;	   /* remaining elements */
} INTLISTS;

// Max coefficient : 35 for now (quartic)
const int K_Max_Coefficient = 35;

typedef struct process {	   /* parameters, function, storage */
    int ((*triproc)(int,int,int,VERTICES));		   /* triangle output function */
    double size, delta;		   /* cube size, normal delta */
    int bounds[3];	/* cube range within lattice */
	//rajoutee 8/3/01
	double coe[K_Max_Coefficient];
	int dual;
    POINT_J start;		   /* start POINT_J_J on surface */
    CUBES *cubes;		   /* active cubes */
    VERTICES vertices;		   /* surface vertices */
    CENTERLIST **centers;	   /* cube center hash table */
    CORNERLIST **corners;	   /* corner value hash table */
    EDGELIST **edges;		   /* edge and vertex id hash table */
	//**EP**
	Volume_Tree_Node *VTNode;
} PROCESS;

void *calloc();

//class added for polygonal data storage
class Shapejb {

protected:	
	VERTICES svertices;	
	
public:
    int nbtris;
    int trivert[MAX_NUMB_TRI];
	double coe[K_Max_Coefficient];
	double size;
	int bounds[3];
	double b_box[3];
	int dual;
	Shapejb(int bo[3],double s,double bbox[3]);

	Shapejb()
    {
		nbtris=0;		
		dual = 0;
	}

	~Shapejb()
    {
        if (svertices.ptr) 
            delete [] svertices.ptr;
    }

	int Nbtris()
    {
		return nbtris;
	}

	VERTICES* Svertices ()
    {
		return &svertices;
	}
	
    int Trivert(int i)
    {
		return trivert[i];
	}

	Shapejb *solidshape(Volume_Tree_Node&, Vec3f&);
};

#ifdef SGIGFX /**************************************************************/

#include "gl.h"

/* triangle: called by polygonize() for each triangle; set SGI lines */
int triangle (int i1, int i2, int i3, VERTICES vertices);


#else /***********************************************************************/

static int gntris;	     /* global needed by application */
static VERTICES gvertices;  /* global needed by application */


/* triangle: called by polygonize() for each triangle; write to stdout */
int triangle (int i1, int i2, int i3, VERTICES vertices);

#endif /**********************************************************************/


/**** An Implicit Surface Polygonizer ****/


/* polygonize: polygonize the implicit surface function
 *   arguments are:
 *	 double function (x, y, z)
 *		 double x, y, z (an arbitrary 3D POINT_J)
 *	     the implicit surface function
 *	     return negative for inside, positive for outside
 *	 double size
 *	     width of the partitioning cube
 *	 int bounds
 *	     max. range of cubes (+/- on the three axes) from first cube
 *	 double x, y, z
 *	     coordinates of a starting POINT_J on or near the surface
 *	     may be defaulted to 0., 0., 0.
 *	 int triproc (i1, i2, i3, vertices)
 *		 int i1, i2, i3 (indices into the vertex array)
 *		 VERTICES vertices (the vertex array, indexed from 0)
 *	     called for each triangle
 *	     the triangle coordinates are (for i = i1, i2, i3):
 *		 vertices.ptr[i].position.x, .y, and .z
 *	     vertices are ccw when viewed from the out (positive) side
 *		 in a left-handed coordinate system
 *	     vertex normals POINT_J outwards
 *	     return 1 to continue, 0 to abort
 *	 int mode
 *	     TET: decompose cube and polygonize six tetrahedra
 *	     NOTET: polygonize cube directly
 *   returns error or NULL
 */
char *polygonize (Volume_Tree_Node&/*//**EP**(double[],double,double,double, int))*/,
				  double size,
				  int bounds[3],
				  float &x,
				  float &y,
				  float &z,
				  int ((*triproc)(int,int,int,VERTICES)),
				  int mode);


/* testface: given cube at lattice (i, j, k), and four corners of face,
 * if surface crosses face, compute other four corners of adjacent cube
 * and add new_j cube to cube stack */
void testface (int i, int j, int k, CUBE *old, int face, int c1, int c2, int c3, int c4, PROCESS *p);
/* setcorner: return corner with the given lattice location
   set (and cache) its function value */
CORNER *setcorner (PROCESS *p, int i, int j, int k);

/* find: search for POINT_J with value of given sign (0: neg, 1: pos) */
TEST find (int sign, PROCESS *p, double x, double y, double z);



/*** Tetrahedral Polygonization ****/
/* dotet: triangulate the tetrahedron
 * b, c, d should appear clockwise when viewed from a
 * return 0 if client aborts, 1 otherwise */
int dotet (CUBE *cube, int c1, int c2, int c3, int c4, PROCESS *p);


/**** Cubical Polygonization (optional) ****/
#define LB	0  /* left bottom edge	*/
#define LT	1  /* left top edge	*/
#define LN	2  /* left near edge	*/
#define LF	3  /* left far edge	*/
#define RB	4  /* right bottom edge */
#define RT_J	5  /* right top edge	*/
#define RN	6  /* right near edge	*/
#define RF	7  /* right far edge	*/
#define BN	8  /* bottom near edge	*/
#define BF	9  /* bottom far edge	*/
#define TN	10 /* top near edge	*/
#define TF	11 /* top far edge	*/

/* docube: triangulate the cube directly, without decomposition */
int docube (CUBE *cube, PROCESS *p);

/* nextcwedge: return next clockwise edge from given edge around given face */
int nextcwedge (int edge, int face);

/* otherface: return face adjoining edge that is not the given face */
int otherface (int edge, int face);

/* makecubetable: create the 256 entry table for cubical polygonization */
void makecubetable ();


/**** Storage ****/


/* mycalloc: return successful calloc or exit program */
char *mycalloc (int nitems, int nbytes);

/* setcenter: set (i,j,k) entry of table[]
 * return 1 if already set; otherwise, set and return 0 */
int setcenter(CENTERLIST *table[], int i, int j, int k);

/* setedge: set vertex id for edge */
void setedge (EDGELIST *table[], int i1, int j1, int k1, int i2, int j2, int k2, int vid);

/* getedge: return vertex id for edge; return -1 if not set */
int getedge (EDGELIST *table[], int i1, int j1, int k1, int i2, int j2, int k2);

/**** Vertices ****/


/* vertid: return index for vertex on edge:
 * c1->value and c2->value are presumed of different sign
 * return saved index if any; else computebox vertex and save */
int vertid (CORNER *c1, CORNER *c2, PROCESS *p);

/* addtovertices: add v to sequence of vertices */
void addtovertices (VERTICES *vertices, VERTEX v);

/* vnormal: compute unit length surface normal at POINT_J */
void vnormal (POINT_J *pts, PROCESS *p, POINT_J *v);

/* converge: from two POINT_Js of differing sign, converge to zero crossing */
void converge (POINT_J *p1,
			   POINT_J *p2,
			   double v,
			   const Volume_Tree_Node &The_Node/*//**EP**(double[],double,double,double, int))*/,
			   POINT_J *p,
			   double c[K_Max_Coefficient],
			   int dual);

//////////////////////////////////////////////////////////////////////////////////////
//***************** Solid for MPEG4 ************************************************
/////////////////////////////////////////////////////////////////////////////////////

//Compute the SolidGraphic associated to the volumic tree
SolidGraphic *NewDLzK_coe(DensityMatrix& densMatrix, 
                     float grid,
                     Volume_Tree_Node& The_Node,
                     SFString TextureURL,
					 SFString* TypeOfMapping,
                     const SL_Matrix& TransformMatrix,
                     DensityArray UserDensArray = DensityArray(),
                     bool Normals = false,
                     bool manage_color = false);

//Compute the normal and the color at each point on the surface
void    ComputeNormalsAndColors(Volume_Tree_Node *TheNode,
                                VERTICES *Verts, 
                                ZArray<Vec3f> *ColorTable = 0,
                                bool GetNormals = true);

//Get density of one point from a quadric surface
int DLz2_coe (const double c[], double x, double y, double z);

//Get density of one point from a quartic surface
int DLz4_coe (const double c[], double x, double y, double z);

////////////////////////////////
//Inline functions
////////////////////////////////

//return the field value of an hyperplane at a given point
inline double DLz1_coeR (const double c[], double x, double y, double z, int isoValue = 0)
{
    return (x*c[0] + y*c[1] + z*c[2] + c[3]) - isoValue;
}

//return the field value of a quadric surface at a given point
inline double DLz2_coeR (const double c[], double x, double y, double z, int isoValue = 0)
{
	return  (	
	x*(x*c[0] + y*c[1] + z*c[3] + c[6]) +
			y*(         y*c[2] + z*c[4] + c[7]) +
			z*(                  z*c[5] + c[8]) +
	  (                           c[9])) - isoValue;
}

//return the field value of a quartic surface at a given point
inline double DLz4_coeR (const double c[], double x, double y, double z, int isoValue = 0)
{
	return ( x*x*x*( c[0] *x + c[1] *y + c[5] *z + c[15]) +
		    y*y*y*( c[3] *x + c[4] *y + c[8] *z + c[18]) +
		    z*z*z*( c[12]*x + c[13]*y + c[14]*z + c[24]) +
		    
		    x* (x*(y*(y*c[2] + z*c[6] + c[16]) +
			   z*(         z*c[9] + c[19]) +
					        c[25])   +
			y*( y*( z*c[7] + c[17]) +
			    z*( z*c[10]+ c[20]) +
			    c[26])			 +
			z*( z*c[22] + c[28])		 +
			c[31])					 +
		    y* (y* ( z*( z*c[11] + c[21]) + c[27]) +
		        z* ( z*c[23] + c[29])		   +
		       c[32])					 +
		    z* (z*c[30] + c[33])			 +
		    c[34]) - isoValue;
}

//Compute the gradient of a quadric surface at a given point
inline void Dlz2_Gradient(const double *coeff, double x , double y, double z, double *OutGradient)
{
    // derive partielle par rapport  x
    OutGradient[0] = -(2*coeff[0]*x + coeff[1]*y + coeff[3]*z + coeff[6]); 
           
    // derive partielle par rapport  y
    OutGradient[1] = -(coeff[1] * x + 2 * coeff[2] * y + coeff[4] * z + coeff[7]);
        
    // derive partielle par rapport  z
    OutGradient[2] = -(coeff[3] * x + coeff[4]*y + 2 * coeff[5]* z + coeff[8]);
}

//Compute the gradient of a quartic surface at a given point
inline void Dlz4_Gradient(const double *coeff, double x , double y, double z, double *OutGradient)
{   
    // derive partielle par rapport  x
    OutGradient[0] = -(   x* (2*coeff[25]   + x* (3*coeff[15] + 4*coeff[0] *x)) +
                          y* (  coeff[26]   + y* (  coeff[17] +   coeff[3] *y)) +
                          z* (  coeff[28]   + z* (  coeff[22] +   coeff[12]*z)) +

                     x*y* (3*coeff[1]*x + 2*(coeff[16] + coeff[2]*y + coeff[6]*z)) +
                     x*z* (3* coeff[5]*x + 2*(coeff[19] + coeff[9]*z))             +
                     y*z* (  coeff[20]   +    coeff[7]*y +   coeff[10] * z)        +
                     coeff[31]);
           
    // derive partielle par rapport  y
    OutGradient[1] = -(   x* (  coeff[26]   + x* (  coeff[16] +   coeff[1] *x)) +
                        y* (2*coeff[27]   + y* (3*coeff[18] + 4*coeff[4] *y)) +
                        z* (  coeff[29]   + z* (  coeff[23] +   coeff[13]*z)) +

                    x*y* (3*coeff[3]*y + 2*(coeff[17]   + coeff[2]*x + coeff[7]*z)) +
                    x*z* (  coeff[20]   +    coeff[6]*x +   coeff[10]  * z)         +
                    y*z* (3* coeff[8]*y + 2*(coeff[21]   + coeff[11]*z))            +
                    coeff[32]);

    // derive partielle par rapport  z
    OutGradient[2] =  -(  x* (  coeff[28]   + x* (  coeff[19] +   coeff[5] *x)) +
                        y* (  coeff[29]   + y* (  coeff[21] +   coeff[8] *y)) +
                        z* (2*coeff[30]   + z* (  coeff[24] + 4*coeff[14]*z)) +

                    x*y* (  coeff[20]   +    coeff[6]*x +     coeff[7]*y + 2*coeff[10]*z) +
                    x*z*2* (coeff[22]   + coeff[9]*x +   coeff[12]*z)                     +
                    y*z* (2*(coeff[23]  + coeff[11]*y) +  3*coeff[13]*z)                +
                    coeff[33]);
}

///////////////////////////////////////////////////
// Texturing function
///////////////////////////////////////////////////

//Compute the magnetic field of two forces
Point    MagneticField(Point F0, Point F1, float k0, float k1);

//Find the shortest path between two points
Point    ShortestPath(Point p, Point center);

//Compute the texture coordinates
Point    GetTextureCoordinate(Volume_Tree_Node& The_Node,
                              double x, 
                              double y, 
                              double z, 
                              double coeff[10],
                              eAxis axis);

//Construct a cylinder that fits the bounds
void     ConstructCylinder(double Bounds[6], double coeff[10]);

//Construct a Sphere that fits the bounds
void     ConstructSphere(double Bounds[6], double coeff[10]);



//Get the type of mapping
 eMapping GetMappingIndex(SFString* TypeOfMapping);

#ifdef AMTEXTURE

 //Migrate for current isovalue to another
double   Migrate(Volume_Tree_Node& The_Node,
                 float x,
                 float y,
                 float z,
                 int isoValue,
                 int index);


//Get bounds for non gradient mapping purpose
void     GetBounds(int nbvertex, PointArray _pVertex, double Bounds[]);

//Get the minimal bounds of a volume for gradient texturing purpose
void getMinimalBounds(Volume_Tree_Node& The_Node,
                      int nbvertex,
                      PointArray _pVertex,         
                      double Bounds[6]);

//Compute the texture coordinate
void ComputeTextureCoordinate(PointArray &params,
                              int nbvertex, 
                              PointArray &_p, 
                              GShellI *o,
                              SFString* TypeOfMapping,
                              Volume_Tree_Node& The_Node);

//Write an indexed face set VRML files with the GShellI Info.
void WriteIndexedFaceFile(PointArray &_p,
                          int vi,
                          int nbvertex,
                          GVertexI* verts,
                          PointArray &color_table,
                          const SL_Matrix& TransformMatrix);

#endif

//Return the Greater direction Axis.
eAxis    getAxisDirection(double dx,double dy, double dz);

//Return the maximum of three double.
double   maxDouble(double a, double b, double c);


#endif /*_POLYGONISE_H_*/
