// -*- c++ -*-
/***********************************************************************
This software module was originally developed by Alain Mignot (Mignot Informatique Graphique Inc)
 in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. (Mignot Informatique Graphique Inc.) retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2004
************************************************************************/


#include "OpenGl/OpenGl.h"
#include "QuadricProxy.h"
#include "opengl/base/OpenGLVisualRenderer.h"
#include "z2_coo.h"


// support classes



// Constructor

QuadricProxy::QuadricProxy(Quadric* pQuadric)
  : GeometryProxy(pQuadric)
{

    grid  = Global_Grid;
	created = FALSE;
	n_coeff = 10;
	m_nTimeStamp = -1;

//AM 1    densMatrix.dimension[0] = 0;      
//AM 1    densMatrix.dimension[1] = 0;
//AM 1    densMatrix.dimension[2] = 0;
    
//    TypeOfMapping.set_("Planar"); //AM
//   TransformMatrix.Identity();	 

}

// Destructor

QuadricProxy::~QuadricProxy()
{
  // ici destruction

}



Volume_Tree_Node *QuadricProxy::GetVolumeTreeNode()
{    

    int NodeDensities[2];


	ValidateDensities(NodeDensities, GetDensities());




	Volume_Tree_Node *to_return = new SolidNode( Get_Current_Coeff(),
											   n_coeff,
											   GetSolid(),
											   GetbboxSize(),
                                             NodeDensities);

	if ( GetDual() )
		to_return = new Volume_Tree_Unary_Op(SGDLNot, to_return, GetbboxSize());

	return to_return;    
}

float *QuadricProxy::Get_Current_Coeff()
{
	double v6v4[24];
	double coeffd[10];

    const Vec3f& size =GetbboxSize();

	double bx = size.x;
	double by = size.y;
	double bz = size.z;	

	const Vec4f& P0 = GetP0();
	const Vec4f& P1 = GetP1();
	const Vec4f& P2 = GetP2();
	const Vec4f& P3 = GetP3();
	const Vec4f& P4 = GetP4();
	const Vec4f& P5 = GetP5();

    v6v4 [0] = (P0[0] * bx) / 2.0;
    v6v4 [1] = (P0[1] * by) / 2.0;
    v6v4 [2] = (P0[2] * bz) / 2.0;
    v6v4 [3] = (P0[3]);

    v6v4 [4] = (P1[0] * bx) / 2.0;
    v6v4 [5] = (P1[1] * by) / 2.0;
    v6v4 [6] = (P1[2] * bz) / 2.0;
    v6v4 [7] = (P1[3]);

    v6v4 [8] = (P2[0] * bx) / 2.0;
    v6v4 [9] = (P2[1] * by) / 2.0;
    v6v4[10] = (P2[2] * bz) / 2.0;
    v6v4[11] = (P2[3]);

    v6v4[12] = (P3[0] * bx) / 2.0;
    v6v4[13] = (P3[1] * by) / 2.0;
    v6v4[14] = (P3[2] * bz) / 2.0;
    v6v4[15] = (P3[3]);

    v6v4[16] = (P4[0] * bx) / 2.0;
    v6v4[17] = (P4[1] * by) / 2.0;
    v6v4[18] = (P4[2] * bz) / 2.0;
    v6v4[19] = (P4[3]);

    v6v4[20] = (P5[0] * bx) / 2.0;
    v6v4[21] = (P5[1] * by) / 2.0;
    v6v4[22] = (P5[2] * bz) / 2.0;
    v6v4[23] = (P5[3]);

	SGDL_z2_coo( coeffd, v6v4);
	n_coeff = 10;
	for( int i = 0; i < 10; i++) coeff[i] = coeffd[i];
	return coeff;
}


// Render object description

void
QuadricProxy::RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD /* dwTime */)
{
  // a voir pour color
	GLfloat color[] = { 0.0f, 0.0f, 1.0f, 0.75f};

  
  glColor4fv(color);

  RenderDescriptionBox(pRenderer, "Quadric");
}

// Render object

BOOL
QuadricProxy::Render(OpenGLVisualRenderer* pRenderer, DWORD dwTime)
{
 
	Quadric *pQuadric=GetQuadric();

	_needUpdate=long(pQuadric->GetUpdateTime()) > m_nTimeStamp ;

	if (_needUpdate || !created)
	{
		m_nTimeStamp=(long) dwTime;
       
        Evaluate();  
	}
    

//
// faire le rendu  partir due la representation cre dans Evaluate()
	Matrix rMatrix;

	SolidTree->Render( rMatrix);

  return TRUE;
}


// Traverse scene graph, checking for intersection

BOOL
QuadricProxy::Select(OpenGLVisualRenderer* pRenderer, DWORD dwTime, unsigned int& depth)
{
  GLuint selectBuffer[4];

  ::glSelectBuffer(4, selectBuffer);
  ::glRenderMode(GL_SELECT);

  Render(pRenderer, dwTime);

  if (::glRenderMode(GL_RENDER) > 0)
    if (depth >= selectBuffer[1]) {
      depth = selectBuffer[1];

      return TRUE;
    }

#ifdef _DEBUG
  if (pRenderer->CheckGLError("QuadricProxy:"))
    TRACE("QuadricProxy::Render() [%08x]: ", this);
#endif

  return FALSE;
}

// en provenance de GvQuadric

int QuadricProxy::Evaluate(){
    

        SolidTree = GetVolumeTreeNode();
		
         densMatrix.dimension[0] = GetbboxSize().x + 1;
        densMatrix.dimension[1] = GetbboxSize().y + 1;
        densMatrix.dimension[2] = GetbboxSize().z + 1;
        
 
        SolidTree->SetGraphic(NewDLzK_coe(densMatrix,
                            GetGrid(),
                            *SolidTree,
                            TextureURL,
                            &TypeOfMapping,
                            TransformMatrix,
                            DensityArray(),
                            true));
      
	created = true;
	
    return (1);
}

// is p outside the box ? 

Bool QuadricProxy::isOutside(Vec3f &p)
{

    return (!SolidTree->isInside(p.x,p.y,p.z));


	return FALSE;
}
