// -*- c++ -*-



#ifndef __QUADRICPROXY_H__
#define __QUADRICPROXY_H__

#include "../../geometry/GeometryProxy.h"
#include "stdafx.h"
#include "VolumeTreeNode/SolidNode.h"
#include "DensityTLS.h"
#include "VolumeTreeNode/SolidTools.h"


/// Quadric renderer node
/* fields:
        exposedField SF<Vec3f> bboxSize;
         exposedField MF<int> densities;
         exposedField SF<Bool> dual;
         exposedField SF<Vec4f> P0;
         exposedField SF<Vec4f> P1;
         exposedField SF<Vec4f> P2;
         exposedField SF<Vec4f> P3;
         exposedField SF<Vec4f> P4;
         exposedField SF<Vec4f> P5;
         exposedField SF<Bool> solid;
*/

class QuadricProxy
  : public GeometryProxy
{
public:

  /// Constructor

			QuadricProxy(Quadric* pQuadric);

  /// Destructor

  virtual		~QuadricProxy();





  /// Render object

  virtual BOOL		Render(OpenGLVisualRenderer* pRenderer, DWORD dwTime);

  /// Traverse scene graph, checking for intersection

  virtual BOOL		Select(OpenGLVisualRenderer* pRenderer, DWORD dwTime, unsigned int& depth);

  /// Render object description

  virtual void		RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD dwTime);

private:

  // variables ici

     


  // moved methods from Quadric to QuadricProxy
	
public:

	int Evaluate();

  inline Quadric * GetQuadric() const
  { return (Quadric *) GetMediaObject(); }

	float *Get_Current_Coeff();
	Volume_Tree_Node *GetVolumeTreeNode();

   /// Get bounding box
  void GetBbox(Vec3f& rBboxSize, Vec3f& rBboxCenter) const;


  // GetBboxSize
  Vec3f GetbboxSize()
  {
	  return (Vec3f) GetQuadric()->bboxSize;
  }


  // GetDual
  Bool GetDual()
  {
	return (Bool) GetQuadric()->dual;
  }
// SetDual
  void SetDual(Bool dual)
  {
	GetQuadric()->dual = (SF<Bool>) dual;
  }

  // GetSolid
  inline Bool GetSolid()
  {
	return (Bool) GetQuadric()->solid;
  }

// SetSolid
  void SetSolid(Bool solid)
  {
	GetQuadric()->solid = (SF<Bool>) solid;
  }

// GetP0
  Vec4f GetP0()
  {
	return (Vec4f ) ((GetQuadric()->P0));
  }
  

// SetP0
  void SetP0( Vec4f p0)
  {
	GetQuadric()->P0 = p0;
  }

// GetP1
  Vec4f GetP1()
  {
	return (Vec4f ) ((GetQuadric()->P1));
  }
  

// SetP1
  void SetP1( Vec4f p1)
  {
	GetQuadric()->P1 = p1;
  }

  // GetP2
  Vec4f GetP2()
  {
	return (Vec4f ) ((GetQuadric()->P2));
  }
  

// SetP2
  void SetP2( Vec4f p2)
  {
	GetQuadric()->P2 = p2;
  }
  // GetP3
  Vec4f GetP3()
  {
	return (Vec4f ) ((GetQuadric()->P3));
  }
  

// SetP3
  void SetP3( Vec4f p3)
  {
	GetQuadric()->P3 = p3;
  }
  // GetP4
  Vec4f GetP4()
  {
	return (Vec4f ) ((GetQuadric()->P4));
  }
  

// SetP4
  void SetP4( Vec4f p4)
  {
	GetQuadric()->P4 = p4;
  }
  // GetP5
  Vec4f GetP5()
  {
	return (Vec4f ) ((GetQuadric()->P5));
  }
  

// SetP5
  void SetP5( Vec4f p5)
  {
	GetQuadric()->P5 = p5;
  }
// GetDensities
  int *GetDensities()
  {
	return (int *) (&(GetQuadric()->densities[0]));
  }

  // GetGrid
  float GetGrid()
  {
	return grid;
  }

// SetDensities
  void *SetDensities( MF<int>& d)
  {
	GetQuadric()->densities = d;
  }


	//Fields
   DensityMatrix densMatrix; 

	// is this a solid shape ?
	Bool Solid()  { return ( GetSolid() ); }

	// is p outside the shape 

	Bool isOutside(Vec3f &p);
	Bool isSolid() { return Solid(); }

	//protected members
protected : 
	bool _needUpdate;
    //Default Grid Value
    float grid; 
	
	SFString TextureURL;

    SFString TypeOfMapping; // a rentrer manuellement
	SL_Matrix TransformMatrix;
	Bool created;
	Volume_Tree_Node *SolidTree;
    int n_coeff;
	float coeff[10];
		long m_nTimeStamp;
};


#endif // __QUADRICPROXY_H__

