// -*- c++ -*-
/***********************************************************************
This software module was originally developed by Alain Mignot (Mignot Informatique Graphique Inc)
 in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. (Mignot Informatique Graphique Inc.) retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2004
************************************************************************/

#include <typeinfo>
#include "OpenGl/OpenGl.h"
#include "SolidRepProxy.h"
#include "opengl/base/OpenGLVisualRenderer.h"
#include "z2_coo.h"
#include "ImplicitProxy.h"
#include "QuadricProxy.h"
#include "OpenGl/geometry/SphereProxy.h"
#include "OpenGl/geometry/ConeProxy.h"
#include "OpenGl/geometry/CylinderProxy.h"
#include "OpenGl/geometry/BoxProxy.h"
#include "OpenGl/group/TransformProxy.h"

#include "VolumeTreeNode/SolidNode.h"

// support classes



// Constructor

SolidRepProxy::SolidRepProxy(SolidRep* pSolidRep)
  : GeometryProxy(pSolidRep)
{

    grid  = Global_Grid;
	created = FALSE;
		m_nTimeStamp = -1;
	densityListN = 0;
//	dual = FALSE;

//AM 1    densMatrix.dimension[0] = 0;      
//AM 1    densMatrix.dimension[1] = 0;
//AM 1    densMatrix.dimension[2] = 0;
    
//    TypeOfMapping.set_("Planar"); //AM
//   TransformMatrix.Identity();	 

	My_Tree = NULL;

    My_TreeBBox.mi.Set(BBOX_MAX, BBOX_MAX, BBOX_MAX);
    My_TreeBBox.ma.Set(-BBOX_MAX, -BBOX_MAX, -BBOX_MAX);
}

// Destructor

SolidRepProxy::~SolidRepProxy()
{
  // ici destruction
    if ( My_Tree != NULL )
    {
        //Remove a link to My_Tree
        My_Tree->UnRef();
    }

    if(ExpandedTree.GetLength() != 0)
    {
        for(int i = 0; i < ExpandedTree.GetLength(); i++)
            delete ExpandedTree[i];
    }
}




Volume_Tree_Node *SolidRepProxy::GetVolumeTreeNode(BBox *MaxBBox)
{
    Volume_Tree_Node *to_return = NULL;
    
    if ( My_Tree != NULL )
    {
        to_return = My_Tree;
    }
    
    else {
		SF3DNode* solidTree = GetsolidTree();

		// 112204 [MBS] solidOp not in templates v5, use -1 instead
		int SolidOp = -1; //GetsolidOp();

		if (solidTree != NULL)
    {
        to_return = Give_Volume_Tree_Node(solidTree->GetNode(), MaxBBox, SolidOp);        

        //To add a doing nothing node that compute the SolidRep bounding box // temporary patch
        SetMyTree(new Volume_Tree_Matrix(to_return, GetbboxSize(), IdentityMatrix()));
        to_return = My_Tree;
    }
	}

		// 112204 [MBS] dual not in SolidRep node in templates V5, commented
    if ( (to_return != NULL) /*&& (GetDual())*/ )
        to_return = Verified_Dual(to_return, GetbboxSize());

    return to_return;
}



// Render object description

void
SolidRepProxy::RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD /* dwTime */)
{
  // a voir pour color
	GLfloat color[] = { 0.0f, 0.0f, 1.0f, 0.75f};

  
  glColor4fv(color);

  RenderDescriptionBox(pRenderer, "SolidRep");
}

// Render object


BOOL
SolidRepProxy::Render(OpenGLVisualRenderer* pRenderer, DWORD dwTime)
//SolidRepProxy::Render(const Matrix& rTransformation, const CullFlag& rCullFace, OpenGLVisualRenderer* pRenderer, DWORD dwTime)

{
	Matrix rMatrix;
//	MediaObjectProxy::ObjectToWorldMatrix( dwTime, rMatrix) ;

	SolidRep *pSolidRep=GetSolidRep();

	MediaObjectProxy::ViewpointToWorldMatrix( dwTime, rMatrix) ;

	_needUpdate=long(pSolidRep->GetUpdateTime()) > m_nTimeStamp ;

	if (_needUpdate || !created)
	{
		m_nTimeStamp=(long) dwTime;
       
        Evaluate();  
	}
    
 
//
// faire le rendu  partir due la representation cre dans Evaluate()

	SolidTree->Render( rMatrix);

  return TRUE;
}


// Traverse scene graph, checking for intersection

BOOL
SolidRepProxy::Select(OpenGLVisualRenderer* pRenderer, DWORD dwTime, unsigned int& depth)
{
  GLuint selectBuffer[4];

  ::glSelectBuffer(4, selectBuffer);
  ::glRenderMode(GL_SELECT);

  Render(pRenderer, dwTime);

  if (::glRenderMode(GL_RENDER) > 0)
    if (depth >= selectBuffer[1]) {
      depth = selectBuffer[1];

      return TRUE;
    }

#ifdef _DEBUG
  if (pRenderer->CheckGLError("SolidRepProxy:"))
    TRACE("SolidRepProxy::Render() [%08x]: ", this);
#endif

  return FALSE;
}

// en provenance de GvSolidRep

int SolidRepProxy::Evaluate(){
    

		SolidTree = GetVolumeTreeNode();

#if 0 // a v oir
    if(!state.boundingVolume.IsEmpty())
        SolidTree->m_BoundingVolume = state.boundingVolume.Top();
#endif
	//We've been able to create the tree
	if ( SolidTree != NULL )
	{
#if 0 // AM color??
      if ((SolidTree->GetLocalColor())[0] == -1)
         SolidTree->Set_Color(SR_color);
#endif
		//+1 to round
         densMatrix.dimension[0] = GetbboxSize().x + 1;
        densMatrix.dimension[1] = GetbboxSize().y + 1;
        densMatrix.dimension[2] = GetbboxSize().z + 1;


		DensityArray MyArray(GetdensityListN(), GetdensityList());

        MyArray.Sort();


        bool manage_color = true; // attention
        bool manage_normals = true;

		SolidTree->SetGraphic(NewDLzK_coe(densMatrix,
                            grid,
                            *SolidTree,
                            TextureURL,
                            &TypeOfMapping,
                            TransformMatrix,
                            MyArray,
                            manage_normals,
                            manage_color));

	}

#if 0 // a voir
	//We don't know this solid so traverse it and try to get its shell
	else
	{
		solidTree.Traverse(state);
		GvNodeShell *tried_Node_Shell = dynamic_cast<GvNodeShell*>(solidTree.get());
		if ( tried_Node_Shell != NULL )
		{
			SetShell(tried_Node_Shell->GetShell());
		}
	}

#endif
		created = true;
   
    return (1);
}

// is p outside the box ? 

Bool SolidRepProxy::isOutside(Vec3f &p)
{

    return (!SolidTree->isInside(p.x,p.y,p.z));


	// 112204 [MBS] unreachable code => commented
	//return FALSE;
}

//*******************************************************************
//  This function sets the variable My_Tree properly
//
//  By Eric Paquet
//  SGDL Systems inc
//  Date : December 10th, 2001.
//*******************************************************************
void SolidRepProxy::SetMyTree(Volume_Tree_Node *the_other)
{
   if ( My_Tree != NULL )
       My_Tree->UnRef();

    My_Tree = the_other;
    
    //Add a link to the VTN
    if(the_other != NULL)
        the_other->Ref();
}


/***************************************************
//	
//  Return the Volume_Tree_Node associated with a
//	Node
//
//	Author : Eric Paquet, SGDL Systems
//	Date : October 17th, 2001
//
//  Input  : The_Node     , the current working node        

//
//  Output : A Volume_Tree_Node corresponding to the Node

//
/***************************************************/
Volume_Tree_Node *Give_Volume_Tree_Node(MediaObject *The_Node, BBox* MaxBBox, int solidOp)
{
//    ASSERT( The_Node != NULL );
	if ( The_Node == NULL)return NULL;

    Volume_Tree_Node *to_return         = NULL;
    Volume_Tree_Node *child_s_Tree_Node = NULL;
    MediaObject      *temp_Children     = NULL;
    ZArray<Volume_Tree_Node*> All_Children;
    BBox TempBBox((Point)0.0);
    
    int Number_of_Child   = 0;
    int default_Density[] = {1,2};
    
	int Node_code = The_Node->GetNodeCode();

    switch ( Node_code )
    {
	case 0:
	case CODE_Group:
        {
            //Construct a SR with all child of the group

            Number_of_Child = ((MediaObject*)The_Node)->GetChildCount();

            if ( Number_of_Child > 0 )
            {             
                for ( int i = 0; i < Number_of_Child; i++ )
                {
					if( i == 0 ) temp_Children = ((MediaObject*)The_Node)->GetFirstChild();
					else temp_Children = ((MediaObject*)The_Node)->GetNextSibling();

					if( temp_Children == NULL) break;

                    child_s_Tree_Node = Give_Volume_Tree_Node(temp_Children, MaxBBox, solidOp);
                
                    if (child_s_Tree_Node)
						All_Children += child_s_Tree_Node;
                }
             

				// a changer pour creer des Cellules de AND au lieu de cellules d'OR
                if (All_Children.GetLength() == 0)
                    break; //We haven't found any child that are compatible node
                else if ( All_Children.GetLength() == 1 )
                    to_return = All_Children[0];
                else if ( All_Children.GetLength() == 2 )
  //                  to_return = new Volume_Tree_Binary_Op(SGDLOr, All_Children[0], All_Children[1]);
                    to_return = new Volume_Tree_Binary_Op((SGDLBinaryOperator) solidOp, All_Children[0], All_Children[1]);
               else
     //                 to_return = new Volume_Tree_Multiple_Op(SGDLMOr, All_Children);
                    to_return = new Volume_Tree_Multiple_Op( (SGDLMultipleOperator) solidOp, All_Children);

			}
            break;
        }

		        case CODE_Shape:
        {
            //Search for a diffuse color and if it's not black set the VTN to this color
            SFAppearanceNode appearance;
			SFGeometryNode geometry;
            float        *temp_color       = 0;
            Shape      *tried_Shape      = (Shape *)(The_Node);

            appearance = (SFAppearanceNode )(tried_Shape->appearance);          
            geometry = (SFGeometryNode )(tried_Shape->geometry);          


            to_return = Give_Volume_Tree_Node(geometry, MaxBBox, solidOp);
            if (to_return && temp_color) 
                to_return->Set_Color(temp_color);
            break;
        }

		
        case CODE_Implicit:
        {
            //No problemo
			ImplicitProxy* proxy =  (ImplicitProxy*) (The_Node->GetRenderingObject());

            to_return = proxy->GetVolumeTreeNode();

 //           SOLID_Node_In_Solid_Rep_Must_Be_Solid(to_return);
       
#if 0 // a faire
            if (MaxBBox)
            {            
                proxy->getBBox(TempBBox);

                if (MaxBBox->ma == MaxBBox->mi)
                    *MaxBBox = TempBBox;
                else 
                    CheckForMaxBBox(&TempBBox, MaxBBox);
            }
#endif
            break;
        }

        case CODE_Quadric:
        {
            //No problemo
			QuadricProxy* proxy =  (QuadricProxy*) (The_Node->GetRenderingObject());

            to_return = proxy->GetVolumeTreeNode();
//            SOLID_Node_In_Solid_Rep_Must_Be_Solid(to_return);
          
#if 0
            if (MaxBBox)
            {            
                proxy->getBBox(TempBBox);

                if (MaxBBox->ma == MaxBBox->mi)
                    *MaxBBox = TempBBox;
                else 
                    CheckForMaxBBox(&TempBBox, MaxBBox);
            }
#endif
            break;
        }

        case CODE_SolidRep:
        {
            //No problemo
			SolidRepProxy* proxy =  (SolidRepProxy*) (The_Node->GetRenderingObject());

            to_return = proxy->GetVolumeTreeNode(MaxBBox);
//            SOLID_Node_In_Solid_Rep_Must_Be_Solid(to_return);

#if 0

            if (MaxBBox)
            {            
                proxy->getBBox(TempBBox);

                if (MaxBBox->ma == MaxBBox->mi)
                    *MaxBBox = TempBBox;
                else 
                    CheckForMaxBBox(&TempBBox, MaxBBox);
            }
#endif
            break;
        }

		case CODE_Sphere:
        {
            //if it's a VRML Sphere convert it to an Implicit sphere
			SphereProxy* proxy =  (SphereProxy*) (The_Node->GetRenderingObject());

            float rayon = proxy->GetRadius();

            double v6v4[24];
            double coeff[10];
          
            double diameter = 2.0 * rayon;

            v6v4 [0] = (-1.0 * diameter) / 2.0;
            v6v4 [1] = (0.0 * diameter) / 2.0;
            v6v4 [2] = (0.0 * diameter) / 2.0;
            v6v4 [3] = (1.0);

            v6v4 [4] = (1.0 * diameter) / 2.0;
            v6v4 [5] = (0.0 * diameter) / 2.0;
            v6v4 [6] = (0.0 * diameter) / 2.0;
            v6v4 [7] = (1.0);

            v6v4 [8] = (0.0 * diameter) / 2.0;
            v6v4 [9] = (1.0 * diameter) / 2.0;
            v6v4[10] = (0.0 * diameter) / 2.0;
            v6v4[11] = (0.0);

            v6v4[12] = (0.0 * diameter) / 2.0;
            v6v4[13] = (0.0 * diameter) / 2.0;
            v6v4[14] = (1.0 * diameter) / 2.0;
            v6v4[15] = (0.0);

            v6v4[16] = (0.0 * diameter) / 2.0;
            v6v4[17] = (1.0 * diameter) / 2.0;
            v6v4[18] = (0.0 * diameter) / 2.0;
            v6v4[19] = (1.0);

            v6v4[20] = (0.0 * diameter) / 2.0;
            v6v4[21] = (0.0 * diameter) / 2.0;
            v6v4[22] = (1.0 * diameter) / 2.0;
            v6v4[23] = (1.0);

            SGDL_z2_coo( coeff, v6v4);
          
            //Get the VTN
            float the_bbox[] = {2.0 * rayon, 2.0 * rayon, 2.0 * rayon};          
            to_return = new SolidNode(coeff, 10, true, the_bbox, default_Density);
//			SOLID_Node_In_Solid_Rep_Must_Be_Solid(to_return);
          
// MAxxbox a faire
			break;
		}

	    case CODE_Cone:
        {
			//if it's a VRML Cone convert it to an implicit Cone
			ConeProxy* proxy =  (ConeProxy*) (The_Node->GetRenderingObject());
		    float Cone_Bottom_rayon = proxy->GetBottomRadius();
			float Cone_Height = proxy->GetHeight();

			double v6v4[24];
			double coeff[10];
    
			double diameter = 2.0 * Cone_Bottom_rayon;	

			v6v4 [0] = (-1.0 * diameter) / 2.0;
			v6v4 [1] = (-1.0 * Cone_Height) / 2.0;
			v6v4 [2] = (0.0 * diameter) / 2.0;
			v6v4 [3] = (1.0);

			v6v4 [4] = (1.0 * diameter) / 2.0;
			v6v4 [5] = (-1.0 * Cone_Height) / 2.0;
			v6v4 [6] = (0.0 * diameter) / 2.0;
			v6v4 [7] = (1.0);

			v6v4 [8] = (0.0 * diameter) / 2.0;
			v6v4 [9] = (1.0 * Cone_Height) / 2.0;
			v6v4[10] = (0.0 * diameter) / 2.0;
			v6v4[11] = (1.0);

			v6v4[12] = (0.0 * diameter) / 2.0;
			v6v4[13] = (0.0 * Cone_Height) / 2.0;
			v6v4[14] = (1.0 * diameter) / 2.0;
			v6v4[15] = (0.0);

			v6v4[16] = (0.0 * diameter) / 2.0;
			v6v4[17] = (1.0 * Cone_Height) / 2.0;
			v6v4[18] = (0.0 * diameter) / 2.0;
			v6v4[19] = (1.0);

			v6v4[20] = (0.0 * diameter) / 2.0;
			v6v4[21] = (-1.0 * Cone_Height) / 2.0;
			v6v4[22] = (1.0 * diameter) / 2.0;
			v6v4[23] = (1.0);

			SGDL_z2_coo( coeff, v6v4);
			
			//Get the VTN
			float the_bbox[] = {2.0 * Cone_Bottom_rayon, Cone_Height, 2.0 * Cone_Bottom_rayon};
			to_return = new SolidNode(coeff, 10, true, the_bbox, default_Density);
  //          SOLID_Node_In_Solid_Rep_Must_Be_Solid(to_return);
         
// MAxxbox a faire
           			
			break;
		}
       case CODE_Cylinder:
		{
			//if it's a VRML Cylinder convert it to an implicit Cylinder
			CylinderProxy* proxy =  (CylinderProxy*) (The_Node->GetRenderingObject());
			float Cyl_Rayon = proxy->GetRadius();
			float Cyl_Height = proxy->GetHeight();

			double v6v4[24];
			double coeff[10];
    
			double diameter = 2.0 * Cyl_Rayon;	

			v6v4 [0] = (-1.0 * diameter) / 2.0;
			v6v4 [1] = (0.0 * Cyl_Height) / 2.0;
			v6v4 [2] = (0.0 * diameter) / 2.0;
			v6v4 [3] = (1.0);

			v6v4 [4] = (1.0 * diameter) / 2.0;
			v6v4 [5] = (0.0 * Cyl_Height) / 2.0;
			v6v4 [6] = (0.0 * diameter) / 2.0;
			v6v4 [7] = (1.0);

			v6v4 [8] = (0.0 * diameter) / 2.0;
			v6v4 [9] = (1.0 * Cyl_Height) / 2.0;
			v6v4[10] = (0.0 * diameter) / 2.0;
			v6v4[11] = (0.0);

			v6v4[12] = (0.0 * diameter) / 2.0;
			v6v4[13] = (0.0 * Cyl_Height) / 2.0;
			v6v4[14] = (1.0 * diameter) / 2.0;
			v6v4[15] = (0.0);

			v6v4[16] = (0.0 * diameter) / 2.0;
			v6v4[17] = (1.0 * Cyl_Height) / 2.0;
			v6v4[18] = (0.0 * diameter) / 2.0;
			v6v4[19] = (0.0);

			v6v4[20] = (0.0 * diameter) / 2.0;
			v6v4[21] = (0.0 * Cyl_Height) / 2.0;
			v6v4[22] = (1.0 * diameter) / 2.0;
			v6v4[23] = (1.0);

			SGDL_z2_coo( coeff, v6v4);
			
			//Get the VTN
			float the_bbox[] = {2.0 * Cyl_Rayon, Cyl_Height, 2.0 * Cyl_Rayon};
			to_return = new SolidNode(coeff, 10, true, the_bbox, default_Density);
//            SOLID_Node_In_Solid_Rep_Must_Be_Solid(to_return);
         
 //  // MAxxbox a faire //TBM check for correct bbox... maybe ok, but should be verified
     
			break;
		}
	    case CODE_Box:
		{
			//if it's a VRML Box convert it to an implicit Box
			BoxProxy* proxy =  (BoxProxy*) (The_Node->GetRenderingObject());
			const Vec3f& size = proxy->GetSize();

			float delta_x = size.x;
			float delta_y = size.y;
			float delta_z = size.z;

			double v6v4[24];
			double coeff[10];

			v6v4 [0] = (-1.0 * delta_x) / 2.0;
			v6v4 [1] = (0.0 * delta_y) / 2.0;
			v6v4 [2] = (0.0 * delta_z) / 2.0;
			v6v4 [3] = (1.0);

			v6v4 [4] = (1.0 * delta_x) / 2.0;
			v6v4 [5] = (0.0 * delta_y) / 2.0;
			v6v4 [6] = (0.0 * delta_z) / 2.0;
			v6v4 [7] = (1.0);

			v6v4 [8] = (0.0 * delta_x) / 2.0;
			v6v4 [9] = (1.0 * delta_y) / 2.0;
			v6v4[10] = (0.0 * delta_z) / 2.0;
			v6v4[11] = (0.0);

			v6v4[12] = (0.0 * delta_x) / 2.0;
			v6v4[13] = (0.0 * delta_y) / 2.0;
			v6v4[14] = (1.0 * delta_z) / 2.0;
			v6v4[15] = (0.0);

			v6v4[16] = (0.0 * delta_x) / 2.0;
			v6v4[17] = (1.0 * delta_y) / 2.0;
			v6v4[18] = (0.0 * delta_z) / 2.0;
			v6v4[19] = (0.0);

			v6v4[20] = (0.0 * delta_x) / 2.0;
			v6v4[21] = (0.0 * delta_y) / 2.0;
			v6v4[22] = (1.0 * delta_z) / 2.0;
			v6v4[23] = (0.0);

			SGDL_z2_coo( coeff, v6v4);
			
			//Get the VTN
			float the_bbox[] = {delta_x, delta_y, delta_z};
			to_return = new SolidNode(coeff, 10, true, the_bbox, default_Density);
   //         SOLID_Node_In_Solid_Rep_Must_Be_Solid(to_return);
         
 // // MAxxbox a faire
    
			break;
		}



        case CODE_Transform:
        {
            //Get the matrix and set all the child with it
			TransformProxy* proxy =  (TransformProxy*) (The_Node->GetRenderingObject());
          

            Number_of_Child = ((MediaObject*)The_Node)->GetChildCount();

            if ( Number_of_Child > 0 )
            {             
                for ( int i = 0; i < Number_of_Child; i++ )
                {
					if( i == 0 ) temp_Children = ((MediaObject*)The_Node)->GetFirstChild();
					else temp_Children = ((MediaObject*)The_Node)->GetNextSibling();

					if( temp_Children == NULL) break;

                    child_s_Tree_Node = Give_Volume_Tree_Node(temp_Children, (MaxBBox != NULL) ? &TempBBox : NULL, solidOp );
                
                    if (child_s_Tree_Node)
						All_Children += child_s_Tree_Node;
                }  
				
                //A bit of optimization
                if (All_Children.GetLength() == 0)
                    break;
                else if ( All_Children.GetLength() == 1 )
                    to_return = All_Children[0];
                else if ( All_Children.GetLength() == 2 )
     //               to_return = new Volume_Tree_Binary_Op(SGDLOr, All_Children[0], All_Children[1]);
                      to_return = new Volume_Tree_Binary_Op(( SGDLBinaryOperator) solidOp, All_Children[0], All_Children[1]);

				else
      //              to_return = new Volume_Tree_Multiple_Op(SGDLMOr, All_Children);
                    to_return = new Volume_Tree_Multiple_Op(( SGDLMultipleOperator ) solidOp, All_Children);


				Transform * poTransform = (Transform *) (proxy->GetMediaObject());

				const Vec3f& rCenter = poTransform->center;
				const Vec3f& rScale = poTransform->scale;
				const Vec3f& rTranslation = poTransform->translation;
				const Rotation& rScaleOrientation = poTransform->scaleOrientation;
				const Rotation& rRotation = poTransform->rotation;

				Vec3f r;
				if( rRotation.x) r.x = rRotation.theta;
				if( rRotation.y) r.y = rRotation.theta;
				if( rRotation.z) r.z = rRotation.theta;


				SL_Matrix combined_matrix = RotationMatrix( r.x,r.y, r.z);
				combined_matrix.Scale( rScale.x, rScale.y,rScale.z);
				combined_matrix.Translate( rTranslation.x,rTranslation.y,rTranslation.z);

                to_return = Verified_Volume_Tree_Matrix(combined_matrix, to_return);
#if 0                
                if (MaxBBox)
                {                    
                    //transform the bbox
                    TempBBox.mi = combined_matrix * TempBBox.mi;
                    TempBBox.ma = combined_matrix * TempBBox.ma;

                    //Ensure the BBox of the scene contain all the necessary information
                    if (MaxBBox->mi == MaxBBox->ma)                    
                        *MaxBBox = TempBBox;
                    else                    
                        CheckForMaxBBox(&TempBBox, MaxBBox);
                }
#endif
            }
          
            break;
        }



#if 0 // plus tard    
 
	    default:
		{
            if (!MaxBBox)
            {
		        //If we don't know the node, try to get its shell
			    GvNodeShell *tried_GvNodeShell = dynamic_cast<GvNodeShell*>(The_Node);
			    if ( tried_GvNodeShell != NULL )
			    {
				    GglRenderTraversal temp;
                    temp.view = new GView();
				    tried_GvNodeShell->BuildShell(temp);
				    GShell *Unknown_GShell = tried_GvNodeShell->GetShell();
				    if ( Unknown_GShell != NULL )
                    {
					    to_return = new Volume_Tree_GShell(Unknown_GShell);
                        BBox the_GShell_BBox = Unknown_GShell->GetBBox();
                        
                        //-.05 to be able to flatten the solid in Validate_Point
                        float temp_bbox[] = {2 * the_GShell_BBox.ma.x, 
                                             2 * the_GShell_BBox.ma.y,
                                             2 * the_GShell_BBox.ma.z};
                        to_return->Set_Bbox(temp_bbox);
                    }
                    delete temp.view;
                }
            }
            break;
        }
#endif // PLUSTARD
		
        default:
		{
			//there is a problem
			ASSERT( false );
			break;
		}
    }    
    return to_return;
}



/*******************************************************************\
Function: SOLID_Node_In_Solid_Rep_Must_Be_Solid(Volume_Tree_Node *To_verify_Solidity)

  Solidify a SolidNode
  
    Input: The possible SolidNode* to solidify     
    Output: The possible SolidNode* to solidify    
    
      Author:     Eric Paquet SGDL systems
      Created:    2001/11/6	

      
        
*********************************************************************/
void SOLID_Node_In_Solid_Rep_Must_Be_Solid(Volume_Tree_Node *To_verify_Solidity)
{

	
//		To_verify_Solidity->Set_Solidity(true);
}

void SOLID_Node_In_Solid_Rep_Must_Be_Solid(SolidNode *To_verify_Solidity)
{

	
		To_verify_Solidity->Set_Solidity(true);
}

