//*********************************************************************************
//  This class contains all the information that is necessary to build 
//  an implicit volume by his coefficient
//
//	(input)
//	solid : true the solid will be closed else it's open
//	coeff : Table  containing the coefficients
//	transfo_Mat : Transfo to apply to the solid ??
//
//	Author : Eric Paquet
//	Date : October 3th, 2001
//
//	Modif:
// Alain Mignot (MIG) - may 2004

//*********************************************************************************

#ifndef _SOLID_NODE_H_
#define _SOLID_NODE_H_

#include "VolumeTreeNode.h"
#include "Opengl/AFX/Solid/SolidGraphic.h"
#include "OpenGl/OpenGl.h"
#include "opengl/base/Matrix.h"

class SolidNode : public Volume_Tree_Node
{
    //public members
public:
    SolidNode(const double coeff_ptr[],
			 int          coeff_number,
			 bool         is_solid,
			 const float  bbox_ptr[],
			 int          Node_Density[2]);

    SolidNode(const float coeff_ptr[],
			 int         coeff_number,
			 bool        is_solid,
			 const float bbox_ptr[],
			 int         Node_Density[2]);

    SolidNode(void);
    
    int     EvaluateDensity(const double& x, const double& y, const double& z)const;
    
    bool    EvaluateGradient(const double &x,
                             const double &y,
                             const double &z,
                             double       &Distance,
                             double       *Gradient,
                             float        *Color = 0) const;

    double  *GetCoeff    (double x, double y, double z);
    int     GetNumOfCoeff(double x, double y, double z);

    bool    GetColor(const double &x, 
                     const double &y, 
                     const double &z, 
                     float set_color[])const;

    void    GetPrimitives(ZArray<BoundingQuadric*> &outPrimitives,
                          const SL_Matrix &Transform, BBox &outGlobalBBox);

    void    UnRef();
    
	//Accessor
	void    Set_Solidity(bool to_set);

	void SetGraphic( SolidGraphic *graphic);
	SolidGraphic	*GetGraphic();

	bool Render(Matrix &rMatrix);
    
protected:
    //Protected member	
    bool    solid; 
    double  *coeff;	
    int     nCoeff;
    int     density[2];
    
	SolidGraphic	*Graphic;
	int m_displaylist;

	void Manage_At_The_Top();

     ~SolidNode(void);

private:
};


#endif  //_SOLID_NODE_H_