//******************************************************
//SOLID Tools FUNCTIONS
//******************************************************

#include "Opengl/AFX/Solid/stdafx.h"
#include "SolidTools.h"
#include "..\\Polygonise.h"
#include <assert.h>
//AM #include "gcamera.h"
#include "..\\DensityTLS.h" 

/***************************************************
//	
//  Set the first bounding box with th higest x, y an z of the two bounding boxs
//
//	Author : Eric Paquet, SGDL Systems
//	Date : October 17th, 2001
//
//  Input  : float First[]	 , the to be modify bounding box
//	     const float Second[], the other bounding box	 
//
//  Output : Maybe a modify First bounding box
//
/***************************************************/
void Globalize_First_BBox(float First[], const float Second[])
{
	First[0] = First[0] < Second[0] ? Second[0] : First[0];
	First[1] = First[1] < Second[1] ? Second[1] : First[1];
	First[2] = First[2] < Second[2] ? Second[2] : First[2];
}

//*******************************************************************
//	This function change the value of is_inside to false
//	if the point x,y,z is outside bbox
//
//  By Eric Paquet
//  SGDL Systems inc
//  Date : December 10th, 2001.
//*******************************************************************
void Bounding_Box_Correction(const float bbox[],
			     int *value,
			     const double& x,
			     const double& y,
			     const double& z)
    {
    if ( bbox != NULL )
        {
        if ((*value > 0) && 
           ((x < -(bbox[0]/2.0))  ||
            (x >  (bbox[0]/2.0))  ||
            (y < -(bbox[1]/2.0))  ||
            (y >  (bbox[1]/2.0))  ||
            (z < -(bbox[2]/2.0))  ||
            (z >  (bbox[2]/2.0))))
            {
            *value = 0;
            }
        }
    }




//***************************************************
//	
//  Bounding_Box_Gradient_Correction()
//
//	Author : Simon Lavallee, SGDL Systems
//	Date   : March 27, 2002
//
//  Input  : float BBox             -> The bbox           
//           double *Distance       -> Pt to the initial distance
//           double x               -> X coordinate
//           double y               -> Y coordinate
//           double z               -> Z coordinate
//
//  Output : double *InitGradient   -> modified (or not) Gradient
//           double *Distance       -> modified (or not) Distance
//
//  Desc.  : This function checks if the point is outside or very near
//           of a bbox face. If yes, it means that the gradient must
//           be the normal of the face and the field value must be
//           the distance between the point and the face.
//***************************************************
void Bounding_Box_Gradient_Correction(const float   bbox[],
                                      double        *NormalToReturn,
                                      double        *Distance,
                                      const double& x,
                                      const double& y,
                                      const double& z)
{
    //Flags
    bool   IsInsideBBox    = false;
    bool   IsInsideVolume  = EpsilonGreaterOrEqual(*Distance, 0.0);
    bool   KeepMinDistance = true;

    double BBoxDist = INVALID_DISTANCE;    

    double Xdiv2    = 0.0;
    double Ydiv2    = 0.0;
    double Zdiv2    = 0.0;

    double Xdist    = 0.0;
    double Ydist    = 0.0;
    double Zdist    = 0.0;

    double BBoxNormalX  = 0.0;
    double BBoxNormalY  = 0.0;
    double BBoxNormalZ  = 0.0;

    if ( bbox != NULL )
    {
        Xdiv2 = bbox[0]/2.0; //bbox X E [-Xdiv2, Xdiv2]
        Ydiv2 = bbox[1]/2.0; //bbox Y E [-Ydiv2, Ydiv2]
        Zdiv2 = bbox[2]/2.0; //bbox Z E [-Zdiv2, Zdiv2]

        Xdist = fabs(fabs(x) - Xdiv2);
        Ydist = fabs(fabs(y) - Ydiv2);
        Zdist = fabs(fabs(z) - Zdiv2);

        //Check if the points is inside or approximately equal to the
        //bbox. If it is true we need to keep the min distance between
        //the point and the bbox face. Else, keep the max distance.
        IsInsideBBox = (EpsilonGreaterOrEqual(x, -Xdiv2)  &&
                        EpsilonLessOrEqual(x, Xdiv2)      &&
                        EpsilonGreaterOrEqual(y, -Ydiv2)  &&
                        EpsilonLessOrEqual(y, Ydiv2)      &&
                        EpsilonGreaterOrEqual(z, -Zdiv2)  &&
                        EpsilonLessOrEqual(z, Zdiv2));

        if (IsInsideBBox && !IsInsideVolume)
            //The bbox isn't supposed to influence this point
            return;

        if (EpsilonLessOrEqual(x, -Xdiv2))
        {
            BBoxNormalX = -1;
            BBoxNormalY = 0;
            BBoxNormalZ = 0;
            BBoxDist    = Xdist;
        }
        
        else if (EpsilonGreaterOrEqual(x, Xdiv2))
        {
            BBoxNormalX = 1;
            BBoxNormalY = 0;
            BBoxNormalZ = 0;
            BBoxDist    = Xdist;
        }

        if (EpsilonLessOrEqual(y, -Ydiv2)         &&
           ((BBoxDist == INVALID_DISTANCE)        ||
            ( IsInsideBBox && (BBoxDist > Ydist)) || //If inside,  keep the min distance
            (!IsInsideBBox && (BBoxDist < Ydist))))  //If outside, keep the max distance
             
        {
            BBoxNormalX = 0;
            BBoxNormalY = -1;
            BBoxNormalZ = 0;
            BBoxDist    = Ydist;
         }

        else if (EpsilonGreaterOrEqual(y, Ydiv2)       &&
                ((BBoxDist == INVALID_DISTANCE)        ||
                 ( IsInsideBBox && (BBoxDist > Ydist)) ||
                 (!IsInsideBBox && (BBoxDist < Ydist))))
        {
            BBoxNormalX = 0;
            BBoxNormalY = 1;
            BBoxNormalZ = 0;
            BBoxDist    = Ydist;
        }

        if (EpsilonLessOrEqual(z, -Zdiv2)         &&
           ((BBoxDist == INVALID_DISTANCE)        ||
            ( IsInsideBBox && (BBoxDist > Zdist)) ||
            (!IsInsideBBox && (BBoxDist < Zdist))))
        {
            BBoxNormalX = 0;
            BBoxNormalY = 0;
            BBoxNormalZ = -1;
            BBoxDist    = Zdist;
        }

        else if (EpsilonGreaterOrEqual(z, Zdiv2)       &&
                ((BBoxDist == INVALID_DISTANCE)        ||
                 ( IsInsideBBox && (BBoxDist > Zdist)) ||
                 (!IsInsideBBox && (BBoxDist < Zdist))))
        {
            BBoxNormalX = 0;
            BBoxNormalY = 0;
            BBoxNormalZ = 1;
            BBoxDist    = Zdist;
        }
    }

    if ((BBoxDist != INVALID_DISTANCE) && 
        (IsInsideBBox ? (fabs(*Distance) > BBoxDist) : (fabs(*Distance) < BBoxDist)))
    {
        //Check to keep the correct sign of the distance
        if (IsInsideBBox && (*Distance > 0.0))
            *Distance = BBoxDist;
        else
            *Distance = -BBoxDist;

        //If we found a different gradient, it must be the good one
        NormalToReturn[0] = BBoxNormalX;
        NormalToReturn[1] = BBoxNormalY;
        NormalToReturn[2] = BBoxNormalZ;
    }
}


//***************************************************
//	
//  EpsilonLessOrEqual()
//
//	Author : Simon Lavallee, SGDL Systems
//	Date   : March 27, 2002
//
//  Input  : double Value1          -> first  value
//           double Value2          -> second value
//
//  Output : bool -> true = Value1 < Value || 
//                          the two values are approximately equal.
//
//  Desc.  : 
//***************************************************
bool EpsilonLessOrEqual(double Value1, double Value2)
    {
    return ((Value1 < Value2) || (fabs(Value1 - Value2) < EPSILON));
    }

//***************************************************
//	
//  EpsilonGreaterOrEqual()
//
//	Author : Simon Lavallee, SGDL Systems
//	Date   : March 27, 2002
//
//  Input  : double Value1          -> first  value
//           double Value2          -> second value
//
//  Output : bool -> true = Value1 > Value || 
//                          the two values are approximately equal.
//
//  Desc.  : 
//***************************************************
bool EpsilonGreaterOrEqual(double Value1, double Value2)
    {
    return ((Value1 > Value2) || (fabs(Value1 - Value2) < EPSILON));
    }

//***************************************************
//	
//  EpsilonEqual()
//
//	Author : Simon Lavallee, SGDL Systems
//	Date   : March 27, 2002
//
//  Input  : double Value1          -> first  value
//           double Value2          -> second value
//
//  Output : bool -> true = the two values are approximately equal.
//
//  Desc.  : 
//***************************************************
bool EpsilonEqual(double Value1, double Value2)
    {
    return (fabs(Value1 - Value2) < EPSILON);
    }

//*******************************************************************
//	Multiply a Matrix with a point
//
//  By Eric Paquet
//  SGDL Systems inc
//  Date : December 10th, 2001.
//*******************************************************************
void Multiply_Mat_Double(const SL_Matrix& M,
						double x,
						double y,
						double z,
						double *in_x,
						double *in_y,
						double *in_z)
{
    double denom;

    *in_x = x * M.x[0][0] + y * M.x[1][0] + z * M.x[2][0] + M.x[3][0];
    *in_y = x * M.x[0][1] + y * M.x[1][1] + z * M.x[2][1] + M.x[3][1];
    *in_z = x * M.x[0][2] + y * M.x[1][2] + z * M.x[2][2] + M.x[3][2];
    denom = x * M.x[0][3] + y * M.x[1][3] + z * M.x[2][3] + M.x[3][3];

	if (denom == 0.0) 
		return;

    if (denom != 1.0)
	{
		(*in_x) = (*in_x) / denom;
		(*in_y) = (*in_y) / denom;
		(*in_z) = (*in_z) / denom;
	}
}

//*******************************************************************
//  This function verifies if the colour received is -1.0 -1.0 -1.0
//
//  By Eric Paquet
//  SGDL Systems inc
//  Date : December 10th, 2001.
//*******************************************************************
bool Undefined_Color(const float the_color_to_test[])
{
	return the_color_to_test[0] == -1.0 && 
		   the_color_to_test[1] == -1.0 &&
		   the_color_to_test[2] == -1.0;
}

//*******************************************************************
//This function prevent to place two VTNMs one after the other
//
//  By Eric Paquet
//  SGDL Systems inc
//  Date : December 10th, 2001.
//*******************************************************************
Volume_Tree_Node *Verified_Volume_Tree_Matrix(const SL_Matrix& The_Matrix, Volume_Tree_Node *The_VTN)
{
	assert( The_VTN != NULL );
	Volume_Tree_Node *to_return = The_VTN;

	if ( !The_Matrix.IsIdentity() )


	{

#if 0 // remplace le dynamic cast
		Volume_Tree_Matrix *tried_VTNM = dynamic_cast<Volume_Tree_Matrix*>(The_VTN);
		if ( tried_VTNM != NULL )
#else
		Volume_Tree_Matrix *tried_VTNM = (Volume_Tree_Matrix*)(The_VTN);
		int cOK = tried_VTNM->castOK;
		if( cOK == CAST_OK) 
#endif

		{
			tried_VTNM->Set_Matrix(The_Matrix * tried_VTNM->Get_Matrix());


		}
		else
			to_return = new Volume_Tree_Matrix(The_VTN, The_Matrix);

	}

	return to_return;

}

//*******************************************************************
//This node prevent to have two dual in a row
//
//  By Eric Paquet
//  SGDL Systems inc
//  Date : December 10th, 2001.
//*******************************************************************
Volume_Tree_Node *Verified_Dual(Volume_Tree_Node *The_VTN, const float bbox_ptr[])

{
	assert( The_VTN != NULL );
	Volume_Tree_Node *to_return = The_VTN;

#if 0 // enleve le dynamic cast
	Volume_Tree_Unary_Op *tried_unary_op = dynamic_cast<Volume_Tree_Unary_Op*>(The_VTN);

	if ( tried_unary_op != NULL )

#else
		Volume_Tree_Unary_Op *tried_unary_op = (Volume_Tree_Unary_Op*)(The_VTN);
		int cOK = tried_unary_op->castOK;
		if( cOK == CAST_OK_unary_op) 
#endif
	{
		if ( tried_unary_op->Get_Op() == SGDLNot )

		{
			to_return = tried_unary_op->Get_Children();

		}
		else
			to_return = new Volume_Tree_Unary_Op(SGDLNot, The_VTN, bbox_ptr);
	}
	else
		to_return = new Volume_Tree_Unary_Op(SGDLNot, The_VTN, bbox_ptr);

	return to_return;
}

/********************************************************************
  This function returns the greatest common dividor of
  two numbers a and b.  The Euclid's algorithm is used.

  By : Eric Paquet (SGDL Systems inc.)
  Date : November 27th, 2001
*********************************************************************/
int GCD(int a, int b)
{
    int grand = abs(a);
    int petit = abs(b);

    if ( grand < petit )
    {
	int temp = grand;
	grand = petit;
	petit = temp;
    }

    //GCD(?, 0) = ?
    if ( petit == 0 ) return grand;

    do
    {
	int temp = petit;
	petit = abs(grand - petit);
	grand = temp;
    }
    while(petit > 0);

    return grand;
}

/********************************************************************
  This function returns the least common multiple of two
  number a and b.

  By : Eric Paquet (SGDL Systems inc.)
  Date : November 27th, 2001
*********************************************************************/
int LCM(int a, int b)
{
    if ( a == 0 && b == 0 )
	return 0;
    else
	return (a * b) / GCD(a,b);
}

//*****************************************************************
//	
//  GetDistanceToSurface()
//
//	Author : Simon Lavallee, SGDL Systems
//	Date   : April 9th, 2002
//
//  Input  : double        *Coeff   -> Ptr to the fct coeffs
//           int           NbCoeff  -> Number of coeff (10 or 35)
//           double        Normal   -> Vector to project the point
//           const double& x        -> X coordinate
//           const double& y        -> Y coordinate
//           const double& z        -> Z coordinate
//
//  Output : double        Distance -> Approximative distance between a
//                                     given point and the surface of the
//                                     implicit function (signed).
//
//  Desc   : This function calculate the distance between a point and
//           the surface of the implicit function. Note: The distance 
//           returned has the same sign than the field value of the 
//           function at the given point, this is used to know if the 
//           point is inside or outside the implicit shape.
//*****************************************************************
double GetDistanceToSurface(double *Coeff, int NbCoeff, double *Normal, double FieldValue,
                            const double& x, const double& y, const double &z)
{    
    double LastFieldValue = 0.0;

    //The two functions Dlz2_coeR and Dlz4_coeR have the same proto, so
    //I just keep a general pointer to the good function to avoid repeating
    //the same code twice.
#if 0 //AM
	double ((*FieldValueFunc)(double*, double, double, double, int=0))
                          = (NbCoeff == K_QuadricNumberOfCoeff) ? DLz2_coeR : 
                                                                  DLz4_coeR;
#else
	double ((*FieldValueFunc)(const double [], double, double, double, int=0))
                          = (NbCoeff == K_QuadricNumberOfCoeff) ? DLz2_coeR : 
                                                                  DLz4_coeR;

#endif
    //Sign of the initial value (used to know if the points is in or outside).
    double Sign           = (FieldValue >= 0.0) ? 1.0 : -1.0;
    double InitSign       = Sign;

    //The way I calculate the step is a little bit strange but it works...
    //but there is probably a better way that is more optimal.
    //I first check the field value difference between two points that are at a distance
    //of 1.
    double FieldValueDiff = fabs(FieldValue - FieldValueFunc(Coeff, x + Normal[0], 
                                                                    y + Normal[1], 
                                                                    z + Normal[2]));
    //I used this factor as first step, multiplied by a magic number 2 ??? 
    //(this should be checked...)
    double Step           = (fabs(FieldValue) / FieldValueDiff) * 2;

    double ProjX          = x;
    double ProjY          = y;
    double ProjZ          = z;

    //Iteration counter to prevent infinite loop.
    int    Cpt            = 0;    
    
    //The algorithm used to project the point to the surface is the same as the
    //function project_s() in the Luis Vehlo adaptive polygonizer. 
    //We move the points along the vector with a constant step an when we cross 
    //the surface, we move to the reverse direction and divide the step by two,
    //and so on until we've reached the surface.
    do
    {   
        //Keep the last field value
        LastFieldValue = FieldValue;

        //Check if the sign (direction) has changed
        Sign           = (FieldValue >= 0.0) ? 1.0 : -1.0;

        //Move the point along the gradient
        ProjX += Normal[0]*Step*Sign;
        ProjY += Normal[1]*Step*Sign;
        ProjZ += Normal[2]*Step*Sign;

        //Get the field value of the point
        FieldValue = FieldValueFunc(Coeff, ProjX, ProjY, ProjZ);

        if ((LastFieldValue * FieldValue) < 0)
            //If we crossed the surface in the last step, reduce the step by 1/2.
            Step *= 0.5;
        
        else if (fabs(FieldValue) > fabs(LastFieldValue))
        {   
            //This case happens only when we cannot cross the surface
            //by projecting the point along the gradient
            //(very rare, but possible... ?)
            
            //Return the last position
            ProjX -= Normal[0]*Step*Sign;
            ProjY -= Normal[1]*Step*Sign;
            ProjZ -= Normal[2]*Step*Sign;

            //Force the the algorithm to stop here
            FieldValue = 0;
        }                

        if (++Cpt > 100)
        {
            //This case prevents from infinite loop since
            //it's impossible that it takes more than 100
            //iterations to reached to the surface...        
            assert(Cpt < 100);

            //return the field value instead of the distance
            return FieldValueFunc(Coeff, x, y, z);
        }
    }
    while (!EpsilonEqual(FieldValue, 0.0));

    //Return the distance between the surface and the point, multiplied
    //by the sign of the original point (used to know if it is inside 
    //or outside)
    return sqrt((ProjX - x)*(ProjX - x) + 
                (ProjY - y)*(ProjY - y) + 
                (ProjZ - z)*(ProjZ - z)) * InitSign;
}

//********************************************************************
//  Function:  DensityMatrix::CheckDensity()
//
//  Input:     int resulEval : F(x) bring to [0 1 2]
//             DensityArray densArray : The array of density
//  Output:    If resulEval define in densArray.
//  
//  Author:     Eric Baillargeon SGDL Systme Inc.
//  Created:    2001/10/25	
//  Modified:   Simon Lavallee (17-04-2002)
//      
//*********************************************************************/
bool CheckDensity(int resulEval, DensityArray& currentDensity)
{
    if (resulEval == 0)
        return FALSE;
    
    for (int i = 0 ; i < currentDensity.NumberOfDensity; i++)
    {
        if(currentDensity.values[i] == resulEval)
        {
            return TRUE;
        }
    }

    return FALSE;    
}

//EOF********************************************************************