#ifndef _SOLID_TOOLS_
#define _SOLID_TOOLS_

#include "Opengl/AFX/Solid/stdafx.h"



#include "VolumeTreeNode.h"
#include "VolTreeConst.h"
#include "VolTreeBinOp.h"
#include "VolTreeUnOp.h"
#include "VolTreeMulOp.h"
#include "VolTreeMatrix.h"


#include "SolidNode.h"
#include "../DensityTLS.h"


//define
#define     EPSILON             0.001
#define     INVALID_DISTANCE     -1.0

//Global variables
static ZArray<double>    Density_Tab;

//The global grid determine the size of the grid used
//by the JB polygonizer. This size can be modified if the volume
//is too small (see method DensityMatrix::GetAssociatedBBox() 
//in DensityTLS.cpp.
static float            Global_Grid  = 0.2;

//Used by the Kleene operator
const  int              WHICH_AIRE   = 3;

//constants
const int   K_PlanarNumberOfCoeff   = 4;
const int   K_QuadricNumberOfCoeff  = 10;
const int   K_QuarticNumberOfCoeff  = 35;
const int   K_DensityDefaultMaximum = 100;
const int   K_DensityDefaultValues  = -1;

//Fonctions
void    Globalize_First_BBox(float First[], const float Second[]);

//Correct the density if outside the bbox
void    Bounding_Box_Correction(const float bbox[],
		       				    int   *to_return,
							    const double &x,
							    const double &y,
							    const double &z);


//Correct the gradient and the distance if outside the bbox
void    Bounding_Box_Gradient_Correction(const float bbox[],
                                         double *Gradient,
                                         double *Distance,
                                         const double& x,
                                         const double& y,
                                         const double& z);

//Approximative <=, >= and ==.
bool    EpsilonLessOrEqual(double Value1, double Value2);
bool    EpsilonGreaterOrEqual(double Value1, double Value2);
bool    EpsilonEqual(double Value1, double Value2);

//Return the distance between a given point and the surface defined
//by Coeff (by projecting along the gradient).
double  GetDistanceToSurface(double *Coeff, int NbCoeff, double *Normal, double FieldValue, 
                             const double& x, const double& y, const double &z);

//Multiply SL_Matrix by a point
void    Multiply_Mat_Double(const SL_Matrix& M, double x,	double y, double z,
		  				    double *in_x, double *in_y, double *in_z);

//Return true if the_color_to_test == -1 in RGB
bool    Undefined_Color(const float the_color_to_test[]);

//Add a matrix transformation to a Volume_Tree_Node
Volume_Tree_Node *Verified_Volume_Tree_Matrix(const SL_Matrix& The_Matrix, Volume_Tree_Node *The_VTN);

//Prevent from two dual in a row
Volume_Tree_Node *Verified_Dual(Volume_Tree_Node *The_VTN, const float bbox_ptr[]);

//Great Common Divisor
int     GCD(int a, int b);
//Low Common Multiple
int     LCM(int a, int b);

//Check if we have to show the specified density
bool CheckDensity(int resulEval, DensityArray& currentDensity);

//*******************************************************
//*******************************************************
//  Frustum culling utility enum and function
//*******************************************************
//*******************************************************

enum eInterResult
{
    eOUTSIDE = 0,
    eINTERSECT,
    eINSIDE
};

//SWAP two value
inline void SWAP(double V1, double V2)
{
    double l_Temp = V1;
    V1 = V2;
    V2 = l_Temp;
}

//********** 3D functions ***************************
/* AM + tard
//return true if the quadric inQ intersect or is inside the hyperplane inP. If
//the intersection is real, then it is returned in outInter and the projected axis
//is returned in outProjectedAxis.
eInterResult InterQuadricPlane(const double inQ[10], const double inP[4],
                               double outInter[6], eAxis *outProjectedAxis);

//return true if a quadric surface is entirely inside a plane 
//or an hyperplane surface is entirely inside a quadric.
bool    IsQuadricEntirelyInsidePlane(const double inQ[10], const double inP[4]);

//return the line that result in a plane intersection or false if the planes
//are parallel (intersection is imaginary).
bool    PlaneIntersection(const double P1[4], const double P2[4], double L[3], 
                          eAxis inProjectedAxis );

//return the dot product of two points in 3D.
double  DotProduct(const double inPt1[3], const double inPt2[3]);
*/

//********** 2D functions ***************************

//return true if the quadratic curve intersect or is entirely inside a line
eInterResult InterQuadraticLine(const double inQuadratic[6], const double inLine[3]);

//return true if the quadratic curve is entirely inside a line
bool    IsQuadraticEntirelyInsideLine(const double inQ[6], const double inL[3]);

//return the field value of a quadratic 2D function at a given point in 2D
double  QuadraticFieldValue(const double inQ[6], double X, double Y);

//return the gradient (normal) vector to the quadratic function at a given point in 2D
void    QuadraticGradient(const double c[6], double X, double Y, double *outGradient);

//return the dot product of two points in 2D
double  DotProduct2D(const double inPt1[2], const double inPt2[2]);

#endif //_SOLID_TOOLS_
