#ifndef _VOL_TREE_BIN_OP_H_
#define _VOL_TREE_BIN_OP_H_

class Volume_Tree_Node;

enum SGDLBinaryOperator
{
    //KLEENE
    SGDLAnd,
    SGDLOr,
    SGDLImplication,
    SGDLReciprocal,
    //ARITHMETIC
    SGDLPlus,
    SGDLMinus,
    SGDLMulti,
    SGDLExpo,
    SGDLPGCD,
    SGDLPPCM,
    SGDLMod,
    SGDLAbsMinus,
    SGDLMax,
    SGDLMin,
    //TEST
    SGDLEquality,
    SGDLGreaterOrEqual,
    SGDLGreater,
    SGDLLessOrEqual,
    SGDLLess,
    SGDLDifference
};

//******************************************************
// Class Volume_Tree_Binary_Op -> This class represent
// the binary operations in a volume tree.
//******************************************************
class Volume_Tree_Binary_Op : public Volume_Tree_Node
{
public:
    Volume_Tree_Binary_Op(SGDLBinaryOperator,
                          const Volume_Tree_Node *,
                          const Volume_Tree_Node *);

	Volume_Tree_Binary_Op(SGDLBinaryOperator,
                          const Volume_Tree_Node *,
                          const Volume_Tree_Node *,
                          const float bbox_ptr[]);

    Volume_Tree_Binary_Op(const Volume_Tree_Binary_Op &The_Other):
    
    Volume_Tree_Node(false)
    {
	    the_Operation = The_Other.the_Operation;
	    v1 = The_Other.v1;
	    v2 = The_Other.v2;
    }

    //SL- In the minimodeleur, the evaluate density is implemented once
    //for all the type of node in the Volume_Tree_Node class (see VolumeTreeNode.h).
    int   EvaluateDensity(const double& x, const double& y, const double& z)const;
    
    bool    EvaluateGradient(const double &x,
                             const double &y,
                             const double &z,
                             double       &Distance,
                             double       *Gradient,
                             float        *Color = 0) const;

    bool    GetColor(const double &x, 
                     const double &y, 
                     const double &z, 
                     float set_color[])const;

    double *GetCoeff(double x, double y , double z);
    int    GetNumOfCoeff(double x, double y , double z);    
    
    const  Volume_Tree_Binary_Op& operator=(const Volume_Tree_Binary_Op& The_Other);
    
    SGDLBinaryOperator Get_Op(void) const {return the_Operation;}    
    void Set_Op(SGDLBinaryOperator the_other) {the_Operation = the_other;}
    void GetPrimitives(ZArray<BoundingQuadric*> &outPrimitives, const SL_Matrix &Transform, BBox &outGlobalBBox);
    
    void Ref();
    void UnRef();

    void SetGraphic( SolidGraphic *graphic);
	SolidGraphic	*GetGraphic();

	bool Render(Matrix &rMatrix);
protected:
    //Protected member
    SGDLBinaryOperator the_Operation;
    Volume_Tree_Node   *v1;
    Volume_Tree_Node   *v2;

	void Manage_At_The_Top();
    ~Volume_Tree_Binary_Op(void) {}

		SolidGraphic	*Graphic;

private:
};

#endif //_VOL_TREE_BIN_OP_H_