#ifndef _VOL_TREE_MATRIX_H_
#define _VOL_TREE_MATRIX_H_

class Volume_Tree_Node;
class SL_Matrix;


#define CAST_OK 0xFFF

//******************************************************
// Class Volume_Tree_Binary_Op -> This class represent
// the matrix transformations in a volume tree.
//******************************************************
class Volume_Tree_Matrix : public Volume_Tree_Node
{ 
public:
    Volume_Tree_Matrix(Volume_Tree_Node *The_Node,
					   const SL_Matrix &the_transfo = IdentityMatrix());

	Volume_Tree_Matrix(Volume_Tree_Node *The_Node,
					   const float bbox_ptr[],
					   const SL_Matrix &the_transfo = IdentityMatrix());

    //SL- In the minimodeleur, the evaluate density is implemented once
    //for all the type of node in the Volume_Tree_Node class (see VolumeTreeNode.h).
    int     EvaluateDensity(const double& x, const double& y, const double& z)const;

    bool    EvaluateGradient(const double &x,
                             const double &y,
                             const double &z,
                             double       &Distance,
                             double       *Normal,
                             float        *Color = 0) const;

    double  *GetCoeff(double x, double y , double z);
    int     GetNumOfCoeff(double x, double y , double z);

    bool    GetColor(const double &x, 
                     const double &y, 
                     const double &z, 
                     float set_color[])const;

    void    GetPrimitives(ZArray<BoundingQuadric*> &outPrimitives,
                          const SL_Matrix &Transform, BBox &outGlobalBBox);

    SL_Matrix  Get_Matrix(void) const {return The_Matrix;}	
    void    Set_Matrix(const SL_Matrix& the_other)
    {
        The_Matrix = the_other;
        The_Matrix_Inverted = Invert(The_Matrix);
    }

    void    Ref();
    void    UnRef();

    Volume_Tree_Node *v;
	int castOK; // pour remplacer le dynamic cast

  	void SetGraphic( SolidGraphic *graphic);
	SolidGraphic	*GetGraphic();

	bool Render(Matrix & rMatrix);




protected:

    //Protected member   
    SL_Matrix The_Matrix;   
    SL_Matrix The_Matrix_Inverted;

    void   Manage_At_The_Top();

    Volume_Tree_Matrix(void);	
    ~Volume_Tree_Matrix(void){}
    void Minimize(Volume_Tree_Node *The_Node);

		SolidGraphic	*Graphic;

};


#endif //_VOL_TREE_MATRIX_H_