//*******************************************************************
//Volume_Tree_Multiple_Op
//*******************************************************************

#include "../stdafx.h"
#include <assert.h>
#include "Opengl/AFX/Solid/VolumeTreeNode/SolidTools.h"

//*******************************************************************
//  Volume_Tree_Multiple_Op constructor 
//
//  By Eric Paquet
//  SGDL Systems inc
//  Date : December 10th, 2001.
//*******************************************************************
Volume_Tree_Multiple_Op::Volume_Tree_Multiple_Op(SGDLMultipleOperator the_op,
												 const ZArray<Volume_Tree_Node*>& Other_volumes)
{    
	the_Operation = the_op;

	for ( int i = 0; i < Other_volumes.GetLength(); i++ )
// AM		volumes.Append(Other_volumes[i]);
		volumes += Other_volumes[i];
}

//*******************************************************************
//  Volume_Tree_Multiple_Op constructor 
//
//  By Eric Paquet
//  SGDL Systems inc
//  Date : December 10th, 2001.
//*******************************************************************
Volume_Tree_Multiple_Op::Volume_Tree_Multiple_Op(SGDLMultipleOperator the_op,
												 const ZArray<Volume_Tree_Node*>& Other_volumes,
												 const float bbox_ptr[]):
Volume_Tree_Node(bbox_ptr)
{
	the_Operation = the_op;

	for ( int i = 0; i < Other_volumes.GetLength(); i++ )
// AM		volumes.Append(Other_volumes[i]);
	volumes += Other_volumes[i];
}

//*******************************************************************
//  This function returns the density to the point x, y, z 
//
//  By Eric Paquet
//  SGDL Systems inc
//  Date : December 10th, 2001.
//  Modified: Simon Lavallee (added case for modeleur node)
//  Date : April 15th, 2002
//*******************************************************************
int Volume_Tree_Multiple_Op::EvaluateDensity(const double& x, const double& y, const double& z)const
{
    if(At_The_Top && !isInsideBoundingVolume(x, y, z))
        return 0;

    int to_return = 0;
    DensityArray temp = currentDensity;

	switch(the_Operation)
	{
		case SGDLMOr :
//		case SGDLOr :
		{
			//Verify all the nodes
			//Opyimization : if we get one true ve stop
			for ( int i = 0; i < volumes.GetLength(); i++ )
			{   
		        currentDensity.values[0] = -1;
				if ( volumes[i]->isInside(x, y, z) )
				{
					to_return = true;
					break;
				}
			}
			break;
		}
		default: 
		case SGDLMAnd :
//		case SGDLAnd :
	{
			//Verify all the nodes
			//Opyimization : if we get one !true ve stop
			to_return = true;
			for ( int i = 0; i < volumes.GetLength(); i++ )
			{   
		        currentDensity.values[0] = -1;
				if ( !volumes[i]->isInside(x, y, z) )
				{
					to_return = false;
					break;
				}
			}
			break;
		}
#if 0
		default:
		{
			assert(false); //Do not know this operation
		}
#endif
	}

	if ( At_The_Top )
		//This call may modify to_return
		Bounding_Box_Correction(bbox, &to_return, x, y, z);
	
	return to_return;
}

//*******************************************************************
//  Volume_Tree_Multiple_Op::GetPrimitives()
//
//  By Simon Lavallee
//  SGDL Systems inc
//  Date : December 17th, 2002.
//*******************************************************************
void Volume_Tree_Multiple_Op::GetPrimitives(ZArray<BoundingQuadric*> &outPrimitives,
                                            const SL_Matrix &Transform, BBox &outGlobalBBox)
{
    for ( int i = 0; i < volumes.GetLength(); i++ )
    {   
        volumes[i]->GetPrimitives(outPrimitives,Transform, outGlobalBBox);
    }

    if(bbox != NULL)
    {
        BBox l_Bbox;
        l_Bbox.ma.Set(bbox[0] / 2.0, bbox[1] / 2.0, bbox[2] / 2.0);
        l_Bbox.ma.Set(-bbox[0] / 2.0, -bbox[1] / 2.0, -bbox[2] / 2.0);

        if(!Transform.IsIdentity())
            l_Bbox *= Transform;

        outGlobalBBox.Union(l_Bbox);
    }
}

//***************************************************
//	
//  Volume_Tree_Multiple_Op::EvaluateGradient()
//
//	Author : Simon Lavallee, SGDL Systems
//	Date   : March 27th, 2002
//
//  Input  : double x           -> X coordinate
//           double y           -> Y coordinate
//           double z           -> Z coordinate
//
//  Output : double Distance    -> Distance between the point and the surface
//           float  Color       -> Color of the point
//           double *Normal     -> Normal of the point
//           bool               -> true if we've been able to get
//                                 the normal.
//
//  Desc   : This function compute the normal of each
//           of its volumes and keep the one that the 
//           distance to the surface is the nearest of 0.
//***************************************************
bool Volume_Tree_Multiple_Op::EvaluateGradient(const double& x, const double& y, const double& z, 
                                               double& Distance, double* Normal, float *Color) const
{
    double TempDistance      = INVALID_DISTANCE;
    double TempNormal[3]     = {0.0, 0.0, 0.0};
    float  *TempColor        = (Color) ? new float[3] : NULL;
    bool   TempNormalIsValid = false;

    //Get normal, distance and color of the first volume
    bool   NormalIsValid = volumes[0]->EvaluateGradient(x, y, z, Distance, Normal, Color);    

    for (int i = 1; i < volumes.GetLength(); i++ )
        {
        //Get the normal, distance and color of each volume
        TempNormalIsValid = volumes[i]->EvaluateGradient(x, y, z, TempDistance, TempNormal, TempColor);

        //If we've been able to compute the normal, check for the min distance
        //between the point and a surface.
        if ((!NormalIsValid && TempNormalIsValid) ||
            ((NormalIsValid && TempNormalIsValid) &&
             (fabs(TempDistance) < fabs(Distance))))
        {
            //Keep the information from the volume that has the min distance
            //between his surface and the point.
            Normal[0] = TempNormal[0];
            Normal[1] = TempNormal[1];
            Normal[2] = TempNormal[2];
            
            Distance  = TempDistance;

            if (Color)
            {
                Color[0] = TempColor[0];
                Color[1] = TempColor[1];
                Color[2] = TempColor[2];
            }

            NormalIsValid = TempNormalIsValid;
        }            
    }

    if (At_The_Top && NormalIsValid)
    {
        //The bounding box can modify the gradient and the field value...
        Bounding_Box_Gradient_Correction(bbox, Normal, &Distance, x, y, z);        
    }

    if (Color && NormalIsValid)
    {
        delete [] TempColor;
        
        if (!Undefined_Color(color))
        {
            //If this node has its own color, keep it.
            Color[0] = color[0];
            Color[1] = color[1];
            Color[2] = color[2];
        }
    }
    
    return NormalIsValid;
}

//*******************************************************************
//  This function returns the coeffs of the VTN that has x, y, z inside
//
//  By Eric Baillargeon
//  SGDL Systems inc
//  Date : December 10th, 2001.
//*******************************************************************
double * Volume_Tree_Multiple_Op::GetCoeff(double x, double y , double z)
{
    double* tempCoeff = NULL;
    for ( int i = 0; i < volumes.GetLength(); i++ )
    {	
	    if (volumes[i]->isInsideWithoutDensityCheck(x,y,z))
	    {
	        tempCoeff = volumes[i]->GetCoeff(x,y,z);
	        break;
	    }
    }

    //Set the coeffs to 0.0
    if (tempCoeff == NULL)
    {
	    tempCoeff = new double[K_QuadricNumberOfCoeff];
	    for (int i = 0; i < K_QuadricNumberOfCoeff ; i++)
	    {
	        tempCoeff[i] = 0.0;
	    }
    }
    return tempCoeff;
}

//*******************************************************************
//  This function 
//
//  By Eric Baillargeon
//  SGDL Systems inc
//  Date : December 10th, 2001.
//*******************************************************************
int Volume_Tree_Multiple_Op::GetNumOfCoeff(double x, double y , double z)
{
    int tempCoeff = 0;
    for ( int i = 0; i < volumes.GetLength(); i++ )
    {	
	    if (volumes[i]->isInsideWithoutDensityCheck(x,y,z))
	    {
	        tempCoeff = volumes[i]->GetNumOfCoeff(x,y,z);
	        break;
	    }
    }

    //Sets it to 10 by default
    if (tempCoeff == 0x0000)
    {
	    tempCoeff = K_QuadricNumberOfCoeff;
    }
    return tempCoeff;

}

//*******************************************************************
//  This function return the color of one point based on the fact that
//  is or not inside.
//
//  By Eric Paquet
//  SGDL Systems inc
//  Date : December 10th, 2001.
//  Modified : Simon Lavallee (now just get the color)
//  Date : April 15th 2002
//*******************************************************************
bool Volume_Tree_Multiple_Op::GetColor(const double &x, 
                                       const double &y, 
                                       const double &z, 
                                       float set_color[])const
{
	float temp_color[]  = {-1.0, -1.0, -1.0};	
    bool  inside_volume = false;

    //Ensure the set_color is uninitialized
    set_color[0] = -1.0;
    set_color[1] = -1.0;
    set_color[2] = -1.0;

    //Iter on all the VTN in the array
	for (int i = 0; i < volumes.GetLength(); i++ )
	{        
		inside_volume = volumes[i]->GetColor(x, y, z, temp_color) || inside_volume;

		if ( !Undefined_Color(temp_color) )
		{
			if (Undefined_Color(set_color))
            {
                //Get the color
                set_color[0] = temp_color[0];
                set_color[1] = temp_color[1];
                set_color[2] = temp_color[2];
            }
            else
            {
                //If the point is inside more than one volume,
                //we can't determine which color is the
                //good one.

                //Reset color to undefined
                set_color[0] = -1.0;
                set_color[0] = -1.0;
                set_color[0] = -1.0;

                //Force the loop to stop here
                i = volumes.GetLength();
            }
		}
    }
	
	if (!Undefined_Color(color) && (inside_volume || At_The_Top))
	{
		//If this node has its own color, keep it.
        set_color[0] = color[0];
		set_color[1] = color[1];
		set_color[2] = color[2];
	}

    return inside_volume;
}

//*******************************************************************
//  This function manage the variable At_The_Top for all the VTN in
//  the array
//
//  By Eric Paquet
//  SGDL Systems inc
//  Date : December 10th, 2001.
//*******************************************************************
void Volume_Tree_Multiple_Op::Manage_At_The_Top()
{
	At_The_Top = true;

	//Set the other one to false
	for ( int i = 0; i < volumes.GetLength(); i++ )
	{
		volumes[i]->At_The_Top = false;
	}
}

//*******************************************************************
//  This function adds a reference to a Volume_Tree_Multiple_Op
//
//  By Eric Paquet
//  SGDL Systems inc
//  Date : December 10th, 2001.
//*******************************************************************
void Volume_Tree_Multiple_Op::Ref()
{
    ref++;
    for(int i = 0; i < volumes.GetLength(); i++)
        volumes[i]->Ref();
}

//*******************************************************************
//  This function removes a reference to a Volume_Tree_Multiple_Op
//
//  By Eric Paquet
//  SGDL Systems inc
//  Date : December 10th, 2001.
//*******************************************************************
void Volume_Tree_Multiple_Op::UnRef()
{
    ref--;
    for(int i = 0; i < volumes.GetLength(); i++)
        volumes[i]->UnRef();

    if ( ref == 0 )
        delete this;
}

//*******************************************************************
//  This function set the graphic representation to the SolidNode 
//
//  By Alain Mignot
//  Mignot Informatique Graphique Inc.
//  Date : may 2004
//*******************************************************************
	void Volume_Tree_Multiple_Op::SetGraphic( SolidGraphic *Igraphic)
	{
		Graphic = Igraphic;
	}


//*******************************************************************
//  This function Get the graphic representation from the SolidNode 
//
//  By Alain Mignot
//  Mignot Informatique Graphique Inc.
//  Date : may 2004
//*******************************************************************
	SolidGraphic	*Volume_Tree_Multiple_Op::GetGraphic()
	{
		return( Graphic);
	}

//*******************************************************************
//  This function displays the graphic representation of the SolidNode 
//
//  By Alain Mignot
//  Mignot Informatique Graphique Inc.
//  Date : may 2004
//*******************************************************************
	bool Volume_Tree_Multiple_Op::Render(Matrix &rMatrix)
	{



	return FALSE;


	}
//EOF****************************************************************