#ifndef _VOL_TREE_MUL_OP_H_
#define _VOL_TREE_MUL_OP_H_

class Volume_Tree_Node;

enum SGDLMultipleOperator
{
		SGDLMAnd,
		SGDLMOr

};

//******************************************************
// Class Volume_Tree_Binary_Op -> This class represent
// the matrix transformations in a volume tree.
//******************************************************
class Volume_Tree_Multiple_Op : public Volume_Tree_Node
{
public:
    Volume_Tree_Multiple_Op(SGDLMultipleOperator,
                            const ZArray<Volume_Tree_Node*>&);

	Volume_Tree_Multiple_Op(SGDLMultipleOperator,
                            const ZArray<Volume_Tree_Node*>&,
                            const float bbox_ptr[]);

    //SL- In the minimodeleur, the evaluate density is implemented once
    //for all the type of node in the Volume_Tree_Node class (see VolumeTreeNode.h).
    int     EvaluateDensity(const double& x, const double& y, const double& z)const;

    bool    EvaluateGradient(const double &x,
                             const double &y,
                             const double &z,
                             double       &Distance,
                             double       *Gradient,
                             float        *Color = 0) const;

    double  *GetCoeff    (double x, double y, double z);
    int     GetNumOfCoeff(double x, double y, double z);

    bool    GetColor(const double &x, 
                     const double &y, 
                     const double &z, 
                     float set_color[])const;

    void    GetPrimitives(ZArray<BoundingQuadric*> &outPrimitives,
                          const SL_Matrix &Transform, BBox & outGlobalBBox);
     
    void    Ref();
    void    UnRef();

     void SetGraphic( SolidGraphic *graphic);
	SolidGraphic	*GetGraphic();

	bool Render(Matrix &rMatrix);
protected:
    //Protected member
    SGDLMultipleOperator     the_Operation;
    ZArray<Volume_Tree_Node*> volumes;

	void    Manage_At_The_Top();
    ~Volume_Tree_Multiple_Op(void){}

		SolidGraphic	*Graphic;

private:
};

#endif //_VOL_TREE_MUL_OP_H_