#ifndef _VOL_TREE_UN_OP_H_
#define _VOL_TREE_UN_OP_H_

class Volume_Tree_Node;

enum SGDLUnaryOperator
{
    //KLEENE
	SGDLNot,
    //Arithmetic
    SGDLSquareRoot,
    SGDLCubeRoot,
    //TEST
    SGDLEven,
    SGDLOdd
};

//******************************************************
// Class Volume_Tree_Binary_Op -> This class represent
// the unary operations in a volume tree.
//******************************************************
class Volume_Tree_Unary_Op : public Volume_Tree_Node
{
public:
    Volume_Tree_Unary_Op(SGDLUnaryOperator,
                         const Volume_Tree_Node *);

	Volume_Tree_Unary_Op(SGDLUnaryOperator,
                         const Volume_Tree_Node *,
                         const float bbox_ptr[]);

    Volume_Tree_Unary_Op(const Volume_Tree_Unary_Op &The_Other)    
    {
	    the_Operation = The_Other.the_Operation;
	    v = The_Other.v;
    }



    //SL- In the minimodeleur, the evaluate density is implemented once
    //for all the type of node in the Volume_Tree_Node class (see VolumeTreeNode.h).
    int     EvaluateDensity(const double& x, const double& y, const double& z)const;

    bool    EvaluateGradient(const double &x,
                             const double &y,
                             const double &z,
                             double       &Distance,
                             double       *Gradient,
                             float        *Color = 0) const;

    double  *GetCoeff(double x, double y , double z);
    int     GetNumOfCoeff(double x, double y , double z);

    bool    GetColor(const double &x, 
                     const double &y, 
                     const double &z, 
                     float set_color[])const;
    
    const   Volume_Tree_Unary_Op& operator=(const Volume_Tree_Unary_Op& The_Other);

	SGDLUnaryOperator Get_Op(void) const        {return the_Operation;}
    void    Set_Op(SGDLUnaryOperator the_other) {the_Operation = the_other;}
	
    Volume_Tree_Node *Get_Children(void)        {return v;}
    void    GetPrimitives(ZArray<BoundingQuadric*> &outPrimitives,
                          const SL_Matrix &Transform, BBox &outGlobalBBox);

    void    Ref();
    void    UnRef();

	void SetGraphic( SolidGraphic *graphic);
	SolidGraphic	*GetGraphic();

	bool Render(Matrix &rMatrix);
	int castOK; // pour remplacer le dynamic cast

protected:

    //Protected member
    SGDLUnaryOperator the_Operation;
    Volume_Tree_Node  *v;

	void    Manage_At_The_Top();
    ~Volume_Tree_Unary_Op(void) {}

		SolidGraphic	*Graphic;
	int m_displaylist;
private:
};

#endif //_VOL_TREE_UN_OP_H_