//*******************************************************************
//Volume_Tree_Node
//*******************************************************************

#include "../stdafx.h"
#include <assert.h>
#include "Opengl/AFX/Solid/VolumeTreeNode/SolidTools.h"

const float Volume_Tree_Node::f_JuleError = 0.00001;
DensityArray Volume_Tree_Node::currentDensity;

//*******************************************************************
//  Volume_Tree_Node constructor 
//
//  By Eric Paquet
//  SGDL Systems inc
//  Date : December 10th, 2001.
//*******************************************************************
Volume_Tree_Node::Volume_Tree_Node(const float bbox_ptr[]):
ref(0)
{
	assert( bbox_ptr != NULL );
	bbox = new float[3];
	Set_Bbox(bbox_ptr);
	float temp[] = {-1.0, -1.0, -1.0};
	Set_Color(temp);
    ShowAllDensities = true;
    m_BoundingVolume = NULL;
		m_displaylist = 0;
}

//*******************************************************************
//  Volume_Tree_Node constructor 
//
//  By Eric Paquet
//  SGDL Systems inc
//  Date : December 10th, 2001.
//*******************************************************************
Volume_Tree_Node::Volume_Tree_Node():
ref(0)
{
	bbox = NULL;
	float temp[] = {-1.0, -1.0, -1.0};
	Set_Color(temp);
    ShowAllDensities = true;
    m_BoundingVolume = NULL;
		m_displaylist = 0;
}

//*******************************************************************
//  Volume_Tree_Node destructor 
//
//  By Eric Paquet
//  SGDL Systems inc
//  Date : December 10th, 2001.
//*******************************************************************
Volume_Tree_Node::~Volume_Tree_Node(void)
{
	if ( bbox != NULL && ref == 0)
	{
		delete[] bbox;
	}
}

//*******************************************************************
//  This function sets the member currentDensity
//
//  By Eric Baillargeon
//  SGDL Systems inc
//  Date : December 10th, 2001.
//*******************************************************************
void  Volume_Tree_Node::init()
{
    currentDensity.SetDefault(K_DensityDefaultMaximum,K_DensityDefaultValues);
}

//*******************************************************************
//  This function sets the bbox of s VTN 
//
//  By Eric Paquet
//  SGDL Systems inc
//  Date : December 10th, 2001.
//*******************************************************************
void Volume_Tree_Node::Set_Bbox(const float bbox_ptr[])
{
	if ( bbox == NULL )
	{
		bbox = new float[3];
	}

	bbox[0] = bbox_ptr[0];
	bbox[1] = bbox_ptr[1];
	bbox[2] = bbox_ptr[2];
}

//*******************************************************************
//  This function sets the color of a VTN
//
//  By Eric Paquet
//  SGDL Systems inc
//  Date : December 10th, 2001.
//*******************************************************************
void Volume_Tree_Node::Set_Color(const float to_set[])
{
	color[0] = to_set[0];
	color[1] = to_set[1];
	color[2] = to_set[2];
}

//*******************************************************************
//  This function manages the operation + between two VTNs
//
//  By Eric Paquet
//  SGDL Systems inc
//  Date : December 10th, 2001.
//*******************************************************************
Volume_Tree_Binary_Op& Volume_Tree_Node::operator+(const Volume_Tree_Node &The_Other)const
{
	Volume_Tree_Binary_Op *to_return = new Volume_Tree_Binary_Op(SGDLPlus, this, &The_Other);
	return *to_return;
}

//*******************************************************************
//  This function manages the operation * between two VTNs 
//
//  By Eric Paquet
//  SGDL Systems inc
//  Date : December 10th, 2001.
//*******************************************************************
Volume_Tree_Binary_Op& Volume_Tree_Node::operator*(const Volume_Tree_Node &The_Other)const
{
	Volume_Tree_Binary_Op *to_return = new Volume_Tree_Binary_Op(SGDLMulti, this, &The_Other);
	return *to_return;
}

//*******************************************************************
//  This function manages the operation - between two VTNs 
//
//  By Eric Paquet
//  SGDL Systems inc
//  Date : December 10th, 2001.
//*******************************************************************
Volume_Tree_Binary_Op& Volume_Tree_Node::operator-(const Volume_Tree_Node &The_Other)const
{
	Volume_Tree_Binary_Op *to_return = new Volume_Tree_Binary_Op(SGDLMinus, this, &The_Other);
	return *to_return;
}

//*******************************************************************
//  This function manages the operation & between two VTNs
//
//  By Eric Paquet
//  SGDL Systems inc
//  Date : December 10th, 2001.
//*******************************************************************
Volume_Tree_Binary_Op& Volume_Tree_Node::operator&(const Volume_Tree_Node &The_Other)const
{
	Volume_Tree_Binary_Op *to_return = new Volume_Tree_Binary_Op(SGDLAnd, this, &The_Other);
	return *to_return;
}

//*******************************************************************
//  This function manages the operation | between two VTNs 
//
//  By Eric Paquet
//  SGDL Systems inc
//  Date : December 10th, 2001.
//*******************************************************************
Volume_Tree_Binary_Op& Volume_Tree_Node::operator|(const Volume_Tree_Node &The_Other)const
{
	Volume_Tree_Binary_Op *to_return = new Volume_Tree_Binary_Op(SGDLOr, this, &The_Other);
	return *to_return;
}

//*******************************************************************
//  This function manages the operation ! on a VTN 
//
//  By Eric Paquet
//  SGDL Systems inc
//  Date : December 10th, 2001.
//*******************************************************************
Volume_Tree_Unary_Op &Volume_Tree_Node::operator!()
{
	Volume_Tree_Unary_Op *to_return = NULL;
	
	to_return = new Volume_Tree_Unary_Op(SGDLNot, this);
	
	return *to_return;
}

//*******************************************************************
//  This function adjusts x, y or z to the current bbox if they are outside
//
//  By Eric Paquet
//  SGDL Systems inc
//  Date : December 10th, 2001.
//*******************************************************************
void Volume_Tree_Node::Adjust_With_BBox(double& x, double& y, double& z)const
{
	if ( bbox != NULL && At_The_Top)
	{
		double xdiv2 = bbox[0]/2.0;
	    if (x > xdiv2 )
		{	     
		    x = xdiv2;
		}
	    else if (x < -xdiv2 )
		{
		    x = -xdiv2;
		}

		double ydiv2 = bbox[1]/2.0;
	    if ( y > ydiv2 )
		{
		    y = ydiv2;
		}
	    else if ( y < -ydiv2 )
		{
		    y = -ydiv2;
		}
        
		double zdiv2 = bbox[2]/2.0;
	    if ( z > zdiv2 )
		{
            z = zdiv2;
		}
	    else if ( z < -zdiv2 )
		{
		    z = -zdiv2;
		}
	}
}

/********************************************************************\
Function:  

  Input:     const double& x;
	     const double& y;
	     const double& z;
  Output:    bool :	      if the point is inside the volume;
  
    Author:	Eric Baillargeon SGDL Systme Inc.
    Created:	2001/11/23	
     
      
*********************************************************************/
bool Volume_Tree_Node::isInside(const double& x, const double& y, const double& z)const
{   
    if(!isInsideBoundingVolume(x, y, z))
        return false;

    int resulEval = EvaluateDensity(x,y,z);

    if (ShowAllDensities) 
        return (resulEval > 0);
    else
        return CheckDensity(resulEval ,currentDensity );
}

/********************************************************************\
Function:  

  Input:     const double& x;
	     const double& y;
	     const double& z;
  Output:    bool :	      if the density is different of 0
  
    Author:	Eric Paquet SGDL Systme Inc.
    Created:	2001/11/28	
     
      
*********************************************************************/
bool Volume_Tree_Node::isInsideWithoutDensityCheck(const double& x, const double& y, const double& z)const
{
    return EvaluateDensity(x,y,z) > 0;
}

//********************************************************************\
//Function:  
//
//  Input:     const double& x;
//	     const double& y;
//	     const double& z;
//  Output:    bool :	      if the density is different of 0
//  
//    Author:	Simon Lavallee - SGDL Systems Inc.
//    Created:	2001/11/28
//*********************************************************************/
bool Volume_Tree_Node::isInsideBoundingVolume(const double x, const double y, const double z) const
{
    int  l_BoundingVolumeNb = 0;
    bool l_isInside         = true;
    
    if((m_BoundingVolume != NULL) && ((l_BoundingVolumeNb = m_BoundingVolume->GetLength()) > 0))
    {
        l_isInside = false;
        for(int i = 0; (i < l_BoundingVolumeNb) && !l_isInside; i++)
            l_isInside = ((*m_BoundingVolume)[i])->isInside(x, y, z);
    }
    return l_isInside;
}

//*******************************************************************
//  This function set the graphic representation to the SolidNode 
//
//  By Alain Mignot
//  Mignot Informatique Graphique Inc.
//  Date : may 2004
//*******************************************************************
	void Volume_Tree_Node::SetGraphic( SolidGraphic *Igraphic)
	{
		Graphic = Igraphic;
	}


//*******************************************************************
//  This function Get the graphic representation from the SolidNode 
//
//  By Alain Mignot
//  Mignot Informatique Graphique Inc.
//  Date : may 2004
//*******************************************************************
	SolidGraphic	*Volume_Tree_Node::GetGraphic()
	{
		return( Graphic);
	}

//*******************************************************************
//  This function displays the graphic representation of the SolidNode 
//
//  By Alain Mignot
//  Mignot Informatique Graphique Inc.
//  Date : may 2004
//*******************************************************************
	bool Volume_Tree_Node::Render(Matrix &rMatrix)
	{

	::glPushMatrix();

	if(!m_displaylist) {

		m_displaylist = ::glGenLists(1);
		
		::glNewList(m_displaylist, GL_COMPILE_AND_EXECUTE);
		::glPushAttrib(GL_POLYGON_BIT);

//		::glEnable(GL_CULL_FACE);
		::glShadeModel(GL_SMOOTH);
		::glEnable(GL_LIGHTING);
		if( Graphic != NULL) Graphic->Display();

		

		::glPopAttrib();
		::glEndList();


	}
	else
		::glCallList(m_displaylist);

	::glPopMatrix();

	return TRUE;




	}

//EOF*****************************************************************