#ifndef _VOLUME_TREE_NODE_H
#define _VOLUME_TREE_NODE_H

//**************************************************************
// Class Volume_Tree_Node - This is the abstract class of all
// the nodes needed to build a local Volume Tree with primitives,
// transformation matrix and operations. It's mainly used by the
// polygonizer.
//**************************************************************
class DensityArray;
class Volume_Tree_Binary_Op;
class Volume_Tree_Unary_Op;

// AM + tard #include "OpenGl/AFX/Solid/Polygonise.h"

#include <ztl.h>
#include "Opengl/AFX/Solid/SL_matrix.h"
#include "opengl/base/Matrix.h"
#include "../Point.h"
#include "../Bbox.h"
#include "../SolidGraphic.h"


#define CAST_OK_unary_op 0xFF

//SL - added for Frustum culling
struct BoundingQuadric
{
    double Coefficients[10];
    BBox BoundingBox;

    inline bool isInside(const double x, const double y, const double z)
    {
#if 0 // AM +tard
		return(BoundingBox.Inside(Point((float)x, (float)y, (float)z)) &&
               (DLz2_coeR(Coefficients, x, y, z) >= 0.0));
#else
		return(BoundingBox.Inside(Point((float)x, (float)y, (float)z)) );
#endif
    }
};

//***************************************************************
//	 Here is an example of a tree
//  class Point
//						   Volume_Tree_Op
//			  /				       \
//	       Volume_Tree_Op				     Volume_Tree_Op
//	       /	     \				     /		   \
//Volume_Tree_Leaf	   Volume_Tree_Leaf	Volume_Tree_Leaf	 Volume_Tree_Op
//									 /	       \
//							  Volume_Tree_Leaf		Volume_Tree_Leaf
//*****************************************************************

class Volume_Tree_Node
{
public:
    Volume_Tree_Node(const float bbox_ptr[]);
	Volume_Tree_Node();

	//Accessor
	void Set_Bbox(const float bbox_ptr[]);

	//Functions

    //Eb, here we do some initialisation;
    void init();

    virtual void SetGraphic( SolidGraphic *graphic)  ;
	virtual SolidGraphic	*GetGraphic()  ;

	virtual bool Render( Matrix& rMatrix)  ;

    virtual int EvaluateDensity(const double& x, const double& y, const double& z)const = 0;    

    // SL Fonction to evaluate the normal (here, we need to traverse the tree because
    // no API function that return the normal at one point is available).
    virtual bool   EvaluateGradient(const double &x,
                                    const double &y,
                                    const double &z,
                                    double       &Distance,
                                    double       *Gradient,
                                    float        *Color = 0) const = 0;

    virtual bool    isInside( const double& x, const double& y, const double& z)  const;

    bool            isInsideBoundingVolume(const double x, const double y, const double z) const;

    bool            isInsideWithoutDensityCheck(const double& x, 
                                                const double& y, 
                                                const double& z) const;

    virtual double  *GetCoeff(double x, double y , double z)     = 0;

    virtual int     GetNumOfCoeff(double x, double y , double z) = 0;

    virtual bool    GetColor(const double &x, 
                             const double &y, 
                             const double &z, 
                             float set_color[])const = 0;

    virtual void    GetPrimitives(ZArray<BoundingQuadric*> &outPrimitives,
                                  const SL_Matrix &Transform, BBox &outGlobalBBox) = 0;
	
    Volume_Tree_Binary_Op& operator+(const Volume_Tree_Node &The_Other)const;
    Volume_Tree_Binary_Op& operator*(const Volume_Tree_Node &The_Other)const;
    Volume_Tree_Binary_Op& operator-(const Volume_Tree_Node &The_Other)const;
    Volume_Tree_Binary_Op& operator&(const Volume_Tree_Node &The_Other)const;
    Volume_Tree_Binary_Op& operator|(const Volume_Tree_Node &The_Other)const;
    Volume_Tree_Unary_Op&  operator!();

    virtual void    Ref()   {ref++;}
    virtual void    UnRef() {ref--;}

	void            Set_Color(const float to_set[]);
	const float     *GetLocalColor() const {return color;}

    void Adjust_With_BBox(double& x, double& y, double& z)const;    

    //Public members
    ZArray<BoundingQuadric *> *m_BoundingVolume;
    static DensityArray currentDensity;
    bool                ShowAllDensities;
    float               *bbox; 
	bool                At_The_Top;
	static const float  f_JuleError;

protected:
    //Protected member
	float        color[3]; // default = -1.0 = Unknown
    int          ref;    
	virtual void Manage_At_The_Top()=0;
    virtual      ~Volume_Tree_Node(void);

	SolidGraphic	*Graphic;
		int m_displaylist;
private:
};

#endif //_VOLUME_TREE_NODE_H