/*=============================================================================



==============================================================================*/
#ifndef _POINT_H
#define _POINT_H

/******************************************************************************



******************************************************************************/


#include "stdafx.h"
#include <float.h>
#include <math.h>

class SL_Matrix;


/*/////////////////////////////////////////////////////////////////////////////

Point
@class
a 3d point without virtual functions
*/

class Point {
	public:
	
	//! the 3d coordinates
	float x,y,z;

	// constructors
	inline Point(float v_=0.0) { x=v_; y=v_; z=v_; }
	inline Point(const float v[3]) { x=v[0]; y=v[1]; z=v[2];}
	inline Point(float x_, float y_, float z_=0.0) { x=x_; y=y_; z=z_; }
	inline Point(const Point &p) { x=p.x; y=p.y; z=p.z; }


	// Assignment operators
	Point& operator+= (const Point& A);
	Point& operator-= (const Point& A);
	Point& operator*= (const Point& A);
	Point& operator/= (const Point& A);
	Point& operator+= (float A);
	Point& operator-= (float A);
	Point& operator*= (float A);
	Point& operator/= (float A);

		inline float& operator[] (int i) {
		//if (i == 0) return x; else if (i == 1) return y; else return z;
		return *((&x)+i); // assume continous memory
	}

	inline float Len2() const { return (float)(x*x + y*y + z*z); } 	// squared length
	inline float Len() const { return (float)sqrt(Len2()); }		// vector length

		/* assigments */
	inline void Set(float x_=0.0, float y_=0.0, float z_=0.0) { x=x_; y=y_; z=z_; }
	inline void Set(const Point &p) { x=p.x; y=p.y; z=p.z; }
	inline void Set(const float v[3]) { x=v[0]; y=v[1]; z=v[2]; }

	// normalize point
	inline Point& Normalize()	{ float l=Len();
		if (l != 0.0f) l= 1.0f/l; else l=1.0f;
		x *= l; y *=l; z*=l; return(*this);
		}
			// Binary operators returning a new point
	friend Point operator+ (const Point& A, const Point& B);
	friend Point operator- (const Point& A, const Point& B);
	friend Point operator* (const Point& A, const Point& B);
	friend Point operator* (const Point& A, float B);
	friend Point operator* (float A, const Point& B);
	
	friend Point operator* (const SL_Matrix&, const Point&);

	friend int operator== (const Point& A, const Point& B);
	friend int operator!= (const Point& A, const Point& B);

		// return Dot Product

	friend float DotProd(const Point& A, const Point& B);


	//! return Cross Product
	friend Point CrossProd (const Point& A, const Point& B);

		void MinMax(Point &mi,Point &ma) const;

};


inline Point& Point::operator+= (const Point& A)
{
	x += A.x;  y += A.y; z += A.z;
	return *this;
}

inline Point& Point::operator-= (const Point& A)
{
	x -= A.x;  y -= A.y;  z -= A.z;
	return *this;
}

inline Point& Point::operator+= (float A)
{
	x += A;  y += A;  z += A;
	return *this;
}

inline Point& Point::operator-= (float A)
{
	x -= A;  y -= A;  z -= A;
	return *this;
}

inline Point& Point::operator*= (float A)
{
	x *= A;  y *= A;  z *= A;
	return *this;
}

inline Point& Point::operator/= (float A)
{
	x /= A;  y /= A;  z /= A;
	return *this;
}

inline Point& Point::operator*= (const Point& A)
{
	x *= A.x;  y *= A.y;  z *= A.z;
	return *this;
}

inline Point& Point::operator/= (const Point& A)
{
	x /= A.x;  y /= A.y;  z /= A.z;
	return *this;
}


inline Point operator+ (const Point& A, const Point& B)
{
	return Point(A.x + B.x, A.y + B.y, A.z + B.z);
}

inline Point operator- (const Point& A, const Point& B)
{
	return Point(A.x - B.x, A.y - B.y, A.z - B.z);
}

inline Point operator* (const Point& A, const Point& B)
{
	return Point(A.x * B.x, A.y * B.y, A.z * B.z);
}

inline Point operator* (const Point& A, float B)
{
	return Point(A.x * B, A.y * B, A.z * B);
}

inline Point operator* (float A, const Point& B)
{
	return B * A;
}

inline int operator== (const Point& A, const Point& B)
{
	return (A.x == B.x) && (A.y == B.y) && (A.z == B.z);
}

inline int operator!= (const Point& A, const Point& B)
{
	return !(A == B);
}


inline Point CrossProd (const Point& A, const Point& B)
{
	return Point(A.y * B.z - A.z * B.y,
			A.z * B.x - A.x * B.z,
			A.x * B.y - A.y * B.x);
}

inline float DotProd (const Point& A, const Point& B)
{
	return A.x * B.x + A.y * B.y + A.z * B.z;
}

// update minmax
inline void Point::MinMax(Point &mi,Point &ma) const
{
  if (x < mi.x) mi.x = x; else if (x>ma.x) ma.x=x;
  if (y < mi.y) mi.y = y; else if (y>ma.y) ma.y=y;
  if (z < mi.z) mi.z = z; else if (z>ma.z) ma.z=z;
}

#endif //_POINT_H


