/*=============================================================================


==============================================================================*/
// ===================================================================
// matrix.cpp

// ===================================================================
#include "stdafx.h"

#include "SL_matrix.h"
#include "point.h"

static inline double MINOR(const SL_Matrix& M,
                    int r0, int r1, int r2, int c0, int c1, int c2)
{
  return M[r0][c0] * (M[r1][c1] * M[r2][c2] - M[r2][c1] * M[r1][c2]) -
	 M[r0][c1] * (M[r1][c0] * M[r2][c2] - M[r2][c0] * M[r1][c2]) +
	 M[r0][c2] * (M[r1][c0] * M[r2][c1] - M[r2][c0] * M[r1][c1]);
}

SL_Matrix Adjoint(const SL_Matrix& m) 
{
  SL_Matrix A;
  
  A[0][0] =  MINOR(m, 1, 2, 3, 1, 2, 3);
  A[0][1] = -MINOR(m, 0, 2, 3, 1, 2, 3);
  A[0][2] =  MINOR(m, 0, 1, 3, 1, 2, 3);
  A[0][3] = -MINOR(m, 0, 1, 2, 1, 2, 3);
  A[1][0] = -MINOR(m, 1, 2, 3, 0, 2, 3);
  A[1][1] =  MINOR(m, 0, 2, 3, 0, 2, 3);
  A[1][2] = -MINOR(m, 0, 1, 3, 0, 2, 3);
  A[1][3] =  MINOR(m, 0, 1, 2, 0, 2, 3);
  A[2][0] =  MINOR(m, 1, 2, 3, 0, 1, 3);
  A[2][1] = -MINOR(m, 0, 2, 3, 0, 1, 3);
  A[2][2] =  MINOR(m, 0, 1, 3, 0, 1, 3);
  A[2][3] = -MINOR(m, 0, 1, 2, 0, 1, 3);
  A[3][0] = -MINOR(m, 1, 2, 3, 0, 1, 2);
  A[3][1] =  MINOR(m, 0, 2, 3, 0, 1, 2);
  A[3][2] = -MINOR(m, 0, 1, 3, 0, 1, 2);
  A[3][3] =  MINOR(m, 0, 1, 2, 0, 1, 2);

  A.contents = m.contents; // ?? what to return
  return A;
}

void 
Adjoint(const SL_Matrix& m,SL_Matrix &A) 
{
  
  A[0][0] =  MINOR(m, 1, 2, 3, 1, 2, 3);
  A[0][1] = -MINOR(m, 0, 2, 3, 1, 2, 3);
  A[0][2] =  MINOR(m, 0, 1, 3, 1, 2, 3);
  A[0][3] = -MINOR(m, 0, 1, 2, 1, 2, 3);
  A[1][0] = -MINOR(m, 1, 2, 3, 0, 2, 3);
  A[1][1] =  MINOR(m, 0, 2, 3, 0, 2, 3);
  A[1][2] = -MINOR(m, 0, 1, 3, 0, 2, 3);
  A[1][3] =  MINOR(m, 0, 1, 2, 0, 2, 3);
  A[2][0] =  MINOR(m, 1, 2, 3, 0, 1, 3);
  A[2][1] = -MINOR(m, 0, 2, 3, 0, 1, 3);
  A[2][2] =  MINOR(m, 0, 1, 3, 0, 1, 3);
  A[2][3] = -MINOR(m, 0, 1, 2, 0, 1, 3);
  A[3][0] = -MINOR(m, 1, 2, 3, 0, 1, 2);
  A[3][1] =  MINOR(m, 0, 2, 3, 0, 1, 2);
  A[3][2] = -MINOR(m, 0, 1, 3, 0, 1, 2);
  A[3][3] =  MINOR(m, 0, 1, 2, 0, 1, 2);

  A.contents = m.contents; // ?? what to return

}



double Determinant(const SL_Matrix& m)
{
/*		5.5.97 
  return 
	m[0][0] * MINOR(m, 1, 2, 3, 1, 2, 3) -
	 m[0][1] * MINOR(m, 1, 2, 3, 0, 2, 3) +
	 m[0][2] * MINOR(m, 1, 2, 3, 0, 1, 3) -
	 m[0][3] * MINOR(m, 1, 2, 3, 0, 1, 2);
*/
  double ret = 0.0;
  if (m[0][0] != 0.0f) ret = m[0][0] * MINOR(m, 1, 2, 3, 1, 2, 3);
  if (m[0][1] != 0.0f) ret -= m[0][1] * MINOR(m, 1, 2, 3, 0, 2, 3);
  if (m[0][2] != 0.0f) ret += m[0][2] * MINOR(m, 1, 2, 3, 0, 1, 3);
  if (m[0][3] != 0.0f) ret -= m[0][3] * MINOR(m, 1, 2, 3, 0, 1, 2);
  return ret;

}

// Construct an identity matrix.
SL_Matrix
IdentityMatrix()
{
    SL_Matrix M;

	const float one = 1.0f;
	const float zero = 0.0f;
    // help compiler ?
    M.x[0][0] = one;  M.x[1][0] =		M.x[2][0] =		M.x[3][0] =
    M.x[0][1] = zero; M.x[1][1] = one;  M.x[2][1] =		M.x[3][1] =
    M.x[0][2] =       M.x[1][2] = zero;	M.x[2][2] = one;  M.x[3][2] =
    M.x[0][3] =       M.x[1][3] =       M.x[2][3] = zero; M.x[3][3] =one;
/*
    for (int i = 0; i < 4; i++)
	for (int j = 0; j < 4; j++)
	    M.x[i][j] = (i == j) ? 1.0 : 0.0;
*/
	M.contents = 0;
    return M;
}

void
SL_Matrix::Invert()
{
	// identity SL_Matrix, is the inverse for itself
	if (contents ==0)
		return;

	SL_Matrix m;
	


	// assert(!isSingular());
   float det =  (1.0/ Determinant(*this));
   
	Adjoint(*this,m);
/*
	if (det == 1.0f) { 
		*this=m; return;
	}
*/
	for (int i = 0; i < 4; i++)
	for (int j = 0; j < 4; j++)
	    x[i][j] = m[i][j]*det;
    

}


// Invert the given matrix using the above inversion routine.
SL_Matrix
Invert(const SL_Matrix& M)
{
    SL_Matrix InvertMe = M;
    InvertMe.Invert();
    return InvertMe;
}

// Transpose the matrix.
void
SL_Matrix::Transpose()
{
    for (int i = 0; i < 4; i++)
	for (int j = i; j < 4; j++)
	    if (i != j) {
			float temp = x[i][j];
			x[i][j] = x[j][i];
			x[j][i] = temp;
	    }
}

// Transpose the given matrix using the transpose routine above.
SL_Matrix
Transpose(const SL_Matrix& M)
{
    SL_Matrix TransposeMe = M;
    TransposeMe.Transpose();
    return TransposeMe;
}

// Construct a quadric matrix.	After Foley et al. pp. 528-529.
SL_Matrix
QuadricMatrix(float a, float b, float c, float d, float e,
	      float f, float g, float h, float j, float k)
{
    SL_Matrix M;

    M.x[0][0] = a;  M.x[0][1] = d;  M.x[0][2] = f;  M.x[0][3] = g;
    M.x[1][0] = d;  M.x[1][1] = b;  M.x[1][2] = e;  M.x[1][3] = h;
    M.x[2][0] = f;  M.x[2][1] = e;  M.x[2][2] = c;  M.x[2][3] = j;
    M.x[3][0] = g;  M.x[3][1] = h;  M.x[3][2] = j;  M.x[3][3] = k;

	M.contents = SL_Matrix::HAS_ANYTHING;

    return M;
}

SL_Matrix& 
SL_Matrix::MultiplyLeft(const SL_Matrix& A)
{
	SL_Matrix B(*this);
#if 1
	// total unroll 
        x[0][0] = A.x[0][0] * B.x[0][0] + A.x[0][1] * B.x[1][0] + A.x[0][2] * B.x[2][0] + A.x[0][3] * B.x[3][0] ;
        x[0][1] = A.x[0][0] * B.x[0][1] + A.x[0][1] * B.x[1][1] + A.x[0][2] * B.x[2][1] + A.x[0][3] * B.x[3][1] ;
        x[0][2] = A.x[0][0] * B.x[0][2] + A.x[0][1] * B.x[1][2] + A.x[0][2] * B.x[2][2] + A.x[0][3] * B.x[3][2] ;
        x[0][3] = A.x[0][0] * B.x[0][3] + A.x[0][1] * B.x[1][3] + A.x[0][2] * B.x[2][3] + A.x[0][3] * B.x[3][3] ;

        x[1][0] = A.x[1][0] * B.x[0][0] + A.x[1][1] * B.x[1][0] + A.x[1][2] * B.x[2][0] + A.x[1][3] * B.x[3][0] ;
        x[1][1] = A.x[1][0] * B.x[0][1] + A.x[1][1] * B.x[1][1] + A.x[1][2] * B.x[2][1] + A.x[1][3] * B.x[3][1] ;
        x[1][2] = A.x[1][0] * B.x[0][2] + A.x[1][1] * B.x[1][2] + A.x[1][2] * B.x[2][2] + A.x[1][3] * B.x[3][2] ;
        x[1][3] = A.x[1][0] * B.x[0][3] + A.x[1][1] * B.x[1][3] + A.x[1][2] * B.x[2][3] + A.x[1][3] * B.x[3][3] ;

        x[2][0] = A.x[2][0] * B.x[0][0] + A.x[2][1] * B.x[1][0] + A.x[2][2] * B.x[2][0] + A.x[2][3] * B.x[3][0] ;
        x[2][1] = A.x[2][0] * B.x[0][1] + A.x[2][1] * B.x[1][1] + A.x[2][2] * B.x[2][1] + A.x[2][3] * B.x[3][1] ;
        x[2][2] = A.x[2][0] * B.x[0][2] + A.x[2][1] * B.x[1][2] + A.x[2][2] * B.x[2][2] + A.x[2][3] * B.x[3][2] ;
        x[2][3] = A.x[2][0] * B.x[0][3] + A.x[2][1] * B.x[1][3] + A.x[2][2] * B.x[2][3] + A.x[2][3] * B.x[3][3] ;

        x[3][0] = A.x[3][0] * B.x[0][0] + A.x[3][1] * B.x[1][0] + A.x[3][2] * B.x[2][0] + A.x[3][3] * B.x[3][0] ;
        x[3][1] = A.x[3][0] * B.x[0][1] + A.x[3][1] * B.x[1][1] + A.x[3][2] * B.x[2][1] + A.x[3][3] * B.x[3][1] ;
        x[3][2] = A.x[3][0] * B.x[0][2] + A.x[3][1] * B.x[1][2] + A.x[3][2] * B.x[2][2] + A.x[3][3] * B.x[3][2] ;
        x[3][3] = A.x[3][0] * B.x[0][3] + A.x[3][1] * B.x[1][3] + A.x[3][2] * B.x[2][3] + A.x[3][3] * B.x[3][3] ;

#else


    for (int i = 0; i < 4; i++)	{
        x[i][0] = A.x[i][0] * B.x[0][0] + A.x[i][1] * B.x[1][0] + A.x[i][2] * B.x[2][0] + A.x[i][3] * B.x[3][0] ;
        x[i][1] = A.x[i][0] * B.x[0][1] + A.x[i][1] * B.x[1][1] + A.x[i][2] * B.x[2][1] + A.x[i][3] * B.x[3][1] ;
        x[i][2] = A.x[i][0] * B.x[0][2] + A.x[i][1] * B.x[1][2] + A.x[i][2] * B.x[2][2] + A.x[i][3] * B.x[3][2] ;
        x[i][3] = A.x[i][0] * B.x[0][3] + A.x[i][1] * B.x[1][3] + A.x[i][2] * B.x[2][3] + A.x[i][3] * B.x[3][3] ;
	}
#endif
	contents |= A.contents;
    return *this;
}

SL_Matrix& 
SL_Matrix::MultiplyRight(const SL_Matrix& B)
{
	SL_Matrix A(*this);
#if 1
	// total unroll 
        x[0][0] = A.x[0][0] * B.x[0][0] + A.x[0][1] * B.x[1][0] + A.x[0][2] * B.x[2][0] + A.x[0][3] * B.x[3][0] ;
        x[0][1] = A.x[0][0] * B.x[0][1] + A.x[0][1] * B.x[1][1] + A.x[0][2] * B.x[2][1] + A.x[0][3] * B.x[3][1] ;
        x[0][2] = A.x[0][0] * B.x[0][2] + A.x[0][1] * B.x[1][2] + A.x[0][2] * B.x[2][2] + A.x[0][3] * B.x[3][2] ;
        x[0][3] = A.x[0][0] * B.x[0][3] + A.x[0][1] * B.x[1][3] + A.x[0][2] * B.x[2][3] + A.x[0][3] * B.x[3][3] ;

        x[1][0] = A.x[1][0] * B.x[0][0] + A.x[1][1] * B.x[1][0] + A.x[1][2] * B.x[2][0] + A.x[1][3] * B.x[3][0] ;
        x[1][1] = A.x[1][0] * B.x[0][1] + A.x[1][1] * B.x[1][1] + A.x[1][2] * B.x[2][1] + A.x[1][3] * B.x[3][1] ;
        x[1][2] = A.x[1][0] * B.x[0][2] + A.x[1][1] * B.x[1][2] + A.x[1][2] * B.x[2][2] + A.x[1][3] * B.x[3][2] ;
        x[1][3] = A.x[1][0] * B.x[0][3] + A.x[1][1] * B.x[1][3] + A.x[1][2] * B.x[2][3] + A.x[1][3] * B.x[3][3] ;

        x[2][0] = A.x[2][0] * B.x[0][0] + A.x[2][1] * B.x[1][0] + A.x[2][2] * B.x[2][0] + A.x[2][3] * B.x[3][0] ;
        x[2][1] = A.x[2][0] * B.x[0][1] + A.x[2][1] * B.x[1][1] + A.x[2][2] * B.x[2][1] + A.x[2][3] * B.x[3][1] ;
        x[2][2] = A.x[2][0] * B.x[0][2] + A.x[2][1] * B.x[1][2] + A.x[2][2] * B.x[2][2] + A.x[2][3] * B.x[3][2] ;
        x[2][3] = A.x[2][0] * B.x[0][3] + A.x[2][1] * B.x[1][3] + A.x[2][2] * B.x[2][3] + A.x[2][3] * B.x[3][3] ;

        x[3][0] = A.x[3][0] * B.x[0][0] + A.x[3][1] * B.x[1][0] + A.x[3][2] * B.x[2][0] + A.x[3][3] * B.x[3][0] ;
        x[3][1] = A.x[3][0] * B.x[0][1] + A.x[3][1] * B.x[1][1] + A.x[3][2] * B.x[2][1] + A.x[3][3] * B.x[3][1] ;
        x[3][2] = A.x[3][0] * B.x[0][2] + A.x[3][1] * B.x[1][2] + A.x[3][2] * B.x[2][2] + A.x[3][3] * B.x[3][2] ;
        x[3][3] = A.x[3][0] * B.x[0][3] + A.x[3][1] * B.x[1][3] + A.x[3][2] * B.x[2][3] + A.x[3][3] * B.x[3][3] ;

#else


    for (int i = 0; i < 4; i++)	{
        x[i][0] = A.x[i][0] * B.x[0][0] + A.x[i][1] * B.x[1][0] + A.x[i][2] * B.x[2][0] + A.x[i][3] * B.x[3][0] ;
        x[i][1] = A.x[i][0] * B.x[0][1] + A.x[i][1] * B.x[1][1] + A.x[i][2] * B.x[2][1] + A.x[i][3] * B.x[3][1] ;
        x[i][2] = A.x[i][0] * B.x[0][2] + A.x[i][1] * B.x[1][2] + A.x[i][2] * B.x[2][2] + A.x[i][3] * B.x[3][2] ;
        x[i][3] = A.x[i][0] * B.x[0][3] + A.x[i][1] * B.x[1][3] + A.x[i][2] * B.x[2][3] + A.x[i][3] * B.x[3][3] ;
	}
#endif
	contents |= A.contents;
    return *this;
}

// Add corresponding elements of the matrices.
SL_Matrix
operator+ (const SL_Matrix& A, const SL_Matrix& B)
{
    SL_Matrix ret;

    for (int i = 0; i < 4; i++)
	for (int j = 0; j < 4; j++)
	    ret.x[i][j] = A.x[i][j] + B.x[i][j];

	ret.contents = SL_Matrix::HAS_ANYTHING;

    return ret;
}

// Subtract corresponding elements of the matrices.
SL_Matrix
operator- (const SL_Matrix& A, const SL_Matrix& B)
{
    SL_Matrix ret;

    for (int i = 0; i < 4; i++)
	for (int j = 0; j < 4; j++)
	    ret.x[i][j] = A.x[i][j] - B.x[i][j];

	ret.contents = SL_Matrix::HAS_ANYTHING;
    return ret;
}

//added by Chloe Mignot 2001/10/26 SGDL Systemes Inc
// Determine if 2 matrices are equal
bool
operator== (const SL_Matrix& A, const SL_Matrix& B)
{
	for(int i = 0; i < 4; i ++){
		for(int j = 0; j < 4; j++){
			if(A.x[i][j] != B.x[i][j]) return false;
		}
	}
	return true;
	
}


// Transform a point by a SL_Matrix.
Point
operator* (const SL_Matrix& M, const Point& v)
{
    Point ret;
    float denom;

    ret.x = v.x * M.x[0][0] + v.y * M.x[1][0] + v.z * M.x[2][0] + M.x[3][0];
    ret.y = v.x * M.x[0][1] + v.y * M.x[1][1] + v.z * M.x[2][1] + M.x[3][1];
    ret.z = v.x * M.x[0][2] + v.y * M.x[1][2] + v.z * M.x[2][2] + M.x[3][2];
    denom = v.x * M.x[0][3] + v.y * M.x[1][3] + v.z * M.x[2][3] + M.x[3][3];

	//ASSERT(denom != 0.0f);
	if (denom == 0.0f) 
		return ret;

    if (denom != 1.0)
		ret *= (1.0f / denom);

    return ret;
}

SL_Matrix
operator* (const SL_Matrix& A, const SL_Matrix& B)
{
    SL_Matrix ret;
#if 1
    for (int i = 0; i < 4; i++)	{
        ret.x[i][0] = A.x[i][0] * B.x[0][0] + A.x[i][1] * B.x[1][0] + A.x[i][2] * B.x[2][0] + A.x[i][3] * B.x[3][0] ;
        ret.x[i][1] = A.x[i][0] * B.x[0][1] + A.x[i][1] * B.x[1][1] + A.x[i][2] * B.x[2][1] + A.x[i][3] * B.x[3][1] ;
        ret.x[i][2] = A.x[i][0] * B.x[0][2] + A.x[i][1] * B.x[1][2] + A.x[i][2] * B.x[2][2] + A.x[i][3] * B.x[3][2] ;
        ret.x[i][3] = A.x[i][0] * B.x[0][3] + A.x[i][1] * B.x[1][3] + A.x[i][2] * B.x[2][3] + A.x[i][3] * B.x[3][3] ;
	}
#else 

    for (int i = 0; i < 4; i++)
	for (int j = 0; j < 4; j++) {
	    //float subt = 0;
	    //for (int k = 0; k < 4; k++)   subt += A.x[i][k] * B.x[k][j];
	    //ret.x[i][j] = subt;

        ret.x[i][j] = A.x[i][0] * B.x[0][j] + A.x[i][1] * B.x[1][j] + A.x[i][2] * B.x[2][j] + A.x[i][3] * B.x[3][j] ;
	}
#endif

	ret.contents = A.contents | B.contents;
    return ret;
}

// Add corresponding elements of the two matrices.
SL_Matrix&
SL_Matrix::operator+= (const SL_Matrix& A)
{
    for (int i = 0; i < 4; i++)
	for (int j = 0; j < 4; j++)
	    x[i][j] += A.x[i][j];

	contents |= A.contents;
    return *this;
}

// Subtract corresponding elements of the matrices.
SL_Matrix&
SL_Matrix::operator-= (const SL_Matrix& A)
{
    for (int i = 0; i < 4; i++)
	for (int j = 0; j < 4; j++)
	    x[i][j] -= A.x[i][j];

	contents |= A.contents;
    return *this;
}

// Multiply two matrices.
SL_Matrix&
SL_Matrix::operator*= (const SL_Matrix& A)
{
    SL_Matrix ret = *this;

    for (int i = 0; i < 4; i++)
	for (int j = 0; j < 4; j++) {
	    float subt = 0;
	    for (int k = 0; k < 4; k++)
			subt += ret.x[i][k] * A.x[k][j];
	    x[i][j] = subt;
	}
	contents |= A.contents;    
	return *this;
}



// Scale each element of the matrix by A.
SL_Matrix&
SL_Matrix::operator*= (float A)
{
    for (int i = 0; i < 4; i++)
	for (int j = 0; j < 4; j++)
	    x[i][j] *= A;

	contents |= HAS_ANY_SCALE;
    return *this;
}


SL_Matrix&
SL_Matrix::Identity()
{
	const float one = 1.0f;
	const float zero = 0.0f;
    // help compiler ?
    x[0][0] = one;  x[1][0] =		x[2][0] =		x[3][0] =
    x[0][1] = zero; x[1][1] = one;  x[2][1] =		x[3][1] =
    x[0][2] =       x[1][2] = zero;	x[2][2] = one;  x[3][2] =
    x[0][3] =       x[1][3] =       x[2][3] = zero; x[3][3] = one;

/*
    for (int i = 0; i < 4; i++)
	for (int j = 0; j < 4; j++)
	    x[i][j] = (i == j) ? 1.0 : 0.0;
*/
	contents = 0;
    return *this;
}

/*
  unmatrix : Spencer W. Thomas, University of Michigan
  Graphic Gems II, Page 600 + 320
*/
#define XY x
#define XZ y
#define YZ z

int SL_Matrix::Decompose(Point &scale,Point &shear, Point &rotate, Point &translate) const
{ register int i;
  SL_Matrix locmat(*this);
  Point row[3];
  Point cross12;

  /* assume
     det of upper 3*3 matrix non zero
     no perspective part
  */
  /* normalize */

  translate.x = locmat.x[3][0]; locmat.x[3][0]= 0;
  translate.y = locmat.x[3][1]; locmat.x[3][1]= 0;
  translate.z = locmat.x[3][2]; locmat.x[3][2]= 0;

  /* now get scale and shear */
  for(i=0; i< 3; i++) { row[i].Set(locmat.x[i][0],locmat.x[i][1],locmat.x[i][2]); }

  /* Compute x scale factor and normalize first row */
  scale.x = row[0].Len();
  row[0].Normalize();

  shear.XY = DotProd(row[0],row[1]);
  
  row[1]+=row[0] * -shear.XY;

  /* Compute y scale factor and normalize second row */
  scale.y = row[1].Len();
  row[1].Normalize();

  if (scale.y != 0.0) shear.XY /= scale.y;

  /* compute xz and yz shears, orthogonalize 3rd row */
  shear.XZ = DotProd(row[0],row[2]);
  row[2]+=row[0]* -shear.XZ;

  shear.YZ = DotProd(row[1],row[2]);
  row[2]+=row[1]*-shear.YZ;

  scale.z = row[2].Len();
  row[2].Normalize();

  if (scale.z != 0.0) shear.XZ /= scale.z;
  if (scale.z != 0.0) shear.YZ /= scale.z;

  /* at this point the matrix in rows is orthonormal.
     Check for a coordinate system flip. if the determinat is -1
     then negate the matrix and the scaling factors.
  */
  cross12 = CrossProd(row[1],row[2]);

  if (DotProd(row[0],cross12) < 0)
     for(i=0; i <3;i++) {
        row[i]*= -1.0;
        scale[i] *= -1.0;
     }

  /* rotations */
  rotate.y = asin(-row[0].z);

  if (cos(rotate.y) != 0) {
        rotate.x = atan2(row[1].z,row[2].z);
        rotate.z = atan2(row[0].y,row[0].x);
  } else {
        rotate.x = atan2(row[1].x,row[1].y);
        rotate.z = 0;
  }
//  rotate.x = RAD2DEG(rotate.x);
//  rotate.y = RAD2DEG(rotate.y);
//  rotate.z = RAD2DEG(rotate.z);

#if 0
  if (1) {
      printf("\nScale "); VECPRINT(*scale);
      printf("\nShear "); VECPRINT(*shear);
      printf("\nRotate "); VECPRINT(*rotate);
      printf("\nTrans  "); VECPRINT(*translate);
  }
#endif

  return(1);
}


// Basic Transformation as incremental assignment operators

SL_Matrix& SL_Matrix::Scale(float X, float Y, float Z)
{
	*this *= ScaleMatrix(X,Y,Z);
	return(*this);
}

SL_Matrix& SL_Matrix::Scale(const Point &scale)
{
	*this *= ScaleMatrix(scale);
	return(*this);
}

SL_Matrix& SL_Matrix::Translate(const Point& Location)
{
	*this *= TranslationMatrix(Location);
	return(*this);
}

SL_Matrix& SL_Matrix::Translate(float X, float Y, float Z)
{
	*this *= ::TranslationMatrix(X,Y,Z);
	return(*this);
}


SL_Matrix& SL_Matrix::Rotate(const Point& Rotation)
{
	*this *= RotationMatrix(Rotation);
	return(*this);
}

SL_Matrix& SL_Matrix::Rotate(float X, float Y, float Z)
{
	Point p;
	p.x = X;
	p.y = Y;
	p.z = Z;
	*this *= RotationMatrix(p);
	return(*this);
}



// Construct a translation matrix given the location to translate to.
SL_Matrix
TranslationMatrix(const Point& Location)
{
    SL_Matrix M; M.Identity();
    M.x[3][0] = Location.x;
    M.x[3][1] = Location.y;
    M.x[3][2] = Location.z;
	M.contents = SL_Matrix::HAS_TRANSLATION;
    return M;
}

SL_Matrix
TranslationMatrix(float X, float Y, float Z)
{
    SL_Matrix M; M.Identity();
    M.x[3][0] = X;
    M.x[3][1] = Y;
    M.x[3][2] = Z;
	M.contents = SL_Matrix::HAS_TRANSLATION;
    return M;
}

// Construct a scale matrix given the X, Y, and Z parameters
// to scale by.  To scale uniformly, let X==Y==Z.
SL_Matrix
ScaleMatrix(float X, float Y, float Z)
{
    SL_Matrix M; M.Identity();

    M.x[0][0] = X;
    M.x[1][1] = Y;
    M.x[2][2] = Z;

	M.contents = SL_Matrix::HAS_SCALE;

	if ((X <=0) || (Y <=0) || (Z <=0))
		M.contents |= SL_Matrix::HAS_NEGATIVE_SCALE;

	if ((X != Y) || (Y != Z))
		M.contents |= SL_Matrix::HAS_NON_UNIFORM_SCALE;

    return M;
}

SL_Matrix 
ScaleMatrix(const Point& Scale)
{
  return(ScaleMatrix(Scale.x,Scale.y,Scale.z));
}


SL_Matrix 
RotationMatrix(const Point& Rotation)
{
   SL_Matrix M = RotationXMatrix(Rotation.x);
   if (Rotation.y != 0.0) M *= RotationYMatrix(Rotation.y);
   if (Rotation.z != 0.0) M *= RotationZMatrix(Rotation.z);
   M.contents = SL_Matrix::HAS_ROTATION;
   return M;
}

SL_Matrix 
RotationMatrix (float X, float Y, float Z)
{
   SL_Matrix M = RotationXMatrix(X);
   if (Y != 0.0) M *= RotationYMatrix(Y);
   if (Z != 0.0) M *= RotationZMatrix(Z);
   M.contents = SL_Matrix::HAS_ROTATION;
   return M;
}
// Construct a rotation matrix.  Rotates Angle radians about the
// X axis.
SL_Matrix
RotationXMatrix(float Angle)
{
    SL_Matrix M; M.Identity();
    float Cosine = cos(Angle);
    float Sine = sin(Angle);
    M.x[1][1] = Cosine;
    M.x[2][1] = -Sine;
    M.x[1][2] = Sine;
    M.x[2][2] = Cosine;
	M.contents = SL_Matrix::HAS_ROTATION;
    return M;
}

// Construct a rotation matrix.  Rotates Angle radians about the
// Y axis.
SL_Matrix
RotationYMatrix(float Angle)
{
    SL_Matrix M; M.Identity();
    float Cosine = cos(Angle);
    float Sine = sin(Angle);
    M.x[0][0] = Cosine;
    M.x[2][0] = Sine;
    M.x[0][2] = -Sine;
    M.x[2][2] = Cosine;
	M.contents = SL_Matrix::HAS_ROTATION;
    return M;
}

// Construct a rotation SL_Matrix.  Rotates Angle radians about the
// Z axis.
SL_Matrix
RotationZMatrix(float Angle)
{
    SL_Matrix M; M.Identity();
    float Cosine = cos(Angle);
    float Sine = sin(Angle);
    M.x[0][0] = Cosine;
    M.x[1][0] = -Sine;
    M.x[0][1] = Sine;
    M.x[1][1] = Cosine;
	M.contents = SL_Matrix::HAS_ROTATION;
    return M;
}