/***********************************************************************
This software module was originally developed by Dr. Michael Steliaros,
Superscape (UK) Ltd., in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Superscape (UK) Ltd. retains full
right to use the code for his/her own purpose, assign or donate the code
to a third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2001
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Dr. Michael Steliaros at msteliaros@superscape.com
----------------------------------------------------------------------*/

////---------------------------------------------------------------------
////    Basic manifold hull storage classes
////
////    Author:                 Michael K. Steliaros (Superscape)
////    Last modification date: 28-11-2000
////    Creation date:          22-09-2000
////---------------------------------------------------------------------

#include <stdlib.h>
#include <string.h>
#include <assert.h>

#include "hull.h"

const Hull& Hull::operator=(const Hull& From)
{
  SetstrName(From.m_strName);

  m_aVertices = From.m_aVertices;
  m_aNormals = From.m_aNormals;
  m_aUVWs = From.m_aUVWs;
  m_aColours = From.m_aColours;
  m_aFacets = From.m_aFacets;
  m_aEdges = From.m_aEdges;

  InitHash(0);
  InitHash(From.m_cEdgeHashes);

  int Count;

  for(Count = 0; Count < cEdges(); Count++){
    Edge& E = m_aEdges[Count];

    SetiEdge(E.iVertex(1), E.iVertex(0), Count);
  }

  return(*this);
}

void Hull::SetstrName(char *strName)
{
  if(m_strName != NULL){ delete [] m_strName; m_strName = NULL; }

  if(strName != NULL){
    int cLen = strlen(strName) + 1;

    m_strName = new char[cLen];

    strncpy(m_strName, strName, cLen);
  }
}

int Hull::AddVertex(const Vector3D& aVector)
{
  int iPos = m_aVertices.cElements();

  m_aVertices.Append(Vertex(aVector));

  return(iPos);
}

Vector3D* Hull::VertexNormal(const int iVertex)
{
  const int iPrimaryEdge = VertexiPrimaryEdge(iVertex);
  const Edge& E = m_aEdges[iPrimaryEdge];
  int bRight = (E.iVertex(1) == iVertex);

  assert(E.iFacet(bRight) != -1);

  const Facet& F = m_aFacets[E.iFacet(bRight)];
  int iPos = F.iVertexPos(iVertex);

  assert(iPos != -1);

  return(&m_aNormals[F.aVertexNormals(iPos)]);
}

Vector3D* Hull::VertexNormal(const int iVertex,
                             const int iStart,
                             const int cSpan)
{
  int iEnd = iStart + cSpan - 1;
  const int iPrimaryEdge = VertexiPrimaryEdge(iVertex);
  int iCurr = iPrimaryEdge;
  int Count = 0;

  do{
    const Edge& E = m_aEdges[iCurr];
    int bRight = (E.iVertex(1) == iVertex);
    int iPos;

    if((Count == iEnd) || (E.iPred(bRight) == iCurr)){
      if(Count < iEnd){

        assert(E.iFacet(!bRight) != -1);

        const Facet& F = m_aFacets[E.iFacet(!bRight)];

        iPos = F.iVertexPos(iVertex);

        assert(iPos != -1);

        return(&m_aNormals[F.aVertexNormals(iPos)]);
      }

      break;
    }
    else{
      if(Count >= iStart){

        assert(E.iFacet(bRight) != -1);

        const Facet& F = m_aFacets[E.iFacet(bRight)];

        iPos = F.iVertexPos(iVertex);

        assert(iPos != -1);

        return(&m_aNormals[F.aVertexNormals(iPos)]);
      }

      Count++;

      iCurr = E.iPred(bRight);
    }

  }while(iCurr != iPrimaryEdge);

  return(NULL);
}

void Hull::SetVertexNormal(int iVertex,
                           int iNormal)
{
  const int iPrimaryEdge = VertexiPrimaryEdge(iVertex);
  int iCurr = iPrimaryEdge;

  do{
    const Edge& E = m_aEdges[iCurr];
    int bRight = (E.iVertex(1) == iVertex);
    int iPos;

    if(E.iPred(bRight) == iCurr){

      assert(E.iFacet(!bRight) != -1);

      Facet& F = m_aFacets[E.iFacet(!bRight)];

      iPos = F.iVertexPos(iVertex);

      assert(iPos != -1);

      F.aVertexNormals(iPos) = iNormal;

      break;
    }
    else{

      assert(E.iFacet(bRight) != -1);

      Facet& F = m_aFacets[E.iFacet(bRight)];

      iPos = F.iVertexPos(iVertex);

      assert(iPos != -1);

      F.aVertexNormals(iPos) = iNormal;

      iCurr = E.iPred(bRight);
    }

  }while(iCurr != iPrimaryEdge);
}

void Hull::SetVertexNormal(int iVertex,
                           int iStart,
                           int cSpan,
                           int iNormal)
{
  const int iPrimaryEdge = VertexiPrimaryEdge(iVertex);
  int iCurr = iPrimaryEdge;
  int Count = 0;

  cSpan += iStart - 1;

  do{
    const Edge& E = m_aEdges[iCurr];
    int bRight = (E.iVertex(1) == iVertex);
    int iPos;

    if((Count == cSpan) || (E.iPred(bRight) == iCurr)){
      if(Count < cSpan){

        assert(E.iFacet(!bRight) != -1);

        Facet& F = m_aFacets[E.iFacet(!bRight)];

        iPos = F.iVertexPos(iVertex);

        assert(iPos != -1);

        F.aVertexNormals(iPos) = iNormal;
      }

      break;
    }
    else{
      if(Count >= iStart){

        assert(E.iFacet(bRight) != -1);

        Facet& F = m_aFacets[E.iFacet(bRight)];

        iPos = F.iVertexPos(iVertex);

        assert(iPos != -1);

        F.aVertexNormals(iPos) = iNormal;
      }

      Count++;

      iCurr = E.iPred(bRight);
    }

  }while(iCurr != iPrimaryEdge);
}

void Hull::MarkVertexDirty(int iVertex)
{
  Vertex& V = m_aVertices[iVertex];

  V.SetDirty();                                       // Vertex itself is dirty

  if(V.cEdges() > 0){
    const int iPrimaryEdge = VertexiPrimaryEdge(iVertex);
    int iCurr = iPrimaryEdge;

    do{                                                      // Mark neighbours
      const Edge& E = m_aEdges[iCurr];
      int bRight = (E.iVertex(1) == iVertex);

      m_aVertices[E.iVertex(!bRight)].SetDirty();      // Other vertex is dirty

      if(E.iPred(bRight) == iCurr){

        assert(E.iFacet(!bRight) != -1);

        m_aFacets[E.iFacet(!bRight)].SetDirty();              // Facet is dirty

        break;
      }
      else{
        assert(E.iFacet(bRight) != -1);

        m_aFacets[E.iFacet(bRight)].SetDirty();               // Facet is dirty

        iCurr = E.iPred(bRight);
      }

    }while(iCurr != iPrimaryEdge);
  }
}

int Hull::AddNormal(const Vector3D& aNormal)
{
  int iPos = m_aNormals.cElements();

  m_aNormals.Append(aNormal);

  return(iPos);
}

int Hull::AddUVW(const Vector3D& aUVW)
{
  int iPos = m_aUVWs.cElements();

  m_aUVWs.Append(aUVW);

  return(iPos);
}

int Hull::AddColour(const Vector3D& aColour)
{
  int iPos = m_aColours.cElements();

  m_aColours.Append(aColour);

  return(iPos);
}

int Hull::AddEdge(const int iHead,
                  const int iTail,
                  const int iFacet)
{
  int iPos = iEdge(iHead, iTail);

  if(iPos == -1){
    Edge E;

    iPos = cEdges();

    E.SetiVertex(iHead, 1);
    E.SetiVertex(iTail, 0);
    E.SetiFacet(iFacet, 1);
    E.MakeRing(iPos);

    m_aEdges.Append(E);

    SetiEdge(iHead, iTail, iPos);

    m_aVertices[iHead].AddEdge(iPos);
    m_aVertices[iTail].AddEdge(iPos);
  }
  else{
    Edge& E = m_aEdges[iPos];

    if(E.iFacet(1) == -1)
      E.SetiFacet(iFacet, 1);
    else if(E.iFacet(0) == -1)
      E.SetiFacet(iFacet, 0);
  }

  return(iPos);
}

int Hull::iEdge(int iHead,
                int iTail) const
{
  long hash, maxhash = m_cEdgeHashes - 1;
  int Count = 0;
  int iEdge;

  if(iTail < iHead){ int tmp = iHead; iHead = iTail; iTail = tmp; }

  hash = EdgeHash(iHead, iTail);

  while((iEdge = m_aEdgeHashes[hash & maxhash]) != -1){
    const Edge& E = m_aEdges[iEdge];

    if(((E.iVertex(0) == iHead) && (E.iVertex(1) == iTail)) ||
       ((E.iVertex(1) == iHead) && (E.iVertex(0) == iTail)))
      return(iEdge);

    hash++;

    if(++Count == m_cEdgeHashes) break;
  }

  return(-1);
}

void Hull::SetiEdge(int iHead, int iTail, int iEdge)
{
  if(m_cEdgeHashes > 0){
    if(m_cFreeEdgeHashes == 0)
      InitHash(m_cEdgeHashes * 2);

    long hash, maxhash = m_cEdgeHashes - 1, iPos;

    if(iTail < iHead){ int tmp = iHead; iHead = iTail; iTail = tmp; }

    hash = EdgeHash(iHead, iTail);

    while(m_aEdgeHashes[iPos = (hash & maxhash)] != -1)
      hash++;

    m_aEdgeHashes[iPos] = iEdge;

    m_cFreeEdgeHashes--;
  }
}

void Hull::InitHash(const int cHashes)
{
  int bHadTable = (m_aEdgeHashes != NULL);

  if(bHadTable) delete [] m_aEdgeHashes;

  if(cHashes > 0){
    int cReserve = 1;
    int Count = cHashes;

    while(cReserve < Count)                              // Get iNext power of 2
      cReserve <<= 1;

    m_aEdgeHashes = new int[cReserve];
    m_cEdgeHashes = cReserve;
    m_cFreeEdgeHashes = cReserve;

    for(Count = 0; Count < cReserve; Count++)                   // Clear hashes
      m_aEdgeHashes[Count] = -1;

    if(bHadTable){                                                // Rebuild it
      for(Count = 0; Count < cEdges(); Count++){
        Edge& E = m_aEdges[Count];

        SetiEdge(E.iVertex(1), E.iVertex(0), Count);
      }
    }
  }
  else{
    m_cEdgeHashes = 0;
    m_cFreeEdgeHashes = 0;
    m_aEdgeHashes = NULL;
  }
}


int Hull::FacetSplit(const int iFacet, int iA, int iB)
{
  Facet& Old = m_aFacets[iFacet];
  int iNewEdge;

  if(iA > iB){ int tmp = iA; iA = iB; iB = tmp; }

  if((iNewEdge = iEdge(Old.aVertices(iA), Old.aVertices(iB))) == -1){
    int cVertices = iB - iA + 1;
    int Count, Index;

    FacetUnlinkEdges(iFacet);                               // Unlink old facet

    Facet New(Old.cVertices() - cVertices + 2); // New: 0, 1, .. a, b, .. n - 1

    for(Count = 0; Count <= iA; Count++){                  // Prepare new facet
      New.aVertices(Count) = Old.aVertices(Count);
      New.aVertexUVWs(Count) = Old.aVertexUVWs(Count);
      New.aVertexColours(Count) = Old.aVertexColours(Count);
    }
    for(Count = iB, Index = iA + 1; Count < Old.cVertices(); Count++, Index++){
      New.aVertices(Index) = Old.aVertices(Count);
      New.aVertexUVWs(Index) = Old.aVertexUVWs(Count);
      New.aVertexColours(Index) = Old.aVertexColours(Count);
    }

    AddFacet(New);                                             // Add new facet

    iNewEdge = cEdges() - 1;                            // This is the new edge

    m_aEdges[iNewEdge].SetInvisible();      // New edge is invisible by default

    Facet& Old = m_aFacets[iFacet];          // Old facet could have rellocated

                                                // Old becomes: a, a + 1, ... b
    for(Count = iA, Index = 0; Count <= iB; Count++, Index++){
      Old.aVertices(Index) = Old.aVertices(Count);
      Old.aVertexNormals(Index) = Old.iNormal();
      Old.aVertexUVWs(Index) = Old.aVertexUVWs(Count);
      Old.aVertexColours(Index) = Old.aVertexColours(Count);
    }

    Old.TruncateaVertices(cVertices);                 // Fix number of vertices

    FacetNormalRethink(iFacet);

    FacetLinkEdges(iFacet);                                 // Relink old facet
  }

  return(iNewEdge);
}

const float Hull::FacetArea(const int iFacet)
{
  const Facet& F = m_aFacets[iFacet];
  float TotalArea = 0.0;

  if(cVertices() >= 3){
    const Vector3D& A = m_aVertices[F.aVertices(0)].Vector();
    const Vector3D& B = m_aVertices[F.aVertices(1)].Vector();
    const Vector3D& C = m_aVertices[F.aVertices(2)].Vector();
    Vector3D BA = B - A;
    Vector3D CA = C - A;
    Vector3D N = CA % BA;

    TotalArea = N.Magnitude();

    int Count;

    for(Count = 3; Count < F.cVertices(); Count++){
      const Vector3D& T = m_aVertices[F.aVertices(Count)].Vector();
      Vector3D TA = T - A;

      N = TA % CA;

      TotalArea += N.Magnitude();
    }
  }

  return(0.5f * TotalArea);
}

void Hull::Triangulate(int bConvex)
{
  if(bConvex){
    int Count;

    for(Count = 0; Count < cFacets(); Count++)
      while(m_aFacets[Count].cVertices() > 3)
        FacetSplit(Count, 0, 2);
  }
  else{                                                                 // TODO
    assert(bConvex);
  }
}

void Hull::Smooth(const float CreaseAngle,
                  const int CreaseSharpness)
{
  int iVertex;

  assert(CreaseAngle >= 0.0f);

  const float cosCreaseAngle2 = (float)cos(CreaseAngle / 2.0);

  for(iVertex = 0; iVertex < cVertices(); iVertex++){
    Vector3D Normal;
    const int iPrimaryEdge = VertexiPrimaryEdge(iVertex);
    int iCurr = iPrimaryEdge;

    do{
      const Edge& E = m_aEdges[iCurr];
      int bRight = (E.iVertex(1) == iVertex);

      if(E.iPred(bRight) == iCurr){

        const int iFacet = E.iFacet(!bRight);

        assert(iFacet != -1);

        const int iNormal = m_aFacets[iFacet].iNormal();

        Normal += m_aNormals[iNormal] * FacetArea(iFacet);

        break;
      }
      else{

        assert(E.iFacet(bRight) != -1);

        const int iFacet = E.iFacet(bRight);

        assert(iFacet != -1);

        const int iNormal = m_aFacets[iFacet].iNormal();

        Normal += m_aNormals[iNormal] * FacetArea(iFacet);

        iCurr = E.iPred(bRight);
      }

    }while(iCurr != iPrimaryEdge);

    Normal.Unit();

    int iNormal = cNormals();
    int bAddNormal = 0;

    iCurr = iPrimaryEdge;

    do{
      Edge& E = m_aEdges[iCurr];
      int bRight = (E.iVertex(1) == iVertex);
      int iPos;

      if(E.iPred(bRight) == iCurr){

        assert(E.iFacet(!bRight) != -1);

        Facet& F = m_aFacets[E.iFacet(!bRight)];

        iPos = F.iVertexPos(iVertex);

        assert(iPos != -1);

        if(Normal * m_aNormals[F.iNormal()] < cosCreaseAngle2){
          bAddNormal = 1;
          F.aVertexNormals(iPos) = iNormal;
        }
        else{
          E.SetSharpness(CreaseSharpness);
          F.aVertexNormals(iPos) = F.iNormal();
        }

        break;
      }
      else{

        assert(E.iFacet(bRight) != -1);

        Facet& F = m_aFacets[E.iFacet(bRight)];

        iPos = F.iVertexPos(iVertex);

        assert(iPos != -1);

        if(Normal * m_aNormals[F.iNormal()] < cosCreaseAngle2){
          bAddNormal = 1;
          F.aVertexNormals(iPos) = iNormal;
        }
        else{
          E.SetSharpness(CreaseSharpness);
          F.aVertexNormals(iPos) = F.iNormal();
        }

        iCurr = E.iPred(bRight);
      }

    }while(iCurr != iPrimaryEdge);

    if(bAddNormal)
      AddNormal(Normal);
  }
}

void Hull::DebugVertex(FILE* fd, const int iVertex) const
{
  const Vertex& V = m_aVertices[iVertex];

  fprintf(fd, "%d: ", iVertex);

  V.Debug(fd);

  if(V.cEdges() > 0){
    const int iPrimaryEdge = VertexiPrimaryEdge(iVertex);
    int iCurr = iPrimaryEdge;

    fprintf(fd, "  %d -> [", iCurr);

    do{
      const Edge& E = m_aEdges[iCurr];
      int bRight = (E.iVertex(1) == iVertex);

      fprintf(fd, " %d", iCurr);

      if(E.iPred(bRight) == iCurr) break; else iCurr = E.iPred(bRight);

    }while(iCurr != iPrimaryEdge);

    fprintf(fd, "]\n");
  }
}

void Hull::DebugEdge(FILE* fd, const int iEdge) const
{
  fprintf(fd, "%d: ", iEdge);

  m_aEdges[iEdge].Debug(fd);
}

void Hull::DebugFacet(FILE* fd, const int iFacet) const
{
  const Facet& F = m_aFacets[iFacet];

  fprintf(fd, "%d: ", iFacet);

  F.Debug(fd);

  const int iPrimaryEdge = FacetiPrimaryEdge(iFacet);
  int iCurr = iPrimaryEdge;

  fprintf(fd, "  %d -> [", iCurr);

  do{
    const Edge& E = m_aEdges[iCurr];

    fprintf(fd, " %d", iCurr);

    iCurr = E.iSucc(E.iFacet(1) == iFacet);

  }while(iCurr != iPrimaryEdge);

  fprintf(fd, "]\n");
}

void Hull::DebugFacetFull(FILE* fd, const int iFacet) const
{
  DebugFacet(fd, iFacet);

  const Facet& F = m_aFacets[iFacet];
  int Count;

  for(Count = 0; Count < F.cVertices(); Count++)
    DebugVertex(fd, F.aVertices(Count));

  const int iPrimaryEdge = FacetiPrimaryEdge(iFacet);
  int iCurr = iPrimaryEdge;

  do{
    const Edge& E = m_aEdges[iCurr];

    DebugEdge(fd, iCurr);

    iCurr = E.iSucc(E.iFacet(1) == iFacet);

  }while(iCurr != iPrimaryEdge);
}

void Hull::Debug(FILE* fd) const
{
  int Count;

  fprintf(fd, "hull: \"%s\"\n",
          (m_strName != NULL) ? m_strName : "N/A");

  fprintf(fd, "edges (%d):\n", cEdges());

  for(Count = 0; Count < cEdges(); Count++)
    DebugEdge(fd, Count);

  fprintf(fd, "vertices (%d): [x, y, z] nedges [edges] primaryedge\n",
          cVertices());

  for(Count = 0; Count < cVertices(); Count++)
    DebugVertex(fd, Count);

  fprintf(fd, "normals (%d): [x, y, z]\n", cNormals());

  for(Count = 0; Count < cNormals(); Count++){

    fprintf(fd, "%d: ", Count);

    m_aNormals[Count].Debug(fd);
  }

  fprintf(fd, "uvws (%d): [u, v, w]\n", cUVWs());

  for(Count = 0; Count < cUVWs(); Count++){

    fprintf(fd, "%d: ", Count);

    m_aUVWs[Count].Debug(fd);
  }

  fprintf(fd, "colours (%d): [u, v, w]\n", cColours());

  for(Count = 0; Count < cColours(); Count++){

    fprintf(fd, "%d: ", Count);

    m_aColours[Count].Debug(fd);
  }

  fprintf(fd, "facets (%d): nvertices [vertices] [normals] [uvws] [colours] normal\n",
          cFacets());

  for(Count = 0; Count < cFacets(); Count++)
    DebugFacet(fd, Count);
}

void Hull::Stats(FILE* fd) const
{
  int cActualVertices = 0;
  int cActualEdges = 0;
  int cActualFacets = 0;
  int Count;

  for(Count = 0; Count < cVertices(); Count++)
    if(m_aVertices[Count].cEdges() > 0)
      cActualVertices++;

  for(Count = 0; Count < cEdges(); Count++){
    const Edge& E = m_aEdges[Count];

    if((E.iFacet(0) != -1) && (E.iFacet(1) != -1))
      cActualEdges++;
  }

  for(Count = 0; Count < cFacets(); Count++)
    if(m_aFacets[Count].cVertices() > 0)
      cActualFacets++;

  fprintf(fd, "V(%d) E(%d) F(%d) N(%d) T(%d) C(%d)\n",
          cActualVertices,
          cActualEdges,
          cActualFacets,
          cNormals(),
          cUVWs(),
          cColours());
}
