/***********************************************************************
This software module was originally developed by Dr. Michael Steliaros,
Superscape (UK) Ltd., in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Superscape (UK) Ltd. retains full
right to use the code for his/her own purpose, assign or donate the code
to a third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2001
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Dr. Michael Steliaros at msteliaros@superscape.com
----------------------------------------------------------------------*/
////---------------------------------------------------------------------
////    Class definition file:  IMAGENODE.CPP
////    Associated header file: IMAGENODE.H
////
////    Author:					Sean T Ellis
////    Creation date:			10/06/1999
////---------------------------------------------------------------------

#include <math.h>
#include <string.h>
#include "memory.h"

#include "imagenode.h"
#include "vector3d.h"   // For PI

float CssWarpMap::Warp(float f)
{
	int n=m_xVals.cElements();
	
	if (n==0)
		return 0.5f;
	if (n==1)
		return (m_yVals[0]);

	if (m_bSmooth)
	{
		const float MinDx=0.001f;

		if (f<m_xVals[0])
		{
			float dy=m_yVals[1]-m_yVals[0];
			float dx=m_xVals[1]-m_xVals[0];
			if (dx<MinDx)
				dx=MinDx;
			f=m_yVals[0]-(m_xVals[0]-f)*dy/dx;
			return f;
		}
		if (f>m_xVals[n-1])
		{
			float dy=m_yVals[n-1]-m_yVals[n-2];
			float dx=m_xVals[n-1]-m_xVals[n-2];
			if (dx<MinDx)
				dx=MinDx;
			f=m_yVals[n-1]+(f-m_xVals[n-1])*dy/dx;
			return f;
		}
		for (int i=0; i<n-1; i++)
		{
			if (f>=m_xVals[i] && f<=m_xVals[i+1] && m_xVals[i]!=m_xVals[i+1])
			{
				float y0,y1,y2,y3;
				float dx0,dx1,dx2;

				dx1=(m_xVals[i+1]-m_xVals[i]);

				f = (f-m_xVals[i])/dx1;
	
				y1=m_yVals[i];
				y2=m_yVals[i+1];
				if (i>1)
				{
					y0=m_yVals[i-1];
					dx0=m_xVals[i]-m_xVals[i-1];
				}
				else
				{
					y0=y1-(y2-y1);
					dx0=dx1;
				}
				
				if (i<n-2)
				{
					y3=m_yVals[i+2];
					dx2=m_xVals[i+2]-m_xVals[i+1];
				}
				else
				{
					y3=y2+(y2-y1);
					dx2=dx1;
				}

				if (dx0<MinDx)
					dx0=MinDx;
				if (dx2<MinDx)
					dx2=MinDx;

				float m0=(y1-y0)*dx1/dx0;
				float m1=(y2-y1)*dx1/dx1;
				float m2=(y3-y2)*dx1/dx2;

				m0=(m0+m1)*0.5f;
				m2=(m2+m1)*0.5f;
				
				float f2=f*f;
				float f3=f*f2;

				// Create a Hermite spline

				float y=y1*(2*f3-3*f2+1);
				y+=y2*(-2*f3+3*f2);
				y+=m0*(f3-2*f2+f);
				y+=m2*(f3-f2);
				return y;
			}
		}
	}
	else
	{
		if (f<m_xVals[0])
			return m_yVals[0];
		if (f>m_xVals[n-1])
			return m_yVals[n-1];
		for (int i=0; i<n-1; i++)
		{
			if (f>=m_xVals[i] && f<=m_xVals[i+1] && m_xVals[i]!=m_xVals[i+1])
			{
				f = (f-m_xVals[i])/(m_xVals[i+1]-m_xVals[i]);
				return m_yVals[i]+f*(m_yVals[i+1]-m_yVals[i]);
			}
		}
		return m_yVals[n-1];
	}
	return 0.0f;	// to shut compiler up
}

// return seeded random number, in range 0..1
float CssPlasma::SeededRandom(float s)
{
	unsigned int ci=(unsigned int)(s*0xFFFF);
	ci=ci*1664525L+1013904223L;
	ci=ci*1664525L+1013904223L;
	ci=ci*1664525L+1013904223L;
	union {	
		float Float; 
		int Int; // MBS: VC++ 6.0 bug: should be unsigned int
	} IEEEFloatConverter;

	IEEEFloatConverter.Int= (ci & 0x7FFFFF) | 0x3F800000;
	return IEEEFloatConverter.Float-1;
}

///---------------------------------------------------------------------
///
/// CssPlasma::ReCalculate
/// Recache the data for a plasma image. This will create a new image
/// based on the various plasma parameters

void CssPlasma::ReCalculate()
{
	if (!m_bDirty)
		return;

	if(m_cSubdivWidth>0 && m_cSubdivHeight>0)
	{
		int cPixels=1<<(m_cSubdivWidth+m_cSubdivHeight);

		// First we generate a rectangular array of blend values. The values form a kind
		// of fractal patch, looking a bit like a mountain range.
		DynArray<float> aBlendValues;

		// the pseudorandom number generator is seeded using the given seed value
		// This means that a particular plasma will look the same each time
		// it is generated

		// The blend array is filled as follows:
		// Generate an initial zero offset for the top left corner. The fact that the image
		// wraps in x and y will reflect this value to the other 3 corners.
		// Set the stride to be the shorter of the image width and height.
		// Fill the longer edge with zeroes at each 'stride' interval. These will be reflected
		// to the opposite edge.
		// For each level of subdivision, create 3 new values per cell of side 'stride'. These
		// values will go at the midpoints of the top, left and diagonal of the old cell, so
		// dividing each cell into four new ones by the end of the current iteration. The values
		// given to the new points are scaled random offsets from the mean of the adjacent points
		// in the previous subdivision level, where the scale is halved each time
		// the subdivision level is decreased. This produces the fractal characteristic of the
		// plasma. (The roughness value determines at which subdivision level the scale starts
		// decreasing). The whole process stops when every element of the array has been filled in.
		
		m_NextRandomValue=m_Seed;

		m_ImageData.m_Width =1<<m_cSubdivWidth;
		m_ImageData.m_Height=1<<m_cSubdivHeight;

		m_aPixels.Truncate(cPixels * 3);
		m_ImageData.m_pColors=&m_aPixels[0];

		aBlendValues.Truncate(cPixels);
		// set up the top left corner (and thus the other 3 corners too)
		aBlendValues[0]=0;

		m_fMaxVariance=0.0f;

		{
			int w=m_ImageData.m_Width;
			int h=m_ImageData.m_Height;
			int height=h;
			int width=w;
			int step;
			int roughness=height;
			if (width>height)
				roughness=width;
			roughness>>=m_cRoughness;
			
			float rmult=1.0f;
			while (w>h)
			{
				step=w>>1;

				if (w<roughness)
					rmult*=0.5f;

				int p1=0;
				while (p1<width)
				{
					aBlendValues[p1+step]=0.0f;
					p1+=w;
				}
				w=w>>1;
			}
			while (h>w)
			{
				step=h>>1;

				if (h<roughness)
					rmult*=0.5f;

				int p1=0;
				while (p1<height)
				{
					aBlendValues[(p1+step)*width]=0.0f;
					p1+=h;
				}
				h=h>>1;
			}
			// here, w==h
			// width and height are powers of 2
			while (w>1)
			{
				step=w>>1;
				if (w<roughness)
				{
					rmult*=0.5f;
				}
				for (int y=0; y<height; y+=w)
				{
					for (int x=0; x<width; x+=w)
					{
						int x1=x;
						int x2=(x+w)&(width-1);
						int x3=x+step;
						int y1=width*y;
						int y2=width*((y+w)&(height-1));
						int y3=width*(y+step);

						float v1=Random();
						float v2=Random();
						float v3=Random();

						float v4=(v1+v2+v3)*0.333333f;
						v1-=v4; v2-=v4; v3-=v4;	// make v1+v2+v3=0

						if (step<roughness)
						{
							v1=v1*rmult+(aBlendValues[x1+y1]+aBlendValues[x2+y1])*0.5f;
							v2=v2*rmult+(aBlendValues[x1+y1]+aBlendValues[x1+y2])*0.5f;
							v3=v3*rmult+(aBlendValues[x1+y1]+aBlendValues[x2+y1]+aBlendValues[x1+y2]+aBlendValues[x2+y2])*0.25f;
						}

						aBlendValues[x3+y1]=v1;
						aBlendValues[x1+y3]=v2;
						aBlendValues[x3+y3]=v3;
					
						if (v1>m_fMaxVariance)
							m_fMaxVariance=v1;
						else if (-v1>m_fMaxVariance)
							m_fMaxVariance=-v1;
						
						if (v2>m_fMaxVariance)
							m_fMaxVariance=v2;
						else if (-v2>m_fMaxVariance)
							m_fMaxVariance=-v2;
						
						if (v3>m_fMaxVariance)
							m_fMaxVariance=v3;
						else if (-v3>m_fMaxVariance)
							m_fMaxVariance=-v3;

					}
				}
				w=w>>1;
			}

		}

		// Now we'll use the blend values to generate the various 'plasma' bitmaps.
		// Basically we split the bitmap into various shaped cells, and assign colours
		// to each pixel using combinations of the blend value and geometrical values
		// such as distance from cell centre etc.
		float Scale=0.49f/m_fMaxVariance;	// 0.49 to give a bit of leeway for rounding
		{
			// Modify the plasma

			// The first phase generates the cells, deriving a variety of values which are stored
			// in the PlasmaFields array. Two sets of weights are used to make combinations of
			// these values, and the two resulting values are used to interpolate between the four
			// plasma colours

//			CssColor PreMultipliedCol[4];
//			float Alpha[4];
//			for (int clr=0; clr<4; clr++)
//			{
//				Alpha[clr]=m_col[clr].Alpha();
//				PreMultipliedCol[clr]=CssColor(m_col[clr].GetPackedRGBA8())*Alpha[clr];
//			}
		CssColor col01=m_col[0]-m_col[1];
		CssColor col23=m_col[2]-m_col[3];

			int cellwidth=(1<<m_CellSubdivWidth);
			int cellheight=(1<<m_CellSubdivHeight);

			int cellx,celly,celltype;
			float offsetx,offsety;

			int w=m_ImageData.m_Width;
			int h=m_ImageData.m_Height;

			// w,h,cPixels are all powers of 2
			int wmask=(w-1);
			int hmask=(h-1);
			int pixmask=(cPixels-1);

			float PlasmaFields[PF_NUMFIELDS];

			float TotalAWeights=0;
			float TotalBWeights=0;
			float rTotalAWeights=0;
			float rTotalBWeights=0;
                        int j;

			for (j=0; j<PF_NUMFIELDS-2; j++)
			{
				TotalAWeights+=static_cast<float>(fabs(m_aWeights[j]));
				TotalBWeights+=static_cast<float>(fabs(m_bWeights[j]));
			}
			if (TotalAWeights!=0)
				rTotalAWeights=1.0f/TotalAWeights;
			else
				rTotalAWeights=1.0f;
			if (TotalBWeights!=0)
				rTotalBWeights=1.0f/TotalBWeights;
			else
				rTotalBWeights=1.0f;

			float rTotalAWeights2=TotalAWeights;
			float rTotalBWeights2=TotalBWeights;
			for (j=PF_NUMFIELDS-2; j<PF_NUMFIELDS; j++)
			{
				rTotalAWeights2+=static_cast<float>(fabs(m_aWeights[j]));
				rTotalBWeights2+=static_cast<float>(fabs(m_bWeights[j]));
			}
			if (rTotalAWeights2!=0)
				rTotalAWeights2=1.0f/rTotalAWeights2;
			if (rTotalBWeights2!=0)
				rTotalBWeights2=1.0f/rTotalBWeights2;

			float rcw=1.0f/cellwidth;
			float rch=1.0f/cellheight;
			float rw=1.0f/w;
			float rh=1.0f/h;

			for (int y=0; y<h; y++)
			{
				float yf=static_cast<float>(y)*rh;

				PlasmaFields[PF_Y]=yf;

				for (int x=0; x<w; x++)
				{
					float xf=static_cast<float>(x)*rw;
					float xWarp=xf-m_xWarpMap.Warp(yf)+0.5f;
					float yWarp=yf-m_yWarpMap.Warp(xf)+0.5f;

					PlasmaFields[PF_X]=xf;
					PlasmaFields[PF_XWarp]=xWarp;
					PlasmaFields[PF_YWarp]=yWarp;

					int i=x+w*y;
					float f1=Scale*aBlendValues[(i+(cPixels>>2))&pixmask]; // -0.5 .. +0.5
					float f2=Scale*aBlendValues[(i+(cPixels>>1))&pixmask]; // -0.5 .. +0.5

					// dx and dy are distorted versions of x and y. The distortion is
					// generated by looking at various offsets in the blend array.
					int dx=(static_cast<int>((xWarp+f1*0.5*m_Distortion)*w+w+0.5f))&wmask;
					int dy=(static_cast<int>((yWarp+f2*0.5*m_Distortion)*h+h+0.5f))&hmask;

					PlasmaFields[PF_XDist]=static_cast<float>(dx)*rw;
					PlasmaFields[PF_YDist]=static_cast<float>(dy)*rh;

					int di=dx+w*dy;

					switch(m_PlasmaType)
					{
					default:
					case PlasmaCell_Rectangle: // checker variants
					case PlasmaCell_Brick:
					case PlasmaCell_Weave:
						{
							if (m_PlasmaType==PlasmaCell_Brick)
							{
								// staggered square array
//								celly=dy/cellheight;
							celly=dy>>m_CellSubdivHeight;
//								int xoff=((celly&1)*cellwidth)>>1;
							int xoff=((celly&1)<<m_CellSubdivWidth)>>1;
//								cellx=(dx+cellwidth-xoff)/cellwidth-1;
							cellx=((dx+cellwidth-xoff)>>m_CellSubdivWidth)-1;
//								cellx*=cellwidth;
							cellx<<=m_CellSubdivWidth;
								cellx+=xoff;
//								celly*=cellheight;
							celly<<=m_CellSubdivHeight;
							}
							else
							{
								// just use a square grid.
//								cellx=dx/cellwidth;
//								celly=dy/cellheight;
							cellx=dx>>m_CellSubdivWidth;
							celly=dy>>m_CellSubdivHeight;
								celltype=(cellx^celly)&1;

//								cellx*=cellwidth;
//								celly*=cellheight;
							cellx<<=m_CellSubdivWidth;
							celly<<=m_CellSubdivHeight;
							}


							if (m_PlasmaType==PlasmaCell_Weave && celltype==1)
							{
								// for a weave, alternate cells are rotated 90 degrees
								PlasmaFields[PF_YOffset]=(dx-cellx)*rcw;
								PlasmaFields[PF_XOffset]=(dy-celly)*rch;
							}
							else
							{
								PlasmaFields[PF_XOffset]=(dx-cellx)*rcw;
								PlasmaFields[PF_YOffset]=(dy-celly)*rch;
							}

							cellx+=(cellwidth)>>1;
							celly+=(cellheight)>>1;

							offsetx=(dx-cellx)*2.0f*rcw;
							offsety=(dy-celly)*2.0f*rch;

//							PlasmaFields[PF_Edgewards]=static_cast<float>((fabs(offsetx)>fabs(offsety))?fabs(offsetx):fabs(offsety));
							{
								float ox=offsetx;
								float oy=offsety;
								ox = (float)fabs(ox);
								oy = (float)fabs(oy);
								PlasmaFields[PF_Edgewards]=ox>oy?ox:oy;
							}
						}
						break;
					case PlasmaCell_Hex: // hex
						{
							// the following code determines which hexagonal cell the
							// target point falls in. The various 5's 8's and 16's
							// are used to deform a staggered square (brickwall) array
							// into a hex grid.
//							celly=dy/cellheight;
						celly=dy>>m_CellSubdivHeight;
//							int xoff=((celly&1)*cellwidth)>>1;
						int xoff=((celly&1)<<m_CellSubdivWidth)>>1;
//							cellx=(dx+cellwidth-xoff)/cellwidth-1;
						cellx=((dx+cellwidth-xoff)>>m_CellSubdivWidth)-1;
//							cellx*=cellwidth;
						cellx<<=m_CellSubdivWidth;
							cellx+=xoff;
//							celly*=cellheight;
						celly<<=m_CellSubdivHeight;
							int x1=dx-cellx;
							int y1=dy-celly;
							if (y1>=(cellheight*5)>>3)
							{
								y1-=(cellheight*5)>>3;
								if (x1<(cellwidth>>1))
								{
									if (4*cellwidth*y1>3*cellheight*x1)
									{
										celly+=cellheight;
										cellx-=cellwidth>>1;
									}
								}
								else
								{
									x1-=cellwidth;
									if (4*cellwidth*y1>-3*cellheight*x1)
									{
										celly+=cellheight;
										cellx+=cellwidth>>1;
									}
								}
							}
							// get offsets from bottom left of hexagonal cell

							PlasmaFields[PF_XOffset]=(dx-cellx)*rcw;
							PlasmaFields[PF_YOffset]=((dy-celly)*rch*16.0f+3.0f)/19.0f;

							// Getthe coords of the centre of the cell
							cellx+=(cellwidth>>1);
							celly+=(cellheight*5)>>4;

							float ex=(dx-cellx)*rcw;
							float ey=(dy-celly)*rch;

							offsetx=ex*2.0f;
							offsety=ey*2.0f;

							cellx=cellx&wmask;
							celly=celly&hmask;

							if (ex<0)
								ex=-ex;
							if (ey<0)
								ey=-ey;
							if (8*ey<5*ex)
								PlasmaFields[PF_Edgewards]=ex*2.0f;
							else
								PlasmaFields[PF_Edgewards]=(16.0f*ey+12.0f*ex)/11.0f;
						}
						break;
					case PlasmaCell_Ring: // ring
						{
							// concentric rings
							int dxc=dx-(w>>1);
							int dyc=dy-(h>>1);

							offsetx=dxc*rcw;
							offsety=dyc*rch;

							float r=static_cast<float>(sqrt(offsetx*offsetx+offsety*offsety));
							int ri=static_cast<int>(r);

							PlasmaFields[PF_Edgewards]=r-static_cast<float>(ri);

							ri+=1;

//							cellx=(w>>1)-ri*cellwidth;
//							celly=(h>>1)-ri*cellheight;
						cellx=(w>>1)-(ri<<m_CellSubdivWidth);
						celly=(h>>1)-(ri<<m_CellSubdivHeight);

							float rri=0.5f/ri;

							PlasmaFields[PF_XOffset]=(dx-cellx)*rri*rcw;
							PlasmaFields[PF_YOffset]=(dy-celly)*rri*rch;

							rri*=2.828f;

							offsetx*=rri;
							offsety*=rri;
						}
						break;
					}
					PlasmaFields[PF_Radial]=static_cast<float>(sqrt(offsetx*offsetx+offsety*offsety))*0.7071f;
					PlasmaFields[PF_Angle]=static_cast<float>(atan2(offsetx, offsety))/(PI*2.0f)+0.5f;

					cellx=cellx&wmask;
					celly=celly&hmask;

					PlasmaFields[PF_CellRand]=SeededRandom(aBlendValues[cellx+celly*w]);
					PlasmaFields[PF_PixelRand]=SeededRandom(aBlendValues[x+y*w]);

					PlasmaFields[PF_CellPlasma]=Scale*aBlendValues[cellx+celly*w]+0.5f;
					PlasmaFields[PF_PixelPlasma]=Scale*aBlendValues[di]+0.5f;
					PlasmaFields[PF_PixelPlasma2]=Scale*aBlendValues[(cPixels+(cPixels>>1)-di)&pixmask]+0.5f;

					// get weighted sums of values
					int j;
					float a=0,b=0;
					for (j=0; j<PF_NUMFIELDS-2; j++)
					{
						a+=m_aWeights[j]*(PlasmaFields[j]-0.5f);
						b+=m_bWeights[j]*(PlasmaFields[j]-0.5f);
					}
					a=(a*rTotalAWeights)+0.5f;
					b=(b*rTotalBWeights)+0.5f;

					// warp a and b values

					a=m_aWarpMap.WarpClamped(a);
					b=m_bWarpMap.WarpClamped(b);

					// merge in the last two values

					a=(a-0.5f)*TotalAWeights;
					b=(b-0.5f)*TotalBWeights;

					for (j=PF_NUMFIELDS-2; j<PF_NUMFIELDS; j++)
					{
						a+=m_aWeights[j]*(PlasmaFields[j]-0.5f);
						b+=m_bWeights[j]*(PlasmaFields[j]-0.5f);
					}
					a=(a*rTotalAWeights2)+0.5f;
					b=(b*rTotalBWeights2)+0.5f;

					// finally, use the two values to interpolate between the four colours.

//					CssColor col1=PreMultipliedCol[1] + (PreMultipliedCol[0]-PreMultipliedCol[1])*a;
//					CssColor col2=PreMultipliedCol[3] + (PreMultipliedCol[2]-PreMultipliedCol[3])*a;
//					float alp1=Alpha[1] + (Alpha[0]-Alpha[1])*a;
//					float alp2=Alpha[3] + (Alpha[2]-Alpha[3])*a;
//					col1=col1+(col2-col1)*b;
//					alp1=alp1+(alp2-alp1)*b;
//					if (alp1>0)
//					{
//						col1/=alp1;
//					}
//					unsigned int c=col1.GetPackedRGBA8()&~ColorMask_Alpha;
//					c|=(static_cast<TssByte>(alp1*255)&0xFF)<<ColorShift_Alpha;
//					m_aPixels[i]=c;

					CssColor col1=m_col[1] + col01*a;
					CssColor col2=m_col[3] + col23*a;
					col1=col1+(col2-col1)*b;
					// OpenGL expects image pixels upside down.
					int rgb = ((h - y - 1) * w + x) * 3;
					m_aPixels[rgb++]=col1.Red8();
					m_aPixels[rgb++]=col1.Green8();
					m_aPixels[rgb]=col1.Blue8();
				}
			}
		}
	}
	m_bDirty=false;
}

///---------------------------------------------------------------------
///
/// CssPlasma::Random
/// Return a pseudo-random number in the range (-0.5...0.5].

float CssPlasma::Random()
{				   
	m_NextRandomValue=m_NextRandomValue*1664525L+1013904223L;
	
	union {	unsigned int Int; float Float; } IEEEFloatMaker;
	
	// Or in 23 bits of mantissa, now 1.0 .. 1.999+
	IEEEFloatMaker.Int=(m_NextRandomValue&0x7FFFFF)|0x3F800000;
	
	// Return -0.5 .. 0.499+
	return(IEEEFloatMaker.Float-1.5f);
}
