/***********************************************************************
This software module was originally developed by Dr. Michael Steliaros,
Superscape (UK) Ltd., in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Superscape (UK) Ltd. retains full
right to use the code for his/her own purpose, assign or donate the code
to a third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2001
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Dr. Michael Steliaros at msteliaros@superscape.com
----------------------------------------------------------------------*/
////---------------------------------------------------------------------
////    Associated header file: IMAGENODE.H
////    Class definition file:  IMAGENODE.CPP
////
////  Various classes for material nodes.
////
////    Author:          Sean T Ellis
////    Creation date:      11/06/1999
////---------------------------------------------------------------------

//? Is IMAGENODE.h included already?
#ifndef IMAGENODE_H_INCLUDED
#define IMAGENODE_H_INCLUDED 1

#include  <assert.h>
#include  "sscolor.h"
#include  "dynarray.h"

// plasma cell types
enum EssPlasmaCell
{
	PlasmaCell_Rectangle = 0,
	PlasmaCell_Brick,
	PlasmaCell_Weave,
	PlasmaCell_Hex,
	PlasmaCell_Ring,

	PlasmaCell_NextAvailable  // keep this updated when adding new plasma types
};


//-----------------------------------------------------------------------

struct SssImageData
{
	SssImageData() :
		m_Width(0), 
		m_Height(0), 
		m_pColors(NULL)
	{}

	int				m_Width;
	int				m_Height;

	unsigned char*	m_pColors;
};

class CssWarpMap
{
public:
	CssWarpMap()
	{
		m_bSmooth=0;
		m_xVals.Truncate(0);
		m_yVals.Truncate(0);
	}
	
	float Warp(float f);
	float WarpClamped(float f)
	{
		f=Warp(f);
		if (f<0)
			f=0;
		else if (f>1)
			f=1;
		return (f); 
	}
	
	bool  m_bSmooth;
	DynArray<float> m_xVals;
	DynArray<float> m_yVals;
};

///----------------------------------------------------------------------
///
///  CssPlasma
/// A generated Plasma-type image

class CssPlasma
{
public:
	enum EssPlasmaFields
	{
		PF_X = 0,
		PF_Y,
		PF_XWarp,
		PF_YWarp,
		PF_XDist,
		PF_YDist,
		PF_XOffset,
		PF_YOffset,
		PF_Radial,
		PF_Edgewards,
		PF_Angle,
		PF_CellPlasma,
		PF_PixelPlasma,
		PF_PixelPlasma2,
		PF_CellRand,
		PF_PixelRand,
		
		PF_NUMFIELDS
	};
	
	CssPlasma() :
		m_bDirty(true),
		m_cSubdivWidth(0),
		m_cSubdivHeight(0),
		m_cRoughness(0),
		m_Seed(0),
		m_Distortion(0),
		m_PlasmaType(0),
		m_CellSubdivWidth(0),
		m_CellSubdivHeight(0)
	{
		for (int i=0; i<PF_NUMFIELDS; i++)
			m_aWeights[i]=m_bWeights[i]=0;
		
		m_pWarpMap[0]=&m_xWarpMap;
		m_pWarpMap[1]=&m_yWarpMap;
		m_pWarpMap[2]=&m_aWarpMap;
		m_pWarpMap[3]=&m_bWarpMap;
	}
	
	~CssPlasma() { }
	
	void Setcol(int c, const CssColor& col)
	{ assert (c>=0 && c<4); m_col[c]=col; m_bDirty=true; }
	
	void SetcSubdivisions(int cSubdivisions)
	{ assert(cSubdivisions>=0); m_cSubdivWidth=m_cSubdivHeight=cSubdivisions; m_bDirty=true; }
	
	void SetCellSize(int CellSize)
	{ assert(CellSize>=0); m_CellSubdivWidth=m_CellSubdivHeight=CellSize; m_bDirty=true; }
	
	void SetcSubdivHoriz(int cSubdivisions)
	{ assert (cSubdivisions>=0); m_cSubdivWidth=cSubdivisions; m_bDirty=true; }
	
	void SetcSubdivVert(int cSubdivisions)
	{ assert (cSubdivisions>=0); m_cSubdivHeight=cSubdivisions; m_bDirty=true; }
	
	void SetCellWidth(int CellSize)
	{ assert(CellSize>=0); m_CellSubdivWidth=CellSize; m_bDirty=true; }
	
	void SetCellHeight(int CellSize)
	{ assert(CellSize>=0); m_CellSubdivHeight=CellSize; m_bDirty=true; }
	
	void SetcRoughness(int cRoughness)
	{ assert (cRoughness>=0); m_cRoughness=cRoughness; m_bDirty=true; }
	
	void SetPlasmaType(int plastype)
	{ assert(plastype>=0 && plastype<PlasmaCell_NextAvailable); m_PlasmaType=plastype; m_bDirty=true; }
	
	void SetDistortion(float Distortion)
	{ m_Distortion=Distortion; m_bDirty=true; }
	
	void SetAWeight(int n, float w)
	{ assert(n >= 0 && n < 16); m_aWeights[n]=w; m_bDirty=true; }
	
	void SetBWeight(int n, float w)
	{ assert(n >= 0 && n < 16); m_bWeights[n]=w; m_bDirty=true; }
	
	void SetSeed(unsigned int s)
	{ m_Seed=s; m_bDirty=true; }
	
	void SetWarp(int ctl, int xy, int n, float v)
	{
		assert(n<m_pWarpMap[ctl]->m_xVals.cElements());
		if (xy==0)
			m_pWarpMap[ctl]->m_xVals[n]=v;
		else
			m_pWarpMap[ctl]->m_yVals[n]=v;
		m_bDirty=true;
	}
	
	void SetWarpSmooth(int ctl, bool smooth)
	{ assert(ctl >= 0 && ctl <= 3); m_pWarpMap[ctl]->m_bSmooth=smooth; m_bDirty=true; }
	
	void SetNumWarpKnots(int ctl, int n)
	{ assert(ctl >= 0 && ctl <= 3); m_pWarpMap[ctl]->m_xVals.Truncate(n); m_pWarpMap[ctl]->m_yVals.Truncate(n); m_bDirty=true; }
	
	const SssImageData& GetImageData()
	{ if (m_bDirty) ReCalculate(); return(m_ImageData); }
	
protected:
	void ReCalculate();
	
	float Random();
	float SeededRandom(float s);
	
	bool m_bDirty;
	float m_fMaxVariance;
	
	int m_cSubdivWidth;
	int m_cSubdivHeight;
	int m_cRoughness;
	unsigned int m_Seed;
	CssWarpMap m_xWarpMap;
	CssWarpMap m_yWarpMap;
	float m_Distortion;
	int m_PlasmaType;
	int m_CellSubdivWidth;
	int m_CellSubdivHeight;
	float m_aWeights[PF_NUMFIELDS];
	float m_bWeights[PF_NUMFIELDS];
	CssWarpMap m_aWarpMap;
	CssWarpMap m_bWarpMap;
	
	CssColor m_col[4];
	
	SssImageData m_ImageData;
	DynArray<unsigned char> m_aPixels;
	unsigned int m_NextRandomValue;
	CssWarpMap* m_pWarpMap[4];
};

#endif  // !IMAGENODE_H_INCLUDED
