/***********************************************************************
This software module was originally developed by Dr. Michael Steliaros,
Superscape (UK) Ltd., in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Superscape (UK) Ltd. retains full
right to use the code for his/her own purpose, assign or donate the code
to a third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2001
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Dr. Michael Steliaros at msteliaros@superscape.com
----------------------------------------------------------------------*/
////---------------------------------------------------------------------
////    Associated header file: SSCOLOR.H
////    Class definition file:  SSCOLOR.CPP
////
////    Author:					Sean T Ellis
////    Creation date:			09/06/1997
////---------------------------------------------------------------------

//? Is sscolor.h included already?
#ifndef SSCOLOR_H_INCLUDED
#define SSCOLOR_H_INCLUDED 1

///----------------------------------------------------------------------
///
/// EssColorMask
/// EssColorShift
/// Color mask and shift values for packing RGBA values into a 32 bit int.
/// When stored in Intel LSB-at-lowest-address endianness, this is exactly
/// equivalent to the definition of Windows' RGBQUAD structure.

enum EssColorShift
{
  ColorShift_Blue =0,
  ColorShift_Green=8,
  ColorShift_Red  =16,
  ColorShift_Alpha=24,
};

enum EssColorMask
{
  ColorMask_Blue  =( 0xFF << ColorShift_Blue      ),
  ColorMask_Green =( 0xFF << ColorShift_Green     ),
  ColorMask_Red   =( 0xFF << ColorShift_Red       ),
  ColorMask_Alpha =( 0xFF << ColorShift_Alpha     ),
};


///----------------------------------------------------------------------
///
///	CssColor
///
/// Define class structure for RGB color.
///: Red		: Red component of vector.
///: SetRed		: Set red component of vector.
///: Red::Red	: New red component of vector.
///: Green		: Green component of vector.
///: SetGreen	: Set green component of vector.
///: Green::Green	: New green component of vector.
///: Blue		: Blue component of vector.
///: SetBlue	: Set blue component of vector.
///: Blue::Blue	: New blue component of vector.
///: m_Red		: Red component of vector.
///: m_Green	: Green component of vector.
///: m_Blue		: Blue component of vector.

class CssColor
{
	public:
		CssColor();
		CssColor(const CssColor &From);
		CssColor(float Red, float Green, float Blue);
		CssColor(unsigned int RGBA8);
		~CssColor();
		
		// Note: Ranges are not asserted here since it is possible for components to temporarily
		// exceed the range	0.0f...1.0f in intermediate results in calculations.

		//! Red(v) should really be SetRed(v) etc.
		
		float	Red() const				{return(m_Red);} 
		void		Red(float Red)		{m_Red=Red;}
		float	Green() const			{return(m_Green);} 
		void		Green(float Green)	{m_Green=Green;}
		float	Blue() const			{return(m_Blue);}
		void		Blue(float Blue)		{m_Blue=Blue;}

		unsigned char	Red8() const	{return(m_Red <= 0.0f ? 0 : m_Red >= 1.0f ? 255 : static_cast<unsigned char>(m_Red * 255.0f));} 
		unsigned char	Green8() const	{return(m_Green <= 0.0f ? 0 : m_Green >= 1.0f ? 255 : static_cast<unsigned char>(m_Green * 255.0f));} 
		unsigned char	Blue8() const	{return(m_Blue <= 0.0f ? 0 : m_Blue >= 1.0f ? 255 : static_cast<unsigned char>(m_Blue * 255.0f));} 

		unsigned int		GetPackedRGBA8()	const;
		void		SetPackedRGBA8(unsigned int RGB8);

		CssColor&	operator= (const CssColor &From);
		CssColor&	operator+=(const CssColor &From);
		CssColor&	operator-=(const CssColor &From);
		CssColor&	operator*=(const CssColor &From);
		CssColor&	operator*=(const float Scale);
		CssColor&	operator/=(const float Scale);

		CssColor	operator~ () const;				
		
		CssColor	operator+ (const CssColor &From) const;
		CssColor	operator- (const CssColor &From) const;
		CssColor	operator* (const CssColor &From) const;
		CssColor	operator* (const float Scale) const;
		CssColor	operator/ (const float Scale) const;
		bool		operator==(const CssColor &From) const {return (m_Red==From.m_Red && m_Green==From.m_Green && m_Blue==From.m_Blue);};

	protected:
		float	m_Red;
		float	m_Green;
		float	m_Blue;
};

#endif	// !SSCOLOR_H_INCLUDED
