/***********************************************************************
This software module was originally developed by Dr. Michael Steliaros,
Superscape (UK) Ltd., in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Superscape (UK) Ltd. retains full
right to use the code for his/her own purpose, assign or donate the code
to a third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2001
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Dr. Michael Steliaros at msteliaros@superscape.com
----------------------------------------------------------------------*/

////---------------------------------------------------------------------
////    Superscape subdivision surface storage class
////
////    Author:                 Michael K. Steliaros (Superscape)
////    Last modification date: 28-11-2000
////    Creation date:          22-09-2000
////---------------------------------------------------------------------

#ifndef SUBDIVISIONSURFACEXLOOP_H
#define SUBDIVISIONSURFACEXLOOP_H

#include <stdio.h>

#include "dynarray.h"
#include "hull.h"

class SubdivisionSurfaceXLoop
{
  public:
    SubdivisionSurfaceXLoop(const Hull& ControlHull, const int cLevels = 4) : m_bDirty(0)
    {
      m_aLevels.Truncate(cLevels + 1);
      m_aLevels[0] = ControlHull;

      Subdivide();
    }
    ~SubdivisionSurfaceXLoop() { m_aLevels.Truncate(0); }

    char* strName()
      const { return(m_aLevels[0].strName()); }

    int cLevels()
      const { return(m_aLevels.cElements()); }

    Hull& aLevels(int iPos) { return(m_aLevels[iPos]); }
    const Hull& aLevels(int iPos) const { return(m_aLevels[iPos]); }

    int cVertices()
      const { return(m_aLevels[0].cVertices()); }

    // SetVertexVector automatically sets the correct dirty flags
    void SetVertexVector(const int iVertex, const Vector3D& V);

    // None of the SetXXX function below set any dirty flags
    // MarkDirty MUST be called when done
    void SetNormal(const int iNormal, const Vector3D& N);
    void SetUVW(const int iUVW, const Vector3D& UVW);
    void SetColour(const int iColour, const Vector3D& RGB);

    void SetEdgeSharpness(const int iHead,
                          const int iTail,
                          const int Sharpness);
    int EdgeSharpness(const int iHead,
                      const int iTail) const
    {
      const Hull& Control = m_aLevels[0];
      int iEdge = Control.iEdge(iHead, iTail);

      return((iEdge != -1) ? m_aLevels[0].aEdges(iEdge).Sharpness() : -1);
    }

    // Mark entire control hull as dirty; should be followed by a Rethink
    void MarkDirty(void);

    // Partially recalculate affected dirty bits
    void Rethink();

    void Debug(FILE* fd) const;

  private:
    DynArray<Hull> m_aLevels; // Subdivision levels

    int m_bDirty;             // 1 - Surface needs rethinking; 0 - clean

    void SubdivideVertex(const int iPrevious,
                         const int iCurrent,
                         const int iVertex);
    void SubdivideVertexNormalSmooth(const int iCurrent,
                                     const int iVertex,
                                     Vector3D& Normal);
    void SubdivideVertexNormalSharp(const int iCurrent,
                                    const int iVertex,
                                    const int iStart,
                                    const int iSpan,
                                    Vector3D& Normal);
    void SubdivideVertexNormal(const int iCurrent,
                               const int iVertex);

    void SubdivideEdgeVector(const int iPrevious,
                             const int iEdge,
                             Vector3D& V);
    void SubdivideEdgeUVW(const int iCurrent,
                          const int iHead,
                          const int iTail,
                          const int iMidpoint,
                          const int iLeft,
                          const int iRight);
    void SubdivideEdgeColour(const int iCurrent,
                             const int iHead,
                             const int iTail,
                             const int iMidpoint,
                             const int iLeft,
                             const int iRight);
    int SubdivideEdge(const int iPrevious,
                      const int iCurrent,
                      const int iEdge);
    void SubdivideEdgeVisibility(const int iPrevious,
                                 const int iCurrent,
                                 const int iEdge);

    void SubdivideFacet(const int iCurrent,
                        const int iFacet);

    void Subdivide();

    void RethinkVertex(const int iPrevious,
                       const int iCurrent,
                       const int iVertex);
    void RethinkVertexNormal(const int iCurrent,
                             const int iVertex);
};

#endif // SUBDIVISIONSURFACEXLOOP_H
