// -*- Mode: c++ -*-
// $Id: cvec5t.h,v 1.1.1.1 2003/12/24 00:53:24 mikeseven Exp $
// $Source: /big/mpegcvs/MPEG-4/Systems/IM1/OpenGl/AFX/sublib/template/cvec5t.h,v $

/* Subdivide V2.0
Copyright (C) 2000 Henning Biermann, Denis Zorin, NYU

  This file is part of Subdivide.
  
    Subdivide is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2, or (at your option) any
    later version.
    
      Subdivide is distributed in the hope that it will be useful, but WITHOUT
      ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
      FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
      for more details.
      
        You should have received a copy of the GNU General Public License
        along with Subdivide; see the file COPYING.  If not, write to the Free
        Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
02111-1307, USA.  */

#ifndef	__CVEC5T_H__
#define	__CVEC5T_H__

#include <assert.h>
#include <math.h>
#include "compat.h"

template <class F> class CVec5T;

// another VC 6.0 problem: would not accept this definition 
// after the class

template <class F>
::istream& operator>> ( ::istream& is, CVec5T<F>& v) {
    is >> v.x();
    is >> v.y();
    is >> v.z();
    is >> v.u();
    is >> v.v();
    return is;
}

template <class F>
::ostream& operator<<( ::ostream& os, const CVec5T<F>& v) {
    return os << v(0) << " " << v(1) << " " << v(2) << " " << v(3) << " " << v(4);
}


//: template for vectors of length 3
template <class F>
class CVec5T {
public:
    enum{ X = 0, Y = 1, Z = 2, U = 3, V = 4 };
    CVec5T( void ){ _v[X] = 0; _v[Y] = 0; _v[Z] = 0; _v[U] = 0; _v[V] = 0; }
    CVec5T( const CVec5T& c )
    { 
        _v[X] = c._v[X];
        _v[Y] = c._v[Y];
        _v[Z] = c._v[Z]; 
        _v[U] = c._v[U]; 
        _v[V] = c._v[V]; 
    }
    CVec5T( const F& a, const F& b, const F& c)
    { _v[X] = a, _v[Y] = b; _v[Z] = c; _v[U] = F(0); _v[V] = F(0); }
    CVec5T( const F& a, const F& b, const F& c, const F& d, const F& e)
    { _v[X] = a, _v[Y] = b; _v[Z] = c; _v[U] = d; _v[V] = e; }

    //: the argument is required to be zero; assumes that 
    // conversion of 0 to the type F works
    CVec5T( int j) { 
        assert(j == 0); 
        _v[X] = F(0); 
        _v[Y] = F(0); 
        _v[Z] = F(0); 
        _v[U] = F(0);
        _v[V] = F(0);
    }
    
    CVec5T( const F* a )
    { 
        _v[X] = a[X]; 
        _v[Y] = a[Y]; 
        _v[Z] = a[Z]; 
        _v[U] = a[U];
        _v[V] = a[V];
    }
    operator F*( void )
    { return &_v[X]; }
    operator const F*( void ) const
    { return &_v[X]; }
    
    bool read(::istream& is) { return is >> *this; }
    CVec5T& operator=( const CVec5T& c ) 
    { 
        _v[X] = c._v[X]; 
        _v[Y] = c._v[Y];
        _v[Z] = c._v[Z]; 
        _v[U] = c._v[U]; 
        _v[V] = c._v[V]; 
        return *this; 
    }
    CVec5T operator+( const CVec5T& c ) const 
    { return CVec5T( _v[X] + c._v[X], _v[Y] + c._v[Y], _v[Z] + c._v[Z], _v[U] + c._v[U], _v[V] + c._v[V] ); }
    CVec5T operator-( const CVec5T& c ) const
    { return CVec5T( _v[X] - c._v[X], _v[Y] - c._v[Y], _v[Z] - c._v[Z], _v[U] - c._v[U], _v[V] - c._v[V]); }
    CVec5T operator*( const F& s ) const
    { return CVec5T( s * _v[X], s * _v[Y], s * _v[Z], s * _v[U], s * _v[V] ); }
    CVec5T operator*( const CVec5T& c ) const
    { return CVec5T(_v[X] * c._v[X], _v[Y] * c._v[Y], _v[Z] * c._v[Z], _v[U] * c._v[U], _v[V] * c._v[V]); }
    friend CVec5T operator*( const F& s, const CVec5T& c )
    { return CVec5T( s * c._v[X], s * c._v[Y], s * c._v[Z], s * c._v[U], s * c._v[V] ); }
    CVec5T operator/( const F& s ) const
    { return CVec5T( _v[X] / s, _v[Y] / s, _v[Z] / s, _v[U] / s, _v[V] / s ); }  
    CVec5T& operator+=( const CVec5T& c )
    { _v[X] += c._v[X], _v[Y] += c._v[Y]; _v[Z] += c._v[Z]; _v[U] += c._v[U]; _v[V] += c._v[V]; return *this; }
    CVec5T& operator-=( const CVec5T& c )
    { _v[X] -= c._v[X], _v[Y] -= c._v[Y]; _v[Z] -= c._v[Z]; _v[U] -= c._v[U]; _v[V] -= c._v[V]; return *this; }
    CVec5T& operator*=( const F& s )
    { _v[X] *= s, _v[Y] *= s, _v[Z] *= s, _v[U] *= s, _v[V] *= s; return *this; }
    CVec5T& operator/=( const F& s )
    { _v[X] /= s, _v[Y] /= s, _v[Z] /= s, _v[U] /= s, , _v[V] /= s; return *this; }
    CVec5T operator-( void ) const
    { return CVec5T( -_v[X], -_v[Y], -_v[Z], -_v[U], -_v[V]); }
    
    void plus_aX(F a, const CVec5T& u) 
    { _v[X] += a*u[X]; _v[Y] += a*u[X]; _v[Z] += a*u[Z]; _v[U] += a*u[U]; _v[V] += a*u[V]; }
    
    F& x( void ) { return _v[X]; }
    const F& x( void ) const { return _v[X]; }
    F& y( void ) { return _v[Y]; }
    const F& y( void ) const { return _v[Y]; }
    F& z( void ) { return _v[Z]; }
    const F& z( void ) const { return _v[Z]; }
    F& u( void ) { return _v[U]; }
    const F& u( void ) const { return _v[U]; }
    F& v( void ) { return _v[V]; }
    const F& v( void ) const { return _v[V]; }
    F& operator() (const int i)      { return _v[i]; }
    const F& operator() (const int i) const { return _v[i]; }

    //: returns v + t(v1 - v), where v is *this 
    CVec5T lerp( const CVec5T& v1, const F& t ) const
    { return CVec5T( 
    _v[X] + t * (v1._v[X] - _v[X]),
    _v[Y] + t * (v1._v[Y] - _v[Y]), 
    _v[Z] + t * (v1._v[Z] - _v[Z]),
    u[U] + t * (v1._v[U] - _v[U]),
    u[U] + t * (v1._v[V] - _v[V]));     
    }

    //: returns the componentwise minimum
    CVec5T min( const CVec5T& o ) const
    { F a = ::min( _v[X], o._v[X] );
    F b = ::min( _v[Y], o._v[Y] );
    F c = ::min( _v[Z], o._v[Z] );
    return CVec5T( a, b,c); }

    //: returns the componentwise maximum
    CVec5T max( const CVec5T& o ) const
    { F a = ::max( _v[X], o._v[X] );
    F b = ::max( _v[Y], o._v[Y] );
    F c = ::max( _v[Z], o._v[Z] );
    return CVec5T( a, b,c); }
    
    F dot( const CVec5T& c ) const
    { return _v[X] * c._v[X] + _v[Y] * c._v[Y]+ _v[Z] * c._v[Z]; }

    F dot( void ) const
    { return _v[X] * _v[X] + _v[Y] * _v[Y] + _v[Z] * _v[Z]; }

    //: l1 norm (sum of abs values of the components)
    F l1( void ) const
    { F a = fabs( _v[X] ); a += fabs( _v[Y] );a += fabs( _v[Z] ); return a; }

    //: l infinity norm (max of abs values of the components)
    F linfty( void ) const
    { F a = fabs( _v[X] ); a = ::max( a, F(abs( _v[Y] )) );
    a = ::max( a, F(abs( _v[Z] )) );
    return  a; }

    //: l infinity norm (sqrt of the sum of squares )
    F l2( void ) const { return sqrt( dot() ); }

    //: for nonzero vectors, returns the normalized vector
    CVec5T dir( void ) const {
        F a = l2();
        if( ( a  == 0) ) return *this;
        else return *this / a;
    }
    
    int normalize( void )
    { F mag = l2(); return (mag == 0) ? 0 : ( *this *= 1 / mag, 1 ); }
    //:  length |*this -c|
    F dist( const CVec5T& c ) const { return ( *this - c ).l2(); }
    
    CVec5T<F> cross( const CVec5T<F>& b ) const { 
        return 
            CVec5T<F>( y() * b.z() - z() * b.y(),
            z() * b.x() - x() * b.z(),
            x() * b.y() - y() * b.x() );
        
    }
    
    CVec5T<F> abs( ) {
        return CVec5T<F>(fabs( x()), fabs( y()), fabs( z()));
    }
    
    //: conversion of different vectors 
    // note that it is a template    memeber function (parameterzed by G) of
    // the template  class parameterized by F; not all compilers can handle this
    // so it is removed for now (gcc 2.8.1 does not understand it)
    //  template <class G> CVec5T( const CVec5T<G>& u ) { _v[0] = u.x(); _v[1] = u.y();
    //  _v[2] = u.z();  }
    
 protected:
 private:
     F _v[5];
};

typedef CVec5T<float> cvec5f;

#endif	/* __CVEC5T_H__ */








