#include "OpenGl/OpenGl.h"
#ifdef ADV_AUDIO

/***********************************************************************
This software module was originally developed by Riitta Vaananen
(Helsinki University of Technology) in the course of development of
the MPEG-4 Systems (ISO/IEC 14496-1) standard.  This software module
is an implementation of a part of one or more MPEG-4 Systems (ISO/IEC
14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1)
standard. ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1)
free license to this software module or modifications thereof for use
in hardware or software products claiming conformance to the MPEG-4
Systems (ISO/IEC 14496-1). Those intending to use this software module
in hardware or software products are advised that its use may infringe
existing patents.  The original developer of this software module and
his/her company, the subsequent editors and their companies, and
ISO/IEC have no liability for use of this software module or
modifications thereof in an implementation.  Copyright is not released
for non MPEG-4 Systems (ISO/IEC 14496- 1) conforming products.
Helsinki University of Technology retains full right to use the code
for their own purpose, assign or donate the code to a third party and
to inhibit third parties from using the code for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. This copyright notice must be
included in all copies or derivative works. Copyright  1999.
************************************************************************/

#include "AcousticMaterialProxy.h"

// Render object description

/* virtual */ void
AcousticMaterialProxy::RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD /* dwTime */)
{
  RenderDescriptionBox(pRenderer, "AcousticMaterial");
}

// enable material


BOOL
AcousticMaterialProxy::Enable(
    OpenGLVisualRenderer * /* pRenderer */, 
    DWORD /* dwTime */)
{

  AcousticMaterial* pAcousticMaterial = (AcousticMaterial*) GetMediaObject();

  float ambientIntensity = pAcousticMaterial->ambientIntensity;
  float shininess = pAcousticMaterial->shininess;
  float transparency = pAcousticMaterial->transparency;
  const Color3f& diffuseColor = pAcousticMaterial->diffuseColor;
  const Color3f& specularColor = pAcousticMaterial->specularColor;
  const Color3f& emissiveColor = pAcousticMaterial->emissiveColor;

  GLfloat ambient[4] = {diffuseColor.r * ambientIntensity,
			diffuseColor.g * ambientIntensity,
			diffuseColor.b * ambientIntensity,
			1.0f};

  GLfloat specular[4] = {specularColor.r,
			 specularColor.g,
			 specularColor.b,
			 1.0f};

  GLfloat emissive[4] = {emissiveColor.r,
			 emissiveColor.g,
			 emissiveColor.b,
			 1.0f};

  ::glPushAttrib(GL_CURRENT_BIT);

  ::glEnable(GL_LIGHTING);
  ::glEnable(GL_COLOR_MATERIAL);

  ::glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, ambient);

  // SOLARIS PORTING
  //::glColor3fv(diffuseColor);
  glColor3fv(diffuseColor);
  // SOLARIS PORTING - END

  ::glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, specular);
  ::glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, emissive);
  ::glMaterialf(GL_FRONT_AND_BACK, GL_SHININESS, shininess * 128.0f);

  // emissive color is used for drawing lines

  //::glColor3fv(emissiveColor);

  // set screendoor transparency

  if (transparency > 0.0f)
    if (transparency < 1.0f) {
      ::glEnable(GL_POLYGON_STIPPLE);
      ::glPolygonStipple((const GLubyte*) stippleMask[16 - int(transparency * 16.0f)]);
    } else
      ::glColorMask(GL_FALSE, GL_FALSE, GL_FALSE, GL_FALSE);

#ifdef _DEBUG
  {
    GLenum error;

    while ((error = ::glGetError()) != GL_NO_ERROR) {
      TRACE ("diffuseColor: %f, %f, %f\n",
	       diffuseColor.r, diffuseColor.g, diffuseColor.b);
      TRACE ("specularColor: %f, %f, %f\n",
	       specularColor.r, specularColor.g, specularColor.b);
      TRACE ("emissiveColor: %f, %f, %f\n",
	       emissiveColor.r, emissiveColor.g, emissiveColor.b);
      TRACE ("shininess: %f, ambientIntensity: %f, transparency: %f\n",
	       shininess, ambientIntensity, transparency);
      TRACE ("AcousticMaterialProxy::Enable() [%08x]: GL error %d\n", this, error);
    }
  }
#endif

  return TRUE; 
//	 BOOL temp = MaterialProxy::Enable(pRenderer, dwTime);
//	 return temp;
}

#endif // AcousticMaterialProxy
