#include "OpenGl/OpenGl.h"
#ifdef ADV_AUDIO
/***********************************************************************
This software module was originally developed by Riitta Vaananen
(Helsinki University of Technology) in the course of development of
the MPEG-4 Systems (ISO/IEC 14496-1) standard.  This software module
is an implementation of a part of one or more MPEG-4 Systems (ISO/IEC
14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1)
standard. ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1)
free license to this software module or modifications thereof for use
in hardware or software products claiming conformance to the MPEG-4
Systems (ISO/IEC 14496-1). Those intending to use this software module
in hardware or software products are advised that its use may infringe
existing patents.  The original developer of this software module and
his/her company, the subsequent editors and their companies, and
ISO/IEC have no liability for use of this software module or
modifications thereof in an implementation.  Copyright is not released
for non MPEG-4 Systems (ISO/IEC 14496- 1) conforming products.
Helsinki University of Technology retains full right to use the code
for their own purpose, assign or donate the code to a third party and
to inhibit third parties from using the code for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. This copyright notice must be
included in all copies or derivative works. Copyright  1999.
************************************************************************/

#include "Auralize.h"
#include "ProcessSound.h"
#include "../AcousticSceneProxy.h"


//================================================================
//
// class EQFilters: Used for setting up filters in gain-requency  -pair
// form of expressing the directivity of sources  or reflectivity and 
// sound transmission in AcousticMaterial 
// 
//================================================================

EQFilters::EQFilters(float* gain, float* frequencies, int sf, int nof_freq):
nof_filters(0)
{
	int i=0;
	

	int nf = nof_freq;

	if(frequencies[0] > 0)
		nf++;

	if(frequencies[nof_freq-1] < sf/2)
		nf++;


	float* norm_freq = NULL;
	float* gains = NULL;

	norm_freq = new float[nf];
	gains = new float[nf];

	for(i=0;i<nf;i++){
		norm_freq[i] = 0.0;
		gains[i] = 0.0;
	}

	i=0;
	if(frequencies[0] > 0){
		norm_freq[0] = 0.0;
		gains[0] = gain[0];
		i++;
	}

	int j=0;

	float temp = PI/(sf/2);

	// the normalized frequencies (betweem 0 and PI)
	for(j=0; j < nof_freq; j++){
		norm_freq[i] = frequencies[j]*temp;
		gains[i] = gain[j];
		i++;
	}

	if(frequencies[nof_freq-1] < sf/2){
		norm_freq[nf-1] = (float)PI;
		gains[nf-1] = gain[nof_freq-1];
	}

	ComputeCoefficients(norm_freq,gains,nf);


	delete [] norm_freq;
	delete [] gains;
	norm_freq = NULL;
	gains = NULL;
}

EQFilters::~EQFilters()
{
	filters.Clear();
}


// returns a scalar gain; if different from 0, 
void EQFilters::ComputeCoefficients(float* W, float* g, int nf)
{
	// reference gain = 1.0 (EQ filters will be cascaded)
	float G0 = 1.0;

	// cutoff gain for shelving filters (that may be used for
	// the low or the high frequencies)
	float Gc = 0.0;

	// Bandwidth of a shelving filter
	float Wc = 0.0;

	// Cutoff gain for eq filters
	float Gb = 0.0;

	// Bandwidth for eq filters
	float deltaW = 0.0;

	// beta used in the calculation of filter coefficients
	float beta = 0.0;

	// these two floats are used if only 2 frequency points
	// are defined (at 0 and fs/2 frequency)
	float g1 = 0.0; 
	float g2 = 0.0;

	// arrays for storing the a and b coefficients
	float* a = NULL;
	float* b = NULL;
	a = new float[3];
	b = new float[3];
	int i=0;
	for(i=0;i<3;i++){
		a[i] = 0.0;
		b[i] = 0.0;
	}

	int temp = 0;
	MF<int> freq_index;

	// check if the gains are the same for all frequencies
	
	for(i = 0; i < nf-1; i++){
		if(g[i] != g[i+1])
			temp++;
	}

	if(temp == 0){
		if(g[0] != G0){
			b[0] = g[0];
			filters += new IIRFilt(b, a, 0);
		}
		return;
	}

	if(nf > 2) { // if the filtering is not a pure gain (no frequency dependency), 
				 // and more than 2 gains are defined

		// shelving filter for low frequencies
		if(g[0] != G0) {

			// cutoff frequency
			if(g[0] == g[1])
				Wc = (W[1] + W[2])/2.0;
			else
				Wc = W[1]/2.0;

			// boost shelving filter
			if(g[0] > G0){
				if(pow(g[0],2) > 2*(pow(G0,2))){
					Gc = sqrt(2 * pow(G0,2));
				}
				else // 
					Gc = sqrt((pow(G0,2) + pow(g[0],2))/2.0);
			}

			// cut shelving filter
			else { // if (g[0] < G0)
				if(pow(g[0],2) < (pow(G0,2)/2.0)){
					Gc = sqrt(pow(G0,2)/2.0);
				}
				else
					Gc = sqrt((pow(G0,2) + pow(g[0],2))/2.0);
			}

			beta = sqrt((pow(Gc,2) - pow(G0,2))/(pow(g[0],2) - pow(Gc,2))) * tan(Wc/2.0);

			// first order filter coefficients:
			b[0] = (G0 + g[0] * beta)/(1 + beta);
			b[1] = -(G0 - g[0] * beta)/(1 + beta);
			a[0] = 1.0;
			a[1] = -(1 - beta)/(1 + beta);

			filters += new IIRFilt(b,a,1);
		}
		else { // no filter for this
			// do nothing
		}


		// shelving filter for high frequencies
		if(g[nf-1] != G0){

			// cutoff frequency
			if(g[nf-1] == g[nf-2]) 
				Wc = (W[nf-2] + W[nf-3])/2.0;
			else
				Wc = ((float)PI + W[nf-2])/2.0;

			if(g[nf-1] > G0){
				// boost filter
				if(g[nf-1] > G0)
					Gc = sqrt(2.0 * pow(G0,2));
				else // if g[nf-1] < G0
					Gc = sqrt((pow(G0,2) + pow(g[nf-1],2))/2.0);
			}

			else { // g[nf-1 > G0]
				// cut filter
				if(pow(g[nf-1],2) < (pow(G0,2))/2.0)
					Gc = sqrt((pow(G0,2))/2.0);
				else
					Gc = sqrt((pow(G0,2) + pow(g[nf-1],2))/2.0);
			}

			beta = sqrt((pow(Gc,2) - pow(G0,2))/(pow(g[nf-1],2) - pow(Gc,2))) * (1.0/(tan(Wc/2.0)));
			b[0] = (G0 + g[nf-1] * beta)/(1 + beta); 
			b[1] = (G0 - g[nf-1] * beta)/(1 + beta);
			a[0] = 1.0;
			a[1] = (1 - beta)/(1 + beta);

			filters += new IIRFilt(b,a,1);
		}

		
		// check which frequency points still need filters

		if(g[0] != g[1])
			freq_index += 1;
		
		for(i = 2; i < nf-2; i++)
			freq_index += i;

		if(g[nf-2] != g[nf-1])
			freq_index += nf-2;

		// Finally the filters for the remaining frequency points

		int m = 0;
		for(i = 0; i < freq_index.GetLength(); i++){
			m = freq_index[i];
			deltaW = (W[m+1] - W[m-1])/2.0;
			if(g[m] != G0){
				if(g[m] > G0){
					// boost filter
					if(pow(g[m],2) > 2.0 * pow(G0,2.0))
						Gb = sqrt(2.0 * pow(G0,2.0));
					else
						Gb = sqrt((pow(G0,2.0) + pow(g[m],2.0))/2.0);
				}
				else{
					if(pow(g[m],2.0) < (pow(G0,2.0))/2.0)
						Gb = sqrt((pow(G0,2))/2.0);
					else
						Gb = sqrt((pow(G0,2) + pow(g[m],2))/2.0);
				}
				
				beta = sqrt((pow(Gb,2) - pow(G0,2))/(pow(g[m],2) - pow(Gb,2))) * 
					tan(deltaW/2.0);

				b[0] = (G0 + g[m] * beta)/(1 + beta);
				b[1] = -2.0 * (G0 * cos(W[m]))/(1 + beta);
				b[2] = (G0 - g[m] * beta)/(1 + beta);
				
				a[0] = 1.0;
				a[1] = -2.0 * (cos(W[m]))/(1 + beta);
				a[2] = (1 - beta)/(1 + beta);

				filters += new IIRFilt(b,a,2);
			}
		}
		
	}



	
	
	else { // if there are two gains defined
		   // -> define a first order lowpass or highpass filter
	
		Wc = 0.5 * PI;

		if(g[0] > g[1]) {
			// lowpass filter
			g1 = 1.0;
			g2 = g[1]/g[0];

			if(pow(g2,2) < pow(G0,2)/2.0)
				Gc = sqrt((pow(G0,2))/2.0);
			else
				Gc = (pow(G0,2) + pow(g2,2))/2;

			beta = sqrt((pow(Gc,2) - pow(G0,2))/(pow(g2,2) - pow(Gc,2)))/tan(Wc/2.0); 

			b[0] = g[0] * ((G0 + g2 * beta)/(1 + beta)); 
			b[1] = g[0] * ((G0 - g2 * beta)/(1 + beta)); 
			a[0] = 1.0;
			a[1] = (1 - beta)/(1 + beta); 		
		}
		
		else { // g[1] > g[0]
			// highpass filter
			g2 = 1.0;
			g1 = g[0]/g[1];

			if(pow(g1,2) < pow(G0,2)/2.0)
				Gc = sqrt((pow(G0,2))/2.0);
			else
				Gc = (pow(G0,2) + pow(g1,2))/2;

			beta = sqrt((pow(Gc,2) - pow(G0,2))/(pow(g1,2) - pow(Gc,2))) * 
				tan(Wc/2.0);

			b[0] = g2 * g[0] * ((G0 + g1 * beta)/(1 + beta)); 
			b[1] = -g2 * g[0] * ((G0 - g1 * beta)/(1 + beta)); 
			a[0] = 1.0;
			a[1] = - (1 - beta)/(1 + beta); 	
		}

		filters += new IIRFilt(b,a,1);

	}
	delete [] a;
	delete [] b;
	a = NULL;
	b = NULL;
	
	nof_filters = filters.GetLength();
}

void EQFilters::EmptyFilters()
{
	int i=0;
	for(i = 0; i < nof_filters; i++){
		filters[i]->EmptyFilter();
	}
}

float EQFilters::operator ()(float x)
{
	int i=0;
	float temp = x;
	
	for(i = 0; i < nof_filters; i++){
		temp = (*(filters[i]))(temp);
	}

	return temp;
}




//================================================================
//
// class SoundDirectivity
//
//================================================================

SoundDirectivity::SoundDirectivity(int sf,
								   float viewp_from_presenter[4], 
								   Vec3f sound_loc_from_presenter, 
								   Vec3f sound_dir_from_presenter,
								   /*MF<float>& dir_param,*/
								   DirectiveSound* DS,
								   int num_param, 
								   int nof_angles):
no_angles(nof_angles),
DL_filter(NULL),
coeffs_b(NULL),
coeffs_a(NULL),
angles(NULL),
curr_coeffs_b(NULL),
curr_coeffs_a(NULL),
dir_filt(NULL),
filt_1(NULL),
filt_2(NULL),
filter_form(TRUE),
pDL(0),
nof_coeff(0),
angle(0),
offset_index(0),
interp_gain(0),
no_param(num_param)
{

    int i=0;
    int j=0;

	int len_freq = (DS->frequency).GetLength();
	if(len_freq > 0)
		filter_form = FALSE;
	 
	nof_coeff = (num_param-1)/2 + 1;
	
	angles = new float[no_angles];
	for(i=0;i<no_angles;i++)
		angles[i] = DS->angles[i];
	angle = GetAngle(viewp_from_presenter, 
		sound_loc_from_presenter, 
		sound_dir_from_presenter);

	SetInterpAndOffset();


	if(num_param > 1){

		if ((bool)filter_form == TRUE){
			int no_b_and_a =  no_angles*nof_coeff;	
			coeffs_b = new float[no_b_and_a];
			coeffs_a = new float[no_b_and_a];

			for(i=0;i<no_b_and_a;i++){
				coeffs_b[i] = 0.0;
				coeffs_a[i] = 0.0;
			}

		//read the directivity filters
			for(i=0;i<no_angles;i++){
				for(j=0; j < nof_coeff; j++){
					coeffs_b[j + (i*nof_coeff)] = DS->directivity[i*(num_param)+j];
				}

				coeffs_a[i*nof_coeff] = 1.0;
				for(j = 1; j < nof_coeff; j++){
					coeffs_a[j + (i*nof_coeff)] = DS->directivity[i*(num_param)+nof_coeff+j-1];
				}

				filt_arr += new IIRFilt(&(coeffs_b[i*nof_coeff]), &(coeffs_a[i*nof_coeff]), nof_coeff-1);

			}
		
		} // end of filter_form == TRUE

		else { // filter_form = FALSE

			// for storing gains for one angle at a time
			float* gain = new float[num_param];
			float* frequencies = new float[num_param];
			for(i=0;i<num_param;i++){
				gain[i] = 0.0;
				frequencies[i] = DS->frequency[i];
			}

			i = 0;
			j = 0;
			for(i = 0; i < no_angles; i++){
				for(j=0;j<num_param;j++){
					gain[j] = DS->directivity[i*(num_param)+j];
				}
				
				eq_array += new EQFilters(gain, frequencies, sf, num_param);
			}

			delete [] gain;
			delete [] frequencies;
			gain = NULL;
			frequencies = NULL;

		} // end of filter_form = FALSE

	} // end of if (num_param > 1)

	if(num_param == 1)
	{
		// scalar directivity (no frequency dependence)
		coeffs_b = new float[no_angles];

		curr_coeffs_b = new float[num_param];
		curr_coeffs_b[0] = 0.0;
		for(i=0;i<no_angles;i++){
			coeffs_b[i] = 0.0;
		}

		//read the directivity gain
		for(i=0;i<no_angles;i++){
			coeffs_b[i] = DS->directivity[i];
		}

		InterpolateGain();
	}

	
}

SoundDirectivity::~SoundDirectivity(void)
{
	COMPLAIN("SoundDirectivity destruction");
	if(DL_filter != NULL){
		delete [] DL_filter;
		DL_filter = NULL;
	}

	if(coeffs_b){
		delete [] coeffs_b;
		coeffs_b = NULL;
	}
	if(coeffs_a){
		delete [] coeffs_a;
		coeffs_a = NULL;
	}
	if(curr_coeffs_b){
		delete [] curr_coeffs_b;
		curr_coeffs_b = NULL;
	}
	if(curr_coeffs_a){
		delete [] curr_coeffs_a;
		curr_coeffs_a = NULL;
	}
	if(angles){
		delete [] angles;	
		angles = NULL;
	}
	if(dir_filt){
		delete dir_filt;
		dir_filt = NULL;
	}

	if(filt_1){
		delete filt_1;
		filt_1 = NULL;
	}
	if(filt_2){
		delete filt_2;
		filt_2 = NULL;
	}

	filt_arr.Clear();
	eq_array.Clear();
}



float SoundDirectivity::GetAngle(float viewp[4], Vec3f sound_loc, Vec3f sound_dir)
{
	float x = viewp[0]-sound_loc.x;
//	float y = viewp[1]-sound_loc.y;
	float z = viewp[2]-sound_loc.z;

	float abs_sounddir_azi =  (float) sqrt(pow(sound_dir.x,2)+pow(sound_dir.z,2));
	float abs_viewp_azi = (float) sqrt(pow(x,2)+pow(z,2));
	if(fabs(abs_sounddir_azi) < 0.0001)
		abs_sounddir_azi = 0.0001;
	if(fabs(abs_viewp_azi) < 0.0001)
		abs_viewp_azi = 0.0001;

	float cos_theta = (float) (sound_dir.x*x+sound_dir.z*z)/(abs_viewp_azi*abs_sounddir_azi);

	float z_new = (float) cos_theta*abs_viewp_azi;
	float x_new = (float) sqrt(pow(abs_viewp_azi,2)-pow(z_new,2));

	if(fabs(x_new) < 0.0001)
		x_new = 0.0001;
	if(fabs(z_new) < 0.0001)
		z_new = 0.0001;

	float azi = 0.0;
	if(x_new*z_new < 0){
		if(x_new < 0)
			azi = (float) (-atanf(x_new/z_new));
		else
			azi = (float) (-atanf(x_new/z_new) - PI);
	}
	if(x_new*z_new >= 0){
		if(x_new >= 0)
			azi = (float) (-atanf(x_new/z_new));
		else
			azi = (float) (-atanf(x_new/z_new) + PI);	
	}

	azi = ConvertAngle(azi);
	if(azi < 0)
	    azi = -azi;

	return azi;
}

void SoundDirectivity::SetInterpAndOffset()
{
	if(angle <= angles[0]){
		offset_index = 0;
		interp_gain = 0.0;
	}
	else if(angle >= angles[no_angles-1]){
		offset_index = no_angles-2;
		interp_gain = 1.0;
	}
	else{
		int i=0;
		float a = 0.0;
		while(a <= angle){		//what if a = angle? check
			a = angles[i];
			i++;
		}
		offset_index = i-2;
		interp_gain = (angle-angles[offset_index])/(angles[offset_index+1]-angles[offset_index]);
	}
}

void SoundDirectivity::InterpolateCoeffs()
{
	if(nof_coeff == 1){
		curr_coeffs_b[0] = (1-interp_gain)*coeffs_b[offset_index] + interp_gain*coeffs_b[offset_index+1];
		return;
	}

	else {
	int i=0;
	   for(i=0;i<nof_coeff;i++){
			curr_coeffs_a[i] = (1-interp_gain)*coeffs_a[offset_index*nof_coeff+i] + interp_gain*coeffs_a[(offset_index+1)*nof_coeff+i];
			curr_coeffs_b[i] = (1-interp_gain)*coeffs_b[offset_index*nof_coeff+i] + interp_gain*coeffs_b[(offset_index+1)*nof_coeff+i];
		}
	}
}

void SoundDirectivity::InterpolateGain()
{
		curr_coeffs_b[0] = (1-interp_gain)*coeffs_b[offset_index] + interp_gain*coeffs_b[offset_index+1];

}


void SoundDirectivity::UpdateCoeffs(float viewp_from_presenter[4], 
									Vec3f sound_loc_from_presenter, 
									Vec3f sound_dir_from_presenter)
{
	angle = GetAngle(viewp_from_presenter, 
		sound_loc_from_presenter, 
		sound_dir_from_presenter);

	SetInterpAndOffset();
	
	// riitta 03062000
	// InterpolateCoeffs();
	if(no_param /*nof_coeff*/ == 1)
		InterpolateGain();

	//dir_filt->UpdateCoeffs(curr_coeffs_b, curr_coeffs_b);
}


inline //riitta 061799
float SoundDirectivity::operator()(float input)
{
  //  int i=0;
	float out = 0.0;


	// change this to IIRFilts (interpolate between the gains of the filters)
    if(no_param/*nof_coeff*/ > 1){


		if((bool)filter_form == TRUE){
			out = (1-interp_gain) * (*(filt_arr[offset_index]))(input)
				+ interp_gain * (*(filt_arr[offset_index+1]))(input);
		}
		else { // filter_form == FALSE
			out = //(*eq_array[2])(input);
				(1-interp_gain) * (*(eq_array[offset_index]))(input)
				+ interp_gain * (*(eq_array[offset_index+1]))(input);
		}
	}

	// keep this the same (for frequency independent directivity modeling)
	else { // if(no_param/*nof_coeff*/ == 1)
		out = curr_coeffs_b[0]*input;

	}
    return out;
}

//================================================================
//
// class IIRfilt
//
//================================================================


IIRFilt::IIRFilt(float* c_b, float* c_a, int ord):
order(ord),
DL_IIR(NULL),
coeffs_b(NULL),
coeffs_a(NULL)
{
	int i=0;
	if(order == 0){
		coeffs_b = new float[1];
		*coeffs_b = *c_b;
	}

	else {
		no_coeffs = order+1;

		coeffs_b = new float[no_coeffs];
		coeffs_a = new float[no_coeffs];
		DL_IIR = new float[no_coeffs];


		for(i=0;i<no_coeffs;i++)
		{
			coeffs_b[i] = c_b[i];
			coeffs_a[i] = c_a[i];
			DL_IIR[i] = 0.0;
		}

		pDL = 0;
	}    
}

IIRFilt::~IIRFilt(void)
{
	COMPLAIN("IIRFilt destruct");
	if(DL_IIR){
		COMPLAIN("should delete");
		delete [] DL_IIR;
		DL_IIR = NULL;
	}


	if(coeffs_b){
		delete[] coeffs_b;
		coeffs_b = NULL;
	}
	if(coeffs_a){
		delete[] coeffs_a;
		coeffs_a = NULL;
	}
}

void IIRFilt::UpdateCoeffs(float* c_b, float* c_a)
{
	if(order == 0){
		*coeffs_b = *c_b;
	}
	else {
		int i=0;
		for(i=0;i<no_coeffs;i++){
			coeffs_b[i] = c_b[i];
			coeffs_a[i] = c_a[i];
		}

	}
}

void IIRFilt::EmptyFilter()
{

	int i=0;
	if(order > 0){
		for(i = 0; i < no_coeffs; i++)
			DL_IIR[i] = 0.0;
	}

}

// SOLARIS PORTING - Commented inline
//inline //riitta 061799 
float IIRFilt::operator()(float input)
{
 
    if(order == 0){
		return (coeffs_b[0]*input);
	}

	else {
	    int i=0;
		float out = 0.0;

		out = DL_IIR[0] + coeffs_b[0] * input;
				
		for(i = 0; i < order - 1; i++){
				
			DL_IIR[i] = 
				DL_IIR[i+1] +
				coeffs_b[i+1] * input - coeffs_a[i+1] * out;

		}

		DL_IIR[order-1] = input * coeffs_b[order] - out * coeffs_a[order];

		return out;
 
	}

}


//================================================================
//
// class Auralize
//
//================================================================
Auralize::Auralize(int sf, int update,
		float viewp_from_presenter[4],
		DirectiveSound* DS,
		Vec3f sound_dir_from_presenter,
		AcousticScene* pAScene,
		ZArray<ImageSource*>& reflections,
		SoundSource* source):
speedOfSound(DS->speedOfSound),
count(0),
nof_angles(0),
is_directive(0),
pDL(0),
att_dist(DS->distance),
curr_dist(0.0),
dist_gain(1.0),
dist_gain_old(0.0),
samp_freq(sf),
dist_samp(0.0),
dist_samp_old(0.0),
spatialize(DS->spatialize),
roomEffect(DS->roomEffect),
nof_images(0),
dir(NULL),
airabs(NULL),
pan(NULL),
panitd(NULL),
reverb(NULL),
soundsource(source),
useairabs(DS->useAirabs),
update_period(update),
air_a(NULL),
air_b(NULL),
DL_dist(NULL),
dist_samp_ref(NULL),
dist_samp_ref_old(NULL),
dist_gain_ref(NULL),
dist_gain_ref_old(NULL),
vis_ref(NULL),
reflections_initialized(FALSE),
nof_ref(reflections.GetLength()),
dist_delay_rendering(FALSE),
dist_gain_rendering(FALSE)
{
	out[0] = 0.0;
	out[1] = 0.0;
	int i=0;
	
	int rev_len = (pAScene->reverbTime).GetLength();

	// riitta 03112000:
	if((bool)roomEffect == TRUE || speedOfSound > 0){
		dist_delay_rendering = TRUE;
	
		if(speedOfSound <= 0)
			speedOfSound = 340;

		max_dist_delay = (int) ceil((MAX_DIST_METERS/speedOfSound)*sf);
		DL_dist = new float[max_dist_delay + 1];

		for(i=0;i<(max_dist_delay + 1);i++)
			DL_dist[i] = 0.0;

		GetDistance(viewp_from_presenter, source->location/*sound_loc_from_presenter*/, 
			&dist_samp_old, &dist_samp, &curr_dist);
			dist_samp_old = dist_samp;
	}

	int air_length = 60*3;
	
	for(i=0;i<air_length;i++)
		airabscoeffs[i] = 0.0;

	SetAirabsCoeffs(airabscoeffs);


	if((((rev_len == 1) && ((pAScene->reverbTime)[0] != 0)) || 
		(rev_len > 1)) && 
		((bool)roomEffect == TRUE)){
			reverb = new LateReverb(samp_freq, pAScene);
	}

	if((bool)roomEffect == TRUE){
		ReflectionParameters(reflections, viewp_from_presenter);
		
	}

	if((bool)spatialize == TRUE/*(spatialize == 1) || (spatialize == 2)*/){
		panitd = new PanITD(viewp_from_presenter, source->location, 
			samp_freq, TRUE, TRUE, update_period);
	}


	if(att_dist > 0){
		GetDistAtt(curr_dist, &dist_gain, &dist_gain_old);
		dist_gain_rendering = TRUE;
	}


	if((bool)useairabs == TRUE){
		air_a = new float[2];
		air_b = new float[2];
		for(i=0;i<2;i++){
			air_a[i] = 0.0;
			air_b[i] = 0.0;
		}
		GetAirabsCoeffs(curr_dist, air_b, air_a);
		airabs = new IIRFilt(air_b, air_a, 1);
	}
	

	int num_param = 0;

	int	len = (DS->directivity).GetLength();
	nof_angles = (DS->angles).GetLength();	
	if(len > 0 && nof_angles > 0){
		num_param = (int)(len/nof_angles);

	    dir = new SoundDirectivity(sf, 
			viewp_from_presenter,
			source->location,
			sound_dir_from_presenter,
			/*DS->directivity,*/
			DS,
			num_param,
			nof_angles);
	}
}

Auralize::~Auralize(void)
{
if(DL_dist){
	delete [] DL_dist;
	DL_dist = NULL;
}

if(dir != NULL){
	delete dir;
	dir = NULL;
}

if(airabs != NULL){
	delete airabs;
	airabs = NULL;
}


if(pan != NULL){
	delete pan;
	pan = NULL;
}

if(panitd != NULL){
	delete panitd;
	panitd = NULL;
}


if(reverb != NULL){
	delete reverb;	
	reverb = NULL;
}

ref_panitd.Clear();
ref_airabs.Clear();
ref_filt.Clear();
ref_eq.Clear();

}

void
Auralize::SetAirabsCoeffs(float air_coeffs[NOF_AIRCOEFFS])
{
float a_c[NOF_AIRCOEFFS] = {
//air_coeffs = { 
(float)0.980103, (float)0.270488, (float)0.252535,
(float)0.960603, (float)0.274535, (float)0.238883,
(float)0.941491, (float)0.278322, (float)0.225220,
(float)0.922761, (float)0.281864, (float)0.211553,
(float)0.904403, (float)0.285175, (float)0.197893,
(float)0.886412, (float)0.288268, (float)0.184248,
(float)0.868779, (float)0.291156, (float)0.170629,
(float)0.851498, (float)0.293787, (float)0.156966,
(float)0.834561, (float)0.296313, (float)0.143436,
(float)0.817961, (float)0.298664, (float)0.129954,
(float)0.801692, (float)0.300852, (float)0.116529,
(float)0.785747, (float)0.302885, (float)0.103168,
(float)0.770120, (float)0.304772, (float)0.089878,
(float)0.754805, (float)0.306521, (float)0.076666,
(float)0.739794, (float)0.308139, (float)0.063539,
(float)0.725082, (float)0.309598, (float)0.050451,
(float)0.710664, (float)0.311004, (float)0.037550,
(float)0.696533, (float)0.312294, (float)0.024743,
(float)0.682683, (float)0.313474, (float)0.012035,
(float)0.669108, (float)0.314507, (float)-0.000631,
(float)0.655805, (float)0.315477, (float)-0.013135,
(float)0.642766, (float)0.316357, (float)-0.025520,
(float)0.629986, (float)0.317153, (float)-0.037781,
(float)0.617461, (float)0.317869, (float)-0.049915,
(float)0.605186, (float)0.318507, (float)-0.061919,
(float)0.593155, (float)0.319073, (float)-0.073791,
(float)0.581363, (float)0.319570, (float)-0.085528,
(float)0.569806, (float)0.320000, (float)-0.097128,
(float)0.558479, (float)0.320367, (float)-0.108589,
(float)0.547378, (float)0.320674, (float)-0.119910,
(float)0.536497, (float)0.320925, (float)-0.131087,
(float)0.525833, (float)0.321117, (float)-0.142128,
(float)0.515382, (float)0.321260, (float)-0.153022,
(float)0.505138, (float)0.321354, (float)-0.163766,
(float)0.495098, (float)0.321391, (float)-0.174383,
(float)0.485258, (float)0.321387, (float)-0.184848,
(float)0.475613, (float)0.321337, (float)-0.195169,
(float)0.466161, (float)0.321245, (float)-0.205348,
(float)0.456896, (float)0.321111, (float)-0.215384,
(float)0.447816, (float)0.320937, (float)-0.225278,
(float)0.438916, (float)0.320724, (float)-0.235031,
(float)0.430194, (float)0.320474, (float)-0.244645,
(float)0.421645, (float)0.320189, (float)-0.254119,
(float)0.413266, (float)0.319869, (float)-0.263456,
(float)0.405054, (float)0.319515, (float)-0.272656,
(float)0.397005, (float)0.319167, (float)-0.281649,
(float)0.389116, (float)0.318759, (float)-0.290561,
(float)0.381384, (float)0.318323, (float)-0.299338,
(float)0.373805, (float)0.317859, (float)-0.307981,
(float)0.366378, (float)0.317369, (float)-0.316490,
(float)0.359098, (float)0.316853, (float)-0.324869,
(float)0.351963, (float)0.316312, (float)-0.333117,
(float)0.344970, (float)0.315748, (float)-0.341236,
(float)0.338115, (float)0.315161, (float)-0.349229,
(float)0.331397, (float)0.314552, (float)-0.357095,
(float)0.324813, (float)0.313922, (float)-0.364838,
(float)0.318360, (float)0.313272, (float)-0.372458,
(float)0.312034, (float)0.312602, (float)-0.379957,
(float)0.305835, (float)0.311914, (float)-0.387337,
(float)0.299759, (float)0.311208, (float)-0.394599
};

memcpy(air_coeffs,a_c,NOF_AIRCOEFFS*sizeof(float));

}

void Auralize::GetDistance(float viewp[4], Vec3f sloc, float* d_samp_old, float* d_samp, float* c_dist)
{

*c_dist = (float) (sqrt(pow((viewp[0]-sloc.x),2)+pow((viewp[1]-sloc.y),2)+
		pow((viewp[2]-sloc.z),2)));
	*d_samp_old = *d_samp;
*d_samp = (float) floor((*c_dist/(speedOfSound))*samp_freq);
	if(*d_samp >=max_dist_delay)
		*d_samp = max_dist_delay;

}

void Auralize::GetDistAtt(float c_dist, float* gain, float *gain_old)
{
	
	if(c_dist<att_dist){
		*gain_old = *gain;
		*gain = (float) (pow(10.0,(-(60*c_dist)/(att_dist*20))));
	}
	else 
		gain = 0;
}

void Auralize::GetAirabsCoeffs(float dist, float* b, float* a)
{
	int index = (int) floor(dist);
	if(index == 0){
		 b[0] = 1.0;
		 b[1] = 0.0;
		 a[0] = 1.0;
		 a[1] = 0.0;

  }
	else { 
		if(index < 60){
			b[0] = airabscoeffs[index*3];
			b[1] = airabscoeffs[index*3+1];
			a[0] = 1.0;
			a[1] = airabscoeffs[index*3+2];

		}
		else{
			index = 59;

			b[0] = airabscoeffs[index*3];
			 b[1] = airabscoeffs[index*3+1];
			 a[0] = 1.0;
			 a[1] = airabscoeffs[index*3+2];			 
		}
	}
}


void Auralize::UpdateAll(float viewp_from_presenter[4],
						 Vec3f sound_dir_from_presenter,
						 AcousticScene* pAScene, 
						 ZArray<ImageSource*>& reflections, 
						 SoundSource* source)
{
	soundsource = source;

	if((bool)dist_delay_rendering == TRUE){

		GetDistance(viewp_from_presenter, source->location, /*sound_loc_from_presenter,*/ 
			&dist_samp_old, &dist_samp, &curr_dist);
	}

	if((bool)dist_gain_rendering == TRUE){
		GetDistAtt(curr_dist, &dist_gain, &dist_gain_old);
	}

	if(useairabs){
		GetAirabsCoeffs(curr_dist, air_b, air_a);
		if(airabs)
			airabs->UpdateCoeffs(air_b,air_a);
	}

	if(dir && (nof_angles > 1))
	{
		dir->UpdateCoeffs(viewp_from_presenter, source->location/*sound_loc_from_presenter*/, sound_dir_from_presenter);
	}

	int rev_len = (pAScene->reverbTime).GetLength();
	nof_ref = reflections.GetLength();

	if((reverb != NULL) &&
		(((rev_len == 1) && ((pAScene->reverbTime)[0]/*r_time[0]*/ == 0)) ||
		(!roomEffect) )){
			delete reverb;
			reverb = NULL;
	}

	else if((reverb==NULL) &&
			(((rev_len == 1) && ((pAScene->reverbTime)[0]/*r_time[0]*/ != 0)) || 
			(rev_len > 1)) &&
			(roomEffect)){
		reverb = new LateReverb(samp_freq, pAScene);
	}
	else{
		//reverb->Update()
	}
		
	if(roomEffect){
		ReflectionParameters(reflections, viewp_from_presenter);
	}
		

	if((panitd != NULL) && (!spatialize)){
		delete panitd;
		panitd = NULL;
	}


	else if((panitd == NULL) && (spatialize)){
		panitd = new PanITD(viewp_from_presenter, source->location, 
			samp_freq, TRUE, TRUE, update_period);
	}
	else {


		if(panitd != NULL) {
			panitd->Change(viewp_from_presenter, source->location);
		}
	}


}

void Auralize::ReflectionParameters(ZArray<ImageSource*>& ref, float viewp[4])
{
	int i=0;
	float d_old;
	float d;
	float c_d;
	float* a;
	float* b;
	a = NULL;
	b = NULL;
	float gain = 0.0;
	float gain_old = 0.0;
	if((bool)useairabs == TRUE){
		a = new float[2];
		b = new float[2];
		for(i=0;i<2;i++){
			a[i] = 0.0;
			b[i] = 0.0;
		}
	}

	if(reflections_initialized == FALSE){
		reflections_initialized = TRUE;
		dist_samp_ref = new float[MAX_NOF_REF];
		dist_samp_ref_old = new float[MAX_NOF_REF];
		dist_gain_ref = new float[MAX_NOF_REF];
		dist_gain_ref_old = new float[MAX_NOF_REF];
		vis_ref = new BOOL[MAX_NOF_REF];
		for(i=0;i<MAX_NOF_REF;i++){
			dist_samp_ref[i] = 0.0;
			dist_samp_ref_old[i] = 0.0;
			dist_gain_ref[i] = 0.0;
			dist_gain_ref_old[i] = 0.0;
			vis_ref[i] = FALSE;
		}

		for(i=0;i<nof_ref;i++){

			GetDistance(viewp, ref[i]->location, &d_old, &d, &c_d);
			if((bool)useairabs == TRUE)
				GetAirabsCoeffs(c_d, b, a);
			GetDistAtt(c_d, &gain, &gain_old);
			dist_samp_ref_old[i] = d;
			dist_samp_ref[i] = d;
			ref_panitd += new PanITD(viewp, ref[i]->location, samp_freq, 
				TRUE, TRUE, update_period);
			if((bool)useairabs == TRUE){
				ref_airabs += new IIRFilt(b, a ,1);
			}
			dist_gain_ref[i] = gain;
			dist_gain_ref_old[i] = gain;
			vis_ref[i] = ref[i]->visibility;
			if((bool)((ref[i]->a_surf)->filter_form_ref) == TRUE){
				ref_eq += (EQFilters *)NULL;	// SOLARIS PORTING - Added typecast
				if((ref[i]->a_surf)->nof_refcoeffs == 0){
					ref_filt += (IIRFilt *)NULL;	// SOLARIS PORTING - Added typecast
				}
				else {
					ref_filt += new IIRFilt((ref[i]->a_surf)->ref_b, 
					(ref[i]->a_surf)->ref_a, 
					(ref[i]->a_surf)->nof_refcoeffs - 1);	
				}
			}
			else { //if(a_surf->filter_form_ref == FALSE)
				ref_filt += (IIRFilt *)NULL;	// SOLARIS PORTING - Added typecast
				ref_eq += new EQFilters((ref[i]->a_surf)->ref_param,
					(ref[i]->a_surf)->ref_freq,
					samp_freq,
					(ref[i]->a_surf)->num_ref_freq);
			}
		}
	}

	else{
		for(i=0;i<nof_ref;i++){
			d = dist_samp_ref[i];
			d_old = dist_samp_ref_old[i];

			GetDistance(viewp, 
				ref[i]->location, 
				&d_old, 
				&d, 
				&c_d);

			if((bool)useairabs == TRUE)
				GetAirabsCoeffs(c_d,
				b, a);

			gain = dist_gain_ref[i];
			gain_old = dist_gain_ref_old[i];

			GetDistAtt(c_d, 
				&gain, 
				&gain_old);

			dist_samp_ref_old[i] = 	d_old;
			dist_samp_ref[i] = d;
			ref_panitd[i]->Change(viewp, 
				ref[i]->location);

			if(useairabs){
				if(ref_airabs[i] != NULL)
					ref_airabs[i]->UpdateCoeffs(b, a);
			}

			dist_gain_ref[i] = gain;
			dist_gain_ref_old[i] = gain_old;
			vis_ref[i] = ref[i]->visibility;
		}
	}
}

float* Auralize::operator()(float input)
{
	float tmp = input;
	float* tmp_out;

	out[0] = out[1] = 0.0;
	int i;
	int d_low = 0;

	if((bool)dist_gain_rendering == TRUE){
		tmp = input*dist_gain;
	}

	if((bool)dist_delay_rendering == TRUE){
		
		DL_dist[pDL] = tmp; // input;

		float del = dist_samp_old + ((dist_samp-dist_samp_old)/(update_period))*count;
		d_low = (int) floor(del);
		float interp_gain = del - d_low;
		tmp = /*dist_gain**/(DL_dist[(pDL+d_low)%(max_dist_delay+1)]*(1-interp_gain)
				+ DL_dist[(pDL+d_low+1)%(max_dist_delay+1)]*interp_gain);
	}


	if(airabs)
		tmp = (*airabs)(tmp); 

	if(soundsource->visibility == FALSE){	
		for(i=0;i<soundsource->iirfilts.GetLength();i++){
			if(soundsource->iirfilts[i])
				tmp = (*(soundsource->iirfilts[i]))(tmp);
			else{
				if(soundsource->eqfilts[i])
					tmp = (*(soundsource->eqfilts[i]))(tmp);
			}
		}
	}

	if(reverb){
		tmp_out = (*reverb)(tmp*0.5);
		out[0] = tmp_out[0];
		out[1] = tmp_out[1];
	}

	if(dir)
		tmp = (*dir)(tmp);

	if(pan){
		tmp_out = ((*pan)(tmp));
		out[0] += tmp_out[0];
		out[1] += tmp_out[1];
	}
	else if(panitd){
		tmp_out = ((*panitd)(tmp));
		out[0] += tmp_out[0];
		out[1] += tmp_out[1];
	}
	else {
		out[0] += tmp;
		out[1] += tmp;
	}


	
	if(roomEffect){
		for(i=0;i<nof_ref;i++){
			if(vis_ref[i] == TRUE){
				if(dist_samp_ref[i] == dist_samp_ref_old[i]){ // Not moving
					tmp = (dist_gain_ref_old[i])*
						DL_dist[(pDL + (int)floor(dist_samp_ref[i]))%(max_dist_delay+1)];
				}
				else{	
					float del = dist_samp_ref_old[i] + ((dist_samp_ref[i]-dist_samp_ref_old[i])/(update_period))*count;
					d_low = (int) floor(del);
					float interp_gain = del - d_low;
					tmp = dist_gain_ref[i]*
						(DL_dist[(pDL+d_low)%(max_dist_delay+1)]*(1-interp_gain)
						+ DL_dist[(pDL+d_low+1)%(max_dist_delay+1)]*interp_gain);
				}
				if(useairabs)	
					tmp = (*(ref_airabs[i]))(tmp);
				if(ref_filt[i])
					tmp = (*(ref_filt[i]))(tmp);
				// riitta 03072000
				else {
					if(ref_eq[i])
						tmp = (*(ref_eq[i]))(tmp);
				}

				if((bool)spatialize == TRUE){
					tmp_out = (*(ref_panitd[i]))(tmp);
					out[0] += tmp_out[0];
					out[1] += tmp_out[1];
				}
				else {
					out[0] += tmp;
					out[1] += tmp;
				}
			} //if visibility of a reflection == TRUE
		}//for
	}//if

	if((bool)dist_delay_rendering == TRUE){
		pDL = (pDL+max_dist_delay)%(max_dist_delay+1);
	

		if(count < (update_period-1))
			count++;
		else{
			count = 0;
		}
	}

	return (float*)out;

}

#endif
