// -*- c++ -*-
/***********************************************************************
This software module was originally developed by Riitta Vaananen
(Helsinki University of Technology) in the course of development of
the MPEG-4 Systems (ISO/IEC 14496-1) standard.  This software module
is an implementation of a part of one or more MPEG-4 Systems (ISO/IEC
14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1)
standard. ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1)
free license to this software module or modifications thereof for use
in hardware or software products claiming conformance to the MPEG-4
Systems (ISO/IEC 14496-1). Those intending to use this software module
in hardware or software products are advised that its use may infringe
existing patents.  The original developer of this software module and
his/her company, the subsequent editors and their companies, and
ISO/IEC have no liability for use of this software module or
modifications thereof in an implementation.  Copyright is not released
for non MPEG-4 Systems (ISO/IEC 14496- 1) conforming products.
Helsinki University of Technology retains full right to use the code
for their own purpose, assign or donate the code to a third party and
to inhibit third parties from using the code for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. This copyright notice must be
included in all copies or derivative works. Copyright  1999.
************************************************************************/
#include "ProcessSound.h"
inline //riitta 061799
float SoundDirectivity::operator()(float input)
{
    int i=0;
	float out = 0.0;


    if(nof_coeff > 1){

		float tmp_a = 0.0;
		for (i=1;i<nof_coeff;i++) {
			tmp_a -= curr_coeffs_a[i] * DL_filter[pDL + i - 1];
			out += curr_coeffs_b[i] * DL_filter[pDL + i -1];
		}
		out += curr_coeffs_b[0] * (input + tmp_a);
		DL_filter[pDL] = tmp_a + input;
		pDL = (pDL+1)%(nof_coeff-1);

 
	}

	if(nof_coeff == 1){
		out = curr_coeffs_b[0]*input;

	}
    return out;
}

inline //riitta 061799
float IIRFilt::operator()(float input)
{
 
    if(order == 0){
		return (coeffs_b[0]*input);
	}

	else {
	    int i=0;
		float out = 0.0;

		float tmp_a = 0.0;
		for (i=1;i<no_coeffs;i++) {
			tmp_a -= coeffs_a[i] * DL_IIR[pDL + i - 1];
			out += coeffs_b[i] * DL_IIR[pDL + i -1];
		}
		out += coeffs_b[0] * (input + tmp_a);
		DL_IIR[pDL] = tmp_a + input;
		pDL = (pDL+1)%(no_coeffs-1);

		return out;
 
	}

}

/*
inline // riitta 061799
void Auralize::Process(short* input, short* output, int spat_buf_length, float totalgain)
{
	

}
*/

inline //riitta 061799
float* Auralize::operator()(float input)
{
	float tmp = 0.0;;
	float* tmp_out;
//	float outrev[2];
	out[0] = out[1] = 0.0;
	int i;

/*	float d1; 
	float d2;
	float h0;
	float h1;
	float h2;
	float D;
*/
	//float tmp_interp = count/update_period;

	int d_low = 0;
	DL_dist[pDL] = input;

	float del = dist_samp_old + ((dist_samp-dist_samp_old)/(update_period))*count;
	d_low = (int) floor(del);
	float interp_gain = del - d_low;
	tmp = dist_gain*(DL_dist[(pDL+d_low)%(max_dist_delay+1)]*(1-interp_gain)
			+ DL_dist[(pDL+d_low+1)%(max_dist_delay+1)]*interp_gain);

	if(airabs)
		tmp = (*airabs)(tmp); 

	if(soundsource->visibility == FALSE){

		
		for(i=0;i<soundsource->iirfilts.GetLength();i++){
			tmp = (*(soundsource->iirfilts[i]))(tmp);
		}
		/*
		if(transfilt){
			tmp = (*transfilt)(tmp);
		}*/
	}

	if(reverb){
		tmp_out = (*reverb)(tmp*0.5);
		out[0] = tmp_out[0];
		out[1] = tmp_out[1];
	}

	if(dir)
		tmp = (*dir)(tmp);

	if(pan){
		tmp_out = ((*pan)(tmp));
		out[0] += tmp_out[0];
		out[1] += tmp_out[1];
	}
	else if(panitd){
		tmp_out = ((*panitd)(tmp));
		out[0] += tmp_out[0];
		out[1] += tmp_out[1];
	}
	else {
		out[0] += tmp;
		out[1] += tmp;
	}


	
	if(roomEffect){
		for(i=0;i<nof_ref;i++){
			if(vis_ref[i] == TRUE){
				if(dist_samp_ref[i] == dist_samp_ref_old[i]){ // Not moving
					tmp = (dist_gain_ref_old[i])*
						DL_dist[(pDL + (int)floor(dist_samp_ref[i]))%(max_dist_delay+1)];
				}
				else{	
					float del = dist_samp_ref_old[i] + ((dist_samp_ref[i]-dist_samp_ref_old[i])/(update_period))*count;
					d_low = (int) floor(del);
					float interp_gain = del - d_low;
					tmp = dist_gain_ref[i]*
						(DL_dist[(pDL+d_low)%(max_dist_delay+1)]*(1-interp_gain)
						+ DL_dist[(pDL+d_low+1)%(max_dist_delay+1)]*interp_gain);
				}
				if(useairabs)	
					tmp = (*(ref_airabs[i]))(tmp);
				if(ref_filt[i])
					tmp = (*(ref_filt[i]))(tmp);
				if(spatialize == TRUE){
					tmp_out = (*(ref_panitd[i]))(tmp);
					out[0] += tmp_out[0];
					out[1] += tmp_out[1];
				}
				else {
					out[0] += tmp;
					out[1] += tmp;
				}
			} //if
		}//for
	}//if

	pDL = (pDL+max_dist_delay)%(max_dist_delay+1);

	if(count < (update_period-1))
		count++;
	else{
		count = 0;
	}

	return (float*)out;

}



