#include "OpenGl/OpenGl.h"
#ifdef ADV_AUDIO
/**********************************************************************
This software module was originally developed by Riitta Vaananen
(Helsinki University of Technology) in the course of development of
the MPEG-4 Systems (ISO/IEC 14496-1) standard.  This software module
is an implementation of a part of one or more MPEG-4 Systems (ISO/IEC
14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1)
standard. ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1)
free license to this software module or modifications thereof for use
in hardware or software products claiming conformance to the MPEG-4
Systems (ISO/IEC 14496-1). Those intending to use this software module
in hardware or software products are advised that its use may infringe
existing patents.  The original developer of this software module and
his/her company, the subsequent editors and their companies, and
ISO/IEC have no liability for use of this software module or
modifications thereof in an implementation.  Copyright is not released
for non MPEG-4 Systems (ISO/IEC 14496- 1) conforming products.
Helsinki University of Technology retains full right to use the code
for their own purpose, assign or donate the code to a third party and
to inhibit third parties from using the code for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. This copyright notice must be
included in all copies or derivative works. Copyright  1999.
************************************************************************/

#include "ProcessSound.h"
#include "../AcousticSceneProxy.h"

#define FADINGTIME ((float)0.1)

ProcessDirectiveSound::ProcessDirectiveSound(int s_freq,float viewp[4], 
											 DirectiveSound* DS, Matrix tr,
											 int update, 
											 ZArray<AcousticScene*>& pAScene,
											 ZArray<Matrix>& trAScene):
samp_freq(s_freq),
intensity(DS->intensity), 
roomEffect(DS->roomEffect/*FALSE*/),
aural(NULL),
out_gain(1.0),
size_x(-1),
directsound(NULL),
update_period(update),
pCurrentAcousticScene(NULL),
pNewAcousticScene(NULL)	
{

	int i=0;
	for(i=0;i<2;i++){
		out[i] = 0.0;
	}


	view_coord.x = viewp[0];
	view_coord.y = viewp[1];
	view_coord.z = viewp[2];

	
	GetSoundInfo(tr, DS); 

	directsound = new SoundSource(sound_loc,samp_freq);

	pNewAcousticScene = NULL;

	// Check whether the sound source and the viewpoint are inside 
	// one of the acoustic scene regions (deifned by size and center
	// of the AcousticScene node)
	pNewAcousticScene = WhichAcousticScene(pAScene, trAScene);

	if(pNewAcousticScene){

		((AcousticSceneProxy*)pNewAcousticScene->GetRenderingObject())->MakeAcousticPolygons(); 

// riitta: 161199
		int nofSurf = 
			(((AcousticSceneProxy*)pNewAcousticScene->
			GetRenderingObject())->
			a_surfaces).GetLength();
			
		if((bool)roomEffect == TRUE && nofSurf > 0){
			directsound->SetVisibility(viewp, 
				((AcousticSceneProxy*)pNewAcousticScene->GetRenderingObject())->a_surfaces, 
				sound_loc,
				/*imagesources.GetLength()*/nofSurf /*riitta 161199*/); 
		}

		if((bool)roomEffect == TRUE)
			MakeImageSourceArray(viewp, pNewAcousticScene);

		aural = new Auralize(samp_freq, 
			update_period, 
			viewp, 
			DS, 
			sound_dir, 
			pNewAcousticScene,
			imagesources, 
			directsound);

		pCurrentAcousticScene = pNewAcousticScene;
	}
	else pCurrentAcousticScene = NULL; // added 12.7.1999
}

ProcessDirectiveSound::~ProcessDirectiveSound(void)
{
	if(aural){
		delete aural;
		aural = NULL;
	}

	// Riitta 021199
	EmptyImagesources();

	// riitta 2100999
	if(directsound){
		delete directsound;
		directsound = NULL;
	}

	// Riitta 021199
	pCurrentAcousticScene = NULL;
pNewAcousticScene = NULL;
}

void ProcessDirectiveSound::GetSoundInfo(Matrix tr, DirectiveSound* ds)
{
	const Vec3f sound_pos = ds->location;
	const Vec3f sound_d = ds->direction;
	
	old_sound_loc = sound_loc;

	sound_loc = ProcessSound::TransformPoints(tr, sound_pos);
	sound_dir = ProcessSound::TransformPoints(tr, sound_d);
	sound_dir.x = sound_dir.x - tr[12];
	sound_dir.y = sound_dir.y - tr[13];
	sound_dir.z = sound_dir.z - tr[14];


}

/* Updates parameters according to the changes in the viewpoint or
sound position, or in the changes of the environment (i.e., takes into
account movement of acoustic surfaces etc.)*/

void ProcessDirectiveSound::Update(float viewp[4], DirectiveSound* DS, Matrix tr, 
								ZArray<AcousticScene*>& pAScene, ZArray<Matrix>& trAScene)
{
	
	view_coord.x = viewp[0];
	view_coord.y = viewp[1];
	view_coord.z = viewp[2];

	GetSoundInfo(tr, DS);


	directsound->location = sound_loc;

	pNewAcousticScene = WhichAcousticScene(pAScene, trAScene);
	
	if(pNewAcousticScene){
		((AcousticSceneProxy*)pNewAcousticScene->GetRenderingObject())->MakeAcousticPolygons();
		if(pNewAcousticScene == pCurrentAcousticScene){
			int nofSurf = (((AcousticSceneProxy*)pNewAcousticScene->GetRenderingObject())->a_surfaces).GetLength();
			if(nofSurf > 0 && ((bool)roomEffect == TRUE)){
				directsound->SetVisibility(viewp, ((AcousticSceneProxy*)pNewAcousticScene->GetRenderingObject())->
					a_surfaces, sound_loc, nofSurf);
				UpdateImageSourceArray(viewp, pNewAcousticScene);
			}
			if(aural == NULL){ // create new Auralize object
				aural = new Auralize(samp_freq, 
					update_period, 
					viewp, 
					DS, 
					sound_dir,
					pNewAcousticScene,
					imagesources, 
					directsound);
			}

			else if (aural){
				aural->UpdateAll(viewp, 
					sound_dir, 
					pNewAcousticScene,
					imagesources, 
					directsound);

			}
		}
		else { // if the acoustic scene changes, or there wasn't one before
			if(aural) {
				delete aural;
				aural = NULL;



// riitta 161199
				EmptyImagesources();
			}


// riitta 161199

			int nofSurf = (((AcousticSceneProxy*)pNewAcousticScene->
				GetRenderingObject())->
				a_surfaces).GetLength();
			directsound->visibility = TRUE;
			if((bool)roomEffect == TRUE && nofSurf > 0){
				directsound->SetVisibility(viewp, 
					((AcousticSceneProxy*)pNewAcousticScene->GetRenderingObject())->a_surfaces, 
					sound_loc,
					/*imagesources.GetLength()*/nofSurf); 

			}

			if((bool)roomEffect == TRUE)
			MakeImageSourceArray(viewp, 
				pNewAcousticScene);

			aural = new Auralize(samp_freq, 
				update_period, 
				viewp, 
				DS, 
				sound_dir, 
				pNewAcousticScene,
				imagesources, 
				directsound);
		//	pCurrentAcousticScene = pNewAcousticScene;
		}
		pCurrentAcousticScene = pNewAcousticScene; // added 12.7.1999
	}

	else { // if none of the current scenes are active, delete the shit if it exists
		if(NULL != aural){
			delete aural;
			aural = NULL;
		}
		pCurrentAcousticScene = NULL; // changed 12.7.1999

		// Added 021199
		EmptyImagesources();

	}
}
 
BOOL
ProcessDirectiveSound::InsideBbox(AcousticScene* pAS, Matrix& trAS)
{
	
	Vec3f center = pAS->center;
	Vec3f size = pAS->Size;

  	size_x = size.x;
	if(size_x < 0){
		out_gain = 1.0;	
		return TRUE;		
	}

	Vec3f size_2;
	size_2.x = size.x/2;
	size_2.y = size.y/2;
	size_2.z = size.z/2;
	
	Vec3f bB_min = center;
	bB_min -= size_2;
	Vec3f bB_max = center; 
	bB_max += size_2;

	bB_min = ProcessSound::TransformPoints(trAS, bB_min);
	bB_max = ProcessSound::TransformPoints(trAS, bB_max);	

	if((view_coord.x < bB_min.x) || (view_coord.x > bB_max.x) ||
		(view_coord.y < bB_min.y) || (view_coord.y > bB_max.y) ||
		(view_coord.z < bB_min.z) || (view_coord.z > bB_max.z)){	
		out_gain = 0.0;
		return FALSE;
	}

	else if((sound_loc.x < bB_min.x) || (sound_loc.x > bB_max.x) ||
		(sound_loc.y < bB_min.y) || (sound_loc.y > bB_max.y) ||
		(sound_loc.z < bB_min.z) || (sound_loc.z > bB_max.z)) {
		out_gain = 0.0;
		return FALSE;
	}

	else {
		out_gain = 1.0;
		return TRUE;
	}
}

AcousticScene* 
ProcessDirectiveSound::WhichAcousticScene(ZArray<AcousticScene*>& pAScene, 
										  ZArray<Matrix>& trAScene)
{
	int i=0;
	for(i=0;i<pAScene.GetLength();i++){
		if(InsideBbox(pAScene[i], trAScene[i]))
			return pAScene[i];				
	}
	return NULL;
}

void ProcessDirectiveSound::MakeImageSourceArray(float viewp[4], AcousticScene* pAS)
{
	int i=0;
	int l = (((AcousticSceneProxy*)pAS->GetRenderingObject())->a_surfaces).GetLength();
	Vec3f location;
	for(i=0;i<l;i++){
		ISLocation(&location, 
			((AcousticSceneProxy*)pAS->GetRenderingObject())->a_surfaces[i], 
			sound_loc);		
		imagesources += new ImageSource(location, 
			1, 
			((AcousticSceneProxy*)pAS->GetRenderingObject())->a_surfaces[i]);
	} // end of for

	for(i=0;i<l;i++){
		imagesources[i]->SetVisibility(viewp,
			((AcousticSceneProxy*)pAS->GetRenderingObject())->a_surfaces, 
			sound_loc, 
			l);
	}
}

void ProcessDirectiveSound::UpdateImageSourceArray(float viewp[4], AcousticScene* pAS)
{
	int i=0;
	int l = (((AcousticSceneProxy*)pAS->GetRenderingObject())->a_surfaces).GetLength();
	Vec3f location;

	for(i=0;i<l;i++){
		ISLocation(&location, 
			((AcousticSceneProxy*)pAS->GetRenderingObject())->a_surfaces[i], 
			sound_loc);
		imagesources[i]->location = location;
		imagesources[i]->SetVisibility(viewp,
			((AcousticSceneProxy*)pAS->GetRenderingObject())->a_surfaces, 
			sound_loc, 
			l);
	} // end of for
}


void ProcessDirectiveSound::ISLocation(Vec3f* loc, 
									   AcousticSurface* surf, 
									   Vec3f s_loc)
{
	Vec3f n;
	Vec3f c;
	float dist;
	float p;

	n = surf->normal;
	c = surf->surface_point;
	p = n.x * c.x + n.y * c.y + n.z * c.z;
	dist = p - (s_loc.x * n.x + s_loc.y * n.y + s_loc.z * n.z);
	loc->x = s_loc.x + 2 * dist * n.x;
	loc->y = s_loc.y + 2 * dist * n.y;
	loc->z = s_loc.z + 2 * dist * n.z;

}

// riitta 161199:
void ProcessDirectiveSound::EmptyImagesources()
{
	int i=0;
	for(i=0;i<imagesources.GetLength();i++){
		if(imagesources[i]){
			delete imagesources[i];
			imagesources.SetAt(i,NULL);
		}
	}
	imagesources.Clear();
}

float* ProcessDirectiveSound::operator()(float input)
{
	float* tmp;
	float in = input*intensity*out_gain;
	if(aural){
		tmp = (*aural)(in);
		out[0] = tmp[0];
		out[1] = tmp[1];
	}

	else {
		out[0] = 0.0;
		out[1] = 0.0;
	//	out[0] = input;
	//	out[1] = input;
	}
	return out;
}




//========================================================
//
//  class ImageSource
//
//========================================================

ImageSource::ImageSource(Vec3f loc,
						 int ord, 
						 AcousticSurface* surf):
order(ord),
location(loc),
visibility(0),
a_surf(surf)
{
}

ImageSource::~ImageSource()
{
}

void
ImageSource::SetVisibility(float viewp[4], 
						   ZArray<AcousticSurface*>& a_s, 
						   Vec3f s_l, 
						   int nofsurf)
{
	int i=0;
	Vec3f tmp;
	float tmp_vp[4];
	tmp_vp[0] = s_l.x;
	tmp_vp[1] = s_l.y;
	tmp_vp[2] = s_l.z;
	tmp_vp[3] = 0.0;

	// riitta 03072000
	// check first if the reflectivity of a surface is 0.0
	if((bool)a_surf->filter_form_ref == TRUE){
		if(a_surf->nof_refcoeffs == 1 && (a_surf->ref_b)[0] == 0.0){
			visibility = FALSE;
			return;
		}
	}

	Vec3f is;
	visibility = Visibility(viewp, location, a_surf, &is);

	if(visibility == TRUE){
		for(i=0;i<nofsurf;i++){
			if(a_s[i] != a_surf){
				if(Visibility(tmp_vp,
					is,
					a_s[i],
					&tmp) == TRUE){
					visibility = FALSE;
					return;
				}
				if(Visibility(viewp, is, a_s[i], &tmp) == TRUE){
					visibility = FALSE;
					return;
				}
			} // if
		} // for
	}

}

BOOL
ImageSource::Visibility(float viewp[4], Vec3f loc, AcousticSurface* surf, Vec3f* y)
{
	Vec3f t1;;
	Vec3f t2;
	Vec3f s;

	t1.x=viewp[0]-surf->surface_point.x;
	t1.y=viewp[1]-surf->surface_point.y;
	t1.z=viewp[2]-surf->surface_point.z;

	t2.x=loc.x-surf->surface_point.x;
	t2.y=loc.y-surf->surface_point.y;
	t2.z=loc.z-surf->surface_point.z;

	s.x = loc.x-viewp[0];
	s.y = loc.y-viewp[1];
	s.z = loc.z-viewp[2];

	float d1 = t1.x*surf->normal.x + t1.y*surf->normal.y + t1.z*surf->normal.z;
	float d2 = t2.x*surf->normal.x + t2.y*surf->normal.y + t2.z*surf->normal.z;

	if(d1*d2 >= 0){ 
		return FALSE;
	}

	if(fabs(d1) < 0.0000001){
		return FALSE;
	}


	float sn = s.x*surf->normal.x + s.y*surf->normal.y + s.z*surf->normal.z;
	float abs_s = sqrt(pow(s.x,2) + pow(s.y,2) + pow(s.z,2));
	float x;
	if(fabs(sn/abs_s) > 1-0.0000001){
		x = fabs(d1)/abs_s;
		y->x = viewp[0] + s.x*x;
		y->y = viewp[1] + s.y*x;
		y->z = viewp[2] + s.z*x;
	}
	else {
		x = fabs((t1.x*surf->normal.x + t1.y*surf->normal.y + t1.z*surf->normal.z)/
			(sn));
		y->x = viewp[0] + x*s.x;
		y->y = viewp[1] + x*s.y;
		y->z = viewp[2] + x*s.z;
	}

	int i=0;
	float f = 1;
	Vec3f v1; 
	Vec3f v2;
	Vec3f v_new;
	Vec3f v_old;
	float ang = 0.0;
	float abs_v1;
	float abs_v2;
	float abs_vnew;
	float abs_vold;
	float scalar_v;

	int nof_vert = surf->nof_vert;
	for(i=0;i<nof_vert;i++){

		v1.x = surf->vertices[i].x-y->x;
		v1.y = surf->vertices[i].y-y->y;
		v1.z = surf->vertices[i].z-y->z;
		
		abs_v1 = sqrt(pow(v1.x,2) + pow(v1.y,2) + pow(v1.z,2));
		if(abs_v1 < 0.0000001){
			return FALSE;
		}

		if(i == nof_vert-1){
			v2.x = surf->vertices[0].x-y->x;
			v2.y = surf->vertices[0].y-y->y;
			v2.z = surf->vertices[0].z-y->z;
		}
		else{
			v2.x = surf->vertices[i+1].x-y->x;
			v2.y = surf->vertices[i+1].y-y->y;
			v2.z = surf->vertices[i+1].z-y->z;
		}

		abs_v2 = sqrt(pow(v2.x,2) + pow(v2.y,2) + pow(v2.z,2));
		if(abs_v2 < 0.0000001){
			return FALSE;
		}

		v_new.x = v1.y*v2.z - v1.z*v2.y; // vektoritulo
		v_new.y = v1.z*v2.x - v1.x*v2.z;
		v_new.z = v1.x*v2.y - v1.y*v2.x;

		abs_vnew = sqrt(pow(v_new.x,2) + pow(v_new.y,2) + pow(v_new.z,2));

		if(i > 0){
			if(((v_new.x * v_old.x) + (v_new.y * v_old.y) + (v_new.z * v_old.z))/(abs_vnew*abs_vold) < -0.8)
				f = -f;
		}

		v_old = v_new;
		abs_vold = abs_vnew;

		scalar_v = (v1.x*v2.x + v1.y*v2.y + v1.z*v2.z)/(abs_v1 * abs_v2);

		if(scalar_v < (-1.0 + 0.0000001)){
			return FALSE;
		}
		
		if(scalar_v > (1 - 0.0000001)){
		}
		else{
			ang += f*acosf(scalar_v);
		}
	}

	if(ang > 5){
		return TRUE;
	}
	if(ang < 1.0){
		return FALSE;
	}
	return 0;
}


SoundSource::SoundSource(Vec3f loc, float sfreq):
location(loc),
visibility(TRUE),
sf(sfreq)
{
}

SoundSource::~SoundSource()
{

	int i=0;
	for(i=0;i<iirfilts.GetLength();i++){
		delete iirfilts[i];
		iirfilts.SetAt(i,NULL);
	}
	iirfilts.Clear();

	// riitta 03072000
	for(i=0;i<eqfilts.GetLength();i++){
		delete eqfilts[i];
		eqfilts.SetAt(i,NULL);
	}
	eqfilts.Clear();

	// riitta 210999

	for(i=0;i<asurf.GetLength();i++)
		asurf.SetAt(i,NULL);
	asurf.Clear(); // Riitta 021199
}

void SoundSource::SetVisibility(float viewp[4], 
								ZArray<AcousticSurface*>& a_s, 
								Vec3f loc, int nofsurf)
{
	location = loc;
	int i=0;
	BOOL vis;
	Vec3f tmp;
	int t=0;

	visibility = TRUE;

	for(i=0;i<nofsurf;i++){
		vis = Visibility(viewp, location, a_s[i], &tmp);
		if(vis == FALSE){
			t = asurf.Find(a_s[i]);
			if(t >= 0){
				// Riitta 021199
				asurf.DeleteFrom(t);

				if(iirfilts[t]){
					delete iirfilts[t];
					iirfilts.SetAt(t,NULL);					
				}
				iirfilts.DeleteFrom(t);
		
				if(eqfilts[t]){
					delete eqfilts[t];
					eqfilts.SetAt(t,NULL);					
				}
				eqfilts.DeleteFrom(t);
			}
		}
		if(vis == TRUE){
			visibility = FALSE;

			// Source becomes invisible with respect to this surface -> 
			// add new filter to the array
			if(asurf.Find(a_s[i]) < 0){
				asurf += a_s[i];
				// riitta 03072000
				if((bool)a_s[i]->filter_form_trans == TRUE){
					eqfilts += (EQFilters *)NULL; // SOLARIS PORTING - Added typecast
					if(a_s[i]->trans_b /*(l == 1) && ((a_s[i]->trans_b)[0] != 1.0)*/){
						int l = a_s[i]->nof_transcoeffs;
						iirfilts += new IIRFilt(a_s[i]->trans_b, a_s[i]->trans_a, l-1);
					}
				}
				else { // if(a_s[i]->filter_form_ref == FALSE)
					iirfilts += (IIRFilt *)NULL; // SOLARIS PORTING - Added typecast

					eqfilts += new EQFilters(a_s[i]->trans_param,
						a_s[i]->trans_freq,
						sf,
						a_s[i]->num_trans_freq);
						
				}
			}
		}
	}
}

#endif
