/***********************************************************************
This software module was originally developed by Riitta Vaananen
(Helsinki University of Technology) in the course of development of
the MPEG-4 Systems (ISO/IEC 14496-1) standard.  This software module
is an implementation of a part of one or more MPEG-4 Systems (ISO/IEC
14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1)
standard. ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1)
free license to this software module or modifications thereof for use
in hardware or software products claiming conformance to the MPEG-4
Systems (ISO/IEC 14496-1). Those intending to use this software module
in hardware or software products are advised that its use may infringe
existing patents.  The original developer of this software module and
his/her company, the subsequent editors and their companies, and
ISO/IEC have no liability for use of this software module or
modifications thereof in an implementation.  Copyright is not released
for non MPEG-4 Systems (ISO/IEC 14496- 1) conforming products.
Helsinki University of Technology retains full right to use the code
for their own purpose, assign or donate the code to a third party and
to inhibit third parties from using the code for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. This copyright notice must be
included in all copies or derivative works. Copyright  1999.
************************************************************************/

#include "OpenGl/OpenGl.h"
#include "ProcessSound.h"
#include "Player3d/Presenter.h" //SOLARIS PORTING - case typo

ProcessSound::ProcessSound(int stream_id, int s_freq, 
						   Presenter* pres, int update, 
						   MediaObject* parent_sound):
source_type(0),
sound_3D(NULL),
S3D(NULL),
parentSound(parent_sound)
{


#ifdef ADV_AUDIO
	DS = NULL;
	dirsound = NULL;
	m_spat=NULL;
	perceptualsound=NULL;
	m_spatparameters=NULL;
	m_pres=NULL;
#endif


	int i=0;
	for(i=0;i<4;i++)
		viewp[i] = 0.0;

	GetSoundInfo(stream_id, pres);
	SetupSoundProcessing(stream_id, s_freq, viewp, update, pres);
	old_time = pres->GetTime() / 1000.0;

}

ProcessSound::~ProcessSound(void)
{

	if(S3D != NULL){
		S3D = NULL;
	}

	if(sound_3D != NULL){
		delete sound_3D;
		sound_3D = NULL;
	}

#ifdef ADV_AUDIO

  if(dirsound != NULL){
		delete dirsound;
		dirsound = NULL;
	}

   if(DS != NULL){
		DS = NULL; 
	}

  	if(m_spat!=NULL)
		delete(m_spat);
	m_spat=NULL;
	if(m_spatparameters!=NULL)
		delete(m_spatparameters);
	m_spatparameters=NULL;

#endif //ADV_AUDIO

}

void ProcessSound::SetupSoundProcessing(int /*stream_id*/, int s_freq, 
										float viewp[4], int update_period,
										Presenter* pres)
{	
	
	if(source_type == 0){
		sound_3D = new ProcessSound3D(viewp, S3D, tr);
	}



#ifdef ADV_AUDIO

	if(source_type == 2){
		MediaObject* pScene = pres->GetScene();
			
		ZArray<AcousticScene*> rAcousticScenes;
		ZArray<Matrix> rASceneTransformations;
		Matrix rTransform;


		((MediaObjectProxy*)pScene->GetRenderingObject())->FindAcousticScenes(rAcousticScenes, 
						 rASceneTransformations,
						 rTransform);

		dirsound = new ProcessDirectiveSound(s_freq, viewp, DS, tr, 
			update_period, rAcousticScenes, rASceneTransformations);
	}

	if(source_type == 3)
	{
  		m_spat=new CSpatializer;
		m_spatparameters=new CSpatParameters;
	}

#endif // ADV_AUDIO

}


void ProcessSound::GetSoundInfo(int /*stream_id*/, Presenter* pres)
{	
	Time currentTime = pres->GetTime();

	old_time = currentTime;

	// riitta 13022000:
	//ML - changed MediaObject* pScene to Root* pRoot
	//satish added this line 
	MediaObject* pScene = pres->GetScene();
	Root* pRoot = pres->GetScene();
	Viewpoint* pActiveViewpoint = pRoot->GetBoundViewpoint();
	
	// riitta 13022000
//	MediaObject* pScene = pres->GetScene();
//	Viewpoint* pActiveViewpoint = pScene->GetRoot()->GetBoundViewpoint();
	Vec3f position(pActiveViewpoint->position /* GetPosition() */ );
	Rotation orientation(pActiveViewpoint->orientation /* GetOrientation() */ );

	int i=0;

	ZArray<Sound*> soundSources3D;
	ZArray<Sound2D*> soundSources2D;
	ZArray<Matrix> soundTransformations2D;
	ZArray<Matrix> soundTransformations3D;

#ifdef ADV_AUDIO
	ZArray<DirectiveSound*> dirsoundSources; 
	ZArray<Matrix> dirSoundTransformations;
#endif ADV_AUDIO

	Matrix rTransf_sound; 

#ifdef ADV_AUDIO 

	//riitta 13022000 change pScene to pRoot
//	((MediaObjectProxy*)pScene->GetRenderingObject())->FindSoundSources(
			((MediaObjectProxy*)pRoot->GetRenderingObject())->FindSoundSources(
    soundSources3D, soundTransformations3D, 
    soundSources2D, soundTransformations2D,
		dirsoundSources, dirSoundTransformations,
    rTransf_sound);

#else

  ((MediaObjectProxy*)pScene->GetRenderingObject())->FindSoundSources(
    soundSources3D, soundTransformations3D, 
    soundSources2D,soundTransformations2D, 
    rTransf_sound);

#endif // ADV_AUDIO

	for(i=0;i<soundSources3D.GetLength();i++){
		if(soundSources3D[i] == parentSound){
			source_type = 0;
			S3D = soundSources3D[i];
			tr = soundTransformations3D[i];
		}
	}


#ifdef ADV_AUDIO
	for(i=0;i<dirsoundSources.GetLength();i++){
		if(dirsoundSources[i] == parentSound){

			DS = dirsoundSources[i];
			tr = dirSoundTransformations[i];
			PerceptualParameters *mP;
			mP=dirsoundSources[i]->perceptualParameters;
			if(mP)
			{	source_type = 3;
				if(m_spatparameters)
					m_spatparameters->upDate(DS);
				m_pres=pres;
			}
			else
				source_type = 2;
		}
	}

#endif // ADV_AUDIO

	viewp[0] = position.x;
	viewp[1] = position.y;
	viewp[2] = position.z;
	viewp[3] = orientation.theta*orientation.y;
}

void ProcessSound::Update(int stream_id, Presenter* pres)
{
	GetSoundInfo(stream_id, pres);

	if(sound_3D){
		sound_3D->Update(viewp, S3D, tr);
	}

#ifdef ADV_AUDIO
	if(dirsound){

		MediaObject* pScene = pres->GetScene();
			
		ZArray<AcousticScene*> rAcousticScenes;
		ZArray<Matrix> rASceneTransformations;
		Matrix rTransform;

		((MediaObjectProxy*)pScene->GetRenderingObject())->FindAcousticScenes(rAcousticScenes, 
						 rASceneTransformations,
						 rTransform);

		dirsound->Update(viewp, DS, tr, rAcousticScenes, rASceneTransformations);
	}
#endif // ADV_AUDIO

}

Vec3f 
ProcessSound::TransformPoints(Matrix matrix, Vec3f coord)
{

  Vec3f temp;
  int i=0;
//  int j=0;
   
  for (i = 0; i < 3; i++) {
	temp.x += coord[i] * matrix[(i * 4) + 0];
	temp.y += coord[i] * matrix[(i * 4) + 1]; 
    temp.z += coord[i] * matrix[(i * 4) + 2];
  }

  temp.x += matrix[12];
  temp.y += matrix[13];
  temp.z += matrix[14];
  
  return temp;
}

float* ProcessSound::operator()(float input)
{	float* tmp;
	if(sound_3D){
		tmp = (*sound_3D)(input);
		out[0] = tmp[0];
		out[1] = tmp[1];
	}
	else if(dirsound){
		tmp = (*dirsound)(input); out[0] = tmp[0]; out[1] = tmp[1];
	}
	else{
	out[0] = input; 
	out[1] = input;
	}
	return (float*)out;
}


#ifdef ADV_AUDIO

//
//  Function to process Bufflength samples at a time
//
//

void ProcessSound::process(int /*stream_id*/,void* input,short *output,int spat_buf_length)
{

	int i;

//	float* tmp;
	short* temp = (short*)(input);

	if(m_spat) // ... JB: Do the perceptual sound processing here
	{
		float azimuth;
		float directivityAngle;
		float x,y,z;				// coordinates

		GetSoundPositionAndAngles(&azimuth,&directivityAngle,&x,&y,&z); 
		m_spat->Update(m_spatparameters,azimuth,directivityAngle,x,y,z);
		m_spat->DoFiltering(temp,output,spat_buf_length/2);

	}

	else
	{	for(i=0;i<spat_buf_length;i=i+2)	
		{	output[i]	= *temp;
			output[i+1] = *temp;	
			temp ++;
		} // for
	}

}

void ProcessSound::GetSoundPositionAndAngles(	float* azimuth, 
												float* dirAngle,
												float* x, float* y, float* z)
{

		float elevation;

// position of the view point and orientation

		// JBR until 24 April 2000 :
//		MediaObject* pScene = m_pres->GetScene();
//		Viewpoint* pActiveViewpoint = pScene->GetRoot()->GetBoundViewpoint();
		// JBR after 24 April 2000 see // riitta 13022000 :
		Root* pRoot = m_pres->GetScene();
		Viewpoint* pActiveViewpoint = pRoot->GetBoundViewpoint();
		Vec3f position(pActiveViewpoint->position);
		Rotation orientation(pActiveViewpoint->orientation);

	
// base vectors of the view point, the viewing direction is -kv

		vector Ov;					
		vector iv,jv,kv;
		vector direction;

		Ov[0]=position.x;
		Ov[1]=position.y;
		Ov[2]=position.z;

		iv[0]=1.f;		// initial values
		iv[1]=0.f;
		iv[2]=0.f;

		jv[0]=0.f;
		jv[1]=1.f;
		jv[2]=0.f;

		kv[0]=0.f;
		kv[1]=0.f;
		kv[2]=1.f;

		direction[0]=orientation.x;
		direction[1]=orientation.y;
		direction[2]=orientation.z;

		rotateVector(iv,direction,orientation.theta); // after rotation
		rotateVector(jv,direction,orientation.theta);
		rotateVector(kv,direction,orientation.theta);


// position and direction of the Sound Source
		
		vector Os;					
		Vec3f sound_pos = DS->location;
		Vec3f sound_dir = DS->direction;

		sound_pos = TransformPoints(tr, sound_pos);
		sound_dir = TransformPoints(tr, sound_dir);
		sound_dir.x = sound_dir.x - tr[12];
		sound_dir.y = sound_dir.y - tr[13];
		sound_dir.z = sound_dir.z - tr[14];

		Os[0]=sound_pos.x;
		Os[1]=sound_pos.y;
		Os[2]=sound_pos.z;


// azimuth and elevation of sound as seen from the view point

		computeAngles(azimuth,&elevation,Ov,iv,jv,kv,Os);


// directivity angle : view point as "seen" from the Sound Source

		vector ks;
		vector OsOv;				// from sound to viewpoint

		ks[0]=sound_dir.x;
		ks[1]=sound_dir.y;
		ks[2]=sound_dir.z;

		OsOv[0]=Ov[0]-Os[0];
		OsOv[1]=Ov[1]-Os[1];
		OsOv[2]=Ov[2]-Os[2];

		computeAnglesofTwoVectors(dirAngle,OsOv,ks);


// relative position of Sound/View Point

		*x=sound_pos.x-position.x;
		*y=sound_pos.y-position.y;
		*z=sound_pos.z-position.z;


}

#endif





