/***********************************************************************
This software module was originally developed by Riitta Vaananen
(Helsinki University of Technology) in the course of development of
the MPEG-4 Systems (ISO/IEC 14496-1) standard.  This software module
is an implementation of a part of one or more MPEG-4 Systems (ISO/IEC
14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1)
standard. ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1)
free license to this software module or modifications thereof for use
in hardware or software products claiming conformance to the MPEG-4
Systems (ISO/IEC 14496-1). Those intending to use this software module
in hardware or software products are advised that its use may infringe
existing patents.  The original developer of this software module and
his/her company, the subsequent editors and their companies, and
ISO/IEC have no liability for use of this software module or
modifications thereof in an implementation.  Copyright is not released
for non MPEG-4 Systems (ISO/IEC 14496- 1) conforming products.
Helsinki University of Technology retains full right to use the code
for their own purpose, assign or donate the code to a third party and
to inhibit third parties from using the code for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. This copyright notice must be
included in all copies or derivative works. Copyright  1999.
************************************************************************/


// riitta 13022000 change Nodes.h to OpenGl.h

#include "OpenGl/OpenGl.h"
#include "ProcessSound.h"
//#include "core/Nodes.h"

ProcessSound3D::ProcessSound3D(float viewp[4], Sound* S3D, Matrix tr)
{

	pan = NULL;
	ellip = NULL;
	intensity = 1.0;
	priority = 1.0;
	spatialize = S3D->spatialize;
	GetSoundInfo(S3D, tr);

	if((bool)spatialize == TRUE){
		pan = new AmPan(viewp, sound_loc);
	}
	if((ellipse_def[0] && ellipse_def[1]) || (ellipse_def[2] && ellipse_def[3])){
		ellip = new EllipseSound(viewp, sound_loc, sound_dir, ellipse_def);
	}
	intensity = S3D->intensity;
}

ProcessSound3D::~ProcessSound3D()
{

	if(pan){
		delete pan;
		pan = NULL;
	}

	if(ellip){
		delete ellip;
		ellip = NULL;
	}
}

void ProcessSound3D::GetSoundInfo(Sound* S3D, Matrix tr)
{


	const Vec3f sound_pos = S3D->location;
	const Vec3f sound_d = S3D->direction;
	ellipse_def[0] = S3D->minBack;
	ellipse_def[1] = S3D->minFront;
	ellipse_def[2] = S3D->maxBack;
	ellipse_def[3] = S3D->maxFront;


	sound_loc = ProcessSound::TransformPoints(tr, sound_pos);
	sound_dir = ProcessSound::TransformPoints(tr, sound_d);
	sound_dir.x = sound_dir.x - tr[12];
	sound_dir.y = sound_dir.y - tr[13];
	sound_dir.z = sound_dir.z - tr[14];

	intensity = S3D->intensity;
}

void ProcessSound3D::Update(float viewp[4], Sound* S3D, Matrix tr)
{

	GetSoundInfo(S3D, tr);
	if(pan)
		pan->ChangeGains(viewp, sound_loc);

	ellip->ChangeGain(viewp, sound_loc, sound_dir, ellipse_def);

}

float* ProcessSound3D::operator()(float input)
{
	float* tmp;
	float x = input*intensity;
	if(ellip){
		x = (*ellip)(x);			
	}

	if(pan){
		tmp = (*pan)(x);
		out[0] = tmp[0];
		out[1] = tmp[1];
	}
	else{
		out[0] = x;
		out[1] = x;
	}
	return (float*)out;
}



EllipseSound::EllipseSound(float viewp_from_presenter[4], Vec3f sound_loc_from_presenter, Vec3f sound_dir_from_presenter, float ellipse_definitions[4])
{
	Vec3f pos;
	GetRelativePosition(viewp_from_presenter, sound_loc_from_presenter, sound_dir_from_presenter, &pos); 
	gain = ComputeGain(ellipse_definitions, pos);
}

EllipseSound::~EllipseSound()
{
}

void EllipseSound::ChangeGain(float viewp_from_presenter[4], Vec3f sound_loc_from_presenter, 
							  Vec3f sound_dir_from_presenter, float ellipse_definitions[4])
{
	Vec3f pos;
	GetRelativePosition(viewp_from_presenter, sound_loc_from_presenter, sound_dir_from_presenter, &pos); 
	gain = ComputeGain(ellipse_definitions, pos);
}

void EllipseSound::GetRelativePosition(float viewp[4], Vec3f sound_loc, Vec3f sound_dir, Vec3f* p)
{
	//move the origo; x,y,z, are now the viewpoint
	float x = viewp[0]-sound_loc.x;
	float y = viewp[1]-sound_loc.y;
	float z = viewp[2]-sound_loc.z;
	//angle between the direction and viewpoint vectors:
	float abs_sounddir = sqrt(pow(sound_dir.x,2)+pow(sound_dir.y,2)+pow(sound_dir.z,2));
	float abs_viewp = (float) sqrt(pow(x,2)+pow(y,2)+pow(z,2));
	if(abs_viewp == 0)
		abs_viewp = 0.1;
	float cos_theta = (sound_dir.x*x+sound_dir.y*y+sound_dir.z*z)/(abs_viewp*abs_sounddir);
	
	float z_new = cos_theta*abs_viewp;
	float x_new = sqrt(pow(abs_viewp,2)-pow(z_new,2));
	float y_new = 0;	
	
	p->x = x_new; 
	p->y = y_new;
	p->z = z_new;
}
	
float EllipseSound::ComputeGain(float ellip_def[4], Vec3f pos)
{	
	float g = 0.0;
	float min_b = ellip_def[0];
	float min_f = ellip_def[1];
	float max_b = ellip_def[2];
	float max_f = ellip_def[3];
	
	float a_min = (min_f+min_b)/2;	//first ellipse axis
	float b_min = sqrt(min_b*min_f);
	float alpha_min = (min_f-min_b)/2; //origo from focus
	float k = 0.0;
	if(fabs(pos.z) >= 0.00001)	
		k=pos.x/pos.z; //
	
	float dist_foci_min = (min_f-min_b);
	
	float a_max = (max_f+max_b)/2;	//first ellipse axis
	float b_max = sqrt(max_b*max_f);
	float alpha_max = (max_f-max_b)/2; //origo from focus
	float dist_foci_max = (max_f-max_b);
	
	float x_min = 0.0;
	float x_max = 0.0;
	float y_min = 0.0;
	float y_max = 0.0;
	
	
float dist_from_foci_min = 
	sqrt(pow(pos.z,2) + pow(pos.x,2))+
	sqrt(pow((pos.z-dist_foci_min),2) + pow(pos.x,2));
	
float dist_from_foci_max = 
	sqrt(pow(pos.z,2) + pow(pos.x,2))+
	sqrt(pow((pos.z-dist_foci_max),2) + pow(pos.x,2));	

	float d_min = min_f+min_b;
	float d_max = max_f+max_b;
	
	if(dist_from_foci_max > d_max){
		//not heard
		return g;
	}
	
	else if (dist_from_foci_min <= d_min) {
		g = 1.0;
		printf("const gain: g: %f \n",g);
		return g;
	}
	
	else{
		//leikkauspiste min. ellipsin kanssa
		if(pos.z > 0.0001){
			x_min = (alpha_min*pow(b_min,2)+
				a_min*b_min*
				sqrt(pow((a_min*k),2)+pow(b_min,2)-pow((alpha_min*k),2)))/
				(pow(b_min,2)+pow((a_min*k),2));
			y_min = k*x_min;
		}
		else if(pos.z < -0.0001){
			x_min = (alpha_min*pow(b_min,2)-
				a_min*b_min*
				sqrt(pow((a_min*k),2)+pow(b_min,2)-pow((alpha_min*k),2)))/
				(pow(b_min,2)+pow((a_min*k),2));
			y_min = k*x_min;
		}
		else{
			x_min = 0;
			y_min = b_min*sqrt(1-pow((alpha_min/a_min),2));
			
		}
	
	
		if(pos.z >= 0.0001){
			x_max = (alpha_max*pow(b_max,2)+
				a_max*b_max*
				sqrt(pow((a_max*k),2)+pow(b_max,2)-pow((alpha_max*k),2)))/
				(pow(b_max,2)+pow((a_max*k),2));
			y_max = k*x_max;
		}
		else if(pos.z <= -0.0001){
			x_max = (alpha_max*pow(b_max,2)-
				a_max*b_max*
				sqrt(pow((a_max*k),2)+pow(b_max,2)-pow((alpha_max*k),2)))/
				(pow(b_max,2)+pow((a_max*k),2));
			y_max = k*x_max;
		}
	
		else {
			x_max = 0;
			y_max = b_max*sqrt(1-pow((alpha_max/a_max),2));
		}
	
		float y_pos = sqrt(pow(pos.x,2)+pow(pos.y,2));
		float x_pos = pos.z;
		float dist_ellip = sqrt(pow((y_max-y_min),2)+pow((x_max-x_min),2));
		float viewp_dist = sqrt(pow((y_pos-y_min),2)+pow((x_pos-x_min),2));
		g = pow(10.0,(-(viewp_dist/dist_ellip)));
	
	}

return g;
}

float EllipseSound::operator()(float input)
{
	float out = gain*input;
 	return out;
}








