#include "OpenGl/OpenGl.h"
#ifdef ADV_AUDIO
/***********************************************************************
This software module was originally developed by Riitta Vaananen
(Helsinki University of Technology) in the course of development of
the MPEG-4 Systems (ISO/IEC 14496-1) standard.  This software module
is an implementation of a part of one or more MPEG-4 Systems (ISO/IEC
14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1)
standard. ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1)
free license to this software module or modifications thereof for use
in hardware or software products claiming conformance to the MPEG-4
Systems (ISO/IEC 14496-1). Those intending to use this software module
in hardware or software products are advised that its use may infringe
existing patents.  The original developer of this software module and
his/her company, the subsequent editors and their companies, and
ISO/IEC have no liability for use of this software module or
modifications thereof in an implementation.  Copyright is not released
for non MPEG-4 Systems (ISO/IEC 14496- 1) conforming products.
Helsinki University of Technology retains full right to use the code
for their own purpose, assign or donate the code to a third party and
to inhibit third parties from using the code for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. This copyright notice must be
included in all copies or derivative works. Copyright  1999.
************************************************************************/

#include "Reverb.h"

LateReverb::LateReverb(int s_freq, 
					   AcousticScene* pAScene):
Kp(NULL),
Bp(NULL),
Ep(NULL),
NOF_DL(4),
ap_l(0),
gain(1.0),
fb_gain(0),
ap_p(0),
ap_out(0.0),
samp_freq(s_freq),
dl_p(NULL),
lp_out(NULL),
lp_del(NULL),
ap_dl(NULL),
dl_length(NULL),
init_delay(0), //for initial gain and delay
init_gain(1.0),
init_dl(NULL),
init_dlp(0),
dlp_in(NULL)
{
	int i=0;
	int j=0;
	int MAXLENGTH = (int)(s_freq*0.150);

	dl_length = new int[NOF_DL];	
	dl_length[0] = (int) (0.0745*s_freq); // lengths of the delay lines in seconds
	dl_length[1] = (int) (0.0867*s_freq);
	dl_length[2] = (int) (0.0964*s_freq);
	dl_length[3] = (int) (0.1034*s_freq);
	ap_l = (int) (0.006*s_freq);
	ap_gain = 0.0;

	Kp = new float[((int)(NOF_DL))];
	Bp = new float[((int)(NOF_DL))];
	Ep = new float[((int)(NOF_DL))];
	lp_out = new float[NOF_DL];
	lp_del = new float[NOF_DL];
	dl_p = new int[NOF_DL];

	if(pAScene->reverbDelay > 0.0){
		init_delay = (int)(pAScene->reverbDelay * (double)s_freq);
		init_dl = new float[init_delay];
		for(i=0;i<init_delay;i++)
			init_dl[i] = 0.0;
	}

	dlp_in = new int[NOF_DL];

	for(i=0;i<NOF_DL;i++)
			dlp_in[i] = 0;

	for(i=0;i<NOF_DL;i++){
		Kp[i] = 0.0;
		Bp[i] = 0.0;
		Ep[i] = 0.0;
		dl_p[i] = 0;
		lp_out[i] = 0.0;
		lp_del[i] = 0.0;
	}

	// Init delay lines	
	dl = new float*[NOF_DL];
	for(i=0;i<NOF_DL;i++){
		dl[i] = NULL;
		dl[i] = new float[MAXLENGTH];
		for(j=0;j<MAXLENGTH;j++)
			dl[i][j] = 0.0;
	}

	ap_dl = new float[ap_l];
	for(i=0;i<ap_l;i++)
		ap_dl[i] = 0.0;

	fb_gain = -2.0/NOF_DL;

	SetKp_And_Bp(pAScene/*, rev_len*/);
}

LateReverb::~LateReverb()
{
	int i=0;
	for(i=0;i<NOF_DL;i++){
		delete [] dl[i];
		dl[i] = NULL;
	}
	if(dl){
		delete [] dl;
		dl = NULL;
	}
	if(Kp){
		delete [] Kp;
		Kp = NULL;
	}
	if(Bp){
		delete [] Bp;
		Bp = NULL;
	}
	if(Ep){
		delete [] Ep;
		Ep = NULL;
	}
	if(dl_p){
		delete [] dl_p;
		dl_p = NULL;
	}
	if(lp_out){
		delete [] lp_out;
		lp_out = NULL;
	}
	if(lp_del){
		delete [] lp_del;
		lp_del = NULL;
	}
	if(ap_dl){
		delete [] ap_dl;
		ap_dl = NULL;
	}
	if(dl_length){
		delete [] dl_length;
		dl_length = NULL;
	}
}

void LateReverb::SetKp_And_Bp(AcousticScene* pAScene/*, int rev_len*/)
{
	int i=0;
	float rt_0 = 0.0;
	float rt_nyqvist = 0.0;

	float* b_and_k = new float[2];
	int n = (pAScene->reverbTime).GetLength();
	int nof_rtimes = (pAScene->reverbTime).GetLength();

	float* freqs = new float[nof_rtimes];
	float* gains = new float[nof_rtimes];
		
	if(n>1){
		for(i=0;i<nof_rtimes;i++){
			if(pAScene->reverbFreq.GetLength() == 0)
				freqs[0] = 1000;
			else
				freqs[i] = (pAScene->reverbFreq)[i];
							
			gains[i] = (pAScene->reverbTime)[i];
		}


		FitFirstOrderCurve(freqs, gains, nof_rtimes, b_and_k);
		rt_0 = b_and_k[0];
		rt_nyqvist = b_and_k[0] + b_and_k[1]*(samp_freq/2.0);
		if(rt_nyqvist <= 0.0)
			rt_nyqvist = 0.25 * rt_0;
	}

	else {
		if(n==1)
			rt_0 = (pAScene->reverbTime)[0];//reverbtime[0];
		
		rt_nyqvist = 0.25* rt_0;
	}

	for(i=0;i<NOF_DL;i++){
		Kp[i] = pow(10,-(3*dl_length[i])/(samp_freq*rt_0));
		Bp[i] =  1-2/(1+pow(Kp[i],(1-rt_0/rt_nyqvist)));
		Ep[i]=Kp[i]*(1-Bp[i]);
	}

	init_gain = pAScene->reverbLevel/Kp[0];

	delete [] b_and_k;
	delete [] freqs;
	delete [] gains;
}

void LateReverb::FitFirstOrderCurve(float* freqs, float* gains, int nof_rtimes, float* b_and_k)
{
	int i=0;
	float sum_y = 0.0;
	float sum_x = 0.0;
	float sum_x_y = 0.0;
	float sum_sq_x = 0.0;
	for(i=0;i<nof_rtimes;i++){
		sum_y += gains[i];
		sum_x += freqs[i];
		sum_sq_x += pow(freqs[i],2);
		sum_x_y += gains[i]*freqs[i];
	}
	b_and_k[0] = (sum_y*sum_sq_x - sum_x*sum_x_y)/(nof_rtimes*sum_sq_x - pow(sum_x,2));
	b_and_k[1] = (nof_rtimes*sum_x_y - sum_x*sum_y)/(nof_rtimes*sum_sq_x - pow(sum_x,2));

}

/*
void LateReverb::Update(MF<Time>& reverbtime, int rev_len)
{
	float* rev_time = new float[rev_len];
	int i=0;
	for(i=0;i<rev_len;i++){
		rev_time[i] = reverbtime[i];
	}
	SetKp_And_Bp(reverbtime, rev_len);

	delete [] rev_time;
	rev_time = NULL;
}
*/
float* LateReverb::operator()(float input)
{
	int i=0;
	float sum_lp = 0.0; //sum of the lowpass filter outputs
	out[0] = out[1] = 0.0;

	for(i=0;i<NOF_DL;i++){
		sum_lp+=lp_out[i];
	}

	out[0] = lp_out[0] - lp_out[2];
	out[1] = lp_out[1] - lp_out[3];

	if(init_dl){
		init_dl[init_dlp] = input * init_gain;		
		init_dlp = (init_dlp + 1)%init_delay;
		float temp = init_dl[init_dlp];
		for(i=0;i<4;i++){
			dl[i][dl_p[i]] = lp_out[i] -0.5*sum_lp; // beginning of delay line
			dl[i][(dl_p[i]+(1+dl_length[i]-dl_length[0]))%dl_length[i]] = temp+
				dl[i][(dl_p[i]+(1+dl_length[i]-dl_length[0]))%dl_length[i]]; // end of delay line
			dl_p[i] = (dl_p[i]+1)%(dl_length[i]);
			lp_out[i] = Ep[i]*(dl[i][dl_p[i]]+Bp[i]*lp_del[i]);
			lp_del[i] = dl[i][dl_p[i]];
		}
	}

	else{
		float temp = input - 0.5 * sum_lp;
		for(i=0;i<4;i++){
			dlp_in[i] = (dl_p[i] + init_delay)%(dl_length[i]);
			dl[i][dlp_in[i]] = temp + lp_out[i];
			dl_p[i] = (dl_p[i]+1)%(dl_length[i]);
			lp_out[i] = Ep[i]*(dl[i][dl_p[i]]+Bp[i]*lp_del[i]);
			lp_del[i] = dl[i][dl_p[i]];
		}
	}

	return (float*)out;
}

#endif