#ifndef _REVERB_H
#define _REVERB_H

/***********************************************************************
This software module was originally developed by Riitta Vaananen
(Helsinki University of Technology) in the course of development of
the MPEG-4 Systems (ISO/IEC 14496-1) standard.  This software module
is an implementation of a part of one or more MPEG-4 Systems (ISO/IEC
14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1)
standard. ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1)
free license to this software module or modifications thereof for use
in hardware or software products claiming conformance to the MPEG-4
Systems (ISO/IEC 14496-1). Those intending to use this software module
in hardware or software products are advised that its use may infringe
existing patents.  The original developer of this software module and
his/her company, the subsequent editors and their companies, and
ISO/IEC have no liability for use of this software module or
modifications thereof in an implementation.  Copyright is not released
for non MPEG-4 Systems (ISO/IEC 14496- 1) conforming products.
Helsinki University of Technology retains full right to use the code
for their own purpose, assign or donate the code to a third party and
to inhibit third parties from using the code for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. This copyright notice must be
included in all copies or derivative works. Copyright  1999.
************************************************************************/


#include <fstream.h>
#include "core/NodeFld.h"

class AcousticScene;
/** class for late reverberation processing.
*/
class LateReverb
{

public:
	/** Constructor.

	Sets up the reverberator. This includes creation and initialization of 
	delay lines, and absorption filter coefficients, as well as the delay 
	line pointers*/
	LateReverb(int s_freq, 
		AcousticScene* pAScene);

	/** Destructor.
	
	deletes the delay lines and the filter coefficient array */
	~LateReverb();

	/** Operator.
	
	Does the late reverberation processing to each sound sample.
	*/
	float* operator()(float input);

	/** Updates the absorptive filter parameters.*/
	void Update(MF<Time>& reverbtime);

	/** Sets the coefficients according to the reverberation time information. 
	
	The absorptive filters are first order allpole filters, and therefore
	a first order curve is fitted to the reverberation time given at different 
	frequency points. The coefficients are then computed according to the resulting 
	rt60 at the zero and fs/2 frequencies. */
	void SetKp_And_Bp(AcousticScene* pAScene);

private:
	void FitFirstOrderCurve(float* freqs,
	float* gains, 
	int nof_rtimes, 
	float* b_and_k);
	float out[2];
	float* lp_del; // lowpass delay (one sample)
	float* lp_out; // stores the lowpass filter output for the next round
	float ap_out;
	int samp_freq;
	float gain;
	float ap_gain;
		
	int NOF_DL; // Number of delays lines minimum is 4
	float fb_gain;  //gain for reverberation module
	float beta;   // Feedback coefficients
	float *Kp;  // Lowpass filters coefficients
	float *Bp;  // Lowpass filters coefficients
	float *Ep;  // Lowpass filters coefficients
	int *dl_length;            // Delay line lengths
	float **dl;   
	int *dl_p;         // DL pointers for delay lines
	int ap_p;	//pointer of the allpass filter delay line
	float* ap_dl; // allpass delay line
	int ap_l;	//allpass length

	// variables for implementing the initial delay (riitta: 070599)
	int init_delay;
	float *init_dl;
	float init_gain;
	int init_dlp;
	int *dlp_in;

};

#endif
