/***********************************************************************
This software module was originally developed by Riitta Vaananen
(Helsinki University of Technology) in the course of development of
the MPEG-4 Systems (ISO/IEC 14496-1) standard.  This software module
is an implementation of a part of one or more MPEG-4 Systems (ISO/IEC
14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1)
standard. ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1)
free license to this software module or modifications thereof for use
in hardware or software products claiming conformance to the MPEG-4
Systems (ISO/IEC 14496-1). Those intending to use this software module
in hardware or software products are advised that its use may infringe
existing patents.  The original developer of this software module and
his/her company, the subsequent editors and their companies, and
ISO/IEC have no liability for use of this software module or
modifications thereof in an implementation.  Copyright is not released
for non MPEG-4 Systems (ISO/IEC 14496- 1) conforming products.
Helsinki University of Technology retains full right to use the code
for their own purpose, assign or donate the code to a third party and
to inhibit third parties from using the code for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. This copyright notice must be
included in all copies or derivative works. Copyright  1999.
************************************************************************/
#include "OpenGl/OpenGl.h"
#include "Spatialize.h"

PanITD::PanITD(float viewp_from_presenter[4], 
			   Vec3f sound_loc_from_presenter, 
			   int /*sf*/, 
			   BOOL /*t*/, 
			   BOOL l, 
			   int update):
pan(NULL),
update_period(update)
{
	float angle = 0.0;

	if(l == TRUE){
		pan = new AmPan(viewp_from_presenter, sound_loc_from_presenter);
		angle = pan->GetAngle();
	}
	else{
		angle = AmPan::ComputeAzi(viewp_from_presenter, sound_loc_from_presenter);
	}

	angle = ConvertAngle(angle);


}

PanITD::~PanITD()
{
	if(pan){
		delete pan;
	}
}

void
PanITD::Change(float viewp_from_presenter[4], 
			   Vec3f sound_loc_from_presenter)
{
	if(pan){
		pan->ChangeGains(viewp_from_presenter, 
			sound_loc_from_presenter);
		angle = pan->GetAngle();
	}
	else{
		angle = AmPan::ComputeAzi(viewp_from_presenter, 
			sound_loc_from_presenter);
	}
	angle = ConvertAngle(angle);

}

float*
PanITD::operator()(float input)
{
	float* tmp;
	if(pan){
		tmp = (*pan)(input);
		out[0] = tmp[0];
		out[1] = tmp[1];
	}
	else{
		out[0] = out[1] = input;
	}


   return (float*)(out);
}

// This is what has been sent to im1
// Note that in amplitude panning functions the sound source direction is not actually needed
AmPan::AmPan(float viewp_from_presenter[4], 
			 Vec3f sound_loc_from_presenter)
{
	azi_angle = ComputeAzi(viewp_from_presenter, 
		sound_loc_from_presenter);
	gain_left = 0.0;
	gain_right = 0.0;
	out[0]=out[1]=0.0;
	ChangeGains(viewp_from_presenter, 
		sound_loc_from_presenter);
	
}

AmPan::~AmPan(void)
{
}

float AmPan::ComputeAzi(float viewp_from_presenter[4], 
						Vec3f sound_loc_from_presenter)
{
	float v_x = viewp_from_presenter[0];
	float v_z = viewp_from_presenter[2];
	float v_theta = viewp_from_presenter[3];
	float s_x = sound_loc_from_presenter.x;
	float s_z = sound_loc_from_presenter.z;
	float azi = 0.0;
	float theta = v_theta;	

	float x = v_x-s_x; //relative viewpoint (x and z coordinates)
	float z = v_z-s_z;

	if((fabs(z)) < 0.0001) //viewpoint on the x-axis => angle + or -pi/2
		z = 0.0001;

	if(fabs(x) < 0.0001)
		x = 0.0001;

	if(x*z < 0){ //theta in the second or fourth quarter 
		if(x < 0)
			azi = (-atanf(x/z)) + theta; //
		else
			azi = (-atanf(x/z) - PI) + theta;
	}

	if(x*z >= 0){
		if(x >= 0)
			azi = (-atanf(x/z)) + theta;
		else
			azi = (-atanf(x/z) + PI) + theta;	
	}

	return azi;
}


void AmPan::ChangeGains(float viewp_from_presenter[4], Vec3f sound_loc_from_presenter)
{	
	azi_angle = ComputeAzi(viewp_from_presenter, sound_loc_from_presenter);
	gain_right = (1+(sin(azi_angle)/2))*(1 + cos(azi_angle)/2)/1.5; 
	gain_left = (1-(sin(azi_angle)/2))*(1 + cos(azi_angle)/2)/1.5;
//	COMPLAIN("gains l: %f r: %f",gain_left,gain_right);
}

float AmPan::GetAngle()
{
	return azi_angle;
}
float* AmPan::operator()(float input)
{
	out[0] = /*(EXTRA/4)**/gain_left*input;
	out[1] = /*(EXTRA/4)**/gain_right*input;
 	return ((float*)out);
}

float ConvertAngle(float angle)
{
	float a = 0.0;
	if(angle >= ((float)(2*PI)))
	{
		a = angle - (float)floor((float)(angle/(2*PI)));
	}
	else if(angle <= ((float)(-2*PI)))
	{
		a = (float)ceil((float)(angle/(2*PI))) -angle;
	}
	else {a = angle;}

	if(a>(float)(PI))
	{
		a = a-(float)(2*PI);
	}
	if(a<=(float)(-PI)){
		a = a+(float)(2*PI);
	}
	return a;
}


void CombineOrientations(float x1,float y1,float z1,float x2,float y2,float z2, Vec3f* result)
{
	float rot1[4];
	float rot2[4];
	float sum[4];
	int i=0;
	for(i=0;i<4;i++){
		rot1[i] = 0.0;
		rot2[i] = 0.0;
		sum[i] = 0.0;
	}
	XYZ_To_Rotation(x1,y1,z1,rot1);
	XYZ_To_Rotation(x2,y2,z2,rot2);

	for(i=0;i<2;i++){
		sum[i] = rot1[i]+rot2[i]; //sum the elevation and azimuth angles
	}
	//convert back to xyz-coordinates (from spherical)
	result->x = cosf(sum[0])*sinf(sum[1]);	//x
	result->y = -sinf(sum[0]);
	result->z =	cosf(sum[0])*cosf(sum[1]);
}
void XYZ_To_Rotation(float x,float y,float z, float* rot)
{
	rot[2] = 0.0;
	rot[3] = 1;

	if(((x < 0.0001)&&(x > -0.0001)) && ((z < 0.0001)&&(z > -0.0001))){
		// x and z = 0 -> only y-component
		if(y >= 0)
			rot[0] =  (float) (-PI/2); //negative rotation of x axis 
		if(y<0)
			rot[0] = (float) (PI/2); // positive rotation of x axis

		rot[1] = 0.0;
		return;
	}
	else
		rot[0] = -atanf(y/(sqrt(pow(x,2)+pow(z,2)))); // elevation angle 
													  // (negative) rotation around x axis
	//azimuth angle (rotation around y-axis):
	if(((z < 0.0001)&&(z > -0.0001)) && ((x >= 0.0001)&&(x <= -0.0001))){
		// z=0 and x != 0 -> vector in xy plane = alpha_y = + or -90 deg.
		if(x>0)
			rot[1] = (float) (PI/2);
		if(x<0)
			rot[1] = (float) (-PI/2);

	}
	else {
		rot[1] = atanf(x/z);
		if((x<=0) && (z<0)) 
			rot[1] -= (float) PI;
			
		if((x>0) && (z<0))
			rot[1] += (float) PI; 
	}
}
