#ifndef __SPATIALIZE_H__
#define __SPATIALIZE_H__
/***********************************************************************
This software module was originally developed by Riitta Vaananen
(Helsinki University of Technology) in the course of development of
the MPEG-4 Systems (ISO/IEC 14496-1) standard.  This software module
is an implementation of a part of one or more MPEG-4 Systems (ISO/IEC
14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1)
standard. ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1)
free license to this software module or modifications thereof for use
in hardware or software products claiming conformance to the MPEG-4
Systems (ISO/IEC 14496-1). Those intending to use this software module
in hardware or software products are advised that its use may infringe
existing patents.  The original developer of this software module and
his/her company, the subsequent editors and their companies, and
ISO/IEC have no liability for use of this software module or
modifications thereof in an implementation.  Copyright is not released
for non MPEG-4 Systems (ISO/IEC 14496- 1) conforming products.
Helsinki University of Technology retains full right to use the code
for their own purpose, assign or donate the code to a third party and
to inhibit third parties from using the code for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. This copyright notice must be
included in all copies or derivative works. Copyright  1999.
************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <fstream.h>
#include <math.h>
#include <string.h>
#include "Include/defs.h"
#include "core/NodeFld.h"

#ifndef PI
#define PI 3.14159265358979323846
#endif
#define MAX_DIST_METERS ((float) 500.0)
#define MAX_NUM_PARAM ((int) 11)

class AmPan;
float ConvertAngle(float angle);
void XYZ_To_Rotation(float x,float y,float z, float* rot);
void CombineOrientations(float x1,float y1,float z1,float x2,float y2,float z2,Vec3f* result);


/** class for controling both the ITD and amplitude panning */
class PanITD
{
public:
	/** Constructor.
	
	Sets up the Amplitude and time difference panning objects, and 
	the relative angle of the viewpoint according to the arrival 
	of sound. */
	PanITD(float viewp_from_presenter[4], 
		Vec3f sound_loc_from_presenter, 
		int sf, 
		BOOL t, 
		BOOL l, 
		int update);
	/** Destructor.

	AmPan and ITD objects are deleted.*/
	~PanITD();

	/** Updates the angle, which is then used for updating the 
	amplitude and time difference panning functions. */
	void Change(float viewp_from_presenter[4], 
		Vec3f sound_loc_from_presenter);

	/// Calls AmPan and ITD operators at every input sample
	float* operator()(float input);

private:
	int update_period;
	float angle;
	float out[2];

	/// pointer to the amplitude panning object
	AmPan* pan;

};

/** Class for amplitude panning.

uses a cardioid like receiver for panning the sound. I.e., the sounds 
arriving from the back are attenuated more than those from the front.
*/
class AmPan
{
public:
	/** Constructor.
	
	Sets up the left and the right channel gains.
	*/
	AmPan(float viewp_from_presenter[4], 
		Vec3f sound_loc_from_presenter);

	/// Destructor
	~AmPan();

	/// Computes the relative azimuth angle 
	static float ComputeAzi(float viewp_from_presenter[4], 
		Vec3f sound_loc_from_presenter);

	/// Updates the left and right gains
 	void ChangeGains(float viewp_from_presenter[4], 
		Vec3f sound_loc_from_presenter);

	/// Returns the current azimuth angle
	float GetAngle();

	/// Processes the input sound sample and returns an amplitude panned stereo sample
	float* operator()(float input);

	
private:
	float azi_angle;
	float out[2];
	float gain_left;
	float gain_right;

};

#endif
