// This software module was originally developed by Marc Emerit and 
// Jean-Bernard Rault (France Telecom) , in the course of
// development of the MPEG-4 Systems standard (ISO/IEC 14496-1).  This
// software module is an implementation of a part of one or more MPEG-4
// Systems tools as specified by the MPEG-4 Systems standard.  ISO/IEC
// gives users of the MPEG-4 Systems standard free license to this software
// module or modifications thereof for use in hardware or software products
// claiming conformance to the MPEG-4 Systems standard.  Those intending to
// use this software module in hardware or software products are advised
// that its use may infringe existing patents.  The original developer of
// this software module and his/her company, the subsequent editors and
// their companies, and ISO/IEC have no liability for use of this software
// module or modifications thereof in an implementation.  Copyright is not
// released for non MPEG-4 Systems conforming products.  The original
// developer retains full right to use the code for his/her own purpose,
// assign or donate the code to a third party and to inhibit third parties
// from using the code for non MPEG-4 Systems conforming products.  This
// copyright notice must be included in all copies or derivative works.
//
// For more information or to receive an up to date version of this module,
// mail karlo@nta.no or check <http://televr.fou.telenor.no/compositor/>.
//
// Copyright 1998.




// Cluster.cpp: implementation of the CCluster class.
//
//////////////////////////////////////////////////////////////////////
#include <stdlib.h>
#include <time.h>
#include <math.h>
#include "Cluster.h"
// SOLARIS PORTING
#ifndef WIN32
#include "Include/time_def.h"
#endif
// SOLARIS PORTING - END

CCluster::CCluster()
{	
    // PORTING to CC 5.3
    int i = 0;
//	for(int i=0;i<NBCHANNELS;i++)
	for( i=0;i<NBCHANNELS;i++)
		for(int j=0;j<MEMORYSIZE;j++)
			m_delayLine[i][j]=0.f;

	m_outputGain=(float)(1./sqrt(NBCHANNELS));
	for (i=0;i<NBCHANNELS;i++)
		m_indexWrite[i]=0;

// SOLARIS PORTING
#ifdef WIN32
	m_seed=unsigned int(time(NULL));
#else
	m_seed = timeGetTime();
#endif
// SOLARIS PORTING - END
	SetDelays(0.0400f,0.06f,32000.f);
}


CCluster::~CCluster()
{
}


void CCluster::SetDelays(float minDelay,float maxDelay,float sf)
{	
// maxDelay and mindelay are in s
// samplingRate is in Hz, eg. 18000, 44100, ...
	
	int offset;

	ComputeDelays(m_delay,minDelay,maxDelay,NBCHANNELS,m_seed);

	for(int i=0;i<NBCHANNELS;i++)
	{	offset=(int)(m_delay[i]*sf);
	    m_indexRead[i]=((m_indexWrite[i]-offset>=0)?(m_indexWrite[i]-offset):(MEMORYSIZE+m_indexWrite[i]-offset));
	}
	return ;
}


void CCluster::SetOutputGain(float gain)
{
	m_outputGain=gain*(float)(1./sqrt(NBCHANNELS));
	return;
}



void CCluster::DoFiltering(float in[NBCHANNELS][MAXNUMBEROFSAMPLE], float out1[4][MAXNUMBEROFSAMPLE],float out2[NBCHANNELS][MAXNUMBEROFSAMPLE],int SizeBuff)
{

	Hadamard8(&in[0][0],&in[1][0],&in[2][0],&in[3][0],&in[4][0],&in[5][0],&in[6][0],&in[7][0],SizeBuff);

    // PORTING to CC 5.3
    int i = 0;
//	for(int i=0;i<NBCHANNELS;i++)
	for( i=0;i<NBCHANNELS;i++)
	{	WriteMemory(&in[i][0],&m_delayLine[i][0],&m_indexWrite[i],SizeBuff,MEMORYSIZE);
		ReadMemory(&m_delayLine[i][0],&out2[i][0],&m_indexRead[i],SizeBuff,MEMORYSIZE);
	}

#ifdef IMPULSE			// to calculate the impulse response
	m_outputGain=(float)(1./sqrt(NBCHANNELS));
#endif

	for(i=0;i<SizeBuff;i++)
	{	out1[0][i]=(out2[0][i]+out2[4][i])*m_outputGain;
		out1[1][i]=(out2[1][i]+out2[5][i])*m_outputGain;
		out1[2][i]=(out2[2][i]+out2[6][i])*m_outputGain;
		out1[3][i]=(out2[3][i]+out2[7][i])*m_outputGain;
	}

}
