// This software module was originally developed by Marc Emerit and 
// Jean-Bernard Rault (France Telecom) , in the course of
// development of the MPEG-4 Systems standard (ISO/IEC 14496-1).  This
// software module is an implementation of a part of one or more MPEG-4
// Systems tools as specified by the MPEG-4 Systems standard.  ISO/IEC
// gives users of the MPEG-4 Systems standard free license to this software
// module or modifications thereof for use in hardware or software products
// claiming conformance to the MPEG-4 Systems standard.  Those intending to
// use this software module in hardware or software products are advised
// that its use may infringe existing patents.  The original developer of
// this software module and his/her company, the subsequent editors and
// their companies, and ISO/IEC have no liability for use of this software
// module or modifications thereof in an implementation.  Copyright is not
// released for non MPEG-4 Systems conforming products.  The original
// developer retains full right to use the code for his/her own purpose,
// assign or donate the code to a third party and to inhibit third parties
// from using the code for non MPEG-4 Systems conforming products.  This
// copyright notice must be included in all copies or derivative works.
//
// For more information or to receive an up to date version of this module,
// mail karlo@nta.no or check <http://televr.fou.telenor.no/compositor/>.
//
// Copyright 1998.


// Direct.cpp: implementation of the CDirect class.
//
//////////////////////////////////////////////////////////////////////

#include "Direct.h"
#include <math.h>

CDirect::CDirect()
{
	m_FreqLow	= (float) (1.f/32.);
	m_FreqHigh	= (float) (1.f/8.);

	m_gainEQ = 1.f;
	m_b0 = 1.f;
	m_b1 = 0.f;
	m_b2 = 0.f;
	m_a1 = 0.f;
	m_a2 = 0.f;
	m_pos = 0;
	m_memb[0] = 0.0;
	m_memb[1] = 0.0;
	m_mema[0] = 0.0;
	m_mema[1] = 0.0;

}


CDirect::~CDirect()
{
}

void CDirect::SetDirectEqualizer(float FreqLow,float FreqHigh,float GainLow,float GainMid,float GainHigh)
{
	float coef[5];

	m_FreqLow = FreqLow;
	m_FreqHigh = FreqHigh;

	UpdateEQ(GainLow,GainMid,GainHigh,FreqLow,FreqHigh,coef);

	m_b0=coef[0];
	m_b1=coef[1];
	m_b2=coef[2];

	m_a1=coef[3];
	m_a2=coef[4];

	m_gainEQ=GainMid;

}

void CDirect::DoFiltering(float *inBuff, float *outBuff,int SizeBuff)
{
	
	float S;
	
	// equalization 
	

#ifdef IMPULSE				// to calculate the impulse response
	m_gainEQ=1.0;
#endif

	for(int i=0;i<SizeBuff;i++)
	{		S=inBuff[i]*m_b0;
      	    S+=m_b1*m_memb[m_pos];
       	    S-=m_a1*m_mema[m_pos++];
            m_pos&=0x1;
      	    S+=m_b2*m_memb[m_pos];
       	    S-=m_a2*m_mema[m_pos++];
            m_pos&=0x1;

            if(fabs(S)<0.000000001f)				// to avoid underflow 
				S=0.0;

            m_pos++;
            m_pos&=0x1;

            m_memb[m_pos]=inBuff[i];
            m_mema[m_pos]=S;
            outBuff[i] =S*m_gainEQ;
   	}


}
