// This software module was originally developed by Marc Emerit and 
// Jean-Bernard Rault (France Telecom) , in the course of
// development of the MPEG-4 Systems standard (ISO/IEC 14496-1).  This
// software module is an implementation of a part of one or more MPEG-4
// Systems tools as specified by the MPEG-4 Systems standard.  ISO/IEC
// gives users of the MPEG-4 Systems standard free license to this software
// module or modifications thereof for use in hardware or software products
// claiming conformance to the MPEG-4 Systems standard.  Those intending to
// use this software module in hardware or software products are advised
// that its use may infringe existing patents.  The original developer of
// this software module and his/her company, the subsequent editors and
// their companies, and ISO/IEC have no liability for use of this software
// module or modifications thereof in an implementation.  Copyright is not
// released for non MPEG-4 Systems conforming products.  The original
// developer retains full right to use the code for his/her own purpose,
// assign or donate the code to a third party and to inhibit third parties
// from using the code for non MPEG-4 Systems conforming products.  This
// copyright notice must be included in all copies or derivative works.
//
// For more information or to receive an up to date version of this module,
// mail karlo@nta.no or check <http://televr.fou.telenor.no/compositor/>.
//
// Copyright 1998.



// Early.cpp: implementation of the CEarly class.
//
//////////////////////////////////////////////////////////////////////
#include <stdlib.h>
#include "Early.h"
#include <time.h>
#include <memory.h>
#include <math.h>

// SOLARIS PORTING
#ifndef WIN32
#include "Include/time_def.h"
#endif
// SOLARIS PORTING - END


CEarly::CEarly()
{	
	m_indexWrite=0;

	for(int j=0;j<MEMORYSIZE;j++)
			m_delayLine[j]=0.f;

// SOLARIS PORTING
#ifdef WIN32
	m_seed=unsigned int(time(NULL));
#else
	m_seed = timeGetTime();
#endif
// SOLARIS PORTING - END
	SetDelays(0.02f,0.04f,32000.f);

	m_FreqLow = (float) (1.f/32.);
	m_FreqHigh = (float) (1.f/8.);

	for(int i=0;i<2;i++)
	{   m_gainEQ[i] = 1.f;
		m_b0[i] = 1.f;
		m_b1[i] = 0.f;
		m_b2[i] = 0.f;
		m_a1[i] = 0.f;
		m_a2[i] = 0.f;
		m_pos[i] = 0;
		m_memb[i][0] = 0.0;
		m_memb[i][1] = 0.0;
		m_mema[i][0] = 0.0;
		m_mema[i][1] = 0.0;
	}


}


CEarly::~CEarly()
{
}

void CEarly::SetEqualizer(float FreqLow,float FreqHigh,float GainLow,float GainMid,float GainHigh)
{

	float coef[5];

	m_FreqLow = FreqLow;
	m_FreqHigh = FreqHigh;

	for(int i=0;i<2;i++)
	{	UpdateEQ(GainLow,GainMid,GainHigh,FreqLow,FreqHigh,coef);

		m_b0[i]=coef[0];
		m_b1[i]=coef[1];
		m_b2[i]=coef[2];

		m_a1[i]=coef[3];
		m_a2[i]=coef[4];

		m_gainEQ[i]=GainMid;
	}

}

void CEarly::SetDelays(float minDelay,float maxDelay,float sf)
{	
	int offset;

	ComputeDelays(m_delay,minDelay,maxDelay,NBCHANNELS,m_seed);

	for(int i=0;i<NBCHANNELS;i++)
	{	offset=(int)(m_delay[i]*sf);
	    m_indexRead[i]=((m_indexWrite-offset>=0)?(m_indexWrite-offset):(MEMORYSIZE+m_indexWrite-offset));
	}
	return ;

}


void CEarly::DoFiltering(float *inBuff, float out[2][MAXNUMBEROFSAMPLE],float temp[NBCHANNELS][MAXNUMBEROFSAMPLE],int SizeBuff)
{
	
	WriteMemory(inBuff,m_delayLine,&m_indexWrite,SizeBuff,MEMORYSIZE);
    // PORTING to CC 5.3
    int i = 0;
//	for(int i=0;i<NBCHANNELS;i++)
	for( i=0;i<NBCHANNELS;i++)
		ReadMemory(m_delayLine,&temp[i][0],&m_indexRead[i],SizeBuff,MEMORYSIZE);
	

	if(NBCHANNELS==8)
	{
		for(i=0;i<SizeBuff;i++)
		{	out[0][i]=(temp[0][i]+temp[2][i]+temp[4][i]+temp[6][i])*(float)(1./sqrt(NBCHANNELS));  
			out[1][i]=(temp[1][i]+temp[3][i]+temp[5][i]+temp[7][i])*(float)(1./sqrt(NBCHANNELS));
		}
	}


	// equalization 

	float S;	
	
#ifdef IMPULSE				// to calculate the impulse response
	m_gainEQ[0]=1.0;
	m_gainEQ[1]=1.0;
#endif

	for(int ch=0;ch<2;ch++)
		for(i=0;i<SizeBuff;i++)
		{	S =  m_b0[ch]*out[ch][i];
			S += m_b1[ch]*m_memb[ch][m_pos[ch]];
			S -= m_a1[ch]*m_mema[ch][m_pos[ch]++];
			m_pos[ch] &= 0x1;
			S += m_b2[ch]*m_memb[ch][m_pos[ch]];
			S -= m_a2[ch]*m_mema[ch][m_pos[ch]++];
			m_pos[ch] &= 0x1;

			if( fabs(S)<0.000000001f)				// to avoid underflow
				S=0.f;

			m_pos[ch]++; 
			m_pos[ch]&=0x1; 

			m_memb[ch][m_pos[ch]] = out[ch][i];
			m_mema[ch][m_pos[ch]] = S;
			out[ch][i] =S*m_gainEQ[ch];
		}
	return;
}

