// This software module was originally developed by Marc Emerit and 
// Jean-Bernard Rault (France Telecom) , in the course of
// development of the MPEG-4 Systems standard (ISO/IEC 14496-1).  This
// software module is an implementation of a part of one or more MPEG-4
// Systems tools as specified by the MPEG-4 Systems standard.  ISO/IEC
// gives users of the MPEG-4 Systems standard free license to this software
// module or modifications thereof for use in hardware or software products
// claiming conformance to the MPEG-4 Systems standard.  Those intending to
// use this software module in hardware or software products are advised
// that its use may infringe existing patents.  The original developer of
// this software module and his/her company, the subsequent editors and
// their companies, and ISO/IEC have no liability for use of this software
// module or modifications thereof in an implementation.  Copyright is not
// released for non MPEG-4 Systems conforming products.  The original
// developer retains full right to use the code for his/her own purpose,
// assign or donate the code to a third party and to inhibit third parties
// from using the code for non MPEG-4 Systems conforming products.  This
// copyright notice must be included in all copies or derivative works.
//
// For more information or to receive an up to date version of this module,
// mail karlo@nta.no or check <http://televr.fou.telenor.no/compositor/>.
//
// Copyright 1998.


// Early.h: interface for the CEarly class.
//
//////////////////////////////////////////////////////////////////////


#ifndef _Early_H
#define _Early_H

#include "const.h"


extern "C" void UpdateEQ(float GainLow, float GainMid, float GainHigh, float FreqLow, float FreqHigh, float coef[5]);
extern "C" void ComputeDelays(float* tabDelay,float minDelay,float maxDelay,int nb,unsigned int seed);
extern "C" void Hadamard8(float *inPt1,float *inPt2,
					      float *inPt3,float *inPt4,
					      float *inPt5,float *inPt6,
					      float *inPt7,float *inPt8,int SizeBuff);

extern "C"  WriteMemory(float* org,float* dest,int* index, int nb, int size);

extern "C"  void ReadMemory(float *org,float *dest, int* index, int nb, int size);



class CEarly
{
public:
	CEarly();
	virtual ~CEarly();
	void SetEqualizer(float FreqLow,float FreqHigh,float GainLow,float GainMid,float GainHigh);
	void SetDelays(float minDelay,float maxDelay,float sf);
	void DoFiltering(float *inBuff, float out[2][MAXNUMBEROFSAMPLE],float temp[8][MAXNUMBEROFSAMPLE],int SizeBuff);

private :
	void ComputeLR(float* outLeft,float* outRight, float in[8][MAXNUMBEROFSAMPLE],int nb);


	// one delay line 

	float	m_delayLine[MEMORYSIZE];
	int		m_indexRead[NBCHANNELS];
	int		m_indexWrite;
	float	m_delay[NBCHANNELS];
	unsigned int m_seed;


	// 2 equalizers, one for the left channel and one for the right channel:

	float m_b0[2];
	float m_b1[2];
	float m_b2[2];
	float m_a1[2];
	float m_a2[2];

	float m_mema[2][2];
	float m_memb[2][2];

	float m_gainEQ[2];

	int	  m_pos[2];

	float m_FreqLow  ;
	float m_FreqHigh ;


};

#endif
