// This software module was originally developed by Marc Emerit and 
// Jean-Bernard Rault (France Telecom) , in the course of
// development of the MPEG-4 Systems standard (ISO/IEC 14496-1).  This
// software module is an implementation of a part of one or more MPEG-4
// Systems tools as specified by the MPEG-4 Systems standard.  ISO/IEC
// gives users of the MPEG-4 Systems standard free license to this software
// module or modifications thereof for use in hardware or software products
// claiming conformance to the MPEG-4 Systems standard.  Those intending to
// use this software module in hardware or software products are advised
// that its use may infringe existing patents.  The original developer of
// this software module and his/her company, the subsequent editors and
// their companies, and ISO/IEC have no liability for use of this software
// module or modifications thereof in an implementation.  Copyright is not
// released for non MPEG-4 Systems conforming products.  The original
// developer retains full right to use the code for his/her own purpose,
// assign or donate the code to a third party and to inhibit third parties
// from using the code for non MPEG-4 Systems conforming products.  This
// copyright notice must be included in all copies or derivative works.
//
// For more information or to receive an up to date version of this module,
// mail karlo@nta.no or check <http://televr.fou.telenor.no/compositor/>.
//
// Copyright 1998.


// Reverb.cpp: implementation of the CReverb class.
//
//////////////////////////////////////////////////////////////////////
#include <stdlib.h>
#include <math.h>
#include "Reverberation.h"
#include <time.h>
#include <memory.h>
// SOLARIS PORTING
#ifndef WIN32
#include "Include/time_def.h"
#endif
// SOLARIS PORTING - END


CReverb::CReverb()
{	

    // PORTING to CC 5.3
    int i = 0;
//	for(int i=0;i<NBCHANNELS;i++)
	for( i=0;i<NBCHANNELS;i++)
		for(int j=0;j<MEMORYSIZE;j++)
			m_delayLine[i][j]=0.f;

		m_indexWrite=0;

	for(i=0;i<NBCHANNELS;i++)
			m_storage[i]=0.f;

// SOLARIS PORTING
#ifdef WIN32
	m_seed=unsigned int(time(NULL));
#else
	m_seed = timeGetTime();
#endif
// SOLARIS PORTING - END
	SetDelays(0.0529f,0.8f,32000.f);

	m_gain=0.f;
	m_outputGain=0.f;

	m_FreqLow = (float) (1.f/32.);
	m_FreqHigh = (float) (1.f/8.);

	for(i=0;i<NBCHANNELS;i++)
	{   m_gainEQ[i] = 1.f;
		m_b0[i] = 1.f;
		m_b1[i] = 0.f;
		m_b2[i] = 0.f;
		m_a1[i] = 0.f;
		m_a2[i] = 0.f;
		m_pos[i] = 0;
		m_memb[i][0] = 0.0;
		m_memb[i][1] = 0.0;
		m_mema[i][0] = 0.0;
		m_mema[i][1] = 0.0;
	}
}




CReverb::~CReverb()
{
}


void CReverb::SetEqualizer(float FreqLow,float FreqHigh,float TimeLow,float TimeMid,float TimeHigh,float sf)
{	
	float GainLow,GainMid,GainHigh;
	float coef[5];
	
	m_gain=0.f;

	m_FreqLow = FreqLow;
	m_FreqHigh = FreqHigh;

	for(int i=0;i<NBCHANNELS;i++)
	{	

		float attDb=-60.f*m_delay[i]/TimeLow;
		GainLow=(float)pow(10.f,attDb/20.f);

		attDb=-60.f*m_delay[i]/TimeMid;
		GainMid=(float)pow(10.f,attDb/20.f);

		attDb=-60.f*m_delay[i]/TimeHigh;
		GainHigh=(float)pow(10.f,attDb/20.f);

		UpdateEQ(GainLow,GainMid,GainHigh,FreqLow,FreqHigh,coef);

		m_b0[i]=coef[0];
		m_b1[i]=coef[1];
		m_b2[i]=coef[2];

		m_a1[i]=coef[3];
		m_a2[i]=coef[4];

		m_gainEQ[i]=GainMid;

		m_gain+=(GainMid*GainMid);

	}

	m_gain/=NBCHANNELS;						// average gain of the loop
	m_gain=m_gain/(1.f-m_gain);
	m_gain=(float)sqrt(1.f/m_gain);			// gain of the FDN to be compensated for

}


void CReverb::SetOutputGain(float R3)
{	m_outputGain=R3*(float)(1./sqrt(NBCHANNELS));	
}


void CReverb::SetDelays(float minDelay,float density,float sf)
{	
// density and mindelay are in s
// samplingRate is in Hz, eg. 18000, 44100, ...

// the modal density is also the total delay in the reverb. If n delays are supposed to be uniformally
// and randomly distributed between minDelay and maxDelay (n-1 intervals) then the total delay is  
// approximately given by n/2*(maxDelay+minDelay)
	
	
	float maxDelay;
	int offset;

	maxDelay=(2.f*density/(float)NBCHANNELS)-minDelay;

	ComputeDelays(m_delay,minDelay,maxDelay,NBCHANNELS,m_seed);

	for(int i=0;i<NBCHANNELS;i++)
	{	offset=(int)(m_delay[i]*sf);
	    m_indexRead[i]=((m_indexWrite-offset>=0)?(m_indexWrite-offset):(MEMORYSIZE+m_indexWrite-offset));
	}
	return ;
}



void CReverb::DoFiltering(float in[NBCHANNELS][MAXNUMBEROFSAMPLE], float out[4][MAXNUMBEROFSAMPLE],int SizeBuff)
{
	int i,ind;

	if(NBCHANNELS==8)
	{	
		for(ind=0;ind<SizeBuff;ind++)
		{
			for(i=0;i<NBCHANNELS;i++)
				m_storage[i]=m_storage[i]+in[i][ind]*m_gain;

			hadamard8(&m_storage[0],&m_storage[1],&m_storage[2],&m_storage[3],
					&m_storage[4],&m_storage[5],&m_storage[6],&m_storage[7]);
		
			for(int i=0;i<NBCHANNELS;i++)
			{	
				// write in delay lines	
				m_delayLine[i][m_indexWrite]=m_storage[i];

				// read delay lines with delay

				m_storage[i]=m_delayLine[i][m_indexRead[i]];
				m_indexRead[i]++;
				if(m_indexRead[i]>=MEMORYSIZE)
					m_indexRead[i]=0;
			}
			m_indexWrite++;
			if(m_indexWrite>=MEMORYSIZE)
				m_indexWrite=0;

			// equalization 

			float S;	
	
			for(i=0;i<NBCHANNELS;i++)
			{	S=m_storage[i]*m_b0[i];
      			S+=m_b1[i]*m_memb[i][m_pos[i]];
       			S-=m_a1[i]*m_mema[i][m_pos[i]++];
				m_pos[i]&=0x1;
      			S+=m_b2[i]*m_memb[i][m_pos[i]];
       			S-=m_a2[i]*m_mema[i][m_pos[i]++];
				m_pos[i]&=0x1;

				m_pos[i]++;
				m_pos[i]&=0x1;

				if(fabs(S)<0.000000001f)				// to avoid underflow 
					S=0.0;

				m_memb[i][m_pos[i]]=m_storage[i];
				m_mema[i][m_pos[i]]=S;
				m_storage[i] =S*m_gainEQ[i];
   			}
		

#ifdef IMPULSE				// to calculate the impulse response
			m_outputGain=(float)(1./sqrt(NBCHANNELS));
#endif

			out[0][ind]=(m_storage[0]+m_storage[4])*m_outputGain;
			out[1][ind]=(m_storage[1]+m_storage[5])*m_outputGain;
			out[2][ind]=(m_storage[2]+m_storage[6])*m_outputGain;
			out[3][ind]=(m_storage[3]+m_storage[7])*m_outputGain;

		}  // ind loop
	}
	else if(NBCHANNELS==4)
	{
		// to be done !!!!!!!
	}



}

