// This software module was originally developed by Marc Emerit and 
// Jean-Bernard Rault (France Telecom) , in the course of
// development of the MPEG-4 Systems standard (ISO/IEC 14496-1).  This
// software module is an implementation of a part of one or more MPEG-4
// Systems tools as specified by the MPEG-4 Systems standard.  ISO/IEC
// gives users of the MPEG-4 Systems standard free license to this software
// module or modifications thereof for use in hardware or software products
// claiming conformance to the MPEG-4 Systems standard.  Those intending to
// use this software module in hardware or software products are advised
// that its use may infringe existing patents.  The original developer of
// this software module and his/her company, the subsequent editors and
// their companies, and ISO/IEC have no liability for use of this software
// module or modifications thereof in an implementation.  Copyright is not
// released for non MPEG-4 Systems conforming products.  The original
// developer retains full right to use the code for his/her own purpose,
// assign or donate the code to a third party and to inhibit third parties
// from using the code for non MPEG-4 Systems conforming products.  This
// copyright notice must be included in all copies or derivative works.
//
// For more information or to receive an up to date version of this module,
// mail karlo@nta.no or check <http://televr.fou.telenor.no/compositor/>.
//
// Copyright 1998.


// Reverb.h: interface for the CReverb class.
//
//////////////////////////////////////////////////////////////////////


#ifndef _Reverb_H
#define _Reverb_H

#include "const.h"


extern "C" void hadamard8(float *inPt1,float *inPt2,
					   float *inPt3,float *inPt4,
					   float *inPt5,float *inPt6,
					   float *inPt7,float *inPt8);


extern "C" void ComputeDelays(float* tabDelay,float minDelay,float maxDelay,int nb,unsigned int seed);

extern "C" void UpdateEQ(float GainLow, float GainMid, float GainHigh, float FreqLow, float FreqHigh, float coef[5]);


class CReverb
{
public:
	CReverb();
	virtual ~CReverb();
	void SetDelays(float minDelay,float maxDelay,float sf);
	void SetEqualizer(float FreqLow,float FreqHigh,float GainLow,float GainMid,float GainHigh,float sf);
	void DoFiltering(float in[NBCHANNELS][MAXNUMBEROFSAMPLE], float out[4][MAXNUMBEROFSAMPLE],int SizeBuff);
	void SetOutputGain(float R3);

private :

	float m_storage[NBCHANNELS];

	float m_gain;		// compensates for the recursive loop ;depends on the number of delay lines and their delays
	float m_outputGain; // depends on R3
	
	float m_gainloop[NBCHANNELS];


// NBCHANNELS delay lines :

	float m_delayLine[NBCHANNELS][MEMORYSIZE];
	int m_indexRead[NBCHANNELS];
	int m_indexWrite;
	float m_delay[NBCHANNELS];
	unsigned int m_seed;

// NBCHANNELS equalizers:

	float m_b0[NBCHANNELS];
	float m_b1[NBCHANNELS];
	float m_b2[NBCHANNELS];
	float m_a1[NBCHANNELS];
	float m_a2[NBCHANNELS];

	float m_mema[NBCHANNELS][2];
	float m_memb[NBCHANNELS][2];

	float m_gainEQ[NBCHANNELS];

	int	  m_pos[NBCHANNELS];

	float m_FreqLow  ;
	float m_FreqHigh ;

};

#endif
