// This software module was originally developed by Marc Emerit and 
// Jean-Bernard Rault (France Telecom) , in the course of
// development of the MPEG-4 Systems standard (ISO/IEC 14496-1).  This
// software module is an implementation of a part of one or more MPEG-4
// Systems tools as specified by the MPEG-4 Systems standard.  ISO/IEC
// gives users of the MPEG-4 Systems standard free license to this software
// module or modifications thereof for use in hardware or software products
// claiming conformance to the MPEG-4 Systems standard.  Those intending to
// use this software module in hardware or software products are advised
// that its use may infringe existing patents.  The original developer of
// this software module and his/her company, the subsequent editors and
// their companies, and ISO/IEC have no liability for use of this software
// module or modifications thereof in an implementation.  Copyright is not
// released for non MPEG-4 Systems conforming products.  The original
// developer retains full right to use the code for his/her own purpose,
// assign or donate the code to a third party and to inhibit third parties
// from using the code for non MPEG-4 Systems conforming products.  This
// copyright notice must be included in all copies or derivative works.
//
// For more information or to receive an up to date version of this module,
// mail karlo@nta.no or check <http://televr.fou.telenor.no/compositor/>.
//
// Copyright 1998.


// Spatializer.cpp: implementation of the CSpatializer class.
//
//////////////////////////////////////////////////////////////////////

#include "Include/defs.h"
#include <math.h>
#include <stdlib.h>
#include "Spatializer.h"
// SOLARIS PORTING
#ifdef WIN32
#include <io.h>
#endif
// SOLARIS PORTING - END

extern "C" void Hadamard4(float *inPt1,float *inPt2,
					   float *inPt3,float *inPt4,int SizeBuff);


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CSpatializer::CSpatializer()
{
	m_x=1.f;
	m_z=0.f;
	m_y=0.f;	

	m_sf=32000.f;
	m_f0=250.f;				// default values : 250 Hz
	m_f1=4000.f;			//				  : 4000 Hz

	m_l0=0.f;				// time limits of the synthetic room response
	m_l1=0.02f;				// expressed in s : default values
	m_l2=0.04f;
	m_l3=0.1f;
	m_modal=0.8f;			// modal density in s

	m_R0[0]=1.0f;		// direct path energy in lower band
	m_R0[1]=1.0f;		// direct path energy in mid band
	m_R0[2]=1.0f;		// direct path energy in higher band
	m_R1[0]=0.01f;		// early  energy in lower band
	m_R1[1]=0.01f;		// early  energy in mid band
	m_R1[2]=0.01f;		// early  energy in higher band
	m_R2=0.0;			// cluster gain
	m_R3=0.0;			// late diffuse field gain
	m_RT[0]=0.1f;	    // reverberation time in lower band
	m_RT[1]=0.1f;	    // reverberation time in mid band
	m_RT[2]=0.1f;	    // reverberation time in higher band

// input filter

	m_gainEQ = 1.f;		
	m_b0 = 1.f;
	m_b1 = 0.f;
	m_b2 = 0.f;
	m_a1 = 0.f;
	m_a2 = 0.f;
	m_pos = 0;
	m_memb[0] = 0.0;
	m_memb[1] = 0.0;
	m_mema[0] = 0.0;
	m_mema[1] = 0.0;

	m_pos=0;

	m_coefInputFilterLow=1.0;		// inpult filter gain in the three bands
	m_coefInputFilterMid=1.0;
	m_coefInputFilterHigh=1.0;

// room filter

	m_RgainEQ = 1.f;		
	m_Rb0 = 1.f;
	m_Rb1 = 0.f;
	m_Rb2 = 0.f;
	m_Ra1 = 0.f;
	m_Ra2 = 0.f;
	m_Rpos = 0;
	m_Rmemb[0] = 0.0;
	m_Rmemb[1] = 0.0;
	m_Rmema[0] = 0.0;
	m_Rmema[1] = 0.0;

	m_Rpos=0;

	m_coefRoomFilterLow=1.0;		// inpult filter gain in the three bands
	m_coefRoomFilterMid=1.0;
	m_coefRoomFilterHigh=1.0;

	SetDelays();
	SetEqualizers();

	m_bRoomEffect=TRUE;		// indicates whether room FX are to be applied
	m_bSpatialize=TRUE;		// indicates whether room FX are to be applied

}


CSpatializer::~CSpatializer()
{

}

void CSpatializer::init()
{

	float f0,f1;

	f0=m_f0/(m_sf/2.f);									// reduced frequency
	f1=m_f1/(m_sf/2.f);

	SetInputEqualizer();

	SetRoomEqualizer();

	m_direct.SetDirectEqualizer(f0,f1,m_R0[0],m_R0[1],m_R0[2]);

	m_early.SetDelays(m_l1,m_l2,m_sf);
	m_early.SetEqualizer(f0,f1,m_R1[0],m_R1[1],m_R1[2]);

	m_cluster.SetDelays(m_l2-m_l1,m_l3-m_l2,m_sf);
	m_cluster.SetOutputGain(m_R2);

	m_reverb.SetDelays(m_l3-m_l2,m_modal,m_sf);
	m_reverb.SetEqualizer(f0,f1,m_RT[0],m_RT[1],m_RT[2],m_sf);
	m_reverb.SetOutputGain(m_R3);

}

void CSpatializer::SetDelays()
{
	float f0,f1;

	f0=m_f0/(m_sf/2.f);									// reduced frequency
	f1=m_f1/(m_sf/2.f);

	m_early.SetDelays(m_l1,m_l2,m_sf);
	m_cluster.SetDelays(m_l2-m_l1,m_l3-m_l2,m_sf);
	m_reverb.SetDelays(m_l3-m_l2,m_modal,m_sf);
	m_reverb.SetEqualizer(f0,f1,m_RT[0],m_RT[1],m_RT[2],m_sf); // equalizers depends on delays
}

void CSpatializer::SetEqualizers()
{
	float f0,f1;

	f0=m_f0/(m_sf/2.f);									// reduced frequency
	f1=m_f1/(m_sf/2.f);

	m_direct.SetDirectEqualizer(f0,f1,m_R0[0],m_R0[1],m_R0[2]);
	m_early.SetEqualizer(f0,f1,m_R1[0],m_R1[1],m_R1[2]);
	m_cluster.SetOutputGain(m_R2);
	m_reverb.SetEqualizer(f0,f1,m_RT[0],m_RT[1],m_RT[2],m_sf);
	m_reverb.SetOutputGain(m_R3);
}


void CSpatializer::SetInputFilterCoef(float GainLow,float GainMid,float GainHigh)
{
	m_coefInputFilterLow=GainLow;
	m_coefInputFilterMid=GainMid;
	m_coefInputFilterHigh=GainHigh;

	SetInputEqualizer();

}

void CSpatializer::GetInputFilterCoef(float* GainLow,float* GainMid,float* GainHigh)
{
	*GainLow=m_coefInputFilterLow;
	*GainMid=m_coefInputFilterMid;
	*GainHigh=m_coefInputFilterHigh;
}

void CSpatializer::SetInputEqualizer()
{

	float coef[5];
	float f0,f1;

	f0=m_f0/(m_sf/2.f);									// reduced frequency
	f1=m_f1/(m_sf/2.f);

	UpdateEQ(m_coefInputFilterLow,m_coefInputFilterMid,m_coefInputFilterHigh,f0,f1,coef);

	m_b0=coef[0];
	m_b1=coef[1];
	m_b2=coef[2];

	m_a1=coef[3];
	m_a2=coef[4];

	m_gainEQ=m_coefInputFilterMid;
}


void CSpatializer::SetRoomFilterCoef(float GainLow,float GainMid,float GainHigh)
{
	m_coefRoomFilterLow=GainLow;
	m_coefRoomFilterMid=GainMid;
	m_coefRoomFilterHigh=GainHigh;

	SetRoomEqualizer();

}

void CSpatializer::GetRoomFilterCoef(float* GainLow,float* GainMid,float* GainHigh)
{
	*GainLow=m_coefRoomFilterLow;
	*GainMid=m_coefRoomFilterMid;
	*GainHigh=m_coefRoomFilterHigh;
}

void CSpatializer::SetRoomEqualizer()
{

	float coef[5];
	float f0,f1;

	f0=m_f0/(m_sf/2.f);									// reduced frequency
	f1=m_f1/(m_sf/2.f);

	UpdateEQ(m_coefRoomFilterLow,m_coefRoomFilterMid,m_coefRoomFilterHigh,f0,f1,coef);

	m_Rb0=coef[0];
	m_Rb1=coef[1];
	m_Rb2=coef[2];

	m_Ra1=coef[3];
	m_Ra2=coef[4];

	m_gainEQ=m_coefRoomFilterMid;
}



void CSpatializer::InputFiltering(float *inBuff,int SizeBuff)
{
	
	float S;
	float* outBuff=inBuff;


	// equalization 
	

#ifdef IMPULSE				// to calculate the impulse response
	m_gainEQ=1.0;
#endif

	for(int i=0;i<SizeBuff;i++)
	{		S=inBuff[i]*m_b0;
      	    S+=m_b1*m_memb[m_pos];
       	    S-=m_a1*m_mema[m_pos++];
            m_pos&=0x1;
      	    S+=m_b2*m_memb[m_pos];
       	    S-=m_a2*m_mema[m_pos++];
            m_pos&=0x1;

            if(fabs(S)<0.000000001f)				// to avoid underflow 
				S=0.0;

            m_pos++;
            m_pos&=0x1;

            m_memb[m_pos]=inBuff[i];
            m_mema[m_pos]=S;
            outBuff[i] =S*m_gainEQ;
   	}


}

void CSpatializer::RoomFiltering(float *inBuff,int SizeBuff)
{
	
	float S;
	float* outBuff=inBuff;


	// equalization 
	

#ifdef IMPULSE				// to calculate the impulse response
	m_gainEQ=1.0;
#endif

	for(int i=0;i<SizeBuff;i++)
	{		S=inBuff[i]*m_Rb0;
      	    S+=m_Rb1*m_Rmemb[m_Rpos];
       	    S-=m_Ra1*m_Rmema[m_Rpos++];
            m_Rpos&=0x1;
      	    S+=m_Rb2*m_Rmemb[m_Rpos];
       	    S-=m_Ra2*m_Rmema[m_Rpos++];
            m_Rpos&=0x1;

            if(fabs(S)<0.000000001f)				// to avoid underflow 
				S=0.0;

            m_Rpos++;
            m_Rpos&=0x1;

            m_Rmemb[m_Rpos]=inBuff[i];
            m_Rmema[m_Rpos]=S;
            outBuff[i] =S*m_RgainEQ;
   	}


}



void CSpatializer::SetSamplingFrequency(float sf)
{
	m_sf=sf;
	SetDelays();
	SetEqualizers();
}

void CSpatializer::GetSamplingFrequency(float* sf)
{
	*sf=m_sf;
}

void CSpatializer::SetFrequencyBounds(float f0, float f1)
{
	m_f0=f0;
	m_f1=f1;
	SetEqualizers();
}

void CSpatializer::GetFrequencyBounds(float* f0, float* f1)
{
	*f0=m_f0;
	*f1=m_f1;
}

void CSpatializer::SetTimeLimits(float l0, float l1,float l2, float l3, float modal)
{	
	m_l0=l0;
	m_l1=l1;
	m_l2=l2;
	m_l3=l3;
	m_modal=modal;
	SetDelays();
}

void CSpatializer::GetTimeLimits(float* l0, float* l1,float* l2, float* l3, float* modal)
{	
	*l0=m_l0;
	*l1=m_l1;
	*l2=m_l2;
	*l3=m_l3;
	*modal=m_modal;
}


void CSpatializer::SetSourceAzimuth(float angle)
{	
	m_angle=angle;
}

void CSpatializer::GetSourceAzimuth(float* angle)
{	
	*angle=m_angle;
}

void CSpatializer::SetSourcePosition(float x,float z,float y)
{	float Cos;
	float Sin;

	m_x=x;
	m_z=z;
	m_y=y;
	
	Cos=z/(float)sqrt(x*x+z*z);
	Sin=x/(float)sqrt(x*x+z*z);
	
	if(Sin>0.f)
		m_angle=(float)acos(Cos);
	else
		m_angle=-(float)acos(Cos);

	return;
}

void CSpatializer::GetSourcePosition(float* x,float* z,float* y)
{	*x=m_x;
	*z=m_z;
	*y=m_y;
}

void CSpatializer::SetEnergyParameters(float* R0,float* R1,float R2,float R3,float* RT)
{
	m_R0[0]=R0[0];	
	m_R0[1]=R0[1];	
	m_R0[2]=R0[2];	
	m_R1[0]=R1[0];	
	m_R1[1]=R1[1];	
	m_R1[2]=R1[2];
	m_R2=R2;		
	m_R3=R3;		
	m_RT[0]=RT[0];	   
	m_RT[1]=RT[1];	  
	m_RT[2]=RT[2];	   

	SetEqualizers();

}

void CSpatializer::GetEnergyParameters(float* R0,float* R1,float R2,float R3,float* RT)
{
	R0[0]=m_R0[0];	
	R0[1]=m_R0[1];	
	R0[2]=m_R0[2];	
	R1[0]=m_R1[0];	
	R1[1]=m_R1[1];	
	R1[2]=m_R1[2];
	R2=m_R2;		
	R3=m_R3;		
	RT[0]=m_RT[0];	  
	RT[1]=m_RT[1];	  
	RT[2]=m_RT[2];	   

}




/* ================================================================================ */

extern "C" void computePanPotGains(float azimuth,float* gainRight,float* gainLeft);

void CSpatializer::pan(float* in,float* outPt,float angle,int num)
{

	float gain_left;
	float gain_right;

	if(!m_bSpatialize)
		computePanPotGains(0.f,&gain_right,&gain_left);
	else
	    computePanPotGains(angle,&gain_right,&gain_left);

	// attenuation 
	
	for(int i=0;i<num;i++)
	{	outPt[2*i]=in[i]*gain_left;
		outPt[2*i+1]=in[i]*gain_right;		
	}
}
/* ================================================================================ */

void CSpatializer::panAdd(float* in,float* outPt,float angle,int num)
{

	float gain_left;
	float gain_right;

	if(!m_bSpatialize)
		computePanPotGains(0.f,&gain_right,&gain_left);
	else
	    computePanPotGains(angle,&gain_right,&gain_left);

	// attenuation 

	for(int i=0;i<num;i++)
	{	outPt[2*i]+=in[i]*gain_left;
		outPt[2*i+1]+=in[i]*gain_right;		
	}

}

/* ================================================================================ */


float CSpatializer::Energy(float* in,int Size)
{
	float nrj=0.f;
	for(int ind1=0;ind1<Size;ind1++)
	{	nrj+=(*in)*(*in);
		in++;
	}

	return(nrj);
}


/* ================================================================================ */

void CSpatializer::Mix(float in1[4][MAXNUMBEROFSAMPLE],float in2[4][MAXNUMBEROFSAMPLE],float out[4][MAXNUMBEROFSAMPLE],int num)
{
    // PORTING to CC 5.3
    int j = 0;
//	for(int j=0;j<4;j++)
	for( j=0;j<4;j++)
		for(int i=0;i<num;i++)
			out[j][i]=in1[j][i]+in2[j][i];

	Hadamard4(&out[0][0],&out[1][0],&out[2][0],&out[3][0],num);

	for(j=0;j<4;j++)
		for(int i=0;i<num;i++)
			out[j][i]*=0.5;		// 0.5 corresponds to 1/sqrt(4)


}

/*****************************************************************************************/

void CSpatializer::Process(short* ps_in,short* ps_out,int num)
{
	float inSource[MAXNUMBEROFSAMPLE];

	float outSource[MAXNUMBEROFSAMPLE];

	float outEarly[2][MAXNUMBEROFSAMPLE];

	float inCluster[8][MAXNUMBEROFSAMPLE];		// to feed the cluster module

	float outCluster[4][MAXNUMBEROFSAMPLE];		// to render the cluster

	float inReverb[8][MAXNUMBEROFSAMPLE];		// to feed the late reberation module

	float outReverb[4][MAXNUMBEROFSAMPLE];		// to render the reverbered field

	float outDiffuse[4][MAXNUMBEROFSAMPLE];		// to render the complete diffuse field

	float outPt[2*MAXNUMBEROFSAMPLE];

	int i;

	for(i=0;i<num;i++) 
		inSource[i]=(float)ps_in[i];
		
	InputFiltering(inSource,num);						// input filter

	m_direct.DoFiltering(inSource,outSource,num);

	pan(outSource,outPt,m_angle,num);

	if(m_bRoomEffect)
	{	RoomFiltering(inSource,num);

		m_early.DoFiltering(&inSource[0],outEarly,inCluster,num);
		m_cluster.DoFiltering(inCluster,outCluster,inReverb,num);
		m_reverb.DoFiltering(inReverb,outReverb,num);

		Mix(outCluster,outReverb,outDiffuse,num);

		panAdd(outEarly[0],outPt,m_angle-0.35f,num);
		panAdd(outEarly[1],outPt,m_angle+0.35f,num);

		panAdd(outDiffuse[0],outPt,0.7854f,num);	//   PI/4
		panAdd(outDiffuse[1],outPt,2.3562f,num);	//   3*PI/4
		panAdd(outDiffuse[2],outPt,-2.3562f,num);	//   -3*PI/4
		panAdd(outDiffuse[3],outPt,-0.7854f,num);	//   -PI/4

	}
	for(i=0;i<2*num;i++) 
	{	float sample;
		sample=outPt[i];
		if(sample>32767.)
			sample=32767.;
		if(sample<-32768.0)
			sample=-32768;
		ps_out[i]=(short)sample;
	}


}

/*****************************************************************************************/

void CSpatializer::DoFiltering(short* ps_in,short* ps_out,int num)
{
	int i;
	int loop;
	int remainder;
	short* adIn;
	short* adOut;

	loop=num/(int)MAXNUMBEROFSAMPLE;
	remainder=num-(loop*(int)MAXNUMBEROFSAMPLE);

	adIn=ps_in;
	adOut=ps_out;
	for(i=0;i<loop;i++)
	{	Process(adIn,adOut,(int)MAXNUMBEROFSAMPLE);
		adIn+=(int)MAXNUMBEROFSAMPLE;
		adOut+=(2*(int)MAXNUMBEROFSAMPLE);
	}
	Process(adIn,adOut,remainder);	
}



void CSpatializer::Update(CSpatParameters* pspatparameters,
						  float azimuth, 
						  float dirAngle,
						  float x, float y, float z)

{
		float AxisLow,AxisMid,AxisHigh;    // directivity in the axis
		float actual_distance;
		float energetic[15];				// contains the energetic parameters for the considered source


		m_bSpatialize=pspatparameters->m_spatialize;
		m_bRoomEffect=pspatparameters->m_roomEffect;
		
		SetSamplingFrequency(pspatparameters->m_sf);
		SetFrequencyBounds(pspatparameters->m_fmin,pspatparameters->m_fmax);
		SetTimeLimits(pspatparameters->m_l0,
						 pspatparameters->m_l1,
						 pspatparameters->m_l2,
						 pspatparameters->m_l3,
						 pspatparameters->m_modal);

		SetInputFilterCoef(	pspatparameters ->m_InputLow,
							pspatparameters ->m_InputMid,
							pspatparameters ->m_InputHigh);

		SetRoomFilterCoef( pspatparameters ->m_OmniLow,
						   pspatparameters ->m_OmniMid,
						   pspatparameters ->m_OmniHigh);


		pspatparameters->getAxisDirectivity(&AxisLow,&AxisMid,&AxisHigh,dirAngle);
		SetSourceAzimuth(azimuth);

		actual_distance=(x*x+y*y+z*z);
		pspatparameters ->getEnergeticParameters(energetic,actual_distance);


		energetic[0]=sqrt(energetic[0]);
		energetic[1]=sqrt(energetic[1]);
		energetic[2]=sqrt(energetic[2]);

		energetic[3]=sqrt(energetic[3]);
		energetic[4]=sqrt(energetic[4]);
		energetic[5]=sqrt(energetic[5]);

		energetic[6]=sqrt(energetic[6]);
		energetic[7]=sqrt(energetic[7]);

		energetic[0]*=(AxisLow*pspatparameters ->m_DirectLow);
		energetic[1]*=(AxisMid*pspatparameters ->m_DirectMid);
		energetic[2]*=(AxisHigh*pspatparameters ->m_DirectHigh);

		SetEnergyParameters(&energetic[0],&energetic[3],energetic[6],
							 energetic[7],&energetic[8]);



}
