// This software module was originally developed by Marc Emerit and 
// Jean-Bernard Rault (France Telecom) , in the course of
// development of the MPEG-4 Systems standard (ISO/IEC 14496-1).  This
// software module is an implementation of a part of one or more MPEG-4
// Systems tools as specified by the MPEG-4 Systems standard.  ISO/IEC
// gives users of the MPEG-4 Systems standard free license to this software
// module or modifications thereof for use in hardware or software products
// claiming conformance to the MPEG-4 Systems standard.  Those intending to
// use this software module in hardware or software products are advised
// that its use may infringe existing patents.  The original developer of
// this software module and his/her company, the subsequent editors and
// their companies, and ISO/IEC have no liability for use of this software
// module or modifications thereof in an implementation.  Copyright is not
// released for non MPEG-4 Systems conforming products.  The original
// developer retains full right to use the code for his/her own purpose,
// assign or donate the code to a third party and to inhibit third parties
// from using the code for non MPEG-4 Systems conforming products.  This
// copyright notice must be included in all copies or derivative works.
//
// For more information or to receive an up to date version of this module,
// mail karlo@nta.no or check <http://televr.fou.telenor.no/compositor/>.
//
// Copyright 1998.


// Spatializer.h: interface for the CSpatializer class.
//
//////////////////////////////////////////////////////////////////////
#include "Direct.h"
#include "Early.h"
#include "Cluster.h"
#include "Reverberation.h"
#include "const.h"
#include "Spatparameters.h"

class CSpatializer 
{
public:
	CSpatializer();
	virtual ~CSpatializer();

	void SetSamplingFrequency(float sf);
	void GetSamplingFrequency(float* sf);

	void SetFrequencyBounds(float f0, float f1);
	void GetFrequencyBounds(float* f0, float* f1);

	void SetTimeLimits(float l0, float l1,float l2, float l3, float modal);
	void GetTimeLimits(float* l0, float* l1,float* l2, float* l3, float* modal);

	void SetSourcePosition(float x,float z,float y);
	void GetSourcePosition(float* x,float* z,float* y);

	void SetSourceAzimuth(float angle);
	void GetSourceAzimuth(float* angle);
	
	void SetEnergyParameters(float* R0,float* R1,float R2,float R3,float* RT);
	void GetEnergyParameters(float* R0,float* R1,float R2,float R3,float* RT);

	void SetInputFilterCoef(float GainLow,float GainMid,float GainHigh);
	void GetInputFilterCoef(float* GainLow,float* GainMid,float* GainHigh);

	void SetRoomFilterCoef(float GainLow,float GainMid,float GainHigh);
	void GetRoomFilterCoef(float* GainLow,float* GainMid,float* GainHigh);

	void SetSourcePresence(float SourcePresence);
	void SetWarmth(float Warmth);
	void SetBrillance(float Brillance);
	void SetRoomPresence(float RoomPresence);
	void SetEnvelopment(float Envelopment);
	void SetRunningReverberance(float RunningReverberance);
	void SetLateReverberance(float LateReverberance);
	void SetHeaviness(float Heaviness);
	void SetLiveness(float Liveness);

	void DoFiltering(short* ps_in,short* ps_out,int num);
	void Process(short* ps_in,short* ps_out,int num);
	void Update(CSpatParameters* pspatparameters,float azimuth,float dirAngle,
				float x, float y, float z);

private:
	void init();
	void SetDelays();
	void SetEqualizers();

	void SetInputEqualizer();
	void InputFiltering(float *inBuff,int SizeBuff);

	void SetRoomEqualizer();
	void RoomFiltering(float *inBuff,int SizeBuff);

	void pan(float* in,float* outPt[2],float x,float z,int num);
	void pan(float* in,float* outPt,float angle,int num);
	void panAdd(float* in,float* outPt,float angle,int num);
	void Mix(float in1[4][MAXNUMBEROFSAMPLE],float in2[4][MAXNUMBEROFSAMPLE],float out[4][MAXNUMBEROFSAMPLE],int num);

	float Energy(float* in,int size);

	CDirect m_direct;		// direct path 
	CEarly	m_early;		// early reflection
	CCluster m_cluster;		// early diffuse field
	CReverb m_reverb;		// late diffuse field

	float m_x,m_z,m_y;	// coordinate of the source/view point
	float m_angle;		// angle in radians of the source/view point
	float m_sf;			// the operating sampling rate;
	float m_f0,m_f1;	// frequency bounds for the equalizers (reduced frequencies)
	
	float m_l0;			// time limits of the synthetic room response
	float m_l1;			// expressed in ms
	float m_l2;
	float m_l3;
	float m_modal;		// modal density in ms

	float m_R0[3];		// direct path energies		: 3 bands
	float m_R1[3];		// early signals energies	: 3 bands
	float m_R2;			// cluster energy
	float m_R3;			// late diffuse field energy
	float m_RT[3];		// reverberation times		: 3 bands


	// input filter :

	float m_coefInputFilterLow;		// inpult filter gains in the three bands
	float m_coefInputFilterMid;
	float m_coefInputFilterHigh;

	float m_b0;
	float m_b1;
	float m_b2;
	float m_a1;
	float m_a2;

	float m_mema[2];
	float m_memb[2];
	int	  m_pos;

	float m_gainEQ;

	// room filter :

	float m_coefRoomFilterLow;		// inpult filter gains in the three bands
	float m_coefRoomFilterMid;
	float m_coefRoomFilterHigh;

	float m_Rb0;
	float m_Rb1;
	float m_Rb2;
	float m_Ra1;
	float m_Ra2;

	float m_Rmema[2];
	float m_Rmemb[2];
	int	  m_Rpos;

	float m_RgainEQ;

	BOOL  m_bSpatialize;		// if TRUE, spatialialization is performed (default)
	BOOL  m_bRoomEffect;		// if TRUE, room effects are performed (default)

};

