// This software module was originally developed by Marc Emerit and 
// Jean-Bernard Rault (France Telecom) , in the course of
// development of the MPEG-4 Systems standard (ISO/IEC 14496-1).  This
// software module is an implementation of a part of one or more MPEG-4
// Systems tools as specified by the MPEG-4 Systems standard.  ISO/IEC
// gives users of the MPEG-4 Systems standard free license to this software
// module or modifications thereof for use in hardware or software products
// claiming conformance to the MPEG-4 Systems standard.  Those intending to
// use this software module in hardware or software products are advised
// that its use may infringe existing patents.  The original developer of
// this software module and his/her company, the subsequent editors and
// their companies, and ISO/IEC have no liability for use of this software
// module or modifications thereof in an implementation.  Copyright is not
// released for non MPEG-4 Systems conforming products.  The original
// developer retains full right to use the code for his/her own purpose,
// assign or donate the code to a third party and to inhibit third parties
// from using the code for non MPEG-4 Systems conforming products.  This
// copyright notice must be included in all copies or derivative works.
//
// For more information or to receive an up to date version of this module,
// mail karlo@nta.no or check <http://televr.fou.telenor.no/compositor/>.
//
// Copyright 1998.


#ifndef SPAT_PARAMETERS_H
#define SPAT_PARAMETERS_H

#include <stdlib.h>
#include "const.h"
#include "core/Nodes.h"
//ML - added the next line
// SOLARIS PORTING - commented the next line
//#include "core/Nodes2.h"

class CSpatParameters
{
	//
	// Public functions
	//
public:

	// Class constructor 
	CSpatParameters();

	// Class destructor 
	virtual ~CSpatParameters();

	void init(int type);

	void setSf(float sf);

	void getAxisDirectivity(float* AxisLow,float* AxisMid,float* AxisHigh,float directivityAngle);

	void CSpatParameters::getEnergeticParameters(float* m_energetic,float actual_distance);

	void CSpatParameters::setSourcePresence(float value);
	void CSpatParameters::setSourceWarmth(float value);
	void CSpatParameters::setSourceBrilliance(float value);

	void CSpatParameters::setRoomPresence(float value);
	void CSpatParameters::setRunningReverberance(float value);
	void CSpatParameters::setEnvelopment(float value);

	void CSpatParameters::setLateReverberance(float value);
	void CSpatParameters::setHeavyness(float value);
	void CSpatParameters::setLiveness(float value);

	void CSpatParameters::setOmniDirectivity(float* value);
	void CSpatParameters::setDirectivity(float* value);

	void CSpatParameters::setDirectFilterGains(float value1,float value2,float value3);
	void CSpatParameters::setInputFilterGains(float value1,float value2,float value3);
	void CSpatParameters::setRefDistance(float value);
	void CSpatParameters::setFreqLow(float value);
	void CSpatParameters::setFreqHigh(float value);
	void CSpatParameters::setTimeLimits1(float value);
	void CSpatParameters::setTimeLimits2(float value);
	void CSpatParameters::setTimeLimits3(float value);
	void CSpatParameters::setModalDensity(float value);
	void CSpatParameters::setSpatialize(Bool value);
	void CSpatParameters::setRoomEffect(Bool value);

	void CSpatParameters::updateDirectivity(float* tabangles,float* tabfreq,float** tabgain,int num_angles,int num_freq);
	void CSpatParameters::updateOmniDirectivity(float* tabfreq,float* tabgain,int num_freq);

	void CSpatParameters::upDate(DirectiveSound* DS);

// parameters received from the BIFS

	float m_perceptual[9];  // contains the perceptual parameters for the considered source
	float m_referenceDistance;
	float m_l1;
	float m_l2;
	float m_l3;
	float m_modal;
	float m_fmin;
	float m_fmax;
	float m_InputLow,m_InputMid,m_InputHigh;	// Input Filter
	float m_DirectLow,m_DirectMid,m_DirectHigh;	// Direct Filter

	int	  m_nbanglesD,m_nbfreqD;					// for directivity
	float m_tabangleD[10];
	float m_tabfreqD[20];
	float m_directivity[10][20];

	int	  m_nbanglesOD,m_nbfreqOD;					// for omnidirectivity
	float m_tabangleOD[10];
	float m_tabfreqOD[20];
	float m_omnidirectivity[20];

	float m_OD;								// presence of the source at the reference position

	float m_sf;
	float m_l0;

	float m_directivity3B[10][3];
	float m_OmniLow,m_OmniMid,m_OmniHigh;	// omnidirectionnal directivity

	BOOL  m_spatialize;		// if TRUE, spatialialization is performed (default)
	BOOL  m_roomEffect;		// if TRUE, room effects are performed (default)

private:


};

extern "C" void Perceptual2energetic(float* perceptual, float* ener); 

extern "C" void directivityThreeBands(float dirout[10][3],float tab[10][20],float freq[20],
					       int nbfreq,int nbangle,float fe,float fmin,float fmax);

extern "C" void omniDirectivityThreeBands(float* OmniLow,float* OmniMid,float* OmniHigh,float tab[20],float freq[20],
					      int nbfreq,float fe,float fmin,float fmax);

extern "C" void axisDirectivity(float* AxisLow,float* AxisMid,float* AxisHigh,float direct[10][3],float angle[10],int nbangle,float value);

extern "C" void computeDirectivity(float tab[10][20],float tab1[20],float* tabfreq,float* tabangle, 
						float aperturemax,float aperturemin,int numfreq,int numangle);

extern "C" void  rotateVector(	vector V,		// vector to be rotated
								vector D,		// direction of rotation
								float angle);	// angle of rotation


extern "C" void	computeAngles(float* azimuth,		// angle in the OrefXZ plane of Otarget
							  float* elevation,		// elevation wrt the OrefXZ plane of Otarget
							  vector OOr,
					          vector ir,			// the base vectors of the reference base
					          vector jr,			
					          vector kr,
					          vector OOt);

extern "C" void	computeAnglesofTwoVectors(	float* angle,	// result
									vector V1,
									vector V2);

#endif // SPAT_PARAMETERS_H

