/* This software module was originally developed by Marc Emerit and  */
/* Jean-Bernard Rault (France Telecom) , in the course of */
/* development of the MPEG-4 Systems standard (ISO/IEC 14496-1).  This */
/* software module is an implementation of a part of one or more MPEG-4 */
/* Systems tools as specified by the MPEG-4 Systems standard.  ISO/IEC */
/* gives users of the MPEG-4 Systems standard free license to this software */
/* module or modifications thereof for use in hardware or software products */
/* claiming conformance to the MPEG-4 Systems standard.  Those intending to */
/* use this software module in hardware or software products are advised */
/* that its use may infringe existing patents.  The original developer of */
/* this software module and his/her company, the subsequent editors and */
/* their companies, and ISO/IEC have no liability for use of this software */
/* module or modifications thereof in an implementation.  Copyright is not */
/* released for non MPEG-4 Systems conforming products.  The original */
/* developer retains full right to use the code for his/her own purpose, */
/* assign or donate the code to a third party and to inhibit third parties */
/* from using the code for non MPEG-4 Systems conforming products.  This */
/* copyright notice must be included in all copies or derivative works. */
/* // For more information or to receive an up to date version of this module, */
/* mail karlo@nta.no or check <http://televr.fou.telenor.no/compositor/>. */
/* // Copyright 1998. */



#include <math.h>




/*****************************************************************************/

void directivityThreeBands(float dirout[10][3],float tab[10][20],float freq[20],
					       int nbfreq,int nbangle,float fe,float fmin,float fmax)
 {
	float tabout[10][512];
	float	*pf;
	int index;
	float step_freq;
	int i,j;
	int indmin;
	int indmax;
	float s;

	step_freq=fe/1024.f;


	for(j=0;j<nbangle;j++)
	{
		index=0;
		i=0;
		pf=&tabout[j][0];
		*pf=1.0f;				/* directivity dans l'axe par dfault; */
		while(freq[index]<i*step_freq)
		{	i++;
			*pf++=tab[j][index];
		}

		for(;i*step_freq<freq[nbfreq-1] && i<512 ;i++)
		{  while(i*step_freq>freq[index])
			index++;
			if(index>nbfreq)
				*pf++=tab[j][nbfreq-1];
			else
			*pf++=tab[j][index]+(tab[j][index+1]-tab[j][index])*
				(i*step_freq-freq[index])/(freq[index+1]-freq[index]);
		}
		for(;i<512;i++)
			*pf++=tab[j][nbfreq-1];

	}

	indmin=(int)(511.f*fmin*2./fe);
	indmax=(int)(511.f*fmax*2./fe);

	for(j=0;j<nbangle;j++)
	{	
		s=0.f;
		for(i=0;i<indmin;i++)
		{	s+=tabout[j][i];
		
		}
		dirout[j][0]=s/indmin;
		s=0.f;
		for(i=indmin;i<indmax;i++)
		{	s+=tabout[j][i];
		
		}
		dirout[j][1]=s/(indmax-indmin);
		s=0.f;
		for(i=indmax;i<512;i++)
		{	s+=tabout[j][i];
		
		}
		dirout[j][2]=s/(512.f-indmax);
	}

}

/*****************************************************************************/

void omniDirectivityThreeBands(float* OmniLow,float* OmniMid,float* OmniHigh,float tab[20],float freq[20],
							   int nbfreq,float fe,float fmin,float fmax)
 {
	float tabout[512];
	float	*pf;
	int index;
	float step_freq;
	int i;
	int indmin;
	int indmax;
	float s;

	step_freq=fe/1024.f;



		index=0;
		i=0;
		pf=&tabout[0];
		*pf=1.0f;				/* directivity dans l'axe par dfault; */
		while(i*step_freq<freq[index])
		{	i++;
			*pf++=tab[index];
		}

		for(;i*step_freq<=freq[nbfreq-2] && i<512 ;i++)
		{  while(i*step_freq>freq[index])
			index++;
			if(index>nbfreq)
				*pf++=tab[nbfreq-1];
			else
			*pf++=tab[index]+(tab[index+1]-tab[index])*
				(i*step_freq-freq[index])/(freq[index+1]-freq[index]);
		}
		for(;i<512;i++)
			*pf++=tab[nbfreq-1];

	indmin=(int)(511.f*fmin*2./fe);
	indmax=(int)(511.f*fmax*2./fe);

		s=0.f;
		for(i=0;i<indmin;i++)
		{	s+=tabout[i];
		
		}
		*OmniLow=s/indmin;
		s=0.f;
		for(i=indmin;i<indmax;i++)
		{	s+=tabout[i];
		
		}
		*OmniMid=s/(indmax-indmin);
		s=0.f;
		for(i=indmax;i<512;i++)
		{	s+=tabout[i];
		
		}
		*OmniHigh=s/(512.f-indmax);

}


void axisDirectivity(float* AxisLow,float* AxisMid,float* AxisHigh,float direct[10][3],float angle[10],int nbangle,float value)
{
	int i;
	float x;
	i=0;
	
	while(value>angle[i+1] && i<nbangle-1)
		i++;
	x=(value-angle[i])/(angle[i+1]-angle[i]);
	*AxisLow=direct[i][0]+(direct[i+1][0]-direct[i][0])*x;
	*AxisMid=direct[i][1]+(direct[i+1][1]-direct[i][1])*x;
	*AxisHigh=direct[i][2]+(direct[i+1][2]-direct[i][2])*x;

	return;
}


void computeDirectivity(float tab[10][20],float tab1[20],float* tabfreq,float* tabangle, 
						float aperturemax,float aperturemin,int numfreq,int numangle)
{
	int i,j;
	
	float freqmax=20000.;
	float freqmin;
	float freq;
	float anglemax;
	float anglemin=0.;
	float angle;
	float step;
	float stepAperture;
	float K;

	if(aperturemax<aperturemin)
		aperturemax=aperturemin;
	stepAperture=(aperturemax-aperturemin)/(numfreq-1);

	freqmin=freqmax/((float)pow(2.,numfreq-1.f));
	anglemax=4.f*(float)atan(1.);

	step=anglemax/(numangle-1);
	freq=freqmin;

	for(j=0;j<numangle;j++)
	{	tabangle[j]=j*step;
	}
	for(i=0;i<numfreq;i++)
	{	tabfreq[i]=freq;
		freq*=2.;
	}

	for(j=0;j<numangle;j++)
	{	angle=j*step;
		for(i=0;i<numfreq;i++)
		{	tab[j][i]=(float)(-3.f*(1.f-cos(angle))/(1.f-cos(aperturemax-stepAperture*i)));
			tab[j][i]=(float)(pow(10.0,tab[j][i]/20.));	
		}
	}

	for(i=0;i<numfreq;i++)
	{	K=(float)((1.0-cos(aperturemax-stepAperture*i))/(2.0*log(2.0)));
		tab1[i]=K*(1- exp(-1/K));
		tab1[i]=(float)(sqrt(tab1[i]));
	}

}

