
/* This software module was originally developed by Marc Emerit and  */
/* Jean-Bernard Rault (France Telecom) , in the course of */
/* development of the MPEG-4 Systems standard (ISO/IEC 14496-1).  This */
/* software module is an implementation of a part of one or more MPEG-4 */
/* Systems tools as specified by the MPEG-4 Systems standard.  ISO/IEC */
/* gives users of the MPEG-4 Systems standard free license to this software */
/* module or modifications thereof for use in hardware or software products */
/* claiming conformance to the MPEG-4 Systems standard.  Those intending to */
/* use this software module in hardware or software products are advised */
/* that its use may infringe existing patents.  The original developer of */
/* this software module and his/her company, the subsequent editors and */
/* their companies, and ISO/IEC have no liability for use of this software */
/* module or modifications thereof in an implementation.  Copyright is not */
/* released for non MPEG-4 Systems conforming products.  The original */
/* developer retains full right to use the code for his/her own purpose, */
/* assign or donate the code to a third party and to inhibit third parties */
/* from using the code for non MPEG-4 Systems conforming products.  This */
/* copyright notice must be included in all copies or derivative works. */
/* */
/* For more information or to receive an up to date version of this module, */
/* mail karlo@nta.no or check <http://televr.fou.telenor.no/compositor/>. */
/* */
/* Copyright 1998. */


#include <math.h>
#include <time.h>
#include <stdlib.h>
#include <memory.h>
// WIN-SOL release
#ifndef WIN32
#include "Spatparameters.h" 
#endif // WIN32


#define PI  3.14159265359f

float Tamp1,Tamp2;
float Tamp3,Tamp4;
// WIN-SOL release
#ifndef WIN32
extern "C" {
#endif // WIN32

/*satish changed this line */
// WIN-SOL release
#ifdef WIN32
_inline void Hadamard2(float *inPt1,float *inPt2,int SizeBuff) 
#else
inline void Hadamard2(float *inPt1,float *inPt2,int SizeBuff)
#endif // WIN32
{	int ind1;

	for(ind1=0;ind1<SizeBuff;ind1++)
	{	Tamp1 = inPt1[ind1];
		inPt1[ind1]= Tamp1+inPt2[ind1];
		inPt2[ind1]= Tamp1-inPt2[ind1];
	}
}


void Hadamard4(float *inPt1,float *inPt2, float *inPt3,float *inPt4,int SizeBuff)
{	int ind1;
 
	Hadamard2(inPt1,inPt2,SizeBuff);
	Hadamard2(inPt3,inPt4,SizeBuff);

	for(ind1=0;ind1<SizeBuff;ind1++)
	{	Tamp1 = inPt1[ind1];
		Tamp2 = inPt2[ind1];
		inPt1[ind1]= inPt3[ind1]+Tamp1;
		inPt2[ind1]= inPt4[ind1]+Tamp2;
		inPt3[ind1]= Tamp1-inPt3[ind1];
		inPt4[ind1]= Tamp2-inPt4[ind1];
	}
}



void Hadamard8(float *inPt1,float *inPt2,
					   float *inPt3,float *inPt4,
					   float *inPt5,float *inPt6,
					   float *inPt7,float *inPt8,int SizeBuff)
{	int ind1;


	Hadamard4(inPt1,inPt2,inPt3,inPt4,SizeBuff);
	Hadamard4(inPt5,inPt6,inPt7,inPt8,SizeBuff);

	for(ind1=0;ind1<SizeBuff;ind1++)
	{	Tamp1 = inPt1[ind1];
		Tamp2 = inPt2[ind1];
		Tamp3 = inPt3[ind1];
		Tamp4 = inPt4[ind1];
		inPt1[ind1]= inPt5[ind1]+Tamp1;
		inPt2[ind1]= inPt6[ind1]+Tamp2;
		inPt3[ind1]= inPt7[ind1]+Tamp3;
		inPt4[ind1]= inPt8[ind1]+Tamp4;
		inPt5[ind1]= Tamp1-inPt5[ind1];
		inPt6[ind1]= Tamp2-inPt6[ind1];
		inPt7[ind1]= Tamp3-inPt7[ind1];
		inPt8[ind1]= Tamp4-inPt8[ind1];


		/* shuffling and normalizing the energies */
			
		Tamp1=inPt1[ind1];
		inPt1[ind1]= inPt8[ind1]*(float)0.3535533905933; /*1/sqrt(8) */
		inPt8[ind1]= inPt4[ind1]*(float)0.3535533905933; /*1/sqrt(8) */
		inPt4[ind1]= inPt6[ind1]*(float)0.3535533905933; /*1/sqrt(8) */
		inPt6[ind1]= inPt3[ind1]*(float)0.3535533905933; /*1/sqrt(8) */
		inPt3[ind1]= inPt2[ind1]*(float)0.3535533905933; /*1/sqrt(8) */
		inPt2[ind1]= inPt5[ind1]*(float)0.3535533905933; /*1/sqrt(8) */
		inPt5[ind1]= inPt7[ind1]*(float)0.3535533905933; /*1/sqrt(8) */
		inPt7[ind1]= Tamp1*(float)0.3535533905933; /*1/sqrt(8) */

	}


	return;

}

/*satish changed this line */
#ifdef WIN32
_inline void hadamard2(float *inPt1,float *inPt2) 
#else
inline void hadamard2(float *inPt1,float *inPt2)
#endif // WIN32
{	 Tamp1 = *inPt1;
	 *inPt1= Tamp1+*inPt2;
	 *inPt2= Tamp1-*inPt2;
}

/*satish changed this line */
// WIN-SOL release
#ifdef WIN32
_inline void hadamard4(float *inPt1,float *inPt2, 
					   float *inPt3,float *inPt4)
#else
inline void hadamard4(float *inPt1,float *inPt2,
					   float *inPt3,float *inPt4)
#endif // WIN32
{	
	hadamard2(inPt1,inPt2);
	hadamard2(inPt3,inPt4);

	Tamp1 = *inPt1;
	Tamp2 = *inPt2;
	*inPt1= *inPt3+Tamp1;
	*inPt2= *inPt4+Tamp2;
	*inPt3= Tamp1-*inPt3;
	*inPt4= Tamp2-*inPt4;
}



 void hadamard8(float *inPt1,float *inPt2,
				float *inPt3,float *inPt4,
				float *inPt5,float *inPt6,
				float *inPt7,float *inPt8)
{	
	hadamard4(inPt1,inPt2,inPt3,inPt4);
	hadamard4(inPt5,inPt6,inPt7,inPt8);

		Tamp1 = *inPt1;
		Tamp2 = *inPt2;
		Tamp3 = *inPt3;
		Tamp4 = *inPt4;
		*inPt1= *inPt5+Tamp1;
		*inPt2= *inPt6+Tamp2;
		*inPt3= *inPt7+Tamp3;
		*inPt4= *inPt8+Tamp4;
		*inPt5= Tamp1-*inPt5;
		*inPt6= Tamp2-*inPt6;
		*inPt7= Tamp3-*inPt7;
		*inPt8= Tamp4-*inPt8;


		/* shuffling and normalizing the energies */
			
		Tamp1=*inPt1;
		*inPt1= *inPt8*(float)0.3535533905933; /*1/sqrt(8) */
		*inPt8= *inPt4*(float)0.3535533905933; /*1/sqrt(8) */
		*inPt4= *inPt6*(float)0.3535533905933; /*1/sqrt(8) */
		*inPt6= *inPt3*(float)0.3535533905933; /*1/sqrt(8) */
		*inPt3= *inPt2*(float)0.3535533905933; /*1/sqrt(8) */
		*inPt2= *inPt5*(float)0.3535533905933; /*1/sqrt(8) */
		*inPt5= *inPt7*(float)0.3535533905933; /*1/sqrt(8) */
		*inPt7= Tamp1*(float)0.3535533905933; /*1/sqrt(8) */

	return;

}




void ComputeDelays(float* tabDelay,float minDelay,float maxDelay,int nb)
{
	/* compute nb delays that range between minDelay and maxDelay */

	float delay,random,interval;
	int i;

 	srand( (unsigned)time( NULL ) );
	interval=(maxDelay-minDelay)/(float)(nb-1);
	delay=minDelay;

	random=(float) (( rand()/(float)RAND_MAX)-0.5f)*0.25f; /* get a number between -0.25 and 0.25; */
	tabDelay[0]=(float)(delay+interval*fabs(random));
	delay+=interval;

	for(i=1;i<(nb-1);i++)
	{	random=(float)(( rand()/(float)RAND_MAX)-0.5f)*0.25f; /* get a number between -0.25 and 0.25 */
		tabDelay[i]=(float)(delay+interval*random);
		delay+=interval;
	}
	
	random=(float)(( rand()/(float)RAND_MAX)-0.5f)*0.25f; /* get a number between -0.25 and 0.25; */
	tabDelay[nb-1]=(float)(delay-interval*fabs(random));

}



void UpdateEQ(float GainLow, float GainMid, float GainHigh, float FreqLow, float FreqHigh, float coef[5])
{	
	/* NB : the gain are supposed to be given in linear wrt amplitude */

	float r1, r2, k1, k2, k, a1, b1, c1, a2, b2, c2, b0;

	k1 = GainLow/GainMid;
	r1 = (float) (tan(PI*FreqLow)/sqrt(k1));

	a1 =    (r1 - 1) / (r1 + 1); 
	b1 = (k1*r1 - 1) / (k1*r1 + 1);
	c1 = (k1*r1 + 1) / (r1 + 1);

	k2 = GainMid/GainHigh;
	r2 = (float) (tan(PI*FreqHigh)/sqrt(k2));

	a2 =    (r2 - 1) / (r2 + 1); 
	b2 = (k2*r2 - 1) / (k2*r2 + 1);
	c2 = (k2*r2 + 1) / (r2 + 1);


	k = GainHigh/GainMid;

	b0 = c1*c2*k;

	coef[0]=b0;				/*b0 */
	coef[1]=(b1+b2)*b0;		/*b1 */
	coef[2]=b1*b2*b0;		/*b2 */
	coef[3]=a1+a2;			/*a1 */
	coef[4]=a1*a2;			/*a2 */

	return;
}

void WriteMemory(float* org,float* dest,int* index, int nb, int size)
{	
	
	if((*index+nb)>=size)
	{	memcpy(&dest[*index],org,(size-*index)*sizeof(float));
		memcpy(&dest[0],&org[size-*index],(nb-size+*index)*sizeof(float));
	}
	else 
	{	memcpy(&dest[*index],org,nb*sizeof(float));
	}
	
	*index=(*index+nb)%size;

	return ;
}


void ReadMemory(float *org,float *dest, int* index, int nb, int size)
{	

	
	if((*index+nb)>=size)
	{	memcpy(&dest[0],&org[*index],(size-*index)*sizeof(float));
		memcpy(&dest[size-*index],&org[0],(nb-size+*index)*sizeof(float));
	}
	else 
	{	memcpy(&dest[0],&org[*index],nb*sizeof(float));
	}

	*index=(*index+nb)%size;
	
	return ;
}


void Perceptual2energetic(float* perceptual, float* ener)
{	
		#define EPS_MIN 1.0e-12

		static float EdtMin,EdtMax,Rd1Min,Rd1Max;
	
		float SourcePresence;	
		float Warmth;			
		float Brillance;			
		float RoomPresence;			
		float Envelopment;			
		float RunningReverberance;			
		float LateReverberance;			
		float Heaviness;			
		float Liveness;				

		float R1,R1bf,R1hf,R2,R3,OD,ODbf,ODhf,RT,RTbf,RThf,C;
		float Es,Rev,Env;

		float tamp1 ;
		float tamp2 ;
#if 0
		SourcePresence=(float)pow(10.0,perceptual[0]/10.);
		Warmth=(float)pow(10.0,perceptual[1]/10.);
		Brillance=(float)pow(10.0,perceptual[2]/10.);
		RoomPresence=(float)pow(10.0,perceptual[3]/10.);
		RunningReverberance=perceptual[4];
		Envelopment=(float)pow(10.0,perceptual[5]/10.f);
		LateReverberance=perceptual[6];
		Heaviness=perceptual[7];
		Liveness=perceptual[8];
#else	
		SourcePresence=perceptual[0];
		Warmth=perceptual[1];
		Brillance=perceptual[2];
		RoomPresence=perceptual[3];
		RunningReverberance=perceptual[4];
		Envelopment=perceptual[5];
		LateReverberance=perceptual[6];
		Heaviness=perceptual[7];
		Liveness=perceptual[8];
#endif

		Es=SourcePresence;
		Rev=RoomPresence;

		C=(float)pow(10.0,-1.2/LateReverberance);

		tamp1 = Rev/Es;
		tamp2 = (float)(2.0*(1.0+C)/(1.0-C));

		if (tamp1 <= tamp2 )
			R3 = (-C+(float)sqrt(C*C+0.5*(1.0-C)*(1.0-C)*Rev/Es))
			*(float)(4.0*Es/(1.0-C)/(1.0-C));
		else
			R3= Rev+(float)2.0*Es;

		if(R3<(float)EPS_MIN) 
			R3=(float)EPS_MIN;

		tamp1=(float)2.0*Es/R3;
		if (tamp1 <= (float)30.622) 
			EdtMin=(float)(0.4+LateReverberance*(1.0-0.667*log10(1.0+2.0*Es/R3))); 
		else 
			EdtMin=(float)(0.6/log10(1.0+2.0*Es/R3));

		tamp1=Es/R3;
		if (tamp1<= (float)30.622) 
			EdtMax=(float)(0.4+LateReverberance*(1.0-0.667*log10(1.0+Es/R3))); 
		else 
			EdtMax=(float)(0.6/log10(1.0+Es/R3));

		RunningReverberance=RunningReverberance*(EdtMax-EdtMin)+EdtMin;

		if (RunningReverberance > (float)0.4)
			R2 = -Es+R3*(float)(pow(10.0,1.5*(1.0+(0.4-RunningReverberance)/LateReverberance))-1.0);
		else
			R2 = -Es+R3*(float)(pow(10.0,0.6/RunningReverberance)-1.0);

		if(R2<(float)EPS_MIN) 
			R2=(float)EPS_MIN;



		Rd1Min=(float)0.05*R2/Es;
		Rd1Max=(float)(0.30+0.05*R2/Es);

		Envelopment=Envelopment*(Rd1Max-Rd1Min)+Rd1Min;

		Env=Envelopment;

		if(Env<(float)EPS_MIN) 
			Env=(float)EPS_MIN;

		R1 = (Es*Env-(float)0.05*R2)/(float)0.3;
		if(R1<(float)EPS_MIN) 
			R1=(float)EPS_MIN;


		OD = Es - R1;
		if(OD<(float)EPS_MIN) 
			OD=(float)EPS_MIN;

		ener[0]=ODbf=(float)(OD*Warmth);
		ener[1]=(float)(OD);
		ener[2]=ODhf=(float)(OD*Brillance);

		ener[3]=R1bf=(float)(R1*Warmth);
		ener[4]=(float)(R1);
		ener[5]=R1hf=(float)(R1*Brillance);

		ener[6]=(float)(R2);

		ener[7]=(float)(R3);

		ener[8]=RTbf=LateReverberance*Heaviness;
		ener[9]=RT=LateReverberance;
		ener[10]=RThf=LateReverberance*Liveness;

		ener[11]=EdtMin;
		ener[12]=EdtMax;
		ener[13]=Rd1Min;
		ener[14]=Rd1Max;

}
// WIN-SOL release
#ifndef WIN32
}
#endif // WIN32
