
/* This software module was originally developed by Marc Emerit and  */
/* Jean-Bernard Rault (France Telecom) , in the course of */
/* development of the MPEG-4 Systems standard (ISO/IEC 14496-1).  This */
/* software module is an implementation of a part of one or more MPEG-4 */
/* Systems tools as specified by the MPEG-4 Systems standard.  ISO/IEC */
/* gives users of the MPEG-4 Systems standard free license to this software */
/* module or modifications thereof for use in hardware or software products */
/* claiming conformance to the MPEG-4 Systems standard.  Those intending to */
/* use this software module in hardware or software products are advised */
/* that its use may infringe existing patents.  The original developer of */
/* this software module and his/her company, the subsequent editors and */
/* their companies, and ISO/IEC have no liability for use of this software */
/* module or modifications thereof in an implementation.  Copyright is not */
/* released for non MPEG-4 Systems conforming products.  The original */
/* developer retains full right to use the code for his/her own purpose, */
/* assign or donate the code to a third party and to inhibit third parties */
/* from using the code for non MPEG-4 Systems conforming products.  This */
/* copyright notice must be included in all copies or derivative works. */
/* */
/* For more information or to receive an up to date version of this module, */
/* mail karlo@nta.no or check <http://televr.fou.telenor.no/compositor/>. */
/* */
/* Copyright 1998. */

#include "const.h"
#include <math.h>

// WIN-SOL release
#ifndef WIN32
extern "C" {
#endif // WIN32


float normVector(vector V)
{	
	return(V[0]*V[0]+V[1]*V[1]+V[2]*V[2]);
}

void addVectors (vector Sum, vector V1, vector V2)
{	
	Sum[0]=V1[0]+V2[0];
	Sum[1]=V1[1]+V2[1];
	Sum[2]=V1[2]+V2[2];
	return ;
}

void subVectors ( vector Dif, vector V1, vector V2)
{	
	Dif[0]=V1[0]-V2[0];
	Dif[1]=V1[1]-V2[1];
	Dif[2]=V1[2]-V2[2];
	return;
}

void vectorialProduct(vector Vect,vector U, vector V)
{	
	Vect[0]=U[1]*V[2]-U[2]*V[1];
	Vect[1]=U[2]*V[0]-U[0]*V[2];
	Vect[2]=U[0]*V[1]-U[1]*V[0];
	return ;
}

float scalarProduct(vector V1, vector V2)
{	float s;
	s=V1[0]*V2[0]+V1[1]*V2[1]+V1[2]*V2[2];
	return s;
}

void multVector(vector Mul,vector V1, float a)
{	
	Mul[0]=a*V1[0];
	Mul[1]=a*V1[1];
	Mul[2]=a*V1[2];
	return;
}

float computeAngle(float Cos, float Sin)
{	float angle;
	angle=(float)acos(Cos);
	if(Sin<0.f)
		return -angle;
	else
		return angle;

}


	
#define apertureR  2.0944f   /* 90+30 expressed in radian */
#define apertureL  1.0472f   /* 90-30 expressed in radian */


void computePanPotGains(float azimuth,float* gainRight,float* gainLeft)
{	float pan;

	pan=(float)(0.5+0.5*sin(azimuth));
	*gainRight=(1.f-(1.f-pan)*(1.f-pan));
	*gainLeft=(1.f-pan*pan);

	return;
}



void  rotateVector(	vector V,		/* vector to be rotated */
					vector D,		/* direction of rotation */
					float angle)	/* angle of rotation */
{
	float norm,x;
	vector V1,V2,V3;

	norm=normVector(D);
	multVector(V1,D,1.f/norm);				/* V1 is D normalized */

	x=scalarProduct(V,V1);
	multVector(V2,V1,x);					/* V2 is V projected on D */

	vectorialProduct(V3,V1,V);				/* V3 orthogonal to V and V1 */

	V[0]=(float)cos(angle)*(V[0]-V2[0])+(float)sin(angle)*V3[0]+V2[0];
	V[1]=(float)cos(angle)*(V[1]-V2[1])+(float)sin(angle)*V3[1]+V2[1];
	V[2]=(float)cos(angle)*(V[2]-V2[2])+(float)sin(angle)*V3[2]+V2[2];

	return;
}


void	computeAngles(float* azimuth,		/* angle in the OrefXZ plane of Otarget */
					  float* elevation,		/* elevation wrt the OrefXZ plane of Otarget */
					  vector OOr,
					  vector ir,			/* the base vectors of the reference base */
					  vector jr,			
					  vector kr,
					  vector OOt)
	

{	vector OrOt;
	vector kh;			/* normalized projection of OrOt in the plane (Or,kr,ir); */
	float cosAlpha;
	float sinAlpha;
	float cosTheta;
	float sinTheta;
	float norm;
	float x1,x2;
	vector v1,v2;
	
	subVectors(OrOt,OOt,OOr);

	x1=scalarProduct(OrOt,kr);
	x2=scalarProduct(OrOt,ir);
	norm=(float)(sqrt(x1*x1+x2*x2));
	cosAlpha=scalarProduct(OrOt,kr)/norm;
	sinAlpha=scalarProduct(OrOt,ir)/norm;

	multVector(v1,kr,cosAlpha);
	multVector(v2,ir,sinAlpha);
	addVectors(kh,v1,v2);

	x1=scalarProduct(OrOt,kh);
	x2=scalarProduct(OrOt,jr);
	norm=(float)(sqrt(x1*x1+x2*x2));
	cosTheta=scalarProduct(OrOt,kh)/norm;
	sinTheta=scalarProduct(OrOt,jr)/norm;

	*azimuth=computeAngle(cosAlpha,sinAlpha);
	*elevation=computeAngle(cosTheta,sinTheta);

	return;

};

void	computeAnglesofTwoVectors(	float* angle,	/* result */
									vector V1,
									vector V2)
{	float norm1,norm2,cosAlpha;
	
	norm1=(float)(sqrt(V1[0]*V1[0]+V1[1]*V1[1]+V1[2]*V1[2]));
	norm2=(float)(sqrt(V2[0]*V2[0]+V2[1]*V2[1]+V2[2]*V2[2]));

	cosAlpha=scalarProduct(V1,V2)/(norm1*norm2);
	*angle=(float)acos(cosAlpha);

	return;

};
// WIN-SOL release
#ifndef WIN32
} /* SOLARIS PORTING - extern "C" */
#endif // WIN32
