// This software module was originally developed by Marc Emerit and 
// Jean-Bernard Rault (France Telecom) , in the course of
// development of the MPEG-4 Systems standard (ISO/IEC 14496-1).  This
// software module is an implementation of a part of one or more MPEG-4
// Systems tools as specified by the MPEG-4 Systems standard.  ISO/IEC
// gives users of the MPEG-4 Systems standard free license to this software
// module or modifications thereof for use in hardware or software products
// claiming conformance to the MPEG-4 Systems standard.  Those intending to
// use this software module in hardware or software products are advised
// that its use may infringe existing patents.  The original developer of
// this software module and his/her company, the subsequent editors and
// their companies, and ISO/IEC have no liability for use of this software
// module or modifications thereof in an implementation.  Copyright is not
// released for non MPEG-4 Systems conforming products.  The original
// developer retains full right to use the code for his/her own purpose,
// assign or donate the code to a third party and to inhibit third parties
// from using the code for non MPEG-4 Systems conforming products.  This
// copyright notice must be included in all copies or derivative works.
//
// For more information or to receive an up to date version of this module,
// mail karlo@nta.no or check <http://televr.fou.telenor.no/compositor/>.
//
// Copyright 1998.


// CSpatParameters.cpp: implementation of the CSpatParameters class.
//
//////////////////////////////////////////////////////////////////////
#include "Include/defs.h"
#include "Spatparameters.h"
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
// SOLARIS PORTING - include io.h for WIN32
#ifdef WIN32
#include <io.h>
#endif
// SOLARIS PORTING - END

#define PI 3.14159265

CSpatParameters::CSpatParameters()
{	

	init(1);

}


void CSpatParameters::init(int type)

{
	int i;
// Begin Parameters sent in the BIFS

	if(type==0)					// cathedral type
	{
		m_perceptual[0]=0.5f;	//SourcePresence;	
		m_perceptual[1]=1.0f;	//Warmth;			
		m_perceptual[2]=1.0f;	//Brillance;			
		m_perceptual[3]=0.1f;	//RoomPresence;			
		m_perceptual[4]=0.50f;	//RunningReverberance;			
		m_perceptual[5]=0.5f;	//Envelopment;			
		m_perceptual[6]=3.0f;	//LateReverberance;			
		m_perceptual[7]=2.0f;	//Heaviness;			
		m_perceptual[8]=0.5f;	//Liveness;	
	}
	else if(type==1)			// no room effect
	{	m_perceptual[0]=0.5f;	//SourcePresence;	
		m_perceptual[1]=1.0f;	//Warmth;			
		m_perceptual[2]=1.0f;	//Brillance;			
		m_perceptual[3]=0.0001f;//RoomPresence;			
		m_perceptual[4]=1.0f;	//RunningReverberance;			
		m_perceptual[5]=0.0f;	//Envelopment;			
		m_perceptual[6]=2.0f;	//LateReverberance;			
		m_perceptual[7]=1.0f;	//Heaviness;			
		m_perceptual[8]=1.0f;	//Liveness;
	}

	m_OD=m_perceptual[0];
	m_referenceDistance=3;
	m_referenceDistance*=m_referenceDistance;

	m_nbanglesD=m_nbanglesOD=10;
	m_nbfreqD=m_nbfreqOD=20;
	computeDirectivity(m_directivity,m_omnidirectivity,m_tabfreqD,m_tabangleD, // INIT
					2.0,0.0f,m_nbfreqD,m_nbanglesD);

	for(i=0;i<m_nbfreqD;i++)
		m_tabfreqOD[i]=m_tabfreqD[i];

	m_InputLow=m_InputMid=m_InputHigh=1.0f;
	m_DirectLow=m_DirectMid=m_DirectHigh=1.0f;

	m_fmin=250.f;
	m_fmax=1000.f;

	m_l1=0.01f;
	m_l2=0.03f;
	m_l3=0.06f;
	m_modal=0.800f;

	// End Parameters sent in the BIFS


	m_sf=8000.f;
	m_l0=0.f;

	m_OmniLow=m_OmniMid=m_OmniHigh=1.0f;


	directivityThreeBands(m_directivity3B,m_directivity,m_tabfreqD,
					       m_nbfreqD,m_nbanglesD,m_sf,m_fmin,m_fmax);

	omniDirectivityThreeBands(&m_OmniLow,&m_OmniMid,&m_OmniHigh,m_omnidirectivity,m_tabfreqOD,
					      m_nbfreqOD,m_sf,m_fmin,m_fmax);


	m_spatialize=TRUE;		// if TRUE, spatialialization is performed (default)
	m_roomEffect=TRUE;		// if TRUE, room effects are performed (default)


}


CSpatParameters::~CSpatParameters()
{
}

void CSpatParameters::setSf(float sf)
{	m_sf=sf;
	directivityThreeBands(m_directivity3B,m_directivity,m_tabfreqD,
					       m_nbfreqD,m_nbanglesD,m_sf,m_fmin,m_fmax);

	omniDirectivityThreeBands(&m_OmniLow,&m_OmniMid,&m_OmniHigh,m_omnidirectivity,m_tabfreqOD,
					      m_nbfreqOD,m_sf,m_fmin,m_fmax);

}



void CSpatParameters::updateDirectivity(float* tabangles,float* tabfreq,float** tabgain,int num_angles,int num_freq)
{
	int i,j;

	m_nbanglesD=0;
	for(i=0;i<10;i++)
		m_tabangleD[i]=0.0;
	for(i=0;i<20;i++)
		m_tabfreqD[i]=0.0;
	for(i=0;i<10;i++)
	{	for(j=0;j<20;j++)
			m_directivity[i][j]=0.0;
		for(j=0;j<3;j++)
			m_directivity3B[i][j]=0.0;
	}
	
	if(tabangles[0]>0.0)
	{		// 0 radian angle is missing, we add it
		for(j=0;j<num_freq;j++)
		{	m_directivity[0][j]=1.0;
		}
		m_tabangleD[0]=0.0;
		m_nbanglesD++;	
	}

	for(i=0;i<num_angles;i++)
	{	m_tabangleD[i+m_nbanglesD]=tabangles[i];
	}
	for(i=0;i<num_freq;i++)
	{	m_tabfreqD[i]=tabfreq[i];
	}
	for(i=0;(i<num_angles)&&(tabangles[i]<=PI);i++) // values above PI are not considered
	{	for(j=0;j<num_freq;j++)
		{	m_directivity[i+m_nbanglesD][j]=tabgain[i][j];
		}
	}

	if(tabangles[i-1]<PI)
	{		// PI radian is missing, we add it
		for(j=0;j<num_freq;j++)
		{	m_directivity[i+m_nbanglesD][j]=m_directivity[i-1+m_nbanglesD][j];
		}
		m_tabangleD[i+m_nbanglesD]=PI;
		m_nbanglesD++;	
	}

	m_nbanglesD+=i;
	m_nbfreqD=num_freq;

	directivityThreeBands(m_directivity3B,m_directivity,m_tabfreqD,
					       m_nbfreqD,m_nbanglesD,m_sf,m_fmin,m_fmax);

}

void CSpatParameters::updateOmniDirectivity(float* tabfreq,float* tabgain,int num_freq)
{
	int i;
	m_nbfreqOD=num_freq;

	for(i=0;i<num_freq;i++)
	{	m_tabfreqOD[i]=tabfreq[i];
	}
	for(i=0;i<num_freq;i++)
	{	m_omnidirectivity[i]=tabgain[i];
	}
	omniDirectivityThreeBands(&m_OmniLow,&m_OmniMid,&m_OmniHigh,m_omnidirectivity,m_tabfreqOD,
							  num_freq,m_sf,m_fmin,m_fmax);
}

//axisDirectivity(float* AxisLow,float* AxisMid,float* AxisHigh,float direct[10][3],float angle[10],int nbangle,float value);

void CSpatParameters::getAxisDirectivity(float* AxisLow,float* AxisMid,float* AxisHigh,float directivityAngle)
{
	axisDirectivity(AxisLow,AxisMid,AxisHigh,m_directivity3B,m_tabangleD,m_nbanglesD,directivityAngle);

}

void CSpatParameters::getEnergeticParameters(float* energetic,float actual_distance)
{
	m_perceptual[0]=m_OD*m_referenceDistance/actual_distance;
	if(m_perceptual[0]>1.f)
		m_perceptual[0]=1.0f;
	Perceptual2energetic(m_perceptual,energetic);
}

void CSpatParameters::setSourcePresence(float value)
{	m_perceptual[0]=value;
	m_OD=m_perceptual[0];
}
	
void CSpatParameters::setSourceWarmth(float value)
{	m_perceptual[1]=value;
}
	
void CSpatParameters::setSourceBrilliance(float value)
{	m_perceptual[2]=value;
}
	
void CSpatParameters::setRoomPresence(float value)
{	m_perceptual[3]=value;
}	

void CSpatParameters::setRunningReverberance(float value)
{	m_perceptual[4]=value;
}
	
void CSpatParameters::setEnvelopment(float value)
{	m_perceptual[5]=value;
}

void CSpatParameters::setLateReverberance(float value)
{	m_perceptual[6]=value;
}
	
void CSpatParameters::setHeavyness(float value)
{	m_perceptual[7]=value;
}
	
void CSpatParameters::setLiveness(float value)
{	m_perceptual[8]=value;
}


void CSpatParameters::setDirectFilterGains(float value1,float value2,float value3)
	{	m_DirectLow=value1;
		m_DirectMid=value2;
		m_DirectHigh=value3;
	};
	void CSpatParameters::setInputFilterGains(float value1,float value2,float value3)
	{	m_InputLow=value1;
		m_InputMid=value2;
		m_InputHigh=value3;
	};
	void CSpatParameters::setRefDistance(float value)
	{	m_referenceDistance=value;
		m_referenceDistance*=m_referenceDistance;
	};
	void CSpatParameters::setFreqLow(float value)	
	{	m_fmin=value;
	};
	void CSpatParameters::setFreqHigh(float value)	
	{	m_fmax=value;
	};
	void CSpatParameters::setTimeLimits1(float value)	
	{	m_l1=value;
	};
	void CSpatParameters::setTimeLimits2(float value)	
	{	m_l2=value;
	};
	void CSpatParameters::setTimeLimits3(float value)	
	{	m_l3=value;
	};
	void CSpatParameters::setModalDensity(float value)	
	{	m_modal=value;
	};
	void CSpatParameters::setSpatialize(Bool value)
	{	m_spatialize=value;		// if TRUE, spatialialization is performed (default)
	}
	void CSpatParameters::setRoomEffect(Bool value)
	{	m_roomEffect=value;		// if TRUE, room effects are performed (default)
	}


/*********************************************************************************/

//*********************************************************************
//
//		AxisDirectivity & OmniDirectivity in DirectiveSound CLASS
//		"Perceptual Approach"
//
//*********************************************************************


//	Changes of 24 April 2000 : JBR
//
//	a) "frequency" field added to DirectiveSound nodes
//	b) 	PerceptualParameters::directFilterGains has changed
//		from SF<Vec3f> to MF<float>
//
//


void CSpatParameters::upDate(DirectiveSound* DS)
{

	int num_angles	= 0;
	int num_freq	= 0;
	int num_gains	= 0;
	int flag=1;
				
	
//  a) AxisDirectivity

	// first : get the numbers of angles 
	int num_params=(DS->angles).GetLength();
	if(num_params> 0)
		num_angles = DS->angles[0];
	else
		flag=0;

	// second : get the number of frequencies : 24 April 2000 JBR
	num_params=(DS->frequency).GetLength();
	if(num_params> 0)
			num_freq = DS->frequency[0];
	else 
		flag=0;
	if(1+num_freq!= num_params)
		flag=0;

	// third : get the number of gains
	num_gains=(DS->directivity).GetLength();
	if(num_gains<=0)
		flag=0;
	if(num_gains!= num_freq*num_angles)
		flag=0;
					
	// fourth : get the angles,the frequencies and the directivity gains
	if(flag)
	{	float* tabangles;
		tabangles=new float[num_angles];

        // PORTING to CC 5.3
        int i = 0;
//		for(int i=0;i<num_angles;i++)
		for( i=0;i<num_angles;i++)
		{   tabangles[i]=DS->angles[i+1];
		}

		float* tabfreq;
		tabfreq=new float[num_freq];

		for(i=0;i<num_freq;i++)
		{   tabfreq[i]=DS->frequency[i+1];
		}

		float** tabgain;
		tabgain=new float*[num_angles];
		for( i=0;i<num_angles;i++)
			tabgain[i]=new float[num_freq];

		for(i=0;i<num_angles;i++)
		{	for(int j=0;j<num_freq;j++)
			{   tabgain[i][j]=DS->directivity[num_freq*i+j];
			}
		}

		updateDirectivity(tabangles,tabfreq,tabgain,num_angles,num_freq);


		delete[] tabangles;
		delete[] tabfreq;

		for(i=0;i<num_angles;i++)
			delete tabgain[i];
		delete[] tabgain;

	}

//  b) OmnniDirectivity
					
	// first : get the number of frequencies
	num_freq = 0;
	flag=1;
	num_params=(DS->perceptualParameters->omniDirectivity).GetLength();
	if(num_params> 0)
		num_freq=DS->perceptualParameters->omniDirectivity[0];
	if(1+num_freq+num_freq != num_params)
		flag=0;
					
	// second : get the frequencies and the directivity gains
	if(flag)
	{	float* tabfreq;
		tabfreq=new float[num_freq];

        // PORTING to CC 5.3
        int i = 0;
//		for(int i=0;i<num_freq;i++)
		for( i=0;i<num_freq;i++)
		{   tabfreq[i]=DS->perceptualParameters->omniDirectivity[i+1];
		}

		float* tabgain;
		tabgain=new float[num_freq];
		int ind=1+num_freq;
		for(i=0;i<num_freq;i++)
		{	tabgain[i]=DS->perceptualParameters->omniDirectivity[ind+i];
		}

		updateOmniDirectivity(tabfreq,tabgain,num_freq);


		delete[] tabfreq;
		delete[] tabgain;

	}
	
	setSourcePresence(DS->perceptualParameters->sourcePresence);
	setSourceWarmth(DS->perceptualParameters->sourceWarmth);
	setSourceBrilliance(DS->perceptualParameters->sourceBrilliance);
	setRoomPresence(DS->perceptualParameters->roomPresence);
	setRunningReverberance(DS->perceptualParameters->runningReverberance);
	setEnvelopment(DS->perceptualParameters->envelopment);
	setLateReverberance(DS->perceptualParameters->lateReverberance);
	setHeavyness(DS->perceptualParameters->heavyness);
	setLiveness(DS->perceptualParameters->liveness);

	// PerceptualParameters::directFilterGains has changed
	// from SF<Vec3f> to MF<float>
	// 24 April 2000 JBR
	{	int nbgains;

		nbgains=DS->perceptualParameters->directFilterGains.GetLength();
		if(nbgains==3){
			setDirectFilterGains(DS->perceptualParameters->directFilterGains[0],
			                     DS->perceptualParameters->directFilterGains[1],
								 DS->perceptualParameters->directFilterGains[2]);
		}
		else
			setDirectFilterGains(1.f,1.f,1.f);

		nbgains=DS->perceptualParameters->inputFilterGains.GetLength();
		if(nbgains==3){
			setInputFilterGains(DS->perceptualParameters->inputFilterGains[0],
			                    DS->perceptualParameters->inputFilterGains[1],
								DS->perceptualParameters->inputFilterGains[2]);
		}
		else
			setInputFilterGains(1.f,1.f,1.f);
	}
							
	setRefDistance(DS->perceptualParameters->refDistance);
	setFreqLow(DS->perceptualParameters->freqLow);
	setFreqHigh(DS->perceptualParameters->freqHigh);
	setTimeLimits1(DS->perceptualParameters->timeLimit1);
	setTimeLimits2(DS->perceptualParameters->timeLimit2);
	setTimeLimits3(DS->perceptualParameters->timeLimit3);
	setModalDensity(DS->perceptualParameters->modalDensity);

	setSpatialize(DS->spatialize);		// if TRUE, spatialialization is performed (default)
	setRoomEffect(DS->roomEffect);		// if TRUE, room effects are performed (default)


}

