// This software module was originally developed by Kevin Barron (KPN Research), 
// Mike Coleman (Five Bats Research), Edward Cooke (Teltec Ireland), In-Dai Ha 
// (Hyundai Electronics Industries, Co.), Hae-Kwan Kim (Hyundai Electronics 
// Industries, Co.), Matthew Leditschke (CSELT), Zvi Lifshitz (VDOnet Corp.), 
// Karl Anders ygard (Telenor R&D), in the course of development of the MPEG-4 
// Systems standard (ISO/IEC 14496-1).  This software module is an 
// implementation of a part of one or more MPEG-4 Systems tools as specified by 
// the MPEG-4 Systems standard.  ISO/IEC gives users of the MPEG-4 Systems 
// standard free license to this software module or modifications thereof for 
// use in hardware or software products claiming conformance to the MPEG-4 
// Systems standard.  Those intending to use this software module in hardware or 
// software products are advised that its use may infringe existing patents.  
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use 
// of this software module or modifications thereof in an implementation.  
// Copyright is not released for non MPEG-4 Systems conforming products.  The 
// original developer retains full right to use the code for his/her own 
// purpose, assign or donate the code to a third party and to inhibit third 
// parties from using the code for non MPEG-4 Systems conforming products.  This 
// copyright notice must be included in all copies or derivative works.
//
// Copyright 1997.


#include "OpenGl/OpenGl.h"
#include "Presenter.h"
#include "AppearanceProxy.h"
#include "MaterialNodeProxy.h"
#include "TextureNodeProxy.h"
#include "TextureTransformProxy.h"


// enable appearance

BOOL
AppearanceProxy::Enable(OpenGLVisualRenderer* pRenderer, DWORD dwTime)
{
  /* Appearance* pAppearance = (Appearance*) GetMediaObject(); */

  if (/* pAppearance-> */ GetMaterial())
    ((MaterialNodeProxy*) (/* pAppearance-> */ GetMaterial()->GetRenderingObject()))->Enable(pRenderer, dwTime);

  if (/* pAppearance-> */ GetTexture())
    ((TextureNodeProxy*) (/* pAppearance-> */ GetTexture()->GetRenderingObject()))->Enable(pRenderer, dwTime);

  if (/* pAppearance-> */ GetTextureTransform())
    ((TextureTransformProxy*) (/* pAppearance-> */ GetTextureTransform()->GetRenderingObject()))->Enable(dwTime);

  return TRUE;
}

// disable appearance

BOOL
AppearanceProxy::Disable(OpenGLVisualRenderer* pRenderer, DWORD dwTime)
{
  /* Appearance* pAppearance = (Appearance*) GetMediaObject(); */

  if (/* pAppearance-> */ GetMaterial())
    ((MaterialNodeProxy*) (/* pAppearance-> */ GetMaterial()->GetRenderingObject()))->Disable(pRenderer, dwTime);

  if (/* pAppearance-> */ GetTexture())
    ((TextureNodeProxy*) (/* pAppearance-> */ GetTexture()->GetRenderingObject()))->Disable(pRenderer, dwTime);

  if (/* pAppearance-> */ GetTextureTransform())
    ((TextureTransformProxy*) (/* pAppearance-> */ GetTextureTransform()->GetRenderingObject()))->Disable(dwTime);

  return TRUE;
}

// Render object description

/* virtual */ void
AppearanceProxy::RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD dwTime)
{
  /* Appearance* pAppearance = (Appearance*) GetMediaObject(); */

  int children = ((/* pAppearance-> */ GetMaterial() ? 1 : 0) +
		  (/* pAppearance-> */ GetTexture() ? 1 : 0) + 
		  (/* pAppearance-> */ GetTextureTransform() ? 1 : 0));

  GLfloat color[] = { 1.0f, 1.0f, 0.0f, 0.75f};

  // SOLARIS PORTING
  //::glColor4fv(color);
  glColor4fv(color);
  // SOLARIS PORTING - END

  RenderDescriptionBox(pRenderer, "Appearance");

  ::glPushMatrix();

  ::glScalef(1.0f / children, 1.0f, 1.0f);
  ::glTranslatef(0.0f, -1.0f, 0.0f);

  if (/* pAppearance-> */ GetMaterial()) {
    ((MaterialNodeProxy*) /* pAppearance-> */ GetMaterial()->GetRenderingObject())->RenderDescription(pRenderer, dwTime);
    
    ::glTranslatef(1.0f, 0.0f, 0.0f);
  }

  if (/* pAppearance-> */ GetTexture()) {
    ((TextureNodeProxy*) /* pAppearance-> */GetTexture()->GetRenderingObject())->RenderDescription(pRenderer, dwTime);
    
    ::glTranslatef(1.0f, 0.0f, 0.0f);
  }
  
  if (/* pAppearance-> */GetTextureTransform())
    ((TextureTransformProxy*) /* pAppearance-> */GetTextureTransform()->GetRenderingObject())->RenderDescription(pRenderer, dwTime);

  ::glPopMatrix();
}
