// -*- c++ -*-

// This software module was originally developed by Karl Anders ygard
// (Telenor R&D) in the course of development of the MPEG-4 Systems
// standard (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// Copyright 1997, 1998.

#include "OpenGl/OpenGl.h"
#include "ImageTextureProxy.h"
#include "opengl/base/OpenGLVisualRenderer.h"
#include "Stream.h"


// Render object description

/* virtual */ void
ImageTextureProxy::RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD /* dwTime */)
{
  RenderDescriptionBox(pRenderer, "ImageTexture");
}

// Enable texture

/* virtual */ BOOL
ImageTextureProxy::Enable(OpenGLVisualRenderer* pRenderer, DWORD dwTime)
{
  ImageTexture* poTexture = (ImageTexture*) GetMediaObject();
  MediaStream* poStream = poTexture->GetStream();

  // SB $$$$ 99/03/10 Start Stream and Register Channel
  if (m_bFirst)
  {
    Start();
    poTexture->GetExecutive()->RequestChannels();
    m_bFirst = FALSE;
    TRACE("[ImageTextureProxy::Enable] StreamConsumer started \n");
  }
    
  if (poStream) {
    int nSize;
    // DWORD dwTime;
    BOOL bEOU;
    int nStatus;
    BOOL bSkip;
    BOOL bWait;
    long lDelta;
    BOOL bDontClose;
    int nRecurrent;
    
    // SB $$$$ substituted by Register Channel above
    // pRenderer->RegisterStreamConsumer(poTexture);
    
    /***
    unsigned char* pData = (unsigned char*) poStream->Fetch(
    nSize,
				dwTime,
        nRecurrent,
        TRUE,
        FALSE);
    ***/
    bSkip = TRUE;
    bWait = FALSE;
    lDelta = 0;
    bDontClose = TRUE;
    unsigned char * pData = (unsigned char *) poStream->Fetch(
      nSize, dwTime, bEOU, nStatus, bSkip, bWait, lDelta, bDontClose);
    nRecurrent = FALSE;
    //TRACE("[ImageTextureProxy::Enable] Fetch returned Size %4d \n", nSize);
    
    if (pData != END_OF_STREAM) {
      if (pData)
        if (!nRecurrent) {
          int nWidth = (((unsigned short) pData[0]) << 8) | pData[1];
          int nHeight = (((unsigned short) pData[2]) << 8) | pData[3];
          BOOL bRet;

		  //ML - added this check
		  if ((nWidth == 0) || (nHeight == 0))
		  {
			TRACE("[ImageTextureProxy::Enable] image dimensions are %d x %d\n",
				  nWidth, nHeight);
			return TRUE;
		  }

          // enable texture object
          
          // (MediaFormat) (MediaFormat :: RGBA)
          bRet = TextureNodeProxy::Enable(pRenderer,
            poTexture->GetStreamFormat() == MediaFormat::RGBA ? RGBA : RGB,
            nWidth, nHeight, pData + 4);
          
          poStream->Release(TRUE);
          
          return bRet;
        } else
          poStream->Release(TRUE);
    }
    
    return TextureNodeProxy::Enable(pRenderer);
  }
  
  return TRUE;
}
