// This software module was originally developed by Karl Anders ygard
// (Telenor R&D) in the course of development of the MPEG-4 Systems
// standard (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// Copyright 1997, 1998.

#include "OpenGl/OpenGl.h"
#include "Material2DProxy.h"


// Render object description

/* virtual */ void
Material2DProxy::RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD dwTime)
{
  // Material2D* pMaterial = (Material2D*) GetMediaObject();

  int children = (/* pMaterial-> */ GetLineProps() ? 1 : 0);

  GLfloat color[] = { 1.0f, 1.0f, 0.0f, 0.75f};

  //satish changed this line for compilation purposes
        //have to be fixed afterwards

  //::glColor4fv(color);
  glColor4fv(color);

  RenderDescriptionBox(pRenderer, "Material2D");

  ::glPushMatrix();

  ::glScalef(1.0f / children, 1.0f, 1.0f);
  ::glTranslatef(0.0f, -1.0f, 0.0f);

  if (/* pMaterial-> */ GetLineProps()) {
    ((LinePropertiesProxy*) /* pMaterial-> */ GetLineProps()->GetRenderingObject())->RenderDescription(pRenderer, dwTime);
    
    ::glTranslatef(1.0f, 0.0f, 0.0f);
  }
  
  ::glPopMatrix();
}

// Enable material2D

BOOL
Material2DProxy::Enable(OpenGLVisualRenderer* pRenderer, DWORD dwTime)
{
  // Material2D* pMaterial = (Material2D*) GetMediaObject();

  float transparency = /* pMaterial-> */ GetTransparency();
  const Color3f& emissiveColor = /* pMaterial-> */ GetEmissiveColor();

  //satish changed this line for compilation purposes
        //have to be fixed afterwards

  //::glColor3fv(emissiveColor);
  glColor3fv(emissiveColor);

  ::glPushAttrib(GL_POLYGON_BIT);

  if (! /* pMaterial-> */ GetFilled())
    ::glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);

  if (/* pMaterial-> */ GetLineProps())
    ((LinePropertiesProxy*) /* pMaterial-> */ GetLineProps()->GetRenderingObject())->Enable(pRenderer, dwTime);
  
  // set screendoor transparency

  if (transparency > 0.0f)
    if (transparency < 1.0f) {
      ::glEnable(GL_POLYGON_STIPPLE);
      ::glPolygonStipple((const GLubyte*) stippleMask[16 - int(transparency * 16.0f)]);
    } else {
      ::glColorMask(GL_FALSE, GL_FALSE, GL_FALSE, GL_FALSE);
      ::glDepthMask(GL_FALSE);
    }

  pRenderer->SetCurrentMaterial2D(this);

#ifdef _DEBUG
  if (pRenderer->CheckGLError("Material2DProxy:")) {
    TRACE("emissiveColor: %f, %f, %f\n",
	     emissiveColor.r, emissiveColor.g, emissiveColor.b);
    TRACE("transparency: %f\n",
	     transparency);
    TRACE("Material2DProxy::Enable() [%08x]: ", this);
  }
#endif

  return TRUE;
}
