// -*- c++ -*-

// This software module was originally developed by Karl Anders ygard
// (Telenor R&D) in the course of development of the MPEG-4 Systems
// standard (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// Copyright 1997.

#ifndef __MATERIAL2DPROXY_H__
#define __MATERIAL2DPROXY_H__

#include "MaterialNodeProxy.h"

class Material2D;

/// 2D material renderer class

class Material2DProxy
  : public MaterialNodeProxy
{
public:

  /// Constructor

  Material2DProxy(Material2D* pMaterial2D);

  /// Enable material

  virtual BOOL		Enable(OpenGLVisualRenderer* pRenderer, DWORD dwTime);

  /// Disable material

  virtual BOOL		Disable(OpenGLVisualRenderer* pRenderer, DWORD dwTime);

  /// Render object description

  virtual void		RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD dwTime);

  // Stefano Battista
  // moved methods from Material2D to Material2DProxy
public:
  inline Material2D * GetMaterial2D() const
  { return (Material2D *) GetMediaObject(); }
  /// Set emissive color of material2D
  inline void SetEmissiveColor(const SF<Color3f>& rEmissiveColor)
  { GetMaterial2D()->emissiveColor = rEmissiveColor; }
  /// Get emissive color of material2D
  inline const SF<Color3f>& GetEmissiveColor() const
  { return GetMaterial2D()->emissiveColor; }
  /// Set whether primitives are filled
  inline void SetFilled(bool bFilled)
  { GetMaterial2D()->filled = bFilled; }
  /// Get whether primitives are filled
  inline BOOL GetFilled() const
  { return (Bool) GetMaterial2D()->filled; }
  /// Set transparency of material2D
  inline void SetTransparency(float nTransparency)
  { GetMaterial2D()->transparency = nTransparency; }
  /// Get transparency of material2D
  inline float GetTransparency() const
  { return GetMaterial2D()->transparency; }
  /// Get line properties
  inline LineProperties * GetLineProps() const
  { return (LineProperties * )( (GetMaterial2D()->lineProps).GetNode() ); }

};

#include "Material2DProxy.i"

#endif // __MATERIAL2DPROXY_H__
