// This software module was originally developed by Karl Anders ygard
// (Telenor R&D) in the course of development of the MPEG-4 Systems
// standard (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// Copyright 1997, 1998.

#include "OpenGl/OpenGl.h"
#include "MaterialProxy.h"
#include "opengl/base/OpenGLVisualRenderer.h"


// Render object description

/* virtual */ void
MaterialProxy::RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD /* dwTime */)
{
  RenderDescriptionBox(pRenderer, "Material");
}

// enable material

BOOL
MaterialProxy::Enable(OpenGLVisualRenderer* pRenderer, DWORD /* dwTime */)
{
  // Material* pMaterial = (Material*) GetMediaObject();
	
  float ambientIntensity = /* pMaterial-> */ GetAmbientIntensity();
  float shininess = /* pMaterial-> */ GetShininess();
  float transparency = /* pMaterial-> */ GetTransparency();
  const Color3f& diffuseColor = /* pMaterial-> */ GetDiffuseColor();
  const Color3f& specularColor = /* pMaterial-> */ GetSpecularColor();
  const Color3f& emissiveColor = /* pMaterial-> */ GetEmissiveColor();
	
	
	/*--------------------------------------------------------------------------
	set render material yes/no decision for multipass shadowrendering
	- added by Helge Drumm 11-2005 (original code below!!)
	--------------------------------------------------------------------------*/
	unsigned int materialState = pRenderer->GetMaterialEnabled();
	
	// normal situation with object colors
	if (materialState != 0)
	{
  GLfloat ambient[4] = {diffuseColor.r * ambientIntensity,
			diffuseColor.g * ambientIntensity,
			diffuseColor.b * ambientIntensity,
			1.0f};

  GLfloat specular[4] = {specularColor.r,
			 specularColor.g,
			 specularColor.b,
			 1.0f};

  GLfloat emissive[4] = {emissiveColor.r,
			 emissiveColor.g,
			 emissiveColor.b,
			 1.0f};

  ::glPushAttrib(GL_CURRENT_BIT);

  ::glEnable(GL_LIGHTING);
  ::glEnable(GL_COLOR_MATERIAL);

  ::glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, ambient);

  // SOLARIS PORTING
  //::glColor3fv(diffuseColor);
  glColor3fv(diffuseColor);
  // SOLARIS PORTING - END

  ::glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, specular);
  ::glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, emissive);
  ::glMaterialf(GL_FRONT_AND_BACK, GL_SHININESS, shininess * 128.0f);

  // emissive color is used for drawing lines

  //::glColor3fv(emissiveColor);

  // set screendoor transparency
	
	}
	// for rendering lightmaps and shadowmaps with objectcolor = white
	else
		{
		unsigned int emissiveState = pRenderer->GetEmissiveEnabled();

		GLfloat specular[4] = {specularColor.r,
			 specularColor.g,
			 specularColor.b,
			 1.0f};
		
		GLfloat emissive[4] = {emissiveColor.r,
			 emissiveColor.g,
			 emissiveColor.b,
			 1.0f};
		
		if (emissiveState != 0)
		{
			

			if (emissiveColor.r>0 || emissiveColor.g>0 || emissiveColor.b>0)
			{
				::glPushAttrib(GL_CURRENT_BIT);

				::glEnable(GL_COLOR_MATERIAL);

				glColor3fv(emissive);

				::glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, emissive);
				::glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, specular);
				::glMaterialf(GL_FRONT_AND_BACK, GL_SHININESS, shininess * 128.0f);
			}
			else
			{
				::glPushAttrib(GL_CURRENT_BIT);
				::glEnable(GL_COLOR_MATERIAL);
				glColor3f(1,1,1);
				// SOLARIS PORTING - END
				
				::glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, emissive);
				::glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, specular);
				::glMaterialf(GL_FRONT_AND_BACK, GL_SHININESS, shininess * 128.0f);
			}
		}
		else
		{
			::glPushAttrib(GL_CURRENT_BIT);

			//::glEnable(GL_LIGHTING);
			::glEnable(GL_COLOR_MATERIAL);

			//::glMaterialf(GL_FRONT_AND_BACK, GL_AMBIENT, 1, 1, 1, 1);

			// SOLARIS PORTING
			//::glColor3fv(diffuseColor);
			glColor3f(1,1,1);
			// SOLARIS PORTING - END

			::glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, specular);
			::glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, emissive);
			::glMaterialf(GL_FRONT_AND_BACK, GL_SHININESS, shininess * 128.0f);
		}
		
	//	
	}

// -------------------------------------------------------------------------

/* original code

	GLfloat ambient[4] = {diffuseColor.r * ambientIntensity,
			diffuseColor.g * ambientIntensity,
			diffuseColor.b * ambientIntensity,
			1.0f};

  GLfloat specular[4] = {specularColor.r,
			 specularColor.g,
			 specularColor.b,
			 1.0f};

  GLfloat emissive[4] = {emissiveColor.r,
			 emissiveColor.g,
			 emissiveColor.b,
			 1.0f};

  ::glPushAttrib(GL_CURRENT_BIT);

  ::glEnable(GL_LIGHTING);
  ::glEnable(GL_COLOR_MATERIAL);

  ::glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, ambient);

  // SOLARIS PORTING
  //::glColor3fv(diffuseColor);
  glColor3fv(diffuseColor);
  // SOLARIS PORTING - END

  ::glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, specular);
  ::glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, emissive);
  ::glMaterialf(GL_FRONT_AND_BACK, GL_SHININESS, shininess * 128.0f);

  // emissive color is used for drawing lines

  //::glColor3fv(emissiveColor);

  // set screendoor transparency
*/


  if (transparency > 0.0f)
    if (transparency < 1.0f) {
      ::glEnable(GL_POLYGON_STIPPLE);
      ::glPolygonStipple((const GLubyte*) stippleMask[16 - int(transparency * 16.0f)]);
    } else {
      ::glColorMask(GL_FALSE, GL_FALSE, GL_FALSE, GL_FALSE);
      ::glDepthMask(GL_FALSE);
    }

#ifdef _DEBUG
  if (pRenderer->CheckGLError("MaterialProxy::Enable(): GL Error ")) {
    TRACE("diffuseColor: %f, %f, %f\n",
	     diffuseColor.r, diffuseColor.g, diffuseColor.b);
    TRACE("specularColor: %f, %f, %f\n",
	     specularColor.r, specularColor.g, specularColor.b);
    TRACE("emissiveColor: %f, %f, %f\n",
	     emissiveColor.r, emissiveColor.g, emissiveColor.b);
    TRACE("shininess: %f, ambientIntensity: %f, transparency: %f\n",
	     shininess, ambientIntensity, transparency);
  }
#endif

  return TRUE;
}
