// -*- c++ -*-

// This software module was originally developed by Karl Anders ygard
// (Telenor R&D) in the course of development of the MPEG-4 Systems
// standard (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// Copyright 1997, 1998.

#ifndef __MATERIALPROXY_H__
#define __MATERIALPROXY_H__

#include "MaterialNodeProxy.h"

class Material;

/// Material renderer class

class MaterialProxy
  : public MaterialNodeProxy
{
 public:

	 unsigned int materialState;
	 unsigned int emissiveState;
  /// Constructor

			MaterialProxy(Material* pMaterial);

  /// Enable material

  virtual BOOL		Enable(OpenGLVisualRenderer* pRenderer, DWORD dwTime);

  /// Disable material

  virtual BOOL		Disable(OpenGLVisualRenderer* pRenderer, DWORD dwTime);

  /// Render object description

  virtual void		RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD dwTime);

  // Stefano Battista
  // moved methods from Material to MaterialProxy
public:
  /// Get pointer to related Material object
  inline Material * GetMaterial() const
  { return (Material *) GetMediaObject(); }
  /// Set ambient intensity of material
  inline void SetAmbientIntensity(float nAmbientIntensity)
  { GetMaterial()->ambientIntensity = nAmbientIntensity; }
  /// Get ambient intensity of material
  inline float GetAmbientIntensity() const
  { return GetMaterial()->ambientIntensity; }
  /// Set diffuse color of material
  inline void SetDiffuseColor(const SF<Color3f>& rDiffuseColor)
  { GetMaterial()->diffuseColor = rDiffuseColor; }
  /// Get diffuse color of material
  inline const SF<Color3f>& GetDiffuseColor() const
  { return GetMaterial()->diffuseColor; }
  /// Set emissive color of material
  inline void SetEmissiveColor(const SF<Color3f>& rEmissiveColor)
  { GetMaterial()->emissiveColor = rEmissiveColor; }
  /// Get emissive color of material
  inline const SF<Color3f>& GetEmissiveColor() const
  { return GetMaterial()->emissiveColor; } 
  /// Set shininess of material
  inline void SetShininess(float nShininess)
  { GetMaterial()->shininess = nShininess; }
  /// Get shininess of material
  inline float GetShininess() const
  { return GetMaterial()->shininess; }
  /// Set specular color of material
  inline void SetSpecularColor(const SF<Color3f>& rSpecularColor)
  { GetMaterial()->specularColor = rSpecularColor; }
  /// Get specular color of material
  inline const SF<Color3f>& GetSpecularColor() const
  { return GetMaterial()->specularColor; }
  /// Set transparency of material
  inline void SetTransparency(float nTransparency)
  { GetMaterial()->transparency = nTransparency; }
  /// Get transparency of material
  inline float GetTransparency() const
  { return GetMaterial()->transparency; }
};

#include "MaterialProxy.i"

#endif // __MATERIALPROXY_H__
