// -*- c++ -*-

// This software module was originally developed by Karl Anders ygard
// (Telenor R&D) in the course of development of the MPEG-4 Systems
// standard (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
	// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// Copyright 1997, 1998.

#include "OpenGl/OpenGl.h"
#include "MovieTextureProxy.h"
#include "opengl/base/OpenGLPresenter.h"
#include "opengl/base/OpenGLVisualRenderer.h"

#include "Stream.h"
//#include "Executiv.h"
#include "Presenter.h"

#ifdef _SAIT_3DTG_MP4VIDEO_
#include "../../2DPlayer-CSELT/clipping.h"
#include <fstream.h>

void ConvertYUVtripleToRGB(int y, int u, int v, int& r, int& g, int& b)
{
  int tempY = y;
  tempY <<= 16;

  r = Clip((tempY              +  91881 * v - 11760828) >> 16);
  g = Clip((tempY -  22554 * u -  46802 * v +  8877496) >> 16);
  b = Clip((tempY + 116130 * u              - 14864613) >> 16);
}

void ConvertYUV2RGB(int nWidth, int nHeight, unsigned char* pYUVFrame, unsigned char* pRGBFrame)
{
	int x = 0;
	int y = 0;
	unsigned char* pYFrame = pYUVFrame;
	unsigned char* pUFrame = pYUVFrame + nWidth * nHeight;
	unsigned char* pVFrame = pYUVFrame + nWidth * nHeight * 5 / 4;

	for(y = 0; y < nHeight; y+=2)
	{
		for(x = 0; x < nWidth; x+=2)
		{
			int Y1 = pYFrame[nWidth*y+x];
			int Y2 = pYFrame[nWidth*y+x+1];
			int Y3 = pYFrame[nWidth*(y+1)+x];
			int Y4 = pYFrame[nWidth*(y+1)+x+1];

			int U = pUFrame[nWidth*y/4+x/2];
			int V = pVFrame[nWidth*y/4+x/2];

			int red = 0;
			int green = 0;
			int blue = 0;

			ConvertYUVtripleToRGB(Y1, U, V, red, green, blue);
			pRGBFrame[y*nWidth*3+x*3] = red;
			pRGBFrame[y*nWidth*3+x*3+1] = green;
			pRGBFrame[y*nWidth*3+x*3+2] = blue;

			ConvertYUVtripleToRGB(Y2, U, V, red, green, blue);
			pRGBFrame[y*nWidth*3+x*3+3] = red;
			pRGBFrame[y*nWidth*3+x*3+4] = green;
			pRGBFrame[y*nWidth*3+x*3+5] = blue;

			ConvertYUVtripleToRGB(Y3, U, V, red, green, blue);
			pRGBFrame[(y+1)*nWidth*3+x*3] = red;
			pRGBFrame[(y+1)*nWidth*3+x*3+1] = green;
			pRGBFrame[(y+1)*nWidth*3+x*3+2] = blue;

			ConvertYUVtripleToRGB(Y4, U, V, red, green, blue);
			pRGBFrame[(y+1)*nWidth*3+x*3+3] = red;
			pRGBFrame[(y+1)*nWidth*3+x*3+4] = green;
			pRGBFrame[(y+1)*nWidth*3+x*3+5] = blue;
		}
	}
}

void ConvertYUV2RGB2(int nWidth, int nHeight, unsigned char* pYUVFrame, unsigned char* pRGBFrame)
{
	int x = 0;
	int y = 0;
	unsigned char* pYFrame = pYUVFrame;
	unsigned char* pUFrame = pYUVFrame + nWidth * nHeight;
	unsigned char* pVFrame = pYUVFrame + nWidth * nHeight * 5 / 4;
	unsigned char* pRFrame = pRGBFrame;
	unsigned char* pGFrame = pRGBFrame + nWidth * nHeight;
	unsigned char* pBFrame = pRGBFrame + nWidth * nHeight * 2;

	for(y = 0; y < nHeight; y+=2)
	{
		for(x = 0; x < nWidth; x+=2)
		{
			int Y1 = pYFrame[nWidth*y+x];
			int Y2 = pYFrame[nWidth*y+x+1];
			int Y3 = pYFrame[nWidth*(y+1)+x];
			int Y4 = pYFrame[nWidth*(y+1)+x+1];

			int U = pUFrame[nWidth*y/4+x/2];
			int V = pVFrame[nWidth*y/4+x/2];

			int red = 0;
			int green = 0;
			int blue = 0;

			ConvertYUVtripleToRGB(Y1, U, V, red, green, blue);
			pRFrame[y*nWidth+x] = red;
			pGFrame[y*nWidth+x] = green;
			pBFrame[y*nWidth+x] = blue;

			ConvertYUVtripleToRGB(Y2, U, V, red, green, blue);
			pRFrame[y*nWidth+x+1] = red;
			pGFrame[y*nWidth+x+1] = green;
			pBFrame[y*nWidth+x+1] = blue;

			ConvertYUVtripleToRGB(Y3, U, V, red, green, blue);
			pRFrame[(y+1)*nWidth+x] = red;
			pGFrame[(y+1)*nWidth+x] = green;
			pBFrame[(y+1)*nWidth+x] = blue;

			ConvertYUVtripleToRGB(Y4, U, V, red, green, blue);
			pRFrame[(y+1)*nWidth+x+1] = red;
			pGFrame[(y+1)*nWidth+x+1] = green;
			pBFrame[(y+1)*nWidth+x+1] = blue;
		}
	}
}
#endif _SAIT_3DTG_MP4VIDEO_

// Render object description

/* virtual */ void
MovieTextureProxy::RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD /* dwTime */)
{
  RenderDescriptionBox(pRenderer, "MovieTexture");
}

// Enable texture

/* virtual */ BOOL
MovieTextureProxy::Enable(
    OpenGLVisualRenderer * pRenderer, 
    DWORD /* dwTime */)
{
  MovieTexture* pTexture = (MovieTexture*) GetMediaObject();
  MediaStream* pStream = pTexture->GetStream();

  if (pStream)
  {
    TestActive();
    if (GetActive())
    {
      int nSize;
      DWORD dwTime;
      BOOL bEOU;
      int nStatus;
      BOOL bSkip;
      BOOL bWait;
      long lDelta;
      BOOL bDontClose;
      BOOL bRecurrent;


      // SB @@@@ I am calling Fetch without using nRecurrent!!!
      // what is th edefault? guess: FALSE
      bSkip = TRUE;
      bWait = FALSE;
      lDelta = 0;
      bDontClose = TRUE;
      unsigned char * pData = (unsigned char *) pStream->Fetch(
        nSize, dwTime, bEOU, nStatus, bSkip, bWait, lDelta, bDontClose);
      bRecurrent = FALSE;

      if (pData != END_OF_STREAM) {
        if (pData)
          if (!bRecurrent) {
#ifndef _SAIT_3DTG_MP4VIDEO_
            int nWidth = (((unsigned short) pData[0]) << 8) | pData[1];
            int nHeight = (((unsigned short) pData[2]) << 8) | pData[3];
            BOOL bRet;

            // Enable texture object

            bRet = TextureNodeProxy::Enable(pRenderer,
              pTexture->GetStreamFormat() == /* FORMAT_RGBA */ MediaFormat::RGBA ? RGBA : RGB,
              nWidth, nHeight, pData + 4);

            pStream->Release(TRUE);

#else _SAIT_3DTG_MP4VIDEO_
			// return value
			BOOL bRet = false;

			// check nStatus
			if(nStatus != 0)
				return false;

			// header information
			m_nNumberOfFrame = ((int*)pData)[0];
			if(m_nNumberOfFrame < m_nPreviousFrame)
			{
				// header information
				m_nWidth = ((int*)pData)[1];
				m_nHeight = ((int*)pData)[2];
				m_nNumberOfChannels = ((int*)pData)[3];
				m_nBytePerPixel = ((int*)pData)[4];

				// frame information
				m_nRGBFrameSize = m_nWidth * m_nHeight * 3;
				m_nYUVFrameSize = m_nWidth * m_nHeight * 3 / 2;
				m_nCurrentFrame = 0;
				m_nPreviousFrame = 0;

				// video buffer
				m_pVideo = NULL;
			}
			else if(m_nNumberOfFrame > m_nPreviousFrame)
			{
				// check frame
				if(m_nCurrentFrame+1 >= m_nNumberOfFrame)
					m_nCurrentFrame = m_nNumberOfFrame-1;
				else if(m_nCurrentFrame+1 < m_nNumberOfFrame)
					m_nCurrentFrame++;
			}
			else
				return false;

			// video buffer
			m_pVideo = (unsigned char*)realloc((void*)m_pVideo, m_nNumberOfFrame*m_nYUVFrameSize);
			if(!m_pVideo) return false;

			memcpy(m_pVideo + m_nPreviousFrame*m_nYUVFrameSize,
					pData+sizeof(int)*5 + m_nPreviousFrame*m_nYUVFrameSize,
					(m_nNumberOfFrame-m_nPreviousFrame)*m_nYUVFrameSize);

			// convert yuv to rgb
			unsigned char* pRGBFrame = (unsigned char*)malloc(m_nRGBFrameSize);
			ConvertYUV2RGB(m_nWidth, m_nHeight, m_pVideo + m_nCurrentFrame*m_nYUVFrameSize, pRGBFrame);

			// enable
			bRet = TextureNodeProxy::Enable(pRenderer,
				pTexture->GetStreamFormat() == /* FORMAT_RGBA */ MediaFormat::YUV ? RGB : RGBA,
				m_nWidth, m_nHeight, pRGBFrame);

			/*// dump
			ofstream fout_frame;
			ofstream fout_rgb;
			if(m_bDump == false)
			{
				fout_frame.open("FrameCount.txt");
				fout_frame << "number of frame : current frame" << endl;

				fout_rgb.open("RGB-3D.bin", ios::binary);
				m_bDump = true;
			}
			else
			{
				fout_frame.open("FrameCount.txt", ios::app);
				fout_rgb.open("RGB-3D.bin", ios::binary | ios::app);
			}
			fout_frame << m_nNumberOfFrame << " : ";
			fout_frame << m_nCurrentFrame << " (in)" << endl;
			fout_frame.close();

			ConvertYUV2RGB2(m_nWidth, m_nHeight, m_pVideo + m_nCurrentFrame*m_nYUVFrameSize, pRGBFrame);

			fout_rgb.write(pRGBFrame, m_nRGBFrameSize);
			fout_rgb.close();
			//*///

			// previuos frame
			m_nPreviousFrame = m_nNumberOfFrame;

			// free
			free(pRGBFrame);

			pStream->Release(TRUE);
#endif _SAIT_3DTG_MP4VIDEO_

            return bRet;
          } else
            pStream->Release(TRUE);
      }
      else
      {
        if (GetLoop())
        {
#ifdef _SAIT_3DTG_MP4VIDEO_
			// check frame
			if(m_nCurrentFrame+1 >= m_nNumberOfFrame)
			{
				m_nCurrentFrame = 0;
				m_nPreviousFrame = 0;
				m_bDump = false;
			}
#endif _SAIT_3DTG_MP4VIDEO_

          pTexture->CloseStream();
        }

#ifdef _SAIT_3DTG_MP4VIDEO_
		if(m_nCurrentFrame+1 < m_nNumberOfFrame)
		{
			m_nCurrentFrame++;

			// convert yuv to rgb
			unsigned char* pRGBFrame = (unsigned char*)malloc(m_nRGBFrameSize);
			ConvertYUV2RGB(m_nWidth, m_nHeight, m_pVideo + m_nCurrentFrame*m_nYUVFrameSize, pRGBFrame);
			
			// enable
			bool bRet = TextureNodeProxy::Enable(pRenderer,
				pTexture->GetStreamFormat() == /* FORMAT_RGBA */ MediaFormat::YUV ? RGB : RGBA,
				m_nWidth, m_nHeight, pRGBFrame);
			
			/*// dump
			if(m_bDump == true)
			{
				ofstream fout_frame;
				fout_frame.open("FrameCount.txt", ios::app);
				fout_frame << m_nNumberOfFrame << " : ";
				fout_frame << m_nCurrentFrame << " (out)" << endl;
				fout_frame.close();
				
				ConvertYUV2RGB2(m_nWidth, m_nHeight, m_pVideo + m_nCurrentFrame*m_nYUVFrameSize, pRGBFrame);
				
				ofstream fout_rgb;
				fout_rgb.open("RGB-3D.bin", ios::binary | ios::app);
				fout_rgb.write(pRGBFrame, m_nRGBFrameSize);
				fout_rgb.close();
			}
			//*///
			
			// free
			free(pRGBFrame);
		}
#endif _SAIT_3DTG_MP4VIDEO_
      }
    }
    
    return TextureNodeProxy::Enable(pRenderer);
  }
  
  return TRUE;
}


// Stefano Battista
// moved from MovieTexture to Proxy

// Test whether movie should be activated or deactivated

void MovieTextureProxy::TestActive()
{
  MovieTexture * poMovieTexture = GetMovieTexture();
  Time currentTime = poMovieTexture->GetRoot()->GetTime() / 1000.0;

  if (poMovieTexture->isActive.GetValue())
  {
    if ((Time) poMovieTexture->startTime > currentTime)
    {
      Stop();
    }
    else
    {
      if (!(Bool) poMovieTexture->loop)
      {
        if ((Time) poMovieTexture->stopTime <= currentTime && 
            poMovieTexture->stopTime > poMovieTexture->startTime)
        {
          Stop();
        }
      }
    }
  }
  else
  {
    if ((Time) poMovieTexture->startTime <= currentTime)
    {
      if (poMovieTexture->stopTime < poMovieTexture->startTime)
      {
	        Start();
      }
      else
      {
        if ((Bool) poMovieTexture->loop && 
            poMovieTexture->stopTime == poMovieTexture->startTime)
        {
          Start();
        }
        else
        {
          if (poMovieTexture->stopTime > poMovieTexture->startTime && 
              poMovieTexture->stopTime > currentTime)
          {
            Start();
          }
        }
      }
    }
  }
}


// Start movie

void MovieTextureProxy::Start()
{
#ifndef BIFSENC
  MovieTexture * poMovieTexture = GetMovieTexture();
  poMovieTexture->isActive = TRUE;
  if (poMovieTexture->GetStream())
  {
    OpenGLPresenter * poPresenter;
    poPresenter = (OpenGLPresenter *) poMovieTexture->GetExecutive()->GetPresenter();
    poPresenter->GetVisualRenderer()->RegisterStreamConsumer(poMovieTexture);

	//JLF 04/00
	poMovieTexture->GetExecutive()->RequestChannels();
	TRACE("[MovieTextureProxy::Enable] StreamConsumer started \n");
  
  }
#endif
}


// Stop movie

void MovieTextureProxy::Stop()
{
#ifndef BIFSENC
  MovieTexture * poMovieTexture = GetMovieTexture();
  poMovieTexture->isActive = FALSE;
  {
    OpenGLPresenter * poPresenter;
    poPresenter = (OpenGLPresenter *) poMovieTexture->GetExecutive()->GetPresenter();
    poPresenter->GetVisualRenderer()->RegisterStreamConsumer(poMovieTexture);
  }
  poMovieTexture->CloseStream();
  // PauseStream();
#endif
}
