// -*- c++ -*-

// This software module was originally developed by Karl Anders ygard
// (Telenor R&D) in the course of development of the MPEG-4 Systems
// standard (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// Copyright 1997.

#ifndef __MOVIETEXTUREPROXY_H__
#define __MOVIETEXTUREPROXY_H__

#include "TextureNodeProxy.h"

class MovieTexture;

/// Movie texture renderer class

class MovieTextureProxy
  : public TextureNodeProxy
{
public:

  /// Constructor

  MovieTextureProxy(MovieTexture* pMovieTexture);

  /// Render object description

  virtual void		RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD dwTime);

  /// Enable texture

  virtual BOOL		Enable(OpenGLVisualRenderer* pRenderer, DWORD dwTime);

  /// Disable texture

  virtual BOOL		Disable(OpenGLVisualRenderer* pRenderer, DWORD dwTime);
  
  // Stefano Battista
  // moved from MovieTexture to Proxy
public:
  inline MovieTexture * GetMovieTexture()
  { return (MovieTexture *) GetMediaObject(); }
  /// Get whether video is active
  inline BOOL GetActive() /* const */
  { return (Bool) GetMovieTexture()->isActive; }
  /// Get whether video should loop
  inline BOOL GetLoop() /* const */
  { return (Bool) GetMovieTexture()->loop; }

  /// Test whether movie should be activated or deactivated
  void TestActive();
  // Start movie
  void Start();
  // Stop movie
  void Stop();

#ifdef _SAIT_3DTG_MP4VIDEO_
public:
	// header information
	int m_nNumberOfFrame;
	int m_nWidth;
	int m_nHeight;
	int m_nNumberOfChannels;
	int m_nBytePerPixel;

	// frame information
	int m_nRGBFrameSize;
	int m_nYUVFrameSize;
	int m_nCurrentFrame;
	int m_nPreviousFrame;
	bool m_bDump;

	// video buffer
	unsigned char* m_pVideo;

	// destructor
	~MovieTextureProxy()
	{
		delete[] m_pVideo;
	}
#endif _SAIT_3DTG_MP4VIDEO_
};

#include "MovieTextureProxy.i"

#endif // __MOVIETEXTUREPROXY_H__
