// -*- c++ -*-

// This software module was originally developed by Karl Anders ygard
// (Telenor R&D) in the course of development of the MPEG-4 Systems
// standard (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// Copyright 1997, 1998.
// This file was created by K. Mueller, FhG-HHI 2004

#include "OpenGl/OpenGl.h"
#include "MultiTextureProxy.h"
#include "opengl/base/OpenGLVisualRenderer.h"
#include "Stream.h"

// enable MultiTexture

BOOL
MultiTextureProxy::Enable(OpenGLVisualRenderer* pRenderer, DWORD dwTime)
{

  m_uiNumberOfViews = GetMultiTexture()->texture.GetLength();
  m_CameraVector = GetMultiTexture()->cameraVector;

  // Sub-Texture Proxys will be enabled in Indexed face set to bypas the fact, that images 
  // and geometry are handled in totally different parts of the framework

  /*for(UINT i = 0; i < 1; i++)
  {
    if ( GetTexture(i))
      ((TextureNodeProxy*) ( GetTexture(i)->GetRenderingObject()))->Enable(pRenderer, dwTime);
    glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_COLOR, GL_SRC_COLOR);

  }
  */
  pRenderer->SetMultiTexture(this);
  return TRUE;
}

// disable MultiTexture

BOOL
MultiTextureProxy::Disable(OpenGLVisualRenderer* pRenderer, DWORD dwTime)
{

  for(UINT i = 0; i < m_uiNumberOfViews; i++)
  {
    if ( GetTexture(i))
      ((TextureNodeProxy*) ( GetTexture(i)->GetRenderingObject()))->Disable(pRenderer, dwTime);
  }

  return TRUE;
}

// Render object description

/* virtual */ void
MultiTextureProxy::RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD dwTime)
{
  /* Appearance* pAppearance = (Appearance*) GetMediaObject(); */

  int children = ((/* pAppearance-> */ GetTexture(0) ? 1 : 0));

  GLfloat color[] = { 1.0f, 1.0f, 0.0f, 0.75f};

  // SOLARIS PORTING
  //::glColor4fv(color);
  glColor4fv(color);
  // SOLARIS PORTING - END

  RenderDescriptionBox(pRenderer, "MultiTexture");

  ::glPushMatrix();

  ::glScalef(1.0f / children, 1.0f, 1.0f);
  ::glTranslatef(0.0f, -1.0f, 0.0f);

  if ( GetTexture(0)) {
    ((TextureNodeProxy*) GetTexture(0)->GetRenderingObject())->RenderDescription(pRenderer, dwTime);
    
    ::glTranslatef(1.0f, 0.0f, 0.0f);
  }

  ::glPopMatrix();
}
