// -*- c++ -*-

// This software module was originally developed by Karl Anders ygard
// (Telenor R&D) in the course of development of the MPEG-4 Systems
// standard (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// Copyright 1997, 1998.

#ifndef __TEXTURENODEPROXY_H__
#define __TEXTURENODEPROXY_H__

#include "opengl/base/MediaObjectProxy.h"
#include "defs.h"
#include <GL/gl.h>

class TextureNode;
class OpenGLVisualRenderer;

typedef enum {
  RGB,
  RGBA
} TextureType;

/// TextureNode renderer class

class TextureNodeProxy
  : public MediaObjectProxy
{
public:

  /// Constructor
  TextureNodeProxy(MediaObject * pTextureNode);

  /// Destructor

  ~TextureNodeProxy();

	enum TextureFlag;
  /** Enable texture.

      This should be overridden by subclasses.  The actual method which
      will enable the texture, is protected and takes a pointer to the
      actual texture data. */

  virtual BOOL Enable(
    OpenGLVisualRenderer * /* pRenderer */, 
    DWORD /* dwTime */) 
    /* = 0 */ { return TRUE; } /* SB @@@@ */ ;

  /// Disable texture

  virtual BOOL Disable(
    OpenGLVisualRenderer * /* pRenderer */, DWORD /* dwTime */) 
    /* = 0 */ { return TRUE; } /* SB @@@@ */ ;

  /// Render object description

  virtual void		RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD dwTime);

protected:

  /// Enable new texture

  BOOL			Enable(OpenGLVisualRenderer* pRenderer, TextureType eType,
			       int nWidth, int nHeight, unsigned char* pTextureData);

  /// Enable old texture

  BOOL			Enable(OpenGLVisualRenderer* pRenderer);

  /// Disable texture

  BOOL			Disable(OpenGLVisualRenderer* pRenderer);

private:

  // Flag whether texture needs to be rescaled

  BOOL			m_bRescale;

  // Width of texture

  int			m_nTextureWidth;

  // Height of texture

  int			m_nTextureHeight;

  // Width of 

  int			m_nWidth;

  // Height of 

  int			m_nHeight;

  // Texture object

  GLuint		m_nTextureObject;

  // Make an RGB bitmap out of an YUV  

  static void		Yuv2Rgb(unsigned char* rgbdata,
				unsigned char* yptr,
				unsigned char* uptr,
				unsigned char* vptr,
				int lumcols, int lumrows,
				int chrcols, int chrrows);

  static void		Yuv4222Rgb(unsigned char* rgbdata,
				   unsigned char* yptr,
				   unsigned char* uptr,
				   unsigned char* vptr,
				   int lumcols, int lumrows);

  static BOOL initialised;

  static struct yuvconvtable {
    int u1;
    int u2;
    int v1;
    int v2;
  }			convtab[256];
  static int            cliptab[768];

  // SB moved from TextureNode
  // these methods are virtual not to change the Core Nodes...
public:
  // set S direction texture wrapping
  virtual void SetRepeatS(BOOL /* bRepeatS */) 
  /* = 0 */ {} /* SB @@@@ */ ;
  // get S direction texture wrapping
  virtual BOOL GetRepeatS() const 
  /* = 0 */ { return TRUE; } /* SB @@@@ */ ;
  // set T direction texture wrapping
  virtual void SetRepeatT(BOOL /* bRepeatT */) 
  /* = 0 */ {} /* SB @@@@ */ ;
  // get T direction texture wrapping
  virtual BOOL GetRepeatT() const 
  /* = 0 */ { return TRUE; } /* SB @@@@ */ ;

};

#include "TextureNodeProxy.i"

#endif // __TEXTURENODEPROXY_H__
