// This software module was originally developed by Kevin Barron (KPN Research), 
// Mike Coleman (Five Bats Research), Edward Cooke (Teltec Ireland), In-Dai Ha 
// (Hyundai Electronics Industries, Co.), Hae-Kwan Kim (Hyundai Electronics 
// Industries, Co.), Matthew Leditschke (CSELT), Zvi Lifshitz (VDOnet Corp.), 
// Karl Anders ygard (Telenor R&D), in the course of development of the MPEG-4 
// Systems standard (ISO/IEC 14496-1).  This software module is an 
// implementation of a part of one or more MPEG-4 Systems tools as specified by 
// the MPEG-4 Systems standard.  ISO/IEC gives users of the MPEG-4 Systems 
// standard free license to this software module or modifications thereof for 
// use in hardware or software products claiming conformance to the MPEG-4 
// Systems standard.  Those intending to use this software module in hardware or 
// software products are advised that its use may infringe existing patents.  
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use 
// of this software module or modifications thereof in an implementation.  
// Copyright is not released for non MPEG-4 Systems conforming products.  The 
// original developer retains full right to use the code for his/her own 
// purpose, assign or donate the code to a third party and to inhibit third 
// parties from using the code for non MPEG-4 Systems conforming products.  This 
// copyright notice must be included in all copies or derivative works.
//
// Copyright 1997.


#include "OpenGl/OpenGl.h"
#include "TextureTransformProxy.h"

// Render object description

/* virtual */ void
TextureTransformProxy::RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD /* dwTime */)
{
  RenderDescriptionBox(pRenderer, "TextureTransform");
}

// enable texture transformation

BOOL
TextureTransformProxy::Enable(DWORD)
{
  // TextureTransform* pTextureTransform = (TextureTransform*) GetMediaObject();

  const Vec2f& center = /* pTextureTransform-> */ GetCenter();
  const Vec2f& scale = /* pTextureTransform-> */ GetScale();
  const Vec2f& translation = /* pTextureTransform-> */ GetTranslation();
  float rotation = /* pTextureTransform-> */ GetRotation();

  ::glMatrixMode(GL_TEXTURE);
  ::glPushMatrix();

  // undo temporary offset of origo

  if (center.x != 0.0f ||
      center.y != 0.0f)
    ::glTranslatef(-center.x,
		   -center.y,
		   0.0f);

  // do scaling

  if (scale.x != 1.0f ||
      scale.y != 1.0f)
    ::glScalef(scale.x,
	       scale.y,
	       0.0f);

  // do rotation

  if (rotation != 0.0f)
    ::glRotatef(rotation,
		0.0f,
		0.0f,
		1.0f);

  // do temporary offset of origo

  if (center.x != 0.0f ||
      center.y != 0.0f)
    ::glTranslatef(center.x,
		   center.y,
		   0.0f);
  

  // do translation

  if (translation.x != 0.0f ||
      translation.y != 0.0f)
    ::glTranslatef(translation.x,
		   translation.y,
		   0.0f);
  
  ::glMatrixMode(GL_MODELVIEW);

  return TRUE;
}

// disable texture transformation

BOOL
TextureTransformProxy::Disable(DWORD)
{
  ::glMatrixMode(GL_TEXTURE);
  ::glPopMatrix();
  ::glMatrixMode(GL_MODELVIEW);

  return TRUE;
}
