// -*- c++ -*-

// This software module was originally developed by Matthew Leditschke
// (CSELT) and Karl Anders ygard (Telenor R&D), in the course of
// development of the MPEG-4 Systems standard (ISO/IEC 14496-1).  This
// software module is an implementation of a part of one or more MPEG-4
// Systems tools as specified by the MPEG-4 Systems standard.  ISO/IEC
// gives users of the MPEG-4 Systems standard free license to this software
// module or modifications thereof for use in hardware or software products
// claiming conformance to the MPEG-4 Systems standard.  Those intending to
// use this software module in hardware or software products are advised
// that its use may infringe existing patents.  The original developer of
// this software module and his/her company, the subsequent editors and
// their companies, and ISO/IEC have no liability for use of this software
// module or modifications thereof in an implementation.  Copyright is not
// released for non MPEG-4 Systems conforming products.  The original
// developer retains full right to use the code for his/her own purpose,
// assign or donate the code to a third party and to inhibit third parties
// from using the code for non MPEG-4 Systems conforming products.  This
// copyright notice must be included in all copies or derivative works.
//
// Copyright 1997.

#ifndef __AUDIOOBJECTPROXY_H__
#define __AUDIOOBJECTPROXY_H__

#include "opengl/base/MediaObjectProxy.h"

// #include "audio/AudioObject.h"
//#include "Nodes.h"

class OpenGLPresenter;

/// AudioObject renderer node

class AudioObjectProxy
  : public MediaObjectProxy
{
public:

  /// Constructor

  AudioObjectProxy(MediaObject * pAudioObject);

  /** Get data from audio source.

      This function is used by the audio renderer to get more data to play.
      The audio render will ask for a given number of bytes (which will be
      a multiple of the number of bytes required for a single sample), and
      the audio source node will return a pointer to some data and the
      number of bytes of data pointed to.

      The number of bytes returned can be less than that asked for.  If
      fewer bytes were returned to the audio renderer than expected, then
      the audio renderer will ask for more.  If zero bytes are returned,
      then the audio renderer will assume that no data can be given, and
      will insert silence. */

  virtual void		GetData(int nBytesRequired, char*& rpData, int& rnBytesGiven) = 0;

  /** Signal we've used the data.

      This function is used by the audio renderer to indicate that it
      has finished with some data given to it by GetData().  The
      parameters given to this function will be the same as those
      returned by GetData(). */

  virtual void		DataUsed(char* pData, int nBytesGiven) = 0;

  /// Render object

  virtual BOOL		Render(OpenGLVisualRenderer* pRenderer, DWORD dwTime) = 0;

  /// Render object description

  virtual void		RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD dwTime);

  /// Get audio format

  virtual const DecoderAudioFormat*
	                GetFormat() = 0;
};

#endif // __AUDIOOBJECTPROXY_H__
