// -*- c++ -*-

// This software module was originally developed by Matthew Leditschke
// (CSELT) and Karl Anders ygard (Telenor R&D), in the course of
// development of the MPEG-4 Systems standard (ISO/IEC 14496-1).  This
// software module is an implementation of a part of one or more MPEG-4
// Systems tools as specified by the MPEG-4 Systems standard.  ISO/IEC
// gives users of the MPEG-4 Systems standard free license to this software
// module or modifications thereof for use in hardware or software products
// claiming conformance to the MPEG-4 Systems standard.  Those intending to
// use this software module in hardware or software products are advised
// that its use may infringe existing patents.  The original developer of
// this software module and his/her company, the subsequent editors and
// their companies, and ISO/IEC have no liability for use of this software
// module or modifications thereof in an implementation.  Copyright is not
// released for non MPEG-4 Systems conforming products.  The original
// developer retains full right to use the code for his/her own purpose,
// assign or donate the code to a third party and to inhibit third parties
// from using the code for non MPEG-4 Systems conforming products.  This
// copyright notice must be included in all copies or derivative works.
//
// Copyright 1997, 1998.

#include "OpenGl/OpenGl.h"

#include "AudioSourceProxy.h"
#include "opengl/base/OpenGLPresenter.h"

#include "Stream.h"
#include "opengl/base/AudioRenderer.h"

#include <assert.h>


#ifndef WIN32
#include "opengl/base/AudioRendererSol.h"

// Constructor

AudioSourceProxy::AudioSourceProxy(AudioSource* poAudioSource)
: MediaObjectProxy(poAudioSource),
m_skippedHeader(FALSE),
m_nOffset(0),
m_bFirst(TRUE)
{
  m_format.samplesPerSecond = 0;
  m_format.bitsPerSample = 0;
  m_format.numChannels = 0;
}

// render the node
// SOLARIS PORTING - The AudioRenderer thread is started from here
BOOL
AudioSourceProxy::Render(
  OpenGLVisualRenderer* pRenderer , 
  DWORD /* dwTime */)
{
  // Do all the relevant processing - Starting the thread, etc.

  // SB 99/03/03 $$$$ Start Stream & Register Channel
  if (m_bFirst)
  {
    mpoRenderer = pRenderer;
    AudioSource* poAudioSource = (AudioSource*) GetMediaObject();
    MediaStream* pStream = poAudioSource->GetStream();

    // riitta200999
    MediaObject* parent_sound = ((MediaObject*)poAudioSource)->GetParent(0);


    TRACE("[AudioSourceProxy::Render] Stream Ptr %08x \n", pStream);

    Start();
    poAudioSource->GetExecutive()->RequestChannels();
    m_bFirst = FALSE;
    TRACE("[AudioSourceProxy::Render] StreamConsumer started \n");

    // Only proceed once a stream has been attached
    if (pStream)
    {
      /* poAudioSource-> */ TestActive();
    
      if (/* poAudioSource-> */ GetActive())
      {
        // Get a pointer to the audio renderer
        AudioRendererSol* pAR = (AudioRendererSol *)pRenderer->GetPresenter()->GetAudioRenderer();
       
        TRACE("[AudioSourceProxy::Render] active ");

        // Start the AudioRendererSol thread
        ZThread<AudioRendererSol>(
          &AudioRendererSol::Main, 
          pAR
          ).Begin();

      }
    }

  }
  
  
  return TRUE;
}
// SOLARIS PORTING - END
#else

// constructor
FILE* LogFile=NULL;

AudioSourceProxy::AudioSourceProxy(AudioSource* poAudioSource)
: MediaObjectProxy(poAudioSource),
miStreamId(-1),
m_finished(FALSE),
m_bFirst(TRUE)
{
	LogFile=fopen("AudioSourceProxy.txt","w");

}

#endif

// Render object description

/* virtual */ void
AudioSourceProxy::RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD)
{
  RenderDescriptionBox(pRenderer, "AudioSource");
}

// Get audio format

const DecoderAudioFormat*
AudioSourceProxy::GetFormat()
{
  AudioSource* poAudioSource = (AudioSource*) GetMediaObject();
  MediaStream* pStream = poAudioSource->GetStream();
  
  // Only proceed once a stream has been attached
 
  
  if (pStream) {
    /* poAudioSource-> */ TestActive();
    
    if (/* poAudioSource-> */ GetActive()) {
      if (m_format.samplesPerSecond == 0) {
// SOLARIS PORTING - Changed the logic. We dont get the decoder
// information from the stream. Following the WIN32 implementation,
// the default values for Solaris are allocated
/*        int dataSize;
        DWORD rdwTime;
        BOOL rbEOU;
        LPVOID pvDataPtr = pStream->FetchNext(
          dataSize, rdwTime, rbEOU, TRUE);
        
        TRACE("AudioSourceProxy::GetFormat after FetchNext %08x", this);
        
        if (pvDataPtr != NULL &&
          pvDataPtr != END_OF_STREAM &&
          dataSize >= (int) sizeof(DecoderAudioFormat)) {
          ::memcpy(&m_format, pvDataPtr, sizeof(DecoderAudioFormat));
          
          // Release the media stream buffer (but don't discard)
          pStream->Release(FALSE);
        } else
          COMPLAIN("[AudioSourceProxy::GetFormat]: buffer error trying to get audio format\n");*/
        m_format.samplesPerSecond = 8000;
        m_format.bitsPerSample = 16;
        m_format.numChannels = 1;
// SOLARIS PORTING - END
      }
    }
  }
  
  return &m_format;
}

// ============================================================================

//
// AudioSourceProxy::GetData
//
// This function is used by the audio renderer to get more data to play.
// The audio render will ask for a given number of bytes (which will be a
// multiple of the number of bytes required for a single sample), and the
// audio source node will return a pointer to some data, and the number of
// bytes of data pointed to.
//
// The number of bytes returned can be less than that asked for.  If fewer
// bytes were returned to the audio renderer than expected, then the audio
// renderer will ask for more.  If zero bytes are returned, then the audio
// renderer will assume that no data can be given, and will insert silence.
//

void
AudioSourceProxy::GetData(int nBytesRequired,
                          char*& rpData,
                          int& rnBytesGiven)
{
  AudioSource* poAudioSource = (AudioSource*) GetMediaObject();
  MediaStream* pStream = poAudioSource->GetStream();
  
  // Only proceed once a stream has been attached
  
  if (pStream) {
    /* poAudioSource-> */ TestActive();
    
    if (/* poAudioSource-> */ GetActive()) {
      int dataSize;
      DWORD rdwTime;
      BOOL rbEOU;	
      char* pData = (char*) pStream->FetchNext(
        dataSize, rdwTime, rbEOU, FALSE);
      
      TRACE("AudioSourceProxy::GetData after FetchNext %08x\n", pData);

      if (pData)
        // If end of stream has been reached, close the stream and return
        // zero bytes
        
        if (pData != END_OF_STREAM) {
          if (dataSize) {
            // Skip the header data if this is the first time data has been
            // read from this buffer by this function
            
            if (!m_skippedHeader) {
              m_skippedHeader = TRUE;
              m_nOffset += sizeof(DecoderAudioFormat);
            }
            
            // Put in possible offset in data block
            
            pData += m_nOffset;
            dataSize -= m_nOffset;
            
            rpData = pData;
            
            // If the number of bytes requested is less than the number read
            // from the media stream, only return some, and remember where to
            // get the rest from on the next call.
            
            if (dataSize > nBytesRequired) {
              // we only took some of the data
              
              m_nOffset += nBytesRequired;
              rnBytesGiven = nBytesRequired;
            } else {
              // we took all the data; clear m_nOffset
              
              m_nOffset = 0;
              rnBytesGiven = dataSize;
            }
            
            return;
          }
          
          pStream->Release();
        } else
          pStream->Terminate();
    }
  }
  
  rnBytesGiven = 0;
  rpData = NULL;
}

// Signal that we've used the data

void AudioSourceProxy::DataUsed(char* /*pData*/, int /*nBytesGiven*/)
{
  AudioSource* poAudioSource = (AudioSource*) GetMediaObject();
  
  // Release the media stream buffer.  If only part of this buffer was
  // given to the audio renderer, then don't discard it.
  
  if (m_nOffset)
    poAudioSource->GetStream()->Release(FALSE);
  else
    poAudioSource->GetStream()->Release();
}

#ifdef WIN32

// destructor

AudioSourceProxy::~AudioSourceProxy()
{
  // close down the stream

  if (miStreamId != -1)
    ((OpenGLPresenter *) ((MediaObject*) GetMediaObject())->GetExecutive()->GetPresenter())->GetAudioRenderer()->CloseOutputStream(miStreamId);

	fclose(LogFile);

}

// render the node

BOOL
AudioSourceProxy::Render(
  OpenGLVisualRenderer* pRenderer, 
  DWORD /* dwTime */)
{
  mpoRenderer = pRenderer;
  AudioSource* poAudioSource = (AudioSource*) GetMediaObject();
  MediaStream* pStream = poAudioSource->GetStream();


  TRACE("[AudioSourceProxy::Render] Stream Ptr %08x \n", pStream);

  // Only proceed once a stream has been attached
  if (pStream)
  {
    /* poAudioSource-> */ TestActive();
    
    if (/* poAudioSource-> */ GetActive())
    {
      // Get a pointer to the audio renderer
      pAR = pRenderer->GetPresenter()->GetAudioRenderer();
      
      TRACE("[AudioSourceProxy::Render] active with StreamId %4d \n", miStreamId);

      // If this is the first time that this function is called, set up
      // a connection with the audio renderer
      if (miStreamId == -1)
      {
        // Create an audio stream to play this file

		ZThread<AudioSourceProxy>(
            AudioSourceThread, 
            this, 
            THREAD_PRIORITY_TIME_CRITICAL
			).Begin();
      }

    }
  }
  
  return TRUE;
}

#endif // WIN32

void AudioSourceProxy::AudioSourceThread()
{
  AudioSource* poAudioSource = (AudioSource*) GetMediaObject();
  MediaStream* poStream = poAudioSource->GetStream();
  AudioRenderer* poRendAudio = mpoRenderer->GetPresenter()->GetAudioRenderer();

  MediaObject* parent_sound = ((MediaObject*)poAudioSource)->GetParent(0);

  fprintf(LogFile,"Thread starts\n");
  // Repeat until either there is no input data, 
  // or the output buffer is full.
  int iLoopingFlag = 1;

  mlTimeInit = GetMediaObject()->GetRoot()->GetTime();

  while (iLoopingFlag )
  {
		if(!GetActive())
		{
  			miStreamId=-1;//Another chance
			break;
		}
		// Get the next block of data from the decoder (via the media stream)
		int iDataSize;
		DWORD rdwTime;
		BOOL rbEOU;
		BOOL bWait = TRUE;
		unsigned char *  pvDataPtr = (unsigned char *) poStream->FetchNext(
		  iDataSize, rdwTime, rbEOU, bWait);
  
		fprintf(LogFile,"Next AU (%d) %d\n",iDataSize,rdwTime);

    
		// If there is no data, we have finished for now
		if (pvDataPtr == NULL)
		{
			iLoopingFlag = 0;
			fprintf(LogFile,"No data. Exit\n");
			fflush(LogFile);
			miStreamId=-1;//SON
			iLoopingFlag = 0;
		}
		else
		{ // SB $$$$
			  if (pvDataPtr == END_OF_STREAM)
			// If the end of the stream has been reached, we have also finished
			  {
				m_finished = TRUE;
				iLoopingFlag = 0;
				fprintf(LogFile,"END_OF_STREAM. Exit\n");
				fflush(LogFile);
		//		poStream->Terminate();
				miStreamId=-1;//SON

			  }
			  else
			  {				  	  
					if(miStreamId==-1)
					{
						miStreamId = pAR->OpenOutputStream(
						  poStream, 
						  ((int*)pvDataPtr)[2], //Sampling Rate
						  ((int*)pvDataPtr)[0], //ChannelNo
						  ((int*)pvDataPtr)[3], //Symbol size
						  SAMPLEFMT_UNSIGNED_BIG, //Anyway this value is not used in OpenOutputStream
						  ((int*)pvDataPtr)[1], //Window size
						  parent_sound);
					}
				  
				  // SON Check the timestamp only when I render AU

				  // SB $$$$ 99/03/09 added "local" timing for Audio Stream
					mlTimeLocal = GetMediaObject()->GetRoot()->GetTime() - mlTimeInit;
					mlCycles ++;

					fprintf(LogFile,"AU for %d at %d\n",rdwTime , mlTimeLocal);
					fflush(LogFile);
					if (mlTimeLocal < rdwTime)
					{
					  TRACE("[ASP::Thread] sleeping %8d \n", 
						(30 * mlCycles - 270) - mlTimeLocal);
					  Sleep(rdwTime - mlTimeLocal);
					}

					// Otherwise see if this block fits in the output buffer
					if (poRendAudio->QueueFrameForPlaying(miStreamId, pvDataPtr+sizeof(int)*4))
					{
						  // If it does, we have finished with this block from
						  // the stream
						  poStream->Release(TRUE);
					}
					else
					{

						  // Otherwise put this block of data back to be read later
						  // when there is space in the output buffer
						  poStream->Release(FALSE);
						  //	 poStream->Release(TRUE);
						  // SB 99/08/05 ?!?! does this make sense ?!?!
						  // iLoopingFlag = 0;
					}
			  } //!(pvDataPtr == END_OF_STREAM)

		}//!(pvDataPtr == NULL)

  }
  fflush(LogFile);
}
