// -*- c++ -*-

// This software module was originally developed by Matthew Leditschke
// (CSELT) and Karl Anders ygard (Telenor R&D), in the course of
// development of the MPEG-4 Systems standard (ISO/IEC 14496-1).  This
// software module is an implementation of a part of one or more MPEG-4
// Systems tools as specified by the MPEG-4 Systems standard.  ISO/IEC
// gives users of the MPEG-4 Systems standard free license to this software
// module or modifications thereof for use in hardware or software products
// claiming conformance to the MPEG-4 Systems standard.  Those intending to
// use this software module in hardware or software products are advised
// that its use may infringe existing patents.  The original developer of
// this software module and his/her company, the subsequent editors and
// their companies, and ISO/IEC have no liability for use of this software
// module or modifications thereof in an implementation.  Copyright is not
// released for non MPEG-4 Systems conforming products.  The original
// developer retains full right to use the code for his/her own purpose,
// assign or donate the code to a third party and to inhibit third parties
// from using the code for non MPEG-4 Systems conforming products.  This
// copyright notice must be included in all copies or derivative works.
//
// Copyright 1997.

#ifndef __AUDIOSOURCEPROXY_H__
#define __AUDIOSOURCEPROXY_H__

//#include "Nodes.h"

#include "opengl/base/MediaObjectProxy.h"
// Stefano Battista 98/11/12
// #include "audio/AudioSource.h"

class AudioSource;
class OpenGLPresenter;
class FaceProxy;
class AudioRenderer;
//for function pointer table - PA
#include "mpegj-player/MPEGJTerminalInterface.h" 

/// AudioSource renderer node

extern ZMutex ActiveFlagLock;

extern FILE* LogFile;

class AudioSourceProxy
  : public MediaObjectProxy
{
public:
	#ifdef MPGJ
	/***** MPEGJ _ MERGE - handle to the function table - PA *****/
	MPJTerminalDispatchTablePtr di; 
	/***** -end - ****/
	#endif // MPEGJ


  
  /// Constructor
  AudioSourceProxy(AudioSource* pAudioSource);
  
  /// Get data from audio source
  virtual void		GetData(int nBytesRequired, char*& rpData, int& rnBytesGiven);
  
  /// Signal we've used the data
  virtual void		DataUsed(char* pData, int nBytesGiven);
  
  /// Render object
  virtual BOOL		Render(OpenGLVisualRenderer* pRenderer, DWORD dwTime);
  
  /// Get audio format
  virtual const DecoderAudioFormat*
    GetFormat();
  
  /// Render object description
  virtual void		RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD dwTime);
  
private:
  
  /// Format of audio stream
  DecoderAudioFormat	m_format;

  // Have we skipped the audio stream header?
  BOOL			m_skippedHeader;

  // Offset to unused data
  int			m_nOffset;

	AudioRenderer* pAR;
private:
  
  // The stream ID in the audio renderer
  // int streamID;
  BOOL m_finished;
  BOOL m_bFirst;
#ifdef WIN32

public:
  
  ~AudioSourceProxy();
  
#endif
  
  // Stefano Battista
private:
  int miStreamId;
  OpenGLVisualRenderer * mpoRenderer;

  long mlCycles;
  long mlTimeInit;
  long mlTimeLocal;

public:
  AudioSource * GetAudioSource() const
  { return (AudioSource *) GetMediaObject(); }

  void AudioSourceThread();

  void Start()
  {
#ifndef BIFSENC

	  m_bActive = TRUE;
    GetAudioSource()->OpenStream();
	printf("OpenStream is called for audio\n");

#endif
  }

  void Stop()
  {
#ifndef BIFSENC
	//This is a critical phase: not close stream when some rendering / decoding process may use it

//	  ActiveFlagLock.V();

	  m_bActive = FALSE;
	  GetAudioSource()->CloseStream();
    //PauseStream();
	  printf("CloseStream is called for audio\n");
//	  ActiveFlagLock.P();

#endif
  };

  inline BOOL GetActive() const
  {
    return m_bActive;
  };

  void TestActive()
  {
// SOLARIS PORTING - Changed int to DWORD
    Time currentTime = (Time) (DWORD)(GetAudioSource()->GetRoot()->GetTime()) / 1000.0;
//SOLARIS PORTING -END

	AudioSource *poAudioSource = GetAudioSource();
    if (((Time) poAudioSource->startTime <= currentTime) && 
		((poAudioSource->stopTime < poAudioSource->startTime)|| (poAudioSource->stopTime > currentTime )) )
	{
//		m_bActive = TRUE;
		if (m_bFirst)
		{
			poAudioSource->GetExecutive()->RequestChannels();
			m_bFirst = FALSE;
			mlTimeInit = GetMediaObject()->GetRoot()->GetTime();
			mlTimeLocal = 0;
			mlCycles = 0;
			TRACE("[AudioSourceProxy::Render] StreamConsumer started \n");
			Start();
		}

	}
	else if ( ((Time) poAudioSource->startTime > currentTime) || 
			(((Time) poAudioSource->stopTime <= currentTime) && (poAudioSource->stopTime > poAudioSource->startTime)) )
	{
//		m_bActive = FALSE;
		if(!m_bFirst)
		{
			Stop();
			m_bFirst = TRUE;

		}

	}
	else
		m_bActive = FALSE;

/*
    if (m_bActive)
    {
      if ( (Time) poAudioSource->startTime > currentTime || 
		  ((Time) poAudioSource->stopTime <= currentTime && poAudioSource->stopTime > poAudioSource->startTime))
        Stop();
    }
    else
    {
// SOLARIS PORTING - Changed to AudioSource
 //     AudioSource *poAudioSource = GetAudioSource();
      if ((Time) poAudioSource->startTime <= currentTime)
        if (poAudioSource->stopTime < poAudioSource->startTime)
          Start();
        else
          if (poAudioSource->stopTime > poAudioSource->startTime && poAudioSource->stopTime > currentTime)
            Start();
// SOLARIS PORTING - END
    }*/
  };

  // Gabriel TTS stuff
  void SetTTSFaceNode(FaceProxy * /* poFaceProxy */) 
  {};

};

#endif // __AUDIOSOURCEPROXY_H__
