// -*- c++ -*-

// This software module was originally developed by Matthew Leditschke
// (CSELT) and Karl Anders ygard (Telenor R&D), in the course of
// development of the MPEG-4 Systems standard (ISO/IEC 14496-1).  This
// software module is an implementation of a part of one or more MPEG-4
// Systems tools as specified by the MPEG-4 Systems standard.  ISO/IEC
// gives users of the MPEG-4 Systems standard free license to this software
// module or modifications thereof for use in hardware or software products
// claiming conformance to the MPEG-4 Systems standard.  Those intending to
// use this software module in hardware or software products are advised
// that its use may infringe existing patents.  The original developer of
// this software module and his/her company, the subsequent editors and
// their companies, and ISO/IEC have no liability for use of this software
// module or modifications thereof in an implementation.  Copyright is not
// released for non MPEG-4 Systems conforming products.  The original
// developer retains full right to use the code for his/her own purpose,
// assign or donate the code to a third party and to inhibit third parties
// from using the code for non MPEG-4 Systems conforming products.  This
// copyright notice must be included in all copies or derivative works.
//
// Copyright 1997.

#ifndef __AUDIOSOURCEPROXY_H__
#define __AUDIOSOURCEPROXY_H__

#include "opengl/base/MediaObjectProxy.h"
// Stefano Battista 98/11/12
// #include "audio/AudioSource.h"
//#include "Nodes.h"

class OpenGLPresenter;

class FaceProxy; // this is added here, too

/// AudioSource renderer node

class AudioSourceProxy
  : public MediaObjectProxy
{
public:
  
  /// Constructor
  AudioSourceProxy(AudioSource* pAudioSource);
  
  /// Get data from audio source
  virtual void		GetData(int nBytesRequired, char*& rpData, int& rnBytesGiven);
  
  /// Signal we've used the data
  virtual void		DataUsed(char* pData, int nBytesGiven);
  
  /// Render object
  virtual BOOL		Render(OpenGLVisualRenderer* pRenderer, DWORD dwTime);
  
  /// Get audio format
  virtual const DecoderAudioFormat*
    GetFormat();
  
  /// Render object description
  virtual void		RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD dwTime);
  
private:
  
  /// Format of audio stream
  DecoderAudioFormat	m_format;

  // Have we skipped the audio stream header?
  BOOL			m_skippedHeader;

  // Offset to unused data
  int			m_nOffset;

#ifdef WIN32
 
private:
  
  // The stream ID in the audio renderer
  int streamID;
  BOOL m_finished;
  BOOL m_bFirst;
  
public:
  
  ~AudioSourceProxy();
  
#endif
  
  // Stefano Battista
public:
  AudioSource * GetAudioSource() const
  { return (AudioSource *) GetMediaObject(); }

  void Start()
  {
#ifndef BIFSENC
    m_bActive = TRUE;
    GetAudioSource()->OpenStream();
#endif
  }

  void Stop()
  {
#ifndef BIFSENC
    m_bActive = FALSE;
    GetAudioSource()->CloseStream();
    //PauseStream();
#endif
  };

  inline BOOL GetActive() const
  {
    return m_bActive;
  };

  void TestActive()
  {
    Time currentTime = (Time) int(GetAudioSource()->GetRoot()->GetTime()) / 1000.0;
    if (m_bActive)
    {
      if ((Time) startTime > currentTime || ((Time) stopTime <= currentTime && stopTime > startTime))
        Stop();
    }
    else
    {
      if ((Time) startTime <= currentTime)
        if (stopTime < startTime)
          Start();
        else
          if (stopTime > startTime && stopTime > currentTime)
            Start();
    }
  };

    // Gabriel TTS stuff
  void SetTTSFaceNode(FaceProxy * poFaceProxy) 
  {}; // added this by riitta 041999
};

#endif // __AUDIOSOURCEPROXY_H__
