// -*- c++ -*-

// This software module was originally developed by Matthew Leditschke
// (CSELT) and Karl Anders ygard (Telenor R&D), in the course of
// development of the MPEG-4 Systems standard (ISO/IEC 14496-1).  This
// software module is an implementation of a part of one or more MPEG-4
// Systems tools as specified by the MPEG-4 Systems standard.  ISO/IEC
// gives users of the MPEG-4 Systems standard free license to this software
// module or modifications thereof for use in hardware or software products
// claiming conformance to the MPEG-4 Systems standard.  Those intending to
// use this software module in hardware or software products are advised
// that its use may infringe existing patents.  The original developer of
// this software module and his/her company, the subsequent editors and
// their companies, and ISO/IEC have no liability for use of this software
// module or modifications thereof in an implementation.  Copyright is not
// released for non MPEG-4 Systems conforming products.  The original
// developer retains full right to use the code for his/her own purpose,
// assign or donate the code to a third party and to inhibit third parties
// from using the code for non MPEG-4 Systems conforming products.  This
// copyright notice must be included in all copies or derivative works.
//
// Copyright 1997, 1998.

#include "OpenGl/OpenGl.h"
#include "opengl/base/Matrix.h"
#include "Sound2DProxy.h"
#include "AudioObjectProxy.h"


// Render object

BOOL
Sound2DProxy::Render(const Matrix& /* rTransformation */, const CullFlag& /* rCullFlag */,
		     OpenGLVisualRenderer* pRenderer, DWORD dwTime)
{
  /* const AudioObject */ MediaObject * pSound = /* ((Sound2D*) GetMediaObject())-> */ GetSound2D();

  if (pSound)
    return ((AudioObjectProxy*) pSound->GetRenderingObject())->Render(pRenderer, dwTime);
  else
    return FALSE;
}

// Render object description

/* virtual */ void
Sound2DProxy::RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD dwTime)
{
  /* const AudioObject */ MediaObject * pSound = /* ((Sound2D*) GetMediaObject())-> */ GetSound2D();

  RenderDescriptionBox(pRenderer, "Sound2D");

  if (pSound) {
    ::glPushMatrix();
    ::glTranslatef(0.0f, -1.0f, 0.0f);

    ((AudioObjectProxy*) pSound->GetRenderingObject())->RenderDescription(pRenderer, dwTime);

    ::glPopMatrix();
  }
}

// Find sound sources

/* virtual */ int
Sound2DProxy::FindSoundSources(
    ZArray<Sound*>& /* rSources3d */, ZArray<Matrix>& /* rTransformations3d */,
    ZArray<Sound2D*>& rSources2d, ZArray<Matrix>& rTransformations2d,
		const Matrix& rTransform)
{
  rSources2d += (Sound2D*) GetMediaObject();
  rTransformations2d += rTransform;
  
  return 1;
}
