// -*- c++ -*-

// This software module was originally developed by Matthew Leditschke
// (CSELT) and Karl Anders ygard (Telenor R&D), in the course of
// development of the MPEG-4 Systems standard (ISO/IEC 14496-1).  This
// software module is an implementation of a part of one or more MPEG-4
// Systems tools as specified by the MPEG-4 Systems standard.  ISO/IEC
// gives users of the MPEG-4 Systems standard free license to this software
// module or modifications thereof for use in hardware or software products
// claiming conformance to the MPEG-4 Systems standard.  Those intending to
// use this software module in hardware or software products are advised
// that its use may infringe existing patents.  The original developer of
// this software module and his/her company, the subsequent editors and
// their companies, and ISO/IEC have no liability for use of this software
// module or modifications thereof in an implementation.  Copyright is not
// released for non MPEG-4 Systems conforming products.  The original
// developer retains full right to use the code for his/her own purpose,
// assign or donate the code to a third party and to inhibit third parties
// from using the code for non MPEG-4 Systems conforming products.  This
// copyright notice must be included in all copies or derivative works.
//
// Copyright 1997, 1998.

#include "OpenGl/OpenGl.h"
#include "opengl/base/Matrix.h"
#include "SoundProxy.h"
#include "AudioObjectProxy.h"


// Render object

BOOL
SoundProxy::Render(const Matrix& /* rTransformation */, const CullFlag& /* rCullFlag */,
		   OpenGLVisualRenderer* pRenderer, DWORD dwTime)
{
  /* const AudioObject */ MediaObject * pSound = /* ((Sound*) GetMediaObject())-> */ GetSound();

  if (pSound)
    return ((AudioObjectProxy*) pSound->GetRenderingObject())->Render(pRenderer, dwTime);
  else
    return FALSE;
}

// Render object description

/* virtual */ void
SoundProxy::RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD dwTime)
{
  /* const AudioObject */ MediaObject * pSound = /* ((Sound*) GetMediaObject())-> */ GetSound();

  RenderDescriptionBox(pRenderer, "Sound");

  if (pSound) {
    ::glPushMatrix();
    ::glTranslatef(0.0f, -1.0f, 0.0f);

    ((AudioObjectProxy*) pSound->GetRenderingObject())->RenderDescription(pRenderer, dwTime);

    ::glPopMatrix();
  }
}

////////////////////////////////////////////////////////////////////////////////
// Find sound sources

#ifdef ADV_AUDIO

/* virtual */ int
SoundProxy::FindSoundSources(
    ZArray<Sound*>& rSources3d, ZArray<Matrix>& rTransformations3d,
		ZArray<Sound2D*>& /* rSources2d */, ZArray<Matrix>& /* rTransformations2d */,
		ZArray<DirectiveSound*>& /* rDirsources */, ZArray<Matrix>& /* rDirtransformations */,
		const Matrix& rTransform)
{
	rSources3d += (Sound*) GetMediaObject();
	rTransformations3d += rTransform;

  return 1;
}

#else

/* virtual */ int
SoundProxy::FindSoundSources(
    ZArray<Sound*>& rSources3d, ZArray<Matrix>& rTransformations3d,
		ZArray<Sound2D*>& /* rSources2d */, ZArray<Matrix>& /* rTransformations2d */,
		const Matrix& rTransform)
{
  rSources3d += (Sound*) GetMediaObject();
  rTransformations3d += rTransform;
  
  return 1;
}

#endif
