// -*- c++ -*-

// This software module was originally developed by Matthew Leditschke
// (CSELT) and Karl Anders ygard (Telenor R&D), in the course of
// development of the MPEG-4 Systems standard (ISO/IEC 14496-1).  This
// software module is an implementation of a part of one or more MPEG-4
// Systems tools as specified by the MPEG-4 Systems standard.  ISO/IEC
// gives users of the MPEG-4 Systems standard free license to this software
// module or modifications thereof for use in hardware or software products
// claiming conformance to the MPEG-4 Systems standard.  Those intending to
// use this software module in hardware or software products are advised
// that its use may infringe existing patents.  The original developer of
// this software module and his/her company, the subsequent editors and
// their companies, and ISO/IEC have no liability for use of this software
// module or modifications thereof in an implementation.  Copyright is not
// released for non MPEG-4 Systems conforming products.  The original
// developer retains full right to use the code for his/her own purpose,
// assign or donate the code to a third party and to inhibit third parties
// from using the code for non MPEG-4 Systems conforming products.  This
// copyright notice must be included in all copies or derivative works.
//
// Copyright 1997, 1998.

#ifndef __SOUNDPROXY_H__
#define __SOUNDPROXY_H__

#include "opengl/base/Node3DProxy.h"

//#include "Nodes.h"
// SB #### simply removed AudioObject
// #include "audio/AudioObject.h"

class Sound;

/// Sound renderer node

class SoundProxy
  : public Node3DProxy
{
public:

  /// Constructor
  SoundProxy(Sound* pSound);

  /// Render object
  virtual BOOL  Render(
    const Matrix& rTransformation, const CullFlag& rCullFlag,
		OpenGLVisualRenderer* pRenderer, DWORD dwTime);

  /// Render object description

  virtual void  RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD dwTime);

  /// Find sound sources
#ifdef ADV_AUDIO
  virtual int  FindSoundSources(
      ZArray<Sound*>& rSources3d, ZArray<Matrix>& rTransformations3d,
			ZArray<Sound2D*>& rSources2d, ZArray<Matrix>& rTransformations2d,
			ZArray<DirectiveSound*>& rDirsources, ZArray<Matrix>& rDirtransformations,
			const Matrix& rTransform);
#else
  virtual int  FindSoundSources(
    ZArray<Sound*>& rSources3d, ZArray<Matrix>& rTransformations3d,
		ZArray<Sound2D*>& rSources2d, ZArray<Matrix>& rTransformations2d,
		const Matrix& rTransform);
#endif

  // SB 98/12/04
public:
  inline Sound * GetSound3D()
  { return (Sound *) GetMediaObject(); }

  /// Set sound sound
  inline void SetSound(SFAudioNode oSound)
  { GetSound3D()->source /* sound */ = oSound; }
  
  /// Get sound sound
  inline SFAudioNode GetSound()
  { return GetSound3D()->source /* sound */ ; }
  
  /// Get direction of sound source
  //ML - changed return type from const Vec3f& to const Vec3f
  // to remove warning about returning address of temporary variable
  inline const Vec3f GetDirection() /* const */
  { return GetSound3D()->direction; }
  
  /// Get location of sound source
  //ML - changed return type from const Vec3f& to const Vec3f
  // to remove warning about returning address of temporary variable
  inline const Vec3f GetLocation() /* const */
  { return GetSound3D()->location; }
  
  /// Get whether sound source should be spatialized
  inline BOOL GetSpatialize() /* const */
  { return (Bool) GetSound3D()->spatialize; }
  
  /// Get max back
  inline float GetMaxBack() /* const */
  { return GetSound3D()->maxBack; }
  
  /// Get max front
  inline float GetMaxFront() /* const */
  { return GetSound3D()->maxFront; }
  
  /// Get min back
  inline float GetMinBack() /* const */
  { return GetSound3D()->minBack; }
  
  /// Get min front
  inline float GetMinFront() /* const */
  { return GetSound3D()->minFront; }

};

#include "SoundProxy.i"

#endif // __SOUNDPROXY_H__
