// -*- c++ -*-

// This software module was originally developed by Matthew Leditschke
// (CSELT) and Karl Anders ygard (Telenor R&D) in the course of
// development of the MPEG-4 Systems standard (ISO/IEC 14496-1).  This
// software module is an implementation of a part of one or more MPEG-4
// Systems tools as specified by the MPEG-4 Systems standard.  ISO/IEC
// gives users of the MPEG-4 Systems standard free license to this software
// module or modifications thereof for use in hardware or software products
// claiming conformance to the MPEG-4 Systems standard.  Those intending to
// use this software module in hardware or software products are advised
// that its use may infringe existing patents.  The original developer of
// this software module and his/her company, the subsequent editors and
// their companies, and ISO/IEC have no liability for use of this software
// module or modifications thereof in an implementation.  Copyright is not
// released for non MPEG-4 Systems conforming products.  The original
// developer retains full right to use the code for his/her own purpose,
// assign or donate the code to a third party and to inhibit third parties
// from using the code for non MPEG-4 Systems conforming products.  This
// copyright notice must be included in all copies or derivative works.
//
// Copyright 1998.

#ifndef __AUDIORENDERERLINUX_H__
#define __AUDIORENDERERLINUX_H__

#include "AudioRenderer.h"
#include "ztl.h"

#include "Spatialize.h"
#include "audio/Sound.h"

/// IRIX audio renderer

class AudioRendererLinux
  : public AudioRenderer
{
public:
  
  /// Constructor

			AudioRendererLinux(Presenter* pPresenter, HWND hWnd);

  /// Destructor

  virtual		~AudioRendererLinux();

  /// Get the capabilities of this audio renderer

  virtual void		GetCapabilities(Capabilities* pPresentationCaps);

  /// Initialise the renderer

  virtual void		Init();

  /// Stops rendering and performs cleanups.

  virtual void		Terminate();

  virtual void GetViewPosition();
  virtual void GetSoundPosition(Sound* pSound3D, const Matrix& transf_matrix,
				Vec3f& sound_loc, Vec3f& sound_dir, float* ellip_def);
  virtual void InitSpatializeAndEllipse(int i, Sound* pSound3D,
					Vec3f& sound_loc, Vec3f& sound_dir, float* ellip_def);

private:

  int nof_sources;
  float viewp[4];
  AmPan* pan_array[10];
  EllipseSound* ellip_array[10];
  short* m_pSpatializeBuffer;
  float* x; 

private:

  /// Main rendering loop

  void			Main();

  /// The output audio port

  int			m_audioFd;

  /// Samplerate

  int			m_samplesPerSec;

  /// Sample width in bits

  int			m_bitsPerSample;

  /// Sample width in bytes

  int			m_bytesPerSample;

  /// Optimal audio fragment size

  int			m_optimalFragSize;

  /// Number of channels

  int			m_numChannels;

  /// The size of the output queue in samples

  int			m_outputQueueSize;

  /// Buffer used to mix samples

  char*			m_pMixBuffer;

  /// This flag indicates whether thread should terminate

  BOOL			m_bActive;

  /// This mutex indicates thread is running

  ZMutex		m_mtxThread;

  /** Init audio.

      This function is called when the first audio output stream is known.
      The audio output format settings of this first stream are given to
      this function, and the output format of the audio device is set to
      this.  If other streams are added which have a different output
      format, then their format will need to be changed in software. */

  void		      InitAudioOutput(int nSamplesPerSec,
				      int nBitsPerSample,
				      int nNumChannels);

  /** Mix audio.

      Combine the given audio data with the audio data already in the mux
      buffer.  The offset specifies the offset (in bytes) of where to start
      combining the data. */

  void		      CombineData(int offset, char* pSrc, int lenSrc);
};

#include "AudioRendererLinux.i"

#endif // __AUDIORENDERERLINUX_H__

