// This software module was originally developed by Oliver Lietz
// (Heinrich-Hertz-Institute Berlin), Matthew Leditschke (CSELT) and Karl
// Anders ygard (Telenor R&D), in the course of development of the MPEG-4
// Systems standard (ISO/IEC 14496-1).  This software module is an
// implementation of a part of one or more MPEG-4 Systems tools as
// specified by the MPEG-4 Systems standard.  ISO/IEC gives users of the
// MPEG-4 Systems standard free license to this software module or
// modifications thereof for use in hardware or software products claiming
// conformance to the MPEG-4 Systems standard.  Those intending to use this
// software module in hardware or software products are advised that its
// use may infringe existing patents.  The original developer of this
// software module and his/her company, the subsequent editors and their
// companies, and ISO/IEC have no liability for use of this software module
// or modifications thereof in an implementation.  Copyright is not
// released for non MPEG-4 Systems conforming products.  The original
// developer retains full right to use the code for his/her own purpose,
// assign or donate the code to a third party and to inhibit third parties
// from using the code for non MPEG-4 Systems conforming products.  This
// copyright notice must be included in all copies or derivative works.
//
// Copyright 1998.

#include "defs.h"
#include "ztl.h"
#include <assert.h>
#include "AudioRendererSol.h"
#include "Caps.h"
#include "Executiv.h"
#include "Presenter.h"
#include "Stream.h"
#include "opengl/audio/AudioObjectProxy.h"
#include "opengl/audio/AudioSourceProxy.h"
#include "opengl/audio/SoundProxy.h"
#include "opengl/audio/Sound2DProxy.h"
#include <stropts.h>
#include <poll.h>
//satish added this line.. for close()
#include <unistd.h>

//#include "audio/AudioObject.h"
#include "audio/SoundProxy.h"

// Stefano Battista 98/11/12
// #include "group/Root.h"
#include "Nodes.h"

// The length of the output queue (in milliseconds)

const int QUEUE_LENGTH = 1000;

// Stops rendering and performs cleanups

/* virtual */ void
AudioRendererSol::Terminate()
{
  // Flag that the audio renderer is no longer active
  m_bActive = FALSE;
  
  // Wait for this thread to terminate
  m_mtxThread.P();
  m_mtxThread.V();
  
  // Release the audio output related resources
  if (m_audioPort)
    close(m_audioPort);
  
  if (m_pMixBuffer)
    delete m_pMixBuffer;
}

// Initialise the renderer
// SOLARIS PORTING - This does nothing. The AudioRenderer
// thread is started from the AudioSourceProxy's Render when it first comes in
/* virtual */ void
AudioRendererSol::Init()
{
  // Start the main thread of the audio renderer
//    if ( ZThread <AudioRendererSol >(&AudioRendererSol::Main, this).Begin() == -1)
//      throw;
}

// Init audio

void AudioRendererSol::InitAudioOutput(int samplesPerSec, int bitsPerSample, int numChannels)
{
  int fd;

  if ((fd = open ("/dev/audio", O_WRONLY | O_NDELAY, 0)) < 0)
  {
    perror("/dev/audio open failed");
    if (errno == EBUSY)
    {
      COMPLAIN("Sorry, the audio device is busy!\n");
    }
    else
    {
      COMPLAIN("can't open /dev/audio for writing\n");
    }
  }
  // turn NDELAY mode off:
  int flags;
  if ((flags = fcntl (fd, F_GETFL, 0)) < 0)
  {
    COMPLAIN ("fcntl F_GETFL on /dev/audio failed\n");
  }
  flags &= ~O_NDELAY;
  if (fcntl (fd, F_SETFL, flags) < 0)
  {
    COMPLAIN ("fcntl F_SETFL on /dev/audio failed\n");
  }

  m_audioPort = fd;

  // configure the audio device:
  audio_info info;
  AUDIO_INITINFO (&info);
  ioctl (m_audioPort, AUDIO_GETINFO, &info);

#ifdef DEBUG_SET_AUDIOPORTS
  info.output_muted = FALSE;
  info.play.port = 0;
  info.play.port |= AUDIO_SPEAKER;
  info.play.port |= AUDIO_HEADPHONE;
  info.play.port |= AUDIO_LINE_OUT;
#endif

  TRACE( "info:%d:%d:%d:%d", info.play.encoding, info.play.precision, info.play.channels, info.play.sample_rate );

  info.play.encoding = AUDIO_ENCODING_LINEAR;
  info.play.precision = bitsPerSample;
  info.play.channels = numChannels;
  info.play.sample_rate = samplesPerSec;

  if (ioctl (m_audioPort, AUDIO_SETINFO, &info))
  {
// SOLARIS PORTING
    perror("ioctl problem");
// SOLARIS PORTING - END
    COMPLAIN ("configuration of /dev/audio failed\n");
  }

  // Determine and set the length of the output queue (in samples)
  m_outputQueueSize = samplesPerSec * QUEUE_LENGTH / 1000;

  // Remember the output port settings
  m_samplesPerSec  = samplesPerSec;
  m_bitsPerSample  = bitsPerSample;
  m_numChannels    = numChannels;
  m_bytesPerSample = numChannels * bitsPerSample / 8;
  
  // Allocate a buffer to mix samples from different streams in
  m_pMixBuffer = new char[m_outputQueueSize * m_bytesPerSample];
}

// Main rendering loop

void AudioRendererSol::Main()
{
  BOOL first = TRUE;

  // Mark the thread as running.
  m_mtxThread.P();

  // Flag that the audio renderer is now active
  m_bActive = TRUE;

  {
    ZArray<Sound*> soundSources3D;
    ZArray<Sound2D*> soundSources2D;
    ZArray<Matrix> soundTransformations3D;
    ZArray<Matrix> soundTransformations2D;
    ZArray<Matrix> soundDirTransformations;
    ZArray<DirectiveSound*> soundDirsources;
	//satish added this statement
    const Matrix rTransform;
    int numberOfSources;
    
    GetPresenter()->GetExecutive()->LockScene();

	//satish changed this line
    //numberOfSources = ((MediaObjectProxy*) GetPresenter()->GetScene()->GetRenderingObject())->FindSoundSources(soundSources3D, soundTransformations3D, soundSources2D, soundTransformations2D);

    numberOfSources = ((MediaObjectProxy*) GetPresenter()->GetScene()->GetRenderingObject())->FindSoundSources(soundSources3D, soundTransformations3D, soundSources2D, soundTransformations2D, soundDirsources, soundDirTransformations, rTransform);

    GetPresenter()->GetExecutive()->UnlockScene();

    if (numberOfSources > 0) {
      const DecoderAudioFormat* pFormat;

      if (soundSources3D.GetLength()){
		//satish changed this line
	//pFormat = ((AudioObjectProxy*) soundSources3D[0]->GetSound()->GetRenderingObject())->GetFormat();
        pFormat = ((AudioObjectProxy*) ((Sound*) soundSources3D[0])->source.GetNode()->GetRenderingObject())->GetFormat();
	}
// Adding the directive sound
      else if ( soundDirsources.GetLength() )
      {
        pFormat = ((AudioObjectProxy*) ((DirectiveSound*) soundDirsources[0])->source.GetNode()->GetRenderingObject())->GetFormat();
      }
      else{
		//satish changed this line
//	pFormat = ((AudioObjectProxy*) soundSources2D[0]->GetSound()->GetRenderingObject())->GetFormat();
        pFormat = ((AudioObjectProxy*) ((Sound*) soundSources2D[0])->source.GetNode()->GetRenderingObject())->GetFormat();
	}

      if (pFormat) {
	InitAudioOutput(pFormat->samplesPerSecond,
			pFormat->bitsPerSample,
			pFormat->numChannels);

	TRACE("[Audio renderer]: params %ld, %d, %d\n", pFormat->samplesPerSecond, pFormat->bitsPerSample, pFormat->numChannels);
      } else
	// Initialise the audio device with defaults

	InitAudioOutput(8000, 16, 1);
    } else
      // Initialise the audio device with defaults

      InitAudioOutput(8000, 16, 1);
  }

  // This loop runs while the audio renderer is active, ie. up to the next
  // Terminate() call.
  int loc_count = 0;

  while (m_bActive) {
    // how many samples to try to write

    long samplesToWrite = m_outputQueueSize / 2;

    // Only read data from the streams if there is room

    if (samplesToWrite > 0) {
      // Read data from the active buffers, merging the
      // results together into the mixing buffer.

      ZArray<Sound*> soundSources3D;
      ZArray<Sound2D*> soundSources2D;
      ZArray<Matrix> soundTransformations3D;
      ZArray<Matrix> soundTransformations2D;
      ZArray<Matrix> soundDirTransformations;
      ZArray<DirectiveSound*> soundDirsources;
	//satish added this statement;
	const Matrix rTransform;
      int numberOfSources = 0;
      BOOL bufferEmpty = TRUE;

      GetPresenter()->GetExecutive()->LockScene();

      if (GetPresenter()->GetScene())
	//satish changed this statement
        //numberOfSources = ((MediaObjectProxy*) GetPresenter()->GetScene()->GetRenderingObject())->FindSoundSources(soundSources3D, soundTransformations3D, soundSources2D, soundTransformations2D);
        numberOfSources = ((MediaObjectProxy*) GetPresenter()->GetScene()->GetRenderingObject())->FindSoundSources(soundSources3D, soundTransformations3D, soundSources2D, soundTransformations2D, soundDirsources, soundDirTransformations, rTransform);
      GetPresenter()->GetExecutive()->UnlockScene();

      if (numberOfSources > 0) {
	int i;

	for (i = soundSources3D.GetLength() - 1; i >= 0; --i)  {
	  // Determine number of bytes to read from this source

		//satish changed this line
	  //const DecoderAudioFormat* pFormat = ((AudioObjectProxy*) soundSources3D[i]->GetSound()->GetRenderingObject())->GetFormat();
          const DecoderAudioFormat* pFormat = ((AudioObjectProxy*) ((Sound*) soundSources3D[i])->source.GetNode()->GetRenderingObject())->GetFormat();


	  if (pFormat) {
	    int bytesPerSample = pFormat->bitsPerSample / 8;
	    int bytesToWrite = samplesToWrite * bytesPerSample;
	    int bytesWritten = 0;
			
	    // Repeatedly read data for this buffer, until either the buffer
	    // is full, or there is no more data

	    int bytesRead;

	    do {
	      // Read data from the audio node

	      char* pData;
		//satish changed the next line
	     // ((AudioObjectProxy*) soundSources3D[i]->GetSound()->GetRenderingObject())->GetData(bytesToWrite,
                ((AudioObjectProxy*) ((Sound*) soundSources3D[i])->source.GetNode()->GetRenderingObject())->GetData(bytesToWrite,
                   pData, bytesRead);
		//TRACE( "[AudioRendererSol::Main] Count %d", loc_count++ );

	      if (bytesRead > 0) {
		if (first) {
		  // Adjust system overhead first time we get data

			//satish changed the next line
		  //soundSources3D[i]->GetSound()->GetStream()->AdjustClock();
                  ((AudioSourceProxy*) ((Sound*) soundSources3D[i])->source.GetNode()->GetRenderingObject())->GetAudioSource()->GetStream()->AdjustClock();
		  first = FALSE;
		}

		// If this is the first stream, copy it into
		// the mixing buffer.

		if (bufferEmpty)
		  ::memcpy(m_pMixBuffer + bytesWritten, pData, bytesRead);
		else
		  // otherwise combine it with the existing data

		  CombineData(bytesWritten, pData, bytesRead);
				
// SOLARIS PORTING - Moved from bottom
		// Update how much data is left to get

		bytesToWrite -= bytesRead;
		bytesWritten += bytesRead;

		bytesRead = 0; // Make this zero
// SOLARIS PORTING - END
		// Release this data

		//satish changed the next line
		//((AudioObjectProxy*) soundSources3D[i]->GetSound()->GetRenderingObject())->DataUsed(pData, bytesRead);
                ((AudioObjectProxy*) ((Sound*) soundSources3D[i])->source.GetNode()->GetRenderingObject())->DataUsed(pData, bytesRead);
			  
	      }
	    } while (bytesToWrite > 0 && bytesRead > 0);
			
	    if (bufferEmpty && bytesWritten > 0) {
	      bufferEmpty = FALSE;

	      if (bytesToWrite)
		samplesToWrite = bytesWritten / bytesPerSample;
	    }
	  }
	}

// Directive Sources
	for (i = soundDirsources.GetLength() - 1; i >= 0; --i)  {
	  // Determine number of bytes to read from this source

		//satish changed this line
	  //const DecoderAudioFormat* pFormat = ((AudioObjectProxy*) soundDirsources[i]->GetSound()->GetRenderingObject())->GetFormat();
          const DecoderAudioFormat* pFormat = ((AudioObjectProxy*) ((DirectiveSound*) soundDirsources[i])->source.GetNode()->GetRenderingObject())->GetFormat();


	  if (pFormat) {
	    int bytesPerSample = pFormat->bitsPerSample / 8;
	    int bytesToWrite = samplesToWrite * bytesPerSample;
	    int bytesWritten = 0;
			
	    // Repeatedly read data for this buffer, until either the buffer
	    // is full, or there is no more data

	    int bytesRead;

	    do {
	      // Read data from the audio node

	      char* pData;
		//satish changed the next line
	     // ((AudioObjectProxy*) soundDirsources[i]->GetSound()->GetRenderingObject())->GetData(bytesToWrite,
                ((AudioObjectProxy*) ((DirectiveSound*) soundDirsources[i])->source.GetNode()->GetRenderingObject())->GetData(bytesToWrite,
                   pData, bytesRead);
		TRACE( "[AudioRendererSol::Main] Count %d", loc_count++ );

	      if (bytesRead > 0) {
		if (first) {
		  // Adjust system overhead first time we get data

			//satish changed the next line
		  //soundDirsources[i]->GetSound()->GetStream()->AdjustClock();
                  ((AudioSourceProxy*) ((DirectiveSound*) soundDirsources[i])->source.GetNode()->GetRenderingObject())->GetAudioSource()->GetStream()->AdjustClock();
		  first = FALSE;
		}

		// If this is the first stream, copy it into
		// the mixing buffer.

		if (bufferEmpty)
		  ::memcpy(m_pMixBuffer + bytesWritten, pData, bytesRead);
		else
		  // otherwise combine it with the existing data

		  CombineData(bytesWritten, pData, bytesRead);
				
// SOLARIS PORTING - Moved from bottom
		// Update how much data is left to get

		bytesToWrite -= bytesRead;
		bytesWritten += bytesRead;

		bytesRead = 0; // Make this zero
// SOLARIS PORTING - END
		// Release this data

		//satish changed the next line
		//((AudioObjectProxy*) soundDirsources[i]->GetSound()->GetRenderingObject())->DataUsed(pData, bytesRead);
                ((AudioObjectProxy*) ((DirectiveSound*) soundDirsources[i])->source.GetNode()->GetRenderingObject())->DataUsed(pData, bytesRead);
			  
	      }
	    } while (bytesToWrite > 0 && bytesRead > 0);
			
	    if (bufferEmpty && bytesWritten > 0) {
	      bufferEmpty = FALSE;

	      if (bytesToWrite)
		samplesToWrite = bytesWritten / bytesPerSample;
	    }
	  }
	}

	for (i = soundSources2D.GetLength() - 1; i >= 0; --i)  {
	  // Determine number of bytes to read from this source
		//satish changed the next line
	  //const DecoderAudioFormat* pFormat = ((AudioObjectProxy*) soundSources2D[i]->GetSound()->GetRenderingObject())->GetFormat();
          const DecoderAudioFormat* pFormat = ((AudioObjectProxy*) ((Sound2D*) soundSources2D[i])->source.GetNode()->GetRenderingObject())->GetFormat();
	  //AudioObjectProxy* AOP =  (AudioObjectProxy*)soundSources2D[i];
	  //const DecoderAudioFormat* pFormat = AOP->GetFormat();

	  if (pFormat) {
	    int bytesPerSample = pFormat->bitsPerSample / 8;
	    int bytesToWrite = samplesToWrite * bytesPerSample;
	    int bytesWritten = 0;
			
	    // Repeatedly read data for this buffer, until either the buffer
	    // is full, or there is no more data

	    int bytesRead;

	    do {
	      // Read data from the audio node

	      char* pData;


		//satish changed the next line
	      //((AudioObjectProxy*) soundSources2D[i]->GetSound()->GetRenderingObject())->GetData(bytesToWrite, pData, bytesRead);
              ((AudioObjectProxy*) ((Sound2D*) soundSources2D[i])->source.GetNode()->GetRenderingObject())->GetData(bytesToWrite, pData, bytesRead);

	      if (bytesRead > 0) {
		if (first) {
		  // Adjust system overhead first time we get data

		  //soundSources2D[i]->GetSound()->GetStream()->AdjustClock();
// SOLARIS PORTING - Incomplete
                  ((AudioSourceProxy*) ((Sound2D*) soundSources2D[i])->source.GetNode()->GetRenderingObject())->GetAudioSource()->GetStream()->AdjustClock();
		  first = FALSE;
		}

		// If this is the first stream, copy it into
		// the mixing buffer.

		if (bufferEmpty)
		  ::memcpy(m_pMixBuffer + bytesWritten, pData, bytesRead);
		else
		  // otherwise combine it with the existing data

		  CombineData(bytesWritten, pData, bytesRead);
				
// SOLARIS PORTING - Moved from bottom
		// Update how much data is left to get

		bytesToWrite -= bytesRead;
		bytesWritten += bytesRead;

		bytesRead = 0; // Make this zero
// SOLARIS PORTING - END
		// Release this data

		//satish changed the next line
		//((AudioObjectProxy*) soundSources2D[i]->GetSound()->GetRenderingObject())->DataUsed(pData, bytesRead);
                ((AudioObjectProxy*) ((Sound2D*) soundSources2D[i])->source.GetNode()->GetRenderingObject())->DataUsed(pData, bytesRead);
			  
	      }
	    } while (bytesToWrite > 0 && bytesRead > 0);
			
	    if (bufferEmpty && bytesWritten > 0) {
	      bufferEmpty = FALSE;

	      if (bytesToWrite)
		samplesToWrite = bytesWritten / bytesPerSample;
	    }
	  }
	}

	if (!bufferEmpty)
	  // Write combined samples to the audio port

	  ::write(m_audioPort, m_pMixBuffer, samplesToWrite * m_bytesPerSample);
	else
	  // Wait a little instead, so we don't chew up CPU just looking
	  // for more samples

	  ::usleep(1000);
      }
    } // If sampleToWrite > 0
    else
    {
// SOLARIS PORTING
      ::usleep(1000);
// SOLARIS PORTING - END
    }
  }
  
  // Flag that the thread is terminating
  m_mtxThread.V();
}

/// Mix audio

void
AudioRendererSol::CombineData(int offset,
			      char* pSrc,
			      int lenSrc)
{
  DWORD numSamps  = lenSrc / m_bytesPerSample;

  if (m_bitsPerSample <= 8) {
    // Assume 8 bit unsigned format
    unsigned char* oldSamps = (unsigned char*) (m_pMixBuffer + offset);
    unsigned char* newSamps = (unsigned char*) pSrc;

    for (DWORD i = 0; i < numSamps; i++) {
      unsigned short temp = oldSamps[i] + newSamps[i];

      if (temp > 255)
	oldSamps[i] = 255;
      else
	oldSamps[i] = (unsigned char) temp;
    }
  } else {
    // Assume 16 bit signed format
    signed short* oldSamps = (signed short*) (m_pMixBuffer + offset);
    signed short* newSamps = (signed short*) pSrc;

    for (DWORD i = 0; i < numSamps; i++) {
      signed long temp = oldSamps[i] + newSamps[i];

      if (temp < -32768)
	oldSamps[i] = -32768;
      else
	if (temp > 32767)
	  oldSamps[i] = 32767;
	else
	  oldSamps[i] = (signed short) temp;
    }
  }
}
