// This software module was originally developed by Matthew Leditschke
// (CSELT) in the course of development of the MPEG-4 Systems standard
// (ISO/IEC 14496-1).  This software module is an implementation of a part
// of one or more MPEG-4 Systems tools as specified by the MPEG-4 Systems
// standard.  ISO/IEC gives users of the MPEG-4 Systems standard free
// license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// For more information or to receive an up to date version of this module,
// mail karlo@nta.no or check <http://televr.fou.telenor.no/compositor/>.
//
// Copyright 1997.

#ifndef AUDIO_RENDERER_H
#define AUDIO_RENDERER_H

#ifdef WIN32

#include <mmsystem.h>
#include <dsound.h>

// Stefano Battista 98/11/12
// #include "audio/sound.h"
//#include "Nodes.h"

#include "OpenGl/AdvancedAudio/AudioProcess/Spatialize.h"
#include "OpenGl/AdvancedAudio/AudioProcess/ProcessSound.h"

#endif // WIN32

#include "Caps.h"

// Possible values for the sample formats handled by the audio renderer
const unsigned int SAMPLEFMT_SIGNED_LITTLE	 = (1 << 0);
const unsigned int SAMPLEFMT_SIGNED_BIG		 = (1 << 1);
const unsigned int SAMPLEFMT_UNSIGNED_LITTLE = (1 << 2);
const unsigned int SAMPLEFMT_UNSIGNED_BIG	 = (1 << 3);

class Presenter;
class StreamInfo;
class MediaStream;
typedef StreamInfo* StreamInfoPtr;


//
// The definition of the AudioRenderer class.
//

class AudioRenderer
{
	//
	// Public functions
	//
public:

	// Class constructor - given a pointer to the presenter.
	AudioRenderer(Presenter* pPresenter, HWND hWnd);

	// Class destructor - calls terminate if it hasn't already been
	// called.
	virtual ~AudioRenderer();

	void CleanSpatBuffer();

	// Get the capabilities of this audio renderer
	virtual void GetCapabilities (Capabilities* pPresentationCaps);

	// Initialise the renderer
	virtual void Init();

	// Stops rendering and performs cleanups.
	virtual void Terminate();

	// Get presenter
	Presenter* GetPresenter() const;

	// Return TRUE if the audio renderer was initialised successfully.
	BOOL InitialisedOK() { return m_initOK; }


	// Open a stream within the AudioRenderer which audio data can
	// be sent to, returning the number of this stream.  When opening
	// this stream, the format of the audio sample that are to be sent
	// to this stream must be given, comprising the sample rate,
	// number of channels, bits per sample and sample format.  The size
	// of audio frames (in terms of the number of samples) that will be
	// given to QueueFrameForPlaying() is also required.
	//
	// If the stream couldn't be opened, -1 is returned, otherwise the
	// stream number is returned.
	int OpenOutputStream(MediaStream *pStream,
						 unsigned int sampleRate,
						 unsigned int numChannels,
						 unsigned int bitsPerSample,
						 unsigned int sampleFormat,
						 unsigned int frameSize,
						 MediaObject* parent_sound); // riitta 200999

	// Close down the given internal channel in the audio renderer.
	void CloseOutputStream(int streamNumber);


	// Get the length of the buffer, in terms of the delay or time
	// (in milliseconds) it takes samples to pass through it
	unsigned int GetBufferDelayTime(int streamNumber);


	// Put the given frame of audio samples in the queue of samples
	// to be played.  Samples from different channels will be combined
	// together by the audio renderer.  The number of samples in the
	// frame is that given to OpenOutputStream().
	//
	// If TRUE is returned, the data has successfully been queued, and the
	// memory pointed to can be reused.  If FALSE is returned, the data
	// couldn't be queued because of a lack of buffer space.  The frame
	// of data should be queued at a later stage.
	//
	// If bitsPerSample is no more than 8, the samples are to be stored
	// in an array of unsigned chars.  If bitsPerSample is no more than
	// 16, the samples are to be stored in an array of signed shorts.
	// Currently bitsPerSample can't exceed 16.
	//
	// The data is to be stored channel interleaved.
	BOOL QueueFrameForPlaying(int streamNumber,
							 void* ptrToSamples);

//riitta's additions
//#ifdef WIN32
//	BOOL SetupSpatialize(int s_freq, int stream_id);
//	void GetSoundAndViewPositions(int);
//#endif
//RIITTA: end

	
	//
	// Private implementation details
	//
private:

	// Remember a pointer to the presenter that this renderer is part of
	Presenter* m_pPresenter;

#ifdef WIN32

//RIITTA's Addition (for amplitude panning)
//	AmPan* pan;
	//for creating the sound ellipse
//	EllipseSound* ellip;
	// buffer length for spatializer
	int spat_buf_length;
	short* buf_spat;
//	float* viewp;
//	Vec3f sound_loc;
//	Vec3f sound_dir;
//	float* ell_def;
//	float* tr;
//	float* x;
//	Bool spat;

	//added for checking
	int count;
// END of riitta's definitions

	// The DirectSound object which controls the sound card
	LPDIRECTSOUND m_pDirectSound;

	// The array of sound streams which have been created
	StreamInfoPtr* m_streams;

	// The size of this array (number of pointers allocated)
	int m_streamsSize;

	// The number of entries in this array
	int m_numStreams;

	// Sync variable
	BOOL m_bFirst;
#endif

	// Stores if the audio renderer was initialised successfully.
	BOOL m_initOK;
};

#endif // AUDIO_RENDERER_H

