// -*- c++ -*-

// This software module was originally developed by Lars Nilsson (Telenor
// R&D) in the course of development of the MPEG-4 Systems standard
// (ISO/IEC 14496-1).  This software module is an implementation of a part
// of one or more MPEG-4 Systems tools as specified by the MPEG-4 Systems
// standard.  ISO/IEC gives users of the MPEG-4 Systems standard free
// license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// For more information or to receive an up to date version of this module,
// mail karlo@nta.no or check <http://televr.fou.telenor.no/compositor/>.
//
// Copyright 1997.

#include "ztypes.h" // SOLARIS PORTING - Needed for definitions

// Get elements

inline
Matrix::operator float*() const
{
  return (float*) m_elements;
}

// Get elements

// SOLARIS PORTING - restored the const
//satish made this function constant
// WIN-SOL release
#ifdef WIN32
inline float&
Matrix::operator [](int element)
#else
// PORTING to CC 5.3
//inline float& Matrix::operator [](int element) const
inline float& Matrix::operator [](int element)
#endif
{
  return m_elements[element];
}

/// Return whether matrix only has translation

inline BOOL
Matrix::HasOnlyTranslation() const
{
  return (m_elements[ 0] == 1.0f &&
	  m_elements[ 1] == 0.0f &&
	  m_elements[ 2] == 0.0f &&
	  m_elements[ 4] == 0.0f &&
	  m_elements[ 5] == 1.0f &&
	  m_elements[ 6] == 0.0f &&
	  m_elements[ 8] == 0.0f &&
	  m_elements[ 9] == 0.0f &&
	  m_elements[10] == 1.0f);
}

// Translate

inline void
Matrix::Translate(const Vec3f& rTranslation)
{
  Translate(rTranslation.x, rTranslation.y, rTranslation.z);
}

// Rotate

inline void
Matrix::Rotate(const Rotation& rRotation)
{
  Rotate(rRotation.x, rRotation.y, rRotation.z, rRotation.theta);
}
