// -*- c++ -*-

// This software module was originally developed by Karl Anders ygard
// (Telenor R&D) in the course of development of the MPEG-4 Systems
// standard (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// Copyright 1997, 1998.

#ifndef __MEDIAOBJECTPROXY_H__
#define __MEDIAOBJECTPROXY_H__

//#include "ztl.h"
#include "Matrix.h"
//satish included this file for SF
#include "NodeFld.h"
//#include "MediaObj.h"
#include "RendObj.h"

//#include "Presenter.h"

class OpenGLVisualRenderer;
class Sound;
class Sound2D;

// Riitta 99/05/06
#ifdef ADV_AUDIO
class Shape;
class DirectiveSound;
class AcousticScene;
class PerceptualParameters;
//#include "Nodes.h"
#endif // ADV_AUDIO

typedef enum {
  COMPLETELY_INSIDE,
  COMPLETELY_OUTSIDE,
  PARTIALLY_INSIDE
} CullFlag;

/// Media object renderer class

// SB #### derived MediaObjectProxy from RenderingObject
class MediaObjectProxy : public RenderingObject
{
  // SB #### added bboxCenter, bboxSize and methods >>>>
protected:
  SF<Vec3f> bboxCenter;
  SF<Vec3f> bboxSize;
public:
  void GetBbox(SF<Vec3f> & rBboxSize, SF<Vec3f> & rBboxCenter)
  {
    rBboxCenter = (Vec3f) bboxCenter;
    rBboxSize = (Vec3f) bboxSize;
  }
  void SetBbox(SF<Vec3f> & rBboxSize, SF<Vec3f> & rBboxCenter)
  {
    bboxCenter = (Vec3f) rBboxCenter;
    bboxSize = (Vec3f) rBboxSize;
  }
  void GetTransformedBbox(SF<Vec3f> & rBboxSize, SF<Vec3f> & rBboxCenter)
  {
    GetBbox(rBboxSize, rBboxCenter);
  }
  void UpdateBbox();
  // SB @@@@ added bboxCenter, bboxSize and methods <<<<

public:

  /// Constructor
	MediaObjectProxy(MediaObject* pObject);

  /// Destructor
  virtual ~MediaObjectProxy();

  /// Get media object this object is connected to
  inline MediaObject * GetMediaObject() const
  { return m_pObject; };

  // SB 99/05/14 added for testing
  virtual void DumpTrace()
  {
#ifdef _DEBUG
    // MediaObject * poNode = GetMediaObject();
    // poNode->GetPresenter()->Logf("RendMediaObject %08x \n", this);
#endif // _DEBUG
  }

  /// Get drawing order of object
  inline virtual float GetDrawingOrder() const
  { return 0.0f; };

  /// Return TRUE if the Node is a Mouse Sensor
  inline virtual BOOL IsPointingDeviceSensor() const
  { return FALSE; };

  /// Activate sensor
  inline virtual bool Activate(
    OpenGLVisualRenderer* /* pRenderer */,
		float /* fObjectX */, 
    float /* fObjectY */, 
    float /* fObjectZ */,
		int /* nX */, 
    int /* nY */, 
    bool /* bMouseActivated */)
  { return FALSE; };

  /// Test if object is inside frustum
  CullFlag InsideFrustum(
    const Matrix& transform,
		OpenGLVisualRenderer* pRenderer);
  
  /// Decide whether to Cull or Render the object
  inline BOOL CullOrRender(
    const Matrix& rTransformation, 
    const CullFlag& rCullFlag,
		OpenGLVisualRenderer* pRenderer, 
    DWORD dwTime);

  /// Decide whether to Cull or Select the object
  inline void CullOrSelect(
    const Matrix& rTransformation, 
    const CullFlag& rCullFlag,
    OpenGLVisualRenderer* pRenderer, 
    DWORD dwTime,
    MediaObjectProxy*& intersectingObject,
    ZArray<MediaObjectProxy*>& activatedSensors, 
    unsigned int& depth,
    const ZArray<MediaObjectProxy*>& localSensors);

  /// Render (recursively) the object and its children
  virtual BOOL Render(
    const Matrix& /* rTransformation */, 
    const CullFlag& /* rCullFace */,
		OpenGLVisualRenderer* pRenderer, 
    DWORD /* dwTime */);

  /// Check selection and streamin media (recursively) the object and its children
  inline virtual void Select(
    OpenGLVisualRenderer * /* pRenderer */, 
    DWORD /* dwTime */,
		MediaObjectProxy * & /* intersectingObject */, 
    unsigned int & /* depth */)
  {};

  /// Traverse scene graph, checking for activation of sensors
  inline virtual void Select(
    const Matrix& /* rTransformation */, 
    const CullFlag& /* rCullFace */,
    OpenGLVisualRenderer* /* pRenderer */, 
    DWORD /* dwTime */,
    MediaObjectProxy*& /* intersectingObject */,
    ZArray<MediaObjectProxy*>& /* activatedSensors */, 
    unsigned int& /* depth */,
    const ZArray<MediaObjectProxy*>& /* localSensors */)
  {};

  /// Render lights

  inline virtual BOOL RenderLights(
    OpenGLVisualRenderer* /* pRenderer */, 
    DWORD /* dwTime */)
  { return TRUE; }

  /** Render object description.
      This method draws a box with a description of the object. */
  virtual void RenderDescription(
    OpenGLVisualRenderer* pRenderer, 
    DWORD dwTime);

  /** Render object description box.
      This method draws the box with a description of the object. */
  virtual void RenderDescriptionBox(
    OpenGLVisualRenderer* pRenderer, 
    const char* pString) const;

  /** Get object to world matrix.
      This method computes the transformation matrix of the world
      coordinate system into the local coordinate system of this node.
      This is done by backtracking to the top of the scene graph,
      collecting inverse transformations that might be specified in the
      graph.
      This method is required for rendering the scene graph relative to the
      viewpoint. */
  virtual void ObjectToWorldMatrix(
    DWORD dwTime, 
    Matrix& rMatrix) const;

  /// Get viewpoint to world matrix
  virtual void ViewpointToWorldMatrix(
    DWORD dwTime, 
    Matrix& rMatrix) const;
  
  /// Collision detection
  virtual MediaObject * DetectCollision(
    const SF<Vec3f> & rTransformedSize, 
    const SF<Vec3f> & rTransformedCenter,
    const SF<Vec3f> & rSize, 
    const SF<Vec3f> & rCenter,
    const Matrix& rTransform,
    const MediaObject * /* pCallingNode */);
  
#ifdef ADV_AUDIO

  /// Find sound sources
  virtual int FindSoundSources(
    ZArray<Sound*>& /*rSources3d*/, 
    ZArray<Matrix>& /*rTransformations3d*/,
    ZArray<Sound2D*>& /*rSources2d*/, 
    ZArray<Matrix>& /*rTransformations2d*/,
    ZArray<DirectiveSound*>& /*rDirsources*/, 
    ZArray<Matrix>& /*rDirtransformations*/,
    const Matrix& /*rTransform = Matrix()*/)
  { return 0; };
  
  virtual int FindAcousticScenes(
    ZArray<AcousticScene*>&,
    ZArray<Matrix>&,
    const Matrix&)
  {return 0;};
  
  /// Find acoustic surfaces
  virtual int FindAcousticSurfaces(
    ZArray<Shape*>&,				// IndexedFaceSet with AcousticMaterial
    ZArray<Matrix>&,				// Transformations above the AcousticMaterial
    const Matrix&)
  { return 0; };
  
#else
  
  /// Find sound sources
  virtual int FindSoundSources(
    ZArray<Sound*>& /* rSources3d */, 
    ZArray<Matrix>& /* rTransformations3d */,
    ZArray<Sound2D*>& /* rSources2d */, 
    ZArray<Matrix>& /* rTransformations2d */,
    const Matrix& /*rTransform = Matrix()*/)
  { return 0; };
  
#endif // ADV_AUDIO

private:

  /// Media object this object is connected to

  MediaObject * m_pObject;

  // SB @@@@ added >>>>
protected:
  BOOL   m_bActive;
  DWORD  startTime;
  DWORD  stopTime;
  // SB @@@@ added <<<<
};

#include "MediaObjectProxy.inl"

#endif // __MEDIAOBJECTPROXY_H__
