// -*- c++ -*-

// This software module was originally developed by Karl Anders ygard
// (Telenor R&D) in the course of development of the MPEG-4 Systems
// standard (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// Copyright 1997, 1998.

// Constructor

inline
MediaObjectProxy::MediaObjectProxy(MediaObject* pObject)
  : m_pObject(pObject)
{
  startTime = stopTime = 0;
}

// Destructor

/* virtual */ inline
MediaObjectProxy::~MediaObjectProxy()
{
}

// Cull or render the object

inline BOOL
MediaObjectProxy::CullOrRender(
  const Matrix& rTransformation, 
  const CullFlag& rCullFlag,
  OpenGLVisualRenderer* pRenderer, 
  DWORD dwTime)
{
  if (rCullFlag == COMPLETELY_INSIDE)
  {
    return Render(
	  rTransformation, 
	  rCullFlag, 
	  pRenderer, 
	  dwTime);
  }
  else
  {
    CullFlag cullFlag = InsideFrustum(
	  rTransformation, 
	  pRenderer);
    
    if (cullFlag != COMPLETELY_OUTSIDE)
    {
      return Render(
	    rTransformation, 
		cullFlag, 
		pRenderer, 
		dwTime);
    }
#ifdef ADV_AUDIO
// Riitta 99/05/06
	else 
	{
		return Render(
	    rTransformation, 
		COMPLETELY_INSIDE, 
		pRenderer, 
		dwTime);

	} 
#endif // ADV_AUDIO
  }

  return TRUE;
}

// Cull or select the object
    
inline void MediaObjectProxy::CullOrSelect(
  const Matrix& rTransformation, 
  const CullFlag& rCullFlag,
  OpenGLVisualRenderer* pRenderer, 
  DWORD dwTime,
  MediaObjectProxy*& intersectingObject,
  ZArray<MediaObjectProxy*>& activatedSensors, 
  unsigned int& depth,
  const ZArray<MediaObjectProxy*>& localSensors)
{
  if (rCullFlag == COMPLETELY_INSIDE)
  {
    Select(
	  rTransformation, 
	  rCullFlag,
	  pRenderer, 
	  dwTime,
	  intersectingObject,
	  activatedSensors, 
	  depth,
	  localSensors);
  }
  else
  {
    CullFlag cullFlag = InsideFrustum(
	  rTransformation, 
	  pRenderer);
    
    if (cullFlag != COMPLETELY_OUTSIDE)
	{
      Select(
	    rTransformation, 
		cullFlag,
	    pRenderer, 
		dwTime,
	    intersectingObject,
	    activatedSensors, 
		depth,
	    localSensors);
    }
  }
}
