// This software module was originally developed by Karl Anders ygard
// (Telenor R&D) in the course of development of the MPEG-4 Systems
// standard (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// For more information or to receive an up to date version of this module,
// mail karlo@nta.no or check <http://televr.fou.telenor.no/compositor/>.
//
// Copyright 1997.

#include "OpenGl/OpenGl.h"
#include "NodeProxy.h"
#include "Node3DProxy.h"
#include "Node2DProxy.h"
#include "OpenGLPresenter.h"

// SOLARIS PORTING - REMOVED the :: in gl calls

// SB @@@@ ObjectToWorld and ViewpointToWorld moved to MediaObjectProxy

/***
// Get object to world matrix
void NodeProxy::ObjectToWorldMatrix(DWORD dwTime, Matrix& rMatrix) const
{
  MediaObject * pNode = (MediaObject *) GetMediaObject();
  // are we at the root of the scene graph?
  if (pNode->GetParentsNumber() > 0)
    // no, keep going
    ((MediaObjectProxy *) pNode->GetParent(0)->GetRenderingObject())->
        ObjectToWorldMatrix(dwTime, rMatrix);
}

// Get viewpoint to world matrix
void ViewpointToWorldMatrix(DWORD dwTime, Matrix& rMatrix) const
{
  MediaObject * pNode = (MediaObject *) GetMediaObject();
  // are we at the root of the scene graph?
  if (pNode->GetParentsNumber() > 0)
  {
    // no, keep going
    ((MediaObjectProxy *) pNode->GetParent(0)->GetRenderingObject())->
        ViewpointToWorldMatrix(dwTime, rMatrix);
  }
}
***/

// Render lights

/* virtual */ BOOL
NodeProxy::RenderLights(OpenGLVisualRenderer* /* pRenderer */, DWORD /* dwTime */)
{
  return TRUE;
}

// Traverse scene graph, checking for intersection

/* virtual */ void
NodeProxy::Select(
    OpenGLVisualRenderer* /* pRenderer */, 
    DWORD /* dwTime */,
		MediaObjectProxy*& /* intersectingObject */, 
    unsigned int&/* depth */)
{
}

// Traverse scene graph, checking for activation of sensors

/* virtual */ void
NodeProxy::Select(
    const Matrix& /* rTransformation */, 
    const CullFlag& /* rCullFace */,
		OpenGLVisualRenderer* /* pRenderer */, 
    DWORD /* dwTime */,
		MediaObjectProxy*& /* intersectingObject */,
		ZArray<MediaObjectProxy*>& /* activatedSensors */, 
    unsigned int& /* depth */,
		const ZArray<MediaObjectProxy*>& /* localSensors */)
{
}

// Activate sensor

/* virtual */ bool
NodeProxy::Activate(
    OpenGLVisualRenderer* /* pRenderer */,
		float /* fObjectX */, 
    float /* fObjectY */, 
    float /* fObjectZ */,
		int /* nX */, 
    int /* nY */, 
    bool /* bMouseActivated */)
{
  return FALSE;
}

// Return whether object is a pointing device sensor

/* virtual */ BOOL
NodeProxy::IsPointingDeviceSensor() const
{
  return FALSE;
}

// Render object description

/* virtual */ void
NodeProxy::RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD /* dwTime */)
{
  GLfloat color[] = { 0.0f, 1.0f, 1.0f, 0.75f};

  glColor4fv(color);

  RenderDescriptionBox(pRenderer, "Node");
}

// Render object

/* virtual */ BOOL
NodeProxy::Render(const Matrix& /* rTransformation */, const CullFlag& /* rCullFace */,
		  OpenGLVisualRenderer* pRenderer, DWORD /* dwTime */)
{
  if (pRenderer->DrawBBoxes()) {
    /* MediaObject * pNode = (MediaObject *) GetMediaObject(); */

    Vec3f size;
    Vec3f center;

// SOLARIS PORTING
// Created new objects to avoid memory dereferencing
   SF<Vec3f> sf_size, sf_center;
   sf_size = (SF<Vec3f>) size;
   sf_center = (SF<Vec3f>) center;

    /* pNode-> */ GetBbox( sf_size, sf_center);

   size = (Vec3f)sf_size;
   center = (Vec3f)sf_center;

// SOLARIS PORTING - END

    ::glPushAttrib(GL_FOG_BIT);
    ::glDisable(GL_FOG);

    ::glPushMatrix();
    ::glTranslatef(center.x, center.y, center.z);

    if (size.x > -1) {
      size *= 0.5f;

      ::glBegin(GL_LINE_STRIP);
      glVertex3f( size.x,  size.y,  size.z);
      glVertex3f(-size.x,  size.y,  size.z);
      glVertex3f(-size.x, -size.y,  size.z);
      glVertex3f( size.x, -size.y,  size.z);
	       			  
      glVertex3f( size.x,  size.y,  size.z);
      glVertex3f( size.x,  size.y, -size.z);
      glVertex3f( size.x, -size.y, -size.z);
      glVertex3f( size.x, -size.y,  size.z);
      ::glEnd();			  
				  
      ::glBegin(GL_LINE_STRIP);	  
      glVertex3f(-size.x, -size.y,  size.z);
      glVertex3f(-size.x, -size.y, -size.z);
      glVertex3f( size.x, -size.y, -size.z);
      ::glEnd();			  
				  
      ::glBegin(GL_LINE_STRIP);	  
      glVertex3f(-size.x,  size.y,  size.z);
      glVertex3f(-size.x,  size.y, -size.z);
      glVertex3f( size.x,  size.y, -size.z);
      ::glEnd();			  
				  
      ::glBegin(GL_LINES);		  
      glVertex3f(-size.x,  size.y, -size.z);
      glVertex3f(-size.x, -size.y, -size.z);
      ::glEnd();
    }

    ::glBegin(GL_POINTS);
    glVertex3f(0.0f, 0.0f, 0.0f);
    ::glEnd();

    ::glPopMatrix();
    ::glPopAttrib();
  }
  
  return TRUE;
}

// Test if object is inside frustum

CullFlag
NodeProxy::InsideFrustum(const Matrix& transform,
			 OpenGLVisualRenderer* pRenderer)
{
  // MediaObject * pNode = (MediaObject *) GetMediaObject();

  const Vec3f& left = pRenderer->GetLeftFrustumPlane();
  const Vec3f& right = pRenderer->GetRightFrustumPlane();
  const Vec3f& top = pRenderer->GetTopFrustumPlane();
  const Vec3f& bottom = pRenderer->GetBottomFrustumPlane();

  Vec3f size, center;
  Vec3f bll; // back, lower, left
  Vec3f fur; // front, upper, right

// SOLARIS PORTING
// Created new objects to avoid memory dereferencing
  SF<Vec3f> sf_size, sf_center;
  sf_size = (SF<Vec3f>) size;
  sf_center = (SF<Vec3f>) center;

  /* pNode-> */ GetTransformedBbox( sf_size, sf_center);

  size = (Vec3f)sf_size;
  center = (Vec3f)sf_center;

// SOLARIS PORTING - END

  fur = bll = transform.MultiplyPoint(center);

  Vec3f x(transform[0], transform[1], transform[ 2]);
  Vec3f y(transform[4], transform[5], transform[ 6]);
  Vec3f z(transform[8], transform[9], transform[10]);

  x *= size.x;
  y *= size.y;
  z *= size.z;

  size.x = ::fabs(x.x) + ::fabs(y.x) + ::fabs(z.x);
  size.y = ::fabs(x.y) + ::fabs(y.y) + ::fabs(z.y);
  size.z = ::fabs(x.z) + ::fabs(y.z) + ::fabs(z.z);

  size *= 0.5f;

  bll -= size;
  fur += size;

  /* {
     // Draw frustum culling bounding boxes

    ::glPushMatrix();
    ::glLoadIdentity();

    ::glBegin(GL_LINE_STRIP);
    glVertex3f(fur.x, fur.y, fur.z);
    glVertex3f(bll.x, fur.y, fur.z);
    glVertex3f(bll.x, bll.y, fur.z);
    glVertex3f(fur.x, bll.y, fur.z);
	       			  
    glVertex3f(fur.x, fur.y, fur.z);
    glVertex3f(fur.x, fur.y, bll.z);
    glVertex3f(fur.x, bll.y, bll.z);
    glVertex3f(fur.x, bll.y, fur.z);
    ::glEnd();			  
				  
    ::glBegin(GL_LINE_STRIP);	  
    glVertex3f(bll.x, bll.y, fur.z);
    glVertex3f(bll.x, bll.y, bll.z);
    glVertex3f(fur.x, bll.y, bll.z);
    ::glEnd();			  
				  
    ::glBegin(GL_LINE_STRIP);	  
    glVertex3f(bll.x, fur.y, fur.z);
    glVertex3f(bll.x, fur.y, bll.z);
    glVertex3f(fur.x, fur.y, bll.z);
    ::glEnd();			  
				  
    ::glBegin(GL_LINES);		  
    glVertex3f(bll.x, fur.y, bll.z);
    glVertex3f(bll.x, bll.y, bll.z);
    ::glEnd();
    ::glPopMatrix();
  } */

  // First, check if behind frustum

  if (bll.z < 0.0f) {
    CullFlag intersection = COMPLETELY_INSIDE;
    Vec3f fll = bll;
    Vec3f bur = fur;

    bur.z = bll.z;
    fll.z = fur.z;

    // In front of frustum

    if (bll.y > 0.0f) {
      if (bll * top > 0.0f)
	return COMPLETELY_OUTSIDE;
    } else
      if (fll * bottom > 0.0f)
	intersection = PARTIALLY_INSIDE;

    if (bur.y < 0.0f) {
      if (bur * bottom > 0.0f)
	return COMPLETELY_OUTSIDE;
    } else
      if (intersection == COMPLETELY_INSIDE)
	if (fur * top > 0.0f)
	  intersection = PARTIALLY_INSIDE;
    
    if (bll.x > 0.0f) {
      if (bll * right > 0.0f)
	return COMPLETELY_OUTSIDE;
    } else 
      if (intersection == COMPLETELY_INSIDE)
	if (fll * left > 0.0f) 
	  intersection = PARTIALLY_INSIDE;
    
    if (bur.x < 0.0f) {
      if (bur * left > 0.0f)
	return COMPLETELY_OUTSIDE;
    } else 
      if (intersection == COMPLETELY_INSIDE)
	if (fur * right > 0.0f)
	  intersection = PARTIALLY_INSIDE;

    return intersection;
  } else
    return COMPLETELY_OUTSIDE;
}

