// -*- c++ -*-

// This software module was originally developed by Karl Anders ygard
// (Telenor R&D) in the course of development of the MPEG-4 Systems
// standard (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// For more information or to receive an up to date version of this module,
// mail karlo@nta.no or check <http://televr.fou.telenor.no/compositor/>.
//
// Copyright 1997.

#ifndef __NODEPROXY_H__
#define __NODEPROXY_H__

#include "MediaObjectProxy.h"

class Matrix;
class OpenGLPresenter;

/***
class Node;
class TouchSensor;
***/
//#include "Nodes.h"

// CullFlag moved to MediaObjectProxy.h
/***
typedef enum {
  COMPLETELY_INSIDE,
  COMPLETELY_OUTSIDE,
  PARTIALLY_INSIDE
} CullFlag;
***/

/// Node renderer class

class NodeProxy
  : public MediaObjectProxy
{
public:

  /// Constructor

  // Stefano Battista 98/11/09
	//	NodeProxy(Node * pNode);
			NodeProxy(MediaObject * pNode);

  /** Cull or render the object.

      This method will test the object for frustum culling and if it is not
      culled, will call the virtual Render() method. */
  BOOL CullOrRender(
    const Matrix& rTransformation, 
    const CullFlag& rCullFlag,
		OpenGLVisualRenderer* pRenderer, 
    DWORD dwTime);

  // virtual void		ObjectToWorldMatrix(DWORD dwTime, Matrix& rMatrix) const;

  // virtual void ViewpointToWorldMatrix(DWORD dwTime, Matrix& rMatrix) const;

  /// Traverse scene graph, activating lights
  virtual BOOL RenderLights(
    OpenGLVisualRenderer* pRenderer, 
    DWORD dwTime);

  /// Traverse scene graph, checking for intersection
  virtual void Select(
    OpenGLVisualRenderer* pRenderer, 
    DWORD dwTime,
		MediaObjectProxy*& intersectingObject, 
    unsigned int& depth);

  /// Render object description
  virtual void RenderDescription(
    OpenGLVisualRenderer* pRenderer,
		DWORD dwTime);

  /** Cull or select the object.

      This method will test the object for frustum culling and if it is not
      culled, will call the virtual Render() method. */

  void CullOrSelect(
    const Matrix& rTransformation, 
    const CullFlag& rCullFlag,
		OpenGLVisualRenderer* pRenderer, 
    DWORD dwTime,
		MediaObjectProxy*& intersectingObject,
		ZArray<MediaObjectProxy*>& activatedSensors, 
    unsigned int& depth,
		const ZArray<MediaObjectProxy*>& localSensors);

  /** Activate sensor.  

      This method is called when the object is attempted to activate.  If
      the object chooses to respond to the activation, it should return
      TRUE.  If it does not respond, it should return FALSE.

      This method will typically be overloaded by pointing-device sensors
      and drag sensors.	 In pointing-device sensors, the method will
      usually fire off an event.  In drag sensors, this will cause the
      sensor to register with the Presenter for receiving events until the
      sensor is deactivated. */

  virtual bool Activate(
    OpenGLVisualRenderer* pRenderer,
		float fObjectX, 
    float fObjectY, 
    float fObjectZ,
		int nX, 
    int nY, 
    bool bMouseActivated);

  /** Return whether object is a pointing device sensor.

      This method should be overloaded by pointing-device sensors and drag
      sensors. */

  virtual BOOL IsPointingDeviceSensor() const;

  /// Get drawing order of object
  virtual float GetDrawingOrder() const;

protected:

  /** Render the object.

      First, the transformation matrix of the world coordinate system into
      the local coordinate system of the active viewpoint is multiplied
      into the projection matrix.  Then, a simple cull traversal is
      performed, along with registration of lights.  From this pass, a draw
      list is generated along with a list of lights (a collapsed list of
      sound sources might be produced as well).

      In the second pass, objects in the draw list are drawn in the
      framebuffer.

      These two tasks may be done in separate threads for performance; both
      are relatively time consuming, and will pipeline very well on high
      end systems. */

  virtual BOOL Render(
    const Matrix& rTransformation, 
    const CullFlag& rCullFlag,
		OpenGLVisualRenderer* pRenderer, 
    DWORD dwTime);

  /// Traverse scene graph, checking for activation of sensors
  virtual void Select(
    const Matrix& rTransformation, 
    const CullFlag& rCullFlag,
		OpenGLVisualRenderer* pRenderer, 
    DWORD dwTime,
		MediaObjectProxy*& intersectingObject,
		ZArray<MediaObjectProxy*>& activatedSensors, 
    unsigned int& depth,
		const ZArray<MediaObjectProxy*>& localSensors);

private:

  /// Test if object is inside frustum
  CullFlag InsideFrustum(
    const Matrix& transform,
		OpenGLVisualRenderer* pRenderer);
};

#include "NodeProxy.i"

#endif // __NODEPROXY_H__
