// This software module was originally developed by Karl Anders ygard
// (Telenor R&D), Simen Hagen (Telenor R&D) and Eddie Cooke (Teltec
// Ireland) in the course of development of the MPEG-4 Systems standard
// (ISO/IEC 14496-1).  This software module is an implementation of a part
// of one or more MPEG-4 Systems tools as specified by the MPEG-4 Systems
// standard.  ISO/IEC gives users of the MPEG-4 Systems standard free
// license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// Copyright 1997, 1998.

#include "OpenGl/OpenGl.h"
#include "OpenGLApplication.h"

//satish added the next line 
#include "OpenGLVisualRenderer.h"

#include "Executive3d.h"

#include <assert.h>

// SOLARIS PORTING
#ifdef GLUT
#include "glut.h"
#endif
// static members

OpenGLApplication* OpenGLApplication::m_pApplication = NULL;

#if defined(GLUT)

#include <signal.h>

#ifdef MESA_3DFX
extern "C" {
#define XMESA_FX_WINDOW       1
#define XMESA_FX_FULLSCREEN   2
  
  extern GLboolean XMesaSetFXmode(GLint mode);
}
#endif // MESA_3DFX

// Constructor (#ifdef GLUT)

OpenGLApplication::OpenGLApplication(int* argc, char* argv[])
: Application(argc, argv),
m_bFinished(FALSE),
m_nMouseButtons(0),
m_bFullscreen(TRUE)
{
  
  m_pApplication = this;
  
  //m_pExecutive->SetPresenter(new OpenGLPresenter(m_pExecutive, HWND() ));
  
  switch (*argc) {
  case 2:
    m_pBitstreamFilename = argv[1];
    break;
    
  default:
    COMPLAIN("too many arguments\n");
    
  case 1:
    TRACE("Usage: %s [X11 options] [filename]\n", argv[0]);
    exit(-1);
  }
  
  GetExecutive()->Start(m_pBitstreamFilename);
  
// SOLARIS PORTING
// Put all the initialization code here
#ifdef GLUT
  ::glutDisplayFunc(m_glutDisplayFunction);
  ::glutIdleFunc(m_glutIdleFunction);
  ::glutMouseFunc(m_glutMouseFunction);
  ::glutMotionFunc(m_glutMotionFunction);
  ::glutPassiveMotionFunc(m_glutMotionFunction);
  ::glutKeyboardFunc(m_glutKeyboardFunction);
  ::glutReshapeFunc(m_glutReshapeFunction);
  
  ::glutCreateMenu(m_glutMenuFunction);
  ::glutAttachMenu(GLUT_MIDDLE_BUTTON);
  
  ::glutAddMenuEntry("Nice texturing", 0);
  ::glutAddMenuEntry("Fast texturing", 1);
  ::glutAddMenuEntry("Filled polygons", 2);
  ::glutAddMenuEntry("Wireframe", 3);
  ::glutAddMenuEntry("Show scene graph", 4);
  ::glutAddMenuEntry("Show scene graph with memory locations", 5);
  ::glutAddMenuEntry("Hide scene graph", 6);
  ::glutAddMenuEntry("Show bounding boxes", 7);
  ::glutAddMenuEntry("Hide bounding boxes", 8);
  ::glutAddMenuEntry("Enable dithering", 9);
  ::glutAddMenuEntry("Disable dithering", 10);
  ::glutAddMenuEntry("Enable collision detection", 11);
  ::glutAddMenuEntry("Disable collision detection", 12);
  
  ConfigureOpenGL();
#endif
// SOLARIS PORTING - END  
  ::glutMainLoop();
}

#else // ! GLUT

// Constructor (#ifndef GLUT)

// WIN-SOL release
OpenGLApplication::OpenGLApplication(HWND  hWnd ) : 
Application(  hWnd  ),
m_bFinished(FALSE),
m_nMouseButtons(0)
{
  /*** SB #### 99/02/23
  m_pApplication = this;
  
  m_pExecutive->SetPresenter(new OpenGLPresenter(m_pExecutive, hWnd));
  
  m_pBitstreamFilename = "flexmux_channel.1";
  ***/
}

#endif // GLUT

// We got a non-fatal error

//satish changed this line
 /* virtual */ // void OpenGLApplication::OnError(NON_FATAL_ERROR error)
/*virtual*/  void OpenGLApplication::OnError(NON_FATAL_ERROR error, va_list args)
{
  switch (error) {
  case INVALID_STREAM_TYPE:
    COMPLAIN("--- Non-fatal error ---: Invalid stream type \n");
    break;
    
  case STREAM_TYPE_COLLISION:
    COMPLAIN("--- Non-fatal error ---: Stream type collision \n");
    break;
    
  case NO_BIFS_STREAM:
    COMPLAIN("--- Non-fatal error ---: No BIFS stream \n");
    break;
    
  case SCENE_PARSING_ERROR:
    COMPLAIN("--- Non-fatal error ---: Scene parsing error \n");
    break;
    
  case OD_PARSING_ERROR:
    COMPLAIN("--- Non-fatal error ---: OD parsing error \n");
    break;
    
  case OD_UPDATE_NOT_SUPPORTED:
    COMPLAIN("--- Non-fatal error ---: OD update not supported \n");
    break;
    
  case ODID_COLLISION:
    COMPLAIN("--- Non-fatal error ---: ODID collision \n");
    break;
    
  case UNKNOWN_ODID:
    COMPLAIN("--- Non-fatal error ---: Unknown ODID \n");
    break;
    
  case UPDATE_ESD_NOT_SUPPORTED:
    COMPLAIN("--- Non-fatal error ---: Update ESD not supported \n");
    break;
    
  case REMOVE_ESD_NOT_SUPPORTED:
    COMPLAIN("--- Non-fatal error ---: Remove ESD not supported \n");
    break;
    
  case UNKNOWN_DECODER_TYPE:
    COMPLAIN("--- Non-fatal error ---: Unknown decoder type \n");
    break;
    
  case INCOMPATIBLE_DECODER_TYPE:
    COMPLAIN("--- Non-fatal error ---: Incompatible decoder type \n");
    break;
    
  case NO_STREAM:
    COMPLAIN("--- Non-fatal error ---: No stream \n");
    break;
    
  case NO_STREAM_TYPE:
    COMPLAIN("--- Non-fatal error ---: No stream type \n");
    break;
    
    /*** SB #### commented out 99/02/19
    case REMOTE_OD_NOT_SUPPORTED:
    COMPLAIN("--- Non-fatal error ---: Remote OD not supported \n");
    break;
    ***/
  case REMOTE_OD_REQUIRES_URL:
    COMPLAIN("--- Non-fatal error ---: Remote OD requires URL \n");
    break;

  case UNKNOWN_URL_PROTOCOL:
    COMPLAIN("--- Non-fatal error ---: Unknown URL protocol \n");
    break;
    
  case BUFFER_TOO_SMALL:
    COMPLAIN("--- Non-fatal error ---: Buffer too small \n");
    break;
    
  //ML - commented out the next case
  //case OPEN_STREAM_FAILED:
  //  COMPLAIN("[AppOpenGl::OnError] OPEN STREAM FAILED \n");
  //  break;

  //ML - added the remaining cases (except for the default case)
  case DECODER_SETUP_FAILURE:
	  COMPLAIN("--- Non-fatal error ---: DECODER_SETUP_FAILURE \n");
	  break;
	  
  case REMOTE_OD_REQUIRES_NEW_SESSION:
	  COMPLAIN("--- Non-fatal error ---: REMOTE_OD_REQUIRES_NEW_SESSION \n");
	  break;
	  
  case REMOTE_OD_FLAG_REQUIRED:
	  COMPLAIN("--- Non-fatal error ---: REMOTE_OD_FLAG_REQUIRED \n");
	  break;
	  
  case CHANNEL_ADD_FAILED:
	  COMPLAIN("--- Non-fatal error ---: CHANNEL_ADD_FAILED \n");
	  break;
	  
  case CHANNEL_DELETE_FAILED:
	  COMPLAIN("--- Non-fatal error ---: CHANNEL_DELETE_FAILED \n");
	  break;
					  
  case DATA_TRANSMISSION_ERROR:
	  COMPLAIN("--- Non-fatal error ---: DATA_TRANSMISSION_ERROR \n");
	  break;
	  
  case INVALID_URL:
	  COMPLAIN("--- Non-fatal error ---: INVALID_URL \n");
	  break;
	  
  case IPMP_NOT_AVAILABLE:
	  COMPLAIN("--- Non-fatal error ---: IPMP_NOT_AVAILABLE \n");
	  break;
	  
  case NO_ACCESS_TO_PROTECTED_STREAM:
	  COMPLAIN("--- Non-fatal error ---: NO_ACCESS_TO_PROTECTED_STREAM \n");
	  break;
	  
  case MISSING_DESCRIPTOR:
	  COMPLAIN("--- Non-fatal error ---: MISSING_DESCRIPTOR \n");
	  break;
	  
  case TOO_MANY_DESCRIPTORS:
	  COMPLAIN("--- Non-fatal error ---: TOO_MANY_DESCRIPTORS \n");
	  break;
	  
  case INITIAL_OBJECT_DESCRIPTOR_NOT_FOUND:
	  COMPLAIN("--- Non-fatal error ---: INITIAL_OBJECT_DESCRIPTOR_NOT_FOUND \n");
	  break;
	  
  case NO_SLCONFIG_INFO:
	  COMPLAIN("--- Non-fatal error ---: NO_SLCONFIG_INFO \n");
	  break;
	  
  case ERROR_IN_PARSING_INITIAL_OBJECT_DESCRIPTOR:
	  COMPLAIN("--- Non-fatal error ---: ERROR_IN_PARSING_INITIAL_OBJECT_DESCRIPTOR \n");
	  break;
	  
  case BIFS_UPDATE_REQUEST_DOES_NOT_MATCH_SCENE:
	  COMPLAIN("--- Non-fatal error ---: BIFS_UPDATE_REQUEST_DOES_NOT_MATCH_SCENE \n");
	  break;
	  
  case MULTIPLE_ACCESS_OF_A_SINGLE_STREAM_NOT_PERMITTED:
	  COMPLAIN("--- Non-fatal error ---: MULTIPLE_ACCESS_OF_A_SINGLE_STREAM_NOT_PERMITTED \n");
	  break;
	  
  case IPMP_STREAM_ALREADY_IN_USE:
	  COMPLAIN("--- Non-fatal error ---: IPMP_STREAM_ALREADY_IN_USE \n");
	  break;
	  
  case INVALID_BIFS_CONFIG:
	  COMPLAIN("--- Non-fatal error ---: INVALID_BIFS_CONFIG \n");
	  break;
	  
  case CANNOT_ALTER_REMOTE_OD:
	  COMPLAIN("--- Non-fatal error ---: CANNOT_ALTER_REMOTE_OD \n");
	  break;
	  
  case TRYING_TO_UPDATE_UNKNOWN_OD:
	  COMPLAIN("--- Non-fatal error ---: TRYING_TO_UPDATE_UNKNOWN_OD \n");
	  break;
	  
  case TRYING_TO_REMOVE_UNKNOWN_ESD:
	  COMPLAIN("--- Non-fatal error ---: TRYING_TO_REMOVE_UNKNOWN_ESD \n");
	  break;
	  
  case OCI_PARSING_ERROR:
	  COMPLAIN("--- Non-fatal error ---: OCI_PARSING_ERROR \n");
	  break;
	  
  case SERVICE_DETACH_FAILED:
	  COMPLAIN("--- Non-fatal error ---: SERVICE_DETACH_FAILED \n");
	  break;
	  
  case SEND_COMMAND_FAILED:
	  COMPLAIN("--- Non-fatal error ---: SEND_COMMAND_FAILED \n");
	  break;
	  
  case UNEXPECTED_SERVICE_DETACH:
	  COMPLAIN("--- Non-fatal error ---: UNEXPECTED_SERVICE_DETACH \n");
	  break;
	  
  case UNEXPECTED_CHANNEL_DELETE:
	  COMPLAIN("--- Non-fatal error ---: UNEXPECTED_CHANNEL_DELETE \n");
	  break;
	  
  case DMIF_FILTER_NOT_FOUND:
	  COMPLAIN("--- Non-fatal error ---: DMIF_FILTER_NOT_FOUND \n");
	  break;

  default:
    COMPLAIN("[AppOpenGl::OnError] unspecified ERROR... \n");
    break;
  }
}

// run application
//ML - changed RESPONSE_PARAMETER to RESPONSE

//ML - it appears that the core code no longer calls this function to signal
//     that the scene has been loaded.

/* virtual */ void
//satish changed this line
//OpenGLApplication::OnConnect(RESPONSE /* result */)
OpenGLApplication::OnConnect(CONNECTION_RESULT /* result */)
{
#if defined(unix)
  
  ::glutDisplayFunc(m_glutDisplayFunction);
  ::glutIdleFunc(m_glutIdleFunction);
  ::glutMouseFunc(m_glutMouseFunction);
  ::glutMotionFunc(m_glutMotionFunction);
  ::glutPassiveMotionFunc(m_glutMotionFunction);
  ::glutKeyboardFunc(m_glutKeyboardFunction);
  ::glutReshapeFunc(m_glutReshapeFunction);
  
  ::glutCreateMenu(m_glutMenuFunction);
  ::glutAttachMenu(GLUT_MIDDLE_BUTTON);
  
  ::glutAddMenuEntry("Nice texturing", 0);
  ::glutAddMenuEntry("Fast texturing", 1);
  ::glutAddMenuEntry("Filled polygons", 2);
  ::glutAddMenuEntry("Wireframe", 3);
  ::glutAddMenuEntry("Show scene graph", 4);
  ::glutAddMenuEntry("Show scene graph with memory locations", 5);
  ::glutAddMenuEntry("Hide scene graph", 6);
  ::glutAddMenuEntry("Show bounding boxes", 7);
  ::glutAddMenuEntry("Hide bounding boxes", 8);
  ::glutAddMenuEntry("Enable dithering", 9);
  ::glutAddMenuEntry("Disable dithering", 10);
  ::glutAddMenuEntry("Enable collision detection", 11);
  ::glutAddMenuEntry("Disable collision detection", 12);
  
  ConfigureOpenGL();
  
#elif defined(WIN32)
  
  // MSG msg;

  TRACE("[AppOpenGl::OnConnect] before message loop \n");
  // SB $$$$ 99/02/24
  /***
  while (!m_bFinished) {
    while (PeekMessage(&msg, NULL, 0, 0, PM_REMOVE)) 
    {
      TranslateMessage(&msg);
      DispatchMessage(&msg);
    }
    InvalidateRect(msg.hwnd, NULL, FALSE);
  }
  ***/
  TRACE("[AppOpenGl::OnConnect] after message loop \n");
  
#endif // unix || WIN32
}

// Finish

void
OpenGLApplication::Finish()
{
  m_pExecutive->Stop();
  
  // delete this;
  
  exit(0);
}

// Bitstream finished
//ML - changed RESPONSE_PARAMETER to RESPONSE

//satish changed this
///*virtual*/ void OpenGLApplication::OnFinish(RESPONSE result)
/*virtual*/ void OpenGLApplication::OnFinish(SERVICE_TERMINATION result)
{
  m_bFinished = TRUE;
  m_nReturn = result;
}

#if defined(GLUT)

// glut displayfunction

void
OpenGLApplication::m_glutDisplayFunction()
{
  RECT repaint = {-1, -1, -1, -1};
  
  m_pApplication->m_pExecutive->Paint(&repaint);
}

// glut idlefunction

/* static */void
OpenGLApplication::m_glutKeyboardFunction(unsigned char key, int /* x */, int /* y */)
{
  switch (key) {
  case 27:             /* ESC */
  case 'q':
  case 'Q':
    m_pApplication->Finish();
    break;
#ifdef MESA_3DFX
  case ' ':
    m_pApplication->m_bFullscreen = !m_pApplication->m_bFullscreen;
    ::XMesaSetFXmode(m_pApplication->m_bFullscreen ? XMESA_FX_FULLSCREEN : XMESA_FX_WINDOW);
    break;
#endif // MESA_3DFX
  }
}

// glut idlefunction

/* static */ void
OpenGLApplication::m_glutIdleFunction()
{
  if (!m_pApplication->m_bFinished)
    ::glutPostRedisplay();
  else
    ::glutIdleFunc(NULL);
}

// glut menufunction

/* static */ void
OpenGLApplication::m_glutMenuFunction(int nMenuItem)
{
  switch (nMenuItem) {
  case 0:
    ::glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
    
    if (!m_pApplication->m_bFinished) {
// SOLARIS PORTING - Added typecast
      OpenGLVisualRenderer* pRenderer = (OpenGLVisualRenderer*) ((OpenGLPresenter *)m_pApplication->m_pExecutive->GetPresenter()) ->GetVisualRenderer();
      
      pRenderer->SetTextureMagFunc(GL_LINEAR);
      pRenderer->SetTextureMipmapMinFunc(GL_LINEAR_MIPMAP_LINEAR);
      pRenderer->SetTextureMinFunc(GL_LINEAR);
    }
    break;
    
  case 1:
    ::glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_FASTEST);
    
    if (!m_pApplication->m_bFinished) {
// SOLARIS PORTING - Added typecast
      OpenGLVisualRenderer* pRenderer = (OpenGLVisualRenderer*) ((OpenGLPresenter *)m_pApplication->m_pExecutive->GetPresenter()) ->GetVisualRenderer();
      
      pRenderer->SetTextureMagFunc(GL_NEAREST);
      pRenderer->SetTextureMinFunc(GL_NEAREST);
      pRenderer->SetTextureMipmapMinFunc(GL_NEAREST);
    }
    break;
    
  case 2:
    ::glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
    ::glEnable(GL_CULL_FACE);
    break;
    
  case 3:
    ::glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
    ::glDisable(GL_CULL_FACE);
    break;
    
  case 4:
    if (!m_pApplication->m_bFinished) {
// SOLARIS PORTING - Added typecast
      OpenGLVisualRenderer* pRenderer = (OpenGLVisualRenderer*) ((OpenGLPresenter *)m_pApplication->m_pExecutive->GetPresenter()) ->GetVisualRenderer();
      
      pRenderer->DrawSceneGraph(1);
    }
    break;
    
  case 5:
    if (!m_pApplication->m_bFinished) {
// SOLARIS PORTING - Added typecast
      OpenGLVisualRenderer* pRenderer = (OpenGLVisualRenderer*) ((OpenGLPresenter *)m_pApplication->m_pExecutive->GetPresenter()) ->GetVisualRenderer();
      
      pRenderer->DrawSceneGraph(-1);
    }
    break;
    
  case 6:
    if (!m_pApplication->m_bFinished) {
// SOLARIS PORTING - Added typecast
      OpenGLVisualRenderer* pRenderer = (OpenGLVisualRenderer*) ((OpenGLPresenter *)m_pApplication->m_pExecutive->GetPresenter()) ->GetVisualRenderer();
      
      pRenderer->DrawSceneGraph(0);
    }
    break;
    
  case 7:
    if (!m_pApplication->m_bFinished) {
// SOLARIS PORTING - Added typecast
      OpenGLVisualRenderer* pRenderer = (OpenGLVisualRenderer*) ((OpenGLPresenter *)m_pApplication->m_pExecutive->GetPresenter()) ->GetVisualRenderer();
      
      pRenderer->DrawBBoxes(TRUE);
    }
    break;
    
  case 8:
    if (!m_pApplication->m_bFinished) {
// SOLARIS PORTING - Added typecast
      OpenGLVisualRenderer* pRenderer = (OpenGLVisualRenderer*) ((OpenGLPresenter *)m_pApplication->m_pExecutive->GetPresenter()) ->GetVisualRenderer();
      
      pRenderer->DrawBBoxes(FALSE);
    }
    break;
    
  case 9:
    ::glEnable(GL_DITHER);
    break;
    
  case 10:
    ::glDisable(GL_DITHER);
    break;
    
  case 11:
    if (!m_pApplication->m_bFinished) {
// SOLARIS PORTING - Added typecast
      OpenGLVisualRenderer* pRenderer = (OpenGLVisualRenderer*) ((OpenGLPresenter *)m_pApplication->m_pExecutive->GetPresenter()) ->GetVisualRenderer();
      
      pRenderer->CollisionDetection(TRUE);
    }
    break;
    
  case 12:
    if (!m_pApplication->m_bFinished) {
// SOLARIS PORTING - Added typecast
      OpenGLVisualRenderer* pRenderer = (OpenGLVisualRenderer*) ((OpenGLPresenter *)m_pApplication->m_pExecutive->GetPresenter()) ->GetVisualRenderer();
      
      pRenderer->CollisionDetection(FALSE);
    }
    break;
  }
}

// Glut mousefunction

void
OpenGLApplication::m_glutMouseFunction(int nButton, int nState, int nX, int nY)
{
  unsigned int modifierState = 0;

// SOLARIS PORTING
/*  
  if (::glutGetModifiers() && GLUT_ACTIVE_SHIFT)
    modifierState |= MODIFIER_SHIFT;
  
  if (::glutGetModifiers() && GLUT_ACTIVE_CTRL)
    modifierState |= MODIFIER_CTRL;
  
  if (::glutGetModifiers() && GLUT_ACTIVE_ALT)
    modifierState |= MODIFIER_ALT;
*/
// SOLARIS PORTING - END  
  if (!m_pApplication->m_bFinished) {
    USER_EVENT userEvent;
    
    if (nState == GLUT_UP)
      switch (nButton) {
      case GLUT_LEFT_BUTTON:
        userEvent = EVENT_LEFT_BUTTON_UP;
        break;
      case GLUT_MIDDLE_BUTTON:
        userEvent = EVENT_MIDDLE_BUTTON_UP;
        break;
      default:
        userEvent = EVENT_RIGHT_BUTTON_UP;
        break;
    }
    else
      switch (nButton) {
      case GLUT_LEFT_BUTTON:
        userEvent = EVENT_LEFT_BUTTON_DOWN;
        break;
      case GLUT_MIDDLE_BUTTON:
        userEvent = EVENT_MIDDLE_BUTTON_DOWN;
        break;
      default:
        userEvent = EVENT_RIGHT_BUTTON_DOWN;
        break;
    }
    
//SOLARIS PORTING
	//satish changed this line
    //m_pApplication->m_pExecutive->OnMouseEvent(userEvent, nX, nY, modifierState);
    //OpenGLPresenter* opr =  (OpenGLPresenter*) m_pApplication->m_pExecutive->GetPresenter();

    //opr->OnMouseEvent(userEvent, nX, nY, modifierState);
    m_pApplication->m_pExecutive->OnUserEvent(userEvent, nX, nY, modifierState);
// SOLARIS PORTING - END
  }
}

// glut motionfunction

void
OpenGLApplication::m_glutMotionFunction(int nX, int nY)
{
  if (!m_pApplication->m_bFinished){
// SOLARIS PORTING
	//satish changed this line
    //m_pApplication->m_pExecutive->OnMouseEvent(EVENT_MOUSE_MOVE, nX, nY, 0);

	//OpenGLPresenter* opr =  (OpenGLPresenter*) m_pApplication->m_pExecutive->GetPresenter();
    	//opr->OnMouseEvent(EVENT_MOUSE_MOVE, nX, nY, 0);
	m_pApplication->m_pExecutive->OnUserEvent(EVENT_MOUSE_MOVE, nX, nY, 0);
// SOLARIS PORTING - END
	}
}

// glut reshapefunction

void
OpenGLApplication::m_glutReshapeFunction(int nX, int nY)
{
  m_pApplication->m_pExecutive->ReSize(nX, nY);
}

#ifdef WITH_GLIB

#include <glib.h>

static int caught_fatal_sig = 0;
static char** params = (char**) NULL;

static void
on_signal (int sig_num)
{
  if (caught_fatal_sig)
    ::kill(::getpid(), sig_num);
  
  caught_fatal_sig = 1;
  
  switch (sig_num) {
  case SIGHUP:
    ::printf("SIGHUP caught\n");
    g_debug(params[0]);
    return;
  case SIGINT:
    ::printf("SIGINT caught\n");
    break;
  case SIGQUIT:
    ::printf("SIGQUIT caught\n");
    break;
  case SIGABRT:
    ::printf("Abort signal (SIGABRT) caught\n");
    g_debug(params[0]);
    break;
  case SIGBUS:
    ::printf("Bus error (SIGBUS) caught\n");
    g_debug(params[0]);
    break;
  case SIGSEGV:
    ::printf("Invalid memory reference (SIGSEGV) caught\n");
    g_debug(params[0]);
    break;
  case SIGPIPE:
    ::printf("SIGPIPE caught\n");
    break;
  case SIGTERM:
    ::printf("SIGTERM caught\n");
    break;
  case SIGFPE:
    ::printf("Floating point exception (SIGFPE) caught\n");
    g_debug(params[0]);
    break;
  default:
    ::printf("unknown signal\n");
    g_debug(params[0]);
    break;
  }
  
  exit(-1);
}

// Print some debug stats about the application

void
OpenGLApplication::Debug()
{
  g_debug(params[0]);
}

#else // WITH_GLIB

// Print some debug stats about the application

void
OpenGLApplication::Debug()
{
}

#endif // WITH_GLIB

int
main(int argc, char* argv[])
{
#ifdef WITH_GLIB
  params = argv;
  
  signal (SIGHUP, on_signal);
  signal (SIGINT, on_signal);
  signal (SIGQUIT, on_signal);
  signal (SIGABRT, on_signal);
  signal (SIGBUS, on_signal);
  signal (SIGSEGV, on_signal);
  signal (SIGPIPE, on_signal);
  signal (SIGTERM, on_signal);
  signal (SIGFPE, on_signal);
#endif // WITH_GLIB
  ::glutInit(&argc, argv);
  ::glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH | GLUT_MULTISAMPLE);
  
#ifdef MESA_3DFX
  ::glutInitWindowPosition(0, 0);
  ::glutInitWindowSize(640, 480);
#else
  ::glutInitWindowSize(256, 256);
#endif // MESA_3D_FX

 
  ::glutCreateWindow("im1");
  
  OpenGLApplication application(&argc, argv);
}

#elif defined(WIN32)

// SB #### 99/02/23 I need to take away this... >>>>
#if 0

HPALETTE hPalCurrent;
Application* application = NULL;

void CreateRGBPalette(HDC hdc)
{
  PIXELFORMATDESCRIPTOR pfd;
  int n = GetPixelFormat(hdc);
  DescribePixelFormat(hdc, n, sizeof(PIXELFORMATDESCRIPTOR), &pfd);
  
  LOGPALETTE* pPal = (LOGPALETTE*)malloc(sizeof(LOGPALETTE) + 256 * sizeof(PALETTEENTRY));
  pPal->palVersion = 0x300;
  pPal->palNumEntries = 256;
  
  assert(pfd.cColorBits == 8);
  n = 1 << pfd.cColorBits;
  unsigned char byRedMask = (1 << pfd.cRedBits) - 1;
  unsigned char byGreenMask = (1 << pfd.cGreenBits) - 1;
  unsigned char byBlueMask = (1 << pfd.cBlueBits) - 1;
  for (int i = 0; i < n; ++i) {
    pPal->palPalEntry[i].peRed   = ((((i >> pfd.cRedShift) & byRedMask) * 255) / byRedMask);
    pPal->palPalEntry[i].peGreen = ((((i >> pfd.cGreenShift) & byGreenMask) * 255) / byGreenMask);
    pPal->palPalEntry[i].peBlue  = ((((i >> pfd.cBlueShift) & byBlueMask) * 255) / byBlueMask);
    pPal->palPalEntry[i].peFlags = 0;
  }
  
  hPalCurrent = CreatePalette(pPal);
  free(pPal);
}

LRESULT CALLBACK WndProc(HWND hwnd, UINT iMsg, WPARAM wParam, LPARAM lParam)
{
  switch (iMsg) {
  case WM_CREATE:
    {
      HDC hdc = ::GetDC(hwnd);
      HGLRC hglrc;
      
      PIXELFORMATDESCRIPTOR pfd;
      
      pfd.nSize      = sizeof(PIXELFORMATDESCRIPTOR);
      pfd.nVersion   = 1;
      pfd.dwFlags    = (PFD_DOUBLEBUFFER |
        PFD_SUPPORT_OPENGL |
        PFD_DRAW_TO_WINDOW);
      pfd.iPixelType = PFD_TYPE_RGBA;
      pfd.cColorBits = 24;
      pfd.cDepthBits = 32;
      pfd.iLayerType = PFD_MAIN_PLANE;
      
      int pixelformat = ::ChoosePixelFormat(hdc, &pfd);
      
      if (pixelformat == 0) {
        TRACE("Could not choose pixelformat\n");
        ::ReleaseDC(hwnd, hdc);
        return -1;
      }
      
      if (!::SetPixelFormat(hdc, pixelformat, &pfd)) {
        TRACE("Could not set pixelformat\n");
        ::ReleaseDC(hwnd, hdc);
        return -1;
      }
      
      ::DescribePixelFormat(hdc, pixelformat, pfd.nSize, &pfd);
      
      if (!(pfd.dwFlags & PFD_GENERIC_FORMAT))
        TRACE("We have a device index\n");
      else
        TRACE("We have a generic format\n");
      
      hglrc = ::wglCreateContext(hdc);
      if (!hglrc) {
        TRACE("Could not create OpenGL render context");
        ReleaseDC(hwnd, hdc);
        return -1;
      }
      ::wglMakeCurrent(hdc, hglrc);
      
      application = new Application(hwnd);
      application->m_hglrc = hglrc;
      
      application->ConfigureOpenGL();
      
      const unsigned char *vendor   = glGetString( GL_VENDOR );
      
      ::wglMakeCurrent(NULL, NULL);
      
      if (pfd.dwFlags & PFD_NEED_PALETTE)
        CreateRGBPalette(hdc);
      
      ::ReleaseDC(hwnd, hdc);
      
      return 0;
    }
    break;
    
  case WM_SIZE: 
    {
      int x = LOWORD(lParam); // width of client area 
      int y = HIWORD(lParam); // height of client area 
      
      HDC hdc = ::GetDC(hwnd);
      
      ::wglMakeCurrent(hdc, application->m_hglrc);
      application->m_pExecutive->ReSize(x, y);	
      ::wglMakeCurrent(NULL, NULL);
      ::ReleaseDC(hwnd, hdc);
    }
    break;
    
  case WM_PAINT:
    {
      PAINTSTRUCT ps;
      HDC hdc = ::BeginPaint(hwnd, &ps);
      
      HPALETTE hOldPal = SelectPalette(hdc, hPalCurrent, TRUE);
      RealizePalette(hdc);
      
      ::wglMakeCurrent(hdc, application->m_hglrc);
      
      application->m_pExecutive->Paint(&ps.rcPaint);
      
      ::wglMakeCurrent(NULL, NULL);
      
      SelectPalette(hdc, hOldPal, TRUE);
      RealizePalette(hdc);
      
      ::EndPaint(hwnd, &ps);
    }
    break;
    
  case WM_ERASEBKGND:
    return 1;
    break;
    
  case WM_PALETTECHANGED:
    if ((HWND)wParam == hwnd)         // Responding to own message.
      // Nothing to do.
      break;
    
  case WM_QUERYNEWPALETTE:
    {
      HDC hDC = GetDC(hwnd);
      HPALETTE hOldPal = SelectPalette(hDC, hPalCurrent, FALSE);
      UINT iTemp = RealizePalette(hDC);         // Realize drawing palette.
      SelectPalette(hDC, hOldPal, TRUE);
      RealizePalette(hDC);
      ReleaseDC(hwnd, hDC);
      
      if (iTemp)                            // Did the realization
        // change?
        InvalidateRect(hwnd, NULL, TRUE); // Yes, so force a 
      // repaint.
      return(iTemp);
    }
    
  case WM_DESTROY:
    ::wglDeleteContext(application->m_hglrc);
    application->Finish(0);
    ::PostQuitMessage(0);
    
    break;
    
  case WM_LBUTTONDOWN: 
    {
      POINTS point = MAKEPOINTS(lParam);
      HDC hdc = ::GetDC(hwnd);
      
      application->m_nMouseButtons = application->m_nMouseButtons | VisualRenderer::LEFT_BUTTON;;
      ::wglMakeCurrent(hdc, application->m_hglrc);
      application->m_pExecutive->GetPresenter()->
        GetVisualRenderer()->MouseState(point.x, point.y, application->m_nMouseButtons);
      ::wglMakeCurrent(NULL, NULL);
      
      return 0;
    }
    
  case WM_LBUTTONUP: 
    {
      POINTS point = MAKEPOINTS(lParam);
      HDC hdc = ::GetDC(hwnd);
      
      application->m_nMouseButtons = application->m_nMouseButtons & ~VisualRenderer::LEFT_BUTTON;
      ::wglMakeCurrent(hdc, application->m_hglrc);
      application->m_pExecutive->GetPresenter()->
        GetVisualRenderer()->MouseState(point.x, point.y, application->m_nMouseButtons);
      ::wglMakeCurrent(NULL, NULL);
      
      return 0;
    }
    
  case WM_MBUTTONDOWN:
    {
      POINTS point = MAKEPOINTS(lParam);
      HDC hdc = ::GetDC(hwnd);
      
      application->m_nMouseButtons = application->m_nMouseButtons | VisualRenderer::MIDDLE_BUTTON;
      ::wglMakeCurrent(hdc, application->m_hglrc);
      application->m_pExecutive->GetPresenter()->
        GetVisualRenderer()->MouseState(point.x, point.y, application->m_nMouseButtons);
      ::wglMakeCurrent(NULL, NULL);
      
      return 0;
    }
    
  case WM_MBUTTONUP: 
    {
      POINTS point = MAKEPOINTS(lParam);
      HDC hdc = ::GetDC(hwnd);
      
      application->m_nMouseButtons = application->m_nMouseButtons & ~VisualRenderer::MIDDLE_BUTTON;;
      ::wglMakeCurrent(hdc, application->m_hglrc);
      application->m_pExecutive->GetPresenter()->
        GetVisualRenderer()->MouseState(point.x, point.y, application->m_nMouseButtons);
      ::wglMakeCurrent(NULL, NULL);
      
      return 0;
    }
    
  case WM_RBUTTONDOWN:
    {
      POINTS point = MAKEPOINTS(lParam);
      HDC hdc = ::GetDC(hwnd);
      
      application->m_nMouseButtons = application->m_nMouseButtons |  VisualRenderer::RIGHT_BUTTON;
      ::wglMakeCurrent(hdc, application->m_hglrc);
      application->m_pExecutive->GetPresenter()->
        GetVisualRenderer()->MouseState(point.x, point.y, application->m_nMouseButtons);
      ::wglMakeCurrent(NULL, NULL);
      
      return 0;
      
    }
    
  case WM_RBUTTONUP: 
    {
      POINTS point = MAKEPOINTS(lParam);
      HDC hdc = ::GetDC(hwnd);
      
      application->m_nMouseButtons = application->m_nMouseButtons & ~VisualRenderer::RIGHT_BUTTON;
      ::wglMakeCurrent(hdc, application->m_hglrc);
      application->m_pExecutive->GetPresenter()->
        GetVisualRenderer()->MouseState(point.x, point.y, application->m_nMouseButtons);
      ::wglMakeCurrent(NULL, NULL);
      
      return 0;
    }
    
  case WM_MOUSEMOVE: 
    {    
      POINTS point = MAKEPOINTS(lParam);
      HDC hdc = ::GetDC(hwnd);
      
      ::wglMakeCurrent(hdc, application->m_hglrc);
      application->m_pExecutive->GetPresenter()->
        GetVisualRenderer()->MouseState(point.x, point.y, application->m_nMouseButtons);
      ::wglMakeCurrent(NULL, NULL);
      
      break;
    }
  }
  
  return DefWindowProc (hwnd, iMsg, wParam, lParam) ;
}

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE /* hPrevInstance */, LPSTR /* szCmdLine */, int iShowCmd)
{
  static char szAppName[] = "Viewpoint";
  HWND        hwnd;
  WNDCLASSEX  wndclass;
  
  wndclass.cbSize        = sizeof(wndclass);
  wndclass.style         = CS_HREDRAW | CS_VREDRAW | CS_OWNDC;
  wndclass.lpfnWndProc   = WndProc;
  wndclass.cbClsExtra    = 0;
  wndclass.cbWndExtra    = 0;
  wndclass.hInstance     = hInstance;
  wndclass.hIcon         = LoadIcon(NULL, IDI_APPLICATION);
  wndclass.hCursor       = LoadCursor(NULL, IDC_CROSS);
  wndclass.hbrBackground = (HBRUSH) ::GetStockObject(WHITE_BRUSH);
  wndclass.lpszMenuName  = NULL;
  wndclass.lpszClassName = szAppName;
  wndclass.hIconSm       = LoadIcon(NULL, IDI_APPLICATION);
  
  ::RegisterClassEx(&wndclass);
  
  DWORD style = WS_OVERLAPPEDWINDOW | WS_CLIPSIBLINGS | WS_CLIPCHILDREN;
  
  hwnd = ::CreateWindow(szAppName,               // window class name
    "IM1",                   // window caption
    style,                   // window style
    CW_USEDEFAULT,           // initial x position
    CW_USEDEFAULT,           // initial y position
    256,                     // initial x size
    256,                     // initial y size
    NULL,                    // parent window handle
    NULL,                    // window menu handle
    hInstance,               // program instance handle
    NULL);		         // creation parameters
  
  ::ShowWindow(hwnd, iShowCmd);
  ::UpdateWindow(hwnd);
  
  application->GetExecutive()->Start("flexmux_channel.1");
}

#endif // 0
// SB #### 99/02/23 I need to take away this... <<<<

#endif // GLUT || WIN32
