// This software module was originally developed by Edward Cooke (Teltec
// Ireland) and Karl Anders ygard (Telenor R&D) in the course of
// development of the MPEG-4 Systems standard (ISO/IEC 14496-1).  This
// software module is an implementation of a part of one or more MPEG-4
// Systems tools as specified by the MPEG-4 Systems standard.  ISO/IEC
// gives users of the MPEG-4 Systems standard free license to this software
// module or modifications thereof for use in hardware or software products
// claiming conformance to the MPEG-4 Systems standard.  Those intending to
// use this software module in hardware or software products are advised
// that its use may infringe existing patents.  The original developer of
// this software module and his/her company, the subsequent editors and
// their companies, and ISO/IEC have no liability for use of this software
// module or modifications thereof in an implementation.  Copyright is not
// released for non MPEG-4 Systems conforming products.  The original
// developer retains full right to use the code for his/her own purpose,
// assign or donate the code to a third party and to inhibit third parties
// from using the code for non MPEG-4 Systems conforming products.  This
// copyright notice must be included in all copies or derivative works.
//
// Copyright 1997, 1998.

#include "OpenGl/OpenGl.h"
#include "OpenGLVisualRenderer.h"
#include "OpenGLPresenter.h"
#include "OpenGLApplication.h"
#include "NodeProxy.h"
#include "Node3DProxy.h"
#include "Matrix.h"
#include "opengl/bindable/FogProxy.h"
#include "opengl/bindable/BackgroundProxy.h"
#include "opengl/sensors/TouchSensorProxy.h"
#include "opengl/group/AnchorProxy.h"
#ifdef PRE_FDIS
#include "opengl/group/Anchor2DProxy.h"
#endif // PRE_FDIS
#include "opengl/sensors/DragSensorProxy.h"
#include "opengl/sensors/TimeSensorProxy.h"

//added by Helge Drumm 06.12.2006 TU Ilmenau.de, Shadow Stuff
#include "opengl/shadow/ShadowRendering.h"  //


// Stefano Battista 98/11/12
/***
#include "group/Root.h"
#include "bindable/Viewpoint.h"
#include "bindable/Fog.h"
#include "bindable/NavigationInfo.h"
***/

#ifdef MMI
#include "opengl/base/SelectList.h"
// #include "group/Transform.h"
#endif

#if defined(GLUT)
//#include <GL/glut.h>
//satish changed this line
#include "glut.h"
#endif
//#include <GL/glu.h>
#if defined(GLTT)
#include <gltt/GLTTFont.h>
#include <gltt/FTFont.h>
#include <gltt/FTGlyph.h>
#include <gltt/FTInstance.h>
#elif defined(GLC)
#include <GL/glc.h>
#endif

#ifndef WIN32
#include <sys/time.h>
#endif

// Constructor

OpenGLVisualRenderer::OpenGLVisualRenderer(OpenGLPresenter* pPresenter)
: VisualRenderer(pPresenter),
m_pPresenter(pPresenter),
m_fFrameRate(25.0f),
m_nLastTime(-1),
m_fFrames(0.0f),
m_bCollisionDetection(FALSE),
m_nDrawSceneGraph(0),
m_bDrawBBoxes(FALSE),
m_bClearScreen(TRUE),
m_bZBuffer(TRUE),

//added by Helge Drumm 06.12.2006 TU Ilmenau.de, Shadow Stuff
m_bABuffer(FALSE),  // set using accmulation buffer default FALSE
m_bHWShadow(FALSE), // set using hardware shadows default FALSE

m_nMouseButtons(0),
m_nMouseX(-1),
m_nMouseY(-1),
m_nMouseFirstX(0),
m_nMouseFirstY(0),
#if defined(FASTER_PUSSYCAT)
m_nMinFunc(GL_NEAREST),
m_nMipmapMinFunc(GL_NEAREST_MIPMAP_NEAREST),
m_nMagFunc(GL_NEAREST),
#else
m_nMinFunc(GL_LINEAR),
m_nMipmapMinFunc(GL_LINEAR_MIPMAP_LINEAR),
m_nMagFunc(GL_LINEAR),
#endif
m_dAspectRatio(1.0f),
m_dNearClipping(0.5f),

//added by Helge Drumm 06.12.2006 TU Ilmenau.de, Shadow Stuff
m_dFarClipping(10000.0f),
m_nViewportWidth(100),
m_nViewportHeight(100),

m_pCurrentTexture(NULL),
m_pCurrentMaterial(NULL),
m_pActiveNode(NULL),
#ifdef MMI
m_pHeadSelectedNode(NULL),
m_cMMovePoint(0),
m_pTailSelectedNode(NULL),
m_cPoint(0),
m_bMovement(FALSE),
m_nNumberofMovements(0),
m_bSelection(FALSE),
#endif
m_nStereoDisplayList(0),
m_nActiveFont(-1),
azimuth_(0)
{
  m_bColorMatrixExtension = IsExtensionSupported("GL_SGI_color_matrix");
  ::sscanf((const char*) ::glGetString(GL_VERSION), "%f", &m_fOpenGLVersion);
  
  SetFontFamily("SERIF", FALSE, FALSE);
  

    //added by Helge Drumm 06.12.2006 TU Ilmenau.de, Shadow Stuff
  	shadowRend = new ShadowRendering(10, 10, 1.0); 	
	shadowRend->initOpenGLExtensions();

	if(shadowRend->supportsTwoDotZero() & (shadowRend->missingRequiredExtension == 0))
	{
		shadowRend->initOpenGLExtensions();
		shadowRend->initOpenGLStats();
		shadowRend->initGLSLShaders();
		shadowRend->initpBuffers();
	}

}


// Destructor

OpenGLVisualRenderer::~OpenGLVisualRenderer()
{
  // make sure we don't delete an active renderer

  m_mtxRendererActive.P();
  m_mtxRendererActive.V();

#if defined(GLTT)
  for (int i = m_generatedFonts.GetLength() - 1; i >= 0; --i) {
    delete m_generatedFonts[i]->font;
    delete m_generatedFonts[i]->face;
	delete m_generatedFonts[i];
  }
#elif defined(GLC)
  ::glcDeleteContext(m_hGLCContext);
#endif
}


// Find whether opengl extension is supported

BOOL
OpenGLVisualRenderer::IsExtensionSupported(const char* sExtension) const
{
  const char* ext = (const char*) ::glGetString(GL_EXTENSIONS);
  
  if (ext) {
    int len = ::strlen(sExtension);
    
    if (len <= 0)
      return 0;
    
    for (;;) {
      const char* next = ::strchr(ext, ' ');
      
      if (next) {
        if (next - ext == len &&
          ::strncmp(ext, sExtension, len) == 0)
          return TRUE;
      } else
        // end of extension list
        
        return ::strcmp(ext, sExtension) == 0;
      
      ext = next + 1;
    }
  } else
    COMPLAIN("glGetString() returned 0; probably called outside GL context\n");
  
  return FALSE;
}

// Terminate

/* virtual */ void
OpenGLVisualRenderer::Terminate()
{
  //  Stop();
  
  m_pPresenter->GetExecutive()->LockScene();
  m_pPresenter->GetExecutive()->UnlockScene();
}

// Paint

/* virtual */ void
OpenGLVisualRenderer::Paint(RECT* pRect)
{

  m_mtxRendererActive.P();
  
  Executive* pExecutive = m_pPresenter->GetExecutive();
  
  pExecutive->LockScene();
  
  //ML - added this next line, as pScene is also pRoot
  Root* pRoot = m_pPresenter->GetScene();
  MediaObject* pScene = pRoot;




  bool a=true;

  m_bHWShadow;

  //****************************************************************************************************
  //*****************               SWITCH SHADOWRENDERING                                  ************
  //****************************************************************************************************

  if ((pScene != NULL) & (shadowRend->hasOpenGL20) & (shadowRend->missingRequiredExtension == 0) & (m_bHWShadow==true))
  { // >> (active scene)  

      
  // Your need a lot of hardware support ...

    //are there any shadows in scene?
    //added by Helge Drumm 06.12.2006 TU Ilmenau.de, Shadow Stuff
	//m_pPresenter->sgm->SG_FileHandle=fopen("sgmana.log","w+");

	// traverse scene graph and all node reference objects
	m_pPresenter->TraverseSG(2000,pScene,m_pPresenter->sgm->SceneGraph);

	// generate the relation shape and lightsource
	m_pPresenter->sgm->GenerateLightShapeMultiMap(m_pPresenter->sgm->LightGroupMMap);

	// generate shape (with shadow relation) and lightsource multimap
	m_pPresenter->sgm->GenerateRenderSLMap();

	// generate shape (without shadow relation) and lightsource multimap 
	m_pPresenter->sgm->GenerateRenderNSLMap(m_pPresenter->sgm->LightGroupMMap);

	// generate renderlist cast(pass1), recieve(pass2), and no shadow/cast (pass3) 
	m_pPresenter->sgm->GenerateMultipassMaps();
		 
	// Enable for debug shadow relations
		 
	//m_pPresenter->sgm->ShowLightMultiMap();
	//fclose(m_pPresenter->sgm->SG_FileHandle);  

	//check the current scene!!! are there any shadows to render????
	m_pPresenter->sgm->checkShadowRender();

 
  if(m_pPresenter->sgm->sg_info.hasShadowToRender)
  {// >> (shadow rendering)

    MediaObjectProxy * pSceneRenderer = (MediaObjectProxy*) pScene->GetRenderingObject();
	//ML - removed this next line, as pScene is also pRoot
    //Root * pRoot = pScene->GetRoot();
    Viewpoint * pActiveViewpoint = pRoot->GetBoundViewpoint();
    Fog * pActiveFog = pRoot->GetBoundFog();
    Background * pActiveBackground = pRoot->GetBoundBackground();
    
    // Update time sensors
    TimeSensorProxy::UpdateTimeSensors();
    
    
    //added by Helge Drumm 06.12.2006 TU Ilmenau.de, Shadow Stuff
	if(m_pPresenter->sgm->sg_info.hasShadowToRender)
	{
  		shadowRend->light_num_max=m_pPresenter->sgm->sg_info.num_direclights+m_pPresenter->sgm->sg_info.num_spotlights
		+m_pPresenter->sgm->sg_info.num_pointlights;
		shadowRend->initMultipassTextures();

		m_nTextureEnabled=0;
		m_nEmissiveEnabled=0;
		m_nMaterialEnabled=0;
	}
    

    // SB 99/05/14
#ifdef _DEBUG
    // pSceneRenderer->DumpTrace();
#endif // _DEBUG

    Drag();
    Select(m_nMouseButtons, m_nMouseButtons);
    
    if (m_bZBuffer)
      ::glEnable(GL_DEPTH_TEST);
    else
      ::glDisable(GL_DEPTH_TEST);
    
    if (pActiveViewpoint != NULL)
    { // >> (active viewpoint)
      ViewpointProxy * poViewpointProxy = (ViewpointProxy *) pActiveViewpoint->GetRenderingObject();

      { // >> (compute new viewpoint, checking mouse buttons)
        
        Vec3f position(poViewpointProxy->GetPosition());
        Rotation orientation(poViewpointProxy->GetOrientation());
        
        //added by Helge Drumm 06.12.2006 TU Ilmenau.de, Shadow Stuff
			shadowRend->cam_pos[0] = position.x;
			shadowRend->cam_pos[1] = position.y;
			shadowRend->cam_pos[2] = position.z;

			Vec3f oldPosition = position;
			Rotation oldOrientation = orientation;

			//added by Helge Drumm 06.12.2006 TU Ilmenau.de, Shadow Stuff
			shadowRend->movingLight=0;
			shadowRend->quickLightMove=0;
        
        // TRACE("[RendVideo::Paint] MouseButtons %04x \n", m_nMouseButtons);
        if (m_nMouseButtons)
        { // >> (some mouse button)
          Vec3f move;
          float theta = ::cos(orientation.theta * 0.5f);
          Vec3f dir(orientation.x, orientation.y, orientation.z);
          float sw;
          
          // normalise dir
          dir.Normalize();
          
          sw = ::sin(orientation.theta * 0.5f);
          dir.x *= sw;
          dir.y *= sw;
          dir.z *= sw;

//        if (((m_nMouseButtons & LEFT_BUTTON) &&
//            (m_nMouseButtons & RIGHT_BUTTON)) ||
//            (m_nMouseButtons & MIDDLE_BUTTON))
          
					if (m_nMouseButtons & LEFT_BUTTON)
						{ // >> (2 muose buttons)
						azimuth_+=180 * float(m_nMouseFirstX - m_nMouseX) / float(m_nViewportWidth);
						move.z = - float(m_nMouseFirstY - m_nMouseY) / float(m_nViewportHeight);    

    
						} // << (2 muose buttons)
          else
						{ // >> (only 1 muose button)
            if (m_nMouseButtons & LEFT_BUTTON)
            {
              float omega = 5.0f * float(m_nMouseFirstX - m_nMouseX) / float(m_nViewportWidth);
              if (omega != 0.0)
              {
                  float aa = dir.x * dir.x, 
                  ab = dir.x * dir.y,
                  ad = dir.x * theta,
                  bb = dir.y * dir.y, 
                  bc = dir.y * dir.z,
                  cc = dir.z * dir.z, 
                  cd = dir.z * theta,
                  dd = theta * theta;
                
                float s1 =        ::cos(omega * M_PI / 360.0f);
                float sin_angle = ::sin(omega * M_PI / 360.0f);
                Vec3f v1(2.0 * (ab - cd) * sin_angle, (- aa + bb - cc + dd) * sin_angle,  2.0 * (bc + ad) * sin_angle);
                Vec3f v2 = dir;
                float s2 = theta;
                
                dir.x = s1 * v2.x + s2 * v1.x + v1.y * v2.z - v1.z * v2.y;
                dir.y = s1 * v2.y + s2 * v1.y + v1.z * v2.x - v1.x * v2.z;
                dir.z = s1 * v2.z + s2 * v1.z + v1.x * v2.y - v1.y * v2.x;
                theta = s2 * s1 - (v1.x * v2.x + v1.y * v2.y + v1.z * v2.z);
              }
              
              move.z = - float(m_nMouseFirstY - m_nMouseY) / float(m_nViewportHeight);    
            }
            else
            {
              if (m_nMouseButtons & RIGHT_BUTTON)
              {
                move.x =   float(m_nMouseX - m_nMouseFirstX) / float(m_nViewportWidth);
                move.y = - float(m_nMouseY - m_nMouseFirstY) / float(m_nViewportHeight);
              }
            }
          } // << (only right mouse button)
          
          {  
            orientation.x = dir.x;
            orientation.y = dir.y;
            orientation.z = dir.z;
            orientation.theta = 2.0 * ::acos(theta);
            NavigationInfo* pNavigationInfo = pRoot->GetBoundNavigationInfo();
            float speed = (pNavigationInfo ? ((NavigationInfo *) pNavigationInfo->GetRenderingObject())->speed /* GetSpeed() */ : 1.0f) / m_fFrameRate;
            
            orientation.Normalize();
            
            float aa = dir.x * dir.x, ab = dir.x * dir.y, ac = dir.x * dir.z, ad = dir.x * theta;
            float                     bb = dir.y * dir.y, bc = dir.y * dir.z, bd = dir.y * theta;
            float                                         cc = dir.z * dir.z, cd = dir.z * theta;
            float                                                             dd = theta * theta;
            
            float T[3][3] = {{aa - bb - cc + dd,     2.0 * (ab - cd),      2.0 * (ac + bd)},
            {  2.0 * (ab + cd), - aa + bb - cc + dd,      2.0 * (bc - ad)},
            {  2.0 * (ac - bd),     2.0 * (bc + ad), - aa - bb + cc + dd}};
            
            float t[3];
            
            for (int j = 0; j < 3; j++)
              t[j] = move.x * T[j][0] + move.y * T[j][1] + move.z * T[j][2];
            
            position.x += t[0] * speed *100;
            position.y += t[1] * speed *100;
            position.z += t[2] * speed *100;
          }
          
          pActiveViewpoint->position = /* SetPosition */ position;
          pActiveViewpoint->orientation = /* SetOrientation */ orientation;
          
          if (m_bCollisionDetection)
          {
            /* Node */ MediaObject * pNode;
            
            // Do collision detection
            pNode = poViewpointProxy->DetectCollision();
            if (pNode) {
              COMPLAIN("Collision\n");
              
              pActiveViewpoint->position = /* SetPosition */ oldPosition;
              pActiveViewpoint->orientation = /* SetOrientation */ oldOrientation;
            }
          }
        }  // << (some mouse button)
      } // << (compute new viewpoint, checking mouse buttons)
      
      Matrix transform;
      
      // load projection matrix from viewpoint
      
      ::glMatrixMode(GL_PROJECTION);
      
      ::glLoadIdentity();
      
      ::gluPerspective(pActiveViewpoint->fieldOfView /* GetFieldOfView() */ * 180.0f / M_PI,
        m_dAspectRatio,
        m_dNearClipping, m_dFarClipping);
      
      ::glMatrixMode(GL_MODELVIEW);
      
      ::glLoadIdentity();
      
      {
        float xmax, ymax;
        
        ymax = 0.5f * ::tan(pActiveViewpoint->fieldOfView /* GetFieldOfView() */ * 0.5f);
        xmax = ymax * m_dAspectRatio;
        
        Vec3f ll(-xmax, -ymax, -0.5f);
        Vec3f ul(-xmax,  ymax, -0.5f);
        Vec3f lr( xmax, -ymax, -0.5f);
        Vec3f ur( xmax,  ymax, -0.5f);
        
        m_leftFrustumPlane =   ul;
        m_rightFrustumPlane =  lr;
        m_topFrustumPlane =    ur;
        m_bottomFrustumPlane = ll;
        
        m_leftFrustumPlane.Cross(ll);
        m_rightFrustumPlane.Cross(ur);
        m_topFrustumPlane.Cross(ul);
        m_bottomFrustumPlane.Cross(lr);
      }
      
	/*--------------------------------------------------------------------------
	Multipass Rendering for Realtime Shadowing and Lighting
    //added by Helge Drumm 06.12.2006 TU Ilmenau.de, Shadow Stuff
	---------------------------------------------------------------------------*/
			 
	if((shadowRend->hasOpenGL20) & (shadowRend->missingRequiredExtension == 0) & (m_pPresenter->sgm->sg_info.hasShadowToRender))
	{  
			 
			 
	// enable fog, if we have any
				if(pActiveFog)
					{
					((FogProxy*) pActiveFog->GetRenderingObject())->Enable(this, m_pPresenter->GetTime());
					}
	

      
      // set viewpoint to world transformation
      poViewpointProxy->ViewpointToWorldMatrix(m_pPresenter->GetTime(), transform);
      ::glLoadMatrixf(transform);
      
	   glRotatef(azimuth_,0,1,0);

	 			//shadowRend->buildPerspectiveMatrix(shadowRend->eyeFrustumMatrix,pActiveViewpoint->fieldOfView*180.0f / M_PI,
		  //	                                 m_dAspectRatio, m_dNearClipping, m_dFarClipping);
		  
			// Get MODELVIEW_MATRIX (Camera/Viewpoint)
			::glGetFloatv(GL_MODELVIEW_MATRIX,shadowRend->tempcopy);
		  
			// Copy of MODELVIEW_MATRIX as eyeViewMatrix to aviod an modifications on Modelviewmatrix
			shadowRend->copyMatrixf(shadowRend->eyeViewMatrix,shadowRend->tempcopy);
		  
      shadowRend->buildPerspectiveMatrix(shadowRend->eyeFrustumMatrix,pActiveViewpoint->fieldOfView *180.0f / M_PI,
          m_dAspectRatio,m_dNearClipping,m_dFarClipping);

/**** sobald sich die Fenstergre ndert... Variablen in ShadowRendering anpassen!!*/			
		  if(shadowRend->winWidth!= m_nViewportWidth || shadowRend->winHeight!=m_nViewportHeight)
				{
				//shadowRend->set_win(m_nViewportWidth, m_nViewportHeight, m_dAspectRatio);
				
				if(m_nViewportWidth > 512 || m_nViewportHeight > 512)
					{
					shadowRend->set_win(1024, 1024,m_dAspectRatio, 1024);
					}
				else
					{
					shadowRend->set_win(512, 512,m_dAspectRatio, 512);
					}

			//	shadowRend->set_win(512, 512,1,512);
				shadowRend->updatergbpBufferSize();
				if (pActiveFog) shadowRend->updatefogpBufferSize();
// --> pBuffer!
// --> Texturen!!

				// added by M.Frank
				//shadowRend->updatepBufferSize(m_nViewportHeight, m_nViewportWidth);
				//shadowRend->initpBuffers(m_nViewportHeight,m_nViewportWidth);
        
/**** removed by M.Frank
				if(!m_bABuffer)
					{             
					shadowRend->framebuffers_realloc(m_nViewportHeight,m_nViewportWidth);
					}
				shadowRend->updateDepthMapSize();
*/
				}
				
		  shadowRend->light_num_max=m_pPresenter->sgm->sg_info.num_direclights+m_pPresenter->sgm->sg_info.num_spotlights
			                        +m_pPresenter->sgm->sg_info.num_pointlights;

// hier nochmals, wegen dem mehrmaligen Durchlaufen der Schleife		  
		  ::glEnable(GL_LIGHTING);
		  ::glMatrixMode(GL_PROJECTION);
			::glLoadIdentity();
			::gluPerspective(pActiveViewpoint->fieldOfView*180.0f/M_PI, m_dAspectRatio, m_dNearClipping, m_dFarClipping);
			::glMatrixMode(GL_MODELVIEW);
			::glLoadIdentity();
			::glLoadMatrixf(transform);



//----------------------------------------------------------------------------------------------------      
//BEGIN LightMainLoop ********************************************************************************
//----------------------------------------------------------------------------------------------------
			
			for(int lnr=1; lnr<=shadowRend->light_num_max; lnr++)
				{

				//set active lightsource

/*** removed by M.Frank 11-2005 ***
				if(lnr==1 && pActiveBackground)
			   {
			   ((BackgroundProxy*) pActiveBackground->GetRenderingObject())->RenderBackground(this,m_pPresenter->GetTime());
			   }
*/			
				
		::glEnable(GL_DEPTH_TEST);
		::glDepthFunc(GL_LEQUAL);
		::glDepthRange(0.0,1.0);

		m_pPresenter->sgm->SetActiveLight(lnr);
		m_pPresenter->sgm->shapecount=0;
			
		m_nTextureEnabled=0;
		m_nMaterialEnabled=0;

			// *** removed by M.Frank 11-2005 ***
			//::glEnable(GL_LIGHTING);

			//::glClearColor(0,0,0,1);
			//::glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);




//----------------------------------------------------------------------------------------------------
// BEGIN Rendering SpotLight -------------------------------------------------------------------------
//----------------------------------------------------------------------------------------------------
       if(m_pPresenter->sgm->activeLight->NodeCode == CODE_SpotLight)
			  {
			  float beamW,cutOffA,spotExp;
				
				//if(lnr==1 && pActiveBackground)
			  // {
			  // ((BackgroundProxy*) pActiveBackground->GetRenderingObject())->RenderBackground(this, m_pPresenter->GetTime());
			  // }
			  
			  shadowRend->set_lightColor(((NRSpotLight*)m_pPresenter->sgm->activeLight->Light)->color.r,
																	 ((NRSpotLight*)m_pPresenter->sgm->activeLight->Light)->color.g,
																	 ((NRSpotLight*)m_pPresenter->sgm->activeLight->Light)->color.b,
																	 ((NRSpotLight*)m_pPresenter->sgm->activeLight->Light)->intensity);

			  shadowRend->set_ambLightColor(((NRSpotLight*)m_pPresenter->sgm->activeLight->Light)->color.r,
																			((NRSpotLight*)m_pPresenter->sgm->activeLight->Light)->color.g,
																			((NRSpotLight*)m_pPresenter->sgm->activeLight->Light)->color.b,
																			((NRSpotLight*)m_pPresenter->sgm->activeLight->Light)->ambientIntensity);
				
				shadowRend->set_ambLightColorSum();

				shadowRend->set_lightDir(((NRSpotLight*)m_pPresenter->sgm->activeLight->Light)->direction.x,
			                           ((NRSpotLight*)m_pPresenter->sgm->activeLight->Light)->direction.y,
																 ((NRSpotLight*)m_pPresenter->sgm->activeLight->Light)->direction.z,
																 0.0f);

			  shadowRend->set_lightPos(((NRSpotLight*)m_pPresenter->sgm->activeLight->Light)->location.x,
																 ((NRSpotLight*)m_pPresenter->sgm->activeLight->Light)->location.y,
																 ((NRSpotLight*)m_pPresenter->sgm->activeLight->Light)->location.z,
																 1.0f);

				shadowRend->set_attenuation(((NRSpotLight*)m_pPresenter->sgm->activeLight->Light)->attenuation.x,
																		((NRSpotLight*)m_pPresenter->sgm->activeLight->Light)->attenuation.y,
																		((NRSpotLight*)m_pPresenter->sgm->activeLight->Light)->attenuation.z);
				
				shadowRend->set_lightRadius(((NRSpotLight*)m_pPresenter->sgm->activeLight->Light)->radius);

				cutOffA = (((NRSpotLight*)m_pPresenter->sgm->activeLight->Light)->cutOffAngle) * 180.0f / M_PI;
				beamW = (((NRSpotLight*)m_pPresenter->sgm->activeLight->Light)->beamWidth) * 180.0f / M_PI;

				//beamW = (((NRSpotLight*)m_pPresenter->sgm->activeLight->Light)->cutOffAngle) * 180.0f / M_PI;
				//cutOffA = (((NRSpotLight*)m_pPresenter->sgm->activeLight->Light)->beamWidth) * 180.0f / M_PI;

				if(beamW > cutOffA) { beamW = cutOffA; }
				if(beamW == 0 || cutOffA==0) { spotExp = 0; }
				if(cutOffA >= beamW) 
					{
          spotExp = cutOffA/beamW;
					}
				else
					{
					spotExp = 1.0;
					}
				
			  Vec3f &spot_direction = ((NRSpotLight*)m_pPresenter->sgm->activeLight->Light)->direction;
				
				//neu
				//shadowRend->lightSpotCutoff=cutOffA;
				shadowRend->set_spot(cutOffA, spotExp);

			  ::glPushMatrix();
			  shadowRend->copyMatrixff(shadowRend->tempcopy,((NRSpotLight*)(m_pPresenter->sgm->activeLight->Light))->Matrix4x4->m_elements);
				::glLoadMatrixf(shadowRend->tempcopy);
				::glLightfv(GL_LIGHT0,GL_POSITION,shadowRend->light_dir);
				::glPopMatrix();

        shadowRend->setupEyeView();
			  
				m_nTextureEnabled=0;
				m_nMaterialEnabled=0;

			  m_pPresenter->sgm->renderPass = 5;
			  CullFlag cullFlag = PARTIALLY_INSIDE;
					
				// M.Frank added these lines (11-2005)
				// calculates lightmap and saves in corresponding texture
			  shadowRend->renderDiffLightMap(lnr, 2, pSceneRenderer,transform, cullFlag,
                           this, m_pPresenter->GetTime());		// 2 stands for spotlight
				
				m_pPresenter->sgm->shapecount=0;
				m_pPresenter->sgm->renderPass = 5;

				m_nTextureEnabled=0;
				m_nMaterialEnabled=0;
				shadowRend->renderSpecLightMap(lnr, 2, pSceneRenderer,transform, cullFlag,
                           this, m_pPresenter->GetTime());		// 2 stands for spotlight

			  m_pPresenter->sgm->shapecount=0;

	//BEGIN SHADOWS IF THERE ARE ANY

				// render shadowmap for actual lightsource, if there are cast AND receive objects for this lightsource
				if(m_pPresenter->sgm->activeLight->receive_objects>0 && m_pPresenter->sgm->activeLight->cast_objects>0)
					{

					GLdouble tempMatrix[16],tempMatrix2[16],resultMatrix[16];
					Matrix transformX;
					
					shadowRend->penumbra=((NRSpotLight*)m_pPresenter->sgm->activeLight->Light)->parent->penumbra;
		
	// SHARP SHADOWS,

					if((float)shadowRend->penumbra <= 0)
						{
						::glPushMatrix();
						::glGetDoublev(GL_MODELVIEW_MATRIX,tempMatrix);
						shadowRend->copyMatrixf(tempMatrix2,((NRSpotLight*)(m_pPresenter->sgm->activeLight->Light))->Matrix4x4->m_elements);
						shadowRend->multMatrices(resultMatrix,tempMatrix,tempMatrix2);
          
						::glLightfv(GL_LIGHT0,GL_POSITION,shadowRend->light_pos);
						::glLightfv(GL_LIGHT0,GL_SPOT_DIRECTION,spot_direction);
						::glLightf(GL_LIGHT0, GL_SPOT_CUTOFF,cutOffA);
						::glLightf(GL_LIGHT0, GL_SPOT_EXPONENT, spotExp);
						::glLightf(GL_LIGHT0, GL_CONSTANT_ATTENUATION, 1.0f);
						::glLightf(GL_LIGHT0, GL_LINEAR_ATTENUATION, 0.0f);
						::glPopMatrix();

						::glLoadMatrixd(resultMatrix);
						shadowRend->buildPerspectiveMatrix(shadowRend->lightFrustumMatrix,
													((NRSpotLight*)m_pPresenter->sgm->activeLight->Light)->cutOffAngle * 180.0f *2 / M_PI,
													1.0,5.0,500.0);

			
						float pos_light_x[] = {0,0,shadowRend->penumbra,0,-shadowRend->penumbra,0};
						float pos_light_z[] = {0,0,0,shadowRend->penumbra,0,-shadowRend->penumbra};
						
						int softpass;
						float oldLightHeight;
						oldLightHeight = shadowRend->light_pos[2];
						
						shadowRend->buildLookAtMatrix(shadowRend->lightViewMatrix,
						shadowRend->light_pos[0]+shadowRend->light_dir[0],
						shadowRend->light_pos[1]+shadowRend->light_dir[1],
						shadowRend->light_pos[2]+shadowRend->light_dir[2],
						shadowRend->light_dir[0],
						shadowRend->light_dir[1],
						shadowRend->light_dir[2],
						0.0,1.0,0.0); 
						
						shadowRend->multMatrices(tempMatrix,tempMatrix2,shadowRend->lightViewMatrix);
						shadowRend->copyMatrix(shadowRend->lightViewMatrix,tempMatrix);
						
						for (int i=0; i<16; i++)
							{
							transformX[i]=shadowRend->lightViewMatrix[i];
							}
						
						// render cast objects from lights point of view:
						if(m_pPresenter->sgm->activeLight->cast_objects!=0)
							{
							shadowRend->setupLightView(0);
							
							m_nTextureEnabled=0;
							m_nMaterialEnabled=0;
							
							::glClear(GL_DEPTH_BUFFER_BIT);
							m_pPresenter->sgm->renderPass = 1;
							shadowRend->renderDepthMap(lnr, pSceneRenderer, transformX, cullFlag,
														 this, m_pPresenter->GetTime());		// 2 stands for spotlight
							m_pPresenter->sgm->shapecount=0;
							}

						// render receive objects from eyes point of view:
						if(m_pPresenter->sgm->activeLight->receive_objects!=0)
							{
							shadowRend->setupEyeView();

							m_nTextureEnabled=0;
							m_nMaterialEnabled=0;

							m_pPresenter->sgm->renderPass = 2;
							shadowRend->renderShadowMap(lnr, pSceneRenderer, transform, cullFlag,
														 this, m_pPresenter->GetTime());		// 2 stands for spotlight
							m_pPresenter->sgm->shapecount=0;
							}	
						if(m_pPresenter->sgm->activeLight->noshadow_objects!=0)
							{
							m_nTextureEnabled=0;
							m_nMaterialEnabled=0;

							m_pPresenter->sgm->renderPass=3;
							shadowRend->renderShadowMap_pass3(lnr, pSceneRenderer, transform, cullFlag,
														 this, m_pPresenter->GetTime());		// 2 stands for spotlight
							m_pPresenter->sgm->shapecount=0;
							}
							/*
							// render no shadow objects from eyes point of view:
							if(m_pPresenter->sgm->activeLight->noshadow_objects!=0)
								{	
								m_pPresenter->sgm->renderPass=3;
								shadowRend->setupEyeView();
								::glLightfv(GL_LIGHT0, GL_AMBIENT, shadowRend->zero);  // AmbientLight on
								::glLightfv(GL_LIGHT0, GL_SPECULAR,shadowRend->lightColor); // lightColor
								::glLightfv(GL_LIGHT0, GL_DIFFUSE, shadowRend->lightColor);  // lightColor
								pSceneRenderer->CullOrRender(transform, cullFlag,
															this, m_pPresenter->GetTime()); 
								m_pPresenter->sgm->shapecount=0;
							}
							*/
						}	// END SHARP SHADOWS
		
	//BEGIN SOFT SHADOWS

				if((float)shadowRend->penumbra > 0.0f)
					{
					shadowRend->softpass_num=5;
					int softpass;
					
					// begin softpass loop
					for(softpass=0;softpass<=shadowRend->softpass_num;softpass++)
						{

					// calculate transformation matrix	
						::glPushMatrix();
						::glGetDoublev(GL_MODELVIEW_MATRIX,tempMatrix);
						shadowRend->copyMatrixf(tempMatrix2,((NRSpotLight*)(m_pPresenter->sgm->activeLight->Light))->Matrix4x4->m_elements);
						shadowRend->multMatrices(resultMatrix,tempMatrix,tempMatrix2);
          
						::glLightfv(GL_LIGHT0,GL_POSITION,shadowRend->light_pos);
						::glLightfv(GL_LIGHT0,GL_SPOT_DIRECTION,spot_direction);
						::glLightf(GL_LIGHT0, GL_SPOT_CUTOFF,cutOffA);
						::glLightf(GL_LIGHT0, GL_SPOT_EXPONENT, spotExp);
						::glLightf(GL_LIGHT0, GL_CONSTANT_ATTENUATION, 1.0f);
						::glLightf(GL_LIGHT0, GL_LINEAR_ATTENUATION, 0.0f);
						::glPopMatrix();

						::glLoadMatrixd(resultMatrix);
						 shadowRend->buildPerspectiveMatrix(shadowRend->lightFrustumMatrix,
													((NRSpotLight*)m_pPresenter->sgm->activeLight->Light)->cutOffAngle *2 * 180.0f / M_PI,
													1.0,5.0,500.0);

						
						float pos_light_x[] = {0,0,shadowRend->penumbra,0,-shadowRend->penumbra,0};
						float pos_light_z[] = {0,0,0,shadowRend->penumbra,0,-shadowRend->penumbra};
						
						float oldLightHeight;
						oldLightHeight = shadowRend->light_pos[2];
						
						if(shadowRend->softpass_num>0)
						{
						shadowRend->buildLookAtMatrix(shadowRend->lightViewMatrix,
						shadowRend->light_pos[0]+pos_light_x[softpass]+shadowRend->light_dir[0],
						shadowRend->light_pos[1]+shadowRend->light_dir[1],
						shadowRend->light_pos[2]+pos_light_z[softpass]+shadowRend->light_dir[2],
						shadowRend->light_pos[0]+pos_light_x[softpass]+shadowRend->light_dir[0]+shadowRend->light_dir[0],
						shadowRend->light_pos[1]+shadowRend->light_dir[1]+shadowRend->light_dir[1],
						shadowRend->light_pos[2]+pos_light_z[softpass]+shadowRend->light_dir[2]+shadowRend->light_dir[2],
						0.0,1.0,0.0);
						} 
						else{
						shadowRend->buildLookAtMatrix(shadowRend->lightViewMatrix,
						shadowRend->light_pos[0]+shadowRend->light_dir[0],
						shadowRend->light_pos[1]+shadowRend->light_dir[1],
						shadowRend->light_pos[2]+shadowRend->light_dir[2],
						shadowRend->light_dir[0],
						shadowRend->light_dir[1],
						shadowRend->light_dir[2],
						0.0,1.0,0.0); 
						}

						shadowRend->multMatrices(tempMatrix,tempMatrix2,shadowRend->lightViewMatrix);
						shadowRend->copyMatrix(shadowRend->lightViewMatrix,tempMatrix);
						
						for (int i=0; i<16; i++)
							{
							transformX[i]=shadowRend->lightViewMatrix[i];
							}


						// Draw Depth Map
							shadowRend->setupLightView(0);
							
							m_nTextureEnabled=0;
							m_nMaterialEnabled=0;

							::glClear(GL_DEPTH_BUFFER_BIT);
							m_pPresenter->sgm->renderPass = 1;
							shadowRend->renderDepthMap(lnr, pSceneRenderer, transformX, cullFlag,
														 this, m_pPresenter->GetTime());		// 2 stands for spotlight
							m_pPresenter->sgm->shapecount=0;
							
						// render shadowmap 1st pass
							if(m_pPresenter->sgm->activeLight->receive_objects!=0)
								{
								shadowRend->setupEyeView();

								m_nTextureEnabled=0;
								m_nMaterialEnabled=0;

								m_pPresenter->sgm->renderPass = 2;
								shadowRend->renderSoftShadowMap(lnr, softpass, pSceneRenderer, transform, cullFlag,
														 this, m_pPresenter->GetTime());		// 2 stands for spotlight
								m_pPresenter->sgm->shapecount=0;
								}
							
						// render shadowmap 2nd pass
							if(m_pPresenter->sgm->activeLight->noshadow_objects!=0)
								{
								m_nTextureEnabled=0;
								m_nMaterialEnabled=0;

								m_pPresenter->sgm->renderPass=3;
								shadowRend->renderSoftShadowMap_pass3(pSceneRenderer, transform, cullFlag,
														 this, m_pPresenter->GetTime());		// 2 stands for spotlight
								m_pPresenter->sgm->shapecount=0;
								}

							//Compositing of the jittered soft shadow map
							} // END SOFT SHADOWS
							shadowRend->addSoftShadowMaps(lnr);
					}	//END SHADOWES
				
				//compose scenemaps (shadowmap * corresponding lightmap)
				shadowRend->renderDiffSceneLight(lnr);
				shadowRend->renderSpecSceneLight(lnr);

			  } 
				
			}
//----------------------------------------------------------------------------------------------------
// END Rendering SpotLight ---------------------------------------------------------------------------   
//----------------------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------------------
// BEGIN Rendering PointLight ------------------------------------------------------------------------
//---------------------------------------------------------------------------------------------------- 
			 if(m_pPresenter->sgm->activeLight->NodeCode == CODE_PointLight)
			  {

			  /*** removed by M.Frank 11-2005 ***
				//if(lnr==1 && pActiveBackground)
			  // {
			  // ((BackgroundProxy*) pActiveBackground->GetRenderingObject())->RenderBackground(this, m_pPresenter->GetTime());
			  // }
			  //*/
			  shadowRend->set_lightColor(((NRPointLight*)m_pPresenter->sgm->activeLight->Light)->color.r,
																	 ((NRPointLight*)m_pPresenter->sgm->activeLight->Light)->color.g,
																	 ((NRPointLight*)m_pPresenter->sgm->activeLight->Light)->color.b,
																	 ((NRPointLight*)m_pPresenter->sgm->activeLight->Light)->intensity);

			  shadowRend->set_ambLightColor(((NRPointLight*)m_pPresenter->sgm->activeLight->Light)->color.r,
																	  	((NRPointLight*)m_pPresenter->sgm->activeLight->Light)->color.g,
																			((NRPointLight*)m_pPresenter->sgm->activeLight->Light)->color.b,
																			((NRPointLight*)m_pPresenter->sgm->activeLight->Light)->ambientIntensity);
				
				shadowRend->set_ambLightColorSum();

			  shadowRend->set_lightPos(((NRPointLight*)m_pPresenter->sgm->activeLight->Light)->location.x,
																 ((NRPointLight*)m_pPresenter->sgm->activeLight->Light)->location.y,
																 ((NRPointLight*)m_pPresenter->sgm->activeLight->Light)->location.z,
																 1.0f);

				shadowRend->set_attenuation(((NRSpotLight*)m_pPresenter->sgm->activeLight->Light)->attenuation.x,
																		((NRSpotLight*)m_pPresenter->sgm->activeLight->Light)->attenuation.y,
																		((NRSpotLight*)m_pPresenter->sgm->activeLight->Light)->attenuation.z);
	
				shadowRend->set_lightRadius(((NRPointLight*)m_pPresenter->sgm->activeLight->Light)->radius);

/*** removed by M.Frank 11-2005 ***
				::glPushMatrix();
			  shadowRend->copyMatrixff(shadowRend->tempcopy,((NRPointLight*)(m_pPresenter->sgm->activeLight->Light))->Matrix4x4->m_elements);
				::glLoadMatrixf(shadowRend->tempcopy);
				//::glLightfv(GL_LIGHT2,GL_POSITION,shadowRend->light_pos);
				::glPopMatrix();
*/
        shadowRend->setupEyeView();

			   m_pPresenter->sgm->renderPass = 5;
			   CullFlag cullFlag = PARTIALLY_INSIDE;
					
				 // M.Frank changed these lines (11-2005) -----------------------------------------
			   shadowRend->renderDiffLightMap(lnr, 1, pSceneRenderer,transform, cullFlag,
                           this, m_pPresenter->GetTime());		// 1 stands for pointlight
				 //pSceneRenderer->CullOrRender(transform,cullFlag,this,m_pPresenter->GetTime());
				 // -------------------------------------------------------------------------------
				 m_pPresenter->sgm->shapecount=0;
				

				 shadowRend->renderSpecLightMap(lnr, 1, pSceneRenderer,transform, cullFlag,
                           this, m_pPresenter->GetTime());

			   m_pPresenter->sgm->shapecount=0;
               
			   
				 // M.Frank removed these lines (11-2005)
			   
				 //if(m_bABuffer) {
				 //  ::glAccum(GL_ACCUM,1.0f/(float)shadowRend->light_num_max);
			   //}else {
			   //::glReadPixels(0,0,shadowRend->winWidth,shadowRend->winHeight,GL_RGB,GL_UNSIGNED_BYTE,shadowRend->fbuffer[lnr-1]);}
         //      ::glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
			   //::glDisable(GL_LIGHT2);
			   //::glDisable(GL_LIGHTING);
				 //
			  } 
//----------------------------------------------------------------------------------------------------
// END Rendering PointLight --------------------------------------------------------------------------
//----------------------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------------------
// BEGIN Rendering DirectionalLight ------------------------------------------------------------------
//----------------------------------------------------------------------------------------------------
      if(m_pPresenter->sgm->activeLight->NodeCode == CODE_DirectionalLight)
				{
			  //if(lnr==1 && pActiveBackground)
				//	{
				//	((BackgroundProxy*) pActiveBackground->GetRenderingObject())->RenderBackground(this, m_pPresenter->GetTime());
				//	/
			  shadowRend->set_lightColor(((NRDirectionalLight*)m_pPresenter->sgm->activeLight->Light)->color.r,
																	 ((NRDirectionalLight*)m_pPresenter->sgm->activeLight->Light)->color.g,
																	 ((NRDirectionalLight*)m_pPresenter->sgm->activeLight->Light)->color.b,
																	 ((NRDirectionalLight*)m_pPresenter->sgm->activeLight->Light)->intensity);

			  shadowRend->set_ambLightColor(((NRDirectionalLight*)m_pPresenter->sgm->activeLight->Light)->color.r,
																			((NRDirectionalLight*)m_pPresenter->sgm->activeLight->Light)->color.g,
																			((NRDirectionalLight*)m_pPresenter->sgm->activeLight->Light)->color.b,
																			((NRDirectionalLight*)m_pPresenter->sgm->activeLight->Light)->ambientIntensity);
				
				shadowRend->set_ambLightColorSum();

			  shadowRend->set_lightDir(((NRDirectionalLight*)m_pPresenter->sgm->activeLight->Light)->direction.x,
			                           ((NRDirectionalLight*)m_pPresenter->sgm->activeLight->Light)->direction.y,
																 ((NRDirectionalLight*)m_pPresenter->sgm->activeLight->Light)->direction.z,
																 0.0f);
			   
			  ::glPushMatrix();
        shadowRend->copyMatrixff(shadowRend->tempcopy,((NRDirectionalLight*)(m_pPresenter->sgm->activeLight->Light))->Matrix4x4->m_elements);
				::glLoadMatrixf(shadowRend->tempcopy);
				::glPopMatrix();
               
			  shadowRend->setupEyeView();
			    
				m_pPresenter->sgm->renderPass = 5;
			  CullFlag cullFlag = PARTIALLY_INSIDE;

				// M.Frank changed these lines (11-2005) -----------------------------------------
			   shadowRend->renderDiffLightMap(lnr, 3, pSceneRenderer,transform, cullFlag,
                           this, m_pPresenter->GetTime());		// 3 stands for pointlight
			  //pSceneRenderer->CullOrRender(transform,cullFlag,this,m_pPresenter->GetTime());
				// -------------------------------------------------------------------------------
				shadowRend->renderDiffLightMap(lnr, 3, pSceneRenderer,transform, cullFlag,
                           this, m_pPresenter->GetTime());

				m_pPresenter->sgm->shapecount=0;

			  //m_pPresenter->sgm->shapecount=0;
				//
			  //if(m_bABuffer)
				//	{
				// ::glAccum(GL_ACCUM,1.0f/(float)shadowRend->light_num_max);
				//	}
				//else
				//	{
				//  ::glReadPixels(0,0,shadowRend->winWidth,shadowRend->winHeight,GL_RGB,GL_UNSIGNED_BYTE,shadowRend->fbuffer[lnr-1]);
				//	}
				//::glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
				//::glDisable(GL_LIGHT0);
				//::glDisable(GL_LIGHTING);
			} 
//----------------------------------------------------------------------------------------------------
// END Rendering DirectionalLight --------------------------------------------------------------------
//----------------------------------------------------------------------------------------------------

		  }			
//----------------------------------------------------------------------------------------------------
// END LightMainLoop *********************************************************************************
//----------------------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------------------
// BEGIN Fog Loop (added by M.Frank 11-2005) -----------------------------------------------------
//----------------------------------------------------------------------------------------------------

		shadowRend->setupEyeView();

		if(m_pPresenter->sgm->sg_info.hasFogToRender==true)
			{
			shadowRend->set_fogColor(m_pPresenter->sgm->env_fog->color.r,
															 m_pPresenter->sgm->env_fog->color.g,
															 m_pPresenter->sgm->env_fog->color.b,
															 0.0f,
															 m_pPresenter->sgm->env_fog->visRange,
															 m_pPresenter->sgm->env_fog->type);

			//m_pPresenter->sgm->sg_info.hasShadowToRender=false; 
			m_pPresenter->sgm->renderPass = 5;

			CullFlag cullFlag = PARTIALLY_INSIDE;

			shadowRend->renderCameraDepth(pSceneRenderer,transform, cullFlag,
														this, m_pPresenter->GetTime());
			m_pPresenter->sgm->shapecount=0;
			}
			
//----------------------------------------------------------------------------------------------------
// END Fog Loop (added by M.Frank 11-2005) -------------------------------------------------------
//----------------------------------------------------------------------------------------------------


//----------------------------------------------------------------------------------------------------
// BEGIN Ambient Loop (added by M.Frank 11-2005) -----------------------------------------------------
//----------------------------------------------------------------------------------------------------

		
		//shadowRend->setupEyeView();
		m_nEmissiveEnabled=1;

		if (shadowRend->ambLightColorSum[3] > 0)
			{
			m_pPresenter->sgm->renderPass = 5;
			CullFlag cullFlag = PARTIALLY_INSIDE;
			shadowRend->renderAmbientLight(pSceneRenderer,transform, cullFlag,
														this, m_pPresenter->GetTime());
			m_pPresenter->sgm->shapecount=0;
			}
		else
			{
			shadowRend->renderAmbientBlack();
			m_pPresenter->sgm->shapecount=0;
			}	
		
		shadowRend->reset_ambLightColorSum();

		m_nEmissiveEnabled=0;

//----------------------------------------------------------------------------------------------------
// END Ambient Loop (added by M.Frank 11-2005) -------------------------------------------------------
//----------------------------------------------------------------------------------------------------


//----------------------------------------------------------------------------------------------------
// BEGIN Basetexture, Background and Objectcolors Loop (added by M.Frank 11-2005) --------------------
//----------------------------------------------------------------------------------------------------

		shadowRend->activatergbpBuffer();
		
		::glClearColor(1.0f, 1.0f, 1.0f, 1.0f);
		::glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);	
		shadowRend->setupEyeView();
		
	

		if(pActiveFog)
			{
			::glClearColor(m_pPresenter->sgm->env_fog->color.r, m_pPresenter->sgm->env_fog->color.g, m_pPresenter->sgm->env_fog->color.b, 1.0);
			::glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);	
			}
		else if(pActiveBackground)
			{
			((BackgroundProxy*) pActiveBackground->GetRenderingObject())->RenderBackground(this, m_pPresenter->GetTime());   
			}

		m_pPresenter->sgm->renderPass = 5;
		CullFlag cullFlag = PARTIALLY_INSIDE;

		::glEnable(GL_DEPTH_TEST);
		::glDepthFunc(GL_LEQUAL);
		::glDepthRange(0.0,1.0);
		
		m_nTextureEnabled=1;
		m_nMaterialEnabled=1;

		shadowRend->renderBaseTexture(pSceneRenderer,transform, cullFlag,
                           this, m_pPresenter->GetTime());	

		m_pPresenter->sgm->shapecount=0;

		m_nTextureEnabled=0;
		m_nMaterialEnabled=0;
//----------------------------------------------------------------------------------------------------
// END Basetexture, Background and Objectcolors Loop -------------------------------------------------
//----------------------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------------------
// BEGIN Compositing (added by M.Frank 11-2005) ------------------------------------------------------
//----------------------------------------------------------------------------------------------------

	::glClearColor(1.0f, 1.0f, 1.0f, 1.0f);
	::glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	

	::glMatrixMode(GL_PROJECTION);
  ::glLoadIdentity();	
	::glMatrixMode(GL_MODELVIEW);
  ::glLoadIdentity();	

	shadowRend->addDiffLights();
	shadowRend->addSpecLights();
	shadowRend->composeEverything();

  //shadowRend->setupEyeView();
	//shadowRend->paintTexture();

	shadowRend->calculateFramerate();

//----------------------------------------------------------------------------------------------------
// END Compositing -----------------------------------------------------------------------------------
//----------------------------------------------------------------------------------------------------




} 
		
/*--------------------------------------------------------------------------
// END Multipassrendering
back to the original code for painting no shadow scenes
---------------------------------------------------------------------------*/



else // Falls kein OpenGL 2.0 bzw. keine Schatten!!!!
	  {
		// M.Frank removed these lines (11-2005)****		
		m_nTextureEnabled=1;
		m_nMaterialEnabled=1;
		m_nEmissiveEnabled=0;
		//******************************************

			// render background, if we have any
      if (pActiveBackground)
        ((BackgroundProxy*) pActiveBackground->GetRenderingObject())->RenderBackground(this, m_pPresenter->GetTime());
      
      // enable fog, if we have any
      if (pActiveFog)
        ((FogProxy*) pActiveFog->GetRenderingObject())->Enable(this, m_pPresenter->GetTime());
      
      if (!pActiveBackground)
      {
        // clear screen
        if (m_bClearScreen)
        {
          if (m_bZBuffer)
            ::glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
					else
            ::glClear(GL_COLOR_BUFFER_BIT);
        }
      }
      
      // set viewpoint to world transformation
      poViewpointProxy->ViewpointToWorldMatrix(m_pPresenter->GetTime(), transform);
      ::glLoadMatrixf(transform);
      
      // activate lights
      m_nNextGLLight = GL_LIGHT0;
      
			pSceneRenderer->RenderLights(this, m_pPresenter->GetTime());
			
			// did we get any lights?
      if (m_nNextGLLight > GL_LIGHT0)
      {
        CullFlag cullFlag = PARTIALLY_INSIDE;
        
      // render scene
				pSceneRenderer->CullOrRender(transform, cullFlag,
         this, m_pPresenter->GetTime()); 
				// StereoPaint(m_pPresenter->GetTime());
        
        // disable lights
        
        for (int i = GL_LIGHT0; i < m_nNextGLLight; ++i) {
          ::glDisable((GLenum) i);
        }
      } 
      else
      {
        CullFlag cullFlag = PARTIALLY_INSIDE;
        
				pSceneRenderer->CullOrRender(transform, cullFlag,
          this, m_pPresenter->GetTime()); 
        // StereoPaint(m_pPresenter->GetTime());
      }
      
      // disable fog, if we have any
      
      if (pActiveFog)
        ((FogProxy*) pActiveFog->GetRenderingObject())->Disable(this, m_pPresenter->GetTime());
	  } // END IF MULTIPASS check
	} // << (active viewpoint)
	else
    { // >> (NO active viewpoint)
      Matrix transform;
      
      // we don't have a viewpoint; use an orthogonal viewpoint instead
      
      ::glMatrixMode(GL_PROJECTION);
      ::glLoadIdentity();
      
#ifdef SAIT_CONFORMANCE_2003
      ::glOrtho(-m_nViewportWidth, m_nViewportWidth,
        -m_nViewportHeight, m_nViewportHeight,
        -512, 512); 
#elif
      ::glOrtho(-m_nViewportWidth / 2, m_nViewportWidth / 2,
        -m_nViewportHeight / 2, m_nViewportHeight / 2,
        0, 512); 
#endif SAIT_CONFORMANCE_2003
      
      ::glMatrixMode(GL_MODELVIEW);
      
      ::glLoadIdentity();
      
      m_leftFrustumPlane = Vec3f(-m_nViewportWidth / 2, 0.0f, 0.0f);
      m_rightFrustumPlane = Vec3f(m_nViewportWidth / 2, 0.0f, 0.0f);
      m_topFrustumPlane = Vec3f(0.0f, m_nViewportHeight / 2, 0.0f);
      m_bottomFrustumPlane = Vec3f(0.0f, -m_nViewportHeight / 2, 0.0f);
      
      if (m_bClearScreen)
      {
        if (m_bZBuffer)
          ::glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
        else
          ::glClear(GL_COLOR_BUFFER_BIT);
      } 
      // activate lights
      
      m_nNextGLLight = GL_LIGHT0;
      pSceneRenderer->RenderLights(this, m_pPresenter->GetTime());
      
      // did we get any lights?
      if (m_nNextGLLight > GL_LIGHT0)
      {
        CullFlag cullFlag = COMPLETELY_INSIDE;
        
        // render scene
        
				pSceneRenderer->CullOrRender(transform, cullFlag,
          this, m_pPresenter->GetTime()); 
			
        // disable lights
        
        for (int i = GL_LIGHT0; i < m_nNextGLLight; ++i)
          ::glDisable((GLenum) i);
      } 
      else
      {
        CullFlag cullFlag = COMPLETELY_INSIDE;
        
        // render scene
        
				pSceneRenderer->CullOrRender(transform, cullFlag,
          this, m_pPresenter->GetTime()); 
      }
    } // << (NO active viewpoint)
    
    if (m_nDrawSceneGraph)
    {
      ::glMatrixMode(GL_PROJECTION);
      ::glLoadIdentity();
      
      ::glOrtho(0.0, 1.0,
        -9.0, 1.0,
        -1.0, 1.0);
      
      ::glMatrixMode(GL_MODELVIEW);
      
      ::glLoadIdentity();
      
      ::glPushAttrib(GL_ENABLE_BIT);
      
      ::glDisable(GL_ALPHA_TEST);
      ::glDisable(GL_DEPTH_TEST);
      ::glEnable(GL_BLEND);
      ::glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
      
      pSceneRenderer->RenderDescription(this, m_pPresenter->GetTime());
      
      ::glPopAttrib();
    }

   m_pPresenter->sgm->LightGroupMMap.clear();
   m_pPresenter->sgm->LightShapeMMap.clear();
	 m_pPresenter->sgm->ShapeMMap.clear();
	 m_pPresenter->sgm->RenderSLMap.clear();
	 m_pPresenter->sgm->RenderNSLMap.clear();
	 m_pPresenter->sgm->SceneGraph->DeleteAll();
	 m_pPresenter->sgm->ResetLights();
	 m_pPresenter->sgm->globalID=-1;
	 
	 m_pPresenter->sgm->sg_info.num_nrshape_nodes=0;
	 m_pPresenter->sgm->sg_info.num_nrshape_noshadow_nodes=0;
	 m_pPresenter->sgm->sg_info.num_nrshape_shadow_nodes=0;
	 m_pPresenter->sgm->sg_info.num_direclights=0;
	 m_pPresenter->sgm->sg_info.num_spotlights=0;
	 m_pPresenter->sgm->sg_info.num_pointlights=0;
	 m_pPresenter->sgm->sg_info.hasShadowToRender=false;
	 m_pPresenter->sgm->sg_info.hasFogToRender=false;


#ifdef MESA_3DFX
    {
      ::glMatrixMode(GL_PROJECTION);
      ::glLoadIdentity();
      
      ::glOrtho(0.0, m_nViewportWidth,
        m_nViewportHeight, 0,
        -1.0, 1.0);
      
      ::glMatrixMode(GL_MODELVIEW);
      
      ::glLoadIdentity();
      
      ::glPushAttrib(GL_ENABLE_BIT);
      
      ::glDisable(GL_ALPHA_TEST);
      ::glDisable(GL_DEPTH_TEST);
      
      ::glBegin(GL_LINES);
      ::glVertex2f(m_nMouseX - 2, m_nMouseY);
      ::glVertex2f(m_nMouseX + 1, m_nMouseY);
      ::glVertex2f(m_nMouseX, m_nMouseY - 1);
      ::glVertex2f(m_nMouseX, m_nMouseY + 2);
      ::glEnd();
      
      ::glPopAttrib();
    }
#endif
	}// << (shadow rendering)
	else
	{
    //****************************************************************************************************
    //*****************               NO SHADOWRENDERING                                  ************
    //****************************************************************************************************




    MediaObjectProxy * pSceneRenderer = (MediaObjectProxy*) pScene->GetRenderingObject();
	//ML - removed this next line, as pScene is also pRoot
    //Root * pRoot = pScene->GetRoot();
    Viewpoint * pActiveViewpoint = pRoot->GetBoundViewpoint();
    Fog * pActiveFog = pRoot->GetBoundFog();
    Background * pActiveBackground = pRoot->GetBoundBackground();
    
    // Update time sensors
    TimeSensorProxy::UpdateTimeSensors();
    
    

    // SB 99/05/14
#ifdef _DEBUG
    // pSceneRenderer->DumpTrace();
#endif // _DEBUG

    Drag();
    Select(m_nMouseButtons, m_nMouseButtons);
    
    if (m_bZBuffer)
      ::glEnable(GL_DEPTH_TEST);
    else
      ::glDisable(GL_DEPTH_TEST);
    
    if (pActiveViewpoint != NULL)
    { // >> (active viewpoint)
      ViewpointProxy * poViewpointProxy = (ViewpointProxy *) pActiveViewpoint->GetRenderingObject();

      { // >> (compute new viewpoint, checking mouse buttons)
        
        Vec3f position(poViewpointProxy->GetPosition());
        Rotation orientation(poViewpointProxy->GetOrientation());
        


			Vec3f oldPosition = position;
			Rotation oldOrientation = orientation;

        
        // TRACE("[RendVideo::Paint] MouseButtons %04x \n", m_nMouseButtons);
        if (m_nMouseButtons)
        { // >> (some mouse button)
          Vec3f move;
          float theta = ::cos(orientation.theta * 0.5f);
          Vec3f dir(orientation.x, orientation.y, orientation.z);
          float sw;
          
          // normalise dir
          dir.Normalize();
          
          sw = ::sin(orientation.theta * 0.5f);
          dir.x *= sw;
          dir.y *= sw;
          dir.z *= sw;

//        if (((m_nMouseButtons & LEFT_BUTTON) &&
//            (m_nMouseButtons & RIGHT_BUTTON)) ||
//            (m_nMouseButtons & MIDDLE_BUTTON))
          
					if (m_nMouseButtons & LEFT_BUTTON)
						{ // >> (2 muose buttons)
						azimuth_+=180 * float(m_nMouseFirstX - m_nMouseX) / float(m_nViewportWidth);
						move.z = - float(m_nMouseFirstY - m_nMouseY) / float(m_nViewportHeight);    

/*					{
						float omega = 5.0f * float(m_nMouseFirstX - m_nMouseX) / float(m_nViewportWidth);
              
              if (omega != 0.0f)
              {
                float s1 =        ::cos(omega * M_PI / 360.0f);
                float sin_angle = ::sin(omega * M_PI / 360.0f);
                float aa = dir.x * dir.x,                     
                  ac = dir.x * dir.z, 
                  ad = dir.x * theta,
                  bb = dir.y * dir.y, 
                  bc = dir.y * dir.z, 
                  bd = dir.y * theta,
                  cc = dir.z * dir.z, 
                  dd = theta * theta;
                
                Vec3f v1(2.0 * (ac + bd) * sin_angle, 2.0 * (bc - ad) * sin_angle, (- aa - bb + cc + dd) * sin_angle);
                Vec3f v2 = dir;
                
                dir.x = s1 * v2.x + theta * v1.x + v1.y * v2.z - v1.z * v2.y;
                dir.y = s1 * v2.y + theta * v1.y + v1.z * v2.x - v1.x * v2.z;
                dir.z = s1 * v2.z + theta * v1.z + v1.x * v2.y - v1.y * v2.x;
                theta = theta * s1 - (v1.x * v2.x + v1.y * v2.y + v1.z * v2.z);
              }
            }
            
            {
              float omega = 5.0f * float(m_nMouseFirstY - m_nMouseY) / float(m_nViewportHeight);
              
              if (omega != 0.0f)
              { 
                float s1 =        ::cos(omega * M_PI / 360.0f);
                float sin_angle = ::sin(omega * M_PI / 360.0f);
                float aa = dir.x * dir.x, 
                  ab = dir.x * dir.y, 
                  ac = dir.x * dir.z,                    
                  bb = dir.y * dir.y,                    
                  bd = dir.y * theta,	     
                  cc = dir.z * dir.z,
                  cd = dir.z * theta,
                  dd = theta * theta;
                
                Vec3f v1((aa - bb - cc + dd) * sin_angle,  2.0 * (ab + cd) * sin_angle, 2.0 * (ac - bd) * sin_angle);
                Vec3f v2 = dir;
                
                dir.x = s1 * v2.x + theta * v1.x + v1.y * v2.z - v1.z * v2.y;
                dir.y = s1 * v2.y + theta * v1.y + v1.z * v2.x - v1.x * v2.z;
                dir.z = s1 * v2.z + theta * v1.z + v1.x * v2.y - v1.y * v2.x;
                theta = theta * s1 - (v1.x * v2.x + v1.y * v2.y + v1.z * v2.z);
              }
            }
 */      
						} // << (2 muose buttons)
          else
						{ // >> (only 1 muose button)
            if (m_nMouseButtons & LEFT_BUTTON)
            {
              float omega = 5.0f * float(m_nMouseFirstX - m_nMouseX) / float(m_nViewportWidth);
              if (omega != 0.0)
              {
                  float aa = dir.x * dir.x, 
                  ab = dir.x * dir.y,
                  ad = dir.x * theta,
                  bb = dir.y * dir.y, 
                  bc = dir.y * dir.z,
                  cc = dir.z * dir.z, 
                  cd = dir.z * theta,
                  dd = theta * theta;
                
                float s1 =        ::cos(omega * M_PI / 360.0f);
                float sin_angle = ::sin(omega * M_PI / 360.0f);
                Vec3f v1(2.0 * (ab - cd) * sin_angle, (- aa + bb - cc + dd) * sin_angle,  2.0 * (bc + ad) * sin_angle);
                Vec3f v2 = dir;
                float s2 = theta;
                
                dir.x = s1 * v2.x + s2 * v1.x + v1.y * v2.z - v1.z * v2.y;
                dir.y = s1 * v2.y + s2 * v1.y + v1.z * v2.x - v1.x * v2.z;
                dir.z = s1 * v2.z + s2 * v1.z + v1.x * v2.y - v1.y * v2.x;
                theta = s2 * s1 - (v1.x * v2.x + v1.y * v2.y + v1.z * v2.z);
              }
              
              move.z = - float(m_nMouseFirstY - m_nMouseY) / float(m_nViewportHeight);    
            }
            else
            {
              if (m_nMouseButtons & RIGHT_BUTTON)
              {
                move.x =   float(m_nMouseX - m_nMouseFirstX) / float(m_nViewportWidth);
                move.y = - float(m_nMouseY - m_nMouseFirstY) / float(m_nViewportHeight);
              }
            }
          } // << (only right mouse button)
          
          {  
            orientation.x = dir.x;
            orientation.y = dir.y;
            orientation.z = dir.z;
            orientation.theta = 2.0 * ::acos(theta);
            NavigationInfo* pNavigationInfo = pRoot->GetBoundNavigationInfo();
            float speed = (pNavigationInfo ? ((NavigationInfo *) pNavigationInfo->GetRenderingObject())->speed /* GetSpeed() */ : 1.0f) / m_fFrameRate;
            
            orientation.Normalize();
            
            float aa = dir.x * dir.x, ab = dir.x * dir.y, ac = dir.x * dir.z, ad = dir.x * theta;
            float                     bb = dir.y * dir.y, bc = dir.y * dir.z, bd = dir.y * theta;
            float                                         cc = dir.z * dir.z, cd = dir.z * theta;
            float                                                             dd = theta * theta;
            
            float T[3][3] = {{aa - bb - cc + dd,     2.0 * (ab - cd),      2.0 * (ac + bd)},
            {  2.0 * (ab + cd), - aa + bb - cc + dd,      2.0 * (bc - ad)},
            {  2.0 * (ac - bd),     2.0 * (bc + ad), - aa - bb + cc + dd}};
            
            float t[3];
            
            for (int j = 0; j < 3; j++)
              t[j] = move.x * T[j][0] + move.y * T[j][1] + move.z * T[j][2];
            
            position.x += t[0] * speed *100;
            position.y += t[1] * speed *100;
            position.z += t[2] * speed *100;
          }
          
          pActiveViewpoint->position = /* SetPosition */ position;
          pActiveViewpoint->orientation = /* SetOrientation */ orientation;
          
          if (m_bCollisionDetection)
          {
            /* Node */ MediaObject * pNode;
            
            // Do collision detection
            pNode = poViewpointProxy->DetectCollision();
            if (pNode) {
              COMPLAIN("Collision\n");
              
              pActiveViewpoint->position = /* SetPosition */ oldPosition;
              pActiveViewpoint->orientation = /* SetOrientation */ oldOrientation;
            }
          }
        }  // << (some mouse button)
      } // << (compute new viewpoint, checking mouse buttons)
      
      Matrix transform;
      
      // load projection matrix from viewpoint
      
      ::glMatrixMode(GL_PROJECTION);
      
      ::glLoadIdentity();
      
      ::gluPerspective(pActiveViewpoint->fieldOfView /* GetFieldOfView() */ * 180.0f / M_PI,
        m_dAspectRatio,
        m_dNearClipping, m_dFarClipping);
      
      ::glMatrixMode(GL_MODELVIEW);
      
      ::glLoadIdentity();
      
      {
        float xmax, ymax;
        
        ymax = 0.5f * ::tan(pActiveViewpoint->fieldOfView /* GetFieldOfView() */ * 0.5f);
        xmax = ymax * m_dAspectRatio;
        
        Vec3f ll(-xmax, -ymax, -0.5f);
        Vec3f ul(-xmax,  ymax, -0.5f);
        Vec3f lr( xmax, -ymax, -0.5f);
        Vec3f ur( xmax,  ymax, -0.5f);
        
        m_leftFrustumPlane =   ul;
        m_rightFrustumPlane =  lr;
        m_topFrustumPlane =    ur;
        m_bottomFrustumPlane = ll;
        
        m_leftFrustumPlane.Cross(ll);
        m_rightFrustumPlane.Cross(ur);
        m_topFrustumPlane.Cross(ul);
        m_bottomFrustumPlane.Cross(lr);
      }
      

	bool a=0;
		 
	if(a==1)
	{  
			 


} 
		
/*--------------------------------------------------------------------------
// END Multipassrendering
back to the original code for painting no shadow scenes
---------------------------------------------------------------------------*/



else // Falls kein OpenGL 2.0 bzw. keine Schatten!!!!
	  {
		// M.Frank removed these lines (11-2005)****		
		m_nTextureEnabled=1;
		m_nMaterialEnabled=1;
		m_nEmissiveEnabled=0;
		//******************************************

			// render background, if we have any
      if (pActiveBackground)
        ((BackgroundProxy*) pActiveBackground->GetRenderingObject())->RenderBackground(this, m_pPresenter->GetTime());
      
      // enable fog, if we have any
      if (pActiveFog)
        ((FogProxy*) pActiveFog->GetRenderingObject())->Enable(this, m_pPresenter->GetTime());
      
      if (!pActiveBackground)
      {
        // clear screen
        if (m_bClearScreen)
        {
          if (m_bZBuffer)
            ::glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
					else
            ::glClear(GL_COLOR_BUFFER_BIT);
        }
      }
      
      // set viewpoint to world transformation
      poViewpointProxy->ViewpointToWorldMatrix(m_pPresenter->GetTime(), transform);
      ::glLoadMatrixf(transform);
      
      // activate lights
      m_nNextGLLight = GL_LIGHT0;
      
			pSceneRenderer->RenderLights(this, m_pPresenter->GetTime());
			
			// did we get any lights?
      if (m_nNextGLLight > GL_LIGHT0)
      {
        CullFlag cullFlag = PARTIALLY_INSIDE;
        
      // render scene
				pSceneRenderer->CullOrRender(transform, cullFlag,
         this, m_pPresenter->GetTime()); 
				// StereoPaint(m_pPresenter->GetTime());
        
        // disable lights
        
        for (int i = GL_LIGHT0; i < m_nNextGLLight; ++i) {
          ::glDisable((GLenum) i);
        }
      } 
      else
      {
        CullFlag cullFlag = PARTIALLY_INSIDE;
        
				pSceneRenderer->CullOrRender(transform, cullFlag,
          this, m_pPresenter->GetTime()); 
        // StereoPaint(m_pPresenter->GetTime());
      }
      
      // disable fog, if we have any
      
      if (pActiveFog)
        ((FogProxy*) pActiveFog->GetRenderingObject())->Disable(this, m_pPresenter->GetTime());
	  } // END IF MULTIPASS check
	} // << (active viewpoint)
	else
    { // >> (NO active viewpoint)
      Matrix transform;
      
      // we don't have a viewpoint; use an orthogonal viewpoint instead
      
      ::glMatrixMode(GL_PROJECTION);
      ::glLoadIdentity();
      
#ifdef SAIT_CONFORMANCE_2003
      ::glOrtho(-m_nViewportWidth, m_nViewportWidth,
        -m_nViewportHeight, m_nViewportHeight,
        -512, 512); 
#elif
      ::glOrtho(-m_nViewportWidth / 2, m_nViewportWidth / 2,
        -m_nViewportHeight / 2, m_nViewportHeight / 2,
        0, 512); 
#endif SAIT_CONFORMANCE_2003
      
      ::glMatrixMode(GL_MODELVIEW);
      
      ::glLoadIdentity();
      
      m_leftFrustumPlane = Vec3f(-m_nViewportWidth / 2, 0.0f, 0.0f);
      m_rightFrustumPlane = Vec3f(m_nViewportWidth / 2, 0.0f, 0.0f);
      m_topFrustumPlane = Vec3f(0.0f, m_nViewportHeight / 2, 0.0f);
      m_bottomFrustumPlane = Vec3f(0.0f, -m_nViewportHeight / 2, 0.0f);
      
      if (m_bClearScreen)
      {
        if (m_bZBuffer)
          ::glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
        else
          ::glClear(GL_COLOR_BUFFER_BIT);
      } 
      // activate lights
      
      m_nNextGLLight = GL_LIGHT0;
      pSceneRenderer->RenderLights(this, m_pPresenter->GetTime());
      
      // did we get any lights?
      if (m_nNextGLLight > GL_LIGHT0)
      {
        CullFlag cullFlag = COMPLETELY_INSIDE;
        
        // render scene
        
				pSceneRenderer->CullOrRender(transform, cullFlag,
          this, m_pPresenter->GetTime()); 
			
        // disable lights
        
        for (int i = GL_LIGHT0; i < m_nNextGLLight; ++i)
          ::glDisable((GLenum) i);
      } 
      else
      {
        CullFlag cullFlag = COMPLETELY_INSIDE;
        
        // render scene
        
				pSceneRenderer->CullOrRender(transform, cullFlag,
          this, m_pPresenter->GetTime()); 
      }
    } // << (NO active viewpoint)
    
    if (m_nDrawSceneGraph)
    {
      ::glMatrixMode(GL_PROJECTION);
      ::glLoadIdentity();
      
      ::glOrtho(0.0, 1.0,
        -9.0, 1.0,
        -1.0, 1.0);
      
      ::glMatrixMode(GL_MODELVIEW);
      
      ::glLoadIdentity();
      
      ::glPushAttrib(GL_ENABLE_BIT);
      
      ::glDisable(GL_ALPHA_TEST);
      ::glDisable(GL_DEPTH_TEST);
      ::glEnable(GL_BLEND);
      ::glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
      
      pSceneRenderer->RenderDescription(this, m_pPresenter->GetTime());
      
      ::glPopAttrib();
    }


#ifdef MESA_3DFX
    {
      ::glMatrixMode(GL_PROJECTION);
      ::glLoadIdentity();
      
      ::glOrtho(0.0, m_nViewportWidth,
        m_nViewportHeight, 0,
        -1.0, 1.0);
      
      ::glMatrixMode(GL_MODELVIEW);
      
      ::glLoadIdentity();
      
      ::glPushAttrib(GL_ENABLE_BIT);
      
      ::glDisable(GL_ALPHA_TEST);
      ::glDisable(GL_DEPTH_TEST);
      
      ::glBegin(GL_LINES);
      ::glVertex2f(m_nMouseX - 2, m_nMouseY);
      ::glVertex2f(m_nMouseX + 1, m_nMouseY);
      ::glVertex2f(m_nMouseX, m_nMouseY - 1);
      ::glVertex2f(m_nMouseX, m_nMouseY + 2);
      ::glEnd();
      
      ::glPopAttrib();
    }
#endif





	}
 
  } // << (active scene)




  //****************************************************************************************************
  //*****************                   NO SHADOWRENDERING                                  ************
  //****************************************************************************************************
  
  if ((pScene != NULL) & (m_pPresenter->sgm->sg_info.hasShadowToRender==false) & (m_bHWShadow==false))
  { // >> (active scene)  
      
    MediaObjectProxy * pSceneRenderer = (MediaObjectProxy*) pScene->GetRenderingObject();
	//ML - removed this next line, as pScene is also pRoot
    //Root * pRoot = pScene->GetRoot();
    Viewpoint * pActiveViewpoint = pRoot->GetBoundViewpoint();
    Fog * pActiveFog = pRoot->GetBoundFog();
    Background * pActiveBackground = pRoot->GetBoundBackground();
    
    // Update time sensors
    TimeSensorProxy::UpdateTimeSensors();
    
    

    // SB 99/05/14
#ifdef _DEBUG
    // pSceneRenderer->DumpTrace();
#endif // _DEBUG

    Drag();
    Select(m_nMouseButtons, m_nMouseButtons);
    
    if (m_bZBuffer)
      ::glEnable(GL_DEPTH_TEST);
    else
      ::glDisable(GL_DEPTH_TEST);
    
    if (pActiveViewpoint != NULL)
    { // >> (active viewpoint)
      ViewpointProxy * poViewpointProxy = (ViewpointProxy *) pActiveViewpoint->GetRenderingObject();

      { // >> (compute new viewpoint, checking mouse buttons)
        
        Vec3f position(poViewpointProxy->GetPosition());
        Rotation orientation(poViewpointProxy->GetOrientation());
        


			Vec3f oldPosition = position;
			Rotation oldOrientation = orientation;

        
        // TRACE("[RendVideo::Paint] MouseButtons %04x \n", m_nMouseButtons);
        if (m_nMouseButtons)
        { // >> (some mouse button)
          Vec3f move;
          float theta = ::cos(orientation.theta * 0.5f);
          Vec3f dir(orientation.x, orientation.y, orientation.z);
          float sw;
          
          // normalise dir
          dir.Normalize();
          
          sw = ::sin(orientation.theta * 0.5f);
          dir.x *= sw;
          dir.y *= sw;
          dir.z *= sw;

//        if (((m_nMouseButtons & LEFT_BUTTON) &&
//            (m_nMouseButtons & RIGHT_BUTTON)) ||
//            (m_nMouseButtons & MIDDLE_BUTTON))
          
					if (m_nMouseButtons & LEFT_BUTTON)
						{ // >> (2 muose buttons)
						azimuth_+=180 * float(m_nMouseFirstX - m_nMouseX) / float(m_nViewportWidth);
						move.z = - float(m_nMouseFirstY - m_nMouseY) / float(m_nViewportHeight);    

/*					{
						float omega = 5.0f * float(m_nMouseFirstX - m_nMouseX) / float(m_nViewportWidth);
              
              if (omega != 0.0f)
              {
                float s1 =        ::cos(omega * M_PI / 360.0f);
                float sin_angle = ::sin(omega * M_PI / 360.0f);
                float aa = dir.x * dir.x,                     
                  ac = dir.x * dir.z, 
                  ad = dir.x * theta,
                  bb = dir.y * dir.y, 
                  bc = dir.y * dir.z, 
                  bd = dir.y * theta,
                  cc = dir.z * dir.z, 
                  dd = theta * theta;
                
                Vec3f v1(2.0 * (ac + bd) * sin_angle, 2.0 * (bc - ad) * sin_angle, (- aa - bb + cc + dd) * sin_angle);
                Vec3f v2 = dir;
                
                dir.x = s1 * v2.x + theta * v1.x + v1.y * v2.z - v1.z * v2.y;
                dir.y = s1 * v2.y + theta * v1.y + v1.z * v2.x - v1.x * v2.z;
                dir.z = s1 * v2.z + theta * v1.z + v1.x * v2.y - v1.y * v2.x;
                theta = theta * s1 - (v1.x * v2.x + v1.y * v2.y + v1.z * v2.z);
              }
            }
            
            {
              float omega = 5.0f * float(m_nMouseFirstY - m_nMouseY) / float(m_nViewportHeight);
              
              if (omega != 0.0f)
              { 
                float s1 =        ::cos(omega * M_PI / 360.0f);
                float sin_angle = ::sin(omega * M_PI / 360.0f);
                float aa = dir.x * dir.x, 
                  ab = dir.x * dir.y, 
                  ac = dir.x * dir.z,                    
                  bb = dir.y * dir.y,                    
                  bd = dir.y * theta,	     
                  cc = dir.z * dir.z,
                  cd = dir.z * theta,
                  dd = theta * theta;
                
                Vec3f v1((aa - bb - cc + dd) * sin_angle,  2.0 * (ab + cd) * sin_angle, 2.0 * (ac - bd) * sin_angle);
                Vec3f v2 = dir;
                
                dir.x = s1 * v2.x + theta * v1.x + v1.y * v2.z - v1.z * v2.y;
                dir.y = s1 * v2.y + theta * v1.y + v1.z * v2.x - v1.x * v2.z;
                dir.z = s1 * v2.z + theta * v1.z + v1.x * v2.y - v1.y * v2.x;
                theta = theta * s1 - (v1.x * v2.x + v1.y * v2.y + v1.z * v2.z);
              }
            }
 */      
						} // << (2 muose buttons)
          else
						{ // >> (only 1 muose button)
            if (m_nMouseButtons & LEFT_BUTTON)
            {
              float omega = 5.0f * float(m_nMouseFirstX - m_nMouseX) / float(m_nViewportWidth);
              if (omega != 0.0)
              {
                  float aa = dir.x * dir.x, 
                  ab = dir.x * dir.y,
                  ad = dir.x * theta,
                  bb = dir.y * dir.y, 
                  bc = dir.y * dir.z,
                  cc = dir.z * dir.z, 
                  cd = dir.z * theta,
                  dd = theta * theta;
                
                float s1 =        ::cos(omega * M_PI / 360.0f);
                float sin_angle = ::sin(omega * M_PI / 360.0f);
                Vec3f v1(2.0 * (ab - cd) * sin_angle, (- aa + bb - cc + dd) * sin_angle,  2.0 * (bc + ad) * sin_angle);
                Vec3f v2 = dir;
                float s2 = theta;
                
                dir.x = s1 * v2.x + s2 * v1.x + v1.y * v2.z - v1.z * v2.y;
                dir.y = s1 * v2.y + s2 * v1.y + v1.z * v2.x - v1.x * v2.z;
                dir.z = s1 * v2.z + s2 * v1.z + v1.x * v2.y - v1.y * v2.x;
                theta = s2 * s1 - (v1.x * v2.x + v1.y * v2.y + v1.z * v2.z);
              }
              
              move.z = - float(m_nMouseFirstY - m_nMouseY) / float(m_nViewportHeight);    
            }
            else
            {
              if (m_nMouseButtons & RIGHT_BUTTON)
              {
                move.x =   float(m_nMouseX - m_nMouseFirstX) / float(m_nViewportWidth);
                move.y = - float(m_nMouseY - m_nMouseFirstY) / float(m_nViewportHeight);
              }
            }
          } // << (only right mouse button)
          
          {  
            orientation.x = dir.x;
            orientation.y = dir.y;
            orientation.z = dir.z;
            orientation.theta = 2.0 * ::acos(theta);
            NavigationInfo* pNavigationInfo = pRoot->GetBoundNavigationInfo();
            float speed = (pNavigationInfo ? ((NavigationInfo *) pNavigationInfo->GetRenderingObject())->speed /* GetSpeed() */ : 1.0f) / m_fFrameRate;
            
            orientation.Normalize();
            
            float aa = dir.x * dir.x, ab = dir.x * dir.y, ac = dir.x * dir.z, ad = dir.x * theta;
            float                     bb = dir.y * dir.y, bc = dir.y * dir.z, bd = dir.y * theta;
            float                                         cc = dir.z * dir.z, cd = dir.z * theta;
            float                                                             dd = theta * theta;
            
            float T[3][3] = {{aa - bb - cc + dd,     2.0 * (ab - cd),      2.0 * (ac + bd)},
            {  2.0 * (ab + cd), - aa + bb - cc + dd,      2.0 * (bc - ad)},
            {  2.0 * (ac - bd),     2.0 * (bc + ad), - aa - bb + cc + dd}};
            
            float t[3];
            
            for (int j = 0; j < 3; j++)
              t[j] = move.x * T[j][0] + move.y * T[j][1] + move.z * T[j][2];
            
            position.x += t[0] * speed *100;
            position.y += t[1] * speed *100;
            position.z += t[2] * speed *100;
          }
          
          pActiveViewpoint->position = /* SetPosition */ position;
          pActiveViewpoint->orientation = /* SetOrientation */ orientation;
          
          if (m_bCollisionDetection)
          {
            /* Node */ MediaObject * pNode;
            
            // Do collision detection
            pNode = poViewpointProxy->DetectCollision();
            if (pNode) {
              COMPLAIN("Collision\n");
              
              pActiveViewpoint->position = /* SetPosition */ oldPosition;
              pActiveViewpoint->orientation = /* SetOrientation */ oldOrientation;
            }
          }
        }  // << (some mouse button)
      } // << (compute new viewpoint, checking mouse buttons)
      
      Matrix transform;
      
      // load projection matrix from viewpoint
      
      ::glMatrixMode(GL_PROJECTION);
      
      ::glLoadIdentity();
      
      ::gluPerspective(pActiveViewpoint->fieldOfView /* GetFieldOfView() */ * 180.0f / M_PI,
        m_dAspectRatio,
        m_dNearClipping, m_dFarClipping);
      
      ::glMatrixMode(GL_MODELVIEW);
      
      ::glLoadIdentity();
      
      {
        float xmax, ymax;
        
        ymax = 0.5f * ::tan(pActiveViewpoint->fieldOfView /* GetFieldOfView() */ * 0.5f);
        xmax = ymax * m_dAspectRatio;
        
        Vec3f ll(-xmax, -ymax, -0.5f);
        Vec3f ul(-xmax,  ymax, -0.5f);
        Vec3f lr( xmax, -ymax, -0.5f);
        Vec3f ur( xmax,  ymax, -0.5f);
        
        m_leftFrustumPlane =   ul;
        m_rightFrustumPlane =  lr;
        m_topFrustumPlane =    ur;
        m_bottomFrustumPlane = ll;
        
        m_leftFrustumPlane.Cross(ll);
        m_rightFrustumPlane.Cross(ur);
        m_topFrustumPlane.Cross(ul);
        m_bottomFrustumPlane.Cross(lr);
      }
      

	bool a=0;
		 
	if(a==1)
	{  
			 


} 
		
/*--------------------------------------------------------------------------
// END Multipassrendering
back to the original code for painting no shadow scenes
---------------------------------------------------------------------------*/



else // Falls kein OpenGL 2.0 bzw. keine Schatten!!!!
	  {
		// M.Frank removed these lines (11-2005)****		
		m_nTextureEnabled=1;
		m_nMaterialEnabled=1;
		m_nEmissiveEnabled=0;
		//******************************************

			// render background, if we have any
      if (pActiveBackground)
        ((BackgroundProxy*) pActiveBackground->GetRenderingObject())->RenderBackground(this, m_pPresenter->GetTime());
      
      // enable fog, if we have any
      if (pActiveFog)
        ((FogProxy*) pActiveFog->GetRenderingObject())->Enable(this, m_pPresenter->GetTime());
      
      if (!pActiveBackground)
      {
        // clear screen
        if (m_bClearScreen)
        {
          if (m_bZBuffer)
            ::glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
					else
            ::glClear(GL_COLOR_BUFFER_BIT);
        }
      }
      
      // set viewpoint to world transformation
      poViewpointProxy->ViewpointToWorldMatrix(m_pPresenter->GetTime(), transform);
      ::glLoadMatrixf(transform);
      
      // activate lights
      m_nNextGLLight = GL_LIGHT0;
      
			pSceneRenderer->RenderLights(this, m_pPresenter->GetTime());
			
			// did we get any lights?
      if (m_nNextGLLight > GL_LIGHT0)
      {
        CullFlag cullFlag = PARTIALLY_INSIDE;
        
      // render scene
				pSceneRenderer->CullOrRender(transform, cullFlag,
         this, m_pPresenter->GetTime()); 
				// StereoPaint(m_pPresenter->GetTime());
        
        // disable lights
        
        for (int i = GL_LIGHT0; i < m_nNextGLLight; ++i) {
          ::glDisable((GLenum) i);
        }
      } 
      else
      {
        CullFlag cullFlag = PARTIALLY_INSIDE;
        
				pSceneRenderer->CullOrRender(transform, cullFlag,
          this, m_pPresenter->GetTime()); 
        // StereoPaint(m_pPresenter->GetTime());
      }
      
      // disable fog, if we have any
      
      if (pActiveFog)
        ((FogProxy*) pActiveFog->GetRenderingObject())->Disable(this, m_pPresenter->GetTime());
	  } // END IF MULTIPASS check
	} // << (active viewpoint)
	else
    { // >> (NO active viewpoint)
      Matrix transform;
      
      // we don't have a viewpoint; use an orthogonal viewpoint instead
      
      ::glMatrixMode(GL_PROJECTION);
      ::glLoadIdentity();
      
#ifdef SAIT_CONFORMANCE_2003
      ::glOrtho(-m_nViewportWidth, m_nViewportWidth,
        -m_nViewportHeight, m_nViewportHeight,
        -512, 512); 
#elif
      ::glOrtho(-m_nViewportWidth / 2, m_nViewportWidth / 2,
        -m_nViewportHeight / 2, m_nViewportHeight / 2,
        0, 512); 
#endif SAIT_CONFORMANCE_2003
      
      ::glMatrixMode(GL_MODELVIEW);
      
      ::glLoadIdentity();
      
      m_leftFrustumPlane = Vec3f(-m_nViewportWidth / 2, 0.0f, 0.0f);
      m_rightFrustumPlane = Vec3f(m_nViewportWidth / 2, 0.0f, 0.0f);
      m_topFrustumPlane = Vec3f(0.0f, m_nViewportHeight / 2, 0.0f);
      m_bottomFrustumPlane = Vec3f(0.0f, -m_nViewportHeight / 2, 0.0f);
      
      if (m_bClearScreen)
      {
        if (m_bZBuffer)
          ::glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
        else
          ::glClear(GL_COLOR_BUFFER_BIT);
      } 
      // activate lights
      
      m_nNextGLLight = GL_LIGHT0;
      pSceneRenderer->RenderLights(this, m_pPresenter->GetTime());
      
      // did we get any lights?
      if (m_nNextGLLight > GL_LIGHT0)
      {
        CullFlag cullFlag = COMPLETELY_INSIDE;
        
        // render scene
        
				pSceneRenderer->CullOrRender(transform, cullFlag,
          this, m_pPresenter->GetTime()); 
			
        // disable lights
        
        for (int i = GL_LIGHT0; i < m_nNextGLLight; ++i)
          ::glDisable((GLenum) i);
      } 
      else
      {
        CullFlag cullFlag = COMPLETELY_INSIDE;
        
        // render scene
        
				pSceneRenderer->CullOrRender(transform, cullFlag,
          this, m_pPresenter->GetTime()); 
      }
    } // << (NO active viewpoint)
    
    if (m_nDrawSceneGraph)
    {
      ::glMatrixMode(GL_PROJECTION);
      ::glLoadIdentity();
      
      ::glOrtho(0.0, 1.0,
        -9.0, 1.0,
        -1.0, 1.0);
      
      ::glMatrixMode(GL_MODELVIEW);
      
      ::glLoadIdentity();
      
      ::glPushAttrib(GL_ENABLE_BIT);
      
      ::glDisable(GL_ALPHA_TEST);
      ::glDisable(GL_DEPTH_TEST);
      ::glEnable(GL_BLEND);
      ::glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
      
      pSceneRenderer->RenderDescription(this, m_pPresenter->GetTime());
      
      ::glPopAttrib();
    }


#ifdef MESA_3DFX
    {
      ::glMatrixMode(GL_PROJECTION);
      ::glLoadIdentity();
      
      ::glOrtho(0.0, m_nViewportWidth,
        m_nViewportHeight, 0,
        -1.0, 1.0);
      
      ::glMatrixMode(GL_MODELVIEW);
      
      ::glLoadIdentity();
      
      ::glPushAttrib(GL_ENABLE_BIT);
      
      ::glDisable(GL_ALPHA_TEST);
      ::glDisable(GL_DEPTH_TEST);
      
      ::glBegin(GL_LINES);
      ::glVertex2f(m_nMouseX - 2, m_nMouseY);
      ::glVertex2f(m_nMouseX + 1, m_nMouseY);
      ::glVertex2f(m_nMouseX, m_nMouseY - 1);
      ::glVertex2f(m_nMouseX, m_nMouseY + 2);

















      ::glEnd();
      
      ::glPopAttrib();
    }
#endif
  } // << (active scene)
  

  













  // >> (NO active scene)
  if (pScene == NULL)
  { 
    ::glClearColor(0.0f, 0.0f, 0.0f, 0.0f); 
    ::glClear(GL_COLOR_BUFFER_BIT);
    
    // COMPLAIN("OpenGLVisualRenderer::Paint(): no scene?\n");
  } // << (NO active scene)
  
  pExecutive->UnlockScene();
	pExecutive->ActivateRoutes (); // MBS 7/7/03 propagate pending routes.
  
  {
    unsigned long now;
    
#ifndef WIN32
// SOLARIS PORTING 
/*    struct timeval time;
    
    ::gettimeofday(&time, NULL);
    now = time.tv_sec * 1000 + time.tv_usec / 1000;*/
    now = (unsigned long)timeGetTime();
// SOLARIS PORTING - END
#else
    now = timeGetTime();
#endif
    
    if (m_nLastTime == -1)
      m_nLastTime = now;
    else {
      m_fFrames++;
      
      if (m_nLastTime + 500 < (int) now) {
        m_fFrameRate = m_fFrames * 1000.0f / float(now - m_nLastTime);
#ifdef _DEBUG
        // COMPLAIN("current frame rate: %.2f per second\n", m_fFrameRate);
#endif
        m_nLastTime = now;
        m_fFrames = 0.0f;
      }
    }
  }
  
#if defined(GLUT)
  ::glutSwapBuffers();
#elif defined(WIN32)
  ::glFinish();
  ::SwapBuffers(wglGetCurrentDC());
#endif
  
#ifdef _DEBUG
  {
    GLenum error;
    
    error = ::glGetError();
    
    while (error != GL_NO_ERROR) {
      TRACE("GL error: %d\n", error);
      error = ::glGetError();
    }
  }
#endif
  
  VisualRenderer::Paint(pRect);
  
  m_mtxRendererActive.V();
}



// Set mouse state

/* virtual */ void
OpenGLVisualRenderer::MouseState(int nButton, int nX, int nY)
{
  int oldButton = m_nMouseButtons;
  
  if (nButton & ~m_nMouseButtons) {
    m_nMouseFirstX = nX;
    m_nMouseFirstY = nY;
  }
  
  m_nMouseButtons = nButton;
  m_nMouseX = nX;
  m_nMouseY = nY;
  
  // do Select() only on mouse up/down
  
  if ((nButton & RIGHT_BUTTON) !=
    (oldButton & RIGHT_BUTTON)) {
    m_mtxRendererActive.P();
    
    Executive* pExecutive = m_pPresenter->GetExecutive();
    
    pExecutive->LockScene();
    Select(nButton, oldButton);
    pExecutive->UnlockScene();
    
    m_mtxRendererActive.V();
  }
}

// Render string

void
OpenGLVisualRenderer::RenderString(const char* pString)
{
#if defined(GLTT)
  if (m_nActiveFont >= 0) {
    m_generatedFonts[m_nActiveFont]->font->output(pString);
    return;
  }
  
  // By not making GLTT and GLUT font rendering mutually exclusive, we can
  // fallback to GLUT font rendering if GLTT font rendering fails
  
#endif
#if defined(GLUT)
  for (int i = 0; pString[i]; ++i)
    ::glutStrokeCharacter(GLUT_STROKE_ROMAN, pString[i]);
#endif
#if defined(WIN32)
  if (m_nActiveFont >= 0) {
    ::glListBase(m_generatedFonts[m_nActiveFont]->displayListBase);
    
    ::glPushAttrib(GL_POLYGON_BIT);
    ::glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
    ::glCallLists(::strlen(pString), GL_UNSIGNED_BYTE, (const GLvoid*) pString);
    ::glPopAttrib();
    
    ::glListBase(0);
  }
  
#ifdef _DEBUG
  {
    GLenum error;
    
    error = ::glGetError();
    
    while (error != GL_NO_ERROR) {
      TRACE("OpenGLVisualRenderer::RenderString(): GL error %d\n", error);
      error = ::glGetError();
    }
  }
#endif
#endif
}

// Get width of string

/* virtual */ float
OpenGLVisualRenderer::GetStringWidth(const char* pString)
{
#if defined(GLTT)
  if (m_nActiveFont >= 0)
    return m_generatedFonts[m_nActiveFont]->font->getWidth(pString);
  
  // By not making GLTT and GLUT font rendering mutually exclusive, we can
  // fallback to GLUT font rendering if GLTT font rendering fails
  
#endif
#if defined(GLUT)
  float width = 0.0f;
  
  for (int i = 0; pString[i]; ++i)
    width += ::glutStrokeWidth(GLUT_STROKE_ROMAN, pString[i]);
  
  return width;
#endif  
#if defined(WIN32)
  float width = 0.0f;
  
  if (m_nActiveFont >= 0)
    for (int i = 0; pString[i]; ++i)
      width += m_generatedFonts[m_nActiveFont]->width[pString[i]];
    
    return width;
#endif
}

// Get height of string

/* virtual */ float
OpenGLVisualRenderer::GetFontHeight()
{
#if defined(GLTT)
  if (m_nActiveFont >= 0)
    return m_generatedFonts[m_nActiveFont]->height;
  
  // By not making GLTT and GLUT font rendering mutually exclusive, we can
  // fallback to GLUT font rendering if GLTT font rendering fails
  
#endif
#if defined(GLUT)
  return 152.38;
#endif
#if defined(WIN32)
  if (m_nActiveFont >= 0)
    return m_generatedFonts[m_nActiveFont]->height;
#endif
  
  return 0;
}

// Get baseline of string

float
OpenGLVisualRenderer::GetFontBaseline()
{
#if defined(GLTT)
  if (m_nActiveFont >= 0)
    return m_generatedFonts[m_nActiveFont]->baseline;
#endif
#if defined(GLUT)
  return 33.33f;
#endif
#if defined(WIN32)
  return 0.0f;
#endif  
}

/* virtual */ BOOL
OpenGLVisualRenderer::SetFontFamily(const char* pFontname, BOOL bBold, BOOL bItalic)
{
#if defined(GLTT) || defined(WIN32)
  int fontNumber = 0;
  
  while (fontNumber < m_generatedFonts.GetLength())
    if (m_generatedFonts[fontNumber]->italic == bItalic &&
      m_generatedFonts[fontNumber]->bold == bBold &&
      ::strcmp(m_generatedFonts[fontNumber]->fontname, pFontname) == 0) {
      m_nActiveFont = fontNumber;
      
      return TRUE;
    } else
      ++fontNumber;
#endif
    
#if defined(GLTT)
    
    FTFace* pFace = new FTFace;
    GLTTFont* pFont;
    ZString fontname(pFontname);
    
    if (::strcmp(pFontname, "SERIF") == 0)
      fontname = "times";
    else
      if (::strcmp(pFontname, "SANS") == 0)
        fontname = "arial";
      else
        if (::strcmp(pFontname, "TYPEWRITER") == 0)
          fontname = "cour";
        
        if (bBold && !bItalic)
          fontname += "bd";
        else {
          if (bBold)
            fontname += "b";
          if (bItalic)
            fontname += "i";
        }
        
        fontname += ".ttf";
        
        if (pFace)
          if (pFace->open(fontname)) {
            pFont = new GLTTFont(pFace);
            
            if (pFont) {
              pFont->setPrecision(25);
              
              if (pFont->create(240)) {
                GeneratedFont* pGeneratedFont = new GeneratedFont;
                
                pFont->load();
                
                if (pGeneratedFont) {
                  pGeneratedFont->fontname = pFontname;
                  pGeneratedFont->face = pFace;
                  pGeneratedFont->font = pFont;
                  pGeneratedFont->bold = bBold;
                  pGeneratedFont->italic = bItalic;
                  pGeneratedFont->height = pFont->getHeight();
                  pGeneratedFont->baseline = -pFont->getDescender();
                  
                  m_generatedFonts += pGeneratedFont;
                  
                  m_nActiveFont = fontNumber;
                  
                  return TRUE;
                } else
                  COMPLAIN("Out of memory creating font; cannot do font rendering\n");
              } else
                COMPLAIN("Failed to create outline font; cannot do font rendering\n");
              
              delete pFont;
            } else
              COMPLAIN("Failed to create font; cannot do font rendering\n");
          }
          
          delete pFace;
          
          return FALSE;
          
#endif
#if defined(GLUT)
          return TRUE;
#endif
#ifdef WIN32
          CWnd* mainWnd = AfxGetMainWnd();
          CDC* pDC = mainWnd->GetDC();
          
          if (pDC == NULL ||
            pFontname == NULL)
            return FALSE;
          
          LOGFONT logfont;
          GLYPHMETRICSFLOAT gmf[256];
          
          // lfHeight can't be used to change the font size
          logfont.lfHeight		= -12; // use glScale to change size
          logfont.lfWidth		= 0;
          logfont.lfEscapement		= 0;
          logfont.lfOrientation		= logfont.lfEscapement;
          logfont.lfWeight		= bBold ? 700 : 400;
          logfont.lfItalic		= bItalic;
          logfont.lfUnderline		= FALSE;
          logfont.lfStrikeOut		= FALSE;
          logfont.lfCharSet		= ANSI_CHARSET;
          logfont.lfOutPrecision	= OUT_TT_ONLY_PRECIS;
          logfont.lfClipPrecision	= CLIP_DEFAULT_PRECIS;
          logfont.lfQuality		= DEFAULT_QUALITY;
          logfont.lfPitchAndFamily      = FF_DONTCARE|DEFAULT_PITCH;
          
          if (::strcmp(pFontname, "SERIF") == 0)
            ::lstrcpy(logfont.lfFaceName, "Times New Roman");
          else
            if (::strcmp(pFontname, "SANS") == 0)
              ::lstrcpy(logfont.lfFaceName, "Arial");
            else
              if (::strcmp(pFontname, "TYPEWRITER") == 0)
                ::lstrcpy(logfont.lfFaceName, "Courier New");
              else
                ::lstrcpy(logfont.lfFaceName, pFontname);
              
              CFont newfont;
              
              // returns 0 if it fails
              
              BOOL success = newfont.CreateFontIndirect(&logfont);
              CFont* oldfont = pDC->SelectObject(&newfont);
              GeneratedFont* pGeneratedFont;
              
              ASSERT(0 != oldfont);
              
              // Create a set of display lists based on the glyphs of the TrueType font 
              // notice that we really waste the first 32 spaces....
              // if there's a problem delete the display lists
              
              if (success == FALSE) {
                TRACE("CreateFontIndirect failed: probably couldn't find desired font '%s', using default font\n", logfont.lfFaceName);
                
                return FALSE;
              }
              
              pGeneratedFont = new GeneratedFont();
              
              if (pGeneratedFont) {
                pGeneratedFont->displayListBase = ::glGenLists(256);
                
                if (pGeneratedFont->displayListBase &&
                  ::wglUseFontOutlines(pDC->m_hDC, 0, 256, pGeneratedFont->displayListBase, 1.0f, 0.0f, WGL_FONT_POLYGONS, gmf)) {
                  float height = 0.0f;
                  
                  pDC->SelectObject(oldfont);
                  
                  pGeneratedFont->fontname = pFontname;
                  pGeneratedFont->bold = bBold;
                  pGeneratedFont->italic = bItalic;
                  
                  for (int textdimension = 0; textdimension < 256; ++textdimension) {
                    pGeneratedFont->width[textdimension] = gmf[textdimension].gmfBlackBoxX;
                    
                    if (height < gmf[textdimension].gmfBlackBoxY)
                      height = gmf[textdimension].gmfBlackBoxY;
                  }
                  
                  pGeneratedFont->height = height;
                  
                  m_generatedFonts += pGeneratedFont;
                  
                  m_nActiveFont = fontNumber;
                  
                  return TRUE;
                } else {
                  TRACE("Couldn't generate display lists for font '%s', using default font\n", logfont.lfFaceName);
                  
                  ::glDeleteLists(pGeneratedFont->displayListBase, 256);
                  delete pGeneratedFont;
                }
              }
              
              return FALSE;
#endif
}

// Drag object

/* virtual */ void
OpenGLVisualRenderer::Drag()
{
  for (int i = 0; i < m_pActiveDragSensors.GetLength(); ++i)
    m_pActiveDragSensors[i]->Drag(this, m_nMouseX, m_nMouseY);
}

// Select object

/* virtual */ int OpenGLVisualRenderer::Select(
  int nNewButtons, int nOldButtons)
{
  int pickObject = 0;
  
  //ML - changed MediaObject* pScene to Root* pRoot
  //MediaObject* pScene = m_pPresenter->GetScene();
  Root* pRoot = m_pPresenter->GetScene();

  if (pRoot)
  {
    ZArray<MediaObjectProxy*> activatedSensors;
    MediaObjectProxy* intersectingObject = NULL;
    GLdouble objX = 0.0, objY = 0.0, objZ = 0.0;
    
    // if mouse coords are "valid" >>>>
    if (0 <= m_nMouseX && m_nMouseX < m_nViewportWidth &&
      0 <= m_nMouseY && m_nMouseY < m_nViewportHeight)
    {
      Viewpoint* pActiveViewpoint = pRoot->GetBoundViewpoint();
      ViewpointProxy * poViewpointProxy;
      if (pActiveViewpoint != NULL)
        poViewpointProxy = (ViewpointProxy *) pActiveViewpoint->GetRenderingObject();
      else
        poViewpointProxy = NULL;
      MediaObjectProxy* pSceneRenderer = (MediaObjectProxy*) pRoot->GetRenderingObject();
      GLint viewport[] = { 0, 0, m_nViewportWidth, m_nViewportHeight };
      
      ZArray<MediaObjectProxy*> dummyArray;
      unsigned int depth = UINT_MAX;
      Matrix transform;
      
      ::glMatrixMode(GL_PROJECTION);
      ::glLoadIdentity();
      
      ::gluPickMatrix(double(m_nMouseX), double(viewport[3] - m_nMouseY), 1.0, 1.0, viewport);
      
      if (pActiveViewpoint)
      {
        CullFlag cullFlag = PARTIALLY_INSIDE;
        
        // load projection matrix from viewpoint
        
        ::gluPerspective(pActiveViewpoint->fieldOfView /* GetFieldOfView() */ * 180.0f / M_PI,
          m_dAspectRatio,
          m_dNearClipping, m_dFarClipping);
        
        ::glMatrixMode(GL_MODELVIEW);
        
        poViewpointProxy->ViewpointToWorldMatrix(m_pPresenter->GetTime(), transform);
        ::glLoadMatrixf(transform);
        
        // check for intersection
        
        ((MediaObjectProxy*) pSceneRenderer)->
        // ((NodeProxy *) pSceneRenderer)->
            CullOrSelect(transform, cullFlag,
						    this, m_pPresenter->GetTime(),
                intersectingObject, activatedSensors, depth, dummyArray);
      }
      else
      {
        CullFlag cullFlag = COMPLETELY_INSIDE;
        
        // we don't have a viewpoint; use an orthogonal viewpoint instead
        
        ::glOrtho(-m_nViewportWidth / 2, m_nViewportWidth / 2,
          -m_nViewportHeight / 2, m_nViewportHeight / 2,
          -256, 256);
        
        ::glMatrixMode(GL_MODELVIEW);
        
        ::glLoadIdentity();
        
        // check for intersection
        
        ((MediaObjectProxy*) pSceneRenderer)->
        // ((NodeProxy *) pSceneRenderer)->
            CullOrSelect(transform, cullFlag,
						    this, m_pPresenter->GetTime(),
                intersectingObject, activatedSensors, depth, dummyArray);
      }
      
      if (intersectingObject)
      {
        GLdouble projection[16];
        GLdouble modelview[16];
        GLint viewport[4];
        
        ::glGetDoublev(GL_MODELVIEW_MATRIX, modelview);
        ::glGetDoublev(GL_PROJECTION_MATRIX, projection);
        ::glGetIntegerv(GL_VIEWPORT, viewport);
        
        if (!::gluUnProject(double(m_nMouseX), double(m_nMouseY), double(depth) / 4294967295.0,
          modelview,
          projection,
          viewport,
          &objX, &objY, &objZ))
          COMPLAIN("gluUnProject() failed?!\n");
      }
    }
    // if mouse coords are "valid" <<<<
    
#ifdef MMI    
    if (intersectingObject) {
      pickObject = 1;
      
      if (m_pHeadSelectedNode == NULL) {
        m_pHeadSelectedNode = new SelectedNode;
        m_pTailSelectedNode = new SelectedNode;
        m_pHeadSelectedNode->InsertAfterNode(m_pTailSelectedNode);
        m_nLastUndo = 0;
      }
      
      if (m_pHeadSelectedNode->next != NULL)
        if (m_pHeadSelectedNode->next->m_nMoveNumber >= 9) {
          for (SelectedNode* i = m_pTailSelectedNode->prev; i->m_nMoveNumber == m_nLastUndo; i = m_pTailSelectedNode->prev) {
            i->FreeSelectedNode();	
            delete i;
          }
          
          m_nLastUndo++;
        }		
        
        SelectedNode* newSelectedNode = new SelectedNode;
        
        newSelectedNode-> selectedObject = (Group*)intersectingObject->GetMediaObject();
        newSelectedNode->m_nMoveNumber = m_nNumberofMovements;
        m_pHeadSelectedNode->InsertAfterNode(newSelectedNode);
    }
#endif
    
    {
      int i;
      
      /* First, if any sensors are already active, and if we have registered
      a mouse click of some sorts, those sensors should be triggered. */
      
      if (!(nNewButtons & RIGHT_BUTTON) &&
        (nOldButtons & RIGHT_BUTTON)) 
      {
        int index;
        
        for (int i = activatedSensors.GetLength() - 1; i >= 0; --i)
        {
          if ((index = m_pActiveTouchSensors.Find((TouchSensorProxy*) activatedSensors[i])) != -1)
          {
            m_pActiveTouchSensors[index]->Trigger(this);
          }
          else
          {
            if ((index = m_pActiveAnchors.Find((AnchorProxy*) activatedSensors[i])) != -1)
              m_pActiveAnchors[index]->Trigger(this);
#ifdef PRE_FDIS
            else
              if ((index = m_pActiveAnchors2D.Find((Anchor2DProxy*) activatedSensors[i])) != -1)
                m_pActiveAnchors2D[index]->Trigger(this);
#endif // PRE_FDIS
          }
        }

        // Deactivate all dragsensors
        while (m_pActiveDragSensors.GetLength())
          m_pActiveDragSensors[0]->Deactivate(this, m_nMouseX, m_nMouseY);
      }
      
      if ((nNewButtons & RIGHT_BUTTON) &&
        !(nOldButtons & RIGHT_BUTTON)) 
      {
        // If we got a mouse down, first deactivate all sensors
        
        while (m_pActiveTouchSensors.GetLength())
          m_pActiveTouchSensors[0]->Deactivate(this, FALSE);
        
        while (m_pActiveAnchors.GetLength())
          m_pActiveAnchors[0]->Deactivate(this, FALSE);
        
#ifdef PRE_FDIS
        while (m_pActiveAnchors2D.GetLength())
          m_pActiveAnchors2D[0]->Deactivate(this, FALSE);
#endif // PRE_FDIS

        // Activate all sensors
        for (i = activatedSensors.GetLength() - 1; i >= 0; --i)
        {
          activatedSensors[i]->Activate(this, objX, objY, objZ, m_nMouseX, m_nMouseY, true);
        }
      } 
      else 
      {
        // Deactivate sensors that we no longer have
        for (i = 0; i < m_pActiveTouchSensors.GetLength();)
        {
          if (activatedSensors.Find(m_pActiveTouchSensors[i]) != -1)
            activatedSensors.DeleteFrom(activatedSensors.Find(m_pActiveTouchSensors[i++]));
          else
            m_pActiveTouchSensors[i]->Deactivate(this, nNewButtons & RIGHT_BUTTON);
        }
        for (i = 0; i < m_pActiveAnchors.GetLength();)
        {
          if (activatedSensors.Find(m_pActiveAnchors[i]) != -1)
            activatedSensors.DeleteFrom(activatedSensors.Find(m_pActiveAnchors[i++]));
          else
            m_pActiveAnchors[i]->Deactivate(this, nNewButtons & RIGHT_BUTTON);
        }
#ifdef PRE_FDIS
        for (i = 0; i < m_pActiveAnchors2D.GetLength();)
        {
          if (activatedSensors.Find(m_pActiveAnchors2D[i]) != -1)
            activatedSensors.DeleteFrom(activatedSensors.Find(m_pActiveAnchors2D[i++]));
          else
            m_pActiveAnchors2D[i]->Deactivate(this, nNewButtons & RIGHT_BUTTON);
        }
#endif // PRE_FDIS
        for (i = 0; i < m_pActiveDragSensors.GetLength(); ++i)
        {
          if (activatedSensors.Find(m_pActiveDragSensors[i]) != -1)
            activatedSensors.DeleteFrom(activatedSensors.Find(m_pActiveDragSensors[i]));
        }

        // Activate new sensors
        for (i = activatedSensors.GetLength() - 1; i >= 0; --i)
        {
          activatedSensors[i]->Activate(this, objX, objY, objZ, m_nMouseX, m_nMouseY, false);
        }
      }
      
      if (m_pActiveTouchSensors.GetLength() ||
        m_pActiveAnchors.GetLength() ||
#ifdef PRE_FDIS
        m_pActiveAnchors2D.GetLength() ||
#endif // PRE_FDIS
        m_pActiveDragSensors.GetLength()) 
        m_pActiveNode = intersectingObject;
      else
        m_pActiveNode = NULL;
    }
  } 
  else
  {
    COMPLAIN("OpenGLVisualRenderer::Select(): no scene?\n");
  }

#ifdef _DEBUG  
  {
    GLenum error;
    
    error = ::glGetError();
    
    while (error != GL_NO_ERROR) {
      TRACE("GL error: %d\n", error);
      error = ::glGetError();
    }
  }
#endif
  
  return pickObject;
}

// Resize the display

/* virtual */ void
OpenGLVisualRenderer::ReSize(int nWidth, int nHeight)
{
  m_nViewportWidth = nWidth;
  m_nViewportHeight = nHeight;
  
  ::glViewport(0, 0, nWidth, nHeight);
  
  m_dAspectRatio = double(nWidth) / double(nHeight);
  
  if (m_nStereoDisplayList) {
    ::glDeleteLists(m_nStereoDisplayList, 1);
    m_nStereoDisplayList = 0;
  }
}

#ifdef _DEBUG

// Check for OpenGL errors

BOOL
OpenGLVisualRenderer::CheckGLError(const char* pString)
{
  GLenum error = ::glGetError();
  
  if (error) {
    ::printf(pString);
    
    do {
      ::printf(" %d", error);
	  error = ::glGetError();
    } while (error);
	//ML - changed this line to remove a warning
    //} while ((error = ::glGetError()));

    ::printf("\n");
    
#ifndef WIN32
    ((OpenGLApplication*) m_pPresenter->GetExecutive()->GetApplication())->Debug();
#endif
    
    return TRUE;
  } else
    return FALSE;
}

#endif

/* Generate stereo perspective matrix.

  This routine does a stereoscopic offset perspective projection for one
  eye's view.  The "center of interest" of the scene, which will be
  rendered at a zero parallax setting, is assumed to be at the coordinate
  system origin (0.0, 0.0, 0.0).
  
    Parameters:
    
      fovy_degrees = field of view angle in vertical dimension, specified
      in degrees
      
        distance = the distance from the virtual cameras to the "center of
        interest" of the scene
        
          left_eye = TRUE or FALSE
          
            znear, zfar = the near and far clipping plane, relative to the
            "center of interest" of the scene
            
              aspect = the aspect ratio of the frustrum that will be set up
              appropriate to the dimensions of the viewport
              
called by redraw()   */

void
OpenGLVisualRenderer::StereoPerspective(float fFovyDegrees, float fDistance, BOOL bLeftEye,
                                        float fZNear, float fZFar, float fAspect)
{
  double left, right, top, bottom;
  double clip_near, clip_far;
  double camera_separation;
  double eyeview;
  double horz_tan, vert_tan, fovy_radians, perspective_offset;
  
  clip_near = fDistance - fZNear;
  clip_far = fDistance - fZFar;
  
  fovy_radians = double(fFovyDegrees) * M_PI / 180.0;
  horz_tan = ::tan(fovy_radians * fAspect / 2.0);
  vert_tan = ::tan(fovy_radians / 2.0);
  
  camera_separation = fDistance * horz_tan / 10.0f;
  eyeview = camera_separation / 2.0;
  if (bLeftEye)
    eyeview = -eyeview;
  
  top = vert_tan * clip_near;
  bottom = -top;
  right = horz_tan * clip_near;
  left = -right;
  
  perspective_offset = eyeview * (clip_near / fDistance);
  right -= perspective_offset;
  left -= perspective_offset;
  
  ::glFrustum(left, right, bottom, top, clip_near, clip_far);
  ::glTranslated(-eyeview, 0.0, -fDistance);
}

// Stereo paint method

void
OpenGLVisualRenderer::StereoPaint(DWORD dwTime)
{
  //ML - changed MediaObject* pScene to Root* pRoot
  //MediaObject * pScene = m_pPresenter->GetScene();
  Root* pRoot = m_pPresenter->GetScene();
  MediaObjectProxy* pSceneRenderer = (MediaObjectProxy*) pRoot->GetRenderingObject();
  Viewpoint* pActiveViewpoint = pRoot->GetBoundViewpoint();
  ViewpointProxy * poViewpointProxy = (ViewpointProxy *) pActiveViewpoint->GetRenderingObject();
  
  GLint displayList;
  
  Matrix dummy;
  CullFlag cullFlag = PARTIALLY_INSIDE;
  
  displayList = ::glGenLists(1);
  
  ::glNewList(displayList, GL_COMPILE);
  pSceneRenderer->CullOrRender(dummy, cullFlag,
    this, dwTime);
  ::glEndList();
  
  {
    Matrix transform;
    
    // load projection matrix from viewpoint
    
    ::glMatrixMode(GL_PROJECTION);
    ::glLoadIdentity();
    StereoPerspective(pActiveViewpoint->fieldOfView /* GetFieldOfView() */ * 180.0f / M_PI,
      0.5f, TRUE,
      -m_dNearClipping, -m_dFarClipping, m_dAspectRatio);
    poViewpointProxy->ViewpointToWorldMatrix(0, transform);
    ::glMultMatrixf(transform);
    
    // render left eye
    
    ::glViewport(0, 0, m_nViewportWidth, m_nViewportHeight / 2);
    ::glMatrixMode(GL_MODELVIEW);
    ::glCallList(displayList);
  }
  
  {  
    Matrix transform;
    
    // load projection matrix from viewpoint
    
    ::glMatrixMode(GL_PROJECTION);
    ::glLoadIdentity();
    
    StereoPerspective(pActiveViewpoint->fieldOfView /* GetFieldOfView() */ * 180.0f / M_PI,
      0.5f, FALSE,
      -m_dNearClipping, -m_dFarClipping, m_dAspectRatio);
    poViewpointProxy->ViewpointToWorldMatrix(0, transform);
    ::glMultMatrixf(transform);
    
    // render right eye
    
    ::glViewport(0, m_nViewportHeight / 2, m_nViewportWidth, (m_nViewportHeight + 1) / 2);
    ::glMatrixMode(GL_MODELVIEW);
    ::glCallList(displayList);
  }  
  
  ::glDeleteLists(displayList, 1);
  
  if (m_nStereoDisplayList)
    ::glCallList(m_nStereoDisplayList);
  else {
    m_nStereoDisplayList = ::glGenLists(1);
    ::glNewList(m_nStereoDisplayList, GL_COMPILE_AND_EXECUTE);
    
    ::glMatrixMode(GL_PROJECTION);
    ::glLoadIdentity();
    ::glOrtho(0.0, m_nViewportWidth, 0.0, m_nViewportHeight, -1.0, 1.0);
    ::glMatrixMode(GL_MODELVIEW);
    ::glLoadIdentity();
    ::glViewport(0, 0, m_nViewportWidth, m_nViewportHeight);
    
    // Disable stuff that's likely to slow down glDrawPixels.
    
    ::glPushAttrib(GL_ENABLE_BIT);
    
    ::glDisable(GL_ALPHA_TEST);
    ::glDisable(GL_DEPTH_TEST);
    ::glDisable(GL_FOG);
    ::glDisable(GL_LIGHTING);
    
#ifdef GL_SGIS_multisample
    ::glDisable(GL_MULTISAMPLE_SGIS);
#endif
    
    {
      int width = m_nViewportWidth;
      int height = m_nViewportHeight;
      
      // textures must have size and width == 2^n
      
      int numshifts;
      
      // align width
      
      numshifts = -1;
      
      for (int i = 0; i < 16; ++i) {
        if (width & 1)
          numshifts = i;
        
        width >>= 1;
      }
      
      width = 1 << numshifts;
      if (width < m_nViewportWidth)
        width <<= 1;
      
      // align height
      
      numshifts = -1;
      
      for (int j = 0; j < 15; ++j) {
        if (height & 1)
          numshifts = j;
        
        height >>= 1;
      }
      
      height = 1 << numshifts;
      if (height < m_nViewportHeight)
        height <<= 1;
      
      ::glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);
      ::glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
      ::glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
      
      ::glEnable(GL_TEXTURE_2D);
      
      ::glMatrixMode(GL_TEXTURE);
      ::glLoadIdentity();
      
      if (width != m_nViewportWidth ||
        height != m_nViewportHeight)
        ::glScalef(float(m_nViewportWidth) / float(width),
        float(m_nViewportHeight) / float(height), 1.0f);
      
      ::glMatrixMode(GL_MODELVIEW);
      
#ifndef GL_VERSION_1_1
      ::glCopyTexImage2DEXT(GL_TEXTURE_2D, 0, GL_RGB,
        0, 0,
        width, height,
        0);
#else
      ::glCopyTexImage2D(GL_TEXTURE_2D, 0, GL_RGB,
        0, 0,
        width, height,
        0);
#endif
      
      ::glBegin(GL_LINES);
      
      {
        int swap = 0;
        
        for (int k = 0; k < m_nViewportHeight; ++k) {
	//satish changed this block
	/*
          ::glTexCoord2f(0.0f, (swap * 0.5f) + 0.5f * float(k) / m_nViewportHeight);
          ::glVertex2i(0, k);
          ::glTexCoord2f(1.0f, (swap * 0.5f) + 0.5f * float(k) / m_nViewportHeight);
          ::glVertex2i(m_nViewportWidth - 1, k);
         */ 
          glTexCoord2f(0.0f, (swap * 0.5f) + 0.5f * float(k) / m_nViewportHeight);
          glVertex2i(0, k);
          glTexCoord2f(1.0f, (swap * 0.5f) + 0.5f * float(k) / m_nViewportHeight);
          glVertex2i(m_nViewportWidth - 1, k);
          
          swap = !swap;
        }
      }
      
      ::glEnd();
    }
    
    ::glPopAttrib();
    
    ::glEndList();
  }
}

// Get capabilities

inline /* virtual */ void
OpenGLVisualRenderer::GetCapabilities(Capabilities* pCapabilities)
{
  // Stefano Battista 
  // @@@@ 98/11/10 changed from tagFormat to MediaFormat
  pCapabilities->AddFormatCapability((MediaFormat) (MediaFormat :: RGBA), 0);
  pCapabilities->AddFormatCapability((MediaFormat) (MediaFormat :: RGB), 0);
  pCapabilities->AddFormatCapability((MediaFormat) (MediaFormat :: FBA), 0);
#ifdef _SAIT_3DTG_MP4VIDEO_
  pCapabilities->AddFormatCapability((MediaFormat) (MediaFormat :: YUV), 0);
#endif _SAIT_3DTG_MP4VIDEO_
/***
  pCapabilities->AddFormatCapability(FORMAT_RGBA, 0);
  pCapabilities->AddFormatCapability(FORMAT_RGB, 0);
  pCapabilities->AddFormatCapability(FORMAT_FBA, 0);
***/
  /* if (m_bColorMatrixExtension) {
    pCapabilities->AddFormatCapability(FORMAT_YUV411, 0);
    pCapabilities->AddFormatCapability(FORMAT_YUV422, 0);
    pCapabilities->AddFormatCapability(FORMAT_YUV444, 0);
  } */
}


#ifdef MMI

Viewpoint* OpenGLVisualRenderer::GetActiveViewPoint()
{
  //ML - changed MediaObject* pScene to Root* pRoot
  //MediaObject* pScene = m_pPresenter->GetScene();
  Root* pRoot = m_pPresenter->GetScene();
  
  if (!pRoot) return NULL;
  Viewpoint* pActiveViewpoint = pRoot->GetBoundViewpoint();
  return pActiveViewpoint;
  
}

void OpenGLVisualRenderer::SetActiveViewpoint(Viewpoint * pViewpoint)
{
  //ML - changed MediaObject* pScene to Root* pRoot
  //MediaObject* pScene = m_pPresenter->GetScene();
  Root* pRoot = m_pPresenter->GetScene();
  
  if (pRoot) 
    pRoot->PushViewpoint(pViewpoint);
  
}

void OpenGLVisualRenderer::LBDownAction(UINT nFlags, CPoint point)
{
  m_cPoint = point;
  m_nFlags = nFlags;
  m_bSelection = Select((int) point.x, (int) point.y);
}

void OpenGLVisualRenderer::LBUpAction(UINT /* nFlags */, CPoint /* point */)
{
#if 0
  float xvalue,yvalue;
  BOOL success = FALSE;
  m_nFlags = nFlags;
  
  if (m_bSelection && m_pHeadSelectedNode != NULL && m_bMovement) {
    point.x = m_cMMovePoint.x - m_cPoint.x;
    point.y = m_cMMovePoint.y - m_cPoint.y;
    int currentindex = m_pHeadSelectedNode->next->m_nMoveNumber;
    for (SelectedNode* i = m_pHeadSelectedNode->next; i != m_pTailSelectedNode; i = i->next) {
      if (currentindex == i->m_nMoveNumber) {
        m_pParentGroup = i->selectedObject;
        
        const ZArray<Group*>& groups = m_pParentGroup->GetGroupParents();
        
        while (dynamic_cast<Group *>(m_pParentGroup) == NULL)
          for (int pcounter = 0; pcounter < groups.GetLength(); ++pcounter)
            if (dynamic_cast<Group *>(groups[pcounter]) != NULL) {
              m_pParentGroup = groups[pcounter];
              pcounter = groups.GetLength();
            }
            
            Transform*	pUseTransform;
            Group* pParentMObject;
            pUseTransform = new Transform;
            
			//ML - changed MediaObject* pScene to Root* pRoot
            //Root* pRoot = m_pPresenter->GetScene();
            MediaObject** pRoot = m_pPresenter->GetScene();
            Viewpoint* pActiveViewpoint = pRoot->GetBoundViewpoint();
            if (pActiveViewpoint) {
              xvalue = (point.x * m_dAspectRatio) / 100;			
              yvalue = (point.y * -m_dAspectRatio)/ 100;
            }
            else {
              xvalue = point.x;
              yvalue = point.y * -1;
            }
            
            Vec3f translation1(xvalue, yvalue, 0);
            pUseTransform->SetTranslation(translation1);
            pUseTransform->SetExecutive(m_pPresenter->GetExecutive());
            pUseTransform->RegisterObject();
            
            pParentMObject = (Group*)m_pParentGroup->GetFirstChild();
            ((Transform *)pUseTransform)->AddChild((Node3D *)pParentMObject);
            ((Transform *)m_pParentGroup)->AddChild((Node3D *)pUseTransform);
            ((Group *)m_pParentGroup)->RemoveChild((Node3D *)pParentMObject);
            ((Node3D *)pParentMObject)->AddParent((Group *)pUseTransform);
            ((Node3D *)pUseTransform)->AddParent((Group *)m_pParentGroup);
            
            i->selectedObject = pUseTransform;
      }
    }
    ++m_nNumberofMovements;
    TRACE( "\n ++ %d", m_nNumberofMovements );
  } 
  m_bMovement = FALSE;
  
  return;
#endif
}

void OpenGLVisualRenderer::MouseMovement(UINT nFlags, CPoint point)
{
  switch(nFlags) {
		case MK_LBUTTON :
      if (m_pHeadSelectedNode != NULL) 
        m_cMMovePoint = point;
      m_bMovement = TRUE;
      break;
    default :
      m_bMovement = FALSE;
  }
  
}

BOOL OpenGLVisualRenderer::AllowUndo()
{
  if ((m_pHeadSelectedNode == NULL) || (m_pHeadSelectedNode->next == m_pTailSelectedNode))
    return FALSE;
  else
    return TRUE;
}

BOOL OpenGLVisualRenderer::UndoTransformation()
{
  //#ED
  /*
  Group *pChildObject;
  BOOL success = FALSE;
  Group* pObjectChoosen;
  
    int currentindex = m_pHeadSelectedNode->next->m_nMoveNumber;
    SelectedNode* p;
    
      for (SelectedNode* i = m_pHeadSelectedNode->next; i != m_pTailSelectedNode;) {
      if (currentindex == i->m_nMoveNumber) {
      pChildObject = i->selectedObject;
      
        pObjectChoosen = (Group*)pChildObject->GetFirstChild();
        
          const ZArray<Group*>& groups = pChildObject->GetGroupParents();
          
            ((Transform *)groups[0])->AddChild((Node3D *)pObjectChoosen);
            ((Group *)pChildObject)->RemoveChild((Node3D *)pObjectChoosen);
            ((Node3D *)pObjectChoosen)->AddParent((Group *)groups[0]);
            
              groups[0]->RemoveChild((Node3D*) pChildObject);
              ((Node3D *)pObjectChoosen)->RemoveParent(pChildObject);
              
                p = i->next;
                i->FreeSelectedNode();	
                delete i;
                i = p;
                }
                else
                i = i->next;
                }
                --m_nNumberofMovements;
                TRACE( "\n -- %d", m_nNumberofMovements );
  */
  return TRUE;
}

#endif
