// -*- c++ -*-

// This software module was originally developed by Edward Cooke (Teltec
// Ireland) and Karl Anders ygard (Telenor R&D) in the course of
// development of the MPEG-4 Systems standard (ISO/IEC 14496-1).  This
// software module is an implementation of a part of one or more MPEG-4
// Systems tools as specified by the MPEG-4 Systems standard.  ISO/IEC
// gives users of the MPEG-4 Systems standard free license to this software
// module or modifications thereof for use in hardware or software products
// claiming conformance to the MPEG-4 Systems standard.  Those intending to
// use this software module in hardware or software products are advised
// that its use may infringe existing patents.  The original developer of
// this software module and his/her company, the subsequent editors and
// their companies, and ISO/IEC have no liability for use of this software
// module or modifications thereof in an implementation.  Copyright is not
// released for non MPEG-4 Systems conforming products.  The original
// developer retains full right to use the code for his/her own purpose,
// assign or donate the code to a third party and to inhibit third parties
// from using the code for non MPEG-4 Systems conforming products.  This
// copyright notice must be included in all copies or derivative works.
//
// Copyright 1997, 1998.

#ifndef __OPENGLVISUALRENDERER_H__
#define __OPENGLVISUALRENDERER_H__

#include "VisualRenderer.h"
#include <GL/gl.h> // SOLARIS PORTING

#include "opengl/bindable/ViewpointProxy.h"
#include "opengl/shadow/ShadowRendering.h"

#if defined(GLTT)
class GLTTFont;
#include <gltt/FTFace.h>
#endif

class OpenGLPresenter;
class TextureNodeProxy;
class MultiTextureProxy;
class Material2DProxy;
class AnchorProxy;
class TouchSensorProxy;
class AnchorProxy;
class Anchor2DProxy;
class DragSensorProxy;
class NodeProxy;

//#ED 3.1
class SelectedNode;

/// OpenGL visual renderer

class OpenGLVisualRenderer
  : public VisualRenderer
{
public:

	// added by Helge Drumm 06.12.2006, TU Ilmenau, Shadow Multipass rendering 
	bool m_nTextureEnabled;
	
	bool m_nMaterialEnabled;

	bool m_nEmissiveEnabled;

	//*****************************************************************

  /// Constructor

         		OpenGLVisualRenderer(OpenGLPresenter* pPresenter);

  /// Destructor

  inline virtual	~OpenGLVisualRenderer();

  /// Stops rendering and performs cleanups

  virtual void		Terminate();

  /// Paint

  virtual void		Paint(RECT* rectangle);

  /// Resize the display

  virtual void		ReSize(int nWidth, int nHeight);

  /// Select object in scene

  virtual int		Select(int nNewButton, int nOldButton);

  /// Drag object in scene

  virtual void		Drag();

  /// Set mouse state

  virtual void		MouseState(int nButtons, int nX, int nY);

  /// Get capabilities of renderer

  virtual void		GetCapabilities(Capabilities* pPresentationCapabilities);

  /// Get presenter

  OpenGLPresenter*	GetPresenter() const;

private:

  // Pointer to our presenter

  OpenGLPresenter*	m_pPresenter;

public:

	//added by Helge Drumm, 06.12.2006, TU Ilmenau, Shadow Rendering
	ShadowRendering *shadowRend;

  /// Toggle collision detection

  BOOL			CollisionDetection(BOOL bCollisionDetection);

  /// Get whether to use collision detection

  BOOL			CollisionDetection() const;

  /// Toggle drawing of scene graph

  int			DrawSceneGraph(int nDrawSceneGraph);

  /// Get whether to draw scene graph

  int			DrawSceneGraph() const;

  /// Toggle drawing of bounding boxes

  int			DrawBBoxes(BOOL bDrawBBoxes);

  /// Get whether to draw bounding boxes

  int			DrawBBoxes() const;

  /// Toggle clearing of screen

  BOOL			ClearScreen(BOOL bClearScreen);

  /// Toggle use of Z buffer

  BOOL			UseZBuffer(BOOL bZBuffer);

  //added by Helge Drumm, 06.12.2006, TU Ilmenau, Shadow Rendering
    /// Toggle use of Accumulation buffer

  BOOL          UseABuffer(BOOL bABuffer);

  /// Toggle use HW ShadowRendering

  BOOL          UseHWShadow(BOOL bHWShadow);  
  
  // Get current frame rate in FPS

  float			GetFrameRate() const;

  // Get left frustum plane

  const Vec3f&		GetLeftFrustumPlane() const;

  // Get right frustum plane

  const Vec3f&		GetRightFrustumPlane() const;

  // Get top frustum plane

  const Vec3f&		GetTopFrustumPlane() const;

  // Get bottom frustum plane

  const Vec3f&		GetBottomFrustumPlane() const;


#ifdef SAIT_CONFORMANCE_2003  
  // Get ViewportHeight
  
  float				GetViewportWidth() const;

  // Get ViewportHeight

  float				GetViewportHeight() const; 	
#endif  SAIT_CONFORMANCE_2003

  // Set current texture

  void			SetCurrentTexture(TextureNodeProxy* pTexture);

  // Get current texture

  TextureNodeProxy*	GetCurrentTexture() const;

  // Set current Multitexture
  void			SetMultiTexture(MultiTextureProxy* pTexture);
  // Get current Multitexture
  MultiTextureProxy*	GetMultiTexture() const;

  // Set current material2D

  void			SetCurrentMaterial2D(Material2DProxy* pMaterial);

  // Get current material2D

  Material2DProxy*	GetCurrentMaterial2D() const;

  // Get texture mag function

  inline GLint		GetTextureMagFunc() const;

  // Set texture min function

  inline void		SetTextureMinFunc(GLint nFunc);

  // Get texture min function

  inline GLint		GetTextureMinFunc() const;

	// Get texture Flag function  -- added by Helge Drumm, 06.12.2006 TU Ilmenau, Shadow Rendering

  inline GLint		GetTextureEnabled() const;

	// Get material Flag function  -- added by Helge Drumm, 06.12.2006 TU Ilmenau, Shadow Rendering

  inline GLint		GetMaterialEnabled() const;


	// Get Emissive Color function  -- added by Helge Drumm, 06.12.2006 TU Ilmenau, Shadow Rendering

  inline GLint		GetEmissiveEnabled() const;

  
  
  // Set texture min function to use with mipmapping

  inline void		SetTextureMipmapMinFunc(GLint nFunc);

  // Get texture min function to use with mipmapping

  inline GLint		GetTextureMipmapMinFunc() const;

  // Set texture mag function

  inline void		SetTextureMagFunc(GLint nFunc);

  // Get next available OpenGL light

  inline int		RegisterLight();

  // Register active TouchSensor

  void			RegisterTouchSensor(TouchSensorProxy* pTouchSensorProxy);

  // Unregister active TouchSensor

  void			UnregisterTouchSensor(TouchSensorProxy* pTouchSensorProxy);

  // Register active Anchor

  void			RegisterAnchor(AnchorProxy* pAnchorProxy);

  // Unregister active Anchor

  void			UnregisterAnchor(AnchorProxy* pAnchorProxy);

  // Register active Anchor2D
							  
  void			RegisterAnchor2D(Anchor2DProxy* pAnchor2DProxy);

  // Unregister active Anchor2D

  void			UnregisterAnchor2D(Anchor2DProxy* pAnchor2DProxy);

  // Register active DragSensor

  void			RegisterDragSensor(DragSensorProxy* pDragSensorProxy);

  // Unregister active DragSensor

  void			UnregisterDragSensor(DragSensorProxy* pDragSensorProxy);

  /// Render string

  void			RenderString(const char* pString);

  /// Get width of string

  virtual float		GetStringWidth(const char* pString);

  /// Get height of font

  virtual float		GetFontHeight();

  /// Get baseline of font

  float			GetFontBaseline();

  /// Set font family

  virtual BOOL		SetFontFamily(const char* pFamily, BOOL bBold, BOOL bItalic);

  // Get OpenGL version

  float                 GetOpenGLVersion() const;

  // Is OpenGL extension supported?

  BOOL			IsExtensionSupported(const char* sExtension) const;

#ifdef _DEBUG

  // Check for OpenGL errors

  BOOL			CheckGLError(const char* pString);

#endif

private:

  // Current framerate

  float			m_fFrameRate;

  // Last time we measured frame rate

  long			m_nLastTime;

  // Number of frames since we last measured framerate

  float			m_fFrames;

  // Should we do collision detection?

  int			m_bCollisionDetection;

  // Should we draw the scene graph?

  int			m_nDrawSceneGraph;

  // Should we draw bounding boxes?

  int			m_bDrawBBoxes;

  // Should we clear the screen?

  BOOL                  m_bClearScreen;

  // Should we use Z buffering?

  BOOL                  m_bZBuffer;

  //added by Helge Drumm, 06.12.2006, TU Ilmenau, Shadow Rendering
  // Should we use A buffering?
  BOOL                 m_bABuffer;
  
  //added by Helge Drumm, 06.12.2006, TU Ilmenau, Shadow Rendering
  // Should we use HW ShadowRendering
  BOOL                 m_bHWShadow;
 
  //added by Helge Drumm, 06.12.2006, TU Ilmenau, Shadow Rendering
  // Pointer ShadowRendering object
  //ShadowRendering      *shadowRend;
  
  
  // Do we have the color matrix extension?

  BOOL			m_bColorMatrixExtension;

  // OpenGL version

  float			m_fOpenGLVersion;

  // Mouse buttons

  int			m_nMouseButtons;

  // Mouse position X

  int			m_nMouseX;

  // Mouse position Y

  int			m_nMouseY;

  // Mouse position X when left button was pressed

  int			m_nMouseFirstX;

  // Mouse position Y when left button was pressed

  int			m_nMouseFirstY;

  // Next available OpenGL light number

  int			m_nNextGLLight;

  // OpenGL texture minifying function

  GLint			m_nMinFunc;

  // OpenGL texture minifying function to use with mipmapping

  GLint			m_nMipmapMinFunc;

  // OpenGL texture magnifying function

  GLint			m_nMagFunc;

  // Current aspect ratio

  double		m_dAspectRatio;

  // Current near clipping

  double		m_dNearClipping;

  // Current far clipping

  double		m_dFarClipping;

  // Current viewport width

  int			m_nViewportWidth;

  // Current viewport height

  int			m_nViewportHeight;

  // azimuth for examing the scene

  int			azimuth_;

  // Current frustum planes

  Vec3f                 m_leftFrustumPlane;
  Vec3f                 m_rightFrustumPlane;
  Vec3f                 m_topFrustumPlane;
  Vec3f                 m_bottomFrustumPlane;

  // Current texture (if any)

  TextureNodeProxy*	m_pCurrentTexture;

  // MultiTexture (if any)
  MultiTextureProxy* m_pMultiTexture;

  // Current properties (if any)

  Material2DProxy*	m_pCurrentMaterial;

  // Current activated touch sensors (if any)

  ZArray<TouchSensorProxy*>
			m_pActiveTouchSensors;

  // Current activated anchors (if any)

  ZArray<AnchorProxy*>	m_pActiveAnchors;

  // Current activated anchors (if any)

#ifdef PRE_FDIS
  ZArray<Anchor2DProxy*>
			m_pActiveAnchors2D;
#endif // PRE_FDIS

  // Current activated drag sensors (if any)

  ZArray<DragSensorProxy*>
			m_pActiveDragSensors;

  // Current activated geometric object with touch sensor (if any)

  MediaObjectProxy * m_pActiveNode;

  // Renderer active mutex

  ZMutex                m_mtxRendererActive;

  // Generate stereo perspective matrix

  void                  StereoPerspective(float fFovyDegrees, float fDistance, BOOL bLeftEye,
					  float fZNear, float fZFar, float fAspect);

  // Stereo paint method

  void                  StereoPaint(DWORD dwTime);

  // interlaced stereo rearrangement display list

  int                   m_nStereoDisplayList;

  // Currently active font

  int                   m_nActiveFont;

#ifdef GLTT

  // Struct which keeps information about the generated GL fonts

  typedef struct {
    ZString             fontname;
    BOOL                bold;
    BOOL                italic;
    FTFace*             face;
    GLTTFont*           font;
    float               height;
    float               baseline;
  } GeneratedFont;

  // Array with information about the generated GL fonts

  ZArray<GeneratedFont*>
	                m_generatedFonts;

#elif defined(WIN32)

  // Struct which keeps information about the generated GL fonts

  typedef struct {
    ZString             fontname;
    BOOL                bold;
    BOOL                italic;
    float               height;
    float               width[256];
    int                 displayListBase;
  } GeneratedFont;

  // Array with information about the generated GL fonts

  ZArray<GeneratedFont*>
                        m_generatedFonts;

#endif

// Various MoMuSys MMI stuff

	UINT m_nFlags;
  // Stefano Battista 98/11/10
	// BOOL TriggerSensors(Node* pNode);
	BOOL TriggerSensors(MediaObject * pNode);

#ifdef MMI

	GLuint GenerateDisplayListForFont(const char* fontname, double xt);

	BOOL m_bSelection;
  // Stefano Battista 98/11/12
	// Group * m_pParentGroup;
	MediaObject * m_pParentGroup;
	BOOL m_bMovement;
	CPoint m_cPoint;
	CPoint m_cMMovePoint;
	SelectedNode* m_pTailSelectedNode;
	SelectedNode* m_pHeadSelectedNode;

public:

	GLuint GetNewDisplayListIDs( GLsizei range );

	BOOL UndoTransformation();
	BOOL AllowUndo();
	int m_nNumberofMovements;
	int m_nLastUndo;
	void MouseMovement(UINT nFlags, CPoint point);
//#ED 3.1
	void LBDownAction(UINT nFlags, CPoint point);
	void LBUpAction(UINT nFlags, CPoint point);

 	void SetActiveViewpoint(Viewpoint* pViewpoint);
	Viewpoint* GetActiveViewPoint();

#endif

};

#include "OpenGLVisualRenderer.i"

#endif // __OPENGLVISUALRENDERER_H__
