// -*- c++ -*-

// This software module was originally developed by Karl Anders ygard
// (Telenor R&D) in the course of development of the MPEG-4 Systems
// standard (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// For more information or to receive an up to date version of this module,
// mail karlo@nta.no or check <http://televr.fou.telenor.no/compositor/>.
//
// Copyright 1997, 1998.

#ifndef __VISUALRENDERER_H__
#define __VISUALRENDERER_H__

//#include "ztl.h"

class MediaObject;
class StreamConsumer;
//#include "Nodes.h"

#include "opengl/sensors/VisibilitySensorProxy.h"
#include "opengl/sensors/ProximitySensorProxy.h"

class Presenter;
class Viewpoint;
class Capabilities;
class VisibilitySensor;
class ProximitySensor;

/** Visual renderer.

    This class implements the visual renderer. */

class VisualRenderer
{
public:

  typedef enum {
    LEFT_BUTTON = 1,
    MIDDLE_BUTTON = 2,
    RIGHT_BUTTON = 4
  } ButtonState;

  /// Constructor

  inline		VisualRenderer(Presenter* pPresenter);

  /// Destructor

  inline virtual	~VisualRenderer();

  /// Initialise renderer

  virtual void		Init();

  /// Stops rendering and performs cleanups

  virtual void		Terminate();

  /// Paint

  virtual void		Paint(RECT* rectangle);

  /// Resize the display

  virtual void		ReSize(int nWidth, int nHeight);

  /// Select object in scene

  virtual int		Select(int nX, int nY);

  /// Get capabilities of renderer

  virtual void		GetCapabilities(Capabilities* pPresentationCapabilities);

  /** Get width of string.

      This method should be overloaded by subclasses that actually
      implement font rendering. */

  virtual float		GetStringWidth(const char* pString) = 0;

  /** Get height of font.

      This method should be overloaded by subclasses that actually
      implement font rendering. */

  virtual float		GetFontHeight() = 0;

  /** Set font family.

      This method should be overloaded by subclasses that actually
      implement font rendering. */

  virtual BOOL		SetFontFamily(const char* pFamily, BOOL bBold, BOOL bItalic) = 0;

  /** Register object that consumes a video stream.

      When a mediastream is not used, it must be closed.  In order to do
      this, we register objects that actually consume streams on a frame to
      frame basis, and shut off mediastreams that have become inactive
      since last frame. */

  void			RegisterStreamConsumer(StreamConsumer * pObject);

  /// Unregister object that consumes a video stream.

  void			UnregisterStreamConsumer(StreamConsumer * pObject);

  /** Register VisibilitySensor that has become visible.

      We register VisibilitySensors that are visible, and shut off sensors
      that have become non-visible since last frame. */

  void			RegisterVisibilitySensor(VisibilitySensor* pVisibilitySensor);

  /** Register ProximitySensor that has become activated.

      We register ProximitySensors that are activated, and shut off sensors
      that have not been activated since last frame. */

  void			RegisterProximitySensor(ProximitySensor* pProximitySensor);

private:

  // SB @@@@ changed from MediaObject to StreamConsumer >>>>
  /// Current active stream consumers
  // ZArray<MediaObject*>  m_activeStreamConsumers;
  ZArray<StreamConsumer *>  m_activeStreamConsumers;
  /// Old active stream consumers
  // ZArray<MediaObject*>  m_oldStreamConsumers;
  ZArray<StreamConsumer *>  m_oldStreamConsumers;
  // SB @@@@ changed from MediaObject to StreamConsumer <<<<

  /// Current visible VisibilitySensors
  ZArray<VisibilitySensor*>  m_visibleVisibilitySensors;
  /// VisibilitySensors that were visible last rendering frame
  ZArray<VisibilitySensor*>  m_oldVisibilitySensors;
  /// Current active ProximitySensors
  ZArray<ProximitySensor*>  m_activeProximitySensors;
  /// ProximitySensors that were active last rendering frame
  ZArray<ProximitySensor*>  m_oldProximitySensors;
};

#include "VisualRenderer.i"

#endif // __VISUALRENDERER_H__
