// -*- c++ -*-

// This software module was originally developed by Karl Anders ygard
// (Telenor R&D) in the course of development of the MPEG-4 Systems
// standard (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// Copyright 1997.

#include "OpenGl/OpenGl.h"
#include "FogProxy.h"
#include "opengl/base/Matrix.h"


// Constructor

FogProxy::FogProxy(Fog* pFog)
  : BindableProxy(pFog)
{
}


// Activate fog

void
FogProxy::Enable(OpenGLVisualRenderer* /* pRenderer */, DWORD dwTime /* dwTime */)
{
  // Fog* pFog = (Fog*) GetMediaObject();

  if (/* pFog-> */ GetVisibilityRange() != 0.0f) {
    const Color3f& color = /* pFog-> */ GetColor();
    const ZString& fogType = /* pFog-> */ GetFogType();
    Matrix transform;
    Vec3f vector(/* pFog-> */ GetVisibilityRange(), 0.0f, 0.0f);

    GLfloat fogColor[4] = {color.r,
			   color.g,
			   color.b,
			   0.0f};

    ::glPushAttrib(GL_COLOR_BUFFER_BIT | GL_FOG_BIT);

    ::glEnable(GL_FOG);

    if (::strcmp(fogType, "LINEAR") == 0) {
      ::glFogi(GL_FOG_MODE, GL_LINEAR);

      ::glFogf(GL_FOG_START, 1.0f);

      /* pFog-> */ ViewpointToWorldMatrix(dwTime, transform);
      ::glFogf(GL_FOG_END, (transform.MultiplyVector(vector)).GetLength());
    } else
      if (::strcmp(fogType, "EXPONENTIAL") == 0) {
	::glFogi(GL_FOG_MODE, GL_EXP);
	::glFogf(GL_FOG_DENSITY, 0.35f);
      }
    
    ::glFogfv(GL_FOG_COLOR, fogColor);
    ::glClearColor(color.r, color.g, color.b, 1.0f);
  }
}

// Deactivate fog

void
FogProxy::Disable(OpenGLVisualRenderer* /* pRenderer */, DWORD /* dwTime */)
{
  ::glPopAttrib();
}

// Render object description

/* virtual */ void
FogProxy::RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD /* dwTime */)
{
  GLfloat color[] = { 1.0f, 0.0f, 0.0f, 0.75f};

  //satish changed this line for compilation purposes
        //have to be fixed afterwards

  //::glColor4fv(color);
  glColor4fv(color);

  RenderDescriptionBox(pRenderer, "Fog");
}
