// -*- c++ -*-

// This software module was originally developed by Karl Anders ygard
// (Telenor R&D) in the course of development of the MPEG-4 Systems
// standard (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// Copyright 1997.

#ifndef __FOGPROXY_H__
#define __FOGPROXY_H__

#include "BindableProxy.h"

// class Fog;
//#include "Nodes.h"

/// Fog renderer 

class FogProxy
  : public BindableProxy
{
public:

  /// Constructor

			FogProxy(Fog* pFog);

  /// Activate fog

  void			Enable(OpenGLVisualRenderer* pRenderer, DWORD dwTime);

  /// Deactivate fog

  void			Disable(OpenGLVisualRenderer* pRenderer, DWORD dwTime);

  /// Render object description

  virtual void		RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD dwTime);

  // Stefano Battista
  // moved methods from Fog to FogProxy
public:
  /// Get pointer to related MediaObject
  inline Fog * GetFog() const
  { return (Fog *) GetMediaObject(); }
  
  /// Set color of fog
  inline void SetColor(const Color3f& rColor)
  { GetFog()->color = rColor; }
  
  /// Get color of fog
  //ML - changed return type from const Color3f& to const Color3f
  // to remove warning about returning address of temporary variable
  inline const Color3f GetColor() const
  { return GetFog()->color; }
  
  /// Set type of fog
  inline void SetFogType(const ZString& rFogType)
  { GetFog()->fogType = rFogType; }
  
  /// Get type of fog
  inline const ZString& GetFogType() const
  { return GetFog()->fogType; }
  
  /// Set visibility range of fog
  inline void SetVisibilityRange(float fVisibilityRange)
  { GetFog()->visibilityRange = fVisibilityRange; }
  
  /// Get visibility range of fog
  inline float GetVisibilityRange() const
  { return GetFog()->visibilityRange; }

};

//ML - removed FogProxy.i

#endif // __FOGPROXY_H__
