// -*- c++ -*-

// This software module was originally developed by Karl Anders ygard
// (Telenor R&D) in the course of development of the MPEG-4 Systems
// standard (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// Copyright 1997.

#include "OpenGl/OpenGl.h"
#include "ViewpointProxy.h"


// Constructor

ViewpointProxy::ViewpointProxy(Viewpoint * poViewpoint)
  : BindableProxy(poViewpoint)
{
  SetRoot(poViewpoint->GetRoot());
}


// Render object description

/* virtual */ void
ViewpointProxy::RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD /* dwTime */)
{
  GLfloat color[] = { 1.0f, 0.0f, 0.0f, 0.75f};
  
  //satish changed this line for compilation purposes
        //have to be fixed afterwards

  //::glColor4fv(color);
  glColor4fv(color);
  
  RenderDescriptionBox(pRenderer, "Viewpoint");
}


// SB @@@@ moved from Proxy

// Concatenate transformation with given matrix

void ViewpointProxy::ConcatenateTransform(Matrix& rTransform) const
{
  const Vec3f& rPosition = GetPosition();
  const Rotation& rOrientation = GetOrientation();
  
  // do translation
  
  if (rPosition.x != 0.0f ||
    rPosition.y != 0.0f ||
    rPosition.z != 0.0f)
    rTransform.Translate(rPosition.x,
			 rPosition.y,
       rPosition.z);
  
  if (rOrientation.theta != 0.0f)
    // do scaleRotation
    
    rTransform.Rotate(rOrientation.theta,
    rOrientation.x,
    rOrientation.y,
    rOrientation.z);
}


// Set bounding box and update parents

void ViewpointProxy::SetBbox(const Vec3f& rBboxSize, const Vec3f& rBboxCenter)
{
  Matrix mat;
  Vec3f size = rBboxSize;
  
  size *= -0.5f;
  size += rBboxCenter;
  
  ConcatenateTransform(mat);
  
  m_transformedBboxCenter = mat.MultiplyPoint(rBboxCenter);
  
  Vec3f x(mat[0], mat[1], mat[2]);
  Vec3f y(mat[4], mat[5], mat[6]);
  Vec3f z(mat[8], mat[9], mat[10]);
  
  x *= rBboxSize.x;
  y *= rBboxSize.y;
  z *= rBboxSize.z;
  
  m_transformedBboxSize.x = ::fabs(x.x) + ::fabs(y.x) + ::fabs(z.x);
  m_transformedBboxSize.y = ::fabs(x.y) + ::fabs(y.y) + ::fabs(z.y);
  m_transformedBboxSize.z = ::fabs(x.z) + ::fabs(y.z) + ::fabs(z.z);
  
// SOLARIS PORTING
// Created new objects to avoid memory dereferencing
  SF<Vec3f> sf_size, sf_center;
  sf_size = (SF<Vec3f>) rBboxSize;
  sf_center = (SF<Vec3f>) rBboxCenter;

  /* GetViewpoint()-> */ SetBbox( sf_size, sf_center);

// SOLARIS PORTING - END
}


// Collision detection

MediaObject * ViewpointProxy::DetectCollision()
{
  Matrix mat;
  Vec3f size, center;
  
  ConcatenateInverseTransform(mat);
  /* GetViewpoint()-> */ GetBbox(size, center);
  
  return /* Node */ MediaObjectProxy::DetectCollision(
    m_transformedBboxSize, 
    m_transformedBboxCenter,
    size, 
    center,
    mat,
    GetViewpoint() /* this */ );
}


// Concatenate inverse transformation with given matrix

void ViewpointProxy::ConcatenateInverseTransform(Matrix& rTransform) const
{
  const Vec3f& rPosition = GetViewpoint()->position;
  const Rotation& rOrientation = GetViewpoint()->orientation;
  
  // do inverse rotation
  
  if (rOrientation.theta != 0.0f)
    rTransform.Rotate(-rOrientation.theta,
    rOrientation.x,
    rOrientation.y,
    rOrientation.z);
  
  // do inverse translation
  
  if (rPosition.x != 0.0f ||
    rPosition.y != 0.0f ||
    rPosition.z != 0.0f)
    rTransform.Translate(-rPosition.x,
			 -rPosition.y,
       -rPosition.z);
}


// Get viewpoint to world matrix

/* virtual */ void
ViewpointProxy::ViewpointToWorldMatrix(
    DWORD dwTime, Matrix& rTransform) const
{
  ConcatenateInverseTransform(rTransform);
  
  MediaObjectProxy::ViewpointToWorldMatrix(dwTime, rTransform);
}


void ViewpointProxy::SetRoot(Root* pRoot)
{ 
  TRACE("[Viewpoint::SetRoot] \n");
  GetMediaObject()->SetRoot(pRoot);
//ML - removed the next if
// had the problem when a viewpoint is being replaced using BIFS REPLACE,
// that the new node is added first, and then the old one deleted
//  if (! GetMediaObject() -> GetRoot() -> GetBoundViewpoint())
  {
    TRACE("[Viewpoint::SetRoot] Push Viewpoint \n");
    GetMediaObject() -> GetRoot() -> PushViewpoint(GetViewpoint());
  }
}
