// -*- c++ -*-

// This software module was originally developed by Karl Anders ygard
// (Telenor R&D) in the course of development of the MPEG-4 Systems
// standard (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// Copyright 1997.

#ifndef __VIEWPOINTPROXY_H__
#define __VIEWPOINTPROXY_H__

#include "BindableProxy.h"
#include "Presenter.h"

class Viewpoint;

/// Viewpoint renderer 

class ViewpointProxy
  : public BindableProxy
{
public:

  /// Constructor
  ViewpointProxy(Viewpoint* pViewpoint);

  // SB 99/05/14 added for testing
  virtual void DumpTrace()
  {
    Viewpoint * poNode = (Viewpoint *) GetMediaObject();
    poNode->GetPresenter()->Logf("RendViewpoint %08x \n", this);
  }

  /// Render object description
  virtual void		RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD dwTime);

  // Stefano Battista
  // moved methods from Viewpoint to ViewpointProxy
private:
  Vec3f m_transformedBboxCenter;
  Vec3f m_transformedBboxSize;
public:
  /// Get pointer to related Viewpoint object
  inline Viewpoint * GetViewpoint() const
  { return (Viewpoint *) GetMediaObject(); }

  /// Set root of object
  /* virtual */ void SetRoot(Root* pRoot);
  //ML - moved body to ViewpointProxy.cpp

  /// set field of view
  inline void SetFieldOfView(float fFieldOfView)
  { GetViewpoint()->fieldOfView = fFieldOfView; }
  /// get field of view
  inline float GetFieldOfView() const
  { return GetViewpoint()->fieldOfView; }
  /// set orientation of viewpoint
  inline void SetOrientation(const SF<Rotation>& rOrientation)
  { if (!((Rotation) (GetViewpoint()->orientation) == rOrientation)) {
      GetViewpoint()->orientation = rOrientation;  
      SetBbox(Vec3f(1.0f, 1.0f, 1.0f), Vec3f()); } }
  /// get orientation of viewpoint
  inline const SF<Rotation>& GetOrientation() const
  { return GetViewpoint()->orientation; }
  /// set position of viewpoint
  inline void SetPosition(const SF<Vec3f>& rPosition)
  { if (!((Vec3f) (GetViewpoint()->position) == rPosition)) {
      GetViewpoint()->position = rPosition;
      SetBbox(Vec3f(1.0f, 1.0f, 1.0f), Vec3f()); } }
  /// get position of viewpoint
  inline const SF<Vec3f>& GetPosition() const
  { return GetViewpoint()->position; }
  /// Get bounding box
  inline /* virtual */ void GetBbox(Vec3f& rBboxSize, Vec3f& rBboxCenter) const
  { rBboxSize = Vec3f(1.0f, 1.0f, 1.0f);
    rBboxCenter = Vec3f(); }
  /// Get bounding box
  inline /* virtual */ void GetTransformedBbox(Vec3f& rBboxSize, Vec3f& rBboxCenter) const
  { rBboxSize = Vec3f(1.0f, 1.0f, 1.0f);
    rBboxCenter = GetPosition(); }

  /// Concatenate transformation with given matrix
  void ConcatenateTransform(Matrix& rTransform) const;
  /// Set bounding box and update parents
  void SetBbox(const Vec3f& rBboxSize, const Vec3f& rBboxCenter);
  /// Collision detection
  MediaObject * DetectCollision();
  /// Concatenate inverse transformation with given matrix
  void ConcatenateInverseTransform(Matrix& rTransform) const;
  /// Compute Viewpoint to World transform
  virtual void ViewpointToWorldMatrix(
    DWORD dwTime, Matrix& rTransform) const;

};

//ML - removed ViewpointProxy.i

#endif // __VIEWPOINTPROXY_H__
