/***************************************************************************/
/*                                                                         */
/* BodyProxy Class Interface											   */
/*                                                                         */
/***************************************************************************/
/*                                                                         */
/* Purpose : The BodyProxy is an the interpreter of the Body Node. It is   */
/* in charge of managing the main body functions concerning Body Animation */
/*                                                                         */
/* Version : 1.02                                                          */
/*                                                                         */
/* Latest Change : 10/08/99                                                */
/*                                                                         */
/* Authors : Fabrice Vergnenegre, Tolga Capin, LIG - EPFL                  */
/*                                                                         */
/***************************************************************************/
/* 10/08/99 : Added the CreateChildren member function to the Grouping     */
/*            class to avoid the assert error when adding a child...       */
/***************************************************************************/

#ifndef __BODYPROXY_H__
#define __BODYPROXY_H__

#include "defs.h"
#include "opengl/base/Node3DProxy.h"
#include "Nodes.h"

class OpenGLPresenter;
class BAPProxy;
class BDPProxy;

#include "opengl/base/OpenGLVisualRenderer.h"
#include "opengl/base/Matrix.h"

/***************************************************************************/
class BodyProxy : public Node3DProxy {

// ttributes
public :
	BOOL bodyIsDefinedInBDP;
	BOOL firstUpdatePassed;
	BAPProxy *poBapProxy;
	BDPProxy *poBdpProxy;

// Constructor(s)
public :
	BodyProxy(Body* pBody);

// Overloaded Members
public :

	virtual void DumpTrace()
		{
		Body * poNode = (Body *) GetMediaObject();
		poNode->GetPresenter()->Logf("RendBody %08x \n", this);
		}
	
	virtual BOOL RenderLights(	OpenGLVisualRenderer* pRenderer, 
								DWORD dwTime);

	virtual void RenderDescription(	OpenGLVisualRenderer* pRenderer, 
									DWORD dwTime);

	virtual void Select(OpenGLVisualRenderer* pRenderer, 
						DWORD dwTime,
						MediaObjectProxy*& intersectingObject, 
						unsigned int& depth);

// Accessers
public:

	inline BOOL GetBodyIsDefinedInBDP() const { return bodyIsDefinedInBDP; }

	inline void SetBodyIsDefinedInBDP(BOOL i) { bodyIsDefinedInBDP=i; }

	inline Body* GetBody() const { return (Body*) GetMediaObject(); }

	inline BDP * GetBDP() const { return GetBody()->bdp; }

	inline BAP * GetBAP() const { return GetBody()->bap; }

	inline MFNodeType<SF3DNode>& GetRenderedBody() const { return GetBody()->renderedBody; }

	inline MediaObject * GetNextSibling() {
		Body * poBody = GetBody();
		MFNodeType<SF3DNode>& roRenderedBody = GetRenderedBody();
		int iCurrentElement = poBody->GetCurrentElement();

		if (roRenderedBody.GetLength() <= iCurrentElement)
			return NULL;
		else {
			MediaObject * poObject = (MediaObject *) roRenderedBody[iCurrentElement];
			poBody->SetCurrentElement(++iCurrentElement);
			return poObject;
		}
	}

	inline MediaObject * GetFirstChild() {
		GetBody()->SetCurrentElement(0);
		return GetNextSibling();
	}

	inline int GetChildCount() const {
		return GetRenderedBody().GetLength(); 
	}

protected :
	virtual BOOL Render(const Matrix& rTransformation, 
						const CullFlag& rCullFlag,
						OpenGLVisualRenderer* pRenderer, 
						DWORD dwTime);

	virtual void Select(const Matrix& rTransformation, 
						const CullFlag& rCullFlag,
						OpenGLVisualRenderer* pRenderer, 
						DWORD dwTime,
						MediaObjectProxy*& intersectingObject,
						ZArray<MediaObjectProxy*>& activatedSensors, 
						unsigned int& depth,
						const ZArray<MediaObjectProxy*>& localSensors);

	BOOL Update(VisualRenderer* pRenderer, 
				DWORD dwTime);

	MediaObject * AddChild(MediaObject * pObjectToAdd, MediaObject * where);

	void UpdateBbox();

	void ConstructBody();
};

#endif
