#ifndef _HEADER_OF_CLASS_CONVERSION_TABLE_
#define _HEADER_OF_CLASS_CONVERSION_TABLE_

/************************************************************/
/* INTERFACE                                                */
/************************************************************/

// INCLUDES
#include <stdio.h>
#include <stdlib.h>

// DEFINES
#define MAX_ELEMENTS_IN_TABLE 100
#define BASE_INDEX 201

// CLASSES
class ConversionTable {

  // Private attributes
  char ** table;
  int numElements;

  // Constructors and Destructors
 public:
  ConversionTable(char * filename=NULL);
  ~ConversionTable();

  // Accessers
 public:
  inline char ** GetTable() const { return table; }
  inline char * GetTable(int i) const { return table[i]; }
  inline int GetNumElements() const { return numElements; }

  // Operators
 public:
  inline char * operator[] (int n) const { return table[n]; }

  // Modifiers
 public:

  // Members
  int FindIdFromName(char * name);
  void LoadTableFromFile(char * filename);
  void PrintTable();
};


/************************************************************/
/* IMPLEMENTATION                                           */
/************************************************************/

// Constructor -----------------------------------------------
ConversionTable::ConversionTable(char * filename)
  : table(NULL),
    numElements(0)
{
  // If the table of conversion is specified, load it
  if (filename!=NULL)
    LoadTableFromFile(filename);
}

// Destructor ------------------------------------------------
ConversionTable::~ConversionTable()
{
  for (int i=0 ; i<numElements ; i++)
    delete table[i];
  delete table;
}

// LoadTableFromFile Member ----------------------------------
void ConversionTable::LoadTableFromFile(char * filename)
{
  // local buffer
  char buffer[40];

  // open the file
  FILE * f = fopen(filename,"rt");

  // initialize the index and table
  numElements=0;
  table = new char*[MAX_ELEMENTS_IN_TABLE];

  // reads the file
  while (!feof(f)) {
    fscanf(f,"%s\n",buffer);
    table[numElements]=new char[strlen(buffer)+1];
    strcpy(table[numElements],buffer);
    numElements++;
  }

  // closes the file
  fclose(f);
}

// PrintTable ------------------------------------------------
void ConversionTable::PrintTable()
{
  for (int i=0 ; i<numElements ; i++)
    printf("[%2.2d] <%s>\n",i,table[i]);
}

// FindIdFromName Member -------------------------------------
int ConversionTable::FindIdFromName(char * name)
{
  int found=-1;

  // parse the table
  for (int i=0 ; i<numElements ; i++)
    if (!strcmp(name,table[i])) {
      found=i;
      break;
    }

  // found ?
  if (found==-1)
    return -1;

  return BASE_INDEX+found;
}

#endif
