// This software module was originally developed by Karl Anders ygard
// (Telenor R&D) in the course of development of the MPEG-4 Systems
// standard (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// Copyright 1997, 1998.

// SB @@@@ I need to add this because it defines
// SFNode<FaceDefTables>::CreateNode(int)
// otherwise LINKER is unhappy
#include "OpenGl/OpenGl.h"
//#include "ztl.h"

//#include "Nodes.h"
//#include "CrNodes.h"

#include "FDPProxy.h"

// constructor

FDPProxy::FDPProxy(FDP* pFDP)
  : MediaObjectProxy(pFDP)
{
#ifndef BIFSENC
  FirstTime = TRUE;

  for (int i = 0; i < 68; i++)
    Previousfapvalues[i] = 0;

  modelNumTransformInfos = 0;
  modelNumIndexedFaceSetInfos = 0;
#endif
}

// destructor

FDPProxy::~FDPProxy()
{
  int i;

  for (i = GetFDP()->GetParentsCount(); i > 0;)
    //ML - replaced this line with the following lines
    //((Face *) GetFDP()->GetParent(--i))->fdp = NULL;
  {
    Face* pFace = (Face*)GetFDP()->GetParent(--i);
    pFace->fdp.SetObject(NULL, pFace);
  }

  for (i = GetFDP()->faceDefTables.GetLength(); i > 0;)
    GetFDP()->faceDefTables[--i]->
      RemoveParent(GetFDP());

  for (i = GetFDP()->faceSceneGraph.GetLength(); i > 0;)
    ((MediaObject *) GetFDP()->faceSceneGraph[--i])->
      RemoveParent(GetFDP());

  // SB hack from Gabriel???
  // FirstTime = FALSE;
}


// Render object description

/* virtual */ void
FDPProxy::RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD /* dwTime */)
{
  RenderDescriptionBox(pRenderer, "FDP");
}


// Update object

BOOL FDPProxy::Update(Face* pFace, DWORD /* dwTime */)
{
#ifndef BIFSENC
  FaceProxy * poFaceProxy = (FaceProxy *) (pFace->GetRenderingObject());
  FAPProxy * poFAPProxy = (FAPProxy *) (poFaceProxy->GetFAP()->GetRenderingObject());

  int fapvalues[68];

  if (FirstTime) {
    calcFAPUsModel();
    makeFAPTable();
    pFace->renderedFace = faceSceneGraph;
  }

  for (int i = 0; i < 68; i++) {
    fapvalues[i] = poFAPProxy->GetFAPValue(i);

    if (fapvalues[i] != Previousfapvalues[i]) {
		if(fapvalues[i] > -MAXVALUE && fapvalues[i] < MAXVALUE) {
			calcValuesFapTables(pFace, i, fapvalues[i]);
		}
		Previousfapvalues[i] = fapvalues[i];
    }
  }

  for (int infoIndex = 0; infoIndex < modelNumIndexedFaceSetInfos; infoIndex++) {
    modelIndexedFaceSetInfos[infoIndex].setLocalVerticesValues();
    modelIndexedFaceSetInfos[infoIndex].initUpdateMask();
  }

#endif

  return TRUE;
}


// SB @@@@ all the following stuff moved from FDP to FDPProxy

#ifndef BIFSENC

// Add FaceDefTables to this node

inline void
FDPProxy::AddFaceDefTables(FaceDefTables* pObject)
{
  //dn
	//GetFDP()->faceDefTables += pObject;
	GetFDP()->faceDefTables.Append(pObject,GetFDP());
}

// Remove FaceDefTables from this node

//dn inline BOOL
//FDPProxy::RemoveFaceDefTables(FaceDefTables* pObject)
//{
  // GetFDP()->faceDefTables -= pObject;
// return TRUE;
//}

// Process the FAPUs

void FDPProxy::calcFAPUsModel()
{
  FDP * poFDP = GetFDP();

  if ((Coordinate *) GetFeaturePointsCoord() != NULL)
  {
    MF<Vec3f> fp = (poFDP->featurePointsCoord->point /* GetPoint() */ );
    if (fp.GetLength() > 0)
    {
			Vec3f fp_2_2 = fp[1];
			Vec3f fp_2_3 = fp[2];
			Vec3f fp_3_1 = fp[14];
			Vec3f fp_3_3 = fp[16];
			Vec3f fp_3_5 = fp[18];
			Vec3f fp_3_6 = fp[19];
			Vec3f fp_8_3 = fp[45];
			Vec3f fp_8_4 = fp[46];
			Vec3f fp_9_4 = fp[56];

			ES		= (fp_3_5.x - fp_3_6.x)/1024;
			ENS		= (0.5 * (fp_3_5.y + fp_3_6.y) - fp_9_4.y)/1024;
			MNS		= (fp_9_4.y - 0.5 * (fp_2_2.y + fp_2_3.y))/1024;
			MW		= (fp_8_3.x - fp_8_4.x)/1024;
			IRISD	= (fp_3_1.y - fp_3_3.y)/1024;
			AU		= 1e-5f;

			int i;

			for (i=0;i<2;i++)	fapFAPU[i] = 1;
			for (i=2;i<5;i++)	fapFAPU[i] = MNS;
			for (i=5;i<7;i++)	fapFAPU[i] = MW;
			for (i=7;i<14;i++)	fapFAPU[i] = MNS;
			fapFAPU[14] = MNS;
			for (i=15;i<18;i++)	fapFAPU[i] = MNS;
			for (i=18;i<22;i++)	fapFAPU[i] = IRISD;
			for (i=22;i<26;i++) fapFAPU[i] = AU;
			for (i=26;i<30;i++) fapFAPU[i] = IRISD;
			for (i=30;i<36;i++) fapFAPU[i] = ENS;
			for (i=36;i<40;i++) fapFAPU[i] = ES;
			for (i=40;i<42;i++) fapFAPU[i] = ENS;
			fapFAPU[42] = MW;
			fapFAPU[43] = MNS;
			fapFAPU[44] = MW;
			fapFAPU[45] = MNS;
			for (i=46;i<50;i++) fapFAPU[i] = AU;
			for (i=50;i<52;i++) fapFAPU[i] = MNS;
			for (i=52;i<54;i++) fapFAPU[i] = MW;
			for (i=54;i<60;i++) fapFAPU[i] = MNS;
			for (i=60;i<68;i++) fapFAPU[i] = ENS;
    }
  }
}


void FDPProxy::makeFAPTable()
{
  int nbFaceDefTables = GetLengthFaceDefTables();
  for (int i=0;i<nbFaceDefTables;i++)
  {
    FaceDefTables * currentFaceDefTables = (FaceDefTables*) GetFaceDefTables(i);
    int fapID = currentFaceDefTables->fapID;
    // Compute for the FaceDefTransform
    int nbFaceDefTransform = faceDefTables[i]->faceDefTransform.GetLength(); /* GetLengthFaceDefTransform() */
    int indexTransformTables = 0;
    int j;
    for(j=0; j<nbFaceDefTransform; j++)
    {
      FaceDefTransform *currentFaceDefTransform;
      currentFaceDefTransform = (FaceDefTransform*)(currentFaceDefTables->faceDefTransform[j]); /* GetFaceDefTransform(j) */
      // SB @@@@ faceScene... has changed semantics 99/01/07
      int nodeId = ((MediaObject *) (currentFaceDefTransform->faceSceneGraphNode))->GetID(); /* GetFaceSceneGraphNode() */
      int fieldID = currentFaceDefTransform->fieldId; /* GetFieldId() */
      TransformSTable transSTable;
      transSTable.setName(nodeId);
      printf("table for node %d \n",nodeId);
      transSTable.setFieldRef(fieldID);
      switch(fieldID){    
      case 1:	{SF<Rotation> rotation=currentFaceDefTransform->rotationDef; /* GetRotationDef() */
        transSTable.setRotation(rotation);
        break;}
      case 2:	{SF<Vec3f> scale=currentFaceDefTransform->scaleDef; /* GetScaleDef() */
        transSTable.setTranslationOrScalefactor(scale);
        break;}
      case 3:	{SF<Vec3f> translation=currentFaceDefTransform->translationDef; /* GetTranslationDef() */
        transSTable.setTranslationOrScalefactor(translation);
        break;}
      }
      FapTables[fapID-1].set1Value(indexTransformTables, transSTable);
      indexTransformTables++;
    }
    // Compute for the FaceDefMesh
    int nbFaceDefMesh = faceDefTables[i]->faceDefMesh.GetLength(); /* GetLengthFaceDefMesh() */
    int indexFaceSetTables = 0;
    for(j=0; j<nbFaceDefMesh; j++)
    {
      FaceDefMesh *currentFaceDefMesh;
      currentFaceDefMesh = (FaceDefMesh*)(currentFaceDefTables->faceDefMesh[j]); /* GetFaceDefMesh(j) */
      // SB @@@@ faceScene... has changed semantics 99/01/07
      int nodeId = ((MediaObject *) (currentFaceDefMesh->faceSceneGraphNode))->GetID(); /* GetfaceSceneGraphNode() */
      int nVertices = currentFaceDefMesh->coordIndex.GetLength(); /* GetcoordIndex() */
      int nIntervalBorders= currentFaceDefMesh->intervalBorders.GetLength(); /* GetintervalBorders() */
      int *intervalBorders = new int[nIntervalBorders];
      int k;
      for(k=0;k<nIntervalBorders;k++){
        intervalBorders[k]=currentFaceDefMesh->intervalBorders[k]; /* GetintervalBorders() */
      }
      IndexedFaceSetSTable faceSetSTable(nVertices, nIntervalBorders-1, intervalBorders);
      faceSetSTable.setName(nodeId);
      for(k=0; k<nVertices; k++){
        VertexDisplacementsInfo displacementsInfo;
        int vertexIndex;
        vertexIndex=currentFaceDefMesh->coordIndex[k]; /* GetcoordIndex() */
        MF<Vec3f> displacements;
        for(int m=0;m<nIntervalBorders-1; m++){
          displacements.InsertAt(m,currentFaceDefMesh->displacements[(nIntervalBorders-1)*k+m]); /* Getdisplacements() */
        }
        displacementsInfo.setValues(vertexIndex, nIntervalBorders-1, displacements);
        faceSetSTable.set1VertexDisplacements(k, displacementsInfo);
      }
      FapTables[fapID-1].set1Value(indexFaceSetTables, faceSetSTable);
      indexFaceSetTables++;
    }
  }
  FirstTime=FALSE;
}

void FDPProxy::calcValuesFapTables(Face* pFace, int FAPId, int fap_value)
{
  FaceProxy * poFaceProxy = (FaceProxy *) pFace->GetRenderingObject();

  if(FapTables[FAPId].getNumNodes()>0){
    int numTransformTables = FapTables[FAPId].getNumTransformTables();
    int i;
    // Update of the Transforms Nodes
    for(i=0; i < numTransformTables; i++)
    {
		    const TransformSTable *transformTable = FapTables[FAPId].getTransformSTable(i);
        const int transformTableName = transformTable->getName();
        int infoExists = FALSE;
        int infoIndex = 0;
        for(; infoIndex<modelNumTransformInfos; infoIndex++)
        {
          if(modelTransformInfos[infoIndex].getName()==transformTableName){
            infoExists = TRUE;
            break;
          }
        }
        if(infoExists == FALSE){
          modelNumTransformInfos++;
          Transform *modelTransformNode = 
            (Transform*) (poFaceProxy->FindNode(transformTableName));
          SF<Vec3f> scaletest = (Vec3f(0, 0, 0));
          if(!((Vec3f) (modelTransformNode->scale) /* GetScale() */ == (Vec3f) scaletest)){ //test of existance
            modelTransformInfos[infoIndex].setNode(modelTransformNode);
            modelTransformInfos[infoIndex].setNeutralValues(transformTable);
            modelTransformInfos[infoIndex].setName(transformTableName);
          } 
          else{
            printf("FDP::calcValuesFapTables: Transform node %d does not exit in downloaded model\n", transformTableName);
          }
        }
        modelTransformInfos[infoIndex].setTransformValue(fap_value*fapFAPU[FAPId]);
    } 
    
    int numIndexedFaceSetTables = FapTables[FAPId].getNumIndexedFaceSetTables();
    for(i=0; i<numIndexedFaceSetTables; i++){
      // Update of the IndexedFaceSet Nodes
      const IndexedFaceSetSTable *faceSetTable = FapTables[FAPId].getIndexedFaceSetSTable(i);
      const int faceSetTableName = faceSetTable->getName(); 
      int infoExists = FALSE;
      int infoIndex = 0;
      for(; infoIndex<modelNumIndexedFaceSetInfos; infoIndex++){
        if(modelIndexedFaceSetInfos[infoIndex].getName()==faceSetTableName){
          infoExists = TRUE;
          break;
        }
      }
      if(infoExists == FALSE){
        modelNumIndexedFaceSetInfos++;
        IndexedFaceSet *modelIndexedFaceSetNode = 
          (IndexedFaceSet*)(poFaceProxy->FindNode(faceSetTableName)); 
        Coordinate *modelCoordNode = modelIndexedFaceSetNode->coord /* GetCoord() */ ;
        if(modelCoordNode->point /* GetPoint() */ .GetLength()!=0){ //test of existance
          modelIndexedFaceSetInfos[infoIndex].setNode(modelCoordNode);
          modelIndexedFaceSetInfos[infoIndex].setNeutralValues();
          modelIndexedFaceSetInfos[infoIndex].initLocalVertexPositions();
          modelIndexedFaceSetInfos[infoIndex].makeUpdateMask();
          modelIndexedFaceSetInfos[infoIndex].setName(faceSetTableName);
        } 
        else{
          printf("Face::calcValuesFapTables: Coordinate node %d does not exit in downloaded model\n", faceSetTableName);
        }
      }
      
      int numVertices = faceSetTable->getNumVertices();
      int numIntervals = faceSetTable->getNumIntervals();
      const int *intervalBorders = faceSetTable->getIntervalBorders();
      
      int zeroIndexInterval = 0;
      int fapIndexInterval = 0;
      
      if(fap_value!=0){
        int j;
        
        if(fap_value>=0){
          fapIndexInterval=numIntervals-1;
        }
        
        for(j=0;j<numIntervals+1; j++){
          if(intervalBorders[j]==0){
            zeroIndexInterval=j;
          }
        }
        for(j=0;j<numIntervals; j++){
          int indexIntervalTest = (fap_value>=intervalBorders[j])&&(fap_value<intervalBorders[j+1]);
          if(indexIntervalTest){
            fapIndexInterval=j;
          }
        }
      }
      
      for(int j=0; j<numVertices; j++){
        Vec3f displacement(0, 0, 0);
        const VertexDisplacementsInfo *displacementsInfo = faceSetTable->getValue(j); 
        if(fap_value==0){
          modelIndexedFaceSetInfos[infoIndex].addVertexDisplacement(displacementsInfo->getVertexIndex(), displacement);
        } else {
          if(fapIndexInterval>=zeroIndexInterval){
            int Ik;
            for(Ik=zeroIndexInterval; Ik<fapIndexInterval; Ik++){
              displacement.x += (intervalBorders[Ik+1]-intervalBorders[Ik])*displacementsInfo->getValues(Ik).x;
              displacement.y += (intervalBorders[Ik+1]-intervalBorders[Ik])*displacementsInfo->getValues(Ik).y;
              displacement.y += (intervalBorders[Ik+1]-intervalBorders[Ik])*displacementsInfo->getValues(Ik).z;
            }
            displacement.x += (fap_value-intervalBorders[Ik])*displacementsInfo->getValues(Ik).x;
            displacement.y += (fap_value-intervalBorders[Ik])*displacementsInfo->getValues(Ik).y;
            displacement.z += (fap_value-intervalBorders[Ik])*displacementsInfo->getValues(Ik).z;
          }else{
            int Ik;
            for(Ik=zeroIndexInterval; Ik>fapIndexInterval+1; Ik--){
              displacement+=(intervalBorders[Ik]-intervalBorders[Ik-1])*displacementsInfo->getValues(Ik-1)[0];
            }
            displacement.x +=(intervalBorders[Ik]-fap_value)*displacementsInfo->getValues(Ik-1).x;
            displacement.y +=(intervalBorders[Ik]-fap_value)*displacementsInfo->getValues(Ik-1).y;
            displacement.z +=(intervalBorders[Ik]-fap_value)*displacementsInfo->getValues(Ik-1).z;
          }
          displacement.x *= fapFAPU[FAPId];
          displacement.y *= fapFAPU[FAPId];
          displacement.z *= fapFAPU[FAPId];
          modelIndexedFaceSetInfos[infoIndex].addVertexDisplacement(displacementsInfo->getVertexIndex(), displacement);
        }
        
        
      }
    }
  }
  
}

// Gabriel & Joern
// Look in the branch under this node for an object with an ID of nID

/* virtual */ MediaObject *
FDPProxy::FindNode(int nID)
{
  MediaObject * pResult = GetFDP()->GetRoot()->FindNode(nID);

  for (int i = GetFDP()->faceSceneGraph.GetLength() - 1; i >= 0 && !pResult; --i)
    pResult = GetFDP()->faceSceneGraph[i]->
      GetRoot()->FindNode(nID);

  return pResult;
}

#endif // ! BIFSENC

