// This software module was originally developed by Karl Anders ygard
// (Telenor R&D) in the course of development of the MPEG-4 Systems
// standard (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// Copyright 1997.

#ifndef __FDPPROXY_H__
#define __FDPPROXY_H__

//#include "defs.h"
//#include "Nodes.h"

#include "opengl/base/MediaObjectProxy.h"
#include "opengl/fba/FaceProxy.h"
#include "opengl/fba/FAPProxy.h"

#include "fba/PartInfo_mpeg.h"

// SB @@@@ FaceDefTables is now in Nodes.h
// #include "fba/FaceDefTables.h"

#define MAXVALUE 10000

class OpenGLPresenter;
class OpenGLVisualRenderer;
class NodeProxy;

/// FDP renderer node

class FDPProxy
  : public MediaObjectProxy
{
public:
  /// Constructor
  FDPProxy(FDP* pFDP);

  /// Destructor
  ~FDPProxy();

  /// Render object description
  virtual void RenderDescription(OpenGLVisualRenderer * pRenderer, DWORD dwTime);

  // SB @@@@ moved from FDP to FDPProxy >>>>
public:
  /// Get pointer to associated Node
  inline FDP * GetFDP()
  { return (FDP *) GetMediaObject(); }
  inline SFNode<class Coordinate> & GetFeaturePointsCoord()
  { return GetFDP()->featurePointsCoord; }
  // Get the number of FaceDefTables
  inline int GetLengthFaceDefTables()
  { return faceDefTables.GetLength(); }
  // Get FaceDefTables node
  inline MFNode<FaceDefTables> & GetFaceDefTables() 
  { return faceDefTables; }
  // Get the ith FaceDefTable node


  //dn
  inline MediaObject * GetFaceDefTables(int i)
 // { return ((MF<MediaObject*>)faceDefTables)[i]; }
{ return (faceDefTables)[i]; }

  //  { return ((MField<MediaObject*>)faceDefTables)[i]; }

  /// Add FaceDefTables to this node
  void			AddFaceDefTables(FaceDefTables* pObject);
  /// Remove FaceDefTables from this node
  BOOL			RemoveFaceDefTables(FaceDefTables* pObject);
  // SB @@@@ moved from FDP to FDPProxy <<<<

private:
  int                   FirstTime;
  float                 Previousfapvalues[68];
  /// The FAPUs
  float                 ES, ENS, MNS, MW, IRISD, AU;
  /// The FAPU for each FAP
  float                 fapFAPU[68];
  /// Number of initialized TransformInfos in the array modelTransformInfos
  int                   modelNumTransformInfos;
  /// Array of TransformInfos; used for animation with FAP sequences;
  /// initialized in calcValuesFapTables()
  TransformInfo         modelTransformInfos[1024];
  /// Number of initialized IndexedFaceSetInfos in the array
  /// modelTransformInfos
  int                   modelNumIndexedFaceSetInfos;
  /// Array of IndexedFaceSetInfos; used for animation with FAP sequences;
  /// initialized in calcValuesFapTables()
  IndexedFaceSetInfo    modelIndexedFaceSetInfos[1024];
  /** Animation-definition tables.
  If a face model is downloaded, the behaviour of FAPs is defined in this field. */
  MFNode<FaceDefTables> faceDefTables;
  /// Rendered Face
  MFNodeType<SF3DNode>	faceSceneGraph;
  /// Array for the ADTs of all FAPs
  AnimationDefinitionTable FapTables[68]; 

public:
  /// Update object
  BOOL Update(Face* pFace, DWORD dwTime);

  // Joern & Gabriel
  /// Find object which expects stream with object descriptor id nObID
  virtual MediaObject * FindNode(int nID);

  /// Process the FAPUs
  void calcFAPUsModel();
  void makeFAPTable();
  void calcValuesFapTables(Face* pFace, int FAPId, int fap_value);
};

#include "FDPProxy.i"

#endif // __FDPPROXY_H__
