// Stefano Battista
// 1998 November 25
#include "OpenGl/OpenGl.h"
//#include "ztl.h"
#include "FITProxy.h"

#ifdef WIN32

// C style function pretending to do Random Integer (SB)
inline rint(double dValue)
{
  double dFloor = floor(dValue);

  if (dValue - dFloor > 0.5)
	return ceil(dValue);
  else
	return dFloor;
}

#endif

/// Constructor
/// Calls MediaObjectProxy constructor
FITProxy::FITProxy(FIT * poFIT) :
MediaObjectProxy(poFIT)
{};

/// Update object
BOOL FITProxy::Update(Face* pFace, DWORD /* dwTime */)
{
#ifndef BIFSENC

  FaceProxy * poFaceProxy = (FaceProxy *) pFace->GetRenderingObject();
  
  int i, j, k, l, m, n, index;
  int interpolation_count = 0;
  FAP * pFAP = pFace->fap /* GetFAP() */ ;
  Viseme * pViseme = (pFAP != NULL) ? ((Viseme *) pFAP->viseme) /* GetViseme() */ : (Viseme *) NULL;
  Expression * pExpression = (pFAP != NULL) ? ((Expression *) pFAP->expression) /* GetExpression() */ : (Expression *) NULL;
  
  if (FirstTime)
  {
    int nTermIndex1 = 0, nTermIndex2 = 0;
    int CoefIndex1 = 0, CoefIndex2 = 0;
    int ImpulseIndex = 0;
    int ExpoIndex1 = 0, ExpoIndex2 = 0;
    PPOLY pnumerator, pdenominator;
    PTERM pterm;
    
    /* Set attributes for each FAP sets */
    m_nFAPSets = 0;
    n = GetFIT()->FAPs.GetLength();
    for (i=0; i<n; i++) {
      if (GetFIT()->FAPs[i] == -1) m_nFAPSets++;
    }
    m_pIndex = (int *) malloc ((m_nFAPSets+1)*sizeof(int));
    m_pbComplete = (BOOL *) malloc (m_nFAPSets*sizeof(BOOL));
    m_pIndex[0] = 0;
    for (index=0, i=0; i<n; i++) {
      if (GetFIT()->FAPs[i] == -1) {
        m_pIndex[++index] = i+1;
      }
    }
    
    /* Organize mapping parameters */
    n = GetFIT()->Graph.GetLength();
    m_nLinks = n/2;
    LinkFuncs = (PFUNC) malloc(m_nLinks*sizeof(FUNC));
    for (l=0, i=0; i<n; i+=2, l++) {
      int from = GetFIT()->Graph[i];
      int to = GetFIT()->Graph[i+1];
      int nParents = m_pIndex[from] - m_pIndex[from-1] - 1;
      int nChildren = m_pIndex[to] - m_pIndex[to-1] - 1;
      BOOL hasDelta = FALSE;
      LinkFuncs[l].nParents = nParents;
      LinkFuncs[l].nChildren = nChildren;
      for (j=m_pIndex[from-1]; ; j++) {
        index = GetFIT()->FAPs[j];
        if (index==-1) break;
        if (index==69 || index==70 || index==71 || 
          index==72 || index==73 || index==74 || index==75) {
          hasDelta = TRUE;
          break;
        }
      }
      LinkFuncs[l].numerator = (PPOLY) malloc(nChildren*sizeof(POLY));
      LinkFuncs[l].denominator = (PPOLY) malloc(nChildren*sizeof(POLY));
      for (j=0; j<nChildren; j++) {
        pnumerator = &LinkFuncs[l].numerator[j];
        pdenominator = &LinkFuncs[l].denominator[j];
        int nTerms1 = GetFIT()->numeratorTerms[nTermIndex1++];
        int nTerms2 = GetFIT()->denominatorTerms[nTermIndex2++];
        pnumerator->nTerms = nTerms1;
        pdenominator->nTerms = nTerms2;
        pnumerator->terms = (PTERM) malloc(nTerms1*sizeof(TERM));
        pdenominator->terms = (PTERM) malloc(nTerms2*sizeof(TERM));
        for (k=0; k<nTerms1; k++) {
          pterm = &pnumerator->terms[k];
          pterm->coef = GetFIT()->numeratorCoefs[CoefIndex1++];
          pterm->delta = TRUE;
          if (hasDelta) {
            index = GetFIT()->numeratorImpulse[ImpulseIndex++];
            if (index != GetFIT()->numeratorImpulse[ImpulseIndex++]) {
              pterm->delta = FALSE;
            }
          }
          pterm->expos = (int*) malloc(nParents*sizeof(int));
          for (m=0; m<nParents; m++) {
            pterm->expos[m] = GetFIT()->numeratorExp[ExpoIndex1++];
          }
        }
        for (k=0; k<nTerms2; k++) {
          pterm = &pdenominator->terms[k];
          pterm->coef = GetFIT()->denominatorCoefs[CoefIndex2++];
          pterm->delta = TRUE;
          pterm->expos = (int*) malloc(nParents*sizeof(int));
          for (m=0; m<nParents; m++) {
            pterm->expos[m] = GetFIT()->denominatorExp[ExpoIndex2++];
          }
        }
      }
    }
    FirstTime = FALSE;
  }
  
  /* get FAP values and FAP masks */
#ifdef ML_CANNOT_FIX
  FAPMasks[0] = FAPMasks[68] = FAPMasks[69] = FAPMasks[70]
    = poFaceProxy->GetFAP()->FAPMask[0];
  FAPMasks[1] = FAPMasks[71] = FAPMasks[72] = FAPMasks[73] = FAPMasks[74] 
    = poFaceProxy->GetFAP()->FAPMask[1];
  if (FAPMasks[0] && pViseme) { // use visemes
    FAPValues[68] = pViseme->viseme_select1; /* GetViseme_select1() */
    FAPValues[69] = pViseme->viseme_select2; /* GetViseme_select2() */
    FAPValues[70] = pViseme->viseme_blend; /* GetViseme_blend() */
  }
  if (FAPMasks[1] && pExpression) { // use expressions
    FAPValues[71] = pExpression->expression_select1; /* GetExpression_select1() */
    FAPValues[72] = pExpression->expression_select2; /* GetExpression_select2() */
    FAPValues[73] = pExpression->expression_intensity1; /* GetExpression_intensity1() */
    FAPValues[74] = pExpression->expression_intensity2; /* GetExpression_intensity2() */
  }
  for (i=2; i<68; i++) {
    FAPMasks[i] = poFaceProxy->GetFAP()->FAPMask[i];
    if (FAPMasks[i]) {
      FAPValues[i] = poFaceProxy->GetFAP()->FAPValues[i];
    }
  }
#endif
  
  for (i=0; i<m_nFAPSets; i++) {
    m_pbComplete[i] = TRUE;
    for (j=m_pIndex[i]; j<m_pIndex[i+1]-1; j++) {
      index = GetFIT()->FAPs[j]-1;
      if (FAPMasks[index] == FALSE) {
        m_pbComplete[i] = FALSE;
        break;
      }
    }
  }
  
  do {
    interpolation_count = 0;
    for (i=0; i<m_nFAPSets; i++) {
      if (m_pbComplete[i] == TRUE) continue;
      m_pbComplete[i] = TRUE;
      for (j=m_pIndex[i]; j<m_pIndex[i+1]-1; j++) {
        index = GetFIT()->FAPs[j]-1;
        if (FAPMasks[index] == FALSE) {
          m_pbComplete[i] = FALSE;
          break;
        }
      }
      if (m_pbComplete[i] == TRUE) continue;
      for (l=0; l<m_nLinks; l++) {
        if (GetFIT()->Graph[2*l+1]-1 == i && m_pbComplete[GetFIT()->Graph[2*l]-1] == TRUE) {
          Interpolate(l);
          interpolation_count++;
          m_pbComplete[i] = TRUE;
          for (j=m_pIndex[i]; j<m_pIndex[i+1]-1;j++) {
            index = GetFIT()->FAPs[j]-1;
            if (FAPMasks[index] == FALSE) {
              FAPMasks[index] = TRUE;
#ifdef ML_CANNOT_FIX
              poFaceProxy->GetFAP()->FAPValues[index] = FAPValues[index];
#endif
            }
          }
          break;
        }
      }
    }
  } while (interpolation_count != 0);
  
#endif // ! BIFSENC
  
  return TRUE;
}


void FITProxy::Interpolate(int l)
{
#ifndef BIFSENC

  int i, j, k, m, n, index;
  int nTerms;
  int ParentsStart, ParentsEnd;
  int ChildrenStart, ChildrenEnd;
  double termValue;
  double numeratorValue, denominatorValue;
  PPOLY pnumerator, pdenominator;
  PTERM pterm;

  ParentsStart = m_pIndex [GetFIT()->Graph[2*l]-1];
  ParentsEnd = m_pIndex [GetFIT()->Graph[2*l]] - 1;
  ChildrenStart = m_pIndex [GetFIT()->Graph[2*l+1]-1];
  ChildrenEnd = m_pIndex [GetFIT()->Graph[2*l+1]] - 1;

  for (i=ChildrenStart, n=0; i<ChildrenEnd; i++, n++) {
    index = GetFIT()->FAPs[i]-1;
    if (FAPMasks[index] == TRUE) continue;
    FAPMasks[index] = TRUE;
    pnumerator = &LinkFuncs[l].numerator[n];
    pdenominator = &LinkFuncs[l].denominator[n];
    nTerms = pnumerator->nTerms;
    numeratorValue = 0;
    for (j=0; j<nTerms; j++) {
      pterm = &pnumerator->terms[j];
      termValue = pterm->coef;
      for (k=ParentsStart, m=0; k<ParentsEnd; k++, m++) {
	if (pterm->delta == FALSE) continue;
	termValue *= pow(FAPValues[GetFIT()->FAPs[k]-1], pterm->expos[m]);
      }
      numeratorValue += termValue;
    }
    nTerms = pdenominator->nTerms;
    denominatorValue = 0;
    for (j=0; j<nTerms; j++) {
      pterm = &pdenominator->terms[j];
      termValue = pterm->coef;
      for (k=ParentsStart, m=0; k<ParentsEnd; k++, m++) {
	termValue *= pow(FAPValues[GetFIT()->FAPs[k]-1], pterm->expos[m]);
      }
      denominatorValue += termValue;
    }
    if (denominatorValue==0) {
      FAPValues[index] = (int) rint(numeratorValue);
    } else {
      FAPValues[index] = (int) rint(numeratorValue/denominatorValue);
    }
  }

#endif // ! BIFSENC

}

