// This software module was originally developed by Karl Anders ygard
// (Telenor R&D) in the course of development of the MPEG-4 Systems
// standard (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// Copyright 1997, 1998.

#include "OpenGl/OpenGl.h"
//#include "defs.h"
#include "FaceProxy.h"
//#include "opengl/base/OpenGLPresenter.h"
#include "opengl/group/GroupProxy.h"

#include "opengl/geometry/ShapeProxy.h"
#include "opengl/appearance/AppearanceProxy.h"
#include "opengl/appearance/MaterialProxy.h"
#include "opengl/geometry/IndexedFaceSetProxy.h"

// Gabriel TTS
#include "opengl/audio/AudioSourceProxy.h" // SOLARIS PORTING - case typo
#include "TtsDecoder/PhonemeFAP_Converter.h"

#ifdef MIRALABFACE

// Build face model

void FaceProxy::ConstructFace()
{
  TRACE("[FaceProxy::ConstructFace] begin \n");

  // This code is courtesy Igor Pandzic, and is subject to the same
  // copyright and copying conditions as the MIRALab facial animation
  // module.
  
  the_surface = NULL;
  the_face = NULL;
  the_region_list = NULL;
  
  Shape * poShape;
  ShapeProxy * poShapeProxy;
  Appearance * poAppearance;
  AppearanceProxy * poAppearanceProxy;
  Material * poMaterial;
  MaterialProxy * poMaterialProxy;
  IndexedFaceSet * poIndexedFaceSet;
  IndexedFaceSetProxy * poIndexedFaceSetProxy;
  Coordinate * poCoordinate;
  Normal * poNormal;
  
  SM_Triangle_ptr triangle_ptr = NULL;
  SM_Mat_ptr      material_ptr = NULL;
  VECTOR_3D vect3D;
  
  float r,g,b,ar,ag,ab,shininess;
  
  unsigned short *index_vert,*normal_index;
  int i, index;
  int material_index, triangle_index; 
  
  init_face(&the_surface,
    &the_face,
    &the_region_list);
  
  /// build a Shape like this for each material in the face
  ///
  ///        Shape
  ///          |
  ///          |-->IndexedFaceSet
  ///          |-->Appearance-->Material
  ///

  for (material_index = 0, material_ptr = the_materials.mat_list;
      material_index < the_materials.mat_list_size;
      material_index++, material_ptr++ ) 
  {
    if (! material_ptr->struct_used)
      continue;
    
    poShape = new Shape;
    poShapeProxy = new ShapeProxy(poShape);
    poAppearance = new Appearance;
    poAppearanceProxy = new AppearanceProxy(poAppearance);
    poMaterial = new Material;
    poMaterialProxy = new MaterialProxy(poMaterial);
    poIndexedFaceSet = new IndexedFaceSet;
    poIndexedFaceSetProxy = new IndexedFaceSetProxy(poIndexedFaceSet);
    poCoordinate = new Coordinate;
    poNormal = new Normal;

    // TRACE("NEW -- Material %4d Shape %08x IFS %08x \n", 
    //   material_index, poShape, poIndexedFaceSet);

    MF<Vec3f> coordinates;
    MF<Vec3f> normals;
    MF<int> indices;
    
    fcoord += poCoordinate;

    poShape->SetExecutive(GetFace()->GetExecutive());
    poShape->SetRenderingObject(poShapeProxy);
    poAppearance->SetExecutive(GetFace()->GetExecutive());
    poAppearance->SetRenderingObject(poAppearanceProxy);
    poMaterial->SetExecutive(GetFace()->GetExecutive());
    poMaterial->SetRenderingObject(poMaterialProxy);
    poIndexedFaceSet->SetExecutive(GetFace()->GetExecutive());
    poIndexedFaceSet->SetRenderingObject(poIndexedFaceSetProxy);
    poCoordinate->SetExecutive(GetFace()->GetExecutive());
    poNormal->SetExecutive(GetFace()->GetExecutive());

    /* --- Materiel Def. */
    SM_material_rgb_get(material_ptr->name, DIFFUSE,&r,&g,&b);
    poMaterial->diffuseColor = /* (SF<Color3f> &) */ (Color3f(r, g, b));
    
    SM_material_rgb_get(material_ptr->name, AMBIENT,&ar,&ag,&ab);
    if (r+g+b != 0.0f)
      poMaterial->ambientIntensity = ((ar+ag+ab)/(r+g+b));
    else
      poMaterial->ambientIntensity = (0.0f);
    SM_material_rgb_get(material_ptr->name, SPECULAR,&r,&g,&b);
    poMaterial->specularColor = /* (SF<Color3f> &) */ (Color3f(r,g,b));
    SM_material_value_get(material_ptr->name, SHININESS,&shininess);
    poMaterial->shininess = (shininess/128.0f);
    
#if defined(_DEBUG)
    {
      const Color3f& color = (Color3f &) poMaterial->diffuseColor;
      fprintf(stderr,"Diffuse%f %f %f\n",color.r,color.g,color.b);
    }
    
    {
      const Color3f& color = (Color3f &) poMaterial->specularColor;
      fprintf(stderr,"Specular%f %f %f\n",color.r,color.g,color.b);
    }
    
    {
      const Color3f& color = (Color3f &) poMaterial->emissiveColor;
      fprintf(stderr,"Emissive%f %f %f\n",color.r,color.g,color.b);
    }
    
    fprintf(stderr,"AmbientIntensity%f\n", poMaterial->ambientIntensity);
    fprintf(stderr,"Shininess%f\n", poMaterial->shininess);
#endif // _DEBUG

/* //ML - replaced these two lines with the next two
    poAppearance->material = poMaterial;
    poShape->appearance = poAppearance;
*/
    poAppearance->material.SetObject(poMaterial, poAppearance);
    poShape->appearance.SetObject(poAppearance,  poShape);

    
    index = 0;
    index_vert = (unsigned short *)malloc(3*the_surface->current_tri_list_size*sizeof(unsigned short));
    normal_index = (unsigned short *)malloc(3*the_surface->current_tri_list_size*sizeof(unsigned short));
    
    for (triangle_index = 0, triangle_ptr = the_surface->tri_list;
        triangle_index < the_surface->current_tri_list_size;
        triangle_index++, triangle_ptr++) 
    {
      if (triangle_ptr->struct_used) 
      {
        /* test on triangle material */
        if(triangle_ptr->material_index == material_index) 
        {
          index_vert[index]   = triangle_ptr->vertex1; 
          index_vert[index+1] = triangle_ptr->vertex2;
          index_vert[index+2] = triangle_ptr->vertex3;
          
          normal_index[index] = triangle_ptr->normal1;
          normal_index[index+1] = triangle_ptr->normal2;
          normal_index[index+2] = triangle_ptr->normal3;
          
          index+=3;
        }
      }
    }
    if ( index != 0 )
    {
      for(i=0;i<index;i+=3)
      {
        SM_vertex_point_get(the_surface,index_vert[i],vect3D);
        coordinates.Append(Vec3f(vect3D[0],vect3D[1],vect3D[2]));
        SM_vertex_point_get(the_surface,index_vert[i+1],vect3D);
        coordinates.Append(Vec3f(vect3D[0],vect3D[1],vect3D[2]));
        SM_vertex_point_get(the_surface,index_vert[i+2],vect3D);
        coordinates.Append(Vec3f(vect3D[0],vect3D[1],vect3D[2]));
      }
	  //dn
      //poCoordinate->point = *((NodeField *) &(coordinates));
      poCoordinate->point = coordinates;
      //ML - replaced this line with the following
	  //poIndexedFaceSet->coord = poCoordinate;
	  poIndexedFaceSet->coord.SetObject(poCoordinate, poIndexedFaceSet);

      for(i=0;i<index;i+=3)
      {
        Vec3f normal;
        SM_vertex_normal_get(the_surface,index_vert[i],normal_index[i],vect3D);
        normal.x = vect3D[0];
        normal.y = vect3D[1];
        normal.z = vect3D[2];
        normal.Normalize();
        normals.Append(normal);
        SM_vertex_normal_get(the_surface,index_vert[i+1],normal_index[i+1],vect3D);
        normal.x = vect3D[0];
        normal.y = vect3D[1];
        normal.z = vect3D[2];
        normal.Normalize();
        normals.Append(normal);
        SM_vertex_normal_get(the_surface,index_vert[i+2],normal_index[i+2],vect3D);
        normal.x = vect3D[0];
        normal.y = vect3D[1];
        normal.z = vect3D[2];
        normal.Normalize();
        normals.Append(normal);
      }
//dn      poNormal->vector = *((NodeField *) &(normals));
       poNormal->vector = normals;
       //ML - replaced this line with the following
	   //poIndexedFaceSet->normal = poNormal;
	   poIndexedFaceSet->normal.SetObject(poNormal, poIndexedFaceSet);
      
      for(i=0;i<index;i+=3)
      {
        indices.Append(i);
        indices.Append(i+1);
        indices.Append(i+2);
        indices.Append(-1);
      }
      
//dn
//      poIndexedFaceSet->coordIndex = *((NodeField *) &(indices));
      poIndexedFaceSet->coordIndex = indices;

    }
    
    //ML - replaced this line with the following
    //poShape->geometry = poIndexedFaceSet;
    poShape->geometry.SetObject(poIndexedFaceSet, poShape);
    poIndexedFaceSet->SetUpdateTime();

    // TRACE("shape %08x geometry %08x \n", 
    //   poShape, (MediaObject *) (poShape->geometry));
    
    AddChild(poShape);
    free(index_vert);
    free(normal_index);

    // TRACE("coord index [0] %4d \n", 
    //   ((IndexedFaceSet *) ((MediaObject *) (poShape->geometry)))
    //   ->coordIndex[0]);
  }

  TRACE("[FaceProxy::ConstructFace] end \n");
}

#endif // MIRALABFACE

// Update object

BOOL FaceProxy::Update(VisualRenderer* pRenderer, DWORD dwTime)
{
  TRACE("[FaceProxy::Update] begin \n");

  MediaObject * poTtsSource = GetFace()->ttsSource;
  if (poTtsSource != NULL)
    ((AudioSourceProxy *) poTtsSource->GetRenderingObject())->
      SetTTSFaceNode(this);

#ifndef BIFSENC
  // SB $$$$ 99/03/18
  // ifdefed out
#ifdef TTS
	if (tts_mode) PhonemeFAPConv->Get1FA_Frame();
#endif // TTS
#endif // ! BIFSENC

  if (GetFAP())
  {
    mpoFapProxy = (FAPProxy *) GetFAP()->GetRenderingObject();
    if (! mpoFapProxy->Update(pRenderer, dwTime))
    {
      return FALSE;
    }
    if (GetFIT())
    {
      mpoFitProxy = (FITProxy *) GetFIT()->GetRenderingObject();
      if (! mpoFitProxy->Update(GetFace() /* this */, dwTime))
      {
        return FALSE;
      }
    }
    if (GetFDP())
    {
      mpoFdpProxy = (FDPProxy *) GetFDP()->GetRenderingObject();
      if (! mpoFdpProxy->Update(GetFace() /* this */, dwTime))
      {
        return FALSE;
      }
    }
    else
    {
#ifdef MIRALABFACE
      if (!m_bInitialised)
      {
        m_bInitialised = TRUE;
        ConstructFace();
      }
      
      Coordinate *poCoordinate;
      
      SM_Mat_ptr      material_ptr = NULL;
      SM_Triangle_ptr triangle_ptr = NULL;
      VECTOR_3D vect3D;
      
      unsigned short *index_vert;
      int i,j,k,index;
      int material_index, triangle_index; 
      int fapvalues[68];
      int fapmask[68];
      
      for (i=2; i<68; i++)
      {
        fapmask[i] = mpoFapProxy->GetFAPMask(i);
        fapvalues[i] = mpoFapProxy->GetFAPValue(i);
      }
      
      run_faps(fapmask,
        fapvalues,
        the_surface,
        the_face,
        the_region_list);
      
      k = 0;
      
      for (material_index = 0, material_ptr = the_materials.mat_list;
      material_index < the_materials.mat_list_size;
      material_index++, material_ptr++ ) 
      {
        if (!material_ptr->struct_used)
          continue;
        
        poCoordinate = fcoord[k++];
        
        index = 0;
        index_vert = (unsigned short *)malloc(3*the_surface->current_tri_list_size*sizeof(unsigned short));
        
        for (triangle_index = 0, triangle_ptr = the_surface->tri_list;
        triangle_index < the_surface->current_tri_list_size;
        triangle_index++, triangle_ptr++)
        {
          if (triangle_ptr->struct_used) 
          {
            /* test on triangle material */
            if(triangle_ptr->material_index == material_index) 
            {
              index_vert[index]   = triangle_ptr->vertex1; 
              index_vert[index+1] = triangle_ptr->vertex2;
              index_vert[index+2] = triangle_ptr->vertex3;
              index+=3;
            }
          }
        }
        if ( index != 0 )
        {
          MF<Vec3f> coordinates;
          
          j = 0;
          for(i=0;i<index;i+=3)
          {
            SM_vertex_point_get(the_surface,index_vert[i],vect3D);
            coordinates.SetAt(j++, /* SF<Vec3f> */ (Vec3f(vect3D[0],vect3D[1],vect3D[2])));
            SM_vertex_point_get(the_surface,index_vert[i+1],vect3D);
            coordinates.SetAt(j++, /* SF<Vec3f> */ (Vec3f(vect3D[0],vect3D[1],vect3D[2])));
            SM_vertex_point_get(the_surface,index_vert[i+2],vect3D);
            coordinates.SetAt(j++, /* SF<Vec3f> */ (Vec3f(vect3D[0],vect3D[1],vect3D[2])));
          }
          
          poCoordinate->point = /* SetPoint */ (coordinates);
        }
        free(index_vert);
      }
#endif // MIRALABFACE
    }
  }
  
  return TRUE;
}

// Render object

/* virtual */ BOOL
FaceProxy::Render(const Matrix& rTransformation, const CullFlag& rCullFlag,
		  OpenGLVisualRenderer* pRenderer, DWORD dwTime)
{
  TRACE("[FaceProxy::Render] begin \n");

  // Face* pFace = (Face*) GetMediaObject();
  ZArray<NodeProxy*> nodes;
  
  Update(pRenderer, dwTime);

  for (MediaObject * pObject = GetFirstChild(); 
      pObject; 
      pObject = GetNextSibling())
  {
    NodeProxy* pProxy = (NodeProxy*) pObject->GetRenderingObject();

    int i = 0;
    float drawingOrder = pProxy->GetDrawingOrder();
    
    while (nodes.GetLength() > i && nodes[i]->GetDrawingOrder() > drawingOrder)
      ++i;
    
    nodes.InsertAt(i, pProxy);
  }

  for (int j = 0; j < nodes.GetLength(); ++j)
  {
    // TRACE("[FaceProxy::Render] Cull/Render node %4d \n", j);
    nodes[j]->CullOrRender(rTransformation, rCullFlag,
			   pRenderer, dwTime);
  }

#ifdef _DEBUG
  if (pRenderer->CheckGLError("FaceProxy:"))
    TRACE("FaceProxy::Render() [%08x]: ", this);
#endif // DEBUG

  return Node3DProxy::Render(rTransformation, rCullFlag,
			     pRenderer, dwTime);
}

// Traverse scene graph, checking for intersection

/* virtual */ void FaceProxy::Select(
  OpenGLVisualRenderer* pRenderer, 
  DWORD dwTime,
	MediaObjectProxy*& intersectingObject, 
  unsigned int& depth)
{
  // Face* pFace = (Face*) GetMediaObject();

  for (MediaObject * pObject = /* pFace-> */ GetFirstChild(); 
      pObject; pObject = /* pFace-> */ GetNextSibling())
    ((MediaObjectProxy*) pObject->GetRenderingObject())->Select(pRenderer, dwTime, intersectingObject, depth);
}

// Add node to draw list

/* virtual */ BOOL
FaceProxy::RenderLights(OpenGLVisualRenderer* pRenderer, DWORD dwTime)
{
  // Face* pFace = (Face*) GetMediaObject();

  for (MediaObject * pObject = /* pFace-> */ GetFirstChild(); 
      pObject; pObject = /* pFace-> */ GetNextSibling())
    ((NodeProxy*) pObject->GetRenderingObject())->RenderLights(pRenderer, dwTime);

  return TRUE;
}

// Render object description

/* virtual */ void
FaceProxy::RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD dwTime)
{
  // Face* pFace = (Face*) GetMediaObject();
  // Stefano Battista 98/11/10
  int children = ((/* pFace-> */ GetFDP() ? 1 : 0) +
		  (/* pFace-> */ GetFAP() ? 1 : 0) +
		  /* ((Group *) pFace)-> */ GetChildCount());

  RenderDescriptionBox(pRenderer, "Face");

  ::glPushMatrix();

  ::glScalef(1.0f / children, 1.0f, 1.0f);
  ::glTranslatef(0.0f, -1.0f, 0.0f);

  if (/* pFace-> */ GetFDP()) {
    ((FDPProxy*) /* pFace-> */ GetFDP()->GetRenderingObject())->RenderDescription(pRenderer, dwTime);
    
    ::glTranslatef(1.0f, 0.0f, 0.0f);
  }

  if (/* pFace-> */ GetFAP()) {
    ((FDPProxy*) /* pFace-> */ GetFAP()->GetRenderingObject())->RenderDescription(pRenderer, dwTime);
    
    ::glTranslatef(1.0f, 0.0f, 0.0f);
  }

  for (MediaObject * pObject = /* pFace-> */ GetFirstChild(); 
      pObject; pObject = /* pFace-> */ GetNextSibling()) {
    ((NodeProxy*) pObject->GetRenderingObject())->RenderDescription(pRenderer, dwTime);

    ::glTranslatef(1.0f, 0.0f, 0.0f);
  }

  ::glPopMatrix();
}

// Traverse scene graph, checking for activation of sensors

/* virtual */ void
FaceProxy::Select(
    const Matrix& rTransformation, 
    const CullFlag& rCullFlag,
    OpenGLVisualRenderer* pRenderer, 
    DWORD dwTime,
    MediaObjectProxy*& intersectingObject,
    ZArray<MediaObjectProxy*>& activatedSensors, 
    unsigned int& depth,
    const ZArray<MediaObjectProxy*>& localSensors)
{
  // Face* pFace = (Face*) GetMediaObject();
  ZArray<MediaObjectProxy*> sensors;
  ZArray<MediaObjectProxy*> nodes;

  // sort objects by drawing order and find pointing device sensors

  for (MediaObject * pObject = /* pFace-> */ GetFirstChild(); 
      pObject; pObject = /* pFace-> */ GetNextSibling()) {
    MediaObjectProxy* pProxy = (MediaObjectProxy*) pObject->GetRenderingObject();

    int i = 0;
    float drawingOrder = pProxy->GetDrawingOrder();
    
    while (nodes.GetLength() > i && nodes[i]->GetDrawingOrder() > drawingOrder)
      ++i;
    
    nodes.InsertAt(i, pProxy);

    if (pProxy->IsPointingDeviceSensor())
      sensors += pProxy;
  }

  // then traverse scene graph further down

  for (int j = 0; j < nodes.GetLength(); ++j)
    nodes[j]->CullOrSelect(rTransformation, rCullFlag,
			   pRenderer, dwTime,
			   intersectingObject,
			   activatedSensors, depth,
			   sensors.GetLength() ? sensors : localSensors);
}


// Add node

/* virtual */ BOOL FaceProxy::AddChild(MediaObject * pObject)
{
  // put the child in the list of our children

  // if (GetFace()->renderedFace.AppendElement(pObject))
  (GetFace()->renderedFace.Append(pObject,GetFDP()));
  {
    // here, we need to recompute the bounding box
    UpdateBbox();

    // put us in the list of the child's parents
    pObject->SetParent(GetFace() /* this */);

    return TRUE;
  }
}

// Update bounding box

/* virtual */ void
FaceProxy::UpdateBbox()
{
  MediaObject * child = GetFirstChild();
  
  if (child) {
    Vec3f center;
    Vec3f size;
    
    ((MediaObjectProxy *) (child->GetRenderingObject())) ->
      GetTransformedBbox((SF<Vec3f>) size, (SF<Vec3f>) center);
    
    Vec3f fur(center);
    Vec3f bll(center);
    
    if (size.x > -1.0f) {
      size *= 0.5f;
      
      fur += size;
      bll -= size;
    }
    
    while ((child = GetNextSibling())) {
      ((MediaObjectProxy *) (child->GetRenderingObject())) ->
        GetTransformedBbox((SF<Vec3f>) size, (SF<Vec3f>) center);
      
      if (size.x > 0.0f) {
        float tmp;
        
        size *= 0.5f;
        
        if (fur.x < (tmp = center.x + size.x)) 
          fur.x = tmp;
        
        if (bll.x > (tmp = center.x - size.x)) 
          bll.x = tmp;
        
        if (fur.y < (tmp = center.y + size.y))
          fur.y = tmp;
        
        if (bll.y > (tmp = center.y - size.y)) 
          bll.y = tmp;
        
        if (fur.z < (tmp = center.z + size.z)) 
          fur.z = tmp;
        
        if (bll.z > (tmp = center.z - size.z)) 
          bll.z = tmp;
      } else {
        if (fur.x < center.x) 
          fur.x = center.x;
        
        if (bll.x > center.x)
          bll.x = center.x;
        
        if (fur.y < center.y)
          fur.y = center.y;
        
        if (bll.y > center.y)
          bll.y = center.y;
        
        if (fur.z < center.z)
          fur.z = center.z;
        
        if (bll.z > center.z)
          bll.z = center.z;
      }
    }      
    
    Vec3f new_size(fur);
    Vec3f new_center(fur);
    
    new_size -= bll;
    
    new_center += bll;
    new_center *= 0.5;
    
    /* GetFace()-> */ GetBbox((SF<Vec3f>) size, (SF<Vec3f>) center);
    
    if (new_center != center || new_size != size) 
      /* GetFace()-> */ SetBbox((SF<Vec3f>) new_size, (SF<Vec3f>) new_center);	
  }
  else
  {
    /* GetFace()-> */ SetBbox((SF<Vec3f>) Vec3f(), (SF<Vec3f>) Vec3f());
  }

  MediaObject * poTtsSource = GetFace()->ttsSource;
  if (poTtsSource != NULL)
    ((AudioSourceProxy *) poTtsSource->GetRenderingObject())->
      SetTTSFaceNode(this);
}

// Look in the branch under this node for an object with an ID of nID

MediaObject * FaceProxy::FindNode(int nID)
{
  // MediaObject * pResult = MediaObject::FindNode(nID);
  MediaObject * pResult = GetFace()->GetRoot()->FindNode(nID);
  
  if (!pResult && GetFIT())
    pResult = GetFIT()->GetRoot()->FindNode(nID);
  
  if (!pResult && GetFAP())
    pResult = GetFAP()->GetRoot()->FindNode(nID);
  
  if (!pResult && GetFDP())
    pResult = GetFDP()->GetRoot()->FindNode(nID);
  
  for (int i = GetRenderedFace().GetLength() - 1; i >= 0 && !pResult; --i)
    pResult = GetRenderedFace()[i]->GetRoot()->FindNode(nID);
  
  return pResult;
}

// TTS Face interface (Gabriel)

void FaceProxy::SetTTS_Mode(int mode)
{
	if(this) tts_mode = mode;
}

void FaceProxy::SetPhonemeFAPConverter(PhonemeFAP_Converter* pfc)
{
	if(this) PhonemeFAPConv = pfc;
}


// Riitta 99/05/06
#ifdef ADV_AUDIO 

/* virtual */ int
FaceProxy::FindSoundSources(
    ZArray<Sound*>& rSources3d, 
    ZArray<Matrix>& rTransformations3d,
    ZArray<Sound2D*>& rSources2d, 
    ZArray<Matrix>& rTransformations2d,
    ZArray<DirectiveSound*>& rDirsources, 
    ZArray<Matrix>& rDirtransformations,
    const Matrix& rTransform)
{
  // Face * poFace = (Face *) GetMediaObject();
  int ret = 0;
  
  for (MediaObject* poObject = /* poFace-> */ GetFirstChild(); 
     poObject; poObject = /* poFace-> */ GetNextSibling())
    ret += ((MediaObjectProxy*) poObject->GetRenderingObject())->
        FindSoundSources(rSources3d, rTransformations3d,
            rSources2d, rTransformations2d,
            rDirsources, rDirtransformations,
            rTransform);
  return ret;
}

/* virtual */ int
FaceProxy::FindAcousticScenes(
    ZArray<AcousticScene*>& rAcousticScenes, 
    ZArray<Matrix>& rASceneTransformations,
    const Matrix& rTransform)
{
  // Face * poFace = (Face *) GetMediaObject();
  int ret = 0;

  for (MediaObject* pObject = /* poFace-> */ GetFirstChild(); 
      pObject; pObject = /* poFace-> */ GetNextSibling())
    ret += ((MediaObjectProxy*) pObject->GetRenderingObject())->
        FindAcousticScenes(
            rAcousticScenes, 
            rASceneTransformations,
            rTransform);
  return ret;
}

/* virtual */ int
FaceProxy::FindAcousticSurfaces(
    ZArray<Shape*>& rAcousticSurfaces, 
    ZArray<Matrix>& rASurfaceTransformations,
    const Matrix& rTransform)
{
  // Face * poFace = (Face *) GetMediaObject();
  int ret = 0;
  
  for (MediaObject* pObject = /* poFace-> */ GetFirstChild(); 
      pObject; pObject = /* poFace-> */ GetNextSibling())
    ret += ((MediaObjectProxy*) pObject->GetRenderingObject())->
        FindAcousticSurfaces(
            rAcousticSurfaces, 
            rASurfaceTransformations,
            rTransform);
  return ret;
}

#else // ! ADV_AUDIO

// Find sound sources

/* virtual */ int
FaceProxy::FindSoundSources(
    ZArray<Sound*>& rSources3d, 
    ZArray<Matrix>& rTransformations3d,
		ZArray<Sound2D*>& rSources2d, 
    ZArray<Matrix>& rTransformations2d,
		const Matrix& rTransform)
{
  // Face* poFace = (Face*) GetMediaObject();
  int ret = 0;

  for (MediaObject * pObject = /* poFace-> */ GetFirstChild(); 
      pObject; pObject = /* poFace-> */ GetNextSibling())
    ret += ((MediaObjectProxy*) pObject->GetRenderingObject())->
        FindSoundSources(
            rSources3d, rTransformations3d,
						rSources2d, rTransformations2d,
						rTransform);

  return ret;
}

#endif // ADV_AUDIO
