// -*- c++ -*-

// This software module was originally developed by Karl Anders ygard
// (Telenor R&D) in the course of development of the MPEG-4 Systems
// standard (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// Copyright 1997.

#ifndef __FACEPROXY_H__
#define __FACEPROXY_H__

//#include "defs.h"

#include "opengl/base/Node3DProxy.h"

// Stefano Battista
// class Face;
// class NodeProxy;
//#include "Nodes.h"

class OpenGLPresenter;
// #include "base/VisualRenderer.h"
#include "opengl/base/OpenGLVisualRenderer.h"

// Stefano Battista
// added includes for Face Model
#include "opengl/base/Matrix.h"

#ifdef MIRALABFACE
#include "face/mpeg_face.h"
#ifndef CPLUSPLUS
#define CPLUSPLUS
#endif // ! CPLUSPLUS
#include "face/SurfMan.h"
#endif // MIRALABFACE

class FITProxy;
class FDPProxy;
class FAPProxy;

class AudioSourceProxy;
class PhonemeFAP_Converter;
class TTS_Interface;

/// Face renderer node

class FaceProxy
  : public Node3DProxy
{
#ifdef MIRALABFACE

  void                  ConstructFace();

  ZArray<Coordinate*>   fcoord;
  SM_Surface_ptr        the_surface;
  SM_Surface_ptr        the_face;
  SM_Region_List_ptr    the_region_list;
  SM_Material           the_material;

#endif

private:
  BOOL                  m_bInitialised;

  // Stefano Battista
  // @@@@ added
  FITProxy * mpoFitProxy;
  FDPProxy * mpoFdpProxy;
  FAPProxy * mpoFapProxy;

  // methods for TTS Face interface (Gabriel)
  int tts_mode;
  PhonemeFAP_Converter* PhonemeFAPConv;

public:

  /// Constructor
  FaceProxy(Face* pFace);

  // SB 99/05/14 added for testing
  virtual void DumpTrace()
  {
    Face * poNode = (Face *) GetMediaObject();
    poNode->GetPresenter()->Logf("RendFace %08x \n", this);
  }

  /// Traverse scene graph, activating lights
  virtual BOOL RenderLights(
    OpenGLVisualRenderer* pRenderer, 
    DWORD dwTime);

  /// Traverse scene graph, checking for intersection
  virtual void Select(
    OpenGLVisualRenderer* pRenderer, 
    DWORD dwTime,
		MediaObjectProxy*& intersectingObject, 
    unsigned int& depth);

  /// Render object description
  virtual void RenderDescription(
    OpenGLVisualRenderer* pRenderer, 
    DWORD dwTime);

#ifdef ADV_AUDIO // riitta 080399

  virtual int	FindSoundSources(
    ZArray<Sound*>& rSources3d, 
    ZArray<Matrix>& rTransformations3d,
    ZArray<Sound2D*>& rSources2d, 
    ZArray<Matrix>& rTransformations2d,
    ZArray<DirectiveSound*>& rDirsources, 
    ZArray<Matrix>& rDirtransformations,
    const Matrix& rTransform);
  
  virtual int		FindAcousticScenes(
    ZArray<AcousticScene*>& rAcousticScenes, 
    ZArray<Matrix>& rASceneTransformations,
    const Matrix& rTransform);
  
  virtual int FindAcousticSurfaces(
    ZArray<Shape*>& rAcousticSurfaces,				// IndexedFaceSet with AcousticMaterial
    ZArray<Matrix>& rASurfaceTransformations,				// Transformations above the AcousticMaterial
    const Matrix& rTransform);
  
#else
  
  virtual int	FindSoundSources(
    ZArray<Sound*>& rSources3d, 
    ZArray<Matrix>& rTransformations3d,
    ZArray<Sound2D*>& rSources2d, 
    ZArray<Matrix>& rTransformations2d,
    const Matrix& rTransform);
  
#endif // ADV_AUDIO
  
protected:

  /// Render object
  virtual BOOL Render(
    const Matrix& rTransformation, 
    const CullFlag& rCullFlag,
    OpenGLVisualRenderer* pRenderer, 
    DWORD dwTime);

  /// Traverse scene graph, checking for activation of sensors
  virtual void Select(
    const Matrix& rTransformation, 
    const CullFlag& rCullFlag,
		OpenGLVisualRenderer* pRenderer, 
    DWORD dwTime,
		MediaObjectProxy*& intersectingObject,
		ZArray<MediaObjectProxy*>& activatedSensors, 
    unsigned int& depth,
		const ZArray<MediaObjectProxy*>& localSensors);

  // Stefano Battista
  /// moved methods from Face to FaceProxy
  BOOL Update(
    VisualRenderer* pRenderer, 
    DWORD dwTime);
  /// Add node
  BOOL AddChild(MediaObject * pObject);
  /// Update bounding box
  void UpdateBbox();
public:
  /// Look in the branch under this node for an object with an ID of nID
  MediaObject * FindNode(int nID);

  // Stefano Battista
  // moved methods from Face to FaceProxy
public:
  /// Retrieve the pointer to the associated Face object
  inline Face * GetFace() const
  { return (Face *) GetMediaObject(); }
  /// Get FIT node
  inline FIT * GetFIT()
  { return GetFace()->fit; }

/*ML - removed this unused function
  /// Set FIT node
  inline void SetFIT(FIT * pFIT)
  {
    if (GetFIT())
      GetFIT()->RemoveParent(GetFace());
    GetFace()->fit = pFIT;
    if (pFIT)
      pFIT->SetParent(GetFace());
  }
*/
  
  /// Get FDP node
  inline FDP * GetFDP()
  { return GetFace()->fdp; }

/*ML - removed this unused function
  /// Set FDP node
  inline void SetFDP(FDP * pFDP)
  {
    if (GetFDP())
      GetFDP()->RemoveParent(GetFace());
    GetFace()->fdp = pFDP;
    if (pFDP)
      pFDP->SetParent(GetFace());
  }
*/
  
  /// Get FAP node
  inline FAP * GetFAP()
  { return GetFace()->fap; }

/*ML - removed this unused function
  /// Set FAP node
  inline void SetFAP(FAP * pFAP)
  {
    if (GetFAP())
      GetFAP()->RemoveParent(GetFace());
    GetFace()->fap = pFAP;
    if (pFAP)
      pFAP->SetParent(GetFace());
  }
*/
  
  /// Get rendered face
  inline MFNodeType<SF3DNode>& GetRenderedFace() const
  { return GetFace()->renderedFace; }
  /// Set rendered face
  inline void SetRenderedFace(MFNodeType<SF3DNode> pRNode)
  { GetFace()->renderedFace = pRNode; }
  /// Get next child in this Face
  inline MediaObject * GetNextSibling()
  {
    Face * poFace = GetFace();
    MFNodeType<SF3DNode>& roRenderedFace = GetRenderedFace();
    int iCurrentElement = poFace->GetCurrentElement();
    if (roRenderedFace.GetLength() <= iCurrentElement)
    {
      return NULL;
    }
    else
    {
      MediaObject * poObject = (MediaObject *) roRenderedFace[iCurrentElement];
      poFace->SetCurrentElement(++iCurrentElement);
      return poObject;
    }
  }
  /// Get first child in this Face
  inline MediaObject * GetFirstChild()
  {
    GetFace()->SetCurrentElement(0);
    return GetNextSibling();
  }
  /// Get number of children in this Face
  inline int GetChildCount() const
  { return GetRenderedFace().GetLength(); }

  // methods for TTS Face interface (Gabriel)
  void SetTTS_Mode(int mode);
  void SetTTSInterface(TTS_Interface* ttsinter);
  void SetPhonemeFAPConverter(PhonemeFAP_Converter* pfc);
};

#include "FaceProxy.i"

#endif // __FACEPROXY_H__
