#include "OpenGl/OpenGl.h"
#include <stdio.h>
#include <stdlib.h>

#include "HAnimParser.h"

//#include "defs.h"
#include "opengl/group/TransformProxy.h"
#include "opengl/geometry/IndexedFaceSetProxy.h"
#include "opengl/geometry/ShapeProxy.h"
//#include "Nodes.h"

#include "SegmentParser.h"

// - CONSTRUCTOR ---------------------------------------
HAnimParser::HAnimParser(char * fileToParse, Executive * pExecutive)
{
	// open the file for reading
	TRACE("[HAnimParser:HAnimParser] Opening file : %s\n",fileToParse);
	parsedFile = fopen(fileToParse,"rt");

	// create the fiorst root of the body
	TRACE("[HAnimParser:HAnimParser] Creating body root\n");
	resultBody=new Transform();
	// Tolga: 00/03/28. Commented out the following. Core is updated so it may work
	// resultBody->CreateChildren(); // trick here, avoid assert error from Core

	// create the table and sets pointers to NULL (secure)
	pointerTable=new Transform*[NUMBER_OF_JOINTS];
    // PORTING to CC 5.3
    int i = 0;
//	for (int i=0 ; i<NUMBER_OF_JOINTS ; i++)
	for ( i=0 ; i<NUMBER_OF_JOINTS ; i++)
		pointerTable[i]=NULL;

	// parse file, creates the hierarchy, aso
	TRACE("[HAnimParser:HAnimParser] Parsing Joints\n");
	if (parseJoints(resultBody, pExecutive)==-1)
		TRACE("[HAnimParser:HAnimParser] Could not parse Joints\n");

	// shortcuts the first Node. ResultBody -> Node0 -> HumanoidRoot
	//                                |                 ^
	//                                +-----------------+
	resultBody=(Transform *)(resultBody->GetChild(1));

	TRACE("\n\nUNUSED POINTER TO JOINTS\n");
	for (i=0 ; i<NUMBER_OF_JOINTS ; i++)
		if (pointerTable[i]==NULL)
			TRACE("JOINT NUMBER %d = %s USELESS\n",i,hanimJointNames[i]);
}

// - DESTRUCTOR ---------------------------------------
HAnimParser::~HAnimParser()
{
  fclose(parsedFile);
}

// - FINDJOINTNUMBERFROMNAME ---------------------------------------
// parse the array of Joint names to find a given Joint,
// if found (multiple possible), fills the given array with numbers
// otherwise return -1
int HAnimParser::findJointNumberFromName(char * name, int * buf)
{
	int nbFound=0;
	
	// loop on the Hanim names to find it
	for (int i=0 ; i<NUMBER_OF_JOINTS ; i++) {
		if (!strcmp(name,hanimJointNames[i])) {
			buf[nbFound++]=i;
		}
	}

	// Joint was not found, return -1
	if (nbFound==0) return -1;
	else return nbFound;
}

// - PARSEJOINTS ---------------------------------------
// recursive, parse a joint definition, even if it
// has joint children -> recursive call to the function
// return 0 if it could read a joint, -1 otherwise
int HAnimParser::parseJoints(Transform * father, Executive * pExecutive)
{
	// locales
	char name[SIZE_OF_PARSING_BUFFER];
	char segment[SIZE_OF_PARSING_BUFFER];
	float x,y,z;
	BOOL isCenter=false;

	// test if a joint is beginning there
	if (fscanf(parsedFile,"DEF hanim_%s Joint {\n",name)!=1) return -1;

	// try to parse the vector (center or translation)
	if (fscanf(parsedFile,"center %f %f %f\n",&x,&y,&z)!=3) {
		if (fscanf(parsedFile,"translation %f %f %f\n",&x,&y,&z)!=3) {
			TRACE("[HAnimParser:parseJoints] Error of syntax int the file\n");
			exit(1);
			}
	}
	else
		isCenter=TRUE;

	TRACE("[HAnimParser:parseJoints] Joint %s found with value (%f,%f,%f)\n",name,x,y,z);

	// create a vector scaled to MPEG4 values
	Vec3f vecteur(x, y, z);

	// create a new Transform node and attaches it to a new Proxy
	Transform * pTransform = new Transform();
	TransformProxy * pTransformProxy = new TransformProxy(pTransform);
	pTransform->SetRenderingObject(pTransformProxy);
	pTransform->SetExecutive(pExecutive);
	pTransform->SetUpdateTime();
	// Tolga: 00/03/28. Commented out the following. Core is updated so it may work
	// pTransform->CreateChildren();

	// sets whether translation or center
	if (isCenter)
		pTransform->center=vecteur;
	else
		pTransform->translation=vecteur;

	// adds the joint to the pointer Table
	int jointPositions[10];
	int nbFound = findJointNumberFromName(name,jointPositions);
	if (nbFound!=-1) {
		for (int i=0 ; i<nbFound ; i++) {
			TRACE("Joint %s added to the pointer table at position %d\n",name,jointPositions[i]);
			// makes that node be pointed by all common joints
			pointerTable[jointPositions[i]]=pTransform;
		}
	}
	else
		TRACE("Could not add that joint to the pointer table !\n");
	
	// Does the joint have a segment attached ?
	if (fscanf(parsedFile,"geometry %s\n",segment)==1) {
		// Yes, parse it
		TRACE("JOINT %s has segment %s attached\n",name,segment);
		SegmentParser s(segment,pExecutive,pTransform);
	}

	// attaches the local hierarchy to the father
	pTransform->SetParent(father);
	father->AddChild(pTransform);

	// does the joint have children ?
	if (fscanf(parsedFile,"children %c\n",&name[SIZE_OF_PARSING_BUFFER-1])==1) {

		// yes, parse of the children
		int res=0;
		while (res==0) {
			res=parseJoints(pTransform,pExecutive);
	    }

		// and parse the end of the children
		fscanf(parsedFile,"]\n");
	}

	// parse the end of joint closed bracket
	fscanf(parsedFile,"}\n");
	
	// everything's OK, return 0
	return 0;
}
