/***************************************************************************/
/*                                                                         */
/* HAnimParser Class Interface											   */
/*                                                                         */
/***************************************************************************/
/*                                                                         */
/* Purpose : This class is devoted to the creation of an IM1 compliant hie-*/
/* rarchy of Transform Nodes, according to the body definition file.       */
/*                                                                         */
/* Version : 1.03                                                          */
/*                                                                         */
/* Latest Change : 10/11/99                                                */
/*                                                                         */
/* Authors : Fabrice Vergnenegre, Tolga Capin, LIG - EPFL                  */
/*                                                                         */
/***************************************************************************/
/* 10/11/99 : Dynamically constructs a table of pointers to the IM1 nodes  */
/* 10/08/99 : Dynamically constructs a hierarchy of IM1 Nodes from the     */
/*            parsed file. Attaches a sphere to these joints for rendering */
/* 10/06/99 : Finished the architecture independent parser, using a local  */
/*            Tree class and Vector3d class for construction of the tree   */
/***************************************************************************/
#ifndef _HEADER_OF_CLASS_HANIM_PARSER_
#define _HEADER_OF_CLASS_HANIM_PARSER_

// Includes for that class
#include <stdio.h>
#include <stdlib.h>
#include "defs.h"
#include "OpenGl/OpenGl.h"
#include "opengl/group/TransformProxy.h"
#include "opengl/geometry/IndexedFaceSetProxy.h"
#include "opengl/geometry/ShapeProxy.h"
#include "Nodes.h"

// Defines for that class
#define SIZE_OF_PARSING_BUFFER 2000
#define NUMBER_OF_JOINTS 78

static char hanimJointNames[NUMBER_OF_JOINTS][20] =
{
  "sacroiliac", "l_hip", "l_knee", "l_ankle", "l_subtalar",
  "l_midtarsal", "l_metatarsal", "r_hip", "r_knee", "r_ankle", "r_subtalar",
  "r_midtarsal", "r_metatarsal", "vl5",  "vl4", "vl3",  "vl2",  "vl1", "vt10",
  "vt9", "vt8", "vt7", "vt6", "vt5", "vt4", "vt3", "vt2",
  "vt1", "vc7", "vc6", "vc5", "vc4", "vc3", "vc2", "vc1", "skullbase",
  "l_sternoclavicular", "l_acromioclavicular", "l_shoulder", "l_elbow",
  "l_wrist", "r_sternoclavicular", "r_acromioclavicular", "r_shoulder", 
  "r_elbow", "r_wrist",
  "HumanoidRoot", "HumanoidRoot",
  "l_thumb1", "l_thumb2", "l_thumb3", 
  "l_index1", "l_index2", "l_index3", 
  "l_middle1", "l_middle2", "l_middle3",
  "l_ring1", "l_ring2", "l_ring3", 
  "l_pinky1", "l_pinky2", "l_pinky3", 
  "r_thumb1", "r_thumb2", "r_thumb3", 
  "r_index1", "r_index2", "r_index3", 
  "r_middle1", "r_middle2", "r_middle3",
  "r_ring1", "r_ring2", "r_ring3", 
  "r_pinky1", "r_pinky2", "r_pinky3"
};


/***************************************************************************/

class HAnimParser {
	FILE * parsedFile;					// handler of the file (recursive use)
	Transform * resultBody;				// the parsed resulted body
	Transform ** pointerTable;			// array of pointers to the joints

// Constructor and Destructor
public:
	HAnimParser(char * fileToParse, Executive * pExecutive);
	~HAnimParser();

// Accessers
public:
	inline FILE * getParsedFile() const { return parsedFile; }
	inline Transform * getResultBody() const { return resultBody; }
	inline Transform ** getPointerTable() const {return pointerTable; }

// Members
public:
	inline int parseJoints(Transform * father, Executive * pExecutive);
	inline int findJointNumberFromName(char * name, int * buffer);
};

#endif
