#ifndef _HEADER_OF_CLASS_SEGMENT_PARSER_
#define _HEADER_OF_CLASS_SEGMENT_PARSER_

#include <stdio.h>
#include <stdlib.h>
#include "defs.h"
#include "opengl/appearance/AppearanceProxy.h"
#include "opengl/appearance/MaterialProxy.h"
#include "opengl/group/TransformProxy.h"
#include "opengl/geometry/IndexedFaceSetProxy.h"
#include "opengl/geometry/ShapeProxy.h"
#include "Nodes.h"

#include "Vector3d.h"
#include "MyFace.h"

// local defines
#define SIZE_OF_PARSING_BUFFER 2000
#define MAX_NUMBER_OF_POINTS_PER_SEGMENT 10000
#define MAX_NUMBER_OF_NORMALS_PER_SEGMENT 10000
#define MAX_NUMBER_OF_NORMAL_INDEX_PER_SEGMENT 10000
#define MAX_NUMBER_OF_TRIANGLES_PER_SEGMENT 10000
#define MAX_NUMBER_OF_VERTICES_PER_FACE 30

// local additional segment transformation structure
typedef struct t_transformations {
  float *translation;
  float *rotation;
  float *scale;
  float *scaleorientation;
} MyTransformations;

/**************************************************************************/
class SegmentParser {

	// FILE to be parsed
	FILE * parsedFile;
	char * fileName;

	// points
	Vector3d ** points;
	int nOfPoints;

	// normals
	Vector3d ** normals;
	int nOfNormals;

	// faces
	MyFace ** faces;
	int nOfFaces;

	// normal index
	MyFace ** normalIndex;
	int nOfNormalIndex;

	// Additional segment informations
	BOOL ccw;
	float creaseAngle;

	// eventually a local transform (head+hands need it)
	MyTransformations * localSegmentTransform;

	// the buffer for parsing
	char buffer[SIZE_OF_PARSING_BUFFER];

	// these are for parsing faces with more than 3 indices
	int bufferOfIndices[MAX_NUMBER_OF_VERTICES_PER_FACE];
	int nbConsecutiveIndices;

// Constructor and destructor
public :
	SegmentParser(char * filename, Executive * pExecutive, Transform * p);
	~SegmentParser();

// Inlines Members
public :

	// Add a point to the list of coordinates
	inline void AddPoint(float x, float y, float z) {
		points[nOfPoints++]=new Vector3d(x,y,z);
	}

	// Add a face to the list of faces
	inline void AddFace(int * b, int nb) {
		faces[nOfFaces++]=new MyFace(b, nb);
	}

	// Add a normal to the list of normals
	inline void AddNormal(float x, float y, float z) {
		normals[nOfNormals++]= new Vector3d(x,y,z);
	}

	// Add a indexed normal to the list of indexedNormals
	inline void AddNormalIndex(int *b, int nb)
	{
		normalIndex[nOfNormalIndex++]= new MyFace(b,nb);
	}

// Members
public :

	// Useful functions for parsing
	char * RemoveCrap(char * s);            // removes spaces
	void GotoLine(char * s);                // go to line beginning with

	// Material related
	Material * ParseMaterial();             // parse a material in the segment file

	// Geometry related
	void ParseGeometry();
	Vector3d** CalculateFaceNormals();
	Vector3d** CalculateVerticesNormals();
	void ApplySegmentTransformations(Transform* pTransform);

	// Lowlevel Parsing functions
	void ParseEventualTransformations();    // parse local transf.
	void ParseCoordinates();                // parse coordinates
	void ParseFaces();                      // parse faces
	void ParseNormals();					// parse normals
	void ParseNormalIndex();				// parse normalIndex
	void ProcessNewBufferOfFaces(int * b, int nb); // as its name says
	void ProcessNewBufferOfNormalIndex(int * b, int nb); // as its name says

	// Node creation
	IndexedFaceSet * CreateIndexedFaceSetNodeAndItsProxy(Executive * pExecutive);
	Appearance * CreateAppearanceNodeAndItsProxy(Executive * pExecutive);
	Shape * CreateShapeNodeAndItsProxy(Executive * pExecutive);
	Transform * CreateTransformNodeAndItsProxy(Executive * pExecutive);
	void CreateMaterialProxy(Material * pMaterial, Executive * pExecutive);
};

#endif
