/********************************************************************************/
/* Vector3d : Simple implementation of a 3D Vector and its geometrical ops.     */
/* Could be replaced by Vec3f in the future...									*/
/* Fabrice Vergnenegre, 10/19/1999, LIG, EPFL                                   */
/********************************************************************************/

#ifndef _HEADER_OF_CLASS_VECTOR3D_
#define _HEADER_OF_CLASS_VECTOR3D_

#include "defs.h"

class Vector3d {
  float x;
  float y;
  float z;

// Constructors
public:
  Vector3d();
  Vector3d(float cx, float cy, float cz);
  Vector3d(Vector3d * p);
  Vector3d(const Vector3d & p);

// Accessers
public:
  inline float GetX() const { return x; }
  inline float GetY() const { return y; }
  inline float GetZ() const { return z; }

// Modifiers
public:
  inline void SetX(float nx) { x=nx; }
  inline void SetY(float ny) { y=ny; }
  inline void SetZ(float nz) { z=nz; }

// Geometric operators
public:
  BOOL operator == (const Vector3d & v) const ;			// vector equality
  float operator * (const Vector3d & v) const ;			// dot product
  const Vector3d & operator * (float a) const ;					// scalar product
  const Vector3d & operator / (float a) const ;					// scalar division
  const Vector3d & operator ^ (const Vector3d & v) const ;		// cross product
  const Vector3d & operator - (const Vector3d & v) const ;		// vector subtraction
  const Vector3d & operator + (const Vector3d & v) const ;		// vector addition
};

#endif
