/* INFO: This file contains skeleton and body pointer info */

/*------------------------------------------------------------------------------*/
/* 										*/
/* 										*/
/*  This software module was originally developed by Ronan Boulic and Tom Molet */
/*  (EPFL-LIG) and edited by Joaquim Esmerado (EPFL-LIG) 			*/
/*  in the course of development of the MPEG-4 SNHC 				*/
/*  VM. This software module is an implementation of a part of one or more 	*/
/*  MPEG-4 SNHC VM tools as specified by the MPEG-4 SNHC VM. ISO/IEC gives 	*/
/*  users of the MPEG-4 SNHC VM free license to this software module or 	*/
/*  modifications thereof for use in hardware or software products 		*/
/*  claiming conformance to the MPEG-4 SNHC VM. Those intending to use 		*/
/*  this software module in hardware or software products are advised that 	*/
/*  its use may infringe existing patents. The original developer of this 	*/
/*  software module and his/her company, the subsequent editors and their 	*/
/*  companies, and ISO/IEC have no liability for use of this software 		*/
/*  module or modifications thereof. Copyright is not released for non 		*/
/*  MPEG-4 SNHC VM conforming products. EPFL-LIG 				*/
/*  retains full right to use the code for its own purpose, assign or 		*/
/*  donate the code to a third party and to inhibit third parties from 		*/
/*  using the code for non MPEG-4 SNHC VM conforming products. This 		*/
/*  copyright notice must be included in all copies or derivative 		*/
/*  works. Copyright (1997).  							*/




#ifndef etall_h
#define etall_h


#ifdef __cplusplus
extern "C" {
#endif

/*
** -------------------------------------
** -- Public define, typedef, functions
** -------------------------------------
*/

#ifndef __cplusplus
#ifndef private
#define	private
#define private_
#endif
#endif

#ifndef rdmm_h
#define rdmm_h

typedef	float		RAD;	    /* radian (rd)	*/
typedef float		MM;	    /* millimeter (mm)	*/
typedef	RAD  		RAD_t;
typedef MM   		MM_t;

#endif



/*
** -------------------------------------
** -- Public define, typedef, functions
** -------------------------------------
*/

/* definition of SKELETON type */

typedef struct skeleton_ SKELETON ;
typedef struct skeleton_ * PSKELETON ;
typedef  SKELETON        SKELETON_t;
typedef  PSKELETON       PSKELETON_t;


/*--------------------------*/
/*	Creation	    */
/*--------------------------*/

extern SKELETON_t *  skeleton_creation(void);

/*--------------------------*/
/*	Destruction	    */
/*--------------------------*/

extern Booleen_t	    skeleton_destroy(SKELETON_t * self);

/*--------------------------*/
/*	Copy		    */
/*--------------------------*/

extern SKELETON_t *  skeleton_copy(const SKELETON_t * src, SKELETON_t * self);




/*
** -------------------------------------
** -- Public define, typedef, functions
** -------------------------------------
*/

/* general orientation convention    */

#define	BODY_Z_UP           0
#define	BODY_Y_UP           1

/* handling mode		    */

#define BODY_PUBLIC_MODE    0


/*  creation status */

#define BODY_INSTANCE	    0
#define BODY_TEMPLATE	    1


#define	BODY_DEFAULT_NB_TN3D	    90

#define	BODY_OPTION_NB_JOINT	    75

#define BODY_N3D_GLOBAL_MOTION	    0
#define	BODY_N3D_VIRTUAL_FLOOR	    1
#define	BODY_N3D_AVERAGE_POSITION   2   /*    Functional location  */

/* situation of the skeleton with respect to the
   average_position node_3D                                         */

#define	BODY_N3D_TR_VERTICAL		3
#define	BODY_N3D_TR_LATERAL		4
#define	BODY_N3D_TR_FRONTAL		5

#define BODY_N3D_BODY_TURN		6
#define BODY_N3D_BODY_ROLL		7
#define BODY_N3D_BODY_TILT		8

/* skeleton hierarchy		*/

#define BODY_N3D_SKELETON_ROOT	        9   /*    Functional location	*/

/* lower body 3D hierarchy	*/

#define BODY_N3D_PELVIC_TILT		10
#define BODY_N3D_PELVIC_TORSION		11
#define BODY_N3D_PELVIC_ROLL		12


/* right leg degrees of freedom		    */

#define BODY_N3D_R_HIP_ABDUCT		16
#define BODY_N3D_R_HIP_FLEXION		17
#define BODY_N3D_R_HIP_TWISTING		18
#define BODY_N3D_R_KNEE_FLEXION		19
#define BODY_N3D_R_KNEE_TWISTING	20
#define BODY_N3D_R_ANKLE_FLEXION	21
#define BODY_N3D_R_ANKLE_SUBTALAR     	22
#define BODY_N3D_R_TOE_FLEXION		23


/* left leg degrees of freedom		    */

#define BODY_N3D_L_HIP_ABDUCT		27
#define BODY_N3D_L_HIP_FLEXION		28
#define BODY_N3D_L_HIP_TWISTING		29
#define BODY_N3D_L_KNEE_FLEXION		30
#define BODY_N3D_L_KNEE_TWISTING	31
#define BODY_N3D_L_ANKLE_FLEXION	32
#define BODY_N3D_L_ANKLE_SUBTALAR       33
#define BODY_N3D_L_TOE_FLEXION		34


#define BODY_N3D_VL1_TILT	35	    /*	    L5	    */

#define BODY_N3D_VL2_ROLL	36	    /*	    L3	    */
#define BODY_N3D_VL2_TILT	37
 
#define BODY_N3D_VL3_ROLL	38	    /*	    L1	    */
#define BODY_N3D_VL3_TILT	39

/* Thoracic group of vertebra, less tilt    |			    */

#define BODY_N3D_VT4_ROLL	40	    /*	    T10	    */
#define BODY_N3D_VT4_TORSION	41
#define BODY_N3D_VT4_TILT	42

#define BODY_N3D_VT5_ROLL	43	    /*	    T6	    */
#define BODY_N3D_VT5_TORSION	44

#define BODY_N3D_VT6_ROLL	45	    /*	    T1	    */
#define BODY_N3D_VT6_TORSION	46
#define BODY_N3D_VT6_TILT	47

/* Cervical group of vertebra, quite free		    */

#define BODY_N3D_VC7_ROLL	48	    /*	    C4	    */
#define BODY_N3D_VC7_TORSION	49
#define BODY_N3D_VC7_TILT	50

#define BODY_N3D_VC8_ROLL	51	    /*	    C2	    */
#define BODY_N3D_VC8_TORSION	52
#define BODY_N3D_VC8_TILT	53

/* Head location and total_height			    */

#define BODY_N3D_HEAD		54	
#define BODY_N3D_TOTAL_HEIGHT	55	

/* right arm 3D hierarchy				    */

#define BODY_N3D_R_CLAV_ABDUCT		56
#define BODY_N3D_R_CLAV_ROTATE		57
#define BODY_N3D_R_SHOULDER_ABDUCT	58
#define BODY_N3D_R_SHOULDER_FLEXION	59
#define BODY_N3D_R_SHOULDER_TWISTING	60
#define BODY_N3D_R_ELBOW_FLEXION	61
#define BODY_N3D_R_ELBOW_TWISTING	62
#define BODY_N3D_R_WRIST_FLEXION	63
#define BODY_N3D_R_WRIST_PIVOT		64
#define BODY_N3D_R_HAND_CENTER		65 

/* left arm 3D hierarchy				    */

#define BODY_N3D_L_CLAV_ABDUCT		66
#define BODY_N3D_L_CLAV_ROTATE		67
#define BODY_N3D_L_SHOULDER_ABDUCT	68
#define BODY_N3D_L_SHOULDER_FLEXION	69
#define BODY_N3D_L_SHOULDER_TWISTING	70
#define BODY_N3D_L_ELBOW_FLEXION	71
#define BODY_N3D_L_ELBOW_TWISTING	72
#define BODY_N3D_L_WRIST_FLEXION	73
#define BODY_N3D_L_WRIST_PIVOT		74
#define BODY_N3D_L_HAND_CENTER		75 
#define BODY_N3D_FIG_VIRTUAL_FLOOR	76 /* one polygonal figure  */

#define BODY_N3D_L_MID_FOOT	        77
#define BODY_N3D_L_HEEL_EXTR	        78 
#define BODY_N3D_L_TOE_BASE	        79 
#define BODY_N3D_L_TOE_EXTR	        80 

#define BODY_N3D_R_MID_FOOT	        81
#define BODY_N3D_R_HEEL_EXTR	        82 
#define BODY_N3D_R_TOE_BASE	        83 
#define BODY_N3D_R_TOE_EXTR	        84 

#define BODY_N3D_L_SCAP_ABDUCT          85
#define BODY_N3D_L_SCAP_ROTATE          86

#define BODY_N3D_R_SCAP_ABDUCT          87
#define BODY_N3D_R_SCAP_ROTATE          88

	/*    Functional location   */
#define BODY_N3D_VISION		        89 

/*''''''''''''''''''''''''''''''''''''''''''''''''''''*/
#define BODY_JOINT_VIRTUAL_FLOOR        74
#define BODY_NAME_VIRTUAL_FLOOR     "virtual_floor"
#define BODY_NAME_FIG_VIRTUAL_FLOOR "fig_virtual_floor"

#define BODY_NAME_AVERAGE_POSITION  "average_position"
/*TO REMOVE*/
/*,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,*/


#define BODY_JOINT_TR_VERTICAL          0
#define BODY_JOINT_TR_LATERAL           1
#define BODY_JOINT_TR_FRONTAL           2

#define BODY_JOINT_BODY_TURN            3
#define BODY_JOINT_BODY_ROLL            4
#define BODY_JOINT_BODY_TILT            5

/* skeleton hierarchy					    */

/* lower body 3D hierarchy				    */

#define BODY_JOINT_PELVIC_TILT		6
#define BODY_JOINT_PELVIC_TORSION	7
#define BODY_JOINT_PELVIC_ROLL		8

/* right leg 3D hierarchy				    */


#define BODY_JOINT_R_HIP_ABDUCT		12
#define BODY_JOINT_R_HIP_FLEXION	13
#define BODY_JOINT_R_HIP_TWISTING	14
#define BODY_JOINT_R_KNEE_FLEXION	15
#define BODY_JOINT_R_KNEE_TWISTING	16
#define BODY_JOINT_R_ANKLE_FLEXION	17
#define BODY_JOINT_R_ANKLE_SUBTALAR	18
#define BODY_JOINT_R_TOE_FLEXION	19

/* left leg 3D hierarchy				    */


#define BODY_JOINT_L_HIP_ABDUCT		23
#define BODY_JOINT_L_HIP_FLEXION	24
#define BODY_JOINT_L_HIP_TWISTING	25
#define BODY_JOINT_L_KNEE_FLEXION	26
#define BODY_JOINT_L_KNEE_TWISTING	27
#define BODY_JOINT_L_ANKLE_FLEXION	28
#define BODY_JOINT_L_ANKLE_SUBTALAR	29
#define BODY_JOINT_L_TOE_FLEXION	30


#define BODY_JOINT_VL1_TILT	31	 /*          L5         */

#define BODY_JOINT_VL2_ROLL	32	 /*          L3		*/
#define BODY_JOINT_VL2_TILT	33

#define BODY_JOINT_VL3_ROLL	34	 /*	     L1		*/
#define BODY_JOINT_VL3_TILT	35

/* Thoracic group of vertebra, less tilt  |			*/

#define BODY_JOINT_VT4_ROLL	36	 /*	    T10		*/
#define BODY_JOINT_VT4_TORSION	37
#define BODY_JOINT_VT4_TILT	38

#define BODY_JOINT_VT5_ROLL	39	/*	    T6		*/
#define BODY_JOINT_VT5_TORSION	40

#define BODY_JOINT_VT6_ROLL	41	/*	    T1		*/
#define BODY_JOINT_VT6_TORSION	42
#define BODY_JOINT_VT6_TILT	43

/* Cervical group of vertebra, quite free			*/

#define BODY_JOINT_VC7_ROLL	44	/*	    C4		*/
#define BODY_JOINT_VC7_TORSION	45
#define BODY_JOINT_VC7_TILT	46

#define BODY_JOINT_VC8_ROLL	47	/*	    C2		*/
#define BODY_JOINT_VC8_TORSION	48
#define BODY_JOINT_VC8_TILT	49

/* right arm 3D hierarchy	*/

#define BODY_JOINT_R_CLAV_ABDUCT	50
#define BODY_JOINT_R_CLAV_ROTATE	51

#define BODY_JOINT_R_SHOULDER_ABDUCT	52
#define BODY_JOINT_R_SHOULDER_FLEXION   53
#define BODY_JOINT_R_SHOULDER_TWISTING  54
#define BODY_JOINT_R_ELBOW_FLEXION	55
#define BODY_JOINT_R_ELBOW_TWISTING     56
#define BODY_JOINT_R_WRIST_FLEXION      57
#define BODY_JOINT_R_WRIST_PIVOT        58

/* left arm 3D hierarchy	*/

#define BODY_JOINT_L_CLAV_ABDUCT	59
#define BODY_JOINT_L_CLAV_ROTATE	60

#define BODY_JOINT_L_SHOULDER_ABDUCT    61
#define BODY_JOINT_L_SHOULDER_FLEXION   62
#define BODY_JOINT_L_SHOULDER_TWISTING  63
#define BODY_JOINT_L_ELBOW_FLEXION      64
#define BODY_JOINT_L_ELBOW_TWISTING     65
#define BODY_JOINT_L_WRIST_FLEXION      66
#define BODY_JOINT_L_WRIST_PIVOT        67


#define BODY_JOINT_L_MID_FOOT	        68
#define BODY_JOINT_L_SCAP_ABDUCT        69
#define BODY_JOINT_L_SCAP_ROTATE        70

#define BODY_JOINT_R_MID_FOOT	        71
#define BODY_JOINT_R_SCAP_ABDUCT        72
#define BODY_JOINT_R_SCAP_ROTATE        73


/* NEW NODES DUE TO VRML2: */

#define	BODY_JOINT_SKULL_ROLL		74
#define BODY_JOINT_SKULL_TORSION	75
#define BODY_JOINT_SKULL_TILT		76




/*
** -------------------------------------
** -- Public define, typedef, functions
** -------------------------------------
*/

/********************************************************************/
/* entry pointer type on a BODY data structure			    */


typedef struct body   BODY;

typedef BODY          BODY_t;





/*--------------------------*/
/*	Creation	    */
/*--------------------------*/

extern BODY_t *	    body_insTance_creation(/*BODY_t * Template_ref,*/ SCENE_t * scene);


/*--------------------------*/
/*	Init.		    */
/*--------------------------*/

extern Booleen_t  body_reseT_in_default_position(BODY_t * self);

/*--------------------------*/
/*	Set		    */
/*--------------------------*/

extern Booleen_t	body_set_orientation(BODY_t * self, 
					     Booleen_t orientation);

/*--------------------------*/
/*	Get		    */
/*--------------------------*/

extern int	    body_get_orientation(BODY_t * self);


/*--------------------------*/
/*	Display		    */
/*--------------------------*/

extern Booleen_t      body_display_ref(BODY_t * instance);



float bodyChangeJointVal( BODY_t *body, int joint_ind, float new_val );
float bodyChangeRHandJointVal( BODY_t *body, int joint_ind, float new_val);
float bodyChangeLHandJointVal( BODY_t *body, int joint_ind, float new_val);

N3D *bodyGetN3D( BODY_t *body, int n3dInd);
N3D *bodyGetLHandN3D( BODY_t *body, int n3dInd);
N3D *bodyGetRHandN3D( BODY_t *body, int n3dInd);

/*
** -------------------------------------
** -- Public define, typedef, functions
** -------------------------------------
*/
#define	BODY_OPTION_NB_HAND	    36	    /* 36 Left & 36 Right */
#define	BODY_OPTION_NB_FOOT	    BODY_OPTION_NB_HAND

#define	BODY_OPTION_NB_J_HAND	    30	    /* 30 Left & 30 Right */
#define	BODY_OPTION_NB_J_FOOT       BODY_OPTION_NB_J_HAND

/*------------------------------------------------------------------*/
/* Optionnal sets of node_3D : left and right hands		    */
/*	motion N3D with a few functional location N3D		    */
/*------------------------------------------------------------------*/

/* left hand 3D hierarchy (optional)	*/

#define BODY_N3D_L_HAND_ROOT		0 

#define BODY_N3D_L_PINKY_FL1		2
#define BODY_N3D_L_PINKY_PIVOT		3
#define BODY_N3D_L_PINKY_TWIST		4
#define BODY_N3D_L_PINKY_FL2		5
#define BODY_N3D_L_PINKY_FL3		6
#define BODY_N3D_L_PINKY_EXTR		7 

#define BODY_N3D_L_RING_FL1		9
#define BODY_N3D_L_RING_PIVOT		10
#define BODY_N3D_L_RING_TWIST		11
#define BODY_N3D_L_RING_FL2		12
#define BODY_N3D_L_RING_FL3		13
#define BODY_N3D_L_RING_EXTR		14 

#define BODY_N3D_L_MIDDLE_FL1		16
#define BODY_N3D_L_MIDDLE_PIVOT		17
#define BODY_N3D_L_MIDDLE_TWIST		18
#define BODY_N3D_L_MIDDLE_FL2		19
#define BODY_N3D_L_MIDDLE_FL3		20
#define BODY_N3D_L_MIDDLE_EXTR		21 

#define BODY_N3D_L_INDEX_FL1		23
#define BODY_N3D_L_INDEX_PIVOT		24
#define BODY_N3D_L_INDEX_TWIST		25
#define BODY_N3D_L_INDEX_FL2		26
#define BODY_N3D_L_INDEX_FL3		27
#define BODY_N3D_L_INDEX_EXTR		28 

#define BODY_N3D_L_THUMB_TW		30
#define BODY_N3D_L_THUMB_ABDUCT		31
#define BODY_N3D_L_THUMB_FL1		32
#define BODY_N3D_L_THUMB_FL2		33
#define BODY_N3D_L_THUMB_FL3		34
#define BODY_N3D_L_THUMB_EXTR		35 

/* right hand 3D hierarchy (optional)	*/

#define BODY_N3D_R_HAND_ROOT		0 

#define BODY_N3D_R_PINKY_FL1		2
#define BODY_N3D_R_PINKY_PIVOT		3
#define BODY_N3D_R_PINKY_TWIST		4
#define BODY_N3D_R_PINKY_FL2		5
#define BODY_N3D_R_PINKY_FL3		6
#define BODY_N3D_R_PINKY_EXTR		7 

#define BODY_N3D_R_RING_FL1		9
#define BODY_N3D_R_RING_PIVOT		10
#define BODY_N3D_R_RING_TWIST		11
#define BODY_N3D_R_RING_FL2		12
#define BODY_N3D_R_RING_FL3		13
#define BODY_N3D_R_RING_EXTR		14 

#define BODY_N3D_R_MIDDLE_FL1		16
#define BODY_N3D_R_MIDDLE_PIVOT		17
#define BODY_N3D_R_MIDDLE_TWIST		18
#define BODY_N3D_R_MIDDLE_FL2		19
#define BODY_N3D_R_MIDDLE_FL3		20
#define BODY_N3D_R_MIDDLE_EXTR		21 

#define BODY_N3D_R_INDEX_FL1		23
#define BODY_N3D_R_INDEX_PIVOT		24
#define BODY_N3D_R_INDEX_TWIST		25
#define BODY_N3D_R_INDEX_FL2		26
#define BODY_N3D_R_INDEX_FL3		27
#define BODY_N3D_R_INDEX_EXTR		28 

#define BODY_N3D_R_THUMB_TW		30
#define BODY_N3D_R_THUMB_ABDUCT		31
#define BODY_N3D_R_THUMB_FL1		32
#define BODY_N3D_R_THUMB_FL2		33
#define BODY_N3D_R_THUMB_FL3		34
#define BODY_N3D_R_THUMB_EXTR		35 


/*------------------------------------------------------------------*/
/* Optionnal sets of node_3D : left and right foots		    */
/*	motion N3D with a few functional location N3D		    */
/*------------------------------------------------------------------*/

#define BODY_N3D_L_FOOT_ROOT		BODY_N3D_L_HAND_ROOT

#define BODY_N3D_R_FOOT_ROOT		BODY_N3D_R_HAND_ROOT



/*------------------------------------------------------------------*/
/* Optionnal sets of JOINT entry_point : left and right hands	    */
/*------------------------------------------------------------------*/




#define BODY_JOINT_L_METAC_PINKY_FL     0
#define BODY_JOINT_L_PINKY_FL1		1
#define BODY_JOINT_L_PINKY_PIVOT	2
#define BODY_JOINT_L_PINKY_TWIST	3
#define BODY_JOINT_L_PINKY_FL2		4
#define BODY_JOINT_L_PINKY_FL3		5
#define BODY_JOINT_L_METAC_RING_FL      6
#define BODY_JOINT_L_RING_FL1		7
#define BODY_JOINT_L_RING_PIVOT		8
#define BODY_JOINT_L_RING_TWIST		9
#define BODY_JOINT_L_RING_FL2		10
#define BODY_JOINT_L_RING_FL3		11
#define BODY_JOINT_L_METAC_MIDDLE_FL    12
#define BODY_JOINT_L_MIDDLE_FL1		13
#define BODY_JOINT_L_MIDDLE_PIVOT	14
#define BODY_JOINT_L_MIDDLE_TWIST	15
#define BODY_JOINT_L_MIDDLE_FL2		16
#define BODY_JOINT_L_MIDDLE_FL3		17
#define BODY_JOINT_L_METAC_INDEX_FL     18
#define BODY_JOINT_L_INDEX_FL1		19
#define BODY_JOINT_L_INDEX_PIVOT	20
#define BODY_JOINT_L_INDEX_TWIST	21
#define BODY_JOINT_L_INDEX_FL2		22
#define BODY_JOINT_L_INDEX_FL3		23
#define BODY_JOINT_L_THUMB_PIVOT        24
#define BODY_JOINT_L_THUMB_TW		25
#define BODY_JOINT_L_THUMB_ABDUCT	26
#define BODY_JOINT_L_THUMB_FL1		27
#define BODY_JOINT_L_THUMB_FL2		28
#define BODY_JOINT_L_THUMB_FL3		29



#define BODY_JOINT_R_METAC_PINKY_FL     0
#define BODY_JOINT_R_PINKY_FL1		1
#define BODY_JOINT_R_PINKY_PIVOT	2
#define BODY_JOINT_R_PINKY_TWIST	3
#define BODY_JOINT_R_PINKY_FL2		4
#define BODY_JOINT_R_PINKY_FL3		5
#define BODY_JOINT_R_METAC_RING_FL      6
#define BODY_JOINT_R_RING_FL1		7
#define BODY_JOINT_R_RING_PIVOT		8
#define BODY_JOINT_R_RING_TWIST		9
#define BODY_JOINT_R_RING_FL2		10
#define BODY_JOINT_R_RING_FL3		11
#define BODY_JOINT_R_METAC_MIDDLE_FL    12
#define BODY_JOINT_R_MIDDLE_FL1		13
#define BODY_JOINT_R_MIDDLE_PIVOT	14
#define BODY_JOINT_R_MIDDLE_TWIST	15
#define BODY_JOINT_R_MIDDLE_FL2		16
#define BODY_JOINT_R_MIDDLE_FL3		17
#define BODY_JOINT_R_METAC_INDEX_FL     18
#define BODY_JOINT_R_INDEX_FL1		19
#define BODY_JOINT_R_INDEX_PIVOT	20
#define BODY_JOINT_R_INDEX_TWIST	21
#define BODY_JOINT_R_INDEX_FL2		22
#define BODY_JOINT_R_INDEX_FL3		23
#define BODY_JOINT_R_THUMB_PIVOT        24
#define BODY_JOINT_R_THUMB_TW		25
#define BODY_JOINT_R_THUMB_ABDUCT	26
#define BODY_JOINT_R_THUMB_FL1		27
#define BODY_JOINT_R_THUMB_FL2		28
#define BODY_JOINT_R_THUMB_FL3		29


/* NEW NODES DUE TO VRML2: */

#define	BODY_JOINT_L_PINKY_FL0		30
#define	BODY_JOINT_R_PINKY_FL0		30

#define BODY_JOINT_L_RING_FL0           31
#define BODY_JOINT_R_RING_FL0           31

#define BODY_JOINT_L_MIDDLE_FL0         32
#define BODY_JOINT_R_MIDDLE_FL0         32

#define BODY_JOINT_L_INDEX_FL0          33
#define BODY_JOINT_R_INDEX_FL0          33



/*
** -------------------------------------
** -- Set ------------------------------
** -------------------------------------
*/
extern Booleen_t	body_set_left_hand(BODY_t * self, Booleen_t activ);
extern Booleen_t	body_set_right_hand(BODY_t * self, Booleen_t activ);



/*
** -------------------------------------
** -- Manipulation ---------------------
** -------------------------------------
*/

extern	Booleen_t	    body_skeleton_hand_fit(BODY_t * instance);


/*
** -------------------------------------
** -- Public define, typedef, functions
** -------------------------------------
*/
#define BODY_NAME_L_HAND_ROOT		    "l_hand_root"


#define BODY_NAME_L_PINKY_FL1		    "l_pinky_fl1"
#define BODY_NAME_L_PINKY_PIVOT		    "l_pinky_pivot"
#define BODY_NAME_L_PINKY_TWIST		    "l_pinky_tw"
#define BODY_NAME_L_PINKY_FL2		    "l_pinky_fl2"
#define BODY_NAME_L_PINKY_FL3		    "l_pinky_fl3"
#define BODY_NAME_L_PINKY_EXTR		    "l_pinky_extr"

#define BODY_NAME_L_RING_FL1		    "l_ring_fl1"
#define BODY_NAME_L_RING_PIVOT		    "l_ring_pivot"
#define BODY_NAME_L_RING_TWIST		    "l_ring_tw"
#define BODY_NAME_L_RING_FL2		    "l_ring_fl2"
#define BODY_NAME_L_RING_FL3		    "l_ring_fl3"
#define BODY_NAME_L_RING_EXTR		    "l_ring_extr"

#define BODY_NAME_L_MIDDLE_FL1		    "l_middle_fl1"
#define BODY_NAME_L_MIDDLE_PIVOT	    "l_middle_pivot"
#define BODY_NAME_L_MIDDLE_TWIST	    "l_middle_tw"
#define BODY_NAME_L_MIDDLE_FL2		    "l_middle_fl2"
#define BODY_NAME_L_MIDDLE_FL3		    "l_middle_fl3"
#define BODY_NAME_L_MIDDLE_EXTR		    "l_middle_extr"

#define BODY_NAME_L_INDEX_FL1		    "l_index_fl1"
#define BODY_NAME_L_INDEX_PIVOT		    "l_index_pivot"
#define BODY_NAME_L_INDEX_TWIST		    "l_index_tw"
#define BODY_NAME_L_INDEX_FL2		    "l_index_fl2"
#define BODY_NAME_L_INDEX_FL3		    "l_index_fl3"
#define BODY_NAME_L_INDEX_EXTR		    "l_index_extr"

#define BODY_NAME_L_THUMB_TW		    "l_thumb_tw"
#define BODY_NAME_L_THUMB_ABDUCT	    "l_thumb_abd"
#define BODY_NAME_L_THUMB_FL1		    "l_thumb_fl1"
#define BODY_NAME_L_THUMB_FL2		    "l_thumb_fl2"
#define BODY_NAME_L_THUMB_FL3		    "l_thumb_fl3"
#define BODY_NAME_L_THUMB_EXTR		    "l_thumb_extr"

#define BODY_NAME_R_HAND_ROOT		    "r_hand_root"


#define BODY_NAME_R_PINKY_FL1		    "r_pinky_fl1"
#define BODY_NAME_R_PINKY_PIVOT		    "r_pinky_pivot"
#define BODY_NAME_R_PINKY_TWIST		    "r_pinky_tw"
#define BODY_NAME_R_PINKY_FL2		    "r_pinky_fl2"
#define BODY_NAME_R_PINKY_FL3		    "r_pinky_fl3"
#define BODY_NAME_R_PINKY_EXTR		    "r_pinky_extr"

#define BODY_NAME_R_RING_FL1		    "r_ring_fl1"
#define BODY_NAME_R_RING_PIVOT		    "r_ring_pivot"
#define BODY_NAME_R_RING_TWIST		    "r_ring_tw"
#define BODY_NAME_R_RING_FL2		    "r_ring_fl2"
#define BODY_NAME_R_RING_FL3		    "r_ring_fl3"
#define BODY_NAME_R_RING_EXTR		    "r_ring_extr"

#define BODY_NAME_R_MIDDLE_FL1		    "r_middle_fl1"
#define BODY_NAME_R_MIDDLE_PIVOT	    "r_middle_pivot"
#define BODY_NAME_R_MIDDLE_TWIST	    "r_middle_tw"
#define BODY_NAME_R_MIDDLE_FL2		    "r_middle_fl2"
#define BODY_NAME_R_MIDDLE_FL3		    "r_middle_fl3"
#define BODY_NAME_R_MIDDLE_EXTR		    "r_middle_extr"

#define BODY_NAME_R_INDEX_FL1		    "r_index_fl1"
#define BODY_NAME_R_INDEX_PIVOT		    "r_index_pivot"
#define BODY_NAME_R_INDEX_TWIST		    "r_index_tw"
#define BODY_NAME_R_INDEX_FL2		    "r_index_fl2"
#define BODY_NAME_R_INDEX_FL3		    "r_index_fl3"
#define BODY_NAME_R_INDEX_EXTR		    "r_index_extr"

#define BODY_NAME_R_THUMB_TW		    "r_thumb_tw"
#define BODY_NAME_R_THUMB_ABDUCT	    "r_thumb_abd"
#define BODY_NAME_R_THUMB_FL1		    "r_thumb_fl1"
#define BODY_NAME_R_THUMB_FL2		    "r_thumb_fl2"
#define BODY_NAME_R_THUMB_FL3		    "r_thumb_fl3"
#define BODY_NAME_R_THUMB_EXTR		    "r_thumb_extr"



/*
** -------------------------------------
** -- Manipulation ---------------------
** -------------------------------------
*/
extern	void	body_init_fix_ini_lhand(BODY_t * self);
extern	void	body_init_fix_ini_rhand(BODY_t * self);


extern	void	body_init_joint_rhand_limits(BODY_t * self);







/*
** -------------------------------------
** -- Public define, typedef, functions
** -------------------------------------
*/


#define	BODY_NAME_GLOBAL_MOTION	    "global_motion"

#define BODY_NAME_TR_VERTICAL       "tr_vertical"
#define BODY_NAME_TR_LATERAL        "tr_lateral"
#define BODY_NAME_TR_FRONTAL        "tr_frontal"

#define BODY_NAME_BODY_TURN         "rt_body_turn"
#define BODY_NAME_BODY_ROLL         "rt_body_roll"
#define BODY_NAME_BODY_TILT         "rt_body_tilt"

#define BODY_NAME_SKELETON_ROOT	    "skeleton_root"

#define BODY_NAME_PELVIC_TILT	    "pelvic_tilt"
#define BODY_NAME_PELVIC_TORSION    "pelvic_torsion"
#define BODY_NAME_PELVIC_ROLL	    "pelvic_roll"


#define BODY_NAME_R_HIP_ABDUCT	    "r_hip_abduct"
#define BODY_NAME_R_HIP_FLEXION	    "r_hip_flexion"
#define BODY_NAME_R_HIP_TWISTING    "r_hip_twisting"
#define BODY_NAME_R_KNEE_FLEXION    "r_knee_flexion"
#define BODY_NAME_R_KNEE_TWISTING   "r_knee_twisting"
#define BODY_NAME_R_ANKLE_FLEXION   "r_ankle_flexion"
#define BODY_NAME_R_ANKLE_SUBTALAR  "r_ankle_subtalar"
#define BODY_NAME_R_MID_FOOT        "r_mid_foot"

#define BODY_NAME_R_TOE_FLEXION	    "r_toe_flexion"


#define BODY_NAME_L_HIP_ABDUCT	    "l_hip_abduct"
#define BODY_NAME_L_HIP_FLEXION	    "l_hip_flexion"
#define BODY_NAME_L_HIP_TWISTING    "l_hip_twisting"
#define BODY_NAME_L_KNEE_FLEXION    "l_knee_flexion"
#define BODY_NAME_L_KNEE_TWISTING   "l_knee_twisting"
#define BODY_NAME_L_ANKLE_FLEXION   "l_ankle_flexion"
#define BODY_NAME_L_ANKLE_SUBTALAR  "l_ankle_subtalar"
#define BODY_NAME_L_MID_FOOT        "l_mid_foot"

#define BODY_NAME_L_TOE_FLEXION	    "l_toe_flexion"

#define BODY_NAME_VL1_TILT	    "vl1_tilt"

#define BODY_NAME_VL2_ROLL	    "vl2_roll"
#define BODY_NAME_VL2_TILT	    "vl2_tilt"
 
#define BODY_NAME_VL3_ROLL	    "vl3_roll"
#define BODY_NAME_VL3_TILT	    "vl3_tilt"

#define BODY_NAME_VT4_ROLL	    "vt4_roll"
#define BODY_NAME_VT4_TORSION	    "vt4_torsion"
#define BODY_NAME_VT4_TILT	    "vt4_tilt"

#define BODY_NAME_VT5_ROLL	    "vt5_roll"
#define BODY_NAME_VT5_TORSION	    "vt5_torsion"

#define BODY_NAME_VT6_ROLL	    "vt6_roll"
#define BODY_NAME_VT6_TORSION	    "vt6_torsion"
#define BODY_NAME_VT6_TILT	    "vt6_tilt"

#define BODY_NAME_VC7_ROLL	    "vc7_roll"
#define BODY_NAME_VC7_TORSION	    "vc7_torsion"
#define BODY_NAME_VC7_TILT	    "vc7_tilt"

#define BODY_NAME_VC8_ROLL	    "vc8_roll"
#define BODY_NAME_VC8_TORSION	    "vc8_torsion"
#define BODY_NAME_VC8_TILT	    "vc8_tilt"

#define BODY_NAME_HEAD		    "head"
#define BODY_NAME_VISION	    "vision"
#define BODY_NAME_TOTAL_HEIGHT	    "total_height"

#define BODY_NAME_L_CLAV_ABDUCT		    "l_clav_abduct"
#define BODY_NAME_L_CLAV_ROTATE		    "l_clav_rotate"
#define BODY_NAME_L_SHOULDER_ABDUCT	    "l_shoulder_abduct"
#define BODY_NAME_L_SCAP_ABDUCT             "l_scap_abduct"
#define BODY_NAME_L_SCAP_ROTATE             "l_scap_rotate"
#define BODY_NAME_L_SHOULDER_FLEXION	    "l_shoulder_flexion"
#define BODY_NAME_L_SHOULDER_TWISTING	    "l_shoulder_twisting"
#define BODY_NAME_L_ELBOW_FLEXION	    "l_elbow_flexion"
#define BODY_NAME_L_ELBOW_TWISTING	    "l_elbow_twisting"
#define BODY_NAME_L_WRIST_FLEXION	    "l_wrist_flexion"
#define BODY_NAME_L_WRIST_PIVOT		    "l_wrist_pivot"
#define BODY_NAME_L_HAND_CENTER		    "l_hand_center"


#define BODY_NAME_R_CLAV_ABDUCT		    "r_clav_abduct"
#define BODY_NAME_R_CLAV_ROTATE		    "r_clav_rotate"
#define BODY_NAME_R_SCAP_ABDUCT             "r_scap_abduct"
#define BODY_NAME_R_SCAP_ROTATE             "r_scap_rotate"
#define BODY_NAME_R_SHOULDER_ABDUCT	    "r_shoulder_abduct"
#define BODY_NAME_R_SHOULDER_FLEXION	    "r_shoulder_flexion"
#define BODY_NAME_R_SHOULDER_TWISTING	    "r_shoulder_twisting"
#define BODY_NAME_R_ELBOW_FLEXION	    "r_elbow_flexion"
#define BODY_NAME_R_ELBOW_TWISTING	    "r_elbow_twisting"
#define BODY_NAME_R_WRIST_FLEXION	    "r_wrist_flexion"
#define BODY_NAME_R_WRIST_PIVOT		    "r_wrist_pivot"
#define BODY_NAME_R_HAND_CENTER		    "r_hand_center"




/*--------------------------*/
/*	Manipulation	    */
/*--------------------------*/

extern  MM_t	body_find_total_height(BODY_t * self);





typedef struct Hand_	HAND;
typedef struct Hand_ *	PHAND;
typedef HAND	FOOT;
typedef HAND *	PFOOT;


typedef HAND	HAND_t;
typedef PHAND	PHAND_t;
typedef FOOT    FOOT_t;
typedef PFOOT   PFOOT_t;





/**	LINE 258 (for line correspondance with lucy struct) **/

/*--------------------------*/
/*	Get		    */
/*--------------------------*/

extern Booleen_t	skeleton_get_spine_origin(const SKELETON_t * self, 
					MM_t * spine_origin);

extern  HAND_t *  hand_copy(HAND_t * src, HAND_t * dst);
extern  FOOT_t *  foot_copy(FOOT_t * src, FOOT_t * dst);



typedef	void *	    TABLE;
typedef TABLE *	    PTABLE;
typedef	TABLE	    TABLE_t;
typedef PTABLE	    PTABLE_t;


/*--------------------------*/
/*	Creation	    */
/*--------------------------*/

extern TABLE_t *	    table_alloc(int size);

extern TABLE_t *	    table_realloc(TABLE_t * self, int size);

/*--------------------------*/
/*	Destruction	    */
/*--------------------------*/

extern Booleen_t	    table_free(TABLE_t * self);
extern Booleen_t	    table_free_n3d(TABLE_t * self, int size);

#ifdef __cplusplus
}
#endif

#ifdef VRML2_OUTPUT
#define VRML2_MAX_NUMFRAMES 2048
#define VRML2_NUM_BAPS 186
extern int vrml2_noframes;
extern float vrml2_bapvalues[VRML2_MAX_NUMFRAMES][VRML2_NUM_BAPS];
#endif

#endif
