
/*------------------------------------------------------------------------------*/
/* 										*/
/* 										*/
/*  This software module was originally developed by Ronan Boulic and Tom Molet */
/*  (EPFL-LIG) and edited by Joaquim Esmerado (EPFL-LIG) 			*/
/*  in the course of development of the MPEG-4 SNHC 				*/
/*  VM. This software module is an implementation of a part of one or more 	*/
/*  MPEG-4 SNHC VM tools as specified by the MPEG-4 SNHC VM. ISO/IEC gives 	*/
/*  users of the MPEG-4 SNHC VM free license to this software module or 	*/
/*  modifications thereof for use in hardware or software products 		*/
/*  claiming conformance to the MPEG-4 SNHC VM. Those intending to use 		*/
/*  this software module in hardware or software products are advised that 	*/
/*  its use may infringe existing patents. The original developer of this 	*/
/*  software module and his/her company, the subsequent editors and their 	*/
/*  companies, and ISO/IEC have no liability for use of this software 		*/
/*  module or modifications thereof. Copyright is not released for non 		*/
/*  MPEG-4 SNHC VM conforming products. EPFL-LIG 				*/
/*  retains full right to use the code for its own purpose, assign or 		*/
/*  donate the code to a third party and to inhibit third parties from 		*/
/*  using the code for non MPEG-4 SNHC VM conforming products. This 		*/
/*  copyright notice must be included in all copies or derivative 		*/
/*  works. Copyright (1997).  							*/






#include "scn1.h"


static double MM_PI = 3.14159265358979323846;

/* structure retaining permanent properties of node_3D			*/

typedef	struct N3D_properties  PROP_N3D ;
struct N3D_properties
{
      Booleen_t   internal; 
      Booleen_t   local;   
      Booleen_t   share;   
      Booleen_t   depend;  
};


static	PROP_N3D    n3d_prop[] ={
  {1, 0, 0, 0},  /*  N3D_NEUTRAL */
  {1, 0, 0, 0},  /*  N3D_WORLD */
  {1, 1, 0, 0},  /*  N3D_JOINT */
  {1, 1, 0, 1},  /*  N3D_FREE */
};

/* private macro to access general characteristics of node_3D		*/

#define	  N3D_get_internal(self)    (n3d_prop[self->type].internal)
#define	  N3D_get_local(self)	    (n3d_prop[self->type].local)
#define	  N3D_get_share(self)	    (n3d_prop[self->type].share)
#define	  N3D_get_depend(self)      (n3d_prop[self->type].depend)


/* static functions	*/

static	void	        n3d_ini(N3D_t *self, UNIT_t *unit, char *idt);

static	N3D_t ** n3d_dupl_lstc(N3D_t *self);

static	Booleen_t n3d_local_ref_refparent_type(N3D_t *self);
static	Booleen_t n3d_local_ref_refchild_type(N3D_t  *self, N3D_t *child);

static	Booleen_t n3d_copy_ref(N3D_t *src, N3D_t *dst); 




/********************/
/*     Creation     */



N3D_t  *n3d_creation(UNIT_t *unit, char *idt)
{
  N3D_t  *self ;
  WORLD_t	*world ;

  if(idt == NULL)  

  {
    fprintf(stderr, "Bad N3D_t name (NULL string)\n"); 
    return(NULL);
  }
	
  if(unit != NULL)
  {

    self = (N3D*) malloc((size_t) sizeof(N3D));
    if(self == NULL) {printf("Memory allocation error\n"); exit(0);}
	
    n3d_ini(self, unit, idt);

    if((world = n3d_get_world(self)) == NULL)
    { WORLD_UNDEFINED("n3d_creation\n"); exit(0);}

    self->idi  = world_new_n3d(world,(void*)self); 

    n3d_set_name(self, idt);
    return(self);
  }
  UNIT_UNDEFINED("n3d creation")
    return(NULL);
}



N3D_t  *n3d_creation_child(N3D_t  *parent, char *idt)
{
  N3D_t  *self , *neutre;

  if(idt == NULL)  
  {
    fprintf(stderr, "Bad N3D name (NULL string)\n"); 
    return(NULL);
  }
	
  if(parent != NULL)
  {
    if((self = n3d_creation(parent->unit, idt)) != NULL)
    {
      /* propagation de certaines caracteristiques du noeud parent*/

      dit_copy(parent->ref,self->ref);


      /* propagation de la coherence de l'internal	*/

      if(N3D_get_internal(parent) != NULL)
      {
	self->parent = parent ;
	n3d_add_child(parent,self);
      }
      else
      {
	printf("warning : a NEUTRAL node is added as parent of the new node\n");

      }
      return(self);
    }
    N3D_UNDEFINED("for parent node in n3d_creation_child")
      return(NULL);
  }
  N3D_UNDEFINED("n3d_creation_child")
    return(NULL);
}


/********************/
/*  Destruction     */



Booleen_t n3d_destroy(N3D_t *self)
{
  WORLD_t	*world ;
  N3D_t  **old_lstc, *world_n3d;
  int	old_nbc , i ;
  Booleen_t no_more_sharing ;

  if(self != NULL)
  {
    world_n3d = n3d_get_world_root(self) ;
    if(world_n3d != NULL)	    world = (WORLD*)world_n3d->str ;
    else		    world = NULL ;

    if(self == world_n3d)
    {
      if(self->nbc >0)
      { 
	printf("warning : destruction of world %s is forbidden \n", 
	       self->idt); 
	printf("  as long as its root node3D holds children node3D\n");
	return(FALSE);
      }
      else
      {
	    
	if(!unit_clear_root( n3d_get_unit(self) ))
	{
	  (void) fprintf(stderr, "n3d_destroy(): unit_clear_root() error\n");
	  return(FALSE);
	}
      }
    }
    else if(self == n3d_get_unit_root(self))
    { 
	    
	    
      if(self->nbc > 0)
      {
	printf("warning : destruction of unit root %s is forbidden\n",
	       self->idt);
	printf("  as long as its root node3D holds children node3D\n");
	return(FALSE);
      }
	    
      if(! unit_clear_root( n3d_get_unit(self)))
      {
	(void) fprintf(stderr, "n3d_destroy(): unit_clear_root() error\n");
	return(FALSE);
      }
    }

    if(self->nbc > 0)
    {
      old_lstc = n3d_dupl_lstc(self) ;
      old_nbc = self->nbc ;


      free(old_lstc);
    }

    self->ini = dit_destroy_ini(self->ini, self->tpini);


    n3d_destroy_data(self);	      

	  

    if(self->parent != NULL)
       n3d_remove_child(self->parent,self);
	  
	  
    free(self);

    return(TRUE);
  }
  N3D_UNDEFINED("n3d_destroy");
  return(FALSE);
}



Booleen_t	n3d_destroy_data(N3D_t *self)
{
  int	dependancy_index ;

  if(self  != NULL)
  { 
    n3d_set_type(self, N3D_NEUTRAL);
    self->str    = NULL ;
    return(TRUE);
  }
  N3D_UNDEFINED("destroy data");
  return(FALSE);
}


/********************/
/*	 Copy       */


Booleen_t n3d_set_name(N3D_t *self, char *name)
{
  static	char prefix[100];
  char	*final_idt ;

  if(self != NULL)
  {
    final_idt = NULL ;
    STR_CPY(final_idt,name);
    sprintf(prefix, "%d\0", n3d_get_id(self));
    while(h3d_exist_name(self, final_idt))
    {
      STR_POST(final_idt, prefix);
    }
    SET_IDT(self,final_idt);
    if(final_idt) free(final_idt);

    return(TRUE);
  }
  N3D_UNDEFINED("set name");
  return(FALSE);
}


/*------------------------------------------------------------------*/
Booleen_t	n3d_set_ini(N3D_t *self, short tpini, DIT_t *ini)
{
  if(self  != NULL)
  {
    self->refok = self->iniok = FALSE;
    if(self->tpini != tpini)
    {
      if(tpini == Z_OTHER)  /* going to Z_OTHER type	      */
      {
	self->ini = dit_edit_ini(self->ini, &(self->tpini));
	if(ini != NULL)	dit_copy(ini, self->ini);
	else	dit_copy(dit_let(IDENTITY), self->ini);
      }
      else		    /* going to predefined init type  */
      {
	if(self->tpini == Z_OTHER) dit_destroy_ini(self->ini,Z_OTHER);
	self->tpini = tpini ;
	self->ini = dit_creation_ini(tpini);
      }
    }
    else
    {
      if(tpini == Z_OTHER)
      {
	if(ini != NULL)	dit_copy(ini, self->ini);	
	else	dit_copy(dit_let(IDENTITY), self->ini);
      }
    }
    return(TRUE);
  }
  N3D_UNDEFINED("n3d_set_ini")
    return(FALSE);
}

/*------------------------------------------------------------------*/
Booleen_t	n3d_set_ini_dir(N3D_t *self, MatriX_t dir)
{
/*char *nd_name = n3d_get_local_name( self );*/
  if(self  != NULL)
  {

    self->refok = self->iniok = FALSE;
    if(self->tpini != Z_OTHER)	    
       /* change tpini to Z_OTHER */
       n3d_set_ini(self, Z_OTHER, NULL);
    if(self->tpini == Z_OTHER)
    {
      if(dir != NULL)
      { 
	matrix_load(self->ini, SENS_DIRECT, dir);
	matrix_inverse(self->ini->dir, self->ini->inv);
      }
      else dit_copy(dit_let(IDENTITY), self->ini);
      return(TRUE);
    }
	    
    printf("warning : change initial DIT type to Z_OTHER\n");
    printf("          prior to set internal matrices\n");
    return(FALSE);
  }
  N3D_UNDEFINED("n3d_set_ini_dir")
    return(FALSE);
}


Booleen_t	n3d_outdate_ref(N3D_t *self) 
{
  if(self  != NULL)
  {
    self->refok = FALSE;
    return(TRUE);
  }
  N3D_UNDEFINED("n3d outdate ref")
    return(FALSE);
}







UNIT	*n3d_get_unit(N3D_t *self)
{	if(self  != NULL)		    return(self->unit);
N3D_UNDEFINED("get unit");  return(NULL);
}

/*---------------------------------------------------------------------*/
N3D_t  *n3d_get_unit_root(N3D_t *self)
{	if(self  != NULL)			return(unit_get_root(self->unit));
N3D_UNDEFINED("get unit root"); return(NULL);
}

/*---------------------------------------------------------------------*/
N3D_t  *n3d_get_world_root(N3D_t *self)
{	if(self  != NULL)	    return(unit_get_scene_root(self->unit));
N3D_UNDEFINED("get world root");  return(NULL);
}

/*---------------------------------------------------------------------*/
WORLD_t	*n3d_get_world(N3D_t *self)
{	if(self  != NULL)	    return(unit_get_world(self->unit));
N3D_UNDEFINED("get world");  return(NULL);
}

/*---------------------------------------------------------------------*/
N3D_t  *n3d_get_parent(N3D_t *self)
{	if(self  != NULL)	    return(self->parent);
N3D_UNDEFINED("get parent");  return(NULL);
}

/*---------------------------------------------------------------------*/
int	n3d_get_nb_children(N3D_t *self)
{	if(self  != NULL)			return(self->nbc);
N3D_UNDEFINED("nb_children");	return(0);
}

/*---------------------------------------------------------------------*/
N3D_t  *n3d_get_child(N3D_t *self, int child_index)
{
  if(self != NULL && self->lstc != NULL)
  {
    if((child_index >= 0) && (child_index < self->nbc))
       return(self->lstc[child_index]);
    printf("warning : bad node_3D child index\n");
    return(NULL);
  }
  N3D_UNDEFINED("get child");  
  return(NULL);
}


/*---------------------------------------------------------------------*/
short	n3d_get_type(N3D_t *self)
{	if(self  != NULL)		return(self->type);
N3D_UNDEFINED("get ");  return(0);
}

Booleen_t	n3d_get_type_name(N3D_t * self, char name[])
{
  return(n3dplug_get_type_name(self, name));
}

/*---------------------------------------------------------------------*/

void	*n3d_get_data(N3D_t *self)
{	if(self  != NULL)		    return(self->str);
N3D_UNDEFINED("get data");  return(NULL);
}

/*---------------------------------------------------------------------*/
int	n3d_get_id(N3D_t *self)
{	if(self  != NULL)			  return(self->idi);
N3D_UNDEFINED("get integer id");  return(ID_ERROR);
}


/*---------------------------------------------------------------------*/
/* danger ! of accumulating memory ; name provide a pointer to a char  */
/* string, this string is first liberated and then allocate space for  */
/* the type name. After use this string space should be liberated expli*/
/* citely by the user... If this mechanism  is not provide the user    */
/* should provide a fixed lenght string pointer...		       */

char *	n3d_get_local_name(N3D_t *self) 
{
  char * name = NULL;

  if(self  != NULL)
  {
    STR_CPY(name, self->idt);  return(name);
  }
  N3D_UNDEFINED("get local name");  return(NULL);
}






N3D_t  *n3d_get_left(N3D_t *self)
{
  int	i;

  if(self  != NULL)
  {
    if(self->parent != NULL)
    {
      for(i=0 ; i< self->parent->nbc ; i++)
	 if(self->parent->lstc[i] == self) break;
      if(i== 0)  return(self->parent->lstc[self->parent->nbc -1]);
      else       return(self->parent->lstc[i-1]);
    }
    else return(self);
  }
  N3D_UNDEFINED("get left");
  return(NULL);
}





N3D_t  *n3d_get_down(N3D_t *self)
{
  if(self  != NULL)
  {
    if(self->nbc >0) return(self->lstc[0]);
    else		   return(self);
  }
  N3D_UNDEFINED("get down");
  return(NULL);
}

/*-------------------------------------------------------------------*/
Booleen_t n3d_get_ini(N3D_t *self, DIT_t *ini)
{	if(self != NULL && ini != NULL)
{ dit_copy(self->ini, ini);	return(TRUE);}
N3D_UNDEFINED("get ini DIT");   return(FALSE);
}

/*-------------------------------------------------------------------*/
Booleen_t n3d_get_ini_dir(N3D_t *self, MatriX_t dir)
{	if(self != NULL && dir != NULL)	
{ matrix_copy(self->ini->dir, dir);	return(TRUE);}
N3D_UNDEFINED("get ini direct matrix");	return(FALSE);
}

/*-------------------------------------------------------------------*/
Booleen_t n3d_get_ini_inv(N3D_t *self, MatriX_t inv)
{	if(self != NULL && inv != NULL)	
{ matrix_copy(self->ini->inv, inv);	 return(TRUE);}
N3D_UNDEFINED("get ini inverse matrix"); return(FALSE);
}

/*-------------------------------------------------------------------*/
Booleen_t n3d_ref_is_todate(N3D_t *self)
{	if(self  != NULL)			 return(self->refok);
N3D_UNDEFINED("ref is todate");  return(FALSE);
}

/*-------------------------------------------------------------------*/
Booleen_t n3d_get_ref_dir(N3D_t *self, MatriX_t dir)
{	if(self != NULL && dir != NULL)	
{ matrix_copy(self->ref->dir, dir);	return(TRUE);}
N3D_UNDEFINED("get ref direct matrix");	return(FALSE);
}

/*-------------------------------------------------------------------*/
Booleen_t n3d_get_ref_inv(N3D_t *self, MatriX_t inv)
{	if(self != NULL && inv != NULL)
{ matrix_copy(self->ref->inv, inv);	 return(TRUE);}
N3D_UNDEFINED("get ref inverse matrix"); return(FALSE);
}

/*-------------------------------------------------------------------*/







			/********************/
			/*  Manipulation    */
			/********************/


Booleen_t	  n3d_attach_data( N3D_t  *self, short   type, void *data)
{
  if(self  != NULL)
  {
    if(self->str == NULL)
    {
      if(!(n3d_prop[type].internal) && self->nbc > 0)
      {
	printf("warning : node is holding child \n");
	printf("          cannot be set to an external type\n");
	return(NO);
      }
      if(data != NULL)
      { 
	if (n3dplug_get_n3d(data, type) != NULL)
	{
	  printf("n3d_attach_data(): data already attached to an N3D\n");
	  return FALSE;
	}
	      
	n3d_set_type(self, type);
	self->str  = data ;
	return(n3dplug_attach_data(self));
      }
	    
      POINTER_UNDEFINED("void","n3d attach data");
      return(FALSE);
    }
    printf("warning : node already holds a data structure\n");
    return(FALSE);
  }
  N3D_UNDEFINED("attach data")
    return(FALSE);
}

/*-------------------------------------------------------------------*/
/* return TRUE if self belong to the family line of ancestor (same unit)*/

Booleen_t	n3d_belong_to_family(N3D_t *self, N3D_t *ancestor)
{
  N3D_t * tself;
    
  if(self != NULL && ancestor != NULL)
  {
    if(self->unit == ancestor->unit)
    {
      tself = self;
      while(tself->parent)
      {
	if(tself->parent == ancestor) return(TRUE);
	tself = tself->parent;
      }
      return(FALSE);
    }
    printf("warning : both node_3D should belong to the same UNIT\n");
  }
  N3D_UNDEFINED("belong to family");
  return(FALSE);
}
/*-------------------------------------------------------------------*/
/* return TRUE if self is a child of parent (same unit)		     */

Booleen_t	n3d_is_child(N3D_t *self, N3D_t *parent, short *index)
{
  int i, nbc;

  if(self != NULL && parent != NULL)
  {
    if(self->unit == parent->unit)
    {
      for(nbc=parent->nbc, i=0 ; i<nbc ;i++)
	 if(parent->lstc[i] == self)
	 { if(index != NULL) *index = (short) i; 
	 return(TRUE);
	 }
      return(FALSE);
    }
    printf("warning : both node_3D should belong to the same UNIT\n");
  }
  N3D_UNDEFINED("is child");
  return(FALSE);
}
/*-------------------------------------------------------------------*/
/* return TRUE if self is the parent of child (same unit)	     */

Booleen_t	n3d_is_parent(N3D_t *self, N3D_t *child)
{
  if(self != NULL && child != NULL)
  { if(child->parent == self) return(TRUE);

  if(self->unit != child->unit)
     printf("warning : both node_3D should belong to the same UNIT\n");
  return(FALSE);
  }
  N3D_UNDEFINED("is parent");
  return(FALSE);
}




/*-------------------------------------------------------------------*/
/* adding a node-3D to the children list of the node_3D self. if the */
/* child was previously attached to another parent it is removed 1st */

Booleen_t n3d_add_child(N3D_t   *self, N3D_t *child)
{
  short	nbchild;
  Booleen_t is_child=FALSE,  is_parent=FALSE ;

  if(self != NULL && child != NULL)
  { /* self must be allowed to hold children node_3D  */
    if(N3D_get_internal(self))
    {/* they must belong to the same UNIT	*/
      if(self->unit == child->unit)
      { /* child must not belong to self ancestors */
	if(!n3d_belong_to_family(self, child))
	{ /* checking if work or part of it already done	*/
	  is_parent=n3d_is_parent(self,child) ;
	  is_child =n3d_is_child(child,self,NULL) ;
	  if(is_parent && is_child) 
	     return(TRUE);
	  else if(!is_parent && is_child)
	  { if(child->parent != NULL)/* attached somewhere else*/
	     n3d_remove_child(child->parent, child); 
	  child->parent = self;
	  return(TRUE);
	  }
	  else 
	  { if(!is_parent)
	  { if(child->parent != NULL)
	     /* attached somewhere else*/
	     n3d_remove_child(child->parent, child); 
	  child->parent = self;
	  }
		
	  nbchild    = self->nbc ;

	  self->lstc = (N3D**) realloc(self->lstc, 
				       (size_t) (nbchild+1)*sizeof(N3D*));
	  if(self->lstc == NULL) 
	  {printf("erreur allocation memoire \n"); exit(0);}

	  self->lstc[nbchild] = child ;
	  self->nbc = nbchild + 1 ;
	  return(TRUE);
	  }
	}
	printf("warning : child_to_be cannot be a parent ancestor\n");
	return(FALSE);
      }
      printf("warning : both node_3D should belong to the same UNIT\n");
      return(FALSE);
    }
    printf("warning : this type of node cannot hold child node\n");
    return(FALSE);
  }
  N3D_UNDEFINED("add child");
  return(FALSE);
}



Booleen_t n3d_remove_child(N3D_t *self, N3D_t  *child)
{
  short	nbchild, memchild;
  int	i,j;
  N3D_t  **lstc,**new;

  if(self != NULL && child != NULL)
  { /* they must belong to the same UNIT	*/
    if(self->unit == child->unit)
    {
      if(n3d_is_child(child, self, &memchild))
      {  
	nbchild = self->nbc ;
	lstc = self->lstc;


	new = (N3D**) malloc((size_t) (nbchild)*sizeof(N3D*));
	if(new == NULL) 
	{
	  printf("erreur allocation memoire \n"); 
	  exit(0);
	}

	for(i=0,j=0 ; i<nbchild ; i++)
	   if(i != memchild) { new[j] = lstc[i] ; j++;} 

	self->nbc = nbchild - 1 ;
	free(lstc);
	self->lstc = new ;
	child->parent = NULL;
	return(TRUE);
      }
      printf("warning : node to remove is not child of parent\n");
      return(FALSE);
    }
    printf("warning : both node_3D should belong to the same UNIT\n");
    return(FALSE);
  }
  N3D_UNDEFINED("remove child");
  return(FALSE);
}





Booleen_t n3d_ref_refparent(N3D_t  *self)
{
  if(self  != NULL)
  {
    if(self->parent != NULL)
    {
		if(N3D_get_local(self))
	 n3d_local_ref_refparent_type(self);
      else
      {
	matrix_mult(self->ini->dir,
		    self->parent->ref->dir,
		    self->ref->dir);
	matrix_inverse(self->ref->dir , self->ref->inv);
      }

      return(TRUE);
    }
  }
  N3D_UNDEFINED("ref_refparent");
  return(FALSE);
}



private N3D_t  *n3d_creation_world(UNIT_t *unit, char *idt)
{
  N3D_t  *self ;
  WORLD_t	*world;

  if(unit != NULL)
  {

    self = (N3D*) malloc((size_t) sizeof(N3D));
    if(self == NULL) {printf("erreur allocation memoire \n"); exit(0);}
	
    n3d_ini(self, unit, idt);

    /* automatic world root creation and attaching to root node  */
	  
    n3d_set_type(self, N3D_WORLD);
    world	     = world_creation();
    self->str  = (void*) world ;
    world_n3d_attach(world, self);
    self->idi  = world_new_n3d(world,(void*)self); 

    return(self);
  }
  UNIT_UNDEFINED("n3d creation world")
    return(NULL);
}



private Booleen_t n3d_set_type(N3D_t *self, short type)
{
  if(self != NULL)
  {
    if((type>=0) && (type< nb_data_types))
    {
      self->type = type;
      /*A	      n3dplug_get_default_color(self, self->col);*/
    }
    else 
       TYPE_UNDEFINED("n3d set type")
	 return(TRUE);
  }
  N3D_UNDEFINED("set type")
    return(FALSE);	
}

private Booleen_t	n3d_set_ref(N3D_t *self, DIT_t *ref)
{
  if(self  != NULL)
  {
    self->refok = TRUE;

    if(ref != NULL)   dit_copy(ref, self->ref);
    else	    dit_copy(dit_let(IDENTITY), self->ref);

    return(TRUE);
  }
  N3D_UNDEFINED("n3d_set_ref")
    return(FALSE);
}


private Booleen_t	n3d_set_ref_dir(N3D_t *self, MatriX_t dir)
{
  if(self  != NULL)
  {
    self->refok = TRUE;

    if(dir != NULL) 
    {
      matrix_load(self->ref, SENS_DIRECT, dir);
      matrix_inverse(self->ref->dir, self->ref->inv);
    }
    else	    dit_copy(dit_let(IDENTITY), self->ref);

    return(TRUE);
  }
  N3D_UNDEFINED("n3d_set_ref_dir")
    return(FALSE);
}





Booleen n3d_get_local (N3D *self)
{
  if (self != NULL)
     return n3d_prop[self->type].local;
  else
  {
    N3D_UNDEFINED("n3d_get_local");
    return FALSE;
  }
}




 
static	void	n3d_ini(N3D_t *self, UNIT	*unit, char *idt)
{
  int i;

  self->idi = ID_ERROR;
  self->idt = NULL ;
  SET_IDT(self, idt);

  self->unit = unit ;
  self->parent  = NULL ;

  self->nbc     = 0    ;
  self->lstc    = NULL ;

  self->tpini   = Z_UNDEFINED ;
  n3d_set_ini(self, IDENTITY, NULL); /* set iniok to FALSE */
  self->iniok   = TRUE ;

  self->ref     = dit_creation();
  self->refok   = self->iniok ;

  self->svis    = TRUE ;
  self->fvis    = FALSE ;
  self->dvis    = FALSE ;
  self->nvis    = FALSE ;
  self->tvis    = FALSE ;
  self->col[0]  = self->col[1]  = self->col[2]  = 1. ;
  self->material_index     = 0;

  n3d_set_type(self, N3D_NEUTRAL);
  self->str     = NULL ;
}








static	Booleen_t n3d_local_ref_refparent_type(N3D_t *self)
{
  JOINT	*joint;
  FREE_	*free;
/*char * nd_name = n3d_get_local_name( self );*/
  switch(self->type)
  {
    case N3D_JOINT:
    joint = (JOINT*)self->str ;
 
    matrix_mult(joint->art->dir , 
		self->ini->dir ,  
		self->ref->dir);
    break;

    case N3D_FREE:
    free = (FREE_*)self->str ;
 
    matrix_mult(free->lib->dir, 
		self->ini->dir ,  
		self->ref->dir);
    break;

    default:
    printf("error : local transformation process missing\n");
    return(FALSE);
  }

  matrix_mult(self->ref->dir , 
	      self->parent->ref->dir , 
	      self->ref->dir);
  matrix_inverse(self->ref->dir , self->ref->inv);
  return(TRUE);
}






static	Booleen_t n3d_local_ref_refchild_type(N3D_t *self, N3D_t *child)
{
  FREE_	*free;
  JOINT	*joint;

  switch(child->type)
  {
    case N3D_JOINT:
    joint = (JOINT*)child->str ;
    matrix_mult(child->ini->inv,
		joint->art->inv,
		self->ref->dir);
    break;


    case N3D_FREE:
    free = (FREE_*)child->str ;
    matrix_mult(child->ini->inv,
		free->lib->inv,
		self->ref->dir);
    break;
  }

  matrix_mult(self->ref->dir,
	      child->ref->dir,
	      self->ref->dir);
  matrix_inverse(self->ref->dir,self->ref->inv);
  return(TRUE);
}



/*----------------------------------------------------------------------*/



/*----------------------------------------------------------------------*/
/* duplication de la liste des descendants d'un noeud passe en parametre*/

static N3D_t  **n3d_dupl_lstc(N3D_t  *self)
{
  N3D_t  **new;
  int	i,nbchild ;

  if(self  != NULL)
  {
    if(self->lstc != NULL)
    { 
      nbchild = self->nbc ;

      new = (N3D**) malloc((size_t) nbchild*sizeof(N3D*));
      if(new == NULL) {printf("erreur allocation memoire \n"); exit(0);}

      for(i=0 ; i<nbchild ; i++) new[i] = self->lstc[i];

      return(new);
    }
    printf("warning : trying to duplicate a non existant node list\n");
    return(NULL);
  }
  N3D_UNDEFINED("duplicate lstc");
  return(NULL);
}





static Booleen_t n3d_copy_ref(N3D_t *src, N3D_t *dst)
{
  if(src != NULL && dst != NULL)
  { n3d_set_ref(dst , src->ref); return(TRUE);}
  N3D_UNDEFINED("n3d copy ref")
    return(FALSE);	
}



const int nb_data_types     =  NB_DATA_TYPES;
const int n3d_max_type_name =  N3D_MAX_TYPE_NAME;


static char	n3d_type_name[NB_DATA_TYPES][N3D_MAX_TYPE_NAME] = {
  "NEUTRAL",
  "WORLD",
  "JOINT",
  "FREE_",
};



Booleen	n3dplug_get_type_name(N3D * self, char name[N3D_MAX_TYPE_NAME])
{	if(self  != NULL)
{
  if(name != NULL)
  {strcpy(name, n3d_type_name[n3d_get_type(self)]);  return(TRUE);}
  else					  return(FALSE);
}
fprintf(stderr, "get type name\n");  
return(FALSE);
}

 
char *	n3dplug_let_type_name(N3D * self)
{	if(self  != NULL)
{
  return( n3d_type_name[n3d_get_type(self)]);
}
fprintf(stderr, "let type name\n");  
return(NULL);
}





Booleen	  n3dplug_attach_data( N3D * self)
{
  switch(n3d_get_type(self))
  {
    case N3D_NEUTRAL:
    return(TRUE);

    case N3D_WORLD:
    return(world_n3d_attach((WORLD*) n3d_get_data(self), self));

    case N3D_JOINT:
    return(joint_n3d_attach((JOINT*) n3d_get_data(self), self));

    case N3D_FREE:
    return(free_n3d_attach((FREE_*) n3d_get_data(self), self));

    default :
    printf("error : type undefined \n");
    return(FALSE);
  }
}



N3D *n3dplug_get_n3d (void *data, short type)
{
  switch(type)
  {
    case N3D_NEUTRAL:
    case N3D_WORLD:
    return(NULL);

    case N3D_JOINT:
    return joint_get_n3d((JOINT*)data);

    case N3D_FREE:
    return free_get_n3d((FREE_*)data);

    default :
    printf("n3dplug_get_n3d() error : type undefined \n");
    return(NULL);
  }
}



#define H3D_TAILLE_CELLULE		9
#define H3D_TAILLE_ELEMENT		5
#define H3D_TAILLE_LIGNE		1
#define H3D_OFFSET_ELEMENT		2
#define H3D_OFFSET_LIGNE		2


static Booleen_t h3d_solve_flow_ref(N3D_t  *start, Booleen_t prev_influence);
Booleen_t h3d_actual_ref(N3D_t  *current, Booleen_t sens, 
			 Booleen_t prev_influence);

static Booleen_t h3d_recurs_exist_name(N3D_t *current, N3D_t *self, 
				       char *name);

Booleen_t	h3d_exist_name(N3D_t *self, char *name)
{
  int	i, nbc ;
  N3D_t  *root, *child ;

  if(self != NULL)
  {
    root = n3d_get_unit_root(self);
	  
    /* UNIT has no hierarchy (root in creation) */
    if(root == NULL) return(NO); 
	  
    if((root != self) && 
       (!strcmp(name, root->idt))) return(TRUE); 
	  
    nbc = n3d_get_nb_children(root);
    for(i=0 ; i< nbc ;i++)
    {
      child = n3d_get_child(root, i);
      if(h3d_recurs_exist_name(child, self, name))
	 return(TRUE);
    }
    return(NO);
  }
  N3D_UNDEFINED("h3d_exist_name()");
  return(NO);	
}



Booleen_t h3d_update_ref_config(N3D_t  *root)
{


  if(root != NULL)
  { 


    h3d_actual_ref(root,DIRECT_PROPAGATION,FALSE);
	  
	  	
    return(TRUE);
  }
  N3D_UNDEFINED("h3d update ref");
  return(FALSE);
}





Booleen_t h3d_actual_ref(N3D_t  *current, Booleen_t sens, 
			 Booleen_t prev_influence)
{
  Booleen_t new_influence = FALSE ;
  N3D_t     *link = NULL, *propar = NULL;
  FREE_t    *free_link=NULL;
  short     i, nbc ;

  new_influence = prev_influence;

  if(current->refok == FALSE)
  {
    current->refok= TRUE;
    new_influence = TRUE;
    if(current->iniok == FALSE) 
       current->iniok= TRUE;
    else /* (current->refok == FALSE) */
    {
      switch(current->type)
      {
	case N3D_JOINT: ((JOINT*)current->str)->artok = TRUE; break;
	case N3D_FREE:  ((FREE_*)current->str)->libok = TRUE; break;
      }
    }
  }
  if(new_influence) 
  {
    n3d_ref_refparent(current);


  }

  for(i=0, nbc=current->nbc ; i < nbc ; i++)
     h3d_actual_ref(current->lstc[i], DIRECT_PROPAGATION,
		    new_influence);

  return(prev_influence);

}






static Booleen_t	h3d_recurs_exist_name(N3D_t *current, N3D_t *self, 
					      char *name)
{
  int	i, nbc ;
  N3D_t  *child ;

  if(self != NULL && current != NULL && current != self)
  {
    if(!strcmp(name, current->idt)) return(TRUE); 
	  
    nbc = n3d_get_nb_children(current);
    for(i=0 ; i< nbc ;i++)
    {
      child = n3d_get_child(current, i);
      if(h3d_recurs_exist_name(child, self, name))
	 return(TRUE);
    }
    return(NO);
  }
  if(self == NULL || current == NULL) 
     N3D_UNDEFINED("h3d_recurs_exist_name()");
  return(NO);	
}






#define UNIT_SAME_WORLD(u1,u2)  ((u1)->scene == (u2)->scene)	


typedef struct call_ CALL, CALL_t;
struct call_
{
      int	  idi;	      
      char	  *idt;	      
      UNIT_t 	  *p;	      
      UNIT_t 	  *c;	      
};

/* structure de description des parametres d'une unite (UNIT)	*/

struct unit_
{
      int	  idi;	      
      char	  *idt;	      
      SCENE_t	  *scene;     
      N3D_t	  *root ;     
      int	  nbp ;	      
      CALL_t	  **lstp ;    
      int	  nbc ;	      
      CALL_t	  **lstc ;    
      Booleen_t visited;    

};

static	void	  unit_ini(UNIT_t *self, SCENE_t *scene, char *idt, 
			   Booleen_t newid);
static	void	  unit_ini_top(UNIT_t *self, SCENE_t *scene, char *idt);

static  Booleen_t unit_destr_dag(UNIT_t *self);
static	UNIT_t *  unit_rcheck_name(UNIT_t *self, char *idt);
static	CALL_t	* unit_rcheck_call_name(UNIT_t *self, char *idt);
static  Booleen_t unit_rcheck_parent(UNIT_t *self, UNIT_t *parent);
static  CALL_t	* unit_check_call_name(UNIT_t *top, char *idt);
static  Booleen_t unit_remove_called(UNIT_t *self, UNIT_t *parent, CALL_t *c);
static  Booleen_t unit_remove_calling(UNIT_t *self, UNIT_t *child, CALL_t *c);


static  Booleen_t unit_add_called_check(UNIT_t *self, UNIT_t *parent, 
					N3D_t *n3d_parent, char  *idt, 
					CALL_t ** to_parent);
static  Booleen_t unit_add_called_update(UNIT_t *self, UNIT_t *parent, 
					 CALL_t * to_parent);
static  void      unit_recurs_destroy(UNIT_t * self);


static	CALL_t	* call_creation(void);
static	void	  call_ini(CALL_t * self);
static	Booleen_t call_destr(CALL_t	*self);

static	int 	  call_get_id(CALL_t*       self);
static	char	* call_get_name(CALL_t	*self);
static	UNIT_t *  call_get_parent(CALL_t	*self);
static	UNIT_t *  call_get_child(CALL_t	*self);




static	Booleen_t call_set_id(CALL_t	*self, int id);
static	Booleen_t call_set_name(CALL_t	*self, char *idt);
static	Booleen_t call_set_parent(CALL_t	*self, UNIT_t *parent);
static	Booleen_t call_set_child(CALL_t	*self, UNIT_t *child);

static	Booleen_t call_set(CALL_t *self, UNIT_t *parent,  UNIT_t *child /*, 
									  DOCK *dock*/ );

static	Booleen_t call_write(CALL_t	*self, FILE    *f);
static	CALL_t	* call_read(FILE	*f);




UNIT_t *unit_new(SCENE_t *scene, char *idt, UNIT_t *parent)
{
  UNIT_t *self,*top;

  if(scene)
  { 
    top = scene_get_top(scene);
    if(top)
    {  
      if(parent && (parent->scene != scene)) 
      {
	WARNING_WORLD 
	  printf("parent UNIT does not belong to the provided SCENE\n");
	return(NULL);
      }
	    
      if(!parent) parent=top;

      self = (UNIT*) malloc((size_t) sizeof(UNIT));
      if(!self) {printf("erreur allocation memoire \n"); exit(0);}
	
      unit_ini(self, scene, idt, YES);


      /* this unit is attached by default to the top or parent unit : */

      unit_add_called(self,parent,NULL,idt);
    }
    else
    { 
      /* this unit will become the top UNIT, creates the WORLD node_3D */
      /* the WORLD node_3D gets the same name as the top UNIT          */

      self = (UNIT*) malloc((size_t) sizeof(UNIT));
      if(!self) {printf("erreur allocation memoire \n"); exit(0);}
	
      unit_ini_top(self, scene, idt);

      if(!(self->root = n3d_creation_world(self, idt)))
      { printf("error : cannot create a WORLD node_3D in unit_new\n");
      unit_lowlevel_destroy(self);
      return(NULL);
      }
    }  
    return(self);
  }
  SCENE_UNDEFINED("UNIT_t new");
  return(NULL);
}


N3D_t	*unit_creation_root(UNIT_t *self, char *idt)
{
  if(self)
  { 
    if(!unit_get_scene_root(self))
    { 
      if(self->root = n3d_creation_world(self, idt)) return(self->root);
      else				    return(NULL);
    }
    else
    { 
      if(idt)   
      { 
	if(self->root = n3d_creation(self,idt)) return(self->root);
	else				    return(NULL);
      }
      else
      { 
	if(self->root = n3d_creation(self,UNIT_DEFAULT_ROOT_NAME)) 
	   return(self->root);
	else				    
	   return(NULL);
      }
    }
  }
  UNIT_UNDEFINED("unit creation root");
  return(NULL);
}





/*-------------------------------------------------------------------*/
UNIT_t *unit_get_top(UNIT_t *self)
{
  if(self)   return(scene_get_top(self->scene)); 
  UNIT_UNDEFINED("get top");
  return(NULL);
}


UNIT_t *  unit_get_child(UNIT_t *self, int childindex)
{
  if(self != NULL)
  {
    if(self->nbc <= childindex || childindex < 0)
    {
      fprintf(stderr, "unit_get_child() : bad index %d\n", 
	      childindex);
      return(NULL);
    }
    if(self->lstc != NULL && self->lstc[childindex] != NULL)
    {
      return(self->lstc[childindex]->c); 
    }
    fprintf(stderr, 
	    "Bad list child entry for index %d\n", childindex);
    fprintf(stderr, 
	    "--> internal consistency broken !\n");
    return(NULL);
  }
  UNIT_UNDEFINED("get child");
  return(NULL);
}

/*-------------------------------------------------------------------*/
N3D_t	*unit_get_root(UNIT_t *self)
{
  if(self)   
     return(self->root); 
  UNIT_UNDEFINED("get root");
  return(NULL);
}

/*-------------------------------------------------------------------*/
N3D_t	*unit_get_scene_root(UNIT_t *self)
{
  if(self)   
     return((scene_get_top(self->scene))->root); 
  UNIT_UNDEFINED("get scene root");
  return(NULL);
}

/*-------------------------------------------------------------------*/
WORLD	*unit_get_world(UNIT_t *self)
{
  if(self)
  {
    N3D_t * root;
	    
    root = (scene_get_top(self->scene))->root; 
    return((WORLD*)(n3d_get_data(root)));
  }
	
  UNIT_UNDEFINED("get world");
  return(NULL);
}



Booleen_t	unit_add_called(UNIT_t *self, UNIT_t *parent, N3D_t *n3d_parent, char  *idt)
{
  CALL_t *    to_parent ;
    
  if(!unit_add_called_check(self, parent, n3d_parent, idt, &to_parent))
  {
    return(NO);
  }
  call_set_id(to_parent, scene_new_call(self->scene, parent,
					self));
  if(!unit_add_called_update(self, parent, to_parent))
  {
    return(NO);
  }
  return(TRUE);
}



Booleen_t unit_check_dag(UNIT_t *to_call, UNIT_t *calling)
{
  if(to_call && calling)
  { 
    unit_reset_visited(unit_get_top(to_call));

    return(!unit_rcheck_parent(calling, to_call));
  }
  return(TRUE);
}




private Booleen_t	    unit_clear_root(UNIT_t *self)
{
  if(self != NULL)
  {
    self->root = NULL;
    return(YES);
  }   
  UNIT_UNDEFINED("clear root");
  return(NO);
}






private void	unit_reset_visited(UNIT_t *self)
{
  int i, imax;

  if(self)
  { 
    self->visited = FALSE ;
    for(i=0, imax=self->nbc ; i<imax ; i++)
    {
      if(self->lstc != NULL && 
	 self->lstc[i] != NULL && self->lstc[i]->c != NULL)
	 unit_reset_visited(self->lstc[i]->c);
    }
  }
}

/*-------------------------------------------------------------------*/

private Booleen_t	unit_lowlevel_destroy(UNIT_t *self)
{
  UNIT_t * pnull;
	
  if(self)
  {
    /* il n'est pas possible de detruire une unite tant qu'elle*/
    /* est appelee par plus d'une une unites parentes.	     */

    if((self->nbp >0)||(self->nbc >0))
    { 
      printf("warning : delayed destruction of unit %s\n",self->idt);
      printf("  as long as it is refered by other units \n");
      return(FALSE);
    }
    else if(self == unit_get_top(self))
    {
      if (self->root)
      {
	printf("warning : delayed destruction of top unit %s\n",self->idt);
	printf("  as long as it owns some node 3d\n");
	return(FALSE);
      }
      /* update scene struct. when destroying top unit */
      pnull = NULL;
      scene_set_top( self->scene, pnull);
    }
    /* update id_unit scene's struct */

    if(self->idt) free(self->idt);
    free(self);

    return(TRUE);
  }
  UNIT_UNDEFINED("unit destroy");
  return(FALSE);
}




/*-------------------------------------------------------------------*/
/* UNIT initialisation, linked to the scene, name checking on  DAG   */
 
static	void	unit_ini(UNIT_t *self, SCENE_t *scene, char *idt, Booleen_t newid)
{
  static  char   prefix[100];
  char           *final_idt=NULL ;

  self->scene   = scene ;
  if(newid)
  {
    /* newid-id-ida (popular song) */
    self->idi = scene_new_unit(scene,(void*)self) ;
  }
  else
  {
    /* file management case */
    self->idi = ID_ERROR;
  }
  self->root    = NULL ;
  self->nbp     = 0 ;
  self->lstp    = NULL ;
  self->nbc     = 0 ;
  self->lstc    = NULL ;
  self->visited = FALSE ;
  self->idt     = NULL ;



  /* first checking if the UNIT name is unique in the SCENE ??  */

  SET_IDT(self, idt);
}

/*-------------------------------------------------------------------*/
/* top UNIT initialisation, linked to the scene */
 
static	void	unit_ini_top(UNIT_t *self, SCENE_t *scene, char *idt)
{
  self->scene   = scene ;
  self->idi     = scene_new_unit(scene,(void*)self) ;
  self->root    = NULL ;
  self->nbp     = 0 ;
  self->lstp    = NULL ;
  self->nbc     = 0 ;
  self->lstc    = NULL ;
  self->visited = FALSE ;
  self->idt     = NULL ;

  SET_IDT(self,idt);
}


/*-------------------------------------------------------------------*/
static Booleen_t	unit_rcheck_parent(UNIT_t *self, UNIT_t *parent)
{
  int	i, imax ;

  for(i=0, imax=self->nbp ; i<imax ;i++)
     if(self->lstp[i]->p == parent) return(TRUE);

  return(FALSE);
}



static Booleen_t	unit_add_called_check(UNIT_t *self, UNIT_t *parent, 
					      N3D_t *n3d_parent, char  *idt, 
					      CALL_t ** to_parent)
{
  /*    DOCK *    default_dock = NULL ;*/
    
  if(self == NULL || parent == NULL || !UNIT_SAME_WORLD(self, parent))
  {
    UNIT_UNDEFINED("add called");
    return(FALSE);
  }
  if(!unit_check_dag(self, parent))
  {
    printf("warning : violating the DAG coherence");
    printf(" by calling unit >%s< from unit >%s<\n",self->idt,parent->idt);
    return(NO);
  }

  *to_parent = call_creation();
  call_set_name(*to_parent,idt);
  call_set(*to_parent,parent,self/*,default_dock*/);
  return(YES);
}


static Booleen_t	unit_add_called_update(UNIT_t *self, UNIT_t *parent, 
					       CALL_t * to_parent)
{
  int	i;
  int	nbp, nbc;
  CALL_t **   lstp, **lstc, **new;

  /* mise a jour de la liste de call parent de p		*/
    
  nbp = self->nbp ;
  lstp = self->lstp ;

  new = (CALL_t**) malloc((size_t) (nbp+1)*sizeof(CALL_t*));
  if(!new) {printf("erreur allocation memoire \n"); exit(0);}
    
  if(nbp > 0)
  {
    if(lstp != NULL)
    {
      for(i=0 ; i<nbp ; i++) new[i] = lstp[i] ;
      free(lstp);
    }
    else
    {
      for(i=0 ; i<nbp ; i++) new[i] = NULL;
    }
  }
  new[nbp] = to_parent ;
  self->nbp = nbp + 1 ;
  self->lstp = new ;
        
  /* mise a jour de la liste de call child de parent	*/
	    
  nbc = parent->nbc ;
  lstc = parent->lstc ;

  new = (CALL_t**) malloc((size_t) (nbc+1)*sizeof(CALL_t*));
  if(!new) {printf("erreur allocation memoire \n"); exit(0);}
    
  if(nbc > 0)
  {
    if(lstc != NULL)
    {
      for(i=0 ; i<nbc ; i++) new[i] = lstc[i] ;
      free(lstc);
    }
    else
    {
      for(i=0 ; i<nbp ; i++) new[i] = NULL;
    }	
  }
  new[nbc] = to_parent ;
  parent->nbc = nbc + 1 ;
  parent->lstc = new ;
    
  return(TRUE);
}


/*-------------------------------------------------------------------*/
/* creation d'un call entre une unite parente appelant une unite fille	*/

static	CALL_t	*call_creation(void)
{
  CALL_t	*self;

  self = (CALL_t*) malloc((size_t) sizeof(CALL_t));
  if(!self) {printf("erreur allocation memoire \n"); exit(0);}
	
  call_ini(self);
  return(self);
}

/*-------------------------------------------------------------------*/
/* Initialisation d'un call qui vient d'etre cree , vide par defaut	*/
 
static	void	call_ini(CALL_t * self)
{
  self->idi  = ID_ERROR;
    
  self->idt  = NULL ;
  self->p	   = NULL ;
  self->c	   = NULL ;
}



static  Booleen_t call_set_id(CALL_t  *self, int id)
{
  if(self != NULL)
  {
    self->idi = id;
    return(YES);
  }
  CALL_UNDEFINED("set id");
  return(NO);
}
/*-------------------------------------------------------------------*/

static	Booleen_t call_set_name(CALL_t  *self, char	*idt)
{
  char	*final_idt=NULL ;

  if(self && idt)
  {
    final_idt = NULL ;
    STR_CPY(final_idt,idt);

    SET_IDT(self,final_idt);
    return(TRUE);
  }
  CALL_UNDEFINED("set name");
  return(FALSE);	
}


/*-------------------------------------------------------------------*/

static	Booleen_t	call_set(CALL_t *self, UNIT_t *parent, UNIT_t *child/*, DOCK *dock*/)
{
  if(self && parent && child) 
  {
    self->p	= parent ;
    self->c	= child ;

    return(TRUE);
  }
  CALL_UNDEFINED("call set");
  return(FALSE);
}




#define DEBUG_GEOM	FALSE


static	VT3D	vt3d_null = { 0. , 0. , 0. };
static	VHOM	vhom_null = { 0. , 0. , 0. , 0. };

/* local visibility and usability of predefined rotational matrices	*/

static	MR3D	mr3d_idt = {
  { 1. , 0. , 0. },
  { 0. , 1. , 0. },
  { 0. , 0. , 1. }
};

/*static*/	MR3D	mr3d_z_on_x 		= {
  { 0. , 1. , 0. },
  { 0. , 0. , 1. },
  { 1. , 0. , 0. }
};


static	MR3D	mr3d_z_on_y 		= {
  { 0. , 0. , 1. },
  { 1. , 0. , 0. },
  { 0. , 1. , 0. }
};


static	MR3D	mr3d_z_on_minus_z 	= {
  { 0. , 1. , 0. },
  { 1. , 0. , 0. },
  { 0. , 0. ,-1. }
};


static	MR3D	mr3d_z_on_minus_x 	= {
  { 0. , 0. , 1. },
  { 0. , 1. , 0. },
  {-1. , 0. , 0. }
};


static	MR3D	mr3d_z_on_minus_y 	= {
  { 1. , 0. , 0. },
  { 0. , 0. , 1. },
  { 0. ,-1. , 0. }
};




static	MR3D	let_mr3d_idt = {
  { 1. , 0. , 0. },
  { 0. , 1. , 0. },
  { 0. , 0. , 1. }
};


static	MR3D	let_mr3d_z_on_x = {
  { 0. , 1. , 0. },
  { 0. , 0. , 1. },
  { 1. , 0. , 0. }
};


static	MR3D	let_mr3d_z_on_y = {
  { 0. , 0. , 1. },
  { 1. , 0. , 0. },
  { 0. , 1. , 0. }
};


static	MR3D	let_mr3d_z_on_minus_z 	= {
  { 0. , 1. , 0. },
  { 1. , 0. , 0. },
  { 0. , 0. ,-1. }
};


static	MR3D	let_mr3d_z_on_minus_x 	= {
  { 0. , 0. , 1. },
  { 0. , 1. , 0. },
  {-1. , 0. , 0. }
};


static	MR3D	let_mr3d_z_on_minus_y 	= {
  { 1. , 0. , 0. },
  { 0. , 0. , 1. },
  { 0. ,-1. , 0. }
};



static	MatriX	matrix_idt		={
  { 1. , 0. , 0. , 0. },
  { 0. , 1. , 0. , 0. },
  { 0. , 0. , 1. , 0. },
  { 0. , 0. , 0. , 1. }
};




static	MatriX	let_matrix_idt = {
  { 1. , 0. , 0., 0. },
  { 0. , 1. , 0., 0. },
  { 0. , 0. , 1., 0. }, 
  { 0. , 0. , 0., 1. }
};


static	MatriX	let_matrix_z_on_x = {
  { 0. , 1. , 0., 0. },
  { 0. , 0. , 1., 0. },
  { 1. , 0. , 0., 0. }, 
  { 0. , 0. , 0., 1. }
};


static	MatriX	let_matrix_z_on_y = {
  { 0. , 0. , 1., 0. },
  { 1. , 0. , 0., 0. },
  { 0. , 1. , 0., 0. }, 
  { 0. , 0. , 0., 1. }
};


static	MatriX	let_matrix_z_on_minus_z 	= {
  { 0. , 1. , 0., 0. },
  { 1. , 0. , 0., 0. },
  { 0. , 0. ,-1., 0. }, 
  { 0. , 0. , 0., 1. }
};


static	MatriX	let_matrix_z_on_minus_x 	= {
  { 0. , 0. , 1., 0. },
  { 0. , 1. , 0., 0. },
  {-1. , 0. , 0., 0. }, 
  { 0. , 0. , 0., 1. }
};


static	MatriX	let_matrix_z_on_minus_y 	= {
  { 1. , 0. , 0., 0. },
  { 0. , 0. , 1., 0. },
  { 0. ,-1. , 0., 0. }, 
  { 0. , 0. , 0., 1. }
};



static	DIT	dit_idt			={
  {
    { 1. , 0. , 0. , 0. },
    { 0. , 1. , 0. , 0. },
    { 0. , 0. , 1. , 0. },
    { 0. , 0. , 0. , 1. },
  },
  {
    { 1. , 0. , 0. , 0. },
    { 0. , 1. , 0. , 0. },
    { 0. , 0. , 1. , 0. },
    { 0. , 0. , 0. , 1. }
  }
} ;

static	DIT	dit_z_on_x		={
  {
    { 0. , 1. , 0. , 0. },
    { 0. , 0. , 1. , 0. },
    { 1. , 0. , 0. , 0. },
    { 0. , 0. , 0. , 1. },
  },
  {
    { 0. , 0. , 1. , 0. },
    { 1. , 0. , 0. , 0. },
    { 0. , 1. , 0. , 0. },
    { 0. , 0. , 0. , 1. }
  }
} ;

static	DIT	dit_z_on_y 		={
  {
    { 0. , 0. , 1. , 0. },
    { 1. , 0. , 0. , 0. },
    { 0. , 1. , 0. , 0. },
    { 0. , 0. , 0. , 1. },
  },
  {
    { 0. , 1. , 0. , 0. },
    { 0. , 0. , 1. , 0. },
    { 1. , 0. , 0. , 0. },
    { 0. , 0. , 0. , 1. }
  }
};

static	DIT	dit_z_on_minus_z 	={
  {
    { 0. , 1. , 0. , 0. },
    { 1. , 0. , 0. , 0. },
    { 0. , 0. ,-1. , 0. },
    { 0. , 0. , 0. , 1. },
  },
  {
    { 0. , 1. , 0. , 0. },
    { 1. , 0. , 0. , 0. },
    { 0. , 0. ,-1. , 0. },
    { 0. , 0. , 0. , 1. }
  }
};

static	DIT	dit_z_on_minus_x 	={
  {
    { 0. , 0. , 1. , 0. },
    { 0. , 1. , 0. , 0. },
    {-1. , 0. , 0. , 0. },
    { 0. , 0. , 0. , 1. },
  },
  {
    { 0. , 0. ,-1. , 0. },
    { 0. , 1. , 0. , 0. },
    { 1. , 0. , 0. , 0. },
    { 0. , 0. , 0. , 1. }
  }
};

static	DIT	dit_z_on_minus_y 	={
  {
    { 1. , 0. , 0. , 0. },
    { 0. , 0. , 1. , 0. },
    { 0. ,-1. , 0. , 0. },
    { 0. , 0. , 0. , 1. },
  },
  {
    { 1. , 0. , 0. , 0. },
    { 0. , 0. ,-1. , 0. },
    { 0. , 1. , 0. , 0. },
    { 0. , 0. , 0. , 1. }
  }
};

static DIT dit_gl_to_bodylib_y_up = {
  {
    { -1,  0,  0,  0 },
    {  0,  1,  0,  0 },
    {  0,  0, -1,  0 },
    {  0,  0,  0,  1 }
  },
  {
    { -1,  0,  0,  0 },
    {  0,  1,  0,  0 },
    {  0,  0, -1,  0 },
    {  0,  0,  0,  1 }
  }
};

static DIT dit_gl_to_bodylib_z_up = {
  {
    { -1,  0,  0,  0 },
    {  0,  0,  1,  0 },
    {  0,  1,  0,  0 },
    {  0,  0,  0,  1 }
  },
  {
    { -1,  0,  0,  0 },
    {  0,  0,  1,  0 },
    {  0,  1,  0,  0 },
    {  0,  0,  0,  1 }
  }
};





static	DIT	let_dit_idt	={
  {
    { 1. , 0. , 0. , 0. },
    { 0. , 1. , 0. , 0. },
    { 0. , 0. , 1. , 0. },
    { 0. , 0. , 0. , 1. },
  },
  {
    { 1. , 0. , 0. , 0. },
    { 0. , 1. , 0. , 0. },
    { 0. , 0. , 1. , 0. },
    { 0. , 0. , 0. , 1. }
  }
} ;

static	DIT	let_dit_z_on_x	={
  {
    { 0. , 1. , 0. , 0. },
    { 0. , 0. , 1. , 0. },
    { 1. , 0. , 0. , 0. },
    { 0. , 0. , 0. , 1. },
  },
  {
    { 0. , 0. , 1. , 0. },
    { 1. , 0. , 0. , 0. },
    { 0. , 1. , 0. , 0. },
    { 0. , 0. , 0. , 1. }
  }
} ;

static	DIT	let_dit_z_on_y 	={
  {
    { 0. , 0. , 1. , 0. },
    { 1. , 0. , 0. , 0. },
    { 0. , 1. , 0. , 0. },
    { 0. , 0. , 0. , 1. },
  },
  {
    { 0. , 1. , 0. , 0. },
    { 0. , 0. , 1. , 0. },
    { 1. , 0. , 0. , 0. },
    { 0. , 0. , 0. , 1. }
  }
};

static	DIT	let_dit_z_on_minus_z 	={
  {
    { 0. , 1. , 0. , 0. },
    { 1. , 0. , 0. , 0. },
    { 0. , 0. ,-1. , 0. },
    { 0. , 0. , 0. , 1. },
  },
  {
    { 0. , 1. , 0. , 0. },
    { 1. , 0. , 0. , 0. },
    { 0. , 0. ,-1. , 0. },
    { 0. , 0. , 0. , 1. }
  }
};

static	DIT	let_dit_z_on_minus_x 	={
  {
    { 0. , 0. , 1. , 0. },
    { 0. , 1. , 0. , 0. },
    {-1. , 0. , 0. , 0. },
    { 0. , 0. , 0. , 1. },
  },
  {
    { 0. , 0. ,-1. , 0. },
    { 0. , 1. , 0. , 0. },
    { 1. , 0. , 0. , 0. },
    { 0. , 0. , 0. , 1. }
  }
};

static	DIT	let_dit_z_on_minus_y 	={
  {
    { 1. , 0. , 0. , 0. },
    { 0. , 0. , 1. , 0. },
    { 0. ,-1. , 0. , 0. },
    { 0. , 0. , 0. , 1. },
  },
  {
    { 1. , 0. , 0. , 0. },
    { 0. , 0. ,-1. , 0. },
    { 0. , 1. , 0. , 0. },
    { 0. , 0. , 0. , 1. }
  }
};

static DIT let_dit_gl_to_bodylib_y_up = {
  {
    { -1,  0,  0,  0 },
    {  0,  1,  0,  0 },
    {  0,  0, -1,  0 },
    {  0,  0,  0,  1 }
  },
  {
    { -1,  0,  0,  0 },
    {  0,  1,  0,  0 },
    {  0,  0, -1,  0 },
    {  0,  0,  0,  1 }
  }
};

static DIT let_dit_gl_to_bodylib_z_up = {
  {
    { -1,  0,  0,  0 },
    {  0,  0,  1,  0 },
    {  0,  1,  0,  0 },
    {  0,  0,  0,  1 }
  },
  {
    { -1,  0,  0,  0 },
    {  0,  0,  1,  0 },
    {  0,  1,  0,  0 },
    {  0,  0,  0,  1 }
  }
};




static void nrerror(char *error_text);	
static double **matmxn_convert(double *a,int nrl,int nrh,int ncl,int nch);	
static void Jinv(double *w,int n,double damping);
static void svd(double *a,int m,int n,double *w,double *v); 
static double pythag(double a,double b);





/*-------------------------------------------------------------------*/
/* allocation d'un DIT : deux MatriX (homogenes) direct et inverse.	*/

DIT*	dit_creation(void)
{
  DIT*	p;

  p = (DIT*)malloc((size_t) sizeof(DIT));
  if(!p) {printf("erreur allocation memoire \n"); exit(0);}

  dit_idt_(p);
  return(p);
}	

/*-------------------------------------------------------------------*/
void	dit_idt_(DIT *p)
{
  matrix_idt_(p->dir);
  matrix_idt_(p->inv);
}

/*-------------------------------------------------------------------*/
void	dit_copy(DIT	*p_src ,
		 DIT	*p_dst)
{
  *p_dst  =  *p_src;
}


/*-------------------------------------------------------------------*/
/* la rotation est produite selon l'axe z				*/


void	dit_actual_rot(float	angle, 
		       DIT	*p)
{
  static	float	mem_angle = 0. ;
  static	float	cos_angle = 1. ;
  static  float   sin_angle = 0. ;

  matrix_idt_(p->dir);
  matrix_idt_(p->inv);

  if(angle != 0.)
  {
    if(angle != mem_angle)
    {
      cos_angle = (float) cos((double) angle);
      sin_angle = (float) sin((double) angle);
      mem_angle = angle ;
    }

    p->dir[0][0]= p->inv[0][0]= p->dir[1][1]= p->inv[1][1]= cos_angle ;

    p->dir[1][0] = p->inv[0][1] = -sin_angle ;
    p->dir[0][1] = p->inv[1][0] = sin_angle ;
  }
}
/*-------------------------------------------------------------------*/



void	dit_actual_trn(float	dist, 
		       DIT	*p)
{
  matrix_idt_(p->dir);
  matrix_idt_(p->inv);

  p->dir[3][2] = dist ;
  p->inv[3][2] = -dist ;
}
/*-------------------------------------------------------------------*/
/* return initial dit entry point for predefined dit or create one	*/

DIT	*dit_creation_ini(short tpini)
{
  switch((int)tpini)
  {
    case IDENTITY:	         return(&dit_idt);
    case Z_ON_X:	         return(&dit_z_on_x);
    case Z_ON_Y:	         return(&dit_z_on_y);
    case Z_ON_MINUS_Z:       return(&dit_z_on_minus_z);
    case Z_ON_MINUS_X:       return(&dit_z_on_minus_x);
    case Z_ON_MINUS_Y:       return(&dit_z_on_minus_y);
    case GL_TO_BODYLIB_Y_UP: return(&dit_gl_to_bodylib_y_up);
    case GL_TO_BODYLIB_Z_UP: return(&dit_gl_to_bodylib_z_up);
    case Z_OTHER:	         return(dit_creation());
    default : printf("error : bad initial DIT type\n");
    return(NULL);
  }
}
/*-------------------------------------------------------------------*/
/* changing to Z_OTHER initial transformation type for editing		*/

DIT	*dit_edit_ini(DIT   *ini, short *tpini)
{
  switch((int)(*tpini))
  {
    case IDENTITY:	    
    case Z_ON_X:	    
    case Z_ON_Y:	    
    case Z_ON_MINUS_Z:  
    case Z_ON_MINUS_X:  
    case Z_ON_MINUS_Y:  
    *tpini = Z_OTHER ;
    return(dit_creation());
    case Z_OTHER:
    return(ini);

    default : printf("error : bad initial DIT type\n");
    printf("type %d\n", (*tpini));
    return(NULL);
  }
}

/*-------------------------------------------------------------------*/

DIT	*dit_destroy_ini(DIT *ini, short tpini)
{
  switch((int)tpini)
  {
    case Z_OTHER:	    
    free((void*)ini);
    return(NULL);
    case IDENTITY:	    
    case Z_ON_X:	    
    case Z_ON_Y:	    
    case Z_ON_MINUS_Z:  
    case Z_ON_MINUS_X:  
    case Z_ON_MINUS_Y:  return(NULL);
    default : printf("error : bad initial DIT type\n");
    return(NULL);
  }
}


/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/

void	matrix_idt_(MatriX matrix)
{
  register float	*m ;

  m = (float*)matrix ; 

  *(m++) = 1.	; *(m++) = 0.	  ; *(m++) = 0.	    ; *(m++) = 0. ;
  *(m++) = 0.     ; *(m++) = 1.	  ; *(m++) = 0.     ; *(m++) = 0. ;
  *(m++) = 0.     ; *(m++) = 0.	  ; *(m++) = 1.	    ; *(m++) = 0. ;
  *(m++) = 0.	; *(m++) = 0.	  ; *(m++) = 0.     ; *m     = 1. ;
}

/*-------------------------------------------------------------------*/
void	matrix_copy(MatriX src, MatriX	dst)
{
  register    float	*ms , *md ;

  ms = (float*)src ; md = (float*)dst ; 

  *(md++)=*(ms++); *(md++)=*(ms++); *(md++)=*(ms++); *(md++)=*(ms++); 
  *(md++)=*(ms++); *(md++)=*(ms++); *(md++)=*(ms++); *(md++)=*(ms++); 
  *(md++)=*(ms++); *(md++)=*(ms++); *(md++)=*(ms++); *(md++)=*(ms++); 
  *(md++)=*(ms++); *(md++)=*(ms++); *(md++)=*(ms++); *md    =*ms; 
}

/*-------------------------------------------------------------------*/
void	matrix_load(DIT	*p, 
		    Booleen	sens, 
		    MatriX	a)
{
  int	i,j;

  if(sens == SENS_DIRECT)
     for(i=0 ; i<4 ; i++)
	for(j=0 ; j<4 ; j++)
	   p->dir[i][j] = a[i][j] ; 
  else
     for(i=0 ; i<4 ; i++)
	for(j=0 ; j<4 ; j++)
	   p->inv[i][j] = a[i][j] ; 
}


/*-------------------------------------------------------------------*/

void	matrix_compose(MR3D	mr, 
		       VT3D	vt, 
		       MatriX	matrix)
{
  register    float	*m , *v , *t ;

  m = (float*)mr ; v = (float*)vt ; t = (float*)matrix ;

  *(t++) = *(m++) ; *(t++) = *(m++) ; *(t++) = *(m++) ; *(t++) = 0. ;
  *(t++) = *(m++) ; *(t++) = *(m++) ; *(t++) = *(m++) ; *(t++) = 0. ;
  *(t++) = *(m++) ; *(t++) = *(m++) ; *(t++) = *(m++) ; *(t++) = 0. ;
  *(t++) = *(v++) ; *(t++) = *(v++) ; *(t++) = *v     ; *t     = 1. ;
}


/*-------------------------------------------------------------------*/
/* use predefined rotation or translation types to produce a MatriX	*/

void	matrix_combine(MR3D	mr, 
		       short    tpini, 
		       VT3D	vt, 
		       MatriX	matrix)
{
  register    float	*m , *v , *t ;

  if(mr)	m = (float*)mr ;
  else
  {
    switch((int)tpini)
    {
      case IDENTITY:	m = (float*)mr3d_idt ;		break; 
      case Z_ON_X:		m = (float*)mr3d_z_on_x ;       break; 
      case Z_ON_Y:		m = (float*)mr3d_z_on_y ;       break; 
      case Z_ON_MINUS_Z:	m = (float*)mr3d_z_on_minus_z ; break; 
      case Z_ON_MINUS_X:	m = (float*)mr3d_z_on_minus_x ; break; 
      case Z_ON_MINUS_Y:	m = (float*)mr3d_z_on_minus_y ; break; 
      default : printf("error : bad MR3D type for combine\n");
      exit(0);
    }
	    
  }

  if(vt)	v = (float*)vt ; 
  else	v = vt3d_null ;

  t = (float*)matrix ;

  *(t++) = *(m++) ; *(t++) = *(m++) ; *(t++) = *(m++) ; *(t++) = 0. ;
  *(t++) = *(m++) ; *(t++) = *(m++) ; *(t++) = *(m++) ; *(t++) = 0. ;
  *(t++) = *(m++) ; *(t++) = *(m++) ; *(t++) = *(m++) ; *(t++) = 0. ;
  *(t++) = *(v++) ; *(t++) = *(v++) ; *(t++) = *v     ; *t     = 1. ;
}

/*-------------------------------------------------------------------*/

void	matrix_decompose(MR3D	    mr, 
		         VT3D	    vt, 
		         MatriX	    matrix)
{
  register    float	*m , *v , *t ;

  m = (float*)mr ; v = (float*)vt ; t = (float*)matrix ;

  *(m++) = *(t++) ; *(m++) = *(t++) ; *(m++) = *(t++) ; t++ ;
  *(m++) = *(t++) ; *(m++) = *(t++) ; *(m++) = *(t++) ; t++ ;
  *(m++) = *(t++) ; *(m++) = *(t++) ; *m     = *(t++) ; t++ ;

  *(v++) = *(t++) ; *(v++) = *(t++) ; *v     = *t ;

}

/*-------------------------------------------------------------------*/
/* Mult. d'un vecteur 3D par une matrice homogene u*a->v  (15.3.93)	*/

void	matrix_mult_vt3d(MatriX	    ma, 
			 VT3D	    vu,
			 VT3D	    vv)
{
  static	VT3D	vw;
  register    float	*w ,*a , *u , *v ;
  register    float	*u0 , *u1 , *u2 ;
 
  w  = (float*)vw ;
  a  = (float*)ma ;
  u  = (float*)vu ; 
  u0 = u++ ; u1 = u++ ; u2 = u ;

  *(w++) = *u0 * *a       +  *u1 * *(a+4)   +  *u2 * *(a+8) + *(a+12);
  *(w++) = *u0 * *(a+1)   +  *u1 * *(a+5)   +  *u2 * *(a+9) + *(a+13);
  *w     = *u0 * *(a+2)   +  *u1 * *(a+6)   +  *u2 * *(a+10) + *(a+14);

  w  = (float*)vw ;
  v  = (float*)vv ;
  *(v++) = *(w++) ; *(v++) = *(w++) ; *v = *w ;

}


/*-------------------------------------------------------------------*/
void	matrix_inverse(MatriX	    matrix,
		       MatriX	    matrix_inv)
{
  static	MR3D	mr;
  static	VT3D	vt;

  matrix_decompose(mr,vt,matrix);
  mr3d_self_transpose(mr);
  vt[0] = - vt[0] ; vt[1] = - vt[1] ; vt[2] = - vt[2] ;
  mr3d_mult_vt3d(mr,vt,vt);
  matrix_compose(mr,vt,matrix_inv);
}

/*-------------------------------------------------------------------*/
/*	effectue le produit matriciel ma*mb et range le resultat dans mc*/

void	matrix_mult(MatriX  ma,
		    MatriX  mb,
		    MatriX  mc)
{
  static	MatriX	mi ;
  register    float   *m , *a , *b , *c ;

  static	float	*a00 , *a01 , *a02 , *a03 ;
  static	float	*a10 , *a11 , *a12 , *a13 ;
  static	float	*a20 , *a21 , *a22 , *a23 ;
  static	float	*a30 , *a31 , *a32 , *a33 ;

  static	float	*b00 , *b01 , *b02 , *b03 ;
  static	float	*b10 , *b11 , *b12 , *b13 ;
  static	float	*b20 , *b21 , *b22 , *b23 ;
  static	float	*b30 , *b31 , *b32 , *b33 ;

  a  = (float*)ma ; b  = (float*)mb  ; c = (float*)mc ; 

  a00 = a++ ; a01 = a++ ; a02 = a++ ; a03 = a++ ;
  a10 = a++ ; a11 = a++ ; a12 = a++ ; a13 = a++ ;
  a20 = a++ ; a21 = a++ ; a22 = a++ ; a23 = a++ ;
  a30 = a++ ; a31 = a++ ; a32 = a++ ; a33 = a ;

  b00 = b++ ; b01 = b++ ; b02 = b++ ; b03 = b++ ;
  b10 = b++ ; b11 = b++ ; b12 = b++ ; b13 = b++ ;
  b20 = b++ ; b21 = b++ ; b22 = b++ ; b23 = b++ ;
  b30 = b++ ; b31 = b++ ; b32 = b++ ; b33 = b ;

  if(( mc == ma) || ( mc == mb))
  { 
    m   = (float*)mi ;

    *(m++) = *a00 * *b00  +  *a01 * *b10  +  *a02 * *b20  +  *a03 * *b30 ;
    *(m++) = *a00 * *b01  +  *a01 * *b11  +  *a02 * *b21  +  *a03 * *b31 ;
    *(m++) = *a00 * *b02  +  *a01 * *b12  +  *a02 * *b22  +  *a03 * *b32 ;
    *(m++) = *a00 * *b03  +  *a01 * *b13  +  *a02 * *b23  +  *a03 * *b33 ;

    *(m++) = *a10 * *b00  +  *a11 * *b10  +  *a12 * *b20  +  *a13 * *b30 ;
    *(m++) = *a10 * *b01  +  *a11 * *b11  +  *a12 * *b21  +  *a13 * *b31 ;
    *(m++) = *a10 * *b02  +  *a11 * *b12  +  *a12 * *b22  +  *a13 * *b32 ;
    *(m++) = *a10 * *b03  +  *a11 * *b13  +  *a12 * *b23  +  *a13 * *b33 ;

    *(m++) = *a20 * *b00  +  *a21 * *b10  +  *a22 * *b20  +  *a23 * *b30 ;
    *(m++) = *a20 * *b01  +  *a21 * *b11  +  *a22 * *b21  +  *a23 * *b31 ;
    *(m++) = *a20 * *b02  +  *a21 * *b12  +  *a22 * *b22  +  *a23 * *b32 ;
    *(m++) = *a20 * *b03  +  *a21 * *b13  +  *a22 * *b23  +  *a23 * *b33 ;

    *(m++) = *a30 * *b00  +  *a31 * *b10  +  *a32 * *b20  +  *a33 * *b30 ;
    *(m++) = *a30 * *b01  +  *a31 * *b11  +  *a32 * *b21  +  *a33 * *b31 ;
    *(m++) = *a30 * *b02  +  *a31 * *b12  +  *a32 * *b22  +  *a33 * *b32 ;
    *(m++) = *a30 * *b03  +  *a31 * *b13  +  *a32 * *b23  +  *a33 * *b33 ;

    m = (float*) mi ;/* reinitialisation de m pour le transfert en c */

    *(c++) = *(m++) ; *(c++) = *(m++) ; *(c++) = *(m++) ; *(c++) = *(m++) ;
    *(c++) = *(m++) ; *(c++) = *(m++) ; *(c++) = *(m++) ; *(c++) = *(m++) ;
    *(c++) = *(m++) ; *(c++) = *(m++) ; *(c++) = *(m++) ; *(c++) = *(m++) ;
    *(c++) = *(m++) ; *(c++) = *(m++) ; *(c++) = *(m++) ; *(c) = *(m) ;
  }
  else
  {
    *(c++) = *a00 * *b00  +  *a01 * *b10  +  *a02 * *b20  +  *a03 * *b30 ;
    *(c++) = *a00 * *b01  +  *a01 * *b11  +  *a02 * *b21  +  *a03 * *b31 ;
    *(c++) = *a00 * *b02  +  *a01 * *b12  +  *a02 * *b22  +  *a03 * *b32 ;
    *(c++) = *a00 * *b03  +  *a01 * *b13  +  *a02 * *b23  +  *a03 * *b33 ;

    *(c++) = *a10 * *b00  +  *a11 * *b10  +  *a12 * *b20  +  *a13 * *b30 ;
    *(c++) = *a10 * *b01  +  *a11 * *b11  +  *a12 * *b21  +  *a13 * *b31 ;
    *(c++) = *a10 * *b02  +  *a11 * *b12  +  *a12 * *b22  +  *a13 * *b32 ;
    *(c++) = *a10 * *b03  +  *a11 * *b13  +  *a12 * *b23  +  *a13 * *b33 ;

    *(c++) = *a20 * *b00  +  *a21 * *b10  +  *a22 * *b20  +  *a23 * *b30 ;
    *(c++) = *a20 * *b01  +  *a21 * *b11  +  *a22 * *b21  +  *a23 * *b31 ;
    *(c++) = *a20 * *b02  +  *a21 * *b12  +  *a22 * *b22  +  *a23 * *b32 ;
    *(c++) = *a20 * *b03  +  *a21 * *b13  +  *a22 * *b23  +  *a23 * *b33 ;

    *(c++) = *a30 * *b00  +  *a31 * *b10  +  *a32 * *b20  +  *a33 * *b30 ;
    *(c++) = *a30 * *b01  +  *a31 * *b11  +  *a32 * *b21  +  *a33 * *b31 ;
    *(c++) = *a30 * *b02  +  *a31 * *b12  +  *a32 * *b22  +  *a33 * *b32 ;
    *(c)   = *a30 * *b03  +  *a31 * *b13  +  *a32 * *b23  +  *a33 * *b33 ;
  }
}




/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/


void 	mr3d_idt_(MR3D	mr)
{
  register float	*m ;

  m = (float*)mr ; 

  *(m++) = 1.	; *(m++) = 0.	  ; *(m++) = 0.	    ; 
  *(m++) = 0.     ; *(m++) = 1.	  ; *(m++) = 0.     ; 
  *(m++) = 0.     ; *(m++) = 0.	  ; *m     = 1.	    ; 
}



/*-------------------------------------------------------------------*/
void	mr3d_self_transpose(MR3D ma)
{
  register    float	w ;
  register    float	*a ;

  a = (float*)ma ;

  w = *(a+1) ; *(a+1) = *(a+3) ; *(a+3) = w ;
  w = *(a+2) ; *(a+2) = *(a+6) ; *(a+6) = w ;
  w = *(a+5) ; *(a+5) = *(a+7) ; *(a+7) = w ;
}

/*-------------------------------------------------------------------*/
/* effectue la multiplication des matrices 3x3   :    a*b -> c		*/

void	mr3d_mult(MR3D  a, MR3D  b, MR3D  c)
{
  MR3D	w;
  register int	i,j,k ;

  for(i=0 ; i<3 ; i++)
  {
    for(j=0 ; j<3 ; j++)
    {
      w[i][j] = 0. ;
      for(k=0 ; k<3 ; k++)
	 w[i][j] += a[i][k]*b[k][j] ;
    }
  }

  for(i=0 ; i<3 ; i++)
     for(j=0 ; j<3 ; j++)
	c[i][j] = w[i][j] ;    
}

/*-------------------------------------------------------------------*/
/* multiplication dans l'espace 3d  u*a  donnant le vecteur v		*/
/* version optimisee (ancienne version en commentaire)			*/

void	mr3d_mult_vt3d(MR3D	    ma,
		       VT3D	    vu,
		       VT3D	    vv)
{
  static	VT3D	vw;
  register    float	*w ,*a , *u , *v ;
  register    float	*u0 , *u1 , *u2 ;
 
  w  = (float*)vw ;
  a  = (float*)ma ;
  u  = (float*)vu ; 
  u0 = u++ ; u1 = u++ ; u2 = u ;

  *(w++) = *u0 * *a       +  *u1 * *(a+3)   +  *u2 * *(a+6);
  *(w++) = *u0 * *(a+1)   +  *u1 * *(a+4)   +  *u2 * *(a+7);
  *w     = *u0 * *(a+2)   +  *u1 * *(a+5)   +  *u2 * *(a+8);

  w  = (float*)vw ;
  v  = (float*)vv ;
  *(v++) = *(w++) ; *(v++) = *(w++) ; *v = *w ;
}

/*-------------------------------------------------------------------*/
#define EPSILON 1e-4

/*-------------------------------------------------------------------*/
/* conversion d'un vecteur de rotation en matrice de rotation		*/
/* en passant par un quaternion (source toolkit matrix_h.c)		*/


void	mr3d_from_axis_angle(MR3D mr, VT3D axis)
{
  VT3D	    k;
  float    ang;               /* scalar angle of rotation */
  float    c, s, v;           /* cosine, sine, versine of ang */

  mr3d_idt_(mr);

  /* these equations are set up for cw rot. */

  ang = sqrt((double)(axis[0]*axis[0] + axis[1]*axis[1] + axis[2]*axis[2])) ;

  if (ang < EPSILON)
     return;

  k[0] = axis[0]/ang ;	      /* normalization */
  k[1] = axis[1]/ang ;
  k[2] = axis[2]/ang ;

  c = cos( ang );             /* in right-handed, ccw rot in left-handed */
  s = sin( ang );
  v = 1.0 - c;                /* versine(x) = 1 - cos(x) */

  mr[0][0] = k[0] * k[0] * v + c;
  mr[0][1] = k[0] * k[1] * v + k[2] * s;
  mr[0][2] = k[0] * k[2] * v - k[1] * s;

  mr[1][0] = k[1] * k[0] * v - k[2] * s;
  mr[1][1] = k[1] * k[1] * v + c;
  mr[1][2] = k[1] * k[2] * v + k[0] * s;

  mr[2][0] = k[2] * k[0] * v + k[1] * s;
  mr[2][1] = k[2] * k[1] * v - k[0] * s;
  mr[2][2] = k[2] * k[2] * v + c;
}

/*-------------------------------------------------------------------*/
/* conversion d'une matrice de rotation en vecteur de rotation		*/
/* en passant par un quaternion (source toolkit matrix_h.c)		*/

void	mr3d_to_axis_angle(VT3D	axis, MR3D mr)
{
  float angle, norm, ww, xx, yy, four_w, two_x;
  double scalar ;
  VHOM  quat ;

  ww = 0.25 * ( 1.0 + mr[0][0] + mr[1][1] + mr[2][2]);

  if (ww > EPSILON)
  {
    quat[3] = /*f*/sqrt((double)ww);
    four_w  = 4.0 * quat[3];
    quat[0] = (mr[1][2] - mr[2][1]) / four_w;
    quat[1] = (mr[2][0] - mr[0][2]) / four_w;
    quat[2] = (mr[0][1] - mr[1][0]) / four_w;
          
  }
  else
  {
    quat[3] = 0.0;
    xx = -0.5 * (mr[1][1] + mr[2][2]);

    if (xx > EPSILON)
    {
      quat[0] = /*f*/sqrt((double)xx);
      two_x = quat[0] * 2.0;
      quat[1] = mr[0][1] / two_x;
      quat[2] = mr[0][2] / two_x;
    }
    else
    {
      quat[0] = 0.0;
      yy = 0.5 * (1.0 - mr[2][2]);

      if (yy > EPSILON)
      {
	quat[1] = /*f*/sqrt((double)yy);
	quat[2] = mr[1][2] / (2.0 * quat[1]);
      }
      else
      {
	quat[1] = 0.0;
	quat[2] = 1.0;
      }
    }
  }


  /* normalize the quaternion */
  norm = /*f*/sqrt((double)
    quat[0] * quat[0] +
    quat[1] * quat[1] +
    quat[2] * quat[2] +
    quat[3] * quat[3]);

  if (norm > EPSILON)
  {
    quat[0] /= norm;
    quat[1] /= norm;
    quat[2] /= norm;
    quat[3] /= norm;
  }
  else
  {
    quat[0] = 0.0;
    quat[1] = 0.0;
    quat[2] = 0.0;
    quat[3] = 1.0;
  }

  /* calculate angle */
  angle = 2.0 * /*f*/acos((double)quat[3]);

  axis[0] = quat[0];
  axis[1] = quat[1];
  axis[2] = quat[2];

  norm = sqrt((double)(axis[0]*axis[0] + axis[1]*axis[1] + axis[2]*axis[2])) ;
  if (norm > EPSILON)
  {
    scalar  = angle/norm ;
    axis[0] = scalar * axis[0] ;
    axis[1] = scalar * axis[1] ;
    axis[2] = scalar * axis[2] ;
  }
  else
  {
    axis[0] = angle ;
    axis[1] = 0. ;
    axis[2] = 0. ;
  }
}



/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/



void	vt3d_copy(VT3D src, VT3D dst)
{
  dst[0] = src[0];
  dst[1] = src[1];
  dst[2] = src[2];
}


void	vt3d_set_null(VT3D dst)
{
  dst[0] = 0.0;
  dst[1] = 0.0;
  dst[2] = 0.0;
}



/*-------------------------------------------------------------------*/

private DIT *	dit_let(short type)
{
  switch((int)type)
  {
    case IDENTITY:	         return(&let_dit_idt);
    case Z_ON_X:	         return(&let_dit_z_on_x);
    case Z_ON_Y:	         return(&let_dit_z_on_y);
    case Z_ON_MINUS_Z:       return(&let_dit_z_on_minus_z);
    case Z_ON_MINUS_X:       return(&let_dit_z_on_minus_x);
    case Z_ON_MINUS_Y:       return(&let_dit_z_on_minus_y);
    case GL_TO_BODYLIB_Y_UP: return(&let_dit_gl_to_bodylib_y_up);
    case GL_TO_BODYLIB_Z_UP: return(&let_dit_gl_to_bodylib_z_up);
    default:    printf("error in let_dit : bad DIT type\n");	    
    return(NULL);
  }
}



static	void	joint_ini(JOINT		*self);

static	void	joint_actual_dit(short	type , 
				 float	new_value, 
				 DIT	*new_dit);


JOINT	*joint_creation(void)
{
  JOINT	*self;

  self = (JOINT*) malloc((size_t) sizeof(JOINT));
  if(!self) {printf("erreur allocation memoire \n"); exit(0);}

  joint_ini(self);
  return(self);
}





/*-------------------------------------------------------------------*/

Booleen	joint_set_limits(JOINT	*self , float	new_min, float new_max)
{
  if(self)
  {
    if(new_min > new_max)
    {
      (void) fprintf(stderr, 
		     "joint_set_limits(): new min & max inconsistants (min > max)\n");
      return(NO);
    }
    if(self->limit)
    {
      if(self->val < new_min)
      {
	self->val = new_min;
	joint_actual_dit(self->type,self->val,self->art);
	self->artok = FALSE ;
	if(self->n3d != NULL)
	   n3d_outdate_ref(self->n3d);
      }
      else if(self->val > new_max)
      {
	self->val = new_max;
	joint_actual_dit(self->type,self->val,self->art);
	self->artok = FALSE ;
	if(self->n3d != NULL)
	   n3d_outdate_ref(self->n3d);
      }
    }
    self->bmin = new_min;
    self->bmax = new_max;
    return(TRUE);
  }
  JOINT_UNDEFINED("set_min");
  return(FALSE);
}


/*-------------------------------------------------------------------*/

Booleen	joint_set_type(JOINT *self, short typ)
{
  if (self)
  {
    switch(typ)
    { 
      /*	 case JOINT_TOTALE:*/
      case JOINT_ROTATION:
      case JOINT_TRANSLATION:
      self->type = typ;
      joint_actual_dit(self->type, self->val, self->art);
      self->artok = FALSE ;
      if(self->n3d != NULL)
	 n3d_outdate_ref(self->n3d);
      return(TRUE);
      default:
      fprintf(stderr, "joint_set_type(): joint type undefined\n");
      return(FALSE);
    }
  }
  JOINT_UNDEFINED("joint_set_type")
    return(FALSE);
}	


/*-------------------------------------------------------------------*/

Booleen	joint_val_actual(JOINT	*self , 
			 float	new_value)
{
  if(self)
  {

    if(self->limit)
    {
      if(new_value > self->bmax)	new_value = self->bmax ;
      else if(new_value < self->bmin)  new_value = self->bmin ;
    }

    self->val = new_value ;
    joint_actual_dit(self->type,self->val,self->art);
    self->artok = FALSE ;
    if(self->n3d != NULL)
       n3d_outdate_ref(self->n3d);
    return(TRUE);
  }
  JOINT_UNDEFINED("val_actual")
    return(FALSE);
}



N3D *   joint_get_n3d(JOINT * self)
{
  if(self != NULL) return(self->n3d);
  return(NULL);
}


/*-------------------------------------------------------------------*/
MatriX	*joint_let_loc_inv(JOINT *self)
{
  if(self){return((MatriX*)(self->art->inv));}
  JOINT_UNDEFINED("let_loc_inv")
    return(FALSE);
}


/*-------------------------------------------------------------------*/

float	joint_get_val(JOINT *self)
{
  if(self){return(self->val);}
  JOINT_UNDEFINED("get_val")
    return(0.);
}



private Booleen	joint_n3d_attach(JOINT *self, N3D *n3d)
{
  if(self)
  {
    if(n3d) self->n3d = n3d ;
    else	  printf("warning : NULL node_3d associated to joint\n");
    return(TRUE);
  }
  JOINT_UNDEFINED("n3d_attach")
    return(FALSE);
}



/*-------------------------------------------------------------------*/
/* Une joint_ est initialisee par def. au type JOINT_ROTATION ds l'etat LIBRE*/


static	void	joint_ini(JOINT	*self)
{
  self->n3d = NULL; /* DEBUG */
	
  self->type   = JOINT_ROTATION ;

  self->artok  = TRUE ;
  self->val    = 0.;
  self->vm1    = 0.;
  self->vm2    = 0.;
  self->limit  = TRUE ;
  self->bmin   = - MM_PI ;
  self->bmax   =   MM_PI ;
  self->vmax   =   MM_PI ;

  self->art    = dit_creation() ;

}


/*-------------------------------------------------------------------*/

static	void	joint_actual_dit(short	type , 
				 float	new_value, 
				 DIT	*new_dit)
{
  switch(type)
  {
    case JOINT_ROTATION:
    /* implicitement selon l'axe z du repere de depart associe	*/
    dit_actual_rot(new_value,new_dit) ;
    break;
    case JOINT_TRANSLATION:
    /* implicitement selon l'axe z du repere de depart associe	*/
    dit_actual_trn(new_value,new_dit) ;
    break;
    default:
    printf("warning : JOINT type not implemented \n");
  } 
}


struct Vec3f {
      float x;
      float y;
      float z;
};

typedef struct Vec3f Vec3f;


static float f_max (float a, float b)
{
  return (a > b ? a : b);
}


/* This is the highest scene file version that is currently supported
   by this module. It is also the version that is written by scene_write() */


struct scene_
{
      char	    *idt;   
      char	    *file;  
      UNIT_t    *top;   
      int id_unit; 
      int id_call; 

};

static	void	  scene_ini(SCENE_t	*self, char *idt);
static  Booleen_t scene_unit_id_write(SCENE_t *self, FILE *f);

static  Booleen_t scene_trace_unit(SCENE_t *self, int index, int new_index);
static  Booleen_t scene_trace_dependancy(SCENE_t *self, int icall, 
					 int new_icall);
static  Booleen_t scene_store_dependancy(SCENE_t *self, int ida, int idb, 
					 Booleen_t mode, int idep);
static  void	  scene_purge_new(SCENE_t *self);

static  Booleen_t scene_solve_dependancy(SCENE_t *self);
static  Booleen_t scene_update_data_call(SCENE_t *self, UNIT_t * va, 
					 UNIT_t * vb, 
					 Booleen_t mode, int index);
static Booleen_t  scene_lowlevel_destroy(SCENE_t   *self);


SCENE_t	*scene_new(char *idt)
{
  SCENE_t	*self;

  self = (SCENE_t*) malloc((size_t) sizeof(SCENE_t));
  if(!self) {printf("memory allocation error\n"); exit(0);}
	
  scene_ini(self, idt);

  if(!(self->top = unit_new(self,SCENE_TOP_UNIT_NAME,NULL)))
  {printf("error in scene creation \n"); exit(0);}

  return(self);
}



/*--------------------------------------------------------------------*/
UNIT	*scene_get_top(SCENE_t *self)
{
  if(self)    return(self->top);
  SCENE_UNDEFINED("get top");
  return(NULL);
}


private int	scene_new_unit(SCENE_t *self, void *v)
{
  if(self)
  {

    return((self->id_unit)++);
  }
  SCENE_UNDEFINED("new unit index");
  return(ID_ERROR);
}



private int	scene_new_call(SCENE_t *self, UNIT_t * parent, UNIT_t * child)
{
  if(self)
  {
    return((self->id_call)++ );


  }
  SCENE_UNDEFINED("new call");
  return(ID_ERROR);
}




private Booleen_t	scene_set_top(SCENE_t *self, UNIT_t * top)
{
  if(self && top)
  {
    if(self->top)  
    {
      if( !unit_lowlevel_destroy(self->top))
      {
	(void) fprintf(stderr, 
		       "scene_set_top(): error occurs in "
		       "unit_destroy() of old top unit\n");
	return(NO);
      }
    }
    self->top = top;
    return(YES);
  }
  else if( self)
  {
    /*  usefull before destroying the scene */
    self->top = NULL;
    return(YES);
  }
  SCENE_UNDEFINED("set top");
  return(NO);
}




static	void	scene_ini(SCENE_t	*self,char *idt)
{
  self->idt = NULL ;
  SET_IDT(self,idt)
    self->top = NULL;

  self->id_unit	    = 0;

  self->id_call	    = 0;
}



/*  This static variable may be defined in local application to
    specify your world's system vertical orientation in one of 
    the predefined below :
    classic mathematician notation for 3D system :
    (Plan XoZ parallel to screen)
    FREE_Z_UP
    classic informatician notation for 3D system display :
    (Plan XoY parallel to screen)
    FREE_Y_UP	    (default)
*/
static	int	free_coord_up = FREE_Y_UP;


static	void	free_ini(FREE_		    *self);
static	void	free_actual_rot(short	    rt_type, 
				VT3D    new_value_rt , 
				float   new_value_a , 
				MR3D    mr);



FREE_	*free_creation(void)
{
  FREE_	*self;

  self = (FREE_*) malloc((size_t) sizeof(FREE_));
  if(!self) {printf("erreur allocation memoire \n"); exit(0);}

  free_ini(self);
  return(self);
}



N3D *   free_get_n3d(FREE_ * self)
{
  if(self != NULL) return(self->n3d);
  return(NULL);
}

/*-------------------------------------------------------------------*/

MatriX	*free_let_loc_inv(FREE_ *self)
{
  if(self){return((MatriX*)(self->lib->inv));}
  FREE_UNDEFINED("let_loc_inv")
    return(FALSE);
}



Booleen	free_n3d_attach(FREE_ *self, N3D *n3d)
{
  if(self)
  {
    if(n3d)
    {
      self->libok = FALSE;
      if(self->n3d != NULL)
	 n3d_outdate_ref(self->n3d);
      self->n3d = n3d ;
      return(TRUE);
    }
    printf("warning : NULL node_3d associated to free\n");
    return(FALSE);
  }
  FREE_UNDEFINED("n3d_attach")
    return(FALSE);
}


/*-------------------------------------------------------------------*/

static	void	free_ini(FREE_	*self)
{
  int	i;

  self->n3d       = NULL;
  self->mode	= FREE_DEFAULT_MODE ;
  self->param     = FALSE ;
  self->rt_type   = RT_XYZ ;
  self->libok     = TRUE ;

  for(i=0 ; i<3 ; i++)
  { 
    self->rt[i]      = 0.;
    self->rtm1[i]    = 0.;
    self->rtm2[i]    = 0.;
  }
  self->a	  = 0. ;
  self->am1	  = 0. ;
  self->am2	  = 0. ;

  self->lib    = dit_creation() ;
  self->libm1  = dit_creation() ;
  self->libm2  = dit_creation() ;

}


/*-------------------------------------------------------------------*/

struct world
{
      N3D	    *n3d;	
      int  id_n3d;	

};

static	void	world_ini(WORLD	*self);


private WORLD *	world_creation(void)
{
  WORLD	*self;

  self = (WORLD*) malloc((size_t) sizeof(WORLD));
  if(!self) {printf("erreur allocation memoire \n"); exit(0);}

  world_ini(self);
  return(self);
}


private int	world_new_n3d(WORLD *self, void *v)
{
  if(self)
  {
    return((self->id_n3d)++);
  }
  WORLD_UNDEFINED("new n3d");
  return(ID_ERROR);
}


private Booleen world_n3d_attach(WORLD *self, N3D *n3d)
{
  if(self)
  { self->n3d = n3d ;
  return(TRUE);
  }
  WORLD_UNDEFINED("attach");
  return(FALSE);
}


static	void	world_ini(WORLD	*self)
{
  self->n3d	= NULL ;

  self->id_n3d	= 0;
}


