
/*------------------------------------------------------------------------------*/
/* 										*/
/* 										*/
/* This software module was originally developed by Ronan Boulic and Tom Molet 	*/
/*  (EPFL-LIG) and edited by Joaquim Esmerado (EPFL-LIG) 			*/
/*  in the course of development of the MPEG-4 SNHC 				*/
/*  VM. This software module is an implementation of a part of one or more 	*/
/*  MPEG-4 SNHC VM tools as specified by the MPEG-4 SNHC VM. ISO/IEC gives 	*/
/*  users of the MPEG-4 SNHC VM free license to this software module or 	*/
/*  modifications thereof for use in hardware or software products 		*/
/*  claiming conformance to the MPEG-4 SNHC VM. Those intending to use 		*/
/*  this software module in hardware or software products are advised that 	*/
/*  its use may infringe existing patents. The original developer of this 	*/
/*  software module and his/her company, the subsequent editors and their 	*/
/*  companies, and ISO/IEC have no liability for use of this software 		*/
/*  module or modifications thereof. Copyright is not released for non 		*/
/*  MPEG-4 SNHC VM conforming products. EPFL-LIG 				*/
/*  retains full right to use the code for its own purpose, assign or 		*/
/*  donate the code to a third party and to inhibit third parties from 		*/
/*  using the code for non MPEG-4 SNHC VM conforming products. This 		*/
/*  copyright notice must be included in all copies or derivative 		*/
/*  works. Copyright (1997).  							*/

#include <stdio.h>
#include <stdlib.h>


#include "scn1.h"
#include "etall.h"
#include "private.h"

#define MaxCharPerLine  80

FILE *SkelDataFile;
FILE *HandDataFile;

static N3D *IO;


static  BODY_t *        body_create(void);

static  void    body_ini(BODY_t * self);

static  void    body_set_tables(BODY_t * self);

static  Booleen_t body_switch_mode(BODY_t * self, MM val1, Booleen_t Glomot);

/*static	void	body_iniT_joint_limits(BODY_t * self);*/

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/



void
nextUsableLine(FILE *ifp) {
  char firstChar;
  char buf[MaxCharPerLine];

  while(feof(ifp) == 0) {
    firstChar = fgetc(ifp);
    if (firstChar == '#') {
      fgets(buf, MaxCharPerLine, ifp);
    } else if(firstChar == '\t' || firstChar == ' ' ||
	      firstChar == '\n') {
      ;
    } else {
      ungetc (firstChar, ifp);
      return;
    }
  }
}



void
read_body_data( FILE *file, char *bodyPartName, float xt[3], float xr[3], float mrdir[3][3])
{
  /*      static FILE *fileP;*/
  char buf[MaxCharPerLine];
  char firstChar;
  int len, i, j,k,l;

  len = strlen( bodyPartName );

  rewind( file );

  while ( !feof( file ) ) {
    nextUsableLine( file );
    fgets(buf, MaxCharPerLine, file);
    if (!strncmp( buf, bodyPartName, len ) ) {
      for ( i=0; i<3; i++ ) {
	nextUsableLine( file );
	fgets(buf, MaxCharPerLine, file);
	sscanf( buf, "%f", &xt[i] );
      }
      for ( i=0; i<3; i++ ) {
	nextUsableLine( file );
	fgets(buf, MaxCharPerLine, file);
	sscanf( buf, "%f", &xr[i] );
      }
      nextUsableLine( file ); j = k = 0;
      while ( (firstChar = fgetc( file )) == 'm') {
	/*                      if (firstChar == 'm') {}*/
	/*                              ungetc ( firstChar, file );*/
	fgets( buf, MaxCharPerLine, file );
	fgets( buf, MaxCharPerLine, file );
	sscanf( buf, "%f", &mrdir[j][k++] );
	nextUsableLine( file );
	if ( k==3 ) { k = 0; j++; }
      }
    }
  }
}





void
read_hands_data( FILE *file, char *bodyPartName,
		 float *val, float xt[3], float xr[3], float xt_x[3], float xr_x[3])
{

  char buf[MaxCharPerLine];
  char firstChar;
  int len, i, j,k,l;

  len = strlen( bodyPartName );
  rewind( file );

  while ( !feof( file ) ) {
    nextUsableLine( file );
    fgets(buf, MaxCharPerLine, file);
    if (!strncmp( buf, bodyPartName, len ) ) {
      nextUsableLine( file );
      if ( (firstChar = fgetc( file )) == 'v') {
	fgets( buf, MaxCharPerLine, file );
	fgets( buf, MaxCharPerLine, file );
	sscanf( buf, "%f", val );

      }
      ungetc ( firstChar, file );
      nextUsableLine( file ); j = k = 0;
      while ( (firstChar = fgetc( file )) == 'l' || firstChar == 'r') {


	for ( i=0; i<3; i++ ) {
	  nextUsableLine( file );
	  fgets( buf, MaxCharPerLine, file );
	  nextUsableLine( file );
	  fgets(buf, MaxCharPerLine, file);
	  sscanf( buf, "%f", &xt[i] );

	}
	for ( i=0; i<3; i++ ) {
	  nextUsableLine( file );
	  fgets( buf, MaxCharPerLine, file );
	  nextUsableLine( file );
	  fgets(buf, MaxCharPerLine, file);
	  sscanf( buf, "%f", &xr[i] ) ;

	}

	for ( i=0; i<3; i++ ) {
	  nextUsableLine( file );
	  fgets(buf, MaxCharPerLine, file);
	  nextUsableLine( file );
	  fgets(buf, MaxCharPerLine, file);
	  sscanf( buf, "%f", &xt_x[i] );

	}
	for ( i=0; i<3; i++ ) {
	  nextUsableLine( file );
	  fgets(buf, MaxCharPerLine, file);
	  nextUsableLine( file );
	  fgets(buf, MaxCharPerLine, file);
	  sscanf( buf, "%f", &xr_x[i] ) ;

	}
      }
    }
  }
}




static	void	body_iniT_tables(BODY_t * self)
{
  char * BuffPath;
  char * BdyDataPath;
  char bodyfile[80];
  char BdyDataName[60];
  char HndDataName[60];

  N3D	*   root, * child;
  JOINT * joint;
  FREE_ * free_;

  float     th;

  float xt[3];
  float xr[3];
  float mrdir_a[3][3];


  N3D_t * root_low_tables;
  N3D	   * child_low_tables;
  JOINT * joint_low_tables;

  N3D_t * root_rleg_tables;
  N3D	   * child2_rleg_tables, * child3_rleg_tables;
  JOINT *  joint_rleg_tables;

  N3D_t * root_lleg_tables;
  N3D	   * child2_lleg_tables, * child3_lleg_tables;
  JOINT * joint_lleg_tables;

  N3D_t * root_high_tables;
  N3D	   * child_high_tables;
  JOINT * joint_high_tables;

  N3D_t * root_vt6up_tables;
  N3D	   * child_vt6up_tables;
  JOINT * joint_vt6up_tables;

  N3D_t * root_larm_tables;
  N3D	   * child_larm_tables;
  JOINT * joint_larm_tables;

  N3D_t * root_rarm_tables;
  N3D	   * child_rarm_tables;
  JOINT * joint_rarm_tables;


  MatriX      matdir;
  MR3D        mrdir;
  VT3D        r_tr;
  VT3D        r_rt;

  BdyDataPath = getenv( "BDYDATAPATH" );

  sprintf( BdyDataName, "%s%s%s",
           (BdyDataPath)?BdyDataPath:"",
           (BdyDataPath)?"/":"",
           "body.dat");

  sprintf( HndDataName, "%s%s%s",
           (BdyDataPath)?BdyDataPath:"",
           (BdyDataPath)?"/":"",
           "hand.dat");

   if (!(SkelDataFile = fopen ( BdyDataName/*"DATA2"*/, "rb" ) )) {
    fprintf( stderr, "Error: could not open file %s\n", BdyDataName/*"DATA2"*/ );
    fprintf( stderr, "Check BDYDATAPATH env variable also\n" );
    exit( 1 );
  }

  if (!(HandDataFile = fopen ( HndDataName/*"DATA5"*/, "rb" ) )) {
    fprintf( stderr, "Error: could not open file %s\n", HndDataName/*"DATA5"*/ );
    fprintf( stderr, "Check BDYDATAPATH env variable also\n" );
    exit( 1 );
  }



  /* -------------------------------------- */


  root = unit_get_root(self->golem);
  if(!root)
  {
    fprintf(stderr, " body_set_tn3d(): h3d root undefined\n" );
    return;
  }
  if(self->tn3d && self->tjoint)
  {

    /* TEST */
    self->tn3d[BODY_N3D_VIRTUAL_FLOOR] = NULL;
    /* TEST */
    
    child = n3d_creation_child(root, "global_sht");
    free_ = free_creation();
    n3d_attach_data(child, N3D_FREE, free_);
      
    self->tn3d[90] = child;
    root = child;

    child = n3d_creation_child(root, BODY_NAME_GLOBAL_MOTION);
    free_ = free_creation();
    n3d_attach_data(child, N3D_FREE, free_);
      
    self->tn3d[BODY_N3D_GLOBAL_MOTION] = child;
    root = child;
     
    root = child;
      
      
    child = n3d_creation_child(root, BODY_NAME_TR_VERTICAL);
    joint = joint_creation();
    n3d_attach_data(child, N3D_JOINT, joint);
      
    self->tn3d[BODY_N3D_TR_VERTICAL] = child;
      
    self->tjoint[BODY_JOINT_TR_VERTICAL] = (JOINT *) n3d_get_data(child);
      
    joint_set_type(joint, JOINT_TRANSLATION); joint->limit = FALSE;

    root = child;
    child = n3d_creation_child(root, BODY_NAME_TR_LATERAL);
    joint = joint_creation();
    n3d_attach_data(child, N3D_JOINT, joint);
      
    self->tn3d[BODY_N3D_TR_LATERAL] = child;
      
    self->tjoint[BODY_JOINT_TR_LATERAL] = (JOINT *) n3d_get_data(child);
      
    joint_set_type(joint, JOINT_TRANSLATION); joint->limit = FALSE;

    root = child;
    child = n3d_creation_child(root, BODY_NAME_TR_FRONTAL);
    joint = joint_creation();
    n3d_attach_data(child, N3D_JOINT, joint);
      
    self->tn3d[BODY_N3D_TR_FRONTAL] = child;
      
    self->tjoint[BODY_JOINT_TR_FRONTAL] = (JOINT *) n3d_get_data(child);
      
    joint_set_type(joint, JOINT_TRANSLATION); joint->limit = FALSE;

    root = child;
    child = n3d_creation_child(root, BODY_NAME_BODY_TURN);
    joint = joint_creation();
    n3d_attach_data(child, N3D_JOINT, joint);
      
    self->tn3d[BODY_N3D_BODY_TURN] = child;
      
    self->tjoint[BODY_JOINT_BODY_TURN] = (JOINT *) n3d_get_data(child);
      
    root = child;
    child = n3d_creation_child(root, BODY_NAME_BODY_ROLL);
    joint = joint_creation();
    n3d_attach_data(child, N3D_JOINT, joint);
      
    self->tn3d[BODY_N3D_BODY_ROLL] = child;
      
    self->tjoint[BODY_JOINT_BODY_ROLL] = (JOINT *) n3d_get_data(child);
      
    root = child;
    child = n3d_creation_child(root, BODY_NAME_BODY_TILT);
    joint = joint_creation();
    n3d_attach_data(child, N3D_JOINT, joint);
      
    self->tn3d[BODY_N3D_BODY_TILT] = child;
      
    self->tjoint[BODY_JOINT_BODY_TILT] = (JOINT *) n3d_get_data(child);
      
    self->spine_origin = 1063.043457;
      
    root = child;
    child = n3d_creation_child(root, BODY_NAME_SKELETON_ROOT);
      
    read_body_data( SkelDataFile, BODY_NAME_SKELETON_ROOT, xt, xr, mrdir_a);
    read_body_data( SkelDataFile, BODY_NAME_SKELETON_ROOT, r_tr, r_rt, mrdir_a);
      
    mr3d_from_axis_angle(mrdir, r_rt);
    matrix_compose(mrdir, r_tr, matdir);
    n3d_set_ini_dir(child, matdir);
      
      
    self->tn3d[BODY_N3D_SKELETON_ROOT] = child;
      
    root = child;
    child = n3d_creation_child(root, BODY_NAME_PELVIC_TILT);
    joint = joint_creation();
    n3d_attach_data(child, N3D_JOINT, joint);
      
      
    root_low_tables = child;
      
    self->tn3d[BODY_N3D_PELVIC_TILT] = root_low_tables;
      
    self->tjoint[BODY_JOINT_PELVIC_TILT] = (JOINT *) n3d_get_data(root_low_tables);
      
    child_low_tables = n3d_creation_child(root_low_tables, BODY_NAME_PELVIC_TORSION);
    joint_low_tables = joint_creation();
    n3d_attach_data(child_low_tables, N3D_JOINT, joint_low_tables);
      
    self->tn3d[BODY_N3D_PELVIC_TORSION] = child_low_tables;
      
    self->tjoint[BODY_JOINT_PELVIC_TORSION] = (JOINT *) n3d_get_data(child_low_tables);
      
    root_low_tables = child_low_tables;
    child_low_tables = n3d_creation_child(root_low_tables, BODY_NAME_PELVIC_ROLL);
    joint_low_tables = joint_creation();
    n3d_attach_data(child_low_tables, N3D_JOINT, joint_low_tables);
      
    self->tn3d[BODY_N3D_PELVIC_ROLL] = child_low_tables;
      
    self->tjoint[BODY_JOINT_PELVIC_ROLL] = (JOINT *) n3d_get_data(child_low_tables);
      
    root_low_tables = child_low_tables; 
      
      
    child2_rleg_tables = n3d_creation_child(root_low_tables, BODY_NAME_R_HIP_FLEXION);
    joint_rleg_tables = joint_creation();
    n3d_attach_data(child2_rleg_tables, N3D_JOINT, joint_rleg_tables);
      
      
    read_body_data( SkelDataFile, BODY_NAME_R_HIP_FLEXION, xt, xr, mrdir_a);
    mr3d_idt_(mrdir); 
    read_body_data( SkelDataFile, BODY_NAME_R_HIP_FLEXION, r_tr, r_rt, mrdir);
      
      
    matrix_compose(mrdir, r_tr, matdir);
    n3d_set_ini_dir(child2_rleg_tables, matdir);
      
    self->tn3d[BODY_N3D_R_HIP_FLEXION] = child2_rleg_tables;
      
    self->tjoint[BODY_JOINT_R_HIP_FLEXION] = (JOINT *) n3d_get_data(child2_rleg_tables);
      
    root_rleg_tables = child2_rleg_tables;
    child2_rleg_tables = n3d_creation_child(root_rleg_tables, BODY_NAME_R_HIP_ABDUCT);
    joint_rleg_tables = joint_creation();
    n3d_attach_data(child2_rleg_tables, N3D_JOINT, joint_rleg_tables);
      
    self->tn3d[BODY_N3D_R_HIP_ABDUCT] = child2_rleg_tables;
      
    self->tjoint[BODY_JOINT_R_HIP_ABDUCT] = (JOINT *) n3d_get_data(child2_rleg_tables);
      
    root_rleg_tables = child2_rleg_tables;
    child2_rleg_tables = n3d_creation_child(root_rleg_tables, BODY_NAME_R_HIP_TWISTING);
    joint_rleg_tables = joint_creation();
    n3d_attach_data(child2_rleg_tables, N3D_JOINT, joint_rleg_tables);
      
    self->tn3d[BODY_N3D_R_HIP_TWISTING] = child2_rleg_tables;
      
    self->tjoint[BODY_JOINT_R_HIP_TWISTING] = (JOINT *) n3d_get_data(child2_rleg_tables);
      
    root_rleg_tables = child2_rleg_tables;
    child2_rleg_tables = n3d_creation_child(root_rleg_tables, BODY_NAME_R_KNEE_FLEXION);
    joint_rleg_tables = joint_creation();
    n3d_attach_data(child2_rleg_tables, N3D_JOINT, joint_rleg_tables);
      
    read_body_data( SkelDataFile, BODY_NAME_R_KNEE_FLEXION, xt, xr, mrdir_a);
    read_body_data( SkelDataFile, BODY_NAME_R_KNEE_FLEXION, r_tr, r_rt, mrdir_a);
      
      
    mr3d_from_axis_angle(mrdir, r_rt);
    matrix_compose(mrdir, r_tr, matdir);
    n3d_set_ini_dir(child2_rleg_tables, matdir);
      
    self->tn3d[BODY_N3D_R_KNEE_FLEXION] = child2_rleg_tables;
      
    self->tjoint[BODY_JOINT_R_KNEE_FLEXION] = (JOINT *) n3d_get_data(child2_rleg_tables);
      
    root_rleg_tables = child2_rleg_tables;
    child2_rleg_tables = n3d_creation_child(root_rleg_tables, BODY_NAME_R_KNEE_TWISTING);
    joint_rleg_tables = joint_creation();
    n3d_attach_data(child2_rleg_tables, N3D_JOINT, joint_rleg_tables);
      
    self->tn3d[BODY_N3D_R_KNEE_TWISTING] = child2_rleg_tables;
      
    self->tjoint[BODY_JOINT_R_KNEE_TWISTING] = (JOINT *) n3d_get_data(child2_rleg_tables);
      
    root_rleg_tables = child2_rleg_tables;
    child2_rleg_tables = n3d_creation_child(root_rleg_tables, BODY_NAME_R_ANKLE_FLEXION);
    joint_rleg_tables = joint_creation();
    n3d_attach_data(child2_rleg_tables, N3D_JOINT, joint_rleg_tables);
      
    read_body_data( SkelDataFile, BODY_NAME_R_ANKLE_FLEXION, xt, xr, mrdir_a);
    read_body_data( SkelDataFile, BODY_NAME_R_ANKLE_FLEXION, r_tr, r_rt, mrdir_a);
      
      
    mr3d_from_axis_angle(mrdir, r_rt);
    matrix_compose(mrdir, r_tr, matdir);
    n3d_set_ini_dir(child2_rleg_tables, matdir);
      
    self->tn3d[BODY_N3D_R_ANKLE_FLEXION] = child2_rleg_tables;
      
    self->tjoint[BODY_JOINT_R_ANKLE_FLEXION] = (JOINT *) n3d_get_data(child2_rleg_tables);
      
    root_rleg_tables = child2_rleg_tables;
    child2_rleg_tables = n3d_creation_child(root_rleg_tables, BODY_NAME_R_ANKLE_SUBTALAR);
    joint_rleg_tables = joint_creation();
    n3d_attach_data(child2_rleg_tables, N3D_JOINT, joint_rleg_tables);
      
    read_body_data( SkelDataFile, BODY_NAME_R_ANKLE_SUBTALAR, xt, xr, mrdir_a);
    read_body_data( SkelDataFile, BODY_NAME_R_ANKLE_SUBTALAR, r_tr, r_rt, mrdir_a);
      
      
    mr3d_from_axis_angle(mrdir, r_rt);
    matrix_compose(mrdir, r_tr, matdir);
    n3d_set_ini_dir(child2_rleg_tables, matdir);

    self->tn3d[BODY_N3D_R_ANKLE_SUBTALAR] = child2_rleg_tables;

    self->tjoint[BODY_JOINT_R_ANKLE_SUBTALAR] = (JOINT *) n3d_get_data(child2_rleg_tables);

    root_rleg_tables = child2_rleg_tables;

    child3_rleg_tables = n3d_creation_child(root_rleg_tables, BODY_NAME_R_MID_FOOT);
    joint_rleg_tables = joint_creation();
    n3d_attach_data(child3_rleg_tables, N3D_JOINT, joint_rleg_tables);

    read_body_data( SkelDataFile, BODY_NAME_R_MID_FOOT, xt, xr, mrdir_a);
    read_body_data( SkelDataFile, BODY_NAME_R_MID_FOOT, r_tr, r_rt, mrdir_a);


    mr3d_from_axis_angle(mrdir, r_rt);
    matrix_compose(mrdir, r_tr, matdir);
    n3d_set_ini_dir(child3_rleg_tables, matdir);

    self->tn3d[BODY_N3D_R_MID_FOOT] = child3_rleg_tables;
  
    self->tjoint[BODY_JOINT_R_MID_FOOT] = (JOINT *) n3d_get_data(child3_rleg_tables);

    root_rleg_tables = child3_rleg_tables;
    child2_rleg_tables = n3d_creation_child(root_rleg_tables, BODY_NAME_R_TOE_FLEXION); 
    joint_rleg_tables = joint_creation();
    n3d_attach_data(child2_rleg_tables, N3D_JOINT, joint_rleg_tables);

    read_body_data( SkelDataFile, BODY_NAME_R_TOE_FLEXION, xt, xr, mrdir_a);
    read_body_data( SkelDataFile, BODY_NAME_R_TOE_FLEXION, r_tr, r_rt, mrdir_a);
      

    mr3d_from_axis_angle(mrdir, r_rt);
    matrix_compose(mrdir, r_tr, matdir);
    n3d_set_ini_dir(child2_rleg_tables, matdir);
      
    self->tn3d[BODY_N3D_R_TOE_FLEXION] = child2_rleg_tables;
      
    self->tjoint[BODY_JOINT_R_TOE_FLEXION] = (JOINT *) n3d_get_data(child2_rleg_tables);
      
    root_rleg_tables = child2_rleg_tables;
      
      
    child2_lleg_tables = n3d_creation_child(root_low_tables, BODY_NAME_L_HIP_FLEXION);
    joint_lleg_tables = joint_creation();
    n3d_attach_data(child2_lleg_tables, N3D_JOINT, joint_lleg_tables);
      
    read_body_data( SkelDataFile, BODY_NAME_L_HIP_FLEXION, xt, xr, mrdir_a);
    mr3d_idt_(mrdir);
    read_body_data( SkelDataFile, BODY_NAME_L_HIP_FLEXION, r_tr, r_rt, mrdir);
    
    
    matrix_compose(mrdir, r_tr, matdir);
    n3d_set_ini_dir(child2_lleg_tables, matdir);
      
    self->tn3d[BODY_N3D_L_HIP_FLEXION] = child2_lleg_tables;
      
    self->tjoint[BODY_JOINT_L_HIP_FLEXION] = (JOINT *) n3d_get_data(child2_lleg_tables);
      
    root_lleg_tables = child2_lleg_tables;
    child2_lleg_tables = n3d_creation_child(root_lleg_tables, BODY_NAME_L_HIP_ABDUCT);
    joint_lleg_tables = joint_creation();
    n3d_attach_data(child2_lleg_tables, N3D_JOINT, joint_lleg_tables);
      
    self->tn3d[BODY_N3D_L_HIP_ABDUCT] = child2_lleg_tables;
    
    self->tjoint[BODY_JOINT_L_HIP_ABDUCT] = (JOINT *) n3d_get_data(child2_lleg_tables);
      
    root_lleg_tables = child2_lleg_tables;
    child2_lleg_tables = n3d_creation_child(root_lleg_tables, BODY_NAME_L_HIP_TWISTING);
    joint_lleg_tables = joint_creation();
    n3d_attach_data(child2_lleg_tables, N3D_JOINT, joint_lleg_tables);
      
    self->tn3d[BODY_N3D_L_HIP_TWISTING] = child2_lleg_tables;
      
    self->tjoint[BODY_JOINT_L_HIP_TWISTING] = (JOINT *) n3d_get_data(child2_lleg_tables);
      
    root_lleg_tables = child2_lleg_tables;
    child2_lleg_tables = n3d_creation_child(root_lleg_tables, BODY_NAME_L_KNEE_FLEXION);
    joint_lleg_tables = joint_creation();
    n3d_attach_data(child2_lleg_tables, N3D_JOINT, joint_lleg_tables);
      
    read_body_data( SkelDataFile, BODY_NAME_L_KNEE_FLEXION, xt, xr, mrdir_a);
    read_body_data( SkelDataFile, BODY_NAME_L_KNEE_FLEXION, r_tr, r_rt, mrdir_a);
      
      
    mr3d_from_axis_angle(mrdir, r_rt);
    matrix_compose(mrdir, r_tr, matdir);
    n3d_set_ini_dir(child2_lleg_tables, matdir);
      
    self->tn3d[BODY_N3D_L_KNEE_FLEXION] = child2_lleg_tables;
      
    self->tjoint[BODY_JOINT_L_KNEE_FLEXION] = (JOINT *) n3d_get_data(child2_lleg_tables);
      
    root_lleg_tables = child2_lleg_tables;
    child2_lleg_tables = n3d_creation_child(root_lleg_tables, BODY_NAME_L_KNEE_TWISTING);
    joint_lleg_tables = joint_creation();
    n3d_attach_data(child2_lleg_tables, N3D_JOINT, joint_lleg_tables);
      
    self->tn3d[BODY_N3D_L_KNEE_TWISTING] = child2_lleg_tables;
      
    self->tjoint[BODY_JOINT_L_KNEE_TWISTING] = (JOINT *) n3d_get_data(child2_lleg_tables);
      
    root_lleg_tables = child2_lleg_tables;
    child2_lleg_tables = n3d_creation_child(root_lleg_tables, BODY_NAME_L_ANKLE_FLEXION);
    joint_lleg_tables = joint_creation();
    n3d_attach_data(child2_lleg_tables, N3D_JOINT, joint_lleg_tables);
      
    read_body_data( SkelDataFile, BODY_NAME_L_ANKLE_FLEXION, xt, xr, mrdir_a);
    read_body_data( SkelDataFile, BODY_NAME_L_ANKLE_FLEXION, r_tr, r_rt, mrdir_a);
      
      
    mr3d_from_axis_angle(mrdir, r_rt);
    matrix_compose(mrdir, r_tr, matdir);
    n3d_set_ini_dir(child2_lleg_tables, matdir);
      
    self->tn3d[BODY_N3D_L_ANKLE_FLEXION] = child2_lleg_tables;
      
    self->tjoint[BODY_JOINT_L_ANKLE_FLEXION] = (JOINT *) n3d_get_data(child2_lleg_tables);

    root_lleg_tables = child2_lleg_tables;
    child2_lleg_tables = n3d_creation_child(root_lleg_tables, BODY_NAME_L_ANKLE_SUBTALAR);
    joint_lleg_tables = joint_creation();
    n3d_attach_data(child2_lleg_tables, N3D_JOINT, joint_lleg_tables);
      
    read_body_data( SkelDataFile, BODY_NAME_L_ANKLE_SUBTALAR, xt, xr, mrdir_a);
    read_body_data( SkelDataFile, BODY_NAME_L_ANKLE_SUBTALAR, r_tr, r_rt, mrdir_a);
      
      
    mr3d_from_axis_angle(mrdir, r_rt);
    matrix_compose(mrdir, r_tr, matdir);
    n3d_set_ini_dir(child2_lleg_tables, matdir);
      
    self->tn3d[BODY_N3D_L_ANKLE_SUBTALAR] = child2_lleg_tables;
      
    self->tjoint[BODY_JOINT_L_ANKLE_SUBTALAR] = (JOINT *) n3d_get_data(child2_lleg_tables);
      
    root_lleg_tables = child2_lleg_tables;
      
    child3_lleg_tables = n3d_creation_child(root_lleg_tables, BODY_NAME_L_MID_FOOT);
    joint_lleg_tables = joint_creation();
    n3d_attach_data(child3_lleg_tables, N3D_JOINT, joint_lleg_tables);
      
    read_body_data( SkelDataFile, BODY_NAME_L_MID_FOOT, xt, xr, mrdir_a);
    read_body_data( SkelDataFile, BODY_NAME_L_MID_FOOT, r_tr, r_rt, mrdir_a);
      
      
    mr3d_from_axis_angle(mrdir, r_rt);
    matrix_compose(mrdir, r_tr, matdir);
    n3d_set_ini_dir(child3_lleg_tables, matdir);
      
    self->tn3d[BODY_N3D_L_MID_FOOT] = child3_lleg_tables;
      
    self->tjoint[BODY_JOINT_L_MID_FOOT] = (JOINT *) n3d_get_data(child3_lleg_tables);
      
    root_lleg_tables = child3_lleg_tables;
    child2_lleg_tables = n3d_creation_child(root_lleg_tables, BODY_NAME_L_TOE_FLEXION);
    joint_lleg_tables = joint_creation();
    n3d_attach_data(child2_lleg_tables, N3D_JOINT, joint_lleg_tables);
      
    read_body_data( SkelDataFile, BODY_NAME_L_TOE_FLEXION, xt, xr, mrdir_a);
    read_body_data( SkelDataFile, BODY_NAME_L_TOE_FLEXION, r_tr, r_rt, mrdir_a);
      
      
    mr3d_from_axis_angle(mrdir, r_rt);
    matrix_compose(mrdir, r_tr, matdir);
    n3d_set_ini_dir(child2_lleg_tables, matdir);
      
    self->tn3d[BODY_N3D_L_TOE_FLEXION] = child2_lleg_tables;
      
    self->tjoint[BODY_JOINT_L_TOE_FLEXION] = (JOINT *) n3d_get_data(child2_lleg_tables);
      
    root_lleg_tables = child2_lleg_tables;
      
IO= child = n3d_creation_child(root, BODY_NAME_VL1_TILT);
    joint = joint_creation();
    n3d_attach_data(child, N3D_JOINT, joint);
      
      
    root_high_tables = child;
      
    self->tn3d[BODY_N3D_VL1_TILT] = root_high_tables;
      
    self->tjoint[BODY_JOINT_VL1_TILT] = (JOINT *) n3d_get_data(root_high_tables);
      
    child_high_tables = n3d_creation_child(root_high_tables, BODY_NAME_VL2_ROLL);
    joint_high_tables = joint_creation();
    n3d_attach_data(child_high_tables, N3D_JOINT, joint_high_tables);
      
    read_body_data( SkelDataFile, BODY_NAME_VL2_ROLL, xt, xr, mrdir_a);
    read_body_data( SkelDataFile, BODY_NAME_VL2_ROLL, r_tr, r_rt, mrdir_a);
      
      
    mr3d_from_axis_angle(mrdir, r_rt);
    matrix_compose(mrdir, r_tr, matdir);
    n3d_set_ini_dir(child_high_tables, matdir);
      
    self->tn3d[BODY_N3D_VL2_ROLL] = child_high_tables;

    self->tjoint[BODY_JOINT_VL2_ROLL] = (JOINT *) n3d_get_data(child_high_tables);
      
    root_high_tables = child_high_tables;
    child_high_tables = n3d_creation_child(root_high_tables, BODY_NAME_VL2_TILT);
    joint_high_tables = joint_creation();
    n3d_attach_data(child_high_tables, N3D_JOINT, joint_high_tables);
      
    self->tn3d[BODY_N3D_VL2_TILT] = child_high_tables;

    self->tjoint[BODY_JOINT_VL2_TILT] = (JOINT *) n3d_get_data(child_high_tables);
      
    root_high_tables = child_high_tables;
    child_high_tables = n3d_creation_child(root_high_tables, BODY_NAME_VL3_ROLL);
    joint_high_tables = joint_creation();
    n3d_attach_data(child_high_tables, N3D_JOINT, joint_high_tables);
      
    read_body_data( SkelDataFile, BODY_NAME_VL3_ROLL, xt, xr, mrdir_a);
    read_body_data( SkelDataFile, BODY_NAME_VL3_ROLL, r_tr, r_rt, mrdir_a);
      
      
    mr3d_from_axis_angle(mrdir, r_rt);
    matrix_compose(mrdir, r_tr, matdir);
    n3d_set_ini_dir(child_high_tables, matdir);
      
    self->tn3d[BODY_N3D_VL3_ROLL] = child_high_tables;
      
    self->tjoint[BODY_JOINT_VL3_ROLL] = (JOINT *) n3d_get_data(child_high_tables);
      
    root_high_tables = child_high_tables;
    child_high_tables = n3d_creation_child(root_high_tables, BODY_NAME_VL3_TILT);
    joint_high_tables = joint_creation();
    n3d_attach_data(child_high_tables, N3D_JOINT, joint_high_tables);
      
    self->tn3d[BODY_N3D_VL3_TILT] = child_high_tables;

    self->tjoint[BODY_JOINT_VL3_TILT] = (JOINT *) n3d_get_data(child_high_tables);
      
    root_high_tables = child_high_tables;
    child_high_tables = n3d_creation_child(root_high_tables, BODY_NAME_VT4_ROLL);
    joint_high_tables = joint_creation();
    n3d_attach_data(child_high_tables, N3D_JOINT, joint_high_tables);
      
    read_body_data( SkelDataFile, BODY_NAME_VT4_ROLL, xt, xr, mrdir_a);
    read_body_data( SkelDataFile, BODY_NAME_VT4_ROLL, r_tr, r_rt, mrdir_a);
      

    mr3d_from_axis_angle(mrdir, r_rt);
    matrix_compose(mrdir, r_tr, matdir);
    n3d_set_ini_dir(child_high_tables, matdir);

    self->tn3d[BODY_N3D_VT4_ROLL] = child_high_tables;
      
    self->tjoint[BODY_JOINT_VT4_ROLL] = (JOINT *) n3d_get_data(child_high_tables);
      
    root_high_tables = child_high_tables;
    child_high_tables = n3d_creation_child(root_high_tables, BODY_NAME_VT4_TORSION);
    joint_high_tables = joint_creation();
    n3d_attach_data(child_high_tables, N3D_JOINT, joint_high_tables);
      
    self->tn3d[BODY_N3D_VT4_TORSION] = child_high_tables;

    self->tjoint[BODY_JOINT_VT4_TORSION] = (JOINT *) n3d_get_data(child_high_tables);

    root_high_tables = child_high_tables;
    child_high_tables = n3d_creation_child(root_high_tables, BODY_NAME_VT4_TILT);
    joint_high_tables = joint_creation();
    n3d_attach_data(child_high_tables, N3D_JOINT, joint_high_tables);
      
    self->tn3d[BODY_N3D_VT4_TILT] = child_high_tables;

    self->tjoint[BODY_JOINT_VT4_TILT] = (JOINT *) n3d_get_data(child_high_tables);

    root_high_tables = child_high_tables;
    child_high_tables = n3d_creation_child(root_high_tables, BODY_NAME_VT5_ROLL);
    joint_high_tables = joint_creation();
    n3d_attach_data(child_high_tables, N3D_JOINT, joint_high_tables);

    read_body_data( SkelDataFile, BODY_NAME_VT5_ROLL, xt, xr, mrdir_a);
    read_body_data( SkelDataFile, BODY_NAME_VT5_ROLL, r_tr, r_rt, mrdir_a);


    mr3d_from_axis_angle(mrdir, r_rt);
    matrix_compose(mrdir, r_tr, matdir);
    n3d_set_ini_dir(child_high_tables, matdir);

    self->tn3d[BODY_N3D_VT5_ROLL] = child_high_tables;
      
    self->tjoint[BODY_JOINT_VT5_ROLL] = (JOINT *) n3d_get_data(child_high_tables);
      
    root_high_tables = child_high_tables;
    child_high_tables = n3d_creation_child(root_high_tables, BODY_NAME_VT5_TORSION);
    joint_high_tables = joint_creation();
    n3d_attach_data(child_high_tables, N3D_JOINT, joint_high_tables);
      
    self->tn3d[BODY_N3D_VT5_TORSION] = child_high_tables;
      
    self->tjoint[BODY_JOINT_VT5_TORSION] = (JOINT *) n3d_get_data(child_high_tables);
      
    root_high_tables = child_high_tables;
    child_high_tables = n3d_creation_child(root_high_tables, BODY_NAME_VT6_ROLL);
    joint_high_tables = joint_creation();
    n3d_attach_data(child_high_tables, N3D_JOINT, joint_high_tables);
      
      
    root_vt6up_tables = child_high_tables;
      
    read_body_data( SkelDataFile, BODY_NAME_VT6_ROLL, xt, xr, mrdir_a);
    read_body_data( SkelDataFile, BODY_NAME_VT6_ROLL, r_tr, r_rt, mrdir_a);
      
      
    mr3d_from_axis_angle(mrdir, r_rt);
    matrix_compose(mrdir, r_tr, matdir);
    n3d_set_ini_dir(child_high_tables, matdir);

    self->tn3d[BODY_N3D_VT6_ROLL] = root_vt6up_tables;

    self->tjoint[BODY_JOINT_VT6_ROLL] = (JOINT *) n3d_get_data(root_vt6up_tables);

    child_vt6up_tables = n3d_creation_child(root_vt6up_tables, BODY_NAME_VT6_TORSION);
    joint_vt6up_tables = joint_creation();
    n3d_attach_data(child_vt6up_tables, N3D_JOINT, joint_vt6up_tables);

    self->tn3d[BODY_N3D_VT6_TORSION] = child_vt6up_tables;

    self->tjoint[BODY_JOINT_VT6_TORSION] = (JOINT *) n3d_get_data(child_vt6up_tables);

    root_vt6up_tables = child_vt6up_tables;
    child_vt6up_tables = n3d_creation_child(root_vt6up_tables, BODY_NAME_VT6_TILT);
    joint_vt6up_tables = joint_creation();
    n3d_attach_data(child_vt6up_tables, N3D_JOINT, joint_vt6up_tables);

    self->tn3d[BODY_N3D_VT6_TILT] = child_vt6up_tables;
      
    self->tjoint[BODY_JOINT_VT6_TILT] = (JOINT *) n3d_get_data(child_vt6up_tables);

    root_vt6up_tables = child_vt6up_tables;
    child_vt6up_tables = n3d_creation_child(root_vt6up_tables, BODY_NAME_VC7_ROLL);
    joint_vt6up_tables = joint_creation();
    n3d_attach_data(child_vt6up_tables, N3D_JOINT, joint_vt6up_tables);

    read_body_data( SkelDataFile, BODY_NAME_VC7_ROLL, xt, xr, mrdir_a);
    read_body_data( SkelDataFile, BODY_NAME_VC7_ROLL, r_tr, r_rt, mrdir_a);
      

    mr3d_from_axis_angle(mrdir, r_rt);
    matrix_compose(mrdir, r_tr, matdir);
    n3d_set_ini_dir(child_vt6up_tables, matdir);

    self->tn3d[BODY_N3D_VC7_ROLL] = child_vt6up_tables;

    self->tjoint[BODY_JOINT_VC7_ROLL] = (JOINT *) n3d_get_data(child_vt6up_tables);

    root_vt6up_tables = child_vt6up_tables;
    child_vt6up_tables = n3d_creation_child(root_vt6up_tables, BODY_NAME_VC7_TORSION);
    joint_vt6up_tables = joint_creation();
    n3d_attach_data(child_vt6up_tables, N3D_JOINT, joint_vt6up_tables);

    self->tn3d[BODY_N3D_VC7_TORSION] = child_vt6up_tables;

    self->tjoint[BODY_JOINT_VC7_TORSION] = (JOINT *) n3d_get_data(child_vt6up_tables);

    root_vt6up_tables = child_vt6up_tables;
    child_vt6up_tables = n3d_creation_child(root_vt6up_tables, BODY_NAME_VC7_TILT);
    joint_vt6up_tables = joint_creation();
    n3d_attach_data(child_vt6up_tables, N3D_JOINT, joint_vt6up_tables);

    self->tn3d[BODY_N3D_VC7_TILT] = child_vt6up_tables;

    self->tjoint[BODY_JOINT_VC7_TILT] = (JOINT *) n3d_get_data(child_vt6up_tables);

    root_vt6up_tables = child_vt6up_tables;
    child_vt6up_tables = n3d_creation_child(root_vt6up_tables, BODY_NAME_VC8_ROLL);
    joint_vt6up_tables = joint_creation();
    n3d_attach_data(child_vt6up_tables, N3D_JOINT, joint_vt6up_tables);

    read_body_data( SkelDataFile, BODY_NAME_VC8_ROLL, xt, xr, mrdir_a);
    read_body_data( SkelDataFile, BODY_NAME_VC8_ROLL, r_tr, r_rt, mrdir_a);


    mr3d_from_axis_angle(mrdir, r_rt);
    matrix_compose(mrdir, r_tr, matdir);
    n3d_set_ini_dir(child_vt6up_tables, matdir);

    self->tn3d[BODY_N3D_VC8_ROLL] = child_vt6up_tables;

    self->tjoint[BODY_JOINT_VC8_ROLL] = (JOINT *) n3d_get_data(child_vt6up_tables);

    root_vt6up_tables = child_vt6up_tables;
    child_vt6up_tables = n3d_creation_child(root_vt6up_tables, BODY_NAME_VC8_TORSION);
    joint_vt6up_tables = joint_creation();
    n3d_attach_data(child_vt6up_tables, N3D_JOINT, joint_vt6up_tables);

    self->tn3d[BODY_N3D_VC8_TORSION] = child_vt6up_tables;

    self->tjoint[BODY_JOINT_VC8_TORSION] = (JOINT *) n3d_get_data(child_vt6up_tables);

    root_vt6up_tables = child_vt6up_tables;
    child_vt6up_tables = n3d_creation_child(root_vt6up_tables, BODY_NAME_VC8_TILT);
    joint_vt6up_tables = joint_creation();
    n3d_attach_data(child_vt6up_tables, N3D_JOINT, joint_vt6up_tables);
      
    self->tn3d[BODY_N3D_VC8_TILT] = child_vt6up_tables;

    self->tjoint[BODY_JOINT_VC8_TILT] = (JOINT *) n3d_get_data(child_vt6up_tables);

    root_vt6up_tables = child_vt6up_tables;
    child_vt6up_tables = n3d_creation_child(root_vt6up_tables, BODY_NAME_HEAD);

    read_body_data( SkelDataFile, BODY_NAME_HEAD, xt, xr, mrdir_a);
    read_body_data( SkelDataFile, BODY_NAME_HEAD, r_tr, r_rt, mrdir_a);


    mr3d_from_axis_angle(mrdir, r_rt);
    matrix_compose(mrdir, r_tr, matdir);
    n3d_set_ini_dir(child_vt6up_tables, matdir);
      
    self->tn3d[BODY_N3D_HEAD] = child_vt6up_tables;

    
    child_vt6up_tables = n3d_creation_child(root_vt6up_tables, BODY_NAME_VISION);

    read_body_data( SkelDataFile, BODY_NAME_VISION, xt, xr, mrdir_a);
    read_body_data( SkelDataFile, BODY_NAME_VISION, r_tr, r_rt, mrdir_a);


    mr3d_from_axis_angle(mrdir, r_rt);
    matrix_compose(mrdir, r_tr, matdir);
    n3d_set_ini_dir(child_vt6up_tables, matdir);

    self->tn3d[BODY_N3D_VISION] = child_vt6up_tables;

    child_vt6up_tables = n3d_creation_child(root_vt6up_tables, BODY_NAME_TOTAL_HEIGHT);

    read_body_data( SkelDataFile, BODY_NAME_TOTAL_HEIGHT, xt, xr, mrdir_a);
    read_body_data( SkelDataFile, BODY_NAME_TOTAL_HEIGHT, r_tr, r_rt, mrdir_a);


    mr3d_from_axis_angle(mrdir, r_rt);
    matrix_compose(mrdir, r_tr, matdir);
    n3d_set_ini_dir(child_vt6up_tables, matdir);


    self->tn3d[BODY_N3D_TOTAL_HEIGHT] = child_vt6up_tables;


    child_high_tables = n3d_creation_child(root_high_tables, BODY_NAME_L_CLAV_ABDUCT);
    joint_high_tables = joint_creation();
    n3d_attach_data(child_high_tables, N3D_JOINT, joint_high_tables);


    root_larm_tables = child_high_tables;

    read_body_data( SkelDataFile, BODY_NAME_L_CLAV_ABDUCT, xt, xr, mrdir_a);
    read_body_data( SkelDataFile, BODY_NAME_L_CLAV_ABDUCT, r_tr, r_rt, mrdir_a);
    

    mr3d_from_axis_angle(mrdir, r_rt);
    matrix_compose(mrdir, r_tr, matdir);
    n3d_set_ini_dir(child_high_tables, matdir);

    self->tn3d[BODY_N3D_L_CLAV_ABDUCT] = root_larm_tables;
    
    self->tjoint[BODY_JOINT_L_CLAV_ABDUCT] = (JOINT *) n3d_get_data(root_larm_tables);
      
    child_larm_tables = n3d_creation_child(root_larm_tables, BODY_NAME_L_CLAV_ROTATE);
    joint_larm_tables = joint_creation();
    n3d_attach_data(child_larm_tables, N3D_JOINT, joint_larm_tables);

    self->tn3d[BODY_N3D_L_CLAV_ROTATE] = child_larm_tables;

    self->tjoint[BODY_JOINT_L_CLAV_ROTATE] = (JOINT *) n3d_get_data(child_larm_tables);

    root_larm_tables = child_larm_tables;
    child_larm_tables = n3d_creation_child(root_larm_tables, BODY_NAME_L_SCAP_ABDUCT);
    joint_larm_tables = joint_creation();
    n3d_attach_data(child_larm_tables, N3D_JOINT, joint_larm_tables);

    read_body_data( SkelDataFile, BODY_NAME_L_SCAP_ABDUCT, xt, xr, mrdir_a);
    read_body_data( SkelDataFile, BODY_NAME_L_SCAP_ABDUCT, r_tr, r_rt, mrdir_a);
    

    mr3d_from_axis_angle(mrdir, r_rt);
    matrix_compose(mrdir, r_tr, matdir);
    n3d_set_ini_dir(child_larm_tables, matdir);

    self->tn3d[BODY_N3D_L_SCAP_ABDUCT] = child_larm_tables;

    self->tjoint[BODY_JOINT_L_SCAP_ABDUCT] = (JOINT *) n3d_get_data(child_larm_tables);

    root_larm_tables = child_larm_tables;
    child_larm_tables = n3d_creation_child(root_larm_tables, BODY_NAME_L_SCAP_ROTATE);
    joint_larm_tables = joint_creation();
    n3d_attach_data(child_larm_tables, N3D_JOINT, joint_larm_tables);

    self->tn3d[BODY_N3D_L_SCAP_ROTATE] = child_larm_tables;

    self->tjoint[BODY_JOINT_L_SCAP_ROTATE] = (JOINT *) n3d_get_data(child_larm_tables);

    root_larm_tables = child_larm_tables;
    child_larm_tables = n3d_creation_child(root_larm_tables, BODY_NAME_L_SHOULDER_FLEXION);
    joint_larm_tables = joint_creation();
    n3d_attach_data(child_larm_tables, N3D_JOINT, joint_larm_tables);

    read_body_data( SkelDataFile, BODY_NAME_L_SHOULDER_FLEXION, xt, xr, mrdir_a);
    read_body_data( SkelDataFile, BODY_NAME_L_SHOULDER_FLEXION, r_tr, r_rt, mrdir_a);


    mr3d_from_axis_angle(mrdir, r_rt);
    matrix_compose(mrdir, r_tr, matdir);
    n3d_set_ini_dir(child_larm_tables, matdir);

    self->tn3d[BODY_N3D_L_SHOULDER_FLEXION] = child_larm_tables;

    self->tjoint[BODY_JOINT_L_SHOULDER_FLEXION] = (JOINT *) n3d_get_data(child_larm_tables);

    root_larm_tables = child_larm_tables;
    child_larm_tables = n3d_creation_child(root_larm_tables, BODY_NAME_L_SHOULDER_ABDUCT);
    joint_larm_tables = joint_creation();
    n3d_attach_data(child_larm_tables, N3D_JOINT, joint_larm_tables);

    self->tn3d[BODY_N3D_L_SHOULDER_ABDUCT] = child_larm_tables;

    self->tjoint[BODY_JOINT_L_SHOULDER_ABDUCT] = (JOINT *) n3d_get_data(child_larm_tables);

    root_larm_tables = child_larm_tables;
    child_larm_tables = n3d_creation_child(root_larm_tables, BODY_NAME_L_SHOULDER_TWISTING);
    joint_larm_tables = joint_creation();
    n3d_attach_data(child_larm_tables, N3D_JOINT, joint_larm_tables);
      
    self->tn3d[BODY_N3D_L_SHOULDER_TWISTING] = child_larm_tables;
      
    self->tjoint[BODY_JOINT_L_SHOULDER_TWISTING] = (JOINT *) n3d_get_data(child_larm_tables);
      
    root_larm_tables = child_larm_tables;
    child_larm_tables = n3d_creation_child(root_larm_tables, BODY_NAME_L_ELBOW_FLEXION);
    joint_larm_tables = joint_creation();
    n3d_attach_data(child_larm_tables, N3D_JOINT, joint_larm_tables);
      
    read_body_data( SkelDataFile, BODY_NAME_L_ELBOW_FLEXION, xt, xr, mrdir_a);
    read_body_data( SkelDataFile, BODY_NAME_L_ELBOW_FLEXION, r_tr, r_rt, mrdir_a);
      
      
    mr3d_from_axis_angle(mrdir, r_rt);
    matrix_compose(mrdir, r_tr, matdir);
    n3d_set_ini_dir(child_larm_tables, matdir);
      
    self->tn3d[BODY_N3D_L_ELBOW_FLEXION] = child_larm_tables;
      
    self->tjoint[BODY_JOINT_L_ELBOW_FLEXION] = (JOINT *) n3d_get_data(child_larm_tables);
      
    root_larm_tables = child_larm_tables;
    child_larm_tables = n3d_creation_child(root_larm_tables, BODY_NAME_L_ELBOW_TWISTING);
    joint_larm_tables = joint_creation();
    n3d_attach_data(child_larm_tables, N3D_JOINT, joint_larm_tables);
      
    self->tn3d[BODY_N3D_L_ELBOW_TWISTING] = child_larm_tables;
      
    self->tjoint[BODY_JOINT_L_ELBOW_TWISTING] = (JOINT *) n3d_get_data(child_larm_tables);
      
    root_larm_tables = child_larm_tables;
    child_larm_tables = n3d_creation_child(root_larm_tables, BODY_NAME_L_WRIST_FLEXION);
    joint_larm_tables = joint_creation();
    n3d_attach_data(child_larm_tables, N3D_JOINT, joint_larm_tables);
      
    read_body_data( SkelDataFile, BODY_NAME_L_WRIST_FLEXION, xt, xr, mrdir_a);
    read_body_data( SkelDataFile, BODY_NAME_L_WRIST_FLEXION, r_tr, r_rt, mrdir_a);
      
      
    mr3d_from_axis_angle(mrdir, r_rt);
    matrix_compose(mrdir, r_tr, matdir);
    n3d_set_ini_dir(child_larm_tables, matdir);
      
    self->tn3d[BODY_N3D_L_WRIST_FLEXION] = child_larm_tables;
      
    self->tjoint[BODY_JOINT_L_WRIST_FLEXION] = (JOINT *) n3d_get_data(child_larm_tables);
      
    root_larm_tables = child_larm_tables;
    child_larm_tables = n3d_creation_child(root_larm_tables, BODY_NAME_L_WRIST_PIVOT);
    joint_larm_tables = joint_creation();
    n3d_attach_data(child_larm_tables, N3D_JOINT, joint_larm_tables);
      
    self->tn3d[BODY_N3D_L_WRIST_PIVOT] = child_larm_tables;
      
    self->tjoint[BODY_JOINT_L_WRIST_PIVOT] = (JOINT *) n3d_get_data(child_larm_tables);
      
    root_larm_tables = child_larm_tables;
    child_larm_tables = n3d_creation_child(root_larm_tables, BODY_NAME_L_HAND_CENTER);
      
    read_body_data( SkelDataFile, BODY_NAME_L_HAND_CENTER, xt, xr, mrdir_a);
    read_body_data( SkelDataFile, BODY_NAME_L_HAND_CENTER, r_tr, r_rt, mrdir_a);
      
      
    mr3d_from_axis_angle(mrdir, r_rt);
    matrix_compose(mrdir, r_tr, matdir);
    n3d_set_ini_dir(child_larm_tables, matdir);
      
    self->tn3d[BODY_N3D_L_HAND_CENTER] = child_larm_tables;
      
    child_high_tables = n3d_creation_child(root_high_tables, BODY_NAME_R_CLAV_ABDUCT);
    joint_high_tables = joint_creation();
    n3d_attach_data(child_high_tables, N3D_JOINT, joint_high_tables);
      
      
    root_rarm_tables = child_high_tables;
      
    read_body_data( SkelDataFile, BODY_NAME_R_CLAV_ABDUCT, xt, xr, mrdir_a);
    read_body_data( SkelDataFile, BODY_NAME_R_CLAV_ABDUCT, r_tr, r_rt, mrdir_a);
      
      
    mr3d_from_axis_angle(mrdir, r_rt);
    matrix_compose(mrdir, r_tr, matdir);
    n3d_set_ini_dir(child_high_tables, matdir);
      
    self->tn3d[BODY_N3D_R_CLAV_ABDUCT] = root_rarm_tables;
      
    self->tjoint[BODY_JOINT_R_CLAV_ABDUCT] = (JOINT *) n3d_get_data(root_rarm_tables);
      
    child_rarm_tables = n3d_creation_child(root_rarm_tables, BODY_NAME_R_CLAV_ROTATE);
    joint_rarm_tables = joint_creation();
    n3d_attach_data(child_rarm_tables, N3D_JOINT, joint_rarm_tables);
      
    self->tn3d[BODY_N3D_R_CLAV_ROTATE] = child_rarm_tables;
      
    self->tjoint[BODY_JOINT_R_CLAV_ROTATE] = (JOINT *) n3d_get_data(child_rarm_tables);
      
    root_rarm_tables = child_rarm_tables;
    child_rarm_tables = n3d_creation_child(root_rarm_tables, BODY_NAME_R_SCAP_ABDUCT);
    joint_rarm_tables = joint_creation();
    n3d_attach_data(child_rarm_tables, N3D_JOINT, joint_rarm_tables);
      
    read_body_data( SkelDataFile, BODY_NAME_R_SCAP_ABDUCT, xt, xr, mrdir_a);
    read_body_data( SkelDataFile, BODY_NAME_R_SCAP_ABDUCT, r_tr, r_rt, mrdir_a);
      
      
    mr3d_from_axis_angle(mrdir, r_rt);
    matrix_compose(mrdir, r_tr, matdir);
    n3d_set_ini_dir(child_rarm_tables, matdir);
      
    self->tn3d[BODY_N3D_R_SCAP_ABDUCT] = child_rarm_tables;
      
    self->tjoint[BODY_JOINT_R_SCAP_ABDUCT] = (JOINT *) n3d_get_data(child_rarm_tables);
      
    root_rarm_tables = child_rarm_tables;
    child_rarm_tables = n3d_creation_child(root_rarm_tables, BODY_NAME_R_SCAP_ROTATE);
    joint_rarm_tables = joint_creation();
    n3d_attach_data(child_rarm_tables, N3D_JOINT, joint_rarm_tables);
      
    self->tn3d[BODY_N3D_R_SCAP_ROTATE] = child_rarm_tables;
      
    self->tjoint[BODY_JOINT_R_SCAP_ROTATE] = (JOINT *) n3d_get_data(child_rarm_tables);
      
    root_rarm_tables = child_rarm_tables;
    child_rarm_tables = n3d_creation_child(root_rarm_tables, BODY_NAME_R_SHOULDER_FLEXION);
    joint_rarm_tables = joint_creation();
    n3d_attach_data(child_rarm_tables, N3D_JOINT, joint_rarm_tables);
      
    read_body_data( SkelDataFile, BODY_NAME_R_SHOULDER_FLEXION, xt, xr, mrdir_a);
    read_body_data( SkelDataFile, BODY_NAME_R_SHOULDER_FLEXION, r_tr, r_rt, mrdir_a);
      
      
    mr3d_from_axis_angle(mrdir, r_rt);
    matrix_compose(mrdir, r_tr, matdir);
    n3d_set_ini_dir(child_rarm_tables, matdir);
      
    self->tn3d[BODY_N3D_R_SHOULDER_FLEXION] = child_rarm_tables;
      
    self->tjoint[BODY_JOINT_R_SHOULDER_FLEXION] = (JOINT *) n3d_get_data(child_rarm_tables);
      
    root_rarm_tables = child_rarm_tables;
    child_rarm_tables = n3d_creation_child(root_rarm_tables, BODY_NAME_R_SHOULDER_ABDUCT);
    joint_rarm_tables = joint_creation();
    n3d_attach_data(child_rarm_tables, N3D_JOINT, joint_rarm_tables);
      
    self->tn3d[BODY_N3D_R_SHOULDER_ABDUCT] = child_rarm_tables;
      
    self->tjoint[BODY_JOINT_R_SHOULDER_ABDUCT] = (JOINT *) n3d_get_data(child_rarm_tables);
      
    root_rarm_tables = child_rarm_tables;
    child_rarm_tables = n3d_creation_child(root_rarm_tables, BODY_NAME_R_SHOULDER_TWISTING);
    joint_rarm_tables = joint_creation();
    n3d_attach_data(child_rarm_tables, N3D_JOINT, joint_rarm_tables);
      
    self->tn3d[BODY_N3D_R_SHOULDER_TWISTING] = child_rarm_tables;
      
    self->tjoint[BODY_JOINT_R_SHOULDER_TWISTING] = (JOINT *) n3d_get_data(child_rarm_tables);
      
    root_rarm_tables = child_rarm_tables;
    child_rarm_tables = n3d_creation_child(root_rarm_tables, BODY_NAME_R_ELBOW_FLEXION);
    joint_rarm_tables = joint_creation();
    n3d_attach_data(child_rarm_tables, N3D_JOINT, joint_rarm_tables);
      
    read_body_data( SkelDataFile, BODY_NAME_R_ELBOW_FLEXION, xt, xr, mrdir_a);
    read_body_data( SkelDataFile, BODY_NAME_R_ELBOW_FLEXION, r_tr, r_rt, mrdir_a);
      
      
    mr3d_from_axis_angle(mrdir, r_rt);
    matrix_compose(mrdir, r_tr, matdir);
    n3d_set_ini_dir(child_rarm_tables, matdir);
      
    self->tn3d[BODY_N3D_R_ELBOW_FLEXION] = child_rarm_tables;
      
    self->tjoint[BODY_JOINT_R_ELBOW_FLEXION] = (JOINT *) n3d_get_data(child_rarm_tables);
      
    root_rarm_tables = child_rarm_tables;
    child_rarm_tables = n3d_creation_child(root_rarm_tables, BODY_NAME_R_ELBOW_TWISTING);
    joint_rarm_tables = joint_creation();
    n3d_attach_data(child_rarm_tables, N3D_JOINT, joint_rarm_tables);
      
    self->tn3d[BODY_N3D_R_ELBOW_TWISTING] = child_rarm_tables;

    self->tjoint[BODY_JOINT_R_ELBOW_TWISTING] = (JOINT *) n3d_get_data(child_rarm_tables);

    root_rarm_tables = child_rarm_tables;
    child_rarm_tables = n3d_creation_child(root_rarm_tables, BODY_NAME_R_WRIST_FLEXION);
    joint_rarm_tables = joint_creation();
    n3d_attach_data(child_rarm_tables, N3D_JOINT, joint_rarm_tables);
      
    read_body_data( SkelDataFile, BODY_NAME_R_WRIST_FLEXION, xt, xr, mrdir_a);
    read_body_data( SkelDataFile, BODY_NAME_R_WRIST_FLEXION, r_tr, r_rt, mrdir_a);
      
      
    mr3d_from_axis_angle(mrdir, r_rt);
    matrix_compose(mrdir, r_tr, matdir);
    n3d_set_ini_dir(child_rarm_tables, matdir);
      
    self->tn3d[BODY_N3D_R_WRIST_FLEXION] = child_rarm_tables;
      
    self->tjoint[BODY_JOINT_R_WRIST_FLEXION] = (JOINT *) n3d_get_data(child_rarm_tables);
      
    root_rarm_tables = child_rarm_tables;
    child_rarm_tables = n3d_creation_child(root_rarm_tables, BODY_NAME_R_WRIST_PIVOT);
    joint_rarm_tables = joint_creation();
    n3d_attach_data(child_rarm_tables, N3D_JOINT, joint_rarm_tables);
      
    self->tn3d[BODY_N3D_R_WRIST_PIVOT] = child_rarm_tables;
      
    self->tjoint[BODY_JOINT_R_WRIST_PIVOT] = (JOINT *) n3d_get_data(child_rarm_tables);
      
    root_rarm_tables = child_rarm_tables;
    child_rarm_tables = n3d_creation_child(root_rarm_tables, BODY_NAME_R_HAND_CENTER);
      
    read_body_data( SkelDataFile, BODY_NAME_R_HAND_CENTER, xt, xr, mrdir_a);
    read_body_data( SkelDataFile, BODY_NAME_R_HAND_CENTER, r_tr, r_rt, mrdir_a);
      
      
    mr3d_from_axis_angle(mrdir, r_rt);
    matrix_compose(mrdir, r_tr, matdir);
    n3d_set_ini_dir(child_rarm_tables, matdir);
      
    self->tn3d[BODY_N3D_R_HAND_CENTER] = child_rarm_tables;

  }
}



static	void	body_iniT_fix_ini(BODY_t * self)
{
  N3D_t *   child;

  int i;
  N3D_t *   globl;
  MatriX      matdir, matdir1;
  MR3D        mrdir;
  VT3D        r_tr;
  VT3D        r_rt;

  static  MatriX  let_matrix_z_on_y = {
    { 0. , 0. , 1., 0. },
    { 1. , 0. , 0., 0. },
    { 0. , 1. , 0., 0. },
    { 0. , 0. , 0., 1. }
  };



  child = self->tn3d[BODY_N3D_TR_LATERAL];
  n3d_set_ini(child, Z_ON_X, NULL);

  child = self->tn3d[BODY_N3D_TR_FRONTAL];
  n3d_set_ini(child, Z_ON_X, NULL);

  child = self->tn3d[BODY_N3D_BODY_TURN];
  n3d_set_ini(child, Z_ON_X, NULL);

  child = self->tn3d[BODY_N3D_BODY_ROLL];
  n3d_set_ini(child, Z_ON_Y, NULL);

  child = self->tn3d[BODY_N3D_BODY_TILT];
  n3d_set_ini(child, Z_ON_Y, NULL);

  child = self->tn3d[BODY_N3D_PELVIC_TILT];
  n3d_set_ini(child, Z_ON_X, NULL);

  child = self->tn3d[BODY_N3D_PELVIC_TORSION];
  n3d_set_ini(child, Z_ON_Y, NULL);

  child = self->tn3d[BODY_N3D_PELVIC_ROLL];
  n3d_set_ini(child, Z_ON_Y, NULL);


  child = self->tn3d[BODY_N3D_R_HIP_ABDUCT];
  n3d_set_ini(child, Z_ON_Y, NULL);

  child = self->tn3d[BODY_N3D_R_HIP_TWISTING];
  n3d_set_ini(child, Z_ON_Y, NULL);

  child = self->tn3d[BODY_N3D_R_KNEE_TWISTING];
  n3d_set_ini(child, Z_ON_X, NULL);


  child = self->tn3d[BODY_N3D_L_HIP_ABDUCT];
  n3d_set_ini(child, Z_ON_Y, NULL);

  child = self->tn3d[BODY_N3D_L_HIP_TWISTING];
  n3d_set_ini(child, Z_ON_Y, NULL);

  child = self->tn3d[BODY_N3D_L_KNEE_TWISTING];
  n3d_set_ini(child, Z_ON_X, NULL);


  child = self->tn3d[BODY_N3D_VL1_TILT];
  n3d_set_ini(child, Z_ON_X, NULL);

  child = self->tn3d[BODY_N3D_VL2_TILT];
  n3d_set_ini(child, Z_ON_Y, NULL);

  child = self->tn3d[BODY_N3D_VL3_TILT];
  n3d_set_ini(child, Z_ON_Y, NULL);

  child = self->tn3d[BODY_N3D_VT4_TORSION];
  n3d_set_ini(child, Z_ON_X, NULL);

  child = self->tn3d[BODY_N3D_VT4_TILT];
  n3d_set_ini(child, Z_ON_X, NULL);

  child = self->tn3d[BODY_N3D_VT5_TORSION];
  n3d_set_ini(child, Z_ON_X, NULL);

  child = self->tn3d[BODY_N3D_VT6_TORSION];
  n3d_set_ini(child, Z_ON_X, NULL);

  child = self->tn3d[BODY_N3D_VT6_TILT];
  n3d_set_ini(child, Z_ON_X, NULL);

  child = self->tn3d[BODY_N3D_VC7_TORSION];
  n3d_set_ini(child, Z_ON_X, NULL);

  child = self->tn3d[BODY_N3D_VC7_TILT];
  n3d_set_ini(child, Z_ON_X, NULL);

  child = self->tn3d[BODY_N3D_VC8_TORSION];
  n3d_set_ini(child, Z_ON_X, NULL);

  child = self->tn3d[BODY_N3D_VC8_TILT];
  n3d_set_ini(child, Z_ON_X, NULL);


  child = self->tn3d[BODY_N3D_L_CLAV_ROTATE];
  n3d_set_ini(child, Z_ON_Y, NULL);

  child = self->tn3d[BODY_N3D_L_SCAP_ROTATE];
  n3d_set_ini(child, Z_ON_Y, NULL);

  child = self->tn3d[BODY_N3D_L_SHOULDER_ABDUCT];
  n3d_set_ini(child, Z_ON_Y, NULL);

  child = self->tn3d[BODY_N3D_L_SHOULDER_TWISTING];
  n3d_set_ini(child, Z_ON_Y, NULL);

  child = self->tn3d[BODY_N3D_L_ELBOW_TWISTING];
  n3d_set_ini(child, Z_ON_Y, NULL);

  child = self->tn3d[BODY_N3D_L_WRIST_PIVOT];
  n3d_set_ini(child, Z_ON_Y, NULL);


  child = self->tn3d[BODY_N3D_R_CLAV_ROTATE];
  n3d_set_ini(child, Z_ON_X, NULL);

  child = self->tn3d[BODY_N3D_R_SCAP_ROTATE];
  n3d_set_ini(child, Z_ON_X, NULL);

  child = self->tn3d[BODY_N3D_R_SHOULDER_ABDUCT];
  n3d_set_ini(child, Z_ON_Y, NULL);

  child = self->tn3d[BODY_N3D_R_SHOULDER_TWISTING];
  n3d_set_ini(child, Z_ON_Y, NULL);

  child = self->tn3d[BODY_N3D_R_ELBOW_TWISTING];
  n3d_set_ini(child, Z_ON_Y, NULL);

  child = self->tn3d[BODY_N3D_R_WRIST_PIVOT];
  n3d_set_ini(child, Z_ON_Y, NULL);

  if(self->lhand == 1)    body_init_fix_ini_lhand(self);

  if(self->rhand == 1)    body_init_fix_ini_rhand(self);


  h3d_update_ref_config(self->tn3d[BODY_N3D_GLOBAL_MOTION]);
}




/* ----------------------------------------------------------------- */



Booleen	    table_free(TABLE * self)
{
  if(self == NULL) return(TRUE);

  free( self );
  return(TRUE);
}


Booleen	    table_free_n3d(TABLE * self, int size)
{
  int	    ind;
    
  for (ind = 0; ind < size; ind++)
  {
    if(self[ind] != NULL) n3d_destroy(self[ind]);
  }

  free( self );
  return(TRUE);
}



void	body_init_fix_ini_lhand(BODY_t * self)
{
  N3D_t *   child;

  child = self->tlhand[BODY_N3D_L_PINKY_PIVOT];

  if(child != NULL) n3d_set_ini(child, Z_ON_Y, NULL);
  child = self->tlhand[BODY_N3D_L_PINKY_TWIST];

  if(child != NULL) n3d_set_ini(child, Z_ON_Y, NULL);
  child = self->tlhand[BODY_N3D_L_RING_PIVOT];

  if(child != NULL) n3d_set_ini(child, Z_ON_Y, NULL);
  child = self->tlhand[BODY_N3D_L_RING_TWIST];

  if(child != NULL) n3d_set_ini(child, Z_ON_Y, NULL);


  child = self->tlhand[BODY_N3D_L_MIDDLE_PIVOT];

  if(child != NULL) n3d_set_ini(child, Z_ON_Y, NULL);
  child = self->tlhand[BODY_N3D_L_MIDDLE_TWIST];

  if(child != NULL) n3d_set_ini(child, Z_ON_Y, NULL);
  child = self->tlhand[BODY_N3D_L_INDEX_PIVOT];

  if(child != NULL) n3d_set_ini(child, Z_ON_Y, NULL);
  child = self->tlhand[BODY_N3D_L_INDEX_TWIST];

  if(child != NULL) n3d_set_ini(child, Z_ON_Y, NULL);

#undef  NEWTHUMB
#define NEWTHUMB

  /* L_THUMB_ABDUCT  : DIFFERENT TO right transform, for symetric behavior */
  child = self->tlhand[BODY_N3D_L_THUMB_ABDUCT];

  if(child != NULL) n3d_set_ini(child, Z_ON_Y, NULL);

  child = self->tlhand[BODY_N3D_L_THUMB_TW];

#ifdef NEWTHUMB
  if(child != NULL) n3d_set_ini(child, Z_ON_Y, NULL);
#else
  if(child != NULL) n3d_set_ini(child, Z_ON_MINUS_Y, NULL);
#endif
}


void	body_init_fix_ini_rhand(BODY_t * self)
{
  N3D_t *   child;

  child = self->trhand[BODY_N3D_R_PINKY_PIVOT];

  if(child != NULL) n3d_set_ini(child, Z_ON_Y, NULL);
  child = self->trhand[BODY_N3D_R_PINKY_TWIST];

  if(child != NULL) n3d_set_ini(child, Z_ON_Y, NULL);
  child = self->trhand[BODY_N3D_R_RING_PIVOT];

  if(child != NULL) n3d_set_ini(child, Z_ON_Y, NULL);
  child = self->trhand[BODY_N3D_R_RING_TWIST];

  if(child != NULL) n3d_set_ini(child, Z_ON_Y, NULL);


  child = self->trhand[BODY_N3D_R_MIDDLE_PIVOT];

  if(child != NULL) n3d_set_ini(child, Z_ON_Y, NULL);
  child = self->trhand[BODY_N3D_R_MIDDLE_TWIST];

  if(child != NULL) n3d_set_ini(child, Z_ON_Y, NULL);
  child = self->trhand[BODY_N3D_R_INDEX_PIVOT];

  if(child != NULL) n3d_set_ini(child, Z_ON_Y, NULL);
  child = self->trhand[BODY_N3D_R_INDEX_TWIST];

  if(child != NULL) n3d_set_ini(child, Z_ON_Y, NULL);

  /* R_THUMB_ABDUCT : DIFFERENT TO  left transform, for symetric behavior */
  child = self->trhand[BODY_N3D_R_THUMB_ABDUCT];

#ifdef NEWTHUMB
  if(child != NULL) n3d_set_ini(child, Z_ON_MINUS_X, NULL);
#else
  if(child != NULL) n3d_set_ini(child, Z_ON_MINUS_Y, NULL);
#endif

  child = self->trhand[BODY_N3D_R_THUMB_TW];

#ifdef NEWTHUMB
  if(child != NULL) n3d_set_ini(child, Z_ON_Y, NULL);
#else
  if(child != NULL) n3d_set_ini(child, Z_ON_X, NULL);
#endif
}





Booleen	body_skeleton_hand_fit(BODY_t * self)
{
  MM_t	val1, val2;
  VT3D_t	r_tr, r_rt, l_tr, l_rt;
    
  if(!self)
  {
    fprintf(stderr, "body_skeleton_hand_fit(): BODY_t UNDEFINED\n");
    return FALSE;
  }

  /*    skeleton = self->skeleton;
	if(skeleton == NULL) return(NO);*/

  if (self->rhand || self->lhand)
  {
	
    h3d_update_ref_config(self->tn3d[BODY_N3D_L_WRIST_PIVOT]);
    h3d_update_ref_config(self->tn3d[BODY_N3D_R_WRIST_PIVOT]);

    return(TRUE);
  }   
  return(TRUE);
}


static	void	body_set_tjoint_lhand_ind(BODY_t * self, int ind, 
					  N3D_t * nod)
{
  if(self && self->tjoint_lhand && ind < self->nb_joint_lhand)
  {
    if( nod != NULL)
    {
      self->tjoint_lhand[ind] = (JOINT_t *) n3d_get_data(nod);
    }
    else
    {
      self->tjoint_lhand[ind] = NULL;
    }
	
    return;
  }
  fprintf(stderr, " body_set_tjoint_lhand_ind(): error\n");
}


static	void	body_set_tjoint_rhand_ind(BODY_t * self, int ind, 
					  N3D_t * nod)
{
  if(self && self->tjoint_rhand && ind < self->nb_joint_rhand)
  {
    if( nod != NULL)
    {
      self->tjoint_rhand[ind] = (JOINT_t *) n3d_get_data(nod);
    }
    else
    {
      self->tjoint_lhand[ind] = NULL;
    }
    return;
  }
  fprintf(stderr, " body_set_tjoint_rhand_ind(): error\n");
}



static	void	body_set_trhand_ind(BODY_t * self, int ind, N3D_t * nod)
{
  if(self && self->trhand && ind < self->nb_rhand)
  {
    self->trhand[ind] = nod;
    return;
  }
  fprintf(stderr, " body_set_trhand_ind(): error\n");
}


static	void	body_set_tlhand_ind(BODY_t * self, int ind, N3D_t * nod)
{
  if(self && self->tlhand && ind < self->nb_lhand)
  {
    self->tlhand[ind] = nod;
    return;
  }
  fprintf(stderr, " body_set_tlhand_ind(): error\n");
}



static	void	body_ini_rhand_tables(BODY_t * self, N3D_t * root)
{
  N3D_t	   * child;
  JOINT_t * joint;
  N3D_t * root1;

  float val, xt[3], xr[3], xt_x[3], xr_x[3];

  VT3D_t r_tr_x,  l_tr_x;
  VT3D_t r_rt_x,  l_rt_x;
  MM_t        val1, val2;
  VT3D_t      r_tr, r_rt, l_tr, l_rt;

  MatriX      matdir,xpto, res;
  MR3D        mrdir;

  MR3D	mrdir_ab;

    MatriX XX;
    MR3D mm1;
    VT3D_t vv1;

  MatriX xpto_ = {
    { 0. , 1. , 0., 0. },
    { 0. , 0. , 1., 0. },
    { 1. , 0. , 0., 0. },
    { 0. , 0. , 0., 1. }
  };

    
  child = n3d_creation_child(root, BODY_NAME_R_HAND_ROOT);
  body_set_trhand_ind(self, BODY_N3D_R_HAND_ROOT, child);

  root = child;


  child = n3d_creation_child(root, BODY_NAME_R_PINKY_FL1);
  joint = joint_creation();
  n3d_attach_data(child, N3D_JOINT, joint);

  read_hands_data( HandDataFile, BODY_NAME_R_PINKY_FL1,&val, xt, xr, xt_x, xr_x);
  read_hands_data( HandDataFile, BODY_NAME_R_PINKY_FL1,&val1, r_tr, r_rt, r_tr_x, r_rt_x);

      
  mr3d_from_axis_angle(mrdir, r_rt);
  matrix_compose(mrdir , r_tr , matdir);

  matrix_decompose( mm1, vv1, xpto_ );
  xt_x[0] = 40.; xt_x[1] = 0.; xt_x[2] = -15.;
  matrix_compose( mm1, xt_x, XX );
  matrix_mult( matdir, XX, res );

  matrix_copy( res, matdir );
      
  n3d_set_ini_dir(child, matdir);

  body_set_trhand_ind(self, BODY_N3D_R_PINKY_FL1, child);
  body_set_tjoint_rhand_ind(self, BODY_JOINT_R_PINKY_FL1, child);

  root1 = child;
  child = n3d_creation_child(root1, BODY_NAME_R_PINKY_PIVOT);
  joint = joint_creation();
  n3d_attach_data(child, N3D_JOINT, joint);
  body_set_trhand_ind(self, BODY_N3D_R_PINKY_PIVOT, child);
  body_set_tjoint_rhand_ind(self, BODY_JOINT_R_PINKY_PIVOT, child);

  root1 = child;
  child = n3d_creation_child(root1, BODY_NAME_R_PINKY_TWIST);
  joint = joint_creation();
  n3d_attach_data(child, N3D_JOINT, joint);
  body_set_trhand_ind(self, BODY_N3D_R_PINKY_TWIST, child);
  body_set_tjoint_rhand_ind(self, BODY_JOINT_R_PINKY_TWIST, child);

  root1 = child;
  child = n3d_creation_child(root1, BODY_NAME_R_PINKY_FL2);
  joint = joint_creation();
  n3d_attach_data(child, N3D_JOINT, joint);

  read_hands_data( HandDataFile, BODY_NAME_R_PINKY_FL2, &val, xt, xr, xt_x, xr_x);
  read_hands_data( HandDataFile, BODY_NAME_R_PINKY_FL2, &val1, xt, xr, xt_x, xr_x);

  vt3d_set_null(r_tr);
  r_tr[2] = val1;
  matrix_combine(NULL, Z_ON_Y, r_tr , matdir);
  n3d_set_ini_dir(child, matdir);

  body_set_trhand_ind(self, BODY_N3D_R_PINKY_FL2, child);
  body_set_tjoint_rhand_ind(self, BODY_JOINT_R_PINKY_FL2, child);

  root1 = child;
  child = n3d_creation_child(root1, BODY_NAME_R_PINKY_FL3);
  joint = joint_creation();
  n3d_attach_data(child, N3D_JOINT, joint);


  read_hands_data( HandDataFile, BODY_NAME_R_PINKY_FL3, &val, xt, xr, xt_x, xr_x);
  read_hands_data( HandDataFile, BODY_NAME_R_PINKY_FL3, &val1, xt, xr, xt_x, xr_x);

  vt3d_set_null(r_tr);
  r_tr[0] = val1;
  matrix_combine(NULL, IDENTITY, r_tr , matdir);
  n3d_set_ini_dir(child, matdir);

  body_set_trhand_ind(self, BODY_N3D_R_PINKY_FL3, child);
  body_set_tjoint_rhand_ind(self, BODY_JOINT_R_PINKY_FL3, child);

  root1 = child;
  child = n3d_creation_child(root1, BODY_NAME_R_PINKY_EXTR);


  read_hands_data( HandDataFile, BODY_NAME_R_PINKY_EXTR, &val, xt, xr, xt_x, xr_x);
  read_hands_data( HandDataFile, BODY_NAME_R_PINKY_EXTR, &val1, xt, xr, xt_x, xr_x);

  vt3d_set_null(r_tr);
  r_tr[0] = val1;
  matrix_combine(NULL, IDENTITY, r_tr , matdir);
  n3d_set_ini_dir(child, matdir);

  body_set_trhand_ind(self, BODY_N3D_R_PINKY_EXTR, child);



  child = n3d_creation_child(root, BODY_NAME_R_RING_FL1);
  joint = joint_creation();
  n3d_attach_data(child, N3D_JOINT, joint);


  read_hands_data( HandDataFile, BODY_NAME_R_RING_FL1, &val, xt, xr, xt_x, xr_x);
  read_hands_data( HandDataFile, BODY_NAME_R_RING_FL1, &val, r_tr, r_rt, xt_x, r_rt_x);
      
  mr3d_from_axis_angle(mrdir, r_rt);
  matrix_compose(mrdir , r_tr , matdir);

  matrix_decompose( mm1, vv1, xpto_ );
  xt_x[0] = 20.; xt_x[1] = 0.; xt_x[2] = -15.;
  matrix_compose( mm1, xt_x, XX );
  matrix_mult( matdir, XX, res ); /* NEEW */


  matrix_copy( res, matdir );
      
  n3d_set_ini_dir(child, matdir);

  body_set_trhand_ind(self, BODY_N3D_R_RING_FL1, child);
  body_set_tjoint_rhand_ind(self, BODY_JOINT_R_RING_FL1, child);

  root1 = child;
  child = n3d_creation_child(root1, BODY_NAME_R_RING_PIVOT);
  joint = joint_creation();
  n3d_attach_data(child, N3D_JOINT, joint);
  body_set_trhand_ind(self, BODY_N3D_R_RING_PIVOT, child);
  body_set_tjoint_rhand_ind(self, BODY_JOINT_R_RING_PIVOT, child);

  root1 = child;
  child = n3d_creation_child(root1, BODY_NAME_R_RING_TWIST);
  joint = joint_creation();
  n3d_attach_data(child, N3D_JOINT, joint);
  body_set_trhand_ind(self, BODY_N3D_R_RING_TWIST, child);
  body_set_tjoint_rhand_ind(self, BODY_JOINT_R_RING_TWIST, child);

  root1 = child;
  child = n3d_creation_child(root1, BODY_NAME_R_RING_FL2);
  joint = joint_creation();
  n3d_attach_data(child, N3D_JOINT, joint);

  read_hands_data( HandDataFile, BODY_NAME_R_RING_FL2, &val, xt, xr, xt_x, xr_x);
  read_hands_data( HandDataFile, BODY_NAME_R_RING_FL2, &val1, xt, xr, xt_x, xr_x);
      
  vt3d_set_null(r_tr);
  r_tr[2] = val1;
  matrix_combine(NULL, Z_ON_Y, r_tr , matdir);
  n3d_set_ini_dir(child, matdir);

  body_set_trhand_ind(self, BODY_N3D_R_RING_FL2, child);
  body_set_tjoint_rhand_ind(self, BODY_JOINT_R_RING_FL2, child);

  root1 = child;
  child = n3d_creation_child(root1, BODY_NAME_R_RING_FL3);
  joint = joint_creation();
  n3d_attach_data(child, N3D_JOINT, joint);

  read_hands_data( HandDataFile, BODY_NAME_R_RING_FL3, &val, xt, xr, xt_x, xr_x);
  read_hands_data( HandDataFile, BODY_NAME_R_RING_FL3, &val1, xt, xr, xt_x, xr_x);

  vt3d_set_null(r_tr);
  r_tr[0] = val1;
  matrix_combine(NULL, IDENTITY, r_tr , matdir);
  n3d_set_ini_dir(child, matdir);

  body_set_trhand_ind(self, BODY_N3D_R_RING_FL3, child);
  body_set_tjoint_rhand_ind(self, BODY_JOINT_R_RING_FL3, child);

  root1 = child;
  child = n3d_creation_child(root1, BODY_NAME_R_RING_EXTR);


  read_hands_data( HandDataFile, BODY_NAME_R_RING_EXTR, &val, xt, xr, xt_x, xr_x);
  read_hands_data( HandDataFile, BODY_NAME_R_RING_EXTR, &val1, xt, xr, xt_x, xr_x);
      
  vt3d_set_null(r_tr);
  r_tr[0] = val1;
  matrix_combine(NULL, IDENTITY, r_tr , matdir);
  n3d_set_ini_dir(child, matdir);

  body_set_trhand_ind(self, BODY_N3D_R_RING_EXTR, child);


  child = n3d_creation_child(root, BODY_NAME_R_MIDDLE_FL1);
  joint = joint_creation();
  n3d_attach_data(child, N3D_JOINT, joint);

  read_hands_data( HandDataFile, BODY_NAME_R_MIDDLE_FL1, &val, xt, xr, xt_x, xr_x);
  read_hands_data( HandDataFile, BODY_NAME_R_MIDDLE_FL1, &val, r_tr, r_rt, xt_x, xr_x);
  

  mr3d_from_axis_angle(mrdir, r_rt);
  matrix_compose(mrdir , r_tr , matdir);
  

  matrix_mult( matdir, xpto_, res );
  matrix_copy( res, matdir );
  
  n3d_set_ini_dir(child, matdir);

  body_set_trhand_ind(self, BODY_N3D_R_MIDDLE_FL1, child);
  body_set_tjoint_rhand_ind(self, BODY_JOINT_R_MIDDLE_FL1, child);

  root1 = child;
  child = n3d_creation_child(root1, BODY_NAME_R_MIDDLE_PIVOT);
  joint = joint_creation();
  n3d_attach_data(child, N3D_JOINT, joint);
  body_set_trhand_ind(self, BODY_N3D_R_MIDDLE_PIVOT, child);
  body_set_tjoint_rhand_ind(self, BODY_JOINT_R_MIDDLE_PIVOT, child);

  root1 = child;
  child = n3d_creation_child(root1, BODY_NAME_R_MIDDLE_TWIST);
  joint = joint_creation();
  n3d_attach_data(child, N3D_JOINT, joint);
  body_set_trhand_ind(self, BODY_N3D_R_MIDDLE_TWIST, child);
  body_set_tjoint_rhand_ind(self, BODY_JOINT_R_MIDDLE_TWIST, child);

  root1 = child;
  child = n3d_creation_child(root1, BODY_NAME_R_MIDDLE_FL2);
  joint = joint_creation();
  n3d_attach_data(child, N3D_JOINT, joint);

  read_hands_data( HandDataFile, BODY_NAME_R_MIDDLE_FL2, &val, xt, xr, xt_x, xr_x);
  read_hands_data( HandDataFile, BODY_NAME_R_MIDDLE_FL2, &val1, xt, xr, xt_x, xr_x);

  vt3d_set_null(r_tr);
  r_tr[2] = val1;
  matrix_combine(NULL, Z_ON_Y, r_tr , matdir);
  n3d_set_ini_dir(child, matdir);

  body_set_trhand_ind(self, BODY_N3D_R_MIDDLE_FL2, child);
  body_set_tjoint_rhand_ind(self, BODY_JOINT_R_MIDDLE_FL2, child);

  root1 = child;
  child = n3d_creation_child(root1, BODY_NAME_R_MIDDLE_FL3);
  joint = joint_creation();
  n3d_attach_data(child, N3D_JOINT, joint);

  read_hands_data( HandDataFile, BODY_NAME_R_MIDDLE_FL3, &val, xt, xr, xt_x, xr_x);
  read_hands_data( HandDataFile, BODY_NAME_R_MIDDLE_FL3, &val1, xt, xr, xt_x, xr_x);
      
  vt3d_set_null(r_tr);
  r_tr[0] = val1;
  matrix_combine(NULL, IDENTITY, r_tr , matdir);
  n3d_set_ini_dir(child, matdir);

  body_set_trhand_ind(self, BODY_N3D_R_MIDDLE_FL3, child);
  body_set_tjoint_rhand_ind(self, BODY_JOINT_R_MIDDLE_FL3, child);

  root1 = child;
  child = n3d_creation_child(root1, BODY_NAME_R_MIDDLE_EXTR);

  read_hands_data( HandDataFile, BODY_NAME_R_MIDDLE_EXTR, &val, xt, xr, xt_x, xr_x);
  read_hands_data( HandDataFile, BODY_NAME_R_MIDDLE_EXTR, &val1, xt, xr, xt_x, xr_x);

  vt3d_set_null(r_tr);
  r_tr[0] = val1;
  matrix_combine(NULL, IDENTITY, r_tr , matdir);
  n3d_set_ini_dir(child, matdir);

  body_set_trhand_ind(self, BODY_N3D_R_MIDDLE_EXTR, child);



  child = n3d_creation_child(root, BODY_NAME_R_INDEX_FL1);
  joint = joint_creation();
  n3d_attach_data(child, N3D_JOINT, joint);

  read_hands_data( HandDataFile, BODY_NAME_R_INDEX_FL1, &val, xt, xr, xt_x, xr_x);
  read_hands_data( HandDataFile, BODY_NAME_R_INDEX_FL1, &val, r_tr, r_rt, r_tr_x, xr_x);
  
  mr3d_from_axis_angle(mrdir, r_rt);
  matrix_compose(mrdir , r_tr , matdir);
  
    
  matrix_combine(NULL, Z_ON_X, r_tr_x, xpto);
  matrix_mult( matdir, xpto, res );
  matrix_copy( res, matdir );
  
  n3d_set_ini_dir(child, matdir);

  body_set_trhand_ind(self, BODY_N3D_R_INDEX_FL1, child);
  body_set_tjoint_rhand_ind(self, BODY_JOINT_R_INDEX_FL1, child);

  root1 = child;
  child = n3d_creation_child(root1, BODY_NAME_R_INDEX_PIVOT);
  joint = joint_creation();
  n3d_attach_data(child, N3D_JOINT, joint);
  body_set_trhand_ind(self, BODY_N3D_R_INDEX_PIVOT, child);
  body_set_tjoint_rhand_ind(self, BODY_JOINT_R_INDEX_PIVOT, child);

  root1 = child;
  child = n3d_creation_child(root1, BODY_NAME_R_INDEX_TWIST);
  joint = joint_creation();
  n3d_attach_data(child, N3D_JOINT, joint);
  body_set_trhand_ind(self, BODY_N3D_R_INDEX_TWIST, child);
  body_set_tjoint_rhand_ind(self, BODY_JOINT_R_INDEX_TWIST, child);

  root1 = child;
  child = n3d_creation_child(root1, BODY_NAME_R_INDEX_FL2);
  joint = joint_creation();
  n3d_attach_data(child, N3D_JOINT, joint);

  read_hands_data( HandDataFile, BODY_NAME_R_INDEX_FL2, &val, xt, xr, xt_x, xr_x);
  read_hands_data( HandDataFile, BODY_NAME_R_INDEX_FL2, &val1, xt, xr, xt_x, xr_x);
      
  vt3d_set_null(r_tr);
  r_tr[2] = val1;
  matrix_combine(NULL, Z_ON_Y, r_tr , matdir);
  n3d_set_ini_dir(child, matdir);

  body_set_trhand_ind(self, BODY_N3D_R_INDEX_FL2, child);
  body_set_tjoint_rhand_ind(self, BODY_JOINT_R_INDEX_FL2, child);

  root1 = child;
  child = n3d_creation_child(root1, BODY_NAME_R_INDEX_FL3);
  joint = joint_creation();
  n3d_attach_data(child, N3D_JOINT, joint);

  read_hands_data( HandDataFile, BODY_NAME_R_INDEX_FL3, &val, xt, xr, xt_x, xr_x);
  read_hands_data( HandDataFile, BODY_NAME_R_INDEX_FL3, &val1, xt, xr, xt_x, xr_x);

  vt3d_set_null(r_tr);
  r_tr[0] = val1;
  matrix_combine(NULL, IDENTITY, r_tr , matdir);
  n3d_set_ini_dir(child, matdir);

  body_set_trhand_ind(self, BODY_N3D_R_INDEX_FL3, child);
  body_set_tjoint_rhand_ind(self, BODY_JOINT_R_INDEX_FL3, child);

  root1 = child;
  child = n3d_creation_child(root1, BODY_NAME_R_INDEX_EXTR);

  read_hands_data( HandDataFile, BODY_NAME_R_INDEX_EXTR, &val, xt, xr, xt_x, xr_x);
  read_hands_data( HandDataFile, BODY_NAME_R_INDEX_EXTR, &val1, xt, xr, xt_x, xr_x);
      
  vt3d_set_null(r_tr);
  r_tr[0] = val1;
  matrix_combine(NULL, IDENTITY, r_tr , matdir);
  n3d_set_ini_dir(child, matdir);

  body_set_trhand_ind(self, BODY_N3D_R_INDEX_EXTR, child);



  child = n3d_creation_child(root, BODY_NAME_R_THUMB_FL1);
  joint = joint_creation();
  n3d_attach_data(child, N3D_JOINT, joint);

  read_hands_data( HandDataFile, BODY_NAME_R_THUMB_FL1, &val, xt, xr, xt_x, xr_x);
  read_hands_data( HandDataFile, BODY_NAME_R_THUMB_FL1, &val, xt, r_rt, r_tr_x, r_rt_x);
  
  vt3d_set_null(r_tr);
  
  mr3d_from_axis_angle(mrdir, r_rt);
  matrix_compose(mrdir , r_tr , matdir);
#ifdef NEWTHUMB
  {
    MatriX_t zonx;
    matrix_combine(NULL, Z_ON_X, r_tr, zonx);
    matrix_mult(zonx, matdir, matdir);
  }
#endif

  mr3d_from_axis_angle(mrdir_ab, r_rt_x);
  matrix_compose(mrdir_ab , r_tr_x , xpto);
  matrix_mult( matdir, xpto, res );
  matrix_copy( res, matdir );
      
  n3d_set_ini_dir(child, matdir);

  body_set_trhand_ind(self, BODY_N3D_R_THUMB_FL1, child);
  body_set_tjoint_rhand_ind(self, BODY_JOINT_R_THUMB_FL1, child);

  root1 = child;
  child = n3d_creation_child(root1, BODY_NAME_R_THUMB_ABDUCT);
  joint = joint_creation();
  n3d_attach_data(child, N3D_JOINT, joint);
  body_set_trhand_ind(self, BODY_N3D_R_THUMB_ABDUCT, child);
  body_set_tjoint_rhand_ind(self, BODY_JOINT_R_THUMB_ABDUCT, child);

  root1 = child;
  child = n3d_creation_child(root1, BODY_NAME_R_THUMB_TW);
  joint = joint_creation();
  n3d_attach_data(child, N3D_JOINT, joint);
  body_set_trhand_ind(self, BODY_N3D_R_THUMB_TW, child);
  body_set_tjoint_rhand_ind(self, BODY_JOINT_R_THUMB_TW, child);

  root1 = child;
  child = n3d_creation_child(root1, BODY_NAME_R_THUMB_FL2);
  joint = joint_creation();
  n3d_attach_data(child, N3D_JOINT, joint);

  read_hands_data( HandDataFile, BODY_NAME_R_THUMB_FL2, &val, xt, xr, xt_x, xr_x);
  read_hands_data( HandDataFile, BODY_NAME_R_THUMB_FL2, &val1, xt, xr, xt_x, xr_x);
      
  vt3d_set_null(r_tr);

#ifdef NEWTHUMB
  r_tr[2] = val1;
  matrix_combine(NULL, Z_ON_Y, r_tr , matdir);
#else
  r_tr[0] = val1;
  matrix_combine(NULL, IDENTITY, r_tr , matdir);
#endif

  n3d_set_ini_dir(child, matdir);

  body_set_trhand_ind(self, BODY_N3D_R_THUMB_FL2, child);
  body_set_tjoint_rhand_ind(self, BODY_JOINT_R_THUMB_FL2, child);

  root1 = child;
  child = n3d_creation_child(root1, BODY_NAME_R_THUMB_FL3);
  joint = joint_creation();
  n3d_attach_data(child, N3D_JOINT, joint);


  read_hands_data( HandDataFile, BODY_NAME_R_THUMB_FL3, &val, xt, xr, xt_x, xr_x);
  read_hands_data( HandDataFile, BODY_NAME_R_THUMB_FL3, &val1, xt, xr, xt_x, xr_x);
      
  vt3d_set_null(r_tr);
  r_tr[0] = val1;
  matrix_combine(NULL, IDENTITY, r_tr , matdir);
  n3d_set_ini_dir(child, matdir);

  body_set_trhand_ind(self, BODY_N3D_R_THUMB_FL3, child);
  body_set_tjoint_rhand_ind(self, BODY_JOINT_R_THUMB_FL3, child);

  root1 = child;
  child = n3d_creation_child(root1, BODY_NAME_R_THUMB_EXTR);


  read_hands_data( HandDataFile, BODY_NAME_R_THUMB_EXTR, &val, xt, xr, xt_x, xr_x);
  read_hands_data( HandDataFile, BODY_NAME_R_THUMB_EXTR, &val1, xt, xr, xt_x, xr_x);
      
  vt3d_set_null(r_tr);
  r_tr[0] = val1;
  matrix_combine(NULL, IDENTITY, r_tr , matdir);
  n3d_set_ini_dir(child, matdir);

  body_set_trhand_ind(self, BODY_N3D_R_THUMB_EXTR, child);

}


static	void	body_ini_lhand_tables(BODY_t * self, N3D_t * root)
{
  N3D_t	   * child;
  JOINT_t * joint;
  N3D_t * root1;

  float val, xt[3], xr[3], xt_x[3], xr_x[3];

  VT3D_t r_tr_x,  l_tr_x;
  VT3D_t r_rt_x,  l_rt_x;
  MM_t        val1, val2;
  VT3D_t      r_tr, r_rt, l_tr, l_rt;

  MatriX      matdir, xpto, res;
  MR3D        mrdir;

    MatriX XX;
    MR3D mm1;
    VT3D_t vv1;

  MR3D        mrdir_ab;

  MatriX xpto_ = {
    { 0. , 1. , 0., 0. },
    { 0. , 0. , 1., 0. },
    { 1. , 0. , 0., 0. },
    { 0. , 0. , 0., 1. }
  };
  
  child = n3d_creation_child(root, BODY_NAME_L_HAND_ROOT);
  body_set_tlhand_ind(self, BODY_N3D_L_HAND_ROOT, child);

  root = child;

  child = n3d_creation_child(root, BODY_NAME_L_PINKY_FL1);
  joint = joint_creation();
  n3d_attach_data(child, N3D_JOINT, joint);



  read_hands_data( HandDataFile, BODY_NAME_L_PINKY_FL1, &val, xt, xr, xt_x, xr_x);
  read_hands_data( HandDataFile, BODY_NAME_L_PINKY_FL1, &val, l_tr, l_rt, l_tr_x, l_rt_x);
      
  mr3d_from_axis_angle(mrdir, l_rt);
  matrix_compose(mrdir , l_tr , matdir);

  matrix_decompose( mm1, vv1, xpto_ );
  xt_x[0] = -40.; xt_x[1] = 0.; xt_x[2] = -15.;
  matrix_compose( mm1, xt_x, XX );
  matrix_mult( matdir, XX, res );

  matrix_copy( res, matdir );
      
  n3d_set_ini_dir(child, matdir);

  body_set_tlhand_ind(self, BODY_N3D_L_PINKY_FL1, child);
  body_set_tjoint_lhand_ind(self, BODY_JOINT_L_PINKY_FL1, child);

  root1 = child;
  child = n3d_creation_child(root1, BODY_NAME_L_PINKY_PIVOT);
  joint = joint_creation();
  n3d_attach_data(child, N3D_JOINT, joint);
  body_set_tlhand_ind(self, BODY_N3D_L_PINKY_PIVOT, child);
  body_set_tjoint_lhand_ind(self, BODY_JOINT_L_PINKY_PIVOT, child);

  root1 = child;
  child = n3d_creation_child(root1, BODY_NAME_L_PINKY_TWIST);
  joint = joint_creation();
  n3d_attach_data(child, N3D_JOINT, joint);
  body_set_tlhand_ind(self, BODY_N3D_L_PINKY_TWIST, child);
  body_set_tjoint_lhand_ind(self, BODY_JOINT_L_PINKY_TWIST, child);

  root1 = child;
  child = n3d_creation_child(root1, BODY_NAME_L_PINKY_FL2);
  joint = joint_creation();
  n3d_attach_data(child, N3D_JOINT, joint);


  read_hands_data( HandDataFile, BODY_NAME_L_PINKY_FL2, &val, xt, xr, xt_x, xr_x);
  read_hands_data( HandDataFile, BODY_NAME_L_PINKY_FL2, &val2, xt, xr, xt_x, xr_x);
      
  vt3d_set_null(r_tr);
  r_tr[2] = val2;
  matrix_combine(NULL, Z_ON_Y, r_tr , matdir);
  n3d_set_ini_dir(child, matdir);

  body_set_tlhand_ind(self, BODY_N3D_L_PINKY_FL2, child);
  body_set_tjoint_lhand_ind(self, BODY_JOINT_L_PINKY_FL2, child);

  root1 = child;
  child = n3d_creation_child(root1, BODY_NAME_L_PINKY_FL3);
  joint = joint_creation();
  n3d_attach_data(child, N3D_JOINT, joint);


  read_hands_data( HandDataFile, BODY_NAME_L_PINKY_FL3, &val, xt, xr, xt_x, xr_x);
  read_hands_data( HandDataFile, BODY_NAME_L_PINKY_FL3, &val2, xt, xr, xt_x, xr_x);

  vt3d_set_null(r_tr);
  r_tr[0] = val2;
  matrix_combine(NULL, IDENTITY, r_tr , matdir);
  n3d_set_ini_dir(child, matdir);

  body_set_tlhand_ind(self, BODY_N3D_L_PINKY_FL3, child);
  body_set_tjoint_lhand_ind(self, BODY_JOINT_L_PINKY_FL3, child);

  root1 = child;
  child = n3d_creation_child(root1, BODY_NAME_L_PINKY_EXTR);


  read_hands_data( HandDataFile, BODY_NAME_L_PINKY_EXTR, &val, xt, xr, xt_x, xr_x);
  read_hands_data( HandDataFile, BODY_NAME_L_PINKY_EXTR, &val2, xt, xr, xt_x, xr_x);
      
  vt3d_set_null(r_tr);
  r_tr[0] = val2;
  matrix_combine(NULL, IDENTITY, r_tr , matdir);
  n3d_set_ini_dir(child, matdir);

  body_set_tlhand_ind(self, BODY_N3D_L_PINKY_EXTR, child);


  child = n3d_creation_child(root, BODY_NAME_L_RING_FL1);
  joint = joint_creation();
  n3d_attach_data(child, N3D_JOINT, joint);


  read_hands_data( HandDataFile, BODY_NAME_L_RING_FL1, &val, xt, xr, xt_x, xr_x);
  read_hands_data( HandDataFile, BODY_NAME_L_RING_FL1, &val, l_tr, l_rt, xt_x, l_rt_x);

  mr3d_from_axis_angle(mrdir, l_rt);
  matrix_compose(mrdir , l_tr , matdir);

  matrix_decompose( mm1, vv1, xpto_ );
  xt_x[0] = -20.; xt_x[1] = 0.; xt_x[2] = -15.;
  matrix_compose( mm1, xt_x, XX );
  matrix_mult( matdir, XX, res ); /* NEEW */

  matrix_copy( res, matdir );
      
  n3d_set_ini_dir(child, matdir);

  body_set_tlhand_ind(self, BODY_N3D_L_RING_FL1, child);
  body_set_tjoint_lhand_ind(self, BODY_JOINT_L_RING_FL1, child);

  root1 = child;
  child = n3d_creation_child(root1, BODY_NAME_L_RING_PIVOT);
  joint = joint_creation();
  n3d_attach_data(child, N3D_JOINT, joint);
  body_set_tlhand_ind(self, BODY_N3D_L_RING_PIVOT, child);
  body_set_tjoint_lhand_ind(self, BODY_JOINT_L_RING_PIVOT, child);

  root1 = child;
  child = n3d_creation_child(root1, BODY_NAME_L_RING_TWIST);
  joint = joint_creation();
  n3d_attach_data(child, N3D_JOINT, joint);
  body_set_tlhand_ind(self, BODY_N3D_L_RING_TWIST, child);
  body_set_tjoint_lhand_ind(self, BODY_JOINT_L_RING_TWIST, child);

  root1 = child;
  child = n3d_creation_child(root1, BODY_NAME_L_RING_FL2);
  joint = joint_creation();
  n3d_attach_data(child, N3D_JOINT, joint);


  read_hands_data( HandDataFile, BODY_NAME_L_RING_FL2, &val, xt, xr, xt_x, xr_x);
  read_hands_data( HandDataFile, BODY_NAME_L_RING_FL2, &val2, xt, xr, xt_x, xr_x);
      
  vt3d_set_null(r_tr);
  r_tr[2] = val2;
  matrix_combine(NULL, Z_ON_Y, r_tr , matdir);
  n3d_set_ini_dir(child, matdir);

  body_set_tlhand_ind(self, BODY_N3D_L_RING_FL2, child);
  body_set_tjoint_lhand_ind(self, BODY_JOINT_L_RING_FL2, child);

  root1 = child;
  child = n3d_creation_child(root1, BODY_NAME_L_RING_FL3);
  joint = joint_creation();
  n3d_attach_data(child, N3D_JOINT, joint);


  read_hands_data( HandDataFile, BODY_NAME_L_RING_FL3, &val, xt, xr, xt_x, xr_x);
  read_hands_data( HandDataFile, BODY_NAME_L_RING_FL3, &val2, xt, xr, xt_x, xr_x);
      
  vt3d_set_null(r_tr);
  r_tr[0] = val2;
  matrix_combine(NULL, IDENTITY, r_tr , matdir);
  n3d_set_ini_dir(child, matdir);

  body_set_tlhand_ind(self, BODY_N3D_L_RING_FL3, child);
  body_set_tjoint_lhand_ind(self, BODY_JOINT_L_RING_FL3, child);

  root1 = child;
  child = n3d_creation_child(root1, BODY_NAME_L_RING_EXTR);


  read_hands_data( HandDataFile, BODY_NAME_L_RING_EXTR, &val, xt, xr, xt_x, xr_x);
  read_hands_data( HandDataFile, BODY_NAME_L_RING_EXTR, &val2, xt, xr, xt_x, xr_x);
      
  vt3d_set_null(r_tr);
  r_tr[0] = val2;
  matrix_combine(NULL, IDENTITY, r_tr , matdir);
  n3d_set_ini_dir(child, matdir);

  body_set_tlhand_ind(self, BODY_N3D_L_RING_EXTR, child);

  child = n3d_creation_child(root, BODY_NAME_L_MIDDLE_FL1);
  joint = joint_creation();
  n3d_attach_data(child, N3D_JOINT, joint);

  read_hands_data( HandDataFile, BODY_NAME_L_MIDDLE_FL1, &val, xt, xr, xt_x, xr_x);
  read_hands_data( HandDataFile, BODY_NAME_L_MIDDLE_FL1, &val, l_tr, l_rt, xt_x, xr_x);
  
  mr3d_from_axis_angle(mrdir, l_rt);
  matrix_compose(mrdir , l_tr , matdir);
      
      
  matrix_mult( matdir, xpto_, res );
  matrix_copy( res, matdir );
      
      
  n3d_set_ini_dir(child, matdir);

  body_set_tlhand_ind(self, BODY_N3D_L_MIDDLE_FL1, child);
  body_set_tjoint_lhand_ind(self, BODY_JOINT_L_MIDDLE_FL1, child);

  root1 = child;
  child = n3d_creation_child(root1, BODY_NAME_L_MIDDLE_PIVOT);
  joint = joint_creation();
  n3d_attach_data(child, N3D_JOINT, joint);
  body_set_tlhand_ind(self, BODY_N3D_L_MIDDLE_PIVOT, child);
  body_set_tjoint_lhand_ind(self, BODY_JOINT_L_MIDDLE_PIVOT, child);

  root1 = child;
  child = n3d_creation_child(root1, BODY_NAME_L_MIDDLE_TWIST);
  joint = joint_creation();
  n3d_attach_data(child, N3D_JOINT, joint);
  body_set_tlhand_ind(self, BODY_N3D_L_MIDDLE_TWIST, child);
  body_set_tjoint_lhand_ind(self, BODY_JOINT_L_MIDDLE_TWIST, child);

  root1 = child;
  child = n3d_creation_child(root1, BODY_NAME_L_MIDDLE_FL2);
  joint = joint_creation();
  n3d_attach_data(child, N3D_JOINT, joint);


  read_hands_data( HandDataFile, BODY_NAME_L_MIDDLE_FL2, &val, xt, xr, xt_x, xr_x);
  read_hands_data( HandDataFile, BODY_NAME_L_MIDDLE_FL2, &val2, xt, xr, xt_x, xr_x);

  vt3d_set_null(r_tr);
  r_tr[2] = val2;
  matrix_combine(NULL, Z_ON_Y, r_tr , matdir);
  n3d_set_ini_dir(child, matdir);

  body_set_tlhand_ind(self, BODY_N3D_L_MIDDLE_FL2, child);
  body_set_tjoint_lhand_ind(self, BODY_JOINT_L_MIDDLE_FL2, child);

  root1 = child;
  child = n3d_creation_child(root1, BODY_NAME_L_MIDDLE_FL3);
  joint = joint_creation();
  n3d_attach_data(child, N3D_JOINT, joint);


  read_hands_data( HandDataFile, BODY_NAME_L_MIDDLE_FL3, &val, xt, xr, xt_x, xr_x);
  read_hands_data( HandDataFile, BODY_NAME_L_MIDDLE_FL3, &val2, xt, xr, xt_x, xr_x);
      
  vt3d_set_null(r_tr);
  r_tr[0] = val2;
  matrix_combine(NULL, IDENTITY, r_tr , matdir);
  n3d_set_ini_dir(child, matdir);

  body_set_tlhand_ind(self, BODY_N3D_L_MIDDLE_FL3, child);
  body_set_tjoint_lhand_ind(self, BODY_JOINT_L_MIDDLE_FL3, child);

  root1 = child;
  child = n3d_creation_child(root1, BODY_NAME_L_MIDDLE_EXTR);


  read_hands_data( HandDataFile, BODY_NAME_L_MIDDLE_EXTR, &val, xt, xr, xt_x, xr_x);
  read_hands_data( HandDataFile, BODY_NAME_L_MIDDLE_EXTR, &val2, xt, xr, xt_x, xr_x);
      
  vt3d_set_null(r_tr);
  r_tr[0] = val2;
  matrix_combine(NULL, IDENTITY, r_tr , matdir);
  n3d_set_ini_dir(child, matdir);

  body_set_tlhand_ind(self, BODY_N3D_L_MIDDLE_EXTR, child);

  child = n3d_creation_child(root, BODY_NAME_L_INDEX_FL1);
  joint = joint_creation();
  n3d_attach_data(child, N3D_JOINT, joint);


  read_hands_data( HandDataFile, BODY_NAME_L_INDEX_FL1, &val, xt, xr, xt_x, xr_x);
  read_hands_data( HandDataFile, BODY_NAME_L_INDEX_FL1, &val, l_tr, l_rt, l_tr_x, xr_x);
      
  mr3d_from_axis_angle(mrdir, l_rt);
  matrix_compose(mrdir , l_tr , matdir);
      

  matrix_combine(NULL, Z_ON_X, l_tr_x, xpto);
  matrix_mult( matdir, xpto, res );
  matrix_copy( res, matdir );
      
  n3d_set_ini_dir(child, matdir);

  body_set_tlhand_ind(self, BODY_N3D_L_INDEX_FL1, child);
  body_set_tjoint_lhand_ind(self, BODY_JOINT_L_INDEX_FL1, child);

  root1 = child;
  child = n3d_creation_child(root1, BODY_NAME_L_INDEX_PIVOT);
  joint = joint_creation();
  n3d_attach_data(child, N3D_JOINT, joint);
  body_set_tlhand_ind(self, BODY_N3D_L_INDEX_PIVOT, child);
  body_set_tjoint_lhand_ind(self, BODY_JOINT_L_INDEX_PIVOT, child);

  root1 = child;
  child = n3d_creation_child(root1, BODY_NAME_L_INDEX_TWIST);
  joint = joint_creation();
  n3d_attach_data(child, N3D_JOINT, joint);
  body_set_tlhand_ind(self, BODY_N3D_L_INDEX_TWIST, child);
  body_set_tjoint_lhand_ind(self, BODY_JOINT_L_INDEX_TWIST, child);

  root1 = child;
  child = n3d_creation_child(root1, BODY_NAME_L_INDEX_FL2);
  joint = joint_creation();
  n3d_attach_data(child, N3D_JOINT, joint);


  read_hands_data( HandDataFile, BODY_NAME_L_INDEX_FL2, &val, xt, xr, xt_x, xr_x);
  read_hands_data( HandDataFile, BODY_NAME_L_INDEX_FL2, &val2, xt, xr, xt_x, xr_x);

  vt3d_set_null(r_tr);
  r_tr[2] = val2;
  matrix_combine(NULL, Z_ON_Y, r_tr , matdir);
  n3d_set_ini_dir(child, matdir);

  body_set_tlhand_ind(self, BODY_N3D_L_INDEX_FL2, child);
  body_set_tjoint_lhand_ind(self, BODY_JOINT_L_INDEX_FL2, child);

  root1 = child;
  child = n3d_creation_child(root1, BODY_NAME_L_INDEX_FL3);
  joint = joint_creation();
  n3d_attach_data(child, N3D_JOINT, joint);


  read_hands_data( HandDataFile, BODY_NAME_L_INDEX_FL3, &val, xt, xr, xt_x, xr_x);
  read_hands_data( HandDataFile, BODY_NAME_L_INDEX_FL3, &val2, xt, xr, xt_x, xr_x);
      
  vt3d_set_null(r_tr);
  r_tr[0] = val2;
  matrix_combine(NULL, IDENTITY, r_tr , matdir);
  n3d_set_ini_dir(child, matdir);

  body_set_tlhand_ind(self, BODY_N3D_L_INDEX_FL3, child);
  body_set_tjoint_lhand_ind(self, BODY_JOINT_L_INDEX_FL3, child);

  root1 = child;
  child = n3d_creation_child(root1, BODY_NAME_L_INDEX_EXTR);


  read_hands_data( HandDataFile, BODY_NAME_L_INDEX_EXTR, &val, xt, xr, xt_x, xr_x);
  read_hands_data( HandDataFile, BODY_NAME_L_INDEX_EXTR, &val2, xt, xr, xt_x, xr_x);
      
  vt3d_set_null(r_tr);
  r_tr[0] = val2;
  matrix_combine(NULL, IDENTITY, r_tr , matdir);
  n3d_set_ini_dir(child, matdir);

  body_set_tlhand_ind(self, BODY_N3D_L_INDEX_EXTR, child);

  child = n3d_creation_child(root, BODY_NAME_L_THUMB_FL1);
  joint = joint_creation();
  n3d_attach_data(child, N3D_JOINT, joint);


  read_hands_data( HandDataFile, BODY_NAME_L_THUMB_FL1, &val, xt, xr, xt_x, xr_x);
  read_hands_data( HandDataFile, BODY_NAME_L_THUMB_FL1, &val, xt, l_rt, l_tr_x, l_rt_x);

  vt3d_set_null(r_tr);
  
  mr3d_from_axis_angle(mrdir, l_rt);
  matrix_compose(mrdir , r_tr , matdir);
#ifdef NEWTHUMB
  {
    MatriX_t zonmx;
    matrix_combine(NULL, Z_ON_MINUS_X, r_tr, zonmx);
    matrix_mult(zonmx, matdir, matdir);
  }
#endif

  mr3d_from_axis_angle(mrdir_ab, l_rt_x);
  matrix_compose(mrdir_ab , l_tr_x , xpto);
  matrix_mult( matdir, xpto, res );
  matrix_copy( res, matdir );
  
  n3d_set_ini_dir(child, matdir);

  body_set_tlhand_ind(self, BODY_N3D_L_THUMB_FL1, child);
  body_set_tjoint_lhand_ind(self, BODY_JOINT_L_THUMB_FL1, child);

  root1 = child;
  child = n3d_creation_child(root1, BODY_NAME_L_THUMB_ABDUCT);
  joint = joint_creation();
  n3d_attach_data(child, N3D_JOINT, joint);
  body_set_tlhand_ind(self, BODY_N3D_L_THUMB_ABDUCT, child);
  body_set_tjoint_lhand_ind(self, BODY_JOINT_L_THUMB_ABDUCT, child);

  root1 = child;
  child = n3d_creation_child(root1, BODY_NAME_L_THUMB_TW);
  joint = joint_creation();
  n3d_attach_data(child, N3D_JOINT, joint);
  body_set_tlhand_ind(self, BODY_N3D_L_THUMB_TW, child);
  body_set_tjoint_lhand_ind(self, BODY_JOINT_L_THUMB_TW, child);

  root1 = child;
  child = n3d_creation_child(root1, BODY_NAME_L_THUMB_FL2);
  joint = joint_creation();
  n3d_attach_data(child, N3D_JOINT, joint);


  read_hands_data( HandDataFile, BODY_NAME_L_THUMB_FL2, &val, xt, xr, xt_x, xr_x);
  read_hands_data( HandDataFile, BODY_NAME_L_THUMB_FL2, &val2, xt, xr, xt_x, xr_x);
      
  vt3d_set_null(r_tr);

#ifdef NEWTHUMB
  r_tr[2] = val2;
  matrix_combine(NULL, Z_ON_Y, r_tr , matdir);
#else
  r_tr[0] = val2;
  matrix_combine(NULL, IDENTITY, r_tr , matdir);
#endif
  n3d_set_ini_dir(child, matdir);

  body_set_tlhand_ind(self, BODY_N3D_L_THUMB_FL2, child);
  body_set_tjoint_lhand_ind(self, BODY_JOINT_L_THUMB_FL2, child);

  root1 = child;
  child = n3d_creation_child(root1, BODY_NAME_L_THUMB_FL3);
  joint = joint_creation();
  n3d_attach_data(child, N3D_JOINT, joint);


  read_hands_data( HandDataFile, BODY_NAME_L_THUMB_FL3, &val, xt, xr, xt_x, xr_x);
  read_hands_data( HandDataFile, BODY_NAME_L_THUMB_FL3, &val2, xt, xr, xt_x, xr_x);
      
  vt3d_set_null(r_tr);
  r_tr[0] = val2;
  matrix_combine(NULL, IDENTITY, r_tr , matdir);
  n3d_set_ini_dir(child, matdir);

  body_set_tlhand_ind(self, BODY_N3D_L_THUMB_FL3, child);
  body_set_tjoint_lhand_ind(self, BODY_JOINT_L_THUMB_FL3, child);

  root1 = child;
  child = n3d_creation_child(root1, BODY_NAME_L_THUMB_EXTR);


  read_hands_data( HandDataFile, BODY_NAME_L_THUMB_EXTR, &val, xt, xr, xt_x, xr_x);
  read_hands_data( HandDataFile, BODY_NAME_L_THUMB_EXTR, &val2, xt, xr, xt_x, xr_x);
      
  vt3d_set_null(r_tr);
  r_tr[0] = val2;
  matrix_combine(NULL, IDENTITY, r_tr , matdir);
  n3d_set_ini_dir(child, matdir);

  body_set_tlhand_ind(self, BODY_N3D_L_THUMB_EXTR, child);

}


static	void	body_set_hand_tables(BODY_t  * self)
{
  N3D_t  * root;

  if(self->rhand && self->trhand == NULL)
  {
    self->nb_rhand = BODY_OPTION_NB_HAND;
    self->trhand = 
      (N3D_t  **) table_alloc(self->nb_rhand * (int) sizeof(N3D_t *));
    self->nb_joint_rhand = BODY_OPTION_NB_J_HAND;
    self->tjoint_rhand = 
      (JOINT_t  **) table_alloc(
	self->nb_joint_rhand * (int) sizeof(JOINT_t *));
    root =    self->tn3d[BODY_N3D_R_WRIST_PIVOT];
    body_ini_rhand_tables(self, root);
      

  }
  if(self->lhand && self->tlhand == NULL)
  {
    self->nb_lhand = BODY_OPTION_NB_HAND;
    self->tlhand = 
      (N3D_t  **) table_alloc(self->nb_lhand * (int) sizeof(N3D_t *));
    self->nb_joint_lhand = BODY_OPTION_NB_J_HAND;
    self->tjoint_lhand = 
      (JOINT_t  **) table_alloc(
	self->nb_joint_lhand * (int) sizeof(JOINT_t *));
    root =    self->tn3d[BODY_N3D_L_WRIST_PIVOT];
    body_ini_lhand_tables(self, root);
      
  }

}
    


Booleen_t	body_set_left_hand(BODY_t * self, Booleen_t activ)
{
  if (self)
  {
    if(self->lhand == activ) return(TRUE);



    if(activ && self->lhand == 2)
    {
      /* moving from one finger to hand */
      /* must destroy previous data */
      /* destruction of ALL associated tables & hierarchy nodes */
      table_free_n3d((TABLE_t *) self->tlhand, BODY_OPTION_NB_HAND);
      self->tlhand = NULL;
      self->nb_lhand = 0;
      table_free((TABLE_t *) self->tjoint_lhand);
      self->tjoint_lhand = NULL;
      self->nb_joint_lhand = 0;

    }
	
    self->lhand = activ;
    if (activ)
    {

	    
      if(!self->tlhand && self->tn3d)
      {	
	body_set_hand_tables(self);


	body_init_fix_ini_lhand(self);
	body_skeleton_hand_fit(self);
	

	h3d_update_ref_config(self->tn3d[BODY_N3D_L_WRIST_PIVOT]);
      }
    }	
    else if(self->tlhand != NULL)
    {   
      /* destruction of ALL associated tables & hierarchy nodes */
      table_free_n3d((TABLE_t *) self->tlhand, BODY_OPTION_NB_HAND);
      self->tlhand = NULL;
      self->nb_lhand = 0;
      table_free((TABLE_t *) self->tjoint_lhand);
      self->tjoint_lhand = NULL;
      self->nb_joint_lhand = 0;

    }

    return(TRUE);
  }
  return(FALSE);
}


Booleen_t	body_set_right_hand(BODY_t * self, Booleen_t activ)
{
  if (self)
  {
    if(self->rhand == activ) return(TRUE);
	


    if(activ && self->rhand == 2)
    {
      /* moving from one finger to hand */
      /* must destroy previous data */
      /* destruction of ALL associated tables & hierarchy nodes */
      table_free_n3d((TABLE_t *) self->trhand, BODY_OPTION_NB_HAND);
      self->trhand = NULL;
      self->nb_rhand = 0;
      table_free((TABLE_t *) self->tjoint_rhand);
      self->tjoint_rhand = NULL;
      self->nb_joint_rhand = 0;


    }
    self->rhand = activ;


    if (activ)
    {
      /* if the tn3d TABLE_t already exist */
      if(!self->trhand && self->tn3d)
      {	
	body_set_hand_tables(self);


	body_init_fix_ini_rhand(self);
	body_skeleton_hand_fit(self);
	

	h3d_update_ref_config(self->tn3d[BODY_N3D_R_WRIST_PIVOT]);
      }
    }
    else if(self->trhand != NULL)
    {   
      /* destruction of ALL associated tables & hierarchy nodes */
      table_free_n3d((TABLE_t *) self->trhand, BODY_OPTION_NB_HAND);
      self->trhand = NULL;
      self->nb_rhand = 0;
      table_free((TABLE_t *) self->tjoint_rhand);
      self->tjoint_rhand = NULL;
      self->nb_joint_rhand = 0;

    }
    return(TRUE);
  }
  return(FALSE);
}



/* ----------------------------------------------------------------- */

static  BODY_t *  body_create(void)
{
  BODY_t *    self;

  self = (BODY_t *)   malloc((unsigned int) sizeof(BODY));
  if(!self)
  {
    fprintf(stderr, " body_create() : memory allocation error\n" );
    exit(0);
  }
  return(self);
}


static  void    body_ini(BODY_t * self)
{
  self->idt         = NULL;

  self->scene       = NULL;
  self->golem       = NULL;

  self->template    = NULL;
  self->nb_instance = 0;

  self->orientation_mode  = BODY_Y_UP;


  self->total_height          = (float) 0.0;
  self->frontal_scaling       = (float) 1.0;
  self->high_lateral_scaling  = (float) 1.0;
  self->low_lateral_scaling   = (float) 1.0;
  self->spine_origin_ratio    = (float) 1.0;

  self->nb_n3d            = 0;
  self->tn3d              = NULL;

  self->joint             = NO;
  self->nb_joint          = 0;
  self->tjoint            = NULL;

  self->rhand             = NO;

  self->nb_rhand          = 0;
  self->trhand            = NULL;
  self->nb_joint_rhand    = 0;
  self->tjoint_rhand      = NULL;

  self->lhand             = NO;

  self->nb_lhand          = 0;
  self->tlhand            = NULL;
  self->nb_joint_lhand    = 0;
  self->tjoint_lhand      = NULL;

  self->rfoot             = NO;
  self->lfoot             = NO;
}






BODY_t *  body_insTance_creation( SCENE * scene)
{
  UNIT *      golem;
  UNIT *      topunit;
  BODY_t *    self;


  if(scene != NULL)   topunit = scene_get_top(scene);
  else
  {
    fprintf(stderr, "SCENE UNDEFINED in body_instance_creation()\n");
    return(NULL);
  }
  if(topunit != NULL) /* create new unit for this INSTANCE */
  {
    golem = unit_new(scene, "BODY_INSTANCE_UNIT", topunit);
    unit_creation_root(golem, "/");
  }
  else
  {
    fprintf(stderr, "TOPUNIT UNDEFINED in body_instance_creation()\n");
    return(NULL);
  }

  if(golem != NULL)
  {
    self = body_create();

    if(self)        body_ini(self);


    self->scene = scene;
    self->golem = golem;

    body_set_tables(self);



    h3d_update_ref_config(self->tn3d[BODY_N3D_GLOBAL_MOTION]);


    if(body_get_orientation(self) == BODY_Y_UP)
    {
      N3D_t *   globl;

      {
	MatriX	matdir;
	MR3D        mrdir;
	VT3D        r_tr = {0.000000,  0.000000,  0.000000};
	VT3D        r_rt = {-1.570796,  0.000000,  0.000000};

	globl = self->tn3d[BODY_N3D_GLOBAL_MOTION];
	mr3d_from_axis_angle(mrdir, r_rt);
	matrix_compose(mrdir, r_tr, matdir);
	n3d_set_ini_dir(globl, matdir);


	{
	  MatriX xpto = {
	    { -1., 0. , 0., 0. },
	    { 0. , 1. , 0., 0. },
	    { 0. , 0. ,-1., 0. },
	    { 0. , 0. , 0., 1. }
	  };
	  N3D_t * sht;
	  
	  sht =  self->tn3d[90];
	  n3d_set_ini_dir(sht, xpto);
	}
      }

    }
        
    body_iniT_fix_ini(self);

    return(self);
  }
  fprintf(stderr, " body_instance_creation(): UNIT Undefined\n" );
  return(NULL);
}






Booleen body_set_orientation(BODY_t * self, Booleen_t orientation)
{

  if(self)
  {
    N3D_t *   globl;


    globl = self->tn3d[BODY_N3D_GLOBAL_MOTION];
    self->orientation_mode = orientation;
	
    if (globl)
    {
      MatriX      matdir;
	    
      /* to keep translation intact (for BODY_xxxxxxx_MODE) */
      n3d_get_ini_dir(globl, matdir);
      if(orientation == BODY_Y_UP)
      {
	matdir[1][2] = (float) -1.0; /*-1*/
	matdir[2][1] = (float) 1.0;/*1*/
	matdir[1][1] = (float) 0.0; /*0*/
	matdir[2][2] = (float) 0.0; /*0*/
	/* NEW: */
	matdir[0][2] = (float) 0.0;
	matdir[1][0] = (float) 0.0;
	matdir[0][0] = (float) 1.0;
	matdir[2][0] = (float) 0.0;
	matdir[0][1] = (float) 0.0;
      }
      else
      {
	matdir[1][2] = (float) 0.0;
	matdir[2][1] = (float) 0.0;
	matdir[1][1] = (float) 1.0;
	matdir[2][2] = (float) 1.0;

      }
      n3d_set_ini_dir(globl, matdir);
      h3d_update_ref_config(globl);
    }
    return(TRUE);
  }
  return(FALSE);
}


extern PSCENE scene;


static  void    body_set_tables(BODY_t * self)
{
  self->nb_n3d = BODY_DEFAULT_NB_TN3D;
  self->tn3d = (N3D_t **) table_alloc(self->nb_n3d * (int) sizeof(N3D_t *));
  self->nb_joint = BODY_OPTION_NB_JOINT;
  self->tjoint = (JOINT **)table_alloc(
    self->nb_joint * (int) sizeof(N3D_t *));

  body_iniT_tables(self);
}



int     body_get_orientation(BODY_t * self)
{
  if(self)
  {
    return(self->orientation_mode);
  }
  return(FALSE);
}




TABLE *     table_alloc(int size)
{
  TABLE *     self;

                          self = (TABLE *) malloc((unsigned int) size);
        if(!self)
  {
                        fprintf(stderr, "table_alloc(): malloc error\n");
    return(NULL);
       }
  return(self);
}


/* ----------------------------------------------------------------- */


static Booleen_t body_switch_mode(BODY_t * self, MM val1, Booleen_t glomot)
{
  if(self)
  {
    N3D_t *      child;
    VT3D_t       v1;
    MR3D_t       mrdir;
    MatriX_t     matdir;



    child = self->tn3d[BODY_N3D_VIRTUAL_FLOOR];
    n3d_set_ini(child, IDENTITY, NULL);
    if(glomot)
    {
      mr3d_idt_(mrdir);
      child = self->tn3d[BODY_N3D_GLOBAL_MOTION];
      if(self->orientation_mode == BODY_Y_UP)
      {
	mrdir[1][2] = (float) -1.;
	mrdir[2][1] = (float) 1.;
	mrdir[1][1] = (float) 0.;
	mrdir[2][2] = (float) 0.;
      }
      vt3d_set_null(v1);
      matrix_compose(mrdir, v1, matdir);
      
      n3d_set_ini_dir(child, matdir);
    }

    child = self->tn3d[BODY_N3D_AVERAGE_POSITION];
    vt3d_set_null(v1);
    mr3d_idt_(mrdir);
    /* val1 -> spine_origin */
    v1[2] = val1;
    matrix_compose(mrdir, v1, matdir);
    n3d_set_ini_dir(child, matdir);

    child = self->tn3d[BODY_N3D_GLOBAL_MOTION];
    h3d_update_ref_config(child);


    return(TRUE);
  }
  return(FALSE);
}





float bodyChangeJointVal( BODY_t *body, int joint_ind, float new_val )
{

  int i, j;
  JOINT *joint_current;

  if (joint_ind == -1) return -10000.0;

  joint_current = body->tjoint[joint_ind];

  /*h3d_actual_ref(unit_get_root(body->golem),DIRECT_PROPAGATION,FALSE); TO REMOVE */

  joint_val_actual( joint_current, new_val );
  h3d_actual_ref(unit_get_root(body->golem),DIRECT_PROPAGATION,FALSE);
  return ( joint_get_val(joint_current) );
}


float bodyChangeRHandJointVal( BODY_t *body, int joint_ind, float new_val )
{
  JOINT *joint_current;

  if (joint_ind == -1) return -10000.0;

  joint_current = body->tjoint_rhand[joint_ind];
  joint_val_actual( joint_current, new_val );
  h3d_actual_ref(unit_get_root(body->golem),DIRECT_PROPAGATION,FALSE);
  return ( joint_get_val(joint_current) );
}



float bodyChangeLHandJointVal(  BODY_t *body, int joint_ind, float new_val )
{
  JOINT *joint_current;

  if (joint_ind == -1) return -10000.0;

  joint_current = body->tjoint_lhand[joint_ind];
  joint_val_actual( joint_current, new_val );
  h3d_actual_ref(unit_get_root(body->golem),DIRECT_PROPAGATION,FALSE);
  return ( joint_get_val(joint_current) );
}


N3D *bodyGetN3D( BODY_t *body, int n3dInd)
{
  return body->tn3d[n3dInd];
}

N3D *bodyGetLHandN3D( BODY_t *body, int n3dInd)
{
  return body->tlhand[n3dInd];
}

N3D *bodyGetRHandN3D( BODY_t *body, int n3dInd)
{
  return body->trhand[n3dInd];
}

int mytest( BODY *bodyview)
{
  int i;
  i=0;
}

