// NOTES for further users :
// This source is dedicated to MPEG4 body part. It's based on EPFLBODY
// stuff from EPFL. The goal of this class is to output a VRML animation
// file from a stream of BAP values. To do so, MPEG4 are converted to hanim
// params, stored in a local array, and dumped to a file when finished.

// The way to use is :
// *1* create a Mpeg4Body object,
// *2* call bodyCreate function,
// *3* fill at each frame the local BAPS of this class,
// *4* call bodyUpdate when finished the frame,
// *5* destroy the object when finished the job, it
//     will automatically dump the anim file.

#include "OpenGl/OpenGl.h"
#include <stdio.h>
#include <stdlib.h>

//#include "defs.h"
//#include "opengl/base/OpenGLPresenter.h"
#include "opengl/group/TransformProxy.h"
//#include "Nodes.h"

#include "mpeg4body.h"
#include "epflbody.h"
#include "hierarchy.h"

// This static is used for a compliant EPFLBODY
// use of that library (global management of
// many bodies referenced by an ID) 
static int numHumans=0;

// USEFULL DEFINES
#define TIME_INTERVAL 1.4            // time interval
#define DEFAULT_FILE "anim.wrl"       // default filename for output
#define MAX_NUM_FRAMES 500           // max number of frames

//*********************************************************************
// Implementation of the class
//*********************************************************************

// Constructor
Mpeg4Body::Mpeg4Body()
  : numFramesStored(0)
{
  // gets the ID of the new body
   bodyId = 0;

   // calls the EPFLBODY init for that object
   epflbody_init( bodyId );
}

// Destructor
Mpeg4Body::~Mpeg4Body()
{
}

// The BAPs management part
int Mpeg4Body::bodySetBAP( int bapId, int value)
{
  bapValues[bapId] = value;
  return TRUE;
}

int Mpeg4Body::bodyGetBAP( int bapId)
{
  return bapValues[bapId];
}

int Mpeg4Body::bodySetBAPs( int values[])
{
  for (int i=1; i<=MPEG4_BODY_NUM_BAPS; i++)
    bapValues[i] = values[i];
  return TRUE;
}

int Mpeg4Body::bodySetBAPMask(int bapId, int value)
{
	bapMasks[bapId]=value;
	return TRUE;
}

int Mpeg4Body::bodyGetBAPMask(int bapId)
{
	return bapMasks[bapId];
}

int Mpeg4Body::bodyGetBAPs( int values[])
{
  for (int i=1; i<=MPEG4_BODY_NUM_BAPS; i++)
    values[i] = bapValues[i];
  return TRUE;
}

char * Mpeg4Body::bodyGetJointName(int pos)
{
	return Hinfo[pos].VRML_name;
}

// Converts a frame of BAPS to hanim values
// and stores the values for later output
int Mpeg4Body::bodyUpdate(Transform ** pointerTable)
{
	int i;
	float tmpVal,angle;

	static MatriX Compensate = {	{ -1, 0, 0, 0},
									{  0,-1, 0, 0},
									{  0, 0,-1, 0},
									{  0, 0, 0, 1} };

	MR3D mx;
	VT3D axis, pos, v;
	MatriX R;
	MatriX bodyMatrices[2+VRML_NB_JOINTS+2*VRML_NB_HAND_JOINTS];

#define M_PI 3.1415926535
#define NBJ (VRML_NB_JOINTS+(2*VRML_NB_HAND_JOINTS))

	/*
	FILE * f = fopen("mpeg4body.bap","at");
	fprintf(f,"NEW FRAME OF BAPS\n");
	for (i=0 ; i<MPEG4_BODY_NUM_BAPS; i++)
		fprintf(f,"BAP[%3d]=%d\n",i,bapValues[i]);
	fprintf(f,"\n\n");
	fclose(f);*/

	// Set EPFLBODY internal values
	for (i=1; i<=MPEG4_BODY_NUM_BAPS; i++)
	{
		tmpVal = M_PI * (float)(bapValues[i-1])/(float)100000.0;
		epflbody_set_val( 0, i, tmpVal);
	}

	
	epflbody_body_update( 0, bodyMatrices);

	for (i=0; i<NBJ; i++)
	{
		matrix_copy( bodyMatrices[i], R);

		// raw copy
        // PORTING to CC 5.3
        int k = 0;
//		for (int k=0 ; k<3 ; k++)
		for ( k=0 ; k<3 ; k++)
			for (int l=0 ; l<3 ; l++)
				mx[k][l] = (float)R[k][l];

		//Column copy
		for (k=0 ; k<3 ; k++)
			v[k] = R[k][3];

		matrix_mult_vt3d( Compensate, v , pos );

		// Pos are calculated in case of a non rotational BAP
		pos[0] /= 1000.;
		pos[1] /= 1000.;
		pos[2] /= 1000.;

		// Get the axis and the rotations around them
		epflbody_mr3dI_to_axis_angle( axis, mx, &angle);

		if (Hinfo[i].order != BODY_TR)
		{	// Case of rotational BAP
			if (pointerTable[i]!=NULL)
				pointerTable[i]->rotation=Rotation(axis[0],axis[1],axis[2],angle);
		}
		else
		{	// Case of position BAP
			TRACE("EPFLBODY TRANSLATES JOINT %d <%3.3f,%3.3f,%3.3f>\n",i,pos[0],pos[1],pos[2]);
			//if (pointerTable[i]!=NULL)
			//	pointerTable[i]->translation=Vec3f(pos[0],pos[1],pos[2]);
		}
	}
	
	return FALSE;
}
