
/*------------------------------------------------------------------------------*/
/* 										*/
/* 										*/
/*  This software module was originally developed by Ronan Boulic and Tom Molet */
/*  (EPFL-LIG) and edited by Joaquim Esmerado (EPFL-LIG) 			*/
/*  in the course of development of the MPEG-4 SNHC 				*/
/*  VM. This software module is an implementation of a part of one or more 	*/
/*  MPEG-4 SNHC VM tools as specified by the MPEG-4 SNHC VM. ISO/IEC gives 	*/
/*  users of the MPEG-4 SNHC VM free license to this software module or 	*/
/*  modifications thereof for use in hardware or software products 		*/
/*  claiming conformance to the MPEG-4 SNHC VM. Those intending to use 		*/
/*  this software module in hardware or software products are advised that 	*/
/*  its use may infringe existing patents. The original developer of this 	*/
/*  software module and his/her company, the subsequent editors and their 	*/
/*  companies, and ISO/IEC have no liability for use of this software 		*/
/*  module or modifications thereof. Copyright is not released for non 		*/
/*  MPEG-4 SNHC VM conforming products. EPFL-LIG 				*/
/*  retains full right to use the code for its own purpose, assign or 		*/
/*  donate the code to a third party and to inhibit third parties from 		*/
/*  using the code for non MPEG-4 SNHC VM conforming products. This 		*/
/*  copyright notice must be included in all copies or derivative 		*/
/*  works. Copyright (1997).  							*/




# ifndef scn1_h
# define scn1_h


/*
** -------------------------------------
** -- Include of Header files ----------
** -------------------------------------
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#ifdef __cplusplus
extern "C" {
#endif

/*
** -------------------------------------
** -- Public define, typedef, functions
** -------------------------------------
*/

#ifndef __cplusplus
#ifndef private
#define private
#endif
#endif

#ifndef	NULL
#define NULL	0
#endif

typedef int	Booleen;
typedef Booleen	Booleen_t;

#ifndef	FALSE
#define FALSE	0
#endif

#ifndef	ECHEC
#define ECHEC	NULL
#endif

#ifndef	NO
#define NO	0
#endif

#ifndef	TRUE
#define TRUE	1
#endif

#ifndef	SUCCES
#define SUCCES	1
#endif

#ifndef	YES
#define YES	1
#endif


/* conversion factors from degree_to_radian and radian_to_degree	*/

#define DG_RD			0.0174533
#define RD_DG			57.29578

/*extern  void       fdbug(FILE * fd);*/



/*
** -------------------------------------
** -- Include of Header files ----------
** -------------------------------------
*/

#include	<stdio.h>	    /* for FILE type	    */




/* unit systems	*/

#define _UNIT_RD    0
#define _UNIT_MM    1
#define _UNIT_NR    2


/* predefined set of DIT transformation for initial DIT of a node_3D */

#define IDENTITE	      0
#define Z_SUR_X		      1
#define Z_SUR_Y		      2
#define Z_SUR_MOINS_Z	      3
#define Z_SUR_MOINS_X	      4
#define Z_SUR_MOINS_Y	      5
#define Z_QUELCONQUE	      6
#define GL_VERS_BODYLIB_Y_UP  7
#define GL_VERS_BODYLIB_Z_UP  8
#define Z_NON_DEFINI	     -1

#define IDENTITY	   IDENTITE
#define Z_ON_X		   Z_SUR_X	
#define Z_ON_Y		   Z_SUR_Y	
#define Z_ON_MINUS_Z	   Z_SUR_MOINS_Z
#define Z_ON_MINUS_X	   Z_SUR_MOINS_X
#define Z_ON_MINUS_Y	   Z_SUR_MOINS_Y
#define Z_OTHER		   Z_QUELCONQUE
#define GL_TO_BODYLIB_Y_UP GL_VERS_BODYLIB_Y_UP
#define GL_TO_BODYLIB_Z_UP GL_VERS_BODYLIB_Z_UP
#define Z_UNDEFINED	   Z_NON_DEFINI

#define AXIS_ANGLE_IDENTITY	0.0000000
#define AXIS_ANGLE_Z_ON_X	1.2091996    /* 2 Pi / (3 sqrt(3)) */
#define AXIS_ANGLE_Z_ON_Y	2.4183992    /* 4 Pi / (3 sqrt(3)) */

/* hierarchy traversal direction */

#define SENS_DIRECT		0

/* some epsilon value to round off zero	*/

#define EPSIL_ZERO	0.000001


typedef float	MatriX[4][4];	/* like "gl.h" MatriX		*/
typedef float	MR3D[3][3];	/* rotation matrix (orthogonale)*/
typedef float	VT3D[3];	/* Vector 3d			*/
typedef float	VHOM[4];	/* Homogeneous Vector 3d	*/
typedef	MatriX  MatriX_t;
typedef MR3D    MR3D_t;
typedef VT3D    VT3D_t;
typedef VHOM    VHOM_t;

/* Direct and Inverse Homogeneous Transformations into a DIT structure*/

typedef struct dit DIT;
typedef struct dit DIT_t;

struct dit
{
  MatriX_t 	dir;	/* Direct  MatriX  (from here to there)	*/
  MatriX_t	inv;	/* Inverse MatriX  (from there to here)	*/
};


/*-------------------------------------------------------------------*/
/*-------------------------------------------------------------------*/

extern	DIT_t *	dit_creation  (void);
extern	void	dit_idt_      (DIT_t *p);

extern	void	dit_copy      (DIT_t *p_src ,
			       DIT_t *p_dst);
extern	void	dit_actual_rot   (float	angle, 
				  DIT_t	*p);
extern	void	dit_actual_trn   (float	dist, 
				  DIT_t	*p);
extern	DIT_t * dit_creation_ini (short tpini);
extern	DIT_t * dit_edit_ini     (DIT_t	*ini, 
				  short	*tpini);
extern	DIT_t * dit_destroy_ini  (DIT_t *ini, 
				  short	tpini);


/*-------------------------------------------------------------------*/

extern	void	matrix_idt_ (MatriX_t matrix);

extern	void	matrix_copy (MatriX_t src, 
			     MatriX_t dst);

extern	void	matrix_load (DIT_t	*p,
			     Booleen_t	sens, 
			     MatriX_t	a);
extern	void	matrix_compose (MR3D_t   mr, 
				VT3D_t	 vt, 
				MatriX_t matrix);
extern	void	matrix_combine (MR3D_t   mr, 
				short	 tpini, 
				VT3D_t	 vt, 
				MatriX_t matrix);
extern	void	matrix_decompose (MR3D_t   mr, 
				  VT3D_t   vt, 
				  MatriX_t matrix);


extern	void	matrix_mult_vt3d (MatriX_t a, 
				  VT3D_t   u,
				  VT3D_t   v);
extern	void	matrix_inverse (MatriX_t matrix,
				MatriX_t matrix_inv);

extern	void	matrix_mult (MatriX_t	ma,
			     MatriX_t	mb,
			     MatriX_t	mc);	


/*-------------------------------------------------------------------*/

extern	void 	mr3d_idt_ (MR3D_t    mr);

extern	void	mr3d_self_transpose (MR3D_t ma);

extern	void	mr3d_mult_vt3d (MR3D_t ma,
				VT3D_t	vu,
				VT3D_t	vv);

extern	void	mr3d_from_axis_angle (MR3D_t mr, 
				      VT3D_t axis);
extern	void	mr3d_to_axis_angle   (VT3D_t axis, 
				      MR3D_t mr);


/*-------------------------------------------------------------------*/


extern	void	vt3d_copy (VT3D_t src,
			   VT3D_t dst);
extern	void	vt3d_set_null (VT3D_t	v);


/*------------------------------------------------------------------*/



/*
** -------------------------------------
** -- Include of Header files ----------
** -------------------------------------
*/


#include <stdio.h>




#define CCOLOR_YELLOW		5
#define CCOLOR_MAGENTA		6
#define CCOLOR_CYAN		7

#define CCOLOR_LORANGE		12

typedef float CColor[3] ;		/* RGB color vector	*/
typedef CColor   CColor_t;


/*
|  UNITS:
|     mm       millimeters
*/


/***
 *  defined functions
 ***/



typedef struct world WORLD;
typedef struct world * PWORLD;
typedef struct world WORLD_t;
typedef struct world * PWORLD_t;

#define WORLD_FRAME_SIZE	1	    /* in millimeter */



typedef struct node_3D N3D;

typedef struct node_3D N3D_t;



typedef struct unit_ UNIT, UNIT_t ;
typedef struct unit_ * PUNIT , * PUNIT_t;




typedef struct scene_ SCENE;
typedef struct scene_ * PSCENE;
typedef struct scene_ SCENE_t;
typedef struct scene_ * PSCENE_t;


#define	SCENE_TOP_UNIT_NAME	    "/"
#define	SCENE_TOP_UNIT_ROOT_NAME    "/"

typedef Booleen_t (*FUNC_ONUNIT_t)(UNIT_t * self, void * data);

/************************************************************************/

extern	SCENE_t * scene_new(char * idt);

extern  UNIT_t *  scene_get_top(SCENE_t * self);




#define	UNIT_DEFAULT_ROOT_NAME    SCENE_TOP_UNIT_ROOT_NAME


/*
** -------------------------------------
** -- Creation -------------------------
** -------------------------------------
*/

extern  UNIT_t	*unit_new(SCENE *scene, char *idt, UNIT_t *parent);
extern	N3D_t	*unit_creation_root(UNIT_t  *self, char *idt);



/*
** -------------------------------------
** -- Get ------------------------------
** -------------------------------------
*/

extern	UNIT_t *  unit_get_top(UNIT_t	*self);

extern	UNIT_t *  unit_get_child(UNIT_t	*self, int childindex);
extern	N3D_t *	unit_get_root(UNIT_t	*self);
extern	N3D_t *   unit_get_scene_root(UNIT_t *self);
extern	WORLD *	unit_get_world(UNIT_t	*self);

/*
** -------------------------------------
** -- Manipulation ---------------------
** -------------------------------------
*/
extern	Booleen_t	unit_same_scene(UNIT_t	*u1, UNIT_t *u2);
extern  Booleen_t	unit_check_lineage(UNIT_t *self, UNIT_t *potential_ancestor);

extern	Booleen_t	unit_add_called(UNIT_t * self, UNIT_t * parent, 
				N3D_t *n3d_parent, char * idt);


extern	Booleen_t unit_check_dag(UNIT_t	*to_call, UNIT_t *calling);


/*
** -------------------------------------
** -- Display --------------------------
** -------------------------------------
*/
extern  Booleen_t unit_display_ref(UNIT_t * self);




/************************************************************************/

extern	Booleen_t     unit_clear_root(UNIT_t * self);

extern  void	      unit_reset_visited(UNIT_t	*self);

extern  Booleen_t     unit_lowlevel_destroy(UNIT_t *self);






typedef struct node_3D N3D;

typedef struct node_3D N3D_t;




/* Booleen de selection de transformation  (reference/transitoire)	*/

#define N3D_REFERENCE_TRANSFORMATION	1

#define N3D_OFFSET_NAME_TEST_VISUAL	5.
#define N3D_OFFSET_NAME_VISUAL		20.
#define N3D_OFFSET_TYPE_VISUAL		-10.

#define N3D_COLOR_TEST		CCOLOR_YELLOW
#define N3D_COLOR_NAME		CCOLOR_CYAN
#define N3D_COLOR_TRANSIENT	CCOLOR_LORANGE
#define N3D_COLOR_REFERENCE	CCOLOR_MAGENTA

#define N3D_FRAME_SIZE		50.
#define N3D_FRAME_SIZE_NEUTRAL	50.

/*
** -------------------------------------
** -- Creation -------------------------
** -------------------------------------
*/
extern  N3D_t *n3d_creation(UNIT	*unit, char *idt);
extern  N3D_t *n3d_creation_child(N3D_t *parent, char *idt);

/*
** -------------------------------------
** -- Destruction ----------------------
** -------------------------------------
*/
extern  Booleen_t n3d_destroy(N3D_t 	*self);

/*
** -------------------------------------
** -- Set ------------------------------
** -------------------------------------
*/
extern  Booleen_t	n3d_set_name(N3D_t *self, char   *name);

extern	Booleen_t	n3d_set_ini(N3D_t  *self, short tpini, DIT_t *ini);

extern	Booleen_t	n3d_set_ini_dir(N3D_t *self, MatriX_t dir);

extern	Booleen_t	n3d_outdate_ref(N3D  * self); 


/*
** -------------------------------------
** -- Get ------------------------------
** -------------------------------------
*/
extern	UNIT_t * n3d_get_unit(N3D_t *self);
extern	N3D_t *  n3d_get_unit_root(N3D_t *self);
extern	WORLD_t	*n3d_get_world(N3D_t *self);
extern	N3D_t *  n3d_get_world_root(N3D_t *self);
extern	N3D_t *  n3d_get_parent(N3D_t *self);


extern	int	 n3d_get_nb_children(N3D_t *self);

extern	N3D_t *  n3d_get_child(N3D_t *self, int child_index);

extern	short	n3d_get_type(N3D_t *self);
extern	Booleen_t n3d_get_type_name(N3D_t *self, char name[]);
extern	void   *n3d_get_data(N3D_t *self);


extern	int	n3d_get_id(N3D_t *self);

extern	char *	n3d_get_local_name(N3D_t *self); 

extern  N3D_t * n3d_get_left(N3D_t *self);

extern  N3D_t * n3d_get_down(N3D_t *self);

extern	Booleen_t	n3d_get_ini(N3D_t *self, DIT_t *ini);
extern	Booleen_t	n3d_get_ini_dir(N3D_t *self, MatriX_t dir);
extern	Booleen_t	n3d_get_ini_inv(N3D_t *self, MatriX_t inv);

extern	Booleen_t	n3d_ref_is_todate(N3D_t *self);

extern	Booleen_t	n3d_get_ref_dir(N3D_t *self, MatriX_t dir);
extern	Booleen_t	n3d_get_ref_inv(N3D_t *self, MatriX_t inv);

/*
** -------------------------------------
** -- Manipulation ---------------------
** -------------------------------------
*/
extern	Booleen_t	n3d_belong_to_family(N3D_t *self, N3D_t *ancestor);
extern	Booleen_t	n3d_is_child(N3D_t *self, N3D_t *parent, short *cindex);
extern	Booleen_t	n3d_is_parent(N3D_t *self, N3D_t *child);

extern  Booleen_t	n3d_add_child(N3D_t *self, N3D_t *child);
extern  Booleen_t	n3d_remove_child(N3D_t *self, N3D_t *child);

extern  Booleen_t	n3d_attach_data( N3D_t *self, 
					short   type, void *data);

extern  Booleen_t       n3d_destroy_data( N3D_t   *self);

extern  Booleen_t n3d_ref_refparent(N3D_t *self);

/*
** -------------------------------------
** -- Display --------------------------
** -------------------------------------
*/
extern  Booleen_t n3d_display_ref(N3D_t *	self, N3D_t *child, 
					Booleen_t test_display);





/* number of data types associated with node_3D	*/

#define NB_DATA_TYPES          11
#define N3D_MAX_TYPE_NAME      12

extern const int  nb_data_types;
extern const int  n3d_max_type_name;


/* data types associated with node_3D		*/

/* Reserved Index of Data Type */
#define	N3D_NEUTRAL	0
#define N3D_WORLD	1
#define N3D_JOINT	2
#define N3D_FREE	3 

/*
** -------------------------------------
** -- Get ------------------------------
** -------------------------------------
*/
extern char *	n3dplug_let_type_name(N3D * self);
extern Booleen	n3dplug_get_type_name(N3D * self, char name[]);

extern Booleen  n3dplug_get_default_color(N3D * self, CColor col); 




/*
** -------------------------------------
** -- define, typedef, functions
** -------------------------------------
*/

#define	    DIRECT_PROPAGATION	    SENS_DIRECT
#define	    INVERSE_PROPAGATION	    SENS_INVERSE

typedef struct box_3D B3D, B3D_t;
typedef struct box_3D * PB3D;

#ifdef global
#undef global
#endif

struct node_3D
{
  char	  *idt;   /* pointer on string identificator(unique in UNIT)*/
  int	  idi;	  /* unique integer in the SCENE it belongs to	    */
  UNIT_t  *unit;  /* UNIT it belongs to				    */
  N3D_t	  *parent;/* parent node_3D in the 3D hierarchy		    */

	
  short	  nbc;	  /* number of children node_3D			    */
  N3D_t	  **lstc; /* pointer to array of node_3D children pointers  */

  short	  tpini;  /* type of initial goemetric transformation	    */
  DIT_t	  *ini;	  /* pointer on initial geom. transformations	    */

	
  Booleen_t iniok;  /* TRUE if ini is consistent with current state   */

  DIT_t	  *ref;	  /* pointer on reference geom. transformations/unit*/
  Booleen_t refok;  /* TRUE if ref is consistent with current state   */


  Booleen_t fvis;	  /* TRUE to visualize the reference frame	    */
  Booleen_t svis;	  /* TRUE to visualize the initial translation	    */
  Booleen_t nvis;	  /* TRUE to visualize the string identifier	    */
  Booleen_t tvis;	  /* TRUE to visualize the type name		    */
  Booleen_t dvis;	  /* TRUE to visualize the data specific visibility */
  CColor_t  col;	  /* node_3D RGB color				    */
  int	  material_index; /* material index for display             */

  short	  type;	  /* node_3D type				    */
  void	  *str;	  /* generic pointer on the associated data structure*/

  N3D_t	  **lsts; /* pointer to the array of sharing node_3D pointer*/
	
};


/********************/
/*     Creation     */ 
/********************/

extern	N3D_t	*n3d_creation_world(UNIT_t *unit, char *idt);


/********************/
/*       Set        */
/********************/

extern  Booleen_t	n3d_set_type(N3D_t * self, short type);
extern	Booleen_t	n3d_set_ref(N3D_t	* self, DIT_t *ref);
extern	Booleen_t	n3d_set_ref_dir(N3D_t * self, MatriX_t dir);


/********************/
/*    Display       */
/********************/

extern void n3d_display_frame(N3D_t *self, Booleen_t ref,
			      Booleen_t test_display);

extern void n3d_display_segment(N3D_t *self, 
				N3D_t *child, 
				Booleen_t ref, 
				Booleen_t test_display);


/********************/
/*	 Misc       */
/********************/

extern Booleen n3d_get_local (N3D *self);



/*
** -------------------------------------
** -- Manipulation ---------------------
** -------------------------------------
*/
extern Booleen	n3dplug_attach_data( N3D * self);

extern N3D *n3dplug_get_n3d(void *data, short type);


/*
** -------------------------------------
** -- Display --------------------------
** -------------------------------------
*/
extern void  n3dplug_display_data(N3D * self, CColor colval);


typedef struct unit_ UNIT, UNIT_t ;


/*
** -------------------------------------
** -- define, typedef, functions
** -------------------------------------
*/

#define	UNIT_DEFAULT_ROOT_NAME    SCENE_TOP_UNIT_ROOT_NAME


/*
** -------------------------------------
** -- Creation -------------------------
** -------------------------------------
*/

extern  UNIT_t	*unit_new(SCENE *scene, char *idt, UNIT_t *parent);
extern	N3D_t	*unit_creation_root(UNIT_t  *self, char *idt);

/*
** -------------------------------------
** -- Set ------------------------------
** -------------------------------------
*/

extern	UNIT_t *  unit_check_name(UNIT_t	*top, char *idt);

/*
** -------------------------------------
** -- Get ------------------------------
** -------------------------------------
*/

extern	UNIT_t *  unit_get_top(UNIT_t	*self);

extern	UNIT_t *  unit_get_child(UNIT_t	*self, int childindex);
extern	N3D_t *	unit_get_root(UNIT_t	*self);
extern	N3D_t *   unit_get_scene_root(UNIT_t *self);
extern	WORLD *	unit_get_world(UNIT_t	*self);

/*
** -------------------------------------
** -- Manipulation ---------------------
** -------------------------------------
*/

extern	Booleen_t	unit_add_called(UNIT_t * self, UNIT_t * parent, 
				N3D_t *n3d_parent, char * idt);

extern	Booleen_t unit_check_dag(UNIT_t	*to_call, UNIT_t *calling);


/*
** -------------------------------------
** -- Display --------------------------
** -------------------------------------
*/
extern  Booleen_t unit_display_ref(UNIT_t * self);


typedef struct joint_ JOINT;

typedef struct joint_ JOINT_t;

#define JOINT_ROTATION       1
#define JOINT_TRANSLATION    2


/************************************************************************/

/*
** -------------------------------------
** -- Creation -------------------------
** -------------------------------------
*/
extern	JOINT_t      	*joint_creation(void);

extern	Booleen_t	joint_set_limits(JOINT_t *self , float	new_min, 
					float new_max);

extern  Booleen_t	joint_val_actual(JOINT_t  *self , 
			      float	new_value);

/*
** -------------------------------------
** -- Get ------------------------------
** -------------------------------------
*/

extern  N3D_t *   joint_get_n3d(JOINT_t * self);

extern	MatriX_t	*joint_let_loc_inv(JOINT_t *self);

extern	float	joint_get_val(JOINT_t *self);






     
/* structure describing the mobility associated to a node 3d		*/

struct joint_
{
  N3D		*n3d;	
  short		type;	
  short		status;	
  Booleen	artok;	
  float		val;	
  float		vm1;	
  float		vm2;	
  Booleen	limit;	
  float		bmin;	
  float		bmax;	
  float		vmax;	
  DIT		*art;	
};


/*
** -------------------------------------
** -- Manipulation ---------------------
** -------------------------------------
*/
extern	Booleen	joint_n3d_attach(JOINT *joint, N3D *n3d);




#define STR_CPY(dest,source) {\
\
    if(dest) free(dest);\
\
    dest = malloc((strlen(source)+1)*sizeof(char));\
    if(!(dest)) \
    {\
      printf("memory allocation failure with source %s\n", source); \
      exit(0);\
    }\
    (dest)[0] = '\0' ; strcpy(dest,source);\
}





#define STR_PREF(dest,prefix) {char *old; old = dest ;\
\
    if(old)  dest = malloc((strlen(old)+strlen(prefix)+1)*sizeof(char));\
\
    else     dest = malloc((strlen(prefix)+1)*sizeof(char));\
    if(!(dest)) \
    {\
      printf("memory allocation failure with prefix %s\n", prefix); \
      exit(0);\
    }\
\
    (dest)[0] = '\0' ; strcat(dest,prefix); strcat(dest, old); free(old);\
}





#define STR_POST(dest,suffix) {char *old; old = dest ;\
\
    if(old)  dest = malloc((strlen(old)+strlen(suffix)+1)*sizeof(char));\
\
    else     dest = malloc((strlen(suffix)+1)*sizeof(char));\
    if(!(dest)) \
    {\
      printf("memory allocation failure with suffix %s\n", suffix); \
      exit(0);\
    }\
\
    (dest)[0] = '\0' ; strcat(dest,old); strcat(dest, suffix); free(old);\
}






#define SET_IDT(p,new_idt) {\
\
    if(p->idt) free(p->idt);\
\
    p->idt = malloc((strlen(new_idt)+1)*sizeof(char));\
    if(!(p->idt)) \
    {\
      printf("memory allocation failure with new_idt %s\n", new_idt); \
      exit(0);\
    }\
    p->idt[0] = '\0' ; strcpy(p->idt,new_idt); \
}






#define PREF_IDT(p,prefix) {char *old, *s; old = p->idt ; \
\
    if(old) s = malloc((strlen(old)+strlen(prefix)+1)*sizeof(char));\
\
    else    s = malloc((strlen(prefix)+1)*sizeof(char));\
    if(!s) {printf("memory allocation failure with prefix %s\n", prefix); \
	    exit(0);}\
    s[0] = '\0' ; strcat(s,prefix); strcat(s, old); p->idt = s ; \
\
    free(old);\
}






#define POST_IDT(p,suffix) {char *old, *s; old = p->idt ; \
\
    if(old) s = malloc((strlen(old)+strlen(suffix)+1)*sizeof(char));\
\
    else    s = malloc((strlen(suffix)+1)*sizeof(char));\
    if(!s) {printf("memory allocation failure with suffix %s\n", suffix); \
	    exit(0);}\
    s[0] = '\0' ; strcat(s,old); strcat(s, suffix); p->idt = s ; \
\
    free(old);\
}





#define	POINTER_UNDEFINED(type, here)\
	printf("warning : undefined %s pointer in %s\n", type, here);

#define	SCENE_UNDEFINED(here)\
		printf("warning : SCENE undefined in %s\n",here);

#define	UNIT_UNDEFINED(here)\
		printf("warning : UNIT undefined in %s\n",here);

#define	CALL_UNDEFINED(here)\
		printf("warning : CALL undefined in %s\n",here);

#define	N3D_UNDEFINED(here)\
		printf("warning : N3D undefined in %s\n",here);

#define	TYPE_UNDEFINED(here)\
		printf("warning : unknown data type in %s\n",here);

#define	JOINT_UNDEFINED(here)\
		printf("warning : JOINT undefined in %s\n",here);

#define	FREE_UNDEFINED(here)\
		printf("warning : FREE undefined in %s\n",here);


#define	WORLD_UNDEFINED(here)\
		printf("warning : WORLD undefined in %s\n",here);


#define WARNING_WORLD   printf("warning : mixing of WORLDs is forbidden\n");




extern	DIT	*dit_let(short type);



/*
** -------------------------------------
** -- define, typedef, functions
** -------------------------------------
*/

#define ID_ERROR		    -1


#define	DEPENDANCY_BILATERAL	1


typedef struct world WORLD;

typedef struct world WORLD_t;


#define WORLD_FRAME_SIZE	1	    /* in millimeter */




/*
** -------------------------------------
** -- Creation -------------------------
** -------------------------------------
*/
extern	WORLD	*world_creation(void);
extern	int	world_new_n3d(WORLD *self, void *v);



/*
** -------------------------------------
** -- Manipulation ---------------------
** -------------------------------------
*/


extern	Booleen world_n3d_attach(WORLD *self, N3D *n3d);




/*
** -------------------------------------
** -- define, typedef, functions
** -------------------------------------
*/

typedef struct free_ FREE_;

typedef struct free_ FREE_t;


#define FREE_DEFAULT_MODE		0

#define  RT_XYZ           0   

#define FREE_FRAME_SIZE		50  /* millimeter   */

#define	FREE_Z_UP   0  /* classic mathematician notation for 
						   3D system */
#define	FREE_Y_UP   1  /* classic informatician notation for 
						   3D system display */

/*********************************************************************/

/*
** -------------------------------------
** -- Creation -------------------------
** -------------------------------------
*/
extern	FREE_t *free_creation(void);


/*
** -------------------------------------
** -- Get ------------------------------
** -------------------------------------
*/
extern  N3D_t *   free_get_n3d(FREE_t *         self);

extern	MatriX_t	*free_let_loc_inv(FREE_t     *self);




/* -> For use only for hierarchy management in n3dmain.c or h3d.c       */     
/* structure decrivant une transformation free associee a un noeud 3d*/

struct free_
{
  N3D		*n3d;	
  short		mode;	
  Booleen	param;	
  short		rt_type;
  VT3D		rt;	
  VT3D		rtm1;	
  VT3D		rtm2;	
  float		a;     
  float		am1;    
  float		am2;    
  Booleen	libok;	
  DIT		*lib;	
  DIT		*libm1;	
  DIT		*libm2;	
};

/*
** -------------------------------------
** -- Manipulation ---------------------
** -------------------------------------
*/
extern	Booleen free_n3d_attach(FREE_	*self, N3D	*n3d);


/*
** -------------------------------------
** -- Display --------------------------
** -------------------------------------
*/
extern	Booleen	free_display(FREE_	*self, CColor col);




#ifdef __cplusplus
}
#endif
#endif


