// -*- c++ -*-

// This software module was originally developed by Karl Anders ygard
// (Telenor R&D) in the course of development of the MPEG-4 Systems
// standard (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// For more information or to receive an up to date version of this module,
// mail karlo@nta.no or check <http://televr.fou.telenor.no/compositor/>.
//
// Copyright 1997, 1998.

#include "OpenGl/OpenGl.h"
#include "BoxProxy.h"
// #include "geometry/Box.h"
#include "opengl/base/OpenGLVisualRenderer.h"


// Render object description

/* virtual */ void
BoxProxy::RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD /* dwTime */)
{
  GLfloat color[] = { 0.0f, 0.0f, 1.0f, 0.75f};

  //satish changed this line for compilation purposes
        //have to be fixed afterwards
  //::glColor4fv(color);
  glColor4fv(color);

  RenderDescriptionBox(pRenderer, "Box");
}

// Render object

/* virtual */ BOOL
BoxProxy::Render(OpenGLVisualRenderer* pRenderer, DWORD /* dwTime */)
{
  // Box* pBox = (Box*) GetMediaObject();

  static GLfloat texCoord[][2] = {{ 0.0f, 1.0f},
				  { 1.0f, 1.0f},
				  { 1.0f, 0.0f},
				  { 0.0f, 0.0f}};

  const Vec3f& size = /* pBox-> */ GetSize();

  float x_side = size.x / 2.0f;
  float y_side = size.y / 2.0f;
  float z_side = size.z / 2.0f;

  if (!m_nDisplayList) {
    m_nDisplayList = ::glGenLists(1);

    ::glNewList(m_nDisplayList, GL_COMPILE_AND_EXECUTE);

    ::glPushAttrib(GL_POLYGON_BIT);

    ::glEnable(GL_CULL_FACE);

    ::glBegin(GL_QUADS);
  //satish changed this line for compilation purposes
        //have to be fixed afterwards
    //::glNormal3f(0.0f, 0.0f, -1.0f);
    glNormal3f(0.0f, 0.0f, -1.0f);

  //satish changed this block for compilation purposes
        //have to be fixed afterwards

	/*
    ::glTexCoord2fv(texCoord[0]); ::glVertex3f( x_side, -y_side, -z_side);
    ::glTexCoord2fv(texCoord[1]); ::glVertex3f(-x_side, -y_side, -z_side);
    ::glTexCoord2fv(texCoord[2]); ::glVertex3f(-x_side,  y_side, -z_side);
    ::glTexCoord2fv(texCoord[3]); ::glVertex3f( x_side,  y_side, -z_side);

	*/
    glTexCoord2fv(texCoord[0]); glVertex3f( x_side, -y_side, -z_side);
    glTexCoord2fv(texCoord[1]); glVertex3f(-x_side, -y_side, -z_side);
    glTexCoord2fv(texCoord[2]); glVertex3f(-x_side,  y_side, -z_side);
    glTexCoord2fv(texCoord[3]); glVertex3f( x_side,  y_side, -z_side);

  //satish changed this block for compilation purposes
        //have to be fixed afterwards

	/*
    ::glNormal3f(0.0f, 1.0f, 0.0f);
    ::glTexCoord2fv(texCoord[0]); ::glVertex3f(-x_side,  y_side,  z_side);
    ::glTexCoord2fv(texCoord[1]); ::glVertex3f( x_side,  y_side,  z_side);
    ::glTexCoord2fv(texCoord[2]); ::glVertex3f( x_side,  y_side, -z_side);
    ::glTexCoord2fv(texCoord[3]); ::glVertex3f(-x_side,  y_side, -z_side);

    ::glNormal3f(0.0f, 0.0f, 1.0f);
    ::glTexCoord2fv(texCoord[0]); ::glVertex3f(-x_side, -y_side,  z_side);
    ::glTexCoord2fv(texCoord[1]); ::glVertex3f( x_side, -y_side,  z_side);
    ::glTexCoord2fv(texCoord[2]); ::glVertex3f( x_side,  y_side,  z_side);
    ::glTexCoord2fv(texCoord[3]); ::glVertex3f(-x_side,  y_side,  z_side);

    ::glNormal3f(-1.0f, 0.0f, 0.0f);
    ::glTexCoord2fv(texCoord[0]); ::glVertex3f(-x_side, -y_side, -z_side);
    ::glTexCoord2fv(texCoord[1]); ::glVertex3f(-x_side, -y_side,  z_side);
    ::glTexCoord2fv(texCoord[2]); ::glVertex3f(-x_side,  y_side,  z_side);
    ::glTexCoord2fv(texCoord[3]); ::glVertex3f(-x_side,  y_side, -z_side);

    ::glNormal3f(0.0f, -1.0f, 0.0f);
    ::glTexCoord2fv(texCoord[0]); ::glVertex3f( x_side, -y_side,  z_side);
    ::glTexCoord2fv(texCoord[1]); ::glVertex3f(-x_side, -y_side,  z_side);
    ::glTexCoord2fv(texCoord[2]); ::glVertex3f(-x_side, -y_side, -z_side);
    ::glTexCoord2fv(texCoord[3]); ::glVertex3f( x_side, -y_side, -z_side);

    ::glNormal3f(1.0f, 0.0f, 0.0f);
    ::glTexCoord2fv(texCoord[0]); ::glVertex3f( x_side, -y_side,  z_side);
    ::glTexCoord2fv(texCoord[1]); ::glVertex3f( x_side, -y_side, -z_side);
    ::glTexCoord2fv(texCoord[2]); ::glVertex3f( x_side,  y_side, -z_side);
    ::glTexCoord2fv(texCoord[3]); ::glVertex3f( x_side,  y_side,  z_side);
	*/
    glNormal3f(0.0f, 1.0f, 0.0f);
    glTexCoord2fv(texCoord[0]); glVertex3f(-x_side,  y_side,  z_side);
    glTexCoord2fv(texCoord[1]); glVertex3f( x_side,  y_side,  z_side);
    glTexCoord2fv(texCoord[2]); glVertex3f( x_side,  y_side, -z_side);
    glTexCoord2fv(texCoord[3]); glVertex3f(-x_side,  y_side, -z_side);

    glNormal3f(0.0f, 0.0f, 1.0f);
    glTexCoord2fv(texCoord[0]); glVertex3f(-x_side, -y_side,  z_side);
    glTexCoord2fv(texCoord[1]); glVertex3f( x_side, -y_side,  z_side);
    glTexCoord2fv(texCoord[2]); glVertex3f( x_side,  y_side,  z_side);
    glTexCoord2fv(texCoord[3]); glVertex3f(-x_side,  y_side,  z_side);

    glNormal3f(-1.0f, 0.0f, 0.0f);
    glTexCoord2fv(texCoord[0]); glVertex3f(-x_side, -y_side, -z_side);
    glTexCoord2fv(texCoord[1]); glVertex3f(-x_side, -y_side,  z_side);
    glTexCoord2fv(texCoord[2]); glVertex3f(-x_side,  y_side,  z_side);
    glTexCoord2fv(texCoord[3]); glVertex3f(-x_side,  y_side, -z_side);

    glNormal3f(0.0f, -1.0f, 0.0f);
    glTexCoord2fv(texCoord[0]); glVertex3f( x_side, -y_side,  z_side);
    glTexCoord2fv(texCoord[1]); glVertex3f(-x_side, -y_side,  z_side);
    glTexCoord2fv(texCoord[2]); glVertex3f(-x_side, -y_side, -z_side);
    glTexCoord2fv(texCoord[3]); glVertex3f( x_side, -y_side, -z_side);

    glNormal3f(1.0f, 0.0f, 0.0f);
    glTexCoord2fv(texCoord[0]); glVertex3f( x_side, -y_side,  z_side);
    glTexCoord2fv(texCoord[1]); glVertex3f( x_side, -y_side, -z_side);
    glTexCoord2fv(texCoord[2]); glVertex3f( x_side,  y_side, -z_side);
    glTexCoord2fv(texCoord[3]); glVertex3f( x_side,  y_side,  z_side);
    ::glEnd();

    ::glPopAttrib();
  
    ::glEndList();
  } else
    ::glCallList(m_nDisplayList);

#ifdef _DEBUG
  if (pRenderer->CheckGLError("BoxProxy:"))
    TRACE("BoxProxy::Render() [%08x]: ", this);
#endif

  return TRUE;
}

// Traverse scene graph, checking for intersection

/* virtual */ BOOL
BoxProxy::Select(OpenGLVisualRenderer* pRenderer, DWORD dwTime, unsigned int& depth)
{
  GLuint selectBuffer[4];

  ::glSelectBuffer(4, selectBuffer);
  ::glRenderMode(GL_SELECT);

  Render(pRenderer, dwTime);

  if (::glRenderMode(GL_RENDER) > 0)
    if (depth > selectBuffer[1]) {
      depth = selectBuffer[1];

      return TRUE;
    }

#ifdef _DEBUG
  if (pRenderer->CheckGLError("BoxProxy:"))
    TRACE("BoxProxy::Render() [%08x]: ", this);
#endif

  return FALSE;
}
