// -*- c++ -*-

// This software module was originally developed by Edward Cooke (Teltec
// Ireland) and Karl Anders ygard (Telenor R&D) in the course of
// development of the MPEG-4 Systems standard (ISO/IEC 14496-1).  This
// software module is an implementation of a part of one or more MPEG-4
// Systems tools as specified by the MPEG-4 Systems standard.  ISO/IEC
// gives users of the MPEG-4 Systems standard free license to this software
// module or modifications thereof for use in hardware or software products
// claiming conformance to the MPEG-4 Systems standard.  Those intending to
// use this software module in hardware or software products are advised
// that its use may infringe existing patents.  The original developer of
// this software module and his/her company, the subsequent editors and
// their companies, and ISO/IEC have no liability for use of this software
// module or modifications thereof in an implementation.  Copyright is not
// released for non MPEG-4 Systems conforming products.  The original
// developer retains full right to use the code for his/her own purpose,
// assign or donate the code to a third party and to inhibit third parties
// from using the code for non MPEG-4 Systems conforming products.  This
// copyright notice must be included in all copies or derivative works.
//
// For more information or to receive an up to date version of this module,
// mail karlo@nta.no or check <http://televr.fou.telenor.no/compositor/>.
//
// Copyright 1998.

#include "OpenGl/OpenGl.h"
#include "ConeProxy.h"
#include "opengl/base/OpenGLVisualRenderer.h"


// Render object description

/* virtual */ void
ConeProxy::RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD /* dwTime */)
{
  GLfloat color[] = { 0.0f, 0.0f, 1.0f, 0.75f};

  //satish changed this line for compilation purposes
        //have to be fixed afterwards

  //::glColor4fv(color);
  glColor4fv(color);

  RenderDescriptionBox(pRenderer, "Cone");
}

// Render object

/* virtual */ BOOL
ConeProxy::Render(OpenGLVisualRenderer* pRenderer, DWORD)
{
  float height = GetHeight();
  float bottomradius = GetBottomRadius();

  if (!m_nDisplayList)
  {
    m_nDisplayList = ::glGenLists(1);

    ::glNewList(m_nDisplayList, GL_COMPILE_AND_EXECUTE);

    ::glPushAttrib(GL_POLYGON_BIT);
    ::glPushMatrix();

    ::glEnable(GL_CULL_FACE);

    ::glRotatef(-90.0f, 1.0f, 0.0f, 0.0f);
    ::glTranslatef(0.0f, 0.0f, -height / 2.0f);
    
    if (pRenderer->GetCurrentTexture() != NULL)
    {
      ::gluQuadricTexture(m_pGluQuadric, GL_TRUE);
    }
    else
    {
      ::gluQuadricTexture(m_pGluQuadric, GL_FALSE);
    }

    if (GetSide())
    {
      TRACE("[Cone] Side \n");
      ::gluCylinder(m_pGluQuadric, bottomradius, 0, height, 32, 1);
    }

    ::glRotatef(180.0f, 1.0f, 0.0f, 0.0f);

    if (GetBottom())
    {
      TRACE("[Cone] Bottom \n");
      ::gluDisk(m_pGluQuadric, 0, bottomradius, 32, 1);
    }

    ::glPopMatrix();
    ::glPopAttrib();

    ::glEndList();
  }
  else
  {
    ::glCallList(m_nDisplayList);
  }

  TRACE("[Cone] DisplayList %4d \n", m_nDisplayList);

  return TRUE;
}

// Traverse scene graph, checking for intersection

/* virtual */ BOOL
ConeProxy::Select(OpenGLVisualRenderer* pRenderer, DWORD dwTime, unsigned int& depth)
{
  GLuint selectBuffer[4];

  ::glSelectBuffer(4, selectBuffer);
  ::glRenderMode(GL_SELECT);

  Render(pRenderer, dwTime);

  if (::glRenderMode(GL_RENDER) > 0)
    if (depth >= selectBuffer[1]) {
      depth = selectBuffer[1];

      return TRUE;
    }

#ifdef _DEBUG
  if (pRenderer->CheckGLError("ConeProxy:"))
    TRACE("ConeProxy::Render() [%08x]: ", this);
#endif

  return FALSE;
}
