// -*- c++ -*-

// This software module was originally developed by Edward Cooke (Teltec
// Ireland) and Karl Anders ygard (Telenor R&D) in the course of
// development of the MPEG-4 Systems standard (ISO/IEC 14496-1).  This
// software module is an implementation of a part of one or more MPEG-4
// Systems tools as specified by the MPEG-4 Systems standard.  ISO/IEC
// gives users of the MPEG-4 Systems standard free license to this software
// module or modifications thereof for use in hardware or software products
// claiming conformance to the MPEG-4 Systems standard.  Those intending to
// use this software module in hardware or software products are advised
// that its use may infringe existing patents.  The original developer of
// this software module and his/her company, the subsequent editors and
// their companies, and ISO/IEC have no liability for use of this software
// module or modifications thereof in an implementation.  Copyright is not
// released for non MPEG-4 Systems conforming products.  The original
// developer retains full right to use the code for his/her own purpose,
// assign or donate the code to a third party and to inhibit third parties
// from using the code for non MPEG-4 Systems conforming products.  This
// copyright notice must be included in all copies or derivative works.
//
// For more information or to receive an up to date version of this module,
// mail karlo@nta.no or check <http://televr.fou.telenor.no/compositor/>.
//
// Copyright 1998.

#ifndef __CONEPROXY_H__
#define __CONEPROXY_H__

#include "GeometryProxy.h"

#include <GL/glu.h>

// class Cone;
//#include "Nodes.h"

/// Cone renderer node

class ConeProxy
  : public GeometryProxy
{
public:

  /// Constructor

			ConeProxy(Cone* pCone);

  /// Destructor

  virtual		~ConeProxy();

  /// Render object

  virtual BOOL		Render(OpenGLVisualRenderer* pRenderer, DWORD dwTime);

  /// Traverse scene graph, checking for intersection

  virtual BOOL		Select(OpenGLVisualRenderer* pRenderer, DWORD dwTime, unsigned int& depth);

  /// Render object description

  virtual void		RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD dwTime);

private:

  // GLU quadric

  GLUquadricObj*	m_pGluQuadric;

  // Stefano Battista
  // moved methods from Cone to ConeProxy
public:
  inline Cone * GetCone() const
  { return (Cone *) GetMediaObject(); }
  /// Get whether bottom is drawn
  inline BOOL GetBottom() const
  { return (Bool) GetCone()->bottom; }
  /// Get bottom radius of Cone
  inline float GetBottomRadius() const
  { return GetCone()->bottomRadius; }
  /// Get height of Cone
  inline float GetHeight() const
  { return GetCone()->height; }
  /// Get whether side is drawn
  inline BOOL GetSide() const
  { return (Bool) GetCone()->side; }
};

#include "ConeProxy.i"

#endif // __CONEPROXY_H__

