// -*- c++ -*-

// This software module was originally developed by Karl Anders ygard
// (Telenor R&D) in the course of development of the MPEG-4 Systems
// standard (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// For more information or to receive an up to date version of this module,
// mail karlo@nta.no or check <http://televr.fou.telenor.no/compositor/>.
//
// Copyright 1997, 1998.

#ifndef __CYLINDERPROXY_H__
#define __CYLINDERPROXY_H__

#include "GeometryProxy.h"

#include <GL/glu.h>

// class Cylinder;
//#include "Nodes.h"

/// Cylinder renderer node

class CylinderProxy
  : public GeometryProxy
{
public:

  /// Constructor

                        CylinderProxy(Cylinder* pCylinder);

  /// Destructor

                        ~CylinderProxy();

  /// Render object

  virtual BOOL		Render(OpenGLVisualRenderer* pRenderer, DWORD dwTime);

  /// Traverse scene graph, checking for intersection

  virtual BOOL		Select(OpenGLVisualRenderer* pRenderer, DWORD dwTime, unsigned int& depth);

  /// Render object description

  virtual void		RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD dwTime);

private:

  // GLU quadric
  GLUquadricObj*	m_pGluQuadric;

  // Stefano Battista
  // moved methods from Cylinder to CylinderProxy
public:
  inline Cylinder * GetCylinder() const
  { return (Cylinder *) GetMediaObject(); }
  /// Get whether bottom is drawn
  inline BOOL GetBottom() const
  { return (Bool) GetCylinder()->bottom; }
  /// Set whether bottom is drawn
  inline void SetBottom(bool bBottom)
  { GetCylinder()->bottom = bBottom; }
  /// Set radius of cylinder
  inline void SetRadius(float fRadius)
  { GetCylinder()->radius = fRadius; }
  /// Get radius of cylinder
  inline float GetRadius() const
  { return GetCylinder()->radius; }
  /// Set height of cylinder
  inline void SetHeight(float fHeight)
  { GetCylinder()->height = fHeight; }
  /// Get height of cylinder
  inline float GetHeight() const
  { return GetCylinder()->height; }
  /// Get whether side is drawn
  inline BOOL GetSide() const
  { return (Bool) GetCylinder()->side; }
  /// Set whether side is drawn
  inline void SetSide(bool bSide)
  { GetCylinder()->side = bSide; }
  /// Get whether top is drawn
  inline BOOL GetTop() const
  { return (Bool) GetCylinder()->top; }
  /// Set whether top is drawn
  inline void SetTop(bool bTop)
  { GetCylinder()->top = bTop; }
  
};

#include "CylinderProxy.i"

#endif // __CYLINDERPROXY_H__

