// -*- c++ -*-

// This software module was originally developed by Karl Anders ygard
// (Telenor R&D) in the course of development of the MPEG-4 Systems
// standard (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// For more information or to receive an up to date version of this module,
// mail karlo@nta.no or check <http://televr.fou.telenor.no/compositor/>.
//
// Copyright 1998.

#include "OpenGl/OpenGl.h"
#include "ElevationGridProxy.h"
#include "opengl/base/OpenGLVisualRenderer.h"

// traverse scene graph, checking for intersection

/* virtual */ void
ElevationGridProxy::Select(
    OpenGLVisualRenderer * /* pRenderer */, 
    DWORD /* dwTime */,
    MediaObjectProxy *& /* intersectingObject */, 
    unsigned int & /* depth */)
{
}

// Traverse scene graph, checking for activation of sensors

/* virtual */ void
ElevationGridProxy::Select(
    const Matrix & /* rTransformation */, 
    const CullFlag& /* rCullFlag */,
		OpenGLVisualRenderer * /* pRenderer */, 
    DWORD /* dwTime */,
    MediaObjectProxy *& /* intersectingObject */,
		ZArray<MediaObjectProxy *> & /* activatedSensors */, 
    unsigned int & /* depth */,
    const ZArray<MediaObjectProxy *> & /* localSensors */)
{
  /* 
  Shape* pShape = (Shape*) GetMediaObject();
  Geometry* pGeometry = pShape->GetGeometry();
    
  if (pGeometry)
    if (((GeometryProxy*) pGeometry->GetRenderingObject())->Select(pRenderer, dwTime, depth)) {
      intersectingObject = this;
      activatedSensors = localSensors;
    }
  */
}

// Render object

/* virtual */ BOOL
ElevationGridProxy::Render(
    const Matrix & /* rTransformation */, 
    const CullFlag & /* rCullFlag */,
		OpenGLVisualRenderer * /* pRenderer */, 
    DWORD /* dwTime */)
{
  /*   
  ElevationGrid* pElevationGrid = (ElevationGrid*) GetMediaObject();

  const MF<int>& coordIndex = pElevationGrid->GetCoordIndex();
  const Coordinate* pCoord = pElevationGrid->GetCoord();
  const MF<int>& colorIndex = ((ElevationGrid*) GetMediaObject())->GetColorIndex();
  const Color* pColor = pElevationGrid->GetColor();
  const MF<int>& normalIndex = pElevationGrid->GetNormalIndex();
  const Normal* pNormal = pElevationGrid->GetNormal();
  const MF<int>& texCoordIndex = pElevationGrid->GetTexCoordIndex();
  const TextureCoordinate* pTexCoord = pElevationGrid->GetTexCoord();
  BOOL colorPerVertex = pElevationGrid->GetColorPerVertex();
  BOOL normalPerVertex = pElevationGrid->GetNormalPerVertex();

  const MF<Vec3f>& coordarray = pCoord->GetPoint();
  const MF<Color3f>& colorarray = pColor->GetColor();
  const MF<Vec3f>& normalarray = pNormal->GetVector();
  const MF<Vec2f>& texCoordarray = pTexCoord->GetPoint();

  ::glPushAttrib(GL_POLYGON_BIT | GL_CURRENT_BIT);

  if (pElevationGrid->GetSolid())
    ::glEnable(GL_CULL_FACE);
  
  if (!pElevationGrid->GetCCW())
    ::glFrontFace(GL_CW);

  if (!pElevationGrid->GetNormal()) {
    COMPLAIN("Automatically generated normals are not supported yet\n");
    ::glDisable(GL_LIGHTING);
  }

  if (colorIndex.GetLength() == 0 && colorPerVertex &&
      normalIndex.GetLength() == 0 && normalPerVertex &&
      texCoordIndex.GetLength() == 0) {
    // special case fast path 

    int coords;

    if (coordIndex.GetLength() == 0)
      coords = coordarray.GetLength();
    else
      coords = coordIndex.GetLength();

    if (long(pElevationGrid->GetUpdateTime()) > m_nTimeStamp ||
	long(pCoord && (pCoord->GetUpdateTime() > m_nTimeStamp)) ||
	long(pColor && (pColor->GetUpdateTime() > m_nTimeStamp)) ||
	long(pNormal && (pNormal->GetUpdateTime() > m_nTimeStamp)) ||
	long(pTexCoord && (pTexCoord->GetUpdateTime() > m_nTimeStamp))) {
      if (!m_nDisplayList)
	m_nDisplayList = ::glGenLists(1);

      m_nTimeStamp = long(dwTime);
    
      ::glNewList(m_nDisplayList, GL_COMPILE_AND_EXECUTE);

#if !defined(GL_VERSION_1_1)
      ::glVertexPointerEXT(3, GL_FLOAT, sizeof(Vec3f), coords, &coordarray[0].x);
      ::glEnable(GL_VERTEX_ARRAY_EXT);
      if (pColor) {
	::glColorPointerEXT(3, GL_FLOAT, sizeof(Color3f), coords, &colorarray[0].r);
	::glEnable(GL_COLOR_ARRAY_EXT);
      }
      if (pNormal) {
	::glNormalPointerEXT(GL_FLOAT, sizeof(Vec3f), coords, &normalarray[0].x);
	::glEnable(GL_NORMAL_ARRAY_EXT);
      }
      if (pTexCoord) {
	::glTexCoordPointerEXT(2, GL_FLOAT, sizeof(Vec2f), coords, &texCoordarray[0].x);
	::glEnable(GL_TEXTURE_COORD_ARRAY_EXT);
      }

      if (coordIndex.GetLength() == 0)
	::glDrawArraysEXT(GL_POLYGON, 0, coords);
      else {
	::glBegin(GL_POLYGON);

	for (int i = 0; i < coords; ++i) {
	  int j = coordIndex[i];

	  if (j != -1)
	    ::glArrayElementEXT(j);
	  else {
	    ::glEnd();
	    ::glBegin(GL_POLYGON);
	  }
	}

	::glEnd();
      }

      ::glDisable(GL_VERTEX_ARRAY_EXT);
      ::glDisable(GL_COLOR_ARRAY_EXT);
      ::glDisable(GL_NORMAL_ARRAY_EXT);
      ::glDisable(GL_TEXTURE_COORD_ARRAY_EXT);
#else
      ::glVertexPointer(3, GL_FLOAT, sizeof(Vec3f), &coordarray[0].x);
      ::glEnableClientState(GL_VERTEX_ARRAY);
      if (pColor) {
	::glColorPointer(3, GL_FLOAT, sizeof(Color), &colorarray[0].r);
	::glEnableClientState(GL_COLOR_ARRAY);
      }
      if (pNormal) {
	::glNormalPointer(GL_FLOAT, sizeof(Vec3f), &normalarray[0].x);
	::glEnableClientState(GL_NORMAL_ARRAY);
      }
      if (pTexCoord) {
	::glTexCoordPointer(2, GL_FLOAT, sizeof(Vec2f), &texCoordarray[0].x);
	::glEnableClientState(GL_TEXTURE_COORD_ARRAY);
      }

      if (coordIndex.GetLength() == 0)
	::glDrawArrays(GL_POLYGON, 0, coords);
      else {
	::glBegin(GL_POLYGON);

	for (int i = 0; i < coords; ++i) {
	  int j = coordIndex[i];

	  if (j != -1)
	    ::glArrayElement(j);
	  else {
	    ::glEnd();
	    ::glBegin(GL_POLYGON);
	  }
	}

	::glEnd();
      }

      ::glDisableClientState(GL_VERTEX_ARRAY);
      ::glDisableClientState(GL_COLOR_ARRAY);
      ::glDisableClientState(GL_NORMAL_ARRAY);
      ::glDisableClientState(GL_TEXTURE_COORD_ARRAY);
#endif

      ::glEndList();
    } else
      ::glCallList(m_nDisplayList);
  } else
    / The following is a little obscure; the reason why the
       inlinedRenderLoop() method isn't started right away is that passing
       inlinedRenderLoop() static boolean arguments allows the compiler to
       evalute parts of the code at compile time and inline the code much
       better.  This of course leads to much larger code; and the compiler
       probably will not inline all of the code.  Still, the code should be
       more efficient.  Doing arc profiling would probably give the
       compiler a chance to do a better job. /

    if (coordIndex.GetLength() != 0)
      if (colorIndex.GetLength() != 0)
	if (colorPerVertex)
	  if (pRenderer->GetCurrentTexture() && pTexCoord)
	    inlinedRenderLoop(TRUE,
			      TRUE, TRUE,
			      normalIndex.GetLength() != 0, normalPerVertex,
			      texCoordIndex.GetLength() != 0, pTexCoord,
			      dwTime);
	  else
	    inlinedRenderLoop(TRUE,
			      TRUE, TRUE,
			      normalIndex.GetLength() != 0, normalPerVertex,
			      FALSE, NULL,
			      dwTime);
	else
	  if (pRenderer->GetCurrentTexture() && pTexCoord)
	    inlinedRenderLoop(TRUE,
			      TRUE, FALSE,
			      normalIndex.GetLength() != 0, normalPerVertex,
			      texCoordIndex.GetLength() != 0, pTexCoord,
			      dwTime);
	  else
	    inlinedRenderLoop(TRUE,
			      TRUE, FALSE,
			      normalIndex.GetLength() != 0, normalPerVertex,
			      FALSE, NULL,
			      dwTime);
      else
	if (colorPerVertex)
	  if (pRenderer->GetCurrentTexture() && pTexCoord)
	    inlinedRenderLoop(TRUE,
			      FALSE, TRUE,
			      normalIndex.GetLength() != 0, normalPerVertex,
			      texCoordIndex.GetLength() != 0, pTexCoord,
			      dwTime);
	  else
	    inlinedRenderLoop(TRUE,
			      FALSE, TRUE,
			      normalIndex.GetLength() != 0, normalPerVertex,
			      FALSE, NULL,
			      dwTime);
	else
	  if (pRenderer->GetCurrentTexture() && pTexCoord)
	    inlinedRenderLoop(TRUE,
			      FALSE, FALSE,
			      normalIndex.GetLength() != 0, normalPerVertex,
			      texCoordIndex.GetLength() != 0, pTexCoord,
			      dwTime);
	  else
	    inlinedRenderLoop(TRUE,
			      FALSE, FALSE,
			      normalIndex.GetLength() != 0, normalPerVertex,
			      FALSE, NULL,
			      dwTime);
    else
      if (colorIndex.GetLength() != 0)
	if (colorPerVertex)
	  if (pRenderer->GetCurrentTexture() && pTexCoord)
	    inlinedRenderLoop(FALSE,
			      TRUE, TRUE,
			      normalIndex.GetLength() != 0, normalPerVertex,
			      texCoordIndex.GetLength() != 0, pTexCoord,
			      dwTime);
	  else
	    inlinedRenderLoop(FALSE,
			      TRUE, TRUE,
			      normalIndex.GetLength() != 0, normalPerVertex,
			      FALSE, NULL,
			      dwTime);
	else
	  if (pRenderer->GetCurrentTexture() && pTexCoord)
	    inlinedRenderLoop(FALSE,
			      TRUE, FALSE,
			      normalIndex.GetLength() != 0, normalPerVertex,
			      texCoordIndex.GetLength() != 0, pTexCoord,
			      dwTime);
	  else
	    inlinedRenderLoop(FALSE,
			      TRUE, FALSE,
			      normalIndex.GetLength() != 0, normalPerVertex,
			      FALSE, NULL,
			      dwTime);
      else
	if (colorPerVertex)
	  if (pRenderer->GetCurrentTexture() && pTexCoord)
	    inlinedRenderLoop(FALSE,
			      FALSE, TRUE,
			      normalIndex.GetLength() != 0, normalPerVertex,
			      texCoordIndex.GetLength() != 0, pTexCoord,
			      dwTime);
	  else
	    inlinedRenderLoop(FALSE,
			      FALSE, TRUE,
			      normalIndex.GetLength() != 0, normalPerVertex,
			      FALSE, NULL,
			      dwTime);
	else
	  if (pRenderer->GetCurrentTexture() && pTexCoord)
	    inlinedRenderLoop(FALSE,
			      FALSE, FALSE,
			      normalIndex.GetLength() != 0, normalPerVertex,
			      texCoordIndex.GetLength() != 0, pTexCoord,
			      dwTime);
	  else
	    inlinedRenderLoop(FALSE,
			      FALSE, FALSE,
			      normalIndex.GetLength() != 0, normalPerVertex,
			      FALSE, NULL,
			      dwTime);

  ::glPopAttrib();
  
#ifdef _DEBUG
  if (pRenderer->CheckGLError("ElevationGridProxy:"))
    COMPLAIN("ElevationGridProxy::Render() [%08x]: ", this);
#endif
*/

  return TRUE;
}

// Render object description

/* virtual */ void
ElevationGridProxy::RenderDescription(
    OpenGLVisualRenderer* pRenderer, 
    DWORD /* dwTime */)
{
  RenderDescriptionBox(pRenderer, "ElevationGrid");
}
