// -*- c++ -*-

// This software module was originally developed by Karl Anders ygard
// (Telenor R&D) in the course of development of the MPEG-4 Systems
// standard (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.	 The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.	 This copyright notice must be included in
// all copies or derivative works.
//
// For more information or to receive an up to date version of this module,
// mail karlo@nta.no or check <http://televr.fou.telenor.no/compositor/>.
//
// Copyright 1998.

#ifndef __ELEVATIONGRIDPROXY_H__
#define __ELEVATIONGRIDPROXY_H__

#include "opengl/base/Node3DProxy.h"


// class ElevationGrid;
// class TextureCoordinate;
//#include "Nodes.h"

/** Elevation grid node.

    The ElevationGridProxy class specifies a three dimensional mesh, located in
    origo of the local coordinate system. */

class ElevationGridProxy
  : public Node3DProxy
{
public:

  /// Constructor

			ElevationGridProxy(ElevationGrid* pElevationGrid);

  /// Render object

  virtual BOOL  	Render(const Matrix& rTransformation, const CullFlag& rCullFlag,
			       OpenGLVisualRenderer* pRenderer, DWORD dwTime);

  /// Traverse scene graph, checking for intersection

  virtual void		Select(
      OpenGLVisualRenderer* pRenderer, 
      DWORD dwTime,
			MediaObjectProxy *& intersectingObject, 
      unsigned int& depth);

   /// Traverse scene graph, checking for activation of sensors

  virtual void          Select(
      const Matrix& rTransformation, 
      const CullFlag& rCullFlag,
			OpenGLVisualRenderer* pRenderer, 
      DWORD dwTime,
			MediaObjectProxy *& intersectingObject,
			ZArray<MediaObjectProxy *>& activatedSensors, 
      unsigned int& depth,
			const ZArray<MediaObjectProxy *>& localSensors);

  /// Render object description

  virtual void		RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD dwTime);

private:

  // render object

  void			inlinedRenderLoop(BOOL bUseCoordIndex,
					  BOOL bUseColorIndex, BOOL bColorPerVertex,
					  BOOL bUseNormalIndex, BOOL bNormalPerVertex,
					  BOOL bUseTexCoordIndex, /* const */ TextureCoordinate* pTexCoord,
					  DWORD dwTime);

  // OpenGL display list

  GLuint		m_nDisplayList;

  // Time stamp of display list

  long			m_nTimeStamp;

  // Stefano Battista
  // moved methods from ElevationGrid to Proxy
public:
  /// Get pointer to associated MediaObject
  inline ElevationGrid * GetElevationGrid()
  { return (ElevationGrid *) GetMediaObject(); }

  //ML - removed this function
  // (caused compilation error and it isn't used)
  /// Set colors
  //inline void SetColor(Color* pColor)
  //{ GetElevationGrid()->color = pColor;
  //  GetElevationGrid()->SetUpdateTime(); }

  /// Get colors
  // SOLARIS PORTING - To core version 5.9rc9 - Appropriate Typecast
  inline Color * GetColor()
  { return (Color*)(MediaObject*)(GetElevationGrid()->color); }

//ML - removed this function
//  /// Set normals
//  inline void SetNormal(Normal* pNormal)
//  { GetElevationGrid()->normal = pNormal;
//    GetElevationGrid()->SetUpdateTime(); }

  /// Get normals
  inline Normal * GetNormal()
  { return GetElevationGrid()->normal; }

//ML - removed this function
//  /// Set texture coordinates
//  inline void SetTexCoord(TextureCoordinate* pTexCoord)
//  { GetElevationGrid()->texCoord = pTexCoord;
//    GetElevationGrid()->SetUpdateTime(); }

  /// Get texture coordinates
  inline SFTextureCoordinateNode& GetTexCoord()
  { return GetElevationGrid()->texCoord; }
  /// Set color per vertex
  inline void SetColorPerVertex(bool bColorPerVertex)
  { GetElevationGrid()->colorPerVertex = bColorPerVertex;
    GetElevationGrid()->SetUpdateTime(); }
  /// Get color per vertex
  inline BOOL GetColorPerVertex() 
  { return (Bool) (GetElevationGrid()->colorPerVertex); }
  /// Set normal per vertex
  inline void SetNormalPerVertex(bool bNormalPerVertex)
  { GetElevationGrid()->normalPerVertex = bNormalPerVertex;
    GetElevationGrid()->SetUpdateTime(); }
  /// Get normal per vertex
  inline BOOL GetNormalPerVertex() 
  { return (BOOL) (Bool) GetElevationGrid()->normalPerVertex; }
  /// Set whether object is solid
  inline void SetSolid(bool bSolid)
  { GetElevationGrid()->solid = bSolid;
    GetElevationGrid()->SetUpdateTime(); }
  /// Get whether object is solid
  inline BOOL GetSolid() 
  { return (Bool) GetElevationGrid()->solid; }
  /// Set whether polygons are counter-clockwise
  inline void SetCCW(bool bCCW)
  { GetElevationGrid()->ccw = bCCW;
    GetElevationGrid()->SetUpdateTime(); }
  /// Get whether polygons are counter-clockwise
  inline BOOL GetCCW() 
  { return (Bool) GetElevationGrid()->ccw; }

};

#include "ElevationGridProxy.i"

#endif // __ELEVATIONGRIDPROXY_H__
