// -*- c++ -*-

// This software module was originally developed by Karl Anders ygard
// (Telenor R&D) in the course of development of the MPEG-4 Systems
// standard (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// For more information or to receive an up to date version of this module,
// mail karlo@nta.no or check <http://televr.fou.telenor.no/compositor/>.
//
// Copyright 1998.

// Stefano Battista 98/11/13


// Constructor

inline
ElevationGridProxy::ElevationGridProxy(ElevationGrid* pElevationGrid)
: Node3DProxy(pElevationGrid),
m_nTimeStamp(-1)
{
}

// Render object

/* This is the actual method that does the work.  
It is inlined, thus removing most of the if statements. */

inline void
ElevationGridProxy::inlinedRenderLoop(
    BOOL /* bUseCoordIndex */,
    BOOL /* bUseColorIndex */, 
    BOOL /* bColorPerVertex */,
    BOOL /* bUseNormalIndex */, 
    BOOL /* bNormalPerVertex */,
    BOOL /* bUseTexCoordIndex */, 
    /* const */ TextureCoordinate * /* pTexCoord */,
    DWORD /* dwTime */)
{
/***

  ElevationGrid * pElevationGrid = GetElevationGrid();
  
  // /& const &/ Coordinate* pCoord = /& pElevationGrid-> &/ GetCoord();
  /& const &/ Color* pColor = /& pElevationGrid-> &/ GetColor();
  /& const &/ Normal* pNormal = /& pElevationGrid-> &/ GetNormal();
  
  /&**
  if (long(pElevationGrid->GetUpdateTime()) > m_nTimeStamp ||
  // long(pCoord && (pCoord->GetUpdateTime() > m_nTimeStamp)) ||
  long(pColor && (pColor->GetUpdateTime() > m_nTimeStamp)) ||
  long(pNormal && (pNormal->GetUpdateTime() > m_nTimeStamp)) ||
  long(pTexCoord && (pTexCoord->GetUpdateTime() > m_nTimeStamp))) {
  **&/
  
  if (long(pElevationGrid->GetUpdateTime()) > m_nTimeStamp ||
    // (pCoord && long(pCoord->GetUpdateTime()) > m_nTimeStamp) ||
    (pColor && long(pColor->GetUpdateTime()) > m_nTimeStamp) ||
    (pNormal && long(pNormal->GetUpdateTime()) > m_nTimeStamp) ||
    (pTexCoord && long(pTexCoord->GetUpdateTime()) > m_nTimeStamp))
  {
    if (!m_nDisplayList)
      m_nDisplayList = ::glGenLists(1);
    
    m_nTimeStamp = (long) dwTime;
    
    ::glNewList(m_nDisplayList, GL_COMPILE_AND_EXECUTE);
    
    /&**
    // const MF<Vec3f>& coordarray = pCoord->GetPoint();
    const MF<Color3f>& colorarray = pColor->GetColor();
    const MF<Vec3f>& normalarray = pNormal->GetVector();
    const MF<Vec2f>& texCoordarray = pTexCoord->GetPoint();
    **&/
    // const MF<Vec3f>& coordarray = pCoord->GetPoint();
    const MF<Color3f>& colorarray = pColor->color;
    const MF<Vec3f>& normalarray = pNormal->vector;
    const MF<Vec2f>& texCoordarray = pTexCoord->point;
    
    int currentColor = 0;
    int currentNormal = 0;
    
    if (bUseCoordIndex)
    {

      BOOL begin = FALSE;
      // SB @@@@ "coordIndex" is NOT a field of ElevatioGrid !!!! >>>>
      int coords = coordIndex.GetLength();
      
      for (int i = 0; i < coords; ++i)
      {
        if (coordIndex[i] == -1)
        {
          if (begin)
          {
            ::glEnd();
            begin = FALSE;
          }
        } 
        else 
        {
          const Vec3f& vertex = coordarray[coordIndex[i]];
          
          if (bColorPerVertex && pColor)
          {
            const Color3f& col = colorarray[bUseColorIndex ? colorIndex[i] : coordIndex[i]];
            ::glColor3f(col.r, col.g, col.b);
          }
          
          if (bNormalPerVertex && pNormal)
          {
            const Vec3f& norm = normalarray[bUseNormalIndex ? normalIndex[i] : coordIndex[i]];
            ::glNormal3f(norm.x, norm.y, norm.z);
          }
          
          if (pTexCoord)
          {
            const Vec2f& texCoord = texCoordarray[bUseTexCoordIndex ? texCoordIndex[i] : coordIndex[i]];
            ::glTexCoord2f(texCoord.x, texCoord.y);
          }
          
          if (!begin)
          {
            ::glBegin(GL_POLYGON);
            if (!bColorPerVertex && pColor)
            {
              const Color3f& col = colorarray[bUseColorIndex ? colorIndex[currentColor++] : currentColor++];
              ::glColor3f(col.r, col.g, col.b);
            }
            
            if (!bNormalPerVertex && pNormal)
            {
              const Vec3f& norm = normalarray[bUseNormalIndex ? normalIndex[currentNormal++] : currentNormal++];
              ::glNormal3f(norm.x, norm.y, norm.z);
            }
            
            begin = TRUE;
          }
          
          ::glVertex3f(vertex.x, vertex.y, vertex.z);
        }
      }
      
      if (begin)
      {
        ::glEnd();
      }
    }
    else // ! bUseCoordIndex
    {
      int coords = texCoordarray.GetLength();
      
      ::glBegin(GL_POLYGON);
      
      for (int i = 0; i < coords; ++i) {
        const Vec3f& vertex = coordarray[i];
        
        if (bColorPerVertex && pColor) {
          const Color3f& col = colorarray[bUseColorIndex ? colorIndex[i] : i];
          
          ::glColor3f(col.r, col.g, col.b);
        }
        
        if (bNormalPerVertex && pNormal) {
          const Vec3f& norm = normalarray[bUseNormalIndex ? normalIndex[i] : i];
          
          ::glNormal3f(norm.x, norm.y, norm.z);
        }
        
        if (pTexCoord) {
          const Vec2f& texCoord = texCoordarray[bUseTexCoordIndex ? texCoordIndex[i] : i];
          
          ::glTexCoord2f(texCoord.x, texCoord.y);
        }
        
        if (!bColorPerVertex && pColor) {
          const Color3f& col = colorarray[bUseColorIndex ? colorIndex[currentColor++] : currentColor++];
          
          ::glColor3f(col.r, col.g, col.b);
        }
        
        if (!bNormalPerVertex && pNormal) {
          const Vec3f& norm = normalarray[bUseNormalIndex ? normalIndex[currentNormal++] : currentNormal++];
          
          ::glNormal3f(norm.x, norm.y, norm.z);
        }
        
        ::glVertex3f(vertex.x, vertex.y, vertex.z);
      }
      
      ::glEnd();
    }
    
    ::glEndList();
  }
  else
  {
    ::glCallList(m_nDisplayList);
  }

***/
}
