// -*- c++ -*-

// This software module was originally developed by Karl Anders ygard
// (Telenor R&D) in the course of development of the MPEG-4 Systems
// standard (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.	 The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.	 This copyright notice must be included in
// all copies or derivative works.
//
// For more information or to receive an up to date version of this module,
// mail karlo@nta.no or check <http://televr.fou.telenor.no/compositor/>.
//
// Copyright 1998.

#ifndef __EXTRUSIONPROXY_H__
#define __EXTRUSIONPROXY_H__

#include "opengl/base/Node3DProxy.h"
//satish added this line
#include "GeometryProxy.h"

#include <GL/gl.h>

class Extrusion;
class TextureCoordinate;

/** Extrusion node.

    The ExtrusionProxy class specifies a three dimensional mesh, located in
    origo of the local coordinate system. */

class ExtrusionProxy
  : public GeometryProxy
{
public:

  /// Constructor

			ExtrusionProxy(Extrusion* pExtrusion);

  /// Render object

  virtual BOOL  	Render(const Matrix& rTransformation, const CullFlag& rCullFlag,
			       OpenGLVisualRenderer* pRenderer, DWORD dwTime);

  /// Traverse scene graph, checking for intersection

  virtual void		Select(
      OpenGLVisualRenderer* pRenderer, 
      DWORD dwTime,
			MediaObjectProxy *& intersectingObject, 
      unsigned int& depth);

   /// Traverse scene graph, checking for activation of sensors

  virtual void          Select(
      const Matrix& rTransformation, 
      const CullFlag& rCullFlag,
			OpenGLVisualRenderer* pRenderer, 
      DWORD dwTime,
			MediaObjectProxy *& intersectingObject,
			ZArray<MediaObjectProxy *>& activatedSensors, 
      unsigned int& depth,
			const ZArray<MediaObjectProxy *>& localSensors);

  /// Render object description

  virtual void		RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD dwTime);

private:

  // render object

  void			inlinedRenderLoop(BOOL bUseCoordIndex,
					  BOOL bUseColorIndex, BOOL bColorPerVertex,
					  BOOL bUseNormalIndex, BOOL bNormalPerVertex,
					  BOOL bUseTexCoordIndex, const TextureCoordinate* pTexCoord,
					  DWORD dwTime);

  // OpenGL display list

  GLuint		m_nDisplayList;

  // Time stamp of display list

  long			m_nTimeStamp;
};

#include "ExtrusionProxy.i"

#endif // __EXTRUSIONPROXY_H__
