// This software module was originally developed by Kevin Barron (KPN Research), 
// Mike Coleman (Five Bats Research), Edward Cooke (Teltec Ireland), In-Dai Ha 
// (Hyundai Electronics Industries, Co.), Hae-Kwan Kim (Hyundai Electronics 
// Industries, Co.), Matthew Leditschke (CSELT), Zvi Lifshitz (VDOnet Corp.), 
// Karl Anders ygard (Telenor R&D), in the course of development of the MPEG-4 
// Systems standard (ISO/IEC 14496-1).  This software module is an 
// implementation of a part of one or more MPEG-4 Systems tools as specified by 
// the MPEG-4 Systems standard.  ISO/IEC gives users of the MPEG-4 Systems 
// standard free license to this software module or modifications thereof for 
// use in hardware or software products claiming conformance to the MPEG-4 
// Systems standard.  Those intending to use this software module in hardware or 
// software products are advised that its use may infringe existing patents.  
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use 
// of this software module or modifications thereof in an implementation.  
// Copyright is not released for non MPEG-4 Systems conforming products.  The 
// original developer retains full right to use the code for his/her own 
// purpose, assign or donate the code to a third party and to inhibit third 
// parties from using the code for non MPEG-4 Systems conforming products.  This 
// copyright notice must be included in all copies or derivative works.
//
// Copyright 1997.


#include "OpenGl/OpenGl.h"
#include "IndexedFaceSet2DProxy.h"
#include "opengl/base/OpenGLVisualRenderer.h"


// traverse scene graph, checking for intersection

/* virtual */ BOOL
IndexedFaceSet2DProxy::Select(
    OpenGLVisualRenderer*, 
    DWORD, 
    unsigned int& depth)
{
  // IndexedFaceSet2D* pIndexedFaceSet2D = (IndexedFaceSet2D*) GetMediaObject();
  GLuint selectBuffer[4];
  
  ::glSelectBuffer(4, selectBuffer);
  ::glRenderMode(GL_SELECT);
  
  ::glPushAttrib(GL_POLYGON_BIT);

  // @@@@ the 2D case does not have "solid" and "ccw" fields
  /***
  if (
    // pIndexedFaceSet2D-> 
    GetSolid())
    ::glEnable(GL_CULL_FACE); 
  if (
    // pIndexedFaceSet2D->
    GetCCW())
    ::glFrontFace(GL_CW);
  ***/

  inlinedRenderLoop(/* pIndexedFaceSet2D-> */ GetCoordIndex().GetLength() != 0,
		    FALSE, FALSE,
		    FALSE, NULL);
  
  ::glPopAttrib();
  
  if (::glRenderMode(GL_RENDER) > 0)
    if (depth >= selectBuffer[1]) {
      depth = selectBuffer[1];
      
      return TRUE;
    }
  
  return FALSE;
}

// Render object

/* virtual */ BOOL
IndexedFaceSet2DProxy::Render(OpenGLVisualRenderer* pRenderer, DWORD)
{
  // IndexedFaceSet2D* pIndexedFaceSet2D = (IndexedFaceSet2D*) GetMediaObject();

  const MF<int>& coordIndex = /* pIndexedFaceSet2D-> */ GetCoordIndex();
  const Coordinate2D* coord = /* pIndexedFaceSet2D-> */ GetCoord();
  const MF<int>& colorIndex = /* ((IndexedFaceSet2D*) GetMediaObject())-> */ GetColorIndex();
  const Color* color = /* pIndexedFaceSet2D-> */ GetColor();
  const MF<int>& texCoordIndex = /* pIndexedFaceSet2D-> */ GetTexCoordIndex();
  const TextureCoordinate* texCoord = /* pIndexedFaceSet2D-> */ (TextureCoordinate*)GetTexCoord().GetNode();
  BOOL colorPerVertex = /* pIndexedFaceSet2D-> */ GetColorPerVertex();

  const MF<Vec2f>& coordarray = coord->point /* GetPoint() */;
  const MF<Color3f>& colorarray = color->color /* GetColor() */;
  const MF<Vec2f>& texCoordarray = texCoord->point /* GetPoint() */;

  ::glPushAttrib(GL_POLYGON_BIT | GL_CURRENT_BIT);

  /***
  SB @@@@ IFS2D does NOT have "solid" and "ccw" fields...

  if (
  // pIndexedFaceSet2D->
  GetSolid())
    ::glEnable(GL_CULL_FACE);
  
  if (! 
  // pIndexedFaceSet2D-> 
  GetCCW())
    ::glFrontFace(GL_CW);
  ***/

  if (colorIndex.GetLength() == 0 && colorPerVertex &&
      texCoordIndex.GetLength() == 0) {
    // special case fast path 

    int coords;

    if (coordIndex.GetLength() == 0)
      coords = coordarray.GetLength();
    else
      coords = coordIndex.GetLength();

#if !defined(GL_VERSION_1_1)
    ::glVertexPointerEXT(2, GL_FLOAT, sizeof(Vec2f), coords, &coordarray[0].x);
    ::glEnable(GL_VERTEX_ARRAY_EXT);
    if (color) {
      ::glColorPointerEXT(3, GL_FLOAT, sizeof(Color), coords, &colorarray[0].r);
      ::glEnable(GL_COLOR_ARRAY_EXT);
    }
    
    if (texCoord) {
      ::glTexCoordPointerEXT(2, GL_FLOAT, sizeof(Vec2f), coords, &texCoordarray[0].x);
      ::glEnable(GL_TEXTURE_COORD_ARRAY_EXT);
    }

    if (coordIndex.GetLength() == 0) {
      ::glDrawArraysEXT(GL_POLYGON, 0, coords);
    } else {
      ::glBegin(GL_POLYGON);

      for (int i = 0; i < coords; ++i) {
	int j = coordIndex[i];

	if (j != -1)
	  ::glArrayElementEXT(j);
	else {
	  ::glEnd();
	  ::glBegin(GL_POLYGON);
	}
      }

      ::glEnd();
    }

    ::glDisable(GL_VERTEX_ARRAY_EXT);
    ::glDisable(GL_COLOR_ARRAY_EXT);
    ::glDisable(GL_TEXTURE_COORD_ARRAY_EXT);
#else
    ::glVertexPointer(2, GL_FLOAT, sizeof(Vec2f), &coordarray[0].x);
    ::glEnableClientState(GL_VERTEX_ARRAY);
    if (color) {
      ::glColorPointer(3, GL_FLOAT, sizeof(Color), &colorarray[0].r);
      ::glEnableClientState(GL_COLOR_ARRAY);
    }

    if (texCoord) {
      ::glTexCoordPointer(2, GL_FLOAT, sizeof(Vec2f), &texCoordarray[0].x);
      ::glEnableClientState(GL_TEXTURE_COORD_ARRAY);
    }

    if (coordIndex.GetLength() == 0) {
      ::glDrawArrays(GL_POLYGON, 0, coords);
    } else {
      ::glBegin(GL_POLYGON);

      for (int i = 0; i < coords; ++i) {
	int j = coordIndex[i];

	if (j != -1)
	  ::glArrayElement(j);
	else {
	  ::glEnd();
	  ::glBegin(GL_POLYGON);
	}
      }

      ::glEnd();
    }

    ::glDisableClientState(GL_VERTEX_ARRAY);
    ::glDisableClientState(GL_COLOR_ARRAY);
    ::glDisableClientState(GL_TEXTURE_COORD_ARRAY);
#endif	
  } else
    /* The following is a little obscure; the reason why the
       inlinedRenderLoop() method isn't started right away is that passing
       inlinedRenderLoop() static boolean arguments allows the compiler to
       evalute parts of the code at compile time and inline the code much
       better.  This of course leads to much larger code; and the compiler
       probably will not inline all of the code.  Still, the code should be
       more efficient.  Doing arc profiling would probably give the
       compiler a chance to do a better job. */

    if (coordIndex.GetLength() != 0)
      if (colorIndex.GetLength() != 0)
	if (colorPerVertex)
	  if (pRenderer->GetCurrentTexture() && texCoord)
	    inlinedRenderLoop(TRUE,
			      TRUE, TRUE,
			      texCoordIndex.GetLength() != 0, texCoord);
	  else	
	    inlinedRenderLoop(TRUE,
			      TRUE, TRUE,
			      FALSE, NULL);
	else
	  if (pRenderer->GetCurrentTexture() && texCoord)
	    inlinedRenderLoop(TRUE,
			      TRUE, FALSE,
			      texCoordIndex.GetLength() != 0, texCoord);
	  else	 
	    inlinedRenderLoop(TRUE,
			      TRUE, FALSE,
			      FALSE, NULL);
      else
	if (colorPerVertex)
	  if (pRenderer->GetCurrentTexture() && texCoord)
	    inlinedRenderLoop(TRUE,
			      FALSE, TRUE,
			      texCoordIndex.GetLength() != 0, texCoord);
	  else		  
	    inlinedRenderLoop(TRUE,
			      FALSE, TRUE,
			      FALSE, NULL);
	else
	  if (pRenderer->GetCurrentTexture() && texCoord)
	    inlinedRenderLoop(TRUE,
			      FALSE, FALSE,
			      texCoordIndex.GetLength() != 0, texCoord);
	  else		
	    inlinedRenderLoop(TRUE,
			      FALSE, FALSE,
			      FALSE, NULL);
    else
      if (colorIndex.GetLength() != 0)
	if (colorPerVertex)
	  if (pRenderer->GetCurrentTexture() && texCoord)
	    inlinedRenderLoop(FALSE,
			      TRUE, TRUE,
			      texCoordIndex.GetLength() != 0, texCoord);
	  else
	    inlinedRenderLoop(FALSE,
			      TRUE, TRUE,
			      FALSE, NULL);
	else
	  if (pRenderer->GetCurrentTexture() && texCoord)
	    inlinedRenderLoop(FALSE,
			      TRUE, FALSE,
			      texCoordIndex.GetLength() != 0, texCoord);
	  else
	    inlinedRenderLoop(FALSE,
			      TRUE, FALSE,
			      FALSE, NULL);
      else
	if (colorPerVertex)
	  if (pRenderer->GetCurrentTexture() && texCoord)
	    inlinedRenderLoop(FALSE,
			      FALSE, TRUE,
			      texCoordIndex.GetLength() != 0, texCoord);
	  else
	    inlinedRenderLoop(FALSE,
			      FALSE, TRUE,
			      FALSE, NULL);
	else
	  if (pRenderer->GetCurrentTexture() && texCoord)
	    inlinedRenderLoop(FALSE,
			      FALSE, FALSE,
			      texCoordIndex.GetLength() != 0, texCoord);
	  else
	    inlinedRenderLoop(FALSE,
			      FALSE, FALSE,
			      FALSE, NULL);

  ::glPopAttrib();
  
#ifdef _DEBUG
  if (pRenderer->CheckGLError("IndexedFaceSet2DProxy:"))
    TRACE("IndexedFaceSet2DProxy::Render() [%08x]: ", this);
#endif

  return TRUE;	 
}
