// -*- c++ -*-

// This software module was originally developed by Karl Anders ygard
// (Telenor R&D), in the course of development of the MPEG-4 Systems
// standard (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// Copyright 1997.

#ifndef __INDEXEDFACESET2DPROXY_H__
#define __INDEXEDFACESET2DPROXY_H__

#include "GeometryProxy.h"

// class TextureCoordinate;
// class IndexedFaceSet2D;
//#include "Nodes.h"
//#include "NodeFld.h"

/** 2D indexed faceset node.

    The IndexedFaceSet2DProxy class specifies a two dimensional mesh,
    located in origo of the local coordinate system. */

class IndexedFaceSet2DProxy
  : public GeometryProxy
{
public:

  /// Constructor

			IndexedFaceSet2DProxy(IndexedFaceSet2D* pIndexedFaceSet2D);

  /// Render object

  virtual BOOL		Render(OpenGLVisualRenderer* pRenderer, DWORD dwTime);

  /// Traverse scene graph, checking for intersection

  virtual BOOL		Select(
      OpenGLVisualRenderer* pRenderer, 
      DWORD dwTime, 
      unsigned int& depth);

private:

  // render object

  void                  inlinedRenderLoop(BOOL bUseCoordIndex,
					  BOOL bUseColorIndex, BOOL bColorPerVertex,
					  BOOL bUseTexCoordIndex, const TextureCoordinate* pTexCoord);

  // Stefano Battista
  // moved methods from IndexedFaceSet2D to Proxy
public:
  /// Get pointer to related MediaObject
  inline IndexedFaceSet2D * GetIndexedFaceSet2D() const
  { return (IndexedFaceSet2D *) GetMediaObject(); }

//ML - removed this function
//  /// Set coordinates
//  inline /* virtual */ void SetCoord(Coordinate2D* pCoord)
//  { GetIndexedFaceSet2D()->coord = pCoord; }

  /// Get coordinates
  inline Coordinate2D* GetCoord()
  { return GetIndexedFaceSet2D()->coord; }  

//ML - removed this function
//  /// Set colors
//  inline /* virtual */ void SetColor(Color* pColor)
//  { GetIndexedFaceSet2D()->color = pColor; }

  /// Get colors
  // SOLARIS PORTING - To core version 5.9rc9 - Appropriate Typecast
  inline Color* GetColor()
  { return (Color*)(MediaObject*)(GetIndexedFaceSet2D()->color); }

//ML - removed this function
//  /// Set texture coordinates
//  inline /* virtual */ void SetTexCoord(TextureCoordinate* pTexCoord)
//  { GetIndexedFaceSet2D()->texCoord = pTexCoord; }

  /// Get texture coordinates
  inline SFTextureCoordinateNode& GetTexCoord() //KM
  { return GetIndexedFaceSet2D()->texCoord; }
  /// Set coordinate indices
  inline void SetCoordIndex(MF<int>& rCoordIndex)
  { GetIndexedFaceSet2D()->coordIndex = rCoordIndex; }
  /// Get coordinate indices
  inline const MF<int>& GetCoordIndex() const
  { return GetIndexedFaceSet2D()->coordIndex; }
  /// Set color indices
  inline void SetColorIndex(MF<int>& rColorIndex)
  { GetIndexedFaceSet2D()->colorIndex = rColorIndex; }
  /// Get color indices
  inline const MF<int>& GetColorIndex() const
  { return GetIndexedFaceSet2D()->colorIndex; }
  /// Set texture coordinate indices
  inline void SetTexCoordIndex(MF<int>& rTexCoordIndex)
  { GetIndexedFaceSet2D()->texCoordIndex = rTexCoordIndex; }
  /// Get texture coordinate indices
  inline const MF<int>& GetTexCoordIndex() const
  { return GetIndexedFaceSet2D()->texCoordIndex; }
  /// Set color per vertex
  inline void SetColorPerVertex(bool bColorPerVertex)
  { GetIndexedFaceSet2D()->colorPerVertex = bColorPerVertex; }
  /// Get color per vertex
  inline BOOL GetColorPerVertex() const
  { return (Bool) GetIndexedFaceSet2D()->colorPerVertex; }
  /***
  /// Set whether object is solid
  inline void SetSolid(BOOL bSolid)
  { GetIndexedFaceSet2D()->solid = bSolid; }
  /// Get whether object is solid
  inline BOOL GetSolid() const
  { return (Bool) GetIndexedFaceSet2D()->solid; }
  /// Set whether polygons are counter-clockwise
  inline void SetCCW(BOOL bCCW)
  { GetIndexedFaceSet2D()->ccw = bCCW; }
  /// Get whether polygons are counter-clockwise
  inline BOOL GetCCW() const
  { return (Bool) GetIndexedFaceSet2D()->ccw; }
  ***/

};

#include "IndexedFaceSet2DProxy.i"

#endif // __INDEXEDFACESET2DPROXY_H__
