// -*- c++ -*-

// This software module was originally developed by Karl Anders ygard
// (Telenor R&D), in the course of development of the MPEG-4 Systems
// standard (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// Copyright 1997.


// constructor

inline
IndexedFaceSet2DProxy::IndexedFaceSet2DProxy(IndexedFaceSet2D* pIndexedFaceSet2D)
  : GeometryProxy(pIndexedFaceSet2D)
{
}

// render object

/* This is the actual method that does the work.  It is inlined, thus
   removing most of the if statements. */

inline void
IndexedFaceSet2DProxy::inlinedRenderLoop(BOOL bUseCoordIndex,
					 BOOL bUseColorIndex, BOOL bColorPerVertex,
					 BOOL bUseTexCoordIndex, const TextureCoordinate* pTexCoord)
{
  // IndexedFaceSet2D* pIndexedFaceSet2D = (IndexedFaceSet2D*) GetMediaObject();

  const MF<int>& coordIndex = /* pIndexedFaceSet2D-> */ GetCoordIndex();
  const Coordinate2D* coord = /* pIndexedFaceSet2D-> */ GetCoord();
  const MF<int>& colorIndex = /* pIndexedFaceSet2D-> */ GetColorIndex();
  const Color* color = /* pIndexedFaceSet2D-> */ GetColor();
  const MF<int>& texCoordIndex = /* pIndexedFaceSet2D-> */ GetTexCoordIndex();

  const MF<Vec2f>& coordarray = coord->point /* GetPoint() */;
  const MF<Color3f>& colors = color->color /* GetColor() */;
  const MF<Vec2f>& texCoords = pTexCoord->point /* GetPoint() */;

  int currentColor = 0;

  if (bUseCoordIndex) {
    BOOL begin = FALSE;
    int coords = coordIndex.GetLength();

    for (int i = 0; i < coords; ++i) {
      if (coordIndex[i] == -1) {
	if (begin) {
	  ::glEnd();
	  begin = FALSE;
	}
      } else {
	const Vec2f& vertex = coordarray[coordIndex[i]];

	if (bColorPerVertex && color)
	  if (bUseColorIndex) {
	    const Color3f& col = colors[colorIndex[i]];

            // SOLARIS PORTING - removed "::"
	    //::glColor3f(col.r, col.g, col.b);
	    glColor3f(col.r, col.g, col.b);
	  } else {
	    const Color3f& col = colors[coordIndex[i]];
	    
            // SOLARIS PORTING - removed "::"
	    //::glColor3f(col.r, col.g, col.b);
	    glColor3f(col.r, col.g, col.b);
	  }

	if (pTexCoord)
	  if (bUseTexCoordIndex) {
	    const Vec2f& texCoord = texCoords[texCoordIndex[i]];
	  
            // SOLARIS PORTING - removed "::"
	    //::glTexCoord2f(texCoord.x, texCoord.y);
	    glTexCoord2f(texCoord.x, texCoord.y);
	  } else {
	    const Vec2f& texCoord = texCoords[coordIndex[i]];
	    
            // SOLARIS PORTING - removed "::"
	    //::glTexCoord2f(texCoord.x, texCoord.y);
	    glTexCoord2f(texCoord.x, texCoord.y);
	  }

	if (!begin) {
	  ::glBegin(GL_POLYGON);

	  if (!bColorPerVertex && color)
	    if (bUseColorIndex) {
	      const Color3f& col = colors[colorIndex[currentColor++]];
	  
              // SOLARIS PORTING - removed "::"
	      //::glColor3f(col.r, col.g, col.b);
	      glColor3f(col.r, col.g, col.b);
	    } else {
	      const Color3f& col = colors[currentColor++];
	    
              // SOLARIS PORTING - removed "::"
	      //::glColor3f(col.r, col.g, col.b);
	      glColor3f(col.r, col.g, col.b);
	    }

	  begin = TRUE;
	}
	    
        // SOLARIS PORTING - removed "::"
	//::glVertex2f(vertex.x, vertex.y);
	glVertex2f(vertex.x, vertex.y);
      }
    }
	
    if (begin)
      ::glEnd();
  } else {
    int coords = coordarray.GetLength();

    ::glBegin(GL_POLYGON);

    for (int i = 0; i < coords; ++i) {
      const Vec2f& vertex = coordarray[i];

      if (bColorPerVertex && color)
	if (bUseColorIndex) {
	  const Color3f& col = colors[colorIndex[i]];
	  
              // SOLARIS PORTING - removed "::"
	      //::glColor3f(col.r, col.g, col.b);
	      glColor3f(col.r, col.g, col.b);
	} else {
	  const Color3f& col = colors[i];
	  
              // SOLARIS PORTING - removed "::"
	      //::glColor3f(col.r, col.g, col.b);
	      glColor3f(col.r, col.g, col.b);
	}
      
	  if (pTexCoord)
	if (bUseTexCoordIndex) {
	  const Vec2f& texCoord = texCoords[texCoordIndex[i]];
	  
            // SOLARIS PORTING - removed "::"
	    //::glTexCoord2f(texCoord.x, texCoord.y);
	    glTexCoord2f(texCoord.x, texCoord.y);
	} else {
	  const Vec2f& texCoord = texCoords[i];
	  
            // SOLARIS PORTING - removed "::"
	    //::glTexCoord2f(texCoord.x, texCoord.y);
	    glTexCoord2f(texCoord.x, texCoord.y);
	}
      
      if (!bColorPerVertex && color)
	if (bUseColorIndex) {
	  const Color3f& col = colors[colorIndex[currentColor++]];
	  
              // SOLARIS PORTING - removed "::"
	      //::glColor3f(col.r, col.g, col.b);
	      glColor3f(col.r, col.g, col.b);
	} else {
	  const Color3f& col = colors[currentColor++];
	  
              // SOLARIS PORTING - removed "::"
	      //::glColor3f(col.r, col.g, col.b);
	      glColor3f(col.r, col.g, col.b);
	}
        // SOLARIS PORTING - removed "::"
	//::glVertex2f(vertex.x, vertex.y);
	glVertex2f(vertex.x, vertex.y);
    }      
    ::glEnd();
  }
}
