// -*- c++ -*-

// This software module was originally developed by Karl Anders ygard
// (Telenor R&D) in the course of development of the MPEG-4 Systems
// standard (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// For more information or to receive an up to date version of this module,
// mail karlo@nta.no or check <http://televr.fou.telenor.no/compositor/>.
//
// Copyright 1997, 1998.

#include "OpenGl/OpenGl.h"
#include "IndexedFaceSetProxy.h"
#include "opengl/base/OpenGLVisualRenderer.h"
#include "../../IM1Decoders/AFX/jwafxstreamdecoder.h"
#include "Stream.h"
#include "opengl/appearance/MultiTextureProxy.h"

// traverse scene graph, checking for intersection

/* virtual */ BOOL
IndexedFaceSetProxy::Select(
    OpenGLVisualRenderer* pRenderer, 
    DWORD dwTime, 
    unsigned int& depth)
{
  GLuint selectBuffer[4];

  ::glSelectBuffer(4, selectBuffer);
  ::glRenderMode(GL_SELECT);

  Render(pRenderer, dwTime);

#ifdef _DEBUG
  if (pRenderer->CheckGLError("IndexedFaceSetProxy:"))
    TRACE("IndexedFaceSetProxy::Select() [%08x]: ", this);
#endif

  if (::glRenderMode(GL_RENDER) > 0)
    if (depth > selectBuffer[1]) {
      depth = selectBuffer[1];

      return TRUE;
    }

  return FALSE;
}

// Render object
BOOL IndexedFaceSetProxy::RenderFromStream(MediaStream* poStream, int id, DWORD dwTime)
{
  BYTE * pData = NULL;
  int nSize = 0;
  BOOL bEOU;
  int nStatus;
  BOOL bSkip = TRUE;
  BOOL bWait = FALSE;
  long lDelta = 0;
  BOOL bDontClose = TRUE;
  DWORD rdvTime;
  
  pData = (unsigned char *) poStream->Fetch(
    nSize, rdvTime, bEOU, nStatus, bSkip, bWait, lDelta, bDontClose);
  

  TRACE("[BitWrapperProxy::Enable] Fetch returned Size %4d \n", nSize);
  
  if (pData)
  {
	//ory DMLAB 2006 12
	if(type==0)
	{
		JWAFX3DMCDecoder* pDecoder = new JWAFX3DMCDecoder;

		pDecoder->SetValues(pData);

		pDecoder->Write("AFX-IndexedFaceSet.debug.vrml", id); 
		Enable(pData);
	}
	else if(type==1)
	{	
		JWAFX3DMCExtensionDecoder* pDecoder = new JWAFX3DMCExtensionDecoder;

		pDecoder->SetValues(pData);

		pDecoder->Write("AFX-IndexedFaceSet.debug.vrml", id);
		Enable(pData);
	}
	else if(type==2)	// FAMC
	{
		m_Data = pData;
	}

	//~ory DMLAB
    return TRUE;
  }
  return FALSE;
}

BOOL IndexedFaceSetProxy::RenderFromBuffer(SFString *pBuffer, int id)
{
  BYTE * pData = NULL;
  int nSize = 0;

  //ory DMLAB 2006 12
  if(type==0)
  {
	  JWAFX3DMCDecoder* pDecoder = new JWAFX3DMCDecoder;

	  // Decode the bitstream
	  pDecoder->Decode((LPBYTE) pBuffer->GetValue(), pBuffer->GetLength(), (LPBYTE) pData, nSize);
  
	  pDecoder->Write("AFX-IndexedFaceSet.debug.vrml", id); 
      Enable(pData);

  }
  else if(type==1)
  {
	  JWAFX3DMCExtensionDecoder* pDecoder = new JWAFX3DMCExtensionDecoder;

	  // Decode the bitstream
	  pDecoder->Decode((LPBYTE) pBuffer->GetValue(), pBuffer->GetLength(), (LPBYTE) pData, nSize);
  
	  pDecoder->Write("AFX-IndexedFaceSet.debug.vrml", id);
  	  Enable(pData);

  }
  //~ory DMLAB

  return TRUE;
}

BOOL IndexedFaceSetProxy::Render(DWORD dwTime)
{
  return TRUE;
}

/* virtual */ BOOL
IndexedFaceSetProxy::Render(OpenGLVisualRenderer* pRenderer, DWORD dwTime)
{
  // INT KM 2007 12 10
  if(type==2)	// FAMC
  {
	RenderFAMC(dwTime);
  }
  // INT KM 2007 12 10

  IndexedFaceSet* pIndexedFaceSet = (IndexedFaceSet*) GetMediaObject();

  const MF<int>& coordIndex = pIndexedFaceSet->coordIndex;
  /* const */ Coordinate * pCoord = pIndexedFaceSet->coord;
  const MF<int>& colorIndex = pIndexedFaceSet->colorIndex;
  // SOLARIS PORTING - To core 5.9rc9 - use GetColor()
  ///* const */ Color* pColor = pIndexedFaceSet->color;
  /* const */ Color* pColor = GetColor();
  const MF<int>& normalIndex = pIndexedFaceSet->normalIndex;
  /* const */ Normal* pNormal = pIndexedFaceSet->normal;
  const MF<int>& texCoordIndex = pIndexedFaceSet->texCoordIndex;

  //TextureCoordinate* pTexCoord = pIndexedFaceSet->texCoord;

// MultiTexture Rendering KM
  int numberOfViews = 1; // default for single textures
  TextureCoordinate** ppTexCoord = NULL;
  int NodeCode = 0;
  if( pIndexedFaceSet->texCoord)
  {
    NodeCode = pIndexedFaceSet->texCoord.GetNode()->GetNodeCode();
  }
  BOOL multiTexFlag = (NodeCode == CODE_MultiTextureCoordinate);
  float* afCosTheta = NULL;
  float* afRelTexWeight = NULL;
  float* afTotTexWeight = NULL;
  // Depending on the undelying node, either obtain a single coord array or the multiTex equivalent
  if( NodeCode == CODE_MultiTextureCoordinate )
  {
    NodeField* pNode = (NodeField*) pIndexedFaceSet->texCoord.GetNode()->GetField(0);
    numberOfViews = ((MFNodeType<SFTextureNode>*)pNode)->GetLength();
    ppTexCoord = new TextureCoordinate*[numberOfViews];
    MultiTextureProxy* mtp = pRenderer->GetMultiTexture();
    if( numberOfViews > mtp->getNumberOfViews()) // check, whether # of views in tex and texCoords differ
    {
      numberOfViews = mtp->getNumberOfViews();
    }
    // assign array for view adaptive weights
    afCosTheta = new float[numberOfViews];
    afRelTexWeight = new float[numberOfViews];
    afTotTexWeight = new float[numberOfViews];

    double rSum = .0;
    for(int view = 0; view < numberOfViews; view++) // now get the texCoord arrays
    {
      ppTexCoord[view] = (TextureCoordinate*)((MFNodeType<SFTextureNode>*)pNode)->GetObject(view);

      GLfloat mvmatrix[16];
      glGetFloatv(GL_MODELVIEW_MATRIX, mvmatrix);
      Matrix rot(mvmatrix);

//      if(m_OldProjMatrix == rot) // nice try, but...
//        return TRUE;
//      if(view == numberOfViews-1)
//        m_OldProjMatrix = rot;

      rot[12] = rot[13] = rot[14] = .0;

      Vec3f viewVec = mtp->getCameraVector()[view];
      viewVec.Normalize();
      Vec3f camVec = mtp->getCameraVector()[0];
      camVec.Normalize();
      Vec3f transVec = rot.MultiplyPoint(viewVec);
      float fDotProduct = camVec*transVec;
      fDotProduct = (fDotProduct<0.0f)?0.0f:fDotProduct;
      afCosTheta[view] = fDotProduct;
      if (afCosTheta[view] != 1.0f)
      {
        afRelTexWeight[view] = (afCosTheta[view])/(1.0f-afCosTheta[view]);
      }
      else {afRelTexWeight[view] = FLT_MAX;}
      rSum +=afRelTexWeight[view];
      
    }
    for ( view = 0; view < numberOfViews; view++) 
    { 
      if (rSum != 0.0f) { afTotTexWeight[view]=afRelTexWeight[view]/rSum; }
      else { afTotTexWeight[view] = 0.0f; }
    }

  }
  else
  {
    ppTexCoord = new TextureCoordinate*[1];
    ppTexCoord[0] = (TextureCoordinate*) pIndexedFaceSet->texCoord.GetNode();
  }

  // Implementation MultiTextureCoord KM
  for(int view = 0; view < numberOfViews; view++)
  {
    if( multiTexFlag)
    {
      MultiTextureProxy* mtp = pRenderer->GetMultiTexture();
      if(view==0) 
      {
        glDisable(GL_BLEND);
      }
      else
      {
        glEnable(GL_BLEND);
        glBlendFunc(GL_SRC_ALPHA, GL_ONE);
        glBlendFunc(GL_DST_ALPHA, GL_ONE);
        //glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
      }
      glColor3f(afTotTexWeight[view], afTotTexWeight[view],afTotTexWeight[view]);
      if ( mtp->GetTexture(view))
        ((TextureNodeProxy*) ( mtp->GetTexture(view)->GetRenderingObject()))->Enable(pRenderer, dwTime);
    }
// End MultiTexture Rendering


  BOOL colorPerVertex = (Bool) (pIndexedFaceSet->colorPerVertex);
  BOOL normalPerVertex = (Bool) (pIndexedFaceSet->normalPerVertex);

  const MF<Vec3f>& coordarray = pCoord->point;
  const MF<Color3f>& colorarray = pColor->color;
  const MF<Vec3f>& normalarray = pNormal->vector;
  const MF<Vec2f>& texCoordarray = ppTexCoord[view]->point; //KM

  ::glPushAttrib(GL_POLYGON_BIT | GL_CURRENT_BIT);

  if ((BOOL) (Bool) (pIndexedFaceSet->solid) /* GetSolid() */)
    ::glEnable(GL_CULL_FACE);

  if (! (BOOL) (Bool) (pIndexedFaceSet->ccw) /* GetCCW() */)
    ::glFrontFace(GL_CW);

  if (! pNormal)
    ::glDisable(GL_LIGHTING);

  if (colorIndex.GetLength() == 0 && colorPerVertex &&
    normalIndex.GetLength() == 0 && normalPerVertex &&
    texCoordIndex.GetLength() == 0)
  {
    TRACE("[IFS::Render] no ColorIndex, no NormalIndex, no TexCoordIndex \n");

    // special case fast path 
  
    int coords;
  
    if (coordIndex.GetLength() == 0)
      coords = coordarray.GetLength();
    else
      coords = coordIndex.GetLength();

    if (long(pIndexedFaceSet->GetUpdateTime()) > m_nTimeStamp ||
      (pCoord && (long(pCoord->GetUpdateTime()) > m_nTimeStamp)) ||
      (pColor && (long(pColor->GetUpdateTime()) > m_nTimeStamp)) ||
      (pNormal && (long(pNormal->GetUpdateTime()) > m_nTimeStamp)) ||
      (ppTexCoord[view] && (long(ppTexCoord[view]->GetUpdateTime()) > m_nTimeStamp)) || multiTexFlag)
    {
      if (! m_nDisplayList)
      {
        m_nDisplayList = ::glGenLists(1);
      }

      if(view == numberOfViews-1)
        m_nTimeStamp = long(dwTime); // carry out only, if all views are processed, otherwise this branch is only accessed once each update time interval


      TRACE("BEFORE NewList -- DisplayList %4d \n", m_nDisplayList);
      ::glNewList(m_nDisplayList, GL_COMPILE_AND_EXECUTE);
      TRACE("AFTER  NewList -- DisplayList %4d \n", m_nDisplayList);
    
      if (pRenderer->GetOpenGLVersion() < 1.1) {
#if defined(GL_EXT_vertex_array) && !defined(WIN32)
        ::glVertexPointerEXT(3, GL_FLOAT, sizeof(Vec3f), coords, &coordarray[0].x);
        ::glEnable(GL_VERTEX_ARRAY_EXT);
        if (pColor) {
          ::glColorPointerEXT(3, GL_FLOAT, sizeof(Color3f), coords, &colorarray[0].r);
          ::glEnable(GL_COLOR_ARRAY_EXT);
        }
        if (pNormal) {
          ::glNormalPointerEXT(GL_FLOAT, sizeof(Vec3f), coords, &normalarray[0].x);
          ::glEnable(GL_NORMAL_ARRAY_EXT);
        }
        if (pTexCoord) {
          ::glTexCoordPointerEXT(2, GL_FLOAT, sizeof(Vec2f), coords, &texCoordarray[0].x);
          ::glEnable(GL_TEXTURE_COORD_ARRAY_EXT);
        }
      
        if (coordIndex.GetLength() == 0)
          ::glDrawArraysEXT(GL_POLYGON, 0, coords);
        else {
          ::glBegin(GL_POLYGON);
        
          for (int i = 0; i < coords; ++i) {
            int j = coordIndex[i];
          
            if (j != -1)
              ::glArrayElementEXT(j);
            else {
              ::glEnd();
              ::glBegin(GL_POLYGON);
            }
          }
        
          ::glEnd();
        }
      
        ::glDisable(GL_VERTEX_ARRAY_EXT);
        ::glDisable(GL_COLOR_ARRAY_EXT);
        ::glDisable(GL_NORMAL_ARRAY_EXT);
        ::glDisable(GL_TEXTURE_COORD_ARRAY_EXT);
#endif
      } 
      else 
      {
        ::glVertexPointer(3, GL_FLOAT, sizeof(Vec3f), &coordarray[0].x);
        ::glEnableClientState(GL_VERTEX_ARRAY);
        if (pColor) {
          ::glColorPointer(3, GL_FLOAT, sizeof(Color), &colorarray[0].r);
          ::glEnableClientState(GL_COLOR_ARRAY);
        }
        if (pNormal) {
          ::glNormalPointer(GL_FLOAT, sizeof(Vec3f), &normalarray[0].x);
          ::glEnableClientState(GL_NORMAL_ARRAY);
        }
        if (ppTexCoord[view]) {
          ::glTexCoordPointer(2, GL_FLOAT, sizeof(Vec2f), &texCoordarray[0].x);
          ::glEnableClientState(GL_TEXTURE_COORD_ARRAY);
        }
      
        if (coordIndex.GetLength() == 0)
        {
          ::glDrawArrays(GL_POLYGON, 0, coords);
        }
        else 
        {
          TRACE("[IFS::Render] with CoordIndex \n");
          ::glBegin(GL_POLYGON);

          int i, j;

          for (i = 0; i < coords; ++i) {
            j = coordIndex[i];
          
            if (j != -1)
              ::glArrayElement(j);
            else {
              ::glEnd();
              ::glBegin(GL_POLYGON);
            }
          }

          ::glEnd();
        }
      
        ::glDisableClientState(GL_VERTEX_ARRAY);
        ::glDisableClientState(GL_COLOR_ARRAY);
        ::glDisableClientState(GL_NORMAL_ARRAY);
        ::glDisableClientState(GL_TEXTURE_COORD_ARRAY);
      }
    
      ::glEndList();

        if( multiTexFlag ) //KM
        {
          if(view < numberOfViews-1) ppTexCoord[view+1]->SetUpdateTime();
          else ppTexCoord[0]->SetUpdateTime();
        }

    } 
    else
    {
      TRACE("BEFORE CallList -- DisplayList %4d \n", m_nDisplayList);
      ::glCallList(m_nDisplayList);
      TRACE("AFTER  CallList -- DisplayList %4d \n", m_nDisplayList);
    }
  }
  else
  {
    TRACE("[IFS::Render] either ColorIndex, or NormalIndex, or TexCoordIndex \n");
    /* The following is a little obscure; the reason why the
    inlinedRenderLoop() method isn't started right away is that passing
    inlinedRenderLoop() static boolean arguments allows the compiler to
    evalute parts of the code at compile time and inline the code much
    better.  This of course leads to much larger code; and the compiler
    probably will not inline all of the code.  Still, the code should be
    more efficient.  Doing arc profiling would probably give the
    compiler a chance to do a better job. */
    if (coordIndex.GetLength() != 0)
    { // 1
      if (colorIndex.GetLength() != 0)
      { // 2
        if (colorPerVertex)
        { // 3
          if (pRenderer->GetCurrentTexture() && ppTexCoord[view])
          {
            inlinedRenderLoop(TRUE,
              TRUE, TRUE,
              normalIndex.GetLength() != 0, normalPerVertex,
              texCoordIndex.GetLength() != 0, ppTexCoord[view],
              dwTime);
          }
          else
          {
            inlinedRenderLoop(TRUE,
              TRUE, TRUE,
              normalIndex.GetLength() != 0, normalPerVertex,
              FALSE, NULL,
              dwTime);
          }
        } // 3
        else
        { // 3
          if (pRenderer->GetCurrentTexture() && ppTexCoord[view])
          {
            inlinedRenderLoop(TRUE,
              TRUE, FALSE,
              normalIndex.GetLength() != 0, normalPerVertex,
              texCoordIndex.GetLength() != 0, ppTexCoord[view],
              dwTime);
          }
          else
          {
            inlinedRenderLoop(TRUE,
              TRUE, FALSE,
              normalIndex.GetLength() != 0, normalPerVertex,
              FALSE, NULL,
              dwTime);
          }
        } // 3
      } // 2
      else
      { // 2
        if (colorPerVertex)
        { // 3
          if (pRenderer->GetCurrentTexture() && ppTexCoord[view])
          {
            inlinedRenderLoop(TRUE,
              FALSE, TRUE,
              normalIndex.GetLength() != 0, normalPerVertex,
              texCoordIndex.GetLength() != 0, ppTexCoord[view],
              dwTime);
          }
          else
          {
            inlinedRenderLoop(TRUE,
              FALSE, TRUE,
              normalIndex.GetLength() != 0, normalPerVertex,
              FALSE, NULL,
              dwTime);
          }
        } // 3
        else
        { // 3
          if (pRenderer->GetCurrentTexture() && ppTexCoord[view])
          {
            inlinedRenderLoop(TRUE,
              FALSE, FALSE,
              normalIndex.GetLength() != 0, normalPerVertex,
              texCoordIndex.GetLength() != 0, ppTexCoord[view],
              dwTime);
          }
          else
          {
            inlinedRenderLoop(TRUE,
              FALSE, FALSE,
              normalIndex.GetLength() != 0, normalPerVertex,
              FALSE, NULL,
              dwTime);
          }
        } // 3
      } // 2
    } // 1
    else
    { // 1
      if (colorIndex.GetLength() != 0)
      { // 2
        if (colorPerVertex)
        { // 3
          if (pRenderer->GetCurrentTexture() && ppTexCoord[view])
          {
            inlinedRenderLoop(FALSE,
              TRUE, TRUE,
              normalIndex.GetLength() != 0, normalPerVertex,
              texCoordIndex.GetLength() != 0, ppTexCoord[view],
              dwTime);
          }
          else
          {
            inlinedRenderLoop(FALSE,
              TRUE, TRUE,
              normalIndex.GetLength() != 0, normalPerVertex,
              FALSE, NULL,
              dwTime);
          }
        } // 3
        else
        { // 3
          if (pRenderer->GetCurrentTexture() && ppTexCoord[view])
          {
            inlinedRenderLoop(FALSE,
              TRUE, FALSE,
              normalIndex.GetLength() != 0, normalPerVertex,
              texCoordIndex.GetLength() != 0, ppTexCoord[view],
              dwTime);
          }
          else
          {
            inlinedRenderLoop(FALSE,
              TRUE, FALSE,
              normalIndex.GetLength() != 0, normalPerVertex,
              FALSE, NULL,
              dwTime);
          }
        } // 3
      } // 2
      else
      { // 2
        if (colorPerVertex)
        { // 3
          if (pRenderer->GetCurrentTexture() && ppTexCoord[view])
          {
            inlinedRenderLoop(FALSE,
              FALSE, TRUE,
              normalIndex.GetLength() != 0, normalPerVertex,
              texCoordIndex.GetLength() != 0, ppTexCoord[view],
              dwTime);
          }
          else
          {
            inlinedRenderLoop(FALSE,
              FALSE, TRUE,
              normalIndex.GetLength() != 0, normalPerVertex,
              FALSE, NULL,
              dwTime);
          }
        } // 3
        else
        { // 3
          if (pRenderer->GetCurrentTexture() && ppTexCoord[view])
          {
            inlinedRenderLoop(FALSE,
              FALSE, FALSE,
              normalIndex.GetLength() != 0, normalPerVertex,
              texCoordIndex.GetLength() != 0, ppTexCoord[view],
              dwTime);
          }
          else
          {
            inlinedRenderLoop(FALSE,
              FALSE, FALSE,
              normalIndex.GetLength() != 0, normalPerVertex,
              FALSE, NULL,
              dwTime);
          }
        } // 3
      } // 2
    } // 1
  }

  ::glPopAttrib();

    if( NodeCode == CODE_MultiTextureCoordinate )
    {
      MultiTextureProxy* mtp = pRenderer->GetMultiTexture();
      if ( mtp->GetTexture(view))
        ((TextureNodeProxy*) ( mtp->GetTexture(view)->GetRenderingObject()))->Disable(pRenderer, dwTime);
      glEnable(GL_BLEND);
    }

  } // end for multitex
  if (afCosTheta) delete[]  afCosTheta;
  if (afRelTexWeight) delete[]  afRelTexWeight;
  if (afTotTexWeight) delete[]  afTotTexWeight;
  if (ppTexCoord) delete[] ppTexCoord;

#ifdef _DEBUG
  if (pRenderer->CheckGLError("IndexedFaceSetProxy:"))
    TRACE("IndexedFaceSetProxy::Render() [%08x] \n", this);
#endif
                              
  return TRUE;
}

// Render object description

/* virtual */ void
IndexedFaceSetProxy::RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD /* dwTime */)
{
  // IndexedFaceSet* pIndexedFaceSet = (IndexedFaceSet*) GetMediaObject();
  int children = ((GetCoord() ? 1 : 0) +
		  (GetColor() ? 1 : 0) +
		  (GetNormal() ? 1 : 0) +
		  (GetTexCoord() ? 1 : 0));

  GLfloat color[] = { 1.0f, 0.0f, 1.0f, 0.75f};

  //satish changed this line for compilation purposes
        //have to be fixed afterwards

  //::glColor4fv(color);
  glColor4fv(color);

  RenderDescriptionBox(pRenderer, "IndexedFaceSet");

  ::glPushMatrix();

  ::glScalef(1.0f / children, 1.0f, 1.0f);
  ::glTranslatef(0.0f, -1.0f, 0.0f);

  if (/* pIndexedFaceSet-> */ GetCoord()) {
    RenderDescriptionBox(pRenderer, "Coordinate");
    
    ::glTranslatef(1.0f, 0.0f, 0.0f);
  }

  if (/* pIndexedFaceSet-> */ GetColor()) {
    RenderDescriptionBox(pRenderer, "Color");
    
    ::glTranslatef(1.0f, 0.0f, 0.0f);
  }

  if (/* pIndexedFaceSet-> */ GetNormal()) {
    RenderDescriptionBox(pRenderer, "Normal");
    
    ::glTranslatef(1.0f, 0.0f, 0.0f);
  }

  if (/* pIndexedFaceSet-> */ GetTexCoord())
    RenderDescriptionBox(pRenderer, "TextureCoordinate");

  ::glPopMatrix();
}


// Stefano Battista
// moved from IndexedFaceSet to Proxy

// ===========================================================
// Update bounding box

void IndexedFaceSetProxy::UpdateBbox()
{
  // Node::UpdateBbox();
  for (int i = m_CollisionParents.GetLength() - 1; i >= 0; --i)
    ( (MediaObjectProxy*) ( m_CollisionParents.ElementAt(i) ->
      GetRenderingObject() ) ) ->
      UpdateBbox();
}


// ===========================================================
// Compute normals

void IndexedFaceSetProxy::ComputeNormals()
{
  // See if we have normals that may need updating
  if (! GetNormal()) {
    // If we do, see if we have placeholders

    if (! m_pAutoNormals){
      m_pAutoNormals = new Normal;
      m_pAutoNormals->SetExecutive(GetIndexedFaceSet()->GetExecutive());
      // m_pAutoNormals->RegisterObject();
      MediaObjectProxy * poNormalProxy = new MediaObjectProxy(m_pAutoNormals);
      m_pAutoNormals->SetRenderingObject(poNormalProxy);
    }
    
    GenerateNormals();
  }
}

// ===========================================================
// Generate normals

void IndexedFaceSetProxy::GenerateNormals()
{
  if (GetCoord()) {
    const MF<Vec3f>& coordarray = GetCoord()->point /* GetPoint() */;
    const MF<int>& coordIndex = GetCoordIndex();
    
    if (coordIndex.GetLength() > 0) {
      ZArray<Vec3f> faceNormals;
      
      ZArray<ZArray<int> > vertexDependencies;
      ZArray<int> uniqueVertexMap;
      
      uniqueVertexMap.Expand(coordarray.GetLength());
      
      {
        ZArray<Vec3f> uniqueVertices;
        int numberOfUniqueVertices = 0;
        
        int point = 0;
        int face = 0;
        
        // first, compute normals of all faces and find all vertex dependencies
        
        for (int i = 0; i < coordIndex.GetLength(); ++i) {
          int coord = coordIndex[i];
          
          if (coord != -1) {
            int uniqueVertex = uniqueVertices.Find(coordarray[coord]);
            
            if (uniqueVertex == -1) {
              uniqueVertices += coordarray[coord];
              
              ++vertexDependencies;
              uniqueVertex = numberOfUniqueVertices++;
            }
            
            uniqueVertexMap[coord] = uniqueVertex;
            vertexDependencies[uniqueVertex] += face;
            
            // Compute normal when we have enough vertices
            
            if (i - point == 2) {
              Vec3f normal = ComputeNormal(
                coordarray[coordIndex[point]],
                coordarray[coordIndex[point + 1]],
                coordarray[coordIndex[point + 2]]);
              
              normal.Normalize();
              
              faceNormals += normal;
            }
          } else {
            face++;
            point = i + 1;
          }
        }
      }
      
      {
        MF<Vec3f> normals;
        
        int face = 0;
        int counter = 0;
        
        m_autoNormalIndex.Remove();
        
        for (int j = 0; j < coordIndex.GetLength(); ++j)
          if (coordIndex[j] != -1) {
            const ZArray<int>& faces = vertexDependencies[uniqueVertexMap[coordIndex[j]]];
            Vec3f normal = faceNormals[face];
            Vec3f vertexNormal;
            
            for (int k = 0; k < faces.GetLength(); ++k)
              if (normal == faceNormals[faces[k]] ||
                ::acos(normal * faceNormals[faces[k]]) <= GetIndexedFaceSet()->creaseAngle)
                vertexNormal += faceNormals[faces[k]];
              
              normals += vertexNormal;
              m_autoNormalIndex += counter++;
          } else {
            m_autoNormalIndex += -1;
            ++face;
          }
          
          m_pAutoNormals->vector /* SetVector */ = (normals);
      }
    } else
      for (int i = 0; i < GetCoord()->point /* GetPoint() */ .GetLength(); ++i)
        printf("IndexedFaceSet::GenerateNormals: nonimplemented stuff here\n");
  }
  
  m_pAutoNormals->SetUpdateTime();
}

// ===========================================================
// Compute normal

Vec3f IndexedFaceSetProxy::ComputeNormal(
    const Vec3f& point_a,
		const Vec3f& point_b,
		const Vec3f& point_c)
{
  Vec3f vector_a(point_a);
  Vec3f vector_b(point_a);

  vector_a -= point_b;
  vector_b -= point_c;

  vector_a.Cross(vector_b);
  vector_a.Normalize();

  return vector_a;
}


#if defined SAIT_CONFORMANCE_2003
void IndexedFaceSetProxy::Enable (unsigned char *pData)
{
	IndexedFaceSet* pIndexedFaceSet = (IndexedFaceSet*) GetMediaObject();

	if(pIndexedFaceSet)
	{
/*		 0	eventIn MF<int> set_colorIndex;
         1	eventIn MF<int> set_coordIndex;
         2	eventIn MF<int> set_normalIndex;
         3	eventIn MF<int> set_texCoordIndex;

         4	exposedField SFColorNode color;
         5	exposedField SFNode<class Coordinate> coord;
         6	exposedField SFNode<class Normal> normal;
         7	exposedField SFNode<class TextureCoordinate> texCoord;
         8	FIELD SF<Bool> ccw;
         9	FIELD MF<int> colorIndex;
         10	FIELD SF<Bool> colorPerVertex;
         11	FIELD SF<Bool> convex;
         12	FIELD MF<int> coordIndex;
         13	FIELD SF<float> creaseAngle;
         14	FIELD MF<int> normalIndex;
         15	FIELD SF<Bool> normalPerVertex;
         16	FIELD SF<Bool> solid;
         17	FIELD MF<int> texCoordIndex;

         18	void Onset_colorIndex();
         19	void Onset_coordIndex();
         20	void Onset_normalIndex();
         21	void Onset_texCoordIndex();
*/

	// 3DMC  ũ⸦ .
	//  ũ⸦ rnOutputLength   rnOutputLength ,
	// rnOutputLength  ũ⸦  !
	// , return FALSE
		int i = 0;
		int j = 0;
		int   nLengthOfTotal = 0;
		Vec3f fdummy;
			
		// ccw
		bool* ccw = (bool*) pData;
		SF<Bool>* pData_ccw = (SF<Bool>*) pIndexedFaceSet->GetField(8);
		*pData_ccw = ccw[0];
		nLengthOfTotal += sizeof(bool); 

		// convex
		bool* convex = (bool*) (pData + nLengthOfTotal);
		SF<Bool>* pData_convex = (SF<Bool>*) pIndexedFaceSet->GetField(11);
		*pData_convex = convex[0];
		nLengthOfTotal += sizeof(bool); 
			
		// creaseAngle
		float* creaseAngle = (float *) (pData + nLengthOfTotal);
		SF<float>* pData_creaseAngle = (SF<float>*) pIndexedFaceSet->GetField(13);
		*pData_creaseAngle = creaseAngle[0];
		nLengthOfTotal += sizeof(float); 

		// solid
		bool* solid = (bool *) ( pData + nLengthOfTotal);
		SF<Bool>* pData_solid = (SF<Bool>*) pIndexedFaceSet->GetField(16);
		*pData_solid = solid[0];
		nLengthOfTotal += sizeof(bool); 

		// Size of Coord
		int* pData_SizeCoord = (int*) (pData + nLengthOfTotal);
		int nSizeOfCoord = pData_SizeCoord[0];
		nLengthOfTotal += sizeof(int); 

		// Coord
		float* pData_coord = (float*) (pData + nLengthOfTotal);
		SFNode<class Coordinate>* pCoordinate = 
			 (SFNode<class Coordinate>*) pIndexedFaceSet->GetField(5);
		//Coordinate* pCoordinateNode = (Coordinate*) pCoordinate->GetNode();

		//MF<Vec3f>* pPoint = (MF<Vec3f>*) pCoordinateNode->GetField(0);
		MF<Vec3f>* pPoint = (MF<Vec3f>*) pCoordinate->GetNode()->GetField(0);

		 for(i = 0, j = 0; i<nSizeOfCoord; i++)
		 {
			 fdummy.x = pData_coord[j++];
			 fdummy.y = pData_coord[j++];
			 fdummy.z = pData_coord[j++];
			 pPoint->SetAt(i, fdummy, FALSE);
		 }
		 nLengthOfTotal += sizeof(float)*nSizeOfCoord; 
     
		// Size of coordIndex
		int* pData_SizeCoordIndex = (int*) (pData + nLengthOfTotal);
		int nSizeOfCoordIndex = pData_SizeCoordIndex[0];
		nLengthOfTotal += sizeof(int); 

		// coordIndex
		int* pData_coordIndex = (int*) (pData + nLengthOfTotal);
		MF<int>* pCoordIndex = (MF<int>*) pIndexedFaceSet->GetField(12);
			
		for(i = 0; i< nSizeOfCoordIndex; i++)
		{
			 pCoordIndex->SetAt(i, pData_coordIndex[i], FALSE);
		}
		nLengthOfTotal += sizeof(int)*nSizeOfCoordIndex; 

		// Size of Color
		int* pData_SizeColor = (int*) (pData + nLengthOfTotal);
		int nSizeOfColor = pData_SizeColor[0];
		nLengthOfTotal += sizeof(int); 

		if(nSizeOfColor > 0)
		{
			// colorPerVertex:  True or False
			bool* colorPerVertex = (bool*) ( pData + nLengthOfTotal);
			SF<Bool>* pData_colorPerVertex = (SF<Bool>*) pIndexedFaceSet->GetField(10);
			*pData_colorPerVertex = colorPerVertex[0];
			nLengthOfTotal += sizeof(bool); 
    			
			// Color
			float* pData_color = (float*) (pData + nLengthOfTotal);
			SFColorNode* pColor = (SFColorNode*) pIndexedFaceSet->GetField(4);
			
			MF<Vec3f>* pValue = (MF<Vec3f>*) pColor->GetNode()->GetField(0);

			for(i = 0, j = 0; i<nSizeOfColor; i++)
			{
				fdummy.x = pData_color[j++];
				fdummy.y = pData_color[j++];
				fdummy.z = pData_color[j++];
				pValue->SetAt(i, fdummy, FALSE);
			}
			nLengthOfTotal += sizeof(float)*nSizeOfColor; 
    
			// Size of Color Index
			int* pData_SizeColorIndex = (int*) (pData + nLengthOfTotal);
			int nSizeOfColorIndex = pData_SizeColorIndex[0];
			nLengthOfTotal += sizeof(int); 

			// Color Index
			if(nSizeOfColorIndex > 0)
			{
				int* pData_colorIndex = (int*) (pData + nLengthOfTotal);
				MF<int>* pColorIndex = (MF<int>*) pIndexedFaceSet->GetField(9);
			
				for(i = 0; i< nSizeOfColorIndex; i++)
				{
					pColorIndex->SetAt(i, pData_colorIndex[i], FALSE);
				}
				nLengthOfTotal += sizeof(int)*nSizeOfColorIndex; 
			}
			else
			{
				if(colorPerVertex[0] == TRUE)
				{
					MF<int>* pColorIndex = (MF<int>*) pIndexedFaceSet->GetField(9);
					for(i = 0; i< nSizeOfCoordIndex; i++)
					{
						pColorIndex->SetAt(i, pData_coordIndex[i], FALSE);
					}
				}
			}
		}


		// Size of Normal
		int* pData_SizeNormal = (int*) (pData + nLengthOfTotal);
		int nSizeOfNormal = pData_SizeNormal[0];
		nLengthOfTotal += sizeof(int); 

		if(nSizeOfNormal > 0) // Normals
		{
			// NormalPerVertex:  True or False
			bool* normalPerVertex = (bool*) ( pData + nLengthOfTotal);
			SF<Bool>* pData_normalPerVertex = (SF<Bool>*) pIndexedFaceSet->GetField(15);
			*pData_normalPerVertex = normalPerVertex[0];
			nLengthOfTotal += sizeof(bool); 
			
			// Normal
			float* pData_normal = (float*) (pData + nLengthOfTotal);
			SFNode<class Normal>* pNormal = 
				 (SFNode<class Normal>*) pIndexedFaceSet->GetField(6);
			
			MF<Vec3f>* pValue = (MF<Vec3f>*) pNormal->GetNode()->GetField(0);

			for(i = 0, j = 0; i<nSizeOfNormal; i++)
			{
				fdummy.x = pData_normal[j++];
				fdummy.y = pData_normal[j++];
				fdummy.z = pData_normal[j++];
				pValue->SetAt(i, fdummy, FALSE);
			}
			nLengthOfTotal += sizeof(float)*nSizeOfNormal; 

			// Size of Normal Index
			int* pData_SizeNormalIndex = (int*) (pData + nLengthOfTotal);
			int nSizeOfNormalIndex = pData_SizeNormalIndex[0];
			nLengthOfTotal += sizeof(int); 

			// NormalIndex
			if(nSizeOfNormalIndex > 0) 
			{
				int* pData_normalIndex = (int*) (pData + nLengthOfTotal);
				MF<int>* pNormalIndex = (MF<int>*) pIndexedFaceSet->GetField(14);
			
				for(i = 0; i< nSizeOfNormalIndex; i++)
				{
					pNormalIndex->SetAt(i, pData_normalIndex[i], FALSE);
				}
				nLengthOfTotal += sizeof(int)*nSizeOfNormalIndex; 
			}
			else  
			{
				if(normalPerVertex[0] == TRUE) 
				{
					MF<int>* pNormalIndex = (MF<int>*) pIndexedFaceSet->GetField(14);
					for(i = 0; i< nSizeOfCoordIndex; i++)
					{
						pNormalIndex->SetAt(i, pData_coordIndex[i], FALSE);
					}
				}
			}
		}

		// Size of TexCoord
		int* pData_SizeTexCoord = (int*) (pData + nLengthOfTotal);
		int nSizeOfTexCoord = pData_SizeTexCoord[0];
		nLengthOfTotal += sizeof(int); 

		if(nSizeOfTexCoord > 0)
		{
			// TexCoord
			float* pData_texcoord = (float*) (pData + nLengthOfTotal);
			SFNode<class TextureCoordinate>* pTexCoord = 
				 (SFNode<class TextureCoordinate>*) pIndexedFaceSet->GetField(7);
			
			MF<Vec2f>* pValue = (MF<Vec2f>*) pTexCoord->GetNode()->GetField(0);
			Vec2f f2dummy;

			 for(i = 0, j = 0; i<nSizeOfTexCoord; i++)
			{
				f2dummy.x = pData_texcoord[j++];
				f2dummy.y = pData_texcoord[j++];
				pValue->SetAt(i, f2dummy, FALSE);
			}
			nLengthOfTotal += sizeof(float)*nSizeOfTexCoord; 
    
			// Size of TexCoordIndex
			int* pData_SizeTexCoordIndex = (int*) (pData + nLengthOfTotal);
			int nSizeOfTexCoordIndex = pData_SizeTexCoordIndex[0];
			nLengthOfTotal += sizeof(int); 

			// TexCoordIndex
			if(nSizeOfTexCoordIndex > 0)
			{
				int* pData_texcoordIndex = (int*) (pData + nLengthOfTotal);
				MF<int>* pTexCoordIndex = (MF<int>*) pIndexedFaceSet->GetField(17);
			
				for(i = 0; i< nSizeOfTexCoordIndex; i++)
				{
					pTexCoordIndex->SetAt(i, pData_texcoordIndex[i], FALSE);
				}
				nLengthOfTotal += sizeof(int)*nSizeOfTexCoordIndex; 
			}
		}
	}// if(pIndexedFaceSet)
}
#endif SAIT_CONFORMANCE_2003

// INT KM 2007 12 10
void IndexedFaceSetProxy::RenderFAMC (DWORD dwTime)
{
	IndexedFaceSet* pIndexedFaceSet = (IndexedFaceSet*) GetMediaObject();
	if(pIndexedFaceSet)
	{
	    ::glDisable(GL_CULL_FACE);

		unsigned char * pIndex = m_Data;
		unsigned int * pIndexUINT = (unsigned int *) pIndex;
		unsigned int nCoord      = (*pIndexUINT); pIndexUINT++;
		unsigned int nNormal     = (*pIndexUINT); pIndexUINT++;
		unsigned int nColor      = (*pIndexUINT); pIndexUINT++;
		unsigned int nAttributes = (*pIndexUINT); pIndexUINT++;

		unsigned int nFrames = (*pIndexUINT); pIndexUINT++;

		unsigned int maxTime = pIndexUINT[nFrames-2];
		unsigned int timeRender = (unsigned int) ((dwTime*5)%maxTime);
		


		// we compute the previous and the next frames for interpolation
		unsigned int * pIndexFramesUINT = pIndexUINT;
		
		unsigned int prev = (*pIndexFramesUINT); pIndexFramesUINT++;
		unsigned int next = 0;
		int frame = 0;
		for (frame = 1; frame < nFrames-1; frame++) {
			next = (*pIndexFramesUINT); pIndexFramesUINT++;
			if ( (timeRender>=prev) && (timeRender<=next) ) {
				break;
			}
			prev = next;
		}
		pIndexUINT += nFrames-1;
		// get layered decomposition
		int * pIndexINT = (int *) pIndexUINT;
		int numberOfDecomposedLayers = (*pIndexINT);   pIndexINT++;
		int numberOfEncodedLayers = (*pIndexINT);   pIndexINT++;

		int renderLayer = (int) (((dwTime*5)/maxTime) % numberOfDecomposedLayers);
//		int renderLayer = (int) (dwTime%numberOfDecomposedLayers);


		if ( numberOfDecomposedLayers > 0) {

			for (int numberOfSpatialLayersLocal = 0; numberOfSpatialLayersLocal < numberOfDecomposedLayers; numberOfSpatialLayersLocal++) {

				int Td = (*pIndexINT);      pIndexINT++;

				if (numberOfSpatialLayersLocal != renderLayer) {
					pIndexINT += Td*3;
				}
				else {
					for(int v = 0; v < pIndexedFaceSet->coordIndex.GetArray().GetLength(); v++) {
						pIndexedFaceSet->coordIndex.GetArray().SetAt(v, -1);
					}

					for(int t = 0; t < Td; t++) {
						for (int k = 0; k < 3; k++) {
							pIndexedFaceSet->coordIndex.GetArray().SetAt(t*4+k, *pIndexINT); pIndexINT++;
						}
						pIndexedFaceSet->coordIndex.GetArray().SetAt(t*4+3, -1);
					}
				}
			}
		}

		//--------------------------------------------------------------

		for (int u = 0; u < pIndexedFaceSet->GetChildrenCount(); u++) {
			MediaObject* pMOChild = pIndexedFaceSet->GetChild(u);
			if (pMOChild->GetNodeCode() == CODE_Coordinate) {// coordinate
				float * pIndexFloat = (float *) pIndexINT;
				Coordinate * coordinate = (Coordinate *) pMOChild;		
				if (nCoord > 0) {
					float * prevFrame = pIndexFloat + (frame-1) * nCoord * 3;
					float * nextFrame = pIndexFloat + frame * nCoord * 3;
	
					float w = (float) (timeRender - prev) / (float)(next-prev);
					Vec3f coord;
					for (int v = 0; v < nCoord; v++) {
						coord.x = (w * nextFrame[v*3+0] + (1.0f-w) * prevFrame[v*3+0]);
						coord.y = (w * nextFrame[v*3+1] + (1.0f-w) * prevFrame[v*3+1]);
						coord.z = (w * nextFrame[v*3+2] + (1.0f-w) * prevFrame[v*3+2]);
						coordinate->point.SetAt(v, coord, FALSE);
					}
				}
			}
			if (pMOChild->GetNodeCode() == CODE_Normal) {// Normal
				float * pIndexFloat = (float *) pIndexINT+ (nFrames-1) * nCoord * 3;
				Normal * normal = (Normal *) pMOChild;		
				if (nNormal > 0) {
					float * prevFrame = pIndexFloat + (frame-1) * nNormal * 3;
					float * nextFrame = pIndexFloat + frame * nNormal * 3;
	
					float w = (float) (timeRender - prev) / (float)(next-prev);
					Vec3f normalVect;
					for (int v = 0; v < nNormal; v++) {
						normalVect.x = (w * nextFrame[v*3+0] + (1.0f-w) * prevFrame[v*3+0]);
						normalVect.y = (w * nextFrame[v*3+1] + (1.0f-w) * prevFrame[v*3+1]);
						normalVect.z = (w * nextFrame[v*3+2] + (1.0f-w) * prevFrame[v*3+2]);
						normal->vector.SetAt(v, normalVect, FALSE);
					}
				}
			}
			if (pMOChild->GetNodeCode() == CODE_Color) {// Color
				float * pIndexFloat = (float *) pIndexINT+ (nFrames-1) * nCoord * 3+ (nFrames-1) * nNormal * 3;
				Color * color = (Color *) pMOChild;		
				if (nColor > 0) {
					float * prevFrame = pIndexFloat + (frame-1) * nColor * 3;
					float * nextFrame = pIndexFloat + frame * nColor * 3;
	
					float w = (float) (timeRender - prev) / (float)(next-prev);
					Color3f colorVect;
					for (int v = 0; v < nColor; v++) {
						colorVect.r = (w * nextFrame[v*3+0] + (1.0f-w) * prevFrame[v*3+0]);
						colorVect.g = (w * nextFrame[v*3+1] + (1.0f-w) * prevFrame[v*3+1]);
						colorVect.b = (w * nextFrame[v*3+2] + (1.0f-w) * prevFrame[v*3+2]);
						color->color.SetAt(v, colorVect, FALSE);
					}
				}
			}
		}

		TRACE("[IndexedFaceSetProxy::RenderFAMC] (%i, %i, %i)(%i, %i)(%i) RL(%i) \n", prev, dwTime, next, frame, nFrames, maxTime, renderLayer);

		pIndexedFaceSet->SetUpdateTime();

	}
}
// INT KM 2007 12 10