// -*- c++ -*-

// This software module was originally developed by Karl Anders ygard
// (Telenor R&D) in the course of development of the MPEG-4 Systems
// standard (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.	 The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.	 This copyright notice must be included in
// all copies or derivative works.
//
// For more information or to receive an up to date version of this module,
// mail karlo@nta.no or check <http://televr.fou.telenor.no/compositor/>.
//
// Copyright 1997, 1998.

#ifndef __INDEXEDFACESETPROXY_H__
#define __INDEXEDFACESETPROXY_H__

#include "..\AFX\AfxStreamProxy.h"

// class IndexedFaceSet;
// class TextureCoordinate;
//#include "Nodes.h"
//#include "NodeFld.h"

#include "MeshDecoder/mesh_decode.h" // SOLARIS PORTING - case typo

/** Indexed faceset node.

    The IndexedfacesetProxy class specifies a three dimensional mesh, located in
    origo of the local coordinate system. */

class IndexedFaceSetProxy
  : public AfxStreamProxy
{
private:
  // SB @@@@ from Node to Proxy >>>>
  ZArray<Collision*>	m_CollisionParents;
  // SB @@@@ from Node to Proxy <<<<

public:

  /// Constructor

			IndexedFaceSetProxy(IndexedFaceSet* pIndexedFaceSet);

  /// Render object

  virtual BOOL		Render(OpenGLVisualRenderer* pRenderer, DWORD dwTime);

  /// from the AfxStreamProxy
  virtual BOOL  RenderFromStream(MediaStream* poStream, int id, DWORD dwTime);
  virtual BOOL  RenderFromBuffer(SFString *buffer, int id);
  virtual BOOL  Render(DWORD dwTime);

  /// Traverse scene graph, checking for intersection

  virtual BOOL		Select(
      OpenGLVisualRenderer* pRenderer, 
      DWORD dwTime, 
      unsigned int& depth);

  /// Render object description

  virtual void		RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD dwTime);

  // Stefano Battista
  // moved variables and methods from IndexedFaceSet to IndexedFaceSetProxy
private:
  /// Automatically generated normals
  Normal * m_pAutoNormals;
  /// Array of normal indices
  MF<int> m_autoNormalIndex;
  /// Mesh decoder
  MeshDecoder *  m_pMeshDecoder;

  Matrix m_OldProjMatrix;
public:
  /// Update bounding box
  virtual void UpdateBbox();
  /// Compute Normals
  void ComputeNormals();
  /// Generate Normals
  void GenerateNormals();
  /// Compute Normal to a triple of points
  Vec3f ComputeNormal(
      const Vec3f& point_a,
		  const Vec3f& point_b,
		  const Vec3f& point_c);

private:

  // render object

  void			inlinedRenderLoop(BOOL bUseCoordIndex,
					  BOOL bUseColorIndex, BOOL bColorPerVertex,
					  BOOL bUseNormalIndex, BOOL bNormalPerVertex,
					  BOOL bUseTexCoordIndex, /* const */ TextureCoordinate* pTexCoord,
					  DWORD dwTime);

  // Time stamp of display list

  long			m_nTimeStamp;

  // Stefano Battista
  // moved methods from IndexedFaceSet to IndexedFaceSetProxy
public:
  /// Get pointer to related IndexedFaceSet object
  inline IndexedFaceSet * GetIndexedFaceSet() const
  { return (IndexedFaceSet *) GetMediaObject(); }

//ML - removed this function
//  /// Set colors
//  inline /* virtual */ void SetColor(Color* pColor)
//  { GetIndexedFaceSet()->color = pColor;
//    GetIndexedFaceSet()->SetUpdateTime(); }

  /// Get colors
  // SOLARIS PORTING - To core version 5.9rc9 - Appropriate Typecast
  inline Color * GetColor()
  { return (Color*)(MediaObject*)( GetIndexedFaceSet()->color); }

//ML - removed this function
//  /// Set coordinates
//  inline /* virtual */ void SetCoord(Coordinate* pCoord)
//  { GetIndexedFaceSet()->coord = pCoord;
//    GetIndexedFaceSet()->SetUpdateTime();    
//    UpdateBbox();
//    ComputeNormals(); }

  /// Get coordinates
  inline Coordinate* GetCoord()
  { return GetIndexedFaceSet()->coord; }

//ML - removed this function
//  /// Set normals
//  inline /* virtual */ void SetNormal(Normal* pNormal)
//  { GetIndexedFaceSet()->normal = pNormal;
//    GetIndexedFaceSet()->SetUpdateTime(); }

  /// Get normals
  inline Normal* GetNormal()
  { if ((Normal*) GetIndexedFaceSet()->normal)
      return GetIndexedFaceSet()->normal;
    else
      return m_pAutoNormals; }

//ML - removed this function
//  /// Set texture coordinates
//  inline /* virtual */ void SetTexCoord(TextureCoordinate* pTexCoord)
//  { GetIndexedFaceSet()->texCoord = pTexCoord;
//    GetIndexedFaceSet()->SetUpdateTime(); }

  /// Get texture coordinates
  //inline TextureCoordinate* GetTexCoord() //KM
  //{ return GetIndexedFaceSet()->texCoord; } //KM
  inline SFTextureCoordinateNode& GetTexCoord() //KM
  { return GetIndexedFaceSet()->texCoord; } //KM
  /// Set coordinate indices
  inline void SetCoordIndex(/* const */ MF<int>& rCoordIndex)
  { 
    GetIndexedFaceSet()->coordIndex = rCoordIndex;
    GetIndexedFaceSet()->SetUpdateTime();
    UpdateBbox();
    ComputeNormals(); }
  /// Get coordinate indices
  inline const MF<int>& GetCoordIndex() const
  { return GetIndexedFaceSet()->coordIndex; }
  /// Set color indices
  inline void SetColorIndex(/* const */ MF<int>& rColorIndex)
  { GetIndexedFaceSet()->colorIndex = rColorIndex;
    GetIndexedFaceSet()->SetUpdateTime(); }
  /// Get color indices
  inline const MF<int>& GetColorIndex() const
  { return GetIndexedFaceSet()->colorIndex; }
  /// Set normal indices
  inline void SetNormalIndex(/* const */ MF<int>& rNormalIndex)
  { GetIndexedFaceSet()->normalIndex = rNormalIndex;
    GetIndexedFaceSet()->SetUpdateTime(); }
  /// Get normal indices
  inline const MF<int>& GetNormalIndex() const
  { if ((Normal*) GetIndexedFaceSet()->normal)
      return GetIndexedFaceSet()->normalIndex;
    else
      return m_autoNormalIndex; }
  /// Set texture coordinate indices
  inline void SetTexCoordIndex(/* const */ MF<int>& rTexCoordIndex)
  { GetIndexedFaceSet()->texCoordIndex = rTexCoordIndex;
    GetIndexedFaceSet()->SetUpdateTime(); }
  /// Get texture coordinate indices
  inline const MF<int>& GetTexCoordIndex() const
  { return GetIndexedFaceSet()->texCoordIndex; }
  /// Get color per vertex
  inline BOOL GetColorPerVertex() const
  { return (Bool) GetIndexedFaceSet()->colorPerVertex; }
  /// Get normal per vertex
  inline BOOL GetNormalPerVertex() const
  { if ((Normal*) GetIndexedFaceSet()->normal)
      return (BOOL) (Bool) GetIndexedFaceSet()->normalPerVertex;
    else
      return TRUE; }
  /// Get whether object is solid
  inline BOOL GetSolid() const
  { return (Bool) GetIndexedFaceSet()->solid; }
  /// Get whether polygons are counter-clockwise
  inline BOOL GetCCW() const
  { return (Bool) GetIndexedFaceSet()->ccw; }

  /// Add collision parent of object  
  inline void AddCollisionParent(Collision * pParent)
  { m_CollisionParents += pParent; } 
  /// Remove collision parent of object
  inline BOOL RemoveCollisionParent(Collision * pParent)
  { return m_CollisionParents -= pParent; }


#if defined SAIT_CONFORMANCE_2003
	void Enable (unsigned char *pData);
#endif SAIT_CONFORMANCE_2003
// INT KM 2007 12 10
	void RenderFAMC (DWORD dwTime);
	unsigned char * m_Data;
// INT KM 2007 12 10
};

#include "IndexedFaceSetProxy.i"

#endif // __INDEXEDFACESETPROXY_H__
