// -*- c++ -*-

// This software module was originally developed by Karl Anders ygard
// (Telenor R&D), in the course of development of the MPEG-4 Systems
// standard (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// Copyright 1997.

#ifndef __INDEXEDLINESET2DPROXY_H__
#define __INDEXEDLINESET2DPROXY_H__

#include "GeometryProxy.h"

class IndexedLineSet2D;
class Color;

/// Indexed lineset2D renderer node

class IndexedLineSet2DProxy
  : public GeometryProxy
{
public:

  /// Constructor

			IndexedLineSet2DProxy(IndexedLineSet2D* pIndexedLineSet2D);

  /// Render object

  virtual BOOL		Render(OpenGLVisualRenderer* pRenderer, DWORD dwTime);

  /// traverse scene graph, checking for intersection

  virtual BOOL		Select(
      OpenGLVisualRenderer* pRenderer, 
      DWORD dwTime, 
      unsigned int& depth);

private:

  // render object

  void                  inlinedRenderLoop(BOOL bUseCI,
					  BOOL bColorPerVertex,
					  const Color* pColor);

  // Stefano Battista
  // moved methods from IndexedLineSet2D to Proxy
public:
  /// Get pointer to associated MediaObject
  inline IndexedLineSet2D * GetIndexedLineSet2D() const
  { return (IndexedLineSet2D *) GetMediaObject(); }

//ML - removed this function
//  /// Set coordinates
//  inline /* virtual */ void SetCoord(Coordinate2D * pCoord)
//  { GetIndexedLineSet2D()->coord = pCoord; }

  /// Get coordinates
  inline Coordinate2D * GetCoord()
  { return GetIndexedLineSet2D()->coord; }

//ML - removed this function
//  /// Set colors
//  inline /* virtual */ void SetColor(Color* pColor)
//  { GetIndexedLineSet2D()->color = pColor; }

  /// Get colors
  // SOLARIS PORTING - To core version 5.9rc9 - Appropriate Typecast
  inline Color * GetColor()
  { return (Color*)(MediaObject*)(GetIndexedLineSet2D()->color); }
  /// Set coordinate indices
  inline void SetCoordIndex(MF<int>& rCoordIndex)
  { GetIndexedLineSet2D()->coordIndex = rCoordIndex; }
  /// Get coordinate indices
  inline const MF<int>& GetCoordIndex() const
  { return GetIndexedLineSet2D()->coordIndex; }
  /// Set color indices
  inline void SetColorIndex(MF<int>& rColorIndex)
  { GetIndexedLineSet2D()->colorIndex = rColorIndex; }
  /// Get color indices
  inline const MF<int>& GetColorIndex() const
  { return GetIndexedLineSet2D()->colorIndex; }
  /// Set color per vertex
  inline void SetColorPerVertex(bool bColorPerVertex)
  { GetIndexedLineSet2D()->colorPerVertex = bColorPerVertex; }
  /// Get color per vertex
  inline BOOL GetColorPerVertex() const
  { return (Bool) (GetIndexedLineSet2D()->colorPerVertex); }

};

#include "IndexedLineSet2DProxy.i"

#endif // __INDEXEDLINESET2DPROXY_H__
