// This software module was originally developed by Karl Anders ygard
// (Telenor R&D), in the course of development of the MPEG-4 Systems
// standard (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// Copyright 1997.

#include "OpenGl/OpenGl.h"
#include "IndexedLineSetProxy.h"

// render object

/* virtual */ BOOL
IndexedLineSetProxy::Render(OpenGLVisualRenderer*, DWORD)
{
  IndexedLineSet* pIndexedLineSet = (IndexedLineSet*) GetMediaObject();
  // SOLARIS PORTING - To core 5.9rc9 - use GetColor()
  //const Color* color = pIndexedLineSet->color /* GetColor() */ ;
  const Color* color =  GetColor()  ;
  const MF<int>& colorIndex = pIndexedLineSet->colorIndex /* GetColorIndex() */ ;
  BOOL colorPerVertex = (BOOL) (Bool) pIndexedLineSet->colorPerVertex /* GetColorPerVertex() */ ;

  ::glPushAttrib(GL_ENABLE_BIT | GL_CURRENT_BIT);

  ::glDisable(GL_LIGHTING);
  ::glDisable(GL_TEXTURE_2D);

  if (colorIndex.GetLength() != 0)
    if (colorPerVertex)
      if (color)
	inlinedRenderLoop(TRUE, TRUE, color);
      else
	inlinedRenderLoop(TRUE, TRUE, NULL);
    else
      if (color)
	inlinedRenderLoop(TRUE, FALSE, color);
      else
	inlinedRenderLoop(TRUE, FALSE, NULL);
  else
    if (colorPerVertex)
      if (color)
	inlinedRenderLoop(FALSE, TRUE, color);
      else
	inlinedRenderLoop(FALSE, TRUE, NULL);
    else
      if (color)
	inlinedRenderLoop(FALSE, FALSE, color);
      else
	inlinedRenderLoop(FALSE, FALSE, NULL);
  
  ::glPopAttrib();

  return TRUE;
}

// traverse scene graph, checking for intersection

/* virtual */ BOOL
IndexedLineSetProxy::Select(
    OpenGLVisualRenderer*, 
    DWORD, 
    unsigned int& depth)
{
  GLuint selectBuffer[4];

  ::glSelectBuffer(4, selectBuffer);
  ::glRenderMode(GL_SELECT);

  inlinedRenderLoop(FALSE, FALSE, NULL);

  if (::glRenderMode(GL_RENDER) > 0)
    if (depth >= selectBuffer[1]) {
      depth = selectBuffer[1];

      return TRUE;
    }

  return FALSE;
}
