// This software module was originally developed by Karl Anders ygard
// (Telenor R&D), in the course of development of the MPEG-4 Systems
// standard (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// Copyright 1997.

#include "OpenGl/OpenGl.h"
#include "RectangleProxy.h"
#include "opengl/base/OpenGLVisualRenderer.h"
#include "opengl/appearance/Material2DProxy.h"


// Render object description

/* virtual */ void
RectangleProxy::RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD /* dwTime */)
{
  GLfloat color[] = { 0.0f, 0.0f, 1.0f, 0.75f};

  //satish changed this line for compilation purposes
        //have to be fixed afterwards
  //::glColor4fv(color);
  glColor4fv(color);

  RenderDescriptionBox(pRenderer, "Rectangle");
}

// Render object

/* virtual */ BOOL
RectangleProxy::Render(OpenGLVisualRenderer* pRenderer, DWORD)
{
  // class Rectangle* pRectangle = (class Rectangle*) GetMediaObject();

  static GLfloat texCoord[][2] = {{ 1.0f, 1.0f},
				  { 0.0f, 1.0f},
				  { 0.0f, 0.0f},
				  { 1.0f, 0.0f}};

  const Vec2f& size = /* pRectangle-> */ GetSize();

  float x_side = size.x / 2.0f;
  float y_side = size.y / 2.0f;

  BOOL filled = TRUE;

  if (pRenderer->GetCurrentMaterial2D())
    filled = ((Material2DProxy *) pRenderer->GetCurrentMaterial2D()->GetMediaObject()->GetRenderingObject())->GetFilled();
  
  if (filled)
    ::glBegin(GL_QUADS);
  else
    ::glBegin(GL_LINE_LOOP);
  
  //satish changed this block for compilation purposes
        //have to be fixed afterwards

 /* 
  ::glNormal3f(0.0f, 0.0f, -1.0f);
  ::glTexCoord2fv(texCoord[0]); ::glVertex2f( x_side,  y_side);
  ::glTexCoord2fv(texCoord[1]); ::glVertex2f(-x_side,  y_side);
  ::glTexCoord2fv(texCoord[2]); ::glVertex2f(-x_side, -y_side);
  ::glTexCoord2fv(texCoord[3]); ::glVertex2f( x_side, -y_side);
*/
  glNormal3f(0.0f, 0.0f, -1.0f);
  glTexCoord2fv(texCoord[0]); glVertex2f( x_side,  y_side);
  glTexCoord2fv(texCoord[1]); glVertex2f(-x_side,  y_side);
  glTexCoord2fv(texCoord[2]); glVertex2f(-x_side, -y_side);
  glTexCoord2fv(texCoord[3]); glVertex2f( x_side, -y_side);
  ::glEnd();
  
#ifdef _DEBUG
  if (pRenderer->CheckGLError("RectangleProxy:")) {
    TRACE ("size: %f, %f\n", size.x, size.y);
    TRACE ("RectangleProxy::Render() [%08x]: ", this);
  }
#endif

  return TRUE;
}

// Traverse scene graph, checking for intersection

/* virtual */ BOOL
RectangleProxy::Select(
    OpenGLVisualRenderer * /* pRenderer */, 
    DWORD /* dwTime */, 
    unsigned int & depth)
{
  GLuint selectBuffer[4];

  // class Rectangle* pRectangle = (class Rectangle*) GetMediaObject();

  const Vec2f& size = /* pRectangle-> */ GetSize();

  float x_side = size.x / 2.0f;
  float y_side = size.y / 2.0f;

  ::glSelectBuffer(4, selectBuffer);
  ::glRenderMode(GL_SELECT);

  ::glBegin(GL_QUADS);
  //satish changed this block for compilation purposes
        //have to be fixed afterwards

/*
  ::glVertex2f( x_side,  y_side);
  ::glVertex2f(-x_side,  y_side);
  ::glVertex2f(-x_side, -y_side);
  ::glVertex2f( x_side, -y_side);
*/
  glVertex2f( x_side,  y_side);
  glVertex2f(-x_side,  y_side);
  glVertex2f(-x_side, -y_side);
  glVertex2f( x_side, -y_side);
  ::glEnd();

  if (::glRenderMode(GL_RENDER) > 0)
    if (depth >= selectBuffer[1]) {
      depth = selectBuffer[1];

      return TRUE;
    }

  return FALSE;
}
