// This software module was originally developed by Kevin Barron (KPN Research), 
// Mike Coleman (Five Bats Research), Edward Cooke (Teltec Ireland), In-Dai Ha 
// (Hyundai Electronics Industries, Co.), Hae-Kwan Kim (Hyundai Electronics 
// Industries, Co.), Matthew Leditschke (CSELT), Zvi Lifshitz (VDOnet Corp.), 
// Karl Anders ygard (Telenor R&D), in the course of development of the MPEG-4 
// Systems standard (ISO/IEC 14496-1).  This software module is an 
// implementation of a part of one or more MPEG-4 Systems tools as specified by 
// the MPEG-4 Systems standard.  ISO/IEC gives users of the MPEG-4 Systems 
// standard free license to this software module or modifications thereof for 
// use in hardware or software products claiming conformance to the MPEG-4 
// Systems standard.  Those intending to use this software module in hardware or 
// software products are advised that its use may infringe existing patents.  
// The original developer of this software module and his/her company, the 
// subsequent editors and their companies, and ISO/IEC have no liability for use 
// of this software module or modifications thereof in an implementation.  
// Copyright is not released for non MPEG-4 Systems conforming products.  The 
// original developer retains full right to use the code for his/her own 
// purpose, assign or donate the code to a third party and to inhibit third 
// parties from using the code for non MPEG-4 Systems conforming products.  This 
// copyright notice must be included in all copies or derivative works.
//
// Copyright 1997.


#ifndef __RECTANGLEPROXY_H__
#define __RECTANGLEPROXY_H__

#include "GeometryProxy.h"

class Rectangle;

/// Rectangle renderer node

class RectangleProxy
  : public GeometryProxy
{
public:
  
  /// Constructor
  RectangleProxy(class Rectangle * pRectangle);
  
  /// Render object
  virtual BOOL		Render(OpenGLVisualRenderer* pRenderer, DWORD dwTime);
  
  /// Traverse scene graph, checking for intersection
  virtual BOOL		Select(
      OpenGLVisualRenderer* pRenderer, 
      DWORD dwTime, 
      unsigned int& depth);
  
  /// Render object description
  virtual void		RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD dwTime);
  
  // SB 98/12/04 moved from Rectangle
public:
  /* inline */ class Rectangle * GetRectangle()
  { return (class Rectangle *) GetMediaObject(); }

  /// Set size of rectangle
  inline void SetSize(const SF<Vec2f>& rSize)
  { GetRectangle()->size = rSize; }

  /// Get size of rectangle
  //ML - changed return type from const Vec3f& to const Vec3f
  // to remove warning about returning address of temporary variable
  inline const Vec2f GetSize() /* const */
  { return GetRectangle()->size; }

  /// Get bounding box
  inline /* virtual */ void GetBbox(Vec3f& rBboxSize, Vec3f& rBboxCenter) /* const */
  { rBboxSize = Vec3f(GetSize().x, GetSize().y, 0.0f);
    rBboxCenter = Vec3f(); }
};

#include "RectangleProxy.i"

#endif // __RECTANGLEPROXY_H__
