// This software module was originally developed by Karl Anders ygard
// (Telenor R&D) in the course of development of the MPEG-4 Systems
// standard (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// For more information or to receive an up to date version of this module,
// mail karlo@nta.no or check <http://televr.fou.telenor.no/compositor/>.
//
// Copyright 1997, 1998.

#include "OpenGl/OpenGl.h"
#include "ShapeProxy.h"
#include "opengl/appearance/AppearanceProxy.h"
#include "opengl/appearance/Material2DProxy.h"
#include "opengl/geometry/GeometryProxy.h"
#include "opengl/base/OpenGLPresenter.h"

// Render object

/* virtual */ BOOL
ShapeProxy::Render(
    const Matrix& rTransformation, 
    const CullFlag& rCullFlag,
    OpenGLVisualRenderer* pRenderer, 
    DWORD dwTime)
{

  (pRenderer->GetPresenter())->sgm->shapecount++;
    
 MediaObject *dummyMO;
 dummyMO = NULL; 
 bool renderthis;
 renderthis=false;

 if((pRenderer->GetPresenter())->sgm->sg_info.hasShadowToRender==true)
 {
  if((pRenderer->GetPresenter())->sgm->checkShapeToRender((pRenderer->GetPresenter())->sgm->shapecount,dummyMO))
	{ 
	 renderthis=true;
	} else
	{
  
	 renderthis=false;
	}

 } else
 {
 
   renderthis=true;
 }
  if(renderthis==true) {
		
  MediaObject * pGeometry = GetGeometry();
  Appearance* pAppearance = GetAppearance();
  
  if (pGeometry)
  {
    // TRACE("[ShapeProxy::Render] geometry %08x \n", pGeometry);
    if (pAppearance) 
    {
      // TRACE("[ShapeProxy::Render] appearance %08x \n", pAppearance);

      Material2D* pMaterial2D = NULL;
      LineProperties* pLineProperties = NULL;
      
      ((AppearanceProxy*) pAppearance->GetRenderingObject())->Enable(pRenderer, dwTime);
      
      if (pRenderer->GetCurrentMaterial2D())
        pMaterial2D = (Material2D*) pRenderer->GetCurrentMaterial2D()->GetMediaObject();
      
      if (pMaterial2D && ! (Bool) pMaterial2D->filled /* GetFilled() */ )
        //pLineProperties = pMaterial2D->lineProps /* GetLineProps() */ ;
        pLineProperties = (LineProperties * )( (pMaterial2D->lineProps).GetNode() );
      
      // TRACE("[ShapeProxy::Render] line props %08x \n", pLineProperties);
      if (! pLineProperties || pLineProperties->width != 0.0f)
      {
#if defined SAIT_CONFORMANCE_2003
		  if(pGeometry->GetNodeCode() == CODE_BitWrapper)
		  {
			  BitWrapperProxy* pBitWrapperProxy = (BitWrapperProxy*) pGeometry->GetRenderingObject();

			  if(pBitWrapperProxy->GetIsDecodingNeeded())
				  pBitWrapperProxy->Render(rTransformation, rCullFlag, pRenderer, dwTime);
			  else
			  {
				  MediaObject* pObject = (MediaObject*) pBitWrapperProxy->GetMediaObject();
				  IndexedFaceSet* pIndexedFaceSet = (IndexedFaceSet*) 
					  ((SFWorldNode*) pObject->GetField(0))->GetNode();
				  ((GeometryProxy*) pIndexedFaceSet->GetRenderingObject())->Render(pRenderer, dwTime);
			  }
		  }
		  else
			  // TRACE("[ShapeProxy::Render] Geometry with Appearance \n");
			  ((GeometryProxy*) pGeometry->GetRenderingObject())->Render(pRenderer, dwTime);
#else
		// TRACE("[ShapeProxy::Render] Geometry with Appearance \n");
        ((GeometryProxy*) pGeometry->GetRenderingObject())->Render(pRenderer, dwTime);
#endif SAIT_CONFORMANCE_2003
      }

      ((AppearanceProxy*) pAppearance->GetRenderingObject())->Disable(pRenderer, dwTime);
    } 
    else
    {
#if defined SAIT_CONFORMANCE_2003
		if(pGeometry->GetNodeCode() == CODE_BitWrapper)
		{
			BitWrapperProxy* pBitWrapperProxy = (BitWrapperProxy*) pGeometry->GetRenderingObject();

			if(pBitWrapperProxy->GetIsDecodingNeeded())
				pBitWrapperProxy->Render(rTransformation, rCullFlag, pRenderer, dwTime);
			else
			{
				MediaObject* pObject = (MediaObject*) pBitWrapperProxy->GetMediaObject();
				IndexedFaceSet* pIndexedFaceSet = (IndexedFaceSet*) 
					((SFWorldNode*) pObject->GetField(0))->GetNode();
				((GeometryProxy*) pIndexedFaceSet->GetRenderingObject())->Render(pRenderer, dwTime);
			}
		}
		else
			// TRACE("[ShapeProxy::Render] Geometry with Appearance \n");
			((GeometryProxy*) pGeometry->GetRenderingObject())->Render(pRenderer, dwTime);
#else
	  // TRACE("[ShapeProxy::Render] Geometry without Appearance \n");
      ((GeometryProxy*) pGeometry->GetRenderingObject())->Render(pRenderer, dwTime);
#endif
    }
  }
  
#ifdef _DEBUG
  if (pRenderer->CheckGLError("ShapeProxy:"))
    TRACE("ShapeProxy::Render() [%08x]: ", this);
#endif
  
  return NodeProxy::Render(rTransformation, rCullFlag,
    pRenderer, dwTime);
 }
}

// Traverse scene graph, checking for intersection

/* virtual */ void
ShapeProxy::Select(
    OpenGLVisualRenderer* pRenderer, 
    DWORD dwTime,
    MediaObjectProxy *& intersectingObject, 
    unsigned int& depth)
{
  MediaObject * pGeometry = GetGeometry();
  Appearance* pAppearance = GetAppearance();
  
  if (pGeometry) {
    if (pAppearance)
      ((AppearanceProxy*) pAppearance->GetRenderingObject())->Enable(pRenderer, dwTime);

#if defined SAIT_CONFORMANCE_2003
		  if(pGeometry->GetNodeCode() == CODE_BitWrapper)
		  {
			  BitWrapperProxy* pBitWrapperProxy = (BitWrapperProxy*) pGeometry->GetRenderingObject();
			  if(!pBitWrapperProxy->GetIsDecodingNeeded())
			  {
				  MediaObject* pObject = (MediaObject*) pBitWrapperProxy->GetMediaObject();
				  IndexedFaceSet* pIndexedFaceSet = (IndexedFaceSet*) 
					  ((SFWorldNode*) pObject->GetField(0))->GetNode();

				  ((GeometryProxy*) pIndexedFaceSet->GetRenderingObject())->Select(pRenderer, dwTime, depth);
			  }
		  }
		  else
			  if (((GeometryProxy*) pGeometry->GetRenderingObject())->Select(pRenderer, dwTime, depth))
				  intersectingObject = this;
#else
    if (((GeometryProxy*) pGeometry->GetRenderingObject())->Select(pRenderer, dwTime, depth))
      intersectingObject = this;
#endif SAIT_CONFORMANCE_2003

    if (pAppearance)
      ((AppearanceProxy*) pAppearance->GetRenderingObject())->Disable(pRenderer, dwTime);
  }
}

// Traverse scene graph, checking for activation of sensors

/* virtual */ void
ShapeProxy::Select(
    const Matrix& /* rTransformation */, 
    const CullFlag& /* rCullFlag */,
    OpenGLVisualRenderer* pRenderer, 
    DWORD dwTime,
    MediaObjectProxy *& intersectingObject,
    ZArray<MediaObjectProxy *>& activatedSensors, 
    unsigned int& depth,
    const ZArray<MediaObjectProxy *>& localSensors)
{
  MediaObject * pGeometry = GetGeometry();
  Appearance* pAppearance = GetAppearance();
  
  if (pGeometry) {
    if (pAppearance)
      ((AppearanceProxy*) pAppearance->GetRenderingObject())->Enable(pRenderer, dwTime);

#if defined SAIT_CONFORMANCE_2003
		  if(pGeometry->GetNodeCode() == CODE_BitWrapper)
		  {
			  BitWrapperProxy* pBitWrapperProxy = (BitWrapperProxy*) pGeometry->GetRenderingObject();
			  if(!pBitWrapperProxy->GetIsDecodingNeeded())
			  {
				  MediaObject* pObject = (MediaObject*) pBitWrapperProxy->GetMediaObject();
				  IndexedFaceSet* pIndexedFaceSet = (IndexedFaceSet*) 
					  ((SFWorldNode*) pObject->GetField(0))->GetNode();

				  if(((GeometryProxy*) pIndexedFaceSet->GetRenderingObject())->Select(pRenderer, dwTime, depth))
				  {
					  intersectingObject = this;
					  activatedSensors = localSensors;
				  }
			  }
		  }
		  else
			  if (((GeometryProxy*) pGeometry->GetRenderingObject())->Select(pRenderer, dwTime, depth))
			  {
				  intersectingObject = this;
				  activatedSensors = localSensors;
			  }
#else
    if (((GeometryProxy*) pGeometry->GetRenderingObject())->Select(pRenderer, dwTime, depth)) {
      intersectingObject = this;
      activatedSensors = localSensors;
    }
#endif SAIT_CONFORMANCE_2003

    if (pAppearance)
      ((AppearanceProxy*) pAppearance->GetRenderingObject())->Disable(pRenderer, dwTime);
  }
}

// Render object description

/* virtual */ void
ShapeProxy::RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD dwTime)
{
  int children = 
    ((GetGeometry() ? 1 : 0) +
		(GetAppearance() ? 1 : 0));
  
  GLfloat color[] = { 1.0f, 0.0f, 1.0f, 0.75f};
  
 //satish changed this line 
  //::glColor4fv(color);
  glColor4fv(color);
  
  RenderDescriptionBox(pRenderer, "Shape");
  
  ::glPushMatrix();
  
  ::glScalef(1.0f / children, 1.0f, 1.0f);
  ::glTranslatef(0.0f, -1.0f, 0.0f);


  if (GetGeometry())
  {
#if defined SAIT_CONFORMANCE_2003
	  MediaObject * pGeometry = GetGeometry();
	  if(pGeometry->GetNodeCode() == CODE_BitWrapper)
	  {
		  BitWrapperProxy* pBitWrapperProxy = (BitWrapperProxy*) pGeometry->GetRenderingObject();
		  if(!pBitWrapperProxy->GetIsDecodingNeeded())
		  {
			  MediaObject* pObject = (MediaObject*) pBitWrapperProxy->GetMediaObject();
			  IndexedFaceSet* pIndexedFaceSet = (IndexedFaceSet*) 
				  ((SFWorldNode*) pObject->GetField(0))->GetNode();

			  ((GeometryProxy*) pIndexedFaceSet->GetRenderingObject())->RenderDescription(pRenderer, dwTime);
		  }
	  }
	  else
		  ((GeometryProxy*) GetGeometry()->GetRenderingObject())->RenderDescription(pRenderer, dwTime);
#else
    ((GeometryProxy*) GetGeometry()->GetRenderingObject())->RenderDescription(pRenderer, dwTime);
    ::glTranslatef(1.0f, 0.0f, 0.0f);
#endif SAIT_CONFORMANCE_2003
  }

  if (GetAppearance())
    ((AppearanceProxy*) GetAppearance()->GetRenderingObject())->RenderDescription(pRenderer, dwTime);
  
  ::glPopMatrix();
}


// Riitta 99/04/29

#ifdef ADV_AUDIO

int ShapeProxy::FindAcousticSurfaces(
    ZArray<Shape*>& rAcousticSurfaces, 
		ZArray<Matrix>& rASurfaceTransformations,
		const Matrix& rTransform)
{
	//check if this IndexedFaceSet is attached AcousticMaterial to
	Shape* pObject = (Shape*)GetMediaObject();

	//int a = ((pObject->appearance)->material)->GetNodeCode();
	//int b = AcousticMaterial::GetCode();

  // SOLARIS PORTING - To core 5.9rc9 - Change to GetApperance
  if(((GetAppearance())->material)->GetNodeCode() == 
    AcousticMaterial::GetCode())
  {
		rAcousticSurfaces += pObject;  
		rASurfaceTransformations += rTransform;
		return 1;
	}
	else 
  {
		return 0;
  }
}

#endif // ADV_AUDIO
