// -*- c++ -*-

// This software module was originally developed by Karl Anders ygard
// (Telenor R&D) in the course of development of the MPEG-4 Systems
// standard (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// For more information or to receive an up to date version of this module,
// mail karlo@nta.no or check <http://televr.fou.telenor.no/compositor/>.
//
// Copyright 1997.

#ifndef __SHAPEPROXY_H__
#define __SHAPEPROXY_H__

#include "opengl/base/NodeProxy.h"


// SOWOO 20030714 begin
#if defined SAIT_CONFORMANCE_2003
#include "opengl/afx/BitWrapperProxy.h"
#endif SAIT_CONFORMANCE_2003
// SOWOO 20030714 end

// class Shape;
//#include "Nodes.h"

/// Shape renderer node

class ShapeProxy
: public NodeProxy
{
public:
  
  /// Constructor
  ShapeProxy(Shape* pShape);
  
  /// Render object
  virtual BOOL Render(
    const Matrix& rTransformation, 
    const CullFlag& rCullFlag,
    OpenGLVisualRenderer* pRenderer, 
    DWORD dwTime);
  
  /// Traverse scene graph, checking for intersection
  virtual void Select(
    OpenGLVisualRenderer* pRenderer, 
    DWORD dwTime,
    MediaObjectProxy *& intersectingObject, 
    unsigned int& depth);
  
  /// Traverse scene graph, checking for activation of sensors
  virtual void Select(
    const Matrix& rTransformation, 
    const CullFlag& rCullFlag,
    OpenGLVisualRenderer* pRenderer, 
    DWORD dwTime,
    MediaObjectProxy *& intersectingObject,
    ZArray<MediaObjectProxy *>& activatedSensors, 
    unsigned int& depth,
    const ZArray<MediaObjectProxy *>& localSensors);
  
  /// Render object description
  virtual void RenderDescription(
    OpenGLVisualRenderer* pRenderer, 
    DWORD dwTime);

  // Riitta: added 99/04/26
#ifdef ADV_AUDIO
  FindAcousticSurfaces(
    ZArray<Shape*>& rAcousticSurfaces, 
  	ZArray<Matrix>& rASurfaceTransformations,
    const Matrix& rTransform);
#endif

  // ***********************************************************************
  // Stefano Battista
  // moved methods from Shape to ShapeProxy
public:
  /// Get pointer to related Shape object
  inline Shape * GetShape()
  { return (Shape *) GetMediaObject(); }

  /// get appearance of geometry
  // SOLARIS PORTING - To core version 5.9rc9 - Appropriate Typecast
  inline Appearance * GetAppearance()
  { return (Appearance*)(MediaObject*)(GetShape()->appearance); }

//ML - removed this function
//  /// set appearance of geometry
//  inline void SetAppearance(Appearance* pAppearance)
//  { if (GetAppearance())
//  GetAppearance()->RemoveParent(GetShape());
//  GetShape()->appearance = pAppearance;
//  if (pAppearance)
//    pAppearance->SetParent(GetShape()); }

  /// get geometry
  inline MediaObject * GetGeometry()
  { return (MediaObject *) GetShape()->geometry; }

//ML - removed this function
//  /// set geometry
//  inline void SetGeometry(MediaObject * pGeometry)
//  { if (pGeometry != GetGeometry()) {
//    if (GetGeometry())
//      GetGeometry()->RemoveParent(GetShape());  
//    GetShape()->geometry = pGeometry;
//    if (pGeometry)
//      pGeometry->SetParent(GetShape()); }
//  }

  /// Get bounding box in local coordinate system
  inline /* virtual */ void GetBbox(Vec3f& rBboxSize, Vec3f& rBboxCenter)
  {
	  if (GetGeometry())
#if defined SAIT_CONFORMANCE_2003
	  {
		  MediaObject * pGeometry = GetGeometry();
		  if(pGeometry->GetNodeCode() == CODE_BitWrapper)
		  {
			  BitWrapperProxy* pBitWrapperProxy = (BitWrapperProxy*) pGeometry->GetRenderingObject();
			  if(!pBitWrapperProxy->GetIsDecodingNeeded())
			  {
				  MediaObject* pObject = (MediaObject*) pBitWrapperProxy->GetMediaObject();
				  IndexedFaceSet* pIndexedFaceSet = (IndexedFaceSet*) 
					  ((SFWorldNode*) pObject->GetField(0))->GetNode();

				  ((MediaObjectProxy*) pIndexedFaceSet->GetRenderingObject())->
					  GetBbox((SF<Vec3f> &) rBboxSize, (SF<Vec3f> &) rBboxCenter);
			  }
		  }
		  else
			  ((MediaObjectProxy *) GetGeometry()->GetRenderingObject()) ->
				  GetBbox((SF<Vec3f> &) rBboxSize, (SF<Vec3f> &) rBboxCenter);
	  }
#else
		  ((MediaObjectProxy *) GetGeometry()->GetRenderingObject()) ->
			GetBbox((SF<Vec3f> &) rBboxSize, (SF<Vec3f> &) rBboxCenter);
#endif SAIT_CONFORMANCE_2003
		  else
			  rBboxSize = rBboxCenter = Vec3f();
  }

  /// Get bounding box in parent's coordinate system
  inline /* virtual */ void GetTransformedBbox(Vec3f& rBboxSize, Vec3f& rBboxCenter)
  { GetBbox(rBboxSize, rBboxCenter); }
  
};

#include "ShapeProxy.i"

#endif // __SHAPEPROXY_H__
