// -*- c++ -*-

// This software module was originally developed by Karl Anders ygard
// (Telenor R&D) in the course of development of the MPEG-4 Systems
// standard (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// Copyright 1997, 1998.

#ifndef __SPHEREPROXY_H__
#define __SPHEREPROXY_H__

#include "GeometryProxy.h"

#include <GL/glu.h>

class Sphere;

/// Sphere renderer node

class SphereProxy
  : public GeometryProxy
{
public:

  /// Constructor

			SphereProxy(Sphere* pSphere);

  /// Destructor

			~SphereProxy();

  /// Render object

  virtual BOOL		Render(OpenGLVisualRenderer* pRenderer, DWORD dwTime);

  /// Traverse scene graph, checking for intersection

  virtual BOOL		Select(
      OpenGLVisualRenderer* pRenderer, 
      DWORD dwTime, 
      unsigned int& depth);

  /// Render object description

  virtual void		RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD dwTime);

private:

  // GLU quadric

  GLUquadricObj*	m_pGluQuadric;

  // Stefano Battista
  // moved from Sphere to Proxy
public:
  /// Get the pointer to the related Sphere object
  inline Sphere * GetSphere() const
  { return (Sphere *) GetMediaObject(); }
  /// Set radius of sphere
  inline void SetRadius(float fRadius)
  { GetSphere()->radius = fRadius; }
  /// Get radius of sphere
  inline float GetRadius() const
  { return GetSphere()->radius; }
  /// Get bounding box
  inline /* virtual */ void GetBbox(Vec3f& rBboxSize, Vec3f& rBboxCenter) const
  { rBboxSize = Vec3f(GetRadius() * 2, GetRadius() * 2, GetRadius() * 2);
    rBboxCenter = Vec3f(); }
};

#include "SphereProxy.i"

#endif // __SPHEREPROXY_H__
