// This software module was originally developed by Edward Cooke (Teltec
// Ireland) and Karl Anders ygard (Telenor R&D), in the course of
// development of the MPEG-4 Systems standard (ISO/IEC 14496-1).  This
// software module is an implementation of a part of one or more MPEG-4
// Systems tools as specified by the MPEG-4 Systems standard.  ISO/IEC
// gives users of the MPEG-4 Systems standard free license to this software
// module or modifications thereof for use in hardware or software products
// claiming conformance to the MPEG-4 Systems standard.  Those intending to
// use this software module in hardware or software products are advised
// that its use may infringe existing patents.  The original developer of
// this software module and his/her company, the subsequent editors and
// their companies, and ISO/IEC have no liability for use of this software
// module or modifications thereof in an implementation.  Copyright is not
// released for non MPEG-4 Systems conforming products.  The original
// developer retains full right to use the code for his/her own purpose,
// assign or donate the code to a third party and to inhibit third parties
// from using the code for non MPEG-4 Systems conforming products.  This
// copyright notice must be included in all copies or derivative works.
//
// Copyright 1998.

#include "OpenGl/OpenGl.h"
#include "TextProxy.h"
#include "opengl/base/OpenGLPresenter.h"
#include "Stream.h"

// Render object description

/* virtual */ void
TextProxy::RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD /* dwTime */)
{
  GLfloat color[] = { 0.0f, 0.0f, 1.0f, 0.75f};
  
  //satish changed this line for compilation purposes
        //have to be fixed afterwards
  //::glColor4fv(color);
  glColor4fv(color);
  
  RenderDescriptionBox(pRenderer, "Text");
}

// Render object

/* virtual */ BOOL
TextProxy::Render(OpenGLVisualRenderer* pRenderer, DWORD /* dwTime */)
{
  // Text* pText = (Text*) GetMediaObject();
  
  FontStyle* pFontStyle = /* pText-> */ GetFontStyle();
  BOOL bLeftToRight = TRUE;
  BOOL bHorizontal = TRUE;
  BOOL bTopToBottom = TRUE;
  float fSpacing = 1.0f;
  
  typedef enum {
    FIRST,
      BEGIN,
      MIDDLE,
      END
  } justify;
  
  justify major = BEGIN;
  justify minor = FIRST;
  
  // SB @@@@ Karl used MFString, Zvi used SFString...
  // VRML has MFString
  const MF<ZString>& strings = /* pText-> */ GetString();
  const MF<float>& lengths = /* pText-> */ GetLength();
  float fMaxExtent = /* pText-> */ GetMaxExtent();
  
  int numberOfStrings = strings.GetLength();
  
  float fontHeight = pRenderer->GetFontHeight();
  float scale = 1.0f;
  float height = 10.0f;
  
  float textHeight = 0.0f;
  float textWidth = 0.0f;
  
//ML - commented out the next line
//#pragma warning("M ha timestamping her")
  
  if (pFontStyle) {
    const MF<ZString>& justify = pFontStyle->justify /* GetJustify() */ ;
    const MF<ZString>& family = pFontStyle->family /* GetFamily() */ ;
    BOOL bold = FALSE;
    BOOL italic = FALSE;
    
    const char* pMajor = (justify.GetLength() >= 1) ? (const char*) justify[0] : NULL;
    const char* pMinor = (justify.GetLength() >= 2) ? (const char*) justify[1] : NULL;
    
    height = pFontStyle->size /* GetSize() */ ;
    
    if (::strcmp(pFontStyle->style /* GetStyle() */ , "BOLDITALIC") == 0) {
      bold = TRUE;
      italic = TRUE;
    } else
      if (::strcmp(pFontStyle->style /* GetStyle() */ , "BOLD") == 0)
        bold = TRUE;
      else
        if (::strcmp(pFontStyle->style /* GetStyle() */ , "ITALIC") == 0)
          italic = TRUE;
        
        for (int i = 0; i < family.GetLength(); ++i)
          if (pRenderer->SetFontFamily(family[i], bold, italic))
            break;
          
          bLeftToRight = (Bool) pFontStyle->leftToRight /* GetLeftToRight() */ ;
          bHorizontal = (Bool) pFontStyle->horizontal /* GetHorizontal() */ ;
          bTopToBottom = (Bool) pFontStyle->topToBottom /* GetTopToBottom() */ ;
          fSpacing = pFontStyle->spacing /* GetSpacing() */ ;
          
          if (pMajor)
            if (::strcmp(pMajor, "FIRST") == 0)
              major = FIRST;
            else
              if (::strcmp(pMajor, "MIDDLE") == 0)
                major = MIDDLE;
              else
                if (::strcmp(pMajor, "END") == 0)
                  major = END;
                
                if (pMinor)
                  if (::strcmp(pMinor, "BEGIN") == 0)
                    minor = BEGIN;
                  else
                    if (::strcmp(pMinor, "MIDDLE") == 0)
                      minor = MIDDLE;
                    else
                      if (::strcmp(pMinor, "END") == 0)
                        minor = END;
  }
  
  ::glPushMatrix();
  
  fSpacing *= fontHeight;
  scale = height / fontHeight;
  fMaxExtent /= scale;
  
  // compute height and width of Text
  
  if (bHorizontal) {
    textHeight = fSpacing * strings.GetLength();
    
    for (int i = strings.GetLength() - 1; i >= 0; --i) {
      float stringWidth = lengths.GetLength() > i ? lengths[i] : 0.0f;
      
      if (stringWidth <= 0.0f)
        stringWidth = pRenderer->GetStringWidth(strings[i]);
      
      if (textWidth < stringWidth)
        textWidth = stringWidth;
    }
    
    if (fMaxExtent > 0.0f)
      if (textWidth >= fMaxExtent)
        ::glScalef(fMaxExtent / textHeight, 1.0f, 1.0f);
  } else {
    textWidth = fSpacing * strings.GetLength();
    
    for (int i = strings.GetLength() - 1; i >= 0; --i) {
      float stringHeight = lengths.GetLength() > i ? lengths[i] : 0.0f;
      
      if (stringHeight <= 0.0f)
        stringHeight = ::strlen(strings[i]) * fSpacing;
      
      if (textHeight < stringHeight)
        textHeight = stringHeight;
    }
    
    if (fMaxExtent > 0.0f)
      if (textHeight >= fMaxExtent)
        ::glScalef(1.0f, fMaxExtent / textHeight, 1.0f);
  }
  
  ::glScalef(scale, scale, 1.0f);
  
  if (bHorizontal) {
    if (minor == MIDDLE)
      if (bTopToBottom)
        ::glTranslatef(0.0f, textHeight / 2.0f, 0.0f);
      else
        ::glTranslatef(0.0f, -textHeight / 2.0f, 0.0f);
      else
        if (minor == END)
          if (bTopToBottom)
            ::glTranslatef(0.0f, textHeight, 0.0f);
          else
            ::glTranslatef(0.0f, -textHeight, 0.0f);
          
          if ((bTopToBottom && minor == FIRST) ||
            !bTopToBottom)
            ::glTranslatef(0.0f, pRenderer->GetFontBaseline(), 0.0f);
          else
            ::glTranslatef(0.0f, pRenderer->GetFontBaseline() - fSpacing, 0.0f);
  } else {
    if (minor == MIDDLE)
      if (bLeftToRight)
        ::glTranslatef(-textWidth / 2.0f, 0.0f, 0.0f);
      else
        ::glTranslatef(textWidth / 2.0f, 0.0f, 0.0f);
      else
        if (minor == END)
          if (bLeftToRight)
            ::glTranslatef(-textWidth, 0.0f, 0.0f);
          else
            ::glTranslatef(textWidth, 0.0f, 0.0f);
  }
  
  // Iterate over all strings
  
  for (int nIndex = 0; nIndex < numberOfStrings; ++nIndex) {
    // Check if string has changed
    
    if (m_currentString.GetLength() <= nIndex ||
      !(m_currentString[nIndex] == strings[nIndex])) {
      // String has changed, must update display list
      
      m_currentString.SetAt(nIndex, strings[nIndex]);
      
      if (m_nDisplayList.GetLength() <= nIndex ||
        !m_nDisplayList[nIndex])
        m_nDisplayList.SetAt(nIndex, ::glGenLists(1));
      
      ::glNewList(m_nDisplayList[nIndex], GL_COMPILE_AND_EXECUTE);
      
      if (!bHorizontal) {
        const ZString& string = strings[nIndex];
        int letters = string.GetLength();
        float stringHeight = letters * fSpacing;
        
        if (!bLeftToRight)
          ::glTranslatef(-fSpacing, 0.0f, 0.0f);
        
        ::glPushMatrix();
        
        if (lengths.GetLength() > nIndex && lengths[nIndex] > 0.0f)
          ::glScalef(1.0f, lengths[nIndex] / (stringHeight * scale), 1.0f);
        
        ::glTranslatef(0.0f, pRenderer->GetFontBaseline(), 0.0f);
        
        if (major == MIDDLE)
          if (bTopToBottom)
            ::glTranslatef(0.0f, stringHeight / 2.0f, 0.0f);
          else
            ::glTranslatef(0.0f, -stringHeight / 2.0f, 0.0f);
          else
            if (major == END)
              if (!bTopToBottom)
                ::glTranslatef(0.0f, -stringHeight, 0.0f);
              else
                ::glTranslatef(0.0f, stringHeight, 0.0f);
              
              for (int nVertical = 0; nVertical < letters; nVertical++) {
                ZString letter;
                
                letter += string[nVertical];
                
                if (bTopToBottom)
                  ::glTranslatef(0.0f, -fSpacing, 0.0f);
                
                ::glPushMatrix();
                if (!bLeftToRight)
                  ::glTranslatef(fontHeight - pRenderer->GetStringWidth(letter), 0.0f, 0.0f);
                pRenderer->RenderString(letter);
                ::glPopMatrix();  
                
                if (!bTopToBottom)
                  ::glTranslatef(0.0f, fSpacing, 0.0f);
              }
              ::glPopMatrix();
              
              if (bLeftToRight)
                ::glTranslatef(fSpacing, 0.0f, 0.0f);
      } else {
        float stringWidth = pRenderer->GetStringWidth(strings[nIndex]);
        ZString string;
        
        ::glPushMatrix();
        
        if (lengths.GetLength() > nIndex && lengths[nIndex] > 0.0f)
          ::glScalef(lengths[nIndex] / (stringWidth * scale), 1.0f, 1.0f);
        
        if (bHorizontal && !bLeftToRight)
          for (int nChar = strings[nIndex].GetLength() - 1; nChar >= 0; --nChar)
            string += strings[nIndex][nChar];
          else
            string = strings[nIndex];
          
          if (major == MIDDLE)
            ::glTranslatef(-stringWidth / 2.0f, 0.0f, 0.0f);
          else
            if ((bLeftToRight && major == END) ||
              (!bLeftToRight && (major == FIRST ||
              major == BEGIN)))
              ::glTranslatef(-stringWidth, 0.0f, 0.0f);
            
            pRenderer->RenderString(string);
            
            ::glPopMatrix();
            
            if (!bTopToBottom)
              ::glTranslatef(0.0f, fSpacing, 0.0f);
            else
              ::glTranslatef(0.0f, -fSpacing, 0.0f);
      }
      
      ::glEndList();
    } else
      // String didn't change; use old display list
      
      ::glCallList(m_nDisplayList[nIndex]);
  }
  
  ::glPopMatrix();
  
#ifdef _DEBUG
  if (pRenderer->CheckGLError("TextProxy:"))
    TRACE ("TextProxy::Render() [%08x]: ", this);
#endif
  
  return TRUE;
}

// Traverse scene graph, checking for intersection

/* virtual */ BOOL
TextProxy::Select(
    OpenGLVisualRenderer* pRenderer, 
    DWORD dwTime, 
    unsigned int& depth)
{
  GLuint selectBuffer[4];
  
  ::glSelectBuffer(4, selectBuffer);
  ::glRenderMode(GL_SELECT);
  
  Render(pRenderer, dwTime);
  
  if (::glRenderMode(GL_RENDER) > 0)
    if (depth >= selectBuffer[1]) {
      depth = selectBuffer[1];
      
      return TRUE;
    }
    
    return FALSE;
}
