// -*- c++ -*-

// This software module was originally developed by Edward Cooke (Teltec
// Ireland) and Karl Anders ygard (Telenor R&D), in the course of
// development of the MPEG-4 Systems standard (ISO/IEC 14496-1).  This
// software module is an implementation of a part of one or more MPEG-4
// Systems tools as specified by the MPEG-4 Systems standard.  ISO/IEC
// gives users of the MPEG-4 Systems standard free license to this software
// module or modifications thereof for use in hardware or software products
// claiming conformance to the MPEG-4 Systems standard.  Those intending to
// use this software module in hardware or software products are advised
// that its use may infringe existing patents.  The original developer of
// this software module and his/her company, the subsequent editors and
// their companies, and ISO/IEC have no liability for use of this software
// module or modifications thereof in an implementation.  Copyright is not
// released for non MPEG-4 Systems conforming products.  The original
// developer retains full right to use the code for his/her own purpose,
// assign or donate the code to a third party and to inhibit third parties
// from using the code for non MPEG-4 Systems conforming products.  This
// copyright notice must be included in all copies or derivative works.
//
// Copyright 1998.

#ifndef __TEXTPROXY_H__
#define __TEXTPROXY_H__

#include "opengl/geometry/GeometryProxy.h"
#include "ztl.h"

class Text;

/// Text renderer class

class TextProxy
  : public GeometryProxy
{
public:

  /// Constructor

			TextProxy(Text* pText);

  /// Destructor

			~TextProxy();

  /// Render object

  virtual BOOL		Render(OpenGLVisualRenderer* pRenderer, DWORD dwTime);

  /// Traverse scene graph, checking for intersection

  virtual BOOL		Select(
      OpenGLVisualRenderer* pRenderer, 
      DWORD dwTime, 
      unsigned int& depth);

  /// Render object description

  virtual void		RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD dwTime);

private:

  /// Displaylists for rendered text

  ZArray<int>		m_nDisplayList;

  /// Current rendered string

  MF<ZString>		m_currentString;

  // SB 98/12/04 moved from Text
public:
  inline Text * GetText()
  { return (Text *) GetMediaObject(); }
  inline FontStyle * GetFontStyle() /* const */
  { return (FontStyle*) (MediaObject *) GetText()->fontStyle; }
  // Get string
  inline const MF<ZString>& GetString() /* const */
  { return GetText()->string; }
  // Get length
  inline const MF<float>& GetLength() /* const */
  { return GetText()->length; }
  // Get max extent
  inline float GetMaxExtent() /* const */
  { return GetText()->maxExtent; }

};

#include "TextProxy.i"

#endif // __TEXTPROXY_H__
