// -*- c++ -*-

// This software module was originally developed by Karl Anders ygard
// (Telenor R&D) in the course of development of the MPEG-4 Systems
// standard (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// Copyright 1998.

#ifndef __TEXTURENODE2DPROXY_H__
#define __TEXTURENODE2DPROXY_H__

#include "opengl/base/Node2DProxy.h"

// class TextureNode2D;
//#include "Nodes.h"

/// TextureNode2D renderer class

class TextureNode2DProxy
  : public Node2DProxy
{
public:

  /// Constructor
  TextureNode2DProxy(MediaObject * pTextureNode2D);

  /// Destructor
  ~TextureNode2DProxy();

  /// Traverse scene graph, checking for intersection
  virtual void		Select(
      OpenGLVisualRenderer* pRenderer, 
      DWORD dwTime,
			MediaObjectProxy *& intersectingObject, 
      unsigned int& depth);

  /// Render object description
  virtual void		RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD dwTime);

protected:

  /// Render object

  virtual BOOL		Render(const Matrix& rTransformation, const CullFlag& rCullFlag,
			       OpenGLVisualRenderer* pRenderer, DWORD dwTime);

  /// Traverse scene graph, checking for activation of sensors

  virtual void          Select(
      const Matrix& rTransformation, 
      const CullFlag& rCullFlag,
			OpenGLVisualRenderer* pRenderer, 
      DWORD dwTime,
			MediaObjectProxy *& intersectingObject,
			ZArray<MediaObjectProxy *>& activatedSensors, 
      unsigned int& depth,
			const ZArray<MediaObjectProxy *>& localSensors);

private:

  // Width of texture

  int			m_nTextureWidth;

  // Height of texture

  int			m_nTextureHeight;

  // Width of decoded image

  int			m_nWidth;

  // Height of decoded image

  int			m_nHeight;

  // Texture object

  GLuint		m_nTextureObject;

  // Make an RGB bitmap out of an YUV  

  static void		Yuv2Rgb(unsigned char* rgbdata,
				unsigned char* yptr,
				unsigned char* uptr,
				unsigned char* vptr,
				int lumcols, int lumrows,
				int chrcols, int chrrows);

  static void		Yuv4222Rgb(unsigned char* rgbdata,
				   unsigned char* yptr,
				   unsigned char* uptr,
				   unsigned char* vptr,
				   int lumcols, int lumrows);

  static BOOL initialised;

  static struct yuvconvtable {
    int u1;
    int u2;
    int v1;
    int v2;
  }			convtab[256];
  static int            cliptab[768];
};

#include "TextureNode2DProxy.i"

#endif // __TEXTURENODE2DPROXY_H__
