// This software module was originally developed by Karl Anders ygard
// (Telenor R&D) in the course of development of the MPEG-4 Systems
// standard (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// Copyright 1998.

#include "OpenGl/OpenGl.h"
#include "Presenter.h"
#include "AnchorProxy.h"
#include "opengl/base/OpenGLVisualRenderer.h"


// Render object description

/* virtual */ void
AnchorProxy::RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD dwTime)
{
  Anchor* pAnchor = (Anchor*) GetMediaObject();
  GLfloat color[] = { 0.0f, 1.0f, 0.0f, 0.75f};
  
	//satish changed this line
  //::glColor4fv (color);
  glColor4fv (color);
  
  RenderDescriptionBox(pRenderer, "Anchor");
  
  ::glPushMatrix();
  // Stefano Battista 98/11/10
  ::glScalef(1.0f / ((Group *) pAnchor)->GetChildCount(), 1.0f, 1.0f);
  ::glTranslatef(0.0f, -1.0f, 0.0f);
  
  for (MediaObject * pObject = pAnchor->GetFirstChild(); pObject; pObject = pAnchor->GetNextSibling())
  {
    ((NodeProxy*) pObject->GetRenderingObject())->RenderDescription(pRenderer, dwTime);
    ::glTranslatef(1.0f, 0.0f, 0.0f);
  }
  
  ::glPopMatrix();
}

// Activate sensor

/* virtual */ bool
AnchorProxy::Activate(
    OpenGLVisualRenderer* pRenderer,
    float /* fObjectX */, 
    float /* fObjectY */, 
    float /* fObjectZ */,
    int /* nX */, 
    int /* nY */, 
    bool bMouseActivated)
{
  // Anchor* pAnchor = (Anchor*) GetMediaObject();
  
  pRenderer->RegisterAnchor(this);
  /* pAnchor-> */ ActivateNode(bMouseActivated);
  
  return TRUE;
}

// Trigger sensor

void
AnchorProxy::Trigger(OpenGLVisualRenderer * /* pRenderer */)
{
  // Anchor* pAnchor = (Anchor*) GetMediaObject();
  
  /* pAnchor-> */ DeactivateNode(TRUE);
}

// Deactivate sensor

void
AnchorProxy::Deactivate(OpenGLVisualRenderer* pRenderer, BOOL bMouseActive)
{
  // Anchor* pAnchor = (Anchor*) GetMediaObject();
  
  if (!bMouseActive)
    pRenderer->UnregisterAnchor(this);
  
  /* pAnchor-> */ DeactivateNode(FALSE);
}

// Activate sensor
bool AnchorProxy::ActivateNode(
    bool bMouseActivated)
{
#ifdef ML_CANNOT_FIX
  Anchor * poAnchor = GetAnchor();
#endif
  /* poAnchor-> */ activated = bMouseActivated;

  //ML - changed const ZString& to SFString
#ifdef ML_CANNOT_FIX
  if (::strcmp(poAnchor->description, "") == 0)
    TRACE ("url: %s\n", (LPCSTR) (SFString) poAnchor->url[0]);
  else
    TRACE ("url: %s\n", (LPCSTR) (SFString) poAnchor->description);
#endif

  return true;
}

// Deactivate sensor
void AnchorProxy::DeactivateNode(bool bMouseOver)
{
  Anchor * poAnchor = GetAnchor();

  if (activated && bMouseOver) {

    char acUrl[256];
#ifdef ML_CANNOT_FIX
    ::strcpy( acUrl, (LPCSTR) (SFString) (poAnchor->url[0]) );
#endif

    TRACE("AnchorProxy::DeactivateNode -- jump to the anchor file \n");
    TRACE("  >>>> jump to %s \n", acUrl);

    if (::strcmp(acUrl, "") != 0)
    {
      Executive * poExecutive = poAnchor->GetExecutive();
      poExecutive->Stop();
      poExecutive;
      poExecutive->Start(acUrl);
    }
    else
    {
      TRACE("Cannot jump to a null URL \n");
    }
  }

  /* poAnchor-> */ activated = FALSE;
/********************************************************
**** commented out 99/06/16 SB

#if defined (HAVE_SNPRINTF) || defined (WIN32)
    int size = 128;
    char* command;
    command = (char*) ::malloc(size);
#if defined (unix)
    while (::snprintf(command, size, "netscape -remote 'openURL(%s)' &", (LPCSTR) (const ZString&) url) == -1)
    {
      size *= 2;
      command = (char*) ::realloc(command, size);
      if (!command)
      {
        TRACE ("Warning: ran out of memory trying to realloc() with size %d\n", size);
        return;
      }
    }
    ::system(command);
#elif defined (WIN32) && ! defined (BIFSENC)
    while (::_snprintf(command, size, "%s", (LPCSTR) (const ZString&) poAnchor->url) == -1)
    {
      size *= 2;
      command = (char*) ::realloc(command, size);
      if (!command)
      {
        TRACE("Warning: ran out of memory trying to realloc() with size %d\n", size);
        return;
      }
    }

		if (! m_pNetObj->startLoading((CString) command, (UINT) m_nMode)) 
    {
			// const char* args = "netscape";
			// const char* args1 = " -h ";

			::_flushall();
			ShellExecute(NULL, "open", command , NULL, NULL, SW_SHOWNORMAL);
			
			/&
			m_nBrowserHandle = ::_spawnlp( _P_NOWAIT, args, args1, command, NULL );
			switch (errno)   
			{
			case E2BIG:
				COMPLAIN("Argument list exceeds 1024 bytes");
				break;
			case EINVAL :
				COMPLAIN("mode argument is invalid");
				break;
			case ENOENT :
				COMPLAIN("File or path is not found");
				break;
			case ENOEXEC :
				COMPLAIN("Specified file is not executable or has invalid executable-file format");
				break;
			case ENOMEM :
				COMPLAIN("Not enough memory is available to execute new process");
				break;
			}
			&/
		}
#endif
#else
    char command[512];
    ::sprintf(command, "netscape -remote 'openURL(%s)' &", (LPCSTR) (const ZString&) url);
#endif

    ::free(command);
  } 
  else
    COMPLAIN("\n");

**** commented out 99/06/16 SB
********************************************************/

}
