// -*- c++ -*-

// This software module was originally developed by Karl Anders ygard
// (Telenor R&D) in the course of development of the MPEG-4 Systems
// standard (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// For more information or to receive an up to date version of this module,
// mail karlo@nta.no or check <http://televr.fou.telenor.no/compositor/>.
//
// Copyright 1998.

#include "OpenGl/OpenGl.h"
#include "BillboardProxy.h"
#include "opengl/base/Matrix.h"
#include "opengl/base/OpenGLPresenter.h"


// Rotate billboard

void
BillboardProxy::PushTransform(DWORD /* dwTime */)
{
  // Billboard* pBillboard = (Billboard*) GetMediaObject();

  const Vec3f& axisOfRotation = /* pBillboard-> */ GetAxisOfRotation();
  Matrix transform;
  Matrix newTransform;

  // This could probably be improved _greatly_ on 

  ::glGetFloatv(GL_MODELVIEW_MATRIX, transform);

  if (axisOfRotation.x != 0 ||
      axisOfRotation.y != 0 ||
      axisOfRotation.z != 0) {
    Vec3f modelToView(-transform[12], -transform[13], -transform[14]);
    Vec3f axis = transform.MultiplyVector(axisOfRotation);
    Vec3f x;

      // Y
      
    axis.Normalize();
    x = axis;

    newTransform[ 4] = x.x;
    newTransform[ 5] = x.y;
    newTransform[ 6] = x.z;

    // X
      
    modelToView.Normalize();
    x.Cross(modelToView);
    x.Normalize();

    newTransform[ 0] = x.x;
    newTransform[ 1] = x.y;
    newTransform[ 2] = x.z;

      // Z
      
    x.Cross(axis);
    x.Normalize();

    newTransform[ 8] = x.x;
    newTransform[ 9] = x.y;
    newTransform[10] = x.z;
  }

  // Put back translation
    
  newTransform[12] = transform[12];
  newTransform[13] = transform[13];
  newTransform[14] = transform[14];
    
  // Put back rotation
    
  newTransform[0] *= (transform.MultiplyVector(Vec3f(1, 0, 0))).GetLength();
  newTransform[5] *= (transform.MultiplyVector(Vec3f(0, 1, 0))).GetLength();
  newTransform[9] *= (transform.MultiplyVector(Vec3f(0, 0, 1))).GetLength();
    
  ::glLoadMatrixf(newTransform);
}

// Render node

/* virtual */ BOOL
BillboardProxy::Render(const Matrix& rTransformation, const CullFlag& /* rCullFlag */,
		       OpenGLVisualRenderer* pRenderer, DWORD dwTime)
{
  BOOL ret;

  ::glPushMatrix();

  PushTransform(dwTime);

#pragma message("A little shortcut here; we cannot be bothered to update rTransformation, and hardwire cullflag to COMPLETELY_INSIDE")

  ret = GroupProxy::Render(rTransformation, COMPLETELY_INSIDE,
			   pRenderer, dwTime);

  ::glPopMatrix();

  return ret;
}

// traverse scene graph, checking for intersection

/* virtual */ void
BillboardProxy::Select(
    OpenGLVisualRenderer* pRenderer, 
    DWORD dwTime,
		MediaObjectProxy*& intersectingObject, 
    unsigned int& depth)
{
  // Billboard* pBillboard = (Billboard*) GetMediaObject();

  // axisOfRotation not referenced
  // const Vec3f& axisOfRotation = /* pBillboard-> */ GetAxisOfRotation();

  ::glPushMatrix();

  PushTransform(dwTime);

  GroupProxy::Select(pRenderer, dwTime, intersectingObject, depth);

  ::glPopMatrix();
}

// Traverse scene graph, checking for activation of sensors

/* virtual */ void BillboardProxy::Select(
    const Matrix& rTransformation, 
    const CullFlag& /* rCullFlag */,
		OpenGLVisualRenderer* pRenderer, 
    DWORD dwTime,
		MediaObjectProxy*& intersectingObject,
		ZArray<MediaObjectProxy*>& activatedSensors, 
    unsigned int& depth,
		const ZArray<MediaObjectProxy*>& localSensors)
{
  ::glPushMatrix();

  PushTransform(dwTime);
    
#pragma message("A little shortcut here; we cannot be bothered to update rTransformation, and hardwire cullflag to COMPLETELY_INSIDE")

  GroupProxy::Select(
    rTransformation, 
    COMPLETELY_INSIDE,
		pRenderer, 
    dwTime,
		intersectingObject,
		activatedSensors, 
    depth,
		localSensors);

  ::glPopMatrix();
}

// Render object description

/* virtual */ void
BillboardProxy::RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD dwTime)
{
  Billboard* pBillboard = (Billboard*) GetMediaObject();

  GLfloat color[] = { 0.0f, 1.0f, 0.0f, 0.75f};

  //satish changed this line for compilation purposes
  //have to be fixed afterwards
  //::glColor4fv(color);
  glColor4fv(color);

  RenderDescriptionBox(pRenderer, "Billboard");

  ::glPushMatrix();
  // Stefano Battista 98/11/10
  ::glScalef(1.0f / ((Group *) pBillboard)->GetChildCount(), 1.0f, 1.0f);
  ::glTranslatef(0.0f, -1.0f, 0.0f);

  for (MediaObject * pObject = ((Group *) pBillboard)->GetFirstChild(); 
      pObject; pObject = ((Group *) pBillboard)->GetNextSibling())
  {
    ((NodeProxy*) pObject->GetRenderingObject())->RenderDescription(pRenderer, dwTime);
    ::glTranslatef(1.0f, 0.0f, 0.0f);
  }

  ::glPopMatrix();
}
