// -*- c++ -*-

// This software module was originally developed by Karl Anders ygard
// (Telenor R&D) in the course of development of the MPEG-4 Systems
// standard (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// For more information or to receive an up to date version of this module,
// mail karlo@nta.no or check <http://televr.fou.telenor.no/compositor/>.
//
// Copyright 1998.

#ifndef __BILLBOARDPROXY_H__
#define __BILLBOARDPROXY_H__

#include "GroupProxy.h"

// class Billboard;
//#include "Nodes.h"

/// Billboard renderer node

class BillboardProxy
  : public GroupProxy
{
public:

  /// Constructor
  BillboardProxy(/* Billboard* */ MediaObject * pBillboard);

  /// Render object and its children
  virtual BOOL Render(
    const Matrix& rTransformation, 
    const CullFlag& rCullFlag,
		OpenGLVisualRenderer* pRenderer, 
    DWORD dwTime);

  /// Traverse scene graph, checking for intersection

  virtual void Select(
    OpenGLVisualRenderer* pRenderer, 
    DWORD dwTime,
		MediaObjectProxy*& intersectingObject, 
    unsigned int& depth);

  /// Traverse scene graph, checking for activation of sensors
  virtual void Select(
    const Matrix& rTransformation, 
    const CullFlag& rCullFlag,
		OpenGLVisualRenderer* pRenderer, 
    DWORD dwTime,
		MediaObjectProxy*& intersectingObject,
		ZArray<MediaObjectProxy*>& activatedSensors, 
    unsigned int& depth,
		const ZArray<MediaObjectProxy*>& localSensors);

  /// Render object description
  virtual void RenderDescription(
    OpenGLVisualRenderer* pRenderer, 
    DWORD dwTime);

private:

  // Rotate billboard
  void                  PushTransform(DWORD dwTime);

public:
  inline Billboard * GetBillboard() const
  { return (Billboard *) GetMediaObject(); };
  /// Get axis of rotation
  // inline const Vec3f& GetAxisOfRotation() const
  inline const SF<Vec3f>& GetAxisOfRotation() const
  { return GetBillboard()->axisOfRotation; };
};

#include "BillboardProxy.i"

#endif // __BILLBOARDPROXY_H__

