// -*- c++ -*-

// This software module was originally developed by Edward Cooke (Teltec
// Ireland) in the course of development of the MPEG-4 Systems standard
// (ISO/IEC 14496-1).  This software module is an implementation of a part
// of one or more MPEG-4 Systems tools as specified by the MPEG-4 Systems
// standard.  ISO/IEC gives users of the MPEG-4 Systems standard free
// license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// Copyright 1998.

#include "OpenGl/OpenGl.h"
#include "opengl/base/Matrix.h"
#include "Executiv.h"
#include "FormProxy.h"
#include "opengl/group/Transform2DProxy.h"


// static members
int FormProxy::AL = 0;

// add node to draw list

/* virtual */ BOOL
FormProxy::Render(const Matrix& rTransformation, const CullFlag& rCullFlag,
		  OpenGLVisualRenderer* pRenderer, DWORD dwTime)
{
  // Form* pForm = (Form*) GetMediaObject();

  if (! /* pForm-> */ rendered) {
    /* pForm-> */ SetChildrenFormation();
    /* pForm-> */ rendered = TRUE;
  }
	
  return GroupProxy::Render(rTransformation, rCullFlag,
			      pRenderer, dwTime);
}

// render object description

/* virtual */ void
FormProxy::RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD dwTime)
{
  Form* pForm = (Form*) GetMediaObject();
  GLfloat color[] = { 0.0f, 1.0f, 0.0f, 0.75f};

  //satish changed this line for compilation purposes
        //have to be fixed afterwards
  //::glColor4fv(color);
  glColor4fv(color);

  RenderDescriptionBox(pRenderer, "Form");

  ::glPushMatrix();
  // Stefano Battista 98/11/10
  ::glScalef(1.0f / ((Group *) pForm)->GetChildCount(), 1.0f, 1.0f);
  ::glTranslatef(0.0f, -1.0f, 0.0f);

  for (MediaObject * pObject = pForm->GetFirstChild(); 
      pObject; pObject = pForm->GetNextSibling())
  {
    ((NodeProxy*) pObject->GetRenderingObject())->RenderDescription(pRenderer, dwTime);
    ::glTranslatef(1.0f, 0.0f, 0.0f);
  }

  ::glPopMatrix();
}


// Stefano Battista
// from Form to FormProxy

#ifndef BIFSENC

// Method applies the constraints sent to it via the ComputConstraints to
// the specific child node at nIndex. This method is only called for children and not
// groups... ComputConstraints will break groups into specific children nodes.
// bboxvalue is the modification value to be applied
// nSpace is used to deplace the child by the amount specified in the constraints
// x is 1 if this is a modification of X axis 0 if the Y axis
// positionmax is 1 if we want to modify the max of the bbox and 0 if the min of the bbox

void FormProxy::PositionBBox(float bBoxOffset, int nSpace, UINT nIndex, UINT x, UINT positionmax)
{
  Form * poForm = GetForm();

	Vec3f bbcenter_current,bbsize_current;
	Vec3f bbcenter_mod,bbsize_mod;
	float mod_x, mod_y;

	( (MediaObjectProxy*) ( ((MFGenericNode) GetChildren())[nIndex] ) ->
    GetRenderingObject() ) ->
    GetBbox((SF<Vec3f>) bbsize_current, (SF<Vec3f>) bbcenter_current);
	if (x) {
		if (positionmax)
			mod_x = bbcenter_current.x + (bbsize_current.x / 2);
		else
			mod_x = bbcenter_current.x - (bbsize_current.x / 2);
		bbcenter_current.x = nSpace + (bBoxOffset - (mod_x - bbcenter_current.x));
	}
	else {
		if (positionmax)
			mod_y = bbcenter_current.y + (bbsize_current.y / 2);
		else
			mod_y = bbcenter_current.y - (bbsize_current.y / 2);
		bbcenter_current.y = nSpace + (bBoxOffset - (mod_y - bbcenter_current.y));
	}
  // #### Currently this doesn't resize/center the bbox ... however it fuckin should !
	( (MediaObjectProxy*) ( ((MFGenericNode) GetChildren())[nIndex] ) ->
    GetRenderingObject() ) ->
    SetBbox ((SF<Vec3f>) bbsize_current, (SF<Vec3f>) bbcenter_current);

	Transform2D*	pUseTransform = new Transform2D;
	Vec2f transformedbbox(bbcenter_current.x, bbcenter_current.y);
  // BAD #### pUseTransform->SetTranslation(transformedbbox);
	pUseTransform->translation = transformedbbox;
	pUseTransform->SetExecutive(poForm->GetExecutive());
  // SB register "manually"
  // pUseTransform->RegisterObject();
  pUseTransform->SetRenderingObject(new Transform2DProxy(pUseTransform));

  Executive* pExecutive = poForm->GetExecutive();
  pExecutive->LockScene();
	pUseTransform->AddChild(/* (Node2D*) */ ((MFGenericNode) GetChildren())[nIndex]);
	poForm->RemoveChild((/* (Node2D*) */ ((MFGenericNode) GetChildren())[nIndex]));
	poForm->AddChild(/* (Node2D *) */ pUseTransform);
  pExecutive->UnlockScene();
}


// Compute the information in the constraint list
void FormProxy::SetChildrenFormation()
{
	//const MF<class ZString>& nConstraint = GetConstraintList();
	//int nNumberofConstraints = nConstraint.GetLength();

  // SB #### stuck here: "constraint" was "int", now is "string"
  /***
	for (int i = 0; i < nNumberofConstraints; i++) {
		switch(nConstraint[i]) {
			case 0:		// AL: Align Left Egdes of all constraints with Left most child
				{
				float xmin;
				// Get min x value of children
				xmin = MinChildbbox( 1 );
				// Calculate Constraints 
				i = ComputeConstraints( ++i, xmin, 0, 1, 0 );
				}
				break;

			case 1:		// AH: Align Centers Horizontally. The (xmin+xmax)/2 of constrained components 
								// become equal to the (xmin+xmax)/2 of the group of constrained components
				{
				float xmin,xmax,gcmin,gcmax;
				int j,first;

				// Get xmin and xmax of group of constrained components
				int index = ++i;
				for (j = nConstraint[index],first = 0; j != -1; j = nConstraint[++index],first++) {
					xmin = ChildBBox( index, 1, 0 ); 
					xmax = ChildBBox( index, 1, 1 );
					if (!first) {
						gcmax = xmax;
						gcmin = xmin;
					}
					gcmax = max (gcmax,xmax);
					gcmin = min (gcmin,xmin);
				}
								
				// Set center value of all constraints to center of group of constraints
				i = AlignCenters( ++i, 1, (gcmax + gcmin) );
			}
			break;
			case 2:		// AR: Align Right edges. The xmax of constrained components become equal to 
								// the xmax of the right-most component.
				{
				float xmax;
				// Get max x value of children
				xmax = MaxChildbbox( 1 );
				// Set max x value of constraints to max x value of all children
				i = ComputeConstraints( ++i, xmax, 0, 1, 1 );
				}
				break;
			case 3:		// AT: Align Top edges. The ymax of all constrained components become equal to
								// the ymax of the top-most component.
				{
				float ymax;
				// Get max y value of children
				ymax = MaxChildbbox( 0 );
				// Calculate Constraints 
				i = ComputeConstraints( ++i, ymax, 0, 0, 1 );
				}
				break;
			case 4:		// AV: Align centers Vertically. The (ymin+ymax)/2 of constrained components 
								// become equal to the (ymin+ymax)/2 of the group of constrained components.
				{
				float ymin,ymax,gcmax,gcmin;
				int j,first;

				// Get xmin and xmax of group of constrained components
				int index = ++i;
				for (j = nConstraint[index],first = 0; j != -1; j = nConstraint[++index],first++) {
					ymin = ChildBBox( index, 0, 0 ); 
					ymax = ChildBBox( index, 0, 1 );
					if (!first) {
						gcmax = ymax;
						gcmin = ymin;
					}
					gcmax = max (gcmax,ymax);
					gcmin = min (gcmin,ymin);
				}
					
				// Set center value of all constraints to center of group of constraints
				i = AlignCenters( ++i, 0, (gcmax + gcmin) );
			}
			break;
			case 5:		// AB: Align Bottom edges. The ymin of constrained components become equal to 
								// the ymin of the bottom-most component.
				{
				float ymin;
				// Get max y value of children
				ymin = MinChildbbox( 0 );
				// Calculate Constraints 
				i = ComputeConstraints( ++i, ymin, 0, 0, 0 );

				}
				break;
			case 6:		// ALspace: Align Left Egdes of second & following constraints by first + nSpace
				{
				float xmin;
				// nSpace is the distance to add to Left Alignment
				//
				// There maybe a problem with nSpace since our origin is center of view
				// in a case where xmin < 0 a nSpace of positive value causes the justification
				// to move towards the xmax...
				int nSpace = nConstraint[++i];
				// Get min x value of first constraint
				xmin = ChildBBox( ++i, 1, 0);
				// Calculate Constraints 
				// Set min x value of all following constraints to min x value of first + nSpace
				i = ComputeConstraints( ++i, xmin, nSpace, 1, 0 );
				}
				break;
			case 7:		// ARspace: Align Right edges by specified space. The xmax of the second 
								// and following components become equal to the xmax of the right-most 
								// component minus the specified space.
				{
				float xmax;
				// nSpace is the distance to add to Left Alignment
				//
				// There maybe a problem with nSpace since our origin is center of view
				// in a case where xmin < 0 a nSpace of positive value causes the justification
				// to move towards the xmax...
				int nSpace = nConstraint[++i];
				// Get max x value of first constraint
				xmax = ChildBBox( ++i, 1, 1);
				// Calculate Constraints 
				// Set max x value of all following constraints to max x value of first + nSpace
				i = ComputeConstraints( ++i, xmax, nSpace, 1, 1 );
				}
				break;
			case 8:		// ATspace: Align Top edges by specified space. The ymax of the second and 
								// following components become equal to the ymax of the top-most component 
								// minus the specified space.
				{
				float ymax;
				// nSpace is the distance to add to Left Alignment
				//
				// There maybe a problem with nSpace since our origin is center of view
				// in a case where xmin < 0 a nSpace of positive value causes the justification
				// to move towards the xmax...
				int nSpace = nConstraint[++i];
				// Get max y value of first constraint
				ymax = ChildBBox( ++i, 0, 1);
				// Calculate Constraints 
				// Set max y value of all following constraints to max y value of first + nSpace
				i = ComputeConstraints( ++i, ymax, nSpace, 0, 1 );
				}
				break;
			case 9:		// ABspace: Align Bottom edges by specified space. The ymin of the second 
								// and following components become equal to the ymin of the bottom-most 
								// component plus the specified space.
				{
				float ymin;
				// nSpace is the distance to add to Left Alignment
				//
				// There maybe a problem with nSpace since our origin is center of view
				// in a case where xmin < 0 a nSpace of positive value causes the justification
				// to move towards the xmax...
				int nSpace = nConstraint[++i];
				// Get min y value of first constraint
				ymin = ChildBBox( ++i, 0, 0);
				// Calculate Constraints 
				// Set min y value of all following constraints to min y value of first + nSpace
				i = ComputeConstraints( ++i, ymin, nSpace, 0, 0 );
				}
				break;
			case 10:	// SH: Spread Horizontally. The differences between the xmin of each component
								// and the xmax of the previous one become all equal. The first and the 
								// last component should be constrained horizontally already.
				{
				// First constrain horizontally the first and last children of list.
				int constraintindex = ++i;
				int xmax, xmin, xmaxf, xmaxs, xminf, xmins, firstposition, setdistance, distance;
				int nChildCount = GetForm()->GetChildCount();
				const MF<class ZString>& nConstraint = GetConstraintList();
				const MF<int>& nGroups = GetGroupsList();
				int nGroupIndex,nCurrentGroupindex,nGroupFound,j,first = 0;
				
				xmax = ChildBBox( constraintindex, 1, 1 );

				for ( j = nConstraint[constraintindex]; j != -1; j = nConstraint[++constraintindex]);
				j = nConstraint[--constraintindex];
				if (j > nChildCount) {
					// Get the location within the group
					nGroupIndex = j - 1 - nChildCount;
					for (nCurrentGroupindex=0, nGroupFound = 0; nGroupFound != nGroupIndex; nCurrentGroupindex++)
						if (nGroups[nCurrentGroupindex] == -1) {
							nGroupFound++;
							nCurrentGroupindex++;
						}

					// Find the corresponding children
					for ( int index = nGroups[nCurrentGroupindex]; index != -1; index = nGroups[++nCurrentGroupindex]) ;
					j = nGroups[--nCurrentGroupindex];
				}

				xmin = ChildBBox( j, 1, 0 );
				distance = xmax - xmin;
				firstposition = 1; setdistance = 0;

				for (j = nConstraint[i]; j != -1; j = nConstraint[++i]) {
				// If we have a group defined
					if (j > nChildCount) {
					// Get the location within the group
						nGroupIndex = j - 1 - nChildCount;
						for (nCurrentGroupindex=0, nGroupFound = 0; nGroupFound != nGroupIndex; nCurrentGroupindex++)
							if (nGroups[nCurrentGroupindex] == -1) 
								nGroupFound++;

						// Find the corresponding children
						for ( int index = nGroups[nCurrentGroupindex]; index != -1; index = nGroups[++nCurrentGroupindex]) {
							if (firstposition) {
								xmaxf = ChildBBox( index, 1, 1 );
								xminf = ChildBBox( index, 1, 0 );
								firstposition = 0;
							} else {
								xmaxs = ChildBBox( index, 1, 1 );
								xmins = ChildBBox( index, 1, 0 );
								firstposition = 1;
								setdistance = 1;
							}

							if (setdistance) {
								PositionBBox((xmaxf + ((xmaxs-xmins) - distance)), 0, (index -1), 1, 1 );
// The SetBbox call in PositionBBox doesn't work so following doesn't either
// but this should be fixed and then used as opposed to non-commented code
//								xmaxf = ChildBBox( index, 1, 1 );
//								xminf = ChildBBox( index, 1, 0 );
								xmaxf = xmaxf + ((xmaxs-xmins) - distance);
								xminf = xmaxf - (xmaxs-xmins);
								firstposition = 0;
								setdistance = 0;
							}
						}	
					} 
					else  {// Not a group definition 
						if (firstposition) {
							xmaxf = ChildBBox( j, 1, 1 );
							xminf = ChildBBox( j, 1, 0 );
							firstposition = 0;
						}
						else {
							xmaxs = ChildBBox( j, 1, 1 );
							xmins = ChildBBox( j, 1, 0 );
							firstposition = 1;
							setdistance = 1;
						}
						if (setdistance) {
							PositionBBox((xmaxf + ((xmaxs-xmins) - distance)), 0, (j -1), 1, 1 );
// The SetBbox call in PositionBBox doesn't work so following doesn't either
// but this should be fixed and then used as opposed to non-commented code
//							xmaxf = ChildBBox( j, 1, 1 );
//							xminf = ChildBBox( j, 1, 0 );
							xmaxf = xmaxf + ((xmaxs-xmins) - distance);
							xminf = xmaxf - (xmaxs-xmins);
							firstposition = 0;
							setdistance = 0;
						}
					}
				}
				}
				break;
/&&&
			case 11:	// Shin: Spread Horizontally in container. The differences between the xmin 
								// of each component and the xmax of the previous one become all equal.
								// References are the edges of the layout.
			case 12:	// SHspace: Spread Horizontally by specified space. The difference between 
								// the xmin of each component and the xmax of the previous one become all 
								// equal to the specified space.
			case 13:	// SV: Spread Vertically. The differences between the ymin of each component 
								// and the ymax of the previous one become all equal. The first and the last
								// component should be constrained vertically already.
			case 14:	// Svin: Spread Vertically in container. The differences between the ymin of
								// each component and the ymax of the previous one become all equal.
								// References are the edges of the layout.
			case 15:	// SVspace: Spread Vertically	by specified space. The difference between 
								// the ymin of each component and the ymax of the previous one become all
								// equal to the specified space.
			break;
&&&/		
		}
	}
  // SB #### stuck here: "constraint" was "int", now is "string"
  ***/

}


// Get Max of X (input x == 1) or Y (input x == 0) value of all boundingboxes of 
// all the children of the form node.
float FormProxy::MaxChildbbox( UINT x )
{
	float maxvalue=0.0,current;
	Vec3f bbcenter_current,bbsize_current;
	int nChildCount = GetForm()->GetChildCount();

	for (int index=0; index < nChildCount; index++) {
	  ( (MediaObjectProxy*) ( ((MFGenericNode) GetChildren())[index] ) ->
      GetRenderingObject() ) ->
      GetBbox((SF<Vec3f>) bbsize_current, (SF<Vec3f>) bbcenter_current);
		if (x) 
			current = bbcenter_current.x + (bbsize_current.x / 2);
		else 
			current = bbcenter_current.y + (bbsize_current.y / 2);
		if (!index)
			maxvalue = current;
		maxvalue = max(maxvalue, current);
	}

	return maxvalue;
}


// Get Min of X (input x == 1) or Y (input x == 0) value of all boundingboxes of 
// all the children of the form node.

float FormProxy::MinChildbbox( UINT x )
{
	float minvalue=0.0,current;
	Vec3f bbcenter_current,bbsize_current;
	int nChildCount = GetForm()->GetChildCount();

	for (int index=0; index < nChildCount; index++) {
	  ( (MediaObjectProxy*) ( ((MFGenericNode) GetChildren())[index] ) ->
      GetRenderingObject() ) ->
      GetBbox((SF<Vec3f>) bbsize_current, (SF<Vec3f>) bbcenter_current);
		if (x) 
			current = bbcenter_current.x - (bbsize_current.x / 2);
		else 
			current = bbcenter_current.y - (bbsize_current.y / 2);
		if (!index)
			minvalue = current;
		minvalue = min(minvalue, current);
	}

	return minvalue;
}


// Method applies the constraints sent to it via the SetChildrenFormation to the constraints
// which start at 'constraintindex'.
// bboxvalue is the modification value to be applied
// nSpace is used to deplace the child by the amount specified in the constraints
// x is 1 if this is a modification of X axis 0 if the Y axis
// positionmax is 1 if we want to modify the max of the bbox and 0 if the min of the bbox
//
// This method handles groups specified in the constraint list also.

int FormProxy::ComputeConstraints(
    int constraintindex, 
    float /*bboxvalue*/, 
    int /*nSpace*/, 
    UINT /*x*/, 
    UINT /*positionmax*/)
{

	//const MF<class ZString>& nConstraint = GetConstraintList();
	//const MF<int>& nGroups = GetGroupsList();
	//const int nChildCount = GetForm()->GetChildCount();
	//int nGroupIndex, nCurrentGroupindex, nGroupFound;

  // SB #### stuck here: "constraint" was "int", now is "string"
  /***
	for ( int j = nConstraint[constraintindex]; j != -1; j = nConstraint[++constraintindex]) {
	// If we have a group defined
		if (j > nChildCount) {
		// Get the location within the group
			nGroupIndex = j - 1 - nChildCount;
			for (nCurrentGroupindex=0, nGroupFound = 0; nGroupFound != nGroupIndex; nCurrentGroupindex++)
				if (nGroups[nCurrentGroupindex] == -1) 
					nGroupFound++;

			// Find the corresponding children
			for ( int index = nGroups[nCurrentGroupindex]; index != -1; index = nGroups[++nCurrentGroupindex]) {
				if(x) {
					if (positionmax)
						PositionBBox( bboxvalue, nSpace, index-1, 1, 1 );
					else
						PositionBBox( bboxvalue, nSpace, index-1, 1, 0 );
				}	else {
					if (positionmax)
						PositionBBox( bboxvalue, nSpace, index-1, 0, 1 );
					else
						PositionBBox( bboxvalue, nSpace, index-1, 0, 0 );
				}	
			}
		} 
		else  {// Not a group definition 
			if(x) {
				if (positionmax)
					PositionBBox( bboxvalue, nSpace, j-1, 1, 1 );
				else
					PositionBBox( bboxvalue, nSpace, j-1, 1, 0 );
				}	else {
				if (positionmax)
					PositionBBox( bboxvalue, nSpace, j-1, 0, 1 );
				else
					PositionBBox( bboxvalue, nSpace, j-1, 0, 0 );
				}	
		}
	}
  // SB #### stuck here: "constraint" was "int", now is "string"
  ***/
	return constraintindex;
}


// Method returns the max ( set input parameter positionmax == 1) or 
// min ( set input parameter positionmax == 0) of the x (input parameter x == 1) or
// y (input parameter y == 0) of the child of the form at index value.
//
// If the child is a group then it returns max or min of the group of children the 
// index specifies.
float FormProxy::ChildBBox(
    int childindex, 
    UINT x, 
    UINT positionmax)
{
	float current;
	Vec3f bbcenter_current,bbsize_current;
	int nChildCount = GetForm()->GetChildCount();
	//const MF<class ZString>& nConstraint = GetConstraintList();
	const MF<int>& nGroups = GetGroupsList();
	int j = childindex;//= nConstraint[childindex];
	int nGroupIndex,nCurrentGroupindex,nGroupFound/*,first = 0*/;

	// If we have a group defined
	if (j > nChildCount) {
	// Get the location within the group
		nGroupIndex = j - 1 - nChildCount;
		for (nCurrentGroupindex=0, nGroupFound = 0; nGroupFound != nGroupIndex; nCurrentGroupindex++)
			if (nGroups[nCurrentGroupindex] == -1) {
				nGroupFound++;
				nCurrentGroupindex++;
			}
		// Now have index into children of form
		j = nGroups[nCurrentGroupindex];
	}

	( (MediaObjectProxy*) ( ((MFGenericNode) GetChildren())[j - 1] ) ->
    GetRenderingObject() ) ->
    GetBbox((SF<Vec3f>) bbsize_current, (SF<Vec3f>) bbcenter_current);
	if (positionmax) {
		if (x) 
			current = bbcenter_current.x + (bbsize_current.x / 2);
		else 
			current = bbcenter_current.y + (bbsize_current.y / 2);
	} else {
		if (x) 
			current = bbcenter_current.x - (bbsize_current.x / 2);
		else 
			current = bbcenter_current.y - (bbsize_current.y / 2);
	}
		// Return bbox value... max or min depends on input parameters
	return current;

		// Add following if we want the max/min of the all the children of the group 
	  // currently the above will just get the max/min of first element in group.
/*
		// Find the corresponding children
		for ( int index = nGroups[nCurrentGroupindex]; index != -1; index = nGroups[++nCurrentGroupindex],first++) {
			((Node2D*)GetChildren()[index - 1])->GetBbox(bbcenter_current, bbsize_current);
			if (positionmax) {
				if (x) 
					current = bbcenter_current.x + (bbsize_current.x / 2);
				else 
					current = bbcenter_current.y + (bbsize_current.y / 2);
				if (!first)
					bbvalue = current;
				// Return Max of Children of Group
				bbvalue = max(bbvalue,current);
			} else {
				if (x) 
					current = bbcenter_current.x - (bbsize_current.x / 2);
				else 
					current = bbcenter_current.y - (bbsize_current.y / 2);
				if (!first)
					bbvalue = current;
				// Return min of Children of Group
				bbvalue = min(bbvalue,current);
			}
		}
		return bbvalue;

	}
	else {
		bbcenter_current = ((Node2D*)GetChildren()[(j-1)])->GetBboxCenter();
		bbsize_current = ((Node2D*)GetChildren()[(j-1)])->GetBboxSize();
		if (positionmax) {
			if (x) 
				current = bbcenter_current.x + (bbsize_current.x / 2);
			else 
				current = bbcenter_current.y + (bbsize_current.y / 2);
		} else {
			if (x) 
				current = bbcenter_current.x - (bbsize_current.x / 2);
			else 
				current = bbcenter_current.y - (bbsize_current.y / 2);
		}
		// Return bbox value... max or min depends on input parameters
		return current;

	}
*/
}


int FormProxy::AlignCenters(int constraintindex, UINT /*x*/, float /*cvalue*/)
{
	//Vec3f bbcenter_current,bbsize_current;
	//int nChildCount = GetForm()->GetChildCount();
	//const MF<class ZString>& nConstraint = GetConstraintList();
	//const MF<int>& nGroups = GetGroupsList();
	//int nGroupIndex,nCurrentGroupindex,nGroupFound,first = 0;

  // SB #### stuck here: "constraint" was "int", now is "string"
  /***
	for (int j = nConstraint[constraintindex]; j != -1; j = nConstraint[++constraintindex]) {
		// If we have a group defined
		if (j > nChildCount) {

			// Get the location within the group
			nGroupIndex = j - 1 - nChildCount;
			for (nCurrentGroupindex=0, nGroupFound = 0; nGroupFound != nGroupIndex; nCurrentGroupindex++)
				if (nGroups[nCurrentGroupindex] == -1) {
					nGroupFound++;
					nCurrentGroupindex++;
				}

			for ( int index = nGroups[nCurrentGroupindex]; index != -1; index = nGroups[++nCurrentGroupindex],first++) {
				(GetForm()->GetChildren()[index - 1])->
            GetBbox((SF<Vec3f> &) bbsize_current, (SF<Vec3f> &) bbcenter_current);
				if (x) 
					bbcenter_current.x = (cvalue) / 2;
				else
					bbcenter_current.y = (cvalue) / 2;
			}
		}
		else { // Not a group
			(GetForm()->GetChildren()[j - 1])->
          GetBbox((SF<Vec3f> &) bbsize_current, (SF<Vec3f> &) bbcenter_current);
			if (x) 
				bbcenter_current.x = (cvalue) / 2;
			else
				bbcenter_current.y = (cvalue) / 2;
		}
	}
  // SB #### stuck here: "constraint" was "int", now is "string"
  ***/
	// Return index;
	return constraintindex;

}

#endif // ! BIFSENC
