// This software module was originally developed by Karl Anders ygard
// (Telenor R&D) in the course of development of the MPEG-4 Systems
// standard (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// For more information or to receive an up to date version of this module,
// mail karlo@nta.no or check <http://televr.fou.telenor.no/compositor/>.
//
// Copyright 1997, 1998.

#include "OpenGl/OpenGl.h"
#include "GroupProxy.h"
#include "Presenter.h"


// Add node to draw list

/* virtual */ BOOL
GroupProxy::Render(const Matrix& rTransformation, const CullFlag& rCullFlag,
		   OpenGLVisualRenderer* pRenderer, DWORD dwTime)
{
  Group* pGroup = (Group*) GetMediaObject();
  ZArray<MediaObjectProxy *> nodes;
  
  for (MediaObject * pObject = pGroup->GetFirstChild(); 
      pObject; pObject = pGroup->GetNextSibling())
  {
    MediaObjectProxy * pProxy = (MediaObjectProxy *) (pObject->GetRenderingObject());

    // SB 99/01/27 commented out. DrawingOrder is NOT used.
    /***
    int i = 0;
    float drawingOrder = pProxy->GetDrawingOrder();
    
    while (nodes.GetLength() > i && nodes[i]->GetDrawingOrder() >= drawingOrder)
      ++i;

    nodes.InsertAt(i, pProxy);
    ***/

    nodes.InsertAt(nodes.GetLength(), pProxy);
  }

  for (int j = 0; j < nodes.GetLength(); ++j)
  {
    nodes[j]->CullOrRender(rTransformation, rCullFlag,
			   pRenderer, dwTime);
  }
  
  return Node3DProxy::Render(rTransformation, rCullFlag,
			     pRenderer, dwTime);
}

// Traverse scene graph, checking for intersection

/* virtual */ void
GroupProxy::Select(
    OpenGLVisualRenderer* pRenderer, 
    DWORD dwTime,
		MediaObjectProxy*& intersectingObject, 
    unsigned int& depth)
{
  Group* pGroup = (Group*) GetMediaObject();

  for (MediaObject * pObject = pGroup->GetFirstChild(); pObject; pObject = pGroup->GetNextSibling())
    ((MediaObjectProxy*) pObject->GetRenderingObject())->Select(pRenderer, dwTime, intersectingObject, depth);
}

// Add node to draw list

/* virtual */ BOOL
GroupProxy::RenderLights(OpenGLVisualRenderer* pRenderer, DWORD dwTime)
{
  Group* pGroup = (Group*) GetMediaObject();

  for (MediaObject * pObject = pGroup->GetFirstChild(); pObject; pObject = pGroup->GetNextSibling())
    ((MediaObjectProxy*) pObject->GetRenderingObject())->RenderLights(pRenderer, dwTime);

  return TRUE;
}

// Render object description

/* virtual */ void
GroupProxy::RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD dwTime)
{
  Group* pGroup = (Group*) GetMediaObject();
  GLfloat color[] = { 0.0f, 1.0f, 0.0f, 0.75f};

	//satish changed this line
  //::glColor4fv(color);
  glColor4fv(color);

  RenderDescriptionBox(pRenderer, "Group");

  ::glPushMatrix();
  // Stefano Battista 98/11/10
  ::glScalef(1.0f / ((Group *) pGroup)->GetChildCount(), 1.0f, 1.0f);
  ::glTranslatef(0.0f, -1.0f, 0.0f);

  for (MediaObject * pObject = pGroup->GetFirstChild(); pObject; pObject = pGroup->GetNextSibling()) {
    ((MediaObjectProxy*) pObject->GetRenderingObject())->RenderDescription(pRenderer, dwTime);

    ::glTranslatef(1.0f, 0.0f, 0.0f);
  }

  ::glPopMatrix();
}

// Traverse scene graph, checking for activation of sensors

/* virtual */ void
GroupProxy::Select(
    const Matrix& rTransformation, 
    const CullFlag& rCullFlag,
		OpenGLVisualRenderer* pRenderer, 
    DWORD dwTime,
		MediaObjectProxy*& intersectingObject,
		ZArray<MediaObjectProxy*>& activatedSensors, 
    unsigned int& depth,
		const ZArray<MediaObjectProxy*>& localSensors)
{
  Group* pGroup = (Group*) GetMediaObject();
  ZArray<MediaObjectProxy *> nodes;
  ZArray<MediaObjectProxy *> sensors;
  
  for (MediaObject * pObject = pGroup->GetFirstChild(); pObject; pObject = pGroup->GetNextSibling())
  {
    MediaObjectProxy * pProxy = (MediaObjectProxy *) (pObject->GetRenderingObject());

    // SB 99/01/27 commented out. DrawingOrder is NOT used.
    /***
    int i = 0;
    float drawingOrder = pProxy->GetDrawingOrder();
    
    while (nodes.GetLength() > i && nodes[i]->GetDrawingOrder() >= drawingOrder)
      ++i;

    nodes.InsertAt(i, pProxy);
    ***/

    nodes.InsertAt(nodes.GetLength(), pProxy);

    if (pProxy->IsPointingDeviceSensor())
      sensors += pProxy;
  }

  // then traverse scene graph further down

  for (int j = 0; j < nodes.GetLength(); ++j)
    nodes[j]->CullOrSelect(
      rTransformation, 
      rCullFlag,
      pRenderer, 
      dwTime,
      intersectingObject,
      activatedSensors, 
      depth,
      sensors.GetLength() ? sensors : localSensors);
}

// SB 99/05/14 added for testing
/* virtual */ void GroupProxy::DumpTrace()
{
  Group * poNode = (Group *) GetMediaObject();
  poNode->GetPresenter()->Logf("RendGroup %08x \n", this);
  for (MediaObject * poChild = poNode->GetFirstChild(); 
    poChild; poChild = poNode->GetNextSibling())
      ((MediaObjectProxy *) poChild->GetRenderingObject())->DumpTrace();
}

// Riitta 99/05/06
#ifdef ADV_AUDIO 

/* virtual */ int
GroupProxy::FindSoundSources(
    ZArray<Sound*>& rSources3d, 
    ZArray<Matrix>& rTransformations3d,
    ZArray<Sound2D*>& rSources2d, 
    ZArray<Matrix>& rTransformations2d,
    ZArray<DirectiveSound*>& rDirsources, 
    ZArray<Matrix>& rDirtransformations,
    const Matrix& rTransform)
{
  Group* pGroup = (Group*) GetMediaObject();
  int ret = 0;
  
  for (MediaObject* pObject = pGroup->GetFirstChild(); 
     pObject; pObject = pGroup->GetNextSibling())
    ret += ((MediaObjectProxy*) pObject->GetRenderingObject())->
        FindSoundSources(rSources3d, rTransformations3d,
            rSources2d, rTransformations2d,
            rDirsources, rDirtransformations,
            rTransform);
  return ret;
}

/* virtual */ int
GroupProxy::FindAcousticScenes(
    ZArray<AcousticScene*>& rAcousticScenes, 
    ZArray<Matrix>& rASceneTransformations,
    const Matrix& rTransform)
{
  Group* pGroup = (Group*) GetMediaObject();
  int ret = 0;

  for (MediaObject* pObject = pGroup->GetFirstChild(); 
      pObject; pObject = pGroup->GetNextSibling())
    ret += ((MediaObjectProxy*) pObject->GetRenderingObject())->
        FindAcousticScenes(
            rAcousticScenes, 
            rASceneTransformations,
            rTransform);
  return ret;
}

// riitta: added 042699
/* virtual */ int
GroupProxy::FindAcousticSurfaces(
    ZArray<Shape*>& rAcousticSurfaces, 
    ZArray<Matrix>& rASurfaceTransformations,
    const Matrix& rTransform)
{
  Group* pGroup = (Group*) GetMediaObject();
  int ret = 0;
  
  for (MediaObject* pObject = pGroup->GetFirstChild(); 
      pObject; pObject = pGroup->GetNextSibling())
    ret += ((MediaObjectProxy*) pObject->GetRenderingObject())->
        FindAcousticSurfaces(
            rAcousticSurfaces, 
            rASurfaceTransformations,
            rTransform);
  return ret;
}

#else // ! ADV_AUDIO

/* virtual */ int
GroupProxy::FindSoundSources(
    ZArray<Sound*>& rSources3d, 
    ZArray<Matrix>& rTransformations3d,
		ZArray<Sound2D*>& rSources2d, 
    ZArray<Matrix>& rTransformations2d,
		const Matrix& rTransform)
{
  Group* pGroup = (Group*) GetMediaObject();
  int ret = 0;

  for (MediaObject* pObject = pGroup->GetFirstChild(); 
      pObject; pObject = pGroup->GetNextSibling())
    ret += ((MediaObjectProxy*) pObject->GetRenderingObject())->
        FindSoundSources(rSources3d, rTransformations3d,
            rSources2d, rTransformations2d,
            rTransform);
  return ret;
}

#endif // ADV_AUDIO
