// This software module was originally developed by Karl Anders ygard
// (Telenor R&D) in the course of development of the MPEG-4 Systems
// standard (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// For more information or to receive an up to date version of this module,
// mail karlo@nta.no or check <http://televr.fou.telenor.no/compositor/>.
//
// Copyright 1998.

#include "OpenGl/OpenGl.h"
#include "LODProxy.h"


// Add node to draw list

/* virtual */ BOOL
LODProxy::Render(const Matrix& rTransformation, const CullFlag& rCullFlag,
                 OpenGLVisualRenderer* pRenderer, DWORD dwTime)
{
  // LOD* pLOD = (LOD*) GetMediaObject();
  
  const MFNodeType<SF3DNode>& level = /* pLOD-> */ GetLevel();
  const MF<float>& range = /* pLOD-> */ GetRange();

  int levels = level.GetLength();
  int ranges = range.GetLength();
  
  if (levels > 0)
  {
    if ((ranges + 1) == levels)
    {
      MediaObjectProxy* pMediaObjectProxy = NULL;
      float distance;
      
      {
        Matrix transformation = rTransformation;
        const Vec3f& center = /* pLOD-> */ GetCenter();
        
        transformation.Translate(center.x, center.y, center.z);
        
        Vec3f translation(transformation[12], transformation[13], transformation[14]);
        
        distance = translation.GetLength();
      }
      
      for (int i = 0; i < levels - 1; ++i)
      {
        if (distance < range[i])
        {
          pMediaObjectProxy = (MediaObjectProxy *)
              (((MFGenericNode) level)[i]->GetRenderingObject());
          break;
        }
      }
      
      if (distance >= range[levels - 2])
        pMediaObjectProxy = (MediaObjectProxy*) 
            (((MFGenericNode) level)[levels - 1]->GetRenderingObject());
      
      // Render selected node
      pMediaObjectProxy->CullOrRender(rTransformation, rCullFlag,
			     pRenderer, dwTime);
    }
    else
    {
      COMPLAIN("LODProxy::Render(): incorrect length of level and range\n");
    }
  }
  
  return Node3DProxy::Render(rTransformation, rCullFlag,
    pRenderer, dwTime);
}

// Traverse scene graph, checking for intersection

/* virtual */ void LODProxy::Select(
    OpenGLVisualRenderer* pRenderer, 
    DWORD dwTime,
    MediaObjectProxy*& intersectingObject, 
    unsigned int& depth)
{
  // LOD * pLOD = (LOD*) GetMediaObject();
  const MFNodeType<SF3DNode>& level = /* pLOD-> */ GetLevel();

  for (int i = level.GetLength() - 1; i >= 0; --i)
    ((MediaObjectProxy *) (((MFGenericNode) level)[i]->GetRenderingObject()))->
        Select(
          pRenderer, 
          dwTime, 
          intersectingObject, 
          depth);
}

// Add node to draw list

/* virtual */ BOOL
LODProxy::RenderLights(OpenGLVisualRenderer* pRenderer, DWORD dwTime)
{
  // LOD* pLOD = (LOD*) GetMediaObject();
  const MFNodeType<SF3DNode>& level = /* pLOD-> */ GetLevel();

  for (int i = level.GetLength() - 1; i >= 0; --i)
    ((MediaObjectProxy*) (((MFGenericNode) level)[i]->GetRenderingObject()))->
        RenderLights(pRenderer, dwTime);

  return TRUE;
}

// Render object description

/* virtual */ void
LODProxy::RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD dwTime)
{
  // LOD* pLOD = (LOD*) GetMediaObject();
  const MFNodeType<SF3DNode>& level = /* pLOD-> */ GetLevel();
  GLfloat color[] = { 0.0f, 1.0f, 0.0f, 0.75f};

  //satish changed this line for compilation purposes
        //have to be fixed afterwards
  //::glColor4fv(color);
  glColor4fv(color);

  RenderDescriptionBox(pRenderer, "LOD");

  ::glPushMatrix();
  ::glScalef(1.0f / /* pLOD-> */ GetLevel().GetLength(), 1.0f, 1.0f);
  ::glTranslatef(0.0f, -1.0f, 0.0f);

  for (int i = level.GetLength() - 1; i >= 0; --i) {
    ((MediaObjectProxy*) (((MFGenericNode) level)[i]->GetRenderingObject()))->
        RenderDescription(pRenderer, dwTime);
    
    ::glTranslatef(1.0f, 0.0f, 0.0f);
  }

  ::glPopMatrix();
}

// Traverse scene graph, checking for activation of sensors

/* virtual */ void
LODProxy::Select(
    const Matrix& rTransformation, 
    const CullFlag& rCullFlag,
		OpenGLVisualRenderer* pRenderer, 
    DWORD dwTime,
		MediaObjectProxy*& intersectingObject,
		ZArray<MediaObjectProxy*>& activatedSensors, 
    unsigned int& depth,
		const ZArray<MediaObjectProxy*>& localSensors)
{
  // LOD* pLOD = (LOD*) GetMediaObject();
  
  const MFNodeType<SF3DNode>& level = /* pLOD-> */ GetLevel();
  const MF<float>& range = /* pLOD-> */ GetRange();
  
  int levels = level.GetLength();
  int ranges = range.GetLength();
  
  if (levels > 0)
  {
    if ((ranges + 1) == levels)
    {
      MediaObjectProxy* pMediaObjectProxy = NULL;
      float distance;

      {
        Matrix transformation = rTransformation;
        const Vec3f& center = /* pLOD-> */ GetCenter();
        transformation.Translate(center.x, center.y, center.z);
        Vec3f translation(transformation[12], transformation[13], transformation[14]);
        distance = translation.GetLength();
      }
      
      for (int i = 0; i < levels - 1; ++i)
      {
        if (distance < range[i])
        {
          pMediaObjectProxy = (MediaObjectProxy*) (((MFGenericNode) level)[i]->GetRenderingObject());
          break;
        }
      }
      
      if (distance >= range[levels - 2])
      {
        pMediaObjectProxy = (MediaObjectProxy*) (((MFGenericNode) level)[levels - 1]->GetRenderingObject());
      }

      // Render selected node
      
      pMediaObjectProxy->CullOrSelect(rTransformation, rCullFlag,
        pRenderer, dwTime,
        intersectingObject,
        activatedSensors, depth,
        localSensors);
    }
  }
  else
  {
      COMPLAIN("LODProxy::Render(): incorrect length of level and range\n");
  }
}

// Riitte 99/05/06
#ifdef ADV_AUDIO

// Find sound sources

/* virtual */ int
LODProxy::FindSoundSources(
    ZArray<Sound*>& rSources3d, 
    ZArray<Matrix>& rTransformations3d,
		ZArray<Sound2D*>& rSources2d, 
    ZArray<Matrix>& rTransformations2d,
		ZArray<DirectiveSound*>& rDirsources, 
    ZArray<Matrix>& rDirtransformations,
		const Matrix& rTransform)
{
  // LOD* pLOD = (LOD*) GetMediaObject();
  const MFNodeType<SF3DNode>& level = GetLevel();
  int ret = 0;
  
  for (int i = level.GetLength() - 1; i >= 0; --i)
    ret += ((MediaObjectProxy*) (((MFGenericNode) level)[i]->GetRenderingObject()))->
        FindSoundSources(rSources3d, rTransformations3d,
            rSources2d, rTransformations2d,
            rDirsources, rDirtransformations,
            rTransform);
  return ret;
}

LODProxy::FindAcousticScenes(
		ZArray<AcousticScene*>& rAcousticScenes, 
		ZArray<Matrix>& rASceneTransformations,
		const Matrix& rTransform)
{
  // LOD* pLOD = (LOD*) GetMediaObject();
  const MFNodeType<SF3DNode>& level = GetLevel();
  int ret = 0;
  
  for (int i = level.GetLength() - 1; i >= 0; --i)
    ret += ((MediaObjectProxy*) (((MFGenericNode) level)[i]->GetRenderingObject()))->
        FindAcousticScenes(rAcousticScenes, 
            rASceneTransformations,
            rTransform);
  return ret;
}

LODProxy::FindAcousticSurfaces(
		ZArray<Shape*>& rAcousticSurfaces, 
		ZArray<Matrix>& rASurfaceTransformations,
		const Matrix& rTransform)
{
  // LOD* pLOD = (LOD*) GetMediaObject();
  const MFNodeType<SF3DNode>& level = GetLevel();
  int ret = 0;
  
  for (int i = level.GetLength() - 1; i >= 0; --i)
    ret += ((MediaObjectProxy*) (((MFGenericNode) level)[i]->GetRenderingObject()))->
        FindAcousticSurfaces(rAcousticSurfaces, 
            rASurfaceTransformations,
            rTransform);
  return ret;
}

#else // ! ADV_AUDIO

/* virtual */ int
LODProxy::FindSoundSources(
    ZArray<Sound*>& rSources3d, 
    ZArray<Matrix>& rTransformations3d,
		ZArray<Sound2D*>& rSources2d, 
    ZArray<Matrix>& rTransformations2d,
		const Matrix& rTransform)
{
  // LOD* pLOD = (LOD*) GetMediaObject();

  const MFNodeType<SF3DNode>& level = /* pLOD-> */ GetLevel();
  int ret = 0;

  for (int i = level.GetLength() - 1; i >= 0; --i)
    ret += ((MediaObjectProxy*) (((MFGenericNode) level)[i]->GetRenderingObject()))->
        FindSoundSources(rSources3d, rTransformations3d,
				    rSources2d, rTransformations2d, rTransform);

  return ret;
}

#endif // ADV_AUDIO
