// -*- c++ -*-

// This software module was originally developed by Karl Anders ygard
// (Telenor R&D) in the course of development of the MPEG-4 Systems
// standard (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// For more information or to receive an up to date version of this module,
// mail karlo@nta.no or check <http://televr.fou.telenor.no/compositor/>.
//
// Copyright 1998.

#ifndef __LODPROXY_H__
#define __LODPROXY_H__

#include "opengl/base/Node3DProxy.h"

/// LOD renderering node

class LODProxy
  : public Node3DProxy
{
public:

  /// Constructor

			LODProxy(LOD* pLOD);

  // Stefano Battista
  // moved from LOD to Proxy
public:
  inline LOD * GetLOD()
  { return (LOD *) GetMediaObject(); }

  /// Get level
  inline const MFNodeType<SF3DNode> & GetLevel() /* const */
  { return GetLOD()->level; }

  /// Get center
  //ML - changed return type from const Vec3f& to const Vec3f
  // to remove warning about returning address of temporary variable
  inline const Vec3f GetCenter() /* const */
  { return GetLOD()->center; }

  /// Get range
  inline const MF<float> & GetRange() /* const */
  { return GetLOD()->range; }

public:
  /// Render object and its children
  virtual BOOL		Render(const Matrix& rTransformation, const CullFlag& rCullFlag,
			       OpenGLVisualRenderer* pRenderer, DWORD dwTime);

  /// Traverse scene graph, activating lights
  virtual BOOL          RenderLights(OpenGLVisualRenderer* pRenderer, DWORD dwTime);

  /// Traverse scene graph, checking for intersection
  virtual void Select(
    OpenGLVisualRenderer* pRenderer, 
    DWORD dwTime,
		MediaObjectProxy*& intersectingObject, 
    unsigned int& depth);

  /// Traverse scene graph, checking for activation of sensors
  virtual void Select(
    const Matrix& rTransformation, 
    const CullFlag& rCullFlag,
    OpenGLVisualRenderer* pRenderer, 
    DWORD dwTime,
		MediaObjectProxy*& intersectingObject,
    ZArray<MediaObjectProxy*>& activatedSensors, 
    unsigned int& depth,
		const ZArray<MediaObjectProxy*>& localSensors);

  /// Render object description
  virtual void		RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD dwTime);

// Riitta 99/05/06
#ifdef ADV_AUDIO 

  /// Find sound sources
  virtual int	FindSoundSources(
    ZArray<Sound*>& rSources3d, 
    ZArray<Matrix>& rTransformations3d,
    ZArray<Sound2D*>& rSources2d, 
    ZArray<Matrix>& rTransformations2d,
    ZArray<DirectiveSound*>& rDirsources, 
    ZArray<Matrix>& rDirtransformations,
    const Matrix& rTransform);
  
  virtual int	FindAcousticScenes(
    ZArray<AcousticScene*>& rAcousticScenes, 
    ZArray<Matrix>& rASceneTransformations,
    const Matrix& rTransform);

  virtual int FindAcousticSurfaces(
    ZArray<Shape*>& rAcousticSurfaces,				// IndexedFaceSet with AcousticMaterial
    ZArray<Matrix>& rASurfaceTransformations,				// Transformations above the AcousticMaterial
    const Matrix& rTransform);
  
#else
  
  virtual int	FindSoundSources(
    ZArray<Sound*>& rSources3d, 
    ZArray<Matrix>& rTransformations3d,
    ZArray<Sound2D*>& rSources2d, 
    ZArray<Matrix>& rTransformations2d,
    const Matrix& rTransform);

#endif // ADV_AUDIO
  
};

#include "LODProxy.i"

#endif // __LODPROXY_H__
