// -*- c++ -*-

// This software module was originally developed by Karl Anders ygard
// (Telenor R&D) in the course of development of the MPEG-4 Systems
// standard (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// Copyright 1998.

#ifndef __SWITCHPROXY_H__
#define __SWITCHPROXY_H__

#include "opengl/base/Node3DProxy.h"

/// Switch renderering node

class SwitchProxy
  : public Node3DProxy
{
public:

  /// Constructor

			SwitchProxy(Switch* pSwitch);

  /// Traverse scene graph, activating lights

  virtual BOOL          RenderLights(OpenGLVisualRenderer* pRenderer, DWORD dwTime);

  /// Traverse scene graph, checking for intersection

  virtual void Select(
      OpenGLVisualRenderer* pRenderer, 
      DWORD dwTime,
			MediaObjectProxy*& intersectingObject, 
      unsigned int& depth);

  /// Render object description

  virtual void		RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD dwTime);

#ifdef ADV_AUDIO

  /// Find Sound Sources in the sub-scene
  virtual int	FindSoundSources(
      ZArray<Sound*>& rSources3d, 
      ZArray<Matrix>& rTransformations3d,
			ZArray<Sound2D*>& rSources2d, 
      ZArray<Matrix>& rTransformations2d,
			ZArray<DirectiveSound*>& rDirsources, 
      ZArray<Matrix>& rDirtransformations,
			const Matrix& rTransform);

  virtual int	FindAcousticScenes(
			ZArray<AcousticScene*>& rAcousticScenes, 
			ZArray<Matrix>& rASceneTransformations,
			const Matrix& rTransform);

  virtual int FindAcousticSurfaces(
      ZArray<Shape*>& rAcousticSurfaces,				// IndexedFaceSet with AcousticMaterial
      ZArray<Matrix>& rASurfaceTransformations,				// Transformations above the AcousticMaterial
		  const Matrix& rTransform);

#else // ! ADV_AUDIO
  
  /// Find Sound Sources in the sub-scene
  virtual int	FindSoundSources(
      ZArray<Sound*>& rSources3d, 
      ZArray<Matrix>& rTransformations3d,
			ZArray<Sound2D*>& rSources2d, 
      ZArray<Matrix>& rTransformations2d,
			const Matrix& rTransform);

#endif // ADV_AUDIO

protected:

  /// Render object and its children

  virtual BOOL		Render(const Matrix& rTransformation, const CullFlag& rCullFlag,
			       OpenGLVisualRenderer* pRenderer, DWORD dwTime);

  /// Traverse scene graph, checking for activation of sensors

  virtual void Select(
    const Matrix& rTransformation, 
    const CullFlag& rCullFlag,
		OpenGLVisualRenderer* pRenderer, 
    DWORD dwTime,
		MediaObjectProxy*& intersectingObject,
		ZArray<MediaObjectProxy*>& activatedSensors, 
    unsigned int& depth,
		const ZArray<MediaObjectProxy*>& localSensors);

  // SB 98/12/04 moved from Switch
public:
  inline Switch * GetSwitch()
  { return (Switch *) GetMediaObject(); }
  // Get child in this switch
  inline MediaObject * GetChild(int nIndex) /* const */
  { return (MediaObject *) GetSwitch()->choice[nIndex]; }
  // Get number of children in this switch
  inline int GetChildCount() /* const */
  { return GetSwitch()->choice.GetLength(); }
  // Get active choice
  inline int GetWhichChoice() /* const */
  { return GetSwitch()->whichChoice; }

};

#include "SwitchProxy.i"

#endif // __SWITCHPROXY_H__
