// This software module was originally developed by Karl Anders ygard
// (Telenor R&D) in the course of development of the MPEG-4 Systems
// standard (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// Copyright 1997, 1998.

#include "OpenGl/OpenGl.h"
#include "Transform2DProxy.h"
#include "opengl/base/Matrix.h"
#include "opengl/base/OpenGLPresenter.h"


// Get viewpoint to world matrix

// (be warned, this has not been tested thoroughly)

/* virtual */ void
Transform2DProxy::ObjectToWorldMatrix(DWORD dwTime, Matrix& rTransform) const
{
  // Transform2D* pTransform2D = (Transform2D*) GetMediaObject();
  
  /* pTransform2D-> */ ConcatenateInverseTransform(rTransform);
  
  GroupProxy::ObjectToWorldMatrix(dwTime, rTransform);
}

// Concatenate transformation with OpenGL matrix

void
Transform2DProxy::ConcatenateOpenGLMatrix(OpenGLVisualRenderer* pRenderer, DWORD /* dwTime */)
{
  Transform2D* pTransform2D = (Transform2D*) GetMediaObject();
  
  const Vec2f& center = pTransform2D->center /* GetCenter() */ ;
  float scaleOrientation = pTransform2D->scaleOrientation /* GetScaleOrientation() */ ;
  float rotation = pTransform2D->rotationAngle /* GetRotationAngle() */ ;
  const Vec2f& scale = pTransform2D->scale /* GetScale() */ ;
  const Vec2f& translation = pTransform2D->translation /* GetTranslation() */ ;
  
  // do translation
  
  if (translation.x != 0.0f ||
    translation.y != 0.0f)
    ::glTranslatef(translation.x,
    translation.y,
    0.0f);
  
  // do temporary offset of origo
  
  if (center.x != 0.0f ||
    center.y != 0.0f)
    ::glTranslatef(center.x,
    center.y,
    0.0f);
  
  // do rotation
  
  if (rotation != 0.0f)
    ::glRotatef(rotation * 180.0f / M_PI,
    0.0f,
    0.0f,
    1.0f);
  
  if (scaleOrientation != 0.0f) {
    // do scaleRotation
    
    ::glRotatef(scaleOrientation * 180.0f / M_PI,
      0.0f,
      0.0f,
      1.0f);
    
    // do scaling
    
    if (scale.x != 1.0f ||
      scale.y != 1.0f)
      ::glScalef(scale.x,
      scale.y,
      1.0f);
    
    // undo scalerotation
    
    ::glRotatef(-scaleOrientation * 180.0f / M_PI,
      0.0f,
      0.0f,
      1.0f);
  } else 
    // do scaling
    
    if (scale.x != 1.0f ||
      scale.y != 1.0f)
      ::glScalef(scale.x,
      scale.y,
      1.0f);
    
    // undo temporary offset of origo
    
    if (center.x != 0.0f ||
      center.y != 0.0f)
      ::glTranslatef(-center.x,
      -center.y,
      0.0f);
    
#ifdef _DEBUG
    if (pRenderer->CheckGLError("TransformProxy:"))
      TRACE("TransformProxy::Render() [%08x]: ", this);
#endif
}

// render node

/* virtual */ BOOL
Transform2DProxy::Render(const Matrix& rTransformation, const CullFlag& rCullFlag,
                         OpenGLVisualRenderer* pRenderer, DWORD dwTime)
{
  // Transform2D* pTransform = (Transform2D*) GetMediaObject();

  Matrix transformation = rTransformation;
  BOOL ret;
  
  ::glPushMatrix();
  
  ConcatenateOpenGLMatrix(pRenderer, dwTime);
  
  /* pTransform-> */ ConcatenateTransform(transformation);
  ret = GroupProxy::Render(transformation, rCullFlag,
    pRenderer, dwTime);
  
  ::glPopMatrix();
  
  if (pRenderer->DrawBBoxes()) {
    Vec3f size;
    Vec3f center;
    
// SOLARIS PORTING
// Created new objects to avoid memory dereferencing
  SF<Vec3f> sf_size, sf_center;
  sf_size = (SF<Vec3f>) size;
  sf_center = (SF<Vec3f>) center;

  /* pTransform-> */ GetTransformedBbox( sf_size, sf_center);

  size = (Vec3f)sf_size;
  center = (Vec3f)sf_center;

// SOLARIS PORTING - END
    
    ::glPushAttrib(GL_FOG_BIT);
    ::glDisable(GL_FOG);
    
    if (size.x > -1) {
      size *= 0.5f;
      
      ::glPushMatrix();
      ::glTranslatef(center.x, center.y, 0.0f);
      
      ::glBegin(GL_LINE_LOOP);
	//satish changed this block for compilation purposes
  	//have to be fixed afterwards
	/*
      ::glVertex2f( size.x,  size.y);
      ::glVertex2f(-size.x,  size.y);
      ::glVertex2f(-size.x, -size.y);
      ::glVertex2f( size.x, -size.y);
	*/
      glVertex2f( size.x,  size.y);
      glVertex2f(-size.x,  size.y);
      glVertex2f(-size.x, -size.y);
      glVertex2f( size.x, -size.y);
      ::glEnd();			  
      
      ::glPopMatrix();
    } else {
      ::glBegin(GL_POINTS);
  	//satish changed this line for compilation purposes
        //have to be fixed afterwards
      //::glVertex2f(center.x, center.y);

      glVertex2f(center.x, center.y);
      ::glEnd();
    }
    
    ::glPopAttrib();
  }
  
  return ret;
}

// render lights

/* virtual */ BOOL
Transform2DProxy::RenderLights(OpenGLVisualRenderer* pRenderer, DWORD dwTime)
{
  BOOL ret;
  
  ::glPushMatrix();
  
  ConcatenateOpenGLMatrix(pRenderer, dwTime);
  
  ret = GroupProxy::RenderLights(pRenderer, dwTime);
  
  ::glPopMatrix();
  
  return ret;
}

// traverse scene graph, checking for intersection

/* virtual */ void
Transform2DProxy::Select(
  OpenGLVisualRenderer* pRenderer, 
  DWORD dwTime,
  MediaObjectProxy*& intersectingObject, 
  unsigned int& depth)
{
  ::glPushMatrix();
  
  ConcatenateOpenGLMatrix(pRenderer, dwTime);
  
  GroupProxy::Select(pRenderer, dwTime, intersectingObject, depth);
  
  ::glPopMatrix();
}

// Traverse scene graph, checking for activation of sensors

/* virtual */ void
Transform2DProxy::Select(
    const Matrix& rTransformation, 
    const CullFlag& rCullFlag,
    OpenGLVisualRenderer* pRenderer, 
    DWORD dwTime,
    MediaObjectProxy*& intersectingObject,
    ZArray<MediaObjectProxy*>& activatedSensors, 
    unsigned int& depth,
    const ZArray<MediaObjectProxy*>& localSensors)
{
  // Transform2D* pTransform = (Transform2D*) GetMediaObject();

  Matrix transformation = rTransformation;
  
  ::glPushMatrix();
  
  ConcatenateOpenGLMatrix(pRenderer, dwTime);
  
  /* pTransform-> */ ConcatenateTransform(transformation);
  GroupProxy::Select(transformation, rCullFlag,
    pRenderer, dwTime,
    intersectingObject,
    activatedSensors, depth,
    localSensors);
  
  ::glPopMatrix();
}

// Render object description

/* virtual */ void
Transform2DProxy::RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD dwTime)
{
  Transform2D* pTransform = (Transform2D*) GetMediaObject();
  GLfloat color[] = { 0.0f, 1.0f, 0.0f, 0.75f};
  
  //satish changed this line for compilation purposes
  //have to be fixed afterwards
  //::glColor4fv(color);
  glColor4fv(color);
  
  RenderDescriptionBox(pRenderer, "Transform2D");
  
  ::glPushMatrix();
  ::glScalef(1.0f / pTransform->GetChildCount(), 1.0f, 1.0f);
  ::glTranslatef(0.0f, -1.0f, 0.0f);
  
  for (MediaObject * pObject = pTransform->GetFirstChild(); pObject; 
      pObject = pTransform->GetNextSibling())
  {
    ((MediaObjectProxy*) pObject->GetRenderingObject())->RenderDescription(pRenderer, dwTime);
    ::glTranslatef(1.0f, 0.0f, 0.0f);
  }
  
  ::glPopMatrix();
}

// Concatenate inverse transformation with given matrix

void Transform2DProxy::ConcatenateInverseTransform(Matrix& rTransform) const
{
  Transform2D * poTransform2D = (Transform2D *) GetMediaObject();

  const Vec2f& rCenter = poTransform2D->center;
  const Vec2f& rScale = poTransform2D->scale;
  const Vec2f& rTranslation = poTransform2D->translation;
  
  // do temporary offset of origo
  
  if (rCenter.x != 0.0f ||
    rCenter.y != 0.0f)
    rTransform.Translate(rCenter.x,
			 rCenter.y,
       0.0f);
  
  if (poTransform2D->scaleOrientation != 0.0f) {
    // do scaleRotation
    
    rTransform.Rotate(poTransform2D->scaleOrientation,
      0.0f,
      0.0f,
      1.0f);
    
    // do inverse scaling
    
    if (rScale.x != 1.0f ||
      rScale.y != 1.0f)
      rTransform.Scale(1.0f / rScale.x,
      1.0f / rScale.y,
      1.0f);
    
    // undo scalerotation
    
    rTransform.Rotate(- poTransform2D->scaleOrientation,
      0.0f,
      0.0f,
      1.0f);
  } else 
    // do inverse scaling
    
    if (rScale.x != 1.0f ||
      rScale.y != 1.0f)
      rTransform.Scale(1.0f / rScale.x,
      1.0f / rScale.y,
      1.0f);
    
    // do inverse rotation
    
    if (poTransform2D->rotationAngle != 0.0f)
      rTransform.Rotate(- poTransform2D->rotationAngle,
      0.0f,
      0.0f,
      1.0f);
    
    // undo temporary offset of origo
    
    if (rCenter.x != 0.0f ||
      rCenter.y != 0.0f)
      rTransform.Translate(-rCenter.x,
      -rCenter.y,
      0.0f);
    
    // do inverse translation
    
    if (rTranslation.x != 0.0f ||
      rTranslation.y != 0.0f)
      rTransform.Translate(-rTranslation.x,
      -rTranslation.y,
      0.0f);
}

// Concatenate transformation with given matrix

void Transform2DProxy::ConcatenateTransform(Matrix& rTransform) const
{
  Transform2D * poTransform2D = (Transform2D *) GetMediaObject();
  
  const Vec2f& rCenter = poTransform2D->center;
  const Vec2f& rScale = poTransform2D->scale;
  const Vec2f& rTranslation = poTransform2D->translation;
  
  // do translation
  
  if (rTranslation.x != 0.0f ||
    rTranslation.y != 0.0f)
    rTransform.Translate(rTranslation.x,
			 rTranslation.y,
       0.0f);
  
  // do temporary offset of origo
  
  if (rCenter.x != 0.0f ||
    rCenter.y != 0.0f)
    rTransform.Translate(rCenter.x,
			 rCenter.y,
       0.0f);
  
  // do rotation
  
  if (poTransform2D->rotationAngle != 0.0f)
    rTransform.Rotate(poTransform2D->rotationAngle,
    0.0f,
    0.0f,
    1.0f);
  
  if (poTransform2D->scaleOrientation != 0.0f) {
    // do scaleRotation
    
    rTransform.Rotate(poTransform2D->scaleOrientation,
      0.0f,
      0.0f,
      1.0f);
    
    // do scaling
    
    if (rScale.x != 1.0f ||
      rScale.y != 1.0f)
      rTransform.Scale(rScale.x,
      rScale.y,
      1.0f);
    
    // undo scalerotation
    
    rTransform.Rotate(- poTransform2D->scaleOrientation,
      0.0f,
      0.0f,
      1.0f);
  } else 
    // do scaling
    
    if (rScale.x != 1.0f ||
      rScale.y != 1.0f)
      rTransform.Scale(rScale.x,
      rScale.y,
      1.0f);
    
    // undo temporary offset of origo
    
    if (rCenter.x != 0.0f ||
      rCenter.y != 0.0f)
      rTransform.Translate(-rCenter.x,
      -rCenter.y,
      0.0f);
    
}

// Get viewpoint to world matrix

/* virtual */ void
Transform2DProxy::ViewpointToWorldMatrix(
    DWORD dwTime, Matrix& rTransform) const
{
  ConcatenateInverseTransform(rTransform);  

  MediaObjectProxy::ViewpointToWorldMatrix(dwTime, rTransform);
}

#ifdef ADV_AUDIO

// Find sound sources

/* virtual */ int
Transform2DProxy::FindSoundSources(
    ZArray<Sound*>& rSources3d, 
    ZArray<Matrix>& rTransformations3d,
		ZArray<Sound2D*>& rSources2d, 
    ZArray<Matrix>& rTransformations2d,
		ZArray<DirectiveSound*>& rDirsources, 
    ZArray<Matrix>& rDirtransformations,
		const Matrix& rTransform)
{
//  Transform2D* pTransform = (Transform2D*) GetMediaObject();
  Matrix newTransform = rTransform;

  ConcatenateTransform(newTransform);
  
  return GroupProxy::FindSoundSources(rSources3d, rTransformations3d,
					rSources2d, rTransformations2d,
					rDirsources, rDirtransformations,
					newTransform);
}

/* virtual */ int
Transform2DProxy::FindAcousticScenes(
    ZArray<AcousticScene*>& rAcousticScenes, 
		ZArray<Matrix>& rASceneTransformations,
		const Matrix& rTransform)
{
//  Transform2D* pTransform = (Transform2D*) GetMediaObject();
  Matrix newTransform = rTransform;

  ConcatenateTransform(newTransform);
  
  return GroupProxy::FindAcousticScenes(rAcousticScenes, 
						rASceneTransformations,
						newTransform);
}

/* virtual */ int
Transform2DProxy::FindAcousticSurfaces(
    ZArray<Shape*>& rAcousticSurfaces, 
		ZArray<Matrix>& rASurfaceTransformations,
		const Matrix& rTransform)
{
//  Transform2D* pTransform = (Transform2D*) GetMediaObject();
  Matrix newTransform = rTransform;

  ConcatenateTransform(newTransform);
  
  return GroupProxy::FindAcousticSurfaces(rAcousticSurfaces, 
			rASurfaceTransformations,
			newTransform);
}

#else // ! ADV_AUDIO

/* virtual */ int
Transform2DProxy::FindSoundSources(
    ZArray<Sound*>& rSources3d, 
    ZArray<Matrix>& rTransformations3d,
    ZArray<Sound2D*>& rSources2d, 
    ZArray<Matrix>& rTransformations2d,
    const Matrix& rTransform)
{
  // Transform2D* pTransform = (Transform2D*) GetMediaObject();

  Matrix newTransform = rTransform;
  
  /* pTransform-> */ ConcatenateTransform(newTransform);
  
  return GroupProxy::FindSoundSources(rSources3d, rTransformations3d,
    rSources2d, rTransformations2d,
    newTransform);
}

#endif // ADV_AUDIO
