// -*- c++ -*-

// This software module was originally developed by Karl Anders ygard
// (Telenor R&D) in the course of development of the MPEG-4 Systems
// standard (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// For more information or to receive an up to date version of this module,
// mail karlo@nta.no or check <http://televr.fou.telenor.no/compositor/>.
//
// Copyright 1997, 1998.

#include "OpenGl/OpenGl.h"
#include "TransformProxy.h"
#include "opengl/base/Matrix.h"
#include "opengl/base/OpenGLPresenter.h"


// Get viewpoint to world matrix

/* virtual */ void
TransformProxy::ObjectToWorldMatrix(DWORD dwTime, Matrix& rTransform) const
{
  // Transform* pTransform = (Transform*) GetMediaObject();
  
  /* pTransform-> */ ConcatenateInverseTransform(rTransform);
  
  GroupProxy::ObjectToWorldMatrix(dwTime, rTransform);
}

// Concatenate transformation with OpenGL matrix

void
TransformProxy::ConcatenateOpenGLMatrix(OpenGLVisualRenderer* pRenderer, DWORD /* dwTime */)
{
  Transform* pTransform = (Transform*) GetMediaObject();
  
  const Vec3f& center = pTransform->center /* GetCenter() */ ;
  const Rotation& scaleOrientation = pTransform->scaleOrientation /* GetScaleOrientation() */ ;
  const Rotation& rotation = pTransform->rotation /* GetRotation() */ ;
  const Vec3f& scale = pTransform->scale /* GetScale() */ ;
  const Vec3f& translation = pTransform->translation /* GetTranslation() */ ;
  
  // do translation
  
  if (translation.x != 0.0f ||
    translation.y != 0.0f ||
    translation.z != 0.0f)
    ::glTranslatef(translation.x,
    translation.y,
    translation.z);
  
  // do temporary offset of origo
  
  if (center.x != 0.0f ||
    center.y != 0.0f ||
    center.z != 0.0f)
    ::glTranslatef(center.x,
    center.y,
    center.z);
  
  // do rotation
  
  if (rotation.theta != 0.0f)
    ::glRotatef(rotation.theta * 180.0f / M_PI,
    rotation.x,
    rotation.y,
    rotation.z);
  
  if (scaleOrientation.theta != 0.0f) {
    // do scaleRotation
    
    ::glRotatef(scaleOrientation.theta * 180.0f / M_PI,
      scaleOrientation.x,
      scaleOrientation.y,
      scaleOrientation.z);
    
    // do scaling
    
    if (scale.x != 1.0f ||
      scale.y != 1.0f ||
      scale.z != 1.0f)
      ::glScalef(scale.x,
      scale.y,
      scale.z);
    
    // undo scalerotation
    
    ::glRotatef(-scaleOrientation.theta * 180.0f / M_PI,
      scaleOrientation.x,
      scaleOrientation.y,
      scaleOrientation.z);
  } else 
    // do scaling
    
    if (scale.x != 1.0f ||
      scale.y != 1.0f ||
      scale.z != 1.0f)
      ::glScalef(scale.x,
      scale.y,
      scale.z);
    
    // undo temporary offset of origo
    
    if (center.x != 0.0f ||
      center.y != 0.0f ||
      center.z != 0.0f)
      ::glTranslatef(-center.x,
      -center.y,
      -center.z);
    
#ifdef _DEBUG
    if (pRenderer->CheckGLError("TransformProxy::Render(): GL Error ")) {
      TRACE("center: %f, %f, %f\n", center.x, center.y, center.z);
      TRACE("scaleOrientation: %f, %f, %f, %f\n", scaleOrientation.x, scaleOrientation.y, scaleOrientation.z, scaleOrientation.theta);
      TRACE("rotation: %f, %f, %f, %f\n", rotation.x, rotation.y, rotation.z, rotation.theta);
      TRACE("scale: %f, %f, %f\n", scale.x, scale.y, scale.z);
      TRACE("translation: %f, %f, %f\n", translation.x, translation.y, translation.z);
    }
#endif
}

// render node

/* virtual */ BOOL
TransformProxy::Render(const Matrix& rTransformation, const CullFlag& rCullFlag,
                       OpenGLVisualRenderer* pRenderer, DWORD dwTime)
{
  // Transform* pTransform = (Transform*) GetMediaObject();
  Matrix transformation = rTransformation;
  BOOL ret;
  
  ::glPushMatrix();
  
  ConcatenateOpenGLMatrix(pRenderer, dwTime);
  /* pTransform-> */ ConcatenateTransform(transformation);
  
  ret = GroupProxy::Render(transformation, rCullFlag,
    pRenderer, dwTime);
  
  ::glPopMatrix();
  
  if (pRenderer->DrawBBoxes()) {
    Vec3f size;
    Vec3f center;
    
// SOLARIS PORTING
// Created new objects to avoid memory dereferencing
  SF<Vec3f> sf_size, sf_center;
  sf_size = (SF<Vec3f>) size;
  sf_center = (SF<Vec3f>) center;

  /* pTransform-> */ GetTransformedBbox( sf_size, sf_center);

  size = (Vec3f)sf_size;
  center = (Vec3f)sf_center;

// SOLARIS PORTING - END
    
    ::glPushAttrib(GL_FOG_BIT);
    ::glDisable(GL_FOG);
    
    if (size.x > -1) {
      size *= 0.5f;
      
      ::glPushMatrix();
      ::glTranslatef(center.x, center.y, center.z);
      
      ::glBegin(GL_LINE_STRIP);
	//satish changed this block
	/*
      ::glVertex3f( size.x,  size.y,  size.z);
      ::glVertex3f(-size.x,  size.y,  size.z);
      ::glVertex3f(-size.x, -size.y,  size.z);
      ::glVertex3f( size.x, -size.y,  size.z);
      
      ::glVertex3f( size.x,  size.y,  size.z);
      ::glVertex3f( size.x,  size.y, -size.z);
      ::glVertex3f( size.x, -size.y, -size.z);
      ::glVertex3f( size.x, -size.y,  size.z);

	*/
      glVertex3f( size.x,  size.y,  size.z);
      glVertex3f(-size.x,  size.y,  size.z);
      glVertex3f(-size.x, -size.y,  size.z);
      glVertex3f( size.x, -size.y,  size.z);
      
      glVertex3f( size.x,  size.y,  size.z);
      glVertex3f( size.x,  size.y, -size.z);
      glVertex3f( size.x, -size.y, -size.z);
      glVertex3f( size.x, -size.y,  size.z);

      ::glEnd();
      
      ::glBegin(GL_LINE_STRIP);
      glVertex3f(-size.x, -size.y,  size.z);
      glVertex3f(-size.x, -size.y, -size.z);
      glVertex3f( size.x, -size.y, -size.z);
      ::glEnd();
      
      ::glBegin(GL_LINE_STRIP);

	//satish changed this block
	/*
      ::glVertex3f(-size.x,  size.y,  size.z);
      ::glVertex3f(-size.x,  size.y, -size.z);
      ::glVertex3f( size.x,  size.y, -size.z);
      ::glEnd();
      
      ::glBegin(GL_LINES);
      ::glVertex3f(-size.x,  size.y, -size.z);
      ::glVertex3f(-size.x, -size.y, -size.z);

	*/
      glVertex3f(-size.x,  size.y,  size.z);
      glVertex3f(-size.x,  size.y, -size.z);
      glVertex3f( size.x,  size.y, -size.z);
      ::glEnd();
      
      ::glBegin(GL_LINES);
      glVertex3f(-size.x,  size.y, -size.z);
      glVertex3f(-size.x, -size.y, -size.z);

      ::glEnd();
      
      ::glPopMatrix();
    } else {
      ::glBegin(GL_POINTS);
	//satish changed this line
      //::glVertex3f(center.x, center.y, center.z);
      glVertex3f(center.x, center.y, center.z);
      ::glEnd();
    }
    
    ::glPopAttrib();
  }
  
  return ret;
}

// render lights

/* virtual */ BOOL
TransformProxy::RenderLights(OpenGLVisualRenderer* pRenderer, DWORD dwTime)
{
  BOOL ret;
  
  ::glPushMatrix();
  
  ConcatenateOpenGLMatrix(pRenderer, dwTime);
  
  ret = GroupProxy::RenderLights(pRenderer, dwTime);
  
  ::glPopMatrix();
  
  return ret;
}

// traverse scene graph, checking for intersection

/* virtual */ void
TransformProxy::Select(
    OpenGLVisualRenderer* pRenderer, 
    DWORD dwTime,
    MediaObjectProxy*& intersectingObject, 
    unsigned int& depth)
{
  ::glPushMatrix();
  
  ConcatenateOpenGLMatrix(pRenderer, dwTime);
  
  GroupProxy::Select(pRenderer, dwTime, intersectingObject, depth);
  
  ::glPopMatrix();
}

// Traverse scene graph, checking for activation of sensors

/* virtual */ void
TransformProxy::Select(
    const Matrix& rTransformation, 
    const CullFlag& rCullFlag,
    OpenGLVisualRenderer* pRenderer, 
    DWORD dwTime,
    MediaObjectProxy*& intersectingObject,
    ZArray<MediaObjectProxy*>& activatedSensors, 
    unsigned int& depth,
    const ZArray<MediaObjectProxy*>& localSensors)
{
  // Transform* pTransform = (Transform*) GetMediaObject();
  Matrix transformation = rTransformation;
  
  ::glPushMatrix();
  
  ConcatenateOpenGLMatrix(pRenderer, dwTime);
  
  /* pTransform-> */ ConcatenateTransform(transformation);
  GroupProxy::Select(transformation, rCullFlag,
    pRenderer, dwTime,
    intersectingObject,
    activatedSensors, depth,
    localSensors);
  
  ::glPopMatrix();
}

// Render object description

/* virtual */ void
TransformProxy::RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD dwTime)
{
  Transform* pTransform = (Transform*) GetMediaObject();
  GLfloat color[] = { 0.0f, 1.0f, 0.0f, 0.75f};
  
	//satish changed this line
  //::glColor4fv(color);
  glColor4fv(color);
  
  RenderDescriptionBox(pRenderer, "Transform");
  
  ::glPushMatrix();
  ::glScalef(1.0f / pTransform->GetChildCount(), 1.0f, 1.0f);
  ::glTranslatef(0.0f, -1.0f, 0.0f);
  
  for (MediaObject * pObject = pTransform->GetFirstChild(); 
  pObject; pObject = pTransform->GetNextSibling())
  {
    ((MediaObjectProxy*) pObject->GetRenderingObject())->RenderDescription(pRenderer, dwTime);
    ::glTranslatef(1.0f, 0.0f, 0.0f);
  }
  
  ::glPopMatrix();
}

/* virtual */ void TransformProxy::DumpTrace()
{
  Transform * poNode = (Transform *) GetMediaObject();
  poNode->GetPresenter()->Logf("RendTransform %08x \n", this);
  for (MediaObject * poChild = poNode->GetFirstChild(); 
    poChild; poChild = poNode->GetNextSibling())
      ((MediaObjectProxy *) poChild->GetRenderingObject())->DumpTrace();
}

#ifdef ADV_AUDIO

// Find sound sources

/* virtual */ int
TransformProxy::FindSoundSources(
    ZArray<Sound*>& rSources3d, 
    ZArray<Matrix>& rTransformations3d,
		ZArray<Sound2D*>& rSources2d, 
    ZArray<Matrix>& rTransformations2d,
		ZArray<DirectiveSound*>& rDirsources, 
    ZArray<Matrix>& rDirtransformations,
		const Matrix& rTransform)
{
  // Transform * pTransform = (Transform *) GetMediaObject();
  Matrix newTransform = rTransform;

  ConcatenateTransform(newTransform);
  
  return GroupProxy::FindSoundSources(
    rSources3d, rTransformations3d,
    rSources2d, rTransformations2d,
		rDirsources, rDirtransformations,
		newTransform); 
}

/* virtual */ int
TransformProxy::FindAcousticScenes(
    ZArray<AcousticScene*>& rAcousticScenes, 
		ZArray<Matrix>& rASceneTransformations,
		const Matrix& rTransform)
{
  // Transform* pTransform = (Transform*) GetMediaObject();
  Matrix newTransform = rTransform;

  ConcatenateTransform(newTransform);
  
  return GroupProxy::FindAcousticScenes(
    rAcousticScenes, 
		rASceneTransformations,
		newTransform); 
}

/* virtual */ int
TransformProxy::FindAcousticSurfaces(
    ZArray<Shape*>& rAcousticSurfaces, 
		ZArray<Matrix>& rASurfaceTransformations,
		const Matrix& rTransform)
{
  // Transform* pTransform = (Transform*) GetMediaObject();

  Matrix newTransform = rTransform;

  ConcatenateTransform(newTransform);
  
  return GroupProxy::FindAcousticSurfaces(
    rAcousticSurfaces, 
		rASurfaceTransformations,
		newTransform); 
}

#else // ! ADV_AUDIO

/* virtual */ int
TransformProxy::FindSoundSources(
    ZArray<Sound*>& rSources3d, 
    ZArray<Matrix>& rTransformations3d,
		ZArray<Sound2D*>& rSources2d, 
    ZArray<Matrix>& rTransformations2d,
		const Matrix& rTransform)
{
  // Transform* pTransform = (Transform*) GetMediaObject();

  Matrix newTransform = rTransform;

  /* pTransform-> */ ConcatenateTransform(newTransform);
  
  return GroupProxy::FindSoundSources(
    rSources3d, rTransformations3d,
		rSources2d, rTransformations2d,
		newTransform);
}

#endif // ADV_AUDIO

// Concatenate inverse transformation with given matrix

void TransformProxy::ConcatenateInverseTransform(Matrix& rTransform) const
{
  Transform * poTransform = (Transform *) GetMediaObject();

  const Vec3f& rCenter = poTransform->center;
  const Vec3f& rScale = poTransform->scale;
  const Vec3f& rTranslation = poTransform->translation;
  const Rotation& rScaleOrientation = poTransform->scaleOrientation;
  const Rotation& rRotation = poTransform->rotation;
  
  // do temporary offset of origo
  
  if (rCenter.x != 0.0f ||
    rCenter.y != 0.0f ||
    rCenter.z != 0.0f)
    rTransform.Translate(rCenter.x,
			 rCenter.y,
       rCenter.z);
  
  if (rScaleOrientation.theta != 0.0f) {
    // do scaleRotation
    
    rTransform.Rotate(rScaleOrientation.theta,
      rScaleOrientation.x,
      rScaleOrientation.y,
      rScaleOrientation.z);
    
    // do inverse scaling
    
    if (rScale.x != 1.0f ||
      rScale.y != 1.0f ||
      rScale.z != 1.0f)
      rTransform.Scale(1.0f / rScale.x,
      1.0f / rScale.y,
      1.0f / rScale.z);
    
    // undo scalerotation
    
    rTransform.Rotate(-rScaleOrientation.theta,
      rScaleOrientation.x,
      rScaleOrientation.y,
      rScaleOrientation.z);
  } else 
    // do inverse scaling
    
    if (rScale.x != 1.0f ||
      rScale.y != 1.0f ||
      rScale.z != 1.0f)
      rTransform.Scale(1.0f / rScale.x,
      1.0f / rScale.y,
      1.0f / rScale.z);
    
    // do inverse rotation
    
    if (rRotation.theta != 0.0f)
      rTransform.Rotate(-rRotation.theta,
      rRotation.x,
      rRotation.y,
      rRotation.z);
    
    // undo temporary offset of origo
    
    if (rCenter.x != 0.0f ||
      rCenter.y != 0.0f ||
      rCenter.z != 0.0f)
      rTransform.Translate(-rCenter.x,
      -rCenter.y,
      -rCenter.z);
    
    // do inverse translation
    
    if (rTranslation.x != 0.0f ||
      rTranslation.y != 0.0f ||
      rTranslation.z != 0.0f)
      rTransform.Translate(-rTranslation.x,
      -rTranslation.y,
      -rTranslation.z);
}

// Concatenate transformation with given matrix

void TransformProxy::ConcatenateTransform(Matrix& rTransform) const
{
  Transform * poTransform = (Transform *) GetMediaObject();

  const Vec3f& rCenter = poTransform->center;
  const Vec3f& rScale = poTransform->scale;
  const Vec3f& rTranslation = poTransform->translation;
  const Rotation& rScaleOrientation = poTransform->scaleOrientation;
  const Rotation& rRotation = poTransform->rotation;
  
  if (rTranslation.x != 0.0f ||
    rTranslation.y != 0.0f ||
    rTranslation.z != 0.0f)
    rTransform.Translate(rTranslation.x,
			 rTranslation.y,
       rTranslation.z);
  
  // do temporary offset of origo
  
  if (rCenter.x != 0.0f ||
    rCenter.y != 0.0f ||
    rCenter.z != 0.0f)
    rTransform.Translate(rCenter.x,
			 rCenter.y,
       rCenter.z);
  
  // do rotation
  
  if (rRotation.theta != 0.0f)
    rTransform.Rotate(rRotation.theta,
    rRotation.x,
    rRotation.y,
    rRotation.z);
  
  if (rScaleOrientation.theta != 0.0f) {
    // do scaleRotation
    
    rTransform.Rotate(rScaleOrientation.theta,
      rScaleOrientation.x,
      rScaleOrientation.y,
      rScaleOrientation.z);
    
    // do scaling
    
    if (rScale.x != 1.0f ||
      rScale.y != 1.0f ||
      rScale.z != 1.0f)
      rTransform.Scale(rScale.x,
      rScale.y,
      rScale.z);
    
    // undo scalerotation
    
    rTransform.Rotate(-rScaleOrientation.theta,
      rScaleOrientation.x,
      rScaleOrientation.y,
      rScaleOrientation.z);
  } else 
    // do scaling
    
    if (rScale.x != 1.0f ||
      rScale.y != 1.0f ||
      rScale.z != 1.0f)
      rTransform.Scale(rScale.x,
      rScale.y,
      rScale.z);
    
    // undo temporary offset of origo
    
    if (rCenter.x != 0.0f ||
      rCenter.y != 0.0f ||
      rCenter.z != 0.0f)
      rTransform.Translate(-rCenter.x,
      -rCenter.y,
      -rCenter.z);
}

// Get viewpoint to world matrix

/* virtual */ void
TransformProxy::ViewpointToWorldMatrix(
    DWORD dwTime, Matrix& rTransform) const
{
  ConcatenateInverseTransform(rTransform);  

  MediaObjectProxy::ViewpointToWorldMatrix(dwTime, rTransform);
}

