// -*- c++ -*-

// This software module was originally developed by Karl Anders ygard
// (Telenor R&D) in the course of development of the MPEG-4 Systems
// standard (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// For more information or to receive an up to date version of this module,
// mail karlo@nta.no or check <http://televr.fou.telenor.no/compositor/>.
//
// Copyright 1997, 1998.

#ifndef __TRANSFORMPROXY_H__
#define __TRANSFORMPROXY_H__

#include "GroupProxy.h"

class Transform;

/// Transform renderer node

class TransformProxy
  : public GroupProxy
{
public:

  /// Constructor
  TransformProxy(Transform* pTransform);

  // SB 99/05/14 added for testing
  virtual void DumpTrace();
  //ML - moved body to TransformProxy.cpp

  /// Render object and its children
  virtual BOOL		RenderLights(OpenGLVisualRenderer* pRenderer, DWORD dwTime);

  /// Get viewpoint to world matrix
  virtual void		ObjectToWorldMatrix(DWORD dwTime, Matrix& rTransformProxy) const;

  /// Traverse scene graph, checking for intersection
  virtual void Select(
    OpenGLVisualRenderer* pRenderer, 
    DWORD dwTime,
		MediaObjectProxy*& intersectingObject, 
    unsigned int& depth);

  /// Render object description

  virtual void		RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD dwTime);

// Riitta 99/05/06
#ifdef ADV_AUDIO

  virtual int	FindSoundSources(
    ZArray<Sound*>& rSources3d, 
    ZArray<Matrix>& rTransformations3d,
    ZArray<Sound2D*>& rSources2d, 
    ZArray<Matrix>& rTransformations2d,
    ZArray<DirectiveSound*>& rDirsources, 
    ZArray<Matrix>& rDirtransformations,
    const Matrix& rTransform);
  
  virtual int	FindAcousticScenes(
    ZArray<AcousticScene*>& rAcousticScenes, 
    ZArray<Matrix>& rASceneTransformations,
    const Matrix& rTransform);
  
  virtual int FindAcousticSurfaces(
    ZArray<Shape*>& rAcousticSurfaces,				// IndexedFaceSet with AcousticMaterial
    ZArray<Matrix>& rASurfaceTransformations,				// Transformations above the AcousticMaterial
    const Matrix& rTransform);
  
#else // ! ADV_AUDIO
  
  virtual int	FindSoundSources(
    ZArray<Sound*>& rSources3d, 
    ZArray<Matrix>& rTransformations3d,
    ZArray<Sound2D*>& rSources2d, 
    ZArray<Matrix>& rTransformations2d,
    const Matrix& rTransform);

#endif // ADV_AUDIO

protected:

  /// Render object and its children

  virtual BOOL Render(
    const Matrix& rTransformation, 
    const CullFlag& rCullFlag,
		OpenGLVisualRenderer* pRenderer, 
    DWORD dwTime);

  /// Traverse scene graph, checking for activation of sensors
  virtual void Select(
    const Matrix& rTransformation, 
    const CullFlag& rCullFlag,
		OpenGLVisualRenderer* pRenderer, 
    DWORD dwTime,
		MediaObjectProxy*& intersectingObject,
		ZArray<MediaObjectProxy*>& activatedSensors, 
    unsigned int& depth,
		const ZArray<MediaObjectProxy*>& localSensors);

private:

  // Concatenate transformation with OpenGL matrix

  void ConcatenateOpenGLMatrix(OpenGLVisualRenderer* pRenderer, DWORD dwTime);

  // SB moved from Transform
public:
  /// Concatenate inverse transformation with given matrix
  void ConcatenateInverseTransform(Matrix& rTransform) const;
  /// Concatenate transformation with given matrix
  void ConcatenateTransform(Matrix& rTransform) const;
  /// Compute Viewpoint to World transform
  virtual void ViewpointToWorldMatrix(
      DWORD dwTime, Matrix& rTransform) const;

};

#include "TransformProxy.i"

#endif // __TRANSFORMPROXY_H__

