#include "OpenGl/OpenGl.h"
#include "NormalInterpolatorProxy.h"

NormalInterpolatorProxy::NormalInterpolatorProxy(MediaObject * poNode) :
InterpolatorNodeProxy(poNode)
{}

inline BOOL NormalInterpolatorProxy::SetValue(
    NormalInterpolator * poNI, int iIndex)
{
  int arraySize = poNI->keyValue.GetLength() / poNI->key.GetLength();
  int offset = arraySize * iIndex;

  // Sanity test
  assert(poNI->keyValue.GetLength() % poNI->key.GetLength() == 0);

  // Get rid of all old values
  poNI->value_changed.Remove();

  // Set normals directly
  for (int j = 0; j < arraySize; ++j)
    poNI->value_changed.Append(poNI->keyValue[offset + j]);
  
  return TRUE;
}

BOOL NormalInterpolatorProxy::Evaluate()
{
  NormalInterpolator * poNI = 
    (NormalInterpolator *) GetMediaObject();

  // Check if we have any keys
  if (poNI->key.GetLength() > 0)
  {
    // If we only have one key
    if (poNI->key.GetLength() == 1)
    {
      return SetValue(poNI, 0);
    }
    else // more keys
    {
      // If set_fraction is smaller than smallest key
      if (poNI->set_fraction <= poNI->key[0])
      {
        return SetValue(poNI, 0);
      }
      else
      {
        // If set_fraction is larger than largest key
        if (poNI->set_fraction >= poNI->key[poNI->key.GetLength() - 1])
        {
          return SetValue(poNI, poNI->key.GetLength() - 1);
        }
        else
        {
          // Search the key space for fraction
          for (int i = poNI->key.GetLength() - 2; i >= 0 ; --i)
          {
            if (poNI->key[i] == poNI->set_fraction)
            {
              // Found exact match
              return SetValue(poNI, i);
            }
            else
            {
              if (poNI->key[i] < poNI->set_fraction)
              {
                // Didn't find exact match; must interpolate
                return InterpolateValue( 
                  i, i + 1, 
                  ((poNI->key[i + 1] - poNI->set_fraction) / 
                  (poNI->key[i + 1] - poNI->key[i])) );
              }
            }
          } // for
        }
      }
    } // one or more keys
  } // some key

  return FALSE;
}

BOOL NormalInterpolatorProxy::InterpolateValue(
    int nMinIndex, int nMaxIndex, float fFraction)
{
  NormalInterpolator * poNI = 
    (NormalInterpolator *) GetMediaObject();

  int arraySize = poNI->keyValue.GetLength() / poNI->key.GetLength();
  int min = arraySize * nMinIndex;
  int max = arraySize * nMaxIndex;

  // Sanity test

  assert(poNI->keyValue.GetLength() % poNI->key.GetLength() == 0);

  // Get rid of all old values

  poNI->value_changed.Remove();

  // Interpolate normals

  for (int i = 0; i < arraySize; ++i) {
    Vec3f tempNormal;

    tempNormal.x = (poNI->keyValue[max + i].x + 
      (poNI->keyValue[min + i].x - poNI->keyValue[max + i].x) * fFraction);
    tempNormal.y = (poNI->keyValue[max + i].y + 
      (poNI->keyValue[min + i].y - poNI->keyValue[max + i].y) * fFraction);
    tempNormal.z = (poNI->keyValue[max + i].z + 
      (poNI->keyValue[min + i].z - poNI->keyValue[max + i].z) * fFraction);

    tempNormal.Normalize();

    poNI->value_changed.Append(tempNormal);
  }

  return TRUE;
}
