#include "OpenGl/OpenGl.h"
#include "ScalarInterpolatorProxy.h"

ScalarInterpolatorProxy::ScalarInterpolatorProxy(MediaObject * poNode) :
InterpolatorNodeProxy(poNode)
{}

inline BOOL ScalarInterpolatorProxy::SetValue(
    ScalarInterpolator * poSI, int iIndex)
{
  poSI->value_changed = poSI->keyValue[iIndex];
  return TRUE;
}

BOOL ScalarInterpolatorProxy::Evaluate()
{
  ScalarInterpolator * poSI = 
    (ScalarInterpolator *) GetMediaObject();

  // Check if we have any keys
  if (poSI->key.GetLength() > 0)
  {
    // If we only have one key
    if (poSI->key.GetLength() == 1)
    {
      return SetValue(poSI, 0);
    }
    else // more keys
    {
      // If set_fraction is smaller than smallest key
      if (poSI->set_fraction <= poSI->key[0])
      {
        return SetValue(poSI, 0);
      }
      else
      {
        // If set_fraction is larger than largest key
        if (poSI->set_fraction >= poSI->key[poSI->key.GetLength() - 1])
        {
          return SetValue(poSI, poSI->key.GetLength() - 1);
        }
        else
        {
          // Search the key space for fraction
          for (int i = poSI->key.GetLength() - 2; i >= 0 ; --i)
          {
            if (poSI->key[i] == poSI->set_fraction)
            {
              // Found exact match
              return SetValue(poSI, i);
            }
            else
            {
              if (poSI->key[i] < poSI->set_fraction)
              {
                // Didn't find exact match; must interpolate
                return InterpolateValue( 
                  i, i + 1, 
                  ((poSI->key[i + 1] - poSI->set_fraction) / 
                  (poSI->key[i + 1] - poSI->key[i])) );
              }
            }
          } // for
        }
      }
    } // one or more keys
  } // some key

  return FALSE;
}

BOOL ScalarInterpolatorProxy::InterpolateValue(
    int nMinIndex, int nMaxIndex, float fFraction)
{
  ScalarInterpolator * poSI = 
    (ScalarInterpolator *) GetMediaObject();

  assert(poSI->keyValue.GetLength() == poSI->key.GetLength());

  // Interpolate scalar

  poSI->value_changed = poSI->keyValue[nMaxIndex] + 
    (poSI->keyValue[nMinIndex] - poSI->keyValue[nMaxIndex]) * fFraction;

  return TRUE;
}
