// -*- c++ -*-

// This software module was originally developed by Karl Anders ygard
// (Telenor R&D) in the course of development of the MPEG-4 Systems
// standard (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// Copyright 1997.

#include "OpenGl/OpenGl.h"
#include "DirectionalLightProxy.h"
#include "opengl/base/OpenGLPresenter.h"


// Render object description

/* virtual */ void
DirectionalLightProxy::RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD /* dwTime */)
{
  GLfloat color[] = { 1.0f, 0.0f, 1.0f, 0.75f};

  //satish changed this line for compilation purposes
        //have to be fixed afterwards

  //::glColor4fv(color);
  glColor4fv(color);

  RenderDescriptionBox(pRenderer, "DirectionalLight");
}

// Render node

/* virtual */ BOOL
DirectionalLightProxy::RenderLights(OpenGLVisualRenderer* pRenderer, DWORD)
{
  // DirectionalLight* pDirectionalLight = (DirectionalLight*) GetMediaObject();

  if (/* pDirectionalLight-> */ GetEnabled()) {
    const Color3f& color = /* pDirectionalLight-> */ GetColor();
    const Vec3f& direction = /* pDirectionalLight-> */ GetDirection();
    float ambientIntensity = /* pDirectionalLight-> */ GetAmbientIntensity();
    float intensity = /* pDirectionalLight-> */ GetIntensity();

    GLenum nLight = (GLenum) pRenderer->RegisterLight();

    GLfloat ambient[4] = {color.r * ambientIntensity,
                          color.g * ambientIntensity,
                          color.b * ambientIntensity,
                          1.0f};

    GLfloat diffuse[4] = {color.r * intensity,
                          color.g * intensity,
                          color.b * intensity,
                          1.0f};

    GLfloat dir[4] = {-direction.x,
                      -direction.y,
                      -direction.z,
                      0.0f};

    ::glEnable(nLight);
    ::glLightfv(nLight, GL_AMBIENT, ambient);
    ::glLightfv(nLight, GL_DIFFUSE, diffuse);
    ::glLightfv(nLight, GL_POSITION, dir);

#ifdef _DEBUG
    if (pRenderer->CheckGLError("DirectionalLightProxy:")) {
      TRACE("color: %f, %f, %f\n",
	       color.r, color.g, color.b);
      TRACE("direction: %f, %f, %f\n",
	       direction.x, direction.y, direction.z);
      TRACE("ambientIntensity: %f\n", ambientIntensity);
      TRACE("intensity: %f\n", intensity);
      TRACE("DirectionalLightProxy::Render() [%08x]\n", this);
    }
#endif
  }

  return TRUE;
}

