// -*- c++ -*-

// This software module was originally developed by Karl Anders ygard
// (Telenor R&D) in the course of development of the MPEG-4 Systems
// standard (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// Copyright 1997.

#ifndef __DIRECTIONALLIGHTPROXY_H__
#define __DIRECTIONALLIGHTPROXY_H__

#include "LightProxy.h"

// Stefano Battista
// class DirectionalLight;
//#include "Nodes.h"

/// Directional lightsource renderer

class DirectionalLightProxy
  : public LightProxy
{
public:

  /// Constructor
  // DirectionalLightProxy(DirectionalLight* pDirectionalLight);
  DirectionalLightProxy(MediaObject * pDirectionalLight);

  /// Traverse scene graph, activating lights
  virtual BOOL		RenderLights(OpenGLVisualRenderer* pRenderer, DWORD dwTime);

  /// Render object description
  virtual void		RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD dwTime);

  // Stefano Battista
  // moved methods from DirectionalLight to DirectionalLightProxy
public:
  inline DirectionalLight * GetDirectionalLight() const
  { return (DirectionalLight *) GetMediaObject(); }
  /// Set direction of lightsource
  inline void SetDirection(const SF<Vec3f>& rDirection)
  { GetDirectionalLight()->direction = rDirection; }
  /// Get direction of lightsource
  inline const SF<Vec3f>& GetDirection() const
  { return GetDirectionalLight()->direction; }
  /// Set ambient intensity of lightsource
  inline void SetAmbientIntensity(float fAmbientIntensity)
  { GetDirectionalLight()->ambientIntensity = fAmbientIntensity; }
  /// Get ambient intensity of lightsource
  inline float GetAmbientIntensity() const
  { return GetDirectionalLight()->ambientIntensity; }
  /// Set color of lightsource
  inline void SetColor(const SF<Color3f>& rColor)
  { GetDirectionalLight()->color = rColor; }
  /// Get color of lightsource
  inline const SF<Color3f>& GetColor() const
  { return GetDirectionalLight()->color; }
  /// Set intensity of lightsource
  inline void SetIntensity(float fIntensity)
  { GetDirectionalLight()->intensity = fIntensity; }
  /// Get intensity of lightsource
  inline float GetIntensity() const
  { return GetDirectionalLight()->intensity; }
  /// Enable or disable lightsource
  inline void SetEnabled(bool bOn)
  { GetDirectionalLight()->on = bOn; }
  /// Is lightsource enabled?
  inline BOOL GetEnabled() const
  { return (BOOL) (Bool) GetDirectionalLight()->on; }
  
};

#include "DirectionalLightProxy.i"

#endif // __DIRECTIONALLIGHTPROXY_H__
