// -*- c++ -*-

// This software module was originally developed by Karl Anders ygard
// (Telenor R&D) in the course of development of the MPEG-4 Systems
// standard (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// Copyright 1997.

#include "OpenGl/OpenGl.h"
#include "PointLightProxy.h"
#include "opengl/base/OpenGLVisualRenderer.h"


// Render object description

/* virtual */ void
PointLightProxy::RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD /* dwTime */)
{
  GLfloat color[] = { 1.0f, 0.0f, 1.0f, 0.75f};

  //satish changed this line for compilation purposes
        //have to be fixed afterwards

//  ::glColor4fv(color);
    glColor4fv(color);

  RenderDescriptionBox(pRenderer, "PointLight");
}

// Render node

/* virtual */ BOOL
PointLightProxy::RenderLights(OpenGLVisualRenderer* pRenderer, DWORD)
{
  // PointLight* pPointLight = (PointLight*) GetMediaObject();

  if (/* pPointLight-> */ GetEnabled()) {
    const Color3f& color = /* pPointLight-> */ GetColor();
    const Vec3f& location = /* pPointLight-> */ GetLocation();
    const Vec3f& attenuation = /* pPointLight-> */ GetAttenuation();
    float ambientIntensity = /* pPointLight-> */ GetAmbientIntensity();
    float intensity = /* pPointLight-> */ GetIntensity();

    GLenum nLight = (GLenum) pRenderer->RegisterLight();
  
    GLfloat ambient[4] = {color.r * ambientIntensity,
			  color.g * ambientIntensity,
			  color.b * ambientIntensity,
			  1.0f};
			       
    GLfloat diffuse[4] = {color.r * intensity,
			  color.g * intensity,
			  color.b * intensity,
			  1.0f};
  
    GLfloat position[4] = {location.x,
			   location.y,
			   location.z,
			   1.0f};

    ::glEnable(nLight);
    ::glLightfv(nLight, GL_AMBIENT, ambient);
    ::glLightfv(nLight, GL_DIFFUSE, diffuse);
    ::glLightfv(nLight, GL_POSITION, position);
    ::glLightf(nLight, GL_CONSTANT_ATTENUATION, attenuation[0]);
    ::glLightf(nLight, GL_LINEAR_ATTENUATION, attenuation[1]);
    ::glLightf(nLight, GL_QUADRATIC_ATTENUATION, attenuation[2]);

#ifdef _DEBUG
    if (pRenderer->CheckGLError("PointLightProxy:")) {
      TRACE("color: %f, %f, %f\n",
	       color.r, color.g, color.b);
      TRACE("location: %f, %f, %f\n",
	       location.x, location.y, location.z);
      TRACE("attenuation: %f, %f, %f\n",
	       attenuation[0], attenuation[1], attenuation[2]);
      TRACE("ambientIntensity: %f\n", ambientIntensity);
      TRACE("intensity: %f\n", intensity);
      TRACE("PointLightProxy::Render() [%08x]\n", this);
    }
#endif
  }

  return TRUE;
}
