// -*- c++ -*-

// This software module was originally developed by Karl Anders ygard
// (Telenor R&D) in the course of development of the MPEG-4 Systems
// standard (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// Copyright 1997.

#ifndef __POINTLIGHTPROXY_H__
#define __POINTLIGHTPROXY_H__

#include "LightProxy.h"

class PointLight;

/// Point lightsource renderer

class PointLightProxy
  : public LightProxy
{
public:

  /// Constructor

                        PointLightProxy(PointLight* pPointLight);

  /// Traverse scene graph, activating lights

  virtual BOOL		RenderLights(OpenGLVisualRenderer* pRenderer, DWORD dwTime);

  /// Render object description

  virtual void		RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD dwTime);

  // SB @@@@ moved from PointLight
public:
  inline PointLight * GetPointLight()
  { return (PointLight *) GetMediaObject(); }
  /// Set attenuation of lightsource
  inline void SetAttenuation(const SF<Vec3f>& rAttenuation)
  { GetPointLight()->attenuation = rAttenuation; }
  /// Get attenuation of lightsource
  inline const SF<Vec3f>& GetAttenuation() /* const */
  { return GetPointLight()->attenuation; }
  /// Set location of lightsource
  inline void SetLocation(const SF<Vec3f>& rLocation)
  { GetPointLight()->location = rLocation; }
  /// Get location of lightsource
  inline const SF<Vec3f>& GetLocation() /* const */
  { return GetPointLight()->location; }
  /// Set radius of lightsource
  inline void SetRadius(float fRadius)
  { GetPointLight()->radius = fRadius; }
  /// Get radius of lightsource
  inline float GetRadius() /* const */
  { return GetPointLight()->radius; }

  // SB @@@@ moved from Light
  /// set ambient intensity of lightsource  
  inline void SetAmbientIntensity(float fAmbientIntensity)
  { GetPointLight()->ambientIntensity = fAmbientIntensity; }
  /// get ambient intensity of lightsource
  inline float GetAmbientIntensity() /* const */
  { return GetPointLight()->ambientIntensity; }
  /// set color of lightsource
  inline void SetColor(const SF<Color3f>& rColor)
  { GetPointLight()->color = rColor; }
  /// get color of lightsource
  inline const SF<Color3f>& GetColor() /* const */
  { return GetPointLight()->color; }
  /// set intensity of lightsource
  inline void SetIntensity(float fIntensity)
  { GetPointLight()->intensity = fIntensity; }
  /// get intensity of lightsource
  inline float GetIntensity() /* const */
  { return GetPointLight()->intensity; }
  /// Enable or disable lightsource
  inline void SetEnabled(bool bOn)
  { GetPointLight()->on = bOn; }
  /// Is lightsource enabled?
  inline BOOL GetEnabled() /* const */
  { return (BOOL) (Bool) GetPointLight()->on; }
  
};

#include "PointLightProxy.i"

#endif // __POINTLIGHTPROXY_H__
