// -*- c++ -*-

// This software module was originally developed by Karl Anders ygard
// (Telenor R&D) in the course of development of the MPEG-4 Systems
// standard (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// Copyright 1997.

#ifndef __SPOTLIGHTPROXY_H__
#define __SPOTLIGHTPROXY_H__

#include "LightProxy.h"

class SpotLight;

/// Spot lightsource renderer

class SpotLightProxy
  : public LightProxy
{
public:

  /// Constructor

                        SpotLightProxy(SpotLight* pSpotLight);

  /// Traverse scene graph, activating lights

  virtual BOOL		RenderLights(OpenGLVisualRenderer* pRenderer, DWORD dwTime);

  /// Render object description

  virtual void		RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD dwTime);

  // SB 98/12/04
public:
  inline SpotLight * GetSpotLight()
  { return (SpotLight *) GetMediaObject(); }
  // Set attenuation of lightsource
  inline void SetAttenuation(const SF<Vec3f>& rAttenuation)
  { GetSpotLight()->attenuation = rAttenuation; }
  // Get attenuation of lightsource
  inline const SF<Vec3f>& GetAttenuation() /* const */
  { return GetSpotLight()->attenuation; }
  // Set beam width of lightsource
  inline void SetBeamWidth(float fBeamWidth)
  { GetSpotLight()->beamWidth = fBeamWidth; }
  // Get beam width of lightsource
  inline float GetBeamWidth() /* const */
  { return GetSpotLight()->beamWidth; }
  // Set cut off angle of lightsource
  inline void SetCutOffAngle(float fCutOffAngle)
  { GetSpotLight()->cutOffAngle = fCutOffAngle; }
  // Set cut off angle of lightsource
  inline float GetCutOffAngle() /* const */
  { return GetSpotLight()->cutOffAngle; }
  // Set direction of lightsource
  inline void SetDirection(const SF<Vec3f>& rDirection)
  { GetSpotLight()->direction = rDirection; }
  // Set direction of lightsource
  inline const SF<Vec3f>& GetDirection() /* const */
  { return GetSpotLight()->direction; }
  // Set location of lightsource
  inline void SetLocation(const SF<Vec3f>& rLocation)
  { GetSpotLight()->location = rLocation; }
  // Get location of lightsource
  inline const SF<Vec3f>& GetLocation() /* const */
  { return GetSpotLight()->location; }
  // Set radius of lightsource
  inline void SetRadius(float fRadius)
  { GetSpotLight()->radius = fRadius; }
  // Get radius of lightsource
  inline float GetRadius() /* const */
  { return GetSpotLight()->radius; }

  // SB @@@@ moved from Light
  /// set ambient intensity of lightsource  
  inline void SetAmbientIntensity(float fAmbientIntensity)
  { GetSpotLight()->ambientIntensity = fAmbientIntensity; }
  /// get ambient intensity of lightsource
  inline float GetAmbientIntensity() /* const */
  { return GetSpotLight()->ambientIntensity; }
  /// set color of lightsource
  inline void SetColor(const SF<Color3f>& rColor)
  { GetSpotLight()->color = rColor; }
  /// get color of lightsource
  inline const SF<Color3f>& GetColor() /* const */
  { return GetSpotLight()->color; }
  /// set intensity of lightsource
  inline void SetIntensity(float fIntensity)
  { GetSpotLight()->intensity = fIntensity; }
  /// get intensity of lightsource
  inline float GetIntensity() /* const */
  { return GetSpotLight()->intensity; }
  /// Enable or disable lightsource
  inline void SetEnabled(bool bOn)
  { GetSpotLight()->on = bOn; }
  /// Is lightsource enabled?
  inline BOOL GetEnabled() /* const */
  { return (BOOL) (Bool) GetSpotLight()->on; }

};

#include "SpotLightProxy.i"

#endif // __SPOTLIGHTPROXY_H__
