// -*- c++ -*-

// This software module was originally developed by Karl Anders ygard
// (Telenor R&D) in the course of development of the MPEG-4 Systems
// standard (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// Copyright 1998.

#include "OpenGl/OpenGl.h"
#include "opengl/base/OpenGLVisualRenderer.h"
#include "DragSensorProxy.h"


// Render object description

/* virtual */ void
DragSensorProxy::RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD /* dwTime */)
{
  GLfloat color[] = { 0.0f, 0.0f, 1.0f, 0.75f};

	//satish changed this line for compilation purposes
	//have to be fixed afterwards
  //::glColor4fv(color);
  glColor4fv(color);

  RenderDescriptionBox(pRenderer, "DragSensor");
}

// Activate sensor

/* virtual */ bool
DragSensorProxy::Activate(
    OpenGLVisualRenderer* pRenderer,
		float fObjectX, float fObjectY, float fObjectZ,
		int nX, int nY, 
    bool bMouseActivated)
{
  if (bMouseActivated) {
    // DragSensor* pDragSensor = (DragSensor*) GetMediaObject();
    
    pRenderer->RegisterDragSensor(this);
    /* pDragSensor-> */ ActivateNode(fObjectX, fObjectY, fObjectZ, nX, nY);
    
    return TRUE;
  } else
    return FALSE;
}

// Return whether object is a pointing device sensor

/* virtual */ BOOL
DragSensorProxy::IsPointingDeviceSensor() const
{
  return TRUE;
}

// Drag sensor

void
DragSensorProxy::Drag(OpenGLVisualRenderer*, int nX, int nY)
{
  // DragSensor* pDragSensor = (DragSensor*) GetMediaObject();

  /* pDragSensor-> */ DragNode(nX, nY);
}

// Deactivate sensor

void
DragSensorProxy::Deactivate(OpenGLVisualRenderer* pRenderer, int nX, int nY)
{
  // DragSensor* pDragSensor = (DragSensor*) GetMediaObject();

  pRenderer->UnregisterDragSensor(this);
  /* pDragSensor-> */ DeactivateNode(nX, nY);
}


// Stefano Battista
// moved from DragSensor to Proxy

// Activate sensor
bool DragSensorProxy::ActivateNode(
    float fObjectX, 
    float fObjectY, 
    float fObjectZ, 
    int nX, 
    int nY)
{
  // DragSensor * poDragSensor = GetDragSensor();

  if ((bool)(Bool) enabled == TRUE)
  {
    isActive = TRUE;
    m_nX = nX;
    m_nY = nY;
    StartSensor(fObjectX, fObjectY, fObjectZ);
    return TRUE;
  }
  else
  {
    return FALSE;
  }
}

// Drag sensor
void DragSensorProxy::DragNode(int nX, int nY)
{
  UpdateSensor(nX - m_nX, m_nY - nY);
}

// Deactivate sensor
void DragSensorProxy::DeactivateNode(int nX, int nY)
{
  isActive = FALSE;
  StopSensor(nX - m_nX, m_nY - nY);
}
