// -*- c++ -*-

// This software module was originally developed by Karl Anders ygard
// (Telenor R&D) in the course of development of the MPEG-4 Systems
// standard (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// Copyright 1998.

#ifndef __DRAGSENSORPROXY_H__
#define __DRAGSENSORPROXY_H__

#include "opengl/base/NodeProxy.h"

class DragSensor;

/// DragSensor renderer class

class DragSensorProxy
  : public NodeProxy
{
// SB moved from DragSensor
private:
  int m_nX;
  int m_nY;

  /// Flags whether auto-offsetting is to be done
  SF<Bool>		autoOffset;
  /// Flags whether sensor is enabled
  SF<Bool>		enabled;
  /// Flags whether pointer is over sensor and user has activated pointer
  SF<Bool>		isActive;
  /// Intersection point of pointer with geometry
  SF<Vec3f>		trackPoint_changed;
public:

  /// Constructor
  DragSensorProxy(MediaObject * pDragSensor);

  /// Render object description
  virtual void  RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD dwTime);

  /// Activate sensor
  virtual bool Activate(
      OpenGLVisualRenderer* pRenderer,
			float fObjectX, float fObjectY, float fObjectZ,
			int nX, int nY, 
      bool bMouseActivated);

  /// Deactivate sensor
  void  Deactivate(OpenGLVisualRenderer* pRenderer, int nX, int nY);

  /// Drag sensor
  void  Drag(OpenGLVisualRenderer* pRenderer, int nX, int nY);

  /// Return whether object is a pointing device sensor
  virtual BOOL  IsPointingDeviceSensor() const;

  // SB moved from DragSensor
private:
  /// Get pointer to DragSensor object
  inline DragSensor * GetDragSensor() const
  { return (DragSensor *) GetMediaObject(); }
  /// Activate sensor
  bool ActivateNode(
      float fObjectX, float fObjectY, float fObjectZ, 
      int nX, int nY);
  /// Drag sensor
  void DragNode(int nX, int nY);
  /// Deactivate sensor
  void DeactivateNode(int nX, int nY);

  // SB @@@@ 99/01/07 EMPTY Start/Update/Stop methods...

  /// Start sensor
  virtual void StartSensor(
    float /* fObjectX */, 
    float /* fObjectY */, 
    float /* fObjectZ */) /* = 0 */ {};

  /// Update sensor 
  virtual void UpdateSensor(
    int /* nX */, 
    int /* nY */) /* = 0 */ {};

  /// Stop sensor 
  virtual void StopSensor(
    int /* nX */, 
    int /* nY */) /* = 0 */ {};
};

#include "DragSensorProxy.i"

#endif // __DRAGSENSORPROXY_H__
