// -*- c++ -*-

// This software module was originally developed by Karl Anders ygard
// (Telenor R&D) in the course of development of the MPEG-4 Systems
// standard (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// Copyright 1998.

#include "OpenGl/OpenGl.h"
#include "TimeSensorProxy.h"


// Static members

/* static */ ZArray<TimeSensorProxy*> TimeSensorProxy::m_sensorList;
/* static */ ZMutex TimeSensorProxy::m_mListLock;

// Render object description

/* virtual */ void
TimeSensorProxy::RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD /* dwTime */)
{
  GLfloat color[] = { 0.0f, 0.0f, 1.0f, 0.75f};
  
	  //satish changed this line for compilation purposes
        //have to be fixed afterwards
  //::glColor4fv(color);
  glColor4fv(color);
  
  RenderDescriptionBox(pRenderer, "TimeSensor");
}

/// Poll time sensors

/* static */ void
TimeSensorProxy::UpdateTimeSensors()
{
  m_mListLock.P();
  
  for (int i = 0; i < m_sensorList.GetLength(); ++i)
    (/* (TimeSensor *) */ m_sensorList[i] /* ->GetMediaObject() */ )->UpdateTimeSensor();
  
  m_mListLock.V();
}


// Poll time sensor

void TimeSensorProxy::UpdateTimeSensor()
{
  TimeSensor * poTimeSensor = GetTimeSensor();
  Time currentTime = poTimeSensor->GetRoot()-> /* GetClock(). */ GetTime() / 1000.0;

  // if Enabled >>>>
  if (GetEnabled()) 
  {
    // if Start <= Current >>>>
    if (GetStartTime() <= currentTime)
    {
      // if time sensor is active or should be activated
      if (
        (GetLoop() && (GetTimeSensor()->stopTime <= GetTimeSensor()->startTime)) || // loop forever
        (GetLoop() && (currentTime < GetTimeSensor()->stopTime)) || // loop until Stop
        (! GetLoop() && (currentTime < GetTimeSensor()->startTime + GetTimeSensor()->cycleInterval)) // loop once
        )
      {
        // if Loop >>>>
        if (GetLoop())
        {
          // activate and initialize
          if (! GetActive()) {
            SetActive(TRUE);
            SetFractionChanged (0.0);
            SetCycleTime (GetStartTime());
          }
          // set the CycleTime within the CURRENT cycle
          while (GetTimeSensor()->cycleTime + GetCycleInterval() < currentTime)
          {
            GetTimeSensor()->cycleTime = 
              GetTimeSensor()->cycleTime + GetCycleInterval();
          }
          // set Fraction
          SetFractionChanged((float) 
              (currentTime - GetCycleTime()) / GetCycleInterval());
        }
        // <<<< if NOT Loop >>>>
        else
        {
          if (GetCycleTime() != GetStartTime() + GetCycleInterval())
          {
            // activate and initialize
            if (! GetActive()) {
              SetActive(TRUE);
              SetFractionChanged (0.0);
              SetCycleTime (GetStartTime());
            }
            // if end of cycle, terminate
            if (GetCycleTime() + GetCycleInterval() < currentTime)
            {
              if (GetCycleTime() != GetStartTime() + GetCycleInterval())
              {
                GetTimeSensor()->cycleTime = (GetStartTime() + GetCycleInterval());
                SetFractionChanged (1.0);
                SetActive (FALSE);
              }
            } 
            // if NOT end of cycle, set Fraction
            else
            {
              SetFractionChanged ((float)
                  (currentTime - GetStartTime()) / GetCycleInterval());
            }
          }
        }
        // if NOT Loop <<<<
        GetTimeSensor()->time = currentTime;
      } 
      // if time sensor has reached stop time; terminate properly
      else
      {
        if (GetActive())
        {
          if (GetLoop())
          {
            while (GetTimeSensor()->cycleTime + GetCycleInterval() < GetStopTime())
            {
              GetTimeSensor()->cycleTime = 
                GetTimeSensor()->cycleTime + GetCycleInterval();
            }
            SetFractionChanged ((float) 
                (GetStopTime() - GetCycleTime()) / GetCycleInterval());
          }
          else
          {
            if (GetCycleTime() != GetStartTime() + GetCycleInterval())
            {
              if (GetCycleTime() + GetCycleInterval() < GetStopTime())
              {
                if (GetCycleTime() != GetStartTime() + GetCycleInterval())
                {
                  GetTimeSensor()->cycleTime = (GetStartTime() + GetCycleInterval());
                  SetFractionChanged (1.0);
                }
              } 
              else
              {
                SetFractionChanged ((float)
                    (GetStopTime() - GetStartTime()) / GetCycleInterval());
              }
            }
          }
          GetTimeSensor()->time = (Time) (GetStopTime());
          SetActive (FALSE);
        }
      }
    }
    // if Start <= Current <<<<
  }
  // <<<< if NOT Enabled >>>>
  else
  {
    // time sensor is active and should be deactivated
    if (GetActive())
    {
      if (GetLoop())
      {
        while (GetTimeSensor()->cycleTime + GetCycleInterval() < currentTime)
        {
          GetTimeSensor()->cycleTime = 
            GetTimeSensor()->cycleTime + GetCycleInterval();
        }        
        SetFractionChanged ((float) 
          (currentTime - GetCycleTime()) / GetCycleInterval());
      }
      else
      {
        if (GetCycleTime() != GetStartTime() + GetCycleInterval())
        {
          if (GetCycleTime() + GetCycleInterval() < currentTime)
          {
            if (GetCycleTime() != GetStartTime() + GetCycleInterval())
            {
              GetTimeSensor()->cycleTime = (GetStartTime() + GetCycleInterval());
              SetFractionChanged (1.0);
            }
          }
          else
          {
            SetFractionChanged ((float)
                (currentTime - GetStartTime()) / GetCycleInterval());
          }
        }
      }
      GetTimeSensor()->time = currentTime;
      SetActive (FALSE);
    }
  }
  // if NOT Enabled <<<<
}
