// -*- c++ -*-

// This software module was originally developed by Karl Anders ygard
// (Telenor R&D) in the course of development of the MPEG-4 Systems
// standard (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// Copyright 1998.

#ifndef __TIMESENSORPROXY_H__
#define __TIMESENSORPROXY_H__

#include "opengl/base/NodeProxy.h"

class TimeSensor;

/// TimeSensor renderer class

class TimeSensorProxy
  : public NodeProxy
{
public:

  /// Constructor

                        TimeSensorProxy(TimeSensor* pTimeSensor);

  /// Destructor

  virtual		~TimeSensorProxy();

  /// Render object description

  virtual void		RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD dwTime);

  /// Poll time sensor

  static void		UpdateTimeSensors();

private:

  // List of timesensors

  static ZArray<TimeSensorProxy*>
			m_sensorList;

  // List lock

  static ZMutex		m_mListLock;

  // SB moved from TimeSensor
public:
  inline TimeSensor * GetTimeSensor()
  { return (TimeSensor *) GetMediaObject(); }

  /// Get length of a cycle
  //ML - changed return type from const Time& to const Time
  // to remove warning about returning address of temporary variable
  inline const Time GetCycleInterval() /* const */
  { return GetTimeSensor()->cycleInterval; }
  /// Set length of a cycle
  inline void SetCycleInterval(const Time& tCycleInterval)
  { GetTimeSensor()->cycleInterval = tCycleInterval; }

  /// Get time within a cycle
  //ML - changed return type from const Time& to const Time
  // to remove warning about returning address of temporary variable
  inline const Time GetCycleTime() /* const */
  { return GetTimeSensor()->cycleTime; }
  /// Set length of a cycle
  inline void SetCycleTime(const Time& tCycleTime)
  { GetTimeSensor()->cycleTime = tCycleTime; }

  /// Get fraction of cycle [0, 1]
  inline const float GetFractionChanged() /* const */
  { return GetTimeSensor()->fraction_changed; }
  /// Set fraction of cycle [0, 1]
  inline void SetFractionChanged(const float fFraction)
  { GetTimeSensor()->fraction_changed = fFraction; }

  /// Get whether sensor is enabled
  inline BOOL GetEnabled() /* const */
  { return (Bool) GetTimeSensor()->enabled; }
  /// Set whether sensor is enabled
  inline void SetEnabled(bool bEnabled)
  { GetTimeSensor()->enabled = bEnabled; }

  /// Get whether sensor is active
  inline BOOL GetActive() /* const */
  { return (Bool) GetTimeSensor()->isActive; }
  /// Set whether sensor is active
  inline void SetActive(bool bActive)
  { GetTimeSensor()->isActive = bActive; }

  /// Get whether sensor should repeat
  inline BOOL GetLoop() /* const */
  { return (Bool) GetTimeSensor()->loop; }
  /// Set whether sensor should repeat
  inline void SetLoop(bool bLoop)
  { GetTimeSensor()->loop = bLoop; }

  /// Get start time of sensor
  //ML - changed return type from const Time& to const Time
  // to remove warning about returning address of temporary variable
  inline const Time GetStartTime() /* const */
  { return GetTimeSensor()->startTime; }
  /// Set start time of sensor
  inline void SetStartTime(const Time& tStartTime)
  { GetTimeSensor()->startTime = tStartTime; }

  /// Get stop time of sensor
  //ML - changed return type from const Time& to const Time
  // to remove warning about returning address of temporary variable
  inline const Time GetStopTime() /* const */
  { return GetTimeSensor()->stopTime; }
  /// Set stop time of sensor
  inline void SetStopTime(const Time& tStopTime)
  { GetTimeSensor()->stopTime = tStopTime; }

  /// Poll time sensor
  void UpdateTimeSensor();

};

#include "TimeSensorProxy.i"

#endif // __TIMESENSORPROXY_H__
