//#ifndef __GLSHADOW_H__
//#define __GLSHADOW_H__



#include <stdlib.h>
#include <stdio.h>
#include <malloc.h>
#include <assert.h>
#include <math.h>
//#include <GL/glut.h>
#include <GL/gl.h>
#include <GL/glu.h>


#include "glstuff.h"  /* declarations for OpenGL extensions used by shadowcast */
//#include "matrix.h"   /* OpenGL-style 4x4 matrix manipulation routines */

/* Some <math.h> files do not define M_PI... */
#ifndef M_PI
#define M_PI 3.14159265358979323846
#endif

#define MAX_SIZE 1024
#define LIGHT_DIMMING 0.2
#define MAG16 65535.0f
#define MAG24 16777215.0f

/* Texture objects. */
enum {
  TO_RESERVED = 0,             /* zero is not a valid OpenGL texture object ID */
  TO_MAP_8BIT,                 /* 1D identity texture, s := [0,255] range */
  TO_MAP_16BIT,                /* 2D identity texture, s*256+t := [0,65535] range */
  TO_DEPTH_MAP,                /* high resolution depth map for shadow mapping */
  TO_LOWRES_DEPTH_MAP,         /* low resolution depth map for shadow mapping */
  TO_DEPTH_MAP_RECT,           /* high resolution depth map for shadow mapping */
  TO_LOWRES_DEPTH_MAP_RECT,    /* low resolution depth map for shadow mapping */
  TO_HW_DEPTH_MAP,             /* high resolution hardware depth map for shadow
                                  mapping */
  TO_LOWRES_HW_DEPTH_MAP,      /* low resolution hardware depth map for shadow
                                  mapping */
  TO_HW_DEPTH_MAP_RECT,        /* high resolution hardware rectangular depth map
                                  for shadow mapping */
  TO_LOWRES_HW_DEPTH_MAP_RECT, /* low resolution hardware rectangular depth map
                                  for shadow mapping */
};


int missingRequiredExtension = 0;  /* Start optimistic. */

int hwLittleEndian;
int littleEndian;

int vsync = 1;
int requestedDepthMapSize = 1024;
int depthMapSize = 1024;

int requestedDepthMapRectWidth = 350;
int requestedDepthMapRectHeight = 300;
int depthMapRectWidth = 350;
int depthMapRectHeight = 300;

GLenum depthMapPrecision = GL_UNSIGNED_BYTE;
GLenum depthMapFormat = GL_LUMINANCE;
GLenum depthMapInternalFormat = GL_INTENSITY8;

GLenum hwDepthMapPrecision = GL_UNSIGNED_SHORT;
GLenum hwDepthMapInternalFormat = GL_DEPTH_COMPONENT16_SGIX;
GLenum hwDepthMapFiltering = GL_LINEAR;

int useCopyTexImage = 1;
int useTextureRectangle = 0;
int mipmapShadowMap = 0;

int img_count;
char img_name[20];
char dep_name[20];


int depthBias8 = 2;
int depthBias16 = 6;
int depthBias24 = 8;
int depthScale8, depthScale16, depthScale24;
GLfloat slopeScale = 4.0;

GLfloat textureLodBias = 0.0;
bool sg_pass;
//GLfloat lv[4];  /* default light position */


//GLUquadricObj *q;

//float eyeAngle = -60.0;
//float eyeHeight = 2.0;
//int xEyeBegin, yEyeBegin, movingEye = 0;
//float lightAngle = 0.85;
//float lightHeight = 8.0;
//int xLightBegin, yLightBegin, movingLight = 0;
int movingLight = 0;
int quickLightMove = 1;
int wireFrame = 0;

GLubyte *framebuffer,*accumbuffer; 
GLubyte *fbuffer[8];
GLubyte *sbuffer[7];
GLfloat *dbuffer;


/* By default, disable back face culling during depth map construction.
   See the comment in the code. */
int depthMapBackFaceCulling = 0;

int winWidth, winHeight;
int needMatrixUpdate = 1;
int needTitleUpdate = 1;
int needDepthMapUpdate = 1;


int rangeMap = 0;

int fullscreen = 0;
int forceEnvCombine = 0;
int useDisplayLists = 1;
int textureFloor;
int showLightViewFrustum = 0;
int showDepthMapMSBs = 1;
//int eyeButton = GLUT_LEFT_BUTTON;
//int lightButton = GLUT_MIDDLE_BUTTON;
int useStencil = 0;
int useDepth24 = 0;
int drawFront = 0;


int hasRegisterCombiners = 0;
int hasTextureEnvCombine = 0;
int hasShadow = 0;
int hasSwapControl = 0;
int hasDepthTexture = 0;
int hasShadowMapSupport = 0;
int useShadowMapSupport = 0;
int hasSeparateSpecularColor = 0;
int hasTextureRectangle = 0;
int hasTextureBorderClamp = 0;
int hasTextureEdgeClamp = 0;
int hasOpenGL12 = 0;

int maxRectangleTextureSize = 0;

double winAspectRatio;

GLdouble eyeFieldOfView;
GLdouble lightFieldOfView;
GLdouble lightNear = 0.5;
GLdouble lightFar = 24.0;

GLfloat  tempcopy[16];
GLdouble eyeViewMatrix[16];
GLdouble eyeFrustumMatrix[16];
GLdouble lightViewMatrix[16];
GLdouble lightInverseViewMatrix[16];
GLdouble lightFrustumMatrix[16];
GLdouble lightInverseFrustumMatrix[16];

GLfloat Ka[4];
GLfloat globalAmbientIntensity[4] = { 0.01, 0.01, 0.01, 1.0 };
GLfloat globalAmbient[4] = {0.01, 0.01, 0.01, 1.0};

GLfloat lightDimColor[] = {LIGHT_DIMMING, LIGHT_DIMMING, LIGHT_DIMMING, 1.0};
GLfloat zero[] = {0.0, 0.0, 0.0, 1.0};
GLfloat lightColor[] = {0.0, 0.0, 0.0, 1.0};
GLfloat ambLightColor[] = {0.0,0.0, 0.0, 1.0};
GLfloat bgLightColor[] = {0.2,0.2,0.2,1.0};

/* Scale and bias [-1,1]^3 clip space into [0,1]^3 texture space. */
GLdouble Smatrix[16] = {
    0.5, 0,   0,   0,
    0,   0.5, 0,   0,
    0,   0,   0.5, 0,
    0.5, 0.5, 0.5, 1.0
};
/* Scale and bias [-1,1]^3 clip space into [0,1]^3 texture space, and then
   move LightSpace Z to S, move 256 * LightSpace Z to T, zero out R, and 
   leave LightSpace Q alone. */
GLdouble RSmatrix[16] = {
    0,   0,   0,   0,
    0,   0,   0,   0,
    0.5, 128, 0,   0,
    0.5, 128, 0,   1.0
};



/* printGLmatrix - handy debugging routine */




GLboolean
invertMatrix(GLdouble *out, const GLdouble *m)
{
/* NB. OpenGL Matrices are COLUMN major. */
#define SWAP_ROWS(a, b) { GLdouble *_tmp = a; (a)=(b); (b)=_tmp; }
#define MAT(m,r,c) (m)[(c)*4+(r)]

 GLdouble wtmp[4][8];
 GLdouble m0, m1, m2, m3, s;
 GLdouble *r0, *r1, *r2, *r3;

 r0 = wtmp[0], r1 = wtmp[1], r2 = wtmp[2], r3 = wtmp[3];

 r0[0] = MAT(m,0,0), r0[1] = MAT(m,0,1),
 r0[2] = MAT(m,0,2), r0[3] = MAT(m,0,3),
 r0[4] = 1.0, r0[5] = r0[6] = r0[7] = 0.0,

 r1[0] = MAT(m,1,0), r1[1] = MAT(m,1,1),
 r1[2] = MAT(m,1,2), r1[3] = MAT(m,1,3),
 r1[5] = 1.0, r1[4] = r1[6] = r1[7] = 0.0,

 r2[0] = MAT(m,2,0), r2[1] = MAT(m,2,1),
 r2[2] = MAT(m,2,2), r2[3] = MAT(m,2,3),
 r2[6] = 1.0, r2[4] = r2[5] = r2[7] = 0.0,

 r3[0] = MAT(m,3,0), r3[1] = MAT(m,3,1),
 r3[2] = MAT(m,3,2), r3[3] = MAT(m,3,3),
 r3[7] = 1.0, r3[4] = r3[5] = r3[6] = 0.0;

 /* choose pivot - or die */
 if (fabs(r3[0])>fabs(r2[0])) SWAP_ROWS(r3, r2);
 if (fabs(r2[0])>fabs(r1[0])) SWAP_ROWS(r2, r1);
 if (fabs(r1[0])>fabs(r0[0])) SWAP_ROWS(r1, r0);
 if (0.0 == r0[0]) {
  return GL_FALSE;
 }

 /* eliminate first variable     */
 m1 = r1[0]/r0[0]; m2 = r2[0]/r0[0]; m3 = r3[0]/r0[0];
 s = r0[1]; r1[1] -= m1 * s; r2[1] -= m2 * s; r3[1] -= m3 * s;
 s = r0[2]; r1[2] -= m1 * s; r2[2] -= m2 * s; r3[2] -= m3 * s;
 s = r0[3]; r1[3] -= m1 * s; r2[3] -= m2 * s; r3[3] -= m3 * s;
 s = r0[4];
 if (s != 0.0) { r1[4] -= m1 * s; r2[4] -= m2 * s; r3[4] -= m3 * s; }
 s = r0[5];
 if (s != 0.0) { r1[5] -= m1 * s; r2[5] -= m2 * s; r3[5] -= m3 * s; }
 s = r0[6];
 if (s != 0.0) { r1[6] -= m1 * s; r2[6] -= m2 * s; r3[6] -= m3 * s; }
 s = r0[7];
 if (s != 0.0) { r1[7] -= m1 * s; r2[7] -= m2 * s; r3[7] -= m3 * s; }

 /* choose pivot - or die */
 if (fabs(r3[1])>fabs(r2[1])) SWAP_ROWS(r3, r2);
 if (fabs(r2[1])>fabs(r1[1])) SWAP_ROWS(r2, r1);
 if (0.0 == r1[1]) {
  return GL_FALSE;
 }

 /* eliminate second variable */
 m2 = r2[1]/r1[1]; m3 = r3[1]/r1[1];
 r2[2] -= m2 * r1[2]; r3[2] -= m3 * r1[2];
 r2[3] -= m2 * r1[3]; r3[3] -= m3 * r1[3];
 s = r1[4]; if (0.0 != s) { r2[4] -= m2 * s; r3[4] -= m3 * s; }
 s = r1[5]; if (0.0 != s) { r2[5] -= m2 * s; r3[5] -= m3 * s; }
 s = r1[6]; if (0.0 != s) { r2[6] -= m2 * s; r3[6] -= m3 * s; }
 s = r1[7]; if (0.0 != s) { r2[7] -= m2 * s; r3[7] -= m3 * s; }

 /* choose pivot - or die */
 if (fabs(r3[2])>fabs(r2[2])) SWAP_ROWS(r3, r2);
 if (0.0 == r2[2]) {
  return GL_FALSE;
 }

 /* eliminate third variable */
 m3 = r3[2]/r2[2];
 r3[3] -= m3 * r2[3], r3[4] -= m3 * r2[4],
 r3[5] -= m3 * r2[5], r3[6] -= m3 * r2[6],
 r3[7] -= m3 * r2[7];

 /* last check */
 if (0.0 == r3[3]) {
  return GL_FALSE;
 }

 s = 1.0/r3[3];              /* now back substitute row 3 */
 r3[4] *= s; r3[5] *= s; r3[6] *= s; r3[7] *= s;

 m2 = r2[3];                 /* now back substitute row 2 */
 s  = 1.0/r2[2];
 r2[4] = s * (r2[4] - r3[4] * m2), r2[5] = s * (r2[5] - r3[5] * m2),
 r2[6] = s * (r2[6] - r3[6] * m2), r2[7] = s * (r2[7] - r3[7] * m2);
 m1 = r1[3];
 r1[4] -= r3[4] * m1, r1[5] -= r3[5] * m1,
 r1[6] -= r3[6] * m1, r1[7] -= r3[7] * m1;
 m0 = r0[3];
 r0[4] -= r3[4] * m0, r0[5] -= r3[5] * m0,
 r0[6] -= r3[6] * m0, r0[7] -= r3[7] * m0;

 m1 = r1[2];                 /* now back substitute row 1 */
 s  = 1.0/r1[1];
 r1[4] = s * (r1[4] - r2[4] * m1), r1[5] = s * (r1[5] - r2[5] * m1),
 r1[6] = s * (r1[6] - r2[6] * m1), r1[7] = s * (r1[7] - r2[7] * m1);
 m0 = r0[2];
 r0[4] -= r2[4] * m0, r0[5] -= r2[5] * m0,
 r0[6] -= r2[6] * m0, r0[7] -= r2[7] * m0;

 m0 = r0[1];                 /* now back substitute row 0 */
 s  = 1.0/r0[0];
 r0[4] = s * (r0[4] - r1[4] * m0), r0[5] = s * (r0[5] - r1[5] * m0),
 r0[6] = s * (r0[6] - r1[6] * m0), r0[7] = s * (r0[7] - r1[7] * m0);

 MAT(out,0,0) = r0[4]; MAT(out,0,1) = r0[5],
 MAT(out,0,2) = r0[6]; MAT(out,0,3) = r0[7],
 MAT(out,1,0) = r1[4]; MAT(out,1,1) = r1[5],
 MAT(out,1,2) = r1[6]; MAT(out,1,3) = r1[7],
 MAT(out,2,0) = r2[4]; MAT(out,2,1) = r2[5],
 MAT(out,2,2) = r2[6]; MAT(out,2,3) = r2[7],
 MAT(out,3,0) = r3[4]; MAT(out,3,1) = r3[5],
 MAT(out,3,2) = r3[6]; MAT(out,3,3) = r3[7]; 

 return GL_TRUE;

#undef MAT
#undef SWAP_ROWS
}

/* dst = transpose(src) */
void
transposeMatrix(GLdouble dst[16], GLdouble src[16])
{
  dst[0] = src[0];
  dst[1] = src[4];
  dst[2] = src[8];
  dst[3] = src[12];

  dst[4] = src[1];
  dst[5] = src[5];
  dst[6] = src[9];
  dst[7] = src[13];

  dst[8] = src[2];
  dst[9] = src[6];
  dst[10] = src[10];
  dst[11] = src[14];

  dst[12] = src[3];
  dst[13] = src[7];
  dst[14] = src[11];
  dst[15] = src[15];
}

/* dst = a + b */
void
addMatrices(GLdouble dst[16], GLdouble a[16], GLdouble b[16])
{
  dst[0] = a[0] + b[0];
  dst[1] = a[1] + b[1];
  dst[2] = a[2] + b[2];
  dst[3] = a[3] + b[3];

  dst[4] = a[4] + b[4];
  dst[5] = a[5] + b[5];
  dst[6] = a[6] + b[6];
  dst[7] = a[7] + b[7];

  dst[8] = a[8] + b[8];
  dst[9] = a[9] + b[9];
  dst[10] = a[10] + b[10];
  dst[11] = a[11] + b[11];

  dst[12] = a[12] + b[12];
  dst[13] = a[13] + b[13];
  dst[14] = a[14] + b[14];
  dst[15] = a[15] + b[15];
}

/* dst = a * b */
void
multMatrices(GLdouble dst[16], const GLdouble a[16], const GLdouble b[16])
{
  int i, j;

  for (i = 0; i < 4; i++) {
    for (j = 0; j < 4; j++) {
      dst[i * 4 + j] =
        b[i * 4 + 0] * a[0 * 4 + j] +
        b[i * 4 + 1] * a[1 * 4 + j] +
        b[i * 4 + 2] * a[2 * 4 + j] +
        b[i * 4 + 3] * a[3 * 4 + j];
    }
  }
}


void
multMatricesdff(GLdouble dst[16], const GLfloat a[16], const GLfloat b[16])
{
  int i, j;

  for (i = 0; i < 4; i++) {
    for (j = 0; j < 4; j++) {
      dst[i * 4 + j] =
        b[i * 4 + 0] * a[0 * 4 + j] +
        b[i * 4 + 1] * a[1 * 4 + j] +
        b[i * 4 + 2] * a[2 * 4 + j] +
        b[i * 4 + 3] * a[3 * 4 + j];
    }
  }
}


void
copyMatrix(GLdouble dst[16], GLdouble src[16])
{
  int i;

  for (i=0; i<16; i++) {
    dst[i] = src[i];
  }
}


void
copyMatrixf(GLdouble dst[16], GLfloat src[16])
{
  int i;

  for (i=0; i<16; i++) {
    dst[i] = src[i];
  }
}

void
copyMatrixff(GLfloat dst[16], GLfloat src[16])
{
  int i;

  for (i=0; i<16; i++) {
    dst[i] = src[i];
  }
}

void
copyMatrixfd(GLfloat dst[16], GLdouble src[16])
{
  int i;

  for (i=0; i<16; i++) {
    dst[i] = src[i];
  }
}


void
buildFrustumMatrix(GLdouble m[16],
                   GLdouble l, GLdouble r, GLdouble b, GLdouble t,
                   GLdouble n, GLdouble f)
{
  m[0] = (2.0*n) / (r-l);
  m[1] = 0.0;
  m[2] = 0.0;
  m[3] = 0.0;

  m[4] = 0.0;
  m[5] = (2.0*n) / (t-b);
  m[6] = 0.0;
  m[7] = 0.0;

  m[8] = (r+l) / (r-l);
  m[9] = (t+b) / (t-b);
  m[10] = -(f+n) / (f-n);
  m[11] = -1.0;

  m[12] = 0.0;
  m[13] = 0.0;
  m[14] = -(2.0*f*n) / (f-n);
  m[15] = 0.0;
}

void
buildPerspectiveMatrix(GLdouble m[16],
                       GLdouble fovy, GLdouble aspect,
                       GLdouble zNear, GLdouble zFar)
{
  GLdouble xmin, xmax, ymin, ymax;

  ymax = zNear * tan(fovy * M_PI / 360.0);
  ymin = -ymax;

  xmin = ymin * aspect;
  xmax = ymax * aspect;

  buildFrustumMatrix(m, xmin, xmax, ymin, ymax, zNear, zFar);
}

/* Build a 4x4 matrix transform based on the parameters for gluLookAt.
 * Code lifted from Brian Paul's MesaGLU.
 */
void
buildLookAtMatrix(GLdouble m[16],
                  GLdouble eyex, GLdouble eyey, GLdouble eyez,
                  GLdouble centerx, GLdouble centery, GLdouble centerz,
                  GLdouble upx, GLdouble upy, GLdouble upz)
{
   GLdouble x[3], y[3], z[3];
   GLdouble mag;

   /* Make rotation matrix */

   /* Z vector */
   z[0] = eyex - centerx;
   z[1] = eyey - centery;
   z[2] = eyez - centerz;
   mag = sqrt( z[0]*z[0] + z[1]*z[1] + z[2]*z[2] );
   if (mag) {  /* mpichler, 19950515 */
      z[0] /= mag;
      z[1] /= mag;
      z[2] /= mag;
   }

   /* Y vector */
   y[0] = upx;
   y[1] = upy;
   y[2] = upz;

   /* X vector = Y cross Z */
   x[0] =  y[1]*z[2] - y[2]*z[1];
   x[1] = -y[0]*z[2] + y[2]*z[0];
   x[2] =  y[0]*z[1] - y[1]*z[0];

   /* Recompute Y = Z cross X */
   y[0] =  z[1]*x[2] - z[2]*x[1];
   y[1] = -z[0]*x[2] + z[2]*x[0];
   y[2] =  z[0]*x[1] - z[1]*x[0];

   /* mpichler, 19950515 */
   /* cross product gives area of parallelogram, which is < 1.0 for
    * non-perpendicular unit-length vectors; so normalize x, y here
    */

   mag = sqrt( x[0]*x[0] + x[1]*x[1] + x[2]*x[2] );
   if (mag) {
      x[0] /= mag;
      x[1] /= mag;
      x[2] /= mag;
   }

   mag = sqrt( y[0]*y[0] + y[1]*y[1] + y[2]*y[2] );
   if (mag) {
      y[0] /= mag;
      y[1] /= mag;
      y[2] /= mag;
   }

#define M(row,col)  m[col*4+row]
   M(0,0) = x[0];  M(0,1) = x[1];  M(0,2) = x[2];  M(0,3) = -x[0]*eyex + -x[1]*eyey + -x[2]*eyez;
   M(1,0) = y[0];  M(1,1) = y[1];  M(1,2) = y[2];  M(1,3) = -y[0]*eyex + -y[1]*eyey + -y[2]*eyez;
   M(2,0) = z[0];  M(2,1) = z[1];  M(2,2) = z[2];  M(2,3) = -z[0]*eyex + -z[1]*eyey + -z[2]*eyez;
   M(3,0) = 0.0;   M(3,1) = 0.0;   M(3,2) = 0.0;   M(3,3) = 1.0;
#undef M
}

/* For debugging purposes. */
void
printMatrix(char *msg, GLdouble m[16])
{
  printf("%s\n", msg);
  printf(" [ %f %f %f %f ]\n", m[0], m[4], m[8], m[12]);
  printf(" [ %f %f %f %f ]\n", m[1], m[5], m[9], m[13]);
  printf(" [ %f %f %f %f ]\n", m[2], m[6], m[10], m[14]);
  printf(" [ %f %f %f %f ]\n\n", m[3], m[7], m[11], m[15]);
}






static void printGLmatrix(GLenum matrix, char *text)
{
  GLdouble m[16]; 
  glGetDoublev(matrix, m);
  printMatrix(text, m);
}


void swapShorts(unsigned int size, GLubyte *depthBuffer)
{
  unsigned int i;
  GLubyte tmp;

  for (i=0; i<size; i++) {
    tmp = depthBuffer[2*i + 0];
    depthBuffer[2*i + 0] = depthBuffer[2*i + 1];
    depthBuffer[2*i + 1] = tmp;
  }
}

void setLittleEndian(void)
{
  unsigned short shortValue = 258;
  unsigned char *byteVersion = (unsigned char*) &shortValue;

  if (byteVersion[1] == 1) {
    assert(byteVersion[0] == 2);
    hwLittleEndian = 1;
  } else {
    assert(byteVersion[0] == 1);
    assert(byteVersion[1] == 2);
    hwLittleEndian = 0;
  }
  littleEndian = hwLittleEndian;
}

#ifdef _WIN32
# define GET_PROC_ADDRESS(x) wglGetProcAddress(x)
#else
# ifdef macintosh
#  ifndef GL_MAC_GET_PROC_ADDRESS_NV
#   define GL_MAC_GET_PROC_ADDRESS_NV 0x84FC
#  endif
#  define GET_PROC_ADDRESS(x) macGetProcAddress(x)
void *(*macGetProcAddress)(char *string);
# else
#  define GET_PROC_ADDRESS(x) glXGetProcAddress(x)
# endif
#endif

int APIENTRY glExtensionSupported(const char *extension)
{ 
  static const GLubyte *extensions = NULL;
  const GLubyte *start;
  GLubyte *where, *terminator;

  /* Extension names should not have spaces. */
  where = (GLubyte *) strchr(extension, ' ');
  if (where || *extension == '\0')
    return 0;

  if (!extensions) {
    extensions = glGetString(GL_EXTENSIONS);
  }
  /* It takes a bit of care to be fool-proof about parsing the
     OpenGL extensions string.  Don't be fooled by sub-strings,
     etc. */
  start = extensions;
  for (;;) {
    /* If your application crashes in the strstr routine below,
       you are probably calling glutExtensionSupported without
       having a current window.  Calling glGetString without
       a current OpenGL context has unpredictable results.
       Please fix your program. */
    where = (GLubyte *) strstr((const char *) start, extension);
    if (!where)
      break;
    terminator = where + strlen(extension);
    if (where == start || *(where - 1) == ' ') {
      if (*terminator == ' ' || *terminator == '\0') {
        return 1;
      }
    }
    start = terminator;
  }
  return 0; 
  }




int supportsOneDotTwo(void)
{
  const char *version;
  int major, minor;

  version = (char *) glGetString(GL_VERSION);
  if (sscanf(version, "%d.%d", &major, &minor) == 2) {
    return major > 1 || minor >= 2;
  }
  return 0;            /* OpenGL version string malformed! */
}


/* initExtensions - Check if required extensions exist. */
void initExtensions(void)
{
  //int missingRequiredExtension = 0;  /* Start optimistic. */
  int cx;
  hasOpenGL12 = supportsOneDotTwo();
  
   
  
  cx = glExtensionSupported("GL_ARB_multitexture");
  if (!glExtensionSupported("GL_ARB_multitexture")) {
    printf("I require the GL_ARB_multitexture OpenGL extension to work.\n");
    missingRequiredExtension = 1;
  }
  if (!glExtensionSupported("GL_EXT_bgra")) {
    printf("I require the GL_EXT_bgra OpenGL extension to work.\n");
    missingRequiredExtension = 1;
  }
  if (glExtensionSupported("GL_NV_register_combiners")) {
    hasRegisterCombiners = 1;
  }
  if (glExtensionSupported("GL_EXT_texture_env_combine")) {
    hasTextureEnvCombine = 1;
  }
  if (glExtensionSupported("GL_SGIX_shadow")) {
    hasShadow = 1;
  }
#ifdef _WIN32
  if (glExtensionSupported("WGL_EXT_swap_control")) {
    hasSwapControl = 1;
  }
#endif
  if (glExtensionSupported("GL_SGIX_depth_texture")) {
    hasDepthTexture = 1;
  }
  if (glExtensionSupported("GL_ARB_texture_border_clamp")) {
    hasTextureBorderClamp = 1;
  }
  if (hasOpenGL12 || glExtensionSupported("GL_EXT_texture_edge_clamp")) {
    hasTextureEdgeClamp = 1;
  }
  if (hasOpenGL12 || glExtensionSupported("GL_EXT_separate_specular_color")) {
    hasSeparateSpecularColor = 1;
  }
//  if (glutExtensionSupported("GL_NV_texture_rectangle")) {
//    hasTextureRectangle = 1;
//    glGetIntegerv(GL_MAX_RECTANGLE_TEXTURE_SIZE_NV, &maxRectangleTextureSize);
//  }

  if (forceEnvCombine) {
    printf("Forcing use of GL_EXT_texture_env_combine.\n");
    hasRegisterCombiners = 0;
    if (!hasTextureEnvCombine) {
      printf("I require the following OpenGL extension to work:\n"
             "  GL_EXT_texture_env_combine\n");
      missingRequiredExtension = 1;
    }
  } else {
    if (!hasRegisterCombiners && !hasTextureEnvCombine) {
      printf("I require one of the following OpenGL extensions to work:\n"
             "  GL_EXT_texture_env_combine\n"
             "  GL_NV_register_combiners\n");
      missingRequiredExtension = 1;
    }
  }
  if (missingRequiredExtension) {
    printf("\nshadowcast: "
           "Exiting because required OpenGL extensions are not supported.\n");
    exit(1);
  }
  if (hasShadow && hasDepthTexture &&
      hasRegisterCombiners && hasSeparateSpecularColor) {
    printf("shadowcast: has hardware shadow mapping support.\n");
    hasShadowMapSupport = 1;
    useShadowMapSupport = 1;
  }

#if defined(NEEDS_GET_PROC_ADDRESS)

# if defined(macintosh)
  glGetPointerv(GL_MAC_GET_PROC_ADDRESS_NV, (GLvoid*) &macGetProcAddress);
# endif

# if !defined(macintosh) 
  /* Retrieve some ARB_multitexture routines. */
  glMultiTexCoord2iARB =
    (PFNGLMULTITEXCOORD2IARBPROC)
    GET_PROC_ADDRESS("glMultiTexCoord2iARB");
  glMultiTexCoord2fvARB =
    (PFNGLMULTITEXCOORD2FVARBPROC)
    GET_PROC_ADDRESS("glMultiTexCoord2fvARB");
  glMultiTexCoord3fARB =
    (PFNGLMULTITEXCOORD3FARBPROC)
    GET_PROC_ADDRESS("glMultiTexCoord3fARB");
  glMultiTexCoord3fvARB =
    (PFNGLMULTITEXCOORD3FVARBPROC)
    GET_PROC_ADDRESS("glMultiTexCoord3fvARB");
  glActiveTextureARB =
    (PFNGLACTIVETEXTUREARBPROC)
    GET_PROC_ADDRESS("glActiveTextureARB");
# endif

  if (hasRegisterCombiners) {
    /* Retrieve all NV_register_combiners routines. */
    glCombinerParameterfvNV =
      (PFNGLCOMBINERPARAMETERFVNVPROC)
      GET_PROC_ADDRESS("glCombinerParameterfvNV");
    glCombinerParameterivNV =
      (PFNGLCOMBINERPARAMETERIVNVPROC)
      GET_PROC_ADDRESS("glCombinerParameterivNV");
    glCombinerParameterfNV =
      (PFNGLCOMBINERPARAMETERFNVPROC)
      GET_PROC_ADDRESS("glCombinerParameterfNV");
    glCombinerParameteriNV =
      (PFNGLCOMBINERPARAMETERINVPROC)
      GET_PROC_ADDRESS("glCombinerParameteriNV");
    glCombinerInputNV =
      (PFNGLCOMBINERINPUTNVPROC)
      GET_PROC_ADDRESS("glCombinerInputNV");
    glCombinerOutputNV =
      (PFNGLCOMBINEROUTPUTNVPROC)
      GET_PROC_ADDRESS("glCombinerOutputNV");
    glFinalCombinerInputNV =
      (PFNGLFINALCOMBINERINPUTNVPROC)
      GET_PROC_ADDRESS("glFinalCombinerInputNV");
    glGetCombinerInputParameterfvNV =
      (PFNGLGETCOMBINERINPUTPARAMETERFVNVPROC)
      GET_PROC_ADDRESS("glGetCombinerInputParameterfvNV");
    glGetCombinerInputParameterivNV =
      (PFNGLGETCOMBINERINPUTPARAMETERIVNVPROC)
      GET_PROC_ADDRESS("glGetCombinerInputParameterivNV");
    glGetCombinerOutputParameterfvNV =
      (PFNGLGETCOMBINEROUTPUTPARAMETERFVNVPROC)
      GET_PROC_ADDRESS("glGetCombinerOutputParameterfvNV");
    glGetCombinerOutputParameterivNV =
      (PFNGLGETCOMBINEROUTPUTPARAMETERIVNVPROC)
      GET_PROC_ADDRESS("glGetCombinerOutputParameterivNV");
    glGetFinalCombinerInputParameterfvNV =
      (PFNGLGETFINALCOMBINERINPUTPARAMETERFVNVPROC)
      GET_PROC_ADDRESS("glGetFinalCombinerInputParameterfvNV");
    glGetFinalCombinerInputParameterivNV =
      (PFNGLGETFINALCOMBINERINPUTPARAMETERIVNVPROC)
      GET_PROC_ADDRESS("glGetFinalCombinerInputParameterivNV");
  }

# ifdef _WIN32
  if (hasSwapControl) {
    /* Retrieve all WGL_EXT_swap_control routines. */
    wglSwapIntervalEXT = 
      (PFNWGLSWAPINTERVALEXTPROC)
      GET_PROC_ADDRESS("wglSwapIntervalEXT");
    wglGetSwapIntervalEXT = 
      (PFNWGLGETSWAPINTERVALEXTPROC)
      GET_PROC_ADDRESS("wglGetSwapIntervalEXT");
  }
# endif  /* _WIN32 */

#endif  /* NEEDS_GET_PROC_ADDRESS */
}


//Light-Functions

/* setAmbientAndDiffuseMaterial - update the ambient and diffuse
   material setting for per-vertex lighting, and if single-pass hardware
   shadow mapping is also supported, update a constant color in the
   NV_register_combiners with the global ambient. */
void setAmbientAndDiffuseMaterial(GLfloat *material)
{
  int i;

  glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, material);

  if (hasShadow && hasRegisterCombiners) {
    for (i=0; i<4; i++) {
      Ka[i] = material[i] * globalAmbientIntensity[i];
    }
    /* The single-pass hardware shadow mapping mode programs
       the register combiners to treat CONSTANT_COLOR1 as the
       global ambient. */
    glCombinerParameterfvNV(GL_CONSTANT_COLOR1_NV, Ka);
  }
}


//Setup Functions

void setupEyeView(void)
{
  glMatrixMode(GL_PROJECTION);
  glLoadMatrixd(eyeFrustumMatrix);

  glMatrixMode(GL_MODELVIEW);
  glLoadMatrixd(eyeViewMatrix);
}

void setupLightView(int square)
{
  glMatrixMode(GL_PROJECTION);
  if (square) {
    glLoadMatrixd(lightFrustumMatrix);
  } else {
    glLoadIdentity();
    glScalef(winAspectRatio, 1, 1);
    glMultMatrixd(lightFrustumMatrix);
  }

  glMatrixMode(GL_MODELVIEW);
  glLoadMatrixd(lightViewMatrix);
}

// Texture Functions

void configTexgen(int rangeMap, int bits)
{
  GLfloat p[4];
  GLfloat wrapScale;
  GLdouble m1[16], m2[16];

  if (rangeMap) {
    /* Generate the depth map space planar distance as the S/Q texture
       coordinate. */

    if ((depthMapPrecision == GL_UNSIGNED_SHORT) && !useShadowMapSupport) {
      glBindTexture(GL_TEXTURE_2D, TO_MAP_16BIT);
      glEnable(GL_TEXTURE_2D);
      
    } else {
      glBindTexture(GL_TEXTURE_1D, TO_MAP_8BIT);
      glEnable(GL_TEXTURE_1D);
      glDisable(GL_TEXTURE_2D);
      
    }

    copyMatrix(m1, RSmatrix);
    wrapScale = (1 << bits);
    m1[0] *= wrapScale;
    m1[4] *= wrapScale;
    m1[8] *= wrapScale;
    m1[12] *= wrapScale;
  } else {
    GLenum target;
    GLuint texobj;

    copyMatrix(m1, Smatrix);
    
     /* Pick the 2D target and texture object. */
      target = GL_TEXTURE_2D;
      if (useShadowMapSupport) {
        if (movingLight && quickLightMove) {
          texobj = TO_LOWRES_HW_DEPTH_MAP;
        } else {
          texobj = TO_HW_DEPTH_MAP;
        }
      } else {
        if (movingLight && quickLightMove) {
          texobj = TO_LOWRES_DEPTH_MAP;
        } else {
          texobj = TO_DEPTH_MAP;
        }
      }
   // }
    glBindTexture(target, texobj);
    glEnable(target);
  }

  multMatrices(m2, m1, lightFrustumMatrix);
  multMatrices(m1, m2, lightViewMatrix);

  /* S and Q are always needed */
  p[0] = m1[0];
  p[1] = m1[4];
  p[2] = m1[8];
  p[3] = m1[12];
  glTexGenfv(GL_S, GL_EYE_PLANE, p);
  p[0] = m1[3];
  p[1] = m1[7];
  p[2] = m1[11];
  p[3] = m1[15];
  glTexGenfv(GL_Q, GL_EYE_PLANE, p);

  glEnable(GL_TEXTURE_GEN_S);
  glEnable(GL_TEXTURE_GEN_Q);

  if (rangeMap && (bits == 0)) {
    /* rangeMap is a 1D mode so it does not need to generate T or R. */
    glDisable(GL_TEXTURE_GEN_T);
    glDisable(GL_TEXTURE_GEN_R);
  } else {
    if (useShadowMapSupport) {
      /* Hardware shadow mapping is a 2D texturing mode (uses S and T) that
         also compares with R so S, T, and R must be generated. */
      p[0] = m1[1];
      p[1] = m1[5];
      p[2] = m1[9];
      p[3] = m1[13];
      glTexGenfv(GL_T, GL_EYE_PLANE, p);

      p[0] = m1[2];
      p[1] = m1[6];
      p[2] = m1[10];
      p[3] = m1[14];
      glTexGenfv(GL_R, GL_EYE_PLANE, p);

      glEnable(GL_TEXTURE_GEN_S);
      glEnable(GL_TEXTURE_GEN_T);
      glEnable(GL_TEXTURE_GEN_R);
    } else {
      /* Standard 2D texture mapping uses S, T, and Q, but does not use R. */
      p[0] = m1[1];
      p[1] = m1[5];
      p[2] = m1[9];
      p[3] = m1[13];
      glTexGenfv(GL_T, GL_EYE_PLANE, p);

      glEnable(GL_TEXTURE_GEN_S);
      glEnable(GL_TEXTURE_GEN_T);
      glDisable(GL_TEXTURE_GEN_R);
    }
  }
}

void disableTexgen(void)
{
  glDisable(GL_TEXTURE_GEN_S);
  glDisable(GL_TEXTURE_GEN_T);
  glDisable(GL_TEXTURE_GEN_R);
  glDisable(GL_TEXTURE_GEN_Q);
  if (hasTextureRectangle) {
    glDisable(GL_TEXTURE_RECTANGLE_NV);
  }
  glDisable(GL_TEXTURE_2D);
  glDisable(GL_TEXTURE_1D);
}


// Combiners Functions

void configCombiners(int pass)
{
  if (hasRegisterCombiners) {
    if (pass == 1) {
      assert(depthMapPrecision == GL_UNSIGNED_SHORT ||
             depthMapPrecision == GL_UNSIGNED_BYTE);
      
      glCombinerParameteriNV(GL_NUM_GENERAL_COMBINERS_NV, 1);

      /* Discard entire RGB combiner stage 0 */
      glCombinerOutputNV(GL_COMBINER0_NV, GL_RGB,
        GL_DISCARD_NV, GL_DISCARD_NV, GL_DISCARD_NV,
        GL_NONE, GL_NONE, GL_FALSE, GL_FALSE, GL_FALSE);

      /* Aa = one */
      glCombinerInputNV(GL_COMBINER0_NV, GL_ALPHA, GL_VARIABLE_A_NV,
        GL_ZERO, GL_UNSIGNED_INVERT_NV, GL_ALPHA);
      /* Ba = texture0a = depth map MSB */
      glCombinerInputNV(GL_COMBINER0_NV, GL_ALPHA, GL_VARIABLE_B_NV,
        GL_TEXTURE0_ARB, GL_SIGNED_IDENTITY_NV,
        littleEndian ? GL_ALPHA : GL_BLUE);
      /* Ca = one */
      glCombinerInputNV(GL_COMBINER0_NV, GL_ALPHA, GL_VARIABLE_C_NV,
        GL_ZERO, GL_UNSIGNED_INVERT_NV, GL_ALPHA);
      /* Da = texture1a = range MSB */
      glCombinerInputNV(GL_COMBINER0_NV, GL_ALPHA, GL_VARIABLE_D_NV,
        GL_TEXTURE1_ARB, GL_SIGNED_NEGATE_NV, GL_ALPHA);

      /* spare0a = 1 * tex0a + 1 * -tex1a = tex0a - tex1a */
      glCombinerOutputNV(GL_COMBINER0_NV, GL_RGB,
        GL_DISCARD_NV, GL_DISCARD_NV, GL_SPARE0_NV,
        GL_NONE, GL_NONE, GL_FALSE, GL_FALSE, GL_FALSE);

    } else {
      assert(pass == 2);
      assert(depthMapPrecision == GL_UNSIGNED_SHORT);

      glCombinerParameteriNV(GL_NUM_GENERAL_COMBINERS_NV, 2);

      /* Argb = one */
      glCombinerInputNV(GL_COMBINER0_NV, GL_RGB, GL_VARIABLE_A_NV,
        GL_ZERO, GL_UNSIGNED_INVERT_NV, GL_RGB);
      /* Brgb = texture0rgb = depth map LSB */
      glCombinerInputNV(GL_COMBINER0_NV, GL_RGB, GL_VARIABLE_B_NV,
        GL_TEXTURE0_ARB, GL_SIGNED_IDENTITY_NV,
        littleEndian ? GL_RGB : GL_ALPHA);
      /* Crgb = one */
      glCombinerInputNV(GL_COMBINER0_NV, GL_RGB, GL_VARIABLE_C_NV,
        GL_ZERO, GL_UNSIGNED_INVERT_NV, GL_RGB);
      /* Drgb = texture1rgb = range LSB */
      glCombinerInputNV(GL_COMBINER0_NV, GL_RGB, GL_VARIABLE_D_NV,
        GL_TEXTURE1_ARB, GL_SIGNED_NEGATE_NV, GL_RGB);

      /* spare0rgb = 1 * tex0rgb + 1 * -tex1rgb = tex0rgb - tex1rgb */
      glCombinerOutputNV(GL_COMBINER0_NV, GL_RGB,
        GL_DISCARD_NV, GL_DISCARD_NV, GL_SPARE0_NV,
        GL_NONE, GL_NONE, GL_FALSE, GL_FALSE, GL_FALSE);

      /* Aa = one */
      glCombinerInputNV(GL_COMBINER0_NV, GL_ALPHA, GL_VARIABLE_A_NV,
        GL_ZERO, GL_UNSIGNED_INVERT_NV, GL_ALPHA);
      /* Ba = texture1a = range MSB */
      glCombinerInputNV(GL_COMBINER0_NV, GL_ALPHA, GL_VARIABLE_B_NV,
        GL_TEXTURE1_ARB, GL_UNSIGNED_IDENTITY_NV, GL_ALPHA);
      /* Ca = one */
      glCombinerInputNV(GL_COMBINER0_NV, GL_ALPHA, GL_VARIABLE_C_NV,
        GL_ZERO, GL_UNSIGNED_INVERT_NV, GL_ALPHA);
      /* Da = texture0a = depth map MSB */
      glCombinerInputNV(GL_COMBINER0_NV, GL_ALPHA, GL_VARIABLE_D_NV,
        GL_TEXTURE0_ARB, GL_UNSIGNED_INVERT_NV,
        littleEndian ? GL_ALPHA : GL_BLUE);

      /* spare0a = 1 * tex1a + 1 * (1-tex0a) - 0.5 = tex1a - tex0a + 0.5 */
      glCombinerOutputNV(GL_COMBINER0_NV, GL_ALPHA,
        GL_DISCARD_NV, GL_DISCARD_NV, GL_SPARE0_NV,
        GL_NONE, GL_BIAS_BY_NEGATIVE_ONE_HALF_NV,
        GL_FALSE, GL_FALSE, GL_FALSE);

      /* Discard entire RGB combiner stage 1 */
      glCombinerOutputNV(GL_COMBINER1_NV, GL_RGB,
        GL_DISCARD_NV, GL_DISCARD_NV, GL_DISCARD_NV,
        GL_NONE, GL_NONE, GL_FALSE, GL_FALSE, GL_FALSE);

      /* Aa = one */
      glCombinerInputNV(GL_COMBINER1_NV, GL_ALPHA, GL_VARIABLE_A_NV,
        GL_ZERO, GL_UNSIGNED_INVERT_NV, GL_ALPHA);
      /* Ba = spare0b = tex0b - tex1b */
      glCombinerInputNV(GL_COMBINER1_NV, GL_ALPHA, GL_VARIABLE_B_NV,
        GL_SPARE0_NV, GL_SIGNED_IDENTITY_NV, GL_BLUE);
      /* Ca = zero */
      glCombinerInputNV(GL_COMBINER1_NV, GL_ALPHA, GL_VARIABLE_C_NV,
        GL_ZERO, GL_UNSIGNED_IDENTITY_NV, GL_ALPHA);
      /* Da = zero */
      glCombinerInputNV(GL_COMBINER1_NV, GL_ALPHA, GL_VARIABLE_D_NV,
        GL_ZERO, GL_UNSIGNED_IDENTITY_NV, GL_ALPHA);

      /* spare0a = (tex1 - tex0 + 0.5) > 0.5 ? 0 : tex0b - tex1b */
      glCombinerOutputNV(GL_COMBINER1_NV, GL_ALPHA,
        GL_DISCARD_NV, GL_DISCARD_NV, GL_SPARE0_NV,
        GL_NONE, GL_NONE, GL_FALSE, GL_FALSE,
        GL_TRUE);  /* Use the MUX operation. */
    }

    /* A = one */
    glFinalCombinerInputNV(GL_VARIABLE_A_NV,
      GL_ZERO, GL_UNSIGNED_INVERT_NV, GL_RGB);
    /* B = primary color */
    glFinalCombinerInputNV(GL_VARIABLE_B_NV,
      GL_PRIMARY_COLOR_NV, GL_UNSIGNED_IDENTITY_NV, GL_RGB);
    /* C = zero */
    glFinalCombinerInputNV(GL_VARIABLE_C_NV,
      GL_ZERO, GL_UNSIGNED_IDENTITY_NV, GL_RGB); //Test GL_FOG
    /* D = zero */
    glFinalCombinerInputNV(GL_VARIABLE_D_NV,
      GL_ZERO, GL_UNSIGNED_IDENTITY_NV, GL_RGB);
    /* RGB = A*B + (1-A)*C + D = primary color */

    /* G = spare0a = max(0, tex0 - tex1) */
    glFinalCombinerInputNV(GL_VARIABLE_G_NV,
      GL_SPARE0_NV, GL_UNSIGNED_IDENTITY_NV, GL_ALPHA);
    /* Alpha = max(0, tex0 - tex1) */

    glEnable(GL_REGISTER_COMBINERS_NV);
  } else {
    glActiveTextureARB(GL_TEXTURE0_ARB);
    glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_COMBINE_EXT);

    glTexEnvi(GL_TEXTURE_ENV, GL_COMBINE_RGB_EXT, GL_REPLACE);
    glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE0_RGB_EXT, GL_PRIMARY_COLOR_EXT);
    glTexEnvi(GL_TEXTURE_ENV, GL_OPERAND0_RGB_EXT, GL_SRC_COLOR);

    glTexEnvi(GL_TEXTURE_ENV, GL_COMBINE_ALPHA_EXT, GL_REPLACE);
    glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE0_ALPHA_EXT, GL_TEXTURE);
    glTexEnvi(GL_TEXTURE_ENV, GL_OPERAND0_ALPHA_EXT, GL_SRC_ALPHA);

    glActiveTextureARB(GL_TEXTURE1_ARB);
    glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_COMBINE_EXT);

    glTexEnvi(GL_TEXTURE_ENV, GL_COMBINE_RGB_EXT, GL_REPLACE);
    glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE0_RGB_EXT, GL_PREVIOUS_EXT);
    glTexEnvi(GL_TEXTURE_ENV, GL_OPERAND0_RGB_EXT, GL_SRC_COLOR);

    glTexEnvi(GL_TEXTURE_ENV, GL_COMBINE_ALPHA_EXT, GL_ADD_SIGNED_EXT);
    glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE0_ALPHA_EXT, GL_PREVIOUS_EXT);
    glTexEnvi(GL_TEXTURE_ENV, GL_OPERAND0_ALPHA_EXT, GL_SRC_ALPHA);
    glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE1_ALPHA_EXT, GL_TEXTURE);
    glTexEnvi(GL_TEXTURE_ENV, GL_OPERAND1_ALPHA_EXT, GL_ONE_MINUS_SRC_ALPHA);
  }
 // if (rcDebug) {
 //   printRegisterCombinersState();
 // }
}

void configCombinersForHardwareShadowPass(int withTexture)
{
  glCombinerParameteriNV(GL_NUM_GENERAL_COMBINERS_NV, 2);
  glCombinerParameterfvNV(GL_CONSTANT_COLOR0_NV, lightDimColor);
  glCombinerParameterfvNV(GL_CONSTANT_COLOR1_NV, Ka);

  /* Argb = one */
  glCombinerInputNV(GL_COMBINER0_NV, GL_RGB, GL_VARIABLE_A_NV,
    GL_ZERO, GL_UNSIGNED_INVERT_NV, GL_RGB);
  /* Brgb = constant0rgb = light dimming */
  glCombinerInputNV(GL_COMBINER0_NV, GL_RGB, GL_VARIABLE_B_NV,
    GL_CONSTANT_COLOR0_NV, GL_UNSIGNED_IDENTITY_NV, GL_RGB);
  /* Crgb = 1.0 - constant0rgb = 1.0 - light dimming */
  glCombinerInputNV(GL_COMBINER0_NV, GL_RGB, GL_VARIABLE_C_NV,
    GL_CONSTANT_COLOR0_NV, GL_UNSIGNED_INVERT_NV, GL_RGB);
  /* Drgb = texture0rgb = unshadowed percentage */
  glCombinerInputNV(GL_COMBINER0_NV, GL_RGB, GL_VARIABLE_D_NV,
    GL_TEXTURE0_ARB, GL_UNSIGNED_IDENTITY_NV, GL_RGB);

  /* spare0rgb = light dimming + (1.0 - light dimming) * unshadowed percentage */
  glCombinerOutputNV(GL_COMBINER0_NV, GL_RGB,
    GL_DISCARD_NV, GL_DISCARD_NV, GL_SPARE0_NV,
    GL_NONE, GL_NONE, GL_FALSE, GL_FALSE, GL_FALSE);

  /* Discard entire Alpha combiner stage 0 */
  glCombinerOutputNV(GL_COMBINER0_NV, GL_ALPHA,
    GL_DISCARD_NV, GL_DISCARD_NV, GL_DISCARD_NV,
    GL_NONE, GL_NONE,
    GL_FALSE, GL_FALSE, GL_FALSE);

  /* Argb = spare0rgb = light dimming + (1.0 - light dimming) * unshadowed percentage */
  glCombinerInputNV(GL_COMBINER1_NV, GL_RGB, GL_VARIABLE_A_NV,
    GL_SPARE0_NV, GL_UNSIGNED_IDENTITY_NV, GL_RGB);
  /* Brgb = primary color = light's diffuse contribution */
  glCombinerInputNV(GL_COMBINER1_NV, GL_RGB, GL_VARIABLE_B_NV,
    GL_PRIMARY_COLOR_NV, GL_UNSIGNED_IDENTITY_NV, GL_RGB);
  /* Crgb = constant 1 = Ka = ambient contribution */
  glCombinerInputNV(GL_COMBINER1_NV, GL_RGB, GL_VARIABLE_C_NV,
    GL_CONSTANT_COLOR1_NV, GL_UNSIGNED_IDENTITY_NV, GL_RGB);
  /* Drgb = 1 */
  glCombinerInputNV(GL_COMBINER1_NV, GL_RGB, GL_VARIABLE_D_NV,
    GL_ZERO, GL_UNSIGNED_INVERT_NV, GL_RGB);

  /* spare0rgb =
     (light dimming + (1.0 - light dimming) * unshadowed percentage) * diffuse +
     ambient */
  glCombinerOutputNV(GL_COMBINER1_NV, GL_RGB,
    GL_DISCARD_NV, GL_DISCARD_NV, GL_SPARE0_NV,
    GL_NONE, GL_NONE, GL_FALSE, GL_FALSE, GL_FALSE);

  /* Discard entire Alpha combiner stage 1 */
  glCombinerOutputNV(GL_COMBINER1_NV, GL_ALPHA,
    GL_DISCARD_NV, GL_DISCARD_NV, GL_DISCARD_NV,
    GL_NONE, GL_NONE, GL_FALSE, GL_FALSE,
    GL_TRUE);  /* Use the MUX operation. */
  
  if (withTexture) {
    /* A = texture1 = decal */
    glFinalCombinerInputNV(GL_VARIABLE_A_NV,
      GL_TEXTURE1_ARB, GL_UNSIGNED_IDENTITY_NV, GL_RGB);
  } else {
    /* A = one */
    glFinalCombinerInputNV(GL_VARIABLE_A_NV,
      GL_ZERO, GL_UNSIGNED_INVERT_NV, GL_RGB);
  }
  /* B = spare0 */
  glFinalCombinerInputNV(GL_VARIABLE_B_NV,
    GL_SPARE0_NV, GL_UNSIGNED_IDENTITY_NV, GL_RGB);
  /* C = zero */
  glFinalCombinerInputNV(GL_VARIABLE_C_NV,
    GL_ZERO, GL_UNSIGNED_IDENTITY_NV, GL_RGB);
  /* E = unshadowed percentage */
  glFinalCombinerInputNV(GL_VARIABLE_E_NV,
    GL_TEXTURE0_ARB, GL_UNSIGNED_IDENTITY_NV, GL_RGB);
  /* F = specular */
  glFinalCombinerInputNV(GL_VARIABLE_F_NV,
    GL_SECONDARY_COLOR_NV, GL_UNSIGNED_IDENTITY_NV, GL_RGB);
  /* D = EF product = unshadowed percentage * specular contribution */
  glFinalCombinerInputNV(GL_VARIABLE_D_NV,
    GL_E_TIMES_F_NV, GL_UNSIGNED_IDENTITY_NV, GL_RGB);
  /* RGB = A*B + (1-A)*C + D = (texture1 OR 1) * space0 + specular * unshadowed */

  /* G = 1 */
  glFinalCombinerInputNV(GL_VARIABLE_G_NV,
    GL_ZERO, GL_UNSIGNED_INVERT_NV, GL_ALPHA);
  /* Alpha = 1 */

  glEnable(GL_REGISTER_COMBINERS_NV);

//  if (rcDebug) {
//    printRegisterCombinersState();
//  }
}

void disableCombiners(void)
{
  if (hasRegisterCombiners) {
    glDisable(GL_REGISTER_COMBINERS_NV);
  }
}





// Update Function

void updateMatrices(void)
{
  buildPerspectiveMatrix(lightFrustumMatrix, 
    lightFieldOfView, 1.0, lightNear, lightFar);

  if (showLightViewFrustum) {
    invertMatrix(lightInverseViewMatrix, lightViewMatrix);
    invertMatrix(lightInverseFrustumMatrix, lightFrustumMatrix);
  }
}

void
updateDepthBias(int delta)
{
  GLfloat scale, bias;

  if (useShadowMapSupport) {
    if (hwDepthMapPrecision == GL_UNSIGNED_SHORT) {
      depthBias16 += delta;
      scale = slopeScale;
      bias = depthBias16 * depthScale16;
    } else {
      depthBias24 += delta;
      scale = slopeScale;
      bias = depthBias24 * depthScale24;
    }
  } else {
    if (depthMapPrecision == GL_UNSIGNED_SHORT) {
      depthBias16 += delta;
      scale = slopeScale;
      bias = depthBias16 * depthScale16;
    } else {
      depthBias8 += delta;
      scale = slopeScale;
      bias = depthBias8 * depthScale8;
    }
  }
  glPolygonOffset(scale, bias);
  needTitleUpdate = 1;
  needDepthMapUpdate = 1;
}

void
updateDepthMapSize(void)
{
  int oldDepthMapSize = depthMapSize;

  depthMapSize = requestedDepthMapSize;
  while ((winWidth < depthMapSize) || (winHeight < depthMapSize)) {
    depthMapSize >>= 1;  // Half the depth map size
  }
  if (depthMapSize < 1) {
    /* Just in case. */
    depthMapSize = 1;
  }
  if (depthMapSize != requestedDepthMapSize) {
    //printf("shadowcast: reducing depth map from %d to %d based on window size %dx%d\n",
    //  requestedDepthMapSize, depthMapSize, winWidth, winHeight);
  }
  if (!useTextureRectangle) {
    if (oldDepthMapSize != depthMapSize) {
      needDepthMapUpdate = 1;
      needTitleUpdate = 1;
      //glutPostRedisplay();
    }
  }
}

void updateDepthScale(void)
{
  GLint depthBits;

  glGetIntegerv(GL_DEPTH_BITS, &depthBits);
  if (depthBits < 24) {
    depthScale24 = 1;
  } else {
    depthScale24 = 1 << (depthBits - 24);
  }
  if (depthBits < 16) {
    depthScale16 = 1;
  } else {
    depthScale16 = 1 << (depthBits - 16);
  }
  if (depthBits < 8) {
    depthScale8 = 1;
  } else {
    depthScale8 = 1 << (depthBits - 8);
  }
  needDepthMapUpdate = 1;
}


void
useBestShadowMapClamping(GLenum target)
{
  if (hasTextureBorderClamp) {
    glTexParameteri(target, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_BORDER_ARB);
    glTexParameteri(target, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_BORDER_ARB);
  } else {
    /* We really want "clamp to border", but this may be good enough. */
    if (hasTextureEdgeClamp) {
      glTexParameteri(target, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
      glTexParameteri(target, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
    } else {
      /* A bad option, but still better than "repeat". */
      glTexParameteri(target, GL_TEXTURE_WRAP_S, GL_CLAMP);
      glTexParameteri(target, GL_TEXTURE_WRAP_T, GL_CLAMP);
    }
  }
}

int WriteBMP(char *file, int width, int height, unsigned char *image)
{
   // Remember that bitmaps are made up of 3 parts, the info header,
   // the file header, and the image.  We will have to write each one.

   FILE *pFile;                  // Our file pointer.
   BITMAPINFOHEADER infoHeader;  // info header.
   BITMAPFILEHEADER fileHeader;  // file header.
   int index;                    // index for the for loop.
   unsigned char tempColors;     // Use to change image from RGB to BGR.

   pFile = fopen(file, "wb");    // Open the file for write.

   if(!pFile) { return 0; }

   // Define whats going in the infoHeader.
   infoHeader.biSize = sizeof(BITMAPINFOHEADER);
   infoHeader.biPlanes        = 1;
   infoHeader.biBitCount      = 24;
   infoHeader.biCompression   = BI_RGB;
   infoHeader.biSizeImage     = width * abs(height) * 3;
   infoHeader.biXPelsPerMeter = 0;
   infoHeader.biYPelsPerMeter = 0;
   infoHeader.biClrUsed       = 0;
   infoHeader.biClrImportant  = 0;
   infoHeader.biWidth         = width;
   infoHeader.biHeight        = height;

   // Define whats going in the file header.
   fileHeader.bfSize       = sizeof(BITMAPFILEHEADER);
   fileHeader.bfType       = 0x4D42;
   fileHeader.bfReserved1  = 0;
   fileHeader.bfReserved2  = 0;
   fileHeader.bfOffBits    = sizeof(BITMAPFILEHEADER) + sizeof(BITMAPINFOHEADER);


   // Change image from RGB to BGR.  Thats how .bmp is saved.
   for(index = 0; index < infoHeader.biSizeImage; index += 3)
      {
         tempColors = image[index];
         image[index] = image[index + 2];
         image[index + 2] = tempColors;
      }

   // Now lets write to the file.
   // Write file header.
   fwrite(&fileHeader, 1, sizeof(BITMAPFILEHEADER), pFile);

   // Write info header.
   fwrite(&infoHeader, 1, sizeof(BITMAPINFOHEADER), pFile);

   // Write image.
   fwrite(image, 1, infoHeader.biSizeImage, pFile);

   // Last but not least close the file.
   fclose(pFile);
   return 0;
}

void ShowDepthMap(void)
{
	GLenum target;
  GLuint texobj;

  //updateDepthMap(pSceneRenderer,mtransform, cullFlag,
   //       this, m_pPresenter->GetTime()););

  glViewport(0, 0, winWidth, winHeight);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glScalef(winAspectRatio, 1, 1);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();

  glColor3f(1,1,1);
  glDisable(GL_LIGHTING);
  glDisable(GL_DEPTH_TEST);

  if (useShadowMapSupport) {
    if (movingLight && quickLightMove) {
      
      
        target = GL_TEXTURE_2D;
        texobj = TO_LOWRES_HW_DEPTH_MAP;
      
    } else {
      
        target = GL_TEXTURE_2D;
        texobj = TO_HW_DEPTH_MAP;
      
    }

    glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_COMBINE_EXT);
    glTexEnvi(GL_TEXTURE_ENV, GL_COMBINE_RGB_EXT, GL_REPLACE);
    glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE0_RGB_EXT, GL_TEXTURE);
    glTexEnvi(GL_TEXTURE_ENV, GL_OPERAND0_RGB_EXT, GL_SRC_COLOR);
    // Turn off shadow mapping comparisons so we can "show" the depth
    //   values directly.  With GL_TEXTURE_COMPARE_SGIX set to GL_FALSE,
    //   a GL_DEPTH_COMPONENT texture behaves as a GL_LUMINANCE texture.
    //   However, while the depth components may be 16-bit or 24-bit values,
    //   be warned that viewing the depth components as luminance values
    //   will only show you the most significant bits. 
    glTexParameteri(target, GL_TEXTURE_COMPARE_SGIX, GL_FALSE);
  } else {
    if (movingLight && quickLightMove) {
      
        target = GL_TEXTURE_2D;
        texobj = TO_LOWRES_DEPTH_MAP;
      
    } else {
      
      
        target = GL_TEXTURE_2D;
        texobj = TO_DEPTH_MAP;
       }

    glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_COMBINE_EXT);
    glTexEnvi(GL_TEXTURE_ENV, GL_COMBINE_RGB_EXT, GL_REPLACE);
    glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE0_RGB_EXT, GL_TEXTURE);
    if (showDepthMapMSBs) {
      if (littleEndian) {
        glTexEnvi(GL_TEXTURE_ENV, GL_OPERAND0_RGB_EXT, GL_SRC_ALPHA);
      } else {
        glTexEnvi(GL_TEXTURE_ENV, GL_OPERAND0_RGB_EXT, GL_SRC_COLOR);
      }
    } else {
      if (littleEndian) {
        glTexEnvi(GL_TEXTURE_ENV, GL_OPERAND0_RGB_EXT, GL_SRC_COLOR);
      } else {
        glTexEnvi(GL_TEXTURE_ENV, GL_OPERAND0_RGB_EXT, GL_SRC_ALPHA);
      }
    }
  }

  glEnable(target);
  glBindTexture(target, texobj);
  glTexEnvf(GL_TEXTURE_FILTER_CONTROL_EXT, GL_TEXTURE_LOD_BIAS_EXT, textureLodBias);

  
    // Standard 2D textures use normalized texture coordinates in the
    //   [0..1]x[0..1] rnage. 
    glBegin(GL_QUADS);
      glTexCoord2f(0,0);
      glVertex2f(-1,-1);
      glTexCoord2f(1,0);
      glVertex2f(1,-1);
      glTexCoord2f(1,1);
      glVertex2f(1,1);
      glTexCoord2f(0,1);
      glVertex2f(-1,1);
    glEnd();
  

  if (useShadowMapSupport) {
    // Re-enable shadow map comparsions. 
    glTexParameteri(target, GL_TEXTURE_COMPARE_SGIX, GL_TRUE);
  }

  glTexEnvf(GL_TEXTURE_FILTER_CONTROL_EXT, GL_TEXTURE_LOD_BIAS_EXT, 0.0);
  glDisable(target);
  glEnable(GL_DEPTH_TEST);
  glEnable(GL_LIGHTING);
}






void initShadowGL(void)
{
    GLubyte texmap[256*256*2];
        unsigned int i, j;
        GLint depthBits;

        #if defined(WIN32)
         if (hasSwapControl) {
          if (vsync) {
           wglSwapIntervalEXT(1);
		  } else {
         wglSwapIntervalEXT(0);
		  }
		 }
        #endif
   
  
		glGetIntegerv(GL_DEPTH_BITS, &depthBits);
		
		if (depthBits >= 24) {
		hwDepthMapPrecision = GL_UNSIGNED_INT;
		hwDepthMapInternalFormat = GL_DEPTH_COMPONENT24_SGIX;
		}

		glClearStencil(0);
		glClearColor(0,0,0,0);

		glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
		glPixelStorei(GL_PACK_ALIGNMENT, 1);

		glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);
		glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);
		glTexGeni(GL_R, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);
		glTexGeni(GL_Q, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);

		glLightModelfv(GL_LIGHT_MODEL_AMBIENT, globalAmbient);

		  // Make 8-bit identity texture that maps (s)=(z) to [0,255]/255. 
		for (i=0; i<256; i++) {
			texmap[i] = i;
			}
		glBindTexture(GL_TEXTURE_1D, TO_MAP_8BIT);
		glTexImage1D(GL_TEXTURE_1D, 0, GL_INTENSITY8,
			256, 0, GL_LUMINANCE, GL_UNSIGNED_BYTE, texmap);
		glTexParameteri(GL_TEXTURE_1D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
		glTexParameteri(GL_TEXTURE_1D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
		glTexParameteri(GL_TEXTURE_1D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);

		// Make 16-bit identity texture that maps (s,t)=(z,z*256) to [0,65535]/65535. //
		for (j=0; j<256; j++) {
			for (i=0; i<256; i++) {
			texmap[j*512+i*2+0] = j;  // Luminance has least sigificant bits. 
			texmap[j*512+i*2+1] = i;  // Alpha has most sigificant bits. 
			}
		}
		glBindTexture(GL_TEXTURE_2D, TO_MAP_16BIT);
		glTexImage2D(GL_TEXTURE_2D, 0, GL_LUMINANCE8_ALPHA8,
			256, 256, 0, GL_LUMINANCE_ALPHA, GL_UNSIGNED_BYTE, texmap);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);

		// GL_LEQUAL ensures that when fragments with equal depth are
		//generated within a single rendering pass, the last fragment
		//results. 
		glDepthFunc(GL_LEQUAL);
		glEnable(GL_DEPTH_TEST);

		glEnable(GL_LIGHTING);
		//lEnable(GL_LIGHT0);
		//glEnable(GL_LIGHT1);
		//glEnable(GL_LIGHT2);
		//glLightfv(GL_LIGHT0, GL_AMBIENT, zero);
		//glLightfv(GL_LIGHT0, GL_SPECULAR, lightColor);
		//glLightfv(GL_LIGHT0, GL_DIFFUSE, lightColor);
        //glLightfv(GL_LIGHT1, GL_AMBIENT, zero);
		//glLightfv(GL_LIGHT1, GL_SPECULAR, lightColor);
		//glLightfv(GL_LIGHT1, GL_DIFFUSE, lightColor);
        //glLightfv(GL_LIGHT2, GL_AMBIENT, zero);
		//glLightfv(GL_LIGHT2, GL_SPECULAR, lightColor);
		//glLightfv(GL_LIGHT2, GL_DIFFUSE, lightColor);
  
		glLineStipple(1, 0xf0f0);

		glMaterialf(GL_FRONT, GL_SHININESS, 30.0);

		glEnable(GL_CULL_FACE);
		glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER, 1);
		#if 000
		// This would work too. 
		glEnable(GL_RESCALE_NORMAL_EXT);
		#else
		glEnable(GL_NORMALIZE);
		#endif

		// Make the NVIDIA logo display list. 
		//  nvidiaLogoDList = makeNVidiaLogo(300);

  
		if (hasRegisterCombiners) {
			depthMapFormat = GL_LUMINANCE_ALPHA;
			depthMapInternalFormat = GL_LUMINANCE8_ALPHA8;
			depthMapPrecision = GL_UNSIGNED_SHORT;
		}
		updateDepthScale();
		updateDepthBias(0);  // Update with no offset change. 

		if (drawFront) {
		glDrawBuffer(GL_FRONT);
		glReadBuffer(GL_FRONT);
		}

		
}




//#endif