#ifndef __SHADOWRENDERING_H__
#define __SHADOWRENDERING_H__


#include <stdlib.h>
#include <stdio.h>
#include <malloc.h>
#include <assert.h>
#include <math.h>

#include "OpenGl/OpenGl.h"
//#include "OpenGL/AFX/DIBR/glext.h"  // Problem mit anderem glext.h
#include "OpenGL/shadow/inc/gl/glext.h"
#include "OpenGL/base/MediaObjectProxy.h"

// added for use of texture_object_2d and glh namespace
#include <OpenGL/shadow/inc/glh/glh_obs.h>
using namespace glh;

  
#ifndef M_PI
#define M_PI 3.14159265358979323846
#endif 

#define MAX_SIZE 1024
#define LIGHT_DIMMING 0.2
#define MAG16 65535.0f
#define MAG24 16777215.0f 


#define DEPTHMAP_SIZE 1024
//#define TEX_SIZE 1024 // jetzt als variable


class ShadowRendering {

public:
	ShadowRendering(int Height, int Width, double AspectRatio);
	virtual ~ShadowRendering();


 /* Texture objects. */
 enum {
   TO_RESERVED = 0,             /* zero is not a valid OpenGL texture object ID */
   TO_MAP_8BIT,                 /* 1D identity texture, s := [0,255] range */
   TO_MAP_16BIT,                /* 2D identity texture, s*256+t := [0,65535] range */
   TO_DEPTH_MAP,                /* high resolution depth map for shadow mapping */
   TO_LOWRES_DEPTH_MAP,         /* low resolution depth map for shadow mapping */
   TO_DEPTH_MAP_RECT,           /* high resolution depth map for shadow mapping */
   TO_LOWRES_DEPTH_MAP_RECT,    /* low resolution depth map for shadow mapping */
   TO_HW_DEPTH_MAP,             /* high resolution hardware depth map for shadow
                                  mapping */
   TO_LOWRES_HW_DEPTH_MAP,      /* low resolution hardware depth map for shadow
                                  mapping */
   TO_HW_DEPTH_MAP_RECT,        /* high resolution hardware rectangular depth map
                                  for shadow mapping */
   TO_LOWRES_HW_DEPTH_MAP_RECT, /* low resolution hardware rectangular depth map
                                  for shadow mapping */
 }; 
	
 int missingRequiredExtension; // default 0;  /* Start optimistic. */

 int hwLittleEndian;
 int littleEndian;

 int vsync;                                   // default 1
 int requestedDepthMapSize;                   // default 1024
 int depthMapSize;                            // default 1024 

 int requestedDepthMapRectWidth;              // default 350
 int requestedDepthMapRectHeight;							// default 300
 int depthMapRectWidth;                       // default 350
 int depthMapRectHeight;                      // default 300

 GLenum depthMapPrecision;                    // default GL_UNSIGNED_BYTE
 GLenum depthMapFormat;												// default GL_LUMINANCE
 GLenum depthMapInternalFormat;               // default GL_INTENSITY8

 GLenum hwDepthMapPrecision;                  // default GL_UNSIGNED_SHORT
 GLenum hwDepthMapInternalFormat;             // default GL_DEPTH_COMPONENT16_SGIX
 GLenum hwDepthMapFiltering;                  // default GL_LINEAR

 int useCopyTexImage;                         // default 1
 int useTextureRectangle;											// default 0	
 int mipmapShadowMap;													// default 0

 int depthBias8;                              // default 2
 int depthBias16;                             // default 6
 int depthBias24;                             // default 8
 int depthScale8, depthScale16, depthScale24;
 GLfloat slopeScale;													// default 4.0

 GLfloat textureLodBias;                      // default 0.0
 bool sg_pass; 

 int movingLight;                             // default 0
 int quickLightMove;                          // default 1
 int wireFrame;                               // default 0

 GLubyte *framebuffer,*accumbuffer; 
 GLubyte *fbuffer[8];
 GLubyte *sbuffer[7];
 GLfloat *dbuffer;

 /* By default, disable back face culling during depth map construction.
   See the comment in the code. */
 int depthMapBackFaceCulling;                 // default  0

 int winWidth, winHeight;
 int needMatrixUpdate;                        // default 1
 int needTitleUpdate;                         // default 1
 int needDepthMapUpdate;                      // default 1


 int rangeMap;                                // default 0

 int fullscreen;                              // default 0
 int forceEnvCombine;                         // default 0
 int useDisplayLists;                         // default 1
 int textureFloor;
 int showLightViewFrustum;                    // default 0
 int showDepthMapMSBs;                        // default 1

 int useStencil;                              // default 0
 int useDepth24;                              // default 0
 int drawFront;                               // default 0


 int hasRegisterCombiners;                    // default 0
 int hasTextureEnvCombine;                    // default 0
 int hasShadow;                               // default 0
 int hasSwapControl;													// default 0
 int hasDepthTexture;													// default 0
 int hasShadowMapSupport;                     // default 0
 int useShadowMapSupport;                     // default 0
 int hasSeparateSpecularColor;                // default 0
 int hasTextureRectangle;                     // default 0
 int hasTextureBorderClamp;                   // default 0
 int hasTextureEdgeClamp;                     // default 0
 int hasOpenGL12;                             // default 0

 int maxRectangleTextureSize;                 // default 0

 int light_num_max;                           // Max number of lightsource
 float itv[5];                                // factors softshadow calculation
 float jitter[5];                             // Jittering factors
 unsigned char softpass_num;                  // Number of softpasses
 float penumbra;                              // penumbra  
 
 double winAspectRatio;

 GLdouble eyeFieldOfView;
 GLdouble lightFieldOfView;
 GLdouble lightNear;                          // default 0.5
 GLdouble lightFar;                           // default 24.0

 GLfloat  tempcopy[16];
 GLdouble eyeViewMatrix[16];
 GLdouble eyeFrustumMatrix[16];

 GLdouble lightViewMatrix[16];
 GLdouble lightInverseViewMatrix[16];
 GLdouble lightFrustumMatrix[16];
 GLdouble lightInverseFrustumMatrix[16];

 GLfloat Ka[4];
 GLfloat globalAmbientIntensity[4];           // default { 0.01, 0.01, 0.01, 1.0 }
 GLfloat globalAmbient[4];                    // default {0.01, 0.01, 0.01, 1.0}
 
 GLfloat light_pos[4];                        // Light position
 GLfloat light_dir[4];                        // Light direction   
 
 GLfloat cam_pos[3];                          // Camerapostion x,y,z
 
 GLfloat lightDimColor[4];                    // default {LIGHT_DIMMING, LIGHT_DIMMING, LIGHT_DIMMING, 1.0}
 GLfloat zero[4];                             // default {0.0, 0.0, 0.0, 1.0}

 GLfloat maximal[4];                          // default {1.0, 1.0, 1.0, 1.0}

 GLfloat lightColor[4];                       // default {0.0, 0.0, 0.0, 1.0}
 GLfloat ambLightColor[4];                    // default {0.0,0.0, 0.0, 1.0}
 GLfloat bgLightColor[4];                     // default {0.2,0.2,0.2,1.0}

//- added by M.Frank 11-2005 --------------------------------------------------------

 GLfloat lightConstantAttenuation;						// default 1.0
 GLfloat lightLinearAttenuation;							// default 0.01
 GLfloat lightQuadraticAttenuation;						// default 0.01
 GLfloat lightSpotCutoff;											// default 40.0
 GLfloat lightSpotExponent;										// default 5.0

 int amb_light_num_max;												// default 0

 GLfloat ambLightColorSum[4];                 // default {0.0,0.0, 0.0, 1.0}
 GLfloat lightRadius;													// default 100

 GLdouble eyeInverseViewMatrix[16];						
 static const GLint hwBorderColor[4];

 GLfloat fogColor[4];													// default { 1.0, 1.0, 1.0, 1.0 }
 bool fogType;																// linear or log, (default lin)
 GLfloat fogVisRange;														
 GLfloat fogStart;														// default 0.0
 GLfloat fogEnd;															// default 1.0
 GLfloat fogDensity;													// default 1.5

 int hasOpenGL20;                             // default 0
 GLfloat white[4];                            // default {1.0, 1.0, 1.0, 1.0}
 GLfloat red[4];															// default {1.0, 0.0, 0.0, 1.0}

 bool softshadowsEnabled;											// default false
 bool counterEnabled;													// default true
 bool perpixelLightingEnabled;								// default true
 int frames;																	// default 0
 float fps;																		// default 0

 GLint TEX_SIZE;


 	/*--------------------------------------------------------------------------
Objects - added by M.Frank 11-2005 
--------------------------------------------------------------------------*/

//textur objects
tex_object_2D light_image, decal, nvlogo, depthmap_camera, ambientlight, basetexture,
diff_lightmap_light0, diff_lightmap_light1, diff_lightmap_light2, diff_lightmap_light3,
diff_lightmap_light4, diff_lightmap_light5, diff_lightmap_light6, diff_lightmap_light7,
spec_lightmap_light0, spec_lightmap_light1, spec_lightmap_light2, spec_lightmap_light3,
spec_lightmap_light4, spec_lightmap_light5, spec_lightmap_light6, spec_lightmap_light7,
shadowmap_light0, shadowmap_light1, shadowmap_light2, shadowmap_light3,
shadowmap_light4, shadowmap_light5, shadowmap_light6, shadowmap_light7,
depthmap_light0, depthmap_light1, depthmap_light2, depthmap_light3,
depthmap_light4, depthmap_light5, depthmap_light6, depthmap_light7,
diff_scenemap, spec_scenemap,
shadowmap_softpass0, shadowmap_softpass1, shadowmap_softpass2, shadowmap_softpass3,shadowmap_softpass4,
curDepthmap, curShadowmap, curSoftShadowmap, curLightmap, test;

display_list screen;

//--------------------------------------------------------------------------

// -----------------------------------------------------------------------------------

 /* Scale and bias [-1,1]^3 clip space into [0,1]^3 texture space. */
 GLdouble Smatrix[16]; // = {
     //0.5, 0,   0,   0,
     //0,   0.5, 0,   0,
     //0,   0,   0.5, 0,
     //0.5, 0.5, 0.5, 1.0
 //};
 /* Scale and bias [-1,1]^3 clip space into [0,1]^3 texture space, and then
   move LightSpace Z to S, move 256 * LightSpace Z to T, zero out R, and 
   leave LightSpace Q alone. */
 GLdouble RSmatrix[16];// = {
     //0,   0,   0,   0,
     //0,   0,   0,   0,
     //0.5, 128, 0,   0,
     //0.5, 128, 0,   1.0
 //};

 GLboolean invertMatrix(GLdouble *out, const GLdouble *m); 
 void transposeMatrix(GLdouble dst[16], GLdouble src[16]);
 void addMatrices(GLdouble dst[16], GLdouble a[16], GLdouble b[16]);
 void multMatrices(GLdouble dst[16], const GLdouble a[16], const GLdouble b[16]);
 void multMatricesdff(GLdouble dst[16], const GLfloat a[16], const GLfloat b[16]);
 void copyMatrix(GLdouble dst[16], GLdouble src[16]);
 void copyMatrixf(GLdouble dst[16], GLfloat src[16]);
 void copyMatrixff(GLfloat dst[16], GLfloat src[16]);
 void copyMatrixfd(GLfloat dst[16], GLdouble src[16]);
 void buildFrustumMatrix(GLdouble m[16],
                         GLdouble l, GLdouble r, GLdouble b, GLdouble t,
                         GLdouble n, GLdouble f);
 void buildPerspectiveMatrix(GLdouble m[16],
                             GLdouble fovy, GLdouble aspect,
                             GLdouble zNear, GLdouble zFar);
 void buildLookAtMatrix(GLdouble m[16],
                        GLdouble eyex, GLdouble eyey, GLdouble eyez,
                        GLdouble centerx, GLdouble centery, GLdouble centerz,
                        GLdouble upx, GLdouble upy, GLdouble upz);
 void swapShorts(unsigned int size, GLubyte *depthBuffer);
 void setLittleEndian(void);
 int APIENTRY glExtensionSupported(const char *extension);
 int supportsOneDotTwo(void);
 // functions for OpenGL and GLSL initialisation - added by M.Frank 11-2005
 void initOpenGLExtensions(void);
 int supportsTwoDotZero(void);
 void cleanExit(int exitval);
 void printInfoLog(GLhandleARB object);
 void addShader(GLhandleARB programObject,
								const GLcharARB *shaderSource,
								GLenum shaderType,
								bool linkProgram);
 void initGLSLShaders(void);
 void initpBuffers(void);
 void updatergbpBufferSize();
 void updatefogpBufferSize();
 void initMultipassTextures(void);
 void initOpenGLStats(void);

 // new functions for multipass shadow rendering - added by M.Frank 11-2005

 void renderDiffLightMap(unsigned int light_number,
										 unsigned int type_of_light,
										 MediaObjectProxy * pSceneRenderer,
										 const Matrix& Transformation, 
										 const CullFlag& CullFlag,
										 OpenGLVisualRenderer* pRenderer, 
										 DWORD dwTime);

  void renderSpecLightMap(unsigned int light_number,
										 unsigned int type_of_light,
										 MediaObjectProxy * pSceneRenderer,
										 const Matrix& Transformation, 
										 const CullFlag& CullFlag,
										 OpenGLVisualRenderer* pRenderer, 
										 DWORD dwTime); 
// Functions for hard Shadow calculation
 void renderDepthMap(unsigned int light_number,
										 MediaObjectProxy * pSceneRenderer,
										 const Matrix& TransformationX, 
										 const CullFlag& CullFlag,
										 OpenGLVisualRenderer* pRenderer, 
										 DWORD dwTime);

 void renderShadowMap(unsigned int light_number,
										 MediaObjectProxy * pSceneRenderer,
										 const Matrix& Transformation,
										 const CullFlag& CullFlag,
										 OpenGLVisualRenderer* pRenderer, 
										 DWORD dwTime);

 void renderShadowMap_pass3(unsigned int light_number,
										 MediaObjectProxy * pSceneRenderer,
										 const Matrix& Transformation,
										 const CullFlag& CullFlag,
										 OpenGLVisualRenderer* pRenderer, 
										 DWORD dwTime);

// Functions for soft Shadow calculation
 void renderSoftDepthMap(unsigned int light_number,
										 MediaObjectProxy * pSceneRenderer,
										 const Matrix& TransformationX, 
										 const CullFlag& CullFlag,
										 OpenGLVisualRenderer* pRenderer, 
										 DWORD dwTime);

 void renderSoftShadowMap(unsigned int light_number,
										 unsigned int softpass_number,
										 MediaObjectProxy * pSceneRenderer,
										 const Matrix& Transformation,
										 const CullFlag& CullFlag,
										 OpenGLVisualRenderer* pRenderer, 
										 DWORD dwTime);

 void renderSoftShadowMap_pass3(MediaObjectProxy * pSceneRenderer,
										 const Matrix& Transformation,
										 const CullFlag& CullFlag,
										 OpenGLVisualRenderer* pRenderer, 
										 DWORD dwTime);

 void addSoftShadowMaps(unsigned int light_number);

 void	renderShadowMapBlank(unsigned int light_number); //fr alle auer Spot und alle ohne Schatten
 void renderSceneMap(void);					//Anzahl aller Lichtquellen (max.8)
 void renderBaseTexture(MediaObjectProxy * pSceneRenderer,
												 const Matrix& Transformation, 
					   						 const CullFlag& CullFlag,
											   OpenGLVisualRenderer* pRenderer, 
												 DWORD dwTime);
 void renderAmbientLight(MediaObjectProxy * pSceneRenderer,
												 const Matrix& Transformation, 
												 const CullFlag& CullFlag,
												 OpenGLVisualRenderer* pRenderer, 
												 DWORD dwTime);
 void renderAmbientBlack(void);			//falls ambient disabled	
 void renderCameraDepth(MediaObjectProxy * pSceneRenderer,
									 const Matrix& Transformation, 
									 const CullFlag& CullFlag,
									 OpenGLVisualRenderer* pRenderer, 
									 DWORD dwTime);			
 void renderDiffSceneLight(unsigned int light_number);
 void renderSpecSceneLight(unsigned int light_number);
 void addDiffLights(void);					//alle diffusen SceneMaps
 void addSpecLights(void);					//alle specularen SceneMaps
 void composeEverything(void);			//SceneMap+Fog+Base+Amb...
 void set_spot(float cutoff, float exponent);
 void set_lightRadius(float radius);
 void set_attenuation(float x,float y,float z);
 void set_ambLightColorSum(void);
 void set_fogColor(float r,float g,float b,float i, float visRange, bool type);
 void reset_ambLightColorSum(void);
 void paintTexture(void);
 void drawFramerate(void);
 void calculateFramerate(void);
 void eye_linear_texgen(void);
 void obj_linear_texgen(void);
 void texgen(bool);
 void activatergbpBuffer(void);
 void configTexgen_v2(int rangeMap, int bits);
 void disableTexgen_v2(void);
 //-----------------------------------
 void initExtensions(void);
 void setAmbientAndDiffuseMaterial(GLfloat *material);
 void setupEyeView(void);
 void setupLightView(int square);
 void configTexgen(int rangeMap, int bits);
 void disableTexgen(void);
 void configCombiners(int pass);
 void configCombinersForHardwareShadowPass(int withTexture);
 void disableCombiners(void);
 void updateMatrices(void);
 void updateDepthBias(int delta);
 void updateDepthMapSize(void);
 void updateDepthScale(void);
 void useBestShadowMapClamping(GLenum target);
 void initShadowGL(void);
 //void updateDepthMap(void);
 void framebuffers_alloc(void);
 void framebuffers_realloc(int Height, int Width);
 void framebuffers_clear(int mode);
 void framebuffers_free(void);
 void sbuffer_accum(void);
 void fbuffer_accum(void);
 void set_win(int Width, int Height,double AspectRatio, int texsize);
 void setup_renderMode(bool hwenable);
 void set_lightColor(float r,float g,float b,float i);
 void set_ambLightColor(float r,float g,float b,float i);
 void set_lightPos(float x, float y,float z,float w);
 void set_lightDir(float x, float y,float z,float w);
 void setup_hwRendering(int mode);
 void setup_dualTexRendering(int mode);
 int WriteBMP(char *file, int width, int height, unsigned char *image);
 void ShowDepthMap(void);
 void DebugDepthMap(char *file, int width, int height, GLfloat *image);
 void updateDepthMap(MediaObjectProxy * pSceneRenderer, const Matrix& rTransformation, 
                      const CullFlag& rCullFlag,
		              OpenGLVisualRenderer* pRenderer, 
                      DWORD dwTime); 
 void drawHardwareShadowPass(MediaObjectProxy * pSceneRenderer, const Matrix& rTransformation, 
                      const CullFlag& rCullFlag,
		              OpenGLVisualRenderer* pRenderer, 
                      DWORD dwTime);


};


#endif
