/*
	This software module was originally developed by ENST in the course of
development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.
	This software module is an implementation of a part of one or more MPEG-4
Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1) 
standard. ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license 
to this software module or modifications thereof for use in hardware or software 
products claiming conformance to the MPEG-4 Systems (ISO/IEC 14496-1). 
Those intending to use this software module in hardware or software products are 
advised that its use may infringe existing patents.
	The original developer of this software module and his/her company, the 
subsequent editors and their companies, and ISO/IEC have no liability for use of 
this software module or modifications thereof in an implementation. Copyright is 
not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
	ENST retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using the code 
for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright    
notice must be included in all copies or derivative works.              
						Copyright  2002.
*/

///////////////////////////////////////////////////////////////////////////
// File:  AnchorProxy.cpp
//
// Author: Jean Le Feuvre - ENST 2002
//
// Description:
//   Anchor proxy object 
//
// Created: August 2002.
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include "MediaObj.h"
#include "Presenter.h"
#include "AnchorProxy.h"

#include "App.h"

AnchorProxy* Presenter::InitAnchor(Anchor* pObj)
{
	return new  AnchorProxy(pObj);
}


AnchorProxy::AnchorProxy(MediaObject* pMO): GroupNodeProxy(pMO),UserEventHandlerProxy(pMO)
{
	m_LastUpdateTime = 0;
}

AnchorProxy::~AnchorProxy()
{	
}

//special case: anchor acts as its own handler
void AnchorProxy::Render(Effects *effects)
{
	DWORD now = 1 + GroupNodeProxy::GetObject()->GetUpdateTime();
	if (m_LastUpdateTime != now) {
		UpdateEnabled();
		m_LastUpdateTime = now;
	}

	if (m_bIsEnabled )
		effects->AddUserEventHandler((UserEventHandlerProxy *) this, &effects->transform);

	RenderChildren(effects);

	if (m_bIsEnabled )
		effects->RemoveHandler((UserEventHandlerProxy *) this);
}


BOOL AnchorProxy::OnUserEvent(EventInfo const* pEvent, Matrix2D *)
{
	// If the button is first pressed, then generate an
	// isActive = TRUE event, and grab all movement events
	if (pEvent->m_event != EVENT_MOUSE_DOWN) return FALSE;

	int i=0;
	while (1) {
		LPCSTR string = (GetAnchor()->GetUrl(i) != NULL) ? GetAnchor()->GetUrl(i)->urlValue : NULL;
		if (!string) break;
		if (GroupNodeProxy::GetPresenter()->GetExecutive()->GetApplication()->Navigate(string))
			break;

		i++;
	}
	return TRUE;	
}


void AnchorProxy::UpdateEnabled()
{
	m_bIsEnabled = FALSE;
	LPCSTR string = (GetAnchor()->GetUrl(0) != NULL) ? GetAnchor()->GetUrl(0)->urlValue : NULL;
	if (string && strcmp(string, "") )
		m_bIsEnabled = TRUE;
}

//forces activation of anchor
void Anchor::Onactivate() 
{
	EventInfo event(EVENT_MOUSE_DOWN, 0, 0, 0);

	AnchorProxy *pObj = (AnchorProxy *) (GroupNodeProxy *) GetRenderingObject();

	pObj->OnUserEvent(&event, NULL);
}
