/*
	This software module was originally developed by ENST in the course of
development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.
	This software module is an implementation of a part of one or more MPEG-4
Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1) 
standard. ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license 
to this software module or modifications thereof for use in hardware or software 
products claiming conformance to the MPEG-4 Systems (ISO/IEC 14496-1). 
Those intending to use this software module in hardware or software products are 
advised that its use may infringe existing patents.
	The original developer of this software module and his/her company, the 
subsequent editors and their companies, and ISO/IEC have no liability for use of 
this software module or modifications thereof in an implementation. Copyright is 
not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
	ENST retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using the code 
for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright    
notice must be included in all copies or derivative works.              
						Copyright  2003.
*/

///////////////////////////////////////////////////////////////////////////
// File:  AudioBufferProxy.h
//
// Author: Jean Le Feuvre - ENST 2003
//
//
///////////////////////////////////////////////////////////////////////////


#ifndef AUDIOBUFFER_PROXY_H
#define AUDIOBUFFER_PROXY_H

#include "Audible.h"
#include "AudioMixer.h"
#include "TimedNode.h"

class AudioBufferProxy : public AudioGroup, TimedNode
{
public:
	AudioBufferProxy(MediaObject* pMO);
	~AudioBufferProxy();

	/*all virtual funcs*/	
	/*from AudioGroup */
	virtual void AddInput(AudioProd *src);
	/*from AudioProd */
	virtual void GetData(DWORD bytesWanted, LPBYTE &ptr, DWORD &bytes_available, DWORD &time_stamp);
	virtual void DataUsed(DWORD bytesUsed);
	virtual float GetIntensity() {return 1.0; }
	virtual float GetSpeed();
	virtual float GetPanLeft() {return 1.0; }
	virtual float GetPanRight() {return 1.0; }
	virtual BOOL IsMuted();
	//AudioBuffer ignores delay since it emulates its clock
	virtual DWORD GetTime(unsigned int audioDelay);
	virtual BOOL IsInitialized() {return m_bIsInit; }

	/*from timedNode */
	virtual void UpdateTimeInfo();
	/*from traversable*/
	virtual void Render(Effects *effects);

	void OnUpdate();

private:
	void CheckMixer();
	void Activate();
	void Deactivate();

	AudioMixer m_Mixer;

	DWORD m_LastUpdateTime;
	int m_lastCount;
	BOOL m_bDone;

	unsigned char *m_pOutBuffer;
	unsigned int m_pOutSize, m_currentWritePos, m_currentReadPos;

	double m_startTime;
	double m_lastLength;

	BOOL m_bIsInit;

	BOOL m_registeredWithRenderer;
	DWORD m_BytesPerSec;
	DWORD m_CurTS;
};

#endif	//AUDIOBUFFER_PROXY_H
