/*
	This software module was originally developed by ENST in the course of
development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.
	This software module is an implementation of a part of one or more MPEG-4
Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1) 
standard. ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license 
to this software module or modifications thereof for use in hardware or software 
products claiming conformance to the MPEG-4 Systems (ISO/IEC 14496-1). 
Those intending to use this software module in hardware or software products are 
advised that its use may infringe existing patents.
	The original developer of this software module and his/her company, the 
subsequent editors and their companies, and ISO/IEC have no liability for use of 
this software module or modifications thereof in an implementation. Copyright is 
not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
	ENST retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using the code 
for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright    
notice must be included in all copies or derivative works.              
						Copyright  2003
*/

///////////////////////////////////////////////////////////////////////////
// File:  AudioMixer.h
//
// Author: Jean Le Feuvre - ENST 2003
//
///////////////////////////////////////////////////////////////////////////


#ifndef _AUDIOMIXER_H
#define _AUDIOMIXER_H

#include "ztl.h"

class AudioProd;
class Presenter;

class AudioMixer
{
public:
	AudioMixer();
	~AudioMixer();
	void SetPresenter(Presenter *pres) {m_pPresenter = pres;}
	float GetOutputIntensity();

	BOOL IsSourceSupported(AudioProd *src);
	//returns TRUE if output SR/nChan changed, 0 otherwise
	BOOL AddSource(AudioProd *src);
	void RemoveSource(AudioProd *src);

	void RemoveAll();

	//By default the mixer works with as many channels as there are in all sources (eg stereo if any stereo input
	//otherwise mono)
	//this forces the mixer to work in a given mode (used by audioBuffer). 
	void SetOutputChannels(unsigned int numChannels);

	//we assume the buffer can always be completely filled for now - to change
	//returns the number of bytes written
	//@checkTiming: if true resyncs audio frame if more than AUDIO_TOLERANCE_MS late
	unsigned int GetMix(void *output, unsigned int output_len, BOOL checkTiming, unsigned int audioDelayMS=0);

	//used to specifically override left/right pan (AudioMix node does that)
	//note that the output shall be reseted by the user before performing a mix
	unsigned int MixStream(void *output, unsigned int output_len, unsigned int streamIndex, float panLeft, float panRight, BOOL checkTiming);

	//these are automatically computed when adding a source
	DWORD m_nSamplesPerSec, m_nBitsPerSample;
	WORD m_nChannels;


	ZMutex m_Lock;
	ZArray<AudioProd *> m_Streams;


private:

	Presenter *m_pPresenter;

	//performs upsampling only
	int AudioMix(signed short *dst, void *src, unsigned int src_len, AudioProd *stream, float volL, float volR, unsigned int &src_read, float speed);

	BOOL m_bChannelsForced;
	/*current audio delay*/
	unsigned int m_audio_delay_ms;
};


#endif //_AUDIOMIXER_H