/*
	This software module was originally developed by ENST in the course of
development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.
	This software module is an implementation of a part of one or more MPEG-4
Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1) 
standard. ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license 
to this software module or modifications thereof for use in hardware or software 
products claiming conformance to the MPEG-4 Systems (ISO/IEC 14496-1). 
Those intending to use this software module in hardware or software products are 
advised that its use may infringe existing patents.
	The original developer of this software module and his/her company, the 
subsequent editors and their companies, and ISO/IEC have no liability for use of 
this software module or modifications thereof in an implementation. Copyright is 
not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
	ENST retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using the code 
for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright    
notice must be included in all copies or derivative works.              
						Copyright  2002.
*/

///////////////////////////////////////////////////////////////////////////
// File:  AudioRenderer.h
//
// Author: Jean Le Feuvre
// Description:
//		audio renderer
//
// Created: December, 2002.
//
//	
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include <assert.h>
#include "dx-errors.h"
#include "App.h"
#include "AudioRenderer.h"
#include "Presenter.h"
#include "Audible.h"



#define DEF_NUM_BUFFERS					8
#define DEF_NUM_BUFFERS_PER_SEC			15

AudioRenderer::AudioRenderer(Presenter* pPresenter, void* pInfo)
{
	ZRegistry reg("Osmo4");
	char temp[10];
	unsigned long s = 10;

	m_pPresenter = pPresenter;
	m_hWnd = (HWND) pInfo;

	m_pDS = NULL;
	m_pSoundBuffer = NULL;
	/*init com*/
	CoInitialize(NULL);
	
	intensity = 0.0f;

	m_nbBufPerSec = DEF_NUM_BUFFERS_PER_SEC;
	if (reg.QueryValue("AudioFPS", (unsigned char *)temp, s)) {
		m_nbBufPerSec = atoi(temp);
		if (m_nbBufPerSec>30) m_nbBufPerSec=30;
	}
	m_numBuffers = DEF_NUM_BUFFERS;
	s=10;
	if (reg.QueryValue("AudioBuffers", (unsigned char *)temp, s)) {
		m_numBuffers = atoi(temp);
		if (m_numBuffers > MAX_NUM_BUFFER) m_numBuffers = MAX_NUM_BUFFER;
	}


	m_bCheckAudio = TRUE;
	if (reg.QueryValue("AudioSkip", (unsigned char *)temp, s)) {
		m_bCheckAudio = temp[0];
	}

	m_bCheckAudio = TRUE;

	m_bActive = TRUE;
	m_bPaused = FALSE;

	m_Mixer.SetPresenter(m_pPresenter);

	// Start the main thread of the audio renderer - higher priority
	ZThread<AudioRenderer> thread(AudioLoop, this, THREAD_PRIORITY_TIME_CRITICAL);
//	ZThread<AudioRenderer> thread(AudioLoop, this, THREAD_PRIORITY_NORMAL);
	thread.Begin();
}

AudioRenderer::~AudioRenderer()
{

	m_bActive = FALSE;
	m_mtxThread.P ();
	m_mtxThread.V ();

	Reset();

	//release direct sound
	m_pDS->Release();
	m_pDS = NULL;

	CoUninitialize();
}


BOOL AudioRenderer::Init()
{

	m_bPaused = FALSE;
	// Initialise the audio format output by the sound card
	memset(&m_Format, 0, sizeof(WAVEFORMATEX));
	m_Format.wFormatTag = WAVE_FORMAT_PCM;

	char msg[500];
	if (m_pDS) return TRUE;

	// Create the DirectSound object, the interface to the sound card
	HRESULT hResult = DirectSoundCreate(NULL, &m_pDS, NULL);
	if (hResult != DS_OK) {
		sprintf(msg, "DirectSoundCreate() failed.\n  %s", HRESULTtoText(hResult));
		m_pPresenter->GetExecutive()->GetApplication()->OnFatalError(AUDIO_INIT_FAILED, msg);
		return FALSE;
	}

	hResult = m_pDS->SetCooperativeLevel(m_hWnd, DSSCL_EXCLUSIVE);
	if (hResult != DS_OK) {
		sprintf(msg, "SetCooperativeLevel() failed.\n  %s", HRESULTtoText(hResult));
		m_pPresenter->GetExecutive()->GetApplication()->OnFatalError(AUDIO_INIT_FAILED, msg);
		return FALSE;
	}
	
	m_Lock.P();
	BOOL res = UpdateOutputFormat();
	m_Lock.V();
	return res;
}

void AudioRenderer::Reset()
{

	if (m_pSoundBuffer) {
		m_pSoundBuffer->Stop();
		m_pSoundBuffer->Release();
		m_pSoundBuffer = NULL;
	}
}


int AudioRenderer::UpdateOutputFormat()
{

	m_Format.cbSize = sizeof (WAVEFORMATEX);
	m_Format.wFormatTag = WAVE_FORMAT_PCM;
	m_Format.nChannels = m_Mixer.m_nChannels;
	m_Format.nSamplesPerSec = m_Mixer.m_nSamplesPerSec;
	m_Format.wBitsPerSample = m_Mixer.m_nBitsPerSample;
	m_Format.nBlockAlign = m_Format.nChannels * m_Format.wBitsPerSample / 8;
	m_Format.nAvgBytesPerSec = m_Format.nSamplesPerSec * m_Format.nBlockAlign;

	/*
		there is a bug in case the audio buffer base size is too small, the signaling seems
		to be out of sync then, so keep the buffer small and increase the number of signal points
	*/
	m_BufferSize = m_Format.nAvgBytesPerSec / m_nbBufPerSec;
	m_TotalBufferSize = m_BufferSize * m_numBuffers;
	m_BufferLengthMS = 1000 / m_nbBufPerSec;
	/*this could b refined, but it works well enough to get decent lip-sync: the audio is advanced of the total 
	length of the audio buffer so that it reaches the sound card in time (this means the begin of the buffer 
	is slightly truncated)*/
	m_audio_delay_ms = m_BufferLengthMS * m_numBuffers;

	//reset DS
	Reset();

	// Set the output format of the sound card
	DSBUFFERDESC primaryDesc;
	memset(&primaryDesc, 0, sizeof(DSBUFFERDESC));
	primaryDesc.dwSize = sizeof(DSBUFFERDESC);
	primaryDesc.dwFlags = DSBCAPS_CTRLPOSITIONNOTIFY | DSBCAPS_GETCURRENTPOSITION2 | DSBCAPS_GLOBALFOCUS;
	primaryDesc.dwBufferBytes = m_TotalBufferSize;
	primaryDesc.lpwfxFormat = &m_Format;


	if (FAILED (m_pDS->CreateSoundBuffer(&primaryDesc, &m_pSoundBuffer, NULL))) return FALSE;

	//reset buffer since we changed its properties
    void *ptr;
    DWORD size;
    if ( FAILED(m_pSoundBuffer->Lock( 0, m_TotalBufferSize, &ptr, &size, NULL, NULL, 0 ) ) )
        return FALSE;

	memset(ptr, 0, (size_t) size);
	m_pSoundBuffer->Unlock(ptr, size, NULL, NULL);
	
	m_bNeedsStart = TRUE;
	return TRUE;
}

void AudioRenderer::GetCapabilities(Capabilities* pCaps)
{
	pCaps->AddFormatCapability(MediaFormat (MediaFormat :: PCM), 2);
}


BOOL AudioRenderer::AddSourceNode(AudioProd *pAudioNode)
{
	if (!pAudioNode) return FALSE;

	if (!m_Mixer.IsSourceSupported(pAudioNode)) return FALSE;

	m_Lock.P();
	if (m_Mixer.AddSource(pAudioNode)) {
		UpdateOutputFormat();
	}
	m_Lock.V();
	return TRUE;
}

void AudioRenderer::RemoveSourceNode(AudioProd *pAudioNode)
{
	m_Lock.P();
	m_Mixer.RemoveSource(pAudioNode);
	m_Lock.V();
}


void AudioRenderer::Pause()
{
	m_bPaused = TRUE;
}

void AudioRenderer::Resume()
{
	m_bPaused = FALSE;
}


BOOL AudioRenderer::RestoreBuffer()
{
	DWORD status;
	HRESULT res;
    m_pSoundBuffer->GetStatus(&status);

    if (status & DSBSTATUS_BUFFERLOST ) {
        while (1) {
            res = m_pSoundBuffer->Restore();
            if (res == DSERR_BUFFERLOST) {
				TRACE("locked in restore\n\n");
				continue;
			}
			break;
        }
        return TRUE;
    }
	return FALSE;
}


void AudioRenderer::GoToSleep()
{
    DWORD writePos=0, playPos=0, maxSize=0;

    if (!m_pSoundBuffer || (m_pSoundBuffer->GetCurrentPosition(&playPos, &writePos) != DS_OK) ) return;

	writePos = m_WritePos;

	/*get available size for write*/
	if (writePos >= playPos) {
		maxSize = m_TotalBufferSize - writePos + playPos;
	} else {
		maxSize = playPos - writePos;
	}
	/*get time needed to read what's written (average per buffer)*/
	maxSize /= m_numBuffers;
	maxSize = (m_BufferSize - maxSize) * m_BufferLengthMS / m_BufferSize;
	/*security, come back earlier...*/
	maxSize *= 0.8;
	Sleep(maxSize);
}

void AudioRenderer::AudioLoop()
{
    void *buf;
    DWORD res, buf_size;
	float intens;
	
	//we're running
	m_mtxThread.P ();

	while (m_bActive) {

		m_Lock.P();

		/*sound buffer not created*/
		if (!m_pSoundBuffer) {
			Sleep(33);
			goto end_loop;
		}

		if (m_bNeedsStart) {
			m_pSoundBuffer->SetCurrentPosition(0L);
			m_pSoundBuffer->Play(0, 0, DSBPLAY_LOOPING);
			/* if not done DS or audio board screws up read/write pointers... */
			Sleep(50);
			m_bNeedsStart = FALSE;
			m_WritePos = 0;
		}

		//make sure we're not overriding something not played yet
		m_pSoundBuffer->GetCurrentPosition(&res, NULL);
		if ((res>m_WritePos) && (m_WritePos + m_BufferSize>res) ) goto end_loop;

		intens = m_Mixer.GetOutputIntensity();
		if (intensity != intens) {
			intensity = intens;
			//this should be made log ...
			m_pSoundBuffer->SetVolume(DSBVOLUME_MIN  * 2 * (1-intens) / 3);
		}


		//check for lost buffers
		if (RestoreBuffer()) {
			TRACE("restored lost buffer\n");
			goto end_loop;
		}
		
		//lock output buffer from current write pos
		if ( FAILED(m_pSoundBuffer->Lock(m_WritePos, m_BufferSize, &buf, &buf_size, NULL, NULL, 0L))) {
			goto end_loop;
		}

		//fill buffer
		if (!m_bPaused) {
			m_Mixer.GetMix(buf, buf_size, m_bCheckAudio, m_audio_delay_ms);
		} else {
			memset(buf, 0, buf_size);
		}
		//release
		m_pSoundBuffer->Unlock(buf, buf_size, NULL, 0);
		m_WritePos += buf_size;
		m_WritePos %= m_TotalBufferSize;

end_loop:
		m_Lock.V();

		GoToSleep();
	}
	
	//we're done
	m_mtxThread.V ();
}


