/*
	This software module was originally developed by ENST in the course of
development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.
	This software module is an implementation of a part of one or more MPEG-4
Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1) 
standard. ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license 
to this software module or modifications thereof for use in hardware or software 
products claiming conformance to the MPEG-4 Systems (ISO/IEC 14496-1). 
Those intending to use this software module in hardware or software products are 
advised that its use may infringe existing patents.
	The original developer of this software module and his/her company, the 
subsequent editors and their companies, and ISO/IEC have no liability for use of 
this software module or modifications thereof in an implementation. Copyright is 
not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
	ENST retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using the code 
for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright    
notice must be included in all copies or derivative works.              
						Copyright  2002.
*/

///////////////////////////////////////////////////////////////////////////
// File:  AudioRenderer.h
//
// Author: Jean Le Feuvre
// Description:
//		audio renderer
//
// Created: january 2003
//
//	
///////////////////////////////////////////////////////////////////////////


#ifndef AUDIORENDERER_H
#define AUDIORENDERER_H


#include <mmsystem.h>
#include <dsound.h>

#include "AudioMixer.h"

class Presenter;
class Capabilities;
class AudioProd;


//number of audio buffers
#define	MAX_NUM_BUFFER		20



class AudioRenderer
{
public:
	AudioRenderer(Presenter* pPresenter, void* pInfo);
	virtual ~AudioRenderer();

	void GetCapabilities (Capabilities* pPresentationCaps);
	BOOL Init();

	BOOL AddSourceNode(AudioProd *pAudioNode);
	void RemoveSourceNode(AudioProd *pAudioNode);

	void Pause();
	void Resume();

private:
	void AudioLoop();

	//returns number of bytes really used in the mix (due to SR conv)
	void Reset();
	BOOL UpdateOutputFormat();

	void GoToSleep();

	unsigned int m_audio_delay_ms;
	unsigned int m_nbBufPerSec;
	BOOL m_bCheckAudio;
	// Remember a pointer to the presenter that this renderer is part of
	Presenter* m_pPresenter;
	HWND m_hWnd;

	//size of one audio buffer (filled in one shot)	
	DWORD m_BufferSize;
	//total size of audio buffers
	DWORD m_TotalBufferSize;
	DWORD m_WritePos;
	BOOL m_bNeedsStart;
	DWORD m_BufferLengthMS;
	unsigned int m_numBuffers;

	//main loop mutex
	ZMutex m_mtxThread;
	//main loop active flag
	BOOL m_bActive;

	BOOL m_bPaused;
	AudioMixer m_Mixer;
	float intensity;
	ZMutex m_Lock;

	WAVEFORMATEX m_Format;

	/*DirectSound vars*/
	LPDIRECTSOUND m_pDS;
	LPDIRECTSOUNDBUFFER m_pSoundBuffer;

	BOOL RestoreBuffer();
};


#endif // AUDIORENDERER_H
