// ChildView.cpp : implementation of the CChildView class
//

#include "defs.h"
#include "Osmose.h"
#include "ChildView.h"
#include "Presenter.h"
#include "MainFrm.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CChildView

CChildView::CChildView() : shiftKey_changed(0), altKey_changed(0), controlKey_changed(0), leftButton_pressed(FALSE)
{
	curZoom = 1.0;
	bGrabbed = 0;
}

CChildView::~CChildView()
{
}


BEGIN_MESSAGE_MAP(CChildView,CWnd )
	//{{AFX_MSG_MAP(CChildView)
	ON_WM_SETFOCUS()
	ON_WM_MOUSEMOVE()
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONDBLCLK()
	ON_WM_LBUTTONUP()
	ON_WM_CHAR()
	ON_WM_SYSKEYDOWN()
	ON_WM_SYSKEYUP()
	ON_WM_KEYDOWN()
	ON_WM_KEYUP()
	ON_WM_PAINT()
	ON_MESSAGE(WM_SETCURSOR,OnSetCursor)
	ON_WM_CLOSE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CChildView message handlers

BOOL CChildView::PreCreateWindow(CREATESTRUCT& cs)
{

//	if (!CWnd::PreCreateWindow(cs)) return FALSE;

	cs.dwExStyle = 0;
	cs.style &= ~WS_BORDER;

	cs.lpszClass = AfxRegisterWndClass(CS_HREDRAW|CS_VREDRAW|CS_DBLCLKS,
		::LoadCursor(NULL, IDC_ARROW), HBRUSH(COLOR_WINDOW+1), NULL);


	return TRUE;
}

void CChildView::OnDraw(CDC* /*pDC*/)
{
	if(GetApp()->GetExecutive())
		GetApp()->GetExecutive()->Paint(NULL);

}

void CChildView::OnSetFocus(CWnd* /*pOldWnd/*/)
{
}


void CChildView::OnPaint()
{
	if (! GetApp()->GetExecutive()) return;

	Presenter *pPres = (Presenter*) GetApp()->GetExecutive()->GetPresenter();
	if (!pPres) return;

	pPres->AskScreenAccess();
	CPaintDC dc(this); // device context for painting

	OnDraw(&dc);

	// Do not call CView::OnPaint() for painting messages
	pPres->ReleaseScreenAccess();
}

void CChildView::OnSetCursor(WPARAM /*wParam*/, LPARAM /*lParam*/)
{
	GetApp()->OverSensor(GetApp()->m_CursorType);
}



void CChildView::NonFatalError(int nError, va_list args)
{

	static LPCSTR szError [] =
	{
		"INVALID STREAM TYPE",						 // 0
		"STREAM TYPE COLLISION",
		"NO BIFS STREAM",
		"SCENE PARSING ERROR",
		"OD PARSING ERROR",
		"OD UPDATE NOT SUPPORTED",
		"ODID COLLISION",
		"UNKNOWN ODID",
		"UPDATE ESD NOT SUPPORTED",
		"REMOVE ESD NOT SUPPORTED",
		"UNKNOWN DECODER TYPE",				// 10
		"INCOMPATIBLE DECODER TYPE",
		"NO STREAM",
		"NO STREAM TYPE",
		"REMOTE OD REQUIRES URL",
		"UNKNOWN URL PROTOCOL",
		"BUFFER TOO SMALL",
		"DECODER SETUP FAILURE",
		"REMOTE OD REQUIRES NEW SESSION",
		"REMOTE OD FLAG REQUIRED",
		"CHANNEL ADD FAILED",				// 20
		"CHANNEL DELETE FAILED",
		"DATA TRANSMISSION ERROR",
		"INVALID URL: %s",
		"IPMP NOT AVAILABLE",
		"NO ACCESS TO PROTECTED STREAM",
		"MISSING DESCRIPTOR",
		"TOO MANY DESCRIPTORS",
		"INITIAL OBJECT DESCRIPTOR NOT FOUND",
		"NO SLCONFIG INFO",
		"ERROR IN PARSING INITIAL OBJECT DESCRIPTOR",	// 30
		"BIFS UPDATE REQUEST DOES NOT MATCH SCENE",
		"MULTIPLE ACCESS OF A SINGLE STREAM NOT PERMITTED",
		"IPMP STREAM ALREADY IN USE",
		"INVALID BIFS CONFIG",
		"CANNOT ALTER REMOTE OD",
		"TRYING TO UPDATE UNKNOWN OD",
		"TRYING TO REMOVE UNKNOWN ESD",
		"OCI PARSING ERROR",
		"SERVICE DETACH FAILED",
		"SEND COMMAND FAILED",							// 40
		"UNEXPECTED SERVICE DETACH",
		"UNEXPECTED CHANNEL DELETE",
		"DMIF FILTER NOT FOUND",
		"URL MUST REFER TO A SINGLE OD: %s",
		"INVALID UI CONFIG",
		"INVALID UI BUFFER",
		"VIDEO RENDER ERROR %s",
		"AUDIO RENDER ERROR %s",
		"SCRIPT ERROR %s",
		"%s",											// 50
		"ONLY OD ID URL ALLOWED: %s",
		"IPMP CANNOT PROCESS OD %d",
		"IPMP CANNOT DELETE OD %d",
		"IPMP CANNOT PROCESS IPMP TOOL %d",
		"IPMP CANNOT REMOVE IPMP TOOL %d"
	};

	ASSERT( nError < sizeof( szError ) ) ;
	OsmoseApp *pApp = GetApp();
	ASSERT( pApp ) ;
	vsprintf(pApp->m_szConsole, szError [nError - 1], args);
	if (nError==SCRIPT_MESSAGE) {
		pApp->m_MessageType = 1;
	} else {
		pApp->m_MessageType = 0;
	}
	CMainFrame* pMainFrame = GetMainFrame() ;
	// in case "Destroy" was already called.
	if ( pMainFrame )
	{
		pMainFrame->PostMessage(WM_CONSOLEMSG);
		pMainFrame->SetForegroundWindow();
	}
}

void CChildView::FatalError(WPARAM wParam, char *msg)
{
	static LPCSTR szError [] =
	{
		"Audio Render failed to init",
		"Video Render failed to init",
	};
	/*non fatal error callback doesn't work ... */
	if (msg && wParam==OTHER_ERROR) {
		MessageBox(msg, "Error", MB_OK);
	} else {
		MessageBox(msg ? msg : "Please restart the player", szError[wParam],MB_OK);
	}
	// in case "Destroy" was already called.
	CMainFrame* pMainFrame = GetMainFrame() ;
	// in case "Destroy" was already called.
	if ( pMainFrame )
	{
		pMainFrame->SetForegroundWindow();
	}
}

void CChildView::SendMouseMove(UINT nFlags, CPoint point, BOOL bDown)
{
	float X, Y;
	VisualRenderer *pRend = ((Presenter*) GetApp()->GetExecutive()->GetPresenter())->GetVisualRenderer();
	pRend->DeviceToSurfaceCoords(point.x, point.y, &X, &Y);

	GetApp()->GetMouseDevice()->DataReceive_Ind(X, Y, 0.0f, bDown, nFlags);
}

void CChildView::OnMouseMove(UINT nFlags, CPoint point)
{
	// Determine if the CTRL and/or the SHIFT keys are pressed
	UINT ctrlKeys = 0;
	if (nFlags & MK_CONTROL)
		ctrlKeys |= CK_CONTROL;
	if (nFlags & MK_SHIFT)
		ctrlKeys |= CK_SHIFT;

	// Pass this message onto the executive
	if(GetApp()->GetExecutive())
		GetApp()->GetExecutive()->OnUserEvent(EVENT_MOUSE_MOVE,
										  point.x, point.y,
										  ctrlKeys);

	SendMouseMove(0, point, FALSE);

	if (bGrabbed && shiftKey_changed) {
		((Presenter *)GetApp()->GetExecutive()->GetPresenter())->AddTranslation(point.x - grabbed.x, point.y - grabbed.y);
		grabbed = point;
	}
	if (!controlKey_changed && shiftKey_changed && bGrabbed)
		GetApp()->OverSensor(SENSOR_MOVECLOSE);

}

void CChildView::OnLButtonDown(UINT nFlags, CPoint point)
{
	// Determine if the CTRL and/or the SHIFT keys are pressed
	UINT ctrlKeys = 0;
	if (nFlags & MK_CONTROL)
		ctrlKeys |= CK_CONTROL;
	if (nFlags & MK_SHIFT)
		ctrlKeys |= CK_SHIFT;

	// Pass this message onto the executive
	GetApp()->GetExecutive()->OnUserEvent(EVENT_MOUSE_DOWN,
										  point.x, point.y,
										  ctrlKeys);

	SendMouseMove(nFlags, point, TRUE);

	//set zoom
	if (controlKey_changed) {
		if (shiftKey_changed) {
			curZoom -= (curZoom>= 1.0) ? 0.25 : 0.1;
		} else {
			curZoom += (curZoom>= 1.0) ? 0.25 : 0.1;
		}
		((Presenter *)GetApp()->GetExecutive()->GetPresenter())->SetZoom(curZoom);
	}
	//set grabbed
	else if (shiftKey_changed) {
		grabbed = point;
		bGrabbed = 1;
	}
}

void CChildView::OnLButtonDblClk(UINT nFlags, CPoint point)
{
	// Determine if the CTRL and/or the SHIFT keys are pressed
	UINT ctrlKeys = 0;
	if (nFlags & MK_CONTROL)
		ctrlKeys |= CK_CONTROL;
	if (nFlags & MK_SHIFT)
		ctrlKeys |= CK_SHIFT;

	// DBLCLK is actually only a mouse down
	GetApp()->GetExecutive()->OnUserEvent(EVENT_MOUSE_DOWN,
										  point.x, point.y,
										  ctrlKeys);

	SendMouseMove(nFlags, point, TRUE);

	//set zoom
	if (controlKey_changed) {
		if (shiftKey_changed) {
			curZoom -= (curZoom>= 1.0) ? 0.5 : 0.1;
			if (curZoom <= 0) curZoom = 0;
		} else {
			curZoom += (curZoom>= 1.0) ? 0.5 : 0.1;
		}
		((Presenter *)GetApp()->GetExecutive()->GetPresenter())->SetZoom(curZoom);
	}
}

void CChildView::OnLButtonUp(UINT nFlags, CPoint point)
{
	// Determine if the CTRL and/or the SHIFT keys are pressed
	UINT ctrlKeys = 0;
	if (nFlags & MK_CONTROL)
		ctrlKeys |= CK_CONTROL;
	if (nFlags & MK_SHIFT)
		ctrlKeys |= CK_SHIFT;

	// Pass this message onto the executive
	GetApp()->GetExecutive()->OnUserEvent(EVENT_MOUSE_UP,
										  point.x, point.y,
										  ctrlKeys);

	leftButton_pressed = FALSE;
	SendMouseMove(nFlags, point, FALSE);

	if (shiftKey_changed) {
		bGrabbed = 0;
	}

}


void CChildView::OnSysKeyUp(UINT nChar, UINT , UINT )
{
	int virtKey = GetCurrentMessage()->wParam;

	// Web3D Behavior
	UINT nActionKeyRelease = GetActionKey(virtKey);
	switch(virtKey) {
	case VK_SHIFT:
		shiftKey_changed = 0;
		break;
	case VK_CONTROL:
		controlKey_changed = 0;
		break;
	case VK_MENU:
		altKey_changed = 0;
		break;

	case VK_RETURN:
		GetMainFrame()->SetFullScreen();
		break;
	}
	GetApp()->GetKeySensorDevice()->DataReceive_Ind(0,nChar,0,nActionKeyRelease,shiftKey_changed,controlKey_changed,altKey_changed);

	::DefWindowProc(m_hWnd,GetCurrentMessage()->message,GetCurrentMessage()->wParam,GetCurrentMessage()->lParam);
}

void CChildView::OnSysKeyDown(UINT nChar, UINT , UINT )
{
	int virtKey = GetCurrentMessage()->wParam;

	// Web3D Behavior
	UINT nActionKeyPress = GetActionKey(virtKey);
	switch(virtKey) {
	case VK_SHIFT:
		shiftKey_changed = 1;
		break;
	case VK_CONTROL:
		controlKey_changed = 1;
		break;
	case VK_MENU:
		altKey_changed = 1;
		break;
/*	case VK_ESCAPE:
		GetMainFrame()->SetFullScreen();
		::DefWindowProc(m_hWnd,GetCurrentMessage()->message,GetCurrentMessage()->wParam,GetCurrentMessage()->lParam);
		return;
*/	}
	GetApp()->GetKeySensorDevice()->DataReceive_Ind(nChar,0,nActionKeyPress,0,shiftKey_changed,controlKey_changed,altKey_changed);

	::DefWindowProc(m_hWnd,GetCurrentMessage()->message,GetCurrentMessage()->wParam,GetCurrentMessage()->lParam);
}


void CChildView::OnChar(UINT nChar, UINT , UINT )
{
	ZString s = CString(nChar);
	GetApp()->GetStringSensorDevice()->DataReceive_Ind(s);

	::DefWindowProc(m_hWnd,GetCurrentMessage()->message,GetCurrentMessage()->wParam,GetCurrentMessage()->lParam);
}

void CChildView::OnKeyDown(UINT nChar, UINT , UINT )
{
	int virtKey = GetCurrentMessage()->wParam;

	// Web3D Behavior
	UINT nActionKeyPress = GetActionKey(virtKey);
	switch(virtKey) {
	case VK_SHIFT:
		shiftKey_changed = 1;
		break;
	case VK_CONTROL:
		controlKey_changed = 1;
		break;
	case VK_MENU:
		altKey_changed = 1;
		break;
	case VK_ESCAPE:
		if (GetMainFrame()->IsFullScreen()) GetMainFrame()->SetFullScreen();
		break;
	}
	if ((virtKey==VK_F4) && altKey_changed) return;

	GetApp()->GetKeySensorDevice()->DataReceive_Ind(nChar,0,nActionKeyPress,0,shiftKey_changed,controlKey_changed,altKey_changed);
	// end
	::DefWindowProc(m_hWnd,GetCurrentMessage()->message,GetCurrentMessage()->wParam,GetCurrentMessage()->lParam);

	if (controlKey_changed) {
		if (shiftKey_changed) {
			GetApp()->OverSensor(SENSOR_ZOOMOUT);
		} else {
			GetApp()->OverSensor(SENSOR_ZOOMIN);
		}
	} else if (shiftKey_changed) {
		GetApp()->OverSensor(bGrabbed ? SENSOR_MOVECLOSE : SENSOR_MOVEOPEN);
	}
}

void CChildView::OnKeyUp(UINT nChar, UINT , UINT )
{
	int virtKey = GetCurrentMessage()->wParam;

	// Web3D Behavior
	UINT nActionKeyRelease = GetActionKey(virtKey);
	switch(virtKey) {
	case VK_SHIFT:
		shiftKey_changed = 0;
		break;
	case VK_CONTROL:
		controlKey_changed = 0;
		break;
	case VK_MENU:
		altKey_changed = 0;
		break;
	}
	GetApp()->GetKeySensorDevice()->DataReceive_Ind(0,nChar,0,nActionKeyRelease,shiftKey_changed,controlKey_changed,altKey_changed);
	// end

	::DefWindowProc(m_hWnd,GetCurrentMessage()->message,GetCurrentMessage()->wParam,GetCurrentMessage()->lParam);

	if (controlKey_changed) {
		if (shiftKey_changed) {
			GetApp()->OverSensor(SENSOR_ZOOMOUT);
		} else {
			GetApp()->OverSensor(SENSOR_ZOOMIN);
		}
	} else if (shiftKey_changed) {
		GetApp()->OverSensor(bGrabbed ? SENSOR_MOVECLOSE : SENSOR_MOVEOPEN);
	} else {
		GetApp()->OverSensor(SENSOR_RESETPRIVATE);
	}
}

UINT CChildView::GetActionKey(UINT nVirtKey)
{
	UINT nActionKey = 0;
	switch (nVirtKey) {
	case VK_HOME:
		nActionKey = 13;
		break;
	case VK_END:
		nActionKey = 14;
		break;
	case VK_PRIOR:
		nActionKey = 15;
		break;
	case VK_NEXT:
		nActionKey = 16;
		break;
	case VK_UP:
		nActionKey = 17;
		break;
	case VK_DOWN:
		nActionKey = 18;
		break;
	case VK_LEFT:
		nActionKey = 19;
		break;
	case VK_RIGHT:
		nActionKey = 20;
		break;
	case VK_F1:
		nActionKey = 1;
		break;
	case VK_F2:
		nActionKey = 2;
		break;
	case VK_F3:
		nActionKey = 3;
		break;
	case VK_F4:
		nActionKey = 4;
		break;
	case VK_F5:
		nActionKey = 5;
		break;
	case VK_F6:
		nActionKey = 6;
		break;
	case VK_F7:
		nActionKey = 7;
		break;
	case VK_F8:
		nActionKey = 8;
		break;
	case VK_F9:
		nActionKey = 9;
		break;
	case VK_F10:
		nActionKey = 10;
		break;
	case VK_F11:
		nActionKey = 11;
		break;
	case VK_F12:
		nActionKey = 12;
		break;
	}
	return nActionKey;
}

ZString CChildView::GetCharString(UINT nChar)
{
	ZString s("");
	switch (nChar) {
	case 0x01:
		s = "CTRL_A";
		break;
	case 0x02:
		s = "CTRL_B";
		break;
	case 0x03:
		s = "CTRL_C";
		break;
	case 0x04:
		s = "CTRL_D";
		break;
	case 0x05:
		s = "CTRL_E";
		break;
	case 0x06:
		s = "CTRL_F";
		break;
	case 0x07:
		s = "CTRL_G";
		break;
	case 0x08:
		s = "BS";
		break;
	case 0x09:
		s = "TAB";
		break;
	case 0x0A:
		s = "LF";
		break;
	case 0x0B:
		s = "CTRL_K";
		break;
	case 0x0C:
		s = "CTRL_L";
		break;
	case 0x0D:
		s = "CR";
		break;
	case 0x0E:
		s = "CTRL_N";
		break;
	case 0x0F:
		s = "CTRL_O";
		break;
	case 0x10:
		s = "CTRL_P";
		break;
	case 0x11:
		s = "CTRL_Q";
		break;
	case 0x12:
		s = "CTRL_R";
		break;
	case 0x13:
		s = "CTRL_S";
		break;
	case 0x14:
		s = "CTRL_T";
		break;
	case 0x15:
		s = "CTRL_U";
		break;
	case 0x16:
		s = "CTRL_V";
		break;
	case 0x17:
		s = "CTRL_W";
		break;
	case 0x18:
		s = "CTRL_X";
		break;
	case 0x19:
		s = "CTRL_Y";
		break;
	case 0x1A:
		s = "CTRL_Z";
		break;
	case 0x1B:
		s = "ESCAPE";
		break;
	default:
		s = CString(nChar);
	}
	return s;
}

ZString CChildView::GetString(UINT virtKey)
{
	ZString s("");
	switch (virtKey) {
	case VK_CLEAR:			// 0C clear key
		s = "CLEAR";
		break;
	case VK_SHIFT:			// 10 shift key
		s = "SHIFT";
		break;
	case VK_CONTROL:		// 11 ctrl key
		s = "CTRL";
		break;
	case VK_MENU:			// 12 alt key
		s = "ALT";
		break;
	case VK_CAPITAL:		// 14 caps lock key
		s = "CAPS";
		break;
	case VK_SELECT:			// 29 select key
		s = "SELECT";
		break;
	case VK_EXECUTE:		// 2B execute key
		s = "EXECUTE";
		break;
	case VK_SNAPSHOT:		// 2C print screen key
		s = "SNAPSHOT";
		break;
	case VK_HELP:			// 2F help key
		s = "HELP";
		break;
	case VK_LWIN:			// 5B Left Windows key (Microsoft Natural Keyboard)
		s = "LWIN";
		break;
	case VK_RWIN:			// 5C Right Windows key (Microsoft Natural Keyboard)
		s = "RWIN";
		break;
	case VK_APPS:			// 5D Applications key (Microsoft Natural Keyboard)
		s = "APPS";
		break;
	case VK_NUMPAD0:		// 60 Numeric keypad 0 key
		s = "NUMPAD0";
		break;
	case VK_NUMPAD1:		// 61 Numeric keypad 1 key
		s = "NUMPAD1";
		break;
	case VK_NUMPAD2:		// 62 Numeric keypad 2 key
		s = "NUMPAD2";
		break;
	case VK_NUMPAD3:		// 63 Numeric keypad 3 key
		s = "NUMPAD3";
		break;
	case VK_NUMPAD4:		// 64 Numeric keypad 4 key
		s = "NUMPAD4";
		break;
	case VK_NUMPAD5:		// 65 Numeric keypad 5 key
		s = "NUMPAD5";
		break;
	case VK_NUMPAD6:		// 66 Numeric keypad 6 key
		s = "NUMPAD6";
		break;
	case VK_NUMPAD7:		// 67 Numeric keypad 7 key
		s = "NUMPAD7";
		break;
	case VK_NUMPAD8:		// 68 Numeric keypad 8 key
		s = "NUMPAD8";
		break;
	case VK_NUMPAD9:		// 69 Numeric keypad 9 key
		s = "NUMPAD9";
		break;
	case VK_MULTIPLY:		// 6A Multiply key
		s = "MULTIPLY";
		break;
	case VK_ADD:			// 6B Add key
		s = "ADD";
		break;
	case VK_SEPARATOR:		// 6C Separator key
		s = "SEPARATOR";
		break;
	case VK_SUBTRACT:		// 6D Subtract key
		s = "SUBSTRACT";
		break;
	case VK_DECIMAL:		// 6E Decimal key
		s = "DECIMAL";
		break;
	case VK_DIVIDE:			// 6F Divide key
		s = "DIVIDE";
		break;
	case VK_NUMLOCK:		// 90 num lock key
		s = "NUMLOCK";
		break;
	case VK_SCROLL:			// 91 scroll lock key
		s = "SCROLL";
		break;
	case VK_PAUSE:			// 13 pause key
		s = "PAUSE";
		break;
	case VK_PRIOR:			// 21 page up key
		s = "PRIOR";
		break;
	case VK_NEXT:			// 22 page down key
		s = "NEXT";
		break;
	case VK_END:			// 23 end key
		s = "END";
		break;
	case VK_HOME:			// 24 home key
		s = "HOME";
		break;
	case VK_LEFT:			// 25 left arrow key
		s = "LEFT";
		break;
	case VK_UP:				// 26 up arrow key
		s = "UP";
		break;
	case VK_RIGHT:			// 27 right arrow key
		s = "RIGHT";
		break;
	case VK_DOWN:			// 28 down arrow key
		s = "DOWN";
		break;
	case VK_INSERT:			// 2D ins key
		s = "INSERT";
		break;
	case VK_DELETE:			// 2E del key
		s = "DELETE";
		break;
	case VK_F1:				// 70 f1 key
		s = "F1";
		break;
	case VK_F2:				// 71 f2 key
		s = "F2";
		break;
	case VK_F3:				// 72 f3 key
		s = "F3";
		break;
	case VK_F4:				// 73 f4 key
		s = "F4";
		break;
	case VK_F5:				// 74 f5 key
		s = "F5";
		break;
	case VK_F6:				// 75 f6 key
		s = "F6";
		break;
	case VK_F7:				// 76 f7 key
		s = "F7";
		break;
	case VK_F8:				// 77 f8 key
		s = "F8";
		break;
	case VK_F9:				// 78 f9 key
		s = "F9";
		break;
	case VK_F10:			// 79 f10 key
		s = "F10";
		break;
	case VK_F11:			// 7A f11 key
		s = "F11";
		break;
	case VK_F12:			// 7B f12 key
		s = "F12";
		break;
	default:
		s = GetCharString(virtKey);
	}
	return s;
}

void CChildView::OnClose()
{
	GetApp()->GetMainWnd()->PostMessage(WM_CLOSE);
}
