/*
	This software module was originally developed by ENST in the course of
development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.
	This software module is an implementation of a part of one or more MPEG-4
Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1) 
standard. ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license 
to this software module or modifications thereof for use in hardware or software 
products claiming conformance to the MPEG-4 Systems (ISO/IEC 14496-1). 
Those intending to use this software module in hardware or software products are 
advised that its use may infringe existing patents.
	The original developer of this software module and his/her company, the 
subsequent editors and their companies, and ISO/IEC have no liability for use of 
this software module or modifications thereof in an implementation. Copyright is 
not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
	ENST retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using the code 
for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright    
notice must be included in all copies or derivative works.              
						Copyright  2002.
*/

///////////////////////////////////////////////////////////////////////////
// File:  Clipper2DProxy.cpp
//
// Authors: Jean Le Feuvre ENST 2002
//
// Description:
//   proxy object for Clipper node.
//
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include "Nodes-INIT.h"
#include "ClipperProxy.h"
#include "GeometryNode.h"
#include "Presenter.h"
#include "Transform2DProxy.h"
#include "TransformMatrix2DProxy.h"

Clipper2DProxy* Presenter::InitClipper2D(Clipper2D* pObj)
{
		return new Clipper2DProxy(pObj);
}

void Clipper2DProxy::Render(Effects *effects)
{
	BOOL bHasChanged = effects->m_bInvalidateAll;
	MediaObject* pMediaObject=GetClipper()->geometry;
	if (! pMediaObject) return;

	effects->m_pAppearanceProxy = NULL;
	effects->m_pGeometryProxy = (GeometryNode *) (pMediaObject)->GetRenderingObject();	
	if (!effects->m_pGeometryProxy) return;

	Matrix2D store;
	store.Set(effects->transform);

	DWORD now = GetObject()->GetUpdateTime() + 1;
	if (now != m_LastUpdateTime) {
		m_LastUpdateTime = now;
		bHasChanged = TRUE;
	}

	MediaObject *pTrans = GetClipper()->transform;
	if (pTrans) {
		Matrix2D mat;
		Traversable *pTrav = (Traversable *) pTrans->GetRenderingObject();
		switch (pTrans->GetNodeCode()) {
		case CODE_Transform2D:
			((Transform2DProxy *)pTrav)->GetMatrix(&mat);
			break;
		case CODE_TransformMatrix2D:
			((TransformMatrix2DProxy *)pTrav)->GetMatrix(&mat);
			break;
		}
		mat.AddTransform(&effects->transform);
		effects->transform.Set(mat);
		now = pTrans->GetUpdateTime() + 1;
		if (now != m_LastUpdateTime) {
			m_LastUpdateTime = now;
			bHasChanged = TRUE;
		}
	}
	if (!effects->m_pClipper) effects->m_pClipper = new VisualClipper();
	//this applies for all following children
	effects->m_bInvalidateAll = bHasChanged;

	effects->m_ClipperMode = ((Bool) GetClipper()->inside) ? 2 : 1;
	if ((Bool) GetClipper()->XOR) effects->m_ClipperMode += 10;

	effects->m_pGeometryProxy->Render(effects);
	effects->m_ClipperMode = 0;
	effects->m_pGeometryProxy = NULL;
	effects->transform.Set(store);

	// Apply these new effects to the children nodes
	RenderChildren(effects);
}

