/*
	This software module was originally developed by ENST in the course of
development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.
	This software module is an implementation of a part of one or more MPEG-4
Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1) 
standard. ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license 
to this software module or modifications thereof for use in hardware or software 
products claiming conformance to the MPEG-4 Systems (ISO/IEC 14496-1). 
Those intending to use this software module in hardware or software products are 
advised that its use may infringe existing patents.
	The original developer of this software module and his/her company, the 
subsequent editors and their companies, and ISO/IEC have no liability for use of 
this software module or modifications thereof in an implementation. Copyright is 
not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
	ENST retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using the code 
for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright    
notice must be included in all copies or derivative works.              
						Copyright  2002.
*/

///////////////////////////////////////////////////////////////////////////
// File: ColorGradient.h
//
// Author: Jean Le Feuvre
//
// Description:
//		Object used to store color gradient (pen and brush)
//
// Created: August 2002
//
///////////////////////////////////////////////////////////////////////////

#include "BRect.h"

#ifndef COLORGRADIENT_H_
#define COLORGRADIENT_H_

#include "VisualPath.h"
#include "VisualTexture.h"

#include "ztypes.h"


enum GRADIENT_TILE
{
	//PAD is default and is NOT supported, we don't repeat the end color
	//of the gradient
	GRADIENT_TILE_PAD = 0,
	GRADIENT_TILE_BASE,
	GRADIENT_TILE_FLIP,
};

class ColorGradient
{
public:
	ColorGradient();
	~ColorGradient();

	void Reset();
	BOOL SetVisualPath(VisualPath *path);
	BOOL SetGradientPoints(PointF* pts, unsigned int numPoints);

	//Note: alpha channel applies to all colors
	BOOL SetGradientColors(Color3f *col, unsigned int numColors, float transparency);
	BOOL SetGradientColors(Color4 *col, unsigned int numColors);
	BOOL SetCenterColor(float x, float y, Color3f col, float transparency);

	//sets the interpolation colors (optional)
	//blend factors range from 0.0 (center) to 1.0 (bounds)
	//you shall specify ALL blend factors, including 0.0 and 1.0, from center to bounds
	BOOL SetBlendColors(Color4 *col, float *blendFactors, unsigned int numColors);
	
	//for linear gradient only
	//base gradient
	BOOL SetLinearGradient(float startx, float starty, float endx, float endy, Color3f start_col, Color3f end_col, float transparency);
	//full linear gradient (at least 2 colors needed
	//blend factors range from 0.0 (center) to 1.0 (bounds)
	//you shall specify ALL blend factors, including 0.0 and 1.0, from center to bounds
	BOOL SetLinearGradient(float startx, float starty, float endx, float endy, Color4 *col, float *blendFactors, unsigned int numColors);

	void SetTransform(Matrix2D *pMat);

	Matrix2D m_Matrix;

	BOOL SetRepeatMode(enum GRADIENT_TILE);

	Gdiplus::PathGradientBrush *m_pBrush;
	Color4 m_BackColor;

	Gdiplus::LinearGradientBrush *m_pLinearBrush;

	Matrix2D m_OriginalLinearMatrix;
	//only used for path gradient
	Gdiplus::GraphicsPath *m_pPath;
};




#endif		//COLORGRADIENT_H_
