/*
	This software module was originally developed by ENST in the course of
development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.
	This software module is an implementation of a part of one or more MPEG-4
Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1) 
standard. ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license 
to this software module or modifications thereof for use in hardware or software 
products claiming conformance to the MPEG-4 Systems (ISO/IEC 14496-1). 
Those intending to use this software module in hardware or software products are 
advised that its use may infringe existing patents.
	The original developer of this software module and his/her company, the 
subsequent editors and their companies, and ISO/IEC have no liability for use of 
this software module or modifications thereof in an implementation. Copyright is 
not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
	ENST retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using the code 
for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright    
notice must be included in all copies or derivative works.              
						Copyright  2002.
*/

///////////////////////////////////////////////////////////////////////////
// File:  ColorMatrix.cpp
//
// Author: Jean Le Feuvre
// Description:
//   2-D transform matrix wrapper for GDIplus.
//
// Created: November, 2002.
//
//	
///////////////////////////////////////////////////////////////////////////

#include "defs.h"
#include "ColorMatrix.h"


ColorMatrix::ColorMatrix()
{
	Reset();
}

void ColorMatrix::Reset()
{
	memset(&m_Matrix, 0, sizeof(Gdiplus::ColorMatrix));
	m_Matrix.m[0][0] = 1.0;
	m_Matrix.m[1][1] = 1.0;
	m_Matrix.m[2][2] = 1.0;
	m_Matrix.m[3][3] = 1.0;
	m_Matrix.m[4][4] = 1.0;
}

BOOL ColorMatrix::IsIdentity()
{
	return m_bIsIdentity;
}


void ColorMatrix::SetIdentityFlag()
{
	ColorMatrix empty;
	m_bIsIdentity = memcmp(m_Matrix.m, empty.m_Matrix.m, sizeof(m_Matrix.m)) ? FALSE : TRUE;
}

void ColorMatrix::Set(float m1, float m2, float m3, float m4, float m5, float m6, float m7, float m8, float m9, float m10, float m11, float m12, float m13, float m14, float m15, float m16, float m17, float m18, float m19, float m20)
{
	Reset();

	//matrix is INVERTED in gdiplus
	m_Matrix.m[0][0] = m1;
	m_Matrix.m[1][0] = m2;
	m_Matrix.m[2][0] = m3;
	m_Matrix.m[3][0] = m4;
	m_Matrix.m[4][0] = m5;

	m_Matrix.m[0][1] = m6;
	m_Matrix.m[1][1] = m7;
	m_Matrix.m[2][1] = m8;
	m_Matrix.m[3][1] = m9;
	m_Matrix.m[4][1] = m10;

	m_Matrix.m[0][2] = m11;
	m_Matrix.m[1][2] = m12;
	m_Matrix.m[2][2] = m13;
	m_Matrix.m[3][2] = m14;
	m_Matrix.m[4][2] = m15;
	
	m_Matrix.m[0][3] = m16;
	m_Matrix.m[1][3] = m17;
	m_Matrix.m[2][3] = m18;
	m_Matrix.m[3][3] = m19;
	m_Matrix.m[4][3] = m20;

	SetIdentityFlag();
}

void ColorMatrix::Set(float *m)
{
	Set(m[0], m[1], m[2], m[3], m[4], m[5], m[6], m[7], m[8], m[9], m[10], m[11], m[12], m[13], m[14], m[15], m[16], m[17], m[18], m[19]);
	SetIdentityFlag();
}

void ColorMatrix::Set(ColorMatrix mat)
{
	memcpy(m_Matrix.m, mat.m_Matrix.m, sizeof(mat.m_Matrix.m));
	SetIdentityFlag();
}

void ColorMatrix::SetGDIPLUS(Gdiplus::ColorMatrix mat)
{
	memcpy(m_Matrix.m, mat.m, sizeof(mat.m));
	SetIdentityFlag();
}

void ColorMatrix::Multiply(ColorMatrix mat)
{
	if (mat.IsIdentity()) return;
	if (IsIdentity()) {
		Set(mat);
		return;
	}

	float m1[20], m2[20];
	float res[20];
	GetElements(m1);
	mat.GetElements(m2);

	res[0] = m1[0]*m2[0] + m1[1]*m2[5] + m1[2]*m2[10] + m1[3]*m2[15];
	res[1] = m1[0]*m2[1] + m1[1]*m2[6] + m1[2]*m2[11] + m1[3]*m2[16];
	res[2] = m1[0]*m2[2] + m1[1]*m2[7] + m1[2]*m2[12] + m1[3]*m2[17];
	res[3] = m1[0]*m2[3] + m1[1]*m2[8] + m1[2]*m2[13] + m1[3]*m2[18];
	res[4] = m1[0]*m2[4] + m1[1]*m2[9] + m1[2]*m2[14] + m1[3]*m2[19] + m1[4];
	
	res[5] = m1[5]*m2[0] + m1[6]*m2[5] + m1[7]*m2[10] + m1[8]*m2[15];
	res[6] = m1[5]*m2[1] + m1[6]*m2[6] + m1[7]*m2[11] + m1[8]*m2[16];
	res[7] = m1[5]*m2[2] + m1[6]*m2[7] + m1[7]*m2[12] + m1[8]*m2[17];
	res[8] = m1[5]*m2[3] + m1[6]*m2[8] + m1[7]*m2[13] + m1[8]*m2[18];
	res[9] = m1[5]*m2[4] + m1[6]*m2[9] + m1[7]*m2[14] + m1[8]*m2[19] + m1[9];
	
	res[10] = m1[10]*m2[0] + m1[11]*m2[5] + m1[12]*m2[10] + m1[13]*m2[15];
	res[11] = m1[10]*m2[1] + m1[11]*m2[6] + m1[12]*m2[11] + m1[13]*m2[16];
	res[12] = m1[10]*m2[2] + m1[11]*m2[7] + m1[12]*m2[12] + m1[13]*m2[17];
	res[13] = m1[10]*m2[3] + m1[11]*m2[8] + m1[12]*m2[13] + m1[13]*m2[18];
	res[14] = m1[10]*m2[4] + m1[11]*m2[9] + m1[12]*m2[14] + m1[13]*m2[19] + m1[14];
	
	res[15] = m1[15]*m2[0] + m1[16]*m2[5] + m1[17]*m2[10] + m1[18]*m2[15];
	res[16] = m1[15]*m2[1] + m1[16]*m2[6] + m1[17]*m2[11] + m1[18]*m2[16];
	res[17] = m1[15]*m2[2] + m1[16]*m2[7] + m1[17]*m2[12] + m1[18]*m2[17];
	res[18] = m1[15]*m2[3] + m1[16]*m2[8] + m1[17]*m2[13] + m1[18]*m2[18];
	res[19] = m1[15]*m2[4] + m1[16]*m2[9] + m1[17]*m2[14] + m1[18]*m2[19] + m1[19];
	
	SetIdentityFlag();
}


void ColorMatrix::GetElements(float *m)
{
	m[0] = m_Matrix.m[0][0];
	m[1] = m_Matrix.m[1][0];
	m[2] = m_Matrix.m[2][0];
	m[3] = m_Matrix.m[3][0];
	m[4] = m_Matrix.m[4][0];

	m[5] = m_Matrix.m[0][1];
	m[6] = m_Matrix.m[1][1];
	m[7] = m_Matrix.m[2][1];
	m[8] = m_Matrix.m[3][1];
	m[9] = m_Matrix.m[4][1];

	m[10] = m_Matrix.m[0][2];
	m[11] = m_Matrix.m[1][2];
	m[12] = m_Matrix.m[2][2];
	m[13] = m_Matrix.m[3][2];
	m[14] = m_Matrix.m[4][2];
	
	m[15] = m_Matrix.m[0][3];
	m[16] = m_Matrix.m[1][3];
	m[17] = m_Matrix.m[2][3];
	m[18] = m_Matrix.m[3][3];
	m[19] = m_Matrix.m[4][3];
}

#define CHECK_COLOR(val)	if (val<0.0) { val=0; } else if (val>1.0) { val=1.0; }

void ColorMatrix::Apply(Color4 &col)
{
	if (m_bIsIdentity) return;

	float r, g, b, a;

	r = (col.r * m_Matrix.m[0][0] + col.g * m_Matrix.m[1][0] + col.b * m_Matrix.m[2][0] + col.a * m_Matrix.m[3][0]) / 255 + m_Matrix.m[4][0];
	g = ( col.r * m_Matrix.m[0][1] + col.g * m_Matrix.m[1][1] + col.b * m_Matrix.m[2][1] + col.a * m_Matrix.m[3][1]) / 255 + m_Matrix.m[4][1];
	b = (col.r * m_Matrix.m[0][2] + col.g * m_Matrix.m[1][2] + col.b * m_Matrix.m[2][2] + col.a * m_Matrix.m[3][2]) / 255 + m_Matrix.m[4][2];
	a = ( col.r * m_Matrix.m[0][3] + col.g * m_Matrix.m[1][3] + col.b * m_Matrix.m[2][3] + col.a * m_Matrix.m[3][3]) / 255 + m_Matrix.m[4][3];

	CHECK_COLOR(r);
	CHECK_COLOR(g);
	CHECK_COLOR(b);
	CHECK_COLOR(a);

	col.r = r * 255;
	col.g = g * 255;
	col.b = b * 255;
	col.a = a * 255;
}

void ColorMatrix::Apply(Color3f &col, float &trans)
{
	if (m_bIsIdentity) return;

	float r, g, b, a;
	trans = 1 - trans;

	r = col.r * m_Matrix.m[0][0] + col.g * m_Matrix.m[1][0] + col.b * m_Matrix.m[2][0] + trans * m_Matrix.m[3][0] + m_Matrix.m[4][0];
	g = col.r * m_Matrix.m[0][1] + col.g * m_Matrix.m[1][1] + col.b * m_Matrix.m[2][1] + trans * m_Matrix.m[3][1] + m_Matrix.m[4][1];
	b = col.r * m_Matrix.m[0][2] + col.g * m_Matrix.m[1][2] + col.b * m_Matrix.m[2][2] + trans * m_Matrix.m[3][2] + m_Matrix.m[4][2];
	a = col.r * m_Matrix.m[0][3] + col.g * m_Matrix.m[1][3] + col.b * m_Matrix.m[2][3] + trans * m_Matrix.m[3][3] + m_Matrix.m[4][3];
	CHECK_COLOR(r);
	CHECK_COLOR(g);
	CHECK_COLOR(b);
	CHECK_COLOR(a);

	col.r = r;;
	col.g = g;
	col.b = b;
	trans = 1 - a;
}



int operator== (const ColorMatrix &mat1, const ColorMatrix &mat2)
{
	return ! memcmp(mat1.m_Matrix.m, mat2.m_Matrix.m, sizeof(mat1.m_Matrix.m));
}
