/*
	This software module was originally developed by ENST in the course of
development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.
	This software module is an implementation of a part of one or more MPEG-4
Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1) 
standard. ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license 
to this software module or modifications thereof for use in hardware or software 
products claiming conformance to the MPEG-4 Systems (ISO/IEC 14496-1). 
Those intending to use this software module in hardware or software products are 
advised that its use may infringe existing patents.
	The original developer of this software module and his/her company, the 
subsequent editors and their companies, and ISO/IEC have no liability for use of 
this software module or modifications thereof in an implementation. Copyright is 
not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
	ENST retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using the code 
for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright    
notice must be included in all copies or derivative works.              
						Copyright  2002.
*/

///////////////////////////////////////////////////////////////////////////
// File:  ColorMatrix.h
//
// Author: Jean Le Feuvre
// Description:
//   color matrix wrapper for GDIplus.
//
// Created: November, 2002.
//
//	
///////////////////////////////////////////////////////////////////////////


#ifndef COLORMATRIX_H
#define COLORMATRIX_H

#include "OsmoseGdiplus.h"
#include "ztypes.h"

class Color4
{
public:
	unsigned char r, g, b, a;

public:
	Color4 (unsigned char rr = 0, unsigned char gg = 0, unsigned char bb = 0, unsigned char aa = 0) :
		r (rr), g (gg), b(bb), a(aa)
	{}
	Color4 operator = (unsigned char c)
	{
		r = g = b = a = c;
		return *this;
	}
	bool operator == (const Color4 &color) const
	{
		return r == color.r && g == color.g && b == color.b && a == color.a;
	}
};

class ColorMatrix
{
	//
	// Public functions
	//
public:
			
	ColorMatrix();

	// Set the matrix to the identity matrix
	void Reset();

	//set the 20 matrix coefficients
	void Set(float m1, float m2, float m3, float m4, float m5, float m6, float m7, float m8, float m9, float m10, float m11, float m12, float m13, float m14, float m15, float m16, float m17, float m18, float m19, float m20);
	void Set(float *coefs);
	void Set(ColorMatrix mat);

	void SetGDIPLUS(Gdiplus::ColorMatrix matrix);
	
	void Multiply(ColorMatrix mat);

	//get the 20 coefs
	void GetElements(float *m);

	void Apply(Color4 &col);
	void Apply(Color3f &col, float &transparency);

	friend int operator== (const ColorMatrix &mat1,	const ColorMatrix &mat2);

	BOOL IsIdentity();

public:
	Gdiplus::ColorMatrix m_Matrix;

private:
	BOOL m_bIsIdentity;
	void SetIdentityFlag();
};


#endif // COLORMATRIX_H

