/*
	This software module was originally developed by ENST in the course of
development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.
	This software module is an implementation of a part of one or more MPEG-4
Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1) 
standard. ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license 
to this software module or modifications thereof for use in hardware or software 
products claiming conformance to the MPEG-4 Systems (ISO/IEC 14496-1). 
Those intending to use this software module in hardware or software products are 
advised that its use may infringe existing patents.
	The original developer of this software module and his/her company, the 
subsequent editors and their companies, and ISO/IEC have no liability for use of 
this software module or modifications thereof in an implementation. Copyright is 
not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
	ENST retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using the code 
for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright    
notice must be included in all copies or derivative works.              
						Copyright  2002.
*/

///////////////////////////////////////////////////////////////////////////
// File: ColorTools.h
//
// Author: Jean Le Feuvre
//
// Description:
//		color tools (yuv->rgb, blending, ...)
//
// Created: February 2003
//
///////////////////////////////////////////////////////////////////////////


#ifndef COLORTOOLS_H
#define COLORTOOLS_H

#include "VisualRender-GDIP.h"


#define clip_col(a) (a<0 ? 0 : (a>255 ? 255 : a) )

#define ConvertYUVtripleToRGB(y, u, v, r, g, b) \
	int t;		\
	t = y - 0.0009267 * (u-128) + 1.4016868* (v-128) ;		\
	r = clip_col(t);		\
	t = y - 0.3436954 * (u-128) - 0.7141690 * (v-128);		\
	g = clip_col(t);			\
	t = y + 1.7721604 * (u-128) + 0.0009902 * (v-128);				\
	b = clip_col(t);			\


//converts YUV buffer to RGB555 (dst) 
void yuv2rgb_555(unsigned char *dst, int dst_stride,
				 unsigned char *y_src, unsigned char * u_src, unsigned char * v_src,
				 int y_stride, int uv_stride, int width, int height, unsigned char const_a, MaterialKey *mk);

//converts YUV buffer to RGB565 (dst) 
void yuv2rgb_565(unsigned char * dst, int dst_stride,
				 unsigned char* y_src, unsigned char* u_src, unsigned char* v_src,
				 int y_stride, int uv_stride, int width, int height, unsigned char const_a, MaterialKey *mk);

//converts YUV buffer to RGB24 (dst) 
void yuv2rgb_24(unsigned char *dst, int dst_stride, unsigned char *y, unsigned char *u, unsigned char * v, 
						 int y_stride, int uv_stride, int width, int height, unsigned char const_a, MaterialKey *mk);

//converts YUV buffer to RGB32 (dst) 
void yuv2rgb_32(unsigned char *dst, int dst_stride, unsigned char *y_src, unsigned char *v_src, unsigned char * u_src,
				 int y_stride, int uv_stride, int width, int height, unsigned char const_a, MaterialKey *mk);

//converts YUVA buffer to RGB32 (dst)
//A plane must be provided
void yuva2rgb_32(unsigned char *dst, int dst_stride, unsigned char *y_src, unsigned char *v_src, unsigned char * u_src, unsigned char *a_src,
				 int y_stride, int uv_stride, int width, int height);




typedef struct
{
	// The top-left point in the source image to start copying
	unsigned int srcStartX;
	unsigned int srcStartY;

	// The dimensions of the portion to be copied
	unsigned int copyWidth;
	unsigned int copyHeight;

	// The top-left point in the destination image to copy to
	unsigned int destStartX;
	unsigned int destStartY;
} CopyParameters;


BOOL SetUpCopyParameters(CopyParameters& cpParams, SrcImage const& srcImage, SurfaceDesc const& surfaceDesc);

void CopyRGBAToRGB555(SrcRGBAImage const& srcImage,
					  SurfaceDesc const& surfaceDesc);
void CopyRGBAToRGB565(SrcRGBAImage const& srcImage,
					  SurfaceDesc const& surfaceDesc);
void CopyRGBAToRGB24bpp(SrcRGBAImage const& srcImage,
						SurfaceDesc const& surfaceDesc);
void CopyRGBAToRGB32bpp(SrcRGBAImage const& srcImage,
						SurfaceDesc const& surfaceDesc);

void CopyYUVToRGB555(SrcYUVAImage const& srcImage,
					 SurfaceDesc const& surfaceDesc);
void CopyYUVToRGB565(SrcYUVAImage const& srcImage,
					 SurfaceDesc const& surfaceDesc);
void CopyYUVToRGB24bpp(SrcYUVAImage const& srcImage,
					   SurfaceDesc const& surfaceDesc);
void CopyYUVToRGB32bpp(SrcYUVAImage const& srcImage,
					   SurfaceDesc const& surfaceDesc);

//returns alpha key for materialKey
unsigned char GetAlphaKey(MaterialKey *mk, unsigned char r, unsigned char g, unsigned char b);

// ApplyAlphaMaskToRGB
//	combines new and old pixel in new according to alpha

#define ApplyAlphaMaskToRGB(red, green, blue, alpha, oldRed, oldGreen, oldBlue) \
	if (alpha == 0) { \
		red = oldRed; green = oldGreen; blue = oldBlue; \
	} else if (alpha<255) {	\
		red   = (red   * alpha + oldRed   * (256 - alpha)) >> 8; \
		green = (green * alpha + oldGreen * (256 - alpha)) >> 8; \
		blue  = (blue  * alpha + oldBlue  * (256 - alpha)) >> 8; \
	}

#define ApplyAlphaMask(comp, alpha, oldComp) \
	clip_col( (!alpha) ? oldComp : ( (alpha==255) ? comp : (oldComp * (255 - alpha) + comp * alpha) >> 8 )	)


#define MAKE_RGB555(red, green, blue) \
	(unsigned short) ((red << 7) & 0x7c00) |  ((green << 2) & 0x03e0) | ((blue >> 3) & 0x001f)


#define MAKE_RGB565(red, green, blue) \
	(unsigned short)(((red & 0xf8) << 8) | ((green & 0xfc) << 3) | ((blue & 0xf8) >> 3))

#define MAKE_RGB32(red, green, blue) \
	(red << 16) | (green << 8) | blue



#endif	//COLORTOOLS_H
