/*
	This software module was originally developed by ENST in the course of
development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.
	This software module is an implementation of a part of one or more MPEG-4
Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1) 
standard. ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license 
to this software module or modifications thereof for use in hardware or software 
products claiming conformance to the MPEG-4 Systems (ISO/IEC 14496-1). 
Those intending to use this software module in hardware or software products are 
advised that its use may infringe existing patents.
	The original developer of this software module and his/her company, the 
subsequent editors and their companies, and ISO/IEC have no liability for use of 
this software module or modifications thereof in an implementation. Copyright is 
not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
	ENST retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using the code 
for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright    
notice must be included in all copies or derivative works.              
						Copyright  2002.
*/

///////////////////////////////////////////////////////////////////////////
// File:  ColorTransformProxy.cpp
//
// Author: Jean Le Feuvre - ENST 2002
//
// Description:
//   proxy object for ColorTransform node.
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include "Nodes-INIT.h"
#include "ColorTransformProxy.h"
#include "Presenter.h"

ColorTransformProxy* Presenter::InitColorTransform(ColorTransform* pObj)
{
	return new ColorTransformProxy(pObj);
}

void ColorTransformProxy::GetMatrix()
{
	ColorTransform *pCT = GetTransform();
	m_Matrix.Set(pCT->mrr, pCT->mrg, pCT->mrb, pCT->mra, pCT->tr, 
			pCT->mgr, pCT->mgg, pCT->mgb, pCT->mga, pCT->tg,
			pCT->mbr, pCT->mbg, pCT->mbb, pCT->mba, pCT->tb,
			pCT->mar, pCT->mag, pCT->mab, pCT->maa, pCT->ta);
}

void ColorTransformProxy::Render(Effects *effects)
{
	Effects *newEffects = new Effects(effects);

	DWORD now = 1 + GetObject()->GetUpdateTime();
	if (now != m_LastUpdateTime) {
		m_LastUpdateTime = now;
		newEffects->m_bInvalidateAll = TRUE;

		//get local matrix
		GetMatrix();
	}

	if (!m_Matrix.IsIdentity()) {
		if (newEffects->m_pColorTransform) {
			newEffects->m_pColorTransform->Multiply(m_Matrix);
		} else {
			newEffects->m_pColorTransform = new ColorMatrix();
			newEffects->m_pColorTransform->Set(m_Matrix);
		}
	}

	// Apply these new effects to the children nodes
	RenderChildren(newEffects);

	delete newEffects;
}

