/*
	This software module was originally developed by ENST in the course of
development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.
	This software module is an implementation of a part of one or more MPEG-4
Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1) 
standard. ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license 
to this software module or modifications thereof for use in hardware or software 
products claiming conformance to the MPEG-4 Systems (ISO/IEC 14496-1). 
Those intending to use this software module in hardware or software products are 
advised that its use may infringe existing patents.
	The original developer of this software module and his/her company, the 
subsequent editors and their companies, and ISO/IEC have no liability for use of 
this software module or modifications thereof in an implementation. Copyright is 
not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
	ENST retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using the code 
for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright    
notice must be included in all copies or derivative works.              
						Copyright  2002.
*/

///////////////////////////////////////////////////////////////////////////
// File: CompositeSurface.h
//
// Authors: Jean Le Feuvre ENST 2002
//
// Modified: 
//
// Description:
//   Composition surface used by Compositetexture2D
//
// Created: September 2002
//
//
///////////////////////////////////////////////////////////////////////////


#ifndef COMPOSITESURFACE_H
#define COMPOSITESURFACE_H

#include "VisualSurface.h"
#include "VisualTexture.h"


class Presenter;

class CompositeSurface : public VisualSurface
{

public:
	CompositeSurface(Presenter *pPresenter);
	~CompositeSurface();


	//creates a new composite surface with requested size in pixels!!
	BOOL Setup(unsigned int pixel_width, unsigned int pixel_height);
	
	virtual void Clear(BRect const* pUpdateRect = NULL);
	virtual float GetWidth();
	virtual float GetHeight();
	virtual BOOL DeviceToSurfaceCoords(int input_X, int input_Y, float *output_X, float *output_Y);
	virtual void DrawBitmap(BRect *clippedDest, BRect *dest, VisualTexture *texture, MaterialKey *mk=NULL);

	VisualTexture *GetTexture() {return m_pTexture; }

	//cleanup of nodes using this surfaces. render must be executed before to force all
	//active bounds to be registered
	void UnregisterNodes();

	//computes default scaling applied to objects (this is needed in case we work with MeterMetrics)
	//public because needs access by CompositeTexture
	virtual BOOL InitObjectMatrix(Matrix2D &mat);

protected:
	//inits the Gdiplus::Graphics object or return false otherwise
	virtual BOOL InitGraphRender(BOOL bForTextDraw = FALSE);
	//releases the Gdiplus::Graphics object
	virtual void ReleaseGraphRender();
	virtual void ResetAntialiasing();

private:
	//offscreen surface
	Gdiplus::Image *m_pImage;
	//physical image size in pixel
	unsigned int m_Width, m_Height;

	BOOL m_bIsClearing, m_bUsePixelMetrics;

	VisualTexture *m_pTexture;

};


#endif	//COMPOSITESURFACE_H