/*
	This software module was originally developed by ENST in the course of
development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.
	This software module is an implementation of a part of one or more MPEG-4
Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1) 
standard. ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license 
to this software module or modifications thereof for use in hardware or software 
products claiming conformance to the MPEG-4 Systems (ISO/IEC 14496-1). 
Those intending to use this software module in hardware or software products are 
advised that its use may infringe existing patents.
	The original developer of this software module and his/her company, the 
subsequent editors and their companies, and ISO/IEC have no liability for use of 
this software module or modifications thereof in an implementation. Copyright is 
not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
	ENST retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using the code 
for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright    
notice must be included in all copies or derivative works.              
						Copyright  2002.
*/

///////////////////////////////////////////////////////////////////////////
// File:  CompositeTexture2DProxy.h
//
// Author: Jean Le Feuvre ENST 
//
// Modified:
//
// Description:
//   The definition of the proxy object used by the presenter
//   to render the CompositeTexture2D node.
//
// Created: September, 2002.
//
//
///////////////////////////////////////////////////////////////////////////


#ifndef COMPOSITETEXTURE2D_PROXY_H
#define COMPOSITETEXTURE2D_PROXY_H


#include "DrawableImage.h"
#include "CompositeSurface.h"
#include "bindable.h"


//we cannot use double heritance on objects deriving from Traversable, thus we duplicate GroupNodeProxy
//functionality in CompositeTexture
class CompositeTexture2DProxy : public DrawableImage
{
public:
	CompositeTexture2DProxy(MediaObject* pMO);
	~CompositeTexture2DProxy();

	// Return a pointer to the ImageTexture node that this node is the proxy for
	CompositeTexture2D* GetCompositeTexture() const { return (CompositeTexture2D *) GetObject(); }

	// Functions inherited from the base classes
	virtual void Update(DrawableContext *pContext);

	DrawableContext *IsPointInTexture(DrawableContext *pContext, float x, float y);

	BOOL HasSensors();

protected:
	virtual BOOL ShouldBuildBrush() {return TRUE; }

private:
	Traversable* GetChild(int n);

	void RenderChildren(Effects *effects);
	void ResortSensors();
	//contains all sensors of this group (avoids double browsing of all children)
	ZArray<Traversable *> m_sensors;
	
	CompositeSurface *m_pSurface;
	DWORD m_LastDrawTime;

	DWORD m_LastUpdateTime;
};


#endif // COMPOSITETEXTURE2D_PROXY_H
