/*
	This software module was originally developed by ENST in the course of
development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.
	This software module is an implementation of a part of one or more MPEG-4
Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1) 
standard. ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license 
to this software module or modifications thereof for use in hardware or software 
products claiming conformance to the MPEG-4 Systems (ISO/IEC 14496-1). 
Those intending to use this software module in hardware or software products are 
advised that its use may infringe existing patents.
	The original developer of this software module and his/her company, the 
subsequent editors and their companies, and ISO/IEC have no liability for use of 
this software module or modifications thereof in an implementation. Copyright is 
not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
	ENST retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using the code 
for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright    
notice must be included in all copies or derivative works.              
						Copyright  2002.
*/

///////////////////////////////////////////////////////////////////////////
// File: CopyYUVA.cpp
//
// Author: Jean Le Feuvre
//
// Description:
//		wrapper for YUV bitmap drawing, actual conversion is in colorTools
//
// Created: February 2003
//
///////////////////////////////////////////////////////////////////////////

#include "defs.h"
#include <assert.h>
#include "VisualRender-GDIP.h"
#include "ColorTools.h"
#include "trace.h"
#include "ddraw.h"

// CopyYUVToRGB555
void CopyYUVToRGB555(SrcYUVAImage const& srcImage, SurfaceDesc const& surfaceDesc)
{
	CopyParameters cpParams;
	if (SetUpCopyParameters(cpParams, srcImage, surfaceDesc)) return;

	unsigned char *dst = (unsigned char*)surfaceDesc.lpSurface;
	dst += cpParams.destStartY * surfaceDesc.lPitch + 2*cpParams.destStartX;
	
	/*here we cheat a bit, the yuv2rgb needs a even width, so in case the width is not (usually due to 
	some translation of the bitmap node) draw one column less...*/
	if (cpParams.copyWidth%2) cpParams.copyWidth -= 1;
	yuv2rgb_555(dst, surfaceDesc.lPitch, srcImage.pY, srcImage.pU, srcImage.pV, srcImage.pitch, srcImage.pitch/2, cpParams.copyWidth, cpParams.copyHeight, srcImage.const_alpha, srcImage.mk);
}

// CopyYUVToRGB565
void CopyYUVToRGB565(SrcYUVAImage const& srcImage, SurfaceDesc const& surfaceDesc)
{
	CopyParameters cpParams;
	if (SetUpCopyParameters(cpParams, srcImage, surfaceDesc)) return;

	unsigned char *dst = (unsigned char*)surfaceDesc.lpSurface;
	dst += cpParams.destStartY * surfaceDesc.lPitch + 2*cpParams.destStartX;
	
	/*here we cheat a bit, the yuv2rgb needs a even width, so in case the width is not (usually due to 
	some translation of the bitmap node) draw one column less...*/
	if (cpParams.copyWidth%2) cpParams.copyWidth -= 1;
	yuv2rgb_565(dst, surfaceDesc.lPitch, srcImage.pY, srcImage.pU, srcImage.pV, srcImage.pitch, srcImage.pitch/2, cpParams.copyWidth, cpParams.copyHeight, srcImage.const_alpha, srcImage.mk);
}

// CopyYUVToRGB24bpp
void CopyYUVToRGB24bpp(SrcYUVAImage const& srcImage,
					   SurfaceDesc const& surfaceDesc)
{
	CopyParameters cpParams;
	if (SetUpCopyParameters(cpParams, srcImage, surfaceDesc)) return;

	unsigned char *dst = (unsigned char*)surfaceDesc.lpSurface;
	dst += cpParams.destStartY * surfaceDesc.lPitch + 3*cpParams.destStartX;
	
	/*here we cheat a bit, the yuv2rgb needs a even width, so in case the width is not (usually due to 
	some translation of the bitmap node) draw one column less...*/
	if (cpParams.copyWidth%2) cpParams.copyWidth -= 1;
	yuv2rgb_24(dst, surfaceDesc.lPitch, srcImage.pY, srcImage.pU, srcImage.pV, srcImage.pitch, srcImage.pitch/2, cpParams.copyWidth, cpParams.copyHeight, srcImage.const_alpha, srcImage.mk);

}

// CopyYUVToRGB32bpp
void CopyYUVToRGB32bpp(SrcYUVAImage const& srcImage, SurfaceDesc const& surfaceDesc)
{
	CopyParameters cpParams;
	if (SetUpCopyParameters(cpParams, srcImage, surfaceDesc)) return;

	unsigned char *dst = (unsigned char*)surfaceDesc.lpSurface;
	dst += cpParams.destStartY * surfaceDesc.lPitch + 4*cpParams.destStartX;
	/*here we cheat a bit, the yuv2rgb needs a even width, so in case the width is not (usually due to 
	some translation of the bitmap node) draw one column less...*/
	if (cpParams.copyWidth%2) cpParams.copyWidth -= 1;

	yuv2rgb_32(dst, surfaceDesc.lPitch, srcImage.pY, srcImage.pU, srcImage.pV, srcImage.pitch, srcImage.pitch/2, cpParams.copyWidth, cpParams.copyHeight, srcImage.const_alpha, srcImage.mk);
}
