/*
	This software module was originally developed by ENST in the course of
development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.
	This software module is an implementation of a part of one or more MPEG-4
Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1) 
standard. ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license 
to this software module or modifications thereof for use in hardware or software 
products claiming conformance to the MPEG-4 Systems (ISO/IEC 14496-1). 
Those intending to use this software module in hardware or software products are 
advised that its use may infringe existing patents.
	The original developer of this software module and his/her company, the 
subsequent editors and their companies, and ISO/IEC have no liability for use of 
this software module or modifications thereof in an implementation. Copyright is 
not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
	ENST retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using the code 
for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright    
notice must be included in all copies or derivative works.              
						Copyright  2002.
*/

///////////////////////////////////////////////////////////////////////////
// File: ColorTools.h
//
// Author: Jean Le Feuvre
//
// Description:
//		direct draw tools
//
// Created: February 2003
//
///////////////////////////////////////////////////////////////////////////



#ifndef DIRECT_DRAW_H
#define DIRECT_DRAW_H


#include <windows.h>
#include <vfw.h>
#include <ddraw.h>
#include <d3d.h>

#include "BRect.h"

//uncomment to force DirectX 3 to be used. surface exports through GDIplus won't work then
//#define USE_DX_3

//only DX7.0 and later can use the v2 interfaces
#if (DIRECTDRAW_VERSION < 0x0700)
#ifndef USE_DX_3
#define USE_DX_3
#endif
#endif

// The different types of RGB displays that are supported
enum RGBpixelType 
{
	RGB565, 
	RGB555,
	RGB24,
	RGB32
};

class DirectDraw
{
public:
	DirectDraw();
	~DirectDraw();
	/*init direct draw, retuns error string or NULL*/
	const char *Init(HWND hwnd, unsigned int compositeWidth, unsigned int compositeHeight);

	//resize the rendering surfaces (primary and RGB offscreen)
	HRESULT Resize(unsigned int newWidth, unsigned int newHeight);

	//blit the offscreen surface to video display (in pDestRect)
	HRESULT RenderSurface(RECT *pDestRect);

	// Lock the offscreen RGB surface for writing.  The details of the
	// surface are written into the given surface descriptor.
	HRESULT LockSurface(DDSURFACEDESC &surfaceDesc);

	// Unlock the offscreen RGB surface, now that it has been finished writing to
	HRESULT UnlockSurface();
	
	// Erase the specifed region of the offscreen RGB surface, filling
	// it with black.  If the rectangle is NULL, erase the entire
	// surface
	HRESULT EraseSurface(RECT const* pUpdateRect, BOOL erasePrimary = FALSE);

	// Get a handle to the device context to use for the offscreen RGB surface.
	HRESULT GetHDC(HDC& hDC);
	HRESULT ReleaseHDC(HDC& hDC);

	// Return the RGB pixel format
	RGBpixelType GetRGBpixelFormat() const { return rgbPixelFormat; }

	//sets full screen and returns NULL if OK or error string
	const char *SetFullScreen(BOOL bFS);

	void SetPreferedYUVFormat(int format);
	/*creates (if needed) a pool surface used for bitmap stretching. If isYUV is TRUE, 
	hardware YUV surface will be created if possible otherwise RGB will be used*/
	HRESULT CreatePoolSurface(unsigned int width, unsigned int height, BOOL isYUV);
	HRESULT LockPool(DDSURFACEDESC &surfaceDesc);
	HRESULT UnlockPool();
	/*scale offscreen pool buffer to RGB offscreen surface*/
	HRESULT ScalePool(RECT src,RECT dstScaled);


	BOOL HasYUVHardware();

	/*use only if hardware YUV*/
	HRESULT WriteYUV(unsigned char *pY, unsigned char *pU, unsigned char *pV, unsigned int width, unsigned int height, unsigned int pitch);

	/*RGB offscreen surface in the format of the primary surface (display format)
	exported for GDIplus frame export...*/
#ifdef USE_DX_3
	LPDIRECTDRAWSURFACE	m_pOffscreen;    
#else
	LPDIRECTDRAWSURFACE7 m_pOffscreen;
#endif


	void CheckYUVHardwareSupport();

private:
	void Shutdown();

	//DirectDraw vars
#ifdef USE_DX_3
	LPDIRECTDRAW lpDD;
	LPDIRECTDRAWSURFACE	m_pPrimary;	
	LPDIRECTDRAWSURFACE	m_pPool;    
	LPDIRECTDRAWCLIPPER	lpClipper;
#else
    LPDIRECTDRAW7 lpDD;
	LPDIRECTDRAWSURFACE7 m_pPrimary;
	LPDIRECTDRAWSURFACE7 m_pPool;
	LPDIRECTDRAWCLIPPER lpClipper;
#endif


	// The RGB pixel type of the current display
	RGBpixelType rgbPixelFormat;

	BOOL GetDisplayMode();
	HWND m_hWnd;

	/*pool surface size is not tied to the offscreen size*/
	unsigned int m_PoolWidth, m_PoolHeight;
	BOOL m_bPoolIsYUV;

	BOOL m_bCheckFormats;

	int m_prefer4CC;

public:
	unsigned int m_Width, m_Height;
	unsigned int m_fsWidth, m_fsHeight;

	BOOL m_bFullScreen;

	/*supported YUV formats for the graphics card*/
	DWORD m_Formats[50];
	int m_numFormats;
	/*current 4CC*/
	int m_fourcc;
};


#endif // DIRECT_DRAW_H
