/*
	This software module was originally developed by ENST in the course of
development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.
	This software module is an implementation of a part of one or more MPEG-4
Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1) 
standard. ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license 
to this software module or modifications thereof for use in hardware or software 
products claiming conformance to the MPEG-4 Systems (ISO/IEC 14496-1). 
Those intending to use this software module in hardware or software products are 
advised that its use may infringe existing patents.
	The original developer of this software module and his/her company, the 
subsequent editors and their companies, and ISO/IEC have no liability for use of 
this software module or modifications thereof in an implementation. Copyright is 
not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
	ENST retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using the code 
for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright    
notice must be included in all copies or derivative works.              
						Copyright  2002.
*/

///////////////////////////////////////////////////////////////////////////
// File:  DiscSensorProxy.cpp
//
// Author: Jean Le Feuvre, august 2002
//
// Description:
//   proxy object for DiscSensor2D node.
//
// Created: August, 2002.
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"

#include "Nodes-INIT.h"
#include "DiscSensorProxy.h"
#include <math.h>
#include "Presenter.h"

#define PI		3.14159265358979323846		/* pi */

DiscSensorProxy* Presenter::InitDiscSensor(DiscSensor* pObj)
{
	return new DiscSensorProxy(pObj);  
}

// DiscSensorProxy::OnUserEvent
BOOL DiscSensorProxy::OnUserEvent(EventInfo const* pEvent, Matrix2D *pSensorMatrix)
{
	// Only generate events if this node is enabled
	if (!(Bool)GetDiscSensor()->enabled) return FALSE;


	// no context, the mouse is not on a object	  
	if (pEvent->m_pContext == NULL) {
		if (m_LastWasNotOver) {
			if (pEvent->m_event == EVENT_MOUSE_UP) {
				GetPresenter()->UnregisterEventHandler(this);
				GetPresenter()->SetTracking(FALSE);
				m_mouseDown = FALSE;
				if ((Bool) GetDiscSensor()->isActive) GetDiscSensor()->isActive = FALSE;

			}

			if (!m_mouseDown) 
				GetPresenter()->UnregisterEventHandler(this);

			return TRUE;
		}
		m_LastWasNotOver = TRUE;
		return FALSE;
	}
	assert(pSensorMatrix);

	m_LastWasNotOver = FALSE;
	
	float X, Y;
	X = pEvent->m_x;
	Y = pEvent->m_y;

	// when the mouse button is first pressed change the state of the sensor
	if ((pEvent->m_event == EVENT_MOUSE_DOWN) && (!m_mouseDown)) {
		m_mouseDown = TRUE;
		//store local matrix
		m_InitialMatrix.Set(*pSensorMatrix);
		//translate to our local coord system
		m_InitialMatrix.ReverseApply(X, Y);
		//compute initial angle
		m_angleStartDrag = atan2(Y, X);
		GetDiscSensor()->isActive = TRUE;

		//and register as active
		GetPresenter()->RegisterEventHandler(this);	
	}
	// sensor is released
	if (pEvent->m_event == EVENT_MOUSE_UP) {
		GetPresenter()->UnregisterEventHandler(this);
		GetPresenter()->SetTracking(FALSE);
		m_mouseDown = FALSE;
		
		//update offset field if autoOffset field is enabled
		if((Bool)(GetDiscSensor()->autoOffset)) 
			GetDiscSensor()->offset = PointToAngle(X, Y, pEvent->m_pContext);

		GetDiscSensor()->isActive = FALSE;
		m_InitialMatrix.SetIdentity();
        return TRUE;
	}
	
	// Generate a trackPoint_changed event and rotation changed
	// if mouse moves after the mouse has been clicked. 
	if ((pEvent->m_event == EVENT_MOUSE_MOVE) && (m_mouseDown) ) {

		GetPresenter()->SetTracking(TRUE);

		//produce the event rotation_changed 
		GetDiscSensor()->rotation_changed = PointToAngle(X, Y, pEvent->m_pContext);

		//generate track_point
		m_InitialMatrix.ReverseApply(X, Y);
		Vec2f point(X, Y);
	   	GetDiscSensor()->trackPoint_changed = point;
	}
	return TRUE;

}

//compute rotation angle :
//	1- translate coords back to original (activation) system
//	2- compute angle
//	3- clip output
double DiscSensorProxy::PointToAngle(float X, float Y, DrawableContext * /*pContext*/)
{
	m_InitialMatrix.ReverseApply(X, Y);
	double result = atan2(Y, X);
	double temp = result ;


	result -= m_angleStartDrag;
	result += GetDiscSensor()->offset;
	if (GetDiscSensor()->minAngle < GetDiscSensor()->maxAngle) {
		if (result < GetDiscSensor()->minAngle) 
			result = GetDiscSensor()->minAngle;

		if (result > GetDiscSensor()->maxAngle)
			result = GetDiscSensor()->maxAngle;

	}
	TRACE("rot_changed %lf - atan %lf\n", result, temp);
	return result;
}
