/*
	This software module was originally developed by ENST in the course of
development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.
	This software module is an implementation of a part of one or more MPEG-4
Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1) 
standard. ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license 
to this software module or modifications thereof for use in hardware or software 
products claiming conformance to the MPEG-4 Systems (ISO/IEC 14496-1). 
Those intending to use this software module in hardware or software products are 
advised that its use may infringe existing patents.
	The original developer of this software module and his/her company, the 
subsequent editors and their companies, and ISO/IEC have no liability for use of 
this software module or modifications thereof in an implementation. Copyright is 
not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
	ENST retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using the code 
for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright    
notice must be included in all copies or derivative works.              
						Copyright  2002.
*/

///////////////////////////////////////////////////////////////////////////
// File:  DiscSensorProxy.h
//
// Author: Jean Le Feuvre ENST 2002
//
///////////////////////////////////////////////////////////////////////////


#ifndef PLANE_SENSOR2D_PROXY_H
#define PLANE_SENSOR2D_PROXY_H


#include "UserEventHandlerProxy.h"
#include "Matrix2D.h"


class DiscSensorProxy : public UserEventHandlerProxy
{
public:
	DiscSensorProxy(MediaObject* pMO) : UserEventHandlerProxy(pMO)
	{
		m_mouseDown = FALSE;
		m_LastWasNotOver = FALSE;
		m_InitialMatrix.SetIdentity();
	}

	// Return a pointer to the DiscSensor node that this node is the proxy for
	DiscSensor* GetDiscSensor() const
	{ return (DiscSensor*)GetObject(); }

	// Functions inherited from the base classes
	BOOL OnUserEvent(EventInfo const* pEvent, Matrix2D *pSensorMatrix);
	
	virtual BOOL IsSensor() { return TRUE; }

	virtual BOOL IsEnabled() {return (BOOL) (Bool) GetDiscSensor()->enabled; }

	//valuator doesn't modify the scene, it only generate events
	virtual BOOL ChangesInvalidateScene() { return FALSE; }

private:
	BOOL m_mouseDown;	// Remember the previous state of the mouse button
	double m_angleStartDrag;
	BOOL m_LastWasNotOver;
	
	double PointToAngle(float X, float Y, DrawableContext *pContext);

	//VRML, CylinderSensor: While the pointing device is activated, trackPoint_changed and rotation_changed events 
	//are output and are interpreted from pointing device motion based on the sensor's local coordinate system at 
	//the time of activation
	//MPEG4: since discSensor is the 2D equivalent of cylinderSensor, behave the same: store the world matrix upon activation
	//and output coords based on that
	Matrix2D m_InitialMatrix;
};


#endif // PLANE_SENSOR2D_PROXY_H
