/*
	This software module was originally developed by ENST in the course of
development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.
	This software module is an implementation of a part of one or more MPEG-4
Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1) 
standard. ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license 
to this software module or modifications thereof for use in hardware or software 
products claiming conformance to the MPEG-4 Systems (ISO/IEC 14496-1). 
Those intending to use this software module in hardware or software products are 
advised that its use may infringe existing patents.
	The original developer of this software module and his/her company, the 
subsequent editors and their companies, and ISO/IEC have no liability for use of 
this software module or modifications thereof in an implementation. Copyright is 
not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
	ENST retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using the code 
for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright    
notice must be included in all copies or derivative works.              
						Copyright  2002.
*/

///////////////////////////////////////////////////////////////////////////
// File:  DrawableImage.h
//
// Author: Jean Le Feuvre - ENST 2002
//
//
///////////////////////////////////////////////////////////////////////////


#ifndef DRAWABLE_IMAGE_H
#define DRAWABLE_IMAGE_H

#include "Nodes.h"
#include "drawable.h"
#include "StreamCo.h"
#include "VisualTexture.h"



class DrawableImage : public DrawableNode
{
public:
	DrawableImage(MediaObject* pMO);
	~DrawableImage();

	//get the StreamConsumer
	StreamConsumer* GetStreamConsumer() const { return (StreamConsumer*)GetObject(); }

	// get the stream that this node reads from.
	MediaStreamImp* GetStream() const { return GetStreamConsumer()->GetStream(); }

	//no Render or Draw for texture
	void Render(Effects * /*effects*/) {}
	virtual void Draw(DrawableContext *) {}
	
	//update the texture info. Textures consuming media shall call this function to update
	//composition memory at render stage
	virtual void Update(DrawableContext *pContext);
	
	//to be called once the texture is drawn in order to release the stream object(s)
	//texture using several objects (MatteTexture) shall oiverride this function
	void Release();

	//this is used by MatteTexture to signal that this texture object should be resfreshed
	//even if unchanged
	//(MatteTexture.overwrite = TRUE)
	void Invalidate() {m_bShouldRedraw = TRUE; }
	
	//the texture object (holds brush info, texture transform/coordinates and pixel data)
	//each proxy is responsible from maintaining this pointer. Pointer is deleted by this class
	//destructor. Ex:
	//-ImageTexture/MovieTexture/PixelTexture init and fill the texture as needed
	//-MatteTexture inits the pointer depending on its "overwrite" field
	VisualTexture *m_pTexture;
	
	//get the textureRepeat info
	void GetTextureRepeat(BOOL &repeatS, BOOL &repeatT);

	// Protected implementation details
protected:
	//to implement in derivate classes. Returns TRUE if video data should be pushed in 
	//associated brush (VisualTexture) or just skipped.
	virtual BOOL ShouldBuildBrush() = 0;

	float GetTransparency(DrawableContext *pDrawableContext) const;

	//stop the stream
	virtual void StopStream();

	//last frame time
	DWORD m_LastFrameTime;
	// Private implementation details
private:
	//reminder for stream Release()
	BOOL m_frameWasFetched;
	
	//invalidates the context of objects using this image
	BOOL m_bShouldRedraw;
	BOOL m_bUpdateNext;
	BOOL m_bLastWasEndOfStream;
	DWORD m_lastFetchTime;
};

#endif // DRAWABLE_IMAGE_H
