/*
	This software module was originally developed by ENST in the course of
development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.
	This software module is an implementation of a part of one or more MPEG-4
Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1) 
standard. ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license 
to this software module or modifications thereof for use in hardware or software 
products claiming conformance to the MPEG-4 Systems (ISO/IEC 14496-1). 
Those intending to use this software module in hardware or software products are 
advised that its use may infringe existing patents.
	The original developer of this software module and his/her company, the 
subsequent editors and their companies, and ISO/IEC have no liability for use of 
this software module or modifications thereof in an implementation. Copyright is 
not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
	ENST retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using the code 
for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright    
notice must be included in all copies or derivative works.              
						Copyright  2002.
*/

///////////////////////////////////////////////////////////////////////////
// File:  EllipseProxy.cpp
//
// Author:		Jean Le Feuvre - ENST 2002
//
// Description:
//   proxy object for Ellipse node 
//
// Created: August 2002.
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include "EllipseProxy.h"
#include "Presenter.h"

// MBS 031223: changed EllipseNode to Ellipse according to spec
EllipseProxy *Presenter::InitEllipse(class Ellipse *pObj)
{
	return new EllipseProxy(pObj); 
}

void EllipseProxy::Render(Effects *effects)
{
	DrawableContext *pContext = InitRenderContext(effects);
	if (!pContext) return;

	//changes in geometry
	BOOL BuildIt = effects->m_bShouldRebuild ? TRUE : FALSE;
	DWORD now = 1 + GetObject()->GetUpdateTime();
	if (now != m_LastUpdateTime) {
		m_LastUpdateTime = now;
		BuildGraph();
		BuildIt = TRUE;
		m_bHasChanged = TRUE;
	}

	//request cliping rect
	m_Path.GetBoundingRect(& pContext->m_boundingRect, &pContext->m_transform, pContext->gProps.lineWidth, pContext->gProps.lineScaled);
	//default for clipping rect is bounding rect
	pContext->m_clippedBoundingRect = pContext->m_boundingRect.GetClipper();

	FinalizeRender(effects, pContext);
}

void EllipseProxy::GetOriginalBounds(BRect &bounds)
{
	m_Path.GetBoundingRect(&bounds, NULL, 0);
}

void EllipseProxy::AddToClipper(VisualClipper *pClipper, DrawableContext *pContext, BOOL bInside, BOOL bUseXOR)
{
	pClipper->AddPath(&m_Path, &pContext->m_transform, bInside, bUseXOR);
}

void EllipseProxy::Draw(DrawableContext * pContext)
{
	if (!SetupDraw(pContext)) return;
	DrawTexture(pContext);
	pContext->m_pVisualSurface->DrawVisualPath(&m_Path);
} 


void EllipseProxy::BuildGraph()
{
	m_Path.Reset();
	Vec2f rad = GetEllipse()->radius;
	m_Path.AddEllipse(rad.x, rad.y);
}

