/*
	This software module was originally developed by ENST in the course of
development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.
	This software module is an implementation of a part of one or more MPEG-4
Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1) 
standard. ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license 
to this software module or modifications thereof for use in hardware or software 
products claiming conformance to the MPEG-4 Systems (ISO/IEC 14496-1). 
Those intending to use this software module in hardware or software products are 
advised that its use may infringe existing patents.
	The original developer of this software module and his/her company, the 
subsequent editors and their companies, and ISO/IEC have no liability for use of 
this software module or modifications thereof in an implementation. Copyright is 
not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
	ENST retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using the code 
for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright    
notice must be included in all copies or derivative works.              
						Copyright  2002.
*/

///////////////////////////////////////////////////////////////////////////
// File:  EmptyProxies.h
//
// Authors: Jean Le Feuvre ENST 2002
//			
// Modified:
//
// Description:
//   All "empty" proxies are defined here (nodes whose proxies are not really usefull ..)
//
// Created: September, 2002.
//
//
///////////////////////////////////////////////////////////////////////////


#ifndef EMPTYPROXIES_H
#define EMPTYPROXIES_H


#include "Traversable.h"

class AppearanceProxy : public Traversable
{
public:
	AppearanceProxy(MediaObject* pMO) : Traversable(pMO) {}

	Appearance* GetAppearance() const { return (Appearance*)GetObject(); }
    MediaObject* GetMaterial2D() const { return  (MediaObject*)(GetAppearance()->material); }

	void Render(Effects *) {}
};


class ColorInterpolatorProxy : public Traversable
{
public:
	ColorInterpolatorProxy(MediaObject* pMO) : Traversable(pMO) {}
	ColorInterpolator* GetColorInterpolator() const { return (ColorInterpolator*)GetObject(); }
	void Render(Effects *) {}
	//doesn't modify the scene
	virtual BOOL ChangesInvalidateScene() { return FALSE; }
};

class Coordinate2DProxy : public Traversable
{
public:
	Coordinate2DProxy(MediaObject* pMO) : Traversable(pMO) {}
	Coordinate2D* GetCoordinate2D() const { return (Coordinate2D*)GetObject(); }
	void Render(Effects *) {} 
};


class ConditionalProxy : public Traversable
{   
public:
	ConditionalProxy(MediaObject* pMO) : Traversable(pMO) {}
	void Render(Effects *) {} 
	//doesn't modify the scene
	virtual BOOL ChangesInvalidateScene() { return FALSE; }
};

class ColorProxy : public Traversable
{
public:
	ColorProxy(MediaObject* pMO) : Traversable(pMO) {}
	Color* GetColor() const { return (Color*)GetObject(); }
	void Render(Effects *) {}
};

class CoordinateInterpolator2DProxy : public Traversable
{
public:
	CoordinateInterpolator2DProxy(MediaObject* pMO) : Traversable(pMO) {}
	CoordinateInterpolator2D* GetCoordinateInterpolator2D() const { return (CoordinateInterpolator2D*)GetObject(); }
	void Render(Effects *) {}
	//doesn't modify the scene
	virtual BOOL ChangesInvalidateScene() { return FALSE; }
};

class FontStyleProxy : public Traversable
{
public:
	FontStyleProxy(MediaObject* pMO) : Traversable(pMO) {}
	void Render(Effects *) {}
};

class LinePropertiesProxy : public Traversable
{
public:
	LinePropertiesProxy(MediaObject* pMO) : Traversable(pMO) {}
	void Render(Effects *) {} 
};

class Material2DProxy : public Traversable
{
public:
	Material2DProxy(MediaObject* pMO) : Traversable(pMO) {}
	Material2D* GetMaterial2D() const { return (Material2D*)GetObject(); }
	LineProperties* GetLineProperties() const { 
		MediaObject *pLineProp = (MediaObject *) GetMaterial2D()->lineProps;
		if (pLineProp && pLineProp->GetNodeCode() == CODE_LineProperties) return (LineProperties *)pLineProp;
		return NULL; 
	}
	XLineProperties* GetXLineProperties() const { 
		MediaObject *pLineProp = (MediaObject *) GetMaterial2D()->lineProps;
		if (pLineProp && pLineProp->GetNodeCode() == CODE_XLineProperties) return (XLineProperties *)pLineProp;
		return NULL; 
	}
	void Render(Effects *) {} 
};

class MaterialKeyProxy : public Traversable
{
public:
	MaterialKeyProxy(MediaObject* pMO) : Traversable(pMO) {}
	MaterialKey* GetMaterialKey() const { return (MaterialKey*)GetObject(); }
   	void Render(Effects *) {} 
};

class PositionInterpolator2DProxy : public Traversable
{
public:
	PositionInterpolator2DProxy(MediaObject* pMO) : Traversable(pMO) {}
	PositionInterpolator2D* GetPositionInterpolator2D() const { return (PositionInterpolator2D*)GetObject(); }
	void Render(Effects *) {}
	//doesn't modify the scene
	virtual BOOL ChangesInvalidateScene() { return FALSE; }
};

class ScalarInterpolatorProxy : public Traversable
{
public:
	ScalarInterpolatorProxy(MediaObject* pMO) : Traversable(pMO) {}
	ScalarInterpolator* GetScalarInterpolator() const { return (ScalarInterpolator*)GetObject(); }
	void Render(Effects *) {}
	//doesn't modify the scene
	virtual BOOL ChangesInvalidateScene() { return FALSE; }
};

class ScriptProxy : public Traversable
{
public:
	ScriptProxy(MediaObject* pMO) : Traversable(pMO) {}
	void Render(Effects *){}
	//doesn't modify the scene
	virtual BOOL ChangesInvalidateScene() { return FALSE; }
};

class TextureTransformProxy : public Traversable
{
public:
	TextureTransformProxy(MediaObject* pMO) : Traversable(pMO) {}
	void Render(Effects *){}
};

class AdvancedFontStyleProxy : public Traversable
{
public:
	AdvancedFontStyleProxy(MediaObject* pMO) : Traversable(pMO) {}
	void Render(Effects *) {}
};

class XLinePropertiesProxy : public Traversable
{
public:
	XLinePropertiesProxy(MediaObject* pMO) : Traversable(pMO) {}
	void Render(Effects *) {} 
};



#endif //EMPTYPROXIES_H

