/*
	This software module was originally developed by ENST in the course of
development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.
	This software module is an implementation of a part of one or more MPEG-4
Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1) 
standard. ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license 
to this software module or modifications thereof for use in hardware or software 
products claiming conformance to the MPEG-4 Systems (ISO/IEC 14496-1). 
Those intending to use this software module in hardware or software products are 
advised that its use may infringe existing patents.
	The original developer of this software module and his/her company, the 
subsequent editors and their companies, and ISO/IEC have no liability for use of 
this software module or modifications thereof in an implementation. Copyright is 
not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
	ENST retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using the code 
for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright    
notice must be included in all copies or derivative works.              
						Copyright  2002.
*/

///////////////////////////////////////////////////////////////////////////
// File: FontEngine.cpp
//
// Author: Jean Le Feuvre
//
// Description:
//		wrapper for font loader
//
// Created: January 2003
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include "VisualPath.h"
#include "FontEngine.h"


#ifndef NO_FONT_ENGINE


#include "itypedll.h"


/*BOLD ratio - 5 gives the less overlapping char on SERIF base font*/
#define IT_BOLD	5
/*italic skewing*/
#define IT_SKEW	5


#ifdef REENTRANT
#define FSPTR	m_pState, 
#define FSPTR0	m_pState
#define FSSTATE (*m_pState)
#else
#define FSPTR	 
#define FSPTR0	 
#define FSSTATE FS_state
#endif

FontEngine::FontEngine()
{
	m_bIsInit = FALSE;
	m_currentStyle = 0;

	m_pState = NULL;
	m_gLib = NULL;
}
FontEngine::~FontEngine()
{
	while (m_Fonts.GetLength()) {
		MY_FONT *pFont = m_Fonts[0];
		m_Fonts.DeleteFrom(0);
		free(pFont->name);
		free(pFont->memptr);
		free(pFont);
	}
	if (m_bIsInit) FS_exit(FSPTR0);

#ifdef REENTRANT
	if (m_pState) free(m_pState);
#endif
	if (m_gLib) FreeLibrary(m_gLib);
}



BOOL FontEngine::Init()
{
	if (m_bIsInit) return TRUE;
	char temp[10];
	unsigned long s = 10;

	ZRegistry reg("Osmo4");
	if (reg.QueryValue("USEITYPE", (unsigned char *)temp, s)) {
		if (! temp[0]) return FALSE;
	}

#ifdef REENTRANT
	m_pState = (FS_STATE *) malloc(sizeof(FS_STATE));
#endif

	m_gLib = LoadLibrary("itype.dll");
	if (!m_gLib) return FALSE;
	FS_init = (FS_INIT)GetProcAddress(m_gLib, "FS_init");
	FS_set_font = (FS_SET_FONT)GetProcAddress(m_gLib, "FS_set_font");
	FS_set_scale = (FS_SET_SCALE)GetProcAddress(m_gLib, "FS_set_scale");
	FS_set_cmap = (FS_SET_CMAP)GetProcAddress(m_gLib, "FS_set_cmap");
	FS_exit = (FS_EXIT)GetProcAddress(m_gLib, "FS_exit");
	FS_get_outline = (FS_GET_OUTLINE)GetProcAddress(m_gLib, "FS_get_outline");
	FS_get_kerning = (FS_GET_KERNING)GetProcAddress(m_gLib, "FS_get_kerning");
	FS_free = (FS_FREE)GetProcAddress(m_gLib, "FS_free");
	FS_get_graymap = (FS_GET_GRAYMAP)GetProcAddress(m_gLib, "FS_get_graymap");
	FS_set_flags = (FS_SET_FLAGS)GetProcAddress(m_gLib, "FS_set_flags");
	FS_free_char = (FS_FREE_CHAR)GetProcAddress(m_gLib, "FS_free_char");
	FS_add_font = (FS_ADD_FONT)GetProcAddress(m_gLib, "FS_add_font");
	FS_delete_font = (FS_DELETE_FONT)GetProcAddress(m_gLib, "FS_delete_font");
	FS_font_metrics = (FS_FONT_METRICS)GetProcAddress(m_gLib, "FS_font_metrics");

	FS_init(FSPTR 4096000);
	/*try to load default fonts*/
	int st = 0;
	SetCurrentFont("Albany for MPEG", st);
	SetCurrentFont("Thorndale for MPEG", st);
	SetCurrentFont("Cumberland for MPEG", st);

	m_bIsInit = TRUE;
	return TRUE;
}

BOOL FontEngine::SetCurrentFont(const char *fontName, int styles)
{
	char _path[MAX_PATH];
	unsigned long size = MAX_PATH;

	m_currentStyle = styles;
	/*check in loaded fonts*/
	if (SetFont(fontName, m_currentStyle)) return 1;

	/*locate and load font in our own fonts*/
	ZRegistry reg("Osmo4");
	if (reg.QueryValue("FONTDIR", (unsigned char *)_path, size)) {
		if (_path[strlen(_path)-1] != '\\') strcat(_path, "\\");
		if (LoadFontInPath(_path, fontName, (m_currentStyle & STYLE_BOLD), (m_currentStyle & STYLE_ITALIC))) return SetFont(fontName, styles);
	}

	/*locate and load font in windows */
	GetWindowsDirectory(_path, size);
	if (_path[strlen(_path)-1] != '\\') strcat(_path, "\\");
	strcat(_path, "Fonts\\");
	if (LoadFontInPath(_path, fontName, (m_currentStyle & STYLE_BOLD), (m_currentStyle & STYLE_ITALIC))) return SetFont(fontName, styles);

	return 0;
}

BOOL FontEngine::SetFont(const char *fontName, int &styles)
{
	char n[1024];
	int found = -1;
	int outStyle = 0;
	BOOL checkBold = FALSE, checkItalic = FALSE;

	if (styles & STYLE_UNDERLINED) outStyle |= STYLE_UNDERLINED;
	if (styles & STYLE_STRIKEOUT) outStyle |= STYLE_STRIKEOUT;
	if (styles & STYLE_BOLD) checkBold = TRUE;
	if (styles & STYLE_ITALIC) checkItalic = TRUE;

	strcpy(n, fontName);
	_strlwr(n);

	/*check in loaded fonts*/
	for (int i=0; i<m_Fonts.GetLength(); i++) {
		if (!strstr(m_Fonts[i]->name, n)) continue;
		/*this is plain ugly*/
		BOOL b = (strstr(n, "bold")!=NULL) ? 1 : 0;
		BOOL it = (strstr(n, "itali")!=NULL) ? 1 : 0;
		if ((b==checkBold) && (it==checkItalic)) {
			FS_set_font(FSPTR m_Fonts[i]->name);
			/*override styles*/
			styles = outStyle;
			FS_set_cmap(FSPTR 3,1);
			return 1;
		}
		if (!b && !it) found = i;
	}
	if (found>= 0) {
		FS_set_font(FSPTR m_Fonts[found]->name);
		FS_set_cmap(FSPTR 3,1);
		return 1;
	}
	return 0;
}



static unsigned char *loadinram(char *file)
{
	int s;
	unsigned char *d;
	FILE *f;
	f = fopen(file, "rb");
	if (!f) return NULL;
	fseek(f, 0, SEEK_END);
	s = ftell(f);
	if (!s) { fclose(f); return NULL; }
	fseek(f, 0, SEEK_SET);
	d = (unsigned char *) malloc(sizeof(unsigned char)*s);
	fread(d, 1, s, f);
	fclose(f);
	return d;
}

BOOL FontEngine::LoadFontInPath(char *path, const char *fontName, BOOL bold, BOOL italic)
{
	char fontDir[MAX_PATH];
	char fontpath[MAX_PATH];
	char fname[MAX_PATH], tmpname[MAX_PATH];
	unsigned char *font_ptr;
	int fontPos, r;
	int found;
	WIN32_FIND_DATA FindData;
	HANDLE SearchH;
	MY_FONT *font;
	BOOL fullCheck = TRUE;

	strcpy(fname, fontName);
	_strlwr(fname);
	if (bold) strcat(fname, " bold");
	if (italic) strcat(fname, " italic");
	found = 0;

	sprintf(fontDir, "%s*.*", path);

	SearchH= FindFirstFile(fontDir, &FindData);
	if (SearchH == INVALID_HANDLE_VALUE) {
		return 0;
	}
	
	while (SearchH != INVALID_HANDLE_VALUE) {
		if (!strcmp(FindData.cFileName, ".") || !strcmp(FindData.cFileName, "..")) goto go_on;
		fullCheck = TRUE;
		/*filter files*/
		if (strstr(FindData.cFileName, ".ttf") || strstr(FindData.cFileName, ".TTF")
			|| strstr(FindData.cFileName, ".ttc") || strstr(FindData.cFileName, ".TTC")
			|| strstr(FindData.cFileName, ".ac3") || strstr(FindData.cFileName, ".AC3")
			|| strstr(FindData.cFileName, ".a3a") || strstr(FindData.cFileName, ".A3A")
			|| strstr(FindData.cFileName, ".a3b") || strstr(FindData.cFileName, ".A3B") 
			) {
			
			if (strstr(FindData.cFileName, ".ac3") || strstr(FindData.cFileName, ".AC3")
				|| strstr(FindData.cFileName, ".a3a") || strstr(FindData.cFileName, ".A3A")
				|| strstr(FindData.cFileName, ".a3b") || strstr(FindData.cFileName, ".A3B") )
				fullCheck = FALSE;
		} else {
			goto go_on;
		}

		font_ptr = NULL;
		sprintf(fontpath, "%s%s", path, FindData.cFileName);
		font_ptr = loadinram(fontpath);
		if (!font_ptr) goto go_on;

		/*figure our the name*/
		fontPos = 0;
		r = FS_add_font(FSPTR "temp_font", (FILECHAR *)0, font_ptr, fontPos);
		if (!r) r = FS_set_font(FSPTR "temp_font");
		if (!r) {
			char *n = FSSTATE.cur_lfnt->ttf->name ? FSSTATE.cur_lfnt->ttf->name->font_name : NULL;
			if (n) {
				strcpy(tmpname, n);
				_strlwr(tmpname);
				if ((fullCheck && !strcmp(tmpname, fname)) || (!fullCheck && strstr(tmpname, fname)) ) {
					font = (MY_FONT *) malloc(sizeof(MY_FONT));
					/*keep name as lowercase*/
					font->name = strdup(tmpname);
					font->memptr = font_ptr;
					font->index = m_Fonts.GetLength();
					m_Fonts += font;
					FS_delete_font(FSPTR "temp_font");
					FS_add_font(FSPTR font->name, (FILECHAR *)0, font->memptr, font->index);
					return 1;
				}
			}
			FS_delete_font(FSPTR "temp_font");
		}
		if (font_ptr) free(font_ptr);

go_on:
		if (!FindNextFile(SearchH, &FindData)) {
			FindClose(SearchH);
			SearchH = INVALID_HANDLE_VALUE;
		}
	}

	/*not found, try without styles*/
	if (bold || italic) return LoadFontInPath(path, fontName, 0, 0);
	/*not found at all*/
	return 0;
}

void FontEngine::SetCurrentSize(float fontSize)
{
	m_currentSize = fontSize*65536;
	if (m_currentStyle & STYLE_ITALIC) {
		FS_set_scale(FSPTR m_currentSize, (__int32) (IT_SKEW * 65536), 0, m_currentSize);
	} else {
		FS_set_scale(FSPTR  m_currentSize, 0, 0, m_currentSize);
	}
	FSSTATE.bold_pct = (m_currentStyle & STYLE_ITALIC) ? ( 0.5 + IT_BOLD * 655.36) : 0;
}

BOOL FontEngine::GetCurrentMetrics(int &ascent, int &descent, int &em_size)
{
	FONT_METRICS fm;
	if (FS_font_metrics(FSPTR &fm)) return FALSE;
	ascent = fm.hhea_ascent;
	descent = fm.hhea_descent;
	em_size = fm.unitsPerEm;
	return TRUE;
}


BOOL FontEngine::GetTextSize(unsigned short *string, float &width, float &height)
{
	if (!string) return FALSE;
	FS_OUTLINE *ol;
	int len = wcslen(string);

	float min_y, max_y;
	min_y = max_y = 0;
	width = height = 0;
	for (int i=0; i<len;i++) {
		ol = FS_get_outline(FSPTR string[i]);
		width += (float) (ol->dx ) / (float) FIXED_ONE;
		float res = (float) ol->hi_y / (float) FIXED_ONE;
		if (res > max_y) max_y = res;
		res = (float) ol->lo_y / (float) FIXED_ONE;
		if (res < min_y) min_y = res;
		FS_free_char(FSPTR ol);
	}
	height = max_y-min_y;	
	return TRUE;
}

void FontEngine::AddTextToPath(VisualPath *path, unsigned short *string, float &start_x, float &start_y, float x_scaling, float y_scaling, float &end_x, float &end_y)
{

	FS_OUTLINE *ol;
	PointF coord[4];
	float o_x, o_y;
	/*if we're asked to scale do it now*/
	if ( (x_scaling!=1.0) || (y_scaling!=1.0)) {
		if (m_currentStyle & STYLE_ITALIC) {
			FS_set_scale(FSPTR x_scaling*m_currentSize, (__int32) (IT_SKEW * 65536), 0, y_scaling*m_currentSize);
		} else {
			FS_set_scale(FSPTR x_scaling*m_currentSize, 0, 0, y_scaling*m_currentSize);
		}
	}
	o_x = start_x;
	o_y = start_y;
	int len = wcslen(string);

	for (int i=0; i<len; i++) {
		ol = FS_get_outline(FSPTR string[i]);
		int k=0;

		for (SHORT j = 0; j<ol->num; j++) {
			switch ((BYTE) ol->type[j]) {
			case FS_LINETO:
				coord[1].x = o_x + (float) ol->x[k] / (float) FIXED_ONE;
				coord[1].y = o_y + (float) ol->y[k] / (float) FIXED_ONE;
				path->AddLine(coord[0].x, coord[0].y, coord[1].x, coord[1].y);
				coord[0].x = coord[1].x;
				coord[0].y = coord[1].y;
				k++;
				break;
			case FS_QUADTO:
				coord[1].x = o_x + (float) ol->x[k] / (float) FIXED_ONE;
				coord[1].y = o_y + (float) ol->y[k] / (float) FIXED_ONE;
				coord[2].x = o_x + (float) ol->x[k+1] / (float) FIXED_ONE;
				coord[2].y = o_y + (float) ol->y[k+1] / (float) FIXED_ONE;
				path->AddBezier(coord, 3, 1.0);
				coord[0].x = coord[2].x;
				coord[0].y = coord[2].y;
				k += 2;
				break;
			case FS_CUBETO:
				coord[1].x = o_x + (float) ol->x[k] / (float) FIXED_ONE;
				coord[1].y = o_y + (float) ol->y[k] / (float) FIXED_ONE;
				coord[2].x = o_x + (float) ol->x[k+1] / (float) FIXED_ONE;
				coord[2].y = o_y + (float) ol->y[k+1] / (float) FIXED_ONE;
				coord[3].x = o_x + (float) ol->x[k+2] / (float) FIXED_ONE;
				coord[3].y = o_y + (float) ol->y[k+2] / (float) FIXED_ONE;
				path->AddBezier(coord, 4, 1.0);
				coord[0].x = coord[3].x;
				coord[0].y = coord[3].y;
				k += 3;
				break;
			case FS_MOVETO:
			default:
				coord[0].x = o_x + (float) ol->x[k] / (float) FIXED_ONE;
				coord[0].y = o_y + (float) ol->y[k] / (float) FIXED_ONE;
				if (!j && !i) {
					start_x = coord[0].x;
					start_y = coord[0].y;
				}
				k++;
				if (j) path->CloseFigure();
				break;
			}
		}
		path->CloseFigure();
		o_x += (float) ol->dx / (float) FIXED_ONE;
		FS_free_char(FSPTR ol);
	}
	end_x = o_x;
	end_y = start_y;

}

#else  //NO_FONT_ENGINE

FontEngine::FontEngine() {m_bIsInit = FALSE;}
FontEngine::~FontEngine() {}
BOOL FontEngine::Init() {return TRUE;}
BOOL FontEngine::SetCurrentFont(const char *fname, int styles) {return FALSE; }
void FontEngine::SetCurrentSize(float size) {}
BOOL FontEngine::GetCurrentMetrics(int &ascent, int &descent, int &em_size) {return FALSE; }
BOOL FontEngine::GetTextSize(unsigned short *string, float &width, float &height) {return FALSE; }
void FontEngine::AddTextToPath(VisualPath *path, unsigned short *string, float &start_x, float &start_y, float x_scaling, float y_scaling, float &end_x, float &end_y) {}

#endif	//NO_FONT_ENGINE
