/*
	This software module was originally developed by ENST in the course of
development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.
	This software module is an implementation of a part of one or more MPEG-4
Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1) 
standard. ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license 
to this software module or modifications thereof for use in hardware or software 
products claiming conformance to the MPEG-4 Systems (ISO/IEC 14496-1). 
Those intending to use this software module in hardware or software products are 
advised that its use may infringe existing patents.
	The original developer of this software module and his/her company, the 
subsequent editors and their companies, and ISO/IEC have no liability for use of 
this software module or modifications thereof in an implementation. Copyright is 
not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
	ENST retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using the code 
for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright    
notice must be included in all copies or derivative works.              
						Copyright  2002.
*/

///////////////////////////////////////////////////////////////////////////
// File: FontEngine.h
//
// Author: Jean Le Feuvre
//
// Description:
//		wrapper for font loader
//
// Created: January 2003
//
///////////////////////////////////////////////////////////////////////////


#ifndef _FONTENGINE_H
#define _FONTENGINE_H

// If you want to use the iType font engine with integrated support for
// MPEG-4 compressed fonts (instead of GDIplus) - undefine NO_FONT_ENGINE.
// For details on using FONT_ENGINE please look at FontEngineAPI.doc in
// IM1\Docs folder.
// DO NOT FORGET to define "include path" to (ITYPE_API)
#define NO_FONT_ENGINE


#ifndef NO_FONT_ENGINE

#include "ztl.h"
#include "port.h"
#include "object.h"
#include "fixed.h"

#endif	//NO_FONT_ENGINE

enum 
{
	STYLE_NONE = 0,
	STYLE_BOLD = 1,
	STYLE_ITALIC = (1<<1),
	STYLE_UNDERLINED = (1<<2),
	STYLE_STRIKEOUT = (1<<3)
};

class VisualPath;

class FontEngine
{
public:
	FontEngine();
	~FontEngine();

	BOOL Init();
	BOOL IsInit() { return m_bIsInit; }

	BOOL SetCurrentFont(const char *fname, int styles);
	void SetCurrentSize(float size);
	BOOL GetCurrentMetrics(int &ascent, int &descent, int &em_size);

	BOOL GetTextSize(unsigned short *string, float &width, float &height);

	void AddTextToPath(VisualPath *path, unsigned short *string, float &start_x, float &start_y, float x_scaling, float y_scaling, float &end_x, float &end_y);


#ifndef NO_FONT_ENGINE

#ifdef REENTRANT
	FS_STATE *GetHandle() {return m_pState;}
#else
	void GetHandle() {}
#endif

#else
	void GetHandle() {}
#endif	//NO_FONT_ENGINE


private:
	BOOL SetFont(const char *fname, int &styles);
	BOOL LoadFontInPath(char *path, const char *fontName, BOOL bold, BOOL italic);

#ifndef NO_FONT_ENGINE
	
	HINSTANCE m_gLib;

#ifdef REENTRANT
	FS_STATE *m_pState;
#endif

	ZArray<MY_FONT *> m_Fonts;

#endif //NO_FONT_ENGINE

	BOOL m_bIsInit;
	int m_currentStyle;
	int m_currentSize;
};

#endif //_FONTENGINE_H