/*
	This software module was originally developed by ENST in the course of
development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.
	This software module is an implementation of a part of one or more MPEG-4
Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1) 
standard. ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license 
to this software module or modifications thereof for use in hardware or software 
products claiming conformance to the MPEG-4 Systems (ISO/IEC 14496-1). 
Those intending to use this software module in hardware or software products are 
advised that its use may infringe existing patents.
	The original developer of this software module and his/her company, the 
subsequent editors and their companies, and ISO/IEC have no liability for use of 
this software module or modifications thereof in an implementation. Copyright is 
not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
	ENST retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using the code 
for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright    
notice must be included in all copies or derivative works.              
						Copyright  2002.
*/

///////////////////////////////////////////////////////////////////////////
// File:  FormProxy.cpp
//
// Author:  Jean Le Feuvre ENST 2002
//
// Description:
//   proxy object for Form node.
//
// Created: August 2002.
//
//	Note: Form spec is wrong regarding align constraint when one of the
//	constrained group is form itself. The implementation thus doesn't follow
//	the spec
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include "MediaObj.h"
#include "Presenter.h"
#include "nodes.h"
#include "FormProxy.h"
#include "drawable.h"

LPCTSTR sAllowedConstraint[]=
{
 _T("AL"),
 _T("AH"),
 _T("AR"),
 _T("AT"),
 _T("AV"),
 _T("AB"),
 _T("SH"),
 _T("SHin"),
 _T("SV"),
 _T("SVin")
} ;
#define NUMBER_OF_CONSTRAINTS 20

int factory_order = 0;

ConstraintsFactoryBase *FormProxy :: TheConstraintsFactory [NUMBER_OF_CONSTRAINTS];

FormProxy* Presenter::InitForm(Form* pObj)
{
	return new  FormProxy(pObj);
}

FormProxy::FormProxy(MediaObject* pMO): GroupNodeProxy(pMO)
{
	m_lastUpdateTime = 0;
	//reset factory order in case a new form is loaded
	factory_order = 0;
	m_drawableContext.m_pColorMatrix = NULL;
}

FormProxy::~FormProxy()
{
	ResetGroupList();
	ResetChildren();
}

#define MAX_GROUP_SIZE	100

void FormProxy::Render(Effects *effects)
{
	int _groupIndex[MAX_GROUP_SIZE];
	
	Effects *newEffects = new Effects(effects);
	newEffects->m_ParentGroup = this;
	//render with a null transform so that we can compute the align/distr
	newEffects->transform.SetIdentity();

	//create the form bounding rect which is referred in the GroupsIndexList as 0 
	InitFormContext();

	RenderChildren(newEffects);
	CenterChildren();

	DWORD now = GetObject()->GetUpdateTime() + 1;
	
	//reset group list and rebuild
	if (m_lastUpdateTime != now) 
		BuildGroupList();

	int i;
	int index, last_index =0;
	Form *pForm = GetForm();

	for (i=0; i<pForm->constraints.GetLength(); i++) {
		index = 0;
		while (1) {
			//bad format
			if (last_index + index >= pForm->groupsIndex.GetLength()) return;
			if (pForm->groupsIndex[last_index + index]==-1) break;
			if (index > MAX_GROUP_SIZE) goto exit;
			_groupIndex[index] = pForm->groupsIndex[last_index + index];
			index += 1;
		}
		ApplyConstraint(pForm->constraints[i], _groupIndex, index);
		index += 1;
		last_index += index;

		//this is needed because a constraint may modify a node bounds that is used by another group
		//not in this constraint
		RefreshGroupBounds();

		//done
		if (last_index >= pForm->groupsIndex.GetLength()) break;
	}

	//finally update all children bounds and clipper info
	for (i=0; i<GetObject()->GetChildCount(); i++) {
		m_Groups[i+1]->UpdateAndDraw(effects, &m_Groups[0]->m_OriginalBounds);
	}

exit:
	ResetChildren();
	delete newEffects;
}

// FormProxy::CreateFormContext
void FormProxy::InitFormContext()
{
	ChildGroup *cg = StartChild();
	cg->m_OriginalBounds.height = ((Vec2f)GetForm()->size).y;
	cg->m_OriginalBounds.width = ((Vec2f)GetForm()->size).x;
	if (cg->m_OriginalBounds.height<0) cg->m_OriginalBounds.height = FLT_MAX;
	if (cg->m_OriginalBounds.width<0) cg->m_OriginalBounds.width = FLT_MAX;

	cg->m_OriginalBounds.left = - cg->m_OriginalBounds.width / 2;
	cg->m_OriginalBounds.top = cg->m_OriginalBounds.height/2;
	//store final bounds
	cg->m_FinalBounds = cg->m_OriginalBounds;

	//register context
	AddToDrawableContextList(&m_drawableContext);
}


void FormProxy::ResetGroupList()
{
	while (m_GroupList.GetLength()) {
		FormGroup * fg = m_GroupList[0];
		m_GroupList.DeleteFrom(0);
		delete fg;
	}
}

void FormProxy::BuildGroupList()
{
	Form *form = GetForm();
	FormGroup *fg = NULL;

	ResetGroupList();

	//add form group
	fg = new FormGroup;
	fg->m_Elements += m_Groups[0];
	m_GroupList += fg;
	fg->ComputeBounds();
	
	fg = NULL;
	for (int i=0; i< form->groups.GetLength(); i++) {
		//start of group
		if (!fg) {
			fg = new FormGroup;
			m_GroupList += fg;
		}
		//end of group
		if (form->groups[i] == -1) {
			fg->ComputeBounds();
			fg = NULL;
			continue;
		}
		if (form->groups[i] >= m_Groups.GetLength()) {
			ResetGroupList();
			return;
		}
		fg->m_Elements += m_Groups[form->groups[i]];
	}
}

void FormProxy::RefreshGroupBounds()
{
	for (int i=0; i<m_GroupList.GetLength(); i++) {
		m_GroupList[i]->ComputeBounds();
	}
}

void FormProxy::CenterChildren()
{
	for (int i=0; i<GetObject()->GetChildCount(); i++) {
		m_Groups[i+1]->m_FinalBounds.left = - m_Groups[i+1]->m_FinalBounds.width/2;
		m_Groups[i+1]->m_FinalBounds.top = m_Groups[i+1]->m_FinalBounds.height/2;
	}
}


// FormProxy::ApplyConstrain
// returns a pointer to the indexes of DrawableContext taken from 
// the Contextset of nodes from the groupsIndex field of a Form
// each group is separated by a -1
void FormProxy::ApplyConstraint(ZString zsConstraint,const int * iSetOfGroup,int iSetLength)
{

	CString csCopyofConstraint((LPCSTR)zsConstraint);
	CString csToken;
	
	//GetFirstToken (example "SH xxx", SH is first token)
	int iFirstTokenEnd=csCopyofConstraint.Find(' ');
	
	if(iFirstTokenEnd!=-1)
		csToken=csCopyofConstraint.Left(iFirstTokenEnd);
	else
		csToken=csCopyofConstraint;

	//find right factory for the token
	int i=0;
	while (1) {
		if (! sAllowedConstraint[i]) return;
		if (! _tcscmp(csToken, sAllowedConstraint[i]) ) break;
		i++;
	}
		
	//allocate the constraint using the right factory
	Constraint * pConstraint= (Constraint *)(TheConstraintsFactory[i])->CreateConstraint();		 
	
	//save form pointer in Constraint class
	pConstraint->m_pFormProxy=this;
	
	//Get second token
	int iSpace;
	if(iFirstTokenEnd!=-1) {
		csToken=csCopyofConstraint.Mid(iFirstTokenEnd);
		csToken.Remove(' ');
		if (!csToken.IsEmpty()) {
			iSpace=_ttoi(csToken);
			pConstraint->m_iSpace=iSpace;
		}
	}
	pConstraint->Apply(iSetOfGroup, iSetLength);
	delete pConstraint;
}

// AL::Apply
// 
// constrain group to the xmin of the lefttop component
void AL::Apply(const int* pGroup, int groupListLength)
{
	float iXmin;
	int i;
	if(m_iSpace==-1) {
		iXmin = m_pFormProxy->GetGroup(pGroup[0])->m_FinalBounds.left;
		//get the min left
		for (i=1; i<groupListLength; i++) {
			if (pGroup[i]==0) {
				iXmin = m_pFormProxy->GetGroup(pGroup[i])->m_FinalBounds.left;
				break;
			}
			if (m_pFormProxy->GetGroup(pGroup[i])->m_FinalBounds.left < iXmin)
				iXmin = m_pFormProxy->GetGroup(pGroup[i])->m_FinalBounds.left;
		}
	} else {
		//The xmin of the second and following components become equal to the xmin of the 
		//first component plus the specified space
		iXmin = m_pFormProxy->GetGroup(pGroup[0])->m_FinalBounds.left + m_iSpace;
	}

	//change transform matrix for all elements of the group and update bounds
	for (i=0; i<groupListLength; i++) {
		//skip the form node
		if(pGroup[i] == 0) continue;
		m_pFormProxy->GetGroup(pGroup[i])->m_FinalBounds.left = iXmin;

		//update this group bounds
		m_pFormProxy->GetGroup(pGroup[i])->UpdateBounds();
	}
}

// AR::Apply
//  
// constrain group to the xmax of the right most component
void AR::Apply(const int*pGroup,int groupListLength)
{
	float iXmax;
	int i;
	BRect *bounds;

	if(m_iSpace==-1) {
		bounds = &m_pFormProxy->GetGroup(pGroup[0])->m_FinalBounds;
		iXmax = bounds->left + bounds->width;
		
		for (i=1; i<groupListLength; i++) {
			bounds = &m_pFormProxy->GetGroup(pGroup[i])->m_FinalBounds;
			if (pGroup[i]==0) {
				iXmax = bounds->left + bounds->width;
				break;
			}
			if (bounds->left + bounds->width > iXmax)
				iXmax = bounds->left + bounds->width;
		}
	} else {
		bounds = &m_pFormProxy->GetGroup(pGroup[0])->m_FinalBounds;
		iXmax = bounds->left + bounds->width - m_iSpace;
	}

	//change transform matrix for all elements of the group and update bounds
	for (i=0; i<groupListLength; i++) {
		//skip the form node
		if(pGroup[i] == 0) continue;
		bounds = &m_pFormProxy->GetGroup(pGroup[i])->m_FinalBounds;
		bounds->left = iXmax - bounds->width;
		//update this group bounds
		m_pFormProxy->GetGroup(pGroup[i])->UpdateBounds();
	}
}

// AT::Apply
// 
// constrain group to the ymax of the top most component
void AT::Apply(const int*pGroup,int groupListLength)
{
	float iYmax;
	int i;

	if(m_iSpace==-1) {
		iYmax = m_pFormProxy->GetGroup(pGroup[0])->m_FinalBounds.top;
	
		//get the min left
		for (i=1; i<groupListLength; i++) {
			if (pGroup[i]==0) {
				iYmax = m_pFormProxy->GetGroup(pGroup[i])->m_FinalBounds.top;
				break;
			}
			if (m_pFormProxy->GetGroup(pGroup[i])->m_FinalBounds.top > iYmax)
				iYmax = m_pFormProxy->GetGroup(pGroup[i])->m_FinalBounds.top;
		}
	} else {
		//The xmin of the second and following components become equal to the xmin of the 
		//first component plus the specified space
		iYmax = m_pFormProxy->GetGroup(pGroup[0])->m_FinalBounds.top - m_iSpace;
	}

	//change transform matrix for all elements of the group and update bounds
	for (i=0; i<groupListLength; i++) {
		//skip the form node
		if(pGroup[i] == 0) continue;
		m_pFormProxy->GetGroup(pGroup[i])->m_FinalBounds.top = iYmax;
		//update this group bounds
		m_pFormProxy->GetGroup(pGroup[i])->UpdateBounds();
	}
} 


// AB::Apply
// 
// constrain group to the ymin of the bottom-most component
void AB::Apply(const int*pGroup,int groupListLength)
{
	float iYmin;
	int i;
	BRect *bounds;

	if(m_iSpace==-1) {
		bounds = &m_pFormProxy->GetGroup(pGroup[0])->m_FinalBounds;
		iYmin = bounds->top - bounds->height;
		for (i=1; i<groupListLength; i++) {
			bounds = &m_pFormProxy->GetGroup(pGroup[i])->m_FinalBounds;
			if (pGroup[i]==0) {
				iYmin = bounds->top - bounds->height;
				break;
			}
			if (bounds->top - bounds->height < iYmin)
				iYmin = bounds->top - bounds->height;
		}
	} else {
		bounds = &m_pFormProxy->GetGroup(pGroup[0])->m_FinalBounds;
		iYmin = bounds->top - bounds->height + m_iSpace;
	}

	//change transform matrix for all elements of the group and update bounds
	for (i=0; i<groupListLength; i++) {
		//skip the form node
		if(pGroup[i] == 0) continue;
		bounds = &m_pFormProxy->GetGroup(pGroup[i])->m_FinalBounds;
		bounds->top = iYmin + bounds->height;
		//update this group bounds
		m_pFormProxy->GetGroup(pGroup[i])->UpdateBounds();
	}
}

//align centers horizontally based on original center
void AH::Apply(const int*pGroup,int groupListLength)
{
	BRect *bounds;
	int i;
	float left, right;
	left = right = 0;

	//compute center
	float center=0;
	for (i=0; i<groupListLength; i++) {
		if(pGroup[i] == 0) {
			bounds = &m_pFormProxy->GetGroup(pGroup[i])->m_FinalBounds;
			center = bounds->left + bounds->width / 2;
			break;
		}
		bounds = &m_pFormProxy->GetGroup(pGroup[i])->m_FinalBounds;
		if (left > bounds->left) left = bounds->left;
		if (right < bounds->left + bounds->width) right = bounds->left + bounds->width;
		center = (left+right)/2;
	}

	//then align center
	for (i=0; i<groupListLength; i++) {
		//skip the form node
		if(pGroup[i] == 0) continue;
		BRect *bounds = &m_pFormProxy->GetGroup(pGroup[i])->m_FinalBounds;
		//translate 
		bounds->left = center - bounds->width/2;
		//update this group bounds
		m_pFormProxy->GetGroup(pGroup[i])->UpdateBounds();
	}
}

//align centers vertically based on original center
void AV::Apply(const int*pGroup,int groupListLength)
{
	int i;
	float top, bottom;
	BRect *bounds;
	top = bottom = 0;

	//compute center
	float center=0;
	for (i=0; i<groupListLength; i++) {
		bounds = &m_pFormProxy->GetGroup(pGroup[i])->m_FinalBounds;
		if(pGroup[i] == 0) {
			bounds = &m_pFormProxy->GetGroup(pGroup[i])->m_FinalBounds;
			center = bounds->top - bounds->height / 2;
			break;
		}
		if (top < bounds->top) top = bounds->top;
		if (bottom > bounds->top - bounds->height) bottom = bounds->top - bounds->height;
		center = (top+bottom)/2;
	}

	//then align center
	for (i=0; i<groupListLength; i++) {
		//skip the form node
		if(pGroup[i] == 0) continue;
		BRect *bounds = &m_pFormProxy->GetGroup(pGroup[i])->m_FinalBounds;
		//translate 
		bounds->top = center + bounds->height/2;
		//update this group bounds
		m_pFormProxy->GetGroup(pGroup[i])->UpdateBounds();
	}
}

// SH::Apply
// 
// The differences between the xmin of each component and the xmax of the
// previous one all become equal, first and last already constrained 
// horizontally
void SH::Apply(const int*pGroup,int groupListLength)
{
	int i, k;
	BRect *bl, *br;
	int iSumOfChildrenLength=0;
	float fSpaceBetweenObjects;

	if (m_iSpace == -1) {
		br = &m_pFormProxy->GetGroup(pGroup[groupListLength-1])->m_FinalBounds;
		bl = &m_pFormProxy->GetGroup(pGroup[0])->m_FinalBounds;
		fSpaceBetweenObjects = br->left - bl->left;
		if(pGroup[0]!=0) fSpaceBetweenObjects -= bl->width;

		for (i=1; i<groupListLength-1; i++)
			iSumOfChildrenLength += m_pFormProxy->GetGroup(pGroup[i])->m_FinalBounds.width;

		fSpaceBetweenObjects -= iSumOfChildrenLength;
		fSpaceBetweenObjects /= (groupListLength-1);
	} else {
		fSpaceBetweenObjects = m_iSpace;
	}	
	
	k = groupListLength - 1;
	if (m_iSpace != -1) k += 1;
	for (i=1;i<k;i++) {
		if (pGroup[i] ==0) continue;
		bl = &m_pFormProxy->GetGroup(pGroup[i-1])->m_FinalBounds;
		br = &m_pFormProxy->GetGroup(pGroup[i])->m_FinalBounds;
		br->left = bl->left + fSpaceBetweenObjects;
		if(pGroup[i-1]!=0) br->left += bl->width;
		//update this group bounds
		m_pFormProxy->GetGroup(pGroup[i])->UpdateBounds();
	}
}

// SHin::Apply
//  
// differences between the xmin of each component and the xmax of the 
// previous one all become equal, references the edges of the  form
void SHin::Apply(const int*pGroup,int groupListLength)
{
	int i;
	int iSumOfChildrenLength=0;
	float fSpaceBetweenObjects = m_pFormProxy->GetGroup(0)->m_FinalBounds.width;

	int actualLength = 0;
	for (i=0; i<groupListLength; i++) {
		//skip form
		if (pGroup[i] != 0) {
			iSumOfChildrenLength += m_pFormProxy->GetGroup(pGroup[i])->m_FinalBounds.width;
			actualLength++;
		}
	}
	fSpaceBetweenObjects -= iSumOfChildrenLength;
	fSpaceBetweenObjects /= (actualLength+1);

	for (i=0; i<groupListLength; i++) {
		if(pGroup[i] == 0) continue;

		if (!i) {
			m_pFormProxy->GetGroup(pGroup[0])->m_FinalBounds.left = m_pFormProxy->GetGroup(0)->m_FinalBounds.left + fSpaceBetweenObjects;
		} else {
			m_pFormProxy->GetGroup(pGroup[i])->m_FinalBounds.left = 
				m_pFormProxy->GetGroup(pGroup[i-1])->m_FinalBounds.left +
				m_pFormProxy->GetGroup(pGroup[i-1])->m_FinalBounds.width +
				fSpaceBetweenObjects;
		}
		//update this group bounds
		m_pFormProxy->GetGroup(pGroup[i])->UpdateBounds();
	}
}

// SV::Apply
// 
// The differences between the ymin of each component and the ymax
// of the previous one all become equal, the first and the last 
// component shall be constrained vertically already
void SV::Apply(const int*pGroup,int groupListLength)
{
	int i;
	int iSumOfChildrenLength=0;
	float fSpaceBetweenObjects;
	BRect *bt, *bb;

	if(m_iSpace == -1) {
		bt = &m_pFormProxy->GetGroup(pGroup[groupListLength-1])->m_FinalBounds;
		bb = &m_pFormProxy->GetGroup(pGroup[0])->m_FinalBounds;
		fSpaceBetweenObjects = bb->top - bt->top;
		if (pGroup[0]!=0) fSpaceBetweenObjects -= bt->height;

		for (i=1; i<groupListLength-1; i++)
			iSumOfChildrenLength += m_pFormProxy->GetGroup(pGroup[i])->m_FinalBounds.height;

		fSpaceBetweenObjects -= iSumOfChildrenLength;
		fSpaceBetweenObjects /= groupListLength-1;
	} else {
		fSpaceBetweenObjects=m_iSpace;
	}	
	
	int k = groupListLength-1;
	if(m_iSpace!=-1) k += 1;
	for (i=1; i<k; i++) {
		if (pGroup[i] == 0) continue;

		m_pFormProxy->GetGroup(pGroup[i])->m_FinalBounds.top = 
			m_pFormProxy->GetGroup(pGroup[i-1])->m_FinalBounds.top - 
			fSpaceBetweenObjects;

		if (pGroup[i-1]!=0) m_pFormProxy->GetGroup(pGroup[i])->m_FinalBounds.top -= 
			m_pFormProxy->GetGroup(pGroup[i-1])->m_FinalBounds.height;

		//update this group bounds
		m_pFormProxy->GetGroup(pGroup[i])->UpdateBounds();
	}
}


// SVin::Apply
// The differences between the ymin of each component and the ymax
// of the previous one all become equal, references are the edges of
// the layout 	
void SVin::Apply(const int*pGroup,int groupListLength)
{
	int i;
	int iSumOfChildrenLength=0;

	float fSpaceBetweenObjects = m_pFormProxy->GetGroup(0)->m_FinalBounds.height;

	int actualLength = 0;
	for (i=0; i<groupListLength;i++) {
		if (pGroup[i] != 0) {
			iSumOfChildrenLength += m_pFormProxy->GetGroup(pGroup[i])->m_FinalBounds.height;
			actualLength++;
		}
	}

	fSpaceBetweenObjects -= iSumOfChildrenLength;
	fSpaceBetweenObjects /= actualLength;
				
	for (i=0; i<groupListLength; i++) {
		if (pGroup[i] == 0) continue;
		if (!i) {
			m_pFormProxy->GetGroup(pGroup[0])->m_FinalBounds.top = m_pFormProxy->GetGroup(0)->m_FinalBounds.top - fSpaceBetweenObjects;
		} else {
			m_pFormProxy->GetGroup(pGroup[i])->m_FinalBounds.top = 
				m_pFormProxy->GetGroup(pGroup[i-1])->m_FinalBounds.top - 
				m_pFormProxy->GetGroup(pGroup[i-1])->m_FinalBounds.height - 
				fSpaceBetweenObjects;				
		}
		//update this group bounds
		m_pFormProxy->GetGroup(pGroup[i])->UpdateBounds();
	}
}


void FormGroup::ComputeBounds()
{
	m_OriginalBounds.MakeEmpty();
	m_FinalBounds.MakeEmpty();
	for (int i=0; i<m_Elements.GetLength(); i++) {
		m_OriginalBounds.Union(m_Elements[i]->m_FinalBounds);
	}
	m_FinalBounds = m_OriginalBounds;
}

void FormGroup::UpdateBounds()
{
	float x, y;
	x = m_FinalBounds.left - m_OriginalBounds.left;
	y = m_FinalBounds.top - m_OriginalBounds.top;

	for (int i=0; i<m_Elements.GetLength(); i++) {
		m_Elements[i]->m_FinalBounds.left += x;
		m_Elements[i]->m_FinalBounds.top += y;
	}
	ComputeBounds();
}
