/*
	This software module was originally developed by ENST in the course of
development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.
	This software module is an implementation of a part of one or more MPEG-4
Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1) 
standard. ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license 
to this software module or modifications thereof for use in hardware or software 
products claiming conformance to the MPEG-4 Systems (ISO/IEC 14496-1). 
Those intending to use this software module in hardware or software products are 
advised that its use may infringe existing patents.
	The original developer of this software module and his/her company, the 
subsequent editors and their companies, and ISO/IEC have no liability for use of 
this software module or modifications thereof in an implementation. Copyright is 
not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
	ENST retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using the code 
for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright    
notice must be included in all copies or derivative works.              
						Copyright  2002.
*/

///////////////////////////////////////////////////////////////////////////
// File:  FormProxy.h
//
// Author: Jean Le Feuvre ENST 2002
//
// Description:
//   Form node implementation class
//   
//
// Created: August ENST 2002
//
///////////////////////////////////////////////////////////////////////////


#ifndef FORM_PROXY_H
#define FORM_PROXY_H


#include "GroupNodeProxy.h"
#include "drawable.h"
#include "Constraint.h"


//Note on Form rendering: all children nodes once prerendered are stored in the specified groups
//a constraint is applied on the group bounds, then all children bounds are modified
//and finally the bounds of the group are re-updated (since elements can b present in 
//several groups

class FormGroup
{
public:

	//list of form children belonging to this group
	ZArray<ChildGroup *> m_Elements;

	//original and final bounds
	BRect m_OriginalBounds, m_FinalBounds;
	
	//compute original bounds
	void ComputeBounds();
	//update all elements bounds and refresh the group bounds
	void UpdateBounds();
};


class FormProxy : public GroupNodeProxy
{
public:
	FormProxy(MediaObject* pMO); 
	~FormProxy();
	
	DECLARE_CONSTRAINT(AL);
	DECLARE_CONSTRAINT(AH);
	DECLARE_CONSTRAINT(AR);
	DECLARE_CONSTRAINT(AT);
	DECLARE_CONSTRAINT(AV);
	DECLARE_CONSTRAINT(AB);

	DECLARE_CONSTRAINT(SH);
	DECLARE_CONSTRAINT(SHin);
	DECLARE_CONSTRAINT(SV);
	DECLARE_CONSTRAINT(SVin);

	//dynamic factory for constraints
	static void StoreFactory(int i,ConstraintsFactoryBase*pFactory) {TheConstraintsFactory[i]=pFactory;}

	// Return a pointer to the Form node that this node is the proxy for
	Form* GetForm() const { return (Form*)GroupNodeProxy::GetObject(); }

	// Functions inherited from the base classes
	void Render(Effects *effects);
	void InitFormContext();
	void ApplyConstraint(ZString zsConstraint, const int *GroupIndexes, int Length);

	int	 GetNumberOfFormChildren() {return m_Groups.GetLength()-1;}
	BOOL GetNextSetFromGroupsIndex(const int *&pSetOfNodes,int &iNumberOfObjects);	
	BOOL GetGroupsFieldByIndex(const int *& pGroups,int &iNumberOfChildren,int index);
	void CreateJointGroupsAndGroupsIndexList(const int * pGroups,int &iNumberOfChildren);

	FormGroup *GetGroup(int i) {
		assert(i < m_GroupList.GetLength());
		return m_GroupList[i];
	}

private:

	void BuildGroupList();
	void ResetGroupList();
	void RefreshGroupBounds();
	void CenterChildren();

	ZArray<FormGroup *> m_GroupList;
	
	static ConstraintsFactoryBase	 *TheConstraintsFactory[];

	DWORD m_lastUpdateTime;

	DrawableContext m_drawableContext;

};

#endif // Form_PROXY_H
