/*
	This software module was originally developed by ENST in the course of
development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.
	This software module is an implementation of a part of one or more MPEG-4
Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1) 
standard. ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license 
to this software module or modifications thereof for use in hardware or software 
products claiming conformance to the MPEG-4 Systems (ISO/IEC 14496-1). 
Those intending to use this software module in hardware or software products are 
advised that its use may infringe existing patents.
	The original developer of this software module and his/her company, the 
subsequent editors and their companies, and ISO/IEC have no liability for use of 
this software module or modifications thereof in an implementation. Copyright is 
not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
	ENST retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using the code 
for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright    
notice must be included in all copies or derivative works.              
						Copyright  2002.
*/

///////////////////////////////////////////////////////////////////////////
// File:  GeometryNode.h
//
// Author: Jean Le Feuvre - ENST 2002
//
///////////////////////////////////////////////////////////////////////////


#ifndef GEOMETRY_NODE_H
#define GEOMETRY_NODE_H



#include "Nodes-INIT.h"
#include "drawable.h"
#include "ShapeProxy.h"
#include "GroupNodeProxy.h"
#include "VisualPath.h"

class DrawableImage;
class VisualTexture;


class GeometryNode : public DrawableNode
{
public:
	GeometryNode(MediaObject* pMO) : DrawableNode(pMO) 
	{
		m_LastUpdateTime = 0;
	}

	virtual BOOL IsTransparent(DrawableContext * /*pContext*/) const { return TRUE; }

	//setup visual aspect properties
	void SetupDrawableContext(DrawableContext *pContext);

	//return TRUE if the given point is over the shape
	virtual BOOL IsPointOver(DrawableContext *pContext, float x, float y);

	MaterialKey *GetMaterialKey(DrawableContext *pContext) const;

	//returns the texture proxy if any
	VisualTexture *GetTexture(DrawableContext * pDrawableContext) const;


	//The method will store a copy of all of the effects parameters
	//locally in this node, and then register this node with the visual
	//renderer for user event processing. Texture info is updtaed here.
	//returns NULL if the node can't be drawned (error or transparent)
	DrawableContext *InitRenderContext(Effects *effects);

	//returns TRUE if the node shape/appearance has changed. 
	//geometry nodes override this function
	virtual BOOL HasChanged(DrawableContext *pContext);

	//this is needed for gradient which compute its bounds from the original 
	//bounds of the object
	virtual void GetOriginalBounds(BRect &bounds) = 0;
	
	//called when the geometry node is used as a clipping object. The clipper is in
	//world coordinates
	virtual void AddToClipper(VisualClipper *clipper, DrawableContext *pContext, BOOL bInside, BOOL bUseXOR = FALSE) = 0;


private:

	Appearance *GetAppearanceWithMaterial(DrawableContext *pContext) const;


	BOOL DrawGradient(DrawableContext *pContext, VisualPath *a_path);

protected:

	//setup all default drawing properties (linestyle, brush, color transparency
	//and cliper. returns FALSE if drawing should be skipped (cliped area empty)
	//a typical geometry node would call SetupDraw, DrawTexture and VisualSurface::DrawVisualPath
	BOOL SetupDraw(DrawableContext *pContext);

	//fills the path with the texture if any. returns FALSE if no texture, TRUE otherwise
	//you must setup the renderer before (cliper, line props, transform ...)
	//this function doesn't perform outline drawing, but will reset the brush settings according
	//to the Material2D.filled flag
	//if no path is given, use the class member path, otherwise use the specified one (this is
	//for indexedFaceSet texturing)
	BOOL DrawTexture(DrawableContext *pContext, VisualPath *a_path=NULL, BOOL noAntiAlias = FALSE);

	// Functions inherited from base classes
	BOOL CheckTransparentSkip(DrawableContext * pContext, BOOL SkipFill=FALSE);
	
	TextureTransform *GetTextureTransform(DrawableContext *pContext) const;

	//path
	VisualPath m_Path;
	//last geometry changes time, updated by subclasses (depends on the actual geometry node)
	DWORD m_LastUpdateTime;

	//last appearance change time (for transparency)
	DWORD m_LastAppearanceUpdateTime;

};


#endif // GEOMETRY_NODE_H
