/*
	This software module was originally developed by ENST in the course of
development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.
	This software module is an implementation of a part of one or more MPEG-4
Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1) 
standard. ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license 
to this software module or modifications thereof for use in hardware or software 
products claiming conformance to the MPEG-4 Systems (ISO/IEC 14496-1). 
Those intending to use this software module in hardware or software products are 
advised that its use may infringe existing patents.
	The original developer of this software module and his/her company, the 
subsequent editors and their companies, and ISO/IEC have no liability for use of 
this software module or modifications thereof in an implementation. Copyright is 
not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
	ENST retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using the code 
for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright    
notice must be included in all copies or derivative works.              
						Copyright  2002.
*/

///////////////////////////////////////////////////////////////////////////
// File:  GradientLinearProxy.cpp
//
// Author: Jean Le Feuvre, ENST 2002
//
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include "Presenter.h"
#include "GradientLinearProxy.h"
#include "VisualPath.h"
#include "Transform2DProxy.h"
#include "TransformMatrix2DProxy.h"


GradientLinearProxy *Presenter::InitGradientLinear(LinearGradient *pObj)
{
	return new GradientLinearProxy(pObj);
}

GradientLinearProxy::GradientLinearProxy(MediaObject* pMO)  : GenericGradient(pMO)
{
	m_LastUpdateTime = 0;
}

GradientLinearProxy::~GradientLinearProxy()
{
}

// MBS 031223: transform isn't in node specs???

void GradientLinearProxy::Update(DrawableContext * pContext)
{
/*	LinearGradient *pLin = GetGradient();
	DWORD now = GetObject()->GetUpdateTime();
	//nothing to do
	if (m_LastUpdateTime == now+1) {
		if (! pLin->transform) return;
		now = pLin->transform->GetUpdateTime();
		if (m_LastUpdateTime == now+1) return;
		m_LastUpdateTime = now+1;
	} else {
		m_LastUpdateTime = now+1;
	}

	pContext->m_bUpdateTexture = TRUE;
*/
	return;
}


void GradientLinearProxy::InitMatrix(BRect *parent_bounds, Matrix2D &matrix)
{

	LinearGradient *pLin = GetGradient();

	int numColors = pLin->key.GetLength();
	//need at least 2 keys
	if (numColors<2) return;

	//set gradient according to parent bounds
	Vec2f start, end;
	start = pLin->startPoint;
	//end = pLin->endpoint; // MBS 031223: enPoint is not a field of Gradient


	//add matrix if any
	/* MBS 031223: transform is not a field of Gradient???
	MediaObject *pObj = pLin->transform;
	if (pObj) {
		switch (pObj->GetNodeCode()) {
		case CODE_Transform2D:
			((Transform2DProxy*)pObj->GetRenderingObject())->GetMatrix(&matrix);
			break;
		case CODE_TransformMatrix2D:
			((TransformMatrix2DProxy*)pObj->GetRenderingObject())->GetMatrix(&matrix);
			break;
		default:
			break;
		}
	}
	*/
	//THIS IS GDIPLUS specific: no support for PAD in linear gradient, thus we compute an equivalent
	//gradient
	BOOL padded = ! (Bool) pLin->spreadMethod;
	
	start.x *= parent_bounds->width;
	end.x *= parent_bounds->width;
	start.y *= parent_bounds->height;
	end.y *= parent_bounds->height;
	float coef[6];
	matrix.GetElements(coef);
	coef[2] *= parent_bounds->width;
	coef[5] *= parent_bounds->height;
	coef[1] *= parent_bounds->width / parent_bounds->height;
	coef[3] *= parent_bounds->height / parent_bounds->width;
	matrix.Set(coef);

	if (padded) {
		float width = end.x - start.x;
		float height = end.y - start.y;
		start.x -= width;
		end.x += width;
		start.y -= height;
		end.y += height;
	}
	
	
	//translate it to the center of the bounds
	matrix.AddTranslation(parent_bounds->left, parent_bounds->top - parent_bounds->height);

	BOOL constOpacity = (pLin->opacity.GetLength()==1) ? TRUE : FALSE;
	float a;

	if (padded) {
		Color4 *colors = new Color4[numColors+2];
		float *blend = new float[numColors+2];

		//finally set interpolation points
		int k=0;
		float offset = 3;
		offset = 1 / offset;

		for (int i=0; i<numColors; i++) {
			a = constOpacity ? pLin->opacity[0] : pLin->opacity[i];
			colors[i+k].a = 255 * a;
			colors[i+k].r = 255 * pLin->keyValue[i].r;
			colors[i+k].g = 255 * pLin->keyValue[i].g;
			colors[i+k].b = 255 * pLin->keyValue[i].b;
		
			blend[i+k] = pLin->key[i]/3 + offset;

			if (!i) {
				blend[1] = blend[0];
				colors[1] = colors[0];
				k=1;
				blend[0] = 0;
			}
		}
		blend[numColors+1] = 1.0;
		colors[numColors+1] = colors[numColors];

		m_Gradient.SetLinearGradient(start.x, start.y, end.x, end.y, colors, blend, numColors+2);
		delete [] colors;
		delete [] blend;
	} else {
		Color4 *colors = new Color4[numColors];
		float *blend = new float[numColors];

		//finally set interpolation points
		for (int i=0; i<numColors; i++) {
			a = constOpacity ? pLin->opacity[0] : pLin->opacity[i];
			colors[i].a = 255 * a;
			colors[i].r = 255 * pLin->keyValue[i].r;
			colors[i].g = 255 * pLin->keyValue[i].g;
			colors[i].b = 255 * pLin->keyValue[i].b;
			blend[i] = pLin->key[i];
		}
		m_Gradient.SetLinearGradient(start.x, start.y, end.x, end.y, colors, blend, numColors);
		delete [] colors;
		delete [] blend;
	}


	switch (pLin->spreadMethod) {
	case 2:
		m_Gradient.SetRepeatMode(GRADIENT_TILE_BASE);
		break;
	case 1:
		m_Gradient.SetRepeatMode(GRADIENT_TILE_FLIP);
		break;
	case 0:
	default:
		m_Gradient.SetRepeatMode(GRADIENT_TILE_FLIP);
//		m_Gradient.SetRepeatMode(GRADIENT_TILE_PAD);
		break;
	}
}


