/*
	This software module was originally developed by ENST in the course of
development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.
	This software module is an implementation of a part of one or more MPEG-4
Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1) 
standard. ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license 
to this software module or modifications thereof for use in hardware or software 
products claiming conformance to the MPEG-4 Systems (ISO/IEC 14496-1). 
Those intending to use this software module in hardware or software products are 
advised that its use may infringe existing patents.
	The original developer of this software module and his/her company, the 
subsequent editors and their companies, and ISO/IEC have no liability for use of 
this software module or modifications thereof in an implementation. Copyright is 
not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
	ENST retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using the code 
for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright    
notice must be included in all copies or derivative works.              
						Copyright  2002.
*/

///////////////////////////////////////////////////////////////////////////
// File:  GradientRadialProxy.cpp
//
// Author: Jean Le Feuvre, ENST 2002
//
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include "Presenter.h"
#include "GradientRadialProxy.h"
#include "VisualPath.h"
#include "Transform2DProxy.h"
#include "TransformMatrix2DProxy.h"



#define SQRTTWO	1.4142135623730950488016887242097

GradientRadialProxy *Presenter::InitGradientRadial(RadialGradient *pObj)
{
	return new GradientRadialProxy(pObj);
}

GradientRadialProxy::GradientRadialProxy(MediaObject* pMO)  : GenericGradient(pMO)
{
	m_LastUpdateTime = 0;
}

GradientRadialProxy::~GradientRadialProxy()
{
}

void GradientRadialProxy::Update(DrawableContext * pContext)
{
	DWORD now = GetObject()->GetUpdateTime();
	//nothing to do
	if (m_LastUpdateTime == now+1) return;
	m_LastUpdateTime = now+1;
	
	RadialGradient *pRad = GetGradient();
	int numColors = pRad->key.GetLength();
	//need at least 2 keys
	if (numColors<2) return;

	pContext->m_bUpdateTexture = TRUE;


}

void GradientRadialProxy::InitMatrix(BRect *parent_bounds, Matrix2D &matrix)
{
	RadialGradient *pRad = GetGradient();
	float radius = pRad->radius;
	if (!radius) return;

	int numColors = pRad->key.GetLength();
	//need at least 2 keys
	if (numColors<2) return;

	
	//add matrix if any
	/* MBS 031223: transform is not a field of Gradient???
	MediaObject *pObj = pRad->transform;
	if (pObj) {
		switch (pObj->GetNodeCode()) {
		case CODE_Transform2D:
			((Transform2DProxy*)pObj->GetRenderingObject())->GetMatrix(&matrix);
			break;
		case CODE_TransformMatrix2D:
			((TransformMatrix2DProxy*)pObj->GetRenderingObject())->GetMatrix(&matrix);
			break;
		default:
			break;
		}
	}
	*/

	//max number of radius repeat to fill the shape
	int repeat = ceil(SQRTTWO / radius);
	if (!(repeat%2) && (pRad->spreadMethod == 1)) repeat += 1;
	else if (pRad->spreadMethod == 0) repeat = 1;


	Vec2f focal = pRad->focalPoint;
	Vec2f center = pRad->center;
	focal.x *= parent_bounds->width;
	focal.y *= parent_bounds->height;
	center.x *= parent_bounds->width;
	center.y *= parent_bounds->height;

	float scale_factor = 1;
	scale_factor /= repeat;

	VisualPath path;
	path.AddEllipse(2 * repeat*radius*parent_bounds->width, 2 * repeat*radius*parent_bounds->height);

	//the new circle should be translated so that the gradient focal for the original and 
	//the new circle match
	Matrix2D tmp;
	tmp.AddTranslation( (1 - repeat) * (focal.x-center.x), (1 - repeat) * (focal.y - center.y));
	path.Transform(&tmp);


	m_Gradient.SetVisualPath(&path);


	m_Gradient.SetCenterColor( (focal.x - center.x),
						 (focal.y - center.y), 
						pRad->keyValue[0], 
						1-pRad->opacity[0]);


	BOOL constOpacity = (pRad->opacity.GetLength()==1) ? TRUE : FALSE;
	float a;
		
	//repeat pattern
	if (pRad->spreadMethod == 2) {

		Color4 *colors = new Color4[numColors*repeat];
		float *blend = new float[numColors*repeat];

		for (int k=0; k<repeat; k++) {
			float offset = k;
			offset /= repeat;

			//finally set interpolation points
			for (int i=0; i<numColors; i++) {
				a = constOpacity ? pRad->opacity[0] : pRad->opacity[i];
				colors[k*numColors + i].a = 255 * a;
				colors[k*numColors + i].r = 255 * pRad->keyValue[i].r;
				colors[k*numColors + i].g = 255 * pRad->keyValue[i].g;
				colors[k*numColors + i].b = 255 * pRad->keyValue[i].b;
				blend[k*numColors + i] = offset + pRad->key[i] * scale_factor;
			}
		}

		m_Gradient.SetBlendColors(colors, blend, numColors * repeat);
		
		delete [] colors;
		delete [] blend;
	}
	//repeat and flip pattern
	else if (pRad->spreadMethod == 1) {
		Color4 *colors = new Color4[numColors*repeat];
		float *blend = new float[numColors*repeat];

		for (int k=0; k<repeat; k++) {
			float offset = k;
			offset /= repeat;

			//finally set interpolation points
			for (int i=0; i<numColors; i++) {
				int index = (k%2) ? (numColors-i-1) : i;

				a = constOpacity ? pRad->opacity[0] : pRad->opacity[index];
				colors[k*numColors + i].a = 255 * a;
				colors[k*numColors + i].r = 255 * pRad->keyValue[index].r;
				colors[k*numColors + i].g = 255 * pRad->keyValue[index].g;
				colors[k*numColors + i].b = 255 * pRad->keyValue[index].b;
				if (k%2) {
					blend[k*numColors + i] = offset + (1-pRad->key[index]) * scale_factor;
				} else {
					blend[k*numColors + i] = offset + pRad->key[i] * scale_factor;
				}
			}
		}

		m_Gradient.SetBlendColors(colors, blend, numColors * repeat);
		
		delete [] colors;
		delete [] blend;
	}
	//fill till end with edge color
	else {
		Color4 *colors = new Color4[numColors+1];
		float *blend = new float[numColors+1];

		//finally set interpolation points
		for (int i=0; i<numColors; i++) {
			a = constOpacity ? pRad->opacity[0] : pRad->opacity[i];
			colors[i].a = 255 * a;
			colors[i].r = 255 * pRad->keyValue[i].r;
			colors[i].g = 255 * pRad->keyValue[i].g;
			colors[i].b = 255 * pRad->keyValue[i].b;
			blend[i] = pRad->key[i] * scale_factor;
		}
		blend[numColors] = 1.0;
		colors[numColors] = colors[numColors-1];

		m_Gradient.SetBlendColors(colors, blend, numColors+1);
		
		delete [] colors;
		delete [] blend;
	}
	
	m_Gradient.SetRepeatMode(GRADIENT_TILE_PAD);

	//rescale the matrix to the parent system
	float coef[6];
	matrix.GetElements(coef);
	coef[2] *= parent_bounds->width;
	coef[5] *= parent_bounds->height;
	coef[1] *= parent_bounds->width / parent_bounds->height;
	coef[3] *= parent_bounds->height / parent_bounds->width;
	matrix.Set(coef);

	//Convert the matrix from texture coordinates to object coordinates
	tmp.SetIdentity();
	tmp.AddTranslation(center.x, center.y);
	tmp.AddTransform(&matrix);
	matrix.Set(tmp);

	tmp.SetIdentity();
	tmp.AddTranslation(-center.x, -center.y);
	matrix.AddTransform(&tmp);

	//and center the gradient
	matrix.AddTranslation(center.x + parent_bounds->left, parent_bounds->top + center.y - parent_bounds->height);

}


