/*
	This software module was originally developed by ENST in the course of
development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.
	This software module is an implementation of a part of one or more MPEG-4
Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1) 
standard. ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license 
to this software module or modifications thereof for use in hardware or software 
products claiming conformance to the MPEG-4 Systems (ISO/IEC 14496-1). 
Those intending to use this software module in hardware or software products are 
advised that its use may infringe existing patents.
	The original developer of this software module and his/her company, the 
subsequent editors and their companies, and ISO/IEC have no liability for use of 
this software module or modifications thereof in an implementation. Copyright is 
not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
	ENST retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using the code 
for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright    
notice must be included in all copies or derivative works.              
						Copyright  2002.
*/

///////////////////////////////////////////////////////////////////////////
// File:  GroupNodeProxy.h
//
// Author: Jean Le Feuvre ENST 2002
//
//
///////////////////////////////////////////////////////////////////////////


#ifndef GROUP_NODE_PROXY_H
#define GROUP_NODE_PROXY_H


#include "Traversable.h"
#include "Drawable.h"

class BindableStack;

//this is needed for form and layout were children can be complex elements, in order
//to calculate the bounds of the child
class ChildGroup
{
public:
	ZArray<DrawableContext *> m_ContextList;
	BRect m_OriginalBounds;
	BRect m_FinalBounds;

	void ComputeBounds();
	void UpdateAndDraw(Effects *parentEffects, BRect *parentClipper);

	BOOL m_bIsTextGroup;
	float m_ascent, m_descent;
};

class GroupNodeProxy : public Traversable
{
public:
	GroupNodeProxy(MediaObject* pMO) : Traversable(pMO) {}

	// Return a pointer to the GroupNode node that is the base class of
	// the node that this is the proxy for
	Traversable* GetChild(int n);
	
	// Functions inherited from the base class
	void AddToDrawableContextList(DrawableContext* pDrawableContext);	

	//used by layer2D for overriding stack behaviour
	virtual BindableStack *GetBackgroundStack() { return NULL; }
	virtual BindableStack *GetViewportStack() { return NULL; }

	// Methods to be called only by classes deriving from this class
protected:
	void RenderChildren(Effects *effects);

	//used by form and layout
	ChildGroup *StartChild();
	void ResetChildren();
	
	ZArray<ChildGroup *> m_Groups;

	void ResortSensors();
	//contains all sensors of this group (avoids double browsing of all children)
	ZArray<Traversable *> m_sensors;

	DWORD m_LastUpdateTime;
	
	GroupNodeProxy *m_pParent;
};


#endif // GROUP_NODE_PROXY_H
