/*
	This software module was originally developed by ENST in the course of
development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.
	This software module is an implementation of a part of one or more MPEG-4
Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1) 
standard. ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license 
to this software module or modifications thereof for use in hardware or software 
products claiming conformance to the MPEG-4 Systems (ISO/IEC 14496-1). 
Those intending to use this software module in hardware or software products are 
advised that its use may infringe existing patents.
	The original developer of this software module and his/her company, the 
subsequent editors and their companies, and ISO/IEC have no liability for use of 
this software module or modifications thereof in an implementation. Copyright is 
not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
	ENST retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using the code 
for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright    
notice must be included in all copies or derivative works.              
						Copyright  2002.
*/

///////////////////////////////////////////////////////////////////////////
// File:  ImageTextureProxy.cpp
//
// Author: Jean Le Feuvre ENST 2002
//
// Description:
//   proxy object for ImageTexture node.
//
// Created: august 2002.
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include "Nodes-INIT.h"
#include "ImageTextureProxy.h"
#include "Stream.h"
#include "Presenter.h"


ImageTextureProxy * Presenter::InitImageTexture(ImageTexture* pObj)
{
	return new ImageTextureProxy(pObj);
}

ImageTextureProxy::ImageTextureProxy(MediaObject* pMO) : DrawableImage(pMO)
{
	m_firstTime=TRUE;
	m_pStream = NULL;
}

ImageTextureProxy::~ImageTextureProxy()
{
}

void ImageTextureProxy::Update(DrawableContext *pContext)
{

	MediaStreamImp *pStream = GetImageTexture()->GetStream();
	if (!pStream) return;

	//close previous stream if changed
	if (m_pStream && (m_pStream != pStream)) {
		m_firstTime = TRUE;
		delete m_pTexture;
		m_pTexture = NULL;
		m_LastFrameTime = 0;
	}

	// The first time that this is called, the stream needs to be started.
	if (m_firstTime) {
		m_pStream = pStream;
		m_firstTime = FALSE;
		GetImageTexture()->OpenStream();
	}

	//call base class to update stream object (this will cretae the texture object if needed)
	DrawableImage::Update(pContext);
	
	//image not ready, invalidate scene
	if (!m_pTexture) {
		if (!pContext->m_hasStreamFinished) GetPresenter()->InvalidateScene(NULL);
		return;
	}

	BOOL rS, rT;
	GetTextureRepeat(rS, rT);
	m_pTexture->SetTextureRepeat(rS, rT);
}
