/*
	This software module was originally developed by ENST in the course of
development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.
	This software module is an implementation of a part of one or more MPEG-4
Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1) 
standard. ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license 
to this software module or modifications thereof for use in hardware or software 
products claiming conformance to the MPEG-4 Systems (ISO/IEC 14496-1). 
Those intending to use this software module in hardware or software products are 
advised that its use may infringe existing patents.
	The original developer of this software module and his/her company, the 
subsequent editors and their companies, and ISO/IEC have no liability for use of 
this software module or modifications thereof in an implementation. Copyright is 
not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
	ENST retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using the code 
for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright    
notice must be included in all copies or derivative works.              
						Copyright  2002.
*/

///////////////////////////////////////////////////////////////////////////
// File:  IndexedLineSet2DProxy.h
//
// Author: Jean Le Feuvre, ENST 2002
//
//
// Description:
//   The definition of the proxy object used by the presenter
//   to render the IndexedFaceFace2D node.
//
// Created: August, 2002.
//
//
///////////////////////////////////////////////////////////////////////////

#include "Defs.h"
#include "Nodes-INIT.h"
#include "IndexedFaceSet2DProxy.h"
#include "Presenter.h"
#include "ColorGradient.h"


IndexedFaceSet2DProxy* Presenter::InitIndexedFaceSet2D(IndexedFaceSet2D* pObj)
{
	return new IndexedFaceSet2DProxy(pObj);
}


void IndexedFaceSet2DProxy::Render(Effects *effects)
{
	if (!GetCoordinate2D()) return;

	//check for geometry change
	BOOL buildIt = effects->m_bShouldRebuild ? TRUE : FALSE;
	DWORD now = 1 + GetObject()->GetUpdateTime();
	if (now > m_LastUpdateTime) {
		m_LastUpdateTime = now;
		buildIt = TRUE;
	}
	//coordinates need to be checked too
	now = 1 + GetCoordinate2D()->GetUpdateTime();
	if (now > m_LastUpdateTime) {
		m_LastUpdateTime = now;
		buildIt = TRUE;
	}
	if (GetColor()) {
		now = 1 + GetColor()->GetUpdateTime();
		if (now > m_LastUpdateTime) {
			m_LastUpdateTime = now;
			buildIt = TRUE;
		}
	}

	if (buildIt) {
		BuildGraph();
		m_bHasChanged = TRUE;
	}

	DrawableContext *pContext = InitRenderContext(effects);
	if (!pContext) return;
	
	m_Path.GetBoundingRect(&pContext->m_boundingRect, &pContext->m_transform, pContext->gProps.lineWidth, pContext->gProps.lineScaled);	
	pContext->m_clippedBoundingRect = pContext->m_boundingRect.GetClipper();

	FinalizeRender(effects, pContext);
}


void IndexedFaceSet2DProxy::GetOriginalBounds(BRect &bounds)
{
	m_Path.GetBoundingRect(&bounds, NULL, 0);
}

void IndexedFaceSet2DProxy::AddToClipper(VisualClipper *pClipper, DrawableContext *pContext, BOOL bInside, BOOL bUseXOR)
{
	pClipper->AddPath(&m_Path, &pContext->m_transform, bInside, bUseXOR);
}

void IndexedFaceSet2DProxy::Draw(DrawableContext * pContext)
{
	//default setup (has to be overriden for line color)
	if (!SetupDraw(pContext)) return;

	Color *indexColor = GetColor();
	//case 1: if a color field is not specified, simply use the path 
	if (!indexColor) {
		DrawTexture(pContext);
		pContext->m_pVisualSurface->DrawVisualPath(&m_Path);
		return;
	}

	//store all default values
	Color3f LineCol = pContext->gProps.lineColor;
	Color3f colFill = pContext->gProps.emissiveColor;

	IndexedFaceSet2D *ifs = GetIndexFaceSet();
	Coordinate2D *coord = GetCoordinate2D();
	int colorLength = ifs->colorIndex.GetLength();
	int length = coord->point.GetLength();
	int i = 0;
	int CoordLength = ifs->coordIndex.GetLength();
	VisualPath path;

//	TextureCoordinate *textCoord = GetTextureCoordinate();


	Vec2f start = coord->point[0];
	Vec2f end;

	if (CoordLength == 0) {
		path.Reset();
		for (i = 0; i < length; i++) {
			if (i==length-1) {
				path.AddLine(coord->point[i].x, coord->point[i].y,
								start.x, start.y);
				end = start;
			} else {
				path.AddLine(coord->point[i].x, coord->point[i].y,
								coord->point[i+1].x, coord->point[i+1].y);
				end = coord->point[i+1];
			}
		}
		//use the indicated color if not filled or no line properties
		if (!pContext->gProps.filled || ! pContext->gProps.hasLineProperties) {
			LineCol = (colorLength > 0) ? indexColor->color[ifs->colorIndex[0]] : indexColor->color[0];
		}
		//get the color for the fill
		colFill = (colorLength > 0) ? indexColor->color[ifs->colorIndex[0]] : indexColor->color[0];

		if ((start.x != end.x) &&(start.y != end.y))
			path.CloseFigure();

		//override pen and brush
		pContext->m_pVisualSurface->SetLineStyle(LineCol,
												pContext->gProps.lineTransparency, 
												pContext->gProps.lineWidth, 
												pContext->gProps.lineCap, 
												pContext->gProps.lineJoin, 
												pContext->gProps.lineStyle, 
												pContext->gProps.miterLimit,
												pContext->gProps.lineScaled,
												pContext->gProps.lineInside);

		pContext->m_pVisualSurface->SetBrushStyle(pContext->gProps.filled, colFill, pContext->gProps.transparency);
		pContext->m_pVisualSurface->DrawVisualPath(&path);
		return;
	}


	int count = 0;

	if (!GetVertex()) {

		start = coord->point[ifs->coordIndex[0]];
		i = 0;
		count = 0;
		while (1) {
			path.Reset();
			while (ifs->coordIndex[i+1] != -1) {	
				if (i==CoordLength-1) {
					path.AddLine(coord->point[ifs->coordIndex[i]].x, coord->point[ifs->coordIndex[i]].y,
									start.x, start.y);

					end = coord->point[ifs->coordIndex[i]];
				} else {
					path.AddLine(coord->point[ifs->coordIndex[i]].x,
									coord->point[ifs->coordIndex[i]].y,
									coord->point[ifs->coordIndex[i + 1]].x,
									coord->point[ifs->coordIndex[i + 1]].y);

					end = coord->point[ifs->coordIndex[i + 1]];
				}
				i++;
				//end of face
				if (i >= CoordLength) break;
			}
			//close if needed
			if ((start.x != end.x) &&(start.y != end.y))
				path.CloseFigure();

			//use the indicated color if not filled or no line properties
			if (!pContext->gProps.filled || ! pContext->gProps.hasLineProperties) {
				LineCol = (colorLength > 0) ? indexColor->color[ifs->colorIndex[count]] : indexColor->color[count];
			}
			//get the color for the fill
			colFill = (colorLength > 0) ? indexColor->color[ifs->colorIndex[count]] : indexColor->color[count];

			//override pen and brush
			pContext->m_pVisualSurface->SetLineStyle(LineCol,
												pContext->gProps.lineTransparency, 
												pContext->gProps.lineWidth, 
												pContext->gProps.lineCap, 
												pContext->gProps.lineJoin, 
												pContext->gProps.lineStyle, 
												pContext->gProps.miterLimit,
												pContext->gProps.lineScaled,
												pContext->gProps.lineInside);
			pContext->m_pVisualSurface->SetBrushStyle(pContext->gProps.filled, colFill, pContext->gProps.transparency);
			pContext->m_pVisualSurface->DrawVisualPath(&path);

			//increment color
			count++;
			//goto next point
			i+=2;

			//done
			if (i >= CoordLength) break;
			start = coord->point[ifs->coordIndex[i]];
		}
		return;
	}

	//gradient fill
	//use a path gradient here
	ColorGradient gradient;

	start = coord->point[ifs->coordIndex[0]];
	int startColor = 0;
	while (1) {
		int numColors = 0;
		path.Reset();

		Vec2f center = start;
		while (ifs->coordIndex[i+1] != -1) {	
			if (i==CoordLength-1) {
				path.AddLine(coord->point[ifs->coordIndex[i]].x, coord->point[ifs->coordIndex[i]].y,
								start.x, start.y);

				end = start;
			} else {
				path.AddLine(coord->point[ifs->coordIndex[i]].x, coord->point[ifs->coordIndex[i]].y,
								coord->point[ifs->coordIndex[i + 1]].x, coord->point[ifs->coordIndex[i + 1]].y);

				end = coord->point[ifs->coordIndex[i+1]];
				center += end;
			}
			i++;
			numColors++;
			//end of face
			if (i >= CoordLength) break;
		}
		//close if needed
		if ((start.x != end.x) || (start.y != end.y)) {
			numColors ++;
			path.AddLine(end.x, end.y, start.x, start.y);
			center += end;
		}
		path.CloseFigure();

		gradient.SetVisualPath(&path);
		float r, g, b;
		r = g = b = 0.0;

		Color3f *colors = new Color3f[numColors+1];
		for (int j=0; j<numColors; j++) {
			if (colorLength>j) {
				colors[j] = indexColor->color[ifs->colorIndex[startColor+j]];
			} else {
				colors[j] = indexColor->color[ifs->coordIndex[startColor+j]];
			}
			r+= colors[j].r;
			g+= colors[j].g;
			b+= colors[j].b;
		}
		if (colorLength>0) {
			colors[numColors] = indexColor->color[ifs->colorIndex[startColor]];
		} else {
			colors[numColors] = indexColor->color[ifs->coordIndex[startColor]];
		}
		r+= colors[numColors].r;
		g+= colors[numColors].g;
		b+= colors[numColors].b;
		r /=numColors;
		g /=numColors;
		b /=numColors;
		center.x /=	numColors;
		center.y /=	numColors;
		
		gradient.SetGradientColors(colors, numColors+1, pContext->gProps.transparency);
		delete [] colors;

		//setup gradient center and its color
		Color3f cent_col(r, g, b);
		gradient.SetCenterColor(center.x, center.y, cent_col, pContext->gProps.transparency);

		//don't forget to scale the gradient so that it fits the path
		gradient.SetTransform(&pContext->m_transform);

		//if not filled no line style is used according to the spec
		pContext->m_pVisualSurface->SetLineStyle(LineCol,
												pContext->gProps.lineTransparency, 
												pContext->gProps.lineWidth, 
												pContext->gProps.lineCap, 
												pContext->gProps.lineJoin, 
												pContext->gProps.filled ? pContext->gProps.lineStyle : 0, 
												pContext->gProps.miterLimit,
												pContext->gProps.lineScaled,
												pContext->gProps.lineInside);
		pContext->m_pVisualSurface->SetBrushStyle(pContext->gProps.filled, colFill, pContext->gProps.transparency);
		pContext->m_pVisualSurface->DrawVisualPath(&path, FALSE, &gradient);

		gradient.Reset();

		//goto next point
		i+=2;
		startColor += numColors + 1;	
		//done
		if (i >= CoordLength) break;
		start = coord->point[ifs->coordIndex[i]];
	}
}

void IndexedFaceSet2DProxy::BuildGraph()
{
	int i;
	m_Path.Reset();
	int CoordLength = GetIndexFaceSet()->coordIndex.GetLength();
	Coordinate2D *coord = GetCoordinate2D();
	IndexedFaceSet2D *ifs = GetIndexFaceSet();

	int length = coord->point.GetLength();

	if (CoordLength > 0) {
		for (i=0; i < CoordLength-1; i++) {
			//skip end of polyline
			if (ifs->coordIndex[i] == -1) {
				m_Path.CloseFigure();
				m_Path.StartNewFigure();
				continue;
			}
			if (ifs->coordIndex[i+1] == -1) continue;

			m_Path.AddLine(coord->point[ifs->coordIndex[i]].x, coord->point[ifs->coordIndex[i]].y,
							coord->point[ifs->coordIndex[i + 1]].x, coord->point[ifs->coordIndex[i + 1]].y);
		}
		m_Path.CloseFigure();
	} else {
		for (i = 0; i < length - 1; i++) {
			m_Path.AddLine(coord->point[i].x, coord->point[i].y, coord->point[i + 1].x, coord->point[i + 1].y);
		}
		m_Path.CloseFigure();
	}
}

