/*
	This software module was originally developed by ENST in the course of
development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.
	This software module is an implementation of a part of one or more MPEG-4
Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1) 
standard. ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license 
to this software module or modifications thereof for use in hardware or software 
products claiming conformance to the MPEG-4 Systems (ISO/IEC 14496-1). 
Those intending to use this software module in hardware or software products are 
advised that its use may infringe existing patents.
	The original developer of this software module and his/her company, the 
subsequent editors and their companies, and ISO/IEC have no liability for use of 
this software module or modifications thereof in an implementation. Copyright is 
not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
	ENST retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using the code 
for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright    
notice must be included in all copies or derivative works.              
						Copyright  2002.
*/

///////////////////////////////////////////////////////////////////////////
// File:  IndexedLineSet2DProxy.h
//
// Author: Jean Le Feuvre, ENST 2002
//
//
// Description:
//   The definition of the proxy object used by the presenter
//   to render the IndexedLineFace2D node.
//
// Created: August, 2002.
//
//
///////////////////////////////////////////////////////////////////////////


#ifndef INDEXEDFACESET_PROXY_H
#define INDEXEDFACESET_PROXY_H


#include "GeometryNode.h"

class IndexedFaceSet2DProxy : public GeometryNode
{
public:
	IndexedFaceSet2DProxy(MediaObject* pMO) : GeometryNode(pMO) {}

	// Return a pointer to the IndexedLineSet2D node that this node is the proxy for
	IndexedFaceSet2D* GetIndexFaceSet() const { return (IndexedFaceSet2D*)GetObject(); }

	// Return a pointer to the Color object belonging to the
	// node for which this node is the proxy
	Color* GetColor() const
	{ return (Color*)(MediaObject*)(GetIndexFaceSet()->color); }

	TextureCoordinate* GetTextureCoordinate() const
	{ return (TextureCoordinate*)(MediaObject*)(GetIndexFaceSet()->texCoord); }

	// Return a pointer to the Properties2D object belonging to the
	// Appearance node that this node is the proxy for.
	Coordinate2D* GetCoordinate2D() const
	{ return (Coordinate2D*)(GetIndexFaceSet()->coord); }

	// Return the colorPerVertex value
	BOOL GetVertex() const
	{
		return (BOOL)(Bool)(GetIndexFaceSet()->colorPerVertex);
	}

	// Functions inherited from the base class
	virtual void Render(Effects *effects);
	virtual void Draw(DrawableContext * pDrawableContext);

	virtual void GetOriginalBounds(BRect &bounds);


	virtual void AddToClipper(VisualClipper *pClipper, DrawableContext *pContext, BOOL bInside, BOOL bUseXOR = FALSE);
	
	// Private implementation details
private:
	void BuildGraph();
};

#endif // INDEXEDFACESET_PROXY_H
