/*
	This software module was originally developed by ENST in the course of
development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.
	This software module is an implementation of a part of one or more MPEG-4
Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1) 
standard. ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license 
to this software module or modifications thereof for use in hardware or software 
products claiming conformance to the MPEG-4 Systems (ISO/IEC 14496-1). 
Those intending to use this software module in hardware or software products are 
advised that its use may infringe existing patents.
	The original developer of this software module and his/her company, the 
subsequent editors and their companies, and ISO/IEC have no liability for use of 
this software module or modifications thereof in an implementation. Copyright is 
not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
	ENST retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using the code 
for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright    
notice must be included in all copies or derivative works.              
						Copyright  2002.
*/

///////////////////////////////////////////////////////////////////////////
// File:  IndexedLineSet2DProxy.cpp
//
// Author: Jean Le Feuvre, ENST 2002
//
// Description:
//   rendering of the IndexedLineSet2D node.
//
// Created: August, 2002
//
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include "Nodes-INIT.h"
#include "IndexedLineSet2DProxy.h"
#include "Presenter.h"

IndexedLineSet2DProxy* Presenter::InitIndexedLineSet2D(IndexedLineSet2D* pObj)
{
	return new IndexedLineSet2DProxy(pObj);
}


void IndexedLineSet2DProxy::Render(Effects *effects)
{
	if (!GetCoordinate2D()) return;
	DrawableContext *pContext = InitRenderContext(effects);
	if (!pContext) return;

	//check for geometry change
	BOOL buildIt = effects->m_bShouldRebuild ? TRUE : FALSE;
	DWORD now = 1 + GetObject()->GetUpdateTime();
	if (now > m_LastUpdateTime) {
		m_LastUpdateTime = now;
		buildIt = TRUE;
	}
	//coordinates need to be checked too
	now = 1 + GetCoordinate2D()->GetUpdateTime();
	if (now > m_LastUpdateTime) {
		m_LastUpdateTime = now;
		buildIt = TRUE;
	}
	if (GetColor()) {
		now = 1 + GetColor()->GetUpdateTime();
		if (now > m_LastUpdateTime) {
			m_LastUpdateTime = now;
			buildIt = TRUE;
		}
	}
	if (buildIt) {
		BuildGraph();
		m_bHasChanged = TRUE;
	}

	m_Path.GetBoundingRect(&pContext->m_boundingRect, &pContext->m_transform, pContext->gProps.lineWidth, pContext->gProps.lineScaled);	
	pContext->m_clippedBoundingRect = pContext->m_boundingRect.GetClipper();

	FinalizeRender(effects, pContext);
}


void IndexedLineSet2DProxy::Draw(DrawableContext *pContext)
{
	VisualPath path;
	ColorGradient gradient;

	if (!SetupDraw(pContext)) return;

	//index line set are NEVER filled
	pContext->m_pVisualSurface->SetBrushStyle(0, Color3f(0.0), 0.0);

	int i = 0;

	//store all default values
	Color3f LineCol = pContext->gProps.lineColor;
	Color3f colFill = pContext->gProps.emissiveColor;

	Color *indexColor = GetColor();

	IndexedLineSet2D *ils = GetIndexLineSet();
	Coordinate2D *coord = GetCoordinate2D();
	int CoordLength = ils->coordIndex.GetLength();
	int colorLength = ils->colorIndex.GetLength();
	int length = coord->point.GetLength();

	//2 cases: if a color field is not specified, simply use the path 
	if (!indexColor) {
		pContext->m_pVisualSurface->DrawVisualPath(&m_Path);
		return;
	}
	
	if (CoordLength == 0) {
		//draw normally (no coordIndex), and apply color as if perVertex is FALSE
		//(not well specified in VRML / MPEG4 specs)
		for (i = 0; i < length - 1; i++) {

			LineCol = (colorLength > 0) ? indexColor->color[ils->colorIndex[i]] : indexColor->color[i];
			pContext->m_pVisualSurface->SetLineStyle(LineCol, 
												pContext->gProps.lineTransparency, 
												pContext->gProps.lineWidth, 
												pContext->gProps.lineCap, 
												pContext->gProps.lineJoin, 
												pContext->gProps.lineStyle, 
												pContext->gProps.miterLimit, 
												pContext->gProps.lineScaled);

			path.Reset();
			path.AddLine(coord->point[i].x,
							coord->point[i].y,
							coord->point[i+1].x,
							coord->point[i+1].y);

			pContext->m_pVisualSurface->DrawVisualPath(&path);
		}
		return;
	}

	//draw line by line
	if (!GetVertex()) {
		int count = 0;
		for (i=0; i < CoordLength-1; i++) {
			//skip end of polyline
			if (ils->coordIndex[i]==-1) {
				m_Path.StartNewFigure();
				count++;
				continue;
			}
			if (ils->coordIndex[i+1]==-1) continue;


			int	ind = (colorLength > 0) ? ils->colorIndex[count] : count;
			if (ind>=indexColor->color.GetLength()) ind = indexColor->color.GetLength();

			LineCol = indexColor->color[ind];
			pContext->m_pVisualSurface->SetLineStyle(LineCol, 
												pContext->gProps.lineTransparency, 
												pContext->gProps.lineWidth, 
												pContext->gProps.lineCap, 
												pContext->gProps.lineJoin, 
												pContext->gProps.lineStyle, 
												pContext->gProps.miterLimit, 
												pContext->gProps.lineScaled);

			path.Reset();
			path.AddLine(coord->point[ils->coordIndex[i]].x,
							coord->point[ils->coordIndex[i]].y,
							coord->point[ils->coordIndex[i + 1]].x,
							coord->point[ils->coordIndex[i + 1]].y);

			pContext->m_pVisualSurface->DrawVisualPath(&path);
		}
		return;
	}

	//use a path gradient here
	Vec2f start, end;
	start = coord->point[ils->coordIndex[0]];
	int startColor = 0;
	while (1) {
		int numColors = 0;
		path.Reset();
		while (ils->coordIndex[i+1] != -1) {
			//end of lines
			if (i+1 >= CoordLength) break;
			path.AddLine(coord->point[ils->coordIndex[i]].x, coord->point[ils->coordIndex[i]].y,
							coord->point[ils->coordIndex[i + 1]].x, coord->point[ils->coordIndex[i + 1]].y);

			end = coord->point[ils->coordIndex[i+1]];
			i++;
			numColors++;
			//end of polyline
			if (i >= CoordLength) break;
		}

		numColors++;

		if (numColors==2) {
			if (colorLength>0) {
				gradient.SetLinearGradient(start.x, start.y, end.x, end.y, 
								indexColor->color[ils->colorIndex[startColor]],
								indexColor->color[ils->colorIndex[startColor+1]], 
								pContext->gProps.lineTransparency);
			} else {
				gradient.SetLinearGradient(start.x, start.y, end.x, end.y, 
					indexColor->color[ils->coordIndex[startColor]],
					indexColor->color[ils->coordIndex[startColor+1]], 
					pContext->gProps.lineTransparency);
			}
		} else {
			gradient.SetVisualPath(&path);
			Color3f *colors = new Color3f[numColors];
			for (int j=0; j<numColors; j++) {
				if (colorLength>0) {
					colors[j] = indexColor->color[ils->colorIndex[startColor+j]];
				} else {
					colors[j] = indexColor->color[ils->coordIndex[startColor+j]];
				}
			}
			gradient.SetGradientColors(colors, numColors, pContext->gProps.lineTransparency);
			delete [] colors;

		}
		pContext->m_pVisualSurface->DrawVisualPath(&path, FALSE, &gradient);

		gradient.Reset();

		//goto next point
		i += 2;
		startColor += numColors + 1;
		//done
		if (i >= CoordLength) break;
		start = coord->point[ils->coordIndex[i]];
	}
}

void IndexedLineSet2DProxy::BuildGraph()
{
	int i;
	m_Path.Reset();
	int CoordLength = GetIndexLineSet()->coordIndex.GetLength();
	Coordinate2D *coord = GetCoordinate2D();
	IndexedLineSet2D *ils = GetIndexLineSet();

	int length = coord->point.GetLength();

	if (CoordLength > 0) {
		for (i=0; i < CoordLength-1; i++) {
			//skip end of polyline
			if (ils->coordIndex[i] == -1) {
				m_Path.StartNewFigure();
				continue;
			}
			if (ils->coordIndex[i+1] == -1) continue;

			m_Path.AddLine(coord->point[ils->coordIndex[i]].x,
							coord->point[ils->coordIndex[i]].y,
							coord->point[ils->coordIndex[i + 1]].x,
							coord->point[ils->coordIndex[i + 1]].y);
		}
	} else {
		for (i = 0; i < length - 1; i++) {
			m_Path.AddLine(coord->point[i].x, 
							coord->point[i].y,
							coord->point[i + 1].x,
							coord->point[i + 1].y);
		}
	}
}

