/*
	This software module was originally developed by ENST in the course of
development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.
	This software module is an implementation of a part of one or more MPEG-4
Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1) 
standard. ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license 
to this software module or modifications thereof for use in hardware or software 
products claiming conformance to the MPEG-4 Systems (ISO/IEC 14496-1). 
Those intending to use this software module in hardware or software products are 
advised that its use may infringe existing patents.
	The original developer of this software module and his/her company, the 
subsequent editors and their companies, and ISO/IEC have no liability for use of 
this software module or modifications thereof in an implementation. Copyright is 
not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
	ENST retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using the code 
for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright    
notice must be included in all copies or derivative works.              
						Copyright  2002.
*/

///////////////////////////////////////////////////////////////////////////
// File:  Matrix2D.h
//
// Author: Jean Le Feuvre
// Description:
//   2-D transform matrix wrapper for GDIplus.
//
// Created: August, 2002.
//
//	Notes:
//		- all angles are in radians and express roations in anticlockwise direction
//	
///////////////////////////////////////////////////////////////////////////


#ifndef MATRIX_2D_H
#define MATRIX_2D_H

#define PI		3.14159265358979323846		/* pi */


#include "OsmoseGdiplus.h"


class Matrix2D
{
	//
	// Public functions
	//
public:
			
	Matrix2D();
	Matrix2D(const Matrix2D& mat);

	// Set the matrix to the identity matrix
	void SetIdentity();

	//set matrix coefficients. mN is defined in Matrix2D node
	void Set(float m1, float m2, float m3, float m4, float m5, float m6);
	void Set(float *coefs);
	void Set(const Matrix2D& mat);

	// Appends the given translation to the matrix
	void AddTranslation(float dx, float dy);

	// appends the given scale to the matrix
	void AddScale(float sx, float sy, float orientation, float center_x=0, float center_y=0);

	// Appends the given rotation to the matrix 
	void AddRotation(float angle, float center_x=0, float center_y=0);

	// Appends the given transformation to the matrix 
	void AddTransform(Matrix2D *trans);

	void Apply(float& x, float& y) const;
	void Apply(long& x, long& y) const;
	void Apply(int& x, int& y) const;

	BOOL ReverseApply(float& x, float& y);

	BOOL HasRotations();
	//get the 6 2D coefs
	void GetElements(float *m);

	BOOL Inverse();

	// Test for equality
	friend int operator== (const Matrix2D& mat1,
						   const Matrix2D& mat2);

	// Test for inequality
	friend int operator!= (const Matrix2D& mat1,
						   const Matrix2D& mat2);

public:
	Gdiplus::Matrix m_Matrix;
};


#endif // MATRIX_2D_H

