//This software module was originally developed by CSELT in the course of   //
//development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.             //
//This software module is an implementation of a part of one or more MPEG-4 //
//Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems        //
//(ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 Systems     //
//(ISO/IEC 14496-1) free license to this software module or modifications   //
//thereof for use in hardware or software products claiming conformance to  //
//the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use this software//
//module in hardware or software products are advised that its use may      //
//infringe existing patents.                                                //
//The original developer of this software module and his/her company, the   //
//subsequent editors and their companies, and ISO/IEC have no liability     //
//for use of this software module or modifications thereof in an            //
//implementation. Copyright is not released for non MPEG-4 Systems          //
//(ISO/IEC 14496-1) conforming products. CSELT S.pA. retains full right     //
//to use the code for his/her own purpose, assign or donate the code to a   //
//third party and to inhibit third parties from using the code for non      //
//MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright      //
//notice must be included in all copies or derivative works.                //
//Copyright  1999.                                                         //
//This work has been partly developed within the framework of the ESPRIT    //
//European Research Project  "MPEG-4 PC".                                   //
//////////////////////////////////////////////////////////////////////////////
//
// File:  MatteTextureProxy.h
//
// Author: marc Patoureaux marc@envivio.com
//
// Modified:
//			Jean Le Feuvre ENST 2002

// Description:
//   The implementation of the proxy object used by the presenter
//   to render the MatteTexture node.
//
// Created: 13 jully 2001.
//
//
///////////////////////////////////////////////////////////////////////////


#ifndef MatteTexture_PROXY_H
#define MatteTexture_PROXY_H


#include "DrawableImage.h"

#define NBOTOKEN 14

class MatteTextureProxy : public DrawableImage
{
public:
	MatteTextureProxy(MediaObject* pMO);
	~MatteTextureProxy();
	 void DeleteObject();


	// Return a pointer to the MatteTexture node that this node is the proxy for
	MatteTexture* GetMatteTexture() const { return (MatteTexture*)GetObject(); }

	// Functions inherited from the base classes
	void Update(DrawableContext* pContext);
    void Release();

protected:
	//to refine for MatteTexture
	virtual BOOL ShouldBuildBrush() {return TRUE; }

private:
    // this is related to NBOTOKEN
    enum oToken { // for operation token
        INVALID,
        OFFSET,
        SCALE,
        BLUR,
        BIAS,
        COLOR_MATRIX,
        REPLACE_ALPHA,
        MULTIPLY_ALPHA,
        CROSS_FADE,
        BLEND,
        ADD,
        A,
        B,
        REVEAL} m_token;
    void parseOperation(LPCSTR operation);

	// Private implementation details

    void ApplyBIAS();
	void ApplySCALE();
    void ApplyBLUR();
    void ApplyCROSS_FADE();
    void ApplyREPLACE_ALPHA();
    void ApplyREVEAL();

	//setup at each prerender step
    DrawableImage *surface_a, *surface_b, *surface_alpha;
	BOOL m_bOverwrite;
	DWORD m_LastUpdateTime;
};


#endif // MatteTexture_PROXY_H
