/*
	This software module was originally developed by ENST in the course of
development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.
	This software module is an implementation of a part of one or more MPEG-4
Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1) 
standard. ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license 
to this software module or modifications thereof for use in hardware or software 
products claiming conformance to the MPEG-4 Systems (ISO/IEC 14496-1). 
Those intending to use this software module in hardware or software products are 
advised that its use may infringe existing patents.
	The original developer of this software module and his/her company, the 
subsequent editors and their companies, and ISO/IEC have no liability for use of 
this software module or modifications thereof in an implementation. Copyright is 
not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
	ENST retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using the code 
for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright    
notice must be included in all copies or derivative works.              
						Copyright  2002.
*/

///////////////////////////////////////////////////////////////////////////
// File:  Nodes-INIT.h
//
// Author: Jean Le Feuvre ENST 2002
//
// Description:
//   all proxy construction from node construction for all nodes integrated in OSMOSE
//
///////////////////////////////////////////////////////////////////////////


#ifndef NODES_INIT_H
#define NODES_INIT_H


#define eventIn		 public:
#define exposedField public:
#define eventOut	 public:
#define field		 public:


#include "StreamCo.h"
#include "Nodes.h"
#include "conditional.h"
#include "bifs.h"
#include "Root.h"
#include "inline.h"
#include "MediaControl.h"
#include "MediaSensor.h"
#include "ScriptImp.h"
#include "Anim.h"

#include "../FlexTime/TemporalTransformImpFlex.h" //PAB+:
#include "../FlexTime/TemporalGroupImpFlex.h"  //PAB+:

class AnchorINIT : public Anchor
{
    RenderingObject *CreateRenderingObject (MediaObject *);
};

class AppearanceINIT : public Appearance
{
    RenderingObject *CreateRenderingObject (MediaObject *);
};


class AudioBufferINIT : public AudioBuffer
{
	//all timed with startTime and stopTime nodes override OnUpdate (see in appropriated proxies)
	void OnUpdate();
	RenderingObject *CreateRenderingObject (MediaObject *);
};

class AudioClipINIT : public AudioClip
{
	//all timed with startTime and stopTime nodes override OnUpdate (see in appropriated proxies)
	void OnUpdate();
	RenderingObject *CreateRenderingObject (MediaObject *);
};

class AudioSourceINIT : public AudioSource
{
	//all timed with startTime and stopTime nodes override OnUpdate (see in appropriated proxies)
	void OnUpdate();
	RenderingObject *CreateRenderingObject (MediaObject *);
};

class Background2DINIT : public Background2D
{
	RenderingObject *CreateRenderingObject (MediaObject *);
};

class BitmapINIT : public Bitmap
{
	RenderingObject *CreateRenderingObject (MediaObject *);
};

class CircleINIT : public Circle
{
	RenderingObject *CreateRenderingObject (MediaObject *);
};

class ColorINIT : public Color
{
	RenderingObject *CreateRenderingObject (MediaObject *);
};

class ColorInterpolatorINIT : public ColorInterpolator
{
	RenderingObject *CreateRenderingObject (MediaObject *);};

class Coordinate2DINIT : public Coordinate2D
{
	RenderingObject *CreateRenderingObject (MediaObject *);
};

class CoordinateInterpolator2DINIT : public CoordinateInterpolator2D
{
	RenderingObject *CreateRenderingObject (MediaObject *);
};

class Curve2DINIT : public Curve2D
{
	RenderingObject *CreateRenderingObject (MediaObject *);
};

class DiscSensorINIT : public DiscSensor
{
	RenderingObject *CreateRenderingObject (MediaObject *);
};

// MBS 031223: changed EllipseNode to Ellipse according to specs.
class EllipseINIT : public Ellipse
{
	RenderingObject *CreateRenderingObject (MediaObject *);
};

class FontStyleINIT : public FontStyle
{
	RenderingObject *CreateRenderingObject (MediaObject *);
};

class FormINIT : public Form
{
	RenderingObject *CreateRenderingObject (MediaObject *);
};

class GroupINIT : public Group
{
	RenderingObject *CreateRenderingObject (MediaObject *);
};

class InlineINIT : public InlineImp
{
	RenderingObject *CreateRenderingObject (MediaObject *);
};

class AnimationStreamINIT : public AnimationStreamImp
{
public:
	//all timed with startTime and stopTime nodes override OnUpdate (see in appropriated proxies)
	void OnUpdate();
    RenderingObject *CreateRenderingObject (MediaObject *);
};


class IndexedLineSet2DINIT : public IndexedLineSet2D
{
	RenderingObject *CreateRenderingObject (MediaObject *);
};

class IndexedFaceSet2DINIT : public IndexedFaceSet2D
{
	RenderingObject *CreateRenderingObject (MediaObject *);
};

class LayoutINIT : public Layout
{
	RenderingObject *CreateRenderingObject (MediaObject *);
};

class Layer2DINIT : public Layer2D
{
	RenderingObject *CreateRenderingObject (MediaObject *);
};

class LinePropertiesINIT : public LineProperties
{
	RenderingObject *CreateRenderingObject (MediaObject *);
};

class Material2DINIT : public Material2D
{
	RenderingObject *CreateRenderingObject (MediaObject *);
};

class MaterialKeyINIT : public MaterialKey
{
	RenderingObject *CreateRenderingObject (MediaObject *);
};

class MediaControlINIT : public MediaControlImp
{
	RenderingObject *CreateRenderingObject (MediaObject *);
};

class MediaSensorINIT : public MediaSensorImp
{
	RenderingObject *CreateRenderingObject (MediaObject *);
};

class MovieTextureINIT : public MovieTexture
{
public:
	//all timed with startTime and stopTime nodes override OnUpdate (see in appropriated proxies)
	void OnUpdate();
	RenderingObject *CreateRenderingObject (MediaObject *);
};



class SynthesizedTextureINIT : public SynthesizedTexture
{
	RenderingObject *CreateRenderingObject (MediaObject *);
};



class MatteTextureINIT : public MatteTexture
{
	RenderingObject *CreateRenderingObject (MediaObject *);
};

class OrderedGroupINIT : public OrderedGroup
{
	RenderingObject *CreateRenderingObject (MediaObject *);
};

class PixelTextureINIT : public PixelTexture
{
	RenderingObject *CreateRenderingObject (MediaObject *);
};

class ValuatorINIT : public Valuator
{
	RenderingObject *CreateRenderingObject (MediaObject *);
};

class PlaneSensor2DINIT : public PlaneSensor2D
{
	RenderingObject *CreateRenderingObject (MediaObject *);
};

class PointSet2DINIT : public PointSet2D
{
	RenderingObject *CreateRenderingObject (MediaObject *);
};

class PositionInterpolator2DINIT : public PositionInterpolator2D
{
	RenderingObject *CreateRenderingObject (MediaObject *);
};

class ProximitySensor2DINIT : public ProximitySensor2D
{
	RenderingObject *CreateRenderingObject (MediaObject *);
};

class RectangleINIT : public Rectangle
{
	RenderingObject *CreateRenderingObject (MediaObject *);
};

class ScalarInterpolatorINIT : public ScalarInterpolator
{
	RenderingObject *CreateRenderingObject (MediaObject *);
};

class ShapeINIT : public Shape
{
	RenderingObject *CreateRenderingObject (MediaObject *);
};

class Sound2DINIT : public Sound2D
{
	RenderingObject *CreateRenderingObject (MediaObject *);
};

class ScriptINIT: public ScriptImp
{
	RenderingObject *CreateRenderingObject (MediaObject *);
};

class SwitchINIT : public Switch
{
	RenderingObject *CreateRenderingObject (MediaObject *);
};

class TextINIT : public Text
{
	RenderingObject *CreateRenderingObject (MediaObject *);
};

class ImageTextureINIT : public ImageTexture
{
	RenderingObject *CreateRenderingObject (MediaObject *);
};

class TermCapINIT : public TermCap
{
	RenderingObject *CreateRenderingObject (MediaObject *);
};

class TimeSensorINIT : public TimeSensor
{
public:
	//all timed with startTime and stopTime nodes override OnUpdate (see in appropriated proxies)
	void OnUpdate();
	RenderingObject *CreateRenderingObject (MediaObject *);
};

class TouchSensorINIT : public TouchSensor
{
	RenderingObject *CreateRenderingObject (MediaObject *);
};

class Transform2DINIT : public Transform2D
{
	RenderingObject *CreateRenderingObject (MediaObject *);
};

class ConditionalINIT : public Conditional
{
	RenderingObject *CreateRenderingObject (MediaObject *);
};

class WorldInfoINIT : public WorldInfo
{
	RenderingObject *CreateRenderingObject (MediaObject *);
};

class TemporalTransformINIT : public TemporalTransformImpFlex
{
	RenderingObject *CreateRenderingObject (MediaObject *);
};

class TemporalGroupINIT : public TemporalGroupImpFlex
{
	RenderingObject *CreateRenderingObject (MediaObject *);
};

class TextureTransformINIT: public TextureTransform
{
	RenderingObject *CreateRenderingObject (MediaObject *);
};

class CompositeTexture2DINIT : public CompositeTexture2D
{
	RenderingObject *CreateRenderingObject (MediaObject *);
};

class GradientRadialINIT : public RadialGradient
{
	RenderingObject *CreateRenderingObject (MediaObject *);
};

class GradientLinearINIT : public LinearGradient
{
	RenderingObject *CreateRenderingObject (MediaObject *);
};


class AdvancedFontStyleINIT : public XFontStyle
{
	RenderingObject *CreateRenderingObject (MediaObject *);
};

class TransformMatrix2DINIT : public TransformMatrix2D
{
	RenderingObject *CreateRenderingObject (MediaObject *);
};

class XLinePropertiesINIT : public XLineProperties
{
	RenderingObject *CreateRenderingObject (MediaObject *);
};

class ViewportINIT : public Viewport
{
	void OnUpdate();
	RenderingObject *CreateRenderingObject (MediaObject *);
};

class Clipper2DINIT : public Clipper2D
{
	RenderingObject *CreateRenderingObject (MediaObject *);
};

class XCurve2DINIT : public XCurve2D
{
	RenderingObject *CreateRenderingObject (MediaObject *);
};

class ColorTransformINIT : public ColorTransform
{
	RenderingObject *CreateRenderingObject (MediaObject *);
};



#endif // NODES_INIT_H
