/*
	This software module was originally developed by ENST in the course of
development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.
	This software module is an implementation of a part of one or more MPEG-4
Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1) 
standard. ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license 
to this software module or modifications thereof for use in hardware or software 
products claiming conformance to the MPEG-4 Systems (ISO/IEC 14496-1). 
Those intending to use this software module in hardware or software products are 
advised that its use may infringe existing patents.
	The original developer of this software module and his/her company, the 
subsequent editors and their companies, and ISO/IEC have no liability for use of 
this software module or modifications thereof in an implementation. Copyright is 
not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
	ENST retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using the code 
for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright    
notice must be included in all copies or derivative works.              
						Copyright  2002.
*/

///////////////////////////////////////////////////////////////////////////
// File:  OrderedGroupProxy.cpp
//
// Author:  Jean Le Feuvre - ENST 2002
//
// Description:
//   proxy object for OrderedGroup node.
//
// Created: August 2002.
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include "MediaObj.h"
#include "Presenter.h"
#include "nodes.h"
#include "OrderedGroupProxy.h"
#include "drawable.h"
#include "trace.h"


static int ComparePriority(const void* elem1, const void* elem2)
{
	priorityPosition * child1 = (priorityPosition *)elem1;
	priorityPosition * child2 = (priorityPosition *)elem2;

	// Compare drawing orders
	if (child1->priority < child2->priority)
		return -1;
	else if (child1->priority > child2->priority)
		return +1;
	else return 0;

}

OrderedGroupProxy* Presenter::InitOrderedGroup(OrderedGroup* pObj)
{
	return new  OrderedGroupProxy(pObj);
}

OrderedGroupProxy::~OrderedGroupProxy()
{
	if(m_priorityPositionList!=NULL) {
		delete[] m_priorityPositionList;	
		m_priorityPositionList=NULL;
		delete[] m_previousPriorityPositionList;
	}
}

//Render the children in the order specified by the order field.
void OrderedGroupProxy::Render(Effects *effects)
{
	int length= GetObject()->GetChildCount();
	int oglen = GetOrderedGroup()->order.GetLength();
	if (oglen==0) {
		RenderChildren(effects);
		return;
	}

	int i;
	DWORD now = 1 + GetObject()->GetUpdateTime();
	
	Effects *newEffects = new Effects(effects);

	//check whether the OrderedGroup node has changed 	
	if( now != m_LastUpdateTime) {
		m_LastUpdateTime = now;

		if(m_priorityPositionList!=NULL) {
			delete[] m_previousPriorityPositionList;
			m_previousPriorityPositionList=m_priorityPositionList;
		}
		
		m_priorityPositionList = new priorityPosition[length];
		for(i=0;i<length;i++) {
			(*(m_priorityPositionList+i)).position = i;
			if (i<oglen)
				(*(m_priorityPositionList+i)).priority = GetOrderedGroup()->order[i];
			else 
				(*(m_priorityPositionList+i)).priority = 0;
		}
		qsort(m_priorityPositionList,length,sizeof(priorityPosition),ComparePriority);

	
		//resort sensors in given order
		m_sensors.Clear();
		for(i=0; i<length; i++) {
			Traversable * pTrav= GetChild((*(m_priorityPositionList+i)).position);
			//skip time sensors
			if(pTrav && pTrav->IsSensor()) 
				m_sensors += pTrav;
		}
		//and invalidate all nodes below otherwise the bounds tracking won't work
		newEffects->m_bInvalidateAll = TRUE;
	}
	
	//fill effects sensor list
	for (i = 0; i <m_sensors.GetLength(); i++) {
		Traversable * pTrav=m_sensors[i];
		newEffects->AddUserEventHandler((UserEventHandlerProxy *)pTrav, &newEffects->transform);
	}

	//prerender children with the new order
	for(i=0; i<length; i++) {
		Traversable * pTrav= GetChild((*(m_priorityPositionList+i)).position);
		if (pTrav) pTrav->Render(newEffects);
	}
	delete newEffects;
} 

