// Osmose.h : main header file for the OSMOSE application
//

#ifndef AFX_OSMOSE_H_
#define AFX_OSMOSE_H_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#ifndef __AFXWIN_H__
	#error include 'defs.h' before including this file for PCH
#endif

#include "resource.h"       // main symbols

/////////////////////////////////////////////////////////////////////////////
// OsmoseApp:
// See Osmose.cpp for the implementation of this class
//

#include "App.h"
#include "Executiv.h"
#include "../UI/UIDevice.h"


#include "OsmoseGdiplus.h"

// User defined messages
enum {
	WM_SCENE_FINISHED = WM_USER + 1, 
	WM_DDRAW_PROBLEM,
	WM_NAVIGATE,
	WM_SETSIZE,
	WM_OPENFILE,
	WM_CONSOLEMSG,
};

//private sensor types
enum
{
	SENSOR_ZOOMIN = SENSOR_ANCHOR + 1,
	SENSOR_ZOOMOUT,
	SENSOR_MOVEOPEN,
	SENSOR_MOVECLOSE,
	SENSOR_RESETPRIVATE,
};


class OsmoseApp : public CWinApp, public Application
{
public:
	OsmoseApp();

	// Functions defined in the Application class
	void OnConnect(RESPONSE result);
	void OnFinish(RESPONSE reason);
	void OnError(NON_FATAL_ERROR nError, va_list);
	void OnFatalError (FATAL_ERROR, LPCSTR msg);
	int Navigate(LPCSTR szAddress);
	void OverSensor(unsigned int sensorType);
	void SetSize(unsigned int width, unsigned int height);
	void ViewportStackChanged(class BindableStack *stack);

	const char *GetApplicationName();
	const char *GetApplicationVersion();
	const char *GetWorldURL();

	// Handles the message generated when the presenter has
	// finished rendering the scene.
	void OnSceneFinished();


	// Return a pointer to the executive
	Executive* GetExecutive() const { return m_pExecutive; }
	
	StringSensorInstance *GetStringSensorDevice() { return &m_keyboardInstance; }
	KeySensorInstance *GetKeySensorDevice() { return &m_Web3DKeySensorInstance; }
	MouseSensorInstance *GetMouseDevice() { return &m_mouseInstance; }
	DeviceFilter *GetFilter() { return &m_deviceFilter; }
	
	// Called when the view is first initialised
	void CreateExecutive(HWND hWnd);

	// Read and write floating point values to the INI file
	float GetProfileFloat(LPCTSTR lpszSection, LPCTSTR lpszEntry, float fDefault);
	BOOL WriteProfileFloat(LPCTSTR lpszSection, LPCTSTR lpszEntry, float value);

	// Read and write the list of network address to the INI file
	void GetProfileNetworkAddresses();
	void WriteProfileNetworkAddresses();

	// Return a reference to the array of network addresses
	ZArray<LPSTR>& GetAddressList() { return m_networkAddresses; }

	int m_CursorType;

public:
	BOOL m_bUseCursors;
	BOOL m_bCursorIsPrivate, m_privateType;

	// The mutex is set while the file is open
	ZSemaphore m_smpFileOpen;

	//render options & co
	BOOL m_clearOnEnd;
	BOOL m_bFastSpeed;
	BOOL m_bAntiAlias;
	BOOL m_bTextAntiAlias;
	BOOL m_bZoom;
	BOOL m_bRatio;
	BOOL m_bWireframe;
	BOOL m_bSplitTracking;
	BOOL m_bShowTiming;
	BOOL m_bUseYUVHW;
	HCURSOR m_hCursorNormal, m_hCursorHand, m_hCursorPlane, m_hCursorDisc, m_hCursorProxy, m_hCursorZoomIn, m_hCursorZoomOut, m_hCursorMove, m_hCursorMoveClose;
	int m_VideoMode;

	char m_szConsole[5000];
	int m_MessageType;
	DWORD m_lastMessageTime;

private:
	
	DeviceFilter m_deviceFilter;
	StringSensorInstance m_keyboardInstance;
	KeySensorInstance m_Web3DKeySensorInstance;
	MouseSensorInstance m_mouseInstance;

	// The executive object
	Executive* m_pExecutive;

	// The timer used to update the playing time
	UINT m_timerID;

	// An array of network address that have been entered
	ZArray<LPSTR> m_networkAddresses;

	//GDIPlus variables
	ULONG_PTR m_gdiToken;

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(OsmoseApp)
	public:
	virtual BOOL InitInstance();
	virtual int ExitInstance();
	//}}AFX_VIRTUAL

// Implementation

public:
	//{{AFX_MSG(OsmoseApp)
	afx_msg void OnAppAbout();
		// NOTE - the ClassWizard will add and remove member functions here.
		//    DO NOT EDIT what you see in these blocks of generated code !
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

inline OsmoseApp *GetApp() { return (OsmoseApp *)AfxGetApp(); }
class CMainFrame;
inline CMainFrame* GetMainFrame() { return (CMainFrame*)GetApp()->m_pMainWnd; }


/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // AFX_OSMOSE_H_
