// OsmoseDoc.cpp : implementation of the COsmoseDoc class
//

#include "defs.h"
#include <assert.h>
#include "trace.h"
#include "Clock.h"
#include "Osmose.h"
#include "MainFrm.h"
#include "OsmoseDoc.h"
#include "EnterNetworkAddress.h"
#include "Presenter.h"

#include "OsmoseView.h"
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// COsmoseDoc

IMPLEMENT_DYNCREATE(COsmoseDoc, CDocument)

BEGIN_MESSAGE_MAP(COsmoseDoc, CDocument)
	//{{AFX_MSG_MAP(COsmoseDoc)
	ON_COMMAND(ID_FILE_RESTART, OnFileRestart)
	ON_UPDATE_COMMAND_UI(ID_FILE_RESTART, OnUpdateFileRestart)
	ON_COMMAND(ID_FILE_CLOSE, OnFileClose)
	ON_UPDATE_COMMAND_UI(ID_FILE_CLOSE, OnUpdateFileClose)
	ON_COMMAND(ID_FILE_OPEN_NETWORK, OnFileOpenNetwork)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// COsmoseDoc construction/destruction

COsmoseDoc::COsmoseDoc()
{
	m_fileOpened = FALSE;
	m_webFileOpened= FALSE;
	m_fileName.Empty();
}

COsmoseDoc::~COsmoseDoc()
{
}

BOOL COsmoseDoc::OnNewDocument()
{
	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// COsmoseDoc serialization

void COsmoseDoc::Serialize(CArchive& /*ar*/)
{
}

/////////////////////////////////////////////////////////////////////////////
// COsmoseDoc diagnostics

#ifdef _DEBUG
void COsmoseDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void COsmoseDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG


/////////////////////////////////////////////////////////////////////////////

//
// COsmoseDoc::DocumentNeedsOpening
//
// This returns TRUE if a file needs to be opened when the executive is
// started.  This only happens when a file is opened in response to being
// double clicked.
//

BOOL COsmoseDoc::DocumentNeedsOpening()
{
	return ((!m_fileName.IsEmpty()) && (!m_fileOpened));
}


/////////////////////////////////////////////////////////////////////////////

// COsmoseDoc::OpenDocument
//
// Open the file whose name is given by the m_fileName data member of
// the class.  Because Executive::Start() is an asynchronous function,
// we can only assume that the file has been successfully opened when
// Application::OnConnect() is called.

// disable "assignment within conditional expression"
#pragma warning (disable : 4706)

BOOL COsmoseDoc::OpenDocument() 
{
	// Close the current file - if one is open
	if (m_fileOpened)
		CloseDocument();

	//Check whether in the command line there is "-U" option
	LPTSTR p_strCLine;
	char *p;
	p_strCLine=GetCommandLine();
	if ((p=strstr(p_strCLine," -U")) || (p=strstr(p_strCLine," -u")))
	{
		p+=4;
		m_fileName=p;//now m_fileName should contain 
	}                //the passed command line unmodified
	                 //excepting for -U/-u

	GetApp()->GetExecutive()->Start(m_fileName);
	TRACE_PRINT("Starting the executive, given a file or URL.\n");
	
	return TRUE;
} /* COsmoseDoc::OpenDocument */

#pragma warning (default : 4706)
	


/////////////////////////////////////////////////////////////////////////////

BOOL COsmoseDoc::OnOpenDocument(LPCTSTR lpszPathName) 
{
	if (!m_fileName.IsEmpty()) {
		GetApp()->GetExecutive()->Stop();
		m_fileName = "";
		//bug in Core close ?
		Sleep(1000);
	}

	// Remember the name of the file
	m_fileName = lpszPathName;

	// If the executive has not yet been created (as is the case if
	// this application started in response to a file being double
	// clicked, then delay opening the document).
	if (GetApp()->GetExecutive() == NULL)
		return TRUE;
	else
		// Open the document with this name
		return OpenDocument();
}


/////////////////////////////////////////////////////////////////////////////

void COsmoseDoc::OnFileOpenNetwork() 
{
	// Close the current file - if one is open
	if (m_fileOpened)
		CloseDocument();

	CEnterNetworkAddress dialog;
	int result = dialog.DoModal();
	
	if (result == IDOK)
	{
		const char* pAddress = dialog.GetNetworkAddress();
		GetApp()->GetExecutive()->Start( pAddress);
		m_fileOpened=TRUE;
		m_fileName=pAddress;
	}

} /* COsmoseDoc::OnFileOpenNetwork */


/////////////////////////////////////////////////////////////////////////////

void COsmoseDoc::OnFileRestart() 
{
	// Only proceed if there is a previous file to restart
	if (!m_fileName.IsEmpty())
	{
		// Close the current file
		CloseDocument();

		// Open the previous file, and start it playing
		OpenDocument();
	}
} /* COsmoseDoc::OnFileRestart */


/////////////////////////////////////////////////////////////////////////////

void COsmoseDoc::CloseDocument() 
{
	// Only proceed if  a file is actually open
	if (m_fileOpened)
	{
		// Stop the executive running this session
		GetApp()->GetExecutive()->Stop();
		m_fileOpened = FALSE;

		// Wait for the application to be notified that the file
		// has been closed
		GetApp()->m_smpFileOpen.P();
		GetApp()->m_smpFileOpen.V();
	}
}

/////////////////////////////////////////////////////////////////////////////

void COsmoseDoc::OnCloseDocument() 
{
	TRACE_PRINT("Entering COsmoseDoc::OnCloseDocument()\n");
	CloseDocument();	
	CDocument::OnCloseDocument();
	TRACE_PRINT("Leaving COsmoseDoc::OnCloseDocument()\n");
} /* COsmoseDoc::OnCloseDocument */


/////////////////////////////////////////////////////////////////////////////

void COsmoseDoc::OnUpdateFileRestart(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(!m_fileName.IsEmpty());
}

void COsmoseDoc::OnFileClose() 
{
	if (m_fileName.IsEmpty()) return;
	GetApp()->GetExecutive()->Stop();
	m_fileName = "";
	GetMainFrame()->SetWindowText("Osmose");
}

void COsmoseDoc::OnUpdateFileClose(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(!m_fileName.IsEmpty());
}

