/*
	This software module was originally developed by ENST in the course of
development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.
	This software module is an implementation of a part of one or more MPEG-4
Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1) 
standard. ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license 
to this software module or modifications thereof for use in hardware or software 
products claiming conformance to the MPEG-4 Systems (ISO/IEC 14496-1). 
Those intending to use this software module in hardware or software products are 
advised that its use may infringe existing patents.
	The original developer of this software module and his/her company, the 
subsequent editors and their companies, and ISO/IEC have no liability for use of 
this software module or modifications thereof in an implementation. Copyright is 
not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
	ENST retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using the code 
for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright    
notice must be included in all copies or derivative works.              
						Copyright  2002.
*/

///////////////////////////////////////////////////////////////////////////
// File:  PixelTextureProxy.cpp
//
// Author: Jean Le Feuvre, ENST 2002
//
// Modified:
//
// Description:
//   proxy object for PixelTexture node.
//
// Created: august, 2002.
//
//
///////////////////////////////////////////////////////////////////////////

#include "defs.h"
#include "PixelTextureProxy.h"
#include "Presenter.h"


PixelTextureProxy* Presenter::InitPixelTexture(PixelTexture* pObj)
{
	return new PixelTextureProxy(pObj);
}

PixelTextureProxy::PixelTextureProxy(MediaObject* pMO) : DrawableImage(pMO)
{
	m_LastUpdateTime = 0;
}

void PixelTextureProxy::Update(DrawableContext * pContext)
{
	DWORD now = GetObject()->GetUpdateTime();
	//nothing to do
	if (m_LastUpdateTime == now+1) return;
	m_LastUpdateTime = now+1;

	PixelTexture *texture = GetPixelTexture();
	Image image = (SF<Image>) texture->image;

	pContext->m_bIsTransparent = TRUE;
	unsigned int stride = image.width;
	MediaFormat colorFormat;
	switch (image.numComponents) {
	case 0:
		colorFormat = MediaFormat :: GRAYSCALE;
		pContext->m_bIsTransparent = FALSE;
		break;
	case 1:
		colorFormat = MediaFormat :: ALPHAGRAYSCALE;
		stride *= 2;
		break;
	case 2:
		colorFormat = MediaFormat :: RGB;
		pContext->m_bIsTransparent = FALSE;
		stride *= 3;
		break;
	case 3:
		colorFormat = MediaFormat :: RGBA;
		stride *= 4;
		break;
	default:
		return;
	}
	//create texture object if needed
	if (!m_pTexture) m_pTexture = new VisualTexture;
	//init texture object
	m_pTexture->Init(image.width, image.height, (unsigned char *)image.pixels, stride, colorFormat, 0, 0);

	//set repeat S and T
	m_pTexture->SetTextureRepeat((BOOL) (Bool) GetPixelTexture()->repeatS, (BOOL) (Bool) GetPixelTexture()->repeatT);
}

