/*
	This software module was originally developed by ENST in the course of
development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.
	This software module is an implementation of a part of one or more MPEG-4
Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1) 
standard. ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license 
to this software module or modifications thereof for use in hardware or software 
products claiming conformance to the MPEG-4 Systems (ISO/IEC 14496-1). 
Those intending to use this software module in hardware or software products are 
advised that its use may infringe existing patents.
	The original developer of this software module and his/her company, the 
subsequent editors and their companies, and ISO/IEC have no liability for use of 
this software module or modifications thereof in an implementation. Copyright is 
not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
	ENST retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using the code 
for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright    
notice must be included in all copies or derivative works.              
						Copyright  2002.
*/

///////////////////////////////////////////////////////////////////////////
// File:  PlaneSensor2DProxy.cpp
//
// Author: Jean Le Feuvre - ENST 2002
//
// Description:
//   proxy object for PlaneSensor2D node.
//
// Created: August 2002
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include "Nodes-INIT.h"
#include "PlaneSensor2DProxy.h"
#include "Presenter.h"


PlaneSensor2DProxy* Presenter::InitPlaneSensor2D(PlaneSensor2D* pObj)
{
	return new PlaneSensor2DProxy(pObj);  
}

BOOL PlaneSensor2DProxy::OnUserEvent(EventInfo const* pEvent, Matrix2D *pSensorMatrix)
{
	// Only generate events if this node is enabled
	if (!(Bool) GetPlaneSensor2D()->enabled) return FALSE;

	// this function is called when the mouse is not on a object
	if (pEvent->m_pContext==NULL) {
		if (m_LastWasNotOver) {
			if (pEvent->m_event == EVENT_MOUSE_UP) {
				GetPresenter()->UnregisterEventHandler(this);
				GetPresenter()->SetTracking(FALSE);
				m_mouseDown = FALSE;
			}
			if (!m_mouseDown)
				GetPresenter()->UnregisterEventHandler(this);

			return TRUE;
		}
		m_LastWasNotOver = TRUE;
		return FALSE;
	}

	assert(pSensorMatrix);

	m_LastWasNotOver = FALSE;

	float X, Y;
	X = pEvent->m_x;
	Y = pEvent->m_y;
	//translate to our local coord system
	pSensorMatrix->ReverseApply(X, Y);

   	// activation
	if ((pEvent->m_event == EVENT_MOUSE_DOWN) && (!m_mouseDown)) {
		m_mouseDown = TRUE;
		m_InitialMatrix.Set(*pSensorMatrix);
		//and store start draging point
		m_pointStartDrag = Vec2f(X, Y);;
		//and register as active
		GetPresenter()->RegisterEventHandler(this);
	}

	//deactivation
	if (pEvent->m_event == EVENT_MOUSE_UP) {
		GetPresenter()->UnregisterEventHandler(this);
		GetPresenter()->SetTracking(FALSE);
		m_mouseDown = FALSE;
		
		//update offset field if autoOffset field is enabled
		if((Bool)(GetPlaneSensor2D()->autoOffset)) 
			GetPlaneSensor2D()->offset = GetTranslation(X, Y, *pSensorMatrix);

		return TRUE;
	}

	//dragging operation
	if ((pEvent->m_event == EVENT_MOUSE_MOVE) && (m_mouseDown)) {

		GetPresenter()->SetTracking(TRUE);

		//generate trackpoint_changed in local coords
		GetPlaneSensor2D()->trackPoint_changed = Vec2f(X, Y);
		
		// generate translation_changed 
		GetPlaneSensor2D()->translation_changed = GetTranslation(X, Y, *pSensorMatrix);
	}
	return TRUE;
} 


Vec2f PlaneSensor2DProxy::GetTranslation(float X, float Y, Matrix2D matrix)
{
	Vec2f result, min, max;
	float a, b;
	//switch our origin to current system
	a = m_pointStartDrag.x;
	b = m_pointStartDrag.y;
	m_InitialMatrix.Apply(a, b);
	matrix.ReverseApply(a, b);

	result.x = X - a + ((Vec2f)(GetPlaneSensor2D()->offset)).x;
	result.y = Y - b + ((Vec2f)(GetPlaneSensor2D()->offset)).y;

	min = GetPlaneSensor2D()->minPosition;
	max = GetPlaneSensor2D()->maxPosition;

	//bounds on X
	if (min.x <= max.x) {
		if (result.x < min.x) result.x = min.x;
		if (result.x > max.x) result.x = max.x;
	}
	//bounds on Y
	if (min.y <= max.y) {
		if (result.y < min.y) result.y = min.y;
		if (result.y > max.y) result.y = max.y;
	}
	return result;
}
