/*
	This software module was originally developed by ENST in the course of
development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.
	This software module is an implementation of a part of one or more MPEG-4
Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1) 
standard. ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license 
to this software module or modifications thereof for use in hardware or software 
products claiming conformance to the MPEG-4 Systems (ISO/IEC 14496-1). 
Those intending to use this software module in hardware or software products are 
advised that its use may infringe existing patents.
	The original developer of this software module and his/her company, the 
subsequent editors and their companies, and ISO/IEC have no liability for use of 
this software module or modifications thereof in an implementation. Copyright is 
not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
	ENST retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using the code 
for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright    
notice must be included in all copies or derivative works.              
						Copyright  2002.
*/

///////////////////////////////////////////////////////////////////////////
// File:  PointSet2DProxy.cpp
//
// Author: Jean Le Feuvre
//
// Created: august 2002.
//
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include "Nodes-INIT.h"
#include "PointSet2DProxy.h"
#include "Presenter.h"

PointSet2DProxy* Presenter::InitPointSet2D(PointSet2D* pObj)
{
	return new PointSet2DProxy(pObj);   
}



void PointSet2DProxy::Render(Effects *effects)
{
	if (!GetCoordinate2D()) return;
	DrawableContext *pContext = InitRenderContext(effects);
	if (!pContext) return;

	//check for geometry change
	BOOL buildIt = effects->m_bShouldRebuild ? TRUE : FALSE;
	DWORD now = GetObject()->GetUpdateTime();
	if (now > m_LastUpdateTime) {
		m_LastUpdateTime = now;
		buildIt = TRUE;
	}
	now = 1 + GetCoordinate2D()->GetUpdateTime();
	if (now > m_LastUpdateTime) {
		m_LastUpdateTime = now;
		buildIt = TRUE;
	}
	if (GetColorNode()) {
		now = 1 + GetColorNode()->GetUpdateTime();
		if (now > m_LastUpdateTime) {
			m_LastUpdateTime = now;
			buildIt = TRUE;
		}
	}
	if (buildIt) {
		BuildGraph();
		m_bHasChanged = TRUE;
	}

	m_Path.GetBoundingRect(& pContext->m_boundingRect, &pContext->m_transform, 1.0, 0 /* IsLineScaled(pContext)*/ );
	pContext->m_clippedBoundingRect = pContext->m_boundingRect.GetClipper();

	FinalizeRender(effects, pContext);
}

void PointSet2DProxy::Draw(DrawableContext * pContext)
{
	if (!SetupDraw(pContext)) return;

	Color3f LineCol = pContext->gProps.emissiveColor;

	//point sets are tricky since they never may not be filled and out lined...
	pContext->m_pVisualSurface->SetBrushStyle(0, LineCol, 0.0);

	Color *colorNode = GetColorNode();

	//2 cases: no color node specified, draw our path with the global color
	if (!colorNode) {
		if (pContext->gProps.hasLineProperties) LineCol = pContext->gProps.lineColor;
		pContext->m_pVisualSurface->SetLineStyle(LineCol, pContext->gProps.lineTransparency, pContext->gProps.lineWidth);
		pContext->m_pVisualSurface->DrawVisualPath(&m_Path);
		return;
	}

	Coordinate2D *coord = GetCoordinate2D();
	VisualPath path;

	//otherwise draw point by point
	//check we have a valid color field
	assert(colorNode->color.GetLength() >= coord->point.GetLength());

	for (int i = 0; i < coord->point.GetLength(); i++) {
		pContext->m_pVisualSurface->SetLineStyle(colorNode->color[i], pContext->gProps.lineTransparency, pContext->gProps.lineWidth);
		
		path.Reset();
		path.AddPoint(coord->point[i].x, coord->point[i].y);
		pContext->m_pVisualSurface->DrawVisualPath(&path);
	}
}

void PointSet2DProxy::BuildGraph()
{
	m_Path.Reset();
	for (int i = 0; i < GetCoordinate2D()->point.GetLength(); i++) {
		m_Path.AddPoint(GetCoordinate2D()->point[i].x, GetCoordinate2D()->point[i].y);
		m_Path.StartNewFigure();
	}
}
