/*
	This software module was originally developed by ENST in the course of
development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.
	This software module is an implementation of a part of one or more MPEG-4
Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1) 
standard. ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license 
to this software module or modifications thereof for use in hardware or software 
products claiming conformance to the MPEG-4 Systems (ISO/IEC 14496-1). 
Those intending to use this software module in hardware or software products are 
advised that its use may infringe existing patents.
	The original developer of this software module and his/her company, the 
subsequent editors and their companies, and ISO/IEC have no liability for use of 
this software module or modifications thereof in an implementation. Copyright is 
not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
	ENST retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using the code 
for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright    
notice must be included in all copies or derivative works.              
						Copyright  2002.
*/

///////////////////////////////////////////////////////////////////////////
// File:  ProximitySensor2DProxy.cpp
//
// Author: Jean Le Feuvre - ENST 2002
//
// Description:
//   proxy object for ProximitySensor2D node.
//
// Created: August 2002
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include "Nodes-INIT.h"
#include "ProximitySensor2DProxy.h"
#include "Presenter.h"


ProximitySensor2DProxy* Presenter::InitProximitySensor2D(ProximitySensor2D* pObj)
{
	return new  ProximitySensor2DProxy(pObj);   
}


BOOL ProximitySensor2DProxy::OnUserEvent(EventInfo const* pEvent, Matrix2D *pSensorMatrix)
{
	// Only generate events if this node is enabled, and no other
	// sensor node has grabbed all motion events
	if (! (Bool)GetProximitySensor2D()->enabled) return FALSE;

	//1- we're in the region
	if (pEvent->m_pContext) {
		assert(pSensorMatrix);
	
		//convert to local system
		float X, Y;
		X = pEvent->m_x;
		Y = pEvent->m_y;
		pSensorMatrix->ReverseApply(X, Y);

		m_lastTime = GetFlexTime() / 1000.;

		//don't use transform since we just reverted our coords
		if (IsInSensor(X, Y)) {
			m_HasLeft = FALSE;
			GetProximitySensor2D()->position_changed = Vec2f(X, Y);
			if(!m_JustEntered) {
				m_JustEntered = TRUE;
				m_InRegion = TRUE;
				GetProximitySensor2D()->isActive = TRUE;
				GetProximitySensor2D()->enterTime = m_lastTime;
				//and register as active
				GetPresenter()->RegisterEventHandler(this);
			}
			return TRUE;
		}
	}
	//2 - we're out (with or without context, depending on the size of the sensor)
	if (m_InRegion) {
		if (m_HasLeft) {
			m_InRegion = FALSE;
			m_JustEntered = FALSE;
			GetProximitySensor2D()->exitTime = m_lastTime;
			GetProximitySensor2D()->isActive = FALSE;
			GetPresenter()->UnregisterEventHandler(this);
			return TRUE;
		}
		m_HasLeft = TRUE;
	}
	return FALSE;
}

BOOL ProximitySensor2DProxy::IsInSensor(float X, float Y)
{
	Vec2f center = (Vec2f)GetProximitySensor2D()->center;
	Vec2f size = (Vec2f)GetProximitySensor2D()->size;

	if (X < center.x-size.x/2) return FALSE;
	if (X > center.x+size.x/2) return FALSE;
	if (Y < center.y-size.y/2) return FALSE;
	if (Y > center.y+size.y/2) return FALSE;
	return TRUE;
}
