/*
	This software module was originally developed by ENST in the course of
development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.
	This software module is an implementation of a part of one or more MPEG-4
Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1) 
standard. ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license 
to this software module or modifications thereof for use in hardware or software 
products claiming conformance to the MPEG-4 Systems (ISO/IEC 14496-1). 
Those intending to use this software module in hardware or software products are 
advised that its use may infringe existing patents.
	The original developer of this software module and his/her company, the 
subsequent editors and their companies, and ISO/IEC have no liability for use of 
this software module or modifications thereof in an implementation. Copyright is 
not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
	ENST retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using the code 
for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright    
notice must be included in all copies or derivative works.              
						Copyright  2002.
*/

///////////////////////////////////////////////////////////////////////////
// File:  RectCollect.cpp
//
// Author: Jean Le Feuvre ENST 2002
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include "RectCollect.h"


void RectCollect::SetRectEmulation(BOOL on)
{
	if (on == m_bEmulateRect) return;
	if (on) MakeRect();
	m_bEmulateRect = on;
	assert(m_List.GetLength()<2);
}

void RectCollect::Union(BRect rc)
{
	if (rc.IsEmpty()) return;

	if (m_bEmulateRect) {
		if (m_List.GetLength()) {
			m_List[0].Union(rc);
		} else {
			m_List += rc;
		}
		return;
	}

	for (int i=0; i<m_List.GetLength(); i++) {
		if (m_List[i].Overlaps(&rc)) {
			m_List[i].Union(&rc);
			return;
		}
	}
	//if we get here we shall add an entry
	m_List += rc;
}


void RectCollect::UpdateCollection()
{
	if (m_bEmulateRect) return;

	//for each item
	for (int i=0; i<m_List.GetLength(); i++) {
		//test if the following can be merged
		for (int j=i+1; j<m_List.GetLength(); j++) {
			if (m_List[i].Overlaps(&m_List[j])) {
				m_List[i].Union(&m_List[j]);
				m_List.DeleteFrom(j);

				UpdateCollection();
				return;
			}
		}
		if (m_List[i].IsEmpty()) {
			m_List.DeleteFrom(i);
			i--;
		}
	}
}

BOOL RectCollect::IsEmpty()
{
	if (m_List.GetLength()) return FALSE;
	return TRUE;
}

void RectCollect::Merge(RectCollect &other)
{
	for (int i =0; i<other.m_List.GetLength();i++) {
		Union(other.m_List[i]);
	}
}

void RectCollect::MakeEmpty()
{
	m_List.Clear();
}

void RectCollect::Intersection(RectCollect &other)
{
	ZArray<BRect> finalList;
	BRect bounds;
	for (int i=0; i<m_List.GetLength(); i++) {
		for (int j=0;j<other.m_List.GetLength();j++) {
			bounds = m_List[i];
			if (m_List[i].Overlaps(&other.m_List[j])) {
				bounds.Intersection(&other.m_List[j]);
				finalList += bounds;
			}
		}
	}
	m_List.Clear();
	while (finalList.GetLength()) {
		m_List += finalList[0];
		finalList.DeleteFrom(0);
	}
}

void RectCollect::MakeRect()
{
	while (m_List.GetLength() > 1) {
		m_List[0].Union(&m_List[1]);
		m_List.DeleteFrom(1);
	}
}

