/*
	This software module was originally developed by ENST in the course of
development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.
	This software module is an implementation of a part of one or more MPEG-4
Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1) 
standard. ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license 
to this software module or modifications thereof for use in hardware or software 
products claiming conformance to the MPEG-4 Systems (ISO/IEC 14496-1). 
Those intending to use this software module in hardware or software products are 
advised that its use may infringe existing patents.
	The original developer of this software module and his/her company, the 
subsequent editors and their companies, and ISO/IEC have no liability for use of 
this software module or modifications thereof in an implementation. Copyright is 
not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
	ENST retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using the code 
for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright    
notice must be included in all copies or derivative works.              
						Copyright  2002.
*/

///////////////////////////////////////////////////////////////////////////
// File:  RectangleProxy.cpp
//
// Author: Jean Le Feuvre - ENST 2002
//
// Description:
//   proxy object for rectangle node.
//
// Created: August, 2002
//
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include "Nodes-INIT.h"
#include "RectangleProxy.h"
#include "Presenter.h"


RectangleProxy* Presenter::InitRectangle(class Rectangle * pObj)
{
	return new  RectangleProxy(pObj);    
}

void RectangleProxy::Render(Effects *effects)
{

	//changes in geometry
	DWORD now = 1 + GetObject()->GetUpdateTime();
	if ((now != m_LastUpdateTime) || effects->m_bShouldRebuild) {
		m_LastUpdateTime = now;
		m_Path.Reset();
		m_Path.AddRectangle(((Vec2f)GetRectangle()->size).x, ((Vec2f)GetRectangle()->size).y);
		m_bHasChanged = TRUE;
	}

	DrawableContext *pContext = InitRenderContext(effects);
	if (!pContext) return;

	m_Path.GetBoundingRect(& pContext->m_boundingRect, &pContext->m_transform, pContext->gProps.lineWidth, pContext->gProps.lineScaled);
	pContext->m_clippedBoundingRect = pContext->m_boundingRect;

	pContext->m_bIsTransparent = FALSE;
	if (GetTexture(pContext) && GetTexture(pContext)->m_bIsTransparent) {
		pContext->m_bIsTransparent = TRUE;
	} else if (pContext->m_pGradient) {
	} else if (!pContext->gProps.filled) {
		pContext->m_bIsTransparent = TRUE;
	} else if (pContext->gProps.transparency > 0.0) {
		pContext->m_bIsTransparent = TRUE;
	} else if (pContext->m_transform.HasRotations() ) {
		pContext->m_bIsTransparent = TRUE;
	}

	FinalizeRender(effects, pContext);
}

void RectangleProxy::GetOriginalBounds(BRect &bounds)
{
	m_Path.GetBoundingRect(&bounds, NULL, 0);
}

void RectangleProxy::AddToClipper(VisualClipper *pClipper, DrawableContext *pContext, BOOL bInside, BOOL bUseXOR)
{
	pClipper->AddPath(&m_Path, &pContext->m_transform, bInside, bUseXOR);
}

void RectangleProxy::Draw(DrawableContext * pContext)
{
	if (!SetupDraw(pContext)) return;
	DrawTexture(pContext);
	pContext->m_pVisualSurface->DrawVisualPath(&m_Path);
}

