/*
	This software module was originally developed by ENST in the course of
development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.
	This software module is an implementation of a part of one or more MPEG-4
Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1) 
standard. ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license 
to this software module or modifications thereof for use in hardware or software 
products claiming conformance to the MPEG-4 Systems (ISO/IEC 14496-1). 
Those intending to use this software module in hardware or software products are 
advised that its use may infringe existing patents.
	The original developer of this software module and his/her company, the 
subsequent editors and their companies, and ISO/IEC have no liability for use of 
this software module or modifications thereof in an implementation. Copyright is 
not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
	ENST retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using the code 
for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright    
notice must be included in all copies or derivative works.              
						Copyright  2002.
*/

///////////////////////////////////////////////////////////////////////////
// File:  SwitchProxy.cpp
//
// Author: Jean Le Feuvre - ENST 2002
//
// Description:
//   proxy object for Switch node.
//
// Created: August 2002.
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include "Nodes-INIT.h"
#include "SwitchProxy.h"
#include "Presenter.h"


SwitchProxy* Presenter::InitSwitch(Switch* pObj)
{
	return new  SwitchProxy(pObj); 
}

void SwitchProxy::Render(Effects *effects)
{
	int nNumberOfChildren = GetSwitch()->choice.GetLength();
	int nWhichChoice = GetSwitch()->whichChoice;

	if (nWhichChoice>=nNumberOfChildren) return;
	if (nWhichChoice<0) return;

	Effects *newEffects = new Effects(effects);

	/*if choice is changing the children must be notified they're inactive - this is needed because 
	switch can contain audio nodes otherwise we wouldn't have to traverse the children...*/
	if (m_nPrevChoice != nWhichChoice) {
		newEffects->m_isSwitchedOff = TRUE;
		for (int i=0; i<nNumberOfChildren; i++) {
			if (i==nWhichChoice) continue;
			Traversable *child = (Traversable *) GetSwitch()->choice[i]->GetRenderingObject();
			if (child) child->Render(newEffects);
		}
		m_nPrevChoice = nWhichChoice;
		newEffects->m_isSwitchedOff = FALSE;
	}

	//fill sensor: only keep the active choice as a sensor if it is. 
	Traversable * pTrav = (Traversable *) GetSwitch()->choice[nWhichChoice]->GetRenderingObject();
	if(pTrav && pTrav->IsSensor())
		newEffects->AddUserEventHandler((UserEventHandlerProxy *)pTrav, &newEffects->transform);

	if(pTrav) pTrav->Render(newEffects);

	delete newEffects;
}