/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/


#include "defs.h"
#include "SynthesizedTextureProxy.h"
#include "ViewportProxy.h"
#include "Stream.h"
#include "Presenter.h"
#include "GeometryNode.h"



extern "C" {
#include "ImageType/ImageType.h"
}


static void	draw_image( VisualTexture *pTexture, image_type *im, image_type *im1 );
static void	draw_image1( VisualTexture *pTexture, image_type *im, image_type *im1 );



SynthesizedTextureProxy *Presenter::InitSynthesizedTexture(SynthesizedTexture* pObj)
{
	return new SynthesizedTextureProxy(pObj);
}



SynthesizedTextureProxy::SynthesizedTextureProxy(MediaObject* pMO) : DrawableImage(pMO)
{
	m_startTime = 0;
	m_bFirstTime = TRUE;
	m_bFirstFrameFetched = FALSE;


	GetPresenter()->RegisterTimedNode(this);
	m_bIsRegistered = TRUE;
	m_bNeedsUnregister = FALSE;

	m_pStream = NULL;

	m_LastDrawTime = 0;
	m_pTexture = NULL;


	m_st = NULL;

	m_im = m_im1 = NULL;
}


SynthesizedTextureProxy::~SynthesizedTextureProxy()
{
	m_pTexture = NULL;

	if( m_st != NULL ){
		delete m_st;
		m_st = NULL;
	}
	if( m_im != NULL ){
		image_destroy( (image_type *)m_im, 1 );
		m_im = NULL;
	}

}



static void	set_vim_image( VisualTexture *pTexture );



void SynthesizedTextureProxy::Update(DrawableContext *pContext)
{
//Effects *effects = new Effects();

int width, height;
//CVimHandler	vimHandler;
int	frame = -1;

	if( m_st == NULL )	return;

	if( m_pTexture == NULL ){
/*
		if( GetSynthesizedTexture()->pixelWidth <= 0 )
			GetSynthesizedTexture()->pixelWidth = 200;

		if( GetSynthesizedTexture()->pixelHeight <= 0 )
			GetSynthesizedTexture()->pixelHeight = 200;

		width = GetSynthesizedTexture()->pixelWidth;
		height = GetSynthesizedTexture()->pixelHeight;
*/

		width = m_st->GetFrameWidth();
		height = m_st->GetFrameHeight();

		GetSynthesizedTexture()->pixelWidth = width;
		GetSynthesizedTexture()->pixelHeight = height;

		m_pTexture = new VisualTexture();
		m_pTexture->InitOffscreen( width, height);


		BRect visibleRect;

		visibleRect.width = width;
		visibleRect.height = height;
		visibleRect.left = 0;
		visibleRect.top = 0;


		m_pTexture->SetActiveRect(&visibleRect);

		m_st->StartPlayer();
	}



	//texture is already rendered
	DWORD now = GetObject()->GetClock() + 1;
	if (m_LastDrawTime == now) return;

	frame = m_st->GetNextFrameToPlay( now );
	if( frame <0 )	return;

	m_im = m_st->Render( frame, (image_type *)m_im );
	if( m_im1 == NULL )
		m_im1 = (void *)image_create( ((image_type *)m_im)->row, ((image_type *)m_im)->column, 4, 1, NULL );

	draw_image1( m_pTexture, (image_type *)m_im, (image_type *)m_im1 );

	m_pTexture->Invalidate();
	TRACE("CTexture changed\n");


	//then draw the texture
	m_LastDrawTime = now;

	pContext->m_bUpdateTexture = TRUE;




	BOOL rS, rT;
//	GetTextureRepeat(rS, rT);
//test
	rS = rT = TRUE;
//end
	m_pTexture->SetTextureRepeat(rS, rT);



		//also force update (stream not finished yet)
	GetPresenter()->InvalidateScene(NULL);
}


int
SynthesizedTextureProxy::LoadStream()
{

	if( m_bFirstTime == TRUE )	return(0);

	MediaStreamImp *pStream = GetSynthesizedTexture()->GetStream();
	if (!pStream) return(0);



	DWORD rdwTime;
	BOOL bEOU;
	LPVOID dataPtr;
	int dataSize;
	int status;

	rdwTime = 0;

	dataPtr = GetStreamConsumer()->Fetch(dataSize, rdwTime, bEOU, status, TRUE,
													FALSE, 0,	TRUE);

	// If there is no data, no updating is required
	if ((dataPtr == NULL) || (dataPtr == END_OF_STREAM) ) {
		return( -1 );
	}


	if( m_st == NULL )
		m_st = new CSt;

	m_st->Decode( (char*)dataPtr, dataSize );

	return( 1 );
}



void SynthesizedTextureProxy::UpdateTimeInfo()
{
	SynthesizedTexture *pTexture = GetSynthesizedTexture();

/*
	pTexture->startTime.ApplyOffset(& m_FlexTimeContext);
	pTexture->stopTime.ApplyOffset(& m_FlexTimeContext);

	double currentTime = GetFlexTime(m_FlexTimeContext) / 1000.;
*/
	double currentTime = GetFlexTime() / 1000.;

	if ( ! (Bool) pTexture->isActive && (pTexture->startTime != m_startTime)) {
		m_startTime = (double) pTexture->startTime;
	}

	//opens stream only at first access
	if (m_bFirstTime) {
		m_bFirstTime = FALSE;
		pTexture->OpenStream();

		//invalidate scene since we just started a new stream
		GetPresenter()->InvalidateScene(NULL);
	}


//ST
	if( m_st == NULL ) {
		LoadStream();
		GetPresenter()->InvalidateScene(NULL);
	}
// ST

	if( m_st != NULL )
		GetPresenter()->InvalidateScene(NULL);


	// Only proceed if the start time has been reached
	if (currentTime < m_startTime || (m_startTime < 0)) return;

	// stop if needed
	if ((currentTime >= pTexture->stopTime) && (pTexture->stopTime > m_startTime)) {
		if ((BOOL) (Bool)pTexture->isActive) {
			pTexture->isActive = FALSE;
			m_bNeedsUnregister = TRUE;
		}
		return;
	}

	// play
	if (! (Bool) pTexture->isActive) {
		pTexture->isActive = TRUE;
		//invalidate scene since we just started a new stream
		GetPresenter()->InvalidateScene(NULL);
	}
}


void SynthesizedTextureProxy::UpdateTranslation()
{
vec3d	*av;
int	i;

SynthesizedTexture *st = GetSynthesizedTexture();


	int	len = st->translation.GetLength();

	if( len == 0 )	return;

	av = (vec3d *) malloc( sizeof(vec3d) * len );


	for( i = 0 ; i < len ; i++ ){
		av[i].x = st->translation[i].x;
		av[i].y = st->translation[i].y;
		av[i].z = st->translation[i].z;
	}


	m_st->SetTranslation( av, len );




	free( av );
}


void SynthesizedTextureProxy::UpdateRotation()
{
vec3d	*av;
int	i;

SynthesizedTexture *st = GetSynthesizedTexture();


	int	len = st->rotation.GetLength();

	if( len == 0 )	return;

	av = (vec3d *) malloc( sizeof(vec3d) * len );


	for( i = 0 ; i < len ; i++ ){
		av[i].x = av[i].y = av[i].z = 0;
		if( st->rotation[i].x == 0 && st->rotation[i].y == 1 && st->rotation[i].z == 0 ){
			av[i].y = st->rotation[i].theta * 180 /3.14;
		}
	}


	m_st->SetRotation( av, len );




	free( av );
}




static void
draw_image( VisualTexture *pTexture, image_type *im, image_type *im1 )
{
int	i,	j;
u_int	*p,	*p1;


	for( i = 0 ; i < im->row ; i++ ){
		p = IMAGE4_PIXEL( im, i, 0 );
//		p1 = IMAGE4_PIXEL( im1,  im1->row-1-i, 0 );
		p1 = IMAGE4_PIXEL( im1,  i, 0 );

		for( j = 0 ; j < im->column ; j++, p++, p1++ )
			*p1 = RGB_REVERSE(*p) | 0xFF000000;
	}


	pTexture->Init(pTexture->m_Width, pTexture->m_Height, im1->data, im1->column_byte, MediaFormat :: RGBA, 0, 0 );

}



static void
draw_image1( VisualTexture *pTexture, image_type *im, image_type *im1 )
{
int	i,	j;
u_int	*p;
u_char	*tp;

	tp = im1->data;

	for( i = 0 ; i < im->row ; i++ ){
//		p = IMAGE4_PIXEL( im,  im->row-1-i, 0 );
		p = IMAGE4_PIXEL( im,  i, 0 );


		for( j = 0 ; j < im->column ; j++, p++ ){
			*tp++ = IMAGE4_RED(*p);
			*tp++ = IMAGE4_GREEN(*p);
			*tp++ = IMAGE4_BLUE(*p);
		}
	}

	int	columnByte = im->column * 3;


	pTexture->Init(pTexture->m_Width, pTexture->m_Height, im1->data, columnByte, MediaFormat :: RGB ); //, 0, 0 );

}


