/***********************************************************************
    This software module  was  originally  developed by Patrice Bazerque
	(IBM Research)
    in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)
    standard.  This software module is an implementation of a part of
    one or more MPEG-4 Systems (ISO/IEC 14496-1) tools  as  specified
    by  the  MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC gives
    users of the MPEG-4 Systems (ISO/IEC 14496-1)  free  license  to
    this software module or modifications thereof for use in hardware
    or software products claiming conformance to the  MPEG-4  Systems
    (ISO/IEC 14496-1). Those intending to use this software module in
    hardware or software  products  are  advised  that  its  use  may
    infringe   existing  patents.  The  original  developer  of  this
    software module and his/her company, the subsequent  editors  and
    their  companies,  and  ISO/IEC have no liability for use of this
    software module or modifications thereof  in  an  implementation.
    Copyright  is not released for non MPEG-4 Systems (ISO/IEC 14496-
    1) conforming  products.  IBM
    retain  full  right to use the code for their own purpose, assign
    or donate the code to a third party and to inhibit third  parties
    from  using  the  code  for  non MPEG-4 Systems (ISO/IEC 14496-1)
    conforming products. This copyright notice must  be  included  in
    all copies or derivative works. Copyright  1999.
************************************************************************/

/**************************************************************************
 * Project Im1Player
 * INTERNATIONAL STANDARD ORGANIZATION
 * ISO/IEC JTC1/SC29/WG11
 * 
 * SUBSYSTEM: BIFS Encoder/Decoder
 * FILE:      TemporalTransformProxy.cpp
 * AUTHOR:    Patrice Bazerque (IBM Research)
 * OVERVIEW:  The implementation of the proxy object used by the presenter
 *   to handle the FlexObject node.
 * 
 * Comment:   Contact: mykim@us.ibm.com
 *
 **************************************************************************/


#include "defs.h"
#include "Nodes-INIT.h"
#include "TemporalTransformProxy.h"
#include "../FlexTime/FlexTimeBase.h"
#include "flxtime.h"
#include "../FlexTime/FlexTimeObject.h"
#include "Presenter.h"


TemporalTransformProxy* Presenter::InitTemporalTransform(TemporalTransform* pObj)
{
	return new  TemporalTransformProxy(pObj); 
}

void TemporalTransformProxy::Render(Effects *effects)
{
	Effects *newEffects = new Effects(effects);

	TemporalTransformImp *ptt = (TemporalTransformImp *) GetTemporalTransform();
	PresenterContext ft_ctx;
	ft_ctx.SetTime(GetFlexTime());
	ptt->Transform(& ft_ctx);

	TemporalTransformImpFlex *pttf = (TemporalTransformImpFlex*) GetTemporalTransform();
	FlexTimeObject *pfto = pttf->GetFTObject();

	// If this object is active then we call prerender method of the children
	if ( (BOOL) (Bool) pttf->active) {
		if (pfto->MaxDelayHasExpired())
			pttf->active = false;
	}
	RenderChildren(newEffects);
	delete newEffects;
}
