/*
	This software module was originally developed by ENST in the course of
development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.
	This software module is an implementation of a part of one or more MPEG-4
Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1) 
standard. ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license 
to this software module or modifications thereof for use in hardware or software 
products claiming conformance to the MPEG-4 Systems (ISO/IEC 14496-1). 
Those intending to use this software module in hardware or software products are 
advised that its use may infringe existing patents.
	The original developer of this software module and his/her company, the 
subsequent editors and their companies, and ISO/IEC have no liability for use of 
this software module or modifications thereof in an implementation. Copyright is 
not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
	ENST retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using the code 
for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright    
notice must be included in all copies or derivative works.              
						Copyright  2002.
*/

///////////////////////////////////////////////////////////////////////////
// File:  TextProxy.cpp
//
// Authors: Jean Le Feuvre
//
//
// Description:
//   text and font setup
//
// Created: august 2002.
//
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include "Nodes-INIT.h"
#include "TextProxy.h"
#include "Presenter.h"
#include "App.h"


TextProxy* Presenter::InitText(Text* pObj)
{
	return new  TextProxy(pObj); 
}

//this shows how to get advancedFontStyle features. 
//NOT IMPLEMENTED in the proxy, only FontStyle supported for now
const char* TextProxy::GetFSfamily() const
{
	FontStyle *fs = GetFontStyle();
	XFontStyle *afs = GetAdvancedFontStyle();
	if (fs) return (LPCSTR)(const ZString&)fs->family[0];
	if (afs) return (LPCSTR)(const ZString&)afs->fontName;
	return "";
}

const char *  TextProxy::GetFSstyle() const
{
	if (GetFontStyle() == NULL) return "";
	return (LPCSTR)(const ZString&)GetFontStyle()->style;
}

float TextProxy::GetFSsize() const
{
	if (GetFontStyle() == NULL) return 1;
	return GetFontStyle()->size;
}

const char * TextProxy::GetFSjustify(int i) const
{
	if ( (GetFontStyle() == NULL) || (i>=GetFontStyle()->justify.GetLength()))
		return "FIRST";

	return (LPCSTR)(const ZString&)GetFontStyle()->justify[i];
}


BOOL TextProxy::GetFShorizontal() const
{
	if (GetFontStyle() == NULL) return TRUE;
	return (Bool)GetFontStyle()->horizontal;
}

float TextProxy::GetFSspacing() const
{
	if (GetFontStyle() == NULL) return 1;
	return GetFontStyle()->spacing;
}

float TextProxy::GetFSMaxExtend() const
{
	return GetText()->maxExtent;
}

const char* TextProxy::GetFSlanguage() const
{
	if (GetFontStyle() == NULL) return "";
	return (LPCSTR)(const ZString&)GetFontStyle()->language;
}

int TextProxy::GetFSleftToRight() const
{
	FontStyle *fs = GetFontStyle();
  	if (fs == NULL) return TRUE;
	return (Bool)(fs->leftToRight);
}

int TextProxy::GetFStopToBottom() const
{
  	if (GetFontStyle() == NULL) return TRUE;
	return (Bool)GetFontStyle()->topToBottom;
}

//
// DrawableText::GetAscentAndDescent
//
// the font ascent and descent are returned, assume lines have same ascent
// and descent
void TextProxy::GetAscentAndDescent(float &ascent,float &descent)
{
	ascent=m_ascent;
	descent=m_descent;
}


void TextProxy::Render(Effects *effects)
{

	// Only proceed if the text string is not empty
	Text *txt = GetText();
	if ((txt == NULL) || !txt->string.GetLength() ) {
		return;
	}
	char const* pData = txt->string[0];
	if ((pData == NULL) || (pData[0] == 0)) {
		return;
	}

	//check for geometry change
	BOOL buildIt = effects->m_bShouldRebuild ? TRUE : FALSE;
	DWORD now = 1 + GetObject()->GetUpdateTime();
	if (now > m_LastUpdateTime) {
		m_LastUpdateTime = now;
		buildIt = TRUE;
	}
	if (GetFontStyle()) {
		now = 1 + GetFontStyle()->GetUpdateTime();
		if (now > m_LastUpdateTime) {
			m_LastUpdateTime = now;
			buildIt = TRUE;
		}
	}
	if (buildIt) {
		BuildGraph(effects);
		m_bHasChanged = TRUE;
	}

	DrawableContext *pContext = InitRenderContext(effects);
	if (!pContext) return;
	pContext->m_isTextNode = TRUE;

	//get the text bounds
	GetBounds(pContext);
	pContext->m_clippedBoundingRect = pContext->m_boundingRect.GetClipper();

	FinalizeRender(effects, pContext);
} 


void TextProxy::GetBounds(DrawableContext *pContext)
{
	pContext->m_boundingRect.MakeEmpty();
	VisualPath path;
	path.AddRectangle(m_Bounds.left, m_Bounds.top, m_Bounds.width, m_Bounds.height);
	path.GetBoundingRect(&pContext->m_boundingRect, &pContext->m_transform, pContext->gProps.lineWidth, pContext->gProps.lineScaled);
}

void TextProxy::GetOriginalBounds(BRect &bounds)
{
	bounds = m_Bounds;
}

void TextProxy::Draw(DrawableContext *pContext)
{
	if (!SetupDraw(pContext)) return;

	for (int i=0; i<m_TextArray.GetLength(); i++) {
		DrawTexture(pContext, m_TextArray[i]);
		pContext->m_pVisualSurface->DrawVisualPath(m_TextArray[i]);
	}
}

void TextProxy::AddToClipper(VisualClipper *pClipper, DrawableContext *pContext, BOOL bInside, BOOL bUseXOR)
{
	for (int i=0; i<m_TextArray.GetLength(); i++) {
		pClipper->AddPath(m_TextArray[i], &pContext->m_transform, bInside, bUseXOR);
	}
}


typedef struct
{
	const char* pString;
	float width, height;
	float x_scaling, y_scaling;
} TextLine;


void TextProxy::ResetPath()
{
	//reset text bounds
	m_Bounds.MakeEmpty();

	while (m_TextArray.GetLength()) {
		VisualPath *ptr = m_TextArray[0];
		m_TextArray.DeleteFrom(0);
		delete ptr;
	}
}

void TextProxy::BuildGraph(Effects *effects)
{
	int i;
	TextLine *lines;

	ResetPath();
	m_Bounds.MakeEmpty();


	if (!GetFShorizontal()) {
		BuildVerticalGraph(effects);
		return;
	}

	//get font info
	const char * fontName = GetFSfamily();
	float fontSize = GetFSsize();
	if (fontSize <= 0) fontSize = 12;
    
	const char * pStyles = GetFSstyle();

	BOOL ltr = GetFSleftToRight();
 
	m_Path.SetFontEngine(GetPresenter()->GetFontEngine());

	if (! m_Path.SetFont(fontName, fontSize, pStyles)) {
		char msg[1024];
		sprintf(msg, "Couldn't find font %s", fontName);
		GetPresenter()->GetExecutive()->GetApp()->OnFatalError(OTHER_ERROR, msg);
		return;
	}
	fontSize = m_Path.AdjustFontSize(fontSize);

	//get spacing
	float line_spacing = GetFSspacing();
	line_spacing *= fontSize;

	float font_line_spacing;
	//compute overall bounding box size
	float tot_width = 0;
	float max_ext = GetFSMaxExtend();
	int NumLines = GetText()->string.GetLength();
	
	//alloc a struc
	lines = (TextLine *) malloc(sizeof(TextLine)*NumLines);
		
	//get metrics
	m_Path.GetFontMetrics(m_ascent, m_descent, font_line_spacing);

	for (i=0; i < NumLines; i++) {
		lines[i].pString = GetText()->string[i];
		lines[i].y_scaling = lines[i].x_scaling = 1.0;
		m_Path.GetTextSize(lines[i].pString, lines[i].width, lines[i].height);

		if (!lines[i].width) continue;
		//check any length info
		float length = GetLength(i);
		if (length > 0) {
			lines[i].x_scaling = length/lines[i].width;
//			lines[i].width = length;
		}
		if (tot_width < lines[i].width * lines[i].x_scaling) tot_width = lines[i].width * lines[i].x_scaling;
	}
	//apply max_extend. VRML doesn't say if all lines should be scale with the same factor... 
	//we assume yes for that will look better
	float max_scale = 1.0;
	if (max_ext>0 && tot_width>max_ext) {
		max_scale = max_ext/tot_width;
		tot_width = max_ext;
	}
	//compute height
	float tot_height = (NumLines-1)*line_spacing + (m_ascent + m_descent);
	//store logical height
	m_Bounds.height = tot_height;


	float start_x, start_y;
	CString major, minor;
	major = GetFSjustify(0);
	minor = GetFSjustify(1);

	BOOL ttb = GetFStopToBottom();
	
	//minor-justification is for the whole layout and store logical top
	if (minor=="MIDDLE") {
		if (ttb) {
			start_y = tot_height/2;
			m_Bounds.top = start_y;
		} else {
			start_y = m_descent + m_ascent - tot_height/2;
			m_Bounds.top = tot_height/2;
		}
	}
	else if (minor=="BEGIN") {
		if (ttb) {
			start_y = m_descent;
			m_Bounds.top = start_y;
		} else {
			m_Bounds.top = m_Bounds.height;
			start_y = m_descent + m_ascent;
		}
	}
	else if (minor=="END") {
		if (ttb) {
			start_y = tot_height;
			m_Bounds.top = start_y;
		} else {
			start_y = -tot_height + 2*m_descent + m_ascent;
			m_Bounds.top = start_y - (m_descent + m_ascent) + tot_height;
		}
	}
	else {
		start_y = m_ascent;
		m_Bounds.top = ttb ? start_y : (tot_height - m_descent);
	}
	
	int int_major;
	//major-justification
	if (major=="MIDDLE") {
		int_major = 0;
	} else if (major=="END") {
		int_major = 1;
	} else {
		int_major = 2;
	}

	BRect rc, final;
	final.MakeEmpty();

	for (i=0; i < NumLines; i++) {
		switch (int_major) {
		//major-justification MIDDLE
		case 0:
			start_x = -lines[i].width/2;
			break;
		//major-justification END
		case 1:
			start_x = (ltr) ? -lines[i].width : 0;
			break;
		//BEGIN, FIRST or default
		default:
			start_x = (ltr) ? 0 : -lines[i].width;
			break;
		}

		//reverse string if right to left
		ZString st;
		const char *string = lines[i].pString;
		if (strlen(string)) {
			if (!ltr) {
				int len = strlen(string);
				for (int k=0; k<len;k++) st[k] = string[len-k-1];
				string = st;
			}
			VisualPath *path = new VisualPath();
			path->SetFontEngine(GetPresenter()->GetFontEngine());
			path->SetFont(fontName, fontSize, pStyles);
			m_TextArray += path;
			path->AddText(string, start_x, start_y, lines[i].x_scaling*max_scale, lines[i].y_scaling, m_ascent);
			//also collect bounds info
			path->GetBoundingRect(&rc, NULL, 0, 0);
			final.Union(&rc);
		}
		if (ttb) {
			start_y -= line_spacing;
		} else {
			start_y += line_spacing;
		}
		//little opt: if no parent check wether we're in the surface rect or not. If we're out stop
		if (! effects->m_ParentGroup) {
			VisualPath tmp;
			tmp.AddRectangle(final.left, final.top, final.width, final.height);
			tmp.GetBoundingRect(&rc, &effects->transform, 0);
			if (ttb && (rc.top + rc.height > effects->m_pSurface->GetClipper().top + effects->m_pSurface->GetClipper().height) ) {
				break;
			}
			else if (!ttb && (rc.top > effects->m_pSurface->GetClipper().top) ) {
				break;
			}
		}
	}

	//update bounds
	m_Bounds.width = final.width;
	m_Bounds.left = final.left;
	//add secure offset in bounds
	float h = m_Bounds.height;
	m_Bounds.height *= 1.05;
	m_Bounds.top += (m_Bounds.height - h)/2;
	free(lines);
}



void TextProxy::BuildVerticalGraph(Effects *effects)
{

	int i;
	TextLine *lines;

	//get font info
	const char * fontName = GetFSfamily();
	float fontSize = GetFSsize();
	if (fontSize <= 0) fontSize = 12;
    
	const char * pStyles = GetFSstyle();
	//get spacing
	float line_spacing = GetFSspacing();
	line_spacing *= fontSize;

	BOOL ltr = GetFSleftToRight();

	m_Path.SetFontEngine(GetPresenter()->GetFontEngine());

	if (! m_Path.SetFont(fontName, fontSize, pStyles)) {
		char msg[1024];
		sprintf(msg, "Couldn't find font %s", fontName);
		GetPresenter()->GetExecutive()->GetApp()->OnFatalError(OTHER_ERROR, msg);
		return;
	}

	//compute overall bounding box size
	float tot_width = 0;
	float tot_height = 0;
	float max_ext = GetFSMaxExtend();
	int NumLines = GetText()->string.GetLength();
	
	//alloc a struc
	lines = (TextLine *) malloc(sizeof(TextLine)*NumLines);
		
	char letter[2];
	letter[1] = 0;

	//get metrics
	float font_line_spacing;
	m_Path.GetFontMetrics(m_ascent, m_descent, font_line_spacing);

	float char_space = m_ascent + m_descent;

	for (i=0; i < NumLines; i++) {
		lines[i].pString = GetText()->string[i];
		lines[i].y_scaling = lines[i].x_scaling = 1.0;

		//what should be the character spacing in this case ?? just use line spacing, VRML spec is
		//quite unclear here...
		lines[i].height = GetText()->string[i].GetLength() * char_space;
		if (!lines[i].height) continue;

		//check any length info
		float length = GetLength(i);
		if (length > 0) 
			lines[i].y_scaling = length/lines[i].height;
		if (tot_height < lines[i].height * lines[i].y_scaling) tot_height = lines[i].height * lines[i].y_scaling;
	}

	//compute width. Note we miss the info on avg char size ...
	tot_width = NumLines*line_spacing;
	//store logical width
	m_Bounds.width = tot_width;

	//apply max_extend. VRML doesn't say if all lines should be scale with the same factor... 
	//we assume yes for that will look better
	float max_scale = 1.0;
	if (max_ext>0 && tot_height>max_ext) {
		max_scale = max_ext/tot_height;
		tot_height = max_ext;
	}
	
	float start_x, start_y;
	CString major, minor;
	major = GetFSjustify(0);
	minor = GetFSjustify(1);

	BOOL ttb = GetFStopToBottom();
	
	//minor-justification is for the whole layout and store logical left
	if (minor=="MIDDLE") {
		if (ltr) {
			start_x = -tot_width/2;
			m_Bounds.left = start_x;
		} else {
			start_x = tot_width/2 - line_spacing;
			m_Bounds.left = - tot_width + line_spacing;
		}
	}
	else if (minor=="END") {
		if (ltr) {
			start_x = -tot_width;
			m_Bounds.left = start_x;
		} else {
			start_x = tot_width-line_spacing;
			m_Bounds.left = 0;
		}
	}
	else {
		if (ltr) {
			start_x = 0;
			m_Bounds.left = start_x;
		} else {
			start_x = -line_spacing;
			m_Bounds.left = -tot_width;
		}
	}

	int int_major;
	if (major=="MIDDLE") {
		int_major = 0;
	} else if (major=="END") {
		int_major = 1;
	} else {
		int_major = 2;
	}

	BRect rc, final;
	final.MakeEmpty();
	for (i=0; i < NumLines; i++) {
		switch (int_major) {
		case 0:
			if (ttb) 
				start_y = lines[i].height/2;
			else
				start_y = -lines[i].height/2 + char_space;
			break;
		case 1:
			if (ttb)
				start_y = lines[i].height;
			else
				start_y = -lines[i].height + char_space;
			break;
		default:
			if (ttb)
				start_y = 0;
			else
				start_y = char_space;
			break;
		}

		//reverse string if right to left
		ZString st;
		const char *string = lines[i].pString;
		if (strlen(string)) {
			VisualPath *path = new VisualPath();
			path->SetFontEngine(GetPresenter()->GetFontEngine());
			path->SetFont(fontName, fontSize, pStyles);
			m_TextArray += path;
		
			if (!ttb) {
				int len = strlen(string);
				for (int k=0; k<len;k++) st[k] = string[len-k-1];
				string = st;
			}
			//draw one by one...
			for (int k=0; k<GetText()->string[i].GetLength(); k++) {
				letter[0] = lines[i].pString[k];
				path->AddText(letter, start_x, start_y, lines[i].x_scaling, lines[i].y_scaling*max_scale, lines[i].height);
				
				if (ttb)
					start_y -= char_space;
				else
					start_y += char_space;
			}

			//also collect bounds info
			path->GetBoundingRect(&rc, NULL, 0, 0);
			final.Union(&rc);
		}

		if (ltr) {
			start_x += line_spacing;
		} else {
			start_x -= line_spacing;
		}

		//little opt: if no parent check wether we're in the surface rect or not. If we're out stop
		if (effects->m_ParentGroup) {
			VisualPath tmp;
			tmp.AddRectangle(final.left, final.top, final.width, final.height);
			tmp.GetBoundingRect(&rc, &effects->transform, 0);
			if (ltr && (rc.left + rc.width > effects->m_pSurface->GetClipper().left + effects->m_pSurface->GetClipper().width) ) {
				break;
			}
			else if (!ltr && (rc.left < effects->m_pSurface->GetClipper().left) ) {
				break;
			}
		}

	}

	//update bounds
	m_Bounds.height = final.height;
	m_Bounds.top = final.top;

	free(lines);
}
