/*
	This software module was originally developed by ENST in the course of
development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.
	This software module is an implementation of a part of one or more MPEG-4
Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1) 
standard. ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license 
to this software module or modifications thereof for use in hardware or software 
products claiming conformance to the MPEG-4 Systems (ISO/IEC 14496-1). 
Those intending to use this software module in hardware or software products are 
advised that its use may infringe existing patents.
	The original developer of this software module and his/her company, the 
subsequent editors and their companies, and ISO/IEC have no liability for use of 
this software module or modifications thereof in an implementation. Copyright is 
not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
	ENST retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using the code 
for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright    
notice must be included in all copies or derivative works.              
						Copyright  2002.
*/

///////////////////////////////////////////////////////////////////////////
// File:  TextProxy.h
//
// Authors: Jean Le Feuvre ENST 2002
//
//
///////////////////////////////////////////////////////////////////////////


#ifndef TEXT_PROXY_H
#define TEXT_PROXY_H


#include "GeometryNode.h"


class TextProxy : public GeometryNode
{
public:
	TextProxy(MediaObject* pMO) : GeometryNode(pMO) {}
	~TextProxy() 
	{
		ResetPath();
	}

	// Return a pointer to the Text2D node that this node is the proxy for
	Text* GetText() const { return (Text*)GetObject(); }

	// Functions inherited from the base class
	virtual void Render(Effects *effects);
	virtual void Draw(DrawableContext *pContext);

	FontStyle *GetFontStyle() const { 
		MediaObject *pNode = GetText()->fontStyle;
		if (pNode && pNode->GetNodeCode() == CODE_FontStyle) return (FontStyle*)pNode;
		return NULL;
	}
	XFontStyle *GetAdvancedFontStyle() const { 
		MediaObject *pNode = GetText()->fontStyle;
		if (pNode && pNode->GetNodeCode() == CODE_XFontStyle) return (XFontStyle *)pNode;
		return NULL;
	}

	int GetLength(int index) const
	{ 
		if(index<GetText()->length.GetLength()) return (int)GetText()->length[index];
		return -1;
	}

	//this is called to check if the point is over the shape when in its BRect
	//we always say true otherwise the cursor has to be over a letter...
	virtual BOOL IsPointOver(DrawableContext * /*pContext*/, float /*x*/, float /*y*/) { return TRUE; }

	// Functions inherited from the base class
	void GetAscentAndDescent(float &ascent,float &descent);

	virtual void GetOriginalBounds(BRect &bounds);

	virtual void AddToClipper(VisualClipper *, DrawableContext *, BOOL , BOOL );


	//
	// Protected implementation details
	//
protected:

	void BuildGraph(Effects *effects);
	void BuildVerticalGraph(Effects *effects);

	// Functions to get the value from the FontStyle node, which
	// use the default values if this node is missing
	const char * GetFSfamily() const;
    BOOL GetFShorizontal() const;
	const char * GetFSjustify(int i) const;
	const char* GetFSlanguage() const;
	BOOL GetFSleftToRight() const;
	float GetFSsize() const;
	float GetFSspacing() const;
	const char * GetFSstyle() const;
	int GetFStopToBottom() const;
	float GetFSMaxExtend() const;

	
	UINT m_justifyFlag;
	float m_ascent, m_descent;	
	BRect m_Bounds;

	void ResetPath();
	void GetBounds(DrawableContext *pContext);

	//Gdiplus is really bad at collecting large amount of objects in a single path, thus we keep 
	//each text line in a dedicated path object. This should also help us a lot with layout
	ZArray<VisualPath *> m_TextArray;
};


#endif // TEXT_PROXY_H
