/*
	This software module was originally developed by ENST in the course of
development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.
	This software module is an implementation of a part of one or more MPEG-4
Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1) 
standard. ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license 
to this software module or modifications thereof for use in hardware or software 
products claiming conformance to the MPEG-4 Systems (ISO/IEC 14496-1). 
Those intending to use this software module in hardware or software products are 
advised that its use may infringe existing patents.
	The original developer of this software module and his/her company, the 
subsequent editors and their companies, and ISO/IEC have no liability for use of 
this software module or modifications thereof in an implementation. Copyright is 
not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
	ENST retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using the code 
for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright    
notice must be included in all copies or derivative works.              
						Copyright  2002.
*/

///////////////////////////////////////////////////////////////////////////
// File:  TimedNode.h
//
// Author: Jean Le Feuvre ENST 2002
//
//	Description
// This is a pure virtual interface to all time-dependant nodes
// (TimeSensor, MovieTexture, AudioSource, AudioClip, AnimationStream, AudioBuffer)
// all these classes must derive from it
//
///////////////////////////////////////////////////////////////////////////


/*
	NOTE: 
		* since v1.2.1, startTime, stopTime and loop comply with VRML notes on time-dependent nodes
	for media nodes (nodes using OD)
		* in order to stay as accurate as possible for time fields (startTime and stopTime), the time value
	is updated as soon as modified (OnUpdate) with the current scene time (time of BIFS stream owning the root)
	This works because WE DO NOT use FlexTime, otherwise the top-level FlexTime parent in the scene should be 
	located to get the flex time context, and flexTime should be propagated at a fixed frame-rate, which we want
	to avoid...
*/


#ifndef TIMEDNODE_H
#define TIMEDNODE_H

/*
	this is used by timed nodes controling ODs (AudioClip/Source, MovieTexture and AnimationStream)
in case of loops: IM1 architecture forces us to restart the OD in 2 rendering cycle in case the OD
is used by 2 nodes otherwise the object would never be closed
*/
enum {
	ST_None,
	ST_Stoped,
	ST_Opened,
};

class TimedNode
{
public:
	virtual void UpdateTimeInfo() = 0;

protected:

#if 0
	/*NO support for flexTime*/
	PresenterContext m_FlexTimeContext;
#endif


	//these are used by all instances to keep track of registration state with the presenster
	BOOL m_bIsRegistered;
	BOOL m_bNeedsUnregister;
};

#endif //TIMEDNODE_H