/*
	This software module was originally developed by ENST in the course of
development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.
	This software module is an implementation of a part of one or more MPEG-4
Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1) 
standard. ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license 
to this software module or modifications thereof for use in hardware or software 
products claiming conformance to the MPEG-4 Systems (ISO/IEC 14496-1). 
Those intending to use this software module in hardware or software products are 
advised that its use may infringe existing patents.
	The original developer of this software module and his/her company, the 
subsequent editors and their companies, and ISO/IEC have no liability for use of 
this software module or modifications thereof in an implementation. Copyright is 
not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
	ENST retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using the code 
for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright    
notice must be included in all copies or derivative works.              
						Copyright  2002.
*/

///////////////////////////////////////////////////////////////////////////
// File: TouchSensorProxy.cpp
//
// Author: Jean Le Feuvre - ENST 2002
//
// Description:
//   proxy object for TouchSensor node.
//
// Created: August 2002.
//
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include "Nodes-INIT.h"
#include "TouchSensorProxy.h"


TouchSensorProxy* Presenter::InitTouchSensor(TouchSensor* pObj)
{
	return new  TouchSensorProxy(pObj);
}

BOOL TouchSensorProxy::OnUserEvent(EventInfo const* pEvent, Matrix2D *pSensorMatrix)
{
	// Only generate events if this node is enabled
	if (!(Bool) GetTouchSensor()->enabled) return FALSE;

	//out of the shape
	if (pEvent->m_pContext == NULL) {

		// isOver=FALSE event in any case
		if ((BOOL)(Bool)GetTouchSensor()->isOver)
			GetTouchSensor()->isOver = FALSE;

		//if we're active we may not deactivate like that. Check mouse state
		if ((BOOL)(Bool)GetTouchSensor()->isActive) {
			//release
			if (pEvent->m_event == EVENT_MOUSE_UP) {
				GetTouchSensor()->isActive = FALSE;
				
				//we don't generate a touchTime because  (VRML quote) 
				//"the pointing device shall be over the geometry (isOver is TRUE)"

				m_mouseDown = FALSE;
				GetPresenter()->UnregisterEventHandler(this);

				return TRUE;
			}
			//otherwise we shall keep tracking the mouse so don't unregister
			return FALSE;
		} 
		//we were not active just unregister
		m_mouseDown = FALSE;
		GetPresenter()->UnregisterEventHandler(this);
		return FALSE;
	}
	assert(pSensorMatrix);

	// Generate an IsOver event if this is the first time that the mouse has moved here
	if ((pEvent->m_event == EVENT_MOUSE_MOVE) && ((BOOL)(Bool)GetTouchSensor()->isOver == FALSE)) {
		GetTouchSensor()->isOver = TRUE;
		// register as active
		GetPresenter()->RegisterEventHandler(this);
	
	}


	// If the button is first pressed, then generate an isActive = TRUE event, and grab all movement events
	if ((pEvent->m_event == EVENT_MOUSE_DOWN) && (!m_mouseDown)) {
		GetTouchSensor()->isActive = TRUE;
		m_mouseDown = TRUE;
	}

	// If the button is first released, then generate an isActive = FALSE event, and ungrab all movement events.
	if ((pEvent->m_event == EVENT_MOUSE_UP) && (m_mouseDown)) {
		GetTouchSensor()->isActive = FALSE;
		m_mouseDown = FALSE;
		// Also generate a touchTime event (in seconds)
		double now = GetFlexTime() / 1000.0;
		GetTouchSensor()->touchTime = now;
	}
	// In all cases generate a hitPoint_changed event
	float X, Y;
	X = pEvent->m_x;
	Y = pEvent->m_y;
	pSensorMatrix->ReverseApply(X, Y);
	GetTouchSensor()->hitPoint_changed = Vec3f(X, Y, 0);
	
	return TRUE;
}

