/*
	This software module was originally developed by ENST in the course of
development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.
	This software module is an implementation of a part of one or more MPEG-4
Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1) 
standard. ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license 
to this software module or modifications thereof for use in hardware or software 
products claiming conformance to the MPEG-4 Systems (ISO/IEC 14496-1). 
Those intending to use this software module in hardware or software products are 
advised that its use may infringe existing patents.
	The original developer of this software module and his/her company, the 
subsequent editors and their companies, and ISO/IEC have no liability for use of 
this software module or modifications thereof in an implementation. Copyright is 
not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
	ENST retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using the code 
for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright    
notice must be included in all copies or derivative works.              
						Copyright  2002.
*/

///////////////////////////////////////////////////////////////////////////
// File:  Transform2DProxy.cpp
//
// Author: Jean Le Feuvre - ENST 2002
//
// Description:
//   proxy object for Transform2D node.
//
// Created: August 2002.
//
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include "Nodes-INIT.h"
#include "Transform2DProxy.h"
#include "Presenter.h"


Transform2DProxy* Presenter::InitTransform2D(Transform2D* pObj)
{
	return new  Transform2DProxy(pObj);
}

void Transform2DProxy::GetMatrix(Matrix2D *mat)
{
	mat->SetIdentity();

	//first apply scaling
	mat->AddScale( ((Vec2f)GetTransform2D()->scale).x, ((Vec2f)GetTransform2D()->scale).y, GetTransform2D()->scaleOrientation);
	//then rotation
	mat->AddRotation(GetTransform2D()->rotationAngle, ((Vec2f)GetTransform2D()->center).x, ((Vec2f)GetTransform2D()->center).y);
	//finally translation
	mat->AddTranslation(((Vec2f)GetTransform2D()->translation).x, ((Vec2f)GetTransform2D()->translation).y);
}

void Transform2DProxy::Render(Effects *effects)
{
	Effects *newEffects = new Effects(effects);

	//get local matrix
	GetMatrix(&newEffects->transform);

	//and move to parent system
	newEffects->transform.AddTransform(&effects->transform);

	// Apply these new effects to the children nodes
	RenderChildren(newEffects);

	delete newEffects;
}