/*
	This software module was originally developed by ENST in the course of
development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.
	This software module is an implementation of a part of one or more MPEG-4
Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1) 
standard. ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license 
to this software module or modifications thereof for use in hardware or software 
products claiming conformance to the MPEG-4 Systems (ISO/IEC 14496-1). 
Those intending to use this software module in hardware or software products are 
advised that its use may infringe existing patents.
	The original developer of this software module and his/her company, the 
subsequent editors and their companies, and ISO/IEC have no liability for use of 
this software module or modifications thereof in an implementation. Copyright is 
not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
	ENST retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using the code 
for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright    
notice must be included in all copies or derivative works.              
						Copyright  2002.
*/

///////////////////////////////////////////////////////////////////////////
// File:  Traversable.cpp
//
// Author: Jean Le Feuvre - ENST 2002
//
// Created: August, 2002.
//
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include <assert.h>
#include "Traversable.h"
#include "Presenter.h"
#include "MediaObj.h"


Effects::Effects()
{
	m_ParentGroup = NULL;
	m_pAudioParent = NULL;
	m_pAppearanceProxy = NULL;
	m_pGeometryProxy = NULL;
	m_pSurface = NULL;
	m_pColorTransform = NULL;
	m_bInvalidateAll = FALSE;
	m_ClipperMode = 0;
	m_isSwitchedOff = FALSE;
	m_pClipper = NULL;

	transform.SetIdentity();
}

Effects::Effects(Effects *parent)
{
	m_bDrawBackground = parent->m_bDrawBackground;
	m_bShouldRebuild = parent->m_bShouldRebuild;
	m_bUseBoundTracking = parent->m_bUseBoundTracking;
	m_pSurface = parent->m_pSurface;
	m_ParentGroup = parent->m_ParentGroup;
	m_pAudioParent = parent->m_pAudioParent;

	m_pAppearanceProxy = NULL;
	m_pGeometryProxy = NULL;
	m_pClipper = NULL;
	m_ClipperMode = 0;

	m_isSwitchedOff = parent->m_isSwitchedOff;
	m_pColorTransform = NULL;
	m_bInvalidateAll = parent->m_bInvalidateAll;

	if (parent->m_pColorTransform) {
		m_pColorTransform = new ColorMatrix();
		m_pColorTransform->Set(*parent->m_pColorTransform);
	}

	transform.Set(parent->transform);
	
	//duplicate all handlers
	for(int i=0; i<parent->m_Handlers.GetLength();i++) {
		m_Handlers += new SensorContext(parent->m_Handlers[i]->m_pHandler, &parent->m_Handlers[i]->m_Matrix);
	}

	//if we have a clipper clone it
	if (parent->m_pClipper) m_pClipper = parent->m_pClipper->Clone();
}

Effects::~Effects()
{
	ResetHandlers();
	if (m_pColorTransform) delete m_pColorTransform;
	if (m_pClipper) delete m_pClipper;
}

void Effects::AddUserEventHandler(UserEventHandlerProxy *handler, Matrix2D *matrix)
{
	if (!handler) return;
	m_Handlers += new SensorContext(handler, matrix);
}

void Effects::RemoveHandler(UserEventHandlerProxy *handler)
{
	if (!handler) return;
	for (int i=0; i<m_Handlers.GetLength(); i++) {
		if (m_Handlers[i]->m_pHandler == handler) {
			SensorContext *ptr = m_Handlers[i];
			m_Handlers.DeleteFrom(i);
			delete ptr;
			return;
		}
	}
}


void Effects::ResetHandlers()
{
	while (m_Handlers.GetLength()) {
		SensorContext *ptr = m_Handlers[0];
		m_Handlers.DeleteFrom(0);
		delete ptr;
	}
}

Traversable::Traversable(MediaObject* pMO) : m_pMediaObject(pMO)
{
	m_pPresenter = (Presenter*) pMO->GetExecutive()->GetPresenter();
}

DWORD Traversable::GetFlexTime(PresenterContext * /*flexTimeContext*/)
{
	return GetObject()->GetRoot()->GetSceneTime();
}