/*
	This software module was originally developed by ENST in the course of
development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.
	This software module is an implementation of a part of one or more MPEG-4
Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1) 
standard. ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license 
to this software module or modifications thereof for use in hardware or software 
products claiming conformance to the MPEG-4 Systems (ISO/IEC 14496-1). 
Those intending to use this software module in hardware or software products are 
advised that its use may infringe existing patents.
	The original developer of this software module and his/her company, the 
subsequent editors and their companies, and ISO/IEC have no liability for use of 
this software module or modifications thereof in an implementation. Copyright is 
not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
	ENST retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using the code 
for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright    
notice must be included in all copies or derivative works.              
						Copyright  2002.
*/

///////////////////////////////////////////////////////////////////////////
// File: ViewportProxy.cpp
//
// Author: Jean Le Feuvre
//
// Modified: 
//
// Description:
//   proxy object for Viewport node.
//
// Created: September 2002.
//
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include "NODES-INIT.h"
#include "ViewportProxy.h"
#include "Presenter.h"
#include "bindable.h"
#include "App.h"
#include "GroupNodeProxy.h"


ViewportProxy *Presenter::InitViewport(Viewport *pObj)
{
	return new ViewportProxy(pObj);
}

void ViewportProxy::DeleteObject() 
{
	while (m_Stacks.GetLength() ) {
		m_Stacks.DeleteFrom(0);
	}
}

void ViewportProxy::Render(Effects *effects)
{
	//register top if needed
	if (m_bFirstTime) {
		BindableStack *stack = GetStack(effects);
		if (stack->IsOnTop(this))
			GetViewport()->isBound = TRUE;
		
		m_bFirstTime = FALSE;
	}
}


BindableStack *ViewportProxy::GetStack(Effects *effects)
{
	BindableStack *stack = NULL;
	
	if (effects->m_ParentGroup) {
		stack = effects->m_ParentGroup->GetViewportStack();
	}
	if (!stack) {
		stack = effects->m_pSurface->GetViewportStack();
	}
	if (!stack) return NULL;

	for (int i=0; i<m_Stacks.GetLength(); i++) {
		if (stack == m_Stacks[i]) return stack;	
	}
	m_Stacks += stack;
	stack->AddBindable(this);
	GetPresenter()->GetExecutive()->GetApplication()->ViewportStackChanged(stack);
	return stack;
}

void ViewportINIT::OnUpdate()
{
	ViewportProxy *pVP = ((ViewportProxy*) GetRenderingObject());
	for (int i=0; i<pVP->m_Stacks.GetLength();i++) {
		GetExecutive()->GetApplication()->ViewportStackChanged(pVP->m_Stacks[i]);
	}
}

void Viewport::Onset_bind()
{
	ViewportProxy *pOnTop, *pVP;
	pVP = ((ViewportProxy*) GetRenderingObject());

	//notify all stacks using this node
	for (int i=0; i<pVP->m_Stacks.GetLength();i++) {
		BindableStack *pStack = pVP->m_Stacks[i];
		BOOL isOnTop = pStack->IsOnTop(pVP);

		GetPresenter()->InvalidateScene(NULL);
	
		if (! (Bool) set_bind) {
			//generate an isBound=FALSE (only in the first iteration)
			if ((Bool) isBound) 
				isBound = FALSE;

			//if on top remove from top and pop the stack
			if (isOnTop) {
				pStack->RemoveFromTop(pVP);
				pOnTop = (ViewportProxy *) pStack->GetTop();
				if ((pOnTop != NULL) && (pOnTop != pVP))
					pOnTop->GetViewport()->isBound=TRUE;
			}
		} else {
			//we're getting active
			if (! (Bool) isBound) {
				isBound = TRUE;
				//we don't have access to a flexTime context so no flexTime for viewPort
				bindTime = GetPresenter()->GetTime();
			}

			//if we're already on top do nothing
			if (!isOnTop) {
				//notify previous if any
				pOnTop = (ViewportProxy *) pStack->GetTop();
				if (pOnTop != NULL) 
					pOnTop->GetViewport()->isBound=FALSE;

				pStack->MoveToTop(pVP);
			}
		}
	}
}


void ViewportProxy::SetupEffects(Effects *effects, float parent_width, float parent_height, BRect &FinalClipper)
{
	if (!parent_width || !parent_height) return;

	Viewport *pVP = GetViewport();

	//setup orientation and position. Note: this acts as TextureTransform
	//on the final result, eg viewport rotated of PI/2 means that nodes under viewport
	//are rotated by -PI/2, same for translation.
	Matrix2D mat;
	Vec2f vec = (Vec2f) pVP->position;
	mat.AddTranslation(-1 * vec.x, -1 * vec.y);

	mat.AddRotation(-1 * pVP->orientation);
	
	effects->transform.AddTransform(&mat);
	mat.Set(effects->transform);	

	//compute scaling ratio
	VisualPath path;
	vec = (Vec2f) pVP->size;
	path.AddRectangle(vec.x, vec.y);
	BRect bounds;
	path.GetBoundingRect(&bounds, &mat, 0);

	float scale_x, scale_y, AR;
	AR = parent_height / parent_width;
	
	FinalClipper.width = bounds.width;
	FinalClipper.height = bounds.height;

	switch (pVP->fit) {
	//covers all area and respect aspect ratio
	case 2:
		if (bounds.width/parent_width > bounds.height/parent_height) {
			bounds.width *= parent_height/bounds.height;
			bounds.height = parent_height;
		} else {
			bounds.height *= parent_width/bounds.width;
			bounds.width = parent_width;
		}
		break;
	//fits inside the area and respect AR
	case 1:
		if (bounds.width/parent_width > bounds.height/parent_height) {
			bounds.height *= parent_width/bounds.width;
			bounds.width = parent_width;
		} else {
			bounds.width *= parent_height/bounds.height;
			bounds.height = parent_height;
		}
		break;
	//fit entirely: nothing to change
	case 0:
		bounds.width = parent_width;
		bounds.height = parent_height;
		break;
	default:
		return;
	}
	scale_x = bounds.width / FinalClipper.width;
	scale_y = bounds.height / FinalClipper.height;

	//init clipper
	FinalClipper.width = bounds.width;
	FinalClipper.height = bounds.height;
	FinalClipper.left = - bounds.width/2;
	FinalClipper.top = bounds.height/2;

	
	mat.SetIdentity();
	if (!pVP->fit) {
		mat.AddScale(scale_x, scale_y, 0.0);
		effects->transform.AddTransform(&mat);
		return;
	}

	float trans_x, trans_y;

	//setup x-alignment
	switch (pVP->alignment[0]) {
	//left align: 
	case -1:
		trans_x = bounds.width/2 - parent_width/2;
		break;
	//right align
	case 1:
		trans_x = parent_width/2 - bounds.width/2;
		break;
	//center
	case 0:
	default:
		trans_x = 0;
		break;
	}

	//setup y-alignment
	switch (pVP->alignment[1]) {
	//left align: 
	case -1:
		trans_y = bounds.height/2 - parent_height/2;
		break;
	//right align
	case 1:
		trans_y = parent_height/2 - bounds.height/2;
		break;
	//center
	case 0:
	default:
		trans_y = 0;
		break;
	}

	mat.AddScale(scale_x, scale_y, 0.0);
	mat.AddTranslation(trans_x, trans_y);
	effects->transform.AddTransform(&mat);

	FinalClipper.left += trans_x;
	FinalClipper.top += trans_y;
}
