//This software module was originally developed by ENST in the course of   //
//development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.             //
//This software module is an implementation of a part of one or more MPEG-4 //
//Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems        //
//(ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 Systems     //
//(ISO/IEC 14496-1) free license to this software module or modifications   //
//thereof for use in hardware or software products claiming conformance to  //
//the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use this software//
//module in hardware or software products are advised that its use may      //
//infringe existing patents.                                                //
//The original developer of this software module and his/her company, the   //
//subsequent editors and their companies, and ISO/IEC have no liability     //
//for use of this software module or modifications thereof in an            //
//implementation. Copyright is not released for non MPEG-4 Systems          //
//(ISO/IEC 14496-1) conforming products. ENST retains full right     //
//to use the code for his/her own purpose, assign or donate the code to a   //
//third party and to inhibit third parties from using the code for non      //
//MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright      //
//notice must be included in all copies or derivative works.                //
//Copyright  2002.                                                         //
//////////////////////////////////////////////////////////////////////////////
//
// File: VisualClipper.cpp
//
// Authors: Jean Le Feuvre ENST 2002
//
// Modified: 
//
// Description:
//   generic shape clipper
//
// Created: October 2002
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include "VisualClipper.h"

VisualClipper::VisualClipper()
{
	m_pRegion = NULL;
}

VisualClipper::~VisualClipper()
{
	if (m_pRegion) delete m_pRegion;
}

BOOL VisualClipper::Set(VisualPath *path, Matrix2D *mat, BOOL bIsClipping)
{
	if (!path) return FALSE;
	if (m_pRegion) delete m_pRegion;
	m_pRegion = new Gdiplus::Region(&path->m_Path);
	if (mat) {
		m_pRegion->Transform(&mat->m_Matrix);
	}
	m_bIsClipping = bIsClipping;
	return TRUE;
}

void VisualClipper::Reset()
{
	if (m_pRegion) delete m_pRegion;
	m_pRegion = NULL;
}

VisualClipper *VisualClipper::Clone()
{
	VisualClipper *clone = new VisualClipper();
	clone->m_pRegion = m_pRegion->Clone();
	clone->m_bIsClipping = m_bIsClipping;
	return clone;
}

BOOL VisualClipper::Transform(Matrix2D *mat)
{
	if (!mat || !m_pRegion) return FALSE;
	m_pRegion->Transform(&mat->m_Matrix);
	return TRUE;
}

BOOL VisualClipper::AddPath(VisualPath *path, Matrix2D *mat, BOOL bIsClipping, BOOL bUseXOR)
{
	//first path added
	if (!m_pRegion) return Set(path, mat, bIsClipping);
	
	if (!path) return FALSE;
	Gdiplus::GraphicsPath *pPath = path->m_Path.Clone();
	if (mat) pPath->Transform(&mat->m_Matrix);
	
	if (m_bIsClipping) {
		//intersection: all points in region and in path
		if (bIsClipping) {
			if (bUseXOR) {
				m_pRegion->Xor(pPath);
			} else {
				m_pRegion->Intersect(pPath);
			}
		}
		//exclude: all points in region and not in path
		else {
			m_pRegion->Exclude(pPath);
		}
	}
	//this is a reversed clipper
	else {
		//Union: all points in region and in path, since the clipper is reversed
		if (!bIsClipping) {
			if (bUseXOR) {
				m_pRegion->Xor(pPath);
			} else {
				m_pRegion->Union(pPath);
			}
		} 
		//all points in path and not in region.
		else {
			m_pRegion->Complement(pPath);
			//reverse the clipper since we're now clipping
			m_bIsClipping = TRUE;
		}
	}
	delete pPath;
	return TRUE;
}

