/*
	This software module was originally developed by ENST in the course of
development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.
	This software module is an implementation of a part of one or more MPEG-4
Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1) 
standard. ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license 
to this software module or modifications thereof for use in hardware or software 
products claiming conformance to the MPEG-4 Systems (ISO/IEC 14496-1). 
Those intending to use this software module in hardware or software products are 
advised that its use may infringe existing patents.
	The original developer of this software module and his/her company, the 
subsequent editors and their companies, and ISO/IEC have no liability for use of 
this software module or modifications thereof in an implementation. Copyright is 
not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
	ENST retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using the code 
for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright    
notice must be included in all copies or derivative works.              
						Copyright  2002.
*/

///////////////////////////////////////////////////////////////////////////
// File: VisualPath.h
//
// Author: Jean Le Feuvre
//
// Description:
//		Object used to store visual path (curves, points, lines)
//
// Created: August 2002
//
///////////////////////////////////////////////////////////////////////////

#include "BRect.h"

#ifndef VISUALPATH_H_
#define VISUALPATH_H_

#include "OsmoseGdiplus.h"
#include "Matrix2D.h"

class FontEngine;


class VisualPath
{

public:
	VisualPath();
	
	void Reset(unsigned int quality_level=0);
	//add centered circle
	void AddCircle(float radius);
	//add centered ellipse
	void AddEllipse(float a_axis, float b_axis);
	//add centered rectangle
	void AddRectangle(float width, float height);
	//add non-centered recatngle
	void AddRectangle(float x, float y, float width, float height);
	//add line
	void AddLine(float startX, float startY, float endX, float endY);
	//add bezier curve (first and last points are the curve ones, others are control points)
	void AddBezier(PointF *points, int NumPoints, float fineness);
	//add elliptic arc
	void AddArc(PointF start, PointF end, PointF focal_a, PointF focal_b, BOOL ClockWise=FALSE);
	//add point
	void AddPoint(float xPos, float yPos);
	//start a new figure in the path
	void StartNewFigure();
	//closes current figure in the path
	void CloseFigure();
	//gets the bounding rect of this path
	//matrix: optional, if not set the path is not transformed
	//pen_width: width used for the pen
	//pen_is_scaled: to use if the scaling is used for the line drawing too
	void GetBoundingRect(BRect *pClipBRect, Matrix2D *matrix, float pen_width, BOOL pen_is_scaled=FALSE);

	BOOL IsPointInPath(float x, float y, Matrix2D *matrix);

	//transform the path. Transformation CANNOT be undone
	void Transform(Matrix2D *mat);

	VisualPath *Clone();

	//text manips

	//sets current working font
	//fontName, fontSize: name /size of the font
	//styles: VRML/MPEG string describing styles (bold, italic, underline, ...)
	//returns FALSE if no font found
	BOOL SetFont(LPCTSTR fontName, float fontSize, LPCTSTR styles);
	void GetTextSize(LPCTSTR string, float &width, float &height);
	//NB: we don't use emHeight since line spacing shall be done by hand in MPEG4
	void GetFontMetrics(float &ascent, float &descent, float &lineSpacing);
	//x and y scaling are used for length control of the string
	//left and top: top-left corner where to start drawing
	//ascent is the offset between @top and baseline. This is because GDIplus starts drawing from top
	//whereas the itype font engine gives us path data starting from baseline
	void AddText(LPCTSTR pString, float left, float top, float x_scaling, float y_scaling, float ascent);

	//the font size will be adjusted to the size chosen so that 
	// inputFontSize = finalFont.ascent+finalFont.descent
	//returns the final size
	float AdjustFontSize(float inSize);

	//exposed as public because the physical renderer needs access to that, and it is likely to change accross
	//different platforms...
	Gdiplus::GraphicsPath m_Path;

	BOOL m_bHasText;

	void SetFontEngine(FontEngine *ptr) {m_pFE = ptr;}

private:
	unsigned int m_BezierResolution;
	float m_FontSize;
	int m_FontStyle;

	Gdiplus::FontFamily *m_pFont;

	FontEngine *m_pFE;

	/*special versions when using font engine*/
	BOOL SetFontFE(LPCTSTR fontName, float fontSize, LPCTSTR styles);
	void GetTextSizeFE(LPCTSTR string, float &width, float &height);
	void GetFontMetricsFE(float &ascent, float &descent, float &lineSpacing);
	void AddTextFE(LPCTSTR pString, float left, float top, float x_scaling, float y_scaling, float ascent);
	float AdjustFontSizeFE(float inSize);
};



#endif	//VISUALPATH_H_
