/*
	This software module was originally developed by ENST in the course of
development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.
	This software module is an implementation of a part of one or more MPEG-4
Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1) 
standard. ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license 
to this software module or modifications thereof for use in hardware or software 
products claiming conformance to the MPEG-4 Systems (ISO/IEC 14496-1). 
Those intending to use this software module in hardware or software products are 
advised that its use may infringe existing patents.
	The original developer of this software module and his/her company, the 
subsequent editors and their companies, and ISO/IEC have no liability for use of 
this software module or modifications thereof in an implementation. Copyright is 
not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
	ENST retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using the code 
for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright    
notice must be included in all copies or derivative works.              
						Copyright  2002.
*/

///////////////////////////////////////////////////////////////////////////
// File: VisualPathText.cpp
//
// Author: Jean Le Feuvre - ENST 2002
//
// Description:
//		Text handling
//
// Created: January 2003
//
///////////////////////////////////////////////////////////////////////////

#include "defs.h"
#include "ztl.h"
#include "VisualPath.h"
#include "FontEngine.h"

#define MAX_TEST_SIZE	5000

/*underline/strikeout width ratio: the line drawn will have 2 * fontSize / IT_STRIKE height */
#define IT_STRIKE	40

BOOL VisualPath::SetFontFE(LPCTSTR fontName, float fontSize, LPCTSTR styles)
{
	char *name = NULL;
	m_FontSize = 0.0;

	if (!m_pFE) return FALSE;

	m_FontStyle = 0;
	if (styles) {
		if (strstr(styles, "BOLDITALIC")) m_FontStyle = STYLE_BOLD | STYLE_ITALIC;
		else if (strstr(styles, "BOLD")) m_FontStyle = STYLE_BOLD;
		else if (strstr(styles, "ITALIC")) m_FontStyle = STYLE_ITALIC;
		if (strstr(styles, "UNDERLINED")) m_FontStyle |= STYLE_UNDERLINED;
		if (strstr(styles, "STRIKEOUT")) m_FontStyle |= STYLE_STRIKEOUT;
	}

	if (!fontName) name = "Thorndale for MPEG";
	else if (!strcmp(fontName, "SANS")) name = "Albany for MPEG";
	else if (!strcmp(fontName, "SERIF")) name = "Thorndale for MPEG";
	else if (!strcmp(fontName, "TYPEWRITER")) name = "Cumberland for MPEG";
	else name = (char *) fontName;

	/*load font*/
	if (!m_pFE->SetCurrentFont(name, m_FontStyle)) {
		if (!m_pFE->SetCurrentFont("Thorndale for MPEG", m_FontStyle))
			return FALSE;
	}
	
	m_FontSize = fontSize;
	m_pFE->SetCurrentSize(m_FontSize);
	return TRUE;
}

float VisualPath::AdjustFontSizeFE(float inSize)
{
	float as, des;
	int a, d, s;
	if (! m_pFE || !m_FontSize) return 0.0;
	m_pFE->GetCurrentMetrics(a, d, s);
	as = a; as /= s;
	des = d; des /= s;
	if (des<0) des = -des;
	m_FontSize = inSize / (as + des);
	m_pFE->SetCurrentSize(m_FontSize);
	return m_FontSize;
}


void VisualPath::GetFontMetricsFE(float &ascent, float &descent, float &lineSpacing)
{
	int a, d, s;
	ascent = descent = lineSpacing = 0;
	if (! m_pFE || !m_FontSize) return;
	m_pFE->GetCurrentMetrics(a, d, s);
	ascent = m_FontSize * a / s;
	descent = m_FontSize * d / s;
	if (descent<0) descent = -descent;
}

void VisualPath::GetTextSizeFE(LPCTSTR string, float &width, float &height)
{
	WCHAR wcText[MAX_TEST_SIZE];
	width = height = 0.0;
	if (!m_pFE || !m_FontSize) return;
	
	int len = strlen(string);
	if(len>=MAX_TEST_SIZE) return;

	MultiByteToWideChar(CP_UTF8, 0, string, strlen(string)+1, wcText, sizeof(wcText)/sizeof(wcText[0]) );
	m_pFE->GetTextSize(wcText, width, height);
}

void VisualPath::AddTextFE(LPCTSTR string, float left, float top, float x_scaling, float y_scaling, float ascent)
{
	float start_x, start_y, end_x, end_y;
	int len;
	WCHAR wcText[MAX_TEST_SIZE];
	if (!m_pFE || !m_FontSize) return;

	len = strlen(string);
	if(len>=MAX_TEST_SIZE) return;
	MultiByteToWideChar(CP_UTF8, 0, string, strlen(string)+1, wcText, sizeof(wcText)/sizeof(wcText[0]) );

	/*switch down bezier res*/
	unsigned int resol = m_BezierResolution;
	m_BezierResolution = 8;

	start_x = left;
	start_y = top - ascent;

	m_pFE->AddTextToPath(this, wcText, start_x, start_y, x_scaling, y_scaling, end_x, end_y);

	/*restore bezier res*/
	m_BezierResolution = resol;
	m_Path.SetFillMode(Gdiplus::FillModeWinding);
	m_bHasText = TRUE;

	/*underlined*/
	if (m_FontStyle & STYLE_UNDERLINED) {
		start_y = top - 9*ascent/8;
		StartNewFigure();
		AddLine(left, start_y + m_FontSize/IT_STRIKE, end_x, start_y + m_FontSize/IT_STRIKE);	
		AddLine(end_x, start_y + m_FontSize/IT_STRIKE, end_x, start_y - m_FontSize/IT_STRIKE);	
		AddLine(end_x, start_y - m_FontSize/IT_STRIKE, left, start_y - m_FontSize/IT_STRIKE);	
		CloseFigure();	
	} 
	/*strikeout*/
	if (m_FontStyle & STYLE_STRIKEOUT) {
		start_y = top - 3*ascent/4;
		StartNewFigure();
		AddLine(left, start_y + m_FontSize/IT_STRIKE, end_x, start_y + m_FontSize/IT_STRIKE);	
		AddLine(end_x, start_y + m_FontSize/IT_STRIKE, end_x, start_y - m_FontSize/IT_STRIKE);	
		AddLine(end_x, start_y - m_FontSize/IT_STRIKE, left, start_y - m_FontSize/IT_STRIKE);	
		CloseFigure();	
	} 
}


BOOL VisualPath::SetFont(LPCTSTR fontName, float fontSize, LPCTSTR styles)
{
	WCHAR wcFontName[MAX_TEST_SIZE];

	/*using font tool*/
	if (m_pFE) return SetFontFE(fontName, fontSize, styles);
		
	if (m_pFont) delete m_pFont;

	if(fontName && strlen(fontName)>=MAX_TEST_SIZE) fontName = NULL;

	if (!fontName) m_pFont = Gdiplus::FontFamily::GenericSerif()->Clone();
	else if (!strcmp(fontName, "SANS")) m_pFont = Gdiplus::FontFamily::GenericSansSerif()->Clone();
	else if (!strcmp(fontName, "SERIF")) m_pFont = Gdiplus::FontFamily::GenericSerif()->Clone();
	else if (!strcmp(fontName, "TYPEWRITER")) m_pFont = Gdiplus::FontFamily::GenericMonospace()->Clone();
	else {
		MultiByteToWideChar(CP_ACP, 0, fontName, strlen(fontName)+1, 
						wcFontName, sizeof(wcFontName)/sizeof(wcFontName[0]) );

		m_pFont = new Gdiplus::FontFamily(wcFontName);
		if (!m_pFont->IsAvailable()) {
			delete m_pFont;
			m_pFont = Gdiplus::FontFamily::GenericSerif()->Clone();
		}
	}

	//setup styles
	m_FontStyle = 0;
	if (styles) {
		if (strstr(styles, "BOLDITALIC")) m_FontStyle |= Gdiplus::FontStyleBoldItalic;
		else if (strstr(styles, "BOLD")) m_FontStyle |= Gdiplus::FontStyleBold;
		else if (strstr(styles, "ITALIC")) m_FontStyle |= Gdiplus::FontStyleItalic;
		
		if (strstr(styles, "UNDERLINED")) {
			m_FontStyle |= Gdiplus::FontStyleUnderline;
			m_Path.SetFillMode(Gdiplus::FillModeWinding);
		}
		if (strstr(styles, "STRIKEOUT")) {
			m_FontStyle |= Gdiplus::FontStyleStrikeout;
			m_Path.SetFillMode(Gdiplus::FillModeWinding);
		}
	}	
	m_FontSize = fontSize;
	return TRUE;
}

float VisualPath::AdjustFontSize(float inSize)
{

	/*using font tool*/
	if (m_pFE) return AdjustFontSizeFE(inSize);

	float as, des;
	as = m_pFont->GetCellAscent(inSize);
	as /= m_pFont->GetEmHeight(inSize);
	des = m_pFont->GetCellDescent(inSize);
	des /= m_pFont->GetEmHeight(inSize);
	m_FontSize = inSize / (as + des);
	return (float) (int) m_FontSize;
}


void VisualPath::GetTextSize(LPCTSTR string, float &width, float &height)
{

	/*using font tool*/
	if (m_pFE) {
		GetTextSizeFE(string, width, height);
		return;
	}
	
	WCHAR wcText[MAX_TEST_SIZE];
	Gdiplus::GraphicsPath path;

	width = height = 0.0;
	if (!m_pFont) return;

	if(strlen(string)>=MAX_TEST_SIZE) return;

	MultiByteToWideChar(CP_UTF8, 0, string, strlen(string)+1, 
					wcText, sizeof(wcText)/sizeof(wcText[0]) );

	Gdiplus::PointF pt(0, 0);
	Gdiplus::StringFormat format(Gdiplus::StringFormatFlagsNoWrap);
	path.AddString(wcText, -1, m_pFont, m_FontStyle, m_FontSize, pt, &format);
	
	Gdiplus::RectF rect;
	path.GetBounds(&rect, NULL, NULL);

	width = rect.Width;
	height = rect.Height;
}

void VisualPath::GetFontMetrics(float &ascent, float &descent, float &lineSpacing)
{
	/*using font tool*/
	if (m_pFE) {
		GetFontMetricsFE(ascent, descent, lineSpacing);
		return;
	}
	
	ascent = descent = 0.0;
	if (!m_pFont) return;

	ascent = m_FontSize*m_pFont->GetCellAscent(m_FontStyle);
	ascent /= m_pFont->GetEmHeight(m_FontStyle);
	descent = m_FontSize*m_pFont->GetCellDescent(m_FontStyle);
	descent /= m_pFont->GetEmHeight(m_FontStyle);
	lineSpacing = m_FontSize*m_pFont->GetLineSpacing(m_FontStyle);
	lineSpacing /= m_pFont->GetEmHeight(m_FontStyle);
}


void VisualPath::AddText(LPCTSTR string, float left, float top, float x_scaling, float y_scaling, float ascent)
{

	/*using font tool*/
	if (m_pFE) {
		AddTextFE(string, left, top, x_scaling, y_scaling, ascent);
		return;
	}

	WCHAR wcText[MAX_TEST_SIZE];
	Gdiplus::GraphicsPath path;

	if (!m_pFont) return;

	if(strlen(string)>MAX_TEST_SIZE) return;

	MultiByteToWideChar(CP_UTF8, 0, string, strlen(string)+1, 
					wcText, sizeof(wcText)/sizeof(wcText[0]) );
	
	Gdiplus::PointF pt(0, 0);

	//start a sub-graph for local transform (since GDI plus inverts Y)
	Gdiplus::StringFormat format(Gdiplus::StringFormatFlagsNoWrap);

	Gdiplus::Matrix mat;
	path.AddString(wcText, -1, m_pFont, m_FontStyle, m_FontSize, pt, &format);
	Gdiplus::RectF rect;
	path.GetBounds(&rect, NULL, NULL);

	mat.Scale(x_scaling, -y_scaling);
	mat.Translate(left - rect.X, -top);

	path.Transform(&mat);
	//add the new path and specify it is part of the current path
	m_Path.AddPath(&path, TRUE);
	
	m_bHasText = TRUE;
}

