/*
	This software module was originally developed by ENST in the course of
development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.
	This software module is an implementation of a part of one or more MPEG-4
Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1) 
standard. ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license 
to this software module or modifications thereof for use in hardware or software 
products claiming conformance to the MPEG-4 Systems (ISO/IEC 14496-1). 
Those intending to use this software module in hardware or software products are 
advised that its use may infringe existing patents.
	The original developer of this software module and his/her company, the 
subsequent editors and their companies, and ISO/IEC have no liability for use of 
this software module or modifications thereof in an implementation. Copyright is 
not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
	ENST retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using the code 
for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright    
notice must be included in all copies or derivative works.              
						Copyright  2002.
*/

///////////////////////////////////////////////////////////////////////////
// File: VisualRender.cpp
//
// Authors: Jean Le Feuvre - ENST 2002
//
// Created: August, 2002.
//
//
///////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include <assert.h>
#include "VisualRender.h"

#include "GeometryNode.h"
#include "ShapeProxy.h"
#include "BRect.h"
#include "Presenter.h"
#include "root.h"
#include "bindable.h"

#ifdef WIN32
// Include the details of the Windows GPIPLUS video renderer
#include "VisualRender-GDIP.h"
#else
#error "only Win32 platform supported for Osmose Player"
#endif


// VisualRenderer::Create
//
// Return a pointer to a visual renderer.  Use this to construct
// a visual render appropriate for the current platform.  A pointer
// to the presenter, and a platform specific parameter are given.
VisualRenderer* VisualRenderer::Create(Presenter* pPresenter, void* pInfo)
{
	VisualRenderer* pVisRend = NULL;
	pVisRend = new VisualRendererGDI(pPresenter, (HWND)pInfo);
	pVisRend->m_pPresenter = pPresenter;
	return pVisRend;
}

// VisualRenderer::VisualRenderer
// Constructor - empty the list of drawable nodes.
VisualRenderer::VisualRenderer() : VisualSurface()
{
}

// VisualRenderer::~VisualRenderer
// Destructor - delete any memory associated with the list of drawable
// nodes.
VisualRenderer::~VisualRenderer()
{
}


// VisualRenderer::Init
// Empty the list of drawable nodes
void VisualRenderer::Init()
{
	m_numberOfDrawables = 0;
}

// VisualRenderer::Terminate
// Empty the list of drawable nodes (without deleting memory).
void VisualRenderer::Terminate()
{
	m_numberOfDrawables = 0;
	Reset();
}

void VisualRenderer::DrawScene(Effects &effects)
{
	Effects *newEffects = new Effects(effects);

	ResetDrawList();
	newEffects->m_pSurface = this;
	newEffects->m_bDrawBackground = FALSE;


	//call base class draw
	VisualSurface::InitDraw(newEffects);
	
	//get the scene object
	Traversable* pRootProxy = (Traversable*) GetPresenter()->GetScene()->GetRenderingObject();
	pRootProxy->Render(newEffects);

	VisualSurface::TerminateDraw(newEffects);

	//and paint the surface
	Paint();

	delete newEffects;
}

BOOL VisualRenderer::SetRenderingOption(enum renderingOptions option, int value)
{
	if (option==VIDEO_FLAG_CLEARONEND) {
		m_clearOnEnd = (BOOL) value;
		return TRUE;
	}
	return FALSE;
}

BOOL VisualRenderer::GetRenderingOption(enum renderingOptions option, int &value)
{
	if (option==VIDEO_FLAG_CLEARONEND) {
		value = m_clearOnEnd;
		return TRUE;
	}
	return FALSE;
}
