/*
	This software module was originally developed by ENST in the course of
development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.
	This software module is an implementation of a part of one or more MPEG-4
Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1) 
standard. ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license 
to this software module or modifications thereof for use in hardware or software 
products claiming conformance to the MPEG-4 Systems (ISO/IEC 14496-1). 
Those intending to use this software module in hardware or software products are 
advised that its use may infringe existing patents.
	The original developer of this software module and his/her company, the 
subsequent editors and their companies, and ISO/IEC have no liability for use of 
this software module or modifications thereof in an implementation. Copyright is 
not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
	ENST retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using the code 
for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright    
notice must be included in all copies or derivative works.              
						Copyright  2002.
*/

///////////////////////////////////////////////////////////////////////////
// File: VisualRender.h
//
// Authors: Jean Le Feuvre ENST 2002
//
//
///////////////////////////////////////////////////////////////////////////


#ifndef VISUALRENDER_H
#define VISUALRENDER_H


#include "ztypes.h"
#include "VisualSurface.h"
#include "drawable.h"
#include "ztl.h"
#include "Nodes.h"
#include "ColorGradient.h"


class Presenter;
class Capabilities;


enum renderingOptions
{
	//set highspeed on or off - boolean value
	VIDEO_FLAG_HIGHSPEED = 0,
	//set antialiasing on or off - boolean value
	VIDEO_FLAG_ANTIALIAS,
	//set antialiasing on or off for text- boolean value overrrides antialiasing settings if set, 
	//if no set is ignored
	VIDEO_FLAG_TEXTANTIALIAS,
	//set scalable zooming on or off - boolean value
	VIDEO_FLAG_SCALABLEZOOM,
	//set ratio keeping on or off - boolean value
	VIDEO_FLAG_KEEPRATIO,
	//clear screen at and of presentation - boolean value
	VIDEO_FLAG_CLEARONEND,
	//restore original aspect - no value needed
	VIDEO_FLAG_RESTOREASPECT,
	//set max number of points computed during bezier curve construction - unsigned int value
	VIDEO_FLAG_BEZIERRESOLUTION,
	//don't fill shapes (neither full nor textures) - boolean value
	VIDEO_FLAG_NOFILL,
	//get the pixelMetrics flags
	VIDEO_FLAG_PIXELMETRICS,
	//turns fullscreen on/off - boolean value
	VIDEO_FLAG_FULLSCREEN,
	//turn YUV hardware suport on/off - boolean value
	VIDEO_FLAG_YUV,
	//set/get YUV hardware format - 4CC value
	VIDEO_FLAG_YUV_FORMAT
};

enum
{
	VIDEO_CAPTURE_STOP = 0,
	VIDEO_CAPTURE_START,
	VIDEO_CAPTURE_SHOT
};

class VisualRenderer : public VisualSurface
{
public:

	// Return a pointer to a visual renderer.  Use this to construct
	// a visual render appropriate for the current platform.  A pointer
	// to the presenter, and a platform specific parameter are given.
	static VisualRenderer* Create(Presenter* pPresenter, void* pInfo = NULL);


	VisualRenderer();
	virtual ~VisualRenderer();
	/*inits renderer*/	
	virtual void Init();
	/*stops renderer*/	
	virtual void Terminate();
	
	//flush rendering surface to screen
	virtual void Paint() = 0;

	// Get the capabilities of this visual renderer
	virtual void GetCapabilities (Capabilities* pPresentationCaps) = 0;

	// Forces the renderer to refresh size and position info
	virtual void UpdateScreenSizeInfo() = 0;

	//returns physical offscreen surface size
	virtual unsigned int GetScreenWidth() = 0;
	virtual unsigned int GetScreenHeight() = 0;
	
	void DrawScene(Effects &effects);

	//set a given option (cf enum renderingOptions )
	virtual BOOL SetRenderingOption(enum renderingOptions option, int value); 
	virtual BOOL GetRenderingOption(enum renderingOptions option, int &value);

	//set the bifs resolution (to compute all aspect ratio)
	virtual void SetBIFSResolution(unsigned int width, unsigned int height, BOOL usePixelMetrics) = 0;

	//mode: 2: shot, 1: start seq, 0: stop seq
	virtual void Screenshot(const char *pathandname, int mode) = 0;
};


#endif // VISUALRENDER_H

