//This software module was originally developed by CSELT in the course of   //
//development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.             //
//This software module is an implementation of a part of one or more MPEG-4 //
//Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems        //
//(ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 Systems     //
//(ISO/IEC 14496-1) free license to this software module or modifications   //
//thereof for use in hardware or software products claiming conformance to  //
//the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use this software//
//module in hardware or software products are advised that its use may      //
//infringe existing patents.                                                //
//The original developer of this software module and his/her company, the   //
//subsequent editors and their companies, and ISO/IEC have no liability     //
//for use of this software module or modifications thereof in an            //
//implementation. Copyright is not released for non MPEG-4 Systems          //
//(ISO/IEC 14496-1) conforming products. CSELT S.pA. retains full right     //
//to use the code for his/her own purpose, assign or donate the code to a   //
//third party and to inhibit third parties from using the code for non      //
//MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright      //
//notice must be included in all copies or derivative works.                //
//Copyright  1999.                                                         //
//This work has been partly developed within the framework of the ESPRIT    //
//European Research Project  "MPEG-4 PC".                                   //
//////////////////////////////////////////////////////////////////////////////
//
// File:  constraint.h
//
// Author: Gianluca Di Cagno  gianluca.dicagno@cselt.it
//
// Modified:
//
// Description:
//   the definition of classes for the FORM NODE constraints 
//   
//
// Created: February 1999
//
//
///////////////////////////////////////////////////////////////////////////
#ifndef CONSTRAINT_H
#define CONSTRAINT_H

//declared in formProxy.cpp
extern int factory_order ;

class Constraint 
{
public :
	
	Constraint() :m_iSpace (-1){}
	virtual void Apply(const int * group,int iNumberOfElementsInGroup)=0;
	static	int GetOrder() {return factory_order ++;}
	FormProxy *m_pFormProxy;
	int m_iSpace;
};

class AL : public Constraint 
{
	public :
		AL():Constraint(){}
		void Apply(const int * group,int iNumberOfElementsInGroup);

};
class AH : public Constraint 
{
	public :
		AH():Constraint(){}
		void Apply(const int * group, int iNumberOfElementsInGroup);

};
class AR : public Constraint 
{
	public :
		AR():Constraint(){}
		void Apply(const int * group,int iNumberOfElementsInGroup);

};
class AT : public Constraint 
{
	public :
		AT():Constraint(){}
		void Apply(const int * group,int iNumberOfElementsInGroup);

};
class AV : public Constraint 
{
	public :
		AV():Constraint(){}
		void Apply(const int * group,int iNumberOfElementsInGroup);

};
class AB : public Constraint 
{
	public :
		AB():Constraint(){}
		void Apply(const int * group,int iNumberOfElementsInGroup);

};

class SH : public Constraint 
{
	public :
		SH():Constraint(){}
		void Apply(const int * group,int iNumberOfElementsInGroup);
};
class SHin : public Constraint 
{
	public :
		SHin():Constraint(){}
		void Apply(const int * group,int iNumberOfElementsInGroup);
};
class SV : public Constraint
{
	public:
		SV():Constraint(){}
		void Apply(const int * group,int iNumberOfElementsInGroup);
};
class SVin : public Constraint
{
	public:
		SVin():Constraint(){}
		void Apply(const int * group,int iNumberOfElementsInGroup);
};

class FormProxy;

class ConstraintsFactoryBase
{
public:
	virtual Constraint *CreateConstraint () = 0;
};


template <class T> class ConstraintsFactory : public ConstraintsFactoryBase
{

		virtual Constraint *CreateConstraint ()   {return new T;}
public :

	ConstraintsFactory ()
	{
		FormProxy::StoreFactory(T::GetOrder(),this); //TheConstraintsFactory[T::GetOrder()]=this;
	}
};

#define DECLARE_CONSTRAINT(X) \
     ConstraintsFactory<class X>  constraintsfactory##X      
#endif