//This software module was originally developed by CSELT in the course of   //
//development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.             //
//This software module is an implementation of a part of one or more MPEG-4 //
//Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems        //
//(ISO/IEC 14496-1) standard. ISO/IEC gives users of the MPEG-4 Systems     //
//(ISO/IEC 14496-1) free license to this software module or modifications   //
//thereof for use in hardware or software products claiming conformance to  //
//the MPEG-4 Systems (ISO/IEC 14496-1). Those intending to use this software//
//module in hardware or software products are advised that its use may      //
//infringe existing patents.                                                //
//The original developer of this software module and his/her company, the   //
//subsequent editors and their companies, and ISO/IEC have no liability     //
//for use of this software module or modifications thereof in an            //
//implementation. Copyright is not released for non MPEG-4 Systems          //
//(ISO/IEC 14496-1) conforming products. CSELT S.pA. retains full right     //
//to use the code for his/her own purpose, assign or donate the code to a   //
//third party and to inhibit third parties from using the code for non      //
//MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright      //
//notice must be included in all copies or derivative works.                //
//Copyright  1999.                                                         //
//This work has been partly developed within the framework of the ESPRIT    //
//European Research Project  "MPEG-4 PC".                                   //
//////////////////////////////////////////////////////////////////////////////
//
// File: dx-errors.cpp
//
// Author: Matthew Leditschke 
//
// Modified:
//
// Description:
//   The implementation of a function which returns a string for a
//   given DirectX error code.  (In case you are interested, this
//   file was automatically generated using a Perl script.)
//
// Created: April, 1997
//
//
////////////////////////////////////////////////////////////////////////////////////


#include "defs.h"
#include <mmsystem.h>
#include <ddraw.h>
#include <dsound.h>
#include "dx-errors.h"


// This is a symbol defined in DirectX 3
#ifndef DDERR_NOTINITIALIZED
#define DDERR_NOTINITIALIZED CO_E_NOTINITIALIZED
#endif


// ========================================================================

/*
 * HRESULTtoText:
 *
 * Return a text string which describes the given HRESULT error code.
 */

char* HRESULTtoText (HRESULT errCode)
{
	char* errText;

	switch (errCode)
	{
	//
	// DirectDraw errors
	//
	case DDERR_ALREADYINITIALIZED:
		errText = "DDERR_ALREADYINITIALIZED: This object is already initialized.\n";
		break;
	case DDERR_CANNOTATTACHSURFACE:
		errText = "DDERR_CANNOTATTACHSURFACE: This surface can not be attached to the requested surface.\n";
		break;
	case DDERR_CANNOTDETACHSURFACE:
		errText = "DDERR_CANNOTDETACHSURFACE: This surface can not be detached from the requested surface.\n";
		break;
	case DDERR_CURRENTLYNOTAVAIL:
		errText = "DDERR_CURRENTLYNOTAVAIL: Support is currently not available.\n";
		break;
	case DDERR_EXCEPTION:
		errText = "DDERR_EXCEPTION: An exception was encountered while performing the requested operation.\n";
		break;
	case DDERR_GENERIC:
		errText = "DDERR_GENERIC: Generic failure.\n";
		break;
	case DDERR_HEIGHTALIGN:
		errText = "DDERR_HEIGHTALIGN: Height of rectangle provided is not a multiple of reqd alignment.\n";
		break;
	case DDERR_INCOMPATIBLEPRIMARY:
		errText = "DDERR_INCOMPATIBLEPRIMARY: Unable to match primary surface creation request with existing primary surface.\n";
		break;
	case DDERR_INVALIDCAPS:
		errText = "DDERR_INVALIDCAPS: One or more of the caps bits passed to the callback are incorrect.\n";
		break;
	case DDERR_INVALIDCLIPLIST:
		errText = "DDERR_INVALIDCLIPLIST: DirectDraw does not support provided Cliplist.\n";
		break;
	case DDERR_INVALIDMODE:
		errText = "DDERR_INVALIDMODE: DirectDraw does not support the requested mode.\n";
		break;
	case DDERR_INVALIDOBJECT:
		errText = "DDERR_INVALIDOBJECT: DirectDraw received a pointer that was an invalid DIRECTDRAW object.\n";
		break;
	case DDERR_INVALIDPARAMS:
		errText = "DDERR_INVALIDPARAMS: One or more of the parameters passed to the callback function are incorrect.\n";
		break;
	case DDERR_INVALIDPIXELFORMAT:
		errText = "DDERR_INVALIDPIXELFORMAT: pixel format was invalid as specified.\n";
		break;
	case DDERR_INVALIDRECT:
		errText = "DDERR_INVALIDRECT: Rectangle provided was invalid.\n";
		break;
	case DDERR_LOCKEDSURFACES:
		errText = "DDERR_LOCKEDSURFACES: Operation could not be carried out because one or more surfaces are locked.\n";
		break;
	case DDERR_NO3D:
		errText = "DDERR_NO3D: There is no 3D present.\n";
		break;
	case DDERR_NOALPHAHW:
		errText = "DDERR_NOALPHAHW: Operation could not be carried out because there is no alpha accleration hardware present or available.\n";
		break;
	case DDERR_NOCLIPLIST:
		errText = "DDERR_NOCLIPLIST: no clip list available.\n";
		break;
	case DDERR_NOCOLORCONVHW:
		errText = "DDERR_NOCOLORCONVHW: Operation could not be carried out because there is no color conversion hardware present or available.\n";
		break;
	case DDERR_NOCOOPERATIVELEVELSET:
		errText = "DDERR_NOCOOPERATIVELEVELSET: Create function called without DirectDraw object method SetCooperativeLevel being called.\n";
		break;
	case DDERR_NOCOLORKEY:
		errText = "DDERR_NOCOLORKEY: Surface doesn't currently have a color key.\n";
		break;
	case DDERR_NOCOLORKEYHW:
		errText = "DDERR_NOCOLORKEYHW: Operation could not be carried out because there is no hardware support of the dest color key.\n";
		break;
	case DDERR_NODIRECTDRAWSUPPORT:
		errText = "DDERR_NODIRECTDRAWSUPPORT: No DirectDraw support possible with current display driver.\n";
		break;
	case DDERR_NOEXCLUSIVEMODE:
		errText = "DDERR_NOEXCLUSIVEMODE: Operation requires the application to have exclusive mode but the application does not have exclusive mode.\n";
		break;
	case DDERR_NOFLIPHW:
		errText = "DDERR_NOFLIPHW: Flipping visible surfaces is not supported.\n";
		break;
	case DDERR_NOGDI:
		errText = "DDERR_NOGDI: There is no GDI present.\n";
		break;
	case DDERR_NOMIRRORHW:
		errText = "DDERR_NOMIRRORHW: Operation could not be carried out because there is no hardware present or available.\n";
		break;
	case DDERR_NOTFOUND:
		errText = "DDERR_NOTFOUND: Requested item was not found.\n";
		break;
	case DDERR_NOOVERLAYHW:
		errText = "DDERR_NOOVERLAYHW: Operation could not be carried out because there is no overlay hardware present or available.\n";
		break;
	case DDERR_NORASTEROPHW:
		errText = "DDERR_NORASTEROPHW: Operation could not be carried out because there is no appropriate raster op hardware present or available.\n";
		break;
	case DDERR_NOROTATIONHW:
		errText = "DDERR_NOROTATIONHW: Operation could not be carried out because there is no rotation hardware present or available.\n";
		break;
	case DDERR_NOSTRETCHHW:
		errText = "DDERR_NOSTRETCHHW: Operation could not be carried out because there is no hardware support for stretching.\n";
		break;
	case DDERR_NOT4BITCOLOR:
		errText = "DDERR_NOT4BITCOLOR: DirectDrawSurface is not in 4 bit color palette and the requested operation requires 4 bit color palette.\n";
		break;
	case DDERR_NOT4BITCOLORINDEX:
		errText = "DDERR_NOT4BITCOLORINDEX: DirectDrawSurface is not in 4 bit color index palette and the requested operation requires 4 bit color index palette.\n";
		break;
	case DDERR_NOT8BITCOLOR:
		errText = "DDERR_NOT8BITCOLOR: DirectDraw Surface is not in 8 bit color mode and the requested operation requires 8 bit color.\n";
		break;
	case DDERR_NOTEXTUREHW:
		errText = "DDERR_NOTEXTUREHW: Operation could not be carried out because there is no texture mapping hardware present or available.\n";
		break;
	case DDERR_NOVSYNCHW:
		errText = "DDERR_NOVSYNCHW: Operation could not be carried out because there is no hardware support for vertical blank synchronized operations.\n";
		break;
	case DDERR_NOZBUFFERHW:
		errText = "DDERR_NOZBUFFERHW: Operation could not be carried out because there is no hardware support for zbuffer blting.\n";
		break;
	case DDERR_NOZOVERLAYHW:
		errText = "DDERR_NOZOVERLAYHW: Overlay surfaces could not be z layered based on their BltOrder because the hardware does not support z layering of overlays.\n";
		break;
	case DDERR_OUTOFCAPS:
		errText = "DDERR_OUTOFCAPS: The hardware needed for the requested operation has already been allocated.\n";
		break;
	case DDERR_OUTOFMEMORY:
		errText = "DDERR_OUTOFMEMORY: DirectDraw does not have enough memory to perform the operation.\n";
		break;
	case DDERR_OUTOFVIDEOMEMORY:
		errText = "DDERR_OUTOFVIDEOMEMORY: DirectDraw does not have enough memory to perform the operation.\n";
		break;
	case DDERR_OVERLAYCANTCLIP:
		errText = "DDERR_OVERLAYCANTCLIP: hardware does not support clipped overlays.\n";
		break;
	case DDERR_OVERLAYCOLORKEYONLYONEACTIVE:
		errText = "DDERR_OVERLAYCOLORKEYONLYONEACTIVE: Can only have ony color key active at one time for overlays.\n";
		break;
	case DDERR_PALETTEBUSY:
		errText = "DDERR_PALETTEBUSY: Access to this palette is being refused because the palette is already locked by another thread.\n";
		break;
	case DDERR_COLORKEYNOTSET:
		errText = "DDERR_COLORKEYNOTSET: No src color key specified for this operation.\n";
		break;
	case DDERR_SURFACEALREADYATTACHED:
		errText = "DDERR_SURFACEALREADYATTACHED: This surface is already attached to the surface it is being attached to.\n";
		break;
	case DDERR_SURFACEALREADYDEPENDENT:
		errText = "DDERR_SURFACEALREADYDEPENDENT: This surface is already a dependency of the surface it is being made a dependency of.\n";
		break;
	case DDERR_SURFACEBUSY:
		errText = "DDERR_SURFACEBUSY: Access to this surface is being refused because the surface is already locked by another thread.\n";
		break;
	case DDERR_CANTLOCKSURFACE:
		errText = "DDERR_CANTLOCKSURFACE: Access to this surface is being refused because no driver exists which can supply a pointer to the surface. This is most likely to happen when attempting to lock the primary surface when no DCI provider is present.\n";
		break;
	case DDERR_SURFACEISOBSCURED:
		errText = "DDERR_SURFACEISOBSCURED: Access to Surface refused because Surface is obscured.\n";
		break;
	case DDERR_SURFACELOST:
		errText = "DDERR_SURFACELOST: Access to this surface is being refused because the surface is gone. The DIRECTDRAWSURFACE object representing this surface should have Restore called on it.\n";
		break;
	case DDERR_SURFACENOTATTACHED:
		errText = "DDERR_SURFACENOTATTACHED: The requested surface is not attached.\n";
		break;
	case DDERR_TOOBIGHEIGHT:
		errText = "DDERR_TOOBIGHEIGHT: Height requested by DirectDraw is too large.\n";
		break;
	case DDERR_TOOBIGSIZE:
		errText = "DDERR_TOOBIGSIZE: Size requested by DirectDraw is too large --  The individual height and width are OK.\n";
		break;
	case DDERR_TOOBIGWIDTH:
		errText = "DDERR_TOOBIGWIDTH: Width requested by DirectDraw is too large.\n";
		break;
	case DDERR_UNSUPPORTED:
		errText = "DDERR_UNSUPPORTED: Action not supported.\n";
		break;
	case DDERR_UNSUPPORTEDFORMAT:
		errText = "DDERR_UNSUPPORTEDFORMAT: FOURCC format requested is unsupported by DirectDraw.\n";
		break;
	case DDERR_UNSUPPORTEDMASK:
		errText = "DDERR_UNSUPPORTEDMASK: Bitmask in the pixel format requested is unsupported by DirectDraw.\n";
		break;
	case DDERR_VERTICALBLANKINPROGRESS:
		errText = "DDERR_VERTICALBLANKINPROGRESS: vertical blank is in progress.\n";
		break;
	case DDERR_WASSTILLDRAWING:
		errText = "DDERR_WASSTILLDRAWING: Informs DirectDraw that the previous Blt which is transfering information to or from this Surface is incomplete.\n";
		break;
	case DDERR_XALIGN:
		errText = "DDERR_XALIGN: Rectangle provided was not horizontally aligned on reqd. boundary.\n";
		break;
	case DDERR_INVALIDDIRECTDRAWGUID:
		errText = "DDERR_INVALIDDIRECTDRAWGUID: The GUID passed to DirectDrawCreate is not a valid DirectDraw driver identifier.\n";
		break;
	case DDERR_DIRECTDRAWALREADYCREATED:
		errText = "DDERR_DIRECTDRAWALREADYCREATED: A DirectDraw object representing this driver has already been created for this process.\n";
		break;
	case DDERR_NODIRECTDRAWHW:
		errText = "DDERR_NODIRECTDRAWHW: A hardware only DirectDraw object creation was attempted but the driver did not support any hardware.\n";
		break;
	case DDERR_PRIMARYSURFACEALREADYEXISTS:
		errText = "DDERR_PRIMARYSURFACEALREADYEXISTS: this process already has created a primary surface.\n";
		break;
	case DDERR_NOEMULATION:
		errText = "DDERR_NOEMULATION: software emulation not available.\n";
		break;
	case DDERR_REGIONTOOSMALL:
		errText = "DDERR_REGIONTOOSMALL: region passed to Clipper::GetClipList is too small.\n";
		break;
	case DDERR_CLIPPERISUSINGHWND:
		errText = "DDERR_CLIPPERISUSINGHWND: an attempt was made to set a clip list for a clipper objec that is already monitoring an hwnd.\n";
		break;
	case DDERR_NOCLIPPERATTACHED:
		errText = "DDERR_NOCLIPPERATTACHED: No clipper object attached to surface object.\n";
		break;
	case DDERR_NOHWND:
		errText = "DDERR_NOHWND: Clipper notification requires an HWND or no HWND has previously been set as the CooperativeLevel HWND.\n";
		break;
	case DDERR_HWNDSUBCLASSED:
		errText = "DDERR_HWNDSUBCLASSED: HWND used by DirectDraw CooperativeLevel has been subclassed, this prevents DirectDraw from restoring state.\n";
		break;
	case DDERR_HWNDALREADYSET:
		errText = "DDERR_HWNDALREADYSET: The CooperativeLevel HWND has already been set. It can not be reset while the process has surfaces or palettes created.\n";
		break;
	case DDERR_NOPALETTEATTACHED:
		errText = "DDERR_NOPALETTEATTACHED: No palette object attached to this surface.\n";
		break;
	case DDERR_NOPALETTEHW:
		errText = "DDERR_NOPALETTEHW: No hardware support for 16 or 256 color palettes.\n";
		break;
	case DDERR_BLTFASTCANTCLIP:
		errText = "DDERR_BLTFASTCANTCLIP: If a clipper object is attached to the source surface passed into a BltFast call.\n";
		break;
	case DDERR_NOBLTHW:
		errText = "DDERR_NOBLTHW: No blter.\n";
		break;
	case DDERR_NODDROPSHW:
		errText = "DDERR_NODDROPSHW: No DirectDraw ROP hardware.\n";
		break;
	case DDERR_OVERLAYNOTVISIBLE:
		errText = "DDERR_OVERLAYNOTVISIBLE: returned when GetOverlayPosition is called on a hidden overlay.\n";
		break;
	case DDERR_NOOVERLAYDEST:
		errText = "DDERR_NOOVERLAYDEST: returned when GetOverlayPosition is called on a overlay that UpdateOverlay has never been called on to establish a destionation.\n";
		break;
	case DDERR_INVALIDPOSITION:
		errText = "DDERR_INVALIDPOSITION: returned when the position of the overlay on the destionation is no longer legal for that destionation.\n";
		break;
	case DDERR_NOTAOVERLAYSURFACE:
		errText = "DDERR_NOTAOVERLAYSURFACE: returned when an overlay member is called for a non-overlay surface.\n";
		break;
	case DDERR_EXCLUSIVEMODEALREADYSET:
		errText = "DDERR_EXCLUSIVEMODEALREADYSET: An attempt was made to set the cooperative level when it was already set to exclusive.\n";
		break;
	case DDERR_NOTFLIPPABLE:
		errText = "DDERR_NOTFLIPPABLE: An attempt has been made to flip a surface that is not flippable.\n";
		break;
	case DDERR_CANTDUPLICATE:
		errText = "DDERR_CANTDUPLICATE: Can't duplicate primary & 3D surfaces, or surfaces that are implicitly created.\n";
		break;
	case DDERR_NOTLOCKED:
		errText = "DDERR_NOTLOCKED: Surface was not locked.  An attempt to unlock a surface that was not locked at all, or by this process, has been attempted.\n";
		break;
	case DDERR_CANTCREATEDC:
		errText = "DDERR_CANTCREATEDC: Windows can not create any more DCs.\n";
		break;
	case DDERR_NODC:
		errText = "DDERR_NODC: No DC was ever created for this surface.\n";
		break;
	case DDERR_WRONGMODE:
		errText = "DDERR_WRONGMODE: This surface can not be restored because it was created in a different mode.\n";
		break;
	case DDERR_IMPLICITLYCREATED:
		errText = "DDERR_IMPLICITLYCREATED: This surface can not be restored because it is an implicitly created surface.\n";
		break;
	case DDERR_NOTPALETTIZED:
		errText = "DDERR_NOTPALETTIZED: The surface being used is not a palette-based surface.\n";
		break;
	case DDERR_UNSUPPORTEDMODE:
		errText = "DDERR_UNSUPPORTEDMODE: The display is currently in an unsupported mode.\n";
		break;
	case DDERR_NOMIPMAPHW:
		errText = "DDERR_NOMIPMAPHW: Operation could not be carried out because there is no mip-map texture mapping hardware present or available.\n";
		break;
	case DDERR_INVALIDSURFACETYPE:
		errText = "DDERR_INVALIDSURFACETYPE: The requested action could not be performed because the surface was of the wrong type.\n";
		break;
	case DDERR_DCALREADYCREATED:
		errText = "DDERR_DCALREADYCREATED: A DC has already been returned for this surface. Only one DC can be retrieved per surface.\n";
		break;
	case DDERR_CANTPAGELOCK:
		errText = "DDERR_CANTPAGELOCK: The attempt to page lock a surface failed.\n";
		break;
	case DDERR_CANTPAGEUNLOCK:
		errText = "DDERR_CANTPAGEUNLOCK: The attempt to page unlock a surface failed.\n";
		break;
	case DDERR_NOTPAGELOCKED:
		errText = "DDERR_NOTPAGELOCKED: An attempt was made to page unlock a surface with no outstanding page locks.\n";
		break;
	case DDERR_NOTINITIALIZED:
		errText = "DDERR_NOTINITIALIZED: An attempt was made to invoke an interface member of a DirectDraw object created by CoCreateInstance() before it was initialized.\n";
		break;

	//
	// DirectSound errors
	//
	case DSERR_ALLOCATED:
		errText = "DSERR_ALLOCATED: The call failed because resources (such as a priority level) were already being used by another caller.\n";
		break;
	case DSERR_CONTROLUNAVAIL:
		errText = "DSERR_CONTROLUNAVAIL: The control (vol,pan,etc.) requested by the caller is not available.\n";
		break;
	case DSERR_INVALIDCALL:
		errText = "DSERR_INVALIDCALL: This call is not valid for the current state of this object.\n";
		break;
	case DSERR_PRIOLEVELNEEDED:
		errText = "DSERR_PRIOLEVELNEEDED: The caller does not have the priority level required for the function to succeed.\n";
		break;
	case DSERR_BADFORMAT:
		errText = "DSERR_BADFORMAT: The specified WAVE format is not supported.\n";
		break;
	case DSERR_NODRIVER:
		errText = "DSERR_NODRIVER: No sound driver is available for use.\n";
		break;
	case DSERR_ALREADYINITIALIZED:
		errText = "DSERR_ALREADYINITIALIZED: This object is already initialized.\n";
		break;
	case DSERR_NOAGGREGATION:
		errText = "DSERR_NOAGGREGATION: This object does not support aggregation.\n";
		break;
	case DSERR_BUFFERLOST:
		errText = "DSERR_BUFFERLOST: The buffer memory has been lost, and must be Restored.\n";
		break;
	case DSERR_OTHERAPPHASPRIO:
		errText = "DSERR_OTHERAPPHASPRIO: Another app has a higher priority level, preventing this call from succeeding.\n";
		break;
	case DSERR_UNINITIALIZED:
		errText = "DSERR_UNINITIALIZED: The Initialize() member on the Direct Sound Object has not been called or called successfully before calls to other members.\n";
		break;

	default:
		errText = "Unknown error code.\n";
		break;
	}

	return errText;
}