// -*- c++ -*-

// This software module was originally developed by Matthew Leditschke
// (CSELT), Zvi Lifshitz (VDOnet Corp.) and Karl Anders ygard (Telenor
// R&D) in the course of development of the MPEG-4 Systems standard
// (ISO/IEC 14496-1).  This software module is an implementation of a part
// of one or more MPEG-4 Systems tools as specified by the MPEG-4 Systems
// standard.  ISO/IEC gives users of the MPEG-4 Systems standard free
// license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// For more information or to receive an up to date version of this module,
// mail karlo@nta.no or check <http://televr.fou.telenor.no/compositor/>.
//
// Copyright 1997.

#ifndef __APPLICATION_H__
#define __APPLICATION_H__


#include "defs.h"
#include "Im1defs.h"

// WIN-SOL release
#ifdef WIN32
class Executive;
#else
class Executive3d;
#endif


/** Application.

    Top object which is instantiated when the application is invoked.  It
    performs the following:

    \begin{enumerate}

    \item Creates the display window

    \item Instantiates \Ref{Executive}

    \item Opens a source (multiplexed) file

    \item Starts an \Ref{Executive} session

    \item Passes control messages to the \Ref{Executive} (in this version
      only Paint and Stop).

    \item Closes the source file
      
    \item Terminates
      
    \end{enumerate}

    */

class Application
{
public:

#ifdef WIN32

  /// Constructor

			Application(HWND hWnd);
#else

  /// Constructor
			Application(int* argc, char* argv[]);

#endif

  /// Destructor

	virtual	~Application();

  /// Get executive

// WIN-SOL release
#ifdef WIN32
	Executive*		GetExecutive() const
#else
	Executive3d*		GetExecutive() const
#endif

	{
		return m_pExecutive;
	}

  /** Signal that a connection was made.

      Shall be overloaded by the derived class.	 Called From Executive
      after a session has been established (or failed).	 The value of
      result might be CONNECTION\_SUCCESS or a failure code.

      OnConnect() might be called before Start() has finished, which would
      typically be the situation in the local file case. */

  virtual void		OnConnect(CONNECTION_RESULT result = CONNECTION_SUCCESS) = 0;

  /** Signal that bitstream finished.

      Shall be overloaded by the derived class.	 Called From Executive when
      the session is terminated, i.e. end of local file reached or server
      indicated end of session.	 In case of unexpected abortion, reason
      indicates the reason. */

  virtual void		OnFinish(SERVICE_TERMINATION reason = END_OF_SERVICE) = 0;

  /// Signal that we got a non-fatal error

  virtual void		OnError (NON_FATAL_ERROR, va_list) = 0;

protected:

  /// Pointer to executive

// WIN-SOL release
#ifdef WIN32
  Executive*		m_pExecutive;
#else
  Executive3d*		m_pExecutive;
#endif


};

#endif // __APPLICATION_H__
