/***********************************************************************
    This software module  was  originally  developed by Zvi  Lifshitz
    (Triton R&D Ltd.) and  Mikael Bourges-Sevenier (France Telecom CNET)
    in the course of development of the MPEG-4 Systems (ISO/IEC 14496-1)
    standard.  This software module is an implementation of a part of
    one or more MPEG-4 Systems (ISO/IEC 14496-1) tools  as  specified
    by  the  MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC gives
    users of the MPEG-4 Systems (ISO/IEC 14496-1)  free  license  to
    this software module or modifications thereof for use in hardware
    or software products claiming conformance to the  MPEG-4  Systems
    (ISO/IEC 14496-1). Those intending to use this software module in
    hardware or software  products  are  advised  that  its  use  may
    infringe   existing  patents.  The  original  developer  of  this
    software module and his/her company, the subsequent  editors  and
    their  companies,  and  ISO/IEC have no liability for use of this
    software module or modifications thereof  in  an  implementation.
    Copyright  is not released for non MPEG-4 Systems (ISO/IEC 14496-
    1) conforming  products.  Triton R&D Ltd. and France  Telecom  CNET
    retain  full  right to use the code for their own purpose, assign
    or donate the code to a third party and to inhibit third  parties
    from  using  the  code  for  non MPEG-4 Systems (ISO/IEC 14496-1)
    conforming products. This copyright notice must  be  included  in
    all copies or derivative works. Copyright  1997.
************************************************************************/


#include "defs.h"
#include "Nodes.h"
// SOLARIS PORTING
// Core 5.7 rc1 - commenting the next 2 lines
//#include "Nodes2.h"
//#include "Nodes3.h"


//
// Firstly list functions whose implementation differs in BIFSENC and BIFSDEC
//

#ifdef BIFSDEC

// SB 99/05/12
#include "OpenGl/Interpolators/ColorInterpolatorProxy.h"
#include "OpenGl/Interpolators/CoordinateInterpolatorProxy.h"
#include "OpenGl/Interpolators/CoordinateInterpolator2DProxy.h"
#include "OpenGl/Interpolators/NormalInterpolatorProxy.h"
#include "OpenGl/Interpolators/OrientationInterpolatorProxy.h"
#include "OpenGl/Interpolators/PositionInterpolatorProxy.h"
#include "OpenGl/Interpolators/PositionInterpolator2DProxy.h"
#include "OpenGl/Interpolators/ScalarInterpolatorProxy.h"

// for AFX animation
#include "OpenGl/AFX/PositionInterpolator4DProxy.h"
#include "OpenGl/AFX/CoordinateInterpolator4DProxy.h"
#include "OpenGl/AFX/PositionAnimatorProxy.h"
#include "OpenGl/AFX/PositionAnimator2DProxy.h"
#include "OpenGl/AFX/ScalarAnimatorProxy.h"


void ColorInterpolator :: Onset_fraction()
{
  ColorInterpolatorProxy * poRend =
    (ColorInterpolatorProxy *) GetRenderingObject();
  if (poRend->Evaluate())
  {
    SetUpdateTime();
  }
}

void CoordinateInterpolator :: Onset_fraction()
{
  CoordinateInterpolatorProxy * poRend =
    (CoordinateInterpolatorProxy *) GetRenderingObject();
  if (poRend->Evaluate())
  {
    SetUpdateTime();
  }
}

void CoordinateInterpolator2D :: Onset_fraction()
{
  CoordinateInterpolator2DProxy * poRend =
    (CoordinateInterpolator2DProxy *) GetRenderingObject();
  if (poRend->Evaluate())
  {
    SetUpdateTime();
  }
}

void NormalInterpolator :: Onset_fraction()
{
  NormalInterpolatorProxy * poRend =
    (NormalInterpolatorProxy *) GetRenderingObject();
  if (poRend->Evaluate())
  {
    SetUpdateTime();
  }
}

void OrientationInterpolator :: Onset_fraction()
{
  OrientationInterpolatorProxy * poRend =
    (OrientationInterpolatorProxy *) GetRenderingObject();
  if (poRend->Evaluate())
  {
    SetUpdateTime();
  }
}

void PositionInterpolator :: Onset_fraction()
{
  PositionInterpolatorProxy * poRend =
    (PositionInterpolatorProxy *) GetRenderingObject();
  if (poRend->Evaluate())
  {
    SetUpdateTime();
  }
}

void PositionInterpolator2D :: Onset_fraction()
{
  PositionInterpolator2DProxy * poRend =
    (PositionInterpolator2DProxy *) GetRenderingObject();
  if (poRend->Evaluate())
  {
    SetUpdateTime();
  }
}

void ScalarInterpolator :: Onset_fraction()
{
  ScalarInterpolatorProxy * poRend =
    (ScalarInterpolatorProxy *) GetRenderingObject();
  if (poRend->Evaluate())
  {
    SetUpdateTime();
  }
}

// MBS 7/3/03: added these methods for 4D interpolators in version 5
void PositionInterpolator4D :: Onset_fraction()
{
  PositionInterpolator4DProxy * poRend =
    (PositionInterpolator4DProxy *) GetRenderingObject();
  if (poRend->Evaluate())
  {
    SetUpdateTime();
  }
}

void CoordinateInterpolator4D :: Onset_fraction()
{
  CoordinateInterpolator4DProxy * poRend =
    (CoordinateInterpolator4DProxy *) GetRenderingObject();
  if (poRend->Evaluate())
  {
    SetUpdateTime();
  }
}

void PositionAnimator :: Onset_fraction()
{
  PositionAnimatorProxy * poRend =
    (PositionAnimatorProxy *) GetRenderingObject();
  if (poRend->Evaluate())
  {
    SetUpdateTime();
  }
}

void PositionAnimator2D :: Onset_fraction()
{
  PositionAnimator2DProxy * poRend =
    (PositionAnimator2DProxy *) GetRenderingObject();
  if (poRend->Evaluate())
  {
    SetUpdateTime();
  }
}

void ScalarAnimator :: Onset_fraction()
{
  ScalarAnimatorProxy * poRend =
    (ScalarAnimatorProxy *) GetRenderingObject();
  if (poRend->Evaluate())
  {
    SetUpdateTime();
  }
}

#else // ! BIFSDEC

void ColorInterpolator :: Onset_fraction() {}
void CoordinateInterpolator :: Onset_fraction() {}
void CoordinateInterpolator2D :: Onset_fraction() {}
void NormalInterpolator :: Onset_fraction() {}
void OrientationInterpolator :: Onset_fraction() {}
void PositionInterpolator :: Onset_fraction() {}
void PositionInterpolator2D :: Onset_fraction() {}
void ScalarInterpolator :: Onset_fraction() {}
void CoordinateInterpolator4D :: Onset_fraction() {} // MBS: for version 5
void PositionInterpolator4D :: Onset_fraction() {} // MBS: for version 5
void PositionAnimator :: Onset_fraction() {} // MBS: for version 5
void PositionAnimator2D :: Onset_fraction() {} // MBS: for version 5
void ScalarAnimator :: Onset_fraction() {} // MBS: for version 5


#endif // BIFSDEC


//
// Now list functions whose implementation is the same for BIFSENC and BIFSDEC
// (namely that the implementation is an empty function body)
//

void Anchor :: OnaddChildren() {}
void Anchor :: OnremoveChildren() {}
void AudioDelay :: OnaddChildren() {}
void AudioDelay :: OnremoveChildren() {}
void AudioFX :: OnaddChildren() {}
void AudioFX :: OnremoveChildren() {}
void AudioMix :: OnaddChildren() {}
void AudioMix :: OnremoveChildren() {}
void AudioSource :: OnaddChildren() {}
void AudioSource :: OnremoveChildren() {}
void AudioSwitch :: OnaddChildren() {}
void AudioSwitch :: OnremoveChildren() {}
void Background :: Onset_bind() {}
void Background2D :: Onset_bind() {}
void Billboard :: OnaddChildren() {}
void Billboard :: OnremoveChildren() {}
void Collision :: OnaddChildren() {}
void Collision :: OnremoveChildren() {}
void CompositeTexture2D :: OnaddChildren() {}
void CompositeTexture2D :: OnremoveChildren() {}
void CompositeTexture3D :: OnaddChildren() {}
void CompositeTexture3D :: OnremoveChildren() {}
void ElevationGrid :: Onset_height() {}
void Extrusion :: Onset_crossSection() {}
void Extrusion :: Onset_orientation() {}
void Extrusion :: Onset_scale() {}
void Extrusion :: Onset_spine() {}
void Fog :: Onset_bind() {}
void Form :: OnaddChildren() {}
void Form :: OnremoveChildren() {}
void Group :: OnaddChildren() {}
void Group :: OnremoveChildren() {}
void Hierarchical3DMesh :: OntriangleBudget() {}
void IndexedFaceSet :: Onset_colorIndex() {}
void IndexedFaceSet :: Onset_coordIndex() {}
void IndexedFaceSet :: Onset_normalIndex() {}
void IndexedFaceSet :: Onset_texCoordIndex() {}
void IndexedFaceSet2D :: Onset_colorIndex() {}
void IndexedFaceSet2D :: Onset_coordIndex() {}
void IndexedFaceSet2D :: Onset_texCoordIndex() {}
void IndexedLineSet :: Onset_colorIndex() {}
void IndexedLineSet :: Onset_coordIndex() {}
void IndexedLineSet2D :: Onset_colorIndex() {}
void IndexedLineSet2D :: Onset_coordIndex() {}
void Layer2D :: OnaddChildren() {}
void Layer2D :: OnremoveChildren() {}
void Layer3D :: OnaddChildren() {}
void Layer3D :: OnremoveChildren() {}
void Layout :: OnaddChildren() {}
void Layout :: OnremoveChildren() {}
void ListeningPoint :: Onset_bind() {}
void NavigationInfo :: Onset_bind() {}
void OrderedGroup :: OnaddChildren() {}
void OrderedGroup :: OnremoveChildren() {}
void TemporalTransform :: OnremoveChildren() {}
void TemporalTransform :: OnaddChildren() {}
void TemporalGroup :: OnremoveChildren() {}
void TemporalGroup :: OnaddChildren() {}
void TermCap :: Onevaluate() {}
void Transform :: OnaddChildren() {}
void Transform :: OnremoveChildren() {}
void Transform2D :: OnaddChildren() {}
void Transform2D :: OnremoveChildren() {}
void Valuator :: OninSFBool() {}
void Valuator :: OninSFColor() {}
void Valuator :: OninMFColor() {}
void Valuator :: OninSFFloat() {}
void Valuator :: OninMFFloat() {}
void Valuator :: OninSFInt32() {}
void Valuator :: OninMFInt32() {}
void Valuator :: OninSFRotation() {}
void Valuator :: OninMFRotation() {}
void Valuator :: OninSFString() {}
void Valuator :: OninMFString() {}
void Valuator :: OninSFTime() {}
void Valuator :: OninSFVec2f() {}
void Valuator :: OninMFVec2f() {}
void Valuator :: OninSFVec3f() {}
void Valuator :: OninMFVec3f() {}
void Viewpoint :: Onset_bind() {}

// SOLARIS PORTING - To core 5.9rc9

void FFD :: OnaddChildren() {}
void FFD :: OnremoveChildren() {}
void MeshGrid :: Onset_colorIndex() {}
void MeshGrid :: Onset_coordIndex() {}
void MeshGrid :: Onset_normalIndex() {}
void MeshGrid :: Onset_texCoordIndex() {}
void SBBone :: OnaddChildren() {}
void SBBone :: OnremoveChildren() {}
void SBSegment :: OnaddChildren() {}
void SBSegment :: OnremoveChildren() {}
void SBSite :: OnaddChildren() {}
void SBSite :: OnremoveChildren() {}
void SubdivisionSurface :: Onset_colorIndex() {}
void SubdivisionSurface :: Onset_coordIndex() {}
void SubdivisionSurface :: Onset_cornerVertexIndex() {}
void SubdivisionSurface :: Onset_creaseEdgeIndex() {}
void SubdivisionSurface :: Onset_creaseVertexIndex() {}
void SubdivisionSurface :: Onset_dartVertexIndex() {}
void SubdivisionSurface :: Onset_texCoordIndex() {}

// SOLARIS PORTING - Merging code with 2D player
void Anchor :: Onactivate() {}

void NurbsCurve :: Onset_colorIndex() {}
void NurbsCurve2D :: Onset_colorIndex() {}
void NurbsSurface :: Onset_colorIndex() {}
void NurbsSurface :: Onset_texColorIndex() {}

// version 6
void Clipper2D :: OnaddChildren() {}
void Clipper2D :: OnremoveChildren() {}
void ColorTransform :: OnaddChildren() {}
void ColorTransform :: OnremoveChildren() {}
void PathLayout :: OnaddChildren() {}
void PathLayout :: OnremoveChildren() {}
void TransformMatrix2D :: OnaddChildren() {}
void TransformMatrix2D :: OnremoveChildren() {}
void Viewport :: Onset_bind() {}

// version 7
void AdvancedAudioBuffer :: OnaddChildren() {}
void AdvancedAudioBuffer :: OnremoveChildren() {}
void AudioChannelConfig :: OnaddChildren() {}
void AudioChannelConfig :: OnremoveChildren() {}
void Transform3DAudio :: OnaddChildren() {}
void Transform3DAudio :: OnremoveChildren() {}
void FootPrintSet :: OnaddChildren() {}
void FootPrintSet :: OnremoveChildren() {}
void Shadow :: OnaddChildren() {}
void Shadow :: OnremoveChildren() {}

