// This software module was originally developed by Matthew Leditschke
// (CSELT), Zvi Lifshitz (VDOnet Corp.) and Karl Anders ygard (Telenor
// R&D) in the course of development of the MPEG-4 Systems standard
// (ISO/IEC 14496-1).  This software module is an implementation of a part
// of one or more MPEG-4 Systems tools as specified by the MPEG-4 Systems
// standard.  ISO/IEC gives users of the MPEG-4 Systems standard free
// license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// For more information or to receive an up to date version of this module,
// mail karlo@nta.no or check <http://televr.fou.telenor.no/compositor/>.
//
// Copyright 1997.

//ML - added defs.h
#include "defs.h"
#include "Presenter.h"
#include "Caps.h"
#include "opengl/base/VisualRenderer.h"
#include "opengl/base/AudioRenderer.h"

//added by Helge Drumm TU Ilmenau, 6.12.2006, Shadow Rendering
#include "../SGManager/SGManager.h"
#include "../SGManager/NRNodes.h"

// SOLARIS PORTING
// Core 5.7rc1 - commenting the next line
//#include "script/encoder/VScript.h"


//*****************************************************************
// SB @@@@ added from Zvi 98/11/22 >>>>

/*-----------------------------------------------------------------
Save system time as an offset, and set the clock on.
-----------------------------------------------------------------*/
void Presenter :: SetClock (BOOL bOn)
{
	if (bOn)
		m_dwOffset = timeGetTime ();
	m_bClockActive = bOn;
}

/*-----------------------------------------------------------------
Return the sysmem time minus the clocko offset of the Presenter's clock
-----------------------------------------------------------------*/
DWORD Presenter :: GetClockTime ()
{
	return timeGetTime () - m_dwOffset;
}

/*-----------------------------------------------------------------
Format a string into the log file
-----------------------------------------------------------------*/
void Presenter :: Logf (LPCSTR szFormat, ...)
{
	fprintf (
		m_fLogFile,
		"Time: %6lu(%6lu): ",
		timeGetTime () - m_dwStartTime,
		GetClockTime ()
	);


	//#defines for va_list, va_start , va_end are defined in Include/defs.h
		va_list argptr;
		va_start (argptr, szFormat);
		vfprintf (m_fLogFile, szFormat, argptr);
		va_end (argptr);

}

// SB @@@@ added from Zvi 98/11/22 <<<<
//*****************************************************************

// The capabilities of the OpenGLPresenter combines the capabilities of the
// visual and audio renderers.

void
Presenter::GetCapabilities(Capabilities *pCaps)
{
  int j;
  // Capabilities caps;
  CapabilitiesImp caps;

  GetVisualRenderer()->GetCapabilities(pCaps);
  GetAudioRenderer()->GetCapabilities(&caps);

  j = pCaps->GetFormatCapabilityLength();

  for (int i = 0; i < caps.GetFormatCapabilityLength(); i++)
    pCaps->AddFormatCapability(caps.GetFormatCapability(i), j++);

  pCaps->AddFormatCapability(MediaFormat::ANIMSTREAM /* FORMAT_ANIMSTREAM */ , 0);
}


// ===========================================================================

/*!
  \brief Initialise the presenter.

  This function is called by the executive as the player is starting up,
  and should only be called once during the life of the program.
*/

void Presenter::Init()
{
	//ML - added the next line
	PresenterBase::Init();
}


// ===========================================================================

/*!
  \brief Terminate the presenter.

  This function is called by the executive as the player is shutting down,
  and should only be called once during the life of the program.
*/

void Presenter::Terminate()
{
	PresenterBase::Terminate();
}

// ===========================================================================
//added by Helge Drumm TU Ilmenau, 6.12.2006, Shadow Rendering

/*!
  \brief Traverse the scenegraph.

  This function is called by the visualrenderer,
  
*/

MediaObject* Presenter::TraverseSG(int iNodeCode,MediaObject* pStartNode,RBaseList *refList)
{
   MediaObject* pNode= NULL;
   RBaseList* refList2;
   MediaObject* tNode = NULL;
   
   char *name;
   
    refList2=sgm->CreateReference(pStartNode->GetName(),pStartNode,refList,pStartNode->GetNodeCode());
    
	refList2->NodeCode=pStartNode->GetNodeCode();
	refList2->NodeID=pStartNode->GetID();
	
	
	if(refList2->parent!=NULL)
	{
	  refList2->Matrix4x4->InitMatrix(refList2->parent->Matrix4x4->m_elements);
	}
	if(refList2->parent->NodeCode==CODE_Shadow)
	{
      NRShadow *partemp =(NRShadow*)refList2->parent;
	  Shadow* temp = (Shadow *)partemp->refnode;
	  int i = refList2->parent->count-1;
	  if(temp->enabled.GetValue()) {
	  refList2->shadow.cast    = temp->cast[i];
	  refList2->shadow.receive = temp->receive[i];

	  } else
	  {
        refList2->shadow.cast    = 0;
	    refList2->shadow.receive = 0;

	  }

	  
	}

	switch(int rID = pStartNode->GetNodeCode())
	{
	
	
	
	case CODE_Transform: { Transform * temp = ((Transform*)((NRTransform*)refList2)->refnode);
		                   ((NRTransform*)refList2)->UpdateValue(temp->center,
							                                     temp->rotation,
																 temp->scale,
																 temp->scaleOrientation,
																 temp->translation);
			               

						   ((NRTransform*)refList2)->SetMatrix();

			               break;
						 }

    case CODE_Fog             : { Fog *temp = ((Fog*)((NRFog*)refList2)->refnode);
								  ((NRFog*)refList2)->UpdateValue(temp->color,
																  strcmp(temp->fogType, "EXPONENTIAL"),
																  temp->visibilityRange);
								  sgm->env_fog=(NRFog*)refList2;
								  
								  
		                           break;}
	case CODE_DirectionalLight: { DirectionalLight *temp = ((DirectionalLight*)((NRDirectionalLight*)refList2)->refnode);
								  ((NRDirectionalLight*)refList2)->UpdateValue(temp->ambientIntensity,
									                                           temp->color,
																			   temp->direction,
																			   temp->intensity,
																			   temp->on.GetValue());
								  if(((NRDirectionalLight *)refList2)->on==true)
							{
                             sgm->LightGroupMMap.insert(std::pair<RBaseList*,RBaseList*>(refList2->parent,refList2));
							}
		                          break;
								}
    case CODE_PointLight: { PointLight *temp = ((PointLight*)((NRPointLight*)refList2)->refnode);
		                    ((NRPointLight *)refList2)->UpdateValue(temp->ambientIntensity,
								                                    temp->attenuation,
																	temp->color,
																	temp->intensity,
																	temp->location,
																	temp->on.GetValue(),
																	temp->radius);
		                    if(((NRPointLight *)refList2)->on==true)
							{
                             sgm->LightGroupMMap.insert(std::pair<RBaseList*,RBaseList*>(refList2->parent,refList2));
							}
							
							
							break;
						  }
	case CODE_Shadow: {  NRShadow *nrshad =  (NRShadow*)refList2;
		
		                 Shadow *temp1 = (Shadow*)nrshad->refnode;
		                 
		                 ((NRShadow*)refList2)->parent->penumbra = temp1->penumbra;
						 
					   break;
					  
					  }


    case CODE_SpotLight: { SpotLight *temp = ((SpotLight *)((NRSpotLight*)refList2)->refnode);
		                   ((NRSpotLight *)refList2)->UpdateValue(temp->ambientIntensity,
							                                      temp->attenuation,
																  temp->beamWidth,
																  temp->color,
																  temp->cutOffAngle,
																  temp->direction,
																  temp->intensity,
																  temp->location,
																  temp->on.GetValue(),
																  temp->radius);
						   if(((NRSpotLight *)refList2)->on==true)
							{
                              sgm->LightGroupMMap.insert(std::pair<RBaseList*,RBaseList*>(refList2->parent,refList2));
							}
		                   break;
						 }

		
	}
	
	for ( int i = 0 ;i < pStartNode->GetChildrenCount();i++ )
	{
		
	   
	   
	   if ( pStartNode->GetChild(i)->GetNodeCode() != iNodeCode )
		{
		   	
		   pNode = TraverseSG(iNodeCode, pStartNode->GetChild(i),refList2);
			
			if (pNode != NULL ) break;
		} else {
			pNode = pStartNode->GetChild(i);
			
			break;
		}
	}
	
	
	return pNode;

}


// ===========================================================================

/*!
  \brief Start the presenter running.

  This is called once the scene has been fully loaded.  This function is
  called every time that a scene is loaded.
*/

void Presenter::Start()
{
    // Reset the timebase to start from now onwards
	m_dwStartTime = timeGetTime ();
    GetScene()->ResetTime();

	ExtractWorldInfo();

	// Set the pointer to NULL ready for the next scene that is loaded.
	m_pWorldInfo = NULL;
} /* Presenter::Start */


// ===========================================================================

/*!
  \brief Stop rendering.

  This function is called when the scene is finished.

  (It appears that the framework does not call this function)
*/

BOOL Presenter::Stop()
{
	SetWorldInfo(NULL);
	ExtractWorldInfo();

	return TRUE;
} /* Presenter::Stop */


// ===========================================================================

/*
  \brief Set which node is the WorldInfo node.

  Record the given pointer to the scene's WorldInfo node.  Use NULL if there
  is no WorldInfo node.
*/

void Presenter::SetWorldInfo(WorldInfo* pWorldInfo)
{
	m_pWorldInfo = pWorldInfo;
}


// ===========================================================================

/*!
  \brief Extract the details of the scene's WorldInfo node.

  This function is called just after the scene has been loaded.
  This function records the title and info fields of the given WorldInfo node
  for later reference.  A pointer to the node cannot be recorded because the
  node is deleted as soon as the scene is finished with.

  If there is no WorldInfo node, then any recorded WorldInfo data will be
  deleted.
*/

void Presenter::ExtractWorldInfo()
{
	// Delete any existing information
	if (m_pWorldInfoTitle)
	{
		delete m_pWorldInfoTitle;
		m_pWorldInfoTitle = NULL;
	}
	if (m_pWorldInfoInfo)
	{
		delete m_pWorldInfoInfo;
		m_pWorldInfoInfo = NULL;
	}

	// Create a local copy of the fields if they are given
	if (m_pWorldInfo)
	{

		//satish  commented this line
		//CString is a microsoft foundation class
// WIN-SOL release
#ifdef WIN32
		m_pWorldInfoTitle = new CString(m_pWorldInfo->title);
#else
		//satish added this block of code as a replacement for above line
		if(m_pWorldInfo->title){

			int title_length = strlen(m_pWorldInfo->title);
			m_pWorldInfoTitle = new char[title_length];
			strcpy(m_pWorldInfoTitle,m_pWorldInfo->title);
		}
#endif



		// Join the info strings into a single string
		if (m_pWorldInfo->info.GetLength() > 0)
		{
			//Satish commented this line
// WIN-SOL release
#ifdef WIN32
			m_pWorldInfoInfo = new CString((LPCSTR)m_pWorldInfo->info[0]);
#else
			//the following three lines are added by Satish

			int infoInfo_length = strlen((LPCSTR)m_pWorldInfo->info[0]);
			m_pWorldInfoInfo = new char[infoInfo_length];
			strcpy(m_pWorldInfoInfo ,(LPCSTR)m_pWorldInfo->info[0]);
			temp = new char[infoInfo_length];
			strcpy(temp,m_pWorldInfoInfo);
#endif


			for (int i = 1; i < m_pWorldInfo->info.GetLength(); i++)
			{

				//satish commented these two lines
// WIN-SOL release
#ifdef WIN32
				*m_pWorldInfoInfo += "\r\n";
				*m_pWorldInfoInfo += m_pWorldInfo->info[i];
#else
			//the following lines of code are written as a replacement for
			// for the above two lines of code(satish)

				infoInfo_length += (2+ strlen(m_pWorldInfo->info[i]));
				temp = new char[infoInfo_length];
				strcat(temp,"\r\n");
				strcat(temp,m_pWorldInfo->info[i]);
#endif

			}
// WIN-SOL release
#ifndef WIN32
			//satish added these lines of code
			m_pWorldInfoInfo = new char[strlen(temp)];
			strcpy(m_pWorldInfoInfo,temp);
			delete [] temp;
#endif

		}
	}
} /* Presenter::ExtractWorldInfo */


//ML - changed void Presenter::InitCLASS to RenderingObject* Presenter::CreateCLASSProxy
// init MediaObject

RenderingObject* Presenter::CreateMediaObjectProxy(MediaObject* /* pObject */)
{
  return NULL;
}


//ML - removed this function
// init GroupNode

/*
virtual void
Presenter::InitGroupNode(GroupNode * pObject)
{
  InitMediaObject(pObject);
}
*/

// init StreamConsumer

RenderingObject* Presenter::CreateStreamConsumerProxy(StreamConsumer * pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init Node

/* virtual */
/***
void
Presenter::InitNode(Node * pObject)
{
  printf("ding\n");

  InitMediaObject(pObject);
}
***/

// init Node3D

/* virtual */
/***
void
Presenter::InitNode3D(Node3D * pObject)
{
  InitNode(pObject);
}
***/

// init Switch

RenderingObject* Presenter::CreateSwitchProxy(Switch* pObject)
{
  // InitGroupNode(pObject);
  return CreateMediaObjectProxy(pObject);
}

// init Group

RenderingObject* Presenter::CreateGroupProxy(Group * pObject)
{
  //ML - commented out next line
  //InitGroupNode(pObject);
  return CreateMediaObjectProxy(pObject);
}

// init Transform

RenderingObject* Presenter::CreateTransformProxy(Transform* pObject)
{
  //ML - commented out next line
  //InitGroupNode(pObject);
  return CreateMediaObjectProxy(pObject);
}

// init Billboard

RenderingObject* Presenter::CreateBillboardProxy(Billboard* pObject)
{
  //ML - commented out next line
  //InitGroupNode(pObject);
  return CreateMediaObjectProxy(pObject);
}

// init Anchor

RenderingObject* Presenter::CreateAnchorProxy(Anchor* pObject)
{
  //ML - commented out next line
  //InitGroupNode(pObject);
  return CreateMediaObjectProxy(pObject);
}

// init Collision

RenderingObject* Presenter::CreateCollisionProxy(Collision* pObject)
{
  //ML - commented out next line
  //InitGroupNode(pObject);
  return CreateMediaObjectProxy(pObject);
}

// init Node2D

/* virtual */
/***
void
Presenter::InitNode2D(Node2D* pObject)
{
  InitNode(pObject);
}
***/

// init Conditional

RenderingObject* Presenter::CreateConditionalProxy(Conditional* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init Switch2D

#ifdef PRE_FDIS
RenderingObject* Presenter::CreateSwitch2DProxy(Switch2D* pObject)
{
  //ML - commented out next line
  //InitGroupNode(pObject);
  return CreateMediaObjectProxy(pObject);
}
#endif // PRE_FDIS

// init Group2D

#ifdef PRE_FDIS
RenderingObject* Presenter::CreateGroup2DProxy(Group2D* pObject)
{
  //ML - commented out next line
  //InitGroupNode(pObject);
  return CreateMediaObjectProxy(pObject);
}
#endif // PRE_FDIS

// init Transform2D

RenderingObject* Presenter::CreateTransform2DProxy(Transform2D* pObject)
{
  //ML - commented out next line
  //InitGroupNode(pObject);
  return CreateMediaObjectProxy(pObject);
}

// init Anchor2D

#ifdef PRE_FDIS
RenderingObject* Presenter::CreateAnchor2DProxy(Anchor2D* pObject)
{
  //ML - commented out next line
  //InitGroupNode(pObject);
  return CreateMediaObjectProxy(pObject);
}
#endif // PRE_FDIS

// init Layout

RenderingObject* Presenter::CreateLayoutProxy(Layout* pObject)
{
  //ML - commented out next line
  //InitGroupNode(pObject);
  return CreateMediaObjectProxy(pObject);
}

// init Form

RenderingObject* Presenter::CreateFormProxy(Form* pObject)
{
  //ML - commented out next line
  //InitGroupNode(pObject);
  return CreateMediaObjectProxy(pObject);
}

// init LOD

RenderingObject* Presenter::CreateLODProxy(LOD* pObject)
{
  // InitGroupNode(pObject);
  return CreateMediaObjectProxy(pObject);
}

// init TextureNode2D

/* virtual */
/***
void
Presenter::InitTextureNode2D(TextureNode2D* pObject)
{
  InitNode2D(pObject);
}
***/

// init VideoObject2D

#ifdef PRE_FDIS
RenderingObject* Presenter::CreateVideoObject2DProxy(VideoObject2D* pObject)
{
  return CreateStreamConsumerProxy(pObject);
}
#endif // PRE_FDIS

// init Image2D

#ifdef PRE_FDIS
RenderingObject* Presenter::CreateImage2DProxy(Image2D* pObject)
{
  return CreateStreamConsumerProxy(pObject);
}
#endif // PRE_FDIS

// init Inline2D

#ifdef PRE_FDIS
RenderingObject* Presenter::CreateInline2DProxy(Inline2D* pObject)
{
  return CreateStreamConsumerProxy(pObject);
}
#endif // PRE_FDIS

// init Bindable

/* virtual */
/***
void
Presenter::InitBindable(Bindable* pObject)
{
  InitNode(pObject);
}
***/

// init Viewpoint

RenderingObject* Presenter::CreateViewpointProxy(Viewpoint* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init WorldInfo

RenderingObject* Presenter::CreateWorldInfoProxy(WorldInfo* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init Fog

RenderingObject* Presenter::CreateFogProxy(Fog* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init Background

RenderingObject* Presenter::CreateBackgroundProxy(Background* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init NavigationInfo

RenderingObject* Presenter::CreateNavigationInfoProxy(NavigationInfo* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init InterpolatorNode

/* virtual */
/***
void
Presenter::InitInterpolatorNode(InterpolatorNode* pObject)
{
  InitNode(pObject);
}
***/

// init Inline

RenderingObject* Presenter::CreateInlineProxy(Inline* pObject)
{
  return CreateStreamConsumerProxy(pObject);
}

// init AnimationStream

RenderingObject* Presenter::CreateAnimationStreamProxy(AnimationStream* pObject)
{
  return CreateStreamConsumerProxy(pObject);
}

// init TermCap

RenderingObject* Presenter::CreateTermCapProxy(TermCap* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init TimeSensor

RenderingObject* Presenter::CreateTimeSensorProxy(TimeSensor* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init ProximitySensor

RenderingObject* Presenter::CreateProximitySensorProxy(ProximitySensor* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init VisibilitySensor

RenderingObject* Presenter::CreateVisibilitySensorProxy(VisibilitySensor* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init DragSensor

/* virtual */
/***
void
Presenter::InitDragSensor(DragSensor* pObject)
{
  InitMediaObject(pObject);
}
***/

// init PlaneSensor

RenderingObject* Presenter::CreatePlaneSensorProxy(PlaneSensor* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init PlaneSensor2D

RenderingObject* Presenter::CreatePlaneSensor2DProxy(PlaneSensor2D* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init SphereSensor

RenderingObject* Presenter::CreateSphereSensorProxy(SphereSensor* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init CylinderSensor

RenderingObject* Presenter::CreateCylinderSensorProxy(CylinderSensor* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init DiscSensor

RenderingObject* Presenter::CreateDiscSensorProxy(DiscSensor* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init TouchSensor

RenderingObject* Presenter::CreateTouchSensorProxy(TouchSensor* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init Appearance

RenderingObject* Presenter::CreateAppearanceProxy(Appearance* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init MaterialNode

/* virtual */
/***
void
Presenter::InitMaterialNode(MaterialNode* pObject)
{
  InitMediaObject(pObject);
}
***/

// init Material

RenderingObject* Presenter::CreateMaterialProxy(Material* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init TextureNode

/* virtual */
/***
void
Presenter::InitTextureNode(TextureNode* pObject)
{
  InitMediaObject(pObject);
}
***/

// init ImageTexture

RenderingObject* Presenter::CreateImageTextureProxy(ImageTexture* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init FontStyle

RenderingObject* Presenter::CreateFontStyleProxy(FontStyle* pObject)
{
  return CreateMediaObjectProxy(pObject);
}


// init MovieTexture

RenderingObject* Presenter::CreateMovieTextureProxy(MovieTexture* pObject)
{
  return CreateStreamConsumerProxy(pObject);
}

// init PixelTexture

RenderingObject* Presenter::CreatePixelTextureProxy(PixelTexture* pObject)
{
  //ML - changed the next line (PixelTexture derives from MediaObject)
  //InitStreamConsumer(pObject);
  return CreateMediaObjectProxy(pObject);
}

// init Material2D

RenderingObject* Presenter::CreateMaterial2DProxy(Material2D* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init LineProperties

RenderingObject* Presenter::CreateLinePropertiesProxy(LineProperties* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init Coordinate

RenderingObject* Presenter::CreateCoordinateProxy(Coordinate* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init Coordinate2D

RenderingObject* Presenter::CreateCoordinate2DProxy(Coordinate2D* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init Normal

RenderingObject* Presenter::CreateNormalProxy(Normal* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init TextureCoordinate

RenderingObject* Presenter::CreateTextureCoordinateProxy(TextureCoordinate* pObject)
{
  return CreateMediaObjectProxy(pObject);
}


// init Color

RenderingObject* Presenter::CreateColorProxy(Color* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init TextureTransform

RenderingObject* Presenter::CreateTextureTransformProxy(TextureTransform* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init Shape

RenderingObject* Presenter::CreateShapeProxy(Shape* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init Geometry

/* virtual */
/***
void
Presenter::InitGeometry(Geometry* pObject)
{
  InitMediaObject(pObject);
}
***/

// init Circle

RenderingObject* Presenter::CreateCircleProxy(Circle* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init Rectangle

RenderingObject* Presenter::CreateRectangleProxy(class Rectangle* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init IndexedFaceSet2D

RenderingObject* Presenter::CreateIndexedFaceSet2DProxy(IndexedFaceSet2D* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init IndexedLineSet2D

RenderingObject* Presenter::CreateIndexedLineSet2DProxy(IndexedLineSet2D* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init PointSet2D

RenderingObject* Presenter::CreatePointSet2DProxy(PointSet2D* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init ElevationGrid

RenderingObject* Presenter::CreateElevationGridProxy(ElevationGrid* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init Extrusion

RenderingObject* Presenter::CreateExtrusionProxy(Extrusion* pObject)
{
  return CreateIndexedFaceSetProxy((IndexedFaceSet *) (MediaObject *) pObject);
}

// init Box

RenderingObject* Presenter::CreateBoxProxy(Box* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init Sphere

RenderingObject* Presenter::CreateSphereProxy(Sphere* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init Cylinder

RenderingObject* Presenter::CreateCylinderProxy(Cylinder* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init Cone

RenderingObject* Presenter::CreateConeProxy(Cone* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init IndexedFaceSet

RenderingObject* Presenter::CreateIndexedFaceSetProxy(IndexedFaceSet* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init IndexedLineSet

RenderingObject* Presenter::CreateIndexedLineSetProxy(IndexedLineSet* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init PointSet

RenderingObject* Presenter::CreatePointSetProxy(PointSet* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init Text

RenderingObject* Presenter::CreateTextProxy(Text* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init StreamingText

#ifdef PRE_FDIS
RenderingObject* Presenter::CreateStreamingTextProxy(StreamingText* pObject)
{
  return CreateStreamConsumerProxy(pObject);
}
#endif // PRE_FDIS

// init Light

/* virtual */
/***
void
Presenter::InitLight(Light* pObject)
{
  InitNode3D(pObject);
}
***/

// init DirectionalLight

RenderingObject* Presenter::CreateDirectionalLightProxy(DirectionalLight* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init PointLight

RenderingObject* Presenter::CreatePointLightProxy(PointLight* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init SpotLight

RenderingObject* Presenter::CreateSpotLightProxy(SpotLight* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init Face

RenderingObject* Presenter::CreateFaceProxy(Face* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init FIT

RenderingObject* Presenter::CreateFITProxy(FIT* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init FDP

RenderingObject* Presenter::CreateFDPProxy(FDP* pObject)
{
  // InitGroupNode(pObject);
  return CreateMediaObjectProxy(pObject);
}

// init FAP

RenderingObject* Presenter::CreateFAPProxy(FAP* pObject)
{
  //ML - changed the next line (FAP derives from MediaObject)
  //InitStreamConsumer(pObject);
  return CreateMediaObjectProxy(pObject);
}

// init Expression

RenderingObject* Presenter::CreateExpressionProxy(Expression* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init Viseme

RenderingObject* Presenter::CreateVisemeProxy(Viseme* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init FaceDefTables

RenderingObject* Presenter::CreateFaceDefTablesProxy(FaceDefTables* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init FaceDefMesh

RenderingObject* Presenter::CreateFaceDefMeshProxy(FaceDefMesh* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init FaceDefTransform

RenderingObject* Presenter::CreateFaceDefTransformProxy(FaceDefTransform* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init Sound

RenderingObject* Presenter::CreateSoundProxy(Sound* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init Sound2D

RenderingObject* Presenter::CreateSound2DProxy(Sound2D* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// init AudioObject

/***
/& virtual &/ void
Presenter::InitAudioObject(AudioObject* pObject)
{
  InitMediaObject(pObject);
}
***/

// init AudioSource

RenderingObject* Presenter::CreateAudioSourceProxy(AudioSource* pObject)
{
  // InitAudioObject(pObject);
  return CreateMediaObjectProxy(pObject);
}

// init AudioSwitch

RenderingObject* Presenter::CreateAudioSwitchProxy(AudioSwitch* pObject)
{
  // InitAudioObject(pObject);
  return CreateMediaObjectProxy(pObject);
}

// init AudioFX

RenderingObject* Presenter::CreateAudioFXProxy(AudioFX* pObject)
{
  // InitAudioObject(pObject);
  return CreateMediaObjectProxy(pObject);
}

// init AudioMix

RenderingObject* Presenter::CreateAudioMixProxy(AudioMix* pObject)
{
  // InitAudioObject(pObject);
  return CreateMediaObjectProxy(pObject);
}

// init AudioDelay

RenderingObject* Presenter::CreateAudioDelayProxy(AudioDelay* pObject)
{
  // InitAudioObject(pObject);
  return CreateMediaObjectProxy(pObject);
}

// init Script

RenderingObject* Presenter::CreateScriptProxy(Script* pObject)
{
  return CreateMediaObjectProxy(pObject);
}

// delete object

void Presenter::DeleteObject(MediaObject* /* pObject */)
{
}


// **************************************************************************

// SB @@@@ added on 99/01/20 for new Nodes >>>>

RenderingObject* Presenter::CreateAudioBufferProxy(AudioBuffer * poNode)
{
  //ML - commented out next line
  //InitGroupNode(poNode);
  return CreateMediaObjectProxy(poNode);
}

RenderingObject* Presenter::CreateLayer2DProxy(Layer2D * poNode)
{
  //ML - commented out next line
  //InitGroupNode(poNode);
  return CreateMediaObjectProxy(poNode);
}

RenderingObject* Presenter::CreateLayer3DProxy(Layer3D * poNode)
{
  //ML - commented out next line
  //InitGroupNode(poNode);
  return CreateMediaObjectProxy(poNode);
}

RenderingObject* Presenter::CreateOrderedGroupProxy(OrderedGroup * poNode)
{
  //ML - commented out next line
  //InitGroupNode(poNode);
  return CreateMediaObjectProxy(poNode);
}

RenderingObject* Presenter::CreateAudioClipProxy(AudioClip * poNode)
{ return CreateStreamConsumerProxy(poNode); }

RenderingObject* Presenter::CreateCompositeTexture2DProxy(CompositeTexture2D * poNode)
{
  //ML - changed the next line (CompositeTexture2D derives from MediaObject)
  //InitStreamConsumer(poNode);
  return CreateMediaObjectProxy(poNode);
}

RenderingObject* Presenter::CreateCompositeTexture3DProxy(CompositeTexture3D * poNode)
{
  //ML - changed the next line (CompositeTexture3D derives from MediaObject)
  //InitStreamConsumer(poNode);
  return CreateMediaObjectProxy(poNode);
}

RenderingObject* Presenter::CreateBackground2DProxy(Background2D * poNode)
{ return CreateMediaObjectProxy(poNode); }
RenderingObject* Presenter::CreateBitmapProxy(Bitmap * poNode)
{ return CreateMediaObjectProxy(poNode); }
RenderingObject* Presenter::CreateColorInterpolatorProxy(ColorInterpolator * poNode)
{ return CreateMediaObjectProxy(poNode); }
RenderingObject* Presenter::CreateCoordinateInterpolatorProxy(CoordinateInterpolator * poNode)
{ return CreateMediaObjectProxy(poNode); }
RenderingObject* Presenter::CreateCoordinateInterpolator2DProxy(CoordinateInterpolator2D * poNode)
{ return CreateMediaObjectProxy(poNode); }
RenderingObject* Presenter::CreateCurve2DProxy(Curve2D * poNode)
{ return CreateMediaObjectProxy(poNode); }
RenderingObject* Presenter::CreateListeningPointProxy(ListeningPoint * poNode)
{ return CreateMediaObjectProxy(poNode); }
RenderingObject* Presenter::CreateNormalInterpolatorProxy(NormalInterpolator * poNode)
{ return CreateMediaObjectProxy(poNode); }
RenderingObject* Presenter::CreateOrientationInterpolatorProxy(OrientationInterpolator * poNode)
{ return CreateMediaObjectProxy(poNode); }
RenderingObject* Presenter::CreatePositionInterpolatorProxy(PositionInterpolator * poNode)
{ return CreateMediaObjectProxy(poNode); }
RenderingObject* Presenter::CreatePositionInterpolator2DProxy(PositionInterpolator2D * poNode)
{ return CreateMediaObjectProxy(poNode); }
RenderingObject* Presenter::CreateProximitySensor2DProxy(ProximitySensor2D * poNode)
{ return CreateMediaObjectProxy(poNode); }
RenderingObject* Presenter::CreateQuantizationParameterProxy(QuantizationParameter * poNode)
{ return CreateMediaObjectProxy(poNode); }
RenderingObject* Presenter::CreateScalarInterpolatorProxy(ScalarInterpolator * poNode)
{ return CreateMediaObjectProxy(poNode); }
RenderingObject* Presenter::CreateValuatorProxy(Valuator * poNode)
{ return CreateMediaObjectProxy(poNode); }

// SB @@@@ added on 99/01/20 for new Nodes <<<<


//ML - removed the following descriptor nodes
#if 0
// SB @@@@ added on 99/01/20 for descriptors Nodes >>>>
void Presenter::InitObjectDescriptor(MediaObject * /* poNode */) {}
void Presenter::InitES_Descriptor(MediaObject * /* poNode */) {}
void Presenter::InitDecoderConfigDescriptor(MediaObject * /* poNode */) {}
void Presenter::InitBIFSDecoderConfig(MediaObject * /* poNode */) {}
void Presenter::InitIPI_Descriptor(MediaObject * /* poNode */) {}
void Presenter::InitIP_InformationDataSet(MediaObject * /* poNode */) {}
void Presenter::InitSupplementaryContentIdentifier(MediaObject * /* poNode */) {}
void Presenter::InitQoS_Descriptor(MediaObject * /* poNode */) {}
void Presenter::InitQoS_Qualifier(MediaObject * /* poNode */) {}
void Presenter::InitExtensionDescriptor(MediaObject * /* poNode */) {}
void Presenter::InitShortExtensionDescriptor(MediaObject * /* poNode */) {}
void Presenter::InitIPMP_DescriptorPointer(MediaObject * /* poNode */) {}
void Presenter::InitALConfigDescriptor(MediaObject * /* poNode */) {}
void Presenter::InitIPMPDescriptor(MediaObject * /* poNode */) {}
// SB @@@@ added on 99/01/20 for descriptors Nodes <<<<
#endif


#ifdef ADV_AUDIO
RenderingObject* Presenter::CreateAcousticMaterialProxy(AcousticMaterial * poNode)
{ return CreateMediaObjectProxy(poNode); }
RenderingObject* Presenter::CreateAcousticSceneProxy(AcousticScene * poNode)
{ return CreateMediaObjectProxy(poNode); }
RenderingObject* Presenter::CreateDirectiveSoundProxy(DirectiveSound * poNode)
{ return CreateMediaObjectProxy(poNode); }
RenderingObject* Presenter::CreatePerceptualParametersProxy(PerceptualParameters * poNode)
{ return CreateMediaObjectProxy(poNode); }

#endif // ADV_AUDIO


//ML - added the following nodes
RenderingObject* Presenter::CreateBAPProxy(BAP * /* poNode */)
	{ return NULL; }
RenderingObject* Presenter::CreateBDPProxy(BDP * /* poNode */)
	{ return NULL; }
RenderingObject* Presenter::CreateBodyProxy(Body * /* poNode */)
	{ return NULL; }
RenderingObject* Presenter::CreateBodyDefTableProxy(BodyDefTable * /* poNode */)
	{ return NULL; }
RenderingObject* Presenter::CreateBodySegmentConnectionHintProxy(BodySegmentConnectionHint * /* poNode */)
	{ return NULL; }
RenderingObject* Presenter::CreateHierarchical3DMeshProxy(Hierarchical3DMesh * /* poNode */)
	{ return NULL; }
RenderingObject* Presenter::CreateMaterialKeyProxy(MaterialKey * /* poNode */)
	{ return NULL; }
RenderingObject* Presenter::CreateServerCommandProxy(ServerCommand * /* poNode */)
	{ return NULL; }

RenderingObject* Presenter::CreateTemporalTransformProxy(TemporalTransform*)
	{ return NULL; }
RenderingObject* Presenter::CreateTemporalGroupProxy(TemporalGroup*)
	{ return NULL; }
RenderingObject* Presenter::CreateMediaSensorProxy(MediaSensor*)
	{ return NULL; }
RenderingObject* Presenter::CreateMediaControlProxy(MediaControl*)
	{ return NULL; }

RenderingObject* Presenter::CreateApplicationWindowProxy(ApplicationWindow*)
    { return NULL; }

  // SOLARIS PORTING - Core 5.7rc1 - Added these three nodes
RenderingObject* Presenter::CreateInputSensorProxy(InputSensor*)
    { return NULL; }

RenderingObject* Presenter::CreateMatteTextureProxy(MatteTexture*)
    { return NULL; }

RenderingObject* Presenter::CreateMediaBufferProxy(MediaBuffer*)
    { return NULL; }

// SOLARIS PORTING - seems to have been omitted
//RenderingObject* Presenter::CreateParticleInitConeProxy(ParticleInitCone *){ return NULL; }
// SOLARIS PORTING - Core 5.9rc9 - END of new nodes

// SOLARIS PORTING - Merging code with 2D player
RenderingObject* Presenter::CreateClipper2DProxy(Clipper2D *){ return NULL; }
RenderingObject* Presenter::CreateColorTransformProxy(ColorTransform *){ return NULL; }
RenderingObject* Presenter::CreateTransformMatrix2DProxy(TransformMatrix2D *){ return NULL; }
RenderingObject* Presenter::CreateViewportProxy(Viewport *){ return NULL; }
RenderingObject* Presenter::CreateXCurve2DProxy(XCurve2D *){ return NULL; }
RenderingObject* Presenter::CreateXLinePropertiesProxy(XLineProperties *){ return NULL; }

// AFX nodes
RenderingObject* Presenter::CreatePointTextureProxy(PointTexture *){ return NULL; }
RenderingObject* Presenter::CreatePositionAnimatorProxy(PositionAnimator *poNode) { return CreateMediaObjectProxy(poNode); }
RenderingObject* Presenter::CreatePositionAnimator2DProxy(PositionAnimator2D *poNode) { return CreateMediaObjectProxy(poNode); }
RenderingObject* Presenter::CreateProceduralTextureProxy(ProceduralTexture *){ return NULL; }
RenderingObject* Presenter::CreateSBBoneProxy(SBBone *poNode){ return  CreateMediaObjectProxy(poNode);}
RenderingObject* Presenter::CreateSBMuscleProxy(SBMuscle *poNode){ return CreateMediaObjectProxy(poNode); }
RenderingObject* Presenter::CreateSBSegmentProxy(SBSegment *poNode){ return CreateMediaObjectProxy(poNode); }
RenderingObject* Presenter::CreateSBSiteProxy(SBSite *poNode){ return CreateMediaObjectProxy(poNode); }
RenderingObject* Presenter::CreateSBSkinnedModelProxy(SBSkinnedModel *poNode){ return CreateMediaObjectProxy(poNode); }
RenderingObject* Presenter::CreateSBVCAnimationProxy(SBVCAnimation *poNode){ return CreateMediaObjectProxy(poNode); }
RenderingObject* Presenter::CreateScalarAnimatorProxy(ScalarAnimator *poNode) { return CreateMediaObjectProxy(poNode); }
RenderingObject* Presenter::CreateSimpleTextureProxy(SimpleTexture *){ return NULL; }
RenderingObject* Presenter::CreateSubdivisionSurfaceProxy(SubdivisionSurface *){ return NULL; }
RenderingObject* Presenter::CreateSubdivSurfaceSectorProxy(SubdivSurfaceSector *){ return NULL; }
RenderingObject* Presenter::CreateSynthesizedTextureProxy(SynthesizedTexture *){ return NULL; }
RenderingObject* Presenter::CreateWaveletSubdivisionSurfaceProxy(WaveletSubdivisionSurface *poNode){ return CreateMediaObjectProxy(poNode); }
RenderingObject* Presenter::CreateCoordinateInterpolator4DProxy(CoordinateInterpolator4D *poNode) { return CreateMediaObjectProxy(poNode); }
RenderingObject* Presenter::CreatePositionInterpolator4DProxy(PositionInterpolator4D *poNode) { return CreateMediaObjectProxy(poNode); }
RenderingObject* Presenter::CreateBitWrapperProxy(BitWrapper *poNode) { return CreateMediaObjectProxy(poNode); }
RenderingObject* Presenter::CreateDepthImageProxy(DepthImage *poNode) { return CreateMediaObjectProxy(poNode); }
RenderingObject* Presenter::CreateFFDProxy(FFD *poNode) { return CreateMediaObjectProxy(poNode); }
RenderingObject* Presenter::CreateMeshGridProxy(MeshGrid *poNode) { return CreateMediaObjectProxy(poNode); }
RenderingObject* Presenter::CreateNonLinearDeformerProxy(NonLinearDeformer *poNode) { return CreateMediaObjectProxy(poNode); }
RenderingObject* Presenter::CreateNurbsCurveProxy(NurbsCurve *poNode) { return CreateMediaObjectProxy(poNode); }
RenderingObject* Presenter::CreateNurbsCurve2DProxy(NurbsCurve2D *poNode) { return CreateMediaObjectProxy(poNode); }
RenderingObject* Presenter::CreateNurbsSurfaceProxy(NurbsSurface *poNode) { return CreateMediaObjectProxy(poNode); }
RenderingObject* Presenter::CreateOctreeImageProxy(OctreeImage *poNode) { return CreateMediaObjectProxy(poNode); }

RenderingObject* Presenter::CreateImplicitProxy(Implicit *poNode) { return CreateMediaObjectProxy(poNode); }
RenderingObject* Presenter::CreateSolidRepProxy(SolidRep *poNode){ return CreateMediaObjectProxy(poNode); }
RenderingObject* Presenter::CreateQuadricProxy(Quadric *poNode){ return CreateMediaObjectProxy(poNode); }

// AFX/AMD1 nodes
RenderingObject* Presenter::CreateDepthImageV2Proxy(DepthImageV2 *) { return NULL; }
RenderingObject* Presenter::CreateMorphShapeProxy(MorphShape *pObject) { return CreateMediaObjectProxy(pObject); }
RenderingObject* Presenter::CreatePointTextureV2Proxy(PointTextureV2 *) { return NULL; }
RenderingObject* Presenter::CreateSimpleTextureV2Proxy(SimpleTextureV2 *) {	return NULL; }
RenderingObject* Presenter::CreateMultiTextureProxy(MultiTexture* pObject) { return CreateMediaObjectProxy(pObject); }
RenderingObject* Presenter::CreateMultiTextureCoordinateProxy(MultiTextureCoordinate* pObject) { return CreateMediaObjectProxy(pObject); }

RenderingObject* Presenter::CreateFootPrintSetProxy(FootPrintSet* pObject) { return CreateMediaObjectProxy(pObject); }
RenderingObject* Presenter::CreateFootPrintProxy(FootPrint* pObject) { return CreateMediaObjectProxy(pObject); }
RenderingObject* Presenter::CreateBuildingPartProxy(BuildingPart *pObject) { return CreateMediaObjectProxy(pObject); }
RenderingObject* Presenter::CreateRoofProxy(Roof *pObject) { return CreateMediaObjectProxy(pObject); }
RenderingObject* Presenter::CreateFacadeProxy(Facade *pObject) { return CreateMediaObjectProxy(pObject); }

// end of AFX nodes

// version 6 nodes
RenderingObject* Presenter::CreateXEllipseProxy(XEllipse *) { return NULL; }
RenderingObject* Presenter::CreateLinearGradientProxy(LinearGradient *) { return NULL; }
RenderingObject* Presenter::CreateRadialGradientProxy(RadialGradient *) { return NULL; }
RenderingObject* Presenter::CreateXFontStyleProxy(XFontStyle *) { return NULL; }
RenderingObject* Presenter::CreatePathLayoutProxy(PathLayout *) { return NULL; }
// end of version 6 nodes

// version 7 nodes
RenderingObject* Presenter::CreateWideSoundProxy(WideSound *) { return NULL; }
RenderingObject* Presenter::CreateTransform3DAudioProxy(Transform3DAudio *) { return NULL; }
RenderingObject* Presenter::CreateSurroundingSoundProxy(SurroundingSound *) { return NULL; }
RenderingObject* Presenter::CreateAudioChannelConfigProxy(AudioChannelConfig *) { return NULL; }
RenderingObject* Presenter::CreateAdvancedAudioBufferProxy(AdvancedAudioBuffer *) { return NULL; }
RenderingObject* Presenter::CreateSBVCAnimationV2Proxy(SBVCAnimationV2 *pObject) { return CreateMediaObjectProxy(pObject); }

//added by Helge Drumm TU Ilmenau, 6.12.2006, Shadow Rendering
RenderingObject* Presenter::CreateShadowProxy(Shadow* pObject) { return CreateMediaObjectProxy(pObject);}

// end of version 7 nodes
