/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************
// SectionFilterEngine.cpp
//
// startSectionFilterEngine):main procedure for section extraction using M2DecoderPlus library.
// JVMLauch: this procedure represent the body of a thread launched by the M2DecTest code in order to instantiate the JVM
// and to execute a Java application.
// Author V. 1.0:  R.Becchini, Societa' Italiana Avionica (19-4-1999).
//
/*-------------------------------------------------------------------------
MPEGJ 
AUTHORS:    Viswanathan Swaminathan - vishy@eng.sun.com 
			Robert Schmidt(RLS)
			Praveen Alavilli(PA) 
			YUNTAI KYONG(YT) - yuntaikyong@ieee.org
-------------------------------------------------------------------------*/
// WIN SOL Release
#ifdef WIN32

#include "defs.h"
#include "resource.h"
#include "..\M2DecoderPlus\global.h" 
#include "..\Mpgj\SI_api_Native\SFilterBridge.h"
//#include "MPEG4Player.h"
//#include "M2DecoderPlus.h"


#define MASKLEN 12

static M2DECFILTERINFO   filterInfo ;
static M2DECSECTIONDATA  sectionData ;
static int filterActionId;
static bool	notStoppedSF; // indicates when the stopSectionFilter has not been called after a call to startSectionFilter

// BB : Make internal variable and access via MPEG4player's get method.
 M2DecoderPlus M2DecObj; 
//M2DecoderPlus* mpegj_M2DecObj;
//extern M2DecoderPlus* getM2DecObj();
// BB: This object now replaces M2DecObj
//M2DecoderPlus* myDecObj;

//extern SommitSfInterface l_cpp;

// BB: Made internally here, external in the MPEG4Player, to resolve link error.
// extern bool M2DOk;
bool M2DOk = false;

void CnewSection( int, unsigned char *, int );

void startSectionFilterEngine(int Pid, 
							  unsigned char *posMask, 
							  unsigned char *negMask, 
							  unsigned char *posValue, 
							  unsigned char *negValue,
							  unsigned int posLen,
							  unsigned int negLen
							  )
{

	// MessageBox(NULL, "*** DEBUG *** came into startSectionFilterEngine", "startStartFilterEngine", MB_OK); // PA - testing

	HRESULT M2Dval ;
	unsigned int i, count = 011;

	notStoppedSF = true;
	filterInfo.length = sizeof(M2DECFILTERINFO);
	filterInfo.PID = Pid;

	for(i = 0; i < posLen; i++)
	{
		filterInfo.posMask[i] = posMask[i];
		filterInfo.posValue[i] = posValue[i];
	}
	for(i = 0; i < negLen; i++)
	{
		filterInfo.negMask[i] = negMask[i];
		filterInfo.negValue[i] = negValue[i];
	}

	filterInfo.negLen = negLen;
	filterInfo.posLen = posLen;
	
	
	
	do{
		M2Dval = M2DecObj.M2DecStartSectionFilter(&filterInfo) ;		

	/*  if( !M2DOk ) {
			MessageBox(NULL, " *** DEBUG *** exiting M2DecStartSectionFilter", "startStartFilterEngine", MB_OK); // PA - testing
			M2DOk = TRUE;
		} */ // PA - testing whether functioning properly or not in case of M2DOK set to TRUE
	}while(!M2DOk);
	
	switch(M2Dval){
		case M2D_OK: break;
		case M2D_ERROR:
			{
				MessageBox(NULL, "M2D_ERROR: errore generico", "Start section filter", MB_OK);
				return;
			}
		case M2D_INVALIDPARAMETER:
			{
				MessageBox(NULL, "M2D_INVALIDPARAMETER: valori dei parametri passati al metodo non corretti", "Start section filter", MB_OK);
				return;
			}
		case M2D_SECFILTERINIT:
			{
				MessageBox(NULL, "M2D_SECFILTERINIT: inizializzazione del thread Section filter", "Start section filter", MB_OK);
				return;
			}
		case M2D_NODEMUX:
			{
				MessageBox(NULL, "M2D_NODEMUX: thread demux non ancora creato", "Start section filter", MB_OK);
				return;
			}
		case M2D_NULLSECFILTERSTART:
			{
				MessageBox(NULL, "M2D_NULLSECFILTERSTART: chiamata al metodo M2DecStartDataDecoder prima di aver terminato il decoder section filter", "Start section filter", MB_OK);
				return;
			}
		case M2D_SFEVENT:
			{
				MessageBox(NULL, "M2D_SFEVENT: creazione fallita evento inizializzazione section filter", "Start section filter", MB_OK);
				return;
			}
		default: 
			{
				MessageBox(NULL, "Unknown return value", "Start section filter", MB_OK);
				return;
			}
	}

	sectionData.length = sizeof(sectionData);

	do 
	{  
		Sleep(10) ;
		count += 1;
		if(count == 3000){
			notStoppedSF = false;
			MessageBox(NULL, "Internal time out expired!!!", "*** DEBUG *** time out 30 seconds", MB_OK);
		}
		M2Dval = M2DecObj.M2DecGetNewSection(&sectionData) ;
		
		//callback
		//CnewSection(filterActionId, sectionData.data, sectionData.length);
	} while (M2Dval != M2D_OK && notStoppedSF) ; // No more sections
	count = 0;               
	
	//callback
	if (notStoppedSF)
		CnewSection(filterActionId, sectionData.data, sectionData.sectionLength);// ... send the section
	
//	M2Dval = M2DecObj->M2DecStopSectionFilter();
//	if(M2Dval == M2D_NULLSECTIONFILTERSTOP){
//		MessageBox(NULL, "Null section filter stop", "Start section filter", MB_OK);
//	}
	
	
	MessageBox(NULL, "*** DEBUG *** exiting startSectionFilterEngine", "startStartFilterEngine", MB_OK); // PA - testing
}

void stopSectionFilterEngine(int foo){
	foo;
	HRESULT M2Dval ;
	
	M2Dval = M2DecObj.M2DecStopSectionFilter();
	if(M2Dval == M2D_NULLSECTIONFILTERSTOP){
		MessageBox(NULL, "Null section filter stop", "Start section filter", MB_OK);
	}
	notStoppedSF = false;
}

void CstartFilter
    ( int index
    , int id
    , SommitSfInterface l
    , int PID
    , unsigned char * posMask
    , unsigned char * posValue
    , unsigned char * negMask
    , unsigned char * negValue
	, int posLen
	, int negLen
    )
{   
	index; // not used, as we only have one section filter
	//l_cpp = l;

	filterActionId = id;

/*	if (PID != filterInfo.PID)
	{
		stop_Channel(TABLE_CHANNEL);				
		setPid(PID, TABLE_CHANNEL);			
	} */

	startSectionFilterEngine(PID, posMask,negMask, posValue, negValue,posLen, negLen);
	l;
}

void CstopFilter( int index ){
	index; // unreferenced because in this implementation we only have one section filter
	stopSectionFilterEngine(0);
}

int setPid(unsigned int pid, unsigned int chan){
	pid = chan; // unreferenced because not implememted
	return 0;
}
#endif // WIN32

