// $Log: AQuantize.cpp,v $
// Revision 1.1.1.1  2003/12/24 00:55:03  mikeseven
// Initial release of the CVS repository for MPEG-4 Systems code
// after the code was corrupted due to reorganization and human 
// errors.
//
// Revision 1.2  2003/04/03 04:29:43  mikeseven
// Initial release of Player3D and MPEG-J for all MPEG-4 reference softwares players with samples.
// For installation, follow instructions in docs/Player3D_setup.doc.
//
// Many thanks to Prashanth Gujjeti and Viswanathan Swaminathan from Sun Microsystems.
//
// -- Mikael Bourges-Sevenier (Mindego)
//
// Revision 1.1.1.2  2003/01/30 16:28:28  payan
// no message
//
// Revision 1.1.1.1  2002/07/18 01:08:23  mikeseven
// Initial check in of July 17, 2002 version
//
//
//Revision 1.x 2000/10/12  Pascal Belin
//QP bug fix
//
// Revision 1.1.2.1  2000/03/10 15:59:32  Alex Cotarmanac'h
// deuxime envoi
//
// Revision 1.1  2000/03/03 18:41:21  Alex Cotarmanac'h
// Version 1 -- Alexandre Cotarmanac_h



#include "defs.h"

#ifdef PREDICTIVE	//AC 20000227

#include "Quantize.h"
#include "BifsDec.h"
#include "ztl.h"
#include "NodeFld.h"
#include <float.h>
#include <math.h>
#define PI 3.1415927
#define QPVAL(type, id) ((type) *(SF<type>*) m_pQP->GetField(id))
int Quantizer::GetQuantizationNbBits(int quantType) 
{
	int nBits=-1;
	switch(quantType) {
	case Q_ROTATION:
	case Q_NORMAL:
		nBits=QPVAL(int,NORMALNB);
		break;
	case Q_2D:
		nBits= QPVAL(int,POS2DNB);
		break;
	case Q_TEXTURECOORDINATE:
	    nBits= QPVAL(int,TCNB);
		break;
	case Q_3D:
		nBits= QPVAL(int,POS3DNB);
		break;
	case Q_DRAWORDER:
		nBits= QPVAL(int,DONB);
		break;
	case Q_COLOR:
		nBits= QPVAL(int,COLORNB);
		break;
	case Q_ANGLE:
		nBits= QPVAL(int,ANGLENB);
		break;
	case Q_SCALE:
		nBits= QPVAL(int,SCALENB);
		break;
	case Q_KEY:
		nBits= QPVAL(int,KEYNB);
		break;
	case Q_SIZE2D: 
	case Q_SIZE3D:
		nBits=QPVAL(int,SIZENB);
		break;
	case Q_LSQ:
		nBits=m_nBits;
		break;
	case Q_COORDIDX:
		nBits=m_nBits;
		break;
		//fprintf(stderr,"[Error:] Don't know how to handle Q_COORDIDX\n");
		break;
	}
  return nBits;
}

//
float Quantizer :: unQuantize(int vq,float IMin, float IMax, int INbBits, bool isUnitVector) 
{
	int nbBits = (isUnitVector)?INbBits-1:INbBits;
	float span = IMax - IMin;
	float epsilon = 1e-8f;
	float unQuantScale=(span <= epsilon )?0:(
				(nbBits==0)?span*.5f: (float) span/ (float)((1<<nbBits) - 1)
		);
//#define UNQUANTIZEDEBUG
#ifdef UNQUANTIZEDEBUG
	printf("[Message] unQuantize : resultat= %f vq= %d Scale=%f\n",IMin + unQuantScale * (float)vq,vq,unQuantScale);
#endif
    return IMin + unQuantScale * (float)vq;

}


int Quantizer :: unQuantizeInt (int vq)
{
  // MBS
  switch(m_nQuantizationCode) {
  case Q_DRAWORDER: {
    float mini=max(m_pMin[0],QPVAL(Vec3f,DOMIN).x);	
	int nBits= QPVAL(int,DONB);
	float maxi=mini + ((1<<nBits) - 1);
    return unQuantize(vq,mini,maxi,nBits,false);
    break;
  }
//AC 20000227
  case Q_COORDIDX:
	  {
		  int res=vq-1; // PB QP14 fix: 0 must stand for the minimum value, like in QP13
		  // if (res == ((1<<m_nBits)-1) ) res=-1;
		  // printf("[QP14] nBits:%d Value:%d\n",m_nBits,res);
		  return res;
		  break;
	  }
//AC end
  default: {
	/*-----------------------------------------------------------------
	Determine offset and number of bits
	-----------------------------------------------------------------*/
	int nBits = 32;
	if (m_nDimension == 1 &&
		m_pMin [0] != FLT_MIN && m_pMax [0] != FLT_MAX
	)
	{	
		int min = m_pMin [0];
		int max = m_pMax [0];
		nBits = BitFieldWidth (max - min + 1);
		return unQuantize(vq,min,max,nBits,false);
	}
	
  }
  }
  return -1;
  // end MBS
}

/*-----------------------------------------------------------------
Parse an float from the BIFS stream
-----------------------------------------------------------------*/
float Quantizer :: unQuantizeFloat (int vq) {
  float mini=0,maxi=0,nBits = 0;
    
  switch(m_nQuantizationCode) {
  case Q_DRAWORDER:
    //fprintf(stderr,"Dequant DrawOrder\n");
    mini=max(m_pMin[0],QPVAL(float,DOMIN));		  
    maxi=min(m_pMax[0],QPVAL(float,DOMAX));
    nBits= QPVAL(int,DONB);
    break;
  case Q_COLOR:
    //fprintf(stderr,"Dequant Color\n");
    mini=max(m_pMin[0],QPVAL(float,COLORMIN));		  
    maxi=min(m_pMax[0],QPVAL(float,COLORMAX));
    nBits= QPVAL(int,COLORNB);
    break;
  case Q_ANGLE:
    //fprintf(stderr,"Dequant Angle\n");
    mini=max(m_pMin[0],QPVAL(float,ANGLEMIN));		  
    maxi=min(m_pMax[0],QPVAL(float,ANGLEMAX));
    nBits= QPVAL(int,ANGLENB);
    break;
  case Q_SCALE:
    //fprintf(stderr,"Dequant Scale\n");
    mini=max(m_pMin[0],QPVAL(float,SCALEMIN));		  
    maxi=min(m_pMax[0],QPVAL(float,SCALEMAX));
    nBits= QPVAL(int,SCALENB);
    break;
  case Q_KEY:
    //fprintf(stderr,"Dequant Key\n");
    mini=max(m_pMin[0],QPVAL(float,KEYMIN));		  
    maxi=min(m_pMax[0],QPVAL(float,KEYMAX));
    nBits= QPVAL(int,KEYNB);
    break;
  case Q_SIZE2D: 
  case Q_SIZE3D: {
    mini=max(m_pMin[0],QPVAL(float,SIZEMIN));
    maxi=min(m_pMax[0],QPVAL(float,SIZEMAX));
    nBits=QPVAL(int,SIZENB);
   break;
  }
	case Q_LSQ:
		//fprintf(stderr,"Dequant LSQ\n");
		mini=0.f;
		maxi=1<<m_nBits;
		nBits=m_nBits;
		break;
	default:
	  fprintf(stderr, "[ERROR] Pas attendu\n");
  }
  return Quantizer::unQuantize(vq,mini,maxi,nBits,false);
}

/*-----------------------------------------------------------------
Parse a Vec3f from the BIFS stream
-----------------------------------------------------------------*/
Vec3f Quantizer :: unQuantizeVec3f (int vq[]) {
  Vec3f nResult;  


  float mini[3],maxi[3],nBits=0;

  switch(m_nQuantizationCode) {
  case Q_3D:
    //fprintf(stderr,"Dequantizing a Vec3f\n");
    mini[0]=max(m_pMin[0],QPVAL(Vec3f,POS3DMIN).x);
    mini[1]=max(m_pMin[0],QPVAL(Vec3f,POS3DMIN).y);
    mini[2]=max(m_pMin[0],QPVAL(Vec3f,POS3DMIN).z);
      
    maxi[0]=min(m_pMax[0],QPVAL(Vec3f,POS3DMAX).x);
    maxi[1]=min(m_pMax[0],QPVAL(Vec3f,POS3DMAX).y);
    maxi[2]=min(m_pMax[0],QPVAL(Vec3f,POS3DMAX).z);
      
    nBits= QPVAL(int,POS3DNB);
    break;
  case Q_NORMAL:
    fprintf(stderr,"ERREUR\n");
    nBits= QPVAL(int,NORMALNB);
    break;
  case Q_SCALE: {
    //fprintf(stderr,"Dequantizing a Scale3D\n");
    for(int i=0;i<3;i++) {
      mini[i]=max(m_pMin[0],QPVAL(float,SCALEMIN));
      maxi[i]=min(m_pMax[0],QPVAL(float,SCALEMAX));
    }
      
    nBits= QPVAL(int,SCALENB);
    break;
  }
  case Q_SIZE3D: {
      //fprintf(stderr,"Dequantizing a Size3D\n");
      for(int i=0;i<3;i++) {
	  mini[i]=max(m_pMin[0],QPVAL(float,SIZEMIN));
	  maxi[i]=min(m_pMax[0],QPVAL(float,SIZEMAX));
      }
      nBits=QPVAL(int,SIZENB);
      break;
  }
  default:
	  fprintf(stderr, "[ERROR] Pas attendu\n");
  }

  nResult.x = Quantizer::unQuantize(vq[0],mini[0],maxi[0],nBits,false);
  nResult.y = Quantizer::unQuantize(vq[1],mini[1],maxi[1],nBits,false);
  nResult.z = Quantizer::unQuantize(vq[2],mini[2],maxi[2],nBits,false);

  return nResult;
}

/*-----------------------------------------------------------------
Parse a Vec2f from the BIFS stream
-----------------------------------------------------------------*/
Vec2f Quantizer ::unQuantizeVec2f (int vq[]) {
  Vec2f nResult;


  float mini[2],maxi[2];
  int nBits = 0;

  switch(m_nQuantizationCode) {
  case Q_2D:
    //fprintf(stderr,"Dequantizing a Vec2f\n");
    mini[0]=max(m_pMin[0],QPVAL(Vec2f,POS2DMIN).x);
    mini[1]=max(m_pMin[0],QPVAL(Vec2f,POS2DMIN).y);
      
    maxi[0]=min(m_pMax[0],QPVAL(Vec2f,POS2DMAX).x);
    maxi[1]=min(m_pMax[0],QPVAL(Vec2f,POS2DMAX).y);
    nBits= QPVAL(int,POS2DNB);
    break;
  case Q_TEXTURECOORDINATE:
    //fprintf(stderr,"Dequantizing a Texture Coordinate\n");
    mini[0]=max(m_pMin[0],QPVAL(float,TCMIN));
    mini[1]=max(m_pMin[0],QPVAL(float,TCMIN));
      
    maxi[0]=min(m_pMax[0],QPVAL(float,TCMAX));
    maxi[1]=min(m_pMax[0],QPVAL(float,TCMAX));
    nBits= QPVAL(int,TCNB);
    break;
  case Q_SCALE: {
    //fprintf(stderr,"Dequantizing a Scale2D\n");
    for(int i=0;i<2;i++) {
      mini[i]=max(m_pMin[0],QPVAL(float,SCALEMIN));
      maxi[i]=min(m_pMax[0],QPVAL(float,SCALEMAX));
    }
      
    nBits= QPVAL(int,SCALENB);
    break;
  }
  case Q_SIZE2D: {
    //fprintf(stderr,"Dequantizing a Size2D\n");
    for(int i=0;i<2;i++) {
      mini[i]=max(m_pMin[0],QPVAL(float,SIZEMIN));	  
      maxi[i]=min(m_pMax[0],QPVAL(float,SIZEMAX));
    }
    nBits=QPVAL(int,SIZENB);
    break;
  }
	default:
	  fprintf(stderr, "[ERROR] Pas attendu\n");
  }

  nResult.x = Quantizer::unQuantize(vq[0],mini[0],maxi[0],nBits,false);
  nResult.y = Quantizer::unQuantize(vq[1],mini[1],maxi[1],nBits,false);
  return nResult;
}

/*-----------------------------------------------------------------
Parse a Rotation from the BIFS stream
-----------------------------------------------------------------*/
Rotation Quantizer :: unQuantizeRotation (int vq[], int direction, int orientation) 
{
	Rotation nResult;
	float natComp[4];//quaternions
	float tan[3];
	float delta=1;
	int nBits=QPVAL(int,NORMALNB);
 	// PORTING to CC 5.3
 	int j = 0;
 	//for (int j=0;j<3;j++) 
    for ( j=0;j<3;j++) 
	{
		float v = Quantizer::unQuantize(
						vq[j]-(1<<(nBits-1)),0,1,nBits,true);
		tan[j]= (float) ::tanf( v * PI/4);
		delta+= tan[j]*tan[j];
	}
    delta = (float)(direction/::sqrtf(delta));

    natComp[(orientation)%4] = delta;
    for (j=0;j<3;j++) 
		natComp[(orientation+j+1)%4] = tan[j]*delta;
  
	// converts quaternion to rotation
	nResult.theta=2*acos(natComp[0]);
	float sin2= sin(nResult.theta/2);
	if (sin2!=0.0)
	{
		for(int i=1;i<4;i++)
			natComp[i] /= sin2;
		nResult.x=natComp[1];
		nResult.y=natComp[2];
		nResult.z=natComp[3];
	}
	else
	{
		nResult.x=1;
		nResult.y=0;
		nResult.z=0;
	}


	return nResult;
}

/*-----------------------------------------------------------------
Parse a Normal from the BIFS stream
-----------------------------------------------------------------*/
Vec3f Quantizer :: unQuantizeNormal(int vq[], int direction, int orientation) 
{
	Vec3f nResult;
	float natComp[3];
	float tan[2];
	float delta=1;
	int nBits=QPVAL(int,NORMALNB);
 	// PORTING to CC 5.3
 	int j = 0;
 	//for (int j=0;j<2;j++) 
    for ( j=0;j<2;j++) 
	{
		float v = Quantizer::unQuantize(vq[j]-(1<<(nBits-1)),0,1,nBits,true);
		tan[j]= (float) ::tanf( v * PI/4);
		delta+= tan[j]*tan[j];
	}
    delta = (float)(direction/::sqrtf(delta));

    natComp[(orientation)%3] = delta;
    for (j=0;j<2;j++) 
		natComp[(orientation+j+1)%3] = tan[j]*delta;

  nResult.x=natComp[0];
  nResult.y=natComp[1];
  nResult.z=natComp[2];
 
  return nResult;
}



/*-----------------------------------------------------------------
Parse a Rotation2D from the BIFS stream
-----------------------------------------------------------------*/
Rotation2D Quantizer :: unQuantizeRotation2D (int vq[]) {
  Rotation2D nResult;
  vq[0];
  fprintf(stderr,"[ERROR] Not implemented Dequant Rotation2D\n");
  return nResult;
}

/*-----------------------------------------------------------------
Parse a Color3f from the BIFS stream
-----------------------------------------------------------------*/
Color3f Quantizer :: unQuantizeColor3f (int vq[]) {
  Color3f nResult;


  //fprintf(stderr,"Dequant Color\n");
	float mini= max(m_pMin[0],QPVAL(float,COLORMIN));
	float maxi= min(m_pMax[0],QPVAL(float,COLORMAX));
	int nBits= QPVAL(int,COLORNB);
	nResult.r= Quantizer::unQuantize(vq[0],mini,maxi,nBits,false);
	nResult.g= Quantizer::unQuantize(vq[1],mini,maxi,nBits,false);
	nResult.b= Quantizer::unQuantize(vq[2],mini,maxi,nBits,false);
	return nResult;
}

#endif	//AC end
