//$Log: AEncQuant.cpp,v $
//Revision 1.1.1.1  2003/12/24 00:55:04  mikeseven
//Initial release of the CVS repository for MPEG-4 Systems code
//after the code was corrupted due to reorganization and human 
//errors.
//
//Revision 1.1.1.2  2003/01/30 16:28:29  payan
//no message
//
//Revision 1.1.1.1  2002/07/18 01:08:24  mikeseven
//Initial check in of July 17, 2002 version
//
//Revision 1.1  2000/03/03 18:41:21  Alex Cotarmanac'h
//Version 1 -- Alexandre Cotarmanac_h
//
//<alexandre.cotarmanac-h@polytechnique.org>

#include "defs.h"
#include "Quantize.h"
#include "BifsEnc.h"
#include "ztl.h"
#include "NodeFld.h"
#include <float.h>
#include <math.h>
#define PI 3.1415927
#define QPVAL(type, id) ((type) *(SF<type>*) m_pQP->GetField(id))




// Gets Data For Floats
void Quantizer :: GetFloatData(int & nBits, float & mini,float &maxi)
{
  assert(m_pQP != NULL);
  switch(m_nQuantizationCode) {
  case Q_DRAWORDER:
    //fprintf(stderr,"Quant DrawOrder\n");
    mini=max(m_pMin[0],QPVAL(float,DOMIN));		  
    maxi=min(m_pMax[0],QPVAL(float,DOMAX));
    nBits= QPVAL(int,DONB);
    break;
  case Q_COLOR:
    //fprintf(stderr,"Quant Color\n");
    mini=max(m_pMin[0],QPVAL(float,COLORMIN));		  
    maxi=min(m_pMax[0],QPVAL(float,COLORMAX));
    nBits= QPVAL(int,COLORNB);
    break;
  case Q_ANGLE:
    //fprintf(stderr,"Quant Angle\n");
    mini=max(m_pMin[0],QPVAL(float,ANGLEMIN));		  
    maxi=min(m_pMax[0],QPVAL(float,ANGLEMAX));
    nBits= QPVAL(int,ANGLENB);
    break;
  case Q_SCALE:
    //fprintf(stderr,"Quant Scale\n");
    mini=max(m_pMin[0],QPVAL(float,SCALEMIN));		  
    maxi=min(m_pMax[0],QPVAL(float,SCALEMAX));
    nBits= QPVAL(int,SCALENB);
    break;
  case Q_KEY:
    //fprintf(stderr,"Quant Key\n");
    mini=max(m_pMin[0],QPVAL(float,KEYMIN));		  
    maxi=min(m_pMax[0],QPVAL(float,KEYMAX));
    nBits= QPVAL(int,KEYNB);
    break;
  case Q_SIZE2D: 
  case Q_SIZE3D: {
    mini=max(m_pMin[0],QPVAL(float,SIZEMIN));
    maxi=min(m_pMax[0],QPVAL(float,SIZEMAX));
    nBits=QPVAL(int,SIZENB);
   break;
  }
	case Q_LSQ:
		//fprintf(stderr,"Quant LSQ\n");
		mini=0.f;
		maxi=1<<m_nBits;
		nBits=m_nBits;
		break;
	default:
	  fprintf(stderr, "[ERROR] Pas attendu\n");
  }
  return;
}

void Quantizer::GetIntData(int & nBits, float & mini,float &maxi)
{
	switch(m_nQuantizationCode) 
	{
		case Q_DRAWORDER: {
			mini=max(m_pMin[0],QPVAL(Vec3f,DOMIN).x);
			nBits= QPVAL(int,DONB);
			maxi= mini + (1 << nBits ) - 1;
			break;
		}
		case Q_COORDIDX:
		{
			//printf("[QP14] Setting NatInt Data nBits:%d \n",m_nBits);
			if (m_nBits>0)
			{
				nBits=m_nBits;
				mini=0;
				maxi= mini + (1 << nBits ) - 1;
				// values are either [0..N-1] or -1 
			}
			else 
			{
				int nOffset = 0;
				nBits = 32;
				if (m_nDimension == 1 &&
					m_pMin [0] != FLT_MIN && m_pMax [0] != FLT_MAX
					)
				{
			  		
					mini = m_pMin [0];
					maxi = m_pMax [0];
					nBits = BitFieldWidth (maxi - mini + 1);
					nOffset = mini;
				}				
			
			}
			break;
		 }
		default: 
		{
			/*-----------------------------------------------------------------
			Determine offset and number of bits
			-----------------------------------------------------------------*/
			int nOffset = 0;
			nBits = 32;
			if (m_nDimension == 1 &&
				m_pMin [0] != FLT_MIN && m_pMax [0] != FLT_MAX
				)
			{
			  	
				mini = m_pMin [0];
				maxi = m_pMax [0];
				nBits = BitFieldWidth (maxi - mini + 1);
				nOffset = mini;
			}				
			break;
		}
	}
}

void Quantizer::GetVec3fData(int & nBits, float mini[], float maxi[])
{
  if(m_nQuantizationCode == Q_NONE) return;

  if(HasEfficientFloat()) {
	fprintf(stderr,"Oups ... GetVec3fData()\n\n\n");
	assert (false);
    return;
  }
  switch(m_nQuantizationCode) {
  case Q_3D:
    //fprintf(stderr,"Quantizing a Vec3f\n");
    mini[0]=max(m_pMin[0],QPVAL(Vec3f,POS3DMIN).x);
    mini[1]=max(m_pMin[0],QPVAL(Vec3f,POS3DMIN).y);
    mini[2]=max(m_pMin[0],QPVAL(Vec3f,POS3DMIN).z);
      
    maxi[0]=min(m_pMax[0],QPVAL(Vec3f,POS3DMAX).x);
    maxi[1]=min(m_pMax[0],QPVAL(Vec3f,POS3DMAX).y);
    maxi[2]=min(m_pMax[0],QPVAL(Vec3f,POS3DMAX).z);
      
    nBits= QPVAL(int,POS3DNB);
    break;
  case Q_NORMAL:
	fprintf(stderr,"Oups ... GetVec3fData()\n\n\n");
	assert (false);
    break;
  case Q_SCALE: {
    //fprintf(stderr,"Quantizing a Scale3D\n");
    for(int i=0;i<3;i++) {
      mini[i]=max(m_pMin[0],QPVAL(float,SCALEMIN));
      maxi[i]=min(m_pMax[0],QPVAL(float,SCALEMAX));
    }
      
    nBits= QPVAL(int,SCALENB);
    break;
  }
  case Q_SIZE3D: {
      for(int i=0;i<3;i++) {
	  mini[i]=max(m_pMin[0],QPVAL(float,SIZEMIN));	  
	  maxi[i]=min(m_pMax[0],QPVAL(float,SIZEMAX));
      }
      nBits=QPVAL(int,SIZENB);
  }
  break;
  }
}
void Quantizer::GetVec2fData (int & nBits, float mini[], float maxi[]) 
{
  if(m_nQuantizationCode == Q_NONE) return;

  if(HasEfficientFloat()) {
 	fprintf(stderr,"Oups ... GetVec3fData()\n\n\n");
	assert (false);
    return;
  }


  switch(m_nQuantizationCode) {
  case Q_2D:
    //fprintf(stderr,"Quantizing a Vec2f\n");
    mini[0]=max(m_pMin[0],QPVAL(Vec2f,POS2DMIN).x);
    mini[1]=max(m_pMin[0],QPVAL(Vec2f,POS2DMIN).y);
      
    maxi[0]=min(m_pMax[0],QPVAL(Vec2f,POS2DMAX).x);
    maxi[1]=min(m_pMax[0],QPVAL(Vec2f,POS2DMAX).y);
    nBits= QPVAL(int,POS2DNB);
    break;
  case Q_TEXTURECOORDINATE:
    //fprintf(stderr,"Quantizing a Texture Coordinate\n");
    mini[0]=max(m_pMin[0],QPVAL(float,TCMIN));
    mini[1]=max(m_pMin[0],QPVAL(float,TCMIN));
    
    maxi[0]=min(m_pMax[0],QPVAL(float,TCMAX));
    maxi[1]=min(m_pMax[0],QPVAL(float,TCMAX));
    nBits= QPVAL(int,TCNB);
    break;
  case Q_SCALE: {
      //fprintf(stderr,"Quantizing a Scale2D\n");
      for(int i=0;i<2;i++) {
	  mini[i]=max(m_pMin[0],QPVAL(float,SCALEMIN));
	  maxi[i]=min(m_pMax[0],QPVAL(float,SCALEMAX));
      }
      
      nBits= QPVAL(int,SCALENB);
      break;
  }
  case Q_SIZE2D: {
      for(int i=0;i<2;i++) {
	  mini[i]=max(m_pMin[0],QPVAL(float,SIZEMIN));
	  maxi[i]=min(m_pMax[0],QPVAL(float,SIZEMAX));
      }
      nBits=QPVAL(int,SIZENB);
  }
  break;
  }
}


void Quantizer::GetColorData(int & nBits, float mini[], float maxi[])
{
  for (int i=0;i<3;i++)
  {
	mini[i]= max(m_pMin[0],QPVAL(float,COLORMIN));
	maxi[i]= min(m_pMax[0],QPVAL(float,COLORMAX));
  }
  nBits= QPVAL(int,COLORNB);
}

void Quantizer::GetRotData(int & nBits)
{
	nBits=QPVAL(int,NORMALNB);
}

void Quantizer::GetNormalData(int & nBits)
{
	nBits=QPVAL(int,NORMALNB);
}
/*
//				GetQuantizationNbBits
int Quantizer::GetQuantizationNbBits(int quantType) 
{
	int nBits=-1;
	switch(quantType) {
	case Q_ROTATION:
	case Q_NORMAL:
		nBits=QPVAL(int,NORMALNB);
		break;
	case Q_2D:
		nBits= QPVAL(int,POS2DNB);
		break;
	case Q_TEXTURECOORDINATE:
	    nBits= QPVAL(int,TCNB);
		break;
	case Q_3D:
		nBits= QPVAL(int,POS3DNB);
		break;
	case Q_DRAWORDER:
		nBits= QPVAL(int,DONB);
		break;
	case Q_COLOR:
		nBits= QPVAL(int,COLORNB);
		break;
	case Q_ANGLE:
		nBits= QPVAL(int,ANGLENB);
		break;
	case Q_SCALE:
		nBits= QPVAL(int,SCALENB);
		break;
	case Q_KEY:
		nBits= QPVAL(int,KEYNB);
		break;
	case Q_SIZE2D: 
	case Q_SIZE3D:
		nBits=QPVAL(int,SIZENB);
		break;
	case Q_LSQ:
		nBits=m_nBits;
		break;
	case Q_COORDIDX:
		fprintf(stderr,"[Error:] Don't know how to handle Q_COORDIDX\n");
		break;
	}
  return nBits;
}
*/


/*
int Quantizer :: QuantizeFloat (float nValue) {
  float mini=0,maxi=0,nBits = 0;
  assert(m_pQP != NULL);
  switch(m_nQuantizationCode) {
  case Q_DRAWORDER:
    //fprintf(stderr,"Quant DrawOrder\n");
    mini=max(m_pMin[0],QPVAL(float,DOMIN));		  
    maxi=min(m_pMax[0],QPVAL(float,DOMAX));
    nBits= QPVAL(int,DONB);
    break;
  case Q_COLOR:
    //fprintf(stderr,"Quant Color\n");
    mini=max(m_pMin[0],QPVAL(float,COLORMIN));		  
    maxi=min(m_pMax[0],QPVAL(float,COLORMAX));
    nBits= QPVAL(int,COLORNB);
    break;
  case Q_ANGLE:
    //fprintf(stderr,"Quant Angle\n");
    mini=max(m_pMin[0],QPVAL(float,ANGLEMIN));		  
    maxi=min(m_pMax[0],QPVAL(float,ANGLEMAX));
    nBits= QPVAL(int,ANGLENB);
    break;
  case Q_SCALE:
    //fprintf(stderr,"Quant Scale\n");
    mini=max(m_pMin[0],QPVAL(float,SCALEMIN));		  
    maxi=min(m_pMax[0],QPVAL(float,SCALEMAX));
    nBits= QPVAL(int,SCALENB);
    break;
  case Q_KEY:
    //fprintf(stderr,"Quant Key\n");
    mini=max(m_pMin[0],QPVAL(float,KEYMIN));		  
    maxi=min(m_pMax[0],QPVAL(float,KEYMAX));
    nBits= QPVAL(int,KEYNB);
    break;
  case Q_SIZE2D: 
  case Q_SIZE3D: {
    mini=max(m_pMin[0],QPVAL(float,SIZEMIN));
    maxi=min(m_pMax[0],QPVAL(float,SIZEMAX));
    nBits=QPVAL(int,SIZENB);
   break;
  }
	case Q_LSQ:
		//fprintf(stderr,"Quant LSQ\n");
		mini=0.f;
		maxi=1<<m_nBits;
		nBits=m_nBits;
		break;
	default:
	  fprintf(stderr, "[ERROR] Pas attendu\n");
  }
  return Quantizer::Quantize(mini,maxi,nBits,nValue);
}

int Quantizer :: QuantizeInt (int nValue) {
  switch(m_nQuantizationCode) {
  case Q_DRAWORDER: {
    float mini=max(m_pMin[0],QPVAL(Vec3f,DOMIN).x);		  
//    int nBits= QPVAL(int,DONB);
    return (nValue - mini);
    break;
  }
  case Q_COORDIDX:
	  {
		  printf("[QP14] nBits:%d Value:%d\n",m_nBits,nValue);
		  pBifs->EncodeInt(nValue,m_nBits,"valueQP");
		  // values are either [0..N-1] or -1 
		  break;
	  }
  default: {
	//-----------------------------------------------------------------
	//Determine offset and number of bits
	//-----------------------------------------------------------------
	int nOffset = 0, nBits = 32;
	if (m_nDimension == 1 &&
	    m_pMin [0] != FLT_MIN && m_pMax [0] != FLT_MAX)
	  {	
		int min = m_pMin [0];
		int max = m_pMax [0];
		nBits = BitFieldWidth (max - min + 1);
		nOffset = min;
	}
	return (nValue - nOffset);
    break;
  }
  }
}
*/