// NatNormal.cpp: implementation of the NatNormal class.
//
//////////////////////////////////////////////////////////////////////
//$Log: NatNormal.cpp,v $
//Revision 1.1.1.1  2003/12/24 00:55:04  mikeseven
//Initial release of the CVS repository for MPEG-4 Systems code
//after the code was corrupted due to reorganization and human 
//errors.
//
//Revision 1.1.1.2  2003/01/30 16:28:29  payan
//no message
//
//Revision 1.1.1.1  2002/07/18 01:08:25  mikeseven
//Initial check in of July 17, 2002 version
//
//Revision 1.1  2000/03/03 18:41:22  Alex Cotarmanac'h
//Version 1 -- Alexandre Cotarmanac_h
//
//<alexandre.cotarmanac-h@polytechnique.org>
#include "defs.h" // defines M_PI
#include <assert.h>
#include <math.h>
#include <float.h>
#include "NatNormal.h"
#include "QuantNormal.h"
#include "Quantize.h"
#include "Token.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
bool NatNormal::m_predictiveNatScheme=false;

NatNormal::NatNormal()
{
}
NatNormal::NatNormal(Quantizer * pQuant, float x, float y, float z,bool isPureNat): 
	m_isPureNat(isPureNat),m_nbbits(-1),m_x(x),m_y(y),m_z(z)
{
		assert(pQuant!=NULL);
		assert( m_x!=0.0 || m_y!=0 || m_z!=0);
		pQuant->GetNormalData(m_nbbits);
		computeQuaternions();
}
NatNormal::NatNormal(int nbbits,float x, float y, float z,bool isPureNat):
	m_isPureNat(isPureNat),m_x(x),m_y(y),m_z(z),m_nbbits(nbbits)
{
		assert( m_x!=0.0 || m_y!=0 || m_z!=0);
		computeQuaternions();
}
NatNormal::NatNormal(int nbbits,float quat[],bool isPureNat):
	m_isPureNat(isPureNat),m_nbbits(nbbits)
{
		m_x=m_quat[0]=quat[0];
		m_y=m_quat[1]=quat[1];
		m_z=m_quat[2]=quat[2];
}
NatNormal::~NatNormal()
{
	
}
NatNormal& NatNormal::operator = (const NatNormal &x)
{
	for (int i=0; i<m_nbComp;i++)
			m_quat[i]=x.m_quat[i];
	m_x=x.m_x;
	m_y=x.m_y;
	m_z=x.m_z;
	m_nbbits=x.m_nbbits;
	return *this;
}
NatNormal NatNormal::parseValue(Quantizer * pQuant)
{
	float c[m_nbComp];
	for (int i=0; i< m_nbComp; i++)
		c[i]= nextToken.GetReal();
	if (nextToken == TID_COMMA ) nextToken.Scan();
	return NatNormal(pQuant,c[0],c[1],c[2],true);
}

QuantNormal NatNormal::Quantize()
{
	int orientation =-1;
	float maxTmp=-FLT_MAX;
	for (int i=0;i<m_nbComp;i++) {
		if (fabs(m_quat[i])>maxTmp) {
		  maxTmp=(float)fabs(m_quat[i]);
		  orientation=i;
		}
	}
	int qv[4];
	qv[3]=orientation;
	qv[2]=(m_quat[orientation]>0)?0:1;//direction
	for (int j=0;j<m_nbComp-1;j++) {
		float v =(float)( 4/M_PI * atan( m_quat[(orientation+j+1) % m_nbComp] / m_quat[orientation]));
		qv[j]= (1<<(m_nbbits-1)) + (v>=0?1:-1)*Quantizer::Quantize(0, 1, m_nbbits-1, (v>=0?v:-v));
	}

	return QuantNormal(m_nbbits,qv[0],qv[1],qv[2],qv[3]);
}

NatNormal & NatNormal::operator -=(NatNormal a) // in this is Quat(this)/Quat(aQi)
{
// here a is Unit Quaternion therefore 1/a (or -a here) is a*
	assert(false);
	NatNormal inva;
	inva.m_quat[0]=a.m_quat[0];
	for (int i=1; i<4; i++) inva.m_quat[i]=-a.m_quat[i];
	return this->operator +=(inva);
}


NatNormal & NatNormal::operator +=(NatNormal aQI) // in this is Quat(this)*Quat(aQi)
{
	assert(false);
	for (int i=0; i<4;i++) m_quat[i]=1;
	return *this;
}

NatNormal operator+ (NatNormal a, NatNormal b)
{
	NatNormal r = a;
	return (r+=b);
}

NatNormal operator- (NatNormal a, NatNormal b)
{
	NatNormal r = a;
	return (r-=b);
}

void NatNormal::computeQuaternions() // just normalize...
{
	float normR = sqrt(m_x*m_x + m_y*m_y + m_z*m_z);
	float scale = 1.0f/normR;
	m_quat[0] = m_x*scale;
	m_quat[1] = m_y*scale;
	m_quat[2] = m_z*scale;
  
}
#ifdef DEBUGPREDICTIVE
void NatNormal::Print(FILE * out)
{
	fprintf(out,"(% 5.2f , % 5.2f , % 5.2f )",m_x,m_y,m_z);
	if (!m_isPureNat) fprintf(out,"D");
	fprintf(out," \t");
	fprintf(out,"((% 5.2f , % 5.2f , % 5.2f ))\n",m_quat[0],m_quat[1],m_quat[2]);
}
#endif


