// NatRot.cpp: implementation of the NatRot class.
//
//////////////////////////////////////////////////////////////////////
//$Log: NatRot.cpp,v $
//Revision 1.1.1.1  2003/12/24 00:55:04  mikeseven
//Initial release of the CVS repository for MPEG-4 Systems code
//after the code was corrupted due to reorganization and human 
//errors.
//
//Revision 1.1.1.2  2003/01/30 16:28:29  payan
//no message
//
//Revision 1.1.1.1  2002/07/18 01:08:25  mikeseven
//Initial check in of July 17, 2002 version
//
//Revision 1.1  2000/03/03 18:41:22  Alex Cotarmanac'h
//Version 1 -- Alexandre Cotarmanac_h
//
//<alexandre.cotarmanac-h@polytechnique.org>
#include "defs.h" // defines M_PI
#include <assert.h>
#include <math.h>
#include <float.h>
#include "NatRot.h"
#include "QuantRot.h"
#include "Quantize.h"
#include "Token.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
bool NatRot::m_predictiveNatScheme=false;

NatRot::NatRot()
{
}
NatRot::NatRot(Quantizer * pQuant, float x, float y, float z,float angle,bool isPureNat): 
	m_isPureNat(isPureNat),m_nbbits(-1),m_x(x),m_y(y),m_z(z),m_angle(angle)
{
		assert(pQuant!=NULL);
		assert( m_x!=0.0 || m_y!=0 || m_z!=0);
		pQuant->GetRotData(m_nbbits);
		computeQuaternions();
}
NatRot::NatRot(int nbbits,float x, float y, float z,float angle,bool isPureNat):
	m_isPureNat(isPureNat),m_x(x),m_y(y),m_z(z),m_nbbits(nbbits),m_angle(angle)
{
		assert( m_x!=0.0 || m_y!=0 || m_z!=0);
		computeQuaternions();
}
NatRot::NatRot(int nbbits,float quat[],bool isPureNat):
	m_isPureNat(isPureNat),m_nbbits(nbbits)
{
		for (int i=0; i<4;i++)
			m_quat[i]=quat[i];
		m_angle=(float)(2*acos(quat[0]));
		float epsilon=FLT_EPSILON;
		if (m_angle!=0 && (fabs(sin(m_angle/2))>epsilon))
		{
			m_x=(float)(quat[1]/sin(m_angle/2));
			m_y=(float)(quat[2]/sin(m_angle/2));
			m_z=(float)(quat[3]/sin(m_angle/2));
		}
		else 
		{
			m_x=0;
			m_y=0;
			m_z=1;
		}
		
}
NatRot::~NatRot()
{
	
}
NatRot& NatRot::operator = (const NatRot &x)
{
	for (int i=0; i<4;i++)
			m_quat[i]=x.m_quat[i];
	m_x=x.m_x;
	m_y=x.m_y;
	m_z=x.m_z;
	m_angle=x.m_angle;
	m_nbbits=x.m_nbbits;
	return *this;
}
NatRot NatRot::parseValue(Quantizer * pQuant)
{
	float c[m_nbComp];
	for (int i=0; i< m_nbComp; i++)
		c[i]= nextToken.GetReal();
	if (nextToken == TID_COMMA ) nextToken.Scan();
	return NatRot(pQuant,c[0],c[1],c[2],c[3],true);
}

QuantRot NatRot::Quantize()
{
	int orientation =-1;
	float maxTmp=-FLT_MAX;
	for (int i=0;i<m_nbComp;i++) {
		if (fabs(m_quat[i])>maxTmp) {
		  maxTmp=(float)fabs(m_quat[i]);
		  orientation=i;
		}
	}
	int qv[4];
	qv[0]=orientation;
	for (int j=1;j<m_nbComp;j++) {
		float v =(float)( 4/M_PI * atan( m_quat[(orientation+j) % m_nbComp] / m_quat[orientation]));
		qv[j]= (1<<(m_nbbits-1)) + (v>=0?1:-1)*Quantizer::Quantize(0, 1, m_nbbits-1, (v>=0?v:-v));
	}

	return QuantRot(m_nbbits,qv[0],qv[1],qv[2],qv[3]);
}

NatRot & NatRot::operator -=(NatRot a) // in this is Quat(this)/Quat(aQi)
{
// here a is Unit Quaternion therefore 1/a (or -a here) is a*
	NatRot inva;
	inva.m_quat[0]=a.m_quat[0];
	for (int i=1; i<4; i++) inva.m_quat[i]=-a.m_quat[i];
	return this->operator +=(inva);
}


NatRot & NatRot::operator +=(NatRot aQI) // in this is Quat(this)*Quat(aQi)
{
/*	float res[4];
	res[0]=m_quat[0]*aQI.m_quat[0] 
			-	m_quat[1]*aQI.m_quat[1] 
			- 	m_quat[2]*aQI.m_quat[2]
			- 	m_quat[3]*aQI.m_quat[3];
	res[1]=m_quat[0]*aQI.m_quat[1] 
			+	m_quat[1]*aQI.m_quat[0] 
			+ 	m_quat[2]*aQI.m_quat[3]
			- 	m_quat[3]*aQI.m_quat[2];
	res[2]=m_quat[0]*aQI.m_quat[2] 
			-	m_quat[1]*aQI.m_quat[3] 
			+ 	m_quat[2]*aQI.m_quat[0]
			+	m_quat[3]*aQI.m_quat[1];
	res[3]=m_quat[0]*aQI.m_quat[3] 
			+	m_quat[1]*aQI.m_quat[2] 
			- 	m_quat[2]*aQI.m_quat[1]
			+	m_quat[3]*aQI.m_quat[0];
*/
	assert(false);
	for (int i=0; i<4;i++) m_quat[i]=1;
	return *this;
}

NatRot operator+ (NatRot a, NatRot b)
{
	NatRot r = a;
	return (r+=b);
}

NatRot operator- (NatRot a, NatRot b)
{
	NatRot r = a;
	return (r-=b);
}

void NatRot::computeQuaternions()
{
	float normR = sqrt(m_x*m_x + m_y*m_y + m_z*m_z);
	m_quat[0]=cos(m_angle/2);
	float scale = sin(m_angle/2)/normR;
	m_quat[1] = m_x*scale;
	m_quat[2] = m_y*scale;
	m_quat[3] = m_z*scale;
  
}
#ifdef DEBUGPREDICTIVE
void NatRot::Print(FILE * out)
{
//	fprintf(stdout,"(%f , %f , %f (%f))",m_x,m_y,m_z, m_angle);
	fprintf(out,"(% 5.2f , % 5.2f , % 5.2f (% 5.2f))",m_x,m_y,m_z,m_angle);
	if (!m_isPureNat) fprintf(out,"D");
	fprintf(out," \t");
	fprintf(out,"((% 5.2f , % 5.2f , % 5.2f % 5.2f))\n",m_quat[0],m_quat[1],m_quat[2],m_quat[3]);
}
#endif


