// NatTypes.cpp: implementation of the NatFloat class.
//
//////////////////////////////////////////////////////////////////////
//$Log: NatTypes.cpp,v $
//Revision 1.1.1.1  2003/12/24 00:55:04  mikeseven
//Initial release of the CVS repository for MPEG-4 Systems code
//after the code was corrupted due to reorganization and human 
//errors.
//
//Revision 1.1.1.2  2003/01/30 16:28:29  payan
//no message
//
//Revision 1.1.1.1  2002/07/18 01:08:25  mikeseven
//Initial check in of July 17, 2002 version
//
//Revision 1.2  2000/03/09 20:46:09  Alex Cotarmanac'h
//Fixed parameter bug
//
//Revision 1.1  2000/03/03 18:41:22  Alex Cotarmanac'h
//Version 1 -- Alexandre Cotarmanac_h
//
//<alexandre.cotarmanac-h@polytechnique.org>
#include "defs.h"
#include <assert.h>
#include "NatTypes.h"
#include "QuantTypes.h"
#include "Quantize.h"
#include "Token.h"
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
bool NatFloat::m_predictiveNatScheme=true;
NatFloat::NatFloat(Quantizer * pQuant,float v,bool isPureNat): 
	m_value(v),m_isPureNat(isPureNat),m_maxi(-1),m_mini(-1),m_nbbits(-1)
{
		assert(pQuant!=NULL);
		pQuant->GetFloatData(m_nbbits,m_mini,m_maxi);
}
NatFloat::NatFloat(int nbbits, float mini,float maxi, float v,bool isPureNat):
	m_value(v),m_isPureNat(isPureNat),m_maxi(maxi),m_mini(mini),m_nbbits(nbbits)
{
}

NatFloat::~NatFloat()
{

}

NatFloat NatFloat::parseValue(Quantizer * pQuant)
{
	float v=nextToken.GetReal();
	if (nextToken == TID_COMMA ) nextToken.Scan();
	return NatFloat(pQuant,v,true);
}

QuantFloat NatFloat::Quantize()
{
	if (m_isPureNat)
		return QuantFloat(m_nbbits,m_mini,m_maxi,
		Quantizer::Quantize(m_mini,m_maxi,m_nbbits,m_value));
	else 
	{
		float sgn = (m_value>=0)?1:-1;
		int   qval= sgn * Quantizer::Quantize(m_mini,m_maxi,m_nbbits,sgn * m_value);
		return QuantFloat(m_nbbits,m_mini,m_maxi,qval);
	}
}

NatFloat & NatFloat::operator -=(NatFloat a)
{
	m_value-= a.m_value;
	m_isPureNat = false; 
	return *this;
}


NatFloat & NatFloat::operator +=(NatFloat aQI)
{
	//assert (aQI.m_pQuant == m_pQuant);
	m_value+=aQI.m_value;
	return *this;
}

NatFloat operator+ (NatFloat a, NatFloat b)
{
	NatFloat r = a;
	return (r+=b);
}

NatFloat operator- (NatFloat a, NatFloat b)
{
	NatFloat r = a;
	return (r-=b);
}

#ifdef DEBUGPREDICTIVE
void NatFloat::Print(FILE * out)
{
	fprintf(out,"% 4.2f",m_value);
	if (!m_isPureNat) fprintf(out,"D");
	fprintf(out," ");
}
#endif


