// NatVec3f.cpp: implementation of the NatVec3f class.
//
//////////////////////////////////////////////////////////////////////
//$Log: NatVec3f.cpp,v $
//Revision 1.1.1.1  2003/12/24 00:55:04  mikeseven
//Initial release of the CVS repository for MPEG-4 Systems code
//after the code was corrupted due to reorganization and human 
//errors.
//
//Revision 1.1.1.2  2003/01/30 16:28:30  payan
//no message
//
//Revision 1.1.1.1  2002/07/18 01:08:25  mikeseven
//Initial check in of July 17, 2002 version
//
//Revision 1.1  2000/03/03 18:41:22  Alex Cotarmanac'h
//Version 1 -- Alexandre Cotarmanac_h
//
//<alexandre.cotarmanac-h@polytechnique.org>
#include "defs.h"
#include <assert.h>
#include "NatVec3f.h"
#include "QuantVec3f.h"
#include "Quantize.h"
#include "Token.h"
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
bool NatVec3f::m_predictiveNatScheme=true;
NatVec3f::NatVec3f()
{
}
NatVec3f::NatVec3f(Quantizer * pQuant, float x, float y, float z,bool isPureNat): 
	m_isPureNat(isPureNat),m_nbbits(-1),m_x(x),m_y(y),m_z(z)
{
		assert(pQuant!=NULL);
		pQuant->GetVec3fData(m_nbbits,m_mini,m_maxi);
}
NatVec3f::NatVec3f(int nbbits, float mini[],float maxi[], float x, float y, float z,bool isPureNat):
	m_isPureNat(isPureNat),m_x(x),m_y(y),m_z(z),m_nbbits(nbbits)
{
		for (int i=0; i< m_nbComp; i++)
		{
			m_mini[i]=mini[i];
			m_maxi[i]=maxi[i];
		}
}

NatVec3f::~NatVec3f()
{

}

NatVec3f NatVec3f::parseValue(Quantizer * pQuant)
{
	float c[m_nbComp];
	for (int i=0; i< m_nbComp; i++)
		c[i]= nextToken.GetReal();
	if (nextToken == TID_COMMA ) nextToken.Scan();
	return NatVec3f(pQuant,c[0],c[1],c[2],true);
}

QuantVec3f NatVec3f::Quantize()
{
	int qv[3];
	float v[3];
	float sgv[3];
	v[0]=m_x;
	v[1]=m_y;
	v[2]=m_z;
	for (int i=0; i<3; i++)
	{
		sgv[i]=(m_isPureNat || (v[i]>=0) )?1:-1;
		v[i]=(m_isPureNat)?v[i]: sgv[i]*v[i]+m_mini[i];
		qv[i]=sgv[i]*Quantizer::Quantize(m_mini[i],m_maxi[i],m_nbbits,v[i]);
	}
	return QuantVec3f(m_nbbits,m_mini,m_maxi,qv[0],qv[1],qv[2]);
}

NatVec3f & NatVec3f::operator -=(NatVec3f a)
{
	m_x-= a.m_x;
	m_y-= a.m_y;
	m_z-= a.m_z;
	m_isPureNat = false; 
	return *this;
}


NatVec3f & NatVec3f::operator +=(NatVec3f aQI)
{
	m_x+=aQI.m_x;
	m_y+=aQI.m_y;
	m_z+=aQI.m_z;
	return *this;
}

NatVec3f operator+ (NatVec3f a, NatVec3f b)
{
	NatVec3f r = a;
	return (r+=b);
}

NatVec3f operator- (NatVec3f a, NatVec3f b)
{
	NatVec3f r = a;
	return (r-=b);
}

#ifdef DEBUGPREDICTIVE
void NatVec3f::Print(FILE * out)
{
//	fprintf(stdout,"(%f , %f , %f)",m_x,m_y,m_z);
	fprintf(out,"(% 5.2f , % 5.2f , % 5.2f)",m_x,m_y,m_z);
	if (!m_isPureNat) fprintf(out,"D");
	fprintf(out," \n");
}
#endif


