// PredictiveEncoder.cpp: implementation of the PredictiveEncoder class.
//
//////////////////////////////////////////////////////////////////////
//$Log: PredictiveEncoder.cpp,v $
//Revision 1.1.1.1  2003/12/24 00:55:04  mikeseven
//Initial release of the CVS repository for MPEG-4 Systems code
//after the code was corrupted due to reorganization and human 
//errors.
//
//Revision 1.1.1.2  2003/01/30 16:28:30  payan
//no message
//
//Revision 1.1.1.1  2002/07/18 01:08:25  mikeseven
//Initial check in of July 17, 2002 version
//
//Revision 1.2.2.1  2000/03/10 15:59:33  Alex Cotarmanac'h
//deuxime envoi
//
//Revision 1.1  2000/03/03 18:41:22  Alex Cotarmanac'h
//Version 1 -- Alexandre Cotarmanac_h
//
//<alexandre.cotarmanac-h@polytechnique.org>
#include "defs.h"
#include "Token.h"
#include "NodeFld.h"
#include "PredictiveEncoderFactory.h"
#include "PredictiveParameters.h"
#include "PredictiveParser.h"
#include "PredictiveEncoder.h"

#include "NodeCode.h"


#ifdef DEBUGPREDICTIVE
#include "ArithModel.h"
#include "AArithDecDebug.h"
#endif
//////////////////////////////////////////////////////////////////////
// PredictiveEncoderError Class
//////////////////////////////////////////////////////////////////////


PredictiveEncoderError::PredictiveEncoderError()
{

}

PredictiveEncoderError::~PredictiveEncoderError()
{

}




PredictiveEncoder::~PredictiveEncoder()
{

}


PredictiveEncoder::PredictiveEncoder(BIFSEncoder *pBifs, Quantizer *pQuant) :
m_pBifs(pBifs), m_pQuant(pQuant), m_params()
{
}




void PredictiveEncoder::Encode(
							   BIFSEncoder *pBifs, 
							   Quantizer *pQuant, 
							   MField *pMField,
							   char * basename)
{
	printf("[PredictiveEncoder::Encode] basename = %s\n",basename);
	//Get a QP
	MediaObject * a_QP;
	if ((a_QP=pBifs->GetCurrentQP()) == NULL)
		 a_QP= SFGenericNode().CreateNode(CODE_QuantizationParameter);
	pQuant->SetQPNode( a_QP );

	PredictiveEncoder * a_penc = 
		PredictiveEncoderFactory::Create(pMField->GetType(),pBifs, pQuant);
#ifdef DEBUGPREDICTIVE
//	char * debugout = new char[strlen(basename)+5];
//	strcpy(debugout, basename);
//	strcat(debugout,".pdo");
//	a_penc->m_out = fopen(debugout,"a+");
//	delete [] debugout;
//	if (a_penc->m_out==NULL) throw PredictiveEncoderError();
#endif
	a_penc->parseNativeArray();
	printf("[PredictiveEncoder::Encode] %d\n",a_penc->getNbFields());
	//AC 20000227
	// store if needed number of points

	pBifs->setLengthCoord(a_penc->getNbFields());
	//end AC
	PredictiveParser::Parse(basename,a_penc->getNbFields(),(a_penc->m_params));
	a_penc->calcPredictiveVector();
	
#ifdef DEBUGPREDICTIVE
//	{ FILE * old = a_penc->m_out; a_penc->m_out=stdout; a_penc->PrintInput(); a_penc->m_out=old;}
//	a_penc->PrintInput();
//	a_penc->PrintQ();
//	a_penc->PrintDelta();
//	a_penc->PrintPminNbBits();
//	a_penc->PrintOutArray();
//	a_penc->DecodeAndPrintOutArray();
#ifdef AADEBUG
	AADebug aad = AADebug(a_penc->m_outArray);
	aad.reset();
	ArithModel model = ArithModel();
	model.init(1<<5);
	fprintf(m_out,"[ ");
	for (int j=1; j< a_penc->getNbFields(); j++)
	{
		fprintf(m_out," %d ",aad.decodeAndUpdate(model));
	}
	fprintf(m_out," ]\n");
#endif //AADEBUG
#endif //DEBUGPREDICTIVE

	a_penc->EncodeData();  // Finally encode data


#ifdef DEBUGPREDICTIVE
//	fclose(a_penc->m_out);
#endif //DEBUGPREDICTIVE
	printf("[PredictiveEncoder::Encode] end\n");

}


void PredictiveEncoder::setParameters(PredictiveParameters p)
{
	m_params = p;
}

int PredictiveEncoder::getNbFields()
{
	return m_nbFields;
}

/*
void PredictiveEncoder::calcPredictiveVector()
{
	int begin=0;
	int end=0;
	NatType * pNatDelta=NULL; // holds difference NatVec[i] - USi
	NatType * pVi;
	QuantType * pQi;
	while (end >=0 && end<= m_nbFields)
	{
		
		end=m_params.nextIntra(begin);		
		fprintf(stdout,"[calcPredictiveVector] begin = %d | end =%d \n",begin,end);
		pVi = m_NatVec.ElementAt(begin);
		pQi = pVi->Quantize();
		m_QuantVec.addElement(pQi);
		Set(pSi, pQi); // Si=Qi
		
		for (int i=begin+1; i<end;i++)
		{
			pVi = m_NatVec.ElementAt(i);
			pNatDelta = Difference(pVi,pSi->unQuantize()); // Vi - U(Si)
			pQi	= pNatDelta->Quantize();
			m_QuantVec.addElement(pQi);
			Add(pSi,pQi); // Si = S(i-1) + Q(Vi-U(Si))
		}

	}
}
*/

