// PredictiveEncoderBase.h: interface for the PredictiveEncoderBase class.
//
//////////////////////////////////////////////////////////////////////
//$Log: PredictiveEncoderBase.h,v $
//Revision 1.1.1.1  2003/12/24 00:55:04  mikeseven
//Initial release of the CVS repository for MPEG-4 Systems code
//after the code was corrupted due to reorganization and human 
//errors.
//
//Revision 1.1.1.2  2003/01/30 16:28:30  payan
//no message
//
//Revision 1.1.1.1  2002/07/18 01:08:26  mikeseven
//Initial check in of July 17, 2002 version
//
//Revision 1.1  2000/03/03 18:41:22  Alex Cotarmanac'h
//Version 1 -- Alexandre Cotarmanac_h
//
//<alexandre.cotarmanac-h@polytechnique.org>
#if !defined(AFX_PREDICTIVEENCODERBASE_H__7CA2BC62_DD48_11D3_9776_525405F4A0F5__INCLUDED_)
#define AFX_PREDICTIVEENCODERBASE_H__7CA2BC62_DD48_11D3_9776_525405F4A0F5__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000
#include "ztl.h"
#include "Token.h"
#include "PredictiveEncoder.h"

template<class NatT, class QuantT> class PredictiveEncoderBase : public PredictiveEncoder  
{
public:
	PredictiveEncoderBase(BIFSEncoder * pBifs, Quantizer * pQuant):
	  PredictiveEncoder(pBifs,pQuant),m_nbbits(0)
	  {}
	//virtual ~PredictiveEncoderBase();
private:
	int m_nbbits;
	ZArray<NatT> m_natArray;
	ZArray<QuantT> m_quantArray;
	ZArray<QuantT> m_PMinArray;
	ZArray<int>	   m_NbBitsArray;
	ZArray<QuantT> m_deltaArray;
	virtual void calcPredictiveVector();
	virtual void parseNativeArray();
	virtual void EncodeData();
	virtual void EncodeArrayQP(int , int , bool , int , bool , QuantT );
#ifdef DEBUGPREDICTIVE
public:
	void PrintInput();
	void PrintQ();
	void PrintDelta();
	void PrintPminNbBits();
	void PrintOutArray();
	void DecodeAndPrintOutArray();
#endif
};
//********************************************************************************
template<class NatT, class QuantT> 
void PredictiveEncoderBase<NatT,QuantT>::calcPredictiveVector()
{
	int begin=0;
	int end=0;
	NatT Vi;
	NatT Delta;	// holds difference V[i] - U(Si)		
	QuantT Qi;
	QuantT::setNbAACModels(m_pQuant);//sets static vars.
	QuantT Si;
	while (end >=0 && end< m_nbFields)
	{
		end=m_params.nextIntra(begin);
		if (end == -1) end=m_nbFields;
//		fprintf(stdout,"[calcPredictiveVector] begin = %d | end = %d \n",begin,end);
		Vi = m_natArray.ElementAt(begin);
		Qi = Vi.Quantize();
		m_quantArray+=Qi;// SetAt(begin)
		Si=Qi;
		for (int i=begin+1; i<end;i++)
		{
			Vi = m_natArray.ElementAt(i);
			if (NatT::m_predictiveNatScheme)
			{
				NatT USi = Si.unQuantize();
				Delta = Vi - USi; // Vi - U(Si)
				Qi	= Delta.Quantize();
				Si += Qi; // Si = S(i-1) + Q(Vi-U(Si))
#ifdef DEBUGPREDICTIVE
			fprintf(m_out,"[ Si =");
			Si.Print(m_out);
			fprintf(m_out,"; Vi =");
			Vi.Print(m_out);
			fprintf(m_out,"; Delta= ");
			Delta.Print(m_out);
			fprintf(m_out,"; Qi= ");
			Qi.Print(m_out);
			fprintf(m_out,"]\n");
#endif
			}
			else 
			{

#ifdef TOTODEBUG
				{
			QuantT QVi = Vi.Quantize();
			QuantT dQVi= QVi - Si ;
			QuantT sQVi=Si + dQVi;
			sQVi.Print(stdout);
				}
#endif
				Qi = Vi.Quantize() - Si; // Q(Vi)-Q(Vi-1)
#ifdef DEBUGPREDICTIVE
			/*m_out=stdout;*/
			fprintf(m_out,"[ Qi =");
			Qi.Print(m_out);
			fprintf(m_out,"; Q(Vi) =");
			(Vi.Quantize()).Print(m_out);
			fprintf(m_out,"; Si= ");
			Si.Print(m_out);
			fprintf(m_out,"]\n");
#endif

				Si += Qi; // ok: it would be easier Si = Vi.Quantize();
			}
			m_quantArray.SetAt(i,QuantT(Qi));

		}	
		m_deltaArray+=m_quantArray[begin];// SetAt(begin)
		m_deltaArray[begin].WriteIntra(m_outArray);
		m_outIndexes.SetAt(begin,m_outArray.GetLength());
		if (end-begin-1>0)
		{
			int nBits;
			// IMPORTANT : CompMin(Zarray &,..., int &)
			// don't want to use ZArray because of ZArray copy
			QuantT PMin	  = QuantT::CompMin(m_quantArray, begin+1, end,nBits);
			m_PMinArray.SetAt(begin, PMin);
			m_NbBitsArray.SetAt(begin, nBits);
			// IMPORTANT : Shift(Zarray &,ZArray &, ...)
			QuantT::Shift(m_deltaArray,m_quantArray,PMin,begin+1,end);
			QuantT::AAEncode(m_outArray, m_outIndexes, 
				m_deltaArray, nBits, begin+1, end,(m_params.getPolicy()==CHOICE));
		}	
		begin=end;
	}
	m_params.Reset();
}
//********************************************************************************
extern int getNbBits(int k);
template<class NatT, class QuantT> 
void PredictiveEncoderBase<NatT,QuantT>::EncodeData()
{
	m_params.Reset();
	m_nbbits=getNbBits(m_nbFields); // Calculate number of bits
	m_pBifs -> EncodeInt (m_nbbits, 5, "NbBits");
	m_pBifs -> EncodeInt (m_nbFields,m_nbbits,"numberOfFields");
	m_pBifs -> EncodeInt (m_params.getPolicy(),2,"IPPolicy");
	// LOOP
	int begin=0;
	int end=0;
	int IntraInterval=0;
	int CompNbBits=0;
	QuantT PMin = QuantT(m_pQuant);
	while (end >=0 && end< m_nbFields)
	{
		end=m_params.nextIntra(begin);
		if (end == -1) end=m_nbFields;
		// DO WE NEED TO ENCODE ARRAY QP ?
		bool hasQP=false;
		bool hasNewIntraInterval=false;
		bool hasCompNbBits=false;
		bool hasCompMin = false;
		if (CompNbBits!=m_NbBitsArray[begin])
		{
			hasCompNbBits=true;
			hasCompMin=true;
			CompNbBits=m_NbBitsArray[begin];
			PMin=m_PMinArray[begin];
		}
		else if (PMin != m_PMinArray[begin])
		{
			hasCompMin=true;
			PMin=m_PMinArray[begin];
		}
		if (m_params.getPolicy()==INTERVAL)
		{
			if (m_params.intraInterval(begin)!=IntraInterval)
			{
				hasNewIntraInterval=true;
				IntraInterval=m_params.intraInterval(begin);
			}
			hasQP= hasNewIntraInterval || hasCompNbBits || hasCompMin;
		}
		else hasQP = hasCompNbBits || hasCompMin;
		if (begin!=0) {
			if (m_params.getPolicy()==CHOICE)  // encode isIntra==true
				m_pBifs->EncodeInt(1,1,"isIntra");
			m_pBifs -> EncodeInt (hasQP,1,"hasQP");
		}
		if (hasQP) EncodeArrayQP(begin, IntraInterval, hasCompNbBits, CompNbBits, hasCompMin, PMin);

		// ENCODE THIS GROUP OF VALUES [BEGIN..END-1]
		int firstndx=(begin)?m_outIndexes[begin-1]:0;
		int lastndx = 0;
		int eltno   = begin;
		while (eltno < end)
		{
			lastndx=m_outIndexes[eltno];
			int value=0;

			if ((m_params.getPolicy()==CHOICE) && (eltno!=begin))  // encode isIntra==false
				m_pBifs->EncodeInt(0,1,"isIntra");
			//for (int l=firstndx;l<lastndx;l++)
			//{
			//	printf("%d ",(m_outArray[l]?1:0));
			//}
//			printf("(nbits=%d)\n",lastndx-firstndx);
			for (int i=firstndx;i<lastndx;i++)
			{
				value = (value<<1) + (m_outArray[i]?1:0);
				if ((i+1-firstndx)%12==0)
				{
					char * msg = (i+1==firstndx+12)?((eltno==begin)?"Intra":"Pred"):"...";
					m_pBifs->EncodeInt(value,12,msg);
					value=0;
				}
			}
			if ((lastndx-firstndx)%12!=0) {
				m_pBifs->EncodeInt(value,(lastndx-firstndx)%12,"#");
			}
			eltno++;
			firstndx=lastndx;
		}
		begin=end;
	}
	printf("[PredictiveEncoderBase] encoded %d in %d bits (Quantization Type is %d,%d)\n",
		m_nbFields,m_outArray.GetLength(),m_pQuant->GetQuantizationType(),m_pQuant->GetQuantizationNbBits());
		
}		
//********************************************************************************	
template<class NatT, class QuantT> 
void PredictiveEncoderBase<NatT,QuantT>::EncodeArrayQP(
	int begin, int IntraInterval, bool hasCompNbBits, int CompNbBits, bool hasCompMin, QuantT PMin)
{
	switch (m_params.getPolicy())
	{
	case INTERVAL :
		m_pBifs->EncodeInt(IntraInterval,m_nbbits,"intraInterval");
		// no break
	default:
		if (begin && hasCompNbBits) m_pBifs->EncodeInt(hasCompNbBits,1,"hasCompNbBits");
		if (hasCompNbBits) m_pBifs->EncodeInt(CompNbBits,5,"CompNbBits");
		if (begin && hasCompMin) m_pBifs->EncodeInt(hasCompMin,1,"hasCompMin");
		if (hasCompMin) PMin.EncodePMin(m_pBifs);
	}
}


//********************************************************************************
//						parseNativeArray		
template<class NatT, class QuantT> 
void PredictiveEncoderBase<NatT,QuantT>::parseNativeArray()
{
	nextToken.Scan();
	if (nextToken == TID_LEFT_BRACKET)
	{
		nextToken.Scan();
		int index=0;
		while (nextToken != TID_RIGHT_BRACKET)
		{
			m_natArray+= NatT::parseValue(m_pQuant);
			index++;
		}
		m_nbFields=index;
		nextToken.Scan();
		return;
	} 
	throw PredictiveEncoderError();
}
//********************************************************************************
//********************************************************************************
#ifdef DEBUGPREDICTIVE
template<class NatT, class QuantT> 
void PredictiveEncoderBase<NatT,QuantT>::DecodeAndPrintOutArray()
{
	
	int begin=0;
	int end=0;
	ZArray<QuantT> decArray;
	ZArray<QuantT> compArray; // compensated Array
	ZArray<NatT>   natArray;
	m_params.Reset();
	int index=0;
	while (end >=0 && end< m_nbFields)
	{
		end=m_params.nextIntra(begin);
		if (end == -1) end=m_nbFields;
		int aadnbBits=m_NbBitsArray[begin];
		fprintf(m_out,"[DecodeAndPrintOutArray] begin = %d | end = %d (Nbbits=%d)[index=%d]\n",begin,end,aadnbBits,index);
		
		index += QuantT::Decode(decArray, m_outArray,index ,end-begin ,aadnbBits,
			m_pQuant,m_out,(m_params.getPolicy()==CHOICE));
		begin=end;
	}
	
	fprintf(m_out,"decArray(%d) =[ ",decArray.GetLength());
	for (int j=0; j< decArray.GetLength(); j++)
	{
		decArray[j].Print(m_out);
	}

	fprintf(m_out," ]\n");
	// COMPENSATION PROCESS
	m_params.Reset();
	index=0;
	begin=0;
	end=0;
	while (end>=0 && end<m_nbFields)
	{
		end=m_params.nextIntra(begin);
		if (end == -1) end=m_nbFields;
		compArray+= decArray[begin];
		natArray += compArray[begin].unQuantize();
		for (int i=begin+1; i<end; i++)
		{
			compArray += ( compArray[i-1] + QuantT::unShift(decArray[i],m_PMinArray[begin]));
			natArray  += (compArray[i].unQuantize());
		}
		begin=end;
	}
	fprintf(m_out,"compArray(%d) =[ ",compArray.GetLength());
	for (j=0; j< compArray.GetLength(); j++)
	{
		compArray[j].Print(m_out);
	}
	fprintf(m_out," ]\n");
	fprintf(m_out,"natArray(%d) =[ ",natArray.GetLength());
	for ( j=0; j< natArray.GetLength(); j++)
	{
		natArray[j].Print(m_out);
	}
	fprintf(m_out," ]\n");
}
//********************************************************************************
template<class NatT, class QuantT> 
void PredictiveEncoderBase<NatT,QuantT>::PrintInput()
{
	fprintf(m_out,"m_natArray(%d)=[ ",m_natArray.GetLength());
	for (int i=0; i<m_natArray.GetLength();i++)
	{
		m_natArray[i].Print(m_out);
	}
	fprintf(m_out,"] \n");
}
//********************************************************************************
template<class NatT, class QuantT> 
void PredictiveEncoderBase<NatT,QuantT>::PrintQ()
{
	fprintf(m_out,"m_quantArray(%d)=[ ",m_quantArray.GetLength());
	for (int i=0; i<m_quantArray.GetLength();i++)
	{
		m_quantArray[i].Print(m_out);
	}
	fprintf(m_out,"] \n");

}
//********************************************************************************
template<class NatT, class QuantT>
void PredictiveEncoderBase<NatT,QuantT>::PrintDelta()
{
	fprintf(m_out,"m_deltaArray(%d)=[ ",m_deltaArray.GetLength());
	for (int i=0; i<m_deltaArray.GetLength();i++)
	{
		m_deltaArray[i].Print(m_out);
	}
	fprintf(m_out,"] \n");

}
//********************************************************************************
template<class NatT, class QuantT> 
void PredictiveEncoderBase<NatT,QuantT>::PrintPminNbBits()
{
	fprintf(m_out,"PminNbBits [ \n");
	m_params.Reset();
	int begin=0;
	int end=0;
	while(end>=0 && end<m_nbFields)
	{
		end=m_params.nextIntra(begin);
		m_PMinArray[begin].Print(m_out);
		fprintf(m_out," nbBits= %d\n,",m_NbBitsArray[begin]);
		begin=end;
	}
	fprintf(m_out,"] \n");
}
//********************************************************************************
template<class NatT, class QuantT> 
void PredictiveEncoderBase<NatT,QuantT>::PrintOutArray()
{
	fprintf(m_out,"OutArray(%d)[ \n",m_outArray.GetLength());
	int nb = m_outIndexes[0];
	int count=0;
	for(int i=0; i< m_outArray.GetLength(); i++)
	{
		while(i==nb)
		{
			nb = m_outIndexes[++count];
			fprintf(m_out,"\n(%d)",count);
		}
		fprintf(m_out," %d ",(m_outArray[i])?1:0);
	}
	fprintf(m_out,"] \n");
}
#endif //DEBUGPREDICTIVE
#endif // !defined(AFX_PREDICTIVEENCODERBASE_H__7CA2BC62_DD48_11D3_9776_525405F4A0F5__INCLUDED_)
