// PredictiveEncoderFactory.cpp: implementation of the PredictiveEncoderFactory class.
//
//////////////////////////////////////////////////////////////////////
//$Log: PredictiveEncoderFactory.cpp,v $
//Revision 1.1.1.1  2003/12/24 00:55:04  mikeseven
//Initial release of the CVS repository for MPEG-4 Systems code
//after the code was corrupted due to reorganization and human 
//errors.
//
//Revision 1.1.1.2  2003/01/30 16:28:30  payan
//no message
//
//Revision 1.1.1.1  2002/07/18 01:08:26  mikeseven
//Initial check in of July 17, 2002 version
//
//Revision 1.1.2.2  2000/03/11 12:48:37  Alex Cotarmanac'h
//Update Vec2f
//
//Revision 1.1.2.1  2000/03/10 15:59:33  Alex Cotarmanac'h
//deuxime envoi
//
//Revision 1.1  2000/03/03 18:41:22  Alex Cotarmanac'h
//Version 1 -- Alexandre Cotarmanac_h
//
//<alexandre.cotarmanac-h@polytechnique.org>
#include "defs.h"
#include "FldTypes.h"
#include "PredictiveEncoderBase.h"
#include "PredictiveEncoderMFFloat.h"
#include "PredictiveEncoderMFVec2f.h"
#include "PredictiveEncoderMFVec3f.h"
#include "PredictiveEncoderMFColor.h"
#include "PredictiveEncoderMFRotation.h"
#include "PredictiveEncoderMFNormal.h"
#include "PredictiveEncoderMFInt32.h"
#include "PredictiveEncoderFactory.h"

PredictiveEncoder * 
PredictiveEncoderFactory::Create(
								 FIELD_TYPE fieldType, 
								 BIFSEncoder * pBifs, 
								 Quantizer * pQuant)
{
	switch (fieldType)
	{
		case MFBOOL : 
			//return new PredictiveMFBOOLEncoder(pBifs,pQuant,pMField) ;
			break;
		case MFCOLOR :      
			return new PredictiveEncoderMFColor(pBifs,pQuant) ;
			break;
		case MFFLOAT : 
			return new PredictiveEncoderMFFloat(pBifs,pQuant);
			break;
		case MFINT32 :  
			return new PredictiveEncoderMFInt32(pBifs,pQuant) ;
			break;
		case MFROTATION :   
			return new PredictiveEncoderMFRot(pBifs,pQuant);
			break;

		case MFTIME : 
			//return new PredictiveMFTIMEEncoder(pBifs,pQuant,pMField);
			break;
		case MFVEC2F :   
			return new PredictiveEncoderMFVec2f(pBifs,pQuant);
			break;
		case MFVEC3F : 
			if (pQuant->GetQuantizationType() == 9) // Normals
				return new PredictiveEncoderMFNormal(pBifs,pQuant);
			return new PredictiveEncoderMFVec3f(pBifs,pQuant);
			break;
		case MFNODE : // CAN NOT USE THE Predictive Encoding ?
		case MFIMAGE : 
		case MFSTRING : 
		case MFURL : 
			break;

	}
	throw PredictiveEncoderError();
	
}
