// PredictiveParameters.cpp: implementation of the PredictiveParameters class.
//
//////////////////////////////////////////////////////////////////////
//$Log: PredictiveParameters.cpp,v $
//Revision 1.1.1.1  2003/12/24 00:55:04  mikeseven
//Initial release of the CVS repository for MPEG-4 Systems code
//after the code was corrupted due to reorganization and human 
//errors.
//
//Revision 1.1.1.2  2003/01/30 16:28:30  payan
//no message
//
//Revision 1.1.1.1  2002/07/18 01:08:26  mikeseven
//Initial check in of July 17, 2002 version
//
//Revision 1.1.2.1  2000/03/10 15:59:33  Alex Cotarmanac'h
//deuxime envoi
//
//Revision 1.1  2000/03/03 18:41:22  Alex Cotarmanac'h
//Version 1 -- Alexandre Cotarmanac_h
//
//<alexandre.cotarmanac-h@polytechnique.org>
#include "defs.h"
#include <assert.h>
#include "ztl.h"
#include "PredictiveParameters.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

PredictiveParameters::PredictiveParameters() :
	m_currentIndex(0), m_finished(false), m_IPPolicy(UNKNOWN)
{
	m_intraArray+=0;
	m_intraIntervalArray+=0;
	printf("creating PredParameters\n");
}	

PredictiveParameters & PredictiveParameters::operator =(const PredictiveParameters &pp)
{
	printf("assignment PredParameters\n");
	if (this != &pp)
	{
		m_intraArray= pp.m_intraArray;
		m_intraIntervalArray=pp.m_intraIntervalArray;
		m_finished=pp.m_finished;
		m_currentIndex=pp.m_currentIndex;
		m_IPPolicy=pp.m_IPPolicy;
	}
	return *this;
}

PredictiveParameters::PredictiveParameters(const PredictiveParameters &pp)
{
	printf("copy PredParameters\n");
	m_intraArray=pp.m_intraArray;
	m_intraIntervalArray=pp.m_intraIntervalArray;

	m_finished=pp.m_finished;
	m_currentIndex=pp.m_currentIndex;
	m_IPPolicy=pp.m_IPPolicy;

}

PredictiveParameters::~PredictiveParameters()
{
	printf("Deleting PredParameters\n");
}

void PredictiveParameters::setIPPolicy(IP_POLICY ip)
{
	m_IPPolicy=ip;
}

void PredictiveParameters::setIntra(int i)
{
	assert (i>0 && m_intraArray[m_currentIndex]<i);
	int a_intra;
	int a_intraInterval;
	switch(m_IPPolicy)
	{
		case INTERVAL:
			assert(m_intraIntervalArray[m_currentIndex] != 0 );

			a_intraInterval = m_intraIntervalArray[m_currentIndex];
			a_intra=m_intraArray[m_currentIndex];

			assert( (i-a_intra)%a_intraInterval == 0 );

			while ( a_intra < i)
			{
				m_currentIndex++;
				a_intra += a_intraInterval;
				m_intraArray.SetAt(m_currentIndex,a_intra);
				m_intraIntervalArray.SetAt(m_currentIndex,a_intraInterval);
			}
			break;
		default:
			m_currentIndex++;
			m_intraArray.SetAt(m_currentIndex,i);
			break;
	}
}

int PredictiveParameters::nextIntra(int i) 
{
	i;
	assert(i>=0);
	assert(m_finished);
	int a_intra=m_intraArray[m_currentIndex];
	a_intra;
	assert (a_intra==i);
	return (m_intraArray[++m_currentIndex]);
}

int PredictiveParameters::intraInterval(int i)
{	
	assert (m_finished);
	assert( (m_IPPolicy == INTERVAL) && i>=0);
	if ( m_intraArray[m_currentIndex-1] != i)
	{
		int k=0;
		while (m_intraArray[k]!=i)
		{	
			k++;
			assert(m_intraArray[k]!=-1);
			assert(m_intraArray[k]<=i);
		}
		return m_intraIntervalArray[k];
	
	}
	return m_intraIntervalArray[m_currentIndex-1];
}

void PredictiveParameters::setIntraInterval(int i)
{
	assert( m_IPPolicy == INTERVAL && i>0);
	m_intraIntervalArray.SetAt(m_currentIndex,i);
}

void PredictiveParameters::finish(int nb_fields)
{
	int a_intra= m_intraArray[m_currentIndex];
	assert( nb_fields >= a_intra);
	if (a_intra < nb_fields)
	{
		m_currentIndex++;
		m_intraArray.SetAt(m_currentIndex,-1);
	}
	m_currentIndex=0;
	m_finished=true;
}




void PredictiveParameters::Reset()
{
	assert( m_finished);
	m_currentIndex=0;
}

IP_POLICY PredictiveParameters::getPolicy()
{
	return m_IPPolicy;
}
