//$Log: PredictiveParser.cpp,v $
//Revision 1.1.1.1  2003/12/24 00:55:04  mikeseven
//Initial release of the CVS repository for MPEG-4 Systems code
//after the code was corrupted due to reorganization and human 
//errors.
//
//Revision 1.1.1.2  2003/01/30 16:28:30  payan
//no message
//
//Revision 1.1.1.1  2002/07/18 01:08:27  mikeseven
//Initial check in of July 17, 2002 version
//
//Revision 1.1  2000/03/03 18:41:22  Alex Cotarmanac'h
//Version 1 -- Alexandre Cotarmanac_h
//
//<alexandre.cotarmanac-h@polytechnique.org>
#include <iostream.h>
#include <string.h>
#include "defs.h"
#include "PredictiveParameters.h"
#include "PredictiveParser.h"



TOKENS PredictiveParser::getToken()
{
	static char delims[] = " \t\n";
	static char buffer[128];
	static isBufferEmpty=true;
	static char * idx;
	if (feof(ifs)) { 
			//ErrorMessage("\n[MESSAGE] EOF");
			return EOFILE;
	}
	if (isBufferEmpty) {
		int c;
		if((c=(int)fgets(buffer,128,ifs))==127)
			ErrorMessage("\nLine is too long (insert a newline)//Bug!!\n");
		if (c==0)
			ErrorMessage("\nfgets returns null pointer\n");
		m_line++;
		//fprintf(stderr,"[MESSAGE] %s",buffer);
		isBufferEmpty=false;
		idx=strtok(buffer,delims);
	}
	else idx=strtok(NULL,delims);
	if (idx==NULL) {
		isBufferEmpty=true;
		return getToken();
	}
	//printf("[PredictiveParser::getToken]%s(%d)\n",idx,m_line);
	if (!stricmp(idx,"{")) return OBRACKET;
	if (!stricmp(idx,"[")) return OBRACKET;
	if (!stricmp(idx,"]")) return CBRACKET;
	if (!stricmp(idx,"}")) return CBRACKET;
	if (!stricmp(idx,",")) return COMMA;
	if (!stricmp(idx,";")) return COMMA;
	if (!stricmp(idx,"predictiveparams")) return PREDICTIVEPARAMS;
	//if (!stricmp(idx,"animtype")) return ANIMTYPE;
	if (!stricmp(idx,"ippolicy")) return IPPOLICY;
	if (!stricmp(idx,"initialarrayqp")) return ARRAYQP;
	if (!stricmp(idx,"arrayqp")) return ARRAYQP;
	if (!stricmp(idx,"intrainterval")) return INTRAINTERVAL;
	//if (!stricmp(idx,"compnbbits")) return COMPNBBITS;
	//if (!stricmp(idx,"compmin")) return COMPMIN;
	if (!stricmp(idx,"intra")) return INTRA;
	if ( sscanf(idx,"%d",&value)!=1) return ERREUR;
	return VALUE;
}

void PredictiveParser::Parse(const char * basename, int nb_fields, PredictiveParameters & a_params)
{
//	printf("[PredictiveParser::Parse] basename= %s , nb_fields=%d\n",basename,nb_fields);
	PredictiveParser a_parser= PredictiveParser(basename, a_params);
	if (a_parser.getToken() != PREDICTIVEPARAMS || a_parser.getToken() != OBRACKET ) {
		a_parser.ErrorMessage(" looking for PredictiveParams {");
		throw PredictiveParserError("MissingPredictiveParams");
	}
	a_parser.ParseBody();
	if (a_parser.getToken() != EOFILE ) {
		a_parser.ErrorMessage(" does not end properly! }");
		throw PredictiveParserError();
	}
	a_params.finish(nb_fields);
}

void PredictiveParser::ParseBody()
{
	if (getToken() != IPPOLICY || getToken() !=VALUE) {
		ErrorMessage(" looking for IPPolicy <value> ");
		return;
	}
	if (value <0 || value>2 ) {
		ErrorMessage(" IPPolicy <value> is 0 1 2");
		return;
	}
	m_rParams.setIPPolicy((enum IP_POLICY)value);
	m_ippolicy=value;
	
	ParseQP(true);				  // Initial ArrayQP
	if (m_ippolicy) ParseExtra();  // 0 -> only predictive
	else 
	{
		if (getToken() != CBRACKET ) {
			ErrorMessage(" looking for closing }");
			return;
		}
	}
}

void PredictiveParser::ParseQP(bool isInitial)
{
	TOKENS t;
	//bool compNbbits=false;
	if ((t=getToken())!=ARRAYQP ) {
		if (t!=CBRACKET || isInitial) {
			ErrorMessage(" looking for closing }");
		}
		return;
	}

	if ( getToken() != OBRACKET){
		ErrorMessage(" looking for ArrayQP { ");
		return;
	}
	t=NONE;
	switch (m_ippolicy)	{
	case 1:
		if ((t=getToken()) != INTRAINTERVAL  || getToken() !=VALUE) {
			if (isInitial) {
				ErrorMessage(" looking for intraInterval <value> ");
				return;
			}	
		}
		else {
			m_intraInterval=value;
			m_rParams.setIntraInterval(m_intraInterval);
			t=NONE;
		}
	default:
		break;
	}
	if (t==NONE) t=getToken();
	if (t != CBRACKET ) {
		ErrorMessage(" looking for closing }");
		return;
	}
}



void PredictiveParser::ParseExtra()
{
	while (getToken()==INTRA)
	{
		if (getToken() != VALUE) {
			ErrorMessage("INTRA <value>");
		}
		
		m_arrayNdx=value;
		m_rParams.setIntra(m_arrayNdx);
		if (m_ippolicy==1 && value%m_intraInterval!=0) {
			ErrorMessage(" value  != 0[intraInterval]");
			return;
		}
		if (getToken() != OBRACKET ) {
			ErrorMessage(" looking for [ after INTRA <value>");
			return;
		}
		
		ParseQP(false);//parse optional ArrayQP
	}
}

void PredictiveParser::ErrorMessage(char *s)
{
	fprintf(stderr,"[ERROR] %s\t(at line %d)\n",s,m_line);
	throw PredictiveParserError(s);
}