//$Log: QuantRot.cpp,v $
//Revision 1.1.1.1  2003/12/24 00:55:04  mikeseven
//Initial release of the CVS repository for MPEG-4 Systems code
//after the code was corrupted due to reorganization and human 
//errors.
//
//Revision 1.2  2003/04/03 04:29:44  mikeseven
//Initial release of Player3D and MPEG-J for all MPEG-4 reference softwares players with samples.
//For installation, follow instructions in docs/Player3D_setup.doc.
//
//Many thanks to Prashanth Gujjeti and Viswanathan Swaminathan from Sun Microsystems.
//
//-- Mikael Bourges-Sevenier (Mindego)
//
//Revision 1.1.1.2  2003/01/30 16:28:31  payan
//no message
//
//Revision 1.1.1.1  2002/07/18 01:08:28  mikeseven
//Initial check in of July 17, 2002 version
//
//Revision 1.1  2000/03/03 18:41:23  Alex Cotarmanac'h
//Version 1 -- Alexandre Cotarmanac_h
//
//<alexandre.cotarmanac-h@polytechnique.org>
#include <assert.h>
#include "defs.h"
#include "NatRot.h"
#include "QuantRot.h"
#include "ztl.h"
#include "ArithModel.h"
#include "BifsEnc.h"
#include <math.h>
#ifdef DEBUGPREDICTIVE
#include "AArithDecoder.h"
#endif
int QuantRot::m_nbAACModels;
//********************************************************************************
extern int getNbBits(int k);

extern void writeInt(ZArray<bool> &o, int n, int nBits);

//********************************************************************************
QuantRot::QuantRot():m_shifted(false)
{

}
QuantRot::~QuantRot()
{

}
//********************************************************************************
QuantRot::QuantRot( Quantizer *pQuant,int orient, int b, int c, int d) : 
	 m_nbbits(-1),m_shifted(false)
{
		assert(pQuant !=NULL);
		pQuant->GetRotData(m_nbbits);
		QuantRot::m_nbAACModels=1;
		m_qv[0]=orient;
		m_qv[1]=b;
		m_qv[2]=c;
		m_qv[3]=d;
		assert (m_nbbits>=0);
}
//********************************************************************************
QuantRot::QuantRot(int nbbits, int orient, int b, int c, int d):
	m_nbbits(nbbits),m_shifted(false)
{
		m_qv[0]=orient;
		m_qv[1]=b;
		m_qv[2]=c;
		m_qv[3]=d;
		assert (m_nbbits>=0);
}
//********************************************************************************
void QuantRot::setNbAACModels(Quantizer * pQuant)//sets static vars
{
		pQuant;
		QuantRot::m_nbAACModels=1;
}
//********************************************************************************
QuantRot QuantRot::CompMin(ZArray<QuantRot> & qFArray, int begin, int end, int &nbits)
{
	QuantRot Min=qFArray[begin];
	QuantRot Max=qFArray[begin];
	for (int i=begin+1; i<end;i++)
	{
#ifdef DEBUGPREDICTIVE
#ifdef DEBUGCOMPMIN
		printf("[ min= ");
		Min.Print();
		printf(" ; max= ");
		Max.Print();
		printf(" ]\n");
#endif
#endif
		for (int j=1; j<4;j++)
		{
			if (qFArray[i].m_qv[j]<Min.m_qv[j]) 
				Min.m_qv[j]=qFArray[i].m_qv[j];
			if (qFArray[i].m_qv[j]>Max.m_qv[j]) 
				Max.m_qv[j]=qFArray[i].m_qv[j];
		}


	}

	Min.m_qv[1]= (Min.m_qv[1]<Min.m_qv[2])?Min.m_qv[1]:Min.m_qv[2];
	Min.m_qv[2]= (Min.m_qv[1]<Min.m_qv[3])?Min.m_qv[1]:Min.m_qv[3];
	Min.m_qv[1]= Min.m_qv[2]; // the Min is set on all comps.
	Min.m_qv[3]= Min.m_qv[2];
	Max.m_qv[1]= (Max.m_qv[1]>Max.m_qv[2])?Max.m_qv[1]:Max.m_qv[2];
	Max.m_qv[2]= (Max.m_qv[1]>Max.m_qv[3])?Max.m_qv[1]:Max.m_qv[3];
	Max.m_qv[1]= Max.m_qv[2]; // the Max is set on all comps.
	Max.m_qv[3]= Max.m_qv[2];
	nbits = getNbBits(Max.m_qv[1] - Min.m_qv[1]);
	// Give a value for Min.m_qv[0]
	Min.m_qv[0]=-1;
	return Min;
}
//********************************************************************************
void QuantRot::Shift(ZArray<QuantRot> &d, ZArray<QuantRot> &q, 
					   QuantRot PMin, int begin, int end)
{
	for (int i=begin; i<end;i++)
	{
		d.SetAt(i,q[i]);
		for (int j=1;j<4;j++) d[i].m_qv[j] -= PMin.m_qv[j];
	}
		
}
//********************************************************************************
QuantRot QuantRot::unShift(QuantRot a, QuantRot Pmin)
{
	QuantRot r= a;
	for (int i=1; i<=m_nbComp;i++)
		r.m_qv[i] = a.m_qv[i] + Pmin.m_qv[i];
	return r;
}
//********************************************************************************
void QuantRot::AAEncode(ZArray<bool> & oA, ZArray<int> &oIndx, ZArray<QuantRot> & iA, 
						  int nbbits, int begin, int end, bool isChoice)
{
	ArithModel model[m_nbComp]; // m_nbComp >= m_nbAACModels
	printf("NbAACModels : %d",m_nbAACModels);
	for (int h=0; h<m_nbAACModels; h++) { 
		model[h]=ArithModel(); 
		model[h].init(1<<nbbits); //compnbbits
	}
	
	ArithEncoder aae= ArithEncoder(oA);
	aae.reset();
	for (int i=begin; i<end;i++)
	{
		
#ifdef DEBUGPREDICTIVE
//#ifdef DEBUGAAENCODE
		printf("{encoding %d : ",i);
		iA[i].Print(stdout);
		printf(" < %d(bits)}\n",1<< nbbits);
//		model[0].PrintFreq();
//		model[0].PrintCumFreq();

//#endif
#endif
		int bits=iA[i].EncodeAA(aae,model);
		if (isChoice && (i!=end-1)) 
		{
			int obits=bits;
			bits+=aae.flush();
			printf("[AAEC]oA.GetLength %d ?  bitsT=%d obits=%d nbits=%d\n",oA.GetLength(),bits+oIndx[i-1],obits,bits);
		}
		//printf("[AAEC]%d in %d bits\n",i,bits);
		if (i!=end-1) oIndx.SetAt(i,bits+oIndx[i-1]/*oA.GetLength()*/);
		//oIndx.SetAt(i,bits+oIndx[i-1]
	}
#ifdef DEBUGPREDICTIVE
//#ifdef DEBUGAAENCODE
//	model[0].PrintFreq();
//#ifdef DEBUGAAENCODE
#endif
	aae.flush();
	oIndx.SetAt(end-1,oA.GetLength());
	printf("{End if this round} begin=%d; end=%d; oIndx[end-1]=%d; outArray.lg()=%d\n",begin,end,oIndx[end-1], oA.GetLength());
	
}
//********************************************************************************
int QuantRot::EncodeAA(ArithEncoder & aae, ArithModel model[] )
{
	int bits=0;
	int bitsRead=0;
	int modelno=0;
	aae.encodeAndUpdate(m_qv[1],model[modelno],"AAC",bitsRead);// bits is updated += bitsRead
	bits+=bitsRead;
	aae.encodeAndUpdate(m_qv[2],model[(m_nbAACModels>1)?++modelno:modelno],"AAC",bitsRead);
	bits+=bitsRead;
	aae.encodeAndUpdate(m_qv[3],model[(m_nbAACModels>1)?++modelno:modelno],"AAC",bitsRead);
	bits+=bitsRead;
	return bits;
}
//********************************************************************************
void QuantRot::WriteIntra(ZArray<bool> &o)
{
	int nbBits = m_nbbits;
	writeInt(o, m_qv[0],2);//orientation in 2 bits
	writeInt(o, m_qv[1], nbBits);
	writeInt(o, m_qv[2], nbBits);
	writeInt(o, m_qv[3], nbBits);
}
//********************************************************************************
QuantRot & QuantRot::operator +=(QuantRot aQi)
{
	assert(m_shifted==false);
	assert(aQi.m_shifted==false);
	shift();
	aQi.shift();

	int dqv[4];
	int scale = (1<<(m_nbbits-1)) -1;
	int inv=1;
	int diffOrientation=0;
	for (int i=1;i<m_nbComp+1;i++)
	{
		dqv[i]=m_qv[i]+aQi.m_qv[i];
		if ( fabs(dqv[i])>scale )
		{	
			diffOrientation=i;
			inv=(dqv[i]>0)?1:-1;
			break;
		}
	}
	if (diffOrientation!=0)
	{
		int k=0;
		{
			for (int j=0; j<m_nbComp-diffOrientation;j++)
			{
				k = ((j+diffOrientation)%m_nbComp);
				dqv[j+1]=inv*(m_qv[k+1]+aQi.m_qv[k+1]);
			}
		}
		k=diffOrientation-1;
		dqv[m_nbComp-diffOrientation+1] = 2*inv*scale-(m_qv[k+1]+aQi.m_qv[k+1]) ;
		for (int j=m_nbComp-diffOrientation+1;j<m_nbComp;j++) {
        k = (j+diffOrientation-1)%m_nbComp;
        dqv[j+1] =inv*(m_qv[k+1]+aQi.m_qv[k+1]);
      }

	}
	dqv[0]=(m_qv[0]+diffOrientation)%(m_nbComp+1);
	{for (int i=0;i<=m_nbComp;i++) m_qv[i]=dqv[i];}
	unshift();
	aQi.unshift();
	return *this;
}
//********************************************************************************
QuantRot & QuantRot::operator -=(QuantRot AQi)
{
	assert(m_shifted==false);
	assert(AQi.m_shifted==false);
	shift();
	AQi.shift();
	int dqv[4];
	int inv=1;
	int scale = (1<<(m_nbbits-1)) -1;
	int diffOrientation= (m_qv[0]-AQi.m_qv[0]+ m_nbComp+1)%(m_nbComp+1);
	if (diffOrientation==0)
	{
		for (int j=1; j<m_nbComp+1;j++)
			dqv[j]=m_qv[j]-AQi.m_qv[j];
	}
	else
	{
		int k=m_nbComp-diffOrientation+1;
		int crossLineDistance=( m_qv[k]+AQi.m_qv[diffOrientation] );
		inv = (crossLineDistance<0) ? -1 : 1;
 		// PORTING to CC 5.3
 		int j = 0;
 		//for (int j=1;j<diffOrientation;j++) {
 		for ( j=1;j<diffOrientation;j++) {
			k = (m_nbComp+j-diffOrientation+1);
			dqv[j]=inv*m_qv[k]-AQi.m_qv[j];
		}

		dqv[diffOrientation]=inv*2*scale-crossLineDistance;
		for ( j=diffOrientation+1;j<m_nbComp+1;j++) {
			k = j-diffOrientation;
			dqv[j]=inv*m_qv[k]-AQi.m_qv[j];
      }

	}
	dqv[0]=(inv*diffOrientation==1)?1:1;
	for (int i=0;i<=m_nbComp;i++) m_qv[i]=dqv[i];
	unshift();
	AQi.unshift();
	return *this;
}
//********************************************************************************
NatRot QuantRot::unQuantize()
{
	assert(m_shifted==false);
	float v[4];
	float quat[4];
	for (int i=1; i<=m_nbComp; i++)
	{
		float t= m_qv[i]-(1<<(m_nbbits-1));// could use shift()..
		v[i]=((t>=0)?1:-1)*Quantizer::unQuantize(((t>=0)?t:-t),0,1,m_nbbits-1,false);
	}
	int k;
	float sum=1;
	for ( k=1; k<=m_nbComp; k++) sum+=(float)( tan(M_PI*v[k]/4)*tan(M_PI*v[k]/4));
	k=m_qv[0];
	quat[k]=(float)(1/sqrt(sum));
	for ( int j=0; j<m_nbComp;j++) 
		quat[(j+k+1)%(m_nbComp+1)]=(float)(tan(M_PI*v[j+1]/4)*quat[k]);
	

	return NatRot(m_nbbits,quat);
}
//********************************************************************************
QuantRot operator+ (QuantRot a, QuantRot b)
{
	QuantRot r = a;
	return (r+=b);
}
//********************************************************************************
QuantRot operator- (QuantRot a, QuantRot b)
{
	QuantRot r = a;
	return (r-=b);
}
//********************************************************************************
/* only for comparing Pmins */
bool operator != (QuantRot a, QuantRot b)
{
	assert(a.m_shifted==false);
	assert(b.m_shifted==false);
	bool res=false;
	for (int i=1; i<=a.m_nbComp; i++) res |= (a.m_qv[i]!=b.m_qv[i]);
	return res;
}
//********************************************************************************
void QuantRot::Encode(BIFSEncoder *pBifs)
{
		pBifs->EncodeInt(m_qv[0],2,"Intra.direction");
		pBifs->EncodeInt(m_qv[1],m_nbbits,"Intra.XQuaternion");
		pBifs->EncodeInt(m_qv[2],m_nbbits,"Intra.YQuaternion");
		pBifs->EncodeInt(m_qv[3],m_nbbits,"Intra.ZQuaternion");
}
//********************************************************************************
void QuantRot::EncodePMin(BIFSEncoder *pBifs)
{

	pBifs->EncodeInt(m_qv[1]+(1<<m_nbbits),m_nbbits+1,"CompMin.XRot");
/*	if (m_nbAACModels>1) 
	{
		pBifs->EncodeInt(m_qv[2]+(1<<m_nbbits),m_nbbits+1,"CompMin.YRot");
		pBifs->EncodeInt(m_qv[3]+(1<<m_nbbits),m_nbbits+1,"CompMin.ZRot");
	}
*/
}
/*** Shifts m_qv[]  to -1 +1 values */
void QuantRot::shift()
{
	assert(m_shifted==false);
	for (int i=1; i<=m_nbComp; i++) m_qv[i]-= (1<<(m_nbbits-1));
	m_shifted=true;
}
//********************************************************************************
/*** Shifts m_qv[] back to positive values */
void QuantRot::unshift()
{
	assert(m_shifted==true);
	for (int i=1; i<=m_nbComp; i++) m_qv[i]+= (1<<(m_nbbits-1));
	m_shifted=false;
}
//********************************************************************************
//********************************************************************************

#ifdef DEBUGPREDICTIVE
//********************************************************************************
int QuantRot::Decode(ZArray<QuantRot> & decArray, ZArray<bool> & m_outArray,int index,
		int nb, int aadnbBits,Quantizer * m_pQuant, FILE * out, bool isChoice)
{
	fprintf(out,"[QuantRot::Decode] index=%d\n",index);
	int ibits=m_pQuant->GetQuantizationNbBits();
	// Read intra
	int orient=0;
	for (int i=0; i<2; i++) orient= (orient<<1) + (m_outArray[i+index]?1:0);
	int vx=0;
	for (i=0; i<ibits; i++) vx= (vx<<1) + (m_outArray[i+index+2]?1:0);
	int vy=0;
	for (i=0; i<ibits; i++) vy= (vy<<1) + (m_outArray[i+index+ibits+2]?1:0);
	int vz=0;
	for (i=0; i<ibits; i++) vz= (vz<<1) + (m_outArray[i+index+2*ibits+2]?1:0);
	QuantRot IntraV= QuantRot(m_pQuant,orient,vx,vy,vz);
	printf("QuantV =  ");
	IntraV.Print(stdout);
	printf("(index=%d)",index);
	
	decArray+=IntraV;

	ArithModel model[m_nbComp]; 
	for (int h=0; h<m_nbComp; h++) { 
		model[h]=ArithModel(); 
		model[h].init(1<<aadnbBits); //compnbbits
	}
	ArithDecoder aad= ArithDecoder(m_outArray,index+(ibits*3+2));
	aad.reset();
	int bitsRead=0;
	int bitsReadFrame=0;
	for (i=0; i< nb-1 ; i++)
	{
		int modelno=0;
		int vx=aad.decodeAndUpdate(model[modelno],bitsReadFrame);
		bitsRead+=bitsReadFrame;
		int vy=aad.decodeAndUpdate(model[(m_nbAACModels>1)?++modelno:modelno],bitsReadFrame);
		bitsRead+=bitsReadFrame;
		int vz=aad.decodeAndUpdate(model[(m_nbAACModels>1)?++modelno:modelno],bitsReadFrame);
		bitsRead+=bitsReadFrame;
		
		if (isChoice && (i+1 < nb-1)) 
		{
			aad.flush(bitsRead);
		}
		QuantRot Value=QuantRot(m_pQuant,-1,vx,vy,vz);
		fprintf(out,"[QuantRot::Decode](%d) value=",i+1);
		Value.Print(out);
		fprintf(out,"  (%d bits) [total =%d]\n",bitsReadFrame,bitsRead);
		decArray+=Value;
		//bitsRead+=bitsReadFrame;
	}
	aad.flush(bitsRead);
	fprintf(out,"Finally read : %d\n",bitsRead);
	return bitsRead+(ibits*m_nbComp+2);
}
//********************************************************************************
void QuantRot::Print(FILE * out)
{
	fprintf(out,"[%d] %d %d %d",m_qv[0], m_qv[1], m_qv[2],m_qv[3]);
}

#endif
