//$Log: QuantVec3f.cpp,v $
//Revision 1.1.1.1  2003/12/24 00:55:05  mikeseven
//Initial release of the CVS repository for MPEG-4 Systems code
//after the code was corrupted due to reorganization and human 
//errors.
//
//Revision 1.1.1.2  2003/01/30 16:28:32  payan
//no message
//
//Revision 1.1.1.1  2002/07/18 01:08:29  mikeseven
//Initial check in of July 17, 2002 version
//
//Revision 1.1  2000/03/03 18:41:23  Alex Cotarmanac'h
//Version 1 -- Alexandre Cotarmanac_h
//
//<alexandre.cotarmanac-h@polytechnique.org>
#include <assert.h>
#include "defs.h"
#include "NatVec3f.h"
#include "QuantVec3f.h"
#include "ztl.h"
#include "ArithModel.h"
#include "BifsEnc.h"
#ifdef DEBUGPREDICTIVE
#include "AArithDecoder.h"
#endif
int QuantVec3f::m_nbAACModels;
//********************************************************************************
extern int getNbBits(int k);

extern void writeInt(ZArray<bool> &o, int n, int nBits);

//********************************************************************************
QuantVec3f::QuantVec3f()
{

}
QuantVec3f::~QuantVec3f()
{

}
//********************************************************************************
QuantVec3f::QuantVec3f( Quantizer *pQuant,int x, int y, int z) : 
	m_x(x), m_y(y), m_z(z), m_nbbits(-1)
{
		assert(pQuant !=NULL);
		pQuant->GetVec3fData(m_nbbits,m_mini,m_maxi);
		switch(pQuant->GetQuantizationType())
		{
		case 1 :
			QuantVec3f::m_nbAACModels=3;
			break;
		default:
			QuantVec3f::m_nbAACModels=1;
		}
		assert (m_nbbits>=0);
}
//********************************************************************************
QuantVec3f::QuantVec3f(int nbbits, float mini[], float maxi[], int x, int y, int z):
	m_x(x), m_y(y), m_z(z),m_nbbits(nbbits)
{
	for (int i=0; i< m_nbComp; i++)
	{
		m_mini[i]=mini[i];
		m_maxi[i]=maxi[i];
	}
}
//********************************************************************************
void QuantVec3f::setNbAACModels(Quantizer * pQuant)//sets static vars
{
	switch(pQuant->GetQuantizationType())
	{
	case 1 :
		QuantVec3f::m_nbAACModels=3;
		break;
	default:
		QuantVec3f::m_nbAACModels=1;
	}
}
//********************************************************************************
QuantVec3f QuantVec3f::CompMin(ZArray<QuantVec3f> & qFArray, int begin, int end, int &nbits)
{
	QuantVec3f Min=qFArray[begin];
	QuantVec3f Max=qFArray[begin];
	for (int i=begin+1; i<end;i++)
	{
#ifdef DEBUGPREDICTIVE
#ifdef DEBUGCOMPMIN
		printf("[ min= ");
		Min.Print();
		printf(" ; max= ");
		Max.Print();
		printf(" ]\n");
#endif
#endif
		if (qFArray[i].m_x<Min.m_x) 
			Min.m_x=qFArray[i].m_x;

		if (qFArray[i].m_y<Min.m_y) 
			Min.m_y=qFArray[i].m_y;

		if (qFArray[i].m_z<Min.m_z) 
			Min.m_z=qFArray[i].m_z;

		if (qFArray[i].m_x>Max.m_x) 
			Max.m_x=qFArray[i].m_x;
		if (qFArray[i].m_y>Max.m_y) 
			Max.m_y=qFArray[i].m_y;
		if (qFArray[i].m_z>Max.m_z) 
			Max.m_z=qFArray[i].m_z;
	}
	if (m_nbAACModels>1) // Position
	{
		// Calculate number of bits.
		int nbbitsx=getNbBits(Max.m_x - Min.m_x);
		int nbbitsy=getNbBits(Max.m_y - Min.m_y);
		int nbbitsz=getNbBits(Max.m_z - Min.m_z);
		int max_xy  =(nbbitsx>nbbitsy)?nbbitsx:nbbitsy;
		int max_xyz =(max_xy>nbbitsz)?max_xy:nbbitsz; 
		nbits=max_xyz;
	}
	else 
	{
		Min.m_x= (Min.m_x<Min.m_y)?Min.m_x:Min.m_y;
		Min.m_y= (Min.m_x<Min.m_z)?Min.m_x:Min.m_z;
		Min.m_x= Min.m_y; // the Min is set on all comps.
		Min.m_z= Min.m_y;
		Max.m_x= (Max.m_x>Max.m_y)?Max.m_x:Max.m_y;
		Max.m_y= (Max.m_x>Max.m_z)?Max.m_x:Max.m_z;
		Max.m_x= Max.m_y; // the Max is set on all comps.
		Max.m_z= Max.m_y;
		nbits = getNbBits(Max.m_x - Min.m_x);
	}
	return Min;
}
//********************************************************************************
void QuantVec3f::Shift(ZArray<QuantVec3f> &d, ZArray<QuantVec3f> &q, 
					   QuantVec3f PMin, int begin, int end)
{
	for (int i=begin; i<end;i++)
		d.SetAt(i,q[i]-PMin);
}
//********************************************************************************
QuantVec3f QuantVec3f::unShift(QuantVec3f a, QuantVec3f Pmin)
{
	return (a+Pmin);
}
//********************************************************************************
void QuantVec3f::AAEncode(ZArray<bool> & oA, ZArray<int> &oIndx, ZArray<QuantVec3f> & iA, 
						  int nbbits, int begin, int end, bool isChoice)
{
	ArithModel model[m_nbComp]; // m_nbComp >= m_nbAACModels
	printf("NbAACModels : %d",m_nbAACModels);
	for (int h=0; h<m_nbAACModels; h++) { 
		model[h]=ArithModel(); 
		model[h].init(1<<nbbits); //compnbbits
	}
	
	ArithEncoder aae= ArithEncoder(oA);
	aae.reset();
	for (int i=begin; i<end;i++)
	{
		
#ifdef DEBUGPREDICTIVE
//#ifdef DEBUGAAENCODE
//		printf("{encoding %d : ",i);
//		iA[i].Print(stdout);
//		printf(" < %d(bits)}\n",1<< nbbits);
//		model[0].PrintFreq();
//		model[0].PrintCumFreq();

//#endif
#endif
		int bits=iA[i].EncodeAA(aae,model);
		if (isChoice && (i!=end-1)) 
		{
			int obits=bits;
			bits+=aae.flush();
			printf("[AAEC]oA.GetLength %d ?  bitsT=%d obits=%d nbits=%d\n",oA.GetLength(),bits+oIndx[i-1],obits,bits);
		}
		//printf("[AAEC]%d in %d bits\n",i,bits);
		if (i!=end-1) oIndx.SetAt(i,bits+oIndx[i-1]/*oA.GetLength()*/);
		//oIndx.SetAt(i,bits+oIndx[i-1]
	}
#ifdef DEBUGPREDICTIVE
//#ifdef DEBUGAAENCODE
//	model[0].PrintFreq();
//#ifdef DEBUGAAENCODE
#endif
	aae.flush();
	oIndx.SetAt(end-1,oA.GetLength());
	printf("{End if this round} begin=%d; end=%d; oIndx[end-1]=%d; outArray.lg()=%d\n",begin,end,oIndx[end-1], oA.GetLength());
	
}
//********************************************************************************
int QuantVec3f::EncodeAA(ArithEncoder & aae, ArithModel model[] )
{
	int bits=0;
	int bitsRead=0;
	int modelno=0;
	aae.encodeAndUpdate(m_x,model[modelno],"AAC",bitsRead);// bits is updated += bitsRead
	bits+=bitsRead;
	aae.encodeAndUpdate(m_y,model[(m_nbAACModels>1)?++modelno:modelno],"AAC",bitsRead);
	bits+=bitsRead;
	aae.encodeAndUpdate(m_z,model[(m_nbAACModels>1)?++modelno:modelno],"AAC",bitsRead);
	bits+=bitsRead;
	return bits;
}
//********************************************************************************
void QuantVec3f::WriteIntra(ZArray<bool> &o)
{
	int nbBits = m_nbbits;
	writeInt(o, m_x, nbBits);
	writeInt(o, m_y, nbBits);
	writeInt(o, m_z, nbBits);
}
//********************************************************************************
QuantVec3f & QuantVec3f::operator +=(QuantVec3f aQI)
{
	m_x+=aQI.m_x;
	m_y+=aQI.m_y;
	m_z+=aQI.m_z;
	return *this;
}
//********************************************************************************
QuantVec3f & QuantVec3f::operator -=(QuantVec3f aQI)
{
	m_x-=aQI.m_x;
	m_y-=aQI.m_y;
	m_z-=aQI.m_z;
	return *this;
}
//********************************************************************************
NatVec3f QuantVec3f::unQuantize()
{
	float x=Quantizer::unQuantize(m_x,m_mini[0],m_maxi[0],m_nbbits,false);
	float y=Quantizer::unQuantize(m_y,m_mini[1],m_maxi[1],m_nbbits,false);
	float z=Quantizer::unQuantize(m_z,m_mini[2],m_maxi[2],m_nbbits,false);
	return NatVec3f(m_nbbits,m_mini,m_maxi,x,y,z);
}
//********************************************************************************
QuantVec3f operator+ (QuantVec3f a, QuantVec3f b)
{
	QuantVec3f r = a;
	return (r+=b);
}
//********************************************************************************
QuantVec3f operator- (QuantVec3f a, QuantVec3f b)
{
	QuantVec3f r = a;
	return (r-=b);
}
//********************************************************************************
bool operator != (QuantVec3f a, QuantVec3f b)
{
	return ((a.m_x!=b.m_x)||(a.m_y!=b.m_y)||(a.m_z!=b.m_z));
}
//********************************************************************************
void QuantVec3f::Encode(BIFSEncoder *pBifs)
{
		pBifs->EncodeInt(m_x,m_nbbits,"Intra.X");
		pBifs->EncodeInt(m_y,m_nbbits,"Intra.Y");
		pBifs->EncodeInt(m_z,m_nbbits,"Intra.Z");
}
//********************************************************************************
void QuantVec3f::EncodePMin(BIFSEncoder *pBifs)
{

	pBifs->EncodeInt(m_x+(1<<m_nbbits),m_nbbits+1,"CompMin.X");
	if (m_nbAACModels>1) // only if Position
	{
		pBifs->EncodeInt(m_y+(1<<m_nbbits),m_nbbits+1,"CompMin.Y");
		pBifs->EncodeInt(m_z+(1<<m_nbbits),m_nbbits+1,"CompMin.Z");
	}
}
//********************************************************************************
//********************************************************************************

#ifdef DEBUGPREDICTIVE
//********************************************************************************
int QuantVec3f::Decode(ZArray<QuantVec3f> & decArray, ZArray<bool> & m_outArray,int index,
		int nb, int aadnbBits,Quantizer * m_pQuant, FILE * out, bool isChoice)
{
	fprintf(out,"[QuantVec3f::Decode] index=%d\n",index);
	int ibits=m_pQuant->GetQuantizationNbBits();
	// Read intra
	int vx=0;
	for (int i=0; i<ibits; i++) vx= (vx<<1) + (m_outArray[i+index]?1:0);
	int vy=0;
	for (i=0; i<ibits; i++) vy= (vy<<1) + (m_outArray[i+index+ibits]?1:0);
	int vz=0;
	for (i=0; i<ibits; i++) vz= (vz<<1) + (m_outArray[i+index+2*ibits]?1:0);
	QuantVec3f IntraV= QuantVec3f(m_pQuant,vx,vy,vz);
	printf("QuantV =  ");
	IntraV.Print(stdout);
	printf("(index=%d)",index);
	
	decArray+=IntraV;

	ArithModel model[m_nbComp]; 
	for (int h=0; h<m_nbComp; h++) { 
		model[h]=ArithModel(); 
		model[h].init(1<<aadnbBits); //compnbbits
	}
	ArithDecoder aad= ArithDecoder(m_outArray,index+(ibits*m_nbComp));
	aad.reset();
	int bitsRead=0;
	int bitsReadFrame=0;
	for (i=0; i< nb-1 ; i++)
	{
		int modelno=0;
		int vx=aad.decodeAndUpdate(model[modelno],bitsReadFrame);
		bitsRead+=bitsReadFrame;
		int vy=aad.decodeAndUpdate(model[(m_nbAACModels>1)?++modelno:modelno],bitsReadFrame);
		bitsRead+=bitsReadFrame;
		int vz=aad.decodeAndUpdate(model[(m_nbAACModels>1)?++modelno:modelno],bitsReadFrame);
		bitsRead+=bitsReadFrame;
		
		if (isChoice && (i+1 < nb-1)) 
		{
			aad.flush(bitsRead);
		}
		QuantVec3f Value=QuantVec3f(m_pQuant,vx,vy,vz);
		fprintf(out,"[QuantVec3f::Decode](%d) value=",i+1);
		Value.Print(out);
		fprintf(out,"  (%d bits) [total =%d]\n",bitsReadFrame,bitsRead);
		decArray+=Value;
		//bitsRead+=bitsReadFrame;
	}
	aad.flush(bitsRead);
	fprintf(out,"Finally read : %d\n",bitsRead);
	return bitsRead+(ibits*m_nbComp);
}
//********************************************************************************
void QuantVec3f::Print(FILE * out)
{
	fprintf(out,"%d %d %d ",m_x, m_y, m_z);
}

#endif
