// -*- c++ -*-

// This software module was originally developed by Lars Nilsson (Telenor
// R&D) in the course of development of the MPEG-4 Systems standard
// (ISO/IEC 14496-1).  This software module is an implementation of a part
// of one or more MPEG-4 Systems tools as specified by the MPEG-4 Systems
// standard.  ISO/IEC gives users of the MPEG-4 Systems standard free
// license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// For more information or to receive an up to date version of this module,
// mail karlo@nta.no or check <http://televr.fou.telenor.no/compositor/>.
//
// Copyright 1997, 1998.
// Modification Marco Albrecht / Helge Drumm
// NRMatrix.h: Schnittstelle fr die Klasse NRMatrix.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_NRMATRIX_H__A94F13C2_7CF5_4447_8994_F3574880B5F3__INCLUDED_)
#define AFX_NRMATRIX_H__A94F13C2_7CF5_4447_8994_F3574880B5F3__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <stdio.h>
#include "../Include/defs.h" // SOLARIS PORTING - for BOOL

class Rotation;
class Vec3f;

/// NRMatrix class

class NRMatrix
{
public:

  /// Constructor

			NRMatrix();

  /// Constructor

			NRMatrix(float matrix[16]);

  /// Constructor

			NRMatrix(const Vec3f& rXAxis,
			       const Vec3f& rYAxis,
			       const Vec3f& rZAxis);

   // MP Build a matrix from a rotation
  			NRMatrix(const Rotation& rot);

  // MP Build a matrix from a translation
  			NRMatrix(const Vec3f& tra);

  /// Init Matrix with values

  void          InitMatrix(float matrix[16]);
			
  /// Translate

  void			Translate(float fX, float fY, float fZ);

  /// Translate

  void			Translate(const Vec3f& rTranslation);

  /// Translate

  void			Scale(float fX, float fY, float fZ);

  /// Rotate

  void			Rotate(float fAngle, float fX, float fY, float fZ);

  /// Rotate

  void			Rotate(const Rotation& rRotation);

  /// MP:: Identity Transform 

  void			LoadIdentityTransform();


  void operator =(const Rotation& q);

  NRMatrix operator*=(const NRMatrix&m);
  NRMatrix operator*(const NRMatrix&m);


  /// Get elements
  // SOLARIS PORTING - Added "const"

// WIN-SOL release
#ifdef WIN32
  float&		operator [](int element);
#else
  // PORTING to CC 5.3
//  float&		operator [](int element) const;
  float&		operator [](int element);
#endif

  /// Get elements

			operator float*() const;

  /// Multiply point with matrix

  Vec3f			MultiplyPoint(const Vec3f& rPoint) const;

  /// Multiply point with matrix

  Vec3f			MultiplyVector(const Vec3f& rPoint) const;

  /// Return whether matrix only has translation

  BOOL			HasOnlyTranslation() const;

#ifdef _DEBUG
    
  /// Debugging output

  void                  Print(FILE* pStream);

#endif


  float			m_elements[16];
  double        m_elementsd[16];
};

#include "NRMatrix.i"


#endif // !defined(AFX_NRMATRIX_H__A94F13C2_7CF5_4447_8994_F3574880B5F3__INCLUDED_)
