/********************************************************************
  This software module was originally developed by Helge Drumm and Marco 
  Albrecht in the context of the IAVAS Project. For the object based
  functionality the IM1 Implementation of MPEG-4 Systems implementation
  is used.
*********************************************************************/

/*--------------------------------------------------------------------
Project I3D: IAVAS 3D Player

FILE	: NRTransform.cpp
AUTHOR	: Helge Drumm, Marco Albrecht
OVERVIEW: Implementation of class NRTransform 
CVS     : $Id: NRTransform.cpp,v 1.2 2007/04/18 17:36:09 drumm Exp $
----------------------------------------------------------------------*/



#include "NRTransform.h"

#ifndef M_PI
#define M_PI 3.14159265358979323846
#endif

//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////

NRTransform::NRTransform(Transform *refnode)
{
   this->refnode = (Transform*) refnode;
   
   this->isGroupingNode=true;
   
   this->center = (0,0,0);
   this->rotation = (0,0,1,0);
   this->scale = (1,1,1);
   this->scaleOrientation = (0,0,1,0);
   this->translation = (0,0,0);

   this->penumbra=0;
   
  
}

NRTransform::~NRTransform()
{

}


void NRTransform::UpdateValue(Vec3f Upd_center,Rotation Upd_rotation,Vec3f Upd_scale,Rotation Upd_scaleOrientation,Vec3f Upd_translation)
{
  this->center.x = Upd_center.x;
  this->center.y = Upd_center.y;
  this->center.z = Upd_center.z;

  this->rotation.theta = Upd_rotation.theta;
  this->rotation.x     = Upd_rotation.x;
  this->rotation.y     = Upd_rotation.y;
  this->rotation.z     = Upd_rotation.z;

  this->scale.x = Upd_scale.x;
  this->scale.y = Upd_scale.y;
  this->scale.z = Upd_scale.z;

  this->scaleOrientation.theta = Upd_scaleOrientation.theta;
  this->scaleOrientation.x     = Upd_scaleOrientation.x;
  this->scaleOrientation.y     = Upd_scaleOrientation.y;
  this->scaleOrientation.z     = Upd_scaleOrientation.z;

  this->translation.x = Upd_translation.x;
  this->translation.y = Upd_translation.y;
  this->translation.z = Upd_translation.z;

}


void NRTransform::SetMatrix(void)
{
 
  /*
  // do temporary offset of origo
  
  if (this->center.x != 0.0f || this->center.y != 0.0f || this->center.z != 0.0f)
    Matrix4x4->Translate(center.x,center.y,center.z);
  
  if (this->scaleOrientation.theta != 0.0f) {
    // do scaleRotation
    
    Matrix4x4->Rotate(scaleOrientation.theta,scaleOrientation.x,scaleOrientation.y,scaleOrientation.z);
    
    // do inverse scaling
    
    if (this->scale.x != 1.0f || this->scale.y != 1.0f || this->scale.z != 1.0f)
     Matrix4x4->Scale(1.0f / scale.x, 1.0f / scale.y, 1.0f / scale.z);
    
    // undo scalerotation
    
   Matrix4x4->Rotate(-scaleOrientation.theta,scaleOrientation.x,scaleOrientation.y,scaleOrientation.z);
  } else 
    // do inverse scaling
    
    if (this->scale.x != 1.0f || this->scale.y != 1.0f || this->scale.z != 1.0f)
      Matrix4x4->Scale(1.0f / scale.x,1.0f / scale.y,1.0f / scale.z);
    
    // do inverse rotation
    
    if (this->rotation.theta != 0.0f)
      Matrix4x4->Rotate(-rotation.theta,
      rotation.x,
      rotation.y,
      rotation.z);
    
    // undo temporary offset of origo
    
    if (this->center.x != 0.0f ||
      this->center.y != 0.0f ||
      this->center.z != 0.0f)
      Matrix4x4->Translate(-center.x,
      -center.y,
      -center.z);
    
    // do inverse translation
    
    if (translation.x != 0.0f ||
      translation.y != 0.0f ||
      translation.z != 0.0f)
      Matrix4x4->Translate(-translation.x,
      -translation.y,
      -translation.z); */

// do translation
   
  if (translation.x != 0.0f || translation.y != 0.0f || translation.z != 0.0f)
    Matrix4x4->Translate(translation.x,translation.y,translation.z);
  
  // do temporary offset of origo
  
  if (center.x != 0.0f || center.y != 0.0f || center.z != 0.0f)
    Matrix4x4->Translate(center.x,center.y,center.z);
  
  // do rotation
  
  if (rotation.theta != 0.0f)
    Matrix4x4->Rotate(rotation.theta * 180.0 / M_PI,rotation.x,rotation.y,rotation.z);
  
  if (scaleOrientation.theta != 0.0f) {
    // do scaleRotation
    
   Matrix4x4->Rotate(scaleOrientation.theta * 180.0 / M_PI,scaleOrientation.x,scaleOrientation.y,scaleOrientation.z);
    
    // do scaling
    
  if (scale.x != 1.0f || scale.y != 1.0f || scale.z != 1.0f)
     Matrix4x4->Scale(scale.x,scale.y,scale.z);
    
    // undo scalerotation
    
    Matrix4x4->Rotate(-scaleOrientation.theta * 180.0 / M_PI,scaleOrientation.x,scaleOrientation.y,scaleOrientation.z);
  } else 
    // do scaling
    
  if (scale.x != 1.0f || scale.y != 1.0f || scale.z != 1.0f)
     Matrix4x4->Scale(scale.x,scale.y,scale.z);
    
    // undo temporary offset of origo
    
  if (center.x != 0.0f || center.y != 0.0f || center.z != 0.0f)
     Matrix4x4->Translate(-center.x,-center.y,-center.z);

}