/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**************************
 ***	AgpRgbRange.c	***
 *************************/

#include        <stdio.h>   
#include	<math.h>

#include        "AgpColor.h"

#define         RGB_RANGE( R, G, B ) \
                        { \
				if( R < 0 )	R = 0;	\
				else	if( R > 255 )	R = 255;	\
				if( G < 0 )	G = 0;	\
				else	if( G > 255 )	G = 255;	\
				if( B < 0 )	B = 0;	\
				else	if( B > 255 )	B = 255;	\
			}


#define         RGB_RANGE1( R ) \
                        { \
			if( R < 0 )	R = 0;	\
			else	if( R > 255 )	R = 255; \
			}
			
#define         RGB_RANGE1__1_254( R ) \
                        { \
			if( R < 1 )	R = 1;	\
			else	if( R > 254 )	R = 254; \
			}


static void     agp_rgb_range_edge( edge_type *e, int Fval );
static void     agp_rgb_range_ridge( ridge_type *e, int Fval );
static void     agp_rgb_range_n3( n3_type *e, int Fval );

static void     agp_rgb_range_hunch( hunch_type *e, int Fval );
static void     agp_rgb_range_bgp( bgp_type *e );


static void	color_rgb_range( qcolor_type *c );


void
agp_rgb_range( gp_array *agp, int Fval )
{
gp_type	*gp;
link_type	*l;
hunch_type	*hunch;
bgp_type	*bgp;
 
int	i,	j;

	if( agp->color_type != AGP_RGB )	return;


 
	for( i = 0 ; i < agp->Ni ; i++ )
	    for( j = 0 ; j < agp->Nj ; j++ ){
		gp = &agp->gp[i][j];
 

		for( bgp = gp->bgp ; bgp != NULL ; bgp = bgp->p )
			agp_rgb_range_bgp( bgp );


		for( hunch = gp->hunch ; hunch != NULL ; hunch = hunch->nh )
			agp_rgb_range_hunch( hunch, Fval );


		for( l = gp->link ; l != NULL ; l = l->nl ){

			switch( l->type ){
			case NF_N3:
				agp_rgb_range_n3( l->pr[0].n3, Fval );
				if( LINK_NEXT( l ) == NULL )
				    agp_rgb_range_n3( l->pr[1].n3, Fval );
			    break;

			case NF_EDGE:
				agp_rgb_range_edge( l->pr[0].edge, Fval );
				if( LINK_NEXT( l ) == NULL )
				    agp_rgb_range_edge( l->pr[1].edge, Fval );
			    break;
			case NF_N3R:
			case NF_RIDGE:
				agp_rgb_range_ridge( l->pr[0].ridge, Fval );
				if( LINK_NEXT( l ) == NULL )
				    agp_rgb_range_ridge( l->pr[1].ridge, Fval );
			    break;
			}
		}
	    }
}



static void
agp_rgb_range_edge( edge_type *e, int Fval )
{
	if( Fval ){
		color_rgb_range( e->side[F_RIGHT].val );

		color_rgb_range( e->side[F_LEFT].val );
	}


	color_rgb_range( e->side[F_RIGHT].bval );

	color_rgb_range( e->side[F_LEFT].bval );
}


static void
agp_rgb_range_n3( n3_type *e, int Fval )
{
	if( Fval ){
		color_rgb_range( e->side[F_RIGHT].val );

		color_rgb_range( e->side[F_LEFT].val );
	}

	color_rgb_range( e->mval );

	color_rgb_range( e->side[F_RIGHT].bval );

	color_rgb_range( e->side[F_LEFT].bval );
}


static void
agp_rgb_range_ridge( ridge_type *e, int Fval )
{
float	width2, rval_minus;
qcolor_type	bg;
int	sp;

	if( Fval == 0 )	return;


	width2 = e->width * e->width;

	for( sp = 0 ; sp < SP_NO ; sp++ ){

		rval_minus = e->lm[sp]*width2;
		bg = e->mval[sp] + rval_minus;

#ifdef QFLOAT
		if( e->mval[sp] <= 255 && e->mval[sp] >= 0 &&
			bg <= 254 && bg >= 1  ) continue;

		if( e->mval[sp] < 0 )	e->mval[sp] = 0;
		else	if( e->mval[sp] > 255 )	e->mval[sp] = 255;

		if( bg < 0 )	bg = 0;
		else	if( bg > 255 )	bg = 255;
			else	continue;
#endif

#ifdef QSHORT
		if( e->mval[sp] <= 4080 && e->mval[sp] >= 0 &&
			bg <= 4080-16 && bg >= 16 ) continue;

		if( e->mval[sp] < 0 )	e->mval[sp] = 0;
		else	if( e->mval[sp] > 4080 )	e->mval[sp] = 4080;

		if( bg < 0 )	bg = 0;
		else	if( bg > 4080 )	bg = 4080;
			else	continue;
#endif

		e->rval[sp] = e->mval[sp] - bg;

		e->lm[sp] = - e->rval[sp] / width2;
	}
}



static void
agp_rgb_range_hunch( hunch_type *e, int Fval )
{
float	width2, rval_minus;
qcolor_type	bg;
int	sp;

	if( Fval == 0 )	return;


	width2 = e->g.width * e->g.width;
	
	for( sp = 0 ; sp < SP_NO ; sp++ ){

		rval_minus = e->lm1[sp]*width2;
		bg = e->gval[sp] + rval_minus;

#ifdef QFLOAT
		if( e->gval[sp] <= 255 && e->gval[sp] >= 0 &&
			bg <= 254 && bg >= 1  ) continue;

		if( e->gval[sp] < 0 )	e->gval[sp] = 0;
		else	if( e->gval[sp] > 255 )	e->gval[sp] = 255;

		if( bg < 0 )	bg = 0;
		else	if( bg > 255 )	bg = 255;
#endif
#ifdef QSHORT
		if( e->gval[sp] <= 4080 && e->gval[sp] >= 0 &&
			bg <= 4080-16 && bg >= 16 ) continue;

		if( e->gval[sp] < 0 )	e->gval[sp] = 0;
		else	if( e->gval[sp] > 4080 )	e->gval[sp] = 4080;

		if( bg < 16 )	bg = 0;
		else	if( bg > 4080-16 )	bg = 4080-16;
#endif

		e->rval[sp] = e->gval[sp] - bg;

		e->lm1[sp] = - e->rval[sp] / width2;
	}
}


static void
agp_rgb_range_bgp( bgp_type *e )
{
	color_rgb_range( e->val );
}


static void
color_rgb_range( qcolor_type *c )
{

#ifdef QFLOAT
	if( c[SP_R] < 0 )	c[SP_R] = 0;
	else	if( c[SP_R] > 255 )	c[SP_R] = 255;

	if( c[SP_G] < 0 )	c[SP_G] = 0;
	else	if( c[SP_G] > 255 )	c[SP_G] = 255;

	if( c[SP_B] < 0 )	c[SP_B] = 0;
	else	if( c[SP_B] > 255 )	c[SP_B] = 255;
#endif


#ifdef QSHORT
	if( c[SP_R] < 0 )	c[SP_R] = 0;
	else	if( c[SP_R] > 4080 )	c[SP_R] = 4080;

	if( c[SP_G] < 0 )	c[SP_G] = 0;
	else	if( c[SP_G] > 4080 )	c[SP_G] = 4080;

	if( c[SP_B] < 0 )	c[SP_B] = 0;
	else	if( c[SP_B] > 4080 )	c[SP_B] = 4080;
#endif
}
