/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/***************************
 ***	AgpDepthBase.c   ***
 ***************************/
#include        <stdio.h>   
#include	<math.h>

#include	"Uigp/igp.h"
#include        "Uview/ViewType.h"


#include        "GpType/GpType.h"



#include	"AgpDepth.h"




static void	agp_layer_base_depth_link( vec2d *ctr,
				link_type *link, int direct,
				plane3d *pl );


static void	view_plane_v2pixel( view_type *vw, plane3d *pl );

static float	depth_plane3d_value( vec2d *p, plane3d *pl );




void
agp_depth_base( gp_array *agp )
{
abr_type	*br;
gp_type		*gp;
bgp_type	*bp;
link_type       *link; 
hunch_type      *h; 
vec2d	v;
int	i,	j;
plane3d		plane;
plane3d		Pbr[32];
view_type	*vw;


	vw = agp->ad->vw;


	view_plane_g2v( vw, &agp->ad->N, &agp->ad->P, &plane );
	view_plane_v2pixel( vw, &plane );


	for( i = 0 ; i < agp->ad->br_no ; i++ ){
		br = &agp->ad->abr[i];

		if( br->Fplane == 1 ){
			view_plane_g2v( vw, &br->N, &br->P, &Pbr[br->no] );
			view_plane_v2pixel( vw, &Pbr[br->no] );
		}
		else	Pbr[br->no] = plane;

	}





	for( i = 0 ; i < agp->Ni ; i++ )
		for( j = 0 ; j < agp->Nj ; j++ ){
			gp = &agp->gp[i][j];

			for( bp = gp->bgp ; bp != NULL ; bp = bp->p ){

				v.x = gp->ctr.x + bp->ctr.x;
				v.y = gp->ctr.y + bp->ctr.y;

				bp->depth = depth_plane3d_value( &v,
						&Pbr[bp->br] );
			}


			for( h = gp->hunch ; h != NULL ; h = h->nh){

				v.x = gp->ctr.x + h->ctr.x;
				v.y = gp->ctr.y + h->ctr.y;

				h->depth = depth_plane3d_value( &v,
							&Pbr[h->br] );
			}


			for( link = gp->link ; link != NULL ; link = link->nl){

				v.x = gp->ctr.x + link->ctr.x;
				v.y = gp->ctr.y + link->ctr.y;

				agp_layer_base_depth_link( &v, link, 0, Pbr );


				if( LINK_NEXT(link) != NULL )	continue;

				v.x += link->v.x;
				v.y += link->v.y;

				agp_layer_base_depth_link( &v, link, 1, Pbr );
			}
		}
}




static void
agp_layer_base_depth_link( vec2d *ctr, link_type *link,
					int direct, plane3d Pbr[] )
{
width_type	*width;
vec2d	w;
float	depth;
plane3d	*pl;

	pl = &Pbr[link->br];

	if( link->Fbr == 0x04 ){
		depth = 0.5*( depth_plane3d_value( ctr, pl ) +
			depth_plane3d_value( ctr, &Pbr[link->br1]) );
		link->pr[direct].p->depth = depth;
		return;
	}


	depth = depth_plane3d_value( ctr, pl );
	link->pr[direct].p->depth = depth;



	if( link->type != NF_N3 )	return;



	width = link->width[direct];

	w.x = ctr->x + width->w[F_RIGHT] * width->bes.y;
	w.y = ctr->y - width->w[F_RIGHT] * width->bes.x;
	width->depth[F_RIGHT] = depth_plane3d_value( &w, pl );


	w.x = ctr->x + width->w[F_LEFT] * width->bes.y;
	w.y = ctr->y - width->w[F_LEFT] * width->bes.x;
	width->depth[F_LEFT] = depth_plane3d_value( &w, pl );
}



static void
view_plane_v2pixel( view_type *vw, plane3d *pl )
{
vec3d	n;

	n = pl->n;
	pl->n.x = vw->Va_inv*n.y;
	pl->n.y = vw->Ua_inv* n.x;
	pl->n.z = n.z + vw->Ua_inv *vw->Ub*n.x + vw->Va_inv*vw->Vb*n.y;
}




static float
depth_plane3d_value( vec2d *p, plane3d *pl )
{
float	depth;

	depth = - pl->d/(p->x * pl->n.x + p->y*pl->n.y - pl->n.z );
	return( depth );
}
