/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/*****************************
 ***	AgpDepthSmooth.c   ***
 *****************************/

#include        <stdio.h>   
#include	<math.h>


#include	"AgpDepth.h"

#include	"LinkTool/TlType.h"



#define		MAX_DIS		16


void	agp_link0x04( gp_array *agp, tl_type tlink[], int *no, int *mbr );


static void	agp_smooth_depth_bgp( gp_type *gp, bgp_type *bgp,
					tl_type link[], int no );
static void	agp_smooth_depth_hunch( gp_type *gp, hunch_type *h,
					tl_type link[], int no );

static void	agp_smooth_depth_link( gp_type *gp, link_type *l,
					tl_type link[], int no );

/*
static void	agp_smooth_depth_jnt( gp_type *gp, jnt_type *jnt,
					tl_type link[], int no );
*/

static void	agp_smooth_depth_point( tl_type link[], int no,
					vec2d *v, int br, float *depth );


void
agp_depth_smooth0x4( gp_array *agp )
{
gp_type	*gp;
link_type	*l;
hunch_type	*hunch;
bgp_type	*bgp;
/*
jnt_type	*jnt;
*/

tl_type	link[200];
int	i,	j,	no,	mbr;


	agp_link0x04( agp, link, &no, &mbr );

	if( no == 0 )	return;

 
	for( i = 0 ; i < agp->Ni ; i++ )
	    for( j = 0 ; j < agp->Nj ; j++ ){
		gp = &agp->gp[i][j];
 

		for( bgp = gp->bgp ; bgp != NULL ; bgp = bgp->p ){
			if( !(mbr & (1L<<bgp->br)) )	continue;
			agp_smooth_depth_bgp( gp, bgp, link, no );
		}


		for( hunch = gp->hunch ; hunch != NULL ; hunch = hunch->nh ){
			if( !(mbr& (1L<<hunch->br)) )	continue;
			agp_smooth_depth_hunch( gp, hunch, link, no );
		}


		for( l = gp->link ; l != NULL ; l = l->nl ){
			if( !(mbr& (1L<<l->br)) )	continue;
			agp_smooth_depth_link( gp, l, link, no );
		}


/*
		for( jnt = gp->jnt ; jnt != NULL ; jnt = jnt->nj )
			agp_smooth_depth_jnt( gp, jnt, link, no );
*/
	    }
}


void
agp_link0x04( gp_array *agp, tl_type tlink[], int *no, int *mbr )
{
gp_type	*gp;
tl_type	*tl;
link_type	*l;
int	i,	j;


	*no = 0;
	*mbr = 0;
	for( i = 0 ; i < agp->Ni ; i++ )
		for( j = 0 ; j < agp->Nj ; j++ ){
			gp = &agp->gp[i][j];
 
			for( l = gp->link ; l != NULL ; l = l->nl ){
				if( l->Fbr != 0x04 )	continue;

				tl = &tlink[(*no)];
				(*no)++;

				tl->ctr.x = gp->ctr.x + l->ctr.x;
				tl->ctr.y = gp->ctr.y + l->ctr.y;
				tl->l = l;

				tl->mbr = ( 1L<<l->br) | ( 1L<<l->br1);

				*mbr |= tl->mbr;
			}
		}
}
				




static void
agp_smooth_depth_bgp( gp_type *gp, bgp_type *bgp, tl_type link[], int no )
{
vec2d	v;
	v.x = gp->ctr.x + bgp->ctr.x;
	v.y = gp->ctr.y + bgp->ctr.y;

	agp_smooth_depth_point( link, no, &v, bgp->br, &bgp->depth );
	
}


static void
agp_smooth_depth_hunch( gp_type *gp, hunch_type *hunch,
					tl_type link[], int no )
{
vec2d	v;
	v.x = gp->ctr.x + hunch->ctr.x;
	v.y = gp->ctr.y + hunch->ctr.y;

	agp_smooth_depth_point( link, no, &v, hunch->br, &hunch->depth );
	
}


/*
static void
agp_smooth_depth_jnt( gp_type *gp, jnt_type *jnt, tl_type link[], int no )
{
vec2d	v;
int	br;

	br = jnt->brn[0]->link->br;


	v.x = gp->ctr.x + jnt->ctr.x;
	v.y = gp->ctr.y + jnt->ctr.y;

	agp_smooth_depth_point( link, no, &v, br, &jnt->depth );
}
*/


static void
agp_smooth_depth_link( gp_type *gp, link_type *l, tl_type link[], int no )
{
vec2d	v;
link_type	*l0;
int	br;

	if( l->Fbr == 0x04 )	return;

	br = l->br;


	v.x = gp->ctr.x + l->ctr.x;
	v.y = gp->ctr.y + l->ctr.y;

	agp_smooth_depth_point( link, no, &v, br, &l->pr[0].p->depth );


	if( (l0 = LINK_PREV(l)) != NULL )
		l0->pl[1] = l->pl[0];

	if( LINK_NEXT(l) != NULL )	return;

	v.x += l->v.x;
	v.y += l->v.y;

	agp_smooth_depth_point( link, no, &v, br, &l->pr[1].p->depth );
}




static void
agp_smooth_depth_point( tl_type link[], int no,
			vec2d *v, int br, float *depth )
{
tl_type	*tl;
link_type	*l;
float	t,	u,	tmp;

	tl_array_select( link, no, v, 1L<<br, &tl, &u, &t );
	if( tl == NULL )	return;

	if( u < 0 )	u = -u;
	if( u > MAX_DIS )	return;


	l = tl->l;
	t /= l->len;

	tmp = t*( l->pr[1].p->depth  - l->pr[0].p->depth) + l->pr[0].p->depth;


	t = u / MAX_DIS;

	*depth = t* ( *depth - tmp ) + tmp;
}
