/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**************************
 ***	AgpJnt.c	***
 **************************/
#include	<stdio.h>

#include        "Uigp/igp.h"
#include	"AgpEditTool.h"


#include        "GpType/GpType.h"

void
agp_jnt_insert( gp_array *agp, jnt_type *jnt )
{
gp_type *gp;
int     i,      j;

        i = (jnt->gctr.x+agp->bgp) * agp->inv_bgp;
        j = (jnt->gctr.y+agp->bgp) * agp->inv_bgp;

        gp = &agp->gp[i][j];


        jnt->ctr.x = jnt->gctr.x - gp->ctr.x;
        jnt->ctr.y = jnt->gctr.y -gp->ctr.y;



        jnt->nj = gp->jnt;
        gp->jnt = jnt;
}





void
agp_jnt_untied( gp_array *agp, jnt_type *jnt )
{
gp_type *gp;
jnt_type	*p;
int     i,      j;
 
	i = ( jnt->gctr.x + agp->bgp ) * agp->inv_bgp;
	j = ( jnt->gctr.y + agp->bgp ) * agp->inv_bgp;
 
	gp = &agp->gp[i][j];

	if( gp->jnt == jnt ){
		gp->jnt = jnt->nj;
		return;
	}

	for( p = gp->jnt ; p->nj != NULL ; p = p->nj ){
		if( p->nj == jnt ){
			p->nj = jnt->nj;
			return;
		}
	}
}


void
gp_jnt_untied( gp_type *gp, jnt_type *jnt )
{
jnt_type	*p;
 
	if( gp->jnt == jnt ){
		gp->jnt = jnt->nj;
		return;
	}

	for( p = gp->jnt ; p->nj != NULL ; p = p->nj ){
		if( p->nj == jnt ){
			p->nj = jnt->nj;
			return;
		}
	}
}



void
agp_jnt_untied_brn( gp_array *agp, jnt_type *jnt, int bno )
{
	jnt_untied_brn( jnt, bno, 1 );


	if( jnt->brn_no == 0 ){
		agp_jnt_untied( agp, jnt );
		free( jnt );
	}
}




int
agp_jnt_reconect_link( gp_array *agp, jnt_type *jnt, int bno, jnt_type *njnt )
{
brn_type	*b;
link_type	*link;

	b = jnt->brn[bno];

	link = b->link;


	if( b->direction == F_END ){
		link->v.x += njnt->gctr.x - jnt->gctr.x;
		link->v.y += njnt->gctr.y - jnt->gctr.y;
		link_set_aux( link );
	}
	else	{
		agp_lnf_clear_link_1( &jnt->gctr, link, agp );

		link->v.x += jnt->gctr.x - njnt->gctr.x;
		link->v.y += jnt->gctr.y - njnt->gctr.y;

		link_set_aux( link );

		agp_lnf_add_link_1( &njnt->gctr, link, agp );
	}


	jnt_connet_link( njnt, -1, link, b->direction );



	jnt_untied_brn( jnt, bno, 1 );



	if( jnt->brn_no == 0 ){
		agp_jnt_untied( agp, jnt );
		free( jnt );

		return( 0 );
	}

	return( jnt->brn_no );
}



int
agp_jnt_connect_to_link( gp_array *agp, jnt_type *jnt, vec2d *ctr, link_type *link, float t )
{
jnt_type	*njnt,	*jnt1;
brn_type	*b;

	b = jnt->brn[0];

	njnt = agp_link_to_jnt_t( agp, ctr, link, t, 0.2F );

	if( b->direction == F_END )
			jnt1 = LINK_PREV_JNT( b->link );
	else	jnt1 = LINK_NEXT_JNT( b->link );

	if( njnt == jnt1 ){
		agp_polylink_delete( agp , jnt->brn[0]->link );
		return( 1 );
	}

	if( njnt->brn_no > 3 )	return( 0 );


	agp_jnt_reconect_link( agp, jnt, 0, njnt );


	return( 1 );
}
