/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/

/****************************
 ***	AgpJntConnect.c   ***
 ****************************/
#include        <stdio.h>   
#include	<math.h>

#include        "AgpEditTool.h"
/*
static void agp_jnt_connect_link_inverse( gp_array *agp, link_type *link );
*/


int
agp_jnt_connect( gp_array *agp, jnt_type *jnt )
{
gp_type	*gp;
int	i,	j;
int	ret;

 
	if( jnt->brn_no != 2 )	return( 0 );

        i = (jnt->gctr.x+agp->bgp) * agp->inv_bgp;
        j = (jnt->gctr.y+agp->bgp) * agp->inv_bgp;


        gp = &agp->gp[i][j];


	switch( jnt->type ){
	case JNT_RIDGE:  
		ret = agp_jnt_ridge_connect( agp, gp, jnt);
	    break;

	case JNT_EDGE:
		ret = agp_jnt_edge_connect( agp, gp, jnt );
	    break;

	case JNT_RN3:
		ret = agp_jnt_rn3_connect( agp, gp, jnt);
	    break;
	}

	return( ret );
}





int
agp_jnt_edge_connect( gp_array *agp, gp_type *gp, jnt_type *jnt )
{
jnt_type	*jnt1;
int	bno1;
link_type	*l0,	*l1;

	link_get_jnt_and_brn_no( jnt->brn[0]->link,
			INV_FLAG( jnt->brn[0]->direction), &jnt1, &bno1 );
	if( jnt1 == jnt )	return( 0 );

	if( jnt->brn[0]->direction == jnt->brn[1]->direction )
		agp_link_inverse( agp, jnt->brn[1]->link );

	if( jnt->brn[0]->direction == F_BEGIN ){
		l0 = jnt->brn[1]->link;
		l1 = jnt->brn[0]->link;
	}
	else	{
		l0 = jnt->brn[0]->link;
		l1 = jnt->brn[1]->link;
	}


	l0->p[F_END].bno = -1;
	l0->p[F_END].p.l = l1;

	l1->p[F_BEGIN].bno = -1;
	l1->p[F_BEGIN].p.l = l0;


	free( l0->pr[1].p );
	l0->pr[1] = l1->pr[0];




	jnt->brn[0]->type = BRN_NULL;
	jnt->brn[1]->type = BRN_NULL;
	gp_untied_jnt( gp, jnt );

	free( jnt->brn[0] );
	free( jnt->brn[1] );
	free( jnt );

	return( 1 );
}




int
agp_jnt_ridge_connect( gp_array *agp, gp_type *gp, jnt_type *jnt )
{
jnt_type	*jnt1;
int	bno1;
link_type	*l0,	*l1;

	link_get_jnt_and_brn_no( jnt->brn[0]->link,
			INV_FLAG( jnt->brn[0]->direction), &jnt1, &bno1 );
	if( jnt1 == jnt )	return( 0 );

	if( jnt->brn[0]->direction == jnt->brn[1]->direction )
		agp_link_inverse( agp, jnt->brn[1]->link );

	if( jnt->brn[0]->direction == F_BEGIN ){
		l0 = jnt->brn[1]->link;
		l1 = jnt->brn[0]->link;
	}
	else	{
		l0 = jnt->brn[0]->link;
		l1 = jnt->brn[1]->link;
	}


	l0->p[F_END].bno = -1;
	l0->p[F_END].p.l = l1;

	l1->p[F_BEGIN].bno = -1;
	l1->p[F_BEGIN].p.l = l0;


	free( l0->pr[1].p );
	l0->pr[1] = l1->pr[0];




	jnt->brn[0]->type = BRN_NULL;
	jnt->brn[1]->type = BRN_NULL;
	gp_untied_jnt( gp, jnt );

	free( jnt->brn[0] );
	free( jnt->brn[1] );
	free( jnt );

	return( 1 );
}



int
agp_jnt_rn3_connect( gp_array *agp, gp_type *gp, jnt_type *jnt )
{
jnt_type	*jnt1;
int	bno1;
link_type	*l0,	*l1;
width_type	*w;
float	t;

	link_get_jnt_and_brn_no( jnt->brn[0]->link,
			INV_FLAG( jnt->brn[0]->direction), &jnt1, &bno1 );
	if( jnt1 == jnt )	return( 0 );


	if( jnt->brn[0]->type != jnt->brn[1]->type )
		return( 0 );

	if( jnt->brn[0]->direction == jnt->brn[1]->direction )
		agp_link_inverse( agp, jnt->brn[1]->link );


// new 30-6
	if( jnt->brn[0]->direction == F_END ){
		link_get_jnt_and_brn_no( jnt->brn[0]->link,
				F_BEGIN, &jnt1, &bno1 );

		agp_lnf_clear_link( &jnt1->gctr, jnt1->brn[bno1]->link, agp );
		agp_lnf_clear_link( &jnt->gctr, jnt->brn[1]->link, agp );
	}
	else	{
		link_get_jnt_and_brn_no( jnt->brn[1]->link,
				F_BEGIN, &jnt1, &bno1 );

		agp_lnf_clear_link( &jnt1->gctr, jnt1->brn[bno1]->link, agp );
		agp_lnf_clear_link( &jnt->gctr, jnt->brn[0]->link, agp );
	}



	if( jnt->brn[0]->direction == F_BEGIN ){
		l0 = jnt->brn[1]->link;
		l1 = jnt->brn[0]->link;
	}
	else	{
		l0 = jnt->brn[0]->link;
		l1 = jnt->brn[1]->link;
	}


	l0->p[F_END].bno = -1;
	l0->p[F_END].p.l = l1;

	l1->p[F_BEGIN].bno = -1;
	l1->p[F_BEGIN].p.l = l0;

	if( l0->type == NF_N3 ){
		free( l0->width[1] );
		w = l0->width[1] = l1->width[0];


		w->bes.x = -( l0->u.y + l1->u.y );
		w->bes.y =  ( l0->u.x + l1->u.x );

		t = hypot( w->bes.x, w->bes.y );
		t = 1.0/t;

		w->bes.x *= t;
		w->bes.y *= t;
	}


	free( l0->pr[1].p );
	l0->pr[1] = l1->pr[0];



	link_in_reposition( l0, l1 );




	jnt->brn[0]->type = BRN_NULL;
	jnt->brn[1]->type = BRN_NULL;
	gp_untied_jnt( gp, jnt );

	free( jnt->brn[0] );
	free( jnt->brn[1] );
	free( jnt );


	agp_lnf_link( &jnt1->gctr, jnt1->brn[bno1]->link, agp );


	return( 1 );
}



/*
static void
agp_jnt_connect_link_inverse( gp_array *agp, link_type *link )
{
link_type	*l,	*nl;
jnt_type	*jnt;
int	bno;
vec2d	ctr;

	for( l = link ; LINK_PREV(l) != NULL ; l = LINK_PREV(l) );

	jnt = l->p[0].p.jnt;
	bno = l->p[0].bno;
	ctr.x = jnt->gctr.x + jnt->brn[bno]->ctr.x;
	ctr.y = jnt->gctr.y + jnt->brn[bno]->ctr.y;

	agp_lnf_clear_link( &ctr, l, agp );


	for(  ; l != NULL ; l = LINK_PREV(l) ){
		link_1_inverse( l );
		nl = l;
	}

	jnt = nl->p[0].p.jnt;
	bno = nl->p[0].bno;

	ctr.x = jnt->gctr.x + jnt->brn[bno]->ctr.x;
	ctr.y = jnt->gctr.y + jnt->brn[bno]->ctr.y;
	agp_lnf_link( &ctr, nl, agp );
}
*/