/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/***********************
 ***    AgpJntN3.c   ***
 ***********************/
#include	<stdio.h>
#include	<math.h>

#include	"Uigp/igp.h"

#include	"GpType/GpType.h"


#include	"AgpEditTool.h"





static void	agp_jnt_n3_get_next_in( jnt_type *jnt, int bno,
					jnt_type **njnt, int *nbno );



jnt_type *
agp_jnt_n3_split( gp_array *agp, jnt_type *jnt )
{ 
jnt_type	*njnt;
brn_type	*b; 
vec2d	v;  
int	bno; 
 
	for( bno = 1 ; bno < 3 ; bno++ ){
		b = jnt->brn[bno];
 
		if( b->type == BRN_IN ){
			jnt_n3_in_delete( jnt, bno );
			continue;
		}
 
		if( b->type == BRN_EDGE ){
			v.x = jnt->gctr.x + b->ctr.x;
			v.y = jnt->gctr.y + b->ctr.y;
			njnt = jnt_alloc_1( JNT_EDGE, &v );
			jnt_connet_link( njnt, -1, b->link, b->direction);
			agp_jnt_insert( agp, njnt );
 
			b->type = BRN_NULL; 
			continue; 
		}
	}   
 
	if( jnt->brn[0]->type != BRN_NULL ){ 
		jnt_n3_to_jnt_rn3( jnt ); 
		return( jnt );   
	} 
 
	free( jnt->brn[2] ); 
	free( jnt->brn[1] );
	free( jnt->brn[0] );
 
	jnt->brn_no = 0; 
 
	agp_jnt_untied( agp, jnt ); 
	free( jnt );    
 
	return( NULL ); 
}






void	
agp_jnt_n3_delete_in( jnt_type *jnt, int bno )
{
jnt_type	*njnt;
int	nbno;

	agp_jnt_n3_get_next_in( jnt, bno, &njnt, &nbno );

	jnt_n3_in_delete( jnt, bno );
	jnt_n3_in_delete( njnt, nbno );
}




static void
agp_jnt_n3_get_next_in( jnt_type *jnt, int bno,
					jnt_type **njnt, int *nbno )
{
link_type	*l,	*nl;
in_type	*in;
int	side,	direct;

	if( jnt->brn[0]->direction == F_BEGIN )
		side = ( bno == 1 ) ? F_LEFT : F_RIGHT;
	else	side = ( bno == 1 ) ? F_RIGHT : F_LEFT;

	l = jnt->brn[0]->link;
	while( 1 ){
		if( l->in[side] != NULL )	break;

		nl = ( jnt->brn[0]->direction == F_BEGIN )?
					LINK_NEXT(l) : LINK_PREV(l);
		if( nl == NULL )	break;

		l = nl;
	}

	if( l->in[side] == NULL ){
		direct = INV_FLAG( jnt->brn[0]->direction );
		*njnt = l->p[direct].p.jnt;
		*nbno = 3 - bno;
		return;
	}


	in = l->in[side];
	if( jnt->brn[0]->direction == F_END )
		for( ; in->p != NULL ; in = in->p );

	*njnt = in->jnt;
	*nbno = in->bno;
}



