/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**********************************
 ***    AgpJntN3Reposition.c    ***
 *********************************/

#include	<stdio.h>
#include	<math.h>

#include	"Uigp/igp.h"

#include	"GpType/GpType.h"



static void	agp_jnt_n3_reposition_brn( gp_array *agp,
					jnt_type *jnt, int bno );

void
agp_jnt_n3_reposition( gp_array *agp, jnt_type *jnt )
{
brn_type	*b;
width_type	*width;
vec2d	u;

	b = jnt->brn[0];
	if( b->type == BRN_NULL )	return;

	if( b->direction == F_BEGIN ){

		link_Ntanget_beg( b->link, &u );

		u.x = -u.x;
		u.y = -u.y;
	}
	else	link_Ntanget_end( b->link, &u );



	jnt->u = u;

	if( b->type == BRN_N3 ){
		if( b->direction == F_BEGIN ){
			width = b->link->width[F_BEGIN];
			width->bes.x = -( -jnt->u.y);
			width->bes.y = -( jnt->u.x);
		}
		else	{
			width = b->link->width[F_END];
			width->bes.x = -jnt->u.y;
			width->bes.y =  jnt->u.x;
		}
	}





	agp_jnt_n3_reposition_brn( agp, jnt, 1 );
	agp_jnt_n3_reposition_brn( agp, jnt, 2 );
}

	


static void
agp_jnt_n3_reposition_brn( gp_array *agp, jnt_type *jnt, int bno )
{
brn_type	*b;
vec2d	ctr,	gctr;
float	w;

	b = jnt->brn[bno];

	w = ( bno == 1 )? jnt->rwidth : -jnt->lwidth;

	if( b->type != BRN_EDGE ){
		b->ctr.x = w * jnt->u.x;
		b->ctr.y = w * jnt->u.y;

		if( b->type == BRN_IN ){
			jnt_n3_in_delete( jnt, bno );
			b->type = BRN_IN;
			jnt_n3_in_set( jnt, bno );
		}
		return;
	}


	ctr.x = w * jnt->u.x;
	ctr.y = w * jnt->u.y;

	if( b->ctr.x == ctr.x && b->ctr.y == ctr.y )	return;

	if( b->direction == F_END ){
		b->link->v.x += ctr.x - b->ctr.x;
		b->link->v.y += ctr.y - b->ctr.y;
		b->ctr = ctr;

		link_set_aux( b->link );
		return;
	}

	gctr.x = jnt->gctr.x + b->ctr.x;
	gctr.y = jnt->gctr.y + b->ctr.y;

	agp_lnf_clear_link_1( &gctr, b->link, agp );

	b->link->v.x += b->ctr.x - ctr.x;
	b->link->v.y += b->ctr.y - ctr.y;
	b->ctr = ctr;

	gctr.x = jnt->gctr.x + b->ctr.x;
	gctr.y = jnt->gctr.y + b->ctr.y;

	agp_lnf_add_link_1( &gctr, b->link, agp );

	link_set_aux( b->link );
}
