/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**************************
 ***	AgpLink2Jnt.c	***
 **************************/
#include	<stdio.h>
#include	<math.h>

#include	"Uigp/igp.h"

#include	"GpType/GpType.h"


#include	"AgpEditTool.h"




jnt_type *
agp_link_to_jnt( gp_array *agp, vec2d *ctr, link_type *link )
{
link_type	*plink;
jnt_type	*jnt;
brn_type	*b;

	plink = LINK_PREV( link );
	if( plink == NULL )	return( NULL );



	jnt = jnt_alloc();

	if( plink->type == NF_EDGE )
        	jnt->type = JNT_EDGE;
	else	if( plink->type == NF_N3 ){
        		jnt->type = JNT_RN3;
			jnt->lwidth = -plink->width[1]->w[F_LEFT];
			jnt->rwidth = plink->width[1]->w[F_RIGHT];
		}
		else	if( plink->type == NF_RIDGE )
        			jnt->type = JNT_RIDGE;

	jnt->gctr = *ctr;
        jnt->jnt_no = -1;
        jnt->Fborder = JNT_IN;



	b = jnt->brn[0] = brn_alloc();
	b->type = plink->type;
	b->ctr.x = b->ctr.y = 0;
	b->link = plink;
	b->direction = F_END;
	plink->p[F_END].bno = 0;
	plink->p[F_END].p.jnt = jnt;


	b = jnt->brn[1] = brn_alloc();
	b->type = link->type;
	b->ctr.x = b->ctr.y = 0;
	b->link = link;
	b->direction = F_BEGIN;
	link->p[F_BEGIN].bno = 1;
	link->p[F_BEGIN].p.jnt = jnt;

	jnt->brn_no = 2;


	jnt_set_aux( jnt );

	agp_jnt_insert( agp, jnt );



	plink->pr[1] = profile_make_copy( plink->pr[1], plink->type );

	if( plink->type == NF_N3 )
		plink->width[1] = width_make_copy( plink->width[1] );


	return( jnt );
}



jnt_type *
agp_link_to_jnt_t( gp_array *agp, vec2d *ctr, link_type *link, float t, float d )
{
jnt_type	*jnt;
link_type	*nl;
vec2d	p;

	if( t < 0 )	t = 0;
	if( t > link->len )	t = link->len;

	if( t <= d ){
		if( LINK_PREV(link) != NULL ){
			jnt = agp_link_to_jnt( agp, ctr, link );
			return( jnt );
		}

		jnt = link->p[0].p.jnt;
		if( jnt->type == JNT_EDGE )	return( jnt );

		t = d;
	}


	if( t > link->len - d ){
		if( LINK_NEXT(link) != NULL ){
			p.x = ctr->x + link->v.x;
			p.y = ctr->y + link->v.y;
			jnt = agp_link_to_jnt( agp, &p, LINK_NEXT(link) );
			return( jnt );
		}

		jnt = link->p[1].p.jnt;
		if( jnt->type == JNT_EDGE )	return( jnt );

		t = link->len - d;
	}


	link_t2xy( ctr, link, t, &p );
	agp_link_split( agp, ctr, link, &p );

	nl = LINK_NEXT(link);
	jnt = agp_link_to_jnt( agp, &p, nl );

	return( jnt );
}