/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**************************** 
 ***    AgpLinkDelete.c   ***
 ****************************/
#include	<stdio.h>
#include	<math.h>

#include	"Uigp/igp.h"

#include	"GpType/GpType.h"


#include	"AgpEditTool.h"



/*
static void	agp_link_find_in( link_type *link, int side, link_type *ln[2]);
*/



void
agp_link_delete( gp_array *agp, vec2d *ctr, link_type *link )
{
jnt_type	*jnt0,	*jnt1;
link_type	*l;
vec2d	v;

	if( link->type == NF_N3 || link->type == NF_N3R ){
		agp_link_delete_n3( agp, ctr, link );
		return;
	}

	if( LINK_PREV(link) != NULL )
		jnt0 = agp_link_to_jnt( agp, ctr, link );
	else	jnt0 = link->p[0].p.jnt;


	if( (l = LINK_NEXT(link) ) != NULL ){
		v.x = ctr->x + link->v.x;
		v.y = ctr->y + link->v.y;

		jnt1 = agp_link_to_jnt( agp, &v, l );
	}
	else	jnt1 = link->p[1].p.jnt;



	agp_lnf_clear_link_1( ctr, link, agp );


	jnt_untied_brn( jnt0, link->p[0].bno, 1 );
	if( jnt0->brn_no == 0 ){
		agp_jnt_untied( agp, jnt0 );
		free( jnt0 );
	}

	jnt_untied_brn( jnt1, link->p[1].bno, 1  );
	if( jnt1->brn_no == 0 ){
		agp_jnt_untied( agp, jnt1 );
		free( jnt1 );
	}


	link_1_free( link );
}


void
agp_link_delete_n3( gp_array *agp, vec2d *ctr, link_type *link )
{
jnt_type	*jnt0,	*jnt1,	*jnt;
link_type	*l;
link_type	*rln[2],	*lln[2];
in_type	*in;
vec2d	v;


	link_in_get( link, F_RIGHT, rln );
	link_in_get( link, F_LEFT, lln );


	while( (in = link->in[F_RIGHT]) != NULL ){
		link->in[F_RIGHT] = in->p;
		jnt = in->jnt;
		jnt->brn[in->bno]->type = BRN_NULL;
		jnt->brn[in->bno]->in = NULL;
		if( jnt->brn[1]->type == BRN_NULL &&
				jnt->brn[2]->type == BRN_NULL )
			jnt_n3_to_jnt_rn3( jnt );

		free( in );
	}

	while( (in = link->in[F_LEFT]) != NULL ){
		link->in[F_LEFT] = in->p;
		jnt = in->jnt;
		jnt->brn[in->bno]->type = BRN_NULL;
		jnt->brn[in->bno]->in = NULL;
		if( jnt->brn[1]->type == BRN_NULL &&
				jnt->brn[2]->type == BRN_NULL )
			jnt_n3_to_jnt_rn3( jnt );

		free( in );
	}





	if( ( l = LINK_PREV(link)) != NULL ){
		l->pr[1] = profile_make_copy( l->pr[1], l->type );
		if( l->width[1] != NULL )
			l->width[1] = width_make_copy( l->width[1] );

		jnt0 = jnt_alloc_1( JNT_N3, ctr );
		jnt_connet_link( jnt0, 0, l, F_END );
		jnt_n3_load_from_link( jnt0 );
		agp_jnt_insert( agp, jnt0 );


		if( rln[0] != NULL ){
			jnt0->brn[1]->type = BRN_IN;
			jnt0->brn[1]->link = rln[0];
			jnt_n3_set_in_direct( jnt0, 1 );
		}

		if( lln[0] != NULL ){
			jnt0->brn[2]->type = BRN_IN;
			jnt0->brn[2]->link = lln[0];
			jnt_n3_set_in_direct( jnt0, 2 );
		}

		if( jnt0->brn[1]->type == BRN_NULL &&
				jnt0->brn[2]->type == BRN_NULL )
			jnt_n3_to_jnt_rn3( jnt0 );

		jnt_set_aux( jnt0 );

	}
	else	{
		jnt0 = link->p[0].p.jnt;
		if( jnt0->type == JNT_N3 )
			agp_jnt_n3_split( agp, jnt0 );

		jnt_untied_brn( jnt0, link->p[0].bno, 1  );
		if( jnt0->brn_no == 0 ){
			agp_jnt_untied( agp, jnt0 );
			free( jnt0 );
			jnt0 = NULL;
		}
	}
			


	if( (l = LINK_NEXT(link) ) != NULL ){
		v.x = ctr->x + link->v.x;
		v.y = ctr->y + link->v.y;

		jnt1 = jnt_alloc_1( JNT_N3, &v );
		jnt_connet_link( jnt1, 0, l, F_BEGIN );
		jnt_n3_load_from_link( jnt1 );
		agp_jnt_insert( agp, jnt1 );

		if( rln[1] != NULL ){
			jnt1->brn[1]->type = BRN_IN;
			jnt1->brn[1]->link = rln[1];
			jnt_n3_set_in_direct( jnt1, 1 );
		}

		if( lln[1] != NULL ){
			jnt1->brn[2]->type = BRN_IN;
			jnt1->brn[2]->link = lln[1];
			jnt_n3_set_in_direct( jnt1, 2 );
		}

		if( jnt1->brn[1]->type == BRN_NULL &&
				jnt1->brn[2]->type == BRN_NULL )
			jnt_n3_to_jnt_rn3( jnt1 );

		jnt_set_aux( jnt1 );
	}
	else	{
		jnt1 = link->p[1].p.jnt;
		if( jnt1->type == JNT_N3 )
			agp_jnt_n3_split( agp, jnt1 );

		jnt_untied_brn( jnt1, link->p[1].bno, 1 );
		if( jnt1->brn_no == 0 ){
			agp_jnt_untied( agp, jnt1 );
			free( jnt1 );
			jnt1 = NULL;
		}
	}



	agp_lnf_clear_link_1( ctr, link, agp );


	link_1_free( link );
}




/*
static void
agp_link_find_in( link_type *link, int side, link_type *ln[2] ) 
{ 
jnt_type	*jnt;
link_type	*l,	*pl;
in_type	*in;
int	bno;



        ln[0] = ln[1] = NULL;


	pl = link;
	while( ( l = LINK_PREV(pl)) != NULL ){
		if( l->in[side] != NULL )	break;
		pl = l;
	}


	if( l != NULL ){
                for( in = l->in[side] ; in->p != NULL ; in = in->p ); 
		if( in->direct == F_END )
			ln[0] = in->jnt->brn[0]->link;
	}
	else	{
               	jnt = pl->p[0].p.jnt; 
               	bno = (side == F_RIGHT)? 2: 1; 
               	if( jnt->type == JNT_N3 && jnt->brn[bno]->type == BRN_IN ) 
                       	ln[0] = jnt->brn[bno]->link;
	}




	if( link->in[side] == NULL ){
		ln[1] = ln[0];
		return;
	}

	for( in = link->in[side] ; in->p != NULL ; in = in->p ); 

	if( in->direct == F_END )
		ln[1] = in->jnt->brn[0]->link;
}
*/
