/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/********************************
 ***	AgpPolylinkDelete.c   ***
 ********************************/

#include        <stdio.h>
#include        <math.h>

#include        "Uigp/igp.h"

#include        "GpType/GpType.h"

#include        "AgpEdit/AgpEditTool.h"





static int	agp_polylink_n3_delete( gp_array *agp, link_type *link );




int	
agp_polylink_delete( gp_array *agp, link_type *link )
{
jnt_type	*jnt,	*njnt;
int	bno,	nbno;
link_type	*l;
vec2d	ctr;

	while( LINK_PREV(link) != NULL )	link =  LINK_PREV(link);


	if( link->type == NF_N3 || link->type == NF_N3R )
		return( agp_polylink_n3_delete( agp, link ) );


	jnt = link->p[0].p.jnt;
	bno = link->p[0].bno;

	for( l = link ; LINK_NEXT(l) != NULL ; l = LINK_NEXT(l) );

	njnt = l->p[1].p.jnt;
	nbno = l->p[1].bno;



	ctr.x = jnt->gctr.x + jnt->brn[bno]->ctr.x;
	ctr.y = jnt->gctr.y + jnt->brn[bno]->ctr.y;

	agp_lnf_clear_link( &ctr, link, agp );

	agp_jnt_untied_brn( agp, jnt, bno );
	nbno = l->p[1].bno;
	agp_jnt_untied_brn( agp, njnt, nbno );


	link_free( link );


	return( 1 );
}



static int
agp_polylink_n3_delete( gp_array *agp, link_type *link )
{
jnt_type	*jnt,	*njnt;
int	bno,	nbno;
link_type	*l;
in_type	*in;
vec2d	ctr;


	jnt = link->p[0].p.jnt;
	bno = link->p[0].bno;

	for( l = link ; LINK_NEXT(l) != NULL ; l = LINK_NEXT(l) );

	njnt = l->p[1].p.jnt;
	nbno = l->p[1].bno;



/*
	if( jnt->type == JNT_N3 && jnt->brn[1]->type == BRN_IN )
		jnt_n3_in_delete_segment( jnt, 1 );
	if( jnt->type == JNT_N3 && jnt->brn[2]->type == BRN_IN )
		jnt_n3_in_delete_segment( jnt, 2 );
*/
	if( jnt->type == JNT_N3 )
		agp_jnt_n3_split( agp, jnt );


/*
	if( njnt->type == JNT_N3 && njnt->brn[1]->type == BRN_IN )
		jnt_n3_in_delete_segment( njnt, 1 );
	if( njnt->type == JNT_N3 && njnt->brn[2]->type == BRN_IN )
		jnt_n3_in_delete_segment( njnt, 2 );
*/
	if( njnt->type == JNT_N3 )
		agp_jnt_n3_split( agp, njnt );


	for( l = link ; l != NULL ; l = LINK_NEXT(l) ){
		while( (in = l->in[F_RIGHT]) != NULL ){
			l->in[F_RIGHT] = in->p;

			in->jnt->brn[in->bno]->type = BRN_NULL;
			in->jnt->brn[in->bno]->in = NULL;
			if( in->jnt->brn[1]->type == BRN_NULL &&
					in->jnt->brn[2]->type == BRN_NULL )
				jnt_n3_to_jnt_rn3( in->jnt );

			free( in );
		}

		while( (in = l->in[F_LEFT]) != NULL ){
			l->in[F_LEFT] = in->p;

			in->jnt->brn[in->bno]->type = BRN_NULL;
			in->jnt->brn[in->bno]->in = NULL;
			if( in->jnt->brn[1]->type == BRN_NULL &&
					in->jnt->brn[2]->type == BRN_NULL )
				jnt_n3_to_jnt_rn3( in->jnt );

			free( in );
		}
        }





	ctr.x = jnt->gctr.x + jnt->brn[bno]->ctr.x;
	ctr.y = jnt->gctr.y + jnt->brn[bno]->ctr.y;

	agp_lnf_clear_link( &ctr, link, agp );



	agp_jnt_untied_brn( agp, jnt, bno );
	agp_jnt_untied_brn( agp, njnt, nbno );

	link_free( link );


	return( 1 );
}
