/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/***********************
 ***   AgpLayer.c    ***
 ***********************/

#include	<stdio.h>
#include	<math.h>

#include	"Uigp/igp.h"

#include	"GpType/GpType.h"

#include	"AgpLayer.h"


void
agp_layer_changh_no( gp_array *agp, int br, int nbr )
{
adepth_type	*ad;
gp_type	*gp;
hunch_type	*h;
bgp_type	*bp;
link_type	*link;
int	i,	j;


	ad = agp->ad;
	for( i = 0 ; i < ad->br_no ; i++ )
		if( ad->abr[i].no == br )
			ad->abr[i].no = nbr;


	ad->Mbr &= ~( 1<<br);
	ad->Mbr |= ( 1<<nbr);


	for( i = 0 ; i < agp->layer_no ; i++ ){
		if( agp->layer[i]->Mbr & (1<<br) ){
			agp->layer[i]->Mbr &= ~( 1<<br);
			agp->layer[i]->Mbr |= ( 1<<nbr);
		}
	}


	for( i = 0 ; i < agp->Ni ; i++ )
		for( j = 0 ; j < agp->Nj ; j++ ){

			gp = &agp->gp[i][j];

			for( bp = gp->bgp ; bp != NULL ; bp = bp->p ){
				if( bp->br != br )	continue;
				bp->br = nbr;
			}


			for( h = gp->hunch ; h != NULL ; h = h->nh ){
				if( h->br != br )	continue;
				h->br = nbr;
			}



			for( link = gp->link ; link != NULL ; link = link->nl ){
				if( link->br != br )	continue;
				link->br = nbr;
			}
		}
}


void
agp_layer_add_const( gp_array *agp, int br0 )
{
adepth_type	*ad;
gp_type	*gp;
hunch_type	*h;
bgp_type	*bp;
link_type	*link;
int	i,	j;

	if( br0 == 0 )	return;

	ad = agp->ad;
	for( i = 0 ; i < ad->br_no ; i++ )
		ad->abr[i].no += br0;

	if( br0 > 0 )
		ad->Mbr <<= br0;
	else	ad->Mbr >>= -br0;


	for( i = 0 ; i < agp->layer_no ; i++ )
		if( br0 > 0 )
			agp->layer[i]->Mbr <<= br0;
		else	agp->layer[i]->Mbr >>= -br0;


	for( i = 0 ; i < agp->Ni ; i++ )
		for( j = 0 ; j < agp->Nj ; j++ ){

			gp = &agp->gp[i][j];

			for( bp = gp->bgp ; bp != NULL ; bp = bp->p ){
				bp->br += br0;
			}


			for( h = gp->hunch ; h != NULL ; h = h->nh ){
				h->br += br0;
			}



			for( link = gp->link ; link != NULL ; link = link->nl ){
				link->br += br0;
				link->br1 += br0;
			}
		}
}


int
agp_layer_no( gp_array *agp, int *layer0, int *layer1 )
{
int	mbr,	i,	no = 0;
int	br0,	br1;
 
	if( agp->ad == NULL )   return(1);

	br0 = br1 = -1;
 
	mbr = agp->ad->Mbr;
 
	for( i = 0 ; mbr != 0 ; i++, mbr >>= 1 ){
		if( !( mbr & 1 ) )	continue;
 
		no++;

		if( br0 == -1 ){
			br0 = br1 = i;
			continue;
		}
 
		br1 = i;
	}

	if( layer0 != NULL )	*layer0 = br0;
	if( layer1 != NULL )	*layer1 = br1;

	return( no );
} 



int
agp_layer_id( gp_array *agp, int iLayer )
{
int	mbr,	id;

	for( mbr = agp->layer[iLayer]->Mbr, id = 0 ;
					(mbr&0x01) == 0 && mbr != 0 ; mbr >>= 1, id++ );

	return( id );
}


int
agp_layer_index( gp_array *agp, int layerId )
{
int	i;

	for( i = 0 ; i < agp->layer_no ; i++ )
		if( agp->layer[i]->Mbr & ( 1<<layerId) )
			return( i );

	return( -1 );
}
