/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**************************
 ***	agpLayerColor.c	***
 **************************/

#include        <stdio.h>   
#include		<math.h>

#include        "AgpTool/AgpTool.h"

#define		RGB_SET( RY, GCr,BCb, color ) \
			{ \
				RY = IMAGE4_RED(color);; \
				GCr = IMAGE4_GREEN(color); \
				BCb = IMAGE4_BLUE(color); \
			}

static void	agp_Rgb_edge( edge_type *e, int Color );
static void	agp_Rgb_ridge( ridge_type *e, int Color );
static void	agp_Rgb_n3( n3_type *e, int Color );
static void	agp_Rgb_bgp( bgp_type *e, int Color );

static void	color_set( qcolor_type c[SP_NO], int color );



void agp_layer_color( gp_array *agp ,int color ,int mbr)
{
gp_type	*gp;
link_type	*link;
bgp_type *bg;

 
int	i,	j;

	

	for( i = 0 ; i < agp->Ni ; i++ )
	    for( j = 0 ; j < agp->Nj ; j++ ){
		gp = &agp->gp[i][j];
 		
		for( bg = gp->bgp ; bg != NULL ; bg = bg->p ){
			if( !(mbr & (1<<bg->br)) )
					continue;

			agp_Rgb_bgp(bg, color);
		}


		for( link = gp->link ; link != NULL ; link = link->nl ){
			if( !(mbr & (1<<link->br)) )	
				continue;

			switch( link->type ){
			case NF_N3:
				agp_Rgb_n3( link->pr[0].n3, color );
				if( LINK_NEXT( link ) == NULL )
					agp_Rgb_n3( link->pr[1].n3, color );
			    break;

			case NF_EDGE:
				agp_Rgb_edge( link->pr[0].edge, color );
				if( LINK_NEXT( link ) == NULL )
					agp_Rgb_edge( link->pr[1].edge, color );
			    break;
			case NF_N3R:
			case NF_RIDGE:
				agp_Rgb_ridge( link->pr[0].ridge, color );
				if( LINK_NEXT( link ) == NULL )
					agp_Rgb_ridge( link->pr[1].ridge, color);
			    break;
			}
		}
	    }
}



static void
agp_Rgb_edge( edge_type *e, int Color )
{
	color_set( e->side[F_RIGHT].val, Color );


	color_set( e->side[F_RIGHT].bval, Color );

}


static void
agp_Rgb_n3( n3_type *e, int Color )
{
	color_set( e->side[F_RIGHT].val, Color );

	color_set( e->mval, Color );

	color_set( e->side[F_RIGHT].bval, Color );
}


static void
agp_Rgb_ridge( ridge_type *e, int Color )
{
	color_set( e->mval, Color );

	color_set( e->rval, 0 );

	color_set( e->lm, 0 );
}


static void
agp_Rgb_bgp( bgp_type *e, int Color )
{
	color_set( e->val, Color );
}


static void
color_set( qcolor_type c[SP_NO], int color )
{
int	r,	g,	b;

	r = IMAGE4_RED(color);
	g = IMAGE4_GREEN(color);
	b = IMAGE4_BLUE(color);

	c[SP_R] = F2QCOLOR( r );
	c[SP_G] = F2QCOLOR( g );
	c[SP_B] = F2QCOLOR( b );
}
