/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/****************************
 ***	AgpLayerDepth.c   ***
 ****************************/
#include	<stdio.h>   
#include	<math.h>

#include	"AgpLayer.h"


static void	agp_layer_depth_gp( gp_type *gp, int br, view_type *Vw, plane3d Pbr[] );

static void	agp_layer_depth_link( vec2d *ctr, link_type *l,
									view_type *Vw, plane3d *plane );

static void	agp_layer_depth_link0x04( vec2d *ctr, link_type *l,
									view_type *Vw, plane3d *plane, plane3d *plane1 );

static void	agp_layer_depth_link_n3( vec2d *ctr, link_type *l,
									view_type *Vw, plane3d *plane );

static float    depth_plane_value( view_type *vw, plane3d *pl, vec2d *p );







void
agp_layer_depth( gp_array *agp, int br )
{
plane3d		Pbr[256];
view_type	*Vw;
abr_type	*abr;
int	i,	j;


	if( agp->ad == NULL )	return;




    Vw = agp->ad->vw;



    for( i = 0 ; i < agp->ad->br_no ; i++ ){
		abr = &agp->ad->abr[i];
		if( abr->Fplane == 0 ){
            if( abr->no == 0 ){
                Pbr[0].n = agp->ad->N;
                Pbr[0].d = VEC3D_INNER( agp->ad->N, agp->ad->P);
            }
            continue;
        }

        Pbr[abr->no].n = abr->N;
        Pbr[abr->no].d = VEC3D_INNER( abr->N, abr->P );
    }


	for( i = 0 ; i < agp->Ni ; i++ )
		for( j = 0 ; j < agp->Nj ; j++ )
			agp_layer_depth_gp( &agp->gp[i][j], br, Vw, Pbr );
}


static void
agp_layer_depth_gp( gp_type *gp, int br, view_type *Vw, plane3d Pbr[] )
{
bgp_type        *bp;
hunch_type      *h;
link_type       *l;

vec2d	p;


	for( bp = gp->bgp ; bp != NULL ; bp = bp->p ){
		if( br != -1 && br != bp->br )	continue;
		p.x = gp->ctr.x + bp->ctr.x;
		p.y = gp->ctr.y + bp->ctr.y;
		bp->depth = depth_plane_value( Vw, &Pbr[bp->br], &p );
	}


	for( h = gp->hunch ; h != NULL ; h = h->nh ){
		if( br != -1 && br != h->br )	continue;
		p.x = gp->ctr.x + h->ctr.x;
		p.y = gp->ctr.y + h->ctr.y;
		h->depth = depth_plane_value( Vw, &Pbr[h->br], &p );
	}

 
	for( l = gp->link ; l != NULL ; l = l->nl ){
		if( br != -1 && br != l->br && br != l->br1 )	continue;

		switch( l->type ){
		case NF_RIDGE:
		case NF_N3R:
			agp_layer_depth_link( &gp->ctr, l, Vw, &Pbr[l->br] );
		    break;

		case NF_EDGE:
			if( l->Fbr == 0x04 )
				agp_layer_depth_link0x04( &gp->ctr, l, Vw, &Pbr[l->br], &Pbr[l->br1] );
			else	agp_layer_depth_link( &gp->ctr, l, Vw, &Pbr[l->br] );
		    break;

		case NF_N3:
			agp_layer_depth_link_n3( &gp->ctr, l, Vw, &Pbr[l->br] );
		    break;
		}
	}
}


 
static void
agp_layer_depth_link( vec2d *ctr, link_type *l, view_type *Vw, plane3d *plane )
{
vec2d	p,	p1;

	p.x = ctr->x + l->ctr.x;
	p.y = ctr->y + l->ctr.y;
	l->pr[0].p->depth = depth_plane_value( Vw, plane, &p );
 
	if( LINK_NEXT(l) != NULL )	return;

	p1.x = p.x + l->v.x;
	p1.y = p.y + l->v.y;
	l->pr[1].p->depth = depth_plane_value( Vw, plane, &p1 );
}


static void
agp_layer_depth_link0x04( vec2d *ctr, link_type *l,
						 view_type *Vw, plane3d *plane, plane3d *plane1 )
{
vec2d	p,	p1;

	p.x = ctr->x + l->ctr.x;
	p.y = ctr->y + l->ctr.y;
	l->pr[0].p->depth = 0.5*(depth_plane_value( Vw, plane, &p ) +
				depth_plane_value( Vw, plane1, &p ) );
 
	if( LINK_NEXT(l) != NULL )	return;

	p1.x = p.x + l->v.x;
	p1.y = p.y + l->v.y;
	l->pr[1].p->depth = 0.5*(depth_plane_value( Vw, plane, &p ) +
				depth_plane_value( Vw, plane1, &p ) );
}





static void
agp_layer_depth_link_n3( vec2d *ctr, link_type *l, view_type *Vw, plane3d *plane )
{
vec2d   v,      w;
profile_type    pr;
width_type      *width;

	v.x = ctr->x + l->ctr.x;
	v.y = ctr->y + l->ctr.y;

	pr = l->pr[0];

	pr.p->depth = depth_plane_value( Vw, plane, &v );


	width = l->width[F_BEGIN];
	w.x = v.x + width->w[F_RIGHT] * width->bes.y;
	w.y = v.y - width->w[F_RIGHT] * width->bes.x;
	width->depth[F_RIGHT] = depth_plane_value( Vw, plane, &w );


	w.x = v.x + width->w[F_LEFT] * width->bes.y;
	w.y = v.y - width->w[F_LEFT] * width->bes.x;
	width->depth[F_LEFT] = depth_plane_value( Vw, plane, &w );



	if( LINK_NEXT(l) != NULL )   return;

	v.x += l->v.x;
	v.y += l->v.y;


	pr = l->pr[1];

	pr.p->depth = depth_plane_value( Vw, plane, &v );

	width = l->width[F_END];
	w.x = v.x + width->w[F_RIGHT] * width->bes.y;
	w.y = v.y - width->w[F_RIGHT] * width->bes.x;
	width->depth[F_RIGHT] = depth_plane_value( Vw, plane, &w );
 
	w.x = v.x + width->w[F_LEFT] * width->bes.y;
	w.y = v.y - width->w[F_LEFT] * width->bes.x;
	width->depth[F_LEFT] = depth_plane_value( Vw, plane, &w );
}









static float
depth_plane_value( view_type *vw, plane3d *pl, vec2d *p )
{
vec2d   v;
float   t,      value;
vec3d   n,      p3;

        v.x = vw->Ua_inv*(p->y - vw->Ub);
        v.y = vw->Va_inv*(p->x - vw->Vb);
 
 
        view_3d_ray( vw, v.x, v.y, &p3, &n );
        t = 1.0 / VEC3D_NORM( n );
        n.x *= t;
        n.y *= t;
        n.z *= t;
 
        t = ( pl->d - VEC3D_INNER( vw->cp, pl->n ) ) / VEC3D_INNER( n, pl->n );
 
        value = t* VEC3D_INNER( n, vw->N );
 
        return( value );
}
