/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**************************
 ***	AgpLayerInit.c	***
 *************************/
#include	<stdio.h>   
#include	<math.h>
#include	"Uigp/igp.h"


#include	"GpType/GpType.h"
#include	"AgpLayer.h"


static void	agp_layer_init_hunch( vec2d *v, hunch_type *h );

static void	agp_layer_init_link( vec2d *ctr, link_type *link );

static void	agp_layer_init_link_n3( vec2d *ctr, link_type *link );

static void	agp_layer_init_bgp( vec2d *ctr, bgp_type *bgp );



extern void	adepth_set_planeD( adepth_type *ad, int layer, vec3d *N, float depth );


void
agp_layer_init( gp_array *agp )
{
layer_type	*layer;
gp_type		*gp;
bgp_type	*bp;
link_type	*link;
hunch_type	*h;
vec2d	v;
int	i,	j;


	for( i = 0 ; i < agp->Ni ; i++ )
		for( j = 0 ; j < agp->Nj ; j++ ){
			gp = &agp->gp[i][j];

			for( bp = gp->bgp ; bp != NULL ; bp = bp->p )
				agp_layer_init_bgp( &gp->ctr, bp );


			for( h = gp->hunch ; h != NULL ; h = h->nh){
				v.x = gp->ctr.x + h->ctr.x;
				v.y = gp->ctr.y + h->ctr.y;

				agp_layer_init_hunch( &v, h );
			}


			for( link = gp->link ; link != NULL ; link = link->nl )
				agp_layer_init_link( &gp->ctr, link );

		}

	if( agp->ad == NULL ){
		agp->ad = adepth_default( 100, agp->row, agp->column );
		agp->type = AGP_DEPTH;
	}

	agp->ad->br_no = 0;

	
	adepth_set_planeD( agp->ad, 0, &agp->ad->N, 100 );

	agp_layer_depth( agp, 0 );




	for( i = 0 ; i < agp->layer_no ; i++ ){
		free( agp->layer[i] );
		agp->layer[i] = NULL;
	}
	agp->layer_no = 0;


	agp->layer_no = 0;
	layer = agp->layer[agp->layer_no++] = LAYER_ALLOC();
	layer->Mbr = (1<<0);
	layer->i0 = 1;
	layer->j0 = 1;
	layer->i1 = agp->Ni-1;
	layer->j1 = agp->Nj-1;
	layer->Fdepth_delta = 0;
}







static void
agp_layer_init_bgp( vec2d *v, bgp_type *bp )
{
	bp->depth = 0;
	bp->br = 0;
}



static void
agp_layer_init_hunch( vec2d *vm, hunch_type *h )
{
	h->depth = 0;
	h->br = 0;
}




static void
agp_layer_init_link( vec2d *ctr, link_type *link )
{
/*
vec2d	v;
*/

	if( link->type == NF_N3 ){
		agp_layer_init_link_n3( ctr, link );
		return;
	}


/*
	v.x = ctr->x + link->ctr.x;
	v.y = ctr->y + link->ctr.y;
*/

	link->br = 0;
	link->Fbr = 0;

	link->pr[0].p->depth = 0;

/*
	v.x += link->v.x;
	v.y += link->v.y;
*/

	if( LINK_NEXT(link) != NULL )	return;

	link->pr[1].p->depth = 0;
}



static void
agp_layer_init_link_n3( vec2d *ctr, link_type *link )
{
/*
vec2d	v,	w;
*/
width_type	*width;

/*
	v.x = ctr->x + link->ctr.x;
	v.y = ctr->y + link->ctr.y;
*/

	link->br = 0;
	link->Fbr = 0;

	link->pr[0].p->depth = 0;

	width = link->width[F_BEGIN];
/*
	w.x = v.x + width->w[F_RIGHT] * width->bes.y;
	w.y = v.y - width->w[F_RIGHT] * width->bes.x;
*/
	width->depth[F_RIGHT] = 0;

/*
	w.x = v.x - width->w[F_LEFT] * width->bes.y;
	w.y = v.y + width->w[F_LEFT] * width->bes.x;
*/
	width->depth[F_LEFT] = 0;
		


	if( LINK_NEXT(link) != NULL )	return;

/*
	v.x += link->v.x;
	v.y += link->v.y;
*/


	link->pr[1].p->depth = 0;

	width = link->width[F_END];
/*
	w.x = v.x + width->w[F_RIGHT] * width->bes.y;
	w.y = v.y - width->w[F_RIGHT] * width->bes.x;
*/
	width->depth[F_RIGHT] = 0;

/*
	w.x = v.x - width->w[F_LEFT] * width->bes.y;
	w.y = v.y + width->w[F_LEFT] * width->bes.x;
*/
	width->depth[F_LEFT] = 0;
}
