/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/*************************** 
 ***    AgpAddBorder.c   ***
 ***************************/
#include	<stdio.h>
#include	<math.h>

#include	"Uigp/igp.h"

#include	"GpType/GpType.h"

#include	"AgpEdit/AgpEditTool.h"

#include	"AgpTool/AgpTool.h"
#include	"LinkTool/LinkTool.h"

#define X 1 /* moving in x or -x direction*/
#define Y 2 /* moving in y or -y direction*/
#define RELEVANT_DIST 0.10F
#define RELEVANT_CTR(ctr, type, coord) \
      (((type == X)  && ((float)fabs(ctr.y - coord) < 1)) || \
	   ((type == -X) && ((float)fabs(ctr.y - coord) < 1)) || \
	   ((type == Y)  && ((float)fabs(ctr.x - coord) < 1)) || \
	   ((type == -Y) && ((float)fabs(ctr.x - coord) < 1)))
#define RELEVANT_JNT(jnt) ( jnt->bno == 1 )
#define GREATE_FOR_LINK(ctr1, ctr2, type) \
      (((type == X)  && (ctr1.x > ctr2.x)) || \
	   ((type == -X) && (ctr1.x < ctr2.x)) || \
	   ((type == Y)  && (ctr1.y > ctr2.y)) || \
	   ((type == -Y) && (ctr1.y < ctr2.y))) 
#define SET_SHIFT(jnt, ctr, type, coord, shift_v, v) \
	 shift_v.x = (type == Y || type == -Y) ? \
	 (coord - ctr.x) : ((v.y != 0) ? (v.x*(coord - ctr.y) / v.y) : 0); \
	  shift_v.y = (type == X || type == -X) ? \
(coord - ctr.y) : ((v.x != 0) ? (v.y*(coord - ctr.x) / v.x) : 0); 

/* static functions */
static jnt_type* agp_border_gp_create_border
(jnt_type* prev_gp_jnt, gp_type *gp, float coord, char direct, gp_array *agp, int br);
static jnt_type *
agp_border_create_jnt( gp_array *agp, gp_type *gp, vec2d *ctr);
static void agp_border_link_profile( link_type *link, int direct, bgp_type *bgp );
static void agp_border_default_profile
( link_type *link, int direct, qcolor_type val[], float depth );
static bgp_type	*agp_border_nearest_bgp1( gp_array *agp, vec2d *gctr);
static bgp_type	* agp_border_get_nearest_bgp( gp_array *agp, vec2d *gctr, int rad );
static int agp_border_new_layer( gp_array *agp );
static void agp_border_links_between_jnts(jnt_type* jnt1, jnt_type* jnt2, gp_array* agp, int br);
static void agp_border_links_aux( link_type *first_link, link_type* last_link, gp_array* agp, int br);
static void agp_border_profile_by_jnt(link_type *link, int direct);
static void agp_border_gp_clear( gp_array *agp, gp_type *gp, float maxX, float maxY );

void agp_border_add( gp_array *agp )
{
	float coord = agp->bgp / 2.0F - 1.8F;
	int j, br = 0;
	gp_type* gp;
	jnt_type *jnt0, *jnt1, *jnt2, *jnt3, *cur_jnt;
	vec2d ctr;

	if ( !agp || agp->layer_no != 1 || agp->ad == NULL || 
		 agp->Ni < 3 || agp->Nj < 3 )
		return; 

	if ( (br = agp_border_new_layer( agp )) == -1 )
		return; 

	ctr.x = ctr.y = coord;
	jnt0 = agp_border_create_jnt(agp, &agp->gp[0][0], &ctr);
	ctr.x = -coord; ctr.y = coord;
	jnt1 = agp_border_create_jnt(agp, &agp->gp[agp->Ni-1][0], &ctr);
	ctr.x = -coord; ctr.y = -coord;
	jnt2 = agp_border_create_jnt(agp, &agp->gp[agp->Ni-1][agp->Nj-1], &ctr);
	ctr.x = coord; ctr.y = -coord;
	jnt3 = agp_border_create_jnt(agp, &agp->gp[0][agp->Nj-1], &ctr);
	cur_jnt = jnt0;
	for ( j = 0; j < agp->Ni-1; j++ )
		if ( ((gp = &agp->gp[j][0]) != NULL) && ( gp->jnt != NULL) ) {
			cur_jnt = agp_border_gp_create_border(cur_jnt, gp, coord, X, agp, br);
		}	

	agp_border_links_between_jnts(cur_jnt, jnt1, agp, br);

	cur_jnt = jnt1;
	for ( j = 0; j < agp->Nj-1; j++ )
		if ( ((gp = &agp->gp[agp->Ni-1][j]) != NULL) && ( gp->jnt != NULL) )
			cur_jnt = agp_border_gp_create_border(cur_jnt, gp, -coord, Y, agp, br);
	agp_border_links_between_jnts(cur_jnt, jnt2, agp, br);

	cur_jnt = jnt2;
	for ( j = agp->Ni-1; j >= 0; j-- )
		if ( ((gp = &agp->gp[j][agp->Nj-1]) != NULL) && ( gp->jnt != NULL) )
			cur_jnt = agp_border_gp_create_border(cur_jnt, gp, -coord, -X, agp, br);
	agp_border_links_between_jnts(cur_jnt, jnt3, agp, br);

	cur_jnt = jnt3;
	for ( j = agp->Nj-1; j >= 0; j-- )
		if ( ((gp = &agp->gp[0][j]) != NULL) && ( gp->jnt != NULL) )
			cur_jnt = agp_border_gp_create_border(cur_jnt, gp, coord, -Y, agp, br);
	agp_border_links_between_jnts(cur_jnt, jnt0, agp, br);
	
	agp_jnt_connect( agp, jnt1 );
	agp_jnt_connect( agp, jnt2 );
	agp_jnt_connect( agp, jnt3 );
}


void agp_border_clear( gp_array *agp )
{
int			i, j;
gp_type		*gp;
float		maxX, maxY;

	
	maxX = (float)(agp->Ni-2) * agp->bgp;
	maxY = (float)(agp->Nj-2) * agp->bgp;

	for ( j = 0; j < agp->Nj-1; j++ )
	if ( ((gp = &agp->gp[0][j]) != NULL) && ( gp->link != NULL) )
		agp_border_gp_clear( agp, gp, maxX, maxY );

	for ( i = 0; i < agp->Ni-1; i++ )
	if ( ((gp = &agp->gp[i][agp->Nj-1]) != NULL) && ( gp->link != NULL) )
		agp_border_gp_clear( agp, gp, maxX, maxY );

	for ( j = agp->Nj-1; j > 0; j-- )
	if ( ((gp = &agp->gp[agp->Ni-1][j]) != NULL) && ( gp->link != NULL) )
		agp_border_gp_clear( agp, gp, maxX, maxY );

	for ( i = agp->Ni-1; i > 0; i-- )
	if ( ((gp = &agp->gp[i][0]) != NULL) && ( gp->link != NULL) )
		agp_border_gp_clear( agp, gp, maxX, maxY );
}


static void 
agp_border_links_between_jnts(jnt_type* jnt1, jnt_type* jnt2, gp_array* agp, int br)
{
	link_type *first_link, *last_link;

	if (jnt1 == jnt2) return;

	first_link = links_from_line(&jnt1->gctr, &jnt2->gctr , 12, NF_EDGE, &last_link ); 
	agp_border_links_aux(first_link, last_link, agp, br);
	agp_lnf_link( &first_link->ctr, first_link, agp );
	jnt_connet_link( jnt1, -1, first_link, F_BEGIN );
	jnt_set_aux( jnt1 );
	jnt_connet_link( jnt2, -1, last_link, F_END );
	jnt_set_aux( jnt2 );

	/* fill first and last profiles*/
	agp_border_profile_by_jnt(first_link, F_BEGIN);
	agp_border_profile_by_jnt(last_link, F_END);
 
	return;
}


static void agp_border_links_aux( link_type *first_link, link_type* last_link, gp_array* agp, int br)
{
	link_type* cur_link, *prev_link;
	vec2d v_ctr;
	bgp_type *bgp2 = NULL;

	for (cur_link = first_link, 	prev_link = first_link; cur_link != NULL; 
			 prev_link = cur_link ,cur_link = LINK_NEXT(cur_link) ) {
		v_ctr.x = cur_link->ctr.x + cur_link->v.x / 2;
		v_ctr.y = cur_link->ctr.y + cur_link->v.y / 2;
		bgp2 = agp_border_nearest_bgp1(agp, &cur_link->ctr);
		agp_border_link_profile(cur_link, F_BEGIN, bgp2);

		cur_link->br = cur_link->br1 = br;
		cur_link->Fbr = 0x1;
	}

	
	v_ctr.x = prev_link->ctr.x + prev_link->v.x;
	v_ctr.y = prev_link->ctr.y + prev_link->v.y;
	bgp2 = agp_border_nearest_bgp1(agp, &v_ctr);
	agp_border_link_profile(prev_link, F_END, bgp2);
}


static void agp_border_profile_by_jnt(link_type *link, int direct)
{
	jnt_type	*jnt;
	edge_type	*e,	*se;
	int	side,	sside;
	int	bno,	sbno;
	
	jnt = link->p[direct].p.jnt;
	bno = link->p[direct].bno;
	
	if (direct == F_BEGIN) {
		sbno = BRN_RIGHT( jnt, bno );
		if( jnt->brn[sbno]->direction == F_BEGIN ){
			se = jnt->brn[sbno]->link->pr[0].edge;
			sside = F_RIGHT;
		}
		else	{
			se = jnt->brn[sbno]->link->pr[1].edge;
			sside = F_LEFT;
		}
	}
	else {
		sbno = BRN_LEFT( jnt, bno );
		
		if( jnt->brn[sbno]->direction == F_END ){
			se = jnt->brn[sbno]->link->pr[1].edge;
			sside = F_RIGHT;
		}
		else	{
			se = jnt->brn[sbno]->link->pr[0].edge;
			sside = F_LEFT;
		}
	}
	
	e = link->pr[direct].edge;
	side = F_LEFT;
	
	e->side[side].bval[SP_Y] = se->side[sside].bval[SP_Y];
	e->side[side].val[SP_Y] = se->side[sside].val[SP_Y];
	
	e->side[side].bval[SP_Q] = se->side[sside].bval[SP_Q];
	e->side[side].val[SP_Q] = se->side[sside].val[SP_Q];
	
	e->side[side].bval[SP_A] = se->side[sside].bval[SP_A];
	e->side[side].val[SP_A] = se->side[sside].val[SP_A];
	return;
}



static jnt_type *
agp_border_create_jnt( gp_array *agp, gp_type *gp, vec2d *ctr)
{
jnt_type	*jnt;
vec2d		gctr;

	gctr.x = gp->ctr.x + ctr->x;
	gctr.y = gp->ctr.y + ctr->y;
	jnt = jnt_alloc_1( JNT_EDGE, &gctr );
	jnt->ctr = *ctr;
	jnt->rwidth = jnt->lwidth = 0.5F;

    jnt->nj = gp->jnt;
    gp->jnt = jnt;

	return jnt;
}


/* get jnt from on of the previous gps and create link betwwen this jnt and all 
	relevant joints in the gp . returns last jnt in this list */
static jnt_type* 
agp_border_gp_create_border 
(jnt_type* prev_gp_jnt, gp_type *gp, float coord, char direct, gp_array* agp, int br)
{
	jnt_type *jnt, *fjnt, *min_jnt, *prev, *prev_min, 
		*last_min_jnt = prev_gp_jnt, *next_fjnt;
	link_type  *link;
	vec2d shift_v;
	

	fjnt = NULL;
	/* calculate new ctrs for all relevant jnts */	
	for  (jnt = gp->jnt; jnt != NULL; jnt = jnt->nj) 
		if ( (jnt->brn_no == 1) && RELEVANT_CTR(jnt->ctr, direct, coord) 
			&& jnt !=  prev_gp_jnt && jnt->brn[0]->link->type == BRN_EDGE) {
			link = jnt->brn[0]->link;
			SET_SHIFT(jnt, jnt->ctr, direct, coord, shift_v, link->v);
			jnt->ctr.x += shift_v.x;
			jnt->ctr.y += shift_v.y;
			jnt->gctr.x += shift_v.x;
			jnt->gctr.y += shift_v.y;

			if (jnt->brn[0]->direction == F_BEGIN) {
				link->ctr.x += shift_v.x;
				link->ctr.y += shift_v.y;
				link->v.x -= shift_v.x;
				link->v.y -= shift_v.y;
			}
			else {
				link->v.x += shift_v.x;
				link->v.y += shift_v.y;
			}
			link_set_aux(link);
			/* find first good */
			if (fjnt == NULL) fjnt = jnt; 
		};
	
	/* if not found relevant joint, exit */
	if (!fjnt) return prev_gp_jnt;
	
	/* get minimal jnt and add link to it*/
	while (1) {			
		min_jnt = fjnt;
		next_fjnt = NULL;
		for  (prev = fjnt, jnt = fjnt->nj; jnt != NULL;  prev = jnt, jnt = jnt->nj) 
			if ( (jnt->brn_no == 1) && RELEVANT_CTR(jnt->ctr, direct, coord) 
				&& jnt !=  prev_gp_jnt && jnt->brn[0]->link->type == BRN_EDGE){
				if (!GREATE_FOR_LINK(jnt->ctr, min_jnt->ctr,  direct)) {
					min_jnt = jnt;				
					prev_min = prev;
					if (next_fjnt == NULL) next_fjnt = min_jnt;
				}
				else {
					if (next_fjnt == NULL) next_fjnt = jnt;
				}
			};
			
		/*add the link here*/

		agp_border_links_between_jnts(last_min_jnt, min_jnt, agp, br);

		if (min_jnt == fjnt) fjnt = next_fjnt;
		else {
			prev_min->nj = min_jnt->nj;
			min_jnt->nj = gp->jnt;
			gp->jnt = min_jnt;
		}
		last_min_jnt = min_jnt;
		if (next_fjnt == NULL) break;
	}

	return last_min_jnt;
}



static void
agp_border_link_profile( link_type *link, int direct, bgp_type *bgp )
{
qcolor_type		val[SP_NO] = {0, 0, 0};
float		depth = 6.0F;

	if ( bgp )
	{
		depth = bgp->depth;
		memcpy( val, bgp->val, sizeof(qcolor_type)*SP_NO );
	}
	agp_border_default_profile( link, direct, val, depth );

}

static void
agp_border_default_profile( link_type *link, int direct, 
						    qcolor_type val[], float depth )
{
edge_type *e;

	if ( link->pr[direct].edge == NULL )
		link->pr[direct] = profile_alloc( NF_EDGE );
	e = link->pr[direct].edge;

	e->side[F_RIGHT].bval[SP_Y] = e->side[F_LEFT].bval[SP_Y] = val[SP_Y];
	e->side[F_RIGHT].bval[SP_Q] = e->side[F_LEFT].bval[SP_Q] = val[SP_Q];
	e->side[F_RIGHT].bval[SP_A] = e->side[F_LEFT].bval[SP_A] = val[SP_A];
	e->side[F_RIGHT].val[SP_Y] = e->side[F_LEFT].val[SP_Y] = val[SP_Y]; 
	e->side[F_RIGHT].val[SP_Q] = e->side[F_LEFT].val[SP_Q] = val[SP_Q];
	e->side[F_RIGHT].val[SP_A] = e->side[F_LEFT].val[SP_A] = val[SP_A];

	e->width = 0.75F;
	e->depth = depth;
}

static bgp_type	*
agp_border_nearest_bgp1( gp_array *agp, vec2d *gctr)
{
bgp_type	*bgp;

	bgp = agp_border_get_nearest_bgp( agp, gctr, 1 );
	if ( !bgp )
		bgp = agp_border_get_nearest_bgp( agp, gctr, 2 );

	return bgp;
}

static bgp_type	*  
agp_border_get_nearest_bgp( gp_array *agp, vec2d *gctr, int rad )
{
gp_type		*gp;
int			i, j, i1, j1;
int			st_i, fin_i, st_j, fin_j;
float		dist, d;
bgp_type	*bp, *bp_nearest;
vec2d		bp_gctr;

    dist		= 1E10;
	bp_nearest	= NULL;

	i		= (int)((gctr->x+agp->bgp) * agp->inv_bgp);
    j		= (int)((gctr->y+agp->bgp) * agp->inv_bgp);

	st_i	= (i-rad <= 0) ? 0 : i-rad;
	fin_i	= (i+rad >= agp->Ni-1) ? agp->Ni-1 : i+rad;
	st_j	= (j-rad <= 0) ? 0 : j-rad;
	fin_j	= (j+rad >= agp->Nj-1) ? agp->Nj-1 : j+rad;
	
	for ( i1 = st_i; i1 <= fin_i; i1++ )
	for ( j1 = st_j; j1 <= fin_j; j1++ )
	{
		if ( (gp = &agp->gp[i1][j1]) != NULL )
		for ( bp = gp->bgp; bp; bp = bp->p )
		{
			bp_gctr.x = gp->ctr.x + bp->ctr.x;
			bp_gctr.y = gp->ctr.y + bp->ctr.y;

			if ( (d = (float)hypot( bp_gctr.x - gctr->x, 
								bp_gctr.y - gctr->y )) < dist )
			{
				dist = d;
				bp_nearest = bp;
			}
		}
	}

	return bp_nearest;
}

static int	
agp_border_new_layer( gp_array *agp )
{
layer_type	*layer;
int bri, br, Mbr;

	layer = agp->layer[0];
	layer->i0 = 1;
	layer->j0 = 1;
	layer->i1 = agp->Ni-1;
	layer->j1 = agp->Nj-1;

	br = -1;
	for ( bri = 0, Mbr = 0x1; bri < 32; bri++, Mbr <<= 1 )
	if ( layer->Mbr & Mbr )
	{
		if ( br != -1 )
			return -1;
		br = bri;
	}

	return br;
}

static void 
agp_border_gp_clear( gp_array *agp, gp_type *gp, float maxX, float maxY )
{
link_type	*link, *link_next;
vec2d		gctr, gctr2;

	link = gp->link;
	do 
	{
		link_next = link->nl;
		gctr = gp->ctr;
		gctr.x += link->ctr.x;
		gctr.y += link->ctr.y;
		gctr2.x = gctr.x + link->v.x;
		gctr2.y = gctr.y + link->v.y;
		if ( (gctr.x < 0.0F  || gctr.x > maxX ||
			  gctr.y < 0.0F  || gctr.y > maxY)  &&
		     (gctr2.x < 0.0F || gctr2.x > maxX ||
			  gctr2.y < 0.0F || gctr2.y > maxY)  )
		{
			agp_link_delete( agp, &gctr, link );
			link_next = gp->link;
		}
	}
	while ( (link = link_next ) != NULL );
}

