/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**************************
 ***	AgpAppend	***
 *************************/
#include	"GpType/GpType.h"


static void	agp_append_layer( gp_array *agp, vec2d *gctr, gp_array *uagp);

void
agp_append( gp_array *agp, vec2d *gctr, gp_array *uagp )
{
gp_type	*gp,	*ugp;
int	i,	j;
int	ui,	uj;
vec2d	ctr;

bgp_type	*bgp;
hunch_type	*h;
link_type	*l;
jnt_type	*jnt;


	if( agp->ad != NULL ){
		uagp->ad->Mbr |= agp->ad->Mbr;
		agp_append_layer( agp, gctr, uagp );
	}


	for( i = 0 ; i < agp->Ni ; i++ )
		for( j = 0, gp = agp->gp[i] ; j < agp->Nj ; j++, gp++ ){

			while( (bgp = gp->bgp ) != NULL ){
				gp->bgp = gp->bgp->p;

				ctr.x = gp->ctr.x + bgp->ctr.x + gctr->x;
				ctr.y = gp->ctr.y + bgp->ctr.y + gctr->y;

				ui = (ctr.x+ uagp->bgp) * uagp->inv_bgp;
				uj = (ctr.y+ uagp->bgp) * uagp->inv_bgp;

				ugp = &uagp->gp[ui][uj];

				bgp->ctr.x = ctr.x - ugp->ctr.x;
				bgp->ctr.y = ctr.y - ugp->ctr.y;
				bgp->p = ugp->bgp;
				ugp->bgp = bgp;
			}



			while( (h = gp->hunch ) != NULL ){
				gp->hunch = gp->hunch->nh;

				ctr.x = gp->ctr.x + h->ctr.x + gctr->x;
				ctr.y = gp->ctr.y + h->ctr.y + gctr->y;

				ui = (ctr.x+ uagp->bgp) * uagp->inv_bgp;
				uj = (ctr.y+ uagp->bgp) * uagp->inv_bgp;

				ugp = &uagp->gp[ui][uj];

				h->ctr.x = ctr.x - ugp->ctr.x;
				h->ctr.y = ctr.y - ugp->ctr.y;
				h->nh = ugp->hunch;
				ugp->hunch = h;
			}


			while( (l = gp->link ) != NULL ){
				gp->link = gp->link->nl;

				ctr.x = gp->ctr.x + l->ctr.x + gctr->x;
				ctr.y = gp->ctr.y + l->ctr.y + gctr->y;

				ui = (ctr.x+ uagp->bgp) * uagp->inv_bgp;
				uj = (ctr.y+ uagp->bgp) * uagp->inv_bgp;

				ugp = &uagp->gp[ui][uj];
				l->ctr.x = ctr.x - ugp->ctr.x;
				l->ctr.y = ctr.y - ugp->ctr.y;
				l->nl = ugp->link;
				ugp->link = l;
			}


			while( (jnt = gp->jnt ) != NULL ){
				gp->jnt = gp->jnt->nj;

				ctr.x = gp->ctr.x + jnt->ctr.x + gctr->x;
				ctr.y = gp->ctr.y + jnt->ctr.y + gctr->y;

				ui = (ctr.x+ uagp->bgp) * uagp->inv_bgp;
				uj = (ctr.y+ uagp->bgp) * uagp->inv_bgp;

				ugp = &uagp->gp[ui][uj];
				jnt->gctr = ctr;
				jnt->ctr.x = ctr.x - ugp->ctr.x;
				jnt->ctr.y = ctr.y - ugp->ctr.y;
				jnt->nj = ugp->jnt;
				ugp->jnt = jnt;
			}
		}
}



static void
agp_append_layer( gp_array *agp, vec2d *gctr, gp_array *uagp )
{
layer_type	*layer;
int	k,	i0,	j0;
vec2d v;

	if( agp->ad == NULL )	return;


	
	
	
	i0 = (gctr->x + uagp->bgp)* uagp->inv_bgp -1;
	j0 = (gctr->y + uagp->bgp)* uagp->inv_bgp -1;

	if( agp->layer_no == 0 ){
		layer = uagp->layer[uagp->layer_no++] = LAYER_ALLOC();

		layer->Mbr = agp->ad->Mbr;
		layer->i0 = i0;
		if( layer->i0 == 0 )    layer->i0 = 1;
 
		layer->j0 = j0;
		if( layer->j0 == 0 )    layer->j0 = 1;
 
		layer->i1 = layer->i0 + agp->Ni-2;
		layer->j1 = layer->j0 + agp->Nj-2;
		layer->Fdepth_delta = 0;

		return;
	}
 
 
 
	for( k = 0 ; k < agp->layer_no ; k++ ){
		layer = uagp->layer[uagp->layer_no++] = LAYER_ALLOC();

		*layer = *agp->layer[k];


		v.x = gctr->x + ( layer->i0 - 1 )*agp->bgp;
		v.y = gctr->y + ( layer->j0 - 1 )*agp->bgp;

		layer->i0 = ( v.x + uagp->bgp )* uagp->inv_bgp;
		layer->j0 = ( v.y + uagp->bgp )* uagp->inv_bgp;
		if( layer->i0 < 1 )	layer->i0 = 1;
		if( layer->j0 < 1 )	layer->j0 = 1;



		v.x = gctr->x + ( layer->i1 )*agp->bgp;
		v.y = gctr->y + ( layer->j1 )*agp->bgp;

		layer->i1 = ( v.x + uagp->bgp )* uagp->inv_bgp;
		layer->j1 = ( v.y + uagp->bgp )* uagp->inv_bgp;

		if( layer->i1 >= uagp->Ni-1 )	layer->i1 = uagp->Ni-1;
		if( layer->j1 >= uagp->Nj-1 )	layer->j1 = uagp->Nj-1;

	}     
}
