/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**********************
 ***	AgpCopy.c   ***
 **********************/
#include	<stdio.h>   
#include	<math.h>

#include	"GpType/GpType.h"

#include	"AgpTool.h"


static adepth_type *	agp_ad_copy( adepth_type *ad, int row, int column );

static void	agp_copy_layer( gp_array *agp, gp_array *nagp );

static void	agp_copy_hunch( vec2d *ctr, hunch_type *hunch, gp_array *agp );

static void	agp_copy_jnt( vec2d *ctr, jnt_type *jnt, gp_array *agp );

static void	agp_copy_bgp( vec2d *ctr, bgp_type *bgp, gp_array *agp );



static jnt_type *	jnt_copy( jnt_type *jnt );
static void	agp_copy_jnt_link( gp_array *agp, jnt_type *jnt );

static void	agp_copy_brn_in_n3( jnt_type *jnt, int bno );


gp_array *
agp_make_copy( gp_array *agp )
{
gp_array	*nagp;

	nagp = agp_alloc();

	agp_copy( agp, nagp, -1, -1, -1 );

	return( nagp );
}




void
agp_copy( gp_array *agp, gp_array *nagp, int row, int column, int bgp )
{
gp_type		*gp;
bgp_type	*bp;
hunch_type	*h;
jnt_type	*jnt;
brn_type	*b;
int	i,	j,	k;
int	ni,	nj;


	if( row == -1 ){
//		row = (agp->row/agp->bgp) * agp->bgp;
//		if( row < agp->row )	row += agp->bgp;
		row = agp->row;
	}

	if( column == -1 ){
//		column = (agp->column/agp->bgp) * agp->bgp;
//		if( column < agp->column )	column += agp->bgp;

		column = agp->column;
	}

	if( bgp == -1 )		bgp = agp->bgp;
	
	

	if( agp->Lni*agp->bgp < 8*bgp )
		ni = nj = 8*bgp;
	else	ni = nj = agp->Lni*agp->bgp;

	agp_clear( nagp, bgp, row, column, ni, nj );

	nagp->type = agp->type;
	nagp->color_type = agp->color_type;
	nagp->ad = agp_ad_copy( agp->ad, row, column );



	agp_copy_layer( agp, nagp );



	nagp->Lcoding = agp->Lcoding;


	/* copy nf and jnt  */
	for( i = 0 ; i < agp->Ni ; i++ )
		for( j = 0 ; j < agp->Nj ; j++ ){
			gp = &agp->gp[i][j];

			for( h = gp->hunch ; h != NULL ; h = h->nh )
				agp_copy_hunch( &gp->ctr, h, nagp );


			for( bp = gp->bgp ; bp != NULL ; bp = bp->p )
				agp_copy_bgp( &gp->ctr, bp, nagp );


			for( jnt = gp->jnt ; jnt != NULL ; jnt = jnt->nj )
				agp_copy_jnt( &gp->ctr, jnt, nagp );
		}



	for( i = 0 ; i < nagp->Ni ; i++ )
		for( j = 0 ; j < nagp->Nj ; j++ ){
			gp = &nagp->gp[i][j];
			for( jnt = gp->jnt ; jnt != NULL ; jnt = jnt->nj )
				agp_copy_jnt_link( nagp, jnt );
		}





	for( i = 0 ; i < nagp->Ni ; i++ )
		for( j = 0 ; j < nagp->Nj ; j++ ){
			gp = &nagp->gp[i][j];

			for( jnt = gp->jnt ; jnt != NULL ; jnt = jnt->nj ){

				if( jnt->type != JNT_N3 )	continue;

				for( k = 0 ; k < jnt->brn_no ; k++ ){
					b = jnt->brn[k];
					if( b->type == BRN_IN )
					   agp_copy_brn_in_n3( jnt, k );
				}

				jnt_n3_set_aux( jnt );
			}

		}

/*
	agp_link_in_aux( nagp );
*/
	agp_link_in_reorder( nagp );
}



static adepth_type *
agp_ad_copy( adepth_type *ad, int row, int column )
{
adepth_type *nad; 
view_type	*vw;
float	d;
int	i;
 
        if( ad == NULL )   return( NULL );
 
 
        nad = ADEPTH_ALLOC();
        nad->P = ad->P;
        nad->N = ad->N;
        nad->br_no = ad->br_no;
	nad->Mbr = ad->Mbr;
 
        nad->vw = VIEW_ALLOC();
        *nad->vw = *ad->vw;

	vw = nad->vw;
	d = (vw->uv.x1 - vw->uv.x0 )* column / vw->Upixel;
	vw->uv.x1 = vw->uv.x0 + d;

	d = (vw->uv.y1 - vw->uv.y0 )* row / vw->Vpixel;
	vw->uv.y0 = vw->uv.y1 - d;
 

        for( i = 0 ; i < ad->br_no ; i++ )
                nad->abr[i] = ad->abr[i];

 
	return( nad );
}


static void
agp_copy_layer( gp_array *agp, gp_array *nagp )
{
layer_type	*layer,	*nlayer;
vec2d	v;
int	i;

	nagp->layer_no = 0;
	for( i = 0 ; i < agp->layer_no ; i++ ){
		layer = agp->layer[i];
		nlayer = nagp->layer[nagp->layer_no++] = LAYER_ALLOC();

		*nlayer = *layer;

		v.x = ( layer->i0 - 1 )*agp->bgp;
		v.y = ( layer->j0 - 1 )*agp->bgp;

		nlayer->i0 = ( v.x + nagp->bgp )* nagp->inv_bgp;
		nlayer->j0 = ( v.y + nagp->bgp )* nagp->inv_bgp;
		if( nlayer->i0 < 1 )	nlayer->i0 = 1;
		if( nlayer->j0 < 1 )	nlayer->j0 = 1;


		v.x = ( layer->i1 )*agp->bgp;
		v.y = ( layer->j1 )*agp->bgp;

		nlayer->i1 = ( v.x + nagp->bgp )* nagp->inv_bgp;
		nlayer->j1 = ( v.y + nagp->bgp )* nagp->inv_bgp;

		if( nlayer->i1 >= nagp->Ni-1 )	nlayer->i1 = nagp->Ni-1;
		if( nlayer->j1 >= nagp->Nj-1 )	nlayer->j1 = nagp->Nj-1;
	}
}








static void
agp_copy_hunch( vec2d *ctr, hunch_type *hunch, gp_array *agp )
{
gp_type	*gp;
hunch_type	*h;
int	i,	j;
vec2d	v;

	v.x = ctr->x + hunch->ctr.x;
	v.y = ctr->y + hunch->ctr.y;


	h = hunch_make_copy( hunch );
	h->ctr = v;

        i = (h->ctr.x+agp->bgp) * agp->inv_bgp;
        j = (h->ctr.y+agp->bgp) * agp->inv_bgp;

	gp = &agp->gp[i][j];


        h->ctr.x -= gp->ctr.x;
        h->ctr.y -= gp->ctr.y;
        h->nh = gp->hunch;
        gp->hunch = h;
}



static void
agp_copy_jnt( vec2d *ctr, jnt_type *jnt, gp_array *agp )
{
gp_type	*gp;
jnt_type	*njnt;
int	i,	j;
vec2d	v;

	v.x = ctr->x + jnt->ctr.x;
	v.y = ctr->y + jnt->ctr.y;

	njnt = jnt->cjnt = jnt_copy( jnt );

	njnt->gctr = v;


        i = (njnt->gctr.x+agp->bgp) * agp->inv_bgp;
        j = (njnt->gctr.y+agp->bgp) * agp->inv_bgp;

	gp = &agp->gp[i][j];


        njnt->ctr.x = njnt->gctr.x - gp->ctr.x;
        njnt->ctr.y = njnt->gctr.y - gp->ctr.y;
        njnt->nj = gp->jnt;
        gp->jnt = njnt;
}



static void
agp_copy_bgp( vec2d *ctr, bgp_type *bgp, gp_array *agp )
{
gp_type	*gp;
bgp_type	*b;
int	i,	j;
vec2d	v;

	v.x = ctr->x + bgp->ctr.x;
	v.y = ctr->y + bgp->ctr.y;


	b = BGP_ALLOC();
	*b = *bgp;
	b->ctr = v;

        i = (b->ctr.x+agp->bgp) * agp->inv_bgp;
        j = (b->ctr.y+agp->bgp) * agp->inv_bgp;

	gp = &agp->gp[i][j];


        b->ctr.x -= gp->ctr.x;
        b->ctr.y -= gp->ctr.y;
        b->p = gp->bgp;
        gp->bgp = b;
}







static jnt_type *
jnt_copy( jnt_type *jnt )
{
jnt_type	*njnt;
int	k;

	njnt = jnt_alloc();
	*njnt = *jnt;

	for( k = 0 ; k < jnt->brn_no ; k++ ){
		njnt->brn[k] = brn_alloc();
		*njnt->brn[k] = *jnt->brn[k];
	}

	return( njnt );
}



static void
agp_copy_jnt_link( gp_array *agp, jnt_type *jnt )
{
jnt_type        *njnt;
brn_type        *b,     *nb;
link_type       *link,  *l;
vec2d   ctr;
int     k;

        for( k = 0 ; k < jnt->brn_no ; k++ ){
                b = jnt->brn[k];
/*
                if( b->type != NF_EDGE && b->type != NF_N3 &&
                                                b->type != NF_RIDGE )
*/
		if( b->type == BRN_NULL || b->type == BRN_IN )
                        continue;

                if( b->direction != F_BEGIN )   continue;

                link = link_make_copy( b->link, b->type );

                b->link = link;
                b->link->p[F_BEGIN].p.jnt = jnt;


                for( l = link ; LINK_NEXT(l ) != NULL ; l = LINK_NEXT(l ) );

                njnt = l->p[F_END].p.jnt->cjnt;
                nb = njnt->brn[l->p[F_END].bno];

                nb->link = l;
                nb->link->p[F_END].p.jnt = njnt;


                ctr.x = jnt->gctr.x + b->ctr.x;
                ctr.y = jnt->gctr.y + b->ctr.y;
		agp_lnf_link( &ctr, b->link, agp );
        }
}




static void
agp_copy_brn_in_n3( jnt_type *jnt, int bno )
{
jnt_type	*njnt,	*pjnt;
brn_type	*b;
int	nbno;

	b = jnt->brn[bno];

	link_get_jnt_and_brn_no( b->link, b->direction, &pjnt, &nbno );

	njnt = pjnt->cjnt;

	b->link = njnt->brn[nbno]->link;

	b->in = NULL;
}

