/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/*************************
 ***	AdepthTool.c   ***
 *************************/

#include	"Uigp/igp.h"

#include	"GpType/AdepthType.h"



adepth_type *
adepth_copy( adepth_type *ad )
{
adepth_type *nad;
int     i;

	if( ad == NULL )   return( NULL );

	nad = ADEPTH_ALLOC();
	nad->P = ad->P; 
	nad->N = ad->N;
	nad->br_no = ad->br_no;
	nad->Mbr = ad->Mbr;

	nad->vw = VIEW_ALLOC();
	*nad->vw = *ad->vw;


	for( i = 0 ; i < ad->br_no ; i++ )
		nad->abr[i] = ad->abr[i];


	return( nad );
}



adepth_type *
adepth_default( float depth, int row, int column )
{
adepth_type	*ad;

	ad = ADEPTH_ALLOC();
	ad->Mbr = 0;
 
	ad->br_no = 0;
 
	ad->vw = VIEW_ALLOC(); 

	view_create_noraml( ad->vw, 1.0/512.0, row, column ); 

	ad->N = ad->vw->N;
	ad->P.x = ad->vw->cp.x + depth *  ad->vw->N.x; 
	ad->P.y = ad->vw->cp.y + depth *  ad->vw->N.y; 
	ad->P.z = ad->vw->cp.z + depth *  ad->vw->N.z; 

	return( ad );
}



void
adepth_set_plane( adepth_type *ad, int layer, vec3d *N, vec3d *P )
{
int	i;
 
	if( layer == -1 ){
		ad->N = *N;
		ad->P = *P;
		return;
	}
 
 
	for( i = 0 ; i < ad->br_no ; i++ ){
		if( ad->abr[i].no != layer )    continue;
 
		ad->abr[i].N = *N;
		ad->abr[i].P = *P;
		ad->abr[i].Fplane = 1;

		return;
	}

	ad->abr[i].N = *N;
	ad->abr[i].P = *P;
	ad->abr[i].no = layer;
	ad->abr[i].Fplane = 1;

	ad->br_no++;

	ad->Mbr |= (1L << layer );
}



void
adepth_set_planeD( adepth_type *ad, int layer, vec3d *N, float depth )
{
vec3d	P;
int	i;

	P.x = ad->vw->cp.x + depth * N->x;
	P.y = ad->vw->cp.y + depth * N->y;
	P.z = ad->vw->cp.z + depth * N->z;

 
	if( layer == -1 ){
		ad->N = *N;
		ad->P = P;
		return;
	}
 
 
	for( i = 0 ; i < ad->br_no ; i++ ){
		if( ad->abr[i].no != layer )    continue;
 
		ad->abr[i].N = *N;
		ad->abr[i].P = P;
		ad->abr[i].Fplane = 1;

		return;
	}

	ad->abr[i].N = *N;
	ad->abr[i].P = P;
	ad->abr[i].no = layer;
	ad->abr[i].Fplane = 1;

	ad->br_no++;

	ad->Mbr |= (1L << layer );
}



void
adepth_set_planeND( adepth_type *ad, int layer, float depth )
{
vec3d	N;

	N.x = 0;
	N.y = 0;
	N.z = -1;

	adepth_set_planeD( ad, layer, &N, depth );
}





int
adepth_get_plane( adepth_type *ad, int layer, vec3d *N, float *d )
{
abr_type	*abr;
int     i;
vec3d	p;
 
	if( layer == 0 ){ 
		*N = ad->N;
		*d = VEC3D_INNER( ad->N, ad->P);
		return(1);
	}
 
 
	for( i = 0 ; i < ad->br_no ; i++ ){
		abr = &ad->abr[i];
		if( abr->no != layer )     continue; 
 
		if( abr->Fplane == 0 )  return( 0 );
 
		*N = abr->N;


		p.x = abr->P.x - ad->vw->cp.x;
		p.y = abr->P.y - ad->vw->cp.y;
		p.z = abr->P.z - ad->vw->cp.z;
		*d = VEC3D_INNER( abr->N, p );
		return( 1 ); 
	}

	return(0);
}


abr_type *
adepth_get_layer_depth( adepth_type *ad, int layer )
{
abr_type	*abr;
int     i;
 
	for( i = 0 ; i < ad->br_no ; i++ ){
		abr = &ad->abr[i];

		if( abr->no == layer )	return( abr );
	}

	return( NULL );
}




void
adepth_rescale( adepth_type *ad, float fk )
{
vec2d   vp,     v;
vec3d   p3;

	view_project_point( ad->vw, &ad->P, &vp );


	ad->vw->uv.x0 *= fk;
	ad->vw->uv.x1 *= fk;
	ad->vw->uv.y0 *= fk;
	ad->vw->uv.y1 *= fk;

	view_set_pixel( ad->vw, ad->vw->Vpixel, ad->vw->Upixel );


	vp.x *= fk;
	vp.y *= fk;
	view_vp2pixel( ad->vw, &vp, &v );


	view_pixel2plane_xyz( ad->vw, &v, &ad->P, &ad->N, &p3 );
	ad->P = p3;
}
