/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/************************************************************************
*									*
*                            AgpClear.c					*
*									*
************************************************************************/
#include        <stdio.h>   
#include	<math.h>

#include        "GpType/GpType.h"


static int	agp_clear_jnt_box( gp_array *agp,
					jnt_type *jnt, int fborder );



void
agp_clear_box( gp_array *agp )
{
int	i,	j;
gp_type	*gp;
jnt_type	*jnt,	*pjnt;
hunch_type	*hunch;
int	Fout,	Fouti,	Fborder;


	for( i = agp->Ni0-1 ; i < agp->Ni1+1 ; i++ ){

	    Fouti = ( i ==  agp->Ni0-1 || i == agp->Ni1 )? 1 : 0;


	    for( j = agp->Nj0-1 ; j < agp->Nj1+1 ; j++ ){

	    	Fout = ( j ==  agp->Nj0-1 || j == agp->Nj1 )? 1 : Fouti;

		Fborder = agp_Fborder_ij( agp, i, j );


		gp = &agp->gp[i][j];


		if( Fout == 0 ){

			while( gp->hunch != NULL ){
				hunch = gp->hunch;
				gp->hunch = gp->hunch->nh;
				free( hunch );
			}
		}
		

		pjnt = NULL;
		for( jnt = gp->jnt ; jnt != NULL ; ){

			if( Fout && jnt->Fborder == JNT_IN ){
				pjnt = jnt;
				jnt = pjnt->nj;
				continue;
			}


			if( agp_clear_jnt_box( agp, jnt, Fborder ) == 0 ){
				pjnt = jnt;
				jnt = pjnt->nj;
				continue;
			}


			if( pjnt == NULL ){
				gp->jnt = jnt->nj;
				free( jnt );
				jnt = gp->jnt;
				continue;
			}

			pjnt->nj = jnt->nj;
			free( jnt );
			jnt = pjnt->nj;
		}
	    }
	}
}



static int
agp_clear_jnt_box( gp_array *agp, jnt_type *jnt, int fborder )
{
int	n,	bno;
brn_type	*b;
vec2d	ctr;


	if( jnt->Fborder == JNT_IN ){

		for( bno = 0 ; bno < jnt->brn_no ; bno++ ){
			b = jnt->brn[bno];

			if( b->type != BRN_EDGE &&
				b->type != BRN_N3 && b->type != BRN_RIDGE )
				continue;

			if( b->direction == F_BEGIN ){
                               
				ctr.x = jnt->gctr.x + b->ctr.x;
				ctr.y = jnt->gctr.y + b->ctr.y; 
				agp_lnf_clear_link( &ctr, b->link, agp );
				link_free( b->link );
			}

			b->type = BRN_NULL;
			free( b );
		}

/*
		free( jnt );
*/

		return( 1 );
	}



	for( bno = 0 ; bno < jnt->brn_no ; bno++ ){
		b = jnt->brn[bno];

		if( b->type != BRN_EDGE && b->type != BRN_N3 &&
						b->type != BRN_RIDGE )
				continue;


		if( b->Fborder == fborder ){
			if( b->direction == F_BEGIN ){
				ctr.x = jnt->gctr.x + b->ctr.x;
				ctr.y = jnt->gctr.y + b->ctr.y; 
				agp_lnf_clear_link( &ctr, b->link, agp );
				link_free( b->link );
			}

			b->type = BRN_NULL;
		}
	}



	if( jnt->type == JNT_N3 ){

		if( jnt->brn[0]->type == BRN_NULL && 
				jnt->brn[1]->type == BRN_NULL &&
				jnt->brn[2]->type == BRN_NULL ){

			free(jnt->brn[0]);
			free(jnt->brn[1]);
			free(jnt->brn[2]);
/*
			free( jnt );
*/
			return( 1 );
		}

		return( 0 );
	}
				

	n = 0;
	for( bno = 0 ; bno < jnt->brn_no ; bno++ ){
		b = jnt->brn[bno];

		if( b->type == BRN_NULL ){

			free(jnt->brn[bno]);
			jnt->brn[bno] = NULL;

			continue;
		}

		if( b->direction == F_BEGIN )
			b->link->p[F_BEGIN].bno = n;
		else	b->link->p[F_END].bno = n;

		jnt->brn[n++] = b;
	}

	jnt->brn_no = n;
	if( jnt->brn_no > 0 )	return( 0 );

	return( 1 );
}
