/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/****************************************************************************/
/*                            GpTool.c                                   */ 

/****************************************************************************/
#include	<stdio.h>

#include        "Uigp/igp.h"


#include        "GpType/GpType.h"
#include        "N3Prime/N3Prime.h"


gp_array *
agp_alloc()
{
gp_array *agp;

	agp = (gp_array *)malloc( sizeof( gp_array ) );

	agp->type = AGP_NORMAL;

	agp->Fn3_prime = 0;

	agp->ad = NULL;
	agp->Ni = agp->Nj = 0;

	agp->row = agp->column = agp->bgp = 0;

	agp->layer_no = 0;
	agp->Mbr = 0;

	agp->vf = NULL;

	return( agp );
}


void
agp_init( gp_array *agp, int bgp, int ni, int nj, int lni, int lnj )
{
int	i,	j,	tmp;
vec2d	v,	v0;
gp_type	*gp;

	agp->row = ni;
	agp->column = nj;

	agp->bgp = bgp;
	agp->inv_bgp = 1.0/bgp;
	v0.x = v0.y = -bgp *0.5;

	tmp = ( ni + agp->bgp-1) /agp->bgp;
	agp->rowR = tmp *agp->bgp;
	agp->Ni = tmp+2;
 
	tmp = ( nj + agp->bgp-1) /agp->bgp;
	agp->columnR = tmp *agp->bgp;
	agp->Nj = tmp+2;

	agp->Lni = lni/agp->bgp;
	agp->Lnj = lnj/agp->bgp;

	agp->type = AGP_NORMAL;
	agp->Fn3_prime = 0;
	agp->ad = NULL;


	gp_array_alloc( agp, agp->Ni, agp->Nj );


	for( i = 0, v.x = v0.x ; i < agp->Ni ; i++, v.x += agp->bgp ){
		gp =  &agp->gp[i][0];
		for( j = 0, v.y = v0.y ; j < agp->Nj ;
					j++, v.y += agp->bgp, gp++ ){
			gp->ctr.x = v.x;
			gp->ctr.y = v.y;

			gp->hunch = NULL;

			gp->jnt = NULL;

			gp->link = NULL;

			gp->bgp = NULL;

		}
	}
}




void
agp_clear( gp_array *agp, int bgp, int ni, int nj, int lni, int lnj )
{
int	i,	j;

	for( i = 0 ; i < agp->Ni; i++ )
		for( j = 0 ; j < agp->Nj; j++ )
			gp_clear( &agp->gp[i][j] );

	if( (agp->type & AGP_DEPTH) && agp->ad != NULL ){
		free( agp->ad->vw );
		free( agp->ad );
		agp->ad = NULL;
	}


	for( i = 0 ; i < agp->layer_no ; i++ ){
		free( agp->layer[i] );
		agp->layer[i] = NULL;
	}
	agp->layer_no = 0;


	if( agp->vf != NULL ){
		free( agp->vf );
		agp->vf = NULL;
	}


	agp->type = AGP_NORMAL;

	gp_array_free( agp );
	agp_init( agp, bgp, ni, nj, lni, lnj );
}


void
agp_destroy( gp_array *agp )
{
int	i,	j;

	if( agp->Fn3_prime ){
		n3_prime_free( agp );
		agp->Fn3_prime = 0;
	}
	

	if( (agp->type & AGP_DEPTH) && agp->ad != NULL ){
		free( agp->ad->vw );
		free( agp->ad );
		agp->ad = NULL;
	}



	for( i = 0 ; i < agp->layer_no ; i++ ){
		free( agp->layer[i] );
		agp->layer[i] = NULL;
	}
	agp->layer_no = 0;


	if( agp->vf != NULL )	free( agp->vf );


		
	for( i = 0 ; i < agp->Ni; i++ )
		for( j = 0 ; j < agp->Nj; j++ )
			gp_clear( &agp->gp[i][j] );

	gp_array_free( agp );

	free( agp );
}




void
gp_clear( gp_type *gp )
{
hunch_type	*hunch;
jnt_type	*jnt;
bgp_type	*bp;

	while( (hunch= gp->hunch) != NULL ){
		gp->hunch = gp->hunch->nh;
		free( hunch );
	}


	while( (jnt= gp->jnt) != NULL ){
		gp->jnt = gp->jnt->nj;
		jnt_free( jnt );
	}


	gp->link = NULL;



	while( (bp = gp->bgp) != NULL ){
		gp->bgp = gp->bgp->p;
		free( bp );
	}
	gp->bgp = NULL;
}


void
gp_array_realloc( gp_array *agp, int ni, int nj )
{
        if( agp->Ni == ni && agp->Nj == nj )  return;

        gp_array_free( agp );

        gp_array_alloc( agp, ni, nj );
}


void
gp_array_alloc( gp_array *agp, int ni, int nj )
{
int          i;  
int          byte;

        agp->Ni  = ni ;
        agp->Nj  = nj ;

        agp->gp = (gp_type **)malloc( agp->Ni * sizeof( gp_type * ) );

        byte = agp->Nj * sizeof(gp_type);

        for( i = 0 ; i < agp->Ni ; i++ )
                agp->gp[i] = (gp_type *)malloc( byte );
}



void
gp_array_free( gp_array *agp )
{
int          i;  
	if( agp->Ni == 0 )	return;

        for( i = 0 ; i < agp->Ni ; i++ )
                free( agp->gp[i] );

        free( agp->gp );
        agp->Ni = agp->Nj = 0;
	agp->type = AGP_NORMAL;

	agp->row = agp->column = agp->bgp = 0;
}



void
gp_clear_hunch( gp_type *gp, hunch_type *hunch )
{
hunch_type *p;

	if( gp->hunch == hunch ){
		gp->hunch = hunch->nh;
		free(hunch);
		return;
	}

	for( p = gp->hunch ; p->nh != NULL ; p = p->nh ){
		if( p->nh == hunch ){
			p->nh = hunch->nh;
			free(hunch);
			return;
		}
	}
}



void
agp_clear_jnt( gp_array *agp, jnt_type *jnt )
{
int	i,	j;
gp_type	*gp;
jnt_type	*p;

	i = (jnt->gctr.x+agp->bgp) * agp->inv_bgp;
        j = (jnt->gctr.y+agp->bgp) * agp->inv_bgp;

	gp = &agp->gp[i][j];

	if( gp->jnt == jnt ){
		gp->jnt = jnt->nj;
		jnt_free(jnt);
		return;
	}

	for( p = gp->jnt ; p->nj != NULL ; p = p->nj ){
		if( p->nj == jnt ){
			p->nj = jnt->nj;
			jnt_free( jnt );
			return;
		}
	}
}


void
agp_untied_jnt( gp_array *agp, jnt_type *jnt )
{
gp_type *gp;
jnt_type	*p;
int     i,      j;
 
	i = ( jnt->gctr.x + agp->bgp ) * agp->inv_bgp;
	j = ( jnt->gctr.y + agp->bgp ) * agp->inv_bgp;
 
	gp = &agp->gp[i][j];

	if( gp->jnt == jnt ){
		gp->jnt = jnt->nj;
		return;
	}

	for( p = gp->jnt ; p->nj != NULL ; p = p->nj ){
		if( p->nj == jnt ){
			p->nj = jnt->nj;
			return;
		}
	}
}


void
gp_untied_jnt( gp_type *gp, jnt_type *jnt )
{
jnt_type	*p;
 
	if( gp->jnt == jnt ){
		gp->jnt = jnt->nj;
		return;
	}

	for( p = gp->jnt ; p->nj != NULL ; p = p->nj ){
		if( p->nj == jnt ){
			p->nj = jnt->nj;
			return;
		}
	}
}

 

void
agp_untied_link( gp_array *agp, vec2d *ctr, link_type *link )
{
gp_type *gp;
link_type       *l;
int     i,      j;
	i = ( ctr->x + agp->bgp ) * agp->inv_bgp;
	j = ( ctr->y + agp->bgp ) * agp->inv_bgp;

	gp = &agp->gp[i][j];

	if( gp->link == link ){
		gp->link = gp->link->nl;
		return;
	}

	for( l = gp->link ; l != NULL ; l = l->nl ){
		if( l->nl != link ) continue;

		l->nl = link->nl;
		return;
	}
}


void
gp_untied_link( gp_type *gp, link_type *link )
{
link_type       *l;

	if( gp->link == link ){
		gp->link = gp->link->nl;
		return;
	}

	for( l = gp->link ; l != NULL ; l = l->nl ){
		if( l->nl != link ) continue;

		l->nl = link->nl;
		return;
	}
}




int
agp_nf_no( gp_array *agp, int type )
{
gp_type		*gp;
jnt_type	*jnt;
hunch_type	*h;
int	i,	j,	n;
int	nf_no;

	nf_no = 0;

	for( i = 0 ; i < agp->Ni ; i++ )
		for( j = 0 ; j < agp->Nj ; j++ ){

			gp = &agp->gp[i][j];

			for( h = gp->hunch ; h != NULL ; h = h->nh )
				if( h->type == type )
					nf_no ++;

			for( jnt = gp->jnt ; jnt != NULL ; jnt = jnt->nj ){
				for( n = 0 ; n < jnt->brn_no ; n++ )
					if( jnt->brn[n]->type == type &&
					     jnt->brn[n]->direction == F_BEGIN )
						nf_no ++;
			}
					
		}

	return( nf_no );
}




int
agp_get_hunchs_array( gp_array *agp, hunch_type *ahunch[] )
{
int	i,	j;
int     i0,     j0,     i1,     j1;
gp_type	*gp;
hunch_type	*h;
int	nf_no;

	nf_no = 0;

	if( (i0 = agp->Ni0) == 1 )      i0 = 0;
	if( (j0 = agp->Nj0) == 1 )      j0 = 0;
 
	if( (i1 = agp->Ni1) == agp->Ni-1 )      i1 = agp->Ni;
	if( (j1 = agp->Nj1) == agp->Nj-1 )      j1 = agp->Nj;
 

	for(i = i0 ; i < i1 ; i++)
		for( j = j0  ; j < j1 ; j++ ){
			gp = &agp->gp[i][j];

			for( h = gp->hunch ; h != NULL ; h = h->nh )
				ahunch[nf_no++] = h;
		}

	return( nf_no );
}

int
agp_get_ridge_array( gp_array *agp, link_type *link[] )
{
int	i,	j,	bno;
int     i0,     j0,     i1,     j1;
gp_type	*gp;
jnt_type	*jnt;
brn_type	*b;
int	l_no;

	l_no = 0;

	if( (i0 = agp->Ni0) == 1 )      i0 = 0;
	if( (j0 = agp->Nj0) == 1 )      j0 = 0;
 
	if( (i1 = agp->Ni1) == agp->Ni-1 )      i1 = agp->Ni;
	if( (j1 = agp->Nj1) == agp->Nj-1 )      j1 = agp->Nj;
 

	for(i = i0 ; i < i1 ; i++)
	    for( j = j0  ; j < j1 ; j++ ){
		gp = &agp->gp[i][j];

		for( jnt = gp->jnt ; jnt != NULL ; jnt = jnt->nj ){

			for( bno = 0 ; bno < jnt->brn_no ; bno++ ){
				b = jnt->brn[bno];

				if( b->type != NF_RIDGE && b->type != NF_N3R )
					continue;

				if( b->direction != F_BEGIN )   continue;
	
				link[l_no++] = b->link;
			}
		}
	    }

	return( l_no );
}



int
agp_get_hunch_array( gp_array *agp, int type, hunch_type *ahunch[] )
{
int	i,	j;
int     i0,     j0,     i1,     j1;
gp_type	*gp;
hunch_type	*h;
int	nf_no;

	nf_no = 0;

	if( (i0 = agp->Ni0) == 1 )      i0 = 0;
	if( (j0 = agp->Nj0) == 1 )      j0 = 0;
 
	if( (i1 = agp->Ni1) == agp->Ni-1 )      i1 = agp->Ni;
	if( (j1 = agp->Nj1) == agp->Nj-1 )      j1 = agp->Nj;
 

	for(i = i0 ; i < i1 ; i++)
		for( j = j0  ; j < j1 ; j++ ){
			gp = &agp->gp[i][j];

			for( h = gp->hunch ; h != NULL ; h = h->nh )
				if( h->type == type )
					ahunch[nf_no++] = h;
		}

	return( nf_no );
}


int
agp_get_link_type_array( gp_array *agp, int type, link_type *link[] )
{
int	i,	j,	bno;
int     i0,     j0,     i1,     j1;
gp_type	*gp;
jnt_type	*jnt;
brn_type	*b;
int	l_no;

	l_no = 0;

	if( (i0 = agp->Ni0) == 1 )      i0 = 0;
	if( (j0 = agp->Nj0) == 1 )      j0 = 0;
 
	if( (i1 = agp->Ni1) == agp->Ni-1 )      i1 = agp->Ni;
	if( (j1 = agp->Nj1) == agp->Nj-1 )      j1 = agp->Nj;
 

	for(i = i0 ; i < i1 ; i++)
	    for( j = j0  ; j < j1 ; j++ ){
		gp = &agp->gp[i][j];

		for( jnt = gp->jnt ; jnt != NULL ; jnt = jnt->nj ){

			for( bno = 0 ; bno < jnt->brn_no ; bno++ ){
				b = jnt->brn[bno];
				if( b->type != type )	continue;
				if( b->direction != F_BEGIN )   continue;
	
				link[l_no++] = b->link;
			}
		}
	    }

	return( l_no );
}




void
agp_set_box( gp_array *agp, int row, int col )
{
	agp->Ni0 = row/ agp->bgp + 1;
	if( (agp->Ni1 = agp->Ni0 + agp->Lni) >= agp->Ni)    agp->Ni1 =agp->Ni-1;
	 
	agp->Nj0 = col/ agp->bgp + 1;
	if( (agp->Nj1 = agp->Nj0 + agp->Lnj) >= agp->Nj)    agp->Nj1 =agp->Nj-1;
 
	agp->X0 = (agp->Ni0-1)*agp->bgp;
	agp->Y0 = (agp->Nj0-1)*agp->bgp;
}

void
agp_set_box_ij( gp_array *agp, int ni0, int nj0 )
{
	agp->Ni0 = ni0;
	if( (agp->Ni1 = agp->Ni0 + agp->Lni) > agp->Ni )    agp->Ni1 =agp->Ni-1;
         
	agp->Nj0 = nj0;
	if( (agp->Nj1 = agp->Nj0 + agp->Lnj) > agp->Nj )    agp->Nj1 =agp->Nj-1;
 
	agp->X0 = (agp->Ni0-1)*agp->bgp;
	agp->Y0 = (agp->Nj0-1)*agp->bgp;
}


int
agp_set_next_box( gp_array *agp )
{
	if( (agp->Nj0 += agp->Lnj ) < agp->Nj ){
		if( (agp->Nj1 = agp->Nj0 + agp->Lnj) > agp->Nj )
			   agp->Nj1 =agp->Nj-1;
		agp->Y0 = (agp->Nj0-1)*agp->bgp;
		return( 1 );
	}

	if( (agp->Ni0 += agp->Lni ) > agp->Ni )
		return( -1 );

		
	if( (agp->Ni1 = agp->Ni0 + agp->Lni) > agp->Ni )    agp->Ni1 =agp->Ni-1;
         
	agp->Nj0 = 1;
	if( (agp->Nj1 = agp->Nj0 + agp->Lnj) > agp->Nj )    agp->Nj1 =agp->Nj-1;
 
	agp->X0 = (agp->Ni0-1)*agp->bgp;
	agp->Y0 = (agp->Nj0-1)*agp->bgp;

	return (0);
}



void
agp_geomtric_aux( gp_array *agp )
{
int	i,	j;
gp_type	*gp;
jnt_type	*jnt;

	for( i = 0 ; i < agp->Ni ; i++ ){

		for( j = 0 ; j < agp->Nj ; j++ ){

			gp = &agp->gp[i][j];

			for( jnt = gp->jnt ; jnt != NULL ; jnt = jnt->nj ){

				if( jnt->type == JNT_RIDGE )	continue;

				switch( jnt->type ){
				case JNT_EDGE:
					jnt_edge_set_aux( jnt );
				    break;
				case JNT_N3:
					jnt_n3_set_aux( jnt );
				    break;
				case JNT_RN3:
					jnt_rn3_set_aux( jnt );
				    break;
				}
			}
		}
	}
}



void
agp_geomtric_aux_box( gp_array *agp )
{
int	i,	j;
gp_type	*gp;
jnt_type	*jnt;

	for( i = agp->Ni0-1 ; i < agp->Ni1+1 ; i++ ){

		for( j = agp->Nj0-1 ; j < agp->Nj1+1 ; j++ ){

			gp = &agp->gp[i][j];

			for( jnt = gp->jnt ; jnt != NULL ; jnt = jnt->nj ){

				if( jnt->type == JNT_RIDGE )	continue;

				switch( jnt->type ){
				case JNT_EDGE:
					jnt_edge_set_aux( jnt );
				    break;
				case JNT_N3:
					jnt_n3_set_aux( jnt );
				    break;
				case JNT_RN3:
					jnt_rn3_set_aux( jnt );
				    break;
				}
			}
		}
	}
}


int
agp_Fborder_xy( gp_array *agp, vec2d *ctr )
{
int	i,	j;
int	fborder;

	i = ( ctr->x + agp->bgp ) * agp->inv_bgp;
	j = ( ctr->y + agp->bgp ) * agp->inv_bgp;

	fborder = 0;
	if( i == agp->Ni0-1 || i == agp->Ni0 )  fborder |= 0x01;
	else    if( i == agp->Ni1-1 || i == agp->Ni1 )  fborder |= 0x02;

	if( j == agp->Nj0-1 || j == agp->Nj0 )  fborder |= 0x04;
	else    if( j == agp->Nj1-1 || j == agp->Nj1 )  fborder |= 0x08;

	return( fborder );
}


int
agp_Fborder_ij( gp_array *agp, int i, int j )
{
int	fborder; 

	fborder = 0;
	if( i == agp->Ni0-1 || i == agp->Ni0 )  fborder |= 0x01;
	else    if( i == agp->Ni1-1 || i == agp->Ni1 )  fborder |= 0x02;

	if( j == agp->Nj0-1 || j == agp->Nj0 )  fborder |= 0x04;
	else    if( j == agp->Nj1-1 || j == agp->Nj1 )  fborder |= 0x08;

	return( fborder );
}
