/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/*********************
 ***    JntN3.c    ***
 *********************/

#include	<stdio.h>
#include	<math.h>

#include	"Uigp/igp.h"

#include	"GpType/JntType.h"




void
jnt_n3_load_u( jnt_type *jnt )
{
brn_type	*b;

	if( !(jnt->Fborder & JNT_STATIC) ){
		b = jnt->brn[0];

		if( b->direction == F_BEGIN ){

			link_Ntanget_beg( b->link, &jnt->u );

			jnt->u.x = -jnt->u.x;
			jnt->u.y = -jnt->u.y;
		}
		else
			link_Ntanget_end( b->link, &jnt->u );
	}


	jnt->brn[1]->ctr.x = jnt->rwidth*jnt->u.x;
	jnt->brn[1]->ctr.y = jnt->rwidth*jnt->u.y;

	jnt->brn[2]->ctr.x = -jnt->lwidth*jnt->u.x;
	jnt->brn[2]->ctr.y = -jnt->lwidth*jnt->u.y;
}



void
jnt_n3_load_from_link( jnt_type *jnt )
{
brn_type	*b;
width_type	*width;

	b = jnt->brn[0];

	if( b->direction == F_BEGIN ){

		link_Ntanget_beg( b->link, &jnt->u );

		jnt->u.x = -jnt->u.x;
		jnt->u.y = -jnt->u.y;

		if( b->link->width[F_BEGIN] != NULL ){
			width = b->link->width[F_BEGIN];

			width->bes.x = -( -jnt->u.y);
			width->bes.y = -( jnt->u.x);

			jnt->rwidth = -width->w[F_LEFT];
			jnt->lwidth = width->w[F_RIGHT];
		}
		else	jnt->rwidth = jnt->lwidth = 0.75;
	}
	else	{
		link_Ntanget_end( b->link, &jnt->u );

		if( b->link->width[F_END] != NULL ){
			width = b->link->width[F_END];

			width->bes.x = -jnt->u.y;
			width->bes.y =  jnt->u.x;

			jnt->rwidth = width->w[F_RIGHT];
			jnt->lwidth = -width->w[F_LEFT];
		}
		else	jnt->rwidth = jnt->lwidth = 0.75;
	}


	jnt->brn[1]->ctr.x = jnt->rwidth*jnt->u.x;
	jnt->brn[1]->ctr.y = jnt->rwidth*jnt->u.y;

	jnt->brn[2]->ctr.x = -jnt->lwidth*jnt->u.x;
	jnt->brn[2]->ctr.y = -jnt->lwidth*jnt->u.y;
}

	


void
jnt_n3_set_aux( jnt_type *jnt )
{
brn_type	*b;
int	i;
float	w;

	for( i = 0 ; i < jnt->brn_no ; i++ ){
		b = jnt->brn[i];

		switch( b->type&0x0f ){
		case BRN_NULL:
			b->type = BRN_NULL;
			b->link = NULL;
			b->in = NULL;

			if( jnt->type == JNT_N3  &&  i == 0 )	break;

			w = ( i == 1 )? jnt->rwidth : -jnt->lwidth;
			b->ctr.x = w * jnt->u.x;
			b->ctr.y = w * jnt->u.y;

		    break;
		case BRN_IN:
			if( b->in == NULL )
				jnt_n3_in_set( jnt, i );
		    break;
		case BRN_EDGE:
		case BRN_N3:
			b->in = NULL;
			break;
		}
	}
}





void
jnt_n3_in_set( jnt_type *jnt, int bno )
{
brn_type	*b;
jnt_type	*njnt;
link_type	*link;
int	nbno,	link_no,	side,	tside;
float	u,	t,	w,	t0;
vec2d	ctr,	v;
in_type	*in,	**p;

	b = jnt->brn[bno];


	ctr.x = jnt->gctr.x + b->ctr.x;
	ctr.y = jnt->gctr.y + b->ctr.y;


	/* find the begining */
	for( link = b->link ; LINK_PREV(link) != NULL ; link = LINK_PREV(link));

	njnt = link->p[F_BEGIN].p.jnt;
	nbno = link->p[F_BEGIN].bno;
	v.x = njnt->gctr.x + njnt->brn[nbno]->ctr.x;
	v.y = njnt->gctr.y + njnt->brn[nbno]->ctr.y;



	link_no = links_distance( &v, link, NULL, &ctr, &b->link, &u, &t );



/*
	if( link_no == 0 && t < 0.5 )	t = 0;
*/
	if( LINK_PREV(b->link) == NULL  && t < 0.5 )	t = 0;
	else	if( LINK_NEXT(b->link) == NULL && b->link->len - t < 0.5 )
			t = b->link->len;
	t0 = t;



	side = ( jnt->brn[0]->direction == F_BEGIN )? 2-bno : bno-1;
	if( jnt->brn[0]->direction != b->direction )
		tside = INV_FLAG( side );
	else	tside = side;



	w = ( bno == 1 )? jnt->rwidth : -jnt->lwidth;
	b->ctr.x = w * jnt->u.x;
	b->ctr.y = w * jnt->u.y;





	in = IN_TYPE_ALLOC();
	in->t = t0;
	in->jnt = jnt;
	in->bno = bno;
	in->p = NULL;

	in->type = (jnt->brn[0]->type == BRN_N3 )? IN_N3 : IN_NULL;
	in->direct = b->direction;


	b->in = in;

	p = &b->link->in[tside];
	while( *p != NULL ){
		if( in->t < (*p)->t ){
			in->p = *p;
			*p = in;
			return;
		}

		if( in->t == (*p)->t &&
				in->jnt->brn[in->bno]->direction == F_END ){
			in->p = *p;
			*p = in;
			return;
		}

		p = &(*p)->p;
	}
	*p = in;
	in->p = NULL;
}




void
jnt_n3_in_delete( jnt_type *jnt, int bno )
{
brn_type *b;
in_type *p;
int	side,	tside;

	b = jnt->brn[bno];
	if( b->in == NULL )     return;

	side = ( jnt->brn[0]->direction == F_BEGIN )? 2-bno : bno-1;
	if( jnt->brn[0]->direction != b->direction )
		tside = INV_FLAG( side );
	else	tside = side;


	if( b->link->in[tside] == b->in ){
		b->link->in[tside] = b->in->p;
		free( b->in );
		b->in = NULL;
		b->type = BRN_NULL;
		return;
	}


	for( p = b->link->in[tside] ; p->p != NULL ; p = p->p ){
		if( p->p != b->in )     continue;

		p->p = b->in->p;
		free( b->in );
		b->in = NULL;
		b->type = BRN_NULL;
		return;
	}
}








void
jnt_n3_in_replace( jnt_type *jnt, int bno, jnt_type *njnt )
{
int	nbno = 3 - bno;

	if( njnt->type == JNT_RN3 )
		jnt_n3_from_jnt_rn3( njnt );

	njnt->brn[nbno]->type = BRN_IN;
	njnt->brn[nbno]->link = jnt->brn[0]->link;
	njnt->brn[nbno]->direction =
		INV_FLAG(jnt->brn[0]->direction);
	jnt_n3_in_set( njnt, nbno );


	jnt_n3_in_delete( jnt, bno );
	jnt->brn[bno]->in = NULL;
	jnt->brn[bno]->type = 0;
}




int
jnt_n3_from_jnt_rn3( jnt_type *jnt )
{
brn_type	*b;

	if( jnt->brn_no != 1 )	return(0);

	b = jnt->brn[1] = brn_alloc();
	b->type = BRN_NULL;
	b->in = NULL;
 
	b = jnt->brn[2] = brn_alloc();
	b->type = BRN_NULL;
	b->in = NULL;
 
	jnt->brn_no = 3; 
	jnt->type = JNT_N3;
 
	jnt_n3_load_from_link( jnt );

	return( 1 );
} 



int
jnt_n3_to_jnt_rn3( jnt_type *jnt )
{
	if( jnt->brn[1]->type != BRN_NULL || jnt->brn[2]->type != BRN_NULL  )
		return( 0 );
 
	free( jnt->brn[1] );
	free( jnt->brn[2] );
	jnt->brn_no = 1;
	jnt->type = JNT_RN3;
 
	jnt_rn3_set_aux( jnt );
 
	return( 1 );
}



void
jnt_n3_set_in_direct( jnt_type *jnt, int bno )
{
link_type	*l;
vec2d	u;
 
	u.x = -jnt->u.y;
	u.y =  jnt->u.x;
	l = jnt->brn[bno]->link;
 
	if( l->v.x *u.x + l->v.y *u.y > 0 )
		jnt->brn[bno]->direction = F_BEGIN;
	else	jnt->brn[bno]->direction = F_END;
}

