/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**************************
 ***	JntRN3.c	***
 **************************/

#include	<stdio.h>
#include	<math.h>

#include	"Uigp/igp.h"

#include	"GpType/JntType.h"
#include	"GpType/GpType.h"



static void	jnt_rn3_brn1( jnt_type *jnt );

static void	jnt_rn3_brn2( jnt_type *jnt );

static void	jnt_rn3_brn_nu( jnt_type *jnt, int bno, vec2d *nu );

/*
static void	jnt_rn3_v( jnt_type *jnt, vec2d *v0, vec2d *v1 );
*/



void
jnt_rn3_set_aux( jnt_type *jnt )
{
	if( jnt->brn_no == 1 )
		jnt_rn3_brn1( jnt );
	else	jnt_rn3_brn2( jnt );

		
}



static void
jnt_rn3_brn1( jnt_type *jnt )
{
brn_type	*b;
width_type	*width;

	b = jnt->brn[0];

/*
	jnt_rn3_brn_nu( jnt, 0, &jnt->u );
*/
	if( b->direction == F_BEGIN ){
 
		link_Ntanget_beg( b->link, &jnt->u );
 
		jnt->u.x = -jnt->u.x;
		jnt->u.y = -jnt->u.y;
 
		if( b->link->width[F_BEGIN] != NULL ){
			width = b->link->width[F_BEGIN];
 
			width->bes.x = -( -jnt->u.y);
			width->bes.y = -( jnt->u.x);
 
/*
			jnt->rwidth = -width->w[F_LEFT];
			jnt->lwidth = width->w[F_RIGHT];
		}
		else    jnt->rwidth = jnt->lwidth = 0.75;
*/
		}
	}
	else    {
		link_Ntanget_end( b->link, &jnt->u );
 
		if( b->link->width[F_END] != NULL ){
			width = b->link->width[F_END];
 
			width->bes.x = -jnt->u.y;
			width->bes.y =  jnt->u.x;
 
/*
			jnt->rwidth = width->w[F_RIGHT];
			jnt->lwidth = -width->w[F_LEFT];
		}
		else    jnt->rwidth = jnt->lwidth = 0.75;
*/
		}
	}
}



static void
jnt_rn3_brn2( jnt_type *jnt )
{
brn_type	*b;
vec2d	u0,	u1;
float	t;


	jnt_rn3_brn_nu( jnt, 0, &u0 );
	jnt_rn3_brn_nu( jnt, 1, &u1 );



	jnt->u.x = u0.x - u1.x;
	jnt->u.y = u0.y - u1.y;

	t = 1.0/hypot( jnt->u.x, jnt->u.y );
	jnt->u.x *= t;
	jnt->u.y *= t;


	b = jnt->brn[0];

	u1.x = jnt->u.y;
	u1.y = -jnt->u.x;
	u0.x = -u1.x;
	u0.y = -u1.y;

	if( b->type == BRN_N3 ){
		if( b->direction == F_END )
			b->link->width[F_END]->bes = u0;
		else
			b->link->width[F_BEGIN]->bes = u1;
	}


	b = jnt->brn[1];


	if( b->type == BRN_N3 ){
		if( b->direction == F_END )
			b->link->width[F_END]->bes = u1;
		else
			b->link->width[F_BEGIN]->bes = u0;
	}
}






static void
jnt_rn3_brn_nu( jnt_type *jnt, int bno, vec2d *nu )
{
brn_type	*b;

	b = jnt->brn[bno];

	if( b->link == NULL ){
		nu->x = nu->y = 0;
		return;
	}

	if( b->direction == F_END ){
		/*
		nu->x = b->link->width[F_END]->bes.y;
		nu->y = -b->link->width[F_END]->bes.x;
		*/
		nu->x = b->link->u.x;
		nu->y = b->link->u.y;
	}
	else	{
		/*
		nu->x = -b->link->width[F_BEGIN]->bes.y;
		nu->y = b->link->width[F_BEGIN]->bes.x;
		*/
		nu->x = -b->link->u.x;
		nu->y = -b->link->u.y;
	}
}



/*
static void
jnt_rn3_v( jnt_type *jnt, vec2d *v0, vec2d *v1 )
{
	v0->x = jnt->gctr.x + jnt->rwidth * jnt->u.x;
	v0->y = jnt->gctr.x + jnt->rwidth * jnt->u.y;

	v1->x = jnt->gctr.x + -jnt->lwidth * jnt->u.x;
	v1->y = jnt->gctr.x + -jnt->lwidth * jnt->u.y;
}
*/
