/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**********************
 ***	JntTool.c   ***
 **********************/
#include	<stdio.h>
#include	<math.h>
#ifndef __SYMBIAN32__
	#include	<malloc.h>
	#include	<memory.h>	
#endif // 	__SYMBIAN_32__


#include	"GpType/JntType.h"


jnt_type *
jnt_alloc()
{
jnt_type *jnt;

	jnt = (jnt_type *)malloc( sizeof( jnt_type ));
	jnt->type = JNT_NULL;
	jnt->brn_no = 0;


	jnt->brn[0] = jnt->brn[1] = jnt->brn[2]  = BRN_NULL;

	jnt->pj[0] = jnt->pj[1] = NULL;

	return( jnt );
}


jnt_type *
jnt_alloc_1( int type, vec2d *ctr )
{
jnt_type	*jnt;
 
	jnt = jnt_alloc();
 
	jnt->type = type;
 
	jnt->gctr = *ctr;
 
	jnt->jnt_no = -1;
 
	jnt->Fborder = JNT_IN;
 
 
	if( jnt->type == JNT_N3 ){
		jnt->brn[0] = brn_alloc();
		jnt->brn[1] = brn_alloc();
		jnt->brn[2] = brn_alloc();
		jnt->brn_no = 3;
	}
 
	return( jnt );
}


brn_type *
brn_alloc()
{
brn_type *b;

	b = (brn_type *)malloc( sizeof(brn_type) );

	b->type = BRN_NULL;
	b->link = NULL;
	b->in = NULL;
	b->ctr.x = b->ctr.y = 0;
 
	b->Fborder = 0;
	b->bno = 0;

	return( b );
}


void
jnt_insert_brn( int type, vec2d *ctr, link_type *link,
		jnt_type *jnt, int bno, jnt_type *njnt, int nbno, int flag )
{
brn_type	*b;
vec2d	v;

	if( (b = jnt->brn[bno]) == NULL )
		b = jnt->brn[bno] = brn_alloc();

	b->type = type;

	b->direction = F_BEGIN;


	if( flag == 1 && jnt->type != JNT_N3 ){
		b->ctr.x = ctr->x - jnt->gctr.x;
		b->ctr.y = ctr->y - jnt->gctr.y;
	}


	b->link = link;

	b->link->p[F_BEGIN].p.jnt = jnt;
	b->link->p[F_BEGIN].bno = bno;





	/* NEXT */

	if( (b = njnt->brn[nbno]) == NULL )
		b = njnt->brn[nbno] = brn_alloc();

	b->type = type;

	b->direction = F_END;




	v = *ctr;
	for( b->link = link ; LINK_NEXT(b->link) != NULL ;
						b->link = LINK_NEXT(b->link) ){
		v.x += b->link->v.x;
		v.y += b->link->v.y;
	}

	if( flag == 1 && njnt->type != JNT_N3 ){
		b->ctr.x = v.x + b->link->v.x - njnt->gctr.x;
		b->ctr.y = v.y + b->link->v.y - njnt->gctr.y;
	}

	b->link->p[F_END].p.jnt = njnt;
	b->link->p[F_END].bno = nbno;
}





void
jnt_free( jnt_type *jnt )
{
brn_type	*b;
int	k;

	for( k = 0 ; k < jnt->brn_no ; k++ ){
		b = jnt->brn[k];

		if( IS_BRN_NULL(b->type) || b->type == BRN_IN ){
			free( b );
			continue;
		}

		if( b->direction == F_BEGIN )
			link_free( b->link );

		free( b );
	}

	free( jnt );
}



void
jnt_free_brn( jnt_type *jnt, int bno )
{
brn_type	*b;
int	k;

        if( jnt->type == JNT_N3 ){

		jnt->brn[bno]->type = BRN_NULL;
		if( bno != 0 )	return;

		if( jnt->brn[1]->type == BRN_NULL &&
			jnt->brn[2]->type == BRN_NULL ){ 
			free( jnt->brn[0] );
			free( jnt->brn[1] );
			free( jnt->brn[2] );

			jnt->brn_no = 0;
		}

		return;
        }


	free( jnt->brn[bno] );
	jnt->brn_no--;
	for( k = bno ; k < jnt->brn_no ; k++ ){
		b = jnt->brn[k] = jnt->brn[k+1];
		b->link->p[b->direction].bno = k;
	}
}


void
jnt_untied_brn( jnt_type *jnt, int bno, int Fn3 )
{
brn_type	*b;
int	k;

        if( jnt->type == JNT_N3 ){

		jnt->brn[bno]->type = BRN_NULL;

		if( Fn3 == 0 )	return;

		if( jnt->brn[1]->type != BRN_NULL ||
				jnt->brn[2]->type != BRN_NULL )
			return;

		if( jnt->brn[0]->type == BRN_NULL ){
			free( jnt->brn[0] );
			free( jnt->brn[1] );
			free( jnt->brn[2] );

			jnt->brn_no = 0;
			jnt->type = JNT_NULL;
		}
		else	jnt_n3_to_jnt_rn3( jnt );

		return;
        }


	free( jnt->brn[bno] );
	jnt->brn_no--;
	for( k = bno ; k < jnt->brn_no ; k++ ){
		b = jnt->brn[k] = jnt->brn[k+1];
		b->link->p[b->direction].bno = k;
	}

	if( jnt->brn_no > 0 && jnt->type == JNT_RN3 )
		jnt_rn3_set_aux( jnt );
}



int
jnt_connet_link( jnt_type *jnt, int bno, link_type *link, int direct )
{
brn_type	*b;
 
	if( bno == -1 ){
		bno = jnt->brn_no;
		jnt->brn[jnt->brn_no++] = brn_alloc();
	}
 
	b = jnt->brn[bno];
 
	b->type = link->type;
 
	b->link = link;
	b->direction = direct;
 
	link->p[direct].p.jnt = jnt;
	link->p[direct].bno = bno;
 
	return( bno );
}



void
jnt_set_aux( jnt_type *jnt )
{
	switch( jnt->type ){
	case JNT_EDGE:
		jnt_edge_set_aux( jnt );
	    break;

	case JNT_N3:
		jnt_n3_set_aux( jnt );
	    break;

	case JNT_RN3:
		jnt_rn3_set_aux( jnt );
	    break;

	case JNT_RIDGE:
		jnt_rn3_set_aux( jnt );
	    break;
	}
}




void
link_get_jnt_and_brn_no( link_type *link, int direct, jnt_type **jnt, int *bno )
{
link_type	*l;

	if( direct == F_END ){
		for( l = link ; LINK_NEXT(l) != NULL ; l =  LINK_NEXT(l) );
		*jnt = l->p[F_END].p.jnt;
		*bno = l->p[F_END].bno;
		return;
	}

	for( l = link ; LINK_PREV(l) != NULL ; l =  LINK_PREV(l) );
	*jnt = l->p[F_BEGIN].p.jnt;
	*bno = l->p[F_BEGIN].bno;
}


void
link_get_jnt( link_type *link, int direct, jnt_type **jnt, int *bno, int *no )
{
link_type	*l;

	if( direct == F_END ){
		for( l = link, *no = 1 ; LINK_NEXT(l) != NULL ;
					l =  LINK_NEXT(l), (*no)++ );
		*jnt = l->p[F_END].p.jnt;
		*bno = l->p[F_END].bno;
		return;
	}

	for( l = link, *no = 1 ; LINK_PREV(l) != NULL ;
						l =  LINK_PREV(l), (*no)++ );
	*jnt = l->p[F_BEGIN].p.jnt;
	*bno = l->p[F_BEGIN].bno;
}





void
jnt_n3_pvector( jnt_type *jnt, vec2d *p )
{
	p->x = -jnt->u.y;
	p->y = jnt->u.x;
}


void
jnt_brn_ctr( jnt_type *jnt, int bno, vec2d *ctr )
{
brn_type	*b;

	b = jnt->brn[bno];

	ctr->x = jnt->gctr.x + b->ctr.x;
	ctr->y = jnt->gctr.y + b->ctr.y;

	if( b->direction == F_END ){
		ctr->x -= b->link->v.x;
		ctr->y -= b->link->v.y;
	}
}


void
jnt_n3_uvector( jnt_type *jnt, vec2d *p )
{
	if( jnt->brn[0]->direction == F_BEGIN ){
		p->x = -jnt->u.x;
		p->y = -jnt->u.y;
	}
	else	*p = jnt->u;
}




void
jnt_load_nf_end_link( link_type *link )
{
jnt_type	*jnt,   *njnt;
brn_type	*b,	*nb;
link_type	*l;
vec2d	v,	ctr;
int	bno;

	jnt = link->p[F_BEGIN].p.jnt;
	bno = link->p[F_BEGIN].bno;
	b = jnt->brn[bno];


	ctr.x = jnt->gctr.x + b->ctr.x;
	ctr.y = jnt->gctr.y + b->ctr.y;
	for( l = link, v = ctr ; LINK_NEXT(l) != NULL ; l = LINK_NEXT(l) ){
		v.x += l->v.x;
		v.y += l->v.y;
	}


	njnt = l->p[F_END].p.jnt;
	bno = l->p[F_END].bno;
	nb = njnt->brn[bno];


	l->v.x = njnt->gctr.x + nb->ctr.x - v.x;
	l->v.y = njnt->gctr.y + nb->ctr.y - v.y;

	link_set_aux( l );
}



/*
void
jnt_set_depth( jnt_type *jnt )
{
brn_type	*b;
float	tmp;
int     i,      no;

	if( jnt->type == JNT_N3 ){
		if( jnt->brn[0]->type == BRN_NULL )
			return;

		b = jnt->brn[0];
		if( b->direction == F_BEGIN )
			tmp = b->link->pr[0].p->depth;
		else    tmp = b->link->pr[1].p->depth;

		jnt->depth = tmp;
		return;
	}



	jnt->depth = 0;
	no = 0;
	for( i = 0 ; i < jnt->brn_no ; i++ ){
		b = jnt->brn[i];
		if( b->type == BRN_NULL || b->type == BRN_IN )  continue;
 
		if( b->direction == F_BEGIN )
			tmp = b->link->pr[0].p->depth;
		else    tmp = b->link->pr[1].p->depth;
 
		jnt->depth += tmp;
		no++;
	}
 
	jnt->depth /= no;
}
*/
