/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/*********************
 ***	LinkIn.c   ***
 *********************/
#include	<stdio.h>

#include	"GpType/GpType.h"
#include	"GpType/JntType.h"
#include	"GpType/LinkType.h"



void
link_in_reposition( link_type *l0, link_type *l1 )
{
in_type *in,    *p,     *in0[2],	*in1[2];

	in0[F_RIGHT] = l0->in[F_RIGHT];
	in0[F_LEFT] = l0->in[F_LEFT];
	l0->in[F_RIGHT] = NULL;
	l0->in[F_LEFT] = NULL;

	in1[F_RIGHT] = l1->in[F_RIGHT];
	in1[F_LEFT] = l1->in[F_LEFT];
	l1->in[F_RIGHT] = NULL;
	l1->in[F_LEFT] = NULL;


	for( in = in0[F_RIGHT] ; in != NULL ; ){
		in->jnt->brn[in->bno]->in = NULL;
		jnt_n3_in_set( in->jnt, in->bno );
		p = in->p;
		free( in );
		in = p;
	}

	for( in = in0[F_LEFT] ; in != NULL ; ){
		in->jnt->brn[in->bno]->in = NULL;
		jnt_n3_in_set( in->jnt, in->bno );
		p = in->p;
		free( in );
		in = p;
	}

	for( in = in1[F_RIGHT] ; in != NULL ; ){
		in->jnt->brn[in->bno]->in = NULL;
		jnt_n3_in_set( in->jnt, in->bno );
		p = in->p;
		free( in );
		in = p;
	}

	for( in = in1[F_LEFT] ; in != NULL ; ){
		in->jnt->brn[in->bno]->in = NULL;
		jnt_n3_in_set( in->jnt, in->bno );
		p = in->p;
		free( in );
		in = p;
	}
}







void
agp_link_in_reorder( gp_array *agp )
{
int     i,      j;
gp_type *gp;   
jnt_type	*jnt;

	for( i = 0 ; i < agp->Ni ; i++ ){

		for( j = 0 ; j < agp->Nj ; j++ ){

			gp = &agp->gp[i][j];

			for( jnt = gp->jnt ; jnt != NULL ; jnt = jnt->nj ){

				if( jnt->type != JNT_N3 )    continue;
				if( jnt->brn[0]->direction != F_BEGIN )
					continue;

				if( jnt->brn[0]->type != BRN_N3 )
					continue;

				link_in_reorder( jnt, F_RIGHT );

				link_in_reorder( jnt, F_LEFT );
			}
		}
	}
}


void
link_in_reorder( jnt_type *jnt, int side )
{
int	flag;
link_type	*l,	*nl;
in_type	**p,	*np,	*cp;

	if( side == F_RIGHT )
		flag = ( jnt->brn[2]->type != BRN_IN )? F_END : F_BEGIN;
	else	flag = ( jnt->brn[1]->type != BRN_IN )? F_END : F_BEGIN;


	for( l = jnt->brn[0]->link ; l != NULL ; l = LINK_NEXT(l) ){
		if( l->in[side] == NULL )	continue;

		for( p = &l->in[side] ; *p != NULL ; ){
			cp = *p;

			if( cp->direct == flag ){
				flag = INV_FLAG( flag );
				p = &cp->p;
				continue;
			}

			if( (np = cp->p ) != NULL ){
				cp->p = np->p;
				np->p = cp;
				*p = np;

				flag = INV_FLAG( flag );
				p = &(np)->p;

				continue;
			}

			nl = LINK_NEXT(l);
// new 25.7
//			if( nl == NULL )	continue;
//
			if( ( np = nl->in[side] ) != NULL ){
				cp->p = np->p;
				np->p = cp;
				*p = NULL;

				continue;
			}
		}
	}
}



void
link_in_get( link_type *link, int side, link_type *ln[2] ) 
{ 
jnt_type	*jnt;
link_type	*l,	*pl;
in_type	*in;
int	bno;



        ln[0] = ln[1] = NULL;


	pl = link;
	while( ( l = LINK_PREV(pl)) != NULL ){
		if( l->in[side] != NULL )	break;
		pl = l;
	}


	if( l != NULL ){
                for( in = l->in[side] ; in->p != NULL ; in = in->p ) ; 
		if( in->direct == F_END )
			ln[0] = in->jnt->brn[0]->link;
	}
	else	{
               	jnt = pl->p[0].p.jnt; 
               	bno = (side == F_RIGHT)? 2: 1; 
               	if( jnt->type == JNT_N3 && jnt->brn[bno]->type == BRN_IN ) 
                       	ln[0] = jnt->brn[bno]->link;
	}




	if( link->in[side] == NULL ){
		ln[1] = ln[0];
		return;
	}

	for( in = link->in[side] ; in->p != NULL ; in = in->p ) ; 
	if( in->direct == F_END )
		ln[1] = in->jnt->brn[0]->link;
}


void
link_delete_in( link_type *link, int side )
{
in_type *in;
jnt_type	*jnt;

	while( (in = link->in[side]) != NULL ){
		link->in[side] = in->p;
		jnt = in->jnt;
		jnt->brn[in->bno]->type = BRN_NULL;
		jnt->brn[in->bno]->in = NULL;
		if( jnt->brn[1]->type == BRN_NULL &&
				jnt->brn[2]->type == BRN_NULL )
			jnt_n3_to_jnt_rn3( jnt );

		free( in );
	}
}