/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/***********************
 ***	LinkTool.c   ***
 ***********************/

#include	<stdio.h>
#include	<math.h>
#ifndef __SYMBIAN32__
	#include	<malloc.h>
	#include	<memory.h>	
#endif // 	__SYMBIAN32__

#include	"Uigp/igp.h"
#include	"Ubox/Box2d.h"
#include	"GpType/LinkType.h"
#include	"GpType/JntType.h"


link_type *
link_alloc( int type, int ino ) 
{ 
link_type	*link,	*l,	*pl;
int	i;
         

	link = LINK_ALLOC();
	link->p[F_BEGIN].p.l = NULL;
	link->p[F_BEGIN].bno = -2;

	link->in[F_RIGHT] = link->in[F_LEFT] = NULL;
	link->width[F_BEGIN] = link->width[F_END] = NULL;

	link->type = type;
	link->br = link->Fbr = 0;

	link->pr[0] = profile_alloc( type );

	for( i = 1, pl = link ; i < ino ; i++ ){
		l = LINK_ALLOC();
		l->p[F_BEGIN].p.l = pl;
		l->p[F_BEGIN].bno = -1;
		pl->p[F_END].p.l = l;
		pl->p[F_END].bno = -1;

		l->in[F_RIGHT] = l->in[F_LEFT] = NULL;
		l->width[F_BEGIN] = l->width[F_END] = NULL;

		l->type = type;
		l->br = l->Fbr = 0;

		l->pr[0] = profile_alloc( type );
		pl->pr[1] = l->pr[0];

		pl = l;
	}

	pl->p[F_END].p.l = NULL;
	pl->p[F_END].bno = -2;

	pl->pr[1] = profile_alloc( type );


	return( link ); 
} 



link_type *
link_1_alloc()
{
link_type	*link;

	link = LINK_ALLOC();
	memset(link,0,sizeof(link_type));
	link->p[F_BEGIN].bno = link->p[F_END].bno = -1;	
	return( link );
}




void
link_alloc_width( link_type *link ) 
{ 
link_type	*l,	*nl;
width_type	*w;
         
	w = link->width[F_BEGIN] = WDITH_TYPE_ALLOC();
	w->depth[F_RIGHT] = w->depth[F_LEFT] = 0;
 
	for( l = link, nl = LINK_NEXT(l) ; nl != NULL ;
					 l = nl, nl = LINK_NEXT(l) ){

		w = l->width[F_END] = nl->width[F_BEGIN] =  WDITH_TYPE_ALLOC();

		w->depth[F_RIGHT] = w->depth[F_LEFT] = 0;
	}

	w = l->width[F_END] = WDITH_TYPE_ALLOC();
	w->depth[F_RIGHT] = w->depth[F_LEFT] = 0;
}



link_type *
link_make_copy( link_type *plink, int type ) 
{ 
link_type	*link,	*l,	*pl,	*el,	*nl;

	pl = l = NULL;
 
	for( el = plink ; el != NULL ;  el = LINK_NEXT(el) ){
		
		l = LINK_ALLOC();
		*l = *el;

		if( pl != NULL ){
			l->p[F_BEGIN].p.l = pl;
			pl->p[F_END].p.l = l;
		}
		else	link = l;

		l->in[F_RIGHT] = l->in[F_LEFT] = NULL;

		if( l->pr[0].p != NULL ){
			l->pr[0] = profile_make_copy( l->pr[0], type );
			if( LINK_NEXT(l) == NULL )
				l->pr[1] = profile_make_copy( l->pr[1], type );
		}

		pl = l;
	}


	for( l = link ;  (nl =  LINK_NEXT(l)) != NULL ;  ){
		l->pr[1] = nl->pr[0];
		l = nl;
	}

	if( type != NF_N3 )	return( link );



	link_alloc_width( link );
	for( l = link, el = plink ; l != NULL ;
				l =  LINK_NEXT(l), el = LINK_NEXT(el) ){
		*l->width[F_BEGIN] = *el->width[F_BEGIN];
		if( LINK_NEXT(l) == NULL )
			*l->width[F_END] = *el->width[F_END];
	}

 
	return( link ); 
}


void
link_free( link_type *link )
{
link_type	*l,	*pl;
in_type	*p,	*np;

	for( l = link ; l != NULL ; /*l = LINK_NEXT(l) */ ){

		if( l->pr[0].p != NULL ){

			free( l->pr[0].p );

			if( LINK_NEXT(l) == NULL )
				free( l->pr[1].p );
		}


		for( p = l->in[F_RIGHT] ; p != NULL ; ){
			np = p->p;
			free( p );
			p = np;
		}

		for( p = l->in[F_LEFT] ; p != NULL ; ){
			np = p->p;
			free( p );
			p = np;
		}

		if( l->width[F_BEGIN] != NULL )
			free( l->width[F_BEGIN] );

		if( LINK_NEXT(l) == NULL  && l->width[F_END] != NULL )
			free( l->width[F_END] );

		pl = l;
		l = LINK_NEXT(l);

		free( pl );
	}
}


void
link_1_free( link_type *l )
{
in_type	*p,	*np;


	if( l->pr[0].p != NULL ){

		free( l->pr[0].p );

		if( LINK_NEXT(l) == NULL )
			free( l->pr[1].p );
	}


	for( p = l->in[F_RIGHT] ; p != NULL ; ){
		np = p->p;
		free( p );
		p = np;
	}

	for( p = l->in[F_LEFT] ; p != NULL ; ){
		np = p->p;
		free( p );
		p = np;
	}

	if( l->width[F_BEGIN] != NULL )
		free( l->width[F_BEGIN] );

	if( LINK_NEXT(l) == NULL  && l->width[F_END] != NULL )
		free( l->width[F_END] );

	free( l );
}

void
link_width_bes( link_type *link )
{
link_type	*l,	*nl;
vec2d	tmp;
float	t;

	link->width[0]->bes.x = -link->u.y;
	link->width[0]->bes.y = link->u.x;

	for( l = link, nl = LINK_NEXT(l) ; nl != NULL ;
					l = nl, nl = LINK_NEXT(nl) ){
		tmp.x = -( l->u.y + nl->u.y );
		tmp.y =  ( l->u.x + nl->u.x );

		t = hypot( tmp.x, tmp.y );
		t = 1.0/t;	

		nl->width[0]->bes.x = t*tmp.x;
		nl->width[0]->bes.y = t*tmp.y;
	}

	l->width[1]->bes.x = -l->u.y;
	l->width[1]->bes.y = l->u.x;
}
	






void 
link_set_aux( link_type *l ) 
{ 
        if( l->v.x == 0.0 && l->v.y == 0 ){ 
                l->len = l->u.x = l->u.y = l->c_prb = l->a = 0.0; 
                return;  
        }
 
        l->len = hypot( l->v.x, l->v.y );
 
        l->u.x = l->v.y/l->len;
        l->u.y = -l->v.x/l->len;
 
        l->c_prb = -4.0 *l->a/( l->len*l->len); 
} 


void
link_t2xy( vec2d *vm, link_type *link, float t, vec2d *p )
{
float	h;

	h = link->c_prb * t * (t - link->len);

	p->x = vm->x  +  t * -link->u.y  +  h * link->u.x;
	p->y = vm->y  +  t *  link->u.x  +  h * link->u.y;
}


void
link_tanget( link_type *l, float t, vec2d *tanget )
{
float	tmp;

	tmp = l->c_prb * ( 2*t - l->len );

	tanget->x = -l->u.y + l->u.x* tmp;
	tanget->y =  l->u.x + l->u.y* tmp;

	tmp = 1.0/hypot( tanget->x, tanget->y );
	tanget->x *= tmp;
	tanget->y *= tmp;
}



void
link_Ntanget_beg( link_type *l, vec2d *t )
{
float	tmp = l->c_prb* l->len;
vec2d	v;

	v.x = -l->u.y - l->u.x* tmp;
	v.y =  l->u.x - l->u.y* tmp;

	tmp = 1.0/hypot( v.x, v.y );

	t->x = v.y*tmp;
	t->y = -v.x*tmp;
}

void
link_Ntanget_end( link_type *l, vec2d *t )
{
float	tmp = l->c_prb* l->len;
vec2d	v;

	v.x = -l->u.y + l->u.x* tmp;
	v.y =  l->u.x + l->u.y* tmp;

	tmp = 1.0/hypot( v.x, v.y );

	t->x = v.y*tmp;
	t->y = -v.x*tmp;
}



	/* point with tanget vector ( 1, 0 )	*/
int
link_tanget_point_10( link_type *l, vec2d *vm, float *t, vec2d *p )
{
float	tmp;

	if( l->c_prb == 0.0 )	return(0);

	tmp = l->c_prb * l->u.y;

	if( ABS( l->u.x ) > ABS( tmp )*l->len )	return( 0 );

	*t = 0.5*( -l->u.x / tmp + l->len);
	link_t2xy( vm, l, *t, p );
	return( 1 );
}



	/* point with tanget vector ( 0, 1 )	*/
int
link_tanget_point_01( link_type *l, vec2d *vm, float *t, vec2d *p )
{
float	tmp;

	if( l->c_prb == 0.0 )	return(0);

	tmp = l->c_prb * l->u.x;

	if( ABS( l->u.y ) > ABS( tmp )*l->len )	return( 0 );

	*t = 0.5*( +l->u.y / tmp + l->len);
	link_t2xy( vm, l, *t, p );
	return( 1 );
}


void
link_box2d( link_type *l, box2d *b )
{
float	tmp,	t,	h;
vec2d	p;
 
	if( l->v.x < 0 ){
		b->x0 = l->v.x;
		b->x1 = 0;
	}
	else    {
		b->x0 = 0;
		b->x1 = l->v.x;
	}

	if( l->v.y < 0 ){
		b->y0 = l->v.y;
		b->y1 = 0;
	}
	else    {
		b->y0 = 0;
		b->y1 = l->v.y;
	}

	if( l->c_prb == 0.0 )   return;


        tmp = l->c_prb * l->u.y;
        if( ABS( l->u.x ) < ABS( tmp )*l->len ){
 
        	t = 0.5*( -l->u.x / tmp + l->len);
        	h = l->c_prb * t * (t - l->len);
 
        	p.y = t *  l->u.x  +  h * l->u.y;

		if( p.y < b->y0 )	b->y0 = p.y;
		else	b->y1 = p.y;
	}


        tmp = l->c_prb * l->u.x;
        if( ABS( l->u.y ) < ABS( tmp )*l->len ){
 
        	t = 0.5*( +l->u.y / tmp + l->len);
 
        	h = l->c_prb * t * (t - l->len);
 
        	p.x = t * -l->u.y  +  h * l->u.x;

		if( p.x < b->x0 )	b->x0 = p.x;
		else	b->x1 = p.x;
	}
}




void	polylink_box2d(link_type *l, box2d *b, link_type **lx0, link_type **lx1, 
					   link_type **ly0, link_type **ly1)
{
	link_type *link;
	box2d tmpBox;
	link_box2d(l,b);
	*lx0 = *lx1 = *ly0 = *ly1 = l;


	for ( link = LINK_NEXT(l) ; link != NULL ; link = LINK_NEXT(link))
	{
		link_box2d(link,&tmpBox);
		if ( tmpBox.x0 < b->x0 )
		{
			*lx0 = link;
			b->x0 = tmpBox.x0;
		}
		if ( tmpBox.x1 < b->x1 )
		{
			*lx1 = link;
			b->x1 = tmpBox.x1;
		}
		if ( tmpBox.y0 < b->y0 )
		{
			*ly0 = link;
			b->y0 = tmpBox.y0;
		}
		if ( tmpBox.y1 < b->y1 )
		{
			*ly1 = link;
			b->y1 = tmpBox.y1;
		}
	}
}



link_type *
link_inverse( link_type *link )
{
link_type	*l,	*l1;

//	l = polylink_get_first_link(link);

	while( (l = LINK_PREV(link)) != NULL )	link = l;

	l = link;

	while(  l != NULL ){
		link_1_inverse( l );

		l1 = l;

		l = LINK_PREV(l);
	}

	return( l1 );
}


void
link_1_inverse( link_type *l )
{
profile_type	pr;
width_type	*w;
link_ptr	p;
in_type	*in,	*pin;
int	Fbr;

	l->v.x = - l->v.x;
	l->v.y = - l->v.y;

	l->u.x = - l->u.x;
	l->u.y = - l->u.y;

	l->a = -l->a;
	l->c_prb = -l->c_prb;




	if( l->Fbr != 16 ){
		Fbr = 0;
		if( l->Fbr &0x01 )	Fbr |= 0x02;
		if( l->Fbr &0x02 )	Fbr |= 0x01;
		l->Fbr = Fbr;
	}




	pr = l->pr[0];
	l->pr[0] = l->pr[1];
	l->pr[1] = pr;

	p = l->p[0];
	l->p[0] = l->p[1];
	l->p[1] = p;


	if( l->pr[0].p == NULL )	return;


	if( LINK_PREV_JNT(l) != NULL )
		l->p[0].p.jnt->brn[l->p[0].bno ]->direction = F_BEGIN;

	if( LINK_NEXT_JNT(l) != NULL )
		l->p[1].p.jnt->brn[l->p[1].bno ]->direction = F_END;


	if( l->type == NF_EDGE ){
		edge_inverse( l->pr[0].edge );
		if( LINK_NEXT(l) == NULL )
			edge_inverse( l->pr[1].edge );
		return;
	}


	if( l->type != NF_N3 )	return;



	n3_inverse( l->pr[0].n3 );

	w = l->width[0];
	l->width[0] = l->width[1];
	l->width[1] = w;

	width_inverse( l->width[0] );

	if( LINK_NEXT(l) == NULL ){
		width_inverse( l->width[1] );
		n3_inverse( l->pr[1].n3 );
	}


	in = l->in[F_RIGHT];
	l->in[F_RIGHT] = l->in[F_LEFT];
	l->in[F_LEFT] = in;



	in = l->in[F_RIGHT];
	l->in[F_RIGHT] = NULL;
	while( in != NULL ){
		in->jnt->brn[in->bno]->direction =
				INV_FLAG( in->jnt->brn[in->bno]->direction );
		in->direct = INV_FLAG( in->direct );
		in->t = l->len - in->t;

		pin = in;
		in = in->p;
		pin->p = l->in[F_RIGHT];
		l->in[F_RIGHT] = pin;
	}

	in = l->in[F_LEFT];
	l->in[F_LEFT] = NULL;
	while( in != NULL ){
		in->jnt->brn[in->bno]->direction =
				INV_FLAG( in->jnt->brn[in->bno]->direction );
		in->direct = INV_FLAG( in->direct );
		in->t = l->len - in->t;

		pin = in;
		in = in->p;
		pin->p = l->in[F_LEFT];
		l->in[F_LEFT] = pin;
	}
}


void
link_set_br( link_type *link, int br, int Fbr )
{
link_type	*l;

	for( l = link ; l != NULL ; l = LINK_NEXT(l) ){
		l->br = br;
		l->Fbr = Fbr;
	}
}


