/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**************************
 ***	LinkTool.c	***
 **************************/

#include	<stdio.h>
#include	<math.h>

#ifndef __SYMBIAN32__
	#include	<malloc.h>
#endif // 	__SYMBIAN_32__

#include	"Uigp/igp.h"
#include	"GpType/JntType.h"
#include	"GpType/LinkType.h"



int
links_distance( vec2d *ctr, link_type  *link, link_type *elink, vec2d  *v,
		link_type **sl, float *u, float *t )
{
link_type	*l;
int	i,	ret;
vec2d	cv;
float	cu,	ct,	dt;
float	dis,	tmp;


	*sl = NULL;
	*u = 100;
	*t = 100;
	ret = -1;

	dis = 100000;

	cv.x = v->x - ctr->x;
	cv.y = v->y - ctr->y;

	for( l = link, i = 0 ; l != elink ; l = LINK_NEXT(l), i++ ){

		cu = l->u.x * cv.x + l->u.y * cv.y;

		ct = -l->u.y * cv.x + l->u.x * cv.y;

		cv.x -= l->v.x;
		cv.y -= l->v.y;



		if( ct < 0 ){
			tmp = cu*cu + 1.44*ct*ct;

			if( cu > 0 )	cu -= ct;
			else	cu += ct;

			ct = 0;
		}
		else	if( (dt = ct - l->len) > 0 ){

				tmp = cu*cu + 1.44*dt*dt;

				if( cu > 0 )	cu += dt;
				else	cu -= dt;
				ct = l->len;
			}
			else	{
					cu -= l->c_prb*ct *(ct - l->len );
					tmp = cu*cu;
				}


		if( tmp >= dis )	continue;

		dis = tmp;


		*u = cu;
		*t = ct;
		*sl = l;
		ret = i;
	}

	return( ret );
}



int
links_distance_n3( vec2d *ctr, link_type  *link, link_type *elink, vec2d  *v,
		link_type **sl, float *u, float *t )
{
link_type	*l,	*el;
width_type	*width;
int         i,	ret;
vec2d       cv;
int	b_flag,	e_flag;
float	cu,	abs_u,	abs_cu,	ct,	abs_ct;
float	tu0,	tu1;


	*u = abs_u = 100;
	*t = 100;

	cv.x = v->x - ctr->x;
	cv.y = v->y - ctr->y;

	l = link;
	width = l->width[F_BEGIN];
	b_flag = ( cv.x *width->bes.x + cv.y *width->bes.y ) > 0 ;

	tu0 = cv.x *width->bes.y - cv.y *width->bes.x;


	if( b_flag == 0 ){
		*u = l->u.x * cv.x + l->u.y * cv.y;
		*t = width->bes.x * cv.x + width->bes.y * cv.y;

		abs_cu = ABS(*u);
		abs_ct = ABS(*t);
		abs_u = MAX( abs_cu, abs_ct );
		*sl = link;
		ret = -1;
	}


	for( l = link, i = 0 ; l != elink ; el = l, l = LINK_NEXT(l), i++ ){

		cv.x -= l->v.x;
		cv.y -= l->v.y;

		width = l->width[F_END];
		e_flag = ( cv.x *width->bes.x + cv.y *width->bes.y ) > 0 ;
		tu1 = cv.x *width->bes.y - cv.y *width->bes.x;

		if( b_flag == 0 || e_flag == 1 ){
			b_flag = e_flag;
			tu0 = tu1;
			continue;
		}

		cu = l->u.x * cv.x + l->u.y * cv.y;


		ct = -l->u.y * cv.x + l->u.x * cv.y + l->len;

		if( ct < 0 ){
			if( ABS(tu0 ) > ABS(cu) )        cu = tu0;
			ct = 0;
		}
		else	if( ct > l->len ){
				if( ABS(tu1 ) > ABS(cu) )        cu = tu1;
				ct = l->len;
			}
			else	cu -= l->c_prb*ct *(ct - l->len );


		abs_cu = ABS( cu );

                if( abs_cu > abs_u ){
                        b_flag = e_flag;
			tu0 = tu1;
                        continue;
                }


		*u = cu;
		*t = ct;
		*sl = l;
		ret = i;

		abs_u = abs_cu;

		b_flag = e_flag;
	}


	if( b_flag == 0 )	return( ret );

	cu = el->u.x * cv.x + el->u.y * cv.y;
	width = el->width[F_END];
	ct = width->bes.x * cv.x + width->bes.y * cv.y;

	abs_cu = ABS(cu);
	abs_ct = ABS(ct);
	abs_cu = MAX( abs_cu, abs_ct );

	if( abs_cu > abs_u )	return(ret);  

	*u = cu;
	*t = ct + el->len;
	*sl = el;
	ret = -2;

	return( ret );
}
