/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**********************
 ***	N3Tool.c    ***
 **********************/
#include	<stdio.h>
#include        "Uigp/igp.h"


#include        "GpType/N3Type.h"


void
n3_afine( n3_type *e0, n3_type *e1, float t, n3_type *e )
{
        e->width[0] = t* e0->width[0] + (1-t)*e1->width[0];
        e->width[1] = t* e0->width[1] + (1-t)*e1->width[1];
 

        e->side[0].val[SP_Y] = t* e0->side[0].val[SP_Y] + (1-t)*e1->side[0].val[SP_Y];
        e->side[1].val[SP_Y] = t* e0->side[1].val[SP_Y] + (1-t)*e1->side[1].val[SP_Y];
        e->mval[SP_Y] = t* e0->mval[SP_Y] + (1-t)*e1->mval[SP_Y];
 
        e->side[0].val[SP_Q] = t* e0->side[0].val[SP_Q] + (1-t)*e1->side[0].val[SP_Q];
        e->side[1].val[SP_Q] = t* e0->side[1].val[SP_Q] + (1-t)*e1->side[1].val[SP_Q];
        e->mval[SP_Q] = t* e0->mval[SP_Q] + (1-t)*e1->mval[SP_Q];
 
        e->side[0].val[SP_A] = t* e0->side[0].val[SP_A] + (1-t)*e1->side[0].val[SP_A];
        e->side[1].val[SP_A] = t* e0->side[1].val[SP_A] + (1-t)*e1->side[1].val[SP_A];
        e->mval[SP_A] = t* e0->mval[SP_A] + (1-t)*e1->mval[SP_A];


        e->side[0].bval[SP_Y] = t* e0->side[0].bval[SP_Y] + (1-t)*e1->side[0].bval[SP_Y];
        e->side[1].bval[SP_Y] = t* e0->side[1].bval[SP_Y] + (1-t)*e1->side[1].bval[SP_Y];
 
        e->side[0].bval[SP_Q] = t* e0->side[0].bval[SP_Q] + (1-t)*e1->side[0].bval[SP_Q];
        e->side[1].bval[SP_Q] = t* e0->side[1].bval[SP_Q] + (1-t)*e1->side[1].bval[SP_Q];
 
        e->side[0].bval[SP_A] = t* e0->side[0].bval[SP_A] + (1-t)*e1->side[0].bval[SP_A];
        e->side[1].bval[SP_A] = t* e0->side[1].bval[SP_A] + (1-t)*e1->side[1].bval[SP_A];
}



void
n3_copy_inverse( n3_type *se, n3_type *e )
{
	e->depth = se->depth;

	e->width[F_RIGHT] = se->width[F_LEFT];
	e->width[F_LEFT] = se->width[F_RIGHT];

	e->mval[SP_Y] = se->mval[SP_Y];
	e->mval[SP_Q] = se->mval[SP_Q];
	e->mval[SP_A] = se->mval[SP_A];

	e->side[F_RIGHT].val[SP_Y] = se->side[F_LEFT].val[SP_Y];
	e->side[F_LEFT].val[SP_Y] = se->side[F_RIGHT].val[SP_Y];

	e->side[F_RIGHT].val[SP_Q] = se->side[F_LEFT].val[SP_Q];
	e->side[F_LEFT].val[SP_Q] = se->side[F_RIGHT].val[SP_Q];

	e->side[F_RIGHT].val[SP_A] = se->side[F_LEFT].val[SP_A];
	e->side[F_LEFT].val[SP_A] = se->side[F_RIGHT].val[SP_A];



	e->side[F_RIGHT].bval[SP_Y] = se->side[F_LEFT].bval[SP_Y];
	e->side[F_LEFT].bval[SP_Y] = se->side[F_RIGHT].bval[SP_Y];

	e->side[F_RIGHT].bval[SP_Q] = se->side[F_LEFT].bval[SP_Q];
	e->side[F_LEFT].bval[SP_Q] = se->side[F_RIGHT].bval[SP_Q];

	e->side[F_RIGHT].bval[SP_A] = se->side[F_LEFT].bval[SP_A];
	e->side[F_LEFT].bval[SP_A] = se->side[F_RIGHT].bval[SP_A];
}


void
n3_inverse( n3_type *e )
{
float	tmp;

	tmp = e->width[F_RIGHT];
	e->width[F_RIGHT] = e->width[F_LEFT];
	e->width[F_LEFT] = tmp;


	tmp = e->side[F_RIGHT].val[SP_Y];
	e->side[F_RIGHT].val[SP_Y] = e->side[F_LEFT].val[SP_Y];
	e->side[F_LEFT].val[SP_Y] = tmp;

	tmp = e->side[F_RIGHT].val[SP_Q];
	e->side[F_RIGHT].val[SP_Q] = e->side[F_LEFT].val[SP_Q];
	e->side[F_LEFT].val[SP_Q] = tmp;

	tmp = e->side[F_RIGHT].val[SP_A];
	e->side[F_RIGHT].val[SP_A] = e->side[F_LEFT].val[SP_A];
	e->side[F_LEFT].val[SP_A] = tmp;


	tmp = e->side[F_RIGHT].bval[SP_Y];
	e->side[F_RIGHT].bval[SP_Y] = e->side[F_LEFT].bval[SP_Y];
	e->side[F_LEFT].bval[SP_Y] = tmp;

	tmp = e->side[F_RIGHT].bval[SP_Q];
	e->side[F_RIGHT].bval[SP_Q] = e->side[F_LEFT].bval[SP_Q];
	e->side[F_LEFT].bval[SP_Q] = tmp;

	tmp = e->side[F_RIGHT].bval[SP_A];
	e->side[F_RIGHT].bval[SP_A] = e->side[F_LEFT].bval[SP_A];
	e->side[F_LEFT].bval[SP_A] = tmp;
}
