/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/***************************
 ***	ProfileTool.c    ***
 ***************************/
#include	<stdio.h>
#ifndef __SYMBIAN32__
	#include	<malloc.h>
	#include	<memory.h>	
#endif // 	__SYMBIAN32__
#include	"Uigp/igp.h"


#include	"GpType/NfType.h"
#include	"GpType/ProfileType.h"


profile_type
profile_alloc( int type )
{
profile_type	pr;

	pr.p = NULL;

	switch( type ){
	case NF_N3_P:
	case NF_EDGE:
		pr.edge = EDGE_CALLOC();
	    break;
	case NF_N3:
		pr.n3 = N3_CALLOC();
	    break;
	case NF_N3R:
	case NF_RIDGE:
		pr.ridge = RIDGE_ALLOC();
	    break;
	}

	if( pr.p != NULL )
		pr.p->depth = 1.0;

	return( pr );
}



profile_type
profile_make_copy( profile_type ppr, int type )
{
profile_type	pr;

	switch( type ){
	case NF_N3_P:
	case NF_EDGE:
		pr.edge = EDGE_ALLOC();
		*pr.edge = *ppr.edge;
	    break;
	case NF_N3:
		pr.n3 = N3_ALLOC();
		*pr.n3 = *ppr.n3;
	    break;
	case NF_N3R:
	case NF_RIDGE:
		pr.ridge = RIDGE_ALLOC();
		*pr.ridge = *ppr.ridge;
	    break;
	}
				

	return( pr );
}


void
profile_copy( int type, profile_type spr, profile_type tpr )
{
	switch( type ){
	case NF_EDGE:
	case NF_N3_P:
		*tpr.edge = *spr.edge;
	    break;
	case NF_N3R:
	case NF_RIDGE:
		*tpr.ridge = *spr.ridge;
	    break;
	case NF_N3:
		*tpr.n3 = *spr.n3;
	    break;
	}
}


void
profile_copy_inverse( int type, profile_type spr, profile_type tpr )
{
	switch( type ){
	case NF_EDGE:
	case NF_N3_P:
		edge_copy_inverse( spr.edge, tpr.edge );
	    break;
	case NF_N3R:
	case NF_RIDGE:
		*tpr.ridge = *spr.ridge;
	    break;
	case NF_N3:
		n3_copy_inverse( spr.n3, tpr.n3 );
	    break;
	}
}

void
profile_inverse( int type, profile_type pr )
{
	switch( type ){
	case NF_EDGE:
	case NF_N3_P:
		edge_inverse( pr.edge );
	    break;
	case NF_N3R:
	case NF_RIDGE:
	    break;
	case NF_N3:
		n3_inverse( pr.n3 );
	    break;
	}
}





void
profile_afine( int type, profile_type pr0, profile_type pr1,
						float t, profile_type pr )
{
	pr.p->depth = t* pr0.p->depth + (1-t)*pr1.p->depth;

	switch( type ){
	case NF_EDGE:
	case NF_N3_P:
		edge_afine( pr0.edge, pr1.edge, t, pr.edge );
	    break;
	case NF_N3R:
	case NF_RIDGE:
		ridge_afine( pr0.ridge, pr1.ridge, t, pr.ridge );
	    break;
	case NF_N3:
		n3_afine( pr0.n3, pr1.n3, t, pr.n3 );
	    break;
	}
}
