/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/************************
 ***    LinkType.h    ***
 ************************/

#ifndef         LINK_TYPE
#define         LINK_TYPE

#include	"NfType.h"

#include	"ProfileType.h"
#include	"WidthType.h"

#include	"Uvec/Vec2d.h"
#include	"Ubox/Box2d.h"

/*
inc	"PlType.h"
*/




typedef struct link_ptr {
	int	bno;
	union	{
		struct link_type *l;
		struct jnt_type *jnt;
	} p; 
}	link_ptr;


typedef struct in_type {
	float	t;

	u_char	type;
	u_char	direct;

	u_char	use;

	s_char	bno;
	struct jnt_type *jnt;

	struct in_type	*p;
}	in_type;


	/* in.type */
#define		IN_NULL		0
#define		IN_N3		1
#define		IN_EDGE		2


	


typedef struct link_type {
	
	
	
    //! type of the link    
	int	type;

	
    //! Begin point of the link 
	vec2d	ctr;

	
    //! Direction vector of the link 
	vec2d	v;		/* len* ( cos(angle) , sin(angle) )	*/
	
    //! maximum of the parabola 
	float	a;		/* maximal height of parabola */


/*	AUX	*/
	
    
    //! length of the link on v 
	float	len;

	
    
    //! right normal to v 
	vec2d	u;		
	
    
    //! 2-nd order coefficient of parabola
	float   c_prb;		



	width_type	*width[2];	/* F_BEGIN and F_END */


	profile_type	pr[2];		/* F_BEGIN and F_END */


	in_type		*in[2];		/* F_RIGHT and F_LEFT */


	
    
    //! pointer to prev and next links/joints 
	link_ptr	p[2];		/* F_BEGIN and F_END */



	/* for Depth */
	u_char	br,	Fbr,	br1;
	u_char	side;



	struct link_type	*nl;	/* next link in the same gp_type cell*/

	struct link_type        *cln;	/* for xp_type cell */

	/* Rendering AUX */
	struct link_type	*rnl;
	struct pl_type	*pl[2];		/* F_BEGIN and F_END */
	float	ra;
	vec2d	rctr;
	vec2d	rv;
} link_type;




#define		LINK_ALLOC()	\
			( (link_type *)malloc( sizeof( link_type ) ) )


#define		IN_TYPE_ALLOC()	\
			( (in_type *)malloc( sizeof( in_type ) ) )



#define		LINK_NEXT( link )	\
		(( link->p[F_END].bno != -1 )? NULL : link->p[F_END].p.l )

#define		LINK_PREV( link )	\
		(( link->p[F_BEGIN].bno != -1 )? NULL : link->p[F_BEGIN].p.l )


#define		LINK_NEXT_JNT( link )	\
		(( link->p[F_END].bno < 0 )? NULL : link->p[F_END].p.jnt )

#define		LINK_PREV_JNT( link )	\
		(( link->p[F_BEGIN].bno < 0 )? NULL : link->p[F_BEGIN].p.jnt )


link_type *	link_alloc( int type, int ino );

link_type *	link_1_alloc();

void		link_alloc_width( link_type *link );

link_type *	link_make_copy( link_type *plink, int type );

void		link_free( link_type *link );
void		link_1_free( link_type *l );



void	link_width_bes( link_type *link );

void	link_t2xy( vec2d *vm, link_type *link, float t, vec2d *p );


void	link_set_aux( link_type *l );



void	link_tanget( link_type *l, float t, vec2d *tanget );

void	link_Ntanget_beg( link_type *l, vec2d *t );
void	link_Ntanget_end( link_type *l, vec2d *t );


	/* point with tanget ( 1, 0 ), if exist, set p and return 1 else 0 */
int	link_tanget_point_10( link_type *l, vec2d *vm, float *t, vec2d *p );

int	link_tanget_point_01( link_type *l, vec2d *vm, float *t, vec2d *p );

void	link_box2d( link_type *l, box2d *b );

void	polylink_box2d(link_type *l, box2d *b, link_type **lx0, link_type **lx1, link_type **ly0, link_type **ly1);


link_type *	link_inverse( link_type *link );
void	link_1_inverse( link_type *l );

void	link_set_br( link_type *link, int br, int Fbr );




	/* LinkTool1.c */
 
int	links_distance( vec2d *ctr, link_type  *link, link_type *elink,
			vec2d  *p,
			link_type **sl, float *u, float *t );

int	links_distance_n3( vec2d *ctr, link_type  *link, link_type *elink,
			vec2d  *p,
			link_type **sl, float *u, float *t );




	/* NfAux.o */
void	link_set_geo_aux( link_type *link );
void	link_1_set_geo_aux( link_type *link );


	/* LinkIn.c */
void	link_in_reposition( link_type *l0, link_type *l1 );

void	link_in_get( link_type *link, int side, link_type *ln[2] );

void	link_delete_in( link_type *link, int side );


	/* LinkSplit.c */
link_type *	link_split( vec2d *ctr, link_type *link, vec2d *p );

#endif
