/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/*************************** 
 ***    LinkFromLine.c   ***
 ***************************/
#include	<stdio.h>
#include	<math.h>

#include	"Uigp/igp.h"

#include	"GpType/GpType.h"

#include	"AgpEdit/AgpEditTool.h"

#include	"LinkTool.h"




float
polylink_length(link_type *link, link_type *endLink)
{
link_type	*l;
float length = 0.0F;

	
	for ( l = link; l != NULL && l != endLink ; l = LINK_NEXT(l))
	{
		length += l->len;
	}	
	if( endLink != NULL )
		length += endLink->len;

	return length;
}


int
polylink_link_no(link_type *link, link_type *endLink )
{
link_type	*l;
int	no;

	for ( l = link, no = 0; l != NULL && l != endLink ; l = LINK_NEXT(l), no++ );

	if( endLink != NULL && endLink == l)	no += 1;

	return no;
}



/*
link_type * 
polylink_max_len( link_type *link, float max_len )
{
link_type	*l,	*l0,	*l1,	*tl;
vec2d	p;



	for( l = link ; l != NULL ; l = LINK_NEXT(l) ){

		if( l->len < max_len  )
			continue;


		if( ABS(l->a) < 0.01 ){ 
			p.x = l->ctr.x + l->v.x;
			p.y = l->ctr.y + l->v.y;
			l0 = links_from_line( &l->ctr, &p, max_len, l->type, &l1 );
		}
		else	l0 = link_reapproximate( &l->ctr, l, max_len, &l1 );


		for( tl = l0, p = l->ctr ; tl != l1 ; tl = LINK_NEXT(tl) ){
			p.x += tl->v.x;
			p.y += tl->v.y;
		}

		l1->v.x = l->ctr.x + l->v.x - p.x;
		l1->v.y = l->ctr.y + l->v.y - p.y;
		link_set_aux( l1 );



		l0->p[0] = l->p[0];
		l1->p[1] = l->p[1];

		if( LINK_PREV(l0) != NULL )
			l0->p[0].p.l->p[1].p.l = l0;

		if( LINK_NEXT(l1) != NULL )
			l1->p[1].p.l->p[0].p.l = l1;

		if( l == link ) link = l0;

		free( l );

		l = l1;
	}

	return( link );
}




link_type * 
link_reapproximate( vec2d *ctr,link_type *link, float max_len, link_type **last )
{
gapp_prm	Qgprm;
gapp_type *gapp;
link_type *l,	*l1;



	Qgprm.dwight = 0.10F;
	Qgprm.err = 0.01F;

	Qgprm.min_len = 3.0F;
	Qgprm.Min_pnumber = 10;
	Qgprm.max_len = 24.0F;

	Qgprm.Max_dev = 1.0F;
	Qgprm.Min_dev = 0.1F;



	gapp = polylink_to_gapp( ctr, link, link );

	geo_approximate( gapp, &Qgprm );

	
	l = link_from_gapp( gapp, NF_EDGE, last );

	
	l->ctr = gapp->v[0];

	gapp_destroy( gapp );


	if( *last != NULL ){
		for( l1 = l ; LINK_NEXT(l1) ; l1 = LINK_NEXT(l1) );
		*last = l1;
	}

	return( l );
}
*/





link_type * 
links_from_line( vec2d *p0, vec2d *p1, float max_dist,
				int type, link_type **last_link )
{
link_type *link, *l;
vec2d	v,	u,	ctr;
float rest_dist,	len;
int	i,	no;

	v.x = p1->x - p0->x;
	v.y = p1->y - p0->y;
	rest_dist = (float)hypot( v.x, v.y );

	no = ceil(rest_dist / max_dist);
	len = (float)(rest_dist / no );
		

	u.x = v.y / rest_dist; 
	u.y = - v.x / rest_dist;

	v.x = len * -u.y;
	v.y = len * u.x;


	link = link_alloc( type, no );



	ctr = *p0;
	for( i = 0, l = link ; i < no ; i++, l = LINK_NEXT(l) ){

		l->ctr = ctr;

		l->v = v;
		l->len = len;
		l->a = l->c_prb = 0;
		l->u = u;


		if( i == no-1 ){
			l->v.x = p1->x - l->ctr.x;
			l->v.y = p1->y - l->ctr.y;

			l->len = l->v.x * -u.y + l->v.y * u.x;

			if( last_link != NULL )	*last_link = l;
		}


		ctr.x += l->v.x;
		ctr.y += l->v.y;
	}


	return link;
}



link_type *
link_from_points( vec2d p[], int no, int Fclose )
{
link_type	*link,	*l;
vec2d	v;
int	i,	n;

	link = link_alloc( NF_EDGE, no );


	v.x = p[0].x;
	v.y = p[0].y;


	n = no;
	n = ( Fclose == 1 )? no -1 : no;

	for( i = 0, l = link ; i < n ; i++, l = LINK_NEXT(l) ){
		l->v.x = p[i+1].x - v.x;
		l->v.y = p[i+1].y - v.y;

		l->a = 0;

		link_set_aux( l );

		v.x += l->v.x;
		v.y += l->v.y;
	}


	if( Fclose == 1 ){
		l->v.x = p[0].x - v.x;
		l->v.y = p[0].y - v.y;

		l->a = 0;

		link_set_aux( l );
	}

	return( link );
}


