/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/************************
 ***   LinkUpdate.c   ***
 ************************/
#include	<math.h>
#include	"Uigp/igp.h"

#include	"GpType/GpType.h"


void
link_set_begin( vec2d *ctr, link_type *link, vec2d *p )  
{
float newLen, oldLen, a;


	a = link->a;
	oldLen = link->len;
 

	link->v.x = ctr->x + link->v.x - p->x;   
	link->v.y = ctr->y + link->v.y - p->y;
	*ctr = *p; 


	newLen = hypot( link->v.x, link->v.y );
	link->a = link->a * ( newLen / oldLen ) ;


	if( ABS(link->a) > 0.25*newLen  )
	{
		if( link->a < 0 )
				link->a = -0.25*newLen ;
		else	link->a = 0.25*newLen ;
	}
	
	link_set_aux( link );
}



void
link_set_end( vec2d *ctr, link_type *link, vec2d *p )  
{

float newLen, oldLen, a;
  
	a = link->a;
	oldLen = link->len;
 

	link->v.x = p->x - ctr->x;
	link->v.y = p->y - ctr->y;

	newLen = hypot( link->v.x, link->v.y );
	link->a = link->a * ( newLen / oldLen ) ;


	if( ABS(link->a) > 0.25*newLen  )
	{
		if( link->a < 0 )
				link->a = -0.25*newLen ;
		else	link->a = 0.25*newLen ;
	}
	
	link_set_aux( link );
}


void
link_set_a_point( vec2d *ctr, link_type *l, vec2d *p )
{
vec2d   dv;
float   t,      u;

	dv.x = p->x - ctr->x;
	dv.y = p->y - ctr->y;

	u = dv.x * l->u.x + dv.y * l->u.y;
	t = -l->u.y * dv.x + l->u.x * dv.y;
	if( t <= 0 || t >= l->len )     return;

	l->c_prb = u /( t * (t - l->len) );

	l->a = -l->c_prb * ( l->len*l->len)/4.0F;

	if( ABS(l->a) > 0.25*l->len )
	{
		if( l->a < 0 )
				l->a = -0.25*l->len;
		else	l->a = 0.25*l->len;
	}

	l->c_prb = -4.0F *l->a/( l->len*l->len);
}


void
link_set_a( vec2d *ctr, link_type *l, float d )
{

	l->a = d;


	if( ABS(l->a) > 0.25*l->len )
	{
		if( l->a < 0 )
				l->a = -0.25*l->len;
		else	l->a = 0.25*l->len;
	}



	l->c_prb = -4.0F *l->a/( l->len*l->len);
}