/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
#include	"GpType/LinkType.h"
#include	"LinkTool.h"


void
link_connect( link_type *l0, link_type *l1 )
{
	l0->p[1].p.l = l1;
	l1->p[0].p.l = l0;

	l0->p[1].bno = l1->p[0].bno = -1;
}


link_type *
polylink_get_last_link(link_type *link)
{
	link_type *l= link;	
	while( link )
	{
		l = link;
		link = LINK_NEXT(link);
	}
    return l;	
}


link_type *
polylink_get_first_link(link_type *link)
{
	link_type *l= link;	
	while( link )
	{
		l = link;
		link = LINK_PREV(link);
	}
    return l;	
}

void
polylink_tanget_end( link_type *link, vec2d *tanget )
{
link_type	*l;

	for( l = link ; LINK_NEXT(l) != NULL ; l = LINK_NEXT(l) );

	link_tanget( l, l->len, tanget );
}


void	
polylink_set_ctr( vec2d *ctr, link_type *link)
{
link_type *l;
vec2d v;

	for( l = link, v = *ctr ; l != NULL ; l = LINK_NEXT(l) ){
		l->ctr = v;

		v.x += l->v.x;
		v.y += l->v.y;
	}
}




void	link_set_gCtr(link_type *l, vec2d *gCtr)
{
	link_type *l1;
	vec2d ctr;
	for ( l1 = l , ctr = *gCtr; l1 != NULL ; l1 = LINK_NEXT(l1))
	{
		l1->ctr = ctr;
		ctr.x += l1->v.x;
		ctr.y += l1->v.y;
	}
}

void	link_set_type(link_type *link, int l_type)
{
	for ( ; link != NULL ; link = LINK_NEXT(link))
	{		
		link->type = l_type ;
	}
	
}



/**** link_end_point ****/
/*!
 * returns the end point of the link itself ( the start point of next link)
 *
 * @param *l : 
 * @param *v : 
 *
 * @return void  : 
 */
void link_end_point(link_type *l, vec2d *v)
{
	if ( l == NULL ) 
	{
		v->x = 0;	
		v->y = 0;
	}
	v->x = l->ctr.x + l->v.x;
	v->y = l->ctr.y + l->v.y;
}






/**** link_set_prev_next ****/
/*!
 * sets second as next of first, and first as prev of second
 *
 * @param *first : 
 * @param *second : 
 *
 * @return void  : 
 */
void link_set_prev_next(link_type *first, link_type *second)
{
	link_type *prev = NULL, *next = NULL;
	if ( first != NULL ) 
	{
		//disconnect from old next
		next = LINK_NEXT(first);		
		// set new next
		first->p[F_END].p.l = second;
		first->p[F_END].bno = -1;
	}
	if ( second != NULL ) 
	{
		//disconnect from old prev
		prev = LINK_PREV(second);		
		// set new prev
		second->p[F_BEGIN].bno = -1;
		second->p[F_BEGIN].p.l = first;
	}

	if ( next != NULL ) 
		next->p[F_BEGIN].p.l = NULL;
	if ( prev != NULL )
		prev->p[F_END].p.l = NULL;
		
}



link_type *
polylink_copy( link_type *plink, link_type *elink ) 
{ 
link_type	*link,	*l,	*pl,	*el;

	pl = l = NULL;
 
	for( el = plink ; el != NULL  && el != elink;  el = LINK_NEXT(el) ){
		
		l = LINK_ALLOC();
		*l = *el;

		if( pl != NULL ){
			l->p[F_BEGIN].p.l = pl;
			pl->p[F_END].p.l = l;
		}
		else	link = l;

		l->in[F_RIGHT] = l->in[F_LEFT] = NULL;

		l->pr[0].p = l->pr[1].p = NULL;

		pl = l;
	}


	link->p[0].p.l = NULL;
	link->p[0].bno = -2;

	pl->p[1].p.l = NULL;
	pl->p[1].bno = -2;

 
	return( link ); 
}
