/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**************************
 ***	N3Prime.c	***
 *************************/

#include	<math.h>

#include	"GpType/GpType.h"

#include	"N3Prime.h"

#include	"N3PrimeIn.h"

static void	n3_prime_JNT_N3( gp_array *agp, jnt_type *jnt );

static void	n3_prime_JNT_RN3( gp_array *agp, jnt_type *jnt );

static void	n3_prime_JNT_N3_link( jnt_type *jnt );
static void	n3_prime_JNT_RN3_link( jnt_type *jnt );


static void	agp_n3_prime_link( jnt_type *jnt, int bno, int side );

static void	n3_prime_JNT_N3_link1( jnt_type *jnt );

static int	get_next_in( vec2d *ctr0, link_type *link0, in_type *in0,
			int side,
			vec2d *ctr1, link_type **link1, in_type **in1 );


static void	n3_prime_JNT_N3_P_aux( gp_array *agp, jnt_type *jnt );


static void	n3_prime_jnt_in_degenric( gp_array *agp, jnt_type *jnt, int i );



void
n3_prime_agp( gp_array *agp )
{
gp_type	*gp;
jnt_type	*jnt;
int	i,	j;


	if( agp->Fn3_prime == 1 )	return;

	n3_prime_in_null( agp );

	for( i = 0 ; i < agp->Ni ; i++ )
		for( j = 0 ; j < agp->Nj ; j++ ){
			gp = &agp->gp[i][j];

			for( jnt = gp->jnt ; jnt != NULL ; jnt = jnt->nj ){
				if( jnt->type == JNT_N3 ){
					n3_prime_JNT_N3( agp, jnt );
					continue;
				}

				if( jnt->type == JNT_RN3 ){
					n3_prime_JNT_RN3( agp, jnt );
					continue;
				}
			}
		}


	for( i = 0 ; i < agp->Ni ; i++ )
		for( j = 0 ; j < agp->Nj ; j++ ){
			gp = &agp->gp[i][j];

			for( jnt = gp->jnt ; jnt != NULL ; jnt = jnt->nj ){
				if( jnt->type != JNT_N3 )	continue;

				if( jnt->brn[1]->type == BRN_IN )
					n3_prime_jnt_in_degenric( agp, jnt, 1 );

				if( jnt->brn[2]->type == BRN_IN )
					n3_prime_jnt_in_degenric( agp, jnt, 2 );
			}
		}






	for( i = 0 ; i < agp->Ni ; i++ )
		for( j = 0 ; j < agp->Nj ; j++ ){
			gp = &agp->gp[i][j];

			for( jnt = gp->jnt ; jnt != NULL ; jnt = jnt->nj ){
				if( jnt->type == JNT_N3 ){
					n3_prime_JNT_N3_link( jnt );

					n3_prime_JNT_N3_link1( jnt );
					continue;
				}

				if( jnt->type == JNT_RN3 ){
					n3_prime_JNT_RN3_link( jnt );

					n3_prime_JNT_N3_link1( jnt );
					continue;
				}
			}
		}




	/* Lnf */
	for( i = 0 ; i < agp->Ni ; i++ )
		for( j = 0 ; j < agp->Nj ; j++ ){
			gp = &agp->gp[i][j];

			for( jnt = gp->jnt ; jnt != NULL ; jnt = jnt->nj ){
				if( jnt->type != JNT_N3 && jnt->type !=JNT_RN3)
					continue;

				if( jnt->pj[0] != NULL )
					n3_prime_JNT_N3_P_aux( agp, jnt->pj[0]);

				if( jnt->pj[1] != NULL )
					n3_prime_JNT_N3_P_aux( agp, jnt->pj[1]);
			}
		}


	agp->Fn3_prime = 1;
}



static void
n3_prime_JNT_N3_P_aux( gp_array *agp, jnt_type *jnt )
{
gp_type	*gp;
brn_type	*b;
int	k;
int	i,	j;

	jnt_edge_set_aux( jnt );


	for( k = 0 ; k <  jnt->brn_no ; k++ ){
		b = jnt->brn[k];
		if( b->direction != F_BEGIN )	continue;
		if( b->type != BRN_N3_P )	continue;
		agp_lnf_link( &jnt->gctr, b->link, agp );
	}




    i = (jnt->gctr.x+agp->bgp) * agp->inv_bgp;
    j = (jnt->gctr.y+agp->bgp) * agp->inv_bgp;



    gp = &agp->gp[i][j];

    jnt->ctr.x = jnt->gctr.x - gp->ctr.x;
    jnt->ctr.y = jnt->gctr.y - gp->ctr.y;

    jnt->nj = gp->jnt;
    gp->jnt = jnt;
}





static void
n3_prime_JNT_N3( gp_array *agp, jnt_type *jnt )
{
jnt_type	*pj;
brn_type	*b,	*pb;

	b = jnt->brn[1];
	

	if( b->type == BRN_IN )	b->in->use = 0;

	if( (b->type != BRN_IN || b->in->type != IN_NULL) &&
	 	!(jnt->brn[0]->type != BRN_N3 && b->type == BRN_NULL) ){
		pj = jnt->pj[0] = jnt_alloc();

		pj->type = JNT_N3_P;
        	pj->gctr.x = jnt->gctr.x + jnt->rwidth * jnt->u.x;
        	pj->gctr.y = jnt->gctr.y + jnt->rwidth * jnt->u.y;

		b->pj = pj;

		if( b->type == BRN_EDGE ){
			pb = pj->brn[pj->brn_no] = brn_alloc();

			*pb = *b;

			b->link->p[ b->direction].p.jnt = pj;
			b->link->p[ b->direction].bno = pj->brn_no;

			pb->ctr.x = pb->ctr.y = 0;
			pj->brn_no++;
		}
	}
	else	jnt->pj[0] = b->pj = NULL;


	b = jnt->brn[2];
	if( b->type == BRN_IN )	b->in->use = 0;

	if( (b->type != BRN_IN || b->in->type != IN_NULL) &&
	 	!(jnt->brn[0]->type != BRN_N3 && b->type == BRN_NULL) ){
		pj = jnt->pj[1] = jnt_alloc();

		pj->type = JNT_N3_P;
        	pj->gctr.x = jnt->gctr.x - jnt->lwidth * jnt->u.x;
        	pj->gctr.y = jnt->gctr.y - jnt->lwidth * jnt->u.y;

		b->pj = pj;

		if( b->type == BRN_EDGE ){
			pb = pj->brn[pj->brn_no] = brn_alloc();

			*pb = *b;

			b->link->p[ b->direction].p.jnt = pj;
			b->link->p[ b->direction].bno = pj->brn_no;

			pb->ctr.x = pb->ctr.y = 0;
			pj->brn_no++;
		}
	}
	else	jnt->pj[1] = b->pj = NULL;
}




static void
n3_prime_JNT_RN3( gp_array *agp, jnt_type *jnt )
{
jnt_type	*pj;

	jnt->pj[0] = jnt->pj[1] = NULL;

/*
	if( jnt->brn_no == 1 && jnt->brn[0]->type != NF_N3 )	return;
*/
	if( jnt->brn[0]->type != NF_N3 ){
		if( jnt->brn_no == 1 )	return;
		if( jnt->brn[1]->type != NF_N3 )	return;
	}

	pj = jnt->pj[0] = jnt_alloc();

	pj->type = JNT_N3_P;
        pj->gctr.x = jnt->gctr.x + jnt->rwidth * jnt->u.x;
        pj->gctr.y = jnt->gctr.y + jnt->rwidth * jnt->u.y;


	pj = jnt->pj[1] = jnt_alloc();

	pj->type = JNT_N3_P;
        pj->gctr.x = jnt->gctr.x - jnt->lwidth * jnt->u.x;
        pj->gctr.y = jnt->gctr.y - jnt->lwidth * jnt->u.y;
}



static void
n3_prime_JNT_N3_link1( jnt_type *jnt )
{
jnt_type	*jnt0,	*jnt1;
link_type	*link;
float	len_inv;
int	flag;

	if( jnt->type == JNT_RN3 && jnt->brn_no == 2 )	return;


	if( jnt->type == JNT_N3 && jnt->brn[1]->type != BRN_NULL &&
				jnt->brn[2]->type != BRN_NULL &&
				jnt->brn[0]->type == BRN_N3 )
		return;

	jnt0 = jnt->pj[0];
        if( jnt0 == NULL && jnt->type == JNT_N3 )
		jnt0 = jnt->brn[1]->pj;

	jnt1 = jnt->pj[1];
        if( jnt1 == NULL && jnt->type == JNT_N3 )
		jnt1 = jnt->brn[2]->pj;


	if( jnt0 == NULL || jnt1 == NULL )	return;


	link = LINK_ALLOC();

        link->p[F_BEGIN].p.l = link->p[F_END].p.l = NULL;
        link->p[F_BEGIN].bno = link->p[F_END].bno = -1;

        link->in[F_RIGHT] = link->in[F_LEFT] = NULL;
        link->width[F_BEGIN] = link->width[F_END] = NULL;
        link->pr[F_BEGIN].edge = link->pr[F_END].edge = NULL;

	link->type = NF_N3_P;
	link->ctr = jnt0->gctr;
        link->v.x = jnt1->gctr.x - jnt0->gctr.x;
        link->v.y = jnt1->gctr.y - jnt0->gctr.y;
        link->len = hypot( link->v.x, link->v.y );

        len_inv = 1.0/link->len;
        link->u.x =  link->v.y * len_inv;
        link->u.y = -link->v.x * len_inv;

        link->a = 0;
        link->c_prb = 0;

	n3_prime_link_connect_jnt( jnt0, link, F_BEGIN );
	n3_prime_link_connect_jnt( jnt1, link, F_END );


        if( jnt->brn[0]->link != NULL ){
                link->br = jnt->brn[0]->link->br;
                link->Fbr = ( jnt->brn[0]->link->Fbr == 0x03 )? 1 : 0;
		link->side = 4;
        }


	if( jnt->type == JNT_RN3 ){
		link->pr[0].edge = n3_prime_profile1( jnt, 0, F_LEFT, 0 );
		link->pr[1].edge = n3_prime_profile1( jnt, 0, F_RIGHT, 0 );
		return;
	}

	if( jnt->brn[0]->type == BRN_N3R ){
		link->pr[0].edge = n3_prime_profile11( jnt, 1, F_END );
		link->pr[1].edge = n3_prime_profile11( jnt, 2, F_BEGIN );
		return;
	}


	flag = jnt->brn[1]->type != BRN_NULL;
	link->pr[0].edge = n3_prime_profile1( jnt, 0, F_LEFT, flag );
	flag = jnt->brn[2]->type != BRN_NULL;
	link->pr[1].edge = n3_prime_profile1( jnt, 0, F_RIGHT, flag );
}




static void
n3_prime_JNT_N3_link( jnt_type *jnt )
{
brn_type	*b;

	b = jnt->brn[0];

	if( b->type == NF_N3 && b->direction == F_BEGIN ){
		agp_n3_prime_link( jnt, 0, F_LEFT );
		agp_n3_prime_link( jnt, 0, F_RIGHT );
	}
}



static void
n3_prime_JNT_RN3_link( jnt_type *jnt )
{
brn_type	*b;

	b = jnt->brn[0];
	if( b->type == NF_N3 && b->direction == F_BEGIN ){
		agp_n3_prime_link( jnt, 0, F_LEFT );
		agp_n3_prime_link( jnt, 0, F_RIGHT );
	}

	if( jnt->brn_no == 1 )	return;

	b = jnt->brn[1];
	if( b->type == NF_N3 && b->direction == F_BEGIN ){
		agp_n3_prime_link( jnt, 1, F_LEFT );
		agp_n3_prime_link( jnt, 1, F_RIGHT );
	}
}






static void
agp_n3_prime_link( jnt_type *jnt, int bno, int side )
{
jnt_type	*jnt0,	*jnt1,	*ejnt;
brn_type	*b;
link_type	*link0,	*link1;
in_type	*in0,	*in1;
vec2d	ctr0,	ctr1;
int	i;
in_type	in_t,	*in00,	*in11;

	b = jnt->brn[bno];

	if( b->type != NF_N3 || b->direction == F_END )	return;


	
	i = ( bno == 0 )? 1 - side : side;
	jnt0 = jnt->pj[i];

	in0 = NULL;
	if( jnt0 == NULL && jnt->type == JNT_N3 ){
		jnt0 = jnt->brn[i+1]->pj;
		if( jnt0 != NULL ){
			in_t.t = 0;
			in0 = &in_t;
			in0->p = b->link->in[side];
		}
	}


	in00 = ( jnt->type == JNT_N3 )? jnt->brn[i+1]->in : NULL;



	link0 = b->link;
	ctr0 = jnt->gctr;

	while( get_next_in( &ctr0, link0, in0, side, &ctr1, &link1, &in1 ) ){

		jnt1 = in1->jnt->brn[in1->bno]->pj;

		if( jnt0 == jnt1 ){
			ctr0 = ctr1;
			link0 = link1;
			in0 = in1;

			continue;
		}


		if( (in1->direct == F_END || in1->use == 0 ) && jnt0 != NULL ){

			n3_prime_link_seg( &ctr0, link0, side, in0,
                                link1, in1, 
                                jnt0, jnt1 );

			if( in1->direct != F_END )	in1->use = 1;
			if( in0 != NULL && in0->direct != F_BEGIN )
				in0->use = 1;

			if( in00 != NULL )	in00->use = 2;
		}


		in00 = NULL;

		ctr0 = ctr1;
		link0 = link1;
		in0 = in1;
		jnt0 = jnt1;
	}

	if( jnt0 == NULL )	return;



	ejnt = link1->p[F_END].p.jnt;
	bno = link1->p[F_END].bno;


	i = ( bno == 0 )? side :  1 - side;
	jnt1 = ejnt->pj[i];

	if( jnt1 == NULL && ejnt->type == JNT_N3 ){
		jnt1 = ejnt->brn[i+1]->pj;
		in_t.t = link1->len;
		in1 = &in_t;
	}
	else	in1 = NULL;


	if( jnt1 == NULL )	return;


	in11 = ( ejnt->type == JNT_N3 )? ejnt->brn[i+1]->in : NULL;
	if( in11 != NULL && in11->use != 0 )	return;



	if( jnt0 == jnt1 && (in0 != NULL || in1 != NULL ) )	return;

	n3_prime_link_seg( &ctr0, link0, side, in0,
                                link1, in1, 
                                jnt0, jnt1 );


	if( in0 != NULL && in0->direct != F_BEGIN )	in0->use = 1;
	if( in00 != NULL )	in00->use = 2;
	if( in11 != NULL )	in11->use = 2;
}






static int
get_next_in( vec2d *ctr0, link_type *link0, in_type *in0, int side,
		vec2d *ctr1, link_type **link1, in_type **in1 )
{
link_type	*l;
in_type	*in;

	for( l = link0, *ctr1 = *ctr0 ; l != NULL  ; l = LINK_NEXT(l) ){

		for( in = (l == link0 && in0 != NULL  )? in0->p : l->in[side] ;
			in != NULL && in->type == IN_NULL ; in = in->p );

		if( in != NULL ){
			*in1 = in;
			*link1 = l;
			return( 1 );
		}


		ctr1->x += l->v.x;
		ctr1->y += l->v.y;
		*link1 = l;
	}

	*in1 = NULL;
	return( 0 );

}



static void
n3_prime_jnt_in_degenric( gp_array *agp, jnt_type *jnt, int i )
{
jnt_type	*njnt;
brn_type	*b,	*nb;
int	bno,	ni,	i1,	nbno;

	b = jnt->brn[i];
	if( b->type != BRN_IN ) return;
	if( b->in->type == IN_NULL )	return;
	if( b->link->type != BRN_N3 )	return;

	if( b->direction == F_BEGIN ){
		if( LINK_NEXT(b->link) != NULL )	return;
		if( b->in->p != NULL )	return;

		if( b->in->t < b->link->len-0.25 )   return;

		njnt = b->link->p[1].p.jnt;
		nbno = b->link->p[1].bno;
	}
	else	{
		if( LINK_PREV(b->link) != NULL )	return;
		if( b->link->in[0] != b->in && b->link->in[1] != b->in )
			return;
		if( b->in->t > 0.25 )   return;

		njnt = b->link->p[0].p.jnt;
		nbno = b->link->p[0].bno;
	}

	bno = 3 - i;
	ni = bno -1;

	if( njnt->type == JNT_N3 ){
		nb = njnt->brn[bno];
		if( nb->type == BRN_IN && (nb->in->type & 0x08) )
			return;
	}

	if( nbno == 1 )	ni = 1 - ni;

	i1 = i - 1;

	free( jnt->pj[i1] );

	jnt->pj[i1] = NULL;
	jnt->brn[i]->pj = njnt->pj[ni];
	jnt->brn[i]->in->type |= 0x08;
}
