/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**************************
 ***	N3PrimeFree.c	***
 *************************/

#include	<math.h>

#include	"GpType/GpType.h"

#include	"N3PrimeIn.h"

static void	n3_prime_JNT_N3_free( jnt_type *jnt, gp_array *agp );
static void	n3_prime_JNT_RN3_free( jnt_type *jnt, gp_array *agp );


static void	n3_prime_JNT_N3_P_free( jnt_type *jnt, gp_array *agp );


void
n3_prime_free( gp_array *agp )
{
gp_type	*gp;
jnt_type	*jnt;
int	i,	j;

	if( agp->Fn3_prime == 0 )	return;

	for( i = 0 ; i < agp->Ni ; i++ )
		for( j = 0 ; j < agp->Nj ; j++ ){
			gp = &agp->gp[i][j];

			for( jnt = gp->jnt ; jnt != NULL ; jnt = jnt->nj ){
				if( jnt->type == JNT_N3 ){
					n3_prime_JNT_N3_free( jnt, agp );
					continue;
				}

				if( jnt->type == JNT_RN3 ){
					n3_prime_JNT_RN3_free( jnt, agp );
					continue;
				}
			}
		}

	agp->Fn3_prime = 0;
}



static void
n3_prime_JNT_N3_free( jnt_type *jnt, gp_array *agp )
{
brn_type	*b;

	b = jnt->brn[1];
	if( b->type == BRN_EDGE ){

		b->link->p[ b->direction].p.jnt = jnt;
		b->link->p[ b->direction].bno = 1;
	}


	b = jnt->brn[2];
	if( b->type == BRN_EDGE ){

		b->link->p[ b->direction].p.jnt = jnt;
		b->link->p[ b->direction].bno = 2;
	}



	b = jnt->brn[1];
	if( b->type == BRN_IN && (b->in->type & 0x08) ){
		b->in->type &= ~0x08;
		b->pj = NULL;
		jnt->pj[0] = NULL;
	}

	b = jnt->brn[2];
	if( b->type == BRN_IN && (b->in->type & 0x08) ){
		b->in->type &= ~0x08;
		b->pj = NULL;
		jnt->pj[1] = NULL;
	}




	if( jnt->pj[0] != NULL ){
		n3_prime_JNT_N3_P_free( jnt->pj[0], agp );
		jnt->pj[0] = NULL;
	}

	if( jnt->pj[1] != NULL ){
		n3_prime_JNT_N3_P_free( jnt->pj[1], agp );
		jnt->pj[1] = NULL;
	}
}


static void
n3_prime_JNT_RN3_free( jnt_type *jnt, gp_array *agp )
{
	if( jnt->pj[0] != NULL ){
		n3_prime_JNT_N3_P_free( jnt->pj[0], agp );
		jnt->pj[0] = NULL;
	}

	if( jnt->pj[1] != NULL ){
		n3_prime_JNT_N3_P_free( jnt->pj[1], agp );
		jnt->pj[1] = NULL;
	}
}







static void
n3_prime_JNT_N3_P_free( jnt_type *jnt, gp_array *agp )
{
brn_type	*b;
int	k;

	for( k = 0 ; k <  jnt->brn_no ; k++ ){
		b = jnt->brn[k];

		if( b->type == BRN_N3_P && b->direction == F_BEGIN ){
			agp_lnf_clear_link( &jnt->gctr, b->link, agp );
			link_free( b->link );
		}

		free( b );

	}

	agp_untied_jnt( agp, jnt );

	free( jnt );
}
