/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/****************************
 ***	N3PrimeInNull.c   ***
 ****************************/

#include        "GpType/GpType.h"

#include        "N3Prime.h"

#include        "N3PrimeIn.h"




static void	n3_prime_in_null_jnt_begin( jnt_type *jnt );
static void	n3_prime_in_null_jnt_end( jnt_type *jnt );


void
n3_prime_in_null( gp_array *agp )
{
int     i,      j;
gp_type *gp;   
jnt_type	*jnt;

	n3_prime_in_null_init( agp );


	for( i = 0 ; i < agp->Ni ; i++ )

		for( j = 0 ; j < agp->Nj ; j++ ){

			gp = &agp->gp[i][j];

			for( jnt = gp->jnt ; jnt != NULL ; jnt = jnt->nj ){

				if( jnt->type != JNT_N3 )    continue;

				if( jnt->brn[0]->type != BRN_N3 ) continue;

				if( jnt->brn[0]->direction == F_BEGIN )
					n3_prime_in_null_jnt_begin( jnt );
				else	n3_prime_in_null_jnt_end( jnt );
			}
		}
}



static void
n3_prime_in_null_jnt_begin( jnt_type *jnt )
{
brn_type	*b;
link_type	*l;
in_type	*r0,	*l0;

	if( jnt->brn[0]->type != BRN_N3 )	return;


	b = jnt->brn[1];
	if( b->type == BRN_NULL || b->link->type == NF_N3R )	return;

	b = jnt->brn[2];
	if( b->type == BRN_NULL || b->link->type == NF_N3R )	return;


	if( jnt->brn[1]->type != BRN_IN && jnt->brn[2]->type != BRN_IN )
		return;


	r0 = l0 = NULL;
	for( l = jnt->brn[0]->link ; l != NULL ; l = LINK_NEXT(l) ){
		if( l->in[F_RIGHT] != NULL && r0 == NULL )
			r0 = l->in[F_RIGHT];

		if( l->in[F_LEFT] != NULL && l0 == NULL )
			l0 = l->in[F_LEFT];
	}


	b = jnt->brn[1];
	if( b->type == BRN_IN && ( l0 == NULL || l0->type != IN_NULL) )
		b->in->type = IN_NULL;

	b = jnt->brn[2];
	if( b->type == BRN_IN && ( r0 == NULL || r0->type != IN_NULL) )
		b->in->type = IN_NULL;
}


static void
n3_prime_in_null_jnt_end( jnt_type *jnt )
{
brn_type	*b;
link_type	*l;
in_type	*r0,	*l0;

	if( jnt->brn[0]->type != BRN_N3 )	return;


	b = jnt->brn[1];
	if( b->type == BRN_NULL || b->link->type == NF_N3R )	return;

	b = jnt->brn[2];
	if( b->type == BRN_NULL || b->link->type == NF_N3R )	return;


	if( jnt->brn[1]->type != BRN_IN && jnt->brn[2]->type != BRN_IN )
		return;


	r0 = l0 = NULL;
	for( l = jnt->brn[0]->link ; l != NULL ; l = LINK_PREV(l) ){
		if( l->in[F_RIGHT] != NULL && r0 == NULL )
			r0 = l->in[F_RIGHT];

		if( l->in[F_LEFT] != NULL && l0 == NULL )
			l0 = l->in[F_LEFT];
	}

	if( r0 != NULL )
		while( r0->p != NULL )	r0 = r0->p;

	if( l0 != NULL )
		while( l0->p != NULL )	l0 = l0->p;


	b = jnt->brn[1];
	if( b->type == BRN_IN && ( r0 == NULL || r0->type != IN_NULL) )
		b->in->type = IN_NULL;

	b = jnt->brn[2];
	if( b->type == BRN_IN && ( l0 == NULL || l0->type != IN_NULL) )
		b->in->type = IN_NULL;

}



void
n3_prime_in_null_init( gp_array *agp )
{
int     i,      j;
gp_type *gp;   
jnt_type	*jnt;
brn_type	*b;

	for( i = 0 ; i < agp->Ni ; i++ )

		for( j = 0 ; j < agp->Nj ; j++ ){

			gp = &agp->gp[i][j];

			for( jnt = gp->jnt ; jnt != NULL ; jnt = jnt->nj ){

				if( jnt->type != JNT_N3 )    continue;

				if( jnt->brn[0]->type != BRN_N3 ) continue;

				b = jnt->brn[1];
				if( b->type == BRN_IN )	b->in->type = IN_N3;

				b = jnt->brn[2];
				if( b->type == BRN_IN )	b->in->type = IN_N3;
			}
		}
}
