/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/*************************
	N3PrimeLink.c
*************************/
#include	<math.h>

#include	"Uigp/igp.h"

#include	"GpType/GpType.h"

#include	"N3PrimeIn.h"

#ifdef PALMOS
	#define hypot(x,y) sqrt((x)*(x) + (y)*(y))
#endif

void
n3_prime_link_seg( vec2d *ctr, link_type *link, int side, in_type *in0,
				link_type *link1, in_type *in1,
				jnt_type *jnt0, jnt_type *jnt1 )
{
link_type	*nlink,	*pl;
link_type	*l,	*nl;
in_type	*in,	*in_1;
vec2d	v,	pctr;

	nlink = pl = NULL;
	pctr = jnt0->gctr;

	in = in0;

	for( l = link, v = *ctr ; ; l = LINK_NEXT(l ) ){
		in_1 = ( l == link1 )? in1 : NULL;

		nl = n3_prime_link_in( &v, l, side, &pctr, in,
						&jnt1->gctr, in_1 );

		if( in == NULL )
			nl->pr[0].edge = n3_prime_profile( l, side, F_BEGIN );
		else	nl->pr[0].edge = n3_prime_profile_in( l, side, in->t );

		if( pl == NULL ){
			nlink = pl = nl;
			nl->p[0].bno = -1;
			nl->p[0].p.l = NULL;
		}
		else	{
			nl->p[F_BEGIN].bno = -1;
			nl->p[F_BEGIN].p.l = pl;

			pl->p[F_END].bno = -1;
			pl->p[F_END].p.l = nl;

			pl->pr[F_END] = nl->pr[F_BEGIN];

			pl = nl;
		}

		pctr.x += nl->v.x;
		pctr.y += nl->v.y;

		v.x += l->v.x;
		v.y += l->v.y;

		in = NULL;

		if( l == link1 )	break;
	}

	if( in1 == NULL )
		pl->pr[F_END].edge = n3_prime_profile( link1, side, F_END );
	else	pl->pr[F_END].edge = n3_prime_profile_in( link1, side, in1->t );


	n3_prime_link_connect_jnt( jnt0, nlink, F_BEGIN );
	n3_prime_link_connect_jnt( jnt1, pl, F_END );
}




link_type *
n3_prime_link_in( vec2d *ctr, link_type *l, int side,
			vec2d *ctr0, in_type *in0,
			vec2d *ctr1, in_type *in1 )
{
link_type	*link;
width_type	*width;
float   w0,	w1,	w,	t,	h;
float	t0,	t1,	dt;
float	len_inv;
vec2d	v1;


	link = LINK_ALLOC();
        link->p[F_BEGIN].p.l = link->p[F_END].p.l = NULL;
        link->p[F_BEGIN].bno = link->p[F_END].bno = -1;

        link->in[F_RIGHT] = link->in[F_LEFT] = NULL;
        link->width[F_BEGIN] = link->width[F_END] = NULL;
        link->pr[F_BEGIN].edge = link->pr[F_END].edge = NULL;

        link->type = NF_N3_P;


	width = l->width[F_BEGIN];
	w0 = l->width[F_BEGIN]->w[side];
	if( in0 == NULL ){
/*
		link->ctr.x = ctr->x + w0* width->bes.y;
		link->ctr.y = ctr->y - w0* width->bes.x;
*/
		link->ctr = *ctr0;
		t0 = 0;
	}
	else	{
		link->ctr = *ctr0;
		t0 = in0->t;
	}


	width = l->width[F_END];
	w1 = l->width[F_END]->w[side];
	if( in1 == NULL ){
	    if( LINK_NEXT(l) != NULL ){
		link->v.x = ctr->x + l->v.x + w1* width->bes.y - link->ctr.x;
		link->v.y = ctr->y + l->v.y - w1* width->bes.x - link->ctr.y;
	    }
	    else {
		link->v.x = ctr1->x - link->ctr.x;
		link->v.y = ctr1->y - link->ctr.y;
	    }
		
		t1 = l->len;
	}
	else	{
		link->v.x = ctr1->x - link->ctr.x;
		link->v.y = ctr1->y - link->ctr.y;
		t1 = in1->t;
	}


	link->len = hypot( link->v.x, link->v.y );

	if( link->len != 0 ){
		len_inv = 1.0/link->len;

        	link->u.x =  link->v.y * len_inv;
        	link->u.y = -link->v.x * len_inv; 


		t = 0.5*(t1+t0);
		dt = t/l->len;
		w = t*w0 + (1-t)*w1;
		h = l->c_prb * t * (t - l->len) + 0.5*(w1+w0);
		v1.x = ctr->x + t * -l->u.y  +  h * l->u.x;
		v1.y = ctr->y + t *  l->u.x  +  h * l->u.y;

		link->a = (v1.x - link->ctr.x )*link->u.x +
				(v1.y - link->ctr.y )*link->u.y;
		link->c_prb = -4.0 *link->a/( link->len*link->len);
	}
	else	{
		link->u = l->u;
		link->a = 0;
		link->c_prb = 0;
	}


	link->br = l->br;
	link->Fbr = l->Fbr & ( 0x10 | (1<<side) );
	link->side = side;


	return( link );
}



void
n3_prime_link_connect_jnt( jnt_type *jnt, link_type *link, int direct )
{
brn_type        *b;

	b = jnt->brn[jnt->brn_no] = brn_alloc();

	b->type = BRN_N3_P;

	b->link = link;
	b->direction = direct;

	link->p[direct].p.jnt = jnt;
	link->p[direct].bno = jnt->brn_no;
	jnt->brn_no++;
}
