/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**************************
 ***	N3PrimeLoad.c	***
 *************************/

#include	<math.h>

#include	"GpType/GpType.h"

#include	"N3Prime.h"

#include	"N3PrimeIn.h"

static void	n3_prime_load_jnt( jnt_type *jnt, int brn_no );


static void	agp_n3_prime_load_link( jnt_type *jnt, int bno, int side );


static int	get_next_in( vec2d *ctr0, link_type *link0, in_type *in0,
			int side,
			vec2d *ctr1, link_type **link1, in_type **in1 );

static jnt_type *	get_null_jnt( jnt_type *jnt, int bno, int side );

static void	n3_prime_load_seg1( vec2d *ctr, link_type *link,
				int side, in_type *in0,
				link_type *link1, in_type *in1,
				jnt_type *jnt0, jnt_type *jnt1 );

static void	n3_prime_load_seg2( vec2d *ctr, link_type *link,
				int side, in_type *in0,
				link_type *link1, in_type *in1,
				jnt_type *jnt0, jnt_type *jnt1 );


static void	n3_prime_load_degenric( gp_array *agp, jnt_type *jnt, int i );

static int	n3_prime_load_find_br( jnt_type *jnt0,
					jnt_type *jnt1, int *bno1 );


void
n3_prime_load( gp_array *agp )
{
gp_type	*gp;
jnt_type	*jnt;
int	i,	j;


	if( agp->Fn3_prime == 0 )	return;



	for( i = 0 ; i < agp->Ni ; i++ )
		for( j = 0 ; j < agp->Nj ; j++ ){
			gp = &agp->gp[i][j];

			for( jnt = gp->jnt ; jnt != NULL ; jnt = jnt->nj ){
				if( jnt->type == JNT_N3 ){
					n3_prime_load_jnt( jnt, 1 );
					continue;
				}

				if( jnt->type == JNT_RN3 ){
					n3_prime_load_jnt( jnt, jnt->brn_no );
					continue;
				}
			}
		}


        for( i = 0 ; i < agp->Ni ; i++ )
                for( j = 0 ; j < agp->Nj ; j++ ){
                        gp = &agp->gp[i][j];

                        for( jnt = gp->jnt ; jnt != NULL ; jnt = jnt->nj ){
                                if( jnt->type != JNT_N3 )       continue;

                                if( jnt->brn[1]->type == BRN_IN )
                                        n3_prime_load_degenric( agp, jnt, 1 );

                                if( jnt->brn[2]->type == BRN_IN )
                                        n3_prime_load_degenric( agp, jnt, 2 );
                        }
                }

}



static void
n3_prime_load_jnt( jnt_type *jnt, int brn_no )
{
brn_type	*b;
int	i;

	for( i = 0 ; i < brn_no ; i++ ){
		b = jnt->brn[i];
		if( b->type != NF_N3 || b->direction != F_BEGIN )
			continue;

		agp_n3_prime_load_link( jnt, i, F_LEFT );
		agp_n3_prime_load_link( jnt, i, F_RIGHT );
	}
}





static void
agp_n3_prime_load_link( jnt_type *jnt, int bno, int side )
{
jnt_type	*jnt0,	*jnt1,	*ejnt;
brn_type	*b;
link_type	*link0,	*link1;
in_type	*in0,	*in1;
in_type	*in00,	*in11;
vec2d	ctr0,	ctr1;
int	i;

	b = jnt->brn[bno];

        i = ( bno == 0 )? 1 - side : side;
        jnt0 = jnt->pj[i];

        if( jnt0 == NULL && jnt->type == JNT_N3 )
                jnt0 = jnt->brn[i+1]->pj;


	if( jnt0 == NULL )
		jnt0 = get_null_jnt( jnt, (side == F_RIGHT)?2:1, side );

	in0 = NULL;

	in00 = ( jnt->type == JNT_N3 )? jnt->brn[i+1]->in : NULL;

	link0 = b->link;
	ctr0 = jnt->gctr;

	while( get_next_in( &ctr0, link0, in0, side, &ctr1, &link1, &in1 ) ){

		jnt1 = in1->jnt->brn[in1->bno]->pj;

		if( jnt0 == jnt1 ){
			ctr0 = ctr1;
			link0 = link1;
			in0 = in1;
			jnt0 = jnt1;

			continue;
		}



		if( link0 == link1 && in0 != NULL && in1 != NULL ){

			ctr0 = ctr1;
			link0 = link1;
			in0 = in1;
			jnt0 = jnt1;

			in00 = NULL;

			continue;
		}


		if( in1->direct == F_END || in00 != NULL && in00->use == 2 ||
			in0 != NULL && in0->use == 1 ){

			n3_prime_load_seg1( &ctr0, link0, side, in0,
                                link1, in1, 
                                jnt0, jnt1 );
		}
		else	n3_prime_load_seg2( &ctr0, link0, side, in0,
                                link1, in1,
                                jnt0, jnt1 );


		ctr0 = ctr1;
		link0 = link1;
		in0 = in1;
		jnt0 = jnt1;

		in00 = NULL;
	}




	ejnt = link1->p[F_END].p.jnt;
	bno = link1->p[F_END].bno;

        i = ( bno == 0 )? side :  1 - side;
        jnt1 = ejnt->pj[i];

        if( jnt1 == NULL && ejnt->type == JNT_N3 )
                jnt1 = ejnt->brn[i+1]->pj;


	if( jnt1 == NULL )
		jnt1 = get_null_jnt( ejnt, (side == F_RIGHT)?1:2, side );


	if( jnt0 == jnt1 && ( in0 != NULL || in1 != NULL ) )	return;


	in11 = ( ejnt->type == JNT_N3 )? ejnt->brn[i+1]->in : NULL;


	if( in11 == NULL || in11->use == 2 )
		n3_prime_load_seg1( &ctr0, link0, side, in0,
                                link1, NULL, jnt0, jnt1 );
	else	n3_prime_load_seg2( &ctr0, link0, side, in0,
                                link1, NULL, jnt0, jnt1 );
}



static jnt_type *
get_null_jnt( jnt_type *jnt, int bno, int side )
{
jnt_type	*jnt0,	*tjnt;
brn_type	*b;
link_type	*l;
in_type	*in,	*p;
int	tside,	tbno;
int	i;


	b = jnt->brn[bno];


        side = ( jnt->brn[0]->direction == F_BEGIN )? 2-bno : bno-1;
        if( jnt->brn[0]->direction != b->direction )
                tside = INV_FLAG( side );
        else    tside = side;


	if( b->direction == F_END ){
		p = NULL;

		for( l = b->link ; l != NULL ; l = LINK_PREV(l) ){
			for( in = l->in[tside] ;
				in != NULL && in != b->in ; in = in->p ){
				if( in->type != IN_NULL )
					p = in;
			}

			if( p != NULL )
				return( p->jnt->brn[p->bno]->pj );

			if( LINK_PREV(l) == NULL ) break;
		}

		tbno = l->p[0].bno;
		tjnt = l->p[0].p.jnt;


		i = ( tbno == 0 )? 1 - tside : tside;
		jnt0 = tjnt->pj[i];

		if( jnt0 == NULL && jnt->type == JNT_N3 )
			jnt0 = tjnt->brn[i+1]->pj;


		return( jnt0 );
	}



	for( l = b->link ; l != NULL ; l = LINK_NEXT(l) ){
		in = ( l == b->link )? b->in : l->in[tside];
		for(  ; in != NULL ; in = in->p )
			if( in->type != IN_NULL )
				return( in->jnt->brn[in->bno]->pj );

		if( LINK_NEXT(l) == NULL ) break;
	}

	tbno = l->p[1].bno;
	tjnt = l->p[1].p.jnt;


	i = ( tbno == 0 )? tside :  1 - tside;
	jnt0 = tjnt->pj[i];

	if( jnt0 == NULL && tjnt->type == JNT_N3 )
		jnt0 = tjnt->brn[i+1]->pj;

	return( jnt0 );
}






static int
get_next_in( vec2d *ctr0, link_type *link0, in_type *in0, int side,
		vec2d *ctr1, link_type **link1, in_type **in1 )
{
link_type	*l;
in_type	*in;

	for( l = link0, *ctr1 = *ctr0 ; l != NULL  ; l = LINK_NEXT(l) ){

		for( in = (l == link0 && in0 != NULL  )? in0->p : l->in[side] ;
			in != NULL && in->type == IN_NULL ; in = in->p );

		if( in != NULL ){
			*in1 = in;
			*link1 = l;
			return( 1 );
		}


		ctr1->x += l->v.x;
		ctr1->y += l->v.y;
		*link1 = l;
	}

	*in1 = NULL;
	return( 0 );

}




static void
n3_prime_load_seg1( vec2d *ctr, link_type *link, int side, in_type *in0,
                                link_type *link1, in_type *in1,
                                jnt_type *jnt0, jnt_type *jnt1 )
{
link_type	*l,	*sl;
n3_type	*n3;
n3_ltype *n3l;
edge_type	*e;
edge_ltype *el,*els;
int	iside;
int	bno,	bno1;


	bno = n3_prime_load_find_br( jnt0, jnt1, &bno1 );
	sl = jnt0->brn[bno]->link;

	iside = INV_FLAG( side );

	for( l = link ; l != NULL ; l = LINK_NEXT(l), sl = LINK_NEXT(sl) ){

		if( l == link && in0 != NULL ){
/*
			if( LINK_NEXT(l) == NULL )	break;
*/
			if( l == link1 )	break;
			continue;
		}


		n3 = l->pr[0].n3;
		n3l = &n3->side[side];
		e = sl->pr[0].edge;
		el = &e->side[side];
		els = &e->side[iside];


		n3->mval[SP_Y] = els->bval[SP_Y];
		n3l->val[SP_Y] = el->val[SP_Y];
		n3l->bval[SP_Y] = el->bval[SP_Y];

		n3->mval[SP_Q] = els->bval[SP_Q];
		n3l->val[SP_Q] = el->val[SP_Q];
		n3l->bval[SP_Q] = el->bval[SP_Q];

		n3->mval[SP_A] = els->bval[SP_A];
		n3l->val[SP_A] = el->val[SP_A];
		n3l->bval[SP_A] = el->bval[SP_A];

/*
		if( LINK_NEXT(l) == NULL )	break;
*/
		if( l == link1 )	break;
	}


	if( in1 != NULL )	return;

	n3 = l->pr[1].n3;
	n3l = &n3->side[side];
	e = sl->pr[1].edge;
	el = &e->side[side];
	els = &e->side[iside];


	n3->mval[SP_Y] = els->bval[SP_Y];
	n3l->val[SP_Y] = el->val[SP_Y];
	n3l->bval[SP_Y] = el->bval[SP_Y];

	n3->mval[SP_Q] = els->bval[SP_Q];
	n3l->val[SP_Q] = el->val[SP_Q];
	n3l->bval[SP_Q] = el->bval[SP_Q];

	n3->mval[SP_A] = els->bval[SP_A];
	n3l->val[SP_A] = el->val[SP_A];
	n3l->bval[SP_A] = el->bval[SP_A];
}




static void
n3_prime_load_seg2( vec2d *ctr, link_type *link, int side, in_type *in0,
                                link_type *link1, in_type *in1,
                                jnt_type *jnt0, jnt_type *jnt1 )
{
link_type	*l,	*slink,	*sl;
n3_type	*n3;
n3_ltype	*n3l;
edge_type	e;
edge_ltype	*el0,*el1;
width_type	*w;
int	side0,	side1;
int	bno,	bno1;
float	t,	u;
vec2d	sctr,	v,	p;


	bno = n3_prime_load_find_br( jnt0, jnt1, &bno1 );
	if( bno == -1 )	return;

	sl = jnt0->brn[bno]->link;

	if( jnt0->brn[bno]->direction == F_BEGIN ){
		side0 = side;
		sctr = jnt0->gctr;
		slink = jnt0->brn[bno]->link;
	}
	else	{
/*
		side0 = side;
*/
		side0 = INV_FLAG( side );
		sctr = jnt1->gctr;
		slink = jnt1->brn[bno1]->link;
	}

	side1 = INV_FLAG( side0 );


	for( l = link, v = *ctr ; l != NULL ; l = LINK_NEXT(l) ){

		if( l == link && in0 != NULL ){
			/*
			if( LINK_NEXT(l) == NULL )	break;
			*/
			v.x += l->v.x;
			v.y += l->v.y;
			if( l == link1 )	break;
			continue;
		}

		w = l->width[0];
		p.x = v.x + w->w[side]* w->bes.y;
		p.y = v.y - w->w[side]* w->bes.x;


		links_distance( &sctr, slink, NULL, &p, &sl, &u, &t );

		t = t / l->len;
		edge_afine( sl->pr[0].edge, sl->pr[1].edge, 1-t, &e );



		n3 = l->pr[0].n3;


		n3l = &n3->side[side];
		el0 = &e.side[side0];
		el1 = &e.side[side1];


		n3->mval[SP_Y] = el1->bval[SP_Y];
		n3l->val[SP_Y] = el0->val[SP_Y];
		n3l->bval[SP_Y] = el0->bval[SP_Y];

		n3->mval[SP_Q] = el1->bval[SP_Q];
		n3l->val[SP_Q] = el0->val[SP_Q];
		n3l->bval[SP_Q] = el0->bval[SP_Q];

		n3->mval[SP_A] = el1->bval[SP_A];
		n3l->val[SP_A] = el0->val[SP_A];
		n3l->bval[SP_A] = el0->bval[SP_A];


		v.x += l->v.x;
		v.y += l->v.y;
		if( l == link1 )	break;
	}


	if( in1 != NULL )	return;


	w = l->width[1];
	p.x = v.x + w->w[side]* w->bes.y;
	p.y = v.y - w->w[side]* w->bes.x;


	links_distance( &sctr, slink, NULL, &p, &sl, &u, &t );

	t = t / sl->len;
	edge_afine( sl->pr[0].edge, sl->pr[1].edge, 1-t, &e );


	n3 = l->pr[1].n3;


	n3l = &n3->side[side];
	el0 = &e.side[side0];
	el1 = &e.side[side1];


	n3->mval[SP_Y] = el1->bval[SP_Y];
	n3l->val[SP_Y] = el0->val[SP_Y];
	n3l->bval[SP_Y] = el0->bval[SP_Y];

	n3->mval[SP_Q] = el1->bval[SP_Q];
	n3l->val[SP_Q] = el0->val[SP_Q];
	n3l->bval[SP_Q] = el0->bval[SP_Q];

	n3->mval[SP_A] = el1->bval[SP_A];
	n3l->val[SP_A] = el0->val[SP_A];
	n3l->bval[SP_A] = el0->bval[SP_A];

}







static int
n3_prime_load_find_br( jnt_type *jnt0, jnt_type *jnt1, int *bno1 )
{
brn_type	*b;
link_type	*l;
int	i,	direct;


	for( i = 0 ; i < jnt0->brn_no ; i++ ){
		b = jnt0->brn[i];
		if( b->link->type != NF_N3_P )	continue;

		direct = INV_FLAG(b->direction);

		for( l = b->link ; l->p[direct].bno < 0 ;
						l = l->p[direct].p.l );

		if( l->p[direct].p.jnt == jnt1 ){
			*bno1 = l->p[direct].bno;
			return( i );
		}
	}

	return( -1 );
}



static void
n3_prime_load_degenric( gp_array *agp, jnt_type *jnt, int i )
{
jnt_type	*njnt;
brn_type	*b,	*nb;
int	nbno;
n3_type	*n3,	*nn3;
n3_ltype	*n3l,	*nn3l;
int	side,	nside;

	b = jnt->brn[i];
	if( b->type != BRN_IN ) return;
	if( !(b->in->type & 0x08) )	return;

	if( b->direction == F_BEGIN ){
		njnt = b->link->p[1].p.jnt;
		nbno = b->link->p[1].bno;
	}
	else	{
		njnt = b->link->p[0].p.jnt;
		nbno = b->link->p[0].bno;
	}


	b = jnt->brn[0];
	if( b->direction == F_BEGIN )
		side = ( i == 1 )? F_LEFT : F_RIGHT;
	else	side = ( i == 1 )? F_RIGHT : F_LEFT;

	nb = njnt->brn[nbno];
	if( nb->direction == F_BEGIN )
		nside = ( 3-i == 1 )? F_LEFT : F_RIGHT;
	else	nside = ( 3-i == 1 )? F_RIGHT : F_LEFT;

	n3 = b->link->pr[b->direction].n3;
	nn3 = nb->link->pr[nb->direction].n3;

	n3l = &n3->side[side];
	nn3l = &nn3->side[nside];

	nn3->mval[SP_Y] = n3l->bval[SP_Y];
	nn3l->bval[SP_Y] = n3->mval[SP_Y];
	nn3l->val[SP_Y] = n3->mval[SP_Y];

	nn3->mval[SP_Q] = n3l->bval[SP_Q];
	nn3l->bval[SP_Q] = n3->mval[SP_Q];
	nn3l->val[SP_Q] = n3->mval[SP_Q];

	nn3->mval[SP_A] = n3l->bval[SP_A];
	nn3l->bval[SP_A] = n3->mval[SP_A];
	nn3l->val[SP_A] = n3->mval[SP_A];
}
