/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/********************************
 ***	AgpChangeN3ToEdge.c   ***
 ********************************/

#include        <stdio.h>
#include        <math.h>

#include        "Uigp/igp.h"

#include        "GpType/GpType.h"

#include        "AgpEdit/AgpEditTool.h"

#include        "AgpTool/AgpTool.h"

#include        "N3Prime.h"


static void 
n3_prime_n3_to_edge_delete_in( gp_array *agp );

static void 
n3_prime_n3_to_edge_delete_polylink( gp_array *agp );

static void 
n3_prime_n3_to_edge_type_to_edge( gp_array *agp );


void 
n3_prime_n3_to_edge( gp_array *agp )
{

	if ( agp->Fn3_prime == 0 )
		n3_prime_agp( agp );

	agp->Fn3_prime = 0;

	n3_prime_n3_to_edge_delete_in( agp );

	n3_prime_n3_to_edge_delete_polylink( agp );

	n3_prime_n3_to_edge_type_to_edge( agp );
}

static void 
n3_prime_n3_to_edge_delete_in( gp_array *agp )
{
gp_type		*gp;
jnt_type	*jnt;
brn_type	*brn;
int			i, j, bno;

	for( i = 0 ; i < agp->Ni ; i++ )
		for( j = 0 ; j < agp->Nj ; j++ )
		{
			gp = &agp->gp[i][j];
			for( jnt = gp->jnt; jnt != NULL ; jnt = jnt->nj )
			{

				if ( jnt->type != JNT_N3 &&  
					 jnt->type != JNT_RN3)
					continue;

				for ( bno = 0; bno < jnt->brn_no; bno++  )
				{
					brn = jnt->brn[bno];

					if ( brn->type != BRN_IN )
						continue;

					jnt_n3_in_delete( jnt, bno );
				}
			}
		}
}

static void 
n3_prime_n3_to_edge_delete_polylink( gp_array *agp )
{
gp_type		*gp;
jnt_type	*jnt, *jnt_next;
brn_type	*brn;
vec2d		gctr;
int			i, j, bno;

	for( i = 0 ; i < agp->Ni ; i++ )
		for( j = 0 ; j < agp->Nj ; j++ )
		{
			gp = &agp->gp[i][j];
			for( jnt = gp->jnt; jnt != NULL ; jnt = jnt_next )
			{
				jnt_next = jnt->nj;

				if ( jnt->type != JNT_N3 &&  
					 jnt->type != JNT_RN3 )
					continue;

				for ( bno = 0; bno < jnt->brn_no; bno++ )
				{
					brn = jnt->brn[bno];

					if ( brn->type == BRN_N3R )
					{
						brn->type = BRN_RIDGE;
						jnt->type = JNT_RIDGE;
						continue;
					}

					if ( brn->direction != F_BEGIN )
						continue;
					if ( brn->type != BRN_N3 )
						continue;

					gctr.x = jnt->gctr.x + brn->ctr.x;
					gctr.y = jnt->gctr.y + brn->ctr.y;
					agp_lnf_clear_link( &gctr, brn->link, agp );
					link_free( brn->link );
				}

				if ( jnt->type == JNT_RIDGE )
				{
					for ( bno = 1; bno < jnt->brn_no; bno++ )
					{
						free( jnt->brn[bno] );
						jnt->brn[bno] = NULL;
					}
					jnt->brn_no = 1;
				}

				if ( jnt->type == JNT_N3 ||  jnt->type == JNT_RN3 )
				{
					for ( bno = 0; bno < jnt->brn_no; bno++ )
					{
						free( jnt->brn[bno] );
						jnt->brn[bno] = NULL;
					}
					jnt->brn_no = 0;
					agp_jnt_untied( agp, jnt );
					free( jnt );
				}
			}
		}
}

static void 
n3_prime_n3_to_edge_type_to_edge( gp_array *agp )
{
gp_type		*gp;
jnt_type	*jnt;
brn_type	*brn;
link_type	*l;
int			i, j, bno;

	for( i = 0 ; i < agp->Ni ; i++ )
		for( j = 0 ; j < agp->Nj ; j++ )
		{
			gp = &agp->gp[i][j];
			for( jnt = gp->jnt; jnt != NULL ; jnt = jnt->nj )
			{
				if ( jnt->type != JNT_N3_P )
					continue;

				jnt->type = JNT_EDGE;

				for ( bno = 0; bno < jnt->brn_no; bno++ )
				{
					brn = jnt->brn[bno];
					if ( brn->type != BRN_N3_P )
						continue;

					for ( l = brn->link; l; l = LINK_NEXT( l ) )
						l->type = NF_EDGE;

					brn->type = BRN_EDGE;
				}
			}

			for( jnt = gp->jnt; jnt != NULL ; jnt = jnt->nj )
			{
				if ( jnt->type != JNT_RIDGE )
					continue;

				for ( bno = 0; bno < jnt->brn_no; bno++ )
				{
					brn = jnt->brn[bno];
					if ( brn->type != BRN_RIDGE )
						continue;

					for ( l = brn->link; l; l = LINK_NEXT( l ) )
						l->type = NF_RIDGE;
				}
			}
		}
}

