/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/******************************
 ***    N3PrimeProfile.c    ***
 ******************************/
#include	<math.h>

#include	"Uigp/igp.h"

#include	"GpType/LinkType.h"

#include	"N3PrimeIn.h"



edge_type *
n3_prime_profile( link_type *link, int side, int direct )
{
edge_type	*e;
width_type	*width;
n3_type		*n3;
n3_ltype	*n3l;

	e = EDGE_ALLOC();

	n3 = link->pr[direct].n3;
	width = link->width[direct];

	if( side == F_RIGHT ){
		e->width = n3->width[F_RIGHT];
		e->depth = width->depth[F_RIGHT];
		n3l = &n3->side[F_RIGHT];

		e->side[F_RIGHT].val[SP_Y] = n3l->val[SP_Y];
		e->side[F_LEFT].val[SP_Y] = n3->mval[SP_Y];

		e->side[F_RIGHT].bval[SP_Y] = n3l->bval[SP_Y];
		e->side[F_LEFT].bval[SP_Y] = n3->mval[SP_Y];


		e->side[F_RIGHT].val[SP_Q] = n3l->val[SP_Q];
		e->side[F_LEFT].val[SP_Q] = n3->mval[SP_Q];

		e->side[F_RIGHT].bval[SP_Q] = n3l->bval[SP_Q];
		e->side[F_LEFT].bval[SP_Q] = n3->mval[SP_Q];


		e->side[F_RIGHT].val[SP_A] = n3l->val[SP_A];
		e->side[F_LEFT].val[SP_A] = n3->mval[SP_A];

		e->side[F_RIGHT].bval[SP_A] = n3l->bval[SP_A];
		e->side[F_LEFT].bval[SP_A] = n3->mval[SP_A];

		return( e );
	}


	e->width = n3->width[F_LEFT];
	e->depth = width->depth[F_LEFT];
	n3l = &n3->side[F_LEFT];
	e->side[F_LEFT].val[SP_Y] = n3l->val[SP_Y];
	e->side[F_RIGHT].val[SP_Y] = n3->mval[SP_Y];

	e->side[F_LEFT].bval[SP_Y] = n3l->bval[SP_Y];
	e->side[F_RIGHT].bval[SP_Y] = n3->mval[SP_Y];



	e->side[F_LEFT].val[SP_Q] = n3l->val[SP_Q];
	e->side[F_RIGHT].val[SP_Q] = n3->mval[SP_Q];

	e->side[F_LEFT].bval[SP_Q] = n3l->bval[SP_Q];
	e->side[F_RIGHT].bval[SP_Q] = n3->mval[SP_Q];


	e->side[F_LEFT].val[SP_A] = n3l->val[SP_A];
	e->side[F_RIGHT].val[SP_A] = n3->mval[SP_A];

	e->side[F_LEFT].bval[SP_A] = n3l->bval[SP_A];
	e->side[F_RIGHT].bval[SP_A] = n3->mval[SP_A];

	return( e );
}
			


edge_type *
n3_prime_profile_in( link_type *link, int side, float t0 )
{
edge_type	*e;
width_type	*width,	*width1;
n3_type	*n3,	*n31;
n3_ltype	*n3l,	*n31l;
float	 t;

	e = EDGE_ALLOC();

	n3 = link->pr[F_BEGIN].n3;
	n31 = link->pr[F_END].n3;
	width = link->width[F_BEGIN];
	width1 = link->width[F_END];
	t = t0 / link->len;

	if( side == F_RIGHT ){
		e->width = t*n3->width[F_RIGHT] + (1-t)*n31->width[F_RIGHT];
		e->depth = t*width->depth[F_RIGHT] +
					(1-t)*width1->depth[F_RIGHT];
		n3l = &n3->side[F_RIGHT];
		n31l = &n31->side[F_RIGHT];
		e->side[F_RIGHT].val[SP_Y] = t*n3l->val[SP_Y] +
						(1-t)*n31l->val[SP_Y];
		e->side[F_LEFT].val[SP_Y] = t*n3->mval[SP_Y] + (1-t)*n31->mval[SP_Y];

		e->side[F_RIGHT].bval[SP_Y] = t*n3l->bval[SP_Y] +
						(1-t)*n31l->bval[SP_Y];
		e->side[F_LEFT].bval[SP_Y] = e->side[F_LEFT].val[SP_Y];


		e->side[F_RIGHT].val[SP_Q] = t*n3l->val[SP_Q] +
						(1-t)*n31l->val[SP_Q];
		e->side[F_LEFT].val[SP_Q] = t*n3->mval[SP_Q] + (1-t)*n31->mval[SP_Q];

		e->side[F_RIGHT].bval[SP_Q] = t*n3l->bval[SP_Q] +
						(1-t)*n31l->bval[SP_Q];
		e->side[F_LEFT].bval[SP_Q] = e->side[F_LEFT].val[SP_Q];



		e->side[F_RIGHT].val[SP_A] = t*n3l->val[SP_A] +
						(1-t)*n31l->val[SP_A];
		e->side[F_LEFT].val[SP_A] = t*n3->mval[SP_A] + (1-t)*n31->mval[SP_A];

		e->side[F_RIGHT].bval[SP_A] = t*n3l->bval[SP_A] +
						(1-t)*n31l->bval[SP_A];
		e->side[F_LEFT].bval[SP_A] = e->side[F_LEFT].val[SP_A];

		return( e );
	}



	e->width = t*n3->width[F_LEFT] + (1-t)*n31->width[F_LEFT];
	e->depth = t*width->depth[F_LEFT] + (1-t)*width1->depth[F_LEFT];
	n3l = &n3->side[F_LEFT];
	n31l = &n31->side[F_LEFT];
	e->side[F_LEFT].val[SP_Y] = t*n3l->val[SP_Y] + (1-t)*n31l->val[SP_Y];
	e->side[F_RIGHT].val[SP_Y] = t*n3->mval[SP_Y] + (1-t)*n31->mval[SP_Y];

	e->side[F_LEFT].bval[SP_Y] = t*n3l->bval[SP_Y] +(1-t)*n31l->bval[SP_Y];
	e->side[F_RIGHT].bval[SP_Y] = e->side[F_RIGHT].val[SP_Y];


	e->side[F_LEFT].val[SP_Q] = t*n3l->val[SP_Q] + (1-t)*n31l->val[SP_Q];
	e->side[F_RIGHT].val[SP_Q] = t*n3->mval[SP_Q] + (1-t)*n31->mval[SP_Q];

	e->side[F_LEFT].bval[SP_Q] = t*n3l->bval[SP_Q] +(1-t)*n31l->bval[SP_Q];
	e->side[F_RIGHT].bval[SP_Q] = e->side[F_RIGHT].val[SP_Q];


	e->side[F_LEFT].val[SP_A] = t*n3l->val[SP_A] + (1-t)*n31l->val[SP_A];
	e->side[F_RIGHT].val[SP_A] = t*n3->mval[SP_A] + (1-t)*n31->mval[SP_A];

	e->side[F_LEFT].bval[SP_A] = t*n3l->bval[SP_A] +(1-t)*n31l->bval[SP_A];
	e->side[F_RIGHT].bval[SP_A] = e->side[F_RIGHT].val[SP_A];


	return( e );
}




edge_type *
n3_prime_profile1( jnt_type *jnt, int bno, int side, int flag )
{
link_type	*link;
edge_type	*e;
width_type	*width;
n3_type		*n3;
n3_ltype	*n3l;
int	ibno,	iside;


	link = jnt->brn[bno]->link;

	if( link->type != NF_N3 ){
		e = EDGE_CALLOC();
		return( e );
	}


	e = EDGE_ALLOC();

	if( jnt->brn[bno]->direction == F_END ){
		n3 = link->pr[F_END].n3;
		width = link->width[F_END];
		side = INV_FLAG( side );
	}
	else	{
		n3 = link->pr[F_BEGIN].n3;
		width = link->width[F_BEGIN];
	}

	e->width = n3->width[side];
	e->depth = width->depth[side];
	n3l = &n3->side[side];
	e->side[F_RIGHT].val[SP_Y] = n3l->val[SP_Y];
	e->side[F_LEFT].val[SP_Y] = n3->mval[SP_Y];

	e->side[F_RIGHT].bval[SP_Y] = n3l->bval[SP_Y];
	e->side[F_LEFT].bval[SP_Y] = n3->mval[SP_Y];



	e->side[F_RIGHT].val[SP_Q] = n3l->val[SP_Q];
	e->side[F_LEFT].val[SP_Q] = n3->mval[SP_Q];

	e->side[F_RIGHT].bval[SP_Q] = n3l->bval[SP_Q];
	e->side[F_LEFT].bval[SP_Q] = n3->mval[SP_Q];


	e->side[F_RIGHT].val[SP_A] = n3l->val[SP_A];
	e->side[F_LEFT].val[SP_A] = n3->mval[SP_A];

	e->side[F_RIGHT].bval[SP_A] = n3l->bval[SP_A];
	e->side[F_LEFT].bval[SP_A] = n3->mval[SP_A];


	if( flag ){
		e->side[F_RIGHT].val[SP_Y] = e->side[F_LEFT].val[SP_Y];
		e->side[F_RIGHT].val[SP_Q] = e->side[F_LEFT].val[SP_Q];
		e->side[F_RIGHT].val[SP_A] = e->side[F_LEFT].val[SP_A];

		e->side[F_RIGHT].bval[SP_Y] = e->side[F_LEFT].bval[SP_Y];
		e->side[F_RIGHT].bval[SP_Q] = e->side[F_LEFT].bval[SP_Q];
		e->side[F_RIGHT].bval[SP_A] = e->side[F_LEFT].bval[SP_A];

/*
		ibno = ( side == F_LEFT )? 2 : 1;
*/
		if( jnt->brn[0]->direction == F_END )
			ibno = ( side == F_LEFT )? 1 : 2;
		else	ibno = ( side == F_LEFT )? 2 : 1;

		if( jnt->brn[ibno]->type == BRN_NULL ){
			iside = INV_FLAG(side);
			e->side[F_RIGHT].bval[SP_Y] = n3->side[iside].bval[SP_Y];
			e->side[F_RIGHT].bval[SP_Q] = n3->side[iside].bval[SP_Q];
			e->side[F_RIGHT].bval[SP_A] = n3->side[iside].bval[SP_A];
		}
	}


	return( e );
}



edge_type *
n3_prime_profile11( jnt_type *jnt, int bno, int direct )
{
brn_type	*b;
link_type	*link;
edge_type	*e,	*ce;


	b = jnt->brn[bno];

	link = jnt->brn[bno]->link;

	e = EDGE_ALLOC();


	ce = link->pr[b->direction].edge;

	if( b->direction == direct ){
		*e = *ce;
		return( e );
	}


	e->width = ce->width;
	e->depth = ce->depth;

	e->side[F_RIGHT].val[SP_Y] = ce->side[F_LEFT].val[SP_Y];
	e->side[F_LEFT].val[SP_Y] = ce->side[F_RIGHT].val[SP_Y];

	e->side[F_RIGHT].bval[SP_Y] = ce->side[F_LEFT].bval[SP_Y];
	e->side[F_LEFT].bval[SP_Y] = ce->side[F_RIGHT].bval[SP_Y];


	e->side[F_RIGHT].val[SP_Q] = ce->side[F_LEFT].val[SP_Q];
	e->side[F_LEFT].val[SP_Q] = ce->side[F_RIGHT].val[SP_Q];

	e->side[F_RIGHT].bval[SP_Q] = ce->side[F_LEFT].bval[SP_Q];
	e->side[F_LEFT].bval[SP_Q] = ce->side[F_RIGHT].bval[SP_Q];


	e->side[F_RIGHT].val[SP_A] = ce->side[F_LEFT].val[SP_A];
	e->side[F_LEFT].val[SP_A] = ce->side[F_RIGHT].val[SP_A];

	e->side[F_RIGHT].bval[SP_A] = ce->side[F_LEFT].bval[SP_A];
	e->side[F_LEFT].bval[SP_A] = ce->side[F_RIGHT].bval[SP_A];

	return( e );
}
