/************************************************************************/

/*                         SetWidth.c                                   */ 

/************************************************************************/
#include        <stdio.h>   
#include	<math.h>
#include	"Uigp/igp.h"
#include        "Uprm/prm.h"

#include	"GpType/GpType.h"

#include	"RelValueIn.h"



static float	set_width( float rval, float lm, float wmin, float wmax );


static float	H_Wmin,	H_Wmax;
static float	R_Wmin = 0.75,	R_Wmax = 1.5;

static GP_prm params[] = {
        {"H-Wmin ", (char *)&H_Wmin,   PRM_FLOAT, "%.2f", "0.75"},
        {"H-Wmax ", (char *)&H_Wmax,   PRM_FLOAT, "%.2f", "2.0"},
        {"R-Wmin ", (char *)&R_Wmin,   PRM_FLOAT, "%.2f", "0.75"},
        {"R-Wmax ", (char *)&R_Wmax,   PRM_FLOAT, "%.2f", "1.5"},
};


void
set_width_get_prm(GP_prm  **prm, int  *prm_no, void  (**check)())
{
        *prm    = params;
        *prm_no  = PRM_NUMBER(params);
        *check = NULL ;
}


void
set_width_init( char *file )
{
        gp_prm_init( file, set_width_get_prm );
}




void set_width_agp(gp_array *agp )
{
gp_type	*gp;
hunch_type	*h;
int	i,	j;
  
	for( i = 0 ; i < agp->Ni ; i++ )
		for( j = 0  ; j < agp->Nj ; j++ ){
			gp = &agp->gp[i][j];

			for( h = gp->hunch ; h != NULL ; h = h->nh ){
    
				switch( h->type ){
				case    NF_HUNCH:
					set_width_len_hunch( h );
					hunch_set_aux( h );
				    break;
				case    NF_RH:
					set_width_hr( h );
					rhunch_set_aux( h );
				    break;
				}
			}
		}
}





void
set_width_len_hunch(hunch_type *h )
{
	h->g.width = set_width( h->rval[SP_Y], h->lm1[SP_Y], H_Wmin, H_Wmax );

	h->g.len = h->g.width / sqrt( h->lm2[SP_Y] / (float)h->lm1[SP_Y] );
}


void
set_width_hr(hunch_type *h )
{
	h->g.width = set_width( h->rval[SP_Y], h->lm1[SP_Y], H_Wmin, H_Wmax );

}



void
set_width_ridge_link( link_type *link )
{
ridge_type	*r;

	r = link->pr[0].ridge;
	r->width = set_width( r->rval[SP_Y], r->lm[SP_Y], R_Wmin, R_Wmax );

	if( LINK_NEXT(link) != NULL ) return;

	r = link->pr[1].ridge;
	r->width = set_width( r->rval[SP_Y], r->lm[SP_Y], R_Wmin, R_Wmax );
}


void
set_width_ridge_profile( ridge_type *r )
{
	r->width = set_width( r->rval[SP_Y], r->lm[SP_Y], R_Wmin, R_Wmax );
}



static float
set_width( float rval, float lm, float wmin, float wmax )
{
float   temp;

 
        temp = - rval / lm;

	if( temp <= wmin*wmin )	     return( wmin );

	if( temp >= wmax*wmax )	     return( wmax );

	return( sqrt( temp ) );
}


