/*********************
 ***	Coding.c   ***
 *********************/
#include        <stdio.h> 
#include	<string.h>  
#include	<math.h>

#include        "Uprm/prm.h"
#include	"Ucode/code.h"

#include	"GpType/Configuration.h"

#include        "Coding.h"

#include        "CodingCenter.h"
#include        "CodingJoint.h"
#include        "CodingLink.h"
#include        "CodingRidge.h"
#include        "CodingProfile.h"
#include        "CodingHunch.h"
#include        "CodingAdepth.h"

#include        "CodingBg.h"
#include        "CodingDepth.h"

#include        "Ep2/Ep2.h"


#include	"CodingTable.h"




#ifndef PLAYER
static GP_prm_group Pgroup[] = {
	{"Codeing-center",	coding_center_get_prm },
	{"Codeing-link",	coding_link_get_prm },
	{"Codeing-n3",		coding_n3_get_prm },
	{"Codeing-ridge",	coding_ridge_get_prm },
	{"Codeing-Profile",	coding_profile_get_prm },
	{"Codeing-bg",		coding_bg_get_prm },
	{"Codeing-Depth",	coding_depth_get_prm },
	{"Codeing-Hunch",	coding_hunch_get_prm },
	{"Codeing-Adepth",	coding_adepth_get_prm },
};


void
coding_get_group( GP_prm_group **prm_group, int *prm_group_no )
{
	*prm_group_no = PRM_GROUP_SIZE(Pgroup);
	*prm_group = Pgroup;
}
#endif


static char	Config_dir[256]= "config";


void
coding_set_config_dir( char *config_dir )
{

	strncpy( Config_dir, config_dir, 256 );
}



coding_type *
coding_init( int Lcoding, int bgp )
{
coding_type	*cprm;
int	i;
#ifndef PLAYER
char file[256];
#endif

	if( Lcoding >= 0 && Lcoding < MAX_CTABLE &&
					(i = Ctable[Lcoding]) != -1 ){

		cprm = coding_set( i, bgp );

		return(cprm);
	}

#ifndef PLAYER


	sprintf( file, "%s/L%d%d.coding", Config_dir, bgp, Lcoding );

	cprm = coding_set_local_prm( file, bgp );

	return( cprm );

#endif

	cprm = coding_set( 0, bgp );

	return( cprm );
}


#ifndef PLAYER
coding_type *
coding_set_local_prm( char *file, int bgp )
{
coding_type	*cprm;

	cprm = ( (coding_type *)malloc( sizeof( coding_type ) ) );


	cprm->ad = coding_adepth_set_lprm( file );


	cprm->cc = coding_center_set_lprm( file, bgp );

	cprm->jc = coding_joint_set();

	cprm->lc = coding_link_set_lprm( file );

	cprm->nc = coding_n3_set_lprm( file );

	cprm->rc = coding_ridge_set_lprm( file );

	cprm->pc = coding_profile_set_lprm( file );


	cprm->hc = coding_hunch_set_lprm( file );


	cprm->bc = coding_bg_set_lprm( file );

	cprm->depth = coding_depth_set_lprm( file );

	return( cprm );
}
#endif
	
	




coding_type *
coding_set( int iCoding, int bgp )
{
coding_type	*cprm;

	cprm = ( (coding_type *)malloc( sizeof( coding_type ) ) );

	cprm->ad = coding_adepth_set();

	cprm->cc = coding_center_set( iCoding, bgp );

	cprm->jc = coding_joint_set();

	cprm->lc = coding_link_set( iCoding, bgp );

	cprm->nc = coding_n3_set( iCoding );


	cprm->rc = coding_ridge_set( iCoding );

	cprm->pc = coding_profile_set( iCoding );

	cprm->hc = coding_hunch_set( iCoding );

	cprm->bc = coding_bg_set( iCoding );

	cprm->depth = coding_depth_set( iCoding );
		

	return( cprm );
}


void
coding_free( coding_type * cprm )
{
	free( cprm->ad );

	free( cprm->cc );

	free(cprm->jc);
	free(cprm->lc);

	free(cprm->nc);

	free( cprm->hc );

	free(cprm->bc);

	free(cprm->rc);

	free(cprm->pc);

	free(cprm->depth);

	free( cprm );
}
