/**************************
 ***	DepthReset.c	***
 **************************/
#include        <stdio.h>   
#include	<math.h>
#include	"Uigp/igp.h"


#include        "GpType/GpType.h"

#include        "Uview/ViewType.h"


static void	depth_plane_link( vec2d *ctr, link_type *link,
							 view_type *Vw, plane3d Pbr[], int DPflag );

static void	depth_plane_link0x04( vec2d *ctr, link_type *link,
								 view_type *Vw, plane3d Pbr[], int DPflag );

static void	depth_plane_link_n3( vec2d *ctr, link_type *link,
								view_type *Vw, plane3d Pbr[], int DPflag );

static void	depth_plane_normal( vec3d *N, vec3d *P,
				view_type *vw, plane3d *plane );

static float	depth_plane_value( vec2d *p, view_type *ve, plane3d	*pl, int flag );






void
coding_depth_plane( gp_array *agp, int flag )
{
abr_type	*br;
gp_type		*gp;
bgp_type	*bp;
link_type       *link; 
hunch_type      *h; 
vec2d	v;
int	i,	j;
plane3d	plane;

view_type	*Vw;
int	DPflag;

plane3d		Pbr[256];




	if( agp->ad == NULL )	return;

	Vw = agp->ad->vw;


	depth_plane_normal( &agp->ad->N, &agp->ad->P, agp->ad->vw, &plane );

	for( i = 0 ; i < agp->ad->br_no ; i++ ){
		br = &agp->ad->abr[i];

		if( br->Fplane != 0 )
			depth_plane_normal( &br->N, &br->P, agp->ad->vw, &Pbr[br->no] );
		else
			Pbr[br->no] = plane;
	}




	DPflag = flag;



	for( i = 0 ; i < agp->Ni ; i++ )
		for( j = 0 ; j < agp->Nj ; j++ ){
			gp = &agp->gp[i][j];

			for( bp = gp->bgp ; bp != NULL ; bp = bp->p ){
				v.x = gp->ctr.x + bp->ctr.x;
				v.y = gp->ctr.y + bp->ctr.y;

				bp->depth += depth_plane_value( &v, Vw, &Pbr[bp->br], flag );
			}


			for( h = gp->hunch ; h != NULL ; h = h->nh){
				v.x = gp->ctr.x + h->ctr.x;
				v.y = gp->ctr.y + h->ctr.y;

				h->depth += depth_plane_value( &v, Vw, &Pbr[h->br], flag );
			}

			for( link = gp->link ; link != NULL ; link = link->nl )
				if( link->type == NF_N3 )
					depth_plane_link_n3( &gp->ctr, link, Vw, Pbr, flag );
				else	depth_plane_link( &gp->ctr, link, Vw, Pbr, flag );
		}
}




static void
depth_plane_link( vec2d *ctr, link_type *link, view_type *Vw, plane3d Pbr[], int DPflag )
{
vec2d	v;
profile_type	pr;

	if( link->Fbr == 0x04 ){
		depth_plane_link0x04( ctr, link, Vw, Pbr, DPflag );
		return;
	}

	v.x = ctr->x + link->ctr.x;
	v.y = ctr->y + link->ctr.y;

	pr = link->pr[0];
	pr.p->depth += depth_plane_value( &v, Vw, &Pbr[link->br], DPflag );

	v.x += link->v.x;
	v.y += link->v.y;

	if( LINK_NEXT(link) != NULL )	return;

	pr = link->pr[1];
	pr.p->depth += depth_plane_value( &v, Vw, &Pbr[link->br], DPflag );
}


static void
depth_plane_link0x04( vec2d *ctr, link_type *link, view_type *Vw, plane3d Pbr[], int DPflag )
{
vec2d	v;
profile_type	pr;

	v.x = ctr->x + link->ctr.x;
	v.y = ctr->y + link->ctr.y;

	pr = link->pr[0];
	pr.p->depth += 0.5*( depth_plane_value( &v, Vw, &Pbr[link->br], DPflag ) +
					depth_plane_value( &v, Vw, &Pbr[link->br1], DPflag) );

	v.x += link->v.x;
	v.y += link->v.y;

	if( LINK_NEXT(link) != NULL )	return;

	pr = link->pr[1];
	pr.p->depth += 0.5*( depth_plane_value( &v, Vw, &Pbr[link->br], DPflag ) +
					depth_plane_value( &v, Vw, &Pbr[link->br1], DPflag) );
}



static void
depth_plane_link_n3( vec2d *ctr, link_type *link, view_type *Vw, plane3d Pbr[], int DPflag )
{
vec2d	v,	w;
profile_type	pr;
width_type	*width;

	v.x = ctr->x + link->ctr.x;
	v.y = ctr->y + link->ctr.y;

	pr = link->pr[0];

	pr.p->depth += depth_plane_value( &v, Vw, &Pbr[link->br], DPflag );


	width = link->width[F_BEGIN];
	w.x = v.x + width->w[F_RIGHT] * width->bes.y;
	w.y = v.y - width->w[F_RIGHT] * width->bes.x;
	width->depth[F_RIGHT] += depth_plane_value( &w, Vw, &Pbr[link->br], DPflag );


	w.x = v.x + width->w[F_LEFT] * width->bes.y;
	w.y = v.y - width->w[F_LEFT] * width->bes.x;
	width->depth[F_LEFT] += depth_plane_value( &w, Vw, &Pbr[link->br], DPflag );
		


	if( LINK_NEXT(link) != NULL )	return;

	v.x += link->v.x;
	v.y += link->v.y;


	pr = link->pr[1];

	pr.p->depth += depth_plane_value( &v, Vw, &Pbr[link->br], DPflag );

	width = link->width[F_END];
	w.x = v.x + width->w[F_RIGHT] * width->bes.y;
	w.y = v.y - width->w[F_RIGHT] * width->bes.x;
	width->depth[F_RIGHT] += depth_plane_value( &w, Vw, &Pbr[link->br], DPflag );

	w.x = v.x + width->w[F_LEFT] * width->bes.y;
	w.y = v.y - width->w[F_LEFT] * width->bes.x;
	width->depth[F_LEFT] += depth_plane_value( &w, Vw, &Pbr[link->br], DPflag );
}



static void
depth_plane_normal( vec3d *N, vec3d *P, view_type *vw, plane3d *plane )
{
vec3d	p1,	p2;

        plane->n.x = VEC3D_INNER( vw->U, *N );
        plane->n.y = VEC3D_INNER( vw->V, *N );
        plane->n.z = -VEC3D_INNER( vw->N, *N );

	p1.x = P->x - vw->cp.x;
	p1.y = P->y - vw->cp.y;
	p1.z = P->z - vw->cp.z;

        p2.x = VEC3D_INNER( vw->U, p1 );
        p2.y = VEC3D_INNER( vw->V, p1 );
        p2.z = -VEC3D_INNER( vw->N, p1 );

	plane->d =  -VEC3D_INNER( p2, plane->n );
}



static float
depth_plane_value( vec2d *p, view_type *vw, plane3d	*pl, int flag )
{
vec2d	v;
float	depth;

	v.x = vw->Ua_inv*(p->y - vw->Ub);
	v.y = vw->Va_inv*(p->x - vw->Vb);

	depth = - pl->d/(v.x * pl->n.x + v.y*pl->n.y - pl->n.z );

	if( flag == 1 )	return( -depth );

	return( depth );
}
