/*******************************
 ***	CodigProfileLoad.c   ***
 *******************************/

#include        <stdio.h>   
#include	<math.h>

#include        "CodingProfile.h"


static void	coding_profile_load_from_jnt_one( jnt_type *jnt );
static void	coding_profile_load_from_jnt_n3r( jnt_type *jnt );



void
coding_profile_load_from_jnt( gp_array *agp )
{
gp_type	*gp;
jnt_type	*jnt;

int	i,	j;


	for( i = 0 ; i < agp->Ni ; i++ )
		for( j = 0 ; j < agp->Nj ; j++ ){

			gp = &agp->gp[i][j];

			for( jnt = gp->jnt ; jnt != NULL ; jnt = jnt->nj ){

				if( jnt->type != JNT_N3 )	continue;

				coding_profile_load_from_jnt_one( jnt );
			}
		}
}




static void
coding_profile_load_from_jnt_one( jnt_type *jnt )
{
brn_type	*b;
n3_type	*n3;
edge_type	*edge;
int	sp,	tside;


	b = jnt->brn[0];
	if( b->type == BRN_NULL )	return;

	if( b->type == BRN_N3R ){
		coding_profile_load_from_jnt_n3r( jnt );
		return;
	}


	if( b->direction == F_BEGIN ){
		n3 = b->link->pr[0].n3;
		tside = F_LEFT;
	}
	else	{
		n3 = b->link->pr[1].n3;
		tside = F_RIGHT;
	}


	b = jnt->brn[1];
	if( b->type == BRN_EDGE ){
		if( b->direction == F_BEGIN ){	/* WHITE */
			edge = b->link->pr[0].edge;
			for( sp = 0 ; sp < SP_NO ; sp++ ){
				edge->side[F_RIGHT].val[sp] = n3->side[tside].val[sp];
				edge->side[F_RIGHT].bval[sp] = n3->side[tside].bval[sp];
				edge->side[F_LEFT].val[sp] = n3->mval[sp];
/*
				edge->side[F_LEFT].bval[sp] = n3->mval[sp];
*/
			}
			edge->width = n3->width[tside];
		}
		else	{	/* BLACK */
			edge = b->link->pr[1].edge;
			for( sp = 0 ; sp < SP_NO ; sp++ ){
				edge->side[F_LEFT].val[sp] = n3->side[tside].val[sp];
				edge->side[F_LEFT].bval[sp] = n3->side[tside].bval[sp];
				edge->side[F_RIGHT].val[sp] = n3->mval[sp];
/*
				edge->side[F_RIGHT].bval[sp] = n3->mval[sp];
*/
			}
			edge->width = n3->width[tside];
		}
	}

	tside = INV_FLAG( tside );



	b = jnt->brn[2];
	if( b->type == BRN_EDGE ){
		if( b->direction == F_END ){
			edge = b->link->pr[1].edge;
			for( sp = 0 ; sp < SP_NO ; sp++ ){
				edge->side[F_RIGHT].val[sp] = n3->side[tside].val[sp];
				edge->side[F_RIGHT].bval[sp] = n3->side[tside].bval[sp];
				edge->side[F_LEFT].val[sp] = n3->mval[sp];
/*
				edge->side[F_LEFT].bval[sp] = n3->mval[sp];
*/
			}

			edge->width = n3->width[tside];
		}
		else	{
			edge = b->link->pr[0].edge;
			for( sp = 0 ; sp < SP_NO ; sp++ ){
				edge->side[F_LEFT].val[sp] = n3->side[tside].val[sp];
				edge->side[F_LEFT].bval[sp] =n3->side[tside].bval[sp];

				edge->side[F_RIGHT].val[sp] = n3->mval[sp];
/*
				edge->side[F_RIGHT].bval[sp] = n3->mval[sp];
*/
			}
			edge->width = n3->width[tside];
		}
	}
}



static void
coding_profile_load_from_jnt_n3r( jnt_type *jnt )
{
brn_type	*b;
ridge_type	*r;
edge_type	*edge;
int	sp,	tside;


	b = jnt->brn[0];
	if( b->type == BRN_NULL )	return;
	if( b->direction == F_BEGIN ){
		r = b->link->pr[0].ridge;
		tside = F_LEFT;
	}
	else	{
		r = b->link->pr[1].ridge;
		tside = F_RIGHT;
	}


	b = jnt->brn[1];
	if( b->type == BRN_EDGE ){
		if( b->direction == F_BEGIN ){	/* WHITE */
			edge = b->link->pr[0].edge;
			for( sp = 0 ; sp < SP_NO ; sp++ ){
				edge->side[F_RIGHT].val[sp] = 
					edge->side[F_RIGHT].bval[sp];
/*
				edge->side[F_LEFT].val[sp] = r->l[sp].mval;
*/
				edge->side[F_LEFT].val[sp] =
					edge->side[F_LEFT].bval[sp];
			}
			edge->width = jnt->rwidth;
		}
		else	{	/* BLACK */
			edge = b->link->pr[1].edge;
			for( sp = 0 ; sp < SP_NO ; sp++ ){
				edge->side[F_LEFT].val[sp] =
					edge->side[F_LEFT].bval[sp];
/*
				edge->side[F_RIGHT].val[sp] = r->l[sp].mval;
*/
				edge->side[F_RIGHT].val[sp] = 
					edge->side[F_RIGHT].bval[sp];
			}
			edge->width = jnt->lwidth;
		}
	}

	tside = INV_FLAG( tside );



	b = jnt->brn[2];
	if( b->type == BRN_EDGE ){
		if( b->direction == F_END ){
			edge = b->link->pr[1].edge;
			for( sp = 0 ; sp < SP_NO ; sp++ ){
				edge->side[F_RIGHT].val[sp] = 
					edge->side[F_RIGHT].bval[sp];
/*
				edge->side[F_LEFT].val[sp] = r->l[sp].mval;
*/
				edge->side[F_LEFT].val[sp] = 
					edge->side[F_LEFT].bval[sp];
			}

			edge->width = jnt->lwidth;
		}
		else	{
			edge = b->link->pr[0].edge;
			for( sp = 0 ; sp < SP_NO ; sp++ ){
				edge->side[F_LEFT].val[sp] = 
					edge->side[F_LEFT].bval[sp];

/*
				edge->side[F_RIGHT].val[sp] = r->l[sp].mval;
*/
				edge->side[F_RIGHT].val[sp] = 
					edge->side[F_RIGHT].bval[sp];
			}
			edge->width = jnt->rwidth;
		}
	}
}
