/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**********************
 ***	DecBg.c    ***
 **********************/
#include	<stdio.h>   
#include	<math.h>

#include	"N3Prime/N3Prime.h"

#include	"EpType/EpType.h"
#include	"Ep/Ep.h"
#include	"EpBgp/EpBgp.h"

#include	"Ep2/Ep2.h"



#include	"Coding/CodingBg.h"
#include	"DecBg.h"

#include	"CodingBgIn.h"





void
decoding_bg_huffman( coding_type *cprm, bst_type *bst_file )
{
	decoding_ep_huffman( cprm->bc, bst_file );
}




void
decoding_bg( gp_array *agp, coding_type *cprm, bst_type *bst )
{
ep2_array	*aep2;
ep_array	*aep;

int	i,	j;

	if( agp->type & AGP_DEPTH ){
		decoding_bg_depth( agp, cprm, bst );
		return;
	}

	n3_prime_agp( agp );
	

	aep2 = aep2_alloc();
	aep = aep_alloc();


	for( i = 1 ; i < agp->Ni-1 ; i += cprm->N ){
		for( j = 1 ; j < agp->Nj-1 ; j += cprm->N ){
 
			ep_create( agp, i, j, cprm->N, cprm->N, 0, aep );


			ep_bgp_get_average_decoding( agp, aep );


			ep2_create( aep, aep2 );
			aep2_br_set( aep2 );

			decoding_ep( aep2,cprm->bc, bst );

			ep_bgp_put( agp, aep, 1 );


			decoding_abs_value_agp( agp, aep );


#ifdef VIM_RELINQUISH
				gp_relinquish_cpu();
#endif
		}
	}


	aep_destroy( aep );
	aep2_destroy( aep2 );


	coding_bg_border( agp );

	n3_prime_load( agp );


	n3_prime_free( agp );
}




void
decoding_bg_depth( gp_array *agp, coding_type *cprm, bst_type *bst )
{
ep2_array	*aep2;
ep_array	*aep;
int	i,	j,	k,	mbr;
layer_type	*layer;
int	ni,	nj;


	
	n3_prime_agp( agp );


	aep2 = aep2_alloc();
	aep = aep_alloc();

	for( k = 0 ; k < agp->layer_no ; k++ ){

		layer = agp->layer[k];

		mbr = layer->Mbr;

		for( i = layer->i0 ; i < layer->i1 ; i += cprm->N ){
			ni = ( i + cprm->N < layer->i1+1 )?
					cprm->N : layer->i1+1-i;

			for( j = layer->j0 ; j < layer->j1 ; j += cprm->N ){

				nj = ( j + cprm->N < layer->j1+1)?
						cprm->N:layer->j1+1-j;

				ep_create( agp, i, j, ni, nj, mbr, aep );


				aep_br_set( aep );
				aep_br_fill( aep );//????


				ep_bgp_get_average_decoding( agp, aep );


				ep2_create( aep, aep2 );

				decoding_layer_bg( aep2, cprm->depth, bst );


				decoding_ep( aep2,cprm->bc, bst );

				ep_bgp_put( agp, aep, 1 );


				decoding_abs_value_agp_br( agp, aep, mbr );

#ifdef VIM_RELINQUISH
				gp_relinquish_cpu();
#endif
			}
		}
	}


	aep_destroy( aep );
	aep2_destroy( aep2 );


	coding_bg_border( agp );

	n3_prime_load( agp );


	n3_prime_free( agp );
}






void
decoding_bg_close( coding_type *cprm )
{
	decoding_ep_close(cprm->bc);
}