/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/********************
 ***	DecBg.c   ***
 *******************/
#include 	"Uigp/igp.h"
#include 	"Ucode/code.h"

#include        "DecBg.h"

#include        "GpType/GpType.h"
#include        "Ep2/Ep2.h"


#include		"Decoding/DecIn.h"
#include        "Coding/CodingBg.h"

#include        "CodingBgIn.h"


static void	decoding_ep2( ep2_array *aep2, codingBg_type *bc );

static void	decoding_ep_bound_put_dvalue( ep2_array *aep2,
			codingBg_type *bc, int i0, int j0, int k, int sp );




static void	decoding_ep_value_0( ep2_array *aep2 );
static void	decoding_ep_bound_value_0( ep2_array *aep2,
					       int i, int j, int k );



static void	decoding_ep_put_dvalue( ep2_array *aep2,
			codingBg_type *bc, int sp, bst_type *bst );







void
decoding_ep_huffman( codingBg_type *bc,bst_type *bst )
{
int	i;

	for( i = 0 ; i < SP_NO ; i ++ ){
		bc->Bval[i].cst = decoding_agp_file_cut_huffman_cst( bst );
		bc->Bdval2[i].cst = decoding_agp_file_cut_huffman_cst( bst );
		bc->Bdval[i].cst = decoding_agp_file_cut_huffman_cst( bst );
	}
}






void
decoding_ep( ep2_array *aep2,codingBg_type *bc, bst_type *bst )
{
int	sp,	flag;


	decoding_ep2( aep2, bc );


	decoding_ep_value_0( aep2 );

	for( sp = 0 ; sp < SP_NO ; sp++ ){
		flag = bst_get( bst );


		if( flag == 1 )
			decoding_ep_put_dvalue( aep2,bc, sp, bst );
	}
}


void decoding_ep_close(codingBg_type *bc)
{
int	i;
	for( i = 0 ; i < SP_NO ; i++ ){
		cst_free( bc->Bval[i].cst );
		cst_free( bc->Bdval[i].cst );
		cst_free( bc->Bdval2[i].cst );
	}
}




static void
decoding_ep2( ep2_array *aep2, codingBg_type *bc )
{
ep2_type	*ep2;
int	i,	j,	k;
bound2_type	*b2;
float	val[3];
int	cno;

	for( i = 0 ; i < aep2->Ni ; i++ )
                for( j = 0 ; j < aep2->Nj ; j++ ){
			ep2 = &aep2->ep[i][j];

			for( k = 0 ; k < ep2->b_no ; k++ ){
				b2 = ep2->bound[k];

				if( b2->br < 0 )	continue;
				if(B2_INDEPENDENT(*b2)){
					cno = CST_GET_SCHAR( bc->Bval[0].cst );
					b2->val[0] = UNCODE( cno, bc->Bval[0] );

					cno = CST_GET_SCHAR( bc->Bval[1].cst );
					b2->val[1] = UNCODE( cno, bc->Bval[1] );

					cno = CST_GET_SCHAR( bc->Bval[2].cst );
					b2->val[2] = UNCODE( cno, bc->Bval[2] );

					continue;
				}

				ep2_prev_value3( aep2, i, j, k, val );

				cno = CST_GET_SCHAR( bc->Bdval2[0].cst );
				b2->val[0] = val[0] +UNZCODE(cno,bc->Bdval2[0]);

				cno = CST_GET_SCHAR( bc->Bdval2[1].cst );
				b2->val[1] = val[1] +UNZCODE(cno,bc->Bdval2[1]);

				cno = CST_GET_SCHAR( bc->Bdval2[2].cst );
				b2->val[2] = val[2] +UNZCODE(cno,bc->Bdval2[2]);

			}
		}
}


static void
decoding_ep_value_0( ep2_array *aep2 )
{
ep2_type	*ep2;
int	i,	j,	k;


	for( i = 0 ; i < aep2->Ni ; i++ )
	    for( j = 0 ; j < aep2->Nj ; j++ ){
		ep2 = &aep2->ep[i][j];


		for( k = 0 ; k < ep2->b_no ; k++ )
			decoding_ep_bound_value_0(aep2, i, j, k );
	    }
}


static void
decoding_ep_bound_value_0( ep2_array *aep2, int i0, int j0, int k )
{
ep2_type	*ep2;
bound2_type	*b2;
ep_type	*ep;
ebgp_type	*bgp;
vec2d	v;
int	i,	j,	fcont;

	ep2 = &aep2->ep[i0][j0];
	b2 = ep2->bound[k];

	if( b2->br < 0 )	return;

        for( i = 0 ; i < 4 ; i++ ){
		if( (ep = ep2->ep[i]) == NULL )	continue;

		fcont = b2->Fbound[i];
		for( j = 0 ; fcont != 0 ; fcont >>= 1, j++ ){
 
                        if( !(fcont & 0x01) )   continue;  

			if( (bgp = ep->bound[j]->Mbgp) == NULL )
				continue;


			ep->bound[j]->bgp = bgp;
			bgp->p = NULL;

 
			v.x = ep->ctr.x + bgp->ctr.x;
			v.y = ep->ctr.y + bgp->ctr.y;
			
			ep2_value3( aep2, i0, j0, k, &v, bgp->val );
		}
	}
}






static void
decoding_ep_put_dvalue( ep2_array *aep2,codingBg_type *bc, int sp, bst_type *bst )
{

ep2_type	*ep2;
bound2_type	*b2;
int	i,	j,	k;


	for( i = 0 ; i < aep2->Ni ; i++ )
	    for( j = 0 ; j < aep2->Nj ; j++ ){
		ep2 = &aep2->ep[i][j];


		for( k = 0 ; k < ep2->b_no ; k++ ){

			b2 = ep2->bound[k];

			if( b2->br < 0 )	continue;

			if (b2->no <= 1)	continue;

		
			if( bst_get( bst ) == 0 )	continue;


			decoding_ep_bound_put_dvalue(aep2,bc, i, j, k, sp );
		}
	    }
}


static void
decoding_ep_bound_put_dvalue( ep2_array *aep2,codingBg_type *bc, int i0, int j0, int k, int sp )
{
int     i,      j,    fcont,	no;
float	sum;
ep2_type	*ep2;
bound2_type	*b2;
ep_type	*ep;
ebgp_type	*bgp;
int	cno;
float	tmp;


	ep2 = &aep2->ep[i0][j0];
	b2 = ep2->bound[k];

	sum = 0;
	no = 0;
        for( i = 0 ; i < 4 ; i++ ){
		if( (ep = ep2->ep[i]) == NULL )	continue;


		fcont = b2->Fbound[i];
		for( j = 0 ; fcont != 0 ; fcont >>= 1, j++ ){
 
                        if( !(fcont & 0x01) )   continue;  

			
			if( (bgp = ep->bound[j]->Mbgp) == NULL )
				continue;


			if( no < b2->no-1 ){
				cno = CST_GET_SCHAR( bc->Bdval[sp].cst );
				tmp = UNZCODE( cno, bc->Bdval[sp] );

				bgp->val[sp] += tmp;
				sum += bgp->val[sp];

				no++;

				continue;
			}

			bgp->val[sp] = b2->val[sp] * b2->no - sum;
                }
	}
}
