/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**************************
 ***	DecAdepth.c	***
 **************************/
#include        <stdio.h>
#include        <math.h>

#include        "Uigp/igp.h"

#include        "GpType/GpType.h"

#include        "Ubst/BstType.h"

#include        "Coding/CodingAdepth.h"

#include        "DecIn.h"



static view_type *	decoding_view( int row, int column,
				codingAdepth_type*  ac, bst_type *bst );

static void	decoding_plane( view_type *vw, vec3d *P, vec3d *N,
				codingAdepth_type *ac, bst_type *bst );

static void	decoding_adepth_layer( coding_type *cprm,
				gp_array *agp, bst_type *bst );

static void	decoding_adepth_float( float *a, code *cd, bst_type *bst );


int
decoding_adepth_layer_no( coding_type *cprm, int row, int column, view_type **vw1, bst_type *bst )
{
//bst_type        *bst;
int	mbr;
int     i,      no,	byte;
view_type *vw;

	byte = bst_get_align_num( bst,2 );

//	bst = bst_get_align_bst( bst_file, byte );

	vw = decoding_view( row, column, cprm->ad, bst );


	mbr = bst_get_int( bst, 16 );

//	bst_destroy( bst );
	if( vw1 != NULL )
		*vw1 = vw;
	else
		free( vw ); 


	for( i = 0, no = 0 ; mbr != 0 ; i++, mbr >>= 1 ){
		if( !( mbr & 1 ) )	continue;
 
		no++;
	}


	return( no );
}


/*
int
decoding_adepth_layer_no( coding_type *cprm, int row, int column, bst_type *bst_file )
{
bst_type        *bst;
int	mbr;
int     i,      no,	byte;
view_type *vw;

	byte = bst_get_align_num( bst_file,2 );

	bst = bst_get_align_bst( bst_file, byte );

	vw = decoding_view( row, column, cprm->ad, bst );


	mbr = bst_get_int( bst, 16 );

	bst_destroy( bst );
	free( vw ); 


	for( i = 0, no = 0 ; mbr != 0 ; i++, mbr >>= 1 ){
		if( !( mbr & 1 ) )	continue;
 
		no++;
	}


	return( no );
}
*/


void
decoding_adepth( coding_type *cprm, bst_type * bst_file, gp_array *agp )
{
adepth_type	*ad;
abr_type	*br;
bst_type        *bst;
int	mbr;
int     i,      no,	byte;

	byte = bst_get_align_num( bst_file,2 );

//	bst = bst_alloc( byte );
//	bst_copy( bst_file, bst, byte );

	bst = bst_get_align_bst( bst_file, byte );

	ad = agp->ad = ADEPTH_ALLOC();


	ad->vw = decoding_view( agp->row, agp->column, cprm->ad, bst );


	ad->Mbr = bst_get_int( bst, 16 );



	ad->br_no = 0;
	for( no = 0, mbr = ad->Mbr ;  mbr != 0 ; no++ , mbr >>= 1 ){
		if( !( mbr & 0x01 ) )	continue;

		br = &ad->abr[ad->br_no++];

		br->no = no;

/*
		br->Fplane = bst_get( bst );

		if( br->Fplane == 0 )	continue;
*/
		br->Fplane = 1;

		decoding_plane( ad->vw, &br->P, &br->N, cprm->ad, bst );
	}



	for( i = 0 ; i < ad->br_no ; i++ ){
		br = &ad->abr[i];
		if( br->Fplane == 0 )   continue;
	
		ad->P = br->P;
		ad->N = br->N;
		break;
	}



	decoding_adepth_layer( cprm, agp, bst );

	bst_free( bst );
}



static view_type *
decoding_view( int row, int column,codingAdepth_type * ac, bst_type *bst )
{
view_type	*vw;
vec2d	v;
float	dt;

	vw = VIEW_ALLOC();

	if( bst_get( bst ) == 1 )
		vw->type = VIEW_PERSPECTIVE;
	else	vw->type = VIEW_PARALLEL;

	if( bst_get( bst ) == 1 ){
		decoding_adepth_float( &vw->cp.x, &ac->Adepth_cp, bst );
		decoding_adepth_float( &vw->cp.y, &ac->Adepth_cp, bst );
		decoding_adepth_float( &vw->cp.z, &ac->Adepth_cp, bst );
	}
	else	{
		vw->cp.x = 0;
		vw->cp.y = 0;
		vw->cp.z = 1;
	}



	if( bst_get( bst ) == 1 ){
		decoding_adepth_float( &vw->N.x, &ac->Adepth_u, bst );
		decoding_adepth_float( &vw->N.y, &ac->Adepth_u, bst );
		vw->N.z = sqrt( 1.0 - vw->N.x*vw->N.x - vw->N.y*vw->N.y );
		if( bst_get( bst ) == 0 )	vw->N.z = -vw->N.z;
	}
	else	{
		vw->N.x = 0;
		vw->N.y = 0;
		vw->N.z = -1;
	}


	if( bst_get( bst ) == 1 ){
		decoding_adepth_float( &vw->V.x, &ac->Adepth_u, bst );
		decoding_adepth_float( &vw->V.y, &ac->Adepth_u, bst );
		vw->V.z = sqrt( 1.0 - vw->V.x*vw->V.x - vw->V.y*vw->V.y );
		if( bst_get( bst ) == 0 )	vw->V.z = -vw->V.z;
	}
	else	{
		vw->V.x = 0;
		vw->V.y = 1;
		vw->V.z = 0;
	}



	if( bst_get( bst ) == 1 ){
		decoding_adepth_float( &dt, &ac->Adepth_dt, bst );
		dt = 1 - dt;
	}
	else	dt = 1;

	dt /= 512;

	

	if( bst_get( bst ) == 1 ){
		decoding_adepth_float( &v.x, &ac->Adepth_p, bst );
		decoding_adepth_float( &v.y, &ac->Adepth_p, bst );
	}
	else	{
		v.x = row * 0.5;
		v.y = column * 0.5;
	}


	vw->uv.x0 = -v.y * dt;
	vw->uv.x1 = ( column - v.y )*dt;

	vw->uv.y1 = v.x *dt;
	vw->uv.y0 = -( row - v.x )*dt;

	vw->dis = 1.0;

	view_per_set( vw, &vw->cp, 1, &vw->N, 1, &vw->V, 1 );
	view_set_pixel( vw, row, column );

	return( vw );
}



static void
decoding_plane( view_type *vw, vec3d *P, vec3d *N, codingAdepth_type*  ac,bst_type *bst )
{
vec3d	dp;
float	d,	tmp;
vec3d	p3,	n;
float	u,	v;

	if( bst_get( bst ) == 1 ){
		decoding_adepth_float( &dp.x, &ac->Adepth_pu, bst );
		decoding_adepth_float( &dp.y, &ac->Adepth_pu, bst );
/*
		N->z = sqrt( 1.0 - N->x*N->x - N->y*N->y );
		if( bst_get( bst ) == 0 )	N->z = -N->z;
*/
		decoding_adepth_float( &dp.z, &ac->Adepth_pu, bst );
		tmp = sqrt( dp.x*dp.x + dp.y*dp.y + dp.z*dp.z );
		dp.x /= tmp;
		dp.y /= tmp;
		dp.z /= tmp;

                N->x = dp.x * vw->U.x + dp.y * vw->V.x + dp.z * vw->N.x;
                N->y = dp.x * vw->U.y + dp.y * vw->V.y + dp.z * vw->N.y;
                N->z = dp.x * vw->U.z + dp.y * vw->V.z + dp.z * vw->N.z;

	}
	else	*N = vw->N;


	decoding_adepth_float( &d, &ac->Adepth_d, bst );



	u = 0.5*(vw->uv.x1 + vw->uv.x0);
	v = 0.5*(vw->uv.y1 + vw->uv.y0);

	view_3d_ray( vw, u, v, &p3, &n );

	tmp = d / VEC3D_INNER( *N, n );

	P->x = vw->cp.x + tmp * n.x;
	P->y = vw->cp.y + tmp * n.y;
	P->z = vw->cp.z + tmp * n.z;

/*
	P->x = vw->cp.x + d * N->x;
	P->y = vw->cp.y + d * N->y;
	P->z = vw->cp.z + d * N->z;
*/
}



static void
decoding_adepth_layer( coding_type *cprm, gp_array *agp, bst_type *bst )
{
layer_type	*layer;
int	i,	mbit,	ibit,	jbit;
 
	agp->layer_no = 0;
	if( !(agp->type & AGP_DEPTH) )  return;
 
	if( agp->type != AGP_TRANSPARENT_U ){
		layer = agp->layer[agp->layer_no++] = layer_alloc();
		layer->Mbr = agp->ad->Mbr;
		layer->i0 = 1;
		layer->j0 = 1;
		layer->i1 = agp->Ni-1;
		layer->j1 = agp->Nj-1;
                 
		return;
	}


	mbit = coding_ilog2( (int)agp->ad->Mbr );
	ibit = coding_ilog2( agp->Ni );
	jbit = coding_ilog2( agp->Nj );

	agp->layer_no = bst_get_int( bst, mbit );

	for( i = 0 ; i < agp->layer_no ; i++ ){
		layer = agp->layer[i] = layer_alloc();

		layer->Mbr = bst_get_int( bst, mbit );
		layer->i0 = bst_get_int( bst, ibit );
		layer->i1 = bst_get_int( bst, ibit );
		layer->j0 = bst_get_int( bst, jbit );
		layer->j1 = bst_get_int( bst, jbit );

		layer->Fdepth_delta = bst_get( bst );

		if( !(cprm->head & CODING_DEPTH_DELTA ) )
			layer->Fdepth_delta = 0;

		if( cprm->head & CODING_LAYER_NAME )
			bst_get_string( bst, layer->name, MAX_LAYER_NAME );

	}


	for( i = 0 ; i < agp->layer_no ; i++ ){
		layer = agp->layer[i];
		if( layer->name[0] == 0 ){
			layer->name[0] = 'A' + i;
			layer->name[1] = 0;
		}
	}
}





static void
decoding_adepth_float( float *a, code *cd, bst_type *bst )
{
int     cno;
	cno = bst_get_int( bst, cd->rbit );
	*a = UNCODE( cno, *cd );
}
