/**********************
 ***	DecAgp.c    ***
 **********************/
#include	<stdio.h>   
#include	<math.h>

#include	"GpType/Configuration.h"


#include	"Uprm/prm.h"

#include	"AgpTool/AgpTool.h"
#include	"AgpDepth/AgpDepth.h"
#include	"AgpColor/AgpColor.h"


#include	"EpType/EpType.h"
#include	"Ep2/Ep2.h"

#include	"Ep/Ep.h"
#include	"EpBgp/EpBgp.h"
#include	"EpValue/EpValue.h"


#include	"RelValue/RelValue.h"
#include	"N3Prime/N3Prime.h"
#include	"AgpLayer/AgpLayer.h"


#include	"Decoding.h"

#include	"Ucode/code.h"


#include	"Coding/Coding.h" 
#include	"DecIn.h"
#include	"Coding/CodingDepth.h"
#include	"Coding/CodingBg.h"
#include	"Coding/CodingProfile.h"
#include	"Coding/CodingLink.h"


#include	"CodingBg/DecBg.h"


static void	decoding_agp( gp_array *agp,
					coding_type *cprm, bst_type *bst );


static void	decoding_agp_ridge_hunch( gp_array *agp,
					coding_type *cprm, bst_type *bst );



static void	decoding_agp_huffman( bst_type *bst_file,
					coding_type *cprm, int Fdepth );


static void	decoding_agp_close( coding_type *cprm, int Fdepth );




gp_array *
decoding_agp_from_bst( bst_type * bst_file )
{
gp_array	*agp;
coding_type	*cprm;
int	type,	row,	column,	bgp,	Lcoding;
bst_type	*bst;


	agp = agp_alloc();




	decoding_agp_bst_read_header( bst_file, &type, &row, &column,
					&bgp, &Lcoding );

	cprm = coding_init( Lcoding, bgp );


	cprm->head = bst_get_align_num( bst_file, 1 );
	cprm->N = bst_get_align_num( bst_file, 1 );



	agp_clear( agp, bgp, row, column, bgp*cprm->N, bgp*cprm->N );
	agp->type = type;

	agp->Lcoding = Lcoding;




	if( agp->type & AGP_DEPTH )
		decoding_adepth( cprm, bst_file, agp );



	bst = decoding_agp_file_cut_bst( bst_file );

	decoding_agp_huffman( bst_file, cprm, agp->type & AGP_DEPTH );



	decoding_agp( agp, cprm, bst);



	bst_free( bst );

	decoding_agp_close( cprm, agp->type & AGP_DEPTH );

	coding_free( cprm );


	return( agp );
}






static void
decoding_agp( gp_array *agp, coding_type *cprm, bst_type *bst )
{

	decoding_center( agp,cprm->cc );


	decoding_joint( agp, cprm, bst );



	agp_geomtric_aux( agp );
	agp_link_in_reorder( agp );



	if( agp->type & AGP_DEPTH )
		decoding_layer( agp, cprm->depth, bst );



	decoding_bg( agp, cprm, bst );


	decoding_profile( agp,cprm->pc, bst );



	decoding_agp_ridge_hunch( agp, cprm, bst );




	if( agp->type & AGP_DEPTH ){
		agp_depth_base( agp );

#ifdef	DEPTH_DELTA
		if( cprm->head & CODING_DEPTH_DELTA )
			decoding_depth( agp, cprm->depth, bst );
#endif
	}



	agp->color_type = AGP_YCrCb;
	agp_YCrCb_Rgb( agp );

	agp_rgb_range( agp, 1 );
}






static void
decoding_agp_ridge_hunch( gp_array *agp, coding_type *cprm, bst_type *bst )
{

hunch_type **ah;
link_type **link;
int     nf_no;
int	i,	j;


	ah = (hunch_type **)malloc( sizeof(hunch_type*) *10000 );
	link = (link_type **)malloc( sizeof(link_type*) *10000 );


	agp->Lni = agp->Lnj = cprm->N;
	for( i = 1 ; i < agp->Ni-1 ; i += agp->Lni )
		for( j = 1 ; j < agp->Nj-1 ; j += agp->Lnj ){
			agp_set_box_ij( agp, i, j );

			nf_no = agp_get_ridge_array( agp, link );
			decoding_ridge( link, nf_no, cprm->rc, bst );

			if( cprm->head & CODING_HUNCH ){
				nf_no = agp_get_hunchs_array( agp, ah );
				decoding_hunch( ah, nf_no, cprm->hc, bst );
			}

		}

	free( ah );
	free( link );
}






		
static void
decoding_agp_close( coding_type *cprm, int Fdepth  )
{
	decoding_center_close(cprm->cc);

	decoding_joint_close( cprm->jc );

	decoding_link_close(cprm->lc);

	decoding_profile_close(cprm->pc);

	decoding_n3_close(cprm->nc);

	decoding_bg_close( cprm );


	if( cprm->head & CODING_RIDGE )
		decoding_ridge_close(cprm->rc);

	if( cprm->head & CODING_HUNCH )
		decoding_hunch_close( cprm->hc );

	if( cprm->head & CODING_DEPTH )
		decoding_layer_close( cprm->depth );

	if( cprm->head & CODING_DEPTH_DELTA )
		decoding_depth_close( cprm->depth );
}



static void
decoding_agp_huffman( bst_type *bst_file, coding_type *cprm, int Fdepth )
{
	decoding_center_huffman( cprm->cc,bst_file );

	decoding_joint_huffman( cprm->jc, bst_file );

	decoding_link_huffman( cprm->lc, bst_file );

	decoding_profile_huffman( cprm->pc, bst_file );

	decoding_n3_huffman( cprm->nc, bst_file );
 
	decoding_bg_huffman( cprm, bst_file );



 
	if( cprm->head & CODING_RIDGE )
		decoding_ridge_huffman( cprm->rc, bst_file );

	if( cprm->head & CODING_HUNCH )
		decoding_hunch_huffman( cprm->hc, bst_file );
 

	if( cprm->head & CODING_DEPTH )
		decoding_layer_huffman( cprm->depth, bst_file );

	if( cprm->head & CODING_DEPTH_DELTA )
		decoding_depth_huffman( cprm->depth, bst_file );
}
