/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/

/**************************
 ***	DecAgpFile.c    ***
 **************************/
#include	<stdio.h>


#include	"Uigp/igp.h"
#include	"Ubst/BstType.h"

#include	"Coding/VimVersion.h"
#include	"Decoding.h"

#include	"DecIn.h"



gp_array *
decoding_agp_from_file( char *file )
{
gp_array	*agp;
FILE	*fp;
int	head,	byte;
bst_type *bst;
int	version;



	if ( (fp = fopen( file, "rb" )) == NULL )
		return( NULL );



	/* VIM signanture */
	if( getc(fp) != 'V' )   return( NULL );

	version = gp_fread_num( fp, 2 );

	if( version < VIM_VERSION0 || version > VIM_VERSION )    return( NULL );

//	if( version != VIM_VERSION )	return( NULL );



       while(  (head = getc( fp )) != EOF ){
                if( head == 'A' )	break;

 		byte = gp_fread_num( fp, 3 );
		fseek( fp, byte, 1L );
	}



	if(  head != 'A' ){
		fclose( fp );
		return( NULL );
	}



	byte = gp_fread_num( fp, 3 );

	bst = bst_read( fp, byte );
	fclose( fp );


	agp = decoding_agp_from_bst( bst );


	bst_free(bst);



	return( agp );
}


int
decoding_agp_file_info( char *file,
				int *type, int *row, int *column,
				int *bgp, int *Lcoding )
{
FILE	*fp;
bst_type	*bst;
int	byte;
int	version;

	if ( (fp = fopen ( file, "rb" )) == NULL )
		return( 0 );



	if( getc(fp) != 'V' )   return( 0 );

	version = gp_fread_num( fp, 2 );

	if( version != VIM_VERSION )	return( 0 );




	if( getc(fp) != 'A' )   return( 0 );
	byte = gp_fread_num( fp, 3 );


	bst = bst_read( fp, 20 );
	fclose( fp );


	decoding_agp_bst_read_header( bst, type, row, column, bgp, Lcoding );

	bst_free(bst);

	return( 1 );
}


int
decoding_agp_bst_info( bst_type *bst,
				int *type, int *row, int *column,
				int *bgp, int *Lcoding, int *layer_no )
{
coding_type	*cprm;

	decoding_agp_bst_read_header( bst, type, row, column, bgp, Lcoding );


	if( ( (*type) & AGP_DEPTH ) == 0 ){
		*layer_no = 1;
		return( 1 );
	}


	cprm = coding_init( *Lcoding, *bgp );


	cprm->head = bst_get_align_num( bst, 1 );
	cprm->N = bst_get_align_num( bst, 1 );


	*layer_no = decoding_adepth_layer_no( cprm, *row, *column, NULL, bst );

	coding_free( cprm );

	bst_rewind( bst );

	return( 1 );
}



int
decoding_agp_bst_info_vw( bst_type *bst,
				int *type, int *row, int *column,
				int *bgp, int *Lcoding, int *layer_no, view_type **vw )
{
coding_type	*cprm;

	decoding_agp_bst_read_header( bst, type, row, column, bgp, Lcoding );


	if( ( (*type) & AGP_DEPTH ) == 0 ){
		*layer_no = 1;
		return( 1 );
	}


	cprm = coding_init( *Lcoding, *bgp );


	cprm->head = bst_get_align_num( bst, 1 );
	cprm->N = bst_get_align_num( bst, 1 );


	*layer_no = decoding_adepth_layer_no( cprm, *row, *column, vw, bst );

	coding_free( cprm );

	bst_rewind( bst );

	return( 1 );
}


