/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
#include        <stdio.h>

#include        "Uigp/igp.h"

#include        "Ucode/code.h"



#include	"DecIn.h"
#include	"Coding/CodingCenter.h"



static void	decoding_center_block4( gp_array *agp,codingCenter_type *cc,
					int i0, int j0, bst_type *bst );


static void	decoding_center_block2( gp_array *agp,
					codingCenter_type *cc, int i0, int j0,
					int cflag, bst_type *bst );


static void	decoding_center_block1( gp_type *gp, codingCenter_type *cc,
					int cflag, bst_type *bst );

static void	decoding_center_box( gp_type *gp, codingCenter_type *cc,
					int nflag, int cflag, bst_type *bst );





void
decoding_center_huffman( codingCenter_type *cc,bst_type *bst_file )
{
        cc->Yes_f.cst = decoding_agp_file_cut_huffman_cst( bst_file );

        cc->Type_f.cst = decoding_agp_file_cut_huffman_cst( bst_file );

        cc->Cbst = decoding_agp_file_cut_bst( bst_file );
}



void
decoding_center( gp_array *agp,codingCenter_type *cc )
{
int     i,      j;
 
 
        for( i = 0 ; i < agp->Ni ; i += 4 )
                for( j = 0 ; j < agp->Nj ; j += 4 )
                        decoding_center_block4( agp,cc, i, j, cc->Cbst );

}





static void
decoding_center_block4( gp_array *agp,codingCenter_type *cc, int i0, int j0, bst_type *bst )
{
int	cno,	cflag;
	
	cno = bst_get( bst );

	if( cno == 0 )	return;


	cflag = bst_get( bst );


	decoding_center_block2( agp,cc, i0, j0, cflag, bst );
	decoding_center_block2( agp,cc, i0, j0 +2, cflag, bst );
	decoding_center_block2( agp,cc, i0 +2, j0, cflag, bst );
	decoding_center_block2( agp,cc, i0 +2, j0 +2, cflag, bst );
}



static void
decoding_center_block2( gp_array *agp,codingCenter_type *cc, int i0, int j0,
					int cflag, bst_type *bst )
{
int	cno;
	if( i0 >= agp->Ni || j0 >= agp->Nj )	return;

	
	cno = bst_get( bst );

	if( cno == 0 )	return;

	if( cflag == 0 )
		cflag = bst_get( bst );

	if( i0 < agp->Ni && j0 < agp->Nj )
		decoding_center_block1( &agp->gp[i0][j0],cc, cflag, bst );
	if( i0 < agp->Ni && j0+1 < agp->Nj )
		decoding_center_block1( &agp->gp[i0][j0+1],cc, cflag, bst );
	if( i0+1 < agp->Ni && j0 < agp->Nj )
		decoding_center_block1( &agp->gp[i0+1][j0],cc, cflag, bst );
	if( i0+1 < agp->Ni && j0+1 < agp->Nj )
		decoding_center_block1( &agp->gp[i0+1][j0+1],cc, cflag, bst );
}



static void
decoding_center_block1( gp_type *gp,codingCenter_type *cc, int cflag, bst_type *bst )
{
int	cno;
int	flag;
	
	cno = bst_get( bst );

	if( cno == 0 )	return;

	flag = CST_GET_UCHAR( cc->Yes_f.cst );

	if( flag & 0x01 )	decoding_center_box( gp,cc, 0, cflag, bst );
	if( flag & 0x02 )	decoding_center_box( gp,cc, 1, cflag, bst );
	if( flag & 0x04 )	decoding_center_box( gp,cc, 2, cflag, bst );
	if( flag & 0x08 )	decoding_center_box( gp,cc, 3, cflag, bst );
}



static void
decoding_center_box( gp_type *gp,codingCenter_type *cc, int nflag, int cflag, bst_type *bst )
{
int	cno,	type;
hunch_type	*hunch,	**phunch;
jnt_type	*jnt,	**pjnt;

	phunch = &gp->hunch;
	while( *phunch != NULL )	phunch = &(*phunch)->nh;
	pjnt = &gp->jnt;
	while( *pjnt != NULL )	pjnt = &(*pjnt)->nj;

	type = 0;
	while( 1 ){
		type = CST_GET_UCHAR( cc->Type_f.cst );

		if( type > NF_RH ){
			jnt = jnt_alloc();
			jnt->Fborder = JNT_IN;
			jnt->type = type - NF_RH;

			cno = bst_get_int( bst, cc->Pctr.rbit-1 );
			if( nflag & 0x2 )	cno += cc->Pctr.hr;
			jnt->ctr.x = UNCODE( cno, cc->Pctr );

			cno = bst_get_int( bst, cc->Pctr.rbit-1 );
			if( nflag & 0x1 )	cno += cc->Pctr.hr;
			jnt->ctr.y = UNCODE( cno, cc->Pctr );

			jnt->nj = NULL;
			*pjnt = jnt;
			pjnt = &jnt->nj;

			if( cflag == 1 )	return;

			if( bst_get( bst ) == 0 )	return;

			continue;
		}


		hunch = hunch_alloc();
		hunch->type = type;

		cno = bst_get_int( bst, cc->Hctr.rbit-1 );
		if( nflag & 0x2 )	cno += cc->Hctr.hr;
		hunch->ctr.x = UNCODE( cno, cc->Hctr );

		cno = bst_get_int( bst, cc->Hctr.rbit-1 );
		if( nflag & 0x1 )	cno += cc->Hctr.hr;
		hunch->ctr.y = UNCODE( cno, cc->Hctr );

		hunch->nh = NULL;
		*phunch = hunch;
		phunch = &hunch->nh;

		if( cflag == 1 )	return;
		if( bst_get( bst ) == 0 )	return;
	}
}




void
decoding_center_close(codingCenter_type *cc)
{
	cst_free( cc->Yes_f.cst );

	cst_free( cc->Type_f.cst );
	
	bst_free( cc->Cbst);
}
