/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/***********************
 ***	DecDepth.c   ***
 ***********************/
#include 	"Uigp/igp.h"



#include	"DecIn.h"
#include	"Coding/CodingDepth.h"


#define	 BSIZE	   2
//#define	 MAX_BR	  20




static void	decoding_depth_br_gp( gp_type *gp, int Br_depth_no[] );
static void	decoding_depth_gp( gp_type *gp,
			codingDepth_type *dc, float Br_depth[] );



void
decoding_depth_huffman(codingDepth_type *dc, bst_type *bst_file )
{
	dc->Bdepth.cst = decoding_agp_file_cut_huffman_cst( bst_file );
	dc->Depth.cst = decoding_agp_file_cut_huffman_cst( bst_file );
}




void
decoding_depth( gp_array *agp, codingDepth_type *dc,bst_type *bst )
{
/*
gp_type	*gp;
jnt_type	*jnt;
*/
int	i,	j,	n,	k,	no,	cno;
float	Br_depth[MAX_BR];
int	Br_depth_no[MAX_BR];




	for( i = 0 ; i < agp->Ni ; i += BSIZE )
	    for( j = 0 ; j < agp->Nj ; j += BSIZE ){


		for( k = 0 ; k < MAX_BR ; k++ ){
			Br_depth[k] = 0;
			Br_depth_no[k] = 0;
		}



		for( k = i ; k < agp->Ni && k < i+BSIZE ; k++ )
		    for( n = j ; n < agp->Nj && n < j+BSIZE ; n++ )
			decoding_depth_br_gp( &agp->gp[k][n], Br_depth_no );


		no = 0;
		for( k = 0 ; k < MAX_BR ; k++ ){
			if( Br_depth_no[k] == 0 )	continue;

			cno = CST_GET_SCHAR( dc->Bdepth.cst );
			Br_depth[k] = UNZCODE( cno, dc->Bdepth);
		}



		for( k = i ; k < agp->Ni && k < i+BSIZE ; k++ )
		    for( n = j ; n < agp->Nj && n < j+BSIZE ; n++ )
			decoding_depth_gp( &agp->gp[k][n],dc, Br_depth );

	    }



/*

	for( i = 0 ; i < agp->Ni ; i++ )
		for( j = 0 ; j < agp->Nj ; j++ ){
			gp = &agp->gp[i][j];
			for( jnt = gp->jnt ; jnt != NULL ; jnt = jnt->nj )
				jnt_set_depth( jnt );
		}
*/

}



void
decoding_depth_close(codingDepth_type *dc)
{
	cst_free( dc->Bdepth.cst );
	cst_free( dc->Depth.cst );
}







static void
decoding_depth_br_gp( gp_type *gp, int Br_depth_no[] )
{
bgp_type	*bp;
hunch_type	*h;
link_type	*l;
int	n;


	for( bp = gp->bgp ; bp != NULL ; bp = bp->p ){
		n = bp->br;
		Br_depth_no[n] ++;
	}


	for( h = gp->hunch ; h != NULL ; h = h->nh ){
		n = h->br;
		Br_depth_no[n] ++;
	}


	for( l = gp->link ; l != NULL ; l = l->nl ){

		switch( l->type ){
		case NF_RIDGE:
		case NF_N3R:
			l->Fbr = 0;
			n = l->br;
			Br_depth_no[n] ++;
		    break;

		case NF_EDGE:
			n = l->br;
			Br_depth_no[n] ++;
		    break;

		case NF_N3:
			n = l->br;
			Br_depth_no[n] ++;
		    break;
		}
	}
}




static void
decoding_depth_gp( gp_type *gp,codingDepth_type *dc, float Br_depth[] )
{
bgp_type	*bp;
hunch_type	*h;
link_type	*l;
int	n,	no,	cno;


	no = 0;

	for( bp = gp->bgp ; bp != NULL ; bp = bp->p ){
		n = bp->br;
		cno = CST_GET_SCHAR( dc->Depth.cst );
		bp->depth += UNZCODE( cno, dc->Depth) + Br_depth[n];
	}


	for( h = gp->hunch ; h != NULL ; h = h->nh ){
		n = h->br;
		cno = CST_GET_SCHAR( dc->Depth.cst );
		h->depth += UNZCODE( cno, dc->Depth) + Br_depth[n];
	}


	for( l = gp->link ; l != NULL ; l = l->nl ){

		switch( l->type ){
		case NF_RIDGE:
		case NF_N3R:
			n = l->br;
			cno = CST_GET_SCHAR( dc->Depth.cst );
			l->pr[0].p->depth +=
					UNZCODE( cno, dc->Depth) + Br_depth[n];

			if( LINK_NEXT(l) != NULL )      break;

			cno = CST_GET_SCHAR( dc->Depth.cst );
			l->pr[1].p->depth +=
					UNZCODE( cno, dc->Depth) + Br_depth[n];

		    break;

		case NF_EDGE:
			n = l->br;
			cno = CST_GET_SCHAR( dc->Depth.cst );
			l->pr[0].p->depth +=
					UNZCODE( cno, dc->Depth) + Br_depth[n];

			if( LINK_NEXT(l) != NULL )      break;

			cno = CST_GET_SCHAR( dc->Depth.cst );
			l->pr[1].p->depth +=
					UNZCODE( cno, dc->Depth) + Br_depth[n];

		    break;

		case NF_N3:
			n = l->br;
			cno = CST_GET_SCHAR( dc->Depth.cst );
			l->pr[0].p->depth +=
					UNZCODE( cno, dc->Depth) + Br_depth[n];

			cno = CST_GET_SCHAR( dc->Depth.cst );
			l->width[0]->depth[F_RIGHT] +=
					UNZCODE( cno, dc->Depth) + Br_depth[n];

			cno = CST_GET_SCHAR( dc->Depth.cst );
			l->width[0]->depth[F_LEFT] +=
					UNZCODE( cno, dc->Depth) + Br_depth[n];


			if( LINK_NEXT(l) != NULL )      break;


			cno = CST_GET_SCHAR( dc->Depth.cst );
			l->pr[1].p->depth +=
					UNZCODE( cno, dc->Depth) + Br_depth[n];

			cno = CST_GET_SCHAR( dc->Depth.cst );
			l->width[1]->depth[F_RIGHT] +=
					UNZCODE( cno, dc->Depth) + Br_depth[n];

			cno = CST_GET_SCHAR( dc->Depth.cst );
			l->width[1]->depth[F_LEFT] +=
					UNZCODE( cno, dc->Depth) + Br_depth[n];

		    break;
		}
	}
}
