/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**********************
 ***   DecHunch.c   ***
 **********************/
#include	<stdio.h>
#include	<math.h>

#include	"Uigp/igp.h"


#include	"GpType/GpType.h"

#include	"Ucode/code.h"


#include	"DecIn.h"
#include	"Coding/CodingHunch.h"

 


static void	decoding_hunch1( hunch_type *h,
			codingHunch_type *hc, int Fqa, bst_type *bst );



void
decoding_hunch_huffman( codingHunch_type *hc, bst_type *bst_file )
{
	hc->Hwidth.cst = decoding_agp_file_cut_huffman_cst( bst_file );
	hc->Htheta.cst = decoding_agp_file_cut_huffman_cst( bst_file );

	hc->Hval[SP_Y].cst = decoding_agp_file_cut_huffman_cst( bst_file );
	hc->Hval[SP_Q].cst = decoding_agp_file_cut_huffman_cst( bst_file );
}



void
decoding_hunch(	hunch_type *ah[], int hunch_no, codingHunch_type *hc, bst_type *bst )
{
int	Fqa,	i;

	if( hunch_no == 0 )	return;

	Fqa = bst_get( bst );


	for( i = 0 ; i < hunch_no ; i++ )      
		decoding_hunch1( ah[i], hc, Fqa, bst );
}



static void
decoding_hunch1( hunch_type *h, codingHunch_type *hc, int Fqa, bst_type *bst )
{
int	cno,	w_cno,	w_mask,	l_cno;
float	t;



	cno = CST_GET_SCHAR( hc->Hval[SP_Y].cst );
	h->rval[SP_Y]  = UNZCODE( cno, hc->Hval[SP_Y] );
	h->gval[SP_Y] += h->rval[SP_Y];


	if( Fqa == 0 && bst_get( bst ) == 0 ){

		cno = CST_GET_SCHAR( hc->Hval[SP_Q].cst );
		h->rval[SP_Q] =  UNZCODE( cno, hc->Hval[SP_Q] );
		h->gval[SP_Q] += h->rval[SP_Q];

		cno = CST_GET_SCHAR( hc->Hval[SP_Q].cst );
		h->rval[SP_A] = UNZCODE( cno, hc->Hval[SP_A] );
		h->gval[SP_A] += h->rval[SP_A];
	}
	else	 h->rval[SP_Q] = h->rval[SP_A] = 0;





	/* WIDTH */
	cno = CST_GET_SCHAR( hc->Hwidth.cst );

	w_mask = (1L<<hc->Hwidth.rbit) -1;
	w_cno = cno & w_mask;
	h->g.width = UNCODE( w_cno, hc->Hwidth ); 


	/* LEN */
	l_cno = cno >> hc->Hwidth.rbit;
	h->g.len = h->g.width + UNZCODE( l_cno, hc->Hlen ); 




	/* THETA */
	if( ABS(h->g.len - h->g.width) >= hc->DeltaR ){
		cno = CST_GET_SCHAR( hc->Htheta.cst );
		h->g.theta = UNCODE( cno, hc->Htheta );

		h->g.u.x = cos( (double)(h->g.theta) );
		h->g.u.y = sin( (double)(h->g.theta) );
	}
	else	 {
		h->g.theta = 0;
		h->g.u.x = 1.0;
		h->g.u.y = 0;
	}




	t = 1.0 /(h->g.width*h->g.width);
	h->lm1[SP_Y] = -h->rval[SP_Y] * t;
	h->lm1[SP_Q] = -h->rval[SP_Q] * t;
	h->lm1[SP_A] = -h->rval[SP_A] * t;

	t = 1.0 /(h->g.len*h->g.len);
	h->lm2[SP_Y] = -h->rval[SP_Y] * t;
	h->lm2[SP_Q] = -h->rval[SP_Q] * t;
	h->lm2[SP_A] = -h->rval[SP_A] * t;
}










void
decoding_hunch_close( codingHunch_type *hc )
{
	cst_free( hc->Hval[SP_Y].cst );
	cst_free( hc->Hval[SP_Q].cst );

	cst_free( hc->Hwidth.cst );
	cst_free( hc->Htheta.cst );
}
