/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
#include        <math.h>

#include        "Uigp/igp.h"

#include 	"Ucode/code.h"



#include	"DecIn.h"
#include	"Coding/CodingJoint.h"
#include	"Coding/CodingLink.h"


static void	decoding_joint_branch( jnt_type *jnt,coding_type *cprm,
			int bno, vec2d *base_u, gp_array *agp, bst_type *bst );

static void	decoding_joint_branchs( jnt_type *jnt, coding_type *cprm,
				gp_array *agp, bst_type *bst);
static void	decoding_joint_branch_agp( gp_array *agp,coding_type *cprm,
				bst_type *bst );

static link_type *	decoding_joint_last_link( gp_array *agp,
			int type, vec2d *ctr, link_type *link,
                        codingJoint_type *jc, bst_type *bst,
			jnt_type **njnt, int *nbno );

static void	decoding_joint_get_jnt_no( gp_type *gp, int type, int *jnt_no);
static jnt_type * decoding_joint_get_jnt( gp_type *gp, int type, int no );


static void	decoding_joint_info( jnt_type *jnt,
					codingN3_type *nc, bst_type *bst );
static void	decoding_joint_info_jnt_n3( jnt_type *jnt,
					codingN3_type *nc, bst_type *bst );
static void	decoding_joint_info_jnt_edge( jnt_type *jnt, bst_type *bst );
static void	decoding_joint_info_jnt_ridge( jnt_type *jnt, bst_type *bst );
static void	decoding_joint_info_jnt_RN3( jnt_type *jnt,
					codingN3_type *nc, bst_type *bst );


static void	decoding_joint_branch_center( jnt_type *jnt, int bno );







void
decoding_joint_huffman( codingJoint_type *jc, bst_type *bst_file )
{
	jc->Jend_f.cst = decoding_agp_file_cut_huffman_cst( bst_file );
}




void
decoding_joint( gp_array *agp,coding_type *cprm, bst_type *bst)
{
gp_type	*gp;
jnt_type	*jnt;
int	i,	j;


        for(i = 0 ; i < agp->Ni ; i++)
                for(j = 0 ; j < agp->Nj ; j++){

			gp = &agp->gp[i][j];
			for( jnt = gp->jnt ; jnt != NULL ; jnt = jnt->nj ){

				jnt->gctr.x = gp->ctr.x + jnt->ctr.x;
				jnt->gctr.y = gp->ctr.y + jnt->ctr.y;

				decoding_joint_info( jnt,cprm->nc, bst );

			}
		}

	decoding_joint_branch_agp( agp, cprm,bst );
}


static void
decoding_joint_branch_agp( gp_array *agp,coding_type *cprm, bst_type *bst )
{
gp_type *gp;
jnt_type        *jnt;
brn_type        *b;
int     i,      j,     bno;

	for( i = 0 ; i < agp->Ni ; i++ )
	    for( j = 0 ; j < agp->Nj ; j++ ){

		gp = &agp->gp[i][j];


		for( jnt = gp->jnt ; jnt != NULL ; jnt = jnt->nj ){

		    if( jnt->type == JNT_N3 ){
			b = jnt->brn[0];
			if( b->type != BRN_N3 && b->type != BRN_N3R )
				       continue;
			if( b->direction != F_BEGIN )	continue;


			decoding_joint_branch( jnt,cprm, 0, &cprm->lc->Base_u, agp, bst );
			continue;
		    }

		    if( jnt->type == JNT_RN3 )
			for( bno = 0 ; bno < jnt->brn_no ; bno++ ){
				b = jnt->brn[bno];
				if( b->type != BRN_N3 && b->type != BRN_N3R )
					continue;
				if( b->direction != F_BEGIN )	continue;

				decoding_joint_branch( jnt, cprm,bno,
						&cprm->lc->Base_u, agp, bst);
			}
		}
	}




	for( i = 0 ; i < agp->Ni ; i++ )
	    for( j = 0 ; j < agp->Nj ; j++ ){

		gp = &agp->gp[i][j];


		for( jnt = gp->jnt ; jnt != NULL ; jnt = jnt->nj )
			decoding_joint_branchs( jnt,cprm, agp, bst );
	    }
}







static void
decoding_joint_info( jnt_type *jnt,codingN3_type *nc ,bst_type *bst )
{

	switch( jnt->type ){   
	case JNT_N3:
		decoding_joint_info_jnt_n3( jnt,nc, bst );
	    break;
	case JNT_EDGE:
		decoding_joint_info_jnt_edge( jnt, bst );
	    break;
	case JNT_RIDGE:
		decoding_joint_info_jnt_ridge( jnt, bst );
	    break;
	case JNT_RN3:
		decoding_joint_info_jnt_RN3( jnt,nc, bst );
	    break;
	}
}



static void
decoding_joint_info_jnt_n3( jnt_type *jnt,codingN3_type *nc, bst_type *bst )
{
int	i,	flag;
char	w[2];
brn_type	*b;

	jnt->brn_no = 3;

	b = jnt->brn[0] = brn_alloc();

	flag = bst_get( bst );
	b->type = ( flag == 0 )? BRN_N3 : BRN_N3R;

	flag = bst_get( bst );
	b->direction = ( flag == 1 )? F_BEGIN : F_END;
	b->ctr.x = b->ctr.y = 0;

	for( i = 1 ; i < jnt->brn_no ; i++ ){
		b = jnt->brn[i] = brn_alloc();

		flag = bst_get_int( bst, 2 );

		switch( flag ){
		case 0:
			b->type = BRN_NULL;
			b->link = NULL;
		    break;
		case 1:
			b->type = BRN_IN;
			b->link = NULL;
		    break;
		case 2:
			b->type = BRN_EDGE;
			b->direction = F_END;
			b->link = NULL;
		    break;
		case 3:
			b->type = BRN_EDGE;
			b->direction = F_BEGIN;
			b->link = NULL;
		    break;
		}
        }


	w[0] = CST_GET_SCHAR( nc->Nwidth.cst );
	w[1] = CST_GET_SCHAR( nc->Nwidth.cst );

	jnt->lwidth = UNZCODE( w[0], nc->Nwidth) + nc->Nwidth.a;
	jnt->rwidth = UNZCODE( w[1], nc->Nwidth) + nc->Nwidth.a;
}


static void
decoding_joint_info_jnt_edge( jnt_type *jnt, bst_type *bst )
{
brn_type        *b;
int	i;
	
 
	jnt->brn_no = bst_get_int( bst, 2 );

        for( i = 0 ; i < jnt->brn_no ; i++ ){
		b = jnt->brn[i] = brn_alloc();
                b->type = BRN_EDGE;
                b->direction = F_BEGIN;
		b->link = NULL;
	}
}


static void
decoding_joint_info_jnt_ridge( jnt_type *jnt, bst_type *bst )
{
brn_type        *b;
int     i;
	
 
	jnt->brn_no = bst_get_int( bst, 2 );

        for( i = 0 ; i < jnt->brn_no ; i++ ){
		b = jnt->brn[i] = brn_alloc();
                b->type = BRN_RIDGE;
                b->direction = F_BEGIN;
		b->link = NULL;
	}
}


static void
decoding_joint_info_jnt_RN3( jnt_type *jnt,codingN3_type *nc, bst_type *bst )
{
brn_type	*b;
s_char	w[10];
int	flag,	i;
	
 
	jnt->brn_no = bst_get_int( bst, 2 );

        for( i = 0 ; i < jnt->brn_no ; i++ ){
		b = jnt->brn[i] = brn_alloc();

		flag = bst_get( bst );
                b->type = ( flag == 0 )? BRN_N3 : BRN_N3R;

                b->direction = F_BEGIN;
		b->link = NULL;
	}

	w[0] = CST_GET_SCHAR( nc->Nwidth.cst );
	w[1] = CST_GET_SCHAR( nc->Nwidth.cst );


	jnt->lwidth = UNZCODE( w[0], nc->Nwidth) + nc->Nwidth.a;
	jnt->rwidth = UNZCODE( w[1], nc->Nwidth) + nc->Nwidth.a;
}




static void
decoding_joint_branchs( jnt_type *jnt, coding_type *cprm,gp_array *agp, bst_type *bst)
{
brn_type        *b;
vec2d   base_u;
int	i;
 
 
	if( jnt->type == JNT_RN3 )	return;


	if( jnt->type == JNT_N3 ){
 
/*
		jnt_n3_uvector( jnt, &base_u );
*/
		base_u = jnt->u;
 
		for( i = 1 ; i < jnt->brn_no ; i++ ){
			b = jnt->brn[i];
			switch( b->type ){
			case BRN_NULL:
                            break;

			case BRN_IN:
				decoding_branch_in( agp, jnt, i, cprm->jc, bst );
				b->in = NULL;
			    break;

			case BRN_EDGE:
				if( b->direction == F_END )	break;

				decoding_joint_branch( jnt,cprm, i,
						&base_u, agp, bst );
			    break;
			}
		}
		return;
	}
 
 
	for( i = 0 ; i < jnt->brn_no ; i++ ){
		b = jnt->brn[i];
		if( b->direction == F_END )   continue;

		decoding_joint_branch( jnt,cprm, i, &cprm->lc->Base_u, agp, bst );
	}
}


static void
decoding_joint_branch( jnt_type *jnt,coding_type *cprm, int bno,
			vec2d *base_u, gp_array *agp, bst_type *bst )
{
jnt_type        *njnt;
brn_type        *b;
link_type	*link,	*nlink;
int	nbno;
vec2d	ctr;
 
        b = jnt->brn[bno];
	decoding_joint_branch_center( jnt, bno );
 


	ctr.x = jnt->gctr.x + b->ctr.x;
	ctr.y = jnt->gctr.y + b->ctr.y;

        link = decoding_link(cprm->lc, &ctr, b->type, base_u, 0 );

	nlink = decoding_joint_last_link( agp, b->type, &ctr, link,
					cprm->jc, bst, &njnt, &nbno );



/*
	jnt_insert_brn( b->type, &ctr, link, jnt, bno, njnt, nbno, 0 );
*/
	jnt_connet_link( jnt, bno, link, F_BEGIN );
	jnt_connet_link( njnt, nbno, nlink, F_END );

	agp_lnf_link( &ctr, link, agp );


	if( jnt->type == JNT_N3 && bno == 0 )
		jnt_n3_load_u( jnt );

//	if( njnt->type == JNT_N3 && bno == 0 )
	if( njnt->type == JNT_N3 && nbno == 0 )
		jnt_n3_load_u( njnt );


	if( b->type == BRN_N3 )
		decoding_n3_width( link ,cprm->nc);


	link_set_geo_aux( link );
}




static void
decoding_joint_branch_center( jnt_type *jnt, int bno )
{
brn_type	*b;

	b = jnt->brn[bno];
	
	if( jnt->type == JNT_N3 )	return;

	b->ctr.x = b->ctr.y = 0.0;
}



void
decoding_joint_close( codingJoint_type *jc )
{
	cst_free( jc->Jend_f.cst );
}



static	link_type *
decoding_joint_last_link( gp_array *agp, int type, vec2d *ctr, link_type *link,
			codingJoint_type *jc, bst_type *bst,
			jnt_type **njnt, int *nbno )
{
link_type	*l;
vec2d	v;
jnt_type	*jnt;
brn_type	*b;
int	bno,	jnt_no,	no;
int	i0,	j0,	i,	j,	di,	dj,	dij;

	for( l = link, v = *ctr ; LINK_NEXT(l) != NULL ; l = LINK_NEXT(l) ){
		v.x += l->v.x;
		v.y += l->v.y;
	}

	i0 = (v.x +agp->bgp) * agp->inv_bgp;
	j0 = (v.y +agp->bgp) * agp->inv_bgp;

	dij = CST_GET_SCHAR( jc->Jend_f.cst );

	di = dij/11 -5;
	dj = dij%11 -5;
	i = i0 + di;
	j = j0 + dj;

	decoding_joint_get_jnt_no( &agp->gp[i][j], type, &jnt_no );
	if( jnt_no > 1 )
		no = bst_get_int( bst, coding_ilog2( jnt_no-1 ) );
	else	no = 0;

	jnt = decoding_joint_get_jnt( &agp->gp[i][j], type, no );


	if( jnt->type == JNT_N3 ){
		bno = 0;
		if( type == BRN_EDGE )
			bno = bst_get( bst ) + 1;
	}
	else	{
		bno = jnt->brn_no++;
		jnt->brn[bno] = brn_alloc();
	}

        decoding_joint_branch_center( jnt, bno );



	b = jnt->brn[bno];
	l->v.x = jnt->gctr.x + b->ctr.x - v.x;
	l->v.y = jnt->gctr.y + b->ctr.y - v.y;
	link_set_aux( l );

	*njnt = jnt;
	*nbno = bno;

	return( l );
}


static void
decoding_joint_get_jnt_no( gp_type *gp, int type, int *jnt_no )
{
static const  int	Mtype[6] = { 0, 0, 8, 6, 18, 18 };
static const int	Jtype[5] = { 0, 2, 4, 8, 16 };

jnt_type	*cjnt;
int	mtype = Mtype[type];

	*jnt_no = 0;
	for( cjnt = gp->jnt ; cjnt != NULL ; cjnt = cjnt->nj ){

		if( !(mtype & Jtype[cjnt->type]) )	continue;

		(*jnt_no) ++;
	}
}

static jnt_type *
decoding_joint_get_jnt( gp_type *gp, int type, int no )
{
static const int	Mtype[6] = { 0, 0, 8, 6, 18, 18 };
static const int	Jtype[5] = { 0, 2, 4, 8, 16 };

jnt_type	*jnt;
int	mtype = Mtype[type],	jnt_no;

	jnt_no = 0;
	for( jnt = gp->jnt ; jnt != NULL ; jnt = jnt->nj ){

		if( !(mtype & Jtype[jnt->type]) )	continue;

		if( jnt_no == no )	return( jnt );

		jnt_no ++;
	}

	return( NULL );
}
