/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/

#include        "Ucode/code.h"

#include 	"Uigp/igp.h"



#include	"DecIn.h"
#include        "Coding/CodingDepth.h"


#define		BSIZE		2



static void	decoding_layer_gp( gp_type *gp,
			int Fbr, int br, int Ffb, codingDepth_type *dc );

static void	decoding_layer_gp_no( gp_type *gp, int *no );


void
decoding_layer_huffman( codingDepth_type *dc, bst_type *bst_file )
{
        dc->Br_f.cst = decoding_agp_file_cut_huffman_cst( bst_file );
        dc->Fbr_f.cst = decoding_agp_file_cut_huffman_cst( bst_file );
}




void
decoding_layer( gp_array *agp, codingDepth_type *dc, bst_type *bst )
{
int	i,	j,	n,	k;
int	no;
int	Fbr,	Ffbr;
int	br,	Lno;


	if( bst_get( bst ) == 0 ){
		Lno = 1;
		Fbr = 0;
		br = bst_get_int( bst, 8 );
	}
	else	Lno = 2;


		

	Ffbr = bst_get( bst );


	if( Lno == 1 && Ffbr == 0 ){
		for( i = 0 ; i < agp->Ni ; i++ )
			for( j = 0 ; j < agp->Nj ; j++ )
				decoding_layer_gp( &agp->gp[i][j], 0, br, 0, dc );
		return;
	}




	for( i = 0 ; i < agp->Ni ; i += BSIZE )
	    for( j = 0 ; j < agp->Nj ; j += BSIZE ){


		no = 0;

		for( k = i ; k < agp->Ni && k < i+BSIZE ; k++ )
		    for( n = j ; n < agp->Nj && n < j+BSIZE ; n++ )
			decoding_layer_gp_no( &agp->gp[k][n], &no );

		if( no == 0 )	continue;


		if( Lno > 1 ){
			Fbr = bst_get( bst );
			if( Fbr == 0 )	br = CST_GET_SCHAR( dc->Br_f.cst );
		}


		Ffbr = bst_get( bst );
 
 
                for( k = i ; k < agp->Ni && k < i+BSIZE ; k++ )
                    for( n = j ; n < agp->Nj && n < j+BSIZE ; n++ )
                        decoding_layer_gp( &agp->gp[k][n], Fbr, br, Ffbr, dc);

	    }
}


static void
decoding_layer_gp_no( gp_type *gp, int *no )
{
hunch_type	*h;
jnt_type	*jnt;
brn_type	*b;
int	i;


	for( h = gp->hunch ; h != NULL ; h = h->nh ){
		(*no)++;
	}


	for( jnt = gp->jnt ; jnt != NULL ; jnt = jnt->nj )
		for( i = 0 ; i < jnt->brn_no ; i++ ){
			b = jnt->brn[i];
			if( b->type == BRN_NULL )	continue;
			if( b->type == BRN_IN )	continue;
			if( b->direction != F_BEGIN )	continue;

			(*no)++;
		}
}



static void
decoding_layer_gp( gp_type *gp, int Fbr, int br, int Ffbr,
						codingDepth_type *dc )
{
hunch_type	*h;
jnt_type	*jnt;
brn_type	*b;
link_type	*l;
int	fbr,	br1;
int	i;



	for( h = gp->hunch ; h != NULL ; h = h->nh ){
		if( Fbr == 1 )	br = CST_GET_SCHAR( dc->Br_f.cst );
		h->br = br;
	}


	fbr = 0;

	for( jnt = gp->jnt ; jnt != NULL ; jnt = jnt->nj ){
	    for( i = 0 ; i < jnt->brn_no ; i++ ){
		b = jnt->brn[i];

		if( b->type == BRN_NULL )	continue;
		if( b->type == BRN_IN )	continue;

		if( b->direction != F_BEGIN )	continue;

		if( Fbr == 1 )	br = CST_GET_SCHAR( dc->Br_f.cst );

		switch( b->type ){
		case NF_RIDGE:
		case NF_N3R:
			for( l = b->link ; l != NULL ; l = LINK_NEXT(l) ){
				l->br = br;
				if( LINK_NEXT(l) == NULL )	break;
			}
		    break;

		case NF_EDGE:
			if( Ffbr == 1 )	fbr = CST_GET_SCHAR(dc->Fbr_f.cst);

			if( fbr == 0x04 )
				br1 = CST_GET_SCHAR( dc->Br_f.cst );
			else	br1 = br;
			for( l = b->link ; l != NULL ; l = LINK_NEXT(l) ){
				l->br = br;
				l->br1 = br1;
				l->Fbr = fbr;
			}
		    break;

		case NF_N3:
			if( Ffbr == 1 )	fbr = CST_GET_SCHAR(dc->Fbr_f.cst);
			for( l = b->link ; l != NULL ; l = LINK_NEXT(l) ){
				l->br = br;
				l->Fbr = fbr;
			}
		    break;
		}
	    }
	}


}





void
decoding_layer_close( codingDepth_type *dc )
{
        cst_free( dc->Br_f.cst );
        cst_free( dc->Fbr_f.cst );
}
