/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
#include        <stdio.h>
#include        <math.h>
 
#include        "Uigp/igp.h"

#include        "GpType/GpType.h"
 
#include        "Ucode/code.h"


#define         MAX_LINK_NO        256



#include	"DecIn.h"
#include	"Coding/CodingLink.h"

static void
decoding_link_one( codingLink_type *lc,vec2d *u, int len, int w, int a, link_type *l );


/* declare in  Coding/CodingLink.h
 *
 * code_type    Llen,   Lwidth, La;
 *
 * cfile_type	Lino_f;
 *
 * vec2d   Base_u;
 */





void  decoding_link_huffman( codingLink_type *lc,bst_type * bst_file   )
{
	lc->Lino_f.cst = decoding_agp_file_cut_huffman_cst( bst_file );
 
	lc->Llen.cst = decoding_agp_file_cut_huffman_cst( bst_file );

	lc->Lwidth.cst = decoding_agp_file_cut_huffman_cst( bst_file );
	
	lc->La.cst = decoding_agp_file_cut_huffman_cst( bst_file );
}






link_type *
decoding_link(codingLink_type *lc, vec2d *ctr, int type, vec2d *base_u, int flag  )
{
signed char	w[MAX_LINK_NO],	len[MAX_LINK_NO],	a[MAX_LINK_NO];
link_type	*link,	*l;
vec2d	*u,	p;
int	i,	n,	ino;

	ino = CST_GET_UCHAR( lc->Lino_f.cst ) + 1;

	link = link_alloc( type, ino );
	if( type == NF_N3 )
		link_alloc_width( link );

	n = ( flag == 1 ) ? ino: ino-1;

	cst_get_buffer( lc->Llen.cst, (u_char *)len, n );
	cst_get_buffer( lc->Lwidth.cst, (u_char *)w, n );
	cst_get_buffer( lc->La.cst, (u_char *)a, ino );


	if( flag == 0 ){
		len[ino-1] = 2.0;
		w[ino-1] = 2.0;
	}



	/* decoding first link */
	u = base_u;

	decoding_link_one( lc,u, len[0], w[0], a[0], link );

	p.x = QCODE( ctr->x, lc->Llen );
	p.y = QCODE( ctr->y, lc->Llen );
	link->v.x += p.x - ctr->x;
	link->v.y += p.y - ctr->y;
	link_set_aux( link );



	u = &link->u;
	for( l = LINK_NEXT(link), i = 1 ; l != NULL ; l = LINK_NEXT(l), i++ ){

		decoding_link_one(lc, u, len[i], w[i], a[i], l );



		link_set_aux( l );

		u = &l->u;
	}

	return( link );
}



void decoding_link_close(codingLink_type *hc)
{
	cst_free( hc->Lino_f.cst );
	cst_free( hc->Llen.cst );
	cst_free( hc->Lwidth.cst );
	cst_free( hc->La.cst );
}




static void
decoding_link_one( codingLink_type *lc,vec2d *u, int len, int w, int a, link_type *l )
{
code_type	ucode;
vec2d	v,	v1;
float	dx,	dy,	dx1,	dy1;

	ucode.d = lc->Llen.d * MAX( ABS(u->x), ABS(u->y) ) -0.0001;
	ucode.inv_d = 1.0/ucode.d;
	v.x =  UNZCODE( len, ucode );
	v.y =  UNZCODE( w, ucode );

	v1.x = v.x * -u->y + v.y * u->x;
	v1.y = v.x * u->x + v.y * u->y;
	l->v.x = QZCODE( v1.x, lc->Llen );
	l->v.y = QZCODE( v1.y, lc->Llen );

	l->a =  UNZCODE( a, lc->La );



	dx = -u->y * (l->v.x-v1.x) + u->x * (l->v.y-v1.y);
	dy =  u->x * (l->v.x-v1.x) + u->y * (l->v.y-v1.y);

	if( ABS(dx) <= 0.5*ucode.d && ABS(dy) <= 0.5*ucode.d )
		return;

	dx1 = dx + -u->y*lc->Llen.d;
	dy1 = dy +  u->x*lc->Llen.d;
	if( ABS(dx1) <= 0.5*ucode.d && ABS(dy1) <= 0.5*ucode.d ){
		l->v.x += lc->Llen.d;
		return;
	}

	dx1 = dx - -u->y*lc->Llen.d;
	dy1 = dy -  u->x*lc->Llen.d;
	if( ABS(dx1) <= 0.5*ucode.d && ABS(dy1) <= 0.5*ucode.d ){
		l->v.x -= lc->Llen.d;
		return;
	}

	dx1 = dx +  u->x*lc->Llen.d;
	dy1 = dy +  u->y*lc->Llen.d;
	if( ABS(dx1) <= 0.5*ucode.d && ABS(dy1) <= 0.5*ucode.d ){
		l->v.y += lc->Llen.d;
		return;
	}

	dx1 = dx -  u->x*lc->Llen.d;
	dy1 = dy -  u->y*lc->Llen.d;
	if( ABS(dx1) <= 0.5*ucode.d && ABS(dy1) <= 0.5*ucode.d ){
		l->v.y -= lc->Llen.d;
		return;
	}
}
