/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/*************************
 ***	DecProfile.c   ***
 *************************/


#include        "Uigp/igp.h"


#include	"DecIn.h"
#include        "Coding/CodingProfile.h"

#include        "Coding/Coding.h"



static void	decoding_profile_edge( gp_array *agp,codingProfile_type *pc,
				vec2d *gctr, link_type *link, bst_type *bst );


static void	decoding_profile_n3( gp_array *agp,codingProfile_type *pc,
				vec2d *lctr, link_type *link, bst_type *bst );


static void	decoding_profile_n3_side( gp_array *agp,codingProfile_type *pc,
				vec2d *lctr, link_type *link, int side,
				int *Fz, bst_type *bst );






void
decoding_profile_huffman(codingProfile_type *pc, bst_type *bst_file )
{
	pc->Ewidth.cst = decoding_agp_file_cut_huffman_cst( bst_file );

	pc->Pval.cst = decoding_agp_file_cut_huffman_cst( bst_file );

	pc->Pnull.cst = decoding_agp_file_cut_huffman_cst( bst_file );
}


void
decoding_profile( gp_array *agp,codingProfile_type *pc, bst_type *bst )
{
int     i,      j,      bno;
gp_type *gp;
jnt_type        *jnt;
brn_type        *b;
vec2d   ctr;

	for( i = 0 ; i < agp->Ni ; i++ )
	    for( j = 0 ; j < agp->Nj ; j++ ){

		gp = &agp->gp[i][j];

		for( jnt = gp->jnt ; jnt != NULL ; jnt = jnt->nj ){

			for( bno = 0 ; bno < jnt->brn_no ; bno++ ){
				b = jnt->brn[bno];

				if( b->direction != F_BEGIN )   continue;

				ctr.x = jnt->gctr.x + b->ctr.x;
				ctr.y = jnt->gctr.y + b->ctr.y;

				if( b->type == BRN_EDGE ){
					decoding_profile_edge( agp,pc,
							&ctr, b->link, bst );
					continue;
				}
				if( b->type == BRN_N3 ){
					decoding_profile_n3( agp,pc,
							&ctr, b->link, bst );
					continue;
				}
			}
		}
	}    


	coding_profile_load_from_jnt( agp );
}




void
decoding_profile_close( codingProfile_type *pc )
{
	cst_free( pc->Ewidth.cst );

	cst_free( pc->Pval.cst );

	cst_free( pc->Pnull.cst );
}








static void
decoding_profile_edge( gp_array *agp,codingProfile_type *pc,
				vec2d *gctr, link_type *link, bst_type *bst )
{
edge_type       *e,	*te;
float	width;
int	w,	tmp;

int	i,	j,	flag;

#ifndef PALMOS
profile_type	pr[400];
int	gi[300],	gno,	pr_no,	Bflag;

qcolor_type	Rval[300];
qcolor_type	Lval[300];
#else
profile_type *	pr;
int	* gi,	gno,	pr_no,	Bflag;

qcolor_type	* Rval;
qcolor_type	* Lval;
	pr = (profile_type *)malloc(sizeof(profile_type) * 400);
	gi = (int *)malloc(sizeof(int) * 300);
	Rval = (qcolor_type	*)malloc(sizeof(qcolor_type) * 300);
	Lval = (qcolor_type	*)malloc(sizeof(qcolor_type) * 300);
#endif



	coding_profile_group_edge( agp, gctr, link,
				pr, &pr_no, gi, &gno, &Bflag );

	if( gno == 0 )
	{
#ifdef PALMOS
		free(pr);
		free(gi);
		free(Rval);
		free(Lval);
#endif
		return;
	}


	if( Bflag != 0 && gno == 1 ){
		tmp = CST_GET_SCHAR( pc->Pnull.cst );
		Rval[0] =  UNZCODE( tmp, pc->Pnull );

		tmp = CST_GET_SCHAR( pc->Pnull.cst );
		Lval[0] =  UNZCODE( tmp, pc->Pnull );

		w = CST_GET_SCHAR( pc->Ewidth.cst );
		width = UNZCODE( w, pc->Ewidth ) + pc->Ewidth.a;

		for( j = gi[0] ; j < gi[0+1] ; j++ ){

			e = pr[j].edge;
			e->width = width;
			e->side[F_RIGHT].val[SP_Y] =
					e->side[F_RIGHT].bval[SP_Y] - Rval[0];
			e->side[F_LEFT].val[SP_Y] =
					e->side[F_LEFT].bval[SP_Y] + Lval[0];

			e->side[F_RIGHT].val[SP_Q] = e->side[F_RIGHT].bval[SP_Q];
			e->side[F_LEFT].val[SP_Q] = e->side[F_LEFT].bval[SP_Q];

			e->side[F_RIGHT].val[SP_A] = e->side[F_RIGHT].bval[SP_A];
			e->side[F_LEFT].val[SP_A] = e->side[F_LEFT].bval[SP_A];
		}
#ifdef PALMOS
		free(pr);
		free(gi);
		free(Rval);
		free(Lval);
#endif
		return;
	}


	if( Bflag == 0x03 && gno == 2 ){
		tmp = CST_GET_SCHAR( pc->Pnull.cst );
		Rval[0] = UNZCODE( tmp, pc->Pnull );

		tmp = CST_GET_SCHAR( pc->Pnull.cst );
		Lval[0] = UNZCODE( tmp, pc->Pnull );

		tmp = CST_GET_SCHAR( pc->Pnull.cst );
		Rval[1] = UNZCODE( tmp, pc->Pnull );

		tmp = CST_GET_SCHAR( pc->Pnull.cst );
		Lval[1] = UNZCODE( tmp, pc->Pnull );

		w = CST_GET_SCHAR( pc->Ewidth.cst );
		width = UNZCODE( w, pc->Ewidth ) + pc->Ewidth.a;

		for( j = gi[0] ; j < gi[0+1] ; j++ ){

			e = pr[j].edge;
			e->width = width;
			e->side[F_RIGHT].val[SP_Y] =
					e->side[F_RIGHT].bval[SP_Y] - Rval[0];
			e->side[F_LEFT].val[SP_Y] =
					e->side[F_LEFT].bval[SP_Y] + Lval[0];

			e->side[F_RIGHT].val[SP_Q] = e->side[F_RIGHT].bval[SP_Q];
			e->side[F_LEFT].val[SP_Q] = e->side[F_LEFT].bval[SP_Q];

			e->side[F_RIGHT].val[SP_A] = e->side[F_RIGHT].bval[SP_A];
			e->side[F_LEFT].val[SP_A] = e->side[F_LEFT].bval[SP_A];
		}

		w = CST_GET_SCHAR( pc->Ewidth.cst );
		width = UNZCODE( w, pc->Ewidth ) + pc->Ewidth.a;

		for( j = gi[1] ; j < gi[1+1] ; j++ ){

			e = pr[j].edge;
			e->width = width;
			e->side[F_RIGHT].val[SP_Y] =
					e->side[F_RIGHT].bval[SP_Y] - Rval[1];
			e->side[F_LEFT].val[SP_Y] =
					e->side[F_LEFT].bval[SP_Y] + Lval[1];

			e->side[F_RIGHT].val[SP_Q] = e->side[F_RIGHT].bval[SP_Q];
			e->side[F_LEFT].val[SP_Q] = e->side[F_LEFT].bval[SP_Q];

			e->side[F_RIGHT].val[SP_A] = e->side[F_RIGHT].bval[SP_A];
			e->side[F_LEFT].val[SP_A] = e->side[F_LEFT].bval[SP_A];
		}
#ifdef PALMOS
		free(pr);
		free(gi);
		free(Rval);
		free(Lval);
#endif
		return;
	}
		

		
	flag = bst_get( bst );
	if( flag == 1 ){
		for( i = 0  ;  i < gno ; i++ ){
			tmp = CST_GET_SCHAR( pc->Pval.cst );
			Rval[i] = UNZCODE( tmp, pc->Pval );
			tmp = CST_GET_SCHAR( pc->Pval.cst );
			Lval[i] = UNZCODE( tmp, pc->Pval );
		}
	}
	else	for( i = 0 ; i < gno ; i++ )
			Rval[i] = Lval[i] = 0;





	for( i = 0 ; i < gno ; i++ ){
		w = CST_GET_SCHAR( pc->Ewidth.cst );
		width = UNZCODE( w, pc->Ewidth ) + pc->Ewidth.a;

		for( j = gi[i] ; j < gi[i+1] ; j++ ){
			e = pr[j].edge;

			e->width = width;
			e->side[F_RIGHT].val[SP_Y] =
					e->side[F_RIGHT].bval[SP_Y] - Rval[i];
			e->side[F_LEFT].val[SP_Y] =
					e->side[F_LEFT].bval[SP_Y] + Lval[i];

			e->side[F_RIGHT].val[SP_Q] = e->side[F_RIGHT].bval[SP_Q];
			e->side[F_LEFT].val[SP_Q] = e->side[F_LEFT].bval[SP_Q];

			e->side[F_RIGHT].val[SP_A] = e->side[F_RIGHT].bval[SP_A];
			e->side[F_LEFT].val[SP_A] = e->side[F_LEFT].bval[SP_A];
		}
	}


	if( Bflag & 0x01 ){
		te = pr[ gi[1] ].edge;
		for( j = gi[0] ; j < gi[0+1] ; j++ ){

			e = pr[j].edge;
			e->side[F_RIGHT].val[SP_Y] =
				te->side[F_RIGHT].bval[SP_Y] - Rval[0] ;
			e->side[F_LEFT].val[SP_Y] =
					te->side[F_LEFT].bval[SP_Y] + Lval[0];
		}
	}

	if( Bflag & 0x02 ){
		te = pr[ gi[gno-1]-1 ].edge;

		for( j = gi[gno-1] ; j < gi[gno] ; j++ ){

			e = pr[j].edge;
			e->side[F_RIGHT].val[SP_Y] =
				te->side[F_RIGHT].bval[SP_Y] - Rval[gno-1] ;
			e->side[F_LEFT].val[SP_Y] =
				te->side[F_LEFT].bval[SP_Y] + Lval[gno-1];
		}
	}
#ifdef PALMOS
		free(pr);
		free(gi);
		free(Rval);
		free(Lval);
#endif
}


static void
decoding_profile_n3( gp_array *agp,codingProfile_type *pc,
				vec2d *lctr, link_type *link, bst_type *bst )
{
int	Fz;

	Fz = -1;


	decoding_profile_n3_side( agp,pc, lctr, link, F_RIGHT, &Fz, bst );

	decoding_profile_n3_side( agp,pc, lctr, link, F_LEFT, &Fz, bst );
}

	




static void
decoding_profile_n3_side( gp_array *agp,codingProfile_type *pc,
				vec2d *lctr, link_type *link, int side,
				int *Fz, bst_type *bst )
{
link_type	*l,	*pl;
jnt_type	*jnt;
in_type	*in;
n3_type	*e;
qcolor_type		dval;
vec2d   ctr;
int     i0,     j0,     i1,     j1;
int	flag;
int	temp;

	i0 = j0 = -100;


        flag = F_BEGIN;
        jnt = link->p[0].p.jnt;
        if( jnt->type == JNT_N3 && jnt->brn[2-side]->type == BRN_IN &&
				jnt->brn[2-side]->link->type == BRN_N3 )
                flag = F_END;


	for( l = link, ctr = *lctr ; l != NULL ; l = LINK_NEXT(l ) ){


		if( flag == F_BEGIN ){
			i1 = (ctr.x+agp->bgp) * agp->inv_bgp;
			j1 = (ctr.y+agp->bgp) * agp->inv_bgp;

			if( i0 != i1 || j0 != j1 ){

				if( *Fz == -1 )	*Fz = bst_get( bst );

				if( *Fz == 1 ){
					temp = CST_GET_SCHAR( pc->Pval.cst );
					dval = UNZCODE( temp, pc->Pval );
				}
				else	dval = 0;
		
				i0 = i1;
				j0 = j1;
			}
		}
		else	dval = 0;

		e = l->pr[0].n3;
		e->side[side].val[SP_Y] = e->side[side].bval[SP_Y] + dval;

		e->side[side].val[SP_Q] = e->side[side].bval[SP_Q];

		e->side[side].val[SP_A] = e->side[side].bval[SP_A];



                if( l->in[side] != NULL ){
                        for( in = l->in[side] ; in->p != NULL ; in = in->p );
			if( in->jnt->brn[in->bno]->link->type == BRN_N3 ){
                        	flag = in->direct;

                        	i0 = j0 = -100;
			}
                }


		ctr.x += l->v.x;
		ctr.y += l->v.y;

		pl = l;
	}




	if( flag == F_BEGIN ){

		i1 = (ctr.x+agp->bgp) * agp->inv_bgp;
		j1 = (ctr.y+agp->bgp) * agp->inv_bgp;

		if( i0 != i1 || j0 != j1 ){

			if( *Fz == -1 )	*Fz = bst_get( bst );

			if( *Fz == 1 ){
				temp = CST_GET_SCHAR( pc->Pval.cst );
				dval = UNZCODE( temp, pc->Pval );

			}
			else	dval = 0;


			i0 = i1;
			j0 = j1;
		}
	}
	else	dval = 0;


	e = pl->pr[1].n3;
	e->side[side].val[SP_Y] = e->side[side].bval[SP_Y] + dval;

	e->side[side].val[SP_Q] = e->side[side].bval[SP_Q];

	e->side[side].val[SP_A] = e->side[side].bval[SP_A];
}
