/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/***********************
 ***	DecRidge.c   ***
 ***********************/
#include	<math.h>

#include 	"Uigp/igp.h"




#include	"DecIn.h"
#include        "Coding/CodingRidge.h"


 


static void	decoding_ridge_link( link_type *link, int Fqa,
				codingRidge_type *rc ,bst_type *bst );



static void	decoding_ridge_val( ridge_type *r, int Fqa, int sign,
				codingRidge_type *rc ,bst_type *bst );



void
decoding_ridge_huffman( codingRidge_type* rc,bst_type *bst_file )
{
	rc->Rlm.cst = decoding_agp_file_cut_huffman_cst( bst_file );

	rc->Rval[SP_Y].cst = decoding_agp_file_cut_huffman_cst( bst_file );

	rc->Rval[SP_Q].cst = decoding_agp_file_cut_huffman_cst( bst_file );

}	



void
decoding_ridge( link_type *link[], int ridge_no,
			codingRidge_type *rc, bst_type *bst )
{
int	Fqa,	i;


	if( ridge_no == 0 )	return;

	Fqa = bst_get( bst );


	for( i = 0 ; i < ridge_no ; i++ )
		decoding_ridge_link( link[i], Fqa, rc, bst );

}



void
decoding_ridge_close(codingRidge_type *rc )
{
	cst_free( rc->Rlm.cst );

	cst_free( rc->Rval[SP_Y].cst );
	cst_free( rc->Rval[SP_Q].cst );

}





static void
decoding_ridge_link( link_type *link, int Fqa,
				codingRidge_type *rc ,bst_type *bst )
{
link_type	*l,	*nl;
ridge_type	*r;
int	sign;

	sign = bst_get( bst );

	for( l = link ; l != NULL ; l = LINK_NEXT(l) ){
		r = l->pr[0].ridge;


		decoding_ridge_val( r, Fqa, sign, rc, bst );

		nl = l;
	}



	r = nl->pr[1].ridge;

	decoding_ridge_val( r, Fqa, sign, rc, bst );
}




#define	WMIN	0.75F
#define	WMIN2	0.5625F

#define	WMAX	1.5F
#define	WMAX2	2.25F


static void
decoding_ridge_val( ridge_type *r, int Fqa, int sign,
				codingRidge_type *rc ,bst_type *bst )
{
int	cno;
float	w2;


	cno = CST_GET_SCHAR( rc->Rlm.cst );
	 r->lm[SP_Y] = UNCODE( cno, rc->Rlm );
	if( sign != 0 )	r->lm[SP_Y] = -r->lm[SP_Y];

	cno = CST_GET_SCHAR( rc->Rval[SP_Y].cst );

	 r->rval[SP_Y] = UNZCODE( cno, rc->Rval[SP_Y] );

	if( sign == 0 )	r->rval[SP_Y] = -r->rval[SP_Y];
	r->mval[SP_Y] += r->rval[SP_Y];



	w2 = - r->rval[SP_Y] / (float)r->lm[SP_Y];
	if( w2 < WMIN2 ){
		r->width = WMIN;
		w2 = WMIN2;
	}
	else	if( w2 > WMAX2 ){
			r->width = WMAX;
			w2 = WMAX2;
		}
			else	r->width = sqrt( w2 );



	if( Fqa == 0 && bst_get( bst ) == 0 ){

		cno = CST_GET_SCHAR( rc->Rval[SP_Q].cst );
		r->rval[SP_Q] = UNZCODE( cno, rc->Rval[SP_Q] );
		r->mval[SP_Q] += r->rval[SP_Q];

		r->lm[SP_Q] =  -r->rval[SP_Q]/w2;


		cno = CST_GET_SCHAR( rc->Rval[SP_Q].cst );
		r->rval[SP_A] = UNZCODE( cno, rc->Rval[SP_A] );
		r->mval[SP_A] += r->rval[SP_A];

		r->lm[SP_A] =  -r->rval[SP_A]/w2;
	}
	else	{
		r->rval[SP_Q] = r->rval[SP_A] = 0;
		r->lm[SP_Q] = r->lm[SP_A] = 0;
	}
}
