/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/

/**************************
 ***   Ep2Bound.c  	***
 **************************/

#include	"Uigp/igp.h"


#include        "Ep2In.h"


static void	ep2_bound( ep2_array *aep2, ep2_type *ep2 );

static bound2_type * ep2_bound_new( ep2_array *aep2, ep2_type *ep2 );

static void	ep2_bound_continuous_componnet( ep2_type *ep2, bound2_type *b,
			int i0, int bno0, short fbound[4], int Fnull[4] );

static bound2_type * ep2_bound_new( ep2_array *ap2, ep2_type *ep2 );

static int	ep_bound_set_null_flag( ep_type *ep );


static void	ep2_bound_order( ep_type *ep, int bi[], int *b_no );



void
ep2_array_bound( ep2_array *aep2 )
{
int	i,	j;


        for( i = 0 ; i < aep2->Ni ; i++ )
                for( j = 0 ; j < aep2->Nj ; j++ )

			ep2_bound( aep2, &aep2->ep[i][j] );
}




static const int	Mc[4][4] = {	{  4,  5,  7,  8 },
				{  3,  4,  6,  7 },
				{  1,  2,  4,  5 },
				{  0,  1,  3,  4 } };


static void
ep2_bound( ep2_array *aep2, ep2_type *ep2 )
{
short	fbound[4];
int	Fnull[4];
int	bi[MAX_BOUND],	b_no;
bound2_type	*b;
int	i,	j,	bno;

	ep2->b_no = 0;
	fbound[0] = fbound[1] = fbound[2] = fbound[3] = 0;
	Fnull[0] = ep_bound_set_null_flag( ep2->ep[0] );
	Fnull[1] = ep_bound_set_null_flag( ep2->ep[1] );
	Fnull[2] = ep_bound_set_null_flag( ep2->ep[2] );
	Fnull[3] = ep_bound_set_null_flag( ep2->ep[3] );


	for( i = 0 ; i < 4 ; i++ ){
		if( ep2->ep[i] == NULL )	continue;

		ep2_bound_order( ep2->ep[i], bi, &b_no );

		for( j = 0 ; j < b_no ; j++ ){
			bno = bi[j];

			if( fbound[i] & (1L<< bno) ) continue;

			b = ep2_bound_new( aep2, ep2 );
			ep2_bound_continuous_componnet( ep2, b, i, bno,
						fbound, Fnull );


			b->Mctr.x /= b->no;
			b->Mctr.y /= b->no;
		}
	}
}

static void
ep2_bound_continuous_componnet( ep2_type *ep2, bound2_type *b2,
			int i0, int bno0, short fbound[4], int Fnull[4] )
{
bound_type	*b;
int	i,	bno,	fb;

	b = ep2->ep[i0]->bound[bno0];

	b->b2_no =  ep2->b_no-1;

	fbound[i0] |= ( 1L<<bno0 );


	if( b->br.no != 255  &&  b2->Sbr == 0 ){
		b2->br = b->br.no;
		b2->Sbr = b->br.flag;
	}


	if( b->Mbgp != NULL ){
		b2->Fbound[i0] |= ( 1L<<bno0 );
		b2->Mctr.x += b->Mbgp->ctr.x;
		b2->Mctr.y += b->Mbgp->ctr.y;

		if( b2->type != B2_OPENED )
			if( Fnull[i0] & (1L<<bno0) )
				 b2->type = B2_OPENED;
		b2->no++;
	}

	for( i = 0 ; i < 4 ; i++ ){
/*
		if( Mc[i0][i] == -1 )	continue;
*/
		if( ep2->ep[i] == NULL )	continue;

		fb = b->Fcont[ Mc[i0][i] ];

		if( fb == 0 )	continue;

		for( bno = 0 ; bno < MAX_BOUND2 ; bno++ ){

			if( !(fb & ( 1L<<bno ))   )	continue;
			if( fbound[i] & (1L<< bno) )	continue;

			ep2_bound_continuous_componnet( ep2, b2,
					i, bno, fbound, Fnull );
		}
	}
}


static bound2_type *
ep2_bound_new( ep2_array *aep2, ep2_type *ep2 )
{
bound2_type	*b;

	AEP2_ALLOC_DATA( aep2, b, bound2_type );
	ep2->bound[ep2->b_no++] = b;


	b->no = 0;

	b->Fbound[0] = b->Fbound[1] = b->Fbound[2] = b->Fbound[3] = 0;

	b->Mctr.x = b->Mctr.y = 0;
	b->type = B2_CLOSED;

	b->br = -1;
	b->Sbr = 0;

	return( b );
}



static int
ep_bound_set_null_flag( ep_type *ep )
{
al_type	*al;
int	flag,	bno;
	if( ep == NULL )	return( 0 );


	flag = 0;

	for( al = ep->al ; al != NULL ; al = al->nal ){
		if( al->bno[F_RIGHT] != al->bno[F_LEFT] )	continue;
		bno = al->bno[F_RIGHT];
		flag |= ( 1L << bno );
	}

	return( flag );
}



static void
ep2_bound_order( ep_type *ep, int bi[], int *b_no )
{
bound_type *ab[MAX_BOUND];
bound_type	*b;
int	i,	j;

	*b_no = 0 ;
	for( i = 0 ; i < ep->b_no ; i++ ){

		b = ep->bound[i];


		if( b->Mbgp == NULL )	continue;

		for( j = *b_no ; j > 0 ; j-- ){

			if( ab[j-1]->Mbgp->ctr.x < b->Mbgp->ctr.x ||
				ab[j-1]->Mbgp->ctr.x == b->Mbgp->ctr.x &&
				ab[j-1]->Mbgp->ctr.y < b->Mbgp->ctr.y )
				break;

			bi[j] = bi[j-1];
			ab[j] = ab[j-1];
		}

		bi[j] = i;
		ab[j] = b;
		(*b_no) ++;
	}
}
