/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/

/**************************
 ***   Bg2Connect.c  	***
 **************************/

#include		"Uigp/igp.h"


#include        "Ep2In.h"


static void		ep2_connect( ep2_type *ep2 );

static void		ep2_connect_clear(ep2_array *aep2 );


void
ep2_array_connect(ep2_array * aep2 )
{
int	i,	j;

	ep2_connect_clear( aep2 );

	for( i = 0 ; i < aep2->Ni ; i++ )
		for( j = 0 ; j < aep2->Nj ; j++ )

			ep2_connect( &aep2->ep[i][j] );
}



static const signed char	Me[4][5] = {	{  0, 1, 2, 3, 6 },
				{  0, 1, 2, 5, 8 },
				{  0, 3, 6, 7, 8 },
				{  2, 5, 6, 7, 8 } };


static const signed char Mn[4][9] = {	{   0,  1,  1,  3, -1, -1,  3, -1, -1 },
				{   1,  1,  2, -1, -1,  5, -1, -1,  5 },
				{   3, -1, -1,  3, -1, -1,  6,  7,  7 },
				{  -1, -1,  5, -1, -1,  5,  7,  7,  8 } };



static void
ep2_connect( ep2_type *ep2 )
{
ep_type	*ep,	*nep;
bound_type	*b,	*nb;
int	i,	l,	k,	bno,	nb_no,	nb2_no;
int	fcont;

	for( i = 0 ; i < 4 ; i++ ){

		if( (ep = ep2->ep[i]) == NULL )	continue;

		for( bno = 0 ; bno < ep->b_no ; bno ++ ){
			b = ep->bound[bno];
			if( b->b2_no == -1 )	continue;

			for( l = 0 ; l < 5 ; l ++ ){
				nb_no = Me[i][l];
				if( (fcont = b->Fcont[nb_no]) == 0 )	continue;

				nep = ep->ng[ nb_no];

				for( k = 0 ; fcont != 0 ; fcont >>= 1, k++ ){

					if( !(fcont & 0x01) )	continue;

					nb = nep->bound[k];
					if( nb->b2_no == -1 )	continue;

					nb2_no = Mn[i][nb_no];
					ep2->bound[b->b2_no]->Fcont[nb2_no] |=
						(1L << nb->b2_no);
				}
			}
		}
	}
}

			

static void
ep2_connect_clear(ep2_array * aep2)
{
int             i, j, bno2, k;
ep2_type       *ep2;
bound2_type    *b2;

	for (i = 0; i < aep2->Ni; i++)
		for (j = 0; j < aep2->Nj; j++) {
			ep2 = &aep2->ep[i][j];
			for (bno2 = 0; bno2 < ep2->b_no; bno2++) {

				b2 = ep2->bound[bno2];

				for (k = 0; k < 9; k++)
					b2->Fcont[k] = 0;
			}
		}
}
