/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/

/**************************
 ***   Ep2Tool.c  	***
 **************************/

#include	"Uigp/igp.h"


#include	"Ep2Type.h"

#include	"Ep2In.h"

#ifdef PALMOS
	#define MALLOC	malloc2
	#define FREE	free2
#else
	#define MALLOC	malloc
	#define FREE	free
#endif

static void	ep2_array_alloc( ep2_array *aep, int ni, int nj );

static void	ep2_array_free( ep2_array *aep );


ep2_array *
aep2_alloc()
{
ep2_array	*aep;

	aep = (ep2_array *)malloc( sizeof( ep2_array ) );

	aep->Ni = aep->Nj = -1;

	aep->NI = aep->NJ = -1;

	aep->ep = NULL;

	return( aep );
}



void
aep2_destroy( ep2_array *aep )
{
	ep2_array_free( aep );

	free( aep );
}


void
ep2_array_init(ep2_array *aep2, ep_array *aep )
{
ep2_type	*ep2;
ep_type		*ep;
int	Ni,	Nj;
int	i,	j,	i2,	j2,	k;
vec2d	v;

	Ni = (aep->Ni-2*LSIZE + 1 ) >> 1;
	Nj = (aep->Nj-2*LSIZE + 1 ) >> 1;

	if( Ni > aep2->NI || Nj > aep2->NJ ){
		ep2_array_free( aep2 );
		ep2_array_alloc( aep2, Ni, Nj );
	}

	aep2->Ni = Ni;
	aep2->Nj = Nj;


 
	aep2->bgp = aep->bgp * 2 ; 
	aep2->bgp_inv = (float)(1. / aep2->bgp); 

	for( i = 0, i2 = LSIZE, v.x = aep->bgp; i < aep2->Ni;
					i++, i2 += 2, v.x += aep2->bgp ){
		ep2 = &aep2->ep[i][0];
		for( j = 0, j2 = LSIZE, v.y = aep->bgp ; j < aep2->Nj;
				ep2++, j++, j2 += 2, v.y += aep2->bgp ){


			ep2->ctr.x = v.x;
			ep2->ctr.y = v.y;
			ep2->b_no = 0;

			ep2->ep[0] = &aep->ep[i2 ][j2 ];

			if( j2 + 1 >= aep->Nj-LSIZE ){
				ep2->ep[1] = ep2->ep[3] = NULL;
				if( i2+1 < aep->Ni-LSIZE )
					ep2->ep[2] = &aep->ep[i2  + 1][j2 ];
				else	ep2->ep[2] = NULL;


				continue;
			}

			ep2->ep[1] = &aep->ep[i2 ][j2 + 1];

			if( i2 + 1 >= aep->Ni-LSIZE ){
				ep2->ep[2] = ep2->ep[3] = NULL;
				continue;
			}

			ep2->ep[2] = &aep->ep[i2  + 1][j2 ];
			ep2->ep[3] = &aep->ep[i2  + 1][j2 + 1];
		}
	}


	aep2->p = aep2->data;

	/* Reset number of bound2 field in bound structures */
	for( i = JSIZE ; i < aep->Ni - JSIZE ; i++ )
		for( j = JSIZE ; j < aep->Nj - JSIZE ; j++ ){
			ep = &aep->ep[i][j];

			for (k = 0; k < ep->b_no; k++)
				ep->bound[k]->b2_no = -1;
		}
}


static void
ep2_array_alloc( ep2_array *aep, int ni, int nj )
{
int	i;  
int	data_size;
 
	aep->NI = aep->Ni  = ni ;
	aep->NJ = aep->Nj  = nj ;
 
	aep->ep = (ep2_type **)MALLOC( aep->NI * sizeof( ep2_type * ) );
 

	aep->ep[0] = (ep2_type *)MALLOC( aep->NI * aep->NJ * sizeof(ep2_type) );
	for( i = 1 ; i < aep->NI ; i++ )
		aep->ep[i] = aep->ep[i-1] + aep->NJ;


	data_size = aep->NI*aep->NJ *( MAX_BOUND2*sizeof(bound2_type) );
	
#ifdef PALMOS
	if (data_size > 64000)
		data_size = 64000;
#endif
	aep->data = (char *)MALLOC( data_size );

	
	aep->p = aep->data;
}
 
 
 
static void
ep2_array_free( ep2_array *aep )
{
	if( aep->ep == NULL )	return;

	FREE( aep->ep[0] );
 
	if( aep->ep != NULL )
		FREE( aep->ep );
	aep->ep = NULL;

	aep->NI = aep->NJ = 0;

	if( aep->data != NULL )
	{
		FREE( aep->data );
	}
	 aep->data = NULL;
}

