/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/***********************
 ***	Ep2Value.c   ***
 ***********************/

#include	<stdio.h>

#include	"Uigp/igp.h"

#include	"Ep2/Ep2In.h"


static void	expW_ep_bound2( ep2Value_type *ep2Value, ep2_type * nbg,
				int nb, int sp, vec2d *v,
				float *val, float *unity, int type );

static void	expW_ep_bound2_all( ep2Value_type *ep2Value, ep2_type *nep, int nb, vec2d *v,
				float val[], float *unity, int type );


#define	WDT		0.01
#define	KF		0.5
#define	KT		2.0;


//static float	H,	Hinv;
//static float	OH,	OHinv;



void
ep2_value_init( ep2Value_type *ep2Value, int bgp )
{
	ep2Value->h = bgp + WDT;
	ep2Value->h_inv = 1.0 / ep2Value->h;

	ep2Value->oh = ep2Value->h / KT;
	ep2Value->oh_inv = KF *(1 / ep2Value->oh);
} 



#ifndef PLAYER
float
ep2_value( vec2d *v, ep2_array *aep2, int i0, int j0, int bno, int sp)
{
int	n,	nb;
float	unity,	val;
ep2_type	*ep,	*nep;
bound2_type	*b;
int	i,	j,	fcont;

	unity = val = 0;

	ep = &aep2->ep[i0][j0];
	b = ep->bound[bno];

	for( i = i0-1, n= 0 ; i <= i0+1 ; i++ ){

		if( i < 0 || i >= aep2->Ni ){
			n += 3;
			continue;
		}

		for( j = j0-1 ; j <= j0+1 ; j++, n++ ){
			if( j < 0 || j >= aep2->Nj )	continue;

			if( n == 4 )	continue;

		    	if( (fcont = b->Fcont[n]) == 0 )	continue;

			nep = &aep2->ep[i][j];

			for( nb = 0 ; fcont != 0 ; fcont >>= 1, nb++ ){
				if( !(fcont & 0x01) )   continue; 

				expW_ep_bound2( &aep2->ep2Value, nep, nb, sp, v,
					&val, &unity, nep->bound[nb]->type );
			}
		}
	}

	expW_ep_bound2( &aep2->ep2Value, ep, bno, sp, v, &val, &unity, (int)B2_CLOSED );

	if (unity != 0)
		return( val / unity );
	else
		return( b->val[sp] );
}



static void 
expW_ep_bound2( ep2Value_type *ep2Value, ep2_type *nep, int nb, int sp, vec2d *v,
	       float *val, float *unity, int type )
{
bound2_type *b;
float   weight;
float   x, y, t;


	b = nep->bound[nb];

	x = v->x - nep->ctr.x;
	y = v->y - nep->ctr.y;


	if( x < 0 )     x = -x;
	if( y < 0 )     y = -y;
	t = ( x > y )? x : y;


	if( type == B2_OPENED ){
		
		if( t >= ep2Value->oh )    return;
		weight = KF - t*ep2Value->oh_inv;
	}
	else	{
		if( t >= ep2Value->h )    return;
		weight = 1 - t*ep2Value->h_inv;
	}



	*unity += weight;
	*val += weight * b->val[sp];

}

#endif




void
ep2_value3( ep2_array *aep2, int i0, int j0, int bno, vec2d *v, float val[] )
{
int	n,	nb;
float	unity;
ep2_type	*ep,	*nep;
bound2_type	*b;
int	i,	j,	fcont;

	unity = val[SP_Y] = val[SP_Q] = val[SP_A] = 0;

	ep = &aep2->ep[i0][j0];
	b = ep->bound[bno];

	for( i = i0-1, n= 0 ; i <= i0+1 ; i++ ){

		if( i < 0 || i >= aep2->Ni ){
			n += 3;
			continue;
		}

		for( j = j0-1 ; j <= j0+1 ; j++, n++ ){
			if( j < 0 || j >= aep2->Nj )	continue;

			if( n == 4 )	continue;

			if( (fcont = b->Fcont[n]) == 0 )	continue;

			nep = &aep2->ep[i][j];


			for( nb = 0 ; fcont != 0 ; fcont >>= 1, nb++ ){
				if( !(fcont & 0x01) )   continue; 

				expW_ep_bound2_all( &aep2->ep2Value, nep, nb, v,
					val, &unity, nep->bound[nb]->type );
			}
		}
	}

	expW_ep_bound2_all( &aep2->ep2Value, ep, bno, v, val, &unity, (int)B2_CLOSED );

	if (unity != 0 ){
		val[SP_Y] /= unity;
		val[SP_Q] /= unity;
		val[SP_A] /= unity;
		return;
	}
}



static void 
expW_ep_bound2_all( ep2Value_type *ep2Value, ep2_type *nep, int nb, vec2d *v,
				float val[], float *unity, int type )
{
bound2_type	*b;
float	weight;
float	x,	y,	t;


	b = nep->bound[nb];

	x = v->x - nep->ctr.x;
	y = v->y - nep->ctr.y;


	if( x < 0 )     x = -x;
	if( y < 0 )     y = -y;
	t = ( x > y )? x : y;


	if( type == B2_OPENED ){
		
		if( t >= ep2Value->oh )    return;
		weight = KF - t*ep2Value->oh_inv;
	}
	else	{
		if( t >= ep2Value->h )    return;
		weight = 1 - t*ep2Value->h_inv;
	}



	*unity += weight;
	val[SP_Y] += weight * b->val[SP_Y];
	val[SP_Q] += weight * b->val[SP_Q];
	val[SP_A] += weight * b->val[SP_A];
}
