/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
#include	<stdio.h>
#include	<stdlib.h>

#include	"Uigp/igp.h"

#include	"Ubst/BstType.h"
#include	"Ucst/CstType.h"

#include	"Bits.h"

#define	ByteLength	8

static int get_uncoded_char(unsigned char cod, unsigned char *repro, unsigned char bmask, unsigned char flag);
static void bit_decoding( FILE *output, unsigned char mask, bst_type *bst);
#ifndef PDA
void 
bitstream_decoding( FILE *cfd, int byte_no, FILE *output )
{
int	i,	n;
bst_type	*bst;
u_char mask;

	mask = getc( cfd );

	if (mask == 0) {
		n = gp_fread_num( cfd, 4 );

		for (i = 0; i < n; i++)
			putc( (char) mask, output);

		return;
	}


	bst = bst_read( cfd, byte_no-1 );


	bit_decoding( output, mask, bst);

	bst_free( bst );
}
#endif


cst_type *
bitstream_decoding_cst( bst_type *bst_file, int byte_no )
{
int	i,	n,	msg_no;
u_char mask,	flag,	repro,	cod;
bst_type	*bst;
cst_type	*cst;

	mask = bst_get_int(bst_file,8);

	if (mask == 0) {
		n = bst_get_align_num(bst_file,4);

		cst = cst_alloc( n );
		for (i = 0; i < n; i++)
			CST_PUT_CHAR( cst, 0 );

		return( cst );
	}


//	bst = bst_alloc( byte_no-1 );
//	bst_copy( bst_file, bst, byte_no-1 );

	bst = bst_get_align_bst( bst_file, byte_no-1 );



	for (i = 0, n = 0; i < ByteLength - 1; i++)
		if ((mask & (1L << i)) != 0)	n++;
	flag =  mask & (1L << (ByteLength - 1));
	if( flag != 0 )	n++;


	msg_no = bst->byte_no*8 / n;
	cst = cst_alloc( msg_no );

	for (i = 0; i < msg_no; i++) {
		cod = (u_char)bst_get_int( bst, n );

		get_uncoded_char( cod, &repro, mask, flag );

		CST_PUT_CHAR( cst, repro );
	}

	bst_free( bst );

	return( cst );
}


static int 
get_uncoded_char(unsigned char cod, unsigned char *repro,
		 unsigned char bmask,
		 unsigned char flag)
{
	int     i, j = 0, n = 0, temp;

	if (bmask == 255) {
		*repro = cod;
		return (int) cod;
	}
	*repro = 0;
	for (i = 0; i < ByteLength - 1; i++)
		if ((bmask & (1L << i)) != 0)
			n++;
	for (i = 0; i < n; i++) {
		while (((bmask & (1L << j)) == 0) && (j < ByteLength))
			j++;
		if ((cod & (1L << i)) != 0)
			*repro |= (1L << j);
		j++;
	}
	if (flag != 0) {
		if ((cod & (1L << n)) != 0)
			temp = -(int) (*repro);
		else
			temp = (int) (*repro);
		*repro |= (1L << (ByteLength - 1));
		*repro = (unsigned char) temp;
	}
	return (int) (*repro);
}


static void 
bit_decoding( FILE *output, unsigned char mask, bst_type *bst)
{
u_char flag,	repro,	cod;
int     i,	n = 0,	msg_num;

	flag = (mask & (1L << (ByteLength - 1)));

	for (i = 0; i < ByteLength - 1; i++)
		if ((mask & (1L << i)) != 0)
			n++;

	if (flag != 0)	n++;

	msg_num = bst->byte_no*8 / n;

	for (i = 0; i < msg_num; i++) {
		cod = (u_char)bst_get_int( bst, n );

		get_uncoded_char( cod, &repro, mask, flag);

		putc( repro, output );
	}
}


