/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
#include "HuffmanLocal.h"

void huff_expand_data_extended( bst_type *bst, FILE *output, NODE *nodes, int root_node );
void huff_expand_data_extended_cst( bst_type *bst, cst_type *cst,
			NODE *nodes, int root_node );


void huffman_decoding_extended( bst_type *ibst, FILE *output )
{
NODE	*nodes;
int	root_node;

	nodes = (NODE *) calloc( 514, sizeof( NODE ) );

	huff_input_counts( ibst, nodes );
	root_node = huff_build_tree( nodes );

	huff_expand_data_extended( ibst, output, nodes, root_node );

	free( (char *) nodes );
}

cst_type *
huffman_decoding_extended_cst( bst_type *bst_file, int byte_no )
{
NODE	*nodes;
int	root_node;
cst_type	*cst;


	nodes = (NODE *) calloc( 514, sizeof( NODE ) );

	huff_input_counts( bst_file, nodes );

	root_node = huff_build_tree( nodes );

	cst = cst_alloc( byte_no*8 );
	huff_expand_data_extended_cst( bst_file, cst, nodes, root_node );

	free( (char *) nodes );
/*	bst_get_align(bst_file);*/
	cst_rewind(cst);

	return( cst );
}


void huff_expand_data_extended( bst_type *bst, FILE *output, NODE *nodes, int root_node )
{
int node;
char mask;
char code;

	for ( ; ; ) {
		node = root_node;

		do {
			if( bst_get( bst ) )
				node = nodes[ node ].child_1;
			else
				node = nodes[ node ].child_0;

		} while ( node > END_OF_STREAM );


		if ( node == END_OF_STREAM )
			break;

		if (node == 0) {
			code = 0;
		}
		else {
			mask = 1L << (node - 1) ;
			code = (char) bst_get_int(bst, node);
			/*now calculate the data*/
			if ( (code & mask) && (code != -128) ) code = -code;
			else code |= mask;
		}
		putc( code, output );
	}
}

void
huff_expand_data_extended_cst( bst_type *bst, cst_type *cst,
			NODE *nodes, int root_node )
{
int node;
char mask, code;

	for ( ; ; ) {

		node = root_node;
		do {
			if( bst_get( bst ) )
				node = nodes[ node ].child_1;
			else
				node = nodes[ node ].child_0;
		} while ( node > END_OF_STREAM );

		if ( node == END_OF_STREAM )
			break;

		if (node == 0) {
			code = 0;
		}
		else {
			mask = 1L << (node - 1) ;
			code = (char) bst_get_int(bst, node);
			/*now calculate the data*/
			if ( (code & mask) && (code != -128) ) code = -code;
			else code |= mask;
		}
		CST_PUT_CHAR( cst, code );
	}
}



