/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/

/**************************
 ***   Bound.c  	***
 **************************/

#include <stdio.h>
#include <math.h>

#include	"Uigp/igp.h"
#include        "Uprm/prm.h"


#include        "GpType/GpType.h"
#include        "EpType/EpType.h"

#include        "Ep.h"
#include        "EpIn.h"



static int	ep_range_al( ep_type *ep, range_type *r, int rno );

static void	ep_bound_al_range( ep_type *ep, range_type *r, int bno );
	

void
ep_bound( ep_type *ep, ep_array *aep )
{
int	Frange[ MAX_RANGE],	Fal[MAX_RANGE];
int	Or_bound;
bound_type	*b;
range_type	*r;
int	i,	j,	mi,	bno;

	ep->b_no = 0;


	if( ep->r_no == 0 ){
		if( ep->b_no > 0 )	return;

		ep->b_no = 1;
/*
		b = ep->bound[0] = BOUND_ALLOC();
*/
		ALLOC_AEP_DATA( ep->bound[0], bound_type, aep )
		b = ep->bound[0];
		b->bgp = NULL;
		b->Mbgp = NULL;
		b->Frange = 0;
		b->range = NULL;
		b->type = BOUND_ALL;


		return;
	}



        for( r = ep->range, i = 0 ; r != NULL ; r = r->nr, i++ ){
		Frange[i] = (1L<<i);
		Fal[i] = ep_range_al( ep, r, i );
	}



        for( r = ep->range, i = 0 ; r != NULL ; r = r->nr, i++ )
		ep_range_Frange( ep, r, i, Frange, Fal, aep->hbgp );




	Or_bound = 0;
        for( i = 0 ; i < ep->r_no ; i++ ){

		mi = 1L<< i;
		if( Or_bound & mi )	continue;

/*
		b = ep->bound[ep->b_no++] = BOUND_ALLOC();
*/

		ALLOC_AEP_DATA( ep->bound[ep->b_no], bound_type, aep )
		b = ep->bound[ep->b_no++];
		b->bgp = NULL;
		b->Mbgp = NULL;

		b->Frange = Frange[i];

        	for( j = i+1 ; j < ep->r_no ; j++ )
			if( Frange[j] & mi )
				b->Frange &= Frange[j];

		b->Frange &= (~Or_bound);
		Or_bound |= b->Frange;

		b->type = BOUND_CROSS;
	}




	for( bno = 0 ; bno < ep->b_no ; bno++ ){
		b = ep->bound[bno];
		b->range = NULL;


        	for( r = ep->range, i = 0 ; r != NULL ; r = r->nr, i++ )
			if( b->Frange &(1L<<i) ){
				r->bno = bno;

				r->p = b->range;
				b->range = r;

				ep_bound_al_range( ep, r, bno );
			}
	}
}



static int
ep_range_al( ep_type *ep, range_type *r, int rno )
{
rlink_type      *rl;
al_type *al;    
int     i,	m,	Fal; 

        Fal = 0;
        for( i = 0 ; i < r->rl_no ; i++ ){
                rl = &r->rl[i];
 
                for( al = ep->al, m = 1 ; al != NULL ; al = al->nal, m <<= 1 ){
                        if( al != rl->al )      continue;
 
                        al->rno[rl->side] = rno;
                        Fal |= m;
 
                        break;
                }
        }

	return( Fal );
}



static void
ep_bound_al_range( ep_type *ep, range_type *r, int bno )
{
rlink_type	*rl;
al_type	*al;
int	i;

	for( i = 0 ; i < r->rl_no ; i++ ){
		rl = &r->rl[i];
		al = rl->al;

		al->bno[rl->side] = bno;
	}
}



void
ep_bound_set_fcorner( ep_type *ep, float hbgp )
{
int	bno;
vec2d	v;
float	dis;

	if( ep->b_no == 1 ){
		ep->corner[0] = ep->corner[1] =
				ep->corner[2] = ep->corner[3] = 0;
		
		return;
	}


	for( bno = 0 ; bno < ep->b_no ; bno++ )
		ep->bound[bno]->Fcorner = 0;


	v.x =  - hbgp;
	v.y =  - hbgp;
	ep->corner[0] = ep_bound_point( ep, &v, &dis );
	if( dis <= 0.0078125 )	ep->corner[0] = -1;
	else	if( dis > 64 ){
			v.x *= 0.5;
			v.y *= 0.5;
			ep->corner[0] = ep_bound_point( ep, &v, &dis );
		}

	v.x =  - hbgp;
	v.y =  + hbgp;
	ep->corner[1] = ep_bound_point( ep, &v, &dis );
	if( dis <= 0.0078125 )	ep->corner[1] = -1;
	else	if( dis > 64 ){
			v.x *= 0.5;
			v.y *= 0.5;
			ep->corner[1] = ep_bound_point( ep, &v, &dis );
		}

	v.x =  + hbgp;
	v.y =  - hbgp;
	ep->corner[2] = ep_bound_point( ep, &v, &dis );
	if( dis <= 0.0078125 )	ep->corner[2] = -1;
	else	if( dis > 64 ){
			v.x *= 0.5;
			v.y *= 0.5;
			ep->corner[2] = ep_bound_point( ep, &v, &dis );
		}

	v.x =  + hbgp;
	v.y =  + hbgp;
	ep->corner[3] = ep_bound_point( ep, &v, &dis );
	if( dis <= 0.0078125 )	ep->corner[3] = -1;
	else	if( dis > 64 ){
			v.x *= 0.5;
			v.y *= 0.5;
			ep->corner[3] = ep_bound_point( ep, &v, &dis );
		}
}
