/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/

/**************************
 ***   BoundCont.c  	***
 **************************/

#include <stdio.h>
#include <math.h>

#include	"Uigp/igp.h"
#include        "Uprm/prm.h"


#include        "EpType/EpType.h"

#include        "EpIn.h"



static void	ep_cont_bound_corenr( ep_type *ep );


static void	ep_count_al( ep_type *ep, int n );



void
ep_cont_init( ep_type *ep )
{
bound_type	*b;
int	i,	n;

	for( i = 0 ; i < ep->b_no ; i++ ){

		b = ep->bound[i]; 

		for( n = 0 ; n < 9 ; n++ )
			b->Fcont[n] = 0;

		b->Fcont[4] |= ( 1<<i);
	}
}



void
ep_cont( ep_type *ep )
{
	ep_cont_bound_corenr( ep );


	ep_count_al( ep, 5 );
	ep_count_al( ep, 6 );
	ep_count_al( ep, 7 );
	ep_count_al( ep, 8 );
}



static void
ep_cont_bound_corenr( ep_type *ep )
{
ep_type *nep;
int	bno,	nbno;

	nep = ep->ng[5];
	if( (bno = ep->corner[1]) >= 0 )
		if( (nbno = nep->corner[0]) >= 0 && nep->b_no > 0 ){
			ep->bound[bno]->Fcont[5] |= (1<<nbno);
			nep->bound[nbno]->Fcont[3] |= (1<<bno);
		}

	if( (bno = ep->corner[3]) >= 0 )
		if( (nbno = nep->corner[2]) >= 0 && nep->b_no > 0 ){
			ep->bound[bno]->Fcont[5] |= (1<<nbno);
			nep->bound[nbno]->Fcont[3] |= (1<<bno);
		}



	nep = ep->ng[6];
	if( (bno = ep->corner[2]) >= 0 )
		if( (nbno = nep->corner[1]) >= 0 && nep->b_no > 0 ){
			ep->bound[bno]->Fcont[6] |= (1<<nbno);
			nep->bound[nbno]->Fcont[2] |= (1<<bno);
		}



	nep = ep->ng[7];
	if( (bno = ep->corner[2]) >= 0 )
		if( (nbno = nep->corner[0]) >= 0 && nep->b_no > 0 ){
			ep->bound[bno]->Fcont[7] |= (1<<nbno);
			nep->bound[nbno]->Fcont[1] |= (1<<bno);
		}

	if( (bno = ep->corner[3]) >= 0 )
		if( (nbno = nep->corner[1]) >= 0 && nep->b_no > 0 ){
			ep->bound[bno]->Fcont[7] |= (1<<nbno);
			nep->bound[nbno]->Fcont[1] |= (1<<bno);
		}



	nep = ep->ng[8];
	if( (bno = ep->corner[3]) >= 0 )
		if( (nbno = nep->corner[0]) >= 0 && nep->b_no > 0 ){
			ep->bound[bno]->Fcont[8] |= (1<<nbno);
			nep->bound[nbno]->Fcont[0] |= (1<<bno);
		}
}





static void
ep_count_al( ep_type *ep, int n )
{
ep_type *nep;
al_type	*al,	*nal;
int	bno,	nbno;

	if( (nep = ep->ng[n]) == NULL )	return;
	if( nep->b_no == 0 )	return;

	for( al = ep->al ; al != NULL ; al = al->nal ){

		for( nal = nep->al ; nal != NULL ; nal = nal->nal ){
			if( al->link != nal->link )	continue;

			bno = al->bno[F_RIGHT];
			nbno = nal->bno[F_RIGHT];
			ep->bound[bno]->Fcont[n] |= (1<<nbno);
			nep->bound[nbno]->Fcont[8-n] |= (1<<bno);

			bno = al->bno[F_LEFT];
			nbno = nal->bno[F_LEFT];
			ep->bound[bno]->Fcont[n] |= (1<<nbno);
			nep->bound[nbno]->Fcont[8-n] |= (1<<bno);

			break;
		}
	}
}
