/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/*************************
 ***	BoundPoint.c   ***
 *************************/

#include	<stdio.h>
#include	<math.h>

#include	"Uigp/igp.h"
#include	"EpType/EpType.h"


#include	"EpIn.h"


static void	ep_al_point( al_type *al, vec2d *v, int *bno, float *dis );



int
ep_bound_point( ep_type *ep, vec2d *v, float *dis )
{
al_type	*al;
int	bno;


	*dis = 1000;

	if( ep->b_no == 1 )	return( 0 );

	bno = -1;


	for( al = ep->al ; al != NULL ; al = al->nal ){

		ep_al_point( al, v, &bno, dis );
	}


	return( bno );
}



int
ep_bound_point_br( ep_type *ep, int br, vec2d *v, float *dis )
{
al_type	*al;
int	bno,	i;


	*dis = 1000;

	if( ep->b_no == 1 )	return( 0 );

	bno = -1;


	for( al = ep->al ; al != NULL ; al = al->nal ){

		if( al->link->br != br )	continue;

		ep_al_point( al, v, &bno, dis );
	}



	if( bno != -1)	return( bno );



	for( i = 0 ; i < ep->b_no ; i++ )
		if( ep->bound[i]->br.no == br )	return( i );


	return( -1 );
}



static void 
ep_al_point( al_type *al, vec2d *v, int *bno, float *dis )
{
vec2d	p;
el_type	*l;
float	u,	t,	dt,	tu;


	l = al->link;
	p.x = v->x - al->ctr.x;
	p.y = v->y - al->ctr.y;


	u = l->u.x * p.x + l->u.y * p.y;
	t = -l->u.y * p.x + l->u.x * p.y;


	if( t < 0 ){
		tu = u*u + 1.5*t*t;

		if( !(al->flag & 0x10) ){
			if( ABS(l->v.y) > ABS(l->v.x) )
				u  = ( l->v.y < 0 )? -p.x : p.x;
			else	u  = ( l->v.x < 0 )? p.y : -p.y;
		}
	}
	else	if( (dt = t - l->len) > 0  ){

			tu = u*u + 1.5*dt*dt;

			if( !(al->flag & 0x20) ){
				if( ABS(l->v.y) > ABS(l->v.x) )
					u  = ( l->v.y < 0 )? -(p.x-l->v.x) :
							(p.x - l->v.x);
				else	u  = ( l->v.x < 0 )? (p.y - l->v.y):
							-(p.y - l->v.y);
			}
		}
		else	{
			if( (u < 0) != (l->c_prb < 0) )
				u -= l->c_prb*t *(t - l->len );

//			u -= l->c_prb*t *(t - l->len );

			tu = u*u;
		}



	if( tu > *dis )	return;

	*dis = tu;
	if( u >= 0 )
		*bno = al->bno[F_RIGHT];
	else	*bno = al->bno[F_LEFT];
}
