/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/*********************
 ***	ElTool.c   ***
 *********************/

#include	<math.h>
#include	"Uigp/igp.h"
#include	"Ubox/Box2d.h"
#include	"EpType/ElType.h"



void
el_t2xy( vec2d *vm, el_type *link, float t, vec2d *p )
{
float	h;

	h = link->c_prb * t * (t - link->len);

	p->x = vm->x  +  t * -link->u.y  +  h * link->u.x;
	p->y = vm->y  +  t *  link->u.x  +  h * link->u.y;
}

                 


void
el_Ntanget_beg( el_type *l, vec2d *t )
{
float	tmp = l->c_prb* l->len;
vec2d	v;

	v.x = -l->u.y - l->u.x* tmp;
	v.y =  l->u.x - l->u.y* tmp;

	tmp = 1.0/hypot( v.x, v.y );

	t->x = v.y*tmp;
	t->y = -v.x*tmp;
}

void
el_Ntanget_end( el_type *l, vec2d *t )
{
float	tmp = l->c_prb* l->len;
vec2d	v;

	v.x = -l->u.y + l->u.x* tmp;
	v.y =  l->u.x + l->u.y* tmp;

	tmp = 1.0/hypot( v.x, v.y );

	t->x = v.y*tmp;
	t->y = -v.x*tmp;
}



	/* point with tanget vector ( 1, 0 )	*/
int
el_tanget_point_10( el_type *l, vec2d *vm, float *t, vec2d *p )
{
float	tmp;

	if( l->c_prb == 0.0 )	return(0);

	tmp = l->c_prb * l->u.y;

	if( ABS( l->u.x ) > ABS( tmp )*l->len )	return( 0 );

	*t = 0.5*( -l->u.x / tmp + l->len);
	el_t2xy( vm, l, *t, p );
	return( 1 );
}


int
el_tanget_point_10_dp( el_type *l, vec2d *p )
{
float	t,	h;
float	tmp;

	if( l->c_prb == 0.0 )	return(0);

	tmp = l->c_prb * l->u.y;

	if( ABS( l->u.x ) > ABS( tmp )*l->len )	return( 0 );

	t = 0.5*( -l->u.x / tmp + l->len);
	h = l->c_prb * t * (t - l->len);

	p->x = t * -l->u.y  +  h * l->u.x;
	p->y = t *  l->u.x  +  h * l->u.y;
	return( 1 );
}



	/* point with tanget vector ( 0, 1 )	*/
int
el_tanget_point_01( el_type *l, vec2d *vm, float *t, vec2d *p )
{
float	tmp;

	if( l->c_prb == 0.0 )	return(0);

	tmp = l->c_prb * l->u.x;

	if( ABS( l->u.y ) > ABS( tmp )*l->len )	return( 0 );

	*t = 0.5*( +l->u.y / tmp + l->len);
	el_t2xy( vm, l, *t, p );
	return( 1 );
}


int
el_tanget_point_01_dp( el_type *l, vec2d *p )
{
float	tmp;
float	t,	h;

	if( l->c_prb == 0.0 )	return(0);

	tmp = l->c_prb * l->u.x;

	if( ABS( l->u.y ) > ABS( tmp )*l->len )	return( 0 );

	t = 0.5*( +l->u.y / tmp + l->len);

	h = l->c_prb * t * (t - l->len);

	p->x = t * -l->u.y  +  h * l->u.x;
	p->y = t *  l->u.x  +  h * l->u.y;

	return( 1 );
}


void
el_box2d( el_type *l, box2d *b )
{
float	tmp,	t,	h;
vec2d	p;

	if( l->v.x < 0 ){
		b->x0 = l->v.x;
		b->x1 = 0;
	}
	else	{
		b->x0 = 0;
		b->x1 = l->v.x;
	}

	if( l->v.y < 0 ){
		b->y0 = l->v.y;
		b->y1 = 0;
	}
	else	{
		b->y0 = 0;
		b->y1 = l->v.y;
	}


	if( l->c_prb == 0.0 )   return;
 

	tmp = l->c_prb * l->u.y;
	if( ABS( l->u.x ) < ABS( tmp )*l->len ){

		t = 0.5*( -l->u.x / tmp + l->len);
		h = l->c_prb * t * (t - l->len);

		p.y = t *  l->u.x  +  h * l->u.y;

		if( p.y < b->y0 )	b->y0 = p.y;
		else    b->y1 = p.y;
	}


	tmp = l->c_prb * l->u.x;
	if( ABS( l->u.y ) < ABS( tmp )*l->len ){

		t = 0.5*( +l->u.y / tmp + l->len);

		h = l->c_prb * t * (t - l->len);

		p.x = t * -l->u.y  +  h * l->u.x;

		if( p.x < b->x0 )	b->x0 = p.x;
		else    b->x1 = p.x;
        }
}
