/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**************************
 ***   EpAgpAl.c      	***
 **************************/

#include <stdio.h>
#include <math.h>

#include	"Uigp/igp.h"
#include        "Uprm/prm.h"

#include        "GpType/GpType.h"

#include        "Ep/Ep.h"

#include        "EpIn.h"


static void	ep_agp_al_link( gp_array *agp, int mbr, ep_array *aep );





void
ep_agp_al( gp_array *agp, int I0, int J0, int Ni, int Nj,
					int mbr, ep_array *aep )
{
	aep->Ni0 = I0;
	if( I0 + Ni >= agp->Ni-1 )	Ni = agp->Ni-1-I0;

	aep->Nj0 = J0;
	if( J0 + Nj >= agp->Nj-1 )	Nj = agp->Nj-1-J0;
	ep_array_clear( aep, agp->bgp, Ni+2*LSIZE, Nj+2*LSIZE );


	aep->BNi = Ni;
	aep->BNj = Nj;
	aep->V0.x = (I0-1) * aep->bgp;

	aep->V0.y = (J0-1) * aep->bgp;



	ep_agp_al_link( agp, mbr, aep );
}






static void
ep_agp_al_link( gp_array *agp, int mbr, ep_array *aep )
{
gp_type		*gp;
link_type	*link,	*l;


int	i,	j;
int	I0,	I1,	J0,	J1;
int	ni,	nj,	d;
box2d	box;
vec2d	end,	ctr;


	ni = aep->Ni -2*JSIZE;
	nj = aep->Nj -2*JSIZE;


	box.x0 = aep->V0.x - aep->bgp;
	box.y0 = aep->V0.y - aep->bgp;
	box.x1 = aep->V0.x + aep->bgp*ni;
	box.y1 = aep->V0.y + aep->bgp*nj;

	I0 = aep->Ni0-1;
	I1 = aep->Ni0 + ni;

	J0 = aep->Nj0-1;
	J1 = aep->Nj0 + nj;

	d = 24 *agp->inv_bgp +0.5;


	for( i = I0-d ; i <= I1+d ; i++ ){
	    if( i < 0 || i >= agp->Ni )	continue;

		gp = &agp->gp[i][J0-d];

	    for( j = J0-d ; j <= J1+d ; j++, gp++ ){
			if( j < 0 || j >= agp->Nj )	continue;

			l = NULL;
			for( link = gp->link ; link != NULL ; link = link->nl ){
				link->cln = l;
				l = link;
			}
			for( link = l ; link != NULL ; link = link->cln ){
/*
			for( link = gp->link ; link != NULL ; link = link->nl ){
*/

				if( mbr != 0 && !(mbr &(1<<link->br)) )	continue;

				if( link->type != NF_EDGE &&
							link->type != NF_N3_P )
					continue;

				if( j < J0 ){
					end.y = gp->ctr.y+link->ctr.y+link->v.y;
					if( end.y < box.y0 )	continue;
				}
				else if( j > J1 ){
					end.y = gp->ctr.y+link->ctr.y+link->v.y;
					if( end.y > box.y1 )	continue;
				}
/*
				else if( i < I0 ){
*/
				if( i < I0 ){
					end.x = gp->ctr.x+link->ctr.x+link->v.x;
					if( end.x < box.x0 )	continue;
				}
				else if( i > I1 ){
					end.x = gp->ctr.x+link->ctr.x+link->v.x;
					if( end.x > box.x1 )	continue;
				}




				ctr.x = gp->ctr.x + link->ctr.x - aep->V0.x;
				ctr.y = gp->ctr.y + link->ctr.y - aep->V0.y;

				if( 4*ABS(link->a) < link->len )
					ep_al_1( &ctr, link, aep );
				else	ep_al( &ctr, link, aep );

			}
		}
	}
}
