/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/

/******************
 ***   EpAl.c  	***
 ******************/

#include <stdio.h>
#include <math.h>

#include	"Uigp/igp.h"
#include        "Uprm/prm.h"


#include        "GpType/GpType.h"

#include        "EpType/EpType.h"
#include        "EpIn.h"




#define		IS_IN_BGP( p, hbgp )	( (p).x > -hbgp && (p).x < hbgp && \
					(p).y > -hbgp && (p).y < hbgp )

#define         IS_IN_BGP_ENV( p, v, hbgp ) \
			( ((p).x -(v).x) > -hbgp && ((p).x -(v).x) < hbgp && \
			  ((p).y -(v).y) > -hbgp && ((p).y -(v).y) < hbgp )




/*
static void	ep_al( vec2d *ctr, el_type *l, ep_array *aep );
*/


static void	ep_el_bes( vec2d *ctr, el_type *l, vec2d bu[],
					int *Flag10, vec2d *v10,
					int *Flag01, vec2d *v01 );



static void	ep_al_one( vec2d *vm, el_type *l,
				vec2d *End, int Flag10, vec2d *V10, int Flag01, vec2d *V01,
				vec2d bu[], ep_type *ep, ep_array *aep );

static void	ep_al_point_flag( vec2d *ctr, el_type *l, vec2d bu[],
                                        float x, float y, int *flag );


al_type *	ep_al_add( vec2d *ctr, el_type *l, vec2d bu[],
					ep_type *ep, ep_array *aep );






/*
void
aep_al( ep_array *aep )
{
ep_type         *ep;
el_type *el;
vec2d   ctr;

int     i,      j;


        for( i = 0 ; i < aep->Ni ; i++ )
		for( j = 0 ; j < aep->Nj ; j++ ){
			ep = &aep->ep[i][j];


			for( el = ep->el ; el != NULL ; el = el->cln ){

				ctr.x = ep->ctr.x + el->ctr.x;
				ctr.y = ep->ctr.y + el->ctr.y;

				if( 4*ABS(el->a) < el->len )
					ep_al_1( &ctr, el, aep );
				else	ep_al( &ctr, el, aep );
			}
		}    
}
*/



void
ep_al( vec2d *ctr, el_type *l, ep_array *aep )
{
vec2d	v0,	v1;
int	i0,	j0,	i1,	j1,	i,	j;
vec2d	bu[2];

vec2d	End,	V01,	V10;
int	Flag10,	Flag01;

	v0 = *ctr;
	v1 = *ctr;

	End.x = ctr->x + l->v.x;
	End.y = ctr->y + l->v.y;
	VEC2D_UPDATE_BOUND( End, v0, v1 );



	ep_el_bes( ctr, l, bu, &Flag10, &V10, &Flag01, &V01 );


	if( Flag01 == 1 )	VEC2D_UPDATE_BOUND( V01, v0, v1 );

	if( Flag10 == 1 )	VEC2D_UPDATE_BOUND( V10, v0, v1 );


	i0 = (v0.x + LSIZE*aep->bgp)* aep->bgp_inv;
	i1 = (v1.x + LSIZE*aep->bgp)* aep->bgp_inv;

	j0 = (v0.y + LSIZE*aep->bgp)* aep->bgp_inv;
	j1 = (v1.y + LSIZE*aep->bgp)* aep->bgp_inv;


	if( i1 < JSIZE || i0 > aep->Ni-JSIZE )	return;
	if( j1 < JSIZE || j0 > aep->Nj-JSIZE )	return;

	i0 = PUSH_TO_RANGE( i0 , JSIZE, aep->Ni-JSIZE );
	i1 = PUSH_TO_RANGE( i1 , JSIZE, aep->Ni-JSIZE-1 );

	j0 = PUSH_TO_RANGE( j0 , JSIZE, aep->Nj-JSIZE );
	j1 = PUSH_TO_RANGE( j1 , JSIZE, aep->Nj-JSIZE-1 );


	for( i = i0 ; i <= i1 ; i++ )
		for( j = j0 ; j <= j1 ; j++ )
			ep_al_one( ctr, l, &End, Flag10, &V10, Flag01, &V01, bu, &aep->ep[i][j], aep );
	
}


static void
ep_al_one( vec2d *vm, el_type *l,
			vec2d *End, int Flag10, vec2d *V10, int Flag01, vec2d *V01,
			 vec2d bu[], ep_type *ep, ep_array *aep )
{
vec2d	ctr;
int	flag;
al_type	*al;

	ctr.x = vm->x - ep->ctr.x;
	ctr.y = vm->y - ep->ctr.y;

	if( IS_IN_BGP(ctr, aep->hbgp) ){
		ep_al_add( &ctr, l, bu, ep, aep );
		return;
	}



	if( IS_IN_BGP_ENV( *End, ep->ctr, aep->hbgp ) ){
		ep_al_add( &ctr, l, bu, ep, aep );
		return;
	}


	if( Flag01 )
		if( IS_IN_BGP_ENV( *V01, ep->ctr, aep->hbgp ) ){
			al = ep_al_add( &ctr, l, bu, ep, aep );
			ep_range_cross_link( al, F_RIGHT, ep, aep);
			ep_range_cross_link( al, F_LEFT, ep, aep );
			return;
		}



	if( Flag10 )
		if( IS_IN_BGP_ENV( *V10, ep->ctr, aep->hbgp ) ){
			al = ep_al_add( &ctr, l, bu, ep, aep );
			ep_range_cross_link( al, F_RIGHT, ep, aep);
			ep_range_cross_link( al, F_LEFT, ep, aep );
			return;
		}



	flag = 0;

	ep_al_point_flag( &ctr, l, bu, -aep->hbgp, -aep->hbgp, &flag );
	ep_al_point_flag( &ctr, l, bu, -aep->hbgp, +aep->hbgp, &flag );
	ep_al_point_flag( &ctr, l, bu, +aep->hbgp, -aep->hbgp, &flag );
	ep_al_point_flag( &ctr, l, bu, +aep->hbgp, +aep->hbgp, &flag );


	if( flag != 0x03 )	return;

	al = ep_al_add( &ctr, l, bu, ep, aep );
	ep_range_cross_link( al, F_RIGHT, ep, aep);
	ep_range_cross_link( al, F_LEFT, ep, aep );
}




static void
ep_el_bes( vec2d *ctr, el_type *l, vec2d bu[],
		int *Flag10, vec2d *v10,
		int *Flag01, vec2d *v01 )
{
float	t;

	if( ABS(l->v.y) > ABS(l->v.x) ){
		*Flag01 = el_tanget_point_01( l, ctr, &t, v01 );
		*Flag10 = 0;

		bu[0].x = bu[1].x = 0;
		bu[0].y = bu[1].y = ( l->v.y < 0 )? -1 : 1;

		if( 4*ABS(l->a) < l->len )	return;
		*Flag10 = el_tanget_point_10( l, ctr, &t, v10 );
		if( *Flag10 == 0 )	return;

		if( l->v.y > 0 ){
			if( v10->y < ctr->y ){
				bu[0].x = ( l->v.x < 0 )? -1 : 1;
				bu[0].y = 0;
				return;
			}

			if( v10->y > ctr->y+l->v.y ){
				bu[1].x = ( l->v.x < 0 )? -1 : 1;
				bu[1].y = 0;
				return;
			}

			return;
		}

		if( v10->y > ctr->y ){
			bu[0].x = ( l->v.x < 0 )? -1 : 1;
			bu[0].y = 0;
			return;
		}

		if( v10->y < ctr->y+l->v.y ){
			bu[1].x = ( l->v.x < 0 )? -1 : 1;
			bu[1].y = 0;
			return;
		}

		return;
	}


	*Flag10 = el_tanget_point_10( l, ctr, &t, v10 );
	*Flag01 = 0;

	bu[0].y = bu[1].y = 0;
	bu[0].x = bu[1].x = ( l->v.x < 0 )? -1 : 1;

	if( 4*ABS(l->a) < l->len )	return;
	*Flag01 = el_tanget_point_01( l, ctr, &t, v01 );
	if( *Flag01 == 0 )	return;

	if( l->v.x > 0 ){
		if( v10->x < ctr->x ){
			bu[0].x = 0;
			bu[0].y = ( l->v.y < 0 )? -1 : 1;
			return;
		}

		if( v10->x > ctr->x+l->v.x ){
			bu[1].x = 0;
			bu[1].y = ( l->v.y < 0 )? -1 : 1;
			return;
		}

		return;
	}

	if( v10->x > ctr->x ){
		bu[0].x = 0;
		bu[0].y = ( l->v.y < 0 )? -1 : 1;
		return;
	}

	if( v10->x < ctr->x+l->v.x ){
		bu[1].x = 0;
		bu[1].y = ( l->v.y < 0 )? -1 : 1;
		return;
	}
}
			
			
		




static void
ep_al_point_flag( vec2d *ctr, el_type *l, vec2d bu[],
					float x, float y, int *flag )
{
vec2d   v,	v1;
float   t,      h,      dp;

        v.x = x - ctr->x;
        v.y = y - ctr->y;

	if( v.x * bu[0].x + v.y* bu[0].y  <= 0 ){
		dp = v.x * bu[0].y - v.y * bu[0].x;
		*flag |= ( 0 < dp )? 0x1 : 0x2;   
		return;
	}

	v1.x = v.x - l->v.x;
	v1.y = v.y - l->v.y;

	if( v1.x * bu[1].x + v1.y* bu[1].y  >= 0 ){
		dp = v1.x * bu[1].y - v1.y * bu[1].x;
		*flag |= ( 0 < dp )? 0x1 : 0x2;   
                return;
        }




        t = -l->u.y * v.x + l->u.x * v.y;

	if( t <= 0 || t >= l->len ){
        	dp = l->u.x * v.x + l->u.y * v.y;
		*flag |= ( 0 < dp )? 0x1 : 0x2;   
                return;
        }

        dp = l->u.x * v.x + l->u.y * v.y;
        h = l->c_prb * t * (t- l->len );

        *flag |= ( h < dp )? 0x1 : 0x2;
}



al_type *
ep_al_add( vec2d *ctr, el_type *l, vec2d bu[], ep_type *ep, ep_array *aep )
{
al_type	*al;
vec2d	end;
	ALLOC_AEP_DATA( al, al_type, aep );

	al->ctr = *ctr;
	al->link = l;

	al->bno[0] = al->bno[1] = -1;


        al->flag = 0x0f;
        if( IS_IN_BGP(*ctr, aep->hbgp) )   al->flag |= 0x10;
        end.x  = ctr->x + l->v.x;
        end.y  = ctr->y + l->v.y;
        if( IS_IN_BGP(end, aep->hbgp) )    al->flag |= 0x20;

	al->nal = ep->al;
	ep->al = al;

	return( al );
}


/*
static void
ep_al_point_flag1( vec2d *ctr, el_type *l, vec2d bu[],
					float x, float y, int *flag, int mask )
{
vec2d   v,	v1;
float   t,      h,      dp;

        v.x = x - ctr->x;
        v.y = y - ctr->y;

	if( v.x * bu[0].x + v.y* bu[0].y  <= 0 ){
		dp = v.x * bu[0].y - v.y * bu[0].x;
		if( 0 < dp )	*flag |= mask;
		return;
	}

	v1.x = v.x - l->v.x;
	v1.y = v.y - l->v.y;

	if( v1.x * bu[1].x + v1.y* bu[1].y  >= 0 ){
		dp = v1.x * bu[1].y - v1.y * bu[1].x;
		if( 0 < dp )	*flag |= mask;
                return;
        }




        t = -l->u.y * v.x + l->u.x * v.y;

	if( t <= 0 || t >= l->len ){
        	dp = l->u.x * v.x + l->u.y * v.y;
		if( 0 < dp )	*flag |= mask;
                return;
        }

        dp = l->u.x * v.x + l->u.y * v.y;
        h = l->c_prb * t * (t- l->len );

	if( h < dp )	*flag |= mask;
}
*/
