/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/

/**************************
 ***   EpAl_1.c  	***
 **************************/

#include <stdio.h>
#include <math.h>

#include	"Uigp/igp.h"
#include        "Uprm/prm.h"


#include        "GpType/GpType.h"

#include        "EpType/EpType.h"
#include        "EpIn.h"





#define		IS_IN_BGP( p, hbgp )	( (p).x > -hbgp && (p).x < hbgp && \
					(p).y > -hbgp && (p).y < hbgp )

#define         IS_IN_BGP_ENV( p, v, hbgp ) \
			( ((p).x -(v).x) > -hbgp && ((p).x -(v).x) < hbgp && \
			  ((p).y -(v).y) > -hbgp && ((p).y -(v).y) < hbgp )





static int	ep_el_bes_1( vec2d *ctr, el_type *l, int *Flag, vec2d *tv );


//static void	ep_al_one_1( vec2d *vm, el_type *l, int bun,
//					ep_type *ep, ep_array *aep );

static void		ep_al_one_1( vec2d *vm, el_type *l,
						vec2d *End, int Flag, vec2d *Tv,
						int bun, ep_type *ep, ep_array *aep );



al_type *	ep_al_add_1( vec2d *ctr, el_type *l, int flag,
						ep_type *ep, ep_array *aep );


/*
static float	ep_al_point_side( vec2d *ctr, el_type *l,
					vec2d *bu, float x, float y );
*/
static float	ep_al_point_side_0( vec2d *ctr, el_type *l, float x, float y );
static float	ep_al_point_side_1( vec2d *ctr, el_type *l, float x, float y );
static float	ep_al_point_side_2( vec2d *ctr, el_type *l, float x, float y );
static float	ep_al_point_side_3( vec2d *ctr, el_type *l, float x, float y );


#if defined(__SYMBIAN32__) && defined(__DEBUG)

static float (* const Aal_point_side[4])(vec2d *ctr, el_type *l, float x, float y ) = { ep_al_point_side_0,
						ep_al_point_side_1,
						ep_al_point_side_2,
						ep_al_point_side_3 };
#else
static const float (* const Aal_point_side[4])(vec2d *ctr, el_type *l, float x, float y ) = { ep_al_point_side_0,
						ep_al_point_side_1,
						ep_al_point_side_2,
						ep_al_point_side_3 };
#endif //defined(__SYMBIAN32__) && defined(_DEBUG) 





void
ep_al_1( vec2d *ctr, el_type *l, ep_array *aep )
{
vec2d	v0,	v1;
int	i0,	j0,	i1,	j1,	i,	j;
int	bun;
vec2d	End,	Tv;
int	Flag;


	v0 = *ctr;
	v1 = *ctr;

	End.x = ctr->x + l->v.x;
	End.y = ctr->y + l->v.y;
	VEC2D_UPDATE_BOUND( End, v0, v1 );



	bun = ep_el_bes_1( ctr, l, &Flag, &Tv );


	if( Flag == 1 )	VEC2D_UPDATE_BOUND( Tv, v0, v1 );



	i0 = (v0.x + LSIZE*aep->bgp)* aep->bgp_inv;
	i1 = (v1.x + LSIZE*aep->bgp)* aep->bgp_inv;

	j0 = (v0.y + LSIZE*aep->bgp)* aep->bgp_inv;
	j1 = (v1.y + LSIZE*aep->bgp)* aep->bgp_inv;


	if( i1 < JSIZE || i0 > aep->Ni-JSIZE )	return;
	if( j1 < JSIZE || j0 > aep->Nj-JSIZE )	return;

	i0 = PUSH_TO_RANGE( i0 , JSIZE, aep->Ni-JSIZE );
	i1 = PUSH_TO_RANGE( i1 , JSIZE, aep->Ni-JSIZE-1 );

	j0 = PUSH_TO_RANGE( j0 , JSIZE, aep->Nj-JSIZE );
	j1 = PUSH_TO_RANGE( j1 , JSIZE, aep->Nj-JSIZE-1 );


	for( i = i0 ; i <= i1 ; i++ )
		for( j = j0 ; j <= j1 ; j++ )
			ep_al_one_1( ctr, l, &End, Flag, &Tv, bun, &aep->ep[i][j], aep );
	
}




static void
ep_al_one_1( vec2d *vm, el_type *l,
			vec2d *End, int Flag, vec2d *Tv,
			int bun, ep_type *ep, ep_array *aep )
{
vec2d	ctr;
int	flag;
al_type	*al;

	ctr.x = vm->x - ep->ctr.x;
	ctr.y = vm->y - ep->ctr.y;

	if( IS_IN_BGP(ctr, aep->hbgp) ){
		if( IS_IN_BGP_ENV( *End, ep->ctr, aep->hbgp ) )
			ep_al_add_1( &ctr, l, 0x10 | 0x20 |0x0f, ep, aep );
		else	ep_al_add_1( &ctr, l, 0x10 |0x0f, ep, aep );
		return;
	}



	if( IS_IN_BGP_ENV( *End, ep->ctr, aep->hbgp ) ){
		ep_al_add_1( &ctr, l, 0x20, ep, aep );
		return;
	}


	if( Flag )
		if( IS_IN_BGP_ENV( *Tv, ep->ctr, aep->hbgp ) ){
			al = ep_al_add_1( &ctr, l, 0x0f, ep, aep );
			ep_range_cross_link( al, F_RIGHT, ep, aep);
			ep_range_cross_link( al, F_LEFT, ep, aep );
			return;
		}



	flag = 0;

	flag |= ( (*Aal_point_side[bun])( &ctr, l, -aep->hbgp, -aep->hbgp) > 0)? 0x1:0x2;
	flag |= ( (*Aal_point_side[bun])( &ctr, l, -aep->hbgp, +aep->hbgp ) > 0)? 0x1:0x2;
	flag |= ( (*Aal_point_side[bun])( &ctr, l, +aep->hbgp, -aep->hbgp ) > 0)? 0x1:0x2;
	flag |= ( (*Aal_point_side[bun])( &ctr, l, +aep->hbgp, +aep->hbgp ) > 0)? 0x1:0x2;


	if( flag != 0x03 )	return;

	al = ep_al_add_1( &ctr, l, 0x0f, ep, aep );
	ep_range_cross_link( al, F_RIGHT, ep, aep);
	ep_range_cross_link( al, F_LEFT, ep, aep );
}





static int
ep_el_bes_1( vec2d *ctr, el_type *l,
		int *Flag, vec2d *tv )
{
float	t;
	if( ABS(l->v.y) > ABS(l->v.x) ){
		*Flag = el_tanget_point_01( l, ctr, &t, tv );

		if( l->v.y < 0 )	return( 0 );
		return( 1 );
	}


	*Flag = el_tanget_point_10( l, ctr, &t, tv );
	if( l->v.x < 0 )	return( 2 );
	return( 3 );

}
			
			
		



al_type *
ep_al_add_1( vec2d *ctr, el_type *l, int flag, ep_type *ep, ep_array *aep )
{
al_type	*al;

	ALLOC_AEP_DATA( al, al_type, aep );

	al->ctr = *ctr;
	al->link = l;

	al->bno[0] = al->bno[1] = -1;


	al->flag = flag;

	al->nal = ep->al;
	ep->al = al;

	return( al );
}




/*
static float
ep_al_point_side( vec2d *ctr, el_type *l, vec2d *bu, float x, float y )
{
vec2d   v,	v1;
float   t,      h,      dp;

        v.x = x - ctr->x;
        v.y = y - ctr->y;

	if( v.x * bu->x + v.y* bu->y  <= 0 ){
		dp = v.x * bu->y - v.y * bu->x;
		return( dp );
	}

	v1.x = v.x - l->v.x;
	v1.y = v.y - l->v.y;

	if( v1.x * bu->x + v1.y* bu->y  >= 0 ){
		dp = v1.x * bu->y - v1.y * bu->x;
		return( dp );
        }




        t = -l->u.y * v.x + l->u.x * v.y;

	if( t <= 0 || t >= l->len ){
        	dp = l->u.x * v.x + l->u.y * v.y;
		return( dp );
        }

        dp = l->u.x * v.x + l->u.y * v.y;
        h = l->c_prb * t * (t- l->len );

	return( dp - h );
}
*/


static float
ep_al_point_side_0( vec2d *ctr, el_type *l, float x, float y )
{		/* { 0, -1 } */
vec2d   v,	v1;
float   t,      h,      dp;

        v.x = x - ctr->x;
        v.y = y - ctr->y;

	if( -v.y  <= 0 )
		return( -v.x );

	v1.x = v.x - l->v.x;
	v1.y = v.y - l->v.y;

	if( -v1.y  >= 0 )
		return( -v1.x );


        t = -l->u.y * v.x + l->u.x * v.y;

	if( t <= 0 || t >= l->len ){
        	dp = l->u.x * v.x + l->u.y * v.y;
		return( dp );
        }

        dp = l->u.x * v.x + l->u.y * v.y;
        h = l->c_prb * t * (t- l->len );

	return( dp - h );
}


static float
ep_al_point_side_1( vec2d *ctr, el_type *l, float x, float y )
{		/* { 0, 1 } */
vec2d   v,	v1;
float   t,      h,      dp;

        v.x = x - ctr->x;
        v.y = y - ctr->y;

	if( v.y  <= 0 )
		return( v.x );

	v1.x = v.x - l->v.x;
	v1.y = v.y - l->v.y;

	if( v1.y  >= 0 )
		return( v1.x );


        t = -l->u.y * v.x + l->u.x * v.y;

	if( t <= 0 || t >= l->len ){
        	dp = l->u.x * v.x + l->u.y * v.y;
		return( dp );
        }

        dp = l->u.x * v.x + l->u.y * v.y;
        h = l->c_prb * t * (t- l->len );

	return( dp - h );
}



static float
ep_al_point_side_2( vec2d *ctr, el_type *l, float x, float y )
{	/* { -1, 0 } */
vec2d   v,	v1;
float   t,      h,      dp;

        v.x = x - ctr->x;
        v.y = y - ctr->y;

	if( -v.x  <= 0 )
		return( v.y );

	v1.x = v.x - l->v.x;
	v1.y = v.y - l->v.y;

	if( -v1.x  >= 0 )
		return( v1.y );


        t = -l->u.y * v.x + l->u.x * v.y;

	if( t <= 0 || t >= l->len ){
        	dp = l->u.x * v.x + l->u.y * v.y;
		return( dp );
        }

        dp = l->u.x * v.x + l->u.y * v.y;
        h = l->c_prb * t * (t- l->len );

	return( dp - h );
}


static float
ep_al_point_side_3( vec2d *ctr, el_type *l, float x, float y )
{	/* { 1, 0 } */
vec2d   v,	v1;
float   t,      h,      dp;

        v.x = x - ctr->x;
        v.y = y - ctr->y;

	if( v.x  <= 0 )
		return( -v.y );

	v1.x = v.x - l->v.x;
	v1.y = v.y - l->v.y;

	if( v1.x  >= 0 )
		return( -v1.y );


        t = -l->u.y * v.x + l->u.x * v.y;

	if( t <= 0 || t >= l->len ){
        	dp = l->u.x * v.x + l->u.y * v.y;
		return( dp );
        }

        dp = l->u.x * v.x + l->u.y * v.y;
        h = l->c_prb * t * (t- l->len );

	return( dp - h );
}
