/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/

/**************************
 ***   EpTool.c  	***
 **************************/

#include <stdio.h>
#include <math.h>

#include	"Uigp/igp.h"


#include	"EpType/EpType.h"
#include	"Ep.h"

#include	"EpIn.h"



static void	ep_array_alloc( ep_array *aep, int ni, int nj );


ep_array *
aep_alloc()
{
ep_array	*aep;

	aep = (ep_array *)malloc( sizeof( ep_array ) );

	aep->Ni = aep->Nj = -1;

	aep->NI = aep->NJ = -1;

	aep->ep = NULL;

	return( aep );
}



void
aep_destroy( ep_array *aep )
{
	ep_array_free( aep );

	free( aep );
}



void
ep_array_init( ep_array *aep, int bgp, int ni, int nj )
{
int     i,	j,	n,	k,	nb;
ep_type	*ep;
vec2d	v,	v0;


	ep_array_alloc( aep, ni, nj );

	aep->bgp = bgp;
	aep->bgp_inv = 1.0/bgp;

	aep->hbgp = 0.5 * aep->bgp;

	epValue_init( &aep->epValue, aep->bgp );

	aep->Ni = ni;
	aep->Nj = nj;


	v0.x = v0.y = -bgp *(LSIZE-0.5);

	for( i = 0, v.x = v0.x ; i < aep->Ni ; i++, v.x += bgp)
		for( j = 0 , v.y = v0.y ; j < aep->Nj ; j++, v.y += bgp ){

			ep = &aep->ep[i][j];
			ep->ctr.x = v.x;
			ep->ctr.y = v.y;


/*
			ep->el = NULL;
*/


			ep->r_no = 0;
			ep->range = NULL;

			ep->al = NULL;


			for( n = i-1, nb = 0 ; n <= i+1 ; n++  )
				for( k = j-1 ; k <= j+1 ; k++ ){
					if( n < 0 || n >= aep->Ni ||
							k < 0 || k >= aep->Nj )
						ep->ng[nb++] = NULL;
					else    ep->ng[nb++] = &aep->ep[n][k];
				}
		}

	aep->p = aep->data;
}




void
ep_array_clear( ep_array *aep, int bgp, int Ni, int Nj )
{
int     i,	j;
ep_type	*ep;

	if( Ni > aep->NI || Nj > aep->NJ || aep->bgp != bgp ){
		ep_array_free( aep );
		ep_array_init( aep, bgp, Ni, Nj );

		return;
	}



	aep->Ni = Ni;
	aep->Nj = Nj;
	for( i = 0 ; i < aep->Ni ; i++ )
		for( j = 0 ; j < aep->Nj ; j++ ){
			ep = &aep->ep[i][j];



/*
			ep->el = NULL;
*/


			ep->range = NULL;
			ep->r_no = 0;


			ep->al = NULL;


			ep->b_no = 0;
		}


	aep->p = aep->data;
}



void
ep_array_realloc( ep_array *aep, int ni, int nj )
{
	if( aep->Ni == ni && aep->Nj == nj )  return;
 
	ep_array_free( aep );
 
	ep_array_alloc( aep, ni, nj );
}
 
 
static void
ep_array_alloc( ep_array *aep, int ni, int nj )
{
int	i;  
int	byte,	data_size;
 
	aep->NI = aep->Ni  = ni ;
	aep->NJ = aep->Nj  = nj ;
 
	aep->ep = (ep_type **)malloc( aep->NI * sizeof( ep_type * ) );
 
	byte = aep->NJ * sizeof(ep_type);
 
	for( i = 0 ; i < aep->NI ; i++ )
		aep->ep[i] = (ep_type *)calloc( 1, byte );


	data_size = aep->NI*aep->NJ *( 8*sizeof(al_type) +
				5*sizeof(edge_type) +
				8*sizeof(range_type) + 16*sizeof(rlink_type) +
				4*sizeof(bound_type) +
				4*sizeof(bgp_type) );

/*
	data_size = aep->NI*aep->NJ *( 8*sizeof(al_type)) +
			(aep->NI)*(aep->NJ) *
				( 8*sizeof(range_type) + 16*sizeof(rlink_type) +
				4*sizeof(bound_type) +
				4*sizeof(bgp_type) );
*/
#ifdef PALMOS
	if( data_size > 32000 )	data_size = 32000;
#endif


	aep->data = (char *)malloc( data_size );
	aep->p = aep->data;
}
 
 
 
void
ep_array_free( ep_array *aep )
{
int	i;  
 
	if( aep->ep == NULL )	return;

	for( i = 0 ; i < aep->NI ; i++ )
		free( aep->ep[i] );
 
	if( aep->ep != NULL )
		free( aep->ep );
	aep->ep = NULL;

	aep->NI = aep->NJ = 0;

	if( aep->data != NULL )
		free( aep->data );
	 aep->data = NULL;
}


#ifndef PLAYER
void
ep_print( ep_array *aep, int i, int j )
{
ep_type	*ep;
bound_type	*b;
int	k;

	ep = &aep->ep[i][j];

	for( k = 0 ; k < ep->b_no ; k++ ){
		b = ep->bound[k];

		if( b->Mbgp == NULL ){
			fprintf( stderr, "___	" );
			continue;
		}

		fprintf( stderr, "%3d	", (int)b->Mbgp->val[0] );
	}
}
#endif
