/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/

#include	"EpType/EpType.h"
#include	"EpIn.h"


typedef struct {
	int	direct;
	int	side;
	al_type	*al;

}	ar_type;


static void	range_one( ep_type *ep, al_type *al, int direct,
						int side, ep_array *aep );

static int	range_next( ep_type *ep, ar_type *ar, ar_type *nar );

static al_type *	al_find( ep_type *ep, el_type *link );



void
ep_range( ep_type *ep, ep_array *aep )
{
al_type	*al;


	for( al = ep->al ; al != NULL ; al = al->nal ){
		if( !(al->flag & 0x30) )	continue;

		al->rno[F_RIGHT] = al->rno[F_LEFT] = 255;
	}


	for( al = ep->al ; al != NULL ; al = al->nal ){
		if( !(al->flag & 0x30) )	continue;

		if( al->rno[F_RIGHT] == 255 ){

			if( !(al->flag & 0x10) )
				range_one( ep, al, F_BEGIN, F_RIGHT, aep );

			else	if( !(al->flag & 0x20) )
					range_one( ep, al, F_END, F_RIGHT, aep);
		}


		if( al->rno[F_LEFT] == 255 ){

			if( !(al->flag & 0x10) )
				range_one( ep, al, F_BEGIN, F_LEFT, aep );

			else	if( !(al->flag & 0x20) )
					range_one( ep, al, F_END, F_LEFT, aep );
		}
	}


	for( al = ep->al ; al != NULL ; al = al->nal ){
		if( !(al->flag & 0x30) )	continue;

		if( al->rno[F_RIGHT] == 255 )
			range_one( ep, al, F_BEGIN, F_RIGHT, aep );

		if( al->rno[F_LEFT] == 255 )
			range_one( ep, al, F_BEGIN, F_LEFT, aep );
	}
}


void
ep_range_cross_link( al_type *al, int side, ep_type *ep, ep_array *aep )
{
range_type      *r;
rlink_type      *rl;

        r = range_alloc( ep, 1, aep );

        rl = &r->rl[0];

        rl->side = side;
        rl->direct = F_BEGIN;

	rl->al = al;
}




static void
range_one( ep_type *ep, al_type *al, int direct, int side, ep_array *aep )
{
ar_type	Aar[128],	*ar;
range_type	*r;
rlink_type	*rl;
int	i,	no;

	Aar[0].al = al;
	Aar[0].side = side;
	Aar[0].direct = direct;
	al->rno[side] = 1;

	no = 1;

	while( range_next( ep, &Aar[no-1], &Aar[no] ) == 1 ){
		ar = &Aar[no];
		if( ar->al->rno[ar->side] == 1 )	break;

		ar->al->rno[ar->side] = 1;
		no++;
	}

	r = range_alloc( ep, no, aep );

	for( i = 0 ; i < no ; i++ ){
		rl = &r->rl[i];
		ar = &Aar[i];

		rl->side = ar->side;
		rl->direct = ar->direct;

		rl->al = ar->al;
	}
}



static int
range_next( ep_type *ep, ar_type *ar, ar_type *nar )
{
al_type	*al,	*nal;
el_type	*l;
jnt_type	*ej;
int	bno,	side;

	al = ar->al;

	if( ar->direct == F_BEGIN ){
		if( !(al->flag & 0x20) )	return( 0 );

		if( (l = LINK_NEXT( al->link)) != NULL ){
			nal = al_find( ep, l );
			if( nal == NULL )	return( 0 );

			nar->al = nal;
			nar->side = ar->side;
			nar->direct = ar->direct;
			return( 1 );
		}

		ej = al->link->p[F_END].p.jnt;
		if( ej == NULL )	return(0);

		bno = al->link->p[F_END].bno;

		if( ar->side == F_RIGHT )
			bno = ( bno < ej->brn_no-1)? bno+1 : 0;
		else	bno = ( bno > 0)? bno-1 : ej->brn_no-1;

		if( ej->brn[bno]->direction == F_END )
			side = INV_FLAG(ar->side);
		else	side = ar->side;


		nal = al_find( ep, ej->brn[bno]->link );
		if( nal == NULL )	return( 0 );

		nar->al = nal;
		nar->side = side;
		nar->direct = ej->brn[bno]->direction;
		return( 1 );
	}


	if( !(al->flag & 0x10) )	return( 0 );

	if( (l = LINK_PREV( al->link)) != NULL ){
		nal = al_find( ep, l );
		if( nal == NULL )	return( 0 );

		nar->al = nal;
		nar->side = ar->side;
		nar->direct = ar->direct;
		return( 1 );
	}

	ej = al->link->p[F_BEGIN].p.jnt;
	if( ej == NULL )	return(0);

	bno = al->link->p[F_BEGIN].bno;

	if( ar->side == F_LEFT )
		bno = ( bno < ej->brn_no-1)? bno+1 : 0;
	else	bno = ( bno > 0)? bno-1 : ej->brn_no-1;

	if( ej->brn[bno]->direction == F_BEGIN )
		side = INV_FLAG(ar->side);
	else	side = ar->side;


	nal = al_find( ep, ej->brn[bno]->link );
	if( nal == NULL )	return( 0 );

	nar->al = nal;
	nar->side = side;
	nar->direct = ej->brn[bno]->direction;
	return( 1 );
}
			

		
static al_type *
al_find( ep_type *ep, el_type *link )
{
al_type	*al;

	for( al = ep->al ; al != NULL ; al = al->nal ){

		if( al->link != link )	continue;

		if( al->flag & 0x30 )	return( al );
		return( NULL );
	}

	return( NULL );
}


	


range_type *
range_alloc( ep_type *ep, int rl_no, ep_array *aep )
{
range_type	*r;

	ep->r_no++;
/*
	r = ( range_type *)malloc( sizeof(range_type) );
*/
	ALLOC_AEP_DATA( r, range_type, aep );
	r->rl_no = rl_no;

/*
	r->rl = ( rlink_type *)malloc( sizeof(rlink_type)*r->rl_no );
*/
	ALLOC_AEP_DATA_N( r->rl, rlink_type, r->rl_no, aep );

	r->nr = ep->range;
	ep->range = r;

	return( r );
}


void
range_free( range_type *r )
{
	free( r->rl );
	free( r );
}
