/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**************************
***	RangePoint.c	***
***************************/

#include	<stdio.h>
#include	<math.h>

#include	"Uigp/igp.h"
#include	"EpType/EpType.h"


#include	"EpIn.h"


static void	ep_al_point_range( al_type *al, vec2d *v, int *rno, float *dis);


void
ep_range_Frange( ep_type *ep, range_type *r, int rno, int Frange[], int Fal[], float hbgp )
{
int	j,	flag_al;
vec2d	rp;

	for( j = 0 ; j < ep->r_no ;  j++ )
		if( Fal[rno] & Fal[j] )	Fal[rno] |= Fal[j];

	ep_range_set_rp( ep, r, &rp, hbgp );
	flag_al = Fal[rno];

	for( j = 0 ; j < ep->r_no ;  j++ ){

		if( flag_al & Fal[j] ){
	 		flag_al |= Fal[j];
			continue;
		}

		if( ep_range_is_point_in( ep, j, &rp ) ){
			Frange[j] |= ( 1L<< rno );
	 		flag_al |= Fal[j];
		}
	}
}



int
ep_range_is_point_in( ep_type *ep, int rno, vec2d *v )
{
al_type	*al;
int	no;
float	dis;


	dis = 1000;
	no = -1;


	for( al = ep->al ; al != NULL ; al = al->nal ){
		if( al->rno[F_RIGHT] != rno && al->rno[F_LEFT] != rno )
			continue;


		ep_al_point_range( al, v, &no, &dis );
	}


	if( rno == no )	return( 1 );
	return( 0 );
}



static void 
ep_al_point_range( al_type *al, vec2d *v, int *rno, float *dis )
{
vec2d	p;
el_type	*l;
float	u,	t,	dt,	tu;


	l = al->link;
	p.x = v->x - al->ctr.x;
	p.y = v->y - al->ctr.y;


	u = l->u.x * p.x + l->u.y * p.y;
	t = -l->u.y * p.x + l->u.x * p.y;


	if( t < 0 ){

		tu = u*u + 1.5*t*t;
		if( !(al->flag & 0x10) ){
			if( ABS(l->v.y) > ABS(l->v.x) )
				u  = ( l->v.y < 0 )? -p.x : p.x;
			else	u  = ( l->v.x < 0 )? p.y : -p.y;
		}
	}
	else	if( (dt = t - l->len) > 0  ){
			tu = u*u + 1.5*dt*dt;

			if( !(al->flag & 0x20) ){
				if( ABS(l->v.y) > ABS(l->v.x) )
					u  = ( l->v.y < 0 )? -(p.x-l->v.x) :
							(p.x - l->v.x);
				else	u  = ( l->v.x < 0 )? (p.y - l->v.y):
							-(p.y - l->v.y);
			}
		}
		else	{
			if( (u < 0) != (l->c_prb < 0) )
				u -= l->c_prb*t *(t - l->len );

			tu = u*u;
		}



	if( tu > *dis )	return;

	*dis = tu;
	if( u >= 0 )
		*rno = al->rno[F_RIGHT];
	else	*rno = al->rno[F_LEFT];
}
