/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/

/*************************l
 ***   RangeRp.c  	***
 **************************/

#include <stdio.h>
#include <math.h>

#include	"Uigp/igp.h"
#include        "Uprm/prm.h"


#include        "GpType/GpType.h"

#include        "EpType/EpType.h"



static void    ep_link_rp( vec2d *vm, el_type *l, vec2d *rp, float hbgp );

static float	ep_link_projection( vec2d *vm, el_type *l,
				float x, float y, vec2d *rp );



void
ep_range_set_rp( ep_type *ep, range_type *r, vec2d *rp, float hbgp )
{
rlink_type	*rl;
al_type	*al;
 

	if( r->rl_no > 1 ){
		rl = &r->rl[0];
		al = rl->al;


		if( rl->direct == F_BEGIN ){
			rp->x = al->ctr.x + al->link->v.x;
			rp->y = al->ctr.y + al->link->v.y;
		}
		else	*rp = al->ctr;

		return;
	}

	al = r->rl[0].al;

	ep_link_rp( &al->ctr, al->link, rp, hbgp );
}






static void
ep_link_rp( vec2d *vm, el_type *l, vec2d *rp, float hbgp )
{
vec2d	v,	tmp;
float	t,	t0;

	v.x = vm->x;
	v.y = vm->y;

	t0 = ep_link_projection( &v, l, 0.0, 0.0, rp );

	if( t0 < hbgp )	return;


	t = ep_link_projection( &v, l, -hbgp, -hbgp, &tmp );
	if( t < t0 ){
		if( t < hbgp ){
			rp->x = tmp.x;
			rp->y = tmp.y;
			return;
		}
		*rp = tmp;
		t0 = t;
	}


	t = ep_link_projection( &v, l, -hbgp, +hbgp, &tmp );
	if( t < t0 ){
		if( t < hbgp ){
			rp->x = tmp.x;
			rp->y = tmp.y;
			return;
		}
		*rp = tmp;
		t0 = t;
	}


	t = ep_link_projection( &v, l, +hbgp, -hbgp, &tmp );
	if( t < t0 ){
		if( t < hbgp ){
			rp->x = tmp.x;
			rp->y = tmp.y;
			return;
		}
		*rp = tmp;
		t0 = t;
	}


	t = ep_link_projection( &v, l, +hbgp, +hbgp, &tmp );
	if( t < t0 ){
		if( t < hbgp ){
			rp->x = tmp.x;
			rp->y = tmp.y;
			return;
		}
		*rp = tmp;
		t0 = t;
	}


	if( t0 == 100 ){
		if( el_tanget_point_01( l, vm, &t, rp ) )
			return;
		if( el_tanget_point_10( l, vm, &t, rp ) )
			return;
		*rp = *vm;
	}
}







static float
ep_link_projection( vec2d *vm, el_type *l, float x, float y, vec2d *rp )
{
float	t,	a,	b;
vec2d	v;

	v.x = x - vm->x;
	v.y = y - vm->y;


	t = -l->u.y * v.x + l->u.x * v.y;

	if( t < 0 || t > l->len )	return( 100.0 );


	el_t2xy( vm, l, t, &v );


	*rp = v;
	a = ABS(rp->x);
	b = ABS(rp->y);
	if( a < b )	return( b );
	return( a );
}
