/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**************************
 ***	EpBgpFill.c	***
 **************************/
#include	<stdio.h>   
#include	<math.h>

#include	"EpType/EpType.h"
#include	"Ep/Ep.h"
#include	"EpBgp/EpBgp.h"
#include	"EpValue/EpValue.h"


#include	"EpBgp.h"


static void	ep_bgp_fill_one( gp_type *gp, ep_type *ep, float hb, epValue_type *epValue );

static void	ep_bgp_fill_add( gp_type *gp,
			float x, float y, ep_type *ep, int bno, epValue_type *epValue );





void
aep_bgp_fill( gp_array *agp, ep_array *aep )
{
int	i,	j,	i1,	j1;
ep_type	*ep;
gp_type	*gp;
float	hbgp = aep->hbgp;



        for( i = LSIZE, i1 = aep->Ni0 ; i < aep->Ni-LSIZE ; i++, i1++ ){
        	if( i1 < 0 || i1 >= agp->Ni-1 )       continue;

        	for( j = LSIZE, j1 = aep->Nj0 ; j < aep->Nj-LSIZE ; j++, j1++ ){
				if( j1 < 0 || j1 >= agp->Nj-1 )   continue;

                gp = &agp->gp[i1][j1];
                ep = &aep->ep[i][j];


				ep_bgp_fill_one( gp, ep, hbgp, &aep->epValue );
			}
		}
}



static void
ep_bgp_fill_one( gp_type *gp, ep_type *ep, float hbgp, epValue_type *epValue )
{
bgp_type	*bp,	**bpp;
int     bno,    bgp_in[4],      a;
float	hb = hbgp *0.5;

	bno = ep_middle_bgp( ep, bgp_in, hbgp );


	bpp = &gp->bgp;
	for( bp = gp->bgp ; bp != NULL ; ){

		if( ABS(bp->ctr.x) != hb || ABS(bp->ctr.y) != hb ){
			*bpp = bp->p;
			free( bp );
			bp = *bpp;
			continue;
		}

		if( bp->ctr.x > 0 )
			a = ( bp->ctr.y < 0 )? 3 : 2;
		else	a = ( bp->ctr.y < 0 )? 0 : 1;

		if( bgp_in[a] == 0 ){
			*bpp = bp->p;
			free( bp );
			bp = *bpp;
			continue;
		}

		bgp_in[a] = 0;
		bpp = &bp->p;
		bp = bp->p;
	}
			
	if( bgp_in[0] ==0 && bgp_in[1] ==0 && bgp_in[2] ==0 && bgp_in[3] ==0 )
		return;


	if( bgp_in[0] == 1 )
		ep_bgp_fill_add( gp, -hb, -hb, ep, bno, epValue );


	if( bgp_in[1] == 1 )
		ep_bgp_fill_add( gp, -hb, +hb, ep, bno, epValue );


	if( bgp_in[3] == 1 )
		ep_bgp_fill_add( gp, +hb, -hb, ep, bno, epValue );
	

	if( bgp_in[2] == 1 )
		ep_bgp_fill_add( gp, +hb, +hb, ep, bno, epValue );
}



static void
ep_bgp_fill_add( gp_type *gp, float x, float y, ep_type *ep, int bno, epValue_type *epValue )
{
bgp_type	*bp;
float	val[3];

	bp = BGP_ALLOC();
	bp->ctr.x = x;
	bp->ctr.y = y;

	ep_bound_value3( epValue, ep, bno, &bp->ctr, val );

	bp->val[0] = F2QCOLOR( val[0] );
	bp->val[1] = F2QCOLOR( val[1] );
	bp->val[2] = F2QCOLOR( val[2] );

	
	bp->p = gp->bgp;
	gp->bgp = bp;
}
