/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/

/**************************
 ***   EpBgpGet.c      	***
 **************************/

#include <stdio.h>
#include <math.h>

#include	"Uigp/igp.h"
#include        "Uprm/prm.h"


#include        "GpType/GpType.h"

#include        "Ep/Ep.h"
#include        "EpBgp.h"


void	ep_bgp_get_average_one( gp_type *gp, ep_type *ep, ep_array *aep );


void
ep_bgp_get_average( gp_array *agp, ep_array *aep )
{
gp_type		*gp;
ep_type		*ep;

int	i,	j,	i1,	j1;


	for( i = JSIZE, i1 = aep->Ni0-1; i < aep->Ni-JSIZE ; i++, i1++ )
		for( j = JSIZE, j1 = aep->Nj0-1; j < aep->Nj-JSIZE ; j++, j1++){

			gp = &agp->gp[i1][j1];

			ep = &aep->ep[i][j];

			ep_bgp_get_average_one( gp, ep, aep );
		}



	/* free boundray bgp */
	ep_bgp_boundray( agp, aep );

}


void
ep_bgp_get_average_decoding( gp_array *agp, ep_array *aep )
{
gp_type		*gp;
ep_type		*ep;

int	i,	j,	i1,	j1;

	if( aep->Ni0 != 1 && aep->Nj0 != 1 ){
		ep = &aep->ep[JSIZE][JSIZE];
		gp = &agp->gp[aep->Ni0-1][aep->Nj0-1];

		ep_bgp_get_average_one( gp, ep, aep );
	}

	if( aep->Ni0 != 1 )
		for( j = JSIZE+1, j1 = aep->Nj0; j < aep->Nj-JSIZE-1 ;j++,j1++){
			gp = &agp->gp[aep->Ni0-1][j1];

			ep = &aep->ep[JSIZE][j];
			ep_bgp_get_average_one( gp, ep, aep );
		}


	if( aep->Nj0 != 1 )
		for( i = JSIZE+1, i1 = aep->Ni0; i < aep->Ni-JSIZE-1 ;i++,i1++){
			gp = &agp->gp[i1][aep->Nj0-1];

			ep = &aep->ep[i][JSIZE];
			ep_bgp_get_average_one( gp, ep, aep );
		}
}





void
ep_bgp_get_average_one( gp_type *gp, ep_type *ep, ep_array *aep )
{
bound_type	*b;
range_type	*r;
rlink_type	*rl;
al_type	*al;
bgp_type	*bp;
int	bno,	i,	no[MAX_BOUND];
edge_type	*e;
float	dis;


	for( bno = 0 ; bno < ep->b_no ; bno++ ){
		ep->bound[bno]->bgp = NULL;
		no[bno] = 0;
	}

	for( bp = gp->bgp ; bp != NULL ; bp = bp->p ){
		bno = ep_bound_point( ep, &bp->ctr, &dis );
		if( bno < 0 )	continue;
		b = ep->bound[bno];
		if( b->bgp == NULL ){
			ALLOC_AEP_DATA( b->bgp, ebgp_type, aep );

			b->bgp->ctr = bp->ctr;
			b->bgp->val[0] =  QCOLOR2F( bp->val[0] );
			b->bgp->val[1] =  QCOLOR2F( bp->val[1] );
			b->bgp->val[2] =  QCOLOR2F( bp->val[2] ); 

			no[bno] = 1;

			b->bgp->p = NULL;
			continue;
		}

		b->bgp->ctr.x += bp->ctr.x;
		b->bgp->ctr.y += bp->ctr.y;

		b->bgp->val[SP_Y] += QCOLOR2F( bp->val[SP_Y] );
		b->bgp->val[SP_Q] += QCOLOR2F( bp->val[SP_Q] );
		b->bgp->val[SP_A] += QCOLOR2F( bp->val[SP_A] );

		no[bno]++;
	}



	for( r = ep->range ; r != NULL ; r = r->nr ){

		b = ep->bound[r->bno];

		for( i = 1 ; i < r->rl_no ; i++ ){

			rl = &r->rl[i];
			al = rl->al;

			e = al->link->pr[rl->direct].edge;
			if( e == NULL )	continue;


			if( b->bgp == NULL ){
				ALLOC_AEP_DATA( b->bgp, ebgp_type, aep);
				b->bgp->p = NULL;
				no[r->bno] = 0;

				b->bgp->ctr.x = b->bgp->ctr.y = 0;
				b->bgp->val[SP_Y] = 0;
				b->bgp->val[SP_Q] = 0;
				b->bgp->val[SP_A] = 0;
			}

			if( rl->direct == F_BEGIN ){
				b->bgp->ctr.x += al->ctr.x;
				b->bgp->ctr.y += al->ctr.y;
			}
			else	{
				b->bgp->ctr.x += al->ctr.x +al->link->v.x;
				b->bgp->ctr.y += al->ctr.y +al->link->v.y;
			}

			b->bgp->val[SP_Y] += QCOLOR2F( e->side[rl->side].bval[SP_Y]);
			b->bgp->val[SP_Q] += QCOLOR2F( e->side[rl->side].bval[SP_Q]);
			b->bgp->val[SP_A] += QCOLOR2F( e->side[rl->side].bval[SP_A]);

			no[r->bno]++;
		}
	}



	for( bno = 0 ; bno < ep->b_no ; bno++ ){
		b = ep->bound[bno];
		if( b->bgp == NULL )	continue;

		b->bgp->ctr.x /= no[bno];
		b->bgp->ctr.y /= no[bno];

		b->bgp->val[SP_Y] /= no[bno];
		b->bgp->val[SP_Q] /= no[bno];
		b->bgp->val[SP_A] /= no[bno];
	}
}
