/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/************************
 ***	EpBgpMctr.c   ***
 ************************/


#include	"Uigp/igp.h"

#include	"EpType/EpType.h"
#include	"Ep/Ep.h"
#include	"EpBgp/EpBgp.h"
#include	"EpValue/EpValue.h"



void
ep_bgp_mctr( ep_type *ep, ep_array *aep )
{
bound_type	*b;
range_type	*r;
rlink_type	*rl;
al_type	*al;
vec2d	p,	v;
int	bno,	no,	i,	j;
float	hb;
int	bgp_in[4];
ebgp_type	*bp;

	if( ep->b_no == 1 && ep->bound[0]->type == BOUND_ALL ){

		b = ep->bound[0];

		ALLOC_AEP_DATA( b->Mbgp, ebgp_type, aep );
		bp = b->Mbgp;
		bp->ctr.x = 0;
		bp->ctr.y = 0;


		return;
	}

	bno = ep_middle_bgp( ep, bgp_in, aep->hbgp );

	hb = 0.5 *aep->hbgp;
	for( i = 0 ; i < ep->b_no ; i++ ){

		b = ep->bound[i];

		p.x = p.y = 0;
		no = 0;

		for( r = b->range ; r != NULL ; r = r->p )
                        for( j = 1 ; j < r->rl_no ; j++ ){

                                rl = &r->rl[j];
				al = rl->al;

                                if( rl->direct == F_BEGIN ){
                                    p.x += al->ctr.x;
                                    p.y += al->ctr.y;
                                }
                                else    {
                                    v.x = al->ctr.x +al->link->v.x;
                                    v.y = al->ctr.y +al->link->v.y;
                                    p.x += v.x;
                                    p.y += v.y;
                                }

				no++;

			}


		if( i == bno ){
			if( bgp_in[0] ){
				p.x += -hb;
				p.y += -hb;
				no++;
			}

			if( bgp_in[1] ){
				p.x += -hb;
				p.y +=  hb;
				no++;
			}

			if( bgp_in[2] ){
				p.x +=  hb;
				p.y +=  hb;
				no++;
			}

			if( bgp_in[3] ){
				p.x +=  hb;
				p.y += -hb;
				no++;
			}
		}


		if( no == 0 )	continue;


		ALLOC_AEP_DATA( b->Mbgp, ebgp_type, aep );
		bp = b->Mbgp;
		bp->ctr.x = p.x / no;
		bp->ctr.y = p.y / no;
	}
}


void
ep_bgp_mctr_value( ep_type *ep, epValue_type *epValue )
{
bound_type	*b;
int	no,	i;
ebgp_type	*bp;


	for( i = 0 ; i < ep->b_no ; i++ ){

		b = ep->bound[i];
		if( b->Mbgp == NULL )	continue;

		no = 0;
		b->Mbgp->val[SP_Y] = b->Mbgp->val[SP_Q] = b->Mbgp->val[SP_A] =0;

		for( bp = b->bgp ; bp != NULL ; bp = bp->p ){

			b->Mbgp->val[SP_Y] += bp->val[SP_Y];
			b->Mbgp->val[SP_Q] += bp->val[SP_Q];
			b->Mbgp->val[SP_A] += bp->val[SP_A];

			no++;
		}

		if( no != 0 ){
			b->Mbgp->val[SP_Y] /= no;
			b->Mbgp->val[SP_Q] /= no;
			b->Mbgp->val[SP_A] /= no;

			continue;
		}

		ep_bound_value3( epValue, ep, i, &b->Mbgp->ctr, b->Mbgp->val );
	}
}
