/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/

/**************************
 ***   EpBgpPut.c    	***
 **************************/

#include <stdio.h>
#include <math.h>

#include	"Uigp/igp.h"

#include        "GpType/Configuration.h"

#include        "GpType/GpType.h"

#include        "EpBgp.h"



static void	ep_bgp_put_middle( gp_type *gp, ep_type *ep, float hbgp, int Fdepth );

static void	ep_bgp_put_al( ep_type *ep, al_type *al, int side );



void
ep_bgp_put( gp_array *agp, ep_array *aep, int Fmiddle )
{
gp_type		*gp;
ep_type		*ep;

al_type	*al;

int	i,	j,	i1,	j1;
int	Fdepth;

	Fdepth = agp->ad != NULL;

	for( i = LSIZE, i1 = aep->Ni0; i < aep->Ni-LSIZE ; i++, i1++ )
		for( j = LSIZE, j1 = aep->Nj0; j < aep->Nj-LSIZE ; j++, j1++ ){

			gp = &agp->gp[i1][j1];

			ep = &aep->ep[i][j];

			if( Fmiddle )
				ep_bgp_put_middle( gp, ep, aep->hbgp, Fdepth );


			for( al = ep->al ; al != NULL ; al = al->nal ){
                                if( !(al->flag & 0x30 ) )       continue;

                                ep_bgp_put_al( ep, al, F_RIGHT);
                                ep_bgp_put_al( ep, al, F_LEFT);
			}

		}

}




static void
ep_bgp_put_al( ep_type *ep, al_type *al, int side )
{
el_type	*l;
ebgp_type	*bp;
int     bno;
edge_type	*e;


	bno = al->bno[side];

	bp = ep->bound[bno]->Mbgp;


	l = al->link;

	if( al->flag & 0x10 ){
		e = l->pr[0].edge;
 
		e->side[side].bval[SP_Y] = F2QCOLOR(bp->val[SP_Y]);
		e->side[side].bval[SP_Q] = F2QCOLOR(bp->val[SP_Q]);
		e->side[side].bval[SP_A] = F2QCOLOR(bp->val[SP_A]);
	}
 
	if( al->flag & 0x20 && LINK_NEXT(l) == NULL ){
		e = l->pr[1].edge;
 
		e->side[side].bval[SP_Y] = F2QCOLOR(bp->val[SP_Y]);
		e->side[side].bval[SP_Q] = F2QCOLOR(bp->val[SP_Q]);
       		e->side[side].bval[SP_A] = F2QCOLOR(bp->val[SP_A]);
	}
}




static void
ep_bgp_put_middle( gp_type *gp, ep_type *ep, float hbgp, int Fdepth )
{
ebgp_type	*bgp;
bgp_type	*bp;
float	hb;
int	bno,	bgp_in[4];
qcolor_type	val[3];
bound_type	*b;
int	br;



	bno = ep_middle_bgp( ep, bgp_in, hbgp );
	if( bno == -1 )		return;


	b = ep->bound[bno];
	
	if( Fdepth ){
		if( b->br.no == 255 )	return;
		br = b->br.no;
	}
	else	br = 0;
	

	if( (bgp = b->Mbgp ) == NULL )	return;

	hb = 0.5 *hbgp;


#ifndef SINGLE_BGP

	val[SP_Y] = F2QCOLOR( bgp->val[SP_Y] );
	val[SP_Q] = F2QCOLOR( bgp->val[SP_Q] );
	val[SP_A] = F2QCOLOR( bgp->val[SP_A] ); 

	if( bgp_in[0] ){
		bp = BGP_ALLOC();
                bp->val[0] = val[0];
                bp->val[1] = val[1];
                bp->val[2] = val[2]; 
		bp->ctr.x = -hb;
		bp->ctr.y = -hb;
		bp->br = br;
		bp->p = gp->bgp;
		gp->bgp = bp;
	}

	if( bgp_in[1] ){
		bp = BGP_ALLOC();
                bp->val[0] = val[0];
                bp->val[1] = val[1];
                bp->val[2] = val[2]; 
		bp->br = br;
		bp->ctr.x = -hb;
		bp->ctr.y = hb;
		bp->p = gp->bgp;
		gp->bgp = bp;
	}

	if( bgp_in[3] ){
		bp = BGP_ALLOC();
                bp->val[0] = val[0];
                bp->val[1] = val[1];
                bp->val[2] = val[2]; 
		bp->br = br;
		bp->ctr.x = hb;
		bp->ctr.y = -hb;
		bp->p = gp->bgp;
		gp->bgp = bp;
	}

	if( bgp_in[2] ){
		bp = BGP_ALLOC();
                bp->val[0] = val[0];
                bp->val[1] = val[1];
                bp->val[2] = val[2]; 
		bp->br = br;
		bp->ctr.x = hb;
		bp->ctr.y = hb;
		bp->p = gp->bgp;
		gp->bgp = bp;
	}
#else
	bp = BGP_ALLOC();
	bp->ctr.x = 0;
	bp->ctr.y = 0;

	bno = 0;//XXXX cheat, reuse this variable
	if( bgp_in[0] ){
		bp->ctr.x += -hb;
		bp->ctr.y += -hb;
		bno++;
	}
	if( bgp_in[1] ){
		bp->ctr.x += -hb;
		bp->ctr.y += hb;
		bno++;
	}
	if( bgp_in[3] ){
		bp->ctr.x += hb;
		bp->ctr.y += -hb;
		bno++;
	}
	if( bgp_in[2] ){
		bp->ctr.x += hb;
		bp->ctr.y += hb;
		bno++;
	}
	if(bno==0)
	{
		free(bp);
		return;
	}
	bp->val[SP_Y] = F2QCOLOR( bgp->val[SP_Y] );
	bp->val[SP_Q] = F2QCOLOR( bgp->val[SP_Q] );
	bp->val[SP_A] = F2QCOLOR( bgp->val[SP_A] );
	bp->ctr.x/=bno;
	bp->ctr.y/=bno;
	bp->br = br;
	bp->p = gp->bgp;
	gp->bgp = bp;
#endif
}
