/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/

/**************************
 ***   EpBgpGet.c      	***
 **************************/

#include <stdio.h>
#include <math.h>

#include	"Uigp/igp.h"


#include        "GpType/GpType.h"

#include        "Ep/Ep.h"
#include        "EpBgp/EpBgp.h"

#include        "DepthEpIn.h"



static void	depth_ep_bgp_get_bgp( ep_type *ep,
				bgp_type *bp, ep_array *aep );

static void	depth_ep_bgp_get_link( ep_type *ep, ep_array *aep );



void
depth_ep_bgp_get( gp_array *agp, ep_array *aep )
{
gp_type		*gp;
ep_type		*ep;


bgp_type	*bp;

int	i,	j,	i1,	j1;


	for( i = JSIZE, i1 = aep->Ni0-1; i < aep->Ni-JSIZE ; i++, i1++ )
		for( j = JSIZE, j1 = aep->Nj0-1; j < aep->Nj-JSIZE ; j++, j1++){

			gp = &agp->gp[i1][j1];

			ep = &aep->ep[i][j];

			for( bp = gp->bgp ; bp != NULL ; bp = bp->p )
				depth_ep_bgp_get_bgp( ep, bp, aep );


			depth_ep_bgp_get_link( ep, aep );
		}



	/* free boundray bgp */
	ep_bgp_boundray( agp, aep );

}


static void
depth_ep_bgp_get_link( ep_type *ep, ep_array *aep )
{
bound_type      *b;
range_type      *r;
rlink_type      *rl;
al_type *al;
edge_type	*e;
ebgp_type	*bgp;
int     bno,    i;


	for( bno = 0 ; bno < ep->b_no ; bno++ ){
		b = ep->bound[bno];
 
		for( r = b->range ; r != NULL ; r = r->p )
			for( i = 1 ; i < r->rl_no ; i++ ){

				rl = &r->rl[i];
				al = rl->al;
 
				e = al->link->pr[rl->direct].edge;
				if( e == NULL ) continue;


				ALLOC_AEP_DATA( bgp, ebgp_type, aep );
 
				bgp->p = b->bgp;
				b->bgp = bgp;
 

				if( rl->direct == F_BEGIN ){
				    b->bgp->ctr.x = al->ctr.x;
				    b->bgp->ctr.y = al->ctr.y;
				}
				else    {
				    b->bgp->ctr.x = al->ctr.x +al->link->v.x;
				    b->bgp->ctr.y = al->ctr.y +al->link->v.y;
				}
 
 
/*
				e = al->link->pr[rl->direct].edge;
*/
				bgp->val[SP_Y] = QCOLOR2F(e->side[rl->side].bval[SP_Y]);
				bgp->val[SP_Q] = QCOLOR2F(e->side[rl->side].bval[SP_Q]);
				bgp->val[SP_A] = QCOLOR2F(e->side[rl->side].bval[SP_A]);
			}
	}
}



static void
depth_ep_bgp_get_bgp( ep_type *ep, bgp_type *bp, ep_array *aep )
{
bound_type	*b;
range_type	*r;
ebgp_type	*bgp;
al_type	*al;
int	bno,	j,	i;
float	dis;

	ALLOC_AEP_DATA( bgp, ebgp_type, aep );
	bgp->ctr = bp->ctr;
	bgp->val[0] = QCOLOR2F( bp->val[0] );
	bgp->val[1] = QCOLOR2F( bp->val[1] );
	bgp->val[2] = QCOLOR2F( bp->val[2] );

/*
	bgp->depth = bp->depth;
	bgp->br = bp->br;
*/



	if( ep->b_no == 1 && ep->bound[0]->br.no == 255 ){
		b = ep->bound[0];
		b->br.no = bp->br;
		b->br.depth = bp->depth;

		bgp->p = b->bgp;
		b->bgp = bgp;

		return;
	}

	for( i = 0 ; i < ep->b_no ; i++ ){
		if( ep->bound[i]->br.no != bp->br )	continue;

		bno = ep_bound_point_br( ep, bp->br, &bp->ctr, &dis );

		if( bno >= 0 ){
			b = ep->bound[bno];
			bgp->p = b->bgp;
			b->bgp = bgp;
			return;
		}
	}


	ALLOC_AEP_DATA( b, bound_type, aep );
	b->Mbgp = b->bgp = NULL;
	b->type = BOUND_ALL;
	b->Fcorner = 0x0f;
	b->Frange = 0;
	b->range = NULL;


	bgp->p = b->bgp;
	b->bgp = bgp;


	b->br.no = bp->br;
	b->br.flag = 0;
	b->br.bno = 0;
	b->br.depth = bp->depth;


	for( j = ep->b_no ; j > 0 ; j-- ){

		if( b->br.no == ep->bound[j-1]->br.no ||
			b->br.depth > ep->bound[j-1]->br.depth )
				break;

		ep->bound[j] =  ep->bound[j-1];
		for( r = ep->bound[j]->range ; r != NULL ; r = r->p )
			r->bno = j;

		for( al = ep->al ; al != NULL ; al = al->nal ){
			if( al->bno[F_RIGHT] == j-1 )
				al->bno[F_RIGHT] = j;
			if( al->bno[F_LEFT] == j-1 )
				al->bno[F_LEFT] = j;
		}
	}

	ep->bound[j] = b;
	ep->b_no++;

	depth_ep_cont_init( ep );
}
