/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/

/**************************
 ***   DepthBound.c  	***
 **************************/

#include <stdio.h>
#include <math.h>

#include	"Uigp/igp.h"


#include        "GpType/GpType.h"
#include        "EpType/EpType.h"

#include        "Ep/Ep.h"
#include        "Ep/EpIn.h"


#include        "DepthEpIn.h"

#define         MAX_RANGE       32

	
static int	ep_range_al( ep_type *ep, range_type *r, int rno );
static void	ep_bound_al_range( ep_type *ep, range_type *r, int bno );

void
depth_ep_bound( ep_type *ep, ep_array *aep )
{
int	Frange[ MAX_RANGE],	BFrnage;
int	Fal[MAX_RANGE],	flag_al;
int	Or_bound;
bound_type	*b;
range_type	*r,	*nr;
vec2d	rp;
int	i,	j,	mi,	bno;
float	w;

	ep->b_no = 0;


	if( ep->r_no == 0 ){
		if( ep->b_no > 0 )	return;

		ep->b_no = 1;
		ALLOC_AEP_DATA( ep->bound[0], bound_type, aep );
		b = ep->bound[0];
		b->bgp = NULL;
		b->Mbgp = NULL;
		b->range = NULL;
		b->type = BOUND_ALL | BOUND_UNSET;
		b->Fcorner = 0x0f;


                b->br.depth = 1.0;
                b->br.no = 255;
                b->br.flag = 0;
                b->br.bno = 0;
                b->bgp = NULL;


		return;
	}



        for( r = ep->range, i = 0 ; r != NULL ; r = r->nr, i++ ){
                Frange[i] = (1<<i);
		Fal[i] = ep_range_al( ep, r, i );
        }


        for( r = ep->range, i = 0 ; r != NULL ; r = r->nr, i++ ){

		ep_range_set_rp( ep, r, &rp, aep->hbgp );
		flag_al = Fal[i];

        	for( nr = ep->range, j = 0 ; j < ep->r_no ;  nr = nr->nr, j++ ){

			if( r->br.no != nr->br.no ) continue;

			if( flag_al & Fal[j] )	continue;

			if( ep_range_is_point_in( ep, j, &rp ) ){
				Frange[j] |= ( 1<< i );
				flag_al |= Fal[j];
			}
		}

	}



	Or_bound = 0;
        for( r = ep->range, i = 0 ; r != NULL ; r = r->nr, i++ ){

		mi = 1<< i;
		if( Or_bound & mi )	continue;

		ALLOC_AEP_DATA( b, bound_type, aep ); 
		b->bgp = NULL;
		b->Mbgp = NULL;

		BFrnage = Frange[i];
        	for( j = i+1 ; j < ep->r_no ; j++ )
			if( Frange[j] & mi )
				BFrnage &= Frange[j];

		BFrnage &= (~Or_bound);
		Or_bound |= BFrnage;

		b->type = BOUND_CROSS;


                b->br.depth = w = 0.0;
                b->br.bno = 0;
                b->br.flag = 0;
                b->range = NULL;
                 
                for( nr = ep->range, j = 0 ; nr != NULL ; nr = nr->nr, j++ ){
                        if( !( BFrnage &(1<<j) ) )    continue;

                        nr->p = b->range;
                        b->range = nr;
 
 
                        /* set DEPTH_TYPE */
                        b->br.depth += nr->br.depth;
                        b->br.no = nr->br.no;
                        w++;
 
 
                        if( b->br.flag == 0 )
                                b->br.flag = nr->br.flag;
                }
                b->br.depth /= w;
 
 
 
                for( j = ep->b_no ; j > 0 ; j-- ){
 
                        if( b->br.no == ep->bound[j-1]->br.no ||
                                        b->br.depth > ep->bound[j-1]->br.depth )                                break;
 
                        ep->bound[j] =  ep->bound[j-1];
                }
 
                ep->bound[j] = b;
                ep->b_no++;

	}




	for( bno = 0 ; bno < ep->b_no ; bno++ ){
		b = ep->bound[bno];
		for( r = b->range ; r != NULL ; r = r->p ){
                        r->bno = bno;
			ep_bound_al_range( ep, r, bno );
		}
	}
}


static int
ep_range_al( ep_type *ep, range_type *r, int rno )
{
rlink_type      *rl;
al_type *al;
int     i,      m,	Fal;

        Fal = 0;

        for( i = 0 ; i < r->rl_no ; i++ ){
                rl = &r->rl[i];

                for( al = ep->al, m = 1 ; al != NULL ; al = al->nal, m <<= 1 ){
                        if( al != rl->al )      continue;

                        al->rno[rl->side] = rno;
                        Fal |= m;

			/* NEW 4-2-97 */
			if( al->link->Fbr & (1<<rl->side) )
				r->br.flag = 1;
			/* END 4-2-97 */

                        break;
                }
        }

	return( Fal );
}


static void
ep_bound_al_range( ep_type *ep, range_type *r, int bno )
{
rlink_type      *rl;
al_type *al;
int     i;
 
        for( i = 0 ; i < r->rl_no ; i++ ){
                rl = &r->rl[i];
                al = rl->al;

                al->bno[rl->side] = bno;
        }
}




void
depth_ep_bound_set_fcorner( ep_type *ep, float hbgp )
{
int     br,	bno,    i;
bound_type	*b;
vec2d	v;
float	dis;

	if( ep->b_no == 1 && ep->bound[0]->type == BOUND_ALL ){
		ep->bound[0]->Fcorner = 0x0f;
		return;
	}

	br = -1;

	for( i = 0 ; i < ep->b_no ; i++ )
		ep->bound[i]->Fcorner = 0;

	for( i = 0 ; i < ep->b_no ; i++ ){
		b = ep->bound[i];
		if( b->br.no == br )	continue;

		br = b->br.no;

		if( b->type == BOUND_ALL ){
			b->Fcorner = 0x0f;
			continue;
		}



		v.x =  - hbgp;
		v.y =  - hbgp;
		bno = ep_bound_point_br( ep, br, &v, &dis );
		if( bno != -1 && dis > 0.01 )
			ep->bound[bno]->Fcorner |= 0x01;

		v.x =  - hbgp;
		v.y =  + hbgp;
		bno = ep_bound_point_br( ep, br, &v, &dis );
		if( bno != -1 && dis > 0.01 )
			ep->bound[bno]->Fcorner |= 0x02;


		v.x =  + hbgp;
		v.y =  - hbgp;
		bno = ep_bound_point_br( ep, br, &v, &dis );
		if( bno != -1 && dis > 0.01 )
			ep->bound[bno]->Fcorner |= 0x04;


		v.x =  + hbgp;
		v.y =  + hbgp;
		bno = ep_bound_point_br( ep, br, &v, &dis );
		if( bno != -1 && dis > 0.01 )
			ep->bound[bno]->Fcorner |= 0x08;


		br = b->br.no;
	}
}
