/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/

/**************************
 ***   BgWieght.c  	***
 **************************/

#include <stdio.h>
#include <math.h>

#include	"Uigp/igp.h"


#include        "EpType/EpType.h"

#include        "DepthEpIn.h"


void	depth_ep_cont_bound_corenr( ep_type *ep, ep_type *nep, int cno );


static void	ep_count_al( ep_type *ep, int n );



void
depth_ep_cont_init( ep_type *ep )
{
bound_type	*b;
int	i,	n;

	for( i = 0 ; i < ep->b_no ; i++ ){

		b = ep->bound[i]; 

		for( n = 0 ; n < 9 ; n++ )
			b->Fcont[n] = 0;

		b->Fcont[4] |= ( 1<<i);
	}
}



void
depth_ep_cont( ep_type *ep )
{
	depth_ep_cont_bound_corenr( ep, ep->ng[5], 5 );
	depth_ep_cont_bound_corenr( ep, ep->ng[6], 6 );
	depth_ep_cont_bound_corenr( ep, ep->ng[7], 7 );
	depth_ep_cont_bound_corenr( ep, ep->ng[8], 8 );


	ep_count_al( ep, 5 );
	ep_count_al( ep, 6 );
	ep_count_al( ep, 7 );
	ep_count_al( ep, 8 );
}





static	const int Mcc[9][4] = {     {	 0x1, 0x8,  0,   0   },	/* 0 */
			      {	 0x1, 0x4,  0x2, 0x8 },	/* 1 */
			      {	 0x2, 0x4,  0,   0   },	/* 2 */
			      {	 0x1, 0x2,  0x4, 0x8 },	/* 3 */
			      {	 0,   0,    0,   0   },	/* 4 */
			      {	 0x2, 0x1,  0x8, 0x4 },	/* 5 */
			      {	 0x4, 0x2,  0,   0   },	/* 6 */
			      {	 0x4, 0x1,  0x8, 0x2 },	/* 7 */
			      {	 0x8, 0x1,  0,   0   } };/* 8 */



void
depth_ep_cont_bound_corenr( ep_type *ep, ep_type *nep, int cno )
{
int	*mcc = (int *)Mcc[cno],	mask,	cmask;
bound_type *b;
int	i,	j;

	if( nep == NULL )	return;
	if( nep->b_no == 0 )	return;

	mask = mcc[0] | mcc[2];
	

	for( i = 0 ; i < ep->b_no ; i++ ){
		b = ep->bound[i];

		if( !(b->Fcorner & mask) )	continue;

		cmask = 0;
		if( b->Fcorner & mcc[0] )	cmask |= mcc[1];
		if( b->Fcorner & mcc[2] )	cmask |= mcc[3];

		for( j = 0 ; j < nep->b_no ; j++ ){

			if( b->br.no != nep->bound[j]->br.no )  continue;

			if( nep->bound[j]->Fcorner & cmask ){
				b->Fcont[cno] |= (1<<j);
				nep->bound[j]->Fcont[8-cno] |= (1<<i);
			}
		}
	}
}
				

static const int	NM[9] = { 0x01,
			0x01 | 0x2,
			0x2,
			0x1 | 0x4,
			0,
			0x2 | 0x8,
			0x4,
			0x4 | 0x8,
			0x8 };


static void
ep_count_al( ep_type *ep, int n )
{
ep_type *nep;
al_type	*al,	*nal;
int	bno,	nbno;
int	nflag = NM[n];

	if( (nep = ep->ng[n]) == NULL )	return;
	if( nep->b_no == 0 )	return;

	for( al = ep->al ; al != NULL ; al = al->nal ){
		if( ( al->flag & nflag ) == 0 )	continue;

		for( nal = nep->al ; nal != NULL ; nal = nal->nal ){
			if( al->link != nal->link )	continue;

			bno = al->bno[F_RIGHT];
			nbno = nal->bno[F_RIGHT];
			ep->bound[bno]->Fcont[n] |= (1<<nbno);
			nep->bound[nbno]->Fcont[8-n] |= (1<<bno);

			bno = al->bno[F_LEFT];
			nbno = nal->bno[F_LEFT];
			ep->bound[bno]->Fcont[n] |= (1<<nbno);
			nep->bound[nbno]->Fcont[8-n] |= (1<<bno);

			break;
		}
	}
}
