/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/

/**********************************
 ***	DepthCompleteNull.c  	***
 *********************************/

#include	<stdio.h>
#include	<math.h>

#include	"Uigp/igp.h"


#include        "GpType/GpType.h"
#include        "EpType/EpType.h"

#include	"DepthEpIn.h"

static void	depth_ep_complete_null_one( ep_type *ep, ep_array *aep );


static void	complete_null_find( ep_type *ep, int cno,
					int Fbr_no, bound_type **bound );


void
depth_ep_complete_null( ep_array *aep )
{
ep_type	*ep;
int	i,	j;

	for( i = LSIZE ; i < aep->Ni-LSIZE ; i++ )
		for( j = LSIZE ; j < aep->Nj-LSIZE ; j++ ){
			ep = &aep->ep[i][j];

			depth_ep_complete_null_one( ep, aep );
		}
}



static void
depth_ep_complete_null_one( ep_type *ep, ep_array *aep )
{
bound_type	*b,	*bound;
range_type	*r;
al_type	*al;
int	i,	n,	Fbr_no;
int	bno,	br,	Fbr;
float	depth;

	Fbr_no = 0;
	if( ep->bound[0]->br.no != 255 )
		for( i = 0, Fbr_no = 0 ; i < ep->b_no ; i++ )
			Fbr_no |= ( 1<< ep->bound[i]->br.no );



	bound = NULL;

	for( n = 0 ; n < 9 ; n++ ){
		if( n == 4 )	continue;
		complete_null_find( ep, n, Fbr_no, &bound );
	}


	if( bound == NULL )	return;

	for( bno = 0 ; bno < ep->b_no ; ){
		b = ep->bound[bno];
		br = b->br.no;
		Fbr = b->br.flag;
		depth = b->br.depth;

		for( i = bno+1 ; i < ep->b_no ; i++ ){
			b = ep->bound[i];
			if( b->br.no != br )	break;
			if( b->br.flag == 1 )	Fbr = 1;
			depth += b->br.depth;
		}

		depth /= ( i - bno );

		if( depth > bound->br.depth )	break;

		if( Fbr == 0 )	return;
		bno = i;
	}


	ALLOC_AEP_DATA( b, bound_type, aep );
	b->type = BOUND_ALL | BOUND_UNSET;
	b->Mbgp = b->bgp = NULL;

	b->range = NULL;
	b->br.depth = bound->br.depth;
	b->br.no = bound->br.no;
	b->br.flag = 1;
	b->br.bno = 2;

	b->Fcorner = 0xf;


        for( i = ep->b_no ; i > bno ; i-- )
                ep->bound[i] =  ep->bound[i-1];

	for( al = ep->al ; al != NULL ; al = al->nal ){
		if( al->bno[F_RIGHT] >= bno )	al->bno[F_RIGHT]++;
		if( al->bno[F_LEFT] >= bno )	al->bno[F_LEFT]++;
	}

	for(  r = ep->range ; r != NULL ; r = r->nr )
		if( r->bno >= bno )	r->bno++;


	ep->bound[bno] = b;


	ep->b_no++;


	depth_ep_cont_init( ep );
}





/*
static int      NM[9] = { 0x01,
                        0x01 | 0x2,
                        0x2,
                        0x1 | 0x4,
                        0,
                        0x2 | 0x8,
                        0x4,
                        0x4 | 0x8,
                        0x8 };
*/
 
static const int      NM_I[9] = { 0x08,
                        0x4 | 0x8,
                        0x4,
                        0x2 | 0x8,
                        0,
                        0x1 | 0x4,
                        0x2,
                        0x01 | 0x2,
                        0x1 };


static const int      Fmc[9] = {      8,
                                12,
                                4, 
                                10,
                                0, 
                                5,
                                2,
                                3,
                                1 };





static void
complete_null_find( ep_type *ep, int cno, int Fbr_no, bound_type **bound )
{
ep_type		*nep;
bound_type	*b;
al_type	*al;
int	bno,	fcorner;

	nep = ep->ng[cno];
	if( nep == NULL )	return;

	fcorner = Fmc[cno];

	for( al = nep->al ; al != NULL ; al = al->nal ){
/*
		if( al->link->Fbr == 0 )	continue;
*/
		if( !(al->link->Fbr & 0x03 ) )	continue;

		if( ( al->flag & NM_I[cno] ) == 0 ) continue;

		if( Fbr_no & (1<< al->link->br) )	continue;

		if( al->link->Fbr == 1 )
			bno = al->bno[F_RIGHT];
		else	bno = al->bno[F_LEFT];

		b = nep->bound[bno];

		if( (b->Fcorner & fcorner) == 0 )       continue;

		if( b->br.depth > ep->bound[0]->br.depth )
			continue;


		if( *bound != NULL && (*bound)->br.depth < b->br.depth )
			continue;

		*bound = b;
	}
}
