/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/

/**********************************
 ***	RenderBgComplete.c  	***
 *********************************/

#include	<stdio.h>
#include	<math.h>

#include	"Uigp/igp.h"


#include        "GpType/GpType.h"
#include        "EpType/EpType.h"

#include	"DepthEpIn.h"

static void	render_ep_complete_intrnal_one( ep_type *ep, 
					int step, ep_array *aep );

static void	render_ep_br_cont( ep_type *ep, int cno, int Fbr_no, int step, float tbgp,
                                        bound_type **bound );



void
depth_ep_complete_intrnal( ep_array *aep, int step )
{
ep_type	*ep;
int	i,	j;



	for( i = LSIZE ; i < aep->Ni-LSIZE ; i++ )
		for( j = LSIZE ; j < aep->Nj-LSIZE ; j++ ){
			ep = &aep->ep[i][j];

			render_ep_complete_intrnal_one( ep, step, aep );
		}
}



static void
render_ep_complete_intrnal_one( ep_type *ep, int step, ep_array *aep )
{
bound_type	*b,	*bound;
range_type	*r;
al_type	*al;
int	i,	Fbr_no;
float	tbgp = aep->bgp*1.5;

	Fbr_no = 0;
	if( ep->bound[0]->br.no != 255 )
		for( i = 0, Fbr_no = 0 ; i < ep->b_no ; i++ )
			Fbr_no |= ( 1<< ep->bound[i]->br.no );



	bound = NULL;

	render_ep_br_cont( ep, 1, Fbr_no, 2, tbgp, &bound );
	render_ep_br_cont( ep, 3, Fbr_no, 2, tbgp, &bound );
	render_ep_br_cont( ep, 5, Fbr_no, 2, tbgp, &bound );
	render_ep_br_cont( ep, 7, Fbr_no, 2, tbgp, &bound );


	if( bound == NULL )	return;


	if( ep->b_no == 1 && ep->bound[0]->br.no == 255 ){
		b = ep->bound[0];
 
		b->type = BOUND_ALL | BOUND_UNSET;
		b->Mbgp = b->bgp = NULL;
/*
		b->Frange = 0;
*/
		b->range = NULL;
		b->br.depth = bound->br.depth;
		b->br.no = bound->br.no;
		b->br.flag = 0;
		b->br.bno = 2;
 
		b->Fcorner = 0xf;

		return;
	}




	ALLOC_AEP_DATA( b, bound_type, aep );
	b->type = BOUND_ALL | BOUND_UNSET;
	b->Mbgp = b->bgp = NULL;
/*
	b->Frange = 0;
*/
	b->range = NULL;
	b->br.depth = bound->br.depth;
	b->br.no = bound->br.no;
	b->br.flag = 0;
	b->br.bno = 2;

	b->Fcorner = 0xf;


        for( i = ep->b_no ; i > 0 ; i-- ){

                if( b->br.no == ep->bound[i-1]->br.no ||
                                b->br.depth >  ep->bound[i-1]->br.depth )
                        break;

                ep->bound[i] =  ep->bound[i-1];

		for( r = ep->bound[i]->range ; r != NULL ; r = r->p )
			r->bno = i;

		for( al = ep->al ; al != NULL ; al = al->nal ){
			if( al->bno[F_RIGHT] == i-1 )
				al->bno[F_RIGHT] = i;
			if( al->bno[F_LEFT] == i-1 )
				al->bno[F_LEFT] = i;
		}
        }

        ep->bound[i] = b;
        ep->b_no++;

	depth_ep_cont_init( ep );
}





static const int	Fmc[9] = {     	8,
			      	12,
				4,
				10,
				0,
				5,
				2,
				3,
				1 };

static void
render_ep_br_cont( ep_type *ep, int cno, int Fbr_no, int step, float tbgp, 
					bound_type **bound )
{
ep_type		*nep;
bound_type	*b;
int	i,	fcorner;
ebgp_type	*bgp;
vec2d	dv;
float	d,	max = 100,	x,	y;

	nep = ep->ng[cno];
	if( nep == NULL )	return;

	fcorner = Fmc[cno];

	for( i = 0 ; i < nep->b_no ; i++ ){
		b = nep->bound[i];

		if( b->br.flag == 1 )	continue;

		if( (int)b->br.bno >= step )	continue;

		if( (b->Fcorner & fcorner) == 0 )	continue;

		if( b->br.no == 255 )	continue;
		if( Fbr_no & (1<< b->br.no) )	continue;

		dv.x = nep->ctr.x - ep->ctr.x;
		dv.y = nep->ctr.y - ep->ctr.y;

		for( bgp = b->bgp ; bgp != NULL ; bgp = bgp->p ){

			if( (x = bgp->ctr.x + dv.x) < 0 )  x = -x;
			if( (y = bgp->ctr.y + dv.y) < 0 )  y = -y;
			d = MAX( x, y );

			if( d > tbgp )	continue;
			if( *bound != NULL && (*bound)->br.depth < b->br.depth )
				continue;

			max = d;
			*bound = b;
		}
	}
}
