/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/

/**************************
 ***   EpValue.c  	***
 **************************/

#include <stdio.h>
#include <math.h>

#include	"Uigp/igp.h"


#include        "EpType/EpType.h"
#include        "Ep/Ep.h"

#include        "EpValue.h"


static void	ep_bgp_value( ebgp_type *bp, int sp, vec2d *v, float tf,
				float *val, float *unity );


#define	WDT	 0.01F

static	float	H,	Hinv;



void
ep_value_init( int bgp )
{
	H = bgp + WDT;
	Hinv = 1.0F / H;
}




float
aep_value( ep_array *aep, vec2d *v, int sp )
{
int     i,      j;
ep_type	*ep;
vec2d	p;

int	bno;
float	val,	dis;
 
        i = ( v->x +LSIZE*aep->bgp) * aep->bgp_inv;
        j = ( v->y +LSIZE*aep->bgp) * aep->bgp_inv;
 
        i = PUSH_TO_RANGE( i, 1, aep->Ni-2 );
        j = PUSH_TO_RANGE( j, 1, aep->Nj-2 );

	ep = &aep->ep[i][j];

	p.x = v->x - ep->ctr.x;
	p.y = v->y - ep->ctr.y;

	bno = ep_bound_point( ep, &p, &dis );
	if( bno == -1 )	return( 0.0 );


	val = ep_bound_value( ep, bno, &p, sp );

	return( val );
}


float
ep_value( ep_type *ep, vec2d *v, int sp )
{
int     bno;
float   dis;
float	val;
 
 
        bno = ep_bound_point( ep, v, &dis );
        if( bno == -1 )	return( 0 );
 
        val = ep_bound_value( ep, bno, v, sp );
 
        return( val );
}


float
ep_bound_value( ep_type *ep, int bno, vec2d *v, int sp )
{
int     i,	n;
ep_type	*nep;
bound_type	*b;
ebgp_type	*bp;
vec2d	ctr;
int	fcont;

float	unity,	val;
 

	unity = val = 0;
	b = ep->bound[bno];


	for( n = 0 ; n < 9 ; n++ ){

		if( (fcont = b->Fcont[n] ) == 0 )	continue;

		nep = ep->ng[n];

		ctr.x = (ep->ctr.x + v->x ) - nep->ctr.x;
		ctr.y = (ep->ctr.y + v->y ) - nep->ctr.y;

               for( i = 0 ; fcont != 0 ; fcont >>= 1, i++ ){
                        if( !(fcont & 0x01) )   continue;

			for( bp = nep->bound[i]->bgp ; bp != NULL ; bp = bp->p )
				ep_bgp_value( bp, sp, &ctr, 1.0, &val,&unity);
		}
	}

	if( unity != 0 )
		return( val / unity );


	for( n = 0 ; n < 9 ; n++ ){

		if( (fcont = b->Fcont[n] ) == 0 )	continue;

		nep = ep->ng[n];
		ctr.x = (ep->ctr.x + v->x ) - nep->ctr.x;
		ctr.y = (ep->ctr.y + v->y ) - nep->ctr.y;


               for( i = 0 ; fcont != 0 ; fcont >>= 1, i++ ){
                        if( !(fcont & 0x01) )   continue;


			for( bp = nep->bound[i]->bgp ; bp != NULL ; bp = bp->p )
				ep_bgp_value( bp, sp, &ctr, 0.666,&val,&unity);
		}

	}


	if( unity != 0 )
		return( val / unity );


	return( 0.0 );
}



static void
ep_bgp_value( ebgp_type *bp, int sp, vec2d *v, float tf,
		float *val, float *unity )
{
float	weight;
float	x,	y,	t;
	

	x = v->x - bp->ctr.x;
	y = v->y - bp->ctr.y;

	if( x < 0 )	x = -x;
	if( y < 0 )	y = -y;

	t = ( x > y )? x : y;

	t *= tf;

	if( t >= H )	return;
	weight = 1 - t*Hinv;

	*unity += weight;
	*val += weight * bp->val[sp];
}
