/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**************************
 ***   EpValueAux.c  	***
 **************************/

#include <stdio.h>
#include <math.h>

#include	"Uigp/igp.h"


#include        "EpType/EpType.h"
#include        "Ep/Ep.h"

#include        "EpValue.h"


static void	ep_bgp_value_aux( bgp_aux_box *baux,
				vec2d *v, float val[SP_NO] );


#define WDT	0.01F
 
static	float	H,	Hinv;
 
 
 
void
ep_value_aux_init( int bgp )
{
	H = bgp + WDT;
	Hinv = 1.0F / H;
} 


void
ep_bg_expand_set_bgp_aux( ep_type *ep, int bno, bgp_aux_box *baux )
{
int     i,	n;
ep_type	*nep;
bound_type	*b;
ebgp_type	*bp;
bgp_aux	*ba;
vec2d	ctr;

	baux->no = 0;
	b = ep->bound[bno];


	for( n = 0 ; n < 9 ; n++ ){

		if( ep->ng[n] == NULL )	continue;

		if( b->Fcont[n] == 0 )	continue;

		nep = ep->ng[n];
		ctr.x = nep->ctr.x - ep->ctr.x;
		ctr.y = nep->ctr.y - ep->ctr.y;

		for( i = 0 ; i < nep->b_no ; i++ ){
			if( !(b->Fcont[n] & (1<<i )) )	continue;

			for( bp = nep->bound[i]->bgp ; bp != NULL ; bp = bp->p){

				ba = &baux->bgp[baux->no++];
				ba->ctr.x = bp->ctr.x + ctr.x;
				ba->ctr.y = bp->ctr.y + ctr.y;

				ba->val[SP_Y] = bp->val[SP_Y];
				ba->val[SP_Q] = bp->val[SP_Q];
				ba->val[SP_A] = bp->val[SP_A];
			}
		}
	}
}




void
ep_bg_expand_set_bound_aux( ep_type *ep, int bno, int bgp, bound_aux *baux )
{
bgp_aux_box baux1;
float	hb;
vec2d	v;

	ep_bg_expand_set_bgp_aux( ep, bno, &baux1 );
	hb = 0.5*bgp;

	VEC2D_SET( v, -hb, -hb );
	ep_bgp_value_aux( &baux1, &v, baux->v0 );


	VEC2D_SET( v, -hb, hb );
	ep_bgp_value_aux( &baux1, &v, baux->v1 );


	VEC2D_SET( v, hb, -hb );
	ep_bgp_value_aux( &baux1, &v, baux->v2 );



	VEC2D_SET( v, hb, hb );
	ep_bgp_value_aux( &baux1, &v, baux->v3 );
}


void
ep_bound_value_aux( bound_aux *b, float x, float y, float val[SP_NO] )
{
float	v0,	v1;

	v0 = y*( b->v1[SP_Y] - b->v0[SP_Y] ) + b->v0[SP_Y];
	v1 = y*( b->v3[SP_Y] - b->v2[SP_Y] ) + b->v2[SP_Y];

	val[SP_Y] = x*( v1 - v0) + v0;


	v0 = y*( b->v1[SP_Q] - b->v0[SP_Q] ) + b->v0[SP_Q];
	v1 = y*( b->v3[SP_Q] - b->v2[SP_Q] ) + b->v2[SP_Q];

	val[SP_Q] = x*( v1 - v0) + v0;


	v0 = y*( b->v1[SP_A] - b->v0[SP_A] ) + b->v0[SP_A];
	v1 = y*( b->v3[SP_A] - b->v2[SP_A] ) + b->v2[SP_A];

	val[SP_A] = x*( v1 - v0) + v0;
}




void
ep_bg_expand_set_bound0_aux( ep_type *ep, int bno, int bgp, bound0_aux *baux )
{
bgp_aux_box baux1;
float	v0[SP_NO],	v1[SP_NO],	v2[SP_NO],	v3[SP_NO];
float	hb,	t;
vec2d	v;

	ep_bg_expand_set_bgp_aux( ep, bno, &baux1 );
	hb = 0.5*bgp;
	t = 1.0/bgp;

	VEC2D_SET( v, -hb, -hb );
	ep_bgp_value_aux( &baux1, &v, v0 );


	VEC2D_SET( v, -hb, hb );
	ep_bgp_value_aux( &baux1, &v, v1 );


	VEC2D_SET( v, hb, -hb );
	ep_bgp_value_aux( &baux1, &v, v2 );



	VEC2D_SET( v, hb, hb );
	ep_bgp_value_aux( &baux1, &v, v3 );



	baux->dy[SP_Y] = t*(v1[SP_Y] - v0[SP_Y] );
	baux->dx[SP_Y] = t*(v2[SP_Y] - v0[SP_Y] );
	baux->dyx[SP_Y] = t*( t*(v3[SP_Y] - v2[SP_Y] ) - baux->dy[SP_Y] );

	baux->dy[SP_Y] += 0.5 *baux->dyx[SP_Y];
	baux->v0[SP_Y] = v0[SP_Y] + 0.5*baux->dx[SP_Y] - 0.5*baux->dy[SP_Y];
	baux->dx[SP_Y] = baux->dx[SP_Y] - 0.5*baux->dyx[SP_Y];


	baux->dy[SP_Q] = t*(v1[SP_Q] - v0[SP_Q] );
	baux->dx[SP_Q] = t*(v2[SP_Q] - v0[SP_Q] );
	baux->dyx[SP_Q] = t*( t*(v3[SP_Q] - v2[SP_Q] ) - baux->dy[SP_Q] );

	baux->dy[SP_Q] += 0.5 *baux->dyx[SP_Q];
	baux->v0[SP_Q] = v0[SP_Q] + 0.5*baux->dx[SP_Q] - 0.5*baux->dy[SP_Q];
	baux->dx[SP_Q] = baux->dx[SP_Q] - 0.5*baux->dyx[SP_Q];


	baux->dy[SP_A] = t*(v1[SP_A] - v0[SP_A] );
	baux->dx[SP_A] = t*(v2[SP_A] - v0[SP_A] );
	baux->dyx[SP_A] = t*( t*(v3[SP_A] - v2[SP_A] ) - baux->dy[SP_A] );

	baux->dy[SP_A] += 0.5 *baux->dyx[SP_A];
	baux->v0[SP_A] = v0[SP_A] + 0.5*baux->dx[SP_A] - 0.5*baux->dy[SP_A];
	baux->dx[SP_A] = baux->dx[SP_A] - 0.5*baux->dyx[SP_A];
}




static void
ep_bgp_value_aux( bgp_aux_box *baux, vec2d *v, float val[SP_NO] )
{
bgp_aux	*bgp;
float	t,	x,	y,	unity,	weight,	tmp;
int     i;

	val[SP_Y] = val[SP_Q] = val[SP_A] = 0;
	unity = 0;

	for( i = 0 ; i < baux->no ; i++ ){
		bgp = &baux->bgp[i];

		x = v->x - bgp->ctr.x;
		y = v->y - bgp->ctr.y;
		if( x < 0 )	x = -x;
		if( y < 0 )	y = -y;

		t = ( x > y )? x : y;

		if( t >= H )	continue;
		weight = 1 - t*Hinv;

		unity += weight;
		val[SP_Y] += weight * bgp->val[SP_Y];
		val[SP_Q] += weight * bgp->val[SP_Q];
		val[SP_A] += weight * bgp->val[SP_A];
	}

	if( unity != 0 ){
		tmp = 1.0/unity;
		val[SP_Y] *= tmp;
		val[SP_Q] *= tmp;
		val[SP_A] *= tmp;

		return;
	}



	for( i = 0 ; i < baux->no ; i++ ){
		bgp = &baux->bgp[i];

		x = v->x - bgp->ctr.x;
		y = v->y - bgp->ctr.y;
		if( x < 0 )	x = -x;
		if( y < 0 )	y = -y;

		t = ( x > y )? x : y;

		t *= 0.6666;

		if( t >= H )	continue;
		weight = 1 - t*Hinv;

		unity += weight;
		val[SP_Y] += weight * bgp->val[SP_Y];
		val[SP_Q] += weight * bgp->val[SP_Q];
		val[SP_A] += weight * bgp->val[SP_A];
	}

	if( unity != 0 ){
		tmp = 1.0/unity;
		val[SP_Y] *= tmp;
		val[SP_Q] *= tmp;
		val[SP_A] *= tmp;
	}
}
