/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**************************
 ***	DepthAexpBgp.c	***
 *************************/

#include        <stdio.h>   
#include	<math.h>

#include        "Expand/ExpIn.h"
#include        "Expand/ExpType.h"

#include        "Zp/ZpType.h"

#include        "Expand/CpType.h"

#include        "DepthExpIn.h"


static void	depth_exp_bgp_put_one1( int i0, int j0, bgp_type *gp_bgp,
				int bgp, cp_array *acp, int br );






void
depth_aexp_bgp( gp_array *agp, zp_array *adp, int br,
				exp_array *aexp, cp_array *acp )
{
gp_type         *gp;
bgp_type	*bgp;
zp_type	*dp;

int     i,      j,      i0,	j0;
mbr_type	mbr;

	mbr = aexp->mbr;

	acp_init( acp, adp->Ni-2, adp->Nj-2 );


	for( i = 1, i0 = 0 ; i < adp->Ni-1 ; i++, i0 += agp->bgp ){
		dp = &adp->dp[i][1];
		gp = &agp->gp[i+adp->Ni0-1][adp->Nj0];

		for( j = 1, j0 = 0 ; j < adp->Nj-1 ;
			j++, j0 += agp->bgp, gp++, dp++ ){

			if( !(dp->Mbr & mbr) )	continue;

			if( !(dp->Mbr_e & mbr) ){
				depth_exp_bgp_put_one1( i, j, gp->bgp,
						agp->bgp, acp, br );
				continue;
			}

			for( bgp = gp->bgp ; bgp != NULL ; bgp = bgp->p ){
				if( bgp->br != br )	continue;
				exp_bgp_put_one( i0, j0, bgp, aexp, i, j,acp);
			}
		}
	}





	i = 0;
	dp = adp->dp[i];
	gp = &agp->gp[i+adp->Ni0-1][adp->Nj0-1];

	for( j = 0 ; j < adp->Nj ; j++, gp++, dp++ ){
		if( !(dp->Mbr & mbr) || (dp->Mbr_e & mbr) )     continue;

		depth_exp_bgp_put_one1( i, j, gp->bgp, agp->bgp,acp, br );
	}


	i = adp->Ni-1;
	dp = adp->dp[i];
	gp = &agp->gp[i+adp->Ni0-1][adp->Nj0-1];

	for( j = 0 ; j < adp->Nj ; j++, gp++, dp++ ){

		if( !(dp->Mbr & mbr) || (dp->Mbr_e & mbr) )     continue;

		depth_exp_bgp_put_one1( i, j, gp->bgp, agp->bgp,acp, br );
	}



	j = 0;
	for( i = 1 ; i < adp->Ni-1 ; i++ ){
		dp = adp->dp[i];
		if( !(dp->Mbr & mbr) || (dp->Mbr_e & mbr) )     continue;

		gp = &agp->gp[i+adp->Ni0-1][adp->Nj0-1];

		depth_exp_bgp_put_one1( i, j, gp->bgp, agp->bgp,acp, br );
	}


	j =  adp->Nj-1;
	for( i = 1  ; i < adp->Ni-1 ; i++ ){
		dp = &adp->dp[i][j];

		if( !(dp->Mbr & mbr) || (dp->Mbr_e & mbr) )     continue;

		gp = &agp->gp[i+adp->Ni0-1][j+adp->Nj0-1];

		depth_exp_bgp_put_one1( i, j, gp->bgp, agp->bgp, acp, br );
	}



	depth_aexp_bgp_end( aexp, acp );
}



static void
depth_exp_bgp_put_one1( int i0, int j0, bgp_type *gp_bgp,
				int bgp, cp_array *acp, int br )
{
int	wi,	wj;
int	w0,	w1,	w2,	w3;
bgp_type	*b;
int   val[SP_NO];
cp_type	*be[4];
float	hbgp = bgp >> 1;



	be[0] = &acp->cp[i0][j0];
	be[1] = be[0]+1;
	be[2] = be[0] + acp->NJ;
	be[3] = be[2]+1;


	for( b = gp_bgp ; b != NULL ; b = b->p ){
//		if( !( mbr & (1L<< b->br)) )     continue;
		if( br != b->br )	continue;

#ifdef QFLOAT
		val[SP_Y] = QCOLOR2F( b->val[SP_Y]) *V_FPF;
		val[SP_Q] = QCOLOR2F( b->val[SP_Q]) *V_FPF;
		val[SP_A] = QCOLOR2F( b->val[SP_A]) *V_FPF;
#endif
 
#ifdef QSHORT
        val[SP_Y] = ((int)b->val[SP_Y]) << ( V_SFP - COLOR_SFP);
        val[SP_Q] = ((int)b->val[SP_Q]) << ( V_SFP - COLOR_SFP);
        val[SP_A] = ((int)b->val[SP_A]) << ( V_SFP - COLOR_SFP);
#endif


		wi = (int)(b->ctr.x+hbgp);
		wj = (int)(b->ctr.y+hbgp);



		if( wi < wj ){
			w0 = bgp-wi;
			w3 = wj;
		}
		else	{
			w0 = bgp-wj;
			w3 = wi;
		}
 
		if( (w1 = bgp - wi) < wj ){
			w1 = wj;
			w2 = wi;
		}
		else	w2 = bgp - wj;



		be[0]->bg_val[SP_Y] += w0 * val[SP_Y];
		be[0]->bg_val[SP_Q] += w0 * val[SP_Q];
		be[0]->bg_val[SP_A] += w0 * val[SP_A];
		be[0]->w += w0;


		be[1]->bg_val[SP_Y] += w1 * val[SP_Y];
		be[1]->bg_val[SP_Q] += w1 * val[SP_Q];
		be[1]->bg_val[SP_A] += w1 * val[SP_A];
		be[1]->w += w1;


		be[2]->bg_val[SP_Y] += w2 * val[SP_Y];
		be[2]->bg_val[SP_Q] += w2 * val[SP_Q];
		be[2]->bg_val[SP_A] += w2 * val[SP_A];
		be[2]->w += w2;

		be[3]->bg_val[SP_Y] += w3 * val[SP_Y];
		be[3]->bg_val[SP_Q] += w3 * val[SP_Q];
		be[3]->bg_val[SP_A] += w3 * val[SP_A];
		be[3]->w += w3;
	}
}
