/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**********************************
 ***	DepthAexpBgpEnd.c	***
 *********************************/

#include        <stdio.h>   
#include	<math.h>

#include        "Expand/ExpType.h"

#include        "DepthExpIn.h"


static void	depth_aexp_bgp_end_one( exp_array *aexp, int i0, int j0,
				int bgp, cp_type *cp );

static int	depth_aexp_bgp_end_fill( exp_array *aexp, cp_array *acp );


void
depth_aexp_bgp_end( exp_array *aexp, cp_array *acp )
{
cp_type	*cp;
int	bgp;
int     i,      j,	algin;


	bgp = aexp->bgp;

	algin = acp->NJ - (acp->Nj - 2);
	cp = &acp->cp[1][1];
	for( i = 0 ; i < aexp->Ni ; i += bgp, cp += algin )
		for( j = 0 ; j < aexp->Nj ; j += bgp, cp++ )
			depth_aexp_bgp_end_one( aexp, i, j, bgp, cp );


	if( depth_aexp_bgp_end_fill( aexp, acp ) == 0 )		return;
	if( depth_aexp_bgp_end_fill( aexp, acp ) == 0 )		return;
	if( depth_aexp_bgp_end_fill( aexp, acp ) == 0 )		return;
	if( depth_aexp_bgp_end_fill( aexp, acp ) == 0 )		return;

/*
	warning( "depth_aexp_bgp_end", "gup" );
*/
}



static void
depth_aexp_bgp_end_one(exp_array *aexp, int i0, int j0, int bgp, cp_type *cp )
{
int	w,	algin;
exp_type	*e,	*ne;
float	t;

	e = &aexp->exp[i0][j0];


        if( i0 < aexp->i0 || i0 >= aexp->i1 ||
                        j0 < aexp->j0 || j0 >= aexp->j1 ){
                e->Fbg = 2;
/*
                cp->w = -2;
*/
		cp->w = 1;      /* NEW */
                return;
        }


	if( e->Fbg != 0 ){
		if( e->Fbg == 2 ){
			cp->w = -2;
			return;
		}
		cp->bg_val[SP_Y]  = EXP_IMAGE_RED( e->bg_val );
		cp->bg_val[SP_Q]  = EXP_IMAGE_GREEN( e->bg_val );
		cp->bg_val[SP_A]  = EXP_IMAGE_BLUE( e->bg_val );
		cp->w = -1;

		return;
	}


	algin = aexp->NJ;

	if( i0 > aexp->i0 )
		for(  w = bgp-1, ne = e - algin ; w > 0 ; w--, ne -= algin ){
			if( ne->Fbg == 0 )	continue;

			if( ne->Fbg == 2 ){
				e->Fbg = 2;
				cp->w = -2;
				return;
			}

			cp->bg_val[SP_Y] += w * EXP_IMAGE_RED( ne->bg_val );
			cp->bg_val[SP_Q] += w * EXP_IMAGE_GREEN( ne->bg_val );
			cp->bg_val[SP_A] += w * EXP_IMAGE_BLUE( ne->bg_val );
			cp->w += w;
			break;
		}


	if( i0 + bgp < aexp->i1 )
		for(  w = bgp-1, ne = e + algin ; w > 0 ; w--, ne += algin ){
			if( ne->Fbg == 0 )	continue;

			if( ne->Fbg == 2 ){
				e->Fbg = 2;
				cp->w = -2;
				return;
			}

			cp->bg_val[SP_Y] += w * EXP_IMAGE_RED( ne->bg_val );
			cp->bg_val[SP_Q] += w * EXP_IMAGE_GREEN( ne->bg_val );
			cp->bg_val[SP_A] += w * EXP_IMAGE_BLUE( ne->bg_val );
			cp->w += w;

			break;
		}


	if( j0 > aexp->j0 )
		for(  w = bgp-1 , ne = e -1 ; w > 0 ; w--, ne-- ){
			if( ne->Fbg == 0 )	continue;

			if( ne->Fbg == 2 ){
				e->Fbg = 2;
				cp->w = -2;
				return;
			}

			cp->bg_val[SP_Y] += w * EXP_IMAGE_RED( ne->bg_val );
			cp->bg_val[SP_Q] += w * EXP_IMAGE_GREEN( ne->bg_val );
			cp->bg_val[SP_A] += w * EXP_IMAGE_BLUE( ne->bg_val );
			cp->w += w;
			break;
		}


	if( j0 + bgp < aexp->j1 )
		for(  w = bgp-1, ne = e +1 ; w > 0 ; w--, ne++ ){
			if( ne->Fbg == 0 )	continue;

			if( ne->Fbg == 2 ){
				e->Fbg = 2;
				cp->w = -2;
				return;
			}

			cp->bg_val[SP_Y] += w * EXP_IMAGE_RED( ne->bg_val );
			cp->bg_val[SP_Q] += w * EXP_IMAGE_GREEN( ne->bg_val );
			cp->bg_val[SP_A] += w * EXP_IMAGE_BLUE( ne->bg_val );
			cp->w += w;

			break;
		}


	if( cp->w == 0 )	return;


	t = 1.0 / cp->w;
	cp->bg_val[SP_Y] *= t;
	cp->bg_val[SP_Q] *= t;
	cp->bg_val[SP_A] *= t;

	e->bg_val = EXP_IMAGE4_RGB( cp->bg_val[SP_Y],
                                                cp->bg_val[SP_Q],
                                                cp->bg_val[SP_A] );

	e->Fbg = 1;

	cp->w = -3;
}



static int
depth_aexp_bgp_end_fill( exp_array *aexp, cp_array *acp )
{
cp_type	*cp,	*cp1;
exp_type	*e,	*e0;
int	bgp,	w,	n;
int     i,      j,	algin,	algin_e;
float	t;


	n = 0;
	bgp = aexp->bgp;

	algin = acp->NJ - (acp->Nj - 2);
	algin_e = aexp->NJ*bgp;
	cp = &acp->cp[1][1];
	e0 = &aexp->exp[0][0];

	for( i = 1 ; i < acp->Ni-1 ; i ++, cp += algin, e0 += algin_e)
		for( j = 1, e = e0 ; j < acp->Nj-1 ; j ++, cp++, e += bgp ){
			if( cp->w != 0 )	continue;

			w = 0;
			cp1 = cp-1;
			if( cp1->w < 0 ){
				if( cp1->w == -2 ){
					cp->w = -2;
					e->Fbg = 2;
					continue;
				}
				cp->bg_val[SP_Y] += cp1->bg_val[SP_Y];
				cp->bg_val[SP_Q] += cp1->bg_val[SP_Q];
				cp->bg_val[SP_A] += cp1->bg_val[SP_A];
				w ++;
			}

			cp1 = cp+1;
			if( cp1->w < 0 ){
				if( cp1->w == -2 ){
					cp->w = -2;
					e->Fbg = 2;
					continue;
				}
				cp->bg_val[SP_Y] += cp1->bg_val[SP_Y];
				cp->bg_val[SP_Q] += cp1->bg_val[SP_Q];
				cp->bg_val[SP_A] += cp1->bg_val[SP_A];
				w ++;
			}


			cp1 = cp - acp->NJ;
			if( cp1->w < 0 ){
				if( cp1->w == -2 ){
					cp->w = -2;
					e->Fbg = 2;
					continue;
				}
				cp->bg_val[SP_Y] += cp1->bg_val[SP_Y];
				cp->bg_val[SP_Q] += cp1->bg_val[SP_Q];
				cp->bg_val[SP_A] += cp1->bg_val[SP_A];
				w ++;
			}


			cp1 = cp + acp->NJ;
			if( cp1->w < 0 ){
				if( cp1->w == -2 ){
					cp->w = -2;
					e->Fbg = 2;
					continue;
				}
				cp->bg_val[SP_Y] += cp1->bg_val[SP_Y];
				cp->bg_val[SP_Q] += cp1->bg_val[SP_Q];
				cp->bg_val[SP_A] += cp1->bg_val[SP_A];
				w ++;
			}

			if( w == 0 )	continue;


			t = 1.0 / w;
			cp->bg_val[SP_Y] *= t;
			cp->bg_val[SP_Q] *= t;
			cp->bg_val[SP_A] *= t;

			e->bg_val = EXP_IMAGE4_RGB( cp->bg_val[SP_Y],
                                                cp->bg_val[SP_Q],
                                                cp->bg_val[SP_A] );

			e->Fbg = 1;

			cp->w = -3;

			n++;
		}

	return( n );
}



/*
void
depth_aexp_bgp_end_update_dp( zp_array *adp, cp_array *acp, int br )
{
zp_type	*dp;
cp_type	*cp,	*cp1;
int	br_mask;
int     i,      j,	algin,	 alginc;

	br_mask = 1L<< br;

	algin = adp->NJ - (adp->Nj - 2);
	alginc = acp->NJ - (acp->Nj - 3);
	dp = &adp->dp[1][1];
	cp = &acp->cp[1][1];
	cp1 = &acp->cp[2][1];
	for( i = 1 ; i < adp->Ni-1 ;
			i++, dp += algin, cp += alginc, cp1 += alginc )
		for( j = 1 ; j < adp->Nj-1 ; j++, dp++, cp++, cp1++ ){

			if( !(dp->Mbr & br_mask) )	continue;
			if( dp->Mbr_e & br_mask )	continue;

			if( cp->w != -3 || cp1->w != -3 ||
				(cp+1)->w != -3 || (cp1+1)->w != -3 )
				dp->Mbr_e |= br_mask;
		}
}
*/