/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/******************************
 ***        DepthExpTr.c	***
 ******************************/

#include        <stdio.h>   
#include		<math.h>

#include		"ImageType/ImageType.h"

#include        "Expand/Expand.h"
#include        "Expand/ExpRidge.h"
#include        "Expand/ExpEdge.h"

#include        "DepthExp.h"

#include        "DepthExpIn.h"
#include        "Zp/ZpType.h"

#include        "ExpEdgeBr.h"






static void	aexp_to_pixel00( image_type *im,
						int R0, int C0, int ni, int nj, int bg_val );

static void 	aexp_to_pixel11( exp_array *aexp, image_type *im,
						int R0, int C0 );

static void	depth_aexp_to_pixel0( exp_array *BAexp[],
						int br[], int br_no, exp_type *se,
						image_type *im,
                        int r0, int c0, int R0, int C0, int ni );

static void	depth_aexp_to_pixel1( exp_array *BAexp[],
						int br[], int br_no, exp_type *se,
						image_type *im, int r0, int c0, int R0, int C0, int ni );

static void	depth_aexp_to_pixel11( exp_array *BAexp[],
						int br[], int br_no, exp_type *se, image_type *im,
						int r0, int c0, int R0, int C0, int ni );

static void	depth_aexp_to_pixel2( exp_array *BAexp[],
						int br[], int br_no, exp_type *se, image_type *im,
						int r0, int c0, int R0, int C0, int ni );

static void	depth_aexp_to_pixel3( exp_array *BAexp[],
						int br[], int br_no, exp_type *se,
						image_type *im, int r0, int c0, int R0, int C0, int ni );

static void	depth_aexp_to_pixel4( exp_array *BAexp[],
						int br[], int br_no, exp_type *se,
						image_type *im, int r0, int c0, int R0, int C0, int ni );

static void	depth_aexp_to_pixel5( exp_array *BAexp[],
						int br[], int br_no, exp_type *se,
						image_type *im, int r0, int c0, int R0, int C0, int ni );

static void	depth_aexp_to_pixel6_9( exp_array *BAexp[],
						int br[], int br_no, exp_type *se,
						image_type *im, int r0, int c0, int R0, int C0, int ni );

typedef void	(*aexp_to_pixel)( exp_array *BAexp[],
                                int br[], int br_no, exp_type *se,
								image_type *im,
                                int r0, int c0, int R0, int C0, int ni );

static const aexp_to_pixel	Aexp_to_pixel[] = {
		depth_aexp_to_pixel0,
		depth_aexp_to_pixel11,
		depth_aexp_to_pixel2,
		depth_aexp_to_pixel3,
		depth_aexp_to_pixel4,
		depth_aexp_to_pixel5,
		depth_aexp_to_pixel6_9,
		depth_aexp_to_pixel6_9,
		depth_aexp_to_pixel6_9,
		depth_aexp_to_pixel6_9,
		depth_aexp_to_pixel6_9,
		depth_aexp_to_pixel6_9,
		depth_aexp_to_pixel6_9,
		depth_aexp_to_pixel6_9,
		depth_aexp_to_pixel6_9,
		depth_aexp_to_pixel6_9,
		depth_aexp_to_pixel6_9 };





void
depth_expand_agp_transparent( gp_array *agp, expAux_type *expA,
				mbr_type Sbr,int color, image_type *im )
{
int	row,	col,	lgp;
int	i,	j;
mbr_type	*sbr;
int	n;

	if( (n = 48 / agp->bgp) < 4 )	n = 4;

	lgp = agp->bgp*n;



	asbr_reset( expA->asbr, agp->row, agp->column, lgp );

	expA->asbr->sno = 0;
	sbr = expA->asbr->sbr;
	expA->Se.bg_val = color;

	for( row = 0, i = 1 ; row < agp->row ; i += n , row += lgp ){
		for( col = 0, j = 1 ; col < agp->column ;
						j += n, col += lgp ){

			xp_agp( agp, i, j, n, n, expA->axp );

			expA->asbr->sno += depth_expand_xp_transparent( agp,
						expA->axp, expA,
						im, row, col,
						Sbr, sbr++, expA->asbr->Fclear  );
		}


		#ifdef VIM_RELINQUISH
				gp_relinquish_cpu();
		#endif
	}

	expA->asbr->Fclear = 1;
}




int
depth_expand_xp_transparent( gp_array *agp, xp_array *axp, expAux_type *expA,
					image_type *im, int R0, int C0,
					mbr_type Sbr, mbr_type *sbr, int Fclear )
{
zp_array	*azp = &expA->azp;
int	i,	j,	r0,	c0,	ni;
zp_type	*dp;


int	Abr[100],	br_no;
box2i	abox[32];




	br_no = azp_create( axp, azp, Abr );


	if( Fclear == 0 )
		if( *sbr == azp->Mbr ){
			if( (Sbr & azp->Mbr ) == azp->Mbr ){
				return( 0 );
			}
		}



	if( br_no == 0 ){

		aexp_to_pixel00( im, R0, C0, axp->Pi, axp->Pj, expA->Se.bg_val );

		*sbr = azp->Mbr;

		return(1);
	}




	azp_box( azp, Abr, br_no, abox );

	for( i = 0 ; i < br_no ; i++ ){
		aexp_init( &expA->aexp[i], axp->Pi, axp->Pj, axp->bgp, &abox[i] );

		expA->aexp[i].mbr = 1L << Abr[i];

		expA->BAexp[ Abr[i] ] = &expA->aexp[i];
	}




	depth_aexp_edge( axp, expA->BAexp, azp->Mbr );


	for( i = 0 ; i < br_no ; i++ ){
		depth_aexp_bgp( agp, azp, Abr[i], &expA->aexp[i], &expA->acp );
		depth_aexp_final( &expA->aexp[i], azp, Abr[i] );
	}


	depth_aexp_ridge( axp, expA->BAexp, azp->Mbr );


	depth_aexp_hunch( axp, expA->BAexp, azp->Mbr );



	if( br_no == 1 &&  azp->Ftransparent == 0  ){
		aexp_to_pixel11( &expA->aexp[0], im, R0, C0 );
		*sbr = azp->Mbr;

		return( 1 );
	}



	if( Fclear == 0 && ( Sbr & *sbr ) != *sbr )
		Fclear = 1;
	ni = axp->bgp;
	for( i = 1, r0 = 0 ; i < azp->Ni-1 ; i++, r0 += ni ){
		dp = &azp->dp[i][1];
		for( j = 1, c0 = 0 ; j < azp->Nj-1 ; j++, dp++, c0 += ni ){

			if( Fclear == 0 && ( Sbr & dp->Mbr ) != dp->Mbr  )
				continue;


			if( dp->br_no == 1  && dp->Mbr_ee == 0  ){
				depth_aexp_to_pixel1( expA->BAexp, dp->br, dp->br_no, &expA->Se,
							im, r0, c0, R0+r0, C0+c0, ni );
				continue;
			}


			(*Aexp_to_pixel[dp->br_no])( expA->BAexp, dp->br, dp->br_no, &expA->Se,
					im, r0, c0, R0+r0, C0+c0, ni );
		}
	}

	*sbr = azp->Mbr;

	return( 1 );
}




static void
depth_aexp_to_pixel1( exp_array *BAexp[], int br[], int br_no, exp_type *se,
		image_type *im,	int r0, int c0, int R0, int C0, int ni )
{
exp_array	*aexp;
int     i,      j,	algin,	nj;

register u_int *y_ptr;

exp_type	*e;


	aexp = BAexp[ br[0] ];

	nj = ni;


	if( C0 + nj > IMAGE_COLUMN(im) )	nj = IMAGE_COLUMN(im) - C0;
	if( R0 + ni > IMAGE_ROW(im) )		ni = IMAGE_ROW(im)-R0;

#ifdef WIN32  
	y_ptr = ((u_int *)im->data) + (im->row-1-R0) * im->column + C0;
	algin = -(im->column + nj);
#else 
	y_ptr = ((u_int *)im->data) + R0 * im->column + C0;
	algin = im->column - nj; 
#endif



	for( i = 0 ; i < ni ; i++, y_ptr += algin ){
		e = &aexp->exp[i+r0][c0];

		for( j = 0 ; j < nj ; j++, e++ ){


			if( e->ubg == 0 ){
				*y_ptr++ = e->bg_val;

				continue;
			}


			*y_ptr++ = e->val;
		}
	}
}



static void
depth_aexp_to_pixel11( exp_array *BAexp[], int br[], int br_no, exp_type *se,
		image_type *im,	int r0, int c0, int R0, int C0, int ni )
{
exp_array	*aexp0;
int     i,      j,	algin,	nj;
int	tmp0,	tmp1,	tmp2;
int	bg_r,	bg_g,	bg_b;

register u_int *y_ptr;

exp_type	*ae[6],	*e,	*eb,	**ep;



	aexp0 = BAexp[ br[0] ];

	ae[1] = se;

	nj = ni;



        if( C0 + nj > IMAGE_COLUMN(im) )        nj = IMAGE_COLUMN(im) - C0;
        if( R0 + ni > IMAGE_ROW(im) )           ni = IMAGE_ROW(im)-R0;


#ifdef WIN32  
	y_ptr = ((u_int *)im->data) + (im->row-1-R0) * im->column + C0;
	algin = -(im->column + nj);
#else 
	y_ptr = ((u_int *)im->data) + R0 * im->column + C0;
	algin = im->column - nj; 
#endif




	for( i = 0 ; i < ni ; i++, y_ptr += algin ){
		ae[0] = &aexp0->exp[i+r0][c0];

                for( j = 0 ; j < nj ;
			j++, ae[0]++ ){

			ep = ae;
			if( (*ep)->Fbg!=1 && (*ep)->Fpr==0 ){
			    ep++;
			}

			e = *ep;

			if( (*ep)->Fpr == 1 ){
			    ep++;
			}

			eb = *ep;

			


			if( e->ubg == 0 ){
				*y_ptr++ = e->bg_val;
				continue;
			}




			if( e->ubg == V_FP ){

				*y_ptr++ = e->val;
				continue;
			}


			/* Transparent */
/*
			if( e->Fpr == 1 ){
				if( e->Fbg == 2 )
					*y_ptr++ = 0xF0000000;
				else	*y_ptr++ = e->val;

				continue;
			}
*/

			/* Transparent */
			if( e->Fpr == 1 && e->Fbg == 2 && eb == se ){
				*y_ptr++ = eb->bg_val;
				continue;
			}


//Grisha 07.11.01
if ( eb->bg_val & 0xF0000000 ) {
	*y_ptr++ = e->val;
	continue;
}
//Grisha 07.11.01

			tmp0 = EXP_IMAGE_BLUE(e->val);
			tmp1 = EXP_IMAGE_GREEN(e->val);
			tmp2 = EXP_IMAGE_RED(e->val);

			bg_r = EXP_IMAGE_RED(eb->bg_val);
			bg_g = EXP_IMAGE_GREEN(eb->bg_val);
			bg_b = EXP_IMAGE_BLUE(eb->bg_val);


 
			tmp0 = (((e->ubg*(tmp0-bg_b))>>V_SFP)+ bg_b) >> V_SFP;

                        tmp1 = (((e->ubg*(tmp1-bg_g))>>V_SFP)+ bg_g) >> V_SFP;

                        tmp2 = (((e->ubg*(tmp2-bg_r))>>V_SFP)+ bg_r) >> V_SFP;

			*y_ptr++ = IMAGE4_RGB( tmp2, tmp1, tmp0 );
		}
	}
}







static void
depth_aexp_to_pixel2( exp_array *BAexp[], int br[], int br_no, exp_type *se,
		image_type *im,	int r0, int c0, int R0, int C0, int ni )
{
exp_array	*aexp0,	*aexp1;
int     i,      j,	algin,	nj;
int	tmp0,	tmp1,	tmp2;
int	bg_r,	bg_g,	bg_b;

register u_int *y_ptr;

exp_type	*ae[6],	*e,	*eb,	**ep;



	aexp0 = BAexp[ br[0] ];
	aexp1 = BAexp[ br[1] ];

	ae[2] = se;

	nj = ni;



        if( C0 + nj > IMAGE_COLUMN(im) )        nj = IMAGE_COLUMN(im) - C0;
        if( R0 + ni > IMAGE_ROW(im) )           ni = IMAGE_ROW(im)-R0;


#ifdef WIN32  
	y_ptr = ((u_int *)im->data) + (im->row-1-R0) * im->column + C0;
	algin = -(im->column + nj);
#else 
	y_ptr = ((u_int *)im->data) + R0 * im->column + C0;
	algin = im->column - nj; 
#endif




	for( i = 0 ; i < ni ; i++, y_ptr += algin ){
		ae[0] = &aexp0->exp[i+r0][c0];
		ae[1] = &aexp1->exp[i+r0][c0];

                for( j = 0 ; j < nj ;
			j++, ae[0]++, ae[1]++ ){

			ep = ae;
			if( (*ep)->Fbg!=1 && (*ep)->Fpr==0 ){
			    ep++;
			    if( (*ep)->Fbg!=1 && (*ep)->Fpr==0 ){
			    	ep++;
			    }
			}

			e = *ep;

			if( (*ep)->Fpr == 1 ){
			    ep++;
				if( (*ep)->Fbg != 1 ){
				    ep++;
				    if( (*ep)->Fbg != 1 ){
					ep++;
				    }
				}
			}

			eb = *ep;

			


			if( e->ubg == 0 ){
				*y_ptr++ = e->bg_val;
				continue;
			}




			if( e->ubg == V_FP ){

				*y_ptr++ = e->val;
				continue;
			}


			/* Transparent */
			if( e->Fpr == 1 && e->Fbg == 2 && eb == se ){
				*y_ptr++ = eb->bg_val;
				continue;
			}



			tmp0 = EXP_IMAGE_BLUE(e->val);
			tmp1 = EXP_IMAGE_GREEN(e->val);
			tmp2 = EXP_IMAGE_RED(e->val);

			bg_r = EXP_IMAGE_RED(eb->bg_val);
			bg_g = EXP_IMAGE_GREEN(eb->bg_val);
			bg_b = EXP_IMAGE_BLUE(eb->bg_val);


 
			tmp0 = (((e->ubg*(tmp0-bg_b))>>V_SFP)+ bg_b) >> V_SFP;

                        tmp1 = (((e->ubg*(tmp1-bg_g))>>V_SFP)+ bg_g) >> V_SFP;

                        tmp2 = (((e->ubg*(tmp2-bg_r))>>V_SFP)+ bg_r) >> V_SFP;

			*y_ptr++ = IMAGE4_RGB( tmp2, tmp1, tmp0 );
		}
	}
}







static void
depth_aexp_to_pixel3( exp_array *BAexp[], int br[], int br_no, exp_type *se,
		image_type *im,	int r0, int c0, int R0, int C0, int ni )
{
exp_array	*aexp0,	*aexp1,	*aexp2;
int     i,      j,	algin,	nj;
int	tmp0,	tmp1,	tmp2;
int	bg_r,	bg_g,	bg_b;

register u_int *y_ptr;

exp_type	*ae[6],	*e,	*eb,	**ep;



	aexp0 = BAexp[ br[0] ];
	aexp1 = BAexp[ br[1] ];
	aexp2 = BAexp[ br[2] ];

	ae[3] = se;

	nj = ni;



        if( C0 + nj > IMAGE_COLUMN(im) )        nj = IMAGE_COLUMN(im) - C0;
        if( R0 + ni > IMAGE_ROW(im) )           ni = IMAGE_ROW(im)-R0;


#ifdef WIN32  
	y_ptr = ((u_int *)im->data) + (im->row-1-R0) * im->column + C0;
	algin = -(im->column + nj);
#else 
	y_ptr = ((u_int *)im->data) + R0 * im->column + C0;
	algin = im->column - nj; 
#endif




	for( i = 0 ; i < ni ; i++, y_ptr += algin ){
		ae[0] = &aexp0->exp[i+r0][c0];
		ae[1] = &aexp1->exp[i+r0][c0];
		ae[2] = &aexp2->exp[i+r0][c0];

                for( j = 0 ; j < nj ;
			j++, ae[0]++, ae[1]++, ae[2]++ ){

			ep = ae;
			if( (*ep)->Fbg!=1 && (*ep)->Fpr==0 ){
			    ep++;
			    if( (*ep)->Fbg!=1 && (*ep)->Fpr==0 ){
			    	ep++;
			    	if( (*ep)->Fbg!=1 && (*ep)->Fpr==0 ){
			    	    ep++;
				}
			    }
			}

			e = *ep;

			if( (*ep)->Fpr == 1 ){
			    ep++;
				if( (*ep)->Fbg != 1 ){
				    ep++;
				    if( (*ep)->Fbg != 1 ){
					ep++;
				    }
				}
			}

			eb = *ep;

			


			if( e->ubg == 0 ){
				*y_ptr++ = e->bg_val;

				continue;
			}




			if( e->ubg == V_FP ){

				*y_ptr++ = e->val;
				continue;
			}


						/* Transparent */
			if( e->Fpr == 1 && e->Fbg == 2 && eb == se ){
				*y_ptr++ = eb->bg_val;
				continue;
			}


			tmp0 = EXP_IMAGE_BLUE(e->val);
			tmp1 = EXP_IMAGE_GREEN(e->val);
			tmp2 = EXP_IMAGE_RED(e->val);

			bg_r = EXP_IMAGE_RED(eb->bg_val);
			bg_g = EXP_IMAGE_GREEN(eb->bg_val);
			bg_b = EXP_IMAGE_BLUE(eb->bg_val);


 
			tmp0 = (((e->ubg*(tmp0-bg_b))>>V_SFP)+ bg_b) >> V_SFP;

			tmp1 = (((e->ubg*(tmp1-bg_g))>>V_SFP)+ bg_g) >> V_SFP;

			tmp2 = (((e->ubg*(tmp2-bg_r))>>V_SFP)+ bg_r) >> V_SFP;


			*y_ptr++ = IMAGE4_RGB( tmp2, tmp1, tmp0 );
		}
	}
}





static void
depth_aexp_to_pixel4( exp_array *BAexp[], int br[], int br_no, exp_type *se,
		image_type *im,	int r0, int c0, int R0, int C0, int ni )
{
exp_array	*aexp0,	*aexp1,	*aexp2,	*aexp3;
int     i,      j,	algin,	nj;
int	tmp0,	tmp1,	tmp2;
int	bg_r,	bg_g,	bg_b;

register u_int *y_ptr;

exp_type	*ae[6],	*e,	*eb,	**ep;



	aexp0 = BAexp[ br[0] ];
	aexp1 = BAexp[ br[1] ];
	aexp2 = BAexp[ br[2] ];
	aexp3 = BAexp[ br[3] ];

	ae[4] = se;

	nj = ni;



	if( C0 + nj > IMAGE_COLUMN(im) )        nj = IMAGE_COLUMN(im) - C0;
        if( R0 + ni > IMAGE_ROW(im) )           ni = IMAGE_ROW(im)-R0;


#ifdef WIN32  
	y_ptr = ((u_int *)im->data) + (im->row-1-R0) * im->column + C0;
	algin = -(im->column + nj);
#else 
	y_ptr = ((u_int *)im->data) + R0 * im->column + C0;
	algin = im->column - nj; 
#endif




	for( i = 0 ; i < ni ; i++, y_ptr += algin ){
		ae[0] = &aexp0->exp[i+r0][c0];
		ae[1] = &aexp1->exp[i+r0][c0];
		ae[2] = &aexp2->exp[i+r0][c0];
		ae[3] = &aexp3->exp[i+r0][c0];

                for( j = 0 ; j < nj ;
			j++, ae[0]++, ae[1]++, ae[2]++, ae[3]++ ){

			ep = ae;
			if( (*ep)->Fbg!=1 && (*ep)->Fpr==0 ){
			    ep++;
			    if( (*ep)->Fbg!=1 && (*ep)->Fpr==0 ){
			    	ep++;
			    	if( (*ep)->Fbg!=1 && (*ep)->Fpr==0 ){
			    	    ep++;
			    	    if( (*ep)->Fbg!=1 && (*ep)->Fpr==0 ){
			    		ep++;
				    }
				}
			    }
			}

			e = *ep;

			if( (*ep)->Fpr == 1 ){
			    ep++;
				if( (*ep)->Fbg != 1 ){
				    ep++;
				    if( (*ep)->Fbg != 1 ){
					ep++;
					if( (*ep)->Fbg != 1 ){
					    ep++;
					}
				    }
				}
			}

			eb = *ep;

			


			if( e->ubg == 0 ){
				*y_ptr++ = e->bg_val;
				continue;
			}




			if( e->ubg == V_FP ){

				*y_ptr++ = e->val;
				continue;
			}


			/* Transparent */
			if( e->Fpr == 1 && e->Fbg == 2 && eb == se ){
				*y_ptr++ = eb->bg_val;
				continue;
			}


			tmp0 = EXP_IMAGE_BLUE(e->val);
			tmp1 = EXP_IMAGE_GREEN(e->val);
			tmp2 = EXP_IMAGE_RED(e->val);


 
			bg_r = EXP_IMAGE_RED(eb->bg_val);
			bg_g = EXP_IMAGE_GREEN(eb->bg_val);
			bg_b = EXP_IMAGE_BLUE(eb->bg_val);


 
			tmp0 = (((e->ubg*(tmp0-bg_b))>>V_SFP)+ bg_b) >> V_SFP;

			tmp1 = (((e->ubg*(tmp1-bg_g))>>V_SFP)+ bg_g) >> V_SFP;

			tmp2 = (((e->ubg*(tmp2-bg_r))>>V_SFP)+ bg_r) >> V_SFP;



			*y_ptr++ = IMAGE4_RGB( tmp2, tmp1, tmp0 );
		}
	}
}




static void
depth_aexp_to_pixel5( exp_array *BAexp[], int br[], int br_no, exp_type *se,
		image_type *im,	int r0, int c0, int R0, int C0, int ni )
{
exp_array	*aexp0,	*aexp1,	*aexp2,	*aexp3,	*aexp4;
int     i,      j,	algin,	nj;
int	tmp0,	tmp1,	tmp2;
int	bg_r,	bg_g,	bg_b;

register u_int *y_ptr;

exp_type	*ae[6],	*e,	*eb,	**ep;



	aexp0 = BAexp[ br[0] ];
	aexp1 = BAexp[ br[1] ];
	aexp2 = BAexp[ br[2] ];
	aexp3 = BAexp[ br[3] ];
	aexp4 = BAexp[ br[4] ];

	ae[5] = se;

	nj = ni;



        if( C0 + nj > IMAGE_COLUMN(im) )        nj = IMAGE_COLUMN(im) - C0;
        if( R0 + ni > IMAGE_ROW(im) )           ni = IMAGE_ROW(im)-R0;


#ifdef WIN32  
	y_ptr = ((u_int *)im->data) + (im->row-1-R0) * im->column + C0;
	algin = -(im->column + nj);
#else 
	y_ptr = ((u_int *)im->data) + R0 * im->column + C0;
	algin = im->column - nj; 
#endif




	for( i = 0 ; i < ni ; i++, y_ptr += algin ){
		ae[0] = &aexp0->exp[i+r0][c0];
		ae[1] = &aexp1->exp[i+r0][c0];
		ae[2] = &aexp2->exp[i+r0][c0];
		ae[3] = &aexp3->exp[i+r0][c0];
		ae[4] = &aexp4->exp[i+r0][c0];

                for( j = 0 ; j < nj ;
			j++, ae[0]++, ae[1]++, ae[2]++, ae[3]++, ae[4]++ ){

			ep = ae;
			if( (*ep)->Fbg!=1 && (*ep)->Fpr==0 ){
			    ep++;
			    if( (*ep)->Fbg!=1 && (*ep)->Fpr==0 ){
			    	ep++;
			    	if( (*ep)->Fbg!=1 && (*ep)->Fpr==0 ){
			    	    ep++;
			    	    if( (*ep)->Fbg!=1 && (*ep)->Fpr==0 ){
			    		ep++;
			    		if( (*ep)->Fbg!=1 && (*ep)->Fpr==0 )
			    		    ep++;
				    }
				}
			    }
			}

			e = *ep;

			if( (*ep)->Fpr == 1 ){
			    ep++;
				if( (*ep)->Fbg != 1 ){
				    ep++;
				    if( (*ep)->Fbg != 1 ){
					ep++;
					if( (*ep)->Fbg != 1 ){
					    ep++;
					    if( (*ep)->Fbg != 1 )
					        ep++;
					}
				    }
				}
			}

			eb = *ep;

			


			if( e->ubg == 0 ){
				*y_ptr++ = e->bg_val;

				continue;
			}




			if( e->ubg == V_FP ){

				*y_ptr++ = e->val;
				continue;
			}


			/* Transparent */
			if( e->Fpr == 1 && e->Fbg == 2 && eb == se ){
				*y_ptr++ = eb->bg_val;
				continue;
			}


			tmp0 = EXP_IMAGE_BLUE(e->val);
			tmp1 = EXP_IMAGE_GREEN(e->val);
			tmp2 = EXP_IMAGE_RED(e->val);


 
			bg_r = EXP_IMAGE_RED(eb->bg_val);
			bg_g = EXP_IMAGE_GREEN(eb->bg_val);
			bg_b = EXP_IMAGE_BLUE(eb->bg_val);


 
			tmp0 = (((e->ubg*(tmp0-bg_b))>>V_SFP)+ bg_b) >> V_SFP;

			tmp1 = (((e->ubg*(tmp1-bg_g))>>V_SFP)+ bg_g) >> V_SFP;

			tmp2 = (((e->ubg*(tmp2-bg_r))>>V_SFP)+ bg_r) >> V_SFP;



			*y_ptr++ = IMAGE4_RGB( tmp2, tmp1, tmp0 );
		}
	}
}



static void
depth_aexp_to_pixel0( exp_array *BAexp[], int br[], int br_no, exp_type *se,
		image_type *im,	int r0, int c0, int R0, int C0, int ni )
{
int     i,      j,	algin,	nj;
register u_int *y_ptr;

	nj = ni;


        if( C0 + nj > IMAGE_COLUMN(im) )        nj = IMAGE_COLUMN(im) - C0;
        if( R0 + ni > IMAGE_ROW(im) )           ni = IMAGE_ROW(im)-R0;

#ifdef WIN32  
	y_ptr = ((u_int *)im->data) + (im->row-1-R0) * im->column + C0;
	algin = -(im->column + nj);
#else 
	y_ptr = ((u_int *)im->data) + R0 * im->column + C0;
	algin = im->column - nj; 
#endif



	for( i = 0 ; i < ni ; i++, y_ptr += algin )
                for( j = 0 ; j < nj ; j++ )
			*y_ptr++ = se->bg_val;
}




static void
aexp_to_pixel00( image_type *im, int R0, int C0, int ni, int nj, int bg_val )
{
register u_int *y_ptr;
int	algin,	i,	j;



        if( C0 + nj > IMAGE_COLUMN(im) )        nj = IMAGE_COLUMN(im) - C0;
        if( R0 + ni > IMAGE_ROW(im) )           ni = IMAGE_ROW(im)-R0;


#ifdef WIN32  
	y_ptr = ((u_int *)im->data) + (im->row-1-R0) * im->column + C0;
	algin = -(im->column + nj);
#else 
	y_ptr = ((u_int *)im->data) + R0 * im->column + C0;
	algin = im->column - nj; 
#endif



	for( i = 0 ; i < ni ; i++, y_ptr += algin )
                for( j = 0 ; j < nj ; j++ )
			*y_ptr++ = bg_val;
}


static void
depth_aexp_to_pixel6_9( exp_array *BAexp[], int br[], int br_no, exp_type *se,
		image_type *im,	int r0, int c0, int R0, int C0, int ni )
{
exp_array	*aexp[MAXBR];
int i, j, algin, nj, k;
int	tmp0,	tmp1,	tmp2;
int	bg_r,	bg_g,	bg_b;

register u_int *y_ptr;

exp_type	*ae[MAXBR+1],	*e,	*eb,	**ep;


	for ( k = 0; k < br_no; k++ )
		aexp[k] = BAexp[ br[k] ];

	ae[br_no] = se;

	nj = ni;



        if( C0 + nj > IMAGE_COLUMN(im) )        nj = IMAGE_COLUMN(im) - C0;
        if( R0 + ni > IMAGE_ROW(im) )           ni = IMAGE_ROW(im)-R0;


#ifdef WIN32  
	y_ptr = ((u_int *)im->data) + (im->row-1-R0) * im->column + C0;
	algin = -(im->column + nj);
#else 
	y_ptr = ((u_int *)im->data) + R0 * im->column + C0;
	algin = im->column - nj; 
#endif


	for( i = 0 ; i < ni ; i++, y_ptr += algin  ){

		for ( k = 0; k < br_no; k++ )
			ae[k] = &aexp[k]->exp[i+r0][c0];

		for( j = 0 ; j < nj ; j++)
		{
			for ( k = 0; j && k < br_no; k++ )
					ae[k]++;

			ep = ae; 
			while ( (*ep)->Fbg!=1 && (*ep)->Fpr==0 )
			    ep++;

			e = *ep;

			if( (*ep)->Fpr == 1 ){
				do 
					ep++;
				while ( (*ep)->Fbg != 1 );
			}

			eb = *ep;

			if( e->ubg == 0 ){
				*y_ptr++ = e->bg_val;

				continue;
			}

			if( e->ubg == V_FP ){

				*y_ptr++ = e->val;
				continue;
			}

			tmp0 = EXP_IMAGE_BLUE(e->val);
			tmp1 = EXP_IMAGE_GREEN(e->val);
			tmp2 = EXP_IMAGE_RED(e->val);
 
			bg_r = EXP_IMAGE_RED(eb->bg_val);
			bg_g = EXP_IMAGE_GREEN(eb->bg_val);
			bg_b = EXP_IMAGE_BLUE(eb->bg_val);
 
			tmp0 = (((e->ubg*(tmp0-bg_b))>>V_SFP)+ bg_b) >> V_SFP;
			tmp1 = (((e->ubg*(tmp1-bg_g))>>V_SFP)+ bg_g) >> V_SFP;
			tmp2 = (((e->ubg*(tmp2-bg_r))>>V_SFP)+ bg_r) >> V_SFP;

			*y_ptr++ = IMAGE4_RGB( tmp2, tmp1, tmp0 );

		}
	}
}



static void
aexp_to_pixel11( exp_array *aexp, image_type *im, int R0, int C0 )
{
int     i,      j;
int	algin,	algin_e,	ni,	nj;
register u_int *y_ptr;

exp_type	*e;


	nj = aexp->Nj-1;
	ni = aexp->Ni-1;

	if( C0 + nj > IMAGE_COLUMN(im) )	nj = IMAGE_COLUMN(im) - C0;
	if( R0 + ni > IMAGE_ROW(im) )		ni = IMAGE_ROW(im)-R0;

#ifdef WIN32
	y_ptr = ((u_int *)im->data) + (im->row-1-R0) * im->column + C0;
	algin = -(im->column + nj);
#else
	y_ptr = ((u_int *)im->data) + R0 * im->column + C0;
	algin = im->column - nj;
#endif


	algin_e = aexp->NJ - nj;



	e = aexp->exp[0];
	for( i = 0 ; i < ni ; i++, y_ptr += algin, e += algin_e ){

                for( j = 0 ; j < nj ; j++, e++ ){


			if( e->ubg == 0 ){
				*y_ptr++ = e->bg_val;
				continue;
			}

 
			*y_ptr++ = e->val;
		}
	}
}
