/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**************************
 ***	ExpEdgeBr.c	***
 **************************/


#include	"Uigp/igp.h"

#include		"GpType/Configuration.h"

#include	"Expand/ExpType.h"

#include	"Expand/ExpEdge.h"

#include	"ExpEdgeBr.h"


static void	expand_edge_prepare_ebg_br( link_type *l, eval_type ebg[2] );


#define		EDGE_UNITY03( k )	EdgeUnity03[k+128]



#ifdef VIM_CONST_EXAPND_TABLE

#include	"EdgeUnity03.tab"
void
expand_edge_br_init()
{
}

#else

static int   EdgeUnity03[256];


void
expand_edge_br_init()
{
int	*Tbg;
int	i;
float	u;

 
	Tbg = &EdgeUnity03[128];
	Tbg[0] = 0.5* V_FPF;
	for( i = 1, u = 0.015625 ; i < 128  ; i++, u+= 0.015625 ){
		if( u >= 1.5 ){
			Tbg[i] = Tbg[-i] = 0;
			continue;
		}
		if( u > 1){
			Tbg[-i] = 0;
			Tbg[i] = (1.0 - (u - 1.0) * 2.0 )* V_FPF;
			continue;
		}
		Tbg[i] = 0.5*( 1.0 + u)* V_FPF;
		Tbg[-i] = 0.5*( 1.0 - u)* V_FPF;
	}



}
#endif


void 
expand_edges_br( vec2d *ctr, link_type *l, exp_array *aexp )
{
box2d	box;
exp_type	*e;
vec2d	V0;
int	i,	j,	k;
int	i0,	i1,	j0,	j1;

int   u0,     uu;
int   t0,       tt;
int     u,      t,      dt;
int     ux,     uy,     len,    c_prb;
int	tfp,	tmp_u,	ubg,	tmp;

eval_type	ebg[2],	*ev;
edge_br_aux	ae;
int     tmp_r,  tmp_g,  tmp_b;
int	bg_r,	bg_g,	bg_b;



	link_box2d( l, &box );


	i0 = ctr->x + box.x0 -2;
	i1 = ctr->x + box.x1 +2.5;

	if( i1 <= aexp->i0 )	return;
	if( i0 >= aexp->i1 )	return;
	if( i0 < aexp->i0 )	i0 = aexp->i0;
	if( i1 > aexp->i1 )	i1 = aexp->i1;


	j0 = ctr->y + box.y0 -2;
	j1 = ctr->y + box.y1 +2.5;

	if( j1 <= aexp->j0 )	return;
	if( j0 >= aexp->j1 )	return;
	if( j0 < aexp->j0 )	j0 = aexp->j0;
	if( j1 > aexp->j1 )	j1 = aexp->j1;



	expand_edge_prepare_ebg_br( l, ebg );

	expand_edge_br_aux( l, &ae );


	V0.x = i0 + 0.5 - ctr->x;
	V0.y = j0 + 0.5 - ctr->y;


	ux = l->u.x * G_FPF;
	uy = l->u.y * G_FPF;
	len = l->len * G_FPF;
	c_prb = l->c_prb * G_FPF;

/*
	u0 = ux * V0.x + uy * V0.y;
	t0 = -uy * V0.x + ux * V0.y;
*/
	u0 = ( l->u.x * V0.x + l->u.y * V0.y) * G_FPF;
	t0 = (-l->u.y * V0.x + l->u.x * V0.y) * G_FPF;


	for( i = i0 ; i < i1 ; i++, u0 += ux, t0 += -uy ){

	    uu = u0;
	    tt = t0;
	    for( j = j0, e = &aexp->exp[i][j0] ; j < j1 ;
				j++, uu += uy, tt += ux, e++ ){


		u = uu;
		t = tt;


		if( t < 0 ){
			if( u < 0 )
				u += t;
			else	u -= t;

			tmp = t*t;
			tmp_u = uu*uu + tmp + (tmp>>1);
			t = 0;
		}
		else	if( (dt = t - len) > 0  ){
				if( u < 0 )
					u -= dt;
				else	u += dt;

				tmp = dt*dt;
				tmp_u = uu*uu + tmp + (tmp>>1);

/*
				t = l->len;
*/
				t = len;
			}
			else	{
					u -= ((c_prb*t *dt ) >> (G_SFP+G_SFP));
					tmp_u = u*u;
				}



		if( tmp_u > e->u )      continue;
 
		e->u = tmp_u;



		tfp = t;



		if( tmp_u <= ( 4L <<16) ){
			ev = ( u >= 0 )? &ebg[0] : &ebg[1];
 
			bg_r = ev->val[SP_Y] + ((t *ev->dval[SP_Y]) >> G_SFP);
			bg_g = ev->val[SP_Q] + ((t *ev->dval[SP_Q]) >> G_SFP);
			bg_b = ev->val[SP_A] + ((t *ev->dval[SP_A]) >> G_SFP);

			e->bg_val = EXP_IMAGE4_RGB( bg_r, bg_g, bg_b );
			e->Fbg = ev->flag;
		}



		k = (u* ( ae.w + t * ae.dw) )>>(G_SFP+G_SFP+ G_SFP-6);
		if( ae.Fbr == 1 )      k = -k;

		if( k <= -64 || k >= 96 )       continue;


		tmp_r = ae.h[SP_Y] + ((t * ae.dh[SP_Y] ) >> G_SFP);
		tmp_g = ae.h[SP_Q] + ((t * ae.dh[SP_Q] ) >> G_SFP);
		tmp_b = ae.h[SP_A] + ((t * ae.dh[SP_A] ) >> G_SFP);

		if( k < 64 ){
//			e->ubg = Tbg[k];
			e->ubg = EDGE_UNITY03(k);
			e->val = EXP_IMAGE4_RGB( tmp_r, tmp_g, tmp_b );
			e->Fpr = 1;
			continue;
		}



		if( e->Fbg == 0 ){
			ev = ( u >= 0 )? &ebg[0] : &ebg[1];

			bg_r = ev->val[SP_Y] + ((t *ev->dval[SP_Y]) >> G_SFP);
			bg_g = ev->val[SP_Q] + ((t *ev->dval[SP_Q]) >> G_SFP);
			bg_b = ev->val[SP_A] + ((t *ev->dval[SP_A]) >> G_SFP);
		}




//		ubg = Tbg[k];
		ubg = EDGE_UNITY03(k);
		tmp_b = bg_b + ((ubg*(tmp_b - bg_b))>>G_SFP);
		tmp_g = bg_g + ((ubg*(tmp_g - bg_g))>>G_SFP);
		tmp_r = bg_r + ((ubg*(tmp_r - bg_r))>>G_SFP);

		e->val = EXP_IMAGE4_RGB( tmp_r, tmp_g, tmp_b );
		

		e->ubg = G_FP;
	    }
	}
}



void
expand_edge_br_aux( link_type *l, edge_br_aux *ae )
{
edge_type      *e,     *ne;
qcolor_type	*val;
int	side;
int	lenFP;
float	len_inv;

        e = l->pr[0].edge;
        ne = l->pr[1].edge;
	len_inv = 1.0 / l->len;


	side = ( l->Fbr == 2) ? F_RIGHT : F_LEFT;

#ifdef  QFLOAT
	lenFP = V_FPF * len_inv;

        ae->h[SP_Y] = e->side[side].val[SP_Y] * V_FPF;
        ae->h[SP_Q] = e->side[side].val[SP_Q] * V_FPF;
        ae->h[SP_A] = e->side[side].val[SP_A] * V_FPF;
#endif

#ifdef  QSHORT
	lenFP = (V_FP>>COLOR_SFP) * len_inv;

	val = e->side[side].val;
        ae->h[SP_Y] = ((int)val[SP_Y]) << ( V_SFP - COLOR_SFP);
        ae->h[SP_Q] = ((int)val[SP_Q]) << ( V_SFP - COLOR_SFP);
        ae->h[SP_A] = ((int)val[SP_A]) << ( V_SFP - COLOR_SFP);
#endif



        ae->dh[SP_Y] = (ne->side[side].val[SP_Y] -
				e->side[side].val[SP_Y] )* lenFP;
        ae->dh[SP_Q] = (ne->side[side].val[SP_Q] -
				e->side[side].val[SP_Q] )* lenFP;
        ae->dh[SP_A] = (ne->side[side].val[SP_A] -
				e->side[side].val[SP_A] )* lenFP;



        ae->w = G_FPF/e->width;
        ae->dw = ( G_FPF/ne->width - ae->w)*len_inv;
	ae->w *= G_FPF;


	ae->len = l->len;

	ae->Fbr = l->Fbr;
}


static void
expand_edge_prepare_ebg_br( link_type *l, eval_type ebg[2] )
{
#ifdef  QFLOAT 
edge_type	*e,	*ne;
float	lenFP;

	e = l->pr[0].edge;
	ne = l->pr[1].edge;

	lenFP = V_FPF / l->len;

	ebg[0].val[SP_Y] = e->side[F_RIGHT].bval[SP_Y] * V_FPF;
	ebg[0].val[SP_Q] = e->side[F_RIGHT].bval[SP_Q] * V_FPF;
	ebg[0].val[SP_A] = e->side[F_RIGHT].bval[SP_A] * V_FPF;

	ebg[0].dval[SP_Y] = (int)(lenFP *( ne->side[F_RIGHT].bval[SP_Y] -
						 e->side[F_RIGHT].bval[SP_Y] ));
	ebg[0].dval[SP_Q] = (int)(lenFP *( ne->side[F_RIGHT].bval[SP_Q] -
						 e->side[F_RIGHT].bval[SP_Q] ));
	ebg[0].dval[SP_A] = (int)(lenFP *( ne->side[F_RIGHT].bval[SP_A] -
						 e->side[F_RIGHT].bval[SP_A] ));


	ebg[1].val[SP_Y] = e->side[F_LEFT].bval[SP_Y] * V_FPF;
	ebg[1].val[SP_Q] = e->side[F_LEFT].bval[SP_Q] * V_FPF;
	ebg[1].val[SP_A] = e->side[F_LEFT].bval[SP_A] * V_FPF;

	ebg[1].dval[SP_Y] = (int)(lenFP *( ne->side[F_LEFT].bval[SP_Y] -
						e->side[F_LEFT].bval[SP_Y] ));
	ebg[1].dval[SP_Q] = (int)(lenFP *( ne->side[F_LEFT].bval[SP_Q] -
						e->side[F_LEFT].bval[SP_Q] ));
	ebg[1].dval[SP_A] = (int)(lenFP *( ne->side[F_LEFT].bval[SP_A] -
						e->side[F_LEFT].bval[SP_A] ));
#endif

#ifdef  QSHORT
edge_type	*e,	*ne;
float	lenFP;
qcolor_type	*bval,	*nbval;

	e = l->pr[0].edge;
	ne = l->pr[1].edge;

	lenFP = (V_FP>>COLOR_SFP) / l->len;

	bval = e->side[F_RIGHT].bval;
	nbval =	ne->side[F_RIGHT].bval;
	ebg[0].val[SP_Y] = ((int)bval[SP_Y]) << ( V_SFP - COLOR_SFP);
	ebg[0].val[SP_Q] = ((int)bval[SP_Q]) << ( V_SFP - COLOR_SFP);
	ebg[0].val[SP_A] = ((int)bval[SP_A]) << ( V_SFP - COLOR_SFP);

	ebg[0].dval[SP_Y] = (int)(lenFP *( nbval[SP_Y] - bval[SP_Y] ));
	ebg[0].dval[SP_Q] = (int)(lenFP *( nbval[SP_Q] - bval[SP_Q] ));
	ebg[0].dval[SP_A] = (int)(lenFP *( nbval[SP_A] - bval[SP_A] ));


	bval = e->side[F_LEFT].bval;
	nbval =	ne->side[F_LEFT].bval;
	ebg[1].val[SP_Y] = ((int)bval[SP_Y]) << ( V_SFP - COLOR_SFP);
	ebg[1].val[SP_Q] = ((int)bval[SP_Q]) << ( V_SFP - COLOR_SFP);
	ebg[1].val[SP_A] = ((int)bval[SP_A]) << ( V_SFP - COLOR_SFP);

	ebg[1].dval[SP_Y] = (int)(lenFP *( nbval[SP_Y] - bval[SP_Y] ));
	ebg[1].dval[SP_Q] = (int)(lenFP *( nbval[SP_Q] - bval[SP_Q] ));
	ebg[1].dval[SP_A] = (int)(lenFP *( nbval[SP_A] - bval[SP_A] ));
#endif



	if( l->Fbr & 0x01 ){
		ebg[0].flag = 2;
		ebg[1].flag = 1;
	}
	else	{
		ebg[0].flag = 1;
		ebg[1].flag = 2;
	}
}



#ifndef PLAYER
void
expand_profile_edge_br( edge_br_aux *ae, float t, float u,
		float val[SP_NO], float *ubg, u_char *Fpr )
{
int	i,	tfp;


	i = u* ( ae->w + t * ae->dw );

	if( ae->Fbr == 1 )	i = -i;

	if( i <= -64 || i >= 96 )	return;

//	*ubg = Tbg[i];
	*ubg = EDGE_UNITY03(i);
	if( i < 64 )	*Fpr = 1;


	tfp = (int)(G_FPF * t );
	val[SP_Y] = ae->h[SP_Y] + t * ae->dh[SP_Y];
	val[SP_Q] = ae->h[SP_Q] + t * ae->dh[SP_Q];
	val[SP_A] = ae->h[SP_A] + t * ae->dh[SP_A];

	val[SP_Y] = ae->h[SP_Y] + ((tfp * ae->dh[SP_Y] ) >> G_SFP);
	val[SP_Q] = ae->h[SP_Q] + ((tfp * ae->dh[SP_Q] ) >> G_SFP);
	val[SP_A] = ae->h[SP_A] + ((tfp * ae->dh[SP_A] ) >> G_SFP);

}
#endif
