/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**************************
 ***	DAexpEdge4.c	***
 *************************/


#include	"Uigp/igp.h"

#include	"GpType/Configuration.h"

#include	"Expand/ExpType.h"
#include	"Expand/ExpEdge.h"

#include	"DepthExpIn.h"


static void	expand_edge_prepare_ebg_br0x4( link_type *l, int Fbr,
			eval_type ebg[2], edge_aux *ae );


#define		EDGE_UNITY04( k )	EdgeUnity04[k+128]
#define		EDGE_PROFILE04( k )	EdgeProfile04[k+128]


#ifdef VIM_CONST_EXAPND_TABLE

#include	"EdgeUnity04.tab"
#include	"EdgeProfile04.tab"

void
expand_edge_br0x4_init()
{
}

#else
static int	EdgeProfile04[256];
static int	EdgeUnity04[256];


void
expand_edge_br0x4_init()
{
int	*Tpr,	*Tbg;
float   t,      u;
int     i,	ti;

	Tpr = &EdgeProfile04[128];
	Tpr[0] = 0;
	for( i = 1, u = 0.015625 ; i < 128 ; i++, u+= 0.015625 ){

		if( u >= 1.0 ){
			Tpr[i] = V_FPF;
			Tpr[-i] = -V_FPF;
			continue;
		}

		t = 1-u;
		ti =  (1 - t*t)*V_FPF;
		Tpr[i] = ti;
		Tpr[-i] = -ti;
	}



	Tbg = &EdgeUnity04[128];
	Tbg[0] = V_FP;
	for( i = 1, u = 0.015625 ; i < 128  ; i++, u+= 0.015625 ){
		if( u < 1){
			Tbg[i] = Tbg[-i] = V_FP;
			continue;
		}
		if( u > 1.5){
			Tbg[i] = Tbg[-i] = 0;
			continue;
		}
		Tbg[i] = Tbg[-i] = (1 - (u - 1.0) *2.0) * V_FPF;
	}

//	gp_write_INTtable_to_file( "D:/Player/EdgeUnity04.tab", "EdgeUnity04", EdgeUnity04, 256 );
//	gp_write_INTtable_to_file( "D:/Player/EdgeProfile04.tab", "EdgeProfile04", EdgeProfile04, 256 );
}
#endif


void 
expand_edge_br0x4( vec2d *ctr, link_type *l, int Fbr, exp_array *aexp )
{
box2d	box;
exp_type	*e;
vec2d	V0;
int	i,	j,	k;
int	i0,	i1,	j0,	j1;

int   u0,     uu;
int   t0,	tt;
int	u,	t,	dt;
int	ux,	uy,	len,	c_prb;

eval_type	ebg[2],	*ev;
edge_aux	ae;
int	tmp_u,	tmp;
int	pr,	val0,	val1,	tfp,	ubg;
int     tmp_r,  tmp_g,  tmp_b;
int	bg_r,	bg_g,	bg_b;


	link_box2d( l, &box );


	i0 = ctr->x + box.x0 -2;
	i1 = ctr->x + box.x1 +2.5;

	if( i1 <= aexp->i0 )    return;
	if( i0 >= aexp->i1 )    return;
	if( i0 < aexp->i0 )     i0 = aexp->i0;
	if( i1 > aexp->i1 )     i1 = aexp->i1;


	j0 = ctr->y + box.y0 -2;
	j1 = ctr->y + box.y1 +2.5;

	if( j1 <= aexp->j0 )    return;
	if( j0 >= aexp->j1 )    return;
	if( j0 < aexp->j0 )     j0 = aexp->j0;
	if( j1 > aexp->j1 )     j1 = aexp->j1;



	expand_edge_aux( l, &ae );

	expand_edge_prepare_ebg_br0x4( l, Fbr, ebg, &ae );


	V0.x = i0 + 0.5 - ctr->x;
	V0.y = j0 + 0.5 - ctr->y;


	len = l->len * G_FPF;
	c_prb = l->c_prb * G_FPF;
	ux = l->u.x * G_FPF;
	uy = l->u.y * G_FPF;


	u0 = ( l->u.x * V0.x + l->u.y * V0.y) * G_FPF;
	t0 = (-l->u.y * V0.x + l->u.x * V0.y) * G_FPF;


	for( i = i0 ; i < i1 ; i++, u0 += ux, t0 += -uy ){

	    uu = u0;
	    tt = t0;
	    for( j = j0, e = &aexp->exp[i][j0] ; j < j1 ;
				j++, uu += uy, tt += ux, e++ ){


		u = uu;
		t = tt;


		if( t < 0 ){
			if( u < 0 )
				u += t;
			else	u -= t;

			tmp = t*t;
			tmp_u = uu*uu + tmp + (tmp>>1);
			t = 0;
		}
		else	if( (dt = t - len) > 0  ){
				if( u < 0 )
					u -= dt;
				else	u += dt;

				tmp = dt*dt;
				tmp_u = uu*uu + tmp + (tmp>>1);

				t = len;
			}
			else	{
					u -= ((c_prb*t *dt) >> (G_SFP+G_SFP));
					tmp_u = u*u;
				}


		if( tmp_u > e->u )	continue;

		e->u = tmp_u;






		if( tmp_u <= ( 4L <<16) ){
			ev = ( u >= 0 )? &ebg[0] : &ebg[1];

			bg_r = ev->val[SP_Y] + ((t *ev->dval[SP_Y]) >> G_SFP);
			bg_g = ev->val[SP_Q] + ((t *ev->dval[SP_Q]) >> G_SFP);
			bg_b = ev->val[SP_A] + ((t *ev->dval[SP_A]) >> G_SFP);

			e->bg_val = EXP_IMAGE4_RGB( bg_r, bg_g, bg_b );
			e->Fbg = ev->flag;
		}





		k = (u* ( ae.w + t * ae.dw) )>>(G_SFP+G_SFP+ G_SFP-6);

 
		if( ABS(k) >= 96 )      continue;
 


		tfp = (int)(t * ae.len_inv);

		if( ABS(k) < 64 ){
//			pr = Tpr[k];
			pr = EDGE_PROFILE04(k);
			val0 = ae.h0[SP_Y] + ((ae.ah0[SP_Y] * pr) >> V_SFP);
			val1 = ae.h1[SP_Y] + ((ae.ah1[SP_Y] * pr) >> V_SFP);
			tmp_r = val0 + ((tfp * (val1 - val0)) >> G_SFP);

			val0 = ae.h0[SP_Q] + ((ae.ah0[SP_Q] * pr) >> V_SFP);
			val1 = ae.h1[SP_Q] + ((ae.ah1[SP_Q] * pr) >> V_SFP);
			tmp_g = val0 + ((tfp * (val1 - val0)) >> G_SFP);

			val0 = ae.h0[SP_A] + ((ae.ah0[SP_A] * pr) >> V_SFP);
			val1 = ae.h1[SP_A] + ((ae.ah1[SP_A] * pr) >> V_SFP);
			tmp_b = val0 + ((tfp * (val1 - val0)) >> G_SFP);

			e->val = EXP_IMAGE4_RGB( tmp_r, tmp_g, tmp_b );

			e->ubg = G_FP;

			if( Fbr == 1 )      k = -k;
			if( k < 0 )	e->Fpr = 1;


			continue;
		}


		if( e->Fbg == 0 ){
			ev = ( u >= 0 )? &ebg[0] : &ebg[1];
			bg_r = ev->val[SP_Y] + ((t *ev->dval[SP_Y]) >> G_SFP);
			bg_g = ev->val[SP_Q] + ((t *ev->dval[SP_Q]) >> G_SFP);
			bg_b = ev->val[SP_A] + ((t *ev->dval[SP_A]) >> G_SFP);
		}


		if( k > 0 ){
			val0 = ae.h0[SP_Y] + ae.ah0[SP_Y];
			val1 = ae.h1[SP_Y] + ae.ah1[SP_Y];
			tmp_r = val0 + ((tfp * (val1 - val0)) >> G_SFP);

			val0 = ae.h0[SP_Q] + ae.ah0[SP_Q];
			val1 = ae.h1[SP_Q] + ae.ah1[SP_Q];
			tmp_g = val0 + ((tfp * (val1 - val0)) >> G_SFP);

			val0 = ae.h0[SP_A] + ae.ah0[SP_A];
			val1 = ae.h1[SP_A] + ae.ah1[SP_A];
			tmp_b = val0 + ((tfp * (val1 - val0)) >> G_SFP);
		}
		else	{
			val0 = ae.h0[SP_Y] - ae.ah0[SP_Y];
			val1 = ae.h1[SP_Y] - ae.ah1[SP_Y];
			tmp_r = val0 + ((tfp * (val1 - val0)) >> G_SFP);

			val0 = ae.h0[SP_Q] - ae.ah0[SP_Q];
			val1 = ae.h1[SP_Q] - ae.ah1[SP_Q];
			tmp_g = val0 + ((tfp * (val1 - val0)) >> G_SFP);

			val0 = ae.h0[SP_A] - ae.ah0[SP_A];
			val1 = ae.h1[SP_A] - ae.ah1[SP_A];
			tmp_b = val0 + ((tfp * (val1 - val0)) >> G_SFP);
		}


//		ubg = Tbg[k];
		ubg = EDGE_UNITY04(k);

		tmp_b = bg_b + ((ubg*(tmp_b - bg_b))>>G_SFP);
		tmp_g = bg_g + ((ubg*(tmp_g - bg_g))>>G_SFP);
		tmp_r = bg_r + ((ubg*(tmp_r - bg_r))>>G_SFP);

		e->val = EXP_IMAGE4_RGB( tmp_r, tmp_g, tmp_b );


		e->ubg = G_FP;

		if( Fbr == 1 )      k = -k;
		if( k < 0 )	e->Fpr = 1;
	    }
	}
}



static void
expand_edge_prepare_ebg_br0x4( link_type *l, int Fbr,
				eval_type ebg[2], edge_aux *ae )
{
edge_type	*e,     *ne;
qcolor_type	*bval,	*nbval;
float lenFP;

	e = l->pr[0].edge;
	ne = l->pr[1].edge;

#ifdef  QFLOAT
	lenFP = V_FPF / l->len;

	ebg[0].val[SP_Y] = e->side[F_RIGHT].bval[SP_Y] * V_FPF;
	ebg[0].val[SP_Q] = e->side[F_RIGHT].bval[SP_Q] * V_FPF;
	ebg[0].val[SP_A] = e->side[F_RIGHT].bval[SP_A] * V_FPF;

	ebg[0].dval[SP_Y] = (int)(lenFP *( ne->side[F_RIGHT].bval[SP_Y] -
					e->side[F_RIGHT].bval[SP_Y] ));
	ebg[0].dval[SP_Q] = (int)(lenFP *( ne->side[F_RIGHT].bval[SP_Q] -
					e->side[F_RIGHT].bval[SP_Q] ));
	ebg[0].dval[SP_A] = (int)(lenFP *( ne->side[F_RIGHT].bval[SP_A] -
					e->side[F_RIGHT].bval[SP_A] ));
 
	 
	ebg[1].val[SP_Y] = e->side[F_LEFT].bval[SP_Y] * V_FPF;
	ebg[1].val[SP_Q] = e->side[F_LEFT].bval[SP_Q] * V_FPF;
	ebg[1].val[SP_A] = e->side[F_LEFT].bval[SP_A] * V_FPF;
 
	ebg[1].dval[SP_Y] = (int)(lenFP *( ne->side[F_LEFT].bval[SP_Y] -
					e->side[F_LEFT].bval[SP_Y] ));
	ebg[1].dval[SP_Q] = (int)(lenFP *( ne->side[F_LEFT].bval[SP_Q] -
					e->side[F_LEFT].bval[SP_Q] ));
	ebg[1].dval[SP_A] = (int)(lenFP *( ne->side[F_LEFT].bval[SP_A] -
					e->side[F_LEFT].bval[SP_A] ));
#endif

#ifdef  QSHORT
	lenFP = (V_FP>>COLOR_SFP) / l->len;

	bval = e->side[F_RIGHT].bval;
	nbval = ne->side[F_RIGHT].bval;

	ebg[0].val[SP_Y] = ((int)bval[SP_Y]) << ( V_SFP - COLOR_SFP);
	ebg[0].val[SP_Q] = ((int)bval[SP_Q]) << ( V_SFP - COLOR_SFP);
	ebg[0].val[SP_A] = ((int)bval[SP_A]) << ( V_SFP - COLOR_SFP);

	ebg[0].dval[SP_Y] = (int)(lenFP *( nbval[SP_Y] - bval[SP_Y] ));
	ebg[0].dval[SP_Q] = (int)(lenFP *( nbval[SP_Q] - bval[SP_Q] ));
	ebg[0].dval[SP_A] = (int)(lenFP *( nbval[SP_A] - bval[SP_A] ));

 
	 
	bval = e->side[F_LEFT].bval;
	nbval = ne->side[F_LEFT].bval;

	ebg[1].val[SP_Y] = ((int)bval[SP_Y]) << ( V_SFP - COLOR_SFP);
	ebg[1].val[SP_Q] = ((int)bval[SP_Q]) << ( V_SFP - COLOR_SFP);
	ebg[1].val[SP_A] = ((int)bval[SP_A]) << ( V_SFP - COLOR_SFP);
 
	ebg[1].dval[SP_Y] = (int)(lenFP *( nbval[SP_Y] - bval[SP_Y] ));
	ebg[1].dval[SP_Q] = (int)(lenFP *( nbval[SP_Q] - bval[SP_Q] ));
	ebg[1].dval[SP_A] = (int)(lenFP *( nbval[SP_A] - bval[SP_A] ));
#endif


	if( Fbr & 0x01 ){
		ebg[0].flag = 2;
		ebg[1].flag = 1;
	}
	else    {
		ebg[0].flag = 1;
		ebg[1].flag = 2;
	}
}
