/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**************************
 ***	AexpFinal.c	***
 *************************/

#include        <stdio.h>   
#include	<math.h>

#include        "ExpType.h"
#include        "ExpIn.h"


static void	aexp_final_col(exp_array *aexp, int i0, int bgp );
static void	aexp_final_row(exp_array *aexp, int j0, int bgp );


static	int	Winv[256];

void
aexp_final_init()
{
int	w;

	for( w = 1 ; w < 256 ; w += 1 )
		Winv[w] = V_FPF / w;
}



void
aexp_final( exp_array *aexp, cp_array *acp )
{
int	bgp;

int     i,      j;


	bgp = aexp->bgp;


	aexp_bgp_end( aexp, acp );
		

	for( j = 0 ; j < aexp->Nj ; j += bgp )
		aexp_final_row( aexp, j, bgp );

	for( i = 0 ; i < aexp->Ni ; i++ )
		aexp_final_col( aexp, i, bgp );
}



static void
aexp_final_row(exp_array *aexp, int j0, int bgp )
{
int	i,	k,	n,	w,	Ni;
exp_type	*e0,	*e1,	*e;
int	t,	val[SP_NO],	val0[SP_NO];

	Ni = aexp->Ni-1;
	for( i = 0 ; i < Ni ; ){

		for( ; i < Ni ; i++ )
			if( aexp->exp[i+1][j0].Fbg == 0 )	break;
		if( i >= Ni )	return;

		for( k = i+2; k <= Ni ; k++ )
			if( aexp->exp[k][j0].Fbg != 0 )        break;

		e0 = &aexp->exp[i][j0];
		if( k > Ni ){
			for( n = i+1 ; n <= Ni ; n++ ){
				e = &aexp->exp[n][j0];
				e->bg_val = e0->bg_val;
				e->Fbg = 1;
			}
			return;
		}

		e1 = &aexp->exp[k][j0];

		w = k - i;
		t = Winv[w];

		val0[SP_Y] = EXP_IMAGE_RED( e0->bg_val);
		val0[SP_Q] = EXP_IMAGE_GREEN( e0->bg_val);
		val0[SP_A] = EXP_IMAGE_BLUE( e0->bg_val);
		val[SP_Y] = EXP_IMAGE_RED( e1->bg_val);
		val[SP_Q] = EXP_IMAGE_GREEN( e1->bg_val);
		val[SP_A] = EXP_IMAGE_BLUE( e1->bg_val);

                val[SP_Y] = (t* (val[SP_Y] - val0[SP_Y] )) >> V_SFP;
                if( val[SP_Y] < 0 )     val[SP_Y]++;
                val[SP_Q] = (t* (val[SP_Q] - val0[SP_Q] )) >> V_SFP;
                if( val[SP_Q] < 0 )     val[SP_Q]++;
                val[SP_A] = (t* (val[SP_A] - val0[SP_A] )) >> V_SFP;
                if( val[SP_A] < 0 )     val[SP_A]++;


		for( n = i+1 ; n < k ; n++ ){
			e = &aexp->exp[n][j0];

			val0[SP_Y] += val[SP_Y];
			val0[SP_Q] += val[SP_Q];
			val0[SP_A] += val[SP_A];

			e->bg_val = EXP_IMAGE4_RGB( val0[SP_Y],
						val0[SP_Q], val0[SP_A] );

			e->Fbg = 1;
		}

		i = k;
	}
}



static void
aexp_final_col(exp_array *aexp, int i0, int bgp )
{
int	w;
exp_type	*e0,	*e1,	*e,	*eend;
int	t,	val[SP_NO],	val0[SP_NO];

	e0 = &aexp->exp[i0][0];
	eend = &aexp->exp[i0][aexp->Nj-1];

	while( e0 != eend ){

		for( ; e0 != eend ; e0++ )
			if( (e0+1)->Fbg == 0 )  break;

		if( e0 == eend || e0+1 == eend )	return;

		for( w = 2, e1 = e0+2; e1 != eend ; w++, e1++ )
			if( e1->Fbg != 0 )        break;

		if( e1->Fbg == 0 ){
			while( (e = e0 + 1) != e1 ){
				e->bg_val = e0->bg_val;
				e->Fbg = 1;
				e0 = e;
			}
			return;
		}

		t = Winv[w];


		val0[SP_Y] = EXP_IMAGE_RED( e0->bg_val);
		val0[SP_Q] = EXP_IMAGE_GREEN( e0->bg_val);
		val0[SP_A] = EXP_IMAGE_BLUE( e0->bg_val);
		val[SP_Y] = EXP_IMAGE_RED( e1->bg_val);
		val[SP_Q] = EXP_IMAGE_GREEN( e1->bg_val);
		val[SP_A] = EXP_IMAGE_BLUE( e1->bg_val);

                val[SP_Y] = (t* (val[SP_Y] - val0[SP_Y] )) >> V_SFP;
                if( val[SP_Y] < 0 )     val[SP_Y]++;
                val[SP_Q] = (t* (val[SP_Q] - val0[SP_Q] )) >> V_SFP;
                if( val[SP_Q] < 0 )     val[SP_Q]++;
                val[SP_A] = (t* (val[SP_A] - val0[SP_A] )) >> V_SFP;
                if( val[SP_A] < 0 )     val[SP_A]++;


		while( (e = e0 + 1) != e1 ){
			val0[SP_Y] += val[SP_Y];
			val0[SP_Q] += val[SP_Q];
			val0[SP_A] += val[SP_A];

			e->bg_val = EXP_IMAGE4_RGB( val0[SP_Y],
						val0[SP_Q], val0[SP_A] );

			e->Fbg = 1;
			e0 = e;
		}

		e0 = e1;
	}
}
