/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**************************
 ***	AexpInit.c	***
 *************************/

#include        <stdio.h>   
#include	<math.h>

#include        "ExpType.h"

#ifdef PALMOS
	#define MALLOC	malloc2
	#define FREE	free2
#else
	#define MALLOC	malloc
	#define FREE	free
#endif

static void	exp_array_alloc( exp_array *aexp, int ni, int nj );



void
aexp_init( exp_array *aexp, int ni, int nj, int bgp, box2i *box )
{
int     i,      j;
exp_type	*e;
int	algin;

	ni += 1;
	nj += 1;


	if( ni > aexp->NI || nj > aexp->NJ ){
		aexp_free( aexp );
		exp_array_alloc( aexp, ni, nj );
	}

	aexp->Ni = ni;
	aexp->Nj = nj;
	aexp->bgp = bgp;

	aexp->mbr = 0;

	if( box == NULL ){
		aexp->i0 = 0;
		aexp->i1 = aexp->Ni;
		aexp->j0 = 0;
		aexp->j1 = aexp->Nj;
	}
	else	{
		aexp->i0 = box->x0;
		aexp->i1 = box->x1+1;
		aexp->j0 = box->y0;
		aexp->j1 = box->y1+1;
	}



	algin = aexp->NJ - ( aexp->j1 - aexp->j0);
	e = &aexp->exp[aexp->i0][aexp->j0];

	for( i = aexp->i0 ; i < aexp->i1 ; i++, e += algin ){
	   

/*
		e = &aexp->exp[i][aexp->j0];
*/


		for( j = aexp->j0 ; j < aexp->j1 ; j++, e++ ){


			e->u = 9L <<  (G_SFP + G_SFP );

			*((int *)&e->Fbg ) = 0;
/*
			e->ubg = 0;
*/
		}
	}
}



static void
exp_array_alloc( exp_array *aexp, int ni, int nj )
{
int	i; 

	aexp->Ni  = aexp->NI = ni;
	aexp->Nj  = aexp->NJ = nj;

	aexp->pixel = aexp->NI * aexp->NI;

	aexp->exp = (exp_type **)MALLOC( aexp->Ni * sizeof( exp_type * ) );
	aexp->exp[0] = (exp_type *)MALLOC( aexp->Ni*aexp->Nj *sizeof(exp_type));

	for( i = 0 ; i < aexp->Ni-1 ; i++ )
		aexp->exp[i+1] = aexp->exp[i] + aexp->Nj;
}



void
aexp_free( exp_array *aexp )
{
	if( aexp->exp != NULL )
	{
		if( aexp->exp[0] != NULL )
			FREE( aexp->exp[0] );

		FREE( aexp->exp );
	}

	aexp->exp = NULL;
 
	aexp->NI = aexp->NJ = 0;
	aexp->Ni = aexp->Nj = 0;
}
