/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**************************
 ***	ExpHunch.c	***
 **************************/
#include	<stdio.h>   
#include	<math.h>

#include	"ExpIn.h"

static const float	H = 0.5;




void
expand_hunchs( vec2d *ctr, hunch_type *h, exp_array *aexp )
{
exp_type	*e;
int	i,	j;
vec2d	v;
float	w2;

float   ax,     ay,     dx,     dy;
int     i0,     j0,     i1,     j1;

int	hx,	hx0;
int	hy,	hy0;
int	ux_x,	ux_y,	uy_x,	uy_y;
int	val[SP_NO],	lm[SP_NO];
int	t,	ubg;
int	tmp_r,	tmp_g,	tmp_b;
int	bg_r,	bg_g,	bg_b;

	ax = ABS( h->g.u.x );
	ay = ABS( h->g.u.y );

	dx = (h->g.width+H) *ax + (h->g.len+2*H) *ay;
	dy = (h->g.width+H) *ay + (h->g.len+2*H) *ax;


	i0 = ctr->x - dx;
	i1 = ctr->x + dx+0.5;

	if( i1 <= aexp->i0 )    return;
	if( i0 >= aexp->i1 )    return;
	if( i0 < aexp->i0 )     i0 = aexp->i0;
	if( i1 > aexp->i1 )     i1 = aexp->i1;



	j0 = ctr->y - dy;
	j1 = ctr->y + dy+0.5;

	if( j1 <= aexp->j0 )    return;
	if( j0 >= aexp->j1 )    return;
	if( j0 < aexp->j0 )     j0 = aexp->j0;
	if( j1 > aexp->j1 )     j1 = aexp->j1;



#ifdef QFLOAT
	val[SP_Y] = h->gval[SP_Y] * V_FPF;
	val[SP_Q] = h->gval[SP_Q] * V_FPF;
	val[SP_A] = h->gval[SP_A] * V_FPF;

	w2 = h->g.width*h->g.width * V_FPF;
	lm[SP_Y] = h->lm1[SP_Y] * w2;
	lm[SP_Q] = h->lm1[SP_Q] * w2;
	lm[SP_A] = h->lm1[SP_A] * w2;
#endif



#ifdef QSHORT
	val[SP_Y] = ((int)h->gval[SP_Y]) << ( V_SFP - COLOR_SFP);
	val[SP_Q] = ((int)h->gval[SP_Q]) << ( V_SFP - COLOR_SFP);
	val[SP_A] = ((int)h->gval[SP_A]) << ( V_SFP - COLOR_SFP);

	w2 = h->g.width*h->g.width * (V_FP>>COLOR_SFP); 
	lm[SP_Y] = h->lm1[SP_Y] * w2;
	lm[SP_Q] = h->lm1[SP_Q] * w2;
	lm[SP_A] = h->lm1[SP_A] * w2;
#endif



	ux_x = (h->g.u.x /h->g.width) * G_FPF;
	ux_y = (h->g.u.y /h->g.width) * G_FPF;

	uy_x = (h->g.u.x /h->g.len) * G_FPF;
	uy_y = (h->g.u.y /h->g.len) * G_FPF;

	v.x = i0 + 0.5 - ctr->x;
	v.y = j0 + 0.5 - ctr->y;


	hx0 =  v.x *ux_x + v.y *ux_y;
	hy0 = -v.x *uy_y + v.y *uy_x;


	for( i = i0 ; i < i1 ; i++, hx0 += ux_x, hy0 -= uy_y ){
		e = &aexp->exp[i][j0];
		for( j = j0, hx = hx0, hy = hy0 ; j < j1 ;
				j++, hx += ux_y, hy += uy_x, e++ ){


			t = hx*hx + hy*hy;


			/* Bg unity */
			if( t > (int)((G_FP+ (G_FP>>1))<< G_SFP) )
				continue;

			/* NEW 20-9-98 */
			if( e->Fpr != 0 )       continue;
			/* END NEW 20-9-98 */



			if( e->u < t )      continue;
			e->u = t;


			t >>= G_SFP;
			if( t < G_FP ){
				tmp_r = val[SP_Y] + (( lm[SP_Y]*t) >>G_SFP);
				tmp_g = val[SP_Q] + (( lm[SP_Q]*t) >>G_SFP);
				tmp_b = val[SP_A] + (( lm[SP_A]*t) >>G_SFP);

				e->val = EXP_IMAGE4_RGB( tmp_r, tmp_g, tmp_b );
				e->ubg = G_FP;

				continue;
			}

			tmp_r = val[SP_Y] + lm[SP_Y];
			tmp_g = val[SP_Q] + lm[SP_Q];
			tmp_b = val[SP_A] + lm[SP_A];


			ubg = 3*G_FP - (t + t);



			bg_r = EXP_IMAGE_RED( e->bg_val);
			bg_g = EXP_IMAGE_GREEN( e->bg_val);
			bg_b = EXP_IMAGE_BLUE( e->bg_val);

			tmp_b = bg_b + ((ubg*(tmp_b - bg_b))>>G_SFP);
			tmp_g = bg_g + ((ubg*(tmp_g - bg_g))>>G_SFP);
			tmp_r = bg_r + ((ubg*(tmp_r - bg_r))>>G_SFP);

			e->val = EXP_IMAGE4_RGB( tmp_r, tmp_g, tmp_b );


			e->ubg = G_FP;
		}
	}
}
	

	



#ifndef	PLAYER

void
expand_hunch( vec2d *ctr, hunch_type *h, vec2d *p,
		float val[SP_NO], float *ubg )
{
float	hy,	hx;
vec2d	v;
float	tubg;
float	t,	ny;
float	w,	w2,	wh2;


	w = h->g.width;


	v.x = p->x - ctr->x;
	v.y = p->y - ctr->y;

	hx = VEC2D_DOT( v, h->g.u );
	if( hx < 0 )	hx = -hx;
	if( hx > w+H )	return;

	hy = -v.x *h->g.u.y + v.y*h->g.u.x;
	if( hy < 0 )   hy = -hy;
	if( hy > h->g.len+2*H )      return;


	ny = hy * h->g.width/h->g.len;
	t = hx*hx + ny*ny;


	/* Bg unity */
	if( t > (wh2 = (w+H)*(w+H)) )	return;
	if( t < (w2 = w*w) )
		tubg = 1;
	else	tubg = ( t-wh2)/(w2-wh2);


	if( t > w2 )	t = w2;
	val[SP_Y] = h->gval[SP_Y] + h->lm1[SP_Y]*t;
	val[SP_Q] = h->gval[SP_Q] + h->lm1[SP_Q]*t;
	val[SP_A] = h->gval[SP_A] + h->lm1[SP_A]*t;

	if( *ubg < tubg )	*ubg = tubg;
}

#endif
