/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/

/**************************
 ***   ExpNf.c		***
 **************************/

#include <stdio.h>
#include <math.h>

#include	"Uigp/igp.h"
#include        "Uprm/prm.h"

#include        "N3Prime/N3PrimeIn.h"

#include        "ExpIn.h"
#include        "ExpRidge.h"
#include        "ExpEdge.h"


void	expand_nf_n3_fill( vec2d *ctr, link_type *l, int side, link_type *el );



int
expand_nf_edge( vec2d *ctr, link_type *link, vec2d *v, float *val, float *ubg )
{
link_type	*l;
float	t,	u;
int	tval[SP_NO],	tubg;
edge_aux	ae;

	*ubg = 0;

	links_distance( ctr, link, NULL, v, &l, &u, &t );

	if( l == NULL )	return( 0 );
	if( ABS(u) > 4.0 )	return(0);


	expand_edge_aux( l, &ae );

	tubg = 0;
	expand_profile_edge( &ae, t*V_FPF, u*V_FPF, tval, &tubg );

	if( tubg == 0 )	return( 0 );

        *ubg = tubg / V_FPF;
        val[SP_R] = tval[SP_R]  / V_FPF;
        val[SP_G] = tval[SP_G]  / V_FPF;
        val[SP_B] = tval[SP_B]  / V_FPF;

	return( 1 );
}





int
expand_nf_n3( vec2d *ctr, link_type *link, vec2d *v, float val[], float *ubg )
{
link_type	*l,	*l0;
float	t,	u,	dt;
vec2d	lctr,	dv;
link_type	el;
edge_aux	ae;
int	side;
int	tval[SP_NO],	tubg;

	*ubg = 0;
	links_distance( ctr, link, NULL, v, &l, &u, &t );

	if( l == NULL )	return(0);
	if( ABS(u) > 5.0 )	return(0);

	for( l0 = link, lctr = *ctr ; l0 != l ; l0 = LINK_NEXT(l0) ){
		lctr.x += l0->v.x;
		lctr.y += l0->v.y;
	}


	side = ( u >= 0 ) ? F_RIGHT : F_LEFT;
	expand_nf_n3_fill( &lctr, l, side, &el );

	dv.x = v->x - el.ctr.x;
	dv.y = v->y - el.ctr.y;


	u = el.u.x * dv.x + el.u.y * dv.y;
	t = -el.u.y * dv.x + el.u.x * dv.y;

	if( t < 0 ){
		if( u > 0 )    u -= t;
		else    u += t;
 
		t = 0;
	}
	else    if( (dt = t - el.len) > 0 ){
 
			if( u > 0 )    u += dt;
			else    u -= dt;
			t = el.len;
		}
		else    u -= el.c_prb*t *(t - el.len );





	expand_edge_aux( &el, &ae );

	free( el.pr[0].edge );
	free( el.pr[1].edge );


	tubg = 0;
	expand_profile_edge( &ae, t*V_FPF, u*V_FPF, tval, &tubg );

	if( tubg == 0 )	return( 0 );

        *ubg = tubg / V_FPF;
        val[SP_R] = tval[SP_R]  / V_FPF;
        val[SP_G] = tval[SP_G]  / V_FPF;
        val[SP_B] = tval[SP_B]  / V_FPF;


	return( 1 );
}


void
expand_nf_n3_fill( vec2d *ctr, link_type *l, int side, link_type *el )
{
width_type	*width;
float	w0,	w1,	t,	h;
vec2d	v1;
float	len_inv;

	width = l->width[F_BEGIN];
	w0 = width->w[side];
	el->ctr.x = ctr->x + w0* width->bes.y;
	el->ctr.y = ctr->y - w0* width->bes.x;


	width = l->width[F_END];
	w1 = width->w[side];
	el->v.x = ctr->x + l->v.x + w1* width->bes.y - el->ctr.x;
	el->v.y = ctr->y + l->v.y - w1* width->bes.x - el->ctr.y;


	el->len = hypot( el->v.x, el->v.y );
	len_inv = 1.0/el->len;
 
	el->u.x =  el->v.y * len_inv;
	el->u.y = -el->v.x * len_inv;
 
 
	t = 0.5*l->len;
	h = l->c_prb * t * (t - l->len) + 0.5*(w1+w0);
	v1.x = ctr->x + t * -l->u.y  +  h * l->u.x;
	v1.y = ctr->y + t *  l->u.x  +  h * l->u.y;
 
	el->a = (v1.x - el->ctr.x )*el->u.x + (v1.y - el->ctr.y )*el->u.y;
	el->c_prb = -4.0 *el->a/( el->len*el->len);


	el->pr[F_BEGIN].edge = n3_prime_profile( l, side, F_BEGIN );
	el->pr[F_END].edge = n3_prime_profile( l, side, F_END );
}
		

		






int
expand_nf_ridge( vec2d *ctr, link_type *link, vec2d *v, float val[], float *ubg)
{
link_type	*l;
float	t,	u;
ridge_aux	ar;
int	tval[SP_NO],	tubg;

	*ubg = 0;

	links_distance( ctr, link, NULL, v, &l, &u, &t );

	if( l == NULL )	return(0);

	if( ABS(u) > 4.0 )	return(0);

	t = PUSH_TO_RANGE( t, 0, l->len );

	expand_ridge_aux( l, &ar );

	tubg = 0;
	expand_profile_ridge( &ar, t*V_FPF, u*V_FPF, tval, &tubg );

	if( tubg == 0 )	return( 0 );

        *ubg = tubg / V_FPF;
        val[SP_R] = tval[SP_R]  / V_FPF;
        val[SP_G] = tval[SP_G]  / V_FPF;
        val[SP_B] = tval[SP_B]  / V_FPF;


	return( 1 );
}


int
expand_nf_hunch( vec2d *ctr, hunch_type *h, vec2d *v, float val[], float *ubg )
{
	*ubg = 0;
	if( h->type == NF_HUNCH )
		expand_hunch( ctr, h, v, val, ubg );
	else	expand_rh( ctr, h, v, val, ubg );

	if( *ubg == 0 )	return( 0 );

	return( 1 );
}

int
expand_nf_rh( vec2d *ctr, hunch_type *h, vec2d *v, float val[], float *ubg )
{
	*ubg = 0;

	expand_rh( ctr, h, v, val, ubg );

	if( *ubg == 0 )	return( 0 );


	return( 1 );
}
