/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**************************
 ***	ExpRidge.c	***
 *************************/


#include	"Uigp/igp.h"
#include	"GpType/Configuration.h"


#include	"ExpType.h"

#include	"ExpRidge.h"

#include	"ExpIn.h"

#define VIM_CONST_EXAPND_TABLE

#define		RIDGE_UNITY( k )	EdgeUnity[k+128]
#define		RIDGE_PROFILE( k )	RidgeProfile[k+128]

#ifdef VIM_CONST_EXAPND_TABLE

#include	"RidgeProfile.tab"
extern int	EdgeUnity[];

void
expand_ridge_init()
{
}

#else
static int    RidgeProfile[256];
extern int    EdgeUnity[];



void
expand_ridge_init()
{
int	*Tpr;
float   u;
int     i;

	Tpr = &RidgeProfile[128];
	Tpr[0] = 0;
	for( i = 1, u = 0.015625 ; i < 127 ; i++, u+= 0.015625 ){
		if( u < 1 )
			Tpr[i] = Tpr[-i] = u*u*V_FPF;
		else	Tpr[i] = Tpr[-i] = V_FPF;
	}

	gp_write_INTtable_to_file( "D:/Player/RidgeProfile.tab", "RidgeProfile", RidgeProfile, 256 );

}

#endif






void 
expand_ridges( vec2d *ctr, link_type *l, exp_array *aexp )
{
box2d	box;
exp_type	*e;
vec2d	V0;
int	i,	j,	Fbes;
int	i0,	i1,	j0,	j1;

int	flag0[2],	flag1[2];
ridge_aux	ar;

int   u0,     uu;
int   t0,       tt;
int     u,      t,      dt;
int     ux,     uy,     len,    c_prb;
int	tmp_u;

int	h,	lm,	pr,	ubg;
int     tmp_r,  tmp_g,  tmp_b;
int	bg_r,	bg_g,	bg_b;
int	k;


	link_box2d( l, &box );
 
	i0 = ctr->x + box.x0 -2;
	i1 = ctr->x + box.x1 +2.5;

	if( i1 <= aexp->i0 )    return;
	if( i0 >= aexp->i1 )    return;
	if( i0 < aexp->i0 )     i0 = aexp->i0;
	if( i1 > aexp->i1 )     i1 = aexp->i1;

 
	j0 = ctr->y + box.y0 -2;
	j1 = ctr->y + box.y1 +2.5;

	if( j1 <= aexp->j0 )    return;
	if( j0 >= aexp->j1 )    return;
	if( j0 < aexp->j0 )     j0 = aexp->j0;
	if( j1 > aexp->j1 )     j1 = aexp->j1;




	Fbes = link_bes( l );
	flag0[F_RIGHT] = ( Fbes & 0x01 )? 1 : 0;
	flag0[F_LEFT] = ( Fbes & 0x02 )? 1 : 0;

	flag1[F_RIGHT] = ( Fbes & 0x04 )? 1 : 0;
	flag1[F_LEFT] = ( Fbes & 0x08 )? 1 : 0;


	expand_ridge_aux( l, &ar );


	V0.x = i0 + 0.5 - ctr->x;
	V0.y = j0 + 0.5 - ctr->y;


	ux = l->u.x * G_FPF;
	uy = l->u.y * G_FPF;
	len = l->len * G_FPF;
	c_prb = l->c_prb * G_FPF;


	u0 = ( l->u.x * V0.x + l->u.y * V0.y) * G_FPF;
	t0 = (-l->u.y * V0.x + l->u.x * V0.y) * G_FPF;


	for( i = i0 ; i < i1 ; i++, u0 += ux, t0 += -uy ){

	    uu = u0;
	    tt = t0;
	    for( j = j0, e = &aexp->exp[i][j0] ; j < j1 ;
					j++, uu += uy, tt += ux, e++ ){


		u = uu;
		t = tt;


		if( t < 0 ){
			if( u < 0 ){
				if( flag0[F_LEFT] == 0 ) continue;
				u += t;
			}
			else	{
				if( flag0[F_RIGHT] == 0 ) continue;
				u -= t;
			}
			t = 0;
		}
		else	if( (dt = t - len) > 0  ){
				if( u < 0 ){
					if( flag1[F_LEFT] == 0 ) continue;
					u -= dt;
				}
				else	{
					if( flag1[F_RIGHT] == 0 ) continue;
					u += dt;
				}

				t = len;
			}
			else	
				u -= ((c_prb*t *dt) >> (G_SFP+G_SFP));


		tmp_u = u*u;
		if( tmp_u > e->u )      continue;


		k = (u* ( ar.w + t * ar.dw) )>>(G_SFP+G_SFP+ G_SFP-6);
		if( k < 0  ) k = -k;
		if( k >= 96 )      continue;

		e->u = tmp_u;
 
		if( k < 64 ){
//			e->ubg = Tbg[k];
			e->ubg = RIDGE_UNITY(k);
//			pr = Tpr[k];
			pr = RIDGE_PROFILE(k);

			h = ar.h[SP_Y] + ((t * ar.dh[SP_Y]) >> G_SFP);
			lm = ar.lm[SP_Y] + ((t * ar.dlm[SP_Y]) >> G_SFP);
			tmp_r = h + ((lm * pr) >> V_SFP);


			h = ar.h[SP_Q] + ((t * ar.dh[SP_Q]) >> G_SFP);
			lm = ar.lm[SP_Q] + ((t * ar.dlm[SP_Q]) >> G_SFP);
			tmp_g = h + ((lm * pr) >> V_SFP);


			h = ar.h[SP_A] + ((t * ar.dh[SP_A]) >> G_SFP);
			lm = ar.lm[SP_A] + ((t* ar.dlm[SP_A]) >> G_SFP);
			tmp_b = h + ((lm * pr) >> V_SFP);

			e->val = EXP_IMAGE4_RGB( tmp_r, tmp_g, tmp_b );

			continue;
		}


		tmp_r = ar.bg[SP_Y] + ((t * ar.dbg[SP_Y]) >> G_SFP);
		tmp_g = ar.bg[SP_Q] + ((t * ar.dbg[SP_Q]) >> G_SFP);
		tmp_b = ar.bg[SP_A] + ((t * ar.dbg[SP_A]) >> G_SFP);

//		ubg = Tbg[k];
		ubg = RIDGE_UNITY(k);


		bg_r = EXP_IMAGE_RED( e->bg_val);
		bg_g = EXP_IMAGE_GREEN( e->bg_val);
		bg_b = EXP_IMAGE_BLUE( e->bg_val);

		tmp_b = bg_b + ((ubg*(tmp_b - bg_b))>>G_SFP);
		tmp_g = bg_g + ((ubg*(tmp_g - bg_g))>>G_SFP);
		tmp_r = bg_r + ((ubg*(tmp_r - bg_r))>>G_SFP);

		e->val = EXP_IMAGE4_RGB( tmp_r, tmp_g, tmp_b );

		e->ubg = G_FP;
	    }
	}
}


void
expand_ridge_aux( link_type *l, ridge_aux *ar )
{
ridge_type	*r,	*nr;
float	w2,	nw2;
float	len_inv;

	r = l->pr[0].ridge;
	nr = l->pr[1].ridge;


	len_inv = 1.0 /l->len;

#ifdef QFLOAT
	ar->h[SP_Y] = r->mval[SP_Y] * V_FPF;
	ar->dh[SP_Y] = ( nr->mval[SP_Y] - r->mval[SP_Y])*len_inv*V_FPF;

	ar->h[SP_Q] = r->mval[SP_Q] * V_FPF;
	ar->dh[SP_Q] = ( nr->mval[SP_Q] - r->mval[SP_Q])*len_inv*V_FPF;

	ar->h[SP_A] = r->mval[SP_A] * V_FPF;
	ar->dh[SP_A] = ( nr->mval[SP_A] - r->mval[SP_A])*len_inv*V_FPF;


	w2 = r->width*r->width * V_FPF;
	nw2 = nr->width*nr->width * V_FPF;

	ar->lm[SP_Y] = r->lm[SP_Y]*w2;
	ar->dlm[SP_Y] = ( nr->lm[SP_Y]*nw2 - ar->lm[SP_Y] )*len_inv;

	ar->lm[SP_Q] = r->lm[SP_Q]*w2;
	ar->dlm[SP_Q] = ( nr->lm[SP_Q]*nw2 - ar->lm[SP_Q] )*len_inv;

	ar->lm[SP_A] = r->lm[SP_A]*w2;
	ar->dlm[SP_A] = ( nr->lm[SP_A]*nw2 - ar->lm[SP_A] )*len_inv;

#endif

#ifdef QSHORT
	ar->h[SP_Y] = ((int)r->mval[SP_Y]) << ( V_SFP - COLOR_SFP);
	ar->dh[SP_Y] = ( nr->mval[SP_Y] - r->mval[SP_Y])*
					len_inv*(V_FP>>COLOR_SFP);

	ar->h[SP_Q] = ((int)r->mval[SP_Q]) << ( V_SFP - COLOR_SFP);
	ar->dh[SP_Q] = ( nr->mval[SP_Q] - r->mval[SP_Q])*
					len_inv*(V_FP>>COLOR_SFP);

	ar->h[SP_A] = ((int)r->mval[SP_A]) << ( V_SFP - COLOR_SFP);
	ar->dh[SP_A] = ( nr->mval[SP_A] - r->mval[SP_A])*len_inv*
					(V_FP>>COLOR_SFP);

	w2 = r->width*r->width * (V_FP>>COLOR_SFP);
	nw2 = nr->width*nr->width * (V_FP>>COLOR_SFP);

	ar->lm[SP_Y] = r->lm[SP_Y]*w2;
	ar->dlm[SP_Y] = ( nr->lm[SP_Y]*nw2 - ar->lm[SP_Y] )*len_inv;

	ar->lm[SP_Q] = r->lm[SP_Q]*w2;
	ar->dlm[SP_Q] = ( nr->lm[SP_Q]*nw2 - ar->lm[SP_Q] )*len_inv;

	ar->lm[SP_A] = r->lm[SP_A]*w2;
	ar->dlm[SP_A] = ( nr->lm[SP_A]*nw2 - ar->lm[SP_A] )*len_inv;

#endif


	ar->bg[SP_Y] = ar->h[SP_Y] + ar->lm[SP_Y];
	ar->dbg[SP_Y] = ar->dh[SP_Y] + ar->dlm[SP_Y];

	ar->bg[SP_Q] = ar->h[SP_Q] + ar->lm[SP_Q];
	ar->dbg[SP_Q] = ar->dh[SP_Q] + ar->dlm[SP_Q];

	ar->bg[SP_A] = ar->h[SP_A] + ar->lm[SP_A];
	ar->dbg[SP_A] = ar->dh[SP_A] + ar->dlm[SP_A];


        ar->w = G_FPF/r->width;
        ar->dw = ( G_FPF/nr->width - ar->w)*len_inv;
        ar->w *= G_FPF;
}


#ifndef PLAYER 
void
expand_profile_ridge( ridge_aux *ar,
			int t, int  u,
			int val[SP_NO], int *ubg )
{
int	h,	lm,	pr;
int	i,	tfp;


	i = (u* ( ar->w + t * ar->dw) )>>(G_SFP+G_SFP+ G_SFP-6);
	if( ABS(i) >= 96 )      return;

//	*ubg = Tbg[i];
	*ubg = RIDGE_UNITY(i);
//	pr = Tpr[i];
	pr = RIDGE_PROFILE(i);

	tfp = t;

        h = ar->h[SP_Y] + ((tfp * ar->dh[SP_Y]) >> G_SFP);
        lm = ar->lm[SP_Y] + ((tfp * ar->dlm[SP_Y]) >> G_SFP);
        val[SP_Y] = (h + ((lm * pr) >> V_SFP)) >> G_SFP;


        h = ar->h[SP_Q] + ((tfp * ar->dh[SP_Q]) >> G_SFP);
        lm = ar->lm[SP_Q] + ((tfp * ar->dlm[SP_Q]) >> G_SFP);
        val[SP_Q] = (h + ((lm * pr) >> V_SFP)) >> G_SFP;


        h = ar->h[SP_A] + ((tfp * ar->dh[SP_A]) >> G_SFP);
        lm = ar->lm[SP_A] + ((tfp* ar->dlm[SP_A]) >> G_SFP);
        val[SP_A] = (h + ((lm * pr) >> V_SFP)) >> G_SFP;
}
#endif
