/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**************************
 ***	LinkBes.c	***
 **************************/


#include	"Uigp/igp.h"

#include	"GpType/GpType.h"


static void	link_bes_begin( link_type *l, int *Fbes );
static void	link_bes_end( link_type *l, int *Fbes );

int
link_bes( link_type *link )
{
int	Fbes;
	
	Fbes = 0x0f;

	link_bes_begin( link, &Fbes );
	link_bes_end( link, &Fbes );

	return( Fbes );
}


static void
link_bes_begin( link_type *l, int *Fbes )
{
link_type	*nl;
jnt_type	*ej;
brn_type	*eb;
vec2d	v;
int	bno,	nbno;


	if( (nl = LINK_PREV(l) ) != NULL ){
	        v.x = -nl->u.y;
		v.y =  nl->u.x;

		if( l->u.x * v.x + l->u.y * v.y > 0 )
			*Fbes &= ~0x02;
		else	*Fbes &= ~0x01;
		

		return;
	}


	ej = l->p[F_BEGIN].p.jnt;
/*
	if( ej == NULL || ej->type != JNT_RIDGE || ej->brn_no == 1 )	return;
*/
	if( ej->brn_no == 1 )	return;

	bno = l->p[F_BEGIN].bno;


	nbno = BRN_LEFT( ej, bno );
	eb = ej->brn[nbno];
	if( eb->link != NULL ){

		if( eb->direction == F_END ){
	        	v.x = -eb->link->u.y;
			v.y =  eb->link->u.x;
		}
		else	{
	        	v.x =  eb->link->u.y;
			v.y = -eb->link->u.x;
		}


		if( l->u.x * v.x + l->u.y * v.y > 0 )
			*Fbes &= ~0x02;
		else	*Fbes &= ~0x01;
	}
	

	if( ej->brn_no == 2 )	return;

	nbno = BRN_RIGHT( ej, bno );
	eb = ej->brn[nbno];
	if( eb->link != NULL ){
		if( eb->direction == F_END ){
	        	v.x = -eb->link->u.y;
			v.y =  eb->link->u.x;
		}
		else	{
	        	v.x =  eb->link->u.y;
			v.y = -eb->link->u.x;
		}


		if( l->u.x * v.x + l->u.y * v.y > 0 )
			*Fbes &= ~0x02;
		else	*Fbes &= ~0x01;
	}
}




static void
link_bes_end( link_type *l, int *Fbes )
{
link_type	*nl;
jnt_type	*ej;
brn_type	*eb;
vec2d	v;
int	bno,	nbno;



	if( (nl = LINK_NEXT(l) ) != NULL ){
	        v.x = -nl->u.y;
		v.y =  nl->u.x;

		if( l->u.x * v.x + l->u.y * v.y > 0 )
			*Fbes &= ~0x04;
		else	*Fbes &= ~0x08;
		

		return;
	}



	ej = l->p[F_END].p.jnt;
/*
	if( ej == NULL || ej->type != JNT_RIDGE || ej->brn_no == 1 )	return;
*/
	if( ej->brn_no == 1 )	return;

	bno = l->p[F_END].bno;


	nbno = BRN_LEFT( ej, bno );
	eb = ej->brn[nbno];
	if( eb->link != NULL ){
		if( eb->direction == F_BEGIN ){
	        	v.x = -eb->link->u.y;
			v.y =  eb->link->u.x;
		}
		else	{
	        	v.x =  eb->link->u.y;
			v.y = -eb->link->u.x;
		}


		if( l->u.x * v.x + l->u.y * v.y > 0 )
			*Fbes &= ~0x04;
		else	*Fbes &= ~0x08;
	}
	

	if( ej->brn_no == 2 )	return;

	nbno = BRN_RIGHT( ej, bno );
	eb = ej->brn[nbno];
	if( eb->link != NULL ){
		if( eb->direction == F_BEGIN ){
	        	v.x = -eb->link->u.y;
			v.y =  eb->link->u.x;
		}
		else	{
	        	v.x =  eb->link->u.y;
			v.y = -eb->link->u.x;
		}


		if( l->u.x * v.x + l->u.y * v.y > 0 )
			*Fbes &= ~0x04;
		else	*Fbes &= ~0x08;
	}
}
