/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/********************
 ***   XpAgp.c    ***
 ********************/

#include	<stdio.h>
#include	<math.h>

#include	"Uigp/igp.h"

#include        "GpType/GpType.h"

#include        "XpType.h"



void
xp_agp( gp_array *agp, int I0, int J0, int Ni, int Nj, xp_array *axp )
{
gp_type		*gp;
xp_type		*xp;
link_type	*link;

int	i,	j,	i0,	j0,	i1,	j1;
int	ni,	nj,	d;
box2d	box;
vec2d	end;



	axp->Ni0 = I0;
	if( I0 + Ni >= agp->Ni-1 )	Ni = agp->Ni-1-I0;

	axp->Nj0 = J0;
	if( J0 + Nj >= agp->Nj-1 )	Nj = agp->Nj-1-J0;

	xp_array_clear( axp, agp->bgp, Ni+2*LXP, Nj+2*LXP );


	axp->BNi = Ni;
	axp->BNj = Nj;
	axp->V0.x = (I0-1) * axp->bgp;
	axp->V0.y = (J0-1) * axp->bgp;

	axp->Pi = Ni*axp->bgp;
	axp->Pj = Nj*axp->bgp;


	agp->Ni0 = I0;
	agp->Ni1 = I0 + Ni;

	agp->Nj0 = J0;
	agp->Nj1 = J0 + Nj;

	agp->X0 = axp->V0.x;
	agp->Y0 = axp->V0.y;




	for( i = JXP, i1 = axp->Ni0-1 ; i < axp->Ni-JXP ; i++, i1++ ){
	    if( i1 < 0 || i1 >= agp->Ni )	continue;

	    xp = &axp->xp[i][JXP];
	    gp = &agp->gp[i1][axp->Nj0-1];

	    for( j = JXP, j1 = axp->Nj0-1 ; j < axp->Nj-JXP ;
							j++, j1++, xp++, gp++ ){

		xp->bgp = gp->bgp;

		xp->hunch = gp->hunch;

		xp->link = NULL;
		xp->rlink = NULL;
		for( link = gp->link ; link != NULL ; link = link->nl ){

			if( link->type == NF_RIDGE || link->type == NF_N3R ){
				link->cln = xp->rlink;
				xp->rlink = link;
				continue;
			}

			if( link->type != NF_EDGE && link->type != NF_N3_P )
				continue;

			link->cln = xp->link;
			xp->link = link;
		}
	    }
	}


	ni = axp->Ni -2*JXP;
	nj = axp->Nj -2*JXP;
	i0 = JXP;
	j0 = JXP;

	box.x0 = axp->V0.x - axp->bgp;
	box.y0 = axp->V0.y - axp->bgp;

	box.x1 = axp->V0.x + axp->Pi + 2*axp->bgp;
	box.y1 = axp->V0.y + axp->Pj + 2*axp->bgp;

	for( d = 0 ; d < 24 ; d += axp->bgp ){
	    i0 -= 1;
	    j0 -= 1;

	    ni += 2;
	    nj += 2;

	    j = j0;
	    j1 = axp->Nj0-(LXP-j0);
	    if( j1 >= 0 ){
		for( i = i0, i1 = axp->Ni0-(LXP-i0) ; i < i0 + ni; i++, i1++){
	    		if( i1 < 0 || i1 >= agp->Ni )	continue;

			gp = &agp->gp[i1][j1];
			xp = &axp->xp[i][j];
			xp->link = NULL;
			xp->rlink = NULL;

			for( link = gp->link ; link != NULL ; link = link->nl ){

				end.y = gp->ctr.y + link->ctr.y +link->v.y;
				if( end.y < box.y0 )	continue;

				if( link->type == NF_RIDGE ||
						link->type == NF_N3R ){
					link->cln = xp->rlink;
					xp->rlink = link;
					continue;
				}


				if( link->type != NF_EDGE &&
						link->type != NF_N3_P )
					continue;

				link->cln = xp->link;
				xp->link = link;
			}
		}
	    }
	    else 
		for( i = 0 ; i < axp->Ni; i++){
                       	xp = &axp->xp[i][j];
			xp->link = NULL;
			xp->rlink = NULL;
		}



		j = j0+nj-1;
		j1 = axp->Nj0-(LXP-j0) +nj-1;
	    if( j1 < agp->Nj ){
		for( i = i0, i1 = axp->Ni0-(LXP-i0) ; i < i0 + ni; i++, i1++){
	    		if( i1 < 0 || i1 >= agp->Ni )	continue;

			gp = &agp->gp[i1][j1];
			xp = &axp->xp[i][j];
			xp->link = NULL;
			xp->rlink = NULL;

			for( link = gp->link ; link != NULL ; link = link->nl ){

				end.y = gp->ctr.y + link->ctr.y +link->v.y;
				if( end.y > box.y1 )	continue;

				if( link->type == NF_RIDGE ||
						link->type == NF_N3R ){
					link->cln = xp->rlink;
					xp->rlink = link;
					continue;
				}


				if( link->type != NF_EDGE &&
						link->type != NF_N3_P )
					continue;

				link->cln = xp->link;
				xp->link = link;
			}
		} 
	    }

	    else
		for( i = 0 ; i < axp->Ni; i++){
                        xp = &axp->xp[i][j];
                        xp->link = NULL;
                        xp->rlink = NULL;
		}



		i = i0;
		i1 = axp->Ni0-(LXP-i0);
	    if( i1 >= 0 ){
		for( j = j0+1, j1 = axp->Nj0-(LXP-j0)+1 ; j < j0 + nj-1; j++, j1++ ){
	    		if( j1 < 0 || j1 >= agp->Nj )	continue;

			gp = &agp->gp[i1][j1];
			xp = &axp->xp[i][j];
			xp->link = NULL;
			xp->rlink = NULL;

			for( link = gp->link ; link != NULL ; link = link->nl ){

				end.x = gp->ctr.x + link->ctr.x +link->v.x;
				if( end.x < box.x0 )	continue;


				if( link->type == NF_RIDGE ||
						link->type == NF_N3R ){
					link->cln = xp->rlink;
					xp->rlink = link;
					continue;
				}


				if( link->type != NF_EDGE &&
						link->type != NF_N3_P )
					continue;

				link->cln = xp->link;
				xp->link = link;
			}
		}
	    }
	    else
		for( j = 0, xp = &axp->xp[i][0] ; j < axp->Nj; j++, xp++ ){
			xp->link = NULL;
                        xp->rlink = NULL;
		}


		i = i0+ni-1;
		i1 = axp->Ni0-(LXP-i0) +ni-1;
	    if( i1 < agp->Ni ){
		for( j = j0+1, j1 = axp->Nj0-(LXP-j0)+1 ; j < j0 + nj-1; j++, j1++ ){
	    		if( j1 < 0 || j1 >= agp->Nj )	continue;

			gp = &agp->gp[i1][j1];
			xp = &axp->xp[i][j];
			xp->link = NULL;
			xp->rlink = NULL;


			for( link = gp->link ; link != NULL ; link = link->nl ){

				end.x = gp->ctr.x + link->ctr.x +link->v.x;
				if( end.x > box.x1 )	continue;


				if( link->type == NF_RIDGE ||
						link->type == NF_N3R ){
					link->cln = xp->rlink;
					xp->rlink = link;
					continue;
				}


				if( link->type != NF_EDGE &&
						link->type != NF_N3_P )
					continue;

				link->cln = xp->link;
				xp->link = link;
			}
		} 
	    }

	    else
		for( j = 0, xp = &axp->xp[i][0] ; j < axp->Nj; j++, xp++ ){
			xp->link = NULL;
                        xp->rlink = NULL;
		}
	}

}
