/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**************************
 ***   XpTool.c  	***
 **************************/

#include	<stdio.h>
#include	<math.h>

#include	"Uigp/igp.h"


#include	"XpType.h"


static void	xp_array_alloc( xp_array *axp, int ni, int nj );


xp_array *
axp_alloc()
{
xp_array *axp;

	axp = (xp_array *)malloc( sizeof( xp_array ) );
	axp->NI = axp->Ni = -1;
	axp->NJ = axp->Nj = -1;
	axp->xp = NULL;

	return( axp );
}


void
axp_destroy( xp_array *axp )
{
	xp_array_free( axp );

	free( axp );
}


void
xp_array_init( xp_array *axp, int bgp, int ni, int nj )
{
int     i,	j;
xp_type	*xp;
vec2d	v,	v0;


	xp_array_alloc( axp, ni, nj );

	axp->bgp = bgp;
	axp->bgp_inv = 1.0/bgp;
	axp->Ni = ni;
	axp->Nj = nj;


	v0.x = v0.y = -bgp *(LXP-0.5);

	for( i = 0, v.x = v0.x ; i < axp->Ni ; i++, v.x += bgp)
		for( j = 0 , v.y = v0.y ; j < axp->Nj ; j++, v.y += bgp ){

			xp = &axp->xp[i][j];
			xp->ctr.x = v.x;
			xp->ctr.y = v.y;

			xp->link = NULL;
			xp->rlink = NULL;


		}
}




void
xp_array_clear( xp_array *axp, int bgp, int Ni, int Nj )
{

	if( Ni > axp->NI || Nj > axp->NJ || axp->bgp != bgp ){
		xp_array_free( axp );
		xp_array_init( axp, bgp, Ni, Nj );

		return;
	}



	axp->Ni = Ni;
	axp->Nj = Nj;
/*
	for( i = 0 ; i < axp->Ni ; i++ ){
		xp = axp->xp[i];

		for( j = 0 ; j < axp->Nj ; j++, xp++ )
			xp->link = NULL;
	}
*/
}



void
xp_array_Fclear( xp_array *axp, int bgp, int Ni, int Nj )
{
xp_type	*xp;
int	i,	j;

	if( Ni > axp->NI || Nj > axp->NJ || axp->bgp != bgp ){
		xp_array_free( axp );
		xp_array_init( axp, bgp, Ni, Nj );

		return;
	}



	for( i = 0 ; i < axp->Ni ; i++ ){
		xp = axp->xp[i];

		for( j = 0 ; j < axp->Nj ; j++, xp++ )
			xp->link = NULL;
			xp->rlink = NULL;
	}

	axp->Ni = Ni;
	axp->Nj = Nj;
}




void
xp_array_realloc( xp_array *axp, int ni, int nj )
{
	if( axp->Ni == ni && axp->Nj == nj )  return;
 
	xp_array_free( axp );
 
	xp_array_alloc( axp, ni, nj );
}
 
 
static void
xp_array_alloc( xp_array *axp, int ni, int nj )
{
xp_type	*xp;
int	i;  
 
	axp->NI = axp->Ni  = ni ;
	axp->NJ = axp->Nj  = nj ;
 
	axp->xp = (xp_type **)malloc( axp->NI * sizeof( xp_type * ) );

	axp->xp[0] = (xp_type *)malloc( axp->NI * axp->NJ * sizeof( xp_type ));
 

        for( i = 1, xp = &axp->xp[0][axp->NJ] ; i < axp->NI ;
                                                i++, xp += axp->NJ )
                axp->xp[i] = xp;

}
 
 
 
void
xp_array_free( xp_array *axp )
{
 
	if( axp->xp == NULL )	return;

	free( axp->xp[0] );
 
	free( axp->xp );
	axp->xp = NULL;

	axp->NI = axp->NJ = 0;
}
