/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/

/**************************
 ***	ZpComplete.c  	***
 **************************/

#include	<stdio.h>
#include	<math.h>

#include	"Uigp/igp.h"


#include        "GpType/GpType.h"
#include        "ZpType.h"



static int	dp_complete_one( zp_array *azp, zp_type *dp, int step );

static void	dp_find_br( zp_type *dp, mbr_type Mbr,
				int *br, float *depth, int step );



void
azp_complete( zp_array *azp )
{
zp_type	*dp;
int	i,	j,	no;

    for( i = 1, no = 0 ; i < azp->Ni-1 ; i++ ){
		dp = &azp->dp[i][1];
		for( j = 1 ; j < azp->Nj-1 ; j++, dp++ )
			no += dp_complete_one( azp, dp, 1 );
	}


	if( no == 0 )	return;


    for( i = 1, no = 0 ; i < azp->Ni-1 ; i++ ){
		dp = &azp->dp[i][1];
		for( j = 1 ; j < azp->Nj-1 ; j++, dp++ )
			no += dp_complete_one( azp, dp, 2 );
	}
}



static int
dp_complete_one( zp_array *azp, zp_type *dp, int step )
{
int	br,	i;
float	depth;


	depth = 1000;
	br = -1;

	dp_find_br( dp - azp->NJ, dp->Mbr, &br, &depth, step );
	dp_find_br( dp - 1, dp->Mbr, &br, &depth, step );
	dp_find_br( dp +1, dp->Mbr, &br, &depth, step );
	dp_find_br( dp + azp->NJ, dp->Mbr, &br, &depth, step );


	if( br == -1 )	return(0);

	if( dp->br_no == 0 ){
//		dp->Mbr |= 1L<< br;
		MBR_SET( dp->Mbr, br );
		dp->br[0] = br;
		dp->depth[0] = depth;
		dp->br_no = 1;
		return( 1 );
	}


	for( i = dp->br_no ; i > 0 ; i-- ){
		if( depth > dp->depth[i-1] )	break;

		dp->br[i] = dp->br[i-1];
		dp->depth[i] = dp->depth[i-1];
	}

	dp->br[i] = br;
	dp->depth[i] = depth;

	dp->br_no++;
//	dp->Mbr |= 1L<< br;
	MBR_SET( dp->Mbr, br );

	dp->step = step;

	return( 1 );
}




static void
dp_find_br( zp_type *dp, mbr_type Mbr, int *br, float *depth, int step )
{
int	i;
mbr_type	mbr;

	if( (Mbr&dp->Mbr) == dp->Mbr )	return;

	if( dp->step >= step )	return;

	for( i = 0 ; i < dp->br_no ; i++ ){
//		mbr = 1L<<dp->br[i];
		mbr = MBR_BIT(dp->br[i] );

		if( Mbr & mbr )	continue;
		if( dp->Mbr_ee & mbr )	continue;

		if( dp->depth[i] >= *depth )	continue;

		*depth = dp->depth[i];
		*br = dp->br[i];
	}
}
