/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**************************
 ***	ZpEdge.c	***
 **************************/
#include        <stdio.h>   

#include        "ZpType.h"


static void	azp_edge_one( vec2d *ctr, link_type *l, zp_array *azp );

static int	azp_edge_br_one( vec2d *ctr,
			link_type *l, int br, int Fbr, zp_array *azp );

void
azp_edge( xp_array *axp, zp_array *azp )
{
xp_type *xp;
link_type *l;
vec2d   ctr;
int     i,      j,	 algin;


	algin = axp->NJ - axp->Nj;

	xp = &axp->xp[0][0];
        for( i = 0 ; i < axp->Ni ; i++, xp += algin ){

		for( j = 0 ; j < axp->Nj ; j++, xp++ ){


			for( l = xp->link ; l != NULL ; l =l->cln){

				ctr.x = l->ctr.x + xp->ctr.x;
				ctr.y = l->ctr.y + xp->ctr.y;

				if( l->Fbr == 0 ){
					azp_edge_one( &ctr, l, azp );
					continue;
				}


				if( l->Fbr & 0x03 ){
					azp_edge_br_one( &ctr, l,
						l->br, l->Fbr, azp);
					continue;
				}

				azp_edge_br_one( &ctr, l, l->br, 0x01, azp);
				azp_edge_br_one( &ctr, l, l->br1, 0x02, azp);

			}
		}
	}

}

#define	DT	4

static void
azp_edge_one( vec2d *ctr, link_type *l, zp_array *azp )
{
float	t;
vec2d	v,	v1,	v2,	p;
float	Ax,	Bx,	Ay,	By;
int	i0,	j0;
//int	br_mask = 1L << l->br;
mbr_type	br_mask;
float	depth,	depth1,	ddepth;
zp_type	*dp,	*pdp;

	ctr->x -= 0.5;
	ctr->y -= 0.5;


	br_mask = MBR_BIT( l->br );


	v.x = ctr->x + azp->bgp;
	v.y = ctr->y + azp->bgp;

	p.x = v.x + l->v.x;
	p.y = v.y + l->v.y;




	Ax = l->c_prb* l->u.x;
	Bx = -l->u.y - Ax * l->len;

	v1.x = (Ax*DT + Bx)*DT;
	v2.x =  Ax*2*DT *DT;



	Ay = l->c_prb* l->u.y;
	By = l->u.x - Ay * l->len;

	v1.y = (Ay*DT + By)*DT;
	v2.y =  Ay*2*DT *DT;


	depth = l->pr[0].edge->depth;
	depth1 = l->pr[1].edge->depth;
	ddepth = DT*( depth1 - depth )/l->len;


	pdp = NULL;
	for( t = 0 ; t < l->len ; t += DT ,
			depth += ddepth,
			v.x += v1.x , v1.x += v2.x,
			v.y += v1.y , v1.y += v2.y ){

		if( v.x < 0 || v.y < 0 )        continue;
		i0 = v.x * azp->bgp_inv;
		j0 = v.y * azp->bgp_inv;

		if( i0 >= azp->Ni || j0 >= azp->Nj )	continue;


		dp = &azp->dp[i0][j0];
		if( pdp == dp )	continue;
		pdp = dp;

		if( dp->Mbr_e & br_mask )	continue;
		dp->Mbr_e |= br_mask;

		dp->br[dp->br_no] = l->br;
		dp->depth[dp->br_no] = depth;
		dp->br_no++;

		azp->Mbr |= br_mask;
	}



	if( p.x < 0 || p.y < 0 )        return;
	i0 = p.x * azp->bgp_inv;
	j0 = p.y * azp->bgp_inv;
	if( i0 >= azp->Ni || j0 >= azp->Nj )	return;

	dp = &azp->dp[i0][j0];
	if( dp == pdp && pdp  )	return;
	if( !(dp->Mbr_e & br_mask) ){
		dp->Mbr_e |= br_mask;

		dp->br[dp->br_no] = l->br;
		dp->depth[dp->br_no] = depth1;
		dp->br_no++;

		azp->Mbr |= br_mask;
	}
}



#define	DT1	2

static int
azp_edge_br_one( vec2d *ctr, link_type *l, int br, int Fbr, zp_array *azp )
{
vec2d	v,	v1,	v2,	end;
float	Ax,	Bx,	Ay,	By;
float	depth0,	depth1,	ddepth;
float	t;
int	i0,	j0;
//int	br_mask = 1L << br;
mbr_type	br_mask;
vec2d	p[200];
float	depth[200];
zp_type	*dp,	*pdp;
int	flag,	i,	no;

float	dh,	dh0;
vec2d	dv0,	dv1;
vec2d	tv;

	ctr->x -= 0.5;
	ctr->y -= 0.5;

	flag = 0;

	br_mask = MBR_BIT( br );

	if( ( Fbr & 0x01 ) ){
		dh = 1.5;
		dh0 = -0.5;
	}
	else	{
		dh = -1.5;
		dh0 = 0.5;
	}


	v.x = ctr->x + azp->bgp;
	v.y = ctr->y + azp->bgp;

	end.x = v.x + l->v.x;
	end.y = v.y + l->v.y;


	Ax = l->c_prb* l->u.x;
	Bx = -l->u.y - Ax * l->len;

	v1.x = (Ax*DT1 + Bx)*DT1;
	v2.x =  Ax*2*DT1 *DT1;



	Ay = l->c_prb* l->u.y;
	By = l->u.x - Ay * l->len;

	v1.y = (Ay*DT1 + By)*DT1;
	v2.y =  Ay*2*DT1 *DT1;


	depth0 = l->pr[0].edge->depth;
	depth1 = l->pr[1].edge->depth;
	ddepth = DT1*( depth1 - depth0 )/l->len;

	dv0.x = dh0 * l->u.x;
	dv0.y = dh0 * l->u.y;

	dv1.x = dh * l->u.x;
	dv1.y = dh * l->u.y;

	tv.x  = azp->Ni*azp->bgp + 2.5;
	tv.y  = azp->Nj*azp->bgp + 2.5;

	pdp = NULL;
	for( t = 0, no = 0 ; t < l->len ; t += DT1 ,
			depth0 += ddepth,
			v.x += v1.x , v1.x += v2.x,
			v.y += v1.y , v1.y += v2.y ){


		if( v.x < -2.5 || v.x > tv.x || 
			v.y < -2.5 || v.y > tv.y )	continue;


		p[no].x = v.x + dv0.x;
		p[no].y = v.y + dv0.y;

		p[no+1].x = v.x +  dv1.x;
		p[no+1].y = v.y +  dv1.y;


		depth[no] = depth[no+1] = depth0;

		no += 2;
	}


	p[no].x = end.x + dv0.x;
	p[no].y = end.y + dv0.y;

	p[no+1].x = end.x +  dv1.x;
	p[no+1].y = end.y +  dv1.y;
	depth[no] = depth[no+1] = depth1;

	no += 2;


	flag = 0;
	pdp = NULL;
	for( i = 0 ; i < no ; i++ ){

		if( p[i].x < 0 || p[i].y < 0 )	continue;

		i0 = p[i].x * azp->bgp_inv;
		j0 = p[i].y * azp->bgp_inv;

		if( i0 >= azp->Ni || j0 >= azp->Nj )	continue;
		flag = 1;

		dp = &azp->dp[i0][j0];
		if( pdp == dp )	continue;
		pdp = dp;

		if( dp->Mbr_ee & br_mask )	continue;
		dp->Mbr_ee |= br_mask;
		if( dp->Mbr_e & br_mask )	continue;
		dp->Mbr_e |= br_mask;

		dp->br[dp->br_no] = br;
		dp->depth[dp->br_no] = depth[i];
		dp->br_no++;
	}


	if( flag == 1 ){
		azp->Mbr |= br_mask;
//		azp->Mbr_e |= br_mask;
		azp->Ftransparent = 1;
	}

	return( flag );
}
