/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**************************
 ***    ZpInit.c       ***
 **************** *********/

#include        <stdio.h>   

#include        "ZpType.h"
#ifdef PALMOS
	#define MALLOC	malloc2
	#define FREE	free2
#else	
	#define MALLOC	malloc
	#define FREE	free
#endif

static void	zp_array_alloc( zp_array *adp, int ni, int nj );


void
azp_init( zp_array *adp, int ni, int nj )
{
int     i,      j;
zp_type	*e;

	ni += 2;
	nj += 2;

	if( ni > adp->NI || nj > adp->NJ ){
		zp_array_free( adp );
		zp_array_alloc( adp, ni, nj );
	}

	adp->Ni = ni;
	adp->Nj = nj;

	adp->Mbr = 0;
//	adp->Mbr_e = 0;
//	adp->Ffilter = 0;
	adp->Ftransparent = 0;


	for( i = 0 ; i < adp->Ni ; i++ )
	    for( j = 0, e = adp->dp[i] ; j < adp->Nj ; j++, e++ ){

		e->Mbr = e->Mbr_e = e->Mbr_ee = e->br_no = 0;
		e->step = 0;
	    }
}



static void
zp_array_alloc( zp_array *adp, int ni, int nj )
{
int	i; 
zp_type	*dp;
	adp->Ni  = adp->NI = ni;
	adp->Nj  = adp->NJ = nj;

	adp->dp = (zp_type **)MALLOC( adp->Ni * sizeof( zp_type * ) );
	adp->dp[0] = (zp_type *)MALLOC( adp->Ni * adp->Nj *sizeof( zp_type ) );

	for( i = 1, dp = &adp->dp[0][adp->Nj] ; i < adp->Ni ;
						i++, dp += adp->Nj )
		adp->dp[i] = dp;
}



void
zp_array_free( zp_array *adp )
{
	if( adp->dp == NULL )	return;

	FREE( adp->dp[0] );
	FREE( adp->dp );

	adp->dp = NULL;
 
	adp->NI = adp->NJ = 0;
	adp->Ni = adp->Nj = 0;
}



#ifdef _DEBUG
void
azp_print( zp_array *adp, int i0, int j0, int ni, int nj )
{
int	i,	j;
zp_type	*dp;

	for( i = i0 ; i < i0+ni; i++ ){
		for( j = j0 ; j < j0+nj; j++ ){
			dp = &adp->dp[i][j];

			if( dp->Mbr != 0 )
				fprintf( stderr, "1   " );
			else	fprintf( stderr, "-   " );
		}
		fprintf( stderr, "\n" );
	}
}
#endif
