/*******************************
 ***   VimFrameControl.cpp   ***
 *******************************/


#include "FrameControl.h"

extern "C" {
#include	"VimageType/VimageType.h"
#include	"Uigp/igp.h"
#include	"GpType/GpType.h"
}


CFrameControl::CFrameControl()
{
	m_framePrevius	= 0;
	m_FrameRate		= 0;
	m_frameDelay	= 0;
	m_tmPrevius		= 0;	
	m_bLoop			= TRUE;	
}

int	CFrameControl::GetPrevFrame()
{
	if( m_iFrame == 0  )	return( -1 );
	
	return( m_iFrame - 1 );
}



int	CFrameControl::GetNextFrame()
{
	if( m_iFrame >= m_Frames-1 )	return( -1 );

	return( m_iFrame + 1 );
}

int	CFrameControl::GetShiftFrame(int iShift)
{
	if( m_iFrame >= m_Frames-1 )	return( -1 );
	if( m_iFrame+iShift >= m_Frames )	
		return( m_Frames-1 );
	return( m_iFrame + iShift );
}



void CFrameControl::SetFrameRate(int FrameRate)
{
	m_FrameRate = FrameRate;

	m_frameDelay = (m_FrameRate != 0)? 1000/m_FrameRate : 0;
}



void CFrameControl::StartPlayer()
{
	m_tmPrevius = 0;
}




int	CFrameControl::GetNextFrameToPlay( u_int time )
{
int	nextFrame;

	if( m_iFrame >= m_Frames-1 && !m_bLoop )
	{
			return( END_OF_ANIMATION );
	}

	// Don't skip frames, play the file as is, or at start
//	if( m_frameDelay == 0 || m_tmPrevius == 0 )
	if( m_tmPrevius == 0 )
	{
		m_tmPrevius = time;


		nextFrame = m_iFrame + 1;

		if( nextFrame >= m_Frames )
			nextFrame = m_Frames-1;

		m_framePrevius = nextFrame;
		return( nextFrame );
	}


	if( m_frameDelay == 0 )
	{
		m_tmPrevius = time;

		nextFrame = m_iFrame + 1;

		if( nextFrame >= m_Frames )
		{	
			if ( m_bLoop )
				nextFrame = 0;
			else 
				return( END_OF_ANIMATION );
		}

		m_framePrevius = nextFrame;
		return( nextFrame );
	}



	int	dt = time - m_tmPrevius;
	int	dframe = (dt+ (m_frameDelay>>1)) / m_frameDelay;
	// dframe = dt/m_frameDelay + 0.5; // 0.5 to be 1 in case of 0

	nextFrame = m_framePrevius + dframe;

	if( nextFrame <= m_iFrame )
		return( WAIT_FOR_NEXT_TIMER );

	// 
	if( nextFrame >= m_Frames )	
	{
		if ( m_bLoop )
		{
			nextFrame = nextFrame - m_Frames;
			m_framePrevius = nextFrame;
			m_tmPrevius     = time;
		}
		else
			return( END_OF_ANIMATION );
//			nextFrame = m_Frames-1;
	}

	return( nextFrame );
}


/*
int	CFrameControl::GetNextFrameToPlay( u_int time )
{
int	nextFrame;

	if( m_iFrame >= m_Frames-1 && !m_bLoop )
	{
			return( END_OF_ANIMATION );
	}

	// Don't skip frames, play the file as is, or at start
	if(  m_tmPrevius == 0 )
	{
		m_tmPrevius = time;


		nextFrame = m_iFrame + 1;
		if( nextFrame >= m_Frames )	
			nextFrame = m_Frames-1;

		m_framePrevius = nextFrame;
		return( nextFrame );
	}


	if( m_frameDelay == 0  )
	{
		m_tmPrevius = time;


		nextFrame = m_iFrame + 1;
		if( nextFrame >= m_Frames ){
			if( !m_bLoop )
				return( END_OF_ANIMATION );

			nextFrame = 0;
		}

		m_framePrevius = nextFrame;
		return( nextFrame );
	}



	int	dt = time - m_tmPrevius;
	int	dframe = (dt+ (m_frameDelay>>1)) / m_frameDelay;
	// dframe = dt/m_frameDelay + 0.5; // 0.5 to be 1 in case of 0

	nextFrame = m_framePrevius + dframe;

	if( nextFrame <= m_iFrame )
		return( WAIT_FOR_NEXT_TIMER );

	// 
	if( nextFrame >= m_Frames )	
	{
		if ( m_bLoop )
		{
			nextFrame = nextFrame - m_Frames;
			m_framePrevius = nextFrame;
			m_tmPrevius     = time;
		}
		else
			nextFrame = m_Frames-1;
	}

	return( nextFrame );
}

*/






