/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
#ifndef         _IMAGE_
#define         _IMAGE_

#ifdef WIN32
#pragma warning(disable :4244)
#endif


#include	"Uigp/igp.h"
#include	"Umat/Mat2fType.h"
#include	"Uvec/Vec2d.h"
#include	"Ubox/Box2d.h"



#include	"PaletteType.h"



typedef struct	image_type {

	int	column;	//width
	int	row;	//height

	int	depth;	// byte per pixel
	int	pad;	// num of additional bytes at the end of the line (complete to 4*n bytes, so we read a whole int)

	int	column_byte;	/* line length : depth*column + pad */

	u_char	*data;

	palette_type	*palette;

	char	*file;
} image_type;


#define		IMAGE_PIXEL( im, row, column )\
		( (im)->data + (row)*(im)->column_byte +\
		(column)*(im)->depth )

#define		IMAGE4_PIXEL( im, row, col ) \
		( ((u_int *)(im)->data) + (row)*(im)->column + (col) )



#define		IMAGE_ROW( im )	((im)->row)
#define		IMAGE_COLUMN( im )	((im)->column)
#define		IMAGE_DATA_ADRESS( im )	((im)->data )

#define		IMAGE_DATA( im )	((im)->data )
#define		IMAGE4_DATA( im )	((u_int *)((im)->data) )


#define		IMAGE_DATA_SIZE( im )   ( IMAGE_ROW( im )* IMAGE_COLUMN( im ) )



/* This defination is only for im->depth == 4 */

#define         SP_0    SP_R
#define         SP_1    SP_G
#define         SP_2    SP_B

//********************************************
// THERE AREIMAGE_RGB MACROS IN PNG AS WELL!!!
// IF YOU ADD PLATFORM, ADD IT THERE AS WELL!!
//********************************************


#ifdef WIN32
#define		IMAGE4_RED( pixel )	(  (pixel) >> 16)
#define		IMAGE4_GREEN( pixel )	( ((pixel) >> 8 ) &0xff )
#define		IMAGE4_BLUE( pixel )	(  (pixel)&0xff )

#define		IMAGE4_RGB( r, g, b ) 	(((r)<<16) | ((g)<<8) | (b) )
#endif


#ifdef __linux
#define		IMAGE4_RED( pixel )	(  (pixel) >> 16)
#define		IMAGE4_GREEN( pixel )	( ((pixel) >> 8 ) &0xff )
#define		IMAGE4_BLUE( pixel )	(  (pixel)&0xff )

#define		IMAGE4_RGB( r, g, b ) 	(((r)<<16) | ((g)<<8) | (b) )
#endif



#ifdef PALMOS

#define		IMAGE4_BLUE( pixel )	( (pixel) >> 16 )
#define		IMAGE4_GREEN( pixel )	( ((pixel) >> 8 ) &0xff )
#define		IMAGE4_RED( pixel )	(  (pixel)&0xff )

#define		IMAGE4_RGB( r, g, b ) 	(((b)<<16) | ((g)<<8) | (r) )
#endif


#ifndef IMAGE4_RGB

#define		IMAGE4_BLUE( pixel )	( (pixel) >> 16 )
#define		IMAGE4_GREEN( pixel )	( ((pixel) >> 8 ) &0xff )
#define		IMAGE4_RED( pixel )	(  (pixel)&0xff )

#define		IMAGE4_RGB( r, g, b ) 	(((b)<<16) | ((g)<<8) | (r) )
#endif




	/* Image.c */
image_type *	image_create( int row, int column,
					int depth, int pad, u_char *data );

void	image_recreate( image_type *im,
			int row, int column, int depth, int pad );

void	image_destroy( image_type *im, int Fdata );

#define	image_destory	image_destroy

image_type *	image_make_copy( image_type *sim );

void	image_box(image_type *im, box2i *b, int *i0, int *j0, int *row, int *column );


image_type *
image_1to4( image_type *sim );

	/* Image4Tool.c */
image_type *	image4_sample( image_type *sim, int srow, int scol );

image_type *	image4_dup( image_type *sim, int drow, int dcol );


void		image4_mark( image_type *im, int row0, int col0,
					int row, int col, int color );

void		image4_const( image_type *im, int color );

void		image4_Vflip( image_type *im );

image_type *	image4_Vflip2( image_type *sim, image_type *tim );

void		image4_copy_in( image_type *sim,
				image_type *im, int row0, int col0 );

image_type * 	image4_copy( image_type *sim );

image_type *	image4_to_red( image_type *sim );

image_type *	image4_to_green( image_type *sim );

image_type *	image4_to_blue( image_type *sim );

image_type *	image4_to_y( image_type *sim );

image_type *	image4_from_y( image_type *sim );

image_type *	image4_rotate90( image_type *sim, image_type *tim );

image_type *	image4_rotate270( image_type *sim, image_type *tim );


	/* ImageSample.c */
image_type *	image_sample( image_type *sim, float srow, float scol );

image_type *	image_sample_transparent( image_type *sim, float srow, float scol );


	/* ImageSamplingInt.c */
image_type * image_sampleic( image_type *sim, int drow, int dcol );
image_type * image_sampleic2( image_type *sim, int drow, int dcol );
image_type * image_sampleic2A( image_type *sim, int drow, int dcol );


	/* ImageSamplingF.c */
image_type * image_samplefc( image_type *sim, int drow, int dcol );


	/* ImageSampleDown.c */
image_type * image_sample_down( image_type *sim, int drow, int dcol );

	/* ImageCut.c */
image_type *	image_cut( image_type *sim,
			int row0, int col0, int row, int col );

image_type *	image4_cut( image_type *sim,
			int row0, int col0, int row, int col );

	/* ImageSubtruct.c */
image_type *	image_subtract( image_type *sim1, image_type *sim0 );
image_type *	image4_subtract( image_type *sim1, image_type *sim0 );


	/* ImageRgbYCrCb.c */
image_type *	image_Rgb_YCrCb( image_type *im, int Falloc );


	/* ImageRgbYiq.c */
image_type *image_Rgb_Yiq( image_type *im, int Falloc );


	/* ImageComparison */
void	image_comparison( image_type *im0, image_type *im1,
                float var[],    /* Variabilty */
                float dev[],
                float snr[],    /* Signal to noise ratio */
                float average[],
                int peak[] );

void	image4_comparisonG( image_type *im0, image_type *im1, float *var, float *dev, float *average );


	/* ImageEqual.c   */
int		image_equal( image_type *im0, image_type *im1 );


	/* ImageCutUarray.c */
void	image4_cut_uarray( image_type *im,
                        int x0, int y0, int row, int column,
                        u_char *aY,
                        u_char *aQ,
                        u_char *aA,
                        int acolumn);



	/* ImageFile.c */
void	image_file_attach_name( image_type *im,
			char *dir, char *name, char *extension );

void	image_file_attach_nameF( image_type *im, char *fname );

int	image_file_size( image_type *im );



        /* ImagePreview.c */
image_type *    image_preview( image_type *im, int size, int color );


        /* ImageTransparent.c */
image_type *    image_transparent(image_type * sim,
                float trans, int backcolor, image_type *im); 

void	image_transparent_set(image_type *bim, box2i *b,
						image_type *im, int transpart_index );

void	image_transparent_copy_in(image_type *bim, box2i *b,
						image_type *im, int transpart_index );

void	imageT_clear( image_type *im, int color );

image_type *    image_border_fade(image_type * sim,
								float trans, int backcolor,
								int r0, int c0, int row, int column, image_type *im); 


	/* ImageHistogram.c */
int	image_histogram( image_type *im, int h[256] );

image_type *	image_histogram_equlization( image_type *sim );

image_type *	image_histogram_curve( image_type *sim, 
		float partL, int limL, float partR, int limR );


	/* ImageConvolution.c */
image_type *	image_convolution( image_type *sim, mat2f_type *m );
float	image_mat2f_inner( image_type *im, int i0, int j0, mat2f_type *m );


	/* ImageEdgePixel.c */
image_type *	image_edge_pixel( image_type *sim, mat2f_type *m, float T );
image_type *	image_edge_pixel_value( image_type *sim, mat2f_type *m, float a, float b );

image_type *	image_edge_gradient( image_type *sim, mat2f_type *mx, mat2f_type *my, float T );
image_type *	image_edge_gradient_value( image_type *sim, mat2f_type *mx, mat2f_type *my, float a, float b );


	/*ImageSline.c */
image_type *	image_sline(  vec2d *n, float d0, float d1, float a,
							int c0, int c1, int c2, int row, int col );
image_type *	image_S_cosinos(  float a, float n, int row, int col );





	/* ImageFourier.c */
image_type *	image_fourier_amplitod( image_type *im ); 

image_type *	image_fourier(image_type *im);

image_type *	image_fourier_inverse(image_type *im);

	/* ImageDitherPat.c */
image_type *	image_dithering_random( image_type *im, image_type *new_im  );

image_type *	image_dithering_pattern1( image_type *im, image_type *new_im );

image_type *	image_dithering_pattern2( image_type *im, image_type *new_im );

	/* ImageDitherOrd.c */
image_type *	image_dithering_ordered_2x2( image_type *im, image_type *new_im );

image_type *	image_dithering_ordered_3x3_1( image_type *im, image_type *new_im );

image_type *	image_dithering_ordered_3x3_2( image_type *im, image_type *new_im );

image_type *	image_dithering_ordered_4x4( image_type *im, image_type *new_im );

	/* ImageDitherErr.c */
image_type *	image_dithering_error_simpler( image_type *im, image_type *new_im );

image_type *	image_dithering_error_floyd( image_type *im, image_type *new_im );

image_type *	image_dithering_error_burke( image_type *im, image_type *new_im );

image_type *	image_dithering_error_jarvis( image_type *im, image_type *new_im );

image_type *	image_dithering_error_stucki( image_type *im, image_type *new_im );


image_type *	image_dithering_error_gray16( image_type *im, image_type *new_im );

image_type *	image_dithering_error_floyd_gray16( image_type *im, image_type *new_im );

image_type *    image_dithering_error_floyd_grey2b( image_type *im, image_type *new_im );


	/* ImageDitherAll.c */
image_type *	image_dithering_all( image_type *im, image_type *new_im );


    /* Image24To8.c */
//image_type * image_24to8_simple(image_type * im, box2i * pBox );

image_type * image_24to8(image_type * im, palette_type * palette, box2i * pBox );

image_type * image_24to8_UNKNOWN(image_type * im, palette_type * palette, box2i * pBox);

image_type * image_24to8_smart(image_type * im, palette_type * palette, box2i * pBox);

image_type * image_24to8_222(image_type * sim, palette_type * p, box2i * pBox);

image_type * image_24to8_gray(image_type * sim, palette_type * p, box2i * pBox);

image_type * image_24to8_HALFTONE(image_type * sim, palette_type * p, box2i * pBox);

image_type * image_24to8_BW(image_type * im, palette_type * p, box2i * pBox);

image_type * image_24to8_WB(image_type * im, palette_type * p, box2i * pBox);

image_type *	image_24to8_ADAPTIVE4(image_type * im, palette_type * palette, box2i * pBox);

void    image_difference(image_type * im1, image_type * im2, box2i * b);


    /* Image24To8DitherFloyd.c */
image_type *	image_24to8_dithering_floyd(image_type * im, palette_type * palette, box2i * pBox );

image_type *	image_24to8_UNKNOWN_dithering_floyd(image_type * im, palette_type * palette, box2i * pBox );

image_type *	image_24to8_smart_dithering_floyd(image_type * im, palette_type * palette, box2i * pBox );

image_type *	image_24to8_222_dithering_floyd(image_type * im, palette_type * palette, box2i * pBox );

image_type *	image_24to8_gray_dithering_floyd(image_type * sim, palette_type * p, box2i * pBox);

image_type *	image_24to8_HALFTONE_dithering_floyd(image_type * im, palette_type * palette, box2i * pBox );

image_type *	image_color_222_dithering_floyd( image_type *im, image_type *new_im );


    /* ImageDitherFloyd.c */
image_type *	image_dithering_floyd(image_type * im, image_type * new_im);


	/*	Image8To24.c */
image_type *	image_8to24( image_type *im, palette_type *palette );
void	image_8to24_copy( image_type *sim, image_type *im, 
						 int row0, int col0, palette_type *p );
void	image_8to24_copy_transparent( image_type *sim, image_type *im, 
									 int row0, int col0, int transparent_index, palette_type *p);


	/* ImageSmartPalette.c */
palette_type *	image_createSmartPalette(int n, image_type * im );

palette_type *	image_createSmartPaletteTransparent( int n, int nImages, image_type *images[] );
palette_type *	image_createSmartPaletteTransparent2( int n, int nImages, image_type *images[] );


	/* ImageAdaptivePalette.c */
void	image_adaptive_palette( image_type *images[], int nImage,
					   palette_type *palette, int nColor );


	/* ImageDifference.c */
void	image_difference_rectangle( image_type *im0, image_type *im1,
							int *i0, int *i1, int *j0, int *j1 );

void	image1_difference_rectangle( image_type *im0, image_type *im1,
							int *i0, int *i1, int *j0, int *j1 );


void	image_color_rectangle( image_type *im, int color,
							  int *i0, int *i1, int *j0, int *j1 );



	/* ImageExtend.c */
image_type *	image_extend( image_type *im, int row, int column, int color, image_type *rim );
image_type *	image_border_rect(image_type * im, int backcolor, int cycle, 
				  int r0, int c0, int row0, int col0);


	/* ImageEnhancement.c */
#define ENHANCEMENT_OBJECT		0
#define ENHANCEMENT_TEXT		1
#define ENHANCEMENT_BACKGROUND	2

image_type *	image_enhancement( image_type *sim, int enhan_type );


	/* ImageGrayEnhancement.c */
image_type *	image_gray_enhancement( image_type *sim, int enhan_type );


	/* ImageNegative.c		   */
void image_negative( image_type *im );


	/* ImageResize.c		   */
image_type *	image_resize( image_type *imd, image_type *ims, int row, int col );

	/* ImageHsiTune.c*/
image_type *	image_HSI_tune( image_type *sim, float HUE, float Saturation, float Intensity );
float	image_Saturation( image_type *sim );


#endif
