// VimageR2.cpp: implementation of the CVimageR2 class.
//
//////////////////////////////////////////////////////////////////////
#include <string.h>

extern "C" {
#include	"Uigp/igp.h"
#include	"ImageType/ImageType.h"
//#include	"ImageIo/ImageIo.h"
#include	"GpType/GpType.h"
#include	"VimageType/VimageType.h"
//#include	"VimageR2/VimageR2Type.h"
#include	"VimCoding\VimCoding.h"
#include	"GpType/Configuration.h"
}



#include "St.h"




//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CSt::CSt()
{
	m_hvim = NULL;

	depth_expand_init();
}

CSt::~CSt()
{
vimage_type	*vim = (vimage_type *)m_hvim;

	if( vim != NULL )
		vimage_destroy( vim );

}


int CSt::GetFrameWidth()
{
vimage_type	*vim = (vimage_type*)m_hvim;

	if( vim == NULL )	return( -1 );

	return( vim->column );
}


int CSt::GetFrameHeight()
{
vimage_type	*vim = (vimage_type*)m_hvim;

	if( vim == NULL )	return( -1 );

	return( vim->row );
}




int CSt::Decode( char *file )
{
vimage_type	*vim = (vimage_type*)m_hvim;
vimage_type	*nvim;

	if( vim_decoding_from_file( &nvim, 1, NULL, 0x01, 1, file ) < 0 )
		return( -1 );

	if( nvim == NULL )	
		return( -1 );


	if( vim != NULL )	vimage_destroy( vim );

	m_hvim = nvim;


	m_Frames = nvim->frame;
	m_iFrame = -1;


	SetFrameRate( nvim->frameRate );


	return( 1 );
}


int CSt::Decode( char *data, int dataSize )
{
vimage_type	*vim = (vimage_type*)m_hvim;
vimage_type	*nvim;
vimHeader_type	*nvh;


	if( vim_decoding_from_buffer( &nvim, 1, &nvh, 0x01, 1, data, dataSize ) < 0 )
		return( -1 );


	vimHeader_free( nvh );


	if( vim != NULL )	vimage_destroy( vim );
	m_hvim = nvim;


	m_Frames = nvim->frame;
	m_iFrame = -1;

	SetFrameRate( nvim->frameRate );

	return( 1 );
}






image_type *
CSt::Render( int frame, image_type *im )
{
vimage_type	*vim = (vimage_type*)m_hvim;


	vimage_configuration( vim, frame );

	if( im != NULL && ( im->row != vim->Srow || im->column != vim->Scolumn) ){
		image_destroy( im, 1 );
		im = NULL;
	}

	if( im == NULL ){
		im = image_create( vim->Srow, vim->Scolumn, 4, 1, NULL );
		vim->Sbr = 0;
		vim->expA->asbr->Fclear = 1;
	}


	vimage_render( vim, vim->ragp );


	depth_expand_agp( vim->ragp, vim->expA, vim->Sbr, vim->bim->color, im );


	m_iFrame = vim->frame_i;


//	image_write_bmp( im, "d:/A/mp4", "sss.bmp" );

	return( im );
}




void
CSt::SetTranslation( vec3d av[], int len )
{
vimage_type	*vim = (vimage_type*)m_hvim;
cast_type *cast;
int	i,	j;

	j = 0;
	if( j < len ){
		vim->Gcp.x = av[j].x;
		vim->Gcp.y = av[j].y;
		vim->Gcp.z = av[j].z;
		j++;
	}



	for( i = 0 ; i < vim->no && j < len ; i++ ){
		cast = vim->cast[i];

		cast->oim->Gcp.x = av[j].x;
		cast->oim->Gcp.y = av[j].y;
		cast->oim->Gcp.z = av[j].z;

		j++;
	}

	vimage_anim_set_aux( vim );
}



void
CSt::SetRotation( vec3d av[], int len )
{
vimage_type	*vim = (vimage_type*)m_hvim;
cast_type *cast;
int	i,	j;

	j = 0;

	if( j < len ){
		vim->Gcp.u1 = av[j].x;
		vim->Gcp.u2 = av[j].y;
		vim->Gcp.u3 = av[j].z;
		j++;
	}


	for( i = 0 ; i < vim->no && j < len ; i++ ){
		cast = vim->cast[i];

		cast->oim->Gcp.u1 = av[j].x;
		cast->oim->Gcp.u2 = av[j].y;
		cast->oim->Gcp.u3 = av[j].z;
		j++;
	}

	vimage_anim_set_aux( vim );
}


void
CSt::SetGlobalConfiguration( vec3d av[], int len )
{
vimage_type	*vim = (vimage_type*)m_hvim;
cast_type *cast;
int	i,	j;

	j = 0;
	if( j < len ){
		vim->Gcp.x = av[j].x;
		vim->Gcp.y = av[j].y;
		vim->Gcp.z = av[j].z;
		j++;
	}

	if( j < len ){
		vim->Gcp.u1 = av[j].x;
		vim->Gcp.u2 = av[j].y;
		vim->Gcp.u3 = av[j].z;
		j++;
	}


	for( i = 0 ; i < vim->no && j < len ; i++ ){
		cast = vim->cast[i];

		cast->oim->Gcp.x = av[j].x;
		cast->oim->Gcp.y = av[j].y;
		cast->oim->Gcp.z = av[j].z;

		j++;
		if( j>= len )	break;

		cast->oim->Gcp.u1 = av[j].x;
		cast->oim->Gcp.u2 = av[j].y;
		cast->oim->Gcp.u3 = av[j].z;
		j++;

		if( j >= len ) break;
	}
}