/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**************************
 ***	BstTool.c	***
 **************************/
#include	<stdio.h>
#include	"Uigp/igp.h"

#include	"BstType.h"


bst_type *
bst_alloc( int byte_no )
{
bst_type *bst;

	bst = (bst_type *)malloc( sizeof(bst_type) );

	bst->BYTE_NO = byte_no;
	if( bst->BYTE_NO > 0 )
		bst->data = (u_char *) calloc( bst->BYTE_NO, 1 );
	else	bst->data = NULL;

	bst->byte_no = 0;

	bst->byte_i = 0;
	bst->byte_ptr =  &bst->data[bst->byte_i];
	bst->MaskIndex = 0;
	bst->mask = 0x80;

	return( bst );
}


void
bst_realloc( bst_type *bst, int byte_no )
{
	if( byte_no > bst->BYTE_NO ){
		free( bst->data );
		bst->BYTE_NO = byte_no;
		bst->data = (u_char *) calloc( bst->BYTE_NO, 1 );
	}

	bst->byte_no = 0;

	bst->byte_i = 0;
	bst->byte_ptr =  &bst->data[bst->byte_i];
	bst->MaskIndex = 0;
	bst->mask = 0x80;
}


void
bst_extend(bst_type *bst, int byte_extention)
{
	u_char* buf = bst->data;
	int old_byte_no = bst->BYTE_NO;
	if ( byte_extention > 0 )
	{
		bst->BYTE_NO += byte_extention;
		bst->data = (u_char *) calloc( bst->BYTE_NO, 1 );
		memcpy(bst->data,buf,old_byte_no);
		free(buf);
		bst->byte_ptr =  &bst->data[bst->byte_i];
	}
}


void
bst_rewind( bst_type *bst )
{
	bst->byte_i = 0;

        bst->byte_ptr =  &bst->data[0];
        bst->MaskIndex = 0;
        bst->mask = 0x80; 
}


bst_type *
bst_read( FILE *fp, int byte_no )
{
bst_type	*bst;

	bst = bst_alloc( byte_no );

	bst->byte_no = fread( bst->data, 1, byte_no, fp );

	bst_rewind( bst );


	return( bst );
}


void
bst_write( FILE *fp, bst_type *bst )
{
int	byte;

	byte = ( bst->MaskIndex == 0 )? bst->byte_i : bst->byte_i +1;

	fwrite( bst->data, 1, byte, fp );

}

/*
void
bst_copy( bst_type *bst_in, bst_type * bst_out, int byte_no)
{
	bst_out->byte_no = byte_no;
	memcpy(bst_out->data, bst_in->byte_ptr, byte_no);
	bst_in->byte_i += byte_no;
	bst_in->byte_ptr += byte_no;
}
*/

void
bst_clear( bst_type *bst )
{
	bst->byte_no = 0;
	bst->byte_i = 0;
	bst->byte_ptr =  &bst->data[bst->byte_i];
	bst->MaskIndex = 0;
}



void
bst_destroy( bst_type *bst )
{
	free( bst->data );

	free( bst );
}




int
bst_write_to_file( bst_type *bst, char *file )
{
FILE	*fp;

        if( (fp = fopen( file, "wb" )) == NULL )	return( -1 );

        bst_write( fp, bst );
        fclose( fp );

	return( 1 );
}


bst_type *
bst_read_from_file( char *file )
{
bst_type	*bst;
FILE	*fp;
int	byte_no;


	if( (byte_no = gp_file_size(file)) == -1 )	return( NULL );

	if( (fp = fopen( file, "rb" )) == NULL )	return( NULL );


	bst = bst_read( fp, byte_no );

	fclose( fp );

	return( bst );
}


bst_type *
bst_create( char data[], int byte_no, int FCopydata )
{
bst_type	*bst;

	
	
	if( FCopydata  == 1 ){
		bst = bst_alloc(byte_no);
		memcpy( bst->data, data, byte_no );
	}
	else	
	{
		bst = (bst_type *)malloc( sizeof(cst_type) );
		bst->data = (unsigned char *)data;
		bst->BYTE_NO = byte_no;
	}

	bst->byte_no = byte_no;



	bst->byte_i = 0;
	bst->byte_ptr =  &bst->data[bst->byte_i];
	bst->MaskIndex = 0;
	bst->mask = 0x80;


	return bst;
}



bst_type *
bst_make_copy( bst_type *bst_in )
{
bst_type	*bst;
int	bytes;

	bytes = MAX( BST_BYTES(bst_in), bst_in->byte_no );

	bst = bst_alloc( bytes + 1 );

	memcpy( bst->data, bst_in->data, bytes);
	bst->byte_no = bytes;


	bst->byte_i = 0;
	bst->byte_ptr =  &bst->data[bst->byte_i];
	bst->MaskIndex = 0;
	bst->mask = 0x80;


	return bst;

}