/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**********************
 ***	CstTool.c   ***
 **********************/
#include	<stdio.h>
#include	<stdlib.h>
#include	"Uigp/igp.h"

#include	"CstType.h"
#include	"Ubst/BstType.h"

cst_type *
cst_alloc( int byte_no )
{
cst_type *cst;

	cst = (cst_type *)malloc( sizeof(cst_type) );

	cst->BYTE_NO = byte_no;
	if( cst->BYTE_NO > 0 )
		cst->data = (u_char *) calloc( cst->BYTE_NO, 1 );
	else	cst->data = NULL;

	cst->byte_no = 0;

	cst->byte_i = 0;
	cst->byte_ptr =  &cst->data[cst->byte_i];

	return( cst );
}


void
cst_realloc( cst_type *cst, int byte_no )
{
	if( byte_no > cst->BYTE_NO ){
		free( cst->data );
		cst->BYTE_NO = byte_no;
		cst->data = (u_char *) calloc( cst->BYTE_NO, 1 );
	}

	cst->byte_no = 0;

	cst->byte_i = 0;
	cst->byte_ptr =  &cst->data[cst->byte_i];
}


void
cst_extend(cst_type *cst, int byte_extention)
{
	u_char* buf = cst->data;
	int location = (int)(cst->byte_ptr - cst->data);
	int old_byte_no = cst->BYTE_NO;
	if ( byte_extention > 0 )
	{
		cst->BYTE_NO += byte_extention;
		cst->data = (u_char *) calloc( cst->BYTE_NO, 1 );
		memcpy(cst->data,buf,old_byte_no);
		free(buf);
		cst->byte_ptr =  &cst->data[location];
	}
}
void
cst_read( FILE *fp, cst_type *cst, int byte_no )
{
	cst_realloc( cst, byte_no );

	cst->byte_no = fread( cst->data, 1, byte_no, fp );

	cst->byte_i = 0;
	cst->byte_ptr =  &cst->data[cst->byte_i];
}


void
cst_write( FILE *fp, cst_type *cst )
{
	fwrite( cst->data, 1, cst->byte_no, fp );
}


void
cst_clear( cst_type *cst )
{
	cst->byte_no = 0;
	cst->byte_i = 0;
	cst->byte_ptr =  &cst->data[cst->byte_i];
}



void
cst_free( cst_type *cst )
{
	free( cst->data );

	free( cst );
}


cst_type *
cst_load_from_file( char *file )
{
cst_type	*cst;
FILE	*fp;
int	byte_no;

	byte_no = gp_file_size( file );

	if ( (fp = fopen ( file, "rb" )) == NULL )
		return( NULL );

	cst = cst_alloc( byte_no );
	cst_read( fp, cst, byte_no );

	fclose( fp );

	return( cst );
}


int
cst_write_to_file( cst_type *cst, char *file )
{
FILE	*fp;

        if( (fp = fopen( file, "wb" )) == NULL )	return( -1 );

        cst_write( fp, cst );
        fclose( fp );

	return( 1 );
}


void
cst_rewind( cst_type *cst )
{
	cst->byte_i = 0;
	cst->byte_ptr =  &cst->data[cst->byte_i];
}


int
cst_get_buffer( cst_type *cst, u_char buf[], int byte_no )
{
int	i;

	if( CST_BYTE_INDEX(cst) + byte_no > cst->byte_no )
		byte_no = cst->byte_no - CST_BYTE_INDEX( cst );

	for( i = 0 ; i < byte_no ; i++ )
		buf[i] = CST_GET_UCHAR( cst );

	return( byte_no );
}



int
cst_put_buffer_int( cst_type *cst, u_char buf[], int byte_no )
{
	int	i;
	for( i = 0 ; i < byte_no ; i++ )
		CST_PUT_CHAR( cst , buf[i]);

	cst->byte_no = cst->byte_ptr - cst->data;
	return byte_no;
}





int
cst_put_buffer( cst_type *cst, char buf[], int byte_no )
{
char *p;
int	i;

	for( p= buf, i = 0 ; i < byte_no ; i++, p++ )
		CST_PUT_CHAR( cst , *p);

	cst->byte_no = cst->byte_ptr - cst->data;

	return byte_no;
}

int
cst_put_string( cst_type *cst, char buf[] )
{
char *p;
	if (!buf)
		return (1);
	for( p = buf ; *p != 0 ; p++ )
		CST_PUT_CHAR( cst , *p);

	return( 1 );
}


void	
cst_put_stringA( cst_type *cst, int align, char *name )
{
char *p;
int	i;

	for( i = 0 ; i < align ; i++ )
		CST_PUT_CHAR( cst , '\t' );

	for( p = name ; *p != 0 ; p++ )
		CST_PUT_CHAR( cst , *p);
}



void	
cst_put_xml_tag_int( cst_type *cst, char *name, int value )
{
char	buf[100];

	sprintf( buf, "<%s>%d</%s>\n", name, value, name );
	cst_put_string( cst, buf );
}

void	
cst_put_xml_tag_string( cst_type *cst, char *name, char *value )
{
char buf [256];
	if (value)
		sprintf( buf, "<%s>%s</%s>\n", name, value, name );
	else
		sprintf( buf, "<%s>0</%s>\n", name,name );
	cst_put_string( cst, buf );
}

void	
cst_put_xml_tag_stringS( cst_type *cst, char *name, char *value, int byte_no )
{
char	buf[256];

	sprintf( buf, "<%s>\n", name );
	cst_put_string( cst, buf );

	cst_put_buffer( cst, value, byte_no );


	sprintf( buf, "</%s>\n", name );
	cst_put_string( cst, buf );
}

void	
cst_put_xml_att_string( cst_type *cst, char *name, char *value )
{
char	*p;
	CST_PUT_CHAR( cst , ' ' );
	for( p = name ; *p != 0 ; p++ )
		CST_PUT_CHAR( cst , *p);

	CST_PUT_CHAR( cst , '=' );

	CST_PUT_CHAR( cst , '\"' );
	for( p = value ; *p != 0 ; p++ )
		CST_PUT_CHAR( cst , *p);

	CST_PUT_CHAR( cst , '\"' );
}


void	
cst_put_xml_att_int( cst_type *cst, char *name, int value, char *ext )
{
char	buf[64],	*p;
	CST_PUT_CHAR( cst , ' ' );
	for( p = name ; *p != 0 ; p++ )
		CST_PUT_CHAR( cst , *p);

	CST_PUT_CHAR( cst , '=' );

	CST_PUT_CHAR( cst , '\"' );

//	itoa( value, buf, 10 );
	sprintf( buf, "%d", value );
	for( p = buf ; *p != 0 ; p++ )
		CST_PUT_CHAR( cst , *p);

	if( ext != NULL )
		for( p = ext ; *p != 0 ; p++ )
			CST_PUT_CHAR( cst , *p);

	CST_PUT_CHAR( cst , '\"' );
}


void	
cst_put_xml_tag_openA( cst_type *cst, int align, char *name )
{
char	buf[100];
int	i;

	for( i = 0 ; i < align ; i++ )	buf[i] = '\t';

	sprintf( buf+align, "<%s>\n", name );
	cst_put_string( cst, buf );
}

void	
cst_put_xml_tag_closeA( cst_type *cst, int align, char *name )
{
char	buf[100];
int	i;

	for( i = 0 ; i < align ; i++ )	buf[i] = '\t';

	sprintf( buf+align, "</%s>\n", name );
	cst_put_string( cst, buf );
}


void	
cst_put_xml_tag_intA( cst_type *cst, int align, char *name, int value )
{
char	buf[100];
int	i;

	for( i = 0 ; i < align ; i++ )	buf[i] = '\t';

	sprintf( buf+align, "<%s>%d</%s>\n", name, value, name );
	cst_put_string( cst, buf );
}

void	
cst_put_xml_tag_floatA( cst_type *cst, int align, char *name, float value )
{
char	buf[100];
int	i;

	for( i = 0 ; i < align ; i++ )	buf[i] = '\t';

	sprintf( buf+align, "<%s>%.4f</%s>\n", name, value, name );
	cst_put_string( cst, buf );
}


void	
cst_put_xml_tag_stringA( cst_type *cst, int align, char *name, char *value )
{
//char	buf[100];
int	i;

	for( i = 0 ; i < align ; i++ )
		CST_PUT_CHAR( cst , '\t' );

	CST_PUT_CHAR( cst , '<' );
	cst_put_string( cst, name );
	CST_PUT_CHAR( cst , '>' );

	cst_put_string( cst, value );


	CST_PUT_CHAR( cst , '<' );
	CST_PUT_CHAR( cst , '//' );
	cst_put_string( cst, name );
	CST_PUT_CHAR( cst , '>' );
	CST_PUT_CHAR( cst , '\n' );
}



void	
cst_put_xml_tag_openB( cst_type *cst,char *name )
{
char	buf[100];

	sprintf( buf, "<%s>", name );
	cst_put_string( cst, buf );
}

void	
cst_put_xml_tag_closeB( cst_type *cst, char *name )
{
char	buf[100];

	sprintf( buf, "</%s>", name );
	cst_put_string( cst, buf );
}


void	
cst_put_xml_tag_intB( cst_type *cst, char *name, int value )
{
char	buf[100];


	sprintf( buf, "<%s>%d</%s>", name, value, name );
	cst_put_string( cst, buf );
}

void	
cst_put_xml_tag_floatB( cst_type *cst, char *name, float value )
{
char	buf[100];


	sprintf( buf, "<%s>%.4f</%s>", name, value, name );
	cst_put_string( cst, buf );
}


void	
cst_put_xml_tag_stringB( cst_type *cst, char *name, char *value )
{
	CST_PUT_CHAR( cst , '<' );
	cst_put_string( cst, name );
	CST_PUT_CHAR( cst , '>' );

	cst_put_string( cst, value );


	CST_PUT_CHAR( cst , '<' );
	CST_PUT_CHAR( cst , '//' );
	cst_put_string( cst, name );
	CST_PUT_CHAR( cst , '>' );
}

cst_type *
cst_create( char data[], int byte_no, int FCopydata )
{
cst_type	*cst;

	
	
	if( FCopydata  == 1 ){
		cst = cst_alloc(byte_no);
		memcpy( cst->data, data, byte_no );
	}
	else	
	{
		cst = (cst_type *)malloc( sizeof(cst_type) );
		cst->data = (unsigned char *)data;
	}
	cst->BYTE_NO = byte_no;


	cst->byte_no = cst->BYTE_NO;

	cst->byte_i = 0;
	cst->byte_ptr =  &cst->data[cst->byte_i];


	return cst;
}



void
cst_destroy( cst_type *cst, int FDeletedata )
{
	if( FDeletedata )
		free( cst->data );

	free( cst );
}


struct bst_type *cst2bst(cst_type* cst, int byte_no)
{
	bst_type *bst = bst_alloc( byte_no );

	memcpy(bst->data,cst->data, byte_no );
	bst->byte_no = bst->BYTE_NO;
	return bst;
}
