/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
#include	<stdio.h>

#ifdef CE
#include <windows.h>
#else
#include	<sys/types.h>
#include	<sys/stat.h>
#ifdef  WIN32
#include	<direct.h>
#else
#include	<dirent.h>
#endif
#endif

#include	<string.h>





#include	"igp.h"






int
gp_filename_extand( char *file1, char *file2 )
{
int	len1,	len2,	k,	k2;


	len1 = strlen( file1 );
	len2 = strlen( file2 );

	if( len2 == 1 ){
		if( strncmp( file2, ".", 1 ) )	return( 0 );
		k2 = 0;
	}
	else	{
		if( strncmp( &file2[len2-2], "/.", 2 ) )	return( 0 );
		k2 = len2-1;
	}

	for( k = len1-1 ; k >= 0 && file1[k] != '/' ; k-- );

	strcpy( &file2[k2], &file1[k+1] );

	return( 1 );
}


int
gp_string_cut_extension( char *s1, char *extansion, char *s2 )
{
int	len,	n;
	len = strlen( s1 );
	n = strlen( extansion );

	strcpy( s2, s1 );

	if( strcmp( &s1[len-n], extansion ) != 0 )
		return( 0 );

	s2[len-n] = '\0';
	return( 1 );
}





/*********  gp_filename_split  **********
* @Description	:  splits a filename (dir/subdir/file.name.ext) 
*					into it's directory (dir/subdir) it's base name (file.name)
*					and it's extension (.ext)
* 
* @Param   char *fname :  path
* @Param  char *dir :  buffer for the directory
* @Param  char *name :  buffer for the base name
* @Param  char *extension :  buffer for the extension
* @Return  void  :  
*****************************************/
void gp_filename_split( char *fname, char *dir, char *name, char *extension )
{
	gp_filename_extract_name(fname, name, 0);
	gp_filename_extract_extension(fname, extension, 0);	
	gp_filename_extract_dir(fname, dir, 0);
}




/*********  gp_filename_extract_dir  **********
* @Description	:  gets the directory of the filename
* 
* @Param  char *fname :  path
* @Param  char *dir :  buffer for the directory
* @Param  int len :  length of that buffer
* @Return  void  :  
*****************************************/
void gp_filename_extract_dir(char *fname, char *dir, int len)
{
char *slash, *backslash, *end;
int	dirlength;


	if ( len == 1)
	{
		dir = '\0';
		return;
	}

	// get dir delimiter
	slash = strrchr(fname, (int)'/');
	backslash = strrchr(fname, (int)'\\');

	// get dir delimiter
	end = NULL;
	if ( slash == NULL )
	{
		if ( backslash == NULL)
		{
			end = NULL;
		}		
		else
			end = backslash + 1;
	}
	else if ( backslash == NULL)
	{
		end = slash + 1;
	}
	else
	{
		if ( (int)(slash - backslash) > 0)
			end = slash + 1;
		else
			end = backslash + 1;
	}


	if ( end != NULL)
	{
		dirlength = (int)(end - fname);
		if ( len > 1 && dirlength > len)
			dirlength = len;
		dirlength--;
		strncpy(dir, fname, dirlength);
		dir[dirlength] = '\0';
	}
	else
	{
		strcpy(dir, ".");
	}
	
	
}

/*********  gp_filename_extract_name  **********
* @Description	:  gets the filename (without extension) out of the name
* 
* @Param   char *fname :a name to analyze  
* @Param  char *name :  result filename
* @Param  int len :  length of the name buffer
* @Return  void  :  
*****************************************/

void gp_filename_extract_name( char *fname, char *name, int len )
{
int	fnameLength,  nameLength;
char *lastpoint;
char *slash, *backslash, *start = NULL;
	
	if ( len == 1 )
	{
		name[0] = '\0';
		return;
	}
	fnameLength = strlen( fname );	

	// get dir delimiter
	slash = strrchr(fname, (int)'/');
	backslash = strrchr(fname, (int)'\\');
	// get extension delimiter
	lastpoint = strrchr(fname,(int)'.');
	//firstpoint = strchr(fname,(int)'.');
	


	// get dir delimiter
	if ( slash == NULL )
	{
		if ( backslash == NULL)
			start = fname ;
		else
			start = backslash + 1;
	}
	else if ( backslash == NULL)
	{
		start = slash + 1;
	}
	else
	{
		if ( (int)(slash - backslash) > 0)
			start = slash + 1;
		else
			start = backslash + 1;
	}

	if ( lastpoint == NULL || lastpoint < start)
	{
		nameLength = fnameLength - (int)(start - fname);
	}
	else
	{		
		nameLength = (int)(lastpoint - start);
	}


	if ( len > 1 && nameLength > len - 1)
		nameLength = len - 1;

	if( nameLength > 0)
	{
		strncpy(name,start, nameLength);
		name[nameLength] = '\0';
	}
	else
		name[0] = '\0';

}



void gp_filename_extract_FullName( char *fname, char *name, int len )
{
int	fnameLength;
char	*start,	*p;

	name[0] = 0;
	fnameLength = strlen( fname );
	if( fnameLength <= 0 )	return;
	

	start = fname;
	for( p = &fname[fnameLength-1] ; p != start ; p-- )
		if( *p == '/' || *p == '\\' ){
			start = p+1;
			break;
		}


	strcpy( name, start);
}



/*********  gp_filename_extract_extension  **********
* @Description	:  gets the extension out of the filename
* 
* @Param   char *fname :  the filename
* @Param  char *extension :  buffer for the extension (".txt")
* @Param  int len :  length of the buffer
* @Return  void  :  
*****************************************/
void gp_filename_extract_extension( char *fname, char *extension, int len )
{
int n;
char *start;
	extension[0] = '\0';

	start = strrchr(fname,(int)'.');
	if ( start == NULL || len == 1)
	{
		extension[0] = '\0';
	}
	else
	{
		n = strlen(start);
		if ( len > 1 && n > len - 1)
			n = len - 1;
		strncpy(extension, start, n);
		extension[n] = '\0';
	}
	
}


void
gp_filename_force_extension( char *file, char *extension )
{
char	dir[256],	name[256],	ext[256];

	gp_filename_split( file, dir, name, ext );

	sprintf( file, "%s/%s%s", dir, name, extension );
}



void 
gp_filename_slash( char *fname )
{
char	*sp,	*tp;

	if( fname[0] == '\\' && fname[1] == '\\' )
		sp = &fname[2];
	else	sp = fname;


	for( tp = sp ; *sp != 0 ; sp++ ){
		if( *sp != '\\' && *sp != '/' ){
			*tp++ = *sp;
			continue;
		}
		*tp++ = '/';

		if( *(sp+1) == '\\' || *(sp+1) == '/' )	sp++;
	}

	*tp = 0;
}
