/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**************************
 ***	ViewCreate.c	***
 *************************/

#include	<math.h>

#include	"Uigp/igp.h"
#include	"Uview/ViewType.h"



static void	view_create_normal_vec( vec3d *cp, float Lv,
						vec2d *n, float *lv );


void
view_create( view_type *vw, vec3d *cp, float Lv, int row, int column )
{
float	lv,	lu,	t;
vec2d	n;

	vw->type = VIEW_PERSPECTIVE;
	vw->Upixel = column;
	vw->Vpixel = row;


	vw->cp.x = 0;
	vw->cp.y = cp->y;
	vw->cp.z = cp->z;
	vw->dis = 1;


	view_create_normal_vec( cp, Lv, &n, &lv );

	vw->N.x = 0;
	vw->N.y = n.x;
	vw->N.z = n.y;

	t = 1/ VEC3D_NORM( vw->N );
	vw->N.x *= t;
	vw->N.y *= t;
	vw->N.z *= t;


	if( ABS( vw->N.z ) < 0.8 ){
		vw->V.x = 0;
		vw->V.y = 0;
		vw->V.z = 1;
	}
	else	{
		vw->V.x = 0;
		vw->V.y = 1;
		vw->V.z = 0;
	}



	vw->uv.y0 = -0.5 *lv;
	vw->uv.y1 = 0.5 *lv;

	lu = lv * ((float)vw->Upixel/ (float)vw->Vpixel);
	vw->uv.x0 = -0.5 *lu;
	vw->uv.x1 = 0.5 *lu;


	view_per_set( vw, &vw->cp, 1, &vw->N, 1, &vw->V, 1 );
	view_set_pixel( vw, row, column );
}



static void
view_create_normal_vec( vec3d *cp, float Lv, vec2d *n, float *lv )
{
vec2d	v0,	v1;
float	t;

	v0.x = -Lv - cp->y;
	v0.y = 0 - cp->z;

	t = hypot( v0.x, v0.y );
	v0.x /= t;
	v0.y /= t;


	v1.x = Lv - cp->y;
	v1.y = 0 - cp->z;

	t = hypot( v1.x, v1.y );
	v1.x /= t;
	v1.y /= t;



	n->x = v0.x + v1.x;
	n->y = v0.y + v1.y;

	t = hypot( n->x, n->y );
	n->x /= t;
	n->y /= t;


	t = 1.0 / ( v1.x *n->x + v1.y *n->y );
	*lv = 2.0 * sqrt( t*t -1 );
}



/*
void
view_create_noraml( view_type *vw, float rf, int row, int column )
{
	vw->type = VIEW_PERSPECTIVE;
	vw->Upixel = column;
	vw->Vpixel = row;

        vw->cp.x = 0;
        vw->cp.y = 0;
        vw->cp.z = 1.0;
 
        vw->rp.x = 0;
        vw->rp.y = 0;
        vw->rp.z = 0;

	vw->dis = 1.0;
 
 
        VEC3D_SET( vw->N, 0, 0, -1 );
        VEC3D_SET( vw->V, 0, 1, 0 );
        VEC3D_SET( vw->U, 1, 0, 0 );



	vw->uv.y0 = -0.5 * rf * vw->Vpixel;
	vw->uv.y1 =  0.5 * rf * vw->Vpixel;

	vw->uv.x0 = -0.5 * rf * vw->Upixel;
	vw->uv.x1 =  0.5 * rf * vw->Upixel;

	view_per_set( vw, &vw->cp, 1, &vw->N, 1, &vw->V, 1 );
	view_set_pixel( vw, row, column );
}
*/
