/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/********************************************************
*							*
*	ViewPerNormal.c :  				*
*							*
********************************************************/
#include	"Uvec/Vec3d.h"
#include	"Ubox/Box2d.h"
#include	"Ujet2/Jet2Type.h"
#include	"Ujet2/Jet23Type.h"

#include	"ViewType.h"


void
view_per_normal_base( view_type *bv, view_type *nbv )
{

	*nbv = *bv;

	nbv->cp.x = 0;
	nbv->cp.y = 0;
	nbv->cp.z = ( nbv->type == VIEW_PERSPECTIVE ) ? bv->dis : 0;

	nbv->rp.x = 0;
	nbv->rp.y = 0;
	nbv->rp.z = 0;


	VEC3D_SET( nbv->N, 0, 0, -1 );
	VEC3D_SET( nbv->V, 0, 1, 0 );
	VEC3D_SET( nbv->U, 1, 0, 0 );
}


#pragma optimize( "", off )
void
view_per_normal( view_type *base, view_type *vw, view_type *nvw )
{
vec3d	dr,	cp;

	*nvw = *vw;

	nvw->N.x = VEC3D_INNER( base->U, vw->N );
	nvw->N.y = VEC3D_INNER( base->V, vw->N );
	nvw->N.z = -VEC3D_INNER( base->N, vw->N );

	nvw->V.x = VEC3D_INNER( base->U, vw->V );
	nvw->V.y = VEC3D_INNER( base->V, vw->V );
	nvw->V.z = -VEC3D_INNER( base->N, vw->V );


	dr.x = vw->rp.x - base->rp.x;
	dr.y = vw->rp.y - base->rp.y;
	dr.z = vw->rp.z - base->rp.z;

	nvw->rp.x = VEC3D_INNER( base->U, dr );
	nvw->rp.y = VEC3D_INNER( base->V, dr );
	nvw->rp.z = -VEC3D_INNER( base->N, dr );


	cp.x = vw->cp.x - base->rp.x;
	cp.y = vw->cp.y - base->rp.y;
	cp.z = vw->cp.z - base->rp.z;

	nvw->cp.x = VEC3D_INNER( base->U, cp );
	nvw->cp.y = VEC3D_INNER( base->V, cp );
	nvw->cp.z = -VEC3D_INNER( base->N, cp );
	

/*
	nvw->cp.x = nvw->rp.x - nvw->dis * nvw->N.x;
	nvw->cp.y = nvw->rp.y - nvw->dis * nvw->N.y;
	nvw->cp.z = nvw->rp.z - nvw->dis * nvw->N.z;
*/

	view_per_set( nvw, &nvw->cp, 1, &nvw->N, 1, &nvw->V, 1 );
}
#pragma optimize( "", on )
