/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/****************************************************************
*								*
*               Module:  ViewWin.h                              *
*								*
****************************************************************/

#include	<math.h>

#include	"Uigp/igp.h"
#include	"ViewType.h"


void
view_3d_point( view_type *vw, float u, float v, vec3d *p )
{
	p->x = vw->rp.x + u* vw->U.x + v* vw->V.x;
	p->y = vw->rp.y + u* vw->U.y + v* vw->V.y;
	p->z = vw->rp.z + u* vw->U.z + v* vw->V.z;
}


void
view_3d_vector( view_type *vw, float u, float v, vec3d *n )
{
	if( vw->type == VIEW_PARALLEL ){
		*n = vw->N;
		return;
	}

	n->x = vw->dis* vw->N.x + u* vw->U.x + v* vw->V.x;
	n->y = vw->dis* vw->N.y + u* vw->U.y + v* vw->V.y;
	n->z = vw->dis* vw->N.z + u* vw->U.z + v* vw->V.z;
}



void
view_3d_ray( view_type *vw, float u, float v, vec3d *p, vec3d *n )
{
	p->x = vw->rp.x + u* vw->U.x + v* vw->V.x;
	p->y = vw->rp.y + u* vw->U.y + v* vw->V.y;
	p->z = vw->rp.z + u* vw->U.z + v* vw->V.z;


	if( vw->type == VIEW_PARALLEL ){
		*n = vw->N;
		return;
	}

	n->x = vw->dis* vw->N.x + u* vw->U.x + v* vw->V.x;
	n->y = vw->dis* vw->N.y + u* vw->U.y + v* vw->V.y;
	n->z = vw->dis* vw->N.z + u* vw->U.z + v* vw->V.z;
}




void
view_cut_point( view_type *vw, vec3d *p, vec3d *rp, vec2d *v )
{
float	t,	ipn;
vec3d	n,	v3;
	n.x = rp->x - p->x;
	n.y = rp->y - p->y;
	n.z = rp->z - p->z;
	VEC3D_NORMAL( n );

	ipn = VEC3D_INNER( vw->N, n );

	t = ( VEC3D_INNER( vw->N, vw->rp ) - VEC3D_INNER( vw->N, *p ) )/ipn;
	v3.x = p->x + t*n.x;
	v3.y = p->y + t*n.y;
	v3.z = p->z + t*n.z;


	v->x = VEC3D_INNER( vw->U, v3 );
	v->y = VEC3D_INNER( vw->V, v3 );
}


void
view_cut_point_n( view_type *vw, vec3d *p, vec3d *n, vec2d *v )
{
float	t,	ipn;
vec3d	v3;

	ipn = VEC3D_INNER( vw->N, *n );

	t = ( VEC3D_INNER( vw->N, vw->rp ) - VEC3D_INNER( vw->N, *p ) )/ipn;
	v3.x = p->x + t*n->x;
	v3.y = p->y + t*n->y;
	v3.z = p->z + t*n->z;

/*	NEW 26-5 */
	v3.x -= vw->rp.x;
	v3.y -= vw->rp.y;
	v3.z -= vw->rp.z;
/*	END NEW */

	v->x = VEC3D_INNER( vw->U, v3 );
	v->y = VEC3D_INNER( vw->V, v3 );
}



int
view_cut_point_ray( view_type *vw, float depth, vec3d *p, vec3d *n, vec2d *v )
{
float	t,	ipn;
float	tv,	tu,	t0,	t1;
vec3d	v3;
vec2d	pp,	pn;

	ipn = VEC3D_INNER( vw->N, *n );

	t = ( VEC3D_INNER( vw->N, vw->rp )+depth - VEC3D_INNER( vw->N, *p ) )/ipn;
	if( t >= 0 ){
		v3.x = p->x + t*n->x;
		v3.y = p->y + t*n->y;
		v3.z = p->z + t*n->z;

/*
		v->x = VEC3D_INNER( vw->U, v3 );
		v->y = VEC3D_INNER( vw->V, v3 );
*/

		view_project_point( vw, &v3, v );

		return( 1 );
	}


	pn.x = VEC3D_INNER( vw->U, *n );
	pn.y = VEC3D_INNER( vw->V, *n );
	v3.x = p->x - vw->rp.x;
	v3.y = p->y - vw->rp.y;
	v3.z = p->z - vw->rp.z;

	pp.x = VEC3D_INNER( vw->U, v3 );
	pp.y = VEC3D_INNER( vw->V, v3 );


	tu = 10000;
	if( pn.x != 0 ){
		t0 = (vw->uv.x0 - pp.x )/pn.x;
		t1 = (vw->uv.x1 - pp.x )/pn.x;
		tu = MAX( t0, t1 );
		if( tu < 0 )	tu = 10000;
	}


	tv = 10000;
	if( pn.y != 0 ){
		t0 = (vw->uv.y0 - pp.y )/pn.y;
		t1 = (vw->uv.y1 - pp.y )/pn.y;
		tv = MAX( t0, t1 );
		if( tv < 0 )	tv = 10000;
	}

	t = MIN( tu, tv );
	if( t < 0 || t == 10000 )	return( 0 );


	v->x = pp.x + t * pn.x;
	v->y = pp.y + t * pn.y;

	return( 2 );
}



void
view_actoal_box( view_type *base, view_type *vw, box2d *box )
{
vec3d	n,	p;
vec2d	v;

	view_3d_ray( vw, vw->uv.x0, vw->uv.y0, &p, &n );
	view_cut_point_n( base, &p, &n, &v );
	BOX2D_INIT( *box, v.x, v.y );


	view_3d_ray( vw, vw->uv.x0, vw->uv.y1, &p, &n );
	view_cut_point_n( base, &p, &n, &v );
	BOX2D_UPDATE( *box, v.x, v.y );
 
	view_3d_ray( vw, vw->uv.x1, vw->uv.y0, &p, &n );
	view_cut_point_n( base, &p, &n, &v );
	BOX2D_UPDATE( *box, v.x, v.y );

	view_3d_ray( vw, vw->uv.x1, vw->uv.y1, &p, &n );
	view_cut_point_n( base, &p, &n, &v );
	BOX2D_UPDATE( *box, v.x, v.y );
}


void
view_actoal_pixel_box( view_type *base, view_type *vw, box2d *box )
{
vec3d	n,	rp;
vec2d	v,	pv;

	view_3d_vector( vw, vw->uv.x0, vw->uv.y0, &n );
	view_3d_point( vw, vw->uv.x0, vw->uv.y0, &rp );
	view_cut_point_n( base, &rp, &n, &v );
	pv.y = base->Ua*v.x + base->Ub;
	pv.x = base->Va*v.y + base->Vb;
	BOX2D_INIT( *box, pv.x, pv.y );


	view_3d_vector( vw, vw->uv.x0, vw->uv.y1, &n );
	view_3d_point( vw, vw->uv.x0, vw->uv.y1, &rp );
	view_cut_point_n( base, &rp, &n, &v );
	pv.y = base->Ua*v.x + base->Ub;
	pv.x = base->Va*v.y + base->Vb;
	BOX2D_UPDATE( *box, pv.x, pv.y );
 
	view_3d_vector( vw, vw->uv.x1, vw->uv.y0, &n );
	view_3d_point( vw, vw->uv.x1, vw->uv.y0, &rp );
	view_cut_point_n( base, &rp, &n, &v );
	pv.y = base->Ua*v.x + base->Ub;
	pv.x = base->Va*v.y + base->Vb;
	BOX2D_UPDATE( *box, pv.x, pv.y );

	view_3d_vector( vw, vw->uv.x1, vw->uv.y1, &n );
	view_3d_point( vw, vw->uv.x1, vw->uv.y1, &rp );
	view_cut_point_n( base, &rp, &n, &v );
	pv.y = base->Ua*v.x + base->Ub;
	pv.x = base->Va*v.y + base->Vb;
	BOX2D_UPDATE( *box, pv.x, pv.y );
}


void
view_actoal_depth_box( view_type *base, float depth,
				view_type *vw, box2d *box )
{
vec3d	n,	p;
vec2d	v;

	view_3d_ray( vw, vw->uv.x0, vw->uv.y0, &p, &n );
	view_cut_point_ray( base, depth, &p, &n, &v );
	BOX2D_INIT( *box, v.x, v.y );


	view_3d_ray( vw, vw->uv.x0, vw->uv.y1, &p, &n );
	view_cut_point_ray( base, depth, &p, &n, &v );
	BOX2D_UPDATE( *box, v.x, v.y );
 
	view_3d_ray( vw, vw->uv.x1, vw->uv.y0, &p, &n );
	view_cut_point_ray( base, depth, &p, &n, &v );
	BOX2D_UPDATE( *box, v.x, v.y );

	view_3d_ray( vw, vw->uv.x1, vw->uv.y1, &p, &n );
	view_cut_point_ray( base, depth, &p, &n, &v );
	BOX2D_UPDATE( *box, v.x, v.y );
}




void
view_actoal_depth_pixel_box( view_type *base, float depth0, float depth1,
						view_type *vw, box2d *box )
{
box2d	box0,	box1;

	view_actoal_depth_box( base, depth0, vw, &box0 );
	view_actoal_depth_box( base, depth1, vw, &box1 );

	BOX2D_UPDATE( box0, box1.x0, box1.y0 );
	BOX2D_UPDATE( box0, box1.x1, box1.y1 );

	box->y0 = base->Ua*box0.x0 + base->Ub;
	box->x0 = base->Va*box0.y1 + base->Vb;

	box->y1 = base->Ua*box0.x1 + base->Ub;
	box->x1 = base->Va*box0.y0 + base->Vb;
}





void
view_project_point( view_type *vw, vec3d *p, vec2d *v )
{
float	t;
vec3d	dv,	v3;


	dv.x = p->x - vw->cp.x;
	dv.y = p->y - vw->cp.y;
	dv.z = p->z - vw->cp.z;


	if( vw->type == VIEW_PARALLEL ){
		v->x = VEC3D_INNER( vw->U, dv );
		v->y = VEC3D_INNER( vw->V, dv );

		return;
	}



	t = ( VEC3D_INNER( vw->N, vw->rp ) - VEC3D_INNER( vw->N, vw->cp ) )/
			VEC3D_INNER( vw->N, dv );
	v3.x = vw->cp.x + t*dv.x;
	v3.y = vw->cp.y + t*dv.y;
	v3.z = vw->cp.z + t*dv.z;

	v3.x -= vw->rp.x;
	v3.y -= vw->rp.y;
	v3.z -= vw->rp.z;

	v->x = VEC3D_INNER( vw->U, v3 );
	v->y = VEC3D_INNER( vw->V, v3 );
}


float
view_project_depth( view_type *vw, float x0, float y0, float z0 )
{
vec3d	v;
float	depth;


	v.x = x0 - vw->cp.x;
	v.y = y0 - vw->cp.y;
	v.z = z0 - vw->cp.z;

	depth = v.x*v.x + v.y*v.y + v.z*v.z;

	return( depth );
}


void
view_vp2pixel( view_type *vw, vec2d *vp, vec2d *p )
{
	p->x = vw->Va *vp->y + vw->Vb;
	p->y = vw->Ua *vp->x + vw->Ub;
}


void
view_pixel2vp( view_type *vw, vec2d *p, vec2d *vp )
{
	vp->x = vw->Ua_inv*(p->y - vw->Ub);
	vp->y = vw->Va_inv*(p->x - vw->Vb);
}


void
view_set_XY( view_type *vw, float x, float y, float z )
{

	vw->type = VIEW_PARALLEL;
	VEC3D_SET( vw->rp, x, y, 0 );
	VEC3D_SET( vw->cp, x, y, z );
	vw->dis = z;

	VEC3D_SET( vw->N, 0, 0, -1 );
	VEC3D_SET( vw->V, 0, 1, 0 );
	VEC3D_SET( vw->U, 1, 0, 0 );

	BOX2D_SET( vw->uv, -1.0, 1.0, -1.0, 1.0 );
}



void
view_pixel2plane_xyz( view_type *vw, vec2d *p, vec3d *P, vec3d *N, vec3d *p3 )
{
vec2d   v;
vec3d   n,      p33;
float   t,      D;

	view_pixel2vp( vw, p, &v );


	view_3d_ray( vw, v.x, v.y, &p33, &n );

	D = VEC3D_INNER( *P, *N );
	t = ( D - VEC3D_INNER( vw->cp, *N ) ) / VEC3D_INNER( n, *N );

	p3->x = vw->cp.x + t *n.x;
	p3->y = vw->cp.y + t *n.y;
	p3->z = vw->cp.z + t *n.z;
}

void
view_pixel2planeD_xyz( view_type *vw, vec2d *p, vec3d *N, float D, vec3d *p3 )
{
vec2d   v;
vec3d   n,      p33;
float   t;

	view_pixel2vp( vw, p, &v );


	view_3d_ray( vw, v.x, v.y, &p33, &n );
/*
	D = VEC3D_INNER( *P, *N );
*/
	t = ( D - VEC3D_INNER( vw->cp, *N ) ) / VEC3D_INNER( n, *N );

	p3->x = vw->cp.x + t *n.x;
	p3->y = vw->cp.y + t *n.y;
	p3->z = vw->cp.z + t *n.z;
}

