/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/************************************************************************
*									*
*			ViewType.h					*
*									*
************************************************************************/

#ifndef		_VEIW_TYPE_
#define		_VEIW_TYPE_


#include	"Uigp/igp.h"
#include	"Uprm/prm.h"



//#include	"Ujet2/Jet2Type.h"
//#include	"Ujet2/Jet23Type.h"
#include	"Uvec/Vec3d.h"
#include	"Ubox/Box2d.h"
#include	"Uconfig3d/Config3d.h"



/* for view_type.type */
#define		VIEW_PARALLEL		1
#define		VIEW_PERSPECTIVE	2


typedef struct view_type {
	int	type;

	vec3d	rp;		/* point on the view plane */
	vec3d	N;		/* normal vector to the view plane */

	vec3d	cp;		/* Center of projection */

	float	dis;		/* distance of cp from the View Plane */
	float	angle;

	vec3d	U,	V;	/* View Plane Axis */
	box2d	uv;


	/* rescaling factor win2pixel */
	int	Upixel,	Vpixel;
	float	Ua,	Ub,	Ua_inv;
	float	Va,	Vb,	Va_inv;



	/* AUX */
	float	INcp,	IUcp,	IVcp;
} view_type ;



#define		VIEW_ALLOC()	( (view_type*)malloc( sizeof(view_type) ) )


	/* ViewLoad.c */
void	view_loads( char *dir, char *fname, int min, int max,
				view_type *vw[], int *vw_no );

void	view_load( char *file, view_type *vw );


	/* ViewDump.c */
void	view_dumps( char *dir, char *fname, view_type *vw[], int vw_no );

void	view_dump( char *dir, char *fname, view_type *vw );


	/* ViewPerPrm.c */
void	view_per_get_prm( GP_prm **prm, int *prm_no, void (**check)() );

void	view_per_init( char *file );

void	view_per_set_lprm();

void	view_per_set_in_prm( view_type *vw );


	/* ViewBasePrm.c */
void	view_base_get_prm( GP_prm **prm, int *prm_no, void (**check)() );

void	view_base_init( char *file );

void	view_base_set_lprm( );




	/* ViewPer.c */
void	view_per_set( view_type *vw, vec3d *cp, int Fpn, vec3d *pn,
					int Fv, vec3d *v, int Fw );

void 	view_set_pixel( view_type *vw, int row, int col );



	/* ViewJet.c */
/*
void	view_per_jmap22( view_type *vw,
				float x0, float y0,  jet2 *f, jmap22 *Q );

void	view_per_jmap23( view_type *vw,
				float x0, float y0, jet23 *f, jmap23 *Q );

void	view_per_z_jet2( view_type *vw, float x0, float y0, jet23 *f, jet2 *z);


void	view_per3_jmap23( view_type *vw,
			jet23 *X, jet23 *Y, jet23 *Z, jmap23 *Q );

void	view_per3_z_jet2( view_type *vw, jet23 *X, jet23 *Y, jet23 *Z, jet2 *z);
*/


	/* ViewPerNormal.c */
void	view_per_normal_base( view_type *bv, view_type *nbv );

void	view_per_normal( view_type *bv, view_type *vw, view_type *nvw );



	/* ViewTool.c */

void	view_project_point( view_type *vw, vec3d *p, vec2d *v );

float	view_project_depth( view_type *vw, float x0, float y0, float z0 );

void	view_3d_vector( view_type *vw, float u, float v, vec3d *n );

void	view_3d_point( view_type *vw, float u, float v, vec3d *p );

void	view_3d_ray( view_type *vw, float u, float v, vec3d *p, vec3d *n );

void	view_cut_point( view_type *vw, vec3d *p, vec3d *rp, vec2d *v );

void	view_cut_point_n( view_type *vw, vec3d *p, vec3d *n, vec2d *v );

int	view_cut_point_ray( view_type *vw, float depth,
					vec3d *p, vec3d *n, vec2d *v );


void	view_vp2pixel( view_type *vw, vec2d *vp, vec2d *p );

void	view_pixel2vp( view_type *vw, vec2d *p, vec2d *vp );

void	view_pixel2plane_xyz( view_type *vw, vec2d *p,
					vec3d *P, vec3d *N, vec3d *p3 );
void	view_pixel2planeD_xyz( view_type *vw, vec2d *p, vec3d *N, float D, vec3d *p3 );



void	view_actoal_box( view_type *base, view_type *vw, box2d *box );
void	view_actoal_pixel_box( view_type *base, view_type *vw, box2d *box );

void	view_actoal_depth_box( view_type *base, float depth,
				view_type *vw, box2d *box );

void	view_actoal_depth_pixel_box( view_type *base,
				float depth0, float depth1,
				view_type *vw, box2d *box );


void	view_set_XY( view_type *vw, float x, float y, float z );




	/* ViewSub.c */
void	view_rescale( view_type *vw, float fk );

void	view_sub_upixel( view_type *bvw, int pixel, view_type *vw );

void	view_crop_pixel( view_type *vw, int row, int column );

void	view_sub_pixel( view_type *bvw, int row, int column, view_type *vw );

void	view_extend_pixel( view_type *vw, int row, int column );

void	view_sub_box( view_type *bvw, box2i *box,
			view_type *vw, int *row0, int *col0 );



	/* ViewPlane.c */
void	view_plane_g2v( view_type *vw, vec3d *N, vec3d *P, plane3d *plane );



	/* ViewCreate.c */
void	view_create( view_type *vw, vec3d *cp, float Lv, int row, int column );

void	view_create_noraml( view_type *vw, float rf, int row, int column );


	/* ViewCamera.c */
void	view_camera_shift( view_type *bvw, float dn, float duv, config3d *cm,
                                                        view_type *vw );

	/* ViewObject.c */
void    view_object_shift( view_type *bvw, vec3d *P,
		float dn, float duv, config3d *cm, view_type *vw, vec3d *dp );

	/* ViewObject1.c */
void    view_object_shift1( view_type *bvw, vec3d *P, vec3d *N,
		float dn, float duv, config3d *cm, view_type *vw, vec3d *dp );
#endif
