/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**********************************
	PrimeDefaultBgp.c
**********************************/

#include        <stdio.h>   
#include	<math.h>

#include        "EpType/EpType.h"

#include        "Ep/Ep.h"

#include        "EpValue/EpValue.h"

#include        "EpDepth/DepthEpIn.h"

#include        "N3Prime/N3Prime.h"

#include        "PrimeIn.h"

#define		EXP_N		8


static void	prim_bgp_one( gp_array *agp, ep_array *aep );
static void	prime_bg_ep( gp_type *gp, ep_type *ep, int br,
						int bno, ep_array *aep );

static void	prime_bg_ep_add( gp_type *gp, ep_type *ep, int bno,
			float x, float y, int br, float depth,
			ep_array *aep );

void
prime_default_bgp( gp_array  *agp )
{
ep_array *aep;
int	i,	j;


	n3_prime_agp( agp );

	aep = aep_alloc();

        for( i = 1 ; i < agp->Ni-1 ; i += EXP_N ){
                for( j = 1 ; j < agp->Nj-1; j += EXP_N ){
 
                        depth_ep_create( agp, i, j, EXP_N, EXP_N, aep, 0 );

			prim_bgp_one( agp, aep );

			prime_default_bgp_1( agp, aep, 3 );
                }
        }

	aep_destroy( aep );

	n3_prime_free( agp );
}




static void
prim_bgp_one( gp_array *agp, ep_array *aep )
{
int     i,      j,	i1,	j1;
int	k,	br;
gp_type	*gp;
ep_type	*ep;

        for( i = LSIZE, i1 = aep->Ni0 ; i < aep->Ni-LSIZE ; i++, i1++ )
        	for( j = LSIZE, j1 = aep->Nj0 ; j < aep->Nj-LSIZE ; j++, j1++ ){

                	ep = &aep->ep[i][j];

			if( ep->bound[0]->br.no == ep->bound[ep->b_no-1]->br.no )
				continue;

                	gp = &agp->gp[i1][j1];


			br = ep->bound[0]->br.no;
			for( k = 1 ; k < ep->b_no ; k++ ){

				if( ep->bound[k]->br.no == br )	continue;

				br = ep->bound[k]->br.no;

				if( ep->bound[k]->br.flag == 1 ) continue;

				prime_bg_ep( gp, ep, br, k, aep );
			}
		}
}



static void
prime_bg_ep( gp_type *gp, ep_type *ep, int br, int bno, ep_array *aep )
{
bgp_type	*bgp;
float	hb,	depth;
int	bgp_in[4],	no;

	if( ep->bound[bno]->type & BOUND_ALL )
		bgp_in[0] = bgp_in[1] = bgp_in[2] = bgp_in[3] = 1;
	else	{
		bno = depth_ep_middle_bgp( ep, br, bgp_in, aep->hbgp );
		if( bno == -1 )		return;
		if( ep->bound[bno]->br.flag == 1 )	return;
	}


	for( bgp = gp->bgp ; bgp != NULL ; bgp = bgp->p ){
		if( bgp->br != br )	continue;

		no = ( bgp->ctr.y < 0 )? 0 : 1;
		if( bgp->ctr.x > 0 )	no += 2;

		bgp_in[no] = 0;
	}

	depth = ep->bound[bno]->br.depth;

	hb = 0.5 * aep->hbgp;

	if( bgp_in[0] != 0 )
		prime_bg_ep_add( gp, ep, bno, -hb, -hb, br, depth, aep );


	if( bgp_in[1] != 0 )
		prime_bg_ep_add( gp, ep, bno, -hb, hb, br, depth, aep );


	if( bgp_in[2] != 0 )
		prime_bg_ep_add( gp, ep, bno, hb, -hb, br, depth, aep );

	if( bgp_in[3] != 0 )
		prime_bg_ep_add( gp, ep, bno, hb, hb, br, depth, aep );
}


static void
prime_bg_ep_add( gp_type *gp, ep_type *ep, int bno,
		float x, float y, int br, float depth, ep_array *aep )
{
bgp_type	*bgp;
ebgp_type	*bp;
vec2d	ctr;
float	val[SP_NO];
float	dis;

	ctr.x = x;
	ctr.y = y;

	if( !(ep->bound[bno]->type & BOUND_ALL) )
		bno = ep_bound_point_br( ep, br, &ctr, &dis );


	if( ep_bound_value3( &aep->epValue, ep, bno, &ctr, val) == 0 )
		return;


	bgp = BGP_ALLOC();
	bgp->br = br;
	bgp->depth = depth;

	bgp->ctr.x = x;
	bgp->ctr.y = y;

	bgp->val[0] = F2QCOLOR(val[0]);
	bgp->val[1] = F2QCOLOR(val[1]);
	bgp->val[2] = F2QCOLOR(val[2]);


	bgp->p = gp->bgp;
	gp->bgp = bgp;


	ALLOC_AEP_DATA( bp, ebgp_type, aep );
/*
	bp->br = br;
	bp->depth = depth;
*/
	bp->ctr.x = x;
	bp->ctr.y = y;

	bp->val[0] = val[0];
	bp->val[1] = val[1];
	bp->val[2] = val[2];

	bp->p = ep->bound[bno]->bgp;
	ep->bound[bno]->bgp = bp;
}
