/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**********************************
 ***	PrimeDefaultBgp_1.c  	***
 *********************************/

#include	<stdio.h>
#include	<math.h>

#include	"Uigp/igp.h"


#include        "GpType/GpType.h"
#include        "EpType/EpType.h"


#include        "EpValue/EpValue.h"
#include        "EpDepth/DepthEpIn.h"

#include	"PrimeIn.h"





static void	prime_ep_complete_intrnal_one( gp_type *gp,
					ep_type *ep, int step, ep_array *aep );

static void	prime_ep_br_cont( ep_type *ep, int cno, int Fbr_no, int step,
                                        bound_type **bound );

static int	prime_ep_br_cont_test( ep_type *ep, int cno, int br );

static void	prime_default_bgp_add( gp_type *gp, ep_type *ep,
								  float hbgp, int bno, epValue_type *epValue );

static float	depth_plane_value( view_type *vw, plane3d *pl, vec2d *p );

static float	prime_default_bgp_add1( gp_type *gp, ep_type *ep,
			int bno, float x, float y, int br, float depth, epValue_type *epValue );

static view_type        *Vw;
 
static int      Fbr[256];
static plane3d          Pbr[256];

static float	Tbgp;


void
prime_default_bgp_1( gp_array *agp, ep_array *aep, int step )
{
ep_type	*ep;
gp_type	*gp;
abr_type	*br;
int	i,	j,	i1,	j1;

        Vw = agp->ad->vw;

        for( i = 0 ; i < 256 ; i++ )
                Fbr[i] = 0;


        for( i = 0 ; i < agp->ad->br_no ; i++ ){
                br = &agp->ad->abr[i];
                if( br->Fplane == 0 ){
			if( br->no == 0 ){
				Pbr[0].n = agp->ad->N;
				Pbr[0].d = VEC3D_INNER( agp->ad->N, agp->ad->P);
				Fbr[0] = 1;
			}
			continue;
		}

		Pbr[br->no].n = br->N;
		Pbr[br->no].d = VEC3D_INNER( br->N, br->P );
		Fbr[br->no] = 1;
        }




	Tbgp = aep->bgp*3.0;

        for( i = LSIZE, i1 = aep->Ni0 ; i < aep->Ni-LSIZE ; i++, i1++ )
                for( j = LSIZE, j1 = aep->Nj0 ; j < aep->Nj-LSIZE ; j++, j1++ ){
			ep = &aep->ep[i][j];
			gp = &agp->gp[i1][j1];

			prime_ep_complete_intrnal_one( gp, ep, step, aep );
		}
}



static void
prime_ep_complete_intrnal_one( gp_type *gp, ep_type *ep, int step, ep_array *aep )
{
bound_type	*b,	*bound;
int	i,	n,	Fbr_no;
/*
range_type	*r;
al_type	*al;
int	j,	k;
*/

	Fbr_no = 0;
	if( ep->bound[0]->br.no != 255 )
		for( i = 0, Fbr_no = 0 ; i < ep->b_no ; i++ )
			Fbr_no |= ( 1<< ep->bound[i]->br.no );



	bound = NULL;

	prime_ep_br_cont( ep, 1, Fbr_no, step, &bound );
	prime_ep_br_cont( ep, 3, Fbr_no, step, &bound );
	prime_ep_br_cont( ep, 5, Fbr_no, step, &bound );
	prime_ep_br_cont( ep, 7, Fbr_no, step, &bound );


	if( bound == NULL )	return;

	if( prime_ep_br_cont_test( ep, 1, bound->br.no ) == 0 )	return;
	if( prime_ep_br_cont_test( ep, 3, bound->br.no ) == 0 )	return;
	if( prime_ep_br_cont_test( ep, 5, bound->br.no ) == 0 )	return;
	if( prime_ep_br_cont_test( ep, 7, bound->br.no ) == 0 )	return;


	if( ep->b_no == 1 && ep->bound[0]->br.no == 255 ){
		b = ep->bound[0];
 
		b->type = BOUND_ALL | BOUND_UNSET;
		b->Mbgp = b->bgp = NULL;
		b->Frange = 0;
		b->range = NULL;
		b->br.depth = bound->br.depth;
		b->br.no = bound->br.no;
		b->br.flag = 0;
		b->br.bno = 2;
 
		b->Fcorner = 0xf;

		return;
	}




	ALLOC_AEP_DATA( b, bound_type, aep );
	b->type = BOUND_ALL | BOUND_UNSET;
	b->Mbgp = b->bgp = NULL;
	b->Frange = 0;
	b->range = NULL;
	b->br.depth = bound->br.depth;
	b->br.no = bound->br.no;
	b->br.flag = 0;
	b->br.bno = step;

	b->Fcorner = 0xf;

        ep->bound[ep->b_no] = b;
        ep->b_no++;



	for( n = 0 ; n < 9 ; n++ )
		b->Fcont[n] = 0;
	b->Fcont[4] =  1 << (ep->b_no-1);

	for( n = 0 ; n < 9 ; n++ ){
		if( n == 0 )	continue;
		depth_ep_cont_bound_corenr( ep, ep->ng[n], n );
	}


	prime_default_bgp_add( gp, ep, aep->hbgp, ep->b_no-1, &aep->epValue );
}





static const int	Fmc[9] = {     	8,
			      	12,
				4,
				10,
				0,
				5,
				2,
				3,
				1 };

static void
prime_ep_br_cont( ep_type *ep, int cno, int Fbr_no, int step,
					bound_type **bound )
{
ep_type		*nep;
bound_type	*b;
int	i,	fcorner;

	nep = ep->ng[cno];
	if( nep == NULL )	return;

	fcorner = Fmc[cno];

	for( i = 0 ; i < nep->b_no ; i++ ){
		b = nep->bound[i];

		if( b->br.flag == 1 )	continue;

		if( (int)b->br.bno >= step )	continue;

		if( (b->Fcorner & fcorner) == 0 )	continue;

		if( b->br.no == 255 )	continue;
		if( Fbr_no & (1<< b->br.no) )	continue;


		if( *bound != NULL && (*bound)->br.depth < b->br.depth )
			continue;
		*bound = b;
	}
}

static int
prime_ep_br_cont_test( ep_type *ep, int cno, int br )
{
ep_type         *nep;
al_type	*al;
 
        nep = ep->ng[cno];
        if( nep == NULL )       return( 1 );

	for( al = nep->al ; al != NULL ; al = al->nal ){
		if( al->link->Fbr == 0x04 )	return( 0 );
		if( al->link->br != br )	continue;
		if( !(al->link->Fbr & 0x03) )	continue;

		if( al->bno[F_RIGHT] == al->bno[F_LEFT] )	return( 0 );
	}

	return( 1 );
}





static void
prime_default_bgp_add( gp_type *gp, ep_type *ep, float hbgp, int bno, epValue_type *epValue )
{
bound_type	*b;

float   hb,     depth;
int	br;


	b = ep->bound[bno];
	depth = b->br.depth;
	br = b->br.no;

	b->br.depth = 0;

	
	hb = 0.5 * hbgp;


	b->br.depth += prime_default_bgp_add1( gp, ep, bno,-hb,-hb, br, depth, epValue );
	b->br.depth += prime_default_bgp_add1( gp, ep, bno,-hb, hb, br, depth, epValue );
	b->br.depth += prime_default_bgp_add1( gp, ep, bno, hb,-hb, br, depth, epValue );
	b->br.depth += prime_default_bgp_add1( gp, ep, bno, hb, hb, br, depth, epValue );

	b->br.depth *= 0.25;
	

/*
	bgp = BGP_ALLOC();
	bgp->ctr.x = -hb;
	bgp->ctr.y = -hb;

	bgp->br = br;
	bgp->depth = depth;
	if( Fbr[br] == 1 ){
		p.x = gp->ctr.x + bgp->ctr.x;
		p.y = gp->ctr.y + bgp->ctr.y;
		bgp->depth = depth_plane_value( Vw, &Pbr[br], &p );
	}
 
	ep_bound_value3( ep, bno, &bgp->ctr, bgp->val );

	b->br.depth += bgp->depth;
 
	bgp->p = gp->bgp;
	gp->bgp = bgp;

	

	bgp = BGP_ALLOC();
	bgp->ctr.x = -hb;
	bgp->ctr.y = hb;

	bgp->br = br;
	bgp->depth = depth;
	if( Fbr[br] == 1 ){
		p.x = gp->ctr.x + bgp->ctr.x;
		p.y = gp->ctr.y + bgp->ctr.y;
		bgp->depth = depth_plane_value( Vw, &Pbr[br], &p );
	}
 
	ep_bound_value3( ep, bno, &bgp->ctr, bgp->val );
	b->br.depth += bgp->depth;
 
	bgp->p = gp->bgp;
	gp->bgp = bgp;

	

	bgp = BGP_ALLOC();
	bgp->ctr.x = hb;
	bgp->ctr.y = -hb;
 
	bgp->br = br;
	bgp->depth = depth;
	if( Fbr[br] == 1 ){
		p.x = gp->ctr.x + bgp->ctr.x;
		p.y = gp->ctr.y + bgp->ctr.y;
		bgp->depth = depth_plane_value( Vw, &Pbr[br], &p );
	}
 
	ep_bound_value3( ep, bno, &bgp->ctr, bgp->val );
	b->br.depth += bgp->depth;
 
	bgp->p = gp->bgp;
	gp->bgp = bgp;

	

	bgp = BGP_ALLOC();
	bgp->ctr.x = hb;
	bgp->ctr.y = hb;

	bgp->br = br;
	bgp->depth = depth;
	if( Fbr[br] == 1 ){
		p.x = gp->ctr.x + bgp->ctr.x;
		p.y = gp->ctr.y + bgp->ctr.y;
		bgp->depth = depth_plane_value( Vw, &Pbr[br], &p );
	}
 
 
	ep_bound_value3( ep, bno, &bgp->ctr, bgp->val );
	b->br.depth += bgp->depth;
 
	bgp->p = gp->bgp;
	gp->bgp = bgp;


	b->br.depth *= 0.25;
*/
}


static float
prime_default_bgp_add1( gp_type *gp, ep_type *ep,
		int bno, float x, float y, int br, float depth, epValue_type *epValue )
{
bgp_type        *bgp;
float	val[3];
vec2d	p;

	bgp = BGP_ALLOC();
	bgp->ctr.x = x;
	bgp->ctr.y = y;

	bgp->br = br;
	bgp->depth = depth;
	if( Fbr[br] == 1 ){
		p.x = gp->ctr.x + bgp->ctr.x;
		p.y = gp->ctr.y + bgp->ctr.y;
		bgp->depth = depth_plane_value( Vw, &Pbr[br], &p );
	}
 
	ep_bound_value3( epValue, ep, bno, &bgp->ctr, val );
	bgp->val[0] = F2QCOLOR(val[0]);
	bgp->val[1] = F2QCOLOR(val[1]);
	bgp->val[2] = F2QCOLOR(val[2]);

	bgp->p = gp->bgp;
	gp->bgp = bgp;


	return( bgp->depth );
}
	








static float
depth_plane_value( view_type *vw, plane3d *pl, vec2d *p )
{
vec2d   v;
float   t,	value;
vec3d	n,	p3;

        v.x = Vw->Ua_inv*(p->y - Vw->Ub);
        v.y = Vw->Va_inv*(p->x - Vw->Vb);


	view_3d_ray( Vw, v.x, v.y, &p3, &n );
	t = 1.0 / VEC3D_NORM( n );
	n.x *= t;
	n.y *= t;
	n.z *= t;
 
	t = ( pl->d - VEC3D_INNER( Vw->cp, pl->n ) ) / VEC3D_INNER( n, pl->n );

	value = t* VEC3D_INNER( n, Vw->N );

	return( value );
}
