/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**************************
 ***	PrimeGeo.c	***
 **************************/
#include        <stdio.h>
#include        <math.h>

#include        "PrimeIn.h"


static int	prime_test_split_edge( jnt_type *jnt );

static jnt_type *	prime_split_edge( jnt_type *jnt, int bno );

static void	prime_jnt_reset_branch( jnt_type *jnt );

static void	prime_jnt_egde_extand( jnt_type *jnt );

static void 	prime_geo_insert_jnt( gp_array *agp, jnt_type *jnt );


int
prime_geo( gp_array *agp )
{
gp_type		*gp;
jnt_type	*jnt,	*njnt;
vec2d	ctr;
link_type	*link;
int	i,	j;
int	bno,	split;


	split = 0;
	
	for( i = 0 ; i < agp->Ni ; i++ )
		for( j = 0 ; j < agp->Nj ; j++ ){
			gp = &agp->gp[i][j];

			for( jnt = gp->jnt ; jnt != NULL ; jnt = jnt->nj ){
				if( jnt->type != JNT_EDGE )	continue;

				if( (bno = prime_test_split_edge( jnt )) == -1 )
					continue;

				split++;

				njnt = prime_split_edge( jnt, bno );
				prime_geo_insert_jnt( agp, njnt );


				ctr = njnt->gctr;
				link = njnt->brn[0]->link;
				if( njnt->brn[0]->direction == F_END ){
					ctr.x -= link->v.x;
					ctr.y -= link->v.y;
				}

				agp_lnf_add_link_1( &ctr, link, agp );
			}
		}

	return( split );
}


static int
prime_test_split_edge( jnt_type *jnt )
{
brn_type        *b;
int     i,      br[MAX_BRN];

	if( jnt->brn_no < 3 )   return( -1 );

 
	for( i = 0 ; i < jnt->brn_no ; i++ ){
		b = jnt->brn[i];

		if( b->link->Fbr == 0x04 )	return( -1 );

		br[i] = b->link->br;
	}
 
	if( br[0] == br[1] ){
		if( br[1] == br[2] )	return( -1 );
		return( 2 );
	}

	return( ( br[1] == br[2] )? 0 : 1 );
}
 




static jnt_type *
prime_split_edge( jnt_type *jnt, int bno )
{
jnt_type        *njnt;
int	i;

	njnt = jnt_alloc();
	njnt->type = jnt->type;
	njnt->gctr = jnt->gctr;
	njnt->ctr = jnt->ctr;
/*
	njnt->depth = jnt->depth;
*/
	njnt->brn_no = 1;
	njnt->brn[0] = jnt->brn[bno];
 
	prime_jnt_reset_branch( njnt );
 

	jnt->brn_no--;
	for( i = bno ; i < jnt->brn_no ; i++ )
		jnt->brn[i] = jnt->brn[i+1];

 
	prime_jnt_reset_branch( jnt );
	jnt_edge_set_aux( jnt );
/*
	jnt_set_depth( jnt );
*/


	prime_jnt_egde_extand( njnt );
	jnt_edge_set_aux( njnt );
/*
	jnt_set_depth( njnt );
*/

	return( njnt );
}


static void
prime_jnt_reset_branch( jnt_type *jnt )
{
brn_type        *b;
int     k;
 
	for( k = 0 ; k < jnt->brn_no ; k++ ){
		b = jnt->brn[k];
		if( b->type != NF_EDGE && b->type != NF_N3 )
			continue;
 
		b->link->p[b->direction].p.jnt = jnt;
		b->link->p[b->direction].bno = k;
	}
}




#define		EXT_LEN		14
static void
prime_jnt_egde_extand( jnt_type *jnt )
{
brn_type	*b;
link_type	*link;

	b = jnt->brn[0];

	link = (link_type *)malloc(sizeof( link_type ) );
	link->in[F_RIGHT] = link->in[F_LEFT] = NULL;
	link->width[F_BEGIN] = link->width[F_END] = NULL;

	link->v.x = -EXT_LEN * b->link->u.y;
	link->v.y =  EXT_LEN * b->link->u.x;
	link->a = 0;
	link->len =  EXT_LEN;

	link->type = b->link->type;
	link->br = b->link->br;
	link->Fbr = b->link->Fbr;



	if( b->direction == F_BEGIN ){

		link->p[F_BEGIN] = b->link->p[F_BEGIN];
		link->p[F_END].bno = -1;
		link->p[F_END].p.l = b->link;
		b->link->p[F_BEGIN].bno = -1;
		b->link->p[F_BEGIN].p.l = link;

		link->pr[0] = profile_make_copy( b->link->pr[0], NF_EDGE );

		link->pr[1] = b->link->pr[0];

		link_set_aux( link );

		jnt->gctr.x -= link->v.x;
		jnt->gctr.y -= link->v.y;
		b->link = link;

		return;
	}

	link->p[F_END] = b->link->p[F_END];
	link->p[F_BEGIN].bno = -1;
	link->p[F_BEGIN].p.l = b->link;
	b->link->p[F_END].bno = -1;
	b->link->p[F_END].p.l = link;


	link->pr[0] = b->link->pr[1];
	link->pr[1] = profile_make_copy( link->pr[0], NF_EDGE );



	link_set_aux( link );


	jnt->gctr.x += link->v.x;
	jnt->gctr.y += link->v.y;
	b->link = link;
	return;
}



static void
prime_geo_insert_jnt( gp_array *agp, jnt_type *jnt )
{
gp_type *gp;
int     i,      j;

        i = (jnt->gctr.x+agp->bgp) * agp->inv_bgp;
        j = (jnt->gctr.y+agp->bgp) * agp->inv_bgp;


        gp = &agp->gp[i][j];

        jnt->ctr.x = jnt->gctr.x - gp->ctr.x;
        jnt->ctr.y = jnt->gctr.y -gp->ctr.y;


        jnt->nj = gp->jnt;
        gp->jnt = jnt;
}
