/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**************************
 ***	OcpLTool.c	***
 **************************/
#include	<stdio.h>
#include	<math.h>

#include	"Uigp/igp.h"


#include	"GpType/GpType.h"


#include	"VimageType/OcpType.h"



static ocp_type * ocp_Copy_Change ( skl_type* skl, skl_type* bskl, ocp_type* socp );

static void	skl_Copy_Change( skl_type *skl0, skl_type *skl1, skl_type *skl );

static void	lnk_Copy_Change( lnk_type *link0, lnk_type *link1,
							vec2d *ctr, lnk_type *link );
static void lnk_shift_Copy_Change(lnk_type* lnk, vec2d* new_ctr);


/*
Description:
	copies animation of one skeleton to another which is close to the source skeleton, but has not to be 
	exactly the same.
Params:
	skl  - target skeleton for animation
	bskl - the source skeleton
	socp - the source animation
Returns:
	ocp_list - the target animation, built with the new skeleton

*/
ocp_list * ocpL_Copy_Change ( skl_type* skl, skl_type* bskl, ocp_list* socp ) 
{
ocp_list	*ocp;
	ocp_type	*bcp,	**cp;

	ocp = ocpL_alloc();

	cp = &ocp->cp;
	for( bcp = socp->cp ; bcp != NULL ; bcp = bcp->next ){

			*cp = ocp_Copy_Change( skl, bskl, bcp );
			ocp->no++;

			cp = &(*cp)->next;

	}

	*cp = NULL;

	return( ocp );
}


/*
Description:
	creates a new copy of scp shifted by the new skeleton's characteristics
Params:
	skl -  the target skeleton
	bskl - source skeleton
	scp -  source cp from which the new ocp would be created.
*/
static ocp_type * ocp_Copy_Change ( skl_type* skl, skl_type* bskl, ocp_type* scp ) 
{
ocp_type	*cp;

	cp = ocp_alloc( skl );


	cp->c = scp->c;
	cp->dp = scp->dp;
	cp->dfno = scp->dfno;
	cp->fno = scp->fno;
	cp->Fshot = scp->Fshot;

	cp->Fgc = scp->Fgc;
	cp->gc = scp->gc;


	skl_topology( cp->skl );
	// copy the skeleton of the new ocp according to the source skl. 
	if( skl->jnt != NULL && bskl->jnt != NULL  )
		skl_Copy_Change( bskl, scp->skl, cp->skl );

	
	return cp;
}




/*
Description:
	copies the source shifted skeleon's position to the new skeleton
Params:
	skl0 - the source skeleton ( cast->baseSkl )
	skl1 - the source shifted skeleton ( ocp->skl )
	skl	 - the new skeleton to change
*/
static void
skl_Copy_Change( skl_type *skl0, skl_type *skl1, skl_type *skl )
{
jskl_type	*jnt,	*j0,	*j1;
bskl_type	*b,	*b0;
int	i;
	// change vectors and center points
	// ONLY FOR dependent joints (Ftopology != 0)
	for( jnt = skl->jnt, j0 = skl0->jnt, j1 = skl1->jnt ; jnt != NULL && j1 != NULL ;
				jnt= jnt->nj, j0 = j0->nj, j1 = j1->nj ){

		if( jnt->Ftopology == 0 )	continue;


		jnt->ctr.x += j1->ctr.x - j0->ctr.x;
		jnt->ctr.y += j1->ctr.y - j0->ctr.y;

//		jnt->ctr =  j1->ctr;

		for( i = 0 ; i < jnt->bno ; i++ ){

			b = &jnt->b[i];
			if( b->direct == F_END )	continue;

			if (i < j0->bno) {
				b0 = &j0->b[i];
				lnk_Copy_Change( j0->b[i].link, j1->b[i].link,
					&jnt->ctr, b->link);
			}
			else {
				lnk_shift_Copy_Change(b->link, &jnt->ctr);
			}
		}

	}
 
	// change vectors and center points
	// ONLY FOR independent joints (Ftopology != 1)
	for( jnt = skl->jnt, j0 = skl0->jnt, j1 = skl1->jnt ; jnt != NULL && j1 != NULL ;
				jnt= jnt->nj, j0 = j0->nj, j1 = j1->nj ){

		if( jnt->Ftopology == 1 )	continue;

		if( jnt->pl != NULL )
			pl_point( jnt->pl, &jnt->ctr );


		for( i = 0 ; i < jnt->bno ; i++ ){

			b = &jnt->b[i];
			if( b->direct == F_END )	continue;

			if (i < j0->bno) {
				b0 = &j0->b[i];
				lnk_Copy_Change( j0->b[i].link, j1->b[i].link,
					&jnt->ctr, b->link);
			}
			else {
				lnk_shift_Copy_Change(b->link, &jnt->ctr);
			}
		}
	}

	skl_update_attach( skl );
}


/*
Description:
	changes the link according to the base and source links' proportions.
Params:
	link0	- base link 
	link1	- source link
	ctr		- target center
	link	- target link
*/
static void
lnk_Copy_Change( lnk_type *link0, lnk_type *link1,
			vec2d *ctr, lnk_type *link )
{
lnk_type	*l,	*l0,	*l1,	*pl;
jskl_type	*jnt;
vec2d	v;
float	t;
	// change the vectors of all it's links
	for( l = link, l0 = link0, l1 = link1, v = *ctr ; l != NULL && l1 != NULL; 
			l = LNK_NEXT(l), l0 = LNK_NEXT(l0), l1 = LNK_NEXT(l1) ){

		l->ctr = v;

		t = l1->len / l0->len;


		l->len *= t;


//		l->a *= t;
		l->a = l1->a *t;

		l->u = l1->u;

		l->v.x = - l->len * l->u.y; // ??? 
		l->v.y =   l->len * l->u.x; // ???



		l->c_prb = -4.0 *l->a/( l->len*l->len);

		lnk_set_aux( l );

		v.x += l->v.x;
		v.y += l->v.y;


		pl = l;
	}

	// change the center of the link
	for(  ; l != NULL ; l = LNK_NEXT(l) ){

		l->ctr = v;

		v.x += l->v.x;
		v.y += l->v.y;


		pl = l;
	}

	

	
	jnt = pl->p[F_END].p.jnt;
	if( jnt != NULL && jnt->Ftopology == 0 )
		jnt->ctr = v;

}

/*
Description:
	the method shifts the old link's position according to the new center point.
Params:
	lnk		- the link to shift
	new_ctr - the new link's center.
*/
static void lnk_shift_Copy_Change(lnk_type* lnk, vec2d* new_ctr)
{
	lnk_type *l;
	vec2d cur_ctr;

	cur_ctr = *new_ctr;
	for( l = lnk; l != NULL ; l = LNK_NEXT(l) ){		
		l->ctr = cur_ctr;
		cur_ctr.x = l->ctr.x + l->v.x;
		cur_ctr.y = l->ctr.y + l->v.y;
	}
}


