/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**************************
 ***	OcpLTool.c	***
 **************************/
#include	<stdio.h>
#include	<math.h>

#include	"Uigp/igp.h"


#include	"GpType/GpType.h"


#include	"VimageType/OcpType.h"



static ocp_type * ocp_Copy_Change_Flip ( skl_type* skl, skl_type* bskl, ocp_type* socp, int Ymax );

static void	skl_Copy_Change_Flip( skl_type *skl0, skl_type *skl1, skl_type *skl );

static void	lnk_Copy_Change_Flip( lnk_type *link0, lnk_type *link1,
							vec2d *ctr, lnk_type *link );
static void lnk_shift_Copy_Change_Flip(lnk_type* lnk, vec2d* new_ctr);
static void reverse_skl_flip (skl_type *skl);



ocp_list * ocpL_Copy_Change_Flip ( skl_type* skl, skl_type* bskl, ocp_list* socp, int Ymax ) 
{
ocp_list	*ocp;
	ocp_type	*bcp,	**cp;

	ocp = ocpL_alloc();

	cp = &ocp->cp;
//	prev_cp = NULL;
	for( bcp = socp->cp ; bcp != NULL ; bcp = bcp->next ){

			*cp = ocp_Copy_Change_Flip( skl, bskl, bcp, Ymax );
//			cp = ocp_Copy_Change( skl, bskl, bcp, Ymax );
			ocp->no++;
/*			(cp)->next = prev_cp;
			prev_cp = cp;*/

			cp = &(*cp)->next;
	}

	*cp = NULL;

//	ocp->cp = cp;

	return( ocp );
}


static ocp_type * ocp_Copy_Change_Flip ( skl_type* skl, skl_type* bskl, ocp_type* scp, int Ymax ) 
{
ocp_type	*cp;

	cp = ocp_alloc( skl );


	cp->c = scp->c;
	cp->c.x = -cp->c.x;
	cp->dp.x = scp->dp.x;
	cp->dp.y = scp->dp.y;
	cp->dfno = scp->dfno;
	cp->Fshot = scp->Fshot;

	cp->Fgc = scp->Fgc;
	cp->gc = scp->gc;


//	skl_topology( cp->skl );

	skl_flipV( scp->skl, Ymax );
	skl_Copy_Change_Flip( bskl, scp->skl, cp->skl );

	
	return cp;
}





static void
skl_Copy_Change_Flip( skl_type *skl0, skl_type *skl1, skl_type *skl )
{
jskl_type	*jnt,	*j0,	*j1;
bskl_type	*b,	*b0;
int	i;

	skl_topology(skl);
	for( jnt = skl->jnt, j0 = skl0->jnt, j1 = skl1->jnt ; jnt != NULL && j1 != NULL ;
				jnt= jnt->nj, j0 = j0->nj, j1 = j1->nj ){

		if( jnt->Ftopology == 0 )	continue;

//		jnt->ctr.x = (1 - t ) *j0->ctr.x + t *j1->ctr.x;
//		jnt->ctr.y = (1 - t ) *j0->ctr.y + t *j1->ctr.y;

		jnt->ctr =  j1->ctr;

		for( i = 0 ; i < jnt->bno ; i++ ){

			b = &jnt->b[i];
			if ( b->direct == F_END )	continue;

			if (i < j0->bno) {
				b0 = &j0->b[i];
				lnk_Copy_Change_Flip( j0->b[i].link, j1->b[i].link,
					&jnt->ctr, b->link);
			}
			else {
				lnk_shift_Copy_Change_Flip(b->link, &jnt->ctr);
			}
		}

	}
				


	for( jnt = skl->jnt, j0 = skl0->jnt, j1 = skl1->jnt ; jnt != NULL && j1 != NULL ;
				jnt= jnt->nj, j0 = j0->nj, j1 = j1->nj ){

		if( jnt->Ftopology == 1 )	continue;

		if( jnt->pl != NULL )
			pl_point( jnt->pl, &jnt->ctr );


		for( i = 0 ; i < jnt->bno ; i++ ){

			b = &jnt->b[i];
			if( b->direct == F_END )	continue;

			if (i < j0->bno) {
				b0 = &j0->b[i];
				lnk_Copy_Change_Flip( j0->b[i].link, j1->b[i].link,
					&jnt->ctr, b->link);
			}
			else {
				lnk_shift_Copy_Change_Flip(b->link, &jnt->ctr);
			}
		}
	}

	skl_update_attach( skl );
}



static void
lnk_Copy_Change_Flip( lnk_type *link0, lnk_type *link1,
			vec2d *ctr, lnk_type *link )
{
lnk_type	*l,	*l0,	*l1,	*pl;
jskl_type	*jnt;
vec2d	v;
float	t;

	for( l = link, l0 = link0, l1 = link1, v = *ctr ; l != NULL && l1 != NULL; 
			l = LNK_NEXT(l), l0 = LNK_NEXT(l0), l1 = LNK_NEXT(l1) ){

		l->ctr = v;

		t = l1->len / l0->len;


		l->len *= t;
		l->a *= t;

		l->u = l1->u;

		l->v.x = - l->len * l->u.y;
		l->v.y =   l->len * l->u.x;



		l->c_prb = -4.0 *l->a/( l->len*l->len);

		lnk_set_aux( l );

		v.x += l->v.x;
		v.y += l->v.y;


		pl = l;
	}


	for(  ; l != NULL ; l = LNK_NEXT(l) ){

		l->ctr = v;

		v.x += l->v.x;
		v.y += l->v.y;


		pl = l;
	}

	


	jnt = pl->p[F_END].p.jnt;
	if( jnt != NULL && jnt->Ftopology == 0 )
		jnt->ctr = v;

}


static void lnk_shift_Copy_Change_Flip(lnk_type* lnk, vec2d* new_ctr)
{
	lnk_type *l;
	vec2d cur_ctr;

	cur_ctr = *new_ctr;
	for( l = lnk; l != NULL ; l = LNK_NEXT(l) ){		
		l->ctr = cur_ctr;
		cur_ctr.x = l->ctr.x + l->v.x;
		cur_ctr.y = l->ctr.y + l->v.y;
	}
}


static void reverse_skl_flip (skl_type *skl)
{
	jskl_type* jnt, *prev_jnt, *temp_jnt;
	if (skl->jnt == NULL || skl->jnt->nj == NULL) return;

	jnt = skl->jnt->nj;
	prev_jnt = skl->jnt;
	prev_jnt->nj = NULL;
	while( jnt != NULL ){
		temp_jnt = jnt;
		jnt = jnt->nj;
		temp_jnt->nj = prev_jnt;
		prev_jnt = temp_jnt;
	}

	skl->jnt = prev_jnt;
}

