/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**********************
 ***	OcpLCopy.c	***
 **********************/
#include	<math.h>

#include	"Uigp/igp.h"


#include	"VimageType/OcpType.h"



/***** ocpL_copy *****
Description:The method copies socp and returns the copy ocp
Params:socp - source ocp
Returns:ocp_list which is copy of the source
*****************************/
ocp_list *ocpL_copy( ocp_list *socp )
{
ocp_list	*ocp;
ocp_type	*cp,	*scp,	*pcp;



	ocp = ocpL_alloc();
	ocp->cp =  NULL;
	pcp = NULL;

	for( scp = socp->cp; scp != NULL && scp != NULL ; scp = scp->next ){
		cp = ocp_make_copy( scp );
		cp->next = NULL;


		if( pcp == NULL )
				ocp->cp = cp;
		else	pcp->next = cp;		
		ocp->no++;

		pcp = cp;
	}


	ocp->acp = ocp->cp;


	return( ocp );
}



/****** ocpL_copy_segment *****
Description:The method copies a segment from a list.
		It generates two fictive keyframes at the begin and end of the segment, to
		ensure that if user tries to copy blank frames at the start and/or the end
		of the segment, he will get them correctly.
Params:socp - the source ocp
	f0 - frame to copy from
	f1 - frame to stop copying
Returns:copy list of the segment.
*****************************/
ocp_list *ocpL_copy_segment( ocp_list *socp, int f0, int f1 )
{
ocp_list	*ocp;
ocp_type	*cp,	*scp,	*pcp;

	ocp = ocpL_alloc();

	//first dummy kf
	ocp->cp = ocp->acp = ocp_make_copy(socp->cp);	
	ocp->cp->Fshot = SHOT_END;
	ocp->cp->fno = -1;
	ocp->cp->dfno = 1;
	ocp->no = 1;


	pcp = ocp->cp;


	// first Frame - may be not a kframe
	if( (cp = ocpLF_copy( socp, f0 )) != NULL ){
		pcp->next = cp;
		cp->fno -= f0;
		ocp->no++;
		cp->Fgc = 1;

		pcp->dfno = cp->fno - pcp->fno;
		pcp = cp;
	}


	
	
	scp = ocpLF_next_kf( socp, f0 );

	for( ; scp != NULL && scp->fno < f1 ; scp = scp->next ){
		cp = ocp_make_copy( scp );
		cp->fno -= f0;
		pcp->dfno = cp->fno - pcp->fno;

		pcp->next = cp;
		ocp->no++;
		pcp = cp;
	}


	// last Frame - may be not a kframe
	if( (cp = ocpLF_copy( socp, f1 )) != NULL ){
		pcp->next = cp;
		cp->fno -= f0;
		ocp->no++;
		cp->Fgc = 1;

		pcp->dfno = cp->fno - pcp->fno;
		pcp = cp;
	}


	// empty segement
	if( ocp->no == 1 ){
		ocpL_destroy( ocp );
		return( NULL );
	}



	ocp->cp->next->Fgc = 1;
	pcp->Fgc = 1;

	

	//end dummy kf
	cp = ocp_make_copy(pcp);
	cp->next = NULL;
	cp->fno = f1 - f0 + 1;
	cp->dfno = 1;
	pcp->next = cp;
	pcp->dfno = cp->fno - pcp->fno;
	ocp->no++;
	
	return( ocp );
}

/*
ocp_list *ocpL_copy_segment( ocp_list *socp, int f0, int f1 )
{
ocp_list	*ocp;
ocp_type	*cp,	*scp,	*pscp,	*pcp, *firstCp = NULL;
int first_copied = 0;
	ocp = ocpL_alloc();

	ocp->cp = ocp->acp = ocp_make_copy(socp->cp);	
	ocp->cp->Fshot = SHOT_END;
	ocp->cp->fno = -1;
	ocp->cp->dfno = 1;
	ocp->no = 1;


	cp = ocpLF_copy( socp, f0 );
	if ( cp != NULL )// ok, it is not a FRAME_BLANK
	{
		ocp->cp->next = cp;
		cp->fno = 0;
		ocp->no++;
		cp->Fgc =1;
		first_copied = 1;
		firstCp = cp;
	}
	else
		cp = ocp->cp;	
	
	
	scp = ocpLF_next_kf( socp, f0 );


	pcp = cp;

	// copy all keyframes in the segment:
	pscp = NULL;
	for( ; scp != NULL && scp->fno <= f1 ; scp = scp->next ){
		cp = ocp_make_copy( scp );
		cp->fno -= f0;
		pcp->dfno = cp->fno - pcp->fno;
		if ( ! first_copied )
		{
			firstCp = cp;
			first_copied = 1;
			cp->Fgc = 1;
		}
		pcp->next = cp;
		ocp->no++;
		pcp = cp;


		pscp = scp;
	}



	if( pscp == NULL ||  pscp->fno != f1 ){
		cp = ocpLF_copy( socp, f1 );
		if ( cp != NULL ){

			if ( ! first_copied ){
				firstCp = cp;
				first_copied = 1;
				cp->Fgc = 1;
			}
			cp->fno -= f0;
			pcp->dfno = cp->fno - pcp->fno;
			pcp->next = cp;
			ocp->no++;
			pcp = cp;
		}
	}

	
	//pcp->Fshot = SHOT_END;
	if (!( CONFIG3D_IS_EQUAL(pcp->gc,firstCp->gc) ))
		pcp->Fgc = 1;

	//end dummy kf
	cp = ocp_make_copy(pcp);
	cp->next = NULL;
	cp->fno = f1 - f0 + 1;
	cp->dfno = 1;
	pcp->next = cp;
	pcp->dfno = cp->fno - pcp->fno;
	ocp->no++;
	
	return( ocp );
}
*/



/***** ocpL_delete *****
Description:The method deletes frame between f0 and f1. It enforces the before and after 
			frames to become keyframes
Params:	ocp  - the list
		f0 - the frame to start deletion from 
		f1 - the frame to end the deletion
Returns:1 if the cast exists after the operation. 0 if not exists
*****************************/
int ocpL_delete( ocp_list *ocp, int f0, int f1 )
{
ocp_type	*cp,	*cp0,	*cp1;



	cp = ocp->cp;
	if ( !cp )
		return 0;
	if ( f1+1 <= cp->fno)
		return 1;
	if( f0-1 < cp->fno ){// cutting starts before first keyframe.
		cp1 = ocpLF_force_kf( ocp, f1+1 ); 
		
		while( (cp = ocp->cp) != NULL && cp->fno <= f1 ){
			ocp->cp = cp->next;
			free( cp );
			ocp->no--;
		}
		if( cp1 != NULL ){// now it is ocp->cp...
			cp1->Fgc = 1;
		}
		ocp->acp = ocp->cp;
		return( 1 );
	}


	cp0 = ocpLF_force_kf( ocp, f0-1 );
	cp1 = ocpLF_force_kf( ocp, f1+1 ); 

	if( cp0 == NULL )
		for( cp0 = ocp->cp ;
			cp0 != NULL && cp0->fno + cp0->dfno < f0-1 ;
			cp0 = cp0->next );

	if( cp0 == NULL )	return( 0 );




	while( (cp = cp0->next) != NULL && cp->fno <= f1 ){
		cp0->next = cp->next;
		free( cp );
		ocp->no--;
	}

	if( cp0->next != NULL )
		cp0->dfno = cp0->next->fno - cp0->fno;
	else 
		cp0->dfno = 1;
	
	cp0->Fshot = SHOT_END;		
	cp0->Fgc = 1;

	if( cp1 != NULL ){
		if ( cp1->Fshot != SHOT_END )
			cp1->Fshot = SHOT_MID;
		cp1->Fgc = 1;
	}


	ocp->acp = ocp->cp;
	if ( ocp->cp == NULL )
		return 0;
	return( 1 );
}




/***** ocpL_delete_shrink *****
Description:The method deletes frames between f0 and f1 from the animation, and then
		reduces the animation (eliminates the gap).
		It enforces the before and after frames to become keyframes. (***should it do it?***)
Params:	ocp  - the list
		f0 - the frame to start deletion from 
		f1 - the frame to end the deletion
Returns:number of frames after the operation ( last cp->fno ) or -1 if there are no cp's
*****************************/
int ocpL_delete_shrink(ocp_list *ocp, int f0, int f1)
{	
	if ( ocpL_delete( ocp, f0, f1 ) != 0 )
		return ocpL_shrink( ocp, f0, f1);
	else
		return -1;

}



/***** ocpL_shrink *****
Description:The method shrinks the ocp by (f1 - f0) frames starting at f0.
Params:ocp - the list
		f0 - where to start the shrinking
		f1 - where to stop.
Returns:last cp->fno in the list, or -1 if no animation. 
*****************************/
int ocpL_shrink(ocp_list *ocp, int f0, int f1)
{
	int diff = f1 - f0 + 1;
	ocp_type *cp;
	if ( ocp->cp == NULL )
		return -1;
	cp = ocpLF_get_kf( ocp, f0 );
	if ( !cp )
	{
		cp = ocp->cp;		
		cp->fno -= diff;
	}
	else
	{
		if ( cp->next )
		{
			cp->dfno -= diff;
			cp->Fshot = SHOT_MID;
		}
		
	}	
	
	// shorten the fno by the right difference.
	while ( cp->next != NULL ) 
	{	
		cp = cp->next;
		cp->fno -= diff;				
	}	
	return cp->fno;
}

/***** ocpL_clip *****
Description:The method clips (deletes) the nodes in the list, who's fno is greater than "frame" 
Params:	ocp - the list
		frame - the frame number to clip from.
Returns: 1 or 0 - success or failure.
*****************************/
int ocpL_clip( ocp_list *ocp, int frame )
{
ocp_type	*cp,	*cp0;

	if( frame < ocp->cp->fno ){
		while( (cp = ocp->cp ) != NULL ){
			ocp->cp = cp->next;
			ocp_free( cp );
			ocp->no--;
		}
		return(1);
	}

	cp0 = ocpLF_force_kf( ocp, frame );
	if( cp0 == NULL )
		cp0 = ocpLF_prev_kf( ocp, frame );

	if( cp0->next == NULL )	return(0);


	while( (cp = cp0->next ) != NULL ){
		cp0->next = cp->next;
		if( cp->next == NULL )	cp0->gc = cp->gc;
		ocp_free( cp );
		ocp->no--;
	}

	cp0->next = NULL;
	cp0->Fshot = SHOT_END;
	cp0->Fgc = 1;

	return(1);
}



/***** ocpL_paste *****
Description:The method pastes animation, starting at frame nFrame0
Params:	ocp		- target animation 
		socp		- source animation ( one to be pasted ) with dummy kf at start and end of it
		nFrame0	- frame number to paste the animation.
		fDeleteKFs	- the flag determines the deletion of the keyframes before 
				  and after the edges of the segment being pasted.
Returns:
*****************************/
int	ocpL_paste( ocp_list *tocp, ocp_list *socp, int nFrame0, int fDeleteKFs )
{
ocp_type	*cp,	*pcp,	*scp;
ocp_type	*cp0,	*cp1;
ocp_type *cpA, *cpB;
ocp_type *tmpCp1;
int	fno;
int FshotA, FshotB; 
int deleteA = 0, deleteB = 0;
	fno = nFrame0 - 1;// ****minus one for the dummy****
	// set the new animation to have correct "fno"
	for( cp = socp->cp ; cp  != NULL ; cp = cp->next ){
		cp->fno = fno;
		fno += cp->dfno;
		cp1 = cp;
	}

	fno -= (cp1->dfno + 1);// last one is dummy which sits one frame after the end of the segment
	//cp1->dfno = 1;
	cpA = ocpLF_prev_kf(tocp,nFrame0);
	if ( cpA != NULL )
	{
		if	((cpA->fno < (nFrame0 - 1) ) &&(cpA->Fshot != SHOT_END) && 
			(socp->no > 2 && socp->cp->next->fno == nFrame0))
			deleteA = 1;// after delete, there will be a new keyframe at (nFrame0 - 1), and we can delete it.
		
		if ( cpA->Fshot == SHOT_END || (socp->cp->next->fno != nFrame0))
			FshotA = SHOT_END;
		else
			FshotA = SHOT_MID;

		cpB = cpA;
	}
	else 
	{
		FshotA = SHOT_END;
		cpB = tocp->cp;
	}
	
	while ( cpB && (cpB->fno + cpB->dfno < fno ))
		cpB = cpB->next;
	if ( cpB == NULL || cpB->fno > fno)// can be greater than fno, if paste takes place before current animation
		FshotB = SHOT_END;
	else 
	{
		if (cpB->fno + cpB->dfno == fno)			
			cpB = cpB->next;
		if ( cpB == NULL )// so the animation would exceed current animation
		{
			FshotB = 0;
		}
		else 
		{
			FshotB = cpB->Fshot;
			deleteB = ((cpB->fno + cpB->dfno > (fno + 1)) && (cpB->Fshot != SHOT_END) );
		}
	}



	// delete the frames to be replaced
	ocpL_delete( tocp, nFrame0, fno );

	if ( fDeleteKFs == 0 )
		deleteA = deleteB = 0;// don't delete the new keyframes
	if( tocp->cp != NULL && nFrame0 >= tocp->cp->fno ){
		cp0 = ocpLF_get_kf( tocp, nFrame0 );
		if ( deleteA )
		{
			cpA = ocpLF_get_kf(tocp,cp0->fno - 1);
			cpA->next = cp0->next;
			tocp->no--;
			free ( cp0 );
			cp0 = cpA;
		}
		cp0->Fshot = FshotA;
		cp1 = cp0->next;
	}
	else	{
		cp0 = NULL;
		cp1 = tocp->cp;
	}
	// cp0 is the closest from below to nFrame0, and cp1 is the next
	pcp = cp0;
	if ( socp->no == 2)// only two dummies
	{
		ocpL_reset_fno(tocp);
		return (1);
	}
	else 
	{
		tmpCp1 = socp->cp;
		socp->cp = socp->cp->next;// so the first to be copied is a real kf
	}
	fno = socp->cp->fno;
	// copy keyframes one by one from the aux-animation till last one - the dummy
	for(	scp = socp->cp ; 
		scp != NULL && scp->next != NULL;/*for last one - the dummy*/
		scp = scp->next )
	{

		cp = ocp_make_copy( scp );
		cp->fno = fno;
		
		if( pcp != NULL ){
			cp->next = pcp->next;
			pcp->next = cp;
			pcp->dfno = cp->fno - pcp->fno;
		}
		else	{
			cp->next = tocp->cp;
			tocp->cp = cp;
		}

		fno += cp->dfno;

		tocp->no++;

		pcp = cp;
	}//for
	

	if ( pcp->Fshot != SHOT_END)// if the animation had blanks at end - don't create inBetween
		pcp->Fshot = FshotB;
	if( pcp->next != NULL && pcp->Fshot != SHOT_END)
	{		
		if ( deleteB )
		{
			cpB = pcp->next;
			pcp->next = cpB->next;
			free ( cpB );
			tocp->no--;
		}
		pcp->dfno = pcp->next->fno - pcp->fno;
	}
	else
		pcp->Fgc = 1;

	ocpL_reset_fno(tocp);
	socp->cp = tmpCp1;
	tmpCp1->fno = tmpCp1->next->fno - tmpCp1->dfno;
	return( 1 );
}


ocp_list *
ocpL_adjustment( skl_type *sskl, ocp_list *socp, skl_type *skl )
{
ocp_list	*ocp;

	if( skl_is_equal( sskl, skl ) == 1 )
		return( socp );

	if( skl->orientation == sskl->orientation ){
		ocp = ocpL_Copy_Change( skl, sskl, socp );

		ocpL_free( socp );

		return( ocp );
	}

	return( NULL );
}




/***** ocpL_exist *****
Description:The method checks whether there are frames between f0 and f1
Params:	ocp	- the list
		f0,f1 - the segment
Returns:1 - there are frames, 0 - no frames in the segment
*****************************/
int ocpL_exist(ocp_list *ocp, int f0, int f1)
{
	ocp_type *cp = ocp->cp;
	if ( cp == NULL )
		return 0;
	for ( ; cp && cp->fno + cp->dfno <= f0; cp = cp->next);
	if ( !cp )
		return 0;
	if (( cp->fno < f0 && cp->Fshot == SHOT_END && (cp->next == NULL || cp->next->fno > f1)) ||
		( cp->fno > f1 ))
		return 0;
	return 1;
}

