/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**************************
 ***	OcpFilter.c	    ***
 **************************/
#include	<stdio.h>
#include	<math.h>

#include	"Uigp/igp.h"


#include	"GpType/GpType.h"


#include	"VimageType/OcpType.h"

ocp_type * ocpLF_prev_kf( ocp_list *ocp, int frame ) ;
 /*
eyal
description:
	The method accepts ocpList, and drops the droppable ocps.	
params:
	ocp_type * ocpL the list of keyframes
	dx,dy	- movement
	ds		- of skeleton
returns:
	how many key frames were dropped
*/
int 
ocpL_filter( ocp_list *ocpL ,float dx, float dy, float ds) 
{
	int frames_dropped=0;
	float t;
	ocp_type *cp0,*cp1,*cp2;
	if (ocpL == NULL) 
		return 0;		
	cp0 = ocpL->cp;
	cp1 = cp0->next;
	if ( cp1 == NULL )
		return 0;
	cp2 = cp1->next;
	/*start the checking from the second keyframe*/
	for(; cp2 != NULL ; cp0 = cp1, cp1=cp2, cp2=cp2->next )
	{	
		t = ((float)( cp1->fno - cp0->fno )) / ( cp2->fno - cp0->fno );	/*****   ????????     ********/	
		if ( ocp_may_be_dropped(cp0,cp1,cp2,t, dx, dy, ds ))
		{
			/*drop the keyframe from the list*/
			cp0->next = cp2;
			cp0->dfno = cp2->fno - cp0->fno;
			ocpL->no--;
			ocp_free(cp1);				
			/* and to keep the cp0 as is, cp1 becomes cp0 */
			cp1 = cp0;	
			frames_dropped ++;
		}
	}	
	return frames_dropped;
}



/*eyal
Description:
	The method checks whether the ocps are close enough ( not exactly equals)
	it checks:
		x,y
		u0,u1,u2
		skeleton 
params: ocp_type *cp,*rcp - the ocps to compare
		float dx,dy - threshold
		float ds - threshold for the skeleton
*/
int
ocp_is_close_enough( ocp_type *cp, ocp_type *rcp ,float dx, float dy ,float ds)
{
	float f;
	f = (cp->c.x - rcp->c.x);
	if( ABS(f) > dx) 
		return 0;
	f = (cp->c.y - rcp->c.y);
	if ( ABS(f)  > dy)
		return 0;

	if( skl_distance( cp->skl , rcp->skl ) > ds )
		return 0;	
	return 1;
}
/*
eyal
description:
	The method accepts 3 keyframes, and checks whether it is possible to drop the 
	middle one. 
params:
	ocp_type *cp0, *cp1, *cp2: the keyframes by order
	float t: the location of the keyframe according to the others (0 - it's the first, 1 - it's the right).
	float dx,dy - threshold
	float ds - threshold for the skeleton
returns:
	0 - may not be dropped
	1 - may be dropped
*/
int 
ocp_may_be_dropped( ocp_type *cp0, ocp_type *cp1, ocp_type * cp2, float t,float dx, float dy, float ds)
{

	ocp_type *cp=0;
	int answer = 0;
	if ( !cp0 || cp0->Fshot == SHOT_END || cp1->Fshot == SHOT_END)
		// not allowed to be dropped, because it is not connected to other keyframes.
		return (0);

	/*create the interpolation*/
	cp = ocp_affine(cp0, cp2, t, NULL);
	/*compare to the middle cp*/
	answer = ocp_is_close_enough(cp1,cp,dx,dy,ds);	
	ocp_free(cp);
	return answer;
}


 
 /*
eyal
description:
	The method accepts ocpList, and frame number, and dropps the kf if it is possible.	
params:
	ocp_type * ocpL the list of keyframes
	dx,dy	- movement
	ds		- of skeleton
	fno		- the desired frame to drop
returns:
	was that frame filterd(1) or not(0)
*/
int			ocpLF_filter(ocp_list *ocpL, float dx, float dy, float ds, int fno)
{	
	int frames_dropped=0;
	float t;
	ocp_type *cp0,*cp1,*cp2;
	if (ocpL == NULL) 
		return 0;		
	cp0  = ocpLF_prev_kf( ocpL, fno );
	if ( cp0 == NULL )
		return 0;
	cp1 = cp0->next;
	if (cp1 == NULL || 
		cp1->next == NULL || 
		cp1->fno != fno ||
		cp0->Fshot == SHOT_END ||
		cp1->Fshot == SHOT_END)
		return 0;
	cp2 = cp1->next;
	t = ((float)( cp1->fno - cp0->fno )) / ( cp2->fno - cp0->fno );	/*****   ????????     ********/	
	if ( ocp_may_be_dropped(cp0,cp1,cp2,t, dx, dy, ds ))
	{
		/*drop the keyframe from the list*/
		cp0->next = cp2;
		cp0->dfno = cp2->fno - cp0->fno;
		ocpL->no--;
		ocp_free(cp1);				
		frames_dropped ++;
	}		
	return frames_dropped;
}


