/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**************************
 ***	OcpLFrame.c	***
 **************************/
#include	<math.h>

#include	"Uigp/igp.h"


#include	"VimageType/OcpType.h"



int
ocpLF_Fshot( ocp_list *ocp, int frame )
{
	if( ocp->cp == NULL )	return( 0 );

	if( frame < ocp->acp->fno )	return( 0 );

	if( frame > ocp->acp->fno && ocp->acp->Fshot == SHOT_END )
		return( 0 );

	if( frame == ocp->acp->fno )	return( ocp->acp->Fshot );

	return( SHOT_FRAME );
}



/***** ocpLF_shot_nieg *****
Description:
Params:
Returns:
*****************************/
int ocpLF_shot_nieg( ocp_list *ocp, int frame )
{
ocp_type	*Lcp,	*Fcp;
int	m;

	if( ocp->cp == NULL )	return( 0 );

	if( frame < ocp->acp->fno )	return( 0 );

	if( frame > ocp->acp->fno && ocp->acp->Fshot == SHOT_END )	return( 0 );

	m = 0x04;

	ocpL_get_shot( ocp, ocp->acp, &Fcp, &Lcp );
	if( Lcp->next != NULL )	m |= 0x01;
	if( Lcp != ocp->cp )	m |= 0x02;

	return( m );
}












/***** ocpLF_shot *****
Description:The method accepts frame number, and returns (in Fcp and Lcp) the keyframes 
			that enclose the shot in which the frame is.
			If the frame is out of the list, NULL will be returned.
Params:		ocp		- the list to search in.
			frame	- frame number (fno) for looking
			Fcp		- the first keyframe in the segment
			Lcp		- the second keyframe in the segment
Returns:	int - number of keyframes in the segment. If the segment is a single keyframe, 
					then "1" is returned
*****************************/
int ocpLF_shot( ocp_list *ocp, int frame, ocp_type **Fcp, ocp_type **Lcp )
{
ocp_type	*cp,	*fcp;
int	n=0;

	if( Fcp != NULL ) *Fcp = NULL;
	if( Lcp != NULL ) *Lcp = NULL;

	fcp = cp = ocp->cp ;

	if( (!cp) || (ocp->no == 0) )	
		return( 0 );
	if( frame < cp->fno )	
		return( 0 );
	
	n = 1;
	while ( (cp != NULL ) && (cp->fno < frame ))
	{
		n++;
		if ( cp->Fshot == SHOT_END )
		{
			fcp = cp->next;
			n = 1;
		}
		cp = cp->next;		
	}
	
	if ( cp != NULL )
	{
		
		while (( cp->next != NULL ) && (cp->Fshot != SHOT_END ) )
		{
			cp = cp->next;
			n ++;	
		}
	}
	if( Fcp != NULL )	*Fcp = fcp;
	if( Lcp != NULL )	*Lcp = cp;

	return n;
	
}









/***** ocpLF_status *****
Description: The method determines the status of frame number "frame"
	there might be 6 states:
		a blank frame				000	
		regular frame		-----	110	
		standalone keyframe   *		001
		end keyframe		--*		101
		start keyframe		  *--	011
		middle keyframe		--*--	111

		
Params:	ocp - the list to check
		frame - the number of frame to query
Returns: a mask according to the state
*****************************/
int ocpLF_status( ocp_list *ocp, int frame )
{


	ocp_type * acp ,*prevCp = NULL;
	if ( ocp == NULL || ocp->cp == NULL )
		return (FRAME_BLANK);
	
	for( acp = ocp->cp ; acp != NULL && acp->fno + acp->dfno <= frame ; prevCp = acp, acp = acp->next )
		;
	if( acp == NULL || acp->fno > frame)	
		return (FRAME_BLANK);
	
	if ( acp->fno == frame )
	{
		if ( prevCp && prevCp->Fshot != SHOT_END)
		{
			if (acp->Fshot == SHOT_END)
				return FRAME_K_END;
			else 
				return FRAME_K_MIDDLE;
		}// has a previous
		else 
		{
			if (acp->Fshot == SHOT_END)
				return FRAME_K_STANDALONE;
			else 
				return FRAME_K_BEGIN;		
		}			
	}
	else 
	{
		if ( acp && acp->Fshot != SHOT_END ) 
			return FRAME_MIDDLE;
		else 
			return FRAME_BLANK;
	}

}


/***** ocpLF_is_kf *****
Description:The method checks whether frame is a keyframe.
Params:	ocp		- the list
		frame	- the index to check
Returns:1 / 0 - keyframe or not
*****************************/
int ocpLF_is_kf(ocp_list *ocp, int frame)
{
	ocp_type * cp = ocp->cp;
	if (( cp == NULL) || ( cp->fno > frame ) )
		return 0;
	while ( cp && (cp->fno < frame ))
		cp = cp->next;			
	if ( !cp || (cp->fno > frame))
		return 0;
	return 1;	
}


int
ocpLF_is_3d_kf( ocp_list *ocp, int frame )
{
	if( ocp->cp == NULL || ocp->acp->fno != frame )	return( 0 );

	return( ocp->acp->Fgc );
}



/***** ocpLF_get_kf *****
Description:The method returns the keyframe that is most close to "frame" from below.
Params:		ocp - the list
			frame-the desired frame number
Returns:	the keyframe which is previous to "frame" or NULL if no such keyframe
			If frame is keyframe, it would be returned.
*****************************/
ocp_type * ocpLF_get_kf( ocp_list *ocp, int frame )
{
ocp_type	*cp;

	if( ocp->cp == NULL || frame < ocp->cp->fno )	return( NULL );


	if( ocp->acp != NULL && frame >= ocp->acp->fno &&
			frame < ocp->acp->fno + ocp->acp->dfno )
		return( ocp->acp );



	for( cp = ocp->cp ;
			cp->next != NULL && cp->fno + cp->dfno <= frame ;
			cp = cp->next );

	return( cp );
}



/***** ocpLF_prev_kf *****
Description:The method returns the keyframe which is previous to "frame"
			If there is none - NULL is returned
Params:		ocp - the list
			frame-the index
Returns:The keyframe previous to frame, or NULL if no such exists.
*****************************/
ocp_type * ocpLF_prev_kf( ocp_list *ocp, int frame )
{
ocp_type	*cp;

	if ( ocp->cp == NULL )	return NULL;


	for( cp = ocp->cp ;
		cp->fno + cp->dfno < frame && cp->next != NULL ; cp = cp->next);

	if ( cp->fno > frame )
		return NULL;
	return( cp );
}


/***** ocpLF_next_kf *****
Description:Returns the keyframe which is the one after "frame" or NULL if no such keyframe
Params:	ocp - the list
		frame - the index number
Returns:which is the on "frame" or first after "frame" or NULL if no such keyframe
*****************************/
ocp_type * ocpLF_next_kf( ocp_list *ocp, int frame )
{
ocp_type	*cp;
	
	if ( !ocp || !(ocp->cp))
		return NULL;
	if( ocp->acp->fno == frame )
		return( ocp->acp->next );
	
	for( cp = ocp->cp ;	cp != NULL && cp->fno + cp->dfno <= frame ;	cp = cp->next );

	if( cp == NULL )	return( NULL );
	
	if ( cp->fno > frame )// happens if the frame is before start of the animation.
		return cp;

	return( cp->next );
}



ocp_type *
ocpLF_force_kf( ocp_list *ocp, int frame )
{
ocp_type	*acp  ,	*cp = NULL;
float	t;
	

	for( acp = ocp->cp ; acp != NULL && acp->fno + acp->dfno <= frame ; acp = acp->next );

	if( acp == NULL )	return(NULL);


	if( acp->fno == frame )	return( acp );

	if( acp->Fshot == SHOT_END )	return( NULL );


	t = ( frame - acp->fno ) / (float)acp->dfno;
	cp = ocp_affine( acp, acp->next, t, NULL );

	cp->Fshot = SHOT_MID;

	cp->next = acp->next;
	acp->next = cp;
	ocp->no++;

	cp->fno = frame;
	cp->dfno = cp->next->fno - cp->fno;

	acp->dfno = cp->fno - acp->fno;


	return( cp );
}






/***** ocpLF_copy *****
Description:The method copies frame number "frame" only if this frame is not FRAME_BLANK
Params:	ocp - the list
		frame - the number of frame to copy
Returns:the frame or NULL
*****************************/
ocp_type * ocpLF_copy( ocp_list *ocp, int frame )
{
ocp_type	*acp,	*cp;
float	t;
	if ( ocp->cp == NULL || ocp->cp->fno > frame)
		return NULL;

	for( acp = ocp->cp ; acp && acp->fno + acp->dfno <= frame ; acp = acp->next );// frame is on acp or after (but before acp->next)
	if ( !acp ) 
		return NULL;
	if( acp->fno == frame ){
		cp = ocp_make_copy( acp );
		//cp->Fgc = 0;
		return( cp );
	}
	if ( acp->Fshot == SHOT_END ) 
		return NULL;// the frame is FRAME_BLANK
	t = ( frame - acp->fno ) / (float)acp->dfno;
	cp = ocp_affine( acp, acp->next, t, NULL );
	//cp->Fgc = 0;

	cp->fno = frame;
	cp->dfno = acp->next->fno - cp->fno;

	return( cp );
}





/***** ocpLF_paste *****
Description:The method pastes keyframe (cp) at frame number "frame"
Params:		ocp	-	the list
			frame	-	the index to insert at
			cp	-	the keyframe to insert
			Fadd	-	flag. If Fadd = 0, then the keyframe is pasted only on another keyframe
			F3d	-	flag. If F3d = 0 params of Fgc won't be pasted.
Returns:
*****************************/
int ocpLF_paste( ocp_list *ocp, int frame, ocp_type *cp, int Fadd, int F3d )
{
ocp_type	*acp,	*cp0;



	acp = ocpLF_force_kf( ocp, frame );

	if ( acp != NULL ){
		acp->c = cp->c;
		acp->dp = cp->dp;

		skl_copy( cp->skl, acp->skl );


		if( F3d == 1 ){

			acp->gc.x = cp->gc.x;
			acp->gc.y = cp->gc.y;
			acp->gc.z = cp->gc.z;

			acp->gc.u1 += cp->c0.u1 - acp->c0.u1;
			acp->gc.u2 += cp->c0.u2 - acp->c0.u2;
			acp->gc.u3 += cp->c0.u3 - acp->c0.u3;

			acp->Fgc = 1;
		}


		return( 1 );
	}


	

	if( Fadd == 0 )	return( 0 );



	cp0 = ocp_make_copy(cp);
	cp0->fno = frame;
	cp0->Fshot = SHOT_END;
	cp0->Fgc = 1;
	

	if( ocp->cp == NULL || ocp->cp->fno > frame ){
		cp0->next = ocp->cp;
		if( cp0->next != NULL )
			cp0->dfno = cp0->next - cp0->next;


		ocp->cp = cp0;
		ocp->no++;
		return (2);
	}
	

	


	acp = ocpLF_get_kf( ocp, frame );

	cp0->next = acp->next;
	acp->next = cp0;
	if( cp0->next != NULL )
		cp0->dfno = cp0->next->fno - cp0->fno;
	acp->dfno = cp0->fno - acp->fno;


	return( 2 );
}






/***** ocpLF_set_tween *****
Description:The method sets or resets keyframe's Fshot to be SHOT_END or SHOT_MID
Params:	cp - keyframe to set/reset
		enable - flag (0 - Fshot would be SHOT_END, 1 - Fshot would be SHOT_END)
Returns:
*****************************/
void	ocpLF_set_tween( ocp_type* cp, int enable)
{
	if( cp != NULL && cp->next != NULL)
	{
		cp->Fgc = 1;
		if ( enable )
			cp->Fshot = SHOT_MID;
		else
		{
			cp->Fshot = SHOT_END;
			if ( cp->next != NULL )
				cp->next->Fgc = 1;
		}
	}
}

